/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package com.thalesgroup.tools.ant.taskdefs;


import java.io.FileInputStream;
import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Task;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.traversal.NodeIterator;
import org.xml.sax.InputSource;

import com.sun.org.apache.xpath.internal.XPathAPI;
import com.sun.org.apache.xpath.internal.objects.XObject;


/**
 * Parse all the CSCProperties.xml files in order to extract 
 * the test id and the title
 */
public class TestIdentification extends Task {


    // The paths of all the CSC properties files.
    private Vector m_paths = new Vector();
    // Store the dependencies into an ANT property.
    private String m_property = null;

    private Vector m_ids = new Vector();


    /**
     * Specify the paths to all the CSC properties files.
     *
     * @param paths The paths to all the CSC properties files.
     */
    public void setPaths(String paths) {
        m_paths = org.apache.tools.ant.util.
                      StringUtils.split(paths.trim(), ' ');
    }


    /**
     * Return the paths to all the CSC properties files.
     *
     * @return A collection of paths.
     */
    public Collection getPaths() {
        return m_paths;
    }


    /**
     * Specify the property to create.
     *
     * @param property The property to create.
     */
    public void setProperty(String property) {
        m_property = property;
    }


    /**
     * Get the name of the property.
     *
     * @return The name of the property.
     */
    public String getProperty() {
        return m_property;
    }


    /**
     * Default constructor.
     */
    public TestIdentification() {
        super();
    }


    /**
     * Check the attributes.
     */
    protected void checkConfiguration() {


        if ((m_paths == null) || m_paths.isEmpty()) {
            throw new BuildException("attribute paths must not be null.",
                                     getLocation());
        }

    }


    /**
     * Runs the task.
     *
     * @throws BuildException The exception raised during task execution.
     */
    public void execute() throws BuildException {
        checkConfiguration();

        // Parse the package description file.
        FileInputStream fis = null;
        InputSource in = null;

        try {
            
            Iterator it = m_paths.iterator();
            while (it.hasNext()) {
                
                String f = null;
                f = (String) it.next();
                fis = new FileInputStream(f);
                in = new InputSource(fis);
                
                // Set up a DOM tree to query.
                DocumentBuilderFactory dfactory =
                    DocumentBuilderFactory.newInstance();
                dfactory.setNamespaceAware(true);
                Document doc = dfactory.newDocumentBuilder().parse(in);
                
                parseRuntest(doc);
                
            }

        } catch (Exception e) {
            throw new BuildException(e);
        } finally {
            if (fis != null) {
                try {
                    fis.close();
                } catch (IOException ioe) {
                    // Ignore.
                }
            }
        }
    }


    /**
     * Parse the runtest node.
     *
     * @param doc The XML document.
     */
    @SuppressWarnings("unchecked")
    protected void parseRuntest(Document doc)
        throws javax.xml.transform.TransformerException, IOException {

        // Find all the tests.
        NodeIterator testNodes =
            XPathAPI.selectNodeIterator(doc, "//runtest/test");
        Vector tests = new Vector();
        
        StringBuffer sb = new StringBuffer();
        Node n;
        while ((n = testNodes.nextNode()) != null) {
            XObject xobj = XPathAPI.eval(n, "@name");
            String testname = xobj.str().trim();

            sb.append(testname);
            sb.append(" ");
            tests.add(n);
            
        }   
        
        Iterator iter = tests.iterator();
        while (iter.hasNext()) {
            Node testNode = (Node) iter.next();
            XObject xobj = XPathAPI.eval(testNode, "@name");
            // Property "csc.testname.id"
            xobj = XPathAPI.eval(testNode, "@id");
            String id = xobj.str().trim();

            if (id.equals("")) {
                log("No id",Project.MSG_WARN);
            }           
            else
                {
                    if(m_ids.contains(id))                   
                        log(id + " **** DUPLICATED ID ****",Project.MSG_WARN);
                    else
                        {
                            log(id,Project.MSG_WARN);
                            m_ids.addElement(id);
                        }
                           
                }
            log("----------");

            // Property "csc.testname.title"
            xobj = XPathAPI.eval(testNode, "title");
            String result = xobj.str().trim();
            if (result.equals("")) {
                log("No description",Project.MSG_WARN);
            }           
            else
                {
                log(result,Project.MSG_WARN);
                }

            log("",Project.MSG_WARN);

        }
    }
}
