/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <fstream>
#include <iostream>
#include <stdio.h>

/*
 * Use:
 * g++ xml_inverter.cpp -o xml_inverter 
 * to compile it.
 */

void usage(std::ostream & os, const std::string &program_name)
{
    os << "\n\nUsage:"
       << " xml_inverter [file_list]" 
       << "\n  ________________________________________________________________"
       << "\n  The file_list parameter is mandatory :  "
       << "\n  It is a text file containing the list of the xml scenario files to be changed."  
       << "\n  It modifies, in the Init-graph and Stop-graph, the order of graph-element"
       << "\n  and graph-element-root as described in the ECR-0591."
       << "\n  ________________________________________________________________"
       << std::endl;
}

int main( int argc, char* argv[])
{
    if (!(argc >= 2))
      {
          usage(std::cout, argv[0]);
          return 0;
      }
    
    std::string file_list_name=argv[1];
    std::string file;
    std::ifstream file_list(file_list_name.c_str());
    std::getline(file_list,file);

    while(!file_list.eof()){
        std::string filemod=file;
        filemod.append(".old");
        int res=rename(file.c_str(),filemod.c_str());
        if(res!=0) 
        {
            std::cerr<<"\nError renaming "<<file<<"\n";
            return res;
        }
        std::ifstream i_file(filemod.c_str());
        std::ofstream o_file(file.c_str(),std::ios::out|std::ios::app);
        std::string f;
        std::string temp;
        try
        {
            while(!i_file.eof())
            {                    
                std::getline(i_file,f);
                if(f.find("<Init-graph")==std::string::npos && f.find("<Stop-graph")==std::string::npos) 
                {
                    if (o_file)
                    {
                        o_file << f<<std::endl;
                    }
                    else
                    {
                        std::cerr << "*** ERROR IN WRITING FILE ***" << std::endl;
                    }
                }else  if(f.find("<Init-graph")!=std::string::npos) 
                {
                    o_file << f<<std::endl;  
                    std::getline(i_file,f);  
                    while(f.find("Graph-element-root")==std::string::npos) 
                    {
                        temp.append(f.c_str());
                        temp.append("#");
                        std::getline(i_file,f);
                    }
                    while(f.find("/Init-graph")==std::string::npos) 
                    {
                        o_file << f<<std::endl; 
                        std::getline(i_file,f);
                    }

                    std::string line= temp;
                    size_t separator=temp.find("#");
                    while(separator!=std::string::npos)
                    {
                        line = temp.substr( 0, separator);
                        temp=temp.substr(separator+1);
                        o_file << line<<std::endl;
                        separator=temp.find("#");
                    }
                    o_file << temp<<std::endl;
                    o_file << f<<std::endl; 
                }else  if(f.find("<Stop-graph")!=std::string::npos) 
                {
                    o_file << f<<std::endl;  
                    std::getline(i_file,f); 
                    std::string temp;
                    while(f.find("Graph-element-root")==std::string::npos) 
                    {
                        temp.append(f.c_str());
                        temp.append("#");
                        std::getline(i_file,f);
                    }
                    while(f.find("/Stop-graph")==std::string::npos)
                    {
                        o_file << f<<std::endl; 
                        std::getline(i_file,f);
                    }
                    std::string line= temp;
                    size_t separator=temp.find("#");
                    while(separator!=std::string::npos)
                    {
                        line = temp.substr( 0, separator);
                        temp=temp.substr(separator+1);
                        o_file << line<<std::endl;
                        separator=temp.find("#");
                    }
                    o_file << temp<<std::endl;
                    o_file << f<<std::endl; 
                }
            }//while
        }//try
        catch(...)
        {
            std::cerr<<"\nERROR"<< file;
        }
        std::cerr<<file<<" modified, a backup is saved in ----> "<<filemod<<std::endl;
        std::getline(file_list,file);
    }
    return 0;
}

