#!/bin/sh
# =====================================================================
# This file is part of CARDAMOM (R) which is jointly developed by THALES
# and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
# Copyright (C) THALES 2000-2003. All rights reserved.
# Copyright (C) THALES 2004-2005. All rights reserved.
# Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
# All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
# License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

 
my_script=`basename $0`
current_dir=`pwd` 


## Usage
usage() {
    echo "$my_script: Gather all Emma informations from Cardamom compilation"
    echo "directory, and use Emma to generate a filtered trace file."
    echo
    echo "usage: $my_script -c cdmw_home -o output_file -f filter_file"
    echo
    echo "options:"
    echo "    -c <cdmw_home>     Cardamom sources directory."
    echo "    -o <output-file>   File to be generated."
    exit 1
}
 

check_emma() {
    emma_classpath=`echo $CLASSPATH | sed s%:%\\\n%g | grep emma.jar` 2> /dev/null
    if [ -z "$emma_classpath" ]; then
        echo "Emma should be installed and emma.jar should be present in CLASSPATH."
        exit 1
    fi
}
 
 

### Check current directory
if [ ! -x $my_script ]; then
    echo "This script should be started from the Build/testcoverage directory."
    exit 1
fi


### Check emma
check_emma
echo "EMMA CLASSPATH: $emma_classpath"


## Parse command line arguments.
while getopts c:o: option; do
    case $option in
        c) cdmw_home=$OPTARG ;;
        o) output_file=$OPTARG ;;
       \?) usage ;;
    esac
done

if [ "$OPTIND" -le "$#" ]
then
   shift $(($OPTIND-1))
   echo "ERROR on input parameter $1"
   usage
   exit 1
fi


### check options
if [ -z "$cdmw_home" ]; then
    echo "Missing -c option."
    usage
fi
if [ -z "$output_file" ]; then
    echo "Missing -o option."
    usage
fi

if [ ! -d $cdmw_home -o ! -d $cdmw_home/Build/testcoverage ]; then
    echo "$cdmw_home is not the Cardamom sources directory"
    usage
fi

touch $output_file
if [ ! -f $output_file ]; then
    usage
fi
rm $output_file


### gather all emma infos from localhost

# gather all coverage data 
emma_coverage_files=""
for i in `find $cdmw_home -name "*.ec"`; do
    emma_coverage_files="$emma_coverage_files -in $i"
done
java emma merge $emma_coverage_files -out ${output_file}





