<?xml version="1.0" encoding="utf-8"?>
<!-- ===================================================================== -->
<!--
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-->
<!-- ===================================================================== -->


<xsl:stylesheet version="1.1"
   xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
   xmlns:UML="org.omg/UML1.3">



<!--
   Main - This template fetches the necessary data from inputs
   to generate a file.

   @param none
-->
<xsl:template name="ccmcif_consumer_cpp">
   <!--
      Parameters below are used for recursiveness.
   -->
   <xsl:param name="_index" select="1"/>

   <xsl:if test="boolean($lifecycleFrameworkNodes/home-impl)">
      <xsl:if test="$_index &lt;= count($lifecycleFrameworkNodes/home-impl)">
         <xsl:variable name="homeImplNode" select="$lifecycleFrameworkNodes[home-impl and position() = $_index]/home-impl"/>

         <!--
            Get the managed component.
         -->
         <xsl:variable name="homeName">
            <xsl:call-template name="getLastToken">
               <xsl:with-param name="_string" select="$homeImplNode/../@factory-idref"/>
               <xsl:with-param name="_separator" select="$xmlSep"/>
            </xsl:call-template>
         </xsl:variable>
         <xsl:variable name="managedComponentId">
            <xsl:call-template name="getManagedComponentId">
               <xsl:with-param name="_homeName" select="$homeName"/>
            </xsl:call-template>
         </xsl:variable>
         <xsl:variable name="managedComponentNode" select="key('classById', $managedComponentId)"/>

         <xsl:if test="boolean($managedComponentNode)">
            <!--
               Find all the consumers.
            -->
            <xsl:variable name="ccmConsumesId" select="key('stereotypeByName', 'CCMConsumes')/@xmi.id"/>
            <xsl:variable name="consumers" select="key('associationByStereotypeId', $ccmConsumesId)[UML:Association.connection/UML:AssociationEnd[1]/@type = $managedComponentNode/@xmi.id]"/>

            <xsl:for-each select="$consumers">
               <xsl:sort select="@xmi.id"/>

               <xsl:variable name="consumerTypeNode" select="key('classById', UML:Association.connection/UML:AssociationEnd[2]/@type)"/>
               <xsl:variable name="consumerTypeName" select="$consumerTypeNode/@name"/>
               <xsl:variable name="consumerClassname" select="concat($consumerTypeName, 'Consumer')"/>
               <xsl:variable name="consumerImplClassname" select="concat($consumerClassname, '_impl')"/>

               <!--
                  Get the output filename.
               -->
               <xsl:variable name="filename">
                  <xsl:call-template name="buildOutputFilename">
                     <xsl:with-param name="_userDirectory" select="$homeImplNode/@directory"/>
                     <xsl:with-param name="_userFilename" select="$homeImplNode/@filename"/>
                     <xsl:with-param name="_defaultFilename" select="$consumerImplClassname"/>
                     <xsl:with-param name="_fileExtension" select="'.cpp'"/>
                  </xsl:call-template>
               </xsl:variable>

               <!--
                  Display a message to the user.
               -->
               <xsl:message>
                  <xsl:value-of select="concat('Generating file ', $filename)"/>
               </xsl:message>

               <!--
                  Generate file.
               -->
               <xsl:document method="text" href="{$filename}">
                  <!--
                     Insert document header.
                  -->
                  <xsl:call-template name="docHeader">
                     <xsl:with-param name="_lang" select="'cpp'"/>
                  </xsl:call-template>

                  <!--
                     Insert the content.
                  -->
                  <xsl:call-template name="ccmcif_consumer_cpp.content">
                     <xsl:with-param name="_scopedHomeImplClassname">
                        <xsl:call-template name="buildHomeImplClassname">
                           <xsl:with-param name="_homeImplNode" select="$homeImplNode"/>
                        </xsl:call-template>
                     </xsl:with-param>
                     <xsl:with-param name="_consumerNode" select="$consumerTypeNode"/>
                     <xsl:with-param name="_consumerClassname" select="$consumerClassname"/>
                     <xsl:with-param name="_consumerVarName" select="UML:Association.connection/UML:AssociationEnd[2]/@name"/>
                     <xsl:with-param name="_componentNode" select="$managedComponentNode"/>
                  </xsl:call-template>
               </xsl:document>
            </xsl:for-each>
         </xsl:if>

         <!--
            Proceed to the next home impl if any.
         -->
         <xsl:call-template name="ccmcif_consumer_cpp">
            <xsl:with-param name="_index" select="$_index + 1"/>
         </xsl:call-template>
      </xsl:if>
   </xsl:if>
</xsl:template> <!-- end of template ccmcif_consumer_cpp -->



<!--
   This template generates the content of the file.

   @param _scopedHomeImplClassname   The scoped home impl classname.
   @param _consumerNode              The consumer type node.
   @param _consumerClassname         The consumer class name.
   @param _consumerVarName           The name of the consumer as declared in IDL.
   @param _componentNode             The node of the component where the consumer is declared.
-->
<xsl:template name="ccmcif_consumer_cpp.content">
   <xsl:param name="_scopedHomeImplClassname"/>
   <xsl:param name="_consumerNode"/>
   <xsl:param name="_consumerClassname"/>
   <xsl:param name="_consumerVarName"/>
   <xsl:param name="_componentNode"/>

   <!--
      Miscellaneous variables.
   -->
   <xsl:variable name="consumerTypeName" select="$_consumerNode/@name"/>
   <xsl:variable name="componentName" select="$_componentNode/@name"/>
   <xsl:variable name="consumerImplClassname" select="concat($_consumerClassname, '_impl')"/>
   <xsl:variable name="cppScopedConsumerTypeName">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_consumerNode"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="cppScopedConsumerClassname" select="concat($cppScopedConsumerTypeName, 'Consumer')"/>
   <xsl:variable name="consumerNamespace">
      <xsl:variable name="homeImplClassname">
         <xsl:call-template name="getLastToken">
            <xsl:with-param name="_string" select="$_scopedHomeImplClassname"/>
         </xsl:call-template>
      </xsl:variable>
      <xsl:call-template name="getLastToken">
         <xsl:with-param name="_string" select="substring-before($_scopedHomeImplClassname, concat($xmlSep, $homeImplClassname))"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="cppConsumerScope">
      <xsl:call-template name="removeLastToken">
         <xsl:with-param name="_scopedName" select="$cppScopedConsumerTypeName"/>
         <xsl:with-param name="_sep" select="$cppSep"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="lowercaseComponentName" select="translate($componentName, $ucase, $lcase)"/>
   <xsl:variable name="lowercaseConsumerTypeName" select="translate($consumerTypeName, $ucase, $lcase)"/>
   <xsl:variable name="cppScopedComponentName">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_componentNode"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="cppComponentScope">
      <xsl:call-template name="removeLastToken">
         <xsl:with-param name="_scopedName" select="$cppScopedComponentName"/>
         <xsl:with-param name="_sep" select="$cppSep"/>
      </xsl:call-template>
   </xsl:variable>

   <!--
      The code generation starts here.
   -->

   <xsl:variable name="pathPrefix">
       <xsl:choose>
           <xsl:when test="string-length($CSC_NAME) > 0">
               <xsl:value-of select="concat($CSC_NAME, $fileSep)"/>
           </xsl:when>
           <xsl:otherwise>
               <xsl:value-of select="''"/>
           </xsl:otherwise>
       </xsl:choose>
   </xsl:variable>

   <![CDATA[
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/orbsupport/Codec.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
// If LifeCycleObjectBase include is not performed before CCMObject_impl
// the compilation fails. This problem could be related to the fact that
// the includes are performed inside of macros ifndef and define. This can
// be a problem in the case of 'cross-include'.
#include <CCMContainer/ccmcif/LifeCycleObjectBase.hpp>
#include <CCMContainer/ccmcif/CCMObject_impl.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/commonsvcs/time/Util.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>
#include <CdmwCcmCifC.h>

    ]]>
   <![CDATA[#include "]]><xsl:value-of select="concat($pathPrefix, $consumerImplClassname, '.hpp')"/><![CDATA["]]>

   DEFINE_VARIABLE_LENGTH_DATA_TRAITS(CdmwCcmCif::TimestampedEvent)
   DEFINE_VALUETYPE_DATA_TRAITS(<xsl:value-of select="$cppScopedConsumerTypeName"/>)

   <xsl:call-template name="openNamespace">
      <xsl:with-param name="_scope" select="concat('Cdmw::CCM::CIF::Cdmw',$cppScopedConsumerTypeName)"/>
      <xsl:with-param name="_separator" select="$cppSep"/>
      <xsl:with-param name="_lastTokenIsNamespace" select="false()"/>
   </xsl:call-template>

   <xsl:value-of select="concat($consumerImplClassname, $cppSep, $consumerImplClassname)"/>(Components::ExecutorLocator_ptr exec_locator,
                        const <![CDATA[std::string&]]> name,
                        Context*                    ctx,
                        PortableServer::Servant     component,
                        CdmwLogging::Logger_ptr     logger)
      : LifeCycleObjectBase(ctx),
        m_executor_locator(Components::ExecutorLocator::_duplicate(exec_locator)),
        m_sink_name(name),
        m_component(component),
        m_logger(CdmwLogging::Logger::_duplicate(logger))
   {
      <![CDATA[CDMW_LOG_FUNCTION(m_logger.in(), "name="<<name);]]>
      // incrementation refcount of component in order to avoid destruction
      // of the component before the destruction of the facet
      <![CDATA[m_component->_add_ref();]]>
   }

   <xsl:value-of select="concat($consumerImplClassname, $cppSep, '~', $consumerImplClassname, '() throw()')"/>
   {
      <![CDATA[CDMW_LOG_FUNCTION(m_logger.in(), "m_sink_name="<<m_sink_name);]]>
      // no remove_ref on component while it's done in var destructor
   }

   void
   <xsl:value-of select="concat($consumerImplClassname, $cppSep, 'push_', $consumerTypeName,
      '(', $cppScopedConsumerTypeName, '* the_', $lowercaseConsumerTypeName, ')')"/>
      throw (CORBA::SystemException)
   {
       CDMW_LOG_FUNCTION(m_logger.in(), "");

       <![CDATA[Cdmw::CCM::CIF::CCMObject_impl* component_impl = dynamic_cast<Cdmw::CCM::CIF::CCMObject_impl*>(m_component.in());
       CDMW_ASSERT(component_impl!=0);
       component_impl->check_validity_of_operational_request();]]>

       CDMW_DEBUG(m_logger.in(), 
           <![CDATA["Consumer " << m_sink_name << " receives an event"]]>);
       try
        {
         <![CDATA[CORBA::Object_var obj = m_executor_locator->obtain_executor(m_sink_name.c_str());]]>

         if (CORBA::is_nil(obj.in()))
         {
            <![CDATA[CDMW_ERROR(m_logger.in(), 
                "Executor for consumer " << m_sink_name << " is nil");
            CORBA::OBJECT_NOT_EXIST ex(OrbSupport::OBJECT_NOT_EXIST,
                                       CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;]]>
         }

         // For monolithic component, push operation is implemented in <xsl:value-of select="$componentName"/> component
         // instead of the consumer executor for segmented component
         <xsl:value-of select="concat($cppComponentScope, $cppSep, 'CCM_', $componentName, '_var', ' ', $lowercaseComponentName, '_exec = ', 
                                      $cppComponentScope, $cppSep, 'CCM_', $componentName, $cppSep, '_narrow(obj.in());')"/>

         if (CORBA::is_nil(<xsl:value-of select="concat($lowercaseComponentName, '_exec.in()')"/>))
         {
            CDMW_DEBUG(m_logger.in(), 
                "Executor for consumer " <![CDATA[<<m_sink_name<<]]> " is not the component executor itself");
            // Component may segmented
            <xsl:value-of select="concat($cppConsumerScope, $cppSep, 'CCM_', $_consumerClassname, '_var consumer_exec = ',
                                         $cppConsumerScope, $cppSep, 'CCM_', $_consumerClassname, $cppSep, '_narrow(obj.in());')"/>

            if (CORBA::is_nil(consumer_exec.in()))
            {
               CDMW_ERROR(m_logger.in(),
                   "Executor for consumer " <![CDATA[<<m_sink_name<<]]> " has wrong type");
               CORBA::OBJECT_NOT_EXIST ex(OrbSupport::OBJECT_NOT_EXIST, 
                                          CORBA::COMPLETED_NO);
               CDMW_LOG_FUNCTION_EXCEPTION(ex);
               throw ex;
            }
            else
            {
               // push event to the consumer executor
               <![CDATA[consumer_exec->push]]>(the_<xsl:value-of select="$lowercaseConsumerTypeName"/>);   
            }
         }
         else
         {
            <xsl:value-of select="$lowercaseComponentName"/><![CDATA[_exec->push_]]><xsl:value-of select="concat($_consumerVarName, '(the_', $lowercaseConsumerTypeName,');')"/>
         }

         <![CDATA[m_executor_locator->release_executor(obj.in());]]>
      }
        catch (const <![CDATA[Components::CCMException& e]]> )
        {
            CDMW_ERROR(m_logger.in(),
                "CCMException received: " <![CDATA[<< e]]>);
            CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError,
                               CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
   }

   void
   <xsl:value-of select="$consumerImplClassname"/>::push_event(Components::EventBase * evt)
      throw (Components::BadEventType, CORBA::SystemException)
   {
      CDMW_LOG_FUNCTION(m_logger.in(), "");
      <xsl:value-of select="$cppScopedConsumerTypeName"/>_var info_evt
         = <xsl:value-of select="$cppScopedConsumerTypeName"/>::_downcast(evt);
      CORBA::add_ref(info_evt.in());
      <![CDATA[
      if (!info_evt.in())
      {
         CDMW_WARN(m_logger.in(),
             "EventBase with unexpected type received");
         Components::BadEventType ex;
         CDMW_LOG_FUNCTION_EXCEPTION(ex);
         throw ex;
      }
      else
      {
        this->push_]]><xsl:value-of select="$consumerTypeName"/>(info_evt.in());
      }
   }

   void
   <xsl:value-of select="$consumerImplClassname"/>::push(const <![CDATA[CORBA::Any& data]]>)
      throw (CosEventComm::Disconnected,
             CORBA::SystemException)
   {
      CDMW_LOG_FUNCTION(m_logger.in(), "");
      <xsl:value-of select="$cppScopedConsumerTypeName"/>_var event;
      /* PCR-0049
      <![CDATA[data >>= event;]]>
      */
      <![CDATA[
      const CORBA::OctetSeq* p_encoded_data;
      data >>= p_encoded_data;
      CORBA::OctetSeq encoded_data(*p_encoded_data);

      Cdmw::OrbSupport::Codec<CdmwCcmCif::TimestampedEvent> codec;
      
      CdmwCcmCif::TimestampedEvent timestamped_event;

      codec.decode(encoded_data, timestamped_event);
      ]]>

      event = <xsl:value-of select="$cppScopedConsumerTypeName"/>::_downcast(timestamped_event.event.in());

      <![CDATA[
      ::CORBA::add_ref(event.in());

      CDMW_DEBUG_EVENT(m_logger.in(), "Event received at: "
        << Cdmw::CommonSvcs::Time::Util::TimeT2string_ISO8601(Cdmw::CommonSvcs::Time::Util::get_current_time())
        << " issued at: " 
        << Cdmw::CommonSvcs::Time::Util::TimeT2string_ISO8601(timestamped_event.timestamp));
      try
      {
         this->push_]]><xsl:value-of select="$consumerTypeName"/>(event.in());
      }
      catch (const <![CDATA[CORBA::SystemException& e]]>)
      {
         CDMW_LOG_FUNCTION_EXCEPTION(e);
         throw;
      }
   }

   void
   <xsl:value-of select="$consumerImplClassname"/>::disconnect_push_consumer()
      throw (CORBA::SystemException)
   {
       CDMW_LOG_FUNCTION(m_logger.in(), "");
   }

   <xsl:call-template name="closeNamespace">
      <xsl:with-param name="_scope" select="concat('Cdmw::CCM::CIF::Cdmw',$cppScopedConsumerTypeName)"/>
      <xsl:with-param name="_separator" select="$cppSep"/>
      <xsl:with-param name="_lastTokenIsNamespace" select="false()"/>
   </xsl:call-template>

   <!--
      The code generation stops here.
   -->

</xsl:template> <!-- end of template ccmcif_consumer_cpp.content -->



</xsl:stylesheet>
