<?xml version="1.0" encoding="utf-8"?>
<!-- ===================================================================== -->
<!--
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-->
<!-- ===================================================================== -->


<xsl:stylesheet version="1.1"
   xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
   xmlns:UML="org.omg/UML1.3">


<!--
   Main - This template fetches the necessary data from inputs
   to generate a file.

   @param none
-->
<xsl:template name="ccmcif_facet_cpp">
   <!--
      Parameters below are used for recursiveness.
   -->
   <xsl:param name="_index" select="1"/>
   
   <xsl:if test="boolean($lifecycleFrameworkNodes/home-impl)">
      <xsl:if test="$_index &lt;= count($lifecycleFrameworkNodes/home-impl)">
         <xsl:variable name="homeImplNode" select="$lifecycleFrameworkNodes[home-impl and position() = $_index]/home-impl"/>

         <!--
            Get the managed component.
         -->
         <xsl:variable name="homeName">
            <xsl:call-template name="getLastToken">
               <xsl:with-param name="_string" select="$homeImplNode/../@factory-idref"/>
               <xsl:with-param name="_separator" select="$xmlSep"/>
            </xsl:call-template>
         </xsl:variable>
         <xsl:variable name="managedComponentId">
            <xsl:call-template name="getManagedComponentId">
               <xsl:with-param name="_homeName" select="$homeName"/>
            </xsl:call-template>
         </xsl:variable>
         <xsl:variable name="managedComponentNode" select="key('classById', $managedComponentId)"/>

         <xsl:if test="boolean($managedComponentNode)">
            <!--
               Find all the facets.
            -->
            <xsl:variable name="ccmProvidesId" select="key('stereotypeByName', 'CCMProvides')/@xmi.id"/>
            <xsl:variable name="facets" select="key('associationByStereotypeId', $ccmProvidesId)[UML:Association.connection/UML:AssociationEnd[1]/@type = $managedComponentNode/@xmi.id]"/>
         
            <xsl:for-each select="$facets">
               <xsl:sort select="@xmi.id"/>

               <xsl:variable name="facetNode" select="key('classById', UML:Association.connection/UML:AssociationEnd[2]/@type)"/>
               <xsl:variable name="facetName" select="$facetNode/@name"/>

               <xsl:variable name="facetImplClassname" select="concat($facetName, '_impl')"/>
               
               <!--
                  Get the output filename.
               -->
               <xsl:variable name="filename">
                  <xsl:call-template name="buildOutputFilename">
                     <xsl:with-param name="_userDirectory" select="$homeImplNode/@directory"/>
                     <xsl:with-param name="_userFilename" select="$homeImplNode/@filename"/>
                     <xsl:with-param name="_defaultFilename" select="$facetImplClassname"/>
                     <xsl:with-param name="_fileExtension" select="'.cpp'"/>
                  </xsl:call-template>
               </xsl:variable>

               <!--
                  Display a message to the user.
               -->
               <xsl:message>
                  <xsl:value-of select="concat('Generating file ', $filename)"/>
               </xsl:message>

               <!--
                  Generate file.
               -->
               <xsl:document method="text" href="{$filename}">
                  <!--
                     Insert document header.
                  -->
                  <xsl:call-template name="docHeader">
                     <xsl:with-param name="_lang" select="'cpp'"/>
                  </xsl:call-template>

                  <!--
                     Insert the content.
                  -->
                  <xsl:call-template name="ccmcif_facet_cpp.content">
                     <xsl:with-param name="_scopedHomeImplClassname">
                        <xsl:call-template name="buildHomeImplClassname">
                           <xsl:with-param name="_homeImplNode" select="$homeImplNode"/>
                        </xsl:call-template>
                     </xsl:with-param>
                     <xsl:with-param name="_facetNode" select="$facetNode"/>
                  </xsl:call-template>
               </xsl:document>
            </xsl:for-each>
         </xsl:if>

         <!--
            Proceed to the next home impl if any.
         -->
         <xsl:call-template name="ccmcif_facet_cpp">
            <xsl:with-param name="_index" select="$_index + 1"/>
         </xsl:call-template>
      </xsl:if>
   </xsl:if>
</xsl:template> <!-- end of template ccmcif_facet_cpp -->



<!--
   This template generates the content of the file.

   @param _scopedHomeImplClassname   The scoped home impl classname.
   @param _facetNode                 The facet node
-->
<xsl:template name="ccmcif_facet_cpp.content">
   <xsl:param name="_scopedHomeImplClassname"/>
   <xsl:param name="_facetNode"/>

   <!--
      Miscellaneous variables.
   -->
   <xsl:variable name="facetName" select="$_facetNode/@name"/>

   <xsl:variable name="cppScopedFacetName">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_facetNode"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="cppFacetScope">
      <xsl:call-template name="removeLastToken">
         <xsl:with-param name="_scopedName" select="$cppScopedFacetName"/>
         <xsl:with-param name="_sep" select="$cppSep"/>
      </xsl:call-template>
   </xsl:variable>

   <!--
      Miscellaneous variables.
   -->
   <xsl:variable name="facetImplClassname" select="concat($facetName, '_impl')"/>

   <!--
      The code generation starts here.
   -->

   <xsl:variable name="pathPrefix">
       <xsl:choose>
           <xsl:when test="string-length($CSC_NAME) > 0">
               <xsl:value-of select="concat($CSC_NAME, $fileSep)"/>
           </xsl:when>
           <xsl:otherwise>
               <xsl:value-of select="''"/>
           </xsl:otherwise>
       </xsl:choose>
   </xsl:variable>
   
   <![CDATA[
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
// If LifeCycleObjectBase include is not performed before CCMObject_impl
// the compilation fails. This problem could be related to the fact that
// the includes are performed inside of macros ifndef and define. This can
// be a problem in the case of 'cross-include'.
#include <CCMContainer/ccmcif/LifeCycleObjectBase.hpp>
#include <CCMContainer/ccmcif/CCMObject_impl.hpp>
   ]]>

   <![CDATA[#include "]]><xsl:value-of select="concat($pathPrefix, $facetImplClassname, '.hpp')"/><![CDATA["]]>

   <xsl:call-template name="openNamespace">
      <xsl:with-param name="_scope" select="$_scopedHomeImplClassname"/>
   </xsl:call-template>

   <xsl:value-of select="concat($facetImplClassname, $cppSep, $facetImplClassname)"/>(Components::ExecutorLocator_ptr exec_locator, 
                        const <![CDATA[std::string&]]> name,
                        Context*                  ctx,
                        CORBA::Object_ptr         comp_ref,
                        PortableServer::Servant   component,
                        CdmwLogging::Logger_ptr   logger)
      : LifeCycleObjectBase(ctx),
        m_executor_locator(Components::ExecutorLocator::_duplicate(exec_locator)),
        m_facet_name(name),
        m_comp_ref(CORBA::Object::_duplicate(comp_ref)),
        m_component(component),
        m_logger(CdmwLogging::Logger::_duplicate(logger))
   {
      <![CDATA[CDMW_LOG_FUNCTION(m_logger.in(), "name="<<name);]]>
      // incrementation refcount of component in order to avoid destruction
      // of the component before the destruction of the facet
      <![CDATA[m_component->_add_ref();]]>
   }

   <xsl:value-of select="concat($facetImplClassname, $cppSep, '~', $facetImplClassname, '() throw()')"/>
   {
      <![CDATA[CDMW_LOG_FUNCTION(m_logger.in(), "m_facet_name="<<m_facet_name);]]>
      // no remove_ref on component while it's done in var destructor
   }

   CORBA::Object_ptr
   <xsl:value-of select="$facetImplClassname"/>::_get_component()
   {
      CDMW_LOG_FUNCTION(m_logger.in(), "");
      <![CDATA[return CORBA::Object::_duplicate(m_comp_ref.in());]]>
   }

    
      <!--
         Facet's operations.
         NOTA: This should also include inherited operations.
      -->
   <xsl:call-template name="ccmcif_facet_cpp.content.operations.all">
      <xsl:with-param name="_interfaceNode" select="$_facetNode"/>
      <xsl:with-param name="_facetName" select="$facetName"/>
      <xsl:with-param name="_facetImplClassname" select="$facetImplClassname"/>
      <xsl:with-param name="_cppFacetScope" select="$cppFacetScope"/>
   </xsl:call-template> 


   <xsl:call-template name="closeNamespace">
      <xsl:with-param name="_scope" select="$_scopedHomeImplClassname"/>
   </xsl:call-template>

   <!--
      The code generation stops here.
   -->

</xsl:template> <!-- end of template ccmcif_facet_cpp.content -->


<!--
   This template generates definitions of operations of a given interface.
   This includes inherited operations.

   @param _interfaceNode       The interface node.
   @param _facetName           The name of the interface.
   @param _facetImplClassname  The Facet implementation class name.
   @param _cppFacetScope       The C++ scope for the class (namespaces ...).
-->
<xsl:template name="ccmcif_facet_cpp.content.operations.all">
   <xsl:param name="_interfaceNode"/>
   <xsl:param name="_facetName"/>
   <xsl:param name="_facetImplClassname"/>
   <xsl:param name="_cppFacetScope"/>

         <xsl:variable name="allBaseInterfaces">
            <xsl:call-template name="getAllBaseClasses">
                <xsl:with-param name="_classNode" select="$_interfaceNode"/>
            </xsl:call-template>
         </xsl:variable>

        <xsl:for-each select="$allBaseInterfaces/node()[string(@xmi.id)]">
            <xsl:sort select="@xmi.id"/>
            <xsl:variable name="_xmi_id" select="./@xmi.id"/>
            <!--
            NOTA: Note the following trick to get back to the initial document context
                  to be able to access operation definitions ... etc. because the result
                  tree ($allBaseInterfaces) is a new xml document a only contains a copy
                  of a subset of the original tree.
            -->
            <xsl:for-each select="$_interfaceNode">
                <xsl:variable name="_interface" select="key('classById', $_xmi_id)"/>

                <xsl:call-template name="ccmcif_facet_cpp.content.operations.own">
                    <xsl:with-param name="_interfaceNode" select="$_interface"/>
                    <xsl:with-param name="_facetName" select="$_facetName"/>
                    <xsl:with-param name="_facetImplClassname" select="$_facetImplClassname"/>
                    <xsl:with-param name="_cppFacetScope" select="$_cppFacetScope"/>
                </xsl:call-template> 
            </xsl:for-each>
        </xsl:for-each>
</xsl:template> <!-- end of template ccmcif_facet_cpp.content.operations.all -->


<!--
   This template generates definitions of operations of a given interface.
   This includes inherited operations.

   @param _interfaceNode       The interface node.
   @param _facetName           The name of the interface.
   @param _facetImplClassname  The Facet implementation class name.
   @param _cppFacetScope       The C++ scope for the class (namespaces ...).
-->
<xsl:template name="ccmcif_facet_cpp.content.operations.own">
   <xsl:param name="_interfaceNode"/>
   <xsl:param name="_facetName"/>
   <xsl:param name="_facetImplClassname"/>
   <xsl:param name="_cppFacetScope"/>


   <xsl:variable name="containedOps"  select="key('operations', $_interfaceNode/@xmi.id)"/>
   <xsl:for-each select="$containedOps">
      <!--
         Get the operation return type.
      -->
      <xsl:variable name="returnType">
         <xsl:call-template name="getType">
            <xsl:with-param name="_id" select="UML:BehavioralFeature.parameter/UML:Parameter[@kind = 'return']/@type"/>
            <xsl:with-param name="_kind" select="'return'"/>
         </xsl:call-template>
      </xsl:variable>
      <xsl:variable name="objRefVarType">
         <xsl:if test="$returnType != 'void'">
            <xsl:call-template name="getType">
               <xsl:with-param name="_id" select="UML:BehavioralFeature.parameter/UML:Parameter[@kind = 'return']/@type"/>
               <xsl:with-param name="_kind" select="'return'"/>
               <xsl:with-param name="_asObjRefVar" select="true()"/>
            </xsl:call-template>
         </xsl:if>
      </xsl:variable>
      
      <!--
         Get the operation name.
      -->
      <xsl:variable name="cppOperationName">
         <xsl:call-template name="mapCppReservedWords">
            <xsl:with-param name="_operationName" select="@name"/>
         </xsl:call-template>
      </xsl:variable>
      <xsl:value-of select="concat($lf, $returnType, $lf, $_facetImplClassname, $cppSep, $cppOperationName, $lf)"/>
         (
            <!--
               IN, OUT and INOUT parameters.
            -->
            <xsl:for-each select="UML:BehavioralFeature.parameter/UML:Parameter[@kind != 'return']">
               <!--
                  Get the parameter type.
               -->
               <xsl:variable name="paramType">
                  <xsl:call-template name="getParameterType">
                     <xsl:with-param name="_paramNode" select="."/>
                  </xsl:call-template>
               </xsl:variable>
               <xsl:value-of select="concat($paramType, ' ', @name)"/>
               <!--
                  Write out comma if necessary.
               -->
               <xsl:if test="position() != last()">
                  <xsl:value-of select="concat(',', $lf)"/>
               </xsl:if>
            </xsl:for-each>
         )
      throw (
         <!--
            Exceptions.
         -->
         <xsl:for-each select="UML:ModelElement.taggedValue/UML:TaggedValue">
            <xsl:if test="@tag = 'raises'">
               <xsl:value-of select="@value"/>,
            </xsl:if>
         </xsl:for-each>
         CORBA::SystemException
      )
      {
           CDMW_LOG_FUNCTION(m_logger.in(), "");
           <![CDATA[Cdmw::CCM::CIF::CCMObject_impl* component_impl = dynamic_cast<Cdmw::CCM::CIF::CCMObject_impl*>(m_component.in());
           CDMW_ASSERT(component_impl!=0);
           component_impl->check_validity_of_operational_request();]]>

           try
            {
             <![CDATA[CORBA::Object_var obj = m_executor_locator->obtain_executor(m_facet_name.c_str());]]>

            if (CORBA::is_nil(obj.in()))
            {
               CDMW_ERROR(m_logger.in(), 
                   "Executor for facet " <![CDATA[<<m_facet_name<<]]> " is nil");
               CORBA::OBJECT_NOT_EXIST ex(OrbSupport::OBJECT_NOT_EXIST, 
                                          CORBA::COMPLETED_NO);
               CDMW_LOG_FUNCTION_EXCEPTION(ex);
               throw ex;
            }

            <xsl:value-of select="concat($_cppFacetScope, $cppSep, 'CCM_', $_facetName, '_var')"/> facet_exec = <xsl:value-of select="concat($_cppFacetScope, $cppSep, 'CCM_', $_facetName)"/>::_narrow(obj.in());
    
            if (CORBA::is_nil(facet_exec.in()))
            {
               CDMW_ERROR(m_logger.in(), 
                   "Executor for facet " <![CDATA[<<m_facet_name<<]]> " has wrong type");
               CORBA::OBJECT_NOT_EXIST ex(OrbSupport::OBJECT_NOT_EXIST, 
                                          CORBA::COMPLETED_NO);
               CDMW_LOG_FUNCTION_EXCEPTION(ex);
               throw ex;
            }
    
            <xsl:if test="string-length($objRefVarType) > 0">
                <xsl:value-of select="concat($objRefVarType, ' result =')"/>
            </xsl:if>
            <![CDATA[facet_exec->]]><xsl:value-of select="$cppOperationName"/>
               (
                  <xsl:for-each select="UML:BehavioralFeature.parameter/UML:Parameter[@kind != 'return']">
                     <xsl:value-of select="@name"/>
                     <xsl:if test="position() != last()">
                        <xsl:text>, </xsl:text>
                     </xsl:if>
                  </xsl:for-each>
               );

            <![CDATA[m_executor_locator->release_executor(facet_exec.in());]]>

            <xsl:if test="string-length($objRefVarType) > 0">
               <xsl:choose>
                  <xsl:when test="contains($objRefVarType, '_var')">
                     return result._retn();
                  </xsl:when>
                  <xsl:otherwise>
                     return result;
                  </xsl:otherwise>
                </xsl:choose>
            </xsl:if>
         }
            catch (const <![CDATA[Components::CCMException& e]]> )
            {
               CDMW_ERROR(m_logger.in(), 
                   "CCMException received: "<![CDATA[<< e]]>);
               CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                                  CORBA::COMPLETED_NO);
               CDMW_LOG_FUNCTION_EXCEPTION(ex);
               throw ex;
            }
       }
   </xsl:for-each>

</xsl:template> <!-- end of template ccmcif_facet_cpp.content.operations.own -->
</xsl:stylesheet>


