<?xml version="1.0" encoding="utf-8"?>
<!-- ===================================================================== -->
<!--
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-->
<!-- ===================================================================== -->


<xsl:stylesheet version="1.1"
   xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
   xmlns:UML="org.omg/UML1.3">


<!--
   Main - This template fetches the necessary data from inputs
   to generate a file when FT.

   @param none
-->
<xsl:template name="ft_session_component_cpp">
   <xsl:call-template name="session_component_cpp">
      <xsl:with-param name="_hasFaultTolerance" select="'yes'"/>
   </xsl:call-template>
</xsl:template> <!-- end of template ft_session_home_cpp -->


<!--
   Main - This template fetches the necessary data from inputs
   to generate a file.

   @param none
-->
<xsl:template name="session_component_cpp">
   <xsl:param name="_hasFaultTolerance" select="'no'"/>
   <!--
      Parameters below are used for recursiveness.
   -->
   <xsl:param name="_index" select="1"/>
   
   <xsl:if test="boolean($lifecycleFrameworkNodes/home-impl) and $lifecycleFrameworkNodes/home-impl/@category = 'SESSION'">
      <xsl:if test="$_index &lt;= count($lifecycleFrameworkNodes/home-impl)">
         <xsl:variable name="homeImplNode" select="$lifecycleFrameworkNodes[home-impl and position() = $_index]/home-impl"/>

         <!--
            Check if the component is segmented
         -->
         <xsl:variable name="isSegmented">
             <xsl:choose>
                 <xsl:when test="boolean($homeImplNode/@segmented)">
                     <xsl:value-of select="$homeImplNode/@segmented"/>
                 </xsl:when>
                 <xsl:otherwise>
                     <xsl:value-of select="'yes'"/>
                 </xsl:otherwise>
             </xsl:choose>
         </xsl:variable>

         <!--
            Get the managed component.
         -->
         <xsl:variable name="homeName">
            <xsl:call-template name="getLastToken">
               <xsl:with-param name="_string" select="$homeImplNode/../@factory-idref"/>
               <xsl:with-param name="_separator" select="$xmlSep"/>
            </xsl:call-template>
         </xsl:variable>
         <xsl:variable name="managedComponentId">
            <xsl:call-template name="getManagedComponentId">
               <xsl:with-param name="_homeName" select="$homeName"/>
            </xsl:call-template>
         </xsl:variable>
         <xsl:variable name="managedComponentNode" select="key('classById', $managedComponentId)"/>

         <xsl:if test="boolean($managedComponentNode)">
            <xsl:variable name="componentImplClassname">
               <xsl:choose>
                  <xsl:when test="$_hasFaultTolerance = 'yes'">
                     <xsl:value-of select="concat('FTSession', $managedComponentNode/@name, '_impl')"/>
                  </xsl:when>
                  <xsl:otherwise>
                     <xsl:value-of select="concat('Session', $managedComponentNode/@name, '_impl')"/>
                  </xsl:otherwise>
               </xsl:choose>
            </xsl:variable>

            <!--
               Get the output filename.
            -->
            <xsl:variable name="filename">
               <xsl:call-template name="buildOutputFilename">
                  <xsl:with-param name="_userDirectory" select="$homeImplNode/@directory"/>
                  <xsl:with-param name="_userFilename" select="$homeImplNode/@filename"/>
                  <xsl:with-param name="_defaultFilename" select="$componentImplClassname"/>
                  <xsl:with-param name="_fileExtension" select="'.cpp'"/>
               </xsl:call-template>
            </xsl:variable>

            <!--
               Display a message to the user.
            -->
            <xsl:message>
               <xsl:value-of select="concat('Generating file ', $filename)"/>
            </xsl:message>

            <!--
               Generate the file.
            -->
            <xsl:document method="text" href="{$filename}">
               <!--
                  Insert the document header.
               -->
               <xsl:call-template name="docHeader">
                  <xsl:with-param name="_lang" select="'cpp'"/>
               </xsl:call-template>
               <!--
                  Insert the content.
               -->
               <xsl:call-template name="session_component_cpp.content">
                  <xsl:with-param name="_scopedHomeImplClassname">
                     <xsl:call-template name="buildHomeImplClassname">
                        <xsl:with-param name="_homeImplNode" select="$homeImplNode"/>
                     </xsl:call-template>
                  </xsl:with-param>
                  <xsl:with-param name="_homeName" select="$homeName" />
                  <xsl:with-param name="_componentNode" select="$managedComponentNode"/>
                  <xsl:with-param name="_isSegmented" select="$isSegmented"/>
                  <xsl:with-param name="_hasFaultTolerance" select="$_hasFaultTolerance"/>
               </xsl:call-template>
            </xsl:document>
         </xsl:if>

         <!--
            Proceed to the next home impl if any.
         -->
         <xsl:call-template name="session_component_cpp">
            <xsl:with-param name="_hasFaultTolerance" select="$_hasFaultTolerance"/>
            <xsl:with-param name="_index" select="$_index + 1"/>
         </xsl:call-template>
      </xsl:if>
   </xsl:if>
</xsl:template> <!-- end of template session_component_cpp -->



<!--
   This template generates the content of the file.

   @param _scopedHomeImplClassname   The scoped home impl classname.
   @param _facetName                 The component name.
-->
<xsl:template name="session_component_cpp.content">
   <xsl:param name="_scopedHomeImplClassname"/>
   <xsl:param name="_homeName"/>
   <xsl:param name="_componentNode"/>
   <xsl:param name="_isSegmented"/>
   <xsl:param name="_hasFaultTolerance"/>
   

   <xsl:variable name="componentName" select="$_componentNode/@name"/>

   <!--
      Miscellaneous variables.
   -->
   <xsl:variable name="cppScopedComponentName">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_componentNode"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="cppComponentScope">
      <xsl:call-template name="removeLastToken">
         <xsl:with-param name="_scopedName" select="$cppScopedComponentName"/>
         <xsl:with-param name="_sep" select="$cppSep"/>
      </xsl:call-template>
   </xsl:variable>
   <!--
      References to some nodes.
   -->
   <xsl:variable name="homeNode" select="key('classByName', $_homeName)"/>
   <xsl:variable name="cppScopedHomeName">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$homeNode"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="cppHomeScope">
      <xsl:call-template name="removeLastToken">
         <xsl:with-param name="_scopedName" select="$cppScopedHomeName"/>
         <xsl:with-param name="_sep" select="$cppSep"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="componentImplClassname">
      <xsl:choose>
         <xsl:when test="$_hasFaultTolerance = 'yes'">
            <xsl:value-of select="concat('FTSession', $componentName, '_impl')"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:value-of select="concat('Session', $componentName, '_impl')"/>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:variable>

   <xsl:variable name="componentObjRefVar" select="concat($cppComponentScope, $cppSep, 'CCM_', $componentName, '_var')"/>
   <xsl:variable name="componentObjRefPtr" select="concat($cppComponentScope, $cppSep, 'CCM_', $componentName, '_ptr')"/>
   <xsl:variable name="generalizations" select="key('generalizationById', $_componentNode/UML:GeneralizableElement.generalization/UML:Generalization/@xmi.idref)"/>
   <xsl:variable name="hasFacets">
      <xsl:call-template name="componentElements">
         <xsl:with-param name="_componentNode" select="$_componentNode"/>
         <xsl:with-param name="_whatToFind" select="'CCMProvides'"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="hasReceptacles">
      <xsl:call-template name="componentElements">
         <xsl:with-param name="_componentNode" select="$_componentNode"/>
         <xsl:with-param name="_whatToFind" select="'CCMUses'"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="hasEmitters">
      <xsl:call-template name="componentElements">
         <xsl:with-param name="_componentNode" select="$_componentNode"/>
         <xsl:with-param name="_whatToFind" select="'CCMEmits'"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="hasPublishers">
      <xsl:call-template name="componentElements">
         <xsl:with-param name="_componentNode" select="$_componentNode"/>
         <xsl:with-param name="_whatToFind" select="'CCMPublishes'"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="hasConsumers">
      <xsl:call-template name="componentElements">
         <xsl:with-param name="_componentNode" select="$_componentNode"/>
         <xsl:with-param name="_whatToFind" select="'CCMConsumes'"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="idlComponentScope">
      <xsl:call-template name="replaceCharsInString">
         <xsl:with-param name="_stringIn" select="$cppComponentScope"/>
         <xsl:with-param name="_charsIn" select="'::'"/>
         <xsl:with-param name="_charsOut" select="'/'"/>
      </xsl:call-template>
   </xsl:variable>



   <!--
      The code generation starts here.
   -->

   <xsl:variable name="pathPrefix">
       <xsl:choose>
           <xsl:when test="string-length($CSC_NAME) > 0">
               <xsl:value-of select="concat($CSC_NAME, $fileSep)"/>
           </xsl:when>
           <xsl:otherwise>
               <xsl:value-of select="''"/>
           </xsl:otherwise>
       </xsl:choose>
   </xsl:variable>
   
   <![CDATA[
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <CCMContainer/ccmcommon/CCMUtils.hpp>
#include <CCMContainer/ccmcif/CCMUtils.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>
   ]]>
   <![CDATA[#include "]]><xsl:value-of select="concat($pathPrefix, $componentImplClassname, '.hpp')"/><![CDATA["]]>
   <![CDATA[
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <SystemMngt/platforminterface/PlatformInterface.hpp>
#include <Foundation/idllib/CdmwDeployment.stub.hpp>
   
   ]]>

<xsl:if test="$_hasFaultTolerance = 'yes'">
<![CDATA[
#include <SystemMngt/platformvaluetypes/PropertyChange_impl.hpp>
#include <SystemMngt/platformlibrary/EventHeaderFactory.hpp>
]]>
</xsl:if>
   
   <!--
      Include all the contained facets.
   -->
   <xsl:call-template name="findAllComponentFacetsOfNode">
      <xsl:with-param name="_componentNode" select="$_componentNode"/>
      <xsl:with-param name="_template" select="'session_component_cpp.content.4'"/>
   </xsl:call-template>

   <!--
      Include all the contained consumers.
   -->
   <xsl:call-template name="findAllComponentConsumersOfNode">
      <xsl:with-param name="_componentNode" select="$_componentNode"/>
      <xsl:with-param name="_template" select="'session_component_cpp.content.24'"/>
   </xsl:call-template>

   <![CDATA[
#include <CCMContainer/idllib/CdmwCcmCif.skel.hpp>
   ]]>
   <![CDATA[#include "]]><xsl:value-of select="concat($pathPrefix, 'CCM_', $componentName, '_SessionContext_impl.hpp')"/><![CDATA["]]>

   <xsl:if test="$_isSegmented = 'no'">
      <xsl:if test="$hasConsumers = 'true'">
         <![CDATA[
#include <Foundation/orbsupport/Codec.hpp>
#include <Foundation/commonsvcs/time/Util.hpp>
#include <CdmwCcmCifC.h>

DEFINE_VARIABLE_LENGTH_DATA_TRAITS(CdmwCcmCif::TimestampedEvent)
         ]]>
         <xsl:call-template name="findAllComponentConsumersOfNode">
            <xsl:with-param name="_componentNode" select="$_componentNode"/>
            <xsl:with-param name="_template" select="'session_component_cpp.content.27'"/>
         </xsl:call-template>
      </xsl:if>
   </xsl:if>

   namespace {
        
      <xsl:if test="$_isSegmented = 'yes'">
      const char* COMP_REP_ID = "<xsl:value-of select="$_componentNode/UML:ModelElement.taggedValue/UML:TaggedValue[@tag = 'typeid']/@value"/>";
      </xsl:if>
      <xsl:if test="$_isSegmented = 'no'">
      const char* COMP_REP_ID = "IDL:thalesgroup.com/Cdmw_<xsl:value-of select="concat($idlComponentScope, '/', $componentName, ':1.0')"/>";
      </xsl:if>
      const char* COMP_NAME   = "<xsl:value-of select="$componentName"/>";

      <!--
         Facets.
      -->
      <xsl:call-template name="findAllComponentFacetsOfNode">
         <xsl:with-param name="_componentNode" select="$_componentNode"/>
         <xsl:with-param name="_template" select="'session_component_cpp.content.5'"/>
      </xsl:call-template>

      <!--
         Receptacles.
      -->
      <xsl:call-template name="findAllComponentReceptaclesOfNode">
         <xsl:with-param name="_componentNode" select="$_componentNode"/>
         <xsl:with-param name="_template" select="'session_component_cpp.content.10'"/>
      </xsl:call-template>
      
      <!--
         Publishers.
      -->

      <xsl:call-template name="findAllComponentPublishersOfNode">
         <xsl:with-param name="_componentNode" select="$_componentNode"/>
         <xsl:with-param name="_template" select="'session_component_cpp.content.13'"/>
      </xsl:call-template>
      
      <!--
         Emitters.
      -->
      
      <xsl:call-template name="findAllComponentEmittersOfNode">
         <xsl:with-param name="_componentNode" select="$_componentNode"/>
         <xsl:with-param name="_template" select="'session_component_cpp.content.14'"/>
      </xsl:call-template>

      <!--
         Consumers.
      -->

      <xsl:call-template name="findAllComponentConsumersOfNode">
         <xsl:with-param name="_componentNode" select="$_componentNode"/>
         <xsl:with-param name="_template" select="'session_component_cpp.content.19'"/>
      </xsl:call-template>
      
      /*
       * If a home, when creating a component, returns an EnterpriseComponent, we map it 
       * in an ExecutorLocator for code optimisation purpose. Thus the code is the same
       * for monolithic executor and segmented executor.
       */
      class <xsl:value-of select="$componentName"/>ExecutorLocator : public CdmwCcmCif::SessionExecutorLocator,
                                    public Cdmw::OrbSupport::RefCountLocalObject
      {
         public:
            <xsl:value-of select="$componentName"/>ExecutorLocator(
                  <xsl:value-of select="$componentObjRefPtr"/> executor,
                  CdmwLogging::Logger_ptr logger)
               : is_a_session_component(false),
               m_comp_executor(<xsl:value-of select="concat($cppComponentScope, $cppSep, 'CCM_', $componentName)"/>::_duplicate(executor)),
               m_logger(CdmwLogging::Logger::_duplicate(logger))
            {
               CDMW_LOG_FUNCTION(m_logger.in(), "");
               try 
               {
                  Components::SessionComponent_var session_comp
                     = Components::SessionComponent::_narrow(m_comp_executor.in());
            
                  if (!CORBA::is_nil(session_comp.in()))
                  {
                     is_a_session_component = true;
                  }
               }
               catch (...)
               {
               }
            };

            ~<xsl:value-of select="$componentName"/>ExecutorLocator() 
            {
                CDMW_LOG_FUNCTION(m_logger.in(), "");
            };

            /*
             * the cif will invoke its obtain_executor operation prior to each invocation to
             * retrieve the implementation for a port. The port name, given in the name
             * parameter, is the same as used in the components interface description 
             * in IDL, or the components name for the main executor. The obtain_executor
             * operation returns a local object reference of the expected type. The 
             * CCMException exception may be raised in case of a system error that 
             * prohibits locating the requested executor.
             */
            //
            // IDL:omg.org/Components/ExecutorLocator/obtain_executor:1.0
            //
            virtual CORBA::Object_ptr obtain_executor(const char* name)
               throw(Components::CCMException,
                     CORBA::SystemException)
            {
               CDMW_LOG_FUNCTION(m_logger.in(), <![CDATA["name="<<name]]>);
               std::string exec_name(name);
               CORBA::Object_var result;

               if (exec_name == "<xsl:value-of select="$componentName"/>")
               {
                  result = <xsl:value-of select="concat($cppComponentScope, $cppSep, 'CCM_', $componentName, $cppSep)"/>_duplicate(m_comp_executor.in());
               }
               <xsl:call-template name="findAllComponentFacetsOfNode">
                  <xsl:with-param name="_componentNode" select="$_componentNode"/>
                  <xsl:with-param name="_template" select="'session_component_cpp.content.6'"/>
               </xsl:call-template>
               <xsl:call-template name="findAllComponentConsumersOfNode">
                  <xsl:with-param name="_componentNode" select="$_componentNode"/>
                  <xsl:with-param name="_template" select="'session_component_cpp.content.20'"/>
               </xsl:call-template>
               else 
               {
                  CDMW_ERROR(m_logger.in(), 
                      <![CDATA["'"<<name<<"' is unkown: cannot obtain executor for it"]]>);
                  CORBA::BAD_PARAM ex(Cdmw::OrbSupport::BAD_PARAMInvalidExecutorName, CORBA::COMPLETED_NO);
                  CDMW_LOG_FUNCTION_EXCEPTION(ex);
                  throw ex;
               }

               return result._retn();
            };

            /* 
             * The release_executor operation is called by the cif once the current
             * invocation on an executor that was obtained through the obtain_executor 
             * operation has finished. The locator can thus release any resources that
             * were acquired as part of the obtain_executor operation.
             */
            //
            // IDL:omg.org/Components/ExecutorLocator/release_executor:1.0
            //
            virtual void release_executor(CORBA::Object_ptr exc)
               throw(Components::CCMException,
                     CORBA::SystemException)
            {
                CDMW_LOG_FUNCTION(m_logger.in(), "");
            };

            /* 
             * The configuration_complete operation is called to propagate the
             * configuration_complete operation on the CCMObject interface to the
             * component implementation.
             */
            //
            // IDL:omg.org/Components/ExecutorLocator/configuration_complete:1.0
            //
            virtual void configuration_complete()
               throw(Components::InvalidConfiguration,
                     CORBA::SystemException)
            {
               CDMW_LOG_FUNCTION(m_logger.in(), "");
               // No equivalent operation for monolithic executor
            };

            //
            // IDL:omg.org/Components/SessionComponent/set_session_context:1.0
            //
            virtual void set_session_context(Components::SessionContext_ptr ctx)
               throw(Components::CCMException,
                     CORBA::SystemException)
            {
               CDMW_LOG_FUNCTION(m_logger.in(), "");
               // check if component implements the SessionComponent local interface
               if (is_a_session_component)
               {
                  CDMW_INFO(m_logger.in(), "Call set_session_context()");
                  Components::SessionComponent_var session_comp 
                     = Components::SessionComponent::_narrow(m_comp_executor.in());
                  <![CDATA[session_comp->set_session_context(ctx);]]>
               }
            };

            /* 
             * The ccm_activate operation is called by the container to notify a 
             * session component that it has been made active. The component instance
             * should perform any initialization required prior to operation invocation.
             * The component may raise the CCMException with the SYSTEM_ERROR minor code
             * to indicate a failure caused by a system level error.
             */
            //
            // IDL:omg.org/Components/SessionComponent/ccm_activate:1.0
            //
            virtual void ccm_activate()
               throw(Components::CCMException,
                     CORBA::SystemException)
            {
               CDMW_LOG_FUNCTION(m_logger.in(), "");
               // check if component implements the SessionComponent local interface
               if (is_a_session_component)
               {
                  CDMW_INFO(m_logger.in(), "Call ccm_activate()");
                  Components::SessionComponent_var session_comp 
                     = Components::SessionComponent::_narrow(m_comp_executor.in());
                  <![CDATA[session_comp->ccm_activate();]]>
               }
            };

            /* 
             * The ccm_passivate operation is called by the container to notify a session
             * component that it has been made inactive. The component instance should
             * release any resources it acquired at activation time.The component may 
             * raise the CCMException with the SYSTEM_ERROR minor code to indicate a 
             * failure caused by a system level error.
             */
            //
            // IDL:omg.org/Components/SessionComponent/ccm_passivate:1.0
            //
            virtual void ccm_passivate()
               throw(Components::CCMException,
                     CORBA::SystemException)
            {
               CDMW_LOG_FUNCTION(m_logger.in(), "");
               // check if component implements the SessionComponent local interface
               if (is_a_session_component)
               {
                  CDMW_INFO(m_logger.in(), "Call ccm_passivate()");
                  Components::SessionComponent_var session_comp 
                     = Components::SessionComponent::_narrow(m_comp_executor.in());
                  <![CDATA[session_comp->ccm_passivate();]]>
               }
            };

            /* 
             * The ccm_remove operation is called by the container when the servant is
             * about to be destroyed. It informs the component that it is about to be
             * destroyed.The component may raise the CCMException with the SYSTEM_ERROR
             * minor code to indicate a failure caused by a system level error.
             */
            //
            // IDL:omg.org/Components/SessionComponent/ccm_remove:1.0
            //
            virtual void ccm_remove()
               throw(Components::CCMException,
                     CORBA::SystemException)
            {
               CDMW_LOG_FUNCTION(m_logger.in(), "");
               // check if component implements the SessionComponent local interface
               if (is_a_session_component)
               {
                  CDMW_INFO(m_logger.in(), "Call ccm_remove()");
                  Components::SessionComponent_var session_comp 
                     = Components::SessionComponent::_narrow(m_comp_executor.in());
                  <![CDATA[session_comp->ccm_remove();]]>
               }
            };

         private:
            <xsl:value-of select="$componentName"/>ExecutorLocator(const <xsl:value-of select="$componentName"/>ExecutorLocator<![CDATA[&);]]>
            void operator=(const <xsl:value-of select="$componentName"/>ExecutorLocator<![CDATA[&);]]>

            bool is_a_session_component;  // indicates that the component executor inherits
                                          // from the SessionComponent local interface.

            <xsl:value-of select="$componentObjRefVar"/> m_comp_executor; // points on Component monolithic executor.
            
            CdmwLogging::Logger_var m_logger;
      };
   };  // end of anonymous namespace

   <xsl:call-template name="openNamespace">
      <xsl:with-param name="_scope" select="concat('Cdmw::CCM::CIF::Cdmw',$cppHomeScope)"/>
      <xsl:with-param name="_separator" select="$cppSep"/>
      <xsl:with-param name="_lastTokenIsNamespace" select="true()"/>
   </xsl:call-template>

   <xsl:if test="$_hasFaultTolerance = 'yes'">
      // FT Activation Handler for the component
    class FTSession<xsl:value-of select="$componentName"/>FTActivationHandler_impl :
         virtual public ::CdmwFT::Location::ActivationHandler,
         virtual public Cdmw::OrbSupport::RefCountLocalObject
    {
    public:
        FTSession<xsl:value-of select="$componentName"/>FTActivationHandler_impl(
            FTSession<xsl:value-of select="$componentName"/>_impl* component_servant,
            CdmwLogging::Logger_ptr logger)
          : m_component_servant(component_servant),
            m_logger(CdmwLogging::Logger::_duplicate(logger))
        {
            CDMW_LOG_FUNCTION(m_logger.in(), "");
        }

        //
        // IDL:thalesgroup.com/CdmwFT/Location/ActivationHandler/activate:1.0
        //
        virtual void activate()
            throw(CORBA::SystemException)
        {
            CDMW_LOG_FUNCTION(m_logger.in(), "");
            CDMW_DEBUG(m_logger.in(), "Component is elected as FT primary");
            // call FTCCMObject_impl::ft_activate()
            <![CDATA[m_component_servant->ft_activate();]]>
        }

        //
        // IDL:thalesgroup.com/CdmwFT/Location/ActivationHandler/deactivate:1.0
        //
        virtual void deactivate()
            throw(CORBA::SystemException)
        {
            // call FTCCMObject_impl::ft_deactivate()
            <![CDATA[m_component_servant->ft_deactivate();]]>
        }

    private:
        FTSession<xsl:value-of select="$componentName"/>FTActivationHandler_impl(const FTSession<xsl:value-of select="$componentName"/>FTActivationHandler_impl<![CDATA[&);]]>
        FTSession<xsl:value-of select="$componentName"/>FTActivationHandler_impl<![CDATA[&]]> operator=(const FTSession<xsl:value-of select="$componentName"/>FTActivationHandler_impl<![CDATA[&);]]>

        FTSession<xsl:value-of select="$componentName"/>_impl* m_component_servant;
        
        CdmwLogging::Logger_var m_logger;

    };
   </xsl:if>


   // Constructor
   <xsl:value-of select="concat($componentImplClassname, $cppSep, $componentImplClassname)"/>(const <![CDATA[std::string&]]> comp_oid,
                         Context*                          ctx,
                         CORBA::Object_ptr                 comp_ref,
                         Components::EnterpriseComponent*  executor,
      <xsl:if test="$_hasFaultTolerance = 'yes'">
                         ::FT::ObjectGroup_ptr             group_ref,
      </xsl:if>
                         CdmwLifeCycle::ObjectRegistration_ptr object_registration,
                         CdmwLogging::LoggerFactory_ptr    logger_factory,
                         bool is_created_by_factory_operation)
      throw (Components::CCMException,
             CORBA::SystemException)
      <xsl:if test="$_hasFaultTolerance = 'no'">
         <xsl:if test="$_isSegmented = 'yes'">
      : CCMObject_impl(comp_oid, ctx, COMP_REP_ID, comp_ref, object_registration, logger_factory),
         </xsl:if>
         <xsl:if test="$_isSegmented = 'no'">
      : CCMObject_impl(comp_oid, ctx, COMP_REP_ID, comp_ref, object_registration, logger_factory, true),
         </xsl:if>
      </xsl:if>
      <xsl:if test="$_hasFaultTolerance = 'yes'">
         <xsl:if test="$_isSegmented = 'yes'">
      : FTCCMObject_impl(comp_oid, ctx, COMP_REP_ID, comp_ref, object_registration, logger_factory),
         </xsl:if>
         <xsl:if test="$_isSegmented = 'no'">
      : FTCCMObject_impl(comp_oid, ctx, COMP_REP_ID, comp_ref, object_registration, logger_factory, true),
         </xsl:if>
      </xsl:if>
      <xsl:if test="$_hasFaultTolerance = 'yes'">
        m_group_ref(::FT::ObjectGroup::_duplicate(group_ref)),
        m_activation_handler(::CdmwFT::Location::ActivationHandler::_nil()),
        m_activation_manager(::CdmwFT::Location::ActivationManager::_nil()),
        m_activation_handler_id(0L),
      </xsl:if>
        m_is_created_by_factory_operation(is_created_by_factory_operation)
      <xsl:call-template name="findAllInterfaceReadWriteAttributes">
         <xsl:with-param name="_interfaceNode" select="$_componentNode"/>
         <xsl:with-param name="_type" select="'Component'"/>
         <xsl:with-param name="_template" select="'session_component_cpp.content.1'"/>
      </xsl:call-template>
   {
      // create logger with domain for ccmcif
      m_logger = logger_factory->get_logger(CCMLoggers::CCMGENERATED_LOG_DOMAIN);
   
      CDMW_LOG_FUNCTION(m_logger.in(), <![CDATA["comp_oid="<<comp_oid]]>);
      
      <xsl:if test="$_hasFaultTolerance = 'yes'">
      // set state to FT INITIAL
      m_state = Cdmw::CCM::CIF::FT_CCM_OBJECT_STATE_INITIAL;
      </xsl:if>
      
      // Check if the home returns an executor locator
      m_executor_locator = Components::ExecutorLocator::_narrow(executor);

      if (CORBA::is_nil(m_executor_locator.in()))
      {
         // executor is not an executor locator
         // map monolithic executor to executor locator
         <xsl:value-of select="$componentObjRefVar"/> comp_exec = <xsl:value-of select="concat($cppComponentScope, $cppSep, 'CCM_',  $componentName)"/>::_narrow(executor);
        
         if (CORBA::is_nil(comp_exec.in()))
         {
            // This is to avoid Assertion failure in debug mode as internal ref count
            // equals 1 (CORBA C++ mapping oblige!)
            _set_ref_count(0);

            CDMW_ERROR(m_logger.in(), "Invalid Component Executor");
            CORBA::BAD_PARAM ex(OrbSupport::BAD_PARAMInvalidComponentExecutor,
                                CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
         }
         m_executor_locator = new <xsl:value-of select="$componentName"/>ExecutorLocator(comp_exec.in(), m_logger.in());

         m_session_locator = Components::SessionComponent::_narrow(m_executor_locator.in());
        
         if (CORBA::is_nil(m_session_locator.in()))
         {
             CDMW_ERROR(m_logger.in(), 
                "Executor Locator should be a session component!");
             CORBA::INTERNAL ex(OrbSupport::INTERNALInvalidExecutor,
                                CORBA::COMPLETED_NO);
             CDMW_LOG_FUNCTION_EXCEPTION(ex);
             throw ex;
         }
      }
      else
      {
         Components::SessionComponent_var m_session_locator 
            = Components::SessionComponent::_narrow(m_executor_locator.in());
        
         if (CORBA::is_nil(m_session_locator.in()))
         {
             CDMW_ERROR(m_logger.in(), 
                "Executor Locator should be a session component!");
             CORBA::BAD_PARAM ex(OrbSupport::BAD_PARAMInvalidComponentExecutor,
                                CORBA::COMPLETED_NO);
             CDMW_LOG_FUNCTION_EXCEPTION(ex);
             throw ex;
         }
      }

      <![CDATA[Components::CCMContext_var ccm_ctx = ctx->get_ccm_context();]]>
      m_context_executor = new CCM_<xsl:value-of select="$componentName"/>_SessionContext_impl(ccm_ctx.in(), this, logger_factory);
      m_session_locator->set_session_context(m_context_executor.in());

      <xsl:if test="$_hasFaultTolerance = 'yes'">
      // component is replicated, it shall be activated by FT framework
      // To do so, we shall create a FT ActivationHandler
      // Context has to be created before since it's required if this component
      // already has been elected as primary (Supervision has to be notified)
      m_activation_handler = new FTSession<xsl:value-of select="$componentName"/>FTActivationHandler_impl(this, m_logger.in());

<![CDATA[         

      CORBA::Object_var am_ref = 
            m_context->resolve_initial_references("FTActivationManager");
      m_activation_manager = 
            CdmwFT::Location::ActivationManager::_narrow(am_ref.in());
      m_activation_handler_id = 
            m_activation_manager->register_activation_handler(m_group_ref.in(),
                                                              m_activation_handler.in());
]]>
      </xsl:if>
      
<![CDATA[      
      //
      // QoS services installation
      //
      
      // get COMPONENT_NAME from ConfigValues
      const char* component_name = 0;
      try {
          Components::ConfigValues_var config_values = 
              ctx->get_config_values(comp_oid);
          for( CORBA::ULong i = 0; i < config_values->length(); i++ ) {
              if( ! strcmp (::CdmwDeployment::COMPONENT_NAME, config_values[i]->name())) 
              {
                  // extract COMPONENT_NAME
                  config_values[i]->value() >>= component_name;
              }
          }
          
      } catch (const Cdmw::Common::NotFoundException& e) {
          // No config values for this component.
          // Cannot get COMPONENT_NAME nor install services
          CDMW_WARN(m_logger.in(), 
              "No ConfigValues set for this component.");
          // continue anyway
      }

      // install ManagedComponentContext service
      if (component_name != 0) {
          try {
              // get ManagedComponentContextFactory object
              CORBA::Object_var obj = 
                  m_context_executor->resolve_service_reference(
                      Cdmw::PlatformMngt::PlatformInterface::MANAGED_COMPONENT_CONTEXT_FACTORY_NAME);
              CdmwPlatformMngt::ManagedComponentContextFactory_var mcContextFactory =
                  CdmwPlatformMngt::ManagedComponentContextFactory::_narrow(obj.in());
                  
              // get ComponentContext object
              CdmwPlatformMngt::ManagedElementContexts::ComponentContext_var mcContext =
                  mcContextFactory->find_managed_component_context(component_name);
                                                                      
              // install service
              Components::Cookie_var cookie =
                  m_context_executor->install_service_reference(
                      "ManagedComponentContext",
                      mcContext.in());
                  
          } catch (const Components::CCMException& e) {
              if (e.reason == Components::OBJECT_NOT_FOUND) { 
                  // Cannot find ManagedComponentContextFactory object
                  CDMW_WARN(m_logger.in(), 
                      "Cannot find service ManagedComponentContextFactory" 
                      << " Cannot install ManagedComponentContext service.");
                  // continue anyway
              } else if (e.reason == Components::SERVICE_INSTALLATION_ERROR) {
                  // Cannot install ManagedComponentContext service
                  CDMW_WARN(m_logger.in(), 
                      "Cannot install ManagedComponentContext service: already installed.");
                  // continue anyway
              } else {
                  // Cannot find ManagedComponentContextFactory object
                  CDMW_WARN(m_logger.in(), 
                      "Cannot install ManagedComponentContext service for unknown reason");
                  // continue anyway
              }
          } catch (const CdmwPlatformMngt::InvalidName& e) {
                  // Cannot find ManagedComponentContext object
                  CDMW_WARN(m_logger.in(), 
                      "Cannot find ManagedComponentContext for component"
                      << component_name);
                  // continue anyway
          }
          catch (const CORBA::Exception & e) {
                  // This is to avoid Assertion failure in debug mode as internal ref count
                  // equals 1 (CORBA C++ mapping oblige!)
                  _set_ref_count(0);
                  CDMW_LOG_FUNCTION_EXCEPTION(e);
          }

      }
      else 
      {
           CDMW_WARN(m_logger.in(), 
               "No COMPONENT_NAME ConfigValue."
               << " Cannot install ManagedComponentContext service");
      }

      // install LoggerFactory service
      try {
          // install service
          Components::Cookie_var cookie =
              m_context_executor->install_service_reference(
                  "LoggerFactory",
                  logger_factory);
      
      } catch (const Components::CCMException& e) {
          if (e.reason == Components::SERVICE_INSTALLATION_ERROR) {
              // Cannot install LoggerFactory service
              CDMW_WARN(m_logger.in(), 
                  "Cannot install LoggerFactory service: already installed.");
              // continue anyway
          } else {
              // Cannot find LoggerFactory object
              CDMW_WARN(m_logger.in(), 
                  "Cannot install LoggerFactory service for unknown reason");
              // continue anyway
          }
      }
]]>      
      
      <xsl:call-template name="findAllComponentFacetsOfNode">
         <xsl:with-param name="_componentNode" select="$_componentNode"/>
         <xsl:with-param name="_template" select="'session_component_cpp.content.7'"/>
      </xsl:call-template>

      <xsl:if test="$hasReceptacles = 'true'">
         try
         {
            <xsl:call-template name="findAllComponentReceptaclesOfNode">
               <xsl:with-param name="_componentNode" select="$_componentNode"/>
               <xsl:with-param name="_template" select="'session_component_cpp.content.11'"/>
            </xsl:call-template>
         }
         catch (const <![CDATA[AlreadyDoneException&]]>)
         {
            // This is to avoid Assertion failure in debug mode as internal ref count
            // equals 1 (CORBA C++ mapping oblige!)
            _set_ref_count(0);

            CDMW_ERROR(m_logger.in(), 
                "Receptacle already declared!");
            // Don't propagate while its probably a code generation error!
         }
         catch (const <![CDATA[InternalErrorException&]]>)
         {
            // This is to avoid Assertion failure in debug mode as internal ref count
            // equals 1 (CORBA C++ mapping oblige!)
            _set_ref_count(0);
        
            CDMW_ERROR(m_logger.in(), 
                "Internal exception raised when declaring receptacle!");
            CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
         }
         catch (const <![CDATA[CORBA::SystemException& e]]>)
         {
            // This is to avoid Assertion failure in debug mode as internal ref count
            // equals 1 (CORBA C++ mapping oblige!)
            _set_ref_count(0);
        
            CDMW_ERROR(m_logger.in(), 
                "System exception raised when declaring receptacle: " <![CDATA[<< e]]>);
            CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
         }
      </xsl:if>

      <xsl:if test="$hasEmitters = 'true' or $hasPublishers = 'true'">
         try
         {
            <xsl:if test="$hasPublishers = 'true'">
               <xsl:call-template name="findAllComponentPublishersOfNode">
                  <xsl:with-param name="_componentNode" select="$_componentNode"/>
                  <xsl:with-param name="_template" select="'session_component_cpp.content.15'"/>
               </xsl:call-template>
            </xsl:if>
            <xsl:if test="$hasEmitters = 'true'">
               <xsl:call-template name="findAllComponentEmittersOfNode">
                  <xsl:with-param name="_componentNode" select="$_componentNode"/>
                  <xsl:with-param name="_template" select="'session_component_cpp.content.16'"/>
               </xsl:call-template>
            </xsl:if>
         }
         catch (const <![CDATA[AlreadyDoneException&]]>)
         {
            // This is to avoid Assertion failure in debug mode as internal ref count
            // equals 1 (CORBA C++ mapping oblige!)
            _set_ref_count(0);

            CDMW_ERROR(m_logger.in(), "Event source already declared!");
            // Don't propagate while its probably a code generation error!
         }
         catch (const <![CDATA[CORBA::SystemException& e]]>)
         {
            // This is to avoid Assertion failure in debug mode as internal ref count
            // equals 1 (CORBA C++ mapping oblige!)
            _set_ref_count(0);
        
            CDMW_ERROR(m_logger.in(), 
                "System exception raised when declaring event source: " <![CDATA[<< e]]>);
            CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
         }
      </xsl:if>

      <xsl:if test="$hasConsumers = 'true'">
         try
         {
            <xsl:call-template name="findAllComponentConsumersOfNode">
               <xsl:with-param name="_componentNode" select="$_componentNode"/>
               <xsl:with-param name="_template" select="'session_component_cpp.content.21'"/>
            </xsl:call-template>
         }
         catch (const <![CDATA[AlreadyDoneException&]]>)
         {
            // This is to avoid Assertion failure in debug mode as internal ref count
            // equals 1 (CORBA C++ mapping oblige!)
            _set_ref_count(0);

            CDMW_ERROR(m_logger.in(), "Consumer already declared!");
            // Don't propagate while its probably a code generation error!
         }
         catch (const <![CDATA[InternalErrorException&]]>)
         {
            // This is to avoid Assertion failure in debug mode as internal ref count
            // equals 1 (CORBA C++ mapping oblige!)
            _set_ref_count(0);
        
            CDMW_ERROR(m_logger.in(), "Internal exception raised when declaring consumer!");
            CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
         }
         catch (const <![CDATA[CORBA::SystemException& e]]>)
         {
            // This is to avoid Assertion failure in debug mode as internal ref count
            // equals 1 (CORBA C++ mapping oblige!)
            _set_ref_count(0);
        
            CDMW_ERROR(m_logger.in(), 
                "System exception raised when declaring consumer: " <![CDATA[<< e]]>);
            CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
         }
      </xsl:if>

   }

   <xsl:value-of select="concat($componentImplClassname, $cppSep, '~', $componentImplClassname, '()')"/>
   {
      CDMW_LOG_FUNCTION(m_logger.in(), "");
      <xsl:if test="$_hasFaultTolerance = 'yes'">
      try
      {
          <![CDATA[m_activation_manager->unregister_activation_handler(m_activation_handler_id);]]>

          // Change state to REMOVED; this will call ccm_passivate and ccm_remove on executor
          m_state = m_state->remove(this);
      }
      catch (const  <![CDATA[Components::CCMException& e]]>)
      {
          CDMW_ERROR(m_logger.in(),
             "Components::CCMException received in  <xsl:value-of select="concat($componentImplClassname, $cppSep, '~', $componentImplClassname, '()')"/>");
      }
      catch (const  <![CDATA[CORBA::SystemException& e]]>)
      {
          CDMW_ERROR(m_logger.in(),
             "CORBA::SystemException received in  <xsl:value-of select="concat($componentImplClassname, $cppSep, '~', $componentImplClassname, '()')"/>");
      }
      catch (...)
      {
          CDMW_ERROR(m_logger.in(),
             "unkown exception received in  <xsl:value-of select="concat($componentImplClassname, $cppSep, '~', $componentImplClassname, '()')"/>");
      }
      </xsl:if>
   }


   /**
    * Component attributes
    */
   <xsl:call-template name="findAllInterfaceReadWriteAttributes">
      <xsl:with-param name="_interfaceNode" select="$_componentNode"/>
      <xsl:with-param name="_type" select="'Component'"/>
      <xsl:with-param name="_template" select="'session_component_cpp.content.2'"/>
      <xsl:with-param name="_hasFaultTolerance" select="$_hasFaultTolerance"/>
   </xsl:call-template>

   /**
    * Supported interfaces
    */

   <!--
      Define supported interfaces operations.
   -->
   <xsl:variable name="allBaseInterfaces">
       <xsl:call-template name="getAllBaseClasses">
           <xsl:with-param name="_classNode" select="$_componentNode"/>
       </xsl:call-template>
   </xsl:variable>
   <xsl:for-each select="$allBaseInterfaces/node()[string(@xmi.id)]">
       <xsl:sort select="@xmi.id"/>
       <xsl:variable name="_xmi_id" select="./@xmi.id"/>
       <!--
       NOTA: Note the following trick to get back to the initial document context
             to be able to access operation definitions ... etc. because the result
             tree ($allBaseInterfaces) is a new xml document a only contains a copy
             of a subset of the original tree.
       -->
       <xsl:for-each select="$_componentNode">
           <xsl:variable name="_interface" select="key('classById', $_xmi_id)"/>
           <xsl:call-template name="session_component_cpp.SupportedInterfaces.Operations.own">
               <xsl:with-param name="_interfaceNode" select="$_interface" />
               <xsl:with-param name="_componentImplClassname" select="$componentImplClassname" />
               <xsl:with-param name="_componentObjRefVar" select="$componentObjRefVar" />
           </xsl:call-template>
       </xsl:for-each>
   </xsl:for-each>







   /**
    * Equivalent IDL methods
    */

   // Facets

   <xsl:call-template name="findAllComponentFacetsOfNode">
      <xsl:with-param name="_componentNode" select="$_componentNode"/>
      <xsl:with-param name="_template" select="'session_component_cpp.Facets.EquivalentIDL.Definition'"/>
      <xsl:with-param name="_isSegmented" select="$_isSegmented"/>
      <xsl:with-param name="_hasFaultTolerance" select="$_hasFaultTolerance"/>
   </xsl:call-template>

   // Receptacles

   <xsl:call-template name="findAllComponentReceptaclesOfNode">
      <xsl:with-param name="_componentNode" select="$_componentNode"/>
      <xsl:with-param name="_template" select="'session_component_cpp.content.12'"/>
      <xsl:with-param name="_hasFaultTolerance" select="$_hasFaultTolerance"/>
   </xsl:call-template>

   // Publishers

   <xsl:call-template name="findAllComponentPublishersOfNode">
      <xsl:with-param name="_componentNode" select="$_componentNode"/>
      <xsl:with-param name="_template" select="'session_component_cpp.content.17'"/>
      <xsl:with-param name="_hasFaultTolerance" select="$_hasFaultTolerance"/>
   </xsl:call-template>
   
   // Emitters

   <xsl:call-template name="findAllComponentEmittersOfNode">
      <xsl:with-param name="_componentNode" select="$_componentNode"/>
      <xsl:with-param name="_template" select="'session_component_cpp.content.18'"/>
      <xsl:with-param name="_hasFaultTolerance" select="$_hasFaultTolerance"/>
   </xsl:call-template>
   
   // Consumers
   
   <xsl:if test="$_isSegmented = 'no'">
      <xsl:if test="$hasConsumers = 'true'">
         void
         <xsl:value-of select="$componentImplClassname"/>::push_event(Components::EventBase * evt)
            throw (Components::BadEventType, CORBA::SystemException)
         {
            CDMW_LOG_FUNCTION(m_logger.in(), "");
            check_is_removed();
   
            bool bad_evt_type_found = true;
            
            <xsl:call-template name="findAllComponentConsumersOfNode">
               <xsl:with-param name="_componentNode" select="$_componentNode"/>
               <xsl:with-param name="_template" select="'session_component_cpp.content.25'"/>
            </xsl:call-template>
            
            if (bad_evt_type_found)
            {
               CDMW_WARN(m_logger.in(),
                   "EventBase with unexpected type received");
               Components::BadEventType ex;
               CDMW_LOG_FUNCTION_EXCEPTION(ex);
               throw ex;
            }
         }
   
         void
         <xsl:value-of select="$componentImplClassname"/>::push(const <![CDATA[CORBA::Any& data]]>)
            throw (CosEventComm::Disconnected, 
                   CORBA::SystemException)
         {
            CDMW_LOG_FUNCTION(m_logger.in(), "");
            check_is_removed();
   
            bool bad_evt_type_found = true;
            
            <xsl:call-template name="findAllComponentConsumersOfNode">
               <xsl:with-param name="_componentNode" select="$_componentNode"/>
               <xsl:with-param name="_template" select="'session_component_cpp.content.26'"/>
            </xsl:call-template>
            
            if (bad_evt_type_found)
            {
               CORBA::TypeCode_var tc = data.type();
               CDMW_WARN(m_logger.in(),
                   "Event with unexpected type received: " <![CDATA[<< tc->id()]]>);
               CORBA::BAD_PARAM ex(OrbSupport::BAD_PARAMBadEventType, CORBA::COMPLETED_NO);
               CDMW_LOG_FUNCTION_EXCEPTION(ex);
               throw ex;
            }
         }
   
         void
         <xsl:value-of select="$componentImplClassname"/>::disconnect_push_consumer()
         throw (CORBA::SystemException)
         {
             CDMW_LOG_FUNCTION(m_logger.in(), "");
             check_is_removed();
         }
   
      </xsl:if>
   </xsl:if>

   <xsl:call-template name="findAllComponentConsumersOfNode">
      <xsl:with-param name="_componentNode" select="$_componentNode"/>
      <xsl:with-param name="_template" select="'session_component_cpp.content.22'"/>
      <xsl:with-param name="_isSegmented" select="$_isSegmented"/>
      <xsl:with-param name="_hasFaultTolerance" select="$_hasFaultTolerance"/>
   </xsl:call-template>

   void
   <xsl:value-of select="$componentImplClassname"/>::configuration_complete()
      throw(Components::InvalidConfiguration,
            CORBA::SystemException)
   {
      CDMW_LOG_FUNCTION(m_logger.in(), "");
      check_is_removed();

      if (!is_configured())
      {
         CDMW_DEBUG(m_logger.in(), "Check if configuration is complete");
      
            <xsl:call-template name="findAllInterfaceReadWriteAttributes">
               <xsl:with-param name="_interfaceNode" select="$_componentNode"/>
               <xsl:with-param name="_type" select="'Component'"/>
               <xsl:with-param name="_template" select="'session_component_cpp.content.3'"/>
            </xsl:call-template>
         
         CCMObject_impl::configuration_complete();
        
         try
         {
            <![CDATA[
            m_executor_locator->configuration_complete();
            
            // change state
            m_state = m_state->configuration_complete(this);
            ]]>

         }
         catch (const <![CDATA[Components::CCMException& e]]>)
         {
            <![CDATA[
            CDMW_ERROR(m_logger.in(), "Components::CCMException raised, reason " << e);
            ]]>
            CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError,
                               CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
         }
      }
   }

   <xsl:value-of select="concat($lf, $cppComponentScope, $cppSep, 'CCM_', $componentName, '*')"/>
   <xsl:value-of select="concat($lf, $componentImplClassname)"/>::obtain_component_executor()
      throw (CORBA::SystemException)
   {
      CDMW_LOG_FUNCTION(m_logger.in(), "");
      check_is_removed();

      try
      {
         <![CDATA[CORBA::Object_var obj = m_executor_locator->obtain_executor(COMP_NAME);]]>

         if (CORBA::is_nil(obj.in()))
         {
            CDMW_ERROR(m_logger.in(), "Component executor is nil");
            CORBA::OBJECT_NOT_EXIST ex(OrbSupport::OBJECT_NOT_EXISTInvalidExecutor,
                                       CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
         }

         <xsl:value-of select="$componentObjRefVar"/> comp_exec = <xsl:value-of select="concat($cppComponentScope, $cppSep, 'CCM_', $componentName)"/>::_narrow(obj.in());
    
         if (CORBA::is_nil(comp_exec.in()))
         {
            CDMW_ERROR(m_logger.in(), 
                "Component executor is not a <xsl:value-of select="concat($cppComponentScope, $cppSep, 'CCM_', $componentName)"/>");
            CORBA::OBJECT_NOT_EXIST ex(OrbSupport::OBJECT_NOT_EXISTInvalidExecutor,
                                       CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
         }
    
         return comp_exec._retn();
      }
      catch (const <![CDATA[Components::CCMException& e]]>)
      {
         CDMW_ERROR(m_logger.in(), 
            "CCMException obtaining component executor: " <![CDATA[<<e]]>);
         CORBA::OBJECT_NOT_EXIST ex(OrbSupport::OBJECT_NOT_EXISTInvalidExecutor,
                                    CORBA::COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(ex);
         throw ex;
      }
      catch (const <![CDATA[CORBA::BAD_PARAM& e]]>)
      {
         CDMW_ERROR(m_logger.in(), 
            "CORBA::BAD_PARAM obtaining component executor");
         CORBA::OBJECT_NOT_EXIST ex(OrbSupport::OBJECT_NOT_EXISTInvalidExecutor,
                                    CORBA::COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(ex);
         throw ex;
      }
   }

   <![CDATA[
   /**
    * Purpose:
    * <p>This returns the servant for the facet/consumer specified by facet_name.
    * This operation is implemented by the well typed component servant.
    *
    * @param facet_name The facet name
    *
    * @return the facet servant.
    */
   ]]>
   <xsl:choose>
      <xsl:when test="$hasFacets = 'true' or $hasConsumers = 'true'">
         PortableServer::Servant
         <xsl:value-of select="$componentImplClassname"/>::get_facet_servant(const <![CDATA[std::string&]]> facet_name) 
            throw (CORBA::SystemException)
         {
            CDMW_LOG_FUNCTION(m_logger.in(), <![CDATA["facet_name="<<facet_name]]>);
            check_is_removed();

            PortableServer::ServantBase_var facet_servant;
          
            <xsl:if test="$hasFacets = 'true'">
               <xsl:call-template name="findAllComponentFacetsOfNode">
                  <xsl:with-param name="_componentNode" select="$_componentNode"/>
                  <xsl:with-param name="_template" select="'session_component_cpp.content.9'"/>
               </xsl:call-template>
            </xsl:if>
            <xsl:if test="$hasConsumers = 'true'">
               <xsl:call-template name="findAllComponentConsumersOfNode">
                  <xsl:with-param name="_componentNode" select="$_componentNode"/>
                  <xsl:with-param name="_template" select="'session_component_cpp.content.23'"/>
               </xsl:call-template>
            </xsl:if>
            {
               // No facet/consumer with this name define for this component
               CDMW_ERROR(m_logger.in(), 
                   "No '" <![CDATA[<< facet_name <<]]> 
                   "' facet or consumer defined for this component");
               CORBA::OBJECT_NOT_EXIST ex(Cdmw::OrbSupport::OBJECT_NOT_EXIST,
                                          CORBA::COMPLETED_NO); 
               CDMW_LOG_FUNCTION_EXCEPTION(ex);
               throw ex;
            }
          
            return facet_servant._retn();
         }
      </xsl:when>
      <xsl:otherwise>
         PortableServer::Servant
         <xsl:value-of select="$componentImplClassname"/>::get_facet_servant(const <![CDATA[std::string&]]> facet_name) 
            throw (CORBA::SystemException)
         {
            CDMW_LOG_FUNCTION(m_logger.in(), <![CDATA["facet_name="<<facet_name]]>);
            check_is_removed();

            // No facet with this name define for this component
             CDMW_ERROR(m_logger.in(), 
                 "No '" <![CDATA[<< facet_name <<]]> 
                 "' facet or consumer defined for this component");
                 
            CORBA::OBJECT_NOT_EXIST ex(Cdmw::OrbSupport::OBJECT_NOT_EXIST,
                                          CORBA::COMPLETED_NO); 
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
         }
      </xsl:otherwise>
   </xsl:choose>

   
   <![CDATA[
    /**
     * Purpose:
     * <p>In case of a session component, this operation will call 
     * it's executor's ccm_activate operation.
     *
     *@exception Components::CCMException Any CCMException thrown by the
     * executor's ccm_activate operation.
     *@exception CORBA::SystemException Any SystemException thrown by the
     * executor's ccm_activate operation.
     */ 
   ]]>
    void <xsl:value-of select="$componentImplClassname"/>::ccm_activate() 
        throw (Components::CCMException, CORBA::SystemException)
    {
        // this is a session component, call ccm_activate
        m_session_locator->ccm_activate();
    }
   
   
   <![CDATA[
    /**
     * Purpose:
     * <p>In case of a session component, this operation will call 
     * it's executor's ccm_passivate operation.
     *
     *@exception Components::CCMException Any CCMException thrown by the
     * executor's ccm_passivate operation.
     *@exception CORBA::SystemException Any SystemException thrown by the
     * executor's ccm_passivate operation.
     */ 
   ]]>
    void <xsl:value-of select="$componentImplClassname"/>::ccm_passivate() 
        throw (Components::CCMException, CORBA::SystemException)
    {
        // this is a session component, call ccm_passivate
        m_session_locator->ccm_passivate();
    }
   
   
   <![CDATA[
    /**
     * Purpose:
     * <p>In case of a session component, this operation will call 
     * it's executor's ccm_remove operation.
     *
     *@exception Components::CCMException Any CCMException thrown by the
     * executor's ccm_remove operation.
     *@exception CORBA::SystemException Any SystemException thrown by the
     * executor's ccm_remove operation.
     */ 
   ]]>
    void <xsl:value-of select="$componentImplClassname"/>::ccm_remove() 
        throw (Components::CCMException, CORBA::SystemException)
    {
        // this is a session component, call ccm_remove
        m_session_locator->ccm_remove();
    }
   
   
   // Change state to REMOVED; this will call ccm_passivate and ccm_remove on executor
   void 
   <xsl:value-of select="$componentImplClassname"/>::advise_executor_of_remove_component()
      throw (CORBA::SystemException)
   {
      CDMW_LOG_FUNCTION(m_logger.in(), "");
      check_is_removed();

      try
      {
         m_state = m_state->remove(this);
      }
      catch (const <![CDATA[Components::CCMException& e]]>)
      {
         <![CDATA[
         CDMW_ERROR(m_logger.in(), "Components::CCMException raised, reason " << e);
         ]]>
         CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError,
                            CORBA::COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(ex);
         throw ex;
      }
   }

 <xsl:if test="$_hasFaultTolerance = 'yes'">
   void 
   <xsl:value-of select="$componentImplClassname"/>::notify_ft_activate()
   <![CDATA[
   {
      CDMW_LOG_FUNCTION(m_logger.in(), "");
      CDMW_DEBUG(m_logger.in(), "notify_ft_activate");
      try {
          // get ManagedComponentContext object
          CORBA::Object_var obj =
              m_context_executor->resolve_service_reference(
                  "ManagedComponentContext");
          CdmwPlatformMngt::ManagedElementContexts::ComponentContextExt_var mcContext =
              CdmwPlatformMngt::ManagedElementContexts::ComponentContextExt::_narrow(obj.in());
          
          // get component element path
          CdmwPlatformMngt::ManagedElement_var element = 
              mcContext->the_managed_element();
          // In case the element was not found 
          if (CORBA::is_nil(element.in()) )
          {
              CDMW_WARN(::Cdmw::CCM::CCMLoggers::Get_ccmgenerated_logger(), "Could not find managed element !");
              return;
          }
          CORBA::String_var element_path = element->element_path();
              
          // create PropertyChange EventHeader
          CdmwPlatformMngtBase::EventHeader header = 
              Cdmw::PlatformMngt::EventHeaderFactory::createHeader (CdmwPlatformMngtBase::INF);
          // create PropertyChange PropertyInfos
          CdmwPlatformMngt::PropertyInfos_var p_infos = new CdmwPlatformMngt::PropertyInfos();
          p_infos->length(1L);
          // create info
          CdmwPlatformMngt::PropertyInfo p_info;
          p_info.property_name = CORBA::string_dup("ReplicationRole");
          CORBA::Any old_value;
          old_value <<= CdmwPlatformMngt::REDSTATE_PASSIVE;
          p_info.property_old_value = old_value;
          CORBA::Any new_value;
          new_value <<= CdmwPlatformMngt::REDSTATE_PRIMARY;
          p_info.property_new_value = new_value;
          p_infos[0L] = p_info;
          // create PropertyChange
          Cdmw::PlatformMngt::PropertyChangeFactory factory;
          CdmwPlatformMngt::PropertyChange_var prop_chg =
              factory.create(header, 
                             element_path.in(),
                             p_infos.in(),
                             "notify_ft_activate");
              
          // notify property change
          CDMW_DEBUG(m_logger.in(), 
              "notify_ft_activate '" << p_info.property_name
              << "' for element " << element_path);
          mcContext->notify_property_change(prop_chg.in());
          
      } catch (const Components::CCMException& e) {
          // Cannot find ManagedComponentContext object
          CDMW_WARN(m_logger.in(), 
              "Cannot ManagedComponentContext." <<
              "Cannot notify Supervision of ft_activate");
          // continue anyway
      } catch (const CORBA::SystemException& e) {
          CDMW_WARN(m_logger.in(), 
              "SystemException received notifying ft_activate : " <<
              Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
          // continue anyway
      }
   }
   ]]>
  
   void 
   <xsl:value-of select="$componentImplClassname"/>::notify_ft_deactivate()
   <![CDATA[
   {
      CDMW_LOG_FUNCTION(m_logger.in(), "");
      CDMW_DEBUG(m_logger.in(), "notify_ft_deactivate");
      try {
          // get ManagedComponentContext object
          CORBA::Object_var obj =
              m_context_executor->resolve_service_reference(
                  "ManagedComponentContext");
          CdmwPlatformMngt::ManagedElementContexts::ComponentContextExt_var mcContext =
              CdmwPlatformMngt::ManagedElementContexts::ComponentContextExt::_narrow(obj.in());
          
          // get component element path
          CdmwPlatformMngt::ManagedElement_var element = 
              mcContext->the_managed_element();
          // In case the element was not found 
          if (CORBA::is_nil(element.in()) )
          {
              CDMW_WARN(::Cdmw::CCM::CCMLoggers::Get_ccmgenerated_logger(), "Could not find managed element !");
              return;
          }
          CORBA::String_var element_path = element->element_path();
              
          // create PropertyChange EventHeader
          CdmwPlatformMngtBase::EventHeader header = 
              Cdmw::PlatformMngt::EventHeaderFactory::createHeader (CdmwPlatformMngtBase::INF);
          // create PropertyChange PropertyInfos
          CdmwPlatformMngt::PropertyInfos_var p_infos = new CdmwPlatformMngt::PropertyInfos();
          p_infos->length(1L);
          // create info
          CdmwPlatformMngt::PropertyInfo p_info;
          p_info.property_name = CORBA::string_dup("ReplicationRole");
          CORBA::Any old_value;
          old_value <<= CdmwPlatformMngt::REDSTATE_PRIMARY;
          p_info.property_old_value = old_value;
          CORBA::Any new_value;
          new_value <<= CdmwPlatformMngt::REDSTATE_PASSIVE;
          p_info.property_new_value = new_value;
          p_infos[0L] = p_info;
          // create PropertyChange
          Cdmw::PlatformMngt::PropertyChangeFactory factory;
          CdmwPlatformMngt::PropertyChange_var prop_chg =
              factory.create(header, 
                             element_path.in(),
                             p_infos.in(),
                             "notify_ft_deactivate");
              
          // notify property change
          CDMW_DEBUG(m_logger.in(), 
              "notify_ft_deactivate '" << p_info.property_name
              << "' for element " << element_path);
          mcContext->notify_property_change(prop_chg.in());
          
      } catch (const Components::CCMException& e) {
          // Cannot find ManagedComponentContext object
          CDMW_WARN(m_logger.in(), 
              "Cannot ManagedComponentContext." <<
              "Cannot notify Supervision of ft_activate");
          // continue anyway
      } catch (const CORBA::SystemException& e) {
          CDMW_WARN(m_logger.in(), 
              "SystemException received notifying ft_activate : " <<
              Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
          // continue anyway
      }
   }
   ]]>
   
 </xsl:if>
 
   <xsl:call-template name="closeNamespace">
      <xsl:with-param name="_scope" select="concat('Cdmw::CCM::CIF::Cdmw',$cppHomeScope)"/>
      <xsl:with-param name="_separator" select="$cppSep"/>
      <xsl:with-param name="_lastTokenIsNamespace" select="true()"/>
   </xsl:call-template>

   <!--
      The code generation stops here.
   -->

</xsl:template> <!-- end of template session_component_cpp.content -->



<!--
-->
<xsl:template name="session_component_cpp.content.1">
   <xsl:param name="_node"/>
   <xsl:param name="_readonlyId"/>
   <xsl:param name="_readonlyEndId"/>
   <xsl:param name="_paramName"/>

   <xsl:if test="not(boolean($_node/@stereotype)) or ($_node/@stereotype = $_readonlyEndId) or ($_node/@stereotype = $_readonlyId)">
      <xsl:if test="not($_node/@stereotype = $_readonlyEndId or $_node/@stereotype = $_readonlyId)">
         <xsl:value-of select="concat($lf, ', m_is_', $_paramName, '_configured(false)')"/>
      </xsl:if>
   </xsl:if>
</xsl:template> <!-- end of template session_component_cpp.content.1 -->



<!--
-->
<xsl:template name="session_component_cpp.content.2">
   <xsl:param name="_node"/>
   <xsl:param name="_readonlyId"/>
   <xsl:param name="_readonlyEndId"/>
   <xsl:param name="_paramName"/>
   <xsl:param name="_paramType"/>
   <xsl:param name="_returnType"/>
   <xsl:param name="_componentImplClassname"/>
   <xsl:param name="_cppOperationName"/>
   <xsl:param name="_componentObjRefVar"/>
   <xsl:param name="_objRefVarType"/>

   <xsl:if test="not(boolean($_node/@stereotype)) or ($_node/@stereotype = $_readonlyEndId) or ($_node/@stereotype = $_readonlyId)">
      <xsl:value-of select="concat($lf, $_returnType, $lf, $_componentImplClassname, $cppSep, $_cppOperationName, '()', $lf)"/>
         throw(CORBA::SystemException)
      {
         CDMW_LOG_FUNCTION(m_logger.in(), "");
         check_is_removed();

         <xsl:value-of select="$_componentObjRefVar"/> comp_exec = obtain_component_executor();
         <xsl:value-of select="$_objRefVarType"/> result = <![CDATA[comp_exec->]]><xsl:value-of select="$_cppOperationName"/>();

         try
         {
             <![CDATA[m_executor_locator->release_executor(comp_exec.in());]]>
         }
         catch (const <![CDATA[Components::CCMException& e]]> )
         {
             <![CDATA[
             CDMW_ERROR(m_logger.in(), "Components::CCMException raised, reason " << e);
             ]]>
             CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                                CORBA::COMPLETED_NO);
             CDMW_LOG_FUNCTION_EXCEPTION(ex);
             throw ex;
         }

            <xsl:if test="string-length($_objRefVarType) > 0">
                <xsl:choose>
                    <xsl:when test="contains($_objRefVarType, '_var')">
                        return result._retn();
                    </xsl:when>
                    <xsl:otherwise>
                        return result;
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:if>
      }

      <xsl:if test="not($_node/@stereotype = $_readonlyEndId or $_node/@stereotype = $_readonlyId)">
         void 
         <xsl:value-of select="concat($_componentImplClassname, $cppSep, $_cppOperationName, '(', $_paramType, ' ', $_paramName, ')')"/>
            throw(CORBA::SystemException)
         {
             CDMW_LOG_FUNCTION(m_logger.in(), "");
             check_is_removed();

             <xsl:value-of select="$_componentObjRefVar"/> comp_exec = obtain_component_executor();
             <![CDATA[comp_exec->]]><xsl:value-of select="concat($_cppOperationName, '(', $_paramName, ');')"/>

             try
             {
                 <![CDATA[m_executor_locator->release_executor(comp_exec.in());]]>
             }
             catch (const <![CDATA[Components::CCMException& e]]> )
             {
                 <![CDATA[
                 CDMW_ERROR(m_logger.in(), "Components::CCMException raised, reason " << e);
                 ]]>
                 CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                                    CORBA::COMPLETED_NO);
                 CDMW_LOG_FUNCTION_EXCEPTION(ex);
                 throw ex;
             }

             // set boolean to true (needed for configuration_completed)
             m_is_<xsl:value-of select="$_paramName"/>_configured = true;
         }
      </xsl:if>
   </xsl:if>
</xsl:template> <!-- end of template session_component_cpp.content.2 -->



<!--
-->
<xsl:template name="session_component_cpp.content.3">
   <xsl:param name="_node"/>
   <xsl:param name="_readonlyId"/>
   <xsl:param name="_readonlyEndId"/>
   <xsl:param name="_paramName"/>

   <xsl:if test="not(boolean($_node/@stereotype)) or ($_node/@stereotype = $_readonlyEndId) or ($_node/@stereotype = $_readonlyId)">
      <xsl:if test="not($_node/@stereotype = $_readonlyEndId or $_node/@stereotype = $_readonlyId)">
      if (!m_is_created_by_factory_operation)
      {
         // check if the attribute have been initialised only if not created by a factory op
         if (!m_is_<xsl:value-of select="$_paramName"/>_configured)
         {
            // component is not well configured
            CDMW_ERROR(m_logger.in(), 
                "Attribut <xsl:value-of select="$_paramName"/> not initialised!");
            Components::InvalidConfiguration ex;
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
         }
      }
      </xsl:if>
   </xsl:if>
</xsl:template> <!-- end of template session_component_cpp.content.3 -->



<!--
-->
<xsl:template name="session_component_cpp.content.4">
   <xsl:param name="_facetName"/>
   
   <xsl:variable name="pathPrefix">
       <xsl:choose>
           <xsl:when test="string-length($CSC_NAME) > 0">
               <xsl:value-of select="concat($CSC_NAME, $fileSep)"/>
           </xsl:when>
           <xsl:otherwise>
               <xsl:value-of select="''"/>
           </xsl:otherwise>
       </xsl:choose>
  </xsl:variable>

  <xsl:value-of select="concat($lf, '#include &quot;', $pathPrefix, $_facetName, '_impl.hpp&quot;')"/>
</xsl:template> <!-- end of template session_component_cpp.content.4 -->



<!--
-->
<xsl:template name="session_component_cpp.content.5">
   <xsl:param name="_facetNode"/>
   <xsl:param name="_facetName"/>

   <xsl:variable name="uppercaseFacetName" select="translate($_facetName, $lcase, $ucase)"/>
   
   <xsl:value-of select="concat($lf, 'const char* FACET_', $uppercaseFacetName, '_NAME')"/> = "<xsl:value-of select="$_facetName"/>";
   <xsl:value-of select="concat($lf, 'const char* FACET_', $uppercaseFacetName, '_REP_ID')"/> = "<xsl:value-of select="$_facetNode/UML:ModelElement.taggedValue/UML:TaggedValue[@tag = 'typeid']/@value"/>";
</xsl:template> <!-- end of template session_component_cpp.content.5 -->



<!--
-->
<xsl:template name="session_component_cpp.content.6">
   <xsl:param name="_facetName"/>
   
   else if (exec_name == "<xsl:value-of select="$_facetName"/>")
   {
      <![CDATA[result = m_comp_executor->]]><xsl:value-of select="concat('get_', $_facetName, '();')"/>
   }
</xsl:template> <!-- end of template session_component_cpp.content.6 -->



<!--
-->
<xsl:template name="session_component_cpp.content.7">
   <xsl:param name="_facetTypeName"/>
   <xsl:param name="_facetName"/>
   
   <xsl:variable name="uppercaseFacetName" select="translate($_facetName, $lcase, $ucase)"/>

   try
   {
      CORBA::Object_var <xsl:value-of select="$_facetName"/>_facet
         = declare_facet(comp_oid, 
                         FACET_<xsl:value-of select="$uppercaseFacetName"/>_NAME, 
                         FACET_<xsl:value-of select="$uppercaseFacetName"/>_REP_ID);
                         
      if (CORBA::is_nil(<xsl:value-of select="$_facetName"/>_facet.in()))
      {
         // This is to avoid Assertion failure in debug mode as internal ref count
         // equals 1 (CORBA C++ mapping oblige!)
         _set_ref_count(0);

         CDMW_ERROR(m_logger.in(), 
            "Internal exception raised when declaring facet <xsl:value-of select="$_facetName"/>!");
         CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, CORBA::COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(ex);
         throw ex;
      }
   }
   catch (const <![CDATA[AlreadyDoneException&]]>)
   {
      // This is to avoid Assertion failure in debug mode as internal ref count
      // equals 1 (CORBA C++ mapping oblige!)
      _set_ref_count(0);
     
      CDMW_ERROR(m_logger.in(), "Facet for_clients already declared!");
      // Don't propagate while its probably a code generation error!
   }
   catch (const <![CDATA[InternalErrorException&]]>)
   {
      // This is to avoid Assertion failure in debug mode as internal ref count
      // equals 1 (CORBA C++ mapping oblige!)
      _set_ref_count(0);

      CDMW_ERROR(m_logger.in(), 
          "Internal exception raised when declaring facet <xsl:value-of select="$_facetTypeName"/>!");
      CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, CORBA::COMPLETED_NO);
      CDMW_LOG_FUNCTION_EXCEPTION(ex);
      throw ex;
   }
</xsl:template> <!-- end of template session_component_cpp.content.7 -->



<!--
-->
<xsl:template name="session_component_cpp.Facets.EquivalentIDL.Definition">
   <xsl:param name="_facetNode"/>
   <xsl:param name="_facetName"/>
   <xsl:param name="_componentImplClassname"/>
   <xsl:param name="_isSegmented"/>
   
   <xsl:variable name="facetTypeName" select="$_facetNode/@name"/>

   <xsl:variable name="scopedReturnType">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_facetNode"/>
      </xsl:call-template>
   </xsl:variable>

   <xsl:variable name="cppFacetScope">
      <xsl:call-template name="removeLastToken">
         <xsl:with-param name="_scopedName" select="$scopedReturnType"/>
         <xsl:with-param name="_sep" select="$cppSep"/>
      </xsl:call-template>
   </xsl:variable>

   <xsl:value-of select="concat($lf, $scopedReturnType, '_ptr')"/>
   <xsl:value-of select="concat($lf, $_componentImplClassname, $cppSep, 'provide_', $_facetName, '()',
                                $lf, 'throw(CORBA::SystemException)', $lf)"/>
   {
      CDMW_LOG_FUNCTION(m_logger.in(), "");
      check_is_removed();

      CORBA::Object_var obj;

      try 
      {
         obj = CCMObject_impl::provide_facet("<xsl:value-of select="$_facetName"/>");
      }
      catch (const <![CDATA[Components::InvalidName &]]>)
      {
         CDMW_ERROR(m_logger.in(), 
             "'<xsl:value-of select="$_facetName"/>' is not a valid facet name");
         CORBA::OBJECT_NOT_EXIST ex(OrbSupport::OBJECT_NOT_EXIST,
                                    CORBA::COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(ex);
         throw ex;
      }

      if (CORBA::is_nil(obj.in()))
      {
         CDMW_ERROR(m_logger.in(), 
             "Facet <xsl:value-of select="$_facetName"/> is nil");
         CORBA::OBJECT_NOT_EXIST ex(OrbSupport::OBJECT_NOT_EXIST, 
                                    CORBA::COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(ex);
         throw ex;
      }

      <xsl:value-of select="concat($scopedReturnType, '_var')"/> facet_ref = <xsl:value-of select="$scopedReturnType"/>::_narrow(obj.in());
       
      if (CORBA::is_nil(facet_ref.in()))
      {
         CDMW_ERROR(m_logger.in(), 
             "Facet <xsl:value-of select="$_facetName"/> is not a <xsl:value-of select="$scopedReturnType"/>");
         CORBA::OBJECT_NOT_EXIST ex(OrbSupport::OBJECT_NOT_EXIST, 
                                    CORBA::COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(ex);
         throw ex;
      }
       
      return facet_ref._retn();
   }

   <xsl:if test="$_isSegmented = 'no'">
      <!--
               facet operation code for monolithic component
       -->
      <xsl:call-template name="session_component_cpp.Monolithic.Facets.Operations.all">
          <xsl:with-param name="_interfaceNode" select="$_facetNode"/>
          <xsl:with-param name="_facetName" select="$_facetName"/>
          <xsl:with-param name="_facetTypeName" select="$facetTypeName"/>
          <xsl:with-param name="_componentImplClassname" select="$_componentImplClassname"/>
          <xsl:with-param name="_cppFacetScope" select="$cppFacetScope"/>
      </xsl:call-template> 

   </xsl:if>
</xsl:template> <!-- end of template session_component_cpp.Facets.EquivalentIDL.Definition -->



<!--
-->
<xsl:template name="session_component_cpp.content.9">
   <xsl:param name="_facetTypeName"/>
   <xsl:param name="_facetName"/>
   
   <xsl:variable name="uppercaseFacetName" select="translate($_facetName, $lcase, $ucase)"/>

   if (facet_name == FACET_<xsl:value-of select="$uppercaseFacetName"/>_NAME)
   {
      facet_servant 
         = new <xsl:value-of select="concat($_facetTypeName, '_impl')"/>(m_executor_locator.in(), 
                   FACET_<xsl:value-of select="$uppercaseFacetName"/>_NAME,
                   m_context,
                   m_comp_ref.in(),
                   this,
                   m_logger.in());
   }
   else
</xsl:template> <!-- end of template session_component_cpp.content.9 -->



<!--
-->
<xsl:template name="session_component_cpp.content.10">
   <xsl:param name="_receptacleNode"/>
   <xsl:param name="_receptacleName"/>

   <xsl:variable name="uppercaseReceptacleName" select="translate($_receptacleName, $lcase, $ucase)"/>
   
   <xsl:value-of select="concat($lf, 'const char* RECEP_', $uppercaseReceptacleName, '_NAME')"/> = "<xsl:value-of select="$_receptacleName"/>";
   <xsl:value-of select="concat($lf, 'const char* RECEP_', $uppercaseReceptacleName, '_REP_ID')"/> = "<xsl:value-of select="$_receptacleNode/UML:ModelElement.taggedValue/UML:TaggedValue[@tag = 'typeid']/@value"/>";
</xsl:template> <!-- end of template session_component_cpp.content.10 -->



<!--
-->
<xsl:template name="session_component_cpp.content.11">
   <xsl:param name="_receptacleName"/>
   
   <xsl:variable name="uppercaseReceptacleName" select="translate($_receptacleName, $lcase, $ucase)"/>

   declare_receptacle(<xsl:value-of select="concat('RECEP_', $uppercaseReceptacleName,'_NAME, RECEP_', $uppercaseReceptacleName,'_REP_ID')"/>);
</xsl:template> <!-- end of template session_component_cpp.content.11 -->



<!--
-->
<xsl:template name="session_component_cpp.content.12">
   <xsl:param name="_receptacleTypeNode"/>
   <xsl:param name="_receptacleName"/>
   <xsl:param name="_componentImplClassname"/>
   
   <xsl:variable name="scopedReturnType">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_receptacleTypeNode"/>
      </xsl:call-template>
   </xsl:variable>

   <xsl:variable name="uppercaseReceptacleName" select="translate($_receptacleName, $lcase, $ucase)"/>

   <xsl:value-of select="concat($lf, 'void', $lf, $_componentImplClassname, $cppSep, 'connect_', $_receptacleName, '(', $scopedReturnType, '_ptr conxn)', $lf)"/>
      throw(Components::AlreadyConnected,
            Components::InvalidConnection,
            CORBA::SystemException)
   {
      CDMW_LOG_FUNCTION(m_logger.in(), "");
      check_is_removed();

      try
      {
          CCMObject_impl::connect(RECEP_<xsl:value-of select="$uppercaseReceptacleName"/>_NAME, conxn);
      }
      catch (const <![CDATA[Components::InvalidName &]]>)
      {
          CDMW_ERROR(m_logger.in(), "Invalid name for receptacle connection!");
          CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                             CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION(ex);
          throw ex;
      }
      catch (const <![CDATA[Components::AlreadyConnected & e]]>)
      {
          CDMW_LOG_FUNCTION_EXCEPTION(e);
          throw;
      }
      catch (const <![CDATA[Components::InvalidConnection & e]]>)
      {
          CDMW_LOG_FUNCTION_EXCEPTION(e);
          throw;
      }
      catch (const <![CDATA[Components::ExceededConnectionLimit &]]>)
      {
          CDMW_ERROR(m_logger.in(), 
              "Exceeded connection limit reached for receptacle connection!");
          CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                             CORBA::COMPLETED_NO);   
          CDMW_LOG_FUNCTION_EXCEPTION(ex);
          throw ex;
      }
      catch (const <![CDATA[CORBA::SystemException & e]]>)
      {
          CDMW_LOG_FUNCTION_EXCEPTION(e);
          throw;
      }
   }

   <xsl:value-of select="concat($lf, $scopedReturnType, '_ptr', $lf, $_componentImplClassname, $cppSep, 'disconnect_', $_receptacleName, '()', $lf)"/>
      throw(Components::NoConnection,
            CORBA::SystemException)
   {
      CDMW_LOG_FUNCTION(m_logger.in(), "");
      check_is_removed();

      try
      {
          <xsl:value-of select="concat($scopedReturnType, '_var')"/> result = get_connection_<xsl:value-of select="$_receptacleName"/>();
          CCMObject_impl::disconnect(RECEP_<xsl:value-of select="$uppercaseReceptacleName"/>_NAME, 0);

          return result._retn();
      }
      catch (const <![CDATA[Components::InvalidName &]]>)
      {
          CDMW_ERROR(m_logger.in(), "Invalid name for receptacle disconnection!");
          CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                            CORBA::COMPLETED_NO);   
          CDMW_LOG_FUNCTION_EXCEPTION(ex);
          throw ex;
      }
      catch (const <![CDATA[Components::NoConnection & e]]>)
      {
          CDMW_LOG_FUNCTION_EXCEPTION(e);
          throw;
      }
      catch (const <![CDATA[Components::InvalidConnection &]]>)
      {
          CDMW_ERROR(m_logger.in(), "Invalid connection for receptacle disconnection!");
          CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                             CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION(ex);
          throw ex;
      }
      catch (const <![CDATA[Components::CookieRequired &]]>)
      {
          CDMW_ERROR(m_logger.in(), "Cookie required for receptacle disconnection!");
          CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                             CORBA::COMPLETED_NO);   
          CDMW_LOG_FUNCTION_EXCEPTION(ex);
          throw ex;
      }
      catch (const <![CDATA[CORBA::SystemException & e]]>)
      {
          CDMW_LOG_FUNCTION_EXCEPTION(e);
          throw;
      }
   }

   <xsl:value-of select="concat($lf, $scopedReturnType, '_ptr', $lf, $_componentImplClassname, $cppSep, 'get_connection_', $_receptacleName, '()', $lf)"/>
      throw(CORBA::SystemException)
   {
      CDMW_LOG_FUNCTION(m_logger.in(), "");
      check_is_removed();

      Components::ConnectionDescriptions_var comp_desc;

      try
      {
         comp_desc = CCMObject_impl::get_connections(RECEP_<xsl:value-of select="$uppercaseReceptacleName"/>_NAME);
      }
      catch (const <![CDATA[Components::InvalidName& e]]>)
      {
         CDMW_ERROR(m_logger.in(), "Invalid name getting receptacle!");
         CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                            CORBA::COMPLETED_NO);   
         CDMW_LOG_FUNCTION_EXCEPTION(ex);
         throw ex;
      }
      catch (const <![CDATA[CORBA::SystemException& e]]> )
      {
         CDMW_LOG_FUNCTION_EXCEPTION(e);
         throw;
      }
      
      <xsl:value-of select="concat($scopedReturnType, '_var')"/> result = <xsl:value-of select="$scopedReturnType"/>::_nil();
      CORBA::Object_var obj = CORBA::Object::_nil();

      if (<![CDATA[comp_desc->]]>length() == 1)
      {
         <![CDATA[obj = CORBA::Object::_duplicate(comp_desc->operator[](0)->objref());]]>

      }

      // The spec says that nil shall only be returned where there is no 
      // connection (no call to connect). If the receptacle is currently
      // connected this operation returns the connected object reference.
      result = <xsl:value-of select="$scopedReturnType"/>::_unchecked_narrow(obj.in());
      return result._retn();
   }
</xsl:template> <!-- end of template session_component_cpp.content.12 -->



<!--
-->
<xsl:template name="session_component_cpp.content.13">
   <xsl:param name="_publisherNode"/>
   <xsl:param name="_publisherName"/>

   <xsl:variable name="uppercasePublisherName" select="translate($_publisherName, $lcase, $ucase)"/>
   <xsl:variable name="repId">
      <xsl:call-template name="replaceCharsInString">
         <xsl:with-param name="_stringIn" select="$_publisherNode/UML:ModelElement.taggedValue/UML:TaggedValue[@tag = 'typeid']/@value"/>
         <xsl:with-param name="_charsIn" select="':1.0'"/>
         <xsl:with-param name="_charsOut" select="'Consumer:1.0'"/>
      </xsl:call-template>
   </xsl:variable>
   
   <xsl:value-of select="concat($lf, 'const char* PUBLISHER_', $uppercasePublisherName, '_NAME')"/> = "<xsl:value-of select="$_publisherName"/>";
   <xsl:value-of select="concat($lf, 'const char* PUBLISHER_', $uppercasePublisherName, '_REP_ID')"/> = "<xsl:value-of select="$repId"/>";
</xsl:template> <!-- end of template session_component_cpp.content.13 -->



<!--
-->
<xsl:template name="session_component_cpp.content.14">
   <xsl:param name="_emitterNode"/>
   <xsl:param name="_emitterName"/>

   <xsl:variable name="uppercaseEmitterName" select="translate($_emitterName, $lcase, $ucase)"/>
   <xsl:variable name="repId">
      <xsl:call-template name="replaceCharsInString">
         <xsl:with-param name="_stringIn" select="$_emitterNode/UML:ModelElement.taggedValue/UML:TaggedValue[@tag = 'typeid']/@value"/>
         <xsl:with-param name="_charsIn" select="':1.0'"/>
         <xsl:with-param name="_charsOut" select="'Consumer:1.0'"/>
      </xsl:call-template>
   </xsl:variable>
   
   <xsl:value-of select="concat($lf, 'const char* EMITTER_', $uppercaseEmitterName, '_NAME')"/> = "<xsl:value-of select="$_emitterName"/>";
   <xsl:value-of select="concat($lf, 'const char* EMITTER_', $uppercaseEmitterName, '_REP_ID')"/> = "<xsl:value-of select="$repId"/>";
</xsl:template> <!-- end of template session_component_cpp.content.14 -->



<!--
-->
<xsl:template name="session_component_cpp.content.15">
   <xsl:param name="_publisherName"/>
   
   <xsl:variable name="uppercasePublisherName" select="translate($_publisherName, $lcase, $ucase)"/>

   declare_publisher(<xsl:value-of select="concat('PUBLISHER_', $uppercasePublisherName,'_NAME, PUBLISHER_', $uppercasePublisherName,'_REP_ID')"/>);
</xsl:template> <!-- end of template session_component_cpp.content.15 -->



<!--
-->
<xsl:template name="session_component_cpp.content.16">
   <xsl:param name="_emitterName"/>
   
   <xsl:variable name="uppercaseEmitterName" select="translate($_emitterName, $lcase, $ucase)"/>

   declare_emitter(<xsl:value-of select="concat('EMITTER_', $uppercaseEmitterName,'_NAME, EMITTER_', $uppercaseEmitterName,'_REP_ID')"/>);
</xsl:template> <!-- end of template session_component_cpp.content.16 -->



<!--
-->
<xsl:template name="session_component_cpp.content.17">
   <xsl:param name="_publisherTypeNode"/>
   <xsl:param name="_publisherName"/>
   <xsl:param name="_componentImplClassname"/>

   <xsl:variable name="scopedReturnType">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_publisherTypeNode"/>
      </xsl:call-template>
   </xsl:variable>

   <xsl:variable name="uppercasePublisherName" select="translate($_publisherName, $lcase, $ucase)"/>

   <xsl:value-of select="concat($lf, 'Components::Cookie*',
                                $lf, $_componentImplClassname, $cppSep, 'subscribe_', $_publisherName, '(', $scopedReturnType, 'Consumer_ptr consumer)')"/>
      throw(Components::ExceededConnectionLimit, CORBA::SystemException)
   {
      CDMW_LOG_FUNCTION(m_logger.in(), "");
      check_is_removed();

      Components::Cookie_var result;
      
      try
      {
          result = CCMObject_impl::subscribe(PUBLISHER_<xsl:value-of select="$uppercasePublisherName"/>_NAME,
                                             consumer);
      }
      catch (const <![CDATA[Components::InvalidName &]]>)
      {
          CDMW_ERROR(m_logger.in(), "Invalid name for publisher subscription!");
          CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                             CORBA::COMPLETED_NO);   
          CDMW_LOG_FUNCTION_EXCEPTION(ex);
          throw ex;
      }
      catch (const <![CDATA[Components::InvalidConnection &]]>)
      {
          // should never appear!
          CDMW_ERROR(m_logger.in(), "Invalid Connection");
          CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                             CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION(ex);
          throw ex;
      }
      catch (const <![CDATA[Components::ExceededConnectionLimit & e]]>)
      {
          CDMW_LOG_FUNCTION_EXCEPTION(e);
          throw;
      }
      catch (const <![CDATA[CORBA::SystemException& e]]>)
      {
          throw;
          CDMW_LOG_FUNCTION_EXCEPTION(e);
      }
      
      return result._retn();
   }

   <xsl:value-of select="concat($lf, $scopedReturnType, 'Consumer_ptr',
                                $lf, $_componentImplClassname, $cppSep, 'unsubscribe_', $_publisherName, '(Components::Cookie* ck)')"/>
      throw(Components::InvalidConnection, CORBA::SystemException)
   {
      CDMW_LOG_FUNCTION(m_logger.in(), "");
      check_is_removed();

      if (!ck) {
         CDMW_ERROR(m_logger.in(), "Cookie is null");
         CORBA::BAD_PARAM ex(OrbSupport::BAD_PARAMNilObjectReference, CORBA::COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(ex);
         throw ex;
      }
       
      <xsl:value-of select="$scopedReturnType"/>Consumer_var result = <xsl:value-of select="$scopedReturnType"/>Consumer::_nil();
      try
      {
         Components::EventConsumerBase_var consumer 
            = CCMObject_impl::get_consumer(PUBLISHER_<xsl:value-of select="$uppercasePublisherName"/>_NAME, ck);

         result = <xsl:value-of select="$scopedReturnType"/>Consumer::_unchecked_narrow(consumer.in());

         CCMObject_impl::unsubscribe(PUBLISHER_<xsl:value-of select="$uppercasePublisherName"/>_NAME, ck);
      }
      catch (const <![CDATA[Components::InvalidName&]]>)
      {
         // internal error should not appear
         CDMW_ERROR(m_logger.in(), "Invalid name for publisher unsubscription!");
         CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                            CORBA::COMPLETED_NO);   
         CDMW_LOG_FUNCTION_EXCEPTION(ex);
         throw ex;
      }
      catch (const <![CDATA[Components::InvalidConnection& e]]>)
      {
         CDMW_LOG_FUNCTION_EXCEPTION(e);
         throw;
      }
      catch (const <![CDATA[CORBA::SystemException& e]]>)
      {
         CDMW_LOG_FUNCTION_EXCEPTION(e);
         throw;
      }

      return result._retn();
   }
</xsl:template> <!-- end of template session_component_cpp.content.17 -->



<!--
-->
<xsl:template name="session_component_cpp.content.18">
   <xsl:param name="_emitterTypeNode"/>
   <xsl:param name="_emitterName"/>
   <xsl:param name="_componentImplClassname"/>

   <xsl:variable name="scopedReturnType">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_emitterTypeNode"/>
      </xsl:call-template>
   </xsl:variable>

   <xsl:variable name="uppercaseEmitterName" select="translate($_emitterName, $lcase, $ucase)"/>

   <xsl:value-of select="concat($lf, 'void',
                                $lf, $_componentImplClassname, $cppSep, 'connect_', $_emitterName, '(', $scopedReturnType, 'Consumer_ptr consumer)')"/>
      throw(Components::AlreadyConnected, CORBA::SystemException)
   {
      CDMW_LOG_FUNCTION(m_logger.in(), "");
      check_is_removed();

      try
      {
         CCMObject_impl::connect_consumer(EMITTER_<xsl:value-of select="$uppercaseEmitterName"/>_NAME, consumer);
      }
      catch (const <![CDATA[Components::InvalidName&]]>)
      {
         // can not appear!
         CDMW_ERROR(m_logger.in(), "Invalid name for emitter conection!");
         CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                            CORBA::COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(ex);
         throw ex;
      }
      catch (const <![CDATA[Components::InvalidConnection&]]>)
      {
         // can not appear!
         CDMW_ERROR(m_logger.in(), "Invalid connection for emitter conection!");
         CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                            CORBA::COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(ex);
         throw ex;
      } 
      catch (const <![CDATA[Components::AlreadyConnected& e]]>)
      {
         CDMW_LOG_FUNCTION_EXCEPTION(e);
         throw;
      }
      catch (const <![CDATA[CORBA::SystemException& e]]>)
      {
         CDMW_LOG_FUNCTION_EXCEPTION(e);
         throw;
      } 
   }

   <xsl:value-of select="concat($lf, $scopedReturnType, 'Consumer_ptr',
                                $lf, $_componentImplClassname, $cppSep, 'disconnect_', $_emitterName, '()')"/>
      throw(Components::NoConnection, CORBA::SystemException)
   {
      CDMW_LOG_FUNCTION(m_logger.in(), "");
      check_is_removed();

      try
      {
         Components::EventConsumerBase_var consumer = 
            CCMObject_impl::disconnect_consumer(EMITTER_<xsl:value-of select="$uppercaseEmitterName"/>_NAME);

         <xsl:value-of select="$scopedReturnType"/>Consumer_var result = 
            <xsl:value-of select="$scopedReturnType"/>Consumer::_narrow(consumer.in());

         if (CORBA::is_nil(result.in()))
         {
            // can not appear!
            CDMW_ERROR(m_logger.in(), "Old connection had wrong type");
            CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                               CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
         }

         return result._retn();
      }
      catch (const <![CDATA[Components::InvalidName&]]>)
      {
         // can not appear!
         CDMW_ERROR(m_logger.in(), "Invalid name for emitter disconection!");
         CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                            CORBA::COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(ex);
         throw ex;
      }
      catch (const <![CDATA[Components::InvalidConnection&]]>)
      {
         // can not appear!
         CDMW_ERROR(m_logger.in(), "Invalid connection for emitter disconection!");
         CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                            CORBA::COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(ex);
         throw ex;
      } 
      catch (const <![CDATA[Components::AlreadyConnected& e]]>)
      {
         CDMW_LOG_FUNCTION_EXCEPTION(e);
         throw;
      }
      catch (const <![CDATA[CORBA::SystemException& e]]>)
      {
         CDMW_LOG_FUNCTION_EXCEPTION(e);
         throw;
      } 
   }
</xsl:template> <!-- end of template session_component_cpp.content.18 -->



<!--
-->
<xsl:template name="session_component_cpp.content.19">
   <xsl:param name="_consumerTypeNode"/>
   <xsl:param name="_consumerName"/>

   <xsl:variable name="uppercaseConsumerName" select="translate($_consumerName, $lcase, $ucase)"/>
   <xsl:variable name="repId">
      <xsl:variable name="partialRepId">
         <xsl:call-template name="replaceCharsInString">
            <xsl:with-param name="_stringIn">
               <xsl:call-template name="getScopedNameOfNode">
                  <xsl:with-param name="_node" select="$_consumerTypeNode"/>
               </xsl:call-template>
            </xsl:with-param>
            <xsl:with-param name="_charsIn" select="$cppSep"/>
            <xsl:with-param name="_charsOut" select="'/'"/>
         </xsl:call-template>
      </xsl:variable>
      <xsl:value-of select="concat('IDL:', $idlPragmaPrefix, '/Cdmw_', $partialRepId, 'Consumer:1.0')"/>
   </xsl:variable>
   
   <xsl:value-of select="concat($lf, 'const char* CONSUMER_', $uppercaseConsumerName, '_NAME')"/> = "<xsl:value-of select="$_consumerName"/>";
   <xsl:value-of select="concat($lf, 'const char* CONSUMER_', $uppercaseConsumerName, '_REP_ID')"/> = "<xsl:value-of select="$repId"/>";
</xsl:template> <!-- end of template session_component_cpp.content.19 -->



<!--
-->
<xsl:template name="session_component_cpp.content.20">
   <xsl:param name="_consumerName"/>
   <xsl:param name="_componentNode"/>

   <xsl:variable name="componentName" select="$_componentNode/@name"/>
   <xsl:variable name="cppScopedComponentName">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_componentNode"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="cppComponentScope">
      <xsl:call-template name="removeLastToken">
         <xsl:with-param name="_scopedName" select="$cppScopedComponentName"/>
         <xsl:with-param name="_sep" select="$cppSep"/>
      </xsl:call-template>
   </xsl:variable>   
   
   else if (exec_name == <xsl:value-of select="concat('CONSUMER_', translate($_consumerName, $lcase, $ucase), '_NAME')"/>)
   {
      result = <xsl:value-of select="concat($cppComponentScope, $cppSep, 'CCM_', $componentName)"/>::_duplicate(m_comp_executor.in());
   }
</xsl:template> <!-- end of template session_component_cpp.content.20 -->



<!--
-->
<xsl:template name="session_component_cpp.content.21">
   <xsl:param name="_consumerName"/>
   
   <xsl:variable name="uppercaseConsumerName" select="translate($_consumerName, $lcase, $ucase)"/>
  {
   CORBA::Object_var consumer =
      declare_consumer(comp_oid.c_str(), <xsl:value-of select="concat('CONSUMER_', $uppercaseConsumerName,'_NAME, CONSUMER_', $uppercaseConsumerName,'_REP_ID')"/>);
  }
</xsl:template> <!-- end of template session_component_cpp.content.21 -->



<!--
-->
<xsl:template name="session_component_cpp.content.22">
   <xsl:param name="_consumerTypeNode"/>
   <xsl:param name="_consumerName"/>
   <xsl:param name="_componentNode"/>
   <xsl:param name="_componentImplClassname"/>
   <xsl:param name="_isSegmented"/>

   <xsl:variable name="consumerTypeName" select="$_consumerTypeNode/@name"/>
   <xsl:variable name="componentName" select="$_componentNode/@name"/>

   <xsl:variable name="cppScopedConsumerTypeName">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_consumerTypeNode"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="cppScopedComponentName">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_componentNode"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="cppComponentScope">
      <xsl:call-template name="removeLastToken">
         <xsl:with-param name="_scopedName" select="$cppScopedComponentName"/>
         <xsl:with-param name="_sep" select="$cppSep"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:if test="$_isSegmented = 'no'">
      <xsl:variable name="cppScopedConsumerName">
         <xsl:call-template name="getScopedNameOfNode">
            <xsl:with-param name="_node" select="$_consumerTypeNode"/>
         </xsl:call-template>
      </xsl:variable>
      <xsl:variable name="cppConsumerScope">
         <xsl:call-template name="removeLastToken">
            <xsl:with-param name="_scopedName" select="$cppScopedConsumerName"/>
            <xsl:with-param name="_sep" select="$cppSep"/>
         </xsl:call-template>
      </xsl:variable>
      <xsl:variable name="lowercaseComponentName" select="translate($componentName, $ucase, $lcase)"/>
      <xsl:variable name="consumerClassname" select="concat($consumerTypeName, 'Consumer')"/>
      <xsl:variable name="lowercaseConsumerTypeName" select="translate($consumerTypeName, $ucase, $lcase)"/>
      
   
   
      void
      <xsl:value-of select="concat($_componentImplClassname, $cppSep, 'push_', $consumerTypeName, '(', $cppScopedConsumerTypeName, '* the_', $lowercaseConsumerTypeName, ')')"/>
         throw (CORBA::SystemException)
      {
          CDMW_LOG_FUNCTION(m_logger.in(), "");
          check_is_removed();
          m_state = m_state->operational_request(this);
   
          try
          {
   
          <![CDATA[CORBA::Object_var obj = m_executor_locator->obtain_executor("]]><xsl:value-of select="$_consumerName"/>");
   
          if (CORBA::is_nil(obj.in()))
          {
              CDMW_ERROR(m_logger.in(), 
                  "Executor for consumer <xsl:value-of select="$_consumerName"/> is nil");
              CORBA::OBJECT_NOT_EXIST ex(OrbSupport::OBJECT_NOT_EXIST, 
                                         CORBA::COMPLETED_NO);
              CDMW_LOG_FUNCTION_EXCEPTION(ex);
              throw ex;
          }
          <xsl:value-of select="concat($cppComponentScope, $cppSep, 'CCM_', $componentName, '_var', ' ', $lowercaseComponentName, '_exec = ', 
                                         $cppComponentScope, $cppSep, 'CCM_', $componentName, $cppSep, '_narrow(obj.in());')"/>
   
           if (CORBA::is_nil(<xsl:value-of select="concat($lowercaseComponentName, '_exec.in()')"/>))
           {
               CDMW_DEBUG(m_logger.in(), 
                   "Executor for consumer <xsl:value-of select="$_consumerName"/> is not the component itself");
           
               // Component may segmented
               <xsl:value-of select="concat($cppConsumerScope, $cppSep, 'CCM_', $consumerClassname, '_var consumer_exec = ',
                                            $cppConsumerScope, $cppSep, 'CCM_', $consumerClassname, $cppSep, '_narrow(obj.in());')"/>
           
               if (CORBA::is_nil(consumer_exec.in()))
               {
                  CDMW_ERROR(m_logger.in(),
                      "Executor for consumer <xsl:value-of select="$_consumerName"/> has wrong type");
                  CORBA::OBJECT_NOT_EXIST ex(OrbSupport::OBJECT_NOT_EXIST, 
                                            CORBA::COMPLETED_NO);
                  CDMW_LOG_FUNCTION_EXCEPTION(ex);
                  throw ex;
               }
               else
               {
                  // push event to the consumer executor
                  <![CDATA[consumer_exec->push]]>(the_<xsl:value-of select="$lowercaseConsumerTypeName"/>);   
               }
           }
           else
           {
               <xsl:value-of select="$lowercaseComponentName"/><![CDATA[_exec->push_]]><xsl:value-of select="concat($_consumerName, '(the_', $lowercaseConsumerTypeName,');')"/>
           }
       
            <![CDATA[m_executor_locator->release_executor(obj.in());]]>
         }
         catch (const <![CDATA[Components::CCMException& e]]> )
         {
            CDMW_ERROR(m_logger.in(), 
                "SystemException received: " <![CDATA[<< e]]>);
            CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                               CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
         }
      }
   </xsl:if>

   <xsl:value-of select="concat($lf, $cppScopedConsumerTypeName, 'Consumer_ptr',
                                $lf, $_componentImplClassname, $cppSep, 'get_consumer_', $_consumerName, '()')"/>
      throw(CORBA::SystemException)
   {
      CDMW_LOG_FUNCTION(m_logger.in(), "");
      check_is_removed();

      CORBA::Object_var obj;

      try 
      {
         obj = CCMObject_impl::get_consumer(CONSUMER_<xsl:value-of select="translate($_consumerName, $lcase, $ucase)"/>_NAME);
      }
      catch (const <![CDATA[Components::InvalidName & e]]>)
      {
         CDMW_ERROR(m_logger.in(), "Invalid consumer name");
         CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                               CORBA::COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(ex);
         throw ex;
      }

      <xsl:value-of select="concat($cppScopedConsumerTypeName, 'Consumer_var consumer_ref = ',
                                   $cppScopedConsumerTypeName, 'Consumer', $cppSep, '_narrow(obj.in());')"/>
       
      if (CORBA::is_nil(consumer_ref.in()))
      {
         CDMW_ERROR(m_logger.in(), "Consumer has wrong type");
         CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                               CORBA::COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(ex);
         throw ex;
      }
       
      return consumer_ref._retn();
   } 
</xsl:template> <!-- end of template session_component_cpp.content.22 -->



<!--
-->
<xsl:template name="session_component_cpp.content.23">
   <xsl:param name="_consumerTypeNode"/>
   <xsl:param name="_consumerName"/>
   
   <xsl:variable name="uppercaseConsumerName" select="translate($_consumerName, $lcase, $ucase)"/>

   <xsl:variable name="cppScopedConsumerTypeName">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_consumerTypeNode"/>
      </xsl:call-template>
   </xsl:variable>

   if (facet_name == CONSUMER_<xsl:value-of select="$uppercaseConsumerName"/>_NAME)
   {
      facet_servant 
         = new <xsl:value-of select="concat('Cdmw::CCM::CIF::Cdmw', $cppScopedConsumerTypeName, 'Consumer_impl')"/>(m_executor_locator.in(), 
                   CONSUMER_<xsl:value-of select="$uppercaseConsumerName"/>_NAME,
                   m_context,
                   this,
                   m_logger.in());
   }
   else
</xsl:template> <!-- end of template session_component_cpp.content.23 -->



<!--
-->
<xsl:template name="session_component_cpp.content.24">
   <xsl:param name="_consumerTypeName"/>
   
   <xsl:variable name="pathPrefix">
       <xsl:choose>
           <xsl:when test="string-length($CSC_NAME) > 0">
               <xsl:value-of select="concat($CSC_NAME, $fileSep)"/>
           </xsl:when>
           <xsl:otherwise>
               <xsl:value-of select="''"/>
           </xsl:otherwise>
       </xsl:choose>
   </xsl:variable>
   
   <xsl:value-of select="concat($lf, '#include &quot;', $pathPrefix, $_consumerTypeName, 'Consumer_impl.hpp&quot;')"/>
</xsl:template> <!-- end of template session_component_cpp.content.24 -->

<!-- Following template is only used for monolithic component -->
<xsl:template name="session_component_cpp.content.25">
   <xsl:param name="_consumerTypeNode"/>

   <xsl:variable name="consumerTypeName" select="$_consumerTypeNode/@name"/>

   <xsl:variable name="cppScopedConsumerTypeName">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_consumerTypeNode"/>
      </xsl:call-template>
   </xsl:variable>

   if (bad_evt_type_found)
   {
      <xsl:value-of select="$cppScopedConsumerTypeName"/>_var info_evt 
         = <xsl:value-of select="$cppScopedConsumerTypeName"/>::_downcast(evt);
      CORBA::add_ref(info_evt.in());

      if (info_evt.in())
      {
         bad_evt_type_found = false;
         <![CDATA[this->push_]]><xsl:value-of select="$consumerTypeName"/>(info_evt.in());
      }
   }
   
</xsl:template> <!-- end of template session_component_cpp.content.25 -->


<!-- Following template is only used for monolithic component -->
<xsl:template name="session_component_cpp.content.26">
   <xsl:param name="_consumerTypeNode"/>
   <xsl:param name="_consumerName"/>
   <xsl:param name="_componentImplClassname"/>

   <xsl:variable name="consumerTypeName" select="$_consumerTypeNode/@name"/>

   <xsl:variable name="cppScopedConsumerTypeName">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_consumerTypeNode"/>
      </xsl:call-template>
   </xsl:variable>

   if (bad_evt_type_found)
   {
      <xsl:value-of select="$cppScopedConsumerTypeName"/>_var event;
      /* PCR-0049
      <![CDATA[data >>= event;]]>
      */

      <![CDATA[
      const CORBA::OctetSeq* p_encoded_data;
      data >>= p_encoded_data;
      CORBA::OctetSeq encoded_data(*p_encoded_data);

      Cdmw::OrbSupport::Codec<CdmwCcmCif::TimestampedEvent> codec;
      
      CdmwCcmCif::TimestampedEvent timestamped_event;

      codec.decode(encoded_data, timestamped_event);
      ]]>

      event = <xsl:value-of select="$cppScopedConsumerTypeName"/>::_downcast(timestamped_event.event.in());

      <![CDATA[
      ::CORBA::add_ref(event.in());
      if (event.in())
      {
          CDMW_DEBUG_EVENT(m_logger.in(), "Event received at: "
            << Cdmw::CommonSvcs::Time::Util::TimeT2string_ISO8601(Cdmw::CommonSvcs::Time::Util::get_current_time())
            << " issued at: " 
            << Cdmw::CommonSvcs::Time::Util::TimeT2string_ISO8601(timestamped_event.timestamp));

          bad_evt_type_found = false;
          try
          {
              this->push_]]><xsl:value-of select="$consumerTypeName"/>(event.in());
          }
          catch (const <![CDATA[CORBA::SystemException& e]]>)
          {
              CDMW_LOG_FUNCTION_EXCEPTION(e);
              throw;
          }
      }
   }
</xsl:template> <!-- end of template session_component_cpp.content.26 -->

<!-- Following template is only used for monolithic component -->
<xsl:template name="session_component_cpp.content.27">
   <xsl:param name="_consumerTypeNode"/>

   <xsl:variable name="cppScopedConsumerTypeName">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_consumerTypeNode"/>
      </xsl:call-template>
   </xsl:variable>

DEFINE_VALUETYPE_DATA_TRAITS(<xsl:value-of select="$cppScopedConsumerTypeName"/>)
</xsl:template> <!-- end of template session_component_cpp.content.27 -->






<!--
   This template generates definitions of operations of a given interface.
   This includes inherited operations.

   @param _interfaceNode           The interface node.
   @param _facetName               The name of the facet.
   @param _facetTypeName           The type of the facet.
   @param _componentImplClassname  The component implementation class name.
   @param _cppFacetScope           The C++ scope for the class (namespaces ...).
-->
<xsl:template name="session_component_cpp.Monolithic.Facets.Operations.all">
   <xsl:param name="_interfaceNode"/>
   <xsl:param name="_facetName"/>
   <xsl:param name="_facetTypeName"/>
   <xsl:param name="_componentImplClassname"/>
   <xsl:param name="_cppFacetScope"/>

   <xsl:variable name="allBaseInterfaces">
       <xsl:call-template name="getAllBaseClasses">
           <xsl:with-param name="_classNode" select="$_interfaceNode"/>
       </xsl:call-template>
   </xsl:variable>

   <xsl:for-each select="$allBaseInterfaces/node()[string(@xmi.id)]">
       <xsl:sort select="@xmi.id"/>
       <xsl:variable name="_xmi_id" select="./@xmi.id"/>
       <!--
       NOTA: Note the following trick to get back to the initial document context
             to be able to access operation definitions ... etc. because the result
             tree ($allBaseInterfaces) is a new xml document a only contains a copy
             of a subset of the original tree.
       -->
       <xsl:for-each select="$_interfaceNode">
           <xsl:variable name="_interface" select="key('classById', $_xmi_id)"/>

           <xsl:call-template name="session_component_cpp.Monolithic.Facets.Operations.own">
                <xsl:with-param name="_interfaceNode" select="$_interface"/>
                <xsl:with-param name="_facetName" select="$_facetName"/>
                <xsl:with-param name="_facetTypeName" select="$_facetTypeName"/>
                <xsl:with-param name="_componentImplClassname" select="$_componentImplClassname"/>
                <xsl:with-param name="_cppFacetScope" select="$_cppFacetScope"/>
           </xsl:call-template> 
       </xsl:for-each>
   </xsl:for-each>
</xsl:template> <!-- end of template session_component_cpp.Monolithic.Facets.Operations.all -->


<!--
   This template generates definitions of operations of a given interface.
   This includes inherited operations.

   @param _interfaceNode           The interface node.
   @param _facetName               The name of the facet.
   @param _facetTypeName           The type of the facet.
   @param _componentImplClassname  The component implementation class name.
   @param _cppFacetScope           The C++ scope for the class (namespaces ...).
-->
<xsl:template name="session_component_cpp.Monolithic.Facets.Operations.own">
   <xsl:param name="_interfaceNode"/>
   <xsl:param name="_facetName"/>
   <xsl:param name="_facetTypeName"/>
   <xsl:param name="_componentImplClassname"/>
   <xsl:param name="_cppFacetScope"/>

      <xsl:variable name="containedOps"  select="key('operations', $_interfaceNode/@xmi.id)"/>
      <xsl:for-each select="$containedOps">
         <!--
            Get the operation return type.
         -->
         <xsl:variable name="returnType">
            <xsl:call-template name="getType">
               <xsl:with-param name="_id" select="UML:BehavioralFeature.parameter/UML:Parameter[@kind = 'return']/@type"/>
               <xsl:with-param name="_kind" select="'return'"/>
            </xsl:call-template>
         </xsl:variable>
         <xsl:variable name="objRefVarType">
            <xsl:if test="$returnType != 'void'">
               <xsl:call-template name="getType">
                  <xsl:with-param name="_id" select="UML:BehavioralFeature.parameter/UML:Parameter[@kind = 'return']/@type"/>
                  <xsl:with-param name="_kind" select="'inout'"/>
                  <xsl:with-param name="_asObjRefVar" select="true()"/>
               </xsl:call-template>
            </xsl:if>
         </xsl:variable>
         
         <!--
            Get the operation name.
         -->
         <xsl:variable name="cppOperationName">
            <xsl:call-template name="mapCppReservedWords">
               <xsl:with-param name="_operationName" select="@name"/>
            </xsl:call-template>
         </xsl:variable>
         <xsl:value-of select="concat($lf, $returnType, $lf, $_componentImplClassname, $cppSep, $cppOperationName, $lf)"/>
            (
               <!--
                  IN, OUT and INOUT parameters.
               -->
               <xsl:for-each select="UML:BehavioralFeature.parameter/UML:Parameter[@kind != 'return']">
                  <!--
                     Get the parameter type.
                  -->
                  <xsl:variable name="paramType">
                     <xsl:call-template name="getParameterType">
                        <xsl:with-param name="_paramNode" select="."/>
                     </xsl:call-template>
                  </xsl:variable>
                  <xsl:value-of select="concat($paramType, ' ', @name)"/>
                  <!--
                     Write out comma if necessary.
                  -->
                  <xsl:if test="position() != last()">
                     <xsl:value-of select="concat(',', $lf)"/>
                  </xsl:if>
               </xsl:for-each>
            )
         throw (
            <!--
               Exceptions.
            -->
            <xsl:for-each select="UML:ModelElement.taggedValue/UML:TaggedValue">
               <xsl:if test="@tag = 'raises'">
                  <xsl:value-of select="@value"/>,
               </xsl:if>
            </xsl:for-each>
            CORBA::SystemException
         )
         {
              CDMW_LOG_FUNCTION(m_logger.in(), "");
              check_is_removed();
              m_state = m_state->operational_request(this);
   
              try
               {
                <![CDATA[CORBA::Object_var obj = m_executor_locator->obtain_executor("]]><xsl:value-of select="$_facetName"/>");
   
               if (CORBA::is_nil(obj.in()))
               {
                  CDMW_ERROR(m_logger.in(), 
                      "Executor for facet <xsl:value-of select="$_facetName"/> is nil");
                  CORBA::OBJECT_NOT_EXIST ex(OrbSupport::OBJECT_NOT_EXIST, 
                                             CORBA::COMPLETED_NO);
                  CDMW_LOG_FUNCTION_EXCEPTION(ex);
                  throw ex;
               }
   
               <xsl:value-of select="concat($_cppFacetScope, $cppSep, 'CCM_', $_facetTypeName, '_var')"/> facet_exec = <xsl:value-of select="concat($_cppFacetScope, $cppSep, 'CCM_', $_facetTypeName)"/>::_narrow(obj.in());
       
               if (CORBA::is_nil(facet_exec.in()))
               {
                  CDMW_ERROR(m_logger.in(), 
                      "Executor for facet <xsl:value-of select="$_facetName"/> has wrong type");
                  CORBA::OBJECT_NOT_EXIST ex(OrbSupport::OBJECT_NOT_EXIST, 
                                             CORBA::COMPLETED_NO);
                  CDMW_LOG_FUNCTION_EXCEPTION(ex);
                  throw ex;
               }
       
               <xsl:if test="string-length($objRefVarType) > 0">
                   <xsl:value-of select="concat($objRefVarType, ' result =')"/>
               </xsl:if>
               <![CDATA[facet_exec->]]><xsl:value-of select="$cppOperationName"/>
                  (
                     <xsl:for-each select="UML:BehavioralFeature.parameter/UML:Parameter[@kind != 'return']">
                        <xsl:value-of select="@name"/>
                        <xsl:if test="position() != last()">
                           <xsl:text>, </xsl:text>
                        </xsl:if>
                     </xsl:for-each>
                  );
   
               <![CDATA[m_executor_locator->release_executor(facet_exec.in());]]>
   
               <xsl:if test="string-length($objRefVarType) > 0">
                  <xsl:choose>
                     <xsl:when test="contains($objRefVarType, '_var')">
                        return result._retn();
                     </xsl:when>
                     <xsl:otherwise>
                        return result;
                     </xsl:otherwise>
                   </xsl:choose>
               </xsl:if>
            }
               catch (const <![CDATA[Components::CCMException& e]]> )
               {
                  CDMW_ERROR(m_logger.in(), 
                      "Exception received: " <![CDATA[<< e]]>);
                  CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                                    CORBA::COMPLETED_NO);
                  CDMW_LOG_FUNCTION_EXCEPTION(ex);
                  throw ex;
               }
          }
      </xsl:for-each>

</xsl:template> <!-- end of template session_component_cpp.Monolithic.Facets.Operations.own -->

<!--
   This template generates definitions of operations of a given interface.
   This includes inherited operations.

   @param _interfaceNode           The interface node.
   @param _componentImplClassname  The component implementation class name.
-->
<xsl:template name="session_component_cpp.SupportedInterfaces.Operations.own">
   <xsl:param name="_interfaceNode"/>
   <xsl:param name="_componentImplClassname"/>
   <xsl:param name="_componentObjRefVar"/>

      <xsl:variable name="containedOps"  select="key('operations', $_interfaceNode/@xmi.id)"/>

      <xsl:for-each select="$containedOps">
         <!--
            Get the operation return type.
         -->
         <xsl:variable name="returnType">
            <xsl:call-template name="getType">
               <xsl:with-param name="_id" select="UML:BehavioralFeature.parameter/UML:Parameter[@kind = 'return']/@type"/>
               <xsl:with-param name="_kind" select="'return'"/>
            </xsl:call-template>
         </xsl:variable>
         <xsl:variable name="objRefVarType">
            <xsl:call-template name="getType">
               <xsl:with-param name="_id" select="UML:BehavioralFeature.parameter/UML:Parameter[@kind = 'return']/@type"/>
               <xsl:with-param name="_kind" select="'inout'"/>
               <xsl:with-param name="_asObjRefVar" select="true()"/>
            </xsl:call-template>
         </xsl:variable>

         <!--
            Operation signature.
         -->
         <xsl:variable name="cppOperationName">
            <xsl:call-template name="mapCppReservedWords">
               <xsl:with-param name="_operationName" select="@name"/>
            </xsl:call-template>
         </xsl:variable>
         <xsl:value-of select="concat($returnType, $lf, $_componentImplClassname, $cppSep, $cppOperationName, $lf)"/>
            (
               <!--
                  IN, OUT and INOUT parameters.
               -->
               <xsl:for-each select="UML:BehavioralFeature.parameter/UML:Parameter[@kind != 'return']">
                  <!--
                     Get the parameter type.
                  -->
                  <xsl:variable name="paramType">
                     <xsl:call-template name="getParameterType">
                        <xsl:with-param name="_paramNode" select="."/>
                     </xsl:call-template>
                  </xsl:variable>
                  <xsl:value-of select="concat($paramType, ' ', @name)"/>
                  <!--
                     Write out comma if necessary.
                  -->
                  <xsl:if test="position() != last()">
                     <xsl:value-of select="concat(',', $lf)"/>
                  </xsl:if>
               </xsl:for-each>
            )
         throw (
            <!--
               Exceptions.
            -->
            <xsl:for-each select="UML:ModelElement.taggedValue/UML:TaggedValue">
               <xsl:if test="@tag = 'raises'">
                  <xsl:value-of select="@value"/>,
               </xsl:if>
            </xsl:for-each>
            CORBA::SystemException
         )
         {
            CDMW_LOG_FUNCTION(m_logger.in(), "");
            check_is_removed();
            m_state = m_state->operational_request(this);

            <xsl:value-of select="$_componentObjRefVar"/> comp_exec = obtain_component_executor();
            <xsl:if test="string-length($objRefVarType) > 0">
                <xsl:value-of select="concat($objRefVarType, ' __result = ')"/>
            </xsl:if>
            <![CDATA[comp_exec->]]><xsl:value-of select="$cppOperationName"/>(
               <xsl:for-each select="UML:BehavioralFeature.parameter/UML:Parameter[@kind != 'return']">
                  <xsl:value-of select="@name"/>
                  <xsl:if test="position() != last()">
                     <xsl:text>, </xsl:text>
                  </xsl:if>
               </xsl:for-each>
            );

            try
            {
                <![CDATA[m_executor_locator->release_executor(comp_exec.in());]]>
            }
            catch (const <![CDATA[Components::CCMException& e]]> )
            {
                CDMW_ERROR(m_logger.in(), 
                    "CCMException received: " <![CDATA[<< e]]>);
                CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                                   CORBA::COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }

            <xsl:if test="string-length($objRefVarType) > 0">
                <xsl:choose>
                    <xsl:when test="contains($objRefVarType, '_var')">
                        return __result._retn();
                    </xsl:when>
                    <xsl:otherwise>
                        return __result;
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:if>
         }
      </xsl:for-each>

</xsl:template> <!-- end of template session_component_cpp.SupportedInterfaces.Operations.own -->


</xsl:stylesheet>


