<?xml version="1.0" encoding="utf-8"?>
<!-- ===================================================================== -->
<!--
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-->
<!-- ===================================================================== -->


<xsl:stylesheet version="1.1"
   xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
   xmlns:UML="org.omg/UML1.3">


<!--
   Main - This template fetches the necessary data from inputs
   to generate a file when FT.

   @param none
-->
<xsl:template name="ft_session_home_hpp">
   <xsl:call-template name="session_home_hpp">
      <xsl:with-param name="_hasFaultTolerance" select="'yes'"/>
   </xsl:call-template>
</xsl:template> <!-- end of template ft_session_home_cpp -->


<!--
   Main - This template fetches the necessary data from inputs
   to generate a file when LB.

   @param none
-->
<xsl:template name="lb_session_home_hpp">
   <xsl:call-template name="session_home_hpp">
      <xsl:with-param name="_hasLoadBalancing" select="'yes'"/>
   </xsl:call-template>
</xsl:template> <!-- end of template lb_session_home_cpp -->



<!--
   Main - This template fetches the necessary data from inputs
   to generate a file. It can be called by ft_session_home_cpp template
   if FT. In this case the parameter _hasFaultTolerance is set to true.
   It can be called by lb_session_home_cpp template
   if LB. In this case the parameter _hasLoadBalancing is set to true.
   
   @param none
-->
<xsl:template name="session_home_hpp">
   <xsl:param name="_hasFaultTolerance" select="'no'"/>
   <xsl:param name="_hasLoadBalancing" select="'no'"/>
   <!--
      Parameters below are used for recursiveness.
   -->
   <xsl:param name="_index" select="1"/>

   <!--
      _hasFaultTolerance and _hasLoadBalancing cannot be 'yes' at the same time.
   -->
   <xsl:if test="($_hasFaultTolerance = 'yes') and ($_hasLoadBalancing = 'yes')">
      <xsl:message terminate="yes">
         ERROR: Cannot generate Session Home with FaultTolerance and LoadBalancing at the same time
      </xsl:message>
   </xsl:if>
   
   <xsl:if test="boolean($lifecycleFrameworkNodes/home-impl) and $lifecycleFrameworkNodes/home-impl/@category = 'SESSION'">
      <xsl:if test="$_index &lt;= count($lifecycleFrameworkNodes/home-impl)">
         <xsl:variable name="homeImplNode" select="$lifecycleFrameworkNodes[home-impl and position() = $_index]/home-impl"/>

         <!--
            Get the home impl class name.
         -->
         <xsl:variable name="scopedHomeImplClassname">
            <xsl:call-template name="buildHomeImplClassname">
               <xsl:with-param name="_homeImplNode" select="$homeImplNode"/>
            </xsl:call-template>
         </xsl:variable>

         <xsl:variable name="shortHomeImplClassname">
            <xsl:call-template name="getLastToken">
               <xsl:with-param name="_string" select="$scopedHomeImplClassname"/>
               <xsl:with-param name="_separator" select="$xmlSep"/>
            </xsl:call-template>
         </xsl:variable>

         <xsl:variable name="homeImplClassname">
            <xsl:choose>
               <xsl:when test="$_hasFaultTolerance = 'yes'">
                  <xsl:value-of select="concat('FT', $shortHomeImplClassname)"/>
               </xsl:when>
               <xsl:when test="$_hasLoadBalancing = 'yes'">
                  <xsl:value-of select="concat('LB', $shortHomeImplClassname)"/>
               </xsl:when>
               <xsl:otherwise>
                  <xsl:value-of select="$shortHomeImplClassname"/>
               </xsl:otherwise>
            </xsl:choose>
         </xsl:variable>

         <!--
            Get the output filename.
         -->
         <xsl:variable name="filename">
            <xsl:call-template name="buildOutputFilename">
               <xsl:with-param name="_userDirectory" select="$homeImplNode/@directory"/>
               <xsl:with-param name="_userFilename" select="$homeImplNode/@filename"/>
               <xsl:with-param name="_defaultFilename" select="$homeImplClassname"/>
               <xsl:with-param name="_fileExtension" select="'.hpp'"/>
            </xsl:call-template>
         </xsl:variable>

         <!--
            Display a message to the user.
         -->
         <xsl:message>
            <xsl:value-of select="concat('Generating file ', $filename)"/>
         </xsl:message>

         <!--
            Generate the file.
         -->
         <xsl:document method="text" href="{$filename}">
            <!--
               Insert the document header.
            -->
            <xsl:call-template name="docHeader">
               <xsl:with-param name="_lang" select="'cpp'"/>
            </xsl:call-template>
            <!--
               Insert the #ifndef #define instructions.
            -->
            <xsl:call-template name="writeDefine">
               <xsl:with-param name="_prefix" select="'CCMCIF'"/>
               <xsl:with-param name="_classname" select="$homeImplClassname"/>
            </xsl:call-template>
            <!--
               Insert the content.
            -->
            <xsl:call-template name="session_home_hpp.content">
               <xsl:with-param name="_scopedHomeImplClassname" select="$scopedHomeImplClassname"/>
               <xsl:with-param name="_homeName">
                  <xsl:call-template name="getLastToken">
                     <xsl:with-param name="_string" select="$homeImplNode/../@factory-idref"/>
                     <xsl:with-param name="_separator" select="$xmlSep"/>
                  </xsl:call-template>
               </xsl:with-param>
               <xsl:with-param name="_hasFaultTolerance" select="$_hasFaultTolerance"/>
               <xsl:with-param name="_hasLoadBalancing" select="$_hasLoadBalancing"/>
               <xsl:with-param name="_fullHomeName" select="$homeImplNode/../@factory-idref"/>
            </xsl:call-template>

<!--
         <xsl:variable name="_fullHomeName" select="$homeImplNode/../@factory-idref"/>
         <xsl:message>
            <xsl:value-of select="concat('The Full home name  is ', $_fullHomeName)"/>
         </xsl:message>
-->
            <!--
               Insert the #endif instruction.
            -->
            <xsl:call-template name="writeDefine">
               <xsl:with-param name="_prefix" select="'CCMCIF'"/>
               <xsl:with-param name="_classname" select="$homeImplClassname"/>
               <xsl:with-param name="_endif" select="true()"/>
            </xsl:call-template>
         </xsl:document>

         <!--
            Proceed to the next home impl if any.
         -->
         <xsl:call-template name="session_home_hpp">
            <xsl:with-param name="_hasFaultTolerance" select="$_hasFaultTolerance"/>
            <xsl:with-param name="_hasLoadBalancing" select="$_hasLoadBalancing"/>
            <xsl:with-param name="_index" select="$_index + 1"/>
         </xsl:call-template>
      </xsl:if>
   </xsl:if>
</xsl:template> <!-- end of template session_home_hpp -->


<!--
   This template generates the content of the file.

   @param _scopedHomeImplClassname   The scoped home impl classname.
   @param _homeName                  The home name.
-->
<xsl:template name="session_home_hpp.content">
   <xsl:param name="_scopedHomeImplClassname"/>
   <xsl:param name="_homeName"/>
   <xsl:param name="_hasFaultTolerance"/>
   <xsl:param name="_hasLoadBalancing"/>
   <xsl:param name="_fullHomeName"/>

    <xsl:variable name="_idl3Filename">
      <xsl:variable name="homeFactoryNode" select="$factoryNodes[@idl-interface =$_fullHomeName]"/>   
       <xsl:choose>
          <xsl:when test="boolean($homeFactoryNode/@idl-filename)">
             <xsl:value-of select="$homeFactoryNode/@idl-filename"/>
         </xsl:when>
          <xsl:otherwise>
             <xsl:variable name="interfaceName">
                <xsl:call-template name="getLastToken">
                   <xsl:with-param name="_string" select="$homeFactoryNode/@idl-interface"/>
                   <xsl:with-param name="_separator" select="$xmlSep"/>
                </xsl:call-template>
             </xsl:variable>
             <xsl:value-of select="concat($interfaceName, '.idl')"/>
          </xsl:otherwise>
       </xsl:choose>
    </xsl:variable>

   <!--
      References to some nodes.
   -->
   <xsl:variable name="homeNode" select="key('classByName', $_homeName)"/>

   <!--
      Miscellaneous variables.
   -->
   <xsl:variable name="shortHomeImplClassname">
      <xsl:call-template name="getLastToken">
         <xsl:with-param name="_string" select="$_scopedHomeImplClassname"/>
         <xsl:with-param name="_separator" select="$xmlSep"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="homeImplClassname">
      <xsl:choose>
         <xsl:when test="$_hasFaultTolerance = 'yes'">
            <xsl:value-of select="concat('FT', $shortHomeImplClassname)"/>
         </xsl:when>
         <xsl:when test="$_hasLoadBalancing = 'yes'">
            <xsl:value-of select="concat('LB', $shortHomeImplClassname)"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:value-of select="$shortHomeImplClassname"/>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:variable>
   <xsl:variable name="cppScopedHomeName">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$homeNode"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="cppHomeScope">
      <xsl:call-template name="removeLastToken">
         <xsl:with-param name="_scopedName" select="$cppScopedHomeName"/>
         <xsl:with-param name="_sep" select="$cppSep"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="scopedManagedComponentName">
      <xsl:variable name="ccmManagesId" select="key('stereotypeByName', 'CCMManages')/@xmi.id"/>
      <xsl:variable name="homeNodeId" select="key('classByName', $_homeName)/@xmi.id"/>
      <xsl:variable name="managesNode" select="key('associationByStereotypeId', $ccmManagesId)[UML:Association.connection/UML:AssociationEnd[1]/@type = $homeNodeId]"/>
      <xsl:variable name="managedComponentId" select="$managesNode/UML:Association.connection/UML:AssociationEnd[2]/@type"/>
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="key('classById', $managedComponentId)"/>
      </xsl:call-template>
   </xsl:variable>

   <!--
      The code generation starts here.
   -->

   <xsl:variable name="pathPrefix">
       <xsl:choose>
           <xsl:when test="string-length($CSC_NAME) > 0">
               <xsl:value-of select="concat($CSC_NAME, $fileSep)"/>
           </xsl:when>
           <xsl:otherwise>
               <xsl:value-of select="''"/>
           </xsl:otherwise>
       </xsl:choose>
   </xsl:variable>


         <xsl:variable name="idlName">
            <xsl:call-template name="getLastToken">
               <xsl:with-param name="_string" select="substring-before($_idl3Filename, '.idl')"/>
               <xsl:with-param name="_separator" select="$fileSep"/>
            </xsl:call-template>
         </xsl:variable>             

         <xsl:variable name="idlDirName">
               <xsl:value-of  select="substring-before($_idl3Filename, $idlName)"/>
         </xsl:variable>


 <xsl:choose>
    <xsl:when test="boolean($cdmwNode/idl-structure)">
      <![CDATA[#include "]]><xsl:value-of select="concat($pathPrefix, $idlDirName, 'Cdmw_', $idlName , '_cif')"/><![CDATA[.skel.hpp"]]>
    </xsl:when>
    <xsl:otherwise>
      <![CDATA[#include "]]><xsl:value-of select="concat($pathPrefix, 'Cdmw_', $idlName , '_cif')"/><![CDATA[.skel.hpp"]]>
    </xsl:otherwise>
 </xsl:choose>

<![CDATA[
#include <CCMContainer/ccmcif/CCMHome_impl.hpp>
#include <CCMContainer/ccmcif/HomeConfiguration_impl.hpp>
#include <CCMContainer/ccmcontainer/HomeAllocator.hpp>
]]>
   <xsl:if test="$_hasFaultTolerance = 'yes'">
      <![CDATA[
#include <CCMContainer/ccmcif/FTCCMHome_impl.hpp>
]]>
   </xsl:if>
   <xsl:if test="$_hasLoadBalancing = 'yes'">
      <![CDATA[
#include <CCMContainer/ccmcif/LBCCMHome_impl.hpp>
]]>
   </xsl:if>


   <xsl:call-template name="openNamespace">
      <xsl:with-param name="_scope" select="concat('Cdmw::CCM::CIF::Cdmw',$cppHomeScope)"/>
      <xsl:with-param name="_separator" select="$cppSep"/>
      <xsl:with-param name="_lastTokenIsNamespace" select="true()"/>
   </xsl:call-template>

   class <xsl:value-of select="$homeImplClassname"/> : virtual public POA_Cdmw_<xsl:value-of select="concat($cppHomeScope, $cppSep, $_homeName, ',')"/>
   <xsl:choose>
      <xsl:when test="$_hasFaultTolerance = 'yes'">
                                                       public FTCCMHome_impl,
      </xsl:when>
      <xsl:when test="$_hasLoadBalancing = 'yes'">
                                                       public LBCCMHome_impl,
      </xsl:when>
      <xsl:otherwise>
                                                       public CCMHome_impl,
      </xsl:otherwise>
   </xsl:choose>
                                                       virtual public HomeConfiguration_impl
   {
      friend class Container::HomeAllocator<![CDATA[<]]><xsl:value-of select="$homeImplClassname"/><![CDATA[>;]]>

      public:
         // Destructor
         ~<xsl:value-of select="$homeImplClassname"/>() throw();

         /**
          * Home attributes
          */

         <xsl:call-template name="findAllInterfaceReadWriteAttributes">
            <xsl:with-param name="_interfaceNode" select="$homeNode"/>
            <xsl:with-param name="_type" select="'Home'"/>
            <xsl:with-param name="_template" select="'session_home_hpp.content.2'"/>
         </xsl:call-template>

         /**
          * Explicit interface
          */

         <xsl:call-template name="findAllHomeFactoryOperations">
            <xsl:with-param name="_homeNode" select="$homeNode"/>
            <xsl:with-param name="_template" select="'session_home_hpp.content.1'"/>
         </xsl:call-template>

         /**
          * Implicit interface
          */

         virtual <xsl:value-of select="$scopedManagedComponentName"/>_ptr
         create()
            throw(Components::CreateFailure,
                  CORBA::SystemException);

         // Keyless home operation

         virtual Components::CCMObject_ptr
         create_component()
            throw(Components::CreateFailure,
                  CORBA::SystemException);

         // CDMW Keyless home operation
         Components::CCMObject_ptr
         create_component_with_config_values(const <![CDATA[Components::ConfigValues&]]> config)
            throw(Components::CreateFailure,
                  CORBA::SystemException);


         <![CDATA[
            /**
             * Purpose:
             * <p>This method acts as a hand shaking between the Servant Provider
             * and the generated Factory implementation. It indicates whether
             * a Servant provided by this Servant Provider may incarnate multiple
             * objects (with ObjectIds provided by this Servant Provider)
             *
             * @return <B>true</B> if a servant may incarnate multiple objects,
             * <B>false</B> otherwise.
             */
            virtual bool may_incarnate_multiple_objects() const throw()
            {
               return false;
            }

            virtual void remove()
                throw(CosLifeCycle::NotRemovable,
                      CORBA::SystemException);
         ]]>

      protected:
         // Constructor
         <xsl:value-of select="$homeImplClassname"/>(Components::HomeExecutorBase_ptr home_exec,
                                const <![CDATA[Cdmw::LifeCycle::ObjectDeactivator&]]> deactivator,
                                CdmwLifeCycle::ObjectRegistration_ptr      object_registration,
                                const <![CDATA[Components::ConfigValues&]]> config)
            throw(CORBA::SystemException);

         <![CDATA[
            /**
             * This returns an array containing repository IDs of all objects
             * supported by the current ServantProvider.
             * <B>NOTA:</B> Repository IDs should be statically allocated; thus derived classes
             * should not delete return pointer(s).
             */
            virtual const char ** get_ids() const throw();
         ]]>

         <![CDATA[
            /**
             * This returns the number of repository Ids supported by the current ServantProvider.
             * <B>NOTA:</B> This should correspond to the number of null terminated strings
             * returned by get_ids() operation.
             */
            virtual CORBA::Long get_number_of_ids() const throw();
         ]]>

         <![CDATA[
            /**
             * Purpose:
             * <p>
             *   This creates new component servant.
             *   This shall be implemented by the well typed home servant.
             *
             * @param ctx      The component context
             * @param executor The component executor
             *
             * @return The new component servant.
             * @exception Components::CreateFailure if a problem occured when creating executor.
             * @exception Any CORBA Exception
             *
             */
            virtual PortableServer::Servant
            create_component_servant(const std::string&  comp_oid,
                                     Context*            ctx,
                                     ComponentInfo&      comp_info)
               throw(Components::CreateFailure,
                     CORBA::SystemException);
         ]]>

         <xsl:if test="($_hasFaultTolerance = 'yes') or ($_hasLoadBalancing = 'yes')">
         <![CDATA[
            /**
             * Purpose:
             * <p>
             *   Notify supervision that a component reference has been added to a 
             *   FT/LB group. Code is in generated Home servant inheriting from this class.
             *
             *@param component_name The component's name
             */
            virtual void notify_add_member(const char* component_name);
         ]]>
         </xsl:if>
   
      private:
         <xsl:value-of select="$homeImplClassname"/>(const <xsl:value-of select="$homeImplClassname"/><![CDATA[&);]]>
         void operator=(const <xsl:value-of select="$homeImplClassname"/><![CDATA[&);]]>

         // Home executor
         <xsl:value-of select="concat($cppHomeScope, $cppSep, 'CCM_', $_homeName, '_var')"/> m_home_executor;
   };


   <xsl:call-template name="closeNamespace">
      <xsl:with-param name="_scope" select="concat('Cdmw::CCM::CIF::Cdmw',$cppHomeScope)"/>
      <xsl:with-param name="_separator" select="$cppSep"/>
      <xsl:with-param name="_lastTokenIsNamespace" select="true()"/>
   </xsl:call-template>

   namespace Cdmw
   {
   namespace CCM
   {
   namespace Container
   {
      <![CDATA[template <>]]>
      PortableServer::Servant
      HomeAllocator<![CDATA[<]]><xsl:value-of select="concat('CIF::Cdmw', $cppHomeScope, $cppSep, $homeImplClassname)"/><![CDATA[>::allocate]]>
                  (Components::HomeExecutorBase_ptr            home_executor,
                   const <![CDATA[Cdmw::LifeCycle::ObjectDeactivator &]]> deactivator,
                   CdmwLifeCycle::ObjectRegistration_ptr      object_registration,
                   const <![CDATA[Components::ConfigValues&]]> config)
         throw(CORBA::SystemException,
               std::bad_alloc);
   }; // end of namespace Container

   }; // end of namespace CCM

   }; // end of namespace Cdmw

   <!--
      The code generation stops here.
   -->

</xsl:template> <!-- end of template session_home_hpp.content -->



<!--
-->
<xsl:template name="session_home_hpp.content.1">
   <xsl:param name="_node"/>
   <xsl:param name="_returnType"/>
   <xsl:param name="_operationName"/>

   // Factory operation

   <xsl:value-of select="concat($lf, 'virtual ', $_returnType, $lf, $_operationName, $lf)"/>
      (
         <!--
            IN, OUT and INOUT parameters.
         -->
         <xsl:for-each select="$_node/UML:BehavioralFeature.parameter/UML:Parameter[@kind != 'return']">
            <!--
               Get the parameter type.
            -->
            <xsl:variable name="paramType">
               <xsl:call-template name="getParameterType">
                  <xsl:with-param name="_paramNode" select="."/>
               </xsl:call-template>
            </xsl:variable>
            <xsl:value-of select="concat($paramType, ' ', @name)"/>
            <!--
               Write out comma if necessary.
            -->
            <xsl:if test="position() != last()">
               <xsl:value-of select="concat(',', $lf)"/>
            </xsl:if>
         </xsl:for-each>
      )
      throw (Components::CreateFailure,
             CORBA::SystemException);
</xsl:template> <!-- end of template session_home_hpp.content.1 -->


<xsl:template name="session_home_hpp.content.2">
   <xsl:param name="_node"/>
   <xsl:param name="_readonlyId"/>
   <xsl:param name="_readonlyEndId"/>
   <xsl:param name="_paramType"/>
   <xsl:param name="_returnType"/>
   <xsl:param name="_cppOperationName"/>

   <xsl:if test="not(boolean($_node/@stereotype)) or ($_node/@stereotype = $_readonlyEndId) or ($_node/@stereotype = $_readonlyId)">
      <xsl:value-of select="concat($lf, 'virtual ', $_returnType, ' ', $_cppOperationName, '()', $lf, 'throw(CORBA::SystemException);', $lf)"/>

      <xsl:if test="not($_node/@stereotype = $_readonlyEndId or $_node/@stereotype = $_readonlyId)">
         <xsl:value-of select="concat($lf, 'virtual void ', $_cppOperationName, '(', $_paramType, ')', $lf, 'throw(CORBA::SystemException);', $lf)"/>
      </xsl:if>
   </xsl:if>
</xsl:template> <!-- end of template session_home_hpp.content.2 -->



</xsl:stylesheet>


