/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_CCM_CIF_CCMCONTEXT_EXT_IMPL_HPP
#define INCL_CCM_CIF_CCMCONTEXT_EXT_IMPL_HPP

#include <CCMContainer/idllib/CdmwComponents.skel.hpp>
#include <CCMContainer/idllib/CdmwCcmContainer.skel.hpp>

#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/logging/CdmwLogging.stub.hpp>
#include <string>
#include <map>


namespace Cdmw {

namespace CCM {

namespace CIF {


/**
*Purpose:
*<p>    [class description]
*    This class implement the Components::CCMContext interface.
*    It will be inherited by all components context implementation.
*
*/

//
// IDL:omg.org/CdmwComponents/CCMContextExt:1.0
//
class CCMContextExt_impl : virtual public CdmwComponents::CCMContextExt,
                           virtual public OrbSupport::RefCountLocalObject
{
public:

    CCMContextExt_impl(Components::CCMContext_ptr ctx,
                       CdmwLogging::LoggerFactory_ptr logger_factory);
     ~CCMContextExt_impl();

    //
    // IDL:omg.org/Components/CCMContext/get_caller_principal:1.0
    //
    virtual Components::Principal_ptr get_caller_principal()
        throw(CORBA::SystemException);

    //
    // IDL:omg.org/Components/CCMContext/get_CCM_home:1.0
    //
    virtual Components::CCMHome_ptr get_CCM_home()
        throw(CORBA::SystemException);

    //
    // IDL:omg.org/Components/CCMContext/get_rollback_only:1.0
    //
    virtual CORBA::Boolean get_rollback_only()
        throw(Components::IllegalState,
              CORBA::SystemException);

    //
    // IDL:omg.org/Components/CCMContext/get_user_transaction:1.0
    //
    virtual Components::Transaction::UserTransaction_ptr get_user_transaction()
        throw(Components::IllegalState,
              CORBA::SystemException);

    //
    // IDL:omg.org/Components/CCMContext/is_caller_in_role:1.0
    //
    virtual CORBA::Boolean is_caller_in_role(const char* role)
        throw(CORBA::SystemException);

    //
    // IDL:omg.org/Components/CCMContext/set_rollback_only:1.0
    //
    virtual void set_rollback_only()
        throw(Components::IllegalState,
              CORBA::SystemException);

    //
    // IDL:omg.org/Components/CCMContext/resolve_service_reference:1.0
    //
    virtual CORBA::Object_ptr resolve_service_reference(const char* service_id) 
        throw(Components::CCMException,
              CORBA::SystemException);
            
    //
    // IDL:omg.org/CdmwCif/CCMContextExt/get_service:1.0
    //
    virtual CORBA::Object_ptr get_service(const char* identifier)
        throw(CdmwComponents::CCMContextExt::ServiceUnavailable,
              CORBA::SystemException);


    //
    // IDL:omg.org/Components/ExtensionContext/install_service_reference:1.0
    //
    virtual Components::Cookie* 
    install_service_reference(const char* service_id, CORBA::Object_ptr objref) 
        throw(Components::CCMException,
              CORBA::SystemException); 
    
    //
    // IDL:omg.org/Components/ExtensionContext/uninstall_service_reference:1.0
    //
    virtual CORBA::Object_ptr uninstall_service_reference(Components::Cookie* ck) 
        throw(Components::CCMException,
              CORBA::SystemException); 
              
private:
    CCMContextExt_impl();
    CCMContextExt_impl(const CCMContextExt_impl&);
    void operator=(const CCMContextExt_impl&);
     
    CdmwCcmContainer::CCM2Context_var m_context;   // container context
    
    // map of installed services
    typedef std::map<std::string, CORBA::Object_var> ServicesMap;
    ServicesMap m_services_map;
    
    // logger to be used in CCMContext
    CdmwLogging::Logger_var m_logger;

};  // End CCMContextExt_impl class

} // End namespace CIF

} // End namespace CCM

} // End namespce Cdmw


#endif  // INCL_CCM_CIF_CCMCONTEXT_EXT_IMPL_HPP

