/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_DEFAULT_CCM_OBJECT_STATES_HPP
#define INCL_DEFAULT_CCM_OBJECT_STATES_HPP

// Cdmw Files
#include <CCMContainer/ccmcif/CCMObjectState.hpp>


namespace Cdmw {

namespace CCM {

namespace CIF {


// Implementation of INITIAL CCMObject's state
class CCMObjectState_Initial : virtual public CCMObjectState
{
public:
    virtual const CCMObjectState* 
    configuration_complete(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);
    
    virtual const CCMObjectState* 
    remove(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);
        
    virtual const CCMObjectState* 
    activate(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);
    
    virtual const CCMObjectState* 
    deactivate(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);

    virtual const CCMObjectState* 
    operational_request(CCMObject_impl* ccm_object) const
        throw (CORBA::SystemException);
};

/**
 * INITIAL CCMObject's state.
 */
static const CCMObjectState* CCM_OBJECT_STATE_INITIAL =
    new CCMObjectState_Initial();


// Implementation of FULLY_ACTIVATED CCMObject's state
class CCMObjectState_FullyActivated : virtual public CCMObjectState
{
public:
    virtual const CCMObjectState* 
    configuration_complete(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);
    
    virtual const CCMObjectState* 
    remove(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);
        
    virtual const CCMObjectState* 
    activate(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);
    
    virtual const CCMObjectState* 
    deactivate(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);

    virtual const CCMObjectState* 
    operational_request(CCMObject_impl* ccm_object) const
        throw (CORBA::SystemException);
};

/**
 * FULLY_ACTIVATED CCMObject's state.
 */
static const CCMObjectState* CCM_OBJECT_STATE_FULLY_ACTIVATED =
    new CCMObjectState_FullyActivated();


// Implementation of REMOVED CCMObject's state
class CCMObjectState_Removed : virtual public CCMObjectState
{
public:
    virtual const CCMObjectState* 
    configuration_complete(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);
    
    virtual const CCMObjectState* 
    remove(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);
        
    virtual const CCMObjectState* 
    activate(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);
    
    virtual const CCMObjectState* 
    deactivate(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);

    virtual const CCMObjectState* 
    operational_request(CCMObject_impl* ccm_object) const
        throw (CORBA::SystemException);
};

/**
 * REMOVED CCMObject's state.
 */
static const CCMObjectState* CCM_OBJECT_STATE_REMOVED =
    new CCMObjectState_Removed();

    
} // End of namespace CIF

} // End of namespace CCM

} // End of namespace Cdmw


#endif  // INCL_DEFAULT_CCM_OBJECT_STATES_HPP

