/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_FT_CCM_OBJECT_STATES_HPP
#define INCL_FT_CCM_OBJECT_STATES_HPP
                      
// Cdmw Files
#include <CCMContainer/ccmcif/CCMObjectState.hpp>


namespace Cdmw {

namespace CCM {

namespace CIF {

// Implementation of INITIAL FTCCMObject's state
class FTCCMObjectState_Initial : virtual public CCMObjectState
{
public:
    virtual const CCMObjectState* 
    configuration_complete(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);
    
    virtual const CCMObjectState* 
    remove(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);
        
    virtual const CCMObjectState* 
    activate(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);
    
    virtual const CCMObjectState* 
    deactivate(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);

    virtual const CCMObjectState* 
    operational_request(CCMObject_impl* ccm_object) const
        throw (CORBA::SystemException);
};

/**
 * INITIAL FTCCMObject's state.
 */
static const CCMObjectState* FT_CCM_OBJECT_STATE_INITIAL =
    new FTCCMObjectState_Initial();


// Implementation of FT_ACTIVATED FTCCMObject's state
class FTCCMObjectState_FTActivated : virtual public CCMObjectState
{
public:
    virtual const CCMObjectState* 
    configuration_complete(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);
    
    virtual const CCMObjectState* 
    remove(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);
        
    virtual const CCMObjectState*
    activate(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);
    
    virtual const CCMObjectState* 
    deactivate(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);

    virtual const CCMObjectState* 
    operational_request(CCMObject_impl* ccm_object) const
        throw (CORBA::SystemException);
};

/**
 * FT_ACTIVATED FTCCMObject's state.
 */
static const CCMObjectState* FT_CCM_OBJECT_STATE_FT_ACTIVATED =
    new FTCCMObjectState_FTActivated();

 
// Implementation of CCM_ACTIVATED FTCCMObject's state
class FTCCMObjectState_CCMActivated : virtual public CCMObjectState
{
public:
    virtual const CCMObjectState* 
    configuration_complete(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);
    
    virtual const CCMObjectState* 
    remove(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);
        
    virtual const CCMObjectState* 
    activate(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);
    
    virtual const CCMObjectState* 
    deactivate(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);

    virtual const CCMObjectState* 
    operational_request(CCMObject_impl* ccm_object) const
        throw (CORBA::SystemException);
};

/**
 * CCM_ACTIVATED FTCCMObject's state.
 */
static const CCMObjectState* FT_CCM_OBJECT_STATE_CCM_ACTIVATED =
    new FTCCMObjectState_CCMActivated();


// Implementation of FULLY_ACTIVATED FTCCMObject's state
class FTCCMObjectState_FullyActivated : virtual public CCMObjectState
{
public:
    virtual const CCMObjectState* 
    configuration_complete(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);
    
    virtual const CCMObjectState* 
    remove(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);
        
    virtual const CCMObjectState* 
    activate(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);
    
    virtual const CCMObjectState* 
    deactivate(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);

    virtual const CCMObjectState* 
    operational_request(CCMObject_impl* ccm_object) const
        throw (CORBA::SystemException);
};

/**
 * FULLY_ACTIVATED FTCCMObject's state.
 */
static const CCMObjectState* FT_CCM_OBJECT_STATE_FULLY_ACTIVATED =
    new FTCCMObjectState_FullyActivated();

    
// Implementation of REMOVED FTCCMObject's state
class FTCCMObjectState_Removed : virtual public CCMObjectState
{
public:
    virtual const CCMObjectState* 
    configuration_complete(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);
    
    virtual const CCMObjectState* 
    remove(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);
        
    virtual const CCMObjectState* 
    activate(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);
    
    virtual const CCMObjectState* 
    deactivate(CCMObject_impl* ccm_object) const
        throw (Components::CCMException, CORBA::SystemException);

    virtual const CCMObjectState* 
    operational_request(CCMObject_impl* ccm_object) const
        throw (CORBA::SystemException);
};

/**
 * REMOVED FTCCMObject's state.
 */
static const CCMObjectState* FT_CCM_OBJECT_STATE_REMOVED =
    new FTCCMObjectState_Removed();


} // End of namespace CIF

} // End of namespace CCM

} // End of namespace Cdmw

#endif  // INCL_FT_CCM_OBJECT_STATES_HPP

