/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_CCM_CIF_LOGGER_IMPL_HPP
#define INCL_CCM_CIF_LOGGER_IMPL_HPP

// Cdmw Files
#include <Foundation/logging/CdmwLogging.skel.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/logging/Logger.hpp>


namespace Cdmw {

namespace CCM {

namespace CIF {


//
// IDL:com.thalesgroup/CdmwLogging/Logger:1.0
//
class Logger_impl : virtual public CdmwLogging::Logger,
                    virtual public OrbSupport::RefCountLocalObject
{

public:

    Logger_impl(const char* component_name,
                const char* domain_name);
    
    ~Logger_impl() throw();

    
    //
    // IDL:com.thalesgroup/CdmwLogging/Logger/log:1.0
    //
    virtual void log(CdmwLogging::LogLevel level,
                     const char * message,
                     const char * file,
                     CORBA::Long line)
        throw(CORBA::SystemException);
    
        
    //
    // IDL:com.thalesgroup/CdmwLogging/Logger/debug:1.0
    //
    virtual void debug(const char * message,
                       const char * file,
                       CORBA::Long line)
        throw(CORBA::SystemException);
    
    
    //
    // IDL:com.thalesgroup/CdmwLogging/Logger/info:1.0
    //
    virtual void info(const char * message,
                      const char * file,
                      CORBA::Long line)
        throw(CORBA::SystemException);
    
    
    //
    // IDL:com.thalesgroup/CdmwLogging/Logger/warn:1.0
    //
    virtual void warn(const char * message,
                      const char * file,
                      CORBA::Long line)
        throw(CORBA::SystemException);
    
    
    //
    // IDL:com.thalesgroup/CdmwLogging/Logger/error:1.0
    //
    virtual void error(const char * message,
                       const char * file,
                       CORBA::Long line)
        throw(CORBA::SystemException);
    
    
    //
    // IDL:com.thalesgroup/CdmwLogging/Logger/fatal:1.0
    //
    virtual void fatal(const char * message,
                       const char * file,
                       CORBA::Long line)
        throw(CORBA::SystemException);
    
    
    //
    // IDL:com.thalesgroup/CdmwLogging/Logger/is_enabled_for:1.0
    //
    virtual CORBA::Boolean is_enabled_for(CdmwLogging::LogLevel level)
        throw(CORBA::SystemException);
    
    
    //
    // IDL:com.thalesgroup/CdmwLogging/Logger/is_debug_enabled:1.0
    //
    virtual CORBA::Boolean is_debug_enabled()
        throw(CORBA::SystemException);
    
    
    //
    // IDL:com.thalesgroup/CdmwLogging/Logger/is_info_enabled:1.0
    //
    virtual CORBA::Boolean is_info_enabled()
        throw(CORBA::SystemException);
    
    
    //
    // IDL:com.thalesgroup/CdmwLogging/Logger/is_warn_enabled:1.0
    //
    virtual CORBA::Boolean is_warn_enabled()
        throw(CORBA::SystemException);
    
    
    //
    // IDL:com.thalesgroup/CdmwLogging/Logger/is_error_enabled:1.0
    //
    virtual CORBA::Boolean is_error_enabled()
        throw(CORBA::SystemException);
    
    
    //
    // IDL:com.thalesgroup/CdmwLogging/Logger/is_fatal_enabled:1.0
    //
    virtual CORBA::Boolean is_fatal_enabled()
        throw(CORBA::SystemException);


private:
    Logger_impl();
    Logger_impl(const Logger_impl&);
    void operator=(const Logger_impl&);

    Cdmw::Logging::Logger* m_logger;
};




} // End of namespace CIF

} // End of namespace CCM

} // End of namespace Cdmw

#endif  // INCL_CCM_CIF_LOGGER_IMPL_HPP

