/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <CCMContainer/ccmcif/LifeCycleObjectBase.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/common/Assert.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>


namespace 
{

} // end anonymous namespace


namespace Cdmw {

namespace CCM {

namespace CIF {



LifeCycleObjectBase::LifeCycleObjectBase(Context* ctx)
    throw(CORBA::SystemException)
    : m_context(ctx),
       m_is_removed(false)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), ""); 
    CDMW_ASSERT(ctx);
}

LifeCycleObjectBase::~LifeCycleObjectBase()
    throw()
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), ""); 
    // do nothing
}

//
// IDL:omg.org/CosLifeCycle/LifeCycleObject/copy:1.0
//
CosLifeCycle::LifeCycleObject_ptr 
LifeCycleObjectBase::copy(CosLifeCycle::FactoryFinder_ptr there,
                          const CosLifeCycle::Criteria& the_criteria)
    throw(CosLifeCycle::NoFactory,
          CosLifeCycle::NotCopyable,
          CosLifeCycle::InvalidCriteria,
          CosLifeCycle::CannotMeetCriteria,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), ""); 
    using namespace Cdmw::OrbSupport;
    throw CORBA::NO_IMPLEMENT(NO_IMPLEMENT,CORBA::COMPLETED_NO);
}

//
// IDL:omg.org/CosLifeCycle/LifeCycleObject/move:1.0
//
void 
LifeCycleObjectBase::move(CosLifeCycle::FactoryFinder_ptr there,
                          const CosLifeCycle::Criteria& the_criteria)
    throw(CosLifeCycle::NoFactory,
          CosLifeCycle::NotMovable,
          CosLifeCycle::InvalidCriteria,
          CosLifeCycle::CannotMeetCriteria,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), ""); 
    using namespace Cdmw::OrbSupport;
    throw CORBA::NO_IMPLEMENT(NO_IMPLEMENT,CORBA::COMPLETED_NO);
}

//
// IDL:omg.org/CosLifeCycle/LifeCycleObject/remove:1.0
//
// This should only be called with the context of a request invocation
void 
LifeCycleObjectBase::remove()
    throw(CosLifeCycle::NotRemovable,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), ""); 
    try {
        if (m_is_removed) {
            CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
                "LifeCycleObject already removed"); 
            CORBA::OBJECT_NOT_EXIST ex(Cdmw::OrbSupport::OBJECT_NOT_EXIST,
                                       CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }

          // deactivate object
        m_context->req_passivate();
          
        m_is_removed = true;

    } catch (const PortableServer::Current::NoContext & ) {
        // Not called within the context of an operation invocation
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
            "Not called within the context of an operation invocation"); 
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL,
                           CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const CORBA::SystemException & e) {
        // rethrow
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
            "SystemException raised: " <<
            Cdmw::OrbSupport::OrbSupport::exception_to_string(e)); 
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw;
    }
}

} // end namespace CIF
} // end namespace CCM
} // end namespace Cdmw

