/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Cdmw CCM files
#include <ccmcif/Logger_impl.hpp>



namespace Cdmw {

namespace CCM {

namespace CIF {

Logger_impl::Logger_impl(const char* component_name,
                         const char* domain_name)
{
    m_logger = Cdmw::Logging::Logger::Get_Logger(component_name, 
                                                 domain_name);
}

Logger_impl::~Logger_impl()
  throw()
{
    // do not delete m_logger since it will be deleted by LogManager
}

//
// IDL:com.thalesgroup/CdmwLogging/Logger/log:1.0
//
void Logger_impl::log(CdmwLogging::LogLevel level,
                      const char * message,
                      const char * file,
                      CORBA::Long line)
    throw(CORBA::SystemException)
{
    m_logger->log(level, message, file, line);
}

    
//
// IDL:com.thalesgroup/CdmwLogging/Logger/debug:1.0
//
void Logger_impl::debug(const char * message,
                        const char * file,
                        CORBA::Long line)
    throw(CORBA::SystemException)
{
    m_logger->debug(message, file, line);
}


//
// IDL:com.thalesgroup/CdmwLogging/Logger/info:1.0
//
void Logger_impl::info(const char * message,
                       const char * file,
                       CORBA::Long line)
    throw(CORBA::SystemException)
{
    m_logger->info(message, file, line);
}


//
// IDL:com.thalesgroup/CdmwLogging/Logger/warn:1.0
//
void Logger_impl::warn(const char * message,
                       const char * file,
                       CORBA::Long line)
    throw(CORBA::SystemException)
{
    m_logger->warn(message, file, line);
}


//
// IDL:com.thalesgroup/CdmwLogging/Logger/error:1.0
//
void Logger_impl::error(const char * message,
                        const char * file,
                        CORBA::Long line)
    throw(CORBA::SystemException)
{
    m_logger->error(message, file, line);
}


//
// IDL:com.thalesgroup/CdmwLogging/Logger/fatal:1.0
//
void Logger_impl::fatal(const char * message,
                        const char * file,
                        CORBA::Long line)
    throw(CORBA::SystemException)
{
    m_logger->fatal(message, file, line);
}


//
// IDL:com.thalesgroup/CdmwLogging/Logger/is_enabled_for:1.0
//
CORBA::Boolean Logger_impl::is_enabled_for(CdmwLogging::LogLevel level)
    throw(CORBA::SystemException)
{
    return m_logger->is_enabled_for(level);
}


//
// IDL:com.thalesgroup/CdmwLogging/Logger/is_debug_enabled:1.0
//
CORBA::Boolean Logger_impl::is_debug_enabled()
    throw(CORBA::SystemException)
{
    return m_logger->is_debug_enabled();
}


//
// IDL:com.thalesgroup/CdmwLogging/Logger/is_info_enabled:1.0
//
CORBA::Boolean Logger_impl::is_info_enabled()
    throw(CORBA::SystemException)
{
    return m_logger->is_info_enabled();
}

//
// IDL:com.thalesgroup/CdmwLogging/Logger/is_warn_enabled:1.0
//
CORBA::Boolean Logger_impl::is_warn_enabled()
    throw(CORBA::SystemException)
{
    return m_logger->is_warn_enabled();
}


//
// IDL:com.thalesgroup/CdmwLogging/Logger/is_error_enabled:1.0
//
CORBA::Boolean Logger_impl::is_error_enabled()
    throw(CORBA::SystemException)
{
    return m_logger->is_error_enabled();
}


//
// IDL:com.thalesgroup/CdmwLogging/Logger/is_fatal_enabled:1.0
//
CORBA::Boolean Logger_impl::is_fatal_enabled()
    throw(CORBA::SystemException)
{
    return m_logger->is_fatal_enabled();
}


} // end namespace CIF
} // end namespace CCM
} // end namespace Cdmw
