/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>

// Cdmw files
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/orbsupport/Codec.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/commonsvcs/time/Util.hpp>
#include <CdmwCcmCifC.h>

// Cdmw CCM files
#include <ccmcif/PushConsumer_impl.hpp>

#include <CCMContainer/ccmcommon/CCMLoggers.hpp>

DEFINE_VARIABLE_LENGTH_DATA_TRAITS(CdmwCcmCif::TimestampedEvent)

DEFINE_VALUETYPE_DATA_TRAITS(CORBA::ValueBase)

namespace
{

} // end anonymous namespace


namespace Cdmw {

namespace CCM {

namespace CIF {

    PushConsumer_impl::PushConsumer_impl(Context*                          ctx,
                                         Components::EventConsumerBase_ptr ccm_consumer)
        : LifeCycleObjectBase(ctx),
          m_ccm_consumer(Components::EventConsumerBase::_duplicate(ccm_consumer))
    {
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");

    }

    PushConsumer_impl::~PushConsumer_impl()
      throw()
    {
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
    }

    void PushConsumer_impl::disconnect_push_consumer()  throw(CORBA::SystemException)
    {
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
    }

    void PushConsumer_impl::push(const CORBA::Any& data) throw(CORBA::SystemException)
    {
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
        Components::EventBase_var evt;

        const CORBA::OctetSeq* p_encoded_data;
        data >>= p_encoded_data;
        CORBA::OctetSeq encoded_data(*p_encoded_data);

        Cdmw::OrbSupport::Codec<CdmwCcmCif::TimestampedEvent> codec;

        CdmwCcmCif::TimestampedEvent timestamped_event;

        codec.decode(encoded_data, timestamped_event);

        if (timestamped_event.event.in() == 0)
        {
            CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
                "PushConsumer_impl push nil event!");
            CORBA::BAD_PARAM ex(OrbSupport::BAD_PARAMBadEventType,
                                CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
        CDMW_DEBUG_EVENT(CCMLoggers::Get_ccmcif_logger(), "Event issued at: " 
            << Cdmw::CommonSvcs::Time::Util::TimeT2string_ISO8601(timestamped_event.timestamp) 
            << " received at: " 
            << Cdmw::CommonSvcs::Time::Util::TimeT2string_ISO8601(Cdmw::CommonSvcs::Time::Util::get_current_time()) 
            << std::endl);

        evt = Components::EventBase::_downcast(timestamped_event.event.in());
        
        CORBA::add_ref(evt.in());
        if (!evt.in())
        {
            CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
                "PushConsumer_impl push nil event!");
            CORBA::BAD_PARAM ex(OrbSupport::BAD_PARAMBadEventType,
                                CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }

        try
        {
            m_ccm_consumer->push_event(evt.in());
        }
        catch (const Components::BadEventType& )
        {
            CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
                "PushConsumer_impl push bad event type!");
            CORBA::BAD_PARAM ex(OrbSupport::BAD_PARAMBadEventType,
                                CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
    }


} // end namespace CIF
} // end namespace CCM
} // end namespace Cdmw
