/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Cdmw files
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/logging/FunctionLogger.hpp>

// Cdmw CCM files
#include <CCMContainer/ccmcif/SessionContextExt_impl.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>

namespace 
{

} // end anonymous namespace


namespace Cdmw {

namespace CCM {

namespace CIF {

//
// IDL:omg.org/CdmwCcmComponents/SessionContextExt:1.0
//  
SessionContextExt_impl::SessionContextExt_impl(
    Components::CCMContext_ptr ctx,
    CdmwLogging::LoggerFactory_ptr logger_factory)
throw (CORBA::SystemException)
: CCMContextExt_impl(ctx, logger_factory)
{
    // create logger with domain for ccmcif
    m_logger = logger_factory->get_logger(CCMLoggers::CCMCIF_LOG_DOMAIN);
    
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    m_session_context = Components::SessionContext::_narrow(ctx);

    if (CORBA::is_nil(m_session_context.in()))
    {
        // This is to avoid Assertion failure in debug mode as internal ref count
        // equals 1 (CORBA C++ mapping oblige!)
        _set_ref_count(0);
        
        CDMW_ERROR(m_logger.in(), "Context is not a SessionContext");
        CORBA::BAD_PARAM ex(OrbSupport::BAD_PARAMWrongContextType, 
                            CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

// Destructor
SessionContextExt_impl::~SessionContextExt_impl()
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
}


//
// IDL:omg.org/Components/SessionContext/get_CCM_object:1.0
//
CORBA::Object_ptr 
SessionContextExt_impl::get_CCM_object()
    throw(Components::IllegalState,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    return m_session_context->get_CCM_object();
}

} // end namespace CIF
} // end namespace CCM
} // end namespace Cdmw

