<?xml version="1.0" encoding="utf-8"?>
<!-- ===================================================================== -->
<!--
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-->
<!-- ===================================================================== -->


<xsl:stylesheet version="1.1"
   xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
   xmlns:UML="org.omg/UML1.3">



<!--
   Main - This template fetches the necessary data from inputs
   to generate a file.

   @param none
-->
<xsl:template name="ccmcif_consumer_java">
   <!--
      Parameters below are used for recursiveness.
   -->
   <xsl:param name="_index" select="1"/>
   
   <xsl:if test="boolean($lifecycleFrameworkNodes/home-impl)">
      <xsl:if test="$_index &lt;= count($lifecycleFrameworkNodes/home-impl)">
         <xsl:variable name="homeImplNode" select="$lifecycleFrameworkNodes[home-impl and position() = $_index]/home-impl"/>

         <!--
            Get the managed component.
         -->
         <xsl:variable name="homeName">
            <xsl:call-template name="getLastToken">
               <xsl:with-param name="_string" select="$homeImplNode/../@factory-idref"/>
               <xsl:with-param name="_separator" select="$xmlSep"/>
            </xsl:call-template>
         </xsl:variable>
         <xsl:variable name="managedComponentId">
            <xsl:call-template name="getManagedComponentId">
               <xsl:with-param name="_homeName" select="$homeName"/>
            </xsl:call-template>
         </xsl:variable>
         <xsl:variable name="managedComponentNode" select="key('classById', $managedComponentId)"/>

         <xsl:if test="boolean($managedComponentNode)">
            <!--
               Find all the consumers.
            -->
            <xsl:variable name="ccmConsumesId" select="key('stereotypeByName', 'CCMConsumes')/@xmi.id"/>
            <xsl:variable name="consumers" select="key('associationByStereotypeId', $ccmConsumesId)[UML:Association.connection/UML:AssociationEnd[1]/@type = $managedComponentNode/@xmi.id]"/>
         
            <xsl:for-each select="$consumers">
               <xsl:sort select="@xmi.id"/>

               <xsl:variable name="consumerTypeNode" select="key('classById', UML:Association.connection/UML:AssociationEnd[2]/@type)"/>
               <xsl:variable name="consumerTypeName" select="$consumerTypeNode/@name"/>
               <xsl:variable name="consumerClassname" select="concat($consumerTypeName, 'Consumer')"/>
               <xsl:variable name="consumerImplClassname" select="concat($consumerClassname, 'Impl')"/>
               
               <!--
                  Get the output filename.
               -->
               <xsl:variable name="filename">
                  <xsl:call-template name="buildOutputFilename">
                     <xsl:with-param name="_userDirectory" select="$homeImplNode/@directory"/>
                     <xsl:with-param name="_userFilename" select="$homeImplNode/@filename"/>
                     <xsl:with-param name="_defaultFilename" select="$consumerImplClassname"/>
                     <xsl:with-param name="_fileExtension" select="'.java'"/>
                  </xsl:call-template>
               </xsl:variable>

               <!--
                  Display a message to the user.
               -->
               <xsl:message>
                  <xsl:value-of select="concat('Generating file ', $filename)"/>
               </xsl:message>

               <!--
                  Generate file.
               -->
               <xsl:document method="text" href="{$filename}">
                  <!--
                     Insert document header.
                  -->
                  <xsl:call-template name="docHeader">
                     <xsl:with-param name="_lang" select="'java'"/>
                  </xsl:call-template>

                  <!--
                     Insert the content.
                  -->
                  <xsl:call-template name="ccmcif_consumer_java.content">
                     <xsl:with-param name="_scopedHomeImplClassname">
                        <xsl:call-template name="buildHomeImplClassname">
                           <xsl:with-param name="_homeImplNode" select="$homeImplNode"/>
                        </xsl:call-template>
                     </xsl:with-param>
                     <xsl:with-param name="_consumerNode" select="$consumerTypeNode"/>
                     <xsl:with-param name="_consumerClassname" select="$consumerClassname"/>
                     <xsl:with-param name="_consumerVarName" select="UML:Association.connection/UML:AssociationEnd[2]/@name"/>
                     <xsl:with-param name="_componentNode" select="$managedComponentNode"/>
                  </xsl:call-template>
               </xsl:document>
            </xsl:for-each>
         </xsl:if>

         <!--
            Proceed to the next home impl if any.
         -->
         <xsl:call-template name="ccmcif_consumer_java">
            <xsl:with-param name="_index" select="$_index + 1"/>
         </xsl:call-template>
      </xsl:if>
   </xsl:if>
</xsl:template> <!-- end of template ccmcif_consumer_java -->



<!--
   This template generates the content of the file.

   @param _scopedHomeImplClassname   The scoped home impl classname.
   @param _consumerNode              The consumer type node.
   @param _consumerClassname         The consumer class name.
   @param _consumerVarName           The name of the consumer as declared in IDL.
   @param _componentNode             The node of the component where the consumer is declared.
-->
<xsl:template name="ccmcif_consumer_java.content">
   <xsl:param name="_scopedHomeImplClassname"/>
   <xsl:param name="_consumerNode"/>
   <xsl:param name="_consumerClassname"/>
   <xsl:param name="_consumerVarName"/>
   <xsl:param name="_componentNode"/>

   <!--
      Miscellaneous variables.
   -->
   <xsl:variable name="consumerTypeName" select="$_consumerNode/@name"/>
   <xsl:variable name="componentName" select="$_componentNode/@name"/>
   <xsl:variable name="consumerImplClassname" select="concat($consumerTypeName, 'ConsumerImpl')"/>
   <xsl:variable name="javaScopedConsumerTypeName">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_consumerNode"/>
         <xsl:with-param name="_lang" select="'java'"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="javaConsumerScope">
      <xsl:call-template name="removeLastToken">
         <xsl:with-param name="_scopedName" select="$javaScopedConsumerTypeName"/>
         <xsl:with-param name="_sep" select="$javaSep"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="javaScopedConsumerPOAName">
      <xsl:variable name="newScopedConsumerName">
         <xsl:call-template name="prefixModuleNameOfNode">
            <xsl:with-param name="_prefix" select="'Cdmw_'"/>
            <xsl:with-param name="_idlPrefix" select="$idlPragmaPrefix"/>
            <xsl:with-param name="_node" select="$_consumerNode"/>
         </xsl:call-template>
      </xsl:variable>
      <xsl:value-of select="concat($newScopedConsumerName, 'ConsumerPOA')"/>
   </xsl:variable>
   <xsl:variable name="lowercaseComponentName" select="translate($componentName, $ucase, $lcase)"/>
   <xsl:variable name="lowercaseConsumerTypeName" select="translate($consumerTypeName, $ucase, $lcase)"/>
   <xsl:variable name="javaScopedComponentName">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_componentNode"/>
         <xsl:with-param name="_lang" select="'java'"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="javaComponentScope">
      <xsl:call-template name="removeLastToken">
         <xsl:with-param name="_scopedName" select="$javaScopedComponentName"/>
         <xsl:with-param name="_sep" select="$javaSep"/>
      </xsl:call-template>
   </xsl:variable>

   <!--
      The code generation starts here.
   -->

<xsl:variable name="package">
   <xsl:call-template name="removeLastToken">
      <xsl:with-param name="_scopedName" select="$_scopedHomeImplClassname"/>
      <xsl:with-param name="_sep" select="$javaSep"/>
   </xsl:call-template>
</xsl:variable>
package <xsl:value-of select="$package"/>;

import cdmw.orbsupport.ExceptionMinorCodes;
import cdmw.ccm.cif.LifeCycleObjectBase;
import cdmw.ccm.common.CCMLoggers;
import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;

public class <xsl:value-of select="$consumerImplClassname"/> 
    extends <xsl:value-of select="$javaScopedConsumerPOAName"/> {
    
    private org.omg.Components.ExecutorLocator executorLocator;

    private String sinkName;

    private org.omg.PortableServer.Servant component;
    
    private LifeCycleObjectBase lifecycleObjectBase;
    
    private com.thalesgroup.CdmwLogging.Logger logger;
    
    public <xsl:value-of select="$consumerImplClassname"/>(
        org.omg.Components.ExecutorLocator execLocator, 
        String name,
        cdmw.ccm.cif.Context ctx,
        org.omg.PortableServer.Servant component,
        com.thalesgroup.CdmwLogging.Logger logger) 
    {
        CDMW_LOG_FUNCTION(logger, "name="+name);
        this.lifecycleObjectBase = new LifeCycleObjectBase(ctx);
        this.executorLocator = execLocator;
        this.sinkName = name;
        this.component = component;
        this.logger = logger;
    }

    public void push_<xsl:value-of select="concat($consumerTypeName, '(', $javaScopedConsumerTypeName, ' the', $consumerTypeName, ')')"/> {
        
        CDMW_LOG_FUNCTION(logger, "");
        CDMW_DEBUG(logger,                    
            "Consumer " + sinkName + " receives an event",
            new Throwable());

        cdmw.ccm.cif.CCMObjectImpl component_impl = (cdmw.ccm.cif.CCMObjectImpl) this.component;

        component_impl.checkValidityOfOperationalRequest();

        try {
            org.omg.CORBA.Object obj = this.executorLocator.obtain_executor(this.sinkName);

            if (obj == null) {
                CDMW_DEBUG(logger, 
                    "Executor for consumer " + sinkName + " is null",
                    new Throwable());
                org.omg.CORBA.OBJECT_NOT_EXIST ex =
                    new org.omg.CORBA.OBJECT_NOT_EXIST(
                        ExceptionMinorCodes.OBJECT_NOT_EXIST,
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                throw ex;
            }

            try {
               // For monolithic component, push operation is implemented in <xsl:value-of select="$componentName"/>
               // component instead of the consumer executor for segmented component
               <xsl:value-of select="concat($javaComponentScope, $javaSep, 'CCM_', $componentName, ' ', $lowercaseComponentName, 'Exec = ', $javaComponentScope, $javaSep, 'CCM_', $componentName, 'Helper.narrow(obj);', $lf)"/>
               <xsl:value-of select="$lowercaseComponentName"/>Exec.push_<xsl:value-of select="concat($_consumerVarName, '(the', $consumerTypeName,');')"/>
            } catch(org.omg.CORBA.BAD_PARAM bp1) {
                CDMW_DEBUG(logger, 
                    "Executor for consumer " + sinkName + 
                    " is not the component executor itself",
                    new Throwable());
                try {
                   // Component may segmented
                   <xsl:value-of select="concat($javaConsumerScope, $javaSep, 'CCM_', $_consumerClassname, ' consumerExec = ', $javaConsumerScope, $javaSep, 'CCM_', $_consumerClassname, 'Helper.narrow(obj);')"/>
                   // push event to the consumer executor
                   consumerExec.push(the<xsl:value-of select="$consumerTypeName"/>);
                } catch(org.omg.CORBA.BAD_PARAM bp2) {
                    CDMW_ERROR(logger, 
                        "Executor for consumer " + sinkName + 
                        " has wrong type",
                        new Throwable());
                    org.omg.CORBA.OBJECT_NOT_EXIST ex =
                        new org.omg.CORBA.OBJECT_NOT_EXIST(
                            ExceptionMinorCodes.OBJECT_NOT_EXIST, 
                            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                    CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                    throw ex;
                }
            }

            this.executorLocator.release_executor(obj);
        } catch(org.omg.Components.CCMException ccme) {
            CDMW_ERROR(logger, 
                "CCMException received: " + ccme,
                new Throwable());
            org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                ExceptionMinorCodes.INTERNALCCMCIFError, 
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
        
    }

    public void push_event(org.omg.Components.EventBase evt)
        throws org.omg.Components.BadEventType 
    {
        CDMW_LOG_FUNCTION(logger, "");
       
       try {
            <xsl:value-of select="$javaScopedConsumerTypeName"/> infoEvt =
               (<xsl:value-of select="$javaScopedConsumerTypeName"/>) evt;

            this.push_<xsl:value-of select="$consumerTypeName"/>(infoEvt);
        } catch(ClassCastException cce) {
            CDMW_WARN(logger, 
                "EventBase with unexpected type received",
                new Throwable());
            org.omg.Components.BadEventType ex =
                new org.omg.Components.BadEventType();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }

    }

    public void push(org.omg.CORBA.Any data)
        throws org.omg.CosEventComm.Disconnected 
    {
        CDMW_LOG_FUNCTION(logger, "");
        
        /* PCR-0049
        <xsl:value-of select="$javaScopedConsumerTypeName"/> event =
            <xsl:value-of select="$javaScopedConsumerTypeName"/>Helper.read(data.create_input_stream());
        */

        try {
            // PCR-0049
            cdmw.orbsupport.Codec codec = new cdmw.orbsupport.Codec();
            com.thalesgroup.CdmwCcmCif.TimestampedEvent timestamped_event = 
                (com.thalesgroup.CdmwCcmCif.TimestampedEvent) codec.decode(data);
            <xsl:value-of select="$javaScopedConsumerTypeName"/> event = 
                (<xsl:value-of select="$javaScopedConsumerTypeName"/>) timestamped_event.event;

            java.text.DateFormat df = new java.text.SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
            java.lang.String time_received = df.format(new java.util.Date());
            //NOTE: timestamped_event.timestamp is time offset from 15-th October 1582(UTC) in millis,
            //      cdmw.ccm.cif.CCMUtils.DELTA_UTC_EPOCH is difference between UTC and Epoch in millis
            java.lang.String time_issued = 
            df.format(new java.util.Date(timestamped_event.timestamp - cdmw.ccm.cif.CCMUtils.DELTA_UTC_EPOCH));

            CCMLoggers.CDMW_DEBUG_EVENT(logger, "Event received at: " + time_received + 
                " issued at: " + time_issued);

            this.push_<xsl:value-of select="$consumerTypeName"/>(event);
        } catch (ClassCastException cce) {
            CDMW_WARN(logger, 
                "Event with wrong type received",
                new Throwable());
            cce.printStackTrace();
            org.omg.CORBA.BAD_PARAM ex = new org.omg.CORBA.BAD_PARAM();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        } catch (org.omg.CORBA.SystemException se) {
            CDMW_LOG_FUNCTION_EXCEPTION(logger, se);
            throw se;
        }

    }

    public void disconnect_push_consumer() 
    {
        CDMW_LOG_FUNCTION(logger, "");
    }
    
    // methods from LifeCycleObjectBase to delegate operations
    
    public org.omg.CosLifeCycle.LifeCycleObject copy(
        org.omg.CosLifeCycle.FactoryFinder there,
        org.omg.CosLifeCycle.NVP[] the_criteria)
        throws
            org.omg.CosLifeCycle.NoFactory,
            org.omg.CosLifeCycle.NotCopyable,
            org.omg.CosLifeCycle.InvalidCriteria,
            org.omg.CosLifeCycle.CannotMeetCriteria 
    {
        CDMW_LOG_FUNCTION(logger, "");
        return this.lifecycleObjectBase.copy(there, the_criteria);
    }
    
    public void move(
        org.omg.CosLifeCycle.FactoryFinder there,
        org.omg.CosLifeCycle.NVP[] the_criteria)
        throws
            org.omg.CosLifeCycle.NoFactory,
            org.omg.CosLifeCycle.NotMovable,
            org.omg.CosLifeCycle.InvalidCriteria,
            org.omg.CosLifeCycle.CannotMeetCriteria 
    {
        CDMW_LOG_FUNCTION(logger, "");
        this.lifecycleObjectBase.move(there, the_criteria);
    }

    public void remove() throws org.omg.CosLifeCycle.NotRemovable {
        CDMW_LOG_FUNCTION(logger, "");
        this.lifecycleObjectBase.remove();
    }

}

   <!--
      The code generation stops here.
   -->

</xsl:template> <!-- end of template ccmcif_consumer_java.content -->



</xsl:stylesheet>


