<?xml version="1.0" encoding="utf-8"?>
<!-- ===================================================================== -->
<!--
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-->
<!-- ===================================================================== -->


<xsl:stylesheet version="1.1"
   xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
   xmlns:UML="org.omg/UML1.3">



<!--
   Main - This template fetches the necessary data from inputs
   to generate a file.

   @param none
-->
<xsl:template name="session_component_java">
   <!--
      Parameters below are used for recursiveness.
   -->
   <xsl:param name="_index" select="1"/>
   
   <xsl:if test="boolean($lifecycleFrameworkNodes/home-impl) and $lifecycleFrameworkNodes/home-impl/@category = 'SESSION'">
      <xsl:if test="$_index &lt;= count($lifecycleFrameworkNodes/home-impl)">
         <xsl:variable name="homeImplNode" select="$lifecycleFrameworkNodes[home-impl and position() = $_index]/home-impl"/>

         <!--
            Check if the component is segmented
         -->
         <xsl:variable name="isSegmented">
             <xsl:choose>
                 <xsl:when test="boolean($homeImplNode/@segmented)">
                     <xsl:value-of select="$homeImplNode/@segmented"/>
                 </xsl:when>
                 <xsl:otherwise>
                     <xsl:value-of select="'yes'"/>
                 </xsl:otherwise>
             </xsl:choose>
         </xsl:variable>

         <!--
            Get the managed component.
         -->
         <xsl:variable name="homeName">
            <xsl:call-template name="getLastToken">
               <xsl:with-param name="_string" select="$homeImplNode/../@factory-idref"/>
               <xsl:with-param name="_separator" select="$xmlSep"/>
            </xsl:call-template>
         </xsl:variable>
         <xsl:variable name="managedComponentId">
            <xsl:call-template name="getManagedComponentId">
               <xsl:with-param name="_homeName" select="$homeName"/>
            </xsl:call-template>
         </xsl:variable>
         <xsl:variable name="managedComponentNode" select="key('classById', $managedComponentId)"/>

         <xsl:if test="boolean($managedComponentNode)">
            <xsl:variable name="componentImplClassname" select="concat('Session', $managedComponentNode/@name, 'Impl')"/>

            <!--
               Get the output filename.
            -->
            <xsl:variable name="filename">
               <xsl:call-template name="buildOutputFilename">
                  <xsl:with-param name="_userDirectory" select="$homeImplNode/@directory"/>
                  <xsl:with-param name="_userFilename" select="$homeImplNode/@filename"/>
                  <xsl:with-param name="_defaultFilename" select="$componentImplClassname"/>
                  <xsl:with-param name="_fileExtension" select="'.java'"/>
               </xsl:call-template>
            </xsl:variable>

            <!--
               Display a message to the user.
            -->
            <xsl:message>
               <xsl:value-of select="concat('Generating file ', $filename)"/>
            </xsl:message>

            <!--
               Generate the file.
            -->
            <xsl:document method="text" href="{$filename}">
               <!--
                  Insert the document header.
               -->
               <xsl:call-template name="docHeader">
                  <xsl:with-param name="_lang" select="'java'"/>
               </xsl:call-template>
               <!--
                  Insert the content.
               -->
               <xsl:call-template name="session_component_java.content">
                  <xsl:with-param name="_scopedHomeImplClassname">
                     <xsl:call-template name="buildHomeImplClassname">
                        <xsl:with-param name="_homeImplNode" select="$homeImplNode"/>
                     </xsl:call-template>
                  </xsl:with-param>
                  <xsl:with-param name="_componentNode" select="$managedComponentNode"/>
                  <xsl:with-param name="_isSegmented" select="$isSegmented"/>
               </xsl:call-template>
            </xsl:document>
         </xsl:if>

         <!--
            Proceed to the next home impl if any.
         -->
         <xsl:call-template name="session_component_java">
            <xsl:with-param name="_index" select="$_index + 1"/>
         </xsl:call-template>
      </xsl:if>
   </xsl:if>
</xsl:template> <!-- end of template session_component_java.content -->



<!--
   This template generates the content of the file.

   @param _scopedHomeImplClassname   The scoped home impl classname.
   @param _facetName                 The component name.
-->
<xsl:template name="session_component_java.content">
   <xsl:param name="_scopedHomeImplClassname"/>
   <xsl:param name="_componentNode"/>
   <xsl:param name="_isSegmented"/>
   
   <xsl:variable name="componentName" select="$_componentNode/@name"/>

   <!--
      Miscellaneous variables.
   -->
   <xsl:variable name="javaScopedComponentName">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_componentNode"/>
         <xsl:with-param name="_lang" select="'java'"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="javaComponentScope">
      <xsl:call-template name="removeLastToken">
         <xsl:with-param name="_scopedName" select="$javaScopedComponentName"/>
         <xsl:with-param name="_sep" select="$javaSep"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="componentImplClassname" select="concat('Session', $componentName, 'Impl')"/>
   <xsl:variable name="componentObjRefVar" select="concat($javaComponentScope, $javaSep, 'CCM_', $componentName)"/>
   <xsl:variable name="hasFacets">
      <xsl:call-template name="componentElements">
         <xsl:with-param name="_componentNode" select="$_componentNode"/>
         <xsl:with-param name="_whatToFind" select="'CCMProvides'"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="hasReceptacles">
      <xsl:call-template name="componentElements">
         <xsl:with-param name="_componentNode" select="$_componentNode"/>
         <xsl:with-param name="_whatToFind" select="'CCMUses'"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="hasEmitters">
      <xsl:call-template name="componentElements">
         <xsl:with-param name="_componentNode" select="$_componentNode"/>
         <xsl:with-param name="_whatToFind" select="'CCMEmits'"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="hasPublishers">
      <xsl:call-template name="componentElements">
         <xsl:with-param name="_componentNode" select="$_componentNode"/>
         <xsl:with-param name="_whatToFind" select="'CCMPublishes'"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="hasConsumers">
      <xsl:call-template name="componentElements">
         <xsl:with-param name="_componentNode" select="$_componentNode"/>
         <xsl:with-param name="_whatToFind" select="'CCMConsumes'"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="idlComponentScope">
      <xsl:call-template name="replaceCharsInString">
         <xsl:with-param name="_stringIn" select="$javaComponentScope"/>
         <xsl:with-param name="_charsIn" select="'$javaSep'"/>
         <xsl:with-param name="_charsOut" select="'/'"/>
      </xsl:call-template>
   </xsl:variable>
   
   <!--
      The code generation starts here.
   -->

<xsl:variable name="package">
   <xsl:call-template name="removeLastToken">
      <xsl:with-param name="_scopedName" select="$_scopedHomeImplClassname"/>
      <xsl:with-param name="_sep" select="$javaSep"/>
   </xsl:call-template>
</xsl:variable>
package <xsl:value-of select="$package"/>;

import cdmw.common.AlreadyDoneException;
import cdmw.common.InternalErrorException;
import cdmw.orbsupport.ExceptionMinorCodes;
import cdmw.ccm.cif.CCMObjectImpl;
import cdmw.ccm.common.CCMLoggers;
import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;
import static cdmw.ccm.common.CCMSystemProperties.TYPECHECKING;
import static cdmw.ccm.common.CCMSystemProperties.TYPECHECKING_TIMEOUT;

   /**
    * Session component
    */

public class Session<xsl:value-of select="$componentName"/>Impl extends CCMObjectImpl
<xsl:if test="$_isSegmented = 'yes'">
    implements <xsl:value-of select="$javaScopedComponentName"/>Operations 
</xsl:if>
<xsl:if test="$_isSegmented = 'no'">
    <xsl:variable name="newScopedComponentName">
        <xsl:call-template name="prefixModuleNameOfNode">
            <xsl:with-param name="_prefix" select="'Cdmw_'"/>
            <xsl:with-param name="_idlPrefix" select="$idlPragmaPrefix"/>
            <xsl:with-param name="_node" select="$_componentNode"/>
        </xsl:call-template>
    </xsl:variable>
    implements <xsl:value-of select="$newScopedComponentName"/>Operations 
</xsl:if>
    
{

<xsl:if test="$_isSegmented = 'yes'">
    public static final String COMP_REP_ID = "<xsl:value-of select="$_componentNode/UML:ModelElement.taggedValue/UML:TaggedValue[@tag = 'typeid']/@value"/>";
</xsl:if>
<xsl:if test="$_isSegmented = 'no'">
    <xsl:variable name="newScopedComponentName">
        <xsl:call-template name="prefixModuleNameOfNode">
            <xsl:with-param name="_prefix" select="'Cdmw_'"/>
            <xsl:with-param name="_idlPrefix" select="$idlPragmaPrefix"/>
            <xsl:with-param name="_node" select="$_componentNode"/>
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="javaIdlPrefix">
        <xsl:call-template name="reverseList">
            <xsl:with-param name="_list" select="$idlPragmaPrefix"/>
            <xsl:with-param name="_separator" select="$javaSep"/>
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="newScopedComponentNameWithoutPrefix">
        <xsl:value-of select="substring-after($newScopedComponentName, concat($javaIdlPrefix, $javaSep))"/>
    </xsl:variable>
    <xsl:variable name="newScopedComponentRepidWithoutPrefix">
        <xsl:call-template name="replaceCharsInString">
            <xsl:with-param name="_stringIn" select="$newScopedComponentNameWithoutPrefix"/>
            <xsl:with-param name="_charsIn" select="$javaSep"/>
            <xsl:with-param name="_charsOut" select="'/'"/>
        </xsl:call-template>
    </xsl:variable>
    public static final String COMP_REP_ID = "IDL:thalesgroup.com/<xsl:value-of select="concat($newScopedComponentRepidWithoutPrefix, ':1.0')"/>";
</xsl:if>

    public static final String COMP_NAME   = "<xsl:value-of select="$componentName"/>";

      <!--
         Facets.
      -->
      <xsl:call-template name="findAllJavaComponentFacetsOfNode">
         <xsl:with-param name="_componentNode" select="$_componentNode"/>
         <xsl:with-param name="_template" select="'session_component_java.content.9'"/>
         <xsl:with-param name="_strategy" select="'Session'"/>
         <xsl:with-param name="_isSegmented" select="$_isSegmented"/>
      </xsl:call-template>

      <!--
         Receptacles.
      -->
      <xsl:call-template name="findAllJavaComponentReceptaclesOfNode">
         <xsl:with-param name="_componentNode" select="$_componentNode"/>
         <xsl:with-param name="_template" select="'session_component_java.content.13'"/>
         <xsl:with-param name="_strategy" select="'Session'"/>
      </xsl:call-template>
      
      <!--
         Publishers.
      -->

      <xsl:call-template name="findAllJavaComponentPublishersOfNode">
         <xsl:with-param name="_componentNode" select="$_componentNode"/>
         <xsl:with-param name="_template" select="'session_component_java.content.15'"/>
         <xsl:with-param name="_strategy" select="'Session'"/>
      </xsl:call-template>
      
      <!--
         Emitters.
      -->
      
      <xsl:call-template name="findAllJavaComponentEmittersOfNode">
         <xsl:with-param name="_componentNode" select="$_componentNode"/>
         <xsl:with-param name="_template" select="'session_component_java.content.16'"/>
         <xsl:with-param name="_strategy" select="'Session'"/>
      </xsl:call-template>

      <!--
         Consumers.
      -->
      
      <xsl:call-template name="findAllJavaComponentConsumersOfNode">
         <xsl:with-param name="_componentNode" select="$_componentNode"/>
         <xsl:with-param name="_template" select="'session_component_java.content.19'"/>
         <xsl:with-param name="_strategy" select="'Session'"/>
         <xsl:with-param name="_isSegmented" select="$_isSegmented"/>
      </xsl:call-template>

    /**
     * If a home, when creating a component, returns an EnterpriseComponent, we map it 
     * in an ExecutorLocator for code optimisation purpose. Thus the code is the same
     * for monolithic executor and segmented executor.
     */
    private class <xsl:value-of select="$componentName"/>ExecutorLocator 
        extends org.omg.CORBA.LocalObject
        implements com.thalesgroup.CdmwCcmCif.SessionExecutorLocator {

        /**
         * indicates that the component executor inherits from the SessionComponent local interface.
         */
        private boolean isSessionComponent = false;

        /**
         * points on Component monolithic executor.
         */
        private <xsl:value-of select="$componentObjRefVar"/> compExecutor;
        
        /**
         * Logger to be used
         */
        private com.thalesgroup.CdmwLogging.Logger logger;

        public <xsl:value-of select="$componentName"/>ExecutorLocator(
            <xsl:value-of select="$componentObjRefVar"/> executor,
            com.thalesgroup.CdmwLogging.Logger logger) 
        {
            CDMW_LOG_FUNCTION(logger, "");
            this.logger = logger;
            this.compExecutor = executor;
            try  {
                org.omg.Components.SessionComponent sessionComp
                    = org.omg.Components.SessionComponentHelper.narrow(this.compExecutor);
            
                if (sessionComp != null) {
                    isSessionComponent = true;
                }
            } catch (Exception e) {}
        }

         /**
          * the cif will invoke its obtain_executor operation prior to each invocation to
          * retrieve the implementation for a port. The port name, given in the name
          * parameter, is the same as used in the components interface description 
          * in IDL, or the components name for the main executor. The obtain_executor
          * operation returns a local object reference of the expected type. The 
          * CCMException exception may be raised in case of a system error that 
          * prohibits locating the requested executor.
          */
         //
         // IDL:omg.org/Components/ExecutorLocator/obtain_executor:1.0
         //
         public org.omg.CORBA.Object obtain_executor(String name)
            throws org.omg.Components.CCMException 
         {
            CDMW_LOG_FUNCTION(logger, "name="+name);
            String execName = name;
            org.omg.CORBA.Object result = null;

            if (execName.equals("<xsl:value-of select="$componentName"/>")) {
               result = this.compExecutor;
            }
            <xsl:call-template name="findAllJavaComponentFacetsOfNode">
               <xsl:with-param name="_componentNode" select="$_componentNode"/>
               <xsl:with-param name="_template" select="'session_component_java.content.10'"/>
               <xsl:with-param name="_strategy" select="'Session'"/>
            </xsl:call-template>
            <xsl:call-template name="findAllJavaComponentConsumersOfNode">
               <xsl:with-param name="_componentNode" select="$_componentNode"/>
               <xsl:with-param name="_template" select="'session_component_java.content.20'"/>
               <xsl:with-param name="_strategy" select="'Session'"/>
            </xsl:call-template>
            else {
               org.omg.CORBA.BAD_PARAM ex = new org.omg.CORBA.BAD_PARAM(
                   ExceptionMinorCodes.BAD_PARAMInvalidExecutorName, 
                   org.omg.CORBA.CompletionStatus.COMPLETED_NO);
               CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
               throw ex;
            }

            return result;
         }

         /** 
          * The release_executor operation is called by the cif once the current
          * invocation on an executor that was obtained through the obtain_executor 
          * operation has finished. The locator can thus release any resources that
          * were acquired as part of the obtain_executor operation.
          */
         //
         // IDL:omg.org/Components/ExecutorLocator/release_executor:1.0
         //
         public void release_executor(org.omg.CORBA.Object exc) 
            throws org.omg.Components.CCMException 
         {
            CDMW_LOG_FUNCTION(logger, "");
         }

         /** 
          * The configuration_complete operation is called to propagate the
          * configuration_complete operation on the CCMObject interface to the
          * component implementation.
          */
         //
         // IDL:omg.org/Components/ExecutorLocator/configuration_complete:1.0
         //
         public void configuration_complete() 
            throws org.omg.Components.InvalidConfiguration {
            CDMW_LOG_FUNCTION(logger, "");
            // No equivalent operation for monolithic executor
         }

         //
         // IDL:omg.org/Components/SessionComponent/set_session_context:1.0
         //
         public void set_session_context(org.omg.Components.SessionContext ctx)
            throws org.omg.Components.CCMException {
            CDMW_LOG_FUNCTION(logger, "");
            // check if component implements the SessionComponent local interface
            if (isSessionComponent) {
               CDMW_INFO(logger, "Call set_session_context()", new Throwable());
               org.omg.Components.SessionComponent sessionComp 
                  = org.omg.Components.SessionComponentHelper.narrow(this.compExecutor);
               sessionComp.set_session_context(ctx);
            }
         }

         /** 
          * The ccm_activate operation is called by the container to notify a 
          * session component that it has been made active. The component instance
          * should perform any initialization required prior to operation invocation.
          * The component may raise the CCMException with the SYSTEM_ERROR minor code
          * to indicate a failure caused by a system level error.
          */
         //
         // IDL:omg.org/Components/SessionComponent/ccm_activate:1.0
         //
         public void ccm_activate() throws org.omg.Components.CCMException {
            CDMW_LOG_FUNCTION(logger, "");
            // check if component implements the SessionComponent local interface
            if (isSessionComponent) {
               CDMW_INFO(logger, "Call ccm_activate()", new Throwable());
               org.omg.Components.SessionComponent sessionComp 
                  = org.omg.Components.SessionComponentHelper.narrow(this.compExecutor);
               sessionComp.ccm_activate();
            }
         }

         /** 
          * The ccm_passivate operation is called by the container to notify a session
          * component that it has been made inactive. The component instance should
          * release any resources it acquired at activation time.The component may 
          * raise the CCMException with the SYSTEM_ERROR minor code to indicate a 
          * failure caused by a system level error.
          */
         //
         // IDL:omg.org/Components/SessionComponent/ccm_passivate:1.0
         //
         public void ccm_passivate() throws org.omg.Components.CCMException {
            CDMW_LOG_FUNCTION(logger, "");
            // check if component implements the SessionComponent local interface
            if (isSessionComponent) {
               CDMW_INFO(logger, "Call ccm_passivate()", new Throwable());
               org.omg.Components.SessionComponent sessionComp 
                  = org.omg.Components.SessionComponentHelper.narrow(this.compExecutor);
               sessionComp.ccm_passivate();
            }
         }

         /**
          * The ccm_remove operation is called by the container when the servant is
          * about to be destroyed. It informs the component that it is about to be
          * destroyed.The component may raise the CCMException with the SYSTEM_ERROR
          * minor code to indicate a failure caused by a system level error.
          */
         //
         // IDL:omg.org/Components/SessionComponent/ccm_remove:1.0
         //
         public void ccm_remove() throws org.omg.Components.CCMException {
            CDMW_LOG_FUNCTION(logger, "");
            // check if component implements the SessionComponent local interface
            if (isSessionComponent) {
               CDMW_INFO(logger, "Call ccm_remove()", new Throwable());
               org.omg.Components.SessionComponent sessionComp 
                  = org.omg.Components.SessionComponentHelper.narrow(this.compExecutor);
               sessionComp.ccm_remove();
            }
         }
     }

    /**
     * Attributes
     */
   
    /**
     * If a home, in creating a component, returns an ExecutorLocator, the cif
     * will invoke its obtain_executor operation prior to each invocation to
     * retrieve the implementation for a port. If the home returns an
     * EnterpriseComponent, we encapsulate it in an executor locator in order to
     * use same code.
     *
     */
   
    /**
     * <xsl:value-of select="$componentName"/> executor locator used to get component exec and facet exec
     */
    private org.omg.Components.ExecutorLocator executorLocator;
    
    /**
     * executor locator must a session component to be advise 
     * of its activation, passivation and removal
     */
    private org.omg.Components.SessionComponent sessionLocator;

    // set to true when component is activated
    boolean isActivated;
    
   <xsl:variable name="contextExecutorClassName">
      <xsl:variable name="tmpClassName">
         <xsl:call-template name="prefixClassName">
            <xsl:with-param name="_prefix" select="'CCM_'"/>
            <xsl:with-param name="_scopedName">
               <xsl:call-template name="prefixModuleNameOfNode">
                  <xsl:with-param name="_prefix" select="'Cdmw_'"/>
                  <xsl:with-param name="_idlPrefix" select="$idlPragmaPrefix"/>
                  <xsl:with-param name="_node" select="$_componentNode"/>
               </xsl:call-template>
            </xsl:with-param>
         </xsl:call-template>
      </xsl:variable>
      <xsl:value-of select="concat($tmpClassName, '_SessionContext')"/>
    </xsl:variable>
    /**
     * <xsl:value-of select="$componentName"/> context executor
     */
    private <xsl:value-of select="$contextExecutorClassName"/> contextExecutor;
   
    /**
     * Logger for logging
     */
    private com.thalesgroup.CdmwLogging.Logger logger;
     
    /**
     * Indicates if current component has been created via a factory operation
     */
    private boolean isCreatedByFactoryOperation;
   
   <xsl:call-template name="findAllJavaInterfaceReadWriteAttributes">
      <xsl:with-param name="_interfaceNode" select="$_componentNode"/>
      <xsl:with-param name="_type" select="'Component'"/>
      <xsl:with-param name="_template" select="'session_component_java.content.4'"/>
      <xsl:with-param name="_strategy" select="'Session'"/>
   </xsl:call-template>

    /**
     * Constructor
     */
    public Session<xsl:value-of select="$componentName"/>Impl(
        String compOid,
        cdmw.ccm.cif.Context ctx,
        org.omg.CORBA.Object compRef,
        org.omg.Components.EnterpriseComponent executor,
        com.thalesgroup.CdmwLifeCycle.ObjectRegistration objectRegistration,
        com.thalesgroup.CdmwLogging.LoggerFactory loggerFactory,
        boolean isCreatedByFactoryOperation)
        throws org.omg.Components.CCMException 
    {

    <xsl:if test="$_isSegmented = 'yes'">
        super(compOid, ctx, COMP_REP_ID, compRef, objectRegistration, loggerFactory);
    </xsl:if>
    <xsl:if test="$_isSegmented = 'no'">
        super(compOid, ctx, COMP_REP_ID, compRef, objectRegistration, loggerFactory, true);
    </xsl:if>
        
        this.logger = loggerFactory.get_logger(
            cdmw.ccm.common.CCMLoggers.CCMGENERATED_LOG_DOMAIN);
        
        CDMW_LOG_FUNCTION(logger, "");
        
        this.isActivated = false;
        this.isCreatedByFactoryOperation = isCreatedByFactoryOperation;

        // Check if the home returns an executor locator
        try {
            this.executorLocator = org.omg.Components.ExecutorLocatorHelper.narrow(executor);
            this.sessionLocator = org.omg.Components.SessionComponentHelper.narrow(this.executorLocator);

            if (this.sessionLocator == null) {
                System.err.println("Executor Locator should be a session component!");
                throw new org.omg.CORBA.BAD_PARAM(
                  ExceptionMinorCodes.BAD_PARAMInvalidComponentExecutor,
                  org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }
        } catch(org.omg.CORBA.BAD_PARAM bp) {
            // executor is not an executor locator
            // map monolithic executor to executor locator
            <xsl:value-of select="$componentObjRefVar"/> compExec = <xsl:value-of select="concat($javaComponentScope,    $javaSep, 'CCM_',  $componentName)"/>Helper.narrow(executor);
        
            if (compExec == null) {
                CDMW_ERROR(logger, "Invalid Component Executor", new Throwable());
                org.omg.CORBA.BAD_PARAM ex = new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMInvalidComponentExecutor,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            }
            this.executorLocator = 
                new <xsl:value-of select="$componentName"/>ExecutorLocator(compExec, this.logger);
            this.sessionLocator = 
                org.omg.Components.SessionComponentHelper.narrow(this.executorLocator);
         
            if (this.sessionLocator == null) {
                CDMW_ERROR(logger, "Executor Locator should be a session component!", new Throwable());
                org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                    ExceptionMinorCodes.INTERNALInvalidExecutor,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            }            
        }

        this.contextExecutor = new CCM_<xsl:value-of select="$componentName"/>_SessionContextImpl(ctx.get_ccm_context(), this, loggerFactory);
        this.sessionLocator.set_session_context(this.contextExecutor);

<![CDATA[      
      //
      // QoS services installation
      //
      
      // get COMPONENT_NAME from ConfigValues
      String componentName = null;
      try {
          org.omg.Components.ConfigValue[] configValues = 
              ctx.getConfigValues(compOid);
          for(org.omg.Components.ConfigValue c : configValues) {
              if(com.thalesgroup.CdmwDeployment.COMPONENT_NAME.value.equals(c.name)) 
              {
                  // extract COMPONENT_NAME
                  componentName = c.value.extract_string();
              }
          }
          
      } catch (cdmw.common.NotFoundException e) {
          // No config values for this component.
          // Cannot get COMPONENT_NAME nor install services
          CDMW_WARN(logger, "No ConfigValues set for this component.", new Throwable());
          // continue anyway
      }

      // install ManagedComponentContext service
      if (componentName != null) {
          try {
              // get ManagedComponentContextFactory object
              org.omg.CORBA.Object obj = 
                  this.contextExecutor.resolve_service_reference(
                      cdmw.platformmngt.PlatformInterface.MANAGED_COMPONENT_CONTEXT_FACTORY_NAME);
              com.thalesgroup.CdmwPlatformMngt.ManagedComponentContextFactory mcContextFactory =
                  com.thalesgroup.CdmwPlatformMngt.ManagedComponentContextFactoryHelper.narrow(obj);
                  
              // get ComponentContext object
              com.thalesgroup.CdmwPlatformMngt.ManagedElementContexts.ComponentContext mcContext =
                  mcContextFactory.find_managed_component_context(componentName);
                  
              // install service
              this.contextExecutor.install_service_reference(
                  "ManagedComponentContext",
                  mcContext);
                  
          } catch (org.omg.Components.CCMException e) {
              if (e.reason == org.omg.Components.CCMExceptionReason.OBJECT_NOT_FOUND) { 
                  // Cannot find ManagedComponentContextFactory object
                  CDMW_WARN(logger, 
                      "Cannot find service ManagedComponentContextFactory" 
                      +" Cannot install ManagedComponentContext service.",
                      new Throwable());
                  // continue anyway
              } else if (e.reason == org.omg.Components.CCMExceptionReason.SERVICE_INSTALLATION_ERROR) {
                  // Cannot install ManagedComponentContext service
                  CDMW_WARN(logger, 
                      "Cannot install ManagedComponentContext service: already installed.",
                      new Throwable());
                  // continue anyway
              } else {
                  // Cannot find ManagedComponentContextFactory object
                  CDMW_WARN(logger, 
                      "Cannot install ManagedComponentContext service for unknown reason",
                      new Throwable());
                  // continue anyway
              }
          } catch (com.thalesgroup.CdmwPlatformMngt.InvalidName e) {
              // Cannot find ManagedComponentContext object
              CDMW_WARN(logger, 
                  "Cannot find service ManagedComponentContext for component" 
                  +componentName,
                  new Throwable());
              // continue anyway
          }
      }
      else 
      {
           CDMW_WARN(logger, 
               "No COMPONENT_NAME ConfigValue."
               +" Cannot install ManagedComponentContext service",
               new Throwable());
      }

      
      // install LoggerFactory service
      try {
          // install service
          this.contextExecutor.install_service_reference(
              "LoggerFactory",
              loggerFactory);
              
      } catch (org.omg.Components.CCMException e) {
          if (e.reason == org.omg.Components.CCMExceptionReason.SERVICE_INSTALLATION_ERROR) {
              // Cannot install LoggerFactory service
              CDMW_WARN(logger, 
                  "Cannot install LoggerFactory service: already installed.",
                  new Throwable());
              // continue anyway
          } else {
              // Cannot find LoggerFactory object
              CDMW_WARN(logger, 
                  "Cannot install LoggerFactory service for unknown reason",
                  new Throwable());
              // continue anyway
          }
      }
]]>      

      <xsl:call-template name="findAllJavaComponentFacetsOfNode">
         <xsl:with-param name="_componentNode" select="$_componentNode"/>
         <xsl:with-param name="_template" select="'session_component_java.content.11'"/>
         <xsl:with-param name="_strategy" select="'Session'"/>
      </xsl:call-template>

      <xsl:if test="$hasReceptacles = 'true'">
         try {
            <xsl:call-template name="findAllJavaComponentReceptaclesOfNode">
               <xsl:with-param name="_componentNode" select="$_componentNode"/>
               <xsl:with-param name="_template" select="'session_component_java.content.14'"/>
               <xsl:with-param name="_strategy" select="'Session'"/>
            </xsl:call-template>
         } catch (AlreadyDoneException ade) {
            CDMW_ERROR(logger, "Receptacle already declared!", new Throwable());
            // Don't propagate while its probably a code generation error!
         <!-- //TODO: remove this catch statement if not needed
         } catch (InternalErrorException iee) {
            System.err.println("Internal exception raised when declaring receptacle!");
            throw new org.omg.CORBA.INTERNAL(
               ExceptionMinorCodes.INTERNAL, 
               org.omg.CORBA.CompletionStatus.COMPLETED_NO);
         -->
         } catch (org.omg.CORBA.SystemException se) {
            CDMW_ERROR(logger, 
                "System exception raised when declaring receptacle!" + se.toString(),
                new Throwable());
            org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
               ExceptionMinorCodes.INTERNAL, 
               org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
         }
      </xsl:if>

      <xsl:if test="$hasEmitters = 'true' or $hasPublishers = 'true'">
         try {
            <xsl:if test="$hasPublishers = 'true'">
               <xsl:call-template name="findAllJavaComponentPublishersOfNode">
                  <xsl:with-param name="_componentNode" select="$_componentNode"/>
                  <xsl:with-param name="_template" select="'session_component_java.content.17'"/>
                  <xsl:with-param name="_strategy" select="'Session'"/>
               </xsl:call-template>
            </xsl:if>
            <xsl:if test="$hasEmitters = 'true'">
               <xsl:call-template name="findAllJavaComponentEmittersOfNode">
                  <xsl:with-param name="_componentNode" select="$_componentNode"/>
                  <xsl:with-param name="_template" select="'session_component_java.content.18'"/>
                  <xsl:with-param name="_strategy" select="'Session'"/>
               </xsl:call-template>
            </xsl:if>
         } catch (AlreadyDoneException ade) {
            CDMW_ERROR(logger, "Event source already declared!", new Throwable());
            // Don't propagate while its probably a code generation error!
         } catch (org.omg.CORBA.SystemException se) {
            CDMW_ERROR(logger, 
                "System exception raised when declaring event source: " + se,
                new Throwable());
            org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
               ExceptionMinorCodes.INTERNAL, 
               org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
         }
      </xsl:if>

      <xsl:if test="$hasConsumers = 'true'">
         try {
            <xsl:call-template name="findAllJavaComponentConsumersOfNode">
               <xsl:with-param name="_componentNode" select="$_componentNode"/>
               <xsl:with-param name="_template" select="'session_component_java.content.21'"/>
               <xsl:with-param name="_strategy" select="'Session'"/>
            </xsl:call-template>
         } catch (AlreadyDoneException ade) {
            CDMW_ERROR(logger, "Consumer already declared!", new Throwable());
            // Don't propagate while its probably a code generation error!
         } catch (InternalErrorException iee) {
            CDMW_ERROR(logger, "Internal exception raised when declaring consumer!", new Throwable());
            throw new org.omg.CORBA.INTERNAL(
               ExceptionMinorCodes.INTERNAL, 
               org.omg.CORBA.CompletionStatus.COMPLETED_NO);
         } catch (org.omg.CORBA.SystemException se) {
            CDMW_ERROR(logger, "System exception raised when declaring consumer: " + se, new Throwable());
            org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
               ExceptionMinorCodes.INTERNAL, 
               org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
         }
      </xsl:if>

      CDMW_LOG_FUNCTION_RETURN(logger, "");
    }

    /**
     * Component attributes
     */

   <xsl:call-template name="findAllJavaInterfaceReadWriteAttributes">
      <xsl:with-param name="_interfaceNode" select="$_componentNode"/>
      <xsl:with-param name="_type" select="'Component'"/>
      <xsl:with-param name="_template" select="'session_component_java.content.3'"/>
      <xsl:with-param name="_strategy" select="'Session'"/>
   </xsl:call-template>

    /**
     * Supported interfaces
     */

   <!--
      Declare supported interfaces operations.
      NOTA: This should also include inherited operations.
   -->
   <xsl:variable name="allBaseInterfaces">
       <xsl:call-template name="getAllBaseClasses">
           <xsl:with-param name="_classNode" select="$_componentNode"/>
       </xsl:call-template>
   </xsl:variable>

   <xsl:if test="$CdmwDebugOn" >
   <xsl:message>
       <xsl:value-of select="concat(' -****- Found ', count($allBaseInterfaces/*), ' interfaces after filtering.')"/> 
   </xsl:message>
   </xsl:if>


   <xsl:for-each select="$allBaseInterfaces/node()[string(@xmi.id)]">
       <xsl:sort select="@xmi.id"/>
       <xsl:variable name="_xmi_id" select="./@xmi.id"/>
       <!--
       NOTA: Note the following trick to get back to the initial document context
             to be able to access operation definitions ... etc. because the result
             tree ($allBaseInterfaces) is a new xml document a only contains a copy
             of a subset of the original tree.
       -->
       <xsl:for-each select="$_componentNode">
           <xsl:variable name="_interface" select="key('classById', $_xmi_id)"/>
           <xsl:call-template name="session_component_java.SupportedInterfaces.operations.definition.own">
               <xsl:with-param name="_interfaceNode" select="$_interface" />
               <xsl:with-param name="_componentObjRefVar" select="$componentObjRefVar" />
           </xsl:call-template>
       </xsl:for-each>
   </xsl:for-each>


   /**
    * Equivalent IDL methods
    */

   // Facets

   <xsl:call-template name="findAllJavaComponentFacetsOfNode">
      <xsl:with-param name="_componentNode" select="$_componentNode"/>
      <xsl:with-param name="_template" select="'session_component_java.content.1'"/>
      <xsl:with-param name="_strategy" select="'Session'"/>
      <xsl:with-param name="_isSegmented" select="$_isSegmented"/>
   </xsl:call-template>

   // Receptacles

   <xsl:call-template name="findAllJavaComponentReceptaclesOfNode">
      <xsl:with-param name="_componentNode" select="$_componentNode"/>
      <xsl:with-param name="_template" select="'session_component_java.content.2'"/>
      <xsl:with-param name="_strategy" select="'Session'"/>
   </xsl:call-template>

   // Consumers
   <xsl:if test="$_isSegmented = 'no'">
       <xsl:if test="$hasConsumers = 'true'">
    public void push_event(org.omg.Components.EventBase evt)
        throws org.omg.Components.BadEventType 
    {
        CDMW_LOG_FUNCTION(logger, "");
        checkIsRemoved();

        boolean bad_evt_type_found = true;
        
        <xsl:call-template name="findAllJavaComponentConsumersOfNode">
           <xsl:with-param name="_componentNode" select="$_componentNode"/>
           <xsl:with-param name="_template" select="'session_component_java.content.25'"/>
        </xsl:call-template>
        
        if (bad_evt_type_found)
        {
            CDMW_WARN(logger,
                      "EventBase with unexpected type received",
                      new Throwable());
            org.omg.Components.BadEventType ex =
                new org.omg.Components.BadEventType();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }    
    }
        
    
    public void push(org.omg.CORBA.Any data)
        throws org.omg.CosEventComm.Disconnected
    {
        CDMW_LOG_FUNCTION(logger, "");
        checkIsRemoved();

        boolean bad_evt_type_found = true;
        
        <xsl:call-template name="findAllJavaComponentConsumersOfNode">
           <xsl:with-param name="_componentNode" select="$_componentNode"/>
           <xsl:with-param name="_template" select="'session_component_java.content.26'"/>
        </xsl:call-template>
        
        if (bad_evt_type_found)
        {
            org.omg.CORBA.TypeCode tc = data.type();
            String typeId = null;
            try {
                typeId = tc.id();
            } catch (org.omg.CORBA.TypeCodePackage.BadKind e) {
                typeId = "unknown type !! (TypeCode::BadKind exception)";
            }
            CDMW_WARN(logger,
                      "Event with unexpected type received: " + typeId,
                      new Throwable());
            org.omg.CORBA.BAD_PARAM ex =
                new org.omg.CORBA.BAD_PARAM();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }    
    }
    
    public void disconnect_push_consumer()
    {
        CDMW_LOG_FUNCTION(logger, "");
        checkIsRemoved();
    }
       </xsl:if>
   </xsl:if>

   <xsl:call-template name="findAllJavaComponentConsumersOfNode">
      <xsl:with-param name="_componentNode" select="$_componentNode"/>
      <xsl:with-param name="_template" select="'session_component_java.content.5'"/>
      <xsl:with-param name="_strategy" select="'Session'"/>
      <xsl:with-param name="_isSegmented" select="$_isSegmented"/>
   </xsl:call-template>

   // Publishers

   <xsl:call-template name="findAllJavaComponentPublishersOfNode">
      <xsl:with-param name="_componentNode" select="$_componentNode"/>
      <xsl:with-param name="_template" select="'session_component_java.content.6'"/>
      <xsl:with-param name="_strategy" select="'Session'"/>
   </xsl:call-template>
   
   // Emitters

   <xsl:call-template name="findAllJavaComponentEmittersOfNode">
      <xsl:with-param name="_componentNode" select="$_componentNode"/>
      <xsl:with-param name="_template" select="'session_component_java.content.7'"/>
      <xsl:with-param name="_strategy" select="'Session'"/>
   </xsl:call-template>

   /**
    * Inherited CCMObject interface implemented by CCMObjectImpl
    */

   public void configuration_complete()
      throws org.omg.Components.InvalidConfiguration {
      
      CDMW_LOG_FUNCTION(logger, "");
      
      if (!is_configured()) {
         if (!this.isCreatedByFactoryOperation) {
            CDMW_DEBUG(logger, "Check if configuration is complete", new Throwable());
            <xsl:call-template name="findAllJavaInterfaceReadWriteAttributes">
               <xsl:with-param name="_interfaceNode" select="$_componentNode"/>
               <xsl:with-param name="_type" select="'Component'"/>
               <xsl:with-param name="_template" select="'session_component_java.content.8'"/>
               <xsl:with-param name="_strategy" select="'Session'"/>
            </xsl:call-template>
         }
         
         super.configuration_complete();
        
         try {
             this.executorLocator.configuration_complete();
             
            // change state
            this.state = this.state.configuration_complete(this);
             
             // no error occurs component is activated
             this.isActivated = true;
         } catch(org.omg.Components.CCMException ccme) {
             CDMW_DEBUG(logger, 
                "CCMException raised, reason: " + ccme.toString(),
                new Throwable());
             org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                 ExceptionMinorCodes.INTERNAL,
                 org.omg.CORBA.CompletionStatus.COMPLETED_NO);
             CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
             throw ex;
         }

      }
      
   }

   /**
    * Inherited Events interface implemented by CCMObjectImpl
    */

   <![CDATA[
      /**
       * This returns the servant for the facet specified by facetName.
       * This operation is implemented by the well typed component servant.
       *
       * @param facetName The facet name
       *
       * @return the facet servant.
       */
   ]]>
   <xsl:choose>
      <xsl:when test="$hasFacets = 'true' or $hasConsumers = 'true'">
         public org.omg.PortableServer.Servant get_facet_servant(String facetName) {
            CDMW_LOG_FUNCTION(logger, "facetName="+facetName);
         
            String facet = facetName;
            org.omg.PortableServer.Servant facetServant = null;
          
            <xsl:if test="$hasFacets = 'true'">
               <xsl:call-template name="findAllJavaComponentFacetsOfNode">
                  <xsl:with-param name="_componentNode" select="$_componentNode"/>
                  <xsl:with-param name="_template" select="'session_component_java.content.12'"/>
                  <xsl:with-param name="_strategy" select="'Session'"/>
               </xsl:call-template>
            </xsl:if>
            <xsl:if test="$hasConsumers = 'true'">
               <xsl:call-template name="findAllJavaComponentConsumersOfNode">
                  <xsl:with-param name="_componentNode" select="$_componentNode"/>
                  <xsl:with-param name="_template" select="'session_component_java.content.22'"/>
                  <xsl:with-param name="_strategy" select="'Session'"/>
               </xsl:call-template>
            </xsl:if>
            {
               // No facet/consumer with this name define for that component
               CDMW_ERROR(logger, 
                  "No '" + facetName + 
                  "' facet or consumer defined for this component",
                  new Throwable());
               org.omg.CORBA.OBJECT_NOT_EXIST ex = new org.omg.CORBA.OBJECT_NOT_EXIST(
                  ExceptionMinorCodes.OBJECT_NOT_EXIST,
                  org.omg.CORBA.CompletionStatus.COMPLETED_NO);
               CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
               throw ex;
            }
          
            return facetServant;
         }
      </xsl:when>
      <xsl:otherwise>
         public org.omg.PortableServer.Servant get_facet_servant(String facetName) {
            CDMW_LOG_FUNCTION(logger, "facetName="+facetName);
            // No facet with this name define for that component
            CDMW_ERROR(logger, 
               "No '" + facetName + 
               "' facet or consumer defined for this component",
               new Throwable());
            org.omg.CORBA.OBJECT_NOT_EXIST ex = new org.omg.CORBA.OBJECT_NOT_EXIST(
               ExceptionMinorCodes.OBJECT_NOT_EXIST,
               org.omg.CORBA.CompletionStatus.COMPLETED_NO); 
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
         }
      </xsl:otherwise>
   </xsl:choose>

   
   <![CDATA[
    /**
     * In case of a session component, this operation will call 
     * it's executor's ccm_activate operation.
     *
     *@exception org.omg.Components.CCMException Any CCMException thrown by the
     * executor's ccm_activate operation.
     *@exception org.omg.CORBA.SystemException Any SystemException thrown by the
     * executor's ccm_activate operation.
     */ 
   ]]>
    public void ccm_activate() 
        throws org.omg.Components.CCMException, 
               org.omg.CORBA.SystemException
    {
        // this is a session component, call ccm_activate
        this.sessionLocator.ccm_activate();
    }

   <![CDATA[
    /**
     * In case of a session component, this operation will call 
     * it's executor's ccm_passivate operation.
     *
     *@exception org.omg.Components.CCMException Any CCMException thrown by the
     * executor's ccm_passivate operation.
     *@exception org.omg.CORBA.SystemException Any SystemException thrown by the
     * executor's ccm_passivate operation.
     */ 
   ]]>
    public void ccm_passivate() 
        throws org.omg.Components.CCMException, 
               org.omg.CORBA.SystemException
    {
        // this is a session component, call ccm_passivate
        this.sessionLocator.ccm_passivate();
    }

   <![CDATA[
    /**
     * In case of a session component, this operation will call 
     * it's executor's ccm_remove operation.
     *
     *@exception org.omg.Components.CCMException Any CCMException thrown by the
     * executor's ccm_remove operation.
     *@exception org.omg.CORBA.SystemException Any SystemException thrown by the
     * executor's ccm_remove operation.
     */ 
   ]]>
    public void ccm_remove() 
        throws org.omg.Components.CCMException, 
               org.omg.CORBA.SystemException
    {
        // this is a session component, call ccm_remove
        this.sessionLocator.ccm_remove();
    }

    // return true if configuration complete has already been called
    protected boolean is_configured() {
        return this.state == 
            cdmw.ccm.cif.DefaultCCMObjectStates.CCM_OBJECT_STATE_FULLY_ACTIVATED;
    }   
    
   
   // Return the component executor
   protected <xsl:value-of select="$javaComponentScope"/>.CCM_<xsl:value-of select="$componentName"/>
   obtain_component_executor() {
      CDMW_LOG_FUNCTION(logger, "");
      org.omg.CORBA.Object obj = null;
      try {
         obj = this.executorLocator.obtain_executor(COMP_NAME);
         
         if (obj == null) {
            CDMW_ERROR(logger, "Component executor is nil", new Throwable());
            org.omg.CORBA.OBJECT_NOT_EXIST ex = new org.omg.CORBA.OBJECT_NOT_EXIST(
                ExceptionMinorCodes.OBJECT_NOT_EXISTInvalidExecutor,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
         }

         <xsl:value-of select="$componentObjRefVar"/> compExec = <xsl:value-of select="concat($javaComponentScope, $javaSep, 'CCM_', $componentName)"/>Helper.narrow(obj);

         if (compExec == null) {
            CDMW_ERROR(logger, 
               "Component executor is not a <xsl:value-of select="concat($javaComponentScope, $javaSep, 'CCM_', $componentName)"/>",
               new Throwable());
            org.omg.CORBA.OBJECT_NOT_EXIST ex = new org.omg.CORBA.OBJECT_NOT_EXIST(
               ExceptionMinorCodes.OBJECT_NOT_EXISTInvalidExecutor, 
               org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
         }

         return compExec;
      } catch(org.omg.Components.CCMException ccme) {
         CDMW_ERROR(logger, 
             "CCMException raised, reason: " + ccme.toString(), 
             new Throwable());
         org.omg.CORBA.OBJECT_NOT_EXIST ex = new org.omg.CORBA.OBJECT_NOT_EXIST(
            ExceptionMinorCodes.OBJECT_NOT_EXISTInvalidExecutor,
            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
         throw ex;
      } catch(org.omg.CORBA.BAD_PARAM bp) {
         CDMW_ERROR(logger, 
            "CORBA::BAD_PARAM obtaining component executor", 
            new Throwable());
         org.omg.CORBA.OBJECT_NOT_EXIST ex = new org.omg.CORBA.OBJECT_NOT_EXIST(
            ExceptionMinorCodes.OBJECT_NOT_EXISTInvalidExecutor,
            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
         throw ex;
      }

   }

   // Change state to REMOVED; this will call ccm_passivate and ccm_remove on executor
   protected void advise_executor_of_remove_component() {
      CDMW_LOG_FUNCTION(logger, "");

      try {
         this.state = this.state.remove(this);
      } catch(org.omg.Components.CCMException ccme) {
         CDMW_ERROR(logger, "CCMException raised, reason: " + ccme, new Throwable());
         org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
            ExceptionMinorCodes.INTERNAL,
            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
         throw ex;
      }
   }

}

   <!--
      The code generation stops here.
   -->

</xsl:template> <!-- end of template session_component_java.content -->



<!--
-->
<xsl:template name="session_component_java.content.1">
   <xsl:param name="_facetTypeNode"/>
   <xsl:param name="_facetName"/>
   <xsl:param name="_isSegmented"/>

   <xsl:variable name="facetTypeName" select="$_facetTypeNode/@name"/>
   
   <xsl:variable name="scopedReturnType">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_facetTypeNode"/>
         <xsl:with-param name="_lang" select="'java'"/>
      </xsl:call-template>
   </xsl:variable>
   
   <xsl:variable name="javaFacetScope">
      <xsl:call-template name="removeLastToken">
         <xsl:with-param name="_scopedName" select="$scopedReturnType"/>
         <xsl:with-param name="_sep" select="$javaSep"/>
      </xsl:call-template>
   </xsl:variable>
   
   <xsl:value-of select="concat($lf, 'public ', $scopedReturnType, ' provide_', $_facetName, '()', $lf, '{', $lf)"/>
      CDMW_LOG_FUNCTION(logger, "");
      org.omg.CORBA.Object obj;

      try {
         obj = super.provide_facet("<xsl:value-of select="$_facetName"/>");
      } catch (org.omg.Components.InvalidName in) {
         CDMW_ERROR(logger, 
            "'<xsl:value-of select="$_facetName"/>' is not a valid facet name", 
            new Throwable());
         org.omg.CORBA.OBJECT_NOT_EXIST ex = new org.omg.CORBA.OBJECT_NOT_EXIST(
            ExceptionMinorCodes.OBJECT_NOT_EXIST,
            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
         throw ex;
      }

      if (obj == null) {
         CDMW_ERROR(logger, 
            "Facet <xsl:value-of select="$_facetName"/> is null", 
            new Throwable());
         org.omg.CORBA.OBJECT_NOT_EXIST ex = new org.omg.CORBA.OBJECT_NOT_EXIST(
            ExceptionMinorCodes.OBJECT_NOT_EXIST, 
            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
         throw ex;
      }

      <xsl:value-of select="$scopedReturnType"/> facetRef = <xsl:value-of select="$scopedReturnType"/>Helper.narrow(obj);
       
      if (facetRef == null) {
         CDMW_ERROR(logger, 
            "Facet <xsl:value-of select="$_facetName"/> is not a <xsl:value-of select="$scopedReturnType"/>", 
            new Throwable());
         org.omg.CORBA.OBJECT_NOT_EXIST ex = new org.omg.CORBA.OBJECT_NOT_EXIST(
            ExceptionMinorCodes.OBJECT_NOT_EXIST, 
            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
         throw ex;
      }
       
      return facetRef;
   }
   
   <xsl:if test="$_isSegmented = 'no'">
      <!--
               facet operation code for monolithic component
       -->
      <xsl:call-template name="session_component_java.Monolithic.Facets.Operations.all">
          <xsl:with-param name="_interfaceNode" select="$_facetTypeNode"/>
          <xsl:with-param name="_facetName" select="$_facetName"/>
          <xsl:with-param name="_facetTypeName" select="$facetTypeName"/>
          <xsl:with-param name="_javaFacetScope" select="$javaFacetScope"/>
      </xsl:call-template> 

   </xsl:if>   
</xsl:template> <!-- end of template session_component_java.content.1 -->



<!--
-->
<xsl:template name="session_component_java.content.2">
   <xsl:param name="_receptacleTypeNode"/>
   <xsl:param name="_receptacleName"/>
   
   <xsl:variable name="scopedReturnType">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_receptacleTypeNode"/>
         <xsl:with-param name="_lang" select="'java'"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="uppercaseReceptacleName" select="translate($_receptacleName, $lcase, $ucase)"/>
   
   <xsl:value-of select="concat($lf, 'public void connect_',  $_receptacleName, '(', $scopedReturnType, ' conxn)', 
      $lf, 'throws org.omg.Components.AlreadyConnected, org.omg.Components.InvalidConnection {', $lf)"/>
      CDMW_LOG_FUNCTION(logger, "");
      try {
          super.connect(RECEP_<xsl:value-of select="$uppercaseReceptacleName"/>_NAME, conxn);
      } catch(org.omg.Components.InvalidName in) {
         CDMW_ERROR(logger, 
            "Invalid name for receptacle connection!", 
            new Throwable());
         org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
             ExceptionMinorCodes.INTERNALCCMCIFError,
             org.omg.CORBA.CompletionStatus.COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
         throw ex;
      } catch(org.omg.Components.ExceededConnectionLimit ecl) {
         CDMW_ERROR(logger, 
            "Exceeded connection limit reached for receptacle connection!", 
            new Throwable());
         org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
             ExceptionMinorCodes.INTERNALCCMCIFError,
             org.omg.CORBA.CompletionStatus.COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
         throw ex;
      }
   }
   
   <xsl:value-of select="concat($lf, 'public ', $scopedReturnType, ' disconnect_', $_receptacleName, '()',
      $lf, 'throws org.omg.Components.NoConnection {', $lf)"/>
      CDMW_LOG_FUNCTION(logger, "");
      <xsl:value-of select="$scopedReturnType"/> result = get_connection_<xsl:value-of select="$_receptacleName"/>();
      try {
          super.disconnect(RECEP_<xsl:value-of select="$uppercaseReceptacleName"/>_NAME, null);
      } catch(org.omg.Components.InvalidName in) {
         CDMW_ERROR(logger, 
            "Invalid name for receptacle disconnection!", 
            new Throwable());
         org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
             ExceptionMinorCodes.INTERNALCCMCIFError,
             org.omg.CORBA.CompletionStatus.COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
         throw ex;
      } catch(org.omg.Components.InvalidConnection ic) {
         CDMW_ERROR(logger, 
            "Invalid connection for receptacle disconnection!", 
            new Throwable());
         org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
             ExceptionMinorCodes.INTERNALCCMCIFError,
             org.omg.CORBA.CompletionStatus.COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
         throw ex;
      } catch(org.omg.Components.CookieRequired cr) {
         CDMW_ERROR(logger, 
            "Cookie required for receptacle disconnection!", 
            new Throwable());
         org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
             ExceptionMinorCodes.INTERNALCCMCIFError,
             org.omg.CORBA.CompletionStatus.COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
         throw ex;
      }
      return result;
   }
   
   <xsl:value-of select="concat($lf, 'public ', $scopedReturnType, ' get_connection_', $_receptacleName, '() {', $lf)"/>
      CDMW_LOG_FUNCTION(logger, "");
      org.omg.Components.ConnectionDescription[] compDesc = null;

      try {
         compDesc = super.get_connections(RECEP_<xsl:value-of select="$uppercaseReceptacleName"/>_NAME);
      } catch (org.omg.Components.InvalidName in) {
         CDMW_ERROR(logger, 
            "Invalid name getting receptacle!", 
            new Throwable());
         org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
            ExceptionMinorCodes.INTERNAL, 
            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
         throw ex;
      } catch (org.omg.CORBA.SystemException se) {
         CDMW_LOG_FUNCTION_EXCEPTION(logger, se);
         throw se;
      }

      org.omg.CORBA.Object obj = null;
      if (compDesc.length == 1) {
         obj = compDesc[0].objref;
      }
      
      // The spec says that nil shall only be returned where there is no 
      // connection (no call to connect). If the receptacle is currently
      // connected this operation returns the connected object reference.
      return <xsl:value-of select="$scopedReturnType"/>Helper.unchecked_narrow(obj);
   }

</xsl:template> <!-- end of template session_component_java.content.2 -->



<!--
-->
<xsl:template name="session_component_java.content.3">
   <xsl:param name="_node"/>
   <xsl:param name="_readonlyId"/>
   <xsl:param name="_readonlyEndId"/>
   <xsl:param name="_paramName"/>
   <xsl:param name="_paramType"/>
   <xsl:param name="_returnType"/>
   <xsl:param name="_javaOperationName"/>
   <xsl:param name="_componentObjRefVar"/>
   <xsl:param name="_objRefVarType"/>

   <xsl:if test="not(boolean($_node/@stereotype)) or ($_node/@stereotype = $_readonlyEndId) or ($_node/@stereotype = $_readonlyId)">
      <xsl:value-of select="concat($lf, 'public ', $_returnType, ' ', $_javaOperationName, '() {', $lf)"/>
         CDMW_LOG_FUNCTION(logger, "");
         <xsl:value-of select="$_componentObjRefVar"/> compExec = obtain_component_executor();
         <xsl:value-of select="$_objRefVarType"/> result = compExec.<xsl:value-of select="$_javaOperationName"/>();

         try {
             this.executorLocator.release_executor(compExec);
         } catch(org.omg.Components.CCMException ccme) {
             CDMW_ERROR(logger, 
                "Exception received: " + ccme, 
                new Throwable());
             org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                 ExceptionMinorCodes.INTERNALCCMCIFError,
                 org.omg.CORBA.CompletionStatus.COMPLETED_NO);
             CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
             throw ex;
         }
         return result;
      }

      <xsl:if test="not($_node/@stereotype = $_readonlyEndId or $_node/@stereotype = $_readonlyId)">
         <xsl:value-of select="concat($lf, 'public void ', $_javaOperationName, '(', $_paramType, ' ', $_paramName, ') {', $lf)"/>
            CDMW_LOG_FUNCTION(logger, "");
            <xsl:value-of select="$_componentObjRefVar"/> compExec = obtain_component_executor();
             compExec.<xsl:value-of select="concat($_javaOperationName, '(', $_paramName, ');')"/>

             try {
                 this.executorLocator.release_executor(compExec);
             } catch(org.omg.Components.CCMException ccme) {
                 CDMW_ERROR(logger, 
                    "Exception received: " + ccme, 
                    new Throwable());
                 org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                     ExceptionMinorCodes.INTERNALCCMCIFError,
                     org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                 CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                 throw ex;
             }

             // set boolean to true (needed for configuration_completed)
             this.is<xsl:value-of select="$_paramName"/>Configured = true;
         }
      </xsl:if>
   </xsl:if>
   
</xsl:template> <!-- end of template session_component_java.content.3 -->



<!--
-->
<xsl:template name="session_component_java.content.4">
   <xsl:param name="_node"/>
   <xsl:param name="_readonlyId"/>
   <xsl:param name="_readonlyEndId"/>
   <xsl:param name="_paramName"/>
   
   <xsl:if test="not(boolean($_node/@stereotype)) or ($_node/@stereotype = $_readonlyEndId) or ($_node/@stereotype = $_readonlyId)">
      <xsl:if test="not($_node/@stereotype = $_readonlyEndId or $_node/@stereotype = $_readonlyId)">
         <xsl:value-of select="concat($lf, 'private boolean is', $_paramName, 'Configured = false;', $lf)"/>
      </xsl:if>
   </xsl:if>
</xsl:template> <!-- end of template session_component_java.content.4 -->



<!--
-->
<xsl:template name="session_component_java.content.5">
   <xsl:param name="_consumerTypeNode"/>
   <xsl:param name="_consumerName"/>
   <xsl:param name="_componentNode"/>
   <xsl:param name="_isSegmented"/>
   
   <xsl:variable name="consumerTypeName" select="$_consumerTypeNode/@name"/>
   <xsl:variable name="componentName" select="$_componentNode/@name"/>
   
   <xsl:variable name="javaScopedConsumerTypeName">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_consumerTypeNode"/>
         <xsl:with-param name="_lang" select="'java'"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="javaScopedComponentName">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_componentNode"/>
         <xsl:with-param name="_lang" select="'java'"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="javaComponentScope">
      <xsl:call-template name="removeLastToken">
         <xsl:with-param name="_scopedName" select="$javaScopedComponentName"/>
         <xsl:with-param name="_sep" select="$javaSep"/>
      </xsl:call-template>
   </xsl:variable>
   
   <xsl:if test="$_isSegmented = 'no'">
      <xsl:variable name="javaScopedConsumerName">
         <xsl:call-template name="getScopedNameOfNode">
            <xsl:with-param name="_node" select="$_consumerTypeNode"/>
            <xsl:with-param name="_lang" select="'java'"/>
         </xsl:call-template>
      </xsl:variable>
      <xsl:variable name="javaConsumerScope">
         <xsl:call-template name="removeLastToken">
            <xsl:with-param name="_scopedName" select="$javaScopedConsumerName"/>
            <xsl:with-param name="_sep" select="$javaSep"/>
         </xsl:call-template>
      </xsl:variable>
      <xsl:variable name="lowercaseComponentName" select="translate($componentName, $ucase, $lcase)"/>
      <xsl:variable name="consumerClassname" select="concat($consumerTypeName, 'Consumer')"/>
      
      public void push_<xsl:value-of select="concat($consumerTypeName, '(', $javaScopedConsumerTypeName, ' the', $consumerTypeName, ')')"/>
      {
          CDMW_LOG_FUNCTION(logger, "");
          checkIsRemoved();
   
          this.state = this.state.operationalRequest(this);

          try {
              org.omg.CORBA.Object obj = 
                  this.executorLocator.obtain_executor("<xsl:value-of select="$_consumerName"/>");
   
              if (obj == null)
              {
                  CDMW_ERROR(logger, 
                      "Executor for consumer <xsl:value-of select="$_consumerName"/> is null",
                      new Throwable());
                  org.omg.CORBA.OBJECT_NOT_EXIST ex =
                      new org.omg.CORBA.OBJECT_NOT_EXIST(
                          ExceptionMinorCodes.OBJECT_NOT_EXIST,
                          org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                  CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                  throw ex;
              }
              
              try {
                  <xsl:value-of select="concat($javaComponentScope, $javaSep, 'CCM_', $componentName, ' ', $lowercaseComponentName, 'Exec = ')"/>
                      <xsl:value-of select="concat($javaComponentScope, $javaSep, 'CCM_', $componentName, 'Helper.narrow(obj);', $lf)"/>
                  <xsl:value-of select="$lowercaseComponentName"/>Exec.push_<xsl:value-of select="concat($_consumerName, '(the', $consumerTypeName,');')"/>
                      
              } catch(org.omg.CORBA.BAD_PARAM bp1) {
                   CDMW_DEBUG(logger, 
                       "Executor for consumer <xsl:value-of select="$_consumerName"/> is not the component itself",
                        new Throwable());
               
                   try {     
                       // Component may segmented
                       <xsl:value-of select="concat($javaConsumerScope, $javaSep, 'CCM_', $consumerClassname, ' consumerExec = ')"/>
                           <xsl:value-of select="concat($javaConsumerScope, $javaSep, 'CCM_', $consumerClassname,  'Helper.narrow(obj);')"/>
                      // push event to the consumer executor
                      consumerExec.push(the<xsl:value-of select="$consumerTypeName"/>);   
               
                   } catch(org.omg.CORBA.BAD_PARAM bp2) {
                      CDMW_ERROR(logger,
                          "Executor for consumer <xsl:value-of select="$_consumerName"/> has wrong type",
                          new Throwable());
                      org.omg.CORBA.OBJECT_NOT_EXIST ex =
                          new org.omg.CORBA.OBJECT_NOT_EXIST(
                              ExceptionMinorCodes.OBJECT_NOT_EXIST, 
                              org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                      CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                      throw ex;
                   }
              }
              
              this.executorLocator.release_executor(obj);
              
         } catch(org.omg.Components.CCMException ccme) {
            CDMW_ERROR(logger, 
                "CCMException received: " + ccme,
                new Throwable());
            org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                ExceptionMinorCodes.INTERNALCCMCIFError, 
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
         }
      }
   </xsl:if>   
   
   <xsl:value-of select="concat($lf, 'public ', $javaScopedConsumerTypeName, 'Consumer',
                                $lf, 'get_consumer_', $_consumerName, '()',
                                $lf, '   {', $lf)"/>
      CDMW_LOG_FUNCTION(logger, "");
      org.omg.CORBA.Object obj = null;

      try {
         obj = super.get_consumer(CONSUMER_<xsl:value-of select="translate($_consumerName, $lcase, $ucase)"/>_NAME);
      } catch (org.omg.Components.InvalidName in) {
         CDMW_ERROR(logger, 
            "Invalid consumer name", 
            new Throwable());
         org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
            ExceptionMinorCodes.INTERNAL, 
            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
         throw ex;
      }

      <xsl:value-of select="concat($javaScopedConsumerTypeName, 'Consumer consumerRef = ',
                                   $javaScopedConsumerTypeName, 'ConsumerHelper.narrow(obj);')"/>

      if (consumerRef == null) {
         CDMW_ERROR(logger, 
            "Consumer has wrong type", 
            new Throwable());
         org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
            ExceptionMinorCodes.INTERNAL, 
            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
         throw ex;
      }
       
      return consumerRef;
   }
</xsl:template> <!-- end of template session_component_java.content.5 -->



<!--
-->
<xsl:template name="session_component_java.content.6">
   <xsl:param name="_publisherTypeNode"/>
   <xsl:param name="_publisherName"/>

   <xsl:variable name="scopedReturnType">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_publisherTypeNode"/>
         <xsl:with-param name="_lang" select="'java'"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="uppercasePublisherName" select="translate($_publisherName, $lcase, $ucase)"/>

   <xsl:value-of select="concat($lf, 'public org.omg.Components.Cookie subscribe_', $_publisherName, '(', $scopedReturnType, 'Consumer consumer)', $lf, 'throws org.omg.Components.ExceededConnectionLimit {', $lf)"/>
      CDMW_LOG_FUNCTION(logger, "");
      org.omg.Components.Cookie result = null;
      
      try {
          result = super.subscribe(PUBLISHER_<xsl:value-of select="$uppercasePublisherName"/>_NAME, consumer);
      } catch (org.omg.Components.InvalidName in) {
          CDMW_ERROR(logger, 
              "Invalid name for publisher subscription!",
              new Throwable());
          org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
             ExceptionMinorCodes.INTERNAL, 
             org.omg.CORBA.CompletionStatus.COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
         throw ex;
      } catch (org.omg.Components.InvalidConnection ic) {
          CDMW_ERROR(logger, "Invalid connection", new Throwable());
          // should never appear!
          org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
             ExceptionMinorCodes.INTERNAL, 
             org.omg.CORBA.CompletionStatus.COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
         throw ex;
      } catch (org.omg.Components.ExceededConnectionLimit ecl) {
          CDMW_LOG_FUNCTION_EXCEPTION(logger, ecl);
          throw ecl;
      } catch (org.omg.CORBA.SystemException se) {
          CDMW_LOG_FUNCTION_EXCEPTION(logger, se);
          throw se;
      }
      
      return result;
   }

   <xsl:value-of select="concat($lf, 'public ', $scopedReturnType, 'Consumer unsubscribe_', $_publisherName, '(org.omg.Components.Cookie ck)', $lf, 'throws org.omg.Components.InvalidConnection {', $lf)"/>
      CDMW_LOG_FUNCTION(logger, "");
      if (ck == null){
         CDMW_ERROR(logger, "Cookie is null", new Throwable());
         org.omg.CORBA.BAD_PARAM ex = new org.omg.CORBA.BAD_PARAM(
            ExceptionMinorCodes.BAD_PARAMNilObjectReference, 
            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
         throw ex;
      }
       
      <xsl:value-of select="$scopedReturnType"/>Consumer result = null;
      try {
         org.omg.Components.EventConsumerBase consumer 
            = super.get_consumer(PUBLISHER_<xsl:value-of select="$uppercasePublisherName"/>_NAME, ck);

         result = <xsl:value-of select="$scopedReturnType"/>ConsumerHelper.unchecked_narrow(consumer);

         super.unsubscribe(PUBLISHER_<xsl:value-of select="$uppercasePublisherName"/>_NAME, ck);
      } catch (org.omg.Components.InvalidName in) {
         // internal error should not appear
         CDMW_ERROR(logger, "Invalid name for publisher unsubscription!", new Throwable());
         org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
            ExceptionMinorCodes.INTERNAL, 
            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
         throw ex;
      } catch (org.omg.Components.InvalidConnection ic) {
         CDMW_LOG_FUNCTION_EXCEPTION(logger, ic);
         throw ic;
      } catch (org.omg.CORBA.SystemException se) {
         CDMW_LOG_FUNCTION_EXCEPTION(logger, se);
         throw se;
      }

      return result;
   }
</xsl:template> <!-- end of template session_component_java.content.6 -->



<!--
-->
<xsl:template name="session_component_java.content.7">
   <xsl:param name="_emitterTypeNode"/>
   <xsl:param name="_emitterName"/>

   <xsl:variable name="scopedReturnType">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_emitterTypeNode"/>
         <xsl:with-param name="_lang" select="'java'"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="uppercaseEmitterName" select="translate($_emitterName, $lcase, $ucase)"/>

   <xsl:value-of select="concat($lf, 'public void connect_', $_emitterName, '(', $scopedReturnType, 'Consumer consumer)',
                                $lf, 'throws org.omg.Components.AlreadyConnected {', $lf)"/>
      CDMW_LOG_FUNCTION(logger, "");
      try {
         super.connect_consumer(EMITTER_<xsl:value-of select="$uppercaseEmitterName"/>_NAME, consumer);
      } catch (org.omg.Components.InvalidName in) {
         // can not appear!
         CDMW_ERROR(logger, "Invalid name for emitter conection!", new Throwable());
         org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
            ExceptionMinorCodes.INTERNAL, 
            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
         throw ex;
      } catch (org.omg.Components.InvalidConnection ic) {
         // can not appear!
         CDMW_ERROR(logger, "Invalid connection for emitter conection!", new Throwable());
         org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
            ExceptionMinorCodes.INTERNAL, 
            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
         throw ex;
      } catch (org.omg.CORBA.SystemException se) {
         CDMW_LOG_FUNCTION_EXCEPTION(logger, se);
         throw se;
      } 
   }

   <xsl:value-of select="concat($lf, 'public ', $scopedReturnType, 'Consumer disconnect_', $_emitterName, '()',
                                $lf, 'throws org.omg.Components.NoConnection {', $lf)"/>
      CDMW_LOG_FUNCTION(logger, "");
      try {
         org.omg.Components.EventConsumerBase consumer = 
            super.disconnect_consumer(EMITTER_<xsl:value-of select="$uppercaseEmitterName"/>_NAME);

         <xsl:value-of select="$scopedReturnType"/>Consumer result = 
            <xsl:value-of select="$scopedReturnType"/>ConsumerHelper.narrow(consumer);

         if (result == null) {
            // can not appear!
            CDMW_ERROR(logger, "Old connection had wrong type", new Throwable());
            org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
               ExceptionMinorCodes.INTERNAL, 
               org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
         }

         return result;
      } catch (org.omg.Components.InvalidName in) {
         // can not appear!
         CDMW_ERROR(logger, "Invalid name for emitter disconection!", new Throwable());
         org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
            ExceptionMinorCodes.INTERNAL, 
            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
         throw ex;
      <!-- //TODO: remove this catch statement if not needed
      } catch (org.omg.Components.InvalidConnection ic) {
         // can not appear!
         throw new org.omg.CORBA.INTERNAL(
            ExceptionMinorCodes.INTERNAL, 
            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
      -->
      } catch (org.omg.CORBA.SystemException se) {
         CDMW_LOG_FUNCTION_EXCEPTION(logger, se);
         throw se;
      } 
   }

</xsl:template> <!-- end of template session_component_java.content.7 -->



<!--
-->
<xsl:template name="session_component_java.content.8">
   <xsl:param name="_node"/>
   <xsl:param name="_readonlyId"/>
   <xsl:param name="_readonlyEndId"/>
   <xsl:param name="_paramName"/>

   <xsl:if test="not(boolean($_node/@stereotype)) or ($_node/@stereotype = $_readonlyEndId) or ($_node/@stereotype = $_readonlyId)">
      <xsl:if test="not($_node/@stereotype = $_readonlyEndId or $_node/@stereotype = $_readonlyId)">
         // check if the attribute have been initialised only if not created by a factory op
         if (!this.is<xsl:value-of select="$_paramName"/>Configured) {
            // component is not well configured
            CDMW_ERROR(logger, 
                "Attribut <xsl:value-of select="$_paramName"/> not initialised!",
                new Throwable());
            org.omg.Components.InvalidConfiguration ex = new org.omg.Components.InvalidConfiguration();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
         }
      </xsl:if>
   </xsl:if>
</xsl:template> <!-- end of template session_component_java.content.8 -->



<!--
-->
<xsl:template name="session_component_java.content.9">
   <xsl:param name="_facetNode"/>
   <xsl:param name="_facetName"/>

   <xsl:variable name="uppercaseFacetName" select="translate($_facetName, $lcase, $ucase)"/>
   
   <xsl:value-of select="concat($lf, 'public static final String FACET_', $uppercaseFacetName, '_NAME')"/> = "<xsl:value-of select="$_facetName"/>";
   <xsl:value-of select="concat($lf, 'public static final String FACET_', $uppercaseFacetName, '_REP_ID')"/> = "<xsl:value-of select="$_facetNode/UML:ModelElement.taggedValue/UML:TaggedValue[@tag = 'typeid']/@value"/>";
</xsl:template> <!-- end of template session_component_java.content.9 -->



<!--
-->
<xsl:template name="session_component_java.content.10">
   <xsl:param name="_facetName"/>
   
   else if (execName.equals("<xsl:value-of select="$_facetName"/>")) {
      result = this.compExecutor.<xsl:value-of select="concat('get_', $_facetName, '();')"/>
   }
</xsl:template> <!-- end of template session_component_java.content.10 -->



<!--
-->
<xsl:template name="session_component_java.content.11">
   <xsl:param name="_facetTypeName"/>
   <xsl:param name="_facetName"/>
   
   <xsl:variable name="uppercaseFacetName" select="translate($_facetName, $lcase, $ucase)"/>

   try {
      org.omg.CORBA.Object <xsl:value-of select="$_facetName"/>_facet
         = declare_facet(compOid, 
                         FACET_<xsl:value-of select="$uppercaseFacetName"/>_NAME, 
                         FACET_<xsl:value-of select="$uppercaseFacetName"/>_REP_ID);
                         
      if (<xsl:value-of select="$_facetName"/>_facet == null) {
         CDMW_ERROR(logger, 
            "Internal exception raised when declaring facet <xsl:value-of select="$_facetName"/>!",
            new Throwable());
         org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
            ExceptionMinorCodes.INTERNAL, 
            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
         throw ex;
      }
   } catch (AlreadyDoneException ade) {
      CDMW_ERROR(logger, "Facet for_clients already declared!", new Throwable());
      // Don't propagate while its probably a code generation error!
   } catch (InternalErrorException iee) {
      CDMW_ERROR(logger, 
          "Internal exception raised when declaring facet <xsl:value-of select="$_facetTypeName"/>!",
          new Throwable());
      org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
         ExceptionMinorCodes.INTERNAL, 
         org.omg.CORBA.CompletionStatus.COMPLETED_NO);
      CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
      throw ex;
   }
</xsl:template> <!-- end of template session_component_java.content.11 -->



<!--
-->
<xsl:template name="session_component_java.content.12">
   <xsl:param name="_facetTypeName"/>
   <xsl:param name="_facetName"/>
   
   <xsl:variable name="uppercaseFacetName" select="translate($_facetName, $lcase, $ucase)"/>

   if (facet.equals(FACET_<xsl:value-of select="$uppercaseFacetName"/>_NAME)) {
      facetServant = new <xsl:value-of select="concat($_facetTypeName, 'Impl')"/>(
         this.executorLocator, 
         FACET_<xsl:value-of select="$uppercaseFacetName"/>_NAME,
         this.context,
         this.compRef,
         this,
         this.logger);
   }
   else
</xsl:template> <!-- end of template session_component_java.content.12 -->



<!--
-->
<xsl:template name="session_component_java.content.13">
   <xsl:param name="_receptacleNode"/>
   <xsl:param name="_receptacleName"/>

   <xsl:variable name="uppercaseReceptacleName" select="translate($_receptacleName, $lcase, $ucase)"/>
   
   <xsl:value-of select="concat($lf, 'public static final String RECEP_', $uppercaseReceptacleName, '_NAME')"/> = "<xsl:value-of select="$_receptacleName"/>";
   <xsl:value-of select="concat($lf, 'public static final String RECEP_', $uppercaseReceptacleName, '_REP_ID')"/> = "<xsl:value-of select="$_receptacleNode/UML:ModelElement.taggedValue/UML:TaggedValue[@tag = 'typeid']/@value"/>";
</xsl:template> <!-- end of template session_component_java.content.13 -->



<!--
-->
<xsl:template name="session_component_java.content.14">
   <xsl:param name="_receptacleName"/>
   
   <xsl:variable name="uppercaseReceptacleName" select="translate($_receptacleName, $lcase, $ucase)"/>

   declare_receptacle(<xsl:value-of select="concat('RECEP_', $uppercaseReceptacleName,'_NAME, RECEP_', $uppercaseReceptacleName,'_REP_ID')"/>);
</xsl:template> <!-- end of template session_component_java.content.14 -->



<!--
-->
<xsl:template name="session_component_java.content.15">
   <xsl:param name="_publisherNode"/>
   <xsl:param name="_publisherName"/>

   <xsl:variable name="uppercasePublisherName" select="translate($_publisherName, $lcase, $ucase)"/>
   <xsl:variable name="repId">
      <xsl:call-template name="replaceCharsInString">
         <xsl:with-param name="_stringIn" select="$_publisherNode/UML:ModelElement.taggedValue/UML:TaggedValue[@tag = 'typeid']/@value"/>
         <xsl:with-param name="_charsIn" select="':1.0'"/>
         <xsl:with-param name="_charsOut" select="'Consumer:1.0'"/>
      </xsl:call-template>
   </xsl:variable>
   
   <xsl:value-of select="concat($lf, 'public static final String PUBLISHER_', $uppercasePublisherName, '_NAME')"/> = "<xsl:value-of select="$_publisherName"/>";
   <xsl:value-of select="concat($lf, 'public static final String PUBLISHER_', $uppercasePublisherName, '_REP_ID')"/> = "<xsl:value-of select="$repId"/>";
</xsl:template> <!-- end of template session_component_java.content.15 -->



<!--
-->
<xsl:template name="session_component_java.content.16">
   <xsl:param name="_emitterNode"/>
   <xsl:param name="_emitterName"/>

   <xsl:variable name="uppercaseEmitterName" select="translate($_emitterName, $lcase, $ucase)"/>
   <xsl:variable name="repId">
      <xsl:call-template name="replaceCharsInString">
         <xsl:with-param name="_stringIn" select="$_emitterNode/UML:ModelElement.taggedValue/UML:TaggedValue[@tag = 'typeid']/@value"/>
         <xsl:with-param name="_charsIn" select="':1.0'"/>
         <xsl:with-param name="_charsOut" select="'Consumer:1.0'"/>
      </xsl:call-template>
   </xsl:variable>
   
   <xsl:value-of select="concat($lf, 'public static final String EMITTER_', $uppercaseEmitterName, '_NAME')"/> = "<xsl:value-of select="$_emitterName"/>";
   <xsl:value-of select="concat($lf, 'public static final String EMITTER_', $uppercaseEmitterName, '_REP_ID')"/> = "<xsl:value-of select="$repId"/>";
</xsl:template> <!-- end of template session_component_java.content.16 -->



<!--
-->
<xsl:template name="session_component_java.content.17">
   <xsl:param name="_publisherName"/>
   
   <xsl:variable name="uppercasePublisherName" select="translate($_publisherName, $lcase, $ucase)"/>

   declare_publisher(<xsl:value-of select="concat('PUBLISHER_', $uppercasePublisherName,'_NAME, PUBLISHER_', $uppercasePublisherName,'_REP_ID')"/>);
</xsl:template> <!-- end of template session_component_java.content.17 -->



<!--
-->
<xsl:template name="session_component_java.content.18">
   <xsl:param name="_emitterName"/>
   
   <xsl:variable name="uppercaseEmitterName" select="translate($_emitterName, $lcase, $ucase)"/>

   declare_emitter(<xsl:value-of select="concat('EMITTER_', $uppercaseEmitterName,'_NAME, EMITTER_', $uppercaseEmitterName,'_REP_ID')"/>);
</xsl:template> <!-- end of template session_component_java.content.18 -->



<!--
-->
<xsl:template name="session_component_java.content.19">
   <xsl:param name="_consumerTypeNode"/>
   <xsl:param name="_consumerName"/>

   <xsl:variable name="uppercaseConsumerName" select="translate($_consumerName, $lcase, $ucase)"/>
   <xsl:variable name="repId">
      <xsl:variable name="partialRepId">
         <xsl:call-template name="replaceCharsInString">
            <xsl:with-param name="_stringIn">
               <xsl:call-template name="getScopedNameOfNode">
                  <xsl:with-param name="_node" select="$_consumerTypeNode"/>
               </xsl:call-template>
            </xsl:with-param>
            <xsl:with-param name="_charsIn" select="$javaSep"/>
            <xsl:with-param name="_charsOut" select="'/'"/>
         </xsl:call-template>
      </xsl:variable>
      <xsl:value-of select="concat('IDL:', $idlPragmaPrefix, '/Cdmw_', $partialRepId, 'Consumer:1.0')"/>
   </xsl:variable>
   
   <xsl:value-of select="concat($lf, 'public static final String CONSUMER_', $uppercaseConsumerName, '_NAME')"/> = "<xsl:value-of select="$_consumerName"/>";
   <xsl:value-of select="concat($lf, 'public static final String CONSUMER_', $uppercaseConsumerName, '_REP_ID')"/> = "<xsl:value-of select="$repId"/>";
</xsl:template> <!-- end of template session_component_java.content.19 -->



<!--
-->
<xsl:template name="session_component_java.content.20">
   <xsl:param name="_consumerName"/>

   else if (execName.equals(<xsl:value-of select="concat('CONSUMER_', translate($_consumerName, $lcase, $ucase), '_NAME')"/>)) {
      result = this.compExecutor;
   }
</xsl:template> <!-- end of template session_component_java.content.20 -->



<!--
-->
<xsl:template name="session_component_java.content.21">
   <xsl:param name="_consumerName"/>
   
   <xsl:variable name="uppercaseConsumerName" select="translate($_consumerName, $lcase, $ucase)"/>

   declare_consumer(compOid, <xsl:value-of select="concat('CONSUMER_', $uppercaseConsumerName,'_NAME, CONSUMER_', $uppercaseConsumerName,'_REP_ID')"/>);
</xsl:template> <!-- end of template session_component_java.content.21 -->



<!--
-->
<xsl:template name="session_component_java.content.22">
   <xsl:param name="_consumerTypeName"/>
   <xsl:param name="_consumerName"/>
   
   <xsl:variable name="uppercaseConsumerName" select="translate($_consumerName, $lcase, $ucase)"/>

   if (facet.equals(CONSUMER_<xsl:value-of select="$uppercaseConsumerName"/>_NAME)) {
      facetServant = new <xsl:value-of select="concat($_consumerTypeName, 'ConsumerImpl')"/>(
         this.executorLocator, 
         CONSUMER_<xsl:value-of select="$uppercaseConsumerName"/>_NAME,
         this.context,
         this,
         this.logger);
   }
   else
</xsl:template> <!-- end of template session_component_java.content.22 -->


<!--
   This template generates definitions of operations of a given interface.
   This includes inherited operations.

   @param _interfaceNode           The interface node.
   @param _facetName               The name of the facet.
   @param _facetTypeName           The type of the facet.
   @param _componentImplClassname  The component implementation class name.
   @param _cppFacetScope           The C++ scope for the class (namespaces ...).
-->
<xsl:template name="session_component_java.Monolithic.Facets.Operations.all">
   <xsl:param name="_interfaceNode"/>
   <xsl:param name="_facetName"/>
   <xsl:param name="_facetTypeName"/>
   <xsl:param name="_javaFacetScope"/>

   <xsl:variable name="allBaseInterfaces">
       <xsl:call-template name="getAllBaseClasses">
           <xsl:with-param name="_classNode" select="$_interfaceNode"/>
       </xsl:call-template>
   </xsl:variable>

   <xsl:for-each select="$allBaseInterfaces/node()[string(@xmi.id)]">
       <xsl:sort select="@xmi.id"/>
       <xsl:variable name="_xmi_id" select="./@xmi.id"/>
       <!--
       NOTA: Note the following trick to get back to the initial document context
             to be able to access operation definitions ... etc. because the result
             tree ($allBaseInterfaces) is a new xml document a only contains a copy
             of a subset of the original tree.
       -->
       <xsl:for-each select="$_interfaceNode">
           <xsl:variable name="_interface" select="key('classById', $_xmi_id)"/>
           <xsl:call-template name="session_component_java.Monolithic.Facets.Operations.own">
               <xsl:with-param name="_interfaceNode" select="$_interface" />
               <xsl:with-param name="_facetName" select="$_facetName" />
               <xsl:with-param name="_facetTypeName" select="$_facetTypeName"/>
               <xsl:with-param name="_javaFacetScope" select="$_javaFacetScope" />
           </xsl:call-template>
       </xsl:for-each>
   </xsl:for-each>
</xsl:template> <!-- end of template session_component_java.Monolithic.Facets.Operations.all -->


<!--
   This template generates definitions of operations of a given interface.
   This does not include inherited operations.

   @param _interfaceNode  The interface node.
-->
<xsl:template name="session_component_java.Monolithic.Facets.Operations.own">
   <xsl:param name="_interfaceNode"/>
   <xsl:param name="_facetName"/>   
   <xsl:param name="_facetTypeName"/>
   <xsl:param name="_javaFacetScope"/>

   <xsl:variable name="containedOps"  select="key('operations', $_interfaceNode/@xmi.id)"/>
   <xsl:for-each select="$containedOps">
      <!--
         Get the operation return type.
      -->
      <xsl:variable name="returnType">
         <xsl:call-template name="getType">
            <xsl:with-param name="_id" select="UML:BehavioralFeature.parameter/UML:Parameter[@kind = 'return']/@type"/>
            <xsl:with-param name="_kind" select="'return'"/>
            <xsl:with-param name="_lang" select="'java'"/>
         </xsl:call-template>
      </xsl:variable>


      <!--
         Operation signature.
      -->
      <xsl:variable name="javaOperationName">
         <xsl:call-template name="mapJavaReservedWords">
            <xsl:with-param name="_operationName" select="@name"/>
         </xsl:call-template>
      </xsl:variable>
      <xsl:value-of select="concat($lf, '    public ', $returnType, ' ', $javaOperationName, '(', $lf)"/>
            <!--
               IN, OUT and INOUT parameters.
            -->
            <xsl:for-each select="UML:BehavioralFeature.parameter/UML:Parameter[@kind != 'return']">
               <!--
                  Get the parameter type.
               -->
               <xsl:variable name="paramType">
                  <xsl:call-template name="getParameterType">
                     <xsl:with-param name="_paramNode" select="."/>
                     <xsl:with-param name="_lang" select="'java'"/>
                  </xsl:call-template>
               </xsl:variable>
               <xsl:value-of select="concat($paramType, ' ', @name)"/>
               <!--
                  Write out comma if necessary.
               -->
               <xsl:if test="position() != last()">
                  <xsl:value-of select="concat(',', $lf)"/>
               </xsl:if>
            </xsl:for-each>
         <xsl:value-of select="'    )'"/>
         <!--
            Exceptions.
         -->
         <xsl:if test="count(UML:ModelElement.taggedValue/UML:TaggedValue[@tag='raises']) > 0">
            <xsl:value-of select="concat(' throws', $lf)"/>
         </xsl:if>
         <xsl:for-each select="UML:ModelElement.taggedValue/UML:TaggedValue">
            <xsl:if test="@tag = 'raises'">
               <xsl:variable name="scopedException">
                  <xsl:call-template name="getScopedException">
                     <xsl:with-param name="_exception" select="@value"/>
                  </xsl:call-template>
               </xsl:variable>
               <xsl:value-of select="concat('        ', $scopedException)"/>
               <xsl:if test="position() != last()">
                  <xsl:value-of select="concat(',', $lf)"/>
               </xsl:if>
            </xsl:if>
         </xsl:for-each>
         <xsl:value-of select="concat(' {', $lf)"/>

        CDMW_LOG_FUNCTION(logger, "");

        this.state = this.state.operationalRequest(this);

        try {
            org.omg.CORBA.Object obj = this.executorLocator.obtain_executor("<xsl:value-of select="$_facetName"/>");

            if (obj == null) {
                CDMW_DEBUG(logger, 
                    "Executor for facet <xsl:value-of select="$_facetName"/> is null",
                    new Throwable());
                org.omg.CORBA.OBJECT_NOT_EXIST ex =
                    new org.omg.CORBA.OBJECT_NOT_EXIST(
                        ExceptionMinorCodes.OBJECT_NOT_EXIST,
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                throw ex;
            }
   
            <xsl:value-of select="concat($_javaFacetScope, $javaSep, 'CCM_', $_facetTypeName)"/> facetExec;
            try {
                facetExec = <xsl:value-of    select="concat($_javaFacetScope, $javaSep, 'CCM_', $_facetTypeName)"/>Helper.narrow(obj);
   
            } catch (org.omg.CORBA.BAD_PARAM e) {
                CDMW_ERROR(logger, 
                    "Executor for facet <xsl:value-of select="$_facetName"/> has wrong type",
                    new Throwable());
                org.omg.CORBA.OBJECT_NOT_EXIST ex =
                    new org.omg.CORBA.OBJECT_NOT_EXIST(
                        ExceptionMinorCodes.OBJECT_NOT_EXIST, 
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                throw ex;
            }
       
            <xsl:if test="not($returnType = 'void')">
               <xsl:value-of select="concat($returnType, ' __result =')"/>
            </xsl:if>
            facetExec.<xsl:value-of select="$javaOperationName"/>(
               <xsl:for-each select="UML:BehavioralFeature.parameter/UML:Parameter[@kind != 'return']">
                  <xsl:value-of select="@name"/>
                  <xsl:if test="position() != last()">
                     <xsl:text>, </xsl:text>
                  </xsl:if>
               </xsl:for-each>
            );

            this.executorLocator.release_executor(facetExec);

            <xsl:if test="not($returnType = 'void')">
            return __result;
            </xsl:if>
        } catch(org.omg.Components.CCMException ccme) {
            CDMW_ERROR(logger, 
                "CCMException received: " + ccme,
                new Throwable());
            org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                ExceptionMinorCodes.INTERNALCCMCIFError, 
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }


    }
   </xsl:for-each>
</xsl:template> <!-- end of template session_component_java.Monolithic.Facets.Operations.own -->




<!--
   This template generates definitions of operations of a given interface.
   This does not include inherited operations.

   @param _interfaceNode  The interface node.
-->
<xsl:template name="session_component_java.SupportedInterfaces.operations.definition.own">
   <xsl:param name="_interfaceNode"/>
   <xsl:param name="_componentObjRefVar"/>

      <xsl:variable name="containedOps"  select="key('operations', $_interfaceNode/@xmi.id)"/>

      <xsl:for-each select="$containedOps">
         <!--
            Get the operation return type.
         -->
         <xsl:variable name="returnType">
            <xsl:call-template name="getType">
               <xsl:with-param name="_id" select="UML:BehavioralFeature.parameter/UML:Parameter[@kind = 'return']/@type"/>
               <xsl:with-param name="_kind" select="'return'"/>
               <xsl:with-param name="_lang" select="'java'"/>
            </xsl:call-template>
         </xsl:variable>

         <!--
            Operation signature.
         -->
         <xsl:variable name="javaOperationName">
            <xsl:call-template name="mapJavaReservedWords">
               <xsl:with-param name="_operationName" select="@name"/>
            </xsl:call-template>
         </xsl:variable>
         <xsl:value-of select="concat('public ', $returnType, $lf, $javaOperationName, '(', $lf)"/>
               <!--
                  IN, OUT and INOUT parameters.
               -->
               <xsl:for-each select="UML:BehavioralFeature.parameter/UML:Parameter[@kind != 'return']">
                  <!--
                     Get the parameter type.
                  -->
                  <xsl:variable name="paramType">
                     <xsl:call-template name="getParameterType">
                        <xsl:with-param name="_paramNode" select="."/>
                        <xsl:with-param name="_lang" select="'java'"/>
                     </xsl:call-template>
                  </xsl:variable>
                  <xsl:value-of select="concat($paramType, ' ', @name)"/>
                  <!--
                     Write out comma if necessary.
                  -->
                  <xsl:if test="position() != last()">
                     <xsl:value-of select="concat(',', $lf)"/>
                  </xsl:if>
               </xsl:for-each>
            <xsl:value-of select="'    )'"/>
            <!--
               Exceptions.
            -->
            <xsl:if test="count(UML:ModelElement.taggedValue/UML:TaggedValue[@tag='raises']) > 0">
               <xsl:value-of select="concat(' throws', $lf)"/>
            </xsl:if>
            <xsl:for-each select="UML:ModelElement.taggedValue/UML:TaggedValue">
               <xsl:if test="@tag = 'raises'">
                  <xsl:variable name="scopedException">
                     <xsl:call-template name="getScopedException">
                        <xsl:with-param name="_exception" select="@value"/>
                     </xsl:call-template>
                  </xsl:variable>
                  <xsl:value-of select="concat('        ', $scopedException)"/>
                  <xsl:if test="position() != last()">
                     <xsl:value-of select="concat(',', $lf)"/>
                  </xsl:if>
               </xsl:if>
            </xsl:for-each>
            <xsl:value-of select="concat(' {', $lf)"/>
            
            CDMW_LOG_FUNCTION(logger, "");

            this.state = this.state.operationalRequest(this);          

            <xsl:value-of select="$_componentObjRefVar"/> compExec = obtain_component_executor();
            <xsl:if test="not($returnType = 'void')">
               <xsl:value-of select="concat($returnType, ' __result = ')"/>
            </xsl:if> 
            compExec.<xsl:value-of select="$javaOperationName"/>(
            <xsl:for-each select="UML:BehavioralFeature.parameter/UML:Parameter[@kind != 'return']">
               <xsl:value-of select="@name"/>
               <xsl:if test="position() != last()">
                  <xsl:text>, </xsl:text>
               </xsl:if>
            </xsl:for-each>
            );

            try {
                this.executorLocator.release_executor(compExec);
            } catch(org.omg.Components.CCMException ccme) {
               CDMW_ERROR(logger, 
                  "Exception received: " + ccme,
                  new Throwable());
               org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                  ExceptionMinorCodes.INTERNALCCMCIFError,
                  org.omg.CORBA.CompletionStatus.COMPLETED_NO);
               CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
               throw ex;
            }
            <xsl:if test="not($returnType = 'void')">
            return __result;
            </xsl:if>
         }
      </xsl:for-each>
</xsl:template> <!-- end of template session_component_java.SupportedInterfaces.operations.definition.own -->


<!-- Following template is only used for monolithic component -->
<xsl:template name="session_component_java.content.25">
   <xsl:param name="_consumerTypeNode"/>

   <xsl:variable name="consumerTypeName" select="$_consumerTypeNode/@name"/>

   <xsl:variable name="javaScopedConsumerTypeName">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_consumerTypeNode"/>
         <xsl:with-param name="_lang" select="'java'"/>
     </xsl:call-template>
   </xsl:variable>

        if (bad_evt_type_found)
        {
            try {
                <xsl:value-of select="$javaScopedConsumerTypeName"/> infoEvt = 
                    (<xsl:value-of select="$javaScopedConsumerTypeName"/>) evt;
                bad_evt_type_found = false;
                this.push_<xsl:value-of select="$consumerTypeName"/>(infoEvt);
            } catch (ClassCastException cce) {
            }
        }
   
</xsl:template> <!-- end of template session_component_java.content.25 -->


<!-- Following template is only used for monolithic component -->
<xsl:template name="session_component_java.content.26">
   <xsl:param name="_consumerTypeNode"/>
   <xsl:param name="_consumerName"/>
   <xsl:param name="_componentImplClassname"/>

   <xsl:variable name="consumerTypeName" select="$_consumerTypeNode/@name"/>

   <xsl:variable name="javaScopedConsumerTypeName">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_consumerTypeNode"/>
         <xsl:with-param name="_lang" select="'java'"/>
      </xsl:call-template>
   </xsl:variable>

        if (bad_evt_type_found)
        {
            /* PCR-0049
            <xsl:value-of select="$javaScopedConsumerTypeName"/> event =
                <xsl:value-of select="$javaScopedConsumerTypeName"/>Helper.read(data.create_input_stream());
            */
            
            try {
                // PCR-0049
                cdmw.orbsupport.Codec codec = new cdmw.orbsupport.Codec();
                com.thalesgroup.CdmwCcmCif.TimestampedEvent timestamped_event = 
                    (com.thalesgroup.CdmwCcmCif.TimestampedEvent) codec.decode(data);

                <xsl:value-of select="$javaScopedConsumerTypeName"/> event =
                    (<xsl:value-of select="$javaScopedConsumerTypeName"/>) timestamped_event.event;
                bad_evt_type_found = false;

                java.text.DateFormat df = new java.text.SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
                java.lang.String time_received = df.format(new java.util.Date());
                //NOTE: timestamped_event.timestamp is time offset from 15-th October 1582(UTC) in millis,
                //      cdmw.ccm.cif.CCMUtils.DELTA_UTC_EPOCH is difference between UTC and Epoch in millis
                java.lang.String time_issued = 
                    df.format(new java.util.Date(timestamped_event.timestamp - cdmw.ccm.cif.CCMUtils.DELTA_UTC_EPOCH));

                CCMLoggers.CDMW_DEBUG_EVENT(logger, "Event received at: " + time_received + 
                    " issued at: " + time_issued);

                this.push_<xsl:value-of select="$consumerTypeName"/>(event);

            } catch (ClassCastException cce) {
            
            } catch (org.omg.CORBA.SystemException se) {
                CDMW_LOG_FUNCTION_EXCEPTION(logger, se);
                throw se;
            }
        }

</xsl:template> <!-- end of template session_component_java.content.26 -->


</xsl:stylesheet>


