<?xml version="1.0" encoding="utf-8"?>
<!-- ===================================================================== -->
<!--
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-->
<!-- ===================================================================== -->


<xsl:stylesheet version="1.1"
   xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
   xmlns:UML="org.omg/UML1.3">



<!--
   Main - This template fetches the necessary data from inputs
   to generate a file.

   @param none
-->
<xsl:template name="session_context_java">
   <!--
      Parameters below are used for recursiveness.
   -->
   <xsl:param name="_index" select="1"/>

   <xsl:if test="boolean($lifecycleFrameworkNodes/home-impl) and $lifecycleFrameworkNodes/home-impl/@category = 'SESSION'">
      <xsl:if test="$_index &lt;= count($lifecycleFrameworkNodes/home-impl)">
         <xsl:variable name="homeImplNode" select="$lifecycleFrameworkNodes[home-impl and position() = $_index]/home-impl"/>

         <!--
            Get the managed component.
         -->
         <xsl:variable name="homeName">
            <xsl:call-template name="getLastToken">
               <xsl:with-param name="_string" select="$homeImplNode/../@factory-idref"/>
               <xsl:with-param name="_separator" select="$xmlSep"/>
            </xsl:call-template>
         </xsl:variable>
         <xsl:variable name="managedComponentId">
            <xsl:call-template name="getManagedComponentId">
               <xsl:with-param name="_homeName" select="$homeName"/>
            </xsl:call-template>
         </xsl:variable>
         <xsl:variable name="managedComponentNode" select="key('classById', $managedComponentId)"/>

         <xsl:if test="boolean($managedComponentNode)">
            <xsl:variable name="contextImplClassname" select="concat('CCM_', $managedComponentNode/@name, '_SessionContextImpl')"/>

            <!--
               Get the output filename.
            -->
            <xsl:variable name="filename">
               <xsl:call-template name="buildOutputFilename">
                  <xsl:with-param name="_userDirectory" select="$homeImplNode/@directory"/>
                  <xsl:with-param name="_userFilename" select="$homeImplNode/@filename"/>
                  <xsl:with-param name="_defaultFilename" select="$contextImplClassname"/>
                  <xsl:with-param name="_fileExtension" select="'.java'"/>
               </xsl:call-template>
            </xsl:variable>

            <!--
               Display a message to the user.
            -->
            <xsl:message>
               <xsl:value-of select="concat('Generating file ', $filename)"/>
            </xsl:message>

            <!--
               Generate the file.
            -->
            <xsl:document method="text" href="{$filename}">
               <!--
                  Insert the document header.
               -->
               <xsl:call-template name="docHeader">
                  <xsl:with-param name="_lang" select="'java'"/>
               </xsl:call-template>
               <!--
                  Insert the content.
               -->
               <xsl:call-template name="session_context_java.content">
                  <xsl:with-param name="_contextImplClassname" select="$contextImplClassname"/>
                  <xsl:with-param name="_scopedHomeImplClassname">
                     <xsl:call-template name="buildHomeImplClassname">
                        <xsl:with-param name="_homeImplNode" select="$homeImplNode"/>
                     </xsl:call-template>
                  </xsl:with-param>
                  <xsl:with-param name="_componentNode" select="$managedComponentNode"/>
               </xsl:call-template>
            </xsl:document>
         </xsl:if>

         <!--
            Proceed to the next home impl if any.
         -->
         <xsl:call-template name="session_context_java">
            <xsl:with-param name="_index" select="$_index + 1"/>
         </xsl:call-template>
      </xsl:if>
   </xsl:if>
</xsl:template> <!-- end of template session_context_java -->



<!--
   This template generates the content of the file.

   @param _contextImplClassname      The session context impl classname.
   @param _scopedHomeImplClassname   The scoped home impl classname.
   @param _componentNode             The component node.
-->
<xsl:template name="session_context_java.content">
   <xsl:param name="_contextImplClassname"/>
   <xsl:param name="_scopedHomeImplClassname"/>
   <xsl:param name="_componentNode"/>

   <!--
      Miscellaneous variables.
   -->
   <xsl:variable name="componentName" select="$_componentNode/@name"/>

   <xsl:variable name="javaScopedComponentName">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_componentNode"/>
         <xsl:with-param name="_lang" select="'java'"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="javaComponentScope">
      <xsl:call-template name="removeLastToken">
         <xsl:with-param name="_scopedName" select="$javaScopedComponentName"/>
         <xsl:with-param name="_sep" select="$javaSep"/>
      </xsl:call-template>
   </xsl:variable>

   <xsl:variable name="ccmSessionContextClassName">
      <xsl:variable name="tmpClassName">
         <xsl:call-template name="prefixClassName">
            <xsl:with-param name="_prefix" select="'CCM_'"/>
            <xsl:with-param name="_scopedName">
               <xsl:call-template name="prefixModuleNameOfNode">
                  <xsl:with-param name="_prefix" select="'Cdmw_'"/>
                  <xsl:with-param name="_idlPrefix" select="$idlPragmaPrefix"/>
                  <xsl:with-param name="_node" select="$_componentNode"/>
               </xsl:call-template>
            </xsl:with-param>
         </xsl:call-template>
      </xsl:variable>
      <xsl:value-of select="concat($tmpClassName, '_SessionContext')"/>
    </xsl:variable>

   <!--
      The code generation starts here.
   -->

<xsl:variable name="package">
   <xsl:call-template name="removeLastToken">
      <xsl:with-param name="_scopedName" select="$_scopedHomeImplClassname"/>
      <xsl:with-param name="_sep" select="$javaSep"/>
   </xsl:call-template>
</xsl:variable>
package <xsl:value-of select="$package"/>;

import cdmw.orbsupport.ExceptionMinorCodes;
import cdmw.ccm.cif.SessionContextExtImpl;
import cdmw.ccm.common.CCMLoggers;
import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;

public class <xsl:value-of select="$_contextImplClassname"/> extends SessionContextExtImpl
   implements <xsl:value-of select="$ccmSessionContextClassName"/> {

    /**
     * Component servant implementation
     */
    private Session<xsl:value-of select="$componentName"/>Impl component;
    
    /**
     * Logger for logging
     */
    com.thalesgroup.CdmwLogging.Logger logger;

    public <xsl:value-of select="$_contextImplClassname"/>(
        org.omg.Components.CCMContext ctx,
        Session<xsl:value-of select="$componentName"/>Impl component,
        com.thalesgroup.CdmwLogging.LoggerFactory loggerFactory) 
    {
        super(ctx, loggerFactory);
        this.logger = loggerFactory.get_logger(
            cdmw.ccm.common.CCMLoggers.CCMGENERATED_LOG_DOMAIN);
        
        CDMW_LOG_FUNCTION(logger, "");
            
        this.component = component;
    }

         <!--
            Receptacles.
         -->

         <xsl:call-template name="findAllJavaComponentReceptaclesOfNode">
            <xsl:with-param name="_componentNode" select="$_componentNode"/>
            <xsl:with-param name="_template" select="'session_context_java.content.1'"/>
         </xsl:call-template>

         <!--
            Emitters.
         -->

         <xsl:call-template name="findAllJavaComponentEmittersOfNode">
            <xsl:with-param name="_componentNode" select="$_componentNode"/>
            <xsl:with-param name="_template" select="'session_context_java.content.2'"/>
         </xsl:call-template>

         <!--
            Publishers.
         -->

         <xsl:call-template name="findAllJavaComponentPublishersOfNode">
            <xsl:with-param name="_componentNode" select="$_componentNode"/>
            <xsl:with-param name="_template" select="'session_context_java.content.2'"/>
         </xsl:call-template>

}

   <!--
      The code generation stops here.
   -->

</xsl:template> <!-- end of template session_context_java.content -->



<!--
-->
<xsl:template name="session_context_java.content.1">
   <xsl:param name="_receptacleTypeNode"/>
   <xsl:param name="_receptacleName"/>

   <xsl:variable name="scopedReturnType">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_receptacleTypeNode"/>
         <xsl:with-param name="_lang" select="'java'"/>
      </xsl:call-template>
   </xsl:variable>

   <xsl:value-of select="concat($lf, 'public ', $scopedReturnType, ' get_connection_', $_receptacleName, '() {', $lf)"/>
        CDMW_LOG_FUNCTION(logger, "");
        <xsl:value-of select="$scopedReturnType"/> recep =
            this.component.get_connection_<xsl:value-of select="$_receptacleName"/>();
        return recep;
   }
</xsl:template> <!-- end of template session_context_java.content.1 -->



<!--
-->
<xsl:template name="session_context_java.content.2">
   <xsl:param name="_eventTypeNode"/>
   <xsl:param name="_eventName"/>

   <xsl:variable name="scopedReturnType">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_eventTypeNode"/>
         <xsl:with-param name="_lang" select="'java'"/>
      </xsl:call-template>
   </xsl:variable>

   <xsl:value-of select="concat($lf, 'public void push_', $_eventName, '(', $scopedReturnType, ' event) {', $lf)"/>
      CDMW_LOG_FUNCTION(logger, "");
      // insert event into an any
      /* PCR-0049
      org.omg.CORBA.Any data = org.omg.CORBA.ORB.init().create_any();
      <xsl:value-of select="$scopedReturnType"/>Helper.insert(data, event);
      */
      cdmw.orbsupport.Codec codec = new cdmw.orbsupport.Codec();
      com.thalesgroup.CdmwCcmCif.TimestampedEvent timestamped_event = 
            new com.thalesgroup.CdmwCcmCif.TimestampedEvent();

      //NOTE: timestamped_event.timestamp is time offset from 15-th October 1582(UTC) in millis,
      //      cdmw.ccm.cif.CCMUtils.DELTA_UTC_EPOCH is difference between UTC and Epoch in millis
      timestamped_event.timestamp = (new java.util.Date()).getTime() + cdmw.ccm.cif.CCMUtils.DELTA_UTC_EPOCH;

      timestamped_event.event = event;

      org.omg.CORBA.Any data = codec.encode(timestamped_event);
      // find proxy to push event
      org.omg.CosEventComm.PushConsumer proxyConsumer =
         this.component.get_proxy_consumer("<xsl:value-of select="$_eventName"/>");

      if (proxyConsumer == null) {
          CDMW_ERROR(logger, 
             "ProxyConsumer for <xsl:value-of select="$_eventName"/> port is nil",
             new Throwable());
          org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
             ExceptionMinorCodes.INTERNAL,
             org.omg.CORBA.CompletionStatus.COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
          throw ex;
      }

      try {
          proxyConsumer.push(data);
      } catch (org.omg.CosEventComm.Disconnected d) {
          CDMW_ERROR(logger, 
             "Cannot send event with <xsl:value-of select="$_eventName"/> port since it's not connected",
             new Throwable());
          org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
             ExceptionMinorCodes.INTERNAL,
             org.omg.CORBA.CompletionStatus.COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
          throw ex;
      }
   }
</xsl:template> <!-- end of template session_context_java.content.2 -->



</xsl:stylesheet>
