<?xml version="1.0" encoding="utf-8"?>
<!-- ===================================================================== -->
<!--
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-->
<!-- ===================================================================== -->


<xsl:stylesheet version="1.1"
   xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
   xmlns:UML="org.omg/UML1.3">


<!--
   Main - This template fetches the necessary data from inputs
   to generate a file when LB.

   @param none
-->
<xsl:template name="lb_session_home_java">
   <xsl:call-template name="session_home_java">
      <xsl:with-param name="_hasLoadBalancing" select="'yes'"/>
   </xsl:call-template>
</xsl:template> <!-- end of template lb_session_home_java -->


<!--
   Main - This template fetches the necessary data from inputs
   to generate a file.

   @param none
-->
<xsl:template name="session_home_java">
   <xsl:param name="_hasLoadBalancing" select="'no'"/>
   <!--
      Parameters below are used for recursiveness.
   -->
   <xsl:param name="_index" select="1"/>

   <xsl:if test="boolean($lifecycleFrameworkNodes/home-impl) and $lifecycleFrameworkNodes/home-impl/@category = 'SESSION'">
      <xsl:if test="$_index &lt;= count($lifecycleFrameworkNodes/home-impl)">
         <xsl:variable name="homeImplNode" select="$lifecycleFrameworkNodes[home-impl and position() = $_index]/home-impl"/>

         <!--
            Check if the component is segmented
         -->
         <xsl:variable name="isSegmented">
             <xsl:choose>
                 <xsl:when test="boolean($homeImplNode/@segmented)">
                     <xsl:value-of select="$homeImplNode/@segmented"/>
                 </xsl:when>
                 <xsl:otherwise>
                     <xsl:value-of select="'yes'"/>
                 </xsl:otherwise>
             </xsl:choose>
         </xsl:variable>

         <!--
            Get the home impl class name.
         -->
         <xsl:variable name="scopedHomeImplClassname">
            <xsl:call-template name="buildHomeImplClassname">
               <xsl:with-param name="_homeImplNode" select="$homeImplNode"/>
            </xsl:call-template>
         </xsl:variable>

         <xsl:variable name="shortHomeImplClassname">
            <xsl:call-template name="getLastToken">
               <xsl:with-param name="_string" select="$scopedHomeImplClassname"/>
               <xsl:with-param name="_separator" select="$xmlSep"/>
            </xsl:call-template>
         </xsl:variable>

         <xsl:variable name="homeImplClassname">
            <xsl:choose>
               <xsl:when test="$_hasLoadBalancing = 'yes'">
                  <xsl:value-of select="concat('LB', $shortHomeImplClassname)"/>
               </xsl:when>
               <xsl:otherwise>
                  <xsl:value-of select="$shortHomeImplClassname"/>
               </xsl:otherwise>
            </xsl:choose>
         </xsl:variable>


         <!--
            Get the output filename.
         -->
         <xsl:variable name="filename">
            <xsl:call-template name="buildOutputFilename">
               <xsl:with-param name="_userDirectory" select="$homeImplNode/@directory"/>
               <xsl:with-param name="_userFilename" select="$homeImplNode/@filename"/>
               <xsl:with-param name="_defaultFilename" select="$homeImplClassname"/>
               <xsl:with-param name="_fileExtension" select="'.java'"/>
            </xsl:call-template>
         </xsl:variable>

         <!--
            Display a message to the user.
         -->
         <xsl:message>
            <xsl:value-of select="concat('Generating file ', $filename)"/>
         </xsl:message>

         <!--
            Generate the file.
         -->
         <xsl:document method="text" href="{$filename}">
            <!--
               Insert the document header.
            -->
            <xsl:call-template name="docHeader">
               <xsl:with-param name="_lang" select="'java'"/>
            </xsl:call-template>
            <!--
               Insert the content.
            -->
            <xsl:call-template name="session_home_java.content">
               <xsl:with-param name="_scopedHomeImplClassname" select="$scopedHomeImplClassname"/>
               <xsl:with-param name="_homeName">
                  <xsl:call-template name="getLastToken">
                     <xsl:with-param name="_string" select="$homeImplNode/../@factory-idref"/>
                     <xsl:with-param name="_separator" select="$xmlSep"/>
                  </xsl:call-template>
               </xsl:with-param>
               <xsl:with-param name="_isSegmented" select="$isSegmented"/>
               <xsl:with-param name="_hasLoadBalancing" select="$_hasLoadBalancing"/>
            </xsl:call-template>
         </xsl:document>

         <!--
            Proceed to the next home impl if any.
         -->
         <xsl:call-template name="session_home_java">
            <xsl:with-param name="_hasLoadBalancing" select="$_hasLoadBalancing"/>
            <xsl:with-param name="_index" select="$_index + 1"/>
         </xsl:call-template>
      </xsl:if>
   </xsl:if>
</xsl:template> <!-- end of template session_home_java -->



<!--
   This template generates the content of the file.

   @param _scopedHomeImplClassname   The scoped home impl classname.
   @param _homeName                  The home name.
-->
<xsl:template name="session_home_java.content">
   <xsl:param name="_scopedHomeImplClassname"/>
   <xsl:param name="_homeName"/>
   <xsl:param name="_isSegmented"/>
   <xsl:param name="_hasLoadBalancing"/>

   <!--
      References to some nodes.
   -->
   <xsl:variable name="homeNode" select="key('classByName', $_homeName)"/>

   <!--
      Miscellaneous variables.
   -->
   <xsl:variable name="shortHomeImplClassname">
      <xsl:call-template name="getLastToken">
         <xsl:with-param name="_string" select="$_scopedHomeImplClassname"/>
         <xsl:with-param name="_separator" select="$xmlSep"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="homeImplClassname">
      <xsl:choose>
         <xsl:when test="$_hasLoadBalancing = 'yes'">
            <xsl:value-of select="concat('LB', $shortHomeImplClassname)"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:value-of select="$shortHomeImplClassname"/>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:variable>
   <xsl:variable name="javaScopedHomeName">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$homeNode"/>
         <xsl:with-param name="_lang" select="'java'"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="javaHomeScope">
      <xsl:call-template name="removeLastToken">
         <xsl:with-param name="_scopedName" select="$javaScopedHomeName"/>
         <xsl:with-param name="_sep" select="$javaSep"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="managedComponentId">
      <xsl:variable name="ccmManagesId" select="key('stereotypeByName', 'CCMManages')/@xmi.id"/>
      <xsl:variable name="homeNodeId" select="$homeNode/@xmi.id"/>
      <xsl:variable name="managesNode" select="key('associationByStereotypeId', $ccmManagesId)[UML:Association.connection/UML:AssociationEnd[1]/@type = $homeNodeId]"/>
      <xsl:value-of select="$managesNode/UML:Association.connection/UML:AssociationEnd[2]/@type"/>
   </xsl:variable>
   <xsl:variable name="scopedManagedComponentName">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="key('classById', $managedComponentId)"/>
         <xsl:with-param name="_lang" select="'java'"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="managedComponentNameScope">
      <xsl:call-template name="removeLastToken">
         <xsl:with-param name="_scopedName" select="$scopedManagedComponentName"/>
         <xsl:with-param name="_sep" select="$javaSep"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="idlmanagedComponentNameScope">
      <xsl:call-template name="replaceCharsInString">
         <xsl:with-param name="_stringIn" select="$managedComponentNameScope"/>
         <xsl:with-param name="_charsIn" select="'$javaSep'"/>
         <xsl:with-param name="_charsOut" select="'/'"/>
      </xsl:call-template>
   </xsl:variable>
   
   <xsl:variable name="javaScopedHomeOperationsName">
      <xsl:variable name="tempName">
         <xsl:call-template name="prefixModuleNameOfNode">
            <xsl:with-param name="_prefix" select="'Cdmw_'"/>
            <xsl:with-param name="_idlPrefix" select="$idlPragmaPrefix"/>
            <xsl:with-param name="_node" select="$homeNode"/>
         </xsl:call-template>
      </xsl:variable>
      <xsl:value-of select="concat($tempName, 'Operations')"/>
   </xsl:variable>
   
   <!--
      References to some nodes.
   -->
   <xsl:variable name="managedComponentNode" select="key('classById', $managedComponentId)"/>

   <!--
      The code generation starts here.
   -->

<xsl:variable name="package">
   <xsl:call-template name="removeLastToken">
      <xsl:with-param name="_scopedName" select="$_scopedHomeImplClassname"/>
      <xsl:with-param name="_sep" select="$javaSep"/>
   </xsl:call-template>
</xsl:variable>
package <xsl:value-of select="$package"/>;

import cdmw.orbsupport.ExceptionMinorCodes;

import cdmw.ccm.cif.HomeConfigurationImpl;
import cdmw.ccm.common.CCMLoggers;
import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;

public class <xsl:value-of select="$homeImplClassname"/> 
<xsl:choose>
   <xsl:when test="$_hasLoadBalancing = 'yes'">
   extends cdmw.ccm.cif.LBCCMHomeImpl
   </xsl:when>
   <xsl:otherwise>
   extends cdmw.ccm.cif.CCMHomeImpl
   </xsl:otherwise>
</xsl:choose>
   implements <xsl:value-of select="$javaScopedHomeOperationsName"/> {
     <xsl:if test="$_isSegmented = 'yes'">         
         public class CdmwComponentTie extends <xsl:value-of select="$scopedManagedComponentName"/>POATie {
     </xsl:if>
     <xsl:if test="$_isSegmented = 'no'">         
         <xsl:variable name="newScopedComponentName">
             <xsl:call-template name="prefixModuleNameOfNode">
                 <xsl:with-param name="_prefix" select="'Cdmw_'"/>
                 <xsl:with-param name="_idlPrefix" select="$idlPragmaPrefix"/>
                 <xsl:with-param name="_node" select="$managedComponentNode"/>
             </xsl:call-template>
         </xsl:variable>
         public class CdmwComponentTie extends <xsl:value-of select="$newScopedComponentName"/>POATie {
     </xsl:if>
              public CdmwComponentTie(<xsl:value-of select="concat('Session', $managedComponentNode/@name, 'Impl')"/> delegate){
                  super(delegate);
              }
              
              public org.omg.CORBA.Object _get_component(){
                  return ((<xsl:value-of select="concat('Session', $managedComponentNode/@name, 'Impl')"/>)_delegate())._get_component();
              }  
         }                    
                
         <xsl:variable name="cdmwHomeRepId">
            <xsl:call-template name="replaceCharsInString">
               <xsl:with-param name="_stringIn" select="$homeNode/UML:ModelElement.taggedValue/UML:TaggedValue[@tag =    'typeid']/@value"/>
               <xsl:with-param name="_charsIn" select="$javaHomeScope"/>
               <xsl:with-param name="_charsOut" select="concat('Cdmw_', $javaHomeScope)"/>
            </xsl:call-template>
         </xsl:variable>
         public static final String HOME_REP_ID = "<xsl:value-of select="$cdmwHomeRepId"/>";
         
      <xsl:if test="$_isSegmented = 'yes'">         
         public static final String COMP_REP_ID = "<xsl:value-of  select="$managedComponentNode/UML:ModelElement.taggedValue/UML:TaggedValue[@tag = 'typeid']/@value"/>";
      </xsl:if>
      <xsl:if test="$_isSegmented = 'no'">         
         <xsl:variable name="newScopedComponentName">
             <xsl:call-template name="prefixModuleNameOfNode">
                 <xsl:with-param name="_prefix" select="'Cdmw_'"/>
                 <xsl:with-param name="_idlPrefix" select="$idlPragmaPrefix"/>
                 <xsl:with-param name="_node" select="$managedComponentNode"/>
             </xsl:call-template>
         </xsl:variable>
         <xsl:variable name="javaIdlPrefix">
             <xsl:call-template name="reverseList">
                 <xsl:with-param name="_list" select="$idlPragmaPrefix"/>
                 <xsl:with-param name="_separator" select="$javaSep"/>
             </xsl:call-template>
         </xsl:variable>
         <xsl:variable name="newScopedComponentNameWithoutPrefix">
             <xsl:value-of select="substring-after($newScopedComponentName, concat($javaIdlPrefix, $javaSep))"/>
         </xsl:variable>
         <xsl:variable name="newScopedComponentRepidWithoutPrefix">
             <xsl:call-template name="replaceCharsInString">
                 <xsl:with-param name="_stringIn" select="$newScopedComponentNameWithoutPrefix"/>
                 <xsl:with-param name="_charsIn" select="$javaSep"/>
                 <xsl:with-param name="_charsOut" select="'/'"/>
             </xsl:call-template>
         </xsl:variable>
         public static final String COMP_REP_ID = "IDL:thalesgroup.com/<xsl:value-of select="concat($newScopedComponentRepidWithoutPrefix, ':1.0')"/>";
      </xsl:if>

         public static final String[] ids = {
            <xsl:call-template name="getObjectList">
               <xsl:with-param name="_node" select="$managedComponentNode"/>
            </xsl:call-template>
            "IDL:omg.org/Components/Events:1.0",
            "IDL:omg.org/Components/Receptacles:1.0",
            "IDL:omg.org/Components/Navigation:1.0",
            "IDL:omg.org/Components/CCMObject:1.0"
         };

         /**
          * private class used to delegate methods from HomeConfigurationImpl
          */
         private class HomeConfigurationImplDelegate extends HomeConfigurationImpl {
            
            public org.omg.Components.IRObject get_component_def() { return null;}
            
            public org.omg.Components.IRObject get_home_def() { return null;}
            
            public void remove_component(org.omg.Components.CCMObject comp) 
               throws org.omg.Components.RemoveFailure {}
         
         }
         
         /**
          * delegate instance
          */
         private HomeConfigurationImplDelegate delegate = new HomeConfigurationImplDelegate();

         /**
          * Home executor
          */
         private <xsl:value-of select="concat($javaHomeScope, $javaSep, 'CCM_', $_homeName)"/> homeExecutor;
         
         /** 
          * Counter for names of component without COMPONENT_NAME ConfigValue.
          */
         private static int counter = 0;
         
         /**
          * Constructor
          */
         protected <xsl:value-of select="$homeImplClassname"/>(
            org.omg.Components.HomeExecutorBase homeExec,
            cdmw.lifecycle.ObjectDeactivator deactivator,
            com.thalesgroup.CdmwLifeCycle.ObjectRegistration objectRegistration,
            org.omg.Components.ConfigValue[] config) 
         {
            super(HOME_REP_ID, COMP_REP_ID, deactivator, objectRegistration, config);
            
            CDMW_LOG_FUNCTION(CCMLoggers.GetCcmgeneratedLogger(), ""); 
            
            try {
                this.homeExecutor = <xsl:value-of select="concat($javaHomeScope, $javaSep, 'CCM_', $_homeName)"/>Helper.narrow(homeExec);
            } catch (org.omg.CORBA.BAD_PARAM e) {
                CDMW_ERROR(CCMLoggers.GetCcmgeneratedLogger(), 
                    "Executor for <xsl:value-of select="$_homeName"/> is not a <xsl:value-of select="concat($javaHomeScope, $javaSep, 'CCM_', $_homeName)"/>",
                    new Throwable()); 
                org.omg.CORBA.BAD_PARAM ex = new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMInvalidHomeExecutor, 
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmgeneratedLogger(), ex);
                throw ex;
            }
            if (this.homeExecutor == null) {
                CDMW_ERROR(CCMLoggers.GetCcmgeneratedLogger(), 
                    "Executor for <xsl:value-of select="$_homeName"/> is empty",
                    new Throwable()); 
                org.omg.CORBA.BAD_PARAM ex = new org.omg.CORBA.BAD_PARAM(
                  ExceptionMinorCodes.BAD_PARAMInvalidHomeExecutor, 
                  org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmgeneratedLogger(), ex);
                throw ex;
            }
         }

        /**
         * Attributes
         */
       <xsl:call-template name="findAllJavaInterfaceReadWriteAttributes">
          <xsl:with-param name="_interfaceNode" select="$homeNode"/>
          <xsl:with-param name="_type" select="'Home'"/>
          <xsl:with-param name="_template" select="'session_home_java.content.2'"/>
          <xsl:with-param name="_strategy" select="'Session'"/>
       </xsl:call-template>

         /**
          * Explicit interface
          */

         <xsl:call-template name="findAllJavaHomeFactoryOperations">
            <xsl:with-param name="_homeNode" select="$homeNode"/>
            <xsl:with-param name="_template" select="'session_home_java.content.1'"/>
         </xsl:call-template>

         /**
          * Implicit interface
          */

         public <xsl:value-of select="$scopedManagedComponentName"/> create()
            throws org.omg.Components.CreateFailure 
         {
            CDMW_LOG_FUNCTION(CCMLoggers.GetCcmgeneratedLogger(), ""); 
            checkIsRemoved(oid);
            
            // Create an empty config
            org.omg.Components.ConfigValue[] config = new org.omg.Components.ConfigValue[0];
            
            org.omg.Components.CCMObject cmp = create_component_with_config_values(config);
            <xsl:value-of select="$scopedManagedComponentName"/> result = <xsl:value-of  select="$scopedManagedComponentName"/>Helper.narrow(cmp);
            
            return result;
         }

         // Keyless home operations

         public org.omg.Components.CCMObject create_component()
            throws org.omg.Components.CreateFailure 
         {
            CDMW_LOG_FUNCTION(CCMLoggers.GetCcmgeneratedLogger(), ""); 
            checkIsRemoved(oid);
            
            // Create an empty config
            org.omg.Components.ConfigValue[] config = new org.omg.Components.ConfigValue[0];
            
            org.omg.Components.CCMObject cmp = create_component_with_config_values(config);
            <xsl:value-of select="$scopedManagedComponentName"/> result = <xsl:value-of  select="$scopedManagedComponentName"/>Helper.narrow(cmp);
            
            return result;
         }

         public org.omg.Components.CCMObject create_component_with_config_values(
                org.omg.Components.ConfigValue[] config)
            throws org.omg.Components.CreateFailure 
         {
            CDMW_LOG_FUNCTION(CCMLoggers.GetCcmgeneratedLogger(), ""); 
            checkIsRemoved(oid);
            
            org.omg.CORBA.Object obj = create_component_ref(config);
            <xsl:value-of select="$scopedManagedComponentName"/> result;
            try {
                result = <xsl:value-of  select="$scopedManagedComponentName"/>Helper.narrow(obj);
            
            } catch (org.omg.CORBA.BAD_PARAM e) {
               CDMW_ERROR(CCMLoggers.GetCcmgeneratedLogger(), 
                   "Created reference is not a <xsl:value-of select="$managedComponentNode/@name"/>!",
                   new Throwable());
               org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                  ExceptionMinorCodes.INTERNAL, 
                  org.omg.CORBA.CompletionStatus.COMPLETED_NO);
               CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmgeneratedLogger(), ex);
               throw ex;
            }

            if (this.configurator != null) {
                try {
                    // a configurator exist!
                    this.configurator.configure(result);
                } catch (org.omg.Components.WrongComponentType wct) {
                   CDMW_ERROR(CCMLoggers.GetCcmgeneratedLogger(), 
                       "Cannot configure created component: wrong component type.",
                       new Throwable());
                   org.omg.Components.CreateFailure ex = new org.omg.Components.CreateFailure(
                      com.thalesgroup.CdmwDeployment.WRONG_COMPONENT_KIND.value);
                   CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmgeneratedLogger(), ex);
                   throw ex;
                }
            }

            return result;
         }
         
   <xsl:if test="$_hasLoadBalancing = 'yes'">
         <![CDATA[
         /**
          * Notify supervision that a component reference has been added to a 
          * LB group. Code is in generated Home servant inheriting from this class.
          *
          * @param component_name The component's name
          */
         protected void notify_add_member(String component_name)
         {
             CDMW_LOG_FUNCTION(CCMLoggers.GetCcmgeneratedLogger(), 
                 "component_name=" + component_name);
             CDMW_DEBUG(CCMLoggers.GetCcmgeneratedLogger(), 
                 "notify_add_member for component " + component_name,
                 new Throwable());
             try {
               com.thalesgroup.CdmwCcmContainer.CCM2Context ccm2context = 
                 this.context.get_ccm_context();
               
               // get ManagedComponentContextFactory object
               org.omg.CORBA.Object obj = 
                   ccm2context.resolve_initial_references(
                       cdmw.platformmngt.PlatformInterface.MANAGED_COMPONENT_CONTEXT_FACTORY_NAME);
               com.thalesgroup.CdmwPlatformMngt.ManagedComponentContextFactory mcContextFactory =
                   com.thalesgroup.CdmwPlatformMngt.ManagedComponentContextFactoryHelper.narrow(obj);
                   
               // get ComponentContext object
               com.thalesgroup.CdmwPlatformMngt.ManagedElementContexts.ComponentContextExt mcContext =
                   mcContextFactory.find_managed_component_context(component_name);
               
               // get component element path
               com.thalesgroup.CdmwPlatformMngt.ManagedElement element = 
                   mcContext.the_managed_element();
               // In case the component_name was not found                  
               if (element == null)
               {
                   CDMW_WARN(CCMLoggers.GetCcmgeneratedLogger(),
                       "\"" + component_name + "\" is not a managed component !",
                       new Throwable());
                   return;
               }
               String element_path = element.element_path();
                   
               // create PropertyChange EventHeader
               com.thalesgroup.CdmwPlatformMngtBase.EventHeader header = 
                   cdmw.platformmngt.PlatformInterface.createHeader(
                       com.thalesgroup.CdmwPlatformMngtBase.INF.value);
               // create PropertyChange PropertyInfos
               com.thalesgroup.CdmwPlatformMngt.PropertyInfo[] p_infos = 
                   new com.thalesgroup.CdmwPlatformMngt.PropertyInfo[1];
               // create info
               org.omg.CORBA.Any old_value = org.omg.CORBA.ORB.init().create_any();
               com.thalesgroup.CdmwPlatformMngt.RedundancyEltStateHelper.insert(
                  old_value, 
                  com.thalesgroup.CdmwPlatformMngt.RedundancyEltState.REDSTATE_NORG);
               org.omg.CORBA.Any new_value = org.omg.CORBA.ORB.init().create_any();            
               com.thalesgroup.CdmwPlatformMngt.RedundancyEltStateHelper.insert(
                  new_value, 
                  com.thalesgroup.CdmwPlatformMngt.RedundancyEltState.REDSTATE_ACTIVE);
               p_infos[0] = new com.thalesgroup.CdmwPlatformMngt.PropertyInfo(
                  "ReplicationRole",
                  new_value,
                  old_value);
               // create PropertyChange
               cdmw.platformmngt.PropertyChangeFactoryImpl factory = 
                   new cdmw.platformmngt.PropertyChangeFactoryImpl();
               com.thalesgroup.CdmwPlatformMngt.PropertyChange prop_chg =
                   factory.create(header, 
                                  element_path,
                                  p_infos,
                                  "notify_add_member");
                   
               // notify property change
               CDMW_DEBUG(CCMLoggers.GetCcmgeneratedLogger(), 
                   "notify_property_change '" + p_infos[0].property_name
                   + "' for element " + element_path,
                   new Throwable());
               mcContext.notify_property_change(prop_chg);
               
             } catch (com.thalesgroup.CdmwCcmContainer.CCM2ContextPackage.InvalidName e) {
               // Cannot find ManagedComponentContextFactory object
               CDMW_WARN(CCMLoggers.GetCcmgeneratedLogger(), 
                   "Cannot find service ManagedComponentContextFactory. " + 
                   "Cannot notify Supervision of add_member for " + component_name,
                   new Throwable());
               // continue anyway
             } catch (com.thalesgroup.CdmwPlatformMngt.InvalidName e) {
               // Cannot find ManagedComponentContext for component_name
               CDMW_WARN(CCMLoggers.GetCcmgeneratedLogger(), 
                   "Cannot find ManagedComponentContext for " + component_name +
                   ". Cannot notify Supervision of add_member for " + component_name,
                   new Throwable());
               // continue anyway
             } catch (org.omg.CORBA.SystemException e) {
               CDMW_WARN(CCMLoggers.GetCcmgeneratedLogger(), 
                   "SystemException received notifying add_member for " + 
                   component_name + " : " + e,
                   new Throwable());
               // continue anyway
             }
         }
         ]]>
   </xsl:if>
   
         <![CDATA[
            /**
             * This method acts as a hand shaking between the Servant Provider
             * and the generated Factory implementation. It indicates whether
             * a Servant provided by this Servant Provider may incarnate multiple
             * objects (with ObjectIds provided by this Servant Provider)
             *
             * @return <B>true</B> if a servant may incarnate multiple objects,
             * <B>false</B> otherwise.
             */
            public boolean mayIncarnateMultipleObjects() {
               CDMW_LOG_FUNCTION(CCMLoggers.GetCcmgeneratedLogger(), ""); 
               return false;
            }

            public void remove() throws org.omg.CosLifeCycle.NotRemovable {
               CDMW_LOG_FUNCTION(CCMLoggers.GetCcmgeneratedLogger(), ""); 
               this.homeExecutor = null;
               super.remove();
            }
         ]]>

         //
         // ServantProvider abstract operation implementation
         //

         <![CDATA[
            /**
             * This returns an array containing repository IDs of all objects
             * supported by the current ServantProvider.
             * <B>NOTA:</B> Repository IDs should be statically allocated; thus derived classes
             * should not delete return pointer(s).
             */
            protected String[] getIds() {
               CDMW_LOG_FUNCTION(CCMLoggers.GetCcmgeneratedLogger(), ""); 
               return ids;
            }
         ]]>

         <![CDATA[
            /**
             * This returns the number of repository Ids supported by the current ServantProvider.
             * <B>NOTA:</B> This should correspond to the number of null terminated strings
             * returned by getIds() operation.
             */
            protected int getNumberOfIds() {
               CDMW_LOG_FUNCTION(CCMLoggers.GetCcmgeneratedLogger(), ""); 
               CDMW_LOG_FUNCTION_RETURN(CCMLoggers.GetCcmgeneratedLogger(), ids.length); 
               return ids.length;
            }
         ]]>

         /**
          * This creates new component servant.
          * This shall be implemented by the well typed home servant.
          *
          * @param ctx      The component context
          * @param executor The component executor
          *
          * @return The new component servant.
          * @exception Components::CreateFailure if a problem occured when creating executor.
          */
         protected org.omg.PortableServer.Servant create_component_servant(
            String compOid,
            cdmw.ccm.cif.Context ctx,
            ComponentInfo compInfo)
            throws org.omg.Components.CreateFailure 
        {
               CDMW_LOG_FUNCTION(CCMLoggers.GetCcmgeneratedLogger(), 
                   "compOid="+compOid); 
               checkIsRemoved(oid);
         
              // get COMPONENT_NAME from ConfigValues
              String componentName = null;
              try {
                  org.omg.Components.ConfigValue[] configValues = 
                      ctx.getConfigValues(compOid);
                  for(org.omg.Components.ConfigValue c : configValues) {
                      if(com.thalesgroup.CdmwDeployment.COMPONENT_NAME.value.equals(c.name)) 
                      {
                          // extract COMPONENT_NAME
                          componentName = c.value.extract_string();
                      }
                  }
                  
              } catch (cdmw.common.NotFoundException e) {
                  // No config values for this component.
              }

              // create LoggerFactory with component name
              if (componentName == null) {
                  componentName = <xsl:value-of select="concat('&quot;', $managedComponentNode/@name, '&quot;')"/>
                      + "#" + (++counter);
              }
              com.thalesgroup.CdmwLogging.LoggerFactory loggerFactory =
                  cdmw.ccm.cif.CCMUtils.CreateLoggerFactory(componentName);

                  
               // Check if an executor has been created at ref creation
               // that is a factory operation has been used to create ref.
               org.omg.Components.EnterpriseComponent compExec = compInfo.compExec;
               boolean isCreatedByFactoryOperation = false;
         
               if (compExec == null) {
                  // Create executor
                  try {
                      CDMW_INFO(CCMLoggers.GetCcmgeneratedLogger(),
                          "Create component executor " + componentName,
                          new Throwable());
                  
                     compExec = this.homeExecutor.create();
                  } catch (org.omg.Components.CCMException ccme) {
                     CDMW_ERROR(CCMLoggers.GetCcmgeneratedLogger(), 
                        "Error occured during component executor creation!",
                        new Throwable());
                     org.omg.Components.CreateFailure ex = 
                        new org.omg.Components.CreateFailure(
                            com.thalesgroup.CdmwDeployment.CREATE_FAILURE.value);
                     CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmgeneratedLogger(), ex);
                     throw ex;
                  }
                  
                  if (compExec == null) {
                     CDMW_ERROR(CCMLoggers.GetCcmgeneratedLogger(), 
                        "Component executor is nil!",
                        new Throwable());
                     org.omg.Components.CreateFailure ex = 
                        new org.omg.Components.CreateFailure(
                            com.thalesgroup.CdmwDeployment.INVALID_COMPONENT_EXECUTOR.value);
                     CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmgeneratedLogger(), ex);
                     throw ex;
                  }
               } else {
                  isCreatedByFactoryOperation = true;
               }

              CDMW_INFO(CCMLoggers.GetCcmgeneratedLogger(),
                  "Create component servant " + componentName,
                  new Throwable());
               
              try {
                  <xsl:value-of select="concat('Session', $managedComponentNode/@name, 'Impl')"/> delegate =
                      new <xsl:value-of select="concat('Session', $managedComponentNode/@name, 'Impl')"/>(
                          compOid, 
                          ctx, 
                          compInfo.compRef, 
                          compExec, 
                          objectRegistration,
                          loggerFactory,
                          isCreatedByFactoryOperation);
                          
                      return new CdmwComponentTie(delegate);    
                          
              } catch(org.omg.Components.CCMException ccme) {
                  CDMW_ERROR(CCMLoggers.GetCcmgeneratedLogger(),
                      "CCMException occured during component executor creation: "+ccme,
                      new Throwable());
                  org.omg.Components.CreateFailure ex =
                      new org.omg.Components.CreateFailure(
                          com.thalesgroup.CdmwDeployment.CREATE_FAILURE.value);
                  CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmgeneratedLogger(), ex);
                  throw ex;
              } catch(org.omg.CORBA.SystemException e) {
                  CDMW_ERROR(CCMLoggers.GetCcmgeneratedLogger(),
                      "SystemException occured during component executor creation: "+e,
                      new Throwable());
                  CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmgeneratedLogger(), e);
                  throw e;
              }
         
         }
         
         protected cdmw.ccm.cif.CCMObjectImpl getCCMObjectImpl(
             org.omg.PortableServer.Servant servant) 
         {
             CDMW_LOG_FUNCTION(CCMLoggers.GetCcmgeneratedLogger(), "");
             try {
                 <xsl:if test="$_isSegmented = 'yes'">         
                     <xsl:value-of select="$scopedManagedComponentName"/>POATie tie =
                         (<xsl:value-of select="$scopedManagedComponentName"/>POATie) servant;
                 </xsl:if>
                 <xsl:if test="$_isSegmented = 'no'">         
                     <xsl:variable name="newScopedComponentName">
                         <xsl:call-template name="prefixModuleNameOfNode">
                             <xsl:with-param name="_prefix" select="'Cdmw_'"/>
                             <xsl:with-param name="_idlPrefix" select="$idlPragmaPrefix"/>
                             <xsl:with-param name="_node" select="$managedComponentNode"/>
                         </xsl:call-template>
                     </xsl:variable>
                     <xsl:value-of select="$newScopedComponentName"/>POATie tie =
                         (<xsl:value-of select="$newScopedComponentName"/>POATie) servant;
                 </xsl:if>
             
                 return (<xsl:value-of select="concat('Session', $managedComponentNode/@name, 'Impl')"/>) tie._delegate();
             } catch(ClassCastException cce) {
                 return null;
             }

         }

         public static org.omg.PortableServer.Servant allocate(
            org.omg.Components.HomeExecutorBase homeExecutor,
            cdmw.lifecycle.ObjectDeactivator deactivator,
            com.thalesgroup.CdmwLifeCycle.ObjectRegistration objectRegistration,
            org.omg.Components.ConfigValue[] config)
         {
           CDMW_LOG_FUNCTION(CCMLoggers.GetCcmgeneratedLogger(), "");
            <xsl:value-of select="$homeImplClassname"/> delegate = 
                new <xsl:value-of select="$homeImplClassname"/>(homeExecutor, deactivator, objectRegistration, config);
            <xsl:variable name="javaScopedHomePOATieName">
               <xsl:variable name="tempName">
                  <xsl:call-template name="prefixModuleNameOfNode">
                     <xsl:with-param name="_prefix" select="'Cdmw_'"/>
                     <xsl:with-param name="_idlPrefix" select="$idlPragmaPrefix"/>
                     <xsl:with-param name="_node" select="$homeNode"/>
                  </xsl:call-template>
               </xsl:variable>
               <xsl:value-of select="concat($tempName, 'POATie')"/>
            </xsl:variable>
            return new <xsl:value-of select="$javaScopedHomePOATieName"/>(delegate);
         }
         
         public static org.omg.PortableServer.Servant getDelegate(
            org.omg.PortableServer.Servant servant) 
         {
            CDMW_LOG_FUNCTION(CCMLoggers.GetCcmgeneratedLogger(), "");
            try {
                <xsl:value-of select="$javaScopedHomePOATieName"/> tie =
                    (<xsl:value-of select="$javaScopedHomePOATieName"/>) servant;
                return (<xsl:value-of select="$homeImplClassname"/>) tie._delegate();
            } catch(ClassCastException cce) {
                return null;
            }
            
         }
         
         //
         // methods from HomeConfigurationImpl
         //
         
         protected org.omg.Components.Configurator configurator = null;
         
         public void set_configurator(org.omg.Components.Configurator cfg) {
            CDMW_LOG_FUNCTION(CCMLoggers.GetCcmgeneratedLogger(), "");
            this.delegate.set_configurator(cfg);
            this.configurator = cfg;
         }
         
         public void set_configuration_values(org.omg.Components.ConfigValue[] config) {
            CDMW_LOG_FUNCTION(CCMLoggers.GetCcmgeneratedLogger(), "");
            this.delegate.set_configuration_values(config);
         }
         
         public void complete_component_configuration(boolean b) {
            CDMW_LOG_FUNCTION(CCMLoggers.GetCcmgeneratedLogger(), "b="+b);
            this.delegate.complete_component_configuration(b);
         }
         
         public void disable_home_configuration() {
            CDMW_LOG_FUNCTION(CCMLoggers.GetCcmgeneratedLogger(), "");
            this.delegate.disable_home_configuration();
         }
         
         public org.omg.Components.Configurator getConfigurator() {
            CDMW_LOG_FUNCTION(CCMLoggers.GetCcmgeneratedLogger(), "");
            return this.configurator;
         }
         
         public org.omg.Components.ConfigValue[] getConfigValues() {
            CDMW_LOG_FUNCTION(CCMLoggers.GetCcmgeneratedLogger(), "");
            return this.delegate.getConfigValues();
         }
         
         public boolean isCompleteCompConfEnable() {
            CDMW_LOG_FUNCTION(CCMLoggers.GetCcmgeneratedLogger(), "");
            return this.delegate.isCompleteCompConfEnable();
         }

}

   <!--
      The code generation stops here.
   -->

</xsl:template> <!-- end of template session_home_java.content -->



<!--
-->
<xsl:template name="session_home_java.content.1">
   <xsl:param name="_node"/>
   <xsl:param name="_returnType"/>
   <xsl:param name="_operationName"/>
   <xsl:param name="_homeImplClassname"/>
   <xsl:param name="_managedComponentName"/>
   <xsl:param name="_scopedManagedComponentName"/>

   // Factory operation

   <xsl:value-of select="concat($lf, 'public ', $_returnType, $lf, $_operationName, '(', $lf)"/>
         <!--
            IN, OUT and INOUT parameters.
         -->
         <xsl:for-each select="$_node/UML:BehavioralFeature.parameter/UML:Parameter[@kind != 'return']">
            <!--
               Get the parameter type.
            -->
            <xsl:variable name="paramType">
               <xsl:call-template name="getParameterType">
                  <xsl:with-param name="_paramNode" select="."/>
                  <xsl:with-param name="_lang" select="'java'"/>
               </xsl:call-template>
            </xsl:variable>
            <xsl:value-of select="concat($paramType, ' ', @name)"/>
            <!--
               Write out comma if necessary.
            -->
            <xsl:if test="position() != last()">
               <xsl:value-of select="concat(',', $lf)"/>
            </xsl:if>
         </xsl:for-each>
      ) throws org.omg.Components.CreateFailure {
         CDMW_LOG_FUNCTION(CCMLoggers.GetCcmgeneratedLogger(), "");
         // Create executor
         org.omg.Components.EnterpriseComponent compExec = null;
         
         <!--try {-->
            compExec = this.homeExecutor.<xsl:value-of select="$_operationName"/> (
               <xsl:for-each select="$_node/UML:BehavioralFeature.parameter/UML:Parameter[@kind != 'return']">
                  <xsl:value-of select="@name"/>
                  <xsl:if test="position() != last()">
                     <xsl:text>, </xsl:text>
                  </xsl:if>
               </xsl:for-each>
               );
         <!-- //TODO: remove this catch statement if not needed
         } catch (org.omg.Components.CCMException ccme) {
            System.err.println("Error occured during component executor creation!");
            throw ccme;
         }
         -->
          
         if (compExec == null) {
            CDMW_ERROR(CCMLoggers.GetCcmgeneratedLogger(), 
                "Component executor is null!",
                new Throwable());
            org.omg.Components.CreateFailure ex =
                new org.omg.Components.CreateFailure(
                    com.thalesgroup.CdmwDeployment.INVALID_COMPONENT_EXECUTOR.value);
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmgeneratedLogger(), ex);
            throw ex;
         }
          
         // Create an empty config
         org.omg.Components.ConfigValue[] config = new org.omg.Components.ConfigValue[0];
         org.omg.CORBA.Object obj = create_component_ref(config, compExec);
         <xsl:value-of select="$_scopedManagedComponentName"/> result;
         try {
            result = <xsl:value-of select="$_scopedManagedComponentName"/>Helper.narrow(obj);
          
         } catch (org.omg.CORBA.BAD_PARAM e) {
            CDMW_ERROR(CCMLoggers.GetCcmgeneratedLogger(), 
                "Created reference is not a <xsl:value-of select="$_managedComponentName"/>!",
                new Throwable());
            org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
               ExceptionMinorCodes.INTERNAL, 
               org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmgeneratedLogger(), ex);
            throw ex;
         }

         return result;
      }
</xsl:template> <!-- end of template session_home_java.content.1 -->


<!--
-->
<xsl:template name="session_home_java.content.2">
   <xsl:param name="_node"/>
   <xsl:param name="_readonlyId"/>
   <xsl:param name="_readonlyEndId"/>
   <xsl:param name="_paramName"/>
   <xsl:param name="_paramType"/>
   <xsl:param name="_returnType"/>
   <xsl:param name="_javaOperationName"/>
   <xsl:param name="_homeObjRefVar"/>
   <xsl:param name="_objRefVarType"/>

   <xsl:if test="not(boolean($_node/@stereotype)) or ($_node/@stereotype = $_readonlyEndId) or ($_node/@stereotype = $_readonlyId)">
      <xsl:value-of select="concat($lf, 'public ', $_returnType, ' ', $_javaOperationName, '() {', $lf)"/>
         CDMW_LOG_FUNCTION(CCMLoggers.GetCcmgeneratedLogger(), "");
         <xsl:value-of select="$_objRefVarType"/> result = homeExecutor.<xsl:value-of select="$_javaOperationName"/>();

         return result;
      }

      <xsl:if test="not($_node/@stereotype = $_readonlyEndId or $_node/@stereotype = $_readonlyId)">
         <xsl:value-of select="concat($lf, 'public void ', $_javaOperationName, '(', $_paramType, ' ', $_paramName, ') {', $lf)"/>
             CDMW_LOG_FUNCTION(CCMLoggers.GetCcmgeneratedLogger(), "");
             homeExecutor.<xsl:value-of select="concat($_javaOperationName, '(', $_paramName, ');')"/>
         }
      </xsl:if>
   </xsl:if>
   
</xsl:template> <!-- end of template session_home_java.content.2 -->



</xsl:stylesheet>


