/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.cif;

public abstract class CCMObjectState
{

    /**
     * State transition performed when configuration_complete() is called
     * on CCMObject.
     *
     * @param ccmObject The CCMObject with this state, for possible actions
     * to be performed during state transition.
     * @return the new CCMObject state
     * @exception org.omg.Components.CCMException any CCMexception raised by 
     * actions during state transition.
     */
    public abstract CCMObjectState configuration_complete(CCMObjectImpl ccmObject)
        throws org.omg.Components.CCMException;
    
    /**
     * State transition performed when remove() is called 
     * on CCMObject
     *
     * @param ccmObject The CCMObject with this state, for possible actions
     * to be performed during state transition.
     * @return the new CCMObject state
     * @exception org.omg.Components.CCMException any CCMexception raised by 
     * actions during state transition.
     */
    public abstract CCMObjectState remove(CCMObjectImpl ccmObject)
        throws org.omg.Components.CCMException;
        
    /**
     * State transition performed when a specific activation is called 
     * on CCMObject (e.g. fault tolerance activation).
     *
     * @param ccmObject The CCMObject with this state, for possible actions
     * to be performed during state transition.
     * @return the new CCMObject state
     * @exception org.omg.Components.CCMException any CCMexception raised by 
     * actions during state transition.
     */
    public abstract CCMObjectState activate(CCMObjectImpl ccmObject)
        throws org.omg.Components.CCMException;
    
    /**
     * State transition performed when a specific deactivation is called 
     * on CCMObject (e.g. fault tolerance deactivation).
     *
     * @param ccmObject The CCMObject with this state, for possible actions
     * to be performed during state transition.
     * @return the new CCMObject state
     * @exception org.omg.Components.CCMException any CCMexception raised by 
     * actions during state transition.
     */
    public abstract CCMObjectState deactivate(CCMObjectImpl ccmObject)
        throws org.omg.Components.CCMException;
        
    /**
     * State transition performed when an operational_request is called 
     * on CCMObject (e.g. a call on a facet).
     *
     * @param ccm_object The CCMObject with this state, for possible actions
     * to be performed during state transition.
     * @return the new CCMObject state
     */
    public abstract CCMObjectState operationalRequest(CCMObjectImpl ccmObject);
}

