/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.cif;

import cdmw.orbsupport.ExceptionMinorCodes;
import cdmw.ccm.common.CCMLoggers;
import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;

//
// IDL:omg.org/Components/HomeConfiguration:1.0
//
public class PushConsumerImpl
    extends com.thalesgroup.CdmwCcmCif.PushConsumerPOA 
    implements cdmw.ccm.common.DisposableObject {

    /**
     * non cdmw consumer
     */
    private org.omg.Components.EventConsumerBase ccmConsumer;

    /**
     * LifeCycleObjectBase object used for delegation
     */
    private LifeCycleObjectBase lifecycleObjectBase;

    /**
     * constructor
     * .
     * @param ctx
     * @param ccmConsumer
     */
    public PushConsumerImpl(
        Context ctx,
        org.omg.Components.EventConsumerBase ccmConsumer) 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        this.lifecycleObjectBase = new LifeCycleObjectBase(ctx);
        this.ccmConsumer = ccmConsumer;
    }

    /**
     * destructor
     */
    public void dispose() {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
    }

    //
    // IDL:omg.org/CosEventComm/PushConsumer/disconnect_push_consumer:1.0
    //
    public void disconnect_push_consumer() {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        // do nothing
    }

    //
    // IDL:omg.org/CosEventComm/PushConsumer/push:1.0
    //
    public void push(org.omg.CORBA.Any data) {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");

        cdmw.orbsupport.Codec codec = new cdmw.orbsupport.Codec();
        org.omg.Components.EventBase evt;
        com.thalesgroup.CdmwCcmCif.TimestampedEvent timestamped_event;
        try {
            timestamped_event = (com.thalesgroup.CdmwCcmCif.TimestampedEvent) codec.decode(data);

        } catch (ClassCastException e) {
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                "PushConsumerImpl push a non event type!",
                new Throwable());
            org.omg.CORBA.BAD_PARAM ex = new org.omg.CORBA.BAD_PARAM(
                ExceptionMinorCodes.BAD_PARAMBadEventType,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
            throw ex;
        }

        if (timestamped_event == null) {
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                "PushConsumerImpl push nil event!",
                new Throwable());
            org.omg.CORBA.BAD_PARAM ex = new org.omg.CORBA.BAD_PARAM(
                ExceptionMinorCodes.BAD_PARAMBadEventType,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
            throw ex;
        }

        java.text.DateFormat df = new java.text.SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
        java.lang.String time_received = df.format(new java.util.Date());
        //NOTE: timestamped_event.timestamp is time offset from 15-th October 1582(UTC) in millis,
        //      cdmw.commonsvcs.time.DELTA_POSIX_DCE is difference between UTC and Epoch in millis
        java.lang.String time_issued = 
            df.format(new java.util.Date(timestamped_event.timestamp - cdmw.ccm.cif.CCMUtils.DELTA_UTC_EPOCH));

        CCMLoggers.CDMW_DEBUG_EVENT(CCMLoggers.GetCcmcifLogger(), "Event received at: " + time_received + 
            " issued at: " + time_issued);

        evt = timestamped_event.event;

        try {
            this.ccmConsumer.push_event(evt);
        } catch (org.omg.Components.BadEventType bet) {
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                "PushConsumerImpl push bad event type!",
                new Throwable());
            org.omg.CORBA.BAD_PARAM ex = new org.omg.CORBA.BAD_PARAM(
                ExceptionMinorCodes.BAD_PARAMBadEventType,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
            throw ex;
        }

    }

    //
    // IDL:omg.org/CosLifeCycle/LifeCycleObject/copy:1.0
    //
    public org.omg.CosLifeCycle.LifeCycleObject copy(
        org.omg.CosLifeCycle.FactoryFinder there,
        org.omg.CosLifeCycle.NVP[] the_criteria)
        throws
            org.omg.CosLifeCycle.NoFactory,
            org.omg.CosLifeCycle.NotCopyable,
            org.omg.CosLifeCycle.InvalidCriteria,
            org.omg.CosLifeCycle.CannotMeetCriteria 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        return this.lifecycleObjectBase.copy(there, the_criteria);
    }

    //
    // IDL:omg.org/CosLifeCycle/LifeCycleObject/move:1.0
    //
    public void move(
        org.omg.CosLifeCycle.FactoryFinder there,
        org.omg.CosLifeCycle.NVP[] the_criteria)
        throws
            org.omg.CosLifeCycle.NoFactory,
            org.omg.CosLifeCycle.NotMovable,
            org.omg.CosLifeCycle.InvalidCriteria,
            org.omg.CosLifeCycle.CannotMeetCriteria 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        this.lifecycleObjectBase.move(there, the_criteria);
    }

    //
    // IDL:omg.org/CosLifeCycle/LifeCycleObject/remove:1.0
    //
    public void remove() throws org.omg.CosLifeCycle.NotRemovable {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        this.lifecycleObjectBase.remove();
    }

}
