/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.cif;

import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;

import cdmw.orbsupport.ExceptionMinorCodes;

/**
 * This class implements the Components.SessionContext interface.
 * It will be inherited by all session components context implementation.
 *
 */
//
// IDL:omg.org/Components/SessionContext:1.0
//
public class SessionContextExtImpl extends CCMContextExtImpl 
    implements com.thalesgroup.CdmwComponents.SessionContextExt 
{

    /**
     * session context given by the container
     */
    private org.omg.Components.SessionContext sessionContext;

    /**
     * Logger for logging
     */
    private com.thalesgroup.CdmwLogging.Logger logger;
    
    
    public SessionContextExtImpl(org.omg.Components.CCMContext ctx,
                                 com.thalesgroup.CdmwLogging.LoggerFactory loggerFactory) 
    {
        super(ctx, loggerFactory);
        
        // create logger with domain for ccmcif
        this.logger = loggerFactory.get_logger(
            cdmw.ccm.common.CCMLoggers.CCMCIF_LOG_DOMAIN);
            
        CDMW_LOG_FUNCTION(logger, "");

        try {
            this.sessionContext = 
                org.omg.Components.SessionContextHelper.narrow(ctx);

        } catch (org.omg.CORBA.BAD_PARAM e) {
            CDMW_ERROR(logger, 
                "Context is not a SessionContext",
                new Throwable());
            org.omg.CORBA.BAD_PARAM ex = new org.omg.CORBA.BAD_PARAM(
                ExceptionMinorCodes.BAD_PARAMWrongContextType, 
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
    }

    //
    // IDL:omg.org/Components/SessionContext/get_CCM_object:1.0
    //
    public org.omg.CORBA.Object get_CCM_object() 
        throws org.omg.Components.IllegalState 
    {
        CDMW_LOG_FUNCTION(logger, "");
        return this.sessionContext.get_CCM_object();
    }

}

