/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "testccmcif/CCM_Client_impl.hpp"
#include "testccmcif/Event_impl.hpp"

#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/common/Assert.hpp"


#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/ui/text/TestRunner.h>
#include <cppunit/TestResult.h>
#include <cppunit/XmlOutputter.h>
#include <cppunit/TextOutputter.h>
#include <fstream>

namespace 
{
    

// Helper function for CORBA::WString creation    
CORBA::WChar* make_wstring(const char* str)
{
    if (str == 0)
        return 0;
    
    CORBA::ULong length = strlen(str);
    CORBA::WString_var ret = CORBA::wstring_alloc(length);
    for (CORBA::ULong i=0; i < length; i++)
    {
        ret[i] = (CORBA::WChar) str[i];
    }
    ret[length] = (CORBA::WChar) '\0';
    
    return ret._retn();
};

/**
*  This class provides an object to test the ccm_activate.
*
**/


class test_ccm_client_activate;
    
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(test_ccm_client_activate, "test_ccm_client_activate");

class test_ccm_client_activate : public CppUnit::TestFixture
{
  BasicDemo::CCM_Client_impl* m_client;
  
 public:
  test_ccm_client_activate()
  {
  };


  ~test_ccm_client_activate() {};

  void do_tests()
  {
     // set number of requested successfull tests
//      set_nbOfRequestedTestOK (29);
    

     TEST_INFO("********************************************");
     TEST_INFO("            ccm_activate start");
     TEST_INFO("********************************************");
     

      ::CommonDemo::Registration_var register_server;


     CORBA::Object_var obj1 = Cdmw::TestUtils::CORBATestManager::instance()->get_object("client_context");
     ::BasicDemo::CCM_Client_Context_var client_context = ::BasicDemo::CCM_Client_Context::_narrow(obj1.in());
      
     CPPUNIT_ASSERT(!CORBA::is_nil(client_context.in()));

     TEST_INFO("Get_connection_to_register...");
      
      bool test_ok = false;
      try
      {
          register_server = client_context->get_connection_to_register();

            if (!CORBA::is_nil(register_server.in()))
            {
                test_ok = true;
            }
      }
      catch (...)
      {
          TEST_INFO("Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);

      // register client
      TEST_INFO("Register client...");
      test_ok = false;
      try
      {
          if (!CORBA::is_nil(register_server.in()))
          {
              CORBA::String_var client_name = register_server->_cxx_register();
              std::cout<<client_name.in()<<std::endl;;
              test_ok = true;
         }
      }
      catch (...)
      {
            TEST_INFO("Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);
     
     //get the connection to the server 
      TEST_INFO("Connect to the display facet...");
      ::BasicDemo::Display_var display;
     test_ok = false;
      try
      {
         display = client_context->get_connection_to_server();

            if (!CORBA::is_nil(display.in()))
            {
                test_ok = true;
            }
      }
      catch (...)
      {
            TEST_INFO("Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);

     //print "hello" via the server
      TEST_INFO("Print hello...");
     test_ok = false;
      try
      {
             if (!CORBA::is_nil(register_server.in()))
            {
                display->print(Cdmw::TestUtils::CORBATestManager::instance()->get_property ("text").c_str());
                test_ok = true;
            }
      }
      catch (...)
      {
            TEST_INFO("Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);

     //print_event "hello" via the server
      TEST_INFO("Print ValueType hello...");
     test_ok = false;
      try
      {
          if (!CORBA::is_nil(register_server.in()))
          {
              BasicDemo::Event_var event = new BasicDemo::Event_impl();
              event->text("Hello ValueType!");
              
              display->print_event(event.in());
              test_ok = true;
          }
      }
      catch (...)
      {
            TEST_INFO("Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);


      TEST_INFO("test_short");
      test_ok = false;
      try
      {
          if (!CORBA::is_nil(register_server.in()))
          {
              CORBA::Short in_ = 1;
              CORBA::Short inout_;              
              CORBA::Short out_;
              CORBA::Short ret_;

              ret_ = display->test_short(in_, inout_, out_);
              if ((inout_ == (in_*2))
                  && (out_ == (in_*3))
                  && (ret_ == (in_*4)))
                  test_ok = true;
          }
      }
      catch (...)
      {
          TEST_INFO("Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok); 

      TEST_INFO("test_long");
      test_ok = false;
      try
      {
          if (!CORBA::is_nil(register_server.in()))
          {
              CORBA::Long in_ = 1;
              CORBA::Long inout_;              
              CORBA::Long out_;
              CORBA::Long ret_;

              ret_ = display->test_long(in_, inout_, out_);
              if ((inout_ == (in_*2))
                  && (out_ == (in_*3))
                  && (ret_ == (in_*4)))
                  test_ok = true;              
          }
      }
      catch (...)
      {
          TEST_INFO("Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);

      TEST_INFO("test_long_long");
      test_ok = false;
      try
      {
          if (!CORBA::is_nil(register_server.in()))
          {
              CORBA::LongLong in_ = 1;
              CORBA::LongLong inout_;              
              CORBA::LongLong out_;
              CORBA::LongLong ret_;

              ret_ = display->test_long_long(in_, inout_, out_);
              if ((inout_ == (in_*2))
                  && (out_ == (in_*3))
                  && (ret_ == (in_*4)))
                  test_ok = true;              
          }
      }
      catch (...)
      {
          TEST_INFO("Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);

      TEST_INFO("test_unsigned_short");
      test_ok = false;
      try
      {
          if (!CORBA::is_nil(register_server.in()))
          {
              CORBA::UShort in_ = 1;
              CORBA::UShort inout_;              
              CORBA::UShort out_;
              CORBA::UShort ret_;

              ret_ = display->test_unsigned_short(in_, inout_, out_);
              if ((inout_ == (in_*2))
                  && (out_ == (in_*3))
                  && (ret_ == (in_*4)))
                  test_ok = true;              
          }
      }
      catch (...)
      {
          TEST_INFO("Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);

      TEST_INFO("test_unsigned_long");
      test_ok = false;
      try
      {
          if (!CORBA::is_nil(register_server.in()))
          {
              CORBA::ULong in_ = 1;
              CORBA::ULong inout_;              
              CORBA::ULong out_;
              CORBA::ULong ret_;

              ret_ = display->test_unsigned_long(in_, inout_, out_);
              if ((inout_ == (in_*2))
                  && (out_ == (in_*3))
                  && (ret_ == (in_*4)))
                  test_ok = true;              
          }
      }
      catch (...)
      {
          TEST_INFO("Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);

      TEST_INFO("test_unsigned_long_long");
      test_ok = false;
      try
      {
          if (!CORBA::is_nil(register_server.in()))
          {
              CORBA::ULongLong in_ = 1;
              CORBA::ULongLong inout_;              
              CORBA::ULongLong out_;
              CORBA::ULongLong ret_;

              ret_ = display->test_unsigned_long_long(in_, inout_, out_);
              if ((inout_ == (in_*2))
                  && (out_ == (in_*3))
                  && (ret_ == (in_*4)))
                  test_ok = true;              
          }
      }
      catch (...)
      {
          TEST_INFO("Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);

      TEST_INFO("test_float");
      test_ok = false;
      try
      {
          if (!CORBA::is_nil(register_server.in()))
          {
              CORBA::Float in_ = 1;
              CORBA::Float inout_;              
              CORBA::Float out_;
              CORBA::Float ret_;

              ret_ = display->test_float(in_, inout_, out_);
              if ((inout_ == (in_*2))
                  && (out_ == (in_*3))
                  && (ret_ == (in_*4)))
                  test_ok = true;              
          }
      }
      catch (...)
      {
          TEST_INFO("Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);

      TEST_INFO("test_double");
      test_ok = false;
      try
      {
          if (!CORBA::is_nil(register_server.in()))
          {
              CORBA::Double in_ = 1;
              CORBA::Double inout_;              
              CORBA::Double out_;
              CORBA::Double ret_;

              ret_ = display->test_double(in_, inout_, out_);
              if ((inout_ == (in_*2))
                  && (out_ == (in_*3))
                  && (ret_ == (in_*4)))
                  test_ok = true;              
          }
      }
      catch (...)
      {
          TEST_INFO("Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);

      TEST_INFO("test_boolean");
      test_ok = false;
      try
      {
          if (!CORBA::is_nil(register_server.in()))
          {
              CORBA::Boolean in_ = 1;
              CORBA::Boolean inout_;              
              CORBA::Boolean out_;
              CORBA::Boolean ret_;

              ret_ = display->test_boolean(in_, inout_, out_);
              if ((inout_ == in_)
                  && (out_ == in_)
                  && (ret_ == in_))
                  test_ok = true;              
          }
      }
      catch (...)
      {
          TEST_INFO("Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);

      TEST_INFO("test_char");
      test_ok = false;
      try
      {
          if (!CORBA::is_nil(register_server.in()))
          {
              CORBA::Char in_ ('1');
              CORBA::Char inout_;              
              CORBA::Char out_;
              CORBA::Char ret_;

              ret_ = display->test_char(in_, inout_, out_);
              if ((inout_ == in_)
                  && (out_ == in_)
                  && (ret_ == in_))
                  test_ok = true;              
          }
      }
      catch (...)
      {
          TEST_INFO("Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);

      TEST_INFO("test_wchar");
      test_ok = false;
      try
      {
          if (!CORBA::is_nil(register_server.in()))
          {
              CORBA::WChar in_ ('1');
              CORBA::WChar inout_;              
              CORBA::WChar out_;
              CORBA::WChar ret_;

              ret_ = display->test_wchar(in_, inout_, out_);
              if ((inout_ == in_)
                  && (out_ == in_)
                  && (ret_ == in_))
                  test_ok = true;              
          }
      }
      catch (...)
      {
          TEST_INFO("Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);

      TEST_INFO("test_octet");
      test_ok = false;
      try
      {
          if (!CORBA::is_nil(register_server.in()))
          {
              CORBA::Octet in_ = 1;
              CORBA::Octet inout_;              
              CORBA::Octet out_;
              CORBA::Octet ret_;

              ret_ = display->test_octet(in_, inout_, out_);
              if ((inout_ == in_)
                  && (out_ == in_)
                  && (ret_ == in_))
                  test_ok = true;              
          }
      }
      catch (...)
      {
          TEST_INFO("Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);

      TEST_INFO("test_function");
      test_ok = false;
      try
      {
          if (!CORBA::is_nil(register_server.in()))
          {
              display->set(1);
              BasicDemo::Display_var inout_ = ::BasicDemo::Display::_duplicate (display.in());
              BasicDemo::Display_var out_;
              BasicDemo::Display_var ret_;

              ret_ = display->test_function(display.in(), inout_.inout(), out_.out());
              
              if ((inout_->get() == display->get())
                  && (out_->get() == display->get())
                  && (ret_->get() == display->get()))
                  test_ok = true;              
          }
      }
      catch (const CORBA::SystemException& ex)
      {
          std::cout<<ex<<std::endl;
      }
      catch (...)
      {
          TEST_INFO("Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);

      TEST_INFO("test_fixed_struct");
      test_ok = false;
      try
      {
          if (!CORBA::is_nil(register_server.in()))
          {
              BasicDemo::s1 in_;
              in_.l=1;
              BasicDemo::s1 inout_;              
              BasicDemo::s1 out_;
              BasicDemo::s1 ret_;

              ret_ = display->test_fixed_struct(in_, inout_, out_);
              if ((inout_.l == in_.l)
                  && (out_.l == in_.l)
                  && (ret_.l == in_.l))
                  test_ok = true;              
          }
      }
      catch (...)
      {
          TEST_INFO("Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);

      TEST_INFO("test_var_struct");
      test_ok = false;
      try
      {
          if (!CORBA::is_nil(register_server.in()))
          {
              BasicDemo::s2 in_;
              in_.s = CORBA::string_dup("1");
              BasicDemo::s2_var inout_ = new BasicDemo::s2();              
              BasicDemo::s2_var out_;
              BasicDemo::s2_var ret_;

             ret_ = display->test_var_struct(in_, inout_.inout(), out_.out());
             if (strcmp(inout_->s, in_.s) == 0 
                 && strcmp(out_->s, in_.s) == 0
                 && strcmp(ret_->s, in_.s) == 0)
                  test_ok = true;              
          }
      }
      catch (...)
      {
          TEST_INFO("Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);

      TEST_INFO("test_fixed_union");
      test_ok = false;
      try
      {
          if (!CORBA::is_nil(register_server.in()))
          {
              ::BasicDemo::Display::union1 in_;
              in_.s(1);
              ::BasicDemo::Display::union1 inout_;
              ::BasicDemo::Display::union1 out_;
              ::BasicDemo::Display::union1 ret_;
              
              ret_ = display->test_fixed_union(in_, inout_, out_);
              if ((inout_.s() == in_.s())  
                  && (out_.s() == in_.s())
                  && (ret_.s() == in_.s()))
                  test_ok = true;              
          }
      }
      catch (...)
      {
          TEST_INFO("Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);

      TEST_INFO("test_var_union");
      test_ok = false;
      try
      {
          if (!CORBA::is_nil(register_server.in()))
          {
              ::BasicDemo::Display::union2 in_;
              in_.s1("1");
              ::BasicDemo::Display::union2 inout_;              
              ::BasicDemo::Display::union2_var out_;
              ::BasicDemo::Display::union2_var ret_;
              
              ret_ = display->test_var_union(in_, inout_, out_.out());
              if (strcmp(inout_.s1(), in_.s1()) == 0  
                  && strcmp(out_->s1(), in_.s1()) == 0
                  && strcmp(ret_->s1(), in_.s1()) == 0)
                  test_ok = true;              
          }
      }
      catch (...)
      {
          TEST_INFO("Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);

      TEST_INFO("test_seq_struct");
      test_ok = false;
      try
      {
          if (!CORBA::is_nil(register_server.in()))
          {
              ::BasicDemo::s1s in_;
              in_.length(1);
              in_[0].l = 1;
              ::BasicDemo::s1s inout_;              
              ::BasicDemo::s1s_var out_;
              ::BasicDemo::s1s_var ret_;
              
              ret_ = display->test_seq_struct(in_, inout_, out_.out());
              if (inout_[0].l == in_[0].l  
                  && out_[0].l == in_[0].l
                  && ret_[0].l == in_[0].l)
                  test_ok = true;              
          }
      }
      catch (...)
      {
          TEST_INFO("Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);

      TEST_INFO("test_string");
      test_ok = false;
      try
      {
          if (!CORBA::is_nil(register_server.in()))
          {
              CORBA::String_var in_("hello");
              CORBA::String_var inout_("hella");
              CORBA::String_var out_;
              CORBA::String_var ret_;
              ret_ = display->test_string(in_.in(), inout_.inout(), out_.out());
              if (strcmp(inout_.in(), in_.in()) == 0  
                  && strcmp(out_.in(), in_.in()) == 0
                  && strcmp(ret_.in(), in_.in()) == 0)
                  test_ok = true;              

          }
      }
      catch (...)
      {
          TEST_INFO("Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);

      TEST_INFO("test_wstring");
      test_ok = false;
      try
      {
          if (!CORBA::is_nil(register_server.in()))
          {
              CORBA::WString_var in_(make_wstring("hello"));
              CORBA::WString_var inout_(make_wstring("hella"));
              CORBA::WString_var out_;
              CORBA::WString_var ret_;
              
              ret_ = display->test_wstring(in_.in(), inout_.inout(), out_.out());
              if (strcmp((char *)inout_.in(), (char *)in_.in()) == 0  
                  && strcmp((char *)out_.in(), (char *)in_.in()) == 0
                  && strcmp((char *)ret_.in(), (char *)in_.in()) == 0)
                  test_ok = true;              
          }
      }
      catch (...)
      {
          TEST_INFO("Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);

      TEST_INFO("test_fixed_array");
      test_ok = false;
      try
      {
          if (!CORBA::is_nil(register_server.in()))
          {
              ::BasicDemo::Display::array1 in_;
              for (int i =0 ; i<5; i++)
                  in_[i] = i;
              ::BasicDemo::Display::array1 inout_;              
              for (int i =0 ; i<5; i++)
                  inout_[i] = i + 5;
              ::BasicDemo::Display::array1 out_;
              ::BasicDemo::Display::array1_slice * ret_;
              
              ret_ = display->test_fixed_array(in_, inout_, out_);
              if (inout_[0] == in_[0]  
                  && out_[0] == in_[0]
                  && ret_[0] == in_[0])
                  test_ok = true;
              
              delete [] ret_;
          }
      }
      catch (...)
      {
          TEST_INFO("Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);

      TEST_INFO("test_var_array");
      test_ok = false;
      try
      {
          if (!CORBA::is_nil(register_server.in()))
          {
              ::BasicDemo::Display::array2 in_;
              for (int i=0 ; i<5; i++)
                  in_[i] = CORBA::string_dup("hello");
              ::BasicDemo::Display::array2 inout_;              
              for (int i=0 ; i<5; i++)
                  inout_[i] = CORBA::string_dup("hella");
              ::BasicDemo::Display::array2_slice * out_;
              ::BasicDemo::Display::array2_slice * ret_;
              ret_ = display->test_var_array(in_, inout_, out_);
              if (strcmp(inout_[0], in_[0]) == 0  
                  && strcmp(out_[0], in_[0]) == 0
                  && strcmp(ret_[0], in_[0])==0 )
                  test_ok = true;    
                  
              delete [] out_;
              delete [] ret_;
          }
      }
      catch (...)
      {
          TEST_INFO("Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);

      TEST_INFO("test_any");
      test_ok = false;
      try
      {
          if (!CORBA::is_nil(register_server.in()))
          {
              CORBA::Any in_;
              CORBA::Short s1 = 2;
              in_ <<= s1;

              CORBA::Any inout_;
              CORBA::Short s2 = 3;
              inout_ <<= s2;

              CORBA::Any_var out_;
              CORBA::Any_var ret_;
              
              ret_ = display->test_any(in_, inout_, out_);
              CORBA::Short short_in, short_inout, short_out, short_ret;
              if ((in_ >>= short_in) &&
                  (inout_ >>= short_inout) &&
                  (out_.in () >>= short_out) &&
                  (ret_.in () >>= short_ret))
              {
                  if (short_ret == short_in
                      && short_inout == short_in
                      && short_out == short_in)
                      test_ok = true;              
              }
          }
      }
      catch (...)
      {
          TEST_INFO("Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);

      TEST_INFO("test_value_type");
      test_ok = false;
      try
      {
          if (!CORBA::is_nil(register_server.in()))
          {
              BasicDemo::Event_var in_ = new BasicDemo::Event_impl();
              BasicDemo::Event_var inout_ = new BasicDemo::Event_impl();
              BasicDemo::Event_var out_;
              BasicDemo::Event_var ret_;

              in_->text("Hello");
              inout_->text("Hella");

              ret_ = display->test_event(in_.in(), inout_.inout(), out_.out());

              if (strcmp(in_->text(), inout_->text()) == 0
                  && strcmp(in_->text(), out_->text()) == 0 
                  && strcmp(in_->text(), ret_->text()) == 0)
                  test_ok = true;
          }
      }
      catch (...)
      {
          TEST_INFO("Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);
      TEST_INFO("********************************************");
      TEST_INFO("            ccm_activate stop");
      TEST_INFO("********************************************");
      
   };

    CPPUNIT_TEST_SUITE( test_ccm_client_activate );
    CPPUNIT_TEST( do_tests );
    CPPUNIT_TEST_SUITE_END();



}; // End of class test_ccm_client_activate
    
} // End of anonymous namespace



//
// IDL:thalesgroup.com/BasicDemo/CCM_Client:1.0
//
BasicDemo::CCM_Client_impl::CCM_Client_impl()
    : m_name(""),
      m_session_context(BasicDemo::CCM_Client_Context::_nil())
{
}

BasicDemo::CCM_Client_impl::~CCM_Client_impl()
{
    std::cout << "CCM_Client_impl::~CCM_Client_impl called!" << std::endl;
}


//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
BasicDemo::CCM_Client_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    std::cout << "Set_session_context called!" << std::endl;
   
    m_session_context = ::BasicDemo::CCM_Client_Context::_narrow(ctx); 
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
BasicDemo::CCM_Client_impl::ccm_activate()
   throw(CORBA::SystemException)
{
    // get_context() performs a duplicate
    ::BasicDemo::CCM_Client_Context_var ctx = get_context();
    Cdmw::TestUtils::CORBATestManager::instance()->add("client_context", ctx.in());
    // get_text() returns a CORBA::String
    CORBA::String_var txt = get_text();
    Cdmw::TestUtils::CORBATestManager::instance()->set_property ("text",txt.in());
    ::CommonDemo::Registration_var register_server;
    register_server = m_session_context->get_connection_to_register();
    CORBA::String_var client_name = register_server->_cxx_register();
    set_name(client_name.in());




    Cdmw::TestUtils::ThreadedTestRunner runner(Cdmw::TestUtils::CORBATestManager::instance()->get_controller());
    runner.addTest( CppUnit::TestFactoryRegistry::getRegistry("test_ccm_client_activate").makeTest() );
    runner.start();
    runner.join();

    sleep(5);
    
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//

void
BasicDemo::CCM_Client_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
   std::cout << "Ccm_passivate called!" << std::endl;
} 



//
// ccm_remove function : it destroys the component (call by the container)
void
BasicDemo::CCM_Client_impl::ccm_remove()
   throw(CORBA::SystemException)
{
   std::cout << "Ccm_remove called!" << std::endl;
}


//
// IDL:thalesgroup.com/BasicDemo/CCM_Client_Executor/name:1.0
//
char*
BasicDemo::CCM_Client_impl::name()
    throw(CORBA::SystemException)
{
    char* name = CORBA::string_dup(m_name.in());
    return name;
}

//
// IDL:thalesgroup.com/BasicDemo/CCM_Client_Executor/event:1.0
//
BasicDemo::Event* 
BasicDemo::CCM_Client_impl::event()
    throw(CORBA::SystemException)
{
    BasicDemo::Event* event = new BasicDemo::Event_impl();
    event->text(m_text.in());

    return event;
}

void 
BasicDemo::CCM_Client_impl::event(BasicDemo::Event* event)
    throw(CORBA::SystemException)
{
    m_text = CORBA::string_dup(event->text());
}

