/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.cif.test;

import cdmw.ccm.container.ConfigValueImpl;
import cdmw.ossupport.OS;
import cdmw.testutils.CORBATestManager;
import cdmw.testutils.TestManager;
import cdmw.ccm.common.CCMSystemProperties;

import org.junit.Before;
import org.junit.Test;
import java.lang.System;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.fail;

public class BasicDemoTest {

    public static final String SERVER_HOME_REP_ID = 
        "IDL:thalesgroup.com/Cdmw_BasicDemo/ServerHome:1.0";
    public static final String SERVER_HOME_SERVANT_NAME = 
        "Cdmw.CCM.CIF.CdmwBasicDemo.SessionServerHome_impl";
    public static final String SERVER_HOME_UID = "ServerHome_1";
    public static final String SERVER_HOME_ENTRY_POINT = 
        "cdmw.ccm.cif.test.CCMServerHomeImpl.createCCMServerHome";
    public static final String SERVER_NAME = "R2D2";
    public static final String SERVER_COMPONENT_NAME = "MyServerComponent";

    public static final String CLIENT_HOME_REP_ID = 
        "IDL:thalesgroup.com/Cdmw_BasicDemo/ClientHome:1.0";
    public static final String CLIENT_HOME_SERVANT_NAME = 
        "Cdmw.CCM.CIF.CdmwBasicDemo.SessionClientHome_impl";
    public static final String CLIENT_HOME_UID = "ClientHome_1";
    public static final String CLIENT_HOME_ENTRY_POINT = 
        "cdmw.ccm.cif.test.CCMClientHomeImpl.createCCMClientHome";
    public static final String CLIENT_COMPONENT_NAME = "MyClientComponent";

    public static final String DISPLAY_FACET_NAME = "for_clients";
    public static final String INVALID_FACET_NAME = "invalid_facet";

    public static final String DISPLAY_RECEP_NAME = "to_server";
    public static final String REGISTER_RECEP_NAME = "to_register";
    public static final String INVALID_RECEP_NAME = "invalid_recep";

    public static final String EVENT_REP_ID = 
        "IDL:acme.com/BasicDemo/Event:1.0";
    public static final String EVENT_ENTRYPOINT = 
        "cdmw.ccm.cif.test.EventFactoryImpl.createEventFactory";
        
    private org.omg.CORBA.ORB orb;
    private org.omg.Components.Deployment.Container clientContainer;
    private org.omg.Components.Deployment.Container serverContainer;
    private org.omg.CosNaming.NamingContext namingContext;
    private boolean isTypeCheckingOn;


    @Before
    public void init() throws Exception {
        CORBATestManager testManager = (CORBATestManager)TestManager.getLocalInstance();

        String containerIORClient
            = (String)testManager.getLocalProperty(
                "cdmw.ccm.cif.test.BasicDemoTest.CONTAINER_IOR_CLIENT");
        String containerIORServer
            = (String)testManager.getLocalProperty(
                "cdmw.ccm.cif.test.BasicDemoTest.CONTAINER_IOR_SERVER");

        this.orb = testManager.getORB();

        org.omg.CORBA.Object obj = this.orb.string_to_object(containerIORClient);

        assertTrue(
            "TEST FAILED : exception raised during test initialisation!",
            obj != null);
        this.clientContainer = org.omg.Components.Deployment.ContainerHelper.narrow(obj);

        obj = this.orb.string_to_object(containerIORServer);

        assertTrue("TEST FAILED : exception raised during test initialisation!", obj != null);
        this.serverContainer = org.omg.Components.Deployment.ContainerHelper.narrow(obj);
        

        java.lang.String isTypeCheckingOnStr = 
            System.getProperty(CCMSystemProperties.TYPECHECKING, "off");
        isTypeCheckingOn = (isTypeCheckingOnStr.equals("on"))? true : false;
        System.out.println("*** " + CCMSystemProperties.TYPECHECKING
                           + " set to " + isTypeCheckingOnStr);

        try {
            // get the naming context
            obj = cdmw.testutils.CORBATestManager.instance().getObject("namingContext");
            this.namingContext =
                org.omg.CosNaming.NamingContextHelper.narrow(obj);

            OS.sleep(TestManager.getTimescale() * 3000);
        }
        catch (cdmw.testutils.CORBATestManager.InvalidObject e) {
            fail("TEST FAILED : InvalidObject raised during test initialisation!");
        }
        catch (InterruptedException ie) {}
    }
    
    @Test
    public void doTests() {
        org.omg.Components.CCMHome serverHome = null;
        com.acme.BasicDemo.ServerHome basicServerHome = null;
        org.omg.Components.CCMObject ccmServerComp = null;
        com.acme.BasicDemo.Server basicServerComp = null;
        com.acme.BasicDemo.Server basicServerComp2 = null;
        com.acme.BasicDemo.Server basicServerComp3 = null;
    
        org.omg.Components.CCMHome clientHome = null;
        com.acme.BasicDemo.ClientHome basicClientHome = null;
        org.omg.Components.CCMObject ccmClientComp = null;
        com.acme.BasicDemo.Client basicClientComp = null;
    
        org.omg.CORBA.Object displayFacetObj = null;
    
        // This tests CIF classes
        //-------------------------------------------------
        System.out.println("CIF classes Tests");
        
        try {
            System.out.println("Installing a ServerHome...");
    
            try {
                org.omg.Components.ConfigValue[] configValues = 
                    new org.omg.Components.ConfigValue[6];

                org.omg.CORBA.Any value = orb.create_any();
                com.thalesgroup.CdmwDeployment.ComponentKindValueHelper.insert(
                    value, 
                    com.thalesgroup.CdmwDeployment.ComponentKindValue.SESSION);
                configValues[0] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.COMPONENT_KIND.value, 
                    value);

                value = orb.create_any();
                com.thalesgroup.CdmwDeployment.ServantLifetimeValueHelper.insert(
                    value, 
                    com.thalesgroup.CdmwDeployment.ServantLifetimeValue.CONTAINER_LIFETIME);
                configValues[1] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.SERVANT_LIFETIME.value, 
                    value);

                value = orb.create_any();
                com.thalesgroup.CdmwDeployment.ThreadingPolicyValueHelper.insert(
                    value, 
                    com.thalesgroup.CdmwDeployment.ThreadingPolicyValue.MULTITHREAD);
                configValues[2] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.THREADING_POLICY.value, 
                    value);

                value = orb.create_any();
                value.insert_string(SERVER_HOME_REP_ID);
                configValues[3] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.HOME_REPOSITORY_ID.value, 
                    value);

                value = orb.create_any();
                value.insert_string(SERVER_HOME_SERVANT_NAME);
                configValues[4] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.HOME_SERVANT_CLASSNAME.value, 
                    value);

                com.thalesgroup.CdmwDeployment.ValuetypeFactoryDescription[] seq
                    = new com.thalesgroup.CdmwDeployment.ValuetypeFactoryDescription[1];
                // Event valuetype
                com.thalesgroup.CdmwDeployment.ValuetypeFactoryDescription desc
                    = new com.thalesgroup.CdmwDeployment.ValuetypeFactoryDescription();
                desc.repid = EVENT_REP_ID;
                desc.factory_entrypoint = EVENT_ENTRYPOINT;
                seq[0] = desc;
                
                value = orb.create_any();
                com.thalesgroup.CdmwDeployment.ValuetypeFactoryDescriptionSeqHelper.insert(
                    value, 
                    seq);
                configValues[5] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.VALUETYPE_FACTORY_DEPENDENCIES.value, 
                    value);

                serverHome = this.serverContainer.install_home(
                    SERVER_HOME_UID,
                    SERVER_HOME_ENTRY_POINT,
                    configValues);
            } catch (org.omg.CORBA.BAD_PARAM bp) {
                fail(bp.toString());
            } catch (org.omg.Components.Deployment.InstallationFailure f) {
                fail("Unexpected exception raised: " + 
                    "Components::Deployment::InstallationFailure " + f.reason);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }
            
            System.out.println("Installing a ClientHome...");
    
            try {
                org.omg.Components.ConfigValue[] configValues = 
                    new org.omg.Components.ConfigValue[6];
                
                org.omg.CORBA.Any value = orb.create_any();
                com.thalesgroup.CdmwDeployment.ComponentKindValueHelper.insert(
                    value, 
                    com.thalesgroup.CdmwDeployment.ComponentKindValue.SESSION);
                configValues[0] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.COMPONENT_KIND.value, 
                    value);

                value = orb.create_any();
                com.thalesgroup.CdmwDeployment.ServantLifetimeValueHelper.insert(
                    value, 
                    com.thalesgroup.CdmwDeployment.ServantLifetimeValue.CONTAINER_LIFETIME);
                configValues[1] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.SERVANT_LIFETIME.value, 
                    value);

                value = orb.create_any();
                com.thalesgroup.CdmwDeployment.ThreadingPolicyValueHelper.insert(
                    value, 
                    com.thalesgroup.CdmwDeployment.ThreadingPolicyValue.MULTITHREAD);
                configValues[2] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.THREADING_POLICY.value, 
                    value);

                value = orb.create_any();
                value.insert_string(CLIENT_HOME_REP_ID);
                configValues[3] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.HOME_REPOSITORY_ID.value, 
                    value);

                value = orb.create_any();
                value.insert_string(CLIENT_HOME_SERVANT_NAME);
                configValues[4] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.HOME_SERVANT_CLASSNAME.value, 
                    value);

                com.thalesgroup.CdmwDeployment.ValuetypeFactoryDescription[] seq
                    = new com.thalesgroup.CdmwDeployment.ValuetypeFactoryDescription[1];
                // Event valuetype
                com.thalesgroup.CdmwDeployment.ValuetypeFactoryDescription desc
                    = new com.thalesgroup.CdmwDeployment.ValuetypeFactoryDescription();
                desc.repid = EVENT_REP_ID;
                desc.factory_entrypoint = EVENT_ENTRYPOINT;
                seq[0] = desc;
                
                value = orb.create_any();
                com.thalesgroup.CdmwDeployment.ValuetypeFactoryDescriptionSeqHelper.insert(
                    value, 
                    seq);
                configValues[5] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.VALUETYPE_FACTORY_DEPENDENCIES.value, 
                    value);

                clientHome = this.clientContainer.install_home(
                    CLIENT_HOME_UID,
                    CLIENT_HOME_ENTRY_POINT,
                    configValues);
            } catch (org.omg.CORBA.BAD_PARAM bp) {
                fail(bp.toString());
            } catch (org.omg.Components.Deployment.InstallationFailure f) {
                fail("Unexpected exception raised: " + 
                    "Components::Deployment::InstallationFailure " + f.reason );
            }
    
            System.out.println("Creating a Server component using create_component() operation...");
            try {
                org.omg.Components.KeylessCCMHome keylessServerHome 
                    = org.omg.Components.KeylessCCMHomeHelper.narrow(serverHome);
    
                assertNotNull("ERROR: Created home is not a keyless home!",
                              keylessServerHome);
                ccmServerComp = keylessServerHome.create_component();

                basicServerComp = 
                    com.acme.BasicDemo.ServerHelper.narrow(ccmServerComp);

                assertTrue(
                    "ERROR: Created component is not a BasicDemo.Server!",
                    basicServerComp != null);
            } catch (org.omg.Components.CreateFailure e) {
                fail("Unexpected exception raised: CreateFailure");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
            
            System.out.println(
                "Creating a Client component using create_component_with_config_values() operation...");
            try {
                org.omg.Components.KeylessCCMHome keylessClientHome 
                    = org.omg.Components.KeylessCCMHomeHelper.narrow(clientHome);
                assertNotNull("ERROR: Created home is not a keyless home!",
                              keylessClientHome);

                org.omg.Components.ConfigValue[] configValues =
                    new org.omg.Components.ConfigValue[1];
                org.omg.CORBA.Any value = orb.create_any();
                value.insert_string(CLIENT_COMPONENT_NAME);
                configValues[0] = 
                    new cdmw.ccm.container.ConfigValueImpl(
                          com.thalesgroup.CdmwDeployment.COMPONENT_NAME.value, 
                          value);

                com.thalesgroup.CdmwCcmCif.KeylessCCMHome cdmwKeylessClientHome = 
                    com.thalesgroup.CdmwCcmCif.KeylessCCMHomeHelper.narrow(clientHome);
                ccmClientComp = cdmwKeylessClientHome.create_component_with_config_values(configValues);
                basicClientComp =
                    com.acme.BasicDemo.ClientHelper.narrow(ccmClientComp);
                assertNotNull(
                    "ERROR: Created component is not a BasicDemo.Client!",
                    basicClientComp);
            }
            catch (org.omg.Components.CreateFailure cf) {
                fail("Unexpected exception raised: CreateFailure");
            }
            catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
            

            System.out.println("Check the Client component is registered "
                               + "in a local Naming Context");
            try {
                org.omg.CosNaming.NameComponent[] name =
                    new org.omg.CosNaming.NameComponent[1];
                name[0] = new org.omg.CosNaming.NameComponent(CLIENT_COMPONENT_NAME, "");
                org.omg.CORBA.Object obj = this.namingContext.resolve(name);

                ccmClientComp = com.acme.BasicDemo.ClientHelper.narrow(obj);
                assertNotNull(
                    "ERROR: registered component is not a BasicDemo.Client!",
                    ccmClientComp);
            }
            catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
                fail("ERROR: component name not found!");
            }
            catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e) {
                fail("ERROR: component name resolve raised CannotProceed!");
            }
            catch (org.omg.CosNaming.NamingContextPackage.InvalidName e) {
                fail("ERROR: component name is invalid!");
            }
            catch (org.omg.CORBA.SystemException e) {
                fail("ERROR: Unexpected exception raised: " + e);
            }


            System.out.println("Creating other Server component using create() operation");
            try {
                basicServerHome 
                    = com.acme.BasicDemo.ServerHomeHelper.narrow(serverHome);
    
                assertTrue(
                    "Home is not a BasicDemo.ServerHome!",
                    basicServerHome != null);
                basicServerComp2 = basicServerHome.create();
                assertNotNull("ERROR: basicServerComp2 is nil!", basicServerComp2);
            }
            catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println(
                "Creating another Server component using create_component_with_config_values() operation...");
            try {
                org.omg.Components.KeylessCCMHome keylessServerHome =
                    org.omg.Components.KeylessCCMHomeHelper.narrow(serverHome);
                assertNotNull("ERROR: Created home is not a keyless home!",
                              keylessServerHome);

                org.omg.Components.ConfigValue[] configValues =
                    new org.omg.Components.ConfigValue[1];
                org.omg.CORBA.Any value = orb.create_any();
                value.insert_string(SERVER_COMPONENT_NAME);
                configValues[0] = 
                    new cdmw.ccm.container.ConfigValueImpl(
                           com.thalesgroup.CdmwDeployment.COMPONENT_NAME.value,
                           value);

                com.thalesgroup.CdmwCcmCif.KeylessCCMHome cdmwKeylessServerHome = 
                    com.thalesgroup.CdmwCcmCif.KeylessCCMHomeHelper.narrow(serverHome);
                ccmServerComp = cdmwKeylessServerHome.create_component_with_config_values(configValues);
                basicServerComp3 =
                    com.acme.BasicDemo.ServerHelper.narrow(ccmServerComp);
                assertNotNull(
                    "ERROR: Created component is not a BasicDemo.Server!",
                    basicServerComp3);
            }
            catch (org.omg.Components.CreateFailure cf) {
                fail("Unexpected exception raised: CreateFailure");
            }
            catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Check the last Server component is registered "
                               + "in a local Naming Context");
            com.acme.BasicDemo.Server serverComp3 = null;
            try {
                org.omg.CosNaming.NameComponent[] name =
                    new org.omg.CosNaming.NameComponent[1];
                name[0] = new org.omg.CosNaming.NameComponent(SERVER_COMPONENT_NAME, "");
                org.omg.CORBA.Object obj = this.namingContext.resolve(name);

                serverComp3 = com.acme.BasicDemo.ServerHelper.narrow(obj);
                assertNotNull(
                    "ERROR: registered component is not a BasicDemo.Server!",
                    serverComp3);
            }
            catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
                fail("ERROR: component name not found!");
            }
            catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e) {
                fail("ERROR: component name resolve raised CannotProceed!");
            }
            catch (org.omg.CosNaming.NamingContextPackage.InvalidName e) {
                fail("ERROR: component name is invalid!");
            }
            catch (org.omg.CORBA.SystemException e) {
                fail("ERROR: Unexpected exception raised: " + e);
            }

            System.out.println(
                "Checking Server component is same using same_component() operation...");
            try {
                boolean result1 = 
                    basicServerComp2.same_component(basicServerComp3);
                boolean result2 =
                    basicServerComp3.same_component(serverComp3);
                
                assertTrue("ERROR: basicServerComp2 is not the same component!",
                           ! result1);
                assertTrue("ERROR: basicServerComp3 is the same component!",
                           result2);
            }
            catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    

            System.out.println("Set to nil server_comp3");
            serverComp3 = null;
            System.out.println("Removing a Server component using "
                               + "remove_component() operation...");
            try {
                serverHome.remove_component(basicServerComp2);
            }
            catch (org.omg.Components.RemoveFailure rf) {
                fail("Unexpected exception raised: RemoveFailure");
            }
            catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println(
                "Removing again the same Server component using remove_component() operation...");
            try {
                serverHome.remove_component(basicServerComp2);
                fail("Expected exception not raised");
            } catch (org.omg.Components.RemoveFailure rf) {
                System.out.println("Expected exception raised: RemoveFailure");
            } catch (org.omg.CORBA.BAD_PARAM bp) {
                fail("Unexpected exception raised: BAD_PARAM");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Testing Navigation operation on Server component...");
            System.out.println("Get_all_facets operation on Server component...");
            try {
                org.omg.Components.FacetDescription[] facetDesc
                    = basicServerComp3.get_all_facets();
    
                assertTrue(
                    "ERROR: Unexpected number of facet found for Server component",
                    facetDesc.length == 1);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Get_all_facets operation on Client component...");
            try {
                org.omg.Components.FacetDescription[] facetDesc
                    = basicClientComp.get_all_facets();
    
                assertTrue(
                    "ERROR: Unexpected facet found for Client component",
                    facetDesc.length == 0);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Get_named_facets operation on Server component...");
            try {
                String[] facetNameList = new String[1];
                facetNameList[0] = DISPLAY_FACET_NAME;
                org.omg.Components.FacetDescription[] facetDesc
                    = basicServerComp.get_named_facets(facetNameList);
    
                assertTrue(
                    "ERROR: Unexpected number of facet found for Server component",
                    facetDesc.length == 1);
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println(
                "Get_named_facets operation on Server component with invalid name...");
            try {
                String[] facetNameList = new String[2];
                facetNameList[0] = DISPLAY_FACET_NAME;
                facetNameList[1] = INVALID_FACET_NAME;
    
                org.omg.Components.FacetDescription[] facetDesc
                    = basicServerComp.get_named_facets(facetNameList);
                fail("Expected exception not raised");
            } catch (org.omg.Components.InvalidName in) {
                System.out.println("Expected exception raised: InvalidName");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("provide_facet operation on Server component...");
            try {
                displayFacetObj
                    = basicServerComp.provide_facet(DISPLAY_FACET_NAME);
    
                assertTrue("ERROR: No facet found for Server component", displayFacetObj != null);
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
            System.out.println("provide_facet operation on Server component with invalid name...");
            try {
                org.omg.CORBA.Object facetObj
                    = basicServerComp.provide_facet(INVALID_FACET_NAME);
                fail("Expected exception not raised");
            } catch (org.omg.Components.InvalidName in) {
                System.out.println("Expected exception raised: InvalidName");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Testing Receptacle operation on Server component...");
            System.out.println("Get_all_receptacles operation on Server component...");
            try {
                org.omg.Components.ReceptacleDescription[] recepDesc
                    = basicServerComp3.get_all_receptacles();
    
                assertTrue(
                    "ERROR: Unexpected receptacle found for Server component",
                    recepDesc.length == 0);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Get_all_receptacles operation on Client component...");
            try {
                org.omg.Components.ReceptacleDescription[] recepDesc
                    = basicClientComp.get_all_receptacles();
    
                assertTrue(
                    "ERROR: Unexpected number of receptacle found for Client component",
                    recepDesc.length == 2);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Get_named_receptacles operation on Client component...");
            try {
                String[] recepNameList = new String[2];
                recepNameList[0] = DISPLAY_RECEP_NAME;
                recepNameList[1] = REGISTER_RECEP_NAME;
                org.omg.Components.ReceptacleDescription[] recepDesc
                    = basicClientComp.get_named_receptacles(recepNameList);
    
                assertTrue(
                    "ERROR: Unexpected number of receptacle found for Client component",
                    recepDesc.length == 2);
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Get_named_receptacles operation on Client component with invalid name...");
            try {
                String[] recepNameList = new String[2];
                recepNameList[0] = DISPLAY_RECEP_NAME;
                recepNameList[1] = INVALID_RECEP_NAME;
    
                org.omg.Components.ReceptacleDescription[] recepDesc
                    = basicClientComp.get_named_receptacles(recepNameList);
                fail("Expected exception not raised");
            } catch (org.omg.Components.InvalidName in) {
                System.out.println("Expected exception raised: InvalidName");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Get_connections operation on Client component...");
            try {
                org.omg.Components.ConnectionDescription[] conxDesc
                    = basicClientComp.get_connections(DISPLAY_RECEP_NAME);
    
                assertTrue(
                    "ERROR: Unexpected connection found for Client component",
                    conxDesc.length == 0);
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println(
                "Get_connections operation on Client component with invalid name...");
            try {
                org.omg.Components.ConnectionDescription[] conxDesc
                    = basicClientComp.get_connections(INVALID_RECEP_NAME);
                fail("Expected exception not raised");
            } catch (org.omg.Components.InvalidName in) {
                System.out.println("Expected exception raised: InvalidName");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println(
                "Connect operation on Client component with invalid name...");
            try {
                org.omg.Components.Cookie ck
                    = basicClientComp.connect(INVALID_RECEP_NAME, displayFacetObj);
                fail("Expected exception not raised");
            } catch (org.omg.Components.InvalidName in) {
                System.out.println("Expected exception raised: InvalidName");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.Components.AlreadyConnected ac) {
                fail("Unexpected exception raised: AlreadyConnected");
            } catch (org.omg.Components.ExceededConnectionLimit ecl) {
                fail("Unexpected exception raised: ExceededConnectionLimit");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println(
                "Connect operation on Client component with invalid connection...");
            try {
                org.omg.Components.Cookie ck
                    = basicClientComp.connect(REGISTER_RECEP_NAME, displayFacetObj);
                if (isTypeCheckingOn) {
                    fail("Expected exception not raised");
                }
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.InvalidConnection ic) {
                if (! isTypeCheckingOn) {
                    fail("Unexpected exception not raised: InvalidConnection");
                }
                System.out.println("Expected exception raised: InvalidConnection");
            } catch (org.omg.Components.AlreadyConnected ac) {
                fail("Unexpected exception raised: AlreadyConnected");
            } catch (org.omg.Components.ExceededConnectionLimit ecl) {
                fail("Unexpected exception raised: ExceededConnectionLimit");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            if (! isTypeCheckingOn) {
                System.out.println("Disconnect operation on Client component...");
                try {
                    basicClientComp.disconnect(REGISTER_RECEP_NAME, null);
                }
                catch (org.omg.Components.InvalidName in) {
                    fail("Unexpected exception raised: InvalidName");
                }
                catch (org.omg.Components.InvalidConnection ic) {
                    fail("Unexpected exception raised: InvalidConnection");
                }
                catch (org.omg.Components.CookieRequired cr) {
                    fail("Unexpected exception raised: CookieRequired");
                }
                catch (org.omg.Components.NoConnection nc) {
                    fail("Unexpected exception raised: NoConnection");
                }
                catch (org.omg.CORBA.SystemException se) {
                    fail("Unexpected exception raised: " + se.toString());
                }
            }

            System.out.println("Connect operation on Client component...");
            try {
                org.omg.Components.Cookie ck = basicClientComp.connect(
                    DISPLAY_RECEP_NAME,
                    displayFacetObj);
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.Components.AlreadyConnected ac) {
                fail("Unexpected exception raised: AlreadyConnected");
            } catch (org.omg.Components.ExceededConnectionLimit ecl) {
                fail("Unexpected exception raised: ExceededConnectionLimit");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Connect operation again on Client component...");
            try {
                org.omg.Components.Cookie ck = basicClientComp.connect(
                    DISPLAY_RECEP_NAME,
                    displayFacetObj);
                fail("Expected exception not raised");
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.Components.AlreadyConnected ac) {
                System.out.println("Expected exception raised: AlreadyConnected");
            } catch (org.omg.Components.ExceededConnectionLimit ecl) {
                fail("Unexpected exception raised: ExceededConnectionLimit");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Disconnect operation on Client component...");
            try {
                basicClientComp.disconnect(DISPLAY_RECEP_NAME, null);
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.Components.CookieRequired cr) {
                fail("Unexpected exception raised: CookieRequired");
            } catch (org.omg.Components.NoConnection nc) {
                fail("Unexpected exception raised: NoConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println(
                "Disconnect operation on Client component with an invalid name...");
            try {
                basicClientComp.disconnect(INVALID_RECEP_NAME, null);
                fail("Expected exception not raised");
            } catch (org.omg.Components.InvalidName in) {
                System.out.println("Expected exception raised: InvalidName");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.Components.CookieRequired cr) {
                fail("Unexpected exception raised: CookieRequired");
            } catch (org.omg.Components.NoConnection nc) {
                fail("Unexpected exception raised: NoConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Disconnect operation again on Client component...");
            try {
                 basicClientComp.disconnect(DISPLAY_RECEP_NAME, null);
                 fail("Expected exception not raised");
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.Components.CookieRequired cr) {
                fail("Unexpected exception raised: CookieRequired");
            } catch (org.omg.Components.NoConnection nc) {
                System.out.println("Expected exception raised: NoConnections");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Connect operation again on Client component...");
            try {
                org.omg.Components.Cookie ck = basicClientComp.connect(
                    DISPLAY_RECEP_NAME,
                    displayFacetObj);
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.Components.AlreadyConnected ac) {
                fail("Unexpected exception raised: AlreadyConnected");
            } catch (org.omg.Components.ExceededConnectionLimit ecl) {
                fail("Unexpected exception raised: ExceededConnectionLimit");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Configuration_complete operation on Client component...");
            try {
                basicClientComp.configuration_complete();
                fail("Expected exception not raised");
            } catch (org.omg.Components.InvalidConfiguration ic) {
                System.out.println("Expected exception raised: InvalidConfiguration");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Connect operation on Client component...");
            try {
                org.omg.Components.Cookie ck = basicClientComp.connect(
                    REGISTER_RECEP_NAME,
                    basicServerComp);
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.Components.AlreadyConnected ac) {
                fail("Unexpected exception raised: AlreadyConnected");
            } catch (org.omg.Components.ExceededConnectionLimit ecl) {
                fail("Unexpected exception raised: ExceededConnectionLimit");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Get_connections operation on Client component...");
            try {
                org.omg.Components.ConnectionDescription[] conxDesc
                    = basicClientComp.get_connections(REGISTER_RECEP_NAME);
    
                assertTrue(
                    "ERROR: Unexpected number of connection found for Client component => "
                        + conxDesc.length,
                    conxDesc.length == 1);
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Get_all_publishers operation on Client component...");
            try {
                org.omg.Components.PublisherDescription[] publisherDesc
                    = basicClientComp.get_all_publishers();
    
                assertTrue(
                    "ERROR: Unexpected number of publisher found for Client component",
                    publisherDesc.length == 0);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Get_all_emitters operation on Client component...");
            try {
                org.omg.Components.EmitterDescription[] emitterDesc
                    = basicClientComp.get_all_emitters();
    
                assertTrue(
                    "ERROR: Unexpected number of emitter found for Client component",
                    emitterDesc.length == 0);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Get_all_consumers operation on Client component...");
            try {
                org.omg.Components.ConsumerDescription[] consumerDesc
                    = basicClientComp.get_all_consumers();
    
                assertTrue(
                    "ERROR: Unexpected number of consumers found for Client component",
                    consumerDesc.length == 0);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Get_ccm_home operation on Client component...");
            try {
                org.omg.Components.CCMHome cHome
                    = basicClientComp.get_ccm_home();
    
                assertTrue(
                    "ERROR: Unexpected home for Client component",
                    cHome._is_equivalent(clientHome));
            }
            catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Get_primary_key operation on Client component...");
            try {
                org.omg.Components.PrimaryKeyBase key
                    = basicClientComp.get_primary_key();
                fail("Expected exception not raised");
            } catch (org.omg.Components.NoKeyAvailable nka) {
                System.out.println("Expected exception raised: NoKeyAvailable");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Get_all_ports operation on Client component...");
            try {
                org.omg.Components.ComponentPortDescription portDesc
                    = basicClientComp.get_all_ports();
    
                org.omg.Components.FacetDescription[] facetDesc
                    = portDesc.facets;
                org.omg.Components.ReceptacleDescription[] recepDesc
                    = portDesc.receptacles;
                org.omg.Components.ConsumerDescription[] consumerDesc
                    = portDesc.consumers;
                org.omg.Components.EmitterDescription[] emitterDesc
                    = portDesc.emitters;
                org.omg.Components.PublisherDescription[] publisherDesc
                    = portDesc.publishers;
    
                assertTrue(
                    "ERROR: unexpected number of facet for Client component",
                    facetDesc.length == 0);
                assertTrue(
                    "ERROR: unexpected number of receptacle for Client component",
                    recepDesc.length == 2);
                assertTrue(
                    "ERROR: unexpected number of consumer for Client component",
                    consumerDesc.length == 0);
                assertTrue(
                    "ERROR: unexpected number of publisher for Client component",
                    publisherDesc.length == 0);
                assertTrue(
                    "ERROR: unexpected number of emitter for Client component",
                    emitterDesc.length == 0);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Remove operation on a Server component...");
            try {
                basicServerComp3.remove();
            } catch (org.omg.Components.RemoveFailure rf) {
                fail("Unexpected exception raised: NoKeyAvailable");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Check the removed Server component is no more "
                               + "registered in a local Naming Context");
            try {
                org.omg.CosNaming.NameComponent[] name =
                    new org.omg.CosNaming.NameComponent[1];
                name[0] = 
                    new org.omg.CosNaming.NameComponent(SERVER_COMPONENT_NAME,
                                                        "");
                
                org.omg.CORBA.Object obj = this.namingContext.resolve(name);
                fail("ERROR component name found!");
            }
            catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
                System.out.println("OK component name not found!");
            }
            catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e) {
                fail("ERROR: component name resolve raised CannotProceed!");
            }
            catch (org.omg.CosNaming.NamingContextPackage.InvalidName e) {
                fail("ERROR: component name is invalid!");
            }
            catch (org.omg.CORBA.SystemException e) {
                fail("ERROR: Unexpected exception raised: " + e);
            }


            System.out.println("Remove operation again on the same Server component...");
            int timescale = TestManager.getTimescale();
            OS.sleep(timescale * 2000);
            try {
                basicServerComp3.remove();
                fail("Expected exception not raised");
            }
            catch (org.omg.Components.RemoveFailure rf) {
                fail("Unexpected exception raised: RemoveFailure");
            }
            catch (org.omg.CORBA.OBJECT_NOT_EXIST one) {
                System.out.println("Expected exception raised: OBJECT_NOT_EXIST");
            }
            catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println(
                "Configuration_complete operation on a removed Server component...");
            try {
                basicServerComp3.configuration_complete();
                fail("Expected exception not raised");
            } catch (org.omg.Components.InvalidConfiguration ic) {
                fail("Unexpected exception raised: InvalidConfiguration");
            } catch (org.omg.CORBA.OBJECT_NOT_EXIST one) {
                System.out.println("Expected exception raised: OBJECT_NOT_EXIST");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println(
                "Configuration_complete operation on an other removed Server component...");
            try {
                basicServerComp2.configuration_complete();
                fail("Expected exception not raised");
            } catch (org.omg.Components.InvalidConfiguration ic) {
                fail("Unexpected exception raised: InvalidConfiguration");
            } catch (org.omg.CORBA.OBJECT_NOT_EXIST one) {
                System.out.println("Expected exception raised: OBJECT_NOT_EXIST");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Configuration_complete operation on Server component...");
            try {
                basicServerComp.configuration_complete();
                fail("Expected exception not raised");
            } catch (org.omg.Components.InvalidConfiguration ic) {
                System.out.println("Expected exception raised: InvalidConfiguration");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Init attribute operation on Server component...");
            try {
                basicServerComp.name(SERVER_NAME);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Configuration_complete operation on Server component...");
            try {
                basicServerComp.configuration_complete();
            } catch (org.omg.Components.InvalidConfiguration ic) {
                fail("Unexpected exception raised: InvalidConfiguration");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Init attribute operation on Client component...");
            try {
                com.acme.BasicDemo.Event event = new EventImpl();
                event.text = "Hello World!";
                basicClientComp.event(event);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Configuration_complete operation on Client component...");
            try {
                basicClientComp.configuration_complete();
            } catch (org.omg.Components.InvalidConfiguration ic) {
                fail("Unexpected exception raised: InvalidConfiguration");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Get name of Server component...");
            try {
                String serverName = basicServerComp.name();
                System.out.println("Server name is " + serverName);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Get name of Client component...");
            try {
                String clientName = basicClientComp.name();
                System.out.println("Server name is " + clientName);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Disconnect operation on Client component...");
            try {
                basicClientComp.disconnect(DISPLAY_RECEP_NAME, null);
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.Components.CookieRequired cr) {
                fail("Unexpected exception raised: CookieRequired");
            } catch (org.omg.Components.NoConnection nc) {
                fail("Unexpected exception raised: NoConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Disconnect operation on Client component...");
            try {
                basicClientComp.disconnect(REGISTER_RECEP_NAME, null);
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.Components.CookieRequired cr) {
                fail("Unexpected exception raised: CookieRequired");
            } catch (org.omg.Components.NoConnection nc) {
                fail("Unexpected exception raised: NoConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Remove operation on the Server home...");
            try {
                this.serverContainer.remove_home(basicServerHome);
            } catch (org.omg.Components.RemoveFailure rf) {
                fail("Unexpected exception raised: NoKeyAvailable");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
            
            System.out.println("Remove operation on the Client home...");
            try {
                this.clientContainer.remove_home(clientHome);
            } catch (org.omg.Components.RemoveFailure rf) {
                fail("Unexpected exception raised: NoKeyAvailable");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Check the Client component is no more "
                               + "registered in a local Naming Context");
            try {
                org.omg.CosNaming.NameComponent[] name =
                    new org.omg.CosNaming.NameComponent[1];
                name[0] = 
                    new org.omg.CosNaming.NameComponent(CLIENT_COMPONENT_NAME,
                                                        "");
                
                org.omg.CORBA.Object obj = this.namingContext.resolve(name);
                fail("ERROR component name found!");
            }
            catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
                System.out.println("OK component name not found!");
            }
            catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e) {
                fail("ERROR: component name resolve raised CannotProceed!");
            }
            catch (org.omg.CosNaming.NamingContextPackage.InvalidName e) {
                fail("ERROR: component name is invalid!");
            }
            catch (org.omg.CORBA.SystemException e) {
                fail("ERROR: Unexpected exception raised: " + e);
            }

        }
        catch (Exception e)  {
            fail("Unexpected exception raised: " + e.toString());
        }
    }
}

