/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.cif.test;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.junit.Before;
import org.junit.Test;

import cdmw.ossupport.OS;
import cdmw.testutils.CORBATestManager;
import cdmw.testutils.TestManager;
import cdmw.testutils.CORBATestManager.InvalidObject;
import cdmw.ccm.container.ConfigValueImpl;

public class TypedBasicDemoTest {
    
    public static final String SERVER_HOME_REP_ID = 
        "IDL:thalesgroup.com/Cdmw_BasicDemo/ServerHome:1.0";
    public static final String SERVER_HOME_SERVANT_NAME = 
        "Cdmw.CCM.CIF.CdmwBasicDemo.SessionServerHome_impl";
    public static final String SERVER_HOME_UID = "ServerHome_1";
    public static final String SERVER_HOME_ENTRY_POINT = 
        "cdmw.ccm.cif.test.CCMServerHomeImpl.createCCMServerHome";
    public static final String SERVER_NAME = "R2D2";

    public static final String CLIENT_HOME_REP_ID = 
        "IDL:thalesgroup.com/Cdmw_BasicDemo/ClientHome:1.0";
    public static final String CLIENT_HOME_SERVANT_NAME = 
        "Cdmw.CCM.CIF.CdmwBasicDemo.SessionClientHome_impl";
    public static final String CLIENT_HOME_UID = "ClientHome_1";
    public static final String CLIENT_HOME_ENTRY_POINT = 
        "cdmw.ccm.cif.test.CCMClientHomeImpl.createCCMClientHome";

    public static final String DISPLAY_FACET_NAME = "for_clients";
    public static final String INVALID_FACET_NAME = "invalid_facet";

    public static final String DISPLAY_RECEP_NAME = "to_server";
    public static final String REGISTER_RECEP_NAME = "to_register";
    public static final String INVALID_RECEP_NAME = "invalid_recep";

    public static final String EVENT_REP_ID = 
        "IDL:acme.com/BasicDemo/Event:1.0";
    public static final String EVENT_ENTRYPOINT = 
        "cdmw.ccm.cif.test.EventFactoryImpl.createEventFactory";

    private org.omg.CORBA.ORB orb;

    private org.omg.Components.Deployment.Container clientContainer;
    private org.omg.Components.Deployment.Container serverContainer;

    @Before
    public void init() throws InvalidObject {
        CORBATestManager testManager = (CORBATestManager)TestManager.getLocalInstance();
        
        this.orb = testManager.getORB();
        
        String containerIORClient
            = (String)testManager.getLocalProperty(
                "cdmw.ccm.cif.test.BasicDemoTest.CONTAINER_IOR_CLIENT");
        String containerIORServer
            = (String)testManager.getLocalProperty(
                "cdmw.ccm.cif.test.BasicDemoTest.CONTAINER_IOR_SERVER");

        org.omg.CORBA.Object obj = this.orb.string_to_object(containerIORClient);

        assertTrue(
            "TEST FAILED : exception raised during test initialisation!",
            obj != null);
        this.clientContainer = org.omg.Components.Deployment.ContainerHelper.narrow(obj);

        obj = this.orb.string_to_object(containerIORServer);

        assertTrue("TEST FAILED : exception raised during test initialisation!", obj != null);
        this.serverContainer = org.omg.Components.Deployment.ContainerHelper.narrow(obj);
        
        try {
            OS.sleep(TestManager.getTimescale() * 3000);
        } catch(InterruptedException ie) {}
    }
    
    @Test
    public void doTests() {      
        org.omg.Components.CCMHome serverHome = null;
        com.acme.BasicDemo.ServerHome basicServerHome = null;
        org.omg.Components.CCMObject ccmServerComp = null;
        com.acme.BasicDemo.Server basicServerComp = null;
    
        org.omg.Components.CCMHome clientHome = null;
        com.acme.BasicDemo.ClientHome basicClientHome = null;
        org.omg.Components.CCMObject ccmClientComp = null;
        com.acme.BasicDemo.Client basicClientComp = null;
    
        com.acme.BasicDemo.Display displayFacetObj = null;
    
        // This tests CIF classes
        //-------------------------------------------------
        System.out.println("CIF classes Tests");
        
        try {
            System.out.println("Install a ServerHome");
            
            try {
                org.omg.Components.ConfigValue[] configValues = 
                    new org.omg.Components.ConfigValue[5];

                org.omg.CORBA.Any value = orb.create_any();
                com.thalesgroup.CdmwDeployment.ComponentKindValueHelper.insert(
                    value, 
                    com.thalesgroup.CdmwDeployment.ComponentKindValue.SESSION);
                configValues[0] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.COMPONENT_KIND.value, 
                    value);

                value = orb.create_any();
                com.thalesgroup.CdmwDeployment.ServantLifetimeValueHelper.insert(
                    value, 
                    com.thalesgroup.CdmwDeployment.ServantLifetimeValue.CONTAINER_LIFETIME);
                configValues[1] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.SERVANT_LIFETIME.value, 
                    value);

                value = orb.create_any();
                com.thalesgroup.CdmwDeployment.ThreadingPolicyValueHelper.insert(
                    value, 
                    com.thalesgroup.CdmwDeployment.ThreadingPolicyValue.MULTITHREAD);
                configValues[2] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.THREADING_POLICY.value, 
                    value);

                value = orb.create_any();
                value.insert_string(SERVER_HOME_REP_ID);
                configValues[3] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.HOME_REPOSITORY_ID.value, 
                    value);

                value = orb.create_any();
                value.insert_string(SERVER_HOME_SERVANT_NAME);
                configValues[4] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.HOME_SERVANT_CLASSNAME.value, 
                    value);

                serverHome = this.serverContainer.install_home(
                    SERVER_HOME_UID,
                    SERVER_HOME_ENTRY_POINT,
                    configValues);
            } catch (org.omg.CORBA.BAD_PARAM bp) {
                fail(bp.toString());
            } catch (org.omg.Components.Deployment.InstallationFailure f) {
                fail("Unexpected exception raised: " + 
                    "Components::Deployment::InstallationFailure " + f.reason);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }
            
            System.out.println("Installing a ClientHome...");
    
            try {
                org.omg.Components.ConfigValue[] configValues = 
                    new org.omg.Components.ConfigValue[6];
                
                org.omg.CORBA.Any value = orb.create_any();
                com.thalesgroup.CdmwDeployment.ComponentKindValueHelper.insert(
                    value, 
                    com.thalesgroup.CdmwDeployment.ComponentKindValue.SESSION);
                configValues[0] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.COMPONENT_KIND.value, 
                    value);

                value = orb.create_any();
                com.thalesgroup.CdmwDeployment.ServantLifetimeValueHelper.insert(
                    value, 
                    com.thalesgroup.CdmwDeployment.ServantLifetimeValue.CONTAINER_LIFETIME);
                configValues[1] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.SERVANT_LIFETIME.value, 
                    value);

                value = orb.create_any();
                com.thalesgroup.CdmwDeployment.ThreadingPolicyValueHelper.insert(
                    value, 
                    com.thalesgroup.CdmwDeployment.ThreadingPolicyValue.MULTITHREAD);
                configValues[2] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.THREADING_POLICY.value, 
                    value);

                value = orb.create_any();
                value.insert_string(CLIENT_HOME_REP_ID);
                configValues[3] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.HOME_REPOSITORY_ID.value, 
                    value);

                value = orb.create_any();
                value.insert_string(CLIENT_HOME_SERVANT_NAME);
                configValues[4] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.HOME_SERVANT_CLASSNAME.value, 
                    value);

                com.thalesgroup.CdmwDeployment.ValuetypeFactoryDescription[] seq
                    = new com.thalesgroup.CdmwDeployment.ValuetypeFactoryDescription[1];
                // Event valuetype
                com.thalesgroup.CdmwDeployment.ValuetypeFactoryDescription desc
                    = new com.thalesgroup.CdmwDeployment.ValuetypeFactoryDescription();
                desc.repid = EVENT_REP_ID;
                desc.factory_entrypoint = EVENT_ENTRYPOINT;
                seq[0] = desc;
                
                value = orb.create_any();
                com.thalesgroup.CdmwDeployment.ValuetypeFactoryDescriptionSeqHelper.insert(
                    value, 
                    seq);
                configValues[5] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.VALUETYPE_FACTORY_DEPENDENCIES.value, 
                    value);

                clientHome = this.clientContainer.install_home(
                    CLIENT_HOME_UID,
                    CLIENT_HOME_ENTRY_POINT,
                    configValues);
            } catch (org.omg.CORBA.BAD_PARAM bp) {
                fail(bp.toString());
            } catch (org.omg.Components.Deployment.InstallationFailure f) {
                fail("Unexpected exception raised: " + 
                    "Components::Deployment::InstallationFailure " + f.reason );
            }
            
            System.out.println("Creating a Client component using create_component() operation...");
            try {
                org.omg.Components.KeylessCCMHome keylessClientHome 
                    = org.omg.Components.KeylessCCMHomeHelper.narrow(clientHome);
    
                assertTrue("ERROR: Created home is not a keyless home!", keylessClientHome != null);
                ccmClientComp = keylessClientHome.create_component();

                basicClientComp
                    = com.acme.BasicDemo.ClientHelper.narrow(ccmClientComp);

                assertTrue("ERROR: Created component is not a BasicDemo.Client!", basicClientComp != null);
            } catch (org.omg.Components.CreateFailure cf) {
                fail("Unexpected exception raised: CreateFailure");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
            
            System.out.println("Create a Server component using create_named_component() operation");
            try {
                basicServerHome = 
                    com.acme.BasicDemo.ServerHomeHelper.narrow(serverHome);
    
                assertTrue(
                    "Home is not a BasicDemo::ServerHome!",
                    basicServerHome != null);

                com.acme.BasicDemo.NamedComponent namedComp
                    = basicServerHome.create_named_component(SERVER_NAME);

                basicServerComp = 
                    com.acme.BasicDemo.ServerHelper.narrow(namedComp);

                assertTrue("ERROR: basicServerComp is nil!", basicServerComp != null);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Testing generated operation on Server component...");
            System.out.println("provide_for_clients operation on Server component...");
            try {
                displayFacetObj = basicServerComp.provide_for_clients();
    
                assertTrue(
                    "ERROR: No facet found for Server component",
                    displayFacetObj != null);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Connect operation on Client component...");
            try {
                basicClientComp.connect_to_server(displayFacetObj);
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.Components.AlreadyConnected ac) {
                fail("Unexpected exception raised: AlreadyConnected");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Connect operation again on Client component...");
            try {
                basicClientComp.connect_to_server(displayFacetObj);
                fail("Expected exception not raised");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.Components.AlreadyConnected ac) {
                System.out.println("Expected exception raised: AlreadyConnected");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Disconnect operation on Client component...");
            try {
                basicClientComp.disconnect_to_server();
            } catch (org.omg.Components.NoConnection nc) {
                fail("Unexpected exception raised: NoConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Disconnect operation again on Client component...");
            try {
                basicClientComp.disconnect_to_server();
                fail("Expected exception not raised");
            } catch (org.omg.Components.NoConnection nc) {
                System.out.println("Expected exception raised: NoConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Connect operation again on Client component...");
            try {
                basicClientComp.connect_to_server(displayFacetObj);
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.Components.AlreadyConnected ac) {
                fail("Unexpected exception raised: AlreadyConnected");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Init attribute operation on Client component...");
            try {
                com.acme.BasicDemo.Event event = new EventImpl();
                event.text = "Hello World!";
                basicClientComp.event(event);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Configuration_complete operation on Client component...");
            try {
                basicClientComp.configuration_complete();
                fail("Expected exception not raised");
            } catch (org.omg.Components.InvalidConfiguration ic) {
                System.out.println("Expected exception raised: InvalidConfiguration");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Connect operation on Client component...");
            try {
                basicClientComp.connect_to_register(basicServerComp);
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.Components.AlreadyConnected ac) {
                fail("Unexpected exception raised: AlreadyConnected");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Get_connections operation on Client component...");
            try {
                com.acme.CommonDemo.Registration registrationConx
                    = basicClientComp.get_connection_to_register();
    
                assertTrue(
                    "ERROR: Unexpected connection found for registration for Client component",
                    registrationConx._is_equivalent(basicServerComp));
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Get_all_ports operation on Client component...");
            try {
                org.omg.Components.ComponentPortDescription portDesc
                    = basicClientComp.get_all_ports();
    
                org.omg.Components.FacetDescription[] facetDesc
                    = portDesc.facets;
                org.omg.Components.ReceptacleDescription[] recepDesc
                    = portDesc.receptacles;
                org.omg.Components.ConsumerDescription[] consumerDesc
                    = portDesc.consumers;
                org.omg.Components.EmitterDescription[] emitterDesc
                    = portDesc.emitters;
                org.omg.Components.PublisherDescription[] publisherDesc
                    = portDesc.publishers;
    
                assertTrue(
                    "ERROR: unexpected number of facet for Client component",
                    facetDesc.length == 0);
                assertTrue(
                    "ERROR: unexpected number of receptacle for Client component",
                    recepDesc.length == 2);
                assertTrue(
                    "ERROR: unexpected number of consumer for Client component",
                    consumerDesc.length == 0);
                assertTrue(
                    "ERROR: unexpected number of publisher for Client component",
                    publisherDesc.length == 0);
                assertTrue(
                    "ERROR: unexpected number of emitter for Client component",
                    emitterDesc.length == 0);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Configuration_complete operation on Server component...");
            try {
                basicServerComp.configuration_complete();
            } catch (org.omg.Components.InvalidConfiguration ic) {
                fail("Unexpected exception raised: InvalidConfiguration");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Configuration_complete operation on Client component...");
            try {
                basicClientComp.configuration_complete();
            } catch (org.omg.Components.InvalidConfiguration ic) {
                fail("Unexpected exception raised: InvalidConfiguration");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Get name of Server component...");
            try {
                String serverName = basicServerComp.name();
                System.out.println("Server name is " + serverName);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Get name of Client component...");
            try {
                String clientName = basicClientComp.name();
                System.out.println("Server name is " + clientName);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Disconnect operation on Client component...");
            try {
                basicClientComp.disconnect_to_server();
            } catch (org.omg.Components.NoConnection nc) {
                fail("Unexpected exception raised: NoConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Disconnect operation on Client component...");
            try {
                basicClientComp.disconnect_to_register();
            } catch (org.omg.Components.NoConnection nc) {
                fail("Unexpected exception raised: NoConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("Remove operation on the Server home...");
            try {
                this.serverContainer.remove_home(basicServerHome);
            } catch (org.omg.Components.RemoveFailure rf) {
                fail("Unexpected exception raised: NoKeyAvailable");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
            
            System.out.println("Remove operation on the Client home...");
            try {
                this.clientContainer.remove_home(clientHome);
            } catch (org.omg.Components.RemoveFailure rf) {
                fail("Unexpected exception raised: NoKeyAvailable");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
        } catch (Exception e) {
            System.out.println("Unexpected exception raised: " + e.toString());
        }
    }
}
