/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <sstream>
#include <string>
#include <fstream>

#include "testftccmcif/TestNavigationWithFT.hpp"
#include "testftccmcif/ftccmcif_cif.skel.hpp"

#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <CCMContainer/ccmcontainer/ConfigValue_impl.hpp>
#include <Foundation/idllib/Components.stub.hpp>
#include <Foundation/idllib/CdmwDeployment.stub.hpp>
#include <CCMContainer/idllib/CdmwCcmCif.stub.hpp>



namespace
{
    const char* COMP_WITH_FACET_REP_ID = "IDL:thalesgroup.com/Cdmw_TestFtCcmCif/ComponentsModule/ComponentWithFacet:1.0";
    const char* COMP_WITH_FACET_HOME_REP_ID = "IDL:acme.com/TestFtCcmCif/ComponentsModule/ComponentWithFacetHome:1.0";
    const char* FT_COMP_WITH_FACET_HOME_SERVANT_NAME = "Cdmw.CCM.CIF.CdmwTestFtCcmCif.ComponentsModule.FTSessionComponentWithFacetHome_impl";
    const char* COMP_WITH_FACET_HOME_UID = "ComponentWithFacetHome_1";
    const char* COMP_WITH_FACET_HOME_ENTRY_POINT = "create_CCM_ComponentWithFacetHome";
    const char* COMP_WITH_FACET_GROUP_NAME = "ComponentWithFacetGroup";

    const char* COMP_WITH_CONSUMER_REP_ID = "IDL:thalesgroup.com/Cdmw_TestFtCcmCif/ComponentsModule/ComponentWithConsumer:1.0";
    const char* COMP_WITH_CONSUMER_HOME_REP_ID = "IDL:acme.com/TestFtCcmCif/ComponentsModule/ComponentWithConsumerHome:1.0";
    const char* COMP_WITH_CONSUMER_HOME_SERVANT_NAME = "Cdmw.CCM.CIF.CdmwTestFtCcmCif.ComponentsModule.SessionComponentWithConsumerHome_impl";
    const char* FT_COMP_WITH_CONSUMER_HOME_SERVANT_NAME = "Cdmw.CCM.CIF.CdmwTestFtCcmCif.ComponentsModule.FTSessionComponentWithConsumerHome_impl";
    const char* COMP_WITH_CONSUMER_HOME_UID = "ComponentWithConsumerHome_1";
    const char* COMP_WITH_CONSUMER_HOME_ENTRY_POINT = "create_CCM_ComponentWithConsumerHome";
    const char* COMP_WITH_CONSUMER_GROUP_NAME = "ComponentWithConsumerGroup";
    const char* PRIMARY_COMPONENT_NAME = "PrimaryComponent";
    const char* BACKUP_COMPONENT_NAME = "BackupComponent";

    const char* TEST_FACET_NAME = "the_facet";
    const char* TEST_CONSUMER_NAME = "the_consumer";

    const char* TEST_EVENT_REP_ID = "IDL:acme.com/TestFtCcmCif/EventTypeModule/TestEvent:1.0";
    const char* TEST_EVENT_ENTRYPOINT = "createTestEventFactory";

}; // End anonymous namespace



namespace TestFtCcmCif
{

CPPUNIT_TEST_SUITE_REGISTRATION( TestNavigationWithFT );

TestNavigationWithFT::TestNavigationWithFT()
    throw(CORBA::Exception)
{
    //
    // Get Replication Manager
    //
    m_orb =  CORBA::ORB::_duplicate( Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());

    try
    {
        // get the naming context
        CORBA::Object_var obj = Cdmw::TestUtils::CORBATestManager::instance()->get_object
              ("namingContext");

        m_nc 
            = CosNaming::NamingContext::_narrow(obj.in());
    }
    catch (const Cdmw::TestUtils::CORBATestManager::InvalidObject& )
    {
        TEST_INFO("TEST FAILED : InvalidObject raised during test initialisation!");
        CPPUNIT_ASSERT(false);
    }
    catch (const CORBA::SystemException & e)
    {
        TEST_INFO("TEST FAILED : system exception raised during test initialisation!");
        throw;
    }
    

    CORBA::Object_var obj;
    try {
        obj = m_orb -> resolve_initial_references("ReplicationManager");
    } catch(const CORBA::ORB::InvalidName&) {
        std::cerr<< "Can't resolve `ReplicationManager'" << std::endl;
        throw;
    }
    if(CORBA::is_nil(obj.in())) {
        std::cerr << "`ReplicationManager' is a nil object reference" << std::endl;
        throw CORBA::INTERNAL();
    }
    try {
        m_replicationManager = CdmwFT::ReplicationManager::_narrow(obj.in());
    } catch (const CORBA::Exception&) {
        std::cerr << "Obj is not a ReplicationManager" << std::endl;
        throw;
    }

    //
    // get primary container
    //
    std::cout << "Get container from PrimaryCompSrv" << std::endl;
    std::string primContainerIOR = "file://" + primProcessName + "_container.IOR";
    CORBA::Object_var primContainerObj = 
    m_orb->string_to_object(primContainerIOR.c_str());
    m_containerPrimary =
    Components::Deployment::Container::_narrow(primContainerObj.in());
    
    //
    // get backup container
    //
    std::cout << "Get container from BackupCompSrv" << std::endl;
    std::string backContainerIOR = "file://" + backProcessName + "_container.IOR";
    CORBA::Object_var backContainerObj = 
    m_orb->string_to_object(backContainerIOR.c_str());
    m_containerBackup =
    Components::Deployment::Container::_narrow(backContainerObj.in());
    
    
    std::cout << "main server initialised" << std::endl;
    
    //
    // Create Locations
    //
    ::FT::Location primLoc;
    primLoc.length(3);
    primLoc[0].id = HostName.c_str();
    primLoc[0].kind = "hostname";
    primLoc[1].id = ApplicationName.c_str();
    primLoc[1].kind = "application";
    primLoc[2].id = primProcessName.c_str();
    primLoc[2].kind = "process";  
    
    m_primaryLocation = primLoc;
    
    ::FT::Location backLoc;
    backLoc.length(3);
    backLoc[0].id = HostName.c_str();
    backLoc[0].kind = "hostname";
    backLoc[1].id = ApplicationName.c_str();
    backLoc[1].kind = "application";
    backLoc[2].id = backProcessName.c_str();
    backLoc[2].kind = "process";
    
    m_backupLocation = backLoc;
}




ComponentsModule::ComponentWithFacet_ptr
TestNavigationWithFT::create_ComponentWithFacet(Components::Deployment::Container_ptr container, 
                                                ComponentsModule::ComponentWithFacet_ptr componentWithFacetGroup,
                                                const std::string& component_name)
    throw (CORBA::Exception)
{
//     add_nbOfRequestedTestOK(2);
    
    Components::CCMHome_var componentWithFacetHome;
    ComponentsModule::ComponentWithFacet_var  componentWithFacet = ComponentsModule::ComponentWithFacet::_nil();

    TEST_INFO("  Install a FT componentWithFacetHome");
    try
    {
        Components::ConfigValues config_values(8);
        config_values.length(6);
        CORBA::Any value;
        value <<= CdmwDeployment::SESSION;
        config_values[0] =
            new Cdmw::CCM::Container::ConfigValue_impl(
                CdmwDeployment::COMPONENT_KIND, 
                value);

        value <<= CdmwDeployment::CONTAINER_LIFETIME;
        config_values[1] =
            new Cdmw::CCM::Container::ConfigValue_impl(
                CdmwDeployment::SERVANT_LIFETIME, 
                value);

        value <<= CdmwDeployment::MULTITHREAD;
        config_values[2] =
            new Cdmw::CCM::Container::ConfigValue_impl(
                CdmwDeployment::THREADING_POLICY, 
                value);

        value <<= COMP_WITH_FACET_HOME_REP_ID;
        config_values[3] =
            new Cdmw::CCM::Container::ConfigValue_impl(
                CdmwDeployment::HOME_REPOSITORY_ID, 
                value);

        value <<= FT_COMP_WITH_FACET_HOME_SERVANT_NAME;
        config_values[4] =
            new Cdmw::CCM::Container::ConfigValue_impl(
                CdmwDeployment::HOME_SERVANT_CLASSNAME, 
                value);

        value <<= CdmwDeployment::WARM_PASSIVE;
        config_values[5] =
            new Cdmw::CCM::Container::ConfigValue_impl(
                CdmwDeployment::FAULT_TOLERANCE_REPLICATION_STYLE,
                value);
                
        componentWithFacetHome =
           container->install_home(COMP_WITH_FACET_HOME_UID,
                                   COMP_WITH_FACET_HOME_ENTRY_POINT,
                                   config_values);
        CPPUNIT_ASSERT(true);
    } catch (const CORBA::BAD_PARAM& e) {
        std::ostringstream os;
        os << "Unexpected BAD_PARAM exception: " << e << std::ends;
        TEST_INFO(os.str());
        CPPUNIT_ASSERT(false);
    } catch (const Components::Deployment::InstallationFailure& e) {
        std::ostringstream os;
        os << "Unexpected InstallationFailure exception: " << e.reason << std::ends;
        TEST_INFO(os.str());
        CPPUNIT_ASSERT(false);
    } catch (const CORBA::SystemException& e) {
        std::ostringstream os;
        os << "Unexpected system exception raised: " << e << std::ends;
        TEST_INFO(os.str());
        CPPUNIT_ASSERT(false);
    } catch (const CORBA::UserException& e) {
        std::ostringstream os;
        os << "Unexpected User exception raised: " << e << std::ends;
        TEST_INFO(os.str());
        CPPUNIT_ASSERT(false);
    }

    TEST_INFO("  Create a ComponentWithFacet with config values");
    try
    {
        CdmwCcmCif::KeylessCCMHome_var keyless_componentWithFacetHome 
            = CdmwCcmCif::KeylessCCMHome::_narrow(componentWithFacetHome.in());

        if (CORBA::is_nil(keyless_componentWithFacetHome.in())) {
            TEST_INFO("ERROR: Created home is not a keyless home!");
            CPPUNIT_ASSERT(false);
        } else {
            Components::ConfigValues config_values(3);
            config_values.length(2);
            CORBA::Any value;
            value <<= COMP_WITH_FACET_GROUP_NAME;
            config_values[0] = 
                new Cdmw::CCM::Container::ConfigValue_impl(
                                            CdmwDeployment::FAULT_TOLERANCE_GROUP_NAME, 
                                            value);
            value <<= component_name.c_str();
            config_values[1] = 
                new Cdmw::CCM::Container::ConfigValue_impl(
                                            CdmwDeployment::COMPONENT_NAME, 
                                            value);

            Components::CCMObject_var obj = 
                keyless_componentWithFacetHome->create_component_with_config_values(config_values);

            componentWithFacet = 
                ComponentsModule::ComponentWithFacet::_narrow(obj.in());

            if (CORBA::is_nil(componentWithFacet.in())) {
                TEST_INFO("ERROR: Created component is not a TestFtCcmCif::ComponentsModule::ComponentWithFacet!");
                CPPUNIT_ASSERT(false);
            } else {
                CPPUNIT_ASSERT(true);
                return componentWithFacet._retn();
            }
        }
    } catch (const Components::CreateFailure& e) {
        std::ostringstream os;
        os << "Unexpected CreateFailure exception: " << e << std::ends;
        TEST_INFO(os.str());
        CPPUNIT_ASSERT(false);
    } catch (const CORBA::SystemException& e) {
        std::ostringstream os;
        os << "Unexpected SystemException exception: " << e << std::ends;
        TEST_INFO(os.str());
        CPPUNIT_ASSERT(false);
    }

    // an error occured
    throw CORBA::INTERNAL();
}




ComponentsModule::ComponentWithConsumer_ptr
TestNavigationWithFT::create_ComponentWithConsumer(Components::Deployment::Container_ptr container, 
                                                   ComponentsModule::ComponentWithConsumer_ptr componentWithConsumerGroup)
    throw (CORBA::Exception)
{
//     add_nbOfRequestedTestOK(7);
    
    Components::CCMHome_var componentWithConsumerHome;
    Components::CCMHome_var FTComponentWithConsumerHome;
    ComponentsModule::ComponentWithConsumer_var componentWithConsumer = ComponentsModule::ComponentWithConsumer::_nil();

    try
    {
        Components::ConfigValues config_values(8);
        config_values.length(6);
        CORBA::Any value;
        value <<= CdmwDeployment::SESSION;
        config_values[0] = 
            new Cdmw::CCM::Container::ConfigValue_impl(
                CdmwDeployment::COMPONENT_KIND, 
                                            value);

        value <<= CdmwDeployment::CONTAINER_LIFETIME;
        config_values[1] = 
            new Cdmw::CCM::Container::ConfigValue_impl(
                CdmwDeployment::SERVANT_LIFETIME, 
                value);

        value <<= CdmwDeployment::MULTITHREAD;
        config_values[2] = 
            new Cdmw::CCM::Container::ConfigValue_impl(
                CdmwDeployment::THREADING_POLICY, 
                value);

        value <<= COMP_WITH_CONSUMER_HOME_REP_ID;
        config_values[3] = 
            new Cdmw::CCM::Container::ConfigValue_impl(
                CdmwDeployment::HOME_REPOSITORY_ID, 
                value);

        value <<= COMP_WITH_CONSUMER_HOME_SERVANT_NAME;
        config_values[4] = 
            new Cdmw::CCM::Container::ConfigValue_impl(
                CdmwDeployment::HOME_SERVANT_CLASSNAME, 
                value);

        CORBA::ULong lg = 1L;
        CdmwDeployment::ValuetypeFactoryDescriptionSeq seq(lg);
        seq.length(lg);
        // Event TestEvent valuetype
        {
            CdmwDeployment::ValuetypeFactoryDescription desc;
            desc.repid = TEST_EVENT_REP_ID;
            desc.factory_entrypoint = TEST_EVENT_ENTRYPOINT;
            seq[0]=desc;
        }            
        value <<= seq;
        config_values[5] = 
            new Cdmw::CCM::Container::ConfigValue_impl(
                CdmwDeployment::VALUETYPE_FACTORY_DEPENDENCIES,
                value);

        TEST_INFO("  Install a non FT componentWithConsumerHome");
        componentWithConsumerHome = 
            container->install_home(COMP_WITH_CONSUMER_HOME_UID,
                                    COMP_WITH_CONSUMER_HOME_ENTRY_POINT,
                                    config_values);

        config_values.length(7);
        value <<= FT_COMP_WITH_CONSUMER_HOME_SERVANT_NAME;
        config_values[4] = 
            new Cdmw::CCM::Container::ConfigValue_impl(
                CdmwDeployment::HOME_SERVANT_CLASSNAME, 
                value);

        value <<= CdmwDeployment::WARM_PASSIVE;
        config_values[6] =
            new Cdmw::CCM::Container::ConfigValue_impl(
                CdmwDeployment::FAULT_TOLERANCE_REPLICATION_STYLE,
                value);

        TEST_INFO("  Install a FT componentWithConsumerHome");
        FTComponentWithConsumerHome = 
            container->install_home(COMP_WITH_CONSUMER_HOME_UID,
                                    COMP_WITH_CONSUMER_HOME_ENTRY_POINT,
                                    config_values);

        CPPUNIT_ASSERT(true);
    
    } catch (const CORBA::BAD_PARAM& e) {
        std::ostringstream os;
        os << "Unexpected BAD_PARAM exception: " << e << std::ends;
        TEST_INFO(os.str());
        CPPUNIT_ASSERT(false);
    } catch (const Components::Deployment::InstallationFailure& e) {
        std::ostringstream os;
        os << "Unexpected InstallationFailure exception: " << e.reason << std::ends;
        TEST_INFO(os.str());
        CPPUNIT_ASSERT(false);
    } catch (const CORBA::SystemException& e) {
        std::ostringstream os;
        os << "Unexpected system exception raised: " << e << std::ends;
        TEST_INFO(os.str());
        CPPUNIT_ASSERT(false);
    } catch (const CORBA::UserException& e) {
        std::ostringstream os;
        os << "Unexpected User exception raised: " << e << std::ends;
        TEST_INFO(os.str());
        CPPUNIT_ASSERT(false);
    }

    TEST_INFO("  Create a ComponentWithConsumer without config values on a FT Home");
    try
    {
        CdmwCcmCif::KeylessCCMHome_var keyless_componentWithConsumerHome 
            = CdmwCcmCif::KeylessCCMHome::_narrow(FTComponentWithConsumerHome.in());

        if (CORBA::is_nil(keyless_componentWithConsumerHome.in())) {
            TEST_INFO("ERROR: Created home is not a keyless home!");
            CPPUNIT_ASSERT(false);
        } else {
            Components::CCMObject_var obj = 
                keyless_componentWithConsumerHome->create_component();

            CPPUNIT_ASSERT(false);
        }
    } catch (const Components::CreateFailure& e) {
        std::ostringstream os;
        os << "Expected CreateFailure exception: " << e << std::ends;
        TEST_INFO(os.str());
        CPPUNIT_ASSERT(true);
    } catch (const CORBA::SystemException& e) {
        std::ostringstream os;
        os << "Unexpected SystemException exception: " << e << std::ends;
        TEST_INFO(os.str());
        CPPUNIT_ASSERT(false);
    }


    /*TEST_INFO("  Create a ComponentWithConsumer with config values on a non FT Home");
    try
    {
        CdmwCcmCif::KeylessCCMHome_var keyless_componentWithConsumerHome 
            = CdmwCcmCif::KeylessCCMHome::_narrow(componentWithConsumerHome.in());

        if (CORBA::is_nil(keyless_componentWithConsumerHome.in())) {
            TEST_INFO("ERROR: Created home is not a keyless home!");
            CPPUNIT_ASSERT(false);
        } else {
            Components::ConfigValues config_values(3);
            config_values.length(1);
            CORBA::Any value;
            value <<= componentWithConsumerGroup;
            config_values[0] = 
                new Cdmw::CCM::Container::ConfigValue_impl(
                                            CdmwDeployment::FAULT_TOLERANCE_GROUP_NAME, 
                                            value);

            Components::CCMObject_var obj = 
                keyless_componentWithConsumerHome->create_component_with_config_values(config_values);

            CPPUNIT_ASSERT(false);
        }
    } catch (const Components::CreateFailure& e) {
        std::ostringstream os;
        os << "Expected CreateFailure exception: " << e << std::ends;
        TEST_INFO(os.str());
        CPPUNIT_ASSERT(true);
    } catch (const CORBA::SystemException& e) {
        std::ostringstream os;
        os << "Unexpected SystemException exception: " << e << std::ends;
        TEST_INFO(os.str());
        CPPUNIT_ASSERT(false);
    }*/


    TEST_INFO("  Create a ComponentWithConsumer with invalid config values on a FT Home");
    try
    {
        CdmwCcmCif::KeylessCCMHome_var keyless_componentWithConsumerHome 
            = CdmwCcmCif::KeylessCCMHome::_narrow(FTComponentWithConsumerHome.in());

        if (CORBA::is_nil(keyless_componentWithConsumerHome.in())) {
            TEST_INFO("ERROR: Created home is not a keyless home!");
            CPPUNIT_ASSERT(false);
        } else {
            Components::ConfigValues config_values(3);
            config_values.length(1);
            CORBA::Any value;
            value <<= componentWithConsumerGroup;
            config_values[0] = 
                new Cdmw::CCM::Container::ConfigValue_impl(
                                            CdmwDeployment::FAULT_TOLERANCE_REPLICATION_STYLE, 
                                            value);

            Components::CCMObject_var obj = 
                keyless_componentWithConsumerHome->create_component_with_config_values(config_values);

            CPPUNIT_ASSERT(false);
        }
    } catch (const Components::CreateFailure& e) {
        std::ostringstream os;
        os << "Expected CreateFailure exception: " << e << std::ends;
        TEST_INFO(os.str());
        CPPUNIT_ASSERT(true);
    } catch (const CORBA::SystemException& e) {
        std::ostringstream os;
        os << "Unexpected SystemException exception: " << e << std::ends;
        TEST_INFO(os.str());
        CPPUNIT_ASSERT(false);
    }

    TEST_INFO("  Create a ComponentWithConsumer with empty config values on a FT Home");
    try
    {
        CdmwCcmCif::KeylessCCMHome_var keyless_componentWithConsumerHome 
            = CdmwCcmCif::KeylessCCMHome::_narrow(FTComponentWithConsumerHome.in());

        if (CORBA::is_nil(keyless_componentWithConsumerHome.in())) {
            TEST_INFO("ERROR: Created home is not a keyless home!");
            CPPUNIT_ASSERT(false);
        } else {
            Components::ConfigValues config_values(3);
            config_values.length(0);

            Components::CCMObject_var obj = 
                keyless_componentWithConsumerHome->create_component_with_config_values(config_values);

            CPPUNIT_ASSERT(false);
        }
    } catch (const Components::CreateFailure& e) {
        std::ostringstream os;
        os << "Expected CreateFailure exception: " << e << std::ends;
        TEST_INFO(os.str());
        CPPUNIT_ASSERT(true);
    } catch (const CORBA::SystemException& e) {
        std::ostringstream os;
        os << "Unexpected SystemException exception: " << e << std::ends;
        TEST_INFO(os.str());
        CPPUNIT_ASSERT(false);
    }

    TEST_INFO("  Create a ComponentWithConsumer with no config values on a FT Home");
    try
    {
        CdmwCcmCif::KeylessCCMHome_var keyless_componentWithConsumerHome 
            = CdmwCcmCif::KeylessCCMHome::_narrow(FTComponentWithConsumerHome.in());

        if (CORBA::is_nil(keyless_componentWithConsumerHome.in())) {
            TEST_INFO("ERROR: Created home is not a keyless home!");
            CPPUNIT_ASSERT(false);
        } else {
            Components::ConfigValues config_values(1);
            config_values.length(0);

            Components::CCMObject_var obj = 
                keyless_componentWithConsumerHome->create_component_with_config_values(config_values);

            CPPUNIT_ASSERT(false);
        }
    } catch (const Components::CreateFailure& e) {
        std::ostringstream os;
        os << "Expected CreateFailure exception: " << e << std::ends;
        TEST_INFO(os.str());
        CPPUNIT_ASSERT(true);
    } catch (const CORBA::SystemException& e) {
        std::ostringstream os;
        os << "Unexpected SystemException exception: " << e << std::ends;
        TEST_INFO(os.str());
        CPPUNIT_ASSERT(false);
    }


    TEST_INFO("  Create a ComponentWithConsumer with config values on a FT Home");
    try
    {
        CdmwCcmCif::KeylessCCMHome_var keyless_componentWithConsumerHome 
            = CdmwCcmCif::KeylessCCMHome::_narrow(FTComponentWithConsumerHome.in());

        if (CORBA::is_nil(keyless_componentWithConsumerHome.in())) {
            TEST_INFO("ERROR: Created home is not a keyless home!");
            CPPUNIT_ASSERT(false);
        } else {
            Components::ConfigValues config_values(1);
            config_values.length(1);
            CORBA::Any value;
            value <<= COMP_WITH_CONSUMER_GROUP_NAME;
            config_values[0] = 
                new Cdmw::CCM::Container::ConfigValue_impl(
                                            CdmwDeployment::FAULT_TOLERANCE_GROUP_NAME, 
                                            value);

            Components::CCMObject_var obj = 
                keyless_componentWithConsumerHome->create_component_with_config_values(config_values);

            componentWithConsumer = 
                ComponentsModule::ComponentWithConsumer::_narrow(obj.in());

            if (CORBA::is_nil(componentWithConsumer.in())) {
                TEST_INFO("ERROR: Created component is not a TestFtCcmCif::ComponentsModule::ComponentWithConsumer!");
                CPPUNIT_ASSERT(false);
            } else {
                CPPUNIT_ASSERT(true);
                return componentWithConsumer._retn();
            }
        }
    } catch (const Components::CreateFailure& e) {
        std::ostringstream os;
        os << "Unexpected CreateFailure exception: " << e << std::ends;
        TEST_INFO(os.str());
        CPPUNIT_ASSERT(false);
    } catch (const CORBA::SystemException& e) {
        std::ostringstream os;
        os << "Unexpected SystemException exception: " << e << std::ends;
        TEST_INFO(os.str());
        CPPUNIT_ASSERT(false);
    }

    // an error occured
    throw CORBA::INTERNAL();
}


void 
TestNavigationWithFT::check_ccm_activate_called(bool should_be_called)
{
    // Check if its ComponentWithConsumer state file has been created:
    std::ifstream is("CCM_ComponentWithConsumer.stt");
    if (is.good() && !should_be_called)
    {
        TEST_INFO(" File CCM_ComponentWithConsumer.stt exists, " <<
                  "but ccm_activate() shouldn't have been called");
        CPPUNIT_ASSERT(false);
    } 
    else if (!is.good() && should_be_called)
    {
        TEST_INFO(" File CCM_ComponentWithConsumer.stt exists, " <<
                  "but ccm_activate() shouldn't have been called");
        CPPUNIT_ASSERT(false);
    }
    else
    {
        CPPUNIT_ASSERT(true);
    }
}


void
TestNavigationWithFT::call_ConfigurationComplete(Components::CCMObject_ptr component)
   throw (CORBA::Exception)
{
//     add_nbOfRequestedTestOK(2);

    try {
       component->configuration_complete();
       CPPUNIT_ASSERT(true);
    } catch (const Components::InvalidConfiguration& e) {
        std::ostringstream os;
        os << "Unexpected InvalidConfiguration exception: " << e << std::ends;
        TEST_INFO(os.str());
        CPPUNIT_ASSERT(false);
    } catch (const CORBA::SystemException& e) {
        std::ostringstream os;
        os << "Unexpected SystemException exception: " << e << std::ends;
        TEST_INFO(os.str());
        CPPUNIT_ASSERT(false);
    }
    /*
    ComponentsModule::ComponentWithFacet_var facetComponent; 
    try {
        facetComponent = ComponentsModule::ComponentWithFacet::_narrow(component);

        if (CORBA::is_nil(facetComponent.in())) 
        {
            // component is a Consumer component
            // Check if its state file has been created:
            std::ifstream is("CCM_ComponentWithConsumer.stt");
            if (is.good())
            {
                TEST_INFO(" File CCM_ComponentWithConsumer.stt shall not exist now");
                CPPUNIT_ASSERT(false);
            }
            else {
                CPPUNIT_ASSERT(true);
            }
        }
        else
        {
            // component is a Facet component
            // Check if its state file has been created:
            std::ifstream is("CCM_ComponentWithFacet.stt");
            if (is.good())
            {
                TEST_INFO(" File CCM_ComponentWithFacet.stt shall not exist now");
                CPPUNIT_ASSERT(false);
            }
            else {
                CPPUNIT_ASSERT(true);
            }
        }
    } catch (const CORBA::Exception& e) {
        std::cerr << "Error while narrowing group to ComponentsModule::ComponentWithFacet: " 
                  << e << std::endl;
        CPPUNIT_ASSERT(false);
        throw;
    }
*/
}

ComponentsModule::ComponentWithFacet_ptr
TestNavigationWithFT::create_ComponentWithFacetGroup()
    throw (CORBA::Exception)
{
//     add_nbOfRequestedTestOK(1);
    
    //
    // create FT::Criteria
    //
    ::FT::Properties prop;
    
    prop.length(4);
    prop[0].nam.length(1);
    prop[0].nam[0].id="org.omg.ft.MinimumNumberReplicas";
    prop[0].val <<= (CORBA::UShort)2;
    
    prop[1].nam.length(1);
    prop[1].nam[0].id="org.omg.ft.ReplicationStyle";
    prop[1].val <<= (CORBA::UShort)2;
    
    prop[2].nam.length(1);
    prop[2].nam[0].id="org.omg.ft.MembershipStyle";
    prop[2].val <<= (CORBA::UShort)0;
    
    prop[3].nam.length(1);
    prop[3].nam[0].id="org.omg.ft.Factories";
    
    ::FT::FactoryInfos factoryInfos;
    factoryInfos.length(2);
    factoryInfos[0].the_factory = ::FT::GenericFactory::_nil();
    factoryInfos[0].the_location = m_primaryLocation;
    ::FT::Criteria factoryCrit;        
    factoryCrit.length(0);
    factoryInfos[0].the_criteria = factoryCrit;
    
    factoryInfos[1].the_factory = ::FT::GenericFactory::_nil();
    factoryInfos[1].the_location = m_backupLocation;
    factoryInfos[1].the_criteria = factoryCrit;
    
    prop[3].val <<= factoryInfos;
    
    ::FT::Criteria ft_criteria;       
    ft_criteria.length(2);
    ft_criteria[0].nam.length(1);
    ft_criteria[0].nam[0].id="org.omg.ft.FTProperties";
    ft_criteria[0].val <<= prop;

    ::FT::Name OGName;
    OGName.length(1);
    OGName[0].id=CORBA::string_dup(COMP_WITH_FACET_GROUP_NAME);
    
    ft_criteria[1].nam.length(1);
    ft_criteria[1].nam[0].id="com.thalesgroup.cdmw.ft.Name";
    ft_criteria[1].val <<= OGName;
    
    const char * repId = COMP_WITH_FACET_REP_ID;
    CORBA::Object_var group;
    ::FT::GenericFactory::FactoryCreationId_var creationId;
    
    //
    // Create group
    //
    try {
        group = m_replicationManager->create_object(repId,
                                                    ft_criteria,
                                                    creationId.out());

    } catch (const CORBA::Exception& e ) {
        std::cerr << "Error during group creation: " << e << std::endl;
        CPPUNIT_ASSERT(false);
        throw;
    }

    ComponentsModule::ComponentWithFacet_var componentGroup; 
    try {
        componentGroup = ComponentsModule::ComponentWithFacet::_narrow(group.in());
    } catch (const CORBA::Exception& e) {
        std::cerr << "Error while narrowing group to ComponentWithFacet: " 
                  << e << std::endl;
        CPPUNIT_ASSERT(false);
        throw;
    }
            
    if (CORBA::is_nil(componentGroup.in())) {
        std::cerr << "Error: componentGroup is null." << std::endl;
        CPPUNIT_ASSERT(false);
        throw CORBA::INTERNAL();
    }

    CPPUNIT_ASSERT(true); 
    return componentGroup._retn();

}
    

void
TestNavigationWithFT::check_local_naming_context(const std::string& component_name, bool is_registered)
    throw (CORBA::Exception)
{
    bool test_ok=false;
    ComponentsModule::ComponentWithFacet_var registered_comp;
    try
    {
        CosNaming::Name name;
        name.length(1);
        name[0].id = CORBA::string_dup(component_name.c_str());

        CORBA::Object_var obj = m_nc->resolve(name);
        TEST_INFO(component_name << " Component name found!")

        registered_comp 
            = ComponentsModule::ComponentWithFacet::_narrow(obj.in());

        if (CORBA::is_nil(registered_comp.in()))
        {
            TEST_INFO("ERROR: registered component is not a ComponentsModule::ComponentWithFacet!")
        }
        else
        {
            test_ok = is_registered;
        }
    }
    catch (const CosNaming::NamingContext::NotFound& )
    {
        TEST_INFO("Exception raised: Component name not found!")
        test_ok = !is_registered;
    }
    catch (const CosNaming::NamingContext::CannotProceed& )
    {
        TEST_INFO("ERROR: component name resolve raised CannotProceed!")
        test_ok=false;
    }
    catch (const CosNaming::NamingContext::InvalidName& )
    {
        TEST_INFO("ERROR: component name is invalid!")
        test_ok=false;
    }
    catch (const CORBA::SystemException& e)
    {
        std::ostringstream os;
        os << "Unexpected exception raised: " << e << std::ends;
        TEST_INFO(os.str());
        test_ok=false;
    }
    CPPUNIT_ASSERT(test_ok);
}

void
TestNavigationWithFT::remove_component(ComponentsModule::ComponentWithFacet_ptr component)
    throw (CORBA::Exception)
{
    bool test_ok=false;
    try
    {
        component->remove();
        test_ok=true;
    }
    catch (const Components::RemoveFailure &)
    {
        TEST_INFO("Unexpected exception raised: NoKeyAvailable");
    }
    catch (const CORBA::SystemException& e)
    {
        std::ostringstream os;
        os << "Unexpected exception raised: " << e << std::ends;
        TEST_INFO(os.str());
        test_ok=false;
    }
    CPPUNIT_ASSERT(test_ok);
}

ComponentsModule::ComponentWithConsumer_ptr
TestNavigationWithFT::create_ComponentWithConsumerGroup()
    throw (CORBA::Exception)
{
//     add_nbOfRequestedTestOK(1);
    
    //
    // create FT::Criteria
    //
    ::FT::Properties prop;
    
    prop.length(4);
    prop[0].nam.length(1);
    prop[0].nam[0].id="org.omg.ft.MinimumNumberReplicas";
    prop[0].val <<= (CORBA::UShort)2;
    
    prop[1].nam.length(1);
    prop[1].nam[0].id="org.omg.ft.ReplicationStyle";
    prop[1].val <<= (CORBA::UShort)2;
    
    prop[2].nam.length(1);
    prop[2].nam[0].id="org.omg.ft.MembershipStyle";
    prop[2].val <<= (CORBA::UShort)0;
    
    prop[3].nam.length(1);
    prop[3].nam[0].id="org.omg.ft.Factories";
    
    ::FT::FactoryInfos factoryInfos;
    factoryInfos.length(2);
    factoryInfos[0].the_factory = ::FT::GenericFactory::_nil();
    factoryInfos[0].the_location = m_primaryLocation;
    ::FT::Criteria factoryCrit;        
    factoryCrit.length(0);
    factoryInfos[0].the_criteria = factoryCrit;
    
    factoryInfos[1].the_factory = ::FT::GenericFactory::_nil();
    factoryInfos[1].the_location = m_backupLocation;
    factoryInfos[1].the_criteria = factoryCrit;
    
    prop[3].val <<= factoryInfos;
    
    ::FT::Criteria ft_criteria;       
    ft_criteria.length(2);
    ft_criteria[0].nam.length(1);
    ft_criteria[0].nam[0].id="org.omg.ft.FTProperties";
    ft_criteria[0].val <<= prop;

    ::FT::Name OGName;
    OGName.length(1);
    OGName[0].id=CORBA::string_dup(COMP_WITH_CONSUMER_GROUP_NAME);
    
    ft_criteria[1].nam.length(1);
    ft_criteria[1].nam[0].id="com.thalesgroup.cdmw.ft.Name";
    ft_criteria[1].val <<= OGName;
    
    const char * repId = COMP_WITH_CONSUMER_REP_ID;
    CORBA::Object_var group;
    ::FT::GenericFactory::FactoryCreationId_var creationId;

    //
    // Create group
    //
    try {
        group = m_replicationManager->create_object(repId,
                                                    ft_criteria,
                                                    creationId.out());

    } catch (const CORBA::Exception& e ) {
        std::cerr << "Error during group creation: " << e << std::endl;
        CPPUNIT_ASSERT(false);
        throw;
    }

    ComponentsModule::ComponentWithConsumer_var componentGroup; 
    try {
        componentGroup = ComponentsModule::ComponentWithConsumer::_narrow(group.in());
    } catch (const CORBA::Exception& e) {
        std::cerr << "Error while narrowing group to ComponentWithConsumer: " 
                  << e << std::endl;
        CPPUNIT_ASSERT(false);
        throw;
    }
            
    if (CORBA::is_nil(componentGroup.in())) {
        std::cerr << "Error: componentGroup is null." << std::endl;
        CPPUNIT_ASSERT(false);
        throw CORBA::INTERNAL();
    }

    CPPUNIT_ASSERT(true); 
    return componentGroup._retn();
}



bool 
TestNavigationWithFT::isGroupReference(CORBA::Object_ptr obj)
    throw (CORBA::Exception)
{
    try {
        std::cout << " --------- isGroupReference ----------" << std::endl;
        CORBA::String_var str = m_orb->object_to_string(obj);
        std::cout << str << std::endl;
        std::cout << " -------------------------------------" << std::endl;
        
        ::FT::ObjectGroupId group_id = m_replicationManager->get_object_group_id(obj);
        std::cout << "--------- isGroupReference group_id = " << group_id << std::endl;
        return true;
            
    } catch (const FT::ObjectGroupNotFound& e) {
        return false;
    } catch (const CORBA::Exception& e) {
        std::cerr << "Exception in ReplicationManager->get_object_group_id(): "
                  << e << std::endl;
        throw;
    }
}




void 
TestNavigationWithFT::testNavigationInterface(ComponentsModule::ComponentWithFacet_ptr group,
                                              ComponentsModule::ComponentWithFacet_ptr primary,
                                              ComponentsModule::ComponentWithFacet_ptr backup)
    throw (CORBA::Exception)
{
    TEST_INFO("  test provide_facet()");
    try {
//         add_nbOfRequestedTestOK(3);

        CORBA::Object_var facetGroup = 
            group->provide_facet(TEST_FACET_NAME);
        if (isGroupReference(facetGroup.in()))
            CPPUNIT_ASSERT(true);
        else
            CPPUNIT_ASSERT(false);

        CORBA::Object_var facetPrimary = 
            primary->provide_facet(TEST_FACET_NAME);
        if (isGroupReference(facetPrimary.in()))
            CPPUNIT_ASSERT(false);
        else
            CPPUNIT_ASSERT(true);

        CORBA::Object_var facetBackup = 
            backup->provide_facet(TEST_FACET_NAME);
        if (isGroupReference(facetBackup.in()))
            CPPUNIT_ASSERT(false);
        else
            CPPUNIT_ASSERT(true);
    } catch (const Components::InvalidName& e) {
        std::cerr << "group->provide_facet() threw InvalidName" << std::endl;
        CPPUNIT_ASSERT(false);
    } catch (const CORBA::Exception& e) {
        std::cerr << "Unexpected exception while testing provide_facet() " 
                  << e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    
    TEST_INFO("  test get_all_facets()");
    try {
        Components::FacetDescriptions_var facetsGroup = 
            group->get_all_facets();
        for (CORBA::ULong i=0; i<facetsGroup->length(); ++i) {
//             add_nbOfRequestedTestOK(1);
                if (isGroupReference(facetsGroup[i]->facet_ref()))
                    CPPUNIT_ASSERT(true);
                else
                    CPPUNIT_ASSERT(false);
        }
        
        Components::FacetDescriptions_var facetsPrimary = 
            primary->get_all_facets();
        for (CORBA::ULong i=0; i<facetsPrimary->length(); ++i) {
//             add_nbOfRequestedTestOK(1);
                if (isGroupReference(facetsPrimary[i]->facet_ref()))
                    CPPUNIT_ASSERT(false);
                else
                    CPPUNIT_ASSERT(true);
        }
        
        Components::FacetDescriptions_var facetsBackup = 
            backup->get_all_facets();
        for (CORBA::ULong i=0; i<facetsBackup->length(); ++i) {
//             add_nbOfRequestedTestOK(1);
                if (isGroupReference(facetsBackup[i]->facet_ref()))
                    CPPUNIT_ASSERT(false);
                else
                    CPPUNIT_ASSERT(true);
        }
    } catch (const CORBA::Exception& e) {
        std::cerr << "Unexpected exception while testing get_all_facets() " 
                  << e << std::endl;
        CPPUNIT_ASSERT(false);
    }

    TEST_INFO("  test get_named_facets()");
    try {
//         add_nbOfRequestedTestOK(3);
        
        Components::NameList nameList;
        nameList.length(1);
        nameList[0] = TEST_FACET_NAME;
        
        Components::FacetDescriptions_var facetsGroup = 
            group->get_named_facets(nameList);
        if (isGroupReference(facetsGroup[(CORBA::ULong) 0]->facet_ref()))
            CPPUNIT_ASSERT(true);
        else
            CPPUNIT_ASSERT(false);

        Components::FacetDescriptions_var facetsPrimary = 
            primary->get_named_facets(nameList);
        if (isGroupReference(facetsPrimary[(CORBA::ULong) 0]->facet_ref()))
            CPPUNIT_ASSERT(false);
        else
            CPPUNIT_ASSERT(true);

        Components::FacetDescriptions_var facetsBackup = 
            backup->get_named_facets(nameList);
        if (isGroupReference(facetsBackup[(CORBA::ULong) 0]->facet_ref()))
            CPPUNIT_ASSERT(false);
        else
            CPPUNIT_ASSERT(true);
        
    } catch (const Components::InvalidName& e) {
        std::cerr << "group->get_named_facets() threw InvalidName" << std::endl;
        CPPUNIT_ASSERT(false);
    } catch (const CORBA::Exception& e) {
        std::cerr << "Unexpected exception while testing get_named_facets() " 
                  << e << std::endl;
        CPPUNIT_ASSERT(false);
    }
}



void 
TestNavigationWithFT::testTypedNavigationInterface(ComponentsModule::ComponentWithFacet_ptr group,
                                                   ComponentsModule::ComponentWithFacet_ptr primary,
                                                   ComponentsModule::ComponentWithFacet_ptr backup)
    throw (CORBA::Exception)
{
    TEST_INFO("  test provide_the_facet()");
    try {
//         add_nbOfRequestedTestOK(3);

        TestFtCcmCif::TestInterface_var facetGroup = 
            group->provide_the_facet();
        if (isGroupReference(facetGroup.in()))
            CPPUNIT_ASSERT(true);
        else
            CPPUNIT_ASSERT(false);

        TestFtCcmCif::TestInterface_var facetPrimary = 
            primary->provide_the_facet();
        if (isGroupReference(facetPrimary.in()))
            CPPUNIT_ASSERT(false);
        else
            CPPUNIT_ASSERT(true);

        TestFtCcmCif::TestInterface_var facetBackup = 
            backup->provide_the_facet();
        if (isGroupReference(facetBackup.in()))
            CPPUNIT_ASSERT(false);
        else
            CPPUNIT_ASSERT(true);
    } catch (const Components::InvalidName& e) {
        std::cerr << "group->provide_the_facet() threw InvalidName" << std::endl;
        CPPUNIT_ASSERT(false);
    } catch (const CORBA::Exception& e) {
        std::cerr << "Unexpected exception while testing provide_the_facet() " 
                  << e << std::endl;
        CPPUNIT_ASSERT(false);
    }
}


void 
TestNavigationWithFT::testEventsInterface(ComponentsModule::ComponentWithConsumer_ptr group,
                                          ComponentsModule::ComponentWithConsumer_ptr primary,
                                          ComponentsModule::ComponentWithConsumer_ptr backup)
    throw (CORBA::Exception)
{
    TEST_INFO("  test get_consumer()");
    try {
//         add_nbOfRequestedTestOK(3);

        Components::EventConsumerBase_var consumerGroup = 
            group->get_consumer(TEST_CONSUMER_NAME);
        if (isGroupReference(consumerGroup.in()))
            CPPUNIT_ASSERT(true);
        else
            CPPUNIT_ASSERT(false);

        Components::EventConsumerBase_var consumerPrimary = 
            primary->get_consumer(TEST_CONSUMER_NAME);
        if (isGroupReference(consumerPrimary.in()))
            CPPUNIT_ASSERT(false);
        else
            CPPUNIT_ASSERT(true);

        Components::EventConsumerBase_var consumerBackup = 
            backup->get_consumer(TEST_CONSUMER_NAME);
        if (isGroupReference(consumerBackup.in()))
            CPPUNIT_ASSERT(false);
        else
            CPPUNIT_ASSERT(true);
    } catch (const Components::InvalidName& e) {
        std::cerr << "group->get_consumer() threw InvalidName" << std::endl;
        CPPUNIT_ASSERT(false);
    } catch (const CORBA::Exception& e) {
        std::cerr << "Unexpected exception while testing get_consumer() " 
                  << e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    
    TEST_INFO("  test get_all_consumers()");
    try {
        Components::ConsumerDescriptions_var consumersGroup = 
            group->get_all_consumers();
        for (CORBA::ULong i=0; i<consumersGroup->length(); ++i) {
//             add_nbOfRequestedTestOK(1);
                if (isGroupReference(consumersGroup[i]->consumer()))
                    CPPUNIT_ASSERT(true);
                else
                    CPPUNIT_ASSERT(false);
        }
        
        Components::ConsumerDescriptions_var consumersPrimary = 
            primary->get_all_consumers();
        for (CORBA::ULong i=0; i<consumersPrimary->length(); ++i) {
//             add_nbOfRequestedTestOK(1);
                if (isGroupReference(consumersPrimary[i]->consumer()))
                    CPPUNIT_ASSERT(false);
                else
                    CPPUNIT_ASSERT(true);
        }
        
        Components::ConsumerDescriptions_var consumersBackup = 
            backup->get_all_consumers();
        for (CORBA::ULong i=0; i<consumersBackup->length(); ++i) {
//             add_nbOfRequestedTestOK(1);
                if (isGroupReference(consumersBackup[i]->consumer()))
                    CPPUNIT_ASSERT(false);
                else
                    CPPUNIT_ASSERT(true);
        }
    } catch (const CORBA::Exception& e) {
        std::cerr << "Unexpected exception while testing get_all_consumers() " 
                  << e << std::endl;
        CPPUNIT_ASSERT(false);
    }

    TEST_INFO("  test get_named_consumers()");
    try {
//         add_nbOfRequestedTestOK(3);
        
        Components::NameList nameList;
        nameList.length(1);
        nameList[0] = TEST_CONSUMER_NAME;
        
        Components::ConsumerDescriptions_var consumersGroup = 
            group->get_named_consumers(nameList);
        if (isGroupReference(consumersGroup[(CORBA::ULong) 0]->consumer()))
            CPPUNIT_ASSERT(true);
        else
            CPPUNIT_ASSERT(false);

        Components::ConsumerDescriptions_var consumersPrimary = 
            primary->get_named_consumers(nameList);
        if (isGroupReference(consumersPrimary[(CORBA::ULong) 0]->consumer()))
            CPPUNIT_ASSERT(false);
        else
            CPPUNIT_ASSERT(true);

        Components::ConsumerDescriptions_var consumersBackup = 
            backup->get_named_consumers(nameList);
        if (isGroupReference(consumersBackup[(CORBA::ULong) 0]->consumer()))
            CPPUNIT_ASSERT(false);
        else
            CPPUNIT_ASSERT(true);
        
    } catch (const Components::InvalidName& e) {
        std::cerr << "group->get_named_consumers() threw InvalidName" << std::endl;
        CPPUNIT_ASSERT(false);
    } catch (const CORBA::Exception& e) {
        std::cerr << "Unexpected exception while testing get_named_consumers() " 
                  << e << std::endl;
        CPPUNIT_ASSERT(false);
    }
}


void 
TestNavigationWithFT::testTypedEventsInterface(ComponentsModule::ComponentWithConsumer_ptr group,
                                               ComponentsModule::ComponentWithConsumer_ptr primary,
                                               ComponentsModule::ComponentWithConsumer_ptr backup)
    throw (CORBA::Exception)
{
    TEST_INFO("  test get_consumer_the_consumer()()");
    try {
//         add_nbOfRequestedTestOK(3);

        TestFtCcmCif::EventTypeModule::TestEventConsumer_var consumerGroup = 
            group->get_consumer_the_consumer();
        if (isGroupReference(consumerGroup.in()))
            CPPUNIT_ASSERT(true);
        else
            CPPUNIT_ASSERT(false);

        TestFtCcmCif::EventTypeModule::TestEventConsumer_var consumerPrimary = 
            primary->get_consumer_the_consumer();
        if (isGroupReference(consumerPrimary.in()))
            CPPUNIT_ASSERT(false);
        else
            CPPUNIT_ASSERT(true);

        TestFtCcmCif::EventTypeModule::TestEventConsumer_var consumerBackup = 
            backup->get_consumer_the_consumer();
        if (isGroupReference(consumerBackup.in()))
            CPPUNIT_ASSERT(false);
        else
            CPPUNIT_ASSERT(true);
    } catch (const Components::InvalidName& e) {
        std::cerr << "group->get_consumer_the_consumer()() threw InvalidName" << std::endl;
        CPPUNIT_ASSERT(false);
    } catch (const CORBA::Exception& e) {
        std::cerr << "Unexpected exception while testing get_consumer_the_consumer()() " 
                  << e << std::endl;
        CPPUNIT_ASSERT(false);
    }
}


void TestNavigationWithFT::do_tests()         
{
    // int timescale = Cdmw::TestUtils::get_timescale();

    // set number of requested successfull tests
//     set_nbOfRequestedTestOK (0);

    try 
    {
        // delete state file
        Cdmw::OsSupport::OS::unlink("CCM_ComponentWithFacet.stt");
        Cdmw::OsSupport::OS::unlink("CCM_ComponentWithConsumer.stt");
    }
    catch(...)
    {
    }
    
    ComponentsModule::ComponentWithFacet_var  componentWithFacetOnPrimary;
    ComponentsModule::ComponentWithConsumer_var componentWithConsumerOnPrimary;
    ComponentsModule::ComponentWithFacet_var  componentWithFacetOnBackup;
    ComponentsModule::ComponentWithConsumer_var componentWithConsumerOnBackup;
    ComponentsModule::ComponentWithFacet_var  componentWithFacetGroup;
    ComponentsModule::ComponentWithConsumer_var componentWithConsumerGroup;

    try {
        TEST_INFO("Create ComponentWithFacet group");
        componentWithFacetGroup = create_ComponentWithFacetGroup();
        
        TEST_INFO("Create ComponentWithConsumer group");
        componentWithConsumerGroup = create_ComponentWithConsumerGroup();

        TEST_INFO("Create primary ComponentWithFacet");
        componentWithFacetOnPrimary = 
            create_ComponentWithFacet(m_containerPrimary.in(),
                                      componentWithFacetGroup.in(),
                                      PRIMARY_COMPONENT_NAME);

        
        TEST_INFO("Check the componentWithFacet component is registered in a local Naming Context");
        check_local_naming_context(PRIMARY_COMPONENT_NAME, true);
    
        TEST_INFO("Create primary ComponentWithConsumer"); 
        componentWithConsumerOnPrimary = 
            create_ComponentWithConsumer(m_containerPrimary.in(),
                                         componentWithConsumerGroup.in());
    
        TEST_INFO("Create backup ComponentWithFacet");
        componentWithFacetOnBackup = 
            create_ComponentWithFacet(m_containerBackup.in(),
                                      componentWithFacetGroup.in(),
                                      BACKUP_COMPONENT_NAME);
        
        TEST_INFO("Check the componentWithFacet component is registered in a local Naming Context");
        check_local_naming_context(BACKUP_COMPONENT_NAME, true);

    
        TEST_INFO("Create backup ComponentWithConsumer"); 
        componentWithConsumerOnBackup = 
            create_ComponentWithConsumer(m_containerBackup.in(),
                                         componentWithConsumerGroup.in());

        TEST_INFO("Check that no ccm_activate has been called yet"); 
        check_ccm_activate_called(false);
                                         
        TEST_INFO("Call configuration_complete on all created components");
        call_ConfigurationComplete(componentWithFacetOnPrimary.in());
        call_ConfigurationComplete(componentWithConsumerOnPrimary.in());
        call_ConfigurationComplete(componentWithFacetOnBackup.in());
        call_ConfigurationComplete(componentWithConsumerOnBackup.in());

        TEST_INFO("Check that ccm_activate has been called"); 
        check_ccm_activate_called(true);
                                         

//         add_nbOfRequestedTestOK(1);
        if (CORBA::is_nil(componentWithFacetGroup.in()))
            CPPUNIT_ASSERT(false);
        else
            CPPUNIT_ASSERT(true);

        TEST_INFO("Has primary ComponentWithFacet been activated ?");
//         add_nbOfRequestedTestOK(1);
        std::ifstream is1("CCM_ComponentWithFacet.stt");
        if (is1.good()) {
            CPPUNIT_ASSERT(true);
        }
        else {
            TEST_INFO(" File CCM_ComponentWithFacet.stt doesn't exist");
            CPPUNIT_ASSERT(false);
        }

        TEST_INFO("Test a valid get_service operation on CCM Context");
        std::ifstream is11("CCM_ComponentWithFacet_get_service_FTCurrent.stt");
        if (is11.good()) {
            std::string result;

            is11 >> result;
            if (result == "OK")
                CPPUNIT_ASSERT(true);
            else
                CPPUNIT_ASSERT(false);
        }
        else {
            TEST_INFO(" File CCM_ComponentWithFacet_get_service_FTCurrent.stt doesn't exist");
            CPPUNIT_ASSERT(false);
        }

        TEST_INFO("Test an unavailable get_service operation on CCM Context");
        std::ifstream is12("CCM_ComponentWithFacet_get_service_RootPOA.stt");
        if (is12.good()) {
            std::string result;

            is12 >> result;
            if (result == "ServiceUnavailable")
                CPPUNIT_ASSERT(true);
            else
                CPPUNIT_ASSERT(false);
        }
        else {
            TEST_INFO(" File CCM_ComponentWithFacet_get_service_RootPOA.stt doesn't exist");
            CPPUNIT_ASSERT(false);
        }


        TEST_INFO("Has primary ComponentWithConsumer been activated ?");
//         add_nbOfRequestedTestOK(1);
        std::ifstream is2("CCM_ComponentWithConsumer.stt");
        if (is2.good()) {
            CPPUNIT_ASSERT(true);
        }
        else {
            TEST_INFO(" File CCM_ComponentWithConsumer.stt doesn't exist");
            CPPUNIT_ASSERT(false);
        }
                                                    
        TEST_INFO("Test Navigation interface");
        testNavigationInterface(componentWithFacetGroup.in(),
                                componentWithFacetOnPrimary.in(),
                                componentWithFacetOnBackup.in());
            
        TEST_INFO("Test Typed Navigation interface");
        testTypedNavigationInterface(componentWithFacetGroup.in(),
                                     componentWithFacetOnPrimary.in(),
                                     componentWithFacetOnBackup.in());
            
        TEST_INFO("Test Events interface");
        testEventsInterface(componentWithConsumerGroup.in(),
                            componentWithConsumerOnPrimary.in(),
                            componentWithConsumerOnBackup.in());
            
        TEST_INFO("Test Typed Events interface");
        testTypedEventsInterface(componentWithConsumerGroup.in(),
                                 componentWithConsumerOnPrimary.in(),
                                 componentWithConsumerOnBackup.in());

        TEST_INFO("_is_equivalent ComponentWithFacet group and primary ref");
//         add_nbOfRequestedTestOK(1);
        if (componentWithFacetGroup->_is_equivalent(componentWithFacetOnPrimary.in()))
            CPPUNIT_ASSERT(false);
        else
            CPPUNIT_ASSERT(true);

        TEST_INFO("Check the primary componentWithFacet component is registered in a local Naming Context");
        check_local_naming_context(PRIMARY_COMPONENT_NAME, true);

        TEST_INFO("Remove operation on a ComponentWithFacet component...");
        remove_component(componentWithFacetOnPrimary.in());

        TEST_INFO("Check the primary componentWithFacet component is no more registered in a local Naming Context");
        check_local_naming_context(PRIMARY_COMPONENT_NAME, false);

        // TODO: add a test when the primary crash
        // to do so add the location name in stt file : to get location
        // do a resolve_initial_references("FTGroupRepository") on the context (deprecated op)
        // kill primary process and advise the Fault Notifier of the death of this process.
            
    } 
    catch (const CORBA::Exception & e) 
    {
            std::ostringstream os;
            os << "Unexpected exception raised: " << e << std::ends;
            TEST_INFO(os.str()); 
            CPPUNIT_ASSERT(false);
    }

    
    try 
    {
        // delete state file
        Cdmw::OsSupport::OS::unlink("CCM_ComponentWithFacet.stt");
        Cdmw::OsSupport::OS::unlink("CCM_ComponentWithConsumer.stt");
    }
    catch(...)
    {
    }
}
        
}; // End namespace 

