/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.testmonophilosopers;

import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import cdmw.ossupport.OS;
import cdmw.testutils.CORBATestManager;
import cdmw.testutils.TestManager;
import cdmw.testutils.CORBATestManager.InvalidObject;
import cdmw.ccm.container.ConfigValueImpl;
import cdmw.ccm.common.CCMSystemProperties;

import com.thalesgroup.CdmwDeployment.ComponentKindValue;
import com.thalesgroup.CdmwDeployment.ComponentKindValueHelper;
import com.thalesgroup.CdmwDeployment.ServantLifetimeValue;
import com.thalesgroup.CdmwDeployment.ServantLifetimeValueHelper;
import com.thalesgroup.CdmwDeployment.ThreadingPolicyValue;
import com.thalesgroup.CdmwDeployment.ThreadingPolicyValueHelper;
import com.thalesgroup.CdmwDeployment.ValuetypeFactoryDescription;
import com.thalesgroup.CdmwDeployment.ValuetypeFactoryDescriptionHelper;
import com.thalesgroup.CdmwDeployment.ValuetypeFactoryDescriptionSeqHelper;

public class PhiloTest {

    public static final String PHILOSOPHER_HOME_REP_ID =
        "IDL:acme.com/Cdmw_Dinner/PhilosopherHome:1.0";
    public static final String PHILOSOPHER_HOME_SERVANT_NAME =
        "Cdmw.CCM.CIF.CdmwDinner.SessionPhilosopherHome_impl";
    public static final String PHILOSOPHER_HOME_UID = "PhilosopherHome_1";
    public static final String PHILOSOPHER_HOME_ENTRY_POINT =
        "cdmw.ccm.testmonophilosopers.CCMPhilosopherHomeImpl.createCCMPhilosopherHome";

    public static final String OBSERVER_HOME_REP_ID =
        "IDL:acme.com/Cdmw_Dinner/ObserverHome:1.0";
    public static final String OBSERVER_HOME_SERVANT_NAME =
        "Cdmw.CCM.CIF.CdmwDinner.SessionObserverHome_impl";
    public static final String OBSERVER_HOME_UID = "ObserverHome_1";
    public static final String OBSERVER_HOME_ENTRY_POINT =
        "cdmw.ccm.testmonophilosopers.CCMObserverHomeImpl.createCCMObserverHome";

    public static final String FORKMANAGER_HOME_REP_ID =
        "IDL:acme.com/Cdmw_Dinner/ForkHome:1.0";
    public static final String FORKMANAGER_HOME_SERVANT_NAME =
        "Cdmw.CCM.CIF.CdmwDinner.SessionForkHome_impl";
    public static final String FORKMANAGER_HOME_UID = "ForkHome_1";
    public static final String FORKMANAGER_HOME_ENTRY_POINT =
        "cdmw.ccm.testmonophilosopers.CCMForkHomeImpl.createCCMForkHome";

    public static final String REGISTRATOR_HOME_REP_ID =
        "IDL:acme.com/Cdmw_Dinner/RegistratorHome:1.0";
    public static final String REGISTRATOR_HOME_SERVANT_NAME =
        "Cdmw.CCM.CIF.CdmwDinner.SessionRegistratorHome_impl";
    public static final String REGISTRATOR_HOME_UID = "RegistratorHome_1";
    public static final String REGISTRATOR_HOME_ENTRY_POINT =
        "cdmw.ccm.testmonophilosopers.CCMRegistratorHomeImpl.createCCMRegistratorHome";

    public static final String FORK_FACET_NAME = "the_fork";

    public static final String FORK_RIGHT_RECEP_NAME = "right";
    public static final String FORK_LEFT_RECEP_NAME = "left";
    public static final String REGISTER_RECEP_NAME = "register";

    public static final String INFO_CONSUMER_NAME = "info";
    public static final String INFO_PUBLISHER_NAME = "info";
    public static final String CONFIG_EMITTER_NAME = "config";

    public static final String EVENT_STATUSINFO_REP_ID =
        "IDL:acme.com/Dinner/StatusInfo:1.0";
    public static final String EVENT_STATUSINFO_ENTRYPOINT =
        "cdmw.ccm.testmonophilosopers.StatusInfoFactoryImpl.createStatusInfoFactory";
    
    static final String CONTAINER_IOR =
        "cdmw.ccm.testmonophilosopers.PhiloTest.CONTAINER_IOR";
    static final String NON_CDMW_CONSUMER_IOR =
        "cdmw.ccm.testmonophilosopers.PhiloTest.NON_CDMW_CONSUMER_IOR";
    static final String CONSUMER_BASE_IOR =
        "cdmw.ccm.testmonophilosopers.PhiloTest.CONSUMER_BASE_IOR";

    private org.omg.CORBA.ORB orb;
    private org.omg.Components.Deployment.Container container;
    private com.acme.Dinner.StatusInfoConsumer consumer;
    private org.omg.Components.EventConsumerBase consumerBase;
    private boolean isTypeCheckingOn;


    @Before
    public void init() throws InvalidObject {
        CORBATestManager testManager = (CORBATestManager)TestManager.getLocalInstance();
        
        this.orb = testManager.getORB();
        try {
            String containerIOR = 
                (String) testManager.getLocalProperty(CONTAINER_IOR);
            org.omg.CORBA.Object obj = this.orb.string_to_object(containerIOR);
            assertTrue(
                "TEST FAILED : exception raised during test initialisation!",
                obj != null);
            this.container =
                org.omg.Components.Deployment.ContainerHelper.narrow(obj);

            String nonCdmwConsumerIOR =
                (String) testManager.getLocalProperty(NON_CDMW_CONSUMER_IOR);
            obj = this.orb.string_to_object(nonCdmwConsumerIOR);
            assertTrue(
                "TEST FAILED : exception raised during test initialisation!",
                obj != null);
            this.consumer =
                com.acme.Dinner.StatusInfoConsumerHelper.narrow(obj);

            String consumerBaseIOR =
                (String) testManager.getLocalProperty(CONSUMER_BASE_IOR);
            obj = this.orb.string_to_object(consumerBaseIOR);
            assertTrue(
                "TEST FAILED : exception raised during test initialisation!",
                obj != null);
            this.consumerBase =
                org.omg.Components.EventConsumerBaseHelper.narrow(obj);

            java.lang.String isTypeCheckingOnStr = 
                System.getProperty(CCMSystemProperties.TYPECHECKING, "off");
            isTypeCheckingOn =
                (isTypeCheckingOnStr.equals("on"))? true : false;
            System.out.println("*** " + CCMSystemProperties.TYPECHECKING
                               + " set to " + isTypeCheckingOnStr);
        }

        catch (org.omg.CORBA.SystemException se) {
            fail("TEST FAILED : system exception raised during test initialisation!");
        }
        catch (Exception e) {
            fail("TEST FAILED : unexpected exception raised during test initialisation!");
        }        
    }    
    
    @Test
    public void doTests() {

        org.omg.Components.CCMHome philosopherHome = null;
        com.acme.Dinner.PhilosopherHome dinnerPhilosopherHome = null;
        org.omg.Components.CCMObject ccmPhilosopherComp = null;
        com.acme.Dinner.Philosopher dinnerPhilosopherComp1 = null;
        com.acme.Dinner.Philosopher dinnerPhilosopherComp2 = null;
        com.acme.Dinner.Philosopher dinnerPhilosopherComp3 = null;
        com.acme.Dinner.Philosopher dinnerPhilosopherComp4 = null;

        org.omg.Components.CCMHome observerHome = null;
        com.acme.Dinner.ObserverHome dinnerObserverHome = null;
        org.omg.Components.CCMObject ccmObserverComp = null;
        com.acme.Dinner.Observer dinnerObserverComp = null;
        org.omg.Components.EventConsumerBase infoConsumer = null;

        org.omg.Components.CCMHome forkHome = null;
        com.acme.Dinner.ForkHome dinnerForkHome = null;
        org.omg.Components.CCMObject ccmForkComp = null;
        com.acme.Dinner.ForkManager dinnerForkComp1 = null;
        com.acme.Dinner.ForkManager dinnerForkComp2 = null;
        com.acme.Dinner.ForkManager dinnerForkComp3 = null;
        com.acme.Dinner.ForkManager dinnerForkComp4 = null;
        org.omg.CORBA.Object forkFacetObj1 = null;
        org.omg.CORBA.Object forkFacetObj2 = null;
        org.omg.CORBA.Object forkFacetObj3 = null;
        org.omg.CORBA.Object forkFacetObj4 = null;

        org.omg.Components.CCMHome registratorHome = null;
        com.acme.Dinner.RegistratorHome dinnerRegistratorHome = null;
        org.omg.Components.CCMObject ccmRegistratorComp = null;
        com.acme.Dinner.Registrator dinnerRegistratorComp = null;

        org.omg.CORBA.Object statusinfoConsumerObj = null;

        org.omg.Components.Cookie recep_ck = null;
        org.omg.Components.Cookie ck1 = null;
        org.omg.Components.Cookie ck2 = null;
        org.omg.Components.Cookie ck2_bis = null;
        org.omg.Components.Cookie ck3 = null;
        org.omg.Components.Cookie ck4 = null;
        org.omg.Components.Cookie ck4_bis = null;
        org.omg.Components.Cookie ck4_ter = null;

        int timescale = TestManager.getTimescale();

        // This tests CIF classes
        //-------------------------------------------------
        System.out.println("Testing CIF classes");

        try {
            System.out.println("Install a PhilosopherHome");
            org.omg.CORBA.Any value = null;
            org.omg.Components.ConfigValue[] configValues;

            try {
                configValues = new org.omg.Components.ConfigValue[5];
                value = this.orb.create_any();
                ComponentKindValueHelper.insert(
                    value,
                    ComponentKindValue.SESSION);
                configValues[0] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.COMPONENT_KIND.value,
                        value);

                value = this.orb.create_any();
                ServantLifetimeValueHelper.insert(
                    value,
                    ServantLifetimeValue.CONTAINER_LIFETIME);
                configValues[1] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.SERVANT_LIFETIME.value,
                        value);

                value = this.orb.create_any();
                ThreadingPolicyValueHelper.insert(
                    value,
                    ThreadingPolicyValue.MULTITHREAD);
                configValues[2] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.THREADING_POLICY.value,
                        value);

                value = this.orb.create_any();
                value.insert_string(PHILOSOPHER_HOME_REP_ID);
                configValues[3] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.HOME_REPOSITORY_ID.value,
                        value);

                value = this.orb.create_any();
                value.insert_string(PHILOSOPHER_HOME_SERVANT_NAME);
                configValues[4] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.HOME_SERVANT_CLASSNAME.value,
                        value);

                philosopherHome =
                    this.container.install_home(
                        PHILOSOPHER_HOME_UID,
                        PHILOSOPHER_HOME_ENTRY_POINT,
                        configValues);
            } catch (org.omg.CORBA.BAD_PARAM bp) {
                fail(bp.toString());
            } catch (org.omg.Components.Deployment.InstallationFailure insf) {
                fail("Unexpected exception raised: Components::Deployment::InstallationFailure");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            } catch (org.omg.CORBA.UserException ue) {
                fail("Unexpected User exception raised: " + ue.toString());
            }

            System.out.println("Install a ObserverHome");

            try {
                configValues = new org.omg.Components.ConfigValue[6];
                value = this.orb.create_any();
                ComponentKindValueHelper.insert(
                    value,
                    ComponentKindValue.SESSION);
                configValues[0] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.COMPONENT_KIND.value,
                        value);

                value = this.orb.create_any();
                ServantLifetimeValueHelper.insert(
                    value,
                    ServantLifetimeValue.CONTAINER_LIFETIME);
                configValues[1] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.SERVANT_LIFETIME.value,
                        value);

                value = this.orb.create_any();
                ThreadingPolicyValueHelper.insert(
                    value,
                    ThreadingPolicyValue.MULTITHREAD);
                configValues[2] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.THREADING_POLICY.value,
                        value);

                value = this.orb.create_any();
                value.insert_string(OBSERVER_HOME_REP_ID);
                configValues[3] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.HOME_REPOSITORY_ID.value,
                        value);

                value = this.orb.create_any();
                value.insert_string(OBSERVER_HOME_SERVANT_NAME);
                configValues[4] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.HOME_SERVANT_CLASSNAME.value,
                        value);

                ValuetypeFactoryDescription[] seq
                    = new ValuetypeFactoryDescription[1];
                // Event StatusInfo valuetype
                ValuetypeFactoryDescription desc = new ValuetypeFactoryDescription();
                desc.repid = EVENT_STATUSINFO_REP_ID;
                desc.factory_entrypoint = EVENT_STATUSINFO_ENTRYPOINT;
                seq[0] = desc;

                value = this.orb.create_any();
                ValuetypeFactoryDescriptionSeqHelper.insert(value, seq);
                configValues[5] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.VALUETYPE_FACTORY_DEPENDENCIES.value,
                    value);

                observerHome =
                    this.container.install_home(
                        OBSERVER_HOME_UID,
                        OBSERVER_HOME_ENTRY_POINT,
                        configValues);
            } catch (org.omg.CORBA.BAD_PARAM bp) {
                fail(bp.toString());
            } catch (org.omg.Components.Deployment.InstallationFailure insf) {
                fail("Unexpected exception raised: Components::Deployment::InstallationFailure");
            }
            System.out.println("Install a ForkHome");

            try {
                configValues = new org.omg.Components.ConfigValue[5];
                value = this.orb.create_any();
                ComponentKindValueHelper.insert(
                    value,
                    ComponentKindValue.SESSION);
                configValues[0] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.COMPONENT_KIND.value,
                        value);

                value = this.orb.create_any();
                ServantLifetimeValueHelper.insert(
                    value,
                    ServantLifetimeValue.CONTAINER_LIFETIME);
                configValues[1] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.SERVANT_LIFETIME.value,
                        value);

                value = this.orb.create_any();
                ThreadingPolicyValueHelper.insert(
                    value,
                    ThreadingPolicyValue.MULTITHREAD);
                configValues[2] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.THREADING_POLICY.value,
                        value);

                value = this.orb.create_any();
                value.insert_string(FORKMANAGER_HOME_REP_ID);
                configValues[3] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.HOME_REPOSITORY_ID.value,
                        value);

                value = this.orb.create_any();
                value.insert_string(FORKMANAGER_HOME_SERVANT_NAME);
                configValues[4] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.HOME_SERVANT_CLASSNAME.value,
                        value);

                forkHome =
                    this.container.install_home(
                        FORKMANAGER_HOME_UID,
                        FORKMANAGER_HOME_ENTRY_POINT,
                        configValues);
            } catch (org.omg.CORBA.BAD_PARAM bp) {
                fail(bp.toString());
            } catch (org.omg.Components.Deployment.InstallationFailure insf) {
                fail("Unexpected exception raised: Components::Deployment::InstallationFailure");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            } catch (org.omg.CORBA.UserException ue) {
                fail("Unexpected User exception raised: " + ue.toString());
            }

            System.out.println("Install a RegistratorHome");

            try {
                configValues = new org.omg.Components.ConfigValue[5];
                value = this.orb.create_any();
                ComponentKindValueHelper.insert(
                    value,
                    ComponentKindValue.SESSION);
                configValues[0] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.COMPONENT_KIND.value,
                        value);

                value = this.orb.create_any();
                ServantLifetimeValueHelper.insert(
                    value,
                    ServantLifetimeValue.CONTAINER_LIFETIME);
                configValues[1] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.SERVANT_LIFETIME.value,
                        value);

                value = this.orb.create_any();
                ThreadingPolicyValueHelper.insert(
                    value,
                    ThreadingPolicyValue.MULTITHREAD);
                configValues[2] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.THREADING_POLICY.value,
                        value);

                value = this.orb.create_any();
                value.insert_string(REGISTRATOR_HOME_REP_ID);
                configValues[3] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.HOME_REPOSITORY_ID.value,
                        value);

                value = this.orb.create_any();
                value.insert_string(REGISTRATOR_HOME_SERVANT_NAME);
                configValues[4] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.HOME_SERVANT_CLASSNAME.value,
                        value);

                registratorHome =
                    this.container.install_home(
                        REGISTRATOR_HOME_UID,
                        REGISTRATOR_HOME_ENTRY_POINT,
                        configValues);
            } catch (org.omg.CORBA.BAD_PARAM bp) {
                fail(bp.toString());
            } catch (org.omg.Components.Deployment.InstallationFailure insf) {
                fail("Unexpected exception raised: Components::Deployment::InstallationFailure");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            } catch (org.omg.CORBA.UserException ue) {
                fail("Unexpected User exception raised: " + ue.toString());
            }

            System.out.println("Create a Philosopher component using create_component() operation");
            try {
                org.omg.Components.KeylessCCMHome keylessPhilosopherHome =
                    org.omg.Components.KeylessCCMHomeHelper.narrow(
                        philosopherHome);

                assertTrue(
                    "ERROR: Created home is not a keyless home!",
                    keylessPhilosopherHome != null);
                ccmPhilosopherComp =
                    keylessPhilosopherHome.create_component();

                dinnerPhilosopherComp1 =
                    com.acme.Dinner.PhilosopherHelper.narrow(
                        ccmPhilosopherComp);

                assertTrue(
                    "ERROR: Created component is not a com.acme.Dinner.Philosopher!",
                    dinnerPhilosopherComp1 != null);
            } catch (org.omg.Components.CreateFailure cf) {
                fail("Unexpected exception raised: CreateFailure");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Create other Philosopher component using create() operation");
            try {
                dinnerPhilosopherHome =
                    com.acme.Dinner.PhilosopherHomeHelper.narrow(
                        philosopherHome);

                assertTrue(
                    "Home is not a com.acme.Dinner.PhilosopherHome!",
                    dinnerPhilosopherHome != null);
                dinnerPhilosopherComp2 = dinnerPhilosopherHome.create();
                dinnerPhilosopherComp3 = dinnerPhilosopherHome.create();
                dinnerPhilosopherComp4 = dinnerPhilosopherHome.create();

                assertTrue(
                    "ERROR: dinnerPhilosopherComp2 is nil!",
                    dinnerPhilosopherComp2 != null);
                assertTrue(
                    "ERROR: dinnerPhilosopherComp3 is nil!",
                    dinnerPhilosopherComp3 != null);
                assertTrue(
                    "ERROR: dinnerPhilosopherComp4 is nil!",
                    dinnerPhilosopherComp4 != null);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Create a Observer component using create_component() operation");
            try {
                org.omg.Components.KeylessCCMHome keylessObserverHome =
                    org.omg.Components.KeylessCCMHomeHelper.narrow(
                        observerHome);

                assertTrue(
                    "ERROR: Created home is not a keyless home!",
                    keylessObserverHome != null);
                ccmObserverComp = keylessObserverHome.create_component();

                dinnerObserverComp = com.acme.Dinner.ObserverHelper.narrow(ccmObserverComp);

                assertTrue(
                    "ERROR: Created component is not a com.acme.Dinner.Observer!",
                    dinnerObserverComp != null);
            } catch (org.omg.Components.CreateFailure cf) {
                fail("Unexpected exception raised: CreateFailure");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Create a ForkManager component using create_component() operation");
            try {
                org.omg.Components.KeylessCCMHome keylessForkHome =
                    org.omg.Components.KeylessCCMHomeHelper.narrow(forkHome);

                assertTrue(
                    "ERROR: Created home is not a keyless home!",
                    keylessForkHome != null);
                ccmForkComp = keylessForkHome.create_component();

                dinnerForkComp1 = com.acme.Dinner.ForkManagerHelper.narrow(ccmForkComp);

                assertTrue(
                    "ERROR: Created component is not a com.acme.Dinner.ForkManager!",
                    dinnerForkComp1 != null);
            } catch (org.omg.Components.CreateFailure cf) {
                fail("Unexpected exception raised: CreateFailure");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Create other ForkManager component using create() operation");
            try {
                dinnerForkHome =
                    com.acme.Dinner.ForkHomeHelper.narrow(forkHome);

                assertTrue(
                    "Home is not a com.acme.Dinner.ForkHome!",
                    dinnerForkHome != null);
                dinnerForkComp2 = dinnerForkHome.create();
                dinnerForkComp3 = dinnerForkHome.create();
                dinnerForkComp4 = dinnerForkHome.create();

                assertTrue(
                    "ERROR: dinnerForkComp2 is nil!",
                    dinnerForkComp2 != null);
                assertTrue(
                    "ERROR: dinnerForkComp3 is nil!",
                    dinnerForkComp3 != null);
                assertTrue(
                    "ERROR: dinnerForkComp4 is nil!",
                    dinnerForkComp4 != null);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Create Registrator component using create() operation");
            try {
                dinnerRegistratorHome =
                    com.acme.Dinner.RegistratorHomeHelper.narrow(
                        registratorHome);

                assertTrue(
                    "Home is not a com.acme.Dinner.RegistratorHome!",
                    dinnerRegistratorHome != null);
                dinnerRegistratorComp = dinnerRegistratorHome.create();

                assertTrue(
                    "ERROR: dinnerRegistratorComp is nil!",
                    dinnerRegistratorComp != null);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Check Philosopher component is same using same_component() operation");
            try {
                assertTrue(
                    "ERROR: dinnerPhilosopherComp2 is not the same component!",
                    ! dinnerPhilosopherComp2.same_component(dinnerPhilosopherComp3));
                assertTrue(
                    "ERROR: dinnerPhilosopherComp3 is the same component!",
                    dinnerPhilosopherComp3.same_component(dinnerPhilosopherComp3));
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Testing Navigation operation on ForkManager1 component...");
            System.out.println("Get_all_facets operation on ForkManager component...");
            try {
                org.omg.Components.FacetDescription[] facetDesc =
                    dinnerForkComp1.get_all_facets();

                assertTrue(
                    "ERROR: Unexpected number of facet found for ForkManager component",
                    facetDesc.length == 1);
                forkFacetObj1 = facetDesc[0].facet_ref;
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Get_named_facets operation on ForkManager2 component...");
            try {
                String[] facetNameList = new String[1];
                facetNameList[0] = FORK_FACET_NAME;
                org.omg.Components.FacetDescription[] facetDesc =
                    dinnerForkComp2.get_named_facets(facetNameList);

                assertTrue(
                    "ERROR: Unexpected number of facet found for Philosopher component",
                    facetDesc.length == 1);
                forkFacetObj2 = facetDesc[0].facet_ref;
            } catch (org.omg.Components.InvalidName ine) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("provide_facet operation on ForkManager3 and ForkManager4 component...");
            try {
                forkFacetObj3 = dinnerForkComp3.provide_facet(FORK_FACET_NAME);
                forkFacetObj4 = dinnerForkComp4.provide_facet(FORK_FACET_NAME);

                assertTrue(
                    "ERROR: No facet found for ForkManager3 component",
                    forkFacetObj3 != null);
                assertTrue(
                    "ERROR: No facet found for ForkManager4 component",
                    forkFacetObj4 != null);
            } catch (org.omg.Components.InvalidName ine) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Testing Receptacle operation on Philosopher component...");
            System.out.println("Get_all_receptacles operation on Philosopher component...");
            try {
                org.omg.Components.ReceptacleDescription[] recepDesc =
                    dinnerPhilosopherComp1.get_all_receptacles();

                assertTrue(
                    "ERROR: Unexpected number of receptacle found for Philosopher component",
                    recepDesc.length == 3);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Connect operation on Philosopher components to ForkManager...");
            try {
                recep_ck =
                    dinnerPhilosopherComp1.connect(
                        FORK_RIGHT_RECEP_NAME,
                        forkFacetObj1);
                recep_ck =
                    dinnerPhilosopherComp1.connect(
                        FORK_LEFT_RECEP_NAME,
                        forkFacetObj2);
                recep_ck =
                    dinnerPhilosopherComp2.connect(
                        FORK_RIGHT_RECEP_NAME,
                        forkFacetObj2);
                recep_ck =
                    dinnerPhilosopherComp2.connect(
                        FORK_LEFT_RECEP_NAME,
                        forkFacetObj3);
                recep_ck =
                    dinnerPhilosopherComp3.connect(
                        FORK_RIGHT_RECEP_NAME,
                        forkFacetObj3);
                recep_ck =
                    dinnerPhilosopherComp3.connect(
                        FORK_LEFT_RECEP_NAME,
                        forkFacetObj4);
                recep_ck =
                    dinnerPhilosopherComp4.connect(
                        FORK_RIGHT_RECEP_NAME,
                        forkFacetObj4);
                recep_ck =
                    dinnerPhilosopherComp4.connect(
                        FORK_LEFT_RECEP_NAME,
                        forkFacetObj1);
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.Components.AlreadyConnected ac) {
                fail("Unexpected exception raised: AlreadyConnected");
            } catch (org.omg.Components.ExceededConnectionLimit ecl) {
                fail("Unexpected exception raised: ExceededConnectionLimit");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Connect operation again on Philosopher components to ForkManager...");
            try {
                org.omg.Components.Cookie ck =
                    dinnerPhilosopherComp1.connect(
                        FORK_RIGHT_RECEP_NAME,
                        forkFacetObj1);
                fail("Expected exception not raised");
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.Components.AlreadyConnected ac) {
                // Expected result
                System.out.println("Expected exception raise: AlreadyConnected");
            } catch (org.omg.Components.ExceededConnectionLimit ecl) {
                fail("Unexpected exception raised: ExceededConnectionLimit");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Connect operation on Philosopher components to Registrator...");
            try {
                org.omg.Components.Cookie ck =
                    dinnerPhilosopherComp1.connect(
                        REGISTER_RECEP_NAME,
                        dinnerRegistratorComp);
                ck =
                    dinnerPhilosopherComp2.connect(
                        REGISTER_RECEP_NAME,
                        dinnerRegistratorComp);
                ck =
                    dinnerPhilosopherComp3.connect(
                        REGISTER_RECEP_NAME,
                        dinnerRegistratorComp);
                ck =
                    dinnerPhilosopherComp4.connect(
                        REGISTER_RECEP_NAME,
                        dinnerRegistratorComp);
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.Components.AlreadyConnected ac) {
                fail("Unexpected exception raised: AlreadyConnected");
            } catch (org.omg.Components.ExceededConnectionLimit ecl) {
                fail("Unexpected exception raised: ExceededConnectionLimit");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Get_connections operation on Philosopher component...");
            try {
                org.omg.Components.ConnectionDescription[] conxDesc =
                    dinnerPhilosopherComp1.get_connections(REGISTER_RECEP_NAME);

                assertTrue(
                    "ERROR: Unexpected number of connection found for Philosopher component => "
                        + conxDesc.length,
                    conxDesc.length == 1);
            } catch (org.omg.Components.InvalidName ine) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Get_all_publishers operation on Philosopher component...");
            try {
                org.omg.Components.PublisherDescription[] publisherDesc =
                    dinnerPhilosopherComp1.get_all_publishers();

                assertTrue(
                    "ERROR: Unexpected number of publisher found for Philosopher component",
                    publisherDesc.length == 1);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Get_all_emitters operation on Philosopher component...");
            try {
                org.omg.Components.EmitterDescription[] emitterDesc =
                    dinnerPhilosopherComp1.get_all_emitters();

                assertTrue(
                    "ERROR: Unexpected number of emitter found for Philosopher component",
                    emitterDesc.length == 1);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Get_all_consumers operation on Observer component...");
            try {
                org.omg.Components.ConsumerDescription[] consumerDesc =
                    dinnerObserverComp.get_all_consumers();

                assertTrue(
                    "ERROR: Unexpected number of consumers found for Observer component",
                    consumerDesc.length == 1);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Get_named_consumers operation on Observer component...");
            try {
                String[] consumerNameList = new String[1];
                consumerNameList[0] = INFO_CONSUMER_NAME;
                org.omg.Components.ConsumerDescription[] consumerDesc =
                    dinnerObserverComp.get_named_consumers(consumerNameList);

                assertTrue(
                    "ERROR: Unexpected number of consumers found for Observer component",
                    consumerDesc.length == 1);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Get_consumers operation on Observer component...");
            try {
                infoConsumer =
                    dinnerObserverComp.get_consumer(INFO_CONSUMER_NAME);

                assertTrue(
                    "ERROR: no consumer found for Observer component",
                    infoConsumer != null);
            } catch (org.omg.Components.InvalidName ine) {
                fail("ERROR: InvalidName exception raised!");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Subscribe operation on Philosopher components"
                               + " with a wrong type of consumer...");
            try {
                ck1 = dinnerPhilosopherComp1.subscribe(
                    INFO_PUBLISHER_NAME,
                    consumerBase
                    );
                if (isTypeCheckingOn) {
                    fail("Exception was expected but not raised");
                }
            }
            catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            }
            catch (org.omg.Components.InvalidConnection ic) {
                if (isTypeCheckingOn)
                    System.out.println("Expected exception raised: InvalidConnection");
                else
                    fail("Unexpected exception raised: InvalidConnection");
            }
            catch (org.omg.Components.ExceededConnectionLimit ecl) {
                fail("Unexpected exception raised: ExceededConnectionLimit");
            }
            catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            if (! isTypeCheckingOn) {
                System.out.println("Unsubscribe operation on Philosopher1 "
                                   + "components from Observer...");
                try {
                    dinnerPhilosopherComp1.unsubscribe(
                        INFO_PUBLISHER_NAME, ck1);
                }
                catch (org.omg.Components.InvalidName in) {
                    fail("Unexpected exception raised: InvalidName");
                }
                catch (org.omg.Components.InvalidConnection ic) {
                    fail("Unexpected exception raised: InvalidConnection");
                }
                catch (org.omg.CORBA.SystemException se) {
                    fail("Unexpected system exception raised: "
                         + se.toString());
                }
            }

            System.out.println("Subscribe operation on Philosopher components from Observer...");
            try {
                ck1 =
                    dinnerPhilosopherComp1.subscribe(
                        INFO_PUBLISHER_NAME,
                        infoConsumer);
                ck2 =
                    dinnerPhilosopherComp2.subscribe(
                        INFO_PUBLISHER_NAME,
                        infoConsumer);
                ck3 =
                    dinnerPhilosopherComp3.subscribe(
                        INFO_PUBLISHER_NAME,
                        infoConsumer);
                ck4 =
                    dinnerPhilosopherComp4.subscribe(
                        INFO_PUBLISHER_NAME,
                        infoConsumer);
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.Components.ExceededConnectionLimit ecl) {
                fail("Unexpected exception raised: ExceededConnectionLimit");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Subscribe operation on Philosopher components from a non cdmw consumer...");
            try {
                ck4_bis =
                    dinnerPhilosopherComp4.subscribe(
                        INFO_PUBLISHER_NAME,
                        this.consumer);
                ck4_ter =
                    dinnerPhilosopherComp4.subscribe(
                        INFO_PUBLISHER_NAME,
                        this.consumer);
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.Components.ExceededConnectionLimit ecl) {
                fail("Unexpected exception raised: ExceededConnectionLimit");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Try to unsubscribe operation on Philosopher1 components with invalid cookie...");
            try {
                dinnerPhilosopherComp1.unsubscribe(
                    INFO_PUBLISHER_NAME,
                    ck2);
                fail("Expected exception not raised");
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.InvalidConnection ic) {
                System.out.println("Expected exception raised: InvalidConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Try to unsubscribe operation on Philosopher1 components with invalid name...");
            try {
                dinnerPhilosopherComp1.unsubscribe(
                    CONFIG_EMITTER_NAME,
                    ck1);
                fail("Expected exception not raised");
            } catch (org.omg.Components.InvalidName in) {
                System.out.println("Expected exception raised: InvalidName");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Unsubscribe operation on Philosopher1 components from Observer...");
            try {
                dinnerPhilosopherComp1.unsubscribe(
                    INFO_PUBLISHER_NAME,
                    ck1);
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("try to subscribe operation on Philosopher1 components with invalid name...");
            try {
                ck1 =
                    dinnerPhilosopherComp1.subscribe(
                        CONFIG_EMITTER_NAME,
                        infoConsumer);
                fail("Expected exception not raised");
            } catch (org.omg.Components.InvalidName in) {
                System.out.println("Expected exception raised: InvalidName");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.Components.ExceededConnectionLimit ecl) {
                fail("Unexpected exception raised: ExceededConnectionLimit");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Subscribe operation on Philosopher1 components from Observer...");
            try {
                ck1 =
                    dinnerPhilosopherComp1.subscribe(
                        INFO_PUBLISHER_NAME,
                        infoConsumer);
                ck2_bis =
                    dinnerPhilosopherComp2.subscribe(
                        INFO_PUBLISHER_NAME,
                        infoConsumer);
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.Components.ExceededConnectionLimit ecl) {
                fail("Unexpected exception raised: ExceededConnectionLimit");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Try to call connect_consumer operation on Philosopher1 components with invalid name...");
            try {
                dinnerPhilosopherComp1.connect_consumer(
                    INFO_PUBLISHER_NAME,
                    infoConsumer);
                fail("Expected exception not raised");
            } catch (org.omg.Components.InvalidName in) {
                System.out.println("Expected exception raised: InvalidName");
            } catch (org.omg.Components.AlreadyConnected ac) {
                fail("Unexpected exception raised: AlreadyConnected");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Try to call connect_consumer operation on "
                               + "Philosopher1 components with a wrong type"
                               + " of consumer...");
            try {
                dinnerPhilosopherComp1.connect_consumer(
                    CONFIG_EMITTER_NAME,
                    consumerBase
                    );
                if (isTypeCheckingOn)
                    fail("Exception was expected but not raised");
            }
            catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            }
            catch (org.omg.Components.AlreadyConnected ac) {
                fail("Unexpected exception raised: AlreadyConnected");
            }
            catch (org.omg.Components.InvalidConnection in) {
                if (isTypeCheckingOn)
                    System.out.println("Expected exception raised: InvalidConnection");
                else
                    fail("Expected exception raised: InvalidConnection");
            }
            catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            if (! isTypeCheckingOn) {
                System.out.println("Disconnect_consumer operation on Philo"
                                   + "sopher1 components from Observer...");
                try {
                    org.omg.Components.EventConsumerBase consumer =
                        dinnerPhilosopherComp1.disconnect_consumer(
                            CONFIG_EMITTER_NAME);
                }
                catch (org.omg.Components.InvalidName in) {
                    fail("Unexpected exception raised: InvalidName");
                }
                catch (org.omg.Components.NoConnection nc) {
                    fail("Unexpected exception raised: NoConnection");
                }
                catch (org.omg.CORBA.SystemException se) {
                    fail("Unexpected system exception raised: "
                         + se.toString());
                }
            }

            System.out.println("Connect_consumer operation on Philosopher1 "
                               + "components from Observer...");
            try {
                dinnerPhilosopherComp1.connect_consumer(
                    CONFIG_EMITTER_NAME,
                    infoConsumer);
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.AlreadyConnected ac) {
                fail("Unexpected exception raised: AlreadyConnected");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Connect_consumer operation on Philosopher3 components from a non Cdmw Consumer...");
            try {
                dinnerPhilosopherComp3.connect_consumer(
                    CONFIG_EMITTER_NAME,
                    this.consumer);
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.AlreadyConnected ac) {
                fail("Unexpected exception raised: AlreadyConnected");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Try to connect_consumer again operation on Philosopher1 components from Observer...");
            try {
                dinnerPhilosopherComp1.connect_consumer(
                    CONFIG_EMITTER_NAME,
                    infoConsumer);
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.AlreadyConnected ac) {
                fail("Unexpected exception raised: AlreadyConnected");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Try to connect_consumer again operation on Philosopher1 components from Observer...");
            try {
                dinnerPhilosopherComp1.connect_consumer(
                    CONFIG_EMITTER_NAME,
                    consumer);
                fail("Expected exception not raised");
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.AlreadyConnected ac) {
                System.out.println("Expected exception raised: AlreadyConnected");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Disconnect_consumer operation on Philosopher1 components from Observer...");
            try {
                org.omg.Components.EventConsumerBase consumer =
                    dinnerPhilosopherComp1.disconnect_consumer(
                        CONFIG_EMITTER_NAME);
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.NoConnection nc) {
                fail("Unexpected exception raised: NoConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Disconnect_consumer operation again on Philosopher1 components from Observer...");
            try {
                org.omg.Components.EventConsumerBase consumer =
                    dinnerPhilosopherComp1.disconnect_consumer(
                        CONFIG_EMITTER_NAME);
                fail("Expected exception not raised");
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.NoConnection nc) {
                System.out.println("Expected exception raised: NoConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Try to connect_consumer again operation on Philosopher1 components from Observer...");
            try {
                dinnerPhilosopherComp1.connect_consumer(
                    CONFIG_EMITTER_NAME,
                    infoConsumer);
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.AlreadyConnected ac) {
                fail("Unexpected exception raised: AlreadyConnected");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Get_ccmHome operation on Observer component...");
            try {
                org.omg.Components.CCMHome ccmHome =
                    dinnerObserverComp.get_ccm_home();

                assertTrue(
                    "ERROR: Unexpected home for Observer component",
                    ccmHome._is_equivalent(observerHome));
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Get_all_ports operation on Philosopher component...");
            try {
                org.omg.Components.ComponentPortDescription portDesc =
                    dinnerPhilosopherComp2.get_all_ports();

                org.omg.Components.FacetDescription[] facetDesc =
                    portDesc.facets;
                org.omg.Components.ReceptacleDescription[] recepDesc =
                    portDesc.receptacles;
                org.omg.Components.ConsumerDescription[] consumerDesc =
                    portDesc.consumers;
                org.omg.Components.EmitterDescription[] emitterDesc =
                    portDesc.emitters;
                org.omg.Components.PublisherDescription[] publisherDesc =
                    portDesc.publishers;

                assertTrue(
                    "ERROR: unexpected number of facet for Philosopher component",
                    facetDesc.length == 0);
                assertTrue(
                    "ERROR: unexpected number of facet for Philosopher component",
                    facetDesc.length == 0);
                assertTrue(
                    "ERROR: unexpected number of receptacle for Philosopher component",
                    recepDesc.length == 3);
                assertTrue(
                    "ERROR: unexpected number of consumer for Philosopher component",
                    consumerDesc.length == 0);
                assertTrue(
                    "ERROR: unexpected number of publisher for Philosopher component",
                    publisherDesc.length == 1);
                assertTrue(
                    "ERROR: unexpected number of emitter for Philosopher component",
                    emitterDesc.length == 1);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("ConfigurationComplete operation on Philosopher components...");
            try {
                System.out.println("ConfigurationComplete operation on Philosopher1 components...");
                dinnerPhilosopherComp1.configuration_complete();

                OS.sleep(timescale * 1000);

                System.out.println("ConfigurationComplete operation on Philosopher2 components...");
                dinnerPhilosopherComp2.configuration_complete();

                OS.sleep(timescale * 1000);

                System.out.println("ConfigurationComplete operation on Philosopher3 components...");
                dinnerPhilosopherComp3.configuration_complete();

                OS.sleep(timescale * 1000);

                System.out.println("ConfigurationComplete operation on Philosopher4 components...");
                dinnerPhilosopherComp4.configuration_complete();
                OS.sleep(timescale * 1000);

            } catch (org.omg.Components.InvalidConfiguration ic) {
                fail("Unexpected exception raised: InvalidConfiguration");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("ConfigurationComplete operation on Observer component...");
            try {
                dinnerObserverComp.configuration_complete();
            } catch (org.omg.Components.InvalidConfiguration ic) {
                fail("Unexpected exception raised: InvalidConfiguration");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Get name of Philosopher component1...");
            try {
                String philosopherName = dinnerPhilosopherComp1.name();
                System.out.println("Philosopher1 name is " + philosopherName);

                philosopherName = dinnerPhilosopherComp2.name();
                System.out.println("Philosopher2 name is " + philosopherName);

                philosopherName = dinnerPhilosopherComp3.name();
                System.out.println("Philosopher3 name is " + philosopherName);

                philosopherName = dinnerPhilosopherComp4.name();
                System.out.println("Philosopher4 name is " + philosopherName);

            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            OS.sleep(timescale * 30000);

            System.out.println("Disconnect operation on Philosopher component...");
            try {
                dinnerPhilosopherComp1.disconnect(FORK_LEFT_RECEP_NAME, null);
                dinnerPhilosopherComp2.disconnect(FORK_LEFT_RECEP_NAME, null);
                dinnerPhilosopherComp3.disconnect(FORK_LEFT_RECEP_NAME, null);
                dinnerPhilosopherComp4.disconnect(FORK_LEFT_RECEP_NAME, null);
                dinnerPhilosopherComp1.disconnect(FORK_RIGHT_RECEP_NAME, null);
                dinnerPhilosopherComp2.disconnect(FORK_RIGHT_RECEP_NAME, null);
                dinnerPhilosopherComp3.disconnect(FORK_RIGHT_RECEP_NAME, null);
                dinnerPhilosopherComp4.disconnect(FORK_RIGHT_RECEP_NAME, null);
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.Components.CookieRequired cr) {
                fail("Unexpected exception raised: CookieRequired");
            } catch (org.omg.Components.NoConnection nc) {
                fail("Unexpected exception raised: NoConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Disconnect operation on Philosopher components...");
            try {
                dinnerPhilosopherComp1.disconnect(REGISTER_RECEP_NAME, null);
                dinnerPhilosopherComp2.disconnect(REGISTER_RECEP_NAME, null);
                dinnerPhilosopherComp3.disconnect(REGISTER_RECEP_NAME, null);
                dinnerPhilosopherComp4.disconnect(REGISTER_RECEP_NAME, null);
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.Components.CookieRequired cr) {
                fail("Unexpected exception raised: CookieRequired");
            } catch (org.omg.Components.NoConnection nc) {
                fail("Unexpected exception raised: NoConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Unsubscribe operation on Philosopher components...");
            try {
                System.out.println("Unsubscribe operation with ck1...");
                dinnerPhilosopherComp1.unsubscribe(INFO_PUBLISHER_NAME, ck1);
                System.out.println("Unsubscribe operation with ck2...");
                dinnerPhilosopherComp2.unsubscribe(INFO_PUBLISHER_NAME, ck2);
                System.out.println("Unsubscribe operation with ck3...");
                dinnerPhilosopherComp3.unsubscribe(INFO_PUBLISHER_NAME, ck3);
                System.out.println("Unsubscribe operation with ck4...");
                dinnerPhilosopherComp4.unsubscribe(INFO_PUBLISHER_NAME, ck4);
                System.out.println("Unsubscribe operation with ck4_ter...");
                dinnerPhilosopherComp4.unsubscribe(
                    INFO_PUBLISHER_NAME,
                    ck4_ter);
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Unsubscribe operation for duplicate connection"
                               + " on Philosopher components. Shall raised "
                               + "InvalidConnection exception.");
            try {
                System.out.println("Unsubscribe operation with ck2_bis...");
                dinnerPhilosopherComp2.unsubscribe(
                    INFO_PUBLISHER_NAME,
                    ck2_bis);
                fail("Expected exception not raised");
            }
            catch (org.omg.Components.InvalidName in) {
                fail("Expected exception raised: InvalidName");
            }
            catch (org.omg.Components.InvalidConnection ic) {
                System.out.println("Unexpected exception raised: InvalidConnection");
            }
            catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            try {
                System.out.println("Unsubscribe operation with ck4_bis...");
                dinnerPhilosopherComp4.unsubscribe(
                    INFO_PUBLISHER_NAME,
                    ck4_bis);
                fail("Expected exception not raised");
            }
            catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            }
            catch (org.omg.Components.InvalidConnection ic) {
                System.out.println("Unexpected exception raised: InvalidConnection");
            }
            catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Disconnect_consumer operation on Philosopher components...");
            try {
                org.omg.Components.EventConsumerBase consumer =
                    dinnerPhilosopherComp1.disconnect_consumer(
                        CONFIG_EMITTER_NAME);
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.NoConnection nc) {
                fail("Unexpected exception raised: NoConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Disconnect_consumer operation on Philosopher3 components from Observer...");
            try {
                org.omg.Components.EventConsumerBase consumer =
                    dinnerPhilosopherComp3.disconnect_consumer(
                        CONFIG_EMITTER_NAME);
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.NoConnection nc) {
                fail("Unexpected exception raised: NoConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Remove operation on the Philosopher home...");
            try {
                this.container.remove_home(dinnerPhilosopherHome);
            } catch (org.omg.Components.RemoveFailure rf) {
                fail("Unexpected exception raised: NoKeyAvailable");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Remove operation on the Observer home...");
            try {
                this.container.remove_home(observerHome);
            } catch (org.omg.Components.RemoveFailure rf) {
                fail("Unexpected exception raised: NoKeyAvailable");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Remove operation on the Fork home...");
            try {
                this.container.remove_home(forkHome);
            } catch (org.omg.Components.RemoveFailure rf) {
                fail("Unexpected exception raised: NoKeyAvailable");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Remove operation on the Registrator home...");
            try {
                this.container.remove_home(registratorHome);
            } catch (org.omg.Components.RemoveFailure rf) {
                fail("Unexpected exception raised: NoKeyAvailable");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

        } catch (Exception e) {
            e.printStackTrace();
            fail("Unexpected exception raised: " + e.toString());
        }
    }
}
