/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <testphilosophers/TestEventTimestamps.hpp>

namespace
{
    //ATTENTION: do NOT use '%' or any other special to sprintf() function symbols in these constants
    static const std::string ARGC_PROPERTY_NAME = "TestEventTimestamps_Argc";
    static const std::string ARGV_PROPERTY_NAME = "TestEventTimestamps_Argv";
}



namespace Cdmw
{

    namespace OsSupport
    {
        /**
        *   @param formatString Format string according to strftime(); format
        *   @return formatted current LOCAL date string
        */
        extern
            std::string get_fmt_current_localtime(char* formatString);

    }// OsSupport
}// Cdmw


namespace Cdmw
{
    namespace CCM
    {
        namespace CIF
        {

    
    CPPUNIT_TEST_SUITE_REGISTRATION( TestEventTimestamps );
    
    TestEventTimestamps::TestEventTimestamps()
    {}
    
    
    TestEventTimestamps::~TestEventTimestamps()
        throw()
    {}

    
    void TestEventTimestamps::verify_time_zone()
    {
        int argc;
        sscanf(Cdmw::TestUtils::TestManager::instance()->get_property(ARGC_PROPERTY_NAME).c_str(), "%d", &argc);

        std::string argv;
        char argv_number_str[4];

        for(int i = 1; i < argc; ++i)
        {
            sprintf(argv_number_str, "_%d", i);

            argv = Cdmw::TestUtils::TestManager::instance()->get_property(ARGV_PROPERTY_NAME + std::string(argv_number_str));

            TEST_INFO("Checking " + argv + " ...");

            bool timestamp_found = false;

            std::ifstream logFile(argv.c_str());

            if(logFile == 0)
            {
                CPPUNIT_FAIL("Could not open file for verification: " + argv);
            }

            char buf[256];
            const std::string received_at = "received at: ";
            const std::string issued_at   = "issued at: ";
            std::string issued_time, received_time, current_time;

            current_time = Cdmw::OsSupport::get_fmt_current_localtime("%Y-%m-%dT%H:%M:%S");

            while(!logFile.getline(buf, 256).eof())
            {                
                std::string logRecord(buf);
                unsigned int begin_pos = logRecord.find(received_at);
    
                if(begin_pos != std::string::npos)
                    {
                        received_time = logRecord.substr(begin_pos + received_at.length(), 19);
                        
                        begin_pos = logRecord.find(issued_at);
                        
                        if(begin_pos != std::string::npos)
                            {
                                timestamp_found = true;
                                
                                issued_time = logRecord.substr(begin_pos + issued_at.length(), 19);
                                
                                if ( !(is_current_timezone(current_time, received_time) && 
                                       is_current_timezone(current_time, issued_time)) )
                                    {
                                        CPPUNIT_FAIL("'" + logRecord + "' is not in current timezone !");
                                    }
                            }
                    }

            }// while(logFile)

            
            logFile.close();
            CPPUNIT_ASSERT(timestamp_found);
        } // for()

    }// verify_time_zone


    bool TestEventTimestamps::is_current_timezone(const std::string& current_time, const std::string& time2)
    {
        cdmw_tm tm_current, tm_time2;

        //parse current_time string to cdmw_tm struct
        sscanf(current_time.c_str(), "%d-%d-%dT%d:%d:%d", 
            &tm_current.tm_year, 
            &tm_current.tm_mon, 
            &tm_current.tm_mday, 
            &tm_current.tm_hour, 
            &tm_current.tm_min, 
            &tm_current.tm_sec);

        //parse time2 string to cdmw_tm struct
        sscanf(time2.c_str(), "%d-%d-%dT%d:%d:%d", 
            &tm_time2.tm_year, 
            &tm_time2.tm_mon, 
            &tm_time2.tm_mday, 
            &tm_time2.tm_hour, 
            &tm_time2.tm_min, 
            &tm_time2.tm_sec);

        int mins_difference = (tm_current.tm_hour - tm_time2.tm_hour)*60 + (tm_current.tm_min - tm_time2.tm_min);

        return (tm_current.tm_year == tm_time2.tm_year &&
                tm_current.tm_mon == tm_time2.tm_mon &&
                tm_current.tm_mday == tm_time2.tm_mday &&
                (mins_difference >= 0 && mins_difference < 60));
    }

        };// CIF
    };// CCM
};// Cdmw


int main(int argc, char *argv[])
{
    if(argc <= 1)
    {
        std::cout<< "Usage: <program> logFile1 [logfile2 ...]" << std::endl;
        return 1;
    }

    std::cout<< "\n\n\n" << std::endl;
    std::cout<< "*******************************************" << std::endl;
    std::cout<< "*   Starting event timestamps test        *" << std::endl;
    std::cout<< "*                                         *" << std::endl;
    std::cout<< "*******************************************" << std::endl;

    char argc_str[3];
    sprintf(argc_str, "%d", argc);

    Cdmw::TestUtils::TestManager::instance()->set_property(ARGC_PROPERTY_NAME, argc_str);

    char argv_number[4];
    for(int i = 0; i < argc; ++i)
    {
        sprintf(argv_number, "_%d", i);
        std::string argv_property_name = ARGV_PROPERTY_NAME + std::string(argv_number);

        Cdmw::TestUtils::TestManager::instance()->set_property(argv_property_name, argv[i]);
    }

    Cdmw::TestUtils::TestManager::instance()->run_tests();
    Cdmw::TestUtils::TestManager::instance()->summary();

    return 0;
}

