/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ccm.cif.test;

import org.junit.Test;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;


public class TestEventTimestamps {
    public static final String ARGS_PROPERTY_NAME = "TestEventTimestamps_Args";
    public static final String ARGC_PROPERTY_NAME = "TestEventTimestamps_Argc";

    public TestEventTimestamps()
    {}
    
    public static void main(String args[]) {
        for(int y = 0; y < args.length; ++y) {
            System.out.println(args[y]);
        }
        if(args.length <= 1) {
            System.out.println("Usage: <program> logFile1 [logfile2 ...]");
            return;
        }

        System.out.println("\n\n\n");
        System.out.println("*******************************************");
        System.out.println("*   Starting event timestamps test        *");
        System.out.println("*                                         *");
        System.out.println("*******************************************");

        System.setProperty(ARGC_PROPERTY_NAME, String.valueOf(args.length));

        for(int i = 0; i < args.length; ++i) {
            System.setProperty(ARGS_PROPERTY_NAME + String.valueOf(i), args[i]);
        }

        cdmw.testutils.TestManager.instance().init("cdmw.ccm.cif.test.TestEventTimestamps");
        cdmw.testutils.TestManager.instance().addTest(TestEventTimestamps.class);
        cdmw.testutils.TestManager.instance().runTests();
        cdmw.testutils.TestManager.instance().printSummary();

        return;
    }


    @Test
    public void verifyTimeZoneTest() {
        boolean test_ok = true;
        int argc = Integer.parseInt(System.getProperty(ARGC_PROPERTY_NAME));

        for(int i = 0; i < argc; ++i) {
            String argv = System.getProperty(ARGS_PROPERTY_NAME + String.valueOf(i));
            java.io.BufferedReader logFile;

            boolean timestamp_found = false;
            boolean has_error = false;
            final String received_at = "received at: ";
            final String issued_at   = "issued at: ";

            java.util.Date issued_time, received_time, current_time;
            java.text.DateFormat date_formatter = new java.text.SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
            current_time = new java.util.Date();

            String logRecord = null;

            try {
                logFile = new java.io.BufferedReader(new java.io.FileReader(argv));

                while((logRecord = logFile.readLine()) != null) { 
                    try {
                            //Search for "received at: " substring in logRecord string
                        int begin_pos = logRecord.indexOf(received_at);

                        if(begin_pos != -1) {
                            //skip "received at: " substring
                            begin_pos += received_at.length();


                            received_time = 
                                date_formatter.parse(logRecord.substring(begin_pos, begin_pos + 19));
                    

                            //Search for "issued at: " substring in logRecord string
                            begin_pos = logRecord.indexOf(issued_at);

                            if(begin_pos != -1) {
                                //skip "issued at: " substring
                                begin_pos += issued_at.length();
                            
                                issued_time = 
                                    date_formatter.parse(logRecord.substring(begin_pos, begin_pos + 19));                              
                    
                                timestamp_found = true;

                                //get elapsed minutes since event's timestamp
                                long received_mins_diff = (current_time.getTime() - received_time.getTime()) / 60000;
                                long issued_mins_diff = (current_time.getTime() - issued_time.getTime()) / 60000;

                                //if elapsed mins greater than hour, than the timezone is wrong
                                if( received_mins_diff < 60 && received_mins_diff >= 0
                                    && issued_mins_diff < 60 && issued_mins_diff >= 0) {


                                } else {
                                    fail(logRecord + "' is not in current timezone !");
                                }
                            }
                        }

                    } catch(java.text.ParseException e) {
                        fail("Wrong received time format in: " + logRecord);
                    }

                }// while (logFile)

                logFile.close();
                assertTrue(timestamp_found);
            }
            catch(java.io.FileNotFoundException e) {
                fail("File " + argv + " not found !");
            }
            catch (java.io.IOException e) {
                System.out.println("Unexpected exception raised while reading file: " + argv);
            }

        }// for i (command line parameters)
    } // verifyTimeZone
}
