/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.cif.test;

import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import cdmw.ossupport.OS;
import cdmw.testutils.CORBATestManager;
import cdmw.testutils.TestManager;
import cdmw.testutils.CORBATestManager.InvalidObject;
import cdmw.ccm.container.ConfigValueImpl;

import com.thalesgroup.CdmwDeployment.ComponentKindValue;
import com.thalesgroup.CdmwDeployment.ComponentKindValueHelper;
import com.thalesgroup.CdmwDeployment.ServantLifetimeValue;
import com.thalesgroup.CdmwDeployment.ServantLifetimeValueHelper;
import com.thalesgroup.CdmwDeployment.ThreadingPolicyValue;
import com.thalesgroup.CdmwDeployment.ThreadingPolicyValueHelper;
import com.thalesgroup.CdmwDeployment.ValuetypeFactoryDescription;
import com.thalesgroup.CdmwDeployment.ValuetypeFactoryDescriptionHelper;
import com.thalesgroup.CdmwDeployment.ValuetypeFactoryDescriptionSeqHelper;

public class TypedPhiloTest {

    public final static String PHILOSOPHER_HOME_REP_ID =
        "IDL:acme.com/Cdmw_Dinner/PhilosopherHome:1.0";
    public final static String PHILOSOPHER_HOME_SERVANT_NAME =
        "Cdmw.CCM.CIF.CdmwDinner.SessionPhilosopherHome_impl";
    public final static String PHILOSOPHER_HOME_UID = "PhilosopherHome_1";
    public final static String PHILOSOPHER_HOME_ENTRY_POINT =
        "cdmw.ccm.cif.test.CCMPhilosopherHomeImpl.createCCMPhilosopherHome";

    public final static String OBSERVER_HOME_REP_ID =
        "IDL:acme.com/Cdmw_Dinner/ObserverHome:1.0";
    public final static String OBSERVER_HOME_SERVANT_NAME =
        "Cdmw.CCM.CIF.CdmwDinner.SessionObserverHome_impl";
    public final static String OBSERVER_HOME_UID = "ObserverHome_1";
    public final static String OBSERVER_HOME_ENTRY_POINT =
        "cdmw.ccm.cif.test.CCMObserverHomeImpl.createCCMObserverHome";

    public final static String FORKMANAGER_HOME_REP_ID =
        "IDL:acme.com/Cdmw_Dinner/ForkHome:1.0";
    public final static String FORKMANAGER_HOME_SERVANT_NAME =
        "Cdmw.CCM.CIF.CdmwDinner.SessionForkHome_impl";
    public final static String FORKMANAGER_HOME_UID = "ForkHome_1";
    public final static String FORKMANAGER_HOME_ENTRY_POINT =
        "cdmw.ccm.cif.test.CCMForkHomeImpl.createCCMForkHome";

    public final static String REGISTRATOR_HOME_REP_ID =
        "IDL:acme.com/Cdmw_Dinner/RegistratorHome:1.0";
    public final static String REGISTRATOR_HOME_SERVANT_NAME =
        "Cdmw.CCM.CIF.CdmwDinner.SessionRegistratorHome_impl";
    public final static String REGISTRATOR_HOME_UID = "RegistratorHome_1";
    public final static String REGISTRATOR_HOME_ENTRY_POINT =
        "cdmw.ccm.cif.test.CCMRegistratorHomeImpl.createCCMRegistratorHome";

    public final static String FORK_FACET_NAME = "the_fork";

    public final static String FORK_RIGHT_RECEP_NAME = "right";
    public final static String FORK_LEFT_RECEP_NAME = "left";
    public final static String REGISTER_RECEP_NAME = "register";

    public final static String INFO_CONSUMER_NAME = "info";
    public final static String INFO_PUBLISHER_NAME = "info";
    public final static String CONFIG_EMITTER_NAME = "config";

    public final static String EVENT_STATUSINFO_REP_ID =
        "IDL:acme.com/Dinner/StatusInfo:1.0";
    public final static String EVENT_STATUSINFO_ENTRYPOINT =
        "cdmw.ccm.cif.test.StatusInfoFactoryImpl.createStatusInfoFactory";

    static final String CONTAINER_IOR =
        "cdmw.ccm.cif.test.TypedPhiloTest.CONTAINER_IOR";
    static final String CONSUMER_IOR =
        "cdmw.ccm.cif.test.TypedPhiloTest.CONSUMER_IOR";
    
    private org.omg.CORBA.ORB orb;
    private org.omg.Components.Deployment.Container container;
    private com.acme.Dinner.StatusInfoConsumer consumer;


    @Before
    public void init() throws InvalidObject {

        CORBATestManager testManager =
            (CORBATestManager) TestManager.getLocalInstance();
        this.orb = testManager.getORB();
        try {
            String containerIOR =
                (String) testManager.getLocalProperty(CONTAINER_IOR);
            org.omg.CORBA.Object obj = this.orb.string_to_object(containerIOR);

            assertTrue(
                "TEST FAILED : exception raised during test initialisation!",
                obj != null);
            this.container =
                org.omg.Components.Deployment.ContainerHelper.narrow(obj);

            String consumerIOR =
                (String) testManager.getLocalProperty(CONSUMER_IOR);
            obj = this.orb.string_to_object(consumerIOR);
            assertTrue(
                "TEST FAILED : exception raised during test initialisation!",
                obj != null);
            this.consumer =
                com.acme.Dinner.StatusInfoConsumerHelper.narrow(obj);
        }
        catch (org.omg.CORBA.SystemException se) {
            fail("TEST FAILED : system exception raised "
                 + "during test initialisation!");
        }
        catch (Exception e) {
            fail("TEST FAILED : unexpected exception raised "
                 + "during test initialisation!");
        }
    }

    @Test
    public void doTests() {
        org.omg.Components.CCMHome philosopherHome = null;
        com.acme.Dinner.PhilosopherHome dinnerPhilosopherHome = null;
        org.omg.Components.CCMObject ccmPhilosopherComp = null;
        com.acme.Dinner.Philosopher dinnerPhilosopherComp1 = null;
        com.acme.Dinner.Philosopher dinnerPhilosopherComp2 = null;
        com.acme.Dinner.Philosopher dinnerPhilosopherComp3 = null;
        com.acme.Dinner.Philosopher dinnerPhilosopherComp4 = null;

        org.omg.Components.CCMHome observerHome = null;
        com.acme.Dinner.ObserverHome dinnerObserverHome = null;
        org.omg.Components.CCMObject ccmObserverComp = null;
        com.acme.Dinner.Observer dinnerObserverComp = null;
        com.acme.Dinner.StatusInfoConsumer infoConsumer = null;

        org.omg.Components.CCMHome forkHome = null;
        com.acme.Dinner.ForkHome dinnerForkHome = null;
        org.omg.Components.CCMObject ccmForkComp = null;
        com.acme.Dinner.ForkManager dinnerForkComp1 = null;
        com.acme.Dinner.ForkManager dinnerForkComp2 = null;
        com.acme.Dinner.ForkManager dinnerForkComp3 = null;
        com.acme.Dinner.ForkManager dinnerForkComp4 = null;
        com.acme.Dinner.Fork forkFacetObj1 = null;
        com.acme.Dinner.Fork forkFacetObj2 = null;
        com.acme.Dinner.Fork forkFacetObj3 = null;
        com.acme.Dinner.Fork forkFacetObj4 = null;

        org.omg.Components.CCMHome registratorHome = null;
        com.acme.Dinner.RegistratorHome dinnerRegistratorHome = null;
        com.acme.Dinner.Registrator dinnerRegistratorComp = null;

        org.omg.Components.Cookie ck1 = null;
        org.omg.Components.Cookie ck2 = null;
        org.omg.Components.Cookie ck2_bis = null;
        org.omg.Components.Cookie ck3 = null;
        org.omg.Components.Cookie ck4 = null;

        int timescale = TestManager.getTimescale();

        // This tests CIF classes
        //-------------------------------------------------
        System.out.println("Testing CIF classes");

        try {
            System.out.println("Install a PhilosopherHome");
            org.omg.CORBA.Any value = null;
            org.omg.Components.ConfigValue[] configValues;

            try {
                configValues = new org.omg.Components.ConfigValue[5];
                value = this.orb.create_any();
                ComponentKindValueHelper.insert(
                    value,
                    ComponentKindValue.SESSION);
                configValues[0] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.COMPONENT_KIND.value,
                        value);

                value = this.orb.create_any();
                ServantLifetimeValueHelper.insert(
                    value,
                    ServantLifetimeValue.CONTAINER_LIFETIME);
                configValues[1] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.SERVANT_LIFETIME.value,
                        value);

                value = this.orb.create_any();
                ThreadingPolicyValueHelper.insert(
                    value,
                    ThreadingPolicyValue.MULTITHREAD);
                configValues[2] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.THREADING_POLICY.value,
                        value);

                value = this.orb.create_any();
                value.insert_string(PHILOSOPHER_HOME_REP_ID);
                configValues[3] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.HOME_REPOSITORY_ID.value,
                        value);

                value = this.orb.create_any();
                value.insert_string(PHILOSOPHER_HOME_SERVANT_NAME);
                configValues[4] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.HOME_SERVANT_CLASSNAME.value,
                        value);

                philosopherHome =
                    this.container.install_home(
                        PHILOSOPHER_HOME_UID,
                        PHILOSOPHER_HOME_ENTRY_POINT,
                        configValues);
            } catch (org.omg.CORBA.BAD_PARAM bp) {
                fail(bp.toString());
            } catch (org.omg.Components.Deployment.InstallationFailure insf) {
                fail("Unexpected exception raised: Components::Deployment::InstallationFailure ");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            } catch (org.omg.CORBA.UserException ue) {
                fail("Unexpected User exception raised: " + ue.toString());
            }

            System.out.println("Install a ObserverHome");

            try {
                configValues = new org.omg.Components.ConfigValue[6];
                value = this.orb.create_any();
                ComponentKindValueHelper.insert(
                    value,
                    ComponentKindValue.SESSION);
                configValues[0] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.COMPONENT_KIND.value,
                        value);

                value = this.orb.create_any();
                ServantLifetimeValueHelper.insert(
                    value,
                    ServantLifetimeValue.CONTAINER_LIFETIME);
                configValues[1] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.SERVANT_LIFETIME.value,
                        value);

                value = this.orb.create_any();
                ThreadingPolicyValueHelper.insert(
                    value,
                    ThreadingPolicyValue.MULTITHREAD);
                configValues[2] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.THREADING_POLICY.value,
                        value);

                value = this.orb.create_any();
                value.insert_string(OBSERVER_HOME_REP_ID);
                configValues[3] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.HOME_REPOSITORY_ID.value,
                        value);

                value = this.orb.create_any();
                value.insert_string(OBSERVER_HOME_SERVANT_NAME);
                configValues[4] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.HOME_SERVANT_CLASSNAME.value,
                        value);

                ValuetypeFactoryDescription[] seq
                    = new ValuetypeFactoryDescription[1];
                // Event StatusInfo valuetype
                ValuetypeFactoryDescription desc = new ValuetypeFactoryDescription();
                desc.repid = EVENT_STATUSINFO_REP_ID;
                desc.factory_entrypoint = EVENT_STATUSINFO_ENTRYPOINT;
                seq[0] = desc;

                value = this.orb.create_any();
                ValuetypeFactoryDescriptionSeqHelper.insert(value, seq);
                configValues[5] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.VALUETYPE_FACTORY_DEPENDENCIES.value,
                    value);

                observerHome =
                    this.container.install_home(
                        OBSERVER_HOME_UID,
                        OBSERVER_HOME_ENTRY_POINT,
                        configValues);
            } catch (org.omg.CORBA.BAD_PARAM bp) {
                fail(bp.toString());
            } catch (org.omg.Components.Deployment.InstallationFailure insf) {
                fail("Unexpected exception raised: Components::Deployment::InstallationFailure");
            }

            System.out.println("Install a ForkHome");

            try {
                configValues = new org.omg.Components.ConfigValue[5];
                value = this.orb.create_any();
                ComponentKindValueHelper.insert(
                    value,
                    ComponentKindValue.SESSION);
                configValues[0] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.COMPONENT_KIND.value,
                        value);

                value = this.orb.create_any();
                ServantLifetimeValueHelper.insert(
                    value,
                    ServantLifetimeValue.CONTAINER_LIFETIME);
                configValues[1] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.SERVANT_LIFETIME.value,
                        value);

                value = this.orb.create_any();
                ThreadingPolicyValueHelper.insert(
                    value,
                    ThreadingPolicyValue.MULTITHREAD);
                configValues[2] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.THREADING_POLICY.value,
                        value);

                value = this.orb.create_any();
                value.insert_string(FORKMANAGER_HOME_REP_ID);
                configValues[3] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.HOME_REPOSITORY_ID.value,
                        value);

                value = this.orb.create_any();
                value.insert_string(FORKMANAGER_HOME_SERVANT_NAME);
                configValues[4] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.HOME_SERVANT_CLASSNAME.value,
                        value);

                forkHome =
                    this.container.install_home(
                        FORKMANAGER_HOME_UID,
                        FORKMANAGER_HOME_ENTRY_POINT,
                        configValues);
            } catch (org.omg.CORBA.BAD_PARAM bp) {
                fail(bp.toString());
            } catch (org.omg.Components.Deployment.InstallationFailure insf) {
                fail("Unexpected exception raised: Components::Deployment::InstallationFailure");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            } catch (org.omg.CORBA.UserException ue) {
                fail("Unexpected User exception raised: " + ue.toString());
            }

            System.out.println("Install a RegistratorHome");

            try {
                configValues = new org.omg.Components.ConfigValue[5];
                value = this.orb.create_any();
                ComponentKindValueHelper.insert(
                    value,
                    ComponentKindValue.SESSION);
                configValues[0] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.COMPONENT_KIND.value,
                        value);

                value = this.orb.create_any();
                ServantLifetimeValueHelper.insert(
                    value,
                    ServantLifetimeValue.CONTAINER_LIFETIME);
                configValues[1] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.SERVANT_LIFETIME.value,
                        value);

                value = this.orb.create_any();
                ThreadingPolicyValueHelper.insert(
                    value,
                    ThreadingPolicyValue.MULTITHREAD);
                configValues[2] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.THREADING_POLICY.value,
                        value);

                value = this.orb.create_any();
                value.insert_string(REGISTRATOR_HOME_REP_ID);
                configValues[3] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.HOME_REPOSITORY_ID.value,
                        value);

                value = this.orb.create_any();
                value.insert_string(REGISTRATOR_HOME_SERVANT_NAME);
                configValues[4] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.HOME_SERVANT_CLASSNAME.value,
                        value);

                registratorHome =
                    this.container.install_home(
                        REGISTRATOR_HOME_UID,
                        REGISTRATOR_HOME_ENTRY_POINT,
                        configValues);
            } catch (org.omg.CORBA.BAD_PARAM bp) {
                fail(bp.toString());
            } catch (org.omg.Components.Deployment.InstallationFailure insf) {
                fail("Unexpected exception raised: Components::Deployment::InstallationFailure");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            } catch (org.omg.CORBA.UserException ue) {
                fail("Unexpected User exception raised: " + ue.toString());
            }

            System.out.println("Create a Philosopher component using create_component() operation");
            try {
                org.omg.Components.KeylessCCMHome keylessPhilosopherHome =
                    org.omg.Components.KeylessCCMHomeHelper.narrow(
                        philosopherHome);

                assertTrue(
                    "ERROR: Created home is not a keyless home!",
                    keylessPhilosopherHome != null);
                ccmPhilosopherComp =
                    keylessPhilosopherHome.create_component();

                dinnerPhilosopherComp1 =
                    com.acme.Dinner.PhilosopherHelper.narrow(
                        ccmPhilosopherComp);

                assertTrue(
                    "ERROR: Created component is not a com.acme.Dinner.Philosopher!",
                    dinnerPhilosopherComp1 != null);
            } catch (org.omg.Components.CreateFailure cf) {
                fail("Unexpected exception raised: CreateFailure");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Create other Philosopher component using create() operation");
            try {
                dinnerPhilosopherHome =
                    com.acme.Dinner.PhilosopherHomeHelper.narrow(
                        philosopherHome);

                assertTrue(
                    "Home is not a com.acme.Dinner.PhilosopherHome!",
                    dinnerPhilosopherHome != null);
                dinnerPhilosopherComp2 = dinnerPhilosopherHome.create();
                dinnerPhilosopherComp3 = dinnerPhilosopherHome.create();
                dinnerPhilosopherComp4 = dinnerPhilosopherHome.create();

                assertTrue(
                    "ERROR: dinnerPhilosopherComp2 is nil!",
                    dinnerPhilosopherComp2 != null);
                assertTrue(
                    "ERROR: dinnerPhilosopherComp3 is nil!",
                    dinnerPhilosopherComp3 != null);
                assertTrue(
                    "ERROR: dinnerPhilosopherComp4 is nil!",
                    dinnerPhilosopherComp4 != null);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Create a Observer component using create_component() operation");
            try {
                org.omg.Components.KeylessCCMHome keylessObserverHome =
                    org.omg.Components.KeylessCCMHomeHelper.narrow(
                        observerHome);

                assertTrue(
                    "ERROR: Created home is not a keyless home!",
                    keylessObserverHome != null);
                ccmObserverComp = keylessObserverHome.create_component();

                dinnerObserverComp =
                    com.acme.Dinner.ObserverHelper.narrow(ccmObserverComp);

                assertTrue(
                    "ERROR: Created component is not a com.acme.Dinner.Observer!",
                    dinnerObserverComp != null);
            } catch (org.omg.Components.CreateFailure cf) {
                fail("Unexpected exception raised: CreateFailure");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Create a ForkManager component using create_component() operation");
            try {
                org.omg.Components.KeylessCCMHome keylessForkHome =
                    org.omg.Components.KeylessCCMHomeHelper.narrow(forkHome);

                assertTrue(
                    "ERROR: Created home is not a keyless home!",
                    keylessForkHome != null);
                ccmForkComp = keylessForkHome.create_component();

                dinnerForkComp1 =
                    com.acme.Dinner.ForkManagerHelper.narrow(ccmForkComp);

                assertTrue(
                    "ERROR: Created component is not a com.acme.Dinner.ForkManager!",
                    dinnerForkComp1 != null);
            } catch (org.omg.Components.CreateFailure cf) {
                fail("Unexpected exception raised: CreateFailure");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Create other ForkManager component using create() operation");
            try {
                dinnerForkHome =
                    com.acme.Dinner.ForkHomeHelper.narrow(forkHome);

                assertTrue(
                    "Home is not a com.acme.Dinner.ForkHome!",
                    dinnerForkHome != null);
                dinnerForkComp2 = dinnerForkHome.create();
                dinnerForkComp3 = dinnerForkHome.create();
                dinnerForkComp4 = dinnerForkHome.create();

                assertTrue(
                    "ERROR: dinnerForkComp2 is nil!",
                    dinnerForkComp2 != null);
                assertTrue(
                    "ERROR: dinnerForkComp3 is nil!",
                    dinnerForkComp3 != null);
                assertTrue(
                    "ERROR: dinnerForkComp4 is nil!",
                    dinnerForkComp4 != null);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Create Registrator component using create() operation");
            try {
                dinnerRegistratorHome =
                    com.acme.Dinner.RegistratorHomeHelper.narrow(
                        registratorHome);

                assertTrue(
                    "Home is not a com.acme.Dinner.RegistratorHome!",
                    dinnerRegistratorHome != null);
                dinnerRegistratorComp = dinnerRegistratorHome.create();

                assertTrue(
                    "ERROR: dinnerRegistratorComp is nil!",
                    dinnerRegistratorComp != null);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Testing generated operation on ForkManager component...");
            System.out.println("provide_the_fork operation on ForkManager component...");
            try {
                forkFacetObj1 = dinnerForkComp1.provide_the_fork();
                forkFacetObj2 = dinnerForkComp2.provide_the_fork();
                forkFacetObj3 = dinnerForkComp3.provide_the_fork();
                forkFacetObj4 = dinnerForkComp4.provide_the_fork();

                assertTrue(
                    "ERROR: no facet found for ForkManager1 component",
                    forkFacetObj1 != null);
                assertTrue(
                    "ERROR: No facet found for ForkManager2 component",
                    forkFacetObj2 != null);
                assertTrue(
                    "ERROR: No facet found for ForkManager3 component",
                    forkFacetObj3 != null);
                assertTrue(
                    "ERROR: No facet found for ForkManager4 component",
                    forkFacetObj4 != null);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Connect operation on Philosopher components to ForkManager...");
            try {
                dinnerPhilosopherComp1.connect_right(forkFacetObj1);
                dinnerPhilosopherComp1.connect_left(forkFacetObj2);
                dinnerPhilosopherComp2.connect_right(forkFacetObj2);
                dinnerPhilosopherComp2.connect_left(forkFacetObj3);
                dinnerPhilosopherComp3.connect_right(forkFacetObj3);
                dinnerPhilosopherComp3.connect_left(forkFacetObj4);
                dinnerPhilosopherComp4.connect_right(forkFacetObj4);
                dinnerPhilosopherComp4.connect_left(forkFacetObj1);
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.Components.AlreadyConnected ac) {
                fail("Unexpected exception raised: AlreadyConnected");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Connect operation again on Philosopher components to ForkManager...");
            try {
                dinnerPhilosopherComp1.connect_right(forkFacetObj1);
                fail("Expected exception not raised");
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.Components.AlreadyConnected ac) {
                System.out.println("Expected exception raised: AlreadyConnected");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Connect operation on Philosopher components to Registrator...");
            try {
                dinnerPhilosopherComp1.connect_register(dinnerRegistratorComp);
                dinnerPhilosopherComp2.connect_register(dinnerRegistratorComp);
                dinnerPhilosopherComp3.connect_register(dinnerRegistratorComp);
                dinnerPhilosopherComp4.connect_register(dinnerRegistratorComp);
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.Components.AlreadyConnected ac) {
                fail("Unexpected exception raised: AlreadyConnected");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Get_connections operation on Philosopher component...");
            try {
                org.omg.Components.ConnectionDescription[] conxDesc =
                    dinnerPhilosopherComp1.get_connections(REGISTER_RECEP_NAME);

                assertTrue(
                    "ERROR: Unexpected number of connection found for Philosopher component => "
                        + conxDesc.length,
                    conxDesc.length == 1);
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Get_all_publishers operation on Philosopher component...");
            try {
                org.omg.Components.PublisherDescription[] publisherDesc =
                    dinnerPhilosopherComp1.get_all_publishers();

                assertTrue(
                    "ERROR: Unexpected number of publisher found for Philosopher component",
                    publisherDesc.length == 1);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Get_all_emitters operation on Philosopher component...");
            try {
                org.omg.Components.EmitterDescription[] emitterDesc =
                    dinnerPhilosopherComp1.get_all_emitters();

                assertTrue(
                    "ERROR: Unexpected number of emitter found for Philosopher component",
                    emitterDesc.length == 1);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Get_all_consumers operation on Observer component...");
            try {
                org.omg.Components.ConsumerDescription[] consumerDesc =
                    dinnerObserverComp.get_all_consumers();

                assertTrue(
                    "ERROR: Unexpected number of consumers found for Observer component",
                    consumerDesc.length == 1);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Get_named_consumers operation on Observer component...");
            try {
                String[] consumerNameList = new String[1];
                consumerNameList[0] = INFO_CONSUMER_NAME;
                org.omg.Components.ConsumerDescription[] consumerDesc =
                    dinnerObserverComp.get_named_consumers(consumerNameList);

                assertTrue(
                    "ERROR: Unexpected number of consumers found for Observer component",
                    consumerDesc.length == 1);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Get_consumer_info operation on Observer component...");
            try {
                infoConsumer = dinnerObserverComp.get_consumer_info();

                assertTrue(
                    "ERROR: no consumer found for Observer component",
                    infoConsumer != null);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Subscribe operation on Philosopher components from Observer...");
            try {
                ck1 = dinnerPhilosopherComp1.subscribe_info(infoConsumer);
                ck2 = dinnerPhilosopherComp2.subscribe_info(infoConsumer);
                ck3 = dinnerPhilosopherComp3.subscribe_info(infoConsumer);
                ck4 = dinnerPhilosopherComp4.subscribe_info(infoConsumer);
            } catch (org.omg.Components.ExceededConnectionLimit ecl) {
                fail("Unexpected exception raised: ExceededConnectionLimit");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Try to unsubscribe operation on Philosopher1 components with invalid cookie...");
            try {
                dinnerPhilosopherComp1.unsubscribe_info(ck2);
            } catch (org.omg.Components.InvalidConnection ic) {
                // Expected exception raised: InvalidConnection
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Unsubscribe operation on Philosopher1 components from Observer...");
            try {
                dinnerPhilosopherComp1.unsubscribe_info(ck1);
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Subscribe operation on Philosopher1 components from Observer...");
            try {
                ck1 = dinnerPhilosopherComp1.subscribe_info(infoConsumer);
                ck2_bis = dinnerPhilosopherComp2.subscribe_info(infoConsumer);
            } catch (org.omg.Components.ExceededConnectionLimit ecl) {
                fail("Unexpected exception raised: ExceededConnectionLimit");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Connect_config operation on Philosopher1 components from Observer...");
            try {
                dinnerPhilosopherComp1.connect_config(infoConsumer);
            } catch (org.omg.Components.AlreadyConnected ac) {
                fail("Unexpected exception raised: AlreadyConnected");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

          
            System.out.println("Try to connect_config again operation on Philosopher1 components from a different consumer...");
            try {
                dinnerPhilosopherComp1.connect_config(consumer);
                fail("Excepted exception not raised");
            }
            catch (org.omg.Components.AlreadyConnected ac) {
                System.out.println("Expected exception raised: AlreadyConnected");
            }
            catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
 
            System.out.println("Try to connect_config again operation on Philosopher1 components from Observer...");
            try {
                dinnerPhilosopherComp1.connect_config(infoConsumer);
            }
            catch (org.omg.Components.AlreadyConnected ac) {
                fail("Unexpected exception raised: AlreadyConnected");
            }
            catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Disconnect_config operation on Philosopher1 components from Observer...");
            try {
                org.omg.Components.EventConsumerBase consumer =
                    dinnerPhilosopherComp1.disconnect_config();
            } catch (org.omg.Components.NoConnection nc) {
                fail("Unexpected exception raised: NoConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Disconnect_config operation again on Philosopher1 components from Observer...");
            try {
                org.omg.Components.EventConsumerBase consumer =
                    dinnerPhilosopherComp1.disconnect_config();
                fail("Expected exception not raised");
            } catch (org.omg.Components.NoConnection nc) {
                System.out.println("Expected exception raised: NoConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Disconnect_config operation on Philosopher1 components from Observer...");
            try {
                org.omg.Components.EventConsumerBase consumer =
                    dinnerPhilosopherComp2.disconnect_config();
                fail("Expected exception not raised");
            } catch (org.omg.Components.NoConnection nc) {
                System.out.println("Unexpected exception raised: NoConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Try to connect_config again operation on Philosopher1 components from Observer...");
            try {
                dinnerPhilosopherComp1.connect_config(infoConsumer);
            } catch (org.omg.Components.AlreadyConnected ac) {
                fail("Unexpected exception raised: AlreadyConnected");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Get_ccm_home operation on Observer component...");
            try {
                org.omg.Components.CCMHome ccmHome =
                    dinnerObserverComp.get_ccm_home();

                assertTrue("ERROR: Unexpected home for Observer component",
                    ccmHome._is_equivalent(observerHome));
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Get_primary_key operation on Observer component...");
            try {
                org.omg.Components.PrimaryKeyBase key =
                    dinnerObserverComp.get_primary_key();
                fail("Expected exception not raised");
            } catch (org.omg.Components.NoKeyAvailable nka) {
                System.out.println("Expected exception raised: NoKeyAvailable");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Get_all_ports operation on Philosopher component...");
            try {
                org.omg.Components.ComponentPortDescription portDesc =
                    dinnerPhilosopherComp2.get_all_ports();

                org.omg.Components.FacetDescription[] facetDesc =
                    portDesc.facets;
                org.omg.Components.ReceptacleDescription[] recepDesc =
                    portDesc.receptacles;
                org.omg.Components.ConsumerDescription[] consumerDesc =
                    portDesc.consumers;
                org.omg.Components.EmitterDescription[] emitterDesc =
                    portDesc.emitters;
                org.omg.Components.PublisherDescription[] publisherDesc =
                    portDesc.publishers;

                assertTrue(
                    "ERROR: unexpected number of facet for Philosopher component",
                    facetDesc.length == 0);
                assertTrue(
                    "ERROR: unexpected number of receptacle for Philosopher component",
                    recepDesc.length == 3);
                assertTrue(
                    "ERROR: unexpected number of consumer for Philosopher component",
                    consumerDesc.length == 0);
                assertTrue(
                    "ERROR: unexpected number of publisher for Philosopher component",
                    publisherDesc.length == 1);
                assertTrue(
                    "ERROR: unexpected number of emitter for Philosopher component",
                    emitterDesc.length == 1);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Configuration_complete operation on Philosopher components...");
            try {
                System.out.println("Configuration_complete operation on Philosopher1 components...");
                dinnerPhilosopherComp1.configuration_complete();

                OS.sleep(timescale * 1000);

                System.out.println("Configuration_complete operation on Philosopher2 components...");
                dinnerPhilosopherComp2.configuration_complete();
                OS.sleep(timescale * 1000);

                System.out.println("Configuration_complete operation on Philosopher3 components...");
                dinnerPhilosopherComp3.configuration_complete();
                OS.sleep(timescale * 1000);

                System.out.println("Configuration_complete operation on Philosopher4 components...");
                dinnerPhilosopherComp4.configuration_complete();
                OS.sleep(timescale * 1000);
            } catch (org.omg.Components.InvalidConfiguration ic) {
                fail("Unexpected exception raised: InvalidConfiguration");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Configuration_complete operation on Observer component...");
            try {
                dinnerObserverComp.configuration_complete();
            } catch (org.omg.Components.InvalidConfiguration ic) {
                fail("Unexpected exception raised: InvalidConfiguration");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Get name of Philosopher component1...");
            try {
                String philosopherName = dinnerPhilosopherComp1.name();
                System.out.println("Philosopher1 name is " + philosopherName);

                philosopherName = dinnerPhilosopherComp2.name();
                System.out.println("Philosopher1 name is " + philosopherName);

                philosopherName = dinnerPhilosopherComp3.name();
                System.out.println("Philosopher1 name is " + philosopherName);

                philosopherName = dinnerPhilosopherComp4.name();
                System.out.println("Philosopher1 name is " + philosopherName);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            OS.sleep(timescale * 10000);

            System.out.println("Disconnect_left and right operation on Philosopher component...");
            try {
                dinnerPhilosopherComp1.disconnect_left();
                dinnerPhilosopherComp2.disconnect_left();
                dinnerPhilosopherComp3.disconnect_left();
                dinnerPhilosopherComp4.disconnect_left();
                dinnerPhilosopherComp1.disconnect_right();
                dinnerPhilosopherComp2.disconnect_right();
                dinnerPhilosopherComp3.disconnect_right();
                dinnerPhilosopherComp4.disconnect_right();
            } catch (org.omg.Components.NoConnection nc) {
                fail("Unexpected exception raised: NoConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Disconnect_register operation on Philosopher components...");
            try {
                dinnerPhilosopherComp1.disconnect_register();
                dinnerPhilosopherComp2.disconnect_register();
                dinnerPhilosopherComp3.disconnect_register();
                dinnerPhilosopherComp4.disconnect_register();
            } catch (org.omg.Components.NoConnection nc) {
                fail("Unexpected exception raised: NoConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Unsubscribe_info operation on Philosopher components...");
            try {
                dinnerPhilosopherComp1.unsubscribe_info(ck1);
                dinnerPhilosopherComp2.unsubscribe_info(ck2);
                dinnerPhilosopherComp3.unsubscribe_info(ck3);
                dinnerPhilosopherComp4.unsubscribe_info(ck4);
            } catch (org.omg.Components.InvalidConnection ic) {
                fail("Unexpected exception raised: InvalidConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Disconnect_config operation on Philosopher components...");
            try {
                org.omg.Components.EventConsumerBase consumer =
                    dinnerPhilosopherComp1.disconnect_config();
            } catch (org.omg.Components.NoConnection nc) {
                fail("Unexpected exception raised: NoConnection");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Remove operation on the Philosopher home...");
            try {
                this.container.remove_home(dinnerPhilosopherHome);
            } catch (org.omg.Components.RemoveFailure rf) {
                fail("Unexpected exception raised: NoKeyAvailable");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Remove operation on the Observer home...");
            try {
                this.container.remove_home(observerHome);
            } catch (org.omg.Components.RemoveFailure rf) {
                fail("Unexpected exception raised: NoKeyAvailable");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Remove operation on the Fork home...");
            try {
                this.container.remove_home(forkHome);
            } catch (org.omg.Components.RemoveFailure rf) {
                fail("Unexpected exception raised: NoKeyAvailable");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Remove operation on the Registrator home...");
            try {
                this.container.remove_home(registratorHome);
            } catch (org.omg.Components.RemoveFailure rf) {
                fail("Unexpected exception raised: NoKeyAvailable");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            // remove container
            this.container.remove();

        } catch (Exception e) {
            fail("Unexpected exception raised: " + e.toString());
        }
    }
}


