/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <sstream>
#include <string>
#include <fstream>

#include "testrecording/TestClient.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "CCMContainer/ccmcontainer/ConfigValue_impl.hpp"
#include "testrecording/RecordingTest_cif.skel.hpp"
#include <Foundation/idllib/CdmwDeployment.skel.hpp>
#include <CCMContainer/idllib/CdmwCcmCif.stub.hpp>

namespace
{
    const char* CLIENT_HOME_REP_ID = "IDL:thalesgroup.com/Cdmw_RecordingTest/ClientHome:1.0";
    const char* CLIENT_HOME_SERVANT_NAME = "Cdmw.CCM.CIF.CdmwRecordingTest.SessionClientHome_impl";
    const char* CLIENT_HOME_UID = "ClientHome_1";
    const char* CLIENT_HOME_ENTRY_POINT = "create_CCM_ClientHome";

    const char* FACETTEST_RECEP_NAME = "to_server";
  
}; // End anonymous namespace

namespace Cdmw
{
namespace CCM
{
namespace CIF
{

/**
*Implementation notes: [if no pertinent write none ]
*<p>
*Portability issues: [if no pertinent write none ]
*<p>
*/


CPPUNIT_TEST_SUITE_REGISTRATION( TestClient );
TestClient::TestClient()
   throw(CORBA::SystemException)
{
    m_orb =  CORBA::ORB::_duplicate( Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());
    try
    {
        CORBA::Object_var obj = m_orb->string_to_object(Cdmw::TestUtils::CORBATestManager::instance()->get_property ("containerIOR").c_str());

        if (CORBA::is_nil(obj.in()))
        {
            TEST_INFO("TEST FAILED : exception raised during test initialisation!");
        }
        else
        {
            m_container = Components::Deployment::Container::_narrow(obj.in());
        }
    }
    catch (const CORBA::SystemException & e)
    {
        TEST_INFO("TEST FAILED : system exception raised during test initialisation!");
        throw;
    }
    catch (...)
    {
        TEST_INFO("TEST FAILED : unexpected exception raised during test initialisation!");
    }
}


TestClient::~TestClient()
    throw()
{
    // does nothing
}


TestClient::TestClient(const TestClient& rhs)
    throw()
    :m_orb(CORBA::ORB::_duplicate(rhs.m_orb)),
     m_container(Components::Deployment::Container::_duplicate(rhs.m_container))
{
    // does nothing
}
        
TestClient&
TestClient::operator=(const TestClient& rhs)
    throw()
{
    if (this != &rhs) {
        m_orb = CORBA::ORB::_duplicate(rhs.m_orb.in());
        m_container = Components::Deployment::Container::_duplicate(rhs.m_container.in());
    }
    return *this;
}

void TestClient::do_tests()        
{
  // set number of requested successfull tests
  //     set_nbOfRequestedTestOK (25);
    int timescale = Cdmw::TestUtils::get_timescale();

    Cdmw::OsSupport::OS::sleep(timescale*2000);
    
    bool test_ok=false;

    Components::CCMHome_var client_home;
    RecordingTest::ClientHome_var basic_client_home;
    Components::CCMObject_var ccm_client_comp;
    RecordingTest::Client_var basic_client_comp;

    /*
     * Config Values used by the context of components
     */

    Components::ConfigValues config_values_home(0);
    config_values_home.length(0);
    CORBA::Any value;

    // facet providing the sleep
    CORBA::Object_var facettester_facet_obj;


    //-------------------------------------------------
    TEST_INFO("Testing client component with default receptacle");
    
    try{
      
      TEST_INFO("Install a ClientHome for a component with a default timeout");
      test_ok=false;
      
      try
        {
            Components::ConfigValues config_values(8);
            config_values.length(5);
            CORBA::Any value;
            value <<= CdmwDeployment::SESSION;
            config_values[0] = new Container::ConfigValue_impl(CdmwDeployment::COMPONENT_KIND, 
                                                   value);

            value <<= CdmwDeployment::CONTAINER_LIFETIME;
            config_values[1] = new Container::ConfigValue_impl(CdmwDeployment::SERVANT_LIFETIME, 
                                                   value);

            value <<= CdmwDeployment::MULTITHREAD;
            config_values[2] = new Container::ConfigValue_impl(CdmwDeployment::THREADING_POLICY, 
                                                   value);

            value <<= CLIENT_HOME_REP_ID;
            config_values[3] = new Container::ConfigValue_impl(CdmwDeployment::HOME_REPOSITORY_ID, 
                                                   value);

            value <<= CLIENT_HOME_SERVANT_NAME;
            config_values[4] = new Container::ConfigValue_impl(CdmwDeployment::HOME_SERVANT_CLASSNAME, 
                                                   value);

            CORBA::ULong lg = 1L;
            CdmwDeployment::ValuetypeFactoryDescriptionSeq seq(lg);
            seq.length(lg);

            client_home 
               = m_container->install_home(CLIENT_HOME_UID,
                                           CLIENT_HOME_ENTRY_POINT,
                                           config_values);

            test_ok=true;
        }
        catch (const CORBA::BAD_PARAM& e)
        {
            TEST_INFO(e);
        }
        catch (const Components::Deployment::InstallationFailure& e)
        {
            std::ostringstream os;
            os << "Unexpected exception raised: Components::Deployment::InstallationFailure " << e.reason << std::ends;
            TEST_INFO(os.str());
        }
        CPPUNIT_ASSERT(test_ok);

        TEST_INFO("Create a Client component using create_component() operation");
        test_ok=false;
        try
        {
          CdmwCcmCif::KeylessCCMHome_var keyless_client_home 
                = CdmwCcmCif::KeylessCCMHome::_narrow(client_home.in());

            if (CORBA::is_nil(keyless_client_home.in()))
            {
                TEST_INFO("ERROR: Created home is not a keyless home!");
            }
            else
            {
                ccm_client_comp
                   = keyless_client_home->create_component_with_config_values(config_values_home);

                basic_client_comp
                    = RecordingTest::Client::_narrow(ccm_client_comp.in());

                if (CORBA::is_nil(basic_client_comp.in()))
                {
                    TEST_INFO("ERROR: Created component is not a RecordingTest::Client!")
                }
                else
                {
                    test_ok=true;
                }
            }
        }
        catch (const Components::CreateFailure& e)
        {
            TEST_INFO("Unexpected exception raised: CreateFailure");
        }
        catch (const CORBA::SystemException& e)
        {
            std::ostringstream os;
            os << "Unexpected exception raised: " << e << std::ends;
            TEST_INFO(os.str());
        }
        CPPUNIT_ASSERT(test_ok);

        TEST_INFO("Recover server facet's ior");
        test_ok=false;
        try
        {
          // get the server facet's ior
          ifstream in("file.ior");
          char s[2048];
          in >> s;
          facettester_facet_obj =
            m_orb->string_to_object(s);

          test_ok = true;
        }
        catch (const CORBA::SystemException& e)
        {
            std::ostringstream os;
            os << "Unexpected exception raised: " << e << std::ends;
            TEST_INFO(os.str());
            test_ok=false;
        }
        CPPUNIT_ASSERT(test_ok);

        TEST_INFO("Connect operation. Client : to_server -> facet ");
        test_ok=false;
        try
        {
            Components::Cookie_var ck
                = basic_client_comp->connect(FACETTEST_RECEP_NAME,
                                             facettester_facet_obj.in());
            test_ok=true;
        }
        catch (const Components::InvalidName& )
        {
            TEST_INFO("Unexpected exception raised: InvalidName");
        }
        catch (const Components::InvalidConnection& )
        {
            TEST_INFO("Expected exception raised: InvalidConnection");
        }
        catch (const Components::AlreadyConnected& )
        {
            TEST_INFO("Unexpected exception raised: AlreadyConnected");
        }
        catch (const Components::ExceededConnectionLimit& )
        {
            TEST_INFO("Unexpected exception raised: ExceededConnectionLimit");
        }
        catch (const CORBA::SystemException& e)
        {
            std::ostringstream os;
            os << "Unexpected exception raised: " << e << std::ends;
            TEST_INFO(os.str());
            test_ok=false;
        }
        CPPUNIT_ASSERT(test_ok);

        TEST_INFO("Configuration_complete operation on Client component...");
        test_ok=false;
        try
        {
            basic_client_comp->configuration_complete();
            test_ok=true;
        }
        catch (const Components::InvalidConfiguration &)
        {
            TEST_INFO("Unexpected exception raised: InvalidConfiguration");
        }
        catch (const CORBA::SystemException& e)
        {
            std::ostringstream os;
            os << "Unexpected exception raised: " << e << std::ends;
            TEST_INFO(os.str());
        }
        CPPUNIT_ASSERT(test_ok);


        TEST_INFO("Disconnect default receptacle of Client component...");
        test_ok=false;
        try
        {
            basic_client_comp->disconnect(FACETTEST_RECEP_NAME, 0);
            test_ok=true;
        }
        catch (const Components::InvalidName& )
        {
            TEST_INFO("Unexpected exception raised: InvalidName");
        }
        catch (const Components::InvalidConnection& )
        {
            TEST_INFO("Unexpected exception raised: InvalidConnection");
        }
        catch (const Components::CookieRequired& )
        {
            TEST_INFO("Unexpected exception raised: CookieRequired");
        }
        catch (const Components::NoConnection& )
        {
            TEST_INFO("Unexpected exception raised: NoConnection");
        }
        catch (const CORBA::SystemException& e)
        {
            std::ostringstream os;
            os << "Unexpected exception raised: " << e << std::ends;
            TEST_INFO(os.str());
            test_ok=false;
        }
        CPPUNIT_ASSERT(test_ok);



        TEST_INFO("Remove operation on the Client home...");
        test_ok=false;
        try
        {
            m_container->remove_home(client_home.in());
            test_ok=true;
        }
        catch (const Components::RemoveFailure &)
        {
            TEST_INFO("Unexpected exception raised: NoKeyAvailable");
        }
        catch (const CORBA::SystemException& e)
        {
            std::ostringstream os;
            os << "Unexpected exception raised: " << e << std::ends;
            TEST_INFO(os.str());
            test_ok=false;
        }
        CPPUNIT_ASSERT(test_ok);

 
    } 
    catch (const CORBA::Exception & e) 
    {
            std::ostringstream os;
            os << "Unexpected exception raised: " << e << std::ends;
            TEST_INFO(os.str()); 
            CPPUNIT_ASSERT(false);
    }

    TEST_INFO("Set to nil client_home"); 
    client_home = Components::CCMHome::_nil();
    TEST_INFO("Set to nil basic_client_home"); 
    basic_client_home = RecordingTest::ClientHome::_nil();
    TEST_INFO("Set to nil ccm_client_comp"); 
    ccm_client_comp = Components::CCMObject::_nil();
    TEST_INFO("Set to nil basic_client_comp"); 
    basic_client_comp = RecordingTest::Client::_nil();

    TEST_INFO("Set to nil tester_facet_obj"); 
    facettester_facet_obj = CORBA::Object::_nil();

 
}
        
}; // End namespace CIF
}; // End namespace CCM
}; // End namespace Cdmw

