/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.cif.test;

import java.io.File;
import java.util.Vector;
import java.util.StringTokenizer;
import java.io.IOException;

import cdmw.testutils.Testable;
import cdmw.common.Options;
import cdmw.ossupport.OS;
import cdmw.ossupport.FileUtilities;
import cdmw.ossupport.ThreadedProcess;

import cdmw.orbsupport.ORBSupport;
import cdmw.orbsupport.StrategyList;
import cdmw.orbsupport.ExceptionMinorCodes;
import cdmw.logging.*;

import cdmw.ccm.container.HomeAllocator;
import cdmw.ccm.container.HomeAllocatorRegistry;
import cdmw.ccm.componentserver.ComponentServerImpl;
import com.thalesgroup.CdmwDeployment.ComponentKindValue;
import com.thalesgroup.CdmwDeployment.ComponentKindValueHelper;
import com.thalesgroup.CdmwPlatformMngt.ProcessCallback;
import com.thalesgroup.CdmwPlatformMngt.ProcessCallbackHelper;
import cdmw.commonsvcs.naming.NamingInterface;
import cdmw.namingandrepository.RepositoryInterface;
import cdmw.common.Locations;
import cdmw.orbsupport.ORBUtils;
import cdmw.cdmwinit.CdmwInterface;
import cdmw.ossupport.ThreadedProcess;

public class MainTest {

    public static final String PROCESS_NAME = "DummyProcessName";
    public static final String APPLICATION_NAME = "DummyApplicationName";
      
    public static final int SUCCESS = 0;
    public static final int FAILURE = 1;  

    public static final int POA_THREAD_POOL_SIZE = 5;

    public static class ComponentInstallationPlugImpl 
        extends com.thalesgroup.CdmwDeployment.ComponentInstallationPOA {

        private org.omg.PortableServer.POA poa;

        public ComponentInstallationPlugImpl(org.omg.PortableServer.POA  poa) {
            this.poa = poa;
        }

        public org.omg.PortableServer.POA _default_POA() {
            return this.poa;
        }

        //
        // IDL:omg.org/Components/Deployment/ComponentInstallation/install:1.0
        //
        public void install(String implUUID, String componentLoc)
            throws org.omg.Components.Deployment.InvalidLocation,
                org.omg.Components.Deployment.InstallationFailure {}
    
        //
        // IDL:omg.org/Components/Deployment/ComponentInstallation/replace:1.0
        //
        public void replace(String implUUID, String componentLoc)
            throws org.omg.Components.Deployment.InvalidLocation,
                org.omg.Components.Deployment.InstallationFailure {}
    
        //
        // IDL:omg.org/Components/Deployment/ComponentInstallation/remove:1.0
        //
        public void remove(String implUUID)
            throws org.omg.Components.Deployment.UnknownImplId,
                org.omg.Components.RemoveFailure {}
    
        //
        // IDL:omg.org/Components/Deployment/ComponentInstallation/get_implementation:1.0
        //
        public String get_implementation(String implUUID)
            throws org.omg.Components.Deployment.UnknownImplId,
                org.omg.Components.Deployment.InstallationFailure  {

            String CLASSPATH=System.getProperty("java.class.path");
            String filePath="";
            StringTokenizer st = new StringTokenizer(CLASSPATH,":");
            while (st.hasMoreTokens()) {
                String token = st.nextToken();
                
                if(token.endsWith("libcdmwRecordingTest.jar"))
                    {
                        filePath = token;              
                    }
            }  
            System.out.println(filePath);
            return filePath;
        };
    
        //
        // IDL:thalesgroup.com/CdmwDeployment/ComponentInstallation/get_valuetypefactory_location:1.0
        //
        public String 
            get_valuetypefactory_location(String implUUID, String repid)
            throws org.omg.Components.Deployment.UnknownImplId,
                com.thalesgroup.CdmwDeployment.UnknownValuetype,
                org.omg.Components.Deployment.InstallationFailure {
            return "";
        };

    }
    
    public static org.omg.PortableServer.POA
        createContainerPOA(org.omg.PortableServer.POA parent) 
        throws org.omg.PortableServer.POAPackage.AdapterAlreadyExists,
            org.omg.PortableServer.POAPackage.InvalidPolicy {
        //
        // Create a POA with Retain and Single-thread policicies
        // to be used to activate the ComponentServer and Container objects.
        // Lifespan policy            = TRANSIENT
        // Id Assignment policy       = SYSTEM_ID
        // Id Uniqueness policy       = UNIQUE_ID
        // Implicit Activation policy = IMPLICIT_ACTIVATION
        // Request Processing policy  = USE_ACTIVE_OBJECT_MAP_ONLY
        // Servant Retention policy   = RETAIN;
        // Thread Policy              = SINGLE_THREAD_MODEL
        //        
        org.omg.PortableServer.POAManager poaManager = parent.the_POAManager();
        org.omg.CORBA.Policy[] policies = new org.omg.CORBA.Policy[7];
        policies[0] = parent.create_id_assignment_policy(
            org.omg.PortableServer.IdAssignmentPolicyValue.SYSTEM_ID);
        policies[1] = parent.create_lifespan_policy(
            org.omg.PortableServer.LifespanPolicyValue.TRANSIENT);
        policies[2] = parent.create_servant_retention_policy(
            org.omg.PortableServer.ServantRetentionPolicyValue.RETAIN);
        policies[3] = parent.create_id_uniqueness_policy(
            org.omg.PortableServer.IdUniquenessPolicyValue.UNIQUE_ID);
        policies[4] = parent.create_request_processing_policy(
            org.omg.PortableServer.RequestProcessingPolicyValue.USE_ACTIVE_OBJECT_MAP_ONLY);
        policies[5] = parent.create_implicit_activation_policy(
            org.omg.PortableServer.ImplicitActivationPolicyValue.IMPLICIT_ACTIVATION);
        policies[6] = parent.create_thread_policy(
            org.omg.PortableServer.ThreadPolicyValue.SINGLE_THREAD_MODEL);

        StrategyList poaStrategies = new StrategyList();

        String CONTAINER_POA_NAME = "ContainerPOA";
        return ORBSupport.createPOA(
            parent,
            CONTAINER_POA_NAME,
            poaManager, 
            policies,
            poaStrategies);
    }
    
    public static int run(org.omg.CORBA.ORB orb, 
                          String containerIOR) {
        ServerThread server = new ServerThread(orb, containerIOR);
    
        server.start();
        orb.run();
        try {
            server.join();
        } catch(InterruptedException ie) {}
    
        return SUCCESS;
    }


    public static ProcessCallback getProcessCallback(org.omg.CORBA.ORB orb)
        throws org.omg.CORBA.TRANSIENT {

        org.omg.CORBA.Object obj = null;
        //
        // Get naming service
        //
        try {
            obj = orb.resolve_initial_references("NameService");
        } catch(org.omg.CORBA.ORBPackage.InvalidName in) {
             System.out.println("Can't resolve `NameService'");
            throw new org.omg.CORBA.TRANSIENT(
                ExceptionMinorCodes.TRANSIENTCannotReachRepository,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        if(obj==null) {
            System.out.println("`NameService' is a nil object reference");
            throw new org.omg.CORBA.TRANSIENT(
                ExceptionMinorCodes.TRANSIENTCannotReachRepository,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        org.omg.CosNaming.NamingContextExt nc
            = org.omg.CosNaming.NamingContextExtHelper.narrow(obj);

        if (nc==null) {
            System.out.println("`NameService' is not a NamingContextExt object reference");
            throw new org.omg.CORBA.TRANSIENT(
                ExceptionMinorCodes.TRANSIENTCannotReachRepository,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        String PROCESS_CALLBACK_BINDING_NAME
            = "CDMW.I/process-callback.simulated";

        // Get reference to the ProcessCallback interface
        ProcessCallback callback = null;
        try {
            NamingInterface ni = new NamingInterface(nc);
            org.omg.CORBA.Object obj_ = ni.resolve(PROCESS_CALLBACK_BINDING_NAME);
            callback = ProcessCallbackHelper.narrow(obj_);
            if (callback==null) {
                System.out.println("Couldn't retrieve reference of the ProcessCallback");
                // TODO: minor cde to be defined
                throw new org.omg.CORBA.TRANSIENT(
                    ExceptionMinorCodes.TRANSIENT,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }
        } catch (Exception e) {
            System.out.println("Couldn't retrieve reference of the ProcessCallback");
            // TODO: minor cde to be defined
            throw new org.omg.CORBA.TRANSIENT(
                ExceptionMinorCodes.TRANSIENT,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        return callback;
    }

    public static void initRepository(org.omg.CORBA.ORB orb) throws Exception {

        //
        // Get naming service
        //
        org.omg.CORBA.Object obj = null;
        try {
            obj = orb.resolve_initial_references("NameService");
        } catch(org.omg.CORBA.ORBPackage.InvalidName in) {
            System.out.println("Can't resolve `NameService'");
            throw in;
        }

        if (obj==null) {
            System.out.println("`NameService' is a nil object reference");
            throw new org.omg.CORBA.TRANSIENT(
                ExceptionMinorCodes.TRANSIENTCannotReachRepository,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        org.omg.CosNaming.NamingContext nc
            = org.omg.CosNaming.NamingContextHelper.narrow(obj);

        if (nc == null) {
            System.out.println("`NameService' is not a NamingContextExt object reference");
            throw new org.omg.CORBA.TRANSIENT(
                ExceptionMinorCodes.TRANSIENTCannotReachRepository,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        String REPOSITORY_NAME = "CDMW.I/repository.simulated";

        // Get reference to the repository interface
        com.thalesgroup.CdmwNamingAndRepository.Repository rep = null;
        try {
            NamingInterface ni = new NamingInterface(nc);
            org.omg.CORBA.Object obj_ = ni.resolve(REPOSITORY_NAME);
            rep = com.thalesgroup.CdmwNamingAndRepository.RepositoryHelper.narrow(obj_);
            if (rep == null)  {
                System.out.println("Couldn't retrieve reference of the Repository");
                throw new org.omg.CORBA.TRANSIENT(
                    ExceptionMinorCodes.TRANSIENTCannotReachRepository,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }
        } catch (Exception e) {
            System.out.println("Couldn't retrieve reference of the Repository");
            throw new org.omg.CORBA.TRANSIENT(
                ExceptionMinorCodes.TRANSIENTCannotReachRepository,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        try {
            RepositoryInterface.init(Locations.CDMW_EVENT_SERVICE_NAME_DOMAIN, rep);
        } catch (Exception e) {
            System.out.println("Couldn't init Repository");
            throw e;
        }
    }

    
    public static void main(String[] args) {

        int status = SUCCESS;
        org.omg.CORBA.ORB orb = null;
        String callbackIOR
            = OS.getOptionValue( args, Options.CALLBACK_IOR_OPTION);
        int timescale = Testable.getTimescale();
        ThreadedProcess idDaemon = null;

        try {

            StrategyList orbStrategies = new StrategyList();
            orbStrategies.addORBThreaded();
            orbStrategies.addPOAThreadPool(POA_THREAD_POOL_SIZE);
            String nameServicePort = "5010";

            String s = "";
            s += ORBUtils.NAMING_PORT_OPTION + "corbaloc::localhost:"
                    + nameServicePort + "/NameService";
            s += " " + Options.SYSTEM_PORT_OPTION + "=" + nameServicePort;
            s += " --CdmwLogLevel=ALL-ORB_TRACE";
            System.out.println("ORBinit args = " + s);
            String[] serverArgs = s.split(" ");
            
            LogManager.Init(serverArgs);
            CdmwInterface.cdmwPreORBInit(serverArgs, "cdmw.generated.ServerCdmwInterface");
            
            s = System.getProperty("org.omg.CORBA.ORBClass");
            assert(s.equals("org.jacorb.orb.ORB"));
            s = System.getProperty("org.omg.CORBA.ORBSingletonClass");
            assert(s.equals("org.jacorb.orb.ORBSingleton"));

            orb = ORBSupport.ORBInit(serverArgs, orbStrategies);

            // Starting the simrepository
            cdmw.tools.SimuUtils.startSimu(System.out);
            System.out.println("Simrepository started");
            OS.sleep(timescale * 5000);

            // init repository
            initRepository(orb);
            System.out.println("Simrepository initialised");

            // Getting ProcessCallback IOR
            callbackIOR = orb.object_to_string(getProcessCallback(orb));    

            // Get the root POA 
            org.omg.CORBA.Object obj = 
                orb.resolve_initial_references("RootPOA");
            org.omg.PortableServer.POA rootPOA = 
                org.omg.PortableServer.POAHelper.narrow(obj);
            
            // Run implementation (Activate the POA manager)
            rootPOA.the_POAManager().activate();
     
    
            // declare Value factory to the orb
            org.omg.CORBA.portable.ValueFactory vf;
    
            vf = new cdmw.ccm.cif.FacetDescriptionFactoryImpl();
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory(
                "IDL:omg.org/Components/FacetDescription:1.0", vf);
           
            vf = new cdmw.ccm.cif.PortDescriptionFactoryImpl();
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory(
                "IDL:omg.org/Components/PortDescription:1.0", vf);
            
            vf = new cdmw.ccm.common.CookieFactoryImpl();
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory(
                "IDL:omg.org/Components/Cookie:1.0", vf);
            
            vf = new cdmw.ccm.cif.ConnectionDescriptionFactoryImpl();
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory(
                "IDL:omg.org/Components/ConnectionDescription:1.0", vf);
            
            vf = new cdmw.ccm.cif.ReceptacleDescriptionFactoryImpl();
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory(
                "IDL:omg.org/Components/ReceptacleDescription:1.0", vf);
            
            vf = new cdmw.ccm.cif.ConsumerDescriptionFactoryImpl();
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory(
                "IDL:omg.org/Components/ConsumerDescription:1.0", vf);
            
            vf = new cdmw.ccm.cif.EmitterDescriptionFactoryImpl();
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory(
                "IDL:omg.org/Components/EmitterDescription:1.0", vf);
            
            vf = new cdmw.ccm.cif.SubscriberDescriptionFactoryImpl();
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory(
                "IDL:omg.org/Components/SubscriberDescription:1.0", vf);
            
            vf = new cdmw.ccm.cif.PublisherDescriptionFactoryImpl();
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory(
                "IDL:omg.org/Components/PublisherDescription:1.0", vf);
            
            vf = new cdmw.ccm.cif.ComponentPortDescriptionFactoryImpl();
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory(
                "IDL:omg.org/Components/ComponentPortDescription:1.0", vf);
    
            vf = new cdmw.ccm.container.ConfigValueFactoryImpl();
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory(
                "IDL:omg.org/Components/ConfigValue:1.0", vf);
    
            HomeAllocator allocator = null;
            // Register HomeAllocator
            allocator  = new HomeAllocator(
                "Cdmw.CCM.CIF.CdmwRecordingTest.SessionServerHome_impl");
            HomeAllocatorRegistry.register(
                "Cdmw.CCM.CIF.CdmwRecordingTest.SessionServerHome_impl", 
                allocator);

            ComponentInstallationPlugImpl compInstallPlug
                = new ComponentInstallationPlugImpl(rootPOA);
            com.thalesgroup.CdmwDeployment.ComponentInstallation compInstall
                = compInstallPlug._this(orb);
    
            org.omg.Components.ConfigValue[] configValues = 
                new org.omg.Components.ConfigValue[1];
            
            org.omg.CORBA.Any value = orb.create_any();
            com.thalesgroup.CdmwDeployment.ComponentKindValueHelper.insert(
                value, 
                com.thalesgroup.CdmwDeployment.ComponentKindValue.SESSION);
            configValues[0] = new cdmw.ccm.container.ConfigValueImpl(
                com.thalesgroup.CdmwDeployment.COMPONENT_KIND.value, 
                value);
    
            // Create an Object registration instance
            // get the admin root context
            System.out.println("get the admin root context");
            com.thalesgroup.CdmwNamingAndRepository.Repository repositoryRef =
                cdmw.namingandrepository.RepositoryInterface.getRepository();
            org.omg.CosNaming.NamingContext adminRootCtx = 
                repositoryRef.resolve_root_context(
                    cdmw.common.Locations.ADMIN_ROOT_CONTEXT_ID);
            cdmw.commonsvcs.naming.NamingInterface ni =
                new cdmw.commonsvcs.naming.NamingInterface(adminRootCtx);

            // bind the new context where component shall be registered
            org.omg.CosNaming.NameComponent[] componentNamePath =
                new org.omg.CosNaming.NameComponent[5];
            componentNamePath[0] =
                new org.omg.CosNaming.NameComponent(cdmw.ossupport.OS.getHostname(), "");
            componentNamePath[1] =
                new org.omg.CosNaming.NameComponent(APPLICATION_NAME, "");
            componentNamePath[2] =
                new org.omg.CosNaming.NameComponent(PROCESS_NAME, "");
            componentNamePath[3] =
                new org.omg.CosNaming.NameComponent("CCM", "");
            componentNamePath[4] =
                new org.omg.CosNaming.NameComponent("Components", "");

            java.lang.String componentPath = ni.toString(componentNamePath);
            cdmw.commonsvcs.naming.NamingInterface niComponent =
                ni.bindNewContext(componentPath, true);
            org.omg.CosNaming.NamingContext namingContext = 
                niComponent.getNamingContext();

            com.thalesgroup.CdmwLifeCycle.ObjectRegistration objectRegistration =
                new cdmw.lifecycle.DefaultObjectRegistration(namingContext, false);

            org.omg.PortableServer.POA containerPOA = 
                createContainerPOA(rootPOA);
            org.omg.Components.HomeRegistration homeRegistration = null;
            com.thalesgroup.CdmwEvent.EventChannelFactory noECF = null;
            org.omg.Components.ConfigValue[] cfgCompserver =
                new org.omg.Components.ConfigValue[0];
            ComponentServerImpl cs =
                new ComponentServerImpl(orb,
                                        containerPOA,
                                        homeRegistration,
                                        objectRegistration,
                                        noECF,
                                        PROCESS_NAME + "1",
                                        APPLICATION_NAME,
                                        cfgCompserver);

            org.omg.PortableServer.Servant serv = cs;
            org.omg.Components.Deployment.ComponentServer theComponentServer = cs._this(orb);
            cs.set_component_installation(compInstall);

            // Create a container instance
            org.omg.Components.Deployment.Container container = 
                theComponentServer.create_container(configValues);
            
            String containerIOR = orb.object_to_string(container);
            
            System.out.println("Server initialised");
            // -----------
        {

          // Start RecordingServer emulator 
         RecordingServerEmulatorThread server = new RecordingServerEmulatorThread(orb, args);
    
          server.start();
          System.out.println("waiting 5 sec..");
          OS.sleep(timescale*5000);            

        }

        {
          // Starting the RecordingDaemon
          String recordingDaemon = "cdmw_recordingdaemon";

          String namingCorbaloc =
              " -ORBInitRef NameService=corbaloc:iiop:localhost:" + nameServicePort + "/NameService";

          String recordingArgs ="";
//           recordingArgs += " --debug_file=./recording.log";
          recordingArgs += " --CdmwProcessPort=59875";
          recordingArgs += " --rebind_to_ns" + namingCorbaloc;
          recordingArgs += " -DInitial_TCPPorts_List=6072";
          recordingArgs += " --recorder_url=";
          recordingArgs += cdmw.ccm.cif.test.TestServer.recordingServerUrl;
            
          System.out.println("start recording daemon");
          System.out.println(recordingDaemon + recordingArgs);
            
          idDaemon =  OS.createProcess(recordingDaemon + recordingArgs);
            
          OS.sleep(timescale*5000);            
        }

        s = ORBUtils.NAMING_PORT_OPTION + "corbaloc::localhost:" 
                       + nameServicePort + "/NameService";
        s += " ";  
        s += Options.CALLBACK_IOR_OPTION;
        s += "=" + callbackIOR;
//         s += " ";  
//         s += Options.SYSTEM_PORT_OPTION;
//         s += "=";
//         s += nameServicePort;
        s += " --CdmwXMLFile=server.xml";
        s += " " + Options.VALIDATE_XML_OPTION;
        s += " --CdmwLogLevel=ALL";

        System.out.println("cdmwInit args = " + s);

        serverArgs = s.split(" ");

        CdmwInterface.cdmwInit(orb, serverArgs, null, "cdmw.generated.ServerCdmwInterface");

        // -----------

        status = run(orb, containerIOR);

        OS.sleep(timescale * 2000);
            
        } catch(Exception e) {
            System.err.println(e.toString());
            status = FAILURE;
        }
        
        if (orb != null) {
            try {
                orb.destroy();
            } catch(Exception e) {
                System.err.println(e.toString());
                status = FAILURE;
            } 
        }

        OS.killProcess(idDaemon);

        // And we kill launched process
        // for the simulated repository, launch process is a script shell
        // which ended when process are launched: no kill of this process
        // to stop sim repository, the stop_simu.sh script shall be called
        try {
            cdmw.tools.SimuUtils.stopSimu(System.out);
            OS.sleep(timescale * 2000);
        } catch (InterruptedException ie) {
            ie.printStackTrace();
            status = FAILURE;
        }
      
        if (status == FAILURE) {
            System.out.println("Main test failed.");
        } else {
            System.out.println("Main test succeeded.");
        }
        System.exit(0);
}

}

