/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.cif.test;

import cdmw.testutils.CORBATestManager;
import cdmw.testutils.TestManager;
import cdmw.ossupport.OS;
import cdmw.orbsupport.*;

public class RecordingServerEmulatorThread extends Thread {
    
    private org.omg.CORBA.ORB orb;
    private String args[];
    
    private String containerIOR;
    
    public RecordingServerEmulatorThread(org.omg.CORBA.ORB orb, String[] args) {
        this.orb = orb;
        this.args = args;
    }
    
    public void run() {       
        try {
            org.omg.PortableServer.POA rootPOA;
            org.omg.PortableServer.POA poa;

            // Resolve Root POA.
            rootPOA = org.omg.PortableServer.POAHelper.narrow(
                                                              orb.resolve_initial_references("RootPOA"));

            //Create IdAssignmentPolicy and Create PolicyList
            PolicyList policies = new PolicyList();

            policies.insert(
                            rootPOA.create_id_assignment_policy(
                                                                org.omg.PortableServer.IdAssignmentPolicyValue.USER_ID));

            poa = rootPOA.create_POA("poa",rootPOA.the_POAManager(), policies.getList());

            // Activate POA manager
            org.omg.PortableServer.POAManager mgr = poa.the_POAManager();
            mgr.activate();

            String RECORDER_NAME = "Recorder";

            // Create our servant.
            RecorderImpl recorder_servant =  new RecorderImpl();

            // Creation OK, activate object
            byte[] oid = RECORDER_NAME.getBytes();
            poa.activate_object_with_id(oid, recorder_servant);

            org.omg.CORBA.Object object = poa.servant_to_reference (recorder_servant);

            cdmw.ccm.cif.test.TestServer.recordingServerUrl = orb.object_to_string(object);
            
        
            // Accept requests
            orb.run();
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(1);
        }
    }
    
}

