/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.cif.test;

import cdmw.ccm.container.ConfigValueImpl;
import cdmw.ossupport.OS;
import cdmw.testutils.CORBATestManager;
import cdmw.testutils.TestManager;

import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

public class TestServer {

    public static final String SERVER_HOME_REP_ID = 
        "IDL:thalesgroup.com/Cdmw_RecordingTest/ServerHome:1.0";
    public static final String SERVER_HOME_SERVANT_NAME = 
        "Cdmw.CCM.CIF.CdmwRecordingTest.SessionServerHome_impl";
    public static final String SERVER_HOME_UID = "ServerHome_1";
    public static final String SERVER_HOME_ENTRY_POINT = 
        "cdmw.ccm.cif.test.CCMServerHomeImpl.createCCMServerHome";
    public static final String SERVER_NAME =  "MyServer";
    public static final String SERVER_COMPONENT_NAME_1 = "MyServerComponent1";
    public static final String SERVER_COMPONENT_NAME_2 = "MyServerComponent2";
    public static String recordingServerUrl = "";

    private org.omg.CORBA.ORB orb;

    private org.omg.Components.Deployment.Container container;

    
    @Before
    public void init() throws Exception {
        CORBATestManager testManager = (CORBATestManager)TestManager.getLocalInstance();

        String containerIOR =
            (String)testManager.getLocalProperty(
                "cdmw.ccm.cif.test.RecordingTest.CONTAINER_IOR");

        this.orb = testManager.getORB();

        org.omg.CORBA.Object obj = this.orb.string_to_object(containerIOR);
        assertTrue("TEST FAILED : exception raised during test initialisation!",
                   obj != null);
                   
        this.container = 
            org.omg.Components.Deployment.ContainerHelper.narrow(obj);

        try {
            OS.sleep(TestManager.getTimescale() * 2000);
        } catch(InterruptedException ie) {}
    }
    
    @Test
    public void doTests() {
        
        org.omg.Components.CCMHome serverHome = null;
        com.acme.RecordingTest.ServerHome basicServerHome = null;
        com.thalesgroup.CdmwCcmCif.KeylessCCMHome keylessServerHome = null;
        org.omg.Components.CCMObject ccmServerComp = null;
        com.acme.RecordingTest.Server basicServerComp1 = null;
        com.acme.RecordingTest.Server basicServerComp2 = null;
        com.acme.RecordingTest.Server basicServerComp3 = null;

//        /*
//         * Config Values used by the context of components
//         */

//         org.omg.Components.ConfigValue[] configValuesHome =
//             new org.omg.Components.ConfigValue[1];
//         org.omg.CORBA.Any a = orb.create_any();

//         // Creating of a config value to register the component for the recording
//         boolean record = true;
        
//         a.insert_boolean(record);
//         configValuesHome[0] = new ConfigValueImpl(
//             com.thalesgroup.CdmwDeployment.RECORDING.value,
//             a);
    
        // facet providing the sleep
        org.omg.CORBA.Object facettesterFacetObj = null;
        
        //-------------------------------------------------

        System.out.println("Testing of ORBClass and ORBSingletonClass ...");

        try {
            String s = "";
            s = System.getProperty("org.omg.CORBA.ORBClass");
            assertTrue("Bad value : org.omg.CORBA.ORBClass='" + s + "'",
                    s.equals("org.jacorb.orb.ORB"));
            s = System.getProperty("org.omg.CORBA.ORBSingletonClass");
            assertTrue("Bad value : org.omg.CORBA.ORBSingletonClass='" + s + "'",
                    s.equals("org.jacorb.orb.ORBSingleton"));

        }
        catch ( Exception e) {
            System.err.println("Unexpected system exception raised: ");
            fail("Unexpected Exception raised: " + e.toString());
        }

        //-------------------------------------------------

        System.out.println("Testing of resolving initial references ...");

        try {
            System.out.println("Testing of resolving initial references : "
                    + com.thalesgroup.CdmwRecording.REC_CONTAINER_MNGR_ID.value);
            org.omg.CORBA.Object obj =  orb.resolve_initial_references(
                    com.thalesgroup.CdmwRecording.REC_CONTAINER_MNGR_ID.value);
            assertTrue(com.thalesgroup.CdmwRecording.REC_CONTAINER_MNGR_ID.value 
                    + " not found", obj != null);
            com.thalesgroup.CdmwRecording.ContainerManager recordingContainerMngr =
                com.thalesgroup.CdmwRecording.ContainerManagerHelper.narrow(obj);

            System.out.println("Testing of resolving initial references : "
                    + "RecordingServer");
            obj = orb.resolve_initial_references("RecordingServer");
            assertTrue("RecordingServer not found", obj != null);
            com.thalesgroup.CdmwRecording.Recorder recordingServer =
                com.thalesgroup.CdmwRecording.RecorderHelper.narrow(obj);

            com.thalesgroup.CdmwRecording.GroupHeader header = 
                new com.thalesgroup.CdmwRecording.GroupHeader();
            byte[] data = new byte[0];
            recordingServer.record(header, data);

        }
        catch ( Exception e) {
            System.err.println("Unexpected system exception raised: ");
            fail("Unexpected Exception raised: " + e.toString());
        }

        System.out.println("Testing server component... ");
    
        try {
            System.out.println("Installing a ServerHome");
    
            try {
                org.omg.Components.ConfigValue[] configValues = 
                    new org.omg.Components.ConfigValue[5];

                org.omg.CORBA.Any value = orb.create_any();
                com.thalesgroup.CdmwDeployment.ComponentKindValueHelper.insert(
                    value, 
                    com.thalesgroup.CdmwDeployment.ComponentKindValue.SESSION);
                configValues[0] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.COMPONENT_KIND.value, 
                    value);

                value = orb.create_any();
                com.thalesgroup.CdmwDeployment.ServantLifetimeValueHelper.insert(
                    value, 
                    com.thalesgroup.CdmwDeployment.ServantLifetimeValue.CONTAINER_LIFETIME);
                configValues[1] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.SERVANT_LIFETIME.value, 
                    value);

                value = orb.create_any();
                com.thalesgroup.CdmwDeployment.ThreadingPolicyValueHelper.insert(
                    value, 
                    com.thalesgroup.CdmwDeployment.ThreadingPolicyValue.MULTITHREAD);
                configValues[2] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.THREADING_POLICY.value, 
                    value);

                value = orb.create_any();
                value.insert_string(SERVER_HOME_REP_ID);
                configValues[3] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.HOME_REPOSITORY_ID.value, 
                    value);

                value = orb.create_any();
                value.insert_string(SERVER_HOME_SERVANT_NAME);
                configValues[4] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.HOME_SERVANT_CLASSNAME.value, 
                    value);

                serverHome = this.container.install_home(
                    SERVER_HOME_UID,
                    SERVER_HOME_ENTRY_POINT,
                    configValues);
                    
            } catch (org.omg.CORBA.BAD_PARAM bp) {
                fail(bp.toString());
            } catch (org.omg.Components.Deployment.InstallationFailure f) {
                fail("Unexpected exception raised: " + 
                    "Components::Deployment::InstallationFailure " + f.reason);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }
        
            
            System.out.println("Create a Server component using create() operation");
            try {
                basicServerHome =
                    com.acme.RecordingTest.ServerHomeHelper.narrow(
                        serverHome);
                assertTrue("Home is not a RecordingTest.ServerHome !",
                           basicServerHome != null);

                basicServerComp1 = basicServerHome.create();

                assertTrue("ERROR: basicServerComp is nil!", 
                           basicServerComp1 != null);
                           
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected CORBA.SystemException raised: " + se.toString());
            }

            System.out.println("Create a Server component using create_component_with_config_values() operation");
            try {
                keylessServerHome =
                    com.thalesgroup.CdmwCcmCif.KeylessCCMHomeHelper.narrow(serverHome);
                assertTrue("ERROR: Home is not a Components.KeylessCCMHome!",
                           keylessServerHome != null);

                org.omg.Components.ConfigValue[] configValues =
                    new org.omg.Components.ConfigValue[2];

                org.omg.CORBA.Any value = orb.create_any();
                value.insert_string(SERVER_COMPONENT_NAME_1);
                configValues[0] = 
                    new cdmw.ccm.container.ConfigValueImpl(
                          com.thalesgroup.CdmwDeployment.COMPONENT_NAME.value, 
                          value);

                value = orb.create_any();
                value.insert_boolean(true);
                configValues[1] = 
                    new cdmw.ccm.container.ConfigValueImpl(
                          com.thalesgroup.CdmwDeployment.RECORDING.value, 
                          value);

                ccmServerComp = 
                    keylessServerHome.create_component_with_config_values(
                        configValues);
                assertTrue("ERROR: ccmServerComp is nil!", 
                           ccmServerComp != null);
                           
                basicServerComp2 =
                    com.acme.RecordingTest.ServerHelper.narrow(ccmServerComp);
                assertTrue("ERROR: basicServerComp is nil!", 
                           basicServerComp2 != null);
                           
            } catch (org.omg.Components.CreateFailure cf) {
                fail("Unexpected exception raised: CreateFailure");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Create a Server component using create_component_with_config_values() operation");
            try {
                keylessServerHome =
                    com.thalesgroup.CdmwCcmCif.KeylessCCMHomeHelper.narrow(serverHome);
                assertTrue("ERROR: Home is not a Components.KeylessCCMHome!",
                           keylessServerHome != null);

                org.omg.Components.ConfigValue[] configValues =
                    new org.omg.Components.ConfigValue[2];

                org.omg.CORBA.Any value = orb.create_any();
                value.insert_string(SERVER_COMPONENT_NAME_2);
                configValues[0] = 
                    new cdmw.ccm.container.ConfigValueImpl(
                          com.thalesgroup.CdmwDeployment.COMPONENT_NAME.value, 
                          value);

                value = orb.create_any();
                value.insert_boolean(true);
                configValues[1] = 
                    new cdmw.ccm.container.ConfigValueImpl(
                          com.thalesgroup.CdmwDeployment.RECORDING.value, 
                          value);

                ccmServerComp = 
                    keylessServerHome.create_component_with_config_values(
                        configValues);
                assertTrue("ERROR: ccmServerComp is nil!", 
                           ccmServerComp != null);
                           
                basicServerComp3 =
                    com.acme.RecordingTest.ServerHelper.narrow(ccmServerComp);
                assertTrue("ERROR: basicServerComp is nil!", 
                           basicServerComp3 != null);
                           
            } catch (org.omg.Components.CreateFailure cf) {
                fail("Unexpected exception raised: CreateFailure");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("tester operation on Server component...");
            try {
                facettesterFacetObj = basicServerComp3.provide_for_clients();
                assertTrue("ERROR: No facet found for Server component", 
                           facettesterFacetObj != null);
                           
                // store the facet's ior
                String ior = orb.object_to_string(facettesterFacetObj);
                java.io.File file = new java.io.File("file.ior");
                try {
                    java.io.Writer out = new java.io.FileWriter(file);
                    out.write(ior);
                    out.close();
                } catch(java.io.IOException e) { 
                    assertTrue("Error writing facet's IOR: "+e,
                               false);
                }
                
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Configuration_complete operation on Server component...");
            try {
                basicServerComp3.configuration_complete();
            } catch (org.omg.Components.InvalidConfiguration ic) {
                fail("Unxpected exception raised: InvalidConfiguration: " + ic);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se);
            }


            // create Client process
            System.out.println("Create Client process");
            cdmw.ossupport.ThreadedProcess clientp = null;
            String classname = "cdmw.ccm.cif.test.MainClient";
            try {
                clientp = OS.createJavaProcess(classname);
                // wait Client process to end
                System.out.println("Wait Client process to end...");
                clientp.getProcess().waitFor();
                System.out.println("Client process ended");
            } catch(java.io.IOException ioe) {
                throw new InterruptedException("Unable to create client process.");
            }

            try {
                OS.sleep(TestManager.getTimescale() * 2000);
            } catch(InterruptedException ie) {}
        
            System.out.println("remove() operation on a Server component...");
            try {
                basicServerComp1.remove();
            } catch (org.omg.Components.RemoveFailure rf) {
                fail("Unexpected exception raised: NoKeyAvailable");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("remove_component() operation on the Server Home");
            try {
                basicServerHome.remove_component(basicServerComp2);
            }
            catch (org.omg.Components.RemoveFailure rf) {
                fail("Unexpected exception raised: RemoveFailure");
            }
            catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
    
            System.out.println("remove_home() operation on the Container...");
            try {
                this.container.remove_home(basicServerHome);
            } catch (org.omg.Components.RemoveFailure rf) {
                fail("Unexpected exception raised: NoKeyAvailable");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
            
        } catch (Exception e)  {
            fail("Unexpected exception raised: " + e.toString());
        }
            
    }
}

