/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef ___TEST_CCM_CLIENT_IMPL_HPP__
#define ___TEST_CCM_CLIENT_IMPL_HPP__

#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/orbsupport/RefCountLocalObject.hpp"
#include "Foundation/osthreads/ThreadHandle.hpp"
#include "Foundation/osthreads/Mutex.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"

#include "testtypecheckingtimeout/UserTest.skel.hpp"

namespace Test
{

// forward declaration
class CCM_Client_impl;
class ClientActivationThread;
class test_ccm_client_activate;
/**
*  This class provides an object to test the ccm_activate.
*
**/



//
// IDL:thalesgroup.com/Test/CCM_Client:1.0
//
class CCM_Client_impl : virtual public ::UserTest::User_CCM_Client,
                        virtual public Cdmw::OrbSupport::RefCountLocalObject
{
     friend class Test::test_ccm_client_activate;

    CCM_Client_impl(const CCM_Client_impl&);
    void operator=(const CCM_Client_impl&);

    ::Test::CCM_Client_Context_var m_context;

     ClientActivationThread*                m_activation_thread;
     Cdmw::OsSupport::Mutex               m_mutex;

     
     
public:

    CCM_Client_impl();
    ~CCM_Client_impl();

    void set_session_context(Components::SessionContext_ptr ctx)
                 throw(CORBA::SystemException);
    void ccm_activate()
                 throw(CORBA::SystemException);
    void ccm_passivate()
                 throw(CORBA::SystemException);
    void ccm_remove()
                 throw(CORBA::SystemException);
;
     ::Test::CCM_Client_Context* get_context()
     {
         return ::Test::CCM_Client_Context::_duplicate(m_context.in());
     }

private:

};
class ClientActivationThread : public Cdmw::OsSupport::Thread 
{
public:
    ClientActivationThread(Test::CCM_Client_impl* client)
         : m_client(client)
     {
     }

    ~ClientActivationThread() throw() {};

protected:
    void
    run() throw()
     {
         Cdmw::TestUtils::CORBATestManager::instance()->add("client", static_cast< ::UserTest::User_CCM_Client_ptr > (m_client));
         Cdmw::TestUtils::ThreadedTestRunner runner(Cdmw::TestUtils::CORBATestManager::instance()->get_controller());
         runner.addTest( CppUnit::TestFactoryRegistry::getRegistry("test_ccm_client_activate").makeTest() );
         runner.start();
         runner.join();
     }

    
private:
    
    ClientActivationThread();
    ClientActivationThread(const ClientActivationThread&);
    ClientActivationThread& operator = (const ClientActivationThread&);
    
    Test::CCM_Client_impl* m_client;

};

}; // End of namespace Test


#endif // ___TEST_CCM_CLIENT_IMPL_HPP__

