/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Unix Files
#include <stdlib.h>


// Cdmw Files
#include "Foundation/common/Assert.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include <Foundation/osthreads/MutexGuard.hpp>
#include "testtypecheckingtimeout/CCM_Client_impl.hpp"
#include <CCMContainer/ccmcommon/CCMSystemProperties.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>

namespace 
{
    
} // End of anonymous namespace

/**
*  This class provides an object to test the ccm_activate.
*
**/


namespace Test
{

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(test_ccm_client_activate, "test_ccm_client_activate");

class test_ccm_client_activate : public CppUnit::TestFixture
{
  Test::CCM_Client_impl* m_client;

 public:
  test_ccm_client_activate()
    {
        CORBA::Object_var obj = Cdmw::TestUtils::CORBATestManager::instance()->get_object("client");
        ::UserTest::User_CCM_Client_var user_ccm_client = ::UserTest::User_CCM_Client::_narrow(obj.in());
        m_client  = reinterpret_cast< ::Test::CCM_Client_impl* > (user_ccm_client.in());
    };

  ~test_ccm_client_activate() {};

    void 
    do_tests()
    {
        // set number of requested successfull tests
        //      set_nbOfRequestedTestOK (4);
        
        TEST_INFO("********************************************");
        TEST_INFO("            ccm_activate start");
        TEST_INFO("********************************************");
        
        CDMW_ASSERT(m_client);
        
        ::Test::Foo_var foo;
        ::Test::CCM_Client_Context_var client_context = m_client->get_context();
      
        TEST_INFO("Client: Get_connection_foo_receptacle...");
        bool test_ok = true;
        try
        {
          
          foo = client_context->get_connection_foo_receptacle();
          
          if(Cdmw::OsSupport::SystemProperties::Get_Property("Num.Test")
             == "1016") {
             TEST_INFO("*** get_connection() not blocking.");
          }
        }
        catch (const CORBA::SystemException& e)
        {
          // InvalidConnection must be caught by SimDemployment
          // during configuration_complete().
          TEST_INFO("Client: Unexpected exception caught: " << e);
          test_ok = false;
        }
        CPPUNIT_ASSERT(test_ok);

        TEST_INFO("********************************************");
        TEST_INFO("            ccm_activate stop");
        TEST_INFO("********************************************");


    } // End of do_test()

    CPPUNIT_TEST_SUITE( test_ccm_client_activate );
    CPPUNIT_TEST( do_tests );
    CPPUNIT_TEST_SUITE_END();
};
    
} //end of namespace

//
// IDL:acme.com/Test/CCM_Client:1.0
//
Test::CCM_Client_impl::CCM_Client_impl()
     : m_context(Test::CCM_Client_Context::_nil())
{
      m_activation_thread = new ClientActivationThread(this);
}

Test::CCM_Client_impl::~CCM_Client_impl()
{
    std::cout << "CCM_Client_impl::~CCM_Client_impl called!" << std::endl;
    delete m_activation_thread;
}


//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
Test::CCM_Client_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    std::cout  << "Client: "<< "Set_session_context called!" << std::endl;
   
    m_context = ::Test::CCM_Client_Context::_narrow(ctx); 
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
Test::CCM_Client_impl::ccm_activate()
   throw(CORBA::SystemException)
{
  // Start thread
  m_activation_thread->start();
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//

void
Test::CCM_Client_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
   std::cout  << "Client: "<< "Ccm_passivate called!" << std::endl;
} 



//
// ccm_remove function : it destroys the component (call by the container)
void
Test::CCM_Client_impl::ccm_remove()
   throw(CORBA::SystemException)
{
   std::cout  << "Client: "<< "Ccm_remove called!" << std::endl;
}



