/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <sstream>
#include <string>

#include <testtypecheckingtimeout/SimDeployment.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>

#include <CCMContainer/ccmcommon/CCMSystemProperties.hpp>
#include <CCMContainer/ccmcontainer/ConfigValue_impl.hpp>
#include <testtypecheckingtimeout/Test_cif.skel.hpp>
#include <Foundation/idllib/CdmwDeployment.skel.hpp>

namespace
{
    const char* CLIENT_HOME_REP_ID = "IDL:acme.com/Cdmw_Test/ClientHome:1.0";
    const char* CLIENT_HOME_SERVANT_NAME = "Cdmw.CCM.CIF.CdmwTest.SessionClientHome_impl";
    const char* CLIENT_HOME_UID = "ClientHome_1";
    const char* CLIENT_HOME_ENTRY_POINT = "create_CCM_ClientHome";

    const char* FOO_FACET_NAME = "the_foo";

    const char* FOO_RECEP_NAME = "foo_receptacle";

}; // End anonymous namespace

namespace Cdmw
{
namespace CCM
{
namespace CIF
{

/**
*Implementation notes: [if no pertinent write none ]
*<p>
*Portability issues: [if no pertinent write none ]
*<p>
*/

CPPUNIT_TEST_SUITE_REGISTRATION( SimDeployment );

SimDeployment::SimDeployment()
   throw(CORBA::SystemException)
{
    m_orb =  CORBA::ORB::_duplicate( Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());

    CORBA::Object_var obj = m_orb->string_to_object(Cdmw::TestUtils::CORBATestManager::instance()->get_property ("containerIOR").c_str());
    try{
            if (CORBA::is_nil(obj.in()))
              {
                TEST_INFO("TEST FAILED : exception raised during test initialisation!");
              }
            else
              {
                m_container = Components::Deployment::Container::_narrow(obj.in());
              }

          }
        catch (const CORBA::SystemException & e)
          {
            TEST_INFO("TEST FAILED : system exception raised during test initialisation!");
            throw;
          }
        catch (...)
          {
            TEST_INFO("TEST FAILED : unexpected exception raised during test initialisation!");
          }
      }


      SimDeployment::~SimDeployment()
        throw()
      {
        // does nothing
      }


      SimDeployment::SimDeployment(const SimDeployment& rhs)
        throw()
        : m_orb(CORBA::ORB::_duplicate(rhs.m_orb))
{
    // does nothing
}
        
SimDeployment&
SimDeployment::operator=(const SimDeployment& rhs)
    throw()
{
    if (this != &rhs) {
        m_orb = CORBA::ORB::_duplicate(rhs.m_orb.in());
        m_container = Components::Deployment::Container::_duplicate(rhs.m_container.in());
    }
    return *this;
}

void SimDeployment::do_tests()        
{
     // set number of requested successfull tests
//     set_nbOfRequestedTestOK (52);
    
    bool test_ok=false;
    Components::CCMHome_var client_home;
    Test::ClientHome_var test_client_home;
    Components::CCMObject_var ccm_client_comp;
    Test::Client_var test_client_comp1;

    CORBA::Object_var foo_facet_obj1;

    Components::Cookie_var recep_ck;

    int timescale = Cdmw::TestUtils::get_timescale();

    // This tests CIF classes
    //-------------------------------------------------
    TEST_INFO("Testing CIF classes");
    
//     try 
//     { 
      
        TEST_INFO("Install a ClientHome");
        test_ok=false;

        try
        {
            Components::ConfigValues config_values(8);
            config_values.length(5);
            CORBA::Any value;
            value <<= CdmwDeployment::SESSION;
            config_values[0] 
                = new Container::ConfigValue_impl(CdmwDeployment::COMPONENT_KIND, 
                                                   value);

            value <<= CdmwDeployment::CONTAINER_LIFETIME;
            config_values[1]
                = new Container::ConfigValue_impl(CdmwDeployment::SERVANT_LIFETIME, 
                                                   value);

            value <<= CdmwDeployment::MULTITHREAD;
            config_values[2]
                = new Container::ConfigValue_impl(CdmwDeployment::THREADING_POLICY, 
                                                   value);

            value <<= CLIENT_HOME_REP_ID;
            config_values[3]
                = new Container::ConfigValue_impl(CdmwDeployment::HOME_REPOSITORY_ID, 
                                                   value);

            value <<= CLIENT_HOME_SERVANT_NAME;
            config_values[4]
                = new Container::ConfigValue_impl(CdmwDeployment::HOME_SERVANT_CLASSNAME, 
                                                   value);
            client_home 
               = m_container->install_home(CLIENT_HOME_UID,
                                           CLIENT_HOME_ENTRY_POINT,
                                           config_values);
            test_ok=true;
        }
        catch (const CORBA::BAD_PARAM& e)
        {
            TEST_INFO(e);
        }
        catch (const Components::Deployment::InstallationFailure& e)
        {
            std::ostringstream os;
            os << "Unexpected exception raised: Components::Deployment::InstallationFailure " << e.reason << std::ends;
            TEST_INFO(os.str());
        }
        catch (const CORBA::SystemException& e)
        {
            std::ostringstream os;
            os << "Unexpected system exception raised: " << e << std::ends;
            TEST_INFO(os.str());
        }
        catch (const CORBA::UserException& e)
        {
            std::ostringstream os;
            os << "Unexpected User exception raised: " << e << std::ends;
            TEST_INFO(os.str());
        }
        CPPUNIT_ASSERT(test_ok);
        
        TEST_INFO("Create a Client component using create_component() operation");
        test_ok=false;
        try
        {
            Components::KeylessCCMHome_var keyless_client_home 
                = Components::KeylessCCMHome::_narrow(client_home.in());

            if (CORBA::is_nil(keyless_client_home.in()))
            {
                TEST_INFO("ERROR: Created home is not a keyless home!");
            }
            else
            {
                ccm_client_comp
                   = keyless_client_home->create_component();

                test_client_comp1
                    = Test::Client::_narrow(ccm_client_comp.in());

                if (CORBA::is_nil(test_client_comp1.in()))
                {
                    TEST_INFO("ERROR: Created component is not a Test::Client!")
                }
                else
                {
                    test_ok=true;
                }
            }
        }
        catch (const Components::CreateFailure& e)
        {
            TEST_INFO("Unexpected exception raised: CreateFailure");
        }
        catch (const CORBA::SystemException& e)
        {
            std::ostringstream os;
            os << "Unexpected exception raised: " << e << std::ends;
            TEST_INFO(os.str());
        }
        CPPUNIT_ASSERT(test_ok);

//         TEST_INFO("provide_facet operation on Client component...");
//         test_ok=false;
//         try
//         {
//             foo_facet_obj1
//                 = test_client_comp1->provide_facet(FOO_FACET_NAME);

//             if (CORBA::is_nil(foo_facet_obj1.in()))
//             {
//                 TEST_INFO("ERROR: No facet found for Client component");
//             }
//             else
//             {
//               if (CORBA::is_nil(foo_facet_obj1.in()))
//                 {
//                     TEST_INFO("ERROR: No facet found for Client component");
//                 }
//                 else
//                 {
//                     test_ok=true;
//                 }
//             }
//         }
//         catch (const Components::InvalidName& e)
//         {
//             TEST_INFO("Unexpected exception raised: InvalidName");
//             test_ok=false;
//         }
//         catch (const CORBA::SystemException& e)
//         {
//             std::ostringstream os;
//             os << "Unexpected exception raised: " << e << std::ends;
//             TEST_INFO(os.str());
//             test_ok=false;
//         }
//         CPPUNIT_ASSERT(test_ok);
        TEST_INFO("Connect operation on Client component to Server...");
        test_ok=false;
        try
        {
          if(Cdmw::OsSupport::SystemProperties::Get_Property("Num.Test") == "1017") {
            Cdmw::OsSupport::SystemProperties::Set_Property(
                                                            Cdmw::CCM::TYPECHECKING_PROPERTY_NAME,
                                                            "on");

            std::cout << "*** " << Cdmw::CCM::TYPECHECKING_PROPERTY_NAME
                      << " set to on to test connect() from CCMObjectImpl and default "
                      << Cdmw::CCM::TYPECHECKING_TIMEOUT_PROPERTY_NAME
                      << " is used "
                      << std::endl;
          }else if (Cdmw::OsSupport::SystemProperties::Get_Property("Num.Test") == "1018"){
            Cdmw::OsSupport::SystemProperties::Set_Property(
                                                            Cdmw::CCM::TYPECHECKING_PROPERTY_NAME,
                                                            "on");
            Cdmw::OsSupport::SystemProperties::Set_Property(
                                                            Cdmw::CCM::TYPECHECKING_TIMEOUT_PROPERTY_NAME,
                                                            "1000");
             std::cout << "*** " << Cdmw::CCM::TYPECHECKING_PROPERTY_NAME
                      << " set to off to test connect() from CCMObjectImpl"
                      << Cdmw::CCM::TYPECHECKING_TIMEOUT_PROPERTY_NAME
                      << " was set"
                      << std::endl;
          } else {
            Cdmw::OsSupport::SystemProperties::Set_Property(
                                                            Cdmw::CCM::TYPECHECKING_PROPERTY_NAME,
                                                            "off");

             std::cout << "*** " << Cdmw::CCM::TYPECHECKING_PROPERTY_NAME
                      << " set to off to test connect() from CCMObjectImpl"
                      << std::endl;
          }
          
          // fake connection to simulate the problem of the PCR-1877
          foo_facet_obj1 = m_orb->string_to_object("corbaloc::localhost:21869/SimUltimateFallback");
          
          OsSupport::OS::sleep(timescale*10000);

          recep_ck
                = test_client_comp1->connect(FOO_RECEP_NAME,
                                             foo_facet_obj1.in());
            test_ok=true;
            
        }
        catch (const Components::InvalidName& )
        {
            TEST_INFO("Unexpected exception raised: InvalidName");
        }
        catch (const Components::InvalidConnection& )
        {
          if (Cdmw::OsSupport::SystemProperties::Get_Property("Num.Test") == "1017" ||
              Cdmw::OsSupport::SystemProperties::Get_Property("Num.Test") == "1018") {
            TEST_INFO("Expected exception raised: InvalidConnection");
            test_ok=true;
          } else{
            test_ok=false;
            TEST_INFO("Unexpected exception raised: InvalidConnection");
          }
        }
        catch (const Components::AlreadyConnected& )
        {
            TEST_INFO("Unexpected exception raised: AlreadyConnected");
        }
        catch (const Components::ExceededConnectionLimit& )
        {
            TEST_INFO("Unexpected exception raised: ExceededConnectionLimit");
        }
        catch (const CORBA::SystemException& e)
        {
            std::ostringstream os;
            os << "Unexpected exception raised: " << e << std::ends;
            TEST_INFO(os.str());
            test_ok=false;
        }
        CPPUNIT_ASSERT(test_ok);

        TEST_INFO("Configuration_complete operation on Client components...");
        test_ok=false;
        try
        {
            test_client_comp1->configuration_complete();

            OsSupport::OS::sleep(timescale*30000);
            test_ok=true;
        }
        catch (const Components::InvalidConfiguration &)
        {
          if (Cdmw::OsSupport::SystemProperties::Get_Property("Num.Test") == "1017" ||
              Cdmw::OsSupport::SystemProperties::Get_Property("Num.Test") == "1018"){
            TEST_INFO("Expected exception raised: InvalidConnection");
            test_ok=true;
          } else{
            test_ok=false;
            TEST_INFO("Unexpected exception raised: InvalidConnection");
          }
        }
        catch (const CORBA::SystemException& e)
        {
            std::ostringstream os;
            os << "Unexpected exception raised: " << e << std::ends;
            TEST_INFO(os.str());
        }
        CPPUNIT_ASSERT(test_ok);


        TEST_INFO("Disconnect operation on Client component...");
        test_ok=false;
        try
        {
            test_client_comp1->disconnect(FOO_RECEP_NAME, 0);
            test_ok=true;
        }
        catch (const Components::InvalidName& )
        {
            TEST_INFO("Unexpected exception raised: InvalidName");
        }
        catch (const Components::InvalidConnection& )
        {
          if (Cdmw::OsSupport::SystemProperties::Get_Property("Num.Test") == "1017" ||
              Cdmw::OsSupport::SystemProperties::Get_Property("Num.Test") == "1018"){
            TEST_INFO("Expected exception raised: InvalidConnection");
            test_ok=true;
          } else{
            test_ok=false;
            TEST_INFO("Unexpected exception raised: InvalidConnection");
          }
        }
        catch (const Components::CookieRequired& )
        {
            TEST_INFO("Unexpected exception raised: CookieRequired");
        }
        catch (const Components::NoConnection& )
        {
          if (Cdmw::OsSupport::SystemProperties::Get_Property("Num.Test") == "1017" ||
              Cdmw::OsSupport::SystemProperties::Get_Property("Num.Test") == "1018"){
            TEST_INFO("Expected exception raised: NoConnection");
            test_ok=true;
          } else {
            test_ok=false;
            TEST_INFO("Unexpected exception raised: NoConnection");
          }
        }
        catch (const CORBA::SystemException& e)
        {
            std::ostringstream os;
            os << "Unexpected exception raised: " << e << std::ends;
            TEST_INFO(os.str());
            test_ok=false;
        }
        CPPUNIT_ASSERT(test_ok);

//         TEST_INFO("Remove operation on the Client home...");
//         test_ok=false;
//         try
//         {
//             m_container->remove_home(test_client_home.in());
//             test_ok=true;
//         }
//         catch (const Components::RemoveFailure &)
//         {
//             TEST_INFO("Unexpected exception raised: NoKeyAvailable");
//         }
//         catch (const CORBA::SystemException& e)
//         {
//             std::ostringstream os;
//             os << "Unexpected exception raised: " << e << std::ends;
//             TEST_INFO(os.str());
//             test_ok=false;
//         }
//         CPPUNIT_ASSERT(test_ok);

//     TEST_INFO("Set to nil client_home"); 
//     client_home = Components::CCMHome::_nil();
//     TEST_INFO("Set to nil test_client_home"); 
//     test_client_home = Test::ClientHome::_nil();
//     TEST_INFO("Set to nil ccm_client_comp"); 
//     ccm_client_comp = Components::CCMObject::_nil();
   
//     TEST_INFO("Set to nil foo_facet_obj's"); 
//     foo_facet_obj1 = CORBA::Object::_nil();
}


        
}; // End namespace CIF
}; // End namespace CCM
}; // End namespace Cdmw

