/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.cif.test;


import java.io.File;
import java.util.Vector;
import java.util.StringTokenizer;

import org.junit.Assert;

import cdmw.common.Options;
import cdmw.common.Locations;
import cdmw.ccm.common.CCMSystemProperties;

import cdmw.ossupport.OS;
import cdmw.ossupport.FileUtilities;
import cdmw.ossupport.ThreadedProcess;

import cdmw.orbsupport.ORBSupport;
import cdmw.orbsupport.StrategyList;
import cdmw.orbsupport.ORBUtils;
import cdmw.orbsupport.ExceptionMinorCodes;
import cdmw.testutils.TestManager;

import cdmw.ccm.container.ConfigValueImpl;
import cdmw.ccm.container.ContainerImpl;
import cdmw.ccm.container.HomeAllocator;
import cdmw.ccm.container.HomeAllocatorRegistry;
import cdmw.ccm.container.HomesServantActivator;

import cdmw.commonsvcs.naming.NamingInterface;
import cdmw.namingandrepository.RepositoryInterface;
import cdmw.orbsupport.PolicyList;

public class MainSimDaemonAndDance {
    
    public static final String APPLICATION_NAME = "DummyApplicationName";
    
    public static final String CDMW_RUNTIME_ROOT = 
        System.getProperty("CDMW_RUNTIME_ROOT");

    public static final String NAMING_SERVICE_PORT_OPTION  = "--NamePort";
    public static final String REPOSITORY_PORT_OPTION      = "--RepPort";
    
    public static final int SUCCESS = 0;
    public static final int FAILURE = 1;

    public static final int POA_THREAD_POOL_SIZE = 5;
    

    public MainSimDaemonAndDance() {}

    public static org.omg.PortableServer.POA createPOA(
        org.omg.PortableServer.POA parent,
        String POAname) throws Exception {
        //
        // Create a POA with NON_RETAIN and Single-thread policicies
        // to be used to activate the ComponentServer and Container objects.
        // Lifespan policy            = TRANSIENT
        // Id Assignment policy       = USER_ID
        // Id Uniqueness policy       = UNIQUE_ID
        // Implicit Activation policy = NO_IMPLICIT_ACTIVATION
        // Servant Retention policy   = NON_RETAIN;
        // Thread Policy              = SINGLE_THREAD_MODEL
        //        
        org.omg.PortableServer.POAManager poaManager = parent.the_POAManager();
        PolicyList policies = new PolicyList();
        policies.insert(
            parent.create_id_assignment_policy(
                org.omg.PortableServer.IdAssignmentPolicyValue.USER_ID));
        policies.insert(
            parent.create_lifespan_policy(
                org.omg.PortableServer.LifespanPolicyValue.TRANSIENT));
        policies.insert(
            parent.create_servant_retention_policy(
                org.omg.PortableServer.ServantRetentionPolicyValue.NON_RETAIN));
        policies.insert(
            parent.create_id_uniqueness_policy(
                org.omg.PortableServer.IdUniquenessPolicyValue.MULTIPLE_ID));
        policies.insert(
            parent.create_request_processing_policy(
                org.omg.PortableServer.RequestProcessingPolicyValue.USE_SERVANT_MANAGER));
        policies.insert(
            parent.create_implicit_activation_policy(
                org.omg.PortableServer.ImplicitActivationPolicyValue.NO_IMPLICIT_ACTIVATION));

        StrategyList poaStrategies = new StrategyList();

        return ORBSupport.createPOA(
            parent,
            POAname,
            poaManager, 
            policies,
            poaStrategies);
    }

    
    public static void main(String[] args) {
        try{
            org.omg.CORBA.ORB orb = null;
    
            // help argument
            if (args.length >= 1) {
                if ( args[0].equals("--help") ) {
                    System.out.println("Usage:\n"
                                       + "java cdmw.ccm.cif.test.MainSimDaemonAndDance options\n"
                                       + "\n"
                                       + "Options:\n"
                                       + "--help                     Display this information.\n");
                    return;
                }
            }

            System.setProperty("Num.Test", OS.getOptionValue(args, "--NumTest"));
        
            System.out.println("*** Num.Test=" + System.getProperty("Num.Test"));

            int timescale = TestManager.getTimescale();
    
            try {
                StrategyList orbStrategies = new StrategyList();
                orbStrategies.addORBThreaded();
                orbStrategies.addPOAThreadPool(POA_THREAD_POOL_SIZE);
    
                String s = " --CdmwLocalisationService=21869";
                String[] serverArgs = s.split(" ");
                orb = ORBSupport.ORBInit(serverArgs, orbStrategies);

                // PCR-0049
                cdmw.orbsupport.Codec.init(orb);
    
                // Get the root POA 
                //
                org.omg.CORBA.Object obj = 
                    orb.resolve_initial_references("RootPOA");
                org.omg.PortableServer.POA rootPOA = 
                    org.omg.PortableServer.POAHelper.narrow(obj);
            
                //
                // Run implementation (Activate the POA manager)
                //
                System.out.println("Activate the POA manager");
                rootPOA.the_POAManager().activate();
     
                System.out.println("Create a new POA");
                org.omg.PortableServer.POA newPOA = 
                    createPOA(rootPOA, "NewPOA");

                System.out.println("Create Simulated UltimateFallback");
                // Convert channelname to OID
                String name = "SimUltimateFallback";
                final String FOO2_REP_ID 
                    = "IDL:com.acme/Test/Foo2:1.0";
                final String FOO3_REP_ID 
                    = "IDL:com.acme/Test/Foo3:1.0";

                byte[] oid = name.getBytes();                
                org.omg.CORBA.Object obj1 = newPOA.create_reference_with_id(oid, FOO2_REP_ID);

                System.out.println("Create Simulated Dance");
                name = "SimDance";
                oid = name.getBytes();    
                org.omg.CORBA.Object obj2 = newPOA.create_reference_with_id(oid, FOO3_REP_ID);

                System.out.println("Create bind corbaloc");
                cdmw.orbsupport.ORBSupport.bindObjectToCorbaloc(orb, "SimUltimateFallback", obj1);
                cdmw.orbsupport.ORBSupport.bindObjectToCorbaloc(orb, "SimDance", obj2);

                System.out.println("String to corbaloc of SimDance");
                org.omg.CORBA.Object dance = orb.string_to_object("corbaloc::localhost:21869/SimDance");
                

                System.out.println("Create simUltimateFallbackAndDance");
                org.omg.PortableServer.ServantManager simUltimateFallbackAndDance =
                    new SimFtUltimateFallbackAndDance(newPOA, dance);

                System.out.println("Set the servant manager");
                newPOA.set_servant_manager(simUltimateFallbackAndDance);

                OS.sleep(timescale * 1000);

            } catch (org.omg.PortableServer.POAPackage.AdapterAlreadyExists aae) {
                throw new org.omg.CORBA.INTERNAL(
                                                 ExceptionMinorCodes.INTERNALLifeCycleFrameworkError,
                                                 org.omg.CORBA.CompletionStatus.COMPLETED_YES);
            } catch (org.omg.PortableServer.POAPackage.InvalidPolicy ip) {
                throw new org.omg.CORBA.INTERNAL(
                                                 ExceptionMinorCodes.INTERNALLifeCycleFrameworkError,
                                                 org.omg.CORBA.CompletionStatus.COMPLETED_YES);
            } catch (org.omg.CORBA.SystemException se) {
                throw se; // rethrow
            } 
        
                  
            // Main loop
            orb.run();
            OS.sleep(timescale * 3000);
            
            if (orb != null) {
                try {
                    orb.destroy();
                } catch(Exception ex) {
                    System.err.println(ex.toString());
                } 
            }        
        
            // And we kill launched process
            try  {
                OS.sleep(timescale * 2000);
            } catch (InterruptedException ie) {
                System.err.println(ie.toString());
            } catch(Exception ex) {
                ex.printStackTrace();
                System.err.println(ex.toString());
            }
        } catch(Exception ex) {
            ex.printStackTrace();
            System.err.println(ex.toString());
        }
        
    }

}


