/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.cif.test;


import java.io.File;
import java.util.Vector;
import java.util.StringTokenizer;

import org.junit.Assert;

import cdmw.common.Options;
import cdmw.common.Locations;
import cdmw.ccm.common.CCMSystemProperties;

import cdmw.ossupport.OS;
import cdmw.ossupport.FileUtilities;
import cdmw.ossupport.ThreadedProcess;

import cdmw.orbsupport.ORBSupport;
import cdmw.orbsupport.StrategyList;
import cdmw.orbsupport.ORBUtils;
import cdmw.orbsupport.ExceptionMinorCodes;
import cdmw.testutils.TestManager;

import cdmw.ccm.container.ConfigValueImpl;
import cdmw.ccm.container.ContainerImpl;
import cdmw.ccm.container.HomeAllocator;
import cdmw.ccm.container.HomeAllocatorRegistry;
import cdmw.ccm.container.HomesServantActivator;

import cdmw.commonsvcs.naming.NamingInterface;
import cdmw.namingandrepository.RepositoryInterface;
import static cdmw.ccm.common.CCMSystemProperties.TYPECHECKING_TIMEOUT;

public class MainTestTypecheckingTimeout {
    

    public MainTestTypecheckingTimeout() {}
    
    public static int run(
        org.omg.CORBA.ORB orb, 
        String containerIOR) {

        ClientThread client = 
            new ClientThread(orb, containerIOR);
        client.start();
        orb.run();
        try {
            client.join();
        } catch(InterruptedException ie) {}
    
        return SUCCESS;
    }
    
    public static com.thalesgroup.CdmwPlatformMngt.ProcessCallback 
        getProcessCallback(org.omg.CORBA.ORB orb) 
        throws org.omg.CORBA.TRANSIENT {
        
        Assert.assertTrue(orb != null);
        
        org.omg.CORBA.Object obj = null;
        //
        // Get naming service
        //
        try {
            obj = orb.resolve_initial_references("NameService");
        } catch(org.omg.CORBA.ORBPackage.InvalidName in) {
            System.err.println("Can't resolve `NameService'");
            throw new org.omg.CORBA.TRANSIENT(
                ExceptionMinorCodes.TRANSIENTCannotReachRepository,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
    
        if (obj == null) {
            System.err.println("`NameService' is a nil object reference");
            throw new org.omg.CORBA.TRANSIENT(
                ExceptionMinorCodes.TRANSIENTCannotReachRepository,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
        
        org.omg.CosNaming.NamingContextExt nc =
            org.omg.CosNaming.NamingContextExtHelper.narrow(obj);
    
        if (nc == null) {
            System.err.println("`NameService' is not a NamingContextExt object reference");
            throw new org.omg.CORBA.TRANSIENT(
                ExceptionMinorCodes.TRANSIENTCannotReachRepository,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
    
        String PROCESS_CALLBACK_BINDING_NAME = "CDMW.I/process-callback.simulated";
    
        // Get reference to the ProcessCallback object
        com.thalesgroup.CdmwPlatformMngt.ProcessCallback callback = null;
        try {
            NamingInterface ni = new NamingInterface(nc);
            org.omg.CORBA.Object obj__ = 
                ni.resolve(PROCESS_CALLBACK_BINDING_NAME);
            callback = 
                com.thalesgroup.CdmwPlatformMngt.ProcessCallbackHelper.narrow(obj__);
            if (callback == null) {
                System.err.println("Couldn't retrieve reference of the ProcessCallback");
                // TODO: Update minor code
                throw new org.omg.CORBA.TRANSIENT(
                    ExceptionMinorCodes.TRANSIENTCannotReachApplicationAgent,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }
        } catch (Exception e) {
            System.err.println("Couldn't retrieve reference of the ProcessCallback");
            // TODO: Update minor code
            throw new org.omg.CORBA.TRANSIENT(
                ExceptionMinorCodes.TRANSIENTCannotReachApplicationAgent,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
        
        return callback;
    }
    
    
    public static void main(String[] args) {

        int status = SUCCESS;
        org.omg.CORBA.ORB orb = null;
        com.thalesgroup.CdmwPlatformMngt.ProcessCallback callback = null;

        // help argument
        if (args.length >= 1) {
            if ( args[0].equals("--help") ) {
                System.out.println("Usage:\n"
                    + "java cdmw.ccm.cif.test.MainTestTypecheckingTimeout options\n"
                    + "\n"
                    + "Options:\n"
                    + "--help                     Display this information.\n"
                    + "\n");
                return;
            }

            if ( OS.getOptionValue(args, CCMSystemProperties.TYPECHECKING).equals("on") )
                System.setProperty(CCMSystemProperties.TYPECHECKING, "on");
            else
                System.setProperty(CCMSystemProperties.TYPECHECKING, "off");
            
            if ( OS.getOptionValue(args, CCMSystemProperties.TYPECHECKING_TIMEOUT).equals("on") )
                    System.setProperty(CCMSystemProperties.TYPECHECKING_TIMEOUT, "on");
            else
                    System.setProperty(CCMSystemProperties.TYPECHECKING_TIMEOUT, "off");            
        }
        
        String callbackIOR = 
            OS.getOptionValue(args, Options.CALLBACK_IOR_OPTION);
    
        boolean unitTests = true;
        // ---- Is it launched by platform management for integration tests? ----
        if (!callbackIOR.equals("no") && !callbackIOR.equals("yes")) {
            unitTests = false;
        }
    
        // get Name Service port
        String nameServicePort = 
            OS.getOptionValue(args, NAMING_SERVICE_PORT_OPTION);
        
        if (nameServicePort.equals("no"))  {
            nameServicePort = "5000";
        } 
        
        // get Repository port
        String repositoryPort = 
            OS.getOptionValue(args, REPOSITORY_PORT_OPTION);
        
        if (repositoryPort.equals("no")) {
            repositoryPort = "5005";
        } 
    
        int timescale = TestManager.getTimescale();
    
        try {
            StrategyList orbStrategies = new StrategyList();
            orbStrategies.addORBThreaded();
            orbStrategies.addPOAThreadPool(POA_THREAD_POOL_SIZE);
    
          //  String[] serverArgs = {"-ORBconfig", "./obtest.conf"};
        String s = ORBUtils.NAMING_PORT_OPTION + "corbaloc::localhost:"
                       + nameServicePort + "/NameService";
            String[] serverArgs = s.split(" ");
            orb = ORBSupport.ORBInit(serverArgs, orbStrategies);

            // PCR-0049
            cdmw.orbsupport.Codec.init(orb);
    
            if (unitTests) {
                // Starting the Name Service
                System.out.println("Starting Name Service on port " + 
                    nameServicePort + "...");
                
                naming = ORBUtils.startNamingService(nameServicePort);
                OS.sleep(timescale * 3000);
    
                // Starting the Repository Simulator :
                String namingCorbaloc = " -ORBInitRef NameService=corbaloc::localhost:" + 
                    nameServicePort + "/NameService";
                
                String simRepositoryArgs = " -ORBEndpoint iiop://localhost:"+ repositoryPort 
                    + namingCorbaloc 
                    + " -DOM " + Locations.CDMW_SERVICES_NAME_DOMAIN
                    + " -DOM " + Locations.CDMW_EVENT_SERVICE_NAME_DOMAIN
                    + " -DOM CDMW/SERVICES/EVENT/FACTORIES"
                    + " -DOM CDMW/SERVICES/EVENT/EVENT_CHANNELS"
                    + " -DOM CDMW/SERVICES/EVENT/TYPED_EVENT_CHANNELS"
                    + " -ROOT " + Locations.ADMIN_ROOT_CONTEXT_ID;
                
                System.out.println("Starting Simulated Repository with the following arguments:");
                System.out.println(simRepositoryArgs);
                
                repository = OS.createProcess(
                    "cdmw_simulated_repository " + simRepositoryArgs);
                OS.sleep(timescale * 20000);
                System.out.println("Simulated Repository started");
    
                // init repository 
                initRepository(orb);
                OS.sleep(timescale * 3000);
                System.out.println("Simrepository initialised");
    
                // Getting callback IOR
                callback = getProcessCallback(orb);
                callbackIOR = orb.object_to_string(callback);
                
                // Recover the bootclasspath
                String bootclasspath = 
                    OS.getOptionValue(args, "BOOTCLASSPATH=");
                System.out.println("MainTestTypecheckingTimeout: BOOTCLASSPATH=" + bootclasspath);

                System.out.println("Launch SimDaemonAndDance");
                String simDaemonAndDance = "java -Xbootclasspath/p:" + bootclasspath + " cdmw.ccm.cif.test.MainSimDaemonAndDance "
                    + " Cdmw.CCM.TypeChecking=" + System.getProperty(CCMSystemProperties.TYPECHECKING);
                System.out.println("MainTestTypecheckingTimeout: simDaemon command=" + simDaemonAndDance);
              
                daemonAndDance = OS.createProcess(simDaemonAndDance);
                OS.sleep(timescale * 20000);

            }
    
            // Get the root POA 
            //
            org.omg.CORBA.Object obj = 
                orb.resolve_initial_references("RootPOA");
            org.omg.PortableServer.POA rootPOA = 
                org.omg.PortableServer.POAHelper.narrow(obj);
            
            //
            // Run implementation (Activate the POA manager)
            //
            System.out.println("Run implementation (Activate the POA manager)");
            rootPOA.the_POAManager().activate();
     
            OS.sleep(timescale * 1000);

//         // Starting the EventChannelManager
//             String profileManagerService = "cdmw_event_channel_manager";
//             String managerArgs =
//                 " " +
//         " " + Options.CALLBACK_IOR_OPTION + "=" + callbackIOR +
//                 " " + Options.PROCESS_PORT_OPTION + "=15327" +
//                 " " + Options.LOCALISATION_SERVICE_OPTION + "=15328" +
//                 " " + Options.PROCESS_XML_FILE_OPTION + "=./EventChannelManager.xml" +
//                 " --ProfileXMLFile=EventChannelConf1.xml" +
//                 " " + Options.VALIDATE_XML_OPTION;

//             System.out.println("Starting EventChannelManager with the following parameters: ");
//             System.out.println(managerArgs);
//             profileManager = OS.createProcess(
//                 profileManagerService + managerArgs);
//             //ProcessThread pt = new ProcessThread(managerArgs);
//             //pt.start();
//             OS.sleep(timescale * 20000);
//             System.out.println("EventChannelManager started");

      
            // EventChannelManager Proc URL is CDMW Test Application/PROC_000 
//             com.thalesgroup.CdmwPlatformMngt.ProcessDelegate proc = null;
//             String procURL = "corbaname::localhost:" + nameServicePort
//                 + "#CDMW.I/ProcessCallback.simulated/CDMW Test Application/PROC_000";
//             obj = orb.string_to_object(procURL);
        
//             if (obj != null) {
//                 proc = com.thalesgroup.CdmwPlatformMngt.ProcessDelegateHelper.narrow(obj);
//             } else {
//                 System.exit(-1);
//             }
        
            OS.sleep(timescale * 3000);
    
            // declare Value factory to the orb
            org.omg.CORBA.portable.ValueFactory vf;
    
            vf = new cdmw.ccm.cif.FacetDescriptionFactoryImpl();
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory(
                "IDL:omg.org/Components/FacetDescription:1.0", vf);
           
            vf = new cdmw.ccm.cif.PortDescriptionFactoryImpl();
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory(
                "IDL:omg.org/Components/PortDescription:1.0", vf);
            
            vf = new cdmw.ccm.common.CookieFactoryImpl();
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory(
                "IDL:omg.org/Components/Cookie:1.0", vf);
            
            vf = new cdmw.ccm.cif.ConnectionDescriptionFactoryImpl();
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory(
                "IDL:omg.org/Components/ConnectionDescription:1.0", vf);
            
            vf = new cdmw.ccm.cif.ReceptacleDescriptionFactoryImpl();
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory(
                "IDL:omg.org/Components/ReceptacleDescription:1.0", vf);
            
            vf = new cdmw.ccm.cif.ConsumerDescriptionFactoryImpl();
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory(
                "IDL:omg.org/Components/ConsumerDescription:1.0", vf);
            
            vf = new cdmw.ccm.cif.EmitterDescriptionFactoryImpl();
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory(
                "IDL:omg.org/Components/EmitterDescription:1.0", vf);
            
            vf = new cdmw.ccm.cif.SubscriberDescriptionFactoryImpl();
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory(
                "IDL:omg.org/Components/SubscriberDescription:1.0", vf);
            
            vf = new cdmw.ccm.cif.PublisherDescriptionFactoryImpl();
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory(
                "IDL:omg.org/Components/PublisherDescription:1.0", vf);
            
            vf = new cdmw.ccm.cif.ComponentPortDescriptionFactoryImpl();
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory(
                "IDL:omg.org/Components/ComponentPortDescription:1.0", vf);
    
            vf = new cdmw.ccm.container.ConfigValueFactoryImpl();
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory(
                "IDL:omg.org/Components/ConfigValue:1.0", vf);
    
            // Register HomeAllocator
            HomeAllocator allocator = new HomeAllocator(
                "Cdmw.CCM.CIF.CdmwTest.SessionClientHome_impl");
            HomeAllocatorRegistry.register(
                "Cdmw.CCM.CIF.CdmwTest.SessionClientHome_impl",
                allocator);
    
            allocator = new HomeAllocator(
                "Cdmw.CCM.CIF.CdmwTest.SessionServerHome_impl");
            HomeAllocatorRegistry.register(
                "Cdmw.CCM.CIF.CdmwTest.SessionServerHome_impl",
                allocator);
            
            ComponentInstallationPlugImpl compInstallImpl 
                = new ComponentInstallationPlugImpl(rootPOA);
            com.thalesgroup.CdmwDeployment.ComponentInstallation compInstall
                = compInstallImpl._this(orb);
    
            org.omg.Components.ConfigValue[] configValues = 
                new org.omg.Components.ConfigValue[1];
            org.omg.CORBA.Any value = orb.create_any();
            com.thalesgroup.CdmwDeployment.ComponentKindValueHelper.insert(
                value, 
                com.thalesgroup.CdmwDeployment.ComponentKindValue.SESSION);
            configValues[0] = new ConfigValueImpl(
                com.thalesgroup.CdmwDeployment.COMPONENT_KIND.value, value);
    
            org.omg.PortableServer.POA homesSerializePOA = null;
            org.omg.PortableServer.POA homesMultithreadPOA = null;
            org.omg.PortableServer.POA containerPOA = 
                createContainerPOA(rootPOA);
            // Create a SingleThread POA for single threaded homes and 
            // a Multithread POA for multithread homes.
            try {
                org.omg.PortableServer.POAManager poaManager = 
                    containerPOA.the_POAManager();
                org.omg.CORBA.Policy[] policies = new org.omg.CORBA.Policy[7];
                policies[0] = containerPOA.create_id_assignment_policy(
                    org.omg.PortableServer.IdAssignmentPolicyValue.USER_ID);
                policies[1] = containerPOA.create_lifespan_policy(
                    org.omg.PortableServer.LifespanPolicyValue.PERSISTENT);
                policies[2] = containerPOA.create_servant_retention_policy(
                    org.omg.PortableServer.ServantRetentionPolicyValue.RETAIN);
                policies[3] = containerPOA.create_id_uniqueness_policy(
                    org.omg.PortableServer.IdUniquenessPolicyValue.UNIQUE_ID);
                policies[4] = containerPOA.create_request_processing_policy(
                    org.omg.PortableServer.RequestProcessingPolicyValue.USE_SERVANT_MANAGER);
                policies[5] = containerPOA.create_implicit_activation_policy(
                    org.omg.PortableServer.ImplicitActivationPolicyValue.NO_IMPLICIT_ACTIVATION);
                // Create a single thread poa
                policies[6] = containerPOA.create_thread_policy(
                    org.omg.PortableServer.ThreadPolicyValue.SINGLE_THREAD_MODEL);
                
                StrategyList poaStrategies = new StrategyList();
                
                String SINGLE_THREAD_HOMES_POA_NAME = "HomeSerializePOA";
                String MULTI_THREAD_HOMES_POA_NAME  = "HomeMultithreadPOA";
                
                homesSerializePOA = ORBSupport.createPOA(
                    containerPOA,
                    SINGLE_THREAD_HOMES_POA_NAME,
                    poaManager, 
                    policies,
                    poaStrategies);
                //
                // Create and install servant activator
                //
                org.omg.PortableServer.ServantActivator theActivator =
                    new HomesServantActivator();
                homesSerializePOA.set_servant_manager(theActivator);
            
                
                // Create a multi thread poa
                policies[6] = rootPOA.create_thread_policy(
                    org.omg.PortableServer.ThreadPolicyValue.ORB_CTRL_MODEL);
                
                homesMultithreadPOA = ORBSupport.createPOA(
                    containerPOA,
                    MULTI_THREAD_HOMES_POA_NAME,
                    poaManager, 
                    policies,
                    poaStrategies);
                //
                // Create and install servant activator
                //
                theActivator = new HomesServantActivator();
                homesMultithreadPOA.set_servant_manager(theActivator);
                
            } catch (org.omg.PortableServer.POAPackage.AdapterAlreadyExists aae) {
                throw new org.omg.CORBA.INTERNAL(
                    ExceptionMinorCodes.INTERNALLifeCycleFrameworkError,
                    org.omg.CORBA.CompletionStatus.COMPLETED_YES);
            } catch (org.omg.PortableServer.POAPackage.InvalidPolicy ip) {
                throw new org.omg.CORBA.INTERNAL(
                    ExceptionMinorCodes.INTERNALLifeCycleFrameworkError,
                    org.omg.CORBA.CompletionStatus.COMPLETED_YES);
            } catch (org.omg.CORBA.SystemException se) {
                throw se; // rethrow
            } 

                  
            // GET EVENT CHANNEL FACTORY
            
            // Get NamingInterface object from RepositoryInterface
            NamingInterface ni = RepositoryInterface.getDomainNamingInterface(
                Locations.CDMW_SERVICES_NAME_DOMAIN);
        
            // Create a container instance
            System.out.println("Create container");
            ContainerImpl containerImpl = new ContainerImpl(
                orb, 
                containerPOA, 
                homesSerializePOA,
                homesMultithreadPOA,
                compInstall, 
                null,
                null,
                null,
                null/*eventChannelFactory*/,
                PROCESS_NAME,
                APPLICATION_NAME,
                configValues);
            System.out.println("Container created");
            
            org.omg.Components.Deployment.Container container = 
                containerImpl._this(orb);
            String containerIOR = orb.object_to_string(container);
    
            // Create a non Cdmw consumer instance
//             System.out.println("Create a non Cdmw consumer instance");
//             NonCdmwStatusInfoConsumerImpl consumerImpl =
//                 new NonCdmwStatusInfoConsumerImpl(rootPOA);
//             com.acme.Test.StatusInfoConsumer consumer = 
//                 consumerImpl._this(orb);
//             String consumerIOR = orb.object_to_string(consumer);
            
//             // Create a ConsumerBase instance
//             EventConsumerBaseImpl consumerBaseImpl =
//                 new EventConsumerBaseImpl(rootPOA);
//             org.omg.Components.EventConsumerBase consumerBase =
//                 consumerBaseImpl._this(orb);
//             String consumerBaseIOR = orb.object_to_string(consumerBase);

//             OS.sleep(timescale * 1000);
//             System.out.println("server initialised");

            // Main loop
            status = run(orb, containerIOR);
            OS.sleep(timescale * 3000);
            
            if (orb != null) {
                try {
                    RepositoryInterface.finish();
                    OS.sleep(timescale * 2000);
                    orb.destroy();
                } catch(Exception ex) {
                    System.err.println(ex.toString());
                } 
            }        
        
            // And we kill launched process
            try  {
                if (naming != null) { OS.killProcess(naming);}
                if (repository != null) { OS.killProcess(repository);}
                if (daemonAndDance != null) { OS.killProcess(daemonAndDance);}
                OS.sleep(timescale * 2000);
            } catch (InterruptedException ie) {
                System.err.println(ie.toString());
            }
        } catch(Exception ex) {
            ex.printStackTrace();
            if (orb != null) { orb.destroy();}
            if (naming != null) { OS.killProcess(naming);}
            if (repository != null) { OS.killProcess(repository);}
            if (daemonAndDance != null) { OS.killProcess(daemonAndDance);}
            System.err.println(ex.toString());
        }
        
    }

}


