/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.cif.test;

import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import cdmw.ossupport.OS;
import cdmw.testutils.CORBATestManager;
import cdmw.testutils.TestManager;
import cdmw.testutils.CORBATestManager.InvalidObject;
import cdmw.ccm.container.ConfigValueImpl;
import cdmw.ccm.common.CCMSystemProperties;

import com.thalesgroup.CdmwDeployment.ComponentKindValue;
import com.thalesgroup.CdmwDeployment.ComponentKindValueHelper;
import com.thalesgroup.CdmwDeployment.ServantLifetimeValue;
import com.thalesgroup.CdmwDeployment.ServantLifetimeValueHelper;
import com.thalesgroup.CdmwDeployment.ThreadingPolicyValue;
import com.thalesgroup.CdmwDeployment.ThreadingPolicyValueHelper;
import com.thalesgroup.CdmwDeployment.ValuetypeFactoryDescription;
import com.thalesgroup.CdmwDeployment.ValuetypeFactoryDescriptionHelper;
import com.thalesgroup.CdmwDeployment.ValuetypeFactoryDescriptionSeqHelper;
import cdmw.orbsupport.StrategyList;
import cdmw.orbsupport.ORBSupport;
import cdmw.orbsupport.PolicyList;

public class SimDeployment {

    public static final String CLIENT_HOME_REP_ID =
        "IDL:acme.com/Cdmw_Test/ClientHome:1.0";
    public static final String CLIENT_HOME_SERVANT_NAME =
        "Cdmw.CCM.CIF.CdmwTest.SessionClientHome_impl";
    public static final String CLIENT_HOME_UID = "ClientHome_1";
    public static final String CLIENT_HOME_ENTRY_POINT =
        "cdmw.ccm.cif.test.CCMClientHomeImpl.createCCMClientHome";

    public static final String SERVER_HOME_REP_ID =
        "IDL:acme.com/Cdmw_Test/ServerHome:1.0";
    public static final String SERVER_HOME_SERVANT_NAME =
        "Cdmw.CCM.CIF.CdmwTest.SessionServerHome_impl";
    public static final String SERVER_HOME_UID = "ServerHome_1";
    public static final String SERVER_HOME_ENTRY_POINT =
        "cdmw.ccm.cif.test.CCMServerHomeImpl.createCCMServerHome";

    public static final String FOO_FACET_NAME = "the_foo";

    public static final String FOO_RECEP_NAME = "foo_receptacle";
    
    static final String CONTAINER_IOR =
        "cdmw.ccm.cif.test.SimDeployment.CONTAINER_IOR";

    public static final String TIMEOUTTESTER_RECEP_SPECIFIC_NAME = 
        "foo_receptacle";

    private org.omg.CORBA.ORB orb;
    private org.omg.Components.Deployment.Container container;
//     private boolean isTypeCheckingOn;


    @Before
    public void init() throws InvalidObject {
        CORBATestManager testManager = (CORBATestManager)TestManager.getLocalInstance();
        
        this.orb = testManager.getORB();
        try {
            String containerIOR = 
                (String) testManager.getLocalProperty(CONTAINER_IOR);
            org.omg.CORBA.Object obj = this.orb.string_to_object(containerIOR);
            assertTrue(
                "TEST FAILED : exception raised during test initialisation!",
                obj != null);
            this.container =
                org.omg.Components.Deployment.ContainerHelper.narrow(obj);

//             java.lang.String isTypeCheckingOnStr = 
//                 System.getProperty(CCMSystemProperties.TYPECHECKING, "off");
//             isTypeCheckingOn =
//                 (isTypeCheckingOnStr.equals("on"))? true : false;
//             System.out.println("*** " + CCMSystemProperties.TYPECHECKING
//                                + " set to " + isTypeCheckingOnStr + " to test connect() from CCMObjectImpl");
        }

        catch (org.omg.CORBA.SystemException se) {
            fail("TEST FAILED : system exception raised during test initialisation!");
        }
        catch (Exception e) {
            fail("TEST FAILED : unexpected exception raised during test initialisation!");
        }        
    }    
 
    public static org.omg.PortableServer.POA createPOA(
        org.omg.PortableServer.POA parent,
        String POAname) throws Exception {
        //
        // Create a POA with NON_RETAIN and Single-thread policicies
        // to be used to activate the ComponentServer and Container objects.
        // Lifespan policy            = TRANSIENT
        // Id Assignment policy       = USER_ID
        // Id Uniqueness policy       = MULTIPLE_ID
        // Implicit Activation policy = NO_IMPLICIT_ACTIVATION
        // Servant Retention policy   = NON_RETAIN;
        // Thread Policy              = SINGLE_THREAD_MODEL
        // Request processing policy  = USE_SERVANT_MANAGER
        //        

        org.omg.PortableServer.POAManager poaManager = parent.the_POAManager();
        PolicyList policies = new PolicyList();
        policies.insert(
            parent.create_id_assignment_policy(
                org.omg.PortableServer.IdAssignmentPolicyValue.USER_ID));
        policies.insert(
            parent.create_lifespan_policy(
                org.omg.PortableServer.LifespanPolicyValue.TRANSIENT));
        policies.insert(
            parent.create_servant_retention_policy(
                org.omg.PortableServer.ServantRetentionPolicyValue.NON_RETAIN));
        policies.insert(
            parent.create_id_uniqueness_policy(
                org.omg.PortableServer.IdUniquenessPolicyValue.MULTIPLE_ID));
        policies.insert(
            parent.create_request_processing_policy(
                org.omg.PortableServer.RequestProcessingPolicyValue.USE_SERVANT_MANAGER));
        policies.insert(
            parent.create_implicit_activation_policy(
                org.omg.PortableServer.ImplicitActivationPolicyValue.NO_IMPLICIT_ACTIVATION));

        StrategyList poaStrategies = new StrategyList();

        return ORBSupport.createPOA(
            parent,
            POAname,
            poaManager, 
            policies,
            poaStrategies);
    }
   
    @Test
    public void doTests() {

        org.omg.Components.CCMHome clientHome = null;
        org.omg.Components.CCMObject ccmClientComp = null;
        com.acme.Test.Client testClientComp1 = null;

        org.omg.Components.CCMHome serverHome = null;
        com.acme.Test.ServerHome testServerHome = null;
        org.omg.Components.CCMObject ccmServerComp = null;
        com.acme.Test.Server testServerComp = null;

        org.omg.CORBA.Object fooFacetObj1 = null;

        org.omg.Components.Cookie recep_ck = null;

        int timescale = TestManager.getTimescale();

        //-------------------------------------------------

        try {
            System.out.println("Install a ClientHome");
            org.omg.CORBA.Any value = null;
            org.omg.Components.ConfigValue[] configValues;

            try {
                configValues = new org.omg.Components.ConfigValue[5];
                value = this.orb.create_any();
                ComponentKindValueHelper.insert(
                    value,
                    ComponentKindValue.SESSION);
                configValues[0] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.COMPONENT_KIND.value,
                        value);

                value = this.orb.create_any();
                ServantLifetimeValueHelper.insert(
                    value,
                    ServantLifetimeValue.CONTAINER_LIFETIME);
                configValues[1] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.SERVANT_LIFETIME.value,
                        value);

                value = this.orb.create_any();
                ThreadingPolicyValueHelper.insert(
                    value,
                    ThreadingPolicyValue.MULTITHREAD);
                configValues[2] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.THREADING_POLICY.value,
                        value);

                value = this.orb.create_any();
                value.insert_string(CLIENT_HOME_REP_ID);
                configValues[3] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.HOME_REPOSITORY_ID.value,
                        value);

                value = this.orb.create_any();
                value.insert_string(CLIENT_HOME_SERVANT_NAME);
                configValues[4] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.HOME_SERVANT_CLASSNAME.value,
                        value);

                clientHome =
                    this.container.install_home(
                        CLIENT_HOME_UID,
                        CLIENT_HOME_ENTRY_POINT,
                        configValues);
            } catch (org.omg.CORBA.BAD_PARAM bp) {
                fail(bp.toString());
            } catch (org.omg.Components.Deployment.InstallationFailure insf) {
                fail("Unexpected exception raised: Components::Deployment::InstallationFailure");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            } catch (org.omg.CORBA.UserException ue) {
                fail("Unexpected User exception raised: " + ue.toString());
            }

            System.out.println("Install a ServerHome");
            value = null;
            configValues = null;

            try {
                configValues = new org.omg.Components.ConfigValue[5];
                value = this.orb.create_any();
                ComponentKindValueHelper.insert(
                    value,
                    ComponentKindValue.SESSION);
                configValues[0] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.COMPONENT_KIND.value,
                        value);

                value = this.orb.create_any();
                ServantLifetimeValueHelper.insert(
                    value,
                    ServantLifetimeValue.CONTAINER_LIFETIME);
                configValues[1] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.SERVANT_LIFETIME.value,
                        value);

                value = this.orb.create_any();
                ThreadingPolicyValueHelper.insert(
                    value,
                    ThreadingPolicyValue.MULTITHREAD);
                configValues[2] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.THREADING_POLICY.value,
                        value);

                value = this.orb.create_any();
                value.insert_string(CLIENT_HOME_REP_ID);
                configValues[3] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.HOME_REPOSITORY_ID.value,
                        value);

                value = this.orb.create_any();
                value.insert_string(SERVER_HOME_SERVANT_NAME);
                configValues[4] =
                    new ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.HOME_SERVANT_CLASSNAME.value,
                        value);

                serverHome =
                    this.container.install_home(
                        SERVER_HOME_UID,
                        SERVER_HOME_ENTRY_POINT,
                        configValues);
            } catch (org.omg.CORBA.BAD_PARAM bp) {
                fail(bp.toString());
            } catch (org.omg.Components.Deployment.InstallationFailure insf) {
                fail("Unexpected exception raised: Components::Deployment::InstallationFailure");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            } catch (org.omg.CORBA.UserException ue) {
                fail("Unexpected User exception raised: " + ue.toString());
            }

            System.out.println("Create a Client component using create_component() operation");
            try {

                

                org.omg.Components.KeylessCCMHome keylessClientHome =
                    org.omg.Components.KeylessCCMHomeHelper.narrow(
                        clientHome);

                assertTrue(
                    "ERROR: Created home is not a keyless home!",
                    keylessClientHome != null);

                ccmClientComp =
                    keylessClientHome.create_component();

                assertTrue(
                    "ERROR: ccmClientComp is nil!",
                    ccmClientComp != null);

                System.out.println("Create Client component");
                testClientComp1 =
                    com.acme.Test.ClientHelper.narrow(
                        ccmClientComp);

                assertTrue(
                    "ERROR: Created component is not a com.acme.Test.Client!",
                    testClientComp1 != null);

                System.out.println("Client component created");
            } catch (org.omg.Components.CreateFailure cf) {
                fail("Unexpected exception raised: CreateFailure");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Create a Server component using create_component() operation");
            try {
                org.omg.Components.KeylessCCMHome keylessServerHome =
                    org.omg.Components.KeylessCCMHomeHelper.narrow(
                        serverHome);

                assertTrue(
                    "ERROR: Created home is not a keyless home!",
                    keylessServerHome != null);
                ccmServerComp = keylessServerHome.create_component();

                testServerComp = com.acme.Test.ServerHelper.narrow(ccmServerComp);

                assertTrue(
                    "ERROR: Created component is not a com.acme.Test.Server!",
                    testServerComp != null);
            } catch (org.omg.Components.CreateFailure cf) {
                fail("Unexpected exception raised: CreateFailure");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("provide_facet operation on Server...");
            try {
                fooFacetObj1 = testServerComp.provide_facet(FOO_FACET_NAME);

                assertTrue(
                    "ERROR: No facet found for Server component",
                    fooFacetObj1 != null);
            } catch (org.omg.Components.InvalidName ine) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("Connect operation on Client component...");
            try {

                // if U-CMC-1019 set Typechecking to on
                if(System.getProperty("Num.Test").equals("1017")) {
                    System.setProperty(CCMSystemProperties.TYPECHECKING, "on");
                    System.out.println("*** " + CCMSystemProperties.TYPECHECKING
                                       + " set to on to test connect() from CCMObjectImpl"
                                       + " and default " + CCMSystemProperties.TYPECHECKING_TIMEOUT
                                       + " is used.");
                }else if (System.getProperty("Num.Test").equals("1018")){
                    System.setProperty(CCMSystemProperties.TYPECHECKING, "on");  
                    System.setProperty(CCMSystemProperties.TYPECHECKING_TIMEOUT, "1000");  
                    System.out.println("*** " + CCMSystemProperties.TYPECHECKING
                                       + " set to on to test connect() from CCMObjectImpl"
                                       + " and " + CCMSystemProperties.TYPECHECKING_TIMEOUT
                                       + " was set");
                    
                } else {
                    System.setProperty(CCMSystemProperties.TYPECHECKING, "off");  
                    System.out.println("*** " + CCMSystemProperties.TYPECHECKING
                                       + " set to off to test connect() from CCMObjectImpl");
                }

                // fake connection to simulate the problem of the PCR-1877
                fooFacetObj1 = orb.string_to_object("corbaloc::localhost:21869/SimUltimateFallback");
         
                String noConnection = System.getProperty("No.Java.Client.Connection");
                if(noConnection != null && noConnection.equals("yes") ) {
                    Thread.sleep(20 * 1000);
                    return;
                } else {
                    System.out.println("Connect the Client receptacle");
                    recep_ck =
                        testClientComp1.connect(
                                                FOO_RECEP_NAME,
                                                fooFacetObj1);   
                }
                
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.InvalidConnection ic) {
                if (System.getProperty("Num.Test").equals("1017") ||
                    System.getProperty("Num.Test").equals("1018")) {
                    assertTrue(true);
                } else {
                    ic.printStackTrace();
                    fail("Client: Unexpected exception raised!");
                }
            } catch (org.omg.Components.AlreadyConnected ac) {
                fail("Unexpected exception raised: AlreadyConnected");
            } catch (org.omg.Components.ExceededConnectionLimit ecl) {
                fail("Unexpected exception raised: ExceededConnectionLimit");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }

            System.out.println("ConfigurationComplete operation on Client components...");
            try {

                testClientComp1.configuration_complete();

            } catch (org.omg.Components.InvalidConfiguration ic) {
                if (System.getProperty("Num.Test").equals("1017") ||
                    System.getProperty("Num.Test").equals("1018")) {
                    assertTrue(true);
                } else {
                    ic.printStackTrace();
                    fail("Client: Unexpected exception raised!");
                }
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }


            System.out.println("Disconnect operation on Client component...");
            try {
                testClientComp1.disconnect(FOO_RECEP_NAME, null);
                System.out.println("Disconnect operation on Client component done");
            } catch (org.omg.Components.InvalidName in) {
                fail("Unexpected exception raised: InvalidName");
            } catch (org.omg.Components.InvalidConnection ic) {
                if (System.getProperty("Num.Test").equals("1017") ||
                    System.getProperty("Num.Test").equals("1018")) {
                    assertTrue(true);
                } else {
                    ic.printStackTrace();
                    fail("Client: Unexpected exception raised!");
                }
            } catch (org.omg.Components.CookieRequired cr) {
                fail("Unexpected exception raised: CookieRequired");
            } catch (org.omg.Components.NoConnection nc) {
                if (System.getProperty("Num.Test").equals("1017") ||
                    System.getProperty("Num.Test").equals("1018")) {
                    assertTrue(true);
                } else {
                    nc.printStackTrace();
                    fail("Client: Unexpected exception raised: NoConnection");
                }
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }


        } catch (Exception e) {
            e.printStackTrace();
            fail("Unexpected exception raised: " + e.toString());
        }
    }
}
