/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <sstream>
#include <string>
#include <fstream>

#include "testwithdefaulttimeout/TestServer.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"
#include <Foundation/osthreads/Condition.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <Foundation/ossupport/ProcessCallback.hpp>
#include <Foundation/common/Assert.hpp>

#include "CCMContainer/ccmcontainer/ConfigValue_impl.hpp"
#include "testwithdefaulttimeout/TestTimeout_cif.skel.hpp"
#include <Foundation/idllib/CdmwDeployment.skel.hpp>
#include <CCMContainer/idllib/CdmwCcmCif.stub.hpp>

namespace
{
    const char* SERVER_HOME_REP_ID = "IDL:thalesgroup.com/Cdmw_TestTimeout/ServerHome:1.0";
    const char* SERVER_HOME_SERVANT_NAME = "Cdmw.CCM.CIF.CdmwTestTimeout.SessionServerHome_impl";
    const char* SERVER_HOME_UID = "ServerHome_1";
    const char* SERVER_HOME_ENTRY_POINT = "create_CCM_ServerHome";
    const char* SERVER_NAME = "R2D2";

}; // End anonymous namespace

namespace Cdmw
{
namespace CCM
{
namespace CIF
{

/**
*Implementation notes: [if no pertinent write none ]
*<p>
*Portability issues: [if no pertinent write none ]
*<p>
*/


class BlockingProcessCallback : public Cdmw::OsSupport::ProcessCallback {
    
    public:
        BlockingProcessCallback() : m_processEnded(false), m_condition(m_mutex) {};

        virtual ~BlockingProcessCallback() {};
        
        virtual void execute(Cdmw::OsSupport::OS::ProcessId processId) throw() {
            CDMW_MUTEX_GUARD(m_mutex);
            m_processEnded = true;
            try {
                m_condition.broadcast();
            } catch (const Cdmw::InternalErrorException& ex) {
                CDMW_ASSERT(false);
            }
        };

        void wait_callback() throw() {
            CDMW_MUTEX_GUARD(m_mutex);
            try {
                while (!m_processEnded) m_condition.wait();
            } catch (const Cdmw::InternalErrorException& ex) {
                CDMW_ASSERT(false);
            }
        };

    private:
        bool m_processEnded;
        Cdmw::OsSupport::Mutex m_mutex;
        Cdmw::OsSupport::Condition m_condition;
};



CPPUNIT_TEST_SUITE_REGISTRATION( TestServer );
TestServer::TestServer()
   throw(CORBA::SystemException)
{
    m_orb =  CORBA::ORB::_duplicate( Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());
    try
    {
        CORBA::Object_var obj = m_orb->string_to_object(Cdmw::TestUtils::CORBATestManager::instance()->get_property ("containerIOR").c_str());

        if (CORBA::is_nil(obj.in()))
        {
            TEST_INFO("TEST FAILED : exception raised during test initialisation!");
        }
        else
        {
            m_container = Components::Deployment::Container::_narrow(obj.in());
        }
    }
    catch (const CORBA::SystemException & e)
    {
        TEST_INFO("TEST FAILED : system exception raised during test initialisation!");
        throw;
    }
    catch (...)
    {
        TEST_INFO("TEST FAILED : unexpected exception raised during test initialisation!");
    }
}


TestServer::~TestServer()
    throw()
{
    // does nothing
}


TestServer::TestServer(const TestServer& rhs)
    throw()
    :m_orb(CORBA::ORB::_duplicate(rhs.m_orb)),
     m_container(Components::Deployment::Container::_duplicate(rhs.m_container))
{
    // does nothing
}
        
TestServer&
TestServer::operator=(const TestServer& rhs)
    throw()
{
    if (this != &rhs) {
        m_orb = CORBA::ORB::_duplicate(rhs.m_orb.in());
        m_container = Components::Deployment::Container::_duplicate(rhs.m_container.in());
    }
    return *this;
}

void TestServer::do_tests()        
{
  // set number of requested successfull tests
  //     set_nbOfRequestedTestOK (25);
    sleep(5);
    
    bool test_ok=false;
    Components::CCMHome_var server_home;
    TestTimeout::ServerHome_var basic_server_home;
    Components::CCMObject_var ccm_server_comp;
    TestTimeout::Server_var basic_server_comp;

    // facet providing the sleep
    CORBA::Object_var timeouttester_facet_obj;

    //-------------------------------------------------
    TEST_INFO("Testing server component");
    
    try 
    {    
        TEST_INFO("Install a ServerHome");
        test_ok=false;

        try
        {
            Components::ConfigValues config_values(8);
            config_values.length(5);
            CORBA::Any value;
            value <<= CdmwDeployment::SESSION;
            config_values[0] 
                = new Container::ConfigValue_impl(CdmwDeployment::COMPONENT_KIND, 
                                                   value);

            value <<= CdmwDeployment::CONTAINER_LIFETIME;
            config_values[1]
                = new Container::ConfigValue_impl(CdmwDeployment::SERVANT_LIFETIME, 
                                                   value);

            value <<= CdmwDeployment::MULTITHREAD;
            config_values[2]
                = new Container::ConfigValue_impl(CdmwDeployment::THREADING_POLICY, 
                                                   value);

            value <<= SERVER_HOME_REP_ID;
            config_values[3]
                = new Container::ConfigValue_impl(CdmwDeployment::HOME_REPOSITORY_ID, 
                                                   value);

            value <<= SERVER_HOME_SERVANT_NAME;
            config_values[4]
                = new Container::ConfigValue_impl(CdmwDeployment::HOME_SERVANT_CLASSNAME, 
                                                   value);
            server_home 
               = m_container->install_home(SERVER_HOME_UID,
                                           SERVER_HOME_ENTRY_POINT,
                                           config_values);
            test_ok=true;
        }
        catch (const CORBA::BAD_PARAM& e)
        {
            TEST_INFO(e);
        }
        catch (const Components::Deployment::InstallationFailure& e)
        {
            std::ostringstream os;
            os << "Unexpected exception raised: Components::Deployment::InstallationFailure " << e.reason << std::ends;
            TEST_INFO(os.str());
        }
        catch (const CORBA::SystemException& e)
        {
            std::ostringstream os;
            os << "Unexpected system exception raised: " << e << std::ends;
            TEST_INFO(os.str());
        }
        catch (const CORBA::UserException& e)
        {
            std::ostringstream os;
            os << "Unexpected User exception raised: " << e << std::ends;
            TEST_INFO(os.str());
        }
        CPPUNIT_ASSERT(test_ok);

        TEST_INFO("Create a Server component using create() operation");
        test_ok=false;
        try
        {
            basic_server_home = TestTimeout::ServerHome::_narrow(server_home.in());

            if (CORBA::is_nil(basic_server_home.in()))
            {
                TEST_INFO("Home is not a BasicDemo::ServerHome!");
            }
            else
            {

                basic_server_comp = basic_server_home->create();

                if (CORBA::is_nil(basic_server_comp.in()))
                {
                    TEST_INFO("ERROR: basic_server_comp is nil!");
                }
                else
                {
                    test_ok=true;
                }
            }
        }
        catch (const CORBA::SystemException& e)
        {
            std::ostringstream os;
            os << "Unexpected exception raised: " << e << std::ends;
            TEST_INFO(os.str());
            test_ok=false;
        }
        CPPUNIT_ASSERT(test_ok);


        TEST_INFO("tester operation on Server component...");
        test_ok=false;
        try
        {            
            timeouttester_facet_obj
                = basic_server_comp->provide_tester();

            if (CORBA::is_nil(timeouttester_facet_obj.in()))
            {
                TEST_INFO("ERROR: No facet found for Server component");
            }
            else
              {
                // store the facet's ior
                CORBA::String_var ref_string =
                  m_orb->object_to_string(timeouttester_facet_obj.in());
                std::ofstream os("file.ior");
                os << ref_string.in() ;
                os.close();
                
                test_ok=true;
            }

        }
        catch (const Components::InvalidName& e)
        {
            TEST_INFO("Unexpected exception raised: InvalidName");
            test_ok=false;
        }
        catch (const CORBA::SystemException& e)
        {
            std::ostringstream os;
            os << "Unexpected exception raised: " << e << std::ends;
            TEST_INFO(os.str());
            test_ok=false;
        }
        CPPUNIT_ASSERT(test_ok);

        TEST_INFO("Configuration_complete operation on Server component...");
        test_ok=false;
        try
        {
            basic_server_comp->configuration_complete();
            test_ok=true;
        }
        catch (const Components::InvalidConfiguration &)
        {
            TEST_INFO("Unexpected exception raised: InvalidConfiguration");
        }
        catch (const CORBA::SystemException& e)
        {
            std::ostringstream os;
            os << "Unexpected exception raised: " << e << std::ends;
            TEST_INFO(os.str());
        }
        CPPUNIT_ASSERT(test_ok);


        // create a process, the process is a client component with a default
        // timeout.
        TEST_INFO("Create ClientWithDefaultTimeout process");
        BlockingProcessCallback callback;
        Cdmw::OsSupport::OS::ProcessId idClientWithDefaultTimeoutProcess;
        {
          std::string args = " --ProcessName=ClientWithDefaultTimeout";
          idClientWithDefaultTimeoutProcess = 
            Cdmw::OsSupport::OS::create_process("ClientWithDefaultTimeout", 
                                                args, 
                                                "", 
                                                NULL, 
                                                &callback);
        }
        
        TEST_INFO("Wait end of ClientWithDefaultTimeout process");
        callback.wait_callback();
        TEST_INFO("ClientWithDefaultTimeout process ended");


    } 
    catch (const CORBA::Exception & e) 
    {
            std::ostringstream os;
            os << "Unexpected exception raised: " << e << std::ends;
            TEST_INFO(os.str()); 
            CPPUNIT_ASSERT(false);
    }

}
        
}; // End namespace CIF
}; // End namespace CCM
}; // End namespace Cdmw

