/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.cif.test;

import cdmw.ccm.container.ConfigValueImpl;
import cdmw.ossupport.OS;
import cdmw.testutils.CORBATestManager;
import cdmw.testutils.TestManager;

import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

public class TestClientWithDefaultTimeout {

    public static final String CLIENT_HOME_REP_ID = 
        "IDL:thalesgroup.com/Cdmw_TestTimeout/ClientHome:1.0";
    public static final String CLIENT_HOME_SERVANT_NAME = 
        "Cdmw.CCM.CIF.CdmwTestTimeout.SessionClientHome_impl";
    public static final String CLIENT_HOME_UID = 
        "ClientHome_1";
    public static final String CLIENT_HOME_ENTRY_POINT = 
        "cdmw.ccm.cif.test.CCMClientHomeImpl.createCCMClientHome";

    public static final String TIMEOUTTESTER_RECEP_DEFAULT_NAME = 
        "default_receptacle";
    public static final String TIMEOUTTESTER_RECEP_SPECIFIC_NAME = 
        "specific_receptacle";

        
    private org.omg.CORBA.ORB orb;

    private org.omg.Components.Deployment.Container container;

    
    @Before
    public void init() throws Exception {
        CORBATestManager testManager = (CORBATestManager)TestManager.getLocalInstance();

        String containerIOR
            = (String)testManager.getLocalProperty(
                "cdmw.ccm.cif.test.TimeoutTest.CONTAINER_IOR");

        this.orb = testManager.getORB();

        org.omg.CORBA.Object obj = this.orb.string_to_object(containerIOR);

        assertTrue(
            "TEST FAILED : exception raised during test initialisation!",
            obj != null);
        this.container = org.omg.Components.Deployment.ContainerHelper.narrow(obj);

        try {
            OS.sleep(TestManager.getTimescale() * 3000);
        } catch(InterruptedException ie) {}
    }
    
    @Test
    public void doTests() {

        org.omg.Components.CCMHome clientHome = null;
        com.thalesgroup.CdmwCcmCif.KeylessCCMHome keylessClientHome = null;
        org.omg.Components.CCMObject ccmClientComp = null;
        com.acme.TestTimeout.Client basicClientComp = null;
        
        // facet providing the sleep
        org.omg.CORBA.Object timeouttesterFacetObj = null;
        
        /*
         * Config Values used by the context of components
         */
        
        org.omg.Components.ConfigValue[] configValuesHome =
            new org.omg.Components.ConfigValue[1];
        org.omg.CORBA.Any a = orb.create_any();
        
        // Creating of a config value storing a sequence UsesPortsTimeouts sequence. The sequence
        // contains UserPortTimeout structures which store the default timeout and the timeout of
        // the specific_receptacle receptacle.
        com.thalesgroup.CdmwCcmCif.UsesPortTimeout[] usesPortTimeouts =
            new com.thalesgroup.CdmwCcmCif.UsesPortTimeout[2];
        
        com.thalesgroup.CdmwCcmCif.UsesPortTimeout usesDefaultPortTimeout = 
            new com.thalesgroup.CdmwCcmCif.UsesPortTimeout();
        com.thalesgroup.CdmwCcmCif.UsesPortTimeout usesSpecificPortTimeout = 
            new com.thalesgroup.CdmwCcmCif.UsesPortTimeout();
        
        usesDefaultPortTimeout.uses_name = "";
        usesDefaultPortTimeout.timeout = 5000;
        
        usesSpecificPortTimeout.uses_name = TIMEOUTTESTER_RECEP_SPECIFIC_NAME;
        usesSpecificPortTimeout.timeout = 2000;
        
        usesPortTimeouts[0] = usesDefaultPortTimeout;
        usesPortTimeouts[1] = usesSpecificPortTimeout;
        
        com.thalesgroup.CdmwCcmCif.UsesPortTimeoutsHelper.insert(
            a, usesPortTimeouts);
        configValuesHome[0] = new ConfigValueImpl(
            com.thalesgroup.CdmwDeployment.USES_PORT_TIMEOUTS.value,
            a);

            
        //-------------------------------------------------
        System.out.println("Testing client component with default receptacle");
    
        try {
            System.out.println("Install a ClientHome for a component with a default timeout");
    
            try {
                org.omg.Components.ConfigValue[] configValues = 
                    new org.omg.Components.ConfigValue[5];

                org.omg.CORBA.Any value = orb.create_any();
                com.thalesgroup.CdmwDeployment.ComponentKindValueHelper.insert(
                    value, 
                    com.thalesgroup.CdmwDeployment.ComponentKindValue.SESSION);
                configValues[0] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.COMPONENT_KIND.value, 
                    value);

                value = orb.create_any();
                com.thalesgroup.CdmwDeployment.ServantLifetimeValueHelper.insert(
                    value, 
                    com.thalesgroup.CdmwDeployment.ServantLifetimeValue.CONTAINER_LIFETIME);
                configValues[1] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.SERVANT_LIFETIME.value, 
                    value);

                value = orb.create_any();
                com.thalesgroup.CdmwDeployment.ThreadingPolicyValueHelper.insert(
                    value, 
                    com.thalesgroup.CdmwDeployment.ThreadingPolicyValue.MULTITHREAD);
                configValues[2] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.THREADING_POLICY.value, 
                    value);

                value = orb.create_any();
                value.insert_string(CLIENT_HOME_REP_ID);
                configValues[3] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.HOME_REPOSITORY_ID.value, 
                    value);

                value = orb.create_any();
                value.insert_string(CLIENT_HOME_SERVANT_NAME);
                configValues[4] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.HOME_SERVANT_CLASSNAME.value, 
                    value);

                clientHome = this.container.install_home(
                    CLIENT_HOME_UID,
                    CLIENT_HOME_ENTRY_POINT,
                    configValues);
                    
            } catch (org.omg.CORBA.BAD_PARAM bp) {
                fail(bp.toString());
            } catch (org.omg.Components.Deployment.InstallationFailure f) {
                fail("Unexpected exception raised: " + 
                    "Components.Deployment.InstallationFailure " + f.reason);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }
        
            
            System.out.println("Create a Client component using create_component() operation");
            try {
                keylessClientHome =
                    com.thalesgroup.CdmwCcmCif.KeylessCCMHomeHelper.narrow(
                        clientHome);
                assertTrue("Home is not a CdmwCcmCif.KeylessCCMHome!",
                           keylessClientHome != null);

                ccmClientComp = 
                    keylessClientHome.create_component_with_config_values(
                        configValuesHome);
                assertTrue("ERROR: ccmClientComp is nil!", 
                           ccmClientComp != null);
                           
                basicClientComp =
                    com.acme.TestTimeout.ClientHelper.narrow(ccmClientComp);
                assertTrue("ERROR: basicClientComp is nil!", 
                           basicClientComp != null);
                           
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Recover server facet's ior");
            try {
                java.io.BufferedReader in = new java.io.BufferedReader(
                    new java.io.FileReader("file.ior") );
                String ior = in.readLine();
                
                timeouttesterFacetObj = orb.string_to_object(ior);
                
            } catch(Exception ioe) {
                assertTrue("Error reading facet's IOR: " + ioe,
                           false);
            }

            org.omg.Components.Cookie cookieDefault = null;
            System.out.println("Connect operation. Client : default_receptacle -> facet ");
            try
            {
                cookieDefault =
                    basicClientComp.connect(TIMEOUTTESTER_RECEP_DEFAULT_NAME,
                                            timeouttesterFacetObj);
            }
            catch (org.omg.Components.InvalidName e)
            {
                assertTrue("Unexpected exception raised: InvalidName",
                           false);
            }
            catch (org.omg.Components.InvalidConnection e)
            {
                assertTrue("Expected exception raised: InvalidConnection",
                           false);
            }
            catch (org.omg.Components.AlreadyConnected e)
            {
                assertTrue("Unexpected exception raised: AlreadyConnected",
                           false);
            }
            catch (org.omg.Components.ExceededConnectionLimit e)
            {
                assertTrue("Unexpected exception raised: ExceededConnectionLimit",
                           false);
            }
            catch (org.omg.CORBA.SystemException e)
            {
                assertTrue("Unexpected exception raised: " +e,
                           false);
            }
            
            System.out.println("Connect operation. Client : specific_receptacle -> facet");
            org.omg.Components.Cookie cookieSpecific = null;
            try
            {
                cookieSpecific =
                    basicClientComp.connect(TIMEOUTTESTER_RECEP_SPECIFIC_NAME,
                                            timeouttesterFacetObj);
            }
            catch (org.omg.Components.InvalidName e)
            {
                assertTrue("Unexpected exception raised: InvalidName",
                           false);
            }
            catch (org.omg.Components.InvalidConnection e)
            {
                assertTrue("Expected exception raised: InvalidConnection",
                           false);
            }
            catch (org.omg.Components.AlreadyConnected e)
            {
                assertTrue("Unexpected exception raised: AlreadyConnected",
                           false);
            }
            catch (org.omg.Components.ExceededConnectionLimit e)
            {
                assertTrue("Unexpected exception raised: ExceededConnectionLimit",
                           false);
            }
            catch (org.omg.CORBA.SystemException e)
            {
                assertTrue("Unexpected exception raised: " +e,
                           false);
            }
            
            System.out.println("Configuration_complete operation on Client component...");
            try {
                basicClientComp.configuration_complete();
            } catch (org.omg.Components.InvalidConfiguration ic) {
                fail("Unexpected exception raised: InvalidConfiguration");
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }
            
            System.out.println("Disconnect default receptacle of Client component...");
            try
            {
                basicClientComp.disconnect(TIMEOUTTESTER_RECEP_DEFAULT_NAME, 
                                           cookieDefault);
            }
            catch (org.omg.Components.InvalidName e)
            {
                assertTrue("Unexpected exception raised: InvalidName",
                           false);
            }
            catch (org.omg.Components.InvalidConnection e)
            {
                assertTrue("Unexpected exception raised: InvalidConnection",
                           false);
            }
            catch (org.omg.Components.CookieRequired e)
            {
                assertTrue("Unexpected exception raised: CookieRequired",
                           false);
            }
            catch (org.omg.Components.NoConnection e)
            {
                assertTrue("Unexpected exception raised: NoConnection",
                           false);
            }
            catch (org.omg.CORBA.SystemException e)
            {
                assertTrue("Unexpected exception raised: " + e,
                           false);
            }
            
            System.out.println("Disconnect specific receptacle of Client component...");
            try
            {
                basicClientComp.disconnect(TIMEOUTTESTER_RECEP_SPECIFIC_NAME, 
                                           cookieSpecific);
            }
            catch (org.omg.Components.InvalidName e)
            {
                assertTrue("Unexpected exception raised: InvalidName",
                           false);
            }
            catch (org.omg.Components.InvalidConnection e)
            {
                assertTrue("Unexpected exception raised: InvalidConnection",
                           false);
            }
            catch (org.omg.Components.CookieRequired e)
            {
                assertTrue("Unexpected exception raised: CookieRequired",
                           false);
            }
            catch (org.omg.Components.NoConnection e)
            {
                assertTrue("Unexpected exception raised: NoConnection",
                           false);
            }
            catch (org.omg.CORBA.SystemException e)
            {
                assertTrue("Unexpected exception raised: " + e,
                           false);
            }
            
            System.out.println("Remove operation on the Client home...");
            try
            {
                container.remove_home(clientHome);
            }
            catch (org.omg.Components.RemoveFailure e)
            {
                assertTrue("Unexpected exception raised: NoKeyAvailable",
                           false);
            }
            catch (org.omg.CORBA.SystemException e)
            {
                assertTrue("Unexpected exception raised: " + e,
                           false);
            }

        } catch (Exception e)  {
            e.printStackTrace();
            fail("Unexpected exception raised: " + e.toString());
        }
        
    }
}

