/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "testwithoutdefaulttimeout/CCM_Client_impl.hpp"

#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/common/Assert.hpp"
#include "CCMContainer/ccmcommon/CCMUtils.hpp"

#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/ui/text/TestRunner.h>
#include <cppunit/TestResult.h>
#include <cppunit/XmlOutputter.h>
#include <cppunit/TextOutputter.h>
#include <fstream>

namespace 
{

  /**
   *  This class provides an object to test the ccm_activate.
   *
   **/

  class test_ccm_client_activate;
    
  CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(test_ccm_client_activate, "test_ccm_client_activate");

  class test_ccm_client_activate : public CppUnit::TestFixture
  {
    TestTimeout::CCM_Client_impl* m_client;
  
  public:
    test_ccm_client_activate()
    {
    };


    ~test_ccm_client_activate() {};
  
    void do_tests()
    {

      TEST_INFO("********************************************");
      TEST_INFO("            ccm_activate start");
      TEST_INFO("********************************************");
     

      ::TestTimeout::TimeoutTester_var server_facet;


      CORBA::Object_var obj1 = Cdmw::TestUtils::CORBATestManager::instance()->get_object("client_context");
      ::TestTimeout::CCM_Client_Context_var client_context = ::TestTimeout::CCM_Client_Context::_narrow(obj1.in());
      
      CPPUNIT_ASSERT(!CORBA::is_nil(client_context.in()));

      TEST_INFO("Get_connection_without_default_receptacle...");
      
      bool test_ok = false;
      try
        {
          server_facet = client_context->get_connection_without_default_receptacle();

          if (!CORBA::is_nil(server_facet.in()))
            {
              test_ok = true;
            }
        }
      catch (...)
        {
          TEST_INFO("Unexpected exception raised!");
        }
      CPPUNIT_ASSERT(test_ok);

      //get the connection to the server 
      TEST_INFO("Connect to the TimeoutTester facet without the default timeout ...");

      test_ok = false;
      try
        {
          if (!CORBA::is_nil(server_facet.in()))
            {
              server_facet->sleep(10);
              TEST_INFO("sleep operation finished");
              test_ok = true;
            }
        }
      catch(CORBA::TIMEOUT)
        {
            TEST_INFO("CORBA::TIMEOUT: exception raised . "
                      << "Expected exception raised!");
        }
      catch (...)
        {
          TEST_INFO("Unexpected exception raised!");
        }
      CPPUNIT_ASSERT(test_ok);

      /*
       * Test of the specific receptacle
       */

      TEST_INFO("Get_connection_specifc_receptacle...");
      test_ok = false;
      try
        {
          server_facet = client_context->get_connection_specific_receptacle();

          if (!CORBA::is_nil(server_facet.in()))
            {
              test_ok = true;
            }
        }
       catch (...)
        {
          TEST_INFO("Unexpected exception raised!");
        }
      CPPUNIT_ASSERT(test_ok);

      //get the connection to the server 
      TEST_INFO("Connect to the TimeoutTester facet with the specific"
                << " receptacle...");
      test_ok = false;
      try
        {
          if (!CORBA::is_nil(server_facet.in()))
            {
              server_facet->sleep(10);
            }
        }
       catch(CORBA::TIMEOUT)
        {
          TEST_INFO("CORBA::TIMEOUT: Expected exception raised!");
          test_ok = true;
        }
      catch (...)
        {
          TEST_INFO("Unexpected exception raised!");
        }
      CPPUNIT_ASSERT(test_ok);

      TEST_INFO("********************************************");
      TEST_INFO("            ccm_activate stop");
      TEST_INFO("********************************************");
      
    };

      CPPUNIT_TEST_SUITE( test_ccm_client_activate );
      CPPUNIT_TEST( do_tests );
      CPPUNIT_TEST_SUITE_END();

  }; // End of class test_ccm_client_activate
    
} // End of anonymous namespace


//
// IDL:thalesgroup.com/TestTimeout/CCM_Client:1.0
//
TestTimeout::CCM_Client_impl::CCM_Client_impl()
  : m_session_context(TestTimeout::CCM_Client_Context::_nil())
{
}

TestTimeout::CCM_Client_impl::~CCM_Client_impl()
{
  std::cout << "CCM_Client_impl::~CCM_Client_impl called!" << std::endl;
}

//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
TestTimeout::CCM_Client_impl::set_session_context(Components::SessionContext_ptr ctx)
  throw(CORBA::SystemException)
{
  std::cout << "Set_session_context called!" << std::endl;
   
  m_session_context = ::TestTimeout::CCM_Client_Context::_narrow(ctx); 
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
TestTimeout::CCM_Client_impl::ccm_activate()
  throw(CORBA::SystemException)
{

  // get_context() performs a duplicate
  ::TestTimeout::CCM_Client_Context_var ctx = get_context();
  Cdmw::TestUtils::CORBATestManager::instance()->add("client_context", ctx.in());

  Cdmw::TestUtils::ThreadedTestRunner runner(Cdmw::TestUtils::CORBATestManager::instance()->get_controller());
  runner.addTest( CppUnit::TestFactoryRegistry::getRegistry("test_ccm_client_activate").makeTest() );
  runner.start();
  runner.join();

  sleep(5);
    
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//

void
TestTimeout::CCM_Client_impl::ccm_passivate()
  throw(CORBA::SystemException)
{
  std::cout << "Ccm_passivate called!" << std::endl;
} 



//
// ccm_remove function : it destroys the component (call by the container)
void
TestTimeout::CCM_Client_impl::ccm_remove()
  throw(CORBA::SystemException)
{
  std::cout << "Ccm_remove called!" << std::endl;
}
