/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.cif.test;

import cdmw.ccm.container.ConfigValueImpl;
import cdmw.ossupport.OS;
import cdmw.testutils.CORBATestManager;
import cdmw.testutils.TestManager;

import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

public class TestServer {

    public static final String SERVER_HOME_REP_ID = 
        "IDL:thalesgroup.com/Cdmw_TestTimeout/ServerHome:1.0";
    public static final String SERVER_HOME_SERVANT_NAME = 
        "Cdmw.CCM.CIF.CdmwTestTimeout.SessionServerHome_impl";
    public static final String SERVER_HOME_UID = 
        "ServerHome_1";
    public static final String SERVER_HOME_ENTRY_POINT = 
        "cdmw.ccm.cif.test.CCMServerHomeImpl.createCCMServerHome";
    public static final String SERVER_NAME = 
        "R2D2";

        
    private org.omg.CORBA.ORB orb;

    private org.omg.Components.Deployment.Container container;

    
    @Before
    public void init() throws Exception {
        CORBATestManager testManager = (CORBATestManager)TestManager.getLocalInstance();

        String containerIOR =
            (String)testManager.getLocalProperty(
                "cdmw.ccm.cif.test.TimeoutTest.CONTAINER_IOR");

        this.orb = testManager.getORB();

        org.omg.CORBA.Object obj = this.orb.string_to_object(containerIOR);
        assertTrue("TEST FAILED : exception raised during test initialisation!",
                   obj != null);
                   
        this.container = 
            org.omg.Components.Deployment.ContainerHelper.narrow(obj);

        try {
            OS.sleep(TestManager.getTimescale() * 3000);
        } catch(InterruptedException ie) {}
    }
    
    @Test
    public void doTests() {
        
        org.omg.Components.CCMHome serverHome = null;
        com.acme.TestTimeout.ServerHome basicServerHome = null;
        org.omg.Components.CCMObject ccmServerComp = null;
        com.acme.TestTimeout.Server basicServerComp = null;
    
        // facet providing the sleep
        org.omg.CORBA.Object timeouttesterFacetObj = null;
        
        //-------------------------------------------------
        System.out.println("Testing server component");
    
        try {
            System.out.println("Installing a ServerHome...");
    
            try {
                org.omg.Components.ConfigValue[] configValues = 
                    new org.omg.Components.ConfigValue[5];

                org.omg.CORBA.Any value = orb.create_any();
                com.thalesgroup.CdmwDeployment.ComponentKindValueHelper.insert(
                    value, 
                    com.thalesgroup.CdmwDeployment.ComponentKindValue.SESSION);
                configValues[0] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.COMPONENT_KIND.value, 
                    value);

                value = orb.create_any();
                com.thalesgroup.CdmwDeployment.ServantLifetimeValueHelper.insert(
                    value, 
                    com.thalesgroup.CdmwDeployment.ServantLifetimeValue.CONTAINER_LIFETIME);
                configValues[1] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.SERVANT_LIFETIME.value, 
                    value);

                value = orb.create_any();
                com.thalesgroup.CdmwDeployment.ThreadingPolicyValueHelper.insert(
                    value, 
                    com.thalesgroup.CdmwDeployment.ThreadingPolicyValue.MULTITHREAD);
                configValues[2] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.THREADING_POLICY.value, 
                    value);

                value = orb.create_any();
                value.insert_string(SERVER_HOME_REP_ID);
                configValues[3] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.HOME_REPOSITORY_ID.value, 
                    value);

                value = orb.create_any();
                value.insert_string(SERVER_HOME_SERVANT_NAME);
                configValues[4] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.HOME_SERVANT_CLASSNAME.value, 
                    value);

                serverHome = this.container.install_home(
                    SERVER_HOME_UID,
                    SERVER_HOME_ENTRY_POINT,
                    configValues);
                    
            } catch (org.omg.CORBA.BAD_PARAM bp) {
                fail(bp.toString());
            } catch (org.omg.Components.Deployment.InstallationFailure f) {
                fail("Unexpected exception raised: " + 
                    "Components::Deployment::InstallationFailure " + f.reason);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected system exception raised: " + se.toString());
            }
        
            
            System.out.println("Create a Server component using create() operation");
            try {
                basicServerHome =
                    com.acme.TestTimeout.ServerHomeHelper.narrow(serverHome);
                assertTrue("Home is not a TestTimeout.ServerHome!",
                           basicServerHome != null);

                basicServerComp = basicServerHome.create();
                assertTrue("ERROR: basicServerComp is nil!", 
                           basicServerComp != null);
                           
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("tester operation on Server component...");
            try {
                timeouttesterFacetObj =
                    basicServerComp.provide_tester();
                assertTrue("ERROR: No facet found for Server component", 
                           timeouttesterFacetObj != null);
                           
                // store the facet's ior
                String ior = orb.object_to_string(timeouttesterFacetObj);
                java.io.File file = new java.io.File("file.ior");
                try {
                    java.io.Writer out = new java.io.FileWriter(file);
                    out.write(ior);
                    out.close();
                } catch(java.io.IOException e) { 
                    assertTrue("Error writing facet's IOR: "+e,
                               false);
                }
                
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se.toString());
            }

            System.out.println("Configuration_complete operation on Server component...");
            try {
                basicServerComp.configuration_complete();
            } catch (org.omg.Components.InvalidConfiguration ic) {
                fail("Unxpected exception raised: InvalidConfiguration: " + ic);
            } catch (org.omg.CORBA.SystemException se) {
                fail("Unexpected exception raised: " + se);
            }


            // create Client process
            cdmw.ossupport.ThreadedProcess clientp = null;
            String classname = "cdmw.ccm.cif.test.MainClientWithoutDefaultTimeout";
            try {
                clientp = OS.createJavaProcess(classname);
                // wait Client process to end
                clientp.getProcess().waitFor();
            } catch(java.io.IOException ioe) {
                throw new InterruptedException("Unable to create client process.");
            }    

        } catch (Exception e)  {
            fail("Unexpected exception raised: " + e.toString());
        }
            
    }
}

