/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef _CDMW_CCM_LOGGERS_HPP_
#define _CDMW_CCM_LOGGERS_HPP_

#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <Foundation/common/Assert.hpp>

namespace Cdmw {
namespace CCM {

    /**
     * Utility class to be used by CARDAMOM only.
     * Returns different loggers for CCM cscis usage.
     */
    class CCMLoggers
    {
        public:
        
        /**
         * CCM Domain for logs in ccmcommon
         */
        static const char* CCMCOMMON_LOG_DOMAIN;

        /**
         * CCM Domain for logs in ccmcif
         */
        static const char* CCMCIF_LOG_DOMAIN;

        /**
         * CCM Domain for logs in ccmcontainer
         */
        static const char* CCMCONTAINER_LOG_DOMAIN;

        /**
         * CCM Domain for logs in ccmcomponentserver
         */
        static const char* CCMCOMPONENTSERVER_LOG_DOMAIN;

        /**
         * CCM Domain for logs in ccm generated code
         */
        static const char* CCMGENERATED_LOG_DOMAIN;

        
        /**
         * Get Logger for ccmcommon
         *
         * @return the Logger to be used in ccmcommon
         */
        inline static Cdmw::Logging::Logger * Get_ccmcommon_logger() throw () 
        {
            static Cdmw::Logging::Logger * Logger__ = 
                Cdmw::Logging::Logger::Get_Logger(CCMCOMMON_LOG_DOMAIN);
            CDMW_ASSERT(Logger__ != 0);
            return Logger__;
        }
        
        /**
         * Get Logger for ccmcif
         *
         * @return the Logger to be used in ccmcif 
         */
        inline static Cdmw::Logging::Logger * Get_ccmcif_logger() throw () 
        {
            static Cdmw::Logging::Logger * Logger__ = 
                Cdmw::Logging::Logger::Get_Logger(CCMCIF_LOG_DOMAIN);
            CDMW_ASSERT(Logger__ != 0);
            return Logger__;
        }
        
        /**
         * Get Logger for ccmcontainer
         *
         * @return the Logger to be used in ccmcontainer
         */
        inline static Cdmw::Logging::Logger * Get_ccmcontainer_logger() throw () 
        {
            static Cdmw::Logging::Logger * Logger__ = 
                Cdmw::Logging::Logger::Get_Logger(CCMCONTAINER_LOG_DOMAIN);
            CDMW_ASSERT(Logger__ != 0);
            return Logger__;
        }
        
        /**
         * Get Logger for ccmcomponentserver
         *
         * @return the Logger to be used in ccmcomponentserver 
         */
        inline static Cdmw::Logging::Logger * Get_ccmcomponentserver_logger() throw () 
        {
            static Cdmw::Logging::Logger * Logger__ = 
                Cdmw::Logging::Logger::Get_Logger(CCMCOMPONENTSERVER_LOG_DOMAIN);
            CDMW_ASSERT(Logger__ != 0);
            return Logger__;
        }
        
        /**
         * Get Logger for ccm generated code
         *
         * @return the Logger to be used in ccm generated code 
         */
        inline static Cdmw::Logging::Logger * Get_ccmgenerated_logger() throw () 
        {
            static Cdmw::Logging::Logger * Logger__ = 
                Cdmw::Logging::Logger::Get_Logger(CCMGENERATED_LOG_DOMAIN);
            CDMW_ASSERT(Logger__ != 0);
            return Logger__;
        }
        
        
        private:
            CCMLoggers();
            ~CCMLoggers();
        
    };    
    

    enum EventLogLevel
    {
        CCM_EVENT_LEVEL = Cdmw::Logging::CDMW_DEBUG_4

    };
}; // namespace CCM
}; // namespace Cdmw

#include <string>
#include <ostream>
#include <sstream>

#ifndef CDMW_TRACE_LEVEL 
#define CDMW_TRACE_LEVEL 6
#endif

//Macros for event's timestamps logging
#if (CDMW_TRACE_LEVEL >= 5 )
#define CDMW_DEBUG_EVENT(logger, msg)                                                \
        if (logger->is_enabled_for(::Cdmw::CCM::CCM_EVENT_LEVEL)) {                  \
            std::ostringstream __log_oss__;                                          \
            __log_oss__ << msg;                                                      \
            logger->log(::Cdmw::CCM::CCM_EVENT_LEVEL, __log_oss__.str().c_str(), __FILE__, __LINE__);\
        }
#else
#define CDMW_DEBUG_EVENT(logger, msg)
#endif

#endif // _CDMW_CCM_LOGGERS_HPP_

