/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_CCM_COMMON_CCMUTILS_HPP
#define INCL_CCM_COMMON_CCMUTILS_HPP

#include <string>
#include <iostream>


// Cdmw Files
#include <Foundation/common/Exception.hpp>
#include <Foundation/orbsupport/CORBA.hpp>

#include <Foundation/idllib/Components.skel.hpp>

// Cdmw CCM Files

namespace Cdmw {

namespace CCM {

namespace Common {


    /**
     *
     */
    unsigned long create_identifier() throw(std::bad_alloc);

    /**
     *
     * prefix_#
     *
     */
    std::string create_unique_identifier(const std::string & prefix, int fill_width = 0)
        throw(std::bad_alloc);

    /**
     * Conversion between OctetSeq and string
     */ 
    CORBA::OctetSeq *
        string_to_OctetSeq(const char* str);
    
    char*
        OctetSeq_to_string(const CORBA::OctetSeq & seq) throw (CORBA::SystemException);
    
    /**
     * Purpose:
     * <p>Read a config value
     *
     *@param name Name of the configuration data
     *@param config A sequence of all configuration values
     *@param value Out CORBA::Any parameter for holding the configuration value
     *
     *@return <B>true</B> if the configration value is found, <B>false</B> 
     * otherwise.
     */
    bool read_config_value(const std::string & name,
                                const Components::ConfigValues & config,
                                CORBA::Any_out value)
    throw();



} // End namespace Common

} // End namespace CCM

} // End namespce Cdmw


#endif  // INCL_CCM_COMMON_CCMUTILS_HPP

