/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.common;

import cdmw.logging.Logger;


/**
 * Utility class to be used by CARDAMOM only.
 * Returns different loggers for CCM cscis usage.
 */
public class CCMLoggers
{
    
        /**
         * CCM Domain for logs in ccmcommon
         */
        public static final String CCMCOMMON_LOG_DOMAIN =
            "CDMW.CCM.ccmcommon";

        /**
         * CCM Domain for logs in ccmcif
         */
        public static final String CCMCIF_LOG_DOMAIN =
            "CDMW.CCM.ccmcif";

        /**
         * CCM Domain for logs in ccmcontainer
         */
        public static final String CCMCONTAINER_LOG_DOMAIN =
            "CDMW.CCM.ccmcontainer";

        /**
         * CCM Domain for logs in ccmcomponentserver
         */
        public static final String CCMCOMPONENTSERVER_LOG_DOMAIN =
            "CDMW.CCM.componentserver";

        /**
         * CCM Domain for logs in ccm generated code
         */
        public static final String CCMGENERATED_LOG_DOMAIN =
            "CDMW.CCM.componentserver";

        
        private static Logger CcmcommonLogger = null;
        private static Logger CcmcifLogger = null;
        private static Logger CcmcontainerLogger = null;
        private static Logger CcmcomponentserverLogger = null;
        private static Logger CcmgeneratedLogger = null;
            

        /**
         * Get Logger for ccmcommon
         *
         * @return the Logger to be used in ccmcommon
         */
        public static Logger GetCcmcommonLogger()
        {
            if (CcmcommonLogger == null)
                CcmcommonLogger = Logger.Get_Logger(CCMCOMMON_LOG_DOMAIN);
            return CcmcommonLogger;
        }
        
        /**
         * Get Logger for ccmcif
         *
         * @return the Logger to be used in ccmcif 
         */
        public static Logger GetCcmcifLogger()
        {
            if (CcmcifLogger == null)
                CcmcifLogger = Logger.Get_Logger(CCMCIF_LOG_DOMAIN);
            return CcmcifLogger;
        }
        
        /**
         * Get Logger for ccmcontainer
         *
         * @return the Logger to be used in ccmcontainer
         */
        public static Logger GetCcmcontainerLogger() 
        {
            if (CcmcontainerLogger == null)
                CcmcontainerLogger =  Logger.Get_Logger(CCMCONTAINER_LOG_DOMAIN);
            return CcmcontainerLogger;
        }
        
        /**
         * Get Logger for ccmcomponentserver
         *
         * @return the Logger to be used in ccmcomponentserver 
         */
        public static Logger GetCcmcomponentserverLogger()
        {
            if (CcmcomponentserverLogger == null)
                CcmcomponentserverLogger = 
                    Logger.Get_Logger(CCMCOMPONENTSERVER_LOG_DOMAIN);
            return CcmcomponentserverLogger;
        }
        
        /**
         * Get Logger for ccm generated code
         *
         * @return the Logger to be used in ccm generated code
         */
        public static Logger GetCcmgeneratedLogger()
        {
            if (CcmgeneratedLogger == null)
                CcmgeneratedLogger = Logger.Get_Logger(CCMGENERATED_LOG_DOMAIN);
            return CcmgeneratedLogger;
        }

        //CCM timestamped events logging        
        public static final long CCM_EVENT_LOG_LEVEL  = cdmw.logging.LogLevel.CDMW_DEBUG_4;

        public static final cdmw.logging.LevelType CCM_EVENT_LEVEL  = 
            new cdmw.logging.LevelType(CCM_EVENT_LOG_LEVEL);
        
        static public void CDMW_DEBUG_EVENT(Logger logger, String msg) {
            if (logger.isEnabledFor(CCM_EVENT_LEVEL)) {
                logger.log(CCM_EVENT_LEVEL, msg, 0+1);
            }
        }
        //
        // Same macro but using com.thalesgroup.CdmwLogging.Logger IDL
        //
        static public void CDMW_DEBUG_EVENT(com.thalesgroup.CdmwLogging.Logger logger, 
                                    String msg)
        {
            cdmw.logging.Logger.CDMW_LOG(logger, CCM_EVENT_LEVEL, msg, 0+1);
        }
        
}

