/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.common;

import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;


public class CCMUtils {

    // Encoding to use converting OIDs to String
    private final static String ENCODING = "ISO-8859-1";
    
    public final static int MAX_COUNTER = Integer.MAX_VALUE;

    private static int counter = 0;

    public synchronized static int createIdentifier() 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcommonLogger(), "");
        if (counter == MAX_COUNTER) {
            counter = 0;
        } else {
            counter++;
        }
        CDMW_LOG_FUNCTION_RETURN(CCMLoggers.GetCcmcommonLogger(), counter);
        return counter;
    }

    public static String createUniqueIdentifier(
        String prefix, int fillWidth) 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcommonLogger(), 
                          "prefix="+prefix+"  fillWidth="+fillWidth);
        int counter = createIdentifier();
        String formattedCounter = String.valueOf(counter);
        int lg = formattedCounter.length();
        if (lg > fillWidth) {
            formattedCounter = formattedCounter.substring(lg - fillWidth, lg);
        } else {
            int diff = fillWidth - lg;
            StringBuffer countBuffer = new StringBuffer();
            while (diff-- > 0) countBuffer.append('0');
            countBuffer.append(formattedCounter);
            formattedCounter = countBuffer.toString();
        }
        String result = prefix + "_" + formattedCounter;
        CDMW_LOG_FUNCTION_RETURN(CCMLoggers.GetCcmcommonLogger(), result);
        return result;
    }

    
    /**
     * Convert an OID into a String using a fixed encoding.
     *
     * @param oid the OID
     * @return the OID as a String
     */
    public static String OidToString(byte[] oid) 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcommonLogger(), "");
        String result = null;
        try {
            result = new String(oid, ENCODING);
        } catch (java.io.UnsupportedEncodingException e) {
            CDMW_WARN(CCMLoggers.GetCcmcommonLogger(),
                "WARNING: UnsupportedEncodingException for "+ ENCODING,
                new Throwable());
            e.printStackTrace();
            // try default encoding
            result = new String(oid);
        }
        CDMW_LOG_FUNCTION_RETURN(CCMLoggers.GetCcmcommonLogger(), result);
        return result;
    }
    
    /**
     * Convert a String into an OID using a fixed encoding.
     *
     * @param oid the OID as a String
     * @return the OID
     */
    public static byte[] StringToOid(String oidStr) 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcommonLogger(), "oidStr="+oidStr);
        try {
            return oidStr.getBytes(ENCODING);
        } catch (java.io.UnsupportedEncodingException e) {
            CDMW_WARN(CCMLoggers.GetCcmcommonLogger(),
                "WARNING: UnsupportedEncodingException for "+ ENCODING,
                new Throwable());
            e.printStackTrace();
            // try default encoding
            return oidStr.getBytes();
        }
    }

    /**
     * Purpose:
     * <p>Read a config value
     *
     *@param name Name of the configuration data
     *@param config A sequence of all configuration values
     *
     *@return An Any object holding the configuration value, 
     * and null if the configuration value is not found.
     */
    public static org.omg.CORBA.Any ReadConfigValue(
            final java.lang.String name,
            final org.omg.Components.ConfigValue[] config
            )
        throws org.omg.CORBA.SystemException
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "name=" + name);
        org.omg.CORBA.Any result = null;
        for (org.omg.Components.ConfigValue c : config) {
            if (name.equals(c.name)) {
                result = c.value;
                break;
            }
        }
        CDMW_LOG_FUNCTION_RETURN(CCMLoggers.GetCcmcommonLogger(), result);
        return result;
    }

    /**
     * Purpose:
     * <p>Read a config value
     *
     *@param name Name of the configuration data
     *@param config A sequence of all configuration values
     *@param value An Any object holding the configuration value
     *
     *@return false if not found
     */
    public static boolean ReadConfigValue(
            String name,
            org.omg.Components.ConfigValue[] config,
            org.omg.CORBA.AnyHolder value
            )
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "name=" + name);
        boolean found = false;
        for (int count = 0; count < config.length; ++count)
        {
            if (name.equals(config[count].name)) 
            {
                value.value = config[count].value;
                found = true;
                break;
            }
        }

        CDMW_LOG_FUNCTION_RETURN(CCMLoggers.GetCcmcommonLogger(), found);
        return found;
    }
}
