/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.componentserver.test;

import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import cdmw.ccm.container.ConfigValueImpl;
import cdmw.ccm.componentserver.ComponentServerImpl;
import cdmw.testutils.CORBATestManager;
import cdmw.testutils.TestManager;
import cdmw.testutils.CORBATestManager.InvalidObject;

import com.thalesgroup.CdmwDeployment.ComponentKindValue;
import com.thalesgroup.CdmwDeployment.ComponentKindValueHelper;

public class ComponentServerTest {

    private org.omg.CORBA.ORB orb;

    private org.omg.PortableServer.POA poa;

    @Before
    public void init() throws InvalidObject {
        CORBATestManager testManager = (CORBATestManager)TestManager.getLocalInstance();
        
        this.orb = testManager.getORB();
        this.poa = testManager.getPOA();
    }

    private org.omg.Components.ConfigValue[]
        createConfigValues(ComponentKindValue componentKind) {

        int nbValues = 1;
        org.omg.Components.ConfigValue[] cfg =
            new org.omg.Components.ConfigValue[nbValues];
        int idx = 0;

        // COMPONENT_KIND
        org.omg.CORBA.Any value = orb.create_any();
        ComponentKindValueHelper.insert(value, componentKind);
        cfg[idx++] = new ConfigValueImpl(
            com.thalesgroup.CdmwDeployment.COMPONENT_KIND.value, value);

        return cfg;
    }

    @Test
    public void doTests() {
        
        System.out.println("Testing Component Server");
        org.omg.Components.Deployment.ComponentServer theComponentServer = null;

        // 1) Create a ComponentServer
        // Create a set of ConfigValues.
        org.omg.Components.ConfigValue[] cfgCompserver =
            new org.omg.Components.ConfigValue[0];
        com.thalesgroup.CdmwEvent.EventChannelFactory noECF = null;
        String processName = "DummyProcessName";
        String applicationName = "DummyApplicationName";
        org.omg.Components.HomeRegistration homeRegistration = null;
        com.thalesgroup.CdmwLifeCycle.ObjectRegistration objectRegistration = null;
        ComponentServerImpl cs = new ComponentServerImpl(
            this.orb,
            this.poa,
            homeRegistration,
            objectRegistration,
            noECF,
            processName,
            applicationName,
            cfgCompserver);
        org.omg.PortableServer.Servant serv = cs;
        theComponentServer = cs._this(this.orb);

        // Testing theComponentServer.get_containers()
        System.out.println("Getting all containers...");
        org.omg.Components.Deployment.Container[] containers =
            theComponentServer.get_containers();
        assertTrue(containers.length == 0);

        // 2) Creating a first container
        org.omg.Components.Deployment.Container container1 = null;
        try {
            System.out.println("Creating a first session container ...");
            org.omg.Components.ConfigValue[] cfg0 =
                createConfigValues(ComponentKindValue.SESSION);
            container1 = theComponentServer.create_container(cfg0);
            assertTrue(container1 != null);
            System.out.println("Checking used config values...");
            System.out.println("Checking number of config values...");
            org.omg.Components.ConfigValue[] cfg1 = container1.configuration();
            assertTrue(cfg1.length == 1);
            System.out.println("Checking content of config values...");
            String name = cfg1[0].name;
            assertTrue(name.equals(com.thalesgroup.CdmwDeployment.COMPONENT_KIND.value));
            org.omg.CORBA.Any value = cfg1[0].value;
            ComponentKindValue componentKind =
                ComponentKindValueHelper.extract(value);
            assertTrue(componentKind.value() == ComponentKindValue.SESSION.value());
        } catch (Exception ex) {
            fail("ex = " + ex.toString());
        }

        org.omg.Components.Deployment.Container container2 = null;
        try {
            System.out.println("Creating a second session container ...");
            org.omg.Components.ConfigValue[] cfg0 =
                createConfigValues(ComponentKindValue.SERVICE);
            container2 = theComponentServer.create_container(cfg0);
            assertTrue(container2 != null);
            System.out.println("Checking used config values...");
            System.out.println("Checking number of config values...");
            org.omg.Components.ConfigValue[] cfg1 = container2.configuration();
            assertTrue(cfg1.length == 1);
            System.out.println("Checking content of config values...");
            String name = cfg1[0].name;
            assertTrue(name.equals(com.thalesgroup.CdmwDeployment.COMPONENT_KIND.value));
            org.omg.CORBA.Any value = cfg1[0].value;
            ComponentKindValue componentKind =
                ComponentKindValueHelper.extract(value);
            assertTrue(componentKind.value() == ComponentKindValue.SERVICE.value());
        } catch (Exception ex) {
            fail("ex = " + ex.toString());
        }

        // Creating an Entity container (NOT supported yet).
        try {
            System.out.println("Creating an Entity container (NOT supported yet) ...");
            org.omg.Components.ConfigValue[] cfg0 =
                createConfigValues(ComponentKindValue.PROCESS);
            org.omg.Components.Deployment.Container container =
                theComponentServer.create_container(cfg0);
            fail();
        } catch (Exception ex) {
            // Expected behaviour
            System.out.println("ex = " + ex.toString());
        }

        // Testing theComponentServer.get_containers()
        System.out.println("Getting all containers...");
        containers = theComponentServer.get_containers();
        assertTrue(containers.length == 2);

        // 3) Removing first container
        System.out.println("Removing first container...");
        try {
            theComponentServer.remove_container(container1);
        } catch (Exception ex) {
            fail("ex = " + ex.toString());
        }
        // Testing theComponentServer.get_containers()
        System.out.println("Getting all containers...");
        containers = theComponentServer.get_containers();
        assertTrue(containers.length == 1);

        System.out.println("Removing first container again...");
        try {
            theComponentServer.remove_container(container1);
            fail();
        } catch (Exception ex) {
            // Expected behaviour
            System.out.println("ex = " + ex.toString());
        }

        // 4) Testing supported interfaces...
        System.out.println("Narrowing to FactoryBase object...");
        com.thalesgroup.CdmwLifeCycle.FactoryBase fact
            = com.thalesgroup.CdmwLifeCycle.FactoryBaseHelper.narrow(
                theComponentServer);
        assertTrue(fact != null);
        System.out.println("Testing supported interfaces...");
        try {
            String[] seq = fact.get_supported_objects();
            assertTrue(seq.length == 3);
            System.out.println("Supported objects:");
            for (int i=0; i < seq.length; ++i) {
                System.out.println("\t" + seq[i]);
            }
        } catch (Exception ex) {
            fail("ex = " + ex.toString());
        }
        System.out.println("Testing support of IDL:thalesgroup.com/CdmwCcmContainer/Container:1.0 ...");
        try {
            String repId = "IDL:thalesgroup.com/CdmwCcmContainer/Container:1.0";
            boolean res = fact.supports(repId);
            assertTrue(res == true);
        } catch (Exception ex) {
            fail("ex = " + ex.toString());
        }
        System.out.println("Testing support of IDL:omg.org/Components/Deployment/Container:1.0 ...");
        try {
            String repId = "IDL:omg.org/Components/Deployment/Container:1.0";
            assertTrue(fact.supports(repId));
        } catch (Exception ex) {
            fail("ex = " + ex.toString());
        }
        System.out.println("Testing support of IDL:thalesgroup.com/CdmwLifeCycle/FactoryBase:1.0 ...");
        try {
            String repId = "IDL:thalesgroup.com/CdmwLifeCycle/FactoryBase:1.0";
            assertTrue(fact.supports(repId));
        } catch (Exception ex) {
            fail("ex = " + ex.toString());
        }
        System.out.println("Testing support of IDL:dummy:1.0 ...");
        try {
            String repId = "IDL:dummy:1.0";
            assertTrue(!fact.supports(repId));
        } catch (Exception ex) {
            fail("ex = " + ex.toString());
        }

        // 5) Removing the component server
        System.out.println("Removing the component server...");
        try {
            theComponentServer.remove();
        } catch (Exception ex) {
            fail("ex = " + ex.toString());
        }
    }
}

