/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "ccmcontainer/Session2Context_impl.hpp"

#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>
#include <CCMContainer/ccmcommon/CCMUtils.hpp>
#include <sstream>
#include <iomanip>

#include "ccmcommon/CCMSystemProperties.hpp"


namespace
{
    
    // **********************************************************************
    // A SERVANT ACTIVATOR
    // **********************************************************************
    class TheServantActivator : 
        public virtual PortableServer::ServantActivator,
        public virtual Cdmw::OrbSupport::RefCountLocalObject
    {
    public:
        
        TheServantActivator(Cdmw::LifeCycle::ServantProviderBase * provider)
            : m_servant_provider(provider)
        {
            CDMW_LOG_FUNCTION(Cdmw::CCM::CCMLoggers::Get_ccmcontainer_logger(), "");
            CDMW_ASSERT(provider != 0);
            PortableServer::Servant servant 
                = dynamic_cast<PortableServer::Servant> (provider);
            CDMW_ASSERT(servant != 0);
            
            servant->_add_ref();
        }
        ~TheServantActivator()
        {
            CDMW_LOG_FUNCTION(Cdmw::CCM::CCMLoggers::Get_ccmcontainer_logger(), "");
            // NOTA: There is no need for a servant provider allocator as this
            // also a home allocator.
            PortableServer::Servant servant 
                = dynamic_cast<PortableServer::Servant> (m_servant_provider);
            CDMW_ASSERT(servant != 0);
            
            servant->_remove_ref();
            
            m_servant_provider = 0;
        }
        
        virtual PortableServer::Servant 
        incarnate(const PortableServer::ObjectId& oid, 
                  PortableServer::POA_ptr         poa)
            throw(PortableServer::ForwardRequest, CORBA::SystemException)
        {
            CDMW_LOG_FUNCTION(Cdmw::CCM::CCMLoggers::Get_ccmcontainer_logger(), "");
            CDMW_ASSERT(m_servant_provider != 0);             
            return m_servant_provider->lookup_servant(oid);
        }
        
        virtual void 
        etherealize(const PortableServer::ObjectId& oid,
                    PortableServer::POA_ptr poa,
                    PortableServer::Servant servant,
                    CORBA::Boolean cleanup_in_progress,
                    CORBA::Boolean remaining_activations)
            throw(CORBA::SystemException)
        {
            CDMW_LOG_FUNCTION(Cdmw::CCM::CCMLoggers::Get_ccmcontainer_logger(), "");
            CDMW_ASSERT(m_servant_provider != 0);
            
            if(!remaining_activations)
            {
                m_servant_provider -> release_servant (oid, poa, servant);
            }
        }
      
    private:        
        Cdmw::LifeCycle::ServantProviderBase * m_servant_provider;
    };
    // **********************************************************************
    // A SERVANT LOCATOR
    // **********************************************************************

    class TheServantLocator : 
            public virtual PortableServer::ServantLocator,
            public virtual Cdmw::OrbSupport::RefCountLocalObject
    {
    public:

        TheServantLocator(Cdmw::LifeCycle::ServantProviderBase * provider)
            : m_servant_provider(provider)
        {
            CDMW_LOG_FUNCTION(Cdmw::CCM::CCMLoggers::Get_ccmcontainer_logger(), "");
            CDMW_ASSERT(provider != 0);
            PortableServer::Servant servant 
                = dynamic_cast<PortableServer::Servant> (provider);
            CDMW_ASSERT(servant != 0);
            
            servant->_add_ref();
        }
        ~TheServantLocator()
        {
            CDMW_LOG_FUNCTION(Cdmw::CCM::CCMLoggers::Get_ccmcontainer_logger(), "");
            // NOTA: There is no need for a servant provider allocator as this
            // also a home allocator.
            PortableServer::Servant servant 
                = dynamic_cast<PortableServer::Servant> (m_servant_provider);
            CDMW_ASSERT(servant != 0);
            
            servant->_remove_ref();
            
            m_servant_provider = 0;
        }

        PortableServer::Servant 
        preinvoke(const PortableServer::ObjectId& oid,
                  PortableServer::POA_ptr poa,
                  const char* operation,
                  PortableServer::ServantLocator::Cookie&
                 the_cookie)
            throw(CORBA::SystemException)
        {
            CDMW_LOG_FUNCTION(Cdmw::CCM::CCMLoggers::Get_ccmcontainer_logger(), "");
            CDMW_ASSERT(m_servant_provider != 0);              
            return m_servant_provider->lookup_servant(oid);
        }
        void 
        postinvoke(const PortableServer::ObjectId& oid,
                   PortableServer::POA_ptr poa,
                   const char* operation,
                   PortableServer::ServantLocator::Cookie the_cookie,
                   PortableServer::Servant the_servant)
            throw(CORBA::SystemException)
        {
            CDMW_LOG_FUNCTION(Cdmw::CCM::CCMLoggers::Get_ccmcontainer_logger(), "");
            CDMW_ASSERT(m_servant_provider != 0);
            m_servant_provider -> release_servant (oid, poa, the_servant);
        }
        private:        
            Cdmw::LifeCycle::ServantProviderBase * m_servant_provider;
    }; // End TheServantLocator

    /**
     *
     */
/*    std::string
    create_port_identifier(const char* name)
        throw(std::bad_alloc)
    {
        try {
            const unsigned long MAX_COUNTER = 0xFFFFFFFF;
            static Cdmw::OsSupport::Mutex count_mutex;
            static unsigned long counter=0L;
            CDMW_MUTEX_GUARD(count_mutex);
            
            if (counter == MAX_COUNTER)
                counter = 0L;
            else
                counter++;
            
            std::stringstream ostr;
            ostr << name << "_" << std::setw(3) << std::setfill('0') << counter;
            
            return ostr.str();
        } catch (...) {
            throw std::bad_alloc();
        }
    }
*/
    /**
     * Creates an Event Channel object through the CDMW's Event Channel Factory.
     */
    ::CosEventChannelAdmin::EventChannel_ptr
    create_event_channel(::CdmwEvent::EventChannelFactory_ptr evt_factory,
                         const std::string &                   process_name,
                         const std::string &                   application_name,
                         ::Components::PortDescription*        port)
        throw (::Components::CreateFailure,CORBA::SystemException)
    {
        CDMW_LOG_FUNCTION(Cdmw::CCM::CCMLoggers::Get_ccmcontainer_logger(), 
            "process_name="<<process_name<<"  application_name="<<application_name);
        CDMW_ASSERT(!CORBA::is_nil(evt_factory));
        CDMW_ASSERT(port != 0);
        
        std::string key;
        try {
            std::string port_id = Cdmw::CCM::Common::create_unique_identifier(port->name(),3);
            
            std::ostringstream ostr;
            ostr << port_id << "@" << process_name << "@" << application_name;
        
            std::string kind("EventChannel");
            
            CosNaming::Name name(1);
            name.length(1);
            name[0].id   = ostr.str().c_str();
            name[0].kind = kind.c_str();
            
            using namespace Cdmw::CommonSvcs::Naming;
            key = NamingInterface::to_string(name);

            CDMW_DEBUG(Cdmw::CCM::CCMLoggers::Get_ccmcontainer_logger(), 
                "Create event channel: " << key);

            ::CosEventChannelAdmin::EventChannel_var ec
                  = evt_factory->create_event_channel(key.c_str());

            return ec._retn();
        } catch (const std::bad_alloc & ex) {
            CDMW_FATAL(Cdmw::CCM::CCMLoggers::Get_ccmcontainer_logger(), 
                "Not enough memory");
            CORBA::NO_MEMORY ex(Cdmw::OrbSupport::NO_MEMORY,
                                CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        } catch (const CdmwLifeCycle::AlreadyExistant & ex) {
            CDMW_ERROR(Cdmw::CCM::CCMLoggers::Get_ccmcontainer_logger(), 
                "Event channel '" << key << "' already exists");
            ::Components::CreateFailure ex(CdmwDeployment::EVENT_CHANNEL_ALREADY_EXISTS);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        } catch (const CdmwLifeCycle::InvalidName & ex) {
            CDMW_ERROR(Cdmw::CCM::CCMLoggers::Get_ccmcontainer_logger(), 
                "Event channel name '" << key << "' is invalid");
            CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNALLifeCycleFrameworkError,
                               CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        } catch (const CdmwEvent::CreationError & ex) {
            CDMW_ERROR(Cdmw::CCM::CCMLoggers::Get_ccmcontainer_logger(), 
                "Error creating event channel '" << key << "' : " << ex);
            ::Components::CreateFailure ex(CdmwDeployment::CREATE_FAILURE);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        } catch (const CORBA::SystemException & ex) {
            CDMW_ERROR(Cdmw::CCM::CCMLoggers::Get_ccmcontainer_logger(), 
                "CORBA System Exception creating event channel '" << key << "' : " <<
                Cdmw::OrbSupport::OrbSupport::exception_to_string(ex));
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw;
        }
    
    }

    
    // Operation computing server_id
    std::string compute_server_id(const std::string& hostname,
                                  const std::string& application_name,
                                  const std::string& process_name)
    {
        CosNaming::Name name(3);
        name.length(3);
        name[0].id = hostname.c_str();
        name[1].id = application_name.c_str();
        name[2].id = process_name.c_str();
        
        return Cdmw::CommonSvcs::Naming::NamingInterface::to_string(name);
    }
    
    
    
}; // End of anonymous namespace



namespace Cdmw
{

namespace CCM
{
namespace Container
{

    
Session2Context_impl::Session2Context_impl(CORBA::ORB_ptr                            orb,
                                           PortableServer::POA_ptr                   parent,
                                           Components::CCMHome_ptr                   home,
                                           const std::string &                       home_id,
                                           ComponentServantProvider*                 servant_provider,
                                           CdmwDeployment::ServantLifetimeValue     servant_lifetime,
                                           CdmwDeployment::ThreadingPolicyValue     threading_policy,
                                           Components::HomeRegistration_ptr          home_registration,
                                           CdmwEvent::EventChannelFactory_ptr       event_channel_factory,
                                           const std::string &                       process_name,
                                           const std::string &                       application_name,
                                           const ValueTypeDependencyList &           valuetype_dependencies)
    throw (CORBA::SystemException)
    : m_orb(CORBA::ORB::_duplicate(orb)),
      m_current(PortableServer::Current::_nil()),
      m_poa(PortableServer::POA::_nil()),
      m_home(Components::CCMHome::_duplicate(home)),
      m_home_registration(Components::HomeRegistration::_duplicate(home_registration)),
      m_servant_lifetime(servant_lifetime),
      m_event_channel_factory(CdmwEvent::EventChannelFactory::_duplicate(event_channel_factory)),
      m_process_name(process_name),
      m_application_name(application_name),
      m_valuetype_dependencies(valuetype_dependencies)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcontainer_logger(), 
        "home_id="<<home_id<<"  process_name="<<process_name<<
        "  application_name="<<application_name);
    CDMW_ASSERT(!CORBA::is_nil(orb));
    
    m_hostname = Cdmw::OsSupport::OS::get_hostname().c_str();

    try {
        CORBA::Object_var obj =
            orb->resolve_initial_references("POACurrent");
        m_current = PortableServer::Current::_narrow(obj.in());
        CDMW_DEBUG(CCMLoggers::Get_ccmcontainer_logger(), 
            "Session2Context_impl::Session2Context_impl() created");
        
        // A Context object is created at home installation. This will setup a whole lifecycle
        // framework for components and components' facets managed by this home.
        //
        // A lifecycle framework contains the following actors:
        //   A - *) An object factory ( the home created by the container)
        //   B - *) An object adapter
        //   C - *) A servant manager
        //   D - *) A servant provider
        //
        // A) Create the object adapter
        // POA policies depend on the component_kind, lifetime_policy, and threading_policy
        
        using namespace Cdmw::OrbSupport;        
        PortableServer::POAManager_var poaManager = parent -> the_POAManager();
        CORBA::PolicyList policies;
        policies.length(7);
        policies[0] =
            parent -> create_id_assignment_policy(PortableServer::USER_ID);

        // By default the created POA will be PERSISTENT
        // It will be transient if SESSION_CONTAINER_PERSISTENT_REF_PROPERTY_NAME
        // System Property is "no"
        std::string persistent =
            Cdmw::OsSupport::SystemProperties::Get_Property(
                Cdmw::CCM::SESSION_CONTAINER_PERSISTENT_REF_PROPERTY_NAME,
                "yes");
        if (persistent == "no")
        {
            policies[1] =
                parent -> create_lifespan_policy(PortableServer::TRANSIENT);
        }
        else
        {
            policies[1] =
                parent -> create_lifespan_policy(PortableServer::PERSISTENT);
        }

        if ((servant_lifetime == CdmwDeployment::COMPONENT_LIFETIME) ||
            (servant_lifetime == CdmwDeployment::CONTAINER_LIFETIME)) {
            policies[2] =
                parent -> create_servant_retention_policy(PortableServer::RETAIN);
        } else if (servant_lifetime == CdmwDeployment::METHOD_LIFETIME) {
            policies[2] =
                parent -> create_servant_retention_policy(PortableServer::NON_RETAIN);
        } else {
            // TRANSACTION_LIFETIME servant lifetime policy is not supported!
            CDMW_FATAL(CCMLoggers::Get_ccmcontainer_logger(), 
                "TRANSACTION_LIFETIME servant lifetime policy is not supported");
            CORBA::NO_IMPLEMENT ex(NO_IMPLEMENTTransactionsNotSupported,
                                   CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
        policies[3] =
            parent -> create_id_uniqueness_policy(PortableServer::UNIQUE_ID);
        policies[4] = parent ->
            create_request_processing_policy(PortableServer::USE_SERVANT_MANAGER);
        policies[5] = parent ->
            create_implicit_activation_policy(PortableServer::NO_IMPLICIT_ACTIVATION);
        if (threading_policy == CdmwDeployment::SERIALIZE) {
            policies[6] = parent ->
                create_thread_policy(PortableServer::SINGLE_THREAD_MODEL);
        } else {
            policies[6] = parent ->
                create_thread_policy(PortableServer::ORB_CTRL_MODEL);
        }
        
        StrategyList poaStrategies;
        
        m_poa = Cdmw::OrbSupport::OrbSupport::create_POA(parent,
                                                         home_id.c_str(),
                                                         poaManager.in(), 
                                                         policies,
                                                         poaStrategies);
        //
        // Create and install servant activator
        //
        {
            PortableServer::ServantManager_var the_servant_manager;
            if (servant_lifetime == CdmwDeployment::METHOD_LIFETIME)
                the_servant_manager = new TheServantLocator(servant_provider);
            else 
                the_servant_manager = new TheServantActivator(servant_provider);
            m_poa -> set_servant_manager(the_servant_manager.in());
        }
    } catch (const CORBA::Exception & ex) {
        // This is to avoid Assertion failure in debug mode as internal ref count
        // equals 1 (CORBA C++ mapping oblige!)
        _set_ref_count(0);
        CDMW_ERROR(CCMLoggers::Get_ccmcontainer_logger(), 
            "CORBA Exception received : " << 
            Cdmw::OrbSupport::OrbSupport::exception_to_string(ex));
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    } catch (...) {
        // This is to avoid Assertion failure in debug mode as internal ref count
        // equals 1 (CORBA C++ mapping oblige!)
        _set_ref_count(0);
        CDMW_ERROR(CCMLoggers::Get_ccmcontainer_logger(), 
            "Unknown exception received");
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL,CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    
}

Session2Context_impl::~Session2Context_impl()
    throw ()
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcontainer_logger(), "");
    // Delete context's POA

    try {
        m_valuetype_dependencies.clear();
        m_poa->destroy(true,false);
        m_poa = PortableServer::POA::_nil();
    } catch (const CORBA::Exception & ex) {
        CDMW_WARN(CCMLoggers::Get_ccmcontainer_logger(), 
            "CORBA Exception received in Session2Context_impl destructor: " << 
            Cdmw::OrbSupport::OrbSupport::exception_to_string(ex));
    }
}

CORBA::Policy_ptr 
Session2Context_impl::create_policy ( CORBA::PolicyType type,
                      const CORBA::Any & value )
  throw (CORBA::SystemException, CORBA::PolicyError)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcontainer_logger(), "");
    return m_orb->create_policy(type,value);
}

PortableServer::ObjectId*
Session2Context_impl::create_ObjectId()
    throw (std::bad_alloc)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcontainer_logger(), "");
    try {
        unsigned long counter = Cdmw::CCM::Common::create_identifier();
        
        std::stringstream ostr;
        ostr << counter;
        
        return PortableServer::string_to_ObjectId(ostr.str().c_str());
    } catch (...) {
        CDMW_ERROR(CCMLoggers::Get_ccmcontainer_logger(), 
            "Unknown exception received");
        std::bad_alloc ex;
        CDMW_LOG_FUNCTION_EXCEPTION(ex.what());
        throw ex;
    }
}


//--- From CCMContext -----------------------------------------------//
//
// IDL:omg.org/Components/CCMContext/get_caller_principal:1.0
//
Components::Principal_ptr
Session2Context_impl::get_caller_principal()
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcontainer_logger(), "");
    CDMW_ERROR(CCMLoggers::Get_ccmcontainer_logger(), 
        "CCMContext::get_caller_principal() NOT IMPLEMENTED");
    CORBA::NO_IMPLEMENT ex(Cdmw::OrbSupport::NO_IMPLEMENTSecurityNotSupported,
                           CORBA::COMPLETED_NO);
    CDMW_LOG_FUNCTION_EXCEPTION(ex);
    throw ex;
}
        
//
// IDL:omg.org/Components/CCMContext/get_CCM_home:1.0
//
Components::CCMHome_ptr
Session2Context_impl::get_CCM_home()
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcontainer_logger(), "");
    return Components::CCMHome::_duplicate(m_home.in());
}

//
// IDL:omg.org/Components/CCMContext/get_rollback_only:1.0
//
CORBA::Boolean
Session2Context_impl::get_rollback_only() //raises (IllegalState);
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcontainer_logger(), "");
    CDMW_ERROR(CCMLoggers::Get_ccmcontainer_logger(), 
        "CCMContext::get_rollback_only() NOT IMPLEMENTED");
    CORBA::NO_IMPLEMENT ex(Cdmw::OrbSupport::NO_IMPLEMENTTransactionsNotSupported,
                           CORBA::COMPLETED_NO);
    CDMW_LOG_FUNCTION_EXCEPTION(ex);
    throw ex;
}

//
// IDL:omg.org/Components/CCMContext/get_user_transaction:1.0
//
Components::Transaction::UserTransaction_ptr
Session2Context_impl::get_user_transaction() // raises (IllegalState);
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcontainer_logger(), "");
    CDMW_ERROR(CCMLoggers::Get_ccmcontainer_logger(), 
        "CCMContext::get_user_transaction() NOT IMPLEMENTED");
    CORBA::NO_IMPLEMENT ex(Cdmw::OrbSupport::NO_IMPLEMENTTransactionsNotSupported,
                           CORBA::COMPLETED_NO);
    CDMW_LOG_FUNCTION_EXCEPTION(ex);
    throw ex;
}
        
//
// IDL:omg.org/Components/CCMContext/is_caller_in_role:1.0
//
CORBA::Boolean
Session2Context_impl::is_caller_in_role(const char* role)
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcontainer_logger(), "");
    CDMW_ERROR(CCMLoggers::Get_ccmcontainer_logger(), 
        "CCMContext::is_caller_in_role() NOT IMPLEMENTED");
    CORBA::NO_IMPLEMENT ex(Cdmw::OrbSupport::NO_IMPLEMENTSecurityNotSupported,
                           CORBA::COMPLETED_NO);
    CDMW_LOG_FUNCTION_EXCEPTION(ex);
    throw ex;
}
        
//
// IDL:omg.org/Components/CCMContext/set_rollback_only:1.0
//
void
Session2Context_impl::Session2Context_impl::set_rollback_only() // raises (IllegalState);
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcontainer_logger(), "");
    CDMW_ERROR(CCMLoggers::Get_ccmcontainer_logger(), 
        "CCMContext::set_rollback_only() NOT IMPLEMENTED");
    CORBA::NO_IMPLEMENT ex(Cdmw::OrbSupport::NO_IMPLEMENTTransactionsNotSupported,
                           CORBA::COMPLETED_NO);
    CDMW_LOG_FUNCTION_EXCEPTION(ex);
    throw ex;
}

//
// IDL:omg.org/Components/CCMContext/resolve_service_reference:1.0
//
CORBA::Object_ptr 
Session2Context_impl::resolve_service_reference(const char* service_id) 
    throw(Components::CCMException,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcontainer_logger(), "");
    CDMW_ERROR(CCMLoggers::Get_ccmcontainer_logger(), 
        "CCMContext::resolve_service_reference() NOT IMPLEMENTED");
    CORBA::NO_IMPLEMENT ex(Cdmw::OrbSupport::NO_IMPLEMENT,
                           CORBA::COMPLETED_NO);
    CDMW_LOG_FUNCTION_EXCEPTION(ex);
    throw ex;
}


//--- From CCM2Context ----------------------------------------------//
//
// IDL:omg.org/Components/CCM2Context/get_home_registration:1.0
//
Components::HomeRegistration_ptr
Session2Context_impl::get_home_registration()
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcontainer_logger(), "");
    return Components::HomeRegistration::_duplicate(m_home_registration.in());
}

//
// IDL:omg.org/Components/CCM2Context/req_passivate:1.0
//
void
Session2Context_impl::req_passivate()
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcontainer_logger(), "");
    if ((m_servant_lifetime != CdmwDeployment::COMPONENT_LIFETIME) &&
        (m_servant_lifetime != CdmwDeployment::CONTAINER_LIFETIME))
    {
        CDMW_ERROR(CCMLoggers::Get_ccmcontainer_logger(), 
            "CCM2Context::req_passivate() available only with COMPONENT_LIFETIME or CONTAINER_LIFETIME");
        Components::PolicyMismatch ex;
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }

    try {
        PortableServer::POA_var poa = m_current->get_POA();
        PortableServer::ObjectId_var oid = m_current->get_object_id();
        
        poa->deactivate_object(oid.in());
    } catch (const PortableServer::Current::NoContext & ) {
        CDMW_ERROR(CCMLoggers::Get_ccmcontainer_logger(), 
            "PortableServer::Current::NoContext exception received");
        Components::PolicyMismatch ex;
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
        // TODO: Raise an issue in the components FTF.
        // throw Component::IllegalState();
    } catch (const PortableServer::POA::ObjectNotActive & ) {
        // should not happen!
        CDMW_ERROR(CCMLoggers::Get_ccmcontainer_logger(), 
            "PortableServer::POA::ObjectNotActive exception received");
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL,CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch  (const PortableServer::POA::WrongPolicy & ) {
        CDMW_ERROR(CCMLoggers::Get_ccmcontainer_logger(), 
            "PortableServer::POA::WrongPolicy exception received");
        Components::PolicyMismatch ex;
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const CORBA::SystemException & e) {
        CDMW_ERROR(CCMLoggers::Get_ccmcontainer_logger(), 
            "SystemException exception received: " <<
            Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL,CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}
        
//
// IDL:omg.org/Components/CCM2Context/get_persistence:1.0
//
Components::CatalogBase_ptr
Session2Context_impl::get_persistence(const char* catalog_type_id)
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcontainer_logger(), "");
    CDMW_ERROR(CCMLoggers::Get_ccmcontainer_logger(), 
        "Persistance not available");
    Components::PersistenceNotAvailable ex;
    CDMW_LOG_FUNCTION_EXCEPTION(ex);
    throw ex;
}

//--- From SessionContext -------------------------------------------//
//
// IDL:omg.org/Components/SessionContext/get_CCM_object:1.0
//
CORBA::Object_ptr
Session2Context_impl::get_CCM_object()
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcontainer_logger(), "");
    //
    // CAUTION: Potential problem in using POA::Current (found in Orbacus/C++ 4.1.0)
    //          Following a narrow within a Home_impl object, the ORB used the same
    //          thread for the is_a() message and the POA::Current return the POA
    //          used for the initial invocation (home operation) and the object id
    //          of the home.
    try {
        PortableServer::POA_var poa = m_current->get_POA();
        PortableServer::ObjectId_var oid = m_current->get_object_id();
        
        return poa->id_to_reference(oid.in());
        
    } catch (const PortableServer::Current::NoContext & ) {
        CDMW_ERROR(CCMLoggers::Get_ccmcontainer_logger(), 
            "Cannot get CCMObject: not in a CORBA request context");
        Components::IllegalState ex;
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const PortableServer::POA::ObjectNotActive & ) {
        // should not happen!
        CDMW_ERROR(CCMLoggers::Get_ccmcontainer_logger(), 
            "PortableServer::POA::ObjectNotActive received");
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL,CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch  (const PortableServer::POA::WrongPolicy & ) {
        // TODO/FIXME: Check for NON_RETAIN servant retention policy
        // should not happen!
        CDMW_ERROR(CCMLoggers::Get_ccmcontainer_logger(), 
            "PortableServer::POA::WrongPolicy received");
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL,CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const CORBA::SystemException & e) {
        CDMW_ERROR(CCMLoggers::Get_ccmcontainer_logger(), 
            "SystemException exception received: " <<
            Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL,CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}
        
//--- From Session2Context ------------------------------------------//
//
// IDL:omg.org/Components/Session2Context/create_ref:1.0
//
CORBA::Object_ptr
Session2Context_impl::create_ref(const char* repid)
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcontainer_logger(), 
        "repid="<<repid);
    try {
        PortableServer::ObjectId_var oid = create_ObjectId();
        
        return m_poa->create_reference_with_id(oid.in(),repid);
    } catch (const std::bad_alloc & ) {
        CDMW_FATAL(CCMLoggers::Get_ccmcontainer_logger(), 
            "Not enough memory");
        CORBA::NO_MEMORY ex(Cdmw::OrbSupport::INTERNAL,CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}
        
//
// IDL:omg.org/Components/Session2Context/create_ref_from_oid:1.0
//
CORBA::Object_ptr
Session2Context_impl::create_ref_from_oid(const ::CORBA::OctetSeq& oid,
                                          const char* repid)
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcontainer_logger(), 
        "repid="<<repid);
    PortableServer::ObjectId_var oid_ =
        new PortableServer::ObjectId(oid.maximum(),
                                     oid.length(),
                                     const_cast<CORBA::Octet*>(oid.get_buffer()),
                                     false);
    return m_poa->create_reference_with_id(oid_.in(),repid);
}
        
//
// IDL:omg.org/Components/Session2Context/get_oid_from_ref:1.0
//
CORBA::OctetSeq*
Session2Context_impl::get_oid_from_ref(CORBA::Object_ptr objref)
    throw (Components::IllegalState, Components::BadComponentReference, CORBA::SystemException)
//  raises (IllegalState, BadComponentReference);
{
//  The get_oid_from_ref operation is used by the component to extract the  oid 
//  encapsulated in the reference. The  objref parameter specifies the reference which 
//  contains the  oid. This operation must be called within an operation invocation. If not 
//  the IllegalState exception shall be raised. If the reference was not created by this 
//  container, the  BadComponentReference with the  WRONG_CONTAINER minor 
//  code is raised.
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcontainer_logger(), "");
    try {
        PortableServer::POA_var poa = m_current->get_POA();
        
        PortableServer::ObjectId_var oid = m_poa->reference_to_id(objref);

        const CORBA::ULong seqmax = oid->maximum();
        const CORBA::ULong seqlen = oid->length();

        CORBA::OctetSeq_var oseq =
              new CORBA::OctetSeq(seqmax, seqlen,
                                  oid->get_buffer(true),
                                  true);
        return oseq._retn();
    } catch (const PortableServer::Current::NoContext & ) {
        CDMW_ERROR(CCMLoggers::Get_ccmcontainer_logger(), 
            "Cannot get CCMObject: not in a CORBA request context");
        Components::IllegalState ex;
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const PortableServer::POA::WrongAdapter & ) {
        CDMW_ERROR(CCMLoggers::Get_ccmcontainer_logger(), 
            "PortableServer::POA::WrongAdapter received");
        Components::BadComponentReference ex(Components::WRONG_CONTAINER);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const PortableServer::POA::WrongPolicy & ) {
        // should not happen!
        CDMW_ERROR(CCMLoggers::Get_ccmcontainer_logger(), 
            "PortableServer::POA::WrongPolicy received");
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL,CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const CORBA::SystemException & e) {
        CDMW_ERROR(CCMLoggers::Get_ccmcontainer_logger(), 
            "SystemException exception received: " <<
            Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL,CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


//
// IDL:thalesgroup.com/CdmwCcmContainer/CCM2Context/server_id:1.0
//
char* Session2Context_impl::server_id()
    throw (CORBA::SystemException)
{
    // Server_id is hostname/applicationname/processname
    // It will never change during process lifetime
    static std::string server_id = compute_server_id(m_hostname, m_application_name,
                                                     m_process_name);
    
    return CORBA::string_dup(server_id.c_str());
}
          
          
//
// IDL:thalesgroup.com/CdmwCcmContainer/CCM2Context/obtain_event_channel_for_emitter:1.0
//
::CosEventChannelAdmin::EventChannel_ptr
Session2Context_impl::obtain_event_channel_for_emitter(::Components::PortDescription* port)
    throw (CdmwCcmContainer::NoChannel, CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcontainer_logger(), 
        "port.name=" << port->name());
    if (!CORBA::is_nil(m_event_channel_factory.in())) {
        try {
            return create_event_channel(m_event_channel_factory.in(),
                                        m_process_name,
                                        m_application_name,
                                        port);
        } catch (const ::Components::CreateFailure & e ) {
            CDMW_ERROR(CCMLoggers::Get_ccmcontainer_logger(), 
                "Error creating event channel for port " <<  port->name() <<
                ": " << e.reason);
            CdmwCcmContainer::NoChannel ex(e.reason);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
    } else {
        CDMW_ERROR(CCMLoggers::Get_ccmcontainer_logger(), 
            "Error creating event channel: no event channel factory");
        CdmwCcmContainer::NoChannel ex(CdmwDeployment::NO_EVENT_CHANNEL_FACTORY);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

//
// IDL:thalesgroup.com/CdmwCcmContainer/CCM2Context/create_event_channel_for_publisher:1.0
//
::CosEventChannelAdmin::EventChannel_ptr
Session2Context_impl::create_event_channel_for_publisher(::Components::PortDescription* port)
    throw (Components::CreateFailure, CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcontainer_logger(), 
        "port.name=" << port->name());
    if (!CORBA::is_nil(m_event_channel_factory.in())) {
        return create_event_channel(m_event_channel_factory.in(),
                                    m_process_name,
                                    m_application_name,
                                    port);
    } else {
        CDMW_ERROR(CCMLoggers::Get_ccmcontainer_logger(), 
            "Error creating event channel: no event channel factory");
        Components::CreateFailure ex(CdmwDeployment::NO_EVENT_CHANNEL_FACTORY);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

//
// IDL:thalesgroup.com/CdmwCcmContainer/CCM2Context/resolve_initial_references:1.0
//
CORBA::Object_ptr
Session2Context_impl::resolve_initial_references(const char* identifier)
    throw (CdmwCcmContainer::CCM2Context::InvalidName, 
           CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcontainer_logger(), 
        "identifier="<<identifier);
    try {
        CDMW_DEBUG(CCMLoggers::Get_ccmcontainer_logger(), 
            "Call ORB::resolve_initial_references ("<<identifier<<")");
        CORBA::Object_var obj = m_orb->resolve_initial_references(identifier);
        return obj._retn();
    } catch (const CORBA::ORB::InvalidName& ) {
        // identifier not defined throw it again to prevent the user.
        CDMW_ERROR(CCMLoggers::Get_ccmcontainer_logger(), 
            "InvalidName: " << identifier);
        CdmwCcmContainer::CCM2Context::InvalidName ex;
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const CORBA::SystemException& e) {
        CDMW_ERROR(CCMLoggers::Get_ccmcontainer_logger(), 
            "SystemException exception received: " <<
            Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL,CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


}; // End namespace Container
}; // End namespace CCM
}; // End namespace Cdmw


