/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef ___CCM_TEST_CCM_SERVICES_CCM_CLIENT_IMPL_HPP__
#define ___CCM_TEST_CCM_SERVICES_CCM_CLIENT_IMPL_HPP__

#include <ccm_services/TestCcmServicesSessionClient.skel.hpp>

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtExternalSystem.stub.hpp>

#include <list>

namespace TestCcmServices
{

class CCM_Client_impl;
    

class test_ccm_services : public CppUnit::TestFixture
{
    CCM_Client_impl * m_ccm_client;
    ::TestCcmServices::CCM_Client_Context_var m_session_context;
    TestCcmServices::Client_var m_component_ref;
    
public:
    test_ccm_services();

    //
    // Testable operations
    //
    void do_tests();

    void test_server_ft_state(bool expect_initial_ft_state_primary)
        throw (CORBA::SystemException, std::exception);
    
    void test_server_notify(const std::string& expected_server_issuer_name,
                            const std::string& expected_server_process_name)
        throw(CORBA::SystemException, std::exception);
    
    void test_server_fatal_error(const std::string& expected_server_issuer_name,
                                 const std::string& expected_server_process_name,
                                 bool expect_degraded_notif)
        throw (CORBA::SystemException, std::exception);
    
    void test_server_exit(const std::string& expected_server_issuer_name,
                          const std::string& expected_server_process_name,
                          CdmwPlatformMngt::External::System_ptr system,
                          bool expect_degraded_notif)
        throw (CORBA::SystemException, std::exception);
    
    // register test suite and test operation
    CPPUNIT_TEST_SUITE( test_ccm_services );
    CPPUNIT_TEST( do_tests );
    CPPUNIT_TEST_SUITE_END();

};
    
    
    
//
// IDL:thalesgroup.com/TestCcmServices/Session_CCM_Client:1.0
//
class CCM_Client_impl : virtual public ::TestCcmServices::Session_CCM_Client,
                        virtual public Cdmw::OrbSupport::RefCountLocalObject
{
    CCM_Client_impl(const CCM_Client_impl&);
    void operator=(const CCM_Client_impl&);


    ::TestCcmServices::CCM_Client_Context_var m_session_context;
     
public:

    CCM_Client_impl();
    ~CCM_Client_impl();
     

    //
    // Session component specific operations
    //
    void set_session_context(Components::SessionContext_ptr ctx)
        throw(CORBA::SystemException);
    void ccm_activate()
        throw(CORBA::SystemException);
    void ccm_passivate()
        throw(CORBA::SystemException);
    void ccm_remove()
        throw(CORBA::SystemException);


    //
    // SupervisionObserver supported interface operations
    //
    void notify(CdmwPlatformMngt::Event*)
        throw(CORBA::SystemException);
    
    //
    // return last received event (or nil if already returned)
    //
    CdmwPlatformMngt::Event* get_last_event();
    
    //
    // return first received event (or nil if already returned)
    //
    CdmwPlatformMngt::Event* get_first_event();
    
    //
    // clear received event list
    //
    void clear_event_list();
    

private:

    ::TestCcmServices::CCM_Client_Context* get_context();
    
    Cdmw::TestUtils::ThreadedTestRunner* m_test_thread;
    
    std::list<CdmwPlatformMngt::Event*> m_event_list;
    
};

}; // End of namespace CcmFtTest


#endif // ___CCM_TEST_CCM_SERVICES_CCM_CLIENT_IMPL_HPP__

