/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <ccm_services/CCM_Server_impl.hpp>

#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>


namespace TestCcmServices
{

//
// IDL:thalesgroup.com/CcmFtTest/ServerModule/CCM_Server:1.0
//
CCM_Server_impl::CCM_Server_impl()
{
    m_session_context = TestCcmServices::CCM_Server_Context::_nil();
    m_ft_initial_state = NOT_FT;
    m_process_context = 
        CdmwPlatformMngt::ManagedElementContexts::ProcessContext::_nil();
    m_component_context = 
        CdmwPlatformMngt::ManagedElementContexts::ComponentContext::_nil();
}

CCM_Server_impl::~CCM_Server_impl()
{
    std::cout << "   ... CCM_Server_impl::~CCM_Server_impl called!" << std::endl;
}


void CCM_Server_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_Server_impl::set_session_context called!" << std::endl;
    m_session_context = ::TestCcmServices::CCM_Server_Context::_narrow(ctx);
    
    // get initial FT State via ProcessContext
    try {
        CORBA::Object_var obj = 
            m_session_context->resolve_service_reference("ManagedProcessContext");
        m_process_context =
            CdmwPlatformMngt::ManagedElementContexts::ProcessContext::_narrow(obj.in());
        
        switch (m_process_context->get_process_redundancy_state()) {
            case CdmwPlatformMngt::REDSTATE_NORG:
                std::cout << "   ... initial FT state: NOT_FT" << std::endl;
                m_ft_initial_state = NOT_FT;
                break;
            
            case CdmwPlatformMngt::REDSTATE_PRIMARY:
                std::cout << "   ... initial FT state: PRIMARY" << std::endl;
                m_ft_initial_state = PRIMARY;
                break;
            
            case CdmwPlatformMngt::REDSTATE_PASSIVE:
                std::cout << "   ... initial FT state: BACKUP" << std::endl;
                m_ft_initial_state = BACKUP;
                break;
            
            default:
                // shouldn't happen
                CDMW_ASSERT(false);
        }
    } catch (const Components::CCMException& e) {
        std::cerr << "ERROR !!! ManagedProcessContext no found !!" << std::endl;
        std::cerr << "Throw CORBA::INTERNAL" << std::endl;
        throw CORBA::INTERNAL();
    } catch (const CORBA::UserException& e) {
        std::cerr << "ERROR !!! UserException getting ManagedProcessContext :" <<
                  Cdmw::OrbSupport::OrbSupport::exception_to_string(e) << std::endl;
        std::cerr << "Throw CORBA::INTERNAL" << std::endl;
        throw CORBA::INTERNAL();
    } catch (const CORBA::SystemException& e) {
        std::cerr << "ERROR !!! SystemException getting ManagedProcessContext :" <<
                  Cdmw::OrbSupport::OrbSupport::exception_to_string(e) << std::endl;
        std::cerr << "Throw CORBA::INTERNAL" << std::endl;
        throw CORBA::INTERNAL();
    }
}

::TestCcmServices::CCM_Server_Context* CCM_Server_impl::get_context()
{
    return ::TestCcmServices::CCM_Server_Context::_duplicate(m_session_context.in());
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CCM_Server_impl::ccm_activate()
   throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_Server_impl::ccm_activate called!" << std::endl;
   
    // get ComponentContext
    try {
        CORBA::Object_var obj = 
            m_session_context->resolve_service_reference("ManagedComponentContext");
        m_component_context =
            CdmwPlatformMngt::ManagedElementContexts::ComponentContext::_narrow(obj.in());
        
        CDMW_ASSERT(!CORBA::is_nil(m_component_context.in()));
        
    } catch (const Components::CCMException& e) {
        std::cerr << "ERROR !!! ManagedComponentContext no found !!" << std::endl;
        std::cerr << "Throw CORBA::INTERNAL" << std::endl;
        throw CORBA::INTERNAL();
    } catch (const CORBA::UserException& e) {
        std::cerr << "ERROR !!! UserException getting ManagedComponentContext : " <<
                  Cdmw::OrbSupport::OrbSupport::exception_to_string(e) << std::endl;
        std::cerr << "Throw CORBA::INTERNAL" << std::endl;
        throw CORBA::INTERNAL();
    } catch (const CORBA::SystemException& e) {
        std::cerr << "ERROR !!! SystemException getting ManagedComponentContext : " <<
                  Cdmw::OrbSupport::OrbSupport::exception_to_string(e) << std::endl;
        std::cerr << "Throw CORBA::INTERNAL" << std::endl;
        throw CORBA::INTERNAL();
    }
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
CCM_Server_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_Server_impl::ccm_passivate called!" << std::endl;
} 


//
// ccm_remove function : it destroys the component (call by the container)
//
void
CCM_Server_impl::ccm_remove()
   throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_Server_impl::ccm_remove called!" << std::endl;
}


//
// facet test_services
//
::TestCcmServices::CCM_TestServices_ptr 
CCM_Server_impl::get_test_services()
    throw (CORBA::SystemException)
{
    // component itself implements the test_services facet
    return TestCcmServices::CCM_TestServices::_duplicate(this);
}


::TestCcmServices::FTState 
CCM_Server_impl::get_initial_ft_state()
  throw (CORBA::SystemException)
{
    // set_session_context should have been called
    CDMW_ASSERT(!CORBA::is_nil(m_process_context.in()));
    
    return m_ft_initial_state;
}
    
::TestCcmServices::FTState 
CCM_Server_impl::get_actual_ft_state()
  throw (CORBA::SystemException)
{
    // set_session_context should have been called
    CDMW_ASSERT(!CORBA::is_nil(m_process_context.in()));
    
    try {
        switch (m_process_context->get_process_redundancy_state()) {
            case CdmwPlatformMngt::REDSTATE_NORG:
                return NOT_FT;
            
            case CdmwPlatformMngt::REDSTATE_PRIMARY:
                return PRIMARY;
            
            case CdmwPlatformMngt::REDSTATE_PASSIVE:
                return BACKUP;
            
            default:
                // shouldn't happen
                CDMW_ASSERT(false);
                // for compiler
                return NOT_FT;
        }
    } catch (const CORBA::SystemException& e) {
        std::cerr << "ERROR !!! SystemException calling get_process_redundancy_state() :" <<
                  Cdmw::OrbSupport::OrbSupport::exception_to_string(e) << std::endl;
        std::cerr << "Throw CORBA::INTERNAL" << std::endl;
        throw CORBA::INTERNAL();
    }
}
    

void 
CCM_Server_impl::call_notify_message(CORBA::Long level, const char * message)
  throw (CORBA::SystemException)
{
    // ccm_activate should have been called
    CDMW_ASSERT(!CORBA::is_nil(m_component_context.in()));
    
    try {
        m_component_context->notify_message(level, message);
        
    } catch (const CORBA::SystemException& e) {
        std::cerr << "ERROR during call_notify_message() : " <<
                  Cdmw::OrbSupport::OrbSupport::exception_to_string(e) << std::endl;
        std::cerr << "Throw CORBA::INTERNAL" << std::endl;
        throw CORBA::INTERNAL();
    }
}
  

void 
CCM_Server_impl::call_notify_fatal_error(const char * error_info)
  throw (CORBA::SystemException)
{
    // ccm_activate should have been called
    CDMW_ASSERT(!CORBA::is_nil(m_component_context.in()));
    
    try {
        m_component_context->notify_fatal_error(error_info);
        
    } catch (const CORBA::SystemException& e) {
        std::cerr << "ERROR during call_notify_fatal_error() : " <<
                  Cdmw::OrbSupport::OrbSupport::exception_to_string(e) << std::endl;
        std::cerr << "Throw CORBA::INTERNAL" << std::endl;
        throw CORBA::INTERNAL();
    }
}
  

void 
CCM_Server_impl::call_notify_user_event(const CORBA::Any & data)
  throw (CORBA::SystemException)
{
    // ccm_activate should have been called
    CDMW_ASSERT(!CORBA::is_nil(m_component_context.in()));
    
    try {
        m_component_context->notify_user_event(data);
        
    } catch (const CORBA::SystemException& e) {
        std::cerr << "ERROR during call_notify_user_event() : " <<
                  Cdmw::OrbSupport::OrbSupport::exception_to_string(e) << std::endl;
        std::cerr << "Throw CORBA::INTERNAL" << std::endl;
        throw CORBA::INTERNAL();
    }
}
  

void 
CCM_Server_impl::call_exit(CORBA::Long exit_code)
  throw (CORBA::SystemException)
{
    // ccm_activate should have been called
    CDMW_ASSERT(!CORBA::is_nil(m_process_context.in()));
    
    try {
        /* */std::cerr << "!!!!!!!!!!!!!!! call exit !!!!!!!" << std::endl;
        m_process_context->exit(exit_code);

    } catch (const CORBA::SystemException& e) {
        std::cerr << "ERROR during call_notify_user_event() : " <<
                  Cdmw::OrbSupport::OrbSupport::exception_to_string(e) << std::endl;
        std::cerr << "Throw CORBA::INTERNAL" << std::endl;
        throw CORBA::INTERNAL();
    }
}


};  // namespace TestCcmServices

