/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Foundation/common/Locations.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtManagedComponent.stub.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>

#include "ccm_supervised_comp/TestSupervisedComponent.hpp"
#include "ccm_supervised_comp/NamingDisplayer.hpp"

namespace Cdmw 
{

namespace CCM 
{

CPPUNIT_TEST_SUITE_REGISTRATION( TestSupervisedComponent );

// constructor
TestSupervisedComponent::TestSupervisedComponent()
{    
    m_orb =  CORBA::ORB::_duplicate( Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());
    m_rootPOA = PortableServer::POA::_duplicate( Cdmw::TestUtils::CORBATestManager::instance()->get_POA());

    m_host_name1 = Cdmw::TestUtils::CORBATestManager::instance()->get_property ("host_name1");
    m_host_name2 = Cdmw::TestUtils::CORBATestManager::instance()->get_property ("host_name2");
}


// destructor
TestSupervisedComponent::~TestSupervisedComponent()
{
}

// do_tests
void TestSupervisedComponent::do_tests()
{
    TEST_INFO("Get the CDMW repository interface");
    CdmwNamingAndRepository::Repository_var repository
        = CdmwNamingAndRepository::Repository::_nil();
    CosNaming::NamingContextExt_var adminRootCtx
        = CosNaming::NamingContextExt::_nil();
    
    try
    {
        CORBA::Object_var obj = 
            Cdmw::NamingAndRepository::RepositoryInterface::get_repository();
        repository = CdmwNamingAndRepository::Repository::_narrow(obj.in());
        CPPUNIT_ASSERT(true);
    }
    catch (const CORBA::BAD_INV_ORDER&)
    {
        TEST_INFO("Cannot get repository interface: BAD_INV_ORDER");
        CPPUNIT_ASSERT(false);
    }
    catch(CORBA::Exception &e)
    {
        TEST_INFO("Cannot get repository interface: CORBA exception raised");
        std::cout << e._name();
        CPPUNIT_ASSERT(false);
    }
    catch(...)
    {
        TEST_INFO("Cannot get repository interface: Unexpected exception raised!");
        CPPUNIT_ASSERT(false);
        return;
    }

    TEST_INFO("Get the admin root context");
    try
    {

        CosNaming::NamingContext_var temp_adminRootCtx =
        repository->resolve_root_context(Common::Locations::ADMIN_ROOT_CONTEXT_ID);

        adminRootCtx = CosNaming::NamingContextExt::_narrow(temp_adminRootCtx.in());
        NamingDisplayer::list_context(adminRootCtx.in());
        CPPUNIT_ASSERT(true);
    
    }
    catch(CORBA::Exception &e)
    {
        TEST_INFO("Cannot resolve_root_context ADMIN_ROOT_CONTEXT: CORBA Exception raised");
        std::cout << e._name();
        CPPUNIT_ASSERT(false);
    }
    catch(...)
    {
        TEST_INFO("Cannot resolve_root_context ADMIN_ROOT_CONTEXT: unexpected exception raised");
        CPPUNIT_ASSERT(false);
    }

    TEST_INFO("Check a context for each host of the test is created : host1");
    CosNaming::Name_var name = new CosNaming::Name;
    name->length(1);
    CosNaming::NameComponent_var nc = new CosNaming::NameComponent;
    nc->id = CORBA::string_dup(m_host_name1.c_str());
    name->operator[](0) = nc.in();

    try
    {
        CORBA::Object_var obj = adminRootCtx->resolve(name.in());
        CPPUNIT_ASSERT(true);
    }
    catch(CORBA::Exception &e)
    {
        std::cout << e._name();
        CPPUNIT_ASSERT(false);
    }

    TEST_INFO("Check a context for each host of the test is created : host2");
    name->length(1);
    nc->id = CORBA::string_dup(m_host_name2.c_str());
    name->operator[](0) = nc.in();

    try
    {
        CORBA::Object_var obj = adminRootCtx->resolve(name.in());
        CPPUNIT_ASSERT(true);
    }
    catch(CORBA::Exception &e)
    {
        std::cout << e._name();
        CPPUNIT_ASSERT(false);
    }


    TEST_INFO("Waiting deployment... and display naming context content");
    Cdmw::OsSupport::OS::sleep(10000);


    CdmwPlatformMngt::ManagedComponent_var mng_comp 
        = CdmwPlatformMngt::ManagedComponent::_nil();
    bool component_found=false;
    while (!component_found)
    {
        TEST_INFO("Resolve \"Fork1\" from the admin root context");
        name->length(6);
        nc->id = CORBA::string_dup(m_host_name1.c_str());
        name->operator[](0) = nc.in();
        nc->id = CORBA::string_dup("DinnerApplication");
        name->operator[](1) = nc.in();
        nc->id = CORBA::string_dup("GreekSchoolProcess");
        name->operator[](2) = nc.in();
        nc->id = CORBA::string_dup("CCM");
        name->operator[](3) = nc.in();
        nc->id = CORBA::string_dup("Components");
        name->operator[](4) = nc.in();
        nc->id = CORBA::string_dup("Fork1");
        name->operator[](5) = nc.in();
    
        try
        {
            CORBA::Object_var obj = adminRootCtx->resolve(name.in());
            
            TEST_INFO("Calling cdmw_initialise_component");
            mng_comp
                = CdmwPlatformMngt::ManagedComponent::_narrow(obj.in());
            component_found = true;

            CdmwPlatformMngtBase::StartupKind kind;
            kind.state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
            kind.startup_mode = 1;
            kind.specified_state_id = CORBA::string_dup("");
            mng_comp->cdmw_initialise_component(kind);

            CPPUNIT_ASSERT(true);
            break;
        }
        catch(CORBA::Exception &e)
        {
            std::cout << e._name();
            if (component_found)
                CPPUNIT_ASSERT(false);
        }

        try
        {
            NamingDisplayer::list_context(adminRootCtx.in());
            CPPUNIT_ASSERT(true);
        }
        catch(CORBA::Exception &e)
        {
            TEST_INFO("Cannot list_context: CORBA Exception raised");
            std::cout << e._name();
            CPPUNIT_ASSERT(false);
        }
        catch(...)
        {
            TEST_INFO("Cannot list_context: unexpected exception raised");
            CPPUNIT_ASSERT(false);
        }
        Cdmw::OsSupport::OS::sleep(2000);
    }

    Cdmw::OsSupport::OS::sleep(10000);
    
    TEST_INFO("waiting cdmw_initialise_component raising OBJECT_NOT_EXIST");
    while (component_found)
    {
        try
        {
            CdmwPlatformMngtBase::StartupKind kind;
            kind.state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
            kind.startup_mode = 1;
            kind.specified_state_id = CORBA::string_dup("");
            mng_comp->cdmw_initialise_component(kind);
        }
        catch (const CORBA::OBJECT_NOT_EXIST& )
        {
            component_found = false;
            CPPUNIT_ASSERT(true);
        }
        catch(const CORBA::Exception &e)
        {
            std::cout << e._name();
            if (component_found)
                CPPUNIT_ASSERT(false);
        }

        try
        {
            NamingDisplayer::list_context(adminRootCtx.in());
            CPPUNIT_ASSERT(true);
        }
        catch(CORBA::Exception &e)
        {
            TEST_INFO("Cannot list_context: CORBA Exception raised");
            std::cout << e._name();
            CPPUNIT_ASSERT(false);
        }
        catch(...)
        {
            TEST_INFO("Cannot list_context: unexpected exception raised");
            CPPUNIT_ASSERT(false);
        }
        Cdmw::OsSupport::OS::sleep(2000);
    }
       
    try
    {
        TEST_INFO("Resolve \"Fork1\" from the admin root context to check if removed");
        name->length(6);
        nc->id = CORBA::string_dup(m_host_name1.c_str());
        name->operator[](0) = nc.in();
        nc->id = CORBA::string_dup("DinnerApplication");
        name->operator[](1) = nc.in();
        nc->id = CORBA::string_dup("GreekSchoolProcess");
        name->operator[](2) = nc.in();
        nc->id = CORBA::string_dup("CCM");
        name->operator[](3) = nc.in();
        nc->id = CORBA::string_dup("Components");
        name->operator[](4) = nc.in();
        nc->id = CORBA::string_dup("Fork1");
        name->operator[](5) = nc.in();
    
        CORBA::Object_var obj = adminRootCtx->resolve(name.in());
        CPPUNIT_ASSERT(false);
            
    }
    catch(const CosNaming::NamingContext::NotFound& )
    {
        TEST_INFO("Object NotFound: OK");
        CPPUNIT_ASSERT(true);
    }
    catch(const CORBA::Exception &e)
    {
        std::cout << e._name();
        CPPUNIT_ASSERT(false);
    }
}

} // end of CCM

} // end of cdmw
