/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.testccmsupervisedcomp;

import cdmw.testutils.CORBATestManager;
import cdmw.ossupport.OS;
import cdmw.orbsupport.StrategyList;
import cdmw.orbsupport.ORBSupport;
import cdmw.platformmngt.PlatformInterface;
import cdmw.cdmwinit.CdmwInterface;

import org.omg.CORBA.ORB;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;
import org.omg.PortableServer.POAManager;

import java.lang.String;

public class MainTest
{

    private static String applicationName;
    private static String processName;
    

    public static void main(String[] args) {

        // get host name
        ORB orb = null;
        String hostName1 = OS.getOptionValue(args, "--host1");
        String hostName2 = OS.getOptionValue(args, "--host2");

        try {
         
            // Initialises the ORB
            StrategyList strategy = new StrategyList();
            strategy.addORBThreaded();
            strategy.addPOAThreadPerConnection();

            // initialise the orb    
            orb = ORBSupport.ORBInit(args, strategy);

            // initialise CDMW
            CdmwInterface.cdmwInit(
                orb, args, null, "cdmw.generated.TestSupervisedComponentInit");

            // get the root POA
            Object obj = orb.resolve_initial_references("RootPOA");
            POA rootPOA = POAHelper.narrow(obj);
            POAManager poaMng = rootPOA.the_POAManager();
            poaMng.activate();

            // Launch the test in a thread
            TestSupervisedComponentThread testThread = 
                new TestSupervisedComponentThread(orb, hostName1, hostName2);
            testThread.start();
            
            // start orb
            orb.run();
        }
                
        catch (org.omg.CORBA.SystemException e) {  
            System.err.println( "FAILURE : CORBA exception raised :"
                                + e.getMessage());
            orb.shutdown(false);
        }
        catch(java.lang.Exception e) {
            System.err.println("FAILURE : Exception raised" + "\r\n"
                               + e.getMessage());
            orb.shutdown(false);
        }
        finally {            
            ORBSupport.ORBCleanup(orb);
            orb.destroy();
        }
    }

}

