/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.testccmsupervisedcomp;

import java.lang.String;
import org.omg.CosNaming.NamingContext;
import org.omg.CosNaming.NamingContextExtHelper;
import org.omg.CosNaming.BindingIterator;
import org.omg.CosNaming.BindingIteratorHolder;
import org.omg.CosNaming.Binding;
import org.omg.CosNaming.BindingListHolder;
import org.omg.CosNaming.BindingType;
import org.omg.CORBA.Object;


class NamingDisplayer
{

    protected static String tab2string (int level)
    {
        String res = new String("");
        for (int i = 0; i < level; i++) {
            res += "    ";
        }
        return res;
    }

    public static void showChunk(final Binding[] bl,
                                 NamingContext nc,
                                 int level)
    {
        for (int i = 0; i < bl.length; i++) {
            // Note from CosNaming specification: 
            //   In struct Binding, binding_name is incorrectly defined
            //   as a Name instead of a NameComponent. This definition is
            //   unchanged for compatibility reasons.
            String tab = tab2string(level);

            System.out.print(tab + bl[i].binding_name[0].id);
            if (bl[i].binding_name[0].kind.equals("")) {
                System.out.print(" . " + bl[i].binding_name[0].kind);
            }

            if (bl[i].binding_type == BindingType.ncontext) {
                System.out.println(" : context");

                try {
                    Object obj = nc.resolve(bl[i].binding_name);
                    NamingContext nc2 = NamingContextExtHelper.narrow(obj);
                    listContext(nc2, level + 1);
                }
                catch (Exception e) {
                    System.err.println("Exception raised in showChunk: " + e);
                }
            }
            else {
                System.out.println(" : reference");
            }
        }
    }


    public static void listContext(NamingContext nc,
                                   int level)
    {
        BindingIteratorHolder it = new BindingIteratorHolder();
        BindingListHolder         bl = new BindingListHolder();
        int CHUNK = 100;                    // Chunk size

        nc.list(CHUNK,bl,it);               // Get first chunk
        showChunk(bl.value, nc, level);     // Print first chunk

        if (it != null) {                   // More bindings?
            boolean more;
            do {
                more = it.value.next_n(CHUNK, bl);// Get next chunk
                showChunk(bl.value, nc, level);   // Print chunk
            }
            while (more);                   // While not done
            it.value.destroy();                   // Clean up
        }
    }

}
