/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.testccmsupervisedcomp;

import cdmw.common.Locations;
import cdmw.namingandrepository.RepositoryInterface;
import cdmw.ossupport.OS;

import com.thalesgroup.CdmwNamingAndRepository.Repository;
import com.thalesgroup.CdmwPlatformMngt.ManagedComponent;
import com.thalesgroup.CdmwPlatformMngt.ManagedComponentHelper;
import com.thalesgroup.CdmwPlatformMngtBase.StartupKind;
import com.thalesgroup.CdmwPlatformMngt.ManagedComponentPackage.BadOrder;
import com.thalesgroup.CdmwPlatformMngtBase.StateRetrieval;

import org.omg.CosNaming.NamingContext;
import org.omg.CosNaming.NameComponent;
import org.omg.CosNaming.NameComponentHelper;
import org.omg.CosNaming.NamingContextPackage.NotFound;
import org.omg.CORBA.Object;

import java.lang.String;

import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.fail;

import cdmw.testutils.CORBATestManager;
import cdmw.testutils.TestManager;


public class TestSupervisedComponent
{

    private org.omg.CORBA.ORB orb;
    private org.omg.PortableServer.POA poa;
    private String hostName1;
    private String hostName2;


    @Before
    public void init() throws Exception {

        CORBATestManager testManager = 
            (CORBATestManager) TestManager.getLocalInstance();
        try {
            this.orb = testManager.getORB();
            this.poa = testManager.getPOA();
            this.hostName1 = (String)testManager.getLocalProperty("hostName1");
            this.hostName2 = (String)testManager.getLocalProperty("hostName2");
        }

        catch (CORBATestManager.InvalidObject e) {
            System.err.println("INIT FAILURE: Cannot start tests: " + e);
            System.exit(-1);
        }
    }

    @Test
    public void doTests()
    {
        Repository rep = null;
        NamingContext adminRootCtx = null;
        int timescale = cdmw.testutils.TestManager.getTimescale();

        try {
            System.out.println("*** Get the CDMW repository interface");
            rep = RepositoryInterface.getRepository();
        }
        catch (Exception e) {
            fail("TEST FAILED : Cannot get repository interface: " + e);
        }

        System.out.println("*** Get the admin root context");
        try {
            adminRootCtx = 
                rep.resolve_root_context(Locations.ADMIN_ROOT_CONTEXT_ID);
            NamingDisplayer.listContext(adminRootCtx, 0);
        }
        catch (Exception e) {
            fail("TEST FAILED : resolve_root_context ADMIN_ROOT_CONTEXT: " +e);
        }

        System.out.println("*** Check a context for each host "
                           + "of the test is created");
        NameComponent[] componentNameHost1 = new NameComponent[1];
        NameComponent[] componentNameHost2 = new NameComponent[1];
        componentNameHost1[0] = new NameComponent(hostName1, "");
        componentNameHost2[0] = new NameComponent(hostName2, "");

        try {
            Object obj1 = adminRootCtx.resolve(componentNameHost1);
            System.out.println("host1 found");
            Object obj2 = adminRootCtx.resolve(componentNameHost2);
            System.out.println("host2 found");
        }
        catch (Exception e) {
            fail("TEST FAILED : host cannot be resolved: " + e);
        }


        System.out.println("*** Waiting deployment... "
                           + "and display naming context content");
        try {
            OS.sleep(timescale * 3000);
        }
        catch (java.lang.InterruptedException e) {}

        ManagedComponent mngComponent = null;
        boolean componentFound = false;
        while (! componentFound) {

            System.out.println("*** Resolve Fork1 from the admin root context");
            componentNameHost1 = new NameComponent[6];
            componentNameHost1[0] = new NameComponent(hostName1, "");
            componentNameHost1[1] = new NameComponent("DinnerApplication", "");
            componentNameHost1[2] = new NameComponent("GreekSchoolProcess","");
            componentNameHost1[3] = new NameComponent("CCM", "");
            componentNameHost1[4] = new NameComponent("Components", "");
            componentNameHost1[5] = new NameComponent("Fork1", "");
    
            try {
                Object obj = adminRootCtx.resolve(componentNameHost1);
                
                System.out.println("Calling cdmw_initialise_component");
                mngComponent = ManagedComponentHelper.narrow(obj);
                componentFound = true;

                StartupKind kind = new StartupKind();
                kind.state_retrieval = StateRetrieval.LAST_STATE;
                kind.startup_mode = 1;
                kind.specified_state_id = "";
                mngComponent.cdmw_initialise_component(kind);
                break;
            }
            catch (BadOrder e) {
                fail("TEST FAILED : component cannot be initialized: " + e);
            }
            catch (Exception e) {
                if (componentFound) {
                    fail("TEST FAILED : host cannot be resolved: " + e);
                }
            }

            try {
                NamingDisplayer.listContext(adminRootCtx, 0);
            }
            catch (Exception e) {
                fail("TEST FAILED : Cannot listContext: " + e);
            }
            try {
                OS.sleep(timescale * 3000);
            }
            catch (java.lang.InterruptedException e) {}
        }
        

        System.out.println("*** Waiting cdmw_initialise_component raising OBJECT_NOT_EXIST");
        try {
            OS.sleep(timescale * 3000);
        }
        catch (java.lang.InterruptedException e) {}
        while (componentFound) {
            try {
                StartupKind kind = new StartupKind();
                kind.state_retrieval = StateRetrieval.LAST_STATE;
                kind.startup_mode = 1;
                kind.specified_state_id = "";
                mngComponent.cdmw_initialise_component(kind);
            }
            catch (BadOrder e) {
                fail("TEST FAILED : component cannot be initialized: " + e);
            }
            catch (org.omg.CORBA.OBJECT_NOT_EXIST e) {
                componentFound = false;
            }
            catch (Exception e) {
                if (componentFound) {
                    fail("TEST FAILED : host cannot be resolved: " + e);
                }
            }

            try {
                NamingDisplayer.listContext(adminRootCtx, 0);
            }
            catch (Exception e) {
                fail("TEST FAILED : Cannot listContext: " + e);
            }
            try {
                OS.sleep(timescale * 3000);
            }
            catch (java.lang.InterruptedException e) {}
        }

        System.out.println("*** Resolve \"Fork1\" from the admin root context "
                           + "to check if removed");
        try {
            Object obj = adminRootCtx.resolve(componentNameHost1);
            fail("TEST FAILED: component found though expected to be removed");
        }
        catch (NotFound e) {
            System.out.println("Object NotFound: OK");
        }
        catch (Exception e) {
            fail("TEST FAILED : unexpected exception: " + e);
        }
    }

}
