/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <ccm_typechecking/CCM_Client_impl.hpp>
#include <ccm_typechecking/TestEvent_impl.hpp>

#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtManagedElementContexts.stub.hpp>



namespace TestCcmTypeChecking
{

//
// IDL:thalesgroup.com/CcmFtTest/ClientModule/CCM_Client:1.0
//
CCM_Client_impl::CCM_Client_impl()
    : m_component_name("UnamedComponent")
{
    m_session_context = TestCcmTypeChecking::CCM_Client_Context::_nil();
}

CCM_Client_impl::~CCM_Client_impl()
{
}


void CCM_Client_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    m_session_context = ::TestCcmTypeChecking::CCM_Client_Context::_narrow(ctx);
}
        
::TestCcmTypeChecking::CCM_Client_Context* CCM_Client_impl::get_context()
{
    return ::TestCcmTypeChecking::CCM_Client_Context::_duplicate(m_session_context.in());
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CCM_Client_impl::ccm_activate()
   throw(CORBA::SystemException)
{
    try {
        // get ManagedComponentContext
        CORBA::Object_var obj = 
            m_session_context->resolve_service_reference("ManagedComponentContext");
        CdmwPlatformMngt::ManagedElementContexts::ComponentContext_var supContext =
            CdmwPlatformMngt::ManagedElementContexts::ComponentContext::_narrow(obj.in());
        CDMW_ASSERT(!CORBA::is_nil(supContext.in()));
     
        // get component, process, application names
        m_component_name = supContext->managed_element_name();
    } catch (const Components::CCMException& e) {
        std::cerr << "ManagedComponentContext not found !!" << std::endl;
    } catch (const CORBA::SystemException& e) {
        std::cerr << "SystemException getting ManagedComponentContext: " 
                  << e << std::endl;
    }

    std::cerr << ">>>>>>" << m_component_name << " is active" << std::endl;
    
    std::cerr << "-----> test connection to Server1" << std::endl;
    try {
        TestTypeChecking_var test_typechecking =
            m_session_context->get_connection_test_server1();
            
        test_typechecking->test_operation(m_component_name.in());
        
        std::cerr << "   ==> connection OK" << std::endl;
    } catch (const CORBA::SystemException& e) {
        std::cerr << "   ==> connection NOT OK: " << e << std::endl;
    }
    
    
    std::cerr << "-----> test connection to Server2" << std::endl;
    try {
        TestTypeChecking_var test_typechecking =
            m_session_context->get_connection_test_server2();
            
        test_typechecking->test_operation(m_component_name.in());
        
        std::cerr << "   ==> connection OK" << std::endl;
    } catch (const CORBA::SystemException& e) {
        std::cerr << "   ==> connection NOT OK: " << e << std::endl;
    }
    
    
    std::cerr << "-----> test connection to Server3" << std::endl;
    try {
        TestTypeChecking_var test_typechecking =
            m_session_context->get_connection_test_server3();
            
        test_typechecking->test_operation(m_component_name.in());
        
        std::cerr << "   ==> connection OK" << std::endl;
    } catch (const CORBA::SystemException& e) {
        std::cerr << "   ==> connection NOT OK: " << e << std::endl;
    }
    
    
    std::cerr << "-----> send event to all servers" << std::endl;
    try {
        TestEvent_var ev = new TestEvent_impl(m_component_name.in());
    
        m_session_context->push_test_event (ev.in());
        
        std::cerr << "   ==> event sent" << std::endl;
    } catch (const CORBA::SystemException& e) {
        std::cerr << "   ==> event not sent: " << e << std::endl;
    }

}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
CCM_Client_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
    std::cerr << ">>>>>>" << m_component_name << " is passive" << std::endl;
} 


//
// ccm_remove function : it destroys the component (call by the container)
//
void
CCM_Client_impl::ccm_remove()
   throw(CORBA::SystemException)
{
    std::cerr << ">>>>>>" << m_component_name << " is removed" << std::endl;
}


};  // namespace TestCcmTypeChecking

