#!/bin/sh
# =====================================================================
# This file is part of CARDAMOM (R) which is jointly developed by THALES
# and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
# Copyright (C) THALES 2000-2003. All rights reserved.
# Copyright (C) THALES 2004-2009. All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
# License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

if [ -z "$ACE_ROOT" ]; then
    echo "Error: ACE_ROOT environment variable is not defined"
    exit 1
fi

if [ -z "$1" -o "$1" = "-h" -o "$1" = "--help" ]; then
    echo "Usage: $0 <DeploymentPlanFile.cdp>"    
    echo "       This srcipt converts a .cdp file into a .cdr file."
    exit 0
fi

if [ ! -f $1 ]; then
    echo "Error: file $1 not found"
    exit 1
fi


CDP_FILE_ARG=$1
CDR_FILE=${CDP_FILE_ARG/%cdp/cdr}    

which perl 2> /dev/null > /dev/null
HAS_PERL=$?
if [ $HAS_PERL -eq 0 ]; then
    which cdmw_cdp_subst_cmd.sh > /dev/null 2>/dev/null
    HAS_CDP_SUBST_CMD=$?
fi

if [ $HAS_CDP_SUBST_CMD -eq 0 ]; then
    echo  "Calling cdmw_cdp_subst_cmd.sh script in order to execute shell command contained in cdp file"
    cdmw_cdp_subst_cmd.sh $CDP_FILE_ARG ${CDP_FILE_ARG/%cdp/tmp.cdp}
    if [ $? -ne 0 ]; then
    echo "Error: problem occurs during execution of cdmw_cdp_subst_cmd.pl"
    exit 1
    fi
    CDP_FILE_ARG=${CDP_FILE_ARG/%cdp/tmp.cdp}
fi



CDP_FILE=${CDP_FILE_ARG/%cdp/_new_format.cdp}
cp $CDP_FILE_ARG $CDP_FILE

echo "Calling cdmw_update_cdp script in order to add xmi:id and xmi:idref"
cdmw_update_cdp.sh $CDP_FILE;

echo "Convert $CDP_FILE into $CDR_FILE"

tmpDir=$(dirname $(dirname $0))/lib/c++
if [ -d $tmpDir ] ; then
    cd $tmpDir
    export LD_LIBRARY_PATH=$(pwd):$LD_LIBRARY_PATH
    cd -
fi

# DAnCE will use this directive to redirect its logs to the CDMW logs, 
# it dynamically calls the following definition 'extern "C" ACE_Service_Object * _make_CdmwLoggerFactoryImpl()'
# that create an instance of TaoDAnCELoggerFactory in library libcdmwccmdeploymenttool
ORBSvcConfDirective="dynamic DAnCE_Logger_Backend_Factory Service_Object * cdmwccmdeploymenttool:_make_CdmwLoggerFactoryImpl() \"$(basename $0) --CdmwLogLevel=ALL\""
$ACE_ROOT/bin/dance --log-level 1 -x $CDP_FILE -w $CDR_FILE -ORBSvcConfDirective "$ORBSvcConfDirective"

diff $CDP_FILE_ARG $CDP_FILE 2>&1 > /dev/null
if [ $? -ne 0 ]
then
   echo ""
   echo "WARNING: .cdp input file has a deprecated format! Please use the script cdmw_update_cdp.sh to update it!"
   echo "Conversion done."
fi

rm $CDP_FILE
if [ $HAS_CDP_SUBST_CMD -eq 0 ]; then
    rm  $CDP_FILE_ARG
fi
