#!/bin/sh
# =====================================================================
# This file is part of CARDAMOM (R) which is jointly developed by THALES
# and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
# Copyright (C) THALES 2000-2003. All rights reserved.
# Copyright (C) THALES 2004-2009. All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
# License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

if [ -z "$ACE_ROOT" ]; then
    echo "Error: ACE_ROOT environment variable is not defined"
    exit 1
fi

SCRIPT_NAME=$0
DANCE_PORT_NUMBER=21666

#usage : give hosts where there are node managers
usage()
{
    echo "Usage : $0 [-h | --help] "
    echo "        --host=NODE_NAME --system-port=PlatformDaemonPort --dance-port=DAnCEPort" 
    echo "        --plan-id=PLAN_ID --component-id=COMPONENT_ID"
    echo "        [--port-name=PORT_NAME --output-file=FILE_NAME]"
    echo "Where "
    echo "-h, --help                : display this help"
    echo "--host                    : the host name hosting the component"
    echo "--system-port             : the local daemon port number"
    echo "                            for the discovery of the FT repository"
    echo "--dance-port              : the internal port of the DAnCE node manager"
    echo "                            if not specified, will use default value $DANCE_PORT_NUMBER"
    echo "--plan-id                 : the deployment plan identifier of the component as specified in the CDP file"
    echo "--component-id            : the component instance identifier as specified in the CDP file"
    echo "--port-name (optional)    : the port name of the component, if any"
    echo "--output-file (optional)  : the path of the file where the corbaloc has to be stored"
    echo "                            if not specified, the corbaloc will be printed out on the standard output"
    echo "--verbose                 : diplays the DAnCE command to execute"
    exit 0
}


while [ -n "$1" ]; do
    case $1 in
        -h) usage;;
        --help) usage;;
        --host=*)
            NODE_NAME=`echo $1 | sed 's/[-_a-zA-Z0-9]*=//'`;;
        --system-port=*)
            DAEMON_PORT=`echo $1 | sed 's/[-_a-zA-Z0-9]*=//'`;;
        --dance-port=*)
            DANCE_PORT_NUMBER=`echo $1 | sed 's/[-_a-zA-Z0-9]*=//'`;;
        --plan-id=*)
            PLAN_ID=`echo $1 | sed 's/[-_a-zA-Z0-9]*=//'`;;     
        --component-id=*)
            COMPONENT_ID=`echo $1 | sed 's/[-_a-zA-Z0-9]*=//'`;;        
        --port-name=*)
            PORT_NAME=`echo $1 | sed 's/[-_a-zA-Z0-9]*=//'`;;
        --verbose)
            VERBOSE=yes;;
        --output-file=*)
            OUTPUT_FILE=`echo $1 | sed 's/[-_a-zA-Z0-9]*=//'`;;
        *)
            BadArgEncounter=1;break;;
    esac
    shift
done


if [ -z "$NODE_NAME" ] ; then
    echo "Error : Missing the host name"
    echo "$SCRIPT_NAME -h for help"
    exit 1
fi

if [ -z "$DAEMON_PORT" ] ; then
    echo "Error : Missing system port"
    echo "$SCRIPT_NAME -h for help"
    exit 1
fi


if [ -z "$PLAN_ID" ] ; then
    echo "Error : Missing Plan ID"
    echo "$SCRIPT_NAME -h for help"
    exit 1
fi

if [ -z "$COMPONENT_ID" ] ; then
    echo "Error : Missing the Component ID"
    echo "$SCRIPT_NAME -h for help"
    exit 1
fi


if [ "$VERBOSE" ] ; then
    echo Executing ${ACE_ROOT}/bin/dance --gen-object-key "$NODE_NAME $PLAN_ID $COMPONENT_ID $PORT_NAME" ...
fi

OBJECT_KEY=`${ACE_ROOT}/bin/dance --gen-object-key "$NODE_NAME $PLAN_ID $COMPONENT_ID $PORT_NAME"`

#echo "Got this object Key = $OBJECT_KEY"

CORBALOC=corbaloc::$NODE_NAME:$DANCE_PORT_NUMBER/$OBJECT_KEY

if [ -n "$OUTPUT_FILE" ] ; then
    echo The reference has been stored in $OUTPUT_FILE
    echo $CORBALOC > $OUTPUT_FILE
else
    echo "$CORBALOC"
fi
