#!/bin/sh
# =====================================================================
# This file is part of CARDAMOM (R) which is jointly developed by THALES
# and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
# Copyright (C) THALES 2000-2003. All rights reserved.
# Copyright (C) THALES 2004-2009. All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
# License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

SCRIPT_NAME=$0
CDMW_PROCESS_NAME="DAnCE"
CDMW_APPLICATION_NAME="CCMDeployment"

usage()
{
    echo "Usage : $0.sh [-h | --help]"
    echo "              --system-port=<Port>"
    echo "              [--execution-manager-host=<executionManagerHost>]"
    echo "              --cdp-file=<deploymentPlan> "
    echo "              [--process-name=<processName>] [--application-name=<applicationName>]"
    echo "Where "
    echo "-h, --help               : display this help"
    echo "--system-port            : <daemonPort> is the port of the CARDAMOM daemon"
    echo "--execution-manager-host : <executionManagerHost> is the host where the "
    echo "                           execution manager is runing (localhost if not specified)"
    echo "--cdp-file               : <deploymentPlan> is the file containing the deployment plan"
    echo "--process-name            : <processName> Process name in use for CARDAMOM logging"
    echo "                            if not specified, \"$CDMW_PROCESS_NAME\" will be used"
    echo "--application-name        : <applicationName> Application name in use for CARDAMOM logging"
    echo "                            if not specified, \"$CDMW_APPLICATION_NAME\" will be used"
    exit 0
}


#check env variables for Xerces and DAnCE
if [ -z "$CIAO_ROOT" ];then
    echo "error : The CIAO_ROOT environment variable is missing."
    exit 1
fi

execManagerHost=`hostname`

while [ -n "$1" ]; do
    case $1 in
        -h) usage;;
        --help) usage;;
        --system-port=*)
            DAEMON_PORT=`echo $1 | sed 's/[-_a-zA-Z0-9]*=//'`;;
        --execution-manager-host=*)
            execManagerHost=`echo $1 | sed 's/[-_a-zA-Z0-9]*=//'`;;
        --cdp-file=*) 
            deploymentPlan=`echo $1 | sed 's/[-_a-zA-Z0-9]*=//'`;;
        --process-name=*) 
            CDMW_PROCESS_NAME=`echo $1 | sed 's/[-_a-zA-Z0-9]*=//'`;;
        --application-name=*) 
            CDMW_APPLICATION_NAME=`echo $1 | sed 's/[-_a-zA-Z0-9]*=//'`;;
        *)
            BadArgEncounter=$1;break;;
    esac
    shift
done

if [ -z "$DAEMON_PORT" ] ; then
    echo "Error : Missing system port"
    echo "$SCRIPT_NAME -h for help"
    exit 1
fi

if [ -z "$execManagerHost" ] ; then
    echo "Error : Missing execution manager port"
    echo "$SCRIPT_NAME -h for help"
    exit 1
fi

if [ -z "$deploymentPlan" ] ; then
    echo "Error : Missing deployment plan"
    echo "$SCRIPT_NAME -h for help"
    exit 1
fi



if [ -z "$BadArgEncounter" ] ; then
    # adjust PATH and LD_LIBRARY_PATH

    export PATH=${ACE_ROOT}/bin:$PATH
    export LD_LIBRARY_PATH=${ACE_ROOT}/lib:@config.XercesCPP.lib.dir@:$LD_LIBRARY_PATH
    
    tmpDir=$(dirname $(dirname $0))/lib/c++
    if [ -d $tmpDir ] ; then
        cd $tmpDir
        export LD_LIBRARY_PATH=$(pwd):$LD_LIBRARY_PATH
        cd -
    fi

    # DAnCE will use this directive to redirect its logs to the CDMW logs, 
    # it dynamically calls the following definition 'extern "C" ACE_Service_Object * _make_CdmwLoggerFactoryImpl()'
    # that create an instance of TaoDAnCELoggerFactory in library libcdmwccmdeploymenttool
    ORBSvcConfDirective="dynamic DAnCE_Logger_Backend_Factory Service_Object * cdmwccmdeploymenttool:_make_CdmwLoggerFactoryImpl() \"$(basename $0) --CdmwLogLevel=ALL --DaemonPort=${DAEMON_PORT} --process-name=${CDMW_PROCESS_NAME} --application-name=${CDMW_APPLICATION_NAME}\""

    ${ACE_ROOT}/bin/dance --log-level 1 -x file://$deploymentPlan -k corbaname::${execManagerHost}:$DAEMON_PORT/NameService\#ExecutionManager -ORBSvcConfDirective "$ORBSvcConfDirective"

    echo "Press <return> to teardown plan"
    read FOO
    ${ACE_ROOT}/bin/dance --log-level 1 -x file://$deploymentPlan -q -k corbaname::${execManagerHost}:$DAEMON_PORT/NameService\#ExecutionManager -ORBSvcConfDirective "$ORBSvcConfDirective"
else
    echo "Error invalid argument : $BadArgEncounter. "
    echo "$SCRIPT_NAME -h for help"
fi
