/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef _INCL_CDMW_CMD_CCMDEPLOYMENTTOOL_TAODANCELOGGER_HPP_
#define _INCL_CDMW_CMD_CCMDEPLOYMENTTOOL_TAODANCELOGGER_HPP_

#if CDMW_ORB_VDR == tao


#include <ace/Log_Msg_Backend.h>
#include <ace/Log_Record.h>

#include <Foundation/logging/Logger.hpp>

namespace Cdmw
{
namespace CMD
{
namespace CcmDeploymentTool
{
    /**
     *Purpose:
     *<p>    This class allows collection of All DAnCE logs
     *
     *Features:
     *<p> Thread safe
     *
     */
    class TaoDAnCELogger : public ACE_Log_Msg_Backend
    {
    public:
        /**
         * Constructor
         */
        TaoDAnCELogger();
            
        /**
         * Destructor
         */
        ~TaoDAnCELogger() throw();

        /**
         * Initialize the event logging facility.
         * @param logger_key The name of the calling program. This name is
         *                   used as the @arg ident in the syslog entries. If
         *                   it is 0 (no name), the application name as
         *                   returned from ACE_Log_Msg::program_name() is used.
         */
        int open(const ACE_TCHAR *logger_key);

        /**
         * Reset the backend (call the close method)
         *
         * @return the status
         */
        int reset(void);

        /**
         * Close the backend completely
         *
         * @return 0
         */
        int close(void);
            
        /** 
         * This is called when we want to log a message.
         *
         * @param log_record the log to record
         *
         * @return 0
         *
         */
        int log(ACE_Log_Record &log_record);
            
    private:
        /** 
         * Copy Constructor
         */
        TaoDAnCELogger(const TaoDAnCELogger&);
        
        /** 
         * Assignement operator
         */
        void operator=(const TaoDAnCELogger&);

        /**
         * Converts a ACE-TAO log level into a CDMW level. 
         * Sets the appropriate correspondences.
         *
         * @param acelevel the initial ACE-TAO log level
         *
         * @return the corresponding CDMW level
         */
        int convert_level(int acelevel);

      
        Cdmw::Logging::Logger* m_logger;
        
      }; // End of class DAnCELogger
} // End namespace CcmDeploymentTool
} // End namespace CMD
} // End namespace Cdmw

#endif //CDMW_ORB_VDR == tao

#endif // __INCL_CDMW_CMD_CCMDEPLOYMENTTOOL_TAODANCELOGGER_HPP__
