/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef _INCL_CDMW_CMD_CCMDEPLOYMENTTOOL_TAODANCELOGGERFACTORY_HPP_
#define _INCL_CDMW_CMD_CCMDEPLOYMENTTOOL_TAODANCELOGGERFACTORY_HPP_

#if CDMW_ORB_VDR == tao

#include <DAnCE/Starter/DAnCELoggerFactory.h>

class ACE_Log_Msg_Backend;

namespace Cdmw
{
namespace CMD
{
namespace CcmDeploymentTool
{    
    class TaoDAnCELogger;

    /**
     *Purpose:
     *<p>    This class is a factory for class TaoDAnCELogger
     *
     *Features:
     *<p> Thread safe
     *
     */
    class TaoDAnCELoggerFactory : public DAnCE::DAnCELoggerFactory
    {
    public:
        /**
         * Initializes object when dynamic linking occurs.
         *
         * @param argc number of arguments
         * @param argv table of arguments
         *
         * @return the status of the initialization
         */
        virtual int init(int argc, ACE_TCHAR *argv[]);

        /**
         * Terminates object when dynamic unlinking occurs.
         *
         * @param argc number of arguments
         * @param argv table of arguments
         *
         * @return the status of the termination
         */
        virtual int fini(void);
        
        /**
         * Return a new allocated pointer to a CDMW Trace logger
         *
         * @param orb pointer on the underlying ORB
         *
         * @return the pointer
         */
        virtual ACE_Log_Msg_Backend* get_logger_backend(CORBA::ORB_ptr orb);
    
    private:
        TaoDAnCELogger* m_logger; /**< CDMW implementation of DAnCE Logger */
        bool m_has_LogCollector; /**< are traces delegated to LogCollector? */
        short m_daemon_port; /**< port used to locate the local NameService */

    }; // End of class DAnCELogger
} // End namespace CcmDeploymentTool
} // End namespace CMD
} // End namespace Cdmw

// Declare dynamic service
extern "C" ACE_Service_Object* _make_CdmwLoggerFactoryImpl ();

#endif //CDMW_ORB_VDR == tao
#endif // __INCL_CDMW_CMD_CCMDEPLOYMENTTOOL_TAODANCELOGGERFACTORY_HPP__
