/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <ace/Log_Msg_Backend.h>

#include <Foundation/common/Assert.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/logging/LogManager.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/commonsvcs/initutils/InitUtils.hpp>

#include "ccmdeploymenttool/TaoDAnCELoggerFactory.hpp"
#include "ccmdeploymenttool/TaoDAnCELogger.hpp"

#include <string>


#define CDMW_TRACE_IN_DANCE(logger, level, msg) \
    { \
        std::ostringstream __log_oss__; \
        __log_oss__ << msg; \
        ACE_Log_Record log_record; \
        log_record.type(level); \
        log_record.msg_data(__log_oss__.str().c_str()); \
        logger->log(log_record); \
    }


namespace Cdmw
{
namespace CMD
{
namespace CcmDeploymentTool
{
    int TaoDAnCELoggerFactory::init(int argc, ACE_TCHAR *argv[])
    {
        Cdmw::Logging::LogManager::Init(argc, argv);
        // Get process name from command line argument (if any).
        std::string process_name = 
           Cdmw::OsSupport::OS::get_option_value (argc, argv, "--process-name");
        if ((process_name == "no") || (process_name == "yes")) {
           // Use default process name
           process_name = "DAnCE";
        }
        Cdmw::Logging::LogManager::Instance()->set_process_name(process_name);
        std::string application_name = 
           Cdmw::OsSupport::OS::get_option_value (argc, argv, "--application-process");
        if ((application_name == "no") || (application_name == "yes")) {
           // Use default process name
           application_name = "CCMDeployment";
        }
        Cdmw::Logging::LogManager::Instance()->set_application_name(application_name);

        this->m_logger = new Cdmw::CMD::CcmDeploymentTool::TaoDAnCELogger();

        std::string str_port =
            Cdmw::OsSupport::OS::get_option_value(
                argc, argv, Common::Options::DAEMON_PORT_OPTION);
        if (! (str_port.compare("yes") && str_port.compare("no")))
        {
            this->m_has_LogCollector = true; // we don't want to retrieve it
            CDMW_TRACE_IN_DANCE(
                this->m_logger, LM_WARNING,
                "Traces will no be sent to daemon: No valid option " 
                << Common::Options::DAEMON_PORT_OPTION << " indicated");
        }
        else
        {
            this->m_has_LogCollector = false;
            this->m_daemon_port = atoi(str_port.c_str());
        }

        return 0;
    }

    int TaoDAnCELoggerFactory::fini(void)
    {
        if (! this->m_has_LogCollector)
        {            
            // Unregister LogCollector for forwarding traces to the Platform Daemon
            Cdmw::CommonSvcs::InitUtils* initUtils = 
                Cdmw::CommonSvcs::InitUtils::Instance();
            initUtils->deactivate_LogCollector();
        }

        if ( this->m_logger != 0x0 )
        {
            delete this->m_logger;
            this->m_logger = 0x0;
        }
        return 0;
    }

    ACE_Log_Msg_Backend*
    TaoDAnCELoggerFactory::get_logger_backend(CORBA::ORB_ptr orb)
    {

        // Init the traces delegation to the Daemon LogCollector.
        // This cannot be performed in init because the ORB object is missing,
        // anyway the get_logger_backend is called only one time by DAnCE.
        //
        if (! this->m_has_LogCollector)
        {
            this->m_has_LogCollector = true; // whatever the result is

            Cdmw::CommonSvcs::InitUtils* initUtils = 
                Cdmw::CommonSvcs::InitUtils::Instance();
            try
            {
                initUtils->activate_LogCollector(orb, this->m_daemon_port);
            }
            catch (const Cdmw::Exception& ex)
            {
                CDMW_TRACE_IN_DANCE(
                    this->m_logger, LM_WARNING,
                    "Traces will no be sent to daemon: " << ex.what());
            }
        }

        // create the Logger
        return this->m_logger;
    }


} // End namespace CcmDeploymentTool
} // End namespace CMD
} // End namespace Cdmw

extern "C" ACE_Service_Object* _make_CdmwLoggerFactoryImpl()\
{\
    return new Cdmw::CMD::CcmDeploymentTool::TaoDAnCELoggerFactory();\
}

