/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef _CDMW_CDR_DEPLOYMENT_PLAN_READER_HPP_
#define _CDMW_CDR_DEPLOYMENT_PLAN_READER_HPP_


#include <DAnCE/Plan_Launcher/Plan_Launcher_Base_Impl.h>

// Cdmw Files
#include <Foundation/idllib/CdmwDnC.skel.hpp>


namespace {
    // forward declaration
    class InternalReader;
}


namespace Cdmw {
namespace CCM {
namespace NodeManager {

    
    /**
     * Class for reading of DeploymentPlan files in CDR format.
     */
    class CDRDeploymentPlanReader
    {
        public:
        
            /**
             * Constructor
             *
             * @param orb ORB reference
             * @param plans_directory Path of directory where to read files
             *
             * @throw CORBA::INTERNAL if an internal error occurs initializing
             * the reader.
             */
            CDRDeploymentPlanReader(CORBA::ORB_ptr orb,
                                    const char* plans_directory)
                throw (CORBA::INTERNAL);
            
            /**
             * Destructor
             */
            virtual ~CDRDeploymentPlanReader();
            
            /**
             * Read a DeploymentPlan file in CDR format and returns
             * the DeploymentPlan structure.
             * 
             * @param deployment_plan_name The DeploymentPlan file name,
             * without '.cdr' file extension.
             * @throw CdmwDnC::PlanError is an error occurs reading the file.
             */
            virtual Deployment::DeploymentPlan* 
            read_deployment_plan(const char* deployment_plan_name)
                throw (CdmwDnC::PlanError);
        
        private:
            CDRDeploymentPlanReader();
            CDRDeploymentPlanReader(const CDRDeploymentPlanReader&);
            void operator=(const CDRDeploymentPlanReader&);
            
            const char* m_plans_directory;
            
            std::auto_ptr<InternalReader> m_internal_reader; 
        
    };    
    

 
}; // namespace NodeManager
}; // namespace CCM
}; // namespace Cdmw

#endif // _CDMW_CDR_DEPLOYMENT_PLAN_READER_HPP_

