/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_CCM_NODE_APPLICATION_IMPL_HPP
#define INCL_CCM_NODE_APPLICATION_IMPL_HPP

// Cdmw Files
#include <Foundation/idllib/CdmwDnC.skel.hpp>
#include <Foundation/orbsupport/RefCountServantBase.hpp>

#include <DAnCE/Deployment/Deployment_DomainApplicationC.h>


namespace Cdmw {

namespace CCM {

namespace NodeManager {

/**
 * Implementation of the CdmwDnC::NodeApplication interface.
 */
class NodeApplication_impl : virtual public POA_CdmwDnC::NodeApplication,
                             virtual public OrbSupport::RefCountServantBase

{

public:

    /**
     * Constructor.
     *
     * @param poa POA for object creations. 
     *            Note: poa should have RETAIN and IMPLICIT_ACTIVATION policies
     *                  (RootPOA fits)
     * @param domain_app The DomainApplication to be used
     * @param local_plan_name The DeploymentPlan name
     */
    NodeApplication_impl(
        PortableServer::POA_ptr poa,
        Deployment::DomainApplication_ptr domain_app,
        const Deployment::Connections& connections,
        const char* local_plan_name);
        
    /**
     * Destructor.
     */
    virtual ~NodeApplication_impl() throw();

    /**
     * Call DomainApplication::finishLaunch()
     *
     * @exception CdmwDnC::StartError If an error occurs during start-up
     */
    virtual void 
    finish_launch()  
        throw (CORBA::SystemException, CdmwDnC::StartError);

    /**
     * Call DomainApplication::start()
     *
     * @exception CdmwDnC::StartError If an error occurs during start-up
     */
    virtual void 
    start() 
        throw (CORBA::SystemException, CdmwDnC::StartError);

        
    /**
     * Return DomainApplicationManager object associated to this 
     * NodeApplicationManager
     *
     * @return The DomainApplicationManager object
     */
    inline Deployment::DomainApplication_ptr get_domain_application()
    {
        return Deployment::DomainApplication::_duplicate(m_domain_app.in());
    }
    

private:
    NodeApplication_impl();
    NodeApplication_impl(const NodeApplication_impl&);
    void operator=(const NodeApplication_impl&);

    PortableServer::POA_var m_poa;
    Deployment::DomainApplication_var m_domain_app;
    Deployment::Connections_var m_connections;
    CORBA::String_var m_local_plan_name;
};




} // End of namespace NodeManager

} // End of namespace CCM

} // End of namespace Cdmw

#endif  // INCL_CCM_NODE_APPLICATION_IMPL_HPP

