/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_CCM_NODE_MANAGER_IMPL_HPP
#define INCL_CCM_NODE_MANAGER_IMPL_HPP

// Cdmw Files
#include <Foundation/idllib/CdmwDnC.skel.hpp>
#include <Foundation/orbsupport/RefCountServantBase.hpp>

#include <ccmnodemanager/CDRDeploymentPlanReader.hpp>
#include <DAnCE/Deployment/Deployment_ExecutionManagerC.h>


namespace Cdmw {

namespace CCM {

namespace NodeManager {

/**
 * Implementation of the CdmwDnC::NodeManager interface.
 */
class NodeManager_impl : virtual public POA_CdmwDnC::NodeManager,
                         virtual public OrbSupport::RefCountServantBase

{

public:

    /**
     * Constructor.
     *
     * @param poa POA for object creations. 
     *            Note: poa should have RETAIN and IMPLICIT_ACTIVATION policies
     *                  (RootPOA fits)
     * @param exec_mgr The ExecutionManager to be used
     * @param plans_directory Directory containing deployment plans
     */
    NodeManager_impl(CORBA::ORB_ptr orb,
                     PortableServer::POA_ptr poa,
                     Deployment::ExecutionManager_ptr exec_mgr,
                     const char* plans_directory)
        throw (CORBA::SystemException);
                                   
    /**
     * Destructor.
     */
    virtual ~NodeManager_impl() throw();

    /**
     * Read the specified DeploymentPlan, and call ExecutionManager::preparePlan()
     *
     * @param local_plan_name DeploymentPlan filename (without file extention)
     * @return A NodeApplicationManager object for this deployment plan
     * @exception CdmwDnC::PlanError If there is a problem with the plan
     * @exception CdmwDnC::StartError If an error occurs during preparation
     */
    virtual CdmwDnC::NodeApplicationManager_ptr 
    prepare_plan(const char* local_plan_name)  
        throw (CORBA::SystemException, CdmwDnC::StartError, CdmwDnC::PlanError);

    /**
     * Call ExecutionManager::destroyManager() and deactivate 
     * NodeApplicationManager object
     *
     * @param appManager The NodeApplicationManager to be destroyed
     * @exception CdmwDnC::InvalidReference If the manager reference is unkown
     * @exception CdmwDnC::StopError If an error occurs during destruction
     */
    virtual void 
    destroy_manager(CdmwDnC::NodeApplicationManager_ptr appManager) 
        throw(CORBA::SystemException, CdmwDnC::StopError, CdmwDnC::InvalidReference);


private:
    NodeManager_impl();
    NodeManager_impl(const NodeManager_impl&);
    void operator=(const NodeManager_impl&);
    
    CORBA::ORB_var m_orb;
    PortableServer::POA_var m_poa;
    Deployment::ExecutionManager_var m_exec_mgr;
    std::auto_ptr<CDRDeploymentPlanReader> m_plan_reader;

};




} // End of namespace NodeManager

} // End of namespace CCM

} // End of namespace Cdmw

#endif  // INCL_CCM_NODE_MANAGER_IMPL_HPP

