/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <ccmnodemanager/CDRDeploymentPlanReader.hpp>
#include <ccmnodemanager/Logger.hpp>

#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>


namespace {
    static const char* PATH_SEPARATOR = "/";
    static const char* CDR_PLAN_EXTENSION = ".cdr";
    
    // DAnCE::Plan_Launcher::Plan_Launcher_Base_Impl constructor 
    // requires argc and argv.
    // Provide only a executable name
/*    static int ARGC_FOR_DANCE = 3;
    static char* ARGV_FOR_DANCE[] = { "CdmwNodeManager", "-d", "foo" };
    
    
    using namespace DAnCE::Plan_Launcher;
    // InternalReader inherits from DAnCE::Plan_Launcher_Base_Impl
    // in order to access read_cdr_plan_file() protected operation.
    // Delegation from CDRDeploymentPlanReader to InternalReader has been 
    // choosen since Plan_Launcher_Base_Impl contructor may throw
    // DAnCE exceptions
    class InternalReader :
        public virtual Plan_Launcher_Base_Impl
    {
        public:
            InternalReader(CORBA::ORB_ptr orb) 
                : Plan_Launcher_Base_Impl(orb, ARGC_FOR_DANCE, ARGV_FOR_DANCE)
            {
            }
            
            Deployment::DeploymentPlan* 
            read_deployment_plan(const char* full_plan_path)
            {
                return this->read_cdr_plan_file(full_plan_path);
            }
    };
*/    
    
    class  InternalReader 
    {
        public:
            InternalReader(CORBA::ORB_ptr orb)
            {
            }
            
            Deployment::DeploymentPlan* 
            read_deployment_plan(const char* full_plan_path)
            {
                FILE* file = ::fopen(full_plan_path, "r");
                if ( 0 == file )
                {
                    ACE_CString s = "read_cdr_plan_file : failed to open file \"";
                    s += full_plan_path;
                    s += "\"";
                    throw DAnCE::Plan_Launcher::Plan_Launcher_Base_Impl::Deployment_Failure (s.c_str());
                }
                
                size_t buf_size;
                ::fread(&buf_size, sizeof(buf_size), 1, file);
                
                char * buf = new char[buf_size];
                ::fread(buf, 1, buf_size, file);
                
                Deployment::DeploymentPlan* res = 0;
                {
                    TAO_InputCDR cdr(buf, buf_size);
                    ACE_NEW_THROW_EX (res,
                            Deployment::DeploymentPlan (),
                            CORBA::NO_MEMORY ());
                    cdr >> *res;
                }
                delete [] buf;
                ::fclose(file);
                return res;
            }
    };
    
}


namespace Cdmw {

namespace CCM {

namespace NodeManager {

    
CDRDeploymentPlanReader::CDRDeploymentPlanReader(CORBA::ORB_ptr orb,
                                                 const char* plans_directory)
    throw (CORBA::INTERNAL)
    : m_plans_directory(plans_directory)
{
    CDMW_LOG_FUNCTION(Logger::Get_Logger(), "plans_directory="<<plans_directory);
    try {
        InternalReader* reader = new InternalReader(orb);
        m_internal_reader = std::auto_ptr<InternalReader>(reader);
        
    } catch(const DAnCE::Plan_Launcher::Plan_Launcher_Base_Impl::Deployment_Failure& e) {
        CDMW_ERROR(Logger::Get_Logger(), 
            "Deployment_Failure raised initializing DAnCE library: " << e.error_);
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch(const CORBA::SystemException& e) {
        CDMW_ERROR(Logger::Get_Logger(), 
            "SystemException raised initializing DAnCE library: " 
            << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch(...) {
        CDMW_ERROR(Logger::Get_Logger(), 
            "Unknown exception raised initializing DAnCE library");
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


CDRDeploymentPlanReader::~CDRDeploymentPlanReader() 
{
    CDMW_LOG_FUNCTION(Logger::Get_Logger(), "");
}


//
// IDL:thalesgroup.com/CdmwDnC/NodeApplication/finish_launch:1.0
//
Deployment::DeploymentPlan* 
CDRDeploymentPlanReader::read_deployment_plan(const char* deployment_plan_name)
    throw (CdmwDnC::PlanError) 
{
    CDMW_LOG_FUNCTION(Logger::Get_Logger(), 
        "deployment_plan_name="<<deployment_plan_name);
        
    // retrieve plan full path
    std::string full_plan_path(m_plans_directory);
    full_plan_path += PATH_SEPARATOR;
    full_plan_path += deployment_plan_name;
    full_plan_path += CDR_PLAN_EXTENSION;
    
    // call static protected operation 
    // Plan_Launcher_Base_Impl::read_cdr_plan_file()
    using namespace DAnCE::Plan_Launcher;
    try {
        CDMW_DEBUG(Logger::Get_Logger(), 
            "Read DeploymentPlan from " << full_plan_path);
        return m_internal_reader->read_deployment_plan(full_plan_path.c_str());
        
    } catch (const Plan_Launcher_Base_Impl::Deployment_Failure& e) {
        CDMW_ERROR(Logger::Get_Logger(), 
            "Error reading DeploymentPlan from "<< full_plan_path << " : "
            << e.error_);
        CdmwDnC::PlanError ex;
        ex.name = deployment_plan_name;
        ex.reason = e.error_.c_str();
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const CORBA::SystemException& e) {
        CDMW_ERROR(Logger::Get_Logger(), 
            "Error reading DeploymentPlan from "<< full_plan_path << " : "
            << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CdmwDnC::PlanError ex;
        ex.name = deployment_plan_name;
        ex.reason = Cdmw::OrbSupport::OrbSupport::exception_to_string(e).c_str();
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (...) {
        CDMW_ERROR(Logger::Get_Logger(), 
            "Unknown error reading DeploymentPlan from "<< full_plan_path);
        CdmwDnC::PlanError ex;
        ex.name = deployment_plan_name;
        ex.reason = "Unknown exception";
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


} // End of namespace NodeManager

} // End of namespace CCM

} // End of namespace Cdmw


