/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <ccm_ft_perf/CCM_Client_impl.hpp>

#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>


namespace
{
    unsigned long LIMIT_FOR_95_PERCENT_OF_CASEE = 600;
    unsigned long LIMIT_FOR_99_PERCENT_OF_CASEE = 750;
}



//
// IDL:thalesgroup.com/CcmFtTest/ClientModule/CCM_Client:1.0
//
CcmFtTest::CCM_Client_impl::CCM_Client_impl()
    : m_session_context(CcmFtTest::ClientModule::CCM_Client_Context::_nil()),
      m_testThread()
{
}

CcmFtTest::CCM_Client_impl::~CCM_Client_impl()
{
    std::cout << "   ... CCM_Client_impl::~CCM_Client_impl called!" << std::endl;
}
     

CcmFtTest::FacetModule::CCM_FacetClientModule_ptr 
CcmFtTest::CCM_Client_impl::get_clientModule ()
    throw (CORBA::SystemException)
{
    std::cout << "   ... CCM_Client_impl::get_client()" << std::endl; 
    return CcmFtTest::FacetModule::CCM_FacetClientModule::_duplicate(this);
}


//
// attribute expectedPrimaryHostname
//
char*
CcmFtTest::CCM_Client_impl::expectedPrimaryHostname()
    throw(CORBA::SystemException)
{
    return CORBA::string_dup(m_expectedPrimaryHostname);
}

void
CcmFtTest::CCM_Client_impl::expectedPrimaryHostname(const char* a)
    throw(CORBA::SystemException)
{
    m_expectedPrimaryHostname =  CORBA::string_dup(a);
}


//
// attribute expectedBackup1Hostname
//
char*
CcmFtTest::CCM_Client_impl::expectedBackupHostname()
    throw(CORBA::SystemException)
{
    return CORBA::string_dup(m_expectedBackupHostname);
}

void
CcmFtTest::CCM_Client_impl::expectedBackupHostname(const char* a)
    throw(CORBA::SystemException)
{
    m_expectedBackupHostname =  CORBA::string_dup(a);
}

//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
CcmFtTest::CCM_Client_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_Client_impl::Set_session_context called!" << std::endl;
   
    m_session_context = ::CcmFtTest::ClientModule::CCM_Client_Context::_narrow(ctx); 
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CcmFtTest::CCM_Client_impl::ccm_activate()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_Client_impl::Ccm_activate called!" << std::endl;
   Cdmw::TestUtils::CORBATestManager::instance()->add("client_context", get_context());
   Cdmw::TestUtils::CORBATestManager::instance()->add("user_ccm_client", static_cast< ::UserCcmFtTest::User_CCM_Client_ptr > (this));
   Cdmw::TestUtils::CORBATestManager::instance()->set_property ("host_name1",expectedPrimaryHostname());
   Cdmw::TestUtils::CORBATestManager::instance()->set_property ("host_name2",expectedBackupHostname());
   m_testThread.start();
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
CcmFtTest::CCM_Client_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_Client_impl::Ccm_passivate called!" << std::endl;
} 



//
// ccm_remove function : it destroys the component (call by the container)
//
void
CcmFtTest::CCM_Client_impl::ccm_remove()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_Client_impl::Ccm_remove called!" << std::endl;
}





//
// Testable do_tests() operation
//
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(CcmFtTest::test_ccm_ft_activate, "test_ccm_ft_activate");

void
CcmFtTest::test_ccm_ft_activate::do_tests()
{
    Cdmw::OsSupport::OS::sleep(2000);
    
    try {

        CORBA::Object_var obj2 = Cdmw::TestUtils::CORBATestManager::instance()->get_object("user_ccm_client");
        ::UserCcmFtTest::User_CCM_Client_var user_ccm_client = ::UserCcmFtTest::User_CCM_Client::_narrow(obj2.in());
        m_user_ccm_client = reinterpret_cast< CcmFtTest::CCM_Client_impl* > (user_ccm_client.in());

        m_expectedPrimaryHostname = Cdmw::TestUtils::CORBATestManager::instance()->get_property ("host_name1");
        m_expectedBackupHostname = Cdmw::TestUtils::CORBATestManager::instance()->get_property ("host_name2");

        int timescale = Cdmw::TestUtils::get_timescale();

        typedef std::vector< unsigned long > TimeVector;
        TimeVector time_vec;

        TEST_INFO("get_connection_serverModule")
        ::CcmFtTest::FacetModule::FacetServerModule_var server = m_session_context->get_connection_serverModule();
        Cdmw::OsSupport::OS::sleep(timescale*1000);

        unsigned int nb_test = 100;
        for (unsigned int i = 0 ; i<nb_test; i++)
        {
            TEST_INFO("");
            TEST_INFO("");
            TEST_INFO("Start server processes "<<i);
            TEST_INFO("");
            TEST_INFO("");
            TEST_INFO("");
            TEST_INFO("");
            
            std::ostringstream proc_init11;
            std::ostringstream proc_init21;
            proc_init11<< " --proc-initialise CcmFtApplication ServerProcess1 " << m_expectedPrimaryHostname;
            proc_init21<< " --proc-initialise CcmFtApplication ServerProcess2 " << m_expectedBackupHostname;
            
            Cdmw::OsSupport::OS::create_process( "platform_admin.sh" , proc_init11.str());   
            Cdmw::OsSupport::OS::sleep(timescale*3000);
            Cdmw::OsSupport::OS::create_process( "platform_admin.sh" , proc_init21.str());   
            Cdmw::OsSupport::OS::sleep(timescale*3000);
            
            std::ostringstream proc_run11;
            std::ostringstream proc_run21;
            proc_run11<< " --proc-run CcmFtApplication ServerProcess1 " << m_expectedPrimaryHostname;
            proc_run21<< " --proc-run CcmFtApplication ServerProcess2 " << m_expectedBackupHostname;
            
            Cdmw::OsSupport::OS::create_process( "platform_admin.sh" , proc_run11.str());   
            Cdmw::OsSupport::OS::create_process( "platform_admin.sh" , proc_run21.str());
            Cdmw::OsSupport::OS::sleep(timescale*3000);
 

            TEST_INFO("Call kill primary on the primary")
            try
            {
                server->kill_primary();
            }
            catch(...)
            {
            }
            TEST_INFO("End of call kill primary on the primary")
            Cdmw::OsSupport::OS::sleep(timescale*2000);

            TEST_INFO("");
            TEST_INFO("Stopr server processes "<<i);
            TEST_INFO("");


            std::ostringstream proc_stop11;
            std::ostringstream proc_stop21;
            proc_stop11<< " --proc-stop CcmFtApplication ServerProcess1 " << m_expectedPrimaryHostname;
            proc_stop21<< " --proc-stop CcmFtApplication ServerProcess2 " << m_expectedBackupHostname;
            Cdmw::OsSupport::OS::create_process( "platform_admin.sh" , proc_stop11.str());   
            Cdmw::OsSupport::OS::create_process( "platform_admin.sh" , proc_stop21.str());
            Cdmw::OsSupport::OS::sleep(timescale*3000);

            TEST_INFO("test n°"<<i);
            TEST_INFO("elapse time : "<<m_user_ccm_client->get_elapse()<<" ms");
            time_vec.push_back(m_user_ccm_client->get_elapse());
            
            
        }

        
        

        if (time_vec.size() != nb_test)
            CPPUNIT_ASSERT(false);

        unsigned long max = time_vec[0];
        unsigned long min = time_vec[0];
        unsigned long ave = 0;
        int nm_test_over_the_limit_for_95 = 0;
        int nm_test_over_the_limit_for_99 = 0;
        
        for(unsigned int i = 0; i<nb_test; i++)
        {
            ave = time_vec[i] + ave;
            if (time_vec[i] > max)
                max = time_vec[i];
            if (time_vec[i] < min)
                min = time_vec[i];
            if (time_vec[i] > LIMIT_FOR_95_PERCENT_OF_CASEE)
                nm_test_over_the_limit_for_95++;
            if (time_vec[i] > LIMIT_FOR_99_PERCENT_OF_CASEE)
                nm_test_over_the_limit_for_99++;
        }
        
        ave = ave/nb_test;

        TEST_INFO("------------------------------");
        TEST_INFO("result : ");
        TEST_INFO("max : " <<max<< " ms");
        TEST_INFO("min : " <<min<< " ms");
        TEST_INFO("average : " <<ave<< " ms");
        TEST_INFO("nb case under "<< LIMIT_FOR_95_PERCENT_OF_CASEE <<" ms :"<<((nb_test - nm_test_over_the_limit_for_95)/nb_test)*100<<"%");
        TEST_INFO("------------------------------");
        TEST_INFO("nb case under "<< LIMIT_FOR_99_PERCENT_OF_CASEE <<" ms :"<<((nb_test - nm_test_over_the_limit_for_99)/nb_test)*100<<"%");
        TEST_INFO("------------------------------");


    } catch (const CORBA::SystemException& e) {
        CPPUNIT_ASSERT(true);
    } catch (const CORBA::Exception& e) {
        TEST_INFO("Unexpected CORBA::Exception: " << e);
        CPPUNIT_ASSERT(false);
    }
}




