/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <sstream>

#include <ccm_ft_request_identification/CCM_ServerWithFacet_impl.hpp>

#include <Foundation/ossupport/OS.hpp>
#include "CCMContainer/idllib/CdmwComponents.stub.hpp"
#include <FaultTolerance/idllib/CdmwFTCurrent.stub.hpp>
//
// IDL:thalesgroup.com/CcmFtTest/CCM_ServerWithFacet:1.0
//
CcmFtTest::CCM_ServerWithFacet_impl::CCM_ServerWithFacet_impl()
{
}

CcmFtTest::CCM_ServerWithFacet_impl::~CCM_ServerWithFacet_impl()
{
    std::cout << "   ... CCM_ServerWithFacet_impl::~CCM_ServerWithFacet_impl called!" << std::endl;
}


//
// facet serverRequestInfo
//
CcmFtTest::FacetModule::CCM_RequestInfo_ptr 
CcmFtTest::CCM_ServerWithFacet_impl::get_serverRequestInfo ()
    throw (CORBA::SystemException)
{
    std::cout << "   ... CCM_ServerWithFacet_impl::get_serverRequestInfo()" << std::endl; 
    return CcmFtTest::FacetModule::CCM_RequestInfo::_duplicate(this);
}

char*
CcmFtTest::CCM_ServerWithFacet_impl::get_request_id()
    throw (CORBA::SystemException)
{
    
    CdmwFT::Current_var ftcurrent;
    try
    {
        CdmwComponents::SessionContextExt_var sc_ext 
           = CdmwComponents::SessionContextExt::_narrow(m_session_context.in());
        
        if (!CORBA::is_nil(sc_ext.in()))
        {
            CORBA::Object_var obj = sc_ext->get_service("FTCurrent");
            ftcurrent = CdmwFT::Current::_narrow(obj.in());
            
            std::cout << "get service FTCurrent OK" << std::endl;
        }
    }
    catch (const CdmwComponents::CCMContextExt::ServiceUnavailable& )
    {
        std::cerr << "ServiceUnavailable Exception raised when getting Service"
                  << std::endl;
        throw;
    }
    catch (const CORBA::SystemException&)
    {
        std::cerr << "System Exception raised when getting Service" 
                  << std::endl;
        throw;
    }
    

    char* client_id = ftcurrent->get_client_id();
    long retention_id = ftcurrent->get_retention_id();

    std::ostringstream id_stream;
    id_stream << client_id << "/" << retention_id;
    std::cout<<"Return client_id/retention_id :"<<id_stream.str()<<std::endl;
    CORBA::String_var result = id_stream.str().c_str();
    return result._retn();
}    


//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
CcmFtTest::CCM_ServerWithFacet_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_ServerWithFacet_impl::Set_session_context called!" << std::endl;
   
    m_session_context = ::CcmFtTest::ServerModule::CCM_ServerWithFacet_Context::_narrow(ctx); 
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CcmFtTest::CCM_ServerWithFacet_impl::ccm_activate()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_ServerWithFacet_impl::Ccm_activate called!" << std::endl;
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//

void
CcmFtTest::CCM_ServerWithFacet_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_ServerWithFacet_impl::Ccm_passivate called!" << std::endl;
} 



//
// ccm_remove function : it destroys the component (call by the container)
void
CcmFtTest::CCM_ServerWithFacet_impl::ccm_remove()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_ServerWithFacet_impl::Ccm_remove called!" << std::endl;
}


