/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef ___DINNER_CCM_PHILOSOPHER_IMPL_HPP__
#define ___DINNER_CCM_PHILOSOPHER_IMPL_HPP__

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/osthreads/ThreadHandle.hpp>
#include "Foundation/osthreads/Mutex.hpp"
#include <Foundation/testutils/CORBATestManager.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>

#include <ccm_philo/UserPhilo.skel.hpp>

namespace Dinner
{

// forward declaration
class CCM_Philosopher_impl;
class PhiloActivationThread;
class test_ccm_philosopher_activate;

enum {
    MAX_NB_MEAL = 3
};

//
// IDL:thalesgroup.com/Dinner/CCM_Philosopher:1.0
//
class CCM_Philosopher_impl : virtual public ::UserDinner::User_CCM_Philosopher,
                        virtual public Cdmw::OrbSupport::RefCountLocalObject
{
     friend class Dinner::test_ccm_philosopher_activate;

    CCM_Philosopher_impl(const CCM_Philosopher_impl&);
    void operator=(const CCM_Philosopher_impl&);

    CORBA::String_var                     m_name;
    volatile bool                         m_is_active;
    CORBA::ULong                          m_state;
    CORBA::ULong                          m_speed;
    bool                                  m_have_left_fork;
    bool                                  m_have_right_fork;
    ::Dinner::CCM_Philosopher_Context_var m_context;

     PhiloActivationThread*               m_activation_thread;
     Cdmw::OsSupport::Mutex               m_mutex;

     
     
public:

    CCM_Philosopher_impl();
    ~CCM_Philosopher_impl();
    //
    // IDL:thalesgroup.com/Dinner/CCM_Philosopher_Executor/name:1.0
    //
    virtual char* name()
        throw(CORBA::SystemException);
     

    void set_session_context(Components::SessionContext_ptr ctx)
                 throw(CORBA::SystemException);
    void ccm_activate()
                 throw(CORBA::SystemException);
    void ccm_passivate()
                 throw(CORBA::SystemException);
    void ccm_remove()
                 throw(CORBA::SystemException);
;

 // additional operation
    void push_state();

    bool is_active()
     {
         m_mutex.lock();
         bool result = m_is_active;
         m_mutex.unlock();
         return result;
     }

     void set_name(const char* name)
     {
         m_mutex.lock();
         m_name = CORBA::string_dup(name);
         m_mutex.unlock();
     }

     ::Dinner::CCM_Philosopher_Context* get_context()
     {
         return ::Dinner::CCM_Philosopher_Context::_duplicate(m_context.in());
     }

private:
         /**
      * Purpose:
      *<p> Set the speed according to the name of the philosopher.
      *@param name The name of the philosopher.
      */
         void set_speed(const char*);
};
/**
*  This class provides an object to test the ccm_activate.
*
**/

class test_ccm_philosopher_activate : public CppUnit::TestFixture
{
    Dinner::CCM_Philosopher_impl* m_philosopher;
    
public:
    test_ccm_philosopher_activate()
    {
        // get & remove object from CORBATestManager since it will be
        // destroyed only at summary, ie after unload of library.
        // (core will be dump at process end if not removed).
        
        std::ostringstream oss; 
        oss << "philosopher" << M_counter++; 
        std::string regName = oss.str();
        
        std::cout << std::endl << "++++++++++++++++++++++++++++++++ Remove object " << regName << std::endl << std::endl;
        CORBA::Object_var obj = Cdmw::TestUtils::CORBATestManager::instance()->remove_object(regName);
// it crashes when deploying with DAnCE so back in time...
        
        //CORBA::Object_var obj = Cdmw::TestUtils::CORBATestManager::instance()->get_object("philosopher");
        ::UserDinner::User_CCM_Philosopher_var user_ccm_philo = ::UserDinner::User_CCM_Philosopher::_narrow(obj.in());
        m_philosopher  = reinterpret_cast< ::Dinner::CCM_Philosopher_impl* > (user_ccm_philo.in());  
        
        std::cout << std::endl << "+++++++TEST+++++++++++++++++++++ m_philosopher == " << m_philosopher << std::endl << std::endl;
    };
    ~test_ccm_philosopher_activate() {};
    
    
    void do_tests();
    CPPUNIT_TEST_SUITE( test_ccm_philosopher_activate );
    CPPUNIT_TEST( do_tests );
    CPPUNIT_TEST_SUITE_END();

private:
    static int M_counter;
};

static Cdmw::OsSupport::Mutex               m_mutex_PhiloActivationThread;

class PhiloActivationThread : public Cdmw::OsSupport::Thread 
{
public:
    PhiloActivationThread(Dinner::CCM_Philosopher_impl* philosopher)
         : m_philosopher(philosopher)
     {
     }

    ~PhiloActivationThread() throw() {};

protected:
    void
    run() throw()
     {
        Cdmw::TestUtils::ThreadedTestRunner runner(Cdmw::TestUtils::CORBATestManager::instance()->get_controller());
        {
            CDMW_MUTEX_GUARD(m_mutex_PhiloActivationThread);
            std::ostringstream oss; 
            oss << "philosopher" << M_counter++; 
            std::string regName = oss.str();
            
            std::cout << std::endl << "++++++++++++++++++++++++++++++++ Register as " << regName << std::endl << std::endl;
            Cdmw::TestUtils::CORBATestManager::instance()->add( regName, static_cast< ::UserDinner::User_CCM_Philosopher_ptr > (m_philosopher));
            
            
            std::cout << std::endl << "++++++++THREAD+++++++++++++++++ m_philosopher == " << m_philosopher << std::endl << std::endl;
            
            runner.addTest( CppUnit::TestFactoryRegistry::getRegistry("test_ccm_philosopher_activate").makeTest() );
        }
        runner.start();
        runner.join();
     }
    
private:

    static int M_counter;
    PhiloActivationThread();
    PhiloActivationThread(const PhiloActivationThread&);
    PhiloActivationThread& operator = (const PhiloActivationThread&);
    
    Dinner::CCM_Philosopher_impl* m_philosopher;

    /**
     * Purpose:
     *<p> Set the speed according to the name of the philosopher.
     *@param name The name of the philosopher.
     *@return The speed of the philosopher.
     */
    void set_speed(const char*);

};

}; // End of namespace Dinner


#endif // ___DINNER_CCM_PHILOSOPHER_IMPL_HPP__

