/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

// Unix Files
#include <stdlib.h>


// Cdmw Files
#include <Foundation/common/Assert.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <Foundation/ossupport/OS.hpp>


#include <ccm_philo/CCM_Philosopher_impl.hpp>
#include <ccm_philo/StatusInfo_impl.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtManagedElementContexts.stub.hpp>
#include <Foundation/logging/CdmwLogging.stub.hpp>
#include <Foundation/logging/Logger.hpp>



namespace 
{
    
void MySrand ()
{
  static bool initialized = false;

  if (!initialized) 
  {
      Cdmw::OsSupport::OS::Timeval tv 
          = Cdmw::OsSupport::OS::get_time ();
      Cdmw::OsSupport::OS::ProcessId pid 
          = Cdmw::OsSupport::OS::get_processId();
    ::srand ((unsigned int) (tv.microseconds + pid) % 991);
    initialized = true;
  }
}

int MyRand ()
{
  MySrand ();
  return (int) rand ();
}

} // End of anonymous namespace

/**
*  This class provides an object to test the ccm_activate.
*
**/
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(Dinner::test_ccm_philosopher_activate, "test_ccm_philosopher_activate");

int Dinner::test_ccm_philosopher_activate::M_counter = 0;
int Dinner::PhiloActivationThread::M_counter = 0;

void 
Dinner::test_ccm_philosopher_activate::do_tests()
{
    CDMW_ASSERT(m_philosopher);

    TEST_INFO("********************************************");
    TEST_INFO("            ccm_activate start");
    TEST_INFO("********************************************");

    ::Dinner::CCM_Philosopher_Context_var philosopher_context = m_philosopher->get_context();
    
    TEST_INFO("Philo: Wait all Forks to be created...");
    bool fork_ready = false;
    while (!fork_ready) {
        try {
            Dinner::Fork_var left = 
                philosopher_context->get_connection_left();
            Dinner::Fork_var right = 
                philosopher_context->get_connection_right();
            
            fork_ready = true;
        } catch (const CORBA::TRANSIENT& e) {
            TEST_INFO(" ... fork not yet ready ...");
            Cdmw::OsSupport::OS::sleep(3000);
        }
    }

    ::Dinner::Registration_var register_server;

    
    /*TEST_INFO("Philo: Testing ManagedComponentContext QoS Service");
    CORBA::String_var real_component_name;
    try {
        // get ManagedComponentContext
        CORBA::Object_var obj = 
            philosopher_context->resolve_service_reference("ManagedComponentContext");
        CdmwPlatformMngt::ManagedElementContexts::ComponentContext_var supContext =
            CdmwPlatformMngt::ManagedElementContexts::ComponentContext::_narrow(obj.in());
        
        // get component, process, application names
        real_component_name = supContext->managed_element_name();
        CORBA::String_var process_name = supContext->process_name();
        CORBA::String_var appli_name = supContext->application_name();
        TEST_INFO("--- Philosopher info from ManagedComponentContext:");
        TEST_INFO("  -     process name = " << process_name.in());
        TEST_INFO("  - application name = " << appli_name.in());
        TEST_INFO("  -   component name = " << real_component_name.in());

        // notify a message to Supervision
        std::string msg = "Greetings from Dr. ";
        msg += real_component_name.in();
        supContext->notify_message(CdmwPlatformMngtBase::INF,
                                   msg.c_str());
        
    } catch (const Components::CCMException& e) {
        TEST_INFO("Philo: ManagedComponentContext no found !!");
        CPPUNIT_ASSERT(false);
    }

    TEST_INFO("Philo: Testing ManagedProcessContext QoS Service");
    try {
        // get ManagedProcessContext
        CORBA::Object_var obj = 
            philosopher_context->resolve_service_reference("ManagedProcessContext");
        CdmwPlatformMngt::ManagedElementContexts::ProcessContext_var supContext =
            CdmwPlatformMngt::ManagedElementContexts::ProcessContext::_narrow(obj.in());*/
            
/* // WARNING PB with this test: since this process is not FT
   // there is no PrimaryBackupGroupRepository local object
   // thus, the resolve_initial_reference for this object leads
   // to TAO multicast discovery process, blocking the process
   // during a few seconds.
   
        // get RedundancyState
        CdmwPlatformMngt::RedundancyEltState state =
            supContext->get_process_redundancy_state();
        TEST_INFO("--- Philosopher info from ManagedProcessContext:");
        switch (state) {
            case CdmwPlatformMngt::REDSTATE_PRIMARY:
                TEST_INFO("  - state = PRIMARY");
                break;
            case CdmwPlatformMngt::REDSTATE_PASSIVE:
                TEST_INFO("  - state = BACKUP");
                break;
            case CdmwPlatformMngt::REDSTATE_NORG:
                TEST_INFO("  - state = NOT FT");
                break;
            default:
                TEST_INFO("  - state = ??? SHOULD'NT HAPPEN !!");
                break;
        }
*/        
/*        // notify a message to Supervision
        std::string msg = "The school accepts the Dr. ";
        msg += real_component_name.in();
        supContext->notify_message(CdmwPlatformMngtBase::INF,
                                   msg.c_str());
        
    }  catch (const Components::CCMException& e) {
        TEST_INFO("Philo: ManagedProcessContext no found !!");
        CPPUNIT_ASSERT(false);
    }
    
    TEST_INFO("Philo: Testing LoggerFactory QoS Service");
    try {
        // get LoggerFactory
        CORBA::Object_var obj = 
            philosopher_context->resolve_service_reference("LoggerFactory");
        CdmwLogging::LoggerFactory_var loggerFactory =
            CdmwLogging::LoggerFactory::_narrow(obj.in());
        // get Logger for DINNER_DOMAIN
        CdmwLogging::Logger_var logger =
            loggerFactory->get_logger("DINNER_DOMAIN");
        
        // log a message
        CDMW_INFO(logger, "Test logging from component");
        
    } catch (const Components::CCMException& e) {
        TEST_INFO("Philo: LoggerFactory no found !!");
        CPPUNIT_ASSERT(false);
    }*/
    
    
    TEST_INFO("Philo: Get_connection_to_register...");
    bool test_ok = false;
    try
    {
        register_server = philosopher_context->get_connection_register();

        if (!CORBA::is_nil(register_server.in()))
        {
            test_ok = true;
        }
    }
    catch (...)
    {
        TEST_INFO("Philo: Unexpected exception raised!");
    }
    CPPUNIT_ASSERT(test_ok);

    // register philosopher
    TEST_INFO("Philo: Register philosopher...");
    test_ok = false;
    try
    {
        if (!CORBA::is_nil(register_server.in()))
        {
            CORBA::String_var philosopher_name;
            
            bool isNameSet = false;
            while (!isNameSet) {
                try {
                    philosopher_name = register_server->_cxx_register();
                    isNameSet = true;
                }
                catch (const CORBA::TRANSIENT& e) {
                    std::cout << "Exception raised: " << e << std::endl;
                    TEST_INFO("register_server->_cxx_register not ready yet, retrying...");
                    Cdmw::OsSupport::OS::sleep(3000);
                }
            }
            m_philosopher->set_speed(philosopher_name.in());
            m_philosopher->set_name(philosopher_name.in());

            std::cout << "Philo: "  << philosopher_name.in() << " is active, speed is " << m_philosopher->m_speed << std::endl;
            test_ok = true;
        }
    }
    catch (const CORBA::SystemException& e)
    {
        std::cout << "System Exception raised: " << e << std::endl;
    }
    catch (const CORBA::Exception& e)
    {
        std::cout << "CORBA Exception raised: " << e << std::endl;
    }
    catch (...)
    {
        TEST_INFO("Philo: Unexpected exception raised!");
    }

    CPPUNIT_ASSERT(test_ok);

    // push config from emitter port 
    TEST_INFO("Philo: push_config using emitter port...");
    test_ok = false;
    try
    {
        Dinner::StatusInfo_var si = new StatusInfo_impl;
        si->name (m_philosopher->m_name.in());
        si->state (Dinner::THINKING);
        si->secs_since_last_meal (m_philosopher->m_state);
        si->has_left_fork (m_philosopher->m_have_left_fork);
        si->has_right_fork (m_philosopher->m_have_right_fork);

        m_philosopher->m_context->push_config (si.in());
        test_ok = true;
    }
    catch (...)
    {
        TEST_INFO("Philo: Unexpected exception raised!");
    }
    CPPUNIT_ASSERT(test_ok);

     
    //get the connection to the server 
    TEST_INFO("Philo: Connect to the fork facet...");
    Dinner::Fork_var left;
    Dinner::Fork_var right;
    test_ok = false;
    try
    {
        left = philosopher_context->get_connection_left();
        right = philosopher_context->get_connection_right();

        if ( (!CORBA::is_nil(left.in())) && (!CORBA::is_nil(right.in())) )
        {
            test_ok = true;
        }
        else
        {
            TEST_INFO("Philo: Facet reference is nil!");
        }
    }
    catch (...)
    {
        TEST_INFO("Philo: Unexpected exception raised!");
    }
    CPPUNIT_ASSERT(test_ok);

    int nb_meal = 0;
    // Let philosopher try to eat
    try
    {
        //we stop after 3 sucessful meal
        while ((m_philosopher->is_active()) && (nb_meal < 3))
        {
            std::cout << "++++++++++++++++++++++++Philo: " << m_philosopher->m_name.in() << " I am " << this << std::endl;
            std::cout << "++++++++++++++++++++++++Philo: " << m_philosopher->m_name.in() << " my philosopher is " << m_philosopher << std::endl;
            std::cout << "++++++++++++++++++++++++Philo: " << m_philosopher->m_name.in() << " my m_is_active is " << m_philosopher->m_is_active << std::endl;
            std::cout << "++++++++++++++++++++++++Philo: " << m_philosopher->m_name.in() << " my m_is_active is at " << &(m_philosopher->m_is_active) << std::endl;
            
            int timescale = Cdmw::TestUtils::get_timescale();
            Cdmw::OsSupport::OS::sleep(timescale*m_philosopher->m_speed);
            
            if (!m_philosopher->is_active())
            {
                // philosopher has been desactivated: get out from loop
                break;
            }
        
            if (m_philosopher->m_have_left_fork && m_philosopher->m_have_right_fork) 
            {
                /*
                 * Eating
                 */
                std::cout << "++++++++++++++++++++++++Philo: " << m_philosopher->m_name.in() << " Eating" << std::endl;
                if (m_philosopher->m_state > 3) 
                {
                    m_philosopher->m_state -= 3;
                    std::cout << "++++++++++++++++++++++++Philo: " << m_philosopher->m_name.in() << " push_state " << std::endl;
                    m_philosopher->push_state ();
                    std::cout << "++++++++++++++++++++++++Philo: " << m_philosopher->m_name.in() << " after push_state " << std::endl;
                    continue;
                }
                m_philosopher->m_state = 0;
                
                try 
                {
                    std::cout << "++++++++++++++++++++++++Philo: " << m_philosopher->m_name.in() << " release left " << std::endl;
                    left->release ();
                    m_philosopher->m_have_left_fork = false;
                    std::cout << "++++++++++++++++++++++++Philo: " << m_philosopher->m_name.in() << " release right " << std::endl;
                    right->release ();
                    m_philosopher->m_have_right_fork = false;
                    std::cout << "++++++++++++++++++++++++Philo: " << m_philosopher->m_name.in() << " release over " << std::endl;
                } 
                catch (...) 
                {
                }
                std::cout << "Philo: " << m_philosopher->m_name.in() << " has eaten." << std::endl;
                
                nb_meal++;
                
                m_philosopher->push_state ();
                std::cout << "++++++++++++++++++++++++Philo: " << m_philosopher->m_name.in() << " after push eaten " << std::endl;
                continue;
            }
            
            m_philosopher->m_state++;
            
            if (m_philosopher->m_state < 3) 
            {
                std::cout << "Philo: " << m_philosopher->m_name.in() << " is thinking." << std::endl;
                m_philosopher->push_state ();
                std::cout << "++++++++++++++++++++++++Philo: " << m_philosopher->m_name.in() << " after push think" << std::endl;
                continue;
            }
            
            if (m_philosopher->m_state == 3) 
            {
                std::cout  << "Philo: "<< m_philosopher->m_name.in() << " is becoming hungry." << std::endl;
            }
            else if (m_philosopher->m_state >= 10) 
            {
                std::cout  << "Philo: "<< m_philosopher->m_name.in() << " is starving." << std::endl;
            }
            
            if (!m_philosopher->m_have_left_fork) 
            {
                try 
                {
                    std::cout << "++++++++++++++++++++++++Philo: " << m_philosopher->m_name.in() << " try get left " << std::endl;
                    left->get ();
                    std::cout << "++++++++++++++++++++++++Philo: " << m_philosopher->m_name.in() << " after try get left " << std::endl;
                    m_philosopher->m_have_left_fork = true;
                }
                catch (const Dinner::InUse& ) 
                {
                    std::cout  << "Philo: "<< m_philosopher->m_name.in() << " has its left fork used." << std::endl;
                }
                catch (const CORBA::Exception&)
                {
                }
                
                if (m_philosopher->m_have_left_fork) 
                {
                    std::cout  << "Philo: "<< m_philosopher->m_name.in() << " has got the left fork." << std::endl;
                    m_philosopher->push_state ();
                    std::cout  << "++++++++++++++++++++++++Philo: "<< m_philosopher->m_name.in() << " has got the left fork. after push" << std::endl;
                    continue;
                }
                else 
                {
                    // std::cout << _name << " did not get the left fork." << std::endl;
                }
            }
            

            if (!m_philosopher->m_have_right_fork) 
            {
                try 
                {
                    std::cout << "++++++++++++++++++++++++Philo: " << m_philosopher->m_name.in() << " try get right " << std::endl;
                    right->get ();
                    std::cout << "++++++++++++++++++++++++Philo: " << m_philosopher->m_name.in() << " after try get right " << std::endl;
                    m_philosopher->m_have_right_fork = true;
                }
                catch (const Dinner::InUse& ) 
                {
                    std::cout  << "Philo: "<< m_philosopher->m_name.in() << " has its right fork used." << std::endl;
                }
                catch (const CORBA::Exception&)
                {
                }
                
                if (m_philosopher->m_have_right_fork) 
                {
                    std::cout  << "Philo: "<< m_philosopher->m_name.in() << " has got the right fork." << std::endl;
                    m_philosopher->push_state ();
                    std::cout  << "++++++++++++++++++++++++Philo: "<< m_philosopher->m_name.in() << " has got the right fork. after push" << std::endl;
                    continue;
                }
                else 
                {
                    // std::cout << m_philosopher->m_name.in() << " did not get the right fork." << std::endl;
                }
            }
            
            if (((m_philosopher->m_have_left_fork && !m_philosopher->m_have_right_fork) ||
                 (!m_philosopher->m_have_left_fork && m_philosopher->m_have_right_fork)) && ((rand() % 3) == 0)) 
            {
                if (m_philosopher->m_have_left_fork) 
                {
                    try 
                    {
                        std::cout << "++++++++++++++++++++++++Philo: " << m_philosopher->m_name.in() << " try release left " << std::endl;
                        left->release ();
                        std::cout << "++++++++++++++++++++++++Philo: " << m_philosopher->m_name.in() << " after try release left " << std::endl;
                        m_philosopher->m_have_left_fork = false;
                        std::cout  << "Philo: "<< "Deadlock prevention! " << m_philosopher->m_name.in()
                                   << " drops the left fork." << std::endl;
                    }
                    catch (...) 
                    {
                    }
                }
                if (m_philosopher->m_have_right_fork) 
                {
                    try 
                    {
                        std::cout << "++++++++++++++++++++++++Philo: " << m_philosopher->m_name.in() << " try release right " << std::endl;
                        right->release ();
                        std::cout << "++++++++++++++++++++++++Philo: " << m_philosopher->m_name.in() << " after try release right " << std::endl;
                        m_philosopher->m_have_right_fork = false;
                        std::cout  << "Philo: "<< "Deadlock prevention! " << m_philosopher->m_name.in()
                                   << " drops the right fork." << std::endl;
                    }
                    catch (...) 
                    {
                    }
                }
            }
            
            std::cout << "++++++++++++++++++++++++Philo: " << m_philosopher->m_name.in() << " endloop before push " << std::endl;
            m_philosopher->push_state ();
            std::cout << "++++++++++++++++++++++++Philo: " << m_philosopher->m_name.in() << " enddloop after push" << std::endl;
        } // End of while (is_active)
        std::cout << "-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=" << std::endl;
        std::cout << "             End of dinner for " << m_philosopher->m_name.in() << std::endl;
        std::cout << "-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=" << std::endl;
        CPPUNIT_ASSERT(true);
    }
    catch (const CORBA::SystemException& e)
    {
        std::cout << "System Exception raised: " << e << std::endl;
    }
    catch (const CORBA::Exception& e)
    {
        std::cout << "CORBA Exception raised: " << e << std::endl;
    }
} // End of do_test()

    

//
// IDL:acme.com/Dinner/CCM_Philosopher:1.0
//
Dinner::CCM_Philosopher_impl::CCM_Philosopher_impl()
    : m_name(""),
      m_is_active(false),
      m_state(0),
      m_have_left_fork(false),
      m_have_right_fork(false),
      m_context(Dinner::CCM_Philosopher_Context::_nil())
{
//    m_speed = 1500 + (MyRand() % 13) * 300; - PCR 0941 with a random,
//  in some cases the speed can be the same.

      m_activation_thread = new PhiloActivationThread(this);
}

Dinner::CCM_Philosopher_impl::~CCM_Philosopher_impl()
{
    std::cout << "CCM_Philosopher_impl::~CCM_Philosopher_impl called!" << std::endl;
}

void Dinner::CCM_Philosopher_impl::set_speed(const char* name){


  const unsigned int number_philosophers = 12; 

  enum {NAME = 0, SPEED = 1};

  CDMW_MUTEX_GUARD(m_mutex);

  const char * philosophers[2][number_philosophers] = {
    {"Socrates", "Descartes", "Pascal", "Plato", "Aristotle", "Pythagoras", "Zeno",
     "Heraclitus", "Democritus", "Descartes", "Kant", "Protagoras"},
    {
      "2700" /*Socrates*/, "3600" /*Descartes*/, "1800"/*Pascal*/, "4800"/*Plato*/, 
      "2400" /*Aristotle*/, "3200" /*Pythagoras*/, "1000"/*Zeno*/,"4200" /*Heraclitus*/,
      "1400"/*Democritus*/, "3500"/*Descartes*/, "5100"/*Kant*/, "2200"/*Protagoras*/
    } 
      };

  m_speed = 0;
  for(unsigned long i = 0; i < number_philosophers; i++){
    if(!strcmp(philosophers[NAME][i], name)){
    m_speed = atol(philosophers[SPEED][i]);
    break;
    }
  }
  
  if(!m_speed)
    std::cerr << __FILE__ << "Speed of " << name << " " << "speed not found !" << __LINE__ << std::endl;

}

//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
Dinner::CCM_Philosopher_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    std::cout  << "Philo: "<< "Set_session_context called!" << std::endl;
   
    m_context = ::Dinner::CCM_Philosopher_Context::_narrow(ctx); 
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
Dinner::CCM_Philosopher_impl::ccm_activate()
   throw(CORBA::SystemException)
{
   if (! m_is_active)
    {
        m_is_active = true;
std::cout << "+++++CCM_ACTIVATE+++++++Philo: " << m_name.in() << " I am " << this << std::endl;
std::cout << "+++++CCM_ACTIVATE+++++++Philo: " << m_name.in() << " my thread is " << m_activation_thread << std::endl;
        
        m_activation_thread->start();
    }
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//

void
Dinner::CCM_Philosopher_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
   std::cout  << "Philo: "<< m_name.in() << "Ccm_passivate called!" << std::endl;

   if (!is_active()) 
   {
       return;
   }

   m_is_active = false;

   std::cout  << "Philo: "<< m_name.in() << " is inactive." << std::endl;
   std::cout << "++++++++++++++++++++++++Philo: " << m_name.in() << " I am " << this << std::endl;
   std::cout << "++++++++++++++++++++++++Philo: " << m_name.in() << " m_is_active == " << m_is_active << std::endl;
   std::cout << "++++++++++++++++++++++++Philo: " << m_name.in() << " try to join " << m_activation_thread << std::endl;
   
    try
    {
       m_activation_thread->join();
    }
    catch (const Cdmw::OsSupport::DeadlockException& )
    {
        std::cout  << "Deadlock Exception raised!" << std::endl;
    }
    catch (const Cdmw::InternalErrorException& )
    {
        std::cout  << "InternalError Exception raised!" << std::endl;
    }

    std::cout  << "Philo: "<< m_name.in() << " thread has been joined" << std::endl;
} 



//
// ccm_remove function : it destroys the component (call by the container)
void
Dinner::CCM_Philosopher_impl::ccm_remove()
   throw(CORBA::SystemException)
{
   std::cout  << "Philo: "<< "Ccm_remove called!" << std::endl;
}

// push status info event
void 
Dinner::CCM_Philosopher_impl::push_state ()
{
    Dinner::StatusInfo_var si = new StatusInfo_impl;
    si->name (m_name.in());
    
    if (m_have_left_fork && m_have_right_fork) 
    {
         si->state (Dinner::EATING);
    }
    else if (m_state < 3) 
    {
         si->state (Dinner::THINKING);
    }
    else if (m_state < 10) 
    {
         si->state (Dinner::HUNGRY);
    }
    else if (m_state < 40) 
    {
         si->state (Dinner::STARVING);
    }
    else 
    {
         si->state (Dinner::DEAD);
    }

    si->secs_since_last_meal (m_state);
    si->has_left_fork (m_have_left_fork);
    si->has_right_fork (m_have_right_fork);

    m_context->push_info (si.in());
}


//
// IDL:acme.com/Dinner/CCM_Philosopher_Executor/name:1.0
//
char*
Dinner::CCM_Philosopher_impl::name()
    throw(CORBA::SystemException)
{
    char* name = CORBA::string_dup(m_name);
    return name;
}

