/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Foundation/idllib/CdmwDnC.stub.hpp>

#include <ccm_testnodemanager/TestCcmNodeManager_IDL2.stub.hpp>

#include <sstream>


namespace {
    const int SUCCESS = 0;
    const int FAILURE = 1;

    const int POA_THREAD_POOL_SIZE = 5;    
    
    const std::string NODE_MANAGER_CORBALOC = 
        "corbaloc::localhost:65432/CdmwNodeManager";
    
    const std::string NODE_MANAGER_NAMING_CONTEXT_CORBALOC = 
        "corbaloc::localhost:65432/NameService";
    
    const std::string NODE_MANAGER_NAMING_CONTEXT_BINDING_NAME = 
        "DeploymentPlans";
    
    const std::string COMPONENT_NAMING_PATH =
        "V-CMD-004-1.DeploymentPlan/Server.Component";
    
    const std::string COMPONENT_FACET_NAMING_PATH =
        "V-CMD-004-1.DeploymentPlan/Server/hello_facet.Port";
    
}; // End anonymous namespace



class TestNodeManager : public CppUnit::TestFixture
{
    public:

    TestNodeManager()
    {
    };

    virtual ~TestNodeManager() throw()
    {
    };


    void do_tests()
    {
        TEST_INFO("TestNodeManager::do_tests");
        try {
            // get ORB
            CORBA::ORB_var orb = 
                Cdmw::TestUtils::CORBATestManager::instance()->get_ORB();
            
            // REQ-SRS-CMD-194: 
            // Test NodeManager is accessible through corbaloc
            TEST_INFO("Get NodeManager via " << NODE_MANAGER_CORBALOC);
            try {
                CORBA::Object_var obj = 
                    orb->string_to_object(NODE_MANAGER_CORBALOC.c_str());
             
                CPPUNIT_ASSERT_MESSAGE("NodeManager is nil",
                                       !CORBA::is_nil(obj.in()));
                    
                CdmwDnC::NodeManager_var node_manager =
                    CdmwDnC::NodeManager::_narrow(obj.in());
                
                CPPUNIT_ASSERT_MESSAGE("Narrow to NodeManager failed",
                                       !CORBA::is_nil(node_manager.in()));
                    
            } catch (const CORBA::SystemException& e) {
                std::ostringstream os;
                os << "Exception getting NodeManager via " 
                   << NODE_MANAGER_CORBALOC << " : " 
                   << Cdmw::OrbSupport::OrbSupport::exception_to_string(e);
                CPPUNIT_FAIL(os.str());
            }
            
            // REQ-SRS-CMD-195: 
            // Test NodeManager's NameService is accessible through corbaloc
            TEST_INFO("Get NodeManager's NameService via " << 
                      NODE_MANAGER_NAMING_CONTEXT_CORBALOC);
            CosNaming::NamingContext_var node_manager_nc =
                CosNaming::NamingContext::_nil();
            try {
                CORBA::Object_var obj = 
                    orb->string_to_object(NODE_MANAGER_NAMING_CONTEXT_CORBALOC.c_str());
             
                CPPUNIT_ASSERT_MESSAGE("NodeManager's NameService is nil",
                                       !CORBA::is_nil(obj.in()));
                    
                node_manager_nc = CosNaming::NamingContext::_narrow(obj.in());
                
                CPPUNIT_ASSERT_MESSAGE("Narrow of NodeManager's NameService failed",
                                       !CORBA::is_nil(node_manager_nc.in()));
                    
            } catch (const CORBA::SystemException& e) {
                std::ostringstream os;
                os << "Exception getting NodeManager's NameService via " 
                   << NODE_MANAGER_NAMING_CONTEXT_CORBALOC << " : " 
                   << Cdmw::OrbSupport::OrbSupport::exception_to_string(e);
                CPPUNIT_FAIL(os.str());
            }
            
            // get local NameService
            CosNaming::NamingContext_var local_nc =
                CosNaming::NamingContext::_nil();
            try {
                CORBA::Object_var obj = 
                    orb->resolve_initial_references("NameService");
             
                CPPUNIT_ASSERT_MESSAGE("local NameService is nil",
                                       !CORBA::is_nil(obj.in()));
                    
                local_nc = CosNaming::NamingContext::_narrow(obj.in());
                
                CPPUNIT_ASSERT_MESSAGE("Narrow of local NameService failed",
                                       !CORBA::is_nil(local_nc.in()));
                    
            } catch (const CORBA::SystemException& e) {
                std::ostringstream os;
                os << "Exception getting local NameService : " 
                   << Cdmw::OrbSupport::OrbSupport::exception_to_string(e);
                CPPUNIT_FAIL(os.str());
            }
            
            // create NamingInterface for local NameService
            Cdmw::CommonSvcs::Naming::NamingInterface local_naming(local_nc.in());
            
            // REQ-SRS-CMD-196: 
            // Get "DeploymentPlans" object from local NameService
            // and check that it's NodeManager's NameService
            TEST_INFO("Get \"" << NODE_MANAGER_NAMING_CONTEXT_BINDING_NAME
                      << "\" NameContext from local NameService");
            CosNaming::NamingContext_var deployment_plans_nc =
                CosNaming::NamingContext::_nil();
            try {
                CORBA::Object_var obj = 
                    local_naming.resolve(NODE_MANAGER_NAMING_CONTEXT_BINDING_NAME);
                
                CPPUNIT_ASSERT_MESSAGE(
                    "Object found is nil",
                    !CORBA::is_nil(obj.in())); 
                    
                deployment_plans_nc = 
                    CosNaming::NamingContext::_narrow(obj.in());
                
                CPPUNIT_ASSERT_MESSAGE(
                    "Narrow of Object found failed",
                    !CORBA::is_nil(deployment_plans_nc.in()));
                
            } catch (const CosNaming::NamingContext::NotFound& e) {
                CPPUNIT_FAIL("Object not found in local NameService");
            } catch (const CosNaming::NamingContext::CannotProceed& e) {
                CPPUNIT_FAIL("CannotProceed exception raised");
            } catch (const CosNaming::NamingContext::InvalidName& e) {
                CPPUNIT_FAIL("InvalidName exception raised");
            } catch (const CORBA::SystemException& e) {
                std::ostringstream os;
                os << "Exception getting " 
                   << NODE_MANAGER_NAMING_CONTEXT_BINDING_NAME << " : " 
                   << Cdmw::OrbSupport::OrbSupport::exception_to_string(e);
                CPPUNIT_FAIL(os.str());
            }
            
            CPPUNIT_ASSERT_MESSAGE(
                "NodeManager's NameService and NameContext found from "
                "local NameService are not equivalent",
                node_manager_nc->_is_equivalent(deployment_plans_nc.in()));
            
            
            // Additional tests: navigate through "DeploymentPlans"
            // NameContext to get component and its facet
            TEST_INFO("Get component and its facet through "
                      << NODE_MANAGER_NAMING_CONTEXT_BINDING_NAME
                      << " NameContext");
            Cdmw::CommonSvcs::Naming::NamingInterface 
                deployment_naming(deployment_plans_nc.in());
                      
            TestCcmNodeManager::Server_var server = 
                TestCcmNodeManager::Server::_nil();
            try {
                CORBA::Object_var obj = 
                    deployment_naming.resolve(COMPONENT_NAMING_PATH);
                    
                CPPUNIT_ASSERT_MESSAGE(
                    "Object found is nil",
                    !CORBA::is_nil(obj.in())); 
                    
                server = TestCcmNodeManager::Server::_narrow(obj.in());
                
                CPPUNIT_ASSERT_MESSAGE(
                    "Narrow of object found failed",
                    !CORBA::is_nil(server.in()));
                
            } catch (const CosNaming::NamingContext::NotFound& e) {
                CPPUNIT_FAIL("Object not found in NameContext");
            } catch (const CosNaming::NamingContext::CannotProceed& e) {
                CPPUNIT_FAIL("CannotProceed exception raised");
            } catch (const CosNaming::NamingContext::InvalidName& e) {
                CPPUNIT_FAIL("InvalidName exception raised");
            } catch (const CORBA::SystemException& e) {
                std::ostringstream os;
                os << "Exception getting " 
                   << COMPONENT_NAMING_PATH << " : " 
                   << Cdmw::OrbSupport::OrbSupport::exception_to_string(e);
                CPPUNIT_FAIL(os.str());
            }
            
            TestCcmNodeManager::HelloWorld_var facet = 
                TestCcmNodeManager::HelloWorld::_nil();
            try {
                CORBA::Object_var obj = 
                    deployment_naming.resolve(COMPONENT_FACET_NAMING_PATH);
                    
                CPPUNIT_ASSERT_MESSAGE(
                    "Object found is nil",
                    !CORBA::is_nil(obj.in())); 
                    
                facet = TestCcmNodeManager::HelloWorld::_narrow(obj.in());
                
                CPPUNIT_ASSERT_MESSAGE(
                    "Narrow of object found failed",
                    !CORBA::is_nil(facet.in()));
                
            } catch (const CosNaming::NamingContext::NotFound& e) {
                CPPUNIT_FAIL("Object not found in NameContext");
            } catch (const CosNaming::NamingContext::CannotProceed& e) {
                CPPUNIT_FAIL("CannotProceed exception raised");
            } catch (const CosNaming::NamingContext::InvalidName& e) {
                CPPUNIT_FAIL("InvalidName exception raised");
            } catch (const CORBA::SystemException& e) {
                std::ostringstream os;
                os << "Exception getting " 
                   << COMPONENT_FACET_NAMING_PATH << " : " 
                   << Cdmw::OrbSupport::OrbSupport::exception_to_string(e);
                CPPUNIT_FAIL(os.str());
            }
            
            try {
                TestCcmNodeManager::HelloWorld_var facet_via_component =
                    server->provide_hello_facet();
                    
                CPPUNIT_ASSERT_MESSAGE(
                    "Object found from NameContext via is not equivalent to component's facet",
                    facet->_is_equivalent(facet_via_component.in()));
                
                facet->say_hello();
                    
            } catch (const CORBA::SystemException& e) {
                std::ostringstream os;
                os << "Exception testing facet found from NameContext via " 
                   << COMPONENT_FACET_NAMING_PATH << " : " 
                   << Cdmw::OrbSupport::OrbSupport::exception_to_string(e);
                CPPUNIT_FAIL(os.str());
            }
            
        } catch (const std::exception& e) {
            TEST_INFO("ERROR during test: " << e.what());
            throw e;
        }
    };


    private:

    TestNodeManager(const TestNodeManager& rhs) throw();

    TestNodeManager& operator=(const TestNodeManager& rhs) throw();

    
    CPPUNIT_TEST_SUITE( TestNodeManager );
    CPPUNIT_TEST( do_tests );
    CPPUNIT_TEST_SUITE_END();
};    
    


CPPUNIT_TEST_SUITE_REGISTRATION( TestNodeManager );

    
int main(int argc, char* argv[])
{    
    int status = SUCCESS;

    CORBA::ORB_var orb = CORBA::ORB::_nil();

    try {
        // call ORB_init
        Cdmw::OrbSupport::StrategyList orb_strategies;
        orb_strategies.add_OrbThreaded();
        orb_strategies.add_PoaThreadPool(POA_THREAD_POOL_SIZE);
        orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, orb_strategies);

        //
        // Get the root POA 
        //
        CORBA::Object_var obj = orb -> resolve_initial_references("RootPOA");
        PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj.in());

        //
        // Run implementation (Activate the POA manager)
        //
        PortableServer::POAManager_var poaManager = rootPOA -> the_POAManager();
        poaManager -> activate();

        
        // Setup signal handling for ORB to be shut down
        Cdmw::OrbSupport::OrbSupport::install_orb_shutdown_signal_handlers(orb.in());

        // Setup test
        Cdmw::TestUtils::CORBATestManager::instance()->init(orb.in(), argc,argv);
        
        // run test
        Cdmw::TestUtils::CORBATestManager::instance()->run_tests();
        
        // display test result
        Cdmw::TestUtils::CORBATestManager::instance()->summary();
        
        Cdmw::OrbSupport::OrbSupport::uninstall_orb_shutdown_signal_handlers(orb.in());
        
    } catch (const Cdmw::Exception & ex) {
        std::cerr << ex.what() << std::endl;
        status = FAILURE;
    } catch(const CORBA::Exception& ex)  {        
        std::cerr << ex << std::endl;
        status = FAILURE;
    }

    if(!CORBA::is_nil(orb.in()))
    {
        try {
            // call ORB_cleanup
            Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
            
            // destroy ORB
            orb->destroy();
        }
        catch(const CORBA::Exception& ex)
        {           
            std::cerr << ex << std::endl;
            status = FAILURE;
        }
    }        

    return status;    
}


