/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
/*

COPYRIGHT

Copyright 1992, 1993, 1994 Sun Microsystems, Inc.  Printed in the United
States of America.  All Rights Reserved.

This product is protected by copyright and distributed under the following
license restricting its use.

The Interface Definition Language Compiler Front End (CFE) is made
available for your use provided that you include this license and copyright
notice on all media and documentation and the software program in which
this product is incorporated in whole or part. You may copy and extend
functionality (but may not remove functionality) of the Interface
Definition Language CFE without charge, but you are not authorized to
license or distribute it to anyone else except as part of a product or
program developed by you or with the express written consent of Sun
Microsystems, Inc. ("Sun").

The names of Sun Microsystems, Inc. and any of its subsidiaries or
affiliates may not be used in advertising or publicity pertaining to
distribution of Interface Definition Language CFE as permitted herein.

This license is effective until terminated by Sun for failure to comply
with this license.  Upon termination, you shall destroy or return all code
and documentation for the Interface Definition Language CFE.

INTERFACE DEFINITION LANGUAGE CFE IS PROVIDED AS IS WITH NO WARRANTIES OF
ANY KIND INCLUDING THE WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS
FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR ARISING FROM A COURSE OF
DEALING, USAGE OR TRADE PRACTICE.

INTERFACE DEFINITION LANGUAGE CFE IS PROVIDED WITH NO SUPPORT AND WITHOUT
ANY OBLIGATION ON THE PART OF Sun OR ANY OF ITS SUBSIDIARIES OR AFFILIATES
TO ASSIST IN ITS USE, CORRECTION, MODIFICATION OR ENHANCEMENT.

SUN OR ANY OF ITS SUBSIDIARIES OR AFFILIATES SHALL HAVE NO LIABILITY WITH
RESPECT TO THE INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY
INTERFACE DEFINITION LANGUAGE CFE OR ANY PART THEREOF.

IN NO EVENT WILL SUN OR ANY OF ITS SUBSIDIARIES OR AFFILIATES BE LIABLE FOR
ANY LOST REVENUE OR PROFITS OR OTHER SPECIAL, INDIRECT AND CONSEQUENTIAL
DAMAGES, EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

Use, duplication, or disclosure by the government is subject to
restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
Technical Data and Computer Software clause at DFARS 252.227-7013 and FAR
52.227-19.

Sun, Sun Microsystems and the Sun logo are trademarks or registered
trademarks of Sun Microsystems, Inc.

SunSoft, Inc.
2550 Garcia Avenue
Mountain View, California  94043

NOTE:

SunOS, SunSoft, Sun, Solaris, Sun Microsystems or the Sun logo are
trademarks or registered trademarks of Sun Microsystems, Inc.

 */


#ifndef INCL_CODEGENERATOR_FMTOSTRINGSTREAM_HPP
#define INCL_CODEGENERATOR_FMTOSTRINGSTREAM_HPP

#include <sstream>

class Identifier;
class UTL_IdList;
class AST_Expression;

// a dummy structure to inform << operator to  put a newline
// and use the current indentation for the succeeding line
struct OST_NL
{
    public:
        OST_NL (void);
};

struct OST_INDENT
{
    //   Operates like a manipulator, increasing the indentation level.
    //   Increase the indentation level, if the "do_now" parameter is
    //   not zero then the <indent> method is called on the stream.
    // 
    OST_INDENT (int do_now = 0);

        const int do_now_;
};

struct OST_UNINDENT
{
    //   Operates like a manipulator, decreasing the indentation level.
    //   Decrease the indentation level, if the "do_now" parameter is
    //   not zero then the <indent> method is called on the stream.
    // 
    OST_UNINDENT (int do_now = 0);

     const int do_now_;
};

extern const OST_NL ost_nl;
extern const OST_INDENT ost_idt;
extern const OST_INDENT ost_idt_nl;
extern const OST_UNINDENT ost_uidt;
extern const OST_UNINDENT ost_uidt_nl;

namespace Cdmw
{
   namespace CodeGenerator
   {
       // Defines an interface for ostringstream with indentation facilities
class FmtOStringStream
       {
           public:

               // FmtOStringStream (std::ostringstream &);
               // constructor.

               /**
                * Purpose:
                * <p> Default Constructor.
                *
                */
    FmtOStringStream (void);

               /**
                * Purpose:
                * <p> Destructor.
                */
    virtual ~FmtOStringStream (void);

               /**
                * Purpose:
                * <p> The ostr operation is used to return the m_stream attribut.
                */
    std::ostringstream & ostr() { return m_stream; }         

               /**
                * Purpose:
                * <p> The incr_indent operation is used to increment the indentation level and by default actually indent the output accordingly.
                *
                *@param flag indicates to indent output or not. 
                *@return The operation status, zero if the operation is successful.
                *
                */
    int incr_indent (unsigned short flag = 1);

               /**
                * Purpose:
                * <p> The decr_indent operation is used to decrease the indentation level and by default actually indent the output accordingly
                *
                *@param flag indicates to indent output or not. 
                *@return The operation status, zero if the operation is successful.
                *
                */
    int decr_indent (unsigned short flag = 1);

               /**
                * Purpose:
                * <p> The reset operation is used to reset indentation level to 0.
                *
                *@return The operation status, zero if the operation is successful.
                *
                */ 
    int reset (void);

               /**
                * Purpose:
                * <p> The indent operation is used to indent starting next line.
                *
                *@return The operation status, zero if the operation is successful.
                *
                */
    int indent (void);

               /**
                * Purpose:
                * <p> The nl operation is used to put a newline and indent on the next line.
                *
                *@return The operation status, zero if the operation is successful.
                *
                */
    int nl (void);

               /**
                * Purpose:
                * <p> The print operation is used to output the SunSoft IDL Identifier Node.
                *
                *@return a FmtOStringStream to ourselves.
                *
                */
    virtual FmtOStringStream &print (Identifier *id);

               /**
                * Purpose:
                * <p> The print operation is used to output the SunSoft IDL UTL_IdList Node which is usually used to maintain a scoped name.
                *
                *@return a FmtOStringStream to ourselves.
                *
                */
    virtual FmtOStringStream &print (UTL_IdList *idl);


               /**
                * Purpose:
                * <p> The print operation is used to output the contents of the AST_Expression node.
                *
                *@return a FmtOStringStream to ourselves.
                *
                */
    virtual FmtOStringStream &print (AST_Expression *expr);

               /*
                *     overloaded operators
                */

               /**
                * Purpose:
                * <p> The overloaded operator '<<' is used to output the char string.
                *
                *@param str The char string.
                *@return a FmtOStringStream to ourselves.
                *
                */
    FmtOStringStream &operator<< (const char *str);

               /**
                * Purpose:
                * <p> The overloaded operator '<<' is used to output the integer.
                *
                *@param num The integer.
                *@return a FmtOStringStream to ourselves.
                *
                */
    FmtOStringStream &operator<< (const unsigned long num);

               /**
                * Purpose:
                * <p> The overloaded operator '<<' is used to output the integer.
                *
                *@param num The integer.
                *@return a FmtOStringStream to ourselves.
                *
                */
    FmtOStringStream &operator<< (const long num);


               // = MANIPULATORS

               /**
                * Purpose:
                * <p> The overloaded operator '<<' is used to output the structure OST_NL.
                *
                *@param nl The structure OST_NL.
                *@return a FmtOStringStream to ourselves.
                *
                */
    FmtOStringStream &operator<< (const OST_NL& nl);

               /**
                * Purpose:
                * <p> The overloaded operator '<<' is used to output the structure OST_INDENT.
                *
                *@param i The structure OST_INDENT.
                *@return a FmtOStringStream to ourselves.
                *
                */
    FmtOStringStream &operator<< (const OST_INDENT& i);

               /**
                * Purpose:
                * <p> The overloaded operator '<<' is used to output the structure OST_UNINDENT.
                *
                *@param i The structure OST_UNINDENT.
                *@return a FmtOStringStream to ourselves.
                *
                */
    FmtOStringStream &operator<< (const OST_UNINDENT& i);

               /**
                * Purpose:
                * <p> The overloaded operator '<<' is used to output an Identifier node.
                *
                *@param id The Identifier.
                *@return a FmtOStringStream to ourselves.
                *
                */
    FmtOStringStream &operator<< (Identifier *id);

               /**
                * Purpose:
                * <p> The overloaded operator '<<' is used to output a scoped name.
                *
                *@param idl The scoped name.
                *@return a FmtOStringStream to ourselves.
                *
                */
    FmtOStringStream &operator<< (UTL_IdList *idl);

               // output an AST_Expression node
               /**
                * Purpose:
                * <p> The overloaded operator '<<' is used to output an AST_Expression node.
                *
                *@param expr The AST_Expression node.
                *@return a FmtOStringStream to ourselves.
                *
                */ 
    FmtOStringStream &operator<< (AST_Expression *expr);

           protected:

               // indentation level
    int m_indent_level;

               // the embedded stringstream
    std::ostringstream m_stream;
                 
           private:

       };

   }; // End of CodeGenerator
}; // End of Cdmw

#endif // INCL_CODEGENERATOR_FMTOSTRINGSTREAM_HPP
