/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_CODE_GENERATOR_UTL_IDENTIFIER_HELPER_HPP
#define INCL_CODE_GENERATOR_UTL_IDENTIFIER_HELPER_HPP

#include "utl_identifier.h"
#include <string>

namespace Cdmw
{
   namespace CodeGenerator
   {
     /* TODO
        portdescrion: pd->id
     */

      /**
       * Purpose: <p> The original_local_name operation should do the following:
         1) make sure to keep '_' escape caracter in generated identifier in IDL
         2) remove the '_' escape character when the identifier is part of another identifier
            such as in provides_XXX
         3) remove any _cxx_ in generated IDL (equivalent to AST_Decl::original_local_name())
         *
         *@param local_name The Identifier.
         *@return An Identifier
      */
      extern Identifier* 
      original_local_name (Identifier * local_name);

      /**
       * Purpose: <p> The is_idl_keyword operation is used to indicate if the char string is an OMG IDL keywords as specified in OMG document
       *  The Common Object Request Broker: Architecture and Specification
       *  July 2002, Version 3.0, formal/02-06-33.
       *
       *@param s The char string.
       *@return A boolean.
       *
       */
      extern bool
      is_idl_keyword(const char* s);

      /**
       * Purpose: <p> The try_escape operation should do the following:
       *  1) make sure to keep '_' escape caracter in generated identifier in IDL
       * 
       *@param local_name The Identifier
       *@return A string.
       */
      extern std::string 
      try_escape (Identifier *local_name);

   }; // End of CodeGenerator
}; // End of Cdmw

#endif // INCL_CODE_GENERATOR_UTL_IDENTIFIER_HELPER_HPP
