/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
/*

COPYRIGHT

Copyright 1992, 1993, 1994 Sun Microsystems, Inc.  Printed in the United
States of America.  All Rights Reserved.

This product is protected by copyright and distributed under the following
license restricting its use.

The Interface Definition Language Compiler Front End (CFE) is made
available for your use provided that you include this license and copyright
notice on all media and documentation and the software program in which
this product is incorporated in whole or part. You may copy and extend
functionality (but may not remove functionality) of the Interface
Definition Language CFE without charge, but you are not authorized to
license or distribute it to anyone else except as part of a product or
program developed by you or with the express written consent of Sun
Microsystems, Inc. ("Sun").

The names of Sun Microsystems, Inc. and any of its subsidiaries or
affiliates may not be used in advertising or publicity pertaining to
distribution of Interface Definition Language CFE as permitted herein.

This license is effective until terminated by Sun for failure to comply
with this license.  Upon termination, you shall destroy or return all code
and documentation for the Interface Definition Language CFE.

INTERFACE DEFINITION LANGUAGE CFE IS PROVIDED AS IS WITH NO WARRANTIES OF
ANY KIND INCLUDING THE WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS
FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR ARISING FROM A COURSE OF
DEALING, USAGE OR TRADE PRACTICE.

INTERFACE DEFINITION LANGUAGE CFE IS PROVIDED WITH NO SUPPORT AND WITHOUT
ANY OBLIGATION ON THE PART OF Sun OR ANY OF ITS SUBSIDIARIES OR AFFILIATES
TO ASSIST IN ITS USE, CORRECTION, MODIFICATION OR ENHANCEMENT.

SUN OR ANY OF ITS SUBSIDIARIES OR AFFILIATES SHALL HAVE NO LIABILITY WITH
RESPECT TO THE INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY
INTERFACE DEFINITION LANGUAGE CFE OR ANY PART THEREOF.

IN NO EVENT WILL SUN OR ANY OF ITS SUBSIDIARIES OR AFFILIATES BE LIABLE FOR
ANY LOST REVENUE OR PROFITS OR OTHER SPECIAL, INDIRECT AND CONSEQUENTIAL
DAMAGES, EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

Use, duplication, or disclosure by the government is subject to
restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
Technical Data and Computer Software clause at DFARS 252.227-7013 and FAR
52.227-19.

Sun, Sun Microsystems and the Sun logo are trademarks or registered
trademarks of Sun Microsystems, Inc.

SunSoft, Inc.
2550 Garcia Avenue
Mountain View, California  94043

NOTE:

SunOS, SunSoft, Sun, Solaris, Sun Microsystems or the Sun logo are
trademarks or registered trademarks of Sun Microsystems, Inc.

 */

#include "engine/BasicIDLVisitor.hpp"
#include "engine/UtlIdentifierHelper.hpp"
#include "engine/FmtOStringStream.hpp"

#include "be_extern.h"

#include "ast_argument.h"
#include "ast_array.h"
#include "ast_attribute.h"
#include "ast_component_fwd.h"
#include "ast_enum.h"
#include "ast_enum_val.h"
#include "ast_eventtype.h"
#include "ast_eventtype_fwd.h"
#include "ast_exception.h"
#include "ast_factory.h"
#include "ast_field.h"
#include "ast_home.h"
#include "ast_operation.h"
#include "ast_root.h"
#include "ast_sequence.h"
#include "ast_string.h"
#include "ast_structure_fwd.h"
#include "ast_union.h"
#include "ast_union_branch.h"
#include "ast_union_fwd.h"
#include "ast_union_label.h"
#include "ast_valuebox.h"
#include "ast_valuetype_fwd.h"
#include "ast_native.h"
#include "utl_exceptlist.h"
#include "utl_identifier.h"
#include "utl_idlist.h"
#include "utl_string.h"
#include "global_extern.h"
#include "nr_extern.h"

#include <sstream>
#include <string.h>

namespace Cdmw
{
   namespace CodeGenerator
   {

      BasicIDLVisitor::BasicIDLVisitor (FmtOStringStream &os)
         : m_os (os),
           disc_type_ (0)
      {
      }

      BasicIDLVisitor::~BasicIDLVisitor (void)
      {
      }

      int
      BasicIDLVisitor::visit_decl (AST_Decl *)
      {
         return 0;
      }

      int
      BasicIDLVisitor::visit_scope (UTL_Scope *node)
      {
         for (UTL_ScopeActiveIterator si (node, UTL_Scope::IK_decls);
              !si.is_done ();
              si.next ())
         {
            AST_Decl *d = si.item ();

            if (d->node_type () == AST_Decl::NT_pre_defined)
            {
               continue;
            }

            // Want to skip the uses_xxxConnection structs added by uses
            // multiple ports.
            // @@@ (JP) This will go away when the visitor is finished, since
            // those uses_xxxConnection structs will not be added to the AST.
            if (ScopeAsDecl (node)->node_type () == AST_Decl::NT_component
                && d->node_type () != AST_Decl::NT_attr)
            {
               continue;
            }

            if (d->ast_accept (this) != 0)
            {
               ACE_ERROR_RETURN ((LM_ERROR,
                                  "BasicIDLVisitor::visit_scope - "
                                  "codegen for scope failed\n"),
                                 -1);
            }
         }

         return 0;
      }

      int
      BasicIDLVisitor::visit_type (AST_Type *)
      {
         return 0;
      }

      int
      BasicIDLVisitor::visit_predefined_type (AST_PredefinedType *)
      {
         return 0;
      }

      int
      BasicIDLVisitor::visit_interface_fwd (AST_InterfaceFwd *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         m_os << ost_nl << ost_nl;

         if (node->is_local ())
         {
            m_os << "local ";
         }

         if (node->is_abstract ())
         {
            m_os << "abstract ";
         }

         m_os << "interface " << try_escape(node->original_local_name ()).c_str() << ";";

         return 0;
      }

      int
      BasicIDLVisitor::visit_valuebox (AST_ValueBox *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         m_os << ost_nl << ost_nl
              << "valuetype " << try_escape(node->original_local_name ()).c_str();

         AST_Type *bt = node->boxed_type ();

         // Keep output statements separate because of side effects.

         if (bt->node_type () == AST_Decl::NT_array)
         {
            this->gen_anonymous_array (bt, node);
         }
         else
         {
            m_os << this->type_name (bt);
         }

         m_os << ";";

         this->check_id_and_version (node);

         return 0;
      }

      int
      BasicIDLVisitor::visit_valuetype (AST_ValueType *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         m_os << ost_nl << ost_nl;

         if (node->is_abstract ())
         {
            m_os << "abstract ";
         }

         if (node->custom ())
         {
            m_os << "custom ";
         }

         m_os << "valuetype " << try_escape(node->original_local_name ()).c_str();

         AST_Decl::NodeType nt = node->node_type ();
         AST_Interface **parents = node->inherits ();
         long ninherits = node->n_inherits ();

         long i = 0;
         for (i = 0; i < ninherits; ++i)
         {
            if (i == 0)
            {
               m_os << " : ";
            }
            else
            {
               m_os << ", ";
            }

            m_os << parents[i]->name ();
         }

         if (nt == AST_Decl::NT_eventtype)
         {
            m_os << (ninherits == 0 ? " : " : ", ")
               << "Components::EventBase";
         }

         AST_Interface **supports = node->supports ();

         for (i = 0; i < node->n_supports (); ++i)
         {
            if (i == 0)
            {
               m_os << " supports ";
            }
            else
            {
               m_os << ", ";
            }

            m_os << supports[i]->name ();
         }

         m_os << ost_nl
              << "{" << ost_idt;

         this->check_id_and_version (node);
         this->check_prefix (node);

         if (this->visit_scope (node) != 0)
         {
            ACE_ERROR_RETURN ((LM_ERROR,
                               "BasicIDLVisitor::visit_valuetype - "
                               "codegen for scope failed\n"),
                              -1);
         }

         m_os << ost_uidt_nl
              << "};";

         return 0;
      }

      int
      BasicIDLVisitor::visit_valuetype_fwd (AST_ValueTypeFwd *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         m_os << ost_nl << ost_nl;

         if (node->is_abstract ())
         {
            m_os << "abstract ";
         }

         (void) node->node_type ();

         m_os << "valuetype " << try_escape(node->original_local_name ()).c_str() << ";";

         return 0;
      }




      int
      BasicIDLVisitor::visit_factory  (AST_Factory *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         m_os << ost_nl;

         m_os << "factory " << try_escape(node->original_local_name ()).c_str() << " (";

         this->gen_params (node, node->argument_count ());

         m_os << ")";

         this->gen_exception_list (node->exceptions ());

         m_os << ";";

         this->check_id_and_version (node);

         return 0;
      }

      int
      BasicIDLVisitor::visit_structure (AST_Structure *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         m_os << ost_nl << ost_nl;

         m_os << "struct " << try_escape(node->original_local_name ()).c_str() << ost_nl
              << "{" << ost_idt;

         this->check_id_and_version (node);

         if (this->visit_scope (node) != 0)
         {
            ACE_ERROR_RETURN ((LM_ERROR,
                               "BasicIDLVisitor::visit_structure - "
                               "codegen for scope failed\n"),
                              -1);
         }

         m_os << ost_uidt_nl
              << "};";

         return 0;
      }

      int
      BasicIDLVisitor::visit_structure_fwd (AST_StructureFwd *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         m_os << ost_nl << ost_nl;

         m_os << "struct " << try_escape(node->original_local_name ()).c_str() << ";";

         return 0;
      }

      int
      BasicIDLVisitor::visit_exception (AST_Exception *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         m_os << ost_nl << ost_nl;

         m_os << "exception " << try_escape(node->original_local_name ()).c_str() << ost_nl
              << "{" << ost_idt;

         this->check_id_and_version (node);

         if (this->visit_scope (node) != 0)
         {
            ACE_ERROR_RETURN ((LM_ERROR,
                               "BasicIDLVisitor::visit_exception - "
                               "codegen for scope failed\n"),
                              -1);
         }

         m_os << ost_uidt_nl
              << "};";

         return 0;
      }

      int
      BasicIDLVisitor::visit_expression (AST_Expression *)
      {
         return 0;
      }

      int
      BasicIDLVisitor::visit_enum (AST_Enum *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         m_os << ost_nl << ost_nl;

         m_os << "enum " << try_escape(node->original_local_name ()).c_str() << ost_nl
              << "{" << ost_idt;

         for (UTL_ScopeActiveIterator i (node, UTL_Scope::IK_decls);
              !i.is_done ();)
         {
            m_os << ost_nl;

            AST_EnumVal *ev = AST_EnumVal::narrow_from_decl (i.item ());
            m_os << ev->original_local_name ();

            // Advance here so the check below will work.
            i.next ();

            if (!i.is_done ())
            {
               m_os << ",";
            }
         }

         m_os << ost_uidt_nl
              << "};";

         this->check_id_and_version (node);

         return 0;
      }

      int
      BasicIDLVisitor::visit_operation (AST_Operation *node)
      {
         this->gen_operation(node);

         this->check_id_and_version (node);

         return 0;
      }

      int
      BasicIDLVisitor::visit_field (AST_Field *node)
      {
         AST_Field::Visibility v = node->visibility ();

         m_os << ost_nl
              << (v == AST_Field::vis_PUBLIC
                  ? "public "
                  : (v == AST_Field::vis_PRIVATE ? "private " : ""));

         AST_Type *ft = node->field_type ();

         if (ft->node_type () == AST_Decl::NT_array)
         {
            this->gen_anonymous_array (ft, node);
         }
         else
         {
            // Keep these statements separate because of side effects.
            m_os << this->type_name (ft);
            m_os << " " << try_escape(node->original_local_name ()).c_str();
         }

         m_os << ";";

         return 0;
      }

      int
      BasicIDLVisitor::visit_argument (AST_Argument *node)
      {
         m_os << ost_nl;

         switch (node->direction ())
         {
            case AST_Argument::dir_IN:
               m_os << "in ";
               break;
            case AST_Argument::dir_INOUT:
               m_os << "inout ";
               break;
            case AST_Argument::dir_OUT:
               m_os << "out ";
               break;
            default:
               return -1;
         }

         m_os << this->type_name (node->field_type ())
              << " " << try_escape(node->original_local_name ()).c_str();

         return 0;
      }

      int
      BasicIDLVisitor::visit_attribute (AST_Attribute *node)
      {
         this->gen_attribute (node);

//         this->check_id_and_version (node);

         return 0;
      }

      int
      BasicIDLVisitor::visit_union (AST_Union *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         this->disc_type_ = node->disc_type ()->unaliased_type ();

         m_os << ost_nl << ost_nl;

         char * identifier = node->disc_type ()->original_local_name()->get_string();

         /**
         m_os << "union " << try_escape(node->original_local_name ()).c_str() << " switch ("
              <<  node->disc_type ()->name () << ")" << ost_nl
              << "{" << ost_idt;
         */
          if (  ( !strcmp(identifier, "Long" )) || 
                ( !strcmp(identifier, "Short" ))  )
          {
             lower(identifier);
             m_os << "union " << try_escape(node->original_local_name ()).c_str() << " switch ("
                  <<  identifier << ")" << ost_nl
                  << "{" << ost_idt;
          }

          else if  ( !strcmp(identifier, "ULong" ) )
          {
             lower(identifier);
             m_os << "union " << try_escape(node->original_local_name ()).c_str() << " switch ("
                  <<  "unsigned long" << ")" << ost_nl
                  << "{" << ost_idt;
          } 
          else if  ( !strcmp(identifier, "UShort" ) )
          {
             lower(identifier);
             m_os << "union " << try_escape(node->original_local_name ()).c_str() << " switch ("
                  <<  "unsigned short" << ")" << ost_nl
                  << "{" << ost_idt;
          }
                     
          else {
             m_os << "union " << try_escape(node->original_local_name ()).c_str() << " switch ("
                  <<  node->disc_type ()->name () << ")" << ost_nl
                  << "{" << ost_idt;
          }

         this->check_id_and_version (node);

         if (this->visit_scope (node) != 0)
         {
            ACE_ERROR_RETURN ((LM_ERROR,
                               "BasicIDLVisitor::visit_union - "
                               "codegen for scope failed\n"),
                              -1);
         }

         m_os << ost_uidt_nl
              << "};";

         return 0;
      }

      int
      BasicIDLVisitor::visit_union_fwd (AST_UnionFwd *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         m_os << ost_nl << ost_nl
              << "union " << try_escape(node->original_local_name ()).c_str() << ";";

         return 0;
      }

      int
      BasicIDLVisitor::visit_union_branch (AST_UnionBranch *node)
      {
         for (unsigned long i = 0; i < node->label_list_length (); ++i)
         {
            if (this->visit_union_label (node->label (i)) != 0)
            {
               ACE_ERROR_RETURN ((LM_ERROR,
                                  "BasicIDLVisitor::visit_union_branch - "
                                  "codegen for label failed\n"),
                                 -1);
            }
         }

         AST_Type *ft = node->field_type ();

         if (ft->node_type () == AST_Decl::NT_array)
         {
            this->gen_anonymous_array (ft, node);
         }
         else
         {
            m_os << this->type_name (ft);
            m_os << " " << try_escape(node->original_local_name ()).c_str();
         }

         m_os << ";";

         return 0;
      }

      int
      BasicIDLVisitor::visit_union_label (AST_UnionLabel *node)
      {
         m_os << ost_nl;

         if (node->label_kind () == AST_UnionLabel::UL_default)
         {
            m_os << "default: ";
         }
         else
         {
            m_os << "case ";
            this->gen_label_value (node);
            m_os << ": ";
         }

         return 0;
      }

      int
      BasicIDLVisitor::visit_constant (AST_Constant *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         m_os << ost_nl << ost_nl;

         m_os << "const ";

         switch (node->et ())
         {
            case AST_Expression::EV_short:
               m_os << "short";
               break;
            case AST_Expression::EV_ushort:
               m_os << "unsigned short";
               break;
            case AST_Expression::EV_long:
               m_os << "long";
               break;
            case AST_Expression::EV_ulong:
               m_os << "unsigned long";
               break;
            case AST_Expression::EV_longlong:
               m_os << "long long";
               break;
            case AST_Expression::EV_ulonglong:
               m_os << "unsigned long long";
               break;
            case AST_Expression::EV_char:
               m_os << "char";
               break;
            case AST_Expression::EV_wchar:
               m_os << "wchar";
               break;
            case AST_Expression::EV_bool:
               m_os << "boolean";
               break;
            case AST_Expression::EV_octet:
               m_os << "octet";
               break;
            case AST_Expression::EV_float:
               m_os << "float";
               break;
            case AST_Expression::EV_double:
               m_os << "double";
               break;
            case AST_Expression::EV_longdouble:
               m_os << "long double";
               break;
            case AST_Expression::EV_string:
               m_os << "string";
               break;
            case AST_Expression::EV_wstring:
               m_os << "wstring";
               break;
            case AST_Expression::EV_enum:
               m_os << node->enum_full_name ();
               break;
            default:
               break;
         }

         m_os <<  " "
              << try_escape(node->original_local_name ()).c_str() << " = " << node->constant_value () << ";";

         this->check_id_and_version (node);

         return 0;
      }

      int
      BasicIDLVisitor::visit_enum_val (AST_EnumVal *)
      {
         return 0;
      }

      int
      BasicIDLVisitor::visit_array (AST_Array *node)
      {
         m_os << node->base_type ()->name ();

         for (unsigned long i = 0; i < node->n_dims (); ++i)
         {
            m_os << "[" << node->dims ()[i] << "]";
         }

         return 0;
      }

      int
      BasicIDLVisitor::visit_sequence (AST_Sequence *node)
      {
         // Keep output statements separate because of side effects.
         m_os << "sequence<";
         m_os << this->type_name (node->base_type ());

         if (!node->unbounded ())
         {
            m_os << ", " << node->max_size ()->ev ()->u.ulval;
         }

         m_os << "> ";

         return 0;
      }

      int
      BasicIDLVisitor::visit_string (AST_String *node)
      {
         m_os << (node->width () > 1 ? "w" : "") << "string";

         unsigned long bound = node->max_size ()->ev ()->u.ulval;

         if (bound > 0)
         {
            m_os << "<" << bound << ">";
         }

         return 0;
      }

      int
      BasicIDLVisitor::visit_typedef (AST_Typedef *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         m_os << ost_nl << ost_nl
              << "typedef ";

         AST_Type *bt = node->base_type ();

         // Keep output statements separate because of side effects.

         if (bt->node_type () == AST_Decl::NT_array)
         {
            this->gen_anonymous_array (bt, node);
         }
         else
         {
            m_os << this->type_name (bt);
            m_os << " " << try_escape(node->original_local_name ()).c_str();
         }

         m_os << ";";

         this->check_id_and_version (node);

         return 0;
      }

      int
      BasicIDLVisitor::visit_root (AST_Root *)
      {
         return -1;
      }

      int
      BasicIDLVisitor::visit_native (AST_Native *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         m_os << ost_nl << ost_nl;

         m_os << "native " << try_escape(node->original_local_name ()).c_str() << ";";

         return 0;
      }

      void
      BasicIDLVisitor::check_prefix (AST_Decl *d)
      {
         if (d->typeid_set ())
         {
            return;
         }

         const char *the_prefix = d->prefix ();
         UTL_Scope *scope = d->defined_in ();
         if ((scope !=0) && (ScopeAsDecl (scope) != 0)) 
         {
            AST_Decl *p = ScopeAsDecl (scope);

            if (ACE_OS::strcmp (the_prefix, p->prefix ()) != 0)
            {
               m_os << ost_nl
                    << "typeprefix " << try_escape(d->original_local_name ()).c_str()
                    << " \"" << the_prefix << "\";";
            }
         }
      }

      void
      BasicIDLVisitor::check_id_and_version (AST_Decl *d)
      {
         if (d->typeid_set ())
         {
            m_os << ost_nl
                 << "typeid " << try_escape(d->original_local_name ()).c_str()
                 << " \"" << d->repoID () << "\";";

            return;
         }

         const char *the_version = d->version ();
         UTL_Scope *scope = d->defined_in ();
         if ((scope !=0) && (ScopeAsDecl (scope) != 0)) 
         {
            AST_Decl *p = ScopeAsDecl (scope);

            if (ACE_OS::strcmp (the_version, p->version ()) != 0)
            {
               m_os << "\n"
                    << "#pragma version " << try_escape(d->original_local_name ()).c_str()
                    << " " << the_version;
            }
         }
      }

      const char *
      BasicIDLVisitor::type_name (AST_Type *t)
      {
         AST_PredefinedType *pdt = 0;

         switch (t->node_type ())
         {
            case AST_Decl::NT_wstring:
            case AST_Decl::NT_string:
            case AST_Decl::NT_sequence:
               // This causes side effects so output statements
               // sending us here should not be concatenated.
               (void) t->ast_accept (this);
               return "";
            case AST_Decl::NT_pre_defined:
               pdt = AST_PredefinedType::narrow_from_decl (t);

               switch (pdt->pt ())
               {
                  case AST_PredefinedType::PT_pseudo:
                     return t->full_name ();
                  case AST_PredefinedType::PT_object:
                     return "Object";
                  case AST_PredefinedType::PT_any:
                     return "any";
                  case AST_PredefinedType::PT_long:
                     return "long";
                  case AST_PredefinedType::PT_ulong:
                     return "unsigned long";
                  case AST_PredefinedType::PT_longlong:
                     return "long long";
                  case AST_PredefinedType::PT_ulonglong:
                     return "unsigned long long";
                  case AST_PredefinedType::PT_short:
                     return "short";
                  case AST_PredefinedType::PT_ushort:
                     return "unsigned short";
                  case AST_PredefinedType::PT_float:
                     return "float";
                  case AST_PredefinedType::PT_double:
                     return "double";
                  case AST_PredefinedType::PT_longdouble:
                     return "long double";
                  case AST_PredefinedType::PT_char:
                     return "char";
                  case AST_PredefinedType::PT_wchar:
                     return "wchar";
                  case AST_PredefinedType::PT_boolean:
                     return "boolean";
                  case AST_PredefinedType::PT_octet:
                     return "octet";
                  case AST_PredefinedType::PT_void:
                     return "void";
                  default:
                     break;
               }
            default:
               return t->full_name ();
         }
      }

      void
      BasicIDLVisitor::gen_anonymous_array (AST_Type *a,
                                       AST_Decl *wrapper)
      {
         AST_Array *array = AST_Array::narrow_from_decl (a);
         AST_Type *bt = array->base_type ();

         m_os << this->type_name (bt);
         m_os << " " << try_escape(wrapper->original_local_name ()).c_str();

         for (unsigned long i = 0; i < array->n_dims (); ++i)
         {
            m_os << "[" << array->dims ()[i]->ev ()->u.ulval << "]";
         }
      }

      void
      BasicIDLVisitor::gen_label_value (AST_UnionLabel *node)
      {
         AST_Expression *val = node->label_val ();
         AST_Expression::AST_ExprValue *ev = val->ev ();


         if (this->disc_type_->node_type () == AST_Decl::NT_enum)
         {
            UTL_Scope *s = this->disc_type_->defined_in ();

            if (s == 0)
            {
               m_os << val->n ();
            }
            else
            {
               m_os << ScopeAsDecl (s)->name () << "::"
                                    << val->n ()->last_component ();
            }

            return;
         }

         
         char buffer[64];
         switch (ev->et)
         {
            case AST_Expression::EV_short:
               ::sprintf (buffer, "%hd", ev->u.sval);
               m_os << buffer;
               //m_os << ev->u.sval;
               break;
            case AST_Expression::EV_ushort:
               ::sprintf (buffer, "%hu", ev->u.sval);
               m_os << buffer;
               //m_os << ev->u.usval;
               break;
            case AST_Expression::EV_long:
               m_os << ev->u.lval;
               break;
            case AST_Expression::EV_ulong:
               m_os << ev->u.ulval;
               break;
            case AST_Expression::EV_longlong:
#if ! defined (ACE_LACKS_LONGLONG_T)
               ::sprintf (buffer, "%lld", ev->u.llval);
               m_os << buffer;
#endif /* ! defined (ACE_LACKS_LONGLONG_T) */
               break;
            case AST_Expression::EV_ulonglong:
#if ! defined (ACE_LACKS_LONGLONG_T)
               ::sprintf (buffer, "ACE_UINT64_LITERAL (");
               ::sprintf (buffer, ACE_UINT64_FORMAT_SPECIFIER, ev->u.ullval);
               ::sprintf (buffer, ")");
               m_os << buffer;
#endif /* ! defined (ACE_LACKS_LONGLONG_T) */
               break;
            case AST_Expression::EV_char:
               ::sprintf (buffer, "%c", ev->u.cval);
               m_os << buffer;
                //m_os << ev->u.cval;
               break;
            case AST_Expression::EV_wchar:
               ::sprintf (buffer, "%lc", ev->u.wcval);
               m_os << buffer;
                //m_os << ev->u.wcval;
               break;
            case AST_Expression::EV_bool:
               m_os << (ev->u.bval ? "true" : "false");
               break;
            case AST_Expression::EV_enum:
               m_os << val->n ();
               break;
            default:
               break;
         }
      }


      void
      BasicIDLVisitor::gen_factories (AST_Home *node,
                                 AST_Interface &)
      {
         AST_Operation **item = 0;

         for (ACE_Unbounded_Queue_Iterator<AST_Operation *> i (node->factories ());
              ! i.done ();
              i.advance ())
         {
            i.next (item);

            m_os << ost_nl << ost_nl
                 << node->managed_component ()->name () << " "
                 << try_escape((*item)->original_local_name ()).c_str() << " (";

            this->gen_params (*item, (*item)->argument_count ());

            m_os << ")";

            UTL_ExceptList *exceps = (*item)->exceptions ();

            if (exceps != 0 && exceps->length () > 0)
            {
               this->gen_exception_list (exceps, "", false);
            }
            else
            {
               m_os << ost_idt_nl
                    << "raises (";
            }
            
            m_os << "Components::CreateFailure);" << ost_uidt;
         }
      }

      void
      BasicIDLVisitor::gen_finders (AST_Home *node,
                               AST_Interface &)
      {
         AST_Operation **item = 0;

         for (ACE_Unbounded_Queue_Iterator<AST_Operation *> i (node->finders ());
              ! i.done ();
              i.advance ())
         {
            i.next (item);

            m_os << ost_nl << ost_nl
                 << node->managed_component ()->name () << " "
                 << try_escape((*item)->original_local_name ()).c_str() << " (";

            this->gen_params (*item, (*item)->argument_count ());

            m_os << ")";

            UTL_ExceptList *exceps = (*item)->exceptions ();

            if (exceps != 0 && exceps->length () > 0)
            {
               this->gen_exception_list (exceps, "", false);
            }
            else
            {
               m_os << ost_idt_nl
                    << "raises (";
            }

            m_os << "Components::FinderFailure);" << ost_uidt;
         }
      }

      void
      BasicIDLVisitor::gen_params (UTL_Scope *s, int arg_count)
      {
         if (arg_count > 0)
         {
            m_os << ost_idt << ost_idt;

            for (UTL_ScopeActiveIterator si (s, UTL_Scope::IK_decls);
                 !si.is_done ();)
            {
               if (si.item ()->ast_accept (this) != 0)
               {
                  ACE_ERROR ((LM_ERROR,
                              "BasicIDLVisitor::gen_params - "
                              "codegen for parameters failed\n"));
               }

               si.next ();

               if (!si.is_done ())
               {
                  m_os << ",";
               }
            }

            m_os << ost_uidt_nl << ost_uidt;
         }
      }

      void
      BasicIDLVisitor::gen_exception_list (UTL_ExceptList *exceptions,
                                      const char *prefix,
                                      bool closed)
      {
         if (exceptions != 0 && exceptions->length () > 0)
         {
            m_os << ost_idt_nl
               << prefix << "raises (";

            for (UTL_ExceptlistActiveIterator ei (exceptions);
                 !ei.is_done ();)
            {
               m_os << ei.item ()->name ();

               ei.next ();

               if (!ei.is_done () || !closed)
               {
                  m_os << ", ";
               }
            }

            if (closed)
            {
               m_os << ")" << ost_uidt;
            }
         }
      }

      void
      BasicIDLVisitor::gen_attribute (AST_Attribute *node)
      {
         bool rd_only = node->readonly ();

         // Keep output statements separate because of side effects.
         // No need to check for anonymous array - anonymous types not
         // accepted by parser for attributes.
         m_os << ost_nl << ost_nl
                              << (rd_only ? "readonly " : "") << "attribute ";
         m_os << this->type_name (node->field_type ());
         m_os << " " << try_escape(node->original_local_name ()).c_str();

         this->gen_exception_list (node->get_get_exceptions (),
                                   rd_only ? "" : "get");

         this->gen_exception_list (node->get_set_exceptions (),
                                   "set");

         m_os << ";";

      }




      void
      BasicIDLVisitor::gen_operation (AST_Operation *node)
      {
         m_os << ost_nl << ost_nl;


         if (node->flags () == AST_Operation::OP_oneway)
         {
            m_os << "oneway ";
         }

         m_os << this->type_name (node->return_type ());

         m_os << " " << try_escape(node->original_local_name ()).c_str() << " (";

         this->gen_params (node, node->argument_count ());

         m_os << ")";

         this->gen_exception_list (node->exceptions ());

         m_os << ";";
      }


      void  BasicIDLVisitor::lower(char *s)
      {
         unsigned int i;
         for (i = 0; i < strlen(s); i++) {
            if (s[i] >= 'A' && s[i] <= 'Z')
               s[i] -= ('A' - 'a');
         }
      }

   }; // End of CodeGenerator
}; // End of Cdmw
