/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
/*

COPYRIGHT

Copyright 1992, 1993, 1994 Sun Microsystems, Inc.  Printed in the United
States of America.  All Rights Reserved.

This product is protected by copyright and distributed under the following
license restricting its use.

The Interface Definition Language Compiler Front End (CFE) is made
available for your use provided that you include this license and copyright
notice on all media and documentation and the software program in which
this product is incorporated in whole or part. You may copy and extend
functionality (but may not remove functionality) of the Interface
Definition Language CFE without charge, but you are not authorized to
license or distribute it to anyone else except as part of a product or
program developed by you or with the express written consent of Sun
Microsystems, Inc. ("Sun").

The names of Sun Microsystems, Inc. and any of its subsidiaries or
affiliates may not be used in advertising or publicity pertaining to
distribution of Interface Definition Language CFE as permitted herein.

This license is effective until terminated by Sun for failure to comply
with this license.  Upon termination, you shall destroy or return all code
and documentation for the Interface Definition Language CFE.

INTERFACE DEFINITION LANGUAGE CFE IS PROVIDED AS IS WITH NO WARRANTIES OF
ANY KIND INCLUDING THE WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS
FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR ARISING FROM A COURSE OF
DEALING, USAGE OR TRADE PRACTICE.

INTERFACE DEFINITION LANGUAGE CFE IS PROVIDED WITH NO SUPPORT AND WITHOUT
ANY OBLIGATION ON THE PART OF Sun OR ANY OF ITS SUBSIDIARIES OR AFFILIATES
TO ASSIST IN ITS USE, CORRECTION, MODIFICATION OR ENHANCEMENT.

SUN OR ANY OF ITS SUBSIDIARIES OR AFFILIATES SHALL HAVE NO LIABILITY WITH
RESPECT TO THE INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY
INTERFACE DEFINITION LANGUAGE CFE OR ANY PART THEREOF.

IN NO EVENT WILL SUN OR ANY OF ITS SUBSIDIARIES OR AFFILIATES BE LIABLE FOR
ANY LOST REVENUE OR PROFITS OR OTHER SPECIAL, INDIRECT AND CONSEQUENTIAL
DAMAGES, EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

Use, duplication, or disclosure by the government is subject to
restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
Technical Data and Computer Software clause at DFARS 252.227-7013 and FAR
52.227-19.

Sun, Sun Microsystems and the Sun logo are trademarks or registered
trademarks of Sun Microsystems, Inc.

SunSoft, Inc.
2550 Garcia Avenue
Mountain View, California  94043

NOTE:

SunOS, SunSoft, Sun, Solaris, Sun Microsystems or the Sun logo are
trademarks or registered trademarks of Sun Microsystems, Inc.

 */

#include "global_extern.h"
#include "be_extern.h"
#include "ast_root.h"
#include "engine/EquivalentIDLVisitor.hpp"
#include "engine/InternalIDLVisitor.hpp"
#include "engine/CdmwCifIDLVisitor.hpp"
#include "engine/CheckingVisitor.hpp"
#include "engine/FmtOStringStream.hpp"

#include "engine/Config.hpp"
#include "utl_string.h"
#include "idl_defines.h"

#include <fstream>


namespace
{
   bool M_equivalent_idl = true;
   bool M_internal_idl = true;
   bool M_cdmw_cif_idl = true;

   std::string 
   create_ifndef_macro (const char *fname,
                        const char *prefix,
                        const char *suffix)
   {
      const int MAX_MACRO_NAME = 1024;
      char macro_name [MAX_MACRO_NAME] = { 0 };

      const char *extension = ::strrchr (fname, '.');

      if (extension == 0)
      {
         // File seems to have no extension, so let us take the name
         // as it is.
         extension = fname;
      }

      ::sprintf (macro_name, prefix);

      size_t offset = ACE_OS::strlen (prefix);

      // Convert letters in fname to upper case.
      for (int i = 0; i < (extension - fname); i++)
      {
         if (isalpha (fname [i]))
         {
            macro_name[i + offset] = static_cast<char> (toupper (fname [i]));
         }
         else if (isdigit (fname [i]))
         {
            macro_name[i + offset] = fname[i];
         }
         else
         {
            macro_name[i + offset] = '_';
         }
      }

      ACE_OS::strcat (macro_name, suffix);

      return std::string(macro_name);
   }

};

TAO_IDL_BE_Export void 
BE_version (void)
{
   COG_INFO("Cdmw_idl3_to_idl2, version " << Cdmw::CodeGenerator::BE_VERSION);
}

TAO_IDL_BE_Export int
BE_init (int & argc , char * argv [])
{
   for (int i = 0; i < argc; ++i)
   {
      if (strcmp(argv[i],"-Dgentarget=equivalent-idl") == 0)
      {
         M_equivalent_idl = true;
         M_internal_idl = false;
         M_cdmw_cif_idl = false;
      } 
      else if (strcmp(argv[i],"-Dgentarget=internal-idl") == 0)
      {
         M_equivalent_idl = false;
         M_internal_idl = true;
         M_cdmw_cif_idl = false;
         i++;
      } 
      else if (strcmp(argv[i],"-Dgentarget=cdmw-cif-idl") == 0)
      {
         M_equivalent_idl = false;
         M_internal_idl = false;
         M_cdmw_cif_idl = true;
      } 
      else if (strcmp(argv[i],"-Dgentarget=all") == 0)
      {
         M_equivalent_idl = true;
         M_internal_idl = true;
         M_cdmw_cif_idl = true;
      }
      else if (strstr(argv[i],"-DexcludedFile") != NULL)
      {
         ACE_CString exfilename = argv[i];
         excludedFileName = 
            exfilename.substr (exfilename.rfind ('=') +1 , exfilename.length() );
      }
   }
 
  idl_global->pass_orb_idl (true);
  return 0;
}

TAO_IDL_BE_Export void
BE_post_init (const char * /* files */ [], long /* nfiles */)
{
}

// Clean up before exit, whether successful or not.
// Need not be exported since it is called only from this file.
TAO_IDL_BE_Export void
BE_cleanup (void)
{
   idl_global->destroy ();
}

// Abort this run of the BE.
TAO_IDL_BE_Export void
BE_abort (void)
{
   COG_FATAL("Fatal Error - Aborting\n");

   BE_cleanup ();

   ACE_OS::exit (1);
}

// Do the work of this BE. This is the starting point for code generation.
TAO_IDL_BE_Export void
BE_produce (void)
{
   // Get the root node.
   AST_Decl *d = idl_global->root ();
   AST_Root *ast_root = AST_Root::narrow_from_decl (d);

   if (ast_root == 0)
   {
      COG_FATAL("No Root\n");
//       ACE_ERROR ((LM_ERROR,
//                   ACE_TEXT ("(%N:%l) BE_produce - ")
//                   ACE_TEXT ("No Root\n")));
      BE_abort ();
   }
  

   ACE_CString fn (idl_global->stripped_filename ()->get_string ());

   ACE_CString fnTmp = fn; 
   fn = fn.substr (0, fn.rfind ('.'));

   ACE_CString fn2(fn), fn3("Cdmw_");
   fn2 += "_cif.idl";
   fn3 += fn2;
   fn += "_IDL2.idl";

   const char *path = be_global->output_dir ();

   ACE_CString target_name;
  
   if (path != 0)
   {
      target_name = path;
      target_name += "/";
   }

   bool idl3type = false;
   bool local_idl3type = false;
   ACE_CString target_name2(target_name),target_name3(target_name);
   target_name2 += fn2;
   target_name3 += fn3;
   target_name += fn;

   // Check whether it is an IDL3 file  
   Cdmw::CodeGenerator::CheckingVisitor visitor0;
   if (visitor0.visit_root (ast_root) == -1)
   {
      ACE_ERROR ((LM_ERROR,
                  ACE_TEXT ("(%N:%l) BE_produce -")
                  ACE_TEXT (" failed to accept visitor\n")));
   }
   idl3type = visitor0.is_idl3;
   local_idl3type = visitor0.is_local_idl3;

   // 1st: Generate Equivalent IDL
   if (M_equivalent_idl)
   {

      Cdmw::CodeGenerator::FmtOStringStream equivalent_idl_os;

      std::string macro_name = create_ifndef_macro (fn.c_str (), "_TAO_IDL_", "_IDL_");

      // Generate the #ifndef ... #define statements.
      
      equivalent_idl_os << ost_nl
                        << "#ifndef "<< macro_name.c_str();
      equivalent_idl_os << ost_nl
                        << "#define "<< macro_name.c_str() << ost_nl << ost_nl;
      equivalent_idl_os << ost_nl
                        << "#include <Components.idl> " << ost_nl << ost_nl;

      if (idl3type == true) 
      {
         Cdmw::CodeGenerator::EquivalentIDLVisitor visitor1(equivalent_idl_os);

         if (visitor1.visit_root (ast_root) == -1)
         { 
            ACE_ERROR ((LM_ERROR,
                        ACE_TEXT ("(%N:%l) BE_produce -")
                        ACE_TEXT (" failed to accept visitor\n")));
         }
      }
      else
      {
         equivalent_idl_os << ost_nl << ost_nl
                           << "#include "<< "\"" << fnTmp.c_str () << "\""; 
      }
      equivalent_idl_os << ost_nl << ost_nl
                        << "#endif /* ifndef " << macro_name.c_str() << " */" << ost_nl << ost_nl;

      std::ofstream equivalent_idl_file (target_name.c_str(), std::ios::out);
      if (equivalent_idl_file)
      {
          equivalent_idl_file << equivalent_idl_os.ostr().str() << std::endl;
          equivalent_idl_file.close();
      } else
      {
          ACE_DEBUG ((LM_DEBUG,
                      "Failed to open file %s for writing.\n",
                      target_name.c_str ()));
          return;
      }
   }



   // 2nd: Generate Internal IDL
   if (M_internal_idl)
   {

      Cdmw::CodeGenerator::FmtOStringStream internal_idl_os;

      std::string macro_name = create_ifndef_macro (fn2.c_str (), "_CDMW_IDL_", "_IDL_");

      // Generate the #ifndef ... #define statements.

      internal_idl_os << ost_nl
                      << "#ifndef "<< macro_name.c_str();
      internal_idl_os << ost_nl
                      << "#define "<< macro_name.c_str() << ost_nl << ost_nl;

      Cdmw::CodeGenerator::InternalIDLVisitor visitor2(internal_idl_os);
      if (local_idl3type) 
         visitor2.localIdl3 = true;
      
      if (visitor2.visit_root (ast_root) == -1)
      {
         ACE_ERROR ((LM_ERROR,
                     ACE_TEXT ("(%N:%l) BE_produce -")
                     ACE_TEXT (" failed to accept visitor\n")));
      }

      internal_idl_os << ost_nl << ost_nl
                      << "#endif /* ifndef " << macro_name.c_str() << " */" << ost_nl << ost_nl;

      std::ofstream internal_idl_file (target_name2.c_str(), std::ios::out);
      if (internal_idl_file)
      {
          internal_idl_file << internal_idl_os.ostr().str() << std::endl;
          internal_idl_file.close();
      } else
      {
          ACE_DEBUG ((LM_DEBUG,
                      "Failed to open file %s for writing.\n",
                      target_name2.c_str ()));
          return;
      }
   }

   // 3nd: Generate CDMW CIF IDL
   if (M_cdmw_cif_idl)
   {

      Cdmw::CodeGenerator::FmtOStringStream cif_idl_os;

      std::string macro_name = create_ifndef_macro (fn3.c_str (), "_CDMW_IDL_", "_IDL_");

      // Generate the #ifndef ... #define statements.
      
      cif_idl_os << ost_nl
                 << "#ifndef "<< macro_name.c_str();
      cif_idl_os << ost_nl
                 << "#define "<< macro_name.c_str() << ost_nl << ost_nl;
  
      Cdmw::CodeGenerator::CdmwCifIDLVisitor visitor3(cif_idl_os);
      if (local_idl3type) 
         visitor3.localIdl3 = true;

      if (visitor3.visit_root (ast_root) == -1)
      {
         ACE_ERROR ((LM_ERROR,
                     ACE_TEXT ("(%N:%l) OST_produce -")
                     ACE_TEXT (" failed to accept visitor\n")));
      }
      
      cif_idl_os << ost_nl << ost_nl
                 << "#endif /* ifndef " << macro_name.c_str() << " */" << ost_nl << ost_nl;

      std::ofstream cif_idl_file (target_name3.c_str(), std::ios::out);
      if (cif_idl_file)
      {
          cif_idl_file << cif_idl_os.ostr().str() << std::endl;
          cif_idl_file.close();
      } else
      {
          ACE_DEBUG ((LM_DEBUG,
                      "Failed to open file %s for writing.\n",
                      target_name3.c_str ()));
          return;
      }

   }

   // Clean up.
   BE_cleanup ();
}

//---------------------------------------------//
// Overloading some BE_GlobalData opeations    //
//---------------------------------------------//
TAO_IDL_BE_Export void
BE_GlobalData::usage (void) const
{
   COG_INFO(" -o <dir>\t\tOutput directory for the generated file."
            << " Default is current directory\n"
            << " -Dgentarget=equivalent-idl\t\tGenerate IDL2 equivalent IDL only\n"
            << " -Dgentarget=internal-idl\t\tGenerate IDL2 internal IDL only for component implementors\n"
            << " -Dgentarget=cdmw-cif-idl\t\tGenerate CARDAMOM Container-specific IDL only\n"
            << " [-Dgentarget=all]\t\tGenerate all above IDL. Default is --all-idl\n");
}
