/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
/*

COPYRIGHT

Copyright 1992, 1993, 1994 Sun Microsystems, Inc.  Printed in the United
States of America.  All Rights Reserved.

This product is protected by copyright and distributed under the following
license restricting its use.

The Interface Definition Language Compiler Front End (CFE) is made
available for your use provided that you include this license and copyright
notice on all media and documentation and the software program in which
this product is incorporated in whole or part. You may copy and extend
functionality (but may not remove functionality) of the Interface
Definition Language CFE without charge, but you are not authorized to
license or distribute it to anyone else except as part of a product or
program developed by you or with the express written consent of Sun
Microsystems, Inc. ("Sun").

The names of Sun Microsystems, Inc. and any of its subsidiaries or
affiliates may not be used in advertising or publicity pertaining to
distribution of Interface Definition Language CFE as permitted herein.

This license is effective until terminated by Sun for failure to comply
with this license.  Upon termination, you shall destroy or return all code
and documentation for the Interface Definition Language CFE.

INTERFACE DEFINITION LANGUAGE CFE IS PROVIDED AS IS WITH NO WARRANTIES OF
ANY KIND INCLUDING THE WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS
FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR ARISING FROM A COURSE OF
DEALING, USAGE OR TRADE PRACTICE.

INTERFACE DEFINITION LANGUAGE CFE IS PROVIDED WITH NO SUPPORT AND WITHOUT
ANY OBLIGATION ON THE PART OF Sun OR ANY OF ITS SUBSIDIARIES OR AFFILIATES
TO ASSIST IN ITS USE, CORRECTION, MODIFICATION OR ENHANCEMENT.

SUN OR ANY OF ITS SUBSIDIARIES OR AFFILIATES SHALL HAVE NO LIABILITY WITH
RESPECT TO THE INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY
INTERFACE DEFINITION LANGUAGE CFE OR ANY PART THEREOF.

IN NO EVENT WILL SUN OR ANY OF ITS SUBSIDIARIES OR AFFILIATES BE LIABLE FOR
ANY LOST REVENUE OR PROFITS OR OTHER SPECIAL, INDIRECT AND CONSEQUENTIAL
DAMAGES, EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

Use, duplication, or disclosure by the government is subject to
restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
Technical Data and Computer Software clause at DFARS 252.227-7013 and FAR
52.227-19.

Sun, Sun Microsystems and the Sun logo are trademarks or registered
trademarks of Sun Microsystems, Inc.

SunSoft, Inc.
2550 Garcia Avenue
Mountain View, California  94043

NOTE:

SunOS, SunSoft, Sun, Solaris, Sun Microsystems or the Sun logo are
trademarks or registered trademarks of Sun Microsystems, Inc.

 */

#include "engine/CdmwCifIDLVisitor.hpp"
#include "engine/UtlIdentifierHelper.hpp"
#include "engine/FmtOStringStream.hpp"

#include "be_extern.h"

#include "ast_argument.h"
#include "ast_array.h"
#include "ast_attribute.h"
#include "ast_component_fwd.h"
#include "ast_enum.h"
#include "ast_enum_val.h"
#include "ast_eventtype.h"
#include "ast_eventtype_fwd.h"
#include "ast_exception.h"
#include "ast_factory.h"
#include "ast_field.h"
#include "ast_home.h"
#include "ast_operation.h"
#include "ast_root.h"
#include "ast_sequence.h"
#include "ast_string.h"
#include "ast_structure_fwd.h"
#include "ast_union.h"
#include "ast_union_branch.h"
#include "ast_union_fwd.h"
#include "ast_union_label.h"
#include "ast_valuebox.h"
#include "ast_valuetype_fwd.h"
#include "ast_native.h"
#include "utl_exceptlist.h"
#include "utl_identifier.h"
#include "utl_idlist.h"
#include "utl_string.h"
#include "global_extern.h"
#include "nr_extern.h"

#include <fstream>

extern ACE_CString excludedFileName;

namespace Cdmw
{
   namespace CodeGenerator
   {

      CdmwCifIDLVisitor::CdmwCifIDLVisitor (FmtOStringStream &os)
         : BasicIDLVisitor(os)
      {
         localIdl3 = false;
      }

      CdmwCifIDLVisitor::~CdmwCifIDLVisitor (void)
      {
      }


      int
      CdmwCifIDLVisitor::visit_module (AST_Module *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         // memorize position before module opening
         std::ostringstream::pos_type posIni = stream().ostr().tellp();

         stream() << ost_nl << ost_nl; 

         std::string name;
         if (node->is_nested()) {
             name = try_escape(node->original_local_name ());
         } else {
             name = node->original_local_name()->get_string ();
         }

         if (node->is_nested()) {
            stream() << "module " << name.c_str() << ost_nl
                     << "{" << ost_idt << ost_nl;
         } else {
            stream() << "module Cdmw_" << name.c_str() << ost_nl
                     << "{" << ost_idt << ost_nl;
         }
//          this->check_id_and_version (node);
//          this->check_prefix (node);

         // memorize position after module opening
         std::ostringstream::pos_type posBeforeScope = stream().ostr().tellp();

         if (this->visit_scope (node) != 0)
         {
            ACE_ERROR_RETURN ((LM_ERROR,
                               "CdmwCifIDLVisitor::visit_module - "
                               "codegen for scope failed\n"),
                              -1);
         }


         // memorize position before module closing
         std::ostringstream::pos_type posAfterScope = stream().ostr().tellp();

         // if empty module then do not open the module
         if (posAfterScope == posBeforeScope) {
           // go back before module opening
           stream().ostr().seekp(posIni);
           // blank opened module
           for (int i=posIni; i<posBeforeScope; i++) stream() << " ";
           stream().ostr().seekp(posIni);
           // unindent what was indented
           stream() << ost_uidt;
         }
         else {
             stream() << ost_uidt_nl << ost_nl;
             if (node->is_nested()) {
                 stream() << "}; // end of module " << name.c_str();
             } else {
                 stream() << "}; // end of module Cdmw_" << name.c_str();
             }
         }
         return 0;
      }

      int
      CdmwCifIDLVisitor::visit_interface (AST_Interface *node)
      {
         if (node->imported ())
         {
            return 0;
         }

//          this->check_id_and_version (node);
//          this->check_prefix (node);


         if (!node->is_local () &&
             !node->is_abstract ())
         {
            stream() << ost_nl << ost_nl;
            stream() << "interface " << try_escape(node->original_local_name ()).c_str() 
               << ": ::" << node->full_name ()
               << ost_idt_nl << ",CosLifeCycle::LifeCycleObject " << ost_uidt_nl
               << "{" << ost_nl
               << "};";

         }

//          if (this->visit_scope (node) != 0)
//          {
//             ACE_ERROR_RETURN ((LM_ERROR,
//                                "CdmwCifIDLVisitor::visit_interface - "
//                                "codegen for scope failed\n"),
//                               -1);
//          }


         return 0;
      }

      int
      CdmwCifIDLVisitor::visit_interface_fwd (AST_InterfaceFwd *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         if (!node->is_local () &&
             !node->is_abstract ())
         {
            stream() << ost_nl << ost_nl
               << "interface " << try_escape(node->original_local_name ()).c_str() << ";";
         }
         return 0;
      }

      int
      CdmwCifIDLVisitor::visit_valuebox (AST_ValueBox *node)
      {
         if (node->imported ())
         {
            return 0;
         }
         // Do not generate
         return 0;
      }

      int
      CdmwCifIDLVisitor::visit_valuetype (AST_ValueType *node)
      {
         if (node->imported ())
         {
            return 0;
         }
         // Do not generate

         return 0;
      }

      int
      CdmwCifIDLVisitor::visit_valuetype_fwd (AST_ValueTypeFwd *node)
      {
         if (node->imported ())
         {
            return 0;
         }
         // Do not generate

         return 0;
      }

      int
      CdmwCifIDLVisitor::visit_component (AST_Component *node) // DONE
      {
         if (node->imported ())
         {
            return 0;
         }


//         this->check_id_and_version (node);


         UTL_Scope * scope = node->defined_in ();
         ACE_CString s_scope("::");
         if (scope != 0)
         {
            AST_Decl *dummy = ScopeAsDecl(scope);
            s_scope += dummy->full_name();
         }

         stream() << ost_nl << ost_nl
            << "local interface CCM_"<< node->original_local_name () << "_SessionContext : "
            << s_scope.c_str() << "::CCM_" << node->original_local_name () << "_Context,"
            << ost_idt_nl << "CdmwComponents::SessionContextExt"<< ost_uidt_nl;


         stream() << ost_nl
            << "{"
            << ost_nl
            << "};";
         stream() << ost_nl << ost_nl
            << "interface " << try_escape(node->original_local_name ()).c_str() << ": ::"<<node->full_name()
            << ost_uidt_nl;

         // Inherit from facet interfaces
         {
            ACE_Unbounded_Queue<AST_Component::port_description> &s =
               node->provides ();
            AST_Component::port_description *pd = 0;
            
            for (ACE_Unbounded_Queue_Iterator<AST_Component::port_description> iter (s);
                 ! iter.done ();
                 iter.advance ())
            {
               iter.next (pd);
               stream() << ", ::"<<pd->impl->name () << ost_nl;
            }
         }
         // Inherit from Consumer interfaces
         {
            ACE_Unbounded_Queue<AST_Component::port_description> &s =
               node->consumes ();
            AST_Component::port_description *pd = 0;
            bool has_consumer = false;

            for (ACE_Unbounded_Queue_Iterator<AST_Component::port_description> iter (s);
                 ! iter.done ();
                 iter.advance ())
            {
               has_consumer = true;
               iter.next (pd);
               stream() << ", ::"<<pd->impl->name () << "Consumer" << ost_nl;
            }
            if (has_consumer)
                stream() << ", CosEventComm::PushConsumer" << ost_nl;
         }

         stream()  << ost_nl
                   << "{"
                   << ost_nl
                   << "};";


         return 0;
      }

      int
      CdmwCifIDLVisitor::visit_component_fwd (AST_ComponentFwd *node) // DONE
      {
         if (node->imported ())
         {
            return 0;
         }

         stream() << ost_nl << ost_nl
            << "local interface CCM_"<< node->original_local_name () << "_SessionContext;";

         stream() << ost_nl << ost_nl
            << "interface " << try_escape(node->original_local_name ()).c_str() << ";";

         return 0;
      }

      int
      CdmwCifIDLVisitor::visit_eventtype (AST_EventType *node) // DONE
      {
         if (node->imported ())
         {
            return 0;
         }

         if (this->visit_valuetype (node) != 0)
         {
            ACE_ERROR_RETURN ((LM_ERROR,
                               "CdmwCifIDLVisitor::visit_eventtype - "
                               "codegen for valuetype failed\n"),
                              -1);
         }

         stream() << ost_nl << ost_nl
            << "interface " << node->original_local_name () << "Consumer : "
            << "::" << node->full_name () << "Consumer,"
            << ost_idt_nl 
            << "CosEventComm::PushConsumer," << ost_nl
            << "CosLifeCycle::LifeCycleObject"
            << ost_uidt_nl
            << "{"
            << ost_nl
            << "};";

         return 0;
      }

      int
      CdmwCifIDLVisitor::visit_eventtype_fwd (AST_EventTypeFwd *node) // DONE
      {
         if (node->imported ())
         {
            return 0;
         }

         if (this->visit_valuetype_fwd (node) != 0)
         {
            ACE_ERROR_RETURN ((LM_ERROR,
                               "CdmwCifIDLVisitor::visit_eventtype_fwd - "
                               "codegen for valuetype_fwd failed\n"),
                              -1);
         }
         stream() << ost_nl << ost_nl
            << "interface " << node->original_local_name () << "Consumer;";

         return 0;
      }

      int
      CdmwCifIDLVisitor::visit_home (AST_Home *node) // DONE
      {
         if (node->imported ())
         {
            return 0;
         }

//         this->check_id_and_version (node);
    
         stream() << ost_nl << ost_nl
            << "interface " << try_escape(node->original_local_name ()).c_str() << " : "
            << "::" << node->full_name () << "," << ost_idt_nl 
            << "Components::HomeConfiguration," << ost_nl
            << "CdmwCcmCif::CCMHome," << ost_nl
            << "CdmwCcmCif::KeylessCCMHome" << ost_uidt_nl
            << "{"
            << ost_nl
            << "};";
         return 0;
      }


      int
      CdmwCifIDLVisitor::visit_factory  (AST_Factory *node)
      {
         if (node->imported ())
         {
            return 0;
         }
         // Do not generate

         return 0;
      }

      int
      CdmwCifIDLVisitor::visit_structure (AST_Structure *node)
      {
         if (node->imported ())
         {
            return 0;
         }
         // Do not generate

         return 0;
      }

      int
      CdmwCifIDLVisitor::visit_structure_fwd (AST_StructureFwd *node)
      {
         if (node->imported ())
         {
            return 0;
         }
         // Do not generate

         return 0;
      }

      int
      CdmwCifIDLVisitor::visit_exception (AST_Exception *node)
      {
         if (node->imported ())
         {
            return 0;
         }
         // Do not generate


         return 0;
      }

      int
      CdmwCifIDLVisitor::visit_expression (AST_Expression *)
      {
         return 0;
      }

      int
      CdmwCifIDLVisitor::visit_enum (AST_Enum *node)
      {
         if (node->imported ())
         {
            return 0;
         }
         // Do not generate

         return 0;
      }


      int
      CdmwCifIDLVisitor::visit_union (AST_Union *node)
      {
         if (node->imported ())
         {
            return 0;
         }
         // Do not generate

         return 0;
      }

      int
      CdmwCifIDLVisitor::visit_union_fwd (AST_UnionFwd *node)
      {
         if (node->imported ())
         {
            return 0;
         }
         // Do not generate

         return 0;
      }

      int
      CdmwCifIDLVisitor::visit_union_branch (AST_UnionBranch *node)
      {
         // Do not generate

         return 0;
      }

      int
      CdmwCifIDLVisitor::visit_union_label (AST_UnionLabel *node)
      {
         // Do not generate

         return 0;
      }

      int
      CdmwCifIDLVisitor::visit_constant (AST_Constant *node)
      {
         if (node->imported ())
         {
            return 0;
         }
         // Do not generate

         return 0;
      }

      int
      CdmwCifIDLVisitor::visit_enum_val (AST_EnumVal *)
      {
         return 0;
      }

      int
      CdmwCifIDLVisitor::visit_array (AST_Array *node)
      {
         // Do not generate

         return 0;
      }

      int
      CdmwCifIDLVisitor::visit_sequence (AST_Sequence *node)
      {
         // Do not generate

         return 0;
      }


      int
      CdmwCifIDLVisitor::visit_root (AST_Root *node)
      {
         
         ACE_CString raw_filename;
         ACE_CString filename;

         ACE_CString filename_bis;
         ACE_CString filename_path;

         std::string line;
         bool raw_file_found;

         for (size_t i = 0; i < idl_global->n_included_idl_files (); ++i)
         {
            raw_file_found = false;
            if (i == 0)
            {
               stream() << ost_nl;
            }

            raw_filename = idl_global->included_idl_files ()[i];
            ifstream excludedFiles ( excludedFileName.c_str());//("ExcludedForGeneration.properties");
            if (excludedFiles.is_open())
            {
               while (! excludedFiles.eof() )
               {
                  getline (excludedFiles,line);
                  if (!strcmp(line.c_str(), raw_filename.c_str())) 
                     raw_file_found = true;
               }
               excludedFiles.close();
            }

            if (raw_filename.find (".pidl") != ACE_CString::npos
                || raw_filename == "orb.idl"
                || raw_filename == "Components.idl" 
                || raw_file_found  ) 
            {   
               filename = raw_filename;
            }
            else
            {               
               if (raw_filename.rfind ('/') < 0)
               {
                  filename = raw_filename.substr (0, raw_filename.rfind ('.')) + "_cif.idl"; 
                  stream() << ost_nl
                           << "#include \"Cdmw_" << filename.c_str () << "\"";
               }
               else 
               {
                  filename_path = raw_filename.substr (0, raw_filename.rfind ('/'));
                  filename_bis  = raw_filename.substr (raw_filename.rfind ('/') + 1, raw_filename.length());
                  filename      = filename_bis.substr (0, filename_bis.rfind ('.')) + "_cif.idl";
                  stream() << ost_nl
                           << "#include \""<< filename_path.c_str ()<<"/Cdmw_" << filename.c_str () << "\"";
               }
            }   
         
         }


         ACE_CString fn (idl_global->stripped_filename ()->get_string ());
         fn = fn.substr (0, fn.rfind ('.'));
         fn += "_cif.idl";

         stream() << ost_nl
            << "#include \"" << fn.c_str () << "\"";
         stream() << ost_nl
            << "#include \"CdmwCcmCif.idl\"";
         stream() << ost_nl
            << "#include \"CdmwComponents.idl\"";


         stream() << ost_nl << ost_nl
                  << "#pragma prefix \"thalesgroup.com\"";


         if  (localIdl3)
         {
            if (this->visit_scope (node) != 0)
            {
               ACE_ERROR_RETURN ((LM_ERROR,
                                  "CdmwCifIDLVisitor::visit_root - "
                                  "codegen for scope failed\n"),
                                 -1);
            }
         }
         return 0;
      }

      int
      CdmwCifIDLVisitor::visit_native (AST_Native *node)
      {
         if (node->imported ())
         {
            return 0;
         }
         // Do not generate

         return 0;
      }



      int
      CdmwCifIDLVisitor::visit_scope (UTL_Scope *node)
      {
         for (UTL_ScopeActiveIterator si (node, UTL_Scope::IK_decls);
              !si.is_done ();
              si.next ())
         {
            AST_Decl *d = si.item ();

            if ( (d->node_type () == AST_Decl::NT_typedef) )
                 //        (d->node_type () == AST_Decl::NT_union) ||
                 //    (d->node_type () == AST_Decl::NT_enum)  )
            {
               continue;
            }

            if (d->node_type () == AST_Decl::NT_pre_defined)
            {
               continue;
            }


            // Want to skip the uses_xxxConnection structs added by uses
            // multiple ports.
            // @@@ (JP) This will go away when the visitor is finished, since
            // those uses_xxxConnection structs will not be added to the AST.

            if (ScopeAsDecl (node)->node_type () == AST_Decl::NT_component
                && d->node_type () != AST_Decl::NT_attr)
            {
               continue;
            }


            if (d->ast_accept (this) != 0)
            {
               ACE_ERROR_RETURN ((LM_ERROR,
                                  "CdmwCifIDLVisitor::visit_scope - "
                                  "codegen for scope failed\n"),
                                 -1);
            }
            
         }

         return 0;
      }


      
   }; // End of CodeGenerator
}; // End of Cdmw
