/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
/*

COPYRIGHT

Copyright 1992, 1993, 1994 Sun Microsystems, Inc.  Printed in the United
States of America.  All Rights Reserved.

This product is protected by copyright and distributed under the following
license restricting its use.

The Interface Definition Language Compiler Front End (CFE) is made
available for your use provided that you include this license and copyright
notice on all media and documentation and the software program in which
this product is incorporated in whole or part. You may copy and extend
functionality (but may not remove functionality) of the Interface
Definition Language CFE without charge, but you are not authorized to
license or distribute it to anyone else except as part of a product or
program developed by you or with the express written consent of Sun
Microsystems, Inc. ("Sun").

The names of Sun Microsystems, Inc. and any of its subsidiaries or
affiliates may not be used in advertising or publicity pertaining to
distribution of Interface Definition Language CFE as permitted herein.

This license is effective until terminated by Sun for failure to comply
with this license.  Upon termination, you shall destroy or return all code
and documentation for the Interface Definition Language CFE.

INTERFACE DEFINITION LANGUAGE CFE IS PROVIDED AS IS WITH NO WARRANTIES OF
ANY KIND INCLUDING THE WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS
FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR ARISING FROM A COURSE OF
DEALING, USAGE OR TRADE PRACTICE.

INTERFACE DEFINITION LANGUAGE CFE IS PROVIDED WITH NO SUPPORT AND WITHOUT
ANY OBLIGATION ON THE PART OF Sun OR ANY OF ITS SUBSIDIARIES OR AFFILIATES
TO ASSIST IN ITS USE, CORRECTION, MODIFICATION OR ENHANCEMENT.

SUN OR ANY OF ITS SUBSIDIARIES OR AFFILIATES SHALL HAVE NO LIABILITY WITH
RESPECT TO THE INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY
INTERFACE DEFINITION LANGUAGE CFE OR ANY PART THEREOF.

IN NO EVENT WILL SUN OR ANY OF ITS SUBSIDIARIES OR AFFILIATES BE LIABLE FOR
ANY LOST REVENUE OR PROFITS OR OTHER SPECIAL, INDIRECT AND CONSEQUENTIAL
DAMAGES, EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

Use, duplication, or disclosure by the government is subject to
restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
Technical Data and Computer Software clause at DFARS 252.227-7013 and FAR
52.227-19.

Sun, Sun Microsystems and the Sun logo are trademarks or registered
trademarks of Sun Microsystems, Inc.

SunSoft, Inc.
2550 Garcia Avenue
Mountain View, California  94043

NOTE:

SunOS, SunSoft, Sun, Solaris, Sun Microsystems or the Sun logo are
trademarks or registered trademarks of Sun Microsystems, Inc.

 */

#include "engine/EquivalentIDLVisitor.hpp"
#include "engine/UtlIdentifierHelper.hpp"
#include "engine/FmtOStringStream.hpp"

#include "be_extern.h"

#include "ast_argument.h"
#include "ast_array.h"
#include "ast_attribute.h"
#include "ast_component_fwd.h"
#include "ast_enum.h"
#include "ast_enum_val.h"
#include "ast_eventtype.h"
#include "ast_eventtype_fwd.h"
#include "ast_exception.h"
#include "ast_factory.h"
#include "ast_field.h"
#include "ast_home.h"
#include "ast_operation.h"
#include "ast_root.h"
#include "ast_sequence.h"
#include "ast_string.h"
#include "ast_structure_fwd.h"
#include "ast_union.h"
#include "ast_union_branch.h"
#include "ast_union_fwd.h"
#include "ast_union_label.h"
#include "ast_valuebox.h"
#include "ast_valuetype_fwd.h"
#include "ast_native.h"
#include "utl_exceptlist.h"
#include "utl_identifier.h"
#include "utl_idlist.h"
#include "utl_string.h"
#include "global_extern.h"
#include "nr_extern.h"

#include "engine/CheckingVisitor.hpp"

#include <fstream>

extern ACE_CString excludedFileName;

namespace Cdmw
{
   namespace CodeGenerator
   {

      EquivalentIDLVisitor::EquivalentIDLVisitor (FmtOStringStream &os)
         : BasicIDLVisitor(os)
      {
      }

      EquivalentIDLVisitor::~EquivalentIDLVisitor (void)
      {
      }


      int
      EquivalentIDLVisitor::visit_module (AST_Module *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         stream() << ost_nl << ost_nl;

         std::string name = try_escape(node->original_local_name ());
         stream() << "module " << name.c_str() << ost_nl
                  << "{" << ost_idt;

         this->check_id_and_version (node);
         this->check_prefix (node);

         if (this->visit_scope (node) != 0)
         {
            ACE_ERROR_RETURN ((LM_ERROR,
                               "EquivalentIDLVisitor::visit_module - "
                               "codegen for scope failed\n"),
                              -1);
         }

         stream() << ost_uidt_nl << ost_nl
                  << "}; // end of module " << name.c_str();

         return 0;
      }

      int
      EquivalentIDLVisitor::visit_interface (AST_Interface *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         stream() << ost_nl << ost_nl;

         if (node->is_local ())
         {
            stream() << "local ";
         }

         if (node->is_abstract ())
         {
            stream() << "abstract ";
         }

         stream() << "interface " << try_escape(node->original_local_name ()).c_str();

         AST_Interface **parents = node->inherits ();

         for (long i = 0; i < node->n_inherits (); ++i)
         {
            if (i == 0)
            {
               stream() << " : ";
            }
            else
            {
               stream() << ", ";
            }

            stream() << parents[i]->name ();
         }

         stream() << ost_nl
                  << "{" << ost_idt;

         this->check_id_and_version (node);
         this->check_prefix (node);



         if (this->visit_scope (node) != 0)
         {
            ACE_ERROR_RETURN ((LM_ERROR,
                               "EquivalentIDLVisitor::visit_interface - "
                               "codegen for scope failed\n"),
                              -1);
         }

         stream() << ost_uidt_nl
                  << "};";


         return 0;
      }

      int
      EquivalentIDLVisitor::visit_interface_fwd (AST_InterfaceFwd *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         stream() << ost_nl << ost_nl;

         if (node->is_local ())
         {
            stream() << "local ";
         }

         if (node->is_abstract ())
         {
            stream() << "abstract ";
         }

         stream() << "interface " << try_escape(node->original_local_name ()).c_str() << ";";

         return 0;
      }

      int
      EquivalentIDLVisitor::visit_valuebox (AST_ValueBox *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         stream() << ost_nl << ost_nl
                  << "valuetype " << try_escape(node->original_local_name ()).c_str();

         AST_Type *bt = node->boxed_type ();

         // Keep output statements separate because of side effects.

         if (bt->node_type () == AST_Decl::NT_array)
         {
            this->gen_anonymous_array (bt, node);
         }
         else
         {
            stream() << this->type_name (bt);
         }

         stream() << ";";

         this->check_id_and_version (node);

         return 0;
      }

      int
      EquivalentIDLVisitor::visit_valuetype (AST_ValueType *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         stream() << ost_nl << ost_nl;

         if (node->is_abstract ())
         {
            stream() << "abstract ";
         }

         if (node->custom ())
         {
            stream() << "custom ";
         }

         stream() << "valuetype " << node->original_local_name ();

         AST_Decl::NodeType nt = node->node_type ();
         AST_Interface **parents = node->inherits ();
         long ninherits = node->n_inherits ();

         long i = 0;
         for (i = 0; i < ninherits; ++i)
         {
            if (i == 0)
            {
               stream() << " : ";
            }
            else
            {
               stream() << ", ";
            }

            stream() << parents[i]->name ();
         }

         if (nt == AST_Decl::NT_eventtype)
         {
            stream() << (ninherits == 0 ? " : " : ", ")
                     << "Components::EventBase";
         }

         AST_Interface **supports = node->supports ();

         for (i = 0; i < node->n_supports (); ++i)
         {
            if (i == 0)
            {
               stream() << " supports ";
            }
            else
            {
               stream() << ", ";
            }

            stream() << supports[i]->name ();
         }

         stream() << ost_nl
                  << "{" << ost_idt;

         this->check_id_and_version (node);
         this->check_prefix (node);

         if (this->visit_scope (node) != 0)
         {
            ACE_ERROR_RETURN ((LM_ERROR,
                               "EquivalentIDLVisitor::visit_valuetype - "
                               "codegen for scope failed\n"),
                              -1);
         }

         stream() << ost_uidt_nl
                  << "};";

         return 0;
      }

      int
      EquivalentIDLVisitor::visit_valuetype_fwd (AST_ValueTypeFwd *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         stream() << ost_nl << ost_nl;

         if (node->is_abstract ())
         {
            stream() << "abstract ";
         }

         (void) node->node_type ();

         stream() << "valuetype " << try_escape(node->original_local_name ()).c_str() << ";";

         return 0;
      }

      int
      EquivalentIDLVisitor::visit_component (AST_Component *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         stream() << ost_nl << ost_nl
                  << "interface " << try_escape(node->original_local_name ()).c_str();

         AST_Component *base = node->base_component ();
         long nsupports = node->n_supports ();

         stream() << " : "
                  << (base != 0 ? base->full_name () : "Components::CCMObject");

         for (long i = 0; i < nsupports; ++i)
         {
            stream() << ", " << node->supports ()[i]->name ();
         }

         stream() << ost_nl
                  << "{" << ost_idt;

         this->check_id_and_version (node);




         if (this->visit_scope (node) != 0)
         {
            ACE_ERROR_RETURN ((LM_ERROR,
                               "EquivalentIDLVisitor::visit_component - "
                               "codegen for scope failed\n"),
                              -1);
         }

         this->gen_provides (node);
         this->gen_uses (node);
         this->gen_publishes (node);
         this->gen_emits (node);
         this->gen_consumes (node);

         stream() << ost_uidt_nl
                  << "};";

         return 0;
      }

      int
      EquivalentIDLVisitor::visit_component_fwd (AST_ComponentFwd *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         stream() << ost_nl << ost_nl;

         stream() << "component " << try_escape(node->original_local_name ()).c_str() << ";";


         return 0;
      }

      int
      EquivalentIDLVisitor::visit_eventtype (AST_EventType *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         if (this->visit_valuetype (node) != 0)
         {
            ACE_ERROR_RETURN ((LM_ERROR,
                               "EquivalentIDLVisitor::visit_eventtype - "
                               "codegen for valuetype failed\n"),
                              -1);
         }

         stream() << ost_nl << ost_nl
                  << "interface " << node->original_local_name () << "Consumer : ";

         AST_Interface *parent = 0;
         AST_Decl::NodeType nt = AST_Decl::NT_native;

         if (node->n_inherits () > 0)
         {
            parent = node->inherits ()[0];
            AST_Type *ut = parent->unaliased_type ();
            nt = ut->node_type ();
         }

         if (node->n_inherits () == 0 || nt == AST_Decl::NT_valuetype)
         {
            stream() << "Components::EventConsumerBase";
         }
         else
         {
            stream() << node->inherits ()[0]->name () << "Consumer";
         }

         stream() << ost_nl
                  << "{" << ost_idt_nl
                  << "void push_" << node->original_local_name () << " (in "
                  << try_escape(node->original_local_name ()).c_str() << " the_"
                  << node->original_local_name () << ");" << ost_uidt_nl
                  << "};";

         return 0;
      }

      int
      EquivalentIDLVisitor::visit_eventtype_fwd (AST_EventTypeFwd *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         if (this->visit_valuetype_fwd (node) != 0)
         {
            ACE_ERROR_RETURN ((LM_ERROR,
                               "EquivalentIDLVisitor::visit_eventtype_fwd - "
                               "codegen for valuetype_fwd failed\n"),
                              -1);
         }

         stream() << ost_nl
                  << "interface " << node->original_local_name () << "Consumer;";


         return 0;
      }

      int
      EquivalentIDLVisitor::visit_home (AST_Home *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         ACE_CString explicit_name = node->original_local_name ()->get_string ();
         explicit_name += "Explicit";

         stream() << ost_nl << ost_nl
                  << "interface " << explicit_name.c_str () << " : ";

         AST_Home *base = node->base_home ();

         if (base == 0)
         {
            stream() << "Components::CCMHome";
         }
         else
         {
            stream() << base->name () << "Explicit";
         }

         stream() << ost_nl
                  << "{" << ost_idt;

         this->check_id_and_version (node);

         // Create a temporary interface node corresponding to the one we
         // just generated above for the home explicit interface.
         UTL_ScopedName *sn =
            this->create_scoped_name (0,
                                      explicit_name.c_str (),
                                      0,
                                      ScopeAsDecl (node->defined_in ()));

         AST_Interface xplicit (sn,
                                0,
                                0,
                                0,
                                0,
                                false,
                                false);
         xplicit.set_defined_in (node->defined_in ());

         // Reset the home's decls to be defined in the explicit home interface.
         this->tranfer_scope_elements (node, xplicit);

         this->gen_factories (node, xplicit);
         this->gen_finders (node, xplicit);

         stream() << ost_uidt_nl
                  << "};" << ost_nl << ost_nl;

         xplicit.destroy ();
         sn->destroy ();
         delete sn;
         sn = 0;

         AST_ValueType *key = node->primary_key ();

         // Generate the implicit home interface and its operations.
         stream() << "interface " << node->original_local_name () << "Implicit"
                  << (key == 0 ? " : Components::KeylessCCMHome" : "")
                  << ost_nl
                  << "{" << ost_idt_nl
                  << node->managed_component ()->name () << " create (";

         if (key != 0)
         {
            stream() << "in " << key->name () << " key";
         }

         stream() << ")" << ost_idt_nl
                  << "raises (Components::CreateFailure";

         if (key != 0)
         {
            stream() << ", Components::InvalidKey, Components::DuplicateKeyValue";
         }

         stream() << ");" << ost_uidt;

         if (key != 0)
         {
            stream() << ost_nl << ost_nl
                     << node->managed_component ()->name ()
                     << " find_by_primary_key (in " << key->name ()
                     << " key)" << ost_idt_nl
                     << "raises (Components::InvalidKey, Components::UnknownKeyValue, "
                     << "Components::FinderFailure);" << ost_uidt;

            stream() << ost_nl << ost_nl
                     << "void remove (in " << key->name () << " key)" << ost_idt_nl
                     << "raises (Components::InvalidKey, Components::UnknownKeyValue, "
                     << "Components::RemoveFailure);" << ost_uidt;

            stream() << ost_nl << ost_nl
                     << key->name () << " get_primary_key (in "
                     << node->managed_component ()->name () << " comp);";
         }

         stream() << ost_uidt_nl
                  << "};";

         // Create equivalent interface.
         stream() << ost_nl << ost_nl
                  << "interface " << try_escape(node->original_local_name ()).c_str() << " : "
                  << node->original_local_name () << "Explicit, "
                  << node->original_local_name () << "Implicit" << ost_nl
                  << "{" << ost_nl
                  << "};";


         return 0;
      }


      int
      EquivalentIDLVisitor::visit_factory  (AST_Factory *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         stream() << ost_nl;

         stream() << "factory " << try_escape(node->original_local_name ()).c_str() << " (";

         this->gen_params (node, node->argument_count ());

         stream() << ")";

         this->gen_exception_list (node->exceptions ());

         stream() << ";";

         this->check_id_and_version (node);

         return 0;
      }

      int
      EquivalentIDLVisitor::visit_structure (AST_Structure *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         stream() << ost_nl << ost_nl;

         stream() << "struct " << try_escape(node->original_local_name ()).c_str() << ost_nl
                  << "{" << ost_idt;

         this->check_id_and_version (node);

         if (this->visit_scope (node) != 0)
         {
            ACE_ERROR_RETURN ((LM_ERROR,
                               "EquivalentIDLVisitor::visit_structure - "
                               "codegen for scope failed\n"),
                              -1);
         }

         stream() << ost_uidt_nl
                  << "};";

         return 0;
      }

      int
      EquivalentIDLVisitor::visit_structure_fwd (AST_StructureFwd *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         stream() << ost_nl << ost_nl;

         stream() << "struct " << try_escape(node->original_local_name ()).c_str() << ";";

         return 0;
      }

      int
      EquivalentIDLVisitor::visit_exception (AST_Exception *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         stream() << ost_nl << ost_nl;

         stream() << "exception " << try_escape(node->original_local_name ()).c_str() << ost_nl
                  << "{" << ost_idt;

         this->check_id_and_version (node);

         if (this->visit_scope (node) != 0)
         {
            ACE_ERROR_RETURN ((LM_ERROR,
                               "EquivalentIDLVisitor::visit_exception - "
                               "codegen for scope failed\n"),
                              -1);
         }

         stream() << ost_uidt_nl
                  << "};";

         return 0;
      }

      int
      EquivalentIDLVisitor::visit_expression (AST_Expression *)
      {
         return 0;
      }

      int
      EquivalentIDLVisitor::visit_enum (AST_Enum *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         stream() << ost_nl << ost_nl;

         stream() << "enum " << try_escape(node->original_local_name ()).c_str() << ost_nl
                  << "{" << ost_idt;

         for (UTL_ScopeActiveIterator i (node, UTL_Scope::IK_decls);
              !i.is_done ();)
         {
            stream() << ost_nl;

            AST_EnumVal *ev = AST_EnumVal::narrow_from_decl (i.item ());
            stream() << ev->original_local_name ();

            // Advance here so the check below will work.
            i.next ();

            if (!i.is_done ())
            {
               stream() << ",";
            }
         }

         stream() << ost_uidt_nl
                  << "};";

         this->check_id_and_version (node);

         return 0;
      }

      int
      EquivalentIDLVisitor::visit_operation (AST_Operation *node)
      {
         this->gen_operation(node);

         this->check_id_and_version (node);

         return 0;
      }

      int
      EquivalentIDLVisitor::visit_field (AST_Field *node)
      {
         AST_Field::Visibility v = node->visibility ();

         stream() << ost_nl
                  << (v == AST_Field::vis_PUBLIC
                      ? "public "
                      : (v == AST_Field::vis_PRIVATE ? "private " : ""));

         AST_Type *ft = node->field_type ();

         if (ft->node_type () == AST_Decl::NT_array)
         {
            this->gen_anonymous_array (ft, node);
         }
         else
         {
            // Keep these statements separate because of side effects.
            stream() << this->type_name (ft);
            stream() << " " << try_escape(node->original_local_name ()).c_str();
         }

         stream() << ";";

         return 0;
      }

      int
      EquivalentIDLVisitor::visit_argument (AST_Argument *node)
      {
         stream() << ost_nl;

         switch (node->direction ())
         {
            case AST_Argument::dir_IN:
               stream() << "in ";
               break;
            case AST_Argument::dir_INOUT:
               stream() << "inout ";
               break;
            case AST_Argument::dir_OUT:
               stream() << "out ";
               break;
            default:
               return -1;
         }

         stream() << this->type_name (node->field_type ())
                  << " " << try_escape(node->original_local_name ()).c_str();

         return 0;
      }

     


      int
      EquivalentIDLVisitor::visit_constant (AST_Constant *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         stream() << ost_nl << ost_nl;

         stream() << "const ";

         switch (node->et ())
         {
            case AST_Expression::EV_short:
               stream() << "short";
               break;
            case AST_Expression::EV_ushort:
               stream() << "unsigned short";
               break;
            case AST_Expression::EV_long:
               stream() << "long";
               break;
            case AST_Expression::EV_ulong:
               stream() << "unsigned long";
               break;
            case AST_Expression::EV_longlong:
               stream() << "long long";
               break;
            case AST_Expression::EV_ulonglong:
               stream() << "unsigned long long";
               break;
            case AST_Expression::EV_char:
               stream() << "char";
               break;
            case AST_Expression::EV_wchar:
               stream() << "wchar";
               break;
            case AST_Expression::EV_bool:
               stream() << "boolean";
               break;
            case AST_Expression::EV_octet:
               stream() << "octet";
               break;
            case AST_Expression::EV_float:
               stream() << "float";
               break;
            case AST_Expression::EV_double:
               stream() << "double";
               break;
            case AST_Expression::EV_longdouble:
               stream() << "long double";
               break;
            case AST_Expression::EV_string:
               stream() << "string";
               break;
            case AST_Expression::EV_wstring:
               stream() << "wstring";
               break;
            case AST_Expression::EV_enum:
               stream() << node->enum_full_name ();
               break;
            default:
               break;
         }

         stream() <<  " "
                  << try_escape(node->original_local_name ()).c_str() << " = " << node->constant_value () << ";";

         this->check_id_and_version (node);

         return 0;
      }

      int
      EquivalentIDLVisitor::visit_enum_val (AST_EnumVal *)
      {
         return 0;
      }

      int
      EquivalentIDLVisitor::visit_array (AST_Array *node)
      {
         stream() << node->base_type ()->name ();

         for (unsigned long i = 0; i < node->n_dims (); ++i)
         {
            stream() << "[" << node->dims ()[i] << "]";
         }

         return 0;
      }

      int
      EquivalentIDLVisitor::visit_sequence (AST_Sequence *node)
      {
         // Keep output statements separate because of side effects.
         stream() << "sequence<";
         stream() << this->type_name (node->base_type ());

         if (!node->unbounded ())
         {
            stream() << ", " << node->max_size ()->ev ()->u.ulval;
         }

         stream() << "> ";

         return 0;
      }

      int
      EquivalentIDLVisitor::visit_string (AST_String *node)
      {
         stream() << (node->width () > 1 ? "w" : "") << "string";

         unsigned long bound = node->max_size ()->ev ()->u.ulval;

         if (bound > 0)
         {
            stream() << "<" << bound << ">";
         }

         return 0;
      }

      int
      EquivalentIDLVisitor::visit_typedef (AST_Typedef *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         stream() << ost_nl << ost_nl
                  << "typedef ";

         AST_Type *bt = node->base_type ();

         // Keep output statements separate because of side effects.

         if (bt->node_type () == AST_Decl::NT_array)
         {
            this->gen_anonymous_array (bt, node);
         }
         else
         {
            stream() << this->type_name (bt);
            stream() << " " << try_escape(node->original_local_name ()).c_str();
         }

         stream() << ";";

         this->check_id_and_version (node);

         return 0;
      }

      int
      EquivalentIDLVisitor::visit_root (AST_Root *node)
      {
        

         ACE_CString raw_filename;
         ACE_CString filename;

         std::string line;
         bool raw_file_found;

         for (size_t i = 0; i < idl_global->n_included_idl_files (); ++i)
         {
             raw_file_found = false;
            if (i == 0)
            {
               stream() << ost_nl;
            }

            raw_filename = idl_global->included_idl_files ()[i];

            ifstream excludedFiles ( excludedFileName.c_str());// "ExcludedForGeneration.properties");
            if (excludedFiles.is_open())
            {
               while (! excludedFiles.eof() )
               {
                  getline (excludedFiles,line);
                  if (!strcmp(line.c_str(), raw_filename.c_str())) 
                     raw_file_found = true;
               }    
              excludedFiles.close(); 
            }   

            if (raw_filename.find (".pidl") != ACE_CString::npos
                || raw_filename == "orb.idl"
                || raw_filename == "Components.idl"
                || raw_file_found  )
            {
               filename = raw_filename;
            }
            else
            {
               filename =
                  raw_filename.substr (0, raw_filename.rfind ('.')) + "_IDL2.idl";
            }

            stream() << ost_nl
                     << "#include \"" << filename.c_str () << "\"";
         }

         const char *pfix = node->prefix ();

         if (ACE_OS::strcmp (pfix, "") != 0)
         {
            stream() << ost_nl << ost_nl
                     << "#pragma prefix \"" << pfix << "\"";
         }
         

         


         if (this->visit_scope (node) != 0)
         {
            ACE_ERROR_RETURN ((LM_ERROR,
                               "EquivalentIDLVisitor::visit_root - "
                               "codegen for scope failed\n"),
                              -1);
         }

         

         return 0;
      }


      void
      EquivalentIDLVisitor::gen_provides (AST_Component *node)
      {
         ACE_Unbounded_Queue<AST_Component::port_description> &s =
            node->provides ();
         AST_Component::port_description *pd = 0;

         for (ACE_Unbounded_Queue_Iterator<AST_Component::port_description> iter (s);
              ! iter.done ();
              iter.advance ())
         {
            iter.next (pd);

            Identifier * id__ = original_local_name(pd->id);

            stream() << ost_nl << ost_nl
                     << pd->impl->name () << " provide_" << id__ << " ();";
            delete id__;


         }
      }

      void
      EquivalentIDLVisitor::gen_uses (AST_Component *node)
      {
         ACE_Unbounded_Queue<AST_Component::port_description> &s =
            node->uses ();
         AST_Component::port_description *pd = 0;

         for (ACE_Unbounded_Queue_Iterator<AST_Component::port_description> iter (s);
              ! iter.done ();
              iter.advance ())
         {
            iter.next (pd);

            stream() << ost_nl << ost_nl;
            Identifier * id__ = original_local_name(pd->id);

            if (pd->is_multiple)
            {
               stream() << "struct " << id__ << "Connection" << ost_nl
                        << "{" << ost_idt_nl
                        << pd->impl->name () << " objref;" << ost_nl
                        << "Components::Cookie ck;" << ost_uidt_nl
                        << "};" << ost_nl << ost_nl
                        << "typedef sequence<" << id__ << "Connection> "
                        << id__ << "Connections;"
                        << ost_nl << ost_nl
                        << "Components::Cookie connect_" << id__ << " (in "
                        << pd->impl->name () << " connection)" << ost_idt_nl
                        << "raises (Components::ExceededConnectionLimit, "
                        << "Components::InvalidConnection);" << ost_uidt_nl << ost_nl
                        << pd->impl->name () << " disconnect_" << id__
                        << " (in Components::Cookie ck)" << ost_idt_nl
                        << "raises (Components::InvalidConnection);"
                        << ost_uidt_nl << ost_nl
                        << id__ << "Connections get_connections_" << id__
                        << " ();";
            }
            else
            {
               stream() << "void connect_" << id__ << " (in "
                        << pd->impl->name () << " conxn)" << ost_idt_nl
                        << "raises (Components::AlreadyConnected, "
                        << "Components::InvalidConnection);" << ost_uidt_nl << ost_nl
                        << pd->impl->name () << " disconnect_" << id__
                        << " ()" << ost_idt_nl
                        << "raises (Components::NoConnection);" << ost_uidt_nl << ost_nl
                        << pd->impl->name () << " get_connection_" << id__
                        << " ();";
            }
            delete id__;
         }
      }

      void
      EquivalentIDLVisitor::gen_publishes (AST_Component *node)
      {
         ACE_Unbounded_Queue<AST_Component::port_description> &s =
            node->publishes ();
         AST_Component::port_description *pd = 0;

         for (ACE_Unbounded_Queue_Iterator<AST_Component::port_description> iter (s);
              ! iter.done ();
              iter.advance ())
         {
            iter.next (pd);

            Identifier * id__ = original_local_name(pd->id);

            stream() << ost_nl << ost_nl
                     << "Components::Cookie subscribe_" << id__
                     << " (in " << pd->impl->name () <<"Consumer consumer)"
                     << ost_idt_nl
                     << "raises (Components::ExceededConnectionLimit);"
                     << ost_uidt_nl << ost_nl
                     << pd->impl->name () << "Consumer unsubscribe_" << id__
                     << " (in Components::Cookie ck)" << ost_idt_nl
                     << "raises (Components::InvalidConnection);" << ost_uidt;
            delete id__;
         }
      }

      void
      EquivalentIDLVisitor::gen_emits (AST_Component *node)
      {
         ACE_Unbounded_Queue<AST_Component::port_description> &s =
            node->emits ();
         AST_Component::port_description *pd = 0;

         for (ACE_Unbounded_Queue_Iterator<AST_Component::port_description> iter (s);
              ! iter.done ();
              iter.advance ())
         {
            iter.next (pd);
            Identifier * id__ = original_local_name(pd->id);

            stream() << ost_nl << ost_nl
                     << "void connect_" << id__
                     << " (in " << pd->impl->name ()
                     << "Consumer consumer)" << ost_idt_nl
                     << "raises (Components::AlreadyConnected);"
                     << ost_uidt_nl << ost_nl
                     << pd->impl->name () << "Consumer disconnect_" << id__
                     << " ()" << ost_idt_nl
                     << "raises (Components::NoConnection);" << ost_uidt;
            delete id__;
         }
      }

      void
      EquivalentIDLVisitor::gen_consumes (AST_Component *node)
      {
         ACE_Unbounded_Queue<AST_Component::port_description> &s =
            node->consumes ();
         AST_Component::port_description *pd = 0;

         for (ACE_Unbounded_Queue_Iterator<AST_Component::port_description> iter (s);
              ! iter.done ();
              iter.advance ())
         {
            iter.next (pd);

            Identifier * id__ = original_local_name(pd->id);
            stream() << ost_nl << ost_nl
                     << pd->impl->name () << "Consumer get_consumer_" << id__
                     << " ();";
            delete id__;
         }
      }




      UTL_ScopedName *
      EquivalentIDLVisitor::create_scoped_name (const char *prefix,
                                                const char *local_name,
                                                const char *suffix,
                                                AST_Decl *parent)
      {
         ACE_CString local_string (prefix,
                                   0,
                                   0);
         local_string += local_name;
         local_string += suffix;
         Identifier *local_id = 0;
         ACE_NEW_RETURN (local_id,
                         Identifier (local_string.fast_rep ()),
                         0);
         UTL_ScopedName *last_segment = 0;
         ACE_NEW_RETURN (last_segment,
                         UTL_ScopedName (local_id,
                                         0),
                         0);
         UTL_ScopedName *full_name =
            static_cast<UTL_ScopedName *> (parent->name ()->copy ());
         full_name->nconc (last_segment);
         return full_name;
      }

      void
      EquivalentIDLVisitor::tranfer_scope_elements (AST_Home *src,
                                                    AST_Interface &dst)
      {
         // Transfer the elements of the home's scope to the temporary
         // explicit home interface.
         for (UTL_ScopeActiveIterator src_iter (src, UTL_Scope::IK_decls);
              ! src_iter.is_done ();
              src_iter.next ())
         {
            AST_Decl *d = src_iter.item ();
            d->set_defined_in (&dst);
            UTL_ScopedName *new_name =
               this->create_scoped_name (0,
                                         d->original_local_name ()->get_string (), // TODO: do we need try_escape()?
                                         0,
                                         &dst);
            d->set_name (new_name);
            dst.add_to_scope (d);
         }

         // Visit the transferred scope elements normally to generate the IDL.
         // This way referenced items will have the interface's name in the
         // scoped name instead of the home's name.
         for (UTL_ScopeActiveIterator dst_iter (&dst, UTL_Scope::IK_decls);
              ! dst_iter.is_done ();
              dst_iter.next ())
         {
            if (dst_iter.item ()->ast_accept (this) != 0)
            {
               ACE_ERROR ((LM_ERROR,
                           "EquivalentIDLVisitor::tranfer_scope_elements - "
                           "codegen for destination scope failed\n"));
            }
         }
      }

      void
      EquivalentIDLVisitor::gen_factories (AST_Home *node,
                                           AST_Interface &)
      {
         AST_Operation **item = 0;

         for (ACE_Unbounded_Queue_Iterator<AST_Operation *> i (node->factories ());
              ! i.done ();
              i.advance ())
         {
            i.next (item);

            stream() << ost_nl << ost_nl
                     << node->managed_component ()->name () << " "
                     << try_escape((*item)->original_local_name ()).c_str() << " (";

            this->gen_params (*item, (*item)->argument_count ());

            stream() << ")";

            UTL_ExceptList *exceps = (*item)->exceptions ();

            if (exceps != 0 && exceps->length () > 0)
            {
               this->gen_exception_list (exceps, "", false);
            }
            else
            {
               stream() << ost_idt_nl
                        << "raises (";
            }

            stream() << "Components::CreateFailure);" << ost_uidt;
         }
      }

      void
      EquivalentIDLVisitor::gen_finders (AST_Home *node,
                                         AST_Interface &)
      {
         AST_Operation **item = 0;

         for (ACE_Unbounded_Queue_Iterator<AST_Operation *> i (node->finders ());
              ! i.done ();
              i.advance ())
         {
            i.next (item);

            stream() << ost_nl << ost_nl
                     << node->managed_component ()->name () << " "
                     << try_escape((*item)->original_local_name ()).c_str() << " (";

            this->gen_params (*item, (*item)->argument_count ());

            stream() << ")";

            UTL_ExceptList *exceps = (*item)->exceptions ();

            if (exceps != 0 && exceps->length () > 0)
            {
               this->gen_exception_list (exceps, "", false);
            }
            else
            {
               stream() << ost_idt_nl
                        << "raises (";
            }

            stream() << "Components::FinderFailure);" << ost_uidt;
         }
      }


   }; // End of CodeGenerator
}; // End of Cdmw
