/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
/*

COPYRIGHT

Copyright 1992, 1993, 1994 Sun Microsystems, Inc.  Printed in the United
States of America.  All Rights Reserved.

This product is protected by copyright and distributed under the following
license restricting its use.

The Interface Definition Language Compiler Front End (CFE) is made
available for your use provided that you include this license and copyright
notice on all media and documentation and the software program in which
this product is incorporated in whole or part. You may copy and extend
functionality (but may not remove functionality) of the Interface
Definition Language CFE without charge, but you are not authorized to
license or distribute it to anyone else except as part of a product or
program developed by you or with the express written consent of Sun
Microsystems, Inc. ("Sun").

The names of Sun Microsystems, Inc. and any of its subsidiaries or
affiliates may not be used in advertising or publicity pertaining to
distribution of Interface Definition Language CFE as permitted herein.

This license is effective until terminated by Sun for failure to comply
with this license.  Upon termination, you shall destroy or return all code
and documentation for the Interface Definition Language CFE.

INTERFACE DEFINITION LANGUAGE CFE IS PROVIDED AS IS WITH NO WARRANTIES OF
ANY KIND INCLUDING THE WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS
FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR ARISING FROM A COURSE OF
DEALING, USAGE OR TRADE PRACTICE.

INTERFACE DEFINITION LANGUAGE CFE IS PROVIDED WITH NO SUPPORT AND WITHOUT
ANY OBLIGATION ON THE PART OF Sun OR ANY OF ITS SUBSIDIARIES OR AFFILIATES
TO ASSIST IN ITS USE, CORRECTION, MODIFICATION OR ENHANCEMENT.

SUN OR ANY OF ITS SUBSIDIARIES OR AFFILIATES SHALL HAVE NO LIABILITY WITH
RESPECT TO THE INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY
INTERFACE DEFINITION LANGUAGE CFE OR ANY PART THEREOF.

IN NO EVENT WILL SUN OR ANY OF ITS SUBSIDIARIES OR AFFILIATES BE LIABLE FOR
ANY LOST REVENUE OR PROFITS OR OTHER SPECIAL, INDIRECT AND CONSEQUENTIAL
DAMAGES, EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

Use, duplication, or disclosure by the government is subject to
restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
Technical Data and Computer Software clause at DFARS 252.227-7013 and FAR
52.227-19.

Sun, Sun Microsystems and the Sun logo are trademarks or registered
trademarks of Sun Microsystems, Inc.

SunSoft, Inc.
2550 Garcia Avenue
Mountain View, California  94043

NOTE:

SunOS, SunSoft, Sun, Solaris, Sun Microsystems or the Sun logo are
trademarks or registered trademarks of Sun Microsystems, Inc.

 */

#include "engine/FmtOStringStream.hpp"
#include "ast_expression.h"
#include "utl_identifier.h"
#include "utl_idlist.h"
#include "utl_string.h"

OST_NL::OST_NL (void)
{
}

OST_INDENT::OST_INDENT (int do_now)
    :  do_now_ (do_now)
{
}

OST_UNINDENT::OST_UNINDENT (int do_now)
    :  do_now_ (do_now)
{
}

const OST_NL ost_nl;
const OST_INDENT ost_idt;
const OST_INDENT ost_idt_nl (1);
const OST_UNINDENT ost_uidt;
const OST_UNINDENT ost_uidt_nl (1);

namespace Cdmw
{
namespace CodeGenerator
{

/*
FmtOStringStream::FmtOStringStream (std::ostringstream & strStream)
    : m_stream(strStream)
{
}
*/
FmtOStringStream::FmtOStringStream (void)
    : m_indent_level (0)
{
}

FmtOStringStream::~FmtOStringStream (void)
{
}

int
FmtOStringStream::incr_indent (unsigned short flag)
{
  m_indent_level++;

  if (flag != 0)
    {
      return this->indent ();
    }
  else
    {
      // Do not indent output.
      return 0;
    }
}

// Indentation
int
FmtOStringStream::decr_indent (unsigned short flag)
{
  this->m_indent_level--;
  // Just in case somebody gets "unindent happy".
  if (this->m_indent_level < 0)
    {
      this->m_indent_level = 0;
    }
  if (flag != 0)
    {
      return this->indent ();
    }
  else
    {
      // Do not indent output.
      return 0;
    }
}

int
FmtOStringStream::reset (void)
{
  this->m_indent_level = 0;
  return 0;
}

// Indented print.
int
FmtOStringStream::indent (void)
{
  // Based on the current indentation level, leave appropriate number of blank
  // spaces in the output.
  if (this->m_indent_level > 0)
    {
      for (int i = 0; i < this->m_indent_level; i++)
        {
            m_stream << " ";
        }
    }

  return 0;
}

int
FmtOStringStream::nl (void)
{
  m_stream << std::endl;
  this->indent ();
  return 0;
}

FmtOStringStream &
FmtOStringStream::print (Identifier *id)
{
  m_stream << id->get_string ();
  return *this;
}

FmtOStringStream &
FmtOStringStream::print (UTL_IdList *idl)
{
  long first = true;
  long second = false;
  Identifier *id = 0;

  for (UTL_IdListActiveIterator i (idl); !i.is_done (); i.next ())
    {
      if (!first)
        {
          *this << "::";
        }
      else if (second)
        {
          first = second = false;
        }

      // Print the identifier.
      id = i.item ();
      *this << id;

      if (first)
        {
          if (::strcmp (id->get_string (), "") != 0)
            {
              // Does not start with a "".
              first = false;
            }
          else
            {
              second = true;
            }
        }
    }

  return *this;
}

FmtOStringStream&
FmtOStringStream::print (AST_Expression *expr)
{
  AST_Expression::AST_ExprValue *ev = expr->ev ();

  if (ev)
  {   
      char buffer[64];
      switch (ev->et)
        {
        case AST_Expression::EV_short:
            ::sprintf (buffer, "%hd", ev->u.sval);
            m_stream << buffer;
          break;
        case AST_Expression::EV_ushort:
          ::sprintf (buffer, "%hu%c", ev->u.usval, 'U');
            m_stream << buffer;
          break;
        case AST_Expression::EV_long:
          ::sprintf (buffer, "%ld", ev->u.lval);
            m_stream << buffer;
          break;
        case AST_Expression::EV_ulong:
          ::sprintf (buffer, "%lu%c", ev->u.ulval, 'U');
            m_stream << buffer;
          break;
        case AST_Expression::EV_longlong:
#if ! defined (ACE_LACKS_LONGLONG_T)
          ::sprintf (buffer, "%lld", ev->u.llval);
            m_stream << buffer;
#endif /* ! defined (ACE_LACKS_LONGLONG_T) */
          break;
        case AST_Expression::EV_ulonglong:
#if ! defined (ACE_LACKS_LONGLONG_T)
          ::sprintf (buffer, "ACE_UINT64_LITERAL (");
          ::sprintf (buffer, ACE_UINT64_FORMAT_SPECIFIER, ev->u.ullval);
          ::sprintf (buffer, ")");
            m_stream << buffer;
#endif /* ! defined (ACE_LACKS_LONGLONG_T) */
          break;
        case AST_Expression::EV_float:
          ::sprintf (buffer, "%f%c", ev->u.fval, 'F');
            m_stream << buffer;
          break;
        case AST_Expression::EV_double:
          ::sprintf (buffer, "%24.16G", ev->u.dval);
            m_stream << buffer;
          break;
        case AST_Expression::EV_longdouble:
          break;
        case AST_Expression::EV_char:
          // isprint() sees \ and ' as printable characters
          // so we have to test for them first.
          if (ev->u.cval == '\\')
            ::sprintf (buffer, "'\\\\'");
          else if (ev->u.cval == '\'')
            ::sprintf (buffer, "'\\''");

          // This handles hex and octal escape sequences
          // that would print out either as weird characters
          // or as an unsigned number too large for a char.
          else if ((unsigned char) ev->u.cval > ACE_CHAR_MAX)
            ::sprintf (buffer, "%hd", ev->u.cval);
          else if (isprint (ev->u.cval))
            ::sprintf (buffer, "'%c'", ev->u.cval);
          else if (iscntrl (ev->u.cval))
            switch (ev->u.cval)
              {
                case '\n':
                  ::sprintf (buffer, "'\\n'");
                  break;
                case '\t':
                  ::sprintf (buffer, "'\\t'");
                  break;
                case '\r':
                  ::sprintf (buffer, "'\\r'");
                  break;
                case '\v':
                  ::sprintf (buffer, "'\\v'");
                  break;
                case '\f':
                  ::sprintf (buffer, "'\\f'");
                  break;
                case '\b':
                  ::sprintf (buffer, "'\\b'");
                  break;
                case '\a':
                  ::sprintf (buffer, "'\\a'");
                  break;
                case '\?':
                  ::sprintf (buffer, "'?'");
                  break;
              default:
                ::sprintf (buffer, "'\\x%x'", ev->u.cval);
              }
           else
            ::sprintf (buffer, "'\\x%x'", ev->u.cval);

            m_stream << buffer;
          break;
        case AST_Expression::EV_wchar:
          ::sprintf (buffer, "L'%lc'", ev->u.wcval);
          m_stream << buffer;
          break;
        case AST_Expression::EV_octet:
          ::sprintf (buffer, "%d", ev->u.oval);
          m_stream << buffer;
          break;
        case AST_Expression::EV_bool:
          ::sprintf (buffer, "%s", ev->u.bval ? "true" : "false");
          m_stream << buffer;
          break;
        case AST_Expression::EV_string:
          ::sprintf (buffer, "\"%s\"", ev->u.strval->get_string ());
          m_stream << buffer;
          break;
        case AST_Expression::EV_wstring:
          ::sprintf (buffer, "L\"%s\"", ev->u.wstrval);
          m_stream << buffer;
          break;
        case AST_Expression::EV_enum:
          this->print (expr->n ());
          break;
        default:
          break;
        }
    }
  else
    {
      // XXXASG: need to add code here
    }

  return *this;
}


FmtOStringStream &
FmtOStringStream::operator<< (const char *str)
{
  m_stream << str;
  return *this;
}

FmtOStringStream &
FmtOStringStream::operator<< (const unsigned long num)
{
  m_stream << num;
  return *this;
}

FmtOStringStream &
FmtOStringStream::operator<< (const long num)
{
  m_stream << num;
  return *this;
}

FmtOStringStream &
FmtOStringStream::operator<< (const OST_NL& i)
{
  m_stream << std::endl;
  this->indent ();

  return *this;
}

FmtOStringStream &
FmtOStringStream::operator<< (const OST_INDENT& i)
{
  this->incr_indent (0);

  if (i.do_now_)
    {
      this->nl ();
    }

  return *this;
}

FmtOStringStream &
FmtOStringStream::operator<< (const OST_UNINDENT& i)
{
  this->decr_indent (0);

  if (i.do_now_)
    {
      this->nl ();
    }

  return *this;
}

FmtOStringStream &
FmtOStringStream::operator<< (Identifier *id)
{
  return this->print (id);
}

FmtOStringStream &
FmtOStringStream::operator<< (UTL_IdList *id)
{
  return this->print (id);
}

FmtOStringStream &
FmtOStringStream::operator<< (AST_Expression *expr)
{
  return this->print (expr);
}

} // End of CodeGenerator
} // End of Cdmw
