/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
/*

COPYRIGHT

Copyright 1992, 1993, 1994 Sun Microsystems, Inc.  Printed in the United
States of America.  All Rights Reserved.

This product is protected by copyright and distributed under the following
license restricting its use.

The Interface Definition Language Compiler Front End (CFE) is made
available for your use provided that you include this license and copyright
notice on all media and documentation and the software program in which
this product is incorporated in whole or part. You may copy and extend
functionality (but may not remove functionality) of the Interface
Definition Language CFE without charge, but you are not authorized to
license or distribute it to anyone else except as part of a product or
program developed by you or with the express written consent of Sun
Microsystems, Inc. ("Sun").

The names of Sun Microsystems, Inc. and any of its subsidiaries or
affiliates may not be used in advertising or publicity pertaining to
distribution of Interface Definition Language CFE as permitted herein.

This license is effective until terminated by Sun for failure to comply
with this license.  Upon termination, you shall destroy or return all code
and documentation for the Interface Definition Language CFE.

INTERFACE DEFINITION LANGUAGE CFE IS PROVIDED AS IS WITH NO WARRANTIES OF
ANY KIND INCLUDING THE WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS
FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR ARISING FROM A COURSE OF
DEALING, USAGE OR TRADE PRACTICE.

INTERFACE DEFINITION LANGUAGE CFE IS PROVIDED WITH NO SUPPORT AND WITHOUT
ANY OBLIGATION ON THE PART OF Sun OR ANY OF ITS SUBSIDIARIES OR AFFILIATES
TO ASSIST IN ITS USE, CORRECTION, MODIFICATION OR ENHANCEMENT.

SUN OR ANY OF ITS SUBSIDIARIES OR AFFILIATES SHALL HAVE NO LIABILITY WITH
RESPECT TO THE INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY
INTERFACE DEFINITION LANGUAGE CFE OR ANY PART THEREOF.

IN NO EVENT WILL SUN OR ANY OF ITS SUBSIDIARIES OR AFFILIATES BE LIABLE FOR
ANY LOST REVENUE OR PROFITS OR OTHER SPECIAL, INDIRECT AND CONSEQUENTIAL
DAMAGES, EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

Use, duplication, or disclosure by the government is subject to
restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
Technical Data and Computer Software clause at DFARS 252.227-7013 and FAR
52.227-19.

Sun, Sun Microsystems and the Sun logo are trademarks or registered
trademarks of Sun Microsystems, Inc.

SunSoft, Inc.
2550 Garcia Avenue
Mountain View, California  94043

NOTE:

SunOS, SunSoft, Sun, Solaris, Sun Microsystems or the Sun logo are
trademarks or registered trademarks of Sun Microsystems, Inc.

 */

#include "engine/InternalIDLVisitor.hpp"
#include "engine/FmtOStringStream.hpp"

#include "engine/UtlIdentifierHelper.hpp"
#include "be_extern.h"

#include "ast_argument.h"
#include "ast_array.h"
#include "ast_attribute.h"
#include "ast_component_fwd.h"
#include "ast_enum.h"
#include "ast_enum_val.h"
#include "ast_eventtype.h"
#include "ast_eventtype_fwd.h"
#include "ast_exception.h"
#include "ast_factory.h"
#include "ast_field.h"
#include "ast_home.h"
#include "ast_operation.h"
#include "ast_root.h"
#include "ast_sequence.h"
#include "ast_string.h"
#include "ast_structure_fwd.h"
#include "ast_union.h"
#include "ast_union_branch.h"
#include "ast_union_fwd.h"
#include "ast_union_label.h"
#include "ast_valuebox.h"
#include "ast_valuetype_fwd.h"
#include "ast_native.h"
#include "utl_exceptlist.h"
#include "utl_identifier.h"
#include "utl_idlist.h"
#include "utl_string.h"
#include "global_extern.h"
#include "nr_extern.h"
#include <fstream>
#include <sstream>

extern ACE_CString excludedFileName;

namespace Cdmw
{
   namespace CodeGenerator
   {

      InternalIDLVisitor::InternalIDLVisitor (FmtOStringStream &os)
         : BasicIDLVisitor(os)
      {
         localIdl3 = false;
      }

      InternalIDLVisitor::~InternalIDLVisitor (void)
      {
      }

      int
      InternalIDLVisitor::visit_scope (UTL_Scope *node)
      {
         for (UTL_ScopeActiveIterator si (node, UTL_Scope::IK_decls);
              !si.is_done ();
              si.next ())
         {
            AST_Decl *d = si.item ();

            if   (d->node_type () == AST_Decl::NT_typedef)
            {
               continue;
            }

            if (d->node_type () == AST_Decl::NT_pre_defined)
            {
               continue;
            }


            // Want to skip the uses_xxxConnection structs added by uses
            // multiple ports.
            // @@@ (JP) This will go away when the visitor is finished, since
            // those uses_xxxConnection structs will not be added to the AST.

            if (ScopeAsDecl (node)->node_type () == AST_Decl::NT_component
                && d->node_type () != AST_Decl::NT_attr)
            {
               continue;
            }


            if (d->ast_accept (this) != 0)
            {
               ACE_ERROR_RETURN ((LM_ERROR,
                                  "InternalIDLVisitor::visit_scope - "
                                  "codegen for scope failed\n"),
                                 -1);
            }
            
         }

         return 0;
      }



      int
      InternalIDLVisitor::visit_module (AST_Module *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         std::string name = try_escape(node->original_local_name ());

         // memorize position before module opening
         std::ostringstream::pos_type posIni = stream().ostr().tellp();

         stream() << ost_nl << ost_nl;

         stream() << "module " << name.c_str() << ost_nl
            << "{" << ost_idt;

//          this->check_id_and_version (node);
//          this->check_prefix (node);

         // memorize position after module opening
         std::ostringstream::pos_type posBeforeScope = stream().ostr().tellp();

         if (this->visit_scope (node) != 0)
         {
            ACE_ERROR_RETURN ((LM_ERROR,
                               "InternalIDLVisitor::visit_module - "
                               "codegen for scope failed\n"),
                              -1);
         }

         // memorize position before module closing
         std::ostringstream::pos_type posAfterScope = stream().ostr().tellp();

         // if empty module then do not open the module
         if (posAfterScope == posBeforeScope) {
           // go back before module opening
           stream().ostr().seekp(posIni);
           // blank opened module
           for (int i=posIni; i<posBeforeScope; i++) stream() << " ";
           stream().ostr().seekp(posIni);
           // unindent what was indented
           stream() << ost_uidt;
         }
         else {
           stream() << ost_uidt_nl << ost_nl
                    << "}; // end of module " << name.c_str();
         }
         return 0;
      }

      int
      InternalIDLVisitor::visit_interface (AST_Interface *node)
      {        

         if (node->imported ())
         {
            return 0;
         }

 //         this->check_id_and_version (node);
//          this->check_prefix (node);


         if (!node->is_local () &&
             !node->is_abstract ())
         {
            stream() << ost_nl << ost_nl;
            stream() << "local interface CCM_" << node->original_local_name () << ":" 
               << try_escape(node->original_local_name ()).c_str()
               << "{" << ost_nl
               << "};";
         }

//          if (this->visit_scope (node) != 0)
//          {
//             ACE_ERROR_RETURN ((LM_ERROR,
//                                "InternalIDLVisitor::visit_interface - "
//                                "codegen for scope failed\n"),
//                               -1);
//          }


         return 0;
      }

      int
      InternalIDLVisitor::visit_interface_fwd (AST_InterfaceFwd *node)
      {
         if (node->imported ())
         {
            return 0;
         }
         if (!node->is_local () &&
             !node->is_abstract ())
         {
            stream() << ost_nl << ost_nl
               << "local interface CCM_" << node->original_local_name () << ";";
         }

         return 0;
      }

      int
      InternalIDLVisitor::visit_valuebox (AST_ValueBox *node)
      {
         if (node->imported ())
         {
            return 0;
         }
         // Do not generate

         return 0;
      }

      int
      InternalIDLVisitor::visit_valuetype (AST_ValueType *node)
      {
         if (node->imported ())
         {
            return 0;
         }
         // Do not generate

         return 0;
      }

      int
      InternalIDLVisitor::visit_valuetype_fwd (AST_ValueTypeFwd *node)
      {
         if (node->imported ())
         {
            return 0;
         }
         // Do not generate

         return 0;
      }

      int
      InternalIDLVisitor::visit_component (AST_Component *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         this->gen_main_executor (node);
         this->gen_monolithic_executor (node);
         this->gen_context (node);


         return 0;
      }

      int
      InternalIDLVisitor::visit_component_fwd (AST_ComponentFwd *node)
      {
         if (node->imported ())
         {
            return 0;
         }
         stream() << ost_nl << ost_nl
            << "local interface CCM_" << node->original_local_name () << "_Executor;"
            << ost_nl
            << "local interface CCM_" << node->original_local_name () << ";"
            << ost_nl
            << "local interface CCM_" << node->original_local_name () << "_Context;"
            << ost_nl;

         return 0;
      }

      int
      InternalIDLVisitor::visit_eventtype (AST_EventType *node)
      {
         if (node->imported ())
         {
            return 0;
         }
         stream() << ost_nl << ost_nl
            << "interface CCM_" << node->original_local_name () << "Consumer"
            << ost_nl
            << "{" << ost_idt_nl
            << "void push (in " << try_escape(node->original_local_name ()).c_str() << " ev);"
            << ost_uidt_nl
            << "};";
         return 0;
      }

      int
      InternalIDLVisitor::visit_eventtype_fwd (AST_EventTypeFwd *node)
      {
         if (node->imported ())
         {
            return 0;
         }
         stream() << ost_nl << ost_nl
            << "local interface CCM_" << node->original_local_name () << "Consumer;"
            << ost_nl;


         return 0;
      }

      int
      InternalIDLVisitor::visit_home (AST_Home *node)
      {
         if (node->imported ())
         {
            return 0;
         }

         this->gen_home_implicit_executor (node);
         this->gen_home_explicit_executor (node);
         this->gen_home_main_executor (node);

         return 0;
      }


      int
      InternalIDLVisitor::visit_factory  (AST_Factory *node)
      {
         if (node->imported ())
         {
            return 0;
         }
         // Do not generate

         return 0;
      }

      int
      InternalIDLVisitor::visit_structure (AST_Structure *node)
      {
         if (node->imported ())
         {
            return 0;
         }
         // Do not generate

         return 0;
      }

      int
      InternalIDLVisitor::visit_structure_fwd (AST_StructureFwd *node)
      {
         if (node->imported ())
         {
            return 0;
         }
         // Do not generate

         return 0;
      }

      int
      InternalIDLVisitor::visit_exception (AST_Exception *node)
      {
         if (node->imported ())
         {
            return 0;
         }
         // Do not generate

         return 0;
      }

      int
      InternalIDLVisitor::visit_expression (AST_Expression *)
      {
         return 0;
      }

      int
      InternalIDLVisitor::visit_enum (AST_Enum *node)
      {
         if (node->imported ())
         {
            return 0;
         }
         // Do not generate

         return 0;
      }


      int
      InternalIDLVisitor::visit_union (AST_Union *node)
      {
         if (node->imported ())
         {
            return 0;
         }
         // Do not generate

         return 0;
      }

      int
      InternalIDLVisitor::visit_union_fwd (AST_UnionFwd *node)
      {
         if (node->imported ())
         {
            return 0;
         }
         // Do not generate

         return 0;
      }

      int
      InternalIDLVisitor::visit_union_branch (AST_UnionBranch *node)
      {
         // Do not generate

         return 0;
      }

      int
      InternalIDLVisitor::visit_union_label (AST_UnionLabel *node)
      {
         // Do not generate

         return 0;
      }

      int
      InternalIDLVisitor::visit_constant (AST_Constant *node)
      {
         if (node->imported ())
         {
            return 0;
         }
         // Do not generate

         return 0;
      }

      int
      InternalIDLVisitor::visit_enum_val (AST_EnumVal *)
      {
         return 0;
      }



      int
      InternalIDLVisitor::visit_root (AST_Root *node)
      {
         ACE_CString raw_filename;
         ACE_CString filename;

         std::string line;
         bool raw_file_found;

         for (size_t i = 0; i < idl_global->n_included_idl_files (); ++i)
         {
            raw_file_found = false;
            if (i == 0)
            {
               stream() << ost_nl;
            }

            raw_filename = idl_global->included_idl_files ()[i];
            ifstream myfile2 ( excludedFileName.c_str());//("ExcludedForGeneration.properties");
            if (myfile2.is_open())
            {
               while (! myfile2.eof() )
               {
                  getline (myfile2,line);
                  if (!strcmp(line.c_str(), raw_filename.c_str())) 
                     raw_file_found = true;
               }
               myfile2.close();
            }

            if (raw_filename.find (".pidl") != ACE_CString::npos
                || raw_filename == "orb.idl"
                || raw_filename == "Components.idl" 
                || raw_file_found  )
            {   
               filename = raw_filename;
            }
            else
            {
               filename = raw_filename.substr (0, raw_filename.rfind ('.')) + "_cif.idl"; 
            }
            stream() << ost_nl
                     << "#include \"" << filename.c_str () << "\"";
         }


         ACE_CString fn (idl_global->stripped_filename ()->get_string ());
         fn = fn.substr (0, fn.rfind ('.'));
         fn += "_IDL2.idl";

         stream() << ost_nl
            << "#include \"" << fn.c_str () << "\"";

         const char *pfix = node->prefix ();

         if (ACE_OS::strcmp (pfix, "") != 0)
         {
            stream() << ost_nl << ost_nl
               << "#pragma prefix \"" << pfix << "\"";
         }
         

         if  (localIdl3)
         {
            if (this->visit_scope (node) != 0)
            {
               ACE_ERROR_RETURN ((LM_ERROR,
                                  "InternalIDLVisitor::visit_root - "
                                  "codegen for scope failed\n"),
                                 -1);
            }
         }

         return 0;
      }

      int
      InternalIDLVisitor::visit_native (AST_Native *node)
      {
         if (node->imported ())
         {
            return 0;
         }
         // Do not generated

         return 0;
      }



      void
      InternalIDLVisitor::gen_context (AST_Component *node)
      {

         AST_Component *base = node->base_component ();

         stream() << ost_nl << ost_nl
            << "local interface CCM_" << node->original_local_name () << "_Context";

         ACE_CString s_base_context("");
         if (base != 0) 
         {
            UTL_Scope * base_scope = base->defined_in ();
            if (base_scope != 0)
            {
               AST_Decl *dummy = ScopeAsDecl(base_scope);
               s_base_context = "::";
               s_base_context += dummy->full_name();
               s_base_context += "::";
            }
            s_base_context += "CCM_";
            s_base_context += base->original_local_name ()->get_string();
            s_base_context += "_Context";
         } 
         else
         {
            s_base_context = "Components::CCMContext";
         }
         stream() << " : " << s_base_context.c_str()
            << ost_nl
            << "{" << ost_idt_nl;
         // Generate additional operations for receptacles and event sources
         {

            ACE_Unbounded_Queue<AST_Component::port_description> &s =
               node->uses ();
            AST_Component::port_description *pd = 0;

            for (ACE_Unbounded_Queue_Iterator<AST_Component::port_description> iter (s);
                 ! iter.done ();
                 iter.advance ())
            {
               iter.next (pd);

               stream() << ost_nl << ost_nl;
               Identifier * id__ = original_local_name(pd->id);

               if (pd->is_multiple)
               {
                  stream() << try_escape(node->original_local_name ()).c_str() 
                     << "::" << id__  << "Connections get_connections_" << id__ << " ();";
               }
               else
               {
                  stream() << pd->impl->name () << " get_connection_" << id__ << " ();";
               }
               delete id__;
            }
         }
         // Publishes
         {

            ACE_Unbounded_Queue<AST_Component::port_description> &s =
               node->publishes ();
            AST_Component::port_description *pd = 0;

            for (ACE_Unbounded_Queue_Iterator<AST_Component::port_description> iter (s);
                 ! iter.done ();
                 iter.advance ())
            {
               iter.next (pd);
               Identifier * id__ = original_local_name(pd->id);
               stream() << ost_nl << ost_nl
                  << "void push_" << id__
                  << " (in " << pd->impl->name () <<" ev);";
               delete id__;
            }
         }
         // Emits
         {
            ACE_Unbounded_Queue<AST_Component::port_description> &s =
               node->emits ();
            AST_Component::port_description *pd = 0;

            for (ACE_Unbounded_Queue_Iterator<AST_Component::port_description> iter (s);
                 ! iter.done ();
                 iter.advance ())
            {
               iter.next (pd);
               Identifier * id__ = original_local_name(pd->id);
               stream() << ost_nl << ost_nl
                  << "void push_" << id__
                  << " (in " << pd->impl->name () <<" ev);";
               delete id__;
            }
         }

         stream() << ost_uidt_nl
            << "};";

      }

      void
      InternalIDLVisitor::gen_main_executor (AST_Component *node)
      {

         AST_Component *base = node->base_component ();

         stream() << ost_nl << ost_nl
            << "local interface CCM_" << node->original_local_name () << "_Executor";

         ACE_CString s_base_context("");
         if (base != 0) 
         {
            UTL_Scope * base_scope = base->defined_in ();
            if (base_scope != 0)
            {
               AST_Decl *dummy = ScopeAsDecl(base_scope);
               s_base_context = "::";
               s_base_context += dummy->full_name();
               s_base_context += "::";
            }
            s_base_context += "CCM_";
            s_base_context += base->original_local_name ()->get_string();
            s_base_context += "_Executor";
         } 
         else
         {
            s_base_context = "Components::EnterpriseComponent";
         }
         stream() << " : " << s_base_context.c_str();
         // Any supported interfaces?         
         long nsupports = node->n_supports ();

         for (long i = 0; i < nsupports; ++i)
         {
            stream() << ost_nl << ", " << node->supports ()[i]->name ();
         }
         stream() << ost_nl
            << "{" << ost_idt_nl;

         // Generate attributes declared by the component
         {

            // Any attributes and operations?
            for (UTL_ScopeActiveIterator decl_iter (node, UTL_Scope::IK_decls);
                 ! decl_iter.is_done ();
                 decl_iter.next ())
            {
               AST_Decl *d = decl_iter.item ();
               AST_Attribute * attr = AST_Attribute::narrow_from_decl(d);
               if ((attr != 0) &&
                   (d->ast_accept (this) != 0))
               {
                  ACE_ERROR ((LM_ERROR,
                              "InternalIDLVisitor::gen_home_explicit_executor - "
                              "codegen for scope failed\n"));
                  return;
               }
            }
         }
         //

         stream() << ost_uidt_nl
            << "};";

      }

      void
      InternalIDLVisitor::gen_monolithic_executor (AST_Component *node)
      {
         stream() << ost_nl << ost_nl
            << "local interface CCM_" << node->original_local_name () << " : " 
            << "CCM_"<< node->original_local_name () << "_Executor"
            << ost_nl
            << "{" << ost_idt_nl;
         // Facet ports
         {

            ACE_Unbounded_Queue<AST_Component::port_description> &s =
               node->provides ();
            AST_Component::port_description *pd = 0;

            for (ACE_Unbounded_Queue_Iterator<AST_Component::port_description> iter (s);
                 ! iter.done ();
                 iter.advance ())
            {
               iter.next (pd);
               Identifier * id__ = original_local_name(pd->id);

               // CCM_<type> get_<name> ();
               
               ACE_CString s_impl("");
               UTL_Scope * base_scope = pd->impl->defined_in ();
               if (base_scope != 0)
               {
                  AST_Decl *dummy = ScopeAsDecl(base_scope);
                  s_impl = "::";
                  s_impl += dummy->full_name();
                  s_impl += "::";
                  s_impl += "CCM_";
                  s_impl += pd->impl->original_local_name ()->get_string();
               } 
               else
               {
                  s_impl += "CCM_";
                  s_impl += pd->impl->original_local_name ()->get_string();
               }


               stream() << ost_nl << ost_nl
                  << s_impl.c_str() << " get_" << id__ << " ();";
               delete id__;

            }
         }
         // Consumer ports
         {
            ACE_Unbounded_Queue<AST_Component::port_description> &s =
               node->consumes ();
            AST_Component::port_description *pd = 0;
            
            for (ACE_Unbounded_Queue_Iterator<AST_Component::port_description> iter (s);
                 ! iter.done ();
                 iter.advance ())
            {
               iter.next (pd);
               
               Identifier * id__ = original_local_name(pd->id);
               stream() << ost_nl << ost_nl
                  << "void push_" << id__ << " (in " << pd->impl->name () << " ev );";
               delete id__;
            }
            
         }
         stream() << ost_uidt_nl
            << "};";

      }


      void 
      InternalIDLVisitor::gen_home_implicit_executor (AST_Home *node)
      {
         AST_ValueType *key = node->primary_key ();

         stream() << ost_nl << ost_nl
            << "local interface CCM_" << node->original_local_name () << "Implicit" << ost_nl
            << "{"<< ost_idt_nl;

         if (key != 0)
         {
            stream() << "Components::EnterpriseComponent "
               << "create (in " << key->name () << " key)" << ost_idt_nl
               << "raises (Components::CCMException);" << ost_uidt_nl
               << "Components::EnterpriseComponent "
               << "find_by_primary_key (in " << key->name () << " key)" << ost_idt_nl
               << "raises (Components::CCMException);" << ost_uidt_nl
               << "void remove (in " << key->name () << " key)" << ost_idt_nl
               << "raises (Components::CCMException);" << ost_uidt_nl;
         } 
         else
         {
            stream() << "Components::EnterpriseComponent create()" << ost_idt_nl
               << "raises (Components::CCMException);" << ost_uidt_nl;
         }

         stream() << ost_uidt_nl
            << "};";
      }
      
      void 
      InternalIDLVisitor::gen_home_explicit_executor (AST_Home *node)
      {
         ACE_CString explicit_name("CCM_");
         explicit_name += node->original_local_name ()->get_string ();
         explicit_name += "Explicit";

         stream() << ost_nl << ost_nl
            << "local interface " << explicit_name.c_str () << " : ";

         
         AST_Home *base = node->base_home ();

         if (base == 0)
         {
            stream() << "Components::HomeExecutorBase";
         }
         else
         {
        ACE_CString s_base_home("");
            UTL_Scope * base_scope = base->defined_in ();
            if (base_scope != 0)
            {
               AST_Decl *dummy = ScopeAsDecl(base_scope);
               s_base_home = "::";
               s_base_home += dummy->full_name();
               s_base_home += "::";
            }
            s_base_home += "CCM_";
            s_base_home += base->original_local_name ()->get_string();
            s_base_home += "Explicit";
            stream() << s_base_home.c_str();
         }

         stream() << ost_idt_nl;
         AST_Interface **parents = node->inherits ();

         for (long i = 0; i < node->n_inherits (); ++i)
         {
            stream() << ", " << parents[i]->name () << ost_nl;
         }
         stream() << ost_uidt_nl
            << "{" << ost_idt_nl;

         // Any attributes and operations?
         for (UTL_ScopeActiveIterator decl_iter (node, UTL_Scope::IK_decls);
              ! decl_iter.is_done ();
              decl_iter.next ())
         {
            AST_Decl *d = decl_iter.item ();
            if (d->ast_accept (this) != 0)
            {
               ACE_ERROR ((LM_ERROR,
                                  "InternalIDLVisitor::gen_home_explicit_executor - "
                                  "codegen for scope failed\n"));
               return;
            }

         }
         // Generate factory operations
         {
            stream() << ost_nl;
            AST_Operation **item = 0;

            for (ACE_Unbounded_Queue_Iterator<AST_Operation *> i (node->factories ());
                 ! i.done ();
                 i.advance ())
            {
               i.next (item);

               stream() << "Components::EnterpriseComponent " << try_escape((*item)->original_local_name ()).c_str() << " (";
               this->gen_params (*item, (*item)->argument_count ());
               stream() << ")";

               UTL_ExceptList *exceps = (*item)->exceptions ();

               if (exceps != 0 && exceps->length () > 0)
               {
                  this->gen_exception_list (exceps, "", true);
               }
               stream() << ";" << ost_nl;
            }
         }

         // Generate finder operations
         { 
            stream() << ost_nl;
            AST_Operation **item = 0;

            for (ACE_Unbounded_Queue_Iterator<AST_Operation *> i (node->finders ());
                 ! i.done ();
                 i.advance ())
            {
               i.next (item);

               stream() << "Components::EnterpriseComponent " << try_escape((*item)->original_local_name ()).c_str() << " (";
               this->gen_params (*item, (*item)->argument_count ());
               stream() << ")";

               UTL_ExceptList *exceps = (*item)->exceptions ();

               if (exceps != 0 && exceps->length () > 0)
               {
                  this->gen_exception_list (exceps, "", true);
               }
               stream() << ";" ;
            }
         }
         stream() << ost_uidt_nl
            << "};";
      }
      
      void 
      InternalIDLVisitor::gen_home_main_executor (AST_Home *node)
      {

         stream() << ost_nl << ost_nl
            << "local interface CCM_" << node->original_local_name () << " : " << ost_idt_nl
            << "CCM_"<< node->original_local_name () << "Explicit," << ost_nl
            << "CCM_"<< node->original_local_name () << "Implicit" << ost_uidt_nl
            << "{" << ost_nl
            << "};";
      }



   }; // End of CodeGenerator
}; // End of Cdmw
