/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "engine/UtlIdentifierHelper.hpp"
#include "ace/os_include/sys/os_types.h"
#include "ace/SString.h"

namespace
{
   /*
    * OMG IDL keywords as specified in OMG document
    *  The Common Object Request Broker: Architecture and Specification
    *  July 2002, Version 3.0, formal/02-06-33
    *
    * NOTA: 'manages' is missing in above document!
    */
   static const char *const OMG_IDL_keywords[] =
   {
      "abstract","any","attribute","boolean","case","char",
      "component","const","consumes","context","custom","default",
      "double","exception","emits","enum","eventtype","factory","FALSE",
      "finder","fixed","float","getraises","home","import","in","inout",
      "interface","local","long","manages","module","multiple","native",
      "Object","octet","oneway","out","primarykey","private","provides",
      "public","publishes","raises","readonly","setraises","sequence",
      "short","string","struct","supports","switch","TRUE","truncatable",
      "typedef","typeid","typeprefix","unsigned","union","uses","ValueBase",
      "valuetype","void","wchar","wstring",0
   };

}; // end of anonymous namespace

namespace Cdmw
{
   namespace CodeGenerator
   {
      /* TODO
         portdescrion: pd->id
         should do the following:
         1) make sure to keep '_' escape caracter in generated identifier in IDL
         2) remove the '_' escape character when the identifier is part of another identifier
            such as in provides_XXX
         3) remove any _cxx_ in generated IDL (equivalent to AST_Decl::original_local_name())
      */
      Identifier* 
      original_local_name (Identifier *local_name)
      {
         Identifier * id = 0;
         // Remove _cxx_ if it is present.
         if (ACE_OS::strstr (local_name->get_string (),
                             "_cxx_")
             == local_name->get_string ())
         {
            // CSting class is good to do this stuff.
            ACE_CString name_str (local_name->get_string ());

            // Remove _cxx_.
            name_str = name_str.substr (ACE_OS::strlen ("_cxx_"));

            // Assign to the Identifier variable.
            id = new Identifier (name_str.c_str ());
         }
         else
         {
            id = local_name->copy ();
         }
         return id;
      }
      
      bool
      is_idl_keyword(const char* s)
      {
         bool result = false;
         for (int i=0;OMG_IDL_keywords[i];++i)
         {
            if (strcmp(OMG_IDL_keywords[i],s)==0) 
            {
               result = true;
               break;
            }
         }
         return result;
      }

      /*
       * should do the following:
       *  1) make sure to keep '_' escape caracter in generated identifier in IDL
       * 
       */
      std::string 
      try_escape (Identifier *local_name)
      {
         std::string s_local_name(local_name->get_string ());
         
         if (is_idl_keyword(s_local_name.c_str()))
            return "_" + s_local_name;
         else
            return s_local_name;
      }




   }; // End of CodeGenerator
}; // End of Cdmw
