<?xml version="1.0" encoding="utf-8"?>
<!-- ===================================================================== -->
<!--
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009. 
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-->
<!-- ===================================================================== -->


<xsl:stylesheet version="1.1" 
   xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
   xmlns:UML="org.omg/UML1.3">


<xsl:template name="ModuleNameParentsOfNode">
   <xsl:param name="_moduleNode"/>

    <xsl:variable name="upperNode" select="key('moduleById', $_moduleNode/@namespace)"/>

    <xsl:if test="boolean($upperNode)">    
       <xsl:variable name="upperNodeName" select="$upperNode/@name"/>

       <xsl:call-template name="ModuleNameParentsOfNode">
         <xsl:with-param name="_moduleNode" select="$upperNode"/>
       </xsl:call-template>

       <xsl:value-of select="concat($upperNodeName, $javaSep)"/>
    </xsl:if> 

</xsl:template>

<!--
   This templates prefixes the module name of a classname node.
   Optionaly a new IDL pragma prefix can be set.
   
   @param _prefix      Name used to prefix the module.
   @param _idlPrefix   IDL prefix can also be set. Optional.
   @param _node        The classname node to change.

-->
<xsl:template name="prefixModuleNameOfNode">
   <xsl:param name="_prefix"/>
   <xsl:param name="_idlPrefix"/>
   <xsl:param name="_node"/>
 
   <xsl:variable name="scopedName">
      <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="$_node"/>
         <xsl:with-param name="_lang" select="'java'"/>
      </xsl:call-template>
   </xsl:variable>

   <xsl:variable name="moduleNode" select="key('moduleById', $_node/@namespace)"/>
   <xsl:variable name="moduleName">
      <xsl:value-of select="$moduleNode/@name"/>
   </xsl:variable>

   <xsl:variable name="parentsModules">
      <xsl:call-template name="ModuleNameParentsOfNode">
        <xsl:with-param name="_moduleNode" select="$moduleNode"/>
      </xsl:call-template>
   </xsl:variable>

   <xsl:variable name="beforeModuleName">
    <xsl:choose>
      <xsl:when test="string-length($parentsModules)>0">
         <xsl:value-of select="substring-before($scopedName, $parentsModules)"/>
      </xsl:when>
      <xsl:otherwise>
         <xsl:value-of select="substring-before($scopedName, $moduleName)"/>
      </xsl:otherwise>
    </xsl:choose>
   </xsl:variable>
   
   <xsl:variable name="afterModuleName">
      <xsl:value-of select="substring-after($scopedName, $moduleName)"/>
   </xsl:variable>


   <xsl:choose>
      <xsl:when test="string-length($_idlPrefix)>0">
         <xsl:variable name="javaIdlPrefix">
            <xsl:call-template name="reverseList">
               <xsl:with-param name="_list" select="$_idlPrefix"/>
               <xsl:with-param name="_separator" select="$javaSep"/>
            </xsl:call-template>
         </xsl:variable>
         <xsl:value-of select="concat($javaIdlPrefix, $javaSep, $_prefix, $parentsModules, $moduleName, $afterModuleName)"/>
      </xsl:when>
      <xsl:otherwise>
         <xsl:value-of select="concat($beforeModuleName, $_prefix, $parentsModules, $moduleName, $afterModuleName)"/>
      </xsl:otherwise>
   </xsl:choose>

</xsl:template>

<!--
   This templates prefixes the class name of a scoped classname.
   
   @param _prefix      Name used to prefix the classname.
   @param _scopedName  The scoped classname to change.

-->
<xsl:template name="prefixClassName">
   <xsl:param name="_prefix"/>
   <xsl:param name="_scopedName"/>
   
   <xsl:variable name="className">
      <xsl:call-template name="getLastToken">
         <xsl:with-param name="_string" select="$_scopedName"/>
         <xsl:with-param name="_separator" select="$javaSep"/>
      </xsl:call-template>
   </xsl:variable>
   
   <xsl:variable name="scope">
      <xsl:call-template name="removeLastToken">
         <xsl:with-param name="_scopedName" select="$_scopedName"/>
         <xsl:with-param name="_sep" select="$javaSep"/>
      </xsl:call-template>
   </xsl:variable>

   <xsl:value-of select="concat($scope, $javaSep, $_prefix, $className)"/>

</xsl:template>



<!--
   This templates returns the scoped name of an exception.
   
   @param _exception   The name of the exception.

-->
<xsl:template name="getScopedException">
   <xsl:param name="_exception"/>

   <xsl:variable name="exceptionName">
      <xsl:call-template name="getLastToken">
         <xsl:with-param name="_string" select="$_exception"/>
         <xsl:with-param name="_separator" select="$cppSep"/>
      </xsl:call-template>
   </xsl:variable>
   
   <xsl:for-each select="key('exceptionByName', $exceptionName)">
     <xsl:sort select="@xmi.id"/>
   
     <!--     <xsl:variable name="_node" select="key('exceptionById', @xmi.id)"/> -->
   
     <xsl:variable name="tmp_fqn">
       <xsl:call-template name="getScopedNameOfNode">
         <xsl:with-param name="_node" select="."/>
         <xsl:with-param name="_lang" select="'cpp'"/>
            </xsl:call-template>
         </xsl:variable>

     <xsl:if test="$tmp_fqn = $_exception">
       <xsl:variable name="theJaveScopedName">
         <xsl:call-template name="getScopedNameOfNode">
           <xsl:with-param name="_node" select="."/>
           <xsl:with-param name="_lang" select="'java'"/>
                             </xsl:call-template> 
                        </xsl:variable>  
       <xsl:value-of select="$theJaveScopedName"/>
                        </xsl:if>

   </xsl:for-each>
</xsl:template> <!-- end of template getScopedException -->


</xsl:stylesheet>
