<?xml version="1.0" encoding="utf-8"?>
<!-- ===================================================================== -->
<!--
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009. 
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-->
<!-- ===================================================================== -->


<xsl:stylesheet version="1.1" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">



<!--
   Main - This template fetches the necessary data from inputs
   to generate a file.

   @param none
-->

<xsl:template name="cdmw_interface_cpp">
   <!--
      Parameters below are used for recursiveness.
   -->
   <xsl:param name="_index" select="1"/>

   <xsl:variable name="programNodes" select="$cdmwNode/programs/program"/>

   <xsl:if test="boolean($programNodes)">
      <xsl:if test="$_index &lt;= count($programNodes)">
         <xsl:variable name="programNode" select="$programNodes[$_index]"/>

         <!--
            Get the output filename.
         -->
         <xsl:variable name="filename">
            <xsl:call-template name="buildOutputFilename">
               <xsl:with-param name="_userDirectory" select="$programNode/@directory"/>
               <xsl:with-param name="_userFilename" select="$programNode/@filename"/>
               <xsl:with-param name="_defaultFilename" select="'CdmwInterface'"/>
               <xsl:with-param name="_fileExtension" select="'.cpp'"/>
            </xsl:call-template>
         </xsl:variable>

         <!--
            Display a message to the user.
         -->
         <xsl:message>
            <xsl:value-of select="concat('Generating file ', $filename)"/>
         </xsl:message>

         <!--
            Generate the file.
         -->
         <xsl:document method="text" href="{$filename}">
            <!--
               Insert the document header.
            -->
            <xsl:call-template name="docHeader">
               <xsl:with-param name="_lang" select="'cpp'"/>
            </xsl:call-template>
            <!--
               Insert the content.
            -->
            <xsl:call-template name="cdmw_interface_cpp.content">
               <xsl:with-param name="_programNode" select="$programNode"/>
            </xsl:call-template>
         </xsl:document>

         <!--
            Proceed to the next node.
         -->
         <xsl:call-template name="cdmw_interface_cpp">
            <xsl:with-param name="_index" select="$_index + 1"/>
         </xsl:call-template>
      </xsl:if>
   </xsl:if>
</xsl:template> <!-- end of template cdmw_interface_cpp -->




<xsl:template name="count.datastore.occurrence">
   <xsl:param name="_occurrences"/>
   <xsl:param name="_datastoreNodes"/>
   <xsl:param name="_datastoreOccurrences"/>
   <!--
      Parameters below are used for recursiveness.
   -->
   <xsl:param name="_index_datastore"/>
   <xsl:choose>
      <xsl:when test="$_index_datastore &lt;= $_occurrences">
         <xsl:variable name="datastoreOccurrences" select="$_datastoreNodes[$_index_datastore]/@occurrences"/>
         <xsl:call-template name="count.datastore.occurrence">
            <xsl:with-param name="_datastoreNodes" select="$_datastoreNodes"/>
            <xsl:with-param name="_occurrences" select="$_occurrences"/>
            <xsl:with-param name="_datastoreOccurrences" select="$_datastoreOccurrences + $datastoreOccurrences"/>
            <xsl:with-param name="_index_datastore" select="$_index_datastore + 1"/>
         </xsl:call-template>
     </xsl:when>
       <xsl:otherwise>
          <xsl:value-of select="$_datastoreOccurrences"/>
       </xsl:otherwise>    
   </xsl:choose>
</xsl:template>

<xsl:template name="datastore.context">
   <xsl:param name="_dataStoresKind"/>
   <xsl:param name="_hasReplyLogging"/>
<![CDATA[ // get information for DataStore context ]]><![CDATA[
     try {
             std::string res;
         unsigned long res_long;       
             res = xml_data->getDataStoresAttributeValue("max-transaction-in-progress"); 
             max_transaction_in_progress = atoi(res.c_str());

             res = xml_data->getDataStoresAttributeValue("max-transaction-done"); 
             max_transaction_done = atoi(res.c_str());

             res = xml_data->getDataStoresAttributeValue("cohort-timeout");
             res_long = atoi(res.c_str());                 
             cohort_timeout.seconds = res_long/1000;
             cohort_timeout.microseconds = (res_long%1000)*1000;

             res = xml_data->getDataStoresAttributeValue("coordinator-timeout");                 
             res_long = atoi(res.c_str());                 
             coordinator_timeout.seconds = res_long/1000;
             coordinator_timeout.microseconds = (res_long%1000)*1000;

             res = xml_data->getDataStoresAttributeValue("number-of-scheduler-thread");
             number_of_scheduler_thread = atoi(res.c_str());

             res = xml_data->getDataStoresAttributeValue("chunk-size");
             chunk_size = atoi(res.c_str());

             res = xml_data->getDataStoresAttributeValue("freeze-size");
             freeze_size = atoi(res.c_str());

             res = xml_data->getDataStoresAttributeValue("freeze-timeout");
             res_long = atoi(res.c_str());                              freeze_timeout.seconds = res_long/1000;
             freeze_timeout.microseconds = (res_long%1000)*1000;
]]>
<xsl:if test="$_dataStoresKind = 'MIOP'">
<![CDATA[    multicast_port = xml_data->getDataStoreAttributeValue("multicast", 1, "multicast-port");

             multicast_domain = xml_data->getDataStoreAttributeValue("multicast", 1, "multicast-domain");

             multicast_IP_address = xml_data->getDataStoreAttributeValue("multicast", 1, "ip-adresse");]]>

</xsl:if>

<![CDATA[    } catch (const Cdmw::BadParameterException& ex) {
                CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                             CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                              CORBA::COMPLETED_NO),ex.what() );
             }
]]>
<![CDATA[PortableGroup::GOA_var rootPOA = InitUtils::get_root_GOA(orb);
         Cdmw::FT::DataStoreContext *datastore_context = new Cdmw::FT::DataStoreContext( ]]>
      <xsl:choose>
     <xsl:when test="$_dataStoresKind = 'MIOP'">
        <![CDATA[orb,
        CDMW_rootPOA.in(),
        multicast_port,
        multicast_domain,
        multicast_IP_address,
        max_transaction_in_progress,
        max_transaction_done,
        cohort_timeout,
        coordinator_timeout,
        number_of_scheduler_thread,
        chunk_size,
        freeze_size,
        freeze_timeout]]>
     </xsl:when>
     <xsl:when test="$_dataStoresKind = 'IIOP'">
        <![CDATA[orb,
        CDMW_rootPOA.in(),
        max_transaction_in_progress,
        max_transaction_done,
        cohort_timeout,
        coordinator_timeout,
        number_of_scheduler_thread,
        chunk_size,
        freeze_size,
        freeze_timeout]]>
     </xsl:when>
      </xsl:choose>
      <![CDATA[ ); ]]>
<![CDATA[
     // TODO clean up manager (since this can only be done at the end of the program execution
     // then this leak is harmless). Also applise to datastore_context and datastores
    Cdmw::FT::TXManager* manager = new Cdmw::FT::TXManager(*datastore_context);
]]>
<![CDATA[
      CdmwFT::StateTransfer::LocalDataStoreInfo localDataStoreInfo;

      CdmwFT::StateTransfer::DataStoreGroup_var ftdatastoregroup_ref = 
          (new Cdmw::FT::DataStoreGroup_impl(CDMW_rootPOA.in(), *datastore_context))->_this();

      localDataStoreInfo.local_data_store = CdmwFT::StateTransfer::DataStoreGroup::_duplicate(ftdatastoregroup_ref.in());
]]>

      <xsl:choose>
     <xsl:when test="$_dataStoresKind = 'MIOP'">
<![CDATA[
      localDataStoreInfo.coordinator = CdmwFT::Transaction::TPCCoordinator::_nil();
      localDataStoreInfo.cohort = CdmwFT::Transaction::TPCCohort::_nil();
]]>
     </xsl:when>
     <xsl:when test="$_dataStoresKind = 'IIOP'">
<![CDATA[
      localDataStoreInfo.coordinator = datastore_context->get_tpc_coordinator();
      localDataStoreInfo.cohort = datastore_context->get_tpc_cohort();
]]>
     </xsl:when>
      </xsl:choose>


     <xsl:choose>
         <xsl:when test="$_hasReplyLogging='yes'">
<![CDATA[
      // incrementation for the reply logging
      localDataStoreInfosSize = localDataStoreInfosSize + 1;
]]>
     </xsl:when>
       <xsl:otherwise>
       <![CDATA[
      // incrementation for the reply logging
      localDataStoreInfosSize = localDataStoreInfosSize + 0;
]]>
   
       </xsl:otherwise>    
      </xsl:choose>




<![CDATA[
      localDataStoreInfo.dsids.length(localDataStoreInfosSize);
      (*localDataStoreInfos)[0] = localDataStoreInfo;
]]>
</xsl:template>

<xsl:template name="datastore.occurrence">
   <xsl:param name="_occurrences"/>
   <xsl:param name="_occurrencesInProgress"/>
   <xsl:param name="_occurrencesInProgressForThisDataStore"/>
   <xsl:param name="_dataStoresKind"/>
   <xsl:param name="_datastoreNodes"/>
   <!--
      Parameters below are used for recursiveness.
   -->
   <xsl:param name="_index_datastore"/>

   <xsl:if test="$_occurrencesInProgress &lt;= $_occurrences">


            <xsl:variable name="datastoreIdref" select="$_datastoreNodes[$_index_datastore]/@idref"/>
        <xsl:variable name="datastoreOccurrences" select="$_datastoreNodes[$_index_datastore]/@occurrences"/>
      
        <!--
               Get the matching node /cdmw/datastore-frameworks/datastore-framework.
            -->
            <xsl:variable name="datastoreTypeNode" select="$DatastoreTypeNodes[@typename = $datastoreIdref]"/>
        <xsl:variable name="_datastoreTypename" select="$datastoreTypeNode/@typename"/>
        <xsl:variable name="datastoreOidType" select="$datastoreTypeNode/@oid-type"/>
        <xsl:variable name="datastoreDataType" select="$datastoreTypeNode/@data-type"/>
        <xsl:variable name="datastoreIdlfilename" select="$datastoreTypeNode/@idl-filename"/>

           <xsl:if test="string-length($_datastoreTypename)=0"> 
         <xsl:message terminate="yes">
               <xsl:value-of select="concat('ERROR ---  ', $datastoreIdref, ' defined as idref does not correspond to a defined Datastore typename ')"/>
             </xsl:message>
           </xsl:if>
      
        <!--
           define the type of the storage home
        -->

        <xsl:variable name="datastoreoidtmp">
               <xsl:call-template name="replaceCharsInString">
                  <xsl:with-param name="_stringIn" select="$datastoreOidType"/>
                  <xsl:with-param name="_charsIn" select="'.'"/>
                  <xsl:with-param name="_charsOut" select="$cppSep"/>
               </xsl:call-template>
            </xsl:variable>
            <xsl:variable name="datastoredatatmp">
               <xsl:call-template name="replaceCharsInString">
                  <xsl:with-param name="_stringIn" select="$datastoreDataType"/>
          <xsl:with-param name="_charsIn" select="'.'"/>
          <xsl:with-param name="_charsOut" select="$cppSep"/>
               </xsl:call-template>
            </xsl:variable>

        <xsl:variable name="datastoreoidissimpletype">
           <xsl:call-template name="isSimpleDataTypeForDataTraits">
              <xsl:with-param name="_dataType" select="$datastoreoidtmp"/>
           </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="datastoredataissimpletype">
           <xsl:call-template name="isSimpleDataTypeForDataTraits">
              <xsl:with-param name="_dataType" select="$datastoredatatmp"/>
           </xsl:call-template>
        </xsl:variable>


        <xsl:variable name="datastoreoid">
           <xsl:choose>
              <xsl:when test="$datastoreoidissimpletype='true'">
             <xsl:call-template name="getSimpleDataTypeForDataTraits">
                <xsl:with-param name="_dataType" select="$datastoreoidtmp"/>
             </xsl:call-template>
          </xsl:when>
          <xsl:otherwise>
             <xsl:value-of select="$datastoreoidtmp"/>
          </xsl:otherwise>
           </xsl:choose>
            </xsl:variable>   
        <xsl:variable name="datastoredata">
           <xsl:choose>
              <xsl:when test="$datastoredataissimpletype='true'">
             <xsl:call-template name="getSimpleDataTypeForDataTraits">
                <xsl:with-param name="_dataType" select="$datastoredatatmp"/>
             </xsl:call-template>
          </xsl:when>
          <xsl:otherwise>
             <xsl:value-of select="$datastoredatatmp"/>
          </xsl:otherwise>
           </xsl:choose>
            </xsl:variable>   
   <!--
       export the specification for the storage home
   -->
<![CDATA[
     try {
             std::string res;

             InitUtils::get_datastore_id_parameter
                         (*(xml_data.get()),
                      "]]><xsl:value-of select="$_datastoreTypename"/><![CDATA[",
                          ]]><xsl:value-of select="$_occurrences"/><![CDATA[,
                          ]]><xsl:value-of select="$_occurrencesInProgressForThisDataStore"/><![CDATA[,
                          datastore_id);

    } catch (const Cdmw::BadParameterException& ex) {
                CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                             CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                              CORBA::COMPLETED_NO),ex.what() );
             }
]]>
<![CDATA[    (*localDataStoreInfos)[0].dsids[dataStore_inc++] = datastore_id;
    typedef Cdmw::FT::TXDataStore<]]><xsl:value-of select="$datastoreoid"/><![CDATA[,]]>
                               <xsl:value-of select="$datastoredata"/><![CDATA[>]]>
                   <xsl:value-of select="$_datastoreTypename"/><![CDATA[;]]>


      <xsl:variable name="datastoreVariable" select="translate($_datastoreTypename, $ucase, $lcase)"/>
      <![CDATA[        new ]]><xsl:value-of select="$_datastoreTypename"/><![CDATA[ (*manager,
            datastore_id,
        orb,
        CDMW_rootPOA.in());]]>

      <!--
          Proceed to the next node.
       -->

       <xsl:choose>
          <xsl:when test="$_occurrencesInProgressForThisDataStore &lt; $datastoreOccurrences">
         <xsl:call-template name="datastore.occurrence">
                <xsl:with-param name="_datastoreNodes" select="$_datastoreNodes"/>
                <xsl:with-param name="_occurrences" select="$_occurrences"/>
        <xsl:with-param name="_occurrencesInProgress" select="$_occurrencesInProgress + 1"/>
        <xsl:with-param name="_occurrencesInProgressForThisDataStore" select="$_occurrencesInProgressForThisDataStore + 1"/>
        <xsl:with-param name="_dataStoresKind" select="$_dataStoresKind"/>
        <xsl:with-param name="_index_datastore" select="$_index_datastore"/>
         </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
         <xsl:call-template name="datastore.occurrence">
                <xsl:with-param name="_datastoreNodes" select="$_datastoreNodes"/>
                <xsl:with-param name="_occurrences" select="$_occurrences"/>
        <xsl:with-param name="_occurrencesInProgress" select="$_occurrencesInProgress + 1"/>
        <xsl:with-param name="_occurrencesInProgressForThisDataStore" select="1"/>
        <xsl:with-param name="_dataStoresKind" select="$_dataStoresKind"/>
        <xsl:with-param name="_index_datastore" select="$_index_datastore + 1"/>
         </xsl:call-template>      
      </xsl:otherwise>
       </xsl:choose>      
   </xsl:if>
</xsl:template>






<!--
   This template generates the content of the file.

   @param _programNode   A reference to a node /cdmw/programs/program.
-->

<xsl:template name="cdmw_interface_cpp.content">
   <xsl:param name="_programNode"/>

   <!--
      References to some nodes.
   -->
   <xsl:variable name="servicesNode" select="$_programNode/services"/>
   <xsl:variable name="frameworkNodes" select="$servicesNode/lifecycle/framework"/>
   <xsl:variable name="datastoreNodes" select="$servicesNode/fault-tolerance/datastores/datastore"/>
   <xsl:variable name="hasDatastoreNodes" select="boolean($servicesNode/fault-tolerance/datastores/datastore)"/>

   <!--
      Miscellaneous variables.
   -->
   <xsl:variable name="hasRepositoryInterface" select="boolean($servicesNode/repository-interface)"/>
   <xsl:variable name="hasPerformance" select="boolean($servicesNode/performance)"/>
   <xsl:variable name="hasTrace" select="boolean($servicesNode/trace)"/>
   <xsl:variable name="hasTime" select="boolean($servicesNode/time)"/>
   <!-- hasRequestFollowup defaults to 'no' -->
   <xsl:variable name="hasRequestFollowup">
      <xsl:choose>
         <xsl:when test="boolean($servicesNode/trace/@request-followup)">
            <xsl:value-of select="$servicesNode/trace/@request-followup"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:value-of select="'no'"/>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:variable>
   <!-- default is enable-buffering=yes -->
   <xsl:variable name="hasBufferingEnabled">
      <xsl:choose>
         <xsl:when test="boolean($servicesNode/trace/@enable-buffering)">
            <xsl:value-of select="$servicesNode/trace/@enable-buffering"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:value-of select="'yes'"/>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:variable>
   <xsl:variable name="hasRecording" select="boolean($servicesNode/recording)"/>
   <xsl:variable name="hasFaultTolerance" select="boolean($servicesNode/fault-tolerance)"/>
   <xsl:variable name="hasLoadBalancing" select="boolean($servicesNode/load-balancing)"/>
   <xsl:variable name="hasWatchdog" select="boolean($servicesNode/watchdog)"/>
   <xsl:variable name="hasDataStore" select="boolean($servicesNode/fault-tolerance/datastores)"/>
   
   <xsl:variable name="hasLifecycle" select="boolean($servicesNode/lifecycle)"/>
   <xsl:variable name="hasHomeImpl">
      <xsl:call-template name="containsHomeImpl">
         <xsl:with-param name="_programNode" select="$_programNode"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="hasEventPorts">
      <xsl:call-template name="containsEventPorts">
         <xsl:with-param name="_programNode" select="$_programNode"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="cdmwinitNamespace" select="'Cdmw.CdmwInit'"/>
   <xsl:variable name="cdmweventNamespace" select="'Cdmw.Event'"/>
   <xsl:variable name="datastoreNamespace" select="'DataStore'"/>

   <!--
      The code generation starts here.
   -->

   <![CDATA[
#include <ConfAndPlug/cdmwinit/CdmwInitSystemProperties.hpp>
#include <ConfAndPlug/cdmwinit/CdmwInterface.hpp>
#include <ConfAndPlug/cdmwinit/CdmwInitConfiguration.hpp>
#include <ConfAndPlug/cdmwinit/InitUtils.hpp>
#include <SystemMngt/platforminterface/PlatformInterface.hpp>
#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/Codec.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/osthreads/Mutex.hpp>
#include <Foundation/osthreads/Condition.hpp>
#include <Foundation/common/Exception.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/common/Locations.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>
   ]]>

   <xsl:if test="$hasLifecycle">
      <![CDATA[
#include <LifeCycle/lifecycleinit/InitUtils.hpp>
      ]]>
   </xsl:if>

   <xsl:if test="$hasRecording">
      <![CDATA[
#include <Foundation/common/Options.hpp>
#include <TraceAndPerf/recordingidl/CdmwRecordingDaemonControl.stub.hpp>
#include <Foundation/idllib/CdmwRecordingContainerManager.stub.hpp>
      ]]>
   </xsl:if>

   <xsl:if test="$hasFaultTolerance">
      <![CDATA[
#include <Foundation/ossupport/OS.hpp>
#include <FaultTolerance/ftlocationmanager/PrimaryBackupGroupRepository_impl.hpp>
#include <FaultTolerance/ftlocationmanager/PrimaryBackupAdmin_impl.hpp>
#include <FaultTolerance/idllib/CdmwFTReplicationManager.stub.hpp>
#include <FaultTolerance/ftcommon/FTConfiguration.hpp>
#include <Foundation/commonsvcs/datastore/StorageHome.hpp>
#include <FaultTolerance/ftstatemanager/TXManager.hpp>
#include <FaultTolerance/ftstatemanager/TXDataStore.hpp>
#include <FaultTolerance/ftstatemanager/DataStoreContext.hpp>
#include <FaultTolerance/ftstatemanager/DataStoreGroup_impl.hpp>
#include <FaultTolerance/ftrequestlogging/FTReplyRecording.hpp>
#include <FaultTolerance/ftinit/FTServiceInit.hpp>
      ]]>
   </xsl:if>

   <xsl:if test="$hasWatchdog">
      <![CDATA[
#include <FaultTolerance/ftwatchdog/WatchdogServiceInit.hpp>
#include <FaultTolerance/ftwatchdog/RequestTimeoutLocalPullMonitorable_impl.hpp>
#include <FaultTolerance/ftcommon/FTSystemProperties.hpp>
      ]]>
   </xsl:if>


   <xsl:if test="$hasDataStore">

      <xsl:variable name="pathPrefix">
          <xsl:choose>
              <xsl:when test="string-length($CSC_NAME) > 0">
                  <xsl:value-of select="concat($CSC_NAME, $fileSep)"/>
              </xsl:when>
              <xsl:otherwise>
                  <xsl:text></xsl:text>
              </xsl:otherwise>
          </xsl:choose>
      </xsl:variable>

      <![CDATA[
#include <FaultTolerance/ftlocationmanager/StatefullPrimaryBackupAdmin_impl.hpp>
#include <FaultTolerance/ftlocationmanager/LocationDataStoreTimeoutHandler_impl.hpp>
       ]]> 

   </xsl:if>

   <xsl:if test="$hasLoadBalancing">
      <![CDATA[
#include <LoadBalancing/idllib/CdmwLBGroupManager.stub.hpp>
      ]]>
   </xsl:if>
   
   <xsl:if test="$hasHomeImpl='true'">
      <![CDATA[
#include <CCMContainer/ccmcif/InitUtils.hpp>
#include <CCMContainer/ccmcontainer/InitUtils.hpp>
#include <CCMContainer/ccmcomponentserver/InitUtils.hpp>
#include <CCMContainer/ccmcontainer/HomeAllocatorRegistry.hpp>
#include <Foundation/common/Locations.hpp>
#include <Foundation/commonsvcs/naming/NamingUtil.hpp>
#include <LifeCycle/lifecycle/DefaultObjectRegistration.hpp>
      ]]>

      <xsl:if test="$hasEventPorts='true'">
        <![CDATA[
#include <Event/idllib/CdmwEvent.stub.hpp>
#include <Event/eventsupport/EventChannelFactory_impl.hpp>
        ]]>
      </xsl:if>
   </xsl:if>
   
   <xsl:variable name="hasFaultToleranceServer">
      <xsl:choose>
         <xsl:when test="boolean($servicesNode/fault-tolerance/@server)">
            <xsl:value-of select="$servicesNode/fault-tolerance/@server"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:value-of select="'no'"/>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:variable>

   <xsl:variable name="hasSystemManagement">
      <xsl:choose>
         <xsl:when test="boolean($servicesNode/system-management/@fatal-handler)">
            <xsl:value-of select="$servicesNode/system-management/@fatal-handler"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:value-of select="'yes'"/>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:variable>
   
   <xsl:variable name="hasIssuerInterceptor">
      <xsl:choose>
         <xsl:when test="boolean($servicesNode/system-management/@command-issuer)">
            <xsl:value-of select="$servicesNode/system-management/@command-issuer"/>
         </xsl:when>
       <xsl:otherwise>
            <xsl:value-of select="'yes'"/>
       </xsl:otherwise>
      </xsl:choose>
   </xsl:variable>
   
   <xsl:variable name="hasLoadBalancingServer">
      <xsl:choose>
         <xsl:when test="boolean($servicesNode/load-balancing/@server)">
            <xsl:value-of select="$servicesNode/load-balancing/@server"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:value-of select="'no'"/>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:variable>

   <xsl:if test="($hasFaultToleranceServer='yes') and ($hasLoadBalancingServer='yes')">
      <xsl:message terminate="yes">
         ERROR: Cannot generate CDMWInit with FaultTolerance server and LoadBalancing server at the same time
      </xsl:message>
   </xsl:if>
   
   <xsl:choose>
      <xsl:when test="$hasFaultToleranceServer='yes'">
         <![CDATA[// generate for FaultToleranceServer]]>
         <xsl:for-each select="$frameworkNodes">
            <xsl:call-template name="includeFramework">
               <xsl:with-param name="_framework" select="."/>
               <xsl:with-param name="_hasHomeImplRef" select="$hasHomeImpl"/>
               <xsl:with-param name="_hasFaultTolerance" select="true()"/>
            </xsl:call-template>
         </xsl:for-each>
      </xsl:when>
      <xsl:when test="$hasLoadBalancingServer='yes'">
         <![CDATA[// generate for LoadBalancingServer]]>
         <xsl:for-each select="$frameworkNodes">
            <xsl:call-template name="includeFramework">
               <xsl:with-param name="_framework" select="."/>
               <xsl:with-param name="_hasHomeImplRef" select="$hasHomeImpl"/>
               <xsl:with-param name="_hasLoadBalancing" select="true()"/>
            </xsl:call-template>
         </xsl:for-each>
      </xsl:when>
      <xsl:otherwise>
         <xsl:for-each select="$frameworkNodes">
            <xsl:call-template name="includeFramework">
               <xsl:with-param name="_framework" select="."/>
               <xsl:with-param name="_hasHomeImplRef" select="$hasHomeImpl"/>
            </xsl:call-template>
         </xsl:for-each>
      </xsl:otherwise>
   </xsl:choose>
   
   <!-- generate #include for datastore typetraits, but
        only for <datastore> nodes which are declared in 
        <fault-tolerance><datastores> node.
   -->
   <xsl:call-template name="generate_all_datastore_type_traits_includes">
      <xsl:with-param name="_datastorenodes" select="$datastoreNodes"/>
   </xsl:call-template>


   <![CDATA[
      #include <memory> // for std::auto_ptr
      #include <string>
      #include <vector>
      #include <sstream>
   ]]>

   <xsl:if test="$hasTrace">
     <xsl:choose>
       <xsl:when test="$hasBufferingEnabled='yes'">
      <![CDATA[
      #include <TraceAndPerf/tracelibrary/Trace.hpp>
      #include <TraceAndPerf/tracelibrary/CdmwTrace.hpp>
      #include <TraceAndPerf/tracelibrary/InitUtils.hpp>
      
      #define CDMW_TRACES_ENABLED
      
      ]]>
         
       </xsl:when>
       <xsl:otherwise>
      <![CDATA[
      // Use of LogManagerCollectorDelegate instead of the FlushAreaMngr/tracelibrary
      // NOTA: Discard of repetitive traces does not work with this configuration.
      #include <Foundation/logging/LogManagerCollectorDelegate.hpp>
      ]]>
         
       </xsl:otherwise>
     </xsl:choose>
   </xsl:if>

    <xsl:if test="$hasRequestFollowup='yes'">
      <![CDATA[
      #include <Foundation/commonsvcs/requestfollowup/NDCServiceInit.hpp>
      ]]>
   </xsl:if>

   
   <xsl:if test="$hasPerformance">
      <![CDATA[
      #include <TraceAndPerf/performancelibrary/InitUtils.hpp>
      
      #define CDMW_PERFORMANCE_ENABLED
      
      ]]>
   </xsl:if>
   <xsl:if test="$hasTime">
      <![CDATA[
       #include <Time/clockservice/CdmwCosClock.hpp>
       #include <Time/idllib/CosClockService.stub.hpp>
       #define CDMW_TIME_SERVICE_ENABLED
       
      ]]>
   </xsl:if>
   <xsl:if test="$hasLoadBalancing">
      <![CDATA[
        #include <LoadBalancing/lbinit/LBServiceInit.hpp>
       #define CDMW_LB_SERVICE_ENABLED
      ]]>
   </xsl:if>
   
   <xsl:if test="$hasRecording">
      <![CDATA[ 
// Anonymous namespace
namespace
{
   CORBA::Object_ptr
   set_recorder_policies(CORBA::ORB_ptr orb, 
                         CORBA::Object_ptr obj,
                         unsigned long callTimeout_millisec)
   {
      CORBA::PolicyList policy_list(2);
      CORBA::ULong i = 0L;
      // Create the timeout policy
      if (callTimeout_millisec != 0) {
          CORBA::Any timeout;
          // Timeout success
          //Set the timeout value as a TimeBase::TimeT (100 nanoseconds units)
          TimeBase::TimeT _timeout = ((unsigned long) (callTimeout_millisec * 1.0e4));
          timeout <<= _timeout;
          policy_list.length( i+1 );
          policy_list[i] = orb->create_policy( Messaging::RELATIVE_RT_TIMEOUT_POLICY_TYPE,
                                               timeout );
          ++i;
      }

      std::string syncWithTarget =
          Cdmw::OsSupport::SystemProperties::Get_Property(Cdmw::CdmwInit::RECORDING_SYNC_WITH_TARGET_PROPERTY_NAME, "no");
      
      Messaging::SyncScope sync_scope = Messaging::SYNC_WITH_SERVER;

      if(syncWithTarget == "yes") {
          sync_scope = Messaging::SYNC_WITH_TARGET;
      }

      CORBA::Any sync_scope_any;
      sync_scope_any <<= sync_scope;
      policy_list.length( i+1 );
      policy_list[i] = orb->create_policy (Messaging::SYNC_SCOPE_POLICY_TYPE,
                                           sync_scope_any);
      // Set the policy
      CORBA::Object_var capsule =
          obj->_set_policy_overrides( policy_list, CORBA::SET_OVERRIDE );
                                           
      return capsule._retn();
   }
}; // End anonymous namespace
      ]]>
   </xsl:if>
   

   //--------------------------------------------------------------------------

   <xsl:if test="$hasEventPorts='true'">
//
// Re-implement EventChannelFactory allocator to set 
// use_profile_manager parameter attribute to false 
// in EventChannelFactory constructor call.
//   
      <xsl:call-template name="openNamespace">
         <xsl:with-param name="_scope" select="$cdmweventNamespace"/>
         <xsl:with-param name="_lastTokenIsNamespace" select="true()"/>
      </xsl:call-template>
   
      <![CDATA[
        template <>
        PortableServer::Servant
        EventChannelFactory_impl_Allocator::allocate(
                CORBA::ORB_ptr orb,
                PortableServer::POA_ptr parent,
                CdmwLifeCycle::ObjectRegistration_ptr name_domain,
                const std::string & name_domain_name,
                const std::string & factory_name,
                Cdmw::OrbSupport::POAThreadingPolicy & threading_policy,
                int & argc, char** argv)
            throw(Cdmw::OutOfMemoryException,
                  Cdmw::BadParameterException,
                  Cdmw::InternalErrorException,
                  CORBA::SystemException)
        {
            try {
                return new Cdmw::Event::EventChannelFactory_impl(orb,
                                                                 parent,
                                                                 name_domain,
                                                                 name_domain_name,
                                                                 factory_name,
                                                                 threading_policy,
                                                                 argc,
                                                                 argv,
                                                                 false);
            }
            catch (const std::bad_alloc &)
            {
                CDMW_THROW(Cdmw::OutOfMemoryException);
            }
        }    
      ]]>
   
      <xsl:call-template name="closeNamespace">
         <xsl:with-param name="_scope" select="$cdmweventNamespace"/>
         <xsl:with-param name="_lastTokenIsNamespace" select="true()"/>
      </xsl:call-template>
   
   //--------------------------------------------------------------------------
   </xsl:if>
   
   <xsl:call-template name="openNamespace">
      <xsl:with-param name="_scope" select="$cdmwinitNamespace"/>
      <xsl:with-param name="_lastTokenIsNamespace" select="true()"/>
   </xsl:call-template>


   <![CDATA[
    void 
    CDMW_preORBInit(int & argc, char** argv) 
       throw (CORBA::SystemException)
    {
   ]]>
   
   
   <xsl:choose>
   <xsl:when test="$hasIssuerInterceptor='yes'">
   <![CDATA[
            // Value of "command-issuer" tag in xml generation file set to 'yes' (default value): 
            // Creating client interceptor in order to set command issuer
            PlatformMngt::PlatformInterface::Create_interceptor();

            ]]>
   </xsl:when>
   <xsl:otherwise>
   <![CDATA[
            // Value of "comman-issuer" tag in xml generation file set to 'no': 
            //No interceptor initialised 
            ]]>
   </xsl:otherwise>
   </xsl:choose>
 

    <xsl:if test="$hasFaultTolerance">
        <![CDATA[
            // generate call to FTServiceInit::Init
            try {
        ]]>
        
        <xsl:choose>
            <xsl:when test="$hasFaultToleranceServer='yes'">
                <![CDATA[Cdmw::FT::FTServiceInit::Init(argc, argv, false);]]>
            </xsl:when>
            <xsl:otherwise>
                <![CDATA[Cdmw::FT::FTServiceInit::Init(argc, argv, true);]]>
            </xsl:otherwise>
        </xsl:choose>
        
        <![CDATA[
            } catch (const CORBA::SystemException & e) {
                try {

                   CDMW_ERROR(InitUtils::Get_Logger(), 
                              "CORBA Exception : " 
                              << Cdmw::OrbSupport::OrbSupport::exception_to_string(e) );
                } catch(...) {
                    CDMW_ASSERT(false);
                }
            }
        ]]>
    </xsl:if>
    <xsl:if test="$hasLoadBalancing">
        <![CDATA[

         #ifdef CDMW_LB_SERVICE_ENABLED
         // *****************************************************************
         // *********         Init The CDMW LB Service              *********
         // *****************************************************************

            try {
                Cdmw::LB::LBServiceInit::Init (argc, argv);
            } catch (const CORBA::SystemException & e) {
               try {

                   CDMW_ERROR(InitUtils::Get_Logger(),
                              "CORBA Exception during LB init: "
                              << Cdmw::OrbSupport::OrbSupport::exception_to_string(e) );
               } catch(...) {
                    CDMW_ASSERT(false);
            }
         }
         #endif // CDMW_LB_SERVICE_ENABLED

      ]]>
   </xsl:if>

    
    <xsl:if test="$hasWatchdog">
      <![CDATA[
         // *****************************************************************
         // *************  setup WatchdogServiceInit  **********
         // *****************************************************************
        try {
            Cdmw::FT::WatchdogServiceInit::Init (argc, argv);
        } catch (const CORBA::SystemException & e) {
           try {
              CDMW_ERROR(InitUtils::Get_Logger(), 
                         "CORBA Exception : " 
                         << Cdmw::OrbSupport::OrbSupport::exception_to_string(e) );
           } catch(...) {
               CDMW_ASSERT(false);
           }
        }        
      ]]>
   </xsl:if>


    <xsl:if test="$hasRequestFollowup='yes'">
      <![CDATA[
         // *****************************************************************
         // *************  setup RequestFollowupInit  **********
         // *****************************************************************
        try {
            Cdmw::CommonSvcs::RequestFollowUp::NDCServiceInit::Init (argc, argv);
        } catch (const CORBA::SystemException & e) {
           try {
              CDMW_ERROR(InitUtils::Get_Logger(), 
                         "CORBA Exception : " 
                         << Cdmw::OrbSupport::OrbSupport::exception_to_string(e) );
           } catch(...) {
               CDMW_ASSERT(false);
           }
        }        
      ]]>
   </xsl:if>
   <![CDATA[
   }
   ]]>


   <![CDATA[
    void 
    CDMW_init(CORBA::ORB_ptr orb, int & argc, char** argv, ProcessControl* ctrl) 
       throw (CORBA::SystemException)
    {

            CDMW_ASSERT(!CORBA::is_nil(orb));
            CdmwProcess_impl * process_impl = 0;

      try {
         // initialise the LogManager
         Cdmw::Logging::LogManager::Init(argc, argv);

         // PCR-0049
         Cdmw::OrbSupport::CodecBase::Init(orb);

         // Initialize XML library
         Cdmw::CdmwInit::InitUtils::init_xml_library();

         // Get the XML init filename. This may raise CORBA::BAD_PARAM, CORBA::NO_MEMORY,
         // or CORBA::INTERNAL    
         std::string xml_file = InitUtils::get_xml_initialisation_file(argc, argv);
        
         // Get the XML documents validation option. This may raise CORBA::NO_MEMORY,
         // or CORBA::INTERNAL
         bool validate_xml = InitUtils::get_xml_validation_option(argc, argv);
        
         CDMW_INFO(InitUtils::Get_Logger(), 
                   "Initialise process with XML file " << xml_file
                   << (validate_xml ? " (it will be validated)"
                                    : " (it will not be validated)"));

         // Parsing xml
         std::auto_ptr<Cdmw::CdmwInit::CdmwInitConfiguration> xml_data(0);
         try {
            std::auto_ptr<Cdmw::CdmwInit::CdmwInitConfiguration> init
               (Cdmw::CdmwInit::CdmwInitConfiguration::ReadConfiguration(xml_file,
                                                                            validate_xml));
            
            xml_data = init;
         } catch (const Cdmw::OutOfMemoryException & ex) {
            CDMW_WARN(InitUtils::Get_Logger(), "Receive Cdmw::OutOfMemoryException "
                                               << ex.what());
            CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                          CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY, 
                                           CORBA::COMPLETED_NO) );
         } catch (const Cdmw::CdmwInit::ConfigurationException & ex) {
            CDMW_WARN(InitUtils::Get_Logger(), "Receive CdmwInit::ConfigurationException "
                                               << ex.what());
            CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                          CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                           CORBA::COMPLETED_NO),
                          ex.what());
         } catch(const Cdmw::InternalErrorException & ex){
            CDMW_WARN(InitUtils::Get_Logger(), "Receive Cdmw::InternalErrorException "
                                               << ex.what());
            CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                          CORBA::INTERNAL(Cdmw::OrbSupport::INTERNALXMLLibraryError,
                                          CORBA::COMPLETED_NO),
                          ex.what() );
         }

         // *****************************************************************
         // *****************  CDMW root POA and servant manager ***********
         // *****************************************************************
         //
         // Create the CDMW root POA. It has its own POA Manager
         // May raise a CORBA::SystemException.
         //
         PortableGroup::GOA_var CDMW_rootPOA = InitUtils::create_cdmw_root_GOA(orb);
        
         PortableServer::POAManager_var poaManager = CDMW_rootPOA -> the_POAManager();
         //
         // Initialize the server so that incoming requests are
         // handled from this point on
         //
         // May raise PortableServer::AdapterInactive or CORBA::SystemException
         try {
            CDMW_DEBUG(InitUtils::Get_Logger(), "Activate POA Manager"); 
            poaManager -> activate(); // TODO: maybe move it?
         } catch (const PortableServer::POAManager::AdapterInactive & ) {
            CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                          CORBA::INTERNAL(Cdmw::OrbSupport::INTERNALORBAdapterInactive,
                                          CORBA::COMPLETED_NO) );
         } catch (const CORBA::SystemException & ex ) {
            CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,
                          ex, "Could not activate the CDMW POA Manager!" );
         }
         

   ]]>

    <xsl:if test="$hasFaultTolerance">
      <![CDATA[
         // *****************************************************************
         // *************  setup FaultTolerance Policies   ******************
         // *****************************************************************

         Cdmw::OrbSupport::OrbSupport::set_endpoint_selector_factory(::Cdmw::FT::FTConfiguration::Get_FT_endpoint_selector_factory());

         size_t duration_time;
         std::string client_id_str;
            
         try {
               std::string res;
               try {
                  res = xml_data->getServiceAttributeValue("fault-tolerance", "request-duration-policy"); 
                  duration_time = atoi(res.c_str());
               } catch (const Cdmw::BadParameterException& ex) {
                  CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                                CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                                 CORBA::COMPLETED_NO),
                                ex.what() );
               }

               try  {
                  client_id_str = xml_data->getServiceAttributeValue("fault-tolerance", "client-id");
                  ::Cdmw::FT::FTConfiguration::Set_client_id(orb, client_id_str.c_str());
               } catch (const Cdmw::BadParameterException& ) {
                  // do nothing : parameter is optional
               }
          } catch (const Cdmw::CdmwInit::XMLErrorException & ex) {
               CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                             CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                              CORBA::COMPLETED_NO),
                             ex.what() );
          } catch (const OutOfMemoryException & ex) {
               CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                             CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                              CORBA::COMPLETED_NO),
                             ex.what() );
          } 

         CORBA::Object_var obj;
         obj = orb->resolve_initial_references("ORBPolicyManager");
        
         CORBA::PolicyManager_var policy_manager 
            = CORBA::PolicyManager::_narrow(obj.in());
        
         // Create and apply an ORB-wide Routed policy
         CDMW_DEBUG(InitUtils::Get_Logger(), "Request duration policy set to " << duration_time);
         CORBA::Any any;
         TimeBase::TimeT duration = duration_time;

         any <<= duration;
         CORBA::PolicyList policies(1);
         policies.length(1);
         policies[0] =
            orb->create_policy(::FT::REQUEST_DURATION_POLICY, any);
        
         policy_manager->set_policy_overrides(policies, CORBA::SET_OVERRIDE);
      ]]>
   </xsl:if>



    <xsl:if test="$hasWatchdog">
      <![CDATA[
         // *****************************************************************
         // *************  INIT WatchdogServiceInit  **********
         // *****************************************************************
         std::string timeout_s("");
         try {
             CORBA::ULong milliseconds = 
                 Cdmw::OsSupport::SystemProperties::Get_Property(
                     Cdmw::FT::WATCHDOG_TIMEOUT_PROPERTY_NAME, 0);

             if (milliseconds == 0)
             {
                timeout_s  
                   = xml_data->getServiceAttributeValue("watchdog", "timeout");
                milliseconds = atoi(timeout_s.c_str());
                Cdmw::FT::WatchdogServiceInit::Set_timeout (milliseconds);
             }

             // export property for debug purpose
             Cdmw::OsSupport::SystemProperties::Set_Property(
               Cdmw::FT::WATCHDOG_TIMEOUT_PROPERTY_NAME, 
               milliseconds);
         } catch (const Cdmw::BadParameterException& ex) {
            CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                          CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                           CORBA::COMPLETED_NO),
                          ex.what() );       
         } catch (const Cdmw::CdmwInit::XMLErrorException & ex) {
            CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                          CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                           CORBA::COMPLETED_NO),
                          ex.what() );
         } catch (const OutOfMemoryException & ex) {
            CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                          CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                           CORBA::COMPLETED_NO),
                          ex.what() );
         } catch (const CORBA::BAD_PARAM &) {
            CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                          CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAM,
                                           CORBA::COMPLETED_NO),
                          "Bad timeout value for Watchdog.");
         }
      ]]>
   </xsl:if>


   <xsl:if test="($hasBufferingEnabled='no') or $hasRecording">
     
      <![CDATA[

         // *****************************************************************
         // *********   Retrieve the Daemon's NameService object      *******
         // *********   reference and register it as LocalNameService *******
         // *********   SystemPort option is now handled by           *******
         // *********   OrbSupport::ORB_init(argc,argv)               *******
         // *****************************************************************
         CosNaming::NamingContext_var local_NameService = CosNaming::NamingContext::_nil();

         std::string system_port =
            Cdmw::OsSupport::SystemProperties::Get_Property(Cdmw::OrbSupport::SYSTEM_PORT_PROPERTY_NAME);
         if (system_port.empty()) {
            CDMW_FATAL(InitUtils::Get_Logger(), "CdmwSystemPort has not be declared"
                       << " in argument. This is required to get the object referennce of"
                       << " the LogCollector and/or the recording daemon and recording server.");
            CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                         CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMMissingPortNumber, 
                                          CORBA::COMPLETED_NO) );
         } else {
            try {
               std::ostringstream ostr;
               ostr << "corbaloc::localhost:"<<system_port<<"/NameService";
               CORBA::Object_var obj__ = orb->string_to_object(ostr.str().c_str());
               local_NameService  = CosNaming::NamingContext::_narrow(obj__.in());
               if (CORBA::is_nil(local_NameService.in())) {
                  ostr << " is not a NameServer reference!";
                  CDMW_FATAL(InitUtils::Get_Logger(), ostr.str());
                  CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                               CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMNilObjectReference, 
                                                CORBA::COMPLETED_NO),
                               ostr.str());
               }
               orb->register_initial_reference("LocalNameService", local_NameService.in());
            } catch (const CORBA::SystemException& ex) {
               CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,
                            ex, "Could not retrieve local NameService's object reference!" );
            } catch (const CORBA::ORB::InvalidName& )  {
               CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                            CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL, CORBA::COMPLETED_NO),
                            "InvalidName => Cannot register LocalNameService.");
            }
         }
      ]]>   

   </xsl:if>




      <![CDATA[
         // *****************************************************************
         // *****************  setup platforminterface object  **************
         // *****************************************************************

         // Did we receive a ProcessControl callback object?
         CdmwPlatformMngt::ProcessDelegate_var process =
            CdmwPlatformMngt::ProcessDelegate::_nil();
         {
            ProcessControl_var process_ctrl_var;
            if (ctrl != 0)
                process_ctrl_var = ctrl;
            else
                process_ctrl_var = new DefaultProcessControl();
            CdmwFT::LocalPullMonitorable_var lpm = CdmwFT::LocalPullMonitorable::_nil();
      ]]>   
   <xsl:if test="$hasWatchdog">
      <![CDATA[
            lpm = new Cdmw::FT::Watchdog::RequestTimeoutLocalPullMonitorable_impl();
      ]]>   
   </xsl:if>
   <![CDATA[         
            // CdmwProcess_impl() will acquire a reference on ProcessControl object and
            // will automatically release it at object's destruction.
            PortableServer::POA_var rootPOA = InitUtils::get_root_POA(orb);
            process_impl = new CdmwProcess_impl(orb,
                                                 rootPOA.in(),
                                                 CDMW_rootPOA.in(),
                                                 process_ctrl_var._retn(), lpm._retn() );
            PortableServer::ServantBase_var process_impl_servant = process_impl;
            
            process = process_impl->_this();
     ]]>
    <xsl:choose>
            <xsl:when test="$hasSystemManagement='yes'">
            <![CDATA[// Set the Property before calling init_platform_interface() which calls
            // PlatformInterface::Setup(). The Property is read in Setup().

            Cdmw::OsSupport::SystemProperties::Set_Property(
               Cdmw::OrbSupport::SYSTEM_MANAGEMENT_FATAL_HANDLER_PROPERTY_NAME, 
               "true");]]>
            </xsl:when>
            <xsl:otherwise>
            <![CDATA[// Set the Property before calling init_platform_interface() which calls PlatformInterface::Setup().
            // The Property is read in Setup().

            Cdmw::OsSupport::SystemProperties::Set_Property(
               Cdmw::OrbSupport::SYSTEM_MANAGEMENT_FATAL_HANDLER_PROPERTY_NAME, 
               "false");]]>
            </xsl:otherwise>
    </xsl:choose>
   <![CDATA[        
            InitUtils::init_platform_interface(orb, argc, argv, process.in());
         }

         //
         // Get the process name. This may raise CORBA::BAD_INV_ORDER
         std::string process_name = InitUtils::get_cdmw_process_name();
         //
         // Get the application name. This may raise CORBA::BAD_INV_ORDER
         std::string application_name = InitUtils::get_cdmw_application_name();  

         CDMW_INFO(InitUtils::Get_Logger(), "CDMW_init ongoing for " 
                   << application_name << "/" << process_name);
         Cdmw::Logging::LogManager::Instance()->set_process_name(process_name);
         Cdmw::Logging::LogManager::Instance()->set_application_name(application_name);

         // Retreive Repository object reference. This may raise CORBA::BAD_PARAM
         // or CORBA::BAD_INV_ORDER
         CdmwNamingAndRepository::Repository_var repository 
            = InitUtils::get_cdmw_repository();

   ]]>


   <xsl:if test="$hasFaultTolerance">
   <![CDATA[
         // *****************************************************************
         // ***********  setup FaultTolerance ReplicationManager  ***********
         // *****************************************************************

         // Get the reference of the Replication Manager and register it in the ORB
         CORBA::Object_var objref;
         const char* replication_manager_name = "ReplicationManager";
            objref =  InitUtils::get_service(replication_manager_name);  

         CdmwFT::ReplicationManager_var replication_manager = CdmwFT::ReplicationManager::_narrow(objref.in());

          try 
         {
             orb->register_initial_reference("ReplicationManager", replication_manager.in());      
         }
         catch (const CORBA::SystemException& ex)
         {
            CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,
                          ex,
                          "Cannot register ReplicationManager." );
          }
          catch (const CORBA::ORB::InvalidName& )
          {
            CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                          CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL, CORBA::COMPLETED_NO),
                                  "InvalidName => Cannot register ReplicationManager.");
          }
     ]]>
    </xsl:if>

      <xsl:choose>
         <xsl:when test="$hasFaultToleranceServer='yes'">

         <xsl:variable name="hasReplyLogging">
           <xsl:choose>
               <xsl:when test="boolean($servicesNode/fault-tolerance/@reply-logging)">
                   <xsl:value-of select="$servicesNode/fault-tolerance/@reply-logging"/>
               </xsl:when>
               <xsl:otherwise>
                   <xsl:value-of select="'no'"/>
               </xsl:otherwise>
           </xsl:choose>
         </xsl:variable>

      <![CDATA[
         // *****************************************************************
         // *************  setup FaultTolerance LocationManager  ************
         // *****************************************************************
         
         // Location information shall be obtain from supervision
         ::FT::Location loc;
         loc.length(3);

         loc[0].id = CORBA::string_dup(Cdmw::OsSupport::OS::get_hostname().c_str());
         loc[0].kind = "hostname";
         loc[1].id = application_name.c_str();
         loc[1].kind = "applicationname";
         loc[2].id = process_name.c_str();
         loc[2].kind = "processname";
         
         CDMW_DEBUG(InitUtils::Get_Logger(), 
                    "Setup FaultTolerance LocationManager for location: " 
                    << Cdmw::CommonSvcs::Naming::NamingInterface::to_string(loc));
            
         // create a reference for the PrimaryBackupGroupRepository
         ::Cdmw::FT::Location::PrimaryBackupGroupRepository_impl * primary_backup_repository
             = ::Cdmw::FT::Location::PrimaryBackupGroupRepository_impl::Get_instance();
         

      ]]>
      <xsl:choose>
      <xsl:when test="$hasDataStore='true'">

      <![CDATA[

      // get the fault notifier reference
      ::FT::FaultNotifier_var fault_notifier = replication_manager->get_fault_notifier();

      // create the DataStoreTimeoutHandler
      CDMW_DEBUG(InitUtils::Get_Logger(), "Create the DataStoreTimeoutHandler");
      Cdmw::FT::LocationDataStoreTimeoutHandler* locationDataStoreTimeoutHandler
          = new Cdmw::FT::LocationDataStoreTimeoutHandler(fault_notifier.in());

      // Register the timeoutHandler into the DataStoreBase
      Cdmw::FT::DataStoreBase::Register_Timeout_Handler(locationDataStoreTimeoutHandler);


      // FaultTolerance use datastore
      CORBA::ULong datastore_id;
      std::string multicast_port;
      std::string multicast_domain;
      std::string multicast_IP_address;
      int max_transaction_in_progress;
      int max_transaction_done;
      OsSupport::OS::Timeval cohort_timeout;
      OsSupport::OS::Timeval coordinator_timeout;
      OsSupport::OS::Timeval freeze_timeout;
      long number_of_scheduler_thread;
      long chunk_size;
      long freeze_size;

      long dataStore_inc=0;
      ]]>

      <xsl:variable name="dataStoresKind" select="$servicesNode/fault-tolerance/datastores/@kind"/>
      <xsl:variable name="countDatastoreNodes" select="count($datastoreNodes)"/>

      <xsl:variable name="dataStoreOccurrence">
         <xsl:call-template name="count.datastore.occurrence">
            <xsl:with-param name="_datastoreNodes" select="$datastoreNodes"/>
            <xsl:with-param name="_occurrences" select="$countDatastoreNodes"/>
            <xsl:with-param name="_datastoreOccurrences" select="0"/>
            <xsl:with-param name="_index_datastore" select="1"/>
         </xsl:call-template>
      </xsl:variable>
      
      <![CDATA[long localDataStoreInfosSize = ]]><xsl:value-of select="$dataStoreOccurrence"/><![CDATA[;
      CdmwFT::StateTransfer::LocalDataStoreInfos_var localDataStoreInfos = 
          new CdmwFT::StateTransfer::LocalDataStoreInfos();
      localDataStoreInfos->length(1);
      ]]>

      <![CDATA[
      std::string hotrestart_arg = 
          ::Cdmw::OsSupport::OS::get_option_value( argc, argv, "--HotRestart");
      bool cold_restart = (hotrestart_arg != "yes");

      CDMW_DEBUG(InitUtils::Get_Logger(), 
                 "Setup StatefullPrimaryBackupAdmin_impl "
                 << (cold_restart ? "with cold restart" : "with hot restart") );
                 
      ::Cdmw::FT::Location::StatefullPrimaryBackupAdmin_impl * statefull_primary_backup_admin =
          new ::Cdmw::FT::Location::StatefullPrimaryBackupAdmin_impl(orb, 
                                                                     CDMW_rootPOA.in(), 
                                                                     loc, 
                                                                     primary_backup_repository, 
                                                                     cold_restart);

       bool reply_logging = false;                                                                            
      ]]>

      <!-- 
          get the kind of datastore (IIOP or MIOP)
      -->

      <xsl:call-template name="datastore.context">
         <xsl:with-param name="_dataStoresKind" select="$dataStoresKind"/>
     <xsl:with-param name="_hasReplyLogging" select="$hasReplyLogging"/>
      </xsl:call-template>

      <xsl:call-template name="datastore.occurrence">
         <xsl:with-param name="_datastoreNodes" select="$datastoreNodes"/>
     <xsl:with-param name="_occurrencesInProgress" select="1"/>
     <xsl:with-param name="_occurrencesInProgressForThisDataStore" select="1"/>
         <xsl:with-param name="_occurrences" select="$dataStoreOccurrence"/>
         <xsl:with-param name="_dataStoresKind" select="$dataStoresKind"/>
         <xsl:with-param name="_index_datastore" select="1"/>
      </xsl:call-template>
         
      <xsl:choose>
         <xsl:when test="$hasReplyLogging='yes'">
<![CDATA[
      CDMW_DEBUG(InitUtils::Get_Logger(), "Setup ReplyLogging");
      reply_logging = true;
      typedef Cdmw::FT::TXDataStore<std::string, CdmwFT::ReplyData> 
          ReplyLogging;

      datastore_id = Cdmw::FT::FT_REPLY_DATASTORE_ID;

      ReplyLogging * logging = new ReplyLogging(*manager,
                                              datastore_id,
                                              orb,
                                              CDMW_rootPOA.in());
    
      (*localDataStoreInfos)[0].dsids.length(dataStore_inc + 1);
      (*localDataStoreInfos)[0].dsids[dataStore_inc++] = Cdmw::FT::FT_REPLY_DATASTORE_ID;

      Cdmw::FT::FTServiceInit::Set_reply_recorder
          (new Cdmw::FT::RequestLogging::ReplyRecording(logging->get_datastore()));

]]>
         </xsl:when>
      </xsl:choose>

      <![CDATA[
      CDMW_DEBUG(InitUtils::Get_Logger(), "Register statefull location");
      ::CdmwFT::Location::StatefullPrimaryBackupAdmin_var statefull_primary_backup_admin_ref = 
          statefull_primary_backup_admin->_this();

      replication_manager->register_statefull_location(
          loc, 
          statefull_primary_backup_admin_ref.in(),
          localDataStoreInfos.in(), 
          reply_logging);
      ]]>

      </xsl:when>
      <xsl:otherwise>
      
      <![CDATA[
      CDMW_DEBUG(InitUtils::Get_Logger(), "Setup PrimaryBackupAdmin_impl");
      ::Cdmw::FT::Location::PrimaryBackupAdmin_impl * primary_backup_admin =
          new ::Cdmw::FT::Location::PrimaryBackupAdmin_impl(orb, 
                                                            CDMW_rootPOA.in(), 
                                                            loc, 
                                                            primary_backup_repository);
         
      ::CdmwFT::Location::PrimaryBackupAdmin_var primary_backup_admin_ref = 
          primary_backup_admin->_this();

      replication_manager->register_location(loc, 
                                             primary_backup_admin_ref.in());
      ]]>
      
      </xsl:otherwise>
      </xsl:choose>
      </xsl:when>
   </xsl:choose>

   <xsl:if test="$hasRepositoryInterface">
      <![CDATA[
         // *****************************************************************
         // *****************  INIT of RepositoryInterface ******************
         // *****************************************************************
         CDMW_DEBUG(InitUtils::Get_Logger(), "Init RepositoryInterface");
         std::string default_domaine_name("");
         try {
            default_domaine_name  
               = xml_data->getServiceAttributeValue("repository-interface", "default-domain-name");
          } catch (const Cdmw::BadParameterException& ex) {
            CDMW_WARN(InitUtils::Get_Logger(), "Receive Cdmw::BadParameterException "
                                               << ex.what());
            CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                          CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                           CORBA::COMPLETED_NO),
                          ex.what() );       
         } catch (const Cdmw::CdmwInit::XMLErrorException & ex) {
            CDMW_WARN(InitUtils::Get_Logger(), "Receive CdmwInit::XMLErrorException "
                                               << ex.what());
            CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                          CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                           CORBA::COMPLETED_NO),
                          ex.what() );
         } catch (const OutOfMemoryException & ex) {
            CDMW_WARN(InitUtils::Get_Logger(), "Receive Cdmw::OutOfMemoryException "
                                               << ex.what());
            CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                          CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                           CORBA::COMPLETED_NO),
                          ex.what() );
         } 

         try {
            Cdmw::NamingAndRepository::RepositoryInterface::init
               (default_domaine_name, repository.in());
         } catch (const CdmwNamingAndRepository::NoNameDomain & ) {
            // Default NameDomain is not a NameDomain! 
            CDMW_WARN(InitUtils::Get_Logger(), 
                      "Receive CdmwNamingAndRepository::NoNameDomain for " 
                      << default_domaine_name);
            CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                          CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMNameDomainNotFound,
                                           CORBA::COMPLETED_NO),
                          default_domaine_name + " is not a NameDomain!" );
         } catch (const CdmwNamingAndRepository::InvalidName &  ex) {
            // Default NameDomain name is ill-formed! 
            CDMW_WARN(InitUtils::Get_Logger(), 
                      "Receive CdmwNamingAndRepository::InvalidName for " 
                      << default_domaine_name);
            CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                          CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidDefaultDomainName,
                                           CORBA::COMPLETED_NO),
                          default_domaine_name + " is ill-formed :" + ex.reason.in() );
         } catch (const CORBA::SystemException & ex) {
            CDMW_WARN(InitUtils::Get_Logger(), 
                      "Receive CORBA::SystemException " 
                      << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex));
            CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,
                          ex,
                          "Cannot initialise RepositoryInterface." );
         }
      ]]>
   </xsl:if>

   <xsl:if test="$hasTrace">
     <xsl:choose>
       <xsl:when test="$hasBufferingEnabled='yes'">
      <![CDATA[
         #ifdef CDMW_TRACES_ENABLED

         // *****************************************************************
         // *********         Init The CDMW Trace Library           *********
         // *****************************************************************
         {
            CDMW_DEBUG(InitUtils::Get_Logger(), "Init Trace Library");
            std::string res;
            size_t flushing_time = 0;
            size_t msg_threshold = 0;
            size_t nb_FlushArea = 0;
            size_t size_FlushArea = 0;
            std::string collectorName;
            std::vector<std::string> collectorNameList;

            try {
               try {
                  res = xml_data->getServiceAttributeValue("trace", "flushing-time"); //FlushAreaMngr::DEFAULT_FLUSHING_TIME;
                  flushing_time = atoi(res.c_str());
                  res = xml_data->getServiceAttributeValue("trace", "flush-area-nb"); //FlushAreaMngr::DEFAULT_AREA_NB;
                  nb_FlushArea = atoi(res.c_str());
                  res = xml_data->getServiceAttributeValue("trace", "flush-area-sz"); //FlushAreaMngr::DEFAULT_AREA_SIZE;
                  size_FlushArea = atoi(res.c_str());
                  collectorName = xml_data->getServiceAttributeValue("trace", "collector-name");
                  collectorNameList.push_back(collectorName);
               }
               catch (const Cdmw::BadParameterException& ex) {
                  CDMW_WARN(InitUtils::Get_Logger(), "Cdmw::BadParameterException received: "
                            << ex.what());
                  CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                                CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                                 CORBA::COMPLETED_NO),
                                ex.what() );
               }

               try  {
                  res = xml_data->getServiceAttributeValue("trace", "msg-threshold"); //FlushAreaMngr::DEFAULT_MSG_THRESHOLD;
                  msg_threshold = atoi(res.c_str());

               } catch (const Cdmw::BadParameterException& ) {
                  // do nothing : parameter is optional
               }
            } catch (const Cdmw::CdmwInit::XMLErrorException & ex) {
               CDMW_WARN(InitUtils::Get_Logger(), "CdmwInit::XMLErrorException received: "
                         << ex.what());
               CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                             CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                              CORBA::COMPLETED_NO),
                             ex.what() );
            } catch (const OutOfMemoryException & ex) {
               CDMW_WARN(InitUtils::Get_Logger(), "Cdmw::OutOfMemoryException received: "
                         << ex.what());
               CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                             CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                              CORBA::COMPLETED_NO),
                             ex.what() );
            }

            // May raise CORBA::SystemException
            Trace::InitUtils::init_trace_library(orb,
                                                 CDMW_rootPOA.in(),
                                                 application_name,
                                                 process_name,
                                                 flushing_time,
                                                 msg_threshold,
                                                 nb_FlushArea,
                                                 size_FlushArea,
                                                 collectorNameList);

            CDMW_TRACE_ACTIVE_FLUSHING();
         }

         #endif // CDMW_TRACES_ENABLED
      ]]>
         
       </xsl:when>
       <xsl:otherwise>    
      <![CDATA[
       
         // *****************************************************************
         // *********   Init The CDMW LogManagerCollectorDelegate   *********
         // *****************************************************************
         {

            CDMW_DEBUG(InitUtils::Get_Logger(), "Init LogManagerCollectorDelegate");
            size_t msg_threshold = 0; // NOTA: Not used in this version
            std::string collectorName;
            try {
               try {
                  collectorName = xml_data->getServiceAttributeValue("trace", "collector-name");
               }
               catch (const Cdmw::BadParameterException& ex) {
                  CDMW_DEBUG(InitUtils::Get_Logger(), "Cdmw::BadParameterException received: "
                            << ex.what());
               }

               try  {
                  std::string res = xml_data->getServiceAttributeValue("trace", "msg-threshold"); //FlushAreaMngr::DEFAULT_MSG_THRESHOLD;
                  msg_threshold = atoi(res.c_str());

               } catch (const Cdmw::BadParameterException& ) {
                  // do nothing : parameter is optional
               }
            } catch (const Cdmw::CdmwInit::XMLErrorException & ex) {
               CDMW_WARN(InitUtils::Get_Logger(), "CdmwInit::XMLErrorException received: "
                         << ex.what());
               CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                             CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                              CORBA::COMPLETED_NO),
                             ex.what() );
            } catch (const OutOfMemoryException & ex) {
               CDMW_WARN(InitUtils::Get_Logger(), "Cdmw::OutOfMemoryException received: "
                         << ex.what());
               CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                             CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                              CORBA::COMPLETED_NO),
                             ex.what() );
            }
            // We have a collector name now. Let's retreive it from PlatformDaemon (NameServer).
            // Use collectorName if provided by user, otherwise use default one.
            if (collectorName.empty()) {
               collectorName = Cdmw::Common::Locations::DEFAULT_LOG_COLLECTOR_ID;
            }
            CdmwLogging::LogCollector_var log_collector__ =
               CdmwLogging::LogCollector::_nil();
            if (CORBA::is_nil(local_NameService.in()))
            {
               CDMW_WARN(InitUtils::Get_Logger(), "No Local NameService available. "
                         << "Trace logs will not be sent to Trace Collector.");
            } else {
               Cdmw::CommonSvcs::Naming::NamingInterface local_ni(local_NameService.in());
               CORBA::Object_var obj__ = CORBA::Object::_nil();
               try {
                  obj__ = local_ni.resolve(collectorName);
                  log_collector__ = CdmwLogging::LogCollector::_narrow(obj__.in());
                  if (CORBA::is_nil(log_collector__.in())) {
                     CDMW_WARN(InitUtils::Get_Logger(), 
                               "'" << collectorName << "' does not denote a valid "
                               << "Log Collector object. "
                               << "Trace logs will not be sent to Trace Collector.");
                  }
               } catch (const CosNaming::NamingContext::InvalidName& ex) {
                  CDMW_WARN(InitUtils::Get_Logger(), 
                            "Could not retrieve reference of the Log Collector "
                            << "'" << collectorName << "' not a valid name. "
                            << "Trace logs will not be sent to Trace Collector.");
               } catch (const CosNaming::NamingContext::NotFound& nf) {
                  CDMW_WARN(InitUtils::Get_Logger(), 
                            "Could not retrieve reference of the Log Collector "
                            << "'" << collectorName << "' not found in the naming. "
                            << "Trace logs will not be sent to Trace Collector.");
               }
            }
            if (!CORBA::is_nil(log_collector__.in())) {
               try {
                  using namespace Cdmw::Logging;
                  LogManagerCollectorDelegate * delegate__ =
                     new LogManagerCollectorDelegate(log_collector__.in());
                  LogManagerDelegate* d__ = 
                     LogManager::Instance()->register_alternate_delegate(delegate__);
                  if (d__) delete d__;
               } catch (const Cdmw::OutOfMemoryException& ex) {
                  CDMW_WARN(InitUtils::Get_Logger(), "Receive Cdmw::OutOfMemoryException "
                            << ex.what() << ". Trace logs will not be sent to Trace Collector.");
               }
            }
         }

      ]]>
       </xsl:otherwise>
     </xsl:choose>
   </xsl:if>


   <xsl:if test="$hasPerformance">
      <![CDATA[
         #ifdef CDMW_PERFORMANCE_ENABLED
         
         // *****************************************************************
         // *********         Init The CDMW Performance Library    *********
         // ***************************************************************** 
         {
            CDMW_DEBUG(InitUtils::Get_Logger(), "Init Performance Library");
            std::string performanceFileName;
            try {
               performanceFileName = xml_data->getServiceAttributeValue("performance", "result-file-name");
            } catch (const Cdmw::BadParameterException& ex ) {
               CDMW_WARN(InitUtils::Get_Logger(), "Cdmw::BadParameterException received: "
                         << ex.what());
               CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                             CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                              CORBA::COMPLETED_NO),
                             ex.what() );
            } catch (const Cdmw::CdmwInit::XMLErrorException & ex ) {
               CDMW_WARN(InitUtils::Get_Logger(), "CdmwInit::XMLErrorException received: "
                         << ex.what());
               CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                             CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                              CORBA::COMPLETED_NO),
                             ex.what() );
            } catch (const OutOfMemoryException & ex ) {
               CDMW_WARN(InitUtils::Get_Logger(), "Cdmw::OutOfMemoryException received: "
                         << ex.what());
               CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                             CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                              CORBA::COMPLETED_NO),
                             ex.what() );
            }
    
            CDMW_DEBUG(InitUtils::Get_Logger(), 
                       "Performance result file will be " << performanceFileName);
            // May raise CORBA::SystemException
            Performance::InitUtils::init_performance_library(CDMW_rootPOA.in(),
                                                             application_name,
                                                             process_name,
                                                             performanceFileName);
         }
         #endif // CDMW_PERFORMANCE_ENABLED
      ]]>
   </xsl:if>

   <xsl:if test="$hasLoadBalancing">
   <![CDATA[
         // *****************************************************************
         // ***********    setup LoadBalancing LBGroupManager     ***********
         // *****************************************************************

         // Try to get the reference of the FT LBGroupManager from Repository
         CdmwLB::LBGroupManager_var lb_group_mgr = CdmwLB::LBGroupManager::_nil();
         try {
             // Get NamingInterface to object_groups (for reading)
             Cdmw::CommonSvcs::Naming::NamingInterface obj_groups_interface =
                 Cdmw::NamingAndRepository::RepositoryInterface::get_domain_naming_interface(
                     "lb_group_manager/object_groups");
    
             // Get LB GroupManager Object Group
             CORBA::Object_var obj_ref = 
                 obj_groups_interface.resolve("LBGroupManagerGroup");
    
             lb_group_mgr = CdmwLB::LBGroupManager::_narrow(obj_ref.in());
         }
         catch (const CdmwNamingAndRepository::NoNameDomain& ex) {
             CDMW_WARN(InitUtils::Get_Logger(), 
                 "FT LBGroupManager not found in Repository " 
                 << "(CdmwNamingAndRepository::NoNameDomain). "
                 << "Assumes non-FT LBGroupManager has been specified "
                 << "with -ORBInitRef...");
         }
         catch (const CdmwNamingAndRepository::InvalidName& ex) {
             // shouldn't happen since names are valid
             CDMW_FATAL(InitUtils::Get_Logger(), 
                "Error retrieving LBGroupManager from Repository: " 
                << "CdmwNamingAndRepository::InvalidName");
             CDMW_ASSERT(false);
         }
         catch (const CosNaming::NamingContext::NotFound& ex) {
             CDMW_WARN(InitUtils::Get_Logger(), 
                 "FT LBGroupManager not found in Repository "
                 << "(CosNaming::NamingContext::NotFound). "
                 << "Assumes non-FT LBGroupManager has been specified "
                 << "with -ORBInitRef...");
         }
         catch (const CosNaming::NamingContext::CannotProceed& ex) {
             CDMW_WARN(InitUtils::Get_Logger(), 
                 "Exception retrieving FT LBGroupManager from Repository " 
                 << "(CosNaming::NamingContext::CannotProceed). "
                 << "Assumes non-FT LBGroupManager has been specified "
                 << "with -ORBInitRef...");
         }
         catch (const CosNaming::NamingContext::InvalidName& ex) {
             // shouldn't happen since names are valid
             CDMW_FATAL(InitUtils::Get_Logger(), 
                "Error retrieving LBGroupManager from Repository: " << ex);
             CDMW_ASSERT(false);
         }
         catch (const CORBA::SystemException& ex) {
             CDMW_WARN(InitUtils::Get_Logger(), 
                 "Exception retrieving FT LBGroupManager from Repository: " << ex
                 << ". Assumes non-FT LBGroupManager has been specified "
                 << "with -ORBInitRef...");
         }

         if (!CORBA::is_nil(lb_group_mgr.in())) {
             try {
                 orb->register_initial_reference("LBGroupManager", lb_group_mgr.in());      
             }
             catch (const CORBA::SystemException& ex)
             {
                 CDMW_ERROR(InitUtils::Get_Logger(), 
                     "Error regitering LBGroupManager as initial reference: " << ex);
                 CDMW_THROW2(Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,
                             ex,
                             "Cannot register LBGroupManager." );
             }
             catch (const CORBA::ORB::InvalidName& ex)
             {
                 CDMW_ERROR(InitUtils::Get_Logger(), 
                     "Error regitering LBGroupManager as initial reference: " << ex);
                 CDMW_THROW2(Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                             CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL, CORBA::COMPLETED_NO),
                             "InvalidName => Cannot register LBGroupManager.");
             }
         }

     ]]>
    </xsl:if>
   
   <xsl:if test="$hasTime">
      <![CDATA[
         #ifdef CDMW_TIME_SERVICE_ENABLED
         
         // *****************************************************************
         // *********         Init The CDMW Time Service           *********
         // ***************************************************************** 
         {
            CDMW_DEBUG(InitUtils::Get_Logger(), "Init Time Service");
            // definition of Corbaname URL :
            //   Note : Time Service default port is set to 4883
               
      
        // initializes the CosClockServices.
        if (Cdmw::clock::CosClock::init (orb, argc, argv) == false)
        {
             CDMW_WARN(InitUtils::Get_Logger(),"It is not possible not initialize clock service!");
               
            }
                      
            }

         #endif // CDMW_TIME_SERVICE_ENABLED
      ]]>
   </xsl:if>
      
   <xsl:if test="$hasRecording">
      <![CDATA[

         // *****************************************************************
         // **********  register initial references for Recording ***********
         // *****************************************************************
         {
            CDMW_ASSERT(!CORBA::is_nil(local_NameService.in()));
            Cdmw::CommonSvcs::Naming::NamingInterface local_ni(local_NameService.in());

            unsigned long callTimeout_millisec = 0;
            try {
               std::string res = 
                  xml_data->getServiceAttributeValue("recording", "recorder-timeout"); 
               callTimeout_millisec = atoi(res.c_str());
            } catch (const Cdmw::BadParameterException& ex) {
              CDMW_DEBUG( InitUtils::Get_Logger(),
                         "recorder-timeout was not found. Call timeout set to unlimited." );
            }

            // Register initial reference for the recording daemon
            //
            CDMW_DEBUG(InitUtils::Get_Logger(), 
                       "CDMW_init: register initial reference of the recording daemon");
            CORBA::Object_var daemon_obj;
            try {
               daemon_obj = local_ni.resolve(CdmwRecording::REC_CONTAINER_MNGR_ID);
            } catch (const CosNaming::NamingContext::NotFound& nf) {
               std::ostringstream ostr;
               ostr << "Could not retrieve reference of the Recording Daemon."
                    << "'" << CdmwRecording::REC_CONTAINER_MNGR_ID << "' not found in the naming.";
               CDMW_FATAL(InitUtils::Get_Logger(), ostr.str());
               CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                            CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAM,
                                             CORBA::COMPLETED_NO), ostr.str());
            }
            if (CORBA::is_nil(daemon_obj.in())) {
               std::ostringstream ostr;
               ostr << "Nil Object Reference for '" << CdmwRecording::REC_CONTAINER_MNGR_ID << "'.";
               CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                            CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMNilObjectReference,
                                             CORBA::COMPLETED_NO), ostr.str());
            }
            try {
               orb->register_initial_reference(CdmwRecording::REC_CONTAINER_MNGR_ID, daemon_obj.in());
            } catch (const CORBA::ORB::InvalidName& )  {
               std::ostringstream ostr;
               ostr << "InvalidName => Cannot register '" << CdmwRecording::REC_CONTAINER_MNGR_ID 
                    <<  "'.";
               CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                            CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL, CORBA::COMPLETED_NO),
                            ostr.str());
            } catch (const CORBA::SystemException& ex) {
               CDMW_FATAL(InitUtils::Get_Logger(), 
                          "CDMW_init: register_initial_reference of " << 
                          CdmwRecording::REC_CONTAINER_MNGR_ID << " failed.");
               throw;
            }
            // 
            try {
               daemon_obj = local_ni.resolve(CdmwRecording::REC_DAEMON_CTRL_ID);
               if (CORBA::is_nil(daemon_obj.in())) {
                  std::ostringstream ostr;
                  ostr << "Nil Object Reference for '" << CdmwRecording::REC_DAEMON_CTRL_ID << "'.";
                  CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                               CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMNilObjectReference,
                                                CORBA::COMPLETED_NO), ostr.str());
               }
            } catch (const CosNaming::NamingContext::NotFound& nf) {
               std::ostringstream ostr;
               ostr << "Could not retrieve reference of the Recording Daemon."
                    << "'" << CdmwRecording::REC_DAEMON_CTRL_ID << "' not found in the naming.";
               CDMW_FATAL(InitUtils::Get_Logger(), ostr.str());
               CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                            CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAM,
                                             CORBA::COMPLETED_NO), ostr.str());
            }
            
            try {
               CdmwRecording::DaemonControl_var daemon = 
                  CdmwRecording::DaemonControl::_narrow(daemon_obj.in());
               
               CORBA::String_var url__ = daemon->recorder_url();
               std::string recording_server_url(url__.in());
               CDMW_DEBUG(InitUtils::Get_Logger(), 
                          "CDWM_init: recording server URL  = " << recording_server_url);
               if ( recording_server_url.empty() ) {
                  CDMW_WARN(InitUtils::Get_Logger(), 
                            "CDMW_init: No Recording server url defined! Thus register_initial_reference() is not perfomed for the Recording server."); 
               } else {
                  //
                  // Register initial reference for the recording server
                  //
                  CDMW_DEBUG(InitUtils::Get_Logger(), 
                             "CDMW_init: register initial reference of the recording server");
                  
                  CORBA::Object_var recording_server_obj = orb->string_to_object(recording_server_url.c_str());
                  if (CORBA::is_nil(recording_server_obj.in())) {
                     std::ostringstream ostr;
                     ostr << "Nil Object Reference for 'RecordingServer' url='" 
                          << recording_server_url << "'.";
                     CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                                  CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMNilObjectReference,
                                                   CORBA::COMPLETED_NO), ostr.str());
                  }
                  // Use SyncScope and relative roundtrip policies to achieve reliable oneways
                          
                  CORBA::Object_var capsule =          
                          set_recorder_policies(orb, 
                                                recording_server_obj.in(),
                                                callTimeout_millisec);
                  orb->register_initial_reference("RecordingServer", capsule.in());
               }
            } catch (const CORBA::ORB::InvalidName& )  {
               CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                            CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL, CORBA::COMPLETED_NO),
                            "InvalidName => Cannot register 'RecordingServer'.");
            } catch (const CORBA::SystemException& ex) {
               CDMW_FATAL(InitUtils::Get_Logger(), "CDMW_init: SystemException raised. register_initial_reference of RecordingServer not performed. "
                          << ex);
               throw;
            }
         }
      ]]>
   </xsl:if>

   <xsl:if test="$hasLifecycle">
      <![CDATA[
         // *****************************************************************
         // *********  Create CDMW Factories POA (persistent POA)  *********
         // ***************************************************************** 
         CDMW_DEBUG(InitUtils::Get_Logger(), "Create CDMW Factories POA");
         // Raises CORBA::SystemException
         PortableServer::POA_var factories_poa 
            = Cdmw::LifeCycle::InitUtils::create_cdmw_LifeCycle_POA(CDMW_rootPOA.in());

         // Create CDMW factories
        
         using namespace Cdmw::LifeCycle;
         CosNaming::NamingContext_var nc_root_context = CosNaming::NamingContext::_nil();
         try {
            nc_root_context
               = repository->resolve_root_context(CdmwNamingAndRepository::DEFAULT_ROOT_CONTEXT);
         } catch (const CdmwNamingAndRepository::Repository::NoRootContext& ) {
            std::string s (CdmwNamingAndRepository::DEFAULT_ROOT_CONTEXT);
            s += " is not a Root Context!";
            CDMW_WARN(InitUtils::Get_Logger(), s);
            CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                          CORBA::INTERNAL(Cdmw::OrbSupport::INTERNALCdmwRepositoryError,
                                          CORBA::COMPLETED_NO),
                          s );
         } catch (const CORBA::SystemException & ex) {
            CDMW_WARN(InitUtils::Get_Logger(), "CORBA::SystemException received: "
                      << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex));
            CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,
                          ex );
         }
         // May raise a CORBA::SystemException 
         Cdmw::CommonSvcs::Naming::NamingInterface root_context (nc_root_context.in());

      ]]>

      <xsl:if test="$hasHomeImpl='true'">
         using namespace Cdmw::CCM::Container;
         // Register Valuetype  factories
         Cdmw::CCM::CIF::InitUtils::register_valuetype_factories(orb);
         Cdmw::CCM::Container::InitUtils::register_valuetype_factories(orb);
      </xsl:if>

      <xsl:for-each select="$frameworkNodes">
         <xsl:variable name="frameworkIdref" select="@idref"/>

         <!--
            Get the matching node /cdmw/lifecycle-frameworks/lifecycle-framework.
         -->
         <xsl:variable name="lifecycleFrameworkNode" select="$lifecycleFrameworkNodes[@id = $frameworkIdref]"/>
         <xsl:variable name="isHomeImpl" select="boolean($lifecycleFrameworkNode/home-impl)"/>

         <!--
            Get the factory classname.
         -->
         <!--
         <xsl:variable name="factoryClassname" select="$lifecycleFrameworkNode/*/@classname"/>
         -->
         <xsl:variable name="factoryClassname">
            <xsl:choose>
               <xsl:when test="$isHomeImpl">
                  <xsl:call-template name="buildHomeImplClassname">
                     <xsl:with-param name="_homeImplNode" select="$lifecycleFrameworkNode/home-impl"/>
                  </xsl:call-template>
               </xsl:when>
               <xsl:otherwise>
                  <xsl:value-of select="$lifecycleFrameworkNode/*/@classname"/>
               </xsl:otherwise>
            </xsl:choose>
         </xsl:variable>

            
         <!--
            Format the class name for output.
         -->
         <xsl:variable name="cppFactoryClassname">
            <xsl:call-template name="replaceCharsInString">
               <xsl:with-param name="_stringIn" select="$factoryClassname"/>
               <xsl:with-param name="_charsIn" select="$xmlSep"/>
               <xsl:with-param name="_charsOut" select="$cppSep"/>
            </xsl:call-template>
         </xsl:variable>

         <!--
            Get the strategy.
         -->
         <xsl:variable name="strategy">
            <xsl:variable name="category" select="$lifecycleFrameworkNode/*/@category"/>
            <xsl:choose>
               <xsl:when test="boolean($category)">
                  <xsl:value-of select="$category"/>
               </xsl:when>
               <xsl:otherwise>
                  <xsl:value-of select="'USER_DEFINED'"/>
               </xsl:otherwise>
            </xsl:choose>
         </xsl:variable>

         <!--
            Register the home allocator.
         -->
         <xsl:if test="$isHomeImpl">
            // Register <xsl:value-of select="$cppFactoryClassname"/> HomeAllocator
            {
               typedef <![CDATA[HomeAllocator<]]><xsl:value-of select="$cppFactoryClassname"/><![CDATA[> CdmwHome_Allocator;]]>
               HomeAllocatorRegistry::Register("<xsl:value-of select="$factoryClassname"/>",
                                               <![CDATA[&CdmwHome_Allocator::TheAllocator);]]>
            }

            <xsl:if test="$hasFaultToleranceServer='yes'">
               <xsl:variable name="ftHomeClassname">
                  <xsl:call-template name="buildHomeImplClassname">
                        <xsl:with-param name="_homeImplNode" select="$lifecycleFrameworkNode/home-impl"/>
                        <xsl:with-param name="_prefix" select="'FT'"/>
                  </xsl:call-template>
               </xsl:variable>

               <xsl:variable name="cppFtFactoryClassname">
                  <xsl:call-template name="replaceCharsInString">
                     <xsl:with-param name="_stringIn" select="$ftHomeClassname"/>
                     <xsl:with-param name="_charsIn" select="$xmlSep"/>
                     <xsl:with-param name="_charsOut" select="$cppSep"/>
                  </xsl:call-template>
               </xsl:variable>

            // Register <xsl:value-of select="$cppFtFactoryClassname"/> HomeAllocator
            {
               typedef <![CDATA[HomeAllocator<]]><xsl:value-of select="$cppFtFactoryClassname"/><![CDATA[> CdmwHome_Allocator;]]>
               HomeAllocatorRegistry::Register("<xsl:value-of select="$ftHomeClassname"/>",
                                               <![CDATA[&CdmwHome_Allocator::TheAllocator);]]>
            }
            </xsl:if>
            
            <xsl:if test="$hasLoadBalancingServer='yes'">
               <xsl:variable name="lbHomeClassname">
                  <xsl:call-template name="buildHomeImplClassname">
                        <xsl:with-param name="_homeImplNode" select="$lifecycleFrameworkNode/home-impl"/>
                        <xsl:with-param name="_prefix" select="'LB'"/>
                  </xsl:call-template>
               </xsl:variable>

               <xsl:variable name="cppLbFactoryClassname">
                  <xsl:call-template name="replaceCharsInString">
                     <xsl:with-param name="_stringIn" select="$lbHomeClassname"/>
                     <xsl:with-param name="_charsIn" select="$xmlSep"/>
                     <xsl:with-param name="_charsOut" select="$cppSep"/>
                  </xsl:call-template>
               </xsl:variable>

            // Register <xsl:value-of select="$cppLbFactoryClassname"/> HomeAllocator
            {
               typedef <![CDATA[HomeAllocator<]]><xsl:value-of select="$cppLbFactoryClassname"/><![CDATA[> CdmwHome_Allocator;]]>
               HomeAllocatorRegistry::Register("<xsl:value-of select="$lbHomeClassname"/>",
                                               <![CDATA[&CdmwHome_Allocator::TheAllocator);]]>
            }
            </xsl:if>

         </xsl:if>

         <!--
            Strategy selection.
         -->
         <xsl:choose>
            <!--
               ENTITY.
            -->
            <xsl:when test="$strategy = 'ENTITY'">
               <xsl:choose>
                  <xsl:when test="$isHomeImpl">
                     <!--
                        TODO: Add component server code here!
                     -->
                  </xsl:when>
                  <xsl:otherwise>
                     // *****************************************************************
                     // ****** <xsl:value-of select="$cppFactoryClassname"/>  ******
                     // ***************************************************************** 
                     {
                        std::string full_name_domain_name;
                        std::string full_factory_name;
                        <![CDATA[
                           typedef std::auto_ptr<Cdmw::OrbSupport::POAThreadingPolicy> threading_auto_ptr;
                           threading_auto_ptr threading_policy(0);
                           // Cdmw::LifeCycle::POAThreadingPolicy * threading_policy = 0;
                           
                           int i = 0;
                        ]]>

                        <xsl:if test="@occurrences > 0">
                           try {
                              <![CDATA[full_name_domain_name =  xml_data->getFrameworkAttributeValue]]>
                                 ("<xsl:value-of select="$frameworkIdref"/>", i, "domain-name");
                              // TODO: If full_name_domain_name and factory_name_domain_name are empty use
                              // default name domain
                              <![CDATA[full_factory_name = xml_data->getFrameworkAttributeValue]]>
                                 ("<xsl:value-of select="$frameworkIdref"/>", i, "full-factory-name");
                      
                           <xsl:choose>
                              <xsl:when test="$lifecycleFrameworkNode/*/@threading = 'SERIALIZE'">
                                 threading_auto_ptr thrdptr(new Cdmw::OrbSupport::SingleThreadPolicy());
                                 threading_policy = thrdptr;
                                 // threading_policy = threading_auto_ptr(new SingleThreadPolicy());
                              </xsl:when>
                              <xsl:when test="$lifecycleFrameworkNode/*/@threading = 'MULTITHREAD'">
                                 std::string xmlThreadingPolicy("");
                                 std::string xmlThreadingAttribute("");
                                 <![CDATA[std::string xmlThreadingAttributeValue = xml_data->getFrameworkMultithreadAttribute]]>
                                    ("<xsl:value-of select="$frameworkIdref"/>", i, xmlThreadingPolicy, xmlThreadingAttribute);
                      
                                 if (!xmlThreadingPolicy.compare("serialize")) {
                                    threading_auto_ptr thrdptr(new Cdmw::OrbSupport::SingleThreadPolicy());
                                    threading_policy = thrdptr; 
                                    // threading_policy = threading_auto_ptr(new SingleThreadPolicy());
                                 } else {
                                    //threadin_policy shall not be NULL
                                    threading_auto_ptr thrdptr(new Cdmw::OrbSupport::ThreadPerConnectionPolicy());
                                    threading_policy = thrdptr; 
                                 }
                              </xsl:when>
                           </xsl:choose>
                           
                           <![CDATA[
                              } catch (const std::bad_alloc & ex) {
                                 CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                                               CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                                                CORBA::COMPLETED_NO),
                                               ex.what() );
                              } catch (const OutOfMemoryException & ex) {
                                 CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                                               CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                                                CORBA::COMPLETED_NO),
                                               ex.what() );
                              } catch (const Cdmw::BadParameterException & ex ) {
                                 CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                                               CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                                                CORBA::COMPLETED_NO),
                                               ex.what() );
                              } catch (const Cdmw::CdmwInit::XMLErrorException & ex) {
                                 CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                                               CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                                                 CORBA::COMPLETED_NO),
                                               ex.what() );
                              }
                           ]]>

                           typedef NamedObjectsFactory_initUtil Util;
                           typedef NamedObjectsFactoryAllocatorBase AllocatorType;

                           // Get the factory Allocator
                           // TODO: Use a user-provided name rather than a naming convention!
                           <![CDATA[AllocatorType & allocator =]]>
                              <xsl:value-of select="$cppFactoryClassname"/>_Allocator::TheAllocator;

                           try {
                              Util::setup_named_object_factory(allocator,
                                                               orb,           // ORB
                                                               factories_poa.in(),    // PARENT POA
                                                               repository.in(),
                                                               root_context,
                                                               full_name_domain_name,
                                                               full_factory_name,
                                                               "", // TODO:
                                                               *threading_policy,
                                                               argc, argv
                                 );
                           <![CDATA[
                              } catch (const Cdmw::OutOfMemoryException & ex) {
                                 std::string s ("Could not allocate memory for ");
                                 s += full_factory_name;
                                 s += "\n";
                                 s += ex.what();
                                 
                                 CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                                               CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                                                CORBA::COMPLETED_NO),
                                               s );
                              } catch (const Cdmw::BadParameterException & ex) {
                                 CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                                               CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMLifeCycleFrameworkInit,
                                                                CORBA::COMPLETED_NO),
                                               ex.what() );
                              } catch (const Cdmw::InternalErrorException & ex ) {
                                 CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                                               CORBA::INTERNAL(Cdmw::OrbSupport::INTERNALLifeCycleFrameworkError,
                                                               CORBA::COMPLETED_NO),
                                               ex.what() );
                              } catch (const CORBA::SystemException & ex) {
                                 CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,
                                               ex );
                              }
                           ]]>
                        </xsl:if>
                     }
                  </xsl:otherwise>
               </xsl:choose>
            </xsl:when>
            <!--
               UNMANAGED_ENTITY.
            -->
            <xsl:when test="$strategy = 'UNMANAGED_ENTITY'">
               // *****************************************************************
               // ****** <xsl:value-of select="$cppFactoryClassname"/>  ******
               // *****************************************************************
               {
                  std::string full_name_domain_name;
                  std::string full_factory_name;
                  <![CDATA[
                     typedef std::auto_ptr<Cdmw::OrbSupport::POAThreadingPolicy> threading_auto_ptr;
                     threading_auto_ptr threading_policy(0);
                     // Cdmw::LifeCycle::POAThreadingPolicy * threading_policy = 0;
            
                     int i = 0;
                  ]]>

                  <xsl:if test="@occurrences > 0">
                     try {
                        <![CDATA[full_name_domain_name = xml_data->getFrameworkAttributeValue]]>
                           ("<xsl:value-of select="$frameworkIdref"/>", i, "domain-name");
                        // TODO: If full_name_domain_name and factory_name_domain_name are empty use
                        // default name domain
                        <![CDATA[full_factory_name = xml_data->getFrameworkAttributeValue]]>
                           ("<xsl:value-of select="$frameworkIdref"/>", i, "full-factory-name");
                
                     <xsl:choose>
                        <xsl:when test="$lifecycleFrameworkNode/*/@threading = 'SERIALIZE'">
                           threading_auto_ptr thrdptr(new Cdmw::OrbSupport::SingleThreadPolicy());
                           threading_policy = thrdptr;
                           // threading_policy = threading_auto_ptr(new SingleThreadPolicy());
                        </xsl:when>
                        <xsl:when test="$lifecycleFrameworkNode/*/@threading = 'MULTITHREAD'">
                           std::string xmlThreadingPolicy("");
                           std::string xmlThreadingAttribute("");
                           <![CDATA[std::string xmlThreadingAttributeValue = xml_data->getFrameworkMultithreadAttribute]]>
                              ("<xsl:value-of select="$frameworkIdref"/>", i, xmlThreadingPolicy, xmlThreadingAttribute);
                
                           if (!xmlThreadingPolicy.compare("serialize")) {
                              threading_auto_ptr thrdptr(new Cdmw::OrbSupport::SingleThreadPolicy());
                              threading_policy = thrdptr;
                              // threading_policy = threading_auto_ptr(new SingleThreadPolicy());
                           } else {
                              //threadin_policy shall not be NULL
                              threading_auto_ptr thrdptr(new Cdmw::OrbSupport::ThreadPerConnectionPolicy());
                              threading_policy = thrdptr; 
                           }

                        </xsl:when>
                     </xsl:choose>

                     <![CDATA[
                        } catch (const std::bad_alloc & ex) {
                           CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                                         CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                                          CORBA::COMPLETED_NO),
                                         ex.what() );
                        } catch (const OutOfMemoryException & ex) {
                           CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                                         CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                                          CORBA::COMPLETED_NO),
                                         ex.what() );
                        } catch (const Cdmw::BadParameterException & ex ) {
                           CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                                         CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                                          CORBA::COMPLETED_NO),
                                         ex.what() );
                        } catch (const Cdmw::CdmwInit::XMLErrorException & ex) {
                           CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                                         CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                                          CORBA::COMPLETED_NO),
                                         ex.what() );
                        }
                     ]]>

                     typedef NamedObjectsFactory_initUtil Util;
                     typedef NamedObjectsFactoryAllocatorBase AllocatorType;

                     // Get the factory Allocator
                     // TODO: Use a user-provided name rather than a naming convention!
                     <![CDATA[AllocatorType & allocator =]]>
                        <xsl:value-of select="$cppFactoryClassname"/>_Allocator::TheAllocator;

                     try {
                        Util::setup_named_object_factory(allocator,
                                                         orb,           // ORB
                                                         factories_poa.in(),    // PARENT POA
                                                         repository.in(),
                                                         root_context,
                                                         full_name_domain_name,
                                                         full_factory_name,
                                                         "", // TODO:
                                                         *threading_policy,
                                                         argc, argv
                           );
                     <![CDATA[
                        } catch (const Cdmw::OutOfMemoryException & ex ) {
                           std::string s ("Could not allocate memory for ");
                           s += full_factory_name;
                           s += "\n";
                           s += ex.what();

                           CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                                         CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                                          CORBA::COMPLETED_NO),
                                         s );
                        } catch (const Cdmw::BadParameterException & ex) {
                           CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                                         CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMLifeCycleFrameworkInit,
                                                          CORBA::COMPLETED_NO),
                                         ex.what() );
                        } catch (const Cdmw::InternalErrorException & ex ){
                           CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                                         CORBA::INTERNAL(Cdmw::OrbSupport::INTERNALLifeCycleFrameworkError,
                                                         CORBA::COMPLETED_NO),
                                         ex.what() );
                        } catch (const CORBA::SystemException & ex ) {
                           CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,
                                         ex );
                        }
                     ]]>
                  </xsl:if>
               }
            </xsl:when>
            <!--
               SESSION.
            -->
            <xsl:when test="$strategy = 'SESSION'">
               <xsl:choose>
                  <xsl:when test="$isHomeImpl">
                     <!--
                        TODO: Add component server code here!
                     -->
                  </xsl:when>
                  <xsl:otherwise>
                     // *****************************************************************
                     // ****** <xsl:value-of select="$cppFactoryClassname"/>  ******
                     // *****************************************************************
                     {
                        std::string full_factory_name;
                        <![CDATA[
                           typedef std::auto_ptr<Cdmw::OrbSupport::POAThreadingPolicy> threading_auto_ptr;
                           threading_auto_ptr threading_policy(0);
                           // Cdmw::LifeCycle::POAThreadingPolicy * threading_policy = 0;
                          
                           int i = 0;
                        ]]>

                        <xsl:if test="@occurrences > 0">
                           try {
                              <![CDATA[full_factory_name = xml_data->getFrameworkAttributeValue]]>
                                 ("<xsl:value-of select="$frameworkIdref"/>", i, "full-factory-name");
                      
                           <xsl:choose>
                              <xsl:when test="$lifecycleFrameworkNode/*/@threading = 'SERIALIZE'">
                                 threading_auto_ptr thrdptr(new Cdmw::OrbSupport::SingleThreadPolicy());
                                 threading_policy = thrdptr;
                                 // threading_policy = threading_auto_ptr(new SingleThreadPolicy());
                              </xsl:when>
                              <xsl:when test="$lifecycleFrameworkNode/*/@threading = 'MULTITHREAD'">
                                 std::string xmlThreadingPolicy("");
                                 std::string xmlThreadingAttribute("");    
                                 <![CDATA[std::string xmlTreadingAttributeValue = xml_data->getFrameworkMultithreadAttribute]]>
                                    ("<xsl:value-of select="$frameworkIdref"/>", i, xmlThreadingPolicy, xmlThreadingAttribute);
                      
                                 if (!xmlThreadingPolicy.compare("serialize")) {
                                    threading_auto_ptr thrdptr(new Cdmw::OrbSupport::SingleThreadPolicy());
                                    threading_policy = thrdptr; 
                                    // threading_policy = threading_auto_ptr(new Cdmw::OrbSupport::SingleThreadPolicy());
                                 } else {
                                    //threadin_policy shall not be NULL
                                    threading_auto_ptr thrdptr(new Cdmw::OrbSupport::ThreadPerConnectionPolicy());
                                    threading_policy = thrdptr; 
                                 }

                              </xsl:when>
                           </xsl:choose>

                           <![CDATA[
                              } catch (const std::bad_alloc & ex) {
                                 CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                                               CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                                                CORBA::COMPLETED_NO),
                                               ex.what() );
                              } catch (const OutOfMemoryException & ex) {
                                 CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                                               CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                                                CORBA::COMPLETED_NO),
                                               ex.what() );
                              } catch (const Cdmw::BadParameterException & ex) {
                                 CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                                               CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                                                CORBA::COMPLETED_NO),
                                               ex.what() );       
                              } catch (const Cdmw::CdmwInit::XMLErrorException & ex ) {
                                 CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                                               CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                                                CORBA::COMPLETED_NO),
                                               ex.what() );
                              }
                           ]]>

                           typedef AnonymousObjectsFactory_initUtil Util;
                           typedef AnonymousObjectsFactoryAllocatorBase AllocatorType;

                           // Get the factory Allocator
                           // TODO: Use a user-provided name rather than a naming convention!
                           <![CDATA[AllocatorType & allocator =]]>
                              <xsl:value-of select="$cppFactoryClassname"/>_Allocator::TheAllocator;

                           try {
                              Util::setup_anonymous_object_factory(allocator,
                                                                   orb,           // ORB
                                                                   factories_poa.in(),    // PARENT POA
                                                                   repository.in(),
                                                                   root_context,
                                                                   full_factory_name,
                                                                   "", // TODO:
                                                                   *threading_policy,
                                                                   argc, argv
                                 );
                            <![CDATA[
                              } catch (const Cdmw::OutOfMemoryException & ex ) {
                                 std::string s ("Could not allocate memory for ");
                                 s += full_factory_name;
                                 s += "\n";
                                 s += ex.what();

                                 CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                                               CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                                                CORBA::COMPLETED_NO),
                                               s );
                              } catch (const Cdmw::BadParameterException & ex) {
                                 CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                                               CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMLifeCycleFrameworkInit,
                                                                CORBA::COMPLETED_NO),
                                               ex.what() );
                              } catch (const Cdmw::InternalErrorException & ex ) {
                                 CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                                               CORBA::INTERNAL(Cdmw::OrbSupport::INTERNALLifeCycleFrameworkError,
                                                               CORBA::COMPLETED_NO),
                                               ex.what() );
                              } catch (const CORBA::SystemException & ex) {
                                 CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,
                                               ex );
                              }
                            ]]>
                        </xsl:if>
                     }
                  </xsl:otherwise>
               </xsl:choose>
            </xsl:when>
            <!--
               SERVICE.
            -->
            <xsl:when test="$strategy = 'SERVICE'">
               <xsl:choose>
                  <xsl:when test="$isHomeImpl">
                     <!--
                        TODO: Add component server code here!
                     -->
                  </xsl:when>
                  <xsl:otherwise>
                     // *****************************************************************
                     // ****** <xsl:value-of select="$cppFactoryClassname"/>  ******
                     // *****************************************************************
                     {
                        std::string full_factory_name;
                        <![CDATA[
                           typedef std::auto_ptr<Cdmw::OrbSupport::POAThreadingPolicy> threading_auto_ptr;
                           threading_auto_ptr threading_policy(0);
                           // Cdmw::LifeCycle::POAThreadingPolicy * threading_policy = 0;
                              
                           int i = 0;
                        ]]>

                        <xsl:if test="@occurrences > 0">
                           try {
                              <![CDATA[full_factory_name = xml_data->getFrameworkAttributeValue]]>
                                 ("<xsl:value-of select="$frameworkIdref"/>", i, "full-factory-name");

                           <xsl:choose>
                              <xsl:when test="$lifecycleFrameworkNode/*/@threading = 'SERIALIZE'">
                                 threading_auto_ptr thrdptr(new Cdmw::OrbSupport::SingleThreadPolicy());
                                 threading_policy = thrdptr;
                                 // threading_policy = threading_auto_ptr(new SingleThreadPolicy());
                              </xsl:when>
                              <xsl:when test="$lifecycleFrameworkNode/*/@threading = 'MULTITHREAD'">
                                 std::string xmlThreadingPolicy("");
                                 std::string xmlThreadingAttribute("");
                                 <![CDATA[std::string xmlThreadingAttributeValue = xml_data->getFrameworkMultithreadAttribute]]>
                                    ("<xsl:value-of select="$frameworkIdref"/>", i, xmlThreadingPolicy, xmlThreadingAttribute);
                      
                                 if (!xmlThreadingPolicy.compare("serialize")) {
                                    threading_auto_ptr thrdptr(new Cdmw::OrbSupport::SingleThreadPolicy());
                                    threading_policy = thrdptr; 
                                    // threading_policy = threading_auto_ptr(new SingleThreadPolicy());
                                 } else {
                                    //threadin_policy shall not be NULL
                                    threading_auto_ptr thrdptr(new Cdmw::OrbSupport::ThreadPerConnectionPolicy());
                                    threading_policy = thrdptr; 
                                 }

                              </xsl:when>
                           </xsl:choose>

                           <![CDATA[
                              } catch (const std::bad_alloc & ex) {
                                 CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                                               CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                                                CORBA::COMPLETED_NO),
                                               ex.what() );
                              } catch (const OutOfMemoryException & ex) {
                                 CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                                               CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                                                CORBA::COMPLETED_NO),
                                               ex.what() );
                              } catch (const Cdmw::BadParameterException & ex ) {
                                 CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                                               CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                                                CORBA::COMPLETED_NO),
                                               ex.what() );
                              } catch (const Cdmw::CdmwInit::XMLErrorException & ex) {
                                 CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                                               CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                                                CORBA::COMPLETED_NO),
                                               ex.what() );
                              }
                           ]]>

                           typedef AnonymousObjectsFactory_initUtil Util;
                           typedef AnonymousObjectsFactoryAllocatorBase AllocatorType;

                           // Get the factory Allocator
                           // TODO: Use a user-provided name rather than a naming convention!
                           <![CDATA[AllocatorType & allocator =]]>
                              <xsl:value-of select="$cppFactoryClassname"/>_Allocator::TheAllocator;

                           try {
                              Util::setup_anonymous_object_factory(allocator,
                                                                   orb,           // ORB
                                                                   factories_poa.in(),    // PARENT POA
                                                                   repository.in(),
                                                                   root_context,
                                                                   full_factory_name,
                                                                   "", // TODO:
                                                                   * threading_policy,
                                                                   argc, argv
                                 );
                           <![CDATA[
                              } catch (const Cdmw::OutOfMemoryException & ex ) {
                                 std::string s ("Could not allocate memory for ");
                                 s += full_factory_name;
                                 s += "\n";
                                 s += ex.what();

                                 CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                                               CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                                                CORBA::COMPLETED_NO),
                                               s );
                              } catch (const Cdmw::BadParameterException & ex) {
                                 CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                                               CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMLifeCycleFrameworkInit,
                                                                CORBA::COMPLETED_NO),
                                               ex.what() );
                              } catch (const Cdmw::InternalErrorException & ex ) {
                                 CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                                               CORBA::INTERNAL(Cdmw::OrbSupport::INTERNALLifeCycleFrameworkError,
                                                               CORBA::COMPLETED_NO),
                                               ex.what() );
                              } catch (const CORBA::SystemException & ex ) {
                                 CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,
                                               ex );
                              }
                           ]]>
                        </xsl:if>
                     }
                  </xsl:otherwise>
               </xsl:choose>
            </xsl:when>
            <xsl:when test="$strategy = 'PROCESS'">
               <!--
                  TODO: Add component server code here!
               -->
            </xsl:when>
            <!--
               USER_DEFINED.
            -->
            <xsl:when test="$strategy = 'USER_DEFINED'">
               <xsl:variable name="isNamed" select="$factoryNodes[@idl-interface = $lifecycleFrameworkNode/@factory-idref]/@kind = 'NAMED'"/>
               <xsl:variable name="namedString">
                  <xsl:choose>
                     <xsl:when test="$isNamed">
                        <xsl:text>named</xsl:text>
                     </xsl:when>
                     <xsl:otherwise>
                        <xsl:text>anonymous</xsl:text>
                     </xsl:otherwise>
                  </xsl:choose>
               </xsl:variable>
               <xsl:variable name="cppFactoryAllocatorClassname">
                  <xsl:call-template name="replaceCharsInString">
                     <xsl:with-param name="_stringIn" select="$lifecycleFrameworkNode/user-factory-impl/factory-allocator/@classname"/>
                     <xsl:with-param name="_charsIn" select="$xmlSep"/>
                     <xsl:with-param name="_charsOut" select="$cppSep"/>
                  </xsl:call-template>
               </xsl:variable>

               // *****************************************************************
               // ******                User defined factory                 ******
               // *****************************************************************
               {
                  std::string full_factory_name;
                  <xsl:if test="$isNamed">std::string full_name_domain_name;</xsl:if>
                  <![CDATA[
                     typedef std::auto_ptr<Cdmw::OrbSupport::POAThreadingPolicy> threading_auto_ptr;
                     threading_auto_ptr threading_policy(0);
                     // Cdmw::LifeCycle::POAThreadingPolicy * threading_policy = 0;
                   
                     int i = 0;
                  ]]>

                  <xsl:if test="@occurrences > 0">
                     try {
                     <xsl:choose>
                        <xsl:when test="$isNamed">
                           <![CDATA[full_name_domain_name =  xml_data->getFrameworkAttributeValue]]>
                              ("<xsl:value-of select="$frameworkIdref"/>", i, "domain-name");
                           // TODO: If full_name_domain_name and factory_name_domain_name are empty use
                           // default name domain
                           <![CDATA[full_factory_name = xml_data->getFrameworkAttributeValue]]>
                              ("<xsl:value-of select="$frameworkIdref"/>", i, "full-factory-name");
                        </xsl:when>
                        <xsl:otherwise>
                           <![CDATA[full_factory_name = xml_data->getFrameworkAttributeValue]]>
                              ("<xsl:value-of select="$frameworkIdref"/>", i, "full-factory-name");
                        </xsl:otherwise>
                     </xsl:choose>

                     <xsl:choose>
                        <xsl:when test="$lifecycleFrameworkNode/*/@threading = 'SERIALIZE'">
                           threading_auto_ptr thrdptr(new Cdmw::OrbSupport::SingleThreadPolicy());
                           threading_policy = thrdptr;
                           // threading_policy = threading_auto_ptr(new Cdmw::OrbSupport::SingleThreadPolicy());
                        </xsl:when>
                        <xsl:when test="$lifecycleFrameworkNode/*/@threading = 'MULTITHREAD'">
                           std::string xmlThreadingPolicy("");
                           std::string xmlThreadingAttribute("");
                           <![CDATA[std::string xmlThreadingAttributeValue = xml_data->getFrameworkMultithreadAttribute]]>
                              ("<xsl:value-of select="$frameworkIdref"/>", i, xmlThreadingPolicy, xmlThreadingAttribute);
                
                           if (!xmlThreadingPolicy.compare("serialize")) {
                              threading_auto_ptr thrdptr(new Cdmw::OrbSupport::SingleThreadPolicy());
                              threading_policy = thrdptr; 
                              // threading_policy = threading_auto_ptr(new Cdmw::OrbSupport::SingleThreadPolicy());
                           } else {
                              //threadin_policy shall not be NULL
                              threading_auto_ptr thrdptr(new Cdmw::OrbSupport::ThreadPerConnectionPolicy());
                              threading_policy = thrdptr; 
                           }

                        </xsl:when>
                     </xsl:choose>

                     <![CDATA[
                        } catch (const std::bad_alloc & ex) {
                           CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                                         CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                                          CORBA::COMPLETED_NO),
                                         ex.what() );
                        } catch (const OutOfMemoryException & ex) {
                           CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                                         CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                                          CORBA::COMPLETED_NO),
                                         ex.what() );
                        } catch (const Cdmw::BadParameterException & ex ) {
                           CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                                         CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                                          CORBA::COMPLETED_NO),
                                         ex.what() );
                        } catch (const Cdmw::CdmwInit::XMLErrorException & ex) {
                           CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                                         CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                                          CORBA::COMPLETED_NO),
                                         ex.what() );
                        }
                     ]]>

                     <xsl:choose>
                        <xsl:when test="$isNamed">
                           typedef NamedObjectsFactory_initUtil Util;
                           typedef NamedObjectsFactoryAllocatorBase AllocatorType;
                        </xsl:when>
                        <xsl:otherwise>
                           typedef AnonymousObjectsFactory_initUtil Util;
                           typedef AnonymousObjectsFactoryAllocatorBase AllocatorType;
                        </xsl:otherwise>
                     </xsl:choose>

                     // Get the factory Allocator
                     // TODO: Use a user-provided name rather than a naming convention!
                     <![CDATA[AllocatorType & allocator =]]>
                        <xsl:value-of select="$cppFactoryAllocatorClassname"/>::TheAllocator;

                     try {
                        Util::setup_<xsl:value-of select="$namedString"/>_object_factory(allocator,
                                                             orb,           // ORB
                                                             factories_poa.in(),    // PARENT POA
                                                             repository.in(),
                                                             root_context,
                                                             <xsl:if test="$isNamed">full_name_domain_name,</xsl:if>
                                                             full_factory_name,
                                                             "", // TODO:
                                                             *threading_policy,
                                                             argc, argv
                           );
                     <![CDATA[
                        } catch (const Cdmw::OutOfMemoryException & ex ) {
                           std::string s ("Could not allocate memory for ");
                           s += full_factory_name;
                           s += "\n";
                           s += ex.what();

                           CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                                         CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                                          CORBA::COMPLETED_NO),
                                         s );
                        } catch (const Cdmw::BadParameterException & ex) {
                           CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                                         CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMLifeCycleFrameworkInit,
                                                          CORBA::COMPLETED_NO),
                                         ex.what() );
                        } catch (const Cdmw::InternalErrorException & ex ) {
                           CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                                         CORBA::INTERNAL(Cdmw::OrbSupport::INTERNALLifeCycleFrameworkError,
                                                         CORBA::COMPLETED_NO),
                                         ex.what() );
                        } catch (const CORBA::SystemException & ex ) {
                           CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,
                                         ex );
                        }
                     ]]>
                  </xsl:if>
               }
            </xsl:when>
         </xsl:choose>
      </xsl:for-each>

      <xsl:if test="$hasHomeImpl='true'">
         <![CDATA[  
         // Default Event Channel Factory
         CdmwEvent::EventChannelFactory_var event_channel_factory 
             = CdmwEvent::EventChannelFactory::_nil();
         ]]>

         <xsl:if test="$hasEventPorts = 'true'">
            <![CDATA[
            // *****************************************************************
            // ******        Create a local Event Channel Factory         ******
            // *****************************************************************
            CDMW_DEBUG(InitUtils::Get_Logger(), "Create a local Event Channel Factory");
            
            // the factory name will be: <application_name>.<processname>.EventChannelFactory
            std::string tmp_ec_factory_name = 
                application_name + "." + process_name + ".EventChannelFactory";
            CosNaming::Name ec_factory_name;
            ec_factory_name.length(1);
            ec_factory_name[0].id = tmp_ec_factory_name.c_str();
            std::string event_channel_factory_name = 
                Cdmw::CommonSvcs::Naming::NamingInterface::to_string(ec_factory_name);

            {
                std::string full_name_domain_name;
                full_name_domain_name = Cdmw::Common::Locations::CDMW_EVENT_SERVICE_NAME_DOMAIN;
                full_name_domain_name += "/EVENT_CHANNELS";
                std::string full_factory_name;
                full_factory_name = Cdmw::Common::Locations::CDMW_EVENT_SERVICE_NAME_DOMAIN;
                full_factory_name += "/FACTORIES/";
                full_factory_name += event_channel_factory_name;
                
                // TAO doesn't support thread-pool policy with a persistent POA
                std::auto_ptr<Cdmw::OrbSupport::POAThreadingPolicy> threading_policy(
                    new Cdmw::OrbSupport::ThreadPerConnectionPolicy());
            
                // Get the factory Allocator
                NamedObjectsFactoryAllocatorBase & allocator =
                    Cdmw::Event::EventChannelFactory_impl_Allocator::TheAllocator;
            
                try {
                    NamedObjectsFactory_initUtil::setup_named_object_factory(
                        allocator,
                        orb,
                        factories_poa.in(),
                        repository.in(),
                        root_context,
                        full_name_domain_name,
                        full_factory_name,
                        "",
                        *threading_policy,
                        argc, argv
                        );
                
                } catch (const Cdmw::OutOfMemoryException & ex ) {
                    std::string s ("Could not allocate memory for ");
                    s += full_factory_name;
                    s += "\n";
                    s += ex.what();
                    CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                                 CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                                  CORBA::COMPLETED_NO),
                                 s );
                } catch (const Cdmw::BadParameterException & ex) {
                    CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                                 CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMLifeCycleFrameworkInit,
                                                  CORBA::COMPLETED_NO),
                                 ex.what() );
                } catch (const Cdmw::InternalErrorException & ex ) {
                    CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                                 CORBA::INTERNAL(Cdmw::OrbSupport::INTERNALLifeCycleFrameworkError,
                                                 CORBA::COMPLETED_NO),
                                 ex.what() );
                } catch (const CORBA::SystemException & ex ) {
                    CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,
                                 ex );
                }
            }
            
            try {
                // Retreive reference to the local Event Channel factory 
                using namespace Cdmw::NamingAndRepository;             
                using namespace Cdmw::CommonSvcs::Naming;
                using namespace Cdmw::Common;

                std::string evt_factories_nd(Locations::CDMW_EVENT_SERVICE_NAME_DOMAIN);
                evt_factories_nd += "/FACTORIES";

                NamingInterface evt_ni = 
                    RepositoryInterface::get_domain_naming_interface(evt_factories_nd);
           
                typedef NamingUtil<CdmwEvent::EventChannelFactory> Util;
                event_channel_factory = Util::resolve_name(evt_ni, event_channel_factory_name);
                
            } catch (const CdmwNamingAndRepository::NoNameDomain &) {
               CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                             CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMNameDomainNotFound,
                                              CORBA::COMPLETED_NO),
                             std::string(Cdmw::Common::Locations::CDMW_EVENT_SERVICE_NAME_DOMAIN) +
                             "/FACTORIES is not a NameDomain!" );
            } catch (const CdmwNamingAndRepository::InvalidName &) {
                CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                             CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL,
                                             CORBA::COMPLETED_NO) );
            } catch (const CosNaming::NamingContext::NotFound &) {
                CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                             CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL,
                                             CORBA::COMPLETED_NO) );
            } catch (const CosNaming::NamingContext::CannotProceed &) {
                CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                             CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL,
                                             CORBA::COMPLETED_NO) );
            } catch (const CosNaming::NamingContext::InvalidName &) {
                CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                             CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL,
                                             CORBA::COMPLETED_NO) );
            } catch (const Cdmw::Common::TypeMismatchException &) {
                CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                             CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL,
                                             CORBA::COMPLETED_NO) );
            } catch (const CORBA::SystemException & ex) {
                CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,ex );
            }
            ]]>
         </xsl:if>

         <![CDATA[
         // Create an Object registration instance
         // get the admin root context
         CDMW_DEBUG(InitUtils::Get_Logger(), "Get the admin root context");
         CdmwNamingAndRepository::Repository_var repository_ref 
             = Cdmw::NamingAndRepository::RepositoryInterface::get_repository();
         CosNaming::NamingContext_var admin_root_ctx 
             = repository_ref->resolve_root_context(
                     Cdmw::Common::Locations::ADMIN_ROOT_CONTEXT_ID);
         Cdmw::CommonSvcs::Naming::NamingInterface ni(admin_root_ctx);
                
         // bind the new context where component shall be registered
         CosNaming::Name component_name_path;
         component_name_path.length(5);
         component_name_path[0].id 
             = CORBA::string_dup(Cdmw::OsSupport::OS::get_hostname().c_str());
         component_name_path[1].id 
             = CORBA::string_dup(application_name.c_str());
         component_name_path[2].id 
             = CORBA::string_dup(process_name.c_str());
         component_name_path[3].id 
             = CORBA::string_dup("CCM");
         component_name_path[4].id 
             = CORBA::string_dup("Components");
        
         CDMW_DEBUG(InitUtils::Get_Logger(), "Bind component context");
         std::string component_path
             = ni.to_string(component_name_path);
         Cdmw::CommonSvcs::Naming::NamingInterface component_ni 
             = ni.new_context();
         ni.rebind_context(component_path, component_ni, true);

         CDMW_DEBUG(InitUtils::Get_Logger(), "Create object_registration");
         CosNaming::NamingContext_var nc = component_ni.name_context();
         CdmwLifeCycle::ObjectRegistration_var  object_registration
             = new Cdmw::LifeCycle::DefaultObjectRegistration(nc.in());

         // Register Managed component naming context in the orb
         // to make it available 
         orb->register_initial_reference("ManagedComponentNamingContext", nc.in());


         // Create the ComponentServer
         CDMW_DEBUG(InitUtils::Get_Logger(), "Create the ComponentServer");
         try {

             CdmwCcmComponentServer::ComponentServer_var the_component_server =
                 Cdmw::CCM::ComponentServer::InitUtils::create_component_server(orb,
                                                                                 CDMW_rootPOA.in(),
                                                                                 event_channel_factory.in(),
                                                                                 object_registration.in(),
                                                                                 process_name,
                                                                                 application_name,
                                                                                 argc,argv);
             // Register it as Cdmw::Common::Locations::ASSEMBLY_AND_DEPLOYMENT_NAME_DOMAIN/
             //      process_name@application_name.ComponentServer
             std::string id = process_name + std::string("@") + application_name;
             std::string kind("ComponentServer");
             CosNaming::Name name(1);
             name.length(1);
             name[0].id   = id.c_str();
             name[0].kind = kind.c_str();

             using namespace Cdmw::NamingAndRepository;             
             using namespace Cdmw::CommonSvcs::Naming;
             using namespace Cdmw::Common;

             std::string key = NamingInterface::to_string(name);

             CdmwNamingAndRepository::NameDomain_var domain =
                 RepositoryInterface::get_domain(Locations::ASSEMBLY_AND_DEPLOYMENT_NAME_DOMAIN);

             try {
                 domain->register_new_factory(key.c_str(),the_component_server.in());
             } catch (const CdmwNamingAndRepository::NameDomain::AlreadyExists &) {
                 // Re-register it again.
                 domain->release_name(key.c_str());
                 domain->register_new_factory(key.c_str(),the_component_server.in());
             } 
         } catch (const CdmwNamingAndRepository::NoNameDomain &) {
            CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                          CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMNameDomainNotFound,
                                           CORBA::COMPLETED_NO),
                          std::string(Cdmw::Common::Locations::ASSEMBLY_AND_DEPLOYMENT_NAME_DOMAIN) +
                          " is not a NameDomain!" );
         } catch (const CdmwNamingAndRepository::InvalidName &) {
             CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                          CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL,
                                          CORBA::COMPLETED_NO) );
         } catch (const CdmwNamingAndRepository::NameDomain::NotRegistered &) {
             CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                          CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL,
                                          CORBA::COMPLETED_NO) );
         } catch (const CORBA::SystemException & ex) {
             CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,ex );
         }
         ]]>
      </xsl:if>
   </xsl:if>

   <![CDATA[

         // Discard requests until run order comes from the platform management
         poaManager->discard_requests(false);

         CDMW_INFO(InitUtils::Get_Logger(), "Acknowledge creation of process "
                   << application_name << "/" << process_name);
         Cdmw::PlatformMngt::PlatformInterface::Acknowledge_creation(process.in());
     
      } catch (const Cdmw::OrbSupport::CORBASystemExceptionWrapper & ex) {
          CDMW_FATAL(InitUtils::Get_Logger(), 
                     "CORBA Exception during CDMW_init: "
                     << ex); 
          ex.raise_SystemException();
      } catch (const CORBA::SystemException & ex) {  
          CDMW_FATAL(InitUtils::Get_Logger(),  
                     "CORBA SystemException during CDMW_init: "
                     << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex));
          throw;
      }

   }

      // CDMW Factory objects will be deactivated at ORB shutdown.
      void
      CDMW_cleanup(CORBA::ORB_ptr orb)
         throw ()
      {
         CDMW_INFO(InitUtils::Get_Logger(), "CDMW_cleanup");
         CDMW_ASSERT(!CORBA::is_nil(orb));
         
   ]]>
   
      <xsl:if test="$hasHomeImpl='true'">
         using namespace Cdmw::CCM::Container;

         // We should unbind "processname"/CCM/Components from AdminRootContext
         // here. But as ORB has just been shutdown in on_stop() operation,
         // no outgoing request are allowed (BAD_INV_ORDER or COMM_FAILURE received)
             
         // Unregister HomeAllocators
         {
            <xsl:for-each select="$frameworkNodes">
               <xsl:variable name="frameworkIdref" select="@idref"/>

               <!--
                  Get the matching node /cdmw/lifecycle-frameworks/lifecycle-framework.
               -->
               <xsl:variable name="lifecycleFrameworkNode" select="$lifecycleFrameworkNodes[@id = $frameworkIdref]"/>
               <xsl:variable name="isHomeImpl" select="boolean($lifecycleFrameworkNode/home-impl)"/>

               <xsl:if test="$isHomeImpl">
                  <xsl:variable name="factoryClassname">
                     <xsl:call-template name="buildHomeImplClassname">
                        <xsl:with-param name="_homeImplNode" select="$lifecycleFrameworkNode/home-impl"/>
                     </xsl:call-template>
                  </xsl:variable>

                  HomeAllocatorRegistry::Unregister("<xsl:value-of select="$factoryClassname"/>");
                  <xsl:if test="$hasFaultToleranceServer='yes'">
                     <xsl:variable name="ftHomeClassname">
                        <xsl:call-template name="buildHomeImplClassname">
                           <xsl:with-param name="_homeImplNode" select="$lifecycleFrameworkNode/home-impl"/>
                           <xsl:with-param name="_prefix" select="'FT'"/>
                        </xsl:call-template>
                     </xsl:variable>
                  HomeAllocatorRegistry::Unregister("<xsl:value-of select="$ftHomeClassname"/>");
                  </xsl:if>
                  <xsl:if test="$hasLoadBalancingServer='yes'">
                     <xsl:variable name="lbHomeClassname">
                        <xsl:call-template name="buildHomeImplClassname">
                           <xsl:with-param name="_homeImplNode" select="$lifecycleFrameworkNode/home-impl"/>
                           <xsl:with-param name="_prefix" select="'LB'"/>
                        </xsl:call-template>
                     </xsl:variable>
                  HomeAllocatorRegistry::Unregister("<xsl:value-of select="$lbHomeClassname"/>");
                  </xsl:if>

               </xsl:if>
            </xsl:for-each>
         }

         // Unregister Valuetype  factories
         Cdmw::CCM::CIF::InitUtils::unregister_valuetype_factories(orb);
         Cdmw::CCM::Container::InitUtils::unregister_valuetype_factories(orb);
     </xsl:if>
   
     <xsl:if test="boolean($hasTrace) and ($hasBufferingEnabled='yes')">
     <![CDATA[
         #ifdef CDMW_TRACES_ENABLED

         // Free all allocated memory and stop the trace service
         try {
            Cdmw::Trace::FlushAreaMngr::cleanup();
         } catch (const Cdmw::InternalErrorException & ) {
            // Do nothing (ignore)
            // TODO: Check if exception has been removed from cleanup() specification.
         }
         
         #endif // CDMW_TRACES_ENABLED

     ]]>
     </xsl:if>
     <xsl:if test="$hasTime">
     <![CDATA[
         #ifdef CDMW_TIME_SERVICE_ENABLED

         Cdmw::clock::CosClock::close();
 
         #endif //CDMW_TIME_SERVICE_ENABLED

     ]]>
     </xsl:if>
     

     <xsl:if test="$hasPerformance">
     <![CDATA[
         #ifdef CDMW_PERFORMANCE_ENABLED
         
         // Free all allocated memory and stop the performance service
         try {
            Cdmw::Performance::PerformanceMngr::cleanup();
         } catch (const Cdmw::BadParameterException & ) {
            // Do nothing (ignore)
              // TODO: Add trace logs here
         }

         #endif // CDMW_PERFORMANCE_ENABLED

      
     ]]>
     </xsl:if>
     
   <![CDATA[

         // Release all static object references held by RepositoryInterface
         Cdmw::NamingAndRepository::RepositoryInterface::finish();
         
         // Release all static object references held by PlatformInterface
         Cdmw::PlatformMngt::PlatformInterface::Cleanup();
 
     ]]>
     
     <xsl:if test="boolean($hasTrace) and not($hasBufferingEnabled='yes')">

   <![CDATA[

         // Release LogCollectorDelegate
         {
            using namespace Cdmw::Logging;
            LogManagerDelegate * d__ = LogManager::Instance()->get_alternate_delegate();
            if (d__) {
               LogManagerCollectorDelegate * delegate__ = dynamic_cast<LogManagerCollectorDelegate*>(d__);
               CDMW_ASSERT(delegate__!=0);
               (void) LogManager::Instance()->unregister_alternate_delegate(d__);
               delete d__;
            }
         }
     ]]>
     </xsl:if>
   <![CDATA[

   
         try {
             // Cleanup XML library
             Cdmw::CdmwInit::InitUtils::cleanup_xml_library();
         } catch (const Cdmw::OrbSupport::CORBASystemExceptionWrapper& ex) {
             CDMW_WARN(InitUtils::Get_Logger(), 
                       "XML library cleanup: " << ex);
         }     
      }

     ]]>

   <xsl:call-template name="closeNamespace">
      <xsl:with-param name="_scope" select="$cdmwinitNamespace"/>
      <xsl:with-param name="_lastTokenIsNamespace" select="true()"/>
   </xsl:call-template>

   <!--
      The code generation ends here.
   -->

</xsl:template> <!-- end of template cdmw_interface_cpp.content -->



</xsl:stylesheet>


