/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "ConfAndPlug/cdmwinit/CdmwProcess_impl.hpp"
#include "ConfAndPlug/cdmwinit/InitUtils.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/orbsupport/RefCountLocalObject.hpp"
#include "Foundation/osthreads/Mutex.hpp"
#include "Foundation/osthreads/Condition.hpp"

#include "SystemMngt/platforminterface/PlatformInterface.hpp"
#include "Foundation/idllib/FTPullMonitorable.skel.hpp"
#include "Foundation/idllib/CdmwFTLocalPullMonitorable.skel.hpp"

#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/FilterMngr.hpp>




// namespace 
// {

//  no more used due to PCR 1857   
// class ProcessStopHandler : public Cdmw::OsSupport::SignalHandler
// {
// public:
//     ProcessStopHandler(CORBA::ORB_ptr orb)
//         : m_orb(CORBA::ORB::_duplicate(orb))
//     {
    
    
//     }
    
//     virtual ~ProcessStopHandler() 
//     {
//     }
    
//     int handle_signal(int sig) throw()
//     {
//         int exit_code = 0;
    
//         if (!CORBA::is_nil(m_orb.in())) {
//             try {
//                 m_orb->shutdown(true); 
//             } catch (const CORBA::SystemException& e) {
//                   std::cerr<<"ProcessStopHandler : ORB shutdown error "<<Cdmw::OrbSupport::OrbSupport::exception_to_string(e)<<std::endl;
//                   exit_code = 1;
//             }
//         }
            
//         Cdmw::OsSupport::OS::sleep(5000);

//         ::exit(exit_code);
        
//         return 0;
//     };
    
// private :
//     CORBA::ORB_var  m_orb;
    
// };

        
// }


// Anonymous namespace
namespace {
    
    /**
     *Purpose: Default LocalPullMonitorable object
     *<p> 
     *
     *
     */
    class DefaultLocalPullMonitorable_impl : virtual public CdmwFT::LocalPullMonitorable,
                                        virtual public Cdmw::OrbSupport::RefCountLocalObject
    {
    public:
        /**
         * Purpose:
         * <p> Constructor
         *
         */ 
        DefaultLocalPullMonitorable_impl()
            throw(CORBA::SystemException)
        {
        }
        
        /**
         * Purpose:
         * <p> Destructor
         * 
         */ 
        virtual 
        ~DefaultLocalPullMonitorable_impl()
            throw()
        {
        }

        /**
         * Purpose:
         * <p> to call to check if object is still living
         * 
         *@exception CORBA::SystemException
         *
         */ 
        virtual CORBA::Boolean is_alive ()
            throw(CORBA::SystemException)
        {
            return true;
        }
        
    private:
        
        /**
         * Purpose:
         * <p>  Copy constructor
         * 
         */ 
        DefaultLocalPullMonitorable_impl (const DefaultLocalPullMonitorable_impl& rhs)
            throw();
        
        /**
         * Purpose:
         * <p> Assignment operator
         * 
         */ 
        DefaultLocalPullMonitorable_impl&
        operator=(const DefaultLocalPullMonitorable_impl& rhs)
            throw();
        
    }; // End class DefaultLocalPullMonitorable_impl



    /**
     *Purpose: Default PullMonitorable object
     *<p> 
     *
     *
     */
    class DefaultPullMonitorable_impl : virtual public POA_FT::PullMonitorable,
                                        virtual public Cdmw::OrbSupport::RefCountServantBase
    {
    public:
        /**
         * Purpose:
         * <p> Constructor
         *
         */ 
        DefaultPullMonitorable_impl(PortableServer::POA_ptr                  poa,
                                    Cdmw::CdmwInit::ProcessControl*          ctrl,
                                    CdmwFT::LocalPullMonitorable_ptr         lpm)
            throw(CORBA::SystemException)
            : m_process_ctrl(ctrl),
              m_poa(PortableServer::POA::_duplicate(poa))
        {
            CDMW_ASSERT(!CORBA::is_nil(poa));
            CDMW_ASSERT(ctrl != 0);
            m_process_ctrl->_add_ref();

            if (CORBA::is_nil(lpm))
                m_local_pull_monitorable = new DefaultLocalPullMonitorable_impl();
            else
                m_local_pull_monitorable = CdmwFT::LocalPullMonitorable::_duplicate(lpm);
        }
        
        /**
         * Purpose:
         * <p> Destructor
         * 
         */ 
        virtual 
        ~DefaultPullMonitorable_impl()
            throw()
        {
        }

        virtual PortableServer::POA_ptr _default_POA() {
            return PortableServer::POA::_duplicate(m_poa.in());
        }
        
        /**
         * Purpose:
         * <p> to call to check if object is still living
         * 
         *@exception CORBA::SystemException
         *
         */ 
        virtual CORBA::Boolean is_alive ()
            throw(CORBA::SystemException)
        {
            bool alive = m_local_pull_monitorable->is_alive();

            if (alive)
            {
                alive = m_process_ctrl->is_alive();
            }
            
            return (alive ? 1:0);
        }
        
    private:
        
        /**
         * Purpose:
         * <p>  Copy constructor
         * 
         */ 
        DefaultPullMonitorable_impl (const DefaultPullMonitorable_impl& rhs)
            throw();
        
        /**
         * Purpose:
         * <p> Assignment operator
         * 
         */ 
        DefaultPullMonitorable_impl&
        operator=(const DefaultPullMonitorable_impl& rhs)
            throw();
        
        /**
         * The actual behaviour of the process
         */  
        Cdmw::CdmwInit::ProcessControl_var      m_process_ctrl;
        
        /*
         * A cdmw local pull monitorable checking if the process is still
         * alive for CARDAMOM
         */
        CdmwFT::LocalPullMonitorable_var  m_local_pull_monitorable;

        /**
         * Parent POA
         */  
        PortableServer::POA_var m_poa;
        
    }; // End class DefaultPullMonitorable_impl


};




namespace Cdmw
{
namespace CdmwInit
{

CdmwProcess_impl::CdmwProcess_impl(CORBA::ORB_ptr           orb,
                                   PortableServer::POA_ptr  poa,
                                   PortableServer::POA_ptr  CDMW_rootPOA,
                                   ProcessControl*          ctrl,
                                   CdmwFT::LocalPullMonitorable_ptr lpm)
    throw(CORBA::SystemException)
    : m_orb(CORBA::ORB::_duplicate(orb)),
      m_poa(PortableServer::POA::_duplicate(poa)),
      m_CDMW_rootPOA(PortableServer::POA::_duplicate(CDMW_rootPOA)),
      m_process_ctrl(ctrl),
      m_default_pull_monitorable(::FT::PullMonitorable::_nil())
{
    CDMW_ASSERT(!CORBA::is_nil(orb));
    CDMW_ASSERT(ctrl != 0);
    m_process_ctrl->_add_ref();

    // Create a default PullMonitorableObject
    DefaultPullMonitorable_impl * pm 
        = new DefaultPullMonitorable_impl(poa, ctrl, lpm);
    m_default_pull_monitorable = pm->_this();
    // Release a ref since the POA is holding one.
    pm->_remove_ref();
}

CdmwProcess_impl::~CdmwProcess_impl()
    throw()
{
}


//
// IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/nb_steps:1.0
//
CORBA::ULong 
CdmwProcess_impl::nb_steps()
    throw(CORBA::SystemException)
{
    return m_process_ctrl->nb_steps();
}


//
// IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/get_service:1.0
//
CORBA::Object_ptr 
CdmwProcess_impl::get_service()
    throw(CORBA::SystemException)
{
    return m_process_ctrl->get_service();
}

//
// IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/get_pull_monitorable:1.0
//
::FT::PullMonitorable_ptr 
CdmwProcess_impl::get_pull_monitorable()
    throw(CORBA::SystemException)
{
    return ::FT::PullMonitorable::_duplicate(m_default_pull_monitorable.in());
}

//
// IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/get_push_monitorable:1.0
//
CdmwPlatformMngt::PushMonitorable_ptr 
CdmwProcess_impl::get_push_monitorable()
    throw(CORBA::SystemException)
{
    return CdmwPlatformMngt::PushMonitorable::_nil();
}


//
// IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/initialise:1.0
//
void CdmwProcess_impl::initialise(const CdmwPlatformMngtBase::StartupKind& startup_kind)
    throw(CORBA::SystemException)
{
    using namespace Cdmw::OsSupport;

    // CDMW POA Managers is in the discarding state. So we have to re-activate it
    // NOTA: We do activate the POA manager before on_initialise()
    //       to enable nested callbacks are needed (user receives an incomming request
    //       while making an outgoing call).
    PortableServer::POAManager_var CDMWpoaManager = m_CDMW_rootPOA -> the_POAManager();
    CDMWpoaManager->activate();
    
    CDMW_DEBUG(InitUtils::Get_Logger(), "Call on_initialise");
    m_process_ctrl->on_initialise(startup_kind);
}

//
// IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/next_step:1.0
//
void CdmwProcess_impl::next_step()
    throw(CdmwPlatformMngt::ProcessDelegate::InvalidStep,
          CORBA::SystemException)
{
    CDMW_DEBUG(InitUtils::Get_Logger(), "Call on_next_step");
    // TODO: Add CDMW specific Init if necessary
    m_process_ctrl->on_next_step();
}

//
// IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/run:1.0
//
void CdmwProcess_impl::run()
    throw(CdmwPlatformMngt::ProcessDelegate::NotReadyToRun,
          CORBA::SystemException)
{
    CDMW_DEBUG(InitUtils::Get_Logger(), "Call on_run");
    // TODO: Add CDMW specific stuff if necessary
    m_process_ctrl->on_run();
}

//
// IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/stop:1.0
//
void CdmwProcess_impl::stop()
    throw(CORBA::SystemException)
{
   
    CDMW_INFO(InitUtils::Get_Logger(), "Stop command received. Stopping the process ...");

    m_process_ctrl->on_stop();
    
    // TODO: Add CDMW specific stuff if necessary
    // then initiate ORB shutdown

    /*    PCR-1857
    ProcessStopHandler * stopHandler = new ProcessStopHandler(m_orb);
    Cdmw::OrbSupport::OrbSupport::install_orb_shutdown_signal_handlers(m_orb.in(), stopHandler);
    int pid = Cdmw::OsSupport::OS::get_processId();
    Cdmw::OsSupport::OS::kill_process (pid, SIGTERM);
    */
    CDMW_LOG(InitUtils::Get_Logger(), Cdmw::Logging::CDMW_DEBUG_1, "Shutting down the ORB ...");

    if (!CORBA::is_nil(m_orb.in())) m_orb->shutdown(false);

    //Cdmw::OrbSupport::OrbSupport::uninstall_orb_shutdown_signal_handlers(m_orb.in());       
}

void CdmwProcess_impl::activate_level(const char *comp, const char *dom, CORBA::ULong level_value) throw(CORBA::SystemException)
{
    //throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO) ;
    Cdmw::Logging::LogManager *p_log_mngr = m_process_ctrl->get_log_manager();

    Cdmw::Logging::FilterMngr *p_filter_mngr = p_log_mngr->get_filter_manager();

    if (strcmp(comp,"!")==0)
      p_filter_mngr->activate_level( dom, level_value);
    else
       p_filter_mngr->activate_level(comp, dom, level_value);

}

void CdmwProcess_impl::deactivate_level(const char *comp, const char *dom, CORBA::ULong level_value) throw(CORBA::SystemException)
{
    //throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO) ;

    Cdmw::Logging::LogManager *p_log_mngr = m_process_ctrl->get_log_manager();

    Cdmw::Logging::FilterMngr *p_filter_mngr = p_log_mngr->get_filter_manager();

    if (strcmp(comp,"!")==0)
      p_filter_mngr->deactivate_level( dom, level_value);
    else
      p_filter_mngr->deactivate_level(comp, dom, level_value);
}

//
// IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/call_command:1.0
//
/*
CdmwPlatformMngt::CommandResponse CdmwProcess_impl::call_command (
                 const char * command_name,
                 const CdmwPlatformMngt::CmdParameterValues & command_args,
                 CORBA::String_out result_info)
     throw(CdmwPlatformMngt::CommandCallBackNotFound,
           CORBA::SystemException)
{
    // TODO
    throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
}
*/
//
// IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/call_command:1.0
//
/*
CdmwPlatformMngt::CommandResponse CdmwProcess_impl::call_entity_command (
                const char * entity_name,
                const char * command_name,
                const CdmwPlatformMngt::CmdParameterValues & command_args,
                CORBA::String_out result_info)
        throw(CdmwPlatformMngt::CommandCallBackNotFound,
              CORBA::SystemException)
{
    // TODO
    throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
}
*/
//
// IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/get_component_redundancy_state:1.0
//
CdmwPlatformMngt::RedundancyEltState CdmwProcess_impl::get_component_redundancy_state(const char* component_name)
            throw(CORBA::SystemException)
{
    // TODO
    throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
}
//
// IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/get_process_redundancy_state:1.0
//
CdmwPlatformMngt::RedundancyEltState CdmwProcess_impl::get_process_redundancy_state()  throw(CORBA::SystemException)
{
    // TODO
    throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
}
//
// IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/get_managed_component:1.0
//
//CdmwPlatformMngt::ManagedComponent_ptr 
CORBA::Object_ptr CdmwProcess_impl::get_managed_component(const char* component_name) throw(CORBA::SystemException)
{
    CosNaming::NamingContext_var naming_context = CosNaming::NamingContextExt::_nil();
    try
    {
        CORBA::Object_var obj = m_orb->resolve_initial_references("ManagedComponentNamingContext");
            CDMW_INFO (InitUtils::Get_Logger(), "get_managed_component: ManagedComponentNamingContext resolved ") ;
        naming_context = CosNaming::NamingContextExt::_narrow(obj.in());
    }
        catch(const CORBA::SystemException& sys_exc)
    {
        CDMW_INFO (InitUtils::Get_Logger(), "exception while trying to resolve ManagedComponentNamingContext ") ;
        throw;
    }
    
    //create a Component Name   
    CosNaming::Name_var name = new CosNaming::Name;
    name->length(1);
    CosNaming::NameComponent_var nc = new CosNaming::NameComponent;
    
    CORBA::Object_var  m_component_obj = CORBA::Object::_nil();

    try
    {   
         CDMW_INFO (InitUtils::Get_Logger(), "resolving  the component from NamingContext:" <<component_name) ;
         CosNaming::Name comp_name(1);
         comp_name.length(1);
         comp_name[0].id   = component_name;
    
         m_component_obj = naming_context->resolve(comp_name);
         CDMW_INFO (InitUtils::Get_Logger(), "resolved the component from NamingContext: "<< component_name);
    }
    catch(CORBA::Exception &e)
    {
        CDMW_INFO (InitUtils::Get_Logger(), 
                   "exception while trying to resolve the component from NamingContext: "<< component_name) ;
        throw  ; // to replace with better exception
    }
    return m_component_obj._retn();
}

}; // End namespace CdmwInit
}; // End namespace Cdmw


