<?xml version="1.0" encoding="utf-8"?>
<!-- ===================================================================== -->
<!--
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009. 
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-->
<!-- ===================================================================== -->


<xsl:stylesheet version="1.1" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">



<!--
   Main - This template fetches the necessary data from inputs
   to generate a file.

   @param _programNode   A reference to a node /cdmw/programs/program.
-->

<xsl:template name="cdmw_interface_java">
   <!--
      Parameters below are used for recursiveness.
   -->
   <xsl:param name="_index" select="1"/>

   <xsl:variable name="programNodes" select="$cdmwNode/programs/program"/>

   <xsl:if test="boolean($programNodes)">
      <xsl:if test="$_index &lt;= count($programNodes)">
         <xsl:variable name="programNode" select="$programNodes[$_index]"/>

         <xsl:variable name="classname">
            <xsl:choose>
               <xsl:when test="boolean($programNode/@classname)">
                  <xsl:call-template name="getLastToken">
                     <xsl:with-param name="_string" select="$programNode/@classname"/>
                     <xsl:with-param name="_separator" select="$xmlSep"/>
                  </xsl:call-template>
               </xsl:when>
               <xsl:otherwise>
                  <xsl:value-of select="$programNode/@filename"/>
               </xsl:otherwise>
            </xsl:choose>
         </xsl:variable>
         
         <xsl:variable name="package">
            <xsl:choose>
               <xsl:when test="boolean($programNode/@classname)">
                  <xsl:call-template name="getScope">
                     <xsl:with-param name="_name" select="$programNode/@classname"/>
                     <xsl:with-param name="_lang" select="'java'"/>
                  </xsl:call-template>
               </xsl:when>
               <xsl:otherwise>
                  <xsl:value-of select="'cdmw.generated'"/>
               </xsl:otherwise>
            </xsl:choose>
         </xsl:variable>
      
         <!--
            Get the output filename.
         -->
         <xsl:variable name="filename">
            <xsl:call-template name="buildOutputFilename">
               <xsl:with-param name="_userDirectory" select="$programNode/@directory"/>
               <xsl:with-param name="_userFilename" select="$classname"/>
               <xsl:with-param name="_defaultFilename" select="'CdmwInterface'"/>
               <xsl:with-param name="_fileExtension" select="'.java'"/>
            </xsl:call-template>
         </xsl:variable>
      
         <!--
            Display a message to the user.
         -->
         <xsl:message>
            <xsl:value-of select="concat('Generating file ', $filename)"/>
         </xsl:message>
      
         <!--
            Generate the file.
         -->
         <xsl:document method="text" href="{$filename}">
            <!--
               Insert the document header.
            -->
            <xsl:call-template name="docHeader">
               <xsl:with-param name="_lang" select="'java'"/>
            </xsl:call-template>
            <!--
               Insert the content.
            -->
            <xsl:call-template name="cdmw_interface_java.content">
               <xsl:with-param name="_programNode" select="$programNode"/>
               <xsl:with-param name="_javaClassname" select="$classname"/>
               <xsl:with-param name="_cdmwInterfacePackage" select="$package"/>
            </xsl:call-template>
         </xsl:document>
      
         <!--
            Proceed to the next node.
         -->
         <xsl:call-template name="cdmw_interface_java">
            <xsl:with-param name="_index" select="$_index + 1"/>
         </xsl:call-template>
      </xsl:if>
   </xsl:if>
</xsl:template> <!-- end of template cdmw_interface_java -->



<!--
   This template generates the content of the file.

   @param _programNode   A reference to a node /cdmw/programs/program.
-->

<xsl:template name="cdmw_interface_java.content">
   <xsl:param name="_programNode"/>
   <xsl:param name="_javaClassname"/>
   <xsl:param name="_cdmwInterfacePackage"/>
   
   <!--
      References to some nodes.
   -->
   <xsl:variable name="servicesNode" select="$_programNode/services"/>
   <xsl:variable name="frameworkNodes" select="$servicesNode/lifecycle/framework"/>

   <!--
      Miscellaneous variables.
   -->
   <xsl:variable name="hasRepositoryInterface" select="boolean($servicesNode/repository-interface)"/>
   <xsl:variable name="hasPerformance" select="boolean($servicesNode/performance)"/>
   <xsl:variable name="hasTrace" select="boolean($servicesNode/trace)"/>
   <xsl:variable name="hasTime" select="boolean($servicesNode/time)"/>
   <xsl:variable name="hasSystemManagement">
      <xsl:choose>
         <xsl:when test="boolean($servicesNode/system-management/@fatal-handler)">
            <xsl:value-of select="$servicesNode/system-management/@fatal-handler"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:value-of select="'yes'"/>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:variable>
   <xsl:variable name="hasIssuerInterceptor">
      <xsl:choose>
         <xsl:when test="boolean($servicesNode/system-management/@command-issuer)">
            <xsl:value-of select="$servicesNode/system-management/@command-issuer"/>
         </xsl:when>
       <xsl:otherwise>
            <xsl:value-of select="'yes'"/>
       </xsl:otherwise>
      </xsl:choose>
   </xsl:variable>
   <!-- hasRequestFollowup defaults to 'no' -->
   <xsl:variable name="hasRequestFollowup">
      <xsl:choose>
         <xsl:when test="boolean($servicesNode/trace/@request-followup)">
            <xsl:value-of select="$servicesNode/trace/@request-followup"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:value-of select="'no'"/>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:variable>
   <!-- default is enable-buffering=yes -->
   <xsl:variable name="hasBufferingEnabled">
      <xsl:choose>
         <xsl:when test="boolean($servicesNode/trace/@enable-buffering)">
            <xsl:value-of select="$servicesNode/trace/@enable-buffering"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:value-of select="'yes'"/>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:variable>
   <xsl:variable name="hasRecording" select="boolean($servicesNode/recording)"/>
   <xsl:variable name="hasLoadBalancing" select="boolean($servicesNode/load-balancing)"/>
   <xsl:variable name="hasFaultTolerance" select="boolean($servicesNode/fault-tolerance)"/>
   <xsl:variable name="hasWatchdog" select="boolean($servicesNode/watchdog)"/>

   <xsl:variable name="hasLifecycle" select="boolean($servicesNode/lifecycle)"/>
   <xsl:variable name="hasHomeImpl">
      <xsl:call-template name="containsHomeImpl">
         <xsl:with-param name="_programNode" select="$_programNode"/>
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="hasEventPorts">
      <xsl:call-template name="containsEventPorts">
         <xsl:with-param name="_programNode" select="$_programNode"/>
      </xsl:call-template>
   </xsl:variable>

   <!--
      The code generation starts here.
   -->

<xsl:if test="string-length($_cdmwInterfacePackage)>0">
package <xsl:value-of select="$_cdmwInterfacePackage"/>;
</xsl:if>

import cdmw.common.Assert;
import cdmw.common.BadOrderException;
import cdmw.common.BadParameterException;
import cdmw.common.InternalErrorException;

import cdmw.orbsupport.ExceptionMinorCodes;
import cdmw.orbsupport.CORBASystemExceptionWrapper;

import cdmw.ossupport.OS;
import cdmw.commonsvcs.naming.NamingInterface;
import cdmw.namingandrepository.RepositoryInterface;
import com.thalesgroup.CdmwNamingAndRepository.DEFAULT_ROOT_CONTEXT;
import com.thalesgroup.CdmwNamingAndRepository.RepositoryPackage.NoRootContext;

<xsl:if test="$hasTrace">
     <xsl:choose>
       <xsl:when test="$hasBufferingEnabled='yes'">
import cdmw.trace.FlushAreaMngr;
import cdmw.trace.TraceDomains;
import cdmw.trace.TraceLevels;
import cdmw.trace.Stream;
       </xsl:when>
     <xsl:otherwise>
// Use of LogManagerCollectorDelegate instead of the FlushAreaMngr/tracelibrary
// NOTA: Discard of repetitive traces does not work with this configuration.
import cdmw.trace.Stream;
import cdmw.trace.FlushAreaMngr;
import cdmw.trace.TraceDomains;
import cdmw.trace.TraceLevels;
import cdmw.logging.LogManagerCollectorDelegate;
import com.thalesgroup.CdmwLogging.LogCollector;
       </xsl:otherwise>
     </xsl:choose>
</xsl:if>

<xsl:if test="$hasRequestFollowup='yes'">
import cdmw.commonsvcs.requestfollowup.NDCServiceInit;
</xsl:if>

<xsl:if test="$hasPerformance">
import cdmw.performance.PerformanceMngr;
</xsl:if>

import cdmw.cdmwinit.InitUtils;
import cdmw.cdmwinit.ProcessControl;
import cdmw.cdmwinit.DefaultProcessControl;
import cdmw.cdmwinit.CdmwProcessImpl;
import cdmw.cdmwinit.XMLErrorException;
import cdmw.cdmwinit.CdmwInitConfiguration;
import cdmw.cdmwinit.ConfigurationException;

import cdmw.lifecycle.POAThreadingPolicy;
import cdmw.lifecycle.ThreadPoolPolicy;
import cdmw.lifecycle.ThreadPerRequestPolicy;
import cdmw.lifecycle.SingleThreadPolicy;
import cdmw.lifecycle.AnonymousObjectsFactoryInitUtil;
import cdmw.lifecycle.NamedObjectsFactoryInitUtil;
   
<xsl:if test="$hasLoadBalancing">
import CdmwLB.LBGroupManager;
import cdmw.lb.lbinit.LBServiceInit;

</xsl:if>

   <xsl:if test="$hasHomeImpl='true'">
      import cdmw.ccm.container.HomeAllocator;
      import cdmw.ccm.container.HomeAllocatorRegistry;
      import cdmw.common.Locations;
      import cdmw.lifecycle.DefaultObjectRegistration;
   </xsl:if>

   <xsl:variable name="hasLoadBalancingServer">
      <xsl:choose>
         <xsl:when test="boolean($servicesNode/load-balancing/@server)">
            <xsl:value-of select="$servicesNode/load-balancing/@server"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:value-of select="'no'"/>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:variable>


   
public class <xsl:value-of select="$_javaClassname"/> {

   <xsl:if test="$hasRecording">
      <![CDATA[
    private static org.omg.CORBA.Object setRecorderPolicies(
                          org.omg.CORBA.ORB orb,
                          org.omg.CORBA.Object obj,
                          long callTimeout_millisec)
    {
        org.omg.CORBA.Policy[] policies = new org.omg.CORBA.Policy[2];
        int i = 0;
        try {
            // Create the timeout policy
//            if (callTimeout_millisec != 0) {
                org.omg.CORBA.Any timeoutAny = orb.create_any();
                // Set the timeout value as a 100 nanoseconds units
                long _timeout = callTimeout_millisec * 10000L;
                timeoutAny.insert_ulonglong(_timeout);
                policies[i++] = orb.create_policy(
                       org.omg.Messaging.RELATIVE_RT_TIMEOUT_POLICY_TYPE.value,
                       timeoutAny);
//            }




              String syncWithTarget =
                  System.getProperty(
                      cdmw.cdmwinit.CdmwInitSystemProperties.RECORDING_SYNC_WITH_TARGET_PROPERTY_NAME,
                      "no");

              short syncScope = org.omg.Messaging.SYNC_WITH_SERVER.value;

              if(syncWithTarget.equals("yes")) {
                  syncScope = org.omg.Messaging.SYNC_WITH_TARGET.value;
              }



            org.omg.CORBA.Any syncScopeAny = orb.create_any();
            syncScopeAny.insert_short(syncScope);
            policies[i++] = orb.create_policy(
                   org.omg.Messaging.SYNC_SCOPE_POLICY_TYPE.value,
                   syncScopeAny);

            org.omg.CORBA.Object capsule = obj._set_policy_override(
                policies,
                org.omg.CORBA.SetOverrideType.SET_OVERRIDE);

            for ( int j=0 ; j<i ; j++ ) {
                policies[j].destroy();
            }

            return capsule;           

        } catch (org.omg.CORBA.PolicyError e) {
            // shouldn't happen
            throw new org.omg.CORBA.INTERNAL(
                ExceptionMinorCodes.INTERNAL,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
    }
      ]]>
   </xsl:if>

    public static void cdmwInit(
        org.omg.CORBA.ORB orb,
        String[] args,
        ProcessControl ctrl)
        throws org.omg.CORBA.SystemException {

        Assert.check(orb != null);

        try {
            // PCR-0049
            cdmw.orbsupport.Codec.init(orb);

            // Get the XML init filename. This may raise BAD_PARAM or INTERNAL
            String xmlFile = InitUtils.getXmlInitialisationFile(args);

            // Get the XML documents validation option. This may raise INTERNAL
            boolean validateXml = InitUtils.getXmlValidationOption(args);

            // Parsing XML
            CdmwInitConfiguration xmlData = null;
            try {
                CdmwInitConfiguration init =
                    CdmwInitConfiguration.readConfiguration(
                        xmlFile,
                        validateXml);
                xmlData = init;
            } catch (InternalErrorException iee) {
                throw new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            } catch (ConfigurationException ce) {
                throw new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }

            // *****************************************************************
            // *****************  CDMW root POA and servant manager ***********
            // *****************************************************************

            // Create the CDMW root POA. It has its own POA Manager
            // May raise a org.omg.CORBA.SystemException.
            org.omg.PortableServer.POA cdmwRootPOA =
                InitUtils.createCdmwRootPOA(orb);

            org.omg.PortableServer.POAManager poaManager =
                cdmwRootPOA.the_POAManager();

            // Initialize the server so that incoming requests are
            // handled from this point on
            // May raise org.omg.PortableServer.POAManager.AdapterInactive
            // or org.omg.CORBA.SystemException
            try {
                poaManager.activate(); // TODO: maybe move it ?
            } catch (
                org.omg.PortableServer.POAManagerPackage.AdapterInactive ai) {
                throw new org.omg.CORBA.INTERNAL(
                    ExceptionMinorCodes.INTERNALORBAdapterInactive,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }

    <xsl:if test="$hasFaultTolerance">
      <![CDATA[
            // *****************************************************************
            // *************  setup FaultTolerance Policies   ******************
            // *****************************************************************
            
            long durationTime;
            String clientId;
               
            try {
                String resFTDuration = 
                    xmlData.getServiceAttributeValue("fault-tolerance", 
                                                     "request-duration-policy"); 
                durationTime = Long.parseLong(resFTDuration);
                
            } catch (BadParameterException bpe) {
                throw new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            } catch (XMLErrorException xmle) {
                throw new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            } catch (NumberFormatException e) {
                throw new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }
            
            try  {
                 clientId = xmlData.getServiceAttributeValue("fault-tolerance", 
                                                             "client-id");
                 cdmw.ft.FTConfiguration.SetClientId(orb, clientId);
            } catch (BadParameterException bpe) {
                 // do nothing : parameter is optional
            } catch (XMLErrorException xmle) {
                throw new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }
            
            org.omg.CORBA.Object obj;
            try {
                obj = orb.resolve_initial_references("ORBPolicyManager");
            } catch (org.omg.CORBA.ORBPackage.InvalidName e) {
                // shouldn't happen if ORB is standard
                throw new org.omg.CORBA.INTERNAL(
                    ExceptionMinorCodes.INTERNAL,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }
            
            org.omg.CORBA.PolicyManager policyManager =
                org.omg.CORBA.PolicyManagerHelper.narrow(obj);
            
            // Create and apply an ORB-wide Routed policy
            cdmw.logging.Logger.CDMW_DEBUG(
                InitUtils.Get_Logger(), 
                "Request duration policy set to " + durationTime,
                new Throwable());
            org.omg.CORBA.Any any = orb.create_any();
            
            any.insert_ulonglong(durationTime);
            org.omg.CORBA.Policy[] policies =
                new org.omg.CORBA.Policy[1];
            try {
                policies[0] =
                   orb.create_policy(org.omg.FT.REQUEST_DURATION_POLICY.value, any);
                
                policyManager.set_policy_overrides(policies, 
                   org.omg.CORBA.SetOverrideType.ADD_OVERRIDE);
                policies[0].destroy();
                
            } catch (org.omg.CORBA.PolicyError e) {
                // shouldn't happen if ORB implements FT
                throw new org.omg.CORBA.INTERNAL(
                    ExceptionMinorCodes.INTERNAL,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            } catch (org.omg.CORBA.InvalidPolicies e) {
                // shouldn't happen if ORB implements FT
                throw new org.omg.CORBA.INTERNAL(
                    ExceptionMinorCodes.INTERNAL,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }            
      ]]>
   </xsl:if>            

   <xsl:if test="$hasWatchdog">
      <![CDATA[
            // *****************************************************************
            // *************  INIT WatchdogServiceInit  **********
            // *****************************************************************
            try {
                // No System.getProperty() performed like in C++ code because
                // WATCHDOG_TIMEOUT_PROPERTY_NAME property is set by default 
                // with the default request_timeout value. Thus the property
                // is not null.
                String str = xmlData.getServiceAttributeValue("watchdog", 
                                                          "timeout");
 
                int milliseconds = Integer.parseInt(str); 
                cdmw.ft.watchdog.WatchdogServiceInit.setTimeout(milliseconds);
            
                // export property for debug purpose
                System.setProperty(
                    cdmw.ft.FTSystemProperties.WATCHDOG_TIMEOUT_PROPERTY_NAME, 
                    str);
                  
            } catch (BadParameterException bpe) {
                throw new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            } catch (XMLErrorException xmle) {
                throw new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            } catch (NumberFormatException e) {
                throw new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            } catch (org.omg.CORBA.BAD_PARAM e) {
                throw new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAM,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }
      ]]>
   </xsl:if>

   <xsl:if test="($hasBufferingEnabled='no') or $hasRecording">
     
      <![CDATA[

         // *****************************************************************
         // *********   Retrieve the Daemon's NameService object      *******
         // *********   reference and register it as LocalNameService *******
         // *********   SystemPort option is now handled by           *******
         // *********   OrbSupport.ORB_init(argc,argv)               *******
         // *****************************************************************
         org.omg.CosNaming.NamingContext local_NameService;

         String system_port = System.getProperty(cdmw.orbsupport.ORBSupport.SYSTEM_PORT_PROPERTY_NAME);
         if (system_port.trim().equals("")) {
            cdmw.logging.Logger.CDMW_FATAL(InitUtils.Get_Logger(), 
                    "CDMW_init: CdmwSystemPort has not be declared"
                    + " in argument. The port allows to perfom"
                    + " register_initial_reference() to access on"
                    + " recording daemon and recording server.",
                    new Throwable());
            throw new org.omg.CORBA.BAD_PARAM(
               ExceptionMinorCodes.BAD_PARAMMissingPortNumber, 
               org.omg.CORBA.CompletionStatus.COMPLETED_NO);
         } else {
            try {
               String corbaloc = "corbaloc::";
               corbaloc += "localhost";
               corbaloc += ":";        
               corbaloc += system_port;
               corbaloc +="/NameService";
               org.omg.CORBA.Object obj__ = orb.string_to_object(corbaloc);
               Assert.check(obj__ != null);

               try {
               local_NameService  = org.omg.CosNaming.NamingContextHelper.narrow(obj__);
               } catch (org.omg.CORBA.SystemException ex) {
                   cdmw.logging.Logger.CDMW_FATAL(InitUtils.Get_Logger(), 
                      "org.omg.CORBA SystemException during CDMW_init: "
                      + cdmw.orbsupport.ORBSupport.exceptionToString(ex),
                      new Throwable());
 
                   throw ex;
               }

               // Register initial reference for the LocalNameService located in the daemon
               ((org.omg.CORBA_2_5.ORB)orb).register_initial_reference("LocalNameService", local_NameService);

            } catch (org.omg.CORBA.ORBPackage.InvalidName ex)  {
                 cdmw.logging.Logger.CDMW_FATAL(InitUtils.Get_Logger(),
                    "CDMW_init: InvalidName => Cannot register LocalNameService."
                    + " failed. " + cdmw.orbsupport.ORBSupport.exceptionToString(ex),
                    new Throwable());
                 throw new CORBASystemExceptionWrapper(
                            new org.omg.CORBA.INTERNAL(
                            ExceptionMinorCodes.INTERNAL,
                            org.omg.CORBA.CompletionStatus.COMPLETED_NO));
            } catch (Exception ex) {
                 cdmw.logging.Logger.CDMW_FATAL(InitUtils.Get_Logger(),
                    "CDMW_init:  Could not retrieve local NameService's object reference! " + ex.getMessage(),
                     new Throwable());
                throw new CORBASystemExceptionWrapper(
                            new org.omg.CORBA.INTERNAL(
                    ExceptionMinorCodes.INTERNAL,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO));
            }
         }
      ]]>   

   </xsl:if>
   
            // *****************************************************************
            // *****************  INIT platforminterface object  **************
            // *****************************************************************

            // Did we receive a ProcessControl callback object?
            ProcessControl processCtrl;
            if (ctrl != null)
                processCtrl = ctrl;
            else
                processCtrl = new DefaultProcessControl();            

            com.thalesgroup.CdmwFT.LocalPullMonitorable lpm = null;
    <xsl:if test="$hasWatchdog">
            lpm = new cdmw.ft.watchdog.RequestTimeoutLocalPullMonitorableImpl();
    </xsl:if>
                
            // CdmwProcessImpl() will acquire a reference on ProcessControl 
            // object and will automatically release it at object's destruction.
            CdmwProcessImpl processImpl =
                new CdmwProcessImpl(
                    orb,
                    InitUtils.getRootPOA(orb),
                    cdmwRootPOA,
                    processCtrl,
                    lpm);
            com.thalesgroup.CdmwPlatformMngt.ProcessDelegate initProcess =
                processImpl._this(orb);
    <xsl:choose>
            <xsl:when test="$hasSystemManagement='yes'">
            <![CDATA[// Set the Property before calling init_platform_interface which calls PlatformInterface.Setup().
            // The Property is read in Setup().
            System.setProperty(
               cdmw.orbsupport.ORBSupport.SYSTEM_MANAGEMENT_FATAL_HANDLER_PROPERTY_NAME, 
               "true");]]>
            </xsl:when>
            <xsl:otherwise>
            <![CDATA[// Set the Property before calling init_platform_interface which calls PlatformInterface.Setup().
            // The Property is read in Setup().
            System.setProperty(
               cdmw.orbsupport.ORBSupport.SYSTEM_MANAGEMENT_FATAL_HANDLER_PROPERTY_NAME, 
               "false");]]>
            </xsl:otherwise>
    </xsl:choose>
            InitUtils.initPlatformInterface(orb, args, initProcess);
            //  From now on, a platform supervisor may initiate a call to 
            //  process.init() method. 
            //  process.init() call is blocked with a condition until CDMW
            //  init has completed?

            // Get the process name.
            // This may raise org.omg.CORBA.BAD_INV_ORDER
            String processName = InitUtils.getCdmwProcessName();

            // Get the process name.
            // This may raise org.omg.CORBA.BAD_INV_ORDER
            String applicationName = InitUtils.getCdmwApplicationName();

            cdmw.logging.LogManager.Instance().setProcessName(processName);
            cdmw.logging.LogManager.Instance().setApplicationName(applicationName);

            // Retrieve Repository object reference.
            // This may raise org.omg.CORBA.BAD_PARAM
            com.thalesgroup.CdmwNamingAndRepository.Repository repository =
                InitUtils.getCdmwRepository();

   <xsl:if test="$hasFaultTolerance">
      <![CDATA[
            // *****************************************************************
            // ***********  setup FaultTolerance ReplicationManager  ***********
            // *****************************************************************
            
            // Get the reference of the Replication Manager and register it in the ORB
            org.omg.CORBA.Object objref =
                InitUtils.getService("ReplicationManager");  
            
            com.thalesgroup.CdmwFT.ReplicationManager replicationManager = 
                com.thalesgroup.CdmwFT.ReplicationManagerHelper.narrow(objref);
            
            try  
            {
                org.omg.CORBA_2_5.ORB orb25 = (org.omg.CORBA_2_5.ORB) orb;
                orb25.register_initial_reference(
                    "ReplicationManager", 
                    replicationManager);      
            }
            catch (org.omg.CORBA.ORBPackage.InvalidName ex)
            {
                throw new org.omg.CORBA.INTERNAL(
                    ExceptionMinorCodes.INTERNAL,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }
      ]]>
   </xsl:if>                
                
      <xsl:if test="$hasRepositoryInterface">
         <![CDATA[
            // *****************************************************************
            // *****************  INIT of RepositoryInterface ******************
            // *****************************************************************
            String defaultDomaineName = "";

            try {
                defaultDomaineName =
                    xmlData.getServiceAttributeValue(
                        "repository-interface",
                        "default-domain-name");
            } catch (BadParameterException bpe) {
                processImpl.setCdmwInitExceptionRaised();
                throw new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            } catch (XMLErrorException xmle) {
                processImpl.setCdmwInitExceptionRaised();
                throw new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }

            try {
                RepositoryInterface.init(defaultDomaineName, repository);
            } catch (
                com.thalesgroup.CdmwNamingAndRepository.NoNameDomain nnd) {
                processImpl.setCdmwInitExceptionRaised();
                // Default NameDomain is not a NameDomain!
                throw new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMNameDomainNotFound,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            } catch (com.thalesgroup.CdmwNamingAndRepository.InvalidName in) {
                processImpl.setCdmwInitExceptionRaised();
                // Default NameDomain name is ill-formed!
                throw new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMInvalidDefaultDomainName,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }
         ]]>
      </xsl:if>

      <xsl:if test="$hasTrace">
         <xsl:choose>
            <xsl:when test="$hasBufferingEnabled='yes'">
         <![CDATA[
            // *****************************************************************
            // *********         Init The CDMW Trace Library          *********
            // *****************************************************************

            String res;
            int flushingTime = 0;
            int msgThreshold = 0;
            int nbFlushArea = 0;
            int sizeFlushArea = 0;
            String collectorName;
            java.util.Vector collectorNameList = new java.util.Vector();

            try {
                try {
                    flushingTime = Integer.parseInt(
                        xmlData.getServiceAttributeValue(
                            "trace",
                            "flushing-time"));
                    nbFlushArea = Integer.parseInt(
                        xmlData.getServiceAttributeValue(
                            "trace",
                            "flush-area-nb"));
                    sizeFlushArea = Integer.parseInt(
                        xmlData.getServiceAttributeValue(
                            "trace",
                            "flush-area-sz"));
                    collectorName =
                        xmlData.getServiceAttributeValue(
                            "trace",
                            "collector-name");
                    collectorNameList.add(collectorName);
                }
                catch (BadParameterException bpe) {
                    processImpl.setCdmwInitExceptionRaised();
                    throw new org.omg.CORBA.BAD_PARAM(
                        ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                }

                try {
                    msgThreshold = Integer.parseInt(
                        xmlData.getServiceAttributeValue(
                            "trace",
                            "msg-threshold"));
                } catch (BadParameterException bpe) {
                    // do nothing : parameter is optional
                }
 
                try {
                    collectorName =
                        xmlData.getServiceAttributeValue(
                            "trace",
                            "collector-name");
                    collectorNameList.add(collectorName);
                } catch (BadParameterException bpe) {
                    // do nothing : parameter is optional
                }

            } catch (XMLErrorException xee) {
                processImpl.setCdmwInitExceptionRaised();
                throw new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            } catch (NumberFormatException nfe) {
                processImpl.setCdmwInitExceptionRaised();
                throw new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }

            // May raise org.omg.CORBA.SystemException
            cdmw.trace.InitUtils.initTraceLibrary(
                cdmwRootPOA,
                applicationName,
                processName,
                flushingTime,
                msgThreshold,
                nbFlushArea,
                sizeFlushArea,
                collectorNameList);

            // active the trace flushing to trace collector
            FlushAreaMngr.getInstance().activateFlushing();
         ]]>
         
       </xsl:when>
         <xsl:otherwise>    
      <![CDATA[
       
         // *****************************************************************
         // *********   Init The CDMW LogManagerCollectorDelegate   *********
         // *****************************************************************
         {

            cdmw.logging.Logger.CDMW_DEBUG(InitUtils.Get_Logger(), 
                                           "Init LogManagerCollectorDelegate",
                                           new Throwable());
            int msg_threshold = 0; // NOTA: Not used in this version
            String collectorName = "";
            try {
               try {
                  collectorName = xmlData.getServiceAttributeValue("trace", "collector-name");
               } catch (BadParameterException ex) {
                  cdmw.logging.Logger.CDMW_DEBUG(InitUtils.Get_Logger(),
                                           "BadParameterException received: ",
                                           new Throwable());
               }

               try  {
                  //FlushAreaMngr DEFAULT_MSG_THRESHOLD;
                  String resThreshold = xmlData.getServiceAttributeValue("trace", "msg-threshold"); 
                  msg_threshold = Integer.valueOf(resThreshold).intValue();
               } catch (BadParameterException e) {
                  // do nothing : parameter is optional
               }
            } catch (XMLErrorException ex) {
               processImpl.setCdmwInitExceptionRaised();
               cdmw.logging.Logger.CDMW_WARN(InitUtils.Get_Logger(), 
                         "XMLErrorException received: "
                         + ex.getMessage(),
                         new Throwable());
               throw new CORBASystemExceptionWrapper(new org.omg.CORBA.BAD_PARAM(
                      ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                      org.omg.CORBA.CompletionStatus.COMPLETED_NO));
            }
            // We have a collector name now. Let's retreive it from PlatformDaemon (NameServer).
            // Use collectorName if provided by user, otherwise use default one.
            if (collectorName.trim().equals("")) {
               collectorName = cdmw.common.Locations.DEFAULT_LOG_COLLECTOR_ID;
            }
            com.thalesgroup.CdmwLogging.LogCollector logCollector__;
            if (local_NameService == null)
            {
               cdmw.logging.Logger.CDMW_WARN(InitUtils.Get_Logger(), "No Local NameService available. "
                         + "Trace logs will not be sent to Trace Collector.",
                         new Throwable());
            } else {
               NamingInterface localNI = new NamingInterface(local_NameService);
               org.omg.CORBA.Object obj__;
               try {
                  obj__ = localNI.resolve(collectorName);
                  logCollector__ = com.thalesgroup.CdmwLogging.LogCollectorHelper.narrow(obj__);
                  if (logCollector__ == null) {
                     cdmw.logging.Logger.CDMW_WARN(InitUtils.Get_Logger(), 
                               "'" + collectorName + "' does not denote a valid "
                               + "Log Collector object. "
                               + "Trace logs will not be sent to Trace Collector.",
                               new Throwable());
                  }
                  else {
                     cdmw.logging.LogManagerCollectorDelegate delegate__ =
                        new cdmw.logging.LogManagerCollectorDelegate(logCollector__);
                     cdmw.logging.LogManagerDelegate d__ = 
                        cdmw.logging.LogManager.Instance().registerAlternateDelegate(delegate__);
                  }
               } catch (org.omg.CosNaming.NamingContextPackage.NotFound nf) {
                  cdmw.logging.Logger.CDMW_WARN(InitUtils.Get_Logger(), 
                            "Could not retrieve reference of the Log Collector "
                            + "'" + collectorName + "' not found in the naming. "
                            + "Trace logs will not be sent to Trace Collector.",
                            new Throwable());
               } catch (org.omg.CosNaming.NamingContextPackage.CannotProceed cp) {
                  cdmw.logging.Logger.CDMW_WARN(InitUtils.Get_Logger(), 
                            "Could not retrieve reference of the Log Collector (cannot proceeed) "
                            + "Trace logs will not be sent to Trace Collector.",
                            new Throwable());
               } catch (org.omg.CosNaming.NamingContextPackage.InvalidName in) {
                  cdmw.logging.Logger.CDMW_WARN(InitUtils.Get_Logger(), 
                            "Could not retrieve reference of the Log Collector (invalid name) "
                            + "Trace logs will not be sent to Trace Collector.",
                            new Throwable());
               }

               
            }
         }

      ]]>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:if>

      <xsl:if test="$hasPerformance">
         <![CDATA[
            // *****************************************************************
            // *********       Init The CDMW Performance Library      *********
            // *****************************************************************

            String performanceFileName = null;
            try {
                performanceFileName =
                    xmlData.getServiceAttributeValue(
                        "performance",
                        "result-file-name");
            } catch (BadParameterException bpe) {
                processImpl.setCdmwInitExceptionRaised();
                throw new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            } catch (XMLErrorException xee) {
                processImpl.setCdmwInitExceptionRaised();
                throw new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }

            // May raise org.omg.CORBA.SystemException
            if (performanceFileName != null) {
                cdmw.performance.InitUtils.initPerformanceLibrary(
                    cdmwRootPOA,
                    performanceFileName);
            }
         ]]>
      </xsl:if>
      
      <xsl:if test="$hasLoadBalancing">
         <![CDATA[
            // *****************************************************************
            // ***********    setup LoadBalancing LBGroupManager     ***********
            // *****************************************************************
      



            CdmwLB.LBGroupManager lbGroupMgr = null;
            try 
            {
                String daemon_port = java.lang.System.getProperty(cdmw.orbsupport.ORBSupport.SYSTEM_PORT_PROPERTY_NAME);
                String corbanameLBGM = "corbaname::localhost:"+daemon_port+"#ReplicationManager/LBGroupManagerGroup";
                org.omg.CORBA.Object objRef =  orb.string_to_object(corbanameLBGM);
                lbGroupMgr = CdmwLB.LBGroupManagerHelper.narrow(objRef);
             } 
            catch (org.omg.CORBA.SystemException ex) {
                System.err.println(
                    "Exception retrieving FT LBGroupManager from Repository: " + ex
                    + ". Assumes non-FT LBGroupManager has been specified "
                    + "with -ORBInitRef...");
            }


            if (lbGroupMgr != null) {
                try {
                    ((org.omg.CORBA_2_5.ORB) orb).register_initial_reference("LBGroupManager", lbGroupMgr);      
                }
                catch (org.omg.CORBA.SystemException ex)
                {
                    System.err.println( 
                       "Error regitering LBGroupManager as initial reference: " + ex);
                    throw ex;
                }
                catch (org.omg.CORBA.ORBPackage.InvalidName ex)
                {
                    System.err.println( 
                        "Error regitering LBGroupManager as initial reference: " + ex);
                    throw new org.omg.CORBA.INTERNAL(
                        ExceptionMinorCodes.INTERNAL,
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                }
            }
         ]]>
      </xsl:if>
   
      <xsl:if test="$hasTime">
         <![CDATA[
            // *****************************************************************
            // *********         Init The CDMW Clock Service           *********
            // *****************************************************************
            
              
            {
                if (! cdmw.clock.CdmwCosClock.init(orb,args))
                   System.err.println("ERROR during Clock Service Initialization");
            }
         ]]>
      </xsl:if>

      <xsl:if test="$hasRecording">
         <![CDATA[
            // *****************************************************************
            // **********  register initial references for Recording ***********
            // *****************************************************************
            //
            
            Assert.check(local_NameService != null); 
            NamingInterface localNi = new NamingInterface(local_NameService);
 
            long callTimeout_millisec = 0L;
            try {
               String resRecordingTimeout = 
                  xmlData.getServiceAttributeValue("recording", "recorder-timeout"); 
                  callTimeout_millisec = Long.parseLong(resRecordingTimeout);
            } catch (BadParameterException ex) {
                  cdmw.logging.Logger.CDMW_WARN(InitUtils.Get_Logger(),
                          "'recorder-timeout' attribut was not found. Call timeout set to unlimited.",
                          new Throwable());
            } catch (XMLErrorException xmle) {
                processImpl.setCdmwInitExceptionRaised();
                throw new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }

            // Register initial reference for the recording daemon
            cdmw.logging.Logger.CDMW_DEBUG(InitUtils.Get_Logger(),
                                           "CDMW_init: register initial reference of the recording daemon",
                                           new Throwable());
            org.omg.CORBA.Object daemonObj; 
            try {
                daemonObj = localNi.resolve(com.thalesgroup.CdmwRecording.REC_CONTAINER_MNGR_ID.value);
            } catch (org.omg.CosNaming.NamingContextPackage.NotFound nf) {
                cdmw.logging.Logger.CDMW_FATAL(InitUtils.Get_Logger(),
                    "CDMW_init: org.omg.CosNaming.NamingContextPackage.NotFound exception. "
                    + "Recording daemon control not found. "
                    + cdmw.orbsupport.ORBSupport.exceptionToString(nf),
                     new Throwable());
                throw new CORBASystemExceptionWrapper(
                            new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAM,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO));
            } catch (Exception e) {
                cdmw.logging.Logger.CDMW_FATAL(InitUtils.Get_Logger(),
                    "CDMW_init: Exception during the resolving of REC_CONTAINER_MNGR_ID in the naming. "
                    + e.getMessage(),
                     new Throwable());
                throw new CORBASystemExceptionWrapper(
                            new org.omg.CORBA.INTERNAL(
                    ExceptionMinorCodes.INTERNAL,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO));
            }

            if (daemonObj == null)
            { 
               cdmw.logging.Logger.CDMW_FATAL(InitUtils.Get_Logger(),
                   "CDMW_init: null Object Reference for '" 
                   + com.thalesgroup.CdmwRecording.REC_CONTAINER_MNGR_ID.value,
                   new Throwable());
                throw new CORBASystemExceptionWrapper(
                            new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAM,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO));
            }

            try {
                ((org.omg.CORBA_2_5.ORB)orb).register_initial_reference(
                    com.thalesgroup.CdmwRecording.REC_CONTAINER_MNGR_ID.value, 
                    daemonObj);
            }
            catch (org.omg.CORBA.ORBPackage.InvalidName in) {
                cdmw.logging.Logger.CDMW_FATAL(InitUtils.Get_Logger(),
                "CDMW_init: register_initial_reference of "
                + com.thalesgroup.CdmwRecording.REC_CONTAINER_MNGR_ID.value
                + " failed. " + cdmw.orbsupport.ORBSupport.exceptionToString(in),
                new Throwable());
                throw new CORBASystemExceptionWrapper(
                            new org.omg.CORBA.INTERNAL(
                    ExceptionMinorCodes.INTERNAL,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO));
            }

            try {
                daemonObj =  localNi.resolve(com.thalesgroup.CdmwRecording.REC_DAEMON_CTRL_ID.value);
            } catch (org.omg.CosNaming.NamingContextPackage.NotFound nf) {
                cdmw.logging.Logger.CDMW_FATAL(InitUtils.Get_Logger(),
                    "CDMW_init: org.omg.CosNaming.NamingContextPackage.NotFound exception. "
                    + "Recording daemon control not found. "
                    + cdmw.orbsupport.ORBSupport.exceptionToString(nf),
                     new Throwable());
                throw new CORBASystemExceptionWrapper(
                    new org.omg.CORBA.INTERNAL(
                       ExceptionMinorCodes.INTERNAL,
                       org.omg.CORBA.CompletionStatus.COMPLETED_NO));
            }catch (Exception e) {
                cdmw.logging.Logger.CDMW_FATAL(InitUtils.Get_Logger(),
                    "CDMW_init:  Exception raised. Recording daemon control not found. " + e.getMessage(),
                     new Throwable());
                throw new CORBASystemExceptionWrapper(
                    new org.omg.CORBA.INTERNAL(
                       ExceptionMinorCodes.INTERNAL,
                       org.omg.CORBA.CompletionStatus.COMPLETED_NO));
            }

            if (daemonObj == null)
            { 
               cdmw.logging.Logger.CDMW_FATAL(InitUtils.Get_Logger(),
                   "CDMW_init: null Object Reference for '" 
                   + com.thalesgroup.CdmwRecording.REC_DAEMON_CTRL_ID.value,
                   new Throwable());
               throw new CORBASystemExceptionWrapper(
                   new org.omg.CORBA.BAD_PARAM(
                       ExceptionMinorCodes.BAD_PARAM,
                       org.omg.CORBA.CompletionStatus.COMPLETED_NO));
            }

            com.thalesgroup.CdmwRecording.DaemonControl daemon = 
               com.thalesgroup.CdmwRecording.DaemonControlHelper.narrow(daemonObj);

            String recordingServerUrl = daemon.recorder_url();
            cdmw.logging.Logger.CDMW_DEBUG(InitUtils.Get_Logger(), 
                "CDWM_init: recording server URL  = "
                + recordingServerUrl,
                new Throwable());

            if ( recordingServerUrl.trim().equals("") ) {
                cdmw.logging.Logger.CDMW_WARN(InitUtils.Get_Logger(), 
                "CDWM_init: No Recording server url defined!"
                + "Thus register_initial_reference() is not perfomed for the Recording server.",
                new Throwable());
            } else {
                // Register initial reference for the recording server
                //
                cdmw.logging.Logger.CDMW_DEBUG(InitUtils.Get_Logger(), 
                    "CDMW_init: register initial reference of the recording server",
                     new Throwable());

                try {
                    org.omg.CORBA.Object recordingServerObj =
                            orb.string_to_object(recordingServerUrl);
                    Assert.check(recordingServerObj != null);
                    // Use SyncScope and relative roundtrip policies to achieve reliable oneways
                    org.omg.CORBA.Object capsule = setRecorderPolicies(
                            orb, 
                            recordingServerObj,
                            callTimeout_millisec);
                    ((org.omg.CORBA_2_5.ORB)orb).register_initial_reference("RecordingServer", capsule);
                }  
                catch (org.omg.CORBA.ORBPackage.InvalidName in) {
                    cdmw.logging.Logger.CDMW_FATAL(InitUtils.Get_Logger(),
                    "CDMW_init:org.omg.CORBA.ORBPackage.InvalidName exception raised. "
                    + "register_initial_reference of RecordingServer not performed. "
                    + cdmw.orbsupport.ORBSupport.exceptionToString(in),
                    new Throwable());
                    throw new CORBASystemExceptionWrapper(
                                new org.omg.CORBA.INTERNAL(
                        ExceptionMinorCodes.INTERNAL,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO));
                }
            }
         ]]>
      </xsl:if>
      <xsl:if test="$hasLifecycle">
         <![CDATA[
            // *****************************************************************
            // *********  Create CDMW Factories POA (persistent POA)  *********
            // *****************************************************************
            // Raises org.omg.CORBA.SystemException
            org.omg.PortableServer.POA factoriesPOA =
                InitUtils.createCdmwLifeCyclePOA(cdmwRootPOA);

            // Create CDMW factories

            org.omg.CosNaming.NamingContext ncRootContext = null;
            try {
                ncRootContext = 
                    repository.resolve_root_context(DEFAULT_ROOT_CONTEXT.value);
            } catch (NoRootContext nrc) {
                processImpl.setCdmwInitExceptionRaised();
                throw new org.omg.CORBA.INTERNAL(
                    ExceptionMinorCodes.INTERNALCdmwRepositoryError,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            } catch (org.omg.CORBA.SystemException se) {
                processImpl.setCdmwInitExceptionRaised();
                throw se;
            }

            // May raise a org.omg.CORBA.SystemException
            NamingInterface rootContext = new NamingInterface(ncRootContext);
         ]]>

      <xsl:if test="$hasHomeImpl='true'">
         // Register Valuetype factories
         cdmw.ccm.cif.InitUtils.registerValuetypeFactories(orb);
         cdmw.ccm.container.InitUtils.registerValuetypeFactories(orb);
      </xsl:if>

      String fullNameDomainName = "";
      String fullFactoryName = "";
      POAThreadingPolicy threadingPolicy = null;
      int i = 0;
      <xsl:for-each select="$frameworkNodes">
         <xsl:variable name="frameworkIdref" select="@idref"/>

         <!--
            Get the matching node /cdmw/lifecycle-frameworks/lifecycle-framework.
         -->
         <xsl:variable name="lifecycleFrameworkNode" select="$lifecycleFrameworkNodes[@id = $frameworkIdref]"/>
         <xsl:variable name="isHomeImpl" select="boolean($lifecycleFrameworkNode/home-impl)"/>

         <!--
            Get the factory classname.
         -->
         <!--
         <xsl:variable name="factoryClassname" select="$lifecycleFrameworkNode/*/@classname"/>
         -->
         <xsl:variable name="factoryClassname">
            <xsl:choose>
               <xsl:when test="$isHomeImpl">
                  <xsl:call-template name="buildHomeImplClassname">
                     <xsl:with-param name="_homeImplNode" select="$lifecycleFrameworkNode/home-impl"/>
                  </xsl:call-template>
               </xsl:when>
               <xsl:otherwise>
                  <xsl:value-of select="$lifecycleFrameworkNode/*/@classname"/>
               </xsl:otherwise>
            </xsl:choose>
         </xsl:variable>

         <!--
            Format the class name for output.
         -->
         <xsl:variable name="javaFactoryClassname">
            <xsl:call-template name="replaceCharsInString">
               <xsl:with-param name="_stringIn" select="$factoryClassname"/>
               <xsl:with-param name="_charsIn" select="$xmlSep"/>
               <xsl:with-param name="_charsOut" select="$javaSep"/>
            </xsl:call-template>
         </xsl:variable>

         <!--
            Get the strategy.
         -->
         <xsl:variable name="strategy">
            <xsl:variable name="category" select="$lifecycleFrameworkNode/*/@category"/>
            <xsl:choose>
               <xsl:when test="boolean($category)">
                  <xsl:value-of select="$category"/>
               </xsl:when>
               <xsl:otherwise>
                  <xsl:value-of select="'USER_DEFINED'"/>
               </xsl:otherwise>
            </xsl:choose>
         </xsl:variable>

         <!--
            Register the home allocator.
         -->
         <xsl:if test="$isHomeImpl">
            // Register <xsl:value-of select="$javaFactoryClassname"/> HomeAllocator
            HomeAllocatorRegistry.register("<xsl:value-of select="$javaFactoryClassname"/>",
                                           new HomeAllocator("<xsl:value-of select="$javaFactoryClassname"/>"));

            <xsl:if test="$hasLoadBalancingServer='yes'">
               <xsl:variable name="lbFactoryClassname">
                  <xsl:call-template name="buildHomeImplClassname">
                        <xsl:with-param name="_homeImplNode" select="$lifecycleFrameworkNode/home-impl"/>
                        <xsl:with-param name="_prefix" select="'LB'"/>
                  </xsl:call-template>
               </xsl:variable>

               <xsl:variable name="javaLbFactoryClassname">
                  <xsl:call-template name="replaceCharsInString">
                     <xsl:with-param name="_stringIn" select="$lbFactoryClassname"/>
                     <xsl:with-param name="_charsIn" select="$xmlSep"/>
                     <xsl:with-param name="_charsOut" select="$javaSep"/>
                  </xsl:call-template>
               </xsl:variable>

            // Register <xsl:value-of select="$javaLbFactoryClassname"/> HomeAllocator
            HomeAllocatorRegistry.register("<xsl:value-of select="$javaLbFactoryClassname"/>",
                                           new HomeAllocator("<xsl:value-of select="$javaLbFactoryClassname"/>"));
            </xsl:if>
         </xsl:if>
         
         <!--
            Strategy selection.
         -->
         <xsl:choose>
            <!--
               ENTITY.
            -->
            <xsl:when test="$strategy = 'ENTITY'">
               <xsl:choose>
                  <xsl:when test="$isHomeImpl">
                     <!--
                        TODO: Add component server code here!
                     -->
                  </xsl:when>
                  <xsl:otherwise>
                     // *****************************************************************
                     // ****** <xsl:value-of select="$javaFactoryClassname"/>  ******
                     // ***************************************************************** 
                     fullNameDomainName = "";
                     fullFactoryName = "";
                     threadingPolicy = null;
                     i = 0;

                     <xsl:if test="@occurrences > 0">
                        try {
                            fullNameDomainName =
                                xmlData.getFrameworkAttributeValue(
                                    "<xsl:value-of select="$frameworkIdref"/>",
                                    i,
                                    "domain-name");
                            fullFactoryName =
                                xmlData.getFrameworkAttributeValue(
                                    "<xsl:value-of select="$frameworkIdref"/>",
                                    i,
                                    "full-factory-name");
                            // TODO: If fullNameDomainName and fullFactoryName are empty
                            // use default name domain
                   
                        <xsl:choose>
                           <xsl:when test="$lifecycleFrameworkNode/*/@threading = 'SERIALIZE'">
                              threadingPolicy = new SingleThreadPolicy();
                           </xsl:when>
                           <xsl:when test="$lifecycleFrameworkNode/*/@threading = 'MULTITHREAD'">
                              String[] values =
                                  xmlData.getFrameworkMultithreadAttribute(
                                      "<xsl:value-of select="$frameworkIdref"/>",
                                      i);
                              String xmlThreadingPolicy = values[0];
                              String xmlThreadingAttribute = values[1];
                              String xmlThreadingAttributeValue = values[2];
                              
                              if (!xmlThreadingPolicy.equals("serialize")) {
                                  threadingPolicy = new SingleThreadPolicy();
                              } else { // threadingPolicy must be initialized
                                  threadingPolicy = new ThreadPoolPolicy(5);
                              }
                           </xsl:when>
                        </xsl:choose>
                        
                        <![CDATA[
                           } catch (NumberFormatException nfe) {
                               processImpl.setCdmwInitExceptionRaised();
                               throw new org.omg.CORBA.BAD_PARAM(
                                   ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                                   org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                           } catch (XMLErrorException xee) {
                               processImpl.setCdmwInitExceptionRaised();
                               // TODO: Add trace logs here
                               throw new org.omg.CORBA.BAD_PARAM(
                                   ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                                   org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                           } catch (BadParameterException bpe) {
                               processImpl.setCdmwInitExceptionRaised();
                               // TODO: Add trace logs here
                               throw new org.omg.CORBA.BAD_PARAM(
                                   ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                                   org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                           }
                        ]]>
                        
                        try {
                            // Get the factory Allocator
                            // TODO: Use a user-provided name rather than a naming convention!
                            <xsl:value-of select="$javaFactoryClassname"/>Allocator allocator = 
                                <xsl:value-of select="$javaFactoryClassname"/>Allocator.theAllocator;
                        
                            NamedObjectsFactoryInitUtil
                                .setupNamedObjectFactory(
                                    allocator,
                                    orb,
                                    factoriesPOA,
                                    repository,
                                    rootContext,
                                    fullNameDomainName,
                                    fullFactoryName,
                                    "", // TODO:
                                    threadingPolicy,
                                    args);

                        <![CDATA[
                        } catch (BadParameterException bpe) {
                            processImpl.setCdmwInitExceptionRaised();
                            throw new org.omg.CORBA.BAD_PARAM(
                                ExceptionMinorCodes.BAD_PARAMLifeCycleFrameworkInit,
                                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                        } catch (InternalErrorException iie) {
                            processImpl.setCdmwInitExceptionRaised();
                            throw new org.omg.CORBA.INTERNAL(
                                ExceptionMinorCodes.INTERNALLifeCycleFrameworkError,
                                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                        }
                        ]]>
                     </xsl:if>
                  </xsl:otherwise>
               </xsl:choose>
            </xsl:when>
            <!--
               UNMANAGED_ENTITY.
            -->
            <xsl:when test="$strategy = 'UNMANAGED_ENTITY'">
               // *****************************************************************
               // ****** <xsl:value-of select="$javaFactoryClassname"/>  ******
               // *****************************************************************
               fullNameDomainName = "";
               fullFactoryName = "";
               threadingPolicy = null;
               i = 0;

               <xsl:if test="@occurrences > 0">
                  try {
                      fullNameDomainName =
                          xmlData.getFrameworkAttributeValue(
                              "<xsl:value-of select="$frameworkIdref"/>",
                              i,
                              "domain-name");
                      fullFactoryName =
                          xmlData.getFrameworkAttributeValue(
                              "<xsl:value-of select="$frameworkIdref"/>",
                              i,
                              "full-factory-name");
                      // TODO: If fullNameDomainName and fullFactoryName are empty
                      // use default name domain
             
                  <xsl:choose>
                     <xsl:when test="$lifecycleFrameworkNode/*/@threading = 'SERIALIZE'">
                        threadingPolicy = new SingleThreadPolicy();
                     </xsl:when>
                     <xsl:when test="$lifecycleFrameworkNode/*/@threading = 'MULTITHREAD'">
                        String[] values =
                            xmlData.getFrameworkMultithreadAttribute(
                                "<xsl:value-of select="$frameworkIdref"/>",
                                i);
                        String xmlThreadingPolicy = values[0];
                        String xmlThreadingAttribute = values[1];
                        String xmlThreadingAttributeValue = values[2];
                        
                        if (!xmlThreadingPolicy.equals("serialize")) {
                            threadingPolicy = new SingleThreadPolicy();
                        } else { // threadingPolicy must be initialized
                            threadingPolicy = new ThreadPoolPolicy(5);
                        }

                     </xsl:when>
                  </xsl:choose>
                  
                  <![CDATA[
                     } catch (NumberFormatException nfe) {
                         processImpl.setCdmwInitExceptionRaised();
                         throw new org.omg.CORBA.BAD_PARAM(
                             ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                             org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                     } catch (XMLErrorException xee) {
                         processImpl.setCdmwInitExceptionRaised();
                         // TODO: Add trace logs here
                         throw new org.omg.CORBA.BAD_PARAM(
                             ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                             org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                     } catch (BadParameterException bpe) {
                         processImpl.setCdmwInitExceptionRaised();
                         // TODO: Add trace logs here
                         throw new org.omg.CORBA.BAD_PARAM(
                             ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                             org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                     }
                  ]]>
                  
                  try {
                      // Get the factory Allocator
                      // TODO: Use a user-provided name rather than a naming convention!
                      <xsl:value-of select="$javaFactoryClassname"/>Allocator allocator = 
                          <xsl:value-of select="$javaFactoryClassname"/>Allocator.theAllocator;
                        
                      NamedObjectsFactoryInitUtil
                          .setupNamedObjectFactory(
                              allocator,
                              orb,
                              factoriesPOA,
                              repository,
                              rootContext,
                              fullNameDomainName,
                              fullFactoryName,
                              "", // TODO:
                              threadingPolicy,
                              args);

                  <![CDATA[
                  } catch (BadParameterException bpe) {
                      processImpl.setCdmwInitExceptionRaised();
                      throw new org.omg.CORBA.BAD_PARAM(
                          ExceptionMinorCodes.BAD_PARAMLifeCycleFrameworkInit,
                          org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                  } catch (InternalErrorException iie) {
                      processImpl.setCdmwInitExceptionRaised();
                      throw new org.omg.CORBA.INTERNAL(
                          ExceptionMinorCodes.INTERNALLifeCycleFrameworkError,
                          org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                  }
                  ]]>
               </xsl:if>
            </xsl:when>
            <!--
               SESSION.
            -->
            <xsl:when test="$strategy = 'SESSION'">
               <xsl:choose>
                  <xsl:when test="$isHomeImpl">
                     <!--
                        TODO: Add component server code here!
                     -->
                  </xsl:when>
                  <xsl:otherwise>
                     // *****************************************************************
                     // ****** <xsl:value-of select="$javaFactoryClassname"/>  ******
                     // *****************************************************************
                     fullFactoryName = "";
                     threadingPolicy = null;
                     i = 0;
                     
                     <xsl:if test="@occurrences > 0">
                        try {
                            fullFactoryName =
                                xmlData.getFrameworkAttributeValue(
                                    "<xsl:value-of select="$frameworkIdref"/>",
                                    i,
                                    "full-factory-name");
                                                  
                        <xsl:choose>
                           <xsl:when test="$lifecycleFrameworkNode/*/@threading = 'SERIALIZE'">
                              threadingPolicy = new SingleThreadPolicy();
                           </xsl:when>
                           <xsl:when test="$lifecycleFrameworkNode/*/@threading = 'MULTITHREAD'">
                              String[] values =
                                  xmlData.getFrameworkMultithreadAttribute(
                                      "<xsl:value-of select="$frameworkIdref"/>",
                                      i);
                              String xmlThreadingPolicy = values[0];
                              String xmlThreadingAttribute = values[1];
                              String xmlThreadingAttributeValue = values[2];
                              
                              if (!xmlThreadingPolicy.equals("serialize")) {
                                  threadingPolicy = new SingleThreadPolicy();
                              } else { // threadingPolicy must be initialized
                                  threadingPolicy = new ThreadPoolPolicy(5);
                              }

                           </xsl:when>
                        </xsl:choose>

                        <![CDATA[
                           } catch (NumberFormatException nfe) {
                               processImpl.setCdmwInitExceptionRaised();
                               throw new org.omg.CORBA.BAD_PARAM(
                                   ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                                   org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                           } catch (XMLErrorException xee) {
                               processImpl.setCdmwInitExceptionRaised();
                               throw new org.omg.CORBA.BAD_PARAM(
                                   ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                                   org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                           } catch (BadParameterException bpe) {
                               processImpl.setCdmwInitExceptionRaised();
                               throw new org.omg.CORBA.BAD_PARAM(
                                   ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                                   org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                           }
                        ]]>

                        try {
                            // Get the factory Allocator
                            // TODO: Use a user-provided name rather than a naming convention!
                            <xsl:value-of select="$javaFactoryClassname"/>Allocator allocator = 
                                <xsl:value-of select="$javaFactoryClassname"/>Allocator.theAllocator;
                        
                            AnonymousObjectsFactoryInitUtil
                                .setupAnonymousObjectFactory(
                                    allocator,
                                    orb,
                                    factoriesPOA,
                                    repository,
                                    rootContext,
                                    fullFactoryName,
                                    "", // TODO:
                                    threadingPolicy, 
                                    args);
                         <![CDATA[
                           } catch (BadParameterException bpe) {
                               processImpl.setCdmwInitExceptionRaised();
                               // TODO: Add trace logs here
                               throw new org.omg.CORBA.BAD_PARAM(
                                   ExceptionMinorCodes.BAD_PARAMLifeCycleFrameworkInit,
                                   org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                           } catch (InternalErrorException iie) {
                               processImpl.setCdmwInitExceptionRaised();
                               // TODO: Add trace logs here
                               throw new org.omg.CORBA.INTERNAL(
                                   ExceptionMinorCodes.INTERNALLifeCycleFrameworkError,
                                   org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                           }
                         ]]>
                     </xsl:if>
                  </xsl:otherwise>
               </xsl:choose>
            </xsl:when>
            <!--
               SERVICE.
            -->
            <xsl:when test="$strategy = 'SERVICE'">
               <xsl:choose>
                  <xsl:when test="$isHomeImpl">
                     <!--
                        TODO: Add component server code here!
                     -->
                  </xsl:when>
                  <xsl:otherwise>
                     // *****************************************************************
                     // ****** <xsl:value-of select="$javaFactoryClassname"/>  ******
                     // *****************************************************************
                     fullFactoryName = "";
                     threadingPolicy = null;
                     i = 0;
                     
                     <xsl:if test="@occurrences > 0">
                        try {
                            fullFactoryName =
                                xmlData.getFrameworkAttributeValue(
                                    "<xsl:value-of select="$frameworkIdref"/>",
                                    i,
                                    "full-factory-name");
                                                  
                        <xsl:choose>
                           <xsl:when test="$lifecycleFrameworkNode/*/@threading = 'SERIALIZE'">
                              threadingPolicy = new SingleThreadPolicy();
                           </xsl:when>
                           <xsl:when test="$lifecycleFrameworkNode/*/@threading = 'MULTITHREAD'">
                              String[] values =
                                  xmlData.getFrameworkMultithreadAttribute(
                                      "<xsl:value-of select="$frameworkIdref"/>",
                                      i);
                              String xmlThreadingPolicy = values[0];
                              String xmlThreadingAttribute = values[1];
                              String xmlThreadingAttributeValue = values[2];
                              
                              if (!xmlThreadingPolicy.equals("serialize")) {
                                  threadingPolicy = new SingleThreadPolicy();
                              } else { // threadingPolicy must be initialized
                                  threadingPolicy = new ThreadPoolPolicy(5);
                              }

                           </xsl:when>
                        </xsl:choose>

                        <![CDATA[
                           } catch (NumberFormatException nfe) {
                               processImpl.setCdmwInitExceptionRaised();
                               throw new org.omg.CORBA.BAD_PARAM(
                                   ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                                   org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                           } catch (XMLErrorException xee) {
                               processImpl.setCdmwInitExceptionRaised();
                               throw new org.omg.CORBA.BAD_PARAM(
                                   ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                                   org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                           } catch (BadParameterException bpe) {
                               processImpl.setCdmwInitExceptionRaised();
                               throw new org.omg.CORBA.BAD_PARAM(
                                   ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                                   org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                           }
                        ]]>

                        try {
                            // Get the factory Allocator
                            // TODO: Use a user-provided name rather than a naming convention!
                            <xsl:value-of select="$javaFactoryClassname"/>Allocator allocator = 
                                <xsl:value-of select="$javaFactoryClassname"/>Allocator.theAllocator;
                        
                            AnonymousObjectsFactoryInitUtil
                                .setupAnonymousObjectFactory(
                                    allocator,
                                    orb,
                                    factoriesPOA,
                                    repository,
                                    rootContext,
                                    fullFactoryName,
                                    "", // TODO:
                                    threadingPolicy, 
                                    args);
                         <![CDATA[
                           } catch (BadParameterException bpe) {
                               processImpl.setCdmwInitExceptionRaised();
                               // TODO: Add trace logs here
                               throw new org.omg.CORBA.BAD_PARAM(
                                   ExceptionMinorCodes.BAD_PARAMLifeCycleFrameworkInit,
                                   org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                           } catch (InternalErrorException iie) {
                               processImpl.setCdmwInitExceptionRaised();
                               // TODO: Add trace logs here
                               throw new org.omg.CORBA.INTERNAL(
                                   ExceptionMinorCodes.INTERNALLifeCycleFrameworkError,
                                   org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                           }
                         ]]>
                     </xsl:if>
                  </xsl:otherwise>
               </xsl:choose>
            </xsl:when>
            <xsl:when test="$strategy = 'PROCESS'">
               <!--
                  TODO: Add component server code here!
               -->
            </xsl:when>
            <!--
               USER_DEFINED.
            -->
            <xsl:when test="$strategy = 'USER_DEFINED'">
               <xsl:variable name="isNamed" select="$factoryNodes[@idl-interface = $lifecycleFrameworkNode/@factory-idref]/@kind = 'NAMED'"/>
               <xsl:variable name="namedString">
                  <xsl:choose>
                     <xsl:when test="$isNamed">
                        <xsl:text>Named</xsl:text>
                     </xsl:when>
                     <xsl:otherwise>
                        <xsl:text>Anonymous</xsl:text>
                     </xsl:otherwise>
                  </xsl:choose>
               </xsl:variable>
               <xsl:variable name="javaFactoryAllocatorClassname">
                  <xsl:call-template name="replaceCharsInString">
                     <xsl:with-param name="_stringIn" select="$lifecycleFrameworkNode/user-factory-impl/factory-allocator/@classname"/>
                     <xsl:with-param name="_charsIn" select="$xmlSep"/>
                     <xsl:with-param name="_charsOut" select="$javaSep"/>
                  </xsl:call-template>
               </xsl:variable>

               // *****************************************************************
               // ******                User defined factory                 ******
               // *****************************************************************
               fullFactoryName = "";;
               <xsl:if test="$isNamed">fullNameDomainName= "";</xsl:if>
               threadingPolicy = null;
               i = 0;

               <xsl:if test="@occurrences > 0">
                  try {
                  <xsl:choose>
                     <xsl:when test="$isNamed">
                        fullNameDomainName =
                          xmlData.getFrameworkAttributeValue(
                              "<xsl:value-of select="$frameworkIdref"/>",
                              i,
                              "domain-name");
                        fullFactoryName =
                          xmlData.getFrameworkAttributeValue(
                              "<xsl:value-of select="$frameworkIdref"/>",
                              i,
                              "full-factory-name");
                        // TODO: If fullNameDomainName and fullFactoryName are empty
                        // use default name domain
                     </xsl:when>
                     <xsl:otherwise>
                        fullFactoryName =
                          xmlData.getFrameworkAttributeValue(
                              "<xsl:value-of select="$frameworkIdref"/>",
                              i,
                              "full-factory-name");
                     </xsl:otherwise>
                  </xsl:choose>
                  
                  <xsl:choose>
                     <xsl:when test="$lifecycleFrameworkNode/*/@threading = 'SERIALIZE'">
                        threadingPolicy = new SingleThreadPolicy();
                     </xsl:when>
                     <xsl:when test="$lifecycleFrameworkNode/*/@threading = 'MULTITHREAD'">
                        String[] values =
                            xmlData.getFrameworkMultithreadAttribute(
                                "<xsl:value-of select="$frameworkIdref"/>",
                                i);
                        String xmlThreadingPolicy = values[0];
                        String xmlThreadingAttribute = values[1];
                        String xmlThreadingAttributeValue = values[2];
                        
                        if (!xmlThreadingPolicy.equals("serialize")) {
                            threadingPolicy = new SingleThreadPolicy();
                        } else { // threadingPolicy must be initialized
                            threadingPolicy = new ThreadPoolPolicy(5);
                        }

                     </xsl:when>
                  </xsl:choose>

                  <![CDATA[
                     } catch (NumberFormatException nfe) {
                         processImpl.setCdmwInitExceptionRaised();
                         throw new org.omg.CORBA.BAD_PARAM(
                             ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                             org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                     } catch (XMLErrorException xee) {
                         processImpl.setCdmwInitExceptionRaised();
                         throw new org.omg.CORBA.BAD_PARAM(
                             ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                             org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                     } catch (BadParameterException bpe) {
                         processImpl.setCdmwInitExceptionRaised();
                         throw new org.omg.CORBA.BAD_PARAM(
                             ExceptionMinorCodes.BAD_PARAMInvalidXMLInitData,
                             org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                     }
                  ]]>

                  try {
                      // Get the factory Allocator
                      // TODO: Use a user-provided name rather than a naming convention!
                      <xsl:value-of select="$javaFactoryAllocatorClassname"/> allocator = 
                          <xsl:value-of select="$javaFactoryAllocatorClassname"/>.theAllocator;
                        
                      <xsl:value-of select="$namedString"/>ObjectsFactoryInitUtil
                          .setup<xsl:value-of select="$namedString"/>ObjectFactory(
                              allocator,
                              orb,
                              factoriesPOA,
                              repository,
                              rootContext,
                              <xsl:if test="$isNamed">fullNameDomainName,</xsl:if>
                              fullFactoryName,
                              "", // TODO:
                              threadingPolicy, 
                              args);
                  <![CDATA[
                     } catch (BadParameterException bpe) {
                         processImpl.setCdmwInitExceptionRaised();
                         throw new org.omg.CORBA.BAD_PARAM(
                             ExceptionMinorCodes.BAD_PARAMLifeCycleFrameworkInit,
                             org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                     } catch (InternalErrorException iie) {
                         processImpl.setCdmwInitExceptionRaised();
                         throw new org.omg.CORBA.INTERNAL(
                             ExceptionMinorCodes.INTERNALLifeCycleFrameworkError,
                             org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                     }
                  ]]>
               </xsl:if>
            </xsl:when>
         </xsl:choose>
      </xsl:for-each>
      
     <xsl:if test="$hasHomeImpl='true'">
        <![CDATA[  
         // Get a Default Event Channel Factory
         com.thalesgroup.CdmwEvent.EventChannelFactory eventChannelFactory = null;
         ]]>

         <xsl:if test="$hasEventPorts = 'true'">
            <![CDATA[
            try {

        NamingInterface evtNi;

        // First see if a Default Event Channel factory is registered in the Admin Root Context
        try {
            org.omg.CosNaming.NamingContext adminRootContext =
                    RepositoryInterface.getRepository().resolve_root_context(
                            cdmw.common.Locations.ADMIN_ROOT_CONTEXT_ID);
            evtNi = new NamingInterface(adminRootContext); 

            String EVENT_CHANNEL_FACTORY_NAME;

            // Get the hostname.
            // Use of toString() before a resolve() to well form the string in the case of a hostname
            // with a dot.
            org.omg.CosNaming.NameComponent[] name = new org.omg.CosNaming.NameComponent[1];
            name[0] = new org.omg.CosNaming.NameComponent();
            name[0].id = cdmw.ossupport.OS.getHostname();
            name[0].kind = "";
            String to_return = "";

            try{
            EVENT_CHANNEL_FACTORY_NAME = cdmw.commonsvcs.naming.NameParser.toString(name); 
            } catch (cdmw.commonsvcs.naming.InvalidNameException e) {
            throw new CORBASystemExceptionWrapper(
                            new org.omg.CORBA.INTERNAL(
                    ExceptionMinorCodes.INTERNAL,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO));
            }

            EVENT_CHANNEL_FACTORY_NAME += "/" + cdmw.common.Locations.LOCAL_EVENT_SERVICE_NAME_DOMAIN;
            EVENT_CHANNEL_FACTORY_NAME += "/FACTORIES/DefaultEventChannelFactory";

            eventChannelFactory = com.thalesgroup.CdmwEvent.EventChannelFactoryHelper.narrow(
                   evtNi.resolve(EVENT_CHANNEL_FACTORY_NAME));

        } catch (org.omg.CosNaming.NamingContextPackage.NotFound nf) { 
            // Retrieve reference to the Default Event Channel factory 
            
            String evtFactoriesNd = 
            Locations.CDMW_EVENT_SERVICE_NAME_DOMAIN + "/FACTORIES";
            
            evtNi = RepositoryInterface.getDomainNamingInterface(evtFactoriesNd);
            
            String EVENT_CHANNEL_FACTORY_NAME = "DefaultEventChannelFactory";
            
            eventChannelFactory = com.thalesgroup.CdmwEvent.EventChannelFactoryHelper.narrow(
                                            evtNi.resolve(EVENT_CHANNEL_FACTORY_NAME));
        } catch (com.thalesgroup.CdmwNamingAndRepository.RepositoryPackage.NoRootContext nrc) {
            throw new CORBASystemExceptionWrapper(
                    new org.omg.CORBA.INTERNAL(
                        ExceptionMinorCodes.INTERNAL,
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO));
        }
            } catch (com.thalesgroup.CdmwNamingAndRepository.NoNameDomain nnd) {
                throw new CORBASystemExceptionWrapper(
                    new org.omg.CORBA.BAD_PARAM(
                        ExceptionMinorCodes.BAD_PARAMNameDomainNotFound,
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO),
                    Locations.CDMW_EVENT_SERVICE_NAME_DOMAIN + "/FACTORIES is not a NameDomain!");
            } catch (com.thalesgroup.CdmwNamingAndRepository.InvalidName in) {
                throw new CORBASystemExceptionWrapper(
                    new org.omg.CORBA.INTERNAL(
                        ExceptionMinorCodes.INTERNAL,
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO));
            } catch (org.omg.CosNaming.NamingContextPackage.NotFound nf) {
                throw new CORBASystemExceptionWrapper(
                    new org.omg.CORBA.INTERNAL(
                        ExceptionMinorCodes.INTERNAL,
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO));
            } catch (org.omg.CosNaming.NamingContextPackage.CannotProceed cp) {
                throw new CORBASystemExceptionWrapper(
                    new org.omg.CORBA.INTERNAL(
                        ExceptionMinorCodes.INTERNAL,
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO));
            } catch (org.omg.CosNaming.NamingContextPackage.InvalidName in) {
                throw new CORBASystemExceptionWrapper(
                    new org.omg.CORBA.INTERNAL(
                        ExceptionMinorCodes.INTERNAL,
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO));
            } catch (org.omg.CORBA.SystemException se) {
                throw new CORBASystemExceptionWrapper(se);
            }
            ]]>
         </xsl:if>
     
         <![CDATA[  
         try {
             // Create an Object registration instance
             // get the admin root context
             cdmw.logging.Logger.CDMW_DEBUG(
                 InitUtils.Get_Logger(), 
                 "Get the admin root context",
                 new Throwable());
             com.thalesgroup.CdmwNamingAndRepository.Repository repository_ref =
                 cdmw.namingandrepository.RepositoryInterface.getRepository();
             org.omg.CosNaming.NamingContext adminRootCtx =
                 repository_ref.resolve_root_context(
                         cdmw.common.Locations.ADMIN_ROOT_CONTEXT_ID);
             cdmw.commonsvcs.naming.NamingInterface ni =
                 new cdmw.commonsvcs.naming.NamingInterface(adminRootCtx);

             // bind the new context where component shall be registered
             org.omg.CosNaming.NameComponent[] componentNamePath =
                 new org.omg.CosNaming.NameComponent[5];
             componentNamePath[0] = new org.omg.CosNaming.NameComponent(cdmw.ossupport.OS.getHostname(), "");
             componentNamePath[1] = new org.omg.CosNaming.NameComponent(applicationName, "");
             componentNamePath[2] = new org.omg.CosNaming.NameComponent(processName, "");
             componentNamePath[3] = new org.omg.CosNaming.NameComponent("CCM", "");
             componentNamePath[4] = new org.omg.CosNaming.NameComponent("Components", "");

             cdmw.logging.Logger.CDMW_DEBUG(
                 InitUtils.Get_Logger(), 
                 "Bind component context",
                 new Throwable());
             java.lang.String componentPath = ni.toString(componentNamePath);
             cdmw.commonsvcs.naming.NamingInterface niComponent =
                 ni.newContext();
             ni.rebindContext(componentPath, niComponent, true);

             cdmw.logging.Logger.CDMW_DEBUG(
                 InitUtils.Get_Logger(), 
                 "Create object_registration",
                 new Throwable());
             org.omg.CosNaming.NamingContext nc = niComponent.getNamingContext();
             com.thalesgroup.CdmwLifeCycle.ObjectRegistration objectRegistration =
                 new cdmw.lifecycle.DefaultObjectRegistration(nc, false);

             // Register Managed component naming context in the orb
             // to make it available 
             org.omg.CORBA_2_5.ORB orb25 = (org.omg.CORBA_2_5.ORB) orb;
             orb25.register_initial_reference("ManagedComponentNamingContext", nc);


             // Create the ComponentServer
             com.thalesgroup.CdmwCcmComponentServer.ComponentServer theComponentServer = 
                 cdmw.ccm.componentserver.InitUtils.createComponentServer(
                     orb,
                     cdmwRootPOA,
                     eventChannelFactory,
                     objectRegistration,
                     processName,
                     applicationName,
                     args);

             // Register it as cdmw.common.locations.ASSEMBLY_AND_DEPLOYMENT_NAME_DOMAIN/
             //     processName@applicationName.ComponentServer
             String id = processName + "@" + applicationName;
             String kind = "ComponentServer";
             org.omg.CosNaming.NameComponent[] name = new org.omg.CosNaming.NameComponent[1];
             name[0] = new org.omg.CosNaming.NameComponent(id, kind);
             String key = NamingInterface.toString(name);

             com.thalesgroup.CdmwNamingAndRepository.NameDomain domain =
                 RepositoryInterface.getDomain(Locations.ASSEMBLY_AND_DEPLOYMENT_NAME_DOMAIN);

             try {
                 domain.register_new_factory(key, theComponentServer);
             } catch (com.thalesgroup.CdmwNamingAndRepository.NameDomainPackage.AlreadyExists ae) {
                 // Re-register it again.
                 domain.release_name(key);
                 domain.register_new_factory(key, theComponentServer);
             } 
         } catch (com.thalesgroup.CdmwNamingAndRepository.NoNameDomain nnd) {
             throw new CORBASystemExceptionWrapper(
                 new org.omg.CORBA.BAD_PARAM(
                     ExceptionMinorCodes.BAD_PARAMNameDomainNotFound,
                     org.omg.CORBA.CompletionStatus.COMPLETED_NO),
                 Locations.ASSEMBLY_AND_DEPLOYMENT_NAME_DOMAIN + " is not a NameDomain!");
         } catch (NoRootContext nrc) {
             throw new org.omg.CORBA.INTERNAL(
                 ExceptionMinorCodes.INTERNALCdmwRepositoryError,
                 org.omg.CORBA.CompletionStatus.COMPLETED_NO);
         } catch (org.omg.CosNaming.NamingContextPackage.NotFound nf) {
             throw new CORBASystemExceptionWrapper(
                 new org.omg.CORBA.INTERNAL(
                     ExceptionMinorCodes.INTERNAL,
                     org.omg.CORBA.CompletionStatus.COMPLETED_NO));
         } catch (org.omg.CosNaming.NamingContextPackage.AlreadyBound ab) {
             throw new CORBASystemExceptionWrapper(
                 new org.omg.CORBA.INTERNAL(
                     ExceptionMinorCodes.INTERNAL,
                     org.omg.CORBA.CompletionStatus.COMPLETED_NO));
         } catch (org.omg.CosNaming.NamingContextPackage.CannotProceed cp) {
             throw new CORBASystemExceptionWrapper(
                 new org.omg.CORBA.INTERNAL(
                     ExceptionMinorCodes.INTERNAL,
                     org.omg.CORBA.CompletionStatus.COMPLETED_NO));
         } catch (org.omg.CosNaming.NamingContextPackage.InvalidName cp) {
             throw new CORBASystemExceptionWrapper(
                 new org.omg.CORBA.INTERNAL(
                     ExceptionMinorCodes.INTERNAL,
                     org.omg.CORBA.CompletionStatus.COMPLETED_NO));
         } catch (org.omg.CORBA.ORBPackage.InvalidName e) {
             // shouldn't happen if ORB is standard
             throw new org.omg.CORBA.INTERNAL(
                 ExceptionMinorCodes.INTERNAL,
                 org.omg.CORBA.CompletionStatus.COMPLETED_NO);
           } catch (com.thalesgroup.CdmwNamingAndRepository.NameDomainPackage.AlreadyExists ae) {
             throw new CORBASystemExceptionWrapper(
                 new org.omg.CORBA.INTERNAL(
                     ExceptionMinorCodes.INTERNAL,
                     org.omg.CORBA.CompletionStatus.COMPLETED_NO));
         } catch (com.thalesgroup.CdmwNamingAndRepository.InvalidName in) {
             throw new CORBASystemExceptionWrapper(
                 new org.omg.CORBA.INTERNAL(
                     ExceptionMinorCodes.INTERNAL,
                     org.omg.CORBA.CompletionStatus.COMPLETED_NO));
         } catch(cdmw.commonsvcs.naming.InvalidNameException ine) {
             throw new CORBASystemExceptionWrapper(
                 new org.omg.CORBA.INTERNAL(
                     ExceptionMinorCodes.INTERNAL,
                     org.omg.CORBA.CompletionStatus.COMPLETED_NO));
         } catch (com.thalesgroup.CdmwNamingAndRepository.NameDomainPackage.NotRegistered nr) {
             throw new CORBASystemExceptionWrapper(
                 new org.omg.CORBA.INTERNAL(
                     ExceptionMinorCodes.INTERNAL,
                     org.omg.CORBA.CompletionStatus.COMPLETED_NO));
         } catch (org.omg.CORBA.SystemException se) {
             throw new CORBASystemExceptionWrapper(se);
         }
         ]]>
      </xsl:if>
   </xsl:if>
   
   <![CDATA[
      // Discard requests until run order comes from the platform management
      try {
          poaManager.discard_requests(false);
      } catch (
          org.omg.PortableServer.POAManagerPackage.AdapterInactive ai) {
          // TODO: Add trace logs here
          throw new org.omg.CORBA.INTERNAL(
              ExceptionMinorCodes.INTERNALORBAdapterInactive,
              org.omg.CORBA.CompletionStatus.COMPLETED_NO);
      }
   
      // advise processImpl that cdmwInit is done
      processImpl.setCdmwInitDone();
    ]]>

      } catch (CORBASystemExceptionWrapper csew) {
   <xsl:choose>
      <xsl:when test="$hasTrace">
      <![CDATA[
          String domain = TraceDomains.CDMW_INIT;
          short level = TraceLevels.FTL;

          if (FlushAreaMngr.isToBeTraced(domain, level)) {
              Stream stream = Stream.getInstance();
              // filename and line number are avalaible in J2SDK 1.4
              stream.setLine((short) 0);
              stream.setFile("unknown");
              stream.setUserDomain(domain);
              stream.setUserLevel(level);
              stream.doFlush();
          } else { // In case there is a problem in the trace library itself!
              System.err.println(
                  domain + " : Fatal Error : " + csew.toString());
          }
      ]]>
      </xsl:when>
      <xsl:otherwise>
      <![CDATA[
          System.err.println(
              "cdmwInit: Fatal Error: " + csew.toString());
      ]]>
      </xsl:otherwise>
   </xsl:choose>
   <![CDATA[
          csew.raiseSystemException();
      }
       
   }

   // CDMW Factory objects are deactivated at ORB shutdown.
   public static void cdmwCleanup(org.omg.CORBA.ORB orb) {
      Assert.check(orb != null);
   ]]>

      <xsl:if test="$hasHomeImpl='true'">

         // We should unbind "processname"/CCM/Components from AdminRootContext
         // here. But as ORB has just been shutdown in on_stop() operation,
         // no outgoing request are allowed (BAD_INV_ORDER or COMM_FAILURE received).
             
         // Unregister HomeAllocators
         <xsl:for-each select="$frameworkNodes">
            <xsl:variable name="frameworkIdref" select="@idref"/>

            <!--
               Get the matching node /cdmw/lifecycle-frameworks/lifecycle-framework.
            -->
            <xsl:variable name="lifecycleFrameworkNode" select="$lifecycleFrameworkNodes[@id = $frameworkIdref]"/>
            <xsl:variable name="isHomeImpl" select="boolean($lifecycleFrameworkNode/home-impl)"/>

            <xsl:if test="$isHomeImpl">
               <xsl:variable name="factoryClassname">
                  <xsl:call-template name="buildHomeImplClassname">
                     <xsl:with-param name="_homeImplNode" select="$lifecycleFrameworkNode/home-impl"/>
                  </xsl:call-template>
               </xsl:variable>

               HomeAllocatorRegistry.unregister("<xsl:value-of select="$factoryClassname"/>");
            </xsl:if>
         </xsl:for-each>
      </xsl:if>

   <xsl:if test="boolean($hasTrace) and ($hasBufferingEnabled='yes')">
      // Free all allocated memory and stop the trace service
      FlushAreaMngr.cleanup();
   </xsl:if>
   <xsl:if test="$hasTime">
      // Close Clock Service
      cdmw.clock.CdmwCosClock.close();
   </xsl:if>

   <xsl:if test="$hasPerformance">
      // Free all allocated memory and stop the performance service
      try {
         PerformanceMngr.cleanup();
      } catch (BadParameterException bpe) {
         // Do nothing (ignore)
         // TODO: Add trace logs here
      }
   </xsl:if>

   <xsl:if test="boolean($hasTrace) and not($hasBufferingEnabled='yes')">

   <![CDATA[

         // Release LogCollectorDelegate
         {
            cdmw.logging.LogManagerDelegate d__ = cdmw.logging.LogManager.Instance().getAlternateDelegate();
            if (d__!=null) {
               Assert.check(d__ instanceof cdmw.logging.LogManagerCollectorDelegate);
               cdmw.logging.LogManager.Instance().unregisterAlternateDelegate(d__);
               d__=null;
            }
         }
     ]]>
     </xsl:if>
 
      // Release all static object references held by RepositoryInterface
      RepositoryInterface.finish();
   }

   <!--
      The code generation ends here.
   -->

   <![CDATA[
    public static void cdmwPreORBInit(String[] args) 
    {
       // We create the set of properties required by the ORB
        cdmw.orbsupport.ORBSupport.setORBProperties();
   ]]>
   <xsl:choose>
   <xsl:when test="$hasIssuerInterceptor='yes'">
   <![CDATA[
            // Value of "command-issuer" tag in xml generation file set to 'yes' (default value): 
            // Creating client interceptor in order to set command issuer
            cdmw.platformmngt.PlatformInterface.createInterceptor();

            ]]>
   </xsl:when>
   <xsl:otherwise>
   <![CDATA[
            // Value of "comman-issuer" tag in xml generation file set to 'no': 
            //No interceptor initialised 
            ]]>
   </xsl:otherwise>
   </xsl:choose>
   
    <xsl:if test="$hasFaultTolerance">
        <![CDATA[
            // generate call to FTServiceInit.Init
            try {
                // only client FT is supported at this time
                cdmw.ftinit.FTServiceInit.Init(args, true);
            } catch (org.omg.CORBA.SystemException e) {
                System.err.println("CDMW_preORBInit: Fatal Error:");
                System.err.println("CORBA Exception : " + e);
                e.printStackTrace();
            }
        ]]>
    </xsl:if>
    <xsl:if test="$hasLoadBalancing">
      <![CDATA[
            // generate call for LBServiceInit.Init
            try {
               cdmw.lb.lbinit.LBServiceInit.init(args);
            } catch (org.omg.CORBA.SystemException e) {
               System.err.println("CDMW_preORBInit: Fatal Error:");
               System.err.println("CORBA Exception : " + e);
               e.printStackTrace();
            }
      ]]>
   </xsl:if>
   <xsl:if test="$hasWatchdog">
      <![CDATA[
         // *****************************************************************
         // *************  setup WatchdogServiceInit  **********
         // *****************************************************************
        try {
            cdmw.ft.watchdog.WatchdogServiceInit.init(args);
        } catch (org.omg.CORBA.SystemException  e) {
           try {
              System.out.println("CDMW_preORBInit: Fatal Error: \n");
              e.printStackTrace();
           } catch(Exception ex) {
               Assert.check(false);
           }
        }
      ]]>
   </xsl:if>
   <xsl:if test="$hasRequestFollowup='yes'">
      <![CDATA[
         // *****************************************************************
         // *************  setup RequestFollowupInit  **********
         // *****************************************************************
        try {
            cdmw.commonsvcs.requestfollowup.NDCServiceInit.init(args);
        } catch (org.omg.CORBA.SystemException  e) {
           try {
              System.out.println("CDMW_preORBInit: Fatal Error: \n");
              e.printStackTrace();
           } catch(Exception ex) {
               Assert.check(false);
           }
        }
      ]]>
   </xsl:if>
   <![CDATA[
   }
   ]]>
}

</xsl:template> <!-- end of template cdmw_interface_java.content -->

</xsl:stylesheet>


