/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <ConfAndPlug/cdmwinit/CdmwInterface.hpp>
#include <Foundation/common/String.hpp>


#include "Time/idllib/CosClockService.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtEntity.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtProcessCallback.skel.hpp"



#include "Foundation/commonsvcs/naming/NamingInterface.hpp"
#include "Repository/repositoryinterface/RepositoryInterface.hpp"
#include "Foundation/common/Options.hpp"
#include "Foundation/common/Locations.hpp"


#include "Foundation/ossupport/OS.hpp"
#include "testclockinit/CodeGenTimeTest.hpp"

CPPUNIT_TEST_SUITE_REGISTRATION( ClockInitGenTest);

ClockInitGenTest::ClockInitGenTest()
  throw()
{
}

ClockInitGenTest::~ClockInitGenTest()
  throw()
{
}


std::string NAMING_SERVICE_PROCESS  = "Naming_Service";
std::string NAMING_SERVICE_PORT     = "5000";
std::string REPOSITORY_PORT         = "5020";

Cdmw::OsSupport::OS::ProcessId  startNamingService()
{
        // Starting the Name Service
        Cdmw::OsSupport::OS::ProcessId idNaming; 

        std::cerr<< " >>>> Starting the Name Service ......... " << std::endl;
        
        idNaming = Cdmw::OsSupport::OS::create_process( NAMING_SERVICE_PROCESS, "-ORBEndpoint iiop://localhost:" + NAMING_SERVICE_PORT );
        std::cerr<< " >>>>  Name Service process started on port " << NAMING_SERVICE_PORT << std::endl;

        return idNaming;

}                               

Cdmw::OsSupport::OS::ProcessId  startSimRepository()
{
        // We allow the user to specify a different port number for the Repository
        Cdmw::OsSupport::OS::ProcessId idRepository; 
        std::string namingCorbaloc =   " -ORBInitRef NameService=corbaloc:iiop:localhost:" + NAMING_SERVICE_PORT + "/NameService";

        // Starting the Repository Simulator
        std::string simRepositoryArgs = "-ORBEndpoint iiop://localhost:" + REPOSITORY_PORT + namingCorbaloc;
        simRepositoryArgs += " -DOM CDMW";
        simRepositoryArgs += " -DOM ";
        simRepositoryArgs += Cdmw::Common::Locations::CDMW_SERVICES_NAME_DOMAIN;
        simRepositoryArgs += "/";
        simRepositoryArgs += Cdmw::Common::Locations::TRACE_COLLECTORS_NAME_DOMAIN;
        simRepositoryArgs += " -ROOT ";
        simRepositoryArgs += Cdmw::Common::Locations::ADMIN_ROOT_CONTEXT_ID;
        
        idRepository = Cdmw::OsSupport::OS::create_process ("cdmw_simulated_repository",
                                           simRepositoryArgs);

        return idRepository;
}

//std::string get_process_callback(CORBA::ORB_ptr orb)
CdmwPlatformMngt::ProcessCallback_ptr get_process_callback(CORBA::ORB_ptr orb)
{
    std::string callbackIOR;
    const char* PROCESS_CALLBACK_BINDING_NAME = "CDMW.I/process-callback.simulated";
    CORBA::Object_var nobj = CORBA::Object::_nil();

    // Get naming service
    try
    {
      nobj = orb -> resolve_initial_references("NameService");
    }
    catch(const CORBA::ORB::InvalidName&)
    {
      std::cerr<< "Can't resolve `NameService'" << std::endl;
    }

    if(CORBA::is_nil(nobj.in()))
    {
      std::cerr<< "`NameService' is a nil object reference" << std::endl;
    }

    CosNaming::NamingContextExt_var nc =
       CosNaming::NamingContextExt::_narrow(nobj.in());

    if(CORBA::is_nil(nc.in()))
    {
       std::cerr<< "`NameService' is not a NamingContextExt object reference" << std::endl;
    }

    // Get reference to the ProcessCallback object from naming
    CdmwPlatformMngt::ProcessCallback_var callback = CdmwPlatformMngt::ProcessCallback::_nil();
    try
    {
      Cdmw::CommonSvcs::Naming::NamingInterface ni(nc.in());
      CORBA::Object_var obj__ = ni.resolve(PROCESS_CALLBACK_BINDING_NAME);
      callback = CdmwPlatformMngt::ProcessCallback::_narrow(obj__.in());
      if (CORBA::is_nil(callback.in()))
      {
    std::cerr<< "Couldn't retrieve reference of the ProcessCallback"
             << "\n(file:" << __FILE__ << " line:" << __LINE__ << ")" << std::endl;
      }
    } catch (...) {
      std::cerr<< "Couldn't retrieve reference of the ProcessCallback"
               << "\n(file:" << __FILE__ << " line:" << __LINE__ << ")" << std::endl;
    }

    //callbackIOR = orb->object_to_string(callback.in());
    //return callbackIOR;
    return callback._retn();
}

void ClockInitGenTest::do_tests()
{
//int main( int argc, char* argv[] )
//{

    try
    {
        Cdmw::OsSupport::OS::ProcessId idNaming, idRepository; 
        char** argv_tmp = NULL;
        int argc_tmp(2);
        std::string str_opt_( "" );


        Cdmw::OrbSupport::StrategyList strategyList;
        strategyList.add_OrbThreaded(); // Mandatory in OrbSupport::ORB_init
        strategyList.add_PoaThreadPerConnection();
        strategyList.add_multicast ();

        str_opt_ += " -ORBInitRef NameService=corbaloc:iiop:localhost:"
                    + NAMING_SERVICE_PORT + "/NameService";

  
        argv_tmp = Cdmw::Common::String::to_char_array(Cdmw::Common::String::to_strings( str_opt_ ) );

        // Initialises the ORB
        CORBA::ORB_var orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc_tmp, argv_tmp, strategyList);

        // Get the root POA
        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj.in());


    idNaming  = startNamingService();
        Cdmw::OsSupport::OS::sleep(3000);
    idRepository  = startSimRepository();

        Cdmw::OsSupport::OS::sleep(3000);

        PortableServer::POAManager_var poaMgr = rootPOA->the_POAManager();
        poaMgr->activate();

        CdmwPlatformMngt::ProcessCallback_var callback = CdmwPlatformMngt::ProcessCallback::_nil(); 
        callback = get_process_callback(orb.in());
        std::string callbackIOR = orb->object_to_string(callback.in());

                  
        char** CDMW_init_argv = 0;
        int CDMW_init_argc = 0;
        std::string str_opt( "" );

        //for (int i=0; i<argc; i++)
        //    str_opt += std::string( argv_tmp[i] ) + " ";
        str_opt += std::string( "./testclockinit --CdmwXMLFile=TestClockInitConf.xml ");
        str_opt +=  Cdmw::Common::Options::CALLBACK_IOR_OPTION;
        str_opt +=   "=";
        str_opt += callbackIOR;

        CDMW_init_argc += 3;
        CDMW_init_argv = Cdmw::Common::String::to_char_array(
            Cdmw::Common::String::to_strings( str_opt ) );



          try
          {
            TEST_INFO("Try CDMW_init");
            Cdmw::CdmwInit::CDMW_init(orb.in(), CDMW_init_argc, CDMW_init_argv);
            CPPUNIT_ASSERT(true);
          }
          catch(const CORBA::SystemException &ex)
          {
            CPPUNIT_ASSERT(false);
          }

        // retrieve a clock.
        TEST_INFO("retrieve a clock.");

        CORBA::Object_var clock_obj = orb->resolve_initial_references ("LocalClock");
        CosClockService::Clock_ptr clock =  CosClockService::Clock::_narrow (clock_obj.in());
        if (CORBA::is_nil (clock))
        {
           std::cerr << "Clock Service not available! " << std::endl;
            CPPUNIT_ASSERT(false);
        }

        
        TimeBase::TimeT curr_time = clock->current_time();

        std::cerr << "Clock Service : current Time " << curr_time << std::endl;
        
        Cdmw::OsSupport::OS::sleep(3000);
        TEST_INFO("Kill The repository and Naming service");
        Cdmw::OsSupport::OS::kill_process(idRepository);
        Cdmw::OsSupport::OS::kill_process(idNaming);
        TEST_INFO("Destroy the ORB");

        try
        {
          Cdmw::CdmwInit::CDMW_cleanup(orb.in());
          CPPUNIT_ASSERT(true);
        }
        catch(const CORBA::SystemException &ex)
        {
          CPPUNIT_ASSERT(false);
        }
        Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
        orb->destroy();
    }
    catch( const CORBA::Exception &e )
    {
        std::cerr << e._name() << std::endl;
    }
    catch( const Cdmw::Exception &e )
    {
        std::cerr << e.what() << std::endl;
    }
    catch(...)
    {
        std::cerr << "Uncaught exception" << std::endl;
    }
}


