/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <sstream> 
#include <Foundation/orbsupport/CORBA.hpp> 
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <ConfAndPlug/cdmwinit/CdmwInterface.hpp>
#include <Foundation/common/String.hpp>

#include "SystemMngt/idllib/CdmwPlatformMngtEntity.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtProcessCallback.skel.hpp"

#include "Foundation/commonsvcs/naming/NamingInterface.hpp"
#include "Repository/repositoryinterface/RepositoryInterface.hpp"
#include "Foundation/common/Options.hpp"
#include "Foundation/common/Locations.hpp"

#include "Foundation/ossupport/OS.hpp"
#include "testrecordinginit/RecordingInitTest.hpp"

CPPUNIT_TEST_SUITE_REGISTRATION(RecordingInitTest);

RecordingInitTest::RecordingInitTest() throw() {
}

RecordingInitTest::~RecordingInitTest() throw() {
}

std::string NAMING_SERVICE_PORT = "5010";
std::string REPOSITORY_PORT = "5020";


//std::string get_process_callback(CORBA::ORB_ptr orb)
CdmwPlatformMngt::ProcessCallback_ptr get_process_callback(CORBA::ORB_ptr orb) {
    std::string callbackIOR;
    const char* PROCESS_CALLBACK_BINDING_NAME =
            "CDMW.I/process-callback.simulated";
    CORBA::Object_var nobj = CORBA::Object::_nil();

    // Get naming service
    try
    {
        nobj = orb -> resolve_initial_references("NameService");
    }
    catch(const CORBA::ORB::InvalidName&)
    {
        std::cerr<< "Can't resolve `NameService'" << std::endl;
    }

    if (CORBA::is_nil(nobj.in())) {
        std::cerr<< "`NameService' is a nil object reference" << std::endl;
    }

    CosNaming::NamingContextExt_var nc =
            CosNaming::NamingContextExt::_narrow(nobj.in());

    if (CORBA::is_nil(nc.in())) {
        std::cerr<< "`NameService' is not a NamingContextExt object reference"
                << std::endl;
    }

    // Get reference to the ProcessCallback object from naming
    CdmwPlatformMngt::ProcessCallback_var callback =
            CdmwPlatformMngt::ProcessCallback::_nil();
    try
    {
        Cdmw::CommonSvcs::Naming::NamingInterface ni(nc.in());
        CORBA::Object_var obj__ = ni.resolve(PROCESS_CALLBACK_BINDING_NAME);
        callback = CdmwPlatformMngt::ProcessCallback::_narrow(obj__.in());
        if (CORBA::is_nil(callback.in()))
        {
            std::cerr<< "Couldn't retrieve reference of the ProcessCallback"
            << "\n(file:" << __FILE__ << " line:" << __LINE__ << ")" << std::endl;
        }
    } catch (...) {
        std::cerr<< "Couldn't retrieve reference of the ProcessCallback"
        << "\n(file:" << __FILE__ << " line:" << __LINE__ << ")" << std::endl;
    }

    //callbackIOR = orb->object_to_string(callback.in());
    //return callbackIOR;
    return callback._retn();
}

void RecordingInitTest::do_tests() {

    CORBA::ORB_var orb =  Cdmw::TestUtils::CORBATestManager::instance()->get_ORB();
    
    try
    {
        //retrieve the callback IOR
        CdmwPlatformMngt::ProcessCallback_var callback = CdmwPlatformMngt::ProcessCallback::_nil();
        callback = get_process_callback(orb.in());
        std::string callbackIOR = orb->object_to_string(callback.in());

        char** CDMW_init_argv = 0;
        int CDMW_init_argc = 0;
        std::stringstream str_opt; 
        str_opt << "./unused_param ";
        str_opt << Cdmw::Common::Options::PROCESS_XML_FILE_OPTION  << "=TestRecordingInitConf.xml ";
        str_opt << Cdmw::Common::Options::CALLBACK_IOR_OPTION << "=" << callbackIOR << " ";
        str_opt << Cdmw::Common::Options::VALIDATE_XML_OPTION << "=yes ";
        
        std::cout << str_opt.str() << std::endl;
        
        CDMW_init_argc += 4;
        CDMW_init_argv = Cdmw::Common::String::to_char_array(
                Cdmw::Common::String::to_strings( str_opt.str() ) );

        try {
            TEST_INFO("Try CDMW_init");
            Cdmw::CdmwInit::CDMW_init(orb.in(), CDMW_init_argc, CDMW_init_argv);
            CPPUNIT_ASSERT(true);
        } catch(const CORBA::SystemException &ex) {
            CPPUNIT_ASSERT(false);
        } catch(...) {
            CPPUNIT_ASSERT(false);
        }
        

        TEST_INFO("Destroy the ORB");
        ///clean the test
        try
        {
            Cdmw::CdmwInit::CDMW_cleanup(orb.in());
            CPPUNIT_ASSERT(true);
        }
        catch(const CORBA::SystemException &ex)
        {
            CPPUNIT_ASSERT(false);
        }
        catch( ... ) 
        {
            CPPUNIT_ASSERT(false);
            std::cerr << "Uncaught exception" << std::endl;
            throw;
        }
        
        Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
        orb->destroy();
    }
    catch( const CORBA::Exception &e )
    {
        std::cerr << e._name() << std::endl;
    }
    catch( const Cdmw::Exception &e ) 
    {
        std::cerr << e.what() << std::endl;
    }
}

