/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.cdmwinit.test;

import cdmw.testutils.CORBATestManager;

import java.io.*;
import java.util.*;

import org.omg.CORBA.*;
import org.omg.CORBA.Object;
import org.omg.PortableServer.*;

import cdmw.common.Options;
import cdmw.common.Locations;
import cdmw.common.BadParameterException;
import cdmw.logging.*;
import static cdmw.logging.Logger.*;
import cdmw.namingandrepository.*;
import cdmw.orbsupport.*;
import cdmw.ossupport.*;
import cdmw.platformmngt.*;
import cdmw.trace.*;

import cdmw.tools.SimRepository;
import cdmw.tools.SimuUtils;

import cdmw.commonsvcs.naming.NamingInterface;
import cdmw.namingandrepository.RepositoryInterface;

import com.thalesgroup.CdmwNamingAndRepository.*;

public class MainTest {
    private static final int POA_THREAD_POOL_SIZE = 5;
    private final static String NAMING_SERVICE_PORT = "5010";
    private final static String REPOSITORY_PORT = "5020";
    private final static String REC_MGT_PORT = "59875";

    public static void initRepository(org.omg.CORBA.ORB orb) throws Exception {

        //
        // Get naming service
        //
        org.omg.CORBA.Object obj = null;
        try {
            obj = orb.resolve_initial_references("NameService");
        } catch (org.omg.CORBA.ORBPackage.InvalidName in) {
            System.out.println("Can't resolve `NameService'");
            throw in;
        }

        if (obj == null) {
            System.out.println("`NameService' is a nil object reference");
            throw new org.omg.CORBA.TRANSIENT(
                    ExceptionMinorCodes.TRANSIENTCannotReachRepository,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        org.omg.CosNaming.NamingContext nc = org.omg.CosNaming.NamingContextHelper
                .narrow(obj);

        if (nc == null) {
            System.out
                    .println("`NameService' is not a NamingContextExt object reference");
            throw new org.omg.CORBA.TRANSIENT(
                    ExceptionMinorCodes.TRANSIENTCannotReachRepository,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        String REPOSITORY_NAME = "CDMW.I/repository.simulated";

        // Get reference to the repository interface
        com.thalesgroup.CdmwNamingAndRepository.Repository rep = null;
        try {
            NamingInterface ni = new NamingInterface(nc);
            org.omg.CORBA.Object obj_ = ni.resolve(REPOSITORY_NAME);
            rep = com.thalesgroup.CdmwNamingAndRepository.RepositoryHelper
                    .narrow(obj_);
            if (rep == null) {
                System.out
                        .println("Couldn't retrieve reference of the Repository");
                throw new org.omg.CORBA.TRANSIENT(
                        ExceptionMinorCodes.TRANSIENTCannotReachRepository,
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }
        } catch (Exception e) {
            System.out.println("Couldn't retrieve reference of the Repository");
            throw new org.omg.CORBA.TRANSIENT(
                    ExceptionMinorCodes.TRANSIENTCannotReachRepository,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        try {
            RepositoryInterface.init(Locations.CDMW_SERVICES_NAME_DOMAIN, rep);
        } catch (Exception e) {
            System.out.println("Couldn't init Repository");
            throw e;
        }

    }

    public static void main(String[] args) {
        String applicationName = "";
        int retCode = 0;

        StrategyList strategy = new StrategyList();
        strategy.addORBThreaded();
        try {
            strategy.addPOAThreadPool(MainTest.POA_THREAD_POOL_SIZE);
        } catch (BadParameterException e) {
            e.printStackTrace();
        }

        ORB orb = null;

        try {

            String s = ORBUtils.NAMING_PORT_OPTION + "corbaloc::localhost:"
                    + NAMING_SERVICE_PORT + "/NameService "
                    + Options.SYSTEM_PORT_OPTION + "=" + NAMING_SERVICE_PORT;
            System.err.println("init ORB" + s);
            String[] serverArgs = s.split(" ");

            // initialise the orb
            orb = ORBSupport.ORBInit(serverArgs, strategy);

            SimuUtils.startSimu(System.out);
            OS.sleep(3000);

            // starting recording daemon
            String rec_daemon_args = "--debug_file=/tmp/recording.log " 
                + " --CdmwProcessPort=" + REC_MGT_PORT 
                + " --bind_to_ns"
                + " -ORBInitRef NameService=corbaloc::localhost:" + NAMING_SERVICE_PORT + "/NameService";
            System.err.println("Start Rec Daemon: " + rec_daemon_args);
            ThreadedProcess recDaemon = OS.createProcess("cdmw_recordingdaemon " + rec_daemon_args);
            
            //initialise the repository
            initRepository(orb);

            Object obj = orb.resolve_initial_references("RootPOA");
            POA rootPOA = POAHelper.narrow(obj);
            POAManager poaMng = rootPOA.the_POAManager();
            poaMng.activate();

            // The logger
            Logger logger = Logger.Get_Logger("TEST", "TEST");

            //start tests
            CORBATestManager testManager = CORBATestManager.instance();

            testManager.init("cdmw.cdmwinit.test.InitConfigurationTest", orb,
                    rootPOA);

            testManager.addTest(InitConfigurationTest.class);

            testManager.runTests();
            testManager.printSummary();

            //stop the system
            if (recDaemon != null) {
                System.out.print("Killing the Recording Daemon...");
                OS.killProcess(recDaemon);
            }
            
            System.out.println("stop simu");
            SimuUtils.stopSimu(System.err);

            
            if (orb != null) {
                System.out.println("cleanup ORB");
                // orb->shutdown(false) is done by the ProcessBehaviour
                ORBSupport.ORBCleanup(orb);
                orb.destroy();
            }

        } catch (SystemException e) {
            System.err.println("FAILURE : CORBA exception raised :"
                    + e.getMessage());
            e.printStackTrace();
            retCode = -1;
        } catch (java.lang.Exception e) {
            System.err.println("FAILURE : Exception raised" + "\r\n"
                    + e.getMessage());
            e.printStackTrace();
            retCode = -1;
        }

        System.exit(retCode);
    }

}
