/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.cdmwinit.test;

import org.junit.Test;
import cdmw.testutils.CORBATestManager;
import org.omg.CORBA.*;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import org.junit.Assert;

import cdmw.common.BadParameterException;
import cdmw.common.Options;
import cdmw.common.Locations;

import cdmw.tools.SimRepository;
import cdmw.tools.SimuUtils;

import cdmw.ossupport.OS;
import cdmw.ossupport.ThreadedProcess;
import cdmw.orbsupport.ExceptionMinorCodes;
import cdmw.orbsupport.ORBUtils;

import cdmw.cdmwinit.CdmwInitConfiguration;

import com.thalesgroup.CdmwPlatformMngt.ProcessCallback;
import com.thalesgroup.CdmwPlatformMngt.ProcessCallbackHelper;

import cdmw.commonsvcs.naming.NamingInterface;

import cdmw.namingandrepository.RepositoryInterface;


public class InitConfigurationTest {

    private final static String NAMING_SERVICE_PROCESS = "Naming_Service";
    private final static String NAMING_SERVICE_PORT = "5010";
    private final static String REPOSITORY_PORT = "5020";



    private static ProcessCallback getProcessCallback(ORB orb) throws TRANSIENT {

        org.omg.CORBA.Object obj = null;

        //
        // Get naming service
        //
        try {
            obj = orb.resolve_initial_references("NameService");
        } catch (org.omg.CORBA.ORBPackage.InvalidName in) {
            System.out.println("Can't resolve `NameService'");
            throw new TRANSIENT(
                    ExceptionMinorCodes.TRANSIENTCannotReachRepository,
                    CompletionStatus.COMPLETED_NO);
        }

        if (obj == null) {
            System.out.println("`NameService' is a nil object reference");
            throw new TRANSIENT(
                    ExceptionMinorCodes.TRANSIENTCannotReachRepository,
                    CompletionStatus.COMPLETED_NO);
        }

        org.omg.CosNaming.NamingContext nc = org.omg.CosNaming.NamingContextHelper
                .narrow(obj);

        if (nc == null) {
            System.out
                    .println("`NameService' is not a NamingContext object reference");
            throw new TRANSIENT(
                    ExceptionMinorCodes.TRANSIENTCannotReachRepository,
                    CompletionStatus.COMPLETED_NO);
        }

        String PROCESS_CALLBACK_BINDING_NAME = "CDMW.I/process-callback.simulated";

        // Get reference to the ApplicationAgent interface
        ProcessCallback callback = null;
        try {
            NamingInterface ni = new NamingInterface(nc);
            obj = ni.resolve(PROCESS_CALLBACK_BINDING_NAME);
            callback = ProcessCallbackHelper.narrow(obj);
            if (callback == null) {
                System.out
                        .println("Couldn't retrieve reference of the ProcessCallback");
                throw new TRANSIENT(
                        ExceptionMinorCodes.TRANSIENTCannotReachApplicationAgent,
                        CompletionStatus.COMPLETED_NO);
            }
        } catch (Exception e) {
            System.out
                    .println("Couldn't retrieve reference of the ProcessCallback");
            throw new TRANSIENT(
                    ExceptionMinorCodes.TRANSIENTCannotReachApplicationAgent,
                    CompletionStatus.COMPLETED_NO);
        }

        return callback;
    }

    
    @Test
    public void doTests() {

        try {
            CORBATestManager testManager = CORBATestManager.instance();
            ORB orb = testManager.getORB();
            
            ProcessCallback callback = getProcessCallback(orb);
            String CallbackIOR = orb.object_to_string(callback);

            try {
                String[] args = new String[3];
                args[0] = "--CdmwXMLFile=TestRequestFollowupInitConf.xml";
                args[1] = Options.CALLBACK_IOR_OPTION + "=" + CallbackIOR;
                args[2] = Options.VALIDATE_XML_OPTION + "=yes ";
                
                System.out.println("Init PlatformInterface");

                cdmw.cdmwinit.CdmwInterface.cdmwInit(orb, args, null,
                        "cdmw.generated.TestRequestFollowupInit");
                assertTrue(true);
            } catch (java.lang.Exception e) {
                e.printStackTrace();
                fail();
            }

            if (orb != null) {
                System.out.println("Cleanup PlatformInterface");
                cdmw.cdmwinit.CdmwInterface.cdmwCleanup(orb,
                        "cdmw.generated.TestRequestFollowupInit");
            }

        } catch (Exception e) {
            e.printStackTrace();
            fail(e.getMessage());
        }

    }

}
