/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// C++ include
#include <iostream>
#include <cstdlib>


// CDMW include
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/Locations.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>
#include <Foundation/commonsvcs/initutils/InitUtils.hpp>
#include <ConfAndPlug/cdmwinit/CdmwInterface.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CORBASystemExceptionWrapper.hpp>
#include <LifeCycle/lifecycleinit/InitUtils.hpp>
#include <SystemMngt/platforminterface/PlatformInterface.hpp>
#include <LifeCycle/lifecycle/FactoryAllocator.hpp>
#include <Event/eventsupport/TypedEventChannelFactory_impl.hpp>
#include <Event/eventsupport/EventChannelFactory_impl.hpp>
#include <ConfAndPlug/cdmwinit/InitUtils.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/LogManager.hpp>
#include "Foundation/common/Locations.hpp"
#include "Foundation/commonsvcs/naming/NamingInterface.hpp"
// CDMW idl include

// Application include
#include "eventchannelmanager/InitUtils.hpp"
#include "eventchannelmanager/EventChannelProfileManager_impl.hpp"
#include "Event/eventsupport/TypedEventChannelFactory_impl.hpp"
#include "Event/eventsupport/EventChannelFactory_impl.hpp"
#include "LifeCycle/lifecycleinit/FactoryRegistration.hpp"
#include "LifeCycle/lifecycle/DefaultObjectRegistration.hpp"
#include "eventchannelmanager/EvtLogger.hpp"

using namespace Cdmw::EventAdmin;
using namespace Cdmw::Common;
using namespace Cdmw::OrbSupport;
using namespace Cdmw::NamingAndRepository;
using namespace Cdmw::PlatformMngt;

namespace
{
    const int POA_THREAD_POOL_SIZE = 5;    
    const char* CCMProfileManager = "--CCMProfileManager";
    
//     void exceptionHandler(void)
//     {
//         std::cout << "Local Event Channel Manager => Unexpected exception raised, file: " << __FILE__ << ", line: " << __LINE__ << std::endl;
//     }
    
    
    void usage()
    {

        std::cout << "usage with CCM: EventChannelProfileManager " 
            << Cdmw::Common::Options::CALLBACK_IOR_OPTION << "=<callbackIOR> "
            << Cdmw::Common::Options::PROCESS_XML_FILE_OPTION << "=<xmlFileName> " 
        << " " << CCMProfileManager << std::endl;
    
    std::cout << "usage without CCM: EventChannelProfileManager " 
            << Cdmw::Common::Options::CALLBACK_IOR_OPTION << "=<callbackIOR> "
            << Cdmw::Common::Options::PROCESS_XML_FILE_OPTION << "=<xmlFileName> " 
        << InitUtils::PROFILE_XML_FILE_OPTION << "=<xmlFileName> "
            << "[" << Cdmw::Common::Options::VALIDATE_XML_OPTION << "]" 
        << std::endl;
    
    }

  void
  create_event_channel_manager (CORBA::ORB_ptr orb, PortableServer::POA_ptr factories_poa, int argc, char** argv)
    throw(CORBA::SystemException)
  {
    using namespace Cdmw::LifeCycle;
    std::string full_factory_name;
    std::string fully_qualified_target_domain_name;
    std::string hostname = Cdmw::OsSupport::OS::get_hostname();
    CdmwLifeCycle::ObjectRegistration_var object_registration;
    CdmwLifeCycle::ObjectRegistration_var factory_registration;

    typedef NamedObjectsFactoryAllocatorBase AllocatorType;

    typedef std::auto_ptr<Cdmw::OrbSupport::POAThreadingPolicy> threading_auto_ptr;

    // getting the default root context from the repository
    CdmwNamingAndRepository::Repository_var repository 
      = RepositoryInterface::get_repository();
    
    //   get the admin root context
    CosNaming::NamingContext_var adminRootCtx =
      repository->resolve_root_context(
                       Locations::ADMIN_ROOT_CONTEXT_ID);

    CosNaming::Name name_;
    name_.length(1);
    name_[0].id = CORBA::string_dup(hostname.c_str());

    Cdmw::CommonSvcs::Naming::NamingInterface ni(adminRootCtx.in());
    
    // if hostname equals to "X.Y" then to_string() returns "X\.Y"
    hostname = fully_qualified_target_domain_name = ni.to_string(name_);
    fully_qualified_target_domain_name += "/";
    fully_qualified_target_domain_name += Locations::LOCAL_EVENT_SERVICE_NAME_DOMAIN;
    fully_qualified_target_domain_name += "/EVENT_CHANNELS";
    
    // Get the factory Allocator
    // TODO: Use a user-provided name rather than a naming convention!
    AllocatorType & event_channel_allocator = 
      Cdmw::Event::EventChannelFactory_impl_Allocator::TheAllocator;
    
    full_factory_name = hostname;
    full_factory_name += "/";
    full_factory_name += Locations::LOCAL_EVENT_SERVICE_NAME_DOMAIN;
    full_factory_name += "/FACTORIES/DefaultEventChannelFactory";
    
    threading_auto_ptr threading_policy(0);
    
    try {
      threading_auto_ptr threading_policy(0);    
      threading_auto_ptr thrdptr(new Cdmw::OrbSupport::ThreadPerConnectionPolicy());
      threading_policy = thrdptr;
    } catch (const std::bad_alloc & ex) {
      CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
           CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                    CORBA::COMPLETED_NO),
           ex.what() );
    } catch (const Cdmw::OutOfMemoryException & ex) {
      CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
           CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                    CORBA::COMPLETED_NO),
           ex.what() );
    } catch (const CORBA::SystemException & ex ) {
      CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,
           ex );
    }
    
    // Create the object registration
    object_registration = new Cdmw::LifeCycle::DefaultObjectRegistration(adminRootCtx.in(), true);
    factory_registration = new Cdmw::LifeCycle::DefaultObjectRegistration(adminRootCtx.in(), true);
    
    try {
      Cdmw::LifeCycle::NamedObjectsFactory_initUtil::Common_setup_factory(event_channel_allocator,
                                      orb,           // ORB
                                      factories_poa,    // PARENT POA
                                      repository.in(),
                                      factory_registration.in(),
                                      object_registration.in(),
                                      fully_qualified_target_domain_name,
                                      full_factory_name,
                                      *threading_policy,
                                      argc, argv
                                      );
      
    } catch (const Cdmw::OutOfMemoryException & ex ) {
      std::string s ("Could not allocate memory for ");
      s += full_factory_name;
      s += "\n";
      s += ex.what();
      
      CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
           CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                    CORBA::COMPLETED_NO),
           s );
    } catch (const Cdmw::BadParameterException & ex) {
      CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
           CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMLifeCycleFrameworkInit,
                    CORBA::COMPLETED_NO),
           ex.what() );
    } catch (const Cdmw::InternalErrorException & ex ) {
      CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
           CORBA::INTERNAL(Cdmw::OrbSupport::INTERNALLifeCycleFrameworkError,
                   CORBA::COMPLETED_NO),
           ex.what() );
    } catch (const CORBA::SystemException & ex ) {
      CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,
           ex );
      }
  }
  
  class ManagerProcessControl : public Cdmw::CdmwInit::ProcessControl
    {
    public:
        
        ManagerProcessControl(EventChannelProfileManager_impl* mng, CORBA::ORB_ptr orb,
                              PortableServer::POA_ptr factories_poa, int argc, char** argv)
            throw(CORBA::SystemException):
            m_profileManager(mng),
            m_profileManagerServant_var(mng),
            m_is_registered(false),
            m_orb(CORBA::ORB::_duplicate(orb)),
            m_factories_poa(PortableServer::POA::_duplicate(factories_poa)),
            m_argc(argc),
            m_argv(argv)
        {
            m_profileManagerServant_var->_add_ref();

        }
               
        ~ManagerProcessControl() throw()
        {
           CDMW_DEBUG(EvtLogger::GetLogger(),
                      "Destructor of ManagerProcessControl called");
            
            if (m_is_registered)
            {              
                try
                {                    
                    m_object_registration->unregister_object(m_name_ManagerProcessControl.c_str());
                }
                catch (const CORBA::Exception& ex)
                {
                }
            }

        }
            
        /**
        * Purpose:
        * <p>
        * the behaviour for the
        * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/initialise:1.0
        * operation
        */
        virtual 
            void 
            on_initialise(const CdmwPlatformMngtBase::StartupKind& startup_kind)
            throw(CORBA::SystemException)
        {
           CDMW_INFO(EvtLogger::GetLogger(),
                     "cdmw_local_event_channel_manager init");

          // Create factory
          create_event_channel_manager (m_orb.in(), m_factories_poa.in(), m_argc, m_argv);

          // getting the default root context from the repository
          CdmwNamingAndRepository::Repository_var repository 
            = RepositoryInterface::get_repository();

          //   get the admin root context
          CosNaming::NamingContext_var adminRootCtx =
            repository->resolve_root_context(
                                             Locations::ADMIN_ROOT_CONTEXT_ID);

          m_object_registration = new Cdmw::LifeCycle::DefaultObjectRegistration(adminRootCtx.in(), true);

          CosNaming::Name contextName_;
          contextName_.length(1);
          contextName_[0].id = CORBA::string_dup(Cdmw::OsSupport::OS::get_hostname().c_str());
          Cdmw::CommonSvcs::Naming::NamingInterface ni(adminRootCtx);

          // if hostname equals to "X.Y" then to_string() returns "X\.Y"
          m_name_ManagerProcessControl = ni.to_string(contextName_);

          m_name_ManagerProcessControl +=  "/";
          m_name_ManagerProcessControl += Cdmw::Common::Locations::LOCAL_EVENT_SERVICE_NAME_DOMAIN;
          m_name_ManagerProcessControl += "/" + EventChannelProfileManager_impl::m_name ;

          // registering profileManager in Naming Context EVENT
          try
            {
            
              CdmwEventAdmin::EventChannelProfileManager_var ref = m_profileManager->_this();
              m_object_registration->register_object(ref.in(), m_name_ManagerProcessControl.c_str());
              m_is_registered = true;
            }
          catch (const CdmwNamingAndRepository::NameDomain::AlreadyExists&)
            {
               CDMW_INFO(EvtLogger::GetLogger(),
                         "EventChannelProfileManager already registred! Continue...");
              // Persistent Server so its not a pb!
              // continue...
            }
        }
        
        
      /**
       * Purpose:
       * <p>
       * the behaviour for the
       * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/run:1.0
       * operation
       */
      virtual 
      void 
      on_run()
        throw(CdmwPlatformMngt::ProcessDelegate::NotReadyToRun, 
              CORBA::SystemException)
      {
         CDMW_INFO(EvtLogger::GetLogger(),
                   "cdmw_local_event_channel_manager run");

        if (m_profileManager == NULL)
          {
             CDMW_ERROR(EvtLogger::GetLogger(),
                        "EventChannelProfileManager object not intanciated");
            throw CdmwPlatformMngt::ProcessDelegate::NotReadyToRun();
          }     
      }
        
      /**
       * Purpose:
       * <p>
       * the behaviour for the
       * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/stop:1.0
       * operation
       */
      virtual 
      void 
      on_stop()
        throw(CORBA::SystemException)
      {
         CDMW_INFO(EvtLogger::GetLogger(),
                   "cdmw_local_event_channel_manager stop");

        try
          {
            m_object_registration->unregister_object(m_name_ManagerProcessControl.c_str());
            m_is_registered = false;
          }
        catch (const CORBA::Exception& ex)
          {
             CDMW_ERROR(EvtLogger::GetLogger(),ex);
             throw;
          }
      }
        
    private:
      EventChannelProfileManager_impl*         m_profileManager;
      PortableServer::ServantBase_var          m_profileManagerServant_var;
      bool                                     m_is_registered;
      CORBA::ORB_var                           m_orb;
      PortableServer::POA_var                  m_factories_poa;
      int                                      m_argc;
      char**                                   m_argv;
      std::string                              m_name_ManagerProcessControl;
      CdmwLifeCycle::ObjectRegistration_var    m_object_registration;
  };
    
};  // end anonymous namespace

//----------------------------------------------------------------
int main(int argc, char** argv)
{
// #ifndef _MSC_VER    

//     std::set_unexpected(&exceptionHandler);

//     // VC++ contrary to the clause 18.6 of the C++ standard, 
//     //  set_unexpected, is not inside the std:: namespace
// #else

//     set_unexpected(&exceptionHandler);

// #endif  
    
    CORBA::ORB_var          orb;
    PortableServer::POA_var rootPOA;
    std::string             XMLFilePath;
    bool                    validate = false;
    bool                    ccmProfileManager = false;
    
    if (argc < 2)
    {
        usage();
        return EXIT_FAILURE;
    } else {
      std::string CCMProfileManager_option =
        Cdmw::OsSupport::OS::get_option_value( argc, argv, ::CCMProfileManager);
      if ( CCMProfileManager_option == "yes") {
        ccmProfileManager = true;
      }
      
      std::string xml_file_option =
        Cdmw::OsSupport::OS::get_option_value( argc, argv, InitUtils::PROFILE_XML_FILE_OPTION);
      
      if (  xml_file_option != "no" && ccmProfileManager == true) {
        usage();
        return EXIT_FAILURE;
      }

    }
    
    // Get the XML file Name : mandatory argument
    try
    { 
      if (ccmProfileManager == false){
        XMLFilePath = InitUtils::get_xml_initialisation_file (argc, argv);
      } 
    }
    catch (const CORBA::NO_MEMORY& e)
    {
        std::cerr << e << std::endl;
        return EXIT_FAILURE;
    }
    catch (const CORBA::INTERNAL& e)
    {
        std::cerr << e << std::endl;
        return EXIT_FAILURE;
    }
    catch (const CORBA::BAD_PARAM& e)
    {
        // Cannot Proceed with no file name
        usage();
        return EXIT_FAILURE;
    }
    
    try
    {
        // Get validation option for xml file
        validate = InitUtils::get_xml_validation_option(argc, argv);
        
        // Init ORB
        Cdmw::OrbSupport::StrategyList orb_strategies;
        orb_strategies.add_OrbThreaded();
        orb_strategies.add_PoaThreadPool(POA_THREAD_POOL_SIZE);

        orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, orb_strategies);
        
        if (CORBA::is_nil(orb.in()))
        {
           CDMW_FATAL(EvtLogger::GetLogger(),"No ORB");
           return EXIT_FAILURE;
        }
        
         // Resolve Root POA.
        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
        rootPOA = PortableServer::POA::_narrow(obj.in());
        
        if (CORBA::is_nil(rootPOA.in()))
        {
           CDMW_FATAL(EvtLogger::GetLogger(),"No RootPOA");
           Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
           orb->destroy();
           return EXIT_FAILURE;
        }
        
        // Get a reference to the POA manager.
        PortableServer::POAManager_var manager = rootPOA->the_POAManager();
        
        if (CORBA::is_nil(manager.in()))
        {
           CDMW_FATAL(EvtLogger::GetLogger(),"No POAmanager");
           Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
           orb->destroy();
           return EXIT_FAILURE;
        }

        // Activate POA manager
        manager->activate();
        
        // Create the single instance of the profile manager
        EventChannelProfileManager_impl* profileManager 
            = EventChannelProfileManager_impl::init(orb.in(),
                                                    rootPOA.in(),
                                                    XMLFilePath, 
                                                    validate,
                                                    ccmProfileManager);
        
        if (profileManager == NULL)
        {
           CDMW_WARN(EvtLogger::GetLogger(),
                     "Impossible to instantiate the EventChannelProfileManager object");
        }  
        
        // *****************************************************************
        // ***************** Init the redirection of logs to the ***********
        // ***************** daemon LogCollector                 ***********
        // *****************************************************************

        std::string system_port =
            Cdmw::OsSupport::SystemProperties::Get_Property(
                Cdmw::OrbSupport::SYSTEM_PORT_PROPERTY_NAME);
        if (system_port.empty()) {
            CDMW_WARN(EvtLogger::GetLogger(), "CdmwSystemPort has not be declared"
                       << " in argument. This is required to get the object referennce of"
                       << " the LogCollector and/or the recording daemon and recording server.");
        } 
        else 
        {
            try 
            {
                 Cdmw::CommonSvcs::InitUtils* initUtils =
                     Cdmw::CommonSvcs::InitUtils::Instance();
                 short port = (short) atoi(system_port.c_str());
                 initUtils->activate_LogCollector(orb, port);

            } 
            catch (const Cdmw::BadParameterException& bde) 
            {
                 CDMW_FATAL(EvtLogger::GetLogger(), bde.what());
                 throw CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidLocation, CORBA::COMPLETED_NO);
            } 
            catch (const Cdmw::Common::NotFoundException& nfe)  
            {
                 CDMW_ERROR(EvtLogger::GetLogger(), nfe.what());
                 throw CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMCorbalocBindingNotFound, CORBA::COMPLETED_NO);
            } // let CORBA::SystemException propagate
        }


        // init Cdmw
        Cdmw::CdmwInit::ProcessControl_var mng_ctrl 
            = new ManagerProcessControl(profileManager, orb.in(), rootPOA.in(), argc, argv);
        Cdmw::CdmwInit::CDMW_init(orb.in(), argc, argv, mng_ctrl.in());

        // wait for event
        orb->run();
    }   
    catch(const CORBA::SystemException& e)
    {    
       CDMW_ERROR(EvtLogger::GetLogger(),e);
    }
    catch(const CORBA::Exception& e)
    {
       CDMW_ERROR(EvtLogger::GetLogger(),e._name() );
    }
    catch(const Cdmw::Exception& e)
    {
       CDMW_ERROR(EvtLogger::GetLogger(),e.what());
    }
    
    if (!CORBA::is_nil(orb.in()))
    {
        try
        {
            Cdmw::CdmwInit::CDMW_cleanup(orb.in());
            // Release LogCollectorDelegate
            Cdmw::CommonSvcs::InitUtils::Instance()->deactivate_LogCollector();
            Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
            orb->destroy();
        }
        catch(const CORBA::Exception& ex)
        {
           CDMW_ERROR(EvtLogger::GetLogger(),ex);
        }
    }        
    
    return EXIT_SUCCESS;
}

