/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.eventadmin.test;

import org.junit.Before;
import org.junit.Test;

import cdmw.common.InternalErrorException;

import cdmw.eventadmin.EventChannelConfiguration;
import cdmw.eventadmin.ConfigurationException;
import cdmw.testutils.CORBATestManager;

import com.thalesgroup.CdmwEventAdmin.EventChannelProperty;
import static junit.framework.Assert.*;

public class EventChannelConfigurationTest {

    public static final String TEST_PARAM_XMLFILENAME_ID = "cdmw.eventadmin.test.EventChannelConfigurationTest.xmlfilename";
    public static final String TEST_PARAM_DOVALIDATE_ID = "cdmw.eventadmin.test.EventChannelConfigurationTest.dovalidate";
    
    private EventChannelConfiguration conf;
    private String xmlFile;
    private boolean doValidate;

    @Before
    public void init() {
        CORBATestManager corbaTestManager = CORBATestManager.instance();
        this.xmlFile = (String) corbaTestManager.getLocalProperty("cdmw.eventadmin.test.EventChannelConfigurationTest.xmlfilename");
        this.doValidate = (Boolean) corbaTestManager.getLocalProperty("cdmw.eventadmin.test.EventChannelConfigurationTest.dovalidate");
    }
    
    @Test
    public void doTests() {
        // This tests EventChannelConfiguration class
        //-------------------------------------------------

        try {
            System.out.println("Instantiate an EventChannelConfiguration object");

            try {
                conf = EventChannelConfiguration.
                    readConfiguration(xmlFile, doValidate);
            } 
            catch (ConfigurationException e) {
                // test failed!
                fail("ConfigurationException raised" + e.what());
            } catch (InternalErrorException e) {
                // test failed!
                fail("InternalErrorException raised");
            }

            System.out.println("Get number of ChannelProfile");
            int nbProfile = 0;
            
            try {
                nbProfile = conf.getNbEventChannelProfile();
                System.out.println("nbProfile = " + nbProfile);
            } catch (Exception e) {
                // test failed!
            }

            System.out.println("Get each channel name and properties");
            try {
                String name = null;
                java.util.Vector properties = null;

                for (int i = 0; i < nbProfile; i++) {
                    name = conf.getNextEventChannelName(i);
                    System.out.println("Channel " + i + " = " +name);
                    System.out.println(name);

                    properties = conf.getEventChannelProfileValue(name);
                    if (properties!=null) {
                        java.util.Enumeration e = properties.elements();
                        System.out.println("Properties number: " 
                            + properties.size());
                        EventChannelProperty property;
                        for( ; e.hasMoreElements() ; ) {
                            property = (EventChannelProperty) e.nextElement();
                            System.out.println(property.name);
                            System.out.println(property.value);
                        }
                    } else {
                        System.out.println("No Properties!");
                    }
                }
            } catch (Exception e) {}
            
        } catch (Exception e) {
            fail("ERROR: Unexpected exception");
        }

    }

}


