/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "eventsupport/EventChannelUtils_TAOOpenFusion.hpp"

#include <string>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>
#include <Foundation/ossupport/OS.hpp>

#include <iostream>


namespace
{
    // Name of the IDL property
    const char* TAO_CEC_PROPERTY = "PROP";


    CosNaming::NamingContext_ptr
    get_admin_root_context() 
      throw (CORBA::SystemException)
    {

      CosNaming::NamingContext_var adminRootCtx;

      // getting the default root context from the repository
      CdmwNamingAndRepository::Repository_var repository 
    = Cdmw::NamingAndRepository::RepositoryInterface::get_repository();
    
      // get the admin root context
      adminRootCtx =
    repository->resolve_root_context(
                     Cdmw::Common::Locations::ADMIN_ROOT_CONTEXT_ID);

      return adminRootCtx._retn();
    }

};

namespace Cdmw
{
namespace Event
{

/**
*Implementation notes: [if no pertinent write none ]
*<p>
*Portability issues: [if no pertinent write none ]
*<p>
*/

/********************************************************************/
// Get TAO CEC factory properties 
// This will read event channel profile if any and return number of
// properties and list of properties.
//
std::string
EventChannelUtils::get_factory_properties(const char* channel_name)
    throw(CdmwEvent::CreationError, 
          CORBA::SystemException)
{
    // An alias to a CORBA _var type of the object    
    CdmwEventAdmin::EventChannelProfileManager_var manager;
    CdmwEventAdmin::EventChannelProfile_var        channelProfile;
    std::string CECproperties = "";
    
    int i_property = 0;
    int nb_properties = 0;
    std::string propertyName;
    CORBA::Object_var obj;
    std::string profile_name = "";
    
    using namespace Cdmw::NamingAndRepository;

    // Check that a LocalEventManager is registered
    Cdmw::Event::EventChannelUtils::check_registration_Local_channel();

    //
    // Get Event channel properties
    //
    try 
    {
    if (Cdmw::Event::EventChannelUtils::M_local_manager_is_used) {
      //   get the admin root context
      CosNaming::NamingContext_var adminRootCtx = ::get_admin_root_context();
        
      CosNaming::Name name_;
      name_.length(1);
      name_[0].id = CORBA::string_dup(Cdmw::OsSupport::OS::get_hostname().c_str());
      
      // if hostname equals to "X.Y" then to_string() returns "X\.Y"
      Cdmw::CommonSvcs::Naming::NamingInterface ni(adminRootCtx.in());
      
      profile_name = ni.to_string(name_);
      profile_name += "/";
      profile_name += Cdmw::Common::Locations::LOCAL_EVENT_SERVICE_NAME_DOMAIN;
      profile_name += "/";
      profile_name += CdmwEventAdmin::EventChannelProfileManagerName;

      // Get a reference on the EventChannelProfileManager
      obj = ni.resolve(profile_name.c_str());
    }
    else {
      // i) Get NamingInterface object from RepositoryInterface
      Cdmw::CommonSvcs::Naming::NamingInterface ni = 
            Cdmw::NamingAndRepository::RepositoryInterface::get_domain_naming_interface(Cdmw::Common::Locations::CDMW_EVENT_SERVICE_NAME_DOMAIN);
      // ii) Get a reference on the EventChannelProfileManager
      obj = ni.resolve(CdmwEventAdmin::EventChannelProfileManagerName);
    }
      

    }
    catch (CdmwNamingAndRepository::NoNameDomain&)
    {
        throw CdmwEvent::CreationError("Cdmw event service domain not specified.");
    }
    catch (CdmwNamingAndRepository::InvalidName&)
    {
        throw CdmwEvent::CreationError("Cdmw event service domain invalid.");
    }
    catch (CosNaming::NamingContext::NotFound&)
    {
        throw CdmwEvent::CreationError("EventChannelProfileManager not found.");
    }
    catch (CosNaming::NamingContext::CannotProceed&)
    {
        throw CdmwEvent::CreationError("Can not proceed resolve on EventChannelProfileManager.");
    }
    catch (CosNaming::NamingContext::InvalidName&)
    {       
        throw CdmwEvent::CreationError("Invalid name for EventChannelProfileManager.");
    }
    
    
    manager = CdmwEventAdmin::EventChannelProfileManager::_narrow(obj.in());
    if (CORBA::is_nil(manager.in())) 
    {
        throw CdmwEvent::CreationError("EventChannelProfileManager is a nil object reference.");
    }
    
    //
    // iii) Find channel properties
    //
    try 
    {
        channelProfile = manager->find_profile(channel_name);
    }
    catch (CdmwEventAdmin::NotExist&)
    {
        // no profile defined for this channel nothing to do!      
    }
    catch (...)
    {
        // Unexpected exception       
    }
    
    // iv) Find TAO CEC Properties
    if (channelProfile)
    {
        // A profile is defined for this channel,
        // read it.
        nb_properties = channelProfile->length();
        
        for (i_property=0; i_property<nb_properties; i_property++)
        {
            propertyName = (*channelProfile)[i_property].name;
            if (propertyName == TAO_CEC_PROPERTY)
            {
                CECproperties = (*channelProfile)[i_property].value;
                
                // get out 'for' loop
                break; 
            }
        }
    }
    
    return CECproperties;
}    

// Register (name,object) within the repository.
// In case of a TRANSIENT system exception there will be a retry max_retries times.
// if max_retries == 0, then this will do retry forever.
void
EventChannelUtils::register_object
    (CdmwLifeCycle::ObjectRegistration_ptr                      domain,
     const std::string &                                         domain_name,
     CORBA::Object_ptr                                           the_object,
     const char*                                                 object_name,
     signed long                                                 max_retries)
    throw(CdmwLifeCycle::AlreadyExistant,
          CdmwLifeCycle::InvalidName,
          CORBA::SystemException)
{
    bool forever = (max_retries <= 0);

    signed long counter = max_retries;

    do {
        std::string registration_name;
        if (EventChannelUtils::M_local_manager_is_used)
            registration_name = domain_name + "/" + object_name;
        else
            registration_name = object_name;

        try {
            domain->register_object (the_object, registration_name.c_str());
        
            break;
        } catch (const CosNaming::NamingContext::AlreadyBound &) {
            // Previous server which created event_channel could have crash
            // without destroying event_channel.
            // Unregister previous event_channel, and retry to register
            try {
                domain->unregister_object(registration_name.c_str());
            } catch (const CORBA::SystemException& e) {
                // Repository is down or temporally not accessible ?
                // retry...
            }
            // increase counter for this retry not to be counted
            ++counter;
        
        } catch (const CosNaming::NamingContext::InvalidName & e) {
            // no "reason" field available
            throw CdmwLifeCycle::InvalidName();
        } catch (const CORBA::TRANSIENT &) {
            // Retry
        } catch (const CORBA::SystemException &) {
            // Cancel registration ????
            throw;
        }
    } while (forever || (--counter > 0) );

    if (!forever && (counter <= 0))
        throw CORBA::TRANSIENT(Cdmw::OrbSupport::TRANSIENTCannotReachRepository,
                               CORBA::COMPLETED_YES);
}


// Check that a LocalEventManager is registered
void
EventChannelUtils::check_registration_Local_channel()
    throw(CORBA::SystemException)
{
  
  static bool check_already_done = false;

  if ( check_already_done == false ) {
    check_already_done = true ;
    Cdmw::CommonSvcs::Naming::NamingInterface ni(::get_admin_root_context());
    
    try {
      
      std::string name = Cdmw::OsSupport::OS::get_hostname();
      
      CosNaming::Name name_;
      name_.length(1);
      name_[0].id = CORBA::string_dup(name.c_str());
    
      name = ni.to_string(name_);
      name += "/";
      name += Cdmw::Common::Locations::LOCAL_EVENT_SERVICE_NAME_DOMAIN;
      name += "/FACTORIES/DefaultEventChannelFactory";
    
      ni.resolve(name.c_str());
    
      EventChannelUtils::M_local_manager_is_used = true;
    
    } catch (const CosNaming::NamingContext::NotFound& ex) {
      // The DefaultEventChannelManager is not registered
    }
  }

}

          
}; // End namespace Event
}; // End namespace Cdmw


bool Cdmw::Event::EventChannelUtils::M_local_manager_is_used = false;
