/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <string>
#include <sstream>
#include "testeventsupport/TestEventChannelFactory.hpp"
#include "Repository/idllib/CdmwNamingAndRepository.stub.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/common/Options.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "SystemMngt/tools/ProcessAdmin.hpp"

namespace
{
    const char* CHANNEL_1_NAME = "TestEventChannel1";
    const char* CHANNEL_2_NAME = "TestEventChannel2";
    const char* MIS_URL_CHANNEL_NAME = "TestEventChannel3";
    const char* UNDEF_XML_CHANNEL_NAME = "TestEventChannel4";
    const char* INV_CHANNEL_NAME = "Test/EventChannel4";
};

namespace Cdmw
{
    namespace Event
    {

        /**
         *Implementation notes: [if no pertinent write none ]
         *<p>
         *Portability issues: [if no pertinent write none ]
         *<p>
         */

        CPPUNIT_TEST_SUITE_REGISTRATION( TestEventChannelFactory );

        TestEventChannelFactory::TestEventChannelFactory()
            throw()
            : m_test_number(1)
        {
            // does nothing
            CORBA::Object_var obj1 = Cdmw::TestUtils::CORBATestManager::instance()->get_object("factory");
            m_factory = CdmwEvent::EventChannelFactory::_narrow(obj1.in());
            m_orb =  CORBA::ORB::_duplicate( Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());

        }


        TestEventChannelFactory::~TestEventChannelFactory()
            throw()
        {
            // does nothing
        }


        TestEventChannelFactory::TestEventChannelFactory(const TestEventChannelFactory& rhs)
            throw()
            :m_factory(CdmwEvent::EventChannelFactory::_duplicate(rhs.m_factory.in()))
        {
            // does nothing
        }
        
        TestEventChannelFactory&
        TestEventChannelFactory::operator=(const TestEventChannelFactory& rhs)
            throw()
        {
            if (this != &rhs) {
                m_factory = CdmwEvent::EventChannelFactory::_duplicate(rhs.m_factory.in());
            }
            return *this;
        }

        void
        TestEventChannelFactory::next_test()
        {
            m_test_number++;
#if CDMW_ORB_VDR == tao
            // for TAO test number 2 is not valid while there is no event server
            if (m_test_number == 2)
            {
                m_test_number++;
                do_tests();
            }
    
#else
            do_tests();
#endif
        }

        void TestEventChannelFactory::do_tests()        
        {
            // set number of requested successfull tests
            // start/do_tests is recalled several times
            //     set_nbOfRequestedTestOK (0);
    
            using namespace Cdmw::Common;
            using namespace CdmwEvent;
            using namespace Cdmw::OsSupport;
            int timescale = Cdmw::TestUtils::get_timescale();

            bool test_ok=false;
            CosEventChannelAdmin::EventChannel_var channel;
            std::string profileManagerService;
            std::string managerArgs=""; 
            std::string callbackIOR = Cdmw::TestUtils::CORBATestManager::instance()->get_property ("callbackIOR");
            std::string nameServicePort = Cdmw::TestUtils::CORBATestManager::instance()->get_property ("nameServicePort");
            std::string cdmwIdlPath = Cdmw::TestUtils::CORBATestManager::instance()->get_property ("cdmwIdlPath");
            std::string dummy;
            std::string proc_url;
            std::string commands("");
            std::istringstream commands_is(commands);
            OS::ProcessId idProfileManager = 0;
            CdmwPlatformMngt::ProcessDelegate_var proc_ec_mng = CdmwPlatformMngt::ProcessDelegate::_nil();
            CORBA::Object_var obj = CORBA::Object::_nil();
            Cdmw::Tools::ProcessAdmin* admin1 = NULL;
            


            // This tests EventChannelFactory class
            //-------------------------------------------------
            TEST_INFO("Testing EventChannelFactory");

            try 
            {        
                switch (m_test_number)
                {
                    case 1 :
        
                        // add number of requested successfull tests
                        //             add_nbOfRequestedTestOK (1);
    
                        TEST_INFO("Creating a new channel CHANNEL 1 while EventchannelManager is not running");
                        try
                        {
                            channel = m_factory->create_event_channel(CHANNEL_1_NAME);
                        }
                        catch (CdmwEvent::CreationError& e)
                        {
                            test_ok = true;
                            std::ostringstream os;
                            os << "CORBA exception raised: " << e
                               << e.reason.in() << std::ends;
                            TEST_INFO(os.str());
                        }
                        catch (CdmwLifeCycle::AlreadyExistant& e)
                        {
                            std::ostringstream os;
                            os << "Unexpected CORBA exception raised: " << e
                               << e.name.in() << std::ends;
                            TEST_INFO(os.str());
                        }
                        catch (CdmwLifeCycle::InvalidName& e)
                        {
                            std::ostringstream os;
                            os << "Unexpected CORBA exception raised: " << e
                               << e.reason.in() << std::ends;
                            TEST_INFO(os.str());
                        }
                        catch (CORBA::Exception& e)
                        {
                            std::ostringstream os;
                            os << "Unexpected CORBA exception raised: " << e << std::ends;
                            TEST_INFO(os.str());
                        }
                        catch (...)
                        {
                            // test failed!
                            TEST_INFO("Unexpected exception raised!");
                        }
                        CPPUNIT_ASSERT(test_ok);
                        break;
            
                    case 2:
          
                        // add number of requested successfull tests
                        //             add_nbOfRequestedTestOK (1);
    
                        TEST_INFO("Creating a new channel CHANNEL 1 while EventServer is not running");
                        test_ok = false;
                        try
                        {
                            channel = m_factory->create_event_channel(CHANNEL_1_NAME);
                        }
                        catch (CdmwEvent::CreationError& e)
                        {
                            std::ostringstream os;
                            os << "Unexpected CORBA exception raised: " << e
                               << e.reason.in() << std::ends;
                            TEST_INFO(os.str());
                        }
                        catch (CdmwLifeCycle::AlreadyExistant& e)
                        {
                            std::ostringstream os;
                            os << "Unexpected CORBA exception raised: " << e
                               << e.name.in() << std::ends;
                            TEST_INFO(os.str());
                        }
                        catch (CdmwLifeCycle::InvalidName& e)
                        {
                            std::ostringstream os;
                            os << "Unexpected CORBA exception raised: " << e
                               << e.reason.in() << std::ends;
                            TEST_INFO(os.str());
                        }
                        catch (CORBA::TRANSIENT& e)
                        {
                            test_ok = true;
                            std::ostringstream os;
                            os << "CORBA exception raised: " << e << std::ends;
                            TEST_INFO(os.str());
                        }
                        catch (CORBA::Exception& e)
                        {
                            std::ostringstream os;
                            os << "Unexpected CORBA exception raised: " << e << std::ends;
                            TEST_INFO(os.str());
                        }
                        catch (...)
                        {
                            // test failed!
                            TEST_INFO("Unexpected exception raised!");
                        }
                        CPPUNIT_ASSERT(test_ok);
                        break;
            
                    case 3:
          
                        // add number of requested successfull tests
                        //             add_nbOfRequestedTestOK (5);
              
                        // Starting the EventChannelManager

                        profileManagerService = "cdmw_event_channel_manager";
#if (CDMW_ORB_VDR == orbacus )
                        managerArgs=" -ORBconfig obtest.conf ";        
#else
                         managerArgs="";        
#endif
                        
              
                        managerArgs += Options::CALLBACK_IOR_OPTION;
                        managerArgs += "=" + callbackIOR + " " 
                        + Options::PROCESS_PORT_OPTION + "=15327 " 
                        + Options::LOCALISATION_SERVICE_OPTION+ "=15328 " 
                        + " --ProfileXMLFile=EventChannelConf1.xml " 
                        + Options::PROCESS_XML_FILE_OPTION + "=Server.xml "
                        + Options::VALIDATE_XML_OPTION;
                        
              
                        std::cout << " launch EventChannelManager " << managerArgs << std::endl;
                        
                        idProfileManager = OS::create_process(profileManagerService, managerArgs);
                        //std::cin >> dummy;
              
                        OS::sleep(timescale*10000);
                        std::cout << "EventChannelManager started..." << std::endl;
              
                        // EventChannelManager Proc URL is CDMW Test Application/PROC_001 while main 
                        // of this test is PROC_000
                        std::cout << " Getting eventchannelmanager ProcessCallback" << std::endl;
                        
    
                        proc_url = "corbaname::localhost:" + nameServicePort;
                        proc_url += "#CDMW.I/ProcessCallback.simulated/CDMW Test Application/PROC_001";
                        obj = m_orb->string_to_object(proc_url.c_str());
              
                        if (!CORBA::is_nil(obj.in())) 
                        {
                            proc_ec_mng  = CdmwPlatformMngt::ProcessDelegate::_narrow(obj.in());
                        }
                        else
                        {
                            std::cerr << " **** TEST FAILED AT LINE " << __LINE__ 
                                      << " IN FILE: " << __FILE__ << std::endl;
                            break;
                        }
              
                        std::cout << " Getting Process admin" << std::endl;
                        admin1 = new Cdmw::Tools::ProcessAdmin(m_orb.in(),proc_ec_mng.in());
              
                        // send initialise order
                        commands = "init LAST 2\n exit \n";
                        commands_is.str(commands);
                        admin1->run(commands_is,std::cout);
                        OS::sleep(timescale*5000);
              
                        // send run order
                        commands = "run\n exit \n";
                        commands_is.str(commands);
                        admin1->run(commands_is,std::cout);
                        OS::sleep(timescale*3000);

                        TEST_INFO("Creating a new channel CHANNEL 4 undefined in the XML file");
                        test_ok = false;
                        try
                        {
                            channel = m_factory->create_event_channel(UNDEF_XML_CHANNEL_NAME);
                            test_ok = true;
                        }
                        catch (CdmwEvent::CreationError& e)
                        {
                            std::ostringstream os;
                            os << "Unexpected CORBA exception raised: " << e
                               << e.reason.in() << std::ends;
                            TEST_INFO(os.str());
                        }
                        catch (CdmwLifeCycle::InvalidName& e)
                        {
                            std::ostringstream os;
                            os << "Unexpected CORBA exception raised: " << e
                               << e.reason.in() << std::ends;
                            TEST_INFO(os.str());
                        }
                        catch (CdmwLifeCycle::AlreadyExistant& e)
                        {
                            std::ostringstream os;
                            os << "Unexpected CORBA exception raised: " << e
                               << e.name.in() << std::ends;
                            TEST_INFO(os.str());
                        }
                        catch (CORBA::Exception& e)
                        {
                            std::ostringstream os;
                            os << "Unexpected CORBA exception raised: " << e << std::ends;
                            TEST_INFO(os.str());
                        }
                        catch (...)
                        {
                            // test failed!
                            TEST_INFO("Unexpected exception raised!");
                        }
                        CPPUNIT_ASSERT(test_ok);

                        test_ok = false;
                        TEST_INFO("Creating a new channel CHANNEL 1");
                        try
                        {
                            channel = m_factory->create_event_channel(CHANNEL_1_NAME);
                            test_ok = true;
                        }
                        catch (CdmwEvent::CreationError& e)
                        {
                            std::ostringstream os;
                            os << "Unexpected CORBA exception raised: " << e
                               << e.reason.in() << std::ends;
                            TEST_INFO(os.str());
                        }
                        catch (CdmwLifeCycle::AlreadyExistant& e)
                        {
                            std::ostringstream os;
                            os << "Unexpected CORBA exception raised: " << e
                               << e.name.in() << std::ends;
                            TEST_INFO(os.str());
                        }
                        catch (CdmwLifeCycle::InvalidName& e)
                        {
                            std::ostringstream os;
                            os << "Unexpected CORBA exception raised: " << e
                               << e.reason.in() << std::ends;
                            TEST_INFO(os.str());
                        }
                        catch (CORBA::Exception& e)
                        {
                            std::ostringstream os;
                            os << "Unexpected CORBA exception raised: " << e << std::ends;
                            TEST_INFO(os.str());
                        }
                        catch (...)
                        {
                            // test failed!
                            TEST_INFO("Unexpected exception raised!");
                        }
                        CPPUNIT_ASSERT(test_ok);
            
            
                        TEST_INFO("Creating a new channel CHANNEL 2");
                        test_ok = false;
                        try
                        {
                            channel = m_factory->create_event_channel(CHANNEL_2_NAME);
                            test_ok = true;
                        }
                        catch (CdmwEvent::CreationError& e)
                        {
                            std::ostringstream os;
                            os << "Unexpected CORBA exception raised: " << e
                               << e.reason.in() << std::ends;
                            TEST_INFO(os.str());
                        }
                        catch (CdmwLifeCycle::AlreadyExistant& e)
                        {
                            std::ostringstream os;
                            os << "Unexpected CORBA exception raised: " << e
                               << e.name.in() << std::ends;
                            TEST_INFO(os.str());
                        }
                        catch (CdmwLifeCycle::InvalidName& e)
                        {
                            std::ostringstream os;
                            os << "Unexpected CORBA exception raised: " << e
                               << e.reason.in() << std::ends;
                            TEST_INFO(os.str());
                        }
                        catch (CORBA::Exception& e)
                        {
                            std::ostringstream os;
                            os << "Unexpected CORBA exception raised: " << e << std::ends;
                            TEST_INFO(os.str());
                        }
                        catch (...)
                        {
                            // test failed!
                            TEST_INFO("Unexpected exception raised!");
                        }
                        CPPUNIT_ASSERT(test_ok);
            
//                         TEST_INFO("Creating a new channel CHANNEL 1 => should raise AlreadyExistant exception ");
//                         test_ok = false;
//                         try
//                         {
//                             channel = m_factory->create_event_channel(CHANNEL_1_NAME);
//                         }
//                         catch (CdmwEvent::CreationError& e)
//                         {
//                             std::ostringstream os;
//                             os << "Unexpected CORBA exception raised: " << e
//                                << e.reason.in() << std::ends;
//                             TEST_INFO(os.str());
//                         }
//                         catch (CdmwLifeCycle::InvalidName& e)
//                         {
//                             std::ostringstream os;
//                             os << "Unexpected CORBA exception raised: " << e
//                                << e.reason.in() << std::ends;
//                             TEST_INFO(os.str());
//                         }
//                         catch (CdmwLifeCycle::AlreadyExistant&)
//                         {
//                             test_ok = true;
//                         }
//                         catch (CORBA::Exception& e)
//                         {
//                             std::ostringstream os;
//                             os << "Unexpected CORBA exception raised: " << e << std::ends;
//                             TEST_INFO(os.str());
//                         }
//                         catch (...)
//                         {
//                             // test failed!
//                             TEST_INFO("Unexpected exception raised!");
//                         }
//                         CPPUNIT_ASSERT(test_ok);
            
#if CDMW_ORB_VDR == orbacus
                        TEST_INFO("Creating a new channel CHANNEL 3 with no URL defined => should raise CreationError exception ");
                        test_ok = false;
                        try
                        {
                            channel = m_factory->create_event_channel(MIS_URL_CHANNEL_NAME);
                        }
                        catch (CdmwEvent::CreationError& e)
                        {
                            test_ok = true;
                        }
                        catch (CdmwLifeCycle::InvalidName& e)
                        {
                            std::ostringstream os;
                            os << "Unexpected CORBA exception raised: " << e
                               << e.reason.in() << std::ends;
                            TEST_INFO(os.str());
                        }
                        catch (CdmwLifeCycle::AlreadyExistant& e)
                        {
                            std::ostringstream os;
                            os << "Unexpected CORBA exception raised: " << e
                               << e.name.in() << std::ends;
                            TEST_INFO(os.str());
                        }
                        catch (CORBA::Exception& e)
                        {
                            std::ostringstream os;
                            os << "Unexpected CORBA exception raised: " << e << std::ends;
                            TEST_INFO(os.str());
                        }
                        catch (...)
                        {
                            // test failed!
                            TEST_INFO("Unexpected exception raised!");
                        }
                        CPPUNIT_ASSERT(test_ok);
#else
                        TEST_INFO("Creating a new channel CHANNEL 3 with defined Properties");
                        test_ok = false;
                        try
                        {
                            channel = m_factory->create_event_channel(MIS_URL_CHANNEL_NAME);
                            test_ok = true;
                        }
                        catch (CdmwEvent::CreationError& e)
                        {
                            std::ostringstream os;
                            os << "Unexpected CORBA exception raised: " << e
                               << e.reason.in() << std::ends;
                            TEST_INFO(os.str());
                        }
                        catch (CdmwLifeCycle::InvalidName& e)
                        {
                            std::ostringstream os;
                            os << "Unexpected CORBA exception raised: " << e
                               << e.reason.in() << std::ends;
                            TEST_INFO(os.str());
                        }
                        catch (CdmwLifeCycle::AlreadyExistant& e)
                        {
                            std::ostringstream os;
                            os << "Unexpected CORBA exception raised: " << e
                               << e.name.in() << std::ends;
                            TEST_INFO(os.str());
                        }
                        catch (CORBA::Exception& e)
                        {
                            std::ostringstream os;
                            os << "Unexpected CORBA exception raised: " << e << std::ends;
                            TEST_INFO(os.str());
                        }
                        catch (...)
                        {
                            // test failed!
                            TEST_INFO("Unexpected exception raised!");
                        }
                        CPPUNIT_ASSERT(test_ok);
#endif            

                        if (!CORBA::is_nil(m_orb.in()))
                        {
                            try
                            {
                                // send stop order
                                commands = "stop\n exit \n";
                                commands_is.str(commands);
                                std::cout << "Stop cdmw_event_channel_manager" << std::endl;
                                admin1->run(commands_is,std::cout);
                                delete admin1;
                    
#if CDMW_ORB_VDR == orbacus 
                                OS::kill_process(idEventServer);
#endif
                            }
                            catch(const CORBA::Exception& ex)
                            {
                                std::cerr << ex << std::endl;
                                std::cerr << " **** TEST FAILED AT LINE " << __LINE__ 
                                          << " IN FILE: " << __FILE__ << std::endl;
                            }
                        }   
            
                        break;

                    default:
                        TEST_INFO("**** NO MORE TEST AVAILABLE!");
                        break;
                }
            } 
            catch (const CORBA::Exception & e) 
            {
                std::cerr << "ERROR: " << e << std::endl;
            }
        }
        
    }; // End namespace LifeCyle
}; // End namespace Cdmw


