/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.event.test;

import org.junit.*;

import cdmw.testutils.CORBATestManager;

import com.thalesgroup.CdmwEvent.EventChannelFactory;

public class EventChannelFactoryTest {

    public static final String CHANNEL_1_NAME = "TestEventChannel1";

    public static final String CHANNEL_2_NAME = "TestEventChannel2";

    public static final String MIS_URL_CHANNEL_NAME = "TestEventChannel3";

    public static final String UNDEF_XML_CHANNEL_NAME = "TestEventChannel4";

    public static final String INV_CHANNEL_NAME = "Test/EventChannel4";

    /**
     * [Attribute description]
     */
    private EventChannelFactory factory;

    /**
     * indicate the current test number
     */
    private int testNumber;

    /**
     * [Constructor description if necessary]
     * 
     * @param
     * @exception
     */

    @Before
    public void init() {
        this.testNumber = (Integer) CORBATestManager.getLocalInstance()
            .getLocalProperty("cdmw.event.test.EventChannelFactoryTest0.factory");
        this.factory = (EventChannelFactory) CORBATestManager
            .getLocalInstance().getLocalProperty(
                "cdmw.event.test.EventChannelFactoryTest0.testNumber");
    }

    /**
     * Init and start next test
     * 
     * @param
     * @return
     * @exception
     */

    @Test
    public void doTests() {

        boolean testOk = false;
        org.omg.CosEventChannelAdmin.EventChannel channel;

        // This tests EventChannelFactory class
        // -------------------------------------------------

        try {
            switch (testNumber){
                case 1:
                    System.out.println("Creating a new channel CHANNEL 1"
                        + " while EventchannelManager is not running");
                    try {
                        channel = factory.create_event_channel(CHANNEL_1_NAME);
                    }
                    catch (com.thalesgroup.CdmwEvent.CreationError ce) {
                        testOk = true;
                        System.out
                            .println("Unexpected CORBA exception raised: "
                                + ce.reason);
                    }
                    catch (com.thalesgroup.CdmwLifeCycle.AlreadyExistant ae) {
                        System.out
                            .println("Unexpected CORBA exception raised: "
                                + ae.name);
                    }
                    catch (com.thalesgroup.CdmwLifeCycle.InvalidName in) {
                        System.out
                            .println("Unexpected CORBA exception raised: "
                                + in.reason);
                    }
                    catch (Exception e) {
                        // test failed!
                        System.out.println("Unexpected exception raised!");
                        e.printStackTrace();
                    }
                    Assert.assertTrue(testOk);
                    break;

                case 2:
                    System.out.println("Creating a new channel CHANNEL 1"
                        + " while EventServer is not running");
                    testOk = false;
                    try {
                        channel = factory.create_event_channel(CHANNEL_1_NAME);
                    }
                    catch (com.thalesgroup.CdmwEvent.CreationError ce) {
                        System.out
                            .println("Unexpected CORBA exception raised: "
                                + ce.reason);
                        ce.printStackTrace();
                    }
                    catch (com.thalesgroup.CdmwLifeCycle.AlreadyExistant ae) {
                        System.out
                            .println("Unexpected CORBA exception raised: "
                                + ae.name);
                        ae.printStackTrace();
                    }
                    catch (com.thalesgroup.CdmwLifeCycle.InvalidName in) {
                        System.out
                            .println("Unexpected CORBA exception raised: "
                                + in.reason);
                        in.printStackTrace();
                    }
                    catch (org.omg.CORBA.TRANSIENT t) {
                        testOk = true;
                        System.out.println("CORBA exception raised: "
                            + t.getMessage());
                    }
                    catch (Exception e) {
                        // test failed!
                        System.out.println("Unexpected exception raised!");
                        e.printStackTrace();
                    }
                    Assert.assertTrue(testOk);
                    break;

                case 3:
                    System.out
                        .println("Creating a new channel CHANNEL 4 undefined in the XML file");
                    testOk = false;
                    try {
                        channel = factory
                            .create_event_channel(UNDEF_XML_CHANNEL_NAME);
                        testOk = true;
                    }
                    catch (com.thalesgroup.CdmwEvent.CreationError ce) {
                        System.out
                            .println("Unexpected CORBA exception raised: "
                                + ce.reason);
                    }
                    catch (com.thalesgroup.CdmwLifeCycle.AlreadyExistant ae) {
                        System.out
                            .println("Unexpected CORBA exception raised: "
                                + ae.name);
                    }
                    catch (com.thalesgroup.CdmwLifeCycle.InvalidName in) {
                        System.out
                            .println("Unexpected CORBA exception raised: "
                                + in.reason);
                    }
                    catch (Exception e) {
                        // test failed!
                        System.out.println("Unexpected exception raised!");
                        e.printStackTrace();
                    }
                    Assert.assertTrue(testOk);

                    testOk = false;
                    System.out.println("Creating a new channel CHANNEL 1");
                    try {
                        channel = factory.create_event_channel(CHANNEL_1_NAME);
                        testOk = true;
                    }
                    catch (com.thalesgroup.CdmwEvent.CreationError ce) {
                        System.out
                            .println("Unexpected CORBA exception raised: "
                                + ce.reason);
                        ce.printStackTrace();
                    }
                    catch (com.thalesgroup.CdmwLifeCycle.AlreadyExistant ae) {
                        System.out
                            .println("Unexpected CORBA exception raised: "
                                + ae.name);
                        ae.printStackTrace();
                    }
                    catch (com.thalesgroup.CdmwLifeCycle.InvalidName in) {
                        System.out
                            .println("Unexpected CORBA exception raised: "
                                + in.reason);
                        in.printStackTrace();
                    }
                    catch (Exception e) {
                        // test failed!
                        System.out.println("Unexpected exception raised!");
                        e.printStackTrace();
                    }
                    Assert.assertTrue(testOk);

                    System.out.println("Creating a new channel CHANNEL 2");
                    testOk = false;
                    try {
                        channel = factory.create_event_channel(CHANNEL_2_NAME);
                        testOk = true;
                    }
                    catch (com.thalesgroup.CdmwEvent.CreationError ce) {
                        System.out
                            .println("Unexpected CORBA exception raised: "
                                + ce.reason);
                    }
                    catch (com.thalesgroup.CdmwLifeCycle.AlreadyExistant ae) {
                        System.out
                            .println("Unexpected CORBA exception raised: "
                                + ae.name);
                    }
                    catch (com.thalesgroup.CdmwLifeCycle.InvalidName in) {
                        System.out
                            .println("Unexpected CORBA exception raised: "
                                + in.reason);
                    }
                    catch (Exception e) {
                        // test failed!
                        System.out.println("Unexpected exception raised!");
                        e.printStackTrace();
                    }
                    Assert.assertTrue(testOk);

                    System.out.println("Creating a new channel CHANNEL 1 "
                        + "=> should raise AlreadyExistant exception ");
                    testOk = false;
                    try {
                        channel = factory.create_event_channel(CHANNEL_1_NAME);
                    }
                    catch (com.thalesgroup.CdmwEvent.CreationError ce) {
                        System.out
                            .println("Unexpected CORBA exception raised: "
                                + ce.reason);
                    }
                    catch (com.thalesgroup.CdmwLifeCycle.AlreadyExistant ae) {
                        testOk = true;
                    }
                    catch (com.thalesgroup.CdmwLifeCycle.InvalidName in) {
                        System.out
                            .println("Unexpected CORBA exception raised: "
                                + in.reason);
                    }
                    catch (Exception e) {
                        // test failed!
                        System.out.println("Unexpected exception raised!");
                    }
                    Assert.assertTrue(testOk);

                    System.out
                        .println("Creating a new channel CHANNEL 3 with no URL defined "
                            + "=> should raise CreationError exception ");
                    testOk = false;
                    try {
                        channel = factory
                            .create_event_channel(MIS_URL_CHANNEL_NAME);
                    }
                    catch (com.thalesgroup.CdmwEvent.CreationError ce) {
                        testOk = true;
                    }
                    catch (com.thalesgroup.CdmwLifeCycle.AlreadyExistant ae) {
                        System.out
                            .println("Unexpected CORBA exception raised: "
                                + ae.name);
                    }
                    catch (com.thalesgroup.CdmwLifeCycle.InvalidName in) {
                        System.out
                            .println("Unexpected CORBA exception raised: "
                                + in.reason);
                    }
                    catch (Exception e) {
                        // test failed!
                        System.out.println("Unexpected exception raised!");
                    }
                    Assert.assertTrue(testOk);

                    break;

                default:
                    System.out.println("**** NO MORE TEST AVAILABLE!");
                    break;
            }
        }
        catch (Exception e) {
            e.printStackTrace();
        }

    }

}
