/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.event.test;

import cdmw.testutils.CORBATestManager;
import cdmw.testutils.TestManager;
import cdmw.testutils.CORBATestManager.InvalidObject;
import cdmw.ossupport.OS;
import cdmw.event.EventChannel;

import org.junit.Before;
import org.junit.Test;
import org.omg.CORBA.SystemException;

import static junit.framework.Assert.*;

public class CdmwEventServiceTest {

    public static final int NB_RCVMSG = 4;
    public static final String MSG = "Hello world";

    public static final String CHANNEL_1_NAME = "EventChannel1";
    public static final String CHANNEL_2_NAME = "EventChannel2";
    public static final String CHANNEL_3_NAME = "EventChannel3";
    public static final String CHANNEL_4_NAME = "EventChannel4";
    public static final String COMPLETE_CHANNEL_1_NAME
        = "EVENT_CHANNELS/EventChannel1";
    public static final String COMPLETE_CHANNEL_2_NAME
        = "EVENT_CHANNELS/EventChannel2";
    public static final String COMPLETE_CHANNEL_3_NAME
        = "EVENT_CHANNELS/EventChannel3";
    public static final String COMPLETE_CHANNEL_4_NAME
        = "EVENT_CHANNELS/EventChannel4";

    public static boolean end = false;

    public static final String TEST_OBJECT_NAMING_CONTEXT_ID = "cdmw.event.test.CdmwEventServiceTest.NamingContext";
    public static final String TEST_OBJECT_NBTESTS_ID = "cdmw.event.test.CdmwEventServiceTest.Nbtests";
    
    private org.omg.CORBA.ORB orb;
    private org.omg.CosNaming.NamingContext namingcontext;
    private int testNb;

    class PushConsumerImpl extends org.omg.CosEventComm.PushConsumerPOA {

        /**
         * The ORB
         */
        private org.omg.CORBA.ORB orb;

        /**
         * My POA
         */
        private org.omg.PortableServer.POA poa;

        /**
         * ref of the consumer to which msg received is pushed.
         */
        private org.omg.CosEventComm.PushConsumer pushConsumer;

        /**
         * ref of the consumer to consumer id
         */
        private int id;

        public String[] rcvMsg = new String[10];

        public int nbRcvdMsg = 0;

        public PushConsumerImpl ()
    {
    }

        public PushConsumerImpl(
            org.omg.CORBA.ORB orb,
            org.omg.PortableServer.POA poa,
            org.omg.CosEventComm.PushConsumer pushConsumer,
            int id) {

            this.orb = orb;
            this.poa = poa;
            this.pushConsumer = pushConsumer;
            this.id = id;

        }

        public void push(org.omg.CORBA.Any data) throws SystemException {

            try {
                String str = data.extract_string();
                System.out.println("Consumer" + id + " has received: " + str);
                rcvMsg[nbRcvdMsg] = str;
            } catch(SystemException se) {
                System.out.println("Consumer" + id + " has an unknown event");
                rcvMsg[nbRcvdMsg] = "Unknown event";
            }
            nbRcvdMsg++;

            if (pushConsumer!=null) {
                try {
                    pushConsumer.push(data);
                } catch(org.omg.CosEventComm.Disconnected d) {
                    d.printStackTrace();
                }
            }

        }

        public void disconnect_push_consumer()
            throws SystemException {

            System.out.println("Disconnect");
            try {
                byte[] oid = poa.servant_to_id(this);
                poa.deactivate_object(oid);
            } catch(org.omg.PortableServer.POAPackage.ObjectNotActive ona) {
                ona.printStackTrace();
            } catch(org.omg.PortableServer.POAPackage.ServantNotActive sna) {
                sna.printStackTrace();
            } catch(org.omg.PortableServer.POAPackage.WrongPolicy wp) {
                wp.printStackTrace();
            }
            end = true;

        }

        public org.omg.PortableServer.POA _default_POA() {
            return this.poa;
        }

    }

    /**
     * Constructor
     *
     *@param
     *@exception
     */
    @Before
    public void init() {
        CORBATestManager corbaTestManager = (CORBATestManager) CORBATestManager.getLocalInstance();
        
        try {
            this.orb = corbaTestManager.getORB();
        }
        catch (InvalidObject e) {
            fail();
        }
        
        this.namingcontext = (org.omg.CosNaming.NamingContext) corbaTestManager.getLocalProperty(TEST_OBJECT_NAMING_CONTEXT_ID);
        this.testNb = (Integer) corbaTestManager.getLocalProperty(TEST_OBJECT_NBTESTS_ID);
    }
    
    @Test
    public void doTests() {
        org.omg.CosEventComm.PushConsumer pushConsumer = null;
        org.omg.PortableServer.POA rootPOA = null;
        PushConsumerImpl pcImpl = null;

        // This tests EventChannel class
        //-------------------------------------------------
        System.out.println("Testing Cdmw Event Service");

        try {
            // Create the implementation object.
            // Resolve Root POA.
            rootPOA = org.omg.PortableServer.POAHelper.narrow(
                orb.resolve_initial_references("RootPOA"));

            if (rootPOA==null) {
                System.out.println("ERROR: No RootPOA");
                fail();
                return;
            }

            EventChannel eventchannel1 = null;
            EventChannel eventchannel2 = null;
            EventChannel eventchannel3 = null;
            EventChannel eventchannel4 = null;

            System.out.println("Get Event Channel references");
            for (int i=0; i<200; i++) {
                try {
                    eventchannel4 = new EventChannel(COMPLETE_CHANNEL_4_NAME, namingcontext);
                    break;
                } catch (org.omg.CORBA.BAD_PARAM bp) {
                    // try again, maybe event channel is still not created!
                }
                OS.sleep(TestManager.getTimescale() * 20);
            }

            if (eventchannel4 == null) {
                System.out.println("EventChannel4 not found!");
            }

            try {
                eventchannel2 = new EventChannel(COMPLETE_CHANNEL_2_NAME, namingcontext);
                eventchannel3 = new EventChannel(COMPLETE_CHANNEL_3_NAME, namingcontext);
                eventchannel1 = new EventChannel(COMPLETE_CHANNEL_1_NAME, namingcontext);
            } catch (org.omg.CORBA.BAD_PARAM bp) {
                fail("Unexpected exception raised !");
            }

            switch (this.testNb) {
            case 1:
                int loopIndex = 0;

                System.out.println("Get a PushConsumer on EventChannel1");
                try {
                    pushConsumer = eventchannel1.getPushConsumer(null);
                } catch(Exception e) {
                    fail("Unexpected exception raised!");
                }

                System.out.println("Connect local consumer to EventChannel2");
                pcImpl = new PushConsumerImpl(orb, rootPOA, null, testNb);
                try {
                    eventchannel2.connectPushConsumer(pcImpl._this(orb));
                } catch(Exception e) {
                    fail("Unexpected exception raised!");
                }

                OS.sleep(TestManager.getTimescale() * 2000);

                System.out.println("Send message to EventChannel1");
                org.omg.CORBA.Any data = orb.create_any();
                System.out.println("Send <" + MSG + "> to Eventchannel1");
                try {
                    data.insert_string(MSG);
                } catch(SystemException se) {
                    se.printStackTrace();
                }
                pushConsumer.push(data);

                // wait for received msg
                OS.sleep(TestManager.getTimescale() * 1000);
                while (pcImpl.nbRcvdMsg != NB_RCVMSG) {
                    OS.sleep(TestManager.getTimescale() * 10);
                    if (loopIndex == 100) {
                        break;
                    }
                    loopIndex++;
                }

                System.out.println("Number of received message = " + pcImpl.nbRcvdMsg);

                // messages received! Compare them!
                for (int i=0; i<NB_RCVMSG ; i++) {
                    if (!MSG.equals(pcImpl.rcvMsg[i])) {
                        fail();
                    }
                }

                OS.sleep(TestManager.getTimescale() * 2000);
                System.out.println("Destroy EventChannels...");
                try {
                    System.out.println("    Destroy EventChannel2...");
                    eventchannel2.destroy();
                    System.out.println("    Destroy EventChannel3...");
                    eventchannel3.destroy();
                    System.out.println("    Destroy EventChannel1...");
                    eventchannel1.destroy();
                    System.out.println("    Destroy EventChannel4...");
                    eventchannel4.destroy();
                } catch (Exception e) {
                    fail("Unexpected exception raised!");
                }

                OS.sleep(TestManager.getTimescale() * 1000);
                System.out.println("Try to find EventChannel1!");
                try {
                    eventchannel1 = new EventChannel(COMPLETE_CHANNEL_1_NAME, namingcontext);
                    fail("Expected BAD_PARAM exception not received");
                } catch (org.omg.CORBA.BAD_PARAM bp) {
                    System.out.println("Expected exception: " + bp.toString());
                }
                while (!end) {
                    OS.sleep(TestManager.getTimescale() * 100);
                }
                System.out.println("TestProcess1 ending...");
                orb.shutdown(false);
                break;

            case 2:
                System.out.println("Get a PushConsumer on EventChannel3");
                try {
                    pushConsumer = eventchannel3.getPushConsumer(null);
                } catch(Exception e) {
                    fail("Unexpected exception raise!");
                }

                System.out.println("Connect local consumer to EventChannel1");
                pcImpl = new PushConsumerImpl(orb, rootPOA, pushConsumer, testNb);
                try {
                    eventchannel1.connectPushConsumer(pcImpl._this(orb));
                } catch(Exception e) {
                    fail("Unexpected exception raised!");
                }

                // wait for disconnect on consumer
                while (!end) {
                    OS.sleep(TestManager.getTimescale() * 100);
                }

                System.out.println("TestProcess2 ending...");
                orb.shutdown(false);
                break;

            case 3:
                System.out.println("Get a PushConsumer on EventChannel2");
                try {
                    pushConsumer = eventchannel2.getPushConsumer(null);
                } catch(Exception e) {
                    fail("Unexpected exception raised!");
                }

                System.out.println("Connect local consumer to EventChannel3");
                pcImpl = new PushConsumerImpl(orb, rootPOA, pushConsumer, testNb);
                try {
                    eventchannel3.connectPushConsumer(pcImpl._this(orb));
                } catch(Exception e) {
                    fail("Unexpected exception raised!");
                }

                // wait for disconnect on consumer
                while (!end) {
                    OS.sleep(TestManager.getTimescale() * 100);
                }

                System.out.println("TestProcess3 ending...");

                orb.shutdown(false);
                break;

            case 4:
                System.out.println("Get a PushConsumer on EventChannel4");
                try {
                    pushConsumer = eventchannel4.getPushConsumer(null);
                } catch(Exception e) {
                    fail("Unexpected exception raise!");
                }

                System.out.println("Connect local consumer to EventChannel3");
                pcImpl = new PushConsumerImpl(orb, rootPOA, pushConsumer, testNb);
                try {
                    eventchannel3.connectPushConsumer(pcImpl._this(orb));
                } catch(Exception e) {
                    fail("Unexpected exception raised!");
                }

                // wait for disconnect on consumer
                while (!end) {
                    OS.sleep(TestManager.getTimescale() * 100);
                }

                System.out.println("TestProcess4 ending...");

                orb.shutdown(false);
                break;

            default:
                System.out.println("Invalid test number!");
                break;
            }
        } catch (Exception e) {
            fail("ERROR: ");
        }

    }

}

