/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.event.test;

import cdmw.ossupport.OS;
import cdmw.testutils.CORBATestManager;
import cdmw.testutils.TestManager;
import cdmw.testutils.CORBATestManager.InvalidObject;
import cdmw.event.TypedEventChannel;

import org.junit.Before;
import org.junit.Test;
import org.omg.CORBA.SystemException;
import static junit.framework.Assert.*;

public class CdmwTypedEventServiceTest {

    public final static int NB_RCVMSG = 4;
    public final static String MSG = "Hello world";
    public final static String INTERFACE_REF = 
        "IDL:test.event.cdmw/TypedEcho:1.0";

    public final static String CHANNEL_1_NAME = "TypedEventChannel1";
    public final static String CHANNEL_2_NAME = "TypedEventChannel2";
    public final static String CHANNEL_3_NAME = "TypedEventChannel3";
    public final static String CHANNEL_4_NAME = "TypedEventChannel4";
    public final static String COMPLETE_CHANNEL_1_NAME
        = "EVENT_CHANNELS/TypedEventChannel1";
    public final static String COMPLETE_CHANNEL_2_NAME
        = "EVENT_CHANNELS/TypedEventChannel2";
    public final static String COMPLETE_CHANNEL_3_NAME
        = "EVENT_CHANNELS/TypedEventChannel3";
    public final static String COMPLETE_CHANNEL_4_NAME
        = "EVENT_CHANNELS/TypedEventChannel4";

    public static boolean end = false;

//    public static final String TEST_OBJECT_NAMING_CONTEXT_ID = "cdmw.event.test.CdmwTypedEventServiceTest.NamingContext";
//    public static final String TEST_OBJECT_NBTESTS_ID = "cdmw.event.test.CdmwTypedEventServiceTest.Nbtests";

    
    // ----------------------------------------------------------------------
    // EchoImpl implementation
    // ----------------------------------------------------------------------
    public static class EchoImpl extends TypedEchoPOA {

        /**
         * The ORB
         */
        private org.omg.CORBA.ORB orb;

        /**
         * My POA.
         */
        private org.omg.PortableServer.POA poa;

        /**
         * ref of the consumer to which msg received is pushed.
         */
        private TypedEcho pushConsumer;

        /**
         * consumer id
         */
        private int id;

        public String[] rcvMsg = new String[10];
        public int nbRcvdMsg = 0;

        public EchoImpl ()
        {
        }
         
        public EchoImpl(org.omg.CORBA.ORB orb, org.omg.PortableServer.POA poa,
            TypedEcho pushConsumer, int id) {
                this.orb = orb;
                this.poa = poa;
                this.pushConsumer = pushConsumer;
                  this.id = id;
            }

        public void echo_string(String str) throws SystemException {
            System.out.println("Consumer" + id + " has received: " + str );
            rcvMsg[nbRcvdMsg] = str;
            nbRcvdMsg++;
            if (pushConsumer!=null) {
                pushConsumer.echo_string(str);
            }
        }

        public void disconnect_push_consumer() throws SystemException {
            System.out.println("Disconnect");
            try {
                byte[] oid = poa.servant_to_id(this);
                poa.deactivate_object(oid);
            } catch(org.omg.PortableServer.POAPackage.ObjectNotActive ona) {
                ona.printStackTrace();
            } catch(org.omg.PortableServer.POAPackage.ServantNotActive sna) {
                sna.printStackTrace();
            } catch(org.omg.PortableServer.POAPackage.WrongPolicy wp) {
                wp.printStackTrace();
            }
            end = true;
        }

        public void push(org.omg.CORBA.Any data) throws SystemException {
            System.out.println("push any not implementedException e" );
        }

        public org.omg.PortableServer.POA _default_POA() {
            return this.poa;
        }
    }

    private org.omg.CORBA.ORB orb;
    private org.omg.CosNaming.NamingContext namingcontext;
    private int testNb;

    /**
     * Constructor
     *
     *@param
     *@exception
     */
    @Before
    public void init() {
        CORBATestManager corbaTestManager = (CORBATestManager) CORBATestManager.getLocalInstance();
        
        try {
            this.orb = corbaTestManager.getORB();
        }
        catch (InvalidObject e) {
            fail();
        }
        
        this.namingcontext = (org.omg.CosNaming.NamingContext) corbaTestManager.getLocalProperty(CdmwEventServiceTest.TEST_OBJECT_NAMING_CONTEXT_ID);
        this.testNb = (Integer) corbaTestManager.getLocalProperty(CdmwEventServiceTest.TEST_OBJECT_NBTESTS_ID);
    }

    @Test
    public void doTests() {
        TypedEcho pushConsumer = null;
        org.omg.PortableServer.POA rootPOA = null;
        EchoImpl pcImpl = null;
        org.omg.CORBA.Object obj = null;

        // This tests EventChannel class
        //-------------------------------------------------
        System.out.println("Testing Cdmw Typed Event Service");

        try {
            // Create the implementation object.
            // Resolve Root POA.
            rootPOA = org.omg.PortableServer.POAHelper.narrow(
                orb.resolve_initial_references("RootPOA"));

            if (rootPOA==null) {
                fail("ERROR: No RootPOA");
            }

            TypedEventChannel eventchannel1 = null;
            TypedEventChannel eventchannel2 = null;
            TypedEventChannel eventchannel3 = null;
            TypedEventChannel eventchannel4 = null;

            System.out.println("Get Event Channel references");
            for (int i=0; i<200; i++) {
                try {
                    eventchannel4 = new TypedEventChannel(COMPLETE_CHANNEL_4_NAME, namingcontext);
                    break;
                } catch (org.omg.CORBA.BAD_PARAM bp) {
                    // try again, maybe event channel is still not created!
                }
                OS.sleep(TestManager.getTimescale() * 20);
            }

            if (eventchannel4 == null) {
                fail("EventChannel4 not found!");
            }
            try {
                eventchannel1 = new TypedEventChannel(COMPLETE_CHANNEL_1_NAME, namingcontext);
                eventchannel2 = new TypedEventChannel(COMPLETE_CHANNEL_2_NAME, namingcontext);
                eventchannel3 = new TypedEventChannel(COMPLETE_CHANNEL_3_NAME, namingcontext);
            } catch (org.omg.CORBA.BAD_PARAM bp) {
                fail("Unexpected exception raised: " + bp.getMessage());
            }

            switch (testNb) {
            case 1:
                int loopIndex = 0;

                System.out.println("Get a PushConsumer on EventChannel1");
                try {
                    pushConsumer = TypedEchoHelper.narrow(
                        eventchannel1.getTypedPushConsumer(INTERFACE_REF,null));
                } catch(Exception e) {
                    fail("Unexpected exception raise!");
                }

                System.out.println("Connect local consumer to EventChannel2");
                pcImpl = new EchoImpl(orb, rootPOA, null, testNb);
                try {
                    eventchannel2.connectTypedPushConsumer(INTERFACE_REF, pcImpl._this(orb));
                } catch(Exception e) {
                    fail("Unexpected exception raised!");
                }

                OS.sleep(TestManager.getTimescale() * 2000);

                System.out.println("Send message to EventChannel1");

                System.out.println("Send <" + MSG + "> to EventChannel1");
                pushConsumer.echo_string(MSG);

                // wait for received msg
                while (pcImpl.nbRcvdMsg != NB_RCVMSG) {
                    OS.sleep(TestManager.getTimescale() * 10);
                    if (loopIndex == 200) {
                        break;
                    }
                    loopIndex++;
                }

                // messages received! Compare them!
                for (int i=0; i<NB_RCVMSG ; i++) {
                    if (!MSG.equals(pcImpl.rcvMsg[i])) {
                        fail();
                    }
                }

                OS.sleep(TestManager.getTimescale() * 2000);
                System.out.println("Destroy EventChannelsException...");
                try {
                    eventchannel1.destroy();
                    eventchannel2.destroy();
                    eventchannel3.destroy();
                    eventchannel4.destroy();
                } catch (Exception e) {
                    fail("Unexpected exception raised!");
                }

                System.out.println("Try to find EventChannel1!");
                try {
                    eventchannel1 = new TypedEventChannel(COMPLETE_CHANNEL_1_NAME, namingcontext);
                } catch (org.omg.CORBA.BAD_PARAM bp) {
                    fail(bp.getMessage());
                }

                // wait for disconnect on consumer
                while (!end) {
                    OS.sleep(TestManager.getTimescale() * 100);
                }

                System.out.println("TestProcess1 ending...");

                orb.shutdown(false);
                break;

            case 2:
                System.out.println("Get a PushConsumer on EventChannel3");
                try {
                    pushConsumer = TypedEchoHelper.narrow(
                        eventchannel3.getTypedPushConsumer(INTERFACE_REF, null));
                } catch(Exception e) {
                    fail("Unexpected exception raised!");
                    e.printStackTrace();
                }

                System.out.println("Connect local consumer to EventChannel1");
                pcImpl = new EchoImpl(orb, rootPOA, pushConsumer, testNb);
                try {
                    eventchannel1.connectTypedPushConsumer(INTERFACE_REF,pcImpl._this(orb));
                } catch(Exception e) {
                    fail("Unexpected exception raised!");
                    e.printStackTrace();
                }

                // wait for disconnect on consumer
                while (!end) {
                    OS.sleep(TestManager.getTimescale() * 100);
                }

                System.out.println("TestProcess2 ending...");

                orb.shutdown(false);
                break;

            case 3:
                System.out.println("Get a PushConsumer on EventChannel2");
                try {
                    pushConsumer = TypedEchoHelper.narrow(
                        eventchannel2.getTypedPushConsumer(INTERFACE_REF,null));
                } catch(Exception e) {
                    fail("Unexpected exception raised!");
                }

                System.out.println("Connect local consumer to EventChannel3");
                pcImpl = new EchoImpl(orb, rootPOA, pushConsumer, testNb);
                try {
                    eventchannel3.connectTypedPushConsumer(INTERFACE_REF, pcImpl._this(orb));
                } catch(Exception e) {
                    fail("Unexpected exception raised!");
                }

                // wait for disconnect on consumer
                while (!end) {
                    OS.sleep(TestManager.getTimescale() * 100);
                }

                System.out.println("TestProcess3 ending...");

                orb.shutdown(false);
                break;

            case 4:
                System.out.println("Get a PushConsumer on EventChannel4");
                try {
                    pushConsumer = TypedEchoHelper.narrow(
                        eventchannel4.getTypedPushConsumer(INTERFACE_REF,null));
                } catch(Exception e) {
                    fail("Unexpected exception raised!");
                }

                System.out.println("Connect local consumer to EventChannel3");
                pcImpl = new EchoImpl(orb, rootPOA, pushConsumer, testNb);
                try {
                    eventchannel3.connectTypedPushConsumer(INTERFACE_REF, pcImpl._this(orb));
                } catch(Exception e) {
                    fail("Unexpected exception raised!");
                }

                // wait for disconnect on consumer
                while (!end) {
                    OS.sleep(TestManager.getTimescale() * 100);
                }

                System.out.println("TestProcess4 ending...");

                orb.shutdown(false);
                break;

            default:
                System.out.println("Invalid test number!");
                break;
            }
        } catch (Exception e) {
            fail("ERROR: ");
        }

    }

}

