/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_CDMW_FT_ULTIMATE_FALLBACK_FACTORY_HPP
#define INCL_CDMW_FT_ULTIMATE_FALLBACK_FACTORY_HPP

// Standard Files
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/osthreads/Mutex.hpp>

namespace Cdmw
{
namespace FT
{
   class UltimateFallbackFactory
   {
      public:
         
         virtual 
         ~UltimateFallbackFactory() throw() {}
         
         virtual CORBA::Object_ptr 
         create_reference_with_user_id(const PortableServer::ObjectId & oid,
                                       const char *type_id) = 0;
         virtual std::string 
         create_object_key (const PortableServer::ObjectId &oid) = 0;

         virtual PortableServer::POA_ptr get_POA() = 0;
         
         /**
          *@exception CORBA::BAD_INV_ORDER if already done.
          */
         static UltimateFallbackFactory*
         Init(CORBA::ORB_ptr orb,
              const char* host,
              unsigned short int port,
              PortableServer::ServantLocator* locator=0)
            throw (CORBA::SystemException);
         
         /**
          *@return NULL if not initialised.
          */
         static UltimateFallbackFactory*
         Instance()
            throw ();
         static void Cleanup();
      private:
         static OsSupport::Mutex M_mutex;
         static UltimateFallbackFactory* M_Instance;
   };
} // End of namespace FT
} // End of namespace Cdmw

#endif // INCL_CDMW_FT_ULTIMATE_FALLBACK_FACTORY_HPP

