/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "tao/AnyTypeCode/TypeCode.h"
#include "tao/AnyTypeCode/NVList.h"
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/common/Assert.hpp> // for CDMW_ASSERT macro
#include <FaultTolerance/ftcommon/MacroDef.hpp> // for CDMW_FT_DEBUG macro

#include <FaultTolerance/ftcommon/FallbackOid.hpp>
#include <FaultTolerance/ftcommon/Fallback_impl.hpp>

#include <FaultTolerance/ftcommon/FTLogger.hpp>

#include <Foundation/logging/FunctionLogger.hpp>
#include <FaultTolerance/ftcommon/FTObjectIds.hpp>


namespace Cdmw {
namespace FT {

    
    /**
     * Creates a fallback default servant
     *
     * @param orb ...
     * @param poa ...
     */
    Fallback_impl::Fallback_impl(CORBA::ORB_ptr          orb,
                                 PortableServer::POA_ptr poa)
        throw(CORBA::SystemException)
       : m_orb(CORBA::ORB::_duplicate(orb)),
         m_poa(PortableServer::POA::_duplicate(poa)),
         m_current(PortableServer::Current::_nil())
    {
        CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
        CDMW_ASSERT(!CORBA::is_nil(orb));
        CORBA::Object_var obj =
            orb->resolve_initial_references("POACurrent");
        m_current = PortableServer::Current::_narrow(obj.in());
        CDMW_ASSERT(!CORBA::is_nil(m_current.in()));
    }
        
        
    /**
     * Destroys the fallback default servant.
     */
    Fallback_impl::~Fallback_impl(void) throw ()
    {
        CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
    }
    
        
    /**
     * The invoke() method receives requests issued to any CORBA
     * object incarnated by the DSI servant and performs the
     * processing necessary to execute the request.
     *
     * @param request A ServerRequest local object containing request information.
     *
     * @exception TODO...
     */
    void Fallback_impl::invoke (CORBA::ServerRequest_ptr request)
    {
        CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
        PortableServer::ObjectId_var oid = this->get_object_id();
        FallbackOid foid(oid.in());
        this->invoke_i(request,foid);
    }
    
    
    /**
     * The _primary_interface() method receives an ObjectId value and
     * a POA_ptr as input parameters and returns a valid RepositoryId
     * representing the most-derived interface for that oid.
     *
     * @param oid ObjectId of the target object.
     * @param poa PortableServer::POA pseudo-object
     *
     * @return The repository id of the most-derived interface for that oid.
     *
     * @exception TODO...
     */
    CORBA::RepositoryId 
    Fallback_impl::_primary_interface (const PortableServer::ObjectId &oid,
                                       PortableServer::POA_ptr poa)
    {  
        CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
        FallbackOid foid(oid);
        return CORBA::string_dup(foid.get_repository_id().c_str());
    }

}; // namespace FT
}; // namespace Cdmw

