/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <FaultTolerance/ftcommon/FTLogger.hpp>

#include <ace/Log_Msg.h>
#include <ace/Log_Record.h>

namespace Cdmw {
    namespace FT {
       int FTLogger::HexDump(const ::CORBA::OctetSeq& data_seq,
                             const char *caption,
                             std::ostream & os)
       {
          const ACE_TCHAR *text = ACE_TEXT (caption);
          const char *buffer = (const char*)data_seq.get_buffer();
          size_t size = data_seq.length();
          
          ACE_TCHAR* buf = 0;
          const size_t buf_sz =
             ACE_Log_Record::MAXLOGMSGLEN - ACE_Log_Record::VERBOSE_LEN - 58;
          ACE_NEW_RETURN (buf, ACE_TCHAR[buf_sz], -1);
          
          ACE_TCHAR *msg_buf = 0;
          const size_t text_sz = text ? ACE_OS::strlen(text) : 0;
          ACE_NEW_RETURN (msg_buf,
                          ACE_TCHAR[text_sz + 58],
                          -1);
          
          buf[0] = 0; // in case size = 0
          
          const size_t len = ACE::format_hexdump
             (buffer, size, buf, buf_sz / sizeof (ACE_TCHAR) - text_sz);
          
          int sz = 0;
          
          if (text)
             sz = ACE_OS::sprintf (msg_buf,
                                   ACE_LIB_TEXT ("%s - "),
                                   text);
          
          sz += ACE_OS::sprintf (msg_buf + sz,
                                 ACE_LIB_TEXT ("HEXDUMP ")
                                 ACE_SIZE_T_FORMAT_SPECIFIER
                                 ACE_LIB_TEXT (" bytes"),
                                 size);
          
          if (len < size)
             ACE_OS::sprintf (msg_buf + sz,
                              ACE_LIB_TEXT (" (showing first ")
                              ACE_SIZE_T_FORMAT_SPECIFIER
                              ACE_LIB_TEXT (" bytes)"),
                              len);
          
          // Now print out the formatted buffer.
          
          os << msg_buf << "\n" << buf;
          
          delete [] msg_buf;
          return 0;
       }


    } // End namespace FT
} // End namespace Cdmw

