/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <FaultTolerance/ftcommon/UltimateFallbackFactory.hpp>

#include "ftcommon/TAOUltimateFallbackFactory.hpp"

#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>

#include <Foundation/osthreads/MutexGuard.hpp>
#include <Foundation/common/Assert.hpp>
#include <FaultTolerance/ftcommon/FTLogger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <sstream>


namespace Cdmw {
namespace FT {

   OsSupport::Mutex UltimateFallbackFactory::M_mutex;
   UltimateFallbackFactory* UltimateFallbackFactory::M_Instance(0);
   
   
   UltimateFallbackFactory*
   UltimateFallbackFactory::Init(CORBA::ORB_ptr orb,
                                 const char* host,
                                 unsigned short int port,
                                 PortableServer::ServantLocator* locator)
      throw (CORBA::SystemException)
   {
      CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"host=" << host << ",port="<<port);
      CDMW_MUTEX_GUARD (M_mutex);
      if (M_Instance != 0) {
         CORBA::BAD_INV_ORDER ex (Cdmw::OrbSupport::BAD_INV_ORDERInitAlreadyDone,
                                  CORBA::COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(ex);
         throw ex;
      }
      CDMW_INFO(FTLogger::GetLogger(),"Initialising Ultimate Fallbacks to "
                << host << ":" << port);
      M_Instance=new TAOUltimateFallbackFactory(orb, host, port, locator);
      return M_Instance;
   }
   
   UltimateFallbackFactory*
   UltimateFallbackFactory::Instance()
      throw ()
   {
      CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
      CDMW_LOG_FUNCTION_RETURN(M_Instance);
      return M_Instance;
   }
   void UltimateFallbackFactory::Cleanup()
   {
      //fixing memory leak
      if (M_Instance != 0) delete M_Instance;
      M_Instance = 0;
   }

}; // namespace FT
}; // namespace Cdmw

