/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ft;

public class FactoryInfos {
    /**
     * the sequence of FactoryInfo containing each location for each object
     */
    private org.omg.FT.FactoryInfo[] factoryInfos;
    
    /**
     * add the FT location
     *
     * @param hostname        Host name of the location     
     * @param applicationName Application name of the location
     * @param processName     Process name of the location
     */
    public void add_ftLocation(String hostName,
                               String applicationName,
                               String processName) {

        int infosSize = 0;                       
        org.omg.FT.FactoryInfo[] tempFactoryInfos;

        if (factoryInfos != null)
        {
            infosSize = factoryInfos.length;
        }

        org.omg.FT.FactoryInfo factoryInfo = new org.omg.FT.FactoryInfo();
        // No factory passed at the moment
        factoryInfo.the_factory = null;
        // No criteria passed at the moment
        org.omg.FT.Property[] criteriaProp = new org.omg.FT.Property[0];
        factoryInfo.the_criteria =  criteriaProp;
        // Create Location where object will be created
        FTLocation ftLoc = new FTLocation(hostName, applicationName, processName);
        factoryInfo.the_location = ftLoc.get_FT_Location();

        tempFactoryInfos = new org.omg.FT.FactoryInfo[infosSize+1];
        if (factoryInfos != null)
        {
            System.arraycopy(factoryInfos, 0, tempFactoryInfos, 0, factoryInfos.length); 
        }
        tempFactoryInfos[infosSize] = factoryInfo;
        factoryInfos = tempFactoryInfos;
    }

    /**
     * get the current sequence of FactoryInfo
     *
     * @return the FactoryInfo sequence
     */
    public org.omg.FT.FactoryInfo[] get_factoryInfos() {
        return this.factoryInfos;
    }
}
