/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ft;

import org.omg.CORBA.*;

class ORBSpecificFTConfiguration {
    
    private static String clientID;
    
    /**
     * Set the client_id
     *
     * @param orb the orb.
     * @param client_id the client_id for the orb.
     *
     * @return void 
     */
    static void SetClientId(ORB orb, String clientId) {
        ORBSpecificFTConfiguration.clientID = clientId;
    }
    
    /**
     * Get the client_id
     *
     * @param orb the orb.
     *
     * @return client_id the client_id for the orb..
     */
    static String GetClientId(ORB orb) {
        return ORBSpecificFTConfiguration.clientID;
    }

    /**
     * Get the FT endpoint selector factory
     *
     * @return std::string the name of the FT_endpoint_selector_factory for the ORB
     */
    static String GetFTEndpointSelectorFactory() {
        return "FT_Endpoint_Selector_Factory";
    }
    
    
    /**
     * Activate the FT service
     *
     * @param args The list of parameter received of the command line
     */
    static void ActivateFTService(String[] args) {
        // Set JacORB FT properties (required for JacORB to load FT stuff)
        System.setProperty("org.omg.PortableInterceptor.ORBInitializerClass.ft_init" , 
                           "org.jacorb.orb.ft.FaultTolerantClientORBInit");
        System.setProperty("jacorb.ft.iogr_validator", 
                           "org.jacorb.orb.ft.DefaultIOGRValidator");
        System.setProperty("ORBFTSendFullGroupTC", "on");
        
        // If not specified, set number of connection retries to 0.
        // By default JacoRB perform 5 retries at 500ms intervall. This lead to
        // detect a connection failure in 2.5 seconds !!
        if (System.getProperty("jacorb.retries") == null) {
            System.setProperty("jacorb.retries", "0");
        }
        
    }

}
