/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.testPCR1473;

import org.junit.Assert;
import org.junit.Test;


public class MainTest {

    private static org.omg.CORBA.ORB orb;
    
    
    @Test
    public void doTests()
    {
        try {
            // Get ReplicationManager
            System.out.println("Get ReplicationManager");
            org.omg.CORBA.Object ft_mgr_obj = 
                orb.string_to_object("corbaloc::localhost:4555/fault_manager");
            com.thalesgroup.CdmwFT.FTManager fault_manager = 
                com.thalesgroup.CdmwFT.FTManagerHelper.narrow(ft_mgr_obj);
            com.thalesgroup.CdmwFT.ReplicationManager replicationMgr = 
                fault_manager.get_replication_manager();
                
            // Create group
            String rep_id_hello = "IDL:thalesgroup.com/CdmwFT/Test/HelloInterface:1.0";
            org.omg.CORBA.Any factory_creation_id_hello = null;
            org.omg.CORBA.Object group = null;
    
            // set default properties
            org.omg.FT.Property[] prop = new org.omg.FT.Property[2];
            prop[0] = new org.omg.FT.Property();
            prop[1] = new org.omg.FT.Property();
            prop[0].val = orb.create_any();
            prop[1].val = orb.create_any();
    
            prop[0].nam = new org.omg.CosNaming.NameComponent[1];
            prop[0].nam[0] = new org.omg.CosNaming.NameComponent();
            prop[0].nam[0].id = "org.omg.ft.MinimumNumberReplicas";
            prop[0].val.insert_ushort((short) 2);
    
            prop[0].nam[0].kind = new String();
    
            org.omg.CosNaming.NameComponent[][] locs = new org.omg.CosNaming.NameComponent[2][];
            locs[0] = new org.omg.CosNaming.NameComponent[3];
            locs[0][0] = new org.omg.CosNaming.NameComponent();
            locs[0][1] = new org.omg.CosNaming.NameComponent();
            locs[0][2] = new org.omg.CosNaming.NameComponent();
            
            locs[0][0].id = "host1";
            locs[0][0].kind = "hostname";
            locs[0][1].id = "APPL1";
            locs[0][1].kind = "applicationname";
            locs[0][2].id = "P11";
            locs[0][2].kind = "processname";
    
            locs[1] = new org.omg.CosNaming.NameComponent[3];
            locs[1][0] = new org.omg.CosNaming.NameComponent();
            locs[1][1] = new org.omg.CosNaming.NameComponent();
            locs[1][2] = new org.omg.CosNaming.NameComponent();
    
            locs[1][0].id = "host2";
            locs[1][0].kind = "hostname";
            locs[1][1].id = "APPL3";
            locs[1][1].kind = "applicationname";
            locs[1][2].id = "P31";
            locs[1][2].kind = "processname";
    
            int factory_infos_len = locs.length;
            org.omg.FT.FactoryInfo[] factoryInfos = new org.omg.FT.FactoryInfo[factory_infos_len];
            for (int i = 0; i < factory_infos_len; ++i) {
                try {
                    factoryInfos[i] = new org.omg.FT.FactoryInfo();
                    factoryInfos[i].the_factory = null;
                    System.out.println('[' + i + "] " + " --- "
                        + cdmw.commonsvcs.naming.NamingInterface.toString(locs[i]));
    
                    factoryInfos[i].the_location = locs[i];
                    org.omg.FT.Property[] factoryCrit = new org.omg.FT.Property[0];
                    factoryInfos[i].the_criteria = factoryCrit;
                }
                catch (Exception e) {
                    e.printStackTrace();
                    Assert.assertTrue(false);
                }
            }
    
            prop[1].nam = new org.omg.CosNaming.NameComponent[1];
            prop[1].nam[0] = new org.omg.CosNaming.NameComponent();
            prop[1].nam[0].id = "org.omg.ft.Factories";
            prop[1].nam[0].kind = new String();
            org.omg.FT.FactoryInfosHelper.insert(prop[1].val, factoryInfos);
    
            org.omg.FT.Property[] crit_hello = new org.omg.FT.Property[1];
            crit_hello[0] = new org.omg.FT.Property();
            crit_hello[0].val = orb.create_any();                   
            crit_hello[0].nam = new org.omg.CosNaming.NameComponent[1];
            crit_hello[0].nam[0] = new org.omg.CosNaming.NameComponent();
            crit_hello[0].nam[0].kind = new String();
            crit_hello[0].nam[0].id = "org.omg.ft.FTProperties";
            org.omg.FT.PropertiesHelper.insert(crit_hello[0].val, prop);
    
            group = replicationMgr.create_object(
                rep_id_hello,
                crit_hello, 
                new org.omg.CORBA.AnyHolder(factory_creation_id_hello));

            Assert.assertTrue(true);
    
        } catch (Exception e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
    }
    
    
    public static void main(String[] args) {

        try {
            // Initialise FT service
            System.out.println("FT init");
            cdmw.ftinit.FTServiceInit.Init(args, true);
            
            // Initialises the ORB
            cdmw.orbsupport.StrategyList strategyList = new cdmw.orbsupport.StrategyList();
            strategyList.addORBThreaded();
            strategyList.addPOAThreadPerConnection();

            System.out.println("ORB init");
            orb = cdmw.orbsupport.ORBSupport.ORBInit(args, strategyList);

            // Get the root POA
            org.omg.CORBA.Object obj = orb
                .resolve_initial_references("RootPOA");
            org.omg.PortableServer.POA rootPOA = 
                org.omg.PortableServer.POAHelper.narrow(obj);
            org.omg.PortableServer.POAManager poaMgr = rootPOA.the_POAManager();
            poaMgr.activate();

            // TestManager init
            cdmw.testutils.CORBATestManager.instance().init(
                (new Throwable()).getStackTrace()[0].getClassName(), 
                orb,
                rootPOA);

            // Start FTManager
            System.out.println("Start cdmw_ft_manager");
            
            cdmw.ossupport.ThreadedProcess p = 
                cdmw.ossupport.OS.createProcess(
                    "cdmw_ft_manager --CdmwXMLFile=CdmwFaultToleranceManager_conf.xml -ORBDebugLevel 10");
            
            // Wait FTManager start-up
            cdmw.ossupport.OS.sleep(5000);
            
                
            cdmw.testutils.CORBATestManager.instance().addTest(MainTest.class);
            cdmw.testutils.CORBATestManager.instance().runTests();
            cdmw.testutils.CORBATestManager.instance().printSummary();
            
            // Stop FTManager
            System.out.println("Stopping twice cdmw_ft_manager...");
            p.killProcess();
            cdmw.ossupport.OS.killProcess(p); // hack preventing the ft_manager to be still running
            System.out.println("...done.");
            
            System.out.println("Stop ORB");
            cdmw.orbsupport.ORBSupport.ORBCleanup(orb);
            orb.destroy();
            
        }
        catch (org.omg.CORBA.SystemException e) {
            e.printStackTrace();
            System.exit(1);
        }
        catch (org.omg.CORBA.UserException e1) {
            e1.printStackTrace();
            System.exit(1);
        }
        catch (cdmw.common.Exception e2) {
            e2.printStackTrace();
            System.exit(1);
        }
        catch (Exception e3) {
            System.err.println("Unexpected exception");
            e3.printStackTrace();
            System.exit(1);
        }

    }
}
