/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include "testftcommon/TestFTCommon.hpp"
#include "testftcommon/TestHello_impl.hpp"
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include "Foundation/common/String.hpp"
#include <FaultTolerance/idllib/CdmwFTCommon.stub.hpp>
#include <FaultTolerance/idllib/CdmwFTIOGRFactory.stub.hpp>
#include <FaultTolerance/ftcommon/FTConfiguration.hpp>
#include <FaultTolerance/ftcommon/FallbackOid.hpp>
#include <FaultTolerance/ftcommon/UltimateFallbackFactory.hpp>
#include <FaultTolerance/ftcommon/FTManagerGroups.hpp> // for OGID_MIN

#if CDMW_ORB_VDR == tao
#include <FaultTolerance/ftcommon/TAO_IOGRFactory.hpp>
#else
#error "Unsupported ORB"
#endif
#include <string>
#include <fstream>

const char* REP_ID = "IDL:thalesgroup.com/HelloInterface1:1.0";
const ::FT::ObjectGroupId ID = 0xffffff01L;

const std::string HOST("localhost");
const unsigned short int PORT=1234;



CPPUNIT_TEST_SUITE_REGISTRATION( TestFTCommon );
//CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(TestFTCommon, "test_ft_common");

TestFTCommon::TestFTCommon()
{
    m_orb =  CORBA::ORB::_duplicate( Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());
    m_rootPOA = PortableServer::POA::_duplicate( Cdmw::TestUtils::CORBATestManager::instance()->get_POA());
    m_argc = 1;
    m_argv =  Cdmw::Common::String::to_char_array(Cdmw::Common::String::to_strings("Test"));
        
}

TestFTCommon::~TestFTCommon()
{
    for (int i=0; i<m_argc; ++i) {
        delete[] m_argv[i];
    }
    delete[] m_argv;
}

void 
TestFTCommon::do_tests()
{
//     set_nbOfRequestedTestOK(21);
    
    
    try 
    {      
        // create two object hello1
        TEST_INFO("Create three objects");
        Cdmw::HelloInterface1_impl* obj1_impl = new Cdmw::HelloInterface1_impl(m_orb.in());
        Cdmw::HelloInterface1_impl* obj2_impl = new Cdmw::HelloInterface1_impl(m_orb.in());
        Cdmw::HelloInterface2_impl* obj3_impl = new Cdmw::HelloInterface2_impl();
        
        HelloInterface1_var obj1 = obj1_impl->_this();
        HelloInterface1_var obj2 = obj2_impl->_this();
        HelloInterface2_var obj3 = obj3_impl->_this();
        

        ::FT::Location loc1;
        loc1.length(1);
        loc1[0].id="HOST1";
        loc1[0].kind="hostname";

        ::FT::Location loc2;
        loc2.length(1);
        loc2[0].id="HOST2";
        loc2[0].kind="hostname";

        ::CdmwFT::MemberInfo member_info1;
        member_info1.the_reference = CORBA::Object::_duplicate(obj1.in());
        member_info1.the_location = loc1;

        ::CdmwFT::MemberInfo member_info2;
        member_info2.the_reference = CORBA::Object::_duplicate(obj2.in());
        member_info2.the_location = loc2;
        
        TEST_INFO("Create MemberInfos");
        ::CdmwFT::MemberInfos new_members;
        new_members.length(2);
        new_members[0] = member_info1;
        new_members[1] = member_info2;
        
        
        ::CdmwFT::Objects_var no_fallBack = new ::CdmwFT::Objects;     
        no_fallBack->length(0);
        
        ::FT::Location primary_location;
        primary_location.length(0 );

        ::FT::ObjectGroupRefVersion  object_group_ref_version = 1;

        // create an IOGRFactory
        TEST_INFO("Create the IOGRFactory");
        Cdmw::FT::IOGRFactory* m_iogrFactory;

#if CDMW_ORB_VDR == tao
        m_iogrFactory = new Cdmw::FT::TAO_IOGRFactory(m_orb.in(),
                                                      std::cout);
#endif
        ::CdmwFT::MemberInfos no_members;
        no_members.length(0);
        
        TEST_INFO("Try to create an iogr without members");
        try
        {
            CORBA::Object_var iogr = 
            m_iogrFactory->build_iogr(no_members,
                                      primary_location,
                                      ID,
                                      REP_ID,
                                      "my_domain",
                                      object_group_ref_version,
                                      no_fallBack);
            CPPUNIT_ASSERT(false);
        }        
        catch(::FT::ObjectNotCreated)
        {
            CPPUNIT_ASSERT(true);
        }
        catch(...)
        {
            CPPUNIT_ASSERT(false);
        }
        
        
        TEST_INFO("create an iogr with the two previous objects");
        
        CORBA::Object_var iogr = 
        m_iogrFactory->build_iogr(new_members,
                                  primary_location,
                                  ID,
                                  REP_ID,
                                  "my_domain",
                                  object_group_ref_version,
                                  no_fallBack);
        
        
        TEST_INFO("Count the number of profile into the iogr");
        CORBA::ULong nb_profile = m_iogrFactory->profile_count(iogr.in());
        CPPUNIT_ASSERT(nb_profile == 2);

        TEST_INFO("Check if the profile n°1 contains the TAG_FT_GROUP");
        try
        {
            ::FT::TagFTGroupTaggedComponent * tag_group 
              = m_iogrFactory->decode_profile_with_group_tag (iogr.in(),0);
            delete tag_group;
            CPPUNIT_ASSERT(true);
        }
        catch(...)
        {
            CPPUNIT_ASSERT(false);
        }
        
        TEST_INFO("Check if the profile n°2 contains the TAG_FT_GROUP");
        try
        {
            ::FT::TagFTGroupTaggedComponent * tag_group 
              = m_iogrFactory->decode_profile_with_group_tag (iogr.in(),1);
            delete tag_group;
            CPPUNIT_ASSERT(true);
        }
        catch(...)
        {
            CPPUNIT_ASSERT(false);
        }

        TEST_INFO("Check if the profile n°3 doesn't exist");
        try
        {
            ::FT::TagFTGroupTaggedComponent * tag_group 
              = m_iogrFactory->decode_profile_with_group_tag (iogr.in(),2);
            delete tag_group;
            CPPUNIT_ASSERT(false);
        }
        catch(const CdmwFT::ProfileNotFound&)
        {
            CPPUNIT_ASSERT(true);
        }
        catch(...)
        {
            CPPUNIT_ASSERT(false);
        }


        TEST_INFO("Check if no TAG_FT_GROUP is present into a simple ior");
        try
        {
            ::FT::TagFTGroupTaggedComponent * tag_group 
              = m_iogrFactory->decode_profile_with_group_tag (obj1.in(),0);            
            delete tag_group;
            CPPUNIT_ASSERT(false);
        }
        catch(const CdmwFT::TagNotFound&)
        {
            CPPUNIT_ASSERT(true);
        }
        catch(...)
        {
            CPPUNIT_ASSERT(false);
        }


        TEST_INFO("Check if no TAG_FT_PRIMARY is not present into the profile n°1");
        try
        {
            ::FT::TagFTPrimaryTaggedComponent tag_primary 
              = m_iogrFactory->decode_profile_with_primary_tag (iogr.in(),0);            
            CPPUNIT_ASSERT(false);
        }
        catch(const CdmwFT::TagNotFound&)
        {
            CPPUNIT_ASSERT(true);
        }
        catch(...)
        {
            CPPUNIT_ASSERT(false);
        }

        TEST_INFO("Check if no TAG_FT_PRIMARY is not present into the profile n°2");
        try
        {
            ::FT::TagFTPrimaryTaggedComponent tag_primary 
              = m_iogrFactory->decode_profile_with_primary_tag (iogr.in(),1); 
            CPPUNIT_ASSERT(false);
        }
        catch(const CdmwFT::TagNotFound&)
        {
            CPPUNIT_ASSERT(true);
        }
        catch(...)
        {
            CPPUNIT_ASSERT(false);
        }


        TEST_INFO("Build a new iogr with a primary member");
        primary_location.length(1);
        primary_location[0].id="HOST1";
        primary_location[0].kind="hostname";

        iogr = 
        m_iogrFactory->build_iogr(new_members,
                                  primary_location,
                                  ID,
                                  REP_ID,
                                  "my_domain",
                                  object_group_ref_version,
                                  no_fallBack);
        
        
        TEST_INFO("Count the number of profile into the iogr");
        nb_profile = m_iogrFactory->profile_count(iogr.in());
        CPPUNIT_ASSERT(nb_profile == 2);

        TEST_INFO("Check if the profile n°1 contains the TAG_FT_GROUP");
        try
        {
            ::FT::TagFTGroupTaggedComponent * tag_group 
              = m_iogrFactory->decode_profile_with_group_tag (iogr.in(),0);
            delete tag_group;
            CPPUNIT_ASSERT(true);
        }
        catch(...)
        {
            CPPUNIT_ASSERT(false);
        }
        
        TEST_INFO("Check if the profile n°2 contains the TAG_FT_GROUP");
        try
        {
            ::FT::TagFTGroupTaggedComponent * tag_group 
              = m_iogrFactory->decode_profile_with_group_tag (iogr.in(),1);
            delete tag_group;
            CPPUNIT_ASSERT(true);
        }
        catch(...)
        {
            CPPUNIT_ASSERT(false);
        }

        TEST_INFO("Check if the profile n°3 doesn't exist");
        try
        {
            ::FT::TagFTGroupTaggedComponent * tag_group 
              = m_iogrFactory->decode_profile_with_group_tag (iogr.in(),2);
            delete tag_group;
            CPPUNIT_ASSERT(false);
        }
        catch(const CdmwFT::ProfileNotFound&)
        {
            CPPUNIT_ASSERT(true);
        }
        catch(...)
        {
            CPPUNIT_ASSERT(false);
        }


        TEST_INFO("Check if no TAG_FT_GROUP is present into a simple ior");
        try
        {
            ::FT::TagFTGroupTaggedComponent * tag_group 
              = m_iogrFactory->decode_profile_with_group_tag (obj1.in(),0);            
            delete tag_group;
            CPPUNIT_ASSERT(false);
        }
        catch(const CdmwFT::TagNotFound&)
        {
            CPPUNIT_ASSERT(true);
        }
        catch(...)
        {
            CPPUNIT_ASSERT(false);
        }


        TEST_INFO("Check if no TAG_FT_PRIMARY is present into the profile n°1");
        try
        {
            ::FT::TagFTPrimaryTaggedComponent tag_primary 
              = m_iogrFactory->decode_profile_with_primary_tag (iogr.in(),0);            
            CPPUNIT_ASSERT(true);
        }
        catch(...)
        {
            CPPUNIT_ASSERT(false);
        }

        TEST_INFO("Check if no TAG_FT_PRIMARY is not present into the profile n°2");
        try
        {
            ::FT::TagFTPrimaryTaggedComponent tag_primary 
              = m_iogrFactory->decode_profile_with_primary_tag (iogr.in(),1); 
            CPPUNIT_ASSERT(false);
        }
        catch(const CdmwFT::TagNotFound&)
        {
            CPPUNIT_ASSERT(true);
        }
        catch(...)
        {
            CPPUNIT_ASSERT(false);
        }




        member_info2.the_reference = CORBA::Object::_duplicate(obj3.in());
        member_info2.the_location = loc2;
        
        TEST_INFO("Create MemberInfos");
        new_members.length(2);
        new_members[0] = member_info1;
        new_members[1] = member_info2;

        TEST_INFO("Try to create an iogr with different ior type");
        try
        {
            CORBA::Object_var iogr = 
            m_iogrFactory->build_iogr(new_members,
                                      primary_location,
                                      ID,
                                      REP_ID,
                                      "my_domain",
                                      object_group_ref_version,
                                      no_fallBack);
            CPPUNIT_ASSERT(false);
        }        
        catch(const ::FT::ObjectNotCreated&)
        {
            CPPUNIT_ASSERT(true);
        }
        catch(...)
        {
            CPPUNIT_ASSERT(false);
        }

        TEST_INFO("Create MemberInfos");
        new_members.length(2);
        new_members[0] = member_info1;
        new_members[1] = member_info1;

        TEST_INFO("Try to create an iogr with duplicated ior");
        try
        {
            CORBA::Object_var iogr = 
            m_iogrFactory->build_iogr(new_members,
                                      primary_location,
                                      ID,
                                      REP_ID,
                                      "my_domain",
                                      object_group_ref_version,
                                      no_fallBack);
            CPPUNIT_ASSERT(false);
        }        
        catch(const ::FT::ObjectNotCreated&)
        {
            CPPUNIT_ASSERT(true);
        }
        catch(...)
        {
            CPPUNIT_ASSERT(false);
        }

        TEST_INFO("set the client_id on the orb");
        try
        {
            Cdmw::FT::FTConfiguration::Set_client_id(m_orb.in(),"my_client_id");
            CPPUNIT_ASSERT(true);
        }
        catch(...)
        {
            CPPUNIT_ASSERT(false);
        }
        
        TEST_INFO("get the client_id on the orb");
        try
        {
            std::string client_id;
            client_id = Cdmw::FT::FTConfiguration::Get_client_id(m_orb.in());
            std::cout<<"The client_id is :"<<client_id<<std::endl;
            CPPUNIT_ASSERT(client_id == "my_client_id");
        }
        catch(...)
        {
            CPPUNIT_ASSERT(false);
        }

        TEST_INFO("Get the FT_endpoint_selector_factory");
        try
        {
            Cdmw::FT::FTConfiguration::Get_FT_endpoint_selector_factory();
            CPPUNIT_ASSERT(true);
        }
        catch(...)
        {
            CPPUNIT_ASSERT(false);
        }
        
        TEST_INFO("Try to activate the FT service");
        try
        {
            Cdmw::FT::FTConfiguration::Activate_FT_service(m_argc, m_argv);
            CPPUNIT_ASSERT(true);
        }
        catch(...)
        {
            CPPUNIT_ASSERT(false);
        }
        //fixing memory leak ? nope : failed assertion with ref counter
    //obj3_impl->_remove_ref();
#if CDMW_ORB_VDR == tao
        m_iogrFactory->_remove_ref();
#endif  

    } 
    catch (...)
    {
    }
    
}
void 
TestFTCommon::test_FallbackOid()
{
   TEST_INFO("Testing FT FallbackOid class");
   std::string group_domain_id("MyFaultToleranceDomain");
   ::FT::ObjectGroupId  object_group_id(1234L);
   ::FT::ObjectGroupRefVersion group_version(20);
   std::string repository_id("IDL:thalesgroup.com/HelloInterface1:1.0");

   
   Cdmw::FT::FallbackOid foid(group_domain_id.c_str(),
                              object_group_id,
                              group_version,
                              repository_id.c_str(),
                              "");
   CPPUNIT_ASSERT(repository_id == foid.get_repository_id());
   CPPUNIT_ASSERT(group_domain_id == foid.get_group_domain_id());
   CPPUNIT_ASSERT(object_group_id == foid.get_object_group_id());
   CPPUNIT_ASSERT(group_version == foid.get_group_version());
   std::string oid_string = foid.to_string();
   std::string name = foid.name();
   PortableServer::ObjectId_var oid = foid.to_ObjectId();
   TEST_INFO("Fallback ObjectId="<<oid_string);
   TEST_INFO("Fallback name="<<name);
   CORBA::String_var oid_string2 = 
      PortableServer::ObjectId_to_string(oid.in());
   TEST_INFO("oid_string =\t'"<<oid_string<<"'");
   TEST_INFO("oid_string2=\t'"<<oid_string2.in()<<"'");
   CPPUNIT_ASSERT(oid_string == std::string(oid_string2.in()));
   Cdmw::FT::FallbackOid foid2(oid.in());
   CPPUNIT_ASSERT(repository_id == foid2.get_repository_id());
   CPPUNIT_ASSERT(group_domain_id == foid2.get_group_domain_id());
   CPPUNIT_ASSERT(object_group_id == foid2.get_object_group_id());
   CPPUNIT_ASSERT(group_version == foid2.get_group_version());
   CPPUNIT_ASSERT(oid_string == foid2.to_string());
   CPPUNIT_ASSERT(name == foid2.name());
}

void 
TestFTCommon::test_FallbackOid_with_GroupName()
{
   TEST_INFO("Testing FT FallbackOid class with Group Name");
   std::string group_domain_id("MyFaultToleranceDomain");
   ::FT::ObjectGroupId  object_group_id(1234L);
   ::FT::ObjectGroupRefVersion group_version(20);
   std::string repository_id("IDL:thalesgroup.com/HelloInterface1:1.0");
   std::string object_group_name("A/B.kind/C.iogr");

   
   Cdmw::FT::FallbackOid foid(group_domain_id.c_str(),
                              object_group_id,
                              group_version,
                              repository_id.c_str(),
                              object_group_name.c_str());
   CPPUNIT_ASSERT(repository_id == foid.get_repository_id());
   CPPUNIT_ASSERT(group_domain_id == foid.get_group_domain_id());
   CPPUNIT_ASSERT(object_group_id == foid.get_object_group_id());
   CPPUNIT_ASSERT(group_version == foid.get_group_version());
   CPPUNIT_ASSERT(object_group_name == foid.get_object_group_name());
   std::string oid_string = foid.to_string();
   std::string name = foid.name();
   PortableServer::ObjectId_var oid = foid.to_ObjectId();
   TEST_INFO("Fallback ObjectId="<<oid_string);
   TEST_INFO("Fallback name="<<name);
   CORBA::String_var oid_string2 = 
      PortableServer::ObjectId_to_string(oid.in());
   TEST_INFO("oid_string =\t'"<<oid_string<<"'");
   TEST_INFO("oid_string2=\t'"<<oid_string2.in()<<"'");
   CPPUNIT_ASSERT(oid_string == std::string(oid_string2.in()));
   Cdmw::FT::FallbackOid foid2(oid.in());
   CPPUNIT_ASSERT(repository_id == foid2.get_repository_id());
   CPPUNIT_ASSERT(group_domain_id == foid2.get_group_domain_id());
   CPPUNIT_ASSERT(object_group_id == foid2.get_object_group_id());
   CPPUNIT_ASSERT(group_version == foid2.get_group_version());
   CPPUNIT_ASSERT(object_group_name == foid2.get_object_group_name());
   CPPUNIT_ASSERT(oid_string == foid2.to_string());
   CPPUNIT_ASSERT(name == foid2.name());
   // Code bellow uses assertions!
//    TEST_INFO("Testing with an invalid name");
//    try {
//       object_group_name = "A.B.kind/C.iogr";
//       Cdmw::FT::FallbackOid foid3(group_domain_id.c_str(),
//                                  object_group_id,
//                                  group_version,
//                                  repository_id.c_str(),
//                                  object_group_name.c_str());
//       CPPUNIT_ASSERT(false);
//    } catch (const CORBA::BAD_PARAM & ex) {
//       TEST_INFO("Received an exception:'"<< Cdmw::OrbSupport::OrbSupport::exception_to_string(ex) << "'");
//       CPPUNIT_ASSERT(true);
//    }
//    TEST_INFO("Testing with neither group id nor group name");
//    try {
//       object_group_name = "";
//       object_group_id=Cdmw::FT::OGID_MIN;
//       Cdmw::FT::FallbackOid foid3(group_domain_id.c_str(),
//                                  object_group_id,
//                                  group_version,
//                                  repository_id.c_str(),
//                                  object_group_name.c_str());
//       CPPUNIT_ASSERT(false);
//    } catch (const CORBA::BAD_PARAM & ex) {
//       TEST_INFO("Received an exception:'"<< Cdmw::OrbSupport::OrbSupport::exception_to_string(ex) << "'");
//       CPPUNIT_ASSERT(true);
//    }
}

void 
TestFTCommon::test_UltimateFallbackFactory()
{
//   const std::string OIDstring("CDMW_FaultToleranceDomain/grp1.iogr");
   const std::string REPID("IDL:thalesgroup.com/HelloInterface1:1.0");
   const std::string REPID2("IDL:thalesgroup.com/HelloInterface2:1.0");

   std::string group_domain_id("CDMW__FaultToleranceDomain");
   ::FT::ObjectGroupId  object_group_id(1234L);
   ::FT::ObjectGroupRefVersion group_version(20);

   
   Cdmw::FT::FallbackOid foid(group_domain_id.c_str(),
                              object_group_id,
                              group_version,
                              REPID.c_str(),
                              "");

//   PortableServer::ObjectId_var oid = PortableServer::string_to_ObjectId(OIDstring.c_str());
   PortableServer::ObjectId_var oid = foid.to_ObjectId();
   CORBA::String_var oid_s = PortableServer::ObjectId_to_string(oid.in());
   TEST_INFO("Testing FT UltimateFallbackFactory class");
   
   Cdmw::FT::UltimateFallbackFactory * ff = 
      Cdmw::FT::UltimateFallbackFactory::Init(m_orb.in(),
                                              HOST.c_str(),PORT);
   TEST_INFO("Creating a reference with " << REPID << " type_id");
   CORBA::Object_var obj =  
      ff->create_reference_with_user_id(oid,REPID.c_str());
   CPPUNIT_ASSERT(!CORBA::is_nil(obj.in()));
   CORBA::String_var s = m_orb->object_to_string(obj.in());
   std::cerr << s.in()<<std::endl;
   try {
      TEST_INFO("Checking object is of type " << REPID << ".");
      CORBA::Boolean res = obj->_is_a(REPID.c_str());
      CPPUNIT_ASSERT(res == true);
      TEST_INFO("Checking remote object is of type " << REPID2 << ".");
      res = obj->_is_a(REPID2.c_str());
      CPPUNIT_ASSERT(res == false);
      TEST_INFO("Getting remote object OID.");
      HelloInterface1_var obj__ = HelloInterface1::_narrow(obj.in());
      CORBA::String_var oid__ = obj__->get_ObjectId();
      CPPUNIT_ASSERT(std::string(oid_s.in()) == std::string(oid__.in()));
   } catch (const CORBA::Exception& ex) {
      std::cerr << "Caught CORBA Exception:" << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex) << std::endl;
      CPPUNIT_ASSERT(false);
   }
   TEST_INFO("Creating a stringified object key.");
   std::string key = ff->create_object_key(oid);
   std::ostringstream ostr;
   ostr << "corbaloc::" << HOST << ":" << PORT << "/" << key;
   TEST_INFO("create a corbaloc reference '" << ostr.str() << "'");
   try {
      CORBA::Object_var obj = m_orb->string_to_object(ostr.str().c_str());
      CPPUNIT_ASSERT(!CORBA::is_nil(obj.in()));
      TEST_INFO("Getting remote object OID.");
      HelloInterface1_var obj__ = HelloInterface1::_narrow(obj.in());
      CORBA::String_var oid__ = obj__->get_ObjectId();
      CPPUNIT_ASSERT(std::string(oid_s.in()) == std::string(oid__.in()));
      // Shuting down server
      //obj__->shutdown();
   } catch (const CORBA::Exception& ex) {
      std::cerr << "Caught CORBA Exception:" << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex) << std::endl;
      CPPUNIT_ASSERT(false);
   }
   //fixing memory leak 15.9 but cleaning failed assertion in RefCountServantbase.cpp
   //Cdmw::FT::UltimateFallbackFactory::cleanup_UltimateFallbackFactory();
}


void 
TestFTCommon::test_make_ref_object_key()
{
   TEST_INFO("Testing cdmw_make_ref - object-key");
   std::string group_domain_id("MyFaultToleranceDomain");
   ::FT::ObjectGroupId  object_group_id(Cdmw::FT::OGID_MIN);
   ::FT::ObjectGroupRefVersion group_version(0);
   std::string repository_id("");
   //std::string repository_id("IDL:thalesgroup.com/HelloInterface1:1.0");
   std::string object_group_name("A/B.kind/C.iogr");

   
   Cdmw::FT::FallbackOid foid(group_domain_id.c_str(),
                              object_group_id,
                              group_version,
                              repository_id.c_str(),
                              object_group_name.c_str());

   PortableServer::ObjectId_var oid = foid.to_ObjectId();
   CORBA::String_var oid_s = PortableServer::ObjectId_to_string(oid.in());
   
   std::string name(group_domain_id);
   name +="/";
   name += object_group_name;

   const std::string of("f.out");
   std::ostringstream args;
   args << "--name="<<name<<" --kind=object-key --output-file="<<of;
   Cdmw::OsSupport::OS::create_process( "cdmw_make_ref", args.str() ); 
   
   int timescale = Cdmw::TestUtils::get_timescale();
   Cdmw::OsSupport::OS::sleep(timescale*3000);// import the object reference from the file

   std::ifstream is;
   is.open(of.c_str(), std::ifstream::in);
   if (is.good())
   {
      std::string objkey_str;
      is >> objkey_str;
      std::ostringstream ostr;
      ostr << "corbaloc::" << HOST << ":" << PORT << "/" << objkey_str;
      std::cerr << "####################corbaloc=" <<ostr.str()<<std::endl;
      try {
         CORBA::Object_var obj = m_orb->string_to_object(ostr.str().c_str());
         CPPUNIT_ASSERT(!CORBA::is_nil(obj.in()));
         TEST_INFO("Getting remote object OID.");
         HelloInterface1_var obj__ = HelloInterface1::_narrow(obj.in());
         CORBA::String_var oid__ = obj__->get_ObjectId();
         CPPUNIT_ASSERT(std::string(oid_s.in()) == std::string(oid__.in()));
      } catch (const CORBA::Exception& ex) {
         std::cerr << "Caught CORBA Exception:" << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex) << std::endl;
         CPPUNIT_ASSERT(false);
      }
      is.close();
   }
   else
   {
      std::cerr << "Error opening file: '" << of << "'" << std::endl;
      CPPUNIT_ASSERT(false);
   }
}


void 
TestFTCommon::test_make_ref_corbaloc()
{
   TEST_INFO("Testing cdmw_make_ref - corbaloc");
   std::string group_domain_id("MyFaultToleranceDomain");
   ::FT::ObjectGroupId  object_group_id(Cdmw::FT::OGID_MIN);
   ::FT::ObjectGroupRefVersion group_version(0);
   std::string repository_id("");
   //std::string repository_id("IDL:thalesgroup.com/HelloInterface1:1.0");
   std::string object_group_name("A/B.kind/C.iogr");

   
   Cdmw::FT::FallbackOid foid(group_domain_id.c_str(),
                              object_group_id,
                              group_version,
                              repository_id.c_str(),
                              object_group_name.c_str());

   PortableServer::ObjectId_var oid = foid.to_ObjectId();
   CORBA::String_var oid_s = PortableServer::ObjectId_to_string(oid.in());
   
   std::string name(group_domain_id);
   name +="/";
   name += object_group_name;

   const std::string of("f.out");
   std::ostringstream args;
   args << "--name="<<name<<" --kind=corbaloc --host="<<HOST << " --port=" << PORT
        << " --output-file="<<of;
   Cdmw::OsSupport::OS::create_process( "cdmw_make_ref", args.str() ); 
   
   int timescale = Cdmw::TestUtils::get_timescale();
   Cdmw::OsSupport::OS::sleep(timescale*3000);// import the object reference from the file

   std::ifstream is;
   is.open(of.c_str(), std::ifstream::in);
   if (is.good())
   {
      std::string corbaloc_str;
      is >> corbaloc_str;
      // corbaloc_str starts with corbaloc:
      std::string prefix=corbaloc_str.substr(0,9);
      CPPUNIT_ASSERT( prefix == "corbaloc:");
      try {
         CORBA::Object_var obj = m_orb->string_to_object(corbaloc_str.c_str());
         CPPUNIT_ASSERT(!CORBA::is_nil(obj.in()));
         TEST_INFO("Getting remote object OID.");
         HelloInterface1_var obj__ = HelloInterface1::_narrow(obj.in());
         CORBA::String_var oid__ = obj__->get_ObjectId();
         CPPUNIT_ASSERT(std::string(oid_s.in()) == std::string(oid__.in()));
      } catch (const CORBA::Exception& ex) {
         std::cerr << "Caught CORBA Exception:" << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex) << std::endl;
         CPPUNIT_ASSERT(false);
      }
      is.close();
   }
   else
   {
      std::cerr << "Error opening file: '" << of << "'" << std::endl;
      CPPUNIT_ASSERT(false);
   }
}


void 
TestFTCommon::test_make_ref_ior()
{
   TEST_INFO("Testing cdmw_make_ref - ior");
   std::string group_domain_id("MyFaultToleranceDomain");
   ::FT::ObjectGroupId  object_group_id(Cdmw::FT::OGID_MIN);
   ::FT::ObjectGroupRefVersion group_version(0);
   std::string repository_id("IDL:thalesgroup.com/HelloInterface1:1.0");
   std::string object_group_name("A/B.kind/C.iogr");

   
   Cdmw::FT::FallbackOid foid(group_domain_id.c_str(),
                              object_group_id,
                              group_version,
                              repository_id.c_str(),
                              object_group_name.c_str());

   PortableServer::ObjectId_var oid = foid.to_ObjectId();
   CORBA::String_var oid_s = PortableServer::ObjectId_to_string(oid.in());
   
   std::string name(group_domain_id);
   name +="/";
   name += object_group_name;

   const std::string of("f.out");
   std::ostringstream args;
   args << "--name="<<name<<" --kind=ior --host="<<HOST << " --port=" << PORT
        << " --type-id=" << repository_id << " --output-file="<<of;
   Cdmw::OsSupport::OS::create_process( "cdmw_make_ref", args.str() ); 
   
   int timescale = Cdmw::TestUtils::get_timescale();
   Cdmw::OsSupport::OS::sleep(timescale*3000);// import the object reference from the file

   std::ifstream is;
   is.open(of.c_str(), std::ifstream::in);
   if (is.good())
   {
      std::string ior_str;
      is >> ior_str;
      // ior_str starts with IOR:
      std::string prefix=ior_str.substr(0,4);
      CPPUNIT_ASSERT( prefix == "IOR:");
      try {
         CORBA::Object_var obj = m_orb->string_to_object(ior_str.c_str());
         CPPUNIT_ASSERT(!CORBA::is_nil(obj.in()));
         TEST_INFO("Getting remote object OID.");
         HelloInterface1_var obj__ = HelloInterface1::_narrow(obj.in());
         CORBA::String_var oid__ = obj__->get_ObjectId();
         CPPUNIT_ASSERT(std::string(oid_s.in()) == std::string(oid__.in()));
      } catch (const CORBA::Exception& ex) {
         std::cerr << "Caught CORBA Exception:" << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex) << std::endl;
         CPPUNIT_ASSERT(false);
      }
      is.close();
   }
   else
   {
      std::cerr << "Error opening file: '" << of << "'" << std::endl;
      CPPUNIT_ASSERT(false);
   }
}

