/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/Exception.hpp>
#include <Foundation/common/String.hpp>
#include <Foundation/common/Assert.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <FaultTolerance/ftcommon/UltimateFallbackFactory.hpp>
#include <FaultTolerance/ftcommon/FallbackOid.hpp>
#include "testftcommon/TestHello_impl.hpp"
#include <cstdlib>



namespace
{    
   const int SUCCESS = 0;
   const int FAILURE = 1;
   const std::string DUMMY_HOST("localhost");
   const unsigned short int DUMMY_PORT = 1234;

   class TheServantLocator :
         public virtual PortableServer::ServantLocator,
         public virtual Cdmw::OrbSupport::RefCountLocalObject
   {
      public:
         
         TheServantLocator(CORBA::ORB_ptr orb)
            : m_orb(CORBA::ORB::_duplicate(orb))
         {
         }
         ~TheServantLocator()
         {
         }
         
         PortableServer::Servant
         preinvoke(const PortableServer::ObjectId& oid,
                   PortableServer::POA_ptr poa,
                   const char* operation,
                   PortableServer::ServantLocator::Cookie&
                   the_cookie)
            throw(CORBA::SystemException)
         {
            CORBA::String_var s = PortableServer::ObjectId_to_string(oid);
            std::cerr << " --- " << __FUNCTION__ << "::" << operation << "(oid=" << s.in() << ")" << std::endl;

            Cdmw::FT::FallbackOid foid(oid);
            std::cerr << __FUNCTION__ << "::" << operation << " foid.to_string()=" << foid.to_string() << std::endl;
            std::cerr << __FUNCTION__ << "::" << operation << " foid.name()=" << foid.name() << std::endl;
            return new Cdmw::HelloInterface1_impl(m_orb.in());
         }
         void
         postinvoke(const PortableServer::ObjectId& oid,
                    PortableServer::POA_ptr poa,
                    const char* operation,
                    PortableServer::ServantLocator::Cookie the_cookie,
                    PortableServer::Servant the_servant)
            throw(CORBA::SystemException)
         {
            // do nothing!
            the_servant->_remove_ref();
         }
      private:
         CORBA::ORB_var m_orb;
   }; // End TheServantLocator
   
   
};

int main (int argc, char* argv[])
{
   int status = SUCCESS;
   CORBA::ORB_var orb;

   try
   {
      Cdmw::OrbSupport::StrategyList strategyList;
      strategyList.add_OrbThreaded();
      strategyList.add_PoaThreadPool( 10 );
      strategyList.add_multicast();

      orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, strategyList ); 
      
      // Activate the POA manager
      CORBA::Object_var obj
         = orb->resolve_initial_references( "RootPOA" );
      PortableServer::POA_var rootPOA = PortableServer::POA::_narrow( obj.in() );
      
      PortableServer::POAManager_var poaMgr = rootPOA->the_POAManager();
      poaMgr->activate();
   
      PortableServer::ServantLocator_var locator = new TheServantLocator(orb.in());

      (void*)Cdmw::FT::UltimateFallbackFactory::Init(orb.in(),
                                                     DUMMY_HOST.c_str(),DUMMY_PORT,
                                                     locator.in());
      orb->run();
      status=SUCCESS;
   }
   catch(const CORBA::Exception& ex)
   {
      std::cerr << "["<<__LINE__<<"] Unexpected CORBA::Exception:\n"
                << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex)
                << std::endl;
      status = FAILURE;
   } catch (const std::exception& ex)
   {
      std::cerr << "["<<__LINE__<<"] Unexpected std exception:\n"
                << ex.what() << std::endl;
      status = FAILURE;
   }
   
   if(!CORBA::is_nil(orb.in()))
   {
      try
      {
         Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
         orb -> destroy();
      }
      catch(const CORBA::Exception& ex)
      {
         std::cerr << "["<<__LINE__<<"] Unexpected CORBA::Exception:\n"
                   << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex)
                   << std::endl;
         status = FAILURE;
      }
   }
}

