/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "testftcommon/TestFTCommon.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/common/Options.hpp"
#include "Foundation/common/Exception.hpp"
#include "Foundation/common/String.hpp"
#include <Foundation/common/Assert.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/ossupport/OS.hpp>


class TestThread : public Cdmw::OsSupport::Thread
{
   public:
      TestThread(CORBA::ORB_ptr orb)
         : m_orb(CORBA::ORB::_duplicate(orb))
      {
      }
      virtual ~TestThread() throw() {}
      
      virtual void run() throw()
      {
         m_orb->run();
      }
   private:
      CORBA::ORB_var m_orb;
};

int main (int argc, char* argv[])
{
   int result = 0;
   //fixing memory leak
   CORBA::ORB_var orb = CORBA::ORB::_nil(); 
   Cdmw::OsSupport::OS::ProcessId idSvc = 0;
   try {
      idSvc = Cdmw::OsSupport::OS::create_process( "test_ft_common_svc", "--CdmwProcessPort=1234" ); 
      int timescale = Cdmw::TestUtils::get_timescale();
      Cdmw::OsSupport::OS::sleep(timescale*3000);

      Cdmw::OrbSupport::StrategyList strategyList;
      strategyList.add_OrbThreaded();
      strategyList.add_PoaThreadPool( 10 );
      strategyList.add_multicast();
      CORBA::ORB_var orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, strategyList ); 
      
      // Activate the POA manager
      CORBA::Object_var obj
         = orb->resolve_initial_references( "RootPOA" );
      PortableServer::POA_var rootPOA = PortableServer::POA::_narrow( obj.in() );
      
      PortableServer::POAManager_var poaMgr = rootPOA->the_POAManager();
      poaMgr->activate();
//      TestThread * thr = new TestThread(orb.in());
//      thr->start();
      Cdmw::OsSupport::OS::sleep(timescale*1000);
      
      Cdmw::TestUtils::CORBATestManager::instance()->init(orb.in(),rootPOA.in(),argc,argv);
      Cdmw::TestUtils::CORBATestManager::instance()->run_tests();
      Cdmw::TestUtils::CORBATestManager::instance()->summary();
      
      
//      orb->shutdown(false);   
//      thr->join();
      if (!CORBA::is_nil(orb.in()))
      {
        Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
        orb->destroy();
      }
      
      result = Cdmw::TestUtils::CORBATestManager::instance()->was_successful();
      
      
   } catch (const CORBA::SystemException& e) {
      std::cerr << "Unexpected CORBA::SystemException:\n"
                << e << " - minor code: " << e.minor()
                << std::endl;
   }   
   // And we kill the Name Server
   try {
      Cdmw::OsSupport::OS::kill_process( idSvc );
   } catch (const std::exception& ex)
   {
      std::cerr << "["<<__LINE__<<"] Unexpected std exception:\n"
                << ex.what() << std::endl;
   }
   return result;
}

