/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ftinit;

import cdmw.ft.watchdog.FTRequestTimeoutORBInitializer;
import cdmw.logging.LogManager;
import static cdmw.logging.Logger.*;
import cdmw.ft.FTLogger;
import cdmw.orbsupport.ExceptionMinorCodes;


public class FTServiceInit {
    
    /**
     * Purpose: <p> Initialises ORB FT mechanism and creates a group repository
     * and server request interceptor able to manage FT failover mechanism.
     * 
     * @param args
     *            The list of parameter received of the command line
     * @param client_only
     *            Set to true to limit initialisation to client side init. 
     *            Note that only client FT is supported in Java at this time.
     * 
     * @return none
     * 
     * @exception CORBA::SystemException
     *                Any CORBA system exception that may be raised by the
     *                underlying ORB.
     */
    public static void Init(String[] args, boolean clientOnly) {
        try {
            // Init the Logging library
            LogManager.Init(args);
        }
        catch (Exception e) {
            CDMW_ERROR(FTLogger.GetLogger(), 
               "Cannot initialise CARDAMOM SystemProperties or Logging library! \n"
               + "Caught exception : " + e.getMessage(),
               new Throwable());
            throw new org.omg.CORBA.BAD_PARAM(ExceptionMinorCodes.BAD_PARAM,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        cdmw.ft.FTConfiguration.ActivateFTService(args);

        if (clientOnly == false) {
            CDMW_ERROR(FTLogger.GetLogger(), 
               "Fault Tolerance for server side not implemented in Java.",
               new Throwable());
            throw new org.omg.CORBA.NO_IMPLEMENT(
                ExceptionMinorCodes.NO_IMPLEMENTNotYetImplemented,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
        else {
            // do nothing!
        }
    }
}