/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_CDMW_FT_LOCATION_STATEFULL_PRIMARY_BACKUP_ADMIN_IMPL_HPP
#define INCL_CDMW_FT_LOCATION_STATEFULL_PRIMARY_BACKUP_ADMIN_IMPL_HPP

// Cdmw Files
#include <Foundation/orbsupport/RefCountServantBase.hpp>
#include <FaultTolerance/ftlocationmanager/PrimaryBackupAdmin_impl.hpp>


#include <idllib/CdmwFTMembersAdmin.skel.hpp>
#include <idllib/CdmwFTStateFullPrimaryBackupAdmin.skel.hpp>
#include <idllib/CdmwFTStateTransfer.stub.hpp>
#include <functional>
#include <list>
#include <set>
#include <map>

//
// IDL:thalesgroup.com/CdmwFT:1.0
//
namespace Cdmw
{

namespace FT
{

//
// IDL:thalesgroup.com/CdmwFT/Location:1.0
//
namespace Location
{


//
// IDL:thalesgroup.com/CdmwFT/Location/StatefullPrimaryBackupAdmin:1.0
//
class StatefullPrimaryBackupAdmin_impl : virtual public POA_CdmwFT::Location::StatefullPrimaryBackupAdmin,
                                         public PrimaryBackupAdmin_impl
{

   public:
      
      StatefullPrimaryBackupAdmin_impl(CORBA::ORB_ptr orb,
                                       PortableServer::POA_ptr poa, 
                                       const ::FT::Location& the_location,
                                       PrimaryBackupGroupRepository_impl* group_repository,
                                       bool cold_restart,
                                       bool is_ftmanager = false);
      
      ~StatefullPrimaryBackupAdmin_impl();



      //
      // IDL:thalesgroup.com/CdmwFT/Location/StatefullPrimaryBackupAdmin/date_statefull_group_view:1.0
      //
      virtual void update_statefull_group_view(const CdmwFT::Location::StatefullGroupInfos& statefull_group_infos)
         throw(::FT::ObjectGroupNotFound,
               CdmwFT::Location::UnknownDataStoreID,
               CdmwFT::StateTransfer::InvalidDataStoreState,
               CORBA::SystemException);
      
      //
      // IDL:thalesgroup.com/CdmwFT/Location/StatefullPrimaryBackupAdmin/update_statefull_primary:1.0
      //
      virtual void update_statefull_primary(const CdmwFT::Location::StatefullGroupInfos&  statefull_group_infos,
                                            const ::FT::Location& primary_location)
         throw(::FT::ObjectGroupNotFound,
               CdmwFT::Location::UnknownDataStoreID,
               CdmwFT::StateTransfer::InvalidDataStoreState,
               CORBA::SystemException);
      
      //
      // IDL:thalesgroup.com/CdmwFT/Location/StatefullPrimaryBackupAdmin/init_backup_data_stores:1.0
      //
      virtual void init_backup_data_stores(const CdmwFT::StateTransfer::DataStoreInfo& primary_data_store_infos)
         throw(CdmwFT::Location::InitFailed,
               CdmwFT::Location::AlreadyDone,
               CORBA::SystemException);
      
      //
      // IDL:thalesgroup.com/CdmwFT/Location/StatefullPrimaryBackupAdmin/init_primary_data_stores:1.0
      //
      virtual void init_primary_data_stores(const CdmwFT::StateTransfer::LocalDataStoreInfos& local_datastore_infos)
         throw(CdmwFT::Location::InitFailed,
               CdmwFT::Location::AlreadyDone,
               CORBA::SystemException);
      
      //
      // IDL:thalesgroup.com/CdmwFT/Location/StatefullPrimaryBackupAdmin/handle_backup_insertion_failure:1.0
      //
      virtual void handle_backup_insertion_failure(const ::FT::Location& failed_location)
         throw(CORBA::SystemException);
      
      
      
   private:
      StatefullPrimaryBackupAdmin_impl();
      StatefullPrimaryBackupAdmin_impl(const StatefullPrimaryBackupAdmin_impl&);
      void operator=(const StatefullPrimaryBackupAdmin_impl&);
      
      void update_from_group_info(const CdmwFT::Location::StatefullGroupInfos&  statefull_group_infos,
                                  bool is_update_primary = false)
         throw(::FT::ObjectGroupNotFound,
               CdmwFT::Location::UnknownDataStoreID,
               CdmwFT::StateTransfer::InvalidDataStoreState,
               CORBA::SystemException);
      
      
      // list of datastore id already initialised
      typedef std::set < ::CdmwFT::Transaction::DataStoreID >  DataStoreIDSet;
      DataStoreIDSet m_init_primary_datastore_id;
      DataStoreIDSet m_init_backup_datastore_id;
      
      typedef std::set < std::string > LocationSet;
      typedef std::map < Cdmw::FT::DataStoreContext * , LocationSet > ContextLocationSetMap;
      
      
      struct TPCData {
            CdmwFT::Transaction::TPCCoordinator_var m_coordinator;
            CdmwFT::Transaction::TPCCohort_var m_cohort;
            
            TPCData (CdmwFT::Transaction::TPCCoordinator_ptr coordinator,
                     CdmwFT::Transaction::TPCCohort_ptr cohort)
               : m_coordinator(CdmwFT::Transaction::TPCCoordinator::_duplicate(coordinator)),
                 m_cohort(CdmwFT::Transaction::TPCCohort::_duplicate(cohort))
            {
            }
            TPCData ()
               : m_coordinator(CdmwFT::Transaction::TPCCoordinator::_nil()),
                 m_cohort(CdmwFT::Transaction::TPCCohort::_nil())
            {
            }
            TPCData(const TPCData & tpc)
               : m_coordinator(tpc.m_coordinator),
                 m_cohort(tpc.m_cohort)
            {
            }
            TPCData & operator=(const TPCData & rhs)
            {
               if (this != & rhs) {
                  m_coordinator = rhs.m_coordinator;
                  m_cohort = rhs.m_cohort;
               }
               return *this;
            }
      };
      
      struct AddLocationKey
      {
            Cdmw::FT::DataStoreContext * m_context;
            std::string                  m_location;
            
            AddLocationKey(Cdmw::FT::DataStoreContext * context,
                           const char* location)
               : m_context(context),
                 m_location(location) 
            {}
            AddLocationKey(Cdmw::FT::DataStoreContext * context,
                           const std::string & location)
               : m_context(context),
                 m_location(location) 
            {}
            AddLocationKey(const AddLocationKey & alk)
               : m_context(alk.m_context),
                 m_location(alk.m_location) 
            {}
            AddLocationKey()
               : m_context(0),
                 m_location("") 
            {}
            AddLocationKey & operator=(const AddLocationKey & rhs)
            {
               if (this != & rhs) {
                  m_context = rhs.m_context;
                  m_location = rhs.m_location;
               }
               return *this;
            }
      };

      
      struct  AddLocationKey_less
      {
            bool operator() 
               (const AddLocationKey& x, 
                const AddLocationKey& y) const
            {
               if (x.m_context < y.m_context)
               {
                  return true;
               }
               else if (x.m_context == y.m_context)
               {
                  return (x.m_location.compare(y.m_location) < 0);
               }
               else
                  return false;
            };
      };
      typedef std::map<AddLocationKey,TPCData, AddLocationKey_less > AddLocationDataMap;


      PortableServer::POA_var m_poa;
      
      // repository containing group information and the list of primary members
      PrimaryBackupGroupRepository_impl* m_primary_backup_group_repository;
      
      bool          m_is_ftmanager; 
      
      // is cold or hot initialisation
      // 0 is for the cold 
      // 1 is for the hot
      bool m_init_type;
      
      std::string m_primary_location;
      
      ::FT::Location m_the_location;
      typedef std::set< Cdmw::FT::DataStoreContext * > DataStoreContextSet;
      DataStoreContextSet m_contexts;
};

} // End of namespace Location

} // End of namespace FT

} // End of namespace Cdmw

#endif  // INCL_CDMW_FT_LOCATION_STATEFULL_PRIMARY_BACKUP_ADMIN_IMPL_HPP

