/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "ftmanager/ServiceLocator_impl.hpp"
#include "idllib/FT.stub.hpp"
#include "ftstatemanager/DataStore.hpp"

#include <FaultTolerance/ftcommon/FTLogger.hpp>
#include <FaultTolerance/ftcommon/FTObjectIds.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <Foundation/common/Assert.hpp>

namespace Cdmw {
namespace FT {

   ServiceLocator_impl::ServiceLocator_impl(CORBA::ORB_ptr orb, const char* group_domain, int dsid)
      : m_orb(CORBA::ORB::_duplicate(orb)),
        m_group_domain(group_domain)
    {
        CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
        CDMW_ASSERT(group_domain != 0)
        m_iogrInfoStorageHome = new IOGRInfoStorageHome(dsid);
    }
    
    
    ServiceLocator_impl::~ServiceLocator_impl()
    {
        CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
    }
    

    //
    // location service 
    //
    void ServiceLocator_impl::locate_service (const char * name,
                                              const char * sender)
        throw(CORBA::SystemException)
        
    {
        CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"name="<<name<<",sender="<<sender);

        CORBA::Object_var service_registration_group = m_orb->string_to_object(sender);

       ::CdmwServiceLocator::ServiceRegistration_var sender_ = 
            CdmwServiceLocator::ServiceRegistration::_unchecked_narrow (service_registration_group.in());

        CDMW_DEBUG(FTLogger::GetLogger(),"locate_service "<<name);

        try {
           
           // get a storage object for the fault notifier
           IOGRInfoStorageObject objIOGRInfo = m_iogrInfoStorageHome->find_by_oid(1);
           // get the data
           ::CdmwFT::IOGRInfo_var iogrInfo = objIOGRInfo.get();

           std::string sname(name);
           CORBA::Object_var obj = CORBA::Object::_nil();
           if (sname == FAULT_NOTIFIFER_ObjectId)
              obj = iogrInfo->fault_notifier;
           else if (sname == FAULT_DETECTOR_ObjectId)
              obj = iogrInfo->fault_detector;
           else if (sname == REPLICATION_MANAGER_ObjectId)
              obj = iogrInfo->replication_manager;
           else if (sname == m_group_domain) {
              // NOTA: Assume for the moment that the ReplicationManager
              // is the domain manager
              obj = iogrInfo->replication_manager;
           }
           // return the reference of the object group
           if (!CORBA::is_nil(obj.in()))
                sender_->register_service(name, obj._retn());
        }
        catch(const Cdmw::FT::NotFoundException & ex)
        {
           CDMW_DEBUG(FTLogger::GetLogger(),"Received exception '" << ex << "'. Ignored.");
           // do nothing
        }
    }


    
}
}
