/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

// Std include files
#include <iostream>
#include <sstream>
#include <string.h>

// CORBA include files
#include <Foundation/ossupport/OS.hpp>
#include "Foundation/orbsupport/StrategyList.hpp"
#include <Foundation/orbsupport/OrbSupport.hpp>

#include "testacknowledgingsimple/OrbThread.hpp"

#include "testacknowledgingsimple/TestAcknowledgingSimpleMonitoring.hpp"
#include "ftmonitoring/MonitoringLibraryInterface_impl.hpp"
#include "Foundation/common/String.hpp"

using namespace std;

CPPUNIT_TEST_SUITE_REGISTRATION( TestAcknowledgingSimpleMonitoring );

TestAcknowledgingSimpleMonitoring::TestAcknowledgingSimpleMonitoring ()
{
    int argc = 5;
    char** argv = Cdmw::Common::String::to_char_array(Cdmw::Common::String::to_strings("Test --SIMPLE_PULL_CORBALOC corbaloc:miop:1.0@1.0-testmonitoringlib-1/225.1.1.8:4998 --SIMPLE_PUSH_CORBALOC corbaloc:miop:1.0@1.0-testmonitoringlib-2/225.1.1.8:4999"));


    for (int i=0; i<argc; i++) 
        m_argList.push_back(argv[i]);


    m_orb =  CORBA::ORB::_duplicate( Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());
    
    m_timeOn = ::atoi(Cdmw::TestUtils::CORBATestManager::instance()->get_property ("keypressed").c_str());

}

TestAcknowledgingSimpleMonitoring::~TestAcknowledgingSimpleMonitoring ()
{
    m_argList.clear();
}
    
void 
TestAcknowledgingSimpleMonitoring::do_tests()
{
                        
//     set_nbOfRequestedTestOK(9);
    using namespace CdmwFT;

    try {

    cout << "Start Orb thread" << std::endl;
    OrbThread orbThread(m_orb.in());
    orbThread.start();
    Cdmw::OsSupport::OS::sleep(50);
    
    TEST_INFO("Get RootPoa");
    CORBA::Object_var object = m_orb->resolve_initial_references("RootPOA");
    PortableServer::POA_var poa = PortableServer::POA::_narrow(object.in());
    CPPUNIT_ASSERT(true);

    TEST_INFO("Get Poa Manager");
    PortableServer::POAManager_var poaManager =
    poa->the_POAManager();
    CPPUNIT_ASSERT(true);

    TEST_INFO("Activate Poa Manager");
    poaManager->activate();
    CPPUNIT_ASSERT(true);

    TEST_INFO("Init MonitoringLibraryInterface");
        MonitoringLibrary::MonitoringLibraryInterface_impl::Init(m_orb.in(), m_argList);
        CPPUNIT_ASSERT(true);

    TEST_INFO("Resolve initial reference \"MonitoringLibrary\"");
    CORBA::Object_var monitLibObj = m_orb->resolve_initial_references("MonitoringLibrary");
    CPPUNIT_ASSERT(true);
        
    TEST_INFO("Get the reference of the monitoring library");
    MonitoringLibrary::MonitoringLibraryInterface_var lib = 
    MonitoringLibrary::MonitoringLibraryInterface::_narrow(monitLibObj.in());
    CPPUNIT_ASSERT(true);
    
    TEST_INFO("Create the acknowledging simple monitoring");
    MonitoringLibrary::SimpleMonitoring::AcknowledgingSimpleMonitoring_var ackSimpleLib =
      lib.in()->create_acknowledging_simple_monitoring();
    CPPUNIT_ASSERT(true);
            
    
    TEST_INFO("Start the acknowledging simple monitoring");
    ackSimpleLib->start_monitoring();
    CPPUNIT_ASSERT(true);
        
    if (m_timeOn == 0)
    {
          char dummy;
          cin >> dummy;
        }
        else Cdmw::OsSupport::OS::sleep(m_timeOn);

    TEST_INFO("Stop the acknowledging simple monitoring");
    ackSimpleLib->stop_monitoring();
    CPPUNIT_ASSERT(true);
         
        Cdmw::OsSupport::OS::sleep(50);

    cout << "Stop Orb Thread" << std::endl;
    orbThread.destroy();
    orbThread.join();

    } catch (CORBA::Exception &e) {
    std::ostringstream info_stream1;
    info_stream1 << "CORBA Exception caught : " << e << endl;
    TEST_INFO(info_stream1.str());
        CPPUNIT_ASSERT(false);  
    } catch (...) {
    std::ostringstream info_stream2;
    info_stream2 << "Supervision Exception caught : " << endl;
    TEST_INFO(info_stream2.str());  
        CPPUNIT_ASSERT(false);
    }
    
}

