/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

// Std include files
#include <iostream>
#include <sstream>
#include <string.h>
#include <Foundation/orbsupport/OrbSupport.hpp>

#include <Foundation/ossupport/OS.hpp>

#include "testacknowledgingsimple/TestAcknowledgingSimpleMonitoring.hpp"

using namespace std;

int main (int argc, char* argv[])
{
    
    std::string timeOn;
    std::string timeOnOpt =
      Cdmw::OsSupport::OS::get_option_value(argc, argv, "--timeOn");

    if (timeOnOpt == "no")
    {
      cout << "Acknowledging Simple Monitoring will run during 500 ms" << std::endl;
      timeOn=500;
    }
    else if (timeOnOpt == "keypressed")
    {
      cout << "Acknowledging Simple Monitoring on until key pressed" << std::endl;
      cout << "When you have finish to read the message, press key + enter" << std::endl;
      char dummy;
      cin >> dummy;
      timeOn="0";
    }
    else
    {
      timeOn=timeOnOpt;
      cout << "Acknowledging Simple Monitoring will run during "
           << timeOn << " milliseconds" << std::endl;
    }

    Cdmw::OrbSupport::StrategyList strategyList;
    strategyList.add_OrbThreaded(); // Mandatory in OrbSupport::ORB_init
    strategyList.add_multicast();
    // Initialises the ORB
    // fixing memory leak : initialisation separated from declarations
    CORBA::ORB_var orb = CORBA::ORB::_nil();
    orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, strategyList, "main_orb");
    
    // Get the root POA
    CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
    PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj.in());

    Cdmw::TestUtils::CORBATestManager::instance()->init(orb.in(),rootPOA.in(),argc,argv);
    Cdmw::TestUtils::CORBATestManager::instance()->set_property ("keypresses",timeOn);
    Cdmw::TestUtils::CORBATestManager::instance()->run_tests();
    Cdmw::TestUtils::CORBATestManager::instance()->summary();
    //fixing memory leak
    if (! CORBA::is_nil(orb.in()))
    {
        try
        {
           Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
           orb->destroy();
       orb = CORBA::ORB::_nil();
    }
        catch( const CORBA::Exception &e )
        {
            cerr <<__FILE__<<" "<<__LINE__<<" "<< e << endl;
        }
        catch( const Cdmw::Exception &e )
        {
            cerr <<__FILE__<<" "<<__LINE__<<" "<< e << endl;
        }
        catch(...)
        {
            cerr << "Uncaught exception" << endl;
        }
    }   
    return Cdmw::TestUtils::CORBATestManager::instance()->was_successful();
}

