/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

// Std include files
#include <iostream>
#include <sstream>
#include <string.h>
#include <Foundation/orbsupport/OrbSupport.hpp>

#include "testmutual/TestMonitoringLibrary.hpp"

int main (int argc, char* argv[])
{
    

    Cdmw::OrbSupport::StrategyList strategyList;
    strategyList.add_OrbThreaded(); // Mandatory in OrbSupport::ORB_init
    strategyList.add_multicast();
    // Initialises the ORB
    //fixing memory leak : initialisation to nil before operations
    CORBA::ORB_var orb = CORBA::ORB::_nil();
    orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, strategyList, "main_orb");

    // Get the root POA
    CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
    PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj.in());

    Cdmw::TestUtils::CORBATestManager::instance()->init(orb.in(),rootPOA.in(),argc,argv);
    Cdmw::TestUtils::CORBATestManager::instance()->run_tests();
    Cdmw::TestUtils::CORBATestManager::instance()->summary();
    //fixing memory leak
    if (! CORBA::is_nil(orb.in()))
    {
        try
        {
           Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
           orb->destroy();  
        }
        catch( const CORBA::Exception &e )
        {
            std::cerr <<__FILE__<<" "<<__LINE__<<" "<< e << std::endl;
        }
        catch( const Cdmw::Exception &e )
        {
            std::cerr <<__FILE__<<" "<<__LINE__<<" "<< e << std::endl;
        }
        catch(...)
        {
            std::cerr << "Uncaught exception" << std::endl;
        }   
    }    
    
    return Cdmw::TestUtils::CORBATestManager::instance()->was_successful();
}

