/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef _CDMW_FT_REPLICATION_MANAGER_OGDOBSERVER_IMPL_HPP_
#define _CDMW_FT_REPLICATION_MANAGER_OGDOBSERVER_IMPL_HPP_

#include "ftreplicationmanager/ReplicationManagerDefinition.hpp"
#include "ftreplicationmanager/ReplicationManager_impl.hpp"

namespace Cdmw {
namespace FT {
namespace ReplicationManager 
{

    class OGDObserver: public ::Cdmw::CommonSvcs::DataStore::DataStoreObserver
    < ::FT::ObjectGroupId, ::CdmwFT::ObjectGroupData >
    {
    public:
        typedef Cdmw::OrbSupport::type_traits< ::FT::ObjectGroupId > ::_in_type _Oid_in_type;
        typedef Cdmw::OrbSupport::type_traits< ::CdmwFT::ObjectGroupData > ::_in_type _Data_in_type;
        
        OGDObserver(ReplicationManager_impl* rep_mgr)
            : m_rep_mgr(rep_mgr)
        {
        }
        
        ~OGDObserver() throw() {}
        
        virtual void on_insert(_Oid_in_type oid, _Data_in_type data)
            throw ()
        {
            try {
                if (! m_rep_mgr->is_primary())
                    m_rep_mgr->create_grp_ctxt_from_data(oid, data);
            } catch (...) {
            }
        }
        
        virtual void on_update(_Oid_in_type oid, _Data_in_type data)
            throw ()
        {
            try {
                if (! m_rep_mgr->is_primary())
                    m_rep_mgr->update_grp_ctxt_from_data(oid, data);
            } catch (...) {
            }
        }
        
        virtual void on_remove(_Oid_in_type oid)
            throw ()
        {
            try {
                if (! m_rep_mgr->is_primary())
                    m_rep_mgr->remove_grp_ctxt_from_data(oid);
            } catch (...) {
            }
        }
        
    private:
        ReplicationManager_impl* m_rep_mgr;
    };
    
}; // namespace ReplicationManager
}; // namespace FT
}; // namespace Cdmw

#endif // _CDMW_FT_REPLICATION_MANAGER_OGDOBSERVER_IMPL_HPP_

