/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef _CDMW_FT_REPLICATION_MANAGER_OBJECTGROUPDATA_PRINTER_IMPL_HPP_
#define _CDMW_FT_REPLICATION_MANAGER_OBJECTGROUPDATA_PRINTER_IMPL_HPP_

#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include "ftreplicationmanager/ReplicationManager_impl.hpp"
#include "ftreplicationmanager/ReplicationManagerDefinition.hpp"


namespace Cdmw {
namespace FT {


class ObjectGroupDataPrinter
{
public:
    ObjectGroupDataPrinter(std::ostream & os) : m_os(os) { }         
    void operator()(const std::pair< Cdmw::OrbSupport::type_traits< ::FT::ObjectGroupId >::_var_type, 
                                     Cdmw::OrbSupport::type_traits< ::CdmwFT::ObjectGroupData >::_var_type > & x)
    {
        m_os << "GROUP ID : " << x.first << "\n";

        m_os << "\tRepository Id : " << (x.second)->repository_id << "\n"
           << "\tGroup version : " << (x.second)->object_group_ref_version << "\n";

        // Get primary location
        ::FT::Location_var primary_loc =
          get_group_primary_location(x.second.in());
        using namespace Cdmw::CommonSvcs::Naming;
        if (primary_loc->length() > 0L) {
            std::string s_primary
                = NamingInterface::to_string(primary_loc.in());
            m_os << "\tPrimary location :'" << s_primary  << "'\n";
        } else 
            m_os << "\tThere is no primary location" << "\n";
        
        m_os << "\tGroup has " <<  (x.second)->members.length() << " member(s) \n";
        for (CORBA::ULong i = 0L; i < (x.second)->members.length(); ++i) {
            const std::string loc =
                NamingInterface::to_string((x.second)->members[i].the_location);
            m_os << "\t\tLocation : '" <<  loc << "'\n";
        }
        m_os << std::endl;
    }
    
private:
    ::FT::Location *
    get_group_primary_location(const ::CdmwFT::ObjectGroupData &x) const
        throw ()
    {
        // This may appear in dynamic_properties
        using namespace Cdmw::CommonSvcs::Naming;
        for (unsigned int i = 0; i<x.dynamic_properties.length(); i++)
        {
            std::string s_property
            = std::string(x.dynamic_properties[i].nam[0].id);
            const std::string property_name = PRIMARY_LOCATION_PROPERTY_NAME;
            
            if(s_property == property_name)
            {
                const ::FT::Location * val;
                // PW 10-04-2004: added any extraction result check
                if (x.dynamic_properties[i].val >>= val) 
                    return new ::FT::Location(*val);
                else
                    break; // Returns empty location
            }                
        }
        ::FT::Location_var no_location = new ::FT::Location();
        no_location->length(0L);
        return no_location._retn();
    }
    
    std::ostream & m_os;
};


    
}; // namespace FT
}; // namespace Cdmw

#endif // _CDMW_FT_REPLICATION_MANAGER_OBJECTGROUPDATA_PRINTER_IMPL_HPP_

