/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef _CDMW_FT_REPLICATION_MANAGER_DEFINITION_IMPL_HPP_
#define _CDMW_FT_REPLICATION_MANAGER_DEFINITION_IMPL_HPP_

#include "CdmwFTObjectGroupData.stub.hpp"
#include "CdmwFTLocationInfo.stub.hpp"
#include <Foundation/orbsupport/TypeTraits.hpp>

DEFINE_VARIABLE_LENGTH_DATA_TRAITS(::CdmwFT::ObjectGroupData);
DEFINE_VARIABLE_LENGTH_DATA_TRAITS(::CdmwFT::LocationInfo);



#define ENABLE_ACCEPT_DELETE_OBJECT_IF_EMPTY_GROUP  1


// TODO: inforcing minimum number of replicas
//       B) remove_member() and delete_object() may elect a new primary replica if needed.

#ifndef ENABLE_DEBUG_DUMP

#   define DEBUG_DUMP(comment,value_type) 
#   define DEBUG_DUMPALL(comment)

#else

#   define DEBUG_DUMP(comment,value_type) \
do {\
    if (FTLogger::GetLogger()->is_debug_enabled()) {\
        std::ostringstream oss__;\
        oss__ << #comment << std::endl;\
        Cdmw::FT::ObjectGroupDataPrinter print(oss__);\
        print(value_type);\
        CDMW_DEBUG(FTLogger::GetLogger(),oss__.str());\
    }\
} while(0)

#   define DEBUG_DUMPALL(comment) \
do {\
    if (FTLogger::GetLogger()->is_debug_enabled()) {\
        std::ostringstream oss__;\
        oss__ << #comment << std::endl;\
        m_ogdStorageHome->for_each(Cdmw::FT::ObjectGroupDataPrinter(oss__)); \
        CDMW_DEBUG(FTLogger::GetLogger(),oss__.str());\
    }\
} while(0)

#endif



namespace {

    const char* PRIMARY_LOCATION_PROPERTY_NAME = "com.thalesgroup.ft.PrimaryLocation";
    const char* OBJECT_GROUP_NAME = "com.thalesgroup.ft.Name";



    const int DEFAULT_PROPERTIES  = 0;
    const int TYPE_PROPERTIES     = 1;
    const int CREATION_PROPERTIES = 2;
    const int DYNAMIC_PROPERTIES  = 3;
    /*
     * Debug utility: shows the contexts associations and states
     */
    const char* sLocationStateString[]= {
        "INITIAL",
        "INSERTING_PRIMARY",
        "PRIMARY",
        "INSERTING_BACKUP",
        "BACKUP",
        "FAILED"
    };

    const char* sObjectGroupStateString[]= {
        "INITIAL", 
        "FAILED",            
        "LOST_REDUNDANCY", 
        "DEGRADED_REDUNDANCY",
        "FULLY_REDUNDANT"
    };

    const char* sMemberStateString[]= {
        "INITIAL", 
        "BACKUP", 
        "PRIMARY", 
        "FAILED"
    };

    inline  const char* stateness_to_string(const CdmwFT::LocationStateness & s)
    {
        if (s == CdmwFT::LocationStateness_STATEFULL) return "STATEFULL";
        else return "STATELESS";
    }
    inline  const char* stateness_to_string(const CdmwFT::GroupStateness & s)
    {
        if (s == CdmwFT::GroupStateness_STATEFULL) return "STATEFULL";
        else return "STATELESS";
    }

    bool OGDTrueFunction(const std::pair< Cdmw::OrbSupport::type_traits< ::FT::ObjectGroupId >::_var_type, 
                                          Cdmw::OrbSupport::type_traits< ::CdmwFT::ObjectGroupData >::_var_type > & elt)
    {
        return true;    
    }


    /**
     * PW: this function should really be located in the Repository CSCI.
     *
     * Compare two FT::Locations. Locations are assumed to be valid CosNaming::Names.
     * The parameter depth allows to limit the comparison to the 'depth' first name
     * components. When 'depth' has the default value 0, the comparison is made on
     * the whole Locations. 
     *
     * @param l1 The first location to compare
     * @param l2 The second location to compare
     *
     * @return true if locations are equal, false else 
     */
    bool
    Compare_Locations(const ::FT::Location& l1, const ::FT::Location& l2, ::CORBA::ULong depth = 0)
    {
        CORBA::ULong l1_len = l1.length();
        CORBA::ULong l2_len = l2.length();
        CORBA::ULong actual_depth;

        if (l1_len != l2_len) {
          if (depth == 0)                           return false;
          if ((l1_len < depth) || (l2_len < depth)) return false;
          actual_depth = depth;
        }
        else {
          actual_depth = (depth == 0)? l1_len: depth;
        }

        for (CORBA::ULong i=0; i<actual_depth; i++) {
          if (strcmp(l1[i].id.in(), l2[i].id.in()) != 0)      return false;
          if (strcmp(l1[i].kind.in(), l2[i].kind.in()) != 0)  return false;
        }

        return true;
    }



}; // namespace

#endif // _CDMW_FT_REPLICATION_MANAGER_DEFINITION_IMPL_HPP_

