/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/orbsupport/CORBA.hpp>
#include "testftreplicationmanager/TestGenericFactory.hpp"
#include "idllib/CdmwFTReplicationManager.stub.hpp"
#include <Foundation/ossupport/OS.hpp>
#include <exception>
#include <iostream>
#include <ftreplicationmanager/CdmwFTReplicationManagerPushConsumer.stub.hpp>
#include <FaultTolerance/idllib/CdmwFTManager.stub.hpp>

namespace Cdmw {
namespace FT {

CPPUNIT_TEST_SUITE_REGISTRATION( TestGenericFactory );

// constructor
TestGenericFactory::TestGenericFactory()
{
    m_orb =  CORBA::ORB::_duplicate( Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());
    m_rootPOA = PortableServer::POA::_duplicate( Cdmw::TestUtils::CORBATestManager::instance()->get_POA());
}

//destructor
TestGenericFactory::~TestGenericFactory()
{
}

// do_tests
void TestGenericFactory::test_GenericFactory()
{
//     set_nbOfRequestedTestOK (8);

    try
    {
        // Recover the reference of the Replication Manager
        TEST_INFO( "Create the Replication Manager" );

        std::string processArgs(" --CdmwXMLFile=CdmwFaultToleranceManager_conf.xml --CdmwLogLevel=128 --CdmwLogFile=cdmw_ft_manager.");

        std::ostringstream oss;
        oss << Cdmw::OsSupport::OS::get_time().microseconds;

        processArgs += oss.str();
        processArgs += ".log ";

        Cdmw::OsSupport::OS::ProcessId replMngt_id
          = Cdmw::OsSupport::OS::create_process ("cdmw_ft_manager", processArgs);

        int timescale = Cdmw::TestUtils::get_timescale();
        Cdmw::OsSupport::OS::sleep(timescale*20000);
        

        // Get the fault Manager IOR
        std::cout << "Obtain the fault_manager " << std::endl;
        CORBA::Object_var obj = m_orb->string_to_object("corbaloc::localhost:4555/fault_manager");

        CdmwFT::FTManager_var ftManager = CdmwFT::FTManager::_narrow(obj.in());
        obj = ftManager->get_replication_manager();
        CdmwFT::ReplicationManager_var rm = CdmwFT::ReplicationManager::_narrow(obj.in());

        // set default properties
        TEST_INFO( "set_default_properties" );
    
        ::FT::Properties prop;        
    
        prop.length(4);
        prop[0].nam.length(1);
        prop[0].nam[0].id="org.omg.ft.MinimumNumberReplicas";
        prop[0].val <<= (CORBA::UShort)2;
    
        prop[1].nam.length(1);
        prop[1].nam[0].id="org.omg.ft.ReplicationStyle";
        prop[1].val <<= (CORBA::UShort)2;
    
        prop[2].nam.length(1);
        prop[2].nam[0].id="org.omg.ft.MembershipStyle";
        prop[2].val <<= (CORBA::UShort)0;
            
        ::FT::Location loc;
        loc.length(3);
        loc[0].id = "localhost";
        loc[0].kind = "hostname";
        loc[1].id = "APPL";
        loc[1].kind = "application";
        loc[2].id = "PROC";
        loc[2].kind = "process";    
            
        prop[3].nam.length(1);
        prop[3].nam[0].id="org.omg.ft.Factories";
        
        ::FT::FactoryInfos factoryInfos;
        factoryInfos.length(1);
        factoryInfos[0].the_factory = ::FT::GenericFactory::_nil();
        factoryInfos[0].the_location = loc;
        ::FT::Criteria factoryCrit;        
        factoryCrit.length(0);
        factoryInfos[0].the_criteria = factoryCrit;
        
        prop[3].val <<= factoryInfos;


        CPPUNIT_ASSERT(true);

        const char * rid = "IDL:thalesgroup.com/CdmwReplicationManager/HelloInterface1:1.0";
        ::FT::GenericFactory::FactoryCreationId_var factory_creation_id1;
        ::FT::GenericFactory::FactoryCreationId_var factory_creation_id2;
        ::FT::Properties* prop_to_criteria = new ::FT::Properties(prop);        
        ::FT::Criteria crit;        
        crit.length(1);
        crit[0].nam.length(1);
        crit[0].nam[0].id="org.omg.ft.FTProperties";
        crit[0].val <<= prop_to_criteria;

        CORBA::Object_var obj1;
        CORBA::Object_var obj2;
        
        // create a fisrt object with a valid criteria
        TEST_INFO("Create a first object");

        try
        {
            obj1 =  rm->create_object(rid,
                                      crit,
                                      factory_creation_id1.out());
             CPPUNIT_ASSERT(true);
       }
        
        catch( const CORBA::Exception& e )
        {
            std::cerr << e << std::endl;
            CPPUNIT_ASSERT(false);
        }
     
        if (CORBA::is_nil(obj1.in()))
            CPPUNIT_ASSERT(false);
        else
            CPPUNIT_ASSERT(true);

        TEST_INFO("Create a second object");

        prop[2].val <<= (CORBA::UShort)1;
              
        try
        {
            obj2 =  rm->create_object(rid,
                                      crit,
                                      factory_creation_id2.out());
            CPPUNIT_ASSERT(true);
        }
        

        catch( const CORBA::Exception& e )
        {
            std::cerr << e << std::endl;
            CPPUNIT_ASSERT(false);
        }
     
        if (CORBA::is_nil(obj2.in()))
            CPPUNIT_ASSERT(false);
        else
            CPPUNIT_ASSERT(true);        
     
        ::CdmwFT::ReplicationManagerPushConsumer_var consumer_dumper = 
          ::CdmwFT::ReplicationManagerPushConsumer::_narrow(rm.in());

        consumer_dumper->dump_ogd_datastore();
        consumer_dumper->dump_location_datastore();



        // delete the first object
        TEST_INFO("Delete the first object");
        try
        {
            rm->delete_object(factory_creation_id1.in());    
            CPPUNIT_ASSERT(true);
        }
        
        catch( const CORBA::Exception& e )
        {
            std::cerr << e << std::endl;
            CPPUNIT_ASSERT(false);
        }
        
        // delete the fisrt object again
        // this must raise an ::FT::ObjectNotFound exception
        TEST_INFO("Delete the first object (second time)");
        try
        {            
            rm->delete_object(factory_creation_id1.in());
        }
        
        catch ( const ::FT::ObjectNotFound& e)
        {
            CPPUNIT_ASSERT(true); 
        }
        catch( const CORBA::Exception& e )
        {
            std::cerr << e << std::endl;
            CPPUNIT_ASSERT(false);
        }
        
        Cdmw::OsSupport::OS::kill_process(replMngt_id );

    }
    
    catch( const CORBA::Exception& e )
    {
        std::cerr << e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch( const Cdmw::Exception& e )
    {
        std::cerr << e << std::endl;
        CPPUNIT_ASSERT(false);
    }


}

}; // namespace FT
}; // namespace Cdmw


