/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <string>
#include <sstream>

#include <orbsvcs/FaultTolerance/FT_ClientService_Activate.h>

#include <Foundation/common/Options.hpp>
#include <Foundation/common/String.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <FaultTolerance/ftreplicationmanager/InitUtils.hpp>
#include "testftreplicationmanager/TestFaultNotifier.hpp"
#include "testftreplicationmanager/TestPropertyManager.hpp"
#include "testftreplicationmanager/TestGenericFactory.hpp"
#include "testftreplicationmanager/TestObjectGroupManager.hpp"
#include "testftreplicationmanager/TestFailureAndElection.hpp"
#include "testftreplicationmanager/TestFaultDetector.hpp"

using namespace std;
#include "testftreplicationmanager/TypeRepositoryStub.hpp"
#include <idllib/FT.stub.hpp>
#include <idllib/CdmwFTTypeRepository.stub.hpp>
#include "FaultTolerance/ftstatemanager/DataStore.hpp"
#include <FaultTolerance/idllib/CdmwFTManager.stub.hpp>
using namespace Cdmw;


class ReplicationManagerThread : public OsSupport::Thread {

public:
        
    ReplicationManagerThread( int argc, char* argv[], CORBA::ORB_ptr orb, PortableServer::POA_ptr rootPOA)
    {
        m_orb = CORBA::ORB::_duplicate(orb);
    }
        
    ~ReplicationManagerThread() throw() {} ;


    void
    shutdown() 
    {
        m_orb->shutdown(false);
//         m_orb->shutdown(true);
//         m_orb->destroy();
    }

protected:

    void
    run() throw() 
    {
        m_orb->run();   
    }

        
public:

    CORBA::ORB_var  m_orb;

};


                               
int main( int argc, char* argv[] )
{
    int res = 0;
    //fixing memory leak
    CORBA::ORB_var orb = CORBA::ORB::_nil();
    
    try
    {
        // initialize ReplicationManager's interceptor
        Cdmw::FT::ReplicationManager::InitUtils::Pre_init(argc, argv);
        
        Cdmw::OrbSupport::StrategyList strategyList;
        strategyList.add_OrbThreaded(); // Mandatory in OrbSupport::ORB_init
        strategyList.add_multicast();
        // Initialises the ORB
    //fixing leak : declared before
        orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, strategyList);


        // Get the root POA
        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj.in());

        PortableServer::POAManager_var poaMgr = rootPOA->the_POAManager();
        poaMgr->activate();

        ReplicationManagerThread RMThread(argc, argv, orb.in(), rootPOA.in());
        RMThread.start();
        int timescale = Cdmw::TestUtils::get_timescale();
        OsSupport::OS::sleep(timescale*3000);

        // Do test
        Cdmw::TestUtils::CORBATestManager::instance()->init(orb.in(),rootPOA.in(),argc,argv);
        Cdmw::TestUtils::CORBATestManager::instance()->set_property ("process_name",argv[0]);
        Cdmw::TestUtils::CORBATestManager::instance()->run_tests();


        // Stop the ReplicationManagerThread 
        RMThread.shutdown();
        //RMThread.join();

        Cdmw::TestUtils::CORBATestManager::instance()->summary();

        //fixing memory leak
        if (! CORBA::is_nil(orb.in()))
        {
            Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
            orb->destroy();
            orb = CORBA::ORB::_nil();
        }
        RMThread.join();
    }   
    catch( const CORBA::Exception &e )
    {
        cerr << e._name() << endl;
        res = -1;
    }
    catch( const Cdmw::Exception &e )
    {
        cerr << e.what() << endl;
        res = -1;
    }
    catch(...)
    {
        cerr << "Uncaught exception" << endl;
        res = -1;
    }
    return res;
}

