/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>

#include <testsimultaneousaddmember/TestSimultaneousAddMember.hpp>
#include <testsimultaneousaddmember/TestUtils.hpp>
#include <testsimultaneousaddmember/TestFaultNotifier.hpp>
#include <testsimultaneousaddmember/TestFaultDetector.hpp>


#include <testsimultaneousaddmember/TestHello.stub.hpp>


#include <fstream>

// BEGIN TAO TESTS


#include <vector>
#include <algorithm>
#include <iomanip>
#include <numeric>

namespace 
{
} // end of anonymous namespace

// END TAO TESTS

namespace Cdmw 
{

namespace FT 
{
CPPUNIT_TEST_SUITE_REGISTRATION( TestSimultaneousAddMember );

// constructor
TestSimultaneousAddMember::TestSimultaneousAddMember()
{    
   
    m_orb =  
       CORBA::ORB::_duplicate( 
             Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());
    m_rootPOA = 
       PortableServer::POA::_duplicate( 
             Cdmw::TestUtils::CORBATestManager::instance()->get_POA());

    m_hostname 
       = Cdmw::TestUtils::CORBATestManager::instance()->get_property("hostname");
    m_sleep_duration 
       = ::atoi((Cdmw::TestUtils::CORBATestManager::instance()->get_property(
                   "sleep_duration")).c_str());
    m_crash_server_option 
       = Cdmw::TestUtils::CORBATestManager::instance()->get_property(
             "crash_server_option");
    m_change_order_option 
       = Cdmw::TestUtils::CORBATestManager::instance()->get_property(
             "change_order_option");
    CORBA::Object_var obj1 = 
       Cdmw::TestUtils::CORBATestManager::instance()->get_object("hello_object_group");
    m_hello_group = CdmwFT::Test::HelloInterface::_narrow(obj1.in());

    CORBA::Object_var obj2 = 
       Cdmw::TestUtils::CORBATestManager::instance()->get_object("comput_object_group");
    m_comput_group = CdmwFT::Test::ComputInterface::_narrow(obj2.in());

    CORBA::Object_var obj3 = 
       Cdmw::TestUtils::CORBATestManager::instance()->get_object("hello_object");
    m_hello_obj = CdmwFT::Test::HelloInterface::_narrow(obj3.in());

    CORBA::Object_var obj4 = 
       Cdmw::TestUtils::CORBATestManager::instance()->get_object("comput_object");
    m_comput_obj = CdmwFT::Test::ComputInterface::_narrow(obj4.in());
    
    CORBA::Object_var obj5 = Cdmw::TestUtils::CORBATestManager::instance()->get_object("replication_manager");
    m_replication_manager = CdmwFT::ReplicationManager::_narrow(obj5.in());

}

// destructor
TestSimultaneousAddMember::~TestSimultaneousAddMember()
{
}

// do_tests
void TestSimultaneousAddMember::do_tests()
{
    try 
    {
        Cdmw::OsSupport::OS::unlink("hello.stt");
        Cdmw::OsSupport::OS::unlink("hello.act");
        Cdmw::OsSupport::OS::unlink("hello.dct");
        Cdmw::OsSupport::OS::unlink("comput.stt");
        Cdmw::OsSupport::OS::unlink("comput.act");
        Cdmw::OsSupport::OS::unlink("comput.dct");
    }
    catch(...)
    {
    }

    ::FT::Location loc;
    loc.length(3);
    loc[0].id = m_hostname.c_str();
    loc[0].kind = "hostname";
    loc[1].id = "APPL";
    loc[1].kind = "application";
    loc[2].id = "PROC";
    loc[2].kind = "process";    
            
    if (m_change_order_option == "no")
    {
        try
        {
            TEST_INFO("add member on HelloInterface group " + m_hostname);
            CORBA::Object_var obj = m_replication_manager->add_member(m_hello_group.in(),
                                                                      loc,
                                                                      m_hello_obj.in());
             
            TEST_INFO("add member on HelloInterface group OK " + m_hostname);
            CPPUNIT_ASSERT(true);  
        }
        catch (const CORBA::SystemException& e)
        {
            TEST_INFO("CORBA System Exception raised on " << m_hostname << ": " << e);
            CPPUNIT_ASSERT(false);
        }
    
        OsSupport::OS::sleep(m_sleep_duration);

        if (m_crash_server_option == "no")
        {
           try
            {
                TEST_INFO("add member on ComputInterface group " + m_hostname);
                CORBA::Object_var obj = m_replication_manager->add_member(m_comput_group.in(),
                                                                          loc,
                                                                          m_comput_obj.in());
                TEST_INFO("add member on ComputInterface group OK "  + m_hostname);
                CPPUNIT_ASSERT(true);  
            }
            catch (const CORBA::SystemException& e)
            {
                TEST_INFO("CORBA System Exception raised on " << m_hostname << ": " << e);
                CPPUNIT_ASSERT(false);
            }

            OsSupport::OS::sleep(10000);
        }
    }
    else
    {
        try
        {
            TEST_INFO("add member on ComputInterface group " + m_hostname);
            CORBA::Object_var obj = m_replication_manager->add_member(m_comput_group.in(),
                                                                      loc,
                                                                      m_comput_obj.in());
            
            TEST_INFO("add member on ComputInterface group OK "  + m_hostname);
            CPPUNIT_ASSERT(true);  
        }
        catch (const CORBA::SystemException& e)
        {
            TEST_INFO("CORBA System Exception raised on " << m_hostname << ": " << e);
            CPPUNIT_ASSERT(false);
        }
 
        OsSupport::OS::sleep(m_sleep_duration);

        if (m_crash_server_option == "no")
        {
            try
            {
                TEST_INFO("add member on HelloInterface group " + m_hostname);
                CORBA::Object_var obj = m_replication_manager->add_member(m_hello_group.in(),
                                                                          loc,
                                                                          m_hello_obj.in());
                 
                TEST_INFO("add member on HelloInterface group OK " + m_hostname);
                CPPUNIT_ASSERT(true);  
            }
            catch (const CORBA::SystemException& e)
            {
                TEST_INFO("CORBA System Exception raised on " << m_hostname << ": " << e);
                CPPUNIT_ASSERT(false);
            }

            OsSupport::OS::sleep(10000);
        }
    }
    
    // create the Fault Notifier and store it on the Replication Manager
    TEST_INFO( "Create the FaultNotifier" );
    
    Cdmw::FT::TestFaultNotifier* fn_impl = new  Cdmw::FT::TestFaultNotifier();
    ::FT::FaultNotifier_var fn = fn_impl->_this();
    CPPUNIT_ASSERT(true);
    TEST_INFO( "Register the FaultNotifier to the ReplicationManager" );
    
    m_replication_manager->register_fault_notifier(fn.in());
    
    CPPUNIT_ASSERT(true);
    
    // create the Fault Detector and registe it on the Replication Manager
    TEST_INFO( "Create the FaultDetector" );
    
    Cdmw::FT::TestFaultDetector* fd_impl = new  Cdmw::FT::TestFaultDetector();
    ::CdmwFT::FaultManagement::FaultDetector_var fd = fd_impl->_this();
    CPPUNIT_ASSERT(true);
    TEST_INFO( "Register the FaultDetector to the ReplicationManager" );
   
    m_replication_manager->register_fault_detector(fd.in());
   
    CPPUNIT_ASSERT(true);
   
   
    // Recover the FaultNotifier
    TEST_INFO("Recover the FaultNotifier");
    ::FT::FaultNotifier_var fn2 ;
    
    try
    {
        fn2 = m_replication_manager->get_fault_notifier();
        CPPUNIT_ASSERT(true);
    }        
    catch( const CORBA::Exception& e )
    {
        std::cerr << e << std::endl;;
        CPPUNIT_ASSERT(false);
    }
   
    if (CORBA::is_nil(fn2.in()))
        CPPUNIT_ASSERT(false);
    else
        CPPUNIT_ASSERT(true);
   
    // Register the Replication Manager as a consumer of the Fault Notifier
    TEST_INFO("Create a reference of the StructuredPushConsumer (it is the replication manager)");
   
    ::CosNotifyComm::StructuredPushConsumer_var  rmStructuredPushConsumer =
      ::CosNotifyComm::StructuredPushConsumer::_narrow(m_replication_manager.in());
   
    
    ::FT::FaultNotifier::ConsumerId consumerID;
    
    TEST_INFO("Register a reference of the StructuredPushConsumer (it is the replication manager)");
  
    try
    {
        consumerID = fn2->connect_structured_fault_consumer (rmStructuredPushConsumer.in());
        CPPUNIT_ASSERT(true);
    }
    catch( const CORBA::Exception& e )
    {
        std::cerr << e << std::endl;;
        CPPUNIT_ASSERT(false);
    }
    
    // create a fault report to send a bug report to the Fault Notifier
    // (the fault Notifier forward the bug report to the Replication Manager)
    TEST_INFO("Creation of a fault report for the primary member");
   
    CosNotification::StructuredEvent notification;
   
    std::ostringstream ft_domain_id;
    ft_domain_id << "My_" << Cdmw::OsSupport::OS::get_logname();
    ft_domain_id << "FTdomain" << std::ends;
   
    notification.header.fixed_header.event_type.domain_name = CORBA::string_dup("FT_CORBA");
    notification.header.fixed_header.event_type.type_name = CORBA::string_dup("ObjectCrashFault");
    notification.filterable_data.length(2);
    notification.filterable_data[0].name = "FTDomainID";
    notification.filterable_data[0].value <<= ft_domain_id.str().c_str();
    notification.filterable_data[1].name = "Location";
    notification.filterable_data[1].value <<= loc;
    
    TEST_INFO("Send the fault report to the Fault Notifier");
    try
    {
        fn2->push_structured_fault( notification );
        CPPUNIT_ASSERT(true);
    }
    catch( const CORBA::Exception& e )
    {
        std::cerr << e << std::endl;;
        CPPUNIT_ASSERT(false);
    }

    try
    {
        Cdmw::OsSupport::OS::unlink("hello.stt");
        Cdmw::OsSupport::OS::unlink("hello.act");
        Cdmw::OsSupport::OS::unlink("hello.dct");
        Cdmw::OsSupport::OS::unlink("comput.stt");
    }
    catch (...)
    {
    }
}


} // end namespace FT

} // end namespace Cdmw

