/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Standard Files 
#include <string>
#include <fstream>
#include <sstream>
// Cdmw Files
#include <Foundation/common/Options.hpp>
#include <Foundation/common/String.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/testutils/TestManager.hpp>
#include <FaultTolerance/ftinit/FTServiceInit.hpp>
#include <FaultTolerance/ftcommon/FTConfiguration.hpp>
#include <testsimultaneousaddmember/TestUtils.hpp>
#include <FaultTolerance/idllib/CdmwFTManager.stub.hpp>

#include <idllib/FT.stub.hpp>

using namespace Cdmw;
using namespace std;

namespace
{
    const char * rep_id_hello = "IDL:thalesgroup.com/CdmwFT/Test/HelloInterface:1.0";
    const char * rep_id_comput = "IDL:thalesgroup.com/CdmwFT/Test/ComputInterface:1.0";
}
                               
int main( int argc, char* argv[] )
{
    //fixing memory leak
    CORBA::ORB_var orb =CORBA::ORB::_nil();
    try
    {
        // Initialise FT service
        Cdmw::FT::FTServiceInit::Init(argc, argv, true);

        // Initialises the ORB
        OrbSupport::StrategyList strategyList;
        strategyList.add_OrbThreaded();
        strategyList.add_PoaThreadPerConnection();
        
        std::string str_opt(argv[0]);
        str_opt += " ";
        str_opt += Cdmw::Common::Options::LOCALISATION_SERVICE_OPTION;
        str_opt += "=";
        str_opt += "45678";
            
        char** argv__ = Cdmw::Common::String::to_char_array
        ( Cdmw::Common::String::to_strings(str_opt) );
        int argc__(2);
        
        orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc__, argv__, strategyList);
        
    //fixing memory leak
    Cdmw::Common::String::delete_char_array(argv__);
        // Get the root POA
        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj.in());

        PortableServer::POAManager_var poaMgr = rootPOA->the_POAManager();
        poaMgr->activate();

        
        std::cout << "Obtain the ORBs PolicyManager " << std::endl;
        obj = orb->resolve_initial_references("ORBPolicyManager");
        
        CORBA::PolicyManager_var policy_manager 
            = CORBA::PolicyManager::_narrow(obj.in());
        
        // Create and apply an ORB-wide Routed policy
        std::cout << "Create and apply an ORB-wide Routed policy " << std::endl;
        std::cout << "Creating REQUEST_DURATION_POLICY ..." << std::endl;
        CORBA::Any any;
        TimeBase::TimeT duration = 20000000;
        any <<= duration;
        CORBA::PolicyList policies(1);
        policies.length(1);
        policies[0] =
            orb->create_policy(::FT::REQUEST_DURATION_POLICY, any);
        
        policy_manager->set_policy_overrides(policies, CORBA::ADD_OVERRIDE);
        policies[0]->destroy();
        
        using namespace Cdmw::OsSupport;
        OS::ProcessId replMngt_id
        = OS::create_process ("cdmw_ft_manager",
                              " --CdmwXMLFile=CdmwFaultToleranceManager_conf.xml");

        int timescale = Cdmw::TestUtils::get_timescale();
        OsSupport::OS::sleep(timescale*20000);

        // Get the fault Manager IOR
        std::cout << "Obtain the fault_manager " << std::endl;
        obj = orb->string_to_object("corbaloc::localhost:4555/fault_manager");

        CdmwFT::FTManager_var ftManager = CdmwFT::FTManager::_narrow(obj.in());
        obj = ftManager->get_replication_manager();
        CdmwFT::ReplicationManager_var replication_manager 
           = CdmwFT::ReplicationManager::_narrow(obj.in());

        //fixing leak : CORBA::String, not std::string
        //std::string replication_ior_str = orb->object_to_string(replication_manager.in());
        CORBA::String_var replication_ior_str = 
            orb->object_to_string(replication_manager.in()); 

        // export the object reference to a file
        std::cout << " export the replication_manager object reference to a file"
                  << std::endl;
        std::ofstream os("replication_manager.ior");
        os << replication_ior_str;
        os.close();
        
        {
            //
            // Create groups
            //
            std::cout << "Create groups" << std::endl;
            ::FT::ObjectGroup_var comput_group
               = Cdmw::FT::TestUtils::create_group(orb.in(), 
                                                   replication_manager.in(), 
                                                   rep_id_comput);
            //fixing leak : using a CORBA::string_var instead of a std::string
            //std::string comput_group_ior_str = orb->object_to_string(comput_group.in());
        CORBA::String_var comput_group_ior_str = orb->object_to_string(comput_group.in());

            // export the object reference to a file
            std::cout << " export the comput_group object reference to a file"
                  << std::endl;
            std::ofstream os2("comput_group.ior");
        //fixing leak
            // os2 << comput_group_ior_str;
        os2 << std::string(comput_group_ior_str.in());
            os2.close();

            ::FT::ObjectGroup_var hello_group 
               = Cdmw::FT::TestUtils::create_group(orb.in(), 
                                                   replication_manager.in(), 
                                                   rep_id_hello);
            //fixing leak : using a CORBA::String_var, not a std::string
            CORBA::String_var hello_group_ior_str = orb->object_to_string(hello_group.in());

            // export the object reference to a file
            std::cout << " export the hello_group object reference to a file"
                  << std::endl;
            std::ofstream os("hello_group.ior");
            os << (std::string(hello_group_ior_str.in()));
            os.close();

            // Create Server process
            std::cout << "Create test_server process" << std::endl;
            //fixing leak : replication_ior_str.in is a CORBA::String
            OS::ProcessId test_server_id1 
            = OS::create_process ("test_server", 
                " --HostName=A --SleepDuration=3000 -ORBInitRef ReplicationManager=" 
                                  + ((std::string)(replication_ior_str.in())));
            
            OsSupport::OS::sleep(timescale*1000);
        
            //fixing leak : replication_ior_str.in is a CORBA::String .in method
            OS::ProcessId test_server_id2 
            = OS::create_process ("test_server", 
                " --HostName=B -ORBInitRef ReplicationManager=" 
                                  + ((std::string)replication_ior_str.in()));
            
            OsSupport::OS::sleep(timescale*30000);

            OS::wait_process(test_server_id2);
            OS::wait_process(test_server_id1);

            // Create Server process
            std::cout << "Create test_server process again" << std::endl;
        //fixing leak : replication_ior_str.in is a CORBA::String
            test_server_id1 
            = OS::create_process ("test_server", 
                " --HostName=A --SleepDuration=3000 -ORBInitRef ReplicationManager=" 
                                  + ((std::string) replication_ior_str.in()));
            
            OsSupport::OS::sleep(timescale*1000);

            test_server_id2 
            = OS::create_process ("test_server", 
                " --HostName=B --ChangeOrder -ORBInitRef ReplicationManager=" 
                                  + ((std::string)replication_ior_str.in()));

            OsSupport::OS::sleep(timescale*30000);

            OS::wait_process(test_server_id2);
            OS::wait_process(test_server_id1);

            // Create Server process
            std::cout << "Create test_server process again" << std::endl;
            test_server_id1 
            = OS::create_process ("test_server", 
                " --HostName=A --SleepDuration=3000 --CrashServer -ORBInitRef ReplicationManager=" 
                                  + ((std::string)replication_ior_str.in()));
            
            OsSupport::OS::sleep(timescale*1000);

            test_server_id2 
            = OS::create_process ("test_server", 
                " --HostName=B --ChangeOrder -ORBInitRef ReplicationManager=" 
                                  + ((std::string) replication_ior_str.in()));

            OsSupport::OS::sleep(timescale*20000);

            std::cout << "unlink comput_group.ior" << std::endl;
            OsSupport::OS::unlink("comput_group.ior");
            OsSupport::OS::unlink("replication_manager.ior");

            //std::cout << "kill process test_server_id2" << std::endl;
            OS::wait_process(test_server_id2);
            //std::cout << "kill process test_server_id1" << std::endl;
            OS::wait_process(test_server_id1);

        }
    //fixing memory leak
        if (!CORBA::is_nil (orb.in()))
        {
           try
           {
               Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
               orb->destroy();
           }
           catch( const CORBA::Exception &e )
           {
               std::cerr <<__FILE__<<" "<<__LINE__<<" "<< e << std::endl;
           }
           catch( const Cdmw::Exception &e )
           {
               std::cerr <<__FILE__<<" "<<__LINE__<<" "<< e << std::endl;
           }
           catch(...)
           {
               std::cerr << "Uncaught exception" << std::endl;
           }
           orb = CORBA::ORB::_nil();
        }
        OS::kill_process(replMngt_id );
    }
    catch( const CORBA::Exception &e )
    {
        cerr <<__FILE__<<" "<<__LINE__<<" "<< e._name() << endl;
        return EXIT_FAILURE;
    }
    catch( const Cdmw::Exception &e )
    {
        cerr <<__FILE__<<" "<<__LINE__<<" "<< e.what() << endl;
        return EXIT_FAILURE;
    }
    catch(...)
    {
        cerr << "Unexpected exception" << endl;
        return EXIT_FAILURE;
    }

    return EXIT_SUCCESS;
}

