/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "FaultTolerance/ftstatemanager/TXManager.hpp"
#include <FaultTolerance/ftstatemanager/TXDataStoreBase.hpp>
#include <Foundation/osthreads/WriterLockGuard.hpp>
#include <Foundation/osthreads/ReaderLockGuard.hpp>
#include <Foundation/common/Assert.hpp>
#include <sstream>

namespace Cdmw {
namespace FT {

TXDataStoreBase::TXDataStoreMap TXDataStoreBase::M_dsMap;
Cdmw::OsSupport::ReaderWriterLock TXDataStoreBase::M_dsMapLock;

TXDataStoreBase::~TXDataStoreBase()
{
}

void TXDataStoreBase::Register_dataStore(TXDataStoreBase* ds)
    throw (AlreadyExistException)
{
    CDMW_WRITER_LOCK_GUARD(M_dsMapLock);
    TXDataStoreMap::iterator info_pos = M_dsMap.find(ds->get_dsid());
    if (info_pos != M_dsMap.end())
    {
        std::ostringstream msg;
        msg << "TXDataStoreBase: A TXDataStoreBase with " << ds->get_dsid() 
            << " as ID is already registered.";
        throw AlreadyExistException(msg.str());
    }
    else
    {
        M_dsMap[ds->get_dsid()] = ds;
    }
}

void TXDataStoreBase::Unregister_dataStore(const DsIdentifier& dsid)
    throw (NotExistException)
{
    CDMW_WRITER_LOCK_GUARD(M_dsMapLock);
    TXDataStoreMap::iterator info_pos = M_dsMap.find(dsid);
    if (info_pos != M_dsMap.end())
    {
        M_dsMap.erase(dsid);
    }
    else
    {
        throw NotExistException();
    }
}

TXDataStoreBase* TXDataStoreBase::Get_dataStore(const DsIdentifier& dsid)
{
    CDMW_READER_LOCK_GUARD(M_dsMapLock);
    CDMW_ASSERT(M_dsMap.find(dsid) != M_dsMap.end());
    return M_dsMap[dsid];
}


TXDataStoreBase::DataStoreIdList TXDataStoreBase::Get_dataStoreIds()
{
    DataStoreContext::DataStoreIdList lst;
    for (TXDataStoreMap::iterator it = M_dsMap.begin(); 
         it != M_dsMap.end(); 
         ++it) 
    {
        lst.push_back(it->first);
    }
    return lst;
}


}  // End namespace FT
}  // End namespace Cdmw
