/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

// Std include files
#include <iostream>
#include <sstream>
#include <string.h>
#include <set>

#include "teststorageinterface/TestStateTransfer.hpp"
#include "ftstatemanager/DataStoreGroup_impl.hpp"

using namespace std;

namespace Cdmw {
namespace FT {

typedef std::set<int> OidSet;

struct Dump: public std::unary_function<int, void>
{
      void operator()(const TestDataStore::DataMap::value_type& elt)
      {
          std::cout << "key: " << elt.first << " value: " << elt.second << std::endl;
      }
};

CPPUNIT_TEST_SUITE_REGISTRATION( TestStateTransfer );

TestStateTransfer::TestStateTransfer ()
{
    m_orb =  CORBA::ORB::_duplicate( Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());

    PortableServer::POA_ptr poalocal = Cdmw::TestUtils::CORBATestManager::instance()->get_POA();
    PortableGroup::GOA_ptr goalocal = dynamic_cast<PortableGroup::GOA_ptr>(poalocal);
    m_rootGOA = PortableGroup::GOA::_duplicate(goalocal);
}

TestStateTransfer::~TestStateTransfer ()
{
}
    
void 
TestStateTransfer::do_tests()
{
//     set_nbOfRequestedTestOK(17);

    try
    {
        int INT_STRING_HOME1 = 1;
//         int INT_STRING_HOME2 = 2;
//         int INT_STRING_HOME3 = 3;
//         int INT_STRING_HOME4 = 4;
//         int INT_STRING_HOME5 = 5;
        const int MAX_TRANSACTION_DONE = 100;
        const int MAX_TRANSACTION_IN_PROGRESS = 100;

        TEST_INFO("Create empty DataStores");
        Cdmw::OsSupport::OS::Timeval cohort_timeout;
        cohort_timeout.seconds=2;
        cohort_timeout.microseconds=0;
        Cdmw::OsSupport::OS::Timeval coordinator_timeout;
        coordinator_timeout.seconds=2;
        coordinator_timeout.microseconds=0;

        Cdmw::FT::DataStoreContext context1(m_orb.in(), m_rootGOA.in(), 
                                            MAX_TRANSACTION_IN_PROGRESS,
                                            MAX_TRANSACTION_DONE, 
                                            cohort_timeout,
                                            coordinator_timeout,
                                            1);

        Cdmw::FT::DataStoreContext context2(m_orb.in(), m_rootGOA.in(), 
                                            MAX_TRANSACTION_IN_PROGRESS,
                                            MAX_TRANSACTION_DONE, 
                                            cohort_timeout,
                                            coordinator_timeout,
                                            1);

        Cdmw::FT::DataStoreContext context3(m_orb.in(), m_rootGOA.in(), 
                                            MAX_TRANSACTION_IN_PROGRESS,
                                            MAX_TRANSACTION_DONE, 
                                            cohort_timeout,
                                            coordinator_timeout,
                                            1);

        Cdmw::FT::DataStoreContext context4(m_orb.in(), m_rootGOA.in(), 
                                            MAX_TRANSACTION_IN_PROGRESS,
                                            MAX_TRANSACTION_DONE, 
                                            cohort_timeout,
                                            coordinator_timeout,
                                            1);

        Cdmw::FT::DataStoreContext context5(m_orb.in(), m_rootGOA.in(), 
                                            MAX_TRANSACTION_IN_PROGRESS,
                                            MAX_TRANSACTION_DONE, 
                                            cohort_timeout,
                                            coordinator_timeout,
                                            1);

        TestDataStore* myDataStore1 =
            new TestDataStore(INT_STRING_HOME1, m_orb.in(),
                              m_rootGOA.in(), context1);
        // Hack! prevent exception due to reuse of DataStore id
        Cdmw::CommonSvcs::DataStore::DataStoreRegistry::
            Unregister_data_store(INT_STRING_HOME1);
        Cdmw::FT::DataStoreBase::
            Unregister_dataStore(INT_STRING_HOME1);
        TestDataStore* myDataStore2 = 
            new TestDataStore(INT_STRING_HOME1, m_orb.in(),
                              m_rootGOA.in(), context2);
        Cdmw::CommonSvcs::DataStore::DataStoreRegistry::
            Unregister_data_store(INT_STRING_HOME1);
        Cdmw::FT::DataStoreBase::
            Unregister_dataStore(INT_STRING_HOME1);
        TestDataStore* myDataStore3 =
            new TestDataStore(INT_STRING_HOME1, m_orb.in(),
                              m_rootGOA.in(), context3);
        Cdmw::CommonSvcs::DataStore::DataStoreRegistry::
            Unregister_data_store(INT_STRING_HOME1);
        Cdmw::FT::DataStoreBase::
            Unregister_dataStore(INT_STRING_HOME1);
        TestDataStore* myDataStore4 =
            new TestDataStore(INT_STRING_HOME1, m_orb.in(),
                              m_rootGOA.in(), context4);
        Cdmw::CommonSvcs::DataStore::DataStoreRegistry::
            Unregister_data_store(INT_STRING_HOME1);
        Cdmw::FT::DataStoreBase::
            Unregister_dataStore(INT_STRING_HOME1);
        TestDataStore* myDataStore5 =
            new TestDataStore(INT_STRING_HOME1, m_orb.in(),
                              m_rootGOA.in(), context5);
        Cdmw::CommonSvcs::DataStore::DataStoreRegistry::
            Unregister_data_store(INT_STRING_HOME1);
        Cdmw::FT::DataStoreBase::
            Unregister_dataStore(INT_STRING_HOME1);
        CPPUNIT_ASSERT(true);

        TEST_INFO("Create CORBA DataStore interface");
        CdmwFT::StateTransfer::DataStoreGroup_var dsgroup
            = (new Cdmw::FT::DataStoreGroup_impl(m_rootGOA.in(), context1))->_this();
        CPPUNIT_ASSERT(true);

        CdmwFT::StateTransfer::DataStoreIDs ids;
        ids.length(1);
        ids[0] = INT_STRING_HOME1;

        std::string coordinator_location = context1.get_coordinator_location();

        TEST_INFO("Try to copy an inactive DataStore");
        try
        {
            context2.update_with_coordinator(dsgroup.in(), coordinator_location, "here1", ids);
            CPPUNIT_ASSERT(false);
        }
        catch(CdmwFT::StateTransfer::InvalidDataStoreState&)
        {
            CPPUNIT_ASSERT(true);
        }

        TEST_INFO("Activate a DataStore");
        myDataStore1->get_context().activate();
        CPPUNIT_ASSERT(true); 

        TEST_INFO("Copy an empty DataStore");
        context2.update_with_coordinator(dsgroup.in(), coordinator_location, "here1", ids);
        CPPUNIT_ASSERT(true);

        TEST_INFO("Test size of destination DataStore");
        CPPUNIT_ASSERT(myDataStore2->size() == 0);

        TEST_INFO("Add data to source DataStore");
        myDataStore1->insert(1, "one");
        myDataStore1->insert(2, "two");
        myDataStore1->insert(3, "three");
        myDataStore1->insert(4, "four");
        myDataStore1->insert(5, "five");
        CPPUNIT_ASSERT(true);

        TEST_INFO("Copy from one DataStore to another");
        context3.update_with_coordinator(dsgroup.in(), coordinator_location, "here2", ids);
        CPPUNIT_ASSERT(true);

        TEST_INFO("Test size of destination DataStore");
        CPPUNIT_ASSERT(myDataStore3->size() == 5);
        TEST_INFO("Test content of destination DataStore");
        CPPUNIT_ASSERT(equivalent(*myDataStore1, *myDataStore3));
      
        TEST_INFO("Test transfer of DataStore during inserts");
        // Increase initial DataStore size
        for (int i = 10; i < 5000; i++)
        {
            if ((i % 1000) == 0)
            {
                cout << "adding data " << i << endl;
            }

            myDataStore1->insert(i, "something");
        }

        Inserter inserter(*myDataStore1);
        inserter.start();
        Cdmw::OsSupport::OS::sleep(100);

        context4.update_with_coordinator(dsgroup.in(), coordinator_location, "here3", ids);
        cout << "update_with_coordinator done" << endl << flush;
        inserter.join();
        CPPUNIT_ASSERT(true);

        TEST_INFO("Test size of destination DataStore");
        cout << "myDataStore1 size is " << myDataStore1->size() << endl;
        cout << "myDataStore4 size is " << myDataStore4->size() << endl;
        CPPUNIT_ASSERT(myDataStore4->size() == 5495);

        //myDataStore4->for_each(Dump());

        TEST_INFO("Test content of destination DataStore");
        CPPUNIT_ASSERT(equivalent(*myDataStore1, *myDataStore4));

        TEST_INFO("Test transfer of DataStore during updates");

        // Add observer for ordering info
        Observer observer;
        myDataStore5->register_observer(&observer);

        Modifier modifier(*myDataStore1);
        modifier.start();
        Cdmw::OsSupport::OS::sleep(100);

        context5.update_with_coordinator(dsgroup.in(), coordinator_location, "here4", ids);
        cout << "update_with_coordinator done" << endl << flush;
        modifier.join();
        CPPUNIT_ASSERT(true);

        TEST_INFO("Test size of destination DataStore");
        cout << "size is " << myDataStore5->size() << endl;
        CPPUNIT_ASSERT(myDataStore5->size() == 5495);

        TEST_INFO("Test content of destination DataStore");
        CPPUNIT_ASSERT(equivalent(*myDataStore1, *myDataStore5));

        TEST_INFO("Test order of transfer of DataStore during updates");
        CPPUNIT_ASSERT(observer.validate());

        // Delete DataStores
        // End of Hack: register DataStores again before each deletion
        // (else NotExist exception will be thrown)
        Cdmw::FT::DataStoreBase::
            Register_dataStore(myDataStore5);
        Cdmw::CommonSvcs::DataStore::DataStoreRegistry::
            Register_data_store(myDataStore5);
        delete myDataStore5;
        Cdmw::FT::DataStoreBase::
            Register_dataStore(myDataStore4);
        Cdmw::CommonSvcs::DataStore::DataStoreRegistry::
            Register_data_store(myDataStore4);
        delete myDataStore4;
        Cdmw::FT::DataStoreBase::
            Register_dataStore(myDataStore3);
        Cdmw::CommonSvcs::DataStore::DataStoreRegistry::
            Register_data_store(myDataStore3);
        delete myDataStore3;
        Cdmw::FT::DataStoreBase::
            Register_dataStore(myDataStore2);
        Cdmw::CommonSvcs::DataStore::DataStoreRegistry::
            Register_data_store(myDataStore2);
        delete myDataStore2;
        Cdmw::FT::DataStoreBase::
            Register_dataStore(myDataStore1);
        Cdmw::CommonSvcs::DataStore::DataStoreRegistry::
            Register_data_store(myDataStore1);
        delete myDataStore1;
    }
    catch (const CORBA::UserException& ex)
    {
    std::ostringstream info_stream1;
    info_stream1 << "Test State Manager Exception caught : " << ex
                     << std::endl;
    TEST_INFO(info_stream1.str());  
        CPPUNIT_ASSERT(false);
    }
    catch (const CORBA::SystemException& ex)
    {
    std::ostringstream info_stream1;
    info_stream1 << "Test State Manager Exception caught : " << ex
                     << std::endl;
    TEST_INFO(info_stream1.str());  
        CPPUNIT_ASSERT(false);
    }
    catch (...)
    {
    std::ostringstream info_stream1;
    info_stream1 << "Test State Manager Exception caught : " << std::endl;
    TEST_INFO(info_stream1.str());  
        CPPUNIT_ASSERT(false);
    }
}

struct CollectOids: public std::unary_function<int, void>
{
      CollectOids(OidSet& oid_set) : m_set(oid_set) {}

      void operator()(const TestDataStore::DataMap::value_type& elt)
      {
         m_set.insert(elt.first);
      }

      OidSet& m_set;
};


bool TestStateTransfer::equivalent(TestDataStore& ds1, TestDataStore& ds2)
{
   bool result;

   result = ds1.size() == ds2.size();

   if (result)
   {
      OidSet set1, set2;

      ds1.for_each(CollectOids(set1));
      ds2.for_each(CollectOids(set2));

      result = set1 == set2;

      if (result)
      {
         int count = 0;

         for (OidSet::iterator iter = set1.begin(); iter != set1.end(); ++iter)
         {
            if ((count++ % 1000) == 0)
            {
               cout << "equivalent count is " << count << endl;
            }

            result = ds1.select(*iter) == ds2.select(*iter);

            if (!result)
            {
               break;
            }
         }

         cout << "equivalent final count is " << count << endl;
      }
   }

   return result;
}

Inserter::Inserter(TestDataStore& data_store)
    throw (OutOfMemoryException,
           InternalErrorException,
           BadParameterException)
   : m_data_store(data_store)
{
}

void Inserter::run() throw()
{
    for (int i = 0; i < 500; i++)
    {
        m_data_store.insert(i + 20000, "hello world");
        this->sleep(50);
    }
}


Modifier::Modifier(TestDataStore& data_store)
    throw (OutOfMemoryException,
           InternalErrorException,
           BadParameterException)
   : m_data_store(data_store)
{
}

void Modifier::run() throw()
{
    for (int i = 0; i < 500; i++)
    {
        m_data_store.update((i % 5) + 1, "updated");
        //this->sleep(50);
    }
}

Observer::Observer()
   : m_inserts(0),
     m_updates(0),
     m_removes(0),
     m_failure(false)
{
}

void Observer::on_insert(_Oid_in_type oid, _Data_in_type data) throw ()
{
   m_inserts++;
   m_failure = m_updates > 0;

   if (m_failure)
   {
      cout << "Error: insert came after updates" << endl;
   }

   Cdmw::OsSupport::OS::sleep(1);
}

void Observer::on_update(_Oid_in_type oid, _Data_in_type data) throw ()
{
   m_updates++;
   m_update_oids.push_front(oid);
   Cdmw::OsSupport::OS::sleep(1);
}

void Observer::on_remove(_Oid_in_type oid) throw ()
{
   m_removes++;
}

bool Observer::validate()
{
   bool result;

   result = !m_failure &&
      (m_inserts == 5495) && (m_removes == 0) && (m_updates == 5);

   if (!result)
   {
      cout << "counts are inserts: " << m_inserts << " removes: " << m_removes
           << " updates: " << m_updates << endl;
   }
   else
   {
      cout << "counts are inserts: " << m_inserts << " removes: " << m_removes
           << " updates: " << m_updates << endl;
   }

   return result;
}

}; // namespace FT
}; // namespace Cdmw
