/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "teststorageinterface/TestStateManager.hpp"
#include "teststorageinterface/TestStateTransfer.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"

int main (int argc, char* argv[])
{
    
    int ret_code = 0;
    
    Cdmw::OrbSupport::StrategyList strategyList;
    strategyList.add_OrbThreaded();
    strategyList.add_PoaThreadPerRequest();
    strategyList.add_multicast();
    //fixing memory leak
    CORBA::ORB_var orb = CORBA::ORB::_nil();
    orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, strategyList);

    CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
    PortableGroup::GOA_var rootGOA = PortableGroup::GOA::_narrow(obj.in());
    PortableServer::POAManager_var poaMgr = rootGOA->the_POAManager();
    poaMgr->activate();

    Cdmw::TestUtils::CORBATestManager::instance()->init(orb.in(),rootGOA.in(),argc,argv);
    Cdmw::TestUtils::CORBATestManager::instance()->run_tests();
    Cdmw::TestUtils::CORBATestManager::instance()->summary();
    //fixing memory leak
    if (! CORBA::is_nil(orb.in()))
    {
       try
       {
           Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
           orb->destroy();
           orb = CORBA::ORB::_nil(); 
       }
       catch( const CORBA::Exception &e )
       {
           std::cerr <<__FILE__<<" "<<__LINE__<<" "<< e << std::endl;
       }
       catch( const Cdmw::Exception &e )
       {
           std::cerr <<__FILE__<<" "<<__LINE__<<" "<< e << std::endl;
       }
       catch(...)
       {
           std::cerr << "Uncaught exception" << std::endl;
       }
   }
    return ret_code;
}

