/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_CDMW_FT_ULTIMATEFALLBACK_ULTIMATEFALLBACKINIT_HPP 
#define INCL_CDMW_FT_ULTIMATEFALLBACK_ULTIMATEFALLBACKINIT_HPP 

// Cdmw Files
#include <Foundation/orbsupport/CORBA.hpp>

/**
* Root namespace for CDMW runtime.
*/

namespace Cdmw
{
    
/**
* Namespace for CDMW Fault Tolerance types and data.
*/

namespace FT
{

namespace UltimateFallback
{

/**
*Purpose:
*<p>    This class provides an init of the Fault Tolerance Framework
* capability.
*
*Features:
*<p> Thread safe
*
*/
class UltimateFallbackInit 
{
public:

    
    /**
     * Purpose:
     * <p> Creates server request interceptor able to manage FT fallback mechanism.
     * 
     *@param argc The number of parameter received of the command line
     *@param argv The list of parameter received of the command line
     *
     *@return none
     *
     *@exception CORBA::INTERNAL  The CDMW <B>INTERNAL</B> minor code indicates an
     * internal error occured within Cdmw FT library.
     *@exception CORBA::SystemException Any CORBA system exception that
     * may be raised by the underlying ORB.
     */ 
    static 
    void
    Pre_init (int& argc, char** argv)
        throw(CORBA::SystemException);

    /**
     * Purpose:
     * <p> Creates persistent POA for the ultimate fallback servant
     * 
     *@param orb the underlying ORB
     *@param poa 
     *
     *@return none
     *
     *@exception CORBA::INTERNAL  The CDMW <B>INTERNAL</B> minor code indicates an
     * internal error occured within Cdmw FT library.
     *@exception CORBA::SystemException Any CORBA system exception that
     * may be raised by the underlying ORB.
     */ 
    static 
    void
    Init (CORBA::ORB_ptr orb)
        throw(CORBA::SystemException);


protected:    

private:

    UltimateFallbackInit() {}
    ~UltimateFallbackInit() {}

}; // End class FTUltimateFallbackInit 

} // End namespace UltimateFallback
} // End namespace FT
} // End namespace Cdmw



#endif // INCL_CDMW_FT_ULTIMATEFALLBACK_ULTIMATEFALLBACKINIT_HPP

