/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef _CDMW_FT_ULTIMATEFALLBACK_FALLBACK_IMPL_HPP_
#define _CDMW_FT_ULTIMATEFALLBACK_FALLBACK_IMPL_HPP_

#include <FaultTolerance/ftcommon/Fallback_impl.hpp>


namespace Cdmw {
namespace FT {
namespace UltimateFallback {

   /**
    * Default Implementation for fallback objects.
    * A Fallback object only replies to is_a() operations for checking type of the object
    * group but should not process other operations.
    * NOTA:
    *   It is expected that the <invoke> and <_primary_interface>
    *   methods will be only invoked by the POA in the context of
    *   serving a CORBA request. Invoking this method in other
    *   circumstances may lead to unpredictable results.
    */
   class Fallback_impl
      : public ::Cdmw::FT::Fallback_impl
   {

      public:
         /**
          * Creates a fallback default servant
          *
          * @param orb ...
          * @param poa ...
          */
         Fallback_impl(CORBA::ORB_ptr orb, 
                       PortableServer::POA_ptr poa,
                       CosNaming::NamingContextExt_ptr root)
            throw(CORBA::SystemException);
        
         /**
          * Destroys the fallback default servant.
          */
         ~Fallback_impl(void) throw ();


         /**
          * The _is_a() method receives a RepositoryId and returns
          * true if the invoked CORBA::Object is an instance of the
          * type with the specified RepositoryId (or an instance of
          * a inherited type).
          * This operation is called instead of the invoke() operation
          * in case of co-localised call.
          *
          * @param logical_type_id The RepositoryId to be compared.
          *
          * @return true if the invoked CORBA::Object is an instance
          * of the type with the specified RepositoryId.
          */
         virtual CORBA::Boolean _is_a (const char *logical_type_id);
        
      protected:
         virtual void invoke_i (CORBA::ServerRequest_ptr request,
                                const FallbackOid & foid);
         
      private:
         CORBA::Object_ptr get_taget_object(const char* operation,
                                            const FallbackOid & foid)
            throw (CORBA::SystemException);
         
         // Ultimate Fallback Root Context
         CosNaming::NamingContextExt_var m_root;

         // Default constructor.
         Fallback_impl(void)
            throw();
        
         // Hide the copy constructor.
         Fallback_impl(const Fallback_impl & fallback)
            throw();
        
         // Hide the copy assignment operator.
         Fallback_impl &
         operator=(const Fallback_impl & fallback)
            throw();
   };

}; // namespace UltimateFallback
}; // namespace FT
}; // namespace Cdmw

#endif // _CDMW_FT_ULTIMATEFALLBACK_FALLBACK_IMPL_HPP_

