/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Cdmw Files
#include <Foundation/common/Assert.hpp> 
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <FaultTolerance/ftcommon/MacroDef.hpp>
#include <FaultTolerance/ftcommon/FTLogger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <FaultTolerance/ftcommon/FTObjectIds.hpp>
// Local Files
#include <ftultimatefallback/FTORBInitializer.hpp>
#include <ftultimatefallback/FTUltimateFallbackInterceptor_impl.hpp>




namespace Cdmw
{
    
namespace FT
{
 
namespace UltimateFallback
{

FTUltimateFallbackInterceptor_impl* FTORBInitializer::M_interceptor = 0;


// constructor
FTORBInitializer::FTORBInitializer()
{
    CDMW_LOG_FUNCTION(Cdmw::FT::FTLogger::GetLogger(),"");
}

//destructor
FTORBInitializer::~FTORBInitializer()
{
    // Don't log function here because this destructor
    // is called with static desctruction of TAO objects.
    // And thus, it could be destroyed after logger (and its mutex)
}

void
FTORBInitializer::pre_init(PortableInterceptor::ORBInitInfo_ptr info) 
  throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Cdmw::FT::FTLogger::GetLogger(),"");
}

    
//
// Note: the interceptors are registered in post_init()
//       if their constructors require
//       ORBInitInfo::resolve_initial_reference(), which 
//       cannot be called in pre_init().
//
void
FTORBInitializer::post_init(PortableInterceptor::ORBInitInfo_ptr info) 
   throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Cdmw::FT::FTLogger::GetLogger(),"");

    //
    // Create and register the interceptors
    //
    CDMW_DEBUG(Cdmw::FT::FTLogger::GetLogger(),"post_init => Create the interceptors");
    M_interceptor
        = new FTUltimateFallbackInterceptor_impl(info); 

    try
    {
        info->add_server_request_interceptor(M_interceptor);
    }
    catch(const PortableInterceptor::ORBInitInfo::DuplicateName&)
    {
        CDMW_ASSERT(false);
    }
    

}

} // End of namespace UltimateFallback

} // End of namespace FT

} // End of namespace Cdmw



