/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Cdmw Files
#include <Foundation/common/Assert.hpp> 
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>

#if CDMW_ORB_VDR == tao && CDMW_ORB_VER > 13
#include <tao/ORBInitializer_Registry.h>
#endif


// Local Files
#include <ftultimatefallback/FTUltimateFallbackInit.hpp>
#include <ftultimatefallback/FTORBInitializer.hpp>
#include "ftultimatefallback/FallbackHelper.hpp"

#include <FaultTolerance/ftcommon/FTConfiguration.hpp>
#include <FaultTolerance/ftcommon/FTSystemProperties.hpp>
#include <FaultTolerance/ftcommon/Fallback_impl.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <FaultTolerance/ftcommon/FTLogger.hpp>
#include <Foundation/commonsvcs/serviceregistration/ServiceRegistration_impl.hpp>
#include <Foundation/orbsupport/CdmwObjectIds.hpp>
#include <FaultTolerance/ftcommon/UltimateFallbackFactory.hpp>

namespace
{
   const std::string DUMMY_HOST("localhost");
   const unsigned short int DUMMY_PORT = 1234;

//    class Fallback_impl : public Cdmw::FT::Fallback_impl
//    {
//       public:
//          Fallback_impl::Fallback_impl(CORBA::ORB_ptr          orb,
//                                       PortableServer::POA_ptr poa)
//             throw(CORBA::SystemException)
//             : ::Cdmw::FT::Fallback_impl(orb,poa)
//          {
//          }
//          virtual CORBA::Boolean _is_a (const char *logical_type_id)
//          {
//             // This shouldn't happen here as the interceptor should not let the request reach default servant.
//             CDMW_NEVER_HERE();
//             return false;
//          }
//       protected:
//          virtual void invoke_i (CORBA::ServerRequest_ptr request,
//                                 const Cdmw::FT::FallbackOid & foid)
//          {
//             // This shouldn't happen here as the interceptor should not let the request reach default servant.
//             CDMW_NEVER_HERE();
//          }
         
//    };

   // **********************************************************************
   // A SERVANT LOCATOR
   // **********************************************************************
   class TheServantLocator :
         public virtual PortableServer::ServantLocator,
         public virtual Cdmw::OrbSupport::RefCountLocalObject
   {
      public:
         
         TheServantLocator(CosNaming::NamingContextExt_ptr root)
            : m_root(CosNaming::NamingContextExt::_duplicate(root))
         {
            CDMW_LOG_FUNCTION(Cdmw::FT::FTLogger::GetLogger(),"");
            CDMW_ASSERT(!CORBA::is_nil(root));
         }
         ~TheServantLocator()
         {
            CDMW_LOG_FUNCTION(Cdmw::FT::FTLogger::GetLogger(),"");
         }
         
         PortableServer::Servant
         preinvoke(const PortableServer::ObjectId& oid,
                   PortableServer::POA_ptr poa,
                   const char* operation,
                   PortableServer::ServantLocator::Cookie&
                   the_cookie)
            throw(PortableServer::ForwardRequest, CORBA::SystemException)
         {
            CDMW_LOG_FUNCTION(Cdmw::FT::FTLogger::GetLogger(),"operation="<<operation);
            Cdmw::FT::FallbackOid foid(oid);
            CORBA::Object_var obj = 
               Cdmw::FT::UltimateFallback::FallbackHelper::Get_taget_object(m_root.in(),
                                                                            foid);

            // Trigger LOCATION_FORWARD
            CDMW_INFO(Cdmw::FT::FTLogger::GetLogger(),
                      "Ultimate fallback reached for object '" << foid.name()<<"'"
                      << ". Returning latest version reference");
            PortableServer::ForwardRequest ex(obj.in()); 
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;

            CDMW_NEVER_HERE();
            return 0;
         }
         void
         postinvoke(const PortableServer::ObjectId& oid,
                    PortableServer::POA_ptr poa,
                    const char* operation,
                    PortableServer::ServantLocator::Cookie the_cookie,
                    PortableServer::Servant the_servant)
            throw(CORBA::SystemException)
         {
            CDMW_LOG_FUNCTION(Cdmw::FT::FTLogger::GetLogger(),"");
            // TAO bizarity: I don't know why TAO calls postinvoke when preinvoke raise a ForwardRequest?!!!
//            CDMW_NEVER_HERE();
         }
      private:
         CosNaming::NamingContextExt_var m_root;
   }; // End TheServantLocator

}; // End of anonymous namespace



namespace Cdmw
{
    
namespace FT
{

namespace UltimateFallback
{

void
UltimateFallbackInit::Pre_init (int& argc, char** argv)
    throw(CORBA::SystemException)
{
//     // register_interceptors();
//     PortableInterceptor::ORBInitializer_var initializer = new FTORBInitializer();
        
//     PortableInterceptor::register_orb_initializer(initializer.in());
}

void
UltimateFallbackInit::Init (CORBA::ORB_ptr orb)
    throw(CORBA::SystemException)
{
   CDMW_LOG_FUNCTION(Cdmw::FT::FTLogger::GetLogger(),"");

   CORBA::Object_var obj2 = 
      orb->resolve_initial_references(Cdmw::OrbSupport::SERVICE_REGISTRATION_NO_CACHE_ObjectId);
   CosNaming::NamingContextExt_var nc = CosNaming::NamingContextExt::_narrow(obj2.in());
   if (CORBA::is_nil(nc.in())) {
      CDMW_FATAL(Cdmw::FT::FTLogger::GetLogger(),
                 "ServiceRegistration service not registered!");
      CORBA::BAD_INV_ORDER ex (Cdmw::OrbSupport::BAD_INV_ORDER,
                               CORBA::COMPLETED_NO);
      CDMW_LOG_FUNCTION_EXCEPTION(ex);
      throw ex;
   }


   //
   // Create and install servant locator
   //
   {
      PortableServer::ServantLocator* locator = new TheServantLocator(nc.in());
      PortableServer::ServantManager_var the_servant_manager = locator;

      (void*)Cdmw::FT::UltimateFallbackFactory::Init(orb,
                                                     DUMMY_HOST.c_str(),
                                                     DUMMY_PORT,
                                                     locator);
   }



//    // create Fallback default servant
//    ::Fallback_impl* defaultServant =
//       new ::Fallback_impl(orb, root_poa.in());
   
//    try {
//       // set default servant
//       persistent_poa->set_servant(defaultServant);
//       defaultServant->_remove_ref();
//    } catch (const PortableServer::POA::WrongPolicy& e) {
//       // shouldn't happen since m_poa has USE_DEFAULT_SERVANT policy set
//       CDMW_NEVER_HERE();
//    }
   

}


} // End of namespace UltimateFallback

} // End of namespace FT

} // End of namespace Cdmw



