/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

// Cdmw Files
#include <Foundation/common/Assert.hpp> 
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Foundation/commonsvcs/serviceregistration/ServiceRegistration_impl.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <FaultTolerance/idllib/CdmwFTReplicationManager.stub.hpp>
#include <FaultTolerance/ftcommon/FTObjectIds.hpp>
#include <FaultTolerance/ftcommon/FTLogger.hpp>
#include <FaultTolerance/ftcommon/FallbackOid.hpp>
#include <FaultTolerance/ftcommon/FTSystemProperties.hpp>



// Local Files
#include <ftultimatefallback/FTUltimateFallbackInterceptor_impl.hpp>


namespace 
{
    const PortableInterceptor::ORBInitInfo::ObjectId FT_SERVER_REQUEST_INTERCEPTOR_NAME 
        = "FTUltimateFallbackInterceptor";

} // end of anonymous namespace


namespace Cdmw
{
    
namespace FT
{
   
namespace UltimateFallback
{

FTUltimateFallbackInterceptor_impl::FTUltimateFallbackInterceptor_impl(
                 PortableInterceptor::ORBInitInfo_ptr info)
{
    CDMW_LOG_FUNCTION(Cdmw::FT::FTLogger::GetLogger(),"");
}

char*
FTUltimateFallbackInterceptor_impl::name() 
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Cdmw::FT::FTLogger::GetLogger(),"");
    CDMW_LOG_FUNCTION_RETURN(FT_SERVER_REQUEST_INTERCEPTOR_NAME);
    return CORBA::string_dup(FT_SERVER_REQUEST_INTERCEPTOR_NAME);
}

void
FTUltimateFallbackInterceptor_impl::destroy() 
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Cdmw::FT::FTLogger::GetLogger(),"");
}

void
FTUltimateFallbackInterceptor_impl::receive_request_service_contexts(
        PortableInterceptor::ServerRequestInfo_ptr ri)
    throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
   // nothing to do!
   CDMW_LOG_FUNCTION(Cdmw::FT::FTLogger::GetLogger(),"operation="<<ri->operation());
}

void
FTUltimateFallbackInterceptor_impl::receive_request(PortableInterceptor::ServerRequestInfo_ptr ri)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
   CDMW_LOG_FUNCTION(Cdmw::FT::FTLogger::GetLogger(),"operation="<<ri->operation());

    //    * NOTA: object_id, adapter_id and adapter_name are available here!
    //    1- Create a FallbackOid(object_id) in case of adapter_name == "UltimateFallbackPOA"
    //    2- get group_domain_id and perform a discover global service
    //        CORBA::Object_var obj 
    //            = Cdmw::CommonSvcs::ServiceRegistration::ServiceRegistration_impl::get_instance(
    //                  )->get_service(group_domain_id, true);
    //    3- Narrow to naming context 
    //    4- nc->resolve("group_id.gid");
    //    5- raise PortableInterceptor::ForwardRequest 
    //
    // ==> (Daemon's NC == rir("NameService")) -> resolve_str( fallback_oid.to_name()); --> // "group_domain_id/group_id.gid"
    //     ServiceRegistration::get_service(group_domain_id)
    //         -  Have a canonical domain representation in CARDAMOM
    //         -  ( la corbaloc)
    //         -  
    //         -  
    //     Be able to define a (name, group_domain_id) mappping/definition for daemons
    //     default [name = group_domain_id], [protocole = cdm service discovery]
    //     ex: FT_Domain-X = corbaloc::localhost:1234/cdmw.name-domain.FTDomainX
    // 
    
    //    * Q1: Why not use a default servant instead of a PI? 
    //    * Q2: Why not use a servant locator instead of a PI? 
    //
    //  Requirement
    //

   PortableInterceptor::AdapterName_var an = ri->adapter_name();
   std::string poa_name = an[0L].in();
   CDMW_ASSERT(!poa_name.empty());

   if (poa_name == Cdmw::FT::FALLBACK_POA_NAME) {
      PortableServer::ObjectId_var oid = ri->object_id();
      Cdmw::FT::FallbackOid foid(oid.in());
      CDMW_DEBUG(Cdmw::FT::FTLogger::GetLogger(),
                 "Request '"<<ri->operation()<<"' on Ultimate Fallback object\n"
                 "oid='"<<foid.to_string()<<"'\n"
                 "name='"<<foid.name()<<"'");
      Cdmw::CommonSvcs::ServiceRegistration::ServiceRegistration_impl * srv_reg =
         Cdmw::CommonSvcs::ServiceRegistration::ServiceRegistration_impl::Get_instance();
      CosNaming::NamingContextExt_var nc = srv_reg->get_root_context(true);
      if (CORBA::is_nil(nc.in())) {
         CORBA::TRANSIENT ex (Cdmw::OrbSupport::TRANSIENT,
                              CORBA::COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(ex);
         throw ex;
      }
      CORBA::Object_var obj = CORBA::Object::_nil();

      // Handling of resolve_str() user exceptions
      // CosNaming::NamingContext::NotFound -> CORBA::OBJECT_NOT_EXIST
      // CosNaming::NamingContext::CannotProceed --> INV_OBJREF
      // CosNaming::NamingContext::InvalidName --> INV_OBJREF
      // CORBA::SystemException --> rethrow
      try {
         obj = nc->resolve_str(foid.name().c_str());
      } catch (const CosNaming::NamingContext::NotFound & ex) {
         CDMW_WARN(Cdmw::FT::FTLogger::GetLogger(),
                   "Ultimate fallback: object '" << foid.name()<<"' NotFound"
                   << ". Raising CORBA::OBJECT_NOT_EXIST");
         CORBA::OBJECT_NOT_EXIST x(OrbSupport::OBJECT_NOT_EXIST, CORBA::COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(x);
         throw x;
      } catch (const CosNaming::NamingContext::InvalidName & ex) {
         CDMW_WARN(Cdmw::FT::FTLogger::GetLogger(),
                   "Ultimate fallback: object '" << foid.name()<<"' InvalidName"
                   << ". Raising CORBA::INV_OBJREF");
         CORBA::INV_OBJREF x(OrbSupport::INV_OBJREF, CORBA::COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(x);
         throw x;
      } catch (const CosNaming::NamingContext::CannotProceed & ex) {
         CDMW_WARN(Cdmw::FT::FTLogger::GetLogger(),
                   "Ultimate fallback: object '" << foid.name()<<"' CannotProceed"
                   << ". Raising CORBA::INV_OBJREF");
         CORBA::INV_OBJREF x(OrbSupport::INV_OBJREF, CORBA::COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(x);
         throw x;
      }
      catch (const CORBA::SystemException& ex)
      {
         CDMW_WARN(Cdmw::FT::FTLogger::GetLogger(),
                   "Ultimate fallback: System exception raised getting object '" << foid.name()<<"'"
                   << ". Rethrowing exception '" << ex << "'");
         CDMW_LOG_FUNCTION_EXCEPTION(ex);
         throw;
      }

      // Trigger LOCATION_FORWARD_PERM
      CDMW_INFO(Cdmw::FT::FTLogger::GetLogger(),
                "Ultimate fallback reached for object '" << foid.name()<<"'"
                << ". Returning latest version reference");
      PortableInterceptor::ForwardRequest ex(obj.in()); 
      CDMW_LOG_FUNCTION_EXCEPTION(ex);
      throw ex;

   }
    
}
    

void
FTUltimateFallbackInterceptor_impl::send_reply(PortableInterceptor::ServerRequestInfo_ptr ri)
        throw (CORBA::SystemException)
{
    // nothing to do!
   CDMW_LOG_FUNCTION(Cdmw::FT::FTLogger::GetLogger(),"operation="<<ri->operation());
}

void
FTUltimateFallbackInterceptor_impl::send_other(PortableInterceptor::ServerRequestInfo_ptr ri)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
    // nothing to do!
    CDMW_LOG_FUNCTION(Cdmw::FT::FTLogger::GetLogger(),"operation="<<ri->operation());
}

void
FTUltimateFallbackInterceptor_impl::send_exception(PortableInterceptor::ServerRequestInfo_ptr ri)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
    // nothing to do!
    CDMW_LOG_FUNCTION(Cdmw::FT::FTLogger::GetLogger(),"operation="<<ri->operation());
}


} // End of namespace UltimateFallback

} // End of namespace FT

} // End of namespace Cdmw


