/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "tao/AnyTypeCode/TypeCode.h"
#include "tao/AnyTypeCode/NVList.h"
#include "ftultimatefallback/Fallback_impl.hpp"
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/common/Assert.hpp> // for CDMW_ASSERT macro
#include <FaultTolerance/ftcommon/MacroDef.hpp> // for CDMW_FT_DEBUG macro

#include <FaultTolerance/ftcommon/FallbackOid.hpp>

#include <FaultTolerance/ftcommon/FTLogger.hpp>

#include <Foundation/logging/FunctionLogger.hpp>
#include <FaultTolerance/ftcommon/FTObjectIds.hpp>
#include <Foundation/orbsupport/CORBAExceptionHelper.hpp>
#include "ftultimatefallback/FallbackHelper.hpp"

namespace Cdmw {
namespace FT {

namespace UltimateFallback {
    

    
    /**
     * Creates a fallback default servant
     *
     * @param orb ...
     * @param poa ...
     */
    Fallback_impl::Fallback_impl(CORBA::ORB_ptr                  orb,
                                 PortableServer::POA_ptr         poa,
                                 CosNaming::NamingContextExt_ptr root)
        throw(CORBA::SystemException)
       : ::Cdmw::FT::Fallback_impl(orb,poa),
       m_root(CosNaming::NamingContextExt::_duplicate(root))
    {
        CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
        CDMW_ASSERT(!CORBA::is_nil(root));
    }
        
        
    /**
     * Destroys the fallback default servant.
     */
    Fallback_impl::~Fallback_impl(void) throw ()
    {
        CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
    }
    
    
    /**
     * The _is_a() method receives a RepositoryId and returns
     * true if the invoked CORBA::Object is an instance of the
     * type with the specified RepositoryId (or an instance of
     * a inherited type).
     * This operation is called instead of the invoke() operation
     * in case of co-localised call.
     *
     * @param logical_type_id The RepositoryId to be compared.
     *
     * @return true if the invoked CORBA::Object is an instance
     * of the type with the specified RepositoryId.
     */
    CORBA::Boolean Fallback_impl::_is_a (const char *logical_type_id)
    {
        CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"logical_type_id="<<logical_type_id);
        PortableServer::ObjectId_var oid = this->get_object_id();
        Cdmw::FT::FallbackOid foid(oid.in());
        CORBA::Object_var obj = this->get_taget_object("_is_a",foid);
        CORBA::Boolean result = obj->_is_a(logical_type_id);
        CDMW_LOG_FUNCTION_RETURN(result);
        return result;
    }

        
    /**
     * The invoke() method receives requests issued to any CORBA
     * object incarnated by the DSI servant and performs the
     * processing necessary to execute the request.
     *
     * @param request A ServerRequest local object containing request information.
     *
     * @exception TODO...
     */
   void Fallback_impl::invoke_i (CORBA::ServerRequest_ptr request,
                                 const FallbackOid & foid)
    {
        CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"operation="<<request->operation());
        CDMW_NEVER_HERE();
/*
  TODO:
        // Get the operation name for this request
        const char *operation = request->operation ();

        try {
           CORBA::Object_var obj = this->get_taget_object(operation,foid);
           PortableServer::ForwardRequest ex(obj.in()); 
           CORBA::Any any;
           Cdmw::OrbSupport::CORBAExceptionHelper::Insert_Exception<PortableServer::ForwardRequest>(any, ex);
//           any <<= ex;
           request->set_exception(any);
        } catch (const CORBA::SystemException & ex) {
           CORBA::Any any;
           Cdmw::OrbSupport::CORBAExceptionHelper::Insert_SystemException(any, ex);
//           any <<= ex;
           // Pass the exception back to the server request...
           request->set_exception(any);
        }
*/
    }
        
    
   CORBA::Object_ptr Fallback_impl::get_taget_object(const char* operation,
                                                     const FallbackOid & foid)
      throw (CORBA::SystemException)
   {
      CDMW_LOG_FUNCTION(Cdmw::FT::FTLogger::GetLogger(),"operation="<<operation);
      CDMW_DEBUG(Cdmw::FT::FTLogger::GetLogger(),
                 "Request '"<<operation<<"' on Ultimate Fallback object\n"
                 "oid='"<<foid.to_string()<<"'\n"
                 "name='"<<foid.name()<<"'");
      return Cdmw::FT::UltimateFallback::FallbackHelper::Get_taget_object(m_root.in(),
                                                                          foid);
   }


}; // namespace UltimateFallback
}; // namespace FT
}; // namespace Cdmw

