/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_CDMW_FT_LOCATIONFAULTDETECTOR_REQUESTTIMEOUTLOCALPULLMONITORABLE_IMPL_HPP
#define INCL_CDMW_FT_LOCATIONFAULTDETECTOR_REQUESTTIMEOUTLOCALPULLMONITORABLE_IMPL_HPP

#include <Foundation/idllib/CdmwFTLocalPullMonitorable.skel.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>

/**
* Root namespace for CDMW runtime.
*/

namespace Cdmw
{
 
/**
* Namespace for CDMW Fault Tolerance types and data.
*/

namespace FT
{
    
/**
* Namespace for CDMW Fault Tolerance location fault detector functionality
*/

namespace Watchdog
{

/**
*Purpose:
*<p>    This class implements the CdmwFT::LocalPullMonitorable class.
* It checks that no operation call is during more than a specified timeout.
*
*Features:
*<p> Thread safe
*
*/

class RequestTimeoutLocalPullMonitorable_impl : virtual public CdmwFT::LocalPullMonitorable,
                                                virtual public Cdmw::OrbSupport::RefCountLocalObject
{
    public:
        /**
         * Purpose:
         * <p> Constructor
         *
         */ 
        RequestTimeoutLocalPullMonitorable_impl() 
            throw();
        
        /**
         * Purpose:
         * <p> Destructor
         * 
         */ 
        virtual 
        ~RequestTimeoutLocalPullMonitorable_impl()
            throw();

        /**
         * Purpose:
         * <p> to call to check if object is still living
         * 
         *@exception CORBA::SystemException
         *
         */ 
        virtual CORBA::Boolean is_alive ()
            throw(CORBA::SystemException);
        
    private:
        
        /**
         * Purpose:
         * <p>  Copy constructor
         * 
         */ 
        RequestTimeoutLocalPullMonitorable_impl (const RequestTimeoutLocalPullMonitorable_impl& rhs)
            throw();
        
        /**
         * Purpose:
         * <p> Assignment operator
         * 
         */ 
        RequestTimeoutLocalPullMonitorable_impl&
        operator=(const RequestTimeoutLocalPullMonitorable_impl& rhs)
            throw();
        
}; // End class RequestTimeoutLocalPullMonitorable_impl


}; // End namespace Watchdog

}; // End namespace FT

}; // End namespace Cdmw

#endif // INCL_CDMW_FT_LOCATIONFAULTDETECTOR_REQUESTTIMEOUTLOCALPULLMONITORABLE_IMPL_HPP
