/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_CDMW_FT_LOCATION_FAULT_DETECTOR_SERVICE_INIT_HPP 
#define INCL_CDMW_FT_LOCATION_FAULT_DETECTOR_SERVICE_INIT_HPP 

// Cdmw Files
#include <Foundation/orbsupport/CORBA.hpp>

/**
* Root namespace for CDMW runtime.
*/

namespace Cdmw
{
    
/**
* Namespace for CDMW Fault Tolerance types and data.
*/

namespace FT
{


/**
*Purpose:
*<p>    This class provides an init of the Fault Tolerance Framework
* capability (location fault detector part).
*
*Features:
*<p> Thread safe
*
*/
class WatchdogServiceInit 
{
public:

    
    /**
     * Purpose:
     * <p> Initialises location fault detection mechanism : it creates server
     * request interceptor and client request interceptor able to log the time
     * of an incoming and outgoing request.
     * 
     *@param argc The number of parameter received of the command line
     *@param argv The list of parameter received of the command line
     *
     *@return none
     *
     *@exception CORBA::INTERNAL  The CDMW <B>INTERNAL</B> minor code indicates an
     * internal error occured within Cdmw FT library.
     *@exception CORBA::SystemException Any CORBA system exception that
     * may be raised by the underlying ORB.
     */ 
    static 
    void
    Init (int& argc, char** argv)
        throw(CORBA::SystemException);

    //fixing memory leak
    static int Cleanup();
    /**
     * Purpose:
     * <p> Resets the timeout value used in the location fault detection mechanism to a new value.
     * 
     *@param milliseconds maximum elapse time of request processing for the server.
     * (expressed in ms)
     *
     *@return none
     *
     *@exception none.
     */ 
    static
    void
    Set_timeout (CORBA::ULong milliseconds) throw (CORBA::BAD_PARAM);

protected:    

private:

    WatchdogServiceInit() {}
    ~WatchdogServiceInit() {}


}; // End class WatchdogServiceInit

} // End namespace FT
} // End namespace Cdmw



#endif // INCL_CDMW_FT_LOCATION_FAULT_DETECTOR_SERVICE_INIT_HPP

