/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_CDMW_FT_LOCATION_FAULT_DETECTOR_REQUEST_TIMEOUT_CLIENT_REQUEST_INTERCEPTOR_IMPL_HPP
#define INCL_CDMW_FT_LOCATION_FAULT_DETECTOR_REQUEST_TIMEOUT_CLIENT_REQUEST_INTERCEPTOR_IMPL_HPP

// Cdmw Files
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/orbsupport/PortableInterceptor.hpp>


/**
* Root namespace for CDMW runtime.
*/
namespace Cdmw
{
    
/**
* Namespace for CDMW Fault Tolerance types and data.
*/
namespace FT
{

/**
* Namespace for CDMW Fault Tolerance location fault detector functionality
*/
namespace Watchdog
{

/**
*Purpose:
*<p>    This class implements the ClientRequestInterceptor class.
*
*Features:
*<p> Thread safe
*
*/
class RequestTimeoutClientRequestInterceptor_impl : public PortableInterceptor::ClientRequestInterceptor,
                                      public Cdmw::OrbSupport::RefCountLocalObject
{
public:
    /**
     * Purpose:
     * <p> Constructor
     * @param info ORB init information
     * @param request_id_slot_id slot id used to store the date of the
     * outgoing request.
     *
     */ 
    RequestTimeoutClientRequestInterceptor_impl(
          PortableInterceptor::ORBInitInfo_ptr info,
          const PortableInterceptor::SlotId  request_id_slot_id);

    /**
     * Purpose:
     * <p> Return the name of the interceptor
     *
     */
    virtual char*
    name() throw (CORBA::SystemException);

    /**
     * Purpose:
     * <p> Destroy the interceptor
     *
     */
    virtual void
    destroy() throw (CORBA::SystemException);

    /**
     * Purpose:
     * <p> This interception point allows an Interceptor to query request
     * information and modify the service context before the request is 
     * sent to the server. 
     * Here, the date of the outgoing request is stored here.
     * @param ri Client request information
     *
     */
    virtual void
    send_request(
        PortableInterceptor::ClientRequestInfo_ptr ri)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest);

    /**
     * Purpose:
     * <p>  This interception point allows an Interceptor to query information
     * during a Time-Independent Invocation (TII) polling get reply sequence.
     * @param ri Client request information
     *
     */
    virtual void
    send_poll(PortableInterceptor::ClientRequestInfo_ptr ri)
        throw (CORBA::SystemException);

    /**
     * Purpose:
     * <p> This interception point allows an Interceptor to query the 
     * information on a reply after it is returned from the server and 
     * before control is returned to the client.
     * Here the date of the outgoing request is reset and T0 of incoming 
     * request updated to remove the duration of the outgoing request.
     * @param ri Client request information
     */
    virtual void
    receive_reply(PortableInterceptor::ClientRequestInfo_ptr ri)
        throw (CORBA::SystemException);

    /**
     * Purpose:
     * <p> When an exception occurs, this interception point is called. 
     * It allows an Interceptor to query the exception's information before
     * it is raised to the client.
     * @param ri Client request information
     */
    virtual void
    receive_exception(PortableInterceptor::ClientRequestInfo_ptr ri)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest);

    /**
     * Purpose:
     * <p> This interception point allows an Interceptor to query the 
     * information available when a request results in something other 
     * than a normal reply or an exception. For example, a request could 
     * result in a retry (for example, a GIOP Reply with a LOCATION_FORWARD
     * status was received); or on asynchronous calls, the reply does not 
     * immediately follow the request, but control shall return to the client
     * and an ending interception point shall be called.
     * @param ri Client request information
     */
    virtual void
    receive_other(PortableInterceptor::ClientRequestInfo_ptr ri)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest);


private:
    RequestTimeoutClientRequestInterceptor_impl(const RequestTimeoutClientRequestInterceptor_impl&);
    RequestTimeoutClientRequestInterceptor_impl& operator=(const RequestTimeoutClientRequestInterceptor_impl&);

    // current object request id slot id
    PortableInterceptor::SlotId     m_request_id_slot_id;

};

} // End of namespace FTInit

} // End of namespace FT

} // End of namespace Cdmw

#endif // INCL_CDMW_FT_LOCATION_FAULT_DETECTOR_REQUEST_TIMEOUT_CLIENT_REQUEST_INTERCEPTOR_IMPL_HPP

