/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_CDMW_FT_LOCATION_FAULT_DETECTOR_REQUEST_TIMEOUT_ORB_INITIALIZER_IMPL_HPP
#define INCL_CDMW_FT_LOCATION_FAULT_DETECTOR_REQUEST_TIMEOUT_ORB_INITIALIZER_IMPL_HPP

// Cdmw Files
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/orbsupport/PortableInterceptor.hpp>

#include <ftwatchdog/FTRequestTimeoutLogger.hpp>

/**
* Root namespace for CDMW runtime.
*/

namespace Cdmw
{
    
/**
* Namespace for CDMW Fault Tolerance types and data.
*/
    
namespace FT
{
    
/**
* Namespace for CDMW Fault Tolerance location fault detector functionality
*/

namespace Watchdog
{

// forward declaration
class RequestTimeoutServerRequestInterceptor_impl;
class RequestTimeoutClientRequestInterceptor_impl;

/**
*Purpose:
*<p>    This class creates RequestTimeoutServerRequestInterceptor_impl and 
* RequestTimeoutClientRequestInterceptor_impl.
*
*/

class RequestTimeoutORBInitializer: virtual public PortableInterceptor::ORBInitializer,
                                   virtual public Cdmw::OrbSupport::RefCountLocalObject
{
public:

    /**
     * Purpose:
     * <p> Constructor
     *
     */ 
    RequestTimeoutORBInitializer(CORBA::ULong timeout);

    /**
     * Purpose:
     * <p> Destructor
     * 
     */ 
    ~RequestTimeoutORBInitializer();

    /**
     * Purpose:
     * <p> This operation is called during ORB initialization.
     * 
     */ 
    virtual void
    pre_init(PortableInterceptor::ORBInitInfo_ptr info) 
       throw (CORBA::SystemException);

    
    /**
     * Purpose:
     * <p> This operation is called during ORB initialization.
     *
     * Note: the interceptors are registered in post_init()
     *       if their constructors require
     *       ORBInitInfo::resolve_initial_reference(), which XXX
     *       cannot be called in pre_init().
     *
     */
    virtual void
    post_init(PortableInterceptor::ORBInitInfo_ptr info) 
       throw (CORBA::SystemException);

private:
    static RequestTimeoutLogger*                         M_logger;
    static RequestTimeoutServerRequestInterceptor_impl*  M_server_interceptor;
    static RequestTimeoutClientRequestInterceptor_impl*  M_client_interceptor;
};

} // End of namespace Watchdog

} // End of namespace FT

} // End of namespace Cdmw

#endif  // INCL_CDMW_FT_LOCATION_FAULT_DETECTOR_REQUEST_TIMEOUT_ORB_INITIALIZER_IMPL_HPP

