/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Cdmw Files
#include <Foundation/common/Assert.hpp> 
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <FaultTolerance/ftcommon/MacroDef.hpp>
#include <FaultTolerance/ftcommon/FTLogger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>

// Local Files
#include <ftwatchdog/FTRequestTimeoutORBInitializer.hpp>
#include <ftwatchdog/FTRequestTimeoutServerRequestInterceptor_impl.hpp>
#include <ftwatchdog/FTRequestTimeoutClientRequestInterceptor_impl.hpp>
#include <ftwatchdog/FTRequestTimeoutLogger.hpp>




namespace 
{
} // end of anonymous namespace


namespace Cdmw
{
    
namespace FT
{

namespace Watchdog
{
RequestTimeoutServerRequestInterceptor_impl* 
    RequestTimeoutORBInitializer::M_server_interceptor = 0;
RequestTimeoutClientRequestInterceptor_impl* 
    RequestTimeoutORBInitializer::M_client_interceptor = 0;


// constructor
RequestTimeoutORBInitializer::RequestTimeoutORBInitializer(CORBA::ULong timeout)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"timeout" << timeout);
    
    RequestTimeoutLogger::Init_request_timeout_logger(timeout);
}

// destructor
RequestTimeoutORBInitializer::~RequestTimeoutORBInitializer()
{
}

void
RequestTimeoutORBInitializer::pre_init(PortableInterceptor::ORBInitInfo_ptr info) 
  throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
}

    
//
// Note: the interceptors are registered in post_init()
//       if their constructors require
//       ORBInitInfo::resolve_initial_reference(), which 
//       cannot be called in pre_init().
//
void
RequestTimeoutORBInitializer::post_init(PortableInterceptor::ORBInitInfo_ptr info) 
   throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
    //
    // allocate needed slot_id
    //
    CDMW_DEBUG(FTLogger::GetLogger(),"post_init => allocate needed slot_id");
    PortableInterceptor::SlotId  is_request_id_slot_id 
       = info->allocate_slot_id();

    //
    // Create and register the interceptors
    //
    CDMW_DEBUG(FTLogger::GetLogger(),"post_init => Create the interceptors");
    M_server_interceptor
        = new RequestTimeoutServerRequestInterceptor_impl(info, 
                                                          is_request_id_slot_id);

    M_client_interceptor
        = new RequestTimeoutClientRequestInterceptor_impl(info, 
                                                          is_request_id_slot_id);

    try
    {
        info->add_server_request_interceptor(M_server_interceptor);
        info->add_client_request_interceptor(M_client_interceptor);
    }
    catch(const PortableInterceptor::ORBInitInfo::DuplicateName&)
    {
        // can not happened while interceptor is not named!
        CDMW_ASSERT(false);
    }
    

}

} // End of namespace Watchdog

} // End of namespace FT

} // End of namespace Cdmw



