/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Cdmw Files
#include <Foundation/common/Assert.hpp> 
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>

#include <FaultTolerance/ftwatchdog/WatchdogServiceInit.hpp>
#if CDMW_ORB_VDR == tao && CDMW_ORB_VER > 13
#include <tao/ORBInitializer_Registry.h>
#endif

#include <Foundation/logging/LogManager.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <FaultTolerance/ftcommon/FTLogger.hpp>
#include <FaultTolerance/ftcommon/MacroDef.hpp>
#include <FaultTolerance/ftcommon/FTSystemProperties.hpp>


// Local Files
#include <ftwatchdog/FTRequestTimeoutORBInitializer.hpp>
#include <ftwatchdog/FTRequestTimeoutLogger.hpp>

#include <FaultTolerance/ftcommon/FTConfiguration.hpp>
namespace 
{
   /**
    * Default timeout value
    */
   const int CDMW_DEFAULT_WATCHDOG_TIMEOUT_VALUE = 900000; // 15 minutes";
} // end anonymous namespace


namespace Cdmw
{
    
namespace FT
{

void
WatchdogServiceInit::Init (int& argc, char** argv)
    throw(CORBA::SystemException)
{
    // Init the Logging library
    Cdmw::Logging::LogManager::Init(argc, argv);

    // Init System Properties
    Cdmw::OsSupport::SystemProperties::Init(argc, argv);

    // get default request_timeout value if any 
    // (CDMW internal use for debug purpose if CDMW_init is not called)
    int default_request_timeout =
       Cdmw::OsSupport::SystemProperties::Get_Property(
               WATCHDOG_DEFAULT_TIMEOUT_PROPERTY_NAME, 
               CDMW_DEFAULT_WATCHDOG_TIMEOUT_VALUE);
    Cdmw::OsSupport::SystemProperties::Set_Property(
               WATCHDOG_DEFAULT_TIMEOUT_PROPERTY_NAME, 
               default_request_timeout);
    
    // get request_timeout value
    int request_timeout =
       Cdmw::OsSupport::SystemProperties::Get_Property(
               WATCHDOG_TIMEOUT_PROPERTY_NAME, 
               default_request_timeout);
    
    if (request_timeout > 0) 
    {
        CDMW_INFO(FTLogger::GetLogger(), 
                   "register RequestTimeoutORBInitializer with request_timeout of " 
                    << request_timeout << "ms" );

        PortableInterceptor::ORBInitializer_var timeout_initializer 
           = new Cdmw::FT::Watchdog::RequestTimeoutORBInitializer(
                   request_timeout);
        
        PortableInterceptor::register_orb_initializer(timeout_initializer.in());
    } else {
       CDMW_WARN(FTLogger::GetLogger(), 
                 "Invalid timeout value '"
                 << request_timeout 
                 << "'.RequestTimeoutORBInitializer will not be registered!");
    }
}



void
WatchdogServiceInit::Set_timeout (CORBA::ULong milliseconds)
    throw(CORBA::BAD_PARAM)
{
   CDMW_INFO(FTLogger::GetLogger(), 
             "Setting a new timeout value of " 
             << milliseconds << "ms" );
   if (milliseconds > 0) 
   {
      CDMW_INFO(FTLogger::GetLogger(), 
                "Reseting RequestTimeoutORBInitializer with request_timeout of " 
                << milliseconds << "ms" );
      Watchdog::RequestTimeoutLogger::Get_instance()->set_timeout(milliseconds);
      Cdmw::OsSupport::SystemProperties::Set_Property(
         WATCHDOG_TIMEOUT_PROPERTY_NAME, 
         milliseconds);
      
    } else {
       CDMW_ERROR(FTLogger::GetLogger(), 
                 "Invalid timeout value '"<< milliseconds << "'. Value ignored in RequestTimeoutORBInitializer!");
       
       throw CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAM,
                              CORBA::COMPLETED_NO );
    }
}

//fixing memory leak
int WatchdogServiceInit::Cleanup()
{
  return 0;  
}

} // End of namespace FT

} // End of namespace Cdmw



