/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <fstream>

// Cdmw Files
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/ossupport/OS.hpp>

#include <testfttimeoutrequest/TestComput_impl.hpp>
#include <idllib/CdmwFTActivation.skel.hpp>

#include <idllib/CdmwFTCurrent.stub.hpp>



namespace Cdmw
{
namespace Test
{



ComputInterface_impl::ComputInterface_impl(CORBA::ORB_ptr orb, 
                                         std::string location,
                                         ::CdmwFT::Location::ActivationHandler_ptr activation_handler )
    throw(CORBA::SystemException)
    : m_orb(CORBA::ORB::_duplicate(orb)),
      m_location(location),
      m_activation_handler(::CdmwFT::Location::ActivationHandler::_duplicate(activation_handler))
{ 
}


ComputInterface_impl::~ComputInterface_impl()
    throw()
{
}

CORBA::Long
ComputInterface_impl::get_result (CORBA::ULong time_ms)
     throw(CORBA::SystemException)                  
{
    // export the location to a file
    std::ofstream os("comput.stt");
    os << m_location;
    os.close();

    std::cout << m_location << " <####### Comput " << time_ms <<"  ######>" << std::endl;
    Cdmw::OsSupport::OS::sleep(time_ms);

    std::cout << m_location << " <####### End of Comput " << time_ms <<"  ######>" << std::endl;
    try
    {
        std::cout << m_location << "ComputInterface_impl::get_result => resolve_initial_references FTCurrent" << std::endl;
        CORBA::Object_var obj = m_orb->resolve_initial_references("FTCurrent");

        CdmwFT::Current_var ft_current = CdmwFT::Current::_narrow(obj.in());

        ::FT::FTDomainId ft_domain_id = ft_current->get_ft_domain_id();
        std::cout << "    ft_domain_id = " << ft_domain_id << std::endl;

        ::FT::ObjectGroupId group_id = ft_current->get_object_group_id();
        std::cout << "    group_id = " << group_id << std::endl;

        ::FT::ObjectGroupRefVersion group_ref_version 
            = ft_current->get_object_group_ref_version();
        std::cout << "    group_ref_version = " << group_ref_version << std::endl;

        std::string client_id = ft_current->get_client_id();
        std::cout << "    client_id = " << client_id << std::endl;

        CORBA::Long retention_id = ft_current->get_retention_id();
        std::cout << "    retention_id = " << retention_id << std::endl;

        TimeBase::TimeT expiration_time = ft_current->get_expiration_time();
        std::cout << "    expiration_time = " << expiration_time << std::endl;
    }
    catch (const CORBA::ORB::InvalidName& )
    {
        std::cerr << "ComputInterface_impl::get_result => FTCurrent is an unknown ObjectID!" << std::endl;
        throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL,
                                          CORBA::COMPLETED_NO);
    }
    catch (const CdmwFT::Current::NoContext&)
    {
        std::cerr << "ComputInterface_impl::get_result => NoContext because the call is done on an IOR not an IOGR."
                  << std::endl;
    }
    catch (const CORBA::SystemException& e)
    {
        std::cerr << "ComputInterface_impl::get_result => CORBA::SystemException raised : " << e << std::endl;
        throw;
    }
    catch (const CORBA::Exception& e)
    {
        std::cerr << "ComputInterface_impl::get_result => CORBA::Exception raised : " << e << std::endl;
        throw;
    }

    return time_ms;
}

void 
ComputInterface_impl::init(::FT::ObjectGroup_ptr group_ref)
     throw(CORBA::SystemException)                  
{
    try
    {
        CORBA::Object_var obj = m_orb->resolve_initial_references("FTActivationManager");
        CdmwFT::Location::ActivationManager_var  activation_manager 
            = CdmwFT::Location::ActivationManager::_narrow(obj.in());
        
        activation_manager->register_activation_handler(group_ref, 
                                                        m_activation_handler.in()); 
    }
    catch (const FT::ObjectGroupNotFound&)
    {
        std::cerr << "ERROR: Impossible to register FTActivationHandler! Object Group not found!"
            << std::endl;
        throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL,
                                          CORBA::COMPLETED_NO);
    }
    catch (const CdmwFT::Location::HandlerAlreadyExist&)
    {
        std::cerr << "ERROR: Impossible to register FTActivationHandler! Handler already exists!"
            << std::endl;
        throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL,
                                          CORBA::COMPLETED_NO);
    }
    catch (const CORBA::ORB::InvalidName& )
    {
        std::cerr << "ERROR: Impossible to register FTActivationManager! Name is invalid!"
            << std::endl;
        throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL,
                                          CORBA::COMPLETED_NO);
    }
    catch (const CORBA::SystemException& e)
    {
        
        std::cerr << "ERROR: Impossible to register FTActivationHandler!"
            << std::endl;
        std::cerr << e << std::endl;
        throw;
    }
}


}; // End namespace Test

}; // End namespace Cdmw

