/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>

#include <testfttimeoutrequest/TestTimeoutRequest.hpp>
#include <testfttimeoutrequest/TestUtils.hpp>
#include <testfttimeoutrequest/HelloThread.hpp>
#include <testfttimeoutrequest/PullMonitorableThread.hpp>

#include <testfttimeoutrequest/TestHello.stub.hpp>
#include <FaultTolerance/ftwatchdog/RequestTimeoutLocalPullMonitorable_impl.hpp>


#include <fstream>


namespace Cdmw 
{

namespace FT 
{
CPPUNIT_TEST_SUITE_REGISTRATION( TestTimeoutRequest );

// constructor
TestTimeoutRequest::TestTimeoutRequest()
{    
   
    m_orb =  
       CORBA::ORB::_duplicate( 
             Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());
    m_rootPOA = 
       PortableServer::POA::_duplicate( 
             Cdmw::TestUtils::CORBATestManager::instance()->get_POA());

    CORBA::Object_var obj1 = 
       Cdmw::TestUtils::CORBATestManager::instance()->get_object("hello_object_group");
    m_hello_group = CdmwFT::Test::HelloInterface::_narrow(obj1.in());

    CORBA::Object_var obj2 = 
       Cdmw::TestUtils::CORBATestManager::instance()->get_object("comput_object_group");
    m_comput_group = CdmwFT::Test::ComputInterface::_narrow(obj2.in());

    CORBA::Object_var obj3 = 
       Cdmw::TestUtils::CORBATestManager::instance()->get_object("comput_object");
    m_comput_obj = CdmwFT::Test::ComputInterface::_narrow(obj3.in());
    
    CORBA::Object_var obj4 = Cdmw::TestUtils::CORBATestManager::instance()->get_object("replication_manager");
    m_replication_manager = CdmwFT::ReplicationManager::_narrow(obj4.in());

}

// destructor
TestTimeoutRequest::~TestTimeoutRequest()
{
}

// do_tests
void TestTimeoutRequest::do_tests()
{
    // set number of requested successfull tests
    //     set_nbOfRequestedTestOK(45);
    int timescale = Cdmw::TestUtils::get_timescale();

    try 
    {
        Cdmw::OsSupport::OS::unlink("hello.stt");
        Cdmw::OsSupport::OS::unlink("hello.act");
        Cdmw::OsSupport::OS::unlink("hello.dct");
        Cdmw::OsSupport::OS::unlink("comput.stt");
        Cdmw::OsSupport::OS::unlink("comput.act");
        Cdmw::OsSupport::OS::unlink("comput.dct");
    }
    catch(...)
    {
    }

    CdmwFT::Test::HelloInterface_var hello =
          TestUtils::get_hello_ref_from_file(m_orb.in(), 
                                             TestUtils::M_hello_files[0]);
    // Start the HelloThread to make concurrent request
    std::cout << "Start the HelloThread " << std::endl;
    Cdmw::Test::HelloThread hello_thread;
    hello_thread.start();

  
    try
    {
        TEST_INFO("Calling hello on HelloInterface on A...");
        hello->hello(m_comput_obj.in());
        TEST_INFO("hello on HelloInterface on A OK...");
        CPPUNIT_ASSERT(true);  
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);
    }
    
    try
    {
        TEST_INFO("Calling send_TRANSIENT_exception on HelloInterface on A...");
        hello->send_TRANSIENT_exception();
        CPPUNIT_ASSERT(false);  
    }
    catch (const CORBA::TRANSIENT& e)
    {
        TEST_INFO("Expected CORBA::TRANSIENT Exception raised");
        CPPUNIT_ASSERT(true);
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("Unexpected CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);  
    }


  
    std::cout << "Start the PullMonitorableThread " << std::endl;
    Cdmw::Test::PullMonitorableThread pm_thread;
    pm_thread.start();

    try
    {
        TEST_INFO("Calling hello on HelloInterface group (primary on A) and comput obj on C ...");
        std::string hello_group_str = m_orb->object_to_string(m_hello_group.in());
        m_hello_group->hello(m_comput_obj.in());
        CPPUNIT_ASSERT(true);  
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("Unexpected CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);  
    }

    /*TEST_INFO("try to bind an object");
    try
    {
        Cdmw::OrbSupport::OrbSupport::bind_object_to_corbaloc(m_orb.in(), "Hello", m_hello_group.in());
        CPPUNIT_ASSERT(true);
    }
    catch (const CORBA::Exception& e )
    {
            std::cerr << e._name() << std::endl;
            CPPUNIT_ASSERT(false);
    }


    TEST_INFO("try to get an object group with a corbaloc");
    try
    {
        
        CORBA::Object_var obj = m_orb->string_to_object("corbaloc::localhost:45678/Hello");
        std::cout<<m_orb->object_to_string(obj.in())<<std::endl;

        CdmwFT::Test::HelloInterface_var hello2 =  CdmwFT::Test::HelloInterface::_narrow(obj.in());
        
        if ( CORBA::is_nil( hello2.in() ) )
            std::cout<<"Hello is nil"<<std::endl;
        hello2->hello(m_comput_obj.in());
        std::cout<<m_orb->object_to_string(hello.in())<<std::endl;
        CPPUNIT_ASSERT(true);
    }
    catch (const CORBA::Exception& e )
    {
        std::cerr << e._name() << std::endl;
        CPPUNIT_ASSERT(false);
    }*/

    try
    {
        TEST_INFO("Calling hello on HelloInterface on A ...");
        hello->hello(m_comput_obj.in());
        TEST_INFO("hello on HelloInterface on A OK...");
        CPPUNIT_ASSERT(true);  
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);
    }

    try
     {
         TEST_INFO("Calling hello on primary HelloInterface (primary on A) and comput group (primary on C) ...");
         m_hello_group->hello(m_comput_group.in());
         CPPUNIT_ASSERT(true);  
     }
     catch (const CORBA::TRANSIENT& e)
     {
         TEST_INFO("Unexpected CORBA::TRANSIENT Exception raised " << e);
         CPPUNIT_ASSERT(false);
     }
     catch (const CORBA::SystemException& e)
     {
         TEST_INFO("Unexpected CORBA System Exception raised " << e);
         CPPUNIT_ASSERT(false);  
     }

     try
     {
         TEST_INFO("Calling hello on primary HelloInterface on A ...");
         m_hello_group->hello(m_comput_obj.in());
         CPPUNIT_ASSERT(true);
     }
     catch (const CORBA::TRANSIENT& e)
     {
         TEST_INFO("Unexpected CORBA::TRANSIENT Exception raised " << e);
         CPPUNIT_ASSERT(false);  
     }
     catch (const CORBA::SystemException& e)
     {
         TEST_INFO("Unexpected CORBA System Exception raised " << e);
         CPPUNIT_ASSERT(false);  
     }

    OsSupport::OS::sleep(timescale*1000);

    CdmwFT::Test::HelloInterface_var old_hello_group = m_hello_group;

    CORBA::ULong location_index = 1;
    try
    {
        TEST_INFO("Setting the primary...");
        ::FT::ObjectGroup_var object_group 
          = TestUtils::change_primary(m_orb.in(), m_hello_group.in(), 
                                        m_replication_manager.in(), 
                                        location_index);

        OsSupport::OS::sleep(timescale*1000);

        if (!CORBA::is_nil(object_group.in()))
        {
            m_hello_group = CdmwFT::Test::HelloInterface::_narrow(object_group.in());

            if (!CORBA::is_nil(m_hello_group.in()))
            {
                std::ifstream is("hello.act");
                if (is.good())
                {
                    CPPUNIT_ASSERT(false);  
                }
                else
                {
                    // no activation handler registered
                    CPPUNIT_ASSERT(true);  
                }
            }
            else
            {
                CPPUNIT_ASSERT(false);
            }
        }
        else 
        {
            CPPUNIT_ASSERT(false);
        }
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);
    }

    try
    {
        TEST_INFO("Calling init on HelloInterface...");

        for (int i=0; i<3; i++)
        {
            CdmwFT::Test::HelloInterface_var hello3 =
                TestUtils::get_hello_ref_from_file(m_orb.in(), 
                                                   TestUtils::M_hello_files[i]);
            
            hello3->init(m_hello_group.in());
            CPPUNIT_ASSERT(true);  
        }
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);
    }

    try
    {
        TEST_INFO("Calling hello on HelloInterface primary on B");
        m_hello_group->hello(m_comput_group.in());

        OsSupport::OS::sleep(timescale*1000);

        std::ifstream is("hello.stt");
        if (is.good())
        {
            std::string hello_location;
            is >> hello_location;
            // check host activated
            if (hello_location == TestUtils::M_locations[location_index])
            {
                CPPUNIT_ASSERT(true);  
            }
            else
            {
                std::cerr << "Invalid primary elected" << std::endl;
                CPPUNIT_ASSERT(false);
            }
            
            is.close();
        }
        else
        {
            std::cerr << "Error opening reference filename: hello.stt" << std::endl;
            CPPUNIT_ASSERT(false);
        }
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);
    }

     try
    {
        TEST_INFO("Calling hello on HelloInterface on old group ref: send_other is called...");
        old_hello_group->hello(m_comput_group.in());
        CPPUNIT_ASSERT(true);
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);
    }


    for (int i=0; i<7; i++)
    try
    {
        CdmwFT::Test::ComputInterface_var comput_obj =
          TestUtils::get_comput_ref_from_file(m_orb.in(), 
                                             TestUtils::M_comput_files[3]);

        TEST_INFO("Calling hello on HelloInterface (primary on B) and comput on D...");
        m_hello_group->hello(comput_obj.in());
        CPPUNIT_ASSERT(true);
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);
    }

    try
    {
        TEST_INFO("Calling hello on HelloInterface on A ...");
        hello->hello(m_comput_obj.in());
        CPPUNIT_ASSERT(true);
    }
    catch (const CORBA::TRANSIENT& e)
    {
        TEST_INFO("Unexpected CORBA::TRANSIENT Exception raised");
        CPPUNIT_ASSERT(false);  
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("Unexpected CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);  
    }

    try
    {
        TEST_INFO("Calling hello on HelloInterface on A ...");
        hello->hello(m_comput_obj.in());
        CPPUNIT_ASSERT(true);
    }
    catch (const CORBA::TRANSIENT& e)
    {
        TEST_INFO("Unexpected CORBA::TRANSIENT Exception raised");
        CPPUNIT_ASSERT(false);  
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("Unexpected CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);  
    }


    try
    {
        Cdmw::OsSupport::OS::unlink("hello.stt");
        Cdmw::OsSupport::OS::unlink("hello.act");
        Cdmw::OsSupport::OS::unlink("hello.dct");
        Cdmw::OsSupport::OS::unlink("comput.stt");
    }
    catch (...)
    {
    }
}


} // end namespace FT

} // end namespace Cdmw

