/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Standard Files 
#include <string>
#include <fstream>
#include <sstream>
// Cdmw Files
#include <Foundation/common/Options.hpp>
#include <Foundation/common/String.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <FaultTolerance/ftinit/FTServiceInit.hpp>
#include <FaultTolerance/ftcommon/FTConfiguration.hpp>
#include <testfttimeoutrequest/TestTimeoutRequest.hpp>
#include <testfttimeoutrequest/TestUtils.hpp>
#include <FaultTolerance/idllib/CdmwFTManager.stub.hpp>
#include <FaultTolerance/ftwatchdog/WatchdogServiceInit.hpp>
#include <FaultTolerance/ftcommon/FTSystemProperties.hpp>

#include <idllib/FT.stub.hpp>

using namespace Cdmw;
using namespace std;

namespace
{
    const std::string REPLICATION_MANAGER_PORT = "5040";
    const char * rep_id_hello = "IDL:thalesgroup.com/CdmwFT/Test/HelloInterface:1.0";
    const char * rep_id_comput = "IDL:thalesgroup.com/CdmwFT/Test/ComputInterface:1.0";
}
                               
int main( int argc, char* argv[] )
{
    CORBA::ORB_var orb = CORBA::ORB::_nil();
    try
    {
        // Initialise FT service
        Cdmw::FT::FTServiceInit::Init(argc, argv, true);
        Cdmw::FT::WatchdogServiceInit::Init(argc, argv);
        
        // Initialises the ORB
        OrbSupport::StrategyList strategyList;
        strategyList.add_OrbThreaded();
        strategyList.add_PoaThreadPerConnection();
        
        std::string str_opt(argv[0]);
        str_opt += " ";
        str_opt += Cdmw::Common::Options::LOCALISATION_SERVICE_OPTION;
        str_opt += "=";
        str_opt += "45678";
            
        char** argv__ = Cdmw::Common::String::to_char_array
        ( Cdmw::Common::String::to_strings(str_opt) );
        int argc__(2);
        //fixing memory leak : orb declared outside
        orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc__, argv__, strategyList);
        
        // TO be removed walk around!!!!!
        Cdmw::OrbSupport::OrbSupport::set_endpoint_selector_factory(
              Cdmw::FT::FTConfiguration::Get_FT_endpoint_selector_factory());
        
        // Get the root POA
        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj.in());

        PortableServer::POAManager_var poaMgr = rootPOA->the_POAManager();
        poaMgr->activate();

        ::Cdmw::FT:: FTConfiguration::Set_client_id(orb.in(), 
              Cdmw::FT::CLIENT_ID.c_str());
        
        std::cout << "Obtain the ORBs PolicyManager " << std::endl;
        obj = orb->resolve_initial_references("ORBPolicyManager");
        
        CORBA::PolicyManager_var policy_manager 
            = CORBA::PolicyManager::_narrow(obj.in());
        
        // Create and apply an ORB-wide Routed policy
        std::cout << "Create and apply an ORB-wide Routed policy " << std::endl;
        std::cout << "Creating REQUEST_DURATION_POLICY ..." << std::endl;
        CORBA::Any any;
        TimeBase::TimeT duration = 20000000;
        any <<= duration;
        CORBA::PolicyList policies(1);
        policies.length(1);
        policies[0] =
            orb->create_policy(::FT::REQUEST_DURATION_POLICY, any);
        
        policy_manager->set_policy_overrides(policies, CORBA::SET_OVERRIDE);
        
        using namespace Cdmw::OsSupport;
        OS::ProcessId replMngt_id
        = OS::create_process ("cdmw_ft_manager",
                              " --CdmwXMLFile=CdmwFaultToleranceManager_conf.xml");

        int timescale = Cdmw::TestUtils::get_timescale();
        OsSupport::OS::sleep(timescale*20000);
        

        // Get the fault Manager IOR
        std::cout << "Obtain the fault_manager " << std::endl;
        obj = orb->string_to_object("corbaloc::localhost:4555/fault_manager");

        CdmwFT::FTManager_var ftManager = CdmwFT::FTManager::_narrow(obj.in());
        obj = ftManager->get_replication_manager();
        CdmwFT::ReplicationManager_var replication_manager 
           = CdmwFT::ReplicationManager::_narrow(obj.in());

        std::string replication_ior_str = orb->object_to_string(replication_manager.in());

        // export the object reference to a file
        std::cout << " export the replication_manager object reference to a file"
                  << std::endl;
        std::ofstream os("replication_manager.ior");
        os << replication_ior_str;
        os.close();
        
        {
           // Create Server process
            std::cout << "Create test_server process" << std::endl;

            std::string request_timeout_property(
                    Cdmw::FT::WATCHDOG_TIMEOUT_PROPERTY_NAME);
            std::string args = " -D" 
                + request_timeout_property
                + "=500 --FilePath=hello1 --ComputFilePath=comput1 --HostName=A  -ORBInitRef ReplicationManager=" 
                + replication_ior_str;
            std::cout << "test_server "
                      << args      
                      << std::endl;
            OS::ProcessId test_server_id1 
               = OS::create_process ("test_server", args);
            
            args = " -D" 
                + request_timeout_property
                + "=500 --FilePath=hello2 --ComputFilePath=comput2 --HostName=B  -ORBInitRef ReplicationManager=" 
                + replication_ior_str;
            std::cout << "test_server "
                      << args      
                      << std::endl;
            OS::ProcessId test_server_id2 
               = OS::create_process ("test_server", args);
            
            args = " -D" 
                + request_timeout_property
                + "=500 --FilePath=hello3 --ComputFilePath=comput3 --HostName=C  -ORBInitRef ReplicationManager=" 
                + replication_ior_str;
            std::cout << "test_server "
                      << args      
                      << std::endl;
            OS::ProcessId test_server_id3 
               = OS::create_process ("test_server", args);

            args = " -D" 
                + request_timeout_property
                + "=500 --FilePath=hello4 --ComputFilePath=comput4 --HostName=D  -ORBInitRef ReplicationManager=" 
                + replication_ior_str;
            std::cout << "test_server "
                      << args      
                      << std::endl;
            OS::ProcessId test_server_id4 
               = OS::create_process ("test_server", args);
            
            int timescale = Cdmw::TestUtils::get_timescale();
            OsSupport::OS::sleep(timescale*25000);
            
            //
            // Create groups
            //
            std::cout << "Create groups" << std::endl;
            ::FT::ObjectGroup_var hello_group 
               = Cdmw::FT::TestUtils::create_group(orb.in(), 
                                                   replication_manager.in(), 
                                                   rep_id_hello);

            ::FT::ObjectGroup_var comput_group
               = Cdmw::FT::TestUtils::create_group(orb.in(), 
                                                   replication_manager.in(), 
                                                   rep_id_comput);

            std::cout << "Add members to groups" << std::endl;
            hello_group = 
               Cdmw::FT::TestUtils::add_members(orb.in(), 
                                               replication_manager.in(), 
                                               rep_id_hello, 
                                               hello_group.in());
            
            comput_group = 
               Cdmw::FT::TestUtils::add_members(orb.in(), 
                                               replication_manager.in(), 
                                               rep_id_comput, 
                                               comput_group.in());

            CORBA::Object_var comput_obj
               = Cdmw::FT::TestUtils::get_comput_ref_from_file(orb.in(), 
                                                               "comput3");

            ::FT::PullMonitorable_var pull_monitorable[Cdmw::FT::TestUtils::NB_LOCATION];

            for (int i=0; i < Cdmw::FT::TestUtils::NB_LOCATION; i++)
            {
               try 
               {
                   std::cout << "Getting PullMonitorable on host " 
                             << Cdmw::FT::TestUtils::M_locations[i] << std::endl;

                   pull_monitorable[i] 
                       = Cdmw::FT::TestUtils::get_pm_ref_from_file(orb.in(), 
                                                              Cdmw::FT::TestUtils::M_locations[i]);
                   if (CORBA::is_nil(pull_monitorable[i].in()))
                       CPPUNIT_ASSERT(false);
                   else
                       CPPUNIT_ASSERT(true);  
               }
               catch (const CORBA::SystemException& e)
               {
                   cerr <<__FILE__<<" "<<__LINE__<<" "<< e << endl;
                   throw;
               }
            }
            
            OsSupport::OS::sleep(timescale*10000);
            
            // Do test
            Cdmw::TestUtils::CORBATestManager::instance()->init(orb.in(),
                                                                rootPOA.in(),
                                                                argc,argv);
            Cdmw::TestUtils::CORBATestManager::instance()->add("hello_object_group", 
                                                               hello_group.in());
            Cdmw::TestUtils::CORBATestManager::instance()->add("comput_object_group", 
                                                               comput_group.in());
            Cdmw::TestUtils::CORBATestManager::instance()->add("comput_object", 
                                                               comput_obj.in());
            
            Cdmw::TestUtils::CORBATestManager::instance()->add("pm1_object", 
                                                               pull_monitorable[0].in());
            Cdmw::TestUtils::CORBATestManager::instance()->add("pm2_object", 
                                                               pull_monitorable[1].in());
            Cdmw::TestUtils::CORBATestManager::instance()->add("pm3_object", 
                                                               pull_monitorable[2].in());
            Cdmw::TestUtils::CORBATestManager::instance()->add("pm4_object", 
                                                               pull_monitorable[3].in());
 
            Cdmw::TestUtils::CORBATestManager::instance()->add("replication_manager", 
                                                               replication_manager.in());
            Cdmw::TestUtils::CORBATestManager::instance()->run_tests();

            OsSupport::OS::sleep(timescale*10000);
            // clean created file
            std::cout << "unlink hello1" << std::endl;
            OsSupport::OS::unlink("hello1");
            std::cout << "unlink hello2" << std::endl;
            OsSupport::OS::unlink("hello2");
            std::cout << "unlink hello3" << std::endl;
            OsSupport::OS::unlink("hello3");
            std::cout << "unlink hello4" << std::endl;
            OsSupport::OS::unlink("hello4");
            std::cout << "unlink comput1" << std::endl;
            OsSupport::OS::unlink("comput1");
            std::cout << "unlink comput2" << std::endl;
            OsSupport::OS::unlink("comput2");
            std::cout << "unlink comput3" << std::endl;
            OsSupport::OS::unlink("comput3");
            std::cout << "unlink comput4" << std::endl;
            OsSupport::OS::unlink("comput4");
            //OsSupport::OS::unlink("replication_manager.ior");
            std::cout << "unlink A_primary_backup_admin" << std::endl;
            OsSupport::OS::unlink("A_primary_backup_admin.ior");
            std::cout << "unlink B_primary_backup_admin" << std::endl;
            OsSupport::OS::unlink("B_primary_backup_admin.ior");
            std::cout << "unlink C_primary_backup_admin" << std::endl;
            OsSupport::OS::unlink("C_primary_backup_admin.ior");
            

            std::cout << "kill process test_server_id2" << std::endl;
            OS::kill_process(test_server_id2);
            std::cout << "kill process test_server_id4" << std::endl;
            OS::kill_process(test_server_id4);
            std::cout << "kill process test_server_id3" << std::endl;
            OS::kill_process(test_server_id3);
            std::cout << "kill process test_server_id1" << std::endl;
            OS::kill_process(test_server_id1);
            Cdmw::TestUtils::CORBATestManager::instance()->summary();

        }
        //fixing memory leak
        Cdmw::Common::String::delete_char_array (argv__);
        if (! CORBA::is_nil(orb.in()))
        {
            Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
            orb->destroy();
        rootPOA->destroy(1,1);
        }   
        OS::kill_process(replMngt_id );
    }
    catch( const CORBA::Exception &e )
    {
        cerr <<__FILE__<<" "<<__LINE__<<" "<< e._name() << endl;
        return EXIT_FAILURE;
    }
    catch( const Cdmw::Exception &e )
    {
        cerr <<__FILE__<<" "<<__LINE__<<" "<< e.what() << endl;
        return EXIT_FAILURE;
    }
    catch(...)
    {
        cerr << "Unexpected exception" << endl;
        return EXIT_FAILURE;
    }

    return EXIT_SUCCESS;
}

