/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <fstream>

// Cdmw Files
#include <Foundation/common/Options.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <FaultTolerance/ftinit/FTServiceInit.hpp>
#include <FaultTolerance/ftwatchdog/WatchdogServiceInit.hpp>
#include <FaultTolerance/ftwatchdog/RequestTimeoutLocalPullMonitorable_impl.hpp>


#include "ConfAndPlug/cdmwinit/CdmwInterface.hpp"

#include <testfttimeoutrequest/TestHello_impl.hpp>
#include <testfttimeoutrequest/TestComput_impl.hpp>


#include <idllib/CdmwFTActivation.skel.hpp>

#include <idllib/CdmwFTReplicationManager.stub.hpp>
#include <idllib/CdmwFTGroupRepository.stub.hpp>
#include <idllib/FT.stub.hpp>

using namespace std;

namespace 
{

    const std::string FILE_PATH_OPTION  = "--FilePath";
    const std::string FILE_PATH_OPTION2  = "--ComputFilePath";
    const std::string RM_IOR_OPTION  = "--ReplicationMngRef";
    const std::string HOST_NAME_OPTION  = "--HostName";

    const int POA_THREAD_POOL_SIZE = 5;   

    class HelloActivationHandler : virtual public CdmwFT::Location::ActivationHandler, 
                                   virtual public Cdmw::OrbSupport::RefCountLocalObject
    {
        std::string m_location;

    public:
        HelloActivationHandler(std::string location) : m_location(location) { };

        virtual void activate() throw(CORBA::SystemException)
        {
            std::ofstream os("hello.act");
            os << m_location;
            os.close();

            std::cout << m_location << " <####### HELLO ACTIVATION ######>" << std::endl;
        };


        virtual void deactivate() throw(CORBA::SystemException)
        {
            std::ofstream os("hello.dct");
            os << m_location;
            os.close();

            std::cout << m_location << " <####### HELLO DEACTIVATION ######>" << std::endl;
        };
    };

                                   
    class ComputActivationHandler : virtual public CdmwFT::Location::ActivationHandler, 
                                   virtual public Cdmw::OrbSupport::RefCountLocalObject
    {
        std::string m_location;

    public:
        ComputActivationHandler(std::string location) : m_location(location) { };

        virtual void activate() throw(CORBA::SystemException)
        {
            std::ofstream os("comput.act");
            os << m_location;
            os.close();

            std::cout << m_location << " <####### COMPUT ACTIVATION ######>" << std::endl;
        };


        virtual void deactivate() throw(CORBA::SystemException)
        {
            std::ofstream os("comput.dct");
            os << m_location;
            os.close();

            std::cout << m_location << " <####### COMPUT DEACTIVATION ######>" << std::endl;
        };
    };


} // end anonymous namespace


static void
usage(std::ostream & os, const std::string &program_name)
{
    os << "Usage:\n"
        << program_name << " options\n"
        << "\n"
        << "Options:\n"
        << "--help                                         Display this information.\n"
        << "\n"
//        << RM_IOR_OPTION    <<"=IOR  IOR of the ReplicationManager\n"
        << "-ORBInitRef ReplicationManager=<rep_corbaloc>  ReplicationManager corbaloc\n"
        << "[" << FILE_PATH_OPTION <<"]=path          path of the file where hello IOR should be stored.\n"
        << "[" << FILE_PATH_OPTION2 <<"]=path          path of the file where comput IOR should be stored.\n"
        << "[" << HOST_NAME_OPTION <<"]=hostname      hostname where server is launched.\n"
        << std:: endl;
}

                               
int main( int argc, char* argv[] )
{
    // help argument
    if (argc <=2)
    {
        usage (std::cout, argv[0]);
        return EXIT_SUCCESS;
    }

    if (argc >= 2)
    {
        if (strcmp(argv[1], "--help") == 0)
        {
            usage (std::cout, argv[0]);
            return EXIT_SUCCESS;
        }
    }

    // get host name
    std::string host_name = 
       Cdmw::OsSupport::OS::get_option_value (argc, argv, "--HostName");

    // get Name File to save hello interface
    std::string file_name = 
       Cdmw::OsSupport::OS::get_option_value (argc, argv, FILE_PATH_OPTION);
    
    if (file_name  == "no") 
    {
        file_name = "hello.ior";
    } 
    std::cout << host_name << " test_server : file_name = " << file_name << std::endl;

    // get Name File to save hello interface
    std::string file_name2 = 
       Cdmw::OsSupport::OS::get_option_value (argc, argv, FILE_PATH_OPTION2);
    
    if (file_name2  == "no") 
    {
        file_name2 = "comput.ior";
    } 
    std::cout << host_name << " test_server : file_name2 = " << file_name2 << std::endl;

    

    CORBA::ORB_var orb = CORBA::ORB::_nil();

    try
    {
        // Initialise FT service
        std::cout <<  host_name 
                  <<  "test_server: initialise FT service" << std::endl;
        Cdmw::FT::FTServiceInit::Init(argc, argv);
        Cdmw::FT::WatchdogServiceInit::Init(argc, argv);

        // Initialises the ORB
        Cdmw::OrbSupport::StrategyList strategyList;
        strategyList.add_OrbThreaded();
        strategyList.add_PoaThreadPool(POA_THREAD_POOL_SIZE);

        std::cout << host_name 
                  << " test_server: initialises the ORB" 
                  << std::endl;
        orb= Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, strategyList);
  
        // Get the root POA
        std::cout << host_name << " test_server: get the root POA" << std::endl;
        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj.in());

        std::cout << host_name << " test_server: activate the root POA" << std::endl;
        PortableServer::POAManager_var poaMgr = rootPOA->the_POAManager();
        poaMgr->activate();

        // init Cdmw
        std::cout << host_name << " test_server: init Cdmw"<<std::endl;
        Cdmw::CdmwInit::CDMW_init(orb.in(), argc, argv);


        std::cout << host_name << " test_server: create HelloInterface_impl"<<std::endl;
        // Create activation handler, it will be register in init operation of Hello
        std::cout<< host_name << " test_server: create activation handler and register it"
                 <<std::endl;
        CdmwFT::Location::ActivationHandler_var activation_handler 
            = new HelloActivationHandler(host_name);

        Cdmw::Test::HelloInterface_impl * hello_interface 
            = new Cdmw::Test::HelloInterface_impl(orb.in(), host_name, activation_handler.in());
        CdmwFT::Test::HelloInterface_var hello_interface_ref 
            =  hello_interface->_this();

        std::cout << host_name << " test_server: create ComputInterface_impl"<<std::endl;
        // Create activation handler, it will be register in init operation of Hello
        std::cout<< host_name << " test_server: create activation handler and register it"
                 <<std::endl;
        CdmwFT::Location::ActivationHandler_var comput_activation_handler 
            = new ComputActivationHandler(host_name);

        Cdmw::Test::ComputInterface_impl * comput_interface 
            = new Cdmw::Test::ComputInterface_impl(orb.in(), host_name, activation_handler.in());
        CdmwFT::Test::ComputInterface_var comput_interface_ref 
            =  comput_interface->_this();

        
        // export the object reference to a file
        std::cout << host_name << " test_server: export the hello object reference to a file"
                  <<std::endl;
        CORBA::String_var ref_string1 
            = orb->object_to_string(hello_interface_ref.in());
        std::ofstream os1(file_name.c_str());
        os1 << ref_string1.in();
        os1.close();

        // export the object reference to a file
        std::cout << host_name << " test_server: export the comput object reference to a file"
                  <<std::endl;
        CORBA::String_var ref_string2 
            = orb->object_to_string(comput_interface_ref.in());
        std::ofstream os2(file_name2.c_str());
        os2 << ref_string2.in();
        os2.close();


        int timescale = Cdmw::TestUtils::get_timescale();
        Cdmw::OsSupport::OS::sleep(timescale*20000);

        // Setup signal handling for ORB to be shut down
        Cdmw::OrbSupport::OrbSupport::install_orb_shutdown_signal_handlers(orb.in());

        std::cout<< host_name << " test_server: orb run"<<std::endl;
        orb->run();

        Cdmw::OrbSupport::OrbSupport::uninstall_orb_shutdown_signal_handlers(orb.in());

    }
    catch (const CORBA::SystemException &e )
    {
        cerr <<__FILE__<<" "<<__LINE__<<" "<< e << endl;
        return EXIT_FAILURE;
    }
    catch( const CORBA::Exception &e )
    {
        cerr <<__FILE__<<" "<<__LINE__<<" "<< e._name() << endl;
        return EXIT_FAILURE;
    }
    catch( const Cdmw::Exception &e )
    {
        cerr <<__FILE__<<" "<<__LINE__<<" "<< e.what() << endl;
        return EXIT_FAILURE;
    }
    catch(...)
    {
        cerr << "Uncaught exception" << endl;
        return EXIT_FAILURE;
    }

    if (!CORBA::is_nil(orb.in()))
    {
         try
         {
             Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
             orb->destroy();
         }
         catch(const CORBA::Exception& ex)
         {
             std::cerr << ex << std::endl;
             std::cerr << " **** TEST FAILED AT LINE " << __LINE__ 
                       << " IN FILE: " << __FILE__ << std::endl;
             return EXIT_FAILURE;
         }
    }        
    
    return EXIT_SUCCESS;
}

