/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/testutils/CORBATestManager.hpp"

#include <testtimeoutrequest/TestHello.stub.hpp>
#include <testtimeoutrequest/HelloThread.hpp>
#include <testtimeoutrequest/TestUtils.hpp>


namespace Cdmw
{

namespace Test
{
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(HelloTest, "HelloTest");

class HelloTest : public CppUnit::TestFixture
{
    CORBA::ORB_var                    m_orb;
    CdmwFT::Test::HelloInterface_var  m_hello_obj;
    CdmwFT::Test::ComputInterface_var m_comput_obj;
    
 public:
    HelloTest()
    {
        m_orb = CORBA::ORB::_duplicate( 
             Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());

        CORBA::Object_var obj = Cdmw::TestUtils::CORBATestManager::instance(
                )->get_object("comput_object");  
        m_comput_obj = CdmwFT::Test::ComputInterface::_narrow(obj.in());
    }

    void 
    do_tests()
    {
        for (int i=0; i < 4 ; i++)
        {
            try
            {
                TEST_INFO("Calling hello on HelloInterface in Thread...");
                m_hello_obj =
                    Cdmw::FT::TestUtils::get_hello_ref_from_file(m_orb.in(), 
                                        Cdmw::FT::TestUtils::M_hello_files[0]);
                m_hello_obj->hello(m_comput_obj.in());
                TEST_INFO("hello on HelloInterface in Thread OK...");
                CPPUNIT_ASSERT(true);  
            }
            catch (const CORBA::SystemException& e)
            {
                TEST_INFO("CORBA System Exception raised " << e);
                CPPUNIT_ASSERT(false);
            }
        }
        

        TEST_INFO("********************************************");
        TEST_INFO("            HelloTest stop");
        TEST_INFO("********************************************");

    } // End of do_test()
    
    CPPUNIT_TEST_SUITE( HelloTest );
    CPPUNIT_TEST( do_tests );
    CPPUNIT_TEST_SUITE_END();
};

    
HelloThread::HelloThread()
{    
}

HelloThread::~HelloThread() 
throw() 
{
} 

void
HelloThread::run() 
    throw() 
{
   Cdmw::TestUtils::ThreadedTestRunner runner(Cdmw::TestUtils::CORBATestManager::instance()->get_controller());
   runner.addTest( CppUnit::TestFactoryRegistry::getRegistry("HelloTest").makeTest() );
   runner.start();
   runner.join();
}

    
} // end of namespace Test

} // end of namespace Cdmw


