/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/testutils/CORBATestManager.hpp>
#include <Foundation/ossupport/OS.hpp>

#include <testtimeoutrequest/PullMonitorableThread.hpp>
#include <testtimeoutrequest/TestUtils.hpp>


namespace Cdmw
{

namespace Test
{
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(PullMonitorableTest, "PullMonitorableTest");

class PullMonitorableTest : public CppUnit::TestFixture
{
    CORBA::ORB_var             m_orb;
    ::FT::PullMonitorable_var  pull_monitorable[Cdmw::FT::TestUtils::NB_LOCATION];

 public:
    PullMonitorableTest()
    {
        m_orb = CORBA::ORB::_duplicate( 
             Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());

        CORBA::Object_var obj1 = Cdmw::TestUtils::CORBATestManager::instance(
              )->get_object("pm1_object");
        pull_monitorable[0] = ::FT::PullMonitorable::_narrow(obj1.in());

        CORBA::Object_var obj2 = Cdmw::TestUtils::CORBATestManager::instance(
              )->get_object("pm2_object");
        pull_monitorable[1] = ::FT::PullMonitorable::_narrow(obj2.in());

        CORBA::Object_var obj3 = Cdmw::TestUtils::CORBATestManager::instance(
              )->get_object("pm3_object");
        pull_monitorable[2] = ::FT::PullMonitorable::_narrow(obj3.in());

        CORBA::Object_var obj4 = Cdmw::TestUtils::CORBATestManager::instance(
              )->get_object("pm4_object");
        pull_monitorable[3] = ::FT::PullMonitorable::_narrow(obj4.in());
    }

    void 
    do_tests()
    {
        for (int i=0; i < Cdmw::FT::TestUtils::NB_LOCATION; i++)
        {
            try 
            {
                TEST_INFO("Calling is_alive on PullMonitorable on host " 
                          <<  Cdmw::FT::TestUtils::M_locations[i]);
                TEST_INFO("All processes shall be alive"); 
                CORBA::Boolean result = pull_monitorable[i]->is_alive();
                if (result)
                {
                    TEST_INFO("All processes are alive"); 
                    CPPUNIT_ASSERT(true);  
                }
                else
                    CPPUNIT_ASSERT(false);
            }
            catch (const CORBA::SystemException& e)
            {
                TEST_INFO("CORBA System Exception raised " << e);
                CPPUNIT_ASSERT(false);
            }
        }

        OsSupport::OS::sleep(1200);

        for (int i=0; i < Cdmw::FT::TestUtils::NB_LOCATION; i++)
        {
            try 
            {
                TEST_INFO("Calling is_alive on PullMonitorable on host " 
                          <<  Cdmw::FT::TestUtils::M_locations[i]);
                TEST_INFO("Process C shall be on timeout"); 
                
                CORBA::Boolean result = pull_monitorable[i]->is_alive();
                if (i == 2)
                {
                    if (result)
                    {
                        // retry until is_alive return false
                        for (int j=0; j < 30; j++)
                        {
                            result = pull_monitorable[i]->is_alive();
                            if (!result)
                                break;
                            OsSupport::OS::sleep(100);
                        }
                        CPPUNIT_ASSERT(!result);
                    }
                    else
                        CPPUNIT_ASSERT(true);  
                }
                else
                {
                    if (result)
                        CPPUNIT_ASSERT(true);  
                    else
                        CPPUNIT_ASSERT(false);
                }
            }
            catch (const CORBA::SystemException& e)
            {
                TEST_INFO("CORBA System Exception raised " << e);
                CPPUNIT_ASSERT(false);
            }
        }

        int timescale = Cdmw::TestUtils::get_timescale();
        OsSupport::OS::sleep(timescale*1000);

        for (int i=0; i < Cdmw::FT::TestUtils::NB_LOCATION; i++)
        {
            try 
            {
                TEST_INFO("Calling is_alive on PullMonitorable on host " 
                          <<  Cdmw::FT::TestUtils::M_locations[i]);
                TEST_INFO("Process C shall be on timeout"); 
                CORBA::Boolean result = pull_monitorable[i]->is_alive();
                if (i == 2)
                {
                    if (result)
                    {
                        // retry until is_alive return false
                        for (int j=0; j < 30; j++)
                        {
                            result = pull_monitorable[i]->is_alive();
                            if (!result)
                                break;
                            OsSupport::OS::sleep(100);
                        }
                        CPPUNIT_ASSERT(!result);
                    }
                    else
                        CPPUNIT_ASSERT(true);  
                }
                else
                {
                    if (result)
                        CPPUNIT_ASSERT(true);  
                    else
                    {
                        // retry until is_alive return true
                        for (int j=0; j < 30; j++)
                        {
                            result = pull_monitorable[i]->is_alive();
                            if (result)
                                break;
                            OsSupport::OS::sleep(100);
                        }
                        CPPUNIT_ASSERT(result);
                    }
                }
            }
            catch (const CORBA::SystemException& e)
            {
                TEST_INFO("CORBA System Exception raised " << e);
                CPPUNIT_ASSERT(false);
            }
        }
        
        OsSupport::OS::sleep(timescale*5000);

        for (int i=0; i < Cdmw::FT::TestUtils::NB_LOCATION; i++)
        {
            try 
            {
                TEST_INFO("Calling is_alive on PullMonitorable on host " 
                          <<  Cdmw::FT::TestUtils::M_locations[i]);
                TEST_INFO("Process D shall be on timeout"); 
                CORBA::Boolean result = pull_monitorable[i]->is_alive();
                if (i == 3)
                {
                    if (result)
                    {
                        // retry until is_alive return false
                        for (int j=0; j < 30; j++)
                        {
                            result = pull_monitorable[i]->is_alive();
                            if (!result)
                                break;
                            OsSupport::OS::sleep(100);
                        }
                        CPPUNIT_ASSERT(!result);
                    }
                    else
                        CPPUNIT_ASSERT(true);  
                }
                else
                {
                    if (result)
                        CPPUNIT_ASSERT(true);  
                    else
                    {
                        // retry until is_alive return true
                        for (int j=0; j < 30; j++)
                        {
                            result = pull_monitorable[i]->is_alive();
                            if (result)
                                break;
                            OsSupport::OS::sleep(100);
                        }
                        CPPUNIT_ASSERT(result);
                    }
                }
            }
            catch (const CORBA::SystemException& e)
            {
                TEST_INFO("CORBA System Exception raised " << e);
                CPPUNIT_ASSERT(false);
            }
        }

        OsSupport::OS::sleep(timescale*1000);

        for (int i=0; i < Cdmw::FT::TestUtils::NB_LOCATION; i++)
        {
            try 
            {
                TEST_INFO("Calling is_alive on PullMonitorable on host " 
                          <<  Cdmw::FT::TestUtils::M_locations[i]);
                TEST_INFO("Process C shall be on timeout"); 
                CORBA::Boolean result = pull_monitorable[i]->is_alive();
                if (i == 2)
                {
                    if (result)
                    {
                        // retry until is_alive return false
                        for (int j=0; j < 30; j++)
                        {
                            result = pull_monitorable[i]->is_alive();
                            if (!result)
                                break;
                            OsSupport::OS::sleep(100);
                        }
                        CPPUNIT_ASSERT(!result);
                    }
                    else
                        CPPUNIT_ASSERT(true);  
                }
                else
                {
                    if (result)
                        CPPUNIT_ASSERT(true);  
                    else
                        CPPUNIT_ASSERT(false);
                }
            }
            catch (const CORBA::SystemException& e)
            {
                TEST_INFO("CORBA System Exception raised " << e);
                CPPUNIT_ASSERT(false);
            }
        }

        TEST_INFO("********************************************");
        TEST_INFO("            PullMonitorableTest stop");
        TEST_INFO("********************************************");

    } // End of do_test()

    
    CPPUNIT_TEST_SUITE( PullMonitorableTest );
    CPPUNIT_TEST( do_tests );
    CPPUNIT_TEST_SUITE_END();
};

        
PullMonitorableThread::PullMonitorableThread()
{
}

PullMonitorableThread::~PullMonitorableThread()
throw() 
{
}

void
PullMonitorableThread::run() 
    throw() 
{
   Cdmw::TestUtils::ThreadedTestRunner runner(Cdmw::TestUtils::CORBATestManager::instance()->get_controller());
   runner.addTest( CppUnit::TestFactoryRegistry::getRegistry("PullMonitorableTest").makeTest() );
   runner.start();
   runner.join();
}

    
} // end of namespace Test

} // end of namespace Cdmw



