/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>

#include <testtimeoutrequest/TestTimeoutRequest.hpp>
#include <testtimeoutrequest/TestUtils.hpp>
#include <testtimeoutrequest/HelloThread.hpp>
#include <testtimeoutrequest/PullMonitorableThread.hpp>

#include <testtimeoutrequest/TestHello.stub.hpp>
#include <FaultTolerance/ftwatchdog/RequestTimeoutLocalPullMonitorable_impl.hpp>


#include <fstream>


namespace Cdmw 
{

namespace FT 
{
CPPUNIT_TEST_SUITE_REGISTRATION( TestTimeoutRequest );

// constructor
TestTimeoutRequest::TestTimeoutRequest()
{    
   
    m_orb =  
       CORBA::ORB::_duplicate( 
             Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());
    m_rootPOA = 
       PortableServer::POA::_duplicate( 
             Cdmw::TestUtils::CORBATestManager::instance()->get_POA());

    CORBA::Object_var obj = 
       Cdmw::TestUtils::CORBATestManager::instance()->get_object("comput_object");
    m_comput_C = CdmwFT::Test::ComputInterface::_narrow(obj.in());

}

// destructor
TestTimeoutRequest::~TestTimeoutRequest()
{
}

// do_tests
void TestTimeoutRequest::do_tests()
{
    // set number of requested successfull tests
    //     set_nbOfRequestedTestOK(45);
    int timescale = Cdmw::TestUtils::get_timescale();

    try 
    {
        Cdmw::OsSupport::OS::unlink("hello.stt");
        Cdmw::OsSupport::OS::unlink("comput.stt");
    }
    catch(...)
    {
    }

    CdmwFT::Test::HelloInterface_var hello_A =
          TestUtils::get_hello_ref_from_file(m_orb.in(), 
                                             TestUtils::M_hello_files[0]);
    // Start the HelloThread to make concurrent request
    std::cout << "Start the HelloThread " << std::endl;
    Cdmw::Test::HelloThread hello_thread;
    hello_thread.start();

  
    try
    {
        TEST_INFO("Calling hello on HelloInterface on A...");
        hello_A->hello(m_comput_C.in());
        TEST_INFO("hello on HelloInterface on A OK...");
        CPPUNIT_ASSERT(true);  
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);
    }
    
    try
    {
        TEST_INFO("Calling send_TRANSIENT_exception on HelloInterface on A...");
        hello_A->send_TRANSIENT_exception();
        CPPUNIT_ASSERT(false);  
    }
    catch (const CORBA::TRANSIENT& e)
    {
        TEST_INFO("Expected CORBA::TRANSIENT Exception raised");
        CPPUNIT_ASSERT(true);
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("Unexpected CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);  
    }


  
    std::cout << "Start the PullMonitorableThread " << std::endl;
    Cdmw::Test::PullMonitorableThread pm_thread;
    pm_thread.start();

    try
    {
        TEST_INFO("Calling hello on HelloInterface on A and comput obj on C ...");
        hello_A->hello(m_comput_C.in());
        CPPUNIT_ASSERT(true);  
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("Unexpected CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);  
    }

    try
    {
        TEST_INFO("Calling hello on HelloInterface on A ...");
        hello_A->hello(m_comput_C.in());
        TEST_INFO("hello on HelloInterface on A OK...");
        CPPUNIT_ASSERT(true);  
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);
    }

    try
     {
         TEST_INFO("Calling hello on HelloInterface on A and comput on C) ...");
         hello_A->hello(m_comput_C.in());
         CPPUNIT_ASSERT(true);  
     }
     catch (const CORBA::TRANSIENT& e)
     {
         TEST_INFO("Unexpected CORBA::TRANSIENT Exception raised " << e);
         CPPUNIT_ASSERT(false);
     }
     catch (const CORBA::SystemException& e)
     {
         TEST_INFO("Unexpected CORBA System Exception raised " << e);
         CPPUNIT_ASSERT(false);  
     }

     try
     {
         TEST_INFO("Calling hello on HelloInterface on A ...");
         hello_A->hello(m_comput_C.in());
         CPPUNIT_ASSERT(true);
     }
     catch (const CORBA::TRANSIENT& e)
     {
         TEST_INFO("Unexpected CORBA::TRANSIENT Exception raised " << e);
         CPPUNIT_ASSERT(false);  
     }
     catch (const CORBA::SystemException& e)
     {
         TEST_INFO("Unexpected CORBA System Exception raised " << e);
         CPPUNIT_ASSERT(false);  
     }

    OsSupport::OS::sleep(timescale*1000);

    
    CdmwFT::Test::HelloInterface_var hello_B =
          TestUtils::get_hello_ref_from_file(m_orb.in(), 
                                             TestUtils::M_hello_files[1]);
    CdmwFT::Test::ComputInterface_var comput_D =
          TestUtils::get_comput_ref_from_file(m_orb.in(), 
                                             TestUtils::M_comput_files[3]);

    try
    {
        TEST_INFO("Calling hello on HelloInterface on B");
        hello_B->hello(comput_D.in());

        OsSupport::OS::sleep(timescale*1000);
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);
    }


    for (int i=0; i<8; i++)
    try
    {
        
        TEST_INFO("Calling hello on HelloInterface on B and comput on D...");
        hello_B->hello(comput_D.in());
        CPPUNIT_ASSERT(true);
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);
    }

    try
    {
        TEST_INFO("Calling hello on HelloInterface on A ...");
        hello_A->hello(m_comput_C.in());
        CPPUNIT_ASSERT(true);
    }
    catch (const CORBA::TRANSIENT& e)
    {
        TEST_INFO("Unexpected CORBA::TRANSIENT Exception raised");
        CPPUNIT_ASSERT(false);  
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("Unexpected CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);  
    }

    try
    {
        TEST_INFO("Calling hello on HelloInterface on A ...");
        hello_A->hello(m_comput_C.in());
        CPPUNIT_ASSERT(true);
    }
    catch (const CORBA::TRANSIENT& e)
    {
        TEST_INFO("Unexpected CORBA::TRANSIENT Exception raised");
        CPPUNIT_ASSERT(false);  
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("Unexpected CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);  
    }

    try
    {
        Cdmw::OsSupport::OS::unlink("hello.stt");
        Cdmw::OsSupport::OS::unlink("comput.stt");
    }
    catch (...)
    {
    }
}


} // end namespace FT

} // end namespace Cdmw

