/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Standard Files 
#include <string>
#include <iostream>
#include <fstream>
#include <sstream>

// Cdmw Files
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>

#include <testtimeoutrequest/TestUtils.hpp>

#include <idllib/FT.stub.hpp>

namespace
{

} // end anonymous namespace

namespace Cdmw 
{

namespace FT 
{
    const char* TestUtils::M_locations[NB_LOCATION] = {"A", "B", "C", "D"};
    const char* TestUtils::M_hello_files[NB_LOCATION] = {"hello1", "hello2", "hello3", "hello4"};
    const char* TestUtils::M_comput_files[NB_LOCATION] = {"comput1", "comput2", "comput3", "comput4"};
    

CdmwFT::Test::HelloInterface_ptr
TestUtils::get_hello_ref_from_file(CORBA::ORB_ptr orb, std::string file_name)
throw (CORBA::SystemException)
{
    CORBA::Object_var objref;
    CdmwFT::Test::HelloInterface_var hello_ref;

    // import the object reference from the file
    std::ifstream is(file_name.c_str());
    if (is.good())
    {
        std::string objref_str;
        is >> objref_str;
        try
        {
            objref = orb->string_to_object(objref_str.c_str());
            hello_ref = CdmwFT::Test::HelloInterface::_narrow(objref.in());
            
        }
        catch( CORBA::SystemException& e )
        {
            std::cerr << " CORBA System Exception raised: " << e << std::endl;
            std::cerr << " **** TEST FAILED AT LINE " << __LINE__ 
                << " IN FILE: " << __FILE__ << std::endl; 
            throw;
        }
        catch (...)
        {
            throw CORBA::INTERNAL(OrbSupport::INTERNALFaultToleranceError, CORBA::COMPLETED_NO);
        }

        is.close();
    }
    else
    {
        std::cerr << "Error opening reference filename: " << file_name << std::endl;
        throw CORBA::INTERNAL(OrbSupport::INTERNALFaultToleranceError, CORBA::COMPLETED_NO);
    }

    return hello_ref._retn();
}

/**********************************************************************************/

::FT::PullMonitorable_ptr
TestUtils::get_pm_ref_from_file(CORBA::ORB_ptr orb, std::string host_name)
throw (CORBA::SystemException)
{
    CORBA::Object_var objref;
    ::FT::PullMonitorable_var pm_ref;

    // import the object reference from the file
    std::string file_name = host_name + "_pullMonitorable.ior";
    std::ifstream is(file_name.c_str());
    if (is.good())
    {
        std::string objref_str;
        is >> objref_str;
        try
        {
            objref = orb->string_to_object(objref_str.c_str());
            pm_ref = ::FT::PullMonitorable::_narrow(objref.in());
        }
        catch( CORBA::SystemException& e )
        {
            std::cerr << " CORBA System Exception raised: " << e << std::endl;
            std::cerr << " **** TEST FAILED AT LINE " << __LINE__ 
                << " IN FILE: " << __FILE__ << std::endl; 
            throw;
        }
        catch (...)
        {
            throw CORBA::INTERNAL(OrbSupport::INTERNALFaultToleranceError, 
                                  CORBA::COMPLETED_NO);
        }

        is.close();
    }
    else
    {
        std::cerr << "Error opening reference filename: " 
                  << file_name << std::endl;
        throw CORBA::INTERNAL(OrbSupport::INTERNALFaultToleranceError, 
                              CORBA::COMPLETED_NO);
    }

    return pm_ref._retn();
}

/**********************************************************************************/


CdmwFT::Test::ComputInterface_ptr
TestUtils::get_comput_ref_from_file(CORBA::ORB_ptr orb, std::string file_name)
throw (CORBA::SystemException)
{
    CORBA::Object_var objref;
    CdmwFT::Test::ComputInterface_var comput_ref;

    // import the object reference from the file
    std::cout << " get_comput_ref_from_file file_name: " << file_name << std::endl;
    std::ifstream is(file_name.c_str());
    if (is.good())
    {
        std::string objref_str;
        is >> objref_str;
        try
        {
            std::cout << " get_comput_ref_from_file string_to_object " << std::endl;
            objref = orb->string_to_object(objref_str.c_str());
            std::cout << " get_comput_ref_from_file _narrow " << std::endl;
            comput_ref = CdmwFT::Test::ComputInterface::_narrow(objref.in());
        }
        catch( CORBA::SystemException& e )
        {
            std::cerr << " CORBA System Exception raised: " << e << std::endl;
            std::cerr << " **** TEST FAILED AT LINE " << __LINE__ 
                << " IN FILE: " << __FILE__ << std::endl; 
            throw;
        }
        catch (...)
        {
            throw CORBA::INTERNAL(OrbSupport::INTERNALFaultToleranceError, CORBA::COMPLETED_NO);
        }

        is.close();
    }
    else
    {
        std::cerr << "Error opening reference filename: " << file_name << std::endl;
        throw CORBA::INTERNAL(OrbSupport::INTERNALFaultToleranceError, CORBA::COMPLETED_NO);
    }

    return comput_ref._retn();
}

} // end namespace FT

} // end namespace Cdmw

