/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Standard Files 
#include <string>
#include <fstream>
#include <sstream>
// Cdmw Files
#include <Foundation/common/Options.hpp>
#include <Foundation/common/String.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <testtimeoutrequest/TestTimeoutRequest.hpp>
#include <testtimeoutrequest/TestUtils.hpp>
#include <FaultTolerance/ftwatchdog/WatchdogServiceInit.hpp>
#include <FaultTolerance/ftcommon/FTSystemProperties.hpp>


using namespace Cdmw;
using namespace std;

                               
int main( int argc, char* argv[] )
{
    try
    {
        // Initialise watchdog service
        Cdmw::FT::WatchdogServiceInit::Init(argc, argv);
        
        // Initialises the ORB
        OrbSupport::StrategyList strategyList;
        strategyList.add_OrbThreaded();
        strategyList.add_PoaThreadPerConnection();
        
        std::string str_opt(argv[0]);
        str_opt += " ";
        str_opt += Cdmw::Common::Options::LOCALISATION_SERVICE_OPTION;
        str_opt += "=";
        str_opt += "45678";
            
        char** argv__ = Cdmw::Common::String::to_char_array
        ( Cdmw::Common::String::to_strings(str_opt) );
        int argc__(2);
        
        CORBA::ORB_var orb 
           = Cdmw::OrbSupport::OrbSupport::ORB_init(argc__, argv__, strategyList);
    //fixing memory leak
        Cdmw::Common::String::delete_char_array(argv__);
        // Get the root POA
        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj.in());

        PortableServer::POAManager_var poaMgr = rootPOA->the_POAManager();
        poaMgr->activate();

        int timescale = Cdmw::TestUtils::get_timescale();
        OsSupport::OS::sleep(timescale*1000);
        

        {
           // Create Server process
            std::cout << "Create test_server process" << std::endl;

            std::string request_timeout_property(
                    Cdmw::FT::WATCHDOG_TIMEOUT_PROPERTY_NAME);
            std::string args = " -D" 
                + request_timeout_property
                + "=500 --FilePath=hello1 --ComputFilePath=comput1 --HostName=A";
            std::cout << "test_server "
                      << args      
                      << std::endl;
            OsSupport::OS::ProcessId test_server_id1 
               = OsSupport::OS::create_process ("test_server", args);
            
            args = " -D" 
                + request_timeout_property
                + "=500 --FilePath=hello2 --ComputFilePath=comput2 --HostName=B";
            std::cout << "test_server "
                      << args      
                      << std::endl;
            OsSupport::OS::ProcessId test_server_id2 
               = OsSupport::OS::create_process ("test_server", args);
            
            args = " -D" 
                + request_timeout_property
                + "=500 --FilePath=hello3 --ComputFilePath=comput3 --HostName=C";
            std::cout << "test_server "
                      << args      
                      << std::endl;
            OsSupport::OS::ProcessId test_server_id3 
               = OsSupport::OS::create_process ("test_server", args);

            args = " -D" 
                + request_timeout_property
                + "=500 --FilePath=hello4 --ComputFilePath=comput4 --HostName=D";
            std::cout << "test_server "
                      << args      
                      << std::endl;
            OsSupport::OS::ProcessId test_server_id4 
               = OsSupport::OS::create_process ("test_server", args);
            
            int timescale = Cdmw::TestUtils::get_timescale();
            OsSupport::OS::sleep(timescale*25000);
            
            CORBA::Object_var comput_obj
               = Cdmw::FT::TestUtils::get_comput_ref_from_file(orb.in(), 
                                                               "comput3");

            ::FT::PullMonitorable_var pull_monitorable[Cdmw::FT::TestUtils::NB_LOCATION];

            for (int i=0; i < Cdmw::FT::TestUtils::NB_LOCATION; i++)
            {
               try 
               {
                   std::cout << "Getting PullMonitorable on host " 
                             << Cdmw::FT::TestUtils::M_locations[i] << std::endl;

                   pull_monitorable[i] 
                       = Cdmw::FT::TestUtils::get_pm_ref_from_file(orb.in(), 
                                                              Cdmw::FT::TestUtils::M_locations[i]);
                   if (CORBA::is_nil(pull_monitorable[i].in()))
                       CPPUNIT_ASSERT(false);
                   else
                       CPPUNIT_ASSERT(true);  
               }
               catch (const CORBA::SystemException& e)
               {
                   cerr <<__FILE__<<" "<<__LINE__<<" "<< e << endl;
                   throw;
               }
            }
            
            OsSupport::OS::sleep(timescale*10000);
            
            // Do test
            std::cout << "CORBATestManager init..." << std::endl;
            Cdmw::TestUtils::CORBATestManager::instance()->init(orb.in(),
                                                                rootPOA.in(),
                                                                argc,argv);
            std::cout << "CORBATestManager add comput_object ..." << std::endl;
            Cdmw::TestUtils::CORBATestManager::instance()->add("comput_object", 
                                                               comput_obj.in());
            
            std::cout << "CORBATestManager add pm1_object ..." << std::endl;
            Cdmw::TestUtils::CORBATestManager::instance()->add("pm1_object", 
                                                               pull_monitorable[0].in());
            std::cout << "CORBATestManager add pm2_object ..." << std::endl;
            Cdmw::TestUtils::CORBATestManager::instance()->add("pm2_object", 
                                                               pull_monitorable[1].in());
            std::cout << "CORBATestManager add pm3_object ..." << std::endl;
            Cdmw::TestUtils::CORBATestManager::instance()->add("pm3_object", 
                                                               pull_monitorable[2].in());
            std::cout << "CORBATestManager add pm4_object ..." << std::endl;
            Cdmw::TestUtils::CORBATestManager::instance()->add("pm4_object", 
                                                               pull_monitorable[3].in());
 
            std::cout << "CORBATestManager run_tests ..." << std::endl;
            Cdmw::TestUtils::CORBATestManager::instance()->run_tests();

            OsSupport::OS::sleep(timescale*10000);
            // clean created file
            std::cout << "unlink hello1" << std::endl;
            OsSupport::OS::unlink("hello1");
            std::cout << "unlink hello2" << std::endl;
            OsSupport::OS::unlink("hello2");
            std::cout << "unlink hello3" << std::endl;
            OsSupport::OS::unlink("hello3");
            std::cout << "unlink hello4" << std::endl;
            OsSupport::OS::unlink("hello4");
            std::cout << "unlink comput1" << std::endl;
            OsSupport::OS::unlink("comput1");
            std::cout << "unlink comput2" << std::endl;
            OsSupport::OS::unlink("comput2");
            std::cout << "unlink comput3" << std::endl;
            OsSupport::OS::unlink("comput3");
            std::cout << "unlink comput4" << std::endl;
            OsSupport::OS::unlink("comput4");

            std::cout << "kill process test_server_id2" << std::endl;
            OsSupport::OS::kill_process(test_server_id2);
            std::cout << "kill process test_server_id4" << std::endl;
            OsSupport::OS::kill_process(test_server_id4);
            std::cout << "kill process test_server_id3" << std::endl;
            OsSupport::OS::kill_process(test_server_id3);
            std::cout << "kill process test_server_id1" << std::endl;
            OsSupport::OS::kill_process(test_server_id1);
            Cdmw::TestUtils::CORBATestManager::instance()->summary();

        }
        Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
        orb->destroy();
    }
    catch( const CORBA::Exception &e )
    {
        cerr <<__FILE__<<" "<<__LINE__<<" "<< e._name() << endl;
        return EXIT_FAILURE;
    }
    catch( const Cdmw::Exception &e )
    {
        cerr <<__FILE__<<" "<<__LINE__<<" "<< e.what() << endl;
        return EXIT_FAILURE;
    }
    catch(...)
    {
        cerr << "Unexpected exception" << endl;
        return EXIT_FAILURE;
    }

    return EXIT_SUCCESS;
}

