/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ft.watchdog.test.testtimeoutrequest;

import java.io.IOException;
import java.io.PrintStream;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;

import cdmw.cdmwinit.CdmwInterface;
import cdmw.ft.watchdog.*;
import cdmw.orbsupport.*;

public class MainServer {
    private final static String FILE_PATH_OPTION = "--FilePath";

    private final static String FILE_PATH_OPTION2 = "--ComputFilePath";

    private final static String HOST_NAME_OPTION = "--HostName";

    private static final int EXIT_FAILURE = 1;

    private final static int POA_THREAD_POOL_SIZE = 5;

    public static void usage(PrintStream os, String programName) {
        os.print("Usage:\n"
                + programName
                + " options\n"
                + "\n"
                + "Options:\n"
                + "--help                                         Display this information.\n"
                + "\n" + "[" + FILE_PATH_OPTION
                + "]=path          path of the file where hello IOR should be stored.\n"
                + "[" + FILE_PATH_OPTION2
                + "]=path          path of the file where comput IOR should be stored.\n"
                + "[" + HOST_NAME_OPTION
                + "]=hostname      hostname where server is launched.\n");
    }

    public static void main(String[] args) {
        // help argument
        if (args.length == 0) {
            usage(System.out, "MainServer");
            return;
        }

        if (args.length >= 1) {
            if (args[1].equals("--help")) {
                usage(System.out, "MainServer");
                return;
            }
        }

        // get host name
        String hostName = cdmw.ossupport.OS.getOptionValue(args, "--HostName");

        // get Name File to save hello interface
        String fileName = cdmw.ossupport.OS.getOptionValue(args, FILE_PATH_OPTION);

        if (fileName.equals("no")) {
            fileName = "hello.ior";
        }
        System.out.println(hostName + " testServer : fileName = " + fileName);

        // get Name File to save hello interface
        String fileName2 = cdmw.ossupport.OS.getOptionValue(args, FILE_PATH_OPTION2);

        if (fileName2.equals("no")) {
            fileName2 = "comput.ior";
        }
        System.out.println(hostName + " testServer : fileName2 = " + fileName2);

        ORB orb = null;

        try {
            // Initialise FT service
            System.out.println(hostName + "test_server: initialise watchdog service");
            WatchdogServiceInit.init(args);

            // Initialises the ORB
            StrategyList strategyList = new StrategyList();
            strategyList.addORBThreaded();
            strategyList.addPOAThreadPool(POA_THREAD_POOL_SIZE);

            System.out.println(hostName + " test_server: initialises the ORB");
            orb = ORBSupport.ORBInit(args, strategyList);

            // Get the root POA
            System.out.println(hostName + " test_server: get the root POA");
            org.omg.CORBA.Object obj = orb.resolve_initial_references("RootPOA");
            POA rootPOA = POAHelper.narrow(obj);

            System.out.println(hostName + " test_server: activate the root POA");
            POAManager poaMgr = rootPOA.the_POAManager();
            poaMgr.activate();

            // init Cdmw
            System.out.println(hostName + " test_server: init Cdmw");
            CdmwInterface.cdmwInit(orb, args, null, "cdmw.ft.watchdog.test.testtimeoutrequest.ServerCdmwInterface");

            System.out.println(hostName + " test_server: create HelloInterface_impl");

            HelloInterfaceImpl helloInterface = new HelloInterfaceImpl(orb, hostName);
            com.thalesgroup.CdmwFT.Test.HelloInterface helloInterfaceRef = helloInterface._this(orb);

            System.out.println(hostName + " test_server: create ComputInterface_impl");

            ComputInterfaceImpl computInterface = new ComputInterfaceImpl(orb, hostName);
            com.thalesgroup.CdmwFT.Test.ComputInterface computInterfaceRef = computInterface._this(orb);

            // export the object reference to a file
            System.out.println(hostName
                + " test_server: export the hello object reference to a file");
            String refString1 = orb.object_to_string(helloInterfaceRef);
            java.io.FileWriter os1 = new java.io.FileWriter(fileName);
            os1.write(refString1);
            os1.close();

            // export the object reference to a file
            System.out.println(hostName
                + " test_server: export the comput object reference to a file");
            String refString2 = orb.object_to_string(computInterfaceRef);
            java.io.FileWriter os2 = new java.io.FileWriter(fileName2);
            os2.write(refString2);
            os2.close();

            int timescale = cdmw.testutils.TestManager.getTimescale();
            Thread.sleep(timescale * 20000);

            // Setup signal handling for ORB to be shut down
            OrbShutdownSignalHandler handler = new OrbShutdownSignalHandler(orb);
            cdmw.orbsupport.ORBSupport.InstallORBShutdownSignalHandler(orb, handler);

            System.out.println(hostName + " test_server: orb run");
            orb.run();

            cdmw.orbsupport.ORBSupport.UninstallORBShutdownSignalHandler(handler);

            CdmwInterface.cdmwCleanup(orb, "cdmw.ft.watchdog.test.testtimeoutrequest.ServerCdmwInterface");
            
        }
        catch (SystemException e) {
            e.printStackTrace();
            System.exit(EXIT_FAILURE);
        }
        catch (UserException e1) {
            e1.printStackTrace();
            System.exit(EXIT_FAILURE);
        }
        catch (cdmw.common.Exception e2) {
            e2.printStackTrace();
            System.exit(EXIT_FAILURE);
        }
        catch (Exception e2) {
            System.err.println("Uncaught exception");
            e2.printStackTrace();
            System.exit(EXIT_FAILURE);
        }

        if (orb != null) {
            cdmw.orbsupport.ORBSupport.ORBCleanup(orb);
            orb.destroy();
        }

    }
}
