/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ft.watchdog.test.testtimeoutrequest;

import org.junit.Assert;
import org.junit.Test;

import org.omg.CORBA.*;
import org.omg.FT.*;

import cdmw.ossupport.OS;
import cdmw.testutils.*;
import cdmw.testutils.CORBATestManager.InvalidObject;

public class PullMonitorableTest {

    private static final String[] objectIds = {"pm1_object", "pm2_object", "pm3_object",
        "pm4_object"};

    private PullMonitorable[] pullMonitorable = new PullMonitorable[TestUtils.NB_LOCATION];

    private ORB orb;

    public PullMonitorableTest() {
        CORBATestManager corbaTestMngr = CORBATestManager.instance();

        try {
            this.orb = corbaTestMngr.getORB();

            for (int i = 0; i < this.pullMonitorable.length; i++ ) {
                org.omg.CORBA.Object obj = corbaTestMngr.getObject(objectIds[i]);
                this.pullMonitorable[i] = PullMonitorableHelper.narrow(obj);
            }
        }
        catch (InvalidObject ex) {
            ex.printStackTrace();
        }
    }

    @Test
    public void doTests() {
        for (int i = 0; i < TestUtils.NB_LOCATION; i++ ) {
            try {
                System.out.println("Calling is_alive on PullMonitorable on host "
                    + TestUtils.locations[i]);
                System.out.println("All processes shall be alive");
                boolean result = this.pullMonitorable[i].is_alive();
                if (result) {
                    System.out.println("All processes are alive");
                    Assert.assertTrue(true);
                }
                else {
                    Assert.assertTrue(false);
                }
            }
            catch (SystemException e) {
                System.out.println("CORBA System Exception raised ");
                Assert.assertTrue(false);
            }
        }

        try {
            OS.sleep(1200);
        }
        catch (InterruptedException e) {
            Assert.assertTrue(false);
            e.printStackTrace();
        }

        for (int i = 0; i < TestUtils.NB_LOCATION; i++ ) {
            try {
                System.out.println("Calling is_alive on PullMonitorable on host "
                    + TestUtils.locations[i]);
                System.out.println("Process C shall be on timeout");
                boolean result = this.pullMonitorable[i].is_alive();
                if (i == 2) {
                    if (result) {
                        // retry until is_alive return false
                        for (int j = 0; j < 30; j++ ) {
                            result = this.pullMonitorable[i].is_alive();
                            if (!result)
                                break;

                            try {
                                OS.sleep(100);
                            }
                            catch (InterruptedException e) {
                                Assert.assertTrue(false);
                                e.printStackTrace();
                            }
                        }
                        Assert.assertTrue(!result);
                    }
                    else {
                        Assert.assertTrue(true);
                    }
                }
                else {
                    if (result) {
                        Assert.assertTrue(true);
                    }
                    else {
                        Assert.assertTrue(false);
                    }
                }
            }
            catch (SystemException e) {
                System.out.println("CORBA System Exception raised ");
                Assert.assertTrue(false);
            }
        }

        int timeScale = TestManager.getTimescale();
        try {
            OS.sleep(1000);
        }
        catch (InterruptedException e) {
            Assert.assertTrue(false);
            e.printStackTrace();
        }

        for (int i = 0; i < TestUtils.NB_LOCATION; i++ ) {
            try {
                System.out.println("Calling is_alive on PullMonitorable on host "
                    + TestUtils.locations[i]);
                System.out.println("Process C shall be on timeout");
                boolean result = this.pullMonitorable[i].is_alive();
                if (i == 2) {
                    if (result) {
                        // retry until is_alive return false
                        for (int j = 0; j < 30; j++ ) {
                            result = this.pullMonitorable[i].is_alive();
                            if (!result)
                                break;

                            try {
                                OS.sleep(100);
                            }
                            catch (InterruptedException e) {
                                Assert.assertTrue(false);
                                e.printStackTrace();
                            }
                        }
                        Assert.assertTrue(!result);
                    }
                    else {
                        Assert.assertTrue(true);
                    }
                }
                else {
                    if (result) {
                        Assert.assertTrue(true);
                    }
                    else {
                        // retry until is_alive return true
                        for (int j = 0; j < 30; j++ ) {
                            result = this.pullMonitorable[i].is_alive();
                            if (result)
                                break;
                            try {
                                OS.sleep(100);
                            }
                            catch (InterruptedException e) {
                                Assert.assertTrue(false);
                                e.printStackTrace();
                            }
                        }
                        Assert.assertTrue(result);
                    }
                }
            }
            catch (SystemException e) {
                System.out.println("CORBA System Exception raised ");
                Assert.assertTrue(false);
            }
        }

        try {
            OS.sleep(timeScale * 5000);
        }
        catch (InterruptedException e) {
            Assert.assertTrue(false);
            e.printStackTrace();
        }

        for (int i = 0; i < TestUtils.NB_LOCATION; i++ ) {
            try {
                System.out.println("Calling is_alive on PullMonitorable on host "
                    + TestUtils.locations[i]);
                System.out.println("Process D shall be on timeout");
                boolean result = this.pullMonitorable[i].is_alive();
                if (i == 3) {
                    if (result) {
                        // retry until is_alive return false
                        for (int j = 0; j < 30; j++ ) {
                            result = this.pullMonitorable[i].is_alive();
                            if (!result)
                                break;

                            try {
                                OS.sleep(100);
                            }
                            catch (InterruptedException e) {
                                Assert.assertTrue(false);
                                e.printStackTrace();
                            }
                        }
                        Assert.assertTrue(!result);
                    }
                    else {
                        Assert.assertTrue(true);
                    }
                }
                else {
                    if (result) {
                        Assert.assertTrue(true);
                    }
                    else {
                        // retry until is_alive return true
                        for (int j = 0; j < 30; j++ ) {
                            result = this.pullMonitorable[i].is_alive();
                            if (result)
                                break;
                            try {
                                OS.sleep(100);
                            }
                            catch (InterruptedException e) {
                                Assert.assertTrue(false);
                                e.printStackTrace();
                            }
                        }
                        Assert.assertTrue(result);
                    }
                }
            }
            catch (SystemException e) {
                System.out.println("CORBA System Exception raised ");
                Assert.assertTrue(false);
            }
        }

        try {
            OS.sleep(timeScale * 1000);
        }
        catch (InterruptedException e) {
            Assert.assertTrue(false);
            e.printStackTrace();
        }

        for (int i = 0; i < TestUtils.NB_LOCATION; i++ ) {
            try {
                System.out.println("Calling is_alive on PullMonitorable on host "
                    + TestUtils.locations[i]);
                System.out.println("Process C shall be on timeout");
                boolean result = this.pullMonitorable[i].is_alive();
                if (i == 2) {
                    if (result) {
                        // retry until is_alive return false
                        for (int j = 0; j < 30; j++ ) {
                            result = this.pullMonitorable[i].is_alive();
                            if (!result)
                                break;

                            try {
                                OS.sleep(100);
                            }
                            catch (InterruptedException e) {
                                Assert.assertTrue(false);
                                e.printStackTrace();
                            }
                        }
                        Assert.assertTrue(!result);
                    }
                    else {
                        Assert.assertTrue(true);
                    }
                }
                else {
                    if (result) {
                        Assert.assertTrue(true);
                    }
                    else {
                        Assert.assertTrue(false);
                    }
                }
            }
            catch (SystemException e) {
                System.out.println("CORBA System Exception raised ");
                Assert.assertTrue(false);
            }
        }

        System.out.println("********************************************");
        System.out.println("            PullMonitorableTest stop");
        System.out.println("********************************************");
    }
}
