/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ft.watchdog.test.testtimeoutrequest;

import java.io.IOException;

import cdmw.cdmwinit.*;
import cdmw.common.Assert;
import cdmw.orbsupport.*;

import com.thalesgroup.CdmwPlatformMngt.*;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;

public class ServerCdmwInterface {

    public static void cdmwInit(ORB orb, String[] args, cdmw.cdmwinit.ProcessControl ctrl)
        throws SystemException {
        Assert.check(orb != null);
        CdmwProcessImpl processImpl = null;

        try {
            // *****************************************************************
            // ***************** CDMW root POA and servant manager ***********
            // *****************************************************************
            //
            // Create the CDMW root POA. It has its own POA Manager
            // May raise a CORBA::SystemException.
            //
            POA CdmwrootPOA = InitUtils.createCdmwRootPOA(orb);
            POAManager poaManager = CdmwrootPOA.the_POAManager();
            //
            // Initialize the server so that incoming requests are
            // handled from this point on
            //
            // May raise PortableServer::AdapterInactive or
            // CORBA::SystemException
            try {
                poaManager.activate();
            }
            catch (org.omg.PortableServer.POAManagerPackage.AdapterInactive e) {
                throw new org.omg.CORBA.INTERNAL(
                    ExceptionMinorCodes.INTERNALORBAdapterInactive,
                    CompletionStatus.COMPLETED_NO);
            }
            catch (SystemException ex) {
                throw new CORBASystemExceptionWrapper(ex,
                    "Could not activate the CDMW POA Manager!");
            }

            // *****************************************************************
            // ***************** setup platforminterface object **************
            // *****************************************************************

            // Did we receive a ProcessControl callback object?
            ProcessControl processCtrl = null;
            if (ctrl != null)
                processCtrl = ctrl;
            else
                processCtrl = new DefaultProcessControl();

            // We need local request timeout detection, create a 
            // localPullMonitorable object.
            com.thalesgroup.CdmwFT.LocalPullMonitorable pullMonitorable 
                = new cdmw.ft.watchdog.RequestTimeoutLocalPullMonitorableImpl();
                
            // CdmwProcess_impl() will acquire a reference on ProcessControl
            // object and
            // will automatically release it at object's destruction.
            POA rootPOA = InitUtils.getRootPOA(orb);
            processImpl = new CdmwProcessImpl(orb, 
                                              rootPOA, 
                                              CdmwrootPOA, 
                                              processCtrl, 
                                              pullMonitorable);

            ProcessDelegate process = processImpl._this(orb);
            org.omg.FT.PullMonitorable pm = process.get_pull_monitorable();

            String hostName = cdmw.ossupport.OS.getOptionValue(args, "--HostName");

            if (hostName == "no") {
                hostName = "HOST1";
            }
            java.lang.System.out.println(hostName
                + " test_server: export ProcessDelegate the object reference to a file");
            String refstring = orb.object_to_string(pm);
            String pmFileName = hostName + "_pullMonitorable.ior";
            try {
                java.io.FileWriter os1 = new java.io.FileWriter(pmFileName);
                os1.write(refstring);
                os1.close();
            }
            catch (IOException e) {
                e.printStackTrace();
            }
        }
        catch (CORBASystemExceptionWrapper ex) {
            java.lang.System.out.println("CDMW_init: Fatal Error: \n"
                + "CORBA Exception : " + ex.getMessage());
            ex.printStackTrace();
            // ADVISE_EXCEPTION_RAISED(process_impl);
            ex.raiseSystemException();
        }
        catch (SystemException ex) {
            java.lang.System.out.println("CDMW_init: Fatal Error: \n"
                + "CORBA Exception : " + ex.getMessage());
            ex.getMessage();
            // ADVISE_EXCEPTION_RAISED(process_impl);
            throw ex;
        }
    }
}
