/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ft.watchdog.test.testtimeoutrequest;

import java.io.*;

import org.junit.*;
import org.junit.runner.RunWith;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;

import cdmw.ossupport.OS;
import cdmw.testutils.*;
import cdmw.testutils.CORBATestManager.InvalidObject;
import cdmw.testutils.threadtestengine.*;

import static org.junit.Assert.*;

import com.thalesgroup.CdmwFT.*;
import com.thalesgroup.CdmwFT.Test.*;

@RunWith( CdmwThreadedTestClassRunner.class)
public class TestTimeoutRequest {

    /**
     * Field is used for obtaining TestProcessor class instance
     */
    private TestContext testContext;

    private ORB orb;

    private POA rootPOA;

    private HelloInterface helloGroup;

    private ComputInterface computObj;

    public TestTimeoutRequest() {

        try {
            this.orb = CORBATestManager.instance().getORB();

            this.rootPOA = CORBATestManager.instance().getPOA();

            org.omg.CORBA.Object obj = CORBATestManager.instance().getObject(
                "comput_object");
            this.computObj = ComputInterfaceHelper.narrow(obj);
        }
        catch (InvalidObject e) {
            e.printStackTrace();
        }
    }

    @Test
    public void doTests() {
        // set number of requested successfull tests
        // set_nbOfRequestedTestOK(45);
        int timescale = CORBATestManager.getTimescale();

        try {
            cdmw.ossupport.OS.unlink("hello.stt");
            cdmw.ossupport.OS.unlink("comput.stt");
        }
        catch (Exception e) {}

        com.thalesgroup.CdmwFT.Test.HelloInterface helloA = TestUtils
            .getHelloRefFromFile(orb, TestUtils.helloFiles[0]);
        // Start the HelloThread to make concurrent request
        System.out.println("Start the HelloThread");

        TestProcessor tpHello = this.testContext.getTestProcessor();
        boolean tpHelloNotNull = (tpHello != null); 
        tpHello.addTest(HelloTest.class);
        tpHello.runTests();

        try {
            System.out.println("Calling hello on HelloInterface on A...");
            helloA.hello(this.computObj);
            System.out.println("hello on HelloInterface on A OK...");
            assertTrue(true);
        }
        catch (SystemException e) {
            System.out.println("CORBA System Exception raised " + e.getMessage());
            assertTrue(false);
        }

        try {
            System.out.println("Calling send_TRANSIENT_exception on HelloInterface on A...");
            helloA.send_TRANSIENT_exception();
            assertTrue(false);
        }
        catch (TRANSIENT e) {
            System.out.println("Expected CORBA::TRANSIENT Exception raised");
            assertTrue(true);
        }
        catch (SystemException e1) {
            System.out.println("Unexpected CORBA System Exception raised "
                + e1.getMessage());
            assertTrue(false);
        }

        System.out.println("Start the PullMonitorableThread ");
        TestProcessor tpPullMonitorable = this.testContext.getTestProcessor();
        tpPullMonitorable.addTest(PullMonitorableTest.class);
        tpPullMonitorable.runTests();

        try {
            System.out.println("Calling hello on HelloInterface on A and comput obj on C ...");
            helloA.hello(this.computObj);
            assertTrue(true);
        }
        catch (SystemException e) {
            System.out.println("Unexpected CORBA System Exception raised "
                + e.getMessage());
            assertTrue(false);
        }

        try {
            System.out.println("Calling hello on HelloInterface on A ...");
            helloA.hello(this.computObj);
            System.out.println("hello on HelloInterface on A OK...");
            assertTrue(true);
        }
        catch (SystemException e) {
            System.out.println("CORBA System Exception raised " + e.getMessage());
            assertTrue(false);
        }

        try {
            System.out.println("Calling hello on HelloInterface on A and comput group on C ...");
            helloA.hello(computObj);
            assertTrue(true);
        }
        catch (TRANSIENT e) {
            System.out.println("Unexpected CORBA::TRANSIENT Exception raised "
                + e.getMessage());
            assertTrue(false);
        }
        catch (SystemException e) {
            System.out.println("Unexpected CORBA System Exception raised "
                + e.getMessage());
            assertTrue(false);
        }

        try {
            System.out.println("Calling hello on HelloInterface on A ...");
            helloA.hello(this.computObj);
            assertTrue(true);
        }
        catch (TRANSIENT e) {
            System.out.println("Unexpected CORBA::TRANSIENT Exception raised "
                + e.getMessage());
            assertTrue(false);
        }
        catch (SystemException e) {
            System.out.println("Unexpected CORBA System Exception raised "
                + e.getMessage());
            assertTrue(false);
        }

        try {
            OS.sleep(timescale * 1000);
        }
        catch (InterruptedException e1) {
            e1.printStackTrace();
        }

        com.thalesgroup.CdmwFT.Test.HelloInterface helloB = TestUtils
            .getHelloRefFromFile(this.orb, TestUtils.helloFiles[1]);
        com.thalesgroup.CdmwFT.Test.ComputInterface computD = TestUtils
            .getComputRefFromFile(this.orb, TestUtils.computFiles[3]);

        try {
            System.out.println("Calling hello on HelloInterface on B");
            helloB.hello(computD);

            try {
                OS.sleep(timescale * 1000);
            }
            catch (InterruptedException e1) {
                e1.printStackTrace();
            }
        }
        catch (SystemException e) {
            System.out.println("CORBA System Exception raised " + e.getMessage());
            assertTrue(false);
        }

        for (int i = 0; i < 8; i++ ) {
            try {
                System.out.println("Calling hello on HelloInterface B and comput on D...");
                helloB.hello(computD);
                assertTrue(true);
            }
            catch (SystemException e) {
                System.out.println("CORBA System Exception raised " + e.getMessage());
                assertTrue(false);
            }
        }

        try {
            System.out.println("Calling hello on HelloInterface on A ...");
            helloA.hello(this.computObj);
            assertTrue(true);
        }
        catch (TRANSIENT e) {
            System.out.println("Unexpected CORBA::TRANSIENT Exception raised");
            assertTrue(false);
        }
        catch (SystemException e) {
            System.out.println("Unexpected CORBA System Exception raised "
                + e.getMessage());
            assertTrue(false);
        }

        try {
            System.out.println("Calling hello on HelloInterface on A ...");
            helloA.hello(this.computObj);
            assertTrue(true);
        }
        catch (TRANSIENT e) {
            System.out.println("Unexpected CORBA::TRANSIENT Exception raised");
            assertTrue(false);
        }
        catch (SystemException e) {
            assertTrue(false);
        }

        try {
            cdmw.ossupport.OS.unlink("hello.stt");
            cdmw.ossupport.OS.unlink("comput.stt");
        }
        catch (Exception e) {}
    }
}
