/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ft.watchdog.test.testtimeoutrequest;

import java.io.*;

import org.omg.CORBA.*;
import org.omg.FT.*;

import com.thalesgroup.CdmwFT.Test.ComputInterfaceHelper;

import cdmw.orbsupport.*;

public class TestUtils {
    public static final int NB_LOCATION = 4;

    public static final String[] locations = {"A", "B", "C", "D"};

    public static final String[] helloFiles = {"hello1", "hello2", "hello3", "hello4"};

    public static final String[] computFiles = {"comput1", "comput2", "comput3",
        "comput4"};

    public static com.thalesgroup.CdmwFT.Test.HelloInterface getHelloRefFromFile(ORB orb,
        String fileName) throws SystemException {
        org.omg.CORBA.Object objRef = null;
        com.thalesgroup.CdmwFT.Test.HelloInterface helloRef = null;

        // import the object reference from the file
        FileReader is = null;
        try {
            is = new java.io.FileReader(fileName);
        }
        catch (IOException e) {
            System.err.println("Error opening reference filename: " + fileName);
            throw new INTERNAL(ExceptionMinorCodes.INTERNALFaultToleranceError,
                CompletionStatus.COMPLETED_NO);
        }
        
        try {
            BufferedReader bReader = new BufferedReader(is);
            String iorStr = bReader.readLine();
            objRef = orb.string_to_object(iorStr);
            helloRef = com.thalesgroup.CdmwFT.Test.HelloInterfaceHelper.narrow(objRef);
            
            bReader.close();
            
            is.close();
        }
        catch (SystemException e) {
            System.err.println(" CORBA System Exception raised: " + e.getMessage());
            e.printStackTrace();
            throw e;
        }
        catch (IOException e) {
            System.err.println("Error opening reference filename: " + fileName);
            throw new INTERNAL(ExceptionMinorCodes.INTERNALFaultToleranceError,
                CompletionStatus.COMPLETED_NO);
        }

        return helloRef;
    }

    public static org.omg.FT.PullMonitorable getPmRefFromFile(ORB orb, String hostName)
        throws SystemException {
        org.omg.CORBA.Object objRef = null;
        org.omg.FT.PullMonitorable pmRef = null;

        // import the object reference from the file
        String fileName = hostName + "_pullMonitorable.ior";
        FileReader is = null;
        try {
            is = new java.io.FileReader(fileName);
        }
        catch (IOException e) {
            e.printStackTrace();
        }
        try {
            try {
                BufferedReader bReader = new BufferedReader(is);
                String iorStr = bReader.readLine();
                objRef = orb.string_to_object(iorStr);
                pmRef = PullMonitorableHelper.narrow(objRef);
                
                bReader.close();
            }
            catch (SystemException e) {
                System.err.println(" CORBA System Exception raised: " + e.getMessage());
                e.printStackTrace();
                throw e;
            }
            catch (Exception e) {
                throw new INTERNAL(ExceptionMinorCodes.INTERNALFaultToleranceError,
                    CompletionStatus.COMPLETED_NO);
            }
            is.close();
        }
        catch (FileNotFoundException e) {
            System.err.println("Error opening reference filename: " + fileName);
            throw new INTERNAL(ExceptionMinorCodes.INTERNALFaultToleranceError,
                CompletionStatus.COMPLETED_NO);
        }
        catch (IOException e) {
            System.err.println("Error opening reference filename: " + fileName);
            throw new INTERNAL(ExceptionMinorCodes.INTERNALFaultToleranceError,
                CompletionStatus.COMPLETED_NO);
        }

        return pmRef;
    }

    public static com.thalesgroup.CdmwFT.Test.ComputInterface getComputRefFromFile(
        ORB orb, String fileName) throws SystemException {
        org.omg.CORBA.Object objRef;
        com.thalesgroup.CdmwFT.Test.ComputInterface computRef = null;

        // import the object reference from the file
        System.out.println(" get_comput_ref_from_file file_name: " + fileName);

        FileReader is = null;
        try {
            is = new java.io.FileReader(fileName);
        }
        catch (IOException e) {
            e.printStackTrace();
        }
      try {
          try {
              BufferedReader bReader = new BufferedReader(is);
              String iorStr = bReader.readLine();
                objRef = orb.string_to_object(iorStr);
                System.out.println(" get_comput_ref_from_file _narrow ");
                computRef = ComputInterfaceHelper.narrow(objRef);
                
                bReader.close();
            }
            catch (SystemException e) {
                System.err.println(" CORBA System Exception raised: " + e.getMessage());
                e.printStackTrace();
                throw e;
            }
            catch (Exception e) {
                throw new INTERNAL(ExceptionMinorCodes.INTERNALFaultToleranceError,
                    CompletionStatus.COMPLETED_NO);
            }
            is.close();
        }
        catch (IOException e) {
            System.err.println("Error opening reference filename: " + fileName);
            throw new INTERNAL(ExceptionMinorCodes.INTERNALFaultToleranceError,
                CompletionStatus.COMPLETED_NO);
        }

        return computRef;
    }
}
