/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "datastoretest/TestDataStore.hpp"
#include "datastoretest/TestThread.hpp"
namespace Cdmw
{
namespace FT 
{

CPPUNIT_TEST_SUITE_REGISTRATION( TestDataStore );


// constructor
TestDataStore::TestDataStore()
    throw()

{
    m_orb =  CORBA::ORB::_duplicate( Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());
    PortableServer::POA_ptr poalocal = Cdmw::TestUtils::CORBATestManager::instance()->get_POA();
    PortableGroup::GOA_ptr goalocal = dynamic_cast<PortableGroup::GOA_ptr>(poalocal);

    m_rootPOA = PortableGroup::GOA::_duplicate(goalocal);

    if (Cdmw::TestUtils::CORBATestManager::instance()->get_property ("realOverlap") == "yes")
        m_realOverlap = true;
    else
        m_realOverlap = false;
}

// destructor
TestDataStore::~TestDataStore()
   throw()
{
}

void TestDataStore::do_tests()
{
    

    // Timeout for cohort
    Cdmw::OsSupport::OS::Timeval cohort_timeout;
    cohort_timeout.seconds = Cdmw::FT::COHORT_TIMEOUT;
    cohort_timeout.microseconds = 0;
    
    // Timeout for coordinator
    Cdmw::OsSupport::OS::Timeval coordinator_timeout;
    coordinator_timeout.seconds = Cdmw::FT::COORDINATOR_TIMEOUT;
    coordinator_timeout.microseconds = 0;
    
    Cdmw::FT::DataStoreContext* context;
    
    std::cout << "create DataStore for DataPart1" << std::endl;
        // StorageHome for DataPart1
    Cdmw::FT::DataPart1StorageHome* dataPart1StorageHome = NULL;
    
    // DataStorage for DataPart1
    Cdmw::FT::DataPart1Store* dataPart1Store = NULL;
    
    // StorageHome for DataPart2
    Cdmw::FT::DataPart2StorageHome* dataPart2StorageHome = NULL;
    
    // DataStorage for DataPart2
    Cdmw::FT::DataPart2Store* dataPart2Store = NULL;
    

    
    context = new Cdmw::FT::DataStoreContext(m_orb.in(),
                                             m_rootPOA.in(),
                                             Cdmw::FT::MULTICAST_PORT,
                                             Cdmw::FT::MULTICAST_DOMAIN,
                                             Cdmw::FT::MULTICAST_IP,
                                             Cdmw::FT::MAX_TRANSACTION_IN_PROGRESS,
                                             Cdmw::FT::MAX_TRANSACTION_DONE,
                                             cohort_timeout,
                                             coordinator_timeout);
    
    std::cout << "create DataStore for DataPart1" << std::endl;
    dataPart1Store = 
    new Cdmw::FT::DataPart1Store(Cdmw::FT::DATA_STORE_1_IDENTIFIER,
                                 m_orb.in(),
                                 m_rootPOA.in(),
                                 *context);
    
    std::cout << "create DataStorageHome for DataPart1" << std::endl;
    dataPart1StorageHome =
           new Cdmw::FT::DataPart1StorageHome(Cdmw::FT::DATA_STORE_1_IDENTIFIER);
    
    std::cout << "create DataStore for DataPart2" << std::endl;
    dataPart2Store = 
    new Cdmw::FT::DataPart2Store(Cdmw::FT::DATA_STORE_2_IDENTIFIER,
                                 m_orb.in(),
                                 m_rootPOA.in(),
                                 *context);
    
    std::cout << "create DataStorageHome for DataPart2" << std::endl;
    dataPart2StorageHome = 
    new Cdmw::FT::DataPart2StorageHome(Cdmw::FT::DATA_STORE_2_IDENTIFIER);

    Cdmw::FT::TestThread testThread1(m_orb.in(), dataPart1StorageHome, dataPart2StorageHome, 1, m_realOverlap);
    Cdmw::FT::TestThread testThread2(m_orb.in(), dataPart1StorageHome, dataPart2StorageHome, 2, m_realOverlap);
    
    // Start threads 
    testThread1.start();
    testThread2.start();
    
    testThread1.join();
    std::cout << "Thread1 is joined" << std::endl;
    testThread2.join();
    std::cout << "Thread2 is joined" << std::endl;


    CPPUNIT_ASSERT(testThread1.get_result() && testThread2.get_result());

    delete dataPart1Store;
    delete dataPart1StorageHome;
    delete dataPart2Store;
    delete dataPart2StorageHome;
    
}
    
} // End namespace FT 

} // End namespace Cdmw



