/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_MY_PROCESS_BEHAVIOUR_HPP 
#define INCL_MY_PROCESS_BEHAVIOUR_HPP 

#include "SystemMngt/platforminterface/PlatformInterface.hpp"
#include <ConfAndPlug/cdmwinit/ProcessControl.hpp>
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/orbsupport/RefCountLocalObject.hpp"
#include <FaultTolerance/idllib/FT.stub.hpp>
#include <FaultTolerance/idllib/CdmwFTActivation.skel.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include "testftperffailure/TestHello_impl.hpp"
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include "Repository/repositoryinterface/RepositoryInterface.hpp"
#include "Repository/idllib/CdmwNamingAndRepository.stub.hpp"
#include <string>
#include <sstream>
#include <iostream>
#include <fstream>



class HelloActivationHandler : virtual public CdmwFT::Location::ActivationHandler, 
                               virtual public Cdmw::OrbSupport::RefCountLocalObject
{
public:
    HelloActivationHandler(bool backup, ::FT::client_ptr client)
        :m_backup(backup),
         m_client(::FT::client::_duplicate(client))
    {};
    
    virtual void activate() throw(CORBA::SystemException)
    {
        if (m_backup)
        {
            Cdmw::OsSupport::OS::Timeval time_activation;
            time_activation = Cdmw::OsSupport::OS::get_time();
            std::cout<< "***time activation"<<time_activation.seconds <<":"<<time_activation.microseconds<<std::endl;
            m_client->is_primary(Cdmw::OsSupport::OS::get_hostname().c_str());
        }
    };
    
    
    virtual void deactivate() throw(CORBA::SystemException)
    {
    };

private:
    /**
    * The application name.
    */
    bool m_backup;
    ::FT::client_var m_client;
};



/**
* Purpose:
* <p>
* The abstract process behaviour.
*/
//class MyProcessBehaviour : public Cdmw::PlatformMngt::ProcessBehaviour
class MyProcessBehaviour : public Cdmw::CdmwInit::ProcessControl
{

public:

    /**
    * Purpose:
    * <p> The constructor.
    */ 
    MyProcessBehaviour  (CORBA::ORB_ptr orb, PortableServer::POA_ptr rootPOA, std::string the_host1, std::string the_host2)
        : m_hostname1(the_host1),
          m_hostname2(the_host2)
    {
        m_orb = CORBA::ORB::_duplicate(orb);
        m_rootPOA =  PortableServer::POA::_duplicate(rootPOA);

    }


    /**
    * Purpose:
    * <p> The destructor.
    */ 
    virtual ~MyProcessBehaviour() throw ()
    {
     
    }

    
    /**
    * Purpose:
    * <p>
    * the behaviour for the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/nb_steps:1.0
    * attribute
    */
    virtual CORBA::ULong nb_steps() throw(CORBA::SystemException)
    {
        return 1;
    }
    
    
    /**
    * Purpose:
    * <p>
    * the behaviour for the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/get_service:1.0
    * operation
    */
    virtual CORBA::Object_ptr get_service() throw(CORBA::SystemException)
    {
        return CORBA::Object::_nil();
    }
    
    
    /**
    * Purpose:
    * <p>
    * the behaviour for the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/initialise:1.0
    * operation
    */
    virtual void on_initialise(const CdmwPlatformMngtBase::StartupKind& startup_kind)
        throw( CORBA::SystemException)
    {

        CdmwNamingAndRepository::Repository_var repository
        = Cdmw::NamingAndRepository::RepositoryInterface::get_repository();

        // ======================================================
        // Get HELLO name domain from repository
        // ======================================================
        Cdmw::CommonSvcs::Naming::NamingInterface ni =
        Cdmw::NamingAndRepository::RepositoryInterface::get_domain_naming_interface ("dom1/dom2");

        std::string full_name = "dom3/client";
        
        CORBA::Object_var obj;
        
        try
        {
            obj = ni.resolve (full_name);
            CPPUNIT_ASSERT(true);
        }
        catch(const CosNaming::NamingContext::NotFound& e)
        {
            std::cerr<< e << std::endl;
            CPPUNIT_ASSERT(false);
        }
        catch(const CosNaming::NamingContext::CannotProceed& e)
        {
            std::cerr << e << std::endl;
            CPPUNIT_ASSERT(false);
        }
        catch(const CosNaming::NamingContext::InvalidName& e)
        {
            std::cerr << e << std::endl;
            CPPUNIT_ASSERT(false);
        }
        catch (const CORBA::SystemException& e)
        {
            std::cerr << e << std::endl;
            CPPUNIT_ASSERT(false);
        }
        
        m_client = ::FT::client::_narrow(obj.in());

        bool kill = false;
        std::cout<<m_hostname1.c_str()<<" "<< Cdmw::OsSupport::OS::get_hostname().c_str()<<std::endl;
        if (!strcmp(m_hostname1.c_str(), Cdmw::OsSupport::OS::get_hostname().c_str()))
            kill = true;

        Cdmw::server_impl* server = new Cdmw::server_impl(kill, m_client.in());
        m_server = server->_this();


        std::cout << " test_server: on_initialise called" << std::endl;
        obj = m_orb->resolve_initial_references("ReplicationManager");
        CdmwFT::ReplicationManager_var rm = CdmwFT::ReplicationManager::_narrow(obj.in());

        ::FT::Name OGName;
        OGName.length(1);
        OGName[0].id=CORBA::string_dup("ObjectTest");
        CORBA::Object_var obj_group = rm->get_object_group_ref_from_name(OGName);

        bool backup = false;
        std::cout<<m_hostname2.c_str()<<" "<< Cdmw::OsSupport::OS::get_hostname().c_str()<<std::endl;
        if (!strcmp(m_hostname2.c_str(), Cdmw::OsSupport::OS::get_hostname().c_str()))
            backup = true;

        CdmwFT::Location::ActivationHandler_var activation_handler 
        = new HelloActivationHandler(backup, m_client.in());
        
        try
        {
            CORBA::Object_var obj = m_orb->resolve_initial_references("FTActivationManager");
            CdmwFT::Location::ActivationManager_var  activation_manager 
            = CdmwFT::Location::ActivationManager::_narrow(obj.in());
            
            activation_manager->register_activation_handler(obj_group.in(), activation_handler.in()); 
        }
        catch (const FT::ObjectGroupNotFound&)
        {
            std::cerr << "ERROR: Impossible to register FTActivationHandler! Object Group not found!"
                      << std::endl;
        }
        catch (const CORBA::ORB::InvalidName& )
        {
            std::cerr << "ERROR: Impossible to register FTActivationManager! Name is invalid!"
                      << std::endl;
        }
        catch (const CORBA::SystemException& e)
        {
            
            std::cerr << "ERROR: Impossible to register FTActivationHandler!"
                      << std::endl;
            std::cerr << e << std::endl;
        }

        ::FT::Location loc;
        loc.length(3);
        loc[0].id = Cdmw::OsSupport::OS::get_hostname().c_str();
        loc[0].kind = "hostname";
        loc[1].id = Cdmw::PlatformMngt::PlatformInterface::Get_application_name();
        loc[1].kind = "applicationname";        
        loc[2].id = Cdmw::PlatformMngt::PlatformInterface::Get_process_name();
        loc[2].kind = "processname";

        // Add new member on the group
        TEST_INFO("add the new member "<<Cdmw::OsSupport::OS::get_hostname().c_str()<<"/"<<loc[1].id<<"/"<<loc[2].id<<" in group");
        try
        {
            obj_group = rm->add_member(obj_group.in(),
                                 loc,
                                 m_server.in());
        }
        
        catch( CORBA::Exception& e )
        {
            std::cerr << e._name() << std::endl;
            std::cerr << " **** TEST FAILED AT LINE " << __LINE__ 
                      << " IN FILE: " << __FILE__ << std::endl; 
        }
        


    }




    
    /**
    * Purpose:
    * <p>
    * the behaviour for the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/run:1.0
    * operation
    */
    virtual void on_run()
        throw(CORBA::SystemException,
              CdmwPlatformMngt::ProcessDelegate::NotReadyToRun)
    {    
        std::cout << " test_server: on_run called" << std::endl;

    }

    /**
    * Purpose:
    * <p>
    * the behaviour for the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/stop:1.0
    * operation
    */
    virtual void on_stop() throw(CORBA::SystemException)
    {    
        std::cout << " test_server: on_stop called" << std::endl;

    }


    
private:

    /**
    * The ORB.
    */
    CORBA::ORB_var m_orb;
    /**
     * The poa
     */
    PortableServer::POA_var m_rootPOA;
    
    /**
    * The application name.
    */
    std::string m_name;
    std::string m_hostname1;
    std::string m_hostname2;
    ::FT::server_var m_server;
    ::FT::client_var m_client;
};


#endif // INCL_MY_PROCESS_BEHAVIOUR_HPP

