/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <fstream>

// Cdmw Files
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>

#include <testtimeoutrequest/TestHello_impl.hpp>




namespace Cdmw
{
namespace Test
{


HelloInterface_impl::HelloInterface_impl(CORBA::ORB_ptr orb, 
                                         std::string location)
    throw(CORBA::SystemException)
    : m_orb(CORBA::ORB::_duplicate(orb)),
      m_nb_hello(0),
      m_location(location)
{ 
}


HelloInterface_impl::~HelloInterface_impl()
    throw()
{
}

void 
HelloInterface_impl::hello (CdmwFT::Test::ComputInterface_ptr comput, CORBA::ULong time_ms)
     throw(CORBA::SystemException)                  
{
    // export the location to a file
    std::ofstream os("hello.stt");
    os << m_location;
    os.close();

    m_nb_hello++;
    std::cout << m_location << " <####### HELLO " << m_nb_hello <<"  ######>" << std::endl;

    try
    {
        CORBA::ULong result = comput->get_result(time_ms);
        std::cout << "    result = " << result << std::endl;
    }
    catch (const CORBA::SystemException& e)
    {
        std::cerr << "CORBA::SystemException raised : " << e << std::endl;
        throw;
    }
}

void HelloInterface_impl::send_TRANSIENT_exception ()
    throw(CORBA::SystemException)
{
    throw CORBA::TRANSIENT(0, CORBA::COMPLETED_MAYBE);
}
      

}; // End namespace Test

}; // End namespace Cdmw

