/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.valid.testtimeoutrequest;

import java.io.IOException;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;
import cdmw.namingandrepository.RepositoryInterface;
import cdmw.platformmngt.PlatformInterface;
import cdmw.orbsupport.*;

public class MainTestTimeoutRequestServer {

    public static void main(String[] args) {

        try {

            String hostName =  cdmw.ossupport.OS.getHostname();

            System.out.println(" faulttoleranceclient: FT init");
            // Initialise FT service
            cdmw.cdmwinit.CdmwInterface.cdmwPreORBInit(args,"cdmw.generated.TestTimeoutRequestServerCdmwInterface");

            // Check value of ORBClass and ORBSingletonClass (PCR-1937)
            String s = "";
            s = System.getProperty("org.omg.CORBA.ORBClass");
            assert(s.equals("org.jacorb.orb.ORB"));
            s = System.getProperty("org.omg.CORBA.ORBSingletonClass");
            assert(s.equals("org.jacorb.orb.ORBSingleton"));

            // Initialises the ORB
            cdmw.orbsupport.StrategyList strategyList = new cdmw.orbsupport.StrategyList();
            strategyList.addORBThreaded();
            strategyList.addPOAThreadPerConnection();

            System.out.println(" faulttoleranceclient: ORB_init");
            ORB orb = cdmw.orbsupport.ORBSupport.ORBInit(args, strategyList);

            // Get the root POA
            org.omg.CORBA.Object obj = orb.resolve_initial_references("RootPOA");
            POA rootPOA = POAHelper.narrow(obj);

            POAManager poaMgr = rootPOA.the_POAManager();
            System.out.println(" faulttoleranceclient: activate POA");
            poaMgr.activate();

            cdmw.testutils.CORBATestManager.instance().init(
                (new Throwable()).getStackTrace()[0].getClassName(), 
                orb, 
                rootPOA);

            // creates the process behaviour
            MyProcessBehaviour pProcessBehaviour = new MyProcessBehaviour(orb);

            // init Cdmw
            System.out.println(" faulttoleraceclient: init Cdmw");
            cdmw.cdmwinit.CdmwInterface.cdmwInit(orb, args, pProcessBehaviour, 
                "cdmw.generated.TestTimeoutRequestServerCdmwInterface");



            System.out.println( hostName + " MainServer: create HelloInterfaceImpl" );

           HelloInterfaceImpl helloInterface 
                = new HelloInterfaceImpl(orb, hostName);
           com.thalesgroup.CdmwFT.Test.HelloInterface helloInterfaceRef 
                =  helloInterface._this(orb);

            System.out.println( hostName + " MainServer: create ComputInterface_impl" );

           ComputInterfaceImpl comput_interface 
                = new ComputInterfaceImpl(orb, hostName);
           com.thalesgroup.CdmwFT.Test.ComputInterface computInterfaceRef 
                =  comput_interface._this(orb);

        
            // export the object reference to the CDMW Repository
            System.out.println( hostName + " MainServer: export the hello object reference to CDMW Repository" );
            com.thalesgroup.CdmwNamingAndRepository.Repository repository
                = RepositoryInterface.getRepository();

            RepositoryInterface.init ("CDMW",repository);
    
            com.thalesgroup.CdmwNamingAndRepository.NameDomain helloDomain
                = repository.resolve_name_domain ("dom1/dom2/dom3");
     
            // get process name
            String processName = PlatformInterface.getProcessName();

            String helloName = processName + "HelloInterface";
            System.out.println( hostName + " MainServer: register " + helloName );

            try
                { 
                    helloDomain.release_name(helloName);
                }
            catch (com.thalesgroup.CdmwNamingAndRepository.NameDomainPackage.NotRegistered e)
                {
                    // not a problem it's the first time the server is launched
                    System.err.println( "CdmwNamingAndRepository.NameDomain.NotRegistered Exception raised : not a problem it's the first time the server is launched");
                }
            catch (com.thalesgroup.CdmwNamingAndRepository.InvalidName e)
                {
                    System.err.println( "CdmwNamingAndRepository.InvalidName Exception raised" );
                    e.printStackTrace();
                    System.exit(1);

                }
            catch (org.omg.CORBA.SystemException e)
                {
                    System.err.println("ERROR: ------------------------");
                    e.printStackTrace();
                    System.exit(1);
                }

            try
                {
                    Any regId
                        = helloDomain.new_name (helloName);
    
                    // register the object
                    helloDomain.register_object (regId, helloInterfaceRef);
                }
            catch (org.omg.CORBA.SystemException e)
                {
                    System.err.println("ERROR: ------------------------");
                    e.printStackTrace();
                    System.exit(1);
                }
            catch( com.thalesgroup.CdmwNamingAndRepository.NameDomainPackage.AlreadyExists e )
                {
                    e.printStackTrace();
                    System.exit(1);
                }

            // export the object reference to a CDMW Repository
            System.out.println(" MainServer: export the comput object reference to a CDMW Repository");
            String computName = processName + "ComputInterface";
            System.out.println( hostName + " MainServer: register " + computName );

            try 
                {
                    helloDomain.release_name(computName);
                }
            catch (com.thalesgroup.CdmwNamingAndRepository.NameDomainPackage.NotRegistered e)
                {
                    // not a problem it's the first time the server is launched
                    System.err.println( "CdmwNamingAndRepository.NameDomain.NotRegistered Exception raised : not a problem it's the first time the server is launched");
                }
            catch (com.thalesgroup.CdmwNamingAndRepository.InvalidName e)
                {
                    System.err.println( "CdmwNamingAndRepository.InvalidName Exception raised" );
                    e.printStackTrace();
                    System.exit(1);

                }
            catch (org.omg.CORBA.SystemException e)
                {
                    System.err.println("ERROR: ------------------------");
                    e.printStackTrace();
                    System.exit(1);
                }

            try
                {
                    Any regId 
                        = helloDomain.new_name (computName);
    
                    // register the object
                    helloDomain.register_object (regId, computInterfaceRef);
                }
            catch (org.omg.CORBA.SystemException e)
                {
                
                    System.err.println("ERROR: ------------------------");
                    e.printStackTrace();
                    System.exit(1);
                }
            catch( com.thalesgroup.CdmwNamingAndRepository.NameDomainPackage.AlreadyExists e )
                {
                    System.err.println("ERROR: ------------------------");
                    e.printStackTrace();
                    System.exit(1);
                }


            // Setup signal handling for ORB to be shut down
            OrbShutdownSignalHandler sighandler = 
                new OrbShutdownSignalHandler (orb);
            ORBSupport.InstallORBShutdownSignalHandler (orb, sighandler);

            System.out.println( " MainServer: orb run" );

            // Enter in the main loop
            orb.run();

            // orb.shutdown() is done by the stop() of the ProcessBehaviour
            cdmw.cdmwinit.CdmwInterface.cdmwCleanup(orb, 
                "cdmw.generated.TestTimeoutRequestServerCdmwInterface");
            cdmw.orbsupport.ORBSupport.ORBCleanup(orb);
            orb.destroy();
        }
        catch (org.omg.CORBA.SystemException e) {
            e.printStackTrace();
            System.exit(1);
        }
        catch (org.omg.CORBA.UserException e1) {
            e1.printStackTrace();
            System.exit(1);
        }
        catch (cdmw.common.Exception e2) {
            e2.printStackTrace();
            System.exit(1);
        }
        catch (Exception e3) {
            System.err.println("Unexpected exception");
            System.exit(1);
        }

        System.out.println("Press enter to stop the process");
        try {
            System.in.read();
        }
        catch (IOException e) {
            e.printStackTrace();
        }
        System.exit(0);
    }

}
