/*! \mainpage
 *
 * \section Overview
 * <a href="http://cardamom.objectweb.org/">CARDAMOM</a> is a framework to configure, deploy and execute near real-time and 
 * fault-tolerant distributed Command Control and Information Systems (CCIS). 
 * It relies on the following major principles:
 * \li <a href="http://cardamom.objectweb.org/">CARDAMOM</a> is multi-domain, as it is intended to be used by different Units in several domains such as Air Traffic Control, Naval Combat Management, Electronic Warfare, Airborne Command, and Control systems.
 * \li <a href="http://cardamom.objectweb.org/">CARDAMOM</a> is developed by the CDO (Common Development Organisation), a dedicated structure currently grouping together <a href="http://www.thalesgroup.com/">THALES</a> and  <a href="http://www.selex-si.com/">Selex Sistemi Integrati S.p.A.</a>, and ready to welcome new parties. Benefits of such an open development structure are to share the development costs as well as to shorten the delivery times. <a href="http://cardamom.objectweb.org/">CARDAMOM</a> is steered by a Management Board, including representatives of each CDO party. 
 * \li Distribution and promotion of <a href="http://cardamom.objectweb.org/">CARDAMOM</a> as an Open Source product, in order to allow a wide use of the product and thus to ensure its reliability, durability and long-term support. 
 * \li <a href="http://cardamom.objectweb.org/">CARDAMOM</a> is <a href="http://www.corba.org/">CORBA</a>-based. <a href="http://www.corba.org/">CORBA</a> has been selected as it is a non-proprietary middleware standardised by the <a href="http://www.omg.org/">OMG</a> (Object Management Group), a non-profit-making organisation which gathers around 800 companies.
 *      <a href="http://www.corba.org/">CORBA</a> actually supports heterogeneity across hardware platforms and programming languages and thus is suitable to integrate smoothly legacy systems. <a href="http://www.corba.org/">CORBA</a> is not only a plain synchronous client-server model but also supports asynchronous communication (such as <a href="http://www.corba.org/">CORBA</a> messaging). It evolved to new technical features (such as fault tolerance, real-time.) that are particularly relevant to CCIS developed by the CDO parties.
 * \li Support of the <a href="http://www.corba.org/">CORBA</a> Component Model (CCM). One of the major advantages brought by this model is to make a clear separation between the functional properties (application logic) and the non-functional properties (technical services). Therefore this model allows application developers to focus on business code without worrying about the underlying execution platform. Additional benefits are an easier deployment and a better portability as well as an increased reuse of application software.
 *     The CCM is a good organisational model but it deserves enhancements to fulfil our requirements. Works are currently conducted so as to integrate new technical services which have not yet been specified by the <a href="http://www.omg.org/">OMG</a>.
 * \li Support of the <a href="http://www.corba.org/">CORBA</a> Fault Tolerance. As most of CCIS developed in the CDO are mission-critical systems that require a high level of availability, the provision of mechanisms allowing to develop fault-tolerant applications as well as a fault-tolerant middleware platform (with no single point of failure) is requested.
 *     The <a href="http://www.corba.org/">CORBA</a> Fault Tolerance specification provides several replication styles that are well suited to CCIS. <a href="http://cardamom.objectweb.org/">CARDAMOM</a> provides the WARM-Passive replication style, and the appropriate fault-tolerance mechanisms allowing managing transparently failure detection, request retry and redirection. In terms of consistency, an application may use the provided <a href="http://cardamom.objectweb.org/">CARDAMOM</a> facilities such as the State Transfer Framework (DataStore API) that offers an automatic and transparent update of replicas with strong replica consistency.
 * \li Added-value Execution Platform. <a href="http://cardamom.objectweb.org/">CARDAMOM</a> favours the integration of <a href="http://www.corba.org/">CORBA</a> services and functionality implementations that are available on the market (COTS). Nevertheless, <a href="http://www.omg.org/">OMG</a> specifications cover several CDO needs but not all of them. In addition, all the available implementations of some <a href="http://www.corba.org/">CORBA</a> services do not provide the required Quality of Service (QoS). The purpose of <a href="http://cardamom.objectweb.org/">CARDAMOM</a> is to provide an added-value execution platform including services of which functionality and implementation fit with CDO parties requirements.
 *  Needs not covered by <a href="http://www.omg.org/">OMG</a> are prototyped and specified, and then submitted to the <a href="http://www.omg.org/">OMG</a>. In the future, COTS implementation of these new <a href="http://www.omg.org/">OMG</a> specifications will replace the proprietary implementations that have been developed by the <a href="http://cardamom.objectweb.org/">CARDAMOM</a> development team.
 * \li Multi-environment. <a href="http://cardamom.objectweb.org/">CARDAMOM</a> is first developed on a reference platform (PC - Linux RedHat, TAO, JacORB) but is designed to run over other several target hardware platforms including Unix and Windows as well as to rely on several ORBs. It is foreseen to support applications developed in different programming languages, including C++ and Java.
 * \li Operation with a UML Tool Chain. The purpose is to support the <a href="http://www.omg.org/">OMG</a> Model Driven Architecture (MDA). This approach is characterised by the use of a stepped UML based process which includes two models: the Platform Independent Model (PIM) and the PSM (Platform Specific Model). Through several steps including model transformation and code generation, models are realised onto a selected execution platform (<a href="http://www.corba.org/">CORBA</a>, .Net, XML/SOAP, Java.).
 *  In addition to easing the development process, this approach allows to focus and capitalise on domain models regardless of the target execution platform that will be used in the end. It is especially worthwhile as it is well known that technology changes (0.5 to 2 year lifecycle) occur more frequently than domain evolutions (typically 10 to 20 year lifecycle).
 *  Works are performed so as to insure the use of <a href="http://cardamom.objectweb.org/">CARDAMOM</a> with a UML tool chain in the future.
 *
 */

//-------------------------------------------------------------------

/**
 * @defgroup Init Init
 */

/**
 * @defgroup Init_Restart Restart
 * @ingroup  Init
 */

/**
 * @defgroup Init_Reboot Reboot
 * @ingroup  Init
 */

/**
 * @defgroup Init_Retry Retry
 * @ingroup  Init
 */

//-------------------------------------------------------------------

/**
 * @defgroup Running Running
 */

/**
 * @defgroup Running_Restart Restart
 * @ingroup  Running
 */

/**
 * @defgroup Running_Reboot Reboot
 * @ingroup  Running
 */

/**
 * @defgroup Running_Retry Retry
 * @ingroup  Running
 */

//-------------------------------------------------------------------




