/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.common;

/**
 * Base class for CDMW exceptions
 *
 */
public class Exception extends java.lang.Exception {
    
    /**
     * Severity of the exception thrown.
     */
    private String severity;

    /**
     * File throwing the exception.
     */
    private String file;
    
    /**
     * Line throwing the exception.
     */
    private int line;
    
    /**
     * Exception creation time
     */
    private String timestamp;

    /**
     * Exception constructor.
     *
     * @param severity Severity of the current exception
     * @param message Information message
     */
    public Exception(Severity severity, String message) {
        super(message);
        this.severity = severity.toString();

        java.text.DateFormat df = new java.text.SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
        this.timestamp = df.format(new java.util.Date());
    }

    /**
     * Returns the information message.
     *
     * @return String containing information about the exception.
     */
    public String what() {
        String fullMessage;

        fullMessage = "File:" + file + " Line:" + line + " Time:" + timestamp + " Reason:" + getMessage();
        return fullMessage;
    }

    /**
     * Set the file throwing the exception.
     *
     * @param file The file throwing the exception
     */
    public void setFile(String file) {
        this.file = file;
    }

    /**
     * Set the line throwing the exception.
     *
     * @param line The line throwing the exception
     */
    public void setLine(int line) {
        this.line = line;
    }

    /**
     * Set the exception throwing time.
     *
     * @param timestamp The exception throwing time
     */
    public void setTimestamp(String timestmp) {
        this.timestamp = timestmp;
    }

    /**
     * Set the exception throwing time.
     *
     * @param timestmp The exception throwing time
     */
    public void setTimestamp(java.util.Date timestmp) {
        java.text.DateFormat df = new java.text.SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
        this.timestamp = df.format(timestmp);
    }

    /**
     * Returns the severity of the exception
     *
     * @return int  containing the severity of the exception.
     */
    public String getSeverity () {
        return severity;
    }

    public String toString(){
        String fullMessage;

        fullMessage = "File:" + file + " Line:" + line + " Time:" + timestamp + " Reason:" + getMessage();
        return fullMessage;
    }
}

