/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.common;

/**
 * This class provides standard Severity options for CDMW exception handling.
 * It also extends the Comparable interface allowing for sorting of exceptions by type.
 *
 */
public class Severity implements Comparable,java.io.Serializable
{
    // The name of the severity of the exception
    private final String name;

    // Ordinal of the next exception severity level
    private static int nextOrdinal = 0;

    // Assign an ordinal to this suit
    private final int ordinal = nextOrdinal++;

    /**
     * Constructor.
     * @param name severity description
     */
    private Severity (String name)
    {
    this.name = name;
    }

    /**
     * @return a string representation of the object.
     */
    public String toString()
    {
    return this.name;
    } 

    /**
     * @return 0 if the given object is a Severity object with the same level, the delta otherwise.
     */
    public int compareTo(Object o) 
    {
        return ordinal - ((Severity)o).ordinal;
    }

    /**
     * @return the level corresponding to the severity.
     */
    public int getLevel()
    {
        return this.ordinal;
    }

    public static final Severity SAFE = new Severity ("SAFE");
    public static final Severity FATAL = new Severity ("FATAL");
    public static final Severity UNKNOWN = new Severity ("UNKNOWN");

    private static final Severity[] levels = { SAFE, FATAL, UNKNOWN };

    private Object readResolve() throws java.io.ObjectStreamException 
    {
        return levels[ordinal]; // Canonicalize
    }
     
}

