/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <string>
#include <fstream>
#include "testproperties/TestProperties.hpp"
#include "Foundation/common/Properties.hpp"



namespace Cdmw
{

    namespace Common
    {

        /**
        *Implementation notes: [if no pertinent write none ]
        *<p>
        *Portability issues: [if no pertinent write none ]
        *<p>
        */

        CPPUNIT_TEST_SUITE_REGISTRATION( TestProperties );
        
        TestProperties::TestProperties()
        {
        }


        TestProperties::~TestProperties()
        throw()
        {
        }

        void
        TestProperties::do_tests()
        {
            // set number of requested successfull tests
//             set_nbOfRequestedTestOK (16);

            TEST_INFO("Creating a property");
            Properties* properties1 = new Properties();
            std::string prop1 = properties1->get_property("P1");
            CPPUNIT_ASSERT( strcmp(prop1.c_str(), "") == 0 );
            prop1 = properties1->get_property("P1","P11");
            CPPUNIT_ASSERT( strcmp(prop1.c_str(), "P11") == 0 );
            properties1->set_property("P1","P12");
            prop1 = properties1->get_property("P1","P11");
            CPPUNIT_ASSERT( strcmp(prop1.c_str(), "P12") == 0 );
            prop1 = properties1->get_property("P1");
            CPPUNIT_ASSERT( strcmp(prop1.c_str(), "P12") == 0 )
          ;

            Properties* properties2 = new Properties(properties1);

            prop1 = properties2->get_property("P1","P11");
            CPPUNIT_ASSERT( strcmp(prop1.c_str(), "P12") == 0 );
            prop1 = properties2->get_property("P1");
            CPPUNIT_ASSERT( strcmp(prop1.c_str(), "P12") == 0 );
            properties2->set_property("P1","P13");
            properties2->set_property("P2","P21");
            prop1 = properties2->get_property("P1","P11");
            CPPUNIT_ASSERT( strcmp(prop1.c_str(), "P13") == 0 );
            prop1 = properties2->get_property("P1");
            CPPUNIT_ASSERT( strcmp(prop1.c_str(), "P13") == 0 );

            Properties::PropertyNames s;
            properties2->property_names(s);

            CPPUNIT_ASSERT( s.size() == 2 );
            CPPUNIT_ASSERT( s.find("P1") != s.end());
            CPPUNIT_ASSERT( s.find("P2") != s.end());
            CPPUNIT_ASSERT( s.find("P3") == s.end());

            std::ofstream out("prop.txt");
            out<<*properties2;
            out<<"P3=P31"<<std::endl;
            out<<"   #comment"<<std::endl;
            out<<"       !comment"<<std::endl;
            out.close();
            
            std::ifstream in("prop.txt");
            in>>*properties2;
            in.close();

            Properties::PropertyNames s2;
            properties2->property_names(s2);
            std::cout<<s2.size()<<std::endl;
            CPPUNIT_ASSERT( s2.size() == 3 );
            CPPUNIT_ASSERT( s2.find("P1") != s2.end());
            CPPUNIT_ASSERT( s2.find("P2") != s2.end());
            CPPUNIT_ASSERT( s2.find("P3") != s2.end());
            std::ofstream out2("prop2.txt");
            out2<<*properties2;
            out2.close();

            std::ofstream out3("prop3.txt");
            out3<<*properties2;
            out3<<"P3="<<std::endl;
            out3.close();
            
            std::ifstream in3("prop3.txt");
            try
            {
                in3>>*properties2;
                CPPUNIT_ASSERT( false );
            }
            catch(BadParameterException)
            {
                TEST_INFO("BadParameterException");
                CPPUNIT_ASSERT( true );
            }
            
            in.close();

            delete properties1;
            delete properties2;

        {
          /*
           * Tests added for the PCR-0754 to test:
           * - the destructor
           * - the operator=
           * - the copy constructor
           */
          
          Properties* properties1 = new Properties();
          properties1->set_property("P1","P12");
          std::string prop1 = properties1->get_property("P1");
          CPPUNIT_ASSERT( strcmp(prop1.c_str(), "P12") == 0 );
          
          // Test of the copy constructor
          Properties* properties2 = new Properties(*properties1);
          
          // Modification and deletion of properties1 to be sure
          // that properties2 has its own copy of properties1
          properties1->set_property("P1","P123");
          delete properties1;
          std::string prop2 = properties2->get_property("P1");
          CPPUNIT_ASSERT( strcmp(prop2.c_str(), "P12") == 0 );
          
          // Test of the operator=
          Properties* properties3 = new Properties();
          *properties3 = *properties2;

          properties2->set_property("P1","P123");
          delete properties2;
          std::string prop3 = properties3->get_property("P1");
          CPPUNIT_ASSERT( strcmp(prop3.c_str(), "P12") == 0 );
          delete properties3;
        }
        }


    } // End namespace Common
} // End namespace Cdmw

