/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_CDMW_COMMNSVCS_NAMING_RONAMINGCONTEXTEXT_IMPL_HPP
#define INCL_CDMW_COMMNSVCS_NAMING_RONAMINGCONTEXTEXT_IMPL_HPP

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/CosNaming.skel.hpp>
#include <string>
#include <map>

namespace Cdmw
{
namespace CommonSvcs
{
namespace Naming
{

   typedef std::map<std::string,CORBA::Object_var, std::less<std::string> > BindingsMap;

/**
*Purpose:
*<p>
* Base class for Read-Only implementation of a CosNaming::NamingContextExt
* IDL interface.
* A) In order to provide a RO Naming Context, derived classes shall provide
* implementations for the two protected abstract operations resolve() and 
* get_all_bindings().
* B) Implementers willing to provide custom code for the NamingContextExt IDL
* operations can do so by implemting one of the associated protected operations.
* C) new_context(), destroy() can be overloaded if required
* D) Default behaviour for bind_new_context() is new_context() then bind_context().
*<p>
*Features:
*<p>
*Thread safe.
*/
   class RONamingContextExt_impl 
      : virtual public POA_CosNaming::NamingContextExt,
        virtual public PortableServer::RefCountServantBase
   {
         
      public:
         
         /**
          * Purpose:
          * <p>
          * Creates this instance.
          */
         RONamingContextExt_impl();
      
         /**
          * Purpose:
          * <p>
          * Destroys this instance. If the flag m_destroyed is true, the persistent
          * data is actually destroyed from the persistent storage.
          */
         virtual ~RONamingContextExt_impl();
      
         /**
          * Purpose:
          * <p>
          * Implements the
          * IDL:omg.org/CosNaming/NamingContext/bind:1.0
          * operation
          */
         virtual void bind(const CosNaming::Name& n,
                           CORBA::Object_ptr obj)
            throw(CosNaming::NamingContext::NotFound,
                  CosNaming::NamingContext::CannotProceed,
                  CosNaming::NamingContext::InvalidName,
                  CosNaming::NamingContext::AlreadyBound,
                  CORBA::SystemException);
      
         /**
          * Purpose:
          * <p>
          * Implements the
          * IDL:omg.org/CosNaming/NamingContext/rebind:1.0
          * operation
          */
         virtual void rebind(const CosNaming::Name& n,
                             CORBA::Object_ptr obj)
            throw(CosNaming::NamingContext::NotFound,
                  CosNaming::NamingContext::CannotProceed,
                  CosNaming::NamingContext::InvalidName,
                  CORBA::SystemException);
      
         /**
          * Purpose:
          * <p>
          * Implements the
          * IDL:omg.org/CosNaming/NamingContext/bind_context:1.0
          * operation
          */
         virtual void bind_context(const CosNaming::Name& n,
                                   CosNaming::NamingContext_ptr nc)
            throw(CosNaming::NamingContext::NotFound,
                  CosNaming::NamingContext::CannotProceed,
                  CosNaming::NamingContext::InvalidName,
                  CosNaming::NamingContext::AlreadyBound,
                  CORBA::SystemException);
      
         /**
          * Purpose:
          * <p>
          * Implements the
          * IDL:omg.org/CosNaming/NamingContext/rebind_context:1.0
          * operation
          */
         virtual void rebind_context(const CosNaming::Name& n,
                                     CosNaming::NamingContext_ptr nc)
            throw(CosNaming::NamingContext::NotFound,
                  CosNaming::NamingContext::CannotProceed,
                  CosNaming::NamingContext::InvalidName,
                  CORBA::SystemException);
      
         /**
          * Purpose:
          * <p>
          * Implements the
          * IDL:omg.org/CosNaming/NamingContext/resolve:1.0
          * operation
          */
         virtual CORBA::Object_ptr resolve(const CosNaming::Name& n)
            throw(CosNaming::NamingContext::NotFound,
                  CosNaming::NamingContext::CannotProceed,
                  CosNaming::NamingContext::InvalidName,
                  CORBA::SystemException);
      
         /**
          * Purpose:
          * <p>
          * Implements the
          * IDL:omg.org/CosNaming/NamingContext/unbind:1.0
          * operation
          */
         virtual void unbind(const CosNaming::Name& n)
            throw(CosNaming::NamingContext::NotFound,
                  CosNaming::NamingContext::CannotProceed,
                  CosNaming::NamingContext::InvalidName,
                  CORBA::SystemException);
      
         /**
          * Purpose:
          * <p>
          * Implements the
          * IDL:omg.org/CosNaming/NamingContext/new_context:1.0
          * operation
          */
         virtual CosNaming::NamingContext_ptr new_context()
            throw(CORBA::SystemException);
      
         /**
          * Purpose:
          * <p>
          * Implements the
          * IDL:omg.org/CosNaming/NamingContext/bind_new_context:1.0
          * operation
          */
         virtual CosNaming::NamingContext_ptr bind_new_context(const CosNaming::Name& n)
            throw(CosNaming::NamingContext::NotFound,
                  CosNaming::NamingContext::AlreadyBound,
                  CosNaming::NamingContext::CannotProceed,
                  CosNaming::NamingContext::InvalidName,
                  CORBA::SystemException);
      
         /**
          * Purpose:
          * <p>
          * Implements the
          * IDL:omg.org/CosNaming/NamingContext/destroy:1.0
          * operation
          */
         virtual void destroy()
            throw(CosNaming::NamingContext::NotEmpty,
                  CORBA::SystemException);
      
         /**
          * Purpose:
          * <p>
          * Implements the
          * IDL:omg.org/CosNaming/NamingContext/list:1.0
          * operation
          */
         virtual void list(CORBA::ULong how_many,
                           CosNaming::BindingList_out bl,
                           CosNaming::BindingIterator_out bi)
            throw(CORBA::SystemException);
      
      
         /**
          * Purpose:
          * <p>
          * Implements the
          * IDL:omg.org/CosNaming/NamingContextExt/to_string:1.0
          * operation
          */
         virtual CosNaming::NamingContextExt::StringName to_string(const CosNaming::Name& n)
            throw(CosNaming::NamingContext::InvalidName,
                  CORBA::SystemException);
      
         /**
          * Purpose:
          * <p>
          * Implements the
          * IDL:omg.org/CosNaming/NamingContextExt/to_name:1.0
          * operation
          */
         virtual CosNaming::Name* to_name(const char* sn)
            throw(CosNaming::NamingContext::InvalidName,
                  CORBA::SystemException);
      
         /**
          * Purpose:
          * <p>
          * Implements the
          * IDL:omg.org/CosNaming/NamingContextExt/to_url:1.0
          * operation
          *
          * Not yet implemented.
          */
         virtual CosNaming::NamingContextExt::URLString to_url(const char* addr,
                                                               const char* sn)
            throw(CosNaming::NamingContextExt::InvalidAddress,
                  CosNaming::NamingContext::InvalidName,
                  CORBA::SystemException);
      
         /**
          * Purpose:
          * <p>
          * Implements the
          * IDL:omg.org/CosNaming/NamingContextExt/resolve_str:1.0
          * operation
          */
         virtual CORBA::Object_ptr resolve_str(const char* sn)
            throw(CosNaming::NamingContext::NotFound,
                  CosNaming::NamingContext::CannotProceed,
                  CosNaming::NamingContext::InvalidName,
                  CORBA::SystemException);
      protected:
         /**
          * @see Cdmw::CommonSvcs::Naming::NamingInterface::NameComponent_to_string()
          * @exception CosNaming::NamingContext::InvalidName Failure to allocate memory for the output string.
          */
         std::string NameComponent_to_string(const CosNaming::NameComponent & nc) const
            throw (CosNaming::NamingContext::InvalidName);
         CORBA::ULong check_empty_name(const CosNaming::Name& n) const
            throw (CosNaming::NamingContext::InvalidName);
         CosNaming::NamingContextExt_ptr
         get_target_context(const CosNaming::Name& n)
            throw (CosNaming::NamingContext::NotFound,
                   CosNaming::NamingContext::CannotProceed,
                   CosNaming::NamingContext::InvalidName,
                   CORBA::SystemException);

//         virtual CORBA::Object_ptr find(const char*) const throw() = 0;
         virtual CORBA::Object_ptr 
         resolve(const CosNaming::NameComponent& n) 
            throw(CosNaming::NamingContext::NotFound,
                  CosNaming::NamingContext::CannotProceed,
                  CosNaming::NamingContext::InvalidName,
                  CORBA::SystemException) = 0;

         virtual BindingsMap get_all_bindings() const  = 0;
         // Default behaviour
         virtual void bind(const CosNaming::NameComponent& n,
                           CORBA::Object_ptr obj)
            throw(CosNaming::NamingContext::CannotProceed,
                  CosNaming::NamingContext::InvalidName,
                  CosNaming::NamingContext::AlreadyBound,
                  CORBA::SystemException);
         virtual void rebind(const CosNaming::NameComponent& n,
                             CORBA::Object_ptr obj)
            throw(CosNaming::NamingContext::CannotProceed,
                  CosNaming::NamingContext::InvalidName,
                  CORBA::SystemException);
         virtual void bind_context(const CosNaming::NameComponent& n,
                                   CosNaming::NamingContext_ptr nc)
            throw(CosNaming::NamingContext::CannotProceed,
                  CosNaming::NamingContext::InvalidName,
                  CosNaming::NamingContext::AlreadyBound,
                  CORBA::SystemException);
         virtual void rebind_context(const CosNaming::NameComponent& n,
                                     CosNaming::NamingContext_ptr nc)
            throw(CosNaming::NamingContext::CannotProceed,
                  CosNaming::NamingContext::InvalidName,
                  CORBA::SystemException);
         virtual void unbind(const CosNaming::NameComponent& n)
            throw(CosNaming::NamingContext::NotFound,
                  CosNaming::NamingContext::CannotProceed,
                  CosNaming::NamingContext::InvalidName,
                  CORBA::SystemException);
         virtual CosNaming::NamingContext_ptr bind_new_context(const CosNaming::NameComponent& n)
            throw(CosNaming::NamingContext::AlreadyBound,
                  CosNaming::NamingContext::CannotProceed,
                  CosNaming::NamingContext::InvalidName,
                  CORBA::SystemException);

      private:

   }; //End of RONamingContextExt_impl

} // End of namespace Naming
} // End of namespace CommonSvcs
} // End of namespace Cdmw

#endif // INCL_CDMW_COMMNSVCS_NAMING_RONAMINGCONTEXTEXT_IMPL_HPP

