/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/commonsvcs/naming/RONamingContextExt_impl.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Foundation/common/Assert.hpp>

#include <iostream>


namespace Cdmw
{
namespace CommonSvcs
{
namespace Naming
{

RONamingContextExt_impl::RONamingContextExt_impl()
{
}

RONamingContextExt_impl::~RONamingContextExt_impl()
{
}

void RONamingContextExt_impl::bind(const CosNaming::Name& n,
                  CORBA::Object_ptr obj)
    throw(CosNaming::NamingContext::NotFound,
          CosNaming::NamingContext::CannotProceed,
          CosNaming::NamingContext::InvalidName,
          CosNaming::NamingContext::AlreadyBound,
          CORBA::SystemException)
{  
   CORBA::ULong len = check_empty_name(n);
   if (len == 1)
      bind(n[0L],obj);
   else  {
      CosNaming::NamingContextExt_var nc__ = get_target_context(n);
      CosNaming::Name basename(1);
      basename.length(1);
      basename[0] = n[len -1];
      nc__->bind(basename,obj);
   }
}


void RONamingContextExt_impl::rebind(const CosNaming::Name& n,
                    CORBA::Object_ptr obj)
    throw(CosNaming::NamingContext::NotFound,
          CosNaming::NamingContext::CannotProceed,
          CosNaming::NamingContext::InvalidName,
          CORBA::SystemException)
{
   CORBA::ULong len = check_empty_name(n);
   if (len == 1)
      rebind(n[0L],obj);
   else  {
      CosNaming::NamingContextExt_var nc__ = get_target_context(n);
      CosNaming::Name basename(1);
      basename.length(1);
      basename[0] = n[len -1];
      nc__->rebind(basename,obj);
   }
}


void RONamingContextExt_impl::bind_context(const CosNaming::Name& n,
                          CosNaming::NamingContext_ptr nc)
    throw(CosNaming::NamingContext::NotFound,
          CosNaming::NamingContext::CannotProceed,
          CosNaming::NamingContext::InvalidName,
          CosNaming::NamingContext::AlreadyBound,
          CORBA::SystemException)
{
   CORBA::ULong len = check_empty_name(n);
   if (len == 1)
      bind_context(n[0L],nc);
   else  {
      CosNaming::NamingContextExt_var nc__ = get_target_context(n);
      CosNaming::Name basename(1);
      basename.length(1);
      basename[0] = n[len -1];
      nc__->bind_context(basename,nc);
   }
}


void RONamingContextExt_impl::rebind_context(const CosNaming::Name& n,
                            CosNaming::NamingContext_ptr nc)
    throw(CosNaming::NamingContext::NotFound,
          CosNaming::NamingContext::CannotProceed,
          CosNaming::NamingContext::InvalidName,
          CORBA::SystemException)
{
   CORBA::ULong len = check_empty_name(n);
   if (len == 1)
      rebind_context(n[0L],nc);
   else  {
      CosNaming::NamingContextExt_var nc__ = get_target_context(n);
      CosNaming::Name basename(1);
      basename.length(1);
      basename[0] = n[len -1];
      nc__->rebind_context(basename,nc);
   }
}


CORBA::Object_ptr RONamingContextExt_impl::resolve(const CosNaming::Name& n)
   throw(CosNaming::NamingContext::NotFound,
         CosNaming::NamingContext::CannotProceed,
         CosNaming::NamingContext::InvalidName,
         CORBA::SystemException)
{        
   CORBA::ULong len = check_empty_name(n);

   CORBA::Object_var obj = this->resolve(n[0]);
   // Allow return of nil object reference if allowed by application
   // if len > 1 a NotFound exception will be raise after the _narrow bellow.
   
   if (len > 1)
   {
      CosNaming::Name name(len - 1);
      name.length(len - 1);
      
      for (unsigned int i = 1; i< len; i++)
      {
         name[i-1] = n[i];
      }
      
      CosNaming::NamingContextExt_var nc = CosNaming::NamingContextExt::_narrow(obj.in());
      if (CORBA::is_nil(nc.in()))
      {     
         CosNaming::NamingContext::NotFoundReason reason = 
            CosNaming::NamingContext::not_context;
         CosNaming::NamingContext::NotFound ex(reason, n);
         throw ex;
      }
      
      obj = nc->resolve(name);
   }
   
   return obj._retn();
}
    


void RONamingContextExt_impl::unbind(const CosNaming::Name& n)
    throw(CosNaming::NamingContext::NotFound,
          CosNaming::NamingContext::CannotProceed,
          CosNaming::NamingContext::InvalidName,
          CORBA::SystemException)
{
   CORBA::ULong len = check_empty_name(n);
   if (len == 1)
      unbind(n[0L]);
   else  {
      CosNaming::NamingContextExt_var nc__ = get_target_context(n);
      CosNaming::Name basename(1);
      basename.length(1);
      basename[0] = n[len -1];
      nc__->unbind(basename);
   }
}


CosNaming::NamingContext_ptr RONamingContextExt_impl::new_context()
    throw(CORBA::SystemException)
{
   throw CORBA::NO_PERMISSION (OrbSupport::NO_PERMISSION, CORBA::COMPLETED_NO);
   return CosNaming::NamingContext::_nil();
}


CosNaming::NamingContext_ptr RONamingContextExt_impl::bind_new_context(const CosNaming::Name& n)
    throw(CosNaming::NamingContext::NotFound,
          CosNaming::NamingContext::AlreadyBound,
          CosNaming::NamingContext::CannotProceed,
          CosNaming::NamingContext::InvalidName,
          CORBA::SystemException)
{
   CORBA::ULong len = check_empty_name(n);
   if (len == 1)
      return bind_new_context(n[0L]);
   else  {
      CosNaming::NamingContextExt_var nc__ = get_target_context(n);
      CosNaming::Name basename(1);
      basename.length(1);
      basename[0] = n[len -1];
      return nc__->bind_new_context(basename);
   }
   
}

void RONamingContextExt_impl::bind(const CosNaming::NameComponent& n,
                                   CORBA::Object_ptr obj)
   throw(CosNaming::NamingContext::CannotProceed,
         CosNaming::NamingContext::InvalidName,
         CosNaming::NamingContext::AlreadyBound,
         CORBA::SystemException)
{
   throw CORBA::NO_PERMISSION (OrbSupport::NO_PERMISSION, CORBA::COMPLETED_NO);
}

void RONamingContextExt_impl::rebind(const CosNaming::NameComponent& n,
                                     CORBA::Object_ptr obj)
   throw(CosNaming::NamingContext::CannotProceed,
         CosNaming::NamingContext::InvalidName,
         CORBA::SystemException)
{
   throw CORBA::NO_PERMISSION (OrbSupport::NO_PERMISSION, CORBA::COMPLETED_NO); 
}

void RONamingContextExt_impl::bind_context(const CosNaming::NameComponent& n,
                                           CosNaming::NamingContext_ptr nc)
   throw(CosNaming::NamingContext::CannotProceed,
         CosNaming::NamingContext::InvalidName,
         CosNaming::NamingContext::AlreadyBound,
         CORBA::SystemException)
{
   throw CORBA::NO_PERMISSION (OrbSupport::NO_PERMISSION, CORBA::COMPLETED_NO); 
}
void RONamingContextExt_impl::rebind_context(const CosNaming::NameComponent& n,
                                             CosNaming::NamingContext_ptr nc)
   throw(CosNaming::NamingContext::CannotProceed,
         CosNaming::NamingContext::InvalidName,
         CORBA::SystemException)
{
   throw CORBA::NO_PERMISSION (OrbSupport::NO_PERMISSION, CORBA::COMPLETED_NO); 
}
void RONamingContextExt_impl::unbind(const CosNaming::NameComponent& n)
   throw(CosNaming::NamingContext::NotFound,
         CosNaming::NamingContext::CannotProceed,
         CosNaming::NamingContext::InvalidName,
         CORBA::SystemException)
{
   throw CORBA::NO_PERMISSION (OrbSupport::NO_PERMISSION, CORBA::COMPLETED_NO); 
}
CosNaming::NamingContext_ptr RONamingContextExt_impl:: bind_new_context(const CosNaming::NameComponent& n)
   throw(CosNaming::NamingContext::AlreadyBound,
         CosNaming::NamingContext::CannotProceed,
         CosNaming::NamingContext::InvalidName,
         CORBA::SystemException)
{
   CosNaming::NamingContext_var nc = new_context();
   bind_context(n,nc.in());
   return nc._retn();
}

void RONamingContextExt_impl::destroy()
    throw(CosNaming::NamingContext::NotEmpty,
          CORBA::SystemException)
{
   throw CORBA::NO_PERMISSION (OrbSupport::NO_PERMISSION, CORBA::COMPLETED_NO);  
}

void RONamingContextExt_impl::list(CORBA::ULong how_many,
                  CosNaming::BindingList_out bl,
                  CosNaming::BindingIterator_out bi)
    throw(CORBA::SystemException)
{
   BindingsMap all_bindings_map = this->get_all_bindings();
   
   CORBA::ULong nb_bindings = static_cast<CORBA::ULong>(all_bindings_map.size());
   CosNaming::BindingList_var complete_bl = new CosNaming::BindingList(nb_bindings);
   complete_bl->length(nb_bindings);

   CORBA::ULong i = 0;
   for (BindingsMap::const_iterator it = all_bindings_map.begin();
        it != all_bindings_map.end();
        ++it)
   {
      CosNaming::Binding binding;
      
      CosNaming::Name_var name = Cdmw::CommonSvcs::Naming::NamingInterface::to_name((*it).first);
      binding.binding_name = name.in();
      
      binding.binding_type = CosNaming::nobject;
      // Is it a Context
      try {
         CosNaming::NamingContext_var nc = CosNaming::NamingContext::_narrow((*it).second.in());
         if (!CORBA::is_nil(nc.in()))
            binding.binding_type = CosNaming::ncontext;
      } catch (...) {
         // ignore ...
      }
      
      complete_bl[i] = binding;
      
      ++i;
   }
   
   bl = complete_bl._retn();
   bi = CosNaming::BindingIterator::_nil();   
}


CosNaming::NamingContextExt::StringName RONamingContextExt_impl::to_string(
    const CosNaming::Name& n)
    throw(CosNaming::NamingContext::InvalidName,
          CORBA::SystemException)
{
   try
   {
      std::string str = Cdmw::CommonSvcs::Naming::NamingInterface::to_string(n);
      return CORBA::string_dup(str.c_str());
   }
   catch(const Cdmw::CommonSvcs::Naming::InvalidNameException &)
   {
      throw CosNaming::NamingContext::InvalidName();
   }
}


CosNaming::Name* RONamingContextExt_impl::to_name(const char* sn)
    throw(CosNaming::NamingContext::InvalidName,
          CORBA::SystemException)
{
   try
   {
      CosNaming::Name_var name = Cdmw::CommonSvcs::Naming::NamingInterface::to_name(sn);
      return name._retn();
   }
   catch(const Cdmw::CommonSvcs::Naming::InvalidNameException &)
   {
      throw CosNaming::NamingContext::InvalidName();
   }
}


CosNaming::NamingContextExt::URLString RONamingContextExt_impl::to_url(
   const char* addr, const char* sn)
   throw(CosNaming::NamingContextExt::InvalidAddress,
         CosNaming::NamingContext::InvalidName,
         CORBA::SystemException)
{
   throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);    
}


CORBA::Object_ptr RONamingContextExt_impl::resolve_str(const char* sn)
   throw(CosNaming::NamingContext::NotFound,
         CosNaming::NamingContext::CannotProceed,
         CosNaming::NamingContext::InvalidName,
         CORBA::SystemException)
{
   try {
      CosNaming::Name_var name = Cdmw::CommonSvcs::Naming::NamingInterface::to_name(sn);
      return resolve(name.in());
   } catch (const Cdmw::CommonSvcs::Naming::InvalidNameException & ) {
      throw CosNaming::NamingContext::InvalidName();
   }
}
   
std::string RONamingContextExt_impl::NameComponent_to_string(const CosNaming::NameComponent & nc) 
   const throw (CosNaming::NamingContext::InvalidName)
{
   try {
      return Cdmw::CommonSvcs::Naming::NamingInterface::NameComponent_to_string(nc);
   } catch (const std::bad_alloc &) {
      throw CosNaming::NamingContext::InvalidName();
   }
}

CORBA::ULong 
RONamingContextExt_impl::check_empty_name(const CosNaming::Name& n) const
   throw (CosNaming::NamingContext::InvalidName)
{
   CORBA::ULong len = n.length();   
   bool valid = true;
   if (len > 0) {
      // Check that there is no empty NameComponent
      for (CORBA::ULong i = 0L; i < len; i++) {
         if ((strlen(n[i].id.in()) == 0) &&
             (strlen(n[i].kind.in()) == 0))
            valid = false;
      }
   } else
      valid = false;
   if (!valid) // illegal name
      throw CosNaming::NamingContext::InvalidName();
   return len;
}


CosNaming::NamingContextExt_ptr
RONamingContextExt_impl::get_target_context(const CosNaming::Name& n)
      throw (CosNaming::NamingContext::NotFound,
             CosNaming::NamingContext::CannotProceed,
             CosNaming::NamingContext::InvalidName,
             CORBA::SystemException)
{
   CORBA::ULong lg = n.length();
   CDMW_ASSERT(lg > 1);
   lg--;
   CosNaming::Name dirname(lg, lg, const_cast<CosNaming::Name&>(n).get_buffer(), false);
   CORBA::Object_var obj = this->resolve(dirname);
   CosNaming::NamingContextExt_var nc = CosNaming::NamingContextExt::_narrow(obj.in());
   if (CORBA::is_nil(nc.in()))
   {     
      CosNaming::NamingContext::NotFoundReason reason = 
         CosNaming::NamingContext::not_context;
      CosNaming::NamingContext::NotFound ex(reason, dirname);
      throw ex;
   }
   return nc._retn();
}




} // End of namespace Naming
} // End of namespace CommonSvcs
} // End of namespace Cdmw

