/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "test_lib/TestNameParserToName.hpp"
#include "naming/NameParser.hpp"


namespace Cdmw
{
namespace CommonSvcs
{
namespace Naming
{

/**
*Implementation notes: [if no pertinent write none ]
*<p>
*Portability issues: [if no pertinent write none ]
*<p>
*/

CPPUNIT_TEST_SUITE_REGISTRATION( TestNameParserToName );

TestNameParserToName::TestNameParserToName()
    throw()
{
    // does nothing
}


TestNameParserToName::~TestNameParserToName()
    throw()
{
    // does nothing
}


TestNameParserToName::TestNameParserToName(const TestNameParserToName& rhs)
    throw()
{
    // does nothing
}
        
TestNameParserToName&
TestNameParserToName::operator=(const TestNameParserToName& rhs)
    throw()
{
    // does nothing
    return *this;
}


void TestNameParserToName::test_to_name()
{
    TEST_INFO("Test name components with both ID and Kind field");
    {
        using namespace CosNaming;
        std::string input("first.1st/second.2nd");
        try {
            Name_var name = CommonSvcs::Naming::NameParser::to_name(input);
            CPPUNIT_ASSERT(name->length()==2);
            CPPUNIT_ASSERT(strcmp((name.in())[0].id.in(),"first")==0);
            CPPUNIT_ASSERT(strcmp((name.in())[0].kind.in(),"1st")==0);
            CPPUNIT_ASSERT(strcmp((name.in())[1].id.in(),"second")==0);
            CPPUNIT_ASSERT(strcmp((name.in())[1].kind.in(),"2nd")==0);
        } catch (const CommonSvcs::Naming::InvalidNameException&) {
            CPPUNIT_ASSERT(false);
        }
    }
    
    TEST_INFO("Test name components with missing Kind field");
    {
        using namespace CosNaming;
        std::string input("first.1st/second");
        try {
            Name_var name = CommonSvcs::Naming::NameParser::to_name(input);
            CPPUNIT_ASSERT(name->length()==2);
            CPPUNIT_ASSERT(strcmp((name.in())[0].id.in(),"first")==0);
            CPPUNIT_ASSERT(strcmp((name.in())[0].kind.in(),"1st")==0);
            CPPUNIT_ASSERT(strcmp((name.in())[1].id.in(),"second")==0);
            CPPUNIT_ASSERT(strlen((name.in())[1].kind.in())==0);
        } catch (const CommonSvcs::Naming::InvalidNameException&) {
            CPPUNIT_ASSERT(false);
        }
    }

    TEST_INFO("Test name components with missing ID field");
    {
        using namespace CosNaming;
        std::string input("first.1st/.2nd");
        try {
            Name_var name = CommonSvcs::Naming::NameParser::to_name(input);
            CPPUNIT_ASSERT(name->length()==2);
            CPPUNIT_ASSERT(strcmp((name.in())[0].id.in(),"first")==0);
            CPPUNIT_ASSERT(strcmp((name.in())[0].kind.in(),"1st")==0);
            CPPUNIT_ASSERT(strlen((name.in())[1].id.in())==0);
            CPPUNIT_ASSERT(strcmp((name.in())[1].kind.in(),"2nd")==0);
        } catch (const CommonSvcs::Naming::InvalidNameException&) {
            CPPUNIT_ASSERT(false);
        }
    }

    TEST_INFO("Test node with empty ID and Kind field");
    {
        using namespace CosNaming;
        std::string input("first.1st/./third.3rd");
        try {
            Name_var name = CommonSvcs::Naming::NameParser::to_name(input);
            CPPUNIT_ASSERT(name->length()==3);
            CPPUNIT_ASSERT(strcmp((name.in())[0].id.in(),"first")==0);
            CPPUNIT_ASSERT(strcmp((name.in())[0].kind.in(),"1st")==0);
            CPPUNIT_ASSERT(strlen((name.in())[1].id.in())==0);
            CPPUNIT_ASSERT(strlen((name.in())[1].kind.in())==0);
            CPPUNIT_ASSERT(strcmp((name.in())[2].id.in(),"third")==0);
            CPPUNIT_ASSERT(strcmp((name.in())[2].kind.in(),"3rd")==0);
        } catch (const CommonSvcs::Naming::InvalidNameException&) {
            CPPUNIT_ASSERT(false);
        }
    }
    TEST_INFO("Test nodes with characters that are escaped");
    {
        using namespace CosNaming;
        std::string input("first.1st\\.dot/second\\.dot.2nd/third\\/slash.3rd\n");
        try {
            Name_var name = CommonSvcs::Naming::NameParser::to_name(input);
            CPPUNIT_ASSERT(name->length()==3);
            CPPUNIT_ASSERT(strcmp((name.in())[0].id.in(),"first")==0);
            CPPUNIT_ASSERT(strcmp((name.in())[0].kind.in(),"1st.dot")==0);
            CPPUNIT_ASSERT(strcmp((name.in())[1].id.in(),"second.dot")==0);
            CPPUNIT_ASSERT(strcmp((name.in())[1].kind.in(),"2nd")==0);
            CPPUNIT_ASSERT(strcmp((name.in())[2].id.in(),"third/slash")==0);
            CPPUNIT_ASSERT(strcmp((name.in())[2].kind.in(),"3rd\n")==0);
        } catch (const CommonSvcs::Naming::InvalidNameException&) {
            CPPUNIT_ASSERT(false);
        }
    }
    TEST_INFO("Test empty name");
    {
        using namespace CosNaming;
        std::string input(".");
        try {
            Name_var name = CommonSvcs::Naming::NameParser::to_name(input);
            CPPUNIT_ASSERT(name->length()==1);
            CPPUNIT_ASSERT(strlen((name.in())[0].id.in())==0);
            CPPUNIT_ASSERT(strlen((name.in())[0].kind.in())==0);
        } catch (const CommonSvcs::Naming::InvalidNameException&) {
            CPPUNIT_ASSERT(false);
        }
    }
    {
        using namespace CosNaming;
        std::string input("");
        try {
            Name_var name = CommonSvcs::Naming::NameParser::to_name(input);
            // The empty sequence is not a legal name
            CPPUNIT_ASSERT(false);
        } catch (const CommonSvcs::Naming::InvalidNameException&) {
            CPPUNIT_ASSERT(true);
        }
    }
    TEST_INFO("Test ending '.'");
    {
        using namespace CosNaming;
        std::string input("first.1st/second.");
        try {
            Name_var name = CommonSvcs::Naming::NameParser::to_name(input);
            // If a name component has a non-empty id field and an empty kind field, 
            // the stringified representation consists only of the id field. A trailing 
            // '.' character is not permitted.
            CPPUNIT_ASSERT(false);
        } catch (const CommonSvcs::Naming::InvalidNameException&) {
            CPPUNIT_ASSERT(true);
        }
    }
    TEST_INFO("Test starting '/'");
    {
        using namespace CosNaming;
        std::string input("/first.1st/second.2nd");
        try {
            Name_var name = CommonSvcs::Naming::NameParser::to_name(input);
            // The single '.' character is the only representation of a name 
            // component with empty id and kind fields.
            CPPUNIT_ASSERT(false);
        } catch (const CommonSvcs::Naming::InvalidNameException&) {
            CPPUNIT_ASSERT(true);
        }
    }

}

void TestNameParserToName::do_tests()       
{    
    // This tests NameParser class
    //-------------------------------------------------
    // Testing string to name conversion
    //-------------------------------------------------
    // set number of requested successfull tests
//     set_nbOfRequestedTestOK (35);
    
    TEST_INFO("Testing NameParser::to_name operation");
    test_to_name();
}
        
} // End namespace Naming
} // End namespace CommonSvcs
} // End namespace Cdmw

