/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "test_lib/TestNamingInterfaceDestroy.hpp"
#include "test_lib/TestMacroDefs.hpp"
#include "Foundation/commonsvcs/naming/NamingInterface.hpp"
#include "Foundation/commonsvcs/naming/NamingUtil.hpp"

namespace 
{
    typedef Cdmw::CommonSvcs::Naming::NamingUtil<CosNaming::NamingContext> Util;

    bool 
    install_initial_bindings(Cdmw::CommonSvcs::Naming::NamingInterface& n,
                             const std::string &                      base,
                             CORBA::Object_ptr                        obj)
    {    
        bool result = true;
        try {
            Cdmw::CommonSvcs::Naming::NamingInterface ni = n.bind_new_context(base,true);
            
            ni.bind("bin/CVS/Entries",obj,true);
            ni.bind("bin/CVS/Repository",obj,true);
            ni.bind("bin/CVS/Root",obj,true);
            ni.bind("bin/Makefile",obj,true);
            ni.bind("bin/Makefile.mak",obj,true);
            ni.bind("bin/Makefile.tmpl",obj,true);
            ni.bind("bin/mk_idl_include.sh",obj,true);
            ni.bind("bin/CdmwCreateClass",obj,true);
            ni.bind("bin/CdmwCreateCSC",obj,true);
            ni.bind("commonsvcs/naming/c++/CVS/Entries",obj,true);
            ni.bind("commonsvcs/naming/c++/CVS/Entries.Log",obj,true);
            ni.bind("commonsvcs/naming/c++/CVS/Repository",obj,true);
            ni.bind("commonsvcs/naming/c++/CVS/Root",obj,true);
            ni.bind("commonsvcs/naming/c++/incl/CVS/Entries",obj,true);
            ni.bind("commonsvcs/naming/c++/incl/CVS/Repository",obj,true);
            ni.bind("commonsvcs/naming/c++/incl/CVS/Root",obj,true);
            ni.bind("commonsvcs/naming/c++/incl/NameParser.hpp",obj,true);
            ni.bind("commonsvcs/naming/c++/incl/NameParserContext.hpp",obj,true);
            ni.bind("commonsvcs/naming/c++/incl/NameParserFSM.hpp",obj,true);
            ni.bind("commonsvcs/naming/c++/src/CSC.mk",obj,true);
            ni.bind("commonsvcs/naming/c++/src/CVS/Entries",obj,true);
            ni.bind("commonsvcs/naming/c++/src/CVS/Repository",obj,true);
            ni.bind("commonsvcs/naming/c++/src/CVS/Root",obj,true);
            ni.bind("commonsvcs/naming/c++/src/InvalidName.cpp",obj,true);
            ni.bind("commonsvcs/naming/c++/src/main.cpp",obj,true);
            ni.bind("commonsvcs/naming/c++/src/NameParser.cpp",obj,true);
            ni.bind("commonsvcs/naming/c++/src/NameParserContext.cpp",obj,true);
            ni.bind("commonsvcs/naming/c++/src/NameParserFSM.cpp",obj,true);
            ni.bind("commonsvcs/naming/c++/src/NamingInterface.cpp",obj,true);
            Cdmw::CommonSvcs::Naming::NamingInterface n 
                = ni.bind_new_context("commonsvcs/naming/ada.1",true);  
            n   = ni.bind_new_context("commonsvcs/naming/ada.2",true);  
        } catch (...) {
            NI_TEST_CERR("Unexpected error (could'nt install initial bindings)"
                         << "\n(file:" << __FILE__ << " line:" << __LINE__ << ")");
            result = false;
        }
        return result;
    }

};

namespace Cdmw
{
namespace CommonSvcs
{
namespace Naming
{

/**
*Implementation notes: [if no pertinent write none ]
*<p>
*Portability issues: [if no pertinent write none ]
*<p>
*/

CPPUNIT_TEST_SUITE_REGISTRATION( TestNamingInterfaceDestroy );

TestNamingInterfaceDestroy::TestNamingInterfaceDestroy ()
    throw()
{
    CORBA::Object_var obj1 = Cdmw::TestUtils::CORBATestManager::instance()->get_object("context");
    m_context = CosNaming::NamingContext::_narrow(obj1.in());
    CORBA::Object_var obj2 = Cdmw::TestUtils::CORBATestManager::instance()->get_object("object");
    m_test_object = test::TestObject::_narrow(obj2.in());
}


TestNamingInterfaceDestroy::~TestNamingInterfaceDestroy()
    throw()
{
    // does nothing
}


TestNamingInterfaceDestroy::TestNamingInterfaceDestroy(const TestNamingInterfaceDestroy& rhs)
    throw()
    :m_context(CosNaming::NamingContext::_duplicate(rhs.m_context.in())),
     m_test_object(test::TestObject::_duplicate(rhs.m_test_object.in()))
{
    // does nothing
}
        
TestNamingInterfaceDestroy&
TestNamingInterfaceDestroy::operator=(const TestNamingInterfaceDestroy& rhs)
    throw()
{
    // does nothing
    return *this;
}


void TestNamingInterfaceDestroy::do_tests()     
{
    // set number of requested successfull tests
//     set_nbOfRequestedTestOK (10);
    
    NamingInterface ni0(m_context.in());
    
    // This tests NamingInterface class
    //-------------------------------------------------
    // Testing destroy operation
    //-------------------------------------------------
    TEST_INFO("Testing NamingInterface::destroy operation");
    using namespace CosNaming;      
    
    const std::string base("test.destroy/runtime");
        
    // Install test name tree
    if (!install_initial_bindings(ni0, base, m_test_object.in())) {
        CPPUNIT_ASSERT(false);
        return;
    }
    // Install test name tree
//     if (!install_initial_bindings(ni0, "toto/test.destroy/runtime", m_test_object.in())) {
//      CPPUNIT_ASSERT(false);
//      return;
//     }
    // 
    // Additional test to test NamingUtil<>::resolve_name() with a 'NotFound' context
    //
    TEST_INFO("Additional tests to NamingUtil<>::resolve_name() function");
    {
        std::string input(base);
        // Testing NotFound Exception
        input += "/commonsvcs/naming/dummy/ada.1";
        try {
            NamingContext_var nc = Util::resolve_name(ni0,input);
            CPPUNIT_ASSERT(false);
        } catch (const CosNaming::NamingContext::NotFound&) {
            CPPUNIT_ASSERT(true);
        } catch (...) {
            CPPUNIT_ASSERT(false);
        }
        // Testing TypeMismatch Exception
        input = base + "/bin/CVS/Root";
        try {
            NamingContext_var nc = Util::resolve_name(ni0,input);
            CPPUNIT_ASSERT(false);
        } catch (const Cdmw::Common::TypeMismatchException &) {
            CPPUNIT_ASSERT(true);
        } catch (...) {
            CPPUNIT_ASSERT(false);
        }
    }
    // 
    // Destroy commonsvcs/naming/ada.1 naming context
    //
    TEST_INFO("Destroy test.destroy/runtime/commonsvcs/naming/ada.1 (empty) naming context");
    {
        std::string input(base);
        input += "/commonsvcs/naming/ada.1";
        try {
            NamingContext_var nc = Util::resolve_name(ni0,input);
            NamingInterface n(nc.in());
            n.destroy();
            ni0.unbind(input);
            CPPUNIT_ASSERT(true);
        } catch (...) {
            CPPUNIT_ASSERT(false);
        }
        // Forcing destroy
        input = base + "/commonsvcs/naming/ada.2";
        try {
            NamingContext_var nc = Util::resolve_name(ni0,input);
            NamingInterface n(nc.in());
            bool destroyed = n.destroy_all();
            CPPUNIT_ASSERT(destroyed);
            if (destroyed) ni0.unbind(input);
            CPPUNIT_ASSERT(true);
        } catch (...) {
            CPPUNIT_ASSERT(false);
        }
    }

    //
    // Try to destroy a context that's not empty
    //    
    TEST_INFO("Test of destroying a context that's not empty");
    {
        std::string input(base);
        NamingContext_var nc;
        try {
            nc = Util::resolve_name(ni0,input);
            NamingInterface n(nc.in());
            n.destroy();
            CPPUNIT_ASSERT(false);
        } catch (const NamingContext::NotEmpty&) {
            CPPUNIT_ASSERT(true);
        } catch (...) {
            CPPUNIT_ASSERT(false);
        }
        // Forcing destroy
        try {
            NamingInterface n(nc.in());
            CPPUNIT_ASSERT(n.destroy_all());
        } catch (...) {
            CPPUNIT_ASSERT(false);
        }
        // Is it destroyed ?
        try {
            Name_var name = NamingInterface::to_name("toto");
    // The following test (replaced by the bind_new_context) produce
    // a naming&repository AssertionFailedException, Cf PCR-0062
    //      ni0.bind(base + "/toto", m_test_object.in(), false);
            NamingContext_var nc__ = nc->bind_new_context(name.in());
            CPPUNIT_ASSERT(false);
        } catch (const CORBA::OBJECT_NOT_EXIST&) {
            CPPUNIT_ASSERT(true);
            // remove binding
            try {
                ni0.unbind(input);
            } catch (...) {
                // Unexpected error
                CPPUNIT_ASSERT(false);
            }
        } catch (...) {
            CPPUNIT_ASSERT(false);
        }
        
    }
    //
    // Try to destroy a context that's not empty with an undestroyable context
    //       
    // Install test name tree
    if (!install_initial_bindings(ni0, base, m_test_object.in())) {
        CPPUNIT_ASSERT(false);
        return;
    }
    TEST_INFO("Test of destroying a context that's not empty with an undestroyable context");
    {
        // To simulate an undestroyable context, we destroy a context without
        // removing the binding. This will generate a system exception when its
        // destruction is requested.
        std::string input(base);
        input += "/commonsvcs/naming/ada.1";
        try {
            NamingContext_var nc = Util::resolve_name(ni0,input);
            NamingInterface n(nc.in());
            n.destroy();
        } catch (...) {
            CPPUNIT_ASSERT(false);
            return;
        }
    
        // Forcing destroy      
        NamingContext_var nc;
        try {
            nc = Util::resolve_name(ni0,base);
        } catch (...) {
            CPPUNIT_ASSERT(false);
            return;
        }       
        try {
            NamingInterface n(nc.in());
            CPPUNIT_ASSERT(!n.destroy_all());
        } catch (...) {
            CPPUNIT_ASSERT(false);
        }       
        // Is it destroyed
        try {
            NamingInterface n(nc.in());
            NamingInterface::BindingList bnd_list;      
            n.list_context(bnd_list);
            CPPUNIT_ASSERT(bnd_list.size() == 1);
        } catch (...) {
            CPPUNIT_ASSERT(false);
        }       
    }   
}

        
} // End namespace Naming
} // End namespace CommonSvcs
} // End namespace Cdmw

