/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.commonsvcs.naming.test;

import cdmw.commonsvcs.naming.InvalidNameException;
import cdmw.commonsvcs.naming.NamingInterface;
import cdmw.testutils.CORBATestManager;
import cdmw.testutils.CORBATestManager.InvalidObject;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.omg.CORBA.SystemException;
import org.omg.CosNaming.Binding;
import org.omg.CosNaming.BindingHolder;
import org.omg.CosNaming.BindingIterator;
import org.omg.CosNaming.BindingIteratorHolder;
import org.omg.CosNaming.BindingListHolder;
import org.omg.CosNaming.NamingContext;
import org.omg.CosNaming.NamingContextHelper;
import org.omg.CosNaming.NameComponent;

import org.omg.CosNaming.NamingContextPackage.NotFound;
import org.omg.CosNaming.NamingContextPackage.CannotProceed;
import org.omg.CosNaming.NamingContextPackage.InvalidName;
import org.omg.CosNaming.NamingContextPackage.AlreadyBound;
import org.omg.CosNaming.NamingContextPackage.NotEmpty;
import org.omg.CosNaming.NamingContextPackage.NotFoundReason;

public class BindContextTest {
    private static final String NAME_CONTEXT = "cdmw.test";
    private static final String NAME_LONG_CONTEXT = NAME_CONTEXT
        + "/runtime/commonsvcs/naming/c++/test/idl";
    private static final String NAME_OBJECT = NAME_CONTEXT + "/runtime/config.mk";
    private static final String NAME_CONTEXT_OVER_OBJECT = NAME_OBJECT + "/bla.bla";
    private static final String NAME_BOUND_CONTEXT = NAME_CONTEXT
        + "/runtime/commonsvcs/naming/c++";
    private static final String NAME_ARCHIVE_CONTEXT = NAME_CONTEXT
        + "/archive/runtime/commonsvcs/naming/c++";
    private static final String NAME_ARCHIVE_CONTEXT_OBJECT = NAME_CONTEXT
        + "/archive/orbacus.mk";
    private static final String NAME_ARCHIVE_CONTEXT_AFTER_CONTEXT_OBJECT =
            NAME_ARCHIVE_CONTEXT_OBJECT + "/runtime/commonsvcs/naming/c++";

    private static NamingContext namingContext;
    private static TestObject testObject;

    private NamingInterface ni0;

    @BeforeClass
    public static void initTests() throws InvalidObject {
        CORBATestManager testManager = CORBATestManager.instance();

        org.omg.CORBA.Object obj = testManager.getObject("NameService");
        namingContext = org.omg.CosNaming.NamingContextHelper.narrow(obj);
        
        obj = testManager.getObject("TestObject");
        testObject = TestObjectHelper.narrow(obj);
    }

    @AfterClass
    public static void finiTests() {
        namingContext = null;
        testObject = null;
    }

    @Before
    public void init() {
        this.ni0 = new NamingInterface(namingContext);
    }

    @After
    public void fini() {
        // Clear context from bindings
        try {
            BindingListHolder listHolder = new BindingListHolder();
            BindingIteratorHolder iteratorHolder = new BindingIteratorHolder();
            namingContext.list(0, listHolder, iteratorHolder);
            BindingIterator iterator = iteratorHolder.value;
            if (iterator != null) {
                BindingHolder bindingHolder = new BindingHolder();
    
                while (iterator.next_one(bindingHolder)) {
                    Binding binding = bindingHolder.value;
                    namingContext.unbind(binding.binding_name);
                }
    
                iterator.destroy();
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            this.ni0 = null;
        }
    }

    @Test
    public void bindNewContext()
        throws NotFound, CannotProceed, InvalidName {
        // This tests NamingInterface class
        // -------------------------------------------------
        // Testing bind_new_context operation
        // -------------------------------------------------
        System.out
            .println("Testing NamingInterface::bind_new_context operation");

        // This test creates the following bindings in the naming service:
        // cdmw.test/runtime/config.mk
        // cdmw.test/runtime/commonsvcs/naming/c++/test/idl
        System.out.println("Creating cdmw.test binding");
        try {
            NamingInterface ni = this.ni0.bindNewContext(NAME_CONTEXT, false);
            org.omg.CORBA.Object obj = this.ni0.resolve(NAME_CONTEXT);
            assertTrue(obj != null);
        }
        catch (AlreadyBound ab) {
            System.out
                .println("cdmw.test context should be cleared before testing.");
            fail();
        }
    }

    @Test(expected = AlreadyBound.class)
    public void duplicateBindWithoutForce()
        throws NotFound, AlreadyBound, CannotProceed, InvalidName {
        try {
            NamingInterface ni = this.ni0.bindNewContext(NAME_CONTEXT, false);
        }
        catch (Exception e) {
            fail();
        }
        NamingInterface ni = this.ni0.bindNewContext(NAME_CONTEXT, false);
        fail();
    }

    @Test(expected = AlreadyBound.class)
    public void duplicateBindWithForce()
        throws NotFound, AlreadyBound, CannotProceed, InvalidName {
        try {
            NamingInterface ni = this.ni0.bindNewContext(NAME_CONTEXT, false);
        }
        catch (Exception e) {
            fail();
        }
        NamingInterface ni = this.ni0.bindNewContext(NAME_CONTEXT, true);
        fail();
    }

    @Test(expected = NotFound.class)
    public void bindCompoundNameWithoutForce()
        throws NotFound, AlreadyBound, CannotProceed, InvalidName {
        System.out
            .println("Creating cdmw.test/runtime/commonsvcs/naming/c++/test/idl binding");
        NamingInterface ni = this.ni0.bindNewContext(NAME_LONG_CONTEXT, false);
        fail();
    }

    @Test
    public void bindCompoundNameWithForce()
        throws NotFound, AlreadyBound, CannotProceed, InvalidName {
        //
        // Try to bind a context when an intermediate context
        // doesn't exist (Forcing creation)
        //
        NamingInterface ni = this.ni0.bindNewContext(NAME_LONG_CONTEXT, true);
        org.omg.CORBA.Object obj = this.ni0.resolve(NAME_LONG_CONTEXT);
        assertTrue(obj != null);
    }

    @Test
    public void bindObject()
        throws InvalidNameException, NotFound, CannotProceed, AlreadyBound {
        //
        // Try to bind a context when an object is in the path
        //
        // cdmw.test/runtime/config.mk
        System.out.println("Binding a context when an object is in the path");

        try {
            NamingInterface ni = this.ni0.bindNewContext(NAME_LONG_CONTEXT, true);
            NameComponent[] name = NamingInterface.toName(NAME_OBJECT);
            namingContext.bind(name, testObject);
        }
        catch (InvalidName in) {
            fail();
        }
    }

    @Test(expected = AlreadyBound.class)
    public void rebindContextOverBoundObject()
        throws InvalidNameException, NotFound, CannotProceed, InvalidName, AlreadyBound {
        try {
            NamingInterface ni = this.ni0.bindNewContext(NAME_LONG_CONTEXT, true);
            NameComponent[] name = NamingInterface.toName(NAME_OBJECT);
            namingContext.bind(name, testObject);
        }
        catch (AlreadyBound ab) {
            fail();
        }
        NamingInterface ni = ni0.bindNewContext(NAME_OBJECT, true);
        fail();
    }

    @Test(expected = NotFound.class)
    public void rebindCompoundContextNameOverBoundObject()
        throws InvalidNameException, AlreadyBound, NotFound, CannotProceed, InvalidName {
        try {
            NamingInterface ni = this.ni0.bindNewContext(NAME_LONG_CONTEXT, true);
            NameComponent[] name = NamingInterface.toName(NAME_OBJECT);
            namingContext.bind(name, testObject);
        }
        catch (NotFound nf) {
            fail();
        }
        NamingInterface ni = ni0.bindNewContext(NAME_CONTEXT_OVER_OBJECT, true);
        fail();
    }    


    @Test
    public void resolveBoundContext()
        throws NotFound, CannotProceed, InvalidName, AlreadyBound,  InvalidName {
        //-------------------------------------------------
        // Testing bind_context operation
        //-------------------------------------------------
        System.out.println("Testing NamingInterface#bindContext operation");
    
        // This test creates the following bindings in the naming service:
        // cdmw.test/archive/orbacus.mk
        // cdmw.test/archive/commonsvcs/naming/c++
        // (link to cdmw.test/runtime/commonsvcs/naming/c++)
        NamingInterface ni = this.ni0.bindNewContext(NAME_LONG_CONTEXT, true);
    
        // Get a reference to the naming context bound to
        // cdmw.test/runtime/commonsvcs/naming/c++
        NamingContext nc = null;
        // resolve has already been tested
        org.omg.CORBA.Object obj = this.ni0.resolve(NAME_BOUND_CONTEXT);
        nc = NamingContextHelper.narrow(obj);
        assertTrue (nc!=null);
    }
    
    @Test
    public void bindCompoundNameContextWithoutForce()
        throws CannotProceed, NotFound, InvalidName, AlreadyBound {
        NamingContext nc = null;

        NamingInterface ni = this.ni0.bindNewContext(NAME_BOUND_CONTEXT, true);
        nc = ni.getNamingContext();

        //
        // Try to bind a context when an intermediate context doesn't exist
        //
        System.out.println("Attempt to bind " + NAME_ARCHIVE_CONTEXT);
        try {
            this.ni0.bindContext(NAME_ARCHIVE_CONTEXT, nc, false);
            fail();
        } catch (NotFound nf) {
            assertTrue( nf.why.equals(NotFoundReason.missing_node) );
        }
    }
    
    @Test
    public void bindCompoundNameContextWithForce()
        throws NotFound, CannotProceed, InvalidName, AlreadyBound {
        NamingContext nc = null;

        // resolve has already been tested
        NamingInterface ni = this.ni0.bindNewContext(NAME_LONG_CONTEXT, true);
        org.omg.CORBA.Object obj = this.ni0.resolve(NAME_BOUND_CONTEXT);
        nc = NamingContextHelper.narrow(obj);
        
        // Forcing creation
        System.out.println("Forcing bind of" + NAME_ARCHIVE_CONTEXT);
        this.ni0.bindContext(NAME_ARCHIVE_CONTEXT, nc, true);
        obj = this.ni0.resolve(NAME_ARCHIVE_CONTEXT);
        assertTrue(obj!=null);
    }
    
    @Test
    public void bindObjectOnExistingContext()
        throws InvalidNameException, NotFound, CannotProceed, InvalidName, AlreadyBound {
        NamingContext nc = null;

        // resolve has already been tested
        NamingInterface ni = this.ni0.bindNewContext(NAME_LONG_CONTEXT, true);
        org.omg.CORBA.Object obj = this.ni0.resolve(NAME_BOUND_CONTEXT);
        nc = NamingContextHelper.narrow(obj);
        this.ni0.bindContext(NAME_ARCHIVE_CONTEXT, nc, true);

        //
        // Try to bind a context when an object is in the path
        //
        // cdmw.test/archive/orbacus.mk
        System.out.println("Binding a context when an object is in the path");
        NameComponent[] name = NamingInterface.toName(NAME_ARCHIVE_CONTEXT_OBJECT);
        namingContext.bind(name, testObject);
    }
    
    @Test
    public void bindCompoundNameArchiveContextWithoutForce()
        throws CannotProceed, NotFound, InvalidName, AlreadyBound, InvalidNameException {

        // resolve has already been tested
        NamingInterface ni = this.ni0.bindNewContext(NAME_CONTEXT, true);
        NamingContext nc = ni.getNamingContext();
        this.ni0.bindContext(NAME_ARCHIVE_CONTEXT, nc, true);
        
        NameComponent[] name = NamingInterface.toName(NAME_ARCHIVE_CONTEXT_OBJECT);
        namingContext.bind(name, testObject);

        // Expecting NamingContext::NotFound exception
        // ex.why == NamingContext::not_context
        System.out.println("Attempt to bind " + NAME_ARCHIVE_CONTEXT_AFTER_CONTEXT_OBJECT);
        try {
            // to test bind_context with NamingInterface
            NamingInterface n = new NamingInterface(nc);
            this.ni0.bindContext(NAME_ARCHIVE_CONTEXT_AFTER_CONTEXT_OBJECT, nc, false);
            fail();
        } catch (NotFound nf) {
            assertTrue(nf.getMessage(), nf.why.equals(NotFoundReason.not_context));
        }
    }

    @Test
    public void bindCompoundNameArchiveContextWithForce()
        throws NotFound, CannotProceed, InvalidName, AlreadyBound, InvalidNameException {

        // resolve has already been tested
        // resolve has already been tested
        NamingInterface ni = this.ni0.bindNewContext(NAME_CONTEXT, true);
        NamingContext nc = ni.getNamingContext();
        this.ni0.bindContext(NAME_ARCHIVE_CONTEXT, nc, true);
        
        NameComponent[] name = NamingInterface.toName(NAME_ARCHIVE_CONTEXT_OBJECT);
        namingContext.bind(name, testObject);

        // Forcing creation (exception NamingContext::NotFound expected)
        System.out.println("Forcing bind of " + NAME_ARCHIVE_CONTEXT_AFTER_CONTEXT_OBJECT);
        try {
            this.ni0.bindContext(NAME_ARCHIVE_CONTEXT_AFTER_CONTEXT_OBJECT, nc, true);
            fail();
        } catch (NotFound nf) {
            assertTrue( nf.why.equals(NotFoundReason.not_context) );
        }
    }
    
    
    @Test(expected=AlreadyBound.class)
    public void rebindBoundObjectWithoutForce()
        throws InvalidNameException, NotFound, CannotProceed, InvalidName, AlreadyBound {
        NamingContext nc = null;

        try {
            NamingInterface ni = this.ni0.bindNewContext(NAME_LONG_CONTEXT, true);
            org.omg.CORBA.Object obj = this.ni0.resolve(NAME_BOUND_CONTEXT);
            nc = NamingContextHelper.narrow(obj);
            this.ni0.bindContext(NAME_ARCHIVE_CONTEXT, nc, true);
            
            //
            // Try to bind a context when an object is in the path
            //
            // cdmw.test/archive/orbacus.mk
            System.out.println("Binding a context when an object is in the path");
            NameComponent[] name = NamingInterface.toName(NAME_ARCHIVE_CONTEXT_OBJECT);
            namingContext.bind(name, testObject);
        } catch (AlreadyBound ab) {
            fail();
        }

        //
        // Try to bind a context using the name of an existing object
        //
        System.out.println("Test of binding a context using the name of an existing object");
        this.ni0.bindContext(NAME_ARCHIVE_CONTEXT_OBJECT, nc, false);
        fail();
    }
    
    @Test
    public void rebindBoundObjectWithForce()
        throws InvalidNameException, NotFound, CannotProceed, InvalidName, AlreadyBound {
        NamingContext nc = null;

        NamingInterface ni = this.ni0.bindNewContext(NAME_LONG_CONTEXT, true);
        org.omg.CORBA.Object obj = this.ni0.resolve(NAME_BOUND_CONTEXT);
        nc = NamingContextHelper.narrow(obj);
        this.ni0.bindContext(NAME_ARCHIVE_CONTEXT, nc, true);

        //
        // Try to bind a context when an object is in the path
        //
        // cdmw.test/archive/orbacus.mk
        System.out.println("Binding a context when an object is in the path");
        NameComponent[] name = NamingInterface.toName(NAME_ARCHIVE_CONTEXT_OBJECT);
        namingContext.bind(name, testObject);

        // Forcing bind (exception NamingContext::NotFound expected)
        System.out.println(
            "Test of forcing bind of a context using the name of an existing object");
        try {
            this.ni0.bindContext(NAME_ARCHIVE_CONTEXT_OBJECT, nc, true);
            fail();
        } catch (NotFound nf) {
            assertTrue( nf.why.equals(NotFoundReason.not_context) );
        }
    }
    
    @Test(expected=AlreadyBound.class)
    public void rebindCompoundBoundContextWithoutForce()
        throws NotFound, CannotProceed, InvalidName, AlreadyBound {
        NamingContext nc = null;

        try {
            NamingInterface ni = this.ni0.bindNewContext(NAME_LONG_CONTEXT, true);
            org.omg.CORBA.Object obj = this.ni0.resolve(NAME_BOUND_CONTEXT);
            nc = NamingContextHelper.narrow(obj);
            this.ni0.bindContext(NAME_ARCHIVE_CONTEXT, nc, true);
        } catch (AlreadyBound ab) {
            fail();
        }

    
        //
        // Try to bind a context using the name of an existing context
        //
        System.out.println("Test of binding a context using the name of an existing context");
        ni0.bindContext(NAME_ARCHIVE_CONTEXT, nc, false);
        fail();
    }
    
    @Test
    public void rebindCompoundBoundContextWithForce()
        throws NotFound, CannotProceed, InvalidName, AlreadyBound {
        NamingContext nc = null;

        NamingInterface ni = this.ni0.bindNewContext(NAME_LONG_CONTEXT, true);
        org.omg.CORBA.Object obj = this.ni0.resolve(NAME_BOUND_CONTEXT);
        nc = NamingContextHelper.narrow(obj);
        this.ni0.bindContext(NAME_ARCHIVE_CONTEXT, nc, true);

        // Forcing bind
        System.out.println("Test of forcing bind of a context using the name of an existing context");
        ni0.bindContext(NAME_ARCHIVE_CONTEXT, nc, true);
    }
    
    @Test
    public void testRebindContext()
        throws NotFound, CannotProceed, InvalidName, AlreadyBound{
        //-------------------------------------------------
        // Testing rebind_context operation
        //-------------------------------------------------
        System.out.println("Testing NamingInterface::rebind_context operation");
    
        // This test creates the following bindings in the naming service:
        // cdmw.test/archive.2/orbacus.mk
        // cdmw.test/archive.2/commonsvcs/naming/c++
        // (link to cdmw.test/runtime/commonsvcs/naming/c++)
        NamingInterface ni = this.ni0.bindNewContext(NAME_LONG_CONTEXT, true);
    
        // Get a reference to the naming context bound to
        // cdmw.test/runtime/commonsvcs/naming/c++
        NamingContext nc = null;
        String input = "cdmw.test/runtime/commonsvcs/naming/c++";
        try {
            NameComponent[] name = NamingInterface.toName(input);
            org.omg.CORBA.Object obj = namingContext.resolve(name);
            nc = NamingContextHelper.narrow(obj);
            assertTrue(nc != null);
        } catch (InvalidName in) {
            fail("Unexpected error " + in);
        } catch (Exception ex) {
            fail("Unexpected error " + ex);
        }
    
        //
        // Try to rebind a context when an intermediate context doesn't exist
        //
        // Expecting NamingContext::NotFound exception
        // ex.why == NamingContext::not_context
        input = "cdmw.test/archive.2/runtime/commonsvcs/naming/c++";
        System.out.println("Attempt to rebind " + input);
        try {
            ni0.rebindContext(input, nc, false);
            fail();
        } catch (NotFound nf) {
            assertTrue( nf.why.equals(NotFoundReason.missing_node) );
        } catch (Exception e) {
            fail();
        }
    
        // Forcing creation
        System.out.println("Forcing rebind of " + input);
        try {
            // to test rebind_context with NamingInterface
            NamingInterface n = new NamingInterface(nc);
            this.ni0.rebindContext(input, nc, true);
            org.omg.CORBA.Object obj = ni0.resolve(input);
            assertTrue(obj!=null);
        } catch (Exception e) {
            fail();
        }
    
        //
        // Try to rebind a context when an object is in the path
        //
        // cdmw.test/archive.2/orbacus.mk
        input = "cdmw.test/archive.2/orbacus.mk";
        System.out.println("Rebinding a context when an object is in the path");
        try {
            NameComponent[] name = NamingInterface.toName(input);
            namingContext.bind(name, testObject);
        } catch (InvalidName in) {
            fail("Unexpected error " + in );
        } catch (Exception ex) {
            fail("Unexpected error " + ex );
        }
    
        // Expecting NamingContext::NotFound exception
        // ex.why == NamingContext::not_context
        input = "cdmw.test/archive.2/orbacus.mk/runtime/commonsvcs/naming/c++";
        System.out.println("Attempt to rebind " + input);
        try {
            this.ni0.rebindContext(input, nc, false);
            fail();
        } catch (NotFound nf) {
            assertTrue( nf.why.equals(NotFoundReason.not_context) );
        } catch (Exception e) {
            fail();
        }
        
        // Forcing creation (exception NamingContext::NotFound expected)
        System.out.println("Forcing rebind of cdmw.test/archive/orbacus.mk/runtime/commonsvcs/naming/c++");
        try {
            this.ni0.rebindContext(input, nc, true);
            fail();
        } catch (NotFound nf) {
            assertTrue( nf.why.equals(NotFoundReason.not_context) );
        } catch (Exception e) {
            fail();
        }
    
        //
        // Try to rebind a context using the name of an existing object
        //
        input = "cdmw.test/archive.2/orbacus.mk";
        System.out.println("Test of rebinding a context using the name of an existing object");
        try {
            this.ni0.rebindContext(input, nc, false);
            fail();
        } catch (NotFound nf) {
            assertTrue( nf.why.equals(NotFoundReason.not_context) );
        } catch (Exception e) {
            fail();
        }
    
        // Forcing rebind (exception NamingContext::NotFound expected)
        System.out.println("Test of forcing rebind of a context using the name of an existing object");
        try {
            this.ni0.rebindContext(input, nc, true);
            fail();
        } catch (NotFound nf) {
            assertTrue( nf.why.equals(NotFoundReason.not_context) );
        } catch (Exception e) {
            fail();
        }
    
        //
        // Try to rebind a context using the name of an existing context
        //
        input = "cdmw.test/archive.2/runtime/commonsvcs/naming/c++";
        System.out.println("Test of rebinding a context using the name of an existing context");
        try {
            this.ni0.rebindContext(input, nc, false);
        } catch (Exception e) {
            fail();
        }
    
        // Forcing bind
        System.out.println("Test of forcing rebind of a context using the name of an existing context");
        try {
            this.ni0.rebindContext(input, nc, true);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void testNewContext()
        throws NotFound, CannotProceed, InvalidName, AlreadyBound {
        //-------------------------------------------------
        // Testing new_context operation
        //-------------------------------------------------
        System.out.println("Testing NamingInterface::new_context operation");
    
        // This test creates the following bindings in the naming service:
        //     cdmw.test/runtime/commonsvcs/naming/c++/test/incl
        NamingInterface ni = this.ni0.bindNewContext(NAME_LONG_CONTEXT, true);

        String input = "cdmw.test/runtime/commonsvcs/naming/c++/test";
    
        try {
            org.omg.CORBA.Object obj = ni0.resolve(input);
            NamingContext nc = NamingContextHelper.narrow(obj);
    
            assertTrue("Unexpected error (" + input + " not present!)", nc != null);
            ni = new NamingInterface(nc);
            ni.bindContext("incl", ni.newContext(), false);
    
            input += "/incl";
            obj = ni0.resolve(input);
            assertTrue( obj!=null );
        } catch(NotFound nf) {
            switch(nf.why.value()) {
                case NotFoundReason._missing_node:
                    fail("Missing node.");
                    break;
                case NotFoundReason._not_context:
                    fail("Not context.");
                    break;
                case NotFoundReason._not_object:
                    fail("Not object.");
                    break;
                default:
                    fail();
            }
        } catch (Exception e) {
            fail();
        }
    }   
}
