/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.commonsvcs.naming.test;

import cdmw.commonsvcs.naming.NamingInterface;
import cdmw.testutils.CORBATestManager;
import cdmw.testutils.CORBATestManager.InvalidObject;

import org.omg.CosNaming.Binding;
import org.omg.CosNaming.BindingHolder;
import org.omg.CosNaming.BindingIterator;
import org.omg.CosNaming.NamingContext;
import org.omg.CosNaming.NamingContextHelper;
import org.omg.CosNaming.BindingListHolder;
import org.omg.CosNaming.BindingIteratorHolder;
import org.omg.CosNaming.NamingContextPackage.CannotProceed;
import org.omg.CosNaming.NamingContextPackage.InvalidName;
import org.omg.CosNaming.NamingContextPackage.NotFound;

import org.junit.BeforeClass;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.After;
import org.junit.Test;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

public class ListTest {

    private static NamingContext namingContext;
    private static TestObject testObject;

    private NamingInterface ni0;

    @BeforeClass
    public static void initTests() throws InvalidObject {
        CORBATestManager testManager = CORBATestManager.instance();

        org.omg.CORBA.Object obj = testManager.getObject("NameService");
        namingContext = org.omg.CosNaming.NamingContextHelper.narrow(obj);
        
        obj = testManager.getObject("TestObject");
        testObject = TestObjectHelper.narrow(obj);
    }

    @AfterClass
    public static void finiTests() {
        namingContext = null;
        testObject = null;
    }

    @Before
    public void init() {
        this.ni0 = new NamingInterface(namingContext);
    }

    @After
    public void fini() throws NotFound, CannotProceed, InvalidName {
        // Clear context from bindings
        try {
            BindingListHolder listHolder = new BindingListHolder();
            BindingIteratorHolder iteratorHolder = new BindingIteratorHolder();
            namingContext.list(0, listHolder, iteratorHolder);
            BindingIterator iterator = iteratorHolder.value;
            if (iterator != null) {
                BindingHolder bindingHolder = new BindingHolder();
    
                while (iterator.next_one(bindingHolder)) {
                    Binding binding = bindingHolder.value;
                    namingContext.unbind(binding.binding_name);
                }
    
                iterator.destroy();
            }
        }
        finally {
            this.ni0 = null;
        }
    }
    
    @Test
    public void testList() {
        // This tests NamingInterface class
        //-------------------------------------------------
        // Testing list operations
        //-------------------------------------------------
        System.out.println("Testing NamingInterface#list operations");

        // Install test name tree
        int nbBindings = 10;
        int chunkSize = 3;
        try {
            this.ni0.bindNewContext("test.list/ctxt.1", true);
            this.ni0.bindNewContext("test.list/ctxt.2", true);
            this.ni0.bindNewContext("test.list/ctxt.3", true);
            this.ni0.bindNewContext("test.list/ctxt.4", true);
            this.ni0.bindNewContext("test.list/ctxt.5", true);
            this.ni0.bind("test.list/obj.1", testObject, true);
            this.ni0.bind("test.list/obj.2", testObject, true);
            this.ni0.bind("test.list/obj.3", testObject, true);
            this.ni0.bind("test.list/obj.4", testObject, true);
            this.ni0.bind("test.list/obj.5", testObject, true);
        } catch (Exception e) {
            fail("Unexpected error (couldn't install initial bindings)");
        }

        String input = "test.list";
        NamingContext nc = null;
        try {
            org.omg.CORBA.Object obj = this.ni0.resolve(input);
            nc = NamingContextHelper.narrow(obj);
            assertTrue(nc!=null);
        } catch(Exception e) {
            fail("Unexpected error (" + input + " not present!)");
        }

        NamingInterface ni = new NamingInterface(nc);

        //
        // Test list operation
        //
        System.out.println("Test list operation");
        // Iterator reference
        BindingIteratorHolder it = new BindingIteratorHolder();
        // Binding list
        BindingListHolder bl = new BindingListHolder();

        int nb = 0;

        // Get first chunk
        ni.list(chunkSize, bl, it);
        nb += bl.value.length;

        if (it!=null) {
            // More bindings?
            boolean more;
            do {
                // Get next chunk
                more = it.value.next_n(chunkSize,bl);
                nb += bl.value.length;
            } while (more);
            // Clean up iterator
            it.value.destroy();
        }
        assertTrue(nb == nbBindings);

        //
        // Test listContext operation
        //
        System.out.println("Test listContext operation");
        
        BindingListHolder bndList = new BindingListHolder();
        ni.listContext(bndList, chunkSize);
        assertTrue(bndList.value.length == nbBindings);

        // check zero value for how_many parameter
        bndList = new BindingListHolder();
        ni.listContext(bndList, 0);
        assertTrue(bndList.value.length == nbBindings);
    }
}

