/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.commonsvcs.naming.test;

import cdmw.commonsvcs.naming.InvalidNameException;
import cdmw.commonsvcs.naming.NameParser;

import org.junit.Test;
import org.omg.CosNaming.NameComponent;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

public class ToStringTest {  
    
    @Test public void composeIdKindIdKindString() {
        System.out.println("Test name components with both ID and Kind field");
        
        NameComponent[] name = new NameComponent[2];
        NameComponent nc = new NameComponent("first", "1st");
        name[0] = nc;
        nc = new NameComponent("second", "2nd");
        name[1] = nc;

        String reference = "first.1st/second.2nd";
        try {
            String output = NameParser.toString(name);
            assertTrue( reference.equals(output) );
        } catch(InvalidNameException ine) {
            fail(ine.getMessage());
        }
    }

    @Test public void composeIdEmptyKindIdKindString() {
        System.out.println("Test name components with missing Kind field");

        NameComponent[] name = new NameComponent[2];
        NameComponent nc = new NameComponent("first", "");
        name[0] = nc;
        nc = new NameComponent("second", "2nd");
        name[1] = nc;

        String reference = "first/second.2nd";
        try {
            String output = NameParser.toString(name);
            assertTrue( reference.equals(output) );
        } catch(InvalidNameException ine) {
            fail(ine.getMessage());
        }
    }

    @Test public void composeIdKindEmptyIdKindString() {
        System.out.println("Test name components with missing ID field");

        NameComponent[] name = new NameComponent[2];
        NameComponent nc = new NameComponent("first", "1st");
        name[0] = nc;
        nc = new NameComponent("", "2nd");
        name[1] = nc;

        String reference = "first.1st/.2nd";
        try {
            String output = NameParser.toString(name);
            assertTrue( reference.equals(output) );
        } catch(InvalidNameException ine) {
            fail(ine.getMessage());
        }
    }

    @Test public void composeIdKindEmptyIdEmptyKindIdKindString() {
        System.out.println("Test node with empty ID and Kind field");

        NameComponent[] name = new NameComponent[3];
        NameComponent nc = new NameComponent("first", "1st");
        name[0] = nc;
        nc = new NameComponent("", "");
        name[1] = nc;
        nc = new NameComponent("third", "3rd");
        name[2] = nc;

        String reference = "first.1st/./third.3rd";
        try {
            String output = NameParser.toString(name);
            assertTrue( reference.equals(output) );
        } catch(InvalidNameException ine) {
            fail(ine.getMessage());
        }
    }

    @Test public void composeStringWithSpecialCharacters () {
        System.out.println("Test nodes with characters that are escaped");

        NameComponent[] name = new NameComponent[3];
        NameComponent nc = new NameComponent("first", "1st.dot");
        name[0] = nc;
        nc = new NameComponent("second.dot", "2nd");
        name[1] = nc;
        nc = new NameComponent("third/slash", "3rd\n");
        name[2] = nc;

        String reference = "first.1st\\.dot/second\\.dot.2nd/third\\/slash.3rd\n";
        try {
            String output = NameParser.toString(name);
            assertTrue( reference.equals(output) );
        } catch(InvalidNameException ine) {
            fail(ine.getMessage());
        }
    }

    @Test public void composeEmptyIdEmptyKindString () {
        System.out.println("Test empty name");

        NameComponent[] name = new NameComponent[1];
        NameComponent nc = new NameComponent("", "");
        name[0] = nc;

        String reference = ".";
        try {
            String output = NameParser.toString(name);
            assertTrue( reference.equals(output) );
        } catch(InvalidNameException ine) {
            fail(ine.getMessage());
        }
    }

    @Test(expected=InvalidNameException.class) public void composeUsingEmptyName()
        throws InvalidNameException {
        System.out.println("Test empty NameComponent array");

        NameComponent[] name = new NameComponent[0];
        @SuppressWarnings("unused")
            String output = NameParser.toString(name);
        fail("Error: Invalid name component");
    }
}

