/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_CDMW_COMMONSVCS_REQFU_NDC_HPP
#define INCL_CDMW_COMMONSVCS_REQFU_NDC_HPP

#include <string>
#include <list>

#include <Foundation/common/Exception.hpp>


namespace Cdmw
{
namespace CommonSvcs
{
namespace RequestFollowUp
{

    //
    // NDC base class
    //
    class NDC
    {
    public:

        /**
         * Constructor
         *
         * @param ndc string (NDC string format : "%s:%8.8x-%4.4x-%4.4x-%2.2x%2.2x-%2.2x%2.2x%2.2x%2.2x%2.2x%2.2x-%d")
         */
        NDC(const std::string& ndc) 
             throw (BadParameterException);

        /**
         * Constructor
         *
         * @param operation string of current operation
         * @param uuid string of base uuid
         * @param request_id value of request counter
         */
        NDC(const char* operation, const char* uuid, const unsigned int& request_id);
        
        /**
         * Copy Constructor
         */
        NDC(const NDC& ndc) :
            m_operation(ndc.m_operation),
            m_uuid(ndc.m_uuid),
            m_request_id(ndc.m_request_id),
            m_ndc_string(ndc.m_ndc_string)
        {
        }

        /**
         * Destructor
         */
        ~NDC()
        {}
        
        /**
         * Operator=
         */
        NDC& operator = (const NDC& rhs)
        {
            if (this != & rhs) {
                m_operation  = rhs.m_operation;
                m_uuid       = rhs.m_uuid;
                m_request_id = rhs.m_request_id;
                m_ndc_string = rhs.m_ndc_string;
            }
            return *this;            
        }

        const char* get_operation() const
        {
            return m_operation.c_str();
        }

        const char* get_uuid() const
        {
            return m_uuid.c_str();
        }

        unsigned int  get_request_id() const
        {
            return m_request_id;
        }

        const std::string& to_string() const
        {
            // UUID format (defined in $ACE_ROOT/ace/UUID.cpp)
            //    "%8.8x-%4.4x-%4.4x-%2.2x%2.2x-%2.2x%2.2x%2.2x%2.2x%2.2x%2.2x",
            //    timeLow_,
            //    timeMid_,
            //    timeHiAndVersion_,
            //    clockSeqHiAndReserved_,
            //    clockSeqLow_,
            //    node_->nodeID_[0],
            //    node_->nodeID_[1],
            //    node_->nodeID_[2],
            //    node_->nodeID_[3],
            //    node_->nodeID_[4],
            //    node_->nodeID_[5]
            return m_ndc_string;
        }

    private:
        /**
         * Constructor
         */
        NDC()
        {}
        
        std::string m_operation;
        std::string m_uuid;
        unsigned int m_request_id;
        std::string m_ndc_string;
             
    };
    
    typedef std::list<NDC> NDCList;

} // End namespace RequestFollowUp
} // End namespace CommonSvcs
} // End namespace Cdmw

#endif // INCL_CDMW_COMMONSVCS_REQFU_NDC_HPP
