/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_CDMW_COMMONSVCS_REQFU_NDC_FACTORY_HPP
#define INCL_CDMW_COMMONSVCS_REQFU_NDC_FACTORY_HPP

#include <string>
#include <list>
#include <iostream>
#include <sstream>

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/common/Assert.hpp>
#include <Foundation/osthreads/Mutex.hpp>

#include "requestfollowup/NDC.hpp"


namespace Cdmw
{
namespace CommonSvcs
{
namespace RequestFollowUp
{

    //
    // NDCFactory class
    //
    class NDCFactory
    {
    public:
        
        /**
         * Init a NDCLogger.
         * If called more than once, an assert produce a core dump
         */
        static void Init_NDC_factory();
        // fixing memory leak ?
        static void Cleanup_NDC_factory();
        /**
         * Destructor
         */

    ~NDCFactory()
    {
    } 
        
        /**
         * Creation of a new NDC.
         *
         * @param operation of current request follow up
         */
        NDC create_new_ndc(const char* operation);
        
        NDCList from_string(const char* ndcs) const;
        NDCList from_octetseq(const CORBA::OctetSeq& ndcs) const;
        NDCList from_any(const CORBA::Any& ndcs) const;

        std::string      to_string(const NDCList& ndcs) const;
        CORBA::OctetSeq* to_octetseq(const NDCList& ndcs) const;
        CORBA::Any*      to_any(const NDCList& ndcs) const;
        

        /**
         * Accessors
         */
        static NDCFactory* const Get_instance();
        
        inline std::string get_uuid_base() const
        {
            return m_uuid_base;
        }
        
        // Used only in Test, not protected
        inline unsigned int get_request_counter() const
        {
            return m_request_counter;
        }

    private:
        /**
         * Constructor
         */       
        NDCFactory();

        std::string  m_uuid_base;
        unsigned int m_request_counter;

        static NDCFactory*  M_instance;
        
        static Cdmw::OsSupport::Mutex M_mutex;
        
    };    
    

     
} // End namespace RequestFollowUp
} // End namespace CommonSvcs
} // End namespace Cdmw

#endif // INCL_CDMW_COMMONSVCS_REQFU_NDC_FACTORY_HPP

