/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_CDMW_COMMONSVCS_REQFU_NDC_LOGGER_HPP
#define INCL_CDMW_COMMONSVCS_REQFU_NDC_LOGGER_HPP

#include <Foundation/logging/Logger.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <requestfollowup/NDC.hpp>

#define NDC_LOGGER NDCLogger::Get_instance()->Get_Logger()

namespace Cdmw
{
namespace CommonSvcs
{
namespace RequestFollowUp
{
    
    const IOP::ServiceId CDMW_NDC_SERVICE_CONTEXT_ID = 1 | Cdmw::OrbSupport::CDMW_VMCID;
    

    enum NDCInterceptionPoint {
        NDC_SERVER_RECEIVE_REQUEST,
        NDC_SERVER_SEND_REPLY,
        NDC_SERVER_SEND_OTHER,
        NDC_SERVER_SEND_EXECEPTION,
        NDC_CLIENT_SEND_REQUEST,
        NDC_CLIENT_SEND_POLL,
        NDC_CLIENT_RECEIVE_REPLY,
        NDC_CLIENT_RECEIVE_EXCEPTION,
        NDC_CLIENT_RECEIVE_OTHER
    };

    class NDCLogger
    {
    public:
        
        /**
         * init a NDCLogger.
         * if called more than once, an assert produce a core dump
         *
         * @param log_full_ndc_list request follow up log full NDCs list
         * @param log_ndc_operation request follow up log operation of the NDCs
         */
        static void Init_NDC_logger(int log_full_ndc_list, int log_ndc_operation);
        /* clean up a NDClogger */
        static void Cleanup_NDC_Logger();
        /**
         * return the instance of the NDCLogger
         */
        static NDCLogger* Get_instance() throw();
   
        /**
         * Destructor
         */
        ~NDCLogger()
        {
          m_forbidden_operations.clear();
        }

        /**
         * NDC Domain for Logs
         */
        static const char* NDC_LOGGER_NAME;

        inline Cdmw::Logging::Logger * Get_Logger() throw () 
        {
            //static Cdmw::Logging::Logger * Logger__ = Cdmw::Logging::Logger::Get_Logger(NDC_LOGGER_NAME);
            //CDMW_ASSERT(Logger__ != 0);
            return m_ndc_logger;
        }
 
        const char* ip_to_string(const NDCInterceptionPoint& ip) const;
        
        bool is_operation_to_log(const char* operation) const;
        
        void log_request(NDCInterceptionPoint ip, const char* operation, const NDCList& ndc_list) const;

private:
        /**
         * Constructor
         */       
        NDCLogger()
        : m_ndc_logger(Cdmw::Logging::Logger::Get_Logger(NDC_LOGGER_NAME))
        {
        }
        
        std::list<std::string> m_forbidden_operations;
        int m_log_full_ndc_list;  // format of Request Follow UP logging
        int m_log_ndc_operation;  // format of Request Follow UP logging
        Cdmw::Logging::Logger* m_ndc_logger;
        static NDCLogger* M_instance;
    };    
    

     
} // End namespace RequestFollowUp
} // End namespace CommonSvcs
} // End namespace Cdmw

#if (CDMW_TRACE_LEVEL >= 4 )
#define CDMW_REQFU(logger, msg, ndc)                                                                     \
            std::ostringstream __log_oss__;                                                              \
            __log_oss__ << msg;                                                                          \
            logger->log(::Cdmw::Logging::REQ_FOLLOWUP, __log_oss__.str().c_str(), ndc, __FILE__, __LINE__);
#else
#define CDMW_REQFU(logger, msg, ndc)
#endif

#endif // INCL_CDMW_COMMONSVCS_REQFU_NDC_LOGGER_HPP

