/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_CDMW_COMMONSVCS_REQFU_NDC_SERVER_REQUEST_INTERCEPTOR_IMPL_HPP
#define INCL_CDMW_COMMONSVCS_REQFU_NDC_SERVER_REQUEST_INTERCEPTOR_IMPL_HPP

// Cdmw Files

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/CdmwServerRequestInterceptor.hpp>
#include <Foundation/orbsupport/PortableInterceptor.hpp>
#include <requestfollowup/NDCLogger.hpp>

namespace Cdmw
{
namespace CommonSvcs
{
namespace RequestFollowUp
{

/**
*Purpose:
*<p>    This class implements the ServerRequestInterceptor class.
*
*Features:
*<p> Thread safe
*
*/
class NDCServerRequestInterceptor_impl : public Cdmw::OrbSupport::CdmwServerRequestInterceptor
{
public:

    /**
     * Purpose:
     * <p> Constructor
     * @param info ORB init information
     * @param ndc_slot_id slot id used to store the ndc list of the
     * incoming request.
     *
     */ 
    NDCServerRequestInterceptor_impl(
          PortableInterceptor::ORBInitInfo_ptr info,
          const PortableInterceptor::SlotId ndc_slot_id);

    /**
     * Purpose:
     * <p> Return the name of the interceptor
     *
     */
    virtual char*
    name() throw (CORBA::SystemException);

    /**
     * Purpose:
     * <p> Destroy the interceptor
     *
     */
    virtual void
    destroy() throw (CORBA::SystemException);

    

    /**
     * Purpose:
     * <p> This is a -specific operation to enable returning previous
     * logged replies. This operation is used in an ftrequestlogging CSC 
     * and then the code shall be located here if we want to set a slot.
     *
     */ 
    void
    cdmw_ft_interception_point(
      PortableInterceptor::ServerRequestInfo_ptr ri,
      CORBA::OctetSeq_out ocs)
    throw (CORBA::SystemException, PortableInterceptor::ForwardRequest);


    /**
     * Purpose:
     * <p> At this interception point, Interceptors must get their service
     * context information from the incoming request and transfer it to 
     * PortableInterceptor::Current's slots.
     * @param ri Server request information
     */
    virtual void
    receive_request_service_contexts(
        PortableInterceptor::ServerRequestInfo_ptr ri)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest);

    /**
     * Purpose:
     * <p> This interception point allows an Interceptor to query request 
     * information after all the information, including operation parameters,
     * are available.
     * @param ri Server request information
     */
    virtual void
    receive_request(PortableInterceptor::ServerRequestInfo_ptr ri)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest);

    /**
     * Purpose:
     * <p> This interception point allows an Interceptor to query reply 
     * information and modify the reply service context after the target
     * operation has been invoked and before the reply is returned to the
     * client.
     * @param ri Server request information
     */
    virtual void
    send_reply(PortableInterceptor::ServerRequestInfo_ptr ri)
        throw (CORBA::SystemException);

    /**
     * Purpose:
     * <p> This interception point allows an Interceptor to query the 
     * information available when a request results in something other
     * than a normal reply or an exception. A request could result in 
     * a retry (for example, a GIOP Reply with a LOCATION_FORWARD status
     * was received).
     * @param ri Server request information
     */
    virtual void
    send_other(PortableInterceptor::ServerRequestInfo_ptr ri)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest);

    /**
     * Purpose:
     * <p> When an exception occurs, this interception point is called. 
     * It allows an Interceptor to query the exception information and 
     * modify the reply service context before the exception is raised 
     * to the client.
     * @param ri Server request information
     */
    virtual void
    send_exception(PortableInterceptor::ServerRequestInfo_ptr ri)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest);

private:
    NDCServerRequestInterceptor_impl(
          const NDCServerRequestInterceptor_impl&);

    NDCServerRequestInterceptor_impl& operator=(
          const NDCServerRequestInterceptor_impl&);

    void log_incoming_request(
        NDCInterceptionPoint ip,
        PortableInterceptor::ServerRequestInfo_ptr ri)
            throw ();

    void log_outgoing_response(
        NDCInterceptionPoint ip,
        PortableInterceptor::ServerRequestInfo_ptr ri)
            throw ();

    // slot id used to store ndc_list
    PortableInterceptor::SlotId  m_ndc_slot_id;

  // Hack to avoid TAO's problem of not clearing TSC after server request completion
  PortableInterceptor::Current_var m_pi_current;

};


} // End of namespace RequestFollowUp
} // End of namespace CommonSvcs
} // End of namespace Cdmw

#endif // INCL_CDMW_COMMONSVCS_REQFU_NDC_SERVER_REQUEST_INTERCEPTOR_IMPL_HPP

