/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

// standard files
#include <sstream>

// Cdmw Files

// Local Files
#include <requestfollowup/NDCLogger.hpp>
#include <requestfollowup/NDC.hpp>


namespace Cdmw
{
namespace CommonSvcs
{
namespace RequestFollowUp
{

NDC::NDC(const std::string& ndc)
     throw (BadParameterException)

{
    //
    // NDC string format : "%s:%8.8x-%4.4x-%4.4x-%2.2x%2.2x-%2.2x%2.2x%2.2x%2.2x%2.2x%2.2x-%d"
    //

    size_t pos1 = ndc.find_first_of(':'); // first ':' is separator between operation and uuid
    size_t pos2 = ndc.find_last_of('-');  // last '-' is separator between uuid and request_id

    if ( (pos1 == std::string::npos) || (pos2 == std::string::npos) ) {
        CDMW_LOG(NDC_LOGGER, Cdmw::Logging::CDMW_DEBUG_3, "Invalid NDC string format: " << ndc);
        CDMW_THROW2(BadParameterException, "ndc", ndc);
    }

    m_operation = ndc.substr(0, pos1);

    pos1++; // discard separator (:)
    m_uuid  = ndc.substr(pos1, pos2-pos1);

    pos2++; // discard separator (-)

    const char* reqid = ndc.substr(pos2).c_str();
    m_request_id = atoi(reqid);

    std::ostringstream oss;
    oss << m_operation << ":" << m_uuid << "-" << m_request_id;
    m_ndc_string = oss.str();
}


NDC::NDC(const char* operation, const char* uuid, const unsigned int& request_id)
    : m_operation(operation),
      m_uuid(uuid),
      m_request_id(request_id)
{
    std::ostringstream oss;
    oss << m_operation << ":" << m_uuid << "-" << m_request_id;
    m_ndc_string = oss.str();
}


} // End namespace RequestFollowUp
} // End namespace CommonSvcs
} // End namespace Cdmw

