/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "requestfollowup/NDCLogger.hpp"
#include <requestfollowup/NDCFactory.hpp>

namespace
{
}

namespace Cdmw
{
namespace CommonSvcs
{
namespace RequestFollowUp
{

const char* NDCLogger::NDC_LOGGER_NAME = "CDMW.FDS.RequestFollowUp";

NDCLogger*  NDCLogger::M_instance = NULL;

static const char* sNDC_SERVER_RECEIVE_REQUEST     = "SERVER_RECEIVE_REQUEST";
static const char* sNDC_SERVER_SEND_REPLY          = "SERVER_SEND_REPLY";
static const char* sNDC_SERVER_SEND_OTHER          = "SERVER_SEND_OTHER";
static const char* sNDC_SERVER_SEND_EXECEPTION     = "SERVER_SEND_EXECEPTION";
static const char* sNDC_CLIENT_SEND_REQUEST        = "CLIENT_SEND_REQUEST";
static const char* sNDC_CLIENT_SEND_POLL           = "CLIENT_SEND_POLL";
static const char* sNDC_CLIENT_RECEIVE_REPLY       = "CLIENT_RECEIVE_REPLY";
static const char* sNDC_CLIENT_RECEIVE_EXCEPTION   = "CLIENT_RECEIVE_EXCEPTION";
static const char* sNDC_CLIENT_RECEIVE_OTHER       = "CLIENT_RECEIVE_OTHER";
static const char* sNDC_UNKNOWN_IP                 = "UNKNOWN_INTERCEPTION_POINT";


void NDCLogger::Init_NDC_logger(int log_full_ndc_list, int log_ndc_operation)
{
    CDMW_ASSERT(!M_instance);

    M_instance = new NDCLogger();

    // Initialize format of the NDC when logging Request Follow Up
    M_instance->m_log_full_ndc_list = log_full_ndc_list;
    M_instance->m_log_ndc_operation = log_ndc_operation;

    // Initialize operations list we don't log in Request Follow Up
    M_instance->m_forbidden_operations.push_back("put_formatted_messages");
    M_instance->m_forbidden_operations.push_back("add_log");
}

// fixing memory leak
void NDCLogger::Cleanup_NDC_Logger()
{
   delete M_instance;
   M_instance = 0;
}

NDCLogger* NDCLogger::Get_instance() 
    throw() 
{
    CDMW_ASSERT(M_instance);
    return M_instance;
}


const char* NDCLogger::ip_to_string(const NDCInterceptionPoint& ip) const
{
    switch (ip)
    {
    case NDC_SERVER_RECEIVE_REQUEST:
        return sNDC_SERVER_RECEIVE_REQUEST;
    case NDC_SERVER_SEND_REPLY:
        return sNDC_SERVER_SEND_REPLY;
    case NDC_SERVER_SEND_OTHER:
        return sNDC_SERVER_SEND_OTHER;
    case NDC_SERVER_SEND_EXECEPTION:
        return sNDC_SERVER_SEND_EXECEPTION;
    case NDC_CLIENT_SEND_REQUEST:
        return sNDC_CLIENT_SEND_REQUEST;
    case NDC_CLIENT_SEND_POLL:
        return sNDC_CLIENT_SEND_POLL;
    case NDC_CLIENT_RECEIVE_REPLY:
        return sNDC_CLIENT_RECEIVE_REPLY;
    case NDC_CLIENT_RECEIVE_EXCEPTION:
        return sNDC_CLIENT_RECEIVE_EXCEPTION;
    case NDC_CLIENT_RECEIVE_OTHER:
        return sNDC_CLIENT_RECEIVE_OTHER;
    }

    return sNDC_UNKNOWN_IP;
}


bool NDCLogger::is_operation_to_log(const char* operation) const
{
    std::list<std::string>::const_iterator iter = m_forbidden_operations.begin();
    while ( iter != m_forbidden_operations.end() )
    {
        if ( *iter == operation ) {
            return false;
        }
        iter++;
    }

    return true;
}


void NDCLogger::log_request(NDCInterceptionPoint ip, const char* operation, const NDCList& ndc_list) const
{
    std::ostringstream oss;
    
    if ( ndc_list.empty() ) {
        // If no NDC in the list, log an empty NDC.
    }
    else if ( m_log_full_ndc_list == 1 && m_log_ndc_operation == 1 ) {
        oss <<  NDCFactory::Get_instance()->to_string(ndc_list);

    }
    else  if ( m_log_full_ndc_list == 1 && m_log_ndc_operation == 0 ) {
        NDCList::const_iterator iter = ndc_list.begin();
        while ( iter != ndc_list.end() ) {
            if (iter != ndc_list.begin()) {
                oss << '/';
            }
            oss << (*iter).get_uuid() << "-" << (*iter).get_request_id();
            iter++;
        }
    }
    else if ( m_log_full_ndc_list == 0 && m_log_ndc_operation == 1 ) {
        NDC ndc = ndc_list.front();
        oss << ndc.to_string();
    }
    else { // if ( m_log_full_ndc_list == 0 && m_log_ndc_operation == 0 ) {
        NDC ndc = ndc_list.front();
        oss << ndc.get_uuid() << "-" << ndc.get_request_id();
    }
    
    //std::cout << "REQ_FOLLOWUP : " << " ["<< oss.str() << "]\t"<< ip_to_string(ip) << " (" << operation << ")" << std::endl;
    CDMW_REQFU(NDC_LOGGER, ip_to_string(ip) << " (" << operation << ")", oss.str().c_str());
}

} // End namespace RequestFollowUp
} // End namespace CommonSvcs
} // End namespace Cdmw

