/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Cdmw Files
#include <Foundation/common/Assert.hpp> 
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/logging/FunctionLogger.hpp>

// Local Files
#include <requestfollowup/NDCORBInitializer.hpp>
#include <requestfollowup/NDCLogger.hpp>
#include <requestfollowup/NDCServerRequestInterceptor_impl.hpp>
#include <requestfollowup/NDCClientRequestInterceptor_impl.hpp>




namespace 
{
} // end of anonymous namespace


namespace Cdmw
{
namespace CommonSvcs
{
namespace RequestFollowUp
{

// constructor
NDCORBInitializer::NDCORBInitializer()
{
    CDMW_LOG_FUNCTION(NDC_LOGGER, "");
}

// destructor
NDCORBInitializer::~NDCORBInitializer()
{
}

void
NDCORBInitializer::pre_init(PortableInterceptor::ORBInitInfo_ptr info) 
  throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(NDC_LOGGER, "");
}

    
//
// Note: the interceptors are registered in post_init()
//       if their constructors require
//       ORBInitInfo::resolve_initial_reference(), which 
//       cannot be called in pre_init().
//
void
NDCORBInitializer::post_init(PortableInterceptor::ORBInitInfo_ptr info) 
   throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(NDC_LOGGER, "");
    //
    // allocate needed slot_id
    //
    CDMW_DEBUG(NDC_LOGGER,"post_init => allocate needed Request Follow Up slot_id");
    PortableInterceptor::SlotId  request_slot_id = info->allocate_slot_id();

    //
    // Create and register the interceptors
    //
    CDMW_DEBUG(NDC_LOGGER,"post_init => Create the Request Follow Up interceptors");
    //fixing memory leak
    // NDCServerRequestInterceptor_impl* server_interceptor = new NDCServerRequestInterceptor_impl(info, request_slot_id);
    PortableInterceptor::ServerRequestInterceptor_var server_interceptor = new NDCServerRequestInterceptor_impl(info, request_slot_id);

    //NDCClientRequestInterceptor_impl* client_interceptor = new NDCClientRequestInterceptor_impl(info, request_slot_id);
    PortableInterceptor::ClientRequestInterceptor_var client_interceptor = new NDCClientRequestInterceptor_impl(info, request_slot_id);


    try
    {
        info->add_server_request_interceptor(server_interceptor.in());
        info->add_client_request_interceptor(client_interceptor.in());
    }
    catch(const PortableInterceptor::ORBInitInfo::DuplicateName&)
    {
        // can not happened while interceptor is not named!
        CDMW_ASSERT(false);
    }


}

} // End of namespace RequestFollowUp
} // End of namespace CommonSvcs
} // End of namespace Cdmw



