/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Cdmw Files
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>

#include <Foundation/commonsvcs/requestfollowup/NDCServiceInit.hpp>
#if CDMW_ORB_VDR == tao && CDMW_ORB_VER > 13
#include <tao/ORBInitializer_Registry.h>
#endif

#include <Foundation/logging/LogManager.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include "Foundation/common/Exception.hpp"


// Local Files
#include <requestfollowup/NDCORBInitializer.hpp>
#include <requestfollowup/NDCLogger.hpp>
#include <requestfollowup/NDCFactory.hpp>

namespace 
{
   /**
    * Default request follow up format
    */
   const int CDMW_DEFAULT_REQUEST_FOLLOW_UP_LOG_FULL_NDC_LIST = 1; // Log all NDC of the list";
   const char* const REQUEST_FOLLOW_UP_LOG_FULL_NDC_LIST_PROPERTY_NAME =
              "Cdmw.RequestFollowUp.LogFullNDCList";

   const int CDMW_DEFAULT_REQUEST_FOLLOW_UP_LOG_NDC_OPERATION = 1; // Log the operation of a NDC";
   const char* const REQUEST_FOLLOW_UP_LOG_NDC_OPERATION_PROPERTY_NAME =
              "Cdmw.RequestFollowUp.LogNDCOperation";
} // end anonymous namespace


namespace Cdmw
{
namespace CommonSvcs
{
namespace RequestFollowUp
{

void
NDCServiceInit::Init (int& argc, char** argv)
    throw(CORBA::SystemException)
{
    // Init the Logging library
    Cdmw::Logging::LogManager::Init(argc, argv);

    // Init System Properties
    Cdmw::OsSupport::SystemProperties::Init(argc, argv);

    // Get request follow up format
    int log_full_ndc_list = Cdmw::OsSupport::SystemProperties::Get_Property(
               REQUEST_FOLLOW_UP_LOG_FULL_NDC_LIST_PROPERTY_NAME, 
               CDMW_DEFAULT_REQUEST_FOLLOW_UP_LOG_FULL_NDC_LIST);
    int log_ndc_operation = Cdmw::OsSupport::SystemProperties::Get_Property(
               REQUEST_FOLLOW_UP_LOG_NDC_OPERATION_PROPERTY_NAME, 
               CDMW_DEFAULT_REQUEST_FOLLOW_UP_LOG_NDC_OPERATION);
    
    if ( (log_full_ndc_list >= 0) && (log_ndc_operation >= 0) ) 
    {

        NDCLogger::Init_NDC_logger(log_full_ndc_list, log_ndc_operation);
    
        NDCFactory::Init_NDC_factory();

        CDMW_INFO(NDC_LOGGER, "Register NDCORBInitializer." );
        PortableInterceptor::ORBInitializer_var ndc_initializer 
           = new Cdmw::CommonSvcs::RequestFollowUp::NDCORBInitializer();
        
        PortableInterceptor::register_orb_initializer(ndc_initializer.in());

    } else {
       std::cerr << "Invalid Request Follow Up format:"
                 << "log_full_ndc_list=" << log_full_ndc_list 
                 << ", log_ndc_operation=" << log_ndc_operation 
                 << ". NDCORBInitializer will not be registered!"
                 << std::endl;
    }
}
 //fixing memory leak
void
NDCServiceInit::Cleanup ()
    throw(CORBA::SystemException)
{
        NDCFactory::Cleanup_NDC_factory();
        NDCLogger::Cleanup_NDC_Logger();
}

} // End of namespace RequestFollowUp
} // End of namespace CommonSvcs
} // End of namespace Cdmw



