/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.commonsvcs.requestfollowup;

//import java.util.*;

//import org.omg.IOP.*;

import cdmw.common.*;
//import cdmw.logging.*;

//import cdmw.orbsupport.ExceptionMinorCodes;
import static cdmw.logging.Logger.*;
//import static cdmw.logging.FunctionLogger.*;


//
// NDC base class
//
public class NDC
{
    private String m_operation;
    private String m_uuid;
    private int m_requestId;
    private String m_ndcString;

    /**
     * Constructor
     *
     * @param ndc  NDC string format : "%s:%8.8x-%4.4x-%4.4x-%2.2x%2.2x-%2.2x%2.2x%2.2x%2.2x%2.2x%2.2x-%d")
     */
    public NDC(String ndc) 
         throws BadParameterException {
         
        int pos1 = ndc.indexOf(':');      // first ':' is separator between operation and uuid
        int pos2 = ndc.lastIndexOf('-');  // last  '-' is separator between uuid and request_id

        if ( (pos1 == -1) || (pos2 == -1) ) {
            CDMW_WARN(NDCLogger.getInstance().getLogger(), "Invalid NDC string format: " + ndc, 
                    new Throwable());
            BadParameterException ex = new BadParameterException("ndc", ndc);
            throw ex;
        }

        m_operation = ndc.substring(0, pos1);

        pos1++; // discard separator (:)
        m_uuid  = ndc.substring(pos1, pos2);

        pos2++; // discard separator (-)
        try {
            m_requestId = Integer.parseInt(ndc.substring(pos2));
        } catch (NumberFormatException e) {
            CDMW_WARN(NDCLogger.getInstance().getLogger(), "Invalid NDC string format: " + ndc, 
                    new Throwable());
            BadParameterException ex = new BadParameterException("ndc", ndc);
            throw ex;
        }

        m_ndcString = m_operation + ":" + m_uuid + "-" + m_requestId;
    }

    /**
     * Constructor
     *
     * @param operation  string of current operation
     * @param uuid  string of current base UUID
     * @param request_id  value of current request counter
     */
    public NDC(String operation, String uuid, int requesId) {
        m_operation = operation;
        m_uuid = uuid;
        m_requestId = requesId;
        m_ndcString = m_operation + ":" + m_uuid + "-" + m_requestId;
    }

    public String getOperation()
    {
        return m_operation;
    }

    public String getUUID()
    {
        return m_uuid;
    }

    public int  getRequestId()
    {
        return m_requestId;
    }

    public String toString()
    {
        // UUID format (defined in $ACE_ROOT/ace/UUID.cpp)
        //    "%8.8x-%4.4x-%4.4x-%2.2x%2.2x-%2.2x%2.2x%2.2x%2.2x%2.2x%2.2x",
        //    timeLow_,
        //    timeMid_,
        //    timeHiAndVersion_,
        //    clockSeqHiAndReserved_,
        //    clockSeqLow_,
        //    node_->nodeID_[0],
        //    node_->nodeID_[1],
        //    node_->nodeID_[2],
        //    node_->nodeID_[3],
        //    node_->nodeID_[4],
        //    node_->nodeID_[5]
        return m_ndcString;
    }

}

