/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.commonsvcs.requestfollowup;

import java.util.*;

//import org.omg.CORBA.BAD_PARAM;
import org.omg.CORBA.SystemException;

import cdmw.logging.LogManager;
//import cdmw.orbsupport.ExceptionMinorCodes;

import static cdmw.logging.Logger.*;

import cdmw.common.Assert;
import cdmw.orbsupport.ExceptionMinorCodes;
import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;

/**
*Purpose:
*<p> This class provides an init of the Request Follow Up capability.
*
*Features:
*<p> Thread safe
*
*/
public class NDCServiceInit {

    /**
     * Default request follow up format
     */
    private static final int CDMW_DEFAULT_REQUEST_FOLLOW_UP_LOG_FULL_NDC_LIST = 1; // Log all NDC of the list
    public final static String REQUEST_FOLLOW_UP_LOG_FULL_NDC_LIST_PROPERTY_NAME = "Cdmw.RequestFollowUp.LogFullNDCList";
    
    private static final int CDMW_DEFAULT_REQUEST_FOLLOW_UP_LOG_NDC_OPERATION = 1; // Log all NDC of the list
    public final static String REQUEST_FOLLOW_UP_LOG_NDC_OPERATION_PROPERTY_NAME = "Cdmw.RequestFollowUp.LogNDCOperation";

    
    /**
     * Purpose:
     * <p> Initialises Nested Diagnostic Context mechanism : it creates server
     * request interceptor and client request interceptor able to follow up 
     * incoming and outgoing requests.
     *
     *@param args The list of parameter received of the command line
     *
     *@return none
     *
     *@exception CORBA.SystemException Any CORBA system exception that
     * may be raised by the underlying ORB.
     */ 
    public static void init(String[] args) throws SystemException {
        //Init the Logging library
        LogManager.Init(args);
        
        // Get request follow up format properties
        String logFullNDCListStr = System.getProperty(
            REQUEST_FOLLOW_UP_LOG_FULL_NDC_LIST_PROPERTY_NAME, 
            String.valueOf(CDMW_DEFAULT_REQUEST_FOLLOW_UP_LOG_FULL_NDC_LIST));
        String logNDCOperationStr = System.getProperty(
            REQUEST_FOLLOW_UP_LOG_NDC_OPERATION_PROPERTY_NAME,
            String.valueOf(CDMW_DEFAULT_REQUEST_FOLLOW_UP_LOG_NDC_OPERATION));
        System.out.println("logFullNDCList=" + logFullNDCListStr + ", logNDCOperation=" + logNDCOperationStr);

        try {    
            int logFullNDCList = Integer.parseInt(logFullNDCListStr);
            int logNDCOperation = Integer.parseInt(logNDCOperationStr);
            if ( (logFullNDCList >= 0) && (logNDCOperation >= 0) ) {
            
                NDCLogger.initNDCLogger(logFullNDCList, logNDCOperation);
                
                NDCFactory.initNDCFactory();
                
                CDMW_INFO(NDCLogger.getInstance().getLogger(), "register NDCORBInitializer", 
                    new Throwable()); 
    
                System.setProperty(
                    "org.omg.PortableInterceptor.ORBInitializerClass.NDCORBInitializer", 
                    NDCORBInitializer.class.getName());
    
            } else {
                System.err.println( 
                    "Invalid Request Follow Up format:" +
                    " logFullNDCList=" + logFullNDCList + 
                    ", logNDCOperation=" + logNDCOperation + 
                    ". RequestTimeoutORBInitializer will not be registered!");
            }
        } catch (NumberFormatException e) {
            System.err.println( 
                    "Invalid Request Follow Up format:" +
                    " logFullNDCList=" + logFullNDCListStr + 
                    ", logNDCOperation=" + logNDCOperationStr + 
                    ". RequestTimeoutORBInitializer will not be registered!");
        }
    }
    
}
