/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "testrequestfollowup/TestRequestFollowUp.hpp"
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>

#include "requestfollowup/NDCLogger.hpp"
#include "requestfollowup/NDCFactory.hpp"

namespace Cdmw
{
namespace CommonSvcs
{
namespace RequestFollowUp
{

/**
*Implementation notes: [if no pertinent write none ]
*<p>
*Portability issues: [if no pertinent write none ]
*<p>
*/

CPPUNIT_TEST_SUITE_REGISTRATION( TestRequestFollowUp );

TestRequestFollowUp::TestRequestFollowUp ()
    throw()
{
    std::cout << "TestRequestFollowUp: Enter in TestRequestFollowUp()." << std::endl;
    //CORBA::Object_var obj1 = Cdmw::TestUtils::CORBATestManager::instance()->get_object("Root");
    //m_root = CosNaming::NamingContextExt::_narrow(obj1.in());
    //CORBA::Object_var obj2 = Cdmw::TestUtils::CORBATestManager::instance()->get_object("RootNoCache");
    //m_rootNoCache = CosNaming::NamingContextExt::_narrow(obj2.in());
}


TestRequestFollowUp::~TestRequestFollowUp()
    throw()
{
    // does nothing
}


TestRequestFollowUp::TestRequestFollowUp(const TestRequestFollowUp& rfu)
   throw()
   : m_root(CosNaming::NamingContextExt::_duplicate(rfu.m_root.in())),
     m_rootNoCache(CosNaming::NamingContextExt::_duplicate(rfu.m_rootNoCache.in()))
{
   // does nothing
}
        
TestRequestFollowUp&
TestRequestFollowUp::operator=(const TestRequestFollowUp& rfu)
    throw()
{
   if (this != &rfu) {
      m_root = CosNaming::NamingContextExt::_duplicate(rfu.m_root.in());
      m_rootNoCache = CosNaming::NamingContextExt::_duplicate(rfu.m_rootNoCache.in());
   }
   return *this;
}

void TestRequestFollowUp::test_ndc_logger()
{
   std::cout << std::endl << std::endl << std::endl;
   std::cout << "TestRequestFollowUp: Enter in TestRequestFollowUp::test_ndc_logger()." << std::endl;
   TEST_INFO("Testing NDCLogger.");
   try {
       std::cout << "TestRequestFollowUp: Testing NDCLogger::Get_instance()." << std::endl;
       CPPUNIT_ASSERT( NDCLogger::Get_instance() );

       std::cout << "TestRequestFollowUp: Testing NDCLogger::Get_Logger()." << std::endl;
       CPPUNIT_ASSERT( NDCLogger::Get_instance()->Get_Logger() );
       
       std::cout << "TestRequestFollowUp: Testing NDCLogger::ip_to_string()." << std::endl;
       std::string s1 = NDCLogger::Get_instance()->ip_to_string(NDC_SERVER_RECEIVE_REQUEST);
       std::string s2 = NDCLogger::Get_instance()->ip_to_string(NDC_SERVER_SEND_EXECEPTION);
       std::string s3 = NDCLogger::Get_instance()->ip_to_string(NDC_CLIENT_SEND_REQUEST);
       std::string s4 = NDCLogger::Get_instance()->ip_to_string(NDC_CLIENT_RECEIVE_OTHER);
       CPPUNIT_ASSERT( s1 == "SERVER_RECEIVE_REQUEST" );
       CPPUNIT_ASSERT( s2 == "SERVER_SEND_EXECEPTION" );
       CPPUNIT_ASSERT( s3 == "CLIENT_SEND_REQUEST" );
       CPPUNIT_ASSERT( s4 == "CLIENT_RECEIVE_OTHER" );
       
       std::cout << "TestRequestFollowUp: Testing NDCLogger::is_operation_to_log()." << std::endl;
       CPPUNIT_ASSERT( NDCLogger::Get_instance()->is_operation_to_log("AnOperation") );
       CPPUNIT_ASSERT( ! NDCLogger::Get_instance()->is_operation_to_log("add_log") );
       CPPUNIT_ASSERT( ! NDCLogger::Get_instance()->is_operation_to_log("put_formatted_messages") );
       
       std::cout << "TestRequestFollowUp: Testing NDCLogger::log_request(), with empty NDC list." << std::endl;
       NDCList ndc_list;
       NDCLogger::Get_instance()->log_request(NDC_CLIENT_SEND_REQUEST, "Operation1", ndc_list);
       
       std::cout << "TestRequestFollowUp: Testing NDCLogger::log_request(), with 3 NDC in the list." << std::endl;
       //NDCList ndc_list;
       ndc_list.push_back(NDC("Operation1:01234567-89AB-CDEF-0123-456789ABCDEF-12"));
       ndc_list.push_back(NDC("Operation2:01234567-89AB-CDEF-0123-456789ABCDEF-1234"));
       ndc_list.push_back(NDC("Operation3:76543210-BA98-FEDC-3210-FEDCBA987654-123456"));
       NDCLogger::Get_instance()->log_request(NDC_CLIENT_SEND_REQUEST, "Operation2", ndc_list);
   } catch (const CORBA::Exception & ex) {
      std::cerr << "Caught CORBA Exception:" << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex) << std::endl;
      CPPUNIT_ASSERT(false);
   }
}


void TestRequestFollowUp::test_ndc()
{
   std::cout << std::endl << std::endl << std::endl;
   std::cout << "TestRequestFollowUp: Enter in TestRequestFollowUp::test_ndc()." << std::endl;
   
   // NDC string format : "%8.8x-%4.4x-%4.4x-%2.2x%2.2x-%2.2x%2.2x%2.2x%2.2x%2.2x%2.2x-%4d"
   
   TEST_INFO("Testing NDC.");
   try {
       
       NDC ndc1("Operation1:01234567-89AB-CDEF-0123-456789ABCDEF-12");
       NDC ndc2("Operation2", "01234567-89AB-CDEF-0123-456789ABCDEF", 1234);
       NDC ndc3("Operation3:76543210-BA98-FEDC-3210-FEDCBA987654-123456");
       std::cout << "ndc1=" << ndc1.to_string() << std::endl;
       std::cout << "ndc2=" << ndc2.to_string() << std::endl;
       std::cout << "ndc3=" << ndc3.to_string() << std::endl;

       std::cout << "TestRequestFollowUp: Testing NDC::NDC(string)." << std::endl;
       {
       std::string o = ndc1.get_operation();
       std::string u = ndc1.get_uuid();
       unsigned int i = ndc1.get_request_id();
       CPPUNIT_ASSERT( o == "Operation1" );
       CPPUNIT_ASSERT( u == "01234567-89AB-CDEF-0123-456789ABCDEF" );
       CPPUNIT_ASSERT( i == 12 );
       }

       std::cout << "TestRequestFollowUp: Testing NDC::NDC(ope, uuid, req_id)." << std::endl;
       {
       std::string o = ndc2.get_operation();
       std::string u = ndc2.get_uuid();
       unsigned int i = ndc2.get_request_id();
       CPPUNIT_ASSERT( o == "Operation2" );
       CPPUNIT_ASSERT( u == "01234567-89AB-CDEF-0123-456789ABCDEF" );
       CPPUNIT_ASSERT( i == 1234 );
       }
       
       std::cout << "TestRequestFollowUp: Testing NDC::to_string()." << std::endl;
       {
       std::string s1 = ndc1.to_string();
       std::cout << "s1=" << s1 << std::endl;
       CPPUNIT_ASSERT( s1 == "Operation1:01234567-89AB-CDEF-0123-456789ABCDEF-12" );
       std::string s2 = ndc2.to_string();
       std::cout << "s2=" << s2 << std::endl;
       CPPUNIT_ASSERT( s2 == "Operation2:01234567-89AB-CDEF-0123-456789ABCDEF-1234" );
       std::string s3 = ndc3.to_string();
       std::cout << "s3=" << s3 << std::endl;
       CPPUNIT_ASSERT( s3 == "Operation3:76543210-BA98-FEDC-3210-FEDCBA987654-123456" );
       }
       
   } catch (const CORBA::Exception & ex) {
      std::cerr << "Caught CORBA Exception:" << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex) << std::endl;
      CPPUNIT_ASSERT(false);
   }
}


void TestRequestFollowUp::test_ndc_factory()
{
   std::cout << std::endl << std::endl << std::endl;
   std::cout << "TestRequestFollowUp: Enter in RequestFollowUp::test_ndc_factory()." << std::endl;
   TEST_INFO("Testing NDCFactory.");
   try {
       std::cout << "TestRequestFollowUp: Testing NDCFactory::Init_NDC_factory()." << std::endl;
       const NDCFactory* ndc_fac = NDCFactory::Get_instance();
       CPPUNIT_ASSERT( ndc_fac );
       std::string uuid_base = NDCFactory::Get_instance()->get_uuid_base();
       CPPUNIT_ASSERT( uuid_base.size() ==  ACE_Utils::UUID::NIL_UUID.to_string()->length() );
       unsigned int req_id = NDCFactory::Get_instance()->get_request_counter();
       CPPUNIT_ASSERT( (int)req_id >= 0 );
       
       std::cout << "TestRequestFollowUp: Testing NDCFactory::create_new_ndc()." << std::endl;
       {
       NDC ndc = NDCFactory::Get_instance()->create_new_ndc("AnOperation");
       std::string o = ndc.get_operation();
       std::string u = ndc.get_uuid();
       unsigned int ri = ndc.get_request_id();
       std::string s = ndc.to_string();  
       CPPUNIT_ASSERT( o == "AnOperation" );
       CPPUNIT_ASSERT( u == uuid_base );
       CPPUNIT_ASSERT( ri == NDCFactory::Get_instance()->get_request_counter() );
       CPPUNIT_ASSERT( ri == req_id+1 );
       std::ostringstream oss;
       oss << o << ":" << u << "-" << ri;
       std::cout << "s1=" << s << std::endl;
       std::cout << "s2=" << oss.str() << std::endl;
       CPPUNIT_ASSERT( s == oss.str() );
       }
       
       NDC ndc1 = NDCFactory::Get_instance()->create_new_ndc("Operation1");
       NDC ndc2 = NDC("Operation2:01234567-89AB-CDEF-0123-456789ABCDEF-1234");
       NDC ndc3 = NDCFactory::Get_instance()->create_new_ndc("Operation3");
       std::cout << "ndc1=" << ndc1.to_string() << std::endl;
       std::cout << "ndc2=" << ndc2.to_string() << std::endl;
       std::cout << "ndc3=" << ndc3.to_string() << std::endl;
       std::ostringstream oss;
       oss << ndc1.to_string() << "/" << ndc2.to_string() << "/" << ndc3.to_string();
       std::string ndcs = oss.str();
       std::cout << "ndcs=" << ndcs << std::endl;
       
       std::cout << "TestRequestFollowUp: Testing NDCFactory::to_string()." << std::endl;
       {
       NDCList ndc_list;
       ndc_list.push_back(ndc1); ndc_list.push_back(ndc2); ndc_list.push_back(ndc3);
       std::string s1 = NDCFactory::Get_instance()->to_string(ndc_list);
       std::cout << "s1=" << s1 << std::endl;
       CPPUNIT_ASSERT( s1 == ndcs );
       }
       
       std::cout << "TestRequestFollowUp: Testing NDCFactory::from_string()." << std::endl;
       {
       NDCList ndc_list = NDCFactory::Get_instance()->from_string(ndcs.c_str());
       NDC n1 = ndc_list.front(); ndc_list.pop_front();
       std::cout << "n1=" << n1.to_string() << std::endl;
       CPPUNIT_ASSERT( n1.to_string() == ndc1.to_string() );
       NDC n2 = ndc_list.front(); ndc_list.pop_front();
       std::cout << "n2=" << n2.to_string() << std::endl;
       CPPUNIT_ASSERT( n2.to_string() == ndc2.to_string() );
       NDC n3 = ndc_list.front(); ndc_list.pop_front();
       std::cout << "n3=" << n3.to_string() << std::endl;
       CPPUNIT_ASSERT( n3.to_string() == ndc3.to_string() );
       CPPUNIT_ASSERT( ndc_list.size() == 0 );
       }
       
       std::cout << "TestRequestFollowUp: Testing NDCFactory::to_octetseq()." << std::endl;
       {
       NDCList ndc_list = NDCFactory::Get_instance()->from_string(ndcs.c_str());
       const CORBA::OctetSeq* os = NDCFactory::Get_instance()->to_octetseq(ndc_list);
       CPPUNIT_ASSERT( os->length() == ndcs.size() );
       std::string s;
       for (unsigned int i=0; i < os->length() ; i++) {
          s += (*os)[i];
       }
       std::cout << "s=" << s << std::endl;
       //fixing memory leak, returned pointer to delete
       delete os;
       CPPUNIT_ASSERT( s == ndcs );
       }
       
       std::cout << "TestRequestFollowUp: Testing NDCFactory::from_octetseq()." << std::endl;
       {
       CORBA::OctetSeq_var os = new CORBA::OctetSeq(ndcs.size());
       os->length(ndcs.size());
       for (unsigned int i=0; i < ndcs.size(); i++) {
          os[i] = ndcs[i];
       }
       NDCList ndc_list = NDCFactory::Get_instance()->from_octetseq(os);
       NDC n1 = ndc_list.front(); ndc_list.pop_front();
       std::cout << "n1=" << n1.to_string() << std::endl;
       CPPUNIT_ASSERT( n1.to_string() == ndc1.to_string() );
       NDC n2 = ndc_list.front(); ndc_list.pop_front();
       std::cout << "n2=" << n2.to_string() << std::endl;
       CPPUNIT_ASSERT( n2.to_string() == ndc2.to_string() );
       NDC n3 = ndc_list.front(); ndc_list.pop_front();
       std::cout << "n3=" << n3.to_string() << std::endl;
       CPPUNIT_ASSERT( n3.to_string() == ndc3.to_string() );
       }
       
       std::cout << "TestRequestFollowUp: Testing NDCFactory::to_any()." << std::endl;
       {
       NDCList ndc_list = NDCFactory::Get_instance()->from_string(ndcs.c_str());
       const CORBA::Any* a = NDCFactory::Get_instance()->to_any(ndc_list);
       const char* s1;
       (*a) >>= s1;
       std::string s2 = s1;
       std::cout << "s=" << s2 << std::endl;
       CPPUNIT_ASSERT( s2 == ndcs );
       //fixing memory leak
       delete a;
       }
       
       std::cout << "TestRequestFollowUp: Testing NDCFactory::from_any()." << std::endl;
       {
       CORBA::Any a;
       a <<= ndcs.c_str();
       NDCList ndc_list = NDCFactory::Get_instance()->from_any(a);
       NDC n1 = ndc_list.front(); ndc_list.pop_front();
       std::cout << "n1=" << n1.to_string() << std::endl;
       CPPUNIT_ASSERT( n1.to_string() == ndc1.to_string() );
       NDC n2 = ndc_list.front(); ndc_list.pop_front();
       std::cout << "n2=" << n2.to_string() << std::endl;
       CPPUNIT_ASSERT( n2.to_string() == ndc2.to_string() );
       NDC n3 = ndc_list.front(); ndc_list.pop_front();
       std::cout << "n3=" << n3.to_string() << std::endl;
       CPPUNIT_ASSERT( n3.to_string() == ndc3.to_string() );
       }

   } catch (const CORBA::Exception & ex) {
      std::cerr << "Caught CORBA Exception:" << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex) << std::endl;
      CPPUNIT_ASSERT(false);
   }
    


   /* 
   TEST_INFO("Testing resolve_str("<<Config::O1_OBJ<<").");
   try {
      CORBA::Object_var obj =  m_root->resolve_str(Config::O1_OBJ);
      M::Foo_var foo = M::Foo::_narrow(obj.in());
      CORBA::String_var s = foo->op("");
      CPPUNIT_ASSERT(strcmp(s.in(),Config::O1_OBJ) == 0);
   } catch (const CORBA::Exception & ex) {
      std::cerr << "Caught CORBA Exception:" << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex) << std::endl;
      CPPUNIT_ASSERT(false);
   }
   TEST_INFO("Testing resolve_str("<<Config::O2_CTXT<<").");
   try {
      CORBA::Object_var obj =  m_root->resolve_str(Config::O2_CTXT);
      CosNaming::NamingContextExt_var nc = CosNaming::NamingContextExt::_narrow(obj.in());
      CPPUNIT_ASSERT(!CORBA::is_nil(nc.in()));
      obj = nc->resolve_str(Config::O3_OBJ);
      M::Foo_var foo = M::Foo::_narrow(obj.in());
      CORBA::String_var s = foo->op("");
      CPPUNIT_ASSERT(strcmp(s.in(),Config::O3_OBJ) == 0);
      obj = nc->resolve_str(Config::O4_OBJ);
      foo = M::Foo::_narrow(obj.in());
      s = foo->op("");
      CPPUNIT_ASSERT(strcmp(s.in(),Config::O4_OBJ) == 0);
   } catch (const CORBA::Exception & ex) {
      std::cerr << "Caught CORBA Exception:" 
                << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex) << std::endl;
      CPPUNIT_ASSERT(false);
   }
   TEST_INFO("Testing resolve_str("<<Config::O2_CTXT<<"/"<<Config::O3_OBJ<<").");
   try {
      std::ostringstream ostr;
      ostr << Config::O2_CTXT<<"/"<<Config::O3_OBJ;
      CORBA::Object_var obj =  m_root->resolve_str(ostr.str().c_str());
      M::Foo_var foo = M::Foo::_narrow(obj.in());
      CORBA::String_var s = foo->op("");
      CPPUNIT_ASSERT(strcmp(s.in(),Config::O3_OBJ) == 0);
   } catch (const CORBA::Exception & ex) {
      std::cerr << "Caught CORBA Exception:" 
                << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex) << std::endl;
      CPPUNIT_ASSERT(false);
   }
   TEST_INFO("Testing Unknown object resolve_str("<<Config::Unknown_OBJ<<").");
   try {
      CORBA::Object_var obj =  m_root->resolve_str(Config::Unknown_OBJ);
      CPPUNIT_ASSERT(false);
   } catch (const CORBA::Exception & ex) {
      std::cerr << "Caught CORBA Exception:" 
                << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex) << std::endl;
      CPPUNIT_ASSERT(true);
   }
   TEST_INFO("Testing resolve_str("<<Config::Unknown_CTXT<<"/"<<Config::O3_OBJ<<").");
   try {
      std::ostringstream ostr;
      ostr << Config::Unknown_CTXT<<"/"<<Config::O3_OBJ;
      CORBA::Object_var obj =  m_root->resolve_str(ostr.str().c_str());
      CPPUNIT_ASSERT(false);
   } catch (const CORBA::Exception & ex) {
      std::cerr << "Caught CORBA Exception:" 
                << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex) << std::endl;
      CPPUNIT_ASSERT(true);
   }
   */
}

void TestRequestFollowUp::do_tests()     
{
   std::cout << std::endl << std::endl << std::endl;
   std::cout << "TestRequestFollowUp: Enter in do_tests()." << std::endl;
   test_ndc_logger();
   test_ndc();
   test_ndc_factory();
}


        
} // End namespace RequestFollowUp
} // End namespace CommonSvcs
} // End namespace Cdmw

