/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.commonsvcs.requestfollowup.test.testrequestfollowup;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;

import cdmw.commonsvcs.requestfollowup.*;
import cdmw.orbsupport.*;
import cdmw.ossupport.*;
import cdmw.testutils.*;

public class MainTest {

    private static final int EXIT_FAILURE = 1;

    public static void main(String[] args) {
        try {
            // Initialise NDC service
            System.setProperty("Cdmw.RequestFollowUp.LogFullNDCList", "0");
            System.setProperty("Cdmw.RequestFollowUp.LogNDCOperation", "1");
            NDCServiceInit.init(args);

            // Initialises the ORB
            StrategyList strategyList = new StrategyList();
            strategyList.addORBThreaded();
            strategyList.addPOAThreadPerConnection();

            String[] argsList = new String[1];
            argsList[0] = cdmw.common.Options.LOCALISATION_SERVICE_OPTION +
                           "=56789";
            ORB orb = ORBSupport.ORBInit(argsList, strategyList);

            // Get the root POA
            org.omg.CORBA.Object obj = orb.resolve_initial_references("RootPOA");
            POA rootPOA = POAHelper.narrow(obj);

            POAManager poaMgr = rootPOA.the_POAManager();
            poaMgr.activate();

            int timescale = cdmw.testutils.TestManager.getTimescale();
            Thread.sleep(timescale * 1000);

            // Create Server process
            /*
            System.out.println("Create test_server process");
            

            String requestTimeoutProperty =
                FTSystemProperties.WATCHDOG_TIMEOUT_PROPERTY_NAME;

            java.util.Vector prop = new java.util.Vector();
            prop.add(requestTimeoutProperty+"=500");
            
            String args1 = "-D" + requestTimeoutProperty
                + "=500 --FilePath=hello1 --ComputFilePath=comput1 --HostName=A";
            System.out.println("java test_server " + args1);
            ThreadedProcess testServerId1 = OS.createJavaProcess(
                "cdmw.ft.watchdog.test.testtimeoutrequest.MainServer " + args1, prop);

            args1 = "-D" + requestTimeoutProperty
                + "=500 --FilePath=hello2 --ComputFilePath=comput2 --HostName=B";
            System.out.println("java test_server " + args1);
            ThreadedProcess testServerId2 = OS.createJavaProcess(
                "cdmw.ft.watchdog.test.testtimeoutrequest.MainServer " + args1, prop);

            args1 = "-D" + requestTimeoutProperty
                + "=500 --FilePath=hello3 --ComputFilePath=comput3 --HostName=C";
            System.out.println("java test_server " + args1);
            ThreadedProcess testServerId3 = OS.createJavaProcess(
                "cdmw.ft.watchdog.test.testtimeoutrequest.MainServer "+ args1, prop);

            args1 = "-D" + requestTimeoutProperty
                + "=500 --FilePath=hello4 --ComputFilePath=comput4 --HostName=D";
            System.out.println("java test_server " + args1);
            ThreadedProcess testServerId4 = OS.createJavaProcess(
                "cdmw.ft.watchdog.test.testtimeoutrequest.MainServer " + args1, prop);

            timescale = cdmw.testutils.TestManager.getTimescale();
            Thread.sleep(timescale * 25000);

            org.omg.CORBA.Object computObj = TestUtils.getComputRefFromFile(orb, "comput3");

            PullMonitorable[] pullMonitorable = new PullMonitorable[TestUtils.NB_LOCATION];

            for (int i = 0; i < TestUtils.NB_LOCATION; i++ ) {
                try {
                    System.out.println("Getting PullMonitorable on host "
                        + TestUtils.locations[i]);

                    pullMonitorable[i] = TestUtils.getPmRefFromFile(orb,
                        TestUtils.locations[i]);
                    if (pullMonitorable[i] == null)
                        org.junit.Assert.assertTrue(false);
                    else
                        org.junit.Assert.assertTrue(true);
                }
                catch (SystemException e) {
                    e.printStackTrace();
                }
            }
            */

            Thread.sleep(timescale * 1000);

            // Do test
            System.out.println("CORBATestManager init...");
            String testName = "";
            if (args.length > 0) {
                testName = args[0];
            }
            
            CORBATestManager.instance().addTest(TestRequestFollowUp.class);
            
            CORBATestManager.instance().init(testName, orb, rootPOA);
            //System.out.println("CORBATestManager add comput_object ...");
            //cdmw.testutils.CORBATestManager.instance().add("comput_object", computObj);

            System.out.println("CORBATestManager run_tests ...");
            CORBATestManager.instance().runTests();

            Thread.sleep(timescale * 1000);

            CORBATestManager.instance().printSummary();
        }
        catch (cdmw.common.Exception e) {
            e.printStackTrace();
            System.exit(EXIT_FAILURE);
        }
        catch (UserException e) {
            e.printStackTrace();
            System.exit(EXIT_FAILURE);
        }
        catch (SystemException e) {
            e.printStackTrace();
            System.exit(EXIT_FAILURE);
        }
        catch (Exception e) {
            System.err.println("Unexpected exception");
            e.printStackTrace();
            System.exit(EXIT_FAILURE);
        }
    }
}
