/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.commonsvcs.requestfollowup.test.testrequestfollowup;

import java.io.*;
import java.util.UUID;

import org.junit.*;
import org.junit.runner.RunWith;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;

import cdmw.ossupport.OS;
import cdmw.testutils.*;
import cdmw.testutils.CORBATestManager.InvalidObject;
import cdmw.testutils.threadtestengine.*;

import cdmw.commonsvcs.requestfollowup.*;
import cdmw.commonsvcs.requestfollowup.NDCFactory;
import cdmw.common.*;

import org.junit.Assert;
import org.junit.Test;
import static org.junit.Assert.*;

@RunWith( CdmwThreadedTestClassRunner.class)
public class TestRequestFollowUp {

    /**
     * Field is used for obtaining TestProcessor class instance
     */
    private TestContext testContext;

    private ORB orb;

    private POA rootPOA;

    public TestRequestFollowUp() {

        try {
            this.orb = CORBATestManager.instance().getORB();

            this.rootPOA = CORBATestManager.instance().getPOA();

            //org.omg.CORBA.Object obj = CORBATestManager.instance().getObject(
            //    "comput_object");
            //this.computObj = ComputInterfaceHelper.narrow(obj);
        }
        catch (InvalidObject e) {
            e.printStackTrace();
        }
    }

    @Test
    public void doTests() {
        // set number of requested successfull tests
        // set_nbOfRequestedTestOK(45);
        int timescale = CORBATestManager.getTimescale();


        System.out.println("");
        System.out.println(" *** TestRequestFollowUp: Testing NDCLogger...");
        try {
            System.out.println("");
            System.out.println(" * TestRequestFollowUp: Testing NDCLogger.getInstance().");
            assertTrue( NDCLogger.getInstance() + " != null", NDCLogger.getInstance() != null );
            assertTrue( NDCLogger.getInstance().getLogger() + " != null",
                        NDCLogger.getInstance().getLogger() != null );
            
            System.out.println("");
            System.out.println("TestRequestFollowUp: Testing NDCLogger.getInstance().ipToString().");
            String s1 = NDCLogger.getInstance().ipToString(NDCLogger.NDC_SERVER_RECEIVE_REQUEST);
            String s2 = NDCLogger.getInstance().ipToString(NDCLogger.NDC_SERVER_SEND_EXECEPTION);
            String s3 = NDCLogger.getInstance().ipToString(NDCLogger.NDC_CLIENT_SEND_REQUEST);
            String s4 = NDCLogger.getInstance().ipToString(NDCLogger.NDC_CLIENT_RECEIVE_OTHER);
            assertTrue( s1 + " == SERVER_RECEIVE_REQUEST", s1.equals("SERVER_RECEIVE_REQUEST") );
            assertTrue( s1 + " == SERVER_SEND_EXECEPTION", s2.equals("SERVER_SEND_EXECEPTION") );
            assertTrue( s1 + " == CLIENT_SEND_REQUEST", s3.equals("CLIENT_SEND_REQUEST") );
            assertTrue( s1 + " == CLIENT_RECEIVE_OTHER", s4.equals("CLIENT_RECEIVE_OTHER") );
       
            System.out.println("");
            System.out.println("TestRequestFollowUp: Testing NDCLogger::is_operation_to_log().");
            assertTrue( NDCLogger.getInstance().isOperationToLog("AnOperation") + " == true",
                        NDCLogger.getInstance().isOperationToLog("AnOperation") );
            assertTrue( NDCLogger.getInstance().isOperationToLog("add_log") + " == false",
                        ! NDCLogger.getInstance().isOperationToLog("add_log") );
            assertTrue( NDCLogger.getInstance().isOperationToLog("put_formatted_messages") + " == false",
                        ! NDCLogger.getInstance().isOperationToLog("put_formatted_messages") );

            System.out.println("");
            System.out.println("TestRequestFollowUp: Testing NDCLogger::log_request(), with empty NDC List.");
            NDCList ndc_list = new NDCList();
            NDCLogger.getInstance().logRequest( NDCLogger.NDC_CLIENT_SEND_REQUEST, "Operation1", ndc_list );

            System.out.println("");
            System.out.println("TestRequestFollowUp: Testing NDCLogger::log_request(), with 3 NDC in the list.");
            //NDCList ndc_list = new NDCList();
            NDC n1 = new NDC("Operation1:01234567-89AB-CDEF-0123-456789ABCDEF-12");
            NDC n2 = new NDC("Operation2:01234567-89AB-CDEF-0123-456789ABCDEF-1234");
            NDC n3 = new NDC("Operation3:76543210-BA98-FEDC-3210-FEDCBA987654-123456");
            ndc_list.list.addLast( n1 );
            ndc_list.list.addLast( n2 );
            ndc_list.list.addLast( n3 );
            NDCLogger.getInstance().logRequest( NDCLogger.NDC_CLIENT_SEND_REQUEST, "Operation2", ndc_list );
        }
        catch (BadParameterException ex) {
            System.out.println("Invalid NDC string format: " + ex.what());
        }
        catch (SystemException e) {
            System.out.println("Unexpected CORBA System Exception raised " + e.getMessage());
            assertTrue(false);
        }
        //catch (Exception e) {
        //    System.out.println("Unexpected Unknown Exception raised " + e.getMessage());
        //    assertTrue(false);
        //}
        
        System.out.println("");
        System.out.println(" *** TestRequestFollowUp: Testing NDC...");
        try {
            System.out.println("");
            // NDC string format : "%8.8x-%4.4x-%4.4x-%2.2x%2.2x-%2.2x%2.2x%2.2x%2.2x%2.2x%2.2x-%4d"
            NDC ndc1 = new NDC("Operation1:01234567-89AB-CDEF-0123-456789ABCDEF-12");
            NDC ndc2 = new NDC("Operation2", "01234567-89AB-CDEF-0123-456789ABCDEF", 1234);
            NDC ndc3 = new NDC("Operation3:76543210-BA98-FEDC-3210-FEDCBA987654-123456");
            System.out.println("ndc1=" + ndc1.toString());
            System.out.println("ndc2=" + ndc2.toString());
            System.out.println("ndc3=" + ndc3.toString());

            System.out.println("");
            System.out.println(" * TestRequestFollowUp: Testing NDC::NDC(string).");
            {
            String o = ndc1.getOperation();
            String u = ndc1.getUUID();
            int i = ndc1.getRequestId();
            System.out.println("Operation=" + o + ",   UUID=" + u + ",  RequestId=" + i );
            assertTrue( o + " == Operation1", o.equals("Operation1") );
            assertTrue( u + " == 01234567-89AB-CDEF-0123-456789ABCDEF",
                        u.equals("01234567-89AB-CDEF-0123-456789ABCDEF") );
            assertTrue( i + " == 12", i == 12 );
            }

            System.out.println("");
            System.out.println(" * TestRequestFollowUp: Testing NDC::NDC(ope, uuid, req_id).");
            {
            String o = ndc2.getOperation();
            String u = ndc2.getUUID();
            int i = ndc2.getRequestId();
            System.out.println("Operation=" + o + ",   UUID=" + u + ",  RequestId=" + i );
            assertTrue( o + " == Operation2", o.equals("Operation2") );
            assertTrue( u + " == 01234567-89AB-CDEF-0123-456789ABCDEF",
                        u.equals("01234567-89AB-CDEF-0123-456789ABCDEF") );
            assertTrue( i + " == 1234", i == 1234 );
            }

            System.out.println("");
            System.out.println(" * TestRequestFollowUp: Testing NDC::toString().");
            {
            String s1 = ndc1.toString();
            System.out.println("s1=" + s1);
            assertTrue( s1 + " == Operation1:01234567-89AB-CDEF-0123-456789ABCDEF-12",
                        s1.equals("Operation1:01234567-89AB-CDEF-0123-456789ABCDEF-12") );
            String s2 = ndc2.toString();
            System.out.println("s2=" + s2 );
            assertTrue( s2 + " == Operation2:01234567-89AB-CDEF-0123-456789ABCDEF-1234",
                        s2.equals("Operation2:01234567-89AB-CDEF-0123-456789ABCDEF-1234") );
            String s3 = ndc3.toString();
            System.out.println("s3=" + s3);
            assertTrue( s3 + " == Operation3:76543210-BA98-FEDC-3210-FEDCBA987654-123456",
                        s3.equals("Operation3:76543210-BA98-FEDC-3210-FEDCBA987654-123456") );
            }
        }
        catch (BadParameterException ex) {
            System.out.println("Invalid NDC string format: " + ex.what());
        }
        catch (SystemException e) {
            System.out.println("Unexpected CORBA System Exception raised " + e.getMessage());
            assertTrue(false);
        }
        //catch (Exception e) {
        //    System.out.println("Unexpected Unknown Exception raised " + e.getMessage());
        //    assertTrue(false);
        //}

        System.out.println("\n");
        System.out.println(" *** TestRequestFollowUp: Testing NDCFactory.");
        try {
            System.out.println("\n");
            System.out.println("TestRequestFollowUp: Testing NDCFactory::Init_NDC_factory().");
            assertTrue( NDCFactory.getInstance() + " != null", NDCFactory.getInstance() != null );
            String uuid_base = NDCFactory.getInstance().getUUIDBase();
            assertTrue( uuid_base.length() + " == " + UUID.randomUUID().toString().length(),
                        uuid_base.length() == UUID.randomUUID().toString().length() );
            int req_id = NDCFactory.getInstance().getRequestCounter();
            System.out.println("req_id=" + req_id);
            assertTrue( req_id + " >= 0", req_id >= 0 );

            System.out.println("\n");
            System.out.println(" * TestRequestFollowUp: Testing NDCFactory::create_new_ndc().");
            {
            NDC ndc = NDCFactory.getInstance().createNewNDC("AnOperation");
            String o = ndc.getOperation();
            String u = ndc.getUUID();
            int i = ndc.getRequestId();
            String s1 = ndc.toString();  
            assertTrue( o + " == AnOperation", o.equals("AnOperation") );
            assertTrue( u + " == " + uuid_base, u.equals(uuid_base) );
            assertTrue( i + " == " + NDCFactory.getInstance().getRequestCounter(),
                        i == NDCFactory.getInstance().getRequestCounter() );
            assertTrue( i + " == " + (req_id+1), i == req_id+1 );
            String s2 =  o + ":" + u + "-" + i;
            System.out.println("s1=" + s1);
            System.out.println("s2=" + s2);
            assertTrue( s1 + " == " + s2, s1.equals(s2) );
            }

            System.out.println("\n");
            NDC ndc1 = NDCFactory.getInstance().createNewNDC("Operation1");
            NDC ndc2 = new NDC("Operation2:01234567-89AB-CDEF-0123-456789ABCDEF-1234");
            NDC ndc3 = NDCFactory.getInstance().createNewNDC("Operation3");
            System.out.println("ndc1=" + ndc1.toString());
            System.out.println("ndc2=" + ndc2.toString());
            System.out.println("ndc3=" + ndc3.toString());
            String ndcs =  ndc1.toString() + "/" + "Operation2:01234567-89AB-CDEF-0123-456789ABCDEF-1234" + "/" + ndc3.toString();
            System.out.println("ndcs=" + ndcs);

            System.out.println("\n");
            System.out.println(" * TestRequestFollowUp: Testing NDCFactory::toString().");
            {
            NDCList ndc_list = new NDCList();
            ndc_list.list.addLast(ndc1);
            ndc_list.list.addLast(ndc2);
            ndc_list.list.addLast(ndc3);
            String s1 = NDCFactory.getInstance().toString(ndc_list);
            System.out.println("s1=" + s1);
            assertTrue( s1 + " == " + ndcs, s1.equals(ndcs) );
            }

            System.out.println("\n");
            System.out.println(" * TestRequestFollowUp: Testing NDCFactory::fromString().");
            {
            System.out.println("ndcs=" + ndcs);
            NDCList ndc_list = NDCFactory.getInstance().fromString(ndcs);
            System.out.println("nb_ndc=" + ndc_list.list.size());
            NDC n1 = ndc_list.list.removeFirst();
            System.out.println("n1=" + n1.toString());
            assertTrue( n1.toString() + " == " + ndc1.toString(), n1.toString().equals(ndc1.toString()) );
            NDC n2 = ndc_list.list.removeFirst();
            System.out.println("n2=" + n2.toString());
            assertTrue( n2.toString() + " == " + ndc2.toString(), n2.toString().equals(ndc2.toString()) );
            NDC n3 = ndc_list.list.removeFirst();
            System.out.println("n3=" + n3.toString());
            assertTrue( n3.toString() + " == " + ndc3.toString(), n3.toString().equals(ndc3.toString()) );
            assertTrue( ndc_list.list.size() + " == " + 0, ndc_list.list.size() == 0 );
            }

            System.out.println("\n");
            System.out.println(" * TestRequestFollowUp: Testing NDCFactory::fromAny().");
            {
            Any a = this.orb.create_any();
            a.insert_string(ndcs);
            NDCList ndc_list = NDCFactory.getInstance().fromAny(a);
            NDC n1 = ndc_list.list.removeFirst();
            System.out.println("n1=" + n1.toString());
            assertTrue( n1.toString() + " == " + ndc1.toString(), n1.toString().equals(ndc1.toString()) );
            NDC n2 = ndc_list.list.removeFirst();
            System.out.println("n2=" + n2.toString());
            assertTrue( n2.toString() + " == " + ndc2.toString(), n2.toString().equals(ndc2.toString()) );
            NDC n3 = ndc_list.list.removeFirst();
            System.out.println("n3=" + n3.toString());
            assertTrue( n3.toString() + " == " + ndc3.toString(), n3.toString().equals(ndc3.toString()) );
            }
            
            System.out.println("\n");
            System.out.println(" * TestRequestFollowUp: Testing NDCFactory::toAny().");
            {
            NDCList ndc_list = NDCFactory.getInstance().fromString(ndcs);
            Any a = NDCFactory.getInstance().toAny(ndc_list);
            String s1 = a.extract_string();
            System.out.println("s1=" + s1);
            assertTrue( s1 + " == " + ndcs, s1.equals(ndcs));
            }

            System.out.println("\n");
            System.out.println(" * TestRequestFollowUp: Testing NDCFactory::fromOctetSeq().");
            {
            byte [] os = ndcs.getBytes();
            NDCList ndc_list = NDCFactory.getInstance().fromOctetSeq(os);
            assertTrue( os.length + " == " + ndcs.length(), os.length == ndcs.length() );
            NDC n1 = ndc_list.list.removeFirst();
            System.out.println("n1=" + n1.toString());
            assertTrue( n1.toString() + " == " + ndc1.toString(), n1.toString().equals(ndc1.toString()) );
            NDC n2 = ndc_list.list.removeFirst();
            System.out.println("n2=" + n2.toString());
            assertTrue( n2.toString() + " == " + ndc2.toString(), n2.toString().equals(ndc2.toString()) );
            NDC n3 = ndc_list.list.removeFirst();
            System.out.println("n3=" + n3.toString());
            assertTrue( n3.toString() + " == " + ndc3.toString(), n3.toString().equals(ndc3.toString()) );
            }

            System.out.println("\n");
            System.out.println(" * TestRequestFollowUp: Testing NDCFactory::toOctetSeq().");
            {
            NDCList ndc_list = NDCFactory.getInstance().fromString(ndcs);
            byte [] os = NDCFactory.getInstance().toOctetSeq(ndc_list);
            assertTrue( os.length + " == " + ndcs.length(), os.length == ndcs.length() );
            String s1 = new String(os);
            System.out.println("s1=" + s1);
            assertTrue( s1 + " == " + ndcs, s1.equals(ndcs) );
            }

        }
        catch (BadParameterException ex) {
            System.out.println("Invalid NDC string format: " + ex.what());
        }
        catch (SystemException e) {
            System.out.println("Unexpected CORBA System Exception raised " + e.getMessage());
            assertTrue(false);
        }


    }
}
