/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_CDMW_COMMNSVCS_SERVICE_REGISTRATION_NAMINGCONTEXT_IMPL_HPP
#define INCL_CDMW_COMMNSVCS_SERVICE_REGISTRATION_NAMINGCONTEXT_IMPL_HPP

#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/orbsupport/CosNaming.skel.hpp"
#include "Foundation/commonsvcs/serviceregistration/ServiceRegistration_impl.hpp"
#include "Foundation/commonsvcs/naming/RONamingContextExt_impl.hpp"

namespace Cdmw
{
namespace CommonSvcs
{
namespace ServiceRegistration
{

/**
*Purpose:
*<p>
*Actual read/write implementation of the CosNaming::NamingContext
*IDL interface. Rediscovery of service/context only when not in cache.
*<p>
*Features:
*<p>
*Thread safe.
*/
class NamingContext_impl 
   : public Cdmw::CommonSvcs::Naming::RONamingContextExt_impl
{
      
   public:
      
      /**
       * Purpose:
       * <p>
       * Creates this instance.
       */
      NamingContext_impl(ServiceRegistration_impl* serviceRegistration);
      
      /**
       * Purpose:
       * <p>
       * Destroys this instance. If the flag m_destroyed is true, the persistent
       * data is actually destroyed from the persistent storage.
       */
      virtual ~NamingContext_impl();


   protected:
      
      virtual CORBA::Object_ptr 
      resolve(const CosNaming::NameComponent& n)
         throw(CosNaming::NamingContext::NotFound,
               CosNaming::NamingContext::CannotProceed,
               CosNaming::NamingContext::InvalidName,
               CORBA::SystemException);


      virtual void bind(const CosNaming::NameComponent& n,
                        CORBA::Object_ptr obj)
         throw(CosNaming::NamingContext::CannotProceed,
               CosNaming::NamingContext::InvalidName,
               CosNaming::NamingContext::AlreadyBound,
               CORBA::SystemException);
      virtual void rebind(const CosNaming::NameComponent& n,
                          CORBA::Object_ptr obj)
         throw(CosNaming::NamingContext::CannotProceed,
               CosNaming::NamingContext::InvalidName,
               CORBA::SystemException);
      virtual void bind_context(const CosNaming::NameComponent& n,
                                CosNaming::NamingContext_ptr nc)
         throw(CosNaming::NamingContext::CannotProceed,
               CosNaming::NamingContext::InvalidName,
               CosNaming::NamingContext::AlreadyBound,
               CORBA::SystemException);
      virtual void rebind_context(const CosNaming::NameComponent& n,
                                  CosNaming::NamingContext_ptr nc)
         throw(CosNaming::NamingContext::CannotProceed,
               CosNaming::NamingContext::InvalidName,
               CORBA::SystemException);
      virtual void unbind(const CosNaming::NameComponent& n)
         throw(CosNaming::NamingContext::NotFound,
               CosNaming::NamingContext::CannotProceed,
               CosNaming::NamingContext::InvalidName,
               CORBA::SystemException);


      Cdmw::CommonSvcs::Naming::BindingsMap get_all_bindings() const;
      
      
   private:
      /**
       * the servide Registration.
       */
      ServiceRegistration_impl* m_serviceRegistration;
      
      
}; //End of NamingContext_impl

} // End of namespace ServiceRegistration
} // End of namespace CommonSvcs
} // End of namespace Cdmw

#endif // INCL_CDMW_COMMNSVCS_SERVICE_REGISTRATION_NAMINGCONTEXT_IMPL_HPP

