/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include "serviceregistration/RONoCacheNamingContext_impl.hpp"
#include "serviceregistration/SvcLogger.hpp"
#include <Foundation/logging/FunctionLogger.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <iostream>

namespace Cdmw
{
namespace CommonSvcs
{
namespace ServiceRegistration
{


RONoCacheNamingContext_impl::RONoCacheNamingContext_impl(ServiceRegistration_impl* serviceRegistration)
   : m_serviceRegistration(serviceRegistration)
{
   CDMW_LOG_FUNCTION(SvcLogger::GetLogger(),"");
}

RONoCacheNamingContext_impl::~RONoCacheNamingContext_impl()
{
   CDMW_LOG_FUNCTION(SvcLogger::GetLogger(),"");
}


Cdmw::CommonSvcs::Naming::BindingsMap 
RONoCacheNamingContext_impl::get_all_bindings() const
{
   CDMW_LOG_FUNCTION(SvcLogger::GetLogger(),"");
   return m_serviceRegistration->get_known_services();
}

CORBA::Object_ptr 
RONoCacheNamingContext_impl::resolve(const CosNaming::NameComponent& n)
   throw(CosNaming::NamingContext::NotFound,
         CosNaming::NamingContext::CannotProceed,
         CosNaming::NamingContext::InvalidName,
         CORBA::SystemException)
{
   CDMW_LOG_FUNCTION(SvcLogger::GetLogger(),NameComponent_to_string(n));
   CORBA::Object_var obj = CORBA::Object::_nil();
   std::string s = NameComponent_to_string(n);
   try {
      // Force Cache refresh
      obj = m_serviceRegistration->get_service(s.c_str(),true);
   } catch (...) {
      // Ignore
   }
   // NOTA: Service Registration does not allow registration of nil
   //       object references, so a return of a nil object reference 
   //       means that we failed to discover the service within predefined 
   //       time frame.
   //       NamingContext for Ultimate Fallbacks should raise a TRANSIENT system
   //       exception to allow rediscovery of the service again later.
   if (CORBA::is_nil(obj.in())) {
      CORBA::TRANSIENT ex(OrbSupport::TRANSIENTCannotReachNameService, CORBA::COMPLETED_NO);
      CDMW_LOG_FUNCTION_EXCEPTION(ex);
      throw ex;
   }
   return obj._retn();
}

} // End of namespace ServiceRegistration
} // End of namespace CommonSvcs
} // End of namespace Cdmw

