/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include "testserviceregistration/RONamingContext_impl.hpp"
#include <Foundation/logging/FunctionLogger.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <iostream>

namespace Cdmw
{
namespace CommonSvcs
{
namespace ServiceRegistration
{


RONamingContext_impl::RONamingContext_impl(PortableServer::POA_ptr poa,
                                           const char * name,
                                           const Cdmw::CommonSvcs::Naming::BindingsMap& bm)
   : m_poa(PortableServer::POA::_duplicate(poa)),
     m_logger(Cdmw::Logging::Logger::Get_Logger(name)),
     m_bmap(bm)
{
   CDMW_LOG_FUNCTION(m_logger,"");
   std::ostringstream ostr;
   ostr << "Naming Context [" << name << "] created with the following bindings :\n"
        << "{\n";
   using namespace Cdmw::CommonSvcs::Naming;
   for (BindingsMap::const_iterator iter = m_bmap.begin();
        iter != m_bmap.end();
        ++iter)
      ostr << "\t " << (*iter).first << "\n";
   ostr << "}";

   CDMW_INFO(m_logger,ostr.str());
}

RONamingContext_impl::~RONamingContext_impl()
{
   CDMW_LOG_FUNCTION(m_logger,"");
}


Cdmw::CommonSvcs::Naming::BindingsMap 
RONamingContext_impl::get_all_bindings() const
{
   CDMW_LOG_FUNCTION(m_logger,"");
   return m_bmap;
}

CORBA::Object_ptr 
RONamingContext_impl::resolve(const CosNaming::NameComponent& n)
   throw(CosNaming::NamingContext::NotFound,
         CosNaming::NamingContext::CannotProceed,
         CosNaming::NamingContext::InvalidName,
         CORBA::SystemException)
{
   CDMW_LOG_FUNCTION(m_logger,NameComponent_to_string(n));
   CORBA::Object_var obj = CORBA::Object::_nil();
   std::string s = NameComponent_to_string(n);
   Cdmw::CommonSvcs::Naming::BindingsMap::iterator iter = m_bmap.find(s);
   if (iter != m_bmap.end()) {
      obj = (*iter).second;
      CDMW_INFO(m_logger,"resolve("<<s<<") returns object.");
   }
   else {
      CDMW_INFO(m_logger,"resolve("<<s<<") raises NotFound exception.");
      CosNaming::NamingContext::NotFound ex;
      CDMW_LOG_FUNCTION_EXCEPTION(ex);
      throw ex;
   }
   return obj._retn();
}

} // End of namespace ServiceRegistration
} // End of namespace CommonSvcs
} // End of namespace Cdmw

