/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "testserviceregistration/ServiceLocator_impl.hpp"
#include <Foundation/logging/FunctionLogger.hpp>
#include <Foundation/common/Assert.hpp>

namespace Cdmw
{
namespace CommonSvcs
{
namespace ServiceRegistration
{

   ServiceLocator_impl::ServiceLocator_impl(CORBA::ORB_ptr orb,
                                            CosNaming::NamingContextExt_ptr root,
                                            const char * name)
      : m_orb(CORBA::ORB::_duplicate(orb)),
        m_root(CosNaming::NamingContextExt::_duplicate(root)),
        m_logger(Cdmw::Logging::Logger::Get_Logger(name))
    {
        CDMW_LOG_FUNCTION(m_logger,"");
        CDMW_ASSERT(!CORBA::is_nil(orb));
        CDMW_ASSERT(!CORBA::is_nil(root));
        CDMW_ASSERT(!std::string(name).empty());
    }
    
    
    ServiceLocator_impl::~ServiceLocator_impl()
    {
        CDMW_LOG_FUNCTION(m_logger,"");
    }
    
    //
    // location service 
    //
    void ServiceLocator_impl::locate_service (const char * name,
                                              const char * sender)
        throw(CORBA::SystemException)
    {
       CDMW_LOG_FUNCTION(m_logger,"name="<<name<<",sender="<<sender);
       CDMW_INFO(m_logger,"Received a call to locate_service("<<name<<").");
       try {
          CORBA::Object_var obj = m_root->resolve_str(name);
          CORBA::Object_var service_registration_group = m_orb->string_to_object(sender);
          ::CdmwServiceLocator::ServiceRegistration_var sender_ = 
               CdmwServiceLocator::ServiceRegistration::_unchecked_narrow (service_registration_group.in());
          
           // return the reference of the object group
          if (!CORBA::is_nil(obj.in())) {
             CDMW_INFO(m_logger,"Replying with register_service("<<name<<").");
             sender_->register_service(name, obj._retn());
          } else {
             CDMW_INFO(m_logger,"Replying with register_service("<<name<<") and a nil object reference.");
             sender_->register_service(name, obj._retn());
          }
       } catch (const CosNaming::NamingContext::NotFound & ex) {
          CDMW_INFO(m_logger,"Service '"<<name<<"' unknown. Not replying.");
          // do nothing
       } catch (const CORBA::Exception & ex ) {
          CDMW_INFO(m_logger,
                    "Caught CORBA::Exception '"<<ex<<"' while looking for service '"<<name<<"'. Not replying.");  
          // do nothing             
       }
    }

} // End namespace ServiceRegistration
} // End namespace CommonSvcs
} // End namespace Cdmw
