/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "testserviceregistration/TestServiceRegistration.hpp"
#include "testserviceregistration/Config.hpp"
#include "testserviceregistration/Foo.stub.hpp"
#include "Foundation/commonsvcs/naming/NamingInterface.hpp"
#include <Foundation/orbsupport/OrbSupport.hpp>


namespace Cdmw
{
namespace CommonSvcs
{
namespace ServiceRegistration
{

/**
*Implementation notes: [if no pertinent write none ]
*<p>
*Portability issues: [if no pertinent write none ]
*<p>
*/

CPPUNIT_TEST_SUITE_REGISTRATION( TestServiceRegistration );

TestServiceRegistration::TestServiceRegistration ()
    throw()
{
    CORBA::Object_var obj1 = Cdmw::TestUtils::CORBATestManager::instance()->get_object("Root");
    m_root = CosNaming::NamingContextExt::_narrow(obj1.in());
    CORBA::Object_var obj2 = Cdmw::TestUtils::CORBATestManager::instance()->get_object("RootNoCache");
    m_rootNoCache = CosNaming::NamingContextExt::_narrow(obj2.in());
}


TestServiceRegistration::~TestServiceRegistration()
    throw()
{
    // does nothing
}


TestServiceRegistration::TestServiceRegistration(const TestServiceRegistration& rhs)
   throw()
   : m_root(CosNaming::NamingContextExt::_duplicate(rhs.m_root.in())),
     m_rootNoCache(CosNaming::NamingContextExt::_duplicate(rhs.m_rootNoCache.in()))
{
   // does nothing
}
        
TestServiceRegistration&
TestServiceRegistration::operator=(const TestServiceRegistration& rhs)
    throw()
{
   if (this != &rhs) {
      m_root = CosNaming::NamingContextExt::_duplicate(rhs.m_root.in());
      m_rootNoCache = CosNaming::NamingContextExt::_duplicate(rhs.m_rootNoCache.in());
   }
   return *this;
}

void TestServiceRegistration::test_register_service()
{
   // Comming soon ...
}
void TestServiceRegistration::test_register_local_service()
{
   // Comming soon ...
}
void TestServiceRegistration::test_unregister_service()
{
   // Comming soon ...
}
void TestServiceRegistration::test_get_service()
{
   TEST_INFO("Testing resolve_str("<<Config::O1_OBJ<<").");
   try {
      CORBA::Object_var obj =  m_root->resolve_str(Config::O1_OBJ);
      M::Foo_var foo = M::Foo::_narrow(obj.in());
      CORBA::String_var s = foo->op("");
      CPPUNIT_ASSERT(strcmp(s.in(),Config::O1_OBJ) == 0);
   } catch (const CORBA::Exception & ex) {
      std::cerr << "Caught CORBA Exception:" << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex) << std::endl;
      CPPUNIT_ASSERT(false);
   }
   TEST_INFO("Testing resolve_str("<<Config::O2_CTXT<<").");
   try {
      CORBA::Object_var obj =  m_root->resolve_str(Config::O2_CTXT);
      CosNaming::NamingContextExt_var nc = CosNaming::NamingContextExt::_narrow(obj.in());
      CPPUNIT_ASSERT(!CORBA::is_nil(nc.in()));
      obj = nc->resolve_str(Config::O3_OBJ);
      M::Foo_var foo = M::Foo::_narrow(obj.in());
      CORBA::String_var s = foo->op("");
      CPPUNIT_ASSERT(strcmp(s.in(),Config::O3_OBJ) == 0);
      obj = nc->resolve_str(Config::O4_OBJ);
      foo = M::Foo::_narrow(obj.in());
      s = foo->op("");
      CPPUNIT_ASSERT(strcmp(s.in(),Config::O4_OBJ) == 0);
   } catch (const CORBA::Exception & ex) {
      std::cerr << "Caught CORBA Exception:" 
                << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex) << std::endl;
      CPPUNIT_ASSERT(false);
   }
   TEST_INFO("Testing resolve_str("<<Config::O2_CTXT<<"/"<<Config::O3_OBJ<<").");
   try {
      std::ostringstream ostr;
      ostr << Config::O2_CTXT<<"/"<<Config::O3_OBJ;
      CORBA::Object_var obj =  m_root->resolve_str(ostr.str().c_str());
      M::Foo_var foo = M::Foo::_narrow(obj.in());
      CORBA::String_var s = foo->op("");
      CPPUNIT_ASSERT(strcmp(s.in(),Config::O3_OBJ) == 0);
   } catch (const CORBA::Exception & ex) {
      std::cerr << "Caught CORBA Exception:" 
                << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex) << std::endl;
      CPPUNIT_ASSERT(false);
   }
   TEST_INFO("Testing Unknown object resolve_str("<<Config::Unknown_OBJ<<").");
   try {
      CORBA::Object_var obj =  m_root->resolve_str(Config::Unknown_OBJ);
      CPPUNIT_ASSERT(false);
   } catch (const CORBA::Exception & ex) {
      std::cerr << "Caught CORBA Exception:" 
                << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex) << std::endl;
      CPPUNIT_ASSERT(true);
   }
   TEST_INFO("Testing resolve_str("<<Config::Unknown_CTXT<<"/"<<Config::O3_OBJ<<").");
   try {
      std::ostringstream ostr;
      ostr << Config::Unknown_CTXT<<"/"<<Config::O3_OBJ;
      CORBA::Object_var obj =  m_root->resolve_str(ostr.str().c_str());
      CPPUNIT_ASSERT(false);
   } catch (const CORBA::Exception & ex) {
      std::cerr << "Caught CORBA Exception:" 
                << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex) << std::endl;
      CPPUNIT_ASSERT(true);
   }
}

void TestServiceRegistration::do_tests()     
{
   test_register_service();
   test_register_local_service();
   test_unregister_service();
   test_get_service();
}


        
} // End namespace ServiceRegistration
} // End namespace CommonSvcs
} // End namespace Cdmw

