/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "testserviceregistration/TestServiceRegistration.hpp"
#include "testserviceregistration/Config.hpp"
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/Exception.hpp>
#include <Foundation/common/String.hpp>
#include <Foundation/common/Assert.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CdmwObjectIds.hpp>
#include <Foundation/commonsvcs/serviceregistration/ServiceRegistration_impl.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>


class TestThread : public Cdmw::OsSupport::Thread
{
   public:
      TestThread(CORBA::ORB_ptr orb)
         : m_orb(CORBA::ORB::_duplicate(orb))
      {
      }
      virtual ~TestThread() throw() {}
      
      virtual void run() throw()
      {
         m_orb->run();
      }
   private:
      CORBA::ORB_var m_orb;
};
class TestDriver : public Cdmw::OsSupport::Thread
{
   public:
      TestDriver(CORBA::ORB_ptr orb)
         : m_orb(CORBA::ORB::_duplicate(orb))
      {
      }
      virtual ~TestDriver() throw() {}
      
      virtual void run() throw()
      {
         int timescale = Cdmw::TestUtils::get_timescale();
         Cdmw::OsSupport::OS::sleep(timescale*2000);
         CORBA::Object_var obj = m_orb->resolve_initial_references("NameService");
         CDMW_ASSERT(!CORBA::is_nil(obj.in()));
         Cdmw::TestUtils::CORBATestManager::instance()->add("Root", obj.in());
         obj = m_orb->resolve_initial_references(Cdmw::OrbSupport::SERVICE_REGISTRATION_NO_CACHE_ObjectId);
         CDMW_ASSERT(!CORBA::is_nil(obj.in()));
         Cdmw::TestUtils::CORBATestManager::instance()->add("RootNoCache", obj.in());
         Cdmw::TestUtils::CORBATestManager::instance()->run_tests();
         Cdmw::TestUtils::CORBATestManager::instance()->summary();
         m_orb->shutdown(false);
      }
   private:
      CORBA::ORB_var m_orb;
};

void init_service_registration(CORBA::ORB_ptr orb,
                               PortableServer::POA_ptr poa)
{
   using namespace Cdmw::CommonSvcs::ServiceRegistration;
   using namespace Cdmw::OsSupport;
   std::string locator_service_address = 
      SystemProperties::Get_Property(Config::LOCATOR_MIOP_ADDRESS_PROP,
                                     Config::LOCATOR_DEFAULT_MIOP_ADDRESS);
   std::string registration_service_address = 
      SystemProperties::Get_Property(Config::REGISTRATION_MIOP_ADDRESS_PROP,
                                     Config::REGISTRATION_DEFAULT_MIOP_ADDRESS);
   // Get NB retries
   int nb_retries = 
      SystemProperties::Get_Property(Config::NB_RETRIES_PROP, 
                                     Config::DEFAULT_NB_RETRIES);
   // Get Time interval
   int time_interval = 
      SystemProperties::Get_Property(Config::TIME_INTERVAL_PROP,
                                     Config::DEFAULT_TIME_INTERVAL_ms);
   PortableGroup::GOA_var goa = PortableGroup::GOA::_narrow(poa);
   CDMW_ASSERT(!CORBA::is_nil(goa.in()));
   (void*) ServiceRegistration_impl::Init(orb,
                                          goa.in(),
                                          locator_service_address ,
                                          registration_service_address,
                                          nb_retries,
                                          time_interval);
   
};


int main (int argc, char* argv[])
{
   int result = 0;
   Cdmw::OsSupport::OS::ProcessId idSvc = 0;
   try {
      Cdmw::OrbSupport::StrategyList strategyList;
      strategyList.add_OrbThreaded();
      strategyList.add_PoaThreadPool( 10 );
      strategyList.add_multicast();
      
      CORBA::ORB_var orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, strategyList ); 
      
      // Activate the POA manager
      CORBA::Object_var obj
         = orb->resolve_initial_references( "RootPOA" );
      PortableServer::POA_var rootPOA = PortableServer::POA::_narrow( obj.in() );
      
      PortableServer::POAManager_var poaMgr = rootPOA->the_POAManager();
      poaMgr->activate();
      using namespace Cdmw::CommonSvcs::ServiceRegistration;
      std::string locator_service_address = 
         Cdmw::OsSupport::SystemProperties::Get_Property(Config::LOCATOR_MIOP_ADDRESS_PROP,
                                                         Config::LOCATOR_DEFAULT_MIOP_ADDRESS);
      std::ostringstream ostr;
      ostr<<"-D"<<Config::LOCATOR_MIOP_ADDRESS_PROP<<"="<<locator_service_address;

      idSvc = Cdmw::OsSupport::OS::create_process( "testserviceregistration_svc", ostr.str() ); 
      int timescale = Cdmw::TestUtils::get_timescale();
      Cdmw::OsSupport::OS::sleep(timescale*3000);
      init_service_registration(orb.in(),rootPOA.in());

      Cdmw::TestUtils::CORBATestManager::instance()->init(orb.in(),rootPOA.in(),argc,argv);
      std::auto_ptr<TestDriver> thr(new TestDriver(orb.in()));
      thr->start();
      orb->run();
      thr->join();     
      Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
      orb->destroy();      
      result = Cdmw::TestUtils::CORBATestManager::instance()->was_successful();
      
   } catch (const CORBA::SystemException& e) {
      std::cerr << "Unexpected CORBA::SystemException:\n"
                << e << " - minor code: " << e.minor()
                << std::endl;
   }   
   // And we kill the Name Server
   try {
      Cdmw::OsSupport::OS::kill_process( idSvc );
   } catch (const std::exception& ex)
   {
      std::cerr << "["<<__LINE__<<"] Unexpected std exception:\n"
                << ex.what() << std::endl;
   }
   return result;
}

