/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/Exception.hpp>
#include <Foundation/common/String.hpp>
#include <Foundation/common/Assert.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/ossupport/OS.hpp>
#include "testserviceregistration/ServiceLocator_impl.hpp"
#include "testserviceregistration/RONamingContext_impl.hpp"
#include "testserviceregistration/Foo_impl.hpp"
#include "testserviceregistration/Config.hpp"
#include <Foundation/ossupport/SystemProperties.hpp>
#include <cstdlib>


namespace
{    
   const int SUCCESS = 0;
   const int FAILURE = 1;

   CosNaming::NamingContextExt_ptr setup_service_context(CORBA::ORB_ptr orb,
                                                         PortableServer::POA_ptr poa,
                                                         const char * name)
   {
      /*
       *   /-O1
       *   |-O2/-03
       *   |   |-04 
       *
       */
      using namespace Cdmw::CommonSvcs;
      PortableServer::ServantBase_var Obj3_impl 
         = new ServiceRegistration::Foo_impl(orb,poa,
                                             ServiceRegistration::Config::O3_OBJ);
      M::Foo_var Obj3_var = dynamic_cast<ServiceRegistration::Foo_impl*>(Obj3_impl.in())->_this();

      PortableServer::ServantBase_var Obj4_impl 
         = new ServiceRegistration::Foo_impl(orb,poa,
                                             ServiceRegistration::Config::O4_OBJ);
      M::Foo_var Obj4_var = dynamic_cast<ServiceRegistration::Foo_impl*>(Obj4_impl.in())->_this();

      Naming::BindingsMap bm2;
      bm2[ServiceRegistration::Config::O3_OBJ]=CORBA::Object::_duplicate(Obj3_var.in());
      bm2[ServiceRegistration::Config::O4_OBJ]=CORBA::Object::_duplicate(Obj4_var.in());

      PortableServer::ServantBase_var Obj2_impl = 
         new ServiceRegistration::RONamingContext_impl(poa,
                                                       ServiceRegistration::Config::O2_CTXT,
                                                       bm2);
      CosNaming::NamingContextExt_var Obj2_var = 
         dynamic_cast<ServiceRegistration::RONamingContext_impl*>(Obj2_impl.in())->_this();

      PortableServer::ServantBase_var Obj1_impl 
         = new ServiceRegistration::Foo_impl(orb,poa,
                                             ServiceRegistration::Config::O1_OBJ);
      M::Foo_var Obj1_var = dynamic_cast<ServiceRegistration::Foo_impl*>(Obj1_impl.in())->_this();
      
      Naming::BindingsMap bm1;
      bm1[ServiceRegistration::Config::O1_OBJ]=CORBA::Object::_duplicate(Obj1_var.in());
      bm1[ServiceRegistration::Config::O2_CTXT]=CORBA::Object::_duplicate(Obj2_var.in());
      PortableServer::ServantBase_var Obj0_impl =
         new ServiceRegistration::RONamingContext_impl(poa,
                                                       name,
                                                       bm1);
      CosNaming::NamingContextExt_var Obj0_var = 
         dynamic_cast<ServiceRegistration::RONamingContext_impl*>(Obj0_impl.in())->_this();
      return Obj0_var._retn();
   }
   
};

int main (int argc, char* argv[])
{
   int status = SUCCESS;
   CORBA::ORB_var orb;

   try
   {
      Cdmw::OrbSupport::StrategyList strategyList;
      strategyList.add_OrbThreaded();
      strategyList.add_PoaThreadPool( 10 );
      strategyList.add_multicast();

      orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, strategyList ); 
      
      // Activate the POA manager
      CORBA::Object_var obj = orb->resolve_initial_references( "RootPOA" );
      PortableServer::POA_var rootPOA = PortableServer::POA::_narrow( obj.in() );
      
      PortableServer::POAManager_var poaMgr = rootPOA->the_POAManager();
      poaMgr->activate();
      using namespace Cdmw::CommonSvcs::ServiceRegistration;
      CosNaming::NamingContextExt_var service_root =  
         setup_service_context(orb.in(),
                               rootPOA.in(),
                               Config::SERVICE_CONTEXT_NAME);

      // Instantiate a service locator object
      std::string multicast_addr = 
         Cdmw::OsSupport::SystemProperties::Get_Property(Config::LOCATOR_MIOP_ADDRESS_PROP,
                                                         Config::LOCATOR_DEFAULT_MIOP_ADDRESS);
      CORBA::Object_var group = orb->string_to_object(multicast_addr.c_str());
      
      PortableGroup::GOA_var goa = PortableGroup::GOA::_narrow( rootPOA.in() );
      
      PortableServer::ObjectId_var  service_locator_id = goa->create_id_for_reference(group.in());

      ServiceLocator_impl *service_locator_obj 
         = new  ServiceLocator_impl(orb.in(),
                                    service_root.in(),
                                    Config::SERVICE_LOCATOR_NAME);
      PortableServer::ServantBase_var servant = service_locator_obj;

      goa->activate_object_with_id (service_locator_id.in (),service_locator_obj);

      orb->run();
      status=SUCCESS;
   }
   catch(const CORBA::Exception& ex)
   {
      std::cerr << "["<<__LINE__<<"] Unexpected CORBA::Exception:\n"
                << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex)
                << std::endl;
      status = FAILURE;
   } catch (const std::exception& ex)
   {
      std::cerr << "["<<__LINE__<<"] Unexpected std exception:\n"
                << ex.what() << std::endl;
      status = FAILURE;
   }
   
   if(!CORBA::is_nil(orb.in()))
   {
      try
      {
         Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
         orb -> destroy();
      }
      catch(const CORBA::Exception& ex)
      {
         std::cerr << "["<<__LINE__<<"] Unexpected CORBA::Exception:\n"
                   << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex)
                   << std::endl;
         status = FAILURE;
      }
   }
}

