/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <sstream>
#include <limits>
#include <ctime>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>






#include <Foundation/commonsvcs/time/Util.hpp>



// definition of local constants in an unnammed namespace
namespace 
{ 
    // nbr of 100 nsec in 1 sec
    const TimeBase::TimeT HUNDRED_OF_NSEC_IN_SEC = static_cast<TimeBase::TimeT>(10000000);
    
    // TimeVal is in sec/microsec from 1st Jan 1970 (This is different in Windows!!!)
    //
    // Time difference in 100 nanoseconds between DCE and POSIX
    // is 141427 days.  
    const TimeBase::TimeT DELTA_POSIX_DCE = 
                static_cast<TimeBase::TimeT>(141427) * 
                static_cast<TimeBase::TimeT>(86400) *
                HUNDRED_OF_NSEC_IN_SEC;
    // script shell to obtain informaion from ntp tools

    const std::string ntpq_exe                 = "/usr/sbin/ntpq";
    const std::string ntpq_command             = ntpq_exe + " -p -c peers 2>&1";
    const int expectedLinesOuput               = 3;
    const int number_Of_NTPQ_output_fields     = 10; 
 
    int parseNTPLine (char* line);
    bool is_NTPcommand_executable();
    int calculateStatusFromStratumValue(int ntp_current_stratum);
    
    // is Time Server 
    //bool isTS = false;
    //is syncrhonized
    bool isSync = false;
    
    int Sdeg     = 0;
    int Sfail    = 0;
    //const int SdegTC     = 10;
    //const int SfailTC    = 10;
    
    
    
    
 
}

  
namespace Cdmw 
{ 
namespace CommonSvcs 
{ 
  namespace Time 
  {
    namespace Util
    {

      OsSupport::OS::Timeval
         TimeT_2_Timeval (const TimeBase::TimeT& input_time)
      {
          OsSupport::OS::Timeval output_tv = naTimeval;

          // compute from input_time the integral number of seconds since the Epoch
    
          // get time in 100nsec from epc
          TimeBase::TimeT timet_epc = input_time - DELTA_POSIX_DCE;
    
          // if epoch time is valid
          if (timet_epc > 0)
          {
              // get time in sec from epc
              TimeBase::TimeT time_sec = timet_epc / HUNDRED_OF_NSEC_IN_SEC;
              TimeBase::TimeT time_hund_of_nsec = 
                      timet_epc - (time_sec * HUNDRED_OF_NSEC_IN_SEC);

              if (time_sec < static_cast<TimeBase::TimeT>(std::numeric_limits<long>::max()))
              {   
                  // assign tv.seconds
                  output_tv.seconds  = static_cast<long>(time_sec);

                  // assign tv.microseconds
                  output_tv.microseconds = static_cast<long>(time_hund_of_nsec/10);
              }
          }

          return output_tv;
      }

      TimeBase::TimeT
         Timeval_2_TimeT(const OsSupport::OS::Timeval& input_tv)
      {
          TimeBase::TimeT out_time = naTimeT;
          
          // if input Timeval is valid convert it
          if (!(input_tv == naTimeval))
          {
              out_time = 
                  DELTA_POSIX_DCE +
                  static_cast<TimeBase::TimeT>(input_tv.seconds ) * HUNDRED_OF_NSEC_IN_SEC +
                  static_cast<TimeBase::TimeT>(input_tv.microseconds) * 10;
          }
          
          return out_time;
      }
      
      TimeBase::TimeT
         change_base(const TimeBase::TimeT& input_time,
                     Format from, Format to)
      {
          TimeBase::TimeT out_time = input_time;
          
          // if conversion requested and input valid 
          if (from != to && out_time != naTimeT)
          {
              if (from == EPOCH && to == GREGORIAN)
              {
                  out_time += DELTA_POSIX_DCE;
              }
              else if (from == GREGORIAN && to == EPOCH)
              {
                  if (out_time >= DELTA_POSIX_DCE)
                  {
                      out_time -= DELTA_POSIX_DCE;
                  }                
              }
          }
        
          return out_time;
      }
    

      TimeBase::TimeT get_current_time()
             throw (::Cdmw::InternalErrorException)
      {
          // first get TimeVal from OsSupport
          TimeBase::TimeT current_time = 
              Timeval_2_TimeT(::Cdmw::OsSupport::OS::get_time());

          return current_time;
      }

      bool is_synchronized()
             throw (::Cdmw::InternalErrorException) 
      {
         // TO BE COMPLETED
         return true;
      } 
 
  /*****************************************
         * Purpose:
         * <p> calculate the status according with stratum criteria
         *
         * @return   :
         *                EXTERNAL_SYNCHRONIZED                  :  if CurrentStratum < Sdeg
         *                INTERNAL_SYNCHRONIZED                  :  if Sdeg <= CurrentStratum< Sfail
         *                NOT_SYNCHRONIZED                             :  if Sfail <= CurrentStratum
         */
  /*****************************************/ 
    int calculateStatusFromStratumValue(int ntp_current_stratum)
    {
        int result = UNKNOWN;
      
        if (ntp_current_stratum < Sdeg)
            result = EXTERNAL_SYNCHRONIZED; 
        if ((Sdeg <= ntp_current_stratum) && ( ntp_current_stratum < Sfail))
            result = INTERNAL_SYNCHRONIZED; 
        if (Sfail <= ntp_current_stratum )    
            result = NOT_SYNCHRONIZED;   
            
        return result;
    } 
  /*****************************************
         * Purpose:
         * <p> analyse ntpq data and determinate the status
         *
         * @return STATUS_NOT_YET_DETERMINATED  :   if it is needed to parse other NTPQ output lines
         *                LOCAL_SYNCHRONIZED                         :   if the node is TS synchronized on its Local Clock
         *                EXTERNAL_SYNCHRONIZED                  :  if the node is synchronized on an external reference (on the basis of current stratum)
         *                INTERNAL_SYNCHRONIZED                  :  if the node is synchronized on a clock of an internal peer (on the basis of current stratum)
         *                NOT_SYNCHRONIZED                             :  if the node not synchronized  (on the basis of current stratum)
         */
  /*****************************************/
      
      int parseNTPLine (char* line)
      {
            
           char *ntp_fields[number_Of_NTPQ_output_fields];
           int            i                 = 0;
           int            ntp_source_index  = 0;
           bool           isAsteriskInLine  = false;
           bool           isLOCAL           = false;
           int            result            = STATUS_NOT_YET_DETERMINATED;
           std::string    ntp_source;
           ntp_fields[0] = (char*)strtok(line, " ");
           while (ntp_fields[i] != NULL) {
                       ntp_fields[++i] = (char*)strtok(NULL, " ");
           }
           ntp_source = ntp_fields[0];
           if (ntp_source.find("*", 0) == 0) 
           {
              isSync = true;
              isAsteriskInLine = true;
              ntp_source_index = 1;    
           }
           ntp_source = ntp_source.substr(ntp_source_index,ntp_source.length());
           
           if (ntp_source == "LOCAL(0)") // || ntp_source == "127.127.1.0") isLOCAL=true;
                isLOCAL = true;
                
           
           if (!isAsteriskInLine)
              //It is not synchronized : Need to parse the other lines       
              result = STATUS_NOT_YET_DETERMINATED; 
           if (isLOCAL && isAsteriskInLine)
              // TS synchronized on its Local Clock
              result = LOCAL_SYNCHRONIZED;        
           if ( (!isLOCAL) && (isAsteriskInLine) )
              // It is  synchronized with another time source 
              //  the current_stratum is used to set the status     
              {                            
                int current_stratum = atoi(ntp_fields[2]); 
                result = calculateStatusFromStratumValue(current_stratum); 
              } 
           return result;
      }

/*****************************************
         * Purpose:
         * <p> check if ntp command is executable
         *
         * @return false in case of error
         *
         */
/*****************************************/
    bool is_NTPcommand_executable()
      {
         struct stat buf;
         int res = stat(ntpq_exe.c_str(), &buf);      
         if (res == -1) 
            return false;  
         return true;   
         
      }       


    
      
  /*****************************************
         * Purpose:
         * <p> execute NTPQ command and analyse the output in order to determinate the status
         *
         * @return UNKNOWN                                                 :   if it was not possible to execute ntpq command
         *                LOCAL_SYNCHRONIZED                         :   if the node is TS synchronized on its Local Clock
         *                EXTERNAL_SYNCHRONIZED                  :  if the node is synchronized on an external reference (on the basis of current stratum)
         *                INTERNAL_SYNCHRONIZED                  :  if the node is synchronized on a clock of an internal peer (on the basis of current stratum)
         *                NOT_SYNCHRONIZED                             :  if the node not synchronized  
         */
  /*****************************************/
      NTP_Status get_synchronization_status(int Sdegraded, int Sfailed)
             throw (::Cdmw::InternalErrorException)
      {
        NTP_Status status = STATUS_NOT_YET_DETERMINATED;//UNKNOWN;
        isSync            = false;
        Sdeg              = Sdegraded;
        Sfail             = Sfailed;
        char * line       = NULL;
        size_t len        = 0;
        int numberLine    = 0;

        if (!is_NTPcommand_executable()) {  
             // It was not possible to execute the ntpq command
                 throw ::Cdmw::InternalErrorException
                        (" : It was not possible to execute " + ntpq_exe + " command") ; 
                 return UNKNOWN;
        }
        FILE *f = popen(ntpq_command.c_str(), "r");   
        if (f == NULL) {  
                 // It was not possible to execute the ntpq command
                 throw ::Cdmw::InternalErrorException(" : It was not possible to determinate NTP status") ; 
                 return UNKNOWN;
        }
        // get line from ntpq command ( until the status is not determinated) 
        while ( ((getline(&line, &len, f)) != -1) && (status == STATUS_NOT_YET_DETERMINATED)) {
            numberLine++; 
            if ((numberLine >= expectedLinesOuput)  )   
                        // parse only the line where are stored significant datas
                        status = static_cast<NTP_Status> (parseNTPLine(line));
        }                
        if (line) free(line);
        pclose(f);
        
        if (numberLine < expectedLinesOuput)    
                status = NOT_SYNCHRONIZED;
        if ( (status == STATUS_NOT_YET_DETERMINATED) && (!isSync) )
               // It is a Time Server or Time Client not synchronized 
                status = NOT_SYNCHRONIZED;
        return status;
      }


        std::string
        TimeT2string_ISO8601(const TimeBase::TimeT& tmp )
        {
            //Format string - according to ISO 8601
            return ::Cdmw::CommonSvcs::Time::Util::TimeT2formatted_string(tmp, "%Y-%m-%dT%H:%M:%S");        
        }
        
        std::string
        TimeT2formatted_string(const TimeBase::TimeT& tmp, char* format_str)
        {
            ::Cdmw::OsSupport::OS::Timeval tv;
            //Convert Gregorian time of TimeT into Epoch(01/01/1970)
            tv = ::Cdmw::CommonSvcs::Time::Util::TimeT_2_Timeval(tmp);

            char pTimeBuffer[20];
            Cdmw::OsSupport::OS::get_fmt_time(
                pTimeBuffer,//Buffer to return formatted time
                20,         //Size of Buffer
                tv.seconds, //Time to format
                format_str, //Format string
                Cdmw::OsSupport::OS::LOCAL_TIME);//GMT or LOCAL time

            return std::string(pTimeBuffer);
        }

    }; // End namespace Util
  }; // End namespace Time
}; // End namespace CommonSvcs
}; // End namespace Cdmw

