/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "testtimeutil/TestTimeUtil.hpp"

#include "Foundation/commonsvcs/time/Util.hpp"

#include <string>
#include <exception>
#include <iostream>
#include <sstream>

using namespace Cdmw::CommonSvcs::Time;
using namespace Cdmw::OsSupport;

// definition of local constants in an unnammed namespace
namespace 
{ 
    // Time difference in 100 nanoseconds between DCE and POSIX
    // is 141427 days.  
    const TimeBase::TimeT DELTA_POSIX_DCE = 
                static_cast<TimeBase::TimeT>(141427) * 
                static_cast<TimeBase::TimeT>(86400) *
                static_cast<TimeBase::TimeT>(10000000);
}

CPPUNIT_TEST_SUITE_REGISTRATION (TestTimeUtil);

// ----------------------------------------------------------------------
// TestTimeUtil class.
// ----------------------------------------------------------------------
TestTimeUtil::TestTimeUtil ()
  throw()
{
}

TestTimeUtil::~TestTimeUtil()
  throw()
{
}

void TestTimeUtil::do_tests()
{
    try
    {
        TEST_INFO ("get epoch time from ossupport");
        OS::Timeval time1 = OS::get_time();
        CPPUNIT_ASSERT(true);
        
        TEST_INFO ("get dce time from timeutil");
        TimeBase::TimeT timet1 = Util::get_current_time();
        CPPUNIT_ASSERT(true);
        
        TEST_INFO ("convert dce time into epoch time");
        OS::Timeval time2 = Util::TimeT_2_Timeval (timet1);
        
        if (!(time2 == Util::naTimeval))
        {
            CPPUNIT_ASSERT(true);
        }
        else
        {
            CPPUNIT_ASSERT(false);
            return;
        }
        
        TEST_INFO ("check time values");
        
        long time_diff = time2.seconds - time1.seconds;
        
        // test failed if difference is > 1sec
        if (time_diff < 0 || time_diff > 1)
        {
            CPPUNIT_ASSERT(false);
        }
        else
        {
            CPPUNIT_ASSERT(true);
        }
        
        TimeBase::TimeT time_diff2 = 
          (timet1 / 10000000) - static_cast<TimeBase::TimeT>(time1.seconds);
          
        // test failed if difference is > DELTA_POSIX_DCE + 1sec
        TimeBase::TimeT time_diff_max = (static_cast<TimeBase::TimeT>(141427) * 
                                         static_cast<TimeBase::TimeT>(86400)) + 
                                         static_cast<TimeBase::TimeT>(1);
        if (time_diff2 > time_diff_max)
        {
            CPPUNIT_ASSERT(false);
        }
        else
        {
            CPPUNIT_ASSERT(true);
        }
        
        // convert Gregorian TimeT to Epoch
        TimeBase::TimeT timet2 = timet1 - DELTA_POSIX_DCE;
        
        TimeBase::TimeT timet3 = Util::change_base(timet1, Util::GREGORIAN, Util::EPOCH);
        
        if (timet3 == timet2)
        {
            CPPUNIT_ASSERT(true);
        }
        else
        {
            CPPUNIT_ASSERT(false);
        }
        
        TimeBase::TimeT timet4 = Util::change_base(timet2, Util::EPOCH, Util::GREGORIAN);
        
        if (timet4 == timet1)
        {
            CPPUNIT_ASSERT(true);
        }
        else
        {
            CPPUNIT_ASSERT(false);
        }


        TEST_INFO ("check NTP Status of Time Server");
        try {
            int Sdeg  = 6;
            int Sfail = 13;            
            Util::NTP_Status ntpStatus = Util::get_synchronization_status(Sdeg,Sfail);
            if (ntpStatus == Util::UNKNOWN)
            {
                TEST_INFO ("ERROR : it was not possible to obtain NTP status");
                CPPUNIT_ASSERT(false);
            }
            if (ntpStatus == Util::LOCAL_SYNCHRONIZED)
            {
                TEST_INFO ("The host is synchronized on its local clock");
                CPPUNIT_ASSERT(true);
            }
            if (ntpStatus == Util::EXTERNAL_SYNCHRONIZED)
            {
                TEST_INFO ("The host is externally synchronized");
                CPPUNIT_ASSERT(true);
            }
            if (ntpStatus == Util::NOT_SYNCHRONIZED)
            {
                TEST_INFO ("The host is not synchronized");
                CPPUNIT_ASSERT(true);
            }
            if (ntpStatus == Util::INTERNAL_SYNCHRONIZED)
            {
                TEST_INFO ("The host is internal synchronized ");
                CPPUNIT_ASSERT(true);
            }        
            } catch (::Cdmw::InternalErrorException iee)
            {
                TEST_INFO ("ERROR : it was not possible to obtain NTP status");
                CPPUNIT_ASSERT(false);
            }
    }
    catch (const Cdmw::Exception& e)
    {
        std::cout << e.what() << std::endl;
        CPPUNIT_ASSERT(false);
    }

}


