/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef _CDMW_FDS_LOGGING_FATALHANDLER_HPP_
#define _CDMW_FDS_LOGGING_FATALHANDLER_HPP_

#include <Foundation/logging/CdmwLoggingCollector.stub.hpp>


namespace Cdmw
{
    namespace Logging
    {
        /**
         * Handler to be registered in LogManager.
         * The on_fatal_log() operation will be called when a FATAL message
         * is logged.
         */
        class FatalHandler 
        {
            public:
                /**
                 * Destructor.
                 */
                virtual
                ~FatalHandler()
                    throw()
                {
                }


                /**
                 * This operation is called by LogManager when a FATAL message
                 * is logged.
                 *
                 * @param header the message header.
                 * @param msg the message.
                 */
                virtual void
                on_fatal_log(const CdmwLogging::LogHeader& header, const char* msg)
                    throw() = 0;

        };
    } // end namespace Logging
} // end namespace Cdmw


#endif // _CDMW_FDS_LOGGING_FATALHANDLER_HPP_
