/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef _CDMW_FDS_LOGGING_LOGMANAGERDEFAULTDELEGATE_HPP_
#define _CDMW_FDS_LOGGING_LOGMANAGERDEFAULTDELEGATE_HPP_


#include <string>

#include <Foundation/osthreads/Mutex.hpp>
#include <Foundation/common/Exception.hpp>
#include <Foundation/logging/LogManagerDelegate.hpp>
#include <Foundation/logging/FilterMngr.hpp>
#include <fstream>


namespace Cdmw
{
    namespace Logging
    {
        /**
         * This class enables logging in standard output or file.
         */
        class LogManagerDefaultDelegate: public LogManagerDelegate
        {
            public:
                /**
                 * Constructor.
                 */
                LogManagerDefaultDelegate()
                    throw(OutOfMemoryException);

                /**
                 * Constructor.
                 * @param file_name output log file
                 * @exception BadParameterException when the specified file is not writable
                 */
                LogManagerDefaultDelegate(std::string file_name)
                    throw(OutOfMemoryException, BadParameterException);

                /**
                 * Destructor.
                 */
                ~LogManagerDefaultDelegate()
                    throw();

                /**
                 * Send a message to the logger.
                 *
                 * @param context the message context.
                 * @param header the message header.
                 * @param msg the message.
                 *
                 * @return true if successful.
                 */
                virtual bool
                add_log(const CdmwLogging::LogContext& context,
                        const CdmwLogging::LogHeader& header,
                        const char* msg)
                    throw(InternalErrorException);


            protected:

                /**
                 * timestamp_to_string() utility function
                 * @param tv the time to format (only seconds are used)
                 * @param format the format to use
                 */
                std::string timestamp_to_string(CdmwLogging::Timeval tv,
                                                const char* format) const;

                /**
                 * Replaces an empty string by "[]" and truncate to BUFFER_MAX_LENGTH if needed
                 * @param buffer stores the result
                 * @param format how to represent the value
                 * @param value the data to manage
                 */
                void manage_field(char *buffer,
                                  const char *format,
                                  const char *value) const;
            
                /**
                 * Format horizontal log header
                 * @param os the stream where to build the log header
                 * @param context the log context
                 * @param header the log header
                 */
                void format_log(std::ostringstream & os,
                                const CdmwLogging::LogContext& context,
                                const CdmwLogging::LogHeader& header) const
                    throw();

            private:

                std::ofstream m_ofstream;
                std::ostream* m_postream;
                OsSupport::Mutex m_mutex;
        };
    } // end namespace Logging
} // end namespace Cdmw


#endif // _CDMW_FDS_LOGGING_LOGMANAGERDEFAULTDELEGATE_HPP_
