/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/logging/Logger.hpp>
#include <Foundation/common/Assert.hpp>
#include <Foundation/logging/LogManager.hpp>



namespace Cdmw
{
    namespace Logging
    {
        /**
         * Find the logger associated to the specified domain name.
         */
        Logger*
        Logger::Get_Logger(const std::string& domain)
        {
            Logging::LogManager* logMngr = LogManager::Instance();
            return logMngr->get_logger("", domain);
            
        }
        

        /**
         * Find the logger associated to the specified component name and
         * domain name.
         */
        Logger*
        Logger::Get_Logger(const std::string& component_name,
                           const std::string& domain)
        {
            Logging::LogManager* logMngr = LogManager::Instance();
            return logMngr->get_logger(component_name, domain);
        }
        
        /**
         * Get the logger associated to the specified component name and
         * domain name and attached to the specified LogManager.
         * NOTA: If the logger don't exist in the logger list, create a new instance
         */
        Logger*
        Logger::Get_Logger(Logging::LogManager* manager,
                           const std::string& component_name,
                           const std::string& domain)
        {
            CDMW_ASSERT(manager != 0);
            return manager->get_logger(component_name, domain);
        }
        
        
    } // end namespace Logging
} // end namespace Cdmw
