/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.logging;

public class Filter
{
    /**
     * the level mask
     */
    
    protected LevelType m_levels = new LevelType();
    
    
    /**
     * The domain name.
     */

    protected boolean m_domain;
    
    /**
     * flag indicating if filter is associated
     * to a component or to a domain
     */

    protected boolean m_domainFilter;
    
    /**
     * The Level mask
     */

    protected LogLevel Logging = new LogLevel();
    
   /**
    * Purpose:
    * <p>  Create a new Filter.
    *
    */
        
    public Filter() {
    }

   /**
    * Purpose:
    * <p>  Create a new Filter with a specified level mask.
    *
    * @param levels the specified level mask
    */
    
    public Filter(long levels) {
        m_levels.setType(levels);
        m_domainFilter = true; 
    }
    
    /**
    * Purpose:
    * <p>  Create a new Filter with a specified level mask.
    *
    * @param levels the specified level mask
    */

    public Filter(LevelType levels) {
        m_levels.copy(levels);
        m_domainFilter = true; 
    }

    /**
     * Purpose:
     * <p>   Get the level of the filter
     *
     * @return the level value
     */

    public LevelType getLevel() {
        return m_levels;
    }

    /**
     * Purpose:
     * <p>   Set a default level to be set to all newly created filters
     *
     * @param level the default level
     */
    public synchronized void setDefaultLevel(LevelType level) {
        m_levels.setType(level.getType());
    }

    /**
     * Purpose:
     * <p>   Activate a specific level of the filter
     * note : in the input level the level bits to activate are set
     *
     * @param level the level to activate
     */
 
    synchronized public void activateLevel (long level) {       
         m_levels.setType(m_levels.getType() | level);
     }
        
     /**
      * Purpose:
      * <p>   Activate all levels of the filter.
      */
 
    synchronized public void activateAllLevels () {
         m_levels.setType(Logging.ALL_LEVELS_SET);
     }

    /**
     * Purpose:
     * <p>   Deactivate a specific level of the filter
     * note : in the input level the level bits to deactivate are set
     *
     * @param level the level to deactivate
     */

    synchronized public void deactivateLevel (long level) {
         long arg;
         long mlevel;
         arg = ~level;
         mlevel = m_levels.getType();
         long temp = mlevel & arg;
         m_levels.setType(temp);
     }

    /**
     * Purpose:
     * <p>   Deactivate all levels of the filter.
     */

    synchronized public void deactivateAllLevels () {
         m_levels.setType(0);
     }

    /**
     * Purpose:
     * <p>   Check if a level is activated in the filter.
     * note : in the input level the level bits to check are set
     *
     * @param level the level to check
     *
     * @return true if all level bits to check are set or false
     */

    synchronized public boolean isLevelActivated(long level) {
         boolean result = false;
         long activation = m_levels.getType() & level;

         // if result is equal to requested level, filter is activated
         if (activation == level)
         {
             result = true;
         }

         return result;
     }

    /**
     * Purpose:
     * <p>   Check if the filter is associated to a domain or to a component
     *
     * @return true if filter is associated to a domain
     */
  
    public boolean is_domainFilter() {
         return m_domainFilter;
     }
     

    /**
     * Purpose:
     * <p>   Set the filter associated to a component (filter manager)
     */

    public void setComponentFilter() {
         m_domainFilter = false;
     }

}


