/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.logging;

public class LogLevel
{

    /** 
     * Constant identifying all components
     */
    public static final String ALL_COMPONENT_NAMES = "*";

    /** 
     * Constant identifying all domains
     */
    public static final String ALL_DOMAINS = "*";

    /** 
     * Constant identifying the component separator
     */ 
    public static final char COMPONENT_SEP = '.';


    // ------------------------------------------------------------------------
    // ---------------------- BEGIN LOG LEVELS --------------------------------

    /**
     * Default log levels.
     */

    // this level reports a fatal error from CDMW; CDMW is no more
    // able to process safely.
    public static final long FATAL = 1L;
        
    // this level reports an erorr from CDMW.
    public static final long ERROR = 2L;

    // maybe the user has made an error, warn him.
    public static final long WARN  = 4L;

    // used for information purposes.
    public static final long INFO  = 8L;
        
    // used for debug purposes.
    public static final long DEBUG = 16L;

    /**
     * Additional debug levels for CDMW.
     */
    public static final long DEBUG_1 = 0x00000020L;
    public static final long DEBUG_2 = 0x00000040L;
    public static final long DEBUG_3 = 0x00000080L;
    public static final long DEBUG_4 = 0x00000100L;
    public static final long DEBUG_5 = 0x00000200L;
    public static final long DEBUG_6 = 0x00000400L;
    public static final long DEBUG_7 = 0x00000800L;
    public static final long DEBUG_8 = 0x00001000L;
    public static final long DEBUG_9 = 0x00002000L;

    //
    // here users have the possiblity to define their own levels.
    //

    /**
     * Levels internal to CDMW.
     * They are reserved to CDMW, yet users are allowed to use them;
     * but so they assume that they are aware of these levels ownership
     * and cannot be confused.
     */
    public static final long CDMW_DEBUG_1 = 0x01000000L;
    public static final long CDMW_DEBUG_2 = 0x02000000L;
    public static final long CDMW_DEBUG_3 = 0x04000000L;
    public static final long CDMW_DEBUG_4 = 0x08000000L;

    //
    // two vacancies in the mask are remaining
    //

    public static final long REQ_FOLLOWUP = 0x10000000L;
    public static final long CALL_TRACE   = 0x40000000L;
    public static final long ORB_TRACE    = 0x80000000L;

    // ----------------------- END LOG LEVELS ---------------------------------
    // ------------------------------------------------------------------------


    /** 
     * Constant identifying all levels set
     */
    public static final long  ALL_LEVELS_SET = ~0L;

    /** 
     * Constant identifying all levels set
     */
    public static final long  NONE_LEVEL = 0;

    /** 
     * Constant identifying the default level
     */
    public static final long  DEFAULT_LEVEL  = FATAL | ERROR | WARN | INFO;


    public static final String sFATAL        = "FATAL";
    public static final String sERROR        = "ERROR";
    public static final String sWARN         = "WARN";
    public static final String sINFO         = "INFO";
    public static final String sDEBUG        = "DEBUG";
    public static final String sDEBUG_1      = "DEBUG_1";
    public static final String sDEBUG_2      = "DEBUG_2";
    public static final String sDEBUG_3      = "DEBUG_3";
    public static final String sDEBUG_4      = "DEBUG_4";
    public static final String sDEBUG_5      = "DEBUG_5";
    public static final String sDEBUG_6      = "DEBUG_6";
    public static final String sDEBUG_7      = "DEBUG_7";
    public static final String sDEBUG_8      = "DEBUG_8";
    public static final String sDEBUG_9      = "DEBUG_9";
    public static final String sCDMW_DEBUG_1 = "CDMW_DEBUG_1";
    public static final String sCDMW_DEBUG_2 = "CDMW_DEBUG_2";
    public static final String sCDMW_DEBUG_3 = "CDMW_DEBUG_3";
    public static final String sCDMW_DEBUG_4 = "CDMW_DEBUG_4";
    public static final String sREQ_FOLLOWUP = "REQ_FOLLOWUP";
    public static final String sCALL_TRACE   = "CALL_TRACE";
    public static final String sORB_TRACE    = "ORB_TRACE";
    public static final String sALL          = "ALL";
    public static final String sDEFAULT      = "DEFAULT";
    public static final String sNONE         = "NONE";

    /**
     * Return the numeric level as string.
     *
     * @param level the level.
     *
     * @return the level as string.
     */
    public static String LevelToString(long level) {
        
        // if several types are in the level, for example ORB_TRACE + ORB_DEBUG,
        // only ORB_DEBUG is relevant
        if ((level & FATAL) != 0)              { return sFATAL;        }
        else if ((level & ERROR) != 0)         { return sERROR;        }
        else if ((level & WARN) != 0)          { return sWARN;         }
        else if ((level & INFO) != 0)          { return sINFO;         }
        else if ((level & DEBUG) != 0)         { return sDEBUG;        }
        else if ((level &  DEBUG_1) != 0)      { return sDEBUG_1;      }
        else if ((level &  DEBUG_2) != 0)      { return sDEBUG_2;      }
        else if ((level &  DEBUG_3) != 0)      { return sDEBUG_3;      }
        else if ((level &  DEBUG_4) != 0)      { return sDEBUG_4;      }
        else if ((level &  DEBUG_5) != 0)      { return sDEBUG_5;      }
        else if ((level &  DEBUG_6) != 0)      { return sDEBUG_6;      }
        else if ((level &  DEBUG_7) != 0)      { return sDEBUG_7;      }
        else if ((level &  DEBUG_8) != 0)      { return sDEBUG_8;      }
        else if ((level &  DEBUG_9) != 0)      { return sDEBUG_9;      }
        else if ((level &  CDMW_DEBUG_1) != 0) { return sCDMW_DEBUG_1; }
        else if ((level &  CDMW_DEBUG_2) != 0) { return sCDMW_DEBUG_2; }
        else if ((level &  CDMW_DEBUG_3) != 0) { return sCDMW_DEBUG_3; }
        else if ((level &  CDMW_DEBUG_4) != 0) { return sCDMW_DEBUG_4; }
        else if ((level &  REQ_FOLLOWUP) != 0) { return sREQ_FOLLOWUP; }
        else if ((level &  CALL_TRACE) != 0)   { return sCALL_TRACE;   }
        else {  
            // Log level
        String slevel = Long.toHexString(level).toUpperCase();
        if(slevel.length() < 8) {
        // Insert 0 of padding before level in hexadecimal format
        int size = 8 - slevel.length();
        String tmp = String.format("%"+size+"."+size+"s","00000000");
        slevel = tmp + slevel;
        }

            return slevel;
        }
    }

} 

