/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/logging/Logger.hpp>


#include "testfunctionlogger/TestFunctionLogger.hpp"
#include <Foundation/logging/FunctionLogger.hpp>
#include <Foundation/logging/CdmwLogging.stub.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>

using Cdmw::Common::TestFunctionLogger;
using Cdmw::Logging::Logger;

template <typename LOGGER>
class FunctionLoggerUserT
{
 public:
    FunctionLoggerUserT(LOGGER * logger)
        : m_logger(logger) {}
    
    void op1()
    {
        CDMW_LOG_FUNCTION(m_logger,"");
    }
    int op_return(int input1, int input2)
    {
        CDMW_LOG_FUNCTION(m_logger,input1 << "+" << input2);
        CDMW_LOG_FUNCTION_RETURN(input1+input2);
        return input1+input2;
    }
    void op_exception(int input) throw(int)
    {
        CDMW_LOG_FUNCTION(m_logger,"Input=" << input);
        CDMW_LOG_FUNCTION_EXCEPTION(input);
        throw input;
    }
    
 private:
    LOGGER * m_logger;
};
typedef FunctionLoggerUserT<Cdmw::Logging::Logger> FunctionLoggerUser;
typedef FunctionLoggerUserT<CdmwLogging::Logger> FunctionLoggerIDLUser;


CPPUNIT_TEST_SUITE_REGISTRATION( TestFunctionLogger );


class IDLLogger_impl : virtual public CdmwLogging::Logger,
                       virtual public Cdmw::OrbSupport::RefCountLocalObject
{

public:
      IDLLogger_impl(const char* component_name,
                   const char* domain_name)
      {
         m_logger = Cdmw::Logging::Logger::Get_Logger(component_name, 
                                                      domain_name);
      }
      ~IDLLogger_impl() throw()
      {
         // do not delete m_logger since it will be deleted by LogManager
      }
      //
      // IDL:com.thalesgroup/CdmwLogging/Logger/log:1.0
      //
      virtual void log(CdmwLogging::LogLevel level,
                       const char * message,
                       const char * file,
                       CORBA::Long line)
         throw(CORBA::SystemException)
      {
         m_logger->log(level, message, file, line);
      }
      //
      // IDL:com.thalesgroup/CdmwLogging/Logger/debug:1.0
      //
      virtual void debug(const char * message,
                         const char * file,
                         CORBA::Long line)
         throw(CORBA::SystemException)
      {
         m_logger->debug(message, file, line);
      }
      //
      // IDL:com.thalesgroup/CdmwLogging/Logger/info:1.0
      //
      virtual void info(const char * message,
                        const char * file,
                        CORBA::Long line)
         throw(CORBA::SystemException)
      {
         m_logger->info(message, file, line);
      }
      //
      // IDL:com.thalesgroup/CdmwLogging/Logger/warn:1.0
      //
      virtual void warn(const char * message,
                        const char * file,
                        CORBA::Long line)
         throw(CORBA::SystemException)
      {
         m_logger->warn(message, file, line);
      }
      //
      // IDL:com.thalesgroup/CdmwLogging/Logger/error:1.0
      //
      virtual void error(const char * message,
                         const char * file,
                         CORBA::Long line)
         throw(CORBA::SystemException)
      {
         m_logger->error(message, file, line);
      }
      //
      // IDL:com.thalesgroup/CdmwLogging/Logger/fatal:1.0
      //
      virtual void fatal(const char * message,
                         const char * file,
                         CORBA::Long line)
         throw(CORBA::SystemException)
      {
         m_logger->fatal(message, file, line);
      }
      //
      // IDL:com.thalesgroup/CdmwLogging/Logger/is_enabled_for:1.0
      //
      virtual CORBA::Boolean is_enabled_for(CdmwLogging::LogLevel level)
         throw(CORBA::SystemException)
      {
         return m_logger->is_enabled_for(level);
      }
      //
      // IDL:com.thalesgroup/CdmwLogging/Logger/is_debug_enabled:1.0
      //
      virtual CORBA::Boolean is_debug_enabled()
         throw(CORBA::SystemException)
      {
         return m_logger->is_debug_enabled();
      }
      //
      // IDL:com.thalesgroup/CdmwLogging/Logger/is_info_enabled:1.0
      //
      virtual CORBA::Boolean is_info_enabled()
         throw(CORBA::SystemException)
      {
         return m_logger->is_info_enabled();
      }
      //
      // IDL:com.thalesgroup/CdmwLogging/Logger/is_warn_enabled:1.0
      //
      virtual CORBA::Boolean is_warn_enabled()
         throw(CORBA::SystemException)
      {
         return m_logger->is_warn_enabled();
      }
      //
      // IDL:com.thalesgroup/CdmwLogging/Logger/is_error_enabled:1.0
      //
      virtual CORBA::Boolean is_error_enabled()
         throw(CORBA::SystemException)
      {
         return m_logger->is_error_enabled();
      }
      //
      // IDL:com.thalesgroup/CdmwLogging/Logger/is_fatal_enabled:1.0
      //
      virtual CORBA::Boolean is_fatal_enabled()
         throw(CORBA::SystemException)
      {
         return m_logger->is_fatal_enabled();
      }
   private:
      IDLLogger_impl();
      IDLLogger_impl(const IDLLogger_impl&);
      void operator=(const IDLLogger_impl&);
      
      Cdmw::Logging::Logger* m_logger;
};




/**
 * Default constructor.
 */
TestFunctionLogger::TestFunctionLogger()
{
}


/**
 * Destructor.
 */
TestFunctionLogger::~TestFunctionLogger()
    throw()
{
}


/**
 * Run the tests.
 */
void
TestFunctionLogger::do_tests()
{
//     set_nbOfRequestedTestOK(4);

    TEST_INFO("Obtaining a default logger for the \"TEST_FUNCTION_LOGGER\" domain.\n");
    Logger* logger = Cdmw::Logging::Logger::Get_Logger("TEST_FUNCTION_LOGGER");
    CPPUNIT_ASSERT(logger != 0);


    TEST_INFO("Calling the Function logger. VISUAL CHECK!.");
    FunctionLoggerUser flu(logger);

    TEST_INFO("Calling op1 (with no return and no exception).");
    flu.op1();
    CPPUNIT_ASSERT(true);

    int retval = 30;
    TEST_INFO("Calling op_return that return value " << retval);
    int res = flu.op_return(20,10);
    CPPUNIT_ASSERT(res == retval);

    retval = 50;
    TEST_INFO("Calling op_exception that should raise exception with value " << retval);
    try {
        flu.op_exception(retval);
        CPPUNIT_ASSERT(false);
    } catch (int ex) {
        CPPUNIT_ASSERT(ex == retval);
    }

    // Using IDL logger
    {
       TEST_INFO("Obtaining a default logger for the \"TEST_FUNCTION_LOGGER\" domain.\n");
       CdmwLogging::Logger_var IDllogger = new IDLLogger_impl("MyComponent",
                                                              "MyDomain");
       
       TEST_INFO("Calling the Function logger. VISUAL CHECK!.");
       FunctionLoggerIDLUser flu(IDllogger.in());
       
       TEST_INFO("Calling op1 (with no return and no exception).");
       flu.op1();
       CPPUNIT_ASSERT(true);
       
       int retval = 30;
       TEST_INFO("Calling op_return that return value " << retval);
       int res = flu.op_return(20,10);
       CPPUNIT_ASSERT(res == retval);
       
       retval = 50;
       TEST_INFO("Calling op_exception that should raise exception with value " << retval);
       try {
          flu.op_exception(retval);
          CPPUNIT_ASSERT(false);
       } catch (int ex) {
          CPPUNIT_ASSERT(ex == retval);
       }
       TEST_INFO("Ending test.");
    }
}
