/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "testlogfilter/PILOT_FilterMngr.hpp"


namespace Cdmw
{
namespace Logging
{

CPPUNIT_TEST_SUITE_REGISTRATION (PILOT_FilterMngr);

PILOT_FilterMngr::PILOT_FilterMngr()
    throw()
{
}


PILOT_FilterMngr::~PILOT_FilterMngr()
    throw()
{
}


void
PILOT_FilterMngr::do_tests()
    throw()
{
    TEST_INFO("Create an instance of FilterMngr");
    FilterMngr filterMngr(INFO|WARN|ERROR|FATAL);


    // Test A1
    TEST_INFO("Check if INFO, WARN, ERROR, FATAL are activated by default");
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", FATAL) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", DEBUG) == false);
    
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", INFO) == true);

    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", INFO) == true);
    
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain2", INFO) == true);
    
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1b", "domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1b", "domain2", INFO) == true);
    
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain2", INFO) == true);
    
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2a", "domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2a", "domain2", INFO) == true);
    
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2b", "domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2b", "domain2", INFO) == true);
    


    // Test B1
    TEST_INFO("Deactivate every domain of the default component");
    filterMngr.deactivate_level(ALL_DOMAINS, ALL_LEVELS_SET);
    
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", FATAL) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", DEBUG) == false);
    
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", FATAL) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", DEBUG) == false);
    
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", DEBUG) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", DEBUG) == false);
    
    // Test C1
    TEST_INFO("Activate every domain of the default component");
    filterMngr.activate_level(ALL_DOMAINS, ALL_LEVELS_SET);
    
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", FATAL) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", DEBUG) == true);
    
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", FATAL) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", DEBUG) == true);
    
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", DEBUG) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", DEBUG) == false);


    // Test C1a
    TEST_INFO("Deactivate every domain of the default component");
    filterMngr.deactivate_level(ALL_DOMAINS, ALL_LEVELS_SET);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", ERROR) == false);


    // Test C2
    TEST_INFO("Then activate domain1/level INFO");
    filterMngr.activate_level("domain1", INFO);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", ERROR) == false);
    
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", INFO) == true);

    // Test C3
    TEST_INFO("Then activate domain1/level ERROR");
    filterMngr.activate_level("domain1", ERROR);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", ERROR) == false);


    // Test C4
    TEST_INFO("Then activate domain2/level INFO");
    filterMngr.activate_level("domain2", INFO);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", ERROR) == false);


    // Test C5
    TEST_INFO("Then activate domain2/level ERROR");
    filterMngr.activate_level("domain2", ERROR);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", ERROR) == true);
    

    // Test D1
    TEST_INFO("Deactivate every domain of the default component");
    filterMngr.deactivate_level(ALL_DOMAINS, ALL_LEVELS_SET);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", ERROR) == false);

    // Test D2
    TEST_INFO("Then activate all the levels of domain1");
    filterMngr.activate_level("domain1", ALL_LEVELS_SET);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", ERROR) == false);


    // Test D3
    TEST_INFO("Then activate level INFO of any domain");
    filterMngr.activate_level(ALL_DOMAINS, INFO);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", ERROR) == false);


    // Test E1
    TEST_INFO("Deactivate everything");
    filterMngr.deactivate_level(ALL_DOMAINS, ALL_LEVELS_SET);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", ERROR) == false);
    

    // Test E2
    TEST_INFO("Then activate all the levels of domain1");
    filterMngr.activate_level("domain1", ALL_LEVELS_SET);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", ERROR) == false);

    // Test E3
    TEST_INFO("Then deactivate level INFO of any domain");
    filterMngr.deactivate_level(ALL_DOMAINS, INFO);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", ERROR) == false);

    // Test E4
    TEST_INFO("Then activate domain1/level INFO");
    filterMngr.activate_level("domain1", INFO);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", ERROR) == false);



    // Test H1
    TEST_INFO("Activate everything");
    filterMngr.activate_level(ALL_DOMAINS, ALL_LEVELS_SET);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", ERROR) == true);

    // Test H2
    TEST_INFO("Then deactivate all the levels of domain1");
    filterMngr.deactivate_level("domain1", ALL_LEVELS_SET);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", ERROR) == true);
    
    // Test H3
    TEST_INFO("Then activate level INFO of any domain");
    filterMngr.activate_level(ALL_DOMAINS, INFO);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", ERROR) == true);
    
    // Test H4
    TEST_INFO("Then deactivate domain1/level INFO");
    filterMngr.deactivate_level("domain1", INFO);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", ERROR) == true);
    

    // Test ZD1
    TEST_INFO("Deactivate everything");
    filterMngr.deactivate_level(ALL_COMPONENT_NAMES,
                                ALL_DOMAINS,
                                ALL_LEVELS_SET);
                                
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain3", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1b", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1b", "domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2a", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2a", "domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2b", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2b", "domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component3.component3a", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", ERROR) == false);

    // Test ZD2
    TEST_INFO("Then activate all the levels of domain1 of component1");
    filterMngr.activate_level("component1", "domain1", ALL_LEVELS_SET);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1b", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1b", "domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", ERROR) == false);

    // Test ZD3
    TEST_INFO("Then activate level INFO of any domain of component1");
    filterMngr.activate_level("component1", ALL_DOMAINS, INFO);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1b", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1b", "domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", INFO) == false);

    
    // Test ZD3a
    TEST_INFO("Then activate level INFO of domain1 of component1.component1a");
    filterMngr.activate_level("component1.component1a", "domain1", INFO);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1b", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1b", "domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", INFO) == false);
    
    // Test ZD3b
    TEST_INFO("Then deactivate level INFO of domain1 of component1.component1a");
    filterMngr.deactivate_level("component1.component1a", "domain1", INFO);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1b", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1b", "domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", INFO) == false);

    // Test ZD4
    TEST_INFO("Then deactivate level INFO of domain1 of component1");
    filterMngr.deactivate_level("component1", "domain1", INFO);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1b", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1b", "domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", INFO) == false);
    


    // Test ZD5
    TEST_INFO("Then activate all the levels of domain2 of any component");
    filterMngr.activate_level(ALL_COMPONENT_NAMES,
                              "domain2",
                              ALL_LEVELS_SET);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain1", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain2", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain2", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1b", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1b", "domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1b", "domain1", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1b", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1b", "domain2", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1b", "domain2", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain1", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain2", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain2", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2a", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2a", "domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2a", "domain1", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2a", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2a", "domain2", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2a", "domain2", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2b", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2b", "domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2b", "domain1", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2b", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2b", "domain2", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2b", "domain2", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component3.component3a", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component3.component3a", "domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component3.component3a", "domain1", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component3.component3a", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component3.component3a", "domain2", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component3.component3a", "domain2", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", ERROR) == true);


    // Test ZD6
    TEST_INFO("Then deactivate level WARN of any domain of any component");
    filterMngr.deactivate_level(ALL_COMPONENT_NAMES,
                                ALL_DOMAINS,
                                WARN);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain1", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain2", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1b", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1b", "domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1b", "domain1", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1b", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1b", "domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1b", "domain2", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain1", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain2", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2a", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2a", "domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2a", "domain1", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2a", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2a", "domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2a", "domain2", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2b", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2b", "domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2b", "domain1", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2b", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2b", "domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2.component2b", "domain2", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component3.component3a", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component3.component3a", "domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component3.component3a", "domain1", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component3.component3a", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component3.component3a", "domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component3.component3a", "domain2", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain1", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("domain2", ERROR) == true);
    


    // Test ZD7
    TEST_INFO("Then activate level INFO of any domain of component1");
    filterMngr.activate_level("component1", ALL_DOMAINS, INFO);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain3", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain1", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component2", "domain2", ERROR) == true);
    
    // Test ZD7a
    TEST_INFO("Then deactivate level INFO of domain3 of component1");
    filterMngr.deactivate_level("component1", "domain3", INFO);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain3", INFO) == false);
    

    // Test ZD8
    TEST_INFO("Then activate level WARN+ERROR of any domain of component1");
    filterMngr.activate_level("component1", ALL_DOMAINS, WARN|ERROR);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain3", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain3", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain3", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain1", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain2", ERROR) == true);

    // Test ZD9
    TEST_INFO("Then deactivate level WARN of domain2 of component1");
    filterMngr.deactivate_level("component1", "domain2", WARN);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain3", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain3", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain3", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain1", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain2", ERROR) == true);


    // Test ZD10
    TEST_INFO("Then deactivate all the levels of domain2 of component1");
    filterMngr.deactivate_level("component1", "domain2", ALL_LEVELS_SET);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain3", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain3", WARN) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain3", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain1", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain2", ERROR) == true);


    // Test ZD11
    TEST_INFO("Then deactivate level WARN of any domain of component1");
    filterMngr.deactivate_level("component1", ALL_DOMAINS, WARN);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain1", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain2", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain3", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain3", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1", "domain3", ERROR) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain1", INFO) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain1", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain1", ERROR) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain2", INFO) == true);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain2", WARN) == false);
    CPPUNIT_ASSERT(filterMngr.is_activated("component1.component1a", "domain2", ERROR) == true);


}




}; // End namespace Logging

}; // End namespace Cdmw
