/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.logging.test.testlogfilter;

import cdmw.logging.*;

import org.junit.Test;

import static org.junit.Assert.assertTrue;

 
public class TestFilterMngr {

    @Test
    public void testFilterManager() {
        System.out.println ("Creating an instance of FilterMngr.");
        FilterMngr filterMngr = new FilterMngr (LogLevel.INFO | LogLevel.WARN
            | LogLevel.ERROR | LogLevel.FATAL);
 
        System.out.println("Check if INFO, WARN, ERROR, FATAL are activated by default");

        assertTrue(filterMngr.isActivated("domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.FATAL) == true);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.DEBUG) == false);
        
        assertTrue(filterMngr.isActivated("domain2", LogLevel.INFO) == true);
        
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.INFO) == true);
    
        assertTrue(filterMngr.isActivated("component1.component1a", "domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain2", LogLevel.INFO) == true);
    
        assertTrue(filterMngr.isActivated("component1.component1b", "domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1.component1b", "domain2", LogLevel.INFO) == true);
    
        assertTrue(filterMngr.isActivated("component2", "domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component2", "domain2", LogLevel.INFO) == true);
    
        assertTrue(filterMngr.isActivated("component2.component2a", "domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component2.component2a", "domain2", LogLevel.INFO) == true);
    
        assertTrue(filterMngr.isActivated("component2.component2b", "domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component2.component2b", "domain2", LogLevel.INFO) == true);
    
        System.out.println("Deactivate every domain of the default component");
        filterMngr.deactivateLevel(LogLevel.ALL_DOMAINS, LogLevel.ALL_LEVELS_SET);

        assertTrue(filterMngr.isActivated("domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.FATAL) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.DEBUG) == false);
    
        assertTrue(filterMngr.isActivated("domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.FATAL) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.DEBUG) == false);
    
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.DEBUG) == false);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.DEBUG) == false);

        System.out.println("Activate every domain of the default component");
        filterMngr.activateLevel(LogLevel.ALL_DOMAINS, LogLevel.ALL_LEVELS_SET);

        assertTrue(filterMngr.isActivated("domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.FATAL) == true);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.DEBUG) == true);
    
        assertTrue(filterMngr.isActivated("domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.FATAL) == true);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.DEBUG) == true);
    
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.DEBUG) == false);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.DEBUG) == false);
     
        System.out.println("Deactivate every domain of the default component");
        filterMngr.deactivateLevel(LogLevel.ALL_DOMAINS, LogLevel.ALL_LEVELS_SET);

        assertTrue(filterMngr.isActivated("domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.ERROR) == false);

        
        // Test C2
        System.out.println("Then activate domain1/level Logging.INFO");
        filterMngr.activateLevel("domain1", LogLevel.INFO);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.ERROR) == false);
    
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.INFO) == true);

        // Test C3
        System.out.println("Then activate domain1/level Logging.ERROR");
        filterMngr.activateLevel("domain1", LogLevel.ERROR);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.ERROR) == false);


        // Test C4
        System.out.println("Then activate domain2/level Logging.INFO");
        filterMngr.activateLevel("domain2", LogLevel.INFO);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.ERROR) == false);


        // Test C5
        System.out.println("Then activate domain2/level Logging.ERROR");
        filterMngr.activateLevel("domain2", LogLevel.ERROR);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.ERROR) == true);
    

        // Test D1
        System.out.println("Deactivate every domain of the default component");
        filterMngr.deactivateLevel(LogLevel.ALL_DOMAINS, LogLevel.ALL_LEVELS_SET);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.ERROR) == false);

        // Test D2
        System.out.println("Then activate all the levels of domain1");
        filterMngr.activateLevel("domain1", LogLevel.ALL_LEVELS_SET);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.ERROR) == false);


        // Test D3
        System.out.println("Then activate level LogLevel.INFO of any domain");
        filterMngr.activateLevel(LogLevel.ALL_DOMAINS, LogLevel.INFO);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.ERROR) == false);


        // Test E1
        System.out.println("Deactivate everything");
        filterMngr.deactivateLevel(LogLevel.ALL_DOMAINS, LogLevel.ALL_LEVELS_SET);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.ERROR) == false);
    

        // Test E2
        System.out.println("Then activate all the levels of domain1");
        filterMngr.activateLevel("domain1", LogLevel.ALL_LEVELS_SET);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.ERROR) == false);

        // Test E3
        System.out.println("Then deactivate level LogLevel.INFO of any domain");
        filterMngr.deactivateLevel(LogLevel.ALL_DOMAINS, LogLevel.INFO);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.ERROR) == false);

        // Test E4
        System.out.println("Then activate domain1/level LogLevel.INFO");
        filterMngr.activateLevel("domain1", LogLevel.INFO);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.ERROR) == false);



        // Test H1
        System.out.println("Activate everything");
        filterMngr.activateLevel(LogLevel.ALL_DOMAINS, LogLevel.ALL_LEVELS_SET);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.ERROR) == true);

        // Test H2
        System.out.println("Then deactivate all the levels of domain1");
        filterMngr.deactivateLevel("domain1", LogLevel.ALL_LEVELS_SET);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.ERROR) == true);
    
        // Test H3
        System.out.println("Then activate level LogLevel.INFO of any domain");
        filterMngr.activateLevel(LogLevel.ALL_DOMAINS, LogLevel.INFO);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.ERROR) == true);
    
        // Test H4
        System.out.println("Then deactivate domain1/level LogLevel.INFO");
        filterMngr.deactivateLevel("domain1", LogLevel.INFO);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.ERROR) == true);
    

        // Test ZD1
        System.out.println("Deactivate everything");
        filterMngr.deactivateLevel(LogLevel.ALL_COMPONENT_NAMES,
                                    LogLevel.ALL_DOMAINS,
                                    LogLevel.ALL_LEVELS_SET);
                                
        assertTrue(filterMngr.isActivated("domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1", "domain3", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1.component1b", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1.component1b", "domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component2", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component2", "domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component2.component2a", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component2.component2a", "domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component2.component2b", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component2.component2b", "domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component3.component3a", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.ERROR) == false);

        // Test ZD2
        System.out.println("Then activate all the levels of domain1 of component1");
        filterMngr.activateLevel("component1", "domain1", LogLevel.ALL_LEVELS_SET);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1.component1b", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1.component1b", "domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component2", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component2", "domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.ERROR) == false);

        // Test ZD3
        System.out.println("Then activate level LogLevel.INFO of any domain of component1");
        filterMngr.activateLevel("component1", LogLevel.ALL_DOMAINS, LogLevel.INFO);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1.component1b", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1.component1b", "domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component2", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component2", "domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.INFO) == false);

    
        // Test ZD3a
        System.out.println("Then activate level LogLevel.INFO of domain1 of component1.component1a");
        filterMngr.activateLevel("component1.component1a", "domain1", LogLevel.INFO);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1.component1b", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1.component1b", "domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component2", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component2", "domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.INFO) == false);
    
        // Test ZD3b
        System.out.println("Then deactivate level LogLevel.INFO of domain1 of component1.component1a");
        filterMngr.deactivateLevel("component1.component1a", "domain1", LogLevel.INFO);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1.component1b", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1.component1b", "domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component2", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component2", "domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.INFO) == false);

        // Test ZD4
        System.out.println("Then deactivate level LogLevel.INFO of domain1 of component1");
        filterMngr.deactivateLevel("component1", "domain1", LogLevel.INFO);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1.component1b", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1.component1b", "domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component2", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component2", "domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.INFO) == false);
    


        // Test ZD5
        System.out.println("Then activate all the levels of domain2 of any component");
        filterMngr.activateLevel(LogLevel.ALL_COMPONENT_NAMES,
                                  "domain2",
                                  LogLevel.ALL_LEVELS_SET);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain1", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain2", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain2", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("component1.component1b", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1.component1b", "domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component1.component1b", "domain1", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("component1.component1b", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1.component1b", "domain2", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("component1.component1b", "domain2", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("component2", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component2", "domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component2", "domain1", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("component2", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component2", "domain2", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("component2", "domain2", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("component2.component2a", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component2.component2a", "domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component2.component2a", "domain1", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("component2.component2a", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component2.component2a", "domain2", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("component2.component2a", "domain2", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("component2.component2b", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component2.component2b", "domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component2.component2b", "domain1", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("component2.component2b", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component2.component2b", "domain2", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("component2.component2b", "domain2", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("component3.component3a", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component3.component3a", "domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component3.component3a", "domain1", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("component3.component3a", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component3.component3a", "domain2", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("component3.component3a", "domain2", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.ERROR) == true);


        // Test ZD6
        System.out.println("Then deactivate level LogLevel.WARN of any domain of any component");
        filterMngr.deactivateLevel(LogLevel.ALL_COMPONENT_NAMES,
                                    LogLevel.ALL_DOMAINS,
                                    LogLevel.WARN);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain1", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain2", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("component1.component1b", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1.component1b", "domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component1.component1b", "domain1", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("component1.component1b", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1.component1b", "domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component1.component1b", "domain2", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("component2", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component2", "domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component2", "domain1", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("component2", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component2", "domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component2", "domain2", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("component2.component2a", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component2.component2a", "domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component2.component2a", "domain1", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("component2.component2a", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component2.component2a", "domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component2.component2a", "domain2", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("component2.component2b", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component2.component2b", "domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component2.component2b", "domain1", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("component2.component2b", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component2.component2b", "domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component2.component2b", "domain2", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("component3.component3a", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component3.component3a", "domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component3.component3a", "domain1", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("component3.component3a", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component3.component3a", "domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component3.component3a", "domain2", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain1", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("domain2", LogLevel.ERROR) == true);
    


        // Test ZD7
        System.out.println("Then activate level LogLevel.INFO of any domain of component1");
        filterMngr.activateLevel("component1", LogLevel.ALL_DOMAINS, LogLevel.INFO);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("component1", "domain3", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component2", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component2", "domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component2", "domain1", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("component2", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component2", "domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component2", "domain2", LogLevel.ERROR) == true);
    
        // Test ZD7a
        System.out.println("Then deactivate level LogLevel.INFO of domain3 of component1");
        filterMngr.deactivateLevel("component1", "domain3", LogLevel.INFO);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1", "domain3", LogLevel.INFO) == false);
    

        // Test ZD8
        System.out.println("Then activate level LogLevel.WARN+LogLevel.ERROR of any domain of component1");
        filterMngr.activateLevel("component1", LogLevel.ALL_DOMAINS, LogLevel.WARN | LogLevel.ERROR);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("component1", "domain3", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1", "domain3", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("component1", "domain3", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain1", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain2", LogLevel.ERROR) == true);

        // Test ZD9
        System.out.println("Then deactivate level LogLevel.WARN of domain2 of component1");
        filterMngr.deactivateLevel("component1", "domain2", LogLevel.WARN);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("component1", "domain3", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1", "domain3", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("component1", "domain3", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain1", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain2", LogLevel.ERROR) == true);


        // Test ZD10
        System.out.println("Then deactivate all the levels of domain2 of component1");
        filterMngr.deactivateLevel("component1", "domain2", LogLevel.ALL_LEVELS_SET);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("component1", "domain3", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1", "domain3", LogLevel.WARN) == true);
        assertTrue(filterMngr.isActivated("component1", "domain3", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain1", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain2", LogLevel.ERROR) == true);


        // Test ZD11
        System.out.println("Then deactivate level LogLevel.WARN of any domain of component1");
        filterMngr.deactivateLevel("component1", LogLevel.ALL_DOMAINS, LogLevel.WARN);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component1", "domain1", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component1", "domain2", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("component1", "domain3", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1", "domain3", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component1", "domain3", LogLevel.ERROR) == true);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain1", LogLevel.INFO) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain1", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain1", LogLevel.ERROR) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain2", LogLevel.INFO) == true);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain2", LogLevel.WARN) == false);
        assertTrue(filterMngr.isActivated("component1.component1a", "domain2", LogLevel.ERROR) == true);
    }
} 
