/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/logging/Logger.hpp>


#include "testlogging/TestLogging.hpp"
#include <Foundation/logging/FunctionLogger.hpp>


using Cdmw::Common::TestLogging;
using Cdmw::Logging::Logger;

class FunctionLoggerUser
{
 public:
    FunctionLoggerUser(Cdmw::Logging::Logger * logger)
        : m_logger(logger) {}
    
    void op1()
    {
        CDMW_LOG_FUNCTION(m_logger,"");
    }
    int op_return(int input)
    {
        CDMW_LOG_FUNCTION(m_logger,"");
        CDMW_LOG_FUNCTION_RETURN(input);
        return input;
    }
    void op_exception(int input) throw(int)
    {
        CDMW_LOG_FUNCTION(m_logger,"");
        CDMW_LOG_FUNCTION_EXCEPTION(input);
        throw input;
    }
    
 private:
    Cdmw::Logging::Logger * m_logger;
};

CPPUNIT_TEST_SUITE_REGISTRATION( TestLogging );

/**
 * Default constructor.
 */
TestLogging::TestLogging()
{
}


/**
 * Destructor.
 */
TestLogging::~TestLogging()
    throw()
{
}


/**
 * Run the tests.
 */
void
TestLogging::do_tests()
{
//     set_nbOfRequestedTestOK(19);
    
    Logger* logger_def = 0;
    Logger* logger1 = 0;
    Logger* logger2 = 0;

    TEST_INFO("Obtaining a default logger for the \"TEST_LOGGING1\" domain.\n"
              "Only messages with a level <= INFO (ie. lvl <= 8) are displayed.");
    logger_def = Cdmw::Logging::Logger::Get_Logger("TEST_LOGGING1");
    CPPUNIT_ASSERT(logger_def != 0);
    logger1 = logger_def;

    TEST_INFO("Logging an info message.");
    CDMW_INFO(logger_def, "my info message 1");
    if (logger_def->is_info_enabled()) {
        CPPUNIT_ASSERT(true);
    }
    else {
        CPPUNIT_ASSERT(false);
    }

    TEST_INFO("Retrieve the logger of the \"TEST_LOGGING1\" domain.");
    logger2 = Cdmw::Logging::Logger::Get_Logger("TEST_LOGGING1");
    CPPUNIT_ASSERT(logger2 != 0);
    CPPUNIT_ASSERT(logger2 == logger_def);
    TEST_INFO("Logging a warning message.");
    CDMW_WARN(logger2, "my warning message 1");
    if (logger2->is_warn_enabled()) {
        CPPUNIT_ASSERT(true);
    }
    else {
        CPPUNIT_ASSERT(false);
    }

    TEST_INFO("Obtaining a logger for the \"TEST_LOGGING2\" domain.");
    logger1 = Cdmw::Logging::Logger::Get_Logger("TEST_LOGGING2");
    CPPUNIT_ASSERT(logger1 != 0);
    CPPUNIT_ASSERT(logger1 != logger2);
    TEST_INFO("Logging an error message.");
    CDMW_ERROR(logger1, "my error message 1");
    if (logger1->is_error_enabled()) {
        CPPUNIT_ASSERT(true);
    }
    else {
        CPPUNIT_ASSERT(false);
    }

    TEST_INFO("Retrieve the logger of the \"TEST_LOGGING2\" domain.");
    logger2 = Cdmw::Logging::Logger::Get_Logger("TEST_LOGGING2");
    CPPUNIT_ASSERT(logger2 != 0);
    CPPUNIT_ASSERT(logger2 == logger1);
    TEST_INFO("Logging a fatal error message.");
    CDMW_FATAL(logger2, "my fatal error message 1");
    if (logger2->is_fatal_enabled()) {
        CPPUNIT_ASSERT(true);
    }
    else {
        CPPUNIT_ASSERT(false);
    }

    TEST_INFO("Retrieve the logger of the \"TEST_LOGGING1\" domain.");
    logger1 = Cdmw::Logging::Logger::Get_Logger("TEST_LOGGING1");
    CPPUNIT_ASSERT(logger1 != 0);
    CPPUNIT_ASSERT(logger1 != logger2);
    CPPUNIT_ASSERT(logger1 == logger_def);
    TEST_INFO("Logging another fatal error message.");
    CDMW_FATAL(logger1, "my fatal error message 2");
    if (logger1->is_fatal_enabled()) {
        CPPUNIT_ASSERT(true);
    }
    else {
        CPPUNIT_ASSERT(false);
    }

    TEST_INFO("Logging a debug message (lvl > INFO).");
    CDMW_DEBUG(logger_def, "my debug message 1");
    if (logger_def->is_debug_enabled()) {
        CPPUNIT_ASSERT(false);
    }
    else {
        CPPUNIT_ASSERT(true);
    }

    const short level_7 = 7;
    TEST_INFO("Logging a generic message w/ lvl " << level_7 << ".");
    CDMW_LOG(Cdmw::Logging::Logger::Get_Logger("TEST_LOGGING1"), level_7, "my generic message 1");
    if (logger_def->is_enabled_for(level_7)) {
        CPPUNIT_ASSERT(true);
    }
    else {
        CPPUNIT_ASSERT(false);
    }

    const short level_20 = 20;
    TEST_INFO("Logging a generic message w/ lvl " << level_20 << ".");
    CDMW_LOG(Cdmw::Logging::Logger::Get_Logger("TEST_LOGGING2"), level_20, "my generic message 2");
    if (logger2->is_enabled_for(level_20)) {
        CPPUNIT_ASSERT(false);
    }
    else {
        CPPUNIT_ASSERT(true);
    }

    TEST_INFO("Obtaining a default logger for the \"COMPONENT1\" of the \"TEST_LOGGING1\" domain.\n"
              "Only messages with a level <= INFO (ie. lvl <= 8) are displayed.");
    logger_def = Cdmw::Logging::Logger::Get_Logger("COMPONENT1", "TEST_LOGGING1");
    CPPUNIT_ASSERT(logger_def != 0);
    CPPUNIT_ASSERT(logger_def != logger1);
    logger1 = logger_def;

    TEST_INFO("Logging an info message.");
    CDMW_INFO(logger_def, "my info message 2");
    if (logger_def->is_info_enabled()) {
        CPPUNIT_ASSERT(true);
    }
    else {
        CPPUNIT_ASSERT(false);
    }

    TEST_INFO("Retrieve the logger of the \"COMPONENT1\" of the \"TEST_LOGGING1\" domain.");
    logger2 = Cdmw::Logging::Logger::Get_Logger("COMPONENT1", "TEST_LOGGING1");
    CPPUNIT_ASSERT(logger2 != 0);
    CPPUNIT_ASSERT(logger2 == logger1);
    CPPUNIT_ASSERT(logger2 == logger_def);
    TEST_INFO("Logging a warning message.");
    CDMW_WARN(logger2, "my warning message 2");
    if (logger2->is_warn_enabled()) {
        CPPUNIT_ASSERT(true);
    }
    else {
        CPPUNIT_ASSERT(false);
    }

    TEST_INFO("Obtaining a logger of the \"COMPONENT2\" of the \"TEST_LOGGING1\" domain.");
    logger1 = Cdmw::Logging::Logger::Get_Logger("COMPONENT2", "TEST_LOGGING1");
    CPPUNIT_ASSERT(logger1 != 0);
    CPPUNIT_ASSERT(logger1 != logger_def);
    TEST_INFO("Logging an error message.");
    CDMW_ERROR(logger1, "my error message 2");
    if (logger1->is_error_enabled()) {
        CPPUNIT_ASSERT(true);
    }
    else {
        CPPUNIT_ASSERT(false);
    }

    TEST_INFO("Retrieve the logger of the \"COMPONENT2\" of the \"TEST_LOGGING1\" domain.");
    logger2 = Cdmw::Logging::Logger::Get_Logger("COMPONENT2", "TEST_LOGGING1");
    CPPUNIT_ASSERT(logger2 != 0);
    CPPUNIT_ASSERT(logger2 == logger1);
    TEST_INFO("Logging a fatal error message.");
    CDMW_FATAL(logger2, "my fatal error message 3");
    if (logger2->is_fatal_enabled()) {
        CPPUNIT_ASSERT(true);
    }
    else {
        CPPUNIT_ASSERT(false);
    }

    TEST_INFO("Retrieve the logger of the \"COMPONENT1\" of the \"TEST_LOGGING1\" domain.");
    logger1 = Cdmw::Logging::Logger::Get_Logger("COMPONENT1", "TEST_LOGGING1");
    CPPUNIT_ASSERT(logger1 != 0);
    CPPUNIT_ASSERT(logger1 != logger2);
    CPPUNIT_ASSERT(logger1 == logger_def);
    TEST_INFO("Logging another fatal error message.");
    CDMW_FATAL(logger1, "my fatal error message 4");
    if (logger1->is_fatal_enabled()) {
        CPPUNIT_ASSERT(true);
    }
    else {
        CPPUNIT_ASSERT(false);
    }

    TEST_INFO("Logging a debug message (lvl > 8).");
    CDMW_DEBUG(logger_def, "my debug message 2");
    if (logger_def->is_debug_enabled()) {
        CPPUNIT_ASSERT(false);
    }
    else {
        CPPUNIT_ASSERT(true);
    }

    TEST_INFO("Logging a generic message w/ lvl " << level_7 << ".");
    CDMW_LOG(Cdmw::Logging::Logger::Get_Logger("COMPONENT1", "TEST_LOGGING1"), level_7, "my generic message 3");
    if (logger_def->is_debug_enabled()) {
        CPPUNIT_ASSERT(false);
    }
    else {
        CPPUNIT_ASSERT(true);
    }

    TEST_INFO("Logging a generic message w/ lvl " << level_20 << ".");
    CDMW_LOG(Cdmw::Logging::Logger::Get_Logger("COMPONENT2", "TEST_LOGGING1"), level_20, "my generic message 4");
    if (logger2->is_enabled_for(level_20)) {
        CPPUNIT_ASSERT(false);
    }
    else {
        CPPUNIT_ASSERT(true);
    }

    TEST_INFO("Obtaining a default logger for the \"TEST_LOGFUNC\" domain.\n"
              "Only messages with a level <= INFO (ie. lvl <= 8) are displayed.");
    logger1 = Cdmw::Logging::Logger::Get_Logger("TEST_LOGFUNC");
    CPPUNIT_ASSERT(logger1 != 0);
    CPPUNIT_ASSERT(logger1 != logger_def);

    TEST_INFO("Calling the Function logger. VISUAL CHECK!.");
    FunctionLoggerUser flu(logger1);

    TEST_INFO("Calling op1 (with no return and no exception).");
    flu.op1();
    CPPUNIT_ASSERT(true);

    int retval = 30;
    TEST_INFO("Calling op_return that return value " << retval);
    int res = flu.op_return(retval);
    CPPUNIT_ASSERT(res == retval);

    retval = 50;
    TEST_INFO("Calling op_exception that should raise exception with value " << retval);
    try {
        flu.op_exception(retval);
        CPPUNIT_ASSERT(false);
    } catch (int ex) {
        CPPUNIT_ASSERT(ex == retval);
    }
}
