/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.logging.test;

import org.junit.Test;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.fail;

import  cdmw.logging.*;
import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;

public class TestLogger {

    class FunctionLoggerUser
    {
        private Logger logger; 
        private FunctionLogger funcLog;
        
        public FunctionLoggerUser(Logger logger) {
            this.logger = logger; 
        }
        
        public void op1() {
            CDMW_LOG_FUNCTION(this.logger, "xxxxxxxxx");
            CDMW_LOG_FUNCTION(this.logger, "this function was called from that file ...depth=1", 1);
            CDMW_LOG_FUNCTION(this.logger, "this function caller was called from that file ...depth=2", 2);
            CDMW_LOG_FUNCTION_RETURN(this.logger);
        }
        
        public int op_return(int input) {
            CDMW_LOG_FUNCTION(this.logger, ""+input);
            CDMW_LOG_FUNCTION_RETURN(this.logger, input);
            return input;
        }
        
        public void op_exception(int input) throws java.lang.Exception {
            CDMW_LOG_FUNCTION(this.logger, ""+input);
            Exception ex = new java.lang.Exception(String.valueOf(input));
            CDMW_LOG_FUNCTION_EXCEPTION(this.logger, ex);
            throw ex;
        }
    }

    @Test
    public void testLogger() {
        Logger logger_def = null;
        Logger logger1 = null;
        Logger logger2 = null;
        LevelType level;

        System.out.println("Obtaining a default logger for the TEST_LOGGING1 domain");
        System.out.println("Only messages with a level <= INFO (ie lvl <= 8) are displayed");        
        logger_def = Logger.Get_Logger("TEST_LOGGING1");
        assertTrue(logger_def != null);
        logger1 = logger_def;

        System.out.println("Logger created succesfully");

        System.out.println ("Logging an info message");
        
        CDMW_INFO(logger_def,"my info message 1", new Throwable());
        assertTrue(logger_def.isInfoEnabled());

        CDMW_INFO(logger_def,"my info message 1", new Throwable());
        assertTrue(logger_def.isInfoEnabled());

        CDMW_INFO(logger_def,"my info message 1");
        assertTrue(logger_def.isInfoEnabled());
        
        CDMW_INFO(logger_def,"my info message at stack depth 1",1); //depth=1;
        assertTrue(logger_def.isInfoEnabled());
        
        System.out.println("Retrieve the logger of the TEST_LOGGING1 domain");       
        logger2 = Logger.Get_Logger("TEST_LOGGING1");
        assertTrue(logger2 != null);
        assertTrue(logger2 == logger_def);
    
        System.out.println ("Logging a warning message");
        
        CDMW_WARN(logger2,"my warning message 1", new Throwable());
        assertTrue(logger2.isWarnEnabled());

        CDMW_WARN(logger2,"my warning message 1", new Throwable());
        assertTrue(logger2.isInfoEnabled());

        CDMW_WARN(logger2,"my warning message 1");
        assertTrue(logger2.isInfoEnabled());
        
        CDMW_WARN(logger2,"my warning message  at stack depth 2", 2); //depth=2
        assertTrue(logger2.isInfoEnabled());
                
        System.out.println("Obtaining a default logger for the TEST_LOGGING2 domain");
        logger1 = Logger.Get_Logger("TEST_LOGGING2");
        assertTrue(logger1 != null);
        assertTrue(logger1 != logger_def);

        System.out.println ("Logging an error message");
        
        CDMW_ERROR(logger1,"my error message 1", new Throwable());
        assertTrue(logger1.isErrorEnabled());

        CDMW_ERROR(logger1,"my error message 1", new Throwable());
        assertTrue(logger1.isErrorEnabled());
        
        CDMW_ERROR(logger1,"my error message 1");
        assertTrue(logger1.isErrorEnabled());
        
        CDMW_ERROR(logger1,"my error message 1 at stack depth 3", 3);
        assertTrue(logger1.isErrorEnabled());                
        
        System.out.println("Retrieve the logger of the TEST_LOGGING2 domain");       
        logger2 = Logger.Get_Logger("TEST_LOGGING2");
        assertTrue(logger2 != null);
        assertTrue(logger2 == logger1);
    
        System.out.println ("Logging a fatal error message");
        
        CDMW_FATAL(logger2, "my fatal error message 1", new Throwable());
        assertTrue(logger2.isFatalEnabled());
        
        CDMW_FATAL(logger2,"my fatal error message 1", new Throwable());
        assertTrue(logger2.isFatalEnabled());
        
        CDMW_FATAL(logger2,"my fatal error message 1");
        assertTrue(logger2.isFatalEnabled());

        CDMW_FATAL(logger2,"my fatal error message 1 at stack depth 4", 4);
        assertTrue(logger2.isFatalEnabled());
        
        System.out.println("Retrieve the logger of the TEST_LOGGING1 domain");       
        logger1 = Logger.Get_Logger("TEST_LOGGING1");
        assertTrue(logger1 != null);
        assertTrue(logger1 != logger2);
        assertTrue(logger1 == logger_def);
    
        System.out.println ("Logging another fatal error message");
        
        CDMW_FATAL(logger1, "my fatal error message 2", new Throwable());
        assertTrue(logger1.isFatalEnabled());
        
        System.out.println ("Logging a debug message");
        
        CDMW_DEBUG(logger_def, "my debug message 1", new Throwable());
        assertFalse(logger_def.isDebugEnabled());

        CDMW_DEBUG(logger_def,"my debug message 1", new Throwable());
        assertTrue(logger_def.isFatalEnabled());

        CDMW_DEBUG(logger_def,"my debug message 1");
        assertTrue(logger_def.isFatalEnabled());
        
        CDMW_DEBUG(logger_def,"my debug message 1 at stack depth 5", 5);
        assertTrue(logger_def.isFatalEnabled());
                
        level = new LevelType (LogLevel.DEBUG_1);
        CDMW_DEBUG_1(logger_def,"my generic message 1",new Throwable());
        assertFalse(logger_def.isEnabledFor(level));
        
        CDMW_DEBUG_1(logger_def,"my generic message 1 with default stack depth");
        assertFalse(logger_def.isEnabledFor(level));
        
        CDMW_DEBUG_1(logger_def,"my generic message 1 at stack depth 6",6);
        assertFalse(logger_def.isEnabledFor(level));

        level = new LevelType (LogLevel.DEBUG_2);
        CDMW_DEBUG_2(logger_def,"my generic message 2",new Throwable());
        assertFalse(logger_def.isEnabledFor(level));

        CDMW_DEBUG_2(logger_def,"my generic message 2 with default stack depth");
        assertFalse(logger_def.isEnabledFor(level));
        
        CDMW_DEBUG_2(logger_def,"my generic message 2 at stack depth 7",7);
        assertFalse(logger_def.isEnabledFor(level));
                
        level = new LevelType (LogLevel.DEBUG_3);
        CDMW_DEBUG_3(logger_def,"my generic message 3",new Throwable());
        assertFalse(logger_def.isEnabledFor(level));

        CDMW_DEBUG_3(logger_def,"my generic message 3 with default stack depth");
        assertFalse(logger_def.isEnabledFor(level));
        
        CDMW_DEBUG_3(logger_def,"my generic message 3 at stack depth 8",8);
        assertFalse(logger_def.isEnabledFor(level));
        
        level = new LevelType (LogLevel.DEBUG_4);
        CDMW_DEBUG_4(logger_def,"my generic message 4",new Throwable());
        assertFalse(logger_def.isEnabledFor(level));

        CDMW_DEBUG_4(logger_def,"my generic message 4 with default stack depth");
        assertFalse(logger_def.isEnabledFor(level));
        
        CDMW_DEBUG_4(logger_def,"my generic message 4 at stack depth 9",9);
        assertFalse(logger_def.isEnabledFor(level));
        
        level = new LevelType (LogLevel.DEBUG_5);
        CDMW_DEBUG_5(logger_def,"my generic message 5",new Throwable());
        assertFalse(logger_def.isEnabledFor(level));

        level = new LevelType (LogLevel.DEBUG_6);
        CDMW_DEBUG_6(logger_def,"my generic message 6",new Throwable());
        assertFalse(logger_def.isEnabledFor(level));

        level = new LevelType (LogLevel.DEBUG_7);
        CDMW_DEBUG_7(logger_def,"my generic message 7",new Throwable());
        assertFalse(logger_def.isEnabledFor(level));

        level = new LevelType (LogLevel.DEBUG_8);
        CDMW_DEBUG_8(logger_def,"my generic message 8",new Throwable());
        assertFalse(logger_def.isEnabledFor(level));

        level = new LevelType (LogLevel.DEBUG_9);
        CDMW_DEBUG_9(logger_def,"my generic message 9",new Throwable());
        assertFalse(logger_def.isEnabledFor(level));

        System.out.println ("Logging a generic message with level 7.");

        level = new LevelType (7);
        CDMW_LOG(Logger.Get_Logger("TEST_LOGGING1"), level, "my generic message 1", new Throwable());
        assertTrue(logger_def.isEnabledFor(level));
        
        level = new LevelType (20);
        CDMW_LOG(Logger.Get_Logger("TEST_LOGGING2"), level, "my generic message 2", new Throwable());
        assertFalse(logger2.isEnabledFor(level));

        System.out.println("Obtaining a default logger for the COMPONENT1 of the TEST_LOGGING1 domain");
        System.out.println("Only messages with a level <= INFO (ie lvl <= 8) are displayed");        
        logger_def = Logger.Get_Logger ("COMPONENT1", "TEST_LOGGING1");
        assertTrue(logger_def != null);
        assertTrue(logger_def != logger1);
        assertTrue(logger_def != logger2);
        logger1 = logger_def;
        assertTrue(logger1 == logger_def);

        System.out.println("Logger created succesfully");

        System.out.println("Logging an info message");
        
        CDMW_INFO(logger_def, "my info message 2", new Throwable());
        assertTrue(logger_def.isInfoEnabled());

        System.out.println("Retrieve the logger for the COMPONENT1 of the TEST_LOGGING1 domain");
        logger2 = Logger.Get_Logger ("COMPONENT1", "TEST_LOGGING1");
        assertTrue(logger2 != null);
        assertTrue(logger2 == logger_def);

        System.out.println("Logging a warning message");
        
        CDMW_WARN(logger2,"my warning message 2", new Throwable());
        assertTrue(logger2.isWarnEnabled());

        CDMW_ERROR(logger2,"my warning message 2 at stack depth 10", 10);
        assertTrue(logger2.isErrorEnabled());
        
        CDMW_ERROR(logger2,"my warning message 2");
        assertTrue(logger2.isErrorEnabled());
        
        System.out.println("Obtaining a default logger for the COMPONENT2 of the TEST_LOGGING1 domain");
        logger1 = Logger.Get_Logger ("COMPONENT2", "TEST_LOGGING1");
        assertTrue(logger1 != null);
        assertTrue(logger1 != logger_def);

        System.out.println("Logging an error message");
        
        CDMW_ERROR(logger1,"my error message ", new Throwable());
        assertTrue(logger1.isErrorEnabled());
        
        System.out.println("Retrieve the logger for the COMPONENT2 of the TEST_LOGGING1 domain");
        logger2 = Logger.Get_Logger ("COMPONENT2", "TEST_LOGGING1");
        assertTrue(logger2 != null);
        assertTrue(logger2 == logger1);

        System.out.println("Logging a fatal error message");
        
        CDMW_FATAL(logger2, "my fatal error message 3", new Throwable());
        assertTrue(logger2.isFatalEnabled());

        System.out.println("Retrieve the logger for the COMPONENT1 of the TEST_LOGGING1 domain");
        logger1 = Logger.Get_Logger ("COMPONENT1", "TEST_LOGGING1");
        assertTrue(logger1 != null);
        assertTrue(logger1 != logger2);
        assertTrue(logger1 == logger_def);

        System.out.println("Logging another fatal error message");
        
        CDMW_FATAL(logger1, "my fatal error message 4", new Throwable());
        assertTrue(logger1.isFatalEnabled());

        System.out.println("Logging a debug message (lvl >8).");
        CDMW_DEBUG(logger_def, "Logging a debug message (lvl >8).", new Throwable());
        assertFalse(logger_def.isDebugEnabled());

        System.out.println("Logging a generic message with level 7");
        level = new LevelType (7);
        CDMW_LOG(Logger.Get_Logger ("COMPONENT1", "TEST_LOGGING1"), level, "my generic message 3", new Throwable());
        assertFalse(logger_def.isDebugEnabled());
        CDMW_LOG(Logger.Get_Logger ("COMPONENT1", "TEST_LOGGING1"), level, "my generic message 3 with default stack depth");
        assertFalse(logger_def.isDebugEnabled());        
        CDMW_LOG(Logger.Get_Logger ("COMPONENT1", "TEST_LOGGING1"), level, "my generic message 3 at stack depth 10", 10);
        assertFalse(logger_def.isDebugEnabled());        
      
        System.out.println("Logging a generic message with level 20");
        level = new LevelType (20);
        CDMW_LOG(Logger.Get_Logger ("COMPONENT1", "TEST_LOGGING2"), level, "my generic message 4", new Throwable());
        assertFalse(logger_def.isEnabledFor(level));

        System.out.println("Obtaining a default logger for the TEST_LOGFUNC domain");
        System.out.println("Only messages with a level <= INFO (ie lvl <= 8) are displayed");        
        logger1 = Logger.Get_Logger("TEST_LOGFUNC");
        assertTrue(logger1 != null);
        assertTrue(logger1 != logger2);
        assertTrue(logger1 != logger_def);

        System.out.println("Logger created succesfully");

        System.out.println("Calling the Function logger. VISUAL CHECK!.");
        FunctionLoggerUser flu = new FunctionLoggerUser(logger1);

        System.out.println("Calling op1 (with no return and no exception).");
        flu.op1();

        int retval = 30;
        System.out.println("Calling op_return that return value " + retval);
        int res = flu.op_return(retval);
        
        assertFalse(res != retval);
        
        retval = 50;
        System.out.println("Calling op_exception that should raise exception with value " + retval);
        try {
            flu.op_exception(retval);
            fail("FAIL:- Calling op_exception that should raise exception with value " + retval);
        } catch (java.lang.Exception ex) {
            assertTrue(ex.getLocalizedMessage().equals(String.valueOf(retval)));
        }
    }
}
