/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <Foundation/osthreads/MutexGuard.hpp>
#include <Foundation/logging/LogLevel.hpp>
#include <ostream>
#include <iostream>

#include "testlogmanager/LogCollector_impl.hpp"

using namespace Cdmw::Logging::Test;


/**
 * Constructor.
 */
LogCollector_impl::LogCollector_impl(std::string file_name)
    :
    m_ofstream(),
    m_postream(&std::cout),
    m_mutex()
{
    m_ofstream.open(file_name.c_str(), std::ios::out);

    if (! m_ofstream)
    {
        std::cerr << "Couldn't create log file " << file_name << std::endl;
        m_postream = &std::cout;
    }
    else 
    {
        m_postream = &m_ofstream;
        std::cout << "CDMW Log file set to " << file_name << std::endl;
    }
}


/**
 * Destructor.
 */
LogCollector_impl::~LogCollector_impl()
    throw()
{
    try 
    {
        m_postream = &std::cout;
        if (m_ofstream)
        {
            m_ofstream.close();
        }
    }
    catch (...)
    {
        // Ignore ...
    }
}

::CORBA::Boolean
LogCollector_impl::add_log (
    const ::CdmwLogging::StructuredLog & log
    )
    throw(::CORBA::SystemException)
{
    // context not logged
    std::ostringstream buffer;
    buffer << log.ctx.application_name << "-"
           << log.ctx.process_name << "-"
           << log.header.component_name << "-" 
           << log.header.user_domain << "-"
           << Cdmw::Logging::LevelUtilities::Level_To_String(log.header.level)
           << log.message;

    CDMW_MUTEX_GUARD(m_mutex);
    (*m_postream) << buffer.str() << std::endl;

    return true;
}
