/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <fstream>
#include <sstream>
#include <iostream>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/orbsupport/CORBASystemExceptionWrapper.hpp>

#include "testlogmanager/LogCollector_impl.hpp"
#include "testlogmanager/Common.hpp"

namespace 
{
    const int SUCCESS = 0;
    const int FAILURE = 1;

    const int POA_THREAD_POOL_SIZE = 5;

}; // End anonymous namespace

using namespace Cdmw::Logging::Test;

int
main(int argc, char** argv)
{
    try 
    {
        // ===================================================
        // Initialize the ORB
        // ===================================================
        Cdmw::OrbSupport::StrategyList orb_strategies;
        orb_strategies.add_OrbThreaded();
        orb_strategies.add_PoaThreadPool(POA_THREAD_POOL_SIZE);
        
        CORBA::ORB_var orb = Cdmw::OrbSupport::OrbSupport::ORB_init(
            argc, argv, orb_strategies);
        
        // ===================================================
        // Get the root POA 
        // ===================================================
        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var rootPOA =
            PortableServer::POA::_narrow(obj.in());
        
        // ===================================================
        // Activate the root POA manager
        // ===================================================
        PortableServer::POAManager_var poaManager = rootPOA->the_POAManager();
        poaManager->activate();
        

        // ===================================================
        // Activate the LogCollector servant
        // ===================================================
        ::LogCollector_impl* collector_impl =
              new LogCollector_impl(OUTPUT_LOG_FILE);
        CdmwLogging::LogCollector_var collector = collector_impl->_this();

        // We write its ior in a file
        std::ofstream iorFile(LOG_COLLECTOR_IOR_FILE.c_str());
        CORBA::String_var strIor = orb->object_to_string(collector.in());
        iorFile << strIor.in() << std::endl;
        iorFile.close();

        // Setup signal handling for ORB to be shut down
        Cdmw::OrbSupport::OrbSupport::install_orb_shutdown_signal_handlers(
           orb.in());

        orb->run();
        
        Cdmw::OrbSupport::OrbSupport::uninstall_orb_shutdown_signal_handlers(
           orb.in());


        if (!CORBA::is_nil(orb.in())) 
        {
            try 
            {
                Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
                orb->destroy();
            }
            catch (const CORBA::SystemException& e)
            {
                std::cerr << "Unexpected CORBA::SystemException "
                          << "during orb->detroy():\n"
                          << e << " - minor code: " << e.minor()
                          << std::endl;
            }
        }
    }

    catch (const Cdmw::OrbSupport::CORBASystemExceptionWrapper & ex)
    {
        std::cerr << ex.what() << std::endl;
    }
    catch (const Cdmw::Exception & ex)
    {
        std::cerr << ex.what() << std::endl;
    } 
    catch(const CORBA::Exception& ex)  
    {        
        std::cerr << ex << std::endl;
    }
    catch(...)  
    {        
        std::cerr << "unexpected exception" << std::endl;
    }

    return 0;
}
