/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/common/Exception.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/logging/LogManagerCollectorDelegate.hpp>
#include <Foundation/logging/LogLevel.hpp>

#include "testlogmanager/TestLogManager.hpp"
#include "testlogmanager/Common.hpp"
#include <fstream>

using Cdmw::Logging::Test::TestLogManager;

CPPUNIT_TEST_SUITE_REGISTRATION( TestLogManager );

/**
 * Helper class to create any LogManager instances.
 * The inheritance allows to bypass the Singleton pattern.
 */
class MyLogManager: virtual public Cdmw::Logging::LogManager
{
public:
    MyLogManager(int& argc, char** argv)
        throw(Cdmw::OutOfMemoryException,
              Cdmw::InternalErrorException)
        :
        Cdmw::Logging::LogManager(argc, argv)
    {
    }
};

/**
 * Helper class to create any Logger instances.
 * The inheritance allows to bypass the Singleton pattern.
 */
class MyLogger: virtual public Cdmw::Logging::Logger
{
public:
    static Cdmw::Logging::Logger* Get_Logger(
        Cdmw::Logging::LogManager* manager,
        std::string component,
        std::string domain)
    {
        return Cdmw::Logging::Logger::Get_Logger(manager, component, domain);
    }
};

/**
 * Default constructor.
 */
TestLogManager::TestLogManager()
{
    m_orb =  CORBA::ORB::_duplicate(
        Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());
}


/**
 * Destructor.
 */
TestLogManager::~TestLogManager()
    throw()
{
}


/**
 * Run the tests.
 */
void
TestLogManager::test_level_filter()
{
    TEST_INFO("Testing the log level option passing in "
              "LogManager initialization.");
    std::auto_ptr< MyLogManager > logger(0);
    Cdmw::Logging::FilterMngr *filter = 0;

    int argc = 2;
    char* argv[2];
    // argv[0] == "program_name"
    argv[1] = "--CdmwLogLevel=ALL";
    TEST_INFO("Check the option " << argv[1]);
    logger.reset(new MyLogManager(argc, argv));
    filter = logger->get_filter_manager();
    for (Cdmw::Logging::LevelType level = Cdmw::Logging::FATAL;
         level != 0;
         level *= 2)
    {
        CPPUNIT_ASSERT(filter->is_level_activated(level));
    }

    argv[1] = "--CdmwLogLevel=4294967295";
    TEST_INFO("Check the option " << argv[1]);
    logger.reset(new MyLogManager(argc, argv));
    filter = logger->get_filter_manager();
    for (Cdmw::Logging::LevelType level = Cdmw::Logging::FATAL;
         level != 0;
         level *= 2)
    {
        CPPUNIT_ASSERT(filter->is_level_activated(level));
    }

    argv[1] = "--CdmwLogLevel=0xFFFFFFFF";
    TEST_INFO("Check the option " << argv[1]);
    logger.reset(new MyLogManager(argc, argv));
    filter = logger->get_filter_manager();
    for (Cdmw::Logging::LevelType level = Cdmw::Logging::FATAL;
         level != 0;
         level *= 2)
    {
        CPPUNIT_ASSERT(filter->is_level_activated(level));
    }

    argv[1] = "--CdmwLogLevel=0";
    TEST_INFO("Check the option " << argv[1]);
    logger.reset(new MyLogManager(argc, argv));
    filter = logger->get_filter_manager();
    for (Cdmw::Logging::LevelType level = Cdmw::Logging::FATAL;
         level != 0;
         level *= 2)
    {
        CPPUNIT_ASSERT(! filter->is_level_activated(level));
    }

    argv[1] = "--CdmwLogLevel=NONE";
    TEST_INFO("Check the option " << argv[1]);
    logger.reset(new MyLogManager(argc, argv));
    filter = logger->get_filter_manager();
    for (Cdmw::Logging::LevelType level = Cdmw::Logging::FATAL;
         level != 0;
         level *= 2)
    {
        CPPUNIT_ASSERT(! filter->is_level_activated(level));
    }

    argv[1] = "--CdmwLogLevel=1";
    TEST_INFO("Check the option " << argv[1]);
    logger.reset(new MyLogManager(argc, argv));
    filter = logger->get_filter_manager();
    CPPUNIT_ASSERT(filter->is_level_activated(Cdmw::Logging::FATAL));
    for (Cdmw::Logging::LevelType level = Cdmw::Logging::ERROR;
         level != 0;
         level *= 2)
    {
        CPPUNIT_ASSERT(! filter->is_level_activated(level));
    }

    argv[1] = "--CdmwLogLevel=0x1";
    TEST_INFO("Check the option " << argv[1]);
    logger.reset(new MyLogManager(argc, argv));
    filter = logger->get_filter_manager();
    CPPUNIT_ASSERT(filter->is_level_activated(Cdmw::Logging::FATAL));
    for (Cdmw::Logging::LevelType level = Cdmw::Logging::ERROR;
         level != 0;
         level *= 2)
    {
        CPPUNIT_ASSERT(! filter->is_level_activated(level));
    }

    argv[1] = "--CdmwLogLevel=31";
    TEST_INFO("Check the option " << argv[1]);
    logger.reset(new MyLogManager(argc, argv));
    filter = logger->get_filter_manager();
    for (Cdmw::Logging::LevelType level = Cdmw::Logging::FATAL;
         level != Cdmw::Logging::DEBUG_1;
         level *= 2)
    {
        CPPUNIT_ASSERT(filter->is_level_activated(level));
    }
    for (Cdmw::Logging::LevelType level = Cdmw::Logging::DEBUG_1;
         level != 0;
         level *= 2)
    {
        CPPUNIT_ASSERT(! filter->is_level_activated(level));
    }

    argv[1] = "--CdmwLogLevel=31-FATAL";
    TEST_INFO("Check the option " << argv[1]);
    logger.reset(new MyLogManager(argc, argv));
    filter = logger->get_filter_manager();
    CPPUNIT_ASSERT(! filter->is_level_activated(Cdmw::Logging::FATAL));
    for (Cdmw::Logging::LevelType level = Cdmw::Logging::ERROR;
         level != Cdmw::Logging::DEBUG_1;
         level *= 2)
    {
        CPPUNIT_ASSERT(filter->is_level_activated(level));
    }
    for (Cdmw::Logging::LevelType level = Cdmw::Logging::DEBUG_1;
         level != 0;
         level *= 2)
    {
        CPPUNIT_ASSERT(! filter->is_level_activated(level));
    }

    argv[1] = "--CdmwLogLevel=0x1F-FATAL";
    TEST_INFO("Check the option " << argv[1]);
    logger.reset(new MyLogManager(argc, argv));
    filter = logger->get_filter_manager();
    CPPUNIT_ASSERT(! filter->is_level_activated(Cdmw::Logging::FATAL));
    for (Cdmw::Logging::LevelType level = Cdmw::Logging::ERROR;
         level != Cdmw::Logging::DEBUG_1;
         level *= 2)
    {
        CPPUNIT_ASSERT(filter->is_level_activated(level));
    }
    for (Cdmw::Logging::LevelType level = Cdmw::Logging::DEBUG_1;
         level != 0;
         level *= 2)
    {
        CPPUNIT_ASSERT(! filter->is_level_activated(level));
    }

    argv[1] = "--CdmwLogLevel=31-ERROR-INFO-WARN-DEBUG-FATAL";
    TEST_INFO("Check the option " << argv[1]);
    logger.reset(new MyLogManager(argc, argv));
    filter = logger->get_filter_manager();
    for (Cdmw::Logging::LevelType level = Cdmw::Logging::FATAL;
         level != 0;
         level *= 2)
    {
        CPPUNIT_ASSERT(! filter->is_level_activated(level));
    }

    argv[1] = "--CdmwLogLevel=0x1F-ERROR-INFO-WARN-DEBUG-FATAL";
    TEST_INFO("Check the option " << argv[1]);
    logger.reset(new MyLogManager(argc, argv));
    filter = logger->get_filter_manager();
    for (Cdmw::Logging::LevelType level = Cdmw::Logging::FATAL;
         level != 0;
         level *= 2)
    {
        CPPUNIT_ASSERT(! filter->is_level_activated(level));
    }

    argv[1] = "--CdmwLogLevel=31-FATAL-ERROR-INFO-WARN-DEBUG+FATAL";
    TEST_INFO("Check the option " << argv[1]);
    logger.reset(new MyLogManager(argc, argv));
    filter = logger->get_filter_manager();
    CPPUNIT_ASSERT(filter->is_level_activated(Cdmw::Logging::FATAL));
    for (Cdmw::Logging::LevelType level = Cdmw::Logging::ERROR;
         level != 0;
         level *= 2)
    {
        CPPUNIT_ASSERT(! filter->is_level_activated(level));
    }

    argv[1] = "--CdmwLogLevel=FATAL-FATAL";
    TEST_INFO("Check the option " << argv[1]);
    logger.reset(new MyLogManager(argc, argv));
    filter = logger->get_filter_manager();
    for (Cdmw::Logging::LevelType level = Cdmw::Logging::FATAL;
         level != 0;
         level *= 2)
    {
        CPPUNIT_ASSERT(! filter->is_level_activated(level));
    }

    argv[1] = "--CdmwLogLevel=ALL-ERROR";
    TEST_INFO("Check the option " << argv[1]);
    logger.reset(new MyLogManager(argc, argv));
    filter = logger->get_filter_manager();
    CPPUNIT_ASSERT(filter->is_level_activated(Cdmw::Logging::FATAL));
    CPPUNIT_ASSERT(! filter->is_level_activated(Cdmw::Logging::ERROR));
    for (Cdmw::Logging::LevelType level = Cdmw::Logging::WARN;
         level != 0;
         level *= 2)
    {
        CPPUNIT_ASSERT(filter->is_level_activated(level));
    }

    argv[1] = "--CdmwLogLevel=DEFAULT-INFO+CALL_TRACE";
    TEST_INFO("Check the option " << argv[1]);
    logger.reset(new MyLogManager(argc, argv));
    filter = logger->get_filter_manager();
    CPPUNIT_ASSERT(filter->is_level_activated(Cdmw::Logging::FATAL));
    CPPUNIT_ASSERT(filter->is_level_activated(Cdmw::Logging::ERROR));
    CPPUNIT_ASSERT(filter->is_level_activated(Cdmw::Logging::WARN));
    for (Cdmw::Logging::LevelType level = Cdmw::Logging::INFO;
         level != Cdmw::Logging::CALL_TRACE;
         level *= 2)
    {
        CPPUNIT_ASSERT(! filter->is_level_activated(level));
    }
    CPPUNIT_ASSERT(filter->is_level_activated(Cdmw::Logging::CALL_TRACE));
    CPPUNIT_ASSERT(! filter->is_level_activated(Cdmw::Logging::ORB_TRACE));

    argv[1] = "--CdmwLogLevel=ALL-DEFAULT-NONE";
    TEST_INFO("Check the option " << argv[1]);
    logger.reset(new MyLogManager(argc, argv));
    filter = logger->get_filter_manager();
    for (Cdmw::Logging::LevelType level = Cdmw::Logging::FATAL;
         level != Cdmw::Logging::DEBUG;
         level *= 2)
    {
        CPPUNIT_ASSERT(! filter->is_level_activated(level));
    }
    for (Cdmw::Logging::LevelType level = Cdmw::Logging::DEBUG;
         level != 0;
         level *= 2)
    {
        CPPUNIT_ASSERT(filter->is_level_activated(level));
    }

    argv[1] = "--CdmwLogLevel=ALL-FATAL-ERROR-WARN-INFO-DEBUG-DEBUG_1-DEBUG_2-DEBUG_3-DEBUG_4-DEBUG_5-DEBUG_6-DEBUG_7-DEBUG_8-DEBUG_9-CDMW_DEBUG_1-CDMW_DEBUG_2-CDMW_DEBUG_3-CDMW_DEBUG_4-ORB_TRACE-CALL_TRACE";
    TEST_INFO("Check the option " << argv[1]);
    logger.reset(new MyLogManager(argc, argv));
    filter = logger->get_filter_manager();
    CPPUNIT_ASSERT(! filter->is_level_activated(Cdmw::Logging::FATAL));
    CPPUNIT_ASSERT(! filter->is_level_activated(Cdmw::Logging::ERROR));
    CPPUNIT_ASSERT(! filter->is_level_activated(Cdmw::Logging::WARN));
    CPPUNIT_ASSERT(! filter->is_level_activated(Cdmw::Logging::INFO));
    CPPUNIT_ASSERT(! filter->is_level_activated(Cdmw::Logging::DEBUG));
    CPPUNIT_ASSERT(! filter->is_level_activated(Cdmw::Logging::DEBUG_1));
    CPPUNIT_ASSERT(! filter->is_level_activated(Cdmw::Logging::DEBUG_2));
    CPPUNIT_ASSERT(! filter->is_level_activated(Cdmw::Logging::DEBUG_3));
    CPPUNIT_ASSERT(! filter->is_level_activated(Cdmw::Logging::DEBUG_4));
    CPPUNIT_ASSERT(! filter->is_level_activated(Cdmw::Logging::DEBUG_5));
    CPPUNIT_ASSERT(! filter->is_level_activated(Cdmw::Logging::DEBUG_6));
    CPPUNIT_ASSERT(! filter->is_level_activated(Cdmw::Logging::DEBUG_7));
    CPPUNIT_ASSERT(! filter->is_level_activated(Cdmw::Logging::DEBUG_8));
    CPPUNIT_ASSERT(! filter->is_level_activated(Cdmw::Logging::DEBUG_9));
    CPPUNIT_ASSERT(! filter->is_level_activated(Cdmw::Logging::CDMW_DEBUG_1));
    CPPUNIT_ASSERT(! filter->is_level_activated(Cdmw::Logging::CDMW_DEBUG_2));
    CPPUNIT_ASSERT(! filter->is_level_activated(Cdmw::Logging::CDMW_DEBUG_3));
    CPPUNIT_ASSERT(! filter->is_level_activated(Cdmw::Logging::CDMW_DEBUG_4));
    CPPUNIT_ASSERT(! filter->is_level_activated(Cdmw::Logging::ORB_TRACE));
    CPPUNIT_ASSERT(! filter->is_level_activated(Cdmw::Logging::CALL_TRACE));


    argv[1] = "--CdmwLogLevel=FATAL+0x06+DEBUG+8";
    TEST_INFO("Check the option " << argv[1]);
    logger.reset(new MyLogManager(argc, argv));
    filter = logger->get_filter_manager();
    for (Cdmw::Logging::LevelType level = Cdmw::Logging::FATAL;
         level != Cdmw::Logging::DEBUG_1;
         level *= 2)
    {
        CPPUNIT_ASSERT(filter->is_level_activated(level));
    }
    for (Cdmw::Logging::LevelType level = Cdmw::Logging::DEBUG_1;
         level != 0;
         level *= 2)
    {
        CPPUNIT_ASSERT(! filter->is_level_activated(level));
    }

    argv[1] = "--CdmwLogLevel= FATAL+0x06+   DEBUG+ 8-DEBUG_1 - 0x06 ";
    TEST_INFO("Check the option " << argv[1]);
    logger.reset(new MyLogManager(argc, argv));
    filter = logger->get_filter_manager();
    CPPUNIT_ASSERT(filter->is_level_activated(Cdmw::Logging::FATAL));
    CPPUNIT_ASSERT(! filter->is_level_activated(Cdmw::Logging::ERROR));
    CPPUNIT_ASSERT(! filter->is_level_activated(Cdmw::Logging::WARN));
    CPPUNIT_ASSERT(filter->is_level_activated(Cdmw::Logging::INFO));
    CPPUNIT_ASSERT(filter->is_level_activated(Cdmw::Logging::DEBUG));
    for (Cdmw::Logging::LevelType level = Cdmw::Logging::DEBUG_1;
         level != 0;
         level *= 2)
    {
        CPPUNIT_ASSERT(! filter->is_level_activated(level));
    }
}

void
TestLogManager::test_log_collector()
{
    TEST_INFO("Testing the redirection of logs to a remote LogCollector.");

    int timescale = Cdmw::TestUtils::get_timescale();
    std::auto_ptr< MyLogManager > log_manager(0);

    try
    {
        Cdmw::OsSupport::OS::ProcessId pid = 
            Cdmw::OsSupport::OS::create_process(
                "main_log_collector", "", ".", 0, 0);
        Cdmw::OsSupport::OS::sleep(timescale * 2000);

        int argc = 3;
        char* argv[argc];
        argv[0] = "";
        argv[1] = "--CdmwLogLevel=ALL";
        argv[2] = "--CdmwLogVerbose";
        log_manager.reset(new MyLogManager(argc, argv));

        std::string ior_str = "file://" + LOG_COLLECTOR_IOR_FILE;
        CORBA::Object_var obj = m_orb->string_to_object(ior_str.c_str());

        ::CdmwLogging::LogCollector_var collector =
            ::CdmwLogging::LogCollector::_narrow(obj.in());
        CPPUNIT_ASSERT(! CORBA::is_nil(collector.in()));

        Cdmw::Logging::LogManagerCollectorDelegate delegate(collector);
        std::string component = "LOG_MANAGER";
        std::string domain = "TEST_LOG_MANAGER1";
        Logger* logger = MyLogger::Get_Logger(
            log_manager.get(), component, domain);

        logger->log(Cdmw::Logging::INFO, "log 1", __FILE__, __LINE__);

        // logs some traces that are expected to be delegated to the LogCollector
        std::string app_name = "TestLogManager";
        std::string proc_name = "TestDriver";
        log_manager->register_alternate_delegate(&delegate);

        domain = "TEST_LOG_MANAGER2";
        logger = MyLogger::Get_Logger(log_manager.get(), component, domain);
        logger->log(Cdmw::Logging::INFO, "log 2", __FILE__, __LINE__);

        domain = "TEST_LOG_MANAGER3";
        logger = MyLogger::Get_Logger(log_manager.get(), component, domain);
        logger->log(Cdmw::Logging::INFO, "log 3", __FILE__, __LINE__);

        log_manager->set_application_name(app_name);
        domain = "TEST_LOG_MANAGER4";
        logger = MyLogger::Get_Logger(log_manager.get(), component, domain);
        logger->log(Cdmw::Logging::INFO, "log 4", __FILE__, __LINE__);

        log_manager->set_process_name(proc_name);
        domain = "TEST_LOG_MANAGER5";
        logger = MyLogger::Get_Logger(log_manager.get(), component, domain);
        logger->log(Cdmw::Logging::INFO, "log 5", __FILE__, __LINE__);
        Cdmw::OsSupport::OS::sleep(timescale * 2000); // wait logs are treated

        // check the LogCollector has received the logs
        std::string line;
        std::ifstream log(OUTPUT_LOG_FILE.c_str());
        if (log.is_open())
        {
            getline(log, line);
            CPPUNIT_ASSERT(line.find("log 2") < line.size());
            CPPUNIT_ASSERT(line.find("LOG_MANAGER") < line.size());
            CPPUNIT_ASSERT(line.find("TEST_LOG_MANAGER2") < line.size());

            getline(log, line);
            CPPUNIT_ASSERT(line.find("log 3") < line.size());
            CPPUNIT_ASSERT(line.find("LOG_MANAGER") < line.size());
            CPPUNIT_ASSERT(line.find("TEST_LOG_MANAGER3") < line.size());

            getline(log, line);
            CPPUNIT_ASSERT(line.find("log 4") < line.size());
            CPPUNIT_ASSERT(line.find("LOG_MANAGER") < line.size());
            CPPUNIT_ASSERT(line.find("TEST_LOG_MANAGER4") < line.size());
            CPPUNIT_ASSERT(line.find(app_name) < line.size());

            getline(log, line);
            CPPUNIT_ASSERT(line.find("log 5") < line.size());
            CPPUNIT_ASSERT(line.find("LOG_MANAGER") < line.size());
            CPPUNIT_ASSERT(line.find("TEST_LOG_MANAGER5") < line.size());
            CPPUNIT_ASSERT(line.find(proc_name) < line.size());

            getline(log, line);
            CPPUNIT_ASSERT(log.eof());
            log.close();
        }
        else
        {
            CPPUNIT_FAIL("Unable to open log file");
        }

        Cdmw::OsSupport::OS::kill_process(pid);
    }

    catch (const CORBA::Exception& e)
    {
        TEST_INFO("CORBA Exception raised: " << e);
        CPPUNIT_ASSERT(false);
    }
    catch (const std::exception& e)
    {
        TEST_INFO("std::exception raised: " << e.what());
        CPPUNIT_ASSERT(false);
    }
    catch (...)
    {
        TEST_INFO("unexpected exception raised!");
        CPPUNIT_ASSERT(false);
    }
}
