/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.logging.test;

import cdmw.orbsupport.*;

import java.io.FileWriter;
import org.omg.CORBA.*;
import org.omg.PortableServer.POAHelper;
import org.omg.PortableServer.POAPackage.ServantNotActive;
import org.omg.PortableServer.POAPackage.WrongPolicy;

public class MainLogCollector {

    public final static String LOG_COLLECTOR_IOR_FILE = "RefLogCollector.ior";
    public final static String OUTPUT_LOG_FILE = "OutputLogCollector.log";

    private final static int POA_THREAD_POOL_SIZE = 5;
    private final static int SUCCESS = 0;
    private final static int FAILURE = 1;


    static public void main(String args[]) {

        int status = SUCCESS;
        
        // ===================================================
        // Initialize the ORB
        // ===================================================
        StrategyList orbStrategies = null;
        try {
            orbStrategies = new StrategyList();
            orbStrategies.addORBThreaded();
            orbStrategies.addPOAThreadPool(POA_THREAD_POOL_SIZE);
        }
        catch (java.lang.Exception e) {
            e.printStackTrace();
            status = FAILURE;
        }

        org.omg.CORBA.ORB orb = null;

        if (status == SUCCESS) {
            try {
                orb = ORBSupport.ORBInit(args, orbStrategies);
                System.out.println("Created ORB");
            }
            catch (java.lang.Exception e) {
                e.printStackTrace();
                status = FAILURE;
            }
        }

        OrbShutdownSignalHandler sighandler = null;
        if (status == SUCCESS) {
            try {
                // ===================================================
                // Add ORB shutdown hooks
                // ===================================================
                sighandler = new OrbShutdownSignalHandler (orb);
                ORBSupport.InstallORBShutdownSignalHandler (orb, sighandler);
                System.out.println ("Added shutdown Hooks");
                Thread.sleep(1000);
            }
            catch (java.lang.Exception ex) {
                System.err.println (
                        "Caught exception during InstallORBShutdownSignalHandler: "
                        + ex.getMessage());
                ex.printStackTrace();
                status = FAILURE;
            }
        }

        org.omg.PortableServer.POA rootPOA = null;
        if (status == SUCCESS) {
            try {
                // ===================================================
                // Get the root POA
                // ===================================================
                org.omg.CORBA.Object obj = orb.resolve_initial_references("RootPOA");
                rootPOA = POAHelper.narrow(obj);
                System.out.println("Created POA");

                // ===================================================
                // Activate the root POA manager
                // ===================================================
                org.omg.PortableServer.POAManager poaManager = rootPOA.the_POAManager();
                poaManager.activate();
                System.out.println("POA activated");
            }
            catch (java.lang.Exception e) {
                e.printStackTrace();
                status = FAILURE;
            }
        }

        // ===================================================
        // Activate the LogCollector servant
        // ===================================================
        try {
            // create the LogCollector and write its ior in a file
            LogCollectorImpl collectorImpl = new LogCollectorImpl(OUTPUT_LOG_FILE);
            org.omg.CORBA.Object objImpl = rootPOA.servant_to_reference(collectorImpl);
            String strIor = orb.object_to_string(objImpl);
            System.out.println(strIor);
            FileWriter iorFile = new FileWriter(LOG_COLLECTOR_IOR_FILE);
            iorFile.write(strIor);
            iorFile.close();
        }
        catch (java.io.IOException ioe) {
            System.err.println("Cannot Write IOR: " + ioe.getMessage());
            ioe.printStackTrace();
        }
        catch (org.omg.PortableServer.POAPackage.ServantNotActive sna) {
            System.err.println("Cannot activate servant: " + sna.getMessage());
            sna.printStackTrace();
        }
        catch (org.omg.PortableServer.POAPackage.WrongPolicy wp) {
            System.err.println("Cannot apply policy: " + wp.getMessage());
            wp.printStackTrace();
        }

        orb.run();

        if (orb != null) {
            try {
                ORBSupport.ORBCleanup(orb);
                orb.destroy();
            }
            catch (java.lang.Exception e) {
                System.out.println("Unexpected Exception during orb.detroy():");
                e.printStackTrace() ;
            }

            try {
                ORBSupport.UninstallORBShutdownSignalHandler(sighandler);
            }
            catch (java.lang.Exception e) {
                System.out.println("Unexpected Exception during UninstallORBShutdownSignalHandler():");
                e.printStackTrace() ;
            }
        }

        if (status == SUCCESS) {
            System.out.println("Main test successful.");
        }
        else {
            System.out.println("Main test failed.");
        }
    }
}
