/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.logging.test;

import cdmw.orbsupport.*;
import cdmw.testutils.CORBATestManager;

import org.omg.CORBA.*;
import org.omg.PortableServer.POAHelper;

public class MainTest 
{

    private final static int POA_THREAD_POOL_SIZE = 5;
    private final static int SUCCESS = 0;
    private final static int FAILURE = 1;


    public static void main (String[] args) {

        int status = SUCCESS;

        // ===================================================
        // Initialize the ORB
        // ===================================================
        StrategyList orbStrategies = null;

        try {
            orbStrategies = new StrategyList();
            orbStrategies.addORBThreaded();
            orbStrategies.addPOAThreadPool(POA_THREAD_POOL_SIZE);
        }
        catch (java.lang.Exception e) {
            e.printStackTrace();
            status = FAILURE;
        }

        org.omg.CORBA.ORB orb = null;

        if (status == SUCCESS) {
            try {
                orb = ORBSupport.ORBInit(args, orbStrategies);
                System.out.println("Created ORB");
            }
            catch (java.lang.Exception e) {
                e.printStackTrace();
                status = FAILURE;
            }
        }

        OrbShutdownSignalHandler sighandler = null;
        if (status == SUCCESS) {
            try {
                // ===================================================
                // Add ORB shutdown hooks
                // ===================================================
                sighandler = new OrbShutdownSignalHandler (orb);
                ORBSupport.InstallORBShutdownSignalHandler (orb, sighandler);
                System.out.println ("Added shutdown Hooks");
                Thread.sleep(1000);
            }
            catch (java.lang.Exception ex) {
                System.err.println (
                        "Caught exception during InstallORBShutdownSignalHandler: "
                        + ex.getMessage());
                ex.printStackTrace();
                status = FAILURE;
            }
        }

        org.omg.PortableServer.POA rootPOA = null;
        if (status == SUCCESS) {
            try {
                // ===================================================
                // Get the root POA
                // ===================================================
                org.omg.CORBA.Object obj = orb.resolve_initial_references("RootPOA");
                rootPOA = POAHelper.narrow(obj);
                System.out.println("Created POA");

                // ===================================================
                // Activate the root POA manager
                // ===================================================
                org.omg.PortableServer.POAManager poaManager = rootPOA.the_POAManager();
                poaManager.activate();
                System.out.println("POA activated");
            }
            catch (java.lang.Exception e) {
                e.printStackTrace();
                status = FAILURE;
            }
        }

        // ===================================================
        // Run the test
        // ===================================================
        System.out.println("Creating new TestLogManager");
        CORBATestManager testManager = CORBATestManager.instance();
        testManager.init(MainTest.class.getName(), orb, rootPOA);

        testManager.setProperty(
            "TEST_LOGMANAGER_CLASS_NAME", "cdmw.logging.test.MainTest");

        testManager.addTest(TestLogManager.class);

        testManager.runTests();
        testManager.printSummary();
        status = testManager.isSuccessful() ? SUCCESS : FAILURE;

        if (orb != null) {
            try {
                ORBSupport.ORBCleanup(orb);
                orb.destroy();
            }
            catch (java.lang.Exception e) {
                System.out.println("Unexpected Exception during orb.detroy():");
                e.printStackTrace() ;
            }

            try {
                ORBSupport.UninstallORBShutdownSignalHandler(sighandler);
            }
            catch (java.lang.Exception e) {
                System.out.println("Unexpected Exception during UninstallORBShutdownSignalHandler():");
                e.printStackTrace() ;
            }
        }

        if (status == SUCCESS) {
            System.out.println("Main test successful.");
        }
        else {
            System.out.println("Main test failed.");
        }
    }
}


