/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.logging.test;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

import com.thalesgroup.CdmwLogging.LogCollector;
import com.thalesgroup.CdmwLogging.LogCollectorHelper;
import  cdmw.logging.*;
import static cdmw.logging.Logger.*;
import cdmw.common.*;

import cdmw.testutils.CORBATestManager;
import cdmw.testutils.CORBATestManager.InvalidObject;
import cdmw.common.*;
import cdmw.ossupport.*;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.fail;

import org.omg.CORBA.*;


public class TestLogManager
{

    private static org.omg.CORBA.ORB orb;
    private static org.omg.PortableServer.POA poa;
    private static String className;
    private static int timescale;

    
    @BeforeClass
    public static void initTests() throws InvalidObject {
        CORBATestManager testManager = CORBATestManager.instance();
        orb = testManager.getORB();
        poa = testManager.getPOA();
        className = testManager.getProperty("TEST_LOGMANAGER_CLASS_NAME");
        timescale = CORBATestManager.getTimescale();
    }

    @AfterClass
    public static void finishTests() {
        orb = null;
        poa = null;
        className = null;
    }

    /**
     * Helper class to create any LogManager instances.
     * The inheritance allows to bypass the Singleton pattern.
     */
    class MyLogManager extends LogManager
    {
        public MyLogManager(String[] args)
        {
            super(args);
        }
    }

    /**
     * Helper class to create any Logger instances.
     * The inheritance allows to bypass the Singleton pattern.
     */
    class MyLogger extends Logger
    {
        public MyLogger(LogManager logMngr, String componentName, String domain)
        {
            super(logMngr, componentName, domain);
        }
    }


    @Test
    public void testLevelFilter() {
        System.out.println("Testing the log level option passed in "
                + "LogManager initialization.");
        FilterMngr filter;
        MyLogManager logger;
        String []args = new String[2];
        args[0] = "";

        args[1] = "--CdmwLogLevel=ALL";
        System.out.println("check the option " + args[1]);
        logger = new MyLogManager(args);
        filter = logger.getFilterManager();
        for (long level = LogLevel.FATAL;
             level <= LogLevel.ALL_LEVELS_SET;
             level *= 2)
            {
                assertTrue(filter.isLevelActivated(level));
            }

        args[1] = "--CdmwLogLevel=4294967295";
        System.out.println("check the option " + args[1]);
        logger = new MyLogManager(args);
        filter = logger.getFilterManager();
        for (long level = LogLevel.FATAL;
             level <= LogLevel.ALL_LEVELS_SET;
             level *= 2)
            {
                assertTrue(filter.isLevelActivated(level));
            }

        args[1] = "--CdmwLogLevel=0xFFFFFFFF";
        System.out.println("check the option " + args[1]);
        logger = new MyLogManager(args);
        filter = logger.getFilterManager();
        for (long level = LogLevel.FATAL;
             level <= LogLevel.ALL_LEVELS_SET;
             level *= 2)
            {
                assertTrue(filter.isLevelActivated(level));
            }

        args[1] = "--CdmwLogLevel=0";
        System.out.println("check the option " + args[1]);
        logger = new MyLogManager(args);
        filter = logger.getFilterManager();
        for (long level = LogLevel.FATAL;
             level <= LogLevel.ALL_LEVELS_SET;
             level *= 2)
            {
                assertTrue(! filter.isLevelActivated(level));
            }

        args[1] = "--CdmwLogLevel=NONE";
        System.out.println("check the option " + args[1]);
        logger = new MyLogManager(args);
        filter = logger.getFilterManager();
        for (long level = LogLevel.FATAL;
             level <= LogLevel.ALL_LEVELS_SET;
             level *= 2)
            {
                assertTrue(! filter.isLevelActivated(level));
            }

        args[1] = "--CdmwLogLevel=1";
        System.out.println("check the option " + args[1]);
        logger = new MyLogManager(args);
        filter = logger.getFilterManager();
        assertTrue(filter.isLevelActivated(LogLevel.FATAL));
        for (long level = LogLevel.ERROR;
             level <= LogLevel.ALL_LEVELS_SET;
             level *= 2)
            {
                assertTrue(! filter.isLevelActivated(level));
            }

        args[1] = "--CdmwLogLevel=0x1";
        System.out.println("check the option " + args[1]);
        logger = new MyLogManager(args);
        filter = logger.getFilterManager();
        assertTrue(filter.isLevelActivated(LogLevel.FATAL));
        for (long level = LogLevel.ERROR;
             level <= LogLevel.ALL_LEVELS_SET;
             level *= 2)
            {
                assertTrue(! filter.isLevelActivated(level));
            }

        args[1] = "--CdmwLogLevel=31";
        System.out.println("check the option " + args[1]);
        logger = new MyLogManager(args);
        filter = logger.getFilterManager();
        for (long level = LogLevel.FATAL;
             level != LogLevel.DEBUG_1;
             level *= 2)
            {
                assertTrue(filter.isLevelActivated(level));
            }
        for (long level = LogLevel.DEBUG_1;
             level <= LogLevel.ALL_LEVELS_SET;
             level *= 2)
            {
                assertTrue(! filter.isLevelActivated(level));
            }

        args[1] = "--CdmwLogLevel=31-FATAL";
        System.out.println("check the option " + args[1]);
        logger = new MyLogManager(args);
        filter = logger.getFilterManager();
        assertTrue(! filter.isLevelActivated(LogLevel.FATAL));
        for (long level = LogLevel.ERROR;
             level != LogLevel.DEBUG_1;
             level *= 2)
            {
                assertTrue(filter.isLevelActivated(level));
            }
        for (long level = LogLevel.DEBUG_1;
             level <= LogLevel.ALL_LEVELS_SET;
             level *= 2)
            {
                assertTrue(! filter.isLevelActivated(level));
            }

        args[1] = "--CdmwLogLevel=0x1F-FATAL";
        System.out.println("check the option " + args[1]);
        logger = new MyLogManager(args);
        filter = logger.getFilterManager();
        assertTrue(! filter.isLevelActivated(LogLevel.FATAL));
        for (long level = LogLevel.ERROR;
             level != LogLevel.DEBUG_1;
             level *= 2)
            {
                assertTrue(filter.isLevelActivated(level));
            }
        for (long level = LogLevel.DEBUG_1;
             level <= LogLevel.ALL_LEVELS_SET;
             level *= 2)
            {
                assertTrue(! filter.isLevelActivated(level));
            }

        args[1] = "--CdmwLogLevel=31-ERROR-INFO-WARN-DEBUG-FATAL";
        System.out.println("check the option " + args[1]);
        logger = new MyLogManager(args);
        filter = logger.getFilterManager();
        for (long level = LogLevel.FATAL;
             level <= LogLevel.ALL_LEVELS_SET;
             level *= 2)
            {
                assertTrue(! filter.isLevelActivated(level));
            }

        args[1] = "--CdmwLogLevel=0x1F-ERROR-INFO-WARN-DEBUG-FATAL";
        System.out.println("check the option " + args[1]);
        logger = new MyLogManager(args);
        filter = logger.getFilterManager();
        for (long level = LogLevel.FATAL;
             level <= LogLevel.ALL_LEVELS_SET;
             level *= 2)
            {
                assertTrue(! filter.isLevelActivated(level));
            }

        args[1] = "--CdmwLogLevel=31-FATAL-ERROR-INFO-WARN-DEBUG+FATAL";
        System.out.println("check the option " + args[1]);
        logger = new MyLogManager(args);
        filter = logger.getFilterManager();
        assertTrue(filter.isLevelActivated(LogLevel.FATAL));
        for (long level = LogLevel.ERROR;
             level <= LogLevel.ALL_LEVELS_SET;
             level *= 2)
            {
                assertTrue(! filter.isLevelActivated(level));
            }

        args[1] = "--CdmwLogLevel=FATAL-FATAL";
        System.out.println("check the option " + args[1]);
        logger = new MyLogManager(args);
        filter = logger.getFilterManager();
        for (long level = LogLevel.FATAL;
             level <= LogLevel.ALL_LEVELS_SET;
             level *= 2)
            {
                assertTrue(! filter.isLevelActivated(level));
            }

        args[1] = "--CdmwLogLevel=ALL-ERROR";
        System.out.println("check the option " + args[1]);
        logger = new MyLogManager(args);
        filter = logger.getFilterManager();
        assertTrue(filter.isLevelActivated(LogLevel.FATAL));
        assertTrue(! filter.isLevelActivated(LogLevel.ERROR));
        for (long level = LogLevel.WARN;
             level <= LogLevel.ALL_LEVELS_SET;
             level *= 2)
            {
                assertTrue(filter.isLevelActivated(level));
            }

        args[1] = "--CdmwLogLevel=DEFAULT-INFO+CALL_TRACE";
        System.out.println("check the option " + args[1]);
        logger = new MyLogManager(args);
        filter = logger.getFilterManager();
        assertTrue(filter.isLevelActivated(LogLevel.FATAL));
        assertTrue(filter.isLevelActivated(LogLevel.ERROR));
        assertTrue(filter.isLevelActivated(LogLevel.WARN));
        for (long level = LogLevel.INFO;
             level != LogLevel.CALL_TRACE;
             level *= 2)
            {
                assertTrue(! filter.isLevelActivated(level));
            }
        assertTrue(filter.isLevelActivated(LogLevel.CALL_TRACE));
        assertTrue(! filter.isLevelActivated(LogLevel.ORB_TRACE));

        args[1] = "--CdmwLogLevel=ALL-DEFAULT-NONE";
        System.out.println("check the option " + args[1]);
        logger = new MyLogManager(args);
        filter = logger.getFilterManager();
        for (long level = LogLevel.FATAL;
             level != LogLevel.DEBUG;
             level *= 2)
            {
                assertTrue(! filter.isLevelActivated(level));
            }
        for (long level = LogLevel.DEBUG;
             level <= LogLevel.ALL_LEVELS_SET;
             level *= 2)
            {
                assertTrue(filter.isLevelActivated(level));
            }

        args[1] = "--CdmwLogLevel=ALL-FATAL-ERROR-WARN-INFO-DEBUG-DEBUG_1-DEBUG_2-DEBUG_3-DEBUG_4-DEBUG_5-DEBUG_6-DEBUG_7-DEBUG_8-DEBUG_9-CDMW_DEBUG_1-CDMW_DEBUG_2-CDMW_DEBUG_3-CDMW_DEBUG_4-ORB_TRACE-CALL_TRACE";
        System.out.println("check the option " + args[1]);
        logger = new MyLogManager(args);
        filter = logger.getFilterManager();
        assertTrue(! filter.isLevelActivated(LogLevel.FATAL));
        assertTrue(! filter.isLevelActivated(LogLevel.ERROR));
        assertTrue(! filter.isLevelActivated(LogLevel.WARN));
        assertTrue(! filter.isLevelActivated(LogLevel.INFO));
        assertTrue(! filter.isLevelActivated(LogLevel.DEBUG));
        assertTrue(! filter.isLevelActivated(LogLevel.DEBUG_1));
        assertTrue(! filter.isLevelActivated(LogLevel.DEBUG_2));
        assertTrue(! filter.isLevelActivated(LogLevel.DEBUG_3));
        assertTrue(! filter.isLevelActivated(LogLevel.DEBUG_4));
        assertTrue(! filter.isLevelActivated(LogLevel.DEBUG_5));
        assertTrue(! filter.isLevelActivated(LogLevel.DEBUG_6));
        assertTrue(! filter.isLevelActivated(LogLevel.DEBUG_7));
        assertTrue(! filter.isLevelActivated(LogLevel.DEBUG_8));
        assertTrue(! filter.isLevelActivated(LogLevel.DEBUG_9));
        assertTrue(! filter.isLevelActivated(LogLevel.CDMW_DEBUG_1));
        assertTrue(! filter.isLevelActivated(LogLevel.CDMW_DEBUG_2));
        assertTrue(! filter.isLevelActivated(LogLevel.CDMW_DEBUG_3));
        assertTrue(! filter.isLevelActivated(LogLevel.CDMW_DEBUG_4));
        assertTrue(! filter.isLevelActivated(LogLevel.CALL_TRACE));
        assertTrue(! filter.isLevelActivated(LogLevel.ORB_TRACE));


        args[1] = "--CdmwLogLevel=FATAL+0x06+DEBUG+8";
        System.out.println("Check the option " + args[1]);
        logger = new MyLogManager(args);
        filter = logger.getFilterManager();
        for (long level = LogLevel.FATAL;
             level != LogLevel.DEBUG_1;
             level *= 2)
        {
           assertTrue(filter.isLevelActivated(level));
        }
        for (long level = LogLevel.DEBUG_1;
             level != 0;
             level *= 2)
        {
           assertTrue(! filter.isLevelActivated(level));
        }
    
        args[1] = "--CdmwLogLevel=FATAL+0x06+DEBUG+8-DEBUG_1-0x06";
        System.out.println("Check the option " + args[1]);
        logger = new MyLogManager(args);
        filter = logger.getFilterManager();
        assertTrue(filter.isLevelActivated(LogLevel.FATAL));
        assertTrue(! filter.isLevelActivated(LogLevel.ERROR));
        assertTrue(! filter.isLevelActivated(LogLevel.WARN));
        assertTrue(filter.isLevelActivated(LogLevel.INFO));
        assertTrue(filter.isLevelActivated(LogLevel.DEBUG));
        for (long level = LogLevel.DEBUG_1;
             level != 0;
             level *= 2)
        {
           assertTrue(! filter.isLevelActivated(level));
        }

    }


    @Test
    public void testLogCollector() {
        System.out.println("Testing the redirection of logs to a remote LogCollector.");

        try {
            cdmw.ossupport.ThreadedProcess proc = 
                cdmw.ossupport.OS.createJavaProcess("cdmw.logging.test.MainLogCollector");
            Thread.sleep(2000);

            String []args = new String[3];
            args[0] = "";
            args[1] = "--CdmwLogLevel=ALL";
            args[2] = "--CdmwLogVerbose";
            MyLogManager logManager = new MyLogManager(args);

            // get the LogCollector
            org.omg.CORBA.Object obj =
                orb.string_to_object("file://" + MainLogCollector.LOG_COLLECTOR_IOR_FILE);
            LogCollector collector = LogCollectorHelper.narrow(obj);
            assertNotNull(collector);
            cdmw.logging.LogManagerCollectorDelegate delegate = new LogManagerCollectorDelegate(collector);
            assertNotNull(delegate);

            MyLogger logger = new MyLogger(logManager, "LOG_MANAGER", "TEST_LOG_MANAGER1");
            logger.log(Logger.INFO, "log 1", new Throwable());

            // logs some traces that are expected to be delegated to the LogCollector
            String appName = "TestLogManager";
            String procName = "TestDriver";
            logManager.registerAlternateDelegate(delegate);
            logger = new MyLogger(logManager, "LOG_MANAGER", "TEST_LOG_MANAGER2");
            logger.log(Logger.INFO, "log 2", new Throwable());
            logger = new MyLogger(logManager, "LOG_MANAGER", "TEST_LOG_MANAGER3");
            logger.log(Logger.INFO, "log 3", new Throwable());
            logManager.setApplicationName(appName);
            logger = new MyLogger(logManager, "LOG_MANAGER", "TEST_LOG_MANAGER4");
            logger.log(Logger.INFO, "log 4", new Throwable());
            logManager.setProcessName(procName);
            logger = new MyLogger(logManager, "LOG_MANAGER", "TEST_LOG_MANAGER5");
            logger.log(Logger.INFO, "log 5", new Throwable());
            Thread.sleep(2000); // wait logs are treated

            // check the LogCollector has received the logs
            BufferedReader logFile = null;
            try {
                logFile = new BufferedReader(new FileReader(MainLogCollector.OUTPUT_LOG_FILE));
                String line = logFile.readLine();
                assertNotNull(line);
                if (! line.matches(".*LOG_MANAGER.*TEST_LOG_MANAGER2.*log 2.*")) {
                    fail("log 2 not found");
                }
                line = logFile.readLine();
                assertNotNull(line);
                if (! line.matches(".*LOG_MANAGER.*TEST_LOG_MANAGER3.*log 3.*")) {
                    fail("log 3 not found");
                }
                line = logFile.readLine();
                assertNotNull(line);
                if (! line.matches(".*" + appName + ".*LOG_MANAGER.*TEST_LOG_MANAGER4.*log 4.*")) {
                    fail("log 4 not found");
                }
                line = logFile.readLine();
                assertNotNull(line);
                if (! line.matches(".*" + appName + ".*" + procName + ".*LOG_MANAGER.*TEST_LOG_MANAGER5.*log 5.*")) {
                    fail("log 5 not found");
                }
                line = logFile.readLine();
                assertNull(line);
                logFile.close();
            }
            catch (FileNotFoundException fnfe) {
                fail("Unable to open log file");
            }
            catch (IOException ioe) {
                fail("Unable to read log file");
            }

            proc.killProcess();
        }
        catch (java.lang.Exception e) {
            e.printStackTrace();
            fail(e.getMessage());
        }
    }

}
