/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "testorblogging/TestHello_impl.hpp"

// CDMW_TRACE_LEVEL is defined at CDMW configure time to set what type of traces
// will be activated :
// 0:nothing, 1:FATAL, 2:+ERROR, 3:+WARN, 4:+INFO, 5:+DEBUG, 6:+user defined
// In this test we force to 6 to test CDMW_LOG macro with user defined levels

#if defined(CDMW_TRACE_LEVEL)

#if CDMW_TRACE_LEVEL < 6
#undef CDMW_TRACE_LEVEL
#define CDMW_TRACE_LEVEL 6
#endif

#else
#define CDMW_TRACE_LEVEL 6
#endif

#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/LogManager.hpp>

using Cdmw::Logging::Logger;

namespace Cdmw
{

    namespace Logging
    {

        /**
         * Purpose:
         * <p> [Constructor description if necessary]
         * 
         *@param
         *@exception
         */
        TestHello_impl::TestHello_impl(CORBA::ORB_ptr orb)
            throw()
      : m_orb(CORBA::ORB::_duplicate(orb)),
        m_userDefinedLevel(1024)
        {
        }
        
        
        /**
         * Purpose:
         * <p> [ Destructor description if necessary]
         * 
         */
        TestHello_impl::~TestHello_impl()
            throw()
        {
        }
        
        void
        TestHello_impl::hello()
            throw(CORBA::SystemException)
        {
            Logger* logger = Cdmw::Logging::Logger::Get_Logger("TEST_LOGGING");

        // activate userDefinedLevel
        LogManager::Instance()->get_filter_manager()->activate_level
          ("TEST_LOGGING", m_userDefinedLevel);

            CDMW_LOG(logger, m_userDefinedLevel, "HELLO");

        // deactivate userDefinedLevel in "TEST_LOGGING" domain
        LogManager::Instance()->get_filter_manager()->deactivate_level
          ("TEST_LOGGING", m_userDefinedLevel);
        }
        
        void
        TestHello_impl::shutdown()
            throw(CORBA::SystemException)
        {
            Logger* loggerCmp =
          Cdmw::Logging::Logger::Get_Logger("Component1", "TEST_CMP_LOGGING");
        // activate userDefinedLevel in "TEST_CMP_LOGGING" domain of Component1
        LogManager::Instance()->get_filter_manager()->activate_level
          ("Component1", "TEST_CMP_LOGGING", m_userDefinedLevel);

            CDMW_LOG(loggerCmp, m_userDefinedLevel, "CMPTEST");

            Logger* logger = Cdmw::Logging::Logger::Get_Logger("TEST_LOGGING");
            CDMW_LOG(logger, m_userDefinedLevel, "SHUTDOWN");
        
            m_orb->shutdown();
        }



    } // End namespace Logging
} // End namespace Cdmw

