/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <fstream>
#include <sstream>
#include <string>
#include <vector>

#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"

#include "Foundation/common/System.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "testorblogging/TestOrbLoggingDriver.hpp"
#include "testorblogging/TestHello_impl.hpp"


namespace Cdmw
{

    namespace Logging
    {

        /**
        *Implementation notes: [if no pertinent write none ]
        *<p>
        *Portability issues: [if no pertinent write none ]
        *<p>
        */

        CPPUNIT_TEST_SUITE_REGISTRATION( TestOrbLoggingDriver );
        
        TestOrbLoggingDriver::TestOrbLoggingDriver()
        {
            m_orb =  CORBA::ORB::_duplicate( Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());
            m_process_name = Cdmw::TestUtils::CORBATestManager::instance()->get_property ("name");
        }

        TestOrbLoggingDriver::~TestOrbLoggingDriver()
        throw()
        {
        }

        void
        TestOrbLoggingDriver::do_tests()
        {
            try
            {
//             set_nbOfRequestedTestOK (3);
                int timescale = Cdmw::TestUtils::get_timescale();
    
                std::string logFileName("test.log");

                if ( Cdmw::TestUtils::CORBATestManager::instance()->get_property ("client_only") == "no" )
                  {
                    TEST_INFO("Start the server process");

                    std::string processArgs(" --server --CdmwLogLevel=DEBUG+ORB_TRACE --CdmwLogFile=");
                    processArgs += logFileName;
                    processArgs += " -ORBDebugLevel 10";

                    // --CdmwLogLevel=16=DEBUG will handle TAO DEBUG traces (at -ORBDebugLevel 10)
                    OsSupport::OS::create_process( m_process_name, processArgs);
                  }

                if ( Cdmw::TestUtils::CORBATestManager::instance()->get_property ("server_only") == "no" )
                  {
                    OsSupport::OS::sleep(timescale*5000);

                    TEST_INFO( "Getting back the IOR from the file for the multi-thread POA" );
                    std::ifstream iorFile( "TestHello.ior" );
    
                    std::string iorTestHello;
                    iorFile >> iorTestHello;
                    iorFile.close();
    
                    CORBA::Object_var objHello = m_orb->string_to_object( iorTestHello.c_str() );
                    Cdmw::TestHello_var testHello = Cdmw::TestHello::_narrow( objHello.in() );
                
                    CPPUNIT_ASSERT( ! CORBA::is_nil(testHello.in()) );
    
                    TEST_INFO("Call an operation on the IOR");
                    testHello->hello();
                    OsSupport::OS::sleep(timescale*2000);
                
                    TEST_INFO("Shutdown the server...");
                    testHello->shutdown();
                    OsSupport::OS::sleep(timescale*2000);
                
    
                    TEST_INFO("check log file to find HELLO trace");
                    std::ifstream ifs1(logFileName.c_str());
                    std::string s;
                    bool string_found = false;
                    if (ifs1.good())
                      {
                        while(getline(ifs1, s))    
                          {
                            if (s.find("HELLO") != s.npos)
                              {
                                TEST_INFO(s);
                                string_found=true;
                                break;
                              }
                          }
                        ifs1.close();
                      }
                    CPPUNIT_ASSERT(string_found);

                    // Test user defined level has been deactivated after hello call
                    // and before shutdown call
                    // Test (same) user defined level in "Component1" is still activated
                    TEST_INFO("check log file to find CMPTEST trace");
                    std::ifstream ifs1a(logFileName.c_str());
                    string_found = false;
                    if (ifs1a.good())
                      {
                        while(getline(ifs1a, s))    
                          {
                            if (s.find("CMPTEST") != s.npos)
                              {
                                TEST_INFO(s);
                                string_found=true;
                                break;
                              }
                          }
                        ifs1a.close();
                      }
                    CPPUNIT_ASSERT(string_found);

                    TEST_INFO("check log file to find SHUTDOWN trace");
                    std::ifstream ifs1b(logFileName.c_str());
                    string_found = false;
                    if (ifs1b.good())
                      {
                        while(getline(ifs1b, s))    
                          {
                            if (s.find("SHUTDOWN") != s.npos)
                              {
                                TEST_INFO(s);
                                string_found=true;
                                break;
                              }
                          }
                        ifs1b.close();
                      }
                    // but INFO level is deactivated now
                    CPPUNIT_ASSERT(!string_found);


                    if ( Cdmw::TestUtils::CORBATestManager::instance()->get_property ("no_orb_log_check") == "no" )
                      {
                        TEST_INFO("check log file to find TAO trace");
                        std::ifstream ifs2(logFileName.c_str());
                        string_found = false;
                        if (ifs2.good())
                          {
                            while(getline(ifs2, s))    
                              {
                                if (s.find("TAO (") != s.npos)
                                  {
                                    TEST_INFO(s);
                                    string_found=true;
                                    break;
                                  }
                              }
                            ifs2.close();
                          }
                        CPPUNIT_ASSERT(string_found);
                      }
                    OsSupport::OS::unlink(logFileName);
                  }
            }
            catch (const CORBA::Exception& e)
            {
                TEST_INFO("CORBA Exception raised: " << e);
                CPPUNIT_ASSERT(false);
            }
            catch (const std::exception& e)
            {
                TEST_INFO("std::exception raised: " << e.what());
                CPPUNIT_ASSERT(false);
            }
            catch (...)
            {
                TEST_INFO("unexpected exception raised!");
                CPPUNIT_ASSERT(false);
            }
        }

    } // End namespace Logging
} // End namespace Cdmw

