/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


/// level is defined at CDMW configure time to set what type of traces
// will be activated :
// 0:nothing, 1:FATAL, 2:+ERROR, 3:+WARN, 4:+INFO, 5:+DEBUG, 6:+user defined
/// In this test we force to 6 to test CDMW_LOG macro with user defined levels
package cdmw.logging.test.testorblogging;
import cdmw.logging.*;
import cdmw.testutils.Testable;
import cdmw.ossupport.OS;
import cdmw.ossupport.Mutex;
import org.omg.CORBA.ORB;


public class TestHello_impl extends Cdmw.TestHelloPOA
{
    private int m_userDefinedLevel;
    private LogLevel m_logger = new LogLevel();
    private org.omg.CORBA.ORB m_orb;
    private LogLevel Log = new LogLevel();
    private LevelType type = new LevelType();
    public  int CDMW_TRACE_LEVEL = 6;

    TestHello_impl(org.omg.CORBA.ORB orb) {
        m_orb = orb;
        m_userDefinedLevel = 1024;
    }
        
    public void hello() {
        Logger logger = Logger.Get_Logger("TEST_LOGGING");

        // activate userDefinedLevel
        LogManager.Instance().getFilterManager().activateLevel ("TEST_LOGGING", 1024);

        CDMW_LOG(logger, 1024, "HELLO");

        // deactivate userDefinedLevel in "TEST_LOGGING" domain
        LogManager.Instance().getFilterManager().deactivateLevel ("TEST_LOGGING", 1024);
    }
        
    public void shutdown() {
        Logger loggerCmp = Logger.Get_Logger("Component1", "TEST_CMP_LOGGING");
        // activate userDefinedLevel in "TEST_CMP_LOGGING" domain of Component1
        LogManager.Instance().getFilterManager().activateLevel ("Component1", "TEST_CMP_LOGGING", 1024);

        CDMW_LOG(loggerCmp, 1024, "CMPTEST");

        Logger logger = Logger.Get_Logger("TEST_LOGGING");
        CDMW_LOG(logger, m_userDefinedLevel, "SHUTDOWN");
            
        m_orb.shutdown(false);
    }

    public void CDMW_LOG(Logger log, int level, String msg) {
        if (CDMW_TRACE_LEVEL >= 6) {
            type.setType(level);
            Logger.CDMW_LOG(log, type, msg, new Throwable());
        }
    } 
 
    public void CDMW_FATAL(Logger log, String msg) {
        if (CDMW_TRACE_LEVEL >= 1) {
            Logger.CDMW_FATAL(log, msg, new Throwable());
        }
    }

    public void CDMW_ERROR(Logger log, String msg) {
        if (CDMW_TRACE_LEVEL >= 2) {
            Logger.CDMW_ERROR(log, msg, new Throwable());
        }
    }

    public void CDMW_WARN(Logger log, String msg) {
        if (CDMW_TRACE_LEVEL >= 3) {
            Logger.CDMW_WARN(log, msg, new Throwable());
        }
    }

    public void CDMW_INFO(Logger log, String msg) {
        if (CDMW_TRACE_LEVEL >= 4) {
            Logger.CDMW_INFO(log, msg, new Throwable());
        }
    }

    public void CDMW_DEBUG(Logger log, String msg) {
        if (CDMW_TRACE_LEVEL >= 5) {
            Logger.CDMW_DEBUG(log, msg, new Throwable());
        }
    }
}
