/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.logging.test.testorblogging;

import java.util.*;
import java.io.*;
import cdmw.logging.*;
import cdmw.ossupport.*;
import cdmw.orbsupport.*;

import cdmw.testutils.CORBATestManager;
import cdmw.testutils.CORBATestManager.InvalidObject;
import cdmw.common.*;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.fail;

import org.omg.CORBA.*;

public class TestOrbLoggingDriver {
    private static org.omg.CORBA.ORB orb;
    private static String className;
    private static int timescale;

    @BeforeClass
    public static void initTests() throws InvalidObject {
        CORBATestManager testManager = CORBATestManager.instance();
        orb = testManager.getORB();
        className = testManager.getProperty("TEST_ORBLOGGING_CLASS_NAME");
        timescale = CORBATestManager.getTimescale();
    }

    @AfterClass
    public static void finiTests() {
        orb = null;
        className = null;
    }

    @Test
    public void doTests() throws InterruptedException, IOException, FileNotFoundException {

        if (System.getProperty("client_only") == "no") {
            System.out.println("Start the server process");
            try {
                OS.createJavaProcess(className
                    + " --server --CdmwLogLevel=ALL --CdmwLogFile=test.log --no-ORBlog-check");
                System.out.println("Started the server process");
            } catch (IOException ioe) {
                fail("Exception during the process creation:" + ioe.getMessage());
            }
        }

        if (System.getProperty("server_only") == "no") {
            cdmw.ossupport.OS.sleep(timescale * 5000);

            System.out.println("Getting back the IOR from the file for the multi-thread POA");

            FileInputStream iorFile = new FileInputStream("TestHello.ior");
            String iorTestHello;
            int fLength = iorFile.available();
            byte byteRep[] = new byte[fLength];
            iorFile.read(byteRep);
            iorTestHello = new String(byteRep);
            iorFile.close();

            org.omg.CORBA.Object objHello = orb.string_to_object(iorTestHello);
            Cdmw.TestHello testHello = Cdmw.TestHelloHelper.narrow(objHello);

            assertFalse(testHello == null);

            System.out.println("Call an operation on the IOR");
            testHello.hello();
            cdmw.ossupport.OS.sleep(timescale * 2000);

            System.out.println("Shutdown the server...");
            testHello.shutdown();
            cdmw.ossupport.OS.sleep(timescale * 2000);

            System.out.println("check log file to find HELLO trace");

            BufferedReader ifs1 = new BufferedReader(new FileReader("test.log"));

            boolean string_found = false;
            if (ifs1.ready()) {
                String s;

                System.out.println("Reading file test.log");
                while (ifs1.ready()) {
                    s = ifs1.readLine();
                    if (s.indexOf("HELLO") != -1) {
                        System.out.println(s);
                        string_found = true;
                        break;
                    }
                }
                ifs1.close();
            }
            assertTrue(string_found == true);

            // Test user defined level has been deactivated after hello call
            // and before shutdown call
            // Test (same) user defined level in "Component1" is still activated
            System.out.println("check log file to find CMPTEST trace");
            BufferedReader ifs1a = new BufferedReader(new FileReader("test.log"));
            string_found = false;

            if (ifs1a.ready()) {
                String s;
                while (ifs1a.ready()) {
                    s = ifs1a.readLine();
                    if (s.indexOf("CMPTEST") != -1) {
                        System.out.println(s);
                        string_found = true;
                        break;
                    }
                }
                ifs1a.close();
            }
            assertTrue(string_found == true);

            System.out.println("check log file to find SHUTDOWN trace");
            BufferedReader ifs1b = new BufferedReader(new FileReader("test.log"));
            string_found = false;

            if (ifs1b.ready()) {
                String s;
                while (ifs1b.ready()) {
                    s = ifs1b.readLine();
                    if (s.indexOf("SHUTDOWN") != -1) {
                        System.out.println(s);
                        string_found = true;
                        break;
                    }
                }
                ifs1b.close();
            }
            // but INFO level is deactivated now
            assertTrue(string_found == false);

            if (System.getProperty("no_orb_log_check") == "no") {
                System.out.println("check log file to find jacorb trace");
                BufferedReader ifs2 = new BufferedReader(new FileReader("test.log"));
                string_found = false;
                if (ifs2.ready()) {
                    String s;
                    while (ifs2.ready()) {
                        s = ifs2.readLine();
                        if (s.indexOf("jacorb") != -1) {
                            System.out.println(s);
                            string_found = true;
                            break;
                        }
                    }
                    ifs2.close();
                }
                assertTrue(string_found == true);
            }
            cdmw.ossupport.OS.unlink("test.log");
        }
    }
}
