/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_ORBSUPPORT_CORBA_EXCEPTION_HELPER_HPP
#define INCL_ORBSUPPORT_CORBA_EXCEPTION_HELPER_HPP

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include "tao/PI/PIForwardRequestA.h"
#include "tao/AnyTypeCode/SystemExceptionA.h"
/**
* Root namespace for CDMW runtime.
*/

namespace Cdmw
{
   
   /**
    * Namespace for CDMW OrbSupport types and data.
    */
   
   namespace OrbSupport
   {
     /**
      * Helper class to provide marshalling and unmarshalling of exceptions
      * in Any objects.
      */
      class CORBAExceptionHelper
      {
      public:
          /**
           * Reads a SystemException marshalled into the specified Any.
           *
           * @param any Any object containing the exception
           * @return the exception extracted from the Any
           */
          static CORBA::SystemException * 
          Extract_SystemException(const CORBA::Any & any);

          /**
           * Reads a generic Exception marshalled into the specified Any.
           *
           * @param any Any object containing the exception
           * @return the exception extracted from the Any
           */
          template <typename T>
          static T* Extract_Exception(const CORBA::Any & any)
          { 
              T* result = 0;
              const T * ex;
              if (any >>= ex)
                  result = new T(*ex);
              return result;
          }

          /**
           * Inserts a SystemException into the input Any.
           *
           * @param any object that will contain the SystemException
           * @param ex the exception to insert
           */
          static void 
          Insert_SystemException(CORBA::Any & any,
                                 const CORBA::SystemException & ex);

          /**
           * Inserts a generic Exception into the input Any.
           *
           * @param any object that will contain the Exception
           * @param ex the exception to insert
           */
          template <typename T>
          static void Insert_Exception(CORBA::Any & any, const T & ex)
          {
              any <<= ex;
          }
            
      }; // end of CORBAExceptionHelper

    } // End namespace OrbSupport

} // End namespace Cdmw

#endif // INCL_ORBSUPPORT_CORBA_EXCEPTION_HELPER_HPP

