/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_ORBSUPPORT_TAO_SERVER_REQUEST_INTERCEPTOR_HPP
#define INCL_ORBSUPPORT_TAO_SERVER_REQUEST_INTERCEPTOR_HPP

// Cdmw Files
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/orbsupport/PortableInterceptor.hpp>

#if TAO_HAS_EXTENDED_FT_INTERCEPTORS == 1
#include "tao/AnyTypeCode/Any_Impl.h"
#include "tao/AnyTypeCode/DynamicC.h"
#include "tao/AnyTypeCode/PortableInterceptorA.h"
#endif /*TAO_HAS_EXTENDED_FT_INTERCEPTORS*/


namespace Cdmw
{
    
namespace OrbSupport
{


class TaoServerRequestInterceptor : public PortableInterceptor::ServerRequestInterceptor,
                                    public Cdmw::OrbSupport::RefCountLocalObject
{
   public:

      TaoServerRequestInterceptor()
      {
#if TAO_HAS_EXTENDED_FT_INTERCEPTORS == 1
         m_bUseFTInterceptionPoint = true;
#else
         m_bUseFTInterceptionPoint = false;
#endif /*TAO_HAS_EXTENDED_FT_INTERCEPTORS*/         
      }


#if TAO_HAS_EXTENDED_FT_INTERCEPTORS == 1
      /**
       * This is a TAO-specific operation to enable returning previous logged replies.
       * NOTA: Only the primary actually returns a previous reply!
       */  
      virtual void
      tao_ft_interception_point(PortableInterceptor::ServerRequestInfo_ptr ri,
                                CORBA::OctetSeq_out ocs)
         throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
         {
            this->cdmw_ft_interception_point(ri, ocs);
         }
#endif /*TAO_HAS_EXTENDED_FT_INTERCEPTORS*/

      virtual void
      cdmw_ft_interception_point(PortableInterceptor::ServerRequestInfo_ptr ri,
                                CORBA::OctetSeq_out ocs)
         throw (CORBA::SystemException, PortableInterceptor::ForwardRequest) = 0;

   protected:
      bool m_bUseFTInterceptionPoint;
      
   private:
      TaoServerRequestInterceptor(const TaoServerRequestInterceptor&);
      TaoServerRequestInterceptor& operator=(const TaoServerRequestInterceptor&);
      
  
};

} // End of namespace OrbSupport

} // End of namespace Cdmw

#endif // INCL_ORBSUPPORT_TAO_SERVER_REQUEST_INTERCEPTOR_HPP

