/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/orbsupport/CORBAExceptionHelper.hpp"
#include "Foundation/common/Assert.hpp"
#include <iostream>
#include "tao/AnyTypeCode/Any_SystemException.h"
#include "tao/AnyTypeCode/ExceptionA.h"

#define CHECK_SYSTEM_EXCEPTION(name,repid) \
   (strcmp(id,repid) == 0)   \
    do {                     \
       const CORBA::name * ex;                  \
       if (TAO::Any_SystemException::extract (any, \
                                              CORBA::name ::_tao_any_destructor, \
                                              CORBA::_tc_ ## name, \
                                              (const CORBA::SystemException *&) ex, \
                                              &CORBA::name ::_tao_create)) \
          result = new CORBA::name (*ex); \
    } while(0)


#define INSERT_SYSTEM_EXCEPTION(name,repid) \
   (strcmp(id,repid) == 0)   \
    do {                     \
       CORBA::name * ex__ = CORBA::name ::_downcast( (CORBA::Exception*) &ex ); \
       CDMW_ASSERT(ex__ != 0); \
       any <<= *ex__;          \
    } while(0)


/**
* Root namespace for CDMW runtime.
*/

namespace Cdmw
{

   /**
    * Namespace for CDMW OrbSupport types and data.
    */
   
   namespace OrbSupport
   {
      
      
      CORBA::SystemException * 
      CORBAExceptionHelper::Extract_SystemException(const CORBA::Any & any)
      {
         CORBA::SystemException * result = 0;
         CORBA::TypeCode_var tc = any.type ();
         if (tc->kind() == CORBA::tk_except) {
            const char* id = tc->id();
            if CHECK_SYSTEM_EXCEPTION(TRANSIENT,"IDL:omg.org/CORBA/TRANSIENT:1.0");
            else if CHECK_SYSTEM_EXCEPTION(BAD_CONTEXT,"IDL:omg.org/CORBA/BAD_CONTEXT:1.0");
            else if CHECK_SYSTEM_EXCEPTION(NO_RESPONSE,"IDL:omg.org/CORBA/NO_RESPONSE:1.0");
            else if CHECK_SYSTEM_EXCEPTION(OBJ_ADAPTER,"IDL:omg.org/CORBA/OBJ_ADAPTER:1.0");
            else if CHECK_SYSTEM_EXCEPTION(BAD_PARAM,"IDL:omg.org/CORBA/BAD_PARAM:1.0");
            else if CHECK_SYSTEM_EXCEPTION(BAD_INV_ORDER,"IDL:omg.org/CORBA/BAD_INV_ORDER:1.0");
            else if CHECK_SYSTEM_EXCEPTION(COMM_FAILURE,"IDL:omg.org/CORBA/COMM_FAILURE:1.0");
            else if CHECK_SYSTEM_EXCEPTION(INTERNAL,"IDL:omg.org/CORBA/INTERNAL:1.0");
            else if CHECK_SYSTEM_EXCEPTION(OBJECT_NOT_EXIST,"IDL:omg.org/CORBA/OBJECT_NOT_EXIST:1.0");
            else if CHECK_SYSTEM_EXCEPTION(NO_MEMORY,"IDL:omg.org/CORBA/NO_MEMORY:1.0");
            else if CHECK_SYSTEM_EXCEPTION(IMP_LIMIT,"IDL:omg.org/CORBA/IMP_LIMIT:1.0");
            else if CHECK_SYSTEM_EXCEPTION(INV_OBJREF,"IDL:omg.org/CORBA/INV_OBJREF:1.0");
            else if CHECK_SYSTEM_EXCEPTION(UNKNOWN,"IDL:omg.org/CORBA/UNKNOWN:1.0");
            else if CHECK_SYSTEM_EXCEPTION(NO_PERMISSION,"IDL:omg.org/CORBA/NO_PERMISSION:1.0");
            else if CHECK_SYSTEM_EXCEPTION(MARSHAL,"IDL:omg.org/CORBA/MARSHAL:1.0");
            else if CHECK_SYSTEM_EXCEPTION(INITIALIZE,"IDL:omg.org/CORBA/INITIALIZE:1.0");
            else if CHECK_SYSTEM_EXCEPTION(NO_IMPLEMENT,"IDL:omg.org/CORBA/NO_IMPLEMENT:1.0");
            else if CHECK_SYSTEM_EXCEPTION(BAD_TYPECODE,"IDL:omg.org/CORBA/BAD_TYPECODE:1.0");
            else if CHECK_SYSTEM_EXCEPTION(BAD_OPERATION,"IDL:omg.org/CORBA/BAD_OPERATION:1.0");
            else if CHECK_SYSTEM_EXCEPTION(NO_RESOURCES,"IDL:omg.org/CORBA/NO_RESOURCES:1.0");
            else if CHECK_SYSTEM_EXCEPTION(PERSIST_STORE,"IDL:omg.org/CORBA/PERSIST_STORE:1.0");
            else if CHECK_SYSTEM_EXCEPTION(FREE_MEM,"IDL:omg.org/CORBA/FREE_MEM:1.0");
            else if CHECK_SYSTEM_EXCEPTION(INV_IDENT,"IDL:omg.org/CORBA/INV_IDENT:1.0");
            else if CHECK_SYSTEM_EXCEPTION(INV_FLAG,"IDL:omg.org/CORBA/INV_FLAG:1.0");
            else if CHECK_SYSTEM_EXCEPTION(INTF_REPOS,"IDL:omg.org/CORBA/INTF_REPOS:1.0");
            else if CHECK_SYSTEM_EXCEPTION(DATA_CONVERSION,"IDL:omg.org/CORBA/DATA_CONVERSION:1.0");
            else if CHECK_SYSTEM_EXCEPTION(TRANSACTION_REQUIRED,"IDL:omg.org/CORBA/TRANSACTION_REQUIRED:1.0");
            else if CHECK_SYSTEM_EXCEPTION(TRANSACTION_ROLLEDBACK,"IDL:omg.org/CORBA/TRANSACTION_ROLLEDBACK:1.0");
            else if CHECK_SYSTEM_EXCEPTION(INVALID_TRANSACTION,"IDL:omg.org/CORBA/INVALID_TRANSACTION:1.0");
            else if CHECK_SYSTEM_EXCEPTION(INV_POLICY,"IDL:omg.org/CORBA/INV_POLICY:1.0");
            else if CHECK_SYSTEM_EXCEPTION(CODESET_INCOMPATIBLE,"IDL:omg.org/CORBA/CODESET_INCOMPATIBLE:1.0");
            else {// Not supported CORBA::SystemException
               std::cerr << "Not supported CORBA::SystemException!" << std::endl;
            }
         }
         return result;
      }

      
      void
      CORBAExceptionHelper::Insert_SystemException(CORBA::Any & any, const CORBA::SystemException& ex)
      {
         const char* id = ex._rep_id();

         if INSERT_SYSTEM_EXCEPTION(TRANSIENT,"IDL:omg.org/CORBA/TRANSIENT:1.0");
         else if INSERT_SYSTEM_EXCEPTION(BAD_CONTEXT,"IDL:omg.org/CORBA/BAD_CONTEXT:1.0");
         else if INSERT_SYSTEM_EXCEPTION(NO_RESPONSE,"IDL:omg.org/CORBA/NO_RESPONSE:1.0");
         else if INSERT_SYSTEM_EXCEPTION(OBJ_ADAPTER,"IDL:omg.org/CORBA/OBJ_ADAPTER:1.0");
         else if INSERT_SYSTEM_EXCEPTION(BAD_PARAM,"IDL:omg.org/CORBA/BAD_PARAM:1.0");
         else if INSERT_SYSTEM_EXCEPTION(BAD_INV_ORDER,"IDL:omg.org/CORBA/BAD_INV_ORDER:1.0");
         else if INSERT_SYSTEM_EXCEPTION(COMM_FAILURE,"IDL:omg.org/CORBA/COMM_FAILURE:1.0");
         else if INSERT_SYSTEM_EXCEPTION(INTERNAL,"IDL:omg.org/CORBA/INTERNAL:1.0");
         else if INSERT_SYSTEM_EXCEPTION(OBJECT_NOT_EXIST,"IDL:omg.org/CORBA/OBJECT_NOT_EXIST:1.0");
         else if INSERT_SYSTEM_EXCEPTION(NO_MEMORY,"IDL:omg.org/CORBA/NO_MEMORY:1.0");
         else if INSERT_SYSTEM_EXCEPTION(IMP_LIMIT,"IDL:omg.org/CORBA/IMP_LIMIT:1.0");
         else if INSERT_SYSTEM_EXCEPTION(INV_OBJREF,"IDL:omg.org/CORBA/INV_OBJREF:1.0");
         else if INSERT_SYSTEM_EXCEPTION(UNKNOWN,"IDL:omg.org/CORBA/UNKNOWN:1.0");
         else if INSERT_SYSTEM_EXCEPTION(NO_PERMISSION,"IDL:omg.org/CORBA/NO_PERMISSION:1.0");
         else if INSERT_SYSTEM_EXCEPTION(MARSHAL,"IDL:omg.org/CORBA/MARSHAL:1.0");
         else if INSERT_SYSTEM_EXCEPTION(INITIALIZE,"IDL:omg.org/CORBA/INITIALIZE:1.0");
         else if INSERT_SYSTEM_EXCEPTION(NO_IMPLEMENT,"IDL:omg.org/CORBA/NO_IMPLEMENT:1.0");
         else if INSERT_SYSTEM_EXCEPTION(BAD_TYPECODE,"IDL:omg.org/CORBA/BAD_TYPECODE:1.0");
         else if INSERT_SYSTEM_EXCEPTION(BAD_OPERATION,"IDL:omg.org/CORBA/BAD_OPERATION:1.0");
         else if INSERT_SYSTEM_EXCEPTION(NO_RESOURCES,"IDL:omg.org/CORBA/NO_RESOURCES:1.0");
         else if INSERT_SYSTEM_EXCEPTION(PERSIST_STORE,"IDL:omg.org/CORBA/PERSIST_STORE:1.0");
         else if INSERT_SYSTEM_EXCEPTION(FREE_MEM,"IDL:omg.org/CORBA/FREE_MEM:1.0");
         else if INSERT_SYSTEM_EXCEPTION(INV_IDENT,"IDL:omg.org/CORBA/INV_IDENT:1.0");
         else if INSERT_SYSTEM_EXCEPTION(INV_FLAG,"IDL:omg.org/CORBA/INV_FLAG:1.0");
         else if INSERT_SYSTEM_EXCEPTION(INTF_REPOS,"IDL:omg.org/CORBA/INTF_REPOS:1.0");
         else if INSERT_SYSTEM_EXCEPTION(DATA_CONVERSION,"IDL:omg.org/CORBA/DATA_CONVERSION:1.0");
         else if INSERT_SYSTEM_EXCEPTION(TRANSACTION_REQUIRED,"IDL:omg.org/CORBA/TRANSACTION_REQUIRED:1.0");
         else if INSERT_SYSTEM_EXCEPTION(TRANSACTION_ROLLEDBACK,"IDL:omg.org/CORBA/TRANSACTION_ROLLEDBACK:1.0");
         else if INSERT_SYSTEM_EXCEPTION(INVALID_TRANSACTION,"IDL:omg.org/CORBA/INVALID_TRANSACTION:1.0");
         else if INSERT_SYSTEM_EXCEPTION(INV_POLICY,"IDL:omg.org/CORBA/INV_POLICY:1.0");
         else if INSERT_SYSTEM_EXCEPTION(CODESET_INCOMPATIBLE,"IDL:omg.org/CORBA/CODESET_INCOMPATIBLE:1.0");
         else {// Not supported CORBA::SystemException
            std::cerr << "Not supported CORBA::SystemException!" << std::endl;
            any <<= ex;
         }
      }


   } // End namespace OrbSupport

} // End namespace Cdmw


