/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "orbsupport/OrbTaoImpl.hpp"

#if CDMW_ORB_VDR == tao


#include <sstream>
#include <ace/Service_Config.h>

#include <tao/TAO_Internal.h>
#include <tao/Client_Strategy_Factory.h>
#include <tao/Server_Strategy_Factory.h>
#include <tao/IORTable/IORTable.h>
#include "tao/ORB_Core.h"
#include <tao/Stub.h>
#include "tao/Messaging/Messaging.h"
#include "Foundation/common/Assert.hpp"
#include "Foundation/common/String.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include <Foundation/ossupport/SystemProperties.hpp>

#include "orbsupport/AceTaoLogger.hpp"
#include "orbsupport/OrbSpecificProperties.hpp"

namespace Cdmw
{

    namespace OrbSupport
    {


        /**
        *Implementation notes: 
        *<p>
        *Portability issues: none 
        *<p>
        */



        //RTCORBA::RTORB_var OrbTaoImpl::rt_orb(RTCORBA::RTORB::_nil());
        //RTCORBA::Current_var OrbTaoImpl::current(RTCORBA::Current::_nil());

        std::map<CORBA::ORB_ptr, OrbTaoThreadPool*> OrbTaoImpl::OrbToThreadPool;

//        void OrbRunTao::run() throw();

        //
        // Initialises the Tao ORB
        //
        CORBA::ORB_ptr
        OrbTaoImpl::ORB_init( int& argc, char** argv,
                              const StrategyList& strategyList,
                              std::size_t root_poa_port,
                              std::size_t cdmw_root_poa_port,
                              const char* orb_identifier )
        throw( CORBA::SystemException )
        {



            // Initialise service repository with signal set to SIGINT instead of default SIGHUP
            // to allow the process terminates when its father terminates
            ACE_Service_Config service_config (1,
                                               ACE_Service_Config::MAX_SERVICES,
                                               SIGUSR1);

         // Check is ACE_Log_Msg::msg_backend is already set
         // (in case of several calls to ORB_init)
         // If not set, initialize Logging and create AceTaoLogger
         if (ACE_Log_Msg::msg_backend() == 0) {
            // Init CARDAMOM Logging library (log manager) first for use as a backed for ACE Logging
            Cdmw::Logging::LogManager::Init(argc, argv);
            (void)ACE_Log_Msg::msg_backend (new AceTaoLogger());
            ACE_Log_Msg * ace = ACE_Log_Msg::instance ();
            ace->set_flags (ACE_Log_Msg::CUSTOM);
         }

            CORBA::ORB_var theOrb = CORBA::ORB::_nil();
            size_t nbThreads = 0;

            try
            {
                // At this time is ORB SHALL be multi-threaded

                if ( !strategyList.is_OrbThreaded() )
                {
                    throw CORBA::BAD_PARAM(
                        BAD_PARAMMissingORBThreadingStrategy,
                        CORBA::COMPLETED_NO );
                }

                // Initialisation of the 3 catogories of definitions to put
                // in default_svc_conf_entries 
                std::string resourceFactoryDefs = "";
                std::string serverStrategyFactoryDefs = "";
                std::string clientStrategyFactoryDefs = "";

                std::string ORBClientConnectionHandlerPName(CDMW_ORB_PROPERTY_NAME_PREFIX);
                std::string orbId(orb_identifier);
                if (!orbId.empty())
                   ORBClientConnectionHandlerPName += "." + orbId;
                ORBClientConnectionHandlerPName += "." + std::string(TAO_ORB_CLIENT_CONNECTION_HANDLER_PROPERTY_NAME);
        
                std::string ORBClientConnectionHandler =
                   OsSupport::SystemProperties::Get_Property(ORBClientConnectionHandlerPName,
                                                            "RW");
                if (strategyList.is_multicast())
                {
                  resourceFactoryDefs = "dynamic UIPMC_Factory Service_Object * TAO_PortableGroup:_make_TAO_UIPMC_Protocol_Factory() \"\"";
                  resourceFactoryDefs += "\nstatic Resource_Factory \"";
                  if ((ORBClientConnectionHandler == "RW" ) && strategyList.is_PoaThreadPool (nbThreads))
                    resourceFactoryDefs += "-ORBFlushingStrategy blocking ";
                  resourceFactoryDefs += "-ORBProtocolFactory IIOP_Factory -ORBProtocolFactory UIPMC_Factory\"";
                  
                  // REM1: i am not sure in which category PortableGroup_Loader must be
                  // but in fact the 3 categories are processed the same way
                  // inside ORB_init : via a ACE_Service_Config::process_directive
                  // REM2: we could also replace the 2 "dynamic.."  by a process_directive
                  // because these initialisations can be done immediatly. However,
                  // "static Resource_Factory" line can be processed only inside ORB_init
                  serverStrategyFactoryDefs = "dynamic PortableGroup_Loader Service_Object * TAO_PortableGroup:_make_TAO_PortableGroup_Loader() \"\"";
//                } else if ((ORBClientConnectionHandler == "RW" )  && strategyList.is_PoaThreadPool (nbThreads)) {
                } else if (ORBClientConnectionHandler == "RW" ) {
                  resourceFactoryDefs = "static Resource_Factory \"-ORBFlushingStrategy blocking\"";
                }


                // Do we use a thread pool for the POAs ?
                if (strategyList.is_PoaThreadPool (nbThreads))
                {
                    serverStrategyFactoryDefs = serverStrategyFactoryDefs +
                        "\nstatic Server_Strategy_Factory \""
                        "-ORBConcurrency                  reactive "
                        "-ORBPOALock                      thread "
                        "-ORBAllowReactivationOfSystemids 1\"";
/*
                    clientStrategyFactoryDefs = // clientStrategyFactoryDefs + "\n"
                      "static Client_Strategy_Factory \""
                      "-ORBProfileLock                  thread ";
                    
                    if ( ORBClientConnectionHandler == "RW" )
                      clientStrategyFactoryDefs += " -ORBClientConnectionHandler       RW "
                        "-ORBTransportMuxStrategy EXCLUSIVE -ORBConnectStrategy blocked -ORBConnectionHandlerCleanup 1 \"";
                    else
                      clientStrategyFactoryDefs += " -ORBClientConnectionHandler       MT\"";
*/
                }
                else if (strategyList.is_PoaThreadPerRequest ())
                {
                    serverStrategyFactoryDefs = serverStrategyFactoryDefs +
                        "\nstatic Server_Strategy_Factory \""
                        "-ORBConcurrency                  reactive "
                        "-ORBPOALock                      thread "
                        "-ORBAllowReactivationOfSystemids 1\"";
/*
                    clientStrategyFactoryDefs = // clientStrategyFactoryDefs + "\n"
                        "static Client_Strategy_Factory \"" 
                        "-ORBProfileLock                  thread "
                        "-ORBClientConnectionHandler      MT\"";*/
                }
                else if (strategyList.is_PoaThreadPerConnection ())
                {
                    serverStrategyFactoryDefs = serverStrategyFactoryDefs +
                        "\nstatic Server_Strategy_Factory \""
                        "-ORBConcurrency                  thread-per-connection "
                        "-ORBPOALock                      thread "
                        "-ORBAllowReactivationOfSystemids 1\"";
/*
                    clientStrategyFactoryDefs = // clientStrategyFactoryDefs + "\n"
                        "static Client_Strategy_Factory \""
                        "-ORBProfileLock                  thread "
                        "-ORBClientConnectionHandler      MT\"";*/
                }


                clientStrategyFactoryDefs = // clientStrategyFactoryDefs + "\n"
                      "static Client_Strategy_Factory \""
                      "-ORBProfileLock                  thread ";
                    
                if ( ORBClientConnectionHandler == "RW" )
                      clientStrategyFactoryDefs += " -ORBClientConnectionHandler       RW "
                        "-ORBTransportMuxStrategy EXCLUSIVE -ORBConnectStrategy blocked -ORBConnectionHandlerCleanup 1 \"";
                else
                      clientStrategyFactoryDefs += " -ORBClientConnectionHandler       MT\"";
                // This call set definitions that will be processed only inside ORB_init
                // If this method has been called previously, the definitions are overidden
                TAO::ORB::default_svc_conf_entries(
                    resourceFactoryDefs.c_str(),
                    serverStrategyFactoryDefs.c_str(),
                    clientStrategyFactoryDefs.c_str() );


                // We must ensure to be listening on a host and a port known to the client
                std::string str_opt( "" );

                char** argv_tmp = NULL;

                int argc_tmp = argc;

                for (int i = 0; i < argc; i++)
                    str_opt += std::string( argv[i] ) + " ";

                str_opt += std::string( "-ORBDottedDecimalAddresses 1 " );

                argc_tmp += 2;

                // We add both root_poa_port and cdmw_root_poa_port, while TAO is able to
                // create as many POAManager listening on two ports defined at ORB init with
                // the ORBEndpoint option.
                // The pb with Orbaccus was to define an other POA manager with a well_known
                // port!
                if (root_poa_port != 0)
                {
                    std::ostringstream root_poa_endpoint;
                    root_poa_endpoint << "iiop://:" << root_poa_port << std::ends;

                    str_opt += std::string( "-ORBEndpoint ") + root_poa_endpoint.str() + " ";

                    argc_tmp += 2;
                }

                if (cdmw_root_poa_port != 0)
                {
                    std::ostringstream cdmw_root_poa_endpoint;
                    cdmw_root_poa_endpoint << "iiop://:" << cdmw_root_poa_port << std::ends;

                    str_opt += std::string( "-ORBEndpoint ")
                               + cdmw_root_poa_endpoint.str() + " ";
                    argc_tmp += 2;
                }

                //To solve TAO regression on FT_GROUP_VERSION Case 4286
                
                str_opt += std::string( "-ORBFTSendFullGroupTC 1 ");
                argc_tmp += 2;
        // To  solve the TAO problem with Codeset

        str_opt += std::string( "-ORBNegotiateCodesets 0 ");
                argc_tmp += 2;

                argv_tmp = Cdmw::Common::String::to_char_array(
                               Cdmw::Common::String::to_strings( str_opt ) );

                // call the TAO ORB_init

                theOrb = CORBA::ORB_init( argc_tmp, argv_tmp, orb_identifier );


            Cdmw::Common::String::delete_char_array(argv_tmp);
            
        /* For DEBUG      
         std::cerr << "resourceFactoryDefs ==> " << resourceFactoryDefs.c_str() << " <=="<<std::endl;
         std::cerr << "serverStrategyFactoryDefs ==> " << serverStrategyFactoryDefs.c_str() << " <=="<< std::endl;
         std::cerr << "clientStrategyFactoryDefs ==> " << clientStrategyFactoryDefs.c_str() << " <=="<< std::endl;
         std::cerr << "ORBClientConnectionHandler ==>" << ORBClientConnectionHandler.c_str() << " <=="<< std::endl; */

                // We don't need to create a specific CdmwPOAManager while the same port
                // specified with ORBEndpoint option is used for every POA manager.
            }
            catch (const std::bad_alloc &)
            {
                // Orbcaus operations do not catch this exception
                throw CORBA::NO_MEMORY(NO_MEMORY, CORBA::COMPLETED_NO);
            }
            catch (const CORBA::SystemException & )
            {
                throw;
            }

            // 4.6 ORB Initialisation : The OrbId

            // The identifier for the ORB will be a name of type CORBA::ORBid. All ORBid
            // strings other than the empty string are allocated by ORB administrators and are not
            // managed by the OMG. ORBid strings other than the empty string are intended to be
            // used to uniquely identify each ORB used within the same address space in a multi-
            // ORB application. These special ORBid strings are specific to each ORB
            // implementation and the ORB administrator is responsible for ensuring that the names
            // are unambiguous.
            // If an empty ORBid string is passed to ORB_init, then the arg_list arguments shall be
            // examined to determine if they indicate an ORB reference that should be returned. This
            // is achieved by searching the arg_list parameters for one preceded by -ORBid for
            // example, -ORBid example_orb (the white space after the -ORBid tag is
            // ignored) or -ORBidMyFavoriteORB (with no white space following the -ORBid
            // tag). Alternatively, two sequential parameters with the first being the string -ORBid
            // indicates that the second is to be treated as an ORBid parameter. If an empty string is
            // passed and no arg_list parameters indicate the ORB reference to be returned, the
            // default ORB for the environment will be returned.


            // if the ord reference return by the ORB_init is presented in the list, we return the reference whitout do anything
            std::map <CORBA::ORB_ptr, OrbTaoThreadPool*>::iterator i = OrbToThreadPool.find(theOrb.in());

            if (i != OrbToThreadPool.end())
            {
                return (theOrb._retn());
            }

            //else we register the reference in the list with a new OrbTaoThreadPool object (the OrbTaoThreadPool allows the creation of threadpool

            OrbTaoThreadPool* orbTaoThreadPool = NULL;

            std::size_t orbThreadStackSize =
                OsSupport::SystemProperties::Get_Property(
               ORB_THREADS_STACKSIZE_PROPERTY_NAME , 0);
            // If orbThreadStackSize is not defined take cdmwThreadStackSize
            // If cdmwThreadStackSize is not defined take default Cdmw Thread stack size
            if (orbThreadStackSize == 0)
            {
                std::size_t cdmwThreadStackSize =
                    OsSupport::SystemProperties::Get_Property(
                  OsSupport::CDMW_THREADS_STACKSIZE_PROPERTY_NAME , 0);
                if (cdmwThreadStackSize == 0)
                {
                    orbThreadStackSize = OsSupport::Thread::DEFAULT_STACK_SIZE;
                }
                else
                {
                    orbThreadStackSize = cdmwThreadStackSize;
                }
            }

            // Do we use a thread pool for the ORBs ?
            if (strategyList.is_PoaThreadPool (nbThreads))
            {
                // Must be checked by the StrategyList
                CDMW_ASSERT(nbThreads > 0);

                if (nbThreads > 1)
                {
                    // Creation of the thread pool
                    // we decrease the number of thread because the user invoke
                    // an orb->run in his main program
                    orbTaoThreadPool =
                        new OrbTaoThreadPool(nbThreads - 1, orbThreadStackSize, theOrb.in());
                }

/*                CORBA::Object_var object_rt_orb
                = theOrb->resolve_initial_references("RTORB");
                rt_orb = RTCORBA::RTORB::_narrow(object_rt_orb.in());

                CORBA::Object_var object_current
                = theOrb->resolve_initial_references("RTCurrent");
                current = RTCORBA::Current::_narrow(object_current.in());*/
            }
            else if (strategyList.is_PoaThreadPerRequest ())
            {
                nbThreads = 10;
                // Creation of the thread pool
                orbTaoThreadPool =
                    new OrbTaoThreadPool(nbThreads - 1, orbThreadStackSize, theOrb.in());

            }
            else if (strategyList.is_PoaThreadPerConnection ())
            {
            }

            // registered in the map
            OrbToThreadPool[theOrb.in()] = orbTaoThreadPool;
            // If nbThreads < 2 the orbTaoThreadPool registered in the map above is NULL
            if (nbThreads > 1)
            {
                // activation of the threads
                orbTaoThreadPool->start();
                // ask threads to do orb->run
                for (unsigned int i=1 ; i < nbThreads ; i++)
                {
                    orbTaoThreadPool->enqueue(new OrbRunTao(orbTaoThreadPool));
                }
            }

         // Add Support for TAO ORB Connection Timeout
         // Value in ms
         std::string dummy_timeout = 
            OsSupport::SystemProperties::Get_Property(CDMW_ORB_CONNECTION_TIMEOUT_PROPERTY_NAME);
         if (!dummy_timeout.empty()) {
            int timeout = atoi(dummy_timeout.c_str());
            if (timeout <= 0)
               throw CORBA::BAD_PARAM(  
                  BAD_PARAMInvalidOrbConnectionTimeout,
                  CORBA::COMPLETED_NO );

            // Convert to 100th of nanoseconds
            TimeBase::TimeT connection_timeout = 
               static_cast<TimeBase::TimeT>(timeout) * 10000;

            CORBA::Any timeout_as_any;
            timeout_as_any <<= connection_timeout;
            CORBA::PolicyList policies(1);
            policies.length(1);
            policies[0] =
               theOrb->create_policy(TAO::CONNECTION_TIMEOUT_POLICY_TYPE, timeout_as_any);
            CORBA::Object_var obj = theOrb->resolve_initial_references("ORBPolicyManager");
          
            CORBA::PolicyManager_var policy_manager = CORBA::PolicyManager::_narrow(obj.in());
            policy_manager->set_policy_overrides(policies, CORBA::ADD_OVERRIDE);
            policies[0]->destroy ();
         }
      
            //fixing memory leak
            //ACE_Service_Config::close();

            return theOrb._retn();
        }

        //
        // Create a new TAO POA
        //
        PortableServer::POA_ptr
        OrbTaoImpl::create_POA( PortableServer::POA_ptr parentPOA,
                                const char* adapter_name,
                                PortableServer::POAManager_ptr a_POAManager,
                                const CORBA::PolicyList& policies,
                                const StrategyList& strategyList )
        throw( PortableServer::POA::AdapterAlreadyExists,
               PortableServer::POA::InvalidPolicy,
               CORBA::SystemException )
        {
            // Check that parentPOA is not a nil object reference

            if ( CORBA::is_nil( parentPOA ) )
            {
                throw CORBA::BAD_PARAM( BAD_PARAMNilObjectReference,
                                        CORBA::COMPLETED_NO );
            }


            // We make a copy of the user policies

            PortableServer::POAManager_var the_POAManager
            = PortableServer::POAManager::_duplicate(a_POAManager);

            CORBA::PolicyList TAOpolicies(policies);

            size_t nbThreads = 0;

            // Do we use a thread pool for the POAs ?
            if (strategyList.is_PoaThreadPool( nbThreads ))
            {
                     if (nbThreads == 1)
                     {
                         bool is_single_thread_policy_already_defined = false;
                         
                         // Thread pool strategy with only one thread
                         // The user want to have a serialyzed strategy
                         // Check the user has not already declared an 
                         // incompatible policy or a the SINGLE THREAD policy
                         for (CORBA::ULong p_i=0 ; 
                                 p_i< TAOpolicies.length() ;
                                 p_i++)
                         {
                             CORBA::Policy_var a_policy = TAOpolicies[p_i];
                             if (a_policy->policy_type() == 
                                     PortableServer::THREAD_POLICY_ID)
                             {
                                  PortableServer::ThreadPolicy_var thread_policy =
                                      PortableServer::ThreadPolicy::_narrow(TAOpolicies[p_i]);
                                  if (thread_policy->value() == 
                                          PortableServer::ORB_CTRL_MODEL)
                                  {
                                       // incompatible policy given by the user 
                                       // with this strategies
                                       throw PortableServer::POA::InvalidPolicy();
                                  }

                                  if (thread_policy->value() == 
                                          PortableServer::SINGLE_THREAD_MODEL)
                                  {
                                      is_single_thread_policy_already_defined = true;
                                      break;
                                  }
                             }
                         }

                         // add single thread policy is not already defined
                         if (!is_single_thread_policy_already_defined)
                         {
                              TAOpolicies.length (TAOpolicies.length () + 1);

                              TAOpolicies[TAOpolicies.length() - 1] = 
                                  parentPOA->create_thread_policy(
                                          PortableServer::SINGLE_THREAD_MODEL);
                         }
                     }
                }

/*                // configure the threadpool parameter
                CORBA::ULong stacksize = 30000;
                CORBA::Boolean allow_request_buffering = 0;
                CORBA::ULong max_buffered_requests = 0;
                CORBA::ULong max_request_buffer_size = 0;

                // obtain the priority in the current thread
                RTCORBA::Priority default_thread_priority = current->the_priority();

                RTCORBA::ThreadpoolId threadpool_id
                = rt_orb->create_threadpool(stacksize,
                                            nbThreads,
                                            0,
                                            default_thread_priority,
                                            allow_request_buffering,
                                            max_buffered_requests,
                                            max_request_buffer_size);

                CORBA::Policy_var threadpool_policy_1
                = rt_orb->create_threadpool_policy(threadpool_id);

                TAOpolicies.length (TAOpolicies.length () + 2);


                TAOpolicies[TAOpolicies.length() - 2]
                = rt_orb->create_priority_model_policy(RTCORBA::CLIENT_PROPAGATED,
                                                       default_thread_priority);
                TAOpolicies[TAOpolicies.length() - 1]
                = CORBA::Policy::_duplicate(threadpool_policy_1.in());
            }
            else if (strategyList.is_PoaThreadPerRequest())
            {

                nbThreads = 10;
                CORBA::ULong stacksize = 30000;
                CORBA::Boolean allow_request_buffering = 0;
                CORBA::ULong max_buffered_requests = 0;
                CORBA::ULong max_request_buffer_size = 0;
                RTCORBA::Priority default_thread_priority = current->the_priority();

                RTCORBA::ThreadpoolId threadpool_id
                = rt_orb->create_threadpool (stacksize,
                                             nbThreads,
                                             0,
                                             default_thread_priority,
                                             allow_request_buffering,
                                             max_buffered_requests,
                                             max_request_buffer_size);

                CORBA::Policy_var threadpool_policy_1
                = rt_orb->create_threadpool_policy(threadpool_id);

                TAOpolicies.length(TAOpolicies.length () + 2);

                TAOpolicies[TAOpolicies.length() - 1]
                = rt_orb->create_priority_model_policy(RTCORBA::CLIENT_PROPAGATED,
                                                       default_thread_priority);
                TAOpolicies[TAOpolicies.length() - 2]
                = CORBA::Policy::_duplicate(threadpool_policy_1.in());
            }
*/

            // We have inserted our proprietary policies, we create the POA
            return parentPOA->create_POA(adapter_name, the_POAManager.in(), TAOpolicies);

        }




        //
        // Attach a servant to a corbaloc reachable object
        //
        void
        OrbTaoImpl::bind_object_to_corbaloc( CORBA::ORB_ptr orb,
                                             const std::string& corbaloc_name,
                                             CORBA::Object_ptr object )
        throw( CORBA::SystemException )
        {
            CORBA::Object_var iorTableObj =
                orb->resolve_initial_references( "IORTable" );

            IORTable::Table_var iorTable
            = IORTable::Table::_narrow( iorTableObj.in() );

            // Are we not using a specific version of Tao ?
            CDMW_ASSERT( !CORBA::is_nil( iorTable.in() ) );

            // We register our object

            try
            {
                CORBA::String_var ior_string = orb->object_to_string( object );

                iorTable->bind( corbaloc_name.c_str(), ior_string.in() );
            }
            catch ( const IORTable::AlreadyBound & )
            {
                throw CORBA::BAD_PARAM(
                    BAD_PARAMCorbalocBindingAlreadyExists,
                    CORBA::COMPLETED_NO );
            }
            catch ( const CORBA::SystemException & )
            {
                throw;
            }
        }

        void
        OrbTaoImpl::remove_corbaloc_binding( CORBA::ORB_ptr orb,
                                             const std::string& corbaloc_name )
        throw( CORBA::SystemException )
        {
            CORBA::Object_var iorTableObj =
                orb->resolve_initial_references( "IORTable" );

            IORTable::Table_var iorTable
            = IORTable::Table::_narrow( iorTableObj.in() );

            // Are we not using a specific version of Tao ?
            CDMW_ASSERT( !CORBA::is_nil( iorTable.in() ) );

            // We register our object

            try
            {
                iorTable->unbind( corbaloc_name.c_str() );
            }
            catch ( const IORTable::NotFound & )
            {
                throw CORBA::BAD_PARAM(
                    BAD_PARAMCorbalocBindingNotFound,
                    CORBA::COMPLETED_NO );
            }
            catch ( const CORBA::SystemException & )
            {
                throw;
            }
        }


        //
        // Attach a servant to a service reachable object (by resolve_initial_reference)
        //
        void
        OrbTaoImpl::register_initial_reference (CORBA::ORB_ptr orb,
                                                const std::string& service_name,
                                                CORBA::Object_ptr object)
        throw (CORBA::SystemException)
        {
            try
            {
                orb->register_initial_reference (service_name.c_str(), object);
            }
            catch (const CORBA::ORB::InvalidName&)
            {
                throw CORBA::BAD_PARAM(BAD_PARAMInvalidServiceName,
                                       CORBA::COMPLETED_NO);
            }
            catch (const CORBA::SystemException &)
            {
                throw;
            }
        }




        //
        // Transform an exception into a human readable string
        // WARNING: This function should return a maximum of information on the exception!
        //
        std::string
        OrbTaoImpl::exception_to_string (const CORBA::Exception& ex )
        throw (OutOfMemoryException)
        {
            try
            {
                return ex._info().c_str();
            }
            catch ( ... )
            {
                CDMW_THROW(OutOfMemoryException);
            }
        }

        //
        // Transform an exception into a human readable string
        //
        std::string
        OrbTaoImpl::exception_name (const CORBA::SystemException& ex )
        throw (OutOfMemoryException)
        {
            try
            {
                return ex._name();
            }
            catch ( ... )
            {
                CDMW_THROW( OutOfMemoryException );
            }

        }


        //
        // Clean up the threadpool create for the threadpool policies
        //
        void
        OrbTaoImpl::ORB_cleanup (CORBA::ORB_ptr orb)
        throw (CORBA::BAD_INV_ORDER)
        {
            // shutdown the orb to wait the threads completions
            // BAD_INV_ORDER minor code     explanation
            //               1              Dependency exists in IFR preventing destruction of this object
            //             2              Attempt to destroy indestructible objects in IFR
            //             3              Operation would deadlock
            //             4              ORB has shutdown

            try
            {
                orb->shutdown(false);
            }
            catch (const CORBA::BAD_INV_ORDER & ex)
            {
#if (TAO_MINOR_VERSION <3)

                if (ex.minor() != (TAO_OMG_VMCID | 4))
                    throw ;

#else

                if (ex.minor() != (CORBA::OMGVMCID | 4))
                    throw ;

#endif

            }
            catch ( ... )
            {}

            // wait the threads completion
            OrbTaoThreadPool* threadPoolToClean = OrbToThreadPool[orb];
            if (threadPoolToClean != NULL)
            {
              if (threadPoolToClean->nb_threads() != 0)
              {
                threadPoolToClean->stop(true);
              }

              // delete the OrbToThreadPool
              delete threadPoolToClean;
            }

            //  erase the orb reference on the list
            OrbToThreadPool.erase(orb);

         // unset custom backend (default is stderr)
         ACE_Log_Msg * ace = ACE_Log_Msg::instance ();
         ace->set_flags (ACE_Log_Msg::STDERR);
         // unregister and delete the AceTaoLogger
         ACE_Log_Msg_Backend* backend = ACE_Log_Msg::msg_backend (0);
         if (backend != 0) {
             AceTaoLogger* logger = dynamic_cast<AceTaoLogger*>(backend);
             if (logger != 0)
                 delete logger;
         }
         
        }


        //
        // set an endpoint selector factory
        //
        void                
        OrbTaoImpl::set_endpoint_selector_factory (const std::string& endpoint_selector_factory)
        {
            TAO_ORB_Core::set_endpoint_selector_factory (endpoint_selector_factory.c_str());
        }

        //
        // get the repository_id from the object reference
        //
        std::string
        OrbTaoImpl::get_repository_id_from_ior(CORBA::ORB_ptr orb, CORBA::Object_ptr object)
            throw( CORBA::SystemException )
        {
         std::string strTypeHint;
         // "hole in one" using specific TAO code
         strTypeHint = object->_stubobj()->type_id.in();
            
         return strTypeHint;
        }
        

    } // End namespace OrbSupport
} // End namespace Cdmw


#endif //CDMW_ORB_VDR == tao


