/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "orbsupport/TAO_DEBUG.hpp"

#ifdef _USE_TAO_DEBUG_

#include <iostream>
#include <limits.h>


namespace Cdmw
{
namespace OrbSupport
{

        TAO_DEBUG TAO_DEBUG::M_instance;

        static const CORBA::ULong ERROR_CODE = ULONG_MAX;
        
        // refcount offset for CORBA::ORB
        static const int OFFSET_REFCOUNT_ORB = 0x7;
        
        // refcount offset for TAO_ORB_Core
        static const int OFFSET_REFCOUNT_ORB_CORE = 0xCE;
        
        // refcount offset for CORBA::Object
        static const int OFFSET_REFCOUNT_OBJECT = 0x8;
        
        // refcount offset for Cdmw::OrbSupport::RefCountServantBase
        static const int OFFSET_REFCOUNT_SERVANT_CDMW = 0x3;
        
        // refcount offset for PortableServer::RefCountServantBase
        static const int OFFSET_REFCOUNT_SERVANT_ORB = 0x1;
        
        // refcount offset for Cdmw::OrbSupport::RefCountLocalObject
        static const int OFFSET_REFCOUNT_LOCAL_OBJECT_CDMW = 0x3;
        
        // refcount offset for TAO_Local_RefCounted_Object
        static const int OFFSET_REFCOUNT_LOCAL_OBJECT_ORB = 0x8;
        
        // refcount offset for CORBA::ValueFactoryBase
        static const int OFFSET_REFCOUNT_VALUE_FACTORY = 0x1;
        
        // refcount offset for CORBA::DefaultValueRefCountBase
        static const int OFFSET_REFCOUNT_VALUETYPE = 0x1;
        
        
        TAO_DEBUG::TAO_DEBUG()
        {
        }
    
        TAO_DEBUG::~TAO_DEBUG()
        {
            if (m_refcountptr_map.size() != 0) {
                print_all_infos("AT PROCESS END");
            }
        }
        
        
        void TAO_DEBUG::set_ORB(CORBA::ORB_ptr orb)
        {
            if (CORBA::is_nil(orb)) {
                printf("TAO_DEBUG ERROR: ORB is null\n");
                return;
            }
            
            RefcountPtr orb_ptr;
            orb_ptr.base_ptr = orb;
            orb_ptr.offset = OFFSET_REFCOUNT_ORB;
            m_refcountptr_map["- ORB -"] = orb_ptr;

            RefcountPtr orb_core_ptr;
            orb_core_ptr.base_ptr = orb->orb_core();
            orb_core_ptr.offset = OFFSET_REFCOUNT_ORB_CORE;
            m_refcountptr_map["- ORB_Core -"] = orb_core_ptr;

            // ORB_Core::root_poa() performs a _duplicate
            CORBA::Object_var obj = orb->orb_core()->root_poa();
            register_object("- RootPOA -", obj.in());
            
            PortableServer::POA* poa = 
                dynamic_cast<PortableServer::POA*>(obj.in());
            if (poa != 0) {
                // POA::the_POAManager() performs a _duplicate
                PortableServer::POAManager_var mgr = poa->the_POAManager();
                register_object("- RootPOAManager -", mgr.in());
            }
            
        }

        
        void TAO_DEBUG::register_object(const char* name, 
                                        CORBA::Object_ptr obj)
        {
            if (obj == 0) {
                printf("TAO_DEBUG ERROR: Object* %s is NULL\n", name);
                return;
            }
            if (m_refcountptr_map.find(name) != m_refcountptr_map.end()) {
                printf("TAO_DEBUG ERROR: Object %s already registered\n", name);
                return;
            }
            
            // CORBA::Object could be a cdmw local object
            RefCountLocalObject* cdmw_local =
                dynamic_cast<RefCountLocalObject*>(obj);
            if (cdmw_local != 0) {
                register_cdmw_refcount_localobj(name, cdmw_local);
                return;
            }
            
            // CORBA::Object could be a TAO local object
            TAO_Local_RefCounted_Object* tao_local =
                dynamic_cast<TAO_Local_RefCounted_Object*>(obj);
            if (tao_local != 0) {
                register_tao_refcount_localobj(name, tao_local);
                return;
            }
            
            RefcountPtr ptr;
            ptr.base_ptr = obj;
            ptr.offset = OFFSET_REFCOUNT_OBJECT;
            m_refcountptr_map[name] = ptr;
            
            print_info(name, "Registering Object:");
        }

        
        void TAO_DEBUG::register_cdmw_refcount_localobj(
            const char* name, 
            RefCountLocalObject* localobj)
        {
            if (localobj == 0) {
                printf("TAO_DEBUG ERROR: RefCountLocalObject* %s is NULL\n", name);
                return;
            }
            if (m_refcountptr_map.find(name) != m_refcountptr_map.end()) {
                printf("TAO_DEBUG ERROR: RefCountLocalObject %s already registered\n", name);
                return;
            }
            RefcountPtr ptr;
            ptr.base_ptr = localobj;
            ptr.offset = OFFSET_REFCOUNT_LOCAL_OBJECT_CDMW;
            m_refcountptr_map[name] = ptr;

            print_info(name, "Registering RefCountLocalObject:");
        }

        
        void TAO_DEBUG::register_tao_refcount_localobj(
            const char* name, 
            TAO_Local_RefCounted_Object* localobj)
        {
            if (localobj == 0) {
                printf("TAO_DEBUG ERROR: TAO_Local_RefCounted_Object* %s is NULL\n", name);
                return;
            }
            if (m_refcountptr_map.find(name) != m_refcountptr_map.end()) {
                printf("TAO_DEBUG ERROR: TAO_Local_RefCounted_Object %s already registered\n", name);
                return;
            }
            RefcountPtr ptr;
            ptr.base_ptr = localobj;
            ptr.offset = OFFSET_REFCOUNT_LOCAL_OBJECT_ORB;
            m_refcountptr_map[name] = ptr;

            print_info(name, "Registering TAO_Local_RefCounted_Object:");
        }

        
        void TAO_DEBUG::register_cdmw_refcount_servant(
            const char* name, 
            RefCountServantBase* servant)
        {
            if (servant == 0) {
                printf("TAO_DEBUG ERROR: RefCountServantBase* %s is NULL\n", name);
                return;
            }
            if (m_refcountptr_map.find(name) != m_refcountptr_map.end()) {
                printf("TAO_DEBUG ERROR: RefCountServantBase %s already registered\n", name);
                return;
            }
            RefcountPtr ptr;
            ptr.base_ptr = servant;
            ptr.offset = OFFSET_REFCOUNT_SERVANT_CDMW;
            m_refcountptr_map[name] = ptr;

            print_info(name, "Registering RefCountServantBase:");
        }

        
        void TAO_DEBUG::register_tao_refcount_servant(
            const char* name, 
            PortableServer::RefCountServantBase* servant)
        {
            if (servant == 0) {
                printf("TAO_DEBUG ERROR: RefCountServantBase* %s is NULL\n", name);
                return;
            }
            if (m_refcountptr_map.find(name) != m_refcountptr_map.end()) {
                printf("TAO_DEBUG ERROR: RefCountServantBase %s already registered\n", name);
                return;
            }
            RefcountPtr ptr;
            ptr.base_ptr = servant;
            ptr.offset = OFFSET_REFCOUNT_SERVANT_ORB;
            m_refcountptr_map[name] = ptr;

            print_info(name, "Registering PortableServer::RefCountServantBase:");
        }
        
        void TAO_DEBUG::register_value_factory(
            const char* name,
            CORBA::ValueFactoryBase* value_factory)
        {
            if (value_factory == 0) {
                printf("TAO_DEBUG ERROR: ValueFactoryBase* %s is NULL\n", name);
                return;
            }
            if (m_refcountptr_map.find(name) != m_refcountptr_map.end()) {
                printf("TAO_DEBUG ERROR: ValueFactoryBase %s already registered\n", name);
                return;
            }
            RefcountPtr ptr;
            ptr.base_ptr = value_factory;
            ptr.offset = OFFSET_REFCOUNT_VALUE_FACTORY;
            m_refcountptr_map[name] = ptr;

            print_info(name, "Registering ValueFactoryBase:");
        }

        void TAO_DEBUG::register_valuetype(
            const char* name,
            CORBA::DefaultValueRefCountBase* valuetype)
        {
            if (valuetype == 0) {
                printf("TAO_DEBUG ERROR: DefaultValueRefCountBase* %s is NULL\n", name);
                return;
            }
            if (m_refcountptr_map.find(name) != m_refcountptr_map.end()) {
                printf("TAO_DEBUG ERROR: DefaultValueRefCountBase %s already registered\n", name);
                return;
            }
            RefcountPtr ptr;
            ptr.base_ptr = valuetype;
            ptr.offset = OFFSET_REFCOUNT_VALUETYPE;
            m_refcountptr_map[name] = ptr;

            print_info(name, "Registering DefaultValueRefCountBase:");
        }


        
        CORBA::ULong TAO_DEBUG::get_refcount(const char* name)
        {
            RefcountPtrMap::iterator it = m_refcountptr_map.find(name);
            if (it == m_refcountptr_map.end()) {
                printf("TAO_DEBUG ERROR: name %s not registered\n", name);
                return ERROR_CODE;
            }

            return get_refcount(it->second);            
        }

        
        CORBA::ULong TAO_DEBUG::get_refcount(const RefcountPtr& ptr)
        {
            return *(((CORBA::ULong*)ptr.base_ptr) + ptr.offset);
        }
        
        
        void TAO_DEBUG::print_info(const char* name, const char* extra_msg)
        {
            if (extra_msg != 0)
                printf("  /////// %s\n", extra_msg);
            
            RefcountPtrMap::iterator it = m_refcountptr_map.find(name);
            if (it == m_refcountptr_map.end()) {
                printf("TAO_DEBUG ERROR: name %s not registered\n", name);
                return;
            }
            
            printf("  %-25s : addr=%x  refcount=%u\n", 
                   name, 
                   (unsigned int)it->second.base_ptr, 
                   get_refcount(it->second));
        }
        
        
        void TAO_DEBUG::print_all_infos(const char* extra_msg)
        {
            printf("/////////////////// TAO DEBUG INFOS ///////////////////\n");
            
            if (extra_msg != 0)
                printf("  /////// %s\n", extra_msg);
            
            RefcountPtrMap::iterator it = m_refcountptr_map.begin();
            while (it != m_refcountptr_map.end()) {
                print_info(it->first.c_str());
                ++it;
            }

            printf("///////////////////////////////////////////////////////\n");
        }
    
    
}
}


#endif   // _USE_TAO_DEBUG_

