/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.orbsupport;
import cdmw.common.*;


/**
 * Class to encapsulate and extend the CORBA SystemException.
 */
public class CORBASystemExceptionWrapper extends cdmw.common.Exception {

    private org.omg.CORBA.SystemException systemException;

    /**
     * Constructor.
     *
     * @param se wrapped exception
     * @param reason additional comments for the exception
     */
    public CORBASystemExceptionWrapper(
        org.omg.CORBA.SystemException se, String reason) {

        super( Severity.SAFE, se.toString() + " \n" + reason );
        this.systemException = se;

    }
    
    /**
     * Constructor.
     *
     * @param se wrapped exception
     */
    public CORBASystemExceptionWrapper(org.omg.CORBA.SystemException se) {

        super( Severity.SAFE, se.toString());
        this.systemException = se;

    }
            
    /**
     * Propagate the wrapped exception.
     *
     * @exception SystemException 
     */
    public void raiseSystemException() throws org.omg.CORBA.SystemException {
        throw systemException;
    }
    
    /**
     * Acessor to the internal exception.
     * @return the wrapped exception
     */
    public org.omg.CORBA.SystemException getSystemException() {
        return systemException;
    }
    
    /**
     * Static method to create a CORBASystemExceptionWrapper.
     * Can replace the usual constructor.
     *
     * @param se wrapped exception
     * @param reason additional comments for the exception
     */
    public static CORBASystemExceptionWrapper wrapSystemException(
        org.omg.CORBA.SystemException se, String reason) {
    
        return new CORBASystemExceptionWrapper(se, reason);
    
    }

}

