/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// PCR-0049


package cdmw.orbsupport;


import cdmw.common.Assert;


/**
 * Codec provides a mechanism to encode and decode IDL data types 
 * to/from CDR-encoded sequence of octets (CDR encapsulation)
 * Codec must manipulate data traits so if you want to use data like 
 * structure, array, etc... you must use type_traits macros to define
 * type traits for the specific data
 */
public class Codec
{
    // reference to the ORB
    private static org.jacorb.orb.ORBSingleton M_ORB;

    /**
     * Set the reference of the ORB.
     */
    public static void init(final org.omg.CORBA.ORB orb) {
        Assert.check(orb != null);
        M_ORB = (org.jacorb.orb.ORBSingleton)(orb);
    }


    /**
     * Constructor.
     */
    public Codec() {
        Assert.check(M_ORB != null);
    }


    /**
     * CORBA type to CDR encoder.
     *
     * @param value CORBA type to encode
     * @return An Any that holds the OctetSeq, and null if a failure occured.
     */
    public org.omg.CORBA.Any
    encode(final java.io.Serializable value)
    {
        org.omg.CORBA.Any data = null;

        try {
            org.jacorb.orb.CDROutputStream out =
                new org.jacorb.orb.CDROutputStream(M_ORB);
            // set endian flag
            out.beginEncapsulatedArray();
            // encode
            out.write_value((java.io.Serializable) value);
            data = org.omg.CORBA.ORB.init().create_any();
            org.omg.CORBA.OctetSeqHelper.insert(data, out.getBufferCopy());
        } catch (Exception e) {
            System.err.println(e.getMessage());
            e.printStackTrace(System.err);
        }

        return data;
    }


    /**
     * Extracts an OctetSeq from an Any and returns the decoded object.
     *
     * @param data The Any that holds the OctetSeq.
     * @return the decoded object to be casted in appropriate CORBA type, 
     * and null if a failure occured.
     */
    public java.io.Serializable
    decode(final org.omg.CORBA.Any data)
    {
        try {
            byte[] buf = org.omg.CORBA.OctetSeqHelper.extract(data);
            org.jacorb.orb.CDRInputStream in = 
                new org.jacorb.orb.CDRInputStream(M_ORB, buf);
            // set endian flag
            in.openEncapsulatedArray();
            // decode
            return in.read_value();
            
        } catch (Exception e) {
            System.err.println(e.getMessage());
            e.printStackTrace(System.err);
        }

        return null;
    }
}
