/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "testlargesequence/TestLargeSequenceServerProcess.hpp"

#include <fstream>
#include <sstream>

#include "Foundation/common/String.hpp"
#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "testlargesequence/TestLargeSequence_impl.hpp"
#include "testlargesequence/TestLargeSequence.skel.hpp"
#include "testlargesequence/TestLargeSequenceCommon.hpp"

namespace Cdmw
{

    namespace OrbSupport
    {

        const char* TestLargeSequenceServerProcess::SERVER_CORBALOC_NAME = CORBALOC_NAME ;

        TestLargeSequenceServerProcess::TestLargeSequenceServerProcess()
        {
        }

        TestLargeSequenceServerProcess::~TestLargeSequenceServerProcess()
        throw()
        {
        }

        void
        TestLargeSequenceServerProcess::start(int& argc, char** argv)
        {
            CORBA::ORB_var orb = CORBA::ORB::_nil();
            
            try {
                // We initialise the ORB, with a thread pool of 5
                StrategyList strategyList;
                strategyList.add_OrbThreaded();
                strategyList.add_PoaThreadPool(5);
    
                // force CdmwProcessPort and CdmwLocalisationService
                int argc2 = argc + 2;
                char** argv2 = NULL;
                std::string str_opt( "" );

                // copy initial args
                for (int i = 0; i < argc; i++) {
                    str_opt += std::string( argv[i] ) + " ";
                }

                // add args
                std::stringstream str_stream;
                str_stream << "--CdmwProcessPort=" << CDMW_PROCESS_PORT
                           << " --CdmwLocalisationService=" << CDMW_LOCALISATION_PORT;
                str_opt += str_stream.str();
                argv2 = Cdmw::Common::String::to_char_array(
                            Cdmw::Common::String::to_strings( str_opt ) );
                orb = OrbSupport::ORB_init( argc2, argv2, strategyList);

                // activate the root POA
                CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
                PortableServer::POA_var poa = PortableServer::POA::_narrow(obj.in());
                PortableServer::POAManager_var poaMgr = poa->the_POAManager();
            
                // We create the servant server on the RootPOA
                ::test::TestLargeSequence_impl testLargeSequence_impl;    
                ::test::TestLargeSequence_var testLargeSequence = testLargeSequence_impl._this();
    
                // bind object to corbaloc
                OrbSupport::bind_object_to_corbaloc(orb.in(), SERVER_CORBALOC_NAME, testLargeSequence.in());
    
                // We write its ior in a file
                if (OsSupport::OS::file_exists(IOR_FILE_NAME))
                {
                    OsSupport::OS::unlink(IOR_FILE_NAME); // The "unlink" function "removes" the file...
                }
                std::ofstream iorFile(IOR_FILE_NAME);
                CORBA::String_var strIor = orb->object_to_string( testLargeSequence.in() );
                iorFile << strIor.in() << std::endl;
                iorFile.close();

                // Ok, we can start the ORB
                poaMgr->activate();
    
                // Setup signal handling for ORB to be shut down
                Cdmw::OrbSupport::OrbSupport::install_orb_shutdown_signal_handlers(orb.in());
                orb->run();

                Cdmw::OrbSupport::OrbSupport::uninstall_orb_shutdown_signal_handlers(orb.in());
                
            } catch (const CORBA::SystemException& e) {
                std::cerr << "Unexpected CORBA::SystemException:\n"
                          << e << " - minor code: " << e.minor() << std::endl;
            } catch (...) {
                std::cerr << "Unexpected exception" << std::endl;
            }            
            
            if (!CORBA::is_nil(orb.in())) {
                try {
                    OrbSupport::ORB_cleanup(orb.in());
                    orb->destroy();
                } catch (const CORBA::SystemException& e) {
                    std::cerr << "Unexpected CORBA::SystemException during orb->detroy():\n"
                              << e << " - minor code: " << e.minor() << std::endl;
                }
            }
        }

    } // End namespace OrbSupport
} // End namespace Cdmw

