/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "testorbsupport/TestCORBAExceptionHelper.hpp"
#include "Foundation/orbsupport/CORBAExceptionHelper.hpp"
#include "Foundation/orbsupport/CORBA.hpp"

#include "tao/AnyTypeCode/ExceptionA.h"
#include "tao/AnyTypeCode/SystemExceptionA.h"
#include "tao/AnyTypeCode/PortableInterceptorA.h"

#define DEFINE_SYSTEM_EXCEPTION_TEST(name) \
void \
TestCORBAExceptionHelper::test_##name##_extraction() \
{ \
   TEST_INFO( "Testing CORBA::" << #name << " extraction from CORBA::Any..." ); \
   test_extraction<CORBA::name>(); \
}

namespace Cdmw
{

    namespace OrbSupport
    {
       template <typename T>
       void test_extraction()
       {
          // Minor code.
          const CORBA::ULong minor_ = 1L;
          // Completion status.
          const CORBA::CompletionStatus completed_ = CORBA::COMPLETED_NO;
          T ex(minor_,completed_);
          CORBA::Any any;
          Cdmw::OrbSupport::CORBAExceptionHelper::Insert_Exception(any, ex);
          CORBA::SystemException * system_exception = 
             Cdmw::OrbSupport::CORBAExceptionHelper::Extract_SystemException(any);
          CPPUNIT_ASSERT( 0 != dynamic_cast<T*>(system_exception) );
          CPPUNIT_ASSERT( system_exception->minor() == minor_ );
          CPPUNIT_ASSERT( system_exception->completed() == completed_ );
          delete system_exception;
       }

        /**
        *Implementation notes: [if no pertinent write none ]
        *<p>
        *Portability issues: [if no pertinent write none ]
        *<p>
        */

        CPPUNIT_TEST_SUITE_REGISTRATION( TestCORBAExceptionHelper );

        TestCORBAExceptionHelper::TestCORBAExceptionHelper()
        {
        }


        TestCORBAExceptionHelper::~TestCORBAExceptionHelper()
           throw()
        {
        }
       DEFINE_SYSTEM_EXCEPTION_TEST(TRANSIENT);
       DEFINE_SYSTEM_EXCEPTION_TEST(BAD_PARAM);
       DEFINE_SYSTEM_EXCEPTION_TEST(BAD_INV_ORDER);
       DEFINE_SYSTEM_EXCEPTION_TEST(COMM_FAILURE);
       DEFINE_SYSTEM_EXCEPTION_TEST(INTERNAL);
       DEFINE_SYSTEM_EXCEPTION_TEST(OBJECT_NOT_EXIST);
       DEFINE_SYSTEM_EXCEPTION_TEST(NO_MEMORY);
       DEFINE_SYSTEM_EXCEPTION_TEST(IMP_LIMIT);
       DEFINE_SYSTEM_EXCEPTION_TEST(INV_OBJREF);
       DEFINE_SYSTEM_EXCEPTION_TEST(UNKNOWN);
       DEFINE_SYSTEM_EXCEPTION_TEST(NO_PERMISSION);
       DEFINE_SYSTEM_EXCEPTION_TEST(MARSHAL);
       DEFINE_SYSTEM_EXCEPTION_TEST(INITIALIZE);
       DEFINE_SYSTEM_EXCEPTION_TEST(NO_IMPLEMENT);
       DEFINE_SYSTEM_EXCEPTION_TEST(BAD_TYPECODE);
       DEFINE_SYSTEM_EXCEPTION_TEST(BAD_OPERATION);
       DEFINE_SYSTEM_EXCEPTION_TEST(NO_RESOURCES);
       DEFINE_SYSTEM_EXCEPTION_TEST(NO_RESPONSE);
       DEFINE_SYSTEM_EXCEPTION_TEST(PERSIST_STORE);
       DEFINE_SYSTEM_EXCEPTION_TEST(FREE_MEM);
       DEFINE_SYSTEM_EXCEPTION_TEST(INV_IDENT);
       DEFINE_SYSTEM_EXCEPTION_TEST(INV_FLAG);
       DEFINE_SYSTEM_EXCEPTION_TEST(INTF_REPOS);
       DEFINE_SYSTEM_EXCEPTION_TEST(BAD_CONTEXT);
       DEFINE_SYSTEM_EXCEPTION_TEST(OBJ_ADAPTER);
       DEFINE_SYSTEM_EXCEPTION_TEST(DATA_CONVERSION);
       DEFINE_SYSTEM_EXCEPTION_TEST(TRANSACTION_REQUIRED);
       DEFINE_SYSTEM_EXCEPTION_TEST(TRANSACTION_ROLLEDBACK);
       DEFINE_SYSTEM_EXCEPTION_TEST(INVALID_TRANSACTION);
       DEFINE_SYSTEM_EXCEPTION_TEST(INV_POLICY);
       DEFINE_SYSTEM_EXCEPTION_TEST(CODESET_INCOMPATIBLE);

       void TestCORBAExceptionHelper::test_ForwardRequest_extraction()
       {
          TEST_INFO( "Testing PortableInterceptor::ForwardRequest extraction from CORBA::Any..." ); 

          // Minor code.
          CORBA::Object_var forward_ = CORBA::Object::_nil();
          //CORBA::Boolean permanent_ = true;

          PortableInterceptor::ForwardRequest ex(forward_.in());

          CORBA::Any any;
          Cdmw::OrbSupport::CORBAExceptionHelper::Insert_Exception(any, ex);
          {
             PortableInterceptor::ForwardRequest * user_exception =
                Cdmw::OrbSupport::CORBAExceptionHelper::Extract_Exception<PortableInterceptor::ForwardRequest>(any);
             CPPUNIT_ASSERT( 0 != dynamic_cast<PortableInterceptor::ForwardRequest*>(user_exception) );
             CPPUNIT_ASSERT( CORBA::is_nil(user_exception->forward.in()) );
             //CPPUNIT_ASSERT( user_exception->permanent == true );
             delete user_exception;
          }
       }

    } // End namespace OrbSupport
} // End namespace Cdmw

