/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.orbsupport.test;

import java.util.Vector;
import java.util.Enumeration;

import java.io.File;
import java.io.IOException;

import cdmw.ossupport.OS;
import cdmw.orbsupport.ORBUtils;
import cdmw.orbsupport.ORBSupport;
import cdmw.ossupport.ThreadedProcess;
import cdmw.orbsupport.StrategyList;
import cdmw.testutils.TestManager;
import cdmw.common.BadParameterException;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;
import org.omg.PortableServer.POAManagerPackage.AdapterInactive;
import org.omg.PortableServer.POAManager;
import org.omg.CORBA.ORBPackage.InvalidName;

import org.junit.BeforeClass;
import org.junit.Test;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

public class ORBSupportTest {
    private static boolean validOnly;
    private static int timescale;

    @BeforeClass
    public static void initTests() {
        TestManager testManager = TestManager.instance();
        
        validOnly = Boolean.parseBoolean(
            testManager.getProperty("ORB_SUPPORT_TEST_VALID_ONLY"));
        timescale = TestManager.getTimescale();
    }
    
    private String readFromFile(String file) {

        try {
            java.io.BufferedReader in = new java.io.BufferedReader(
                new java.io.FileReader(file) );
            return in.readLine();
        }
        catch(IOException ioe) {
            return null;
        }
    }

    private void doPortNumberTest(org.omg.CORBA.ORB orb) {
        try {
            String sForceDiffProcPort = System.getProperty("force.diff.process.port");
            boolean forceDiff = new Boolean(sForceDiffProcPort).booleanValue();

            String serverMTCorbaloc = "corbaloc::localhost:" +
                                      (ServerProcess.SERVER_MT_PORT + (forceDiff ? 1 : 0)) +
                                      "/" + ServerProcess.SERVER_MT_CORBALOC_NAME;
            System.out.println("Try to contact multi-threaded server via corbaloc: " + serverMTCorbaloc);
            org.omg.CORBA.Object objMT = orb.string_to_object( serverMTCorbaloc );
            ORBSupportServer serverMT = ORBSupportServerHelper.narrow( objMT );

            serverMT.computeSquare((short)5);
        } catch (Exception e) {
            fail("Exception: " + e.toString());
        }

        String serverSTCorbaloc = "corbaloc::localhost:" + ServerProcess.SERVER_ST_PORT + 
                                      "/" + ServerProcess.SERVER_ST_CORBALOC_NAME;
        try {        
            System.out.println("Try to contact single-threaded server via corbaloc: " + serverSTCorbaloc);
            org.omg.CORBA.Object objST = orb.string_to_object( serverSTCorbaloc );
            ORBSupportServer serverST = ORBSupportServerHelper.narrow( objST );

            serverST.computeSquare((short)5);

            // See PCR-0309
            System.out.println("PCR-0309 ==> Remove the corbaloc binding (on server process)");
            serverST.removeBinding();
        } catch (Exception e) {
            fail("Exception: " + e.toString());
        }
        // See PCR-0309
        try {
            System.out.println("PCR-0309 ==> Check that corbaloc can't be used after remove ");
            org.omg.CORBA.Object objST = orb.string_to_object( serverSTCorbaloc );
            ORBSupportServer serverST = ORBSupportServerHelper.narrow( objST );
            fail();
        } catch (Exception e) {
            // Expected behaviour - do nothing intentionally
        }
    }
    
    private void doPOACreationTests(ORBSupportServer server)
        throws InterruptedException {

        System.out.println("Starting a thread with requesting a slow service");
        ClientThread thread_1 = new ClientThread(server,"slow");
        thread_1.start();
        OS.sleep(timescale * 1000);

        System.out.println("Starting a thread with requesting a fast service");
        ClientThread thread_2 = new ClientThread(server,"");
        thread_2.start();

        System.out.println("Sleeping a while to wait the second thread");
        OS.sleep(timescale * 2000);

        System.out.println("Checking if the second thread is blocked");
        System.out.println("  because the signle thread server process");
        System.out.println("  process the slow request of the first one");
        assertTrue(thread_2.getResult() == 0);

        System.out.println("Waiting for the completion of the first thread");
        thread_1.join();

        System.out.println("Sleeping a while to wait the second thread");
        OS.sleep(timescale * 1000);
        System.out.println("Checking the second thread cna have its request processed");
        assertTrue(thread_2.getResult() == 25);
        thread_2.join();
    }

    private void doORBThreadPoolLimitTests(ORBSupportServer server)
        throws InterruptedException {

        int nbThreads = 5;

        System.out.println("Creating five threads with a reference to the server");
        Vector threads = new Vector();

        for (int iThread=0; iThread < nbThreads; ++iThread ) {
            threads.add( new ClientThread(server,"slow") );
        }

        System.out.println("Starting the threads");
        for (Enumeration e = threads.elements() ; e.hasMoreElements() ;) {
            ((ClientThread) e.nextElement()).start();
        }

        System.out.println("Sleeping a while to wait the threads");
        OS.sleep(timescale * 2000);

        System.out.println("Starting a new thread");
        ClientThread testClientThread = new ClientThread( server,"" );
        testClientThread.start();

        System.out.println("Sleeping a while to wait the last thread");
        OS.sleep(timescale * 1000);
        System.out.println("Checking the last thread is blocked by the");
        System.out.println("  limit of the server thread pool");
        assertTrue( testClientThread.getResult() == 0 );

        System.out.println("Waiting the threads completion");
        for (Enumeration e = threads.elements() ; e.hasMoreElements() ;) {
            ((ClientThread) e.nextElement()).join();
        }

        System.out.println("Sleeping a while to wait the last thread");
        OS.sleep(timescale * 1000);

        System.out.println("Checking the last thread is unblocked");
        assertTrue(testClientThread.getResult() == 25);
        testClientThread.join();
    }

    private void doORBMultithreadTests(ORBSupportServer server)
        throws InterruptedException {

        int nbThreads = 4;

        System.out.println("Creating four threads with a reference to the server");
        Vector threads = new Vector();

        for (int iThread=0; iThread < nbThreads; ++iThread ) {
            threads.add( new ClientThread(server, "slow") );
        }

        System.out.println("Starting the threads");
        for (Enumeration e = threads.elements() ; e.hasMoreElements() ;) {
            ((ClientThread) e.nextElement()).start();
        }

        System.out.println("Sleeping a while to wait the threads");
        OS.sleep(timescale * 3000);

        System.out.println("Querying the server how much threads are in service ?");
        int nb = server.getNumberOfThreads();

        assertTrue(nb == nbThreads);

        System.out.println("Waiting for the threads completion");
        for (Enumeration e = threads.elements() ; e.hasMoreElements() ;) {
            ((ClientThread) e.nextElement()).join();
        }
    }

    @Test
    public void doORBTests() throws InterruptedException,AdapterInactive,InvalidName {

        String iorMTFile = "SupportServerMT.ior";
        String iorSTFile = "SupportServerST.ior";
        StrategyList strategies = new StrategyList();
        strategies.addORBThreaded();
        try {
            strategies.addPOAThreadPerRequest();
        } catch(BadParameterException bpe) {
            throw new InterruptedException("Unable to initialize "
                + "the strategy list");
        }
        // ORB init
         org.omg.CORBA.ORB orb = ORBSupport.ORBInit(new String[0], strategies);
        
        // <doORBTest> ORB init successful.

           System.out.println("Starting a CORBA server in another process");

        String classname = "cdmw.orbsupport.test.MainTest --server";
        ThreadedProcess serverp = null;
           try {
            //serverp = OS.createJavaProcess(classname, getProperties());
            Vector props = TestManager.getDefaultProperties();
            String myprop = "force.diff.process.port";
            props.add(myprop + "=" +
                new Boolean(
                    System.getProperty("force.diff.process.port")).booleanValue());
            serverp = OS.createJavaProcess(classname, props);
        } catch(IOException ioe) {
            throw new InterruptedException("Unable to create server process.");
        }

        System.out.println("Sleeping a while to wait the process creation");
        OS.sleep(timescale * 5000);

        System.out.println("Getting back the IOR from the file for the multi-thread POA");
        String iorMT = readFromFile(iorMTFile);

        System.out.println("Getting back the IOR from the file for the single thread POA");
        String iorST = readFromFile(iorSTFile);

        if (iorMT==null || iorST==null ) {
            orb.destroy();
            throw new InterruptedException("Unable to retrieve IOR.");
        } 

        System.out.println("Creating the server stub multi-thread POA");
        org.omg.CORBA.Object objMT = orb.string_to_object( iorMT );
        ORBSupportServer serverMT = ORBSupportServerHelper.narrow( objMT );
        assertTrue(serverMT != null);

        System.out.println( "Creating the server stub single-thread POA" );
        org.omg.CORBA.Object objST = orb.string_to_object( iorST );
        ORBSupportServer serverST = ORBSupportServerHelper.narrow( objST );
        assertTrue(serverST != null);

        if (!validOnly) {
            System.out.println("Invoking a method on the server Multi-thread");
            int resultMT = serverMT.computeSquare((short)5);
            assertTrue(resultMT == 25);

            System.out.println("Invoking a method on the server Single-thread");
            int resultST = serverST.computeSquare((short)5);
            assertTrue(resultST == 25);
        }

        // Check if the server accept concurrent requests
        doORBMultithreadTests(serverMT);

        // Check if the server use a thread pool limiting
        //    the number of request to be handled
        if (!validOnly) {
            doORBThreadPoolLimitTests(serverMT);
        }

       // Check if the server is truely single-threaded
        doPOACreationTests(serverST);
        // Check if servers are listening to the appropriate ports
        doPortNumberTest(orb);
        
        OS.sleep(timescale * 3000);
    
        System.out.println("Killing the server");
        OS.killProcess( serverp );
        System.out.println("Removing IOR files");
        OS.unlink(iorSTFile);
        OS.unlink(iorMTFile);
        orb.destroy();
    }
    
    @Test
    public void doTests() {
        System.out.println("Creating a strategy list");
        StrategyList strategyList = new StrategyList();
        StrategyList strategyList_2 = new StrategyList();
        StrategyList strategyList_3 = new StrategyList();
        // Strategy List successfully created

        System.out.println("Checking the strategy list is empty");
        assertTrue(!strategyList.isORBThreaded());
        assertTrue(!strategyList.isPOAThreadPool());
        assertTrue(!strategyList.isPOAThreadPerRequest());
        assertTrue(!strategyList.isPOAThreadPerConnection());

        System.out.println("Setting the Orb ThreadPerConnection strategy in the list");
        strategyList.addORBThreaded();
        System.out.println("Checking the Orb ThreadPerConnection strategy in the list");
        assertTrue(strategyList.isORBThreaded());

        System.out.println("Setting the Orb ThreadPerRequest strategy in the list");
        strategyList.addORBThreaded();
        System.out.println("Checking the Orb ThreadPerRequest strategy in the list");
        assertTrue(strategyList.isORBThreaded());

        System.out.println("Setting the Poa Thread pool strategy in the list");
        try {
            strategyList.addPOAThreadPool(3);
        } catch (BadParameterException bpe) {}
        System.out.println("Checking the Poa Thread pool strategy in the list");
        assertTrue(strategyList.isPOAThreadPool());
        assertTrue(strategyList.getNbPOAThreads() == 3);

        System.out.println(
            "Setting the POA ThreadPerRequest strategy incompatible with the thread pool");
        try {
            strategyList.addPOAThreadPerRequest();
            fail("Error: POAThreadPool should be already set");
        } catch (BadParameterException bpe) {
            // Success: POAThreadPool already set
        }

        if (!validOnly) {
            System.out.println("Setting the POA Threaded strategy in the list (ThreadPerRequest)");
            try {
                strategyList_2.addPOAThreadPerRequest();
            } catch (BadParameterException bpe) {}
            System.out.println("Checking the POA Threaded strategy in the list");
            assertTrue(strategyList_2.isPOAThreadPerRequest());

            System.out.println("Setting the POA Thread Pool strategy incompatible with the threaded");
            try {
                strategyList_2.addPOAThreadPerRequest();
                fail("Error: POAThreaded should be already set");
            } catch (BadParameterException bpe) {
                // Success: POAThreaded already set
            }

            System.out.println("Setting the POA Threaded strategy in the list (ThreadPerConnection)");
            try {
                strategyList_3.addPOAThreadPerConnection();
            } catch (BadParameterException bpe) {}
            System.out.println("Checking the POA Threaded strategy in the list");
            assertTrue(strategyList_3.isPOAThreadPerConnection());

            System.out.println("Setting the POA Thread Pool strategy incompatible with the threaded");
            try {
                strategyList_3.addPOAThreadPerConnection();
                fail("Error: POAThreaded should be already set");
            } catch (BadParameterException bpe) {
                // Success: POAThreaded already set
            }

            System.out.println("Trying to construct a new strategy from this one");
            StrategyList strategyListCopied = strategyList.copy();
            StrategyList strategyListCopied_2 = strategyList_2.copy();
            System.out.println("Checking the value");
            assertTrue(strategyListCopied.isORBThreaded());
            assertTrue(strategyListCopied.isPOAThreadPool());
            assertTrue(strategyListCopied.getNbPOAThreads() == 3);

            assertTrue(strategyListCopied_2.isPOAThreadPerRequest());

            System.out.println("Trying to assign a new strategy from this one");
            StrategyList strategyListAssigned = strategyList;
            StrategyList strategyListAssigned_2 = strategyList_2;

            System.out.println("Checking the value");
            assertTrue(strategyListAssigned.isORBThreaded());
            assertTrue(strategyListAssigned.isPOAThreadPool());
            assertTrue(strategyListAssigned.getNbPOAThreads() == 3);

            assertTrue(strategyListAssigned_2.isPOAThreadPerRequest());
        }
    }
}

