/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.orbsupport.test;

import java.io.*;
import java.util.*;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.junit.Test;

import cdmw.ossupport.OS;

public class TestShutdownSignalHandler {

    public static final String FILE_PATH = System.getProperty("user.home") + System.getProperty("file.separator") + ".testshutdownhandler.tmp";

    public static final int TIMEOUT = 60000;

    private ProcessHandler processHandler = new ProcessHandler();

    private ErrorStreamChecker errorChecker = new ErrorStreamChecker();

    private Process childProcess;

    private boolean isChildProcessFinished = false;

    private boolean childProcessDestroyed = false;

    private ArrayList<String> Errlist = new ArrayList<String>();

    private class ProcessHandler extends Thread {
        public void run() {
            try {
                TestShutdownSignalHandler.this.childProcess.waitFor();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            TestShutdownSignalHandler.this.isChildProcessFinished = true;
        }
    }

    private class ErrorStreamChecker extends Thread {
        public void run() {
            InputStreamReader streamReader = new InputStreamReader(TestShutdownSignalHandler.this.childProcess                                           .getErrorStream());
            LineNumberReader lineNumberReader = new LineNumberReader(streamReader);
        String str = "";
        
            synchronized (TestShutdownSignalHandler.this.Errlist) {
                try {
            while (str != null)  {             
                        str = lineNumberReader.readLine();
                        if (str != null) {
                            TestShutdownSignalHandler.this.Errlist.add(str);
                }
            }
                
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    @Test
    public void doTests() {
        
        File tmpFile = new File(FILE_PATH);
        tmpFile.delete();
        System.out.println("Create cdmw.orbsupport.test.ShutdownHandlerInstaller process");
        try {
            this.childProcess = OS.createJavaProcess(
                    "cdmw.orbsupport.test.ShutdownHandlerInstaller")
                    .getProcess();
        } catch (IOException e) {
            e.printStackTrace();
        }

        this.processHandler.start();
        this.errorChecker.start();

        int timeOfExecution = 0;
        int sleepTime = 1000;

        while (!this.isChildProcessFinished && timeOfExecution < TIMEOUT) {
            try {
                System.out.println("waiting child process to finish");
                Thread.sleep(sleepTime);
                timeOfExecution += sleepTime;
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }

        timeOfExecution = 0;
        if (!this.isChildProcessFinished) {
            System.out.println("child process ended exceeds timeout");
            new Thread(new Runnable() {
                public void run() {
                    childProcess.destroy();
                    childProcessDestroyed = true;
                }
            }).start();

            while (timeOfExecution < TIMEOUT * 2 && !this.childProcessDestroyed) {
                try {
                    Thread.sleep(sleepTime);
                    timeOfExecution += sleepTime;
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }

            if (this.childProcessDestroyed) {
                System.out.println("child process destroying successfully");
            } else {
                System.out.println("child process destroying exceeds timeout");
            }
        } else {
            System.out.println("child process ended");
        }

        if (this.Errlist.size() > 0) {
            StringBuffer errorString
                = new StringBuffer("Errors are occured during installing ShutdownSignalHandler:\n");
            for (Iterator<String> i = this.Errlist.iterator(); i.hasNext();) {
                errorString.append("\t");
                errorString.append(i.next());
            }
            fail(errorString.toString());
        }

        System.out.println("check if ShutdownSignalHandler was called in child process");
        assertTrue(new File(FILE_PATH).exists());
    
        tmpFile.delete();
    }
}