/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "testvaluetypesequence/TestAnyValueTypePassingDriver.hpp"

#include <fstream>
#include <sstream>
#include <string>
#include <vector>

#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"

#include "Foundation/common/System.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "testvaluetypesequence/TestAnyValueTypePassing.stub.hpp"
#include "testvaluetypesequence/TestAnyValueTypeCommon.hpp"

namespace Cdmw
{
    namespace OrbSupport
    {

        CPPUNIT_TEST_SUITE_REGISTRATION( TestAnyValueTypePassingDriver );

        TestAnyValueTypePassingDriver::TestAnyValueTypePassingDriver()
        {
            m_process_name = "testanyvaluetype";
        }

        TestAnyValueTypePassingDriver::~TestAnyValueTypePassingDriver() throw()
        {
        }

        void TestAnyValueTypePassingDriver::do_tests()
        {
            const std::string IOR_FILE = IOR_FILE_NAME;
    
            StrategyList strategies;
            strategies.add_OrbThreaded();
            strategies.add_PoaThreadPerRequest();
    
            int argc2 = 0;
        //fixing memory leak : declaration separated from affectation
        CORBA::ORB_var orb = CORBA::ORB::_nil();
            orb = OrbSupport::ORB_init(argc2, NULL, strategies);
            
            TEST_INFO( "Getting back the IOR from the file for the multi-thread POA" );
            std::ifstream iorFile( IOR_FILE.c_str() );
            std::string iorStr;
            iorFile >> iorStr;
            iorFile.close();

            TEST_INFO( "Creating the server stub multi-thread POA" );
            CORBA::Object_var obj = orb->string_to_object( iorStr.c_str() );
            ::test::TestAnyValueTypePassing_var server =
                ::test::TestAnyValueTypePassing::_narrow( obj.in() );
            CPPUNIT_ASSERT( ! CORBA::is_nil(server.in()) );
            
            do_any_value_type_passing_test(server);
            //fixing memory leak 
            if (! CORBA::is_nil(orb.in()))
            {
                try
                {
                    Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
                    orb->destroy();
                    orb = CORBA::ORB::_nil(); 
            }
                catch( const CORBA::Exception &e )
                {
                    std::cerr <<__FILE__<<" "<<__LINE__<<" "<< e << std::endl;
                }
                catch( const Cdmw::Exception &e )
                {
                    std::cerr <<__FILE__<<" "<<__LINE__<<" "<< e << std::endl;
                }
                catch(...)
                {
                    std::cerr << "Uncaught exception" << std::endl;
                }
            }
        }
        
        void TestAnyValueTypePassingDriver::do_any_value_type_passing_test( ::test::TestAnyValueTypePassing_ptr server )
        {
      // Memory leaks fixing
          CORBA::Any_var value_any = server->getAnyValue();
          CORBA::Long value_long;
          value_any.in() >>= value_long;
          std::cout << "I read the value: " << value_long << std::endl;
          CPPUNIT_ASSERT( value_long == VALUE_TO_RETURN ); 
        }

    } // End namespace OrbSupport
} // End namespace Cdmw

