/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.orbsupport.test;

import cdmw.orbsupport.ORBSupport;
import cdmw.orbsupport.StrategyList;
import cdmw.orbsupport.PolicyList;

public class ServerProcess {

    private static final String ior_file_name = "SupportServer.ior";
    
    private void writeToFile(String ior, String filename) {

        java.io.File file = new java.io.File(filename);
        String s;
        try {
            // delete any previous file with this name
            file.delete();
            java.io.PrintWriter out = new java.io.PrintWriter(
                new java.io.BufferedWriter( new java.io.FileWriter(file) ) );
            out.println(ior);
            out.close();
        } 
        catch(java.io.IOException e) {}
        catch(java.lang.SecurityException e) {}
    }

    public ServerProcess() {
        // nothing to do
    }

    public void start(String[] args) throws org.omg.CORBA.UserException {
        
        // We initialise the ORB, with a thread pool of 5
        StrategyList strategyList = new StrategyList();
        strategyList.addORBThreaded();
        try {
            strategyList.addPOAThreadPool(5);
        } catch(cdmw.common.BadParameterException bpe) {}
       
        // ORB init
        org.omg.CORBA.ORB orb = ORBSupport.ORBInit(args, strategyList, "");
    
        // And activate the root POA
        org.omg.PortableServer.POA rootPOA =
            org.omg.PortableServer.POAHelper.narrow(
                orb.resolve_initial_references("RootPOA"));
        org.omg.PortableServer.POAManager poaMgr = rootPOA.the_POAManager();
    
        // We create the servant server on the RootPOA
        TestAnyValueTypePassingImpl testAnyValueTypePassingImpl = new TestAnyValueTypePassingImpl();
        TestAnyValueTypePassing testAnyValueTypePassing = testAnyValueTypePassingImpl._this(orb);
    
        // We write its ior in a file
        writeToFile( orb.object_to_string( testAnyValueTypePassing ),
                ior_file_name );
        
        // Ok, we can start the ORB
        poaMgr.activate();
        rootPOA.the_POAManager().activate();
        orb.run();

    }
}

