/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.orbsupport.test;

import java.util.Vector;
import java.util.Enumeration;

import java.io.File;
import java.io.IOException;

import cdmw.ossupport.OS;
import cdmw.orbsupport.ORBUtils;
import cdmw.orbsupport.ORBSupport;
import cdmw.ossupport.ThreadedProcess;
import cdmw.orbsupport.StrategyList;
import cdmw.testutils.Testable;
import cdmw.common.BadParameterException;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;
import org.omg.PortableServer.POAManagerPackage.AdapterInactive;
import org.omg.PortableServer.POAManager;
import org.omg.CORBA.ORBPackage.InvalidName;

public class TestAnyValueTypePassingDriver extends Testable {
    
    private static final int expected_result = 23;
    private static final String ior_file_name = "SupportServer.ior";

    public TestAnyValueTypePassingDriver() {
        this.name = "ORBSupportTest";
        this.out = System.out;
    }

    public String readFromFile(String file) {

        try {
            java.io.BufferedReader in = new java.io.BufferedReader(
                new java.io.FileReader(file) );
            return in.readLine();
        }
        catch(IOException ioe) {
            return null;
        }
    }


    public void doORBTests() throws InterruptedException, AdapterInactive, InvalidName {
        setNbOfRequestedTestOK(4);

        String iorFile = ior_file_name;
        StrategyList strategies = new StrategyList();
        strategies.addORBThreaded();
        try {
            strategies.addPOAThreadPerRequest();
        } catch(BadParameterException bpe) {
            throw new InterruptedException("Unable to initialize "
                + "the strategy list");
        }

        // ORB init
        org.omg.CORBA.ORB orb = ORBSupport.ORBInit(new String[0], strategies);
        
        succeed("<doORBTest> ORB init successful.");

        println("Getting back the IOR from the file for the multi-thread POA");
        String ior = readFromFile(iorFile);

        if (ior == null) {
            orb.destroy();
            throw new InterruptedException("Unable to retrieve IOR.");
        } 

        println("Creating the server stub POA");
        org.omg.CORBA.Object obj = orb.string_to_object( ior );
        TestAnyValueTypePassing server = TestAnyValueTypePassingHelper.narrow( obj );
        check( server != null );
        
        org.omg.CORBA.Any any_result = server.getAnyValue();
        int long_result = any_result.extract_long();
        println("I read the value: " + long_result);
        check( long_result == expected_result ); 
        succeed();

    }
    
    public void doTests() {

        try {
            doORBTests();
        } catch(InterruptedException ie) {
            ie.printStackTrace();
        }catch(AdapterInactive ai) {
            System.out.println(ai.getMessage());
        }catch(InvalidName in) {
            System.out.println(in.getMessage());
        }

    }

}

