/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_LINUXSHAREDOBJECT_OS_HPP
#define INCL_LINUXSHAREDOBJECT_OS_HPP

#ifdef linux

#include <string>
#include <list>
#include <map>
#include <dlfcn.h>
#include <Foundation/common/MacroDefs.hpp>
#include <Foundation/common/Exception.hpp>
#include <Foundation/osthreads/Mutex.hpp>
#include <Foundation/ossupport/SharedObjectHandle.hpp>


namespace Cdmw {
    namespace OsSupport {

        class LinuxSharedObjectHandle {
            public:
                typedef void* OsShlibHandle;
                typedef void* FunctionHandle;
        };

        class LinuxSharedObjectImpl;

        class LinuxSharedObject : public SharedObjectHandle_T<LinuxSharedObjectHandle>
        {
            public:
                /**
                 * dlopen implementation
                 */

                static LinuxSharedObject*
                dlopen(const char* libname, int flag = RTLD_LAZY);

                /**
                 * dlclose implementation
                 */

                int
                dlclose(void);

                /**
                 * dlsym implementation
                 */

                LinuxSharedObjectHandle::FunctionHandle
                dlsym(const char* symname);

                /**
                 * dlerror implementation
                 */

                static std::string
                dlerror(void);

                /**
                 * print
                 */

                void
                print(std::ostream& os);

            private:
                /**
                 * Constructor
                 */

                explicit LinuxSharedObject(LinuxSharedObjectImpl* impl);

                /**
                 * Copy constructor
                 */

                LinuxSharedObject(const LinuxSharedObject& so);

                /**
                 * Operator =
                 */

                LinuxSharedObject&
                operator=(const LinuxSharedObject& rhs);

            public:

                /**
                 * destructor
                 */
                virtual ~LinuxSharedObject();

                /**
                 * Equality and inequality operations
                 */

                bool
                is_null(void)
                {
                    return m_OS_handle == NULL;
                }


                bool
                operator==(const LinuxSharedObject& rhs) const
                {
                    return m_OS_handle == rhs.m_OS_handle;
                }

                bool
                operator!=(const LinuxSharedObject& rhs) const
                {
                    return m_OS_handle != rhs.m_OS_handle;
                }

                bool
                operator!() const
                {
                    return m_OS_handle == NULL;
                }

                operator bool() const
                {
                    return m_OS_handle != NULL;
                }

            private:
                /**
                 * Constructor
                 */
                LinuxSharedObject();

                /**
                 * Handle of the library
                 */
                OsShlibHandle m_OS_handle;

                LinuxSharedObjectImpl* m_Linux_shared_object_impl;
        };

    } // End namespace ossupport
} // End namespace Cdmw

#endif
#endif
