/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_OSSUPPORT_SIGHANDLER_HPP
#define INCL_OSSUPPORT_SIGHANDLER_HPP

#include <Foundation/ossupport/SignalHandler.hpp>
#include <Foundation/osthreads/Mutex.hpp>
#include <map>



namespace Cdmw
{
    namespace OsSupport
    {

        class SigWaitThread;

        /**
         * Purpose:
         *<p> This class manages the signal handling
         *
         */

        class Signals
        {
        public:
            
    
            // destructor
            ~Signals();

            /**
             * Purpose:
             * <p> Register a new Event_handler for the signal
             * and return the old one
             * 
             *@param sig the signal number
             *@param seh the handler for this signal
             *
             *@return the old handler if previously registered
             *
             */
            SignalHandler* register_handler(int sig, 
                                            SignalHandler* seh);
            

            /**
             * Purpose:
             * <p> Unregister the handler for the signal
             * and return the old one
             * 
             *@param sig the signal number
             *
             *@return the handler being unregistered
             *
             */
            SignalHandler* unregister_handler(int sig);
            
            
            /**
             * Purpose:
             * <p> return the handler for the signal
             * 
             *@param sig the signal number
             *
             *@return the handler for the signal
             *
             */
            SignalHandler* get_handler(int sig);
            
            
            /**
             * Purpose:
             *<p> This method is called when a signal is received and if 
             * this handler is registered for this signal
             *
             *@param sig signal to handle
             *
             *@return true if the signal has been registered, else false
             */
             bool handle_signal(int sig) throw();
            
            /**
             * Purpose:
             * <p> return the instance of the class
             * 
             *@return the instance of the class
             *
             */
            static Signals* Instance();
            
        private:
            
            // constructor
            Signals();


            typedef std::map < int, SignalHandler* > HandlerMap;
            // map for the handler
            HandlerMap m_handlerMap;
            
            // Instance
            static Signals * M_Instance;

            //Mutex
            Mutex m_mutex;

};



    } // End namespace ossupport
} // End namespace Cdmw

#endif // INCL_OSSUPPORTSIGHANDLER__HPP
