/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <ossupport/SigWaitThread.hpp>
#include <Foundation/ossupport/Signals.hpp>
#include <Foundation/common/Assert.hpp>
#include <iostream>

namespace 
{
    // There has to be a signal handler in order to be able to
    // sigwait() on SIGCHLD.  The information passed is ignored as it
    // will be recovered by the waitpid() call.
    
    extern "C" void
    sigchld_handler (int)
    {
        // Ignore.
    }
}


namespace Cdmw
{
    namespace OsSupport
    {

        SigWaitThread::SigWaitThread (Signals* handler )
            : m_handler(handler)
        {
        }
        
        SigWaitThread::~SigWaitThread() 
            throw(){}
        
        void SigWaitThread::run() 
            throw ()
        {
            struct sigaction sa;
            memset (&sa, 0, sizeof (sa));
            sa.sa_handler = sigchld_handler;
            // We only want signals when the things exit.
            //sa.sa_flags = SA_NOCLDSTOP;
            if (sigaction (SIGCHLD, &sa, NULL) != 0)
          CDMW_ASSERT(0);
            
            sigset_t signal_to_catch, set, oset;
            sigfillset(&signal_to_catch);
        // For portability, do not block SIGSTOP & SIGKILL
        sigdelset(&signal_to_catch,SIGKILL);
        sigdelset(&signal_to_catch,SIGSTOP);
            int caught;
            
            // loop
            while(1)
            {
                // wait a signal
                if (sigwait(&signal_to_catch, &caught) == 0)
                {
                    if (!m_handler->handle_signal(caught)) {
                        // call the default handler for this signal
                        sigemptyset(&set);
                        sigaddset(&set, caught);
                        pthread_kill(pthread_self(), caught);
                        pthread_sigmask(SIG_UNBLOCK, &set, &oset);
                        pthread_sigmask(SIG_SETMASK, &oset, NULL);
                    }
                } 
            }    
        }
        

    } // End namespace ossupport
} // End namespace Cdmw

