/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "testossupport/TestOSDriver.hpp"

#include <cstdio>
#include <fstream>

#include "Foundation/common/MacroDefs.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/common/String.hpp"
//#include "Foundation/ossupport/ProcessCallback.hpp"
 

namespace Cdmw
{

    namespace OsSupport
    {

        /**
        *Implementation notes: [if no pertinent write none ]
        *<p>
        *Portability issues: [if no pertinent write none ]
        *<p>
        */
        extern "C"
        {
            double test(double r)
            {
                return r;
            };
        }
        
        CPPUNIT_TEST_SUITE_REGISTRATION( TestOSDriver );
        
        TestOSDriver::TestOSDriver( )
        {
            if (Cdmw::TestUtils::TestManager::instance()->get_property ("test") == "OS" 
                || Cdmw::TestUtils::TestManager::instance()->get_property ("test") == "all") 
                m_do_test = true;
            else
                m_do_test = false;
            
        }


        TestOSDriver::~TestOSDriver()
        throw()
        {
        }

        void
        TestOSDriver::test_get_option_values()
        {
            if (!m_do_test)
                return;
            TEST_INFO("Testing get_option_values");
            {
               std::string args("arg0 --opt1= arg1 --opt1=val1 --arg2 --opt1=val2");
               std::vector< std::string > vect__ = 
                  Cdmw::Common::String::to_strings(args);

               char** argv__ = Cdmw::Common::String::to_char_array(vect__);
               int argc__(vect__.size());

               
               try {
                  std::list<std::string> values;

                  std::string opt1("opt1");
                  
                  OS::get_option_values (argc__, argv__, 
                                         opt1,
                                         values);
                  CPPUNIT_ASSERT( values.size() == 3);

                  std::list<std::string>::const_iterator iter= values.begin();
                  CPPUNIT_ASSERT( *iter == "yes");
                  iter++;
                  CPPUNIT_ASSERT( *iter == "val1");
                  iter++;
                  CPPUNIT_ASSERT( *iter == "val2");

                  std::string opt2("opt2");
                  values.clear();
                  OS::get_option_values (argc__, argv__, 
                                         opt2,
                                         values);
                  CPPUNIT_ASSERT( values.size() == 0);
                  
               } catch (...) {
                  Cdmw::Common::String::delete_char_array(argv__);
                  throw;
               }
               
               Cdmw::Common::String::delete_char_array(argv__);
               
            }

        }
        void
        TestOSDriver::do_tests()
        {
            if (!m_do_test)
                return;
            
            // set number of requested successfull tests
//             set_nbOfRequestedTestOK (24);
            int timescale = Cdmw::TestUtils::get_timescale();


            // TODO: finding how to better test the returned date.
            TEST_INFO("Trying to get the time");

            try
            {
                OS::Timeval timeval = OS::get_time();
                TEST_INFO("Seconds are     : " << timeval.seconds);
                TEST_INFO("Microseconds are: " << timeval.microseconds);
                CPPUNIT_ASSERT( timeval.seconds > 1000000000 );
            }
            catch ( ...)
            {
                CPPUNIT_ASSERT(false);
            }

            TEST_INFO("Try to compare two times");
            try
            {
                OS::Timeval timeval1 = OS::get_time();
                OS::sleep(timescale*1000);
                OS::Timeval timeval2 = OS::get_time();
                CPPUNIT_ASSERT( timeval2 > timeval1 );
                CPPUNIT_ASSERT( timeval1 < timeval2 );
                CPPUNIT_ASSERT( !(timeval2 == timeval1) );
            }
            catch ( ...)
            {
                CPPUNIT_ASSERT(false);
            }

            TEST_INFO("Trying to get the hostname");
            try
            {
                std::string hostname = OS::get_hostname();
                TEST_INFO( hostname );
                CPPUNIT_ASSERT(true);
            }
            catch (...)
            {
                CPPUNIT_ASSERT(false);
            }

            TEST_INFO("Trying to get the user name");
            try
            {
                std::string logname = OS::get_logname();
                TEST_INFO( logname );
                CPPUNIT_ASSERT(true);
            }
            catch (...)
            {
                CPPUNIT_ASSERT(false);
            }

            std::auto_ptr<OS::SharedObjectHandle> handle;
            std::auto_ptr<OS::SharedObjectHandle> handle2;
            std::auto_ptr<OS::SharedObjectHandle> handle3;
#ifdef AIX

            std::string libraryName = "libc.a(shr.o)";
            std::string FullPathLibraryName = "/usr/lib/libc.a(shr.o)";
            std::string functionName = "atoi";
            int dlopenFlags = RTLD_MEMBER | RTLD_LAZY;
#else

            std::string libraryName = "libm.so";
            std::string FullPathLibraryName = "/usr/lib/libm.so";
            std::string functionName = "cos";
            int dlopenFlags = RTLD_LAZY;
#endif



            TEST_INFO("Trying to load a non-existent dynamic library");

            try
            {
                std::auto_ptr<OS::SharedObjectHandle> handle_tmp(OS::dlopen("libmazeazerazreza.so"));
                handle = handle_tmp;
                CPPUNIT_ASSERT (handle->is_null());
            }
            catch ( ...)
            {
                CPPUNIT_ASSERT(false);
            }


            TEST_INFO("Trying to load a non-loadable dynamic library");

            try
            {
                std::auto_ptr<OS::SharedObjectHandle> handle_tmp(OS::dlopen("/usr/lib/libc.so"));
                handle = handle_tmp;
                CPPUNIT_ASSERT (handle->is_null());
            }
            catch ( ...)
            {
                CPPUNIT_ASSERT(false);
            }


            TEST_INFO("Trying to load a dynamic library");

            try
            {
                std::auto_ptr<OS::SharedObjectHandle> handle_tmp(OS::dlopen(libraryName.c_str(), dlopenFlags));
                handle = handle_tmp;
                CPPUNIT_ASSERT (!handle->is_null());
            }
            catch ( ...)
            {
                CPPUNIT_ASSERT(false);
            }


            TEST_INFO("Trying to load a dynamic library (second time in an other handle)");

            try
            {
                std::auto_ptr<OS::SharedObjectHandle> handle_tmp(OS::dlopen(libraryName.c_str(), dlopenFlags));
                handle2 = handle_tmp;
                CPPUNIT_ASSERT (!handle2->is_null());
            }
            catch ( ...)
            {
                CPPUNIT_ASSERT(false);
            }

            TEST_INFO("Trying to execute a function via a dynamic library");

            try
            {
                typedef double (*fun_t)(double);
                fun_t fun = NULL;
                fun = (fun_t)OS::dlsym(handle.get(), functionName.c_str());
                CPPUNIT_ASSERT (fun != NULL);
                std::cout << "cos result :" << (*fun)(3.1415927 / 4) << std::endl;
            }
            catch ( ...)
            {
                CPPUNIT_ASSERT(false);
            }


            TEST_INFO("Trying to execute a non-existent function via a dynamic library");

            try
            {
                typedef double (*fun_t)(double);
                fun_t fun2 = NULL;
                fun2 = (fun_t) OS::dlsym(handle.get(), "cosjhjkhjk");

                if (fun2 == NULL)
                {
                    CPPUNIT_ASSERT (OS::dlerror() != "");
                }
            }
            catch ( ...)
            {
                CPPUNIT_ASSERT(false);
            }

            TEST_INFO("Trying to unload a dynamic library");

            try
            {
                CPPUNIT_ASSERT (OS::dlclose(handle.get()) == 0);
            }
            catch ( ...)
            {
                CPPUNIT_ASSERT(false);
            }

            TEST_INFO("Trying to execute a function via a dynamic library already closed");

            try
            {
                typedef double (*fun_t)(double);
                fun_t fun = NULL;
                fun = (fun_t)OS::dlsym(handle.get(), functionName.c_str());
                CPPUNIT_ASSERT (fun != NULL);
                std::cout << "cos result :" << (*fun)(3.1415927 / 2) << std::endl;
            }
            catch ( ...)
            {
                CPPUNIT_ASSERT(false);
            }

            TEST_INFO("Trying to unload a dynamic library before unloaded (but open 2 times)");

            try
            {
                CPPUNIT_ASSERT (OS::dlclose(handle.get()) == 0 );
            }
            catch ( ...)
            {
                CPPUNIT_ASSERT(false);
            }



            TEST_INFO("Trying to unload a dynamic library before unloaded (reference counter = 0)");

            try
            {

                CPPUNIT_ASSERT (OS::dlclose(handle.get()) != 0 );
            }
            catch ( ...)
            {
                CPPUNIT_ASSERT(false);
            }

            TEST_INFO("Trying to unload a dynamic library before unloaded (second handle and reference counter = 0)");

            try
            {
                CPPUNIT_ASSERT (OS::dlclose(handle2.get()) != 0 );
            }
            catch ( ...)
            {
                CPPUNIT_ASSERT(false);
            }


            TEST_INFO("Trying to load all functions by a dlopen(NULL)");

            try
            {
                std::auto_ptr<OS::SharedObjectHandle> handle_tmp(OS::dlopen(NULL, RTLD_LAZY | RTLD_GLOBAL));
                handle = handle_tmp;
                CPPUNIT_ASSERT (!handle->is_null());
            }
            catch ( ...)
            {
                CPPUNIT_ASSERT(false);
            }

            TEST_INFO("Trying to load all functions by a dlopen(NULL) (second time)");

            try
            {
                std::auto_ptr<OS::SharedObjectHandle> handle_tmp(OS::dlopen(NULL, RTLD_LAZY | RTLD_GLOBAL));
                handle = handle_tmp;
                CPPUNIT_ASSERT (!handle->is_null());
            }
            catch ( ...)
            {
                CPPUNIT_ASSERT(false);
            }

            TEST_INFO("Trying to execute a function via the previous");

            try
            {

                typedef double (*fun_t)(double);
                fun_t fun = NULL;
                fun = (fun_t)OS::dlsym(handle.get(), "test");
                CPPUNIT_ASSERT (fun(3) == 3);
            }
            catch ( ...)
            {
                CPPUNIT_ASSERT(false);
            }

            TEST_INFO("Trying to unload a dynamic library loaded with  dlopen(NULL)");

            try
            {
                CPPUNIT_ASSERT (OS::dlclose(handle.get()) == 0 );
            }
            catch ( ...)
            {
                CPPUNIT_ASSERT(false);
            }


            TEST_INFO("Trying to execute a function via the previous");

            try
            {

                typedef double (*fun_t)(double);
                fun_t fun = NULL;
                fun = (fun_t)OS::dlsym(handle.get(), "test");
                CPPUNIT_ASSERT (fun(3) == 3);
            }
            catch ( ...)
            {
                CPPUNIT_ASSERT(false);
            }

            TEST_INFO("Trying to unload a dynamic library loaded with  dlopen(NULL)(ref counter = 1)");

            try
            {
                CPPUNIT_ASSERT (OS::dlclose(handle.get()) == 0 );
            }
            catch ( ...)
            {
                CPPUNIT_ASSERT(false);
            }

            TEST_INFO("Trying to load a dynamic library with a full path");

            try
            {
                std::auto_ptr<OS::SharedObjectHandle> handle_tmp(OS::dlopen(FullPathLibraryName.c_str(), dlopenFlags));
                handle3 = handle_tmp;
                CPPUNIT_ASSERT (!handle3->is_null());
            }
            catch ( ...)
            {
                CPPUNIT_ASSERT(false);
            }


            TEST_INFO("Trying to unload a dynamic library (fullpath)");

            try
            {

                CPPUNIT_ASSERT (OS::dlclose(handle3.get()) == 0 );
            }
            catch ( ...)
            {
                CPPUNIT_ASSERT(false);
            }

        }

        void
        TestOSDriver::test_basename()
        {
            TEST_INFO("Trying to test basename");

            // examples taken from dirname/basename man page
            CPPUNIT_ASSERT(OS::basename("/usr/lib")  == "lib");
            CPPUNIT_ASSERT(OS::basename("/usr/")  == "usr");
            CPPUNIT_ASSERT(OS::basename("usr")  == "usr");
            CPPUNIT_ASSERT(OS::basename("/")  == "/");
            CPPUNIT_ASSERT(OS::basename(".")  == ".");
            CPPUNIT_ASSERT(OS::basename("..")  == "..");

            // test that glibc bug http://linuxtesting.org/results/report?num=S0207
            // is overcome / fixed. The test strings have been taken from the bug
            // description. 
            CPPUNIT_ASSERT(OS::basename("err////")  == "err");
            CPPUNIT_ASSERT(OS::basename("er//")  == "er");
            CPPUNIT_ASSERT(OS::basename("////")  == "/");
            CPPUNIT_ASSERT(OS::basename("//")  == "/");
        }

        void
        TestOSDriver::test_dirname()
        {
            TEST_INFO("Trying to test dirname");

            // examples taken from dirname/basename man page
            CPPUNIT_ASSERT(OS::dirname("/usr/lib")  == "/usr");
            CPPUNIT_ASSERT(OS::dirname("/usr/")  == "/");
            CPPUNIT_ASSERT(OS::dirname("usr")  == ".");
            CPPUNIT_ASSERT(OS::dirname("/")  == "/");
            CPPUNIT_ASSERT(OS::dirname(".")  == ".");
            CPPUNIT_ASSERT(OS::dirname("..")  == ".");


            CPPUNIT_ASSERT(OS::dirname("err////")  == ".");
            CPPUNIT_ASSERT(OS::dirname("er//")  == ".");
            CPPUNIT_ASSERT(OS::dirname("////")  == "/");

            // this seems to be a buggy dirname implementation,
            // give more output to speed up analisys if smd. else runs
            // this test.
      
            std::string buggyInput = "//";
              
            TEST_INFO("Trying to dirname  " << buggyInput << " this gave " << OS::dirname(buggyInput) << " This should have been /"); 

            CPPUNIT_ASSERT(OS::dirname(buggyInput)  == "/");
        }
        
    } // End namespace OsSupport
} // End namespace Cdmw
