/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "testossupport/TestOSProcessDriver.hpp"

#include <cstdio>
#include <fstream>
#include <unistd.h>
#include <cstdlib>
#include <string>
#include <sstream>
#include <iostream>
#include <Foundation/common/System.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <sys/time.h>
#include <fstream>
#include "Foundation/common/MacroDefs.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/ossupport/EndingCallback.hpp"

#include "Foundation/common/Assert.hpp"
#include "Foundation/osthreads/MutexGuard.hpp"
#include "Foundation/osthreads/Condition.hpp"

namespace Cdmw
{

    namespace OsSupport
    {

        /**
        *Implementation notes: [if no pertinent write none ]
        *<p>
        *Portability issues: [if no pertinent write none ]
        *<p>
        */
/*
        extern "C"
        {
            double test(double r)
            {
                return r;
            };
        }
*/

        class TestProcessCallback : public EndingCallback
        {

            public:

                TestProcessCallback()
                        : m_processId(0)
                {}

                virtual ~TestProcessCallback()
                { }

                OS::ProcessId getProcessId()
                {
                    return m_processId;
                }
                
                void execute(OsSupport::OS::ProcessId processId) throw()
                {
                    m_processId = processId;
                }

            private:

                OS::ProcessId m_processId;

        };


    /**
     * This EndingCallback is used to block a thread while a process is not finished.
     */
    class BlockingProcessCallback : public Cdmw::OsSupport::ProcessCallback {
        
        public:
            BlockingProcessCallback() : m_processEnded(false), m_condition(m_mutex) {};

            virtual ~BlockingProcessCallback() {};
            
            virtual void execute(Cdmw::OsSupport::OS::ProcessId processId) throw() {
                CDMW_MUTEX_GUARD(m_mutex);
                m_processEnded = true;
                try {
                    m_condition.broadcast();
                } catch (const Cdmw::InternalErrorException& ex) {
                    CDMW_ASSERT(false);
                }
            };
    
            void wait_callback() throw() {
                CDMW_MUTEX_GUARD(m_mutex);
                try {
                    while (!m_processEnded) m_condition.wait();
                } catch (const Cdmw::InternalErrorException& ex) {
                    CDMW_ASSERT(false);
                }
            };
    
            bool is_process_ended() {
                CDMW_MUTEX_GUARD(m_mutex);
                return m_processEnded;
            };
            
        private:
            bool m_processEnded;
            Cdmw::OsSupport::Mutex m_mutex;
            Cdmw::OsSupport::Condition m_condition;
    };
    

        CPPUNIT_TEST_SUITE_REGISTRATION( TestOSProcessDriver );



        TestOSProcessDriver::TestOSProcessDriver(  )
        {
            m_processName = Cdmw::TestUtils::TestManager::instance()->get_property ("process");
            
            if (Cdmw::TestUtils::TestManager::instance()->get_property ("valid") == "yes") 
                m_valid_only = true;
            else
                m_valid_only = false;
            
            if (Cdmw::TestUtils::TestManager::instance()->get_property ("test") == "Process" 
                || Cdmw::TestUtils::TestManager::instance()->get_property ("test") == "all") 
                m_do_test = true;
            else
                m_do_test = false;
        }



        TestOSProcessDriver::~TestOSProcessDriver()
        throw()
        {
        }


        void
        TestOSProcessDriver::do_tests()
        {
            if (!m_do_test)
                return;
            int timescale = Cdmw::TestUtils::get_timescale(); 

            {   
                OS::sleep(timescale*2000);
        
                if( OS::file_exists("Process_env.dat") )
                    OS::unlink("Process_env.dat");
                        
                TEST_INFO("Checking PROCESS CREATION specifying ENVIROMENT VARIABLES");
                TEST_INFO("TEST 1 : Using three different enviroment variables");                
                 
                std::string process_data="Process_env.dat";
                                               
                std::string variable1="FIRST_VARIABLE";
                std::string variable2="SECOND_VARIABLE";
                std::string variable3="THIRD_VARIABLE";                
                                                     
                std::string value1="FIRST_VALUE";
                std::string value2="SECOND_VALUE";
                std::string value3="THIRD_VALUE"; 
                                            
                std::vector<std::string> p_env;                
                p_env.push_back(variable1+"="+value1);
                p_env.push_back(variable2+"="+value2);
                p_env.push_back(variable3+"="+value3);               
               
                std::string p_args=variable1+" "+variable2+" "+variable3; 
                
                TestProcessCallback creationCallback1;
                TestProcessCallback creationCallback2;
                // warning : the ending callback is created on the stack in this basic
                // test. A normal use requires that the ending callback survives
                // until the ending is notified asynchronously.
                TestProcessCallback endingCallback1;
                TestProcessCallback endingCallback2;
                
                OS::ProcessId pid_created1;
                OS::ProcessId pid_created2;
         
                try
        {
                    pid_created1 = OS::create_process("process_env",p_args,p_env,".",&creationCallback1, &endingCallback1);
                    OS::sleep(timescale*9000);                    
                }
                catch(...)
                {
                    std::cerr<< "\nError in process creation !";
                    TEST_INFO("PROCESS CREATION FAILED");
                    CPPUNIT_ASSERT(false); 
                }                 
                              
                 TEST_INFO("Checking if we have been notified of the process creation");
                 CPPUNIT_ASSERT( pid_created1 == creationCallback1.getProcessId());

                 TEST_INFO("Checking if we have been notified of the process termination");
                 CPPUNIT_ASSERT( pid_created1 == endingCallback1.getProcessId());  
                 TEST_INFO("Checking the file_data has been created by process");
                 CPPUNIT_ASSERT( OS::file_exists(process_data) );
                    
                    ifstream i_file(process_data.c_str());
                    std::string f;
                    unsigned int i=0;
                                                           
                    if(i_file)
                    {                    
                        while (std::getline(i_file,f))
                        {                                                   
                            switch (i)
                            {
                                case (2) :
                                    TEST_INFO("Checking the values saved in the file process_data ");
                                    CPPUNIT_ASSERT( f==value3 );
                                    break;
                                case (1) :
                                    TEST_INFO("Checking the values saved in the file process_data ");
                                    CPPUNIT_ASSERT( f==value2 );
                                    i++;
                                    break;
                                case (0) :
                                    TEST_INFO("Checking the values saved in the file process_data ");
                                    CPPUNIT_ASSERT( f==value1 );
                                    i++;
                                    break;                                                  
                            }                                                                        
                        }   
                        TEST_INFO("Check if there are any variables missing in process_data");
                        CPPUNIT_ASSERT( p_env.size()==(i+1) );
                        i_file.close();     
                    }
                    else
                    {
                        std::cerr<<"\nERROR READING "<< process_data;
                        CPPUNIT_ASSERT(false);
                    } 
               try
               {    
                   OS::unlink(process_data.c_str());
               }
               catch(...)
               {
                   std::cerr<<"ERROR DELETING "<<process_data;
               }
              
              try
              {
                 
                  TEST_INFO("TEST 2: Using a null variables vector");                  
                  std::vector<std::string> p_env2;
                  pid_created2 = OS::create_process("process_env","",p_env2,".",&creationCallback2, &endingCallback2);
                  OS::sleep(timescale*7000);
                  TEST_INFO("Checking the process creation with a NULL variables vector");
                  CPPUNIT_ASSERT(true);
                                                          
               }
               catch(...)
               {
                CPPUNIT_ASSERT(false);
               } 
                 
                 
                 TEST_INFO("Checking if we have been notified of the process creation");
                 CPPUNIT_ASSERT( pid_created2 == creationCallback2.getProcessId());

                 TEST_INFO("Checking if we have been notified of the process termination");
                 CPPUNIT_ASSERT( pid_created2 == endingCallback2.getProcessId());
                 
                 TEST_INFO("Checking the file_data has been created by process");
                 CPPUNIT_ASSERT( OS::file_exists(process_data) );  
                           
                 TEST_INFO("Checking if the file process_data is correctly empty");
                 CPPUNIT_ASSERT( !std::getline(i_file,f) );                
               
               try
               {    
                   OS::unlink(process_data.c_str());
               }
               catch(...)
               {
                   std::cerr<<"ERROR DELETING "<< process_data;
               }        
            
                
           }  
         
        {   TEST_INFO("Test death of many process at the same time");
            int nb_process = 10;
            
            typedef std::map<Cdmw::OsSupport::OS::ProcessId, TestProcessCallback*> CallbackMap;
            CallbackMap creationCBs, endingCBs;
            
            for (int i=0; i < nb_process; ++i) {
                TestProcessCallback* creationCB = new TestProcessCallback();
                TestProcessCallback* endingCB = new TestProcessCallback();
                
                std::string launcherPath = "/bin/sleep";
                std::string launcherArgs = "30";
                std::string home = ".";

                Cdmw::OsSupport::OS::ProcessId pid =
                    Cdmw::OsSupport::OS::create_process(
                        launcherPath,
                        launcherArgs,
                        home,
                        creationCB,
                        endingCB);
                   
                creationCBs[pid] = creationCB;
                endingCBs[pid] = endingCB;
                        
                TEST_INFO("   process " << i << " created. PID=" << pid);
            }
            
            TEST_INFO("   check creation of all processes");
            for (CallbackMap::iterator it = creationCBs.begin();
                 it != creationCBs.end();
                 it++)
            {
                if (it->first != it->second->getProcessId()) {
                    TEST_INFO("      PID " << it->first << " not created !! "
                              << "getProcessId()=" << it->second->getProcessId());
                    CPPUNIT_ASSERT(false); 
                }
            }

            TEST_INFO("   killing all processes");
            TestProcessCallback* killCreationCB = new TestProcessCallback();
            BlockingProcessCallback* killEndingCB = new BlockingProcessCallback();
            std::string launcherPath = "/usr/bin/kill";
            std::ostringstream launcherArgs;
            launcherArgs << "-9";
            for (CallbackMap::iterator it = creationCBs.begin();
                 it != creationCBs.end();
                 it++)
            {
                launcherArgs << " " << it->first;
            }            
            std::string home = ".";
            TEST_INFO("       kill " << launcherArgs.str());
            
            Cdmw::OsSupport::OS::ProcessId killPid =
                Cdmw::OsSupport::OS::create_process(
                    launcherPath,
                    launcherArgs.str(),
                    home,
                    killCreationCB,
                    killEndingCB);
            TEST_INFO("   process kill created. PID=" << killPid);
            
            
            TEST_INFO("   wait kill to complete");
            killEndingCB->wait_callback();
            Cdmw::OsSupport::OS::sleep(1000);
            delete killEndingCB;
            delete killCreationCB;
            
            TEST_INFO("   check ending callbacks");
            for (CallbackMap::iterator it = endingCBs.begin();
                 it != endingCBs.end();
                 it++)
            {
                if (it->first != it->second->getProcessId()) {
                    TEST_INFO("      PID " << it->first << " not ended !! "
                              << "getProcessId()=" << it->second->getProcessId());
                    CPPUNIT_ASSERT(false);
                }
                TEST_INFO("----Test exit_code OK");
                TEST_INFO("getProcessId()=" << it->second->getProcessId());
                if(dynamic_cast<EndingCallback*> (it->second)) {
                    TEST_INFO("get_exit_code()="<<it->second->get_exit_code());
                    delete it->second;
                }
            }
            for (CallbackMap::iterator it = creationCBs.begin();
                 it != creationCBs.end();
                 it++)
                 
            {
                delete it->second;
            }
            
            TEST_INFO("Test death of many process OK");
        }
        
        

            // set number of requested successfull tests
//             if (!m_valid_only) set_nbOfRequestedTestOK(17);
//             else set_nbOfRequestedTestOK(3); 

            //int timescale = Cdmw::TestUtils::get_timescale();

            // This part is only for unit test
            if (!m_valid_only)
            {
                TEST_INFO("Remove a temporary file name");
                std::string tmp_file_str = "Cdmw";
                ::unlink(tmp_file_str.c_str());

                TEST_INFO("Checking this file not exists");
                CPPUNIT_ASSERT( ! OS::file_exists(tmp_file_str) );

                // Process Creation
            
                TEST_INFO("Creating a new process");

                TestProcessCallback creationCallback;

                // warning : the ending callback is created on the stack in this basic
                // test. A normal use requires that the ending callback survives
                // until the ending is notified asynchronously.
                TestProcessCallback endingCallback;

                OS::ProcessId pid_created = OS::create_process(m_processName, "--create-file=" + tmp_file_str,
                                                               "", &creationCallback, &endingCallback);

                // We sleep a while to wait the process and file creation
                OS::sleep(timescale*4000);

                TEST_INFO("Checking if we have been notified of the process creation");
                CPPUNIT_ASSERT( pid_created == creationCallback.getProcessId());

                TEST_INFO("Checking if we have been notified of the process termination");
                CPPUNIT_ASSERT( pid_created == endingCallback.getProcessId());

                TEST_INFO("Checking the file has been created by process");
                CPPUNIT_ASSERT( OS::file_exists(tmp_file_str) );

                {
                    TEST_INFO("Checking if a process can retreive its own process id");
                    std::ifstream pidfile(tmp_file_str.c_str());
                    OS::ProcessId pid_retreived;
                    pidfile >> pid_retreived;
                    pidfile.close();
                    CPPUNIT_ASSERT( pid_created == pid_retreived);
                }

                TEST_INFO("Trying to execute a non executable file");

                try
                {
                    OS::create_process(tmp_file_str, "");

                }
                catch ( const BadParameterException& )
                {

                    CPPUNIT_ASSERT(true);

                }
                catch (...)
                {

                    CPPUNIT_ASSERT(false);
                }

                TEST_INFO("Trying to delete an existing file");

                try
                {
                    OS::unlink(tmp_file_str);
                    CPPUNIT_ASSERT( ! OS::file_exists(tmp_file_str) );

                }
                catch ( ... )
                {

                    CPPUNIT_ASSERT(false);

                }

                TEST_INFO("Trying to delete an unexisting file");

                try
                {
                    OS::unlink(tmp_file_str);
                    CPPUNIT_ASSERT(false);

                }
                catch ( const BadParameterException& )
                {

                    CPPUNIT_ASSERT(true);

                }


                // Process Creation with working directory and set with PATH environment

                TEST_INFO("Creating a new process defined by PATH and with working directory");
                TEST_INFO(m_processName.c_str());

                TestProcessCallback creationCallback2;

                // warning : the ending callback is created on the stack in this basic
                // test. A normal use requires that the ending callback survives
                // until the ending is notified asynchronously.
                TestProcessCallback endingCallback2;

                OS::ProcessId pid_created2 = OS::create_process("test_process", "--create-file=" + tmp_file_str,
                                                                ".", &creationCallback2, &endingCallback2);

                // We sleep a while to wait the process and file creation
                OS::sleep(timescale*4000);

                TEST_INFO("Checking if we have been notified of the process creation");
                CPPUNIT_ASSERT( pid_created2 == creationCallback2.getProcessId());

                TEST_INFO("Checking if we have been notified of the process termination");
                CPPUNIT_ASSERT( pid_created2 == endingCallback2.getProcessId());

                TEST_INFO("Checking the file has been created by process");
                CPPUNIT_ASSERT( OS::file_exists(tmp_file_str) );
            
            
                // delete the created file
                try
                {
                    OS::unlink(tmp_file_str);
                }
                catch ( ... )
                {
                }
// OK until here        
            
                // Process Creation with not existing working directory

                OS::sleep(timescale*5000);
                TEST_INFO("Creating a new process with not existing working directory");

                TestProcessCallback creationCallback3;

                // warning : the ending callback is created on the stack in this basic
                // test. A normal use requires that the ending callback survives
                // until the ending is notified asynchronously.
                TestProcessCallback endingCallback3;

                OS::ProcessId pid_created3 = OS::create_process(m_processName, "--create-file=" + tmp_file_str,
                                                                "/notexistdir", &creationCallback3, &endingCallback3);

                // We sleep a while to wait the process and file creation
                OS::sleep(timescale*4000);

                TEST_INFO("Checking if we have been notified of the process creation");
                CPPUNIT_ASSERT( pid_created3 == creationCallback3.getProcessId());

                TEST_INFO("Checking if we have been notified of the process termination");
                CPPUNIT_ASSERT( pid_created3 == endingCallback3.getProcessId());

                // The process has not started effectiveley in fact (exec has not been called)
                // Actually, the fork() was successful, but a _exit() is performed
                // before exec(). This is preferable to simple exit() since 
                // we don't know what static destructors can perform 
                // (cleaning in shared memory...) 
                // This leads to not deleted memory in child process,
                // since static destructors haven't been called.
                // THUS VALGRIND MEMORY LEAK REPORTS ON CHILD PROCESS ARE NORMAL.
                //
                TEST_INFO("Checking the file is not created by process");
                CPPUNIT_ASSERT( !OS::file_exists(tmp_file_str) );            
            

                try
                {
                    OS::unlink(tmp_file_str);
                }
                catch ( ... )
                {
                }
            }
            
            // Test of kill process

            TestProcessCallback creationCallback4;
            // warning : the ending callback is created on the stack in this basic
            // test. A normal use requires that the ending callback survives
            // until the ending is notified asynchronously.
            TestProcessCallback endingCallback4;

            OS::ProcessId pid_created4 = OS::create_process("test_process", "--sleep=9",
                                        ".", &creationCallback4, &endingCallback4);

            // We sleep a while to wait the process
            // (for this test it must be significatly < --sleep * 1000 *timescale )
            OS::sleep(timescale*4000);

            TEST_INFO("Checking if we have been notified of the process creation");
            CPPUNIT_ASSERT( pid_created4 == creationCallback4.getProcessId());
            TEST_INFO("Checking that we have not been notified of the process termination");
            CPPUNIT_ASSERT( pid_created4 != endingCallback4.getProcessId());
            TEST_INFO("Perform a kill on the created process");
            OS::kill_process( pid_created4 );
            OS::sleep(timescale*500);
            TEST_INFO("Checking if we have been notified of the process termination");
            CPPUNIT_ASSERT( pid_created4 == endingCallback4.getProcessId());
        }


    } // End namespace OsSupport
} // End namespace Cdmw

