/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ossupport.test;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.assertTrue;

import cdmw.ossupport.Barrier;
import cdmw.ossupport.OS;
import cdmw.testutils.TestManager;

public class BarrierTest {

    private static boolean validOnly;
    private static int timescale;
    
    @BeforeClass
    public static void initTests() {
        TestManager testManager = TestManager.instance();
        
        validOnly = Boolean.parseBoolean(
            testManager.getProperty("OSS_SUPPORT_TEST_VALID_ONLY"));
        timescale = TestManager.getTimescale();
        
        System.out.println("===================================================");
        System.out.println("Starting test " + BarrierTest.class.getName());
        System.out.println("===================================================");
    }
    
    @AfterClass
    public static void finiTests() {
        System.out.println("===================================================");
        System.out.println("Ending test " + BarrierTest.class.getName());
        System.out.println("===================================================");        
    }

    @Test
    public void doTimedOutTests()
        throws InterruptedException {

        if (validOnly) {
            return;
        }
        
        System.out.println("Creating a barrier for 3 threads");
        Barrier b = new Barrier(3);

        System.out.println("Creating 1 thread waiting on the barrier");
        System.out.println("   with a value to be changed ");
        int val1 = 0;
        BarrierThread bt1 = new BarrierThread(b, val1);

        System.out.println("Starting the thread");
        bt1.start();

        System.out.println("Waiting on the Barrier with a timeout");
        int status = b.waitForAll(500);
        assertTrue( status == Barrier.TIMEDOUT );

        System.out.println("Creating a second thread waiting on the barrier");
        System.out.println("   with a value to be changed ");
        int val2 = 0;
        BarrierThread bt2 = new BarrierThread(b, val2);
        System.out.println("Starting the thread");
        bt2.start();

        System.out.println("Sleeping a while to see if all thread have gone across the Barrier");
        OS.sleep(timescale * 2000);
        assertTrue(bt1.getVal() == 999 && bt2.getVal() == 999);

        bt1.join();
        bt2.join();
    }

    @Test
    public void doBasicTests()
        throws InterruptedException {

        System.out.println("Creating a barrier for 3 threads");
        Barrier b = new Barrier(3);

        System.out.println("Creating 2 threads waiting on the barrier");
        System.out.println("   with a value to be changed for each one");
        int val1 = 0;
        int val2 = 0;
        BarrierThread bt1 = new BarrierThread(b, val1);
        BarrierThread bt2 = new BarrierThread(b, val2);

        System.out.println("Starting all threads");
        bt1.start();
        bt2.start();

        System.out.println("Sleeping a while to see if threads are blocked by the Barrier");
        OS.sleep(timescale * 2000);
        assertTrue(bt1.getVal() == 0 && bt2.getVal() == 0);

        System.out.println("Waiting on the Barrier and see if we unblocks the Barrier");
        b.waitForAll();
        // Threads unblocked

        System.out.println("Sleeping a while to see if all thread have gone across the Barrier");
        OS.sleep(timescale * 2000);
        assertTrue( bt1.getVal() == 999 && bt2.getVal() == 999);

        bt1.join();
        bt2.join();
    }
}

