/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ossupport.test;

import java.io.File;
import java.io.IOException;

import cdmw.testutils.TestManager;
import cdmw.ossupport.OS;

public class MainTest {

    public static void printHelp() {

        System.out.println("Cdmw OsSupport Unit testing");
        System.out.println(" options are:");
        System.out.println("   --help:           print this message");
        System.out.println("   --test-all:       test all the CSC");
        System.out.println("   --test-OS:        test the OS class");
        System.out.println("   --test-Mutex:     test the Mutex class");
        System.out.println("   --test-Lock:      test the ReadWriteLock class");
        System.out.println("   --test-Barrier:   test the Barrier class");

    }

    public static void main(String[] args) {

        boolean all = false;
        boolean one = false;
        boolean test = false;

        // Creation of a file to test (needed for the other tests)
        String tmpFile = OS.getOptionValue(args,"--create-file");
        if ( !tmpFile.equals("no") ) {
            File file = new File(tmpFile);
            try {
                if (file.createNewFile()) {
                    System.out.println("File " + file + " created successfully.");
                } else {
                    System.out.println("Unable to create the file " +
                        file +": the named file already exists.");
                }
            } catch(IOException ioe) {
                System.out.println("Unable to create the file " +
                    file + ": " + ioe.getMessage());
            }
            one = true;
        }

        TestManager testManager = TestManager.instance();
        
        testManager.init(MainTest.class.getName());
        
        // Sleep
        String sleep = OS.getOptionValue(args,"--sleep");
        if ( !sleep.equals("no") ) {
            try {
                int time = TestManager.getTimescale() * Integer.parseInt(sleep);
                OS.sleep(time * 1000);
                System.out.println("Sleep of " + time + " successful.");
            } catch(InterruptedException ie) {
                System.out.println("Unable to sleep process: " +
                    ie.getMessage() );
            } catch(NumberFormatException nfe) {
                System.out.println("Unable to sleep process: " +
                    nfe.getMessage() );
            }
            one = true;
        }

        if ( OS.getOptionValue(args,"--valid-only").equals("yes") ) {
            testManager.setProperty("OSS_SUPPORT_TEST_VALID_ONLY", Boolean.TRUE.toString());
        }
        else {
            testManager.setProperty("OSS_SUPPORT_TEST_VALID_ONLY", Boolean.FALSE.toString());
        }
        
        // No arguments we do all the test
        if ( args.length == 0 ) {
            all = true;
        }

        // --test-all supersedes all other options
        if ( OS.getOptionValue(args,"--test-all").equals("yes") ) {
            all = true;
        }

        // Testing the OS support functions
        if ( all || OS.getOptionValue(args,"--test-OS").equals("yes") ) {
            testManager.addTest(OSTest.class);
            one = true;
            test = true;
        }
        
        // Testing the Mutex object
        if ( all || OS.getOptionValue(args,"--test-Mutex").equals("yes") ) {
            testManager.addTest(MutexTest.class);
            one = true;
            test = true;
        }

        // Testing the ReadWriteLock object
        if ( all || OS.getOptionValue(args,"--test-Lock").equals("yes") ) {
            testManager.addTest(ReadWriteLockTest.class);
            one = true;
            test = true;
        }

        // Testing the Barrier object
        if ( all || OS.getOptionValue(args,"--test-Barrier").equals("yes") ) {
            testManager.addTest(BarrierTest.class);
            one = true;
            test = true;
        }
        
        // Help message
        if ( !one || OS.getOptionValue(args,"--help").equals("yes") ) {
            printHelp();
        } else if (test || all) {
            testManager.runTests();
            testManager.printSummary();            
        }
    }

}

