/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ossupport.test;

import java.util.Vector;
import java.util.Enumeration;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import static org.junit.Assert.assertTrue;

import cdmw.ossupport.ReadWriteLock;

import cdmw.ossupport.OS;
import cdmw.testutils.TestManager;

public class ReadWriteLockTest {
    
    private static boolean validOnly;
    private static int timescale;
    
    @BeforeClass
    public static void initTests() {
        TestManager testManager = TestManager.instance();
        
        validOnly = Boolean.parseBoolean(
            testManager.getProperty("OSS_SUPPORT_TEST_VALID_ONLY"));
        timescale = TestManager.getTimescale();
        
        System.out.println("===================================================");
        System.out.println("Starting test " + ReadWriteLockTest.class.getName());
        System.out.println("===================================================");        
    }
    
    @AfterClass
    public static void finiTests() {
        System.out.println("===================================================");
        System.out.println("Ending test " + ReadWriteLockTest.class.getName());
        System.out.println("===================================================");        
    }

    @Test
    public void doBasicTests()
        throws InterruptedException {

        System.out.println("Creating the ReadWriteLock");
        ReadWriteLock lock = new ReadWriteLock();
        // ReadWriteLock created successfully.

        System.out.println("Creating the value to be protected");
        int val = 0;

        BasicReaderThread brt = new BasicReaderThread(lock, val);
        BasicWriterThread bwt = new BasicWriterThread(lock, val);

        System.out.println("Locking as a reader");
        lock.readLock();
        // Locked successfully

        System.out.println("Starting the Writer thread");
        bwt.start();

        System.out.println("Sleeping a while to see if the writer change the variable");
        OS.sleep(timescale * 2000);
        assertTrue(bwt.getVal() == 0);

        System.out.println("Starting the Reader thread");
        brt.start();
        System.out.println("Checking if the Reader thread has not been blocked by the lock");
        brt.join();
        // The Reader has not been blocked

        System.out.println("Unlocking the lock");
        lock.readUnlock();

        System.out.println("Sleeping a while to see if the writer change the variable");
        OS.sleep(timescale * 2000);
        assertTrue(bwt.getVal() != 0);
        bwt.join();
    }

    @Test
    public void doConcurrentTests()
        throws InterruptedException {

        if (validOnly) {
            return;
        }
        
        // The time max in each thread is 3s * NB_TEST_LOCKS
        //    as each thread sleep between 0 and 3s
        int nbTestLocks = 5;

        System.out.println("Creating the ReadWriteLock");
        ReadWriteLock lock = new ReadWriteLock();
        // ReadWriteLock created successfully.

        System.out.println("Creating the object protected by the lock");
        ReadWriteLockThread rwlt = new ReadWriteLockThread(System.out);
        rwlt.start();

        System.out.println("Creating the Two writer threads");
        Vector threads = new Vector();

        for(int iWriterThread=0; iWriterThread < 50; ++iWriterThread) {
            threads.add( new WriterThread(lock,rwlt,nbTestLocks) );
        }

        for(int iReaderThread=0; iReaderThread < 8; ++iReaderThread) {
            threads.add( new ReaderThread(lock,rwlt,nbTestLocks) );
        }

        System.out.println("Starting all threads");
        for (Enumeration e = threads.elements() ; e.hasMoreElements() ;) {
            ( (Thread)e.nextElement() ).start();
         }

         System.out.println("Waiting for the completion of all threads");
         for (Enumeration e = threads.elements() ; e.hasMoreElements() ;) {
            ( (Thread)e.nextElement() ).join();
         }

        rwlt.join();

        // Concurrent tests successful
    }
}

