/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
 
#include "Foundation/testutils/TestManager.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/ossupport/SystemProperties.hpp"
#include "Foundation/ossupport/Signals.hpp"
#include "Foundation/ossupport/SignalHandler.hpp"
#include "Foundation/common/String.hpp"
#include <fstream>
#include "Foundation/ossupport/OS.hpp"
#include "testsignal/TestSignal.hpp"
#include <sstream>

///////////////////////////////////////////////////////////////////////////////

CPPUNIT_TEST_SUITE_REGISTRATION( TestSignal );
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(TestSignal, "test_testsignal");

///////////////////////////////////////////////////////////////////////////////


class EventHandler1 : public Cdmw::OsSupport::SignalHandler
{
public:
    EventHandler1() {m_call = false;} ;
    
    virtual ~EventHandler1() throw () {} ;
    
    int handle_signal(int sig)
         throw ()
    {
        m_call = true;
        std::cout<<"EventHandler1 called"<<std::endl;
        return sig;
    }
    

    bool handler_called()
    {
        return m_call;
    }
    
private:
    bool m_call;
};


class EventHandler2 : public Cdmw::OsSupport::SignalHandler
{
public:
    EventHandler2() {m_call = false;} ;
    
    virtual ~EventHandler2() throw () {} ;
    
    int handle_signal(int sig)
         throw ()
    {
        m_call = true;
        std::cout<<"EventHandler2 called"<<std::endl;
        return sig;
    }
    

    bool handler_called()
    {
        return m_call;
    }
    
private:
    bool m_call;
};

class EventHandler3 : public Cdmw::OsSupport::SignalHandler
{
public:
    EventHandler3() {m_call = false;} ;
    
    virtual ~EventHandler3() throw () {};
    
    int handle_signal(int sig)
         throw ()
    {
        m_call = true;
        std::cout<<"EventHandler3 called"<<std::endl;
        return sig;
    }
    

    bool handler_called()
    {
        return m_call;
    }
    
private:
    bool m_call;
};

TestSignal::TestSignal()
{

}

///////////////////////////////////////////////////////////////////////////////

TestSignal::~TestSignal()
{

}

///////////////////////////////////////////////////////////////////////////////

void 
TestSignal::do_test()
{
    
    EventHandler1* eh1 = new EventHandler1();
    EventHandler2* eh2 = new EventHandler2();
    EventHandler3* eh3 = new EventHandler3();
    
    Cdmw::OsSupport::SignalHandler* oldseh = Cdmw::OsSupport::Signals::Instance()->register_handler(SIGUSR1, eh1);
    if (oldseh == 0)
        CPPUNIT_ASSERT(true);
    else
        CPPUNIT_ASSERT(false);
    
    oldseh = Cdmw::OsSupport::Signals::Instance()->register_handler(SIGUSR2, eh2);
    if (oldseh == 0)
        CPPUNIT_ASSERT(true);
    else
        CPPUNIT_ASSERT(false);

    oldseh = Cdmw::OsSupport::Signals::Instance()->register_handler(SIGUSR2, eh3);
    if (oldseh == eh2)
        CPPUNIT_ASSERT(true);
    else
        CPPUNIT_ASSERT(false);

    
    int pid = Cdmw::OsSupport::OS::get_processId();
    std::cout<<"pid: "<<pid<<std::endl;

    std::ostringstream sigusr1;
    std::ostringstream sigusr2;
    sigusr1<<"-10 "<<pid;
    sigusr2<<"-12 "<<pid;

    Cdmw::OsSupport::OS::create_process("kill", sigusr1.str());

    int timescale = Cdmw::TestUtils::get_timescale();
    Cdmw::OsSupport::OS::sleep(timescale*500);
    
    CPPUNIT_ASSERT(eh1->handler_called());

    Cdmw::OsSupport::OS::create_process("kill", sigusr2.str());

    Cdmw::OsSupport::OS::sleep(timescale*500);

    
    CPPUNIT_ASSERT(eh3->handler_called());


    oldseh = Cdmw::OsSupport::Signals::Instance()->unregister_handler(SIGUSR1);
    if (oldseh == eh1)
        CPPUNIT_ASSERT(true);
    else
        CPPUNIT_ASSERT(false);

    oldseh = Cdmw::OsSupport::Signals::Instance()->unregister_handler(SIGUSR2);
    if (oldseh == eh3)
        CPPUNIT_ASSERT(true);
    else
        CPPUNIT_ASSERT(false);
    
    oldseh = Cdmw::OsSupport::Signals::Instance()->unregister_handler(SIGUSR2);
    if (oldseh == 0)
        CPPUNIT_ASSERT(true);
    else
        CPPUNIT_ASSERT(false);
    
    delete eh1;
    delete eh2;
    delete eh3;
}



///////////////////////////////////////////////////////////////////////////////
//  End Of File
///////////////////////////////////////////////////////////////////////////////

