/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
 
#include "Foundation/testutils/TestManager.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/ossupport/SystemProperties.hpp"

#include "testsystemproperties/TestSystemProperties.hpp"

///////////////////////////////////////////////////////////////////////////////

CPPUNIT_TEST_SUITE_REGISTRATION( TestSystemProperties );
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(TestSystemProperties, "test_systemproperties");

///////////////////////////////////////////////////////////////////////////////

TestSystemProperties::TestSystemProperties()
{

}

///////////////////////////////////////////////////////////////////////////////

TestSystemProperties::~TestSystemProperties()
{

}

///////////////////////////////////////////////////////////////////////////////

void 
TestSystemProperties::do_test()
{

    Cdmw::Common::Properties* prop = new Cdmw::Common::Properties();
    prop->set_property("DefaultProperty", "DefaultProperty");
    Cdmw::OsSupport::SystemProperties::Init(prop);

    // This will only test specificities of SystemProperties
    // properties is already test in common
    TEST_INFO("Get default property ");
    std::string prop0 = Cdmw::OsSupport::SystemProperties::Get_Property("DefaultProperty", "Def");
    CPPUNIT_ASSERT( prop0 == "DefaultProperty" );
    TEST_INFO("Get unknown property ");
    std::string prop1 = Cdmw::OsSupport::SystemProperties::Get_Property("TestProp1", "Def");
    CPPUNIT_ASSERT( prop1 == "Def" );
    TEST_INFO("Get unknown property with int value returned");
    int value1 = Cdmw::OsSupport::SystemProperties::Get_Property("TestProp1", 0);
    CPPUNIT_ASSERT( value1 == 0 );

    TEST_INFO("Set a property");
    Cdmw::OsSupport::SystemProperties::Set_Property("TestProp1", "10");
    TEST_INFO("Get known property ");
    prop1 = Cdmw::OsSupport::SystemProperties::Get_Property("TestProp1", "Def");
    CPPUNIT_ASSERT( prop1 == "10" );
    TEST_INFO("Get known property with int value returned");
    value1 = Cdmw::OsSupport::SystemProperties::Get_Property("TestProp1", 0);
    CPPUNIT_ASSERT( value1 == 10 );
    
    delete prop;
}

///////////////////////////////////////////////////////////////////////////////
//  End Of File
///////////////////////////////////////////////////////////////////////////////

