/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
 
#include "Foundation/testutils/TestManager.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/ossupport/SystemProperties.hpp"
#include "Foundation/common/String.hpp"
#include <fstream>

#include "testsystemproperties2/TestSystemProperties.hpp"

///////////////////////////////////////////////////////////////////////////////

CPPUNIT_TEST_SUITE_REGISTRATION( TestSystemProperties );
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(TestSystemProperties, "test_systemproperties");

///////////////////////////////////////////////////////////////////////////////

TestSystemProperties::TestSystemProperties()
{

}

///////////////////////////////////////////////////////////////////////////////

TestSystemProperties::~TestSystemProperties()
{

}

///////////////////////////////////////////////////////////////////////////////

void 
TestSystemProperties::do_test()
{

    
    std::string filename="file.txt";
    std::string filename2="file2.txt";
    std::string str_opt("test_systemproperties -DCdmwPropertyFile=");
    str_opt += filename;
    
    char** argv__ = Cdmw::Common::String::to_char_array
    ( Cdmw::Common::String::to_strings(str_opt) );
    int argc__(2);
    
    
    
    TEST_INFO("Init the SystemProperties with a unknow file ");
    try
    {
        Cdmw::OsSupport::SystemProperties::Init(argc__,  argv__);
        CPPUNIT_ASSERT(false);
    }
    catch(const Cdmw::BadParameterException&e)
    {
        TEST_INFO(e.what());
        CPPUNIT_ASSERT(true);
    }
    
    std::ofstream out(filename.c_str());
    out<<"P3="<<std::endl;
    out.close();

    TEST_INFO("Init the SystemProperties with a file which contains a bad value");
    try
    {
        Cdmw::OsSupport::SystemProperties::Init(argc__,  argv__);
        CPPUNIT_ASSERT(false);
    }
    catch(const Cdmw::BadParameterException&e)
    {
        TEST_INFO(e.what());
        CPPUNIT_ASSERT(true);
    }

    Cdmw::Common::String::delete_char_array(argv__);
    str_opt = "test_systemproperties -Dprop";
    argv__ = Cdmw::Common::String::to_char_array
    ( Cdmw::Common::String::to_strings(str_opt) );

    /* The following test has been commented while the now only a warning is raised in this case
       TEST_INFO("Init the SystemProperties with a bad parameter ");
    try
    {
        Cdmw::OsSupport::SystemProperties::Init(argc__,  argv__);
        CPPUNIT_ASSERT(true);
    }
    catch(const Cdmw::BadParameterException&e)
    {
        TEST_INFO(e.what());
        CPPUNIT_ASSERT(false);
    }*/
 
    Cdmw::Common::String::delete_char_array(argv__);
    str_opt = "test_systemproperties -Dprop1=P1  -DCdmwPropertyFile=";
    str_opt += filename2;
    argv__ = Cdmw::Common::String::to_char_array
    ( Cdmw::Common::String::to_strings(str_opt) );
    argc__ = 3;
    
    std::ofstream out2(filename2.c_str());
    out2<<"prop2=P2"<<std::endl;
    out2.close();


       TEST_INFO("Init the SystemProperties with a good parameter ");
    try
    {
        Cdmw::OsSupport::SystemProperties::Init(argc__,  argv__);
        CPPUNIT_ASSERT(true);
    }
    catch(const Cdmw::BadParameterException&e)
    {
        CPPUNIT_ASSERT(false);
    }

    TEST_INFO("Get default property ");
    std::string prop1 = Cdmw::OsSupport::SystemProperties::Get_Property("prop1");
    CPPUNIT_ASSERT( prop1 == "P1" );
    std::string prop2 = Cdmw::OsSupport::SystemProperties::Get_Property("prop2");
    CPPUNIT_ASSERT( prop2 == "P2" );

    Cdmw::OsSupport::SystemProperties::Instance()->store(std::cout);

    Cdmw::Common::String::delete_char_array(argv__);
}


///////////////////////////////////////////////////////////////////////////////
//  End Of File
///////////////////////////////////////////////////////////////////////////////

