/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_OSSUPPORT_RECURSIVE_MUTEX_HPP
#define INCL_OSSUPPORT_RECURSIVE_MUTEX_HPP

#include <Foundation/common/Exception.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/osthreads/AbstractMutex.hpp>

namespace Cdmw
{

    namespace OsSupport
    {

        class RecursiveMutexImpl;

        /**
        *Purpose:
        *<p>    Implementation of the RecursiveMutex class with
        *        compilation firewall idiom (pimpl)
        *
        *Features:
        *<p> Thread safe, exception safe
        *
        *@see  MutexGuard 
        *@see  Condition 
        *@see  RecursiveMutexImpl
        *
        */

        class RecursiveMutex : public AbstractMutex
        {

            public:

                /**
                * Purpose:
                * <p> construct the mutex
                *
                *@exception OutOfMemoryException
                *@exception InternalErrorException      
                */
                RecursiveMutex()
                throw (
                    OutOfMemoryException,
                    InternalErrorException);



                //
                // Dtor
                //
                virtual
                ~RecursiveMutex()
                throw() ;


             
                virtual
                void lock ()
                    throw (InternalErrorException);


                virtual
                void unlock()
                throw (PermissionDeniedException, InternalErrorException);


                /**
                * Purpose:
                * <p> return true if the mutex is locked else false
                *
                */
                virtual
                bool is_locked()
                throw ();


                /**
                * Purpose:
                * <p> Returns the underlying implementation object. This
                * method is intended to be used by the library for internal
                * purpose.
                *
                */
                RecursiveMutexImpl&
                get_implementation()
                throw();



            protected:


            private:

                //
                // Copy Ctor
                //
                RecursiveMutex(const RecursiveMutex&);



                //
                // Assignement operator
                //
                RecursiveMutex&
                operator=(const RecursiveMutex&);



                /**
                * True implementation of the RecursiveMutex using the
                * compiler firewall idiom (pimpl)
                */
                std::auto_ptr<RecursiveMutexImpl> m_spImpl;

        };


    } // End namespace osthreads
} // End namespace Cdmw



#endif //INCL_OSSUPPORT_RECURSIVE_MUTEX_HPP

