/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_OSSUPPORT_WriterRecursiveLockGuard_HPP
#define INCL_OSSUPPORT_WriterRecursiveLockGuard_HPP

#include <Foundation/common/Exception.hpp>
#include <Foundation/common/MacroDefs.hpp>


namespace Cdmw
{

    namespace OsSupport
    {

        class ReaderWriterRecursiveMutex;


        /**
        *Purpose:
        *<p>    This class uses the "Ressource acquisition is
        *        initialisation" to automatically 
        *        writeLock/writeUnlock
        *        the associated ReaderWriterRecursiveMutex.
        *
        *Features:
        *<p> Thread safe, exception safe
        *
        *@see  ReaderWriterRecursiveMutex
        *
        */

        class WriterRecursiveLockGuard
        {

            public:

                /**
                * Purpose:
                * <p> Construct the object and acquire the lock for writing.
                *
                * @exception OutOfMemoryException
                * @exception InternalErrorException          
                */

                WriterRecursiveLockGuard(ReaderWriterRecursiveMutex& lock )
                    throw (OutOfMemoryException,
                           InternalErrorException);


                //
                // Dtor
                //
                virtual
                ~WriterRecursiveLockGuard()
                throw();


            protected:


            private:

                //
                // Copy ctor
                //
                WriterRecursiveLockGuard(const WriterRecursiveLockGuard& rhs);

                //
                // Assignement operator
                //
                WriterRecursiveLockGuard&
                operator=(const WriterRecursiveLockGuard& rhs);

                /**
                * ReaderWriterRecursiveMutex used by the guard.
                */
                ReaderWriterRecursiveMutex& m_lock;



        }

        ; // End class WriterRecursiveLockGuard

    } // End namespace osthreads

} // End namespace Cdmw

#ifndef CDMW_MAKE_UNIQUE_NAME
# error "CDMW_MAKE_UNIQUE_NAME: required macro not found!!!"
#endif

/**
* Convenient macro to create a temporary variable with
* WriterRecursiveLockGuard.
*/
#define CDMW_WRITER_RECURSIVE_MUTEX_GUARD(lock) Cdmw::OsSupport::WriterRecursiveLockGuard CDMW_MAKE_UNIQUE_NAME(lockGuard)(lock)

#endif // INCL_OSSUPPORT_WriterRecursiveLockGuard_HPP

