/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/osthreads/MutexGuard.hpp"

#include "Foundation/osthreads/AbstractMutex.hpp"
#include "Foundation/common/Assert.hpp"


namespace Cdmw
{

    namespace OsSupport
    {



        //
        // Construct and acquire the associated Mutex
        //
        MutexGuard::MutexGuard (AbstractMutex& mutex)
        throw (OutOfMemoryException,
               AlreadyDoneException,
               InternalErrorException)

                : m_mutex(mutex)
        {
            m_mutex.lock();
        }



        //
        // Destroy and release the associated Mutex
        //
        MutexGuard::~MutexGuard()
        throw()
        {
            try
            {
                m_mutex.unlock();
            }
            catch ( ...)
            {
                CDMW_ASSERT (false);
            }

        }



    } // End namespace osthreads

} // End namespace Cdmw

