/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/osthreads/RecursiveMutex.hpp"
#include "osthreads/RecursiveMutexImpl.hpp"


namespace Cdmw
{

    namespace OsSupport
    {

        //
        // Ctor
        //
        RecursiveMutex::RecursiveMutex()
        throw (OutOfMemoryException, InternalErrorException)
            : m_spImpl (NULL)
        {

            try
            {
                #   ifdef _MSC_VER
                // The auto_ptr implementation shipped with visual is not compliant
                //  and doesn't provide reset(), but it broken as it uses an assignement
                //  operator with const auto_ptr&, we use this deviation
                m_spImpl = std::auto_ptr<RecursiveMutexImpl> (new RecursiveMutexImpl());
                #   else

                m_spImpl.reset (new RecursiveMutexImpl);
                #       endif 

            }
            catch (const std::bad_alloc&)
            {

                CDMW_THROW(OutOfMemoryException);
            }
        }


        //
        // Dtor
        //
        RecursiveMutex::~RecursiveMutex() throw()
        {
            // Nothing to do
        }



        //
        // Locks the mutex
        //
        void

        RecursiveMutex::lock ()
            throw (InternalErrorException)
        {
           m_spImpl->lock();
        }


        //
        // Unlocks the mutex
        //
        void
        RecursiveMutex::unlock()
        throw (PermissionDeniedException, InternalErrorException)
        { 
          m_spImpl->unlock();
        }

        bool
        RecursiveMutex::is_locked()
        throw ()
        {
            return m_spImpl->is_locked();
        }


    } // End namespace OsSupport
} // End namespace Cdmw



