/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <string>
#include <numeric>

#include <cppunit/extensions/HelperMacros.h>
#include "Foundation/common/MacroDefs.hpp"
#include "Foundation/osthreads/MutexGuard.hpp"
#include "Foundation/osthreads/AtomicCount.hpp"
#include "Foundation/osthreads/Thread.hpp"
#include "Foundation/testutils/TestManager.hpp"

#include "testosthreads/TestAtomicCount.hpp"

using namespace Cdmw::TestOsThread;

CPPUNIT_TEST_SUITE_REGISTRATION( TestAtomicCount );

namespace 
{
    const int MAX_IDX   = 100;
    const int POOL_SIZE =  100;
    Cdmw::OsSupport::AtomicCount a_idx(0);
    int   i_idx=0;
            
    // vector filled with atomic count (sum should be ok)
    std::vector<int> v_ac(MAX_IDX,0);
    // vector filled with simple index (sum could fail)
    std::vector<int> v_si(MAX_IDX,0);

    class ACThread 
        : public Cdmw::OsSupport::Thread
    {
        protected:
            virtual void run() throw ()
        {
            ++a_idx;
            ++i_idx;
               
            {
                CDMW_MUTEX_GUARD();
                if(a_idx < MAX_IDX)
                   v_ac[a_idx] = 1;
                
                if(i_idx < MAX_IDX)
                    v_si[i_idx] = 1;
            }
        }
    };
      

    ACThread pool[POOL_SIZE];

}
      
void  
TestAtomicCount::setUp()
{
    srand(1);
    for(int i=0;i<POOL_SIZE;++i)
    {
        pool[i].start();
    }
    
}

void  
TestAtomicCount::ac_in_action()
{
    for(int i=0;i<POOL_SIZE;++i)
        pool[i].join();
    
    int res1 = std::accumulate(v_ac.begin(),v_ac.end(),0);
    int res2 = std::accumulate(v_si.begin(),v_si.end(),0);
    
    std::ostringstream oss;
    // now all thread should be complete so we can proceed with check
    oss << "The first  sum is: "<<res1<<std::endl;
    oss << "The second sum is: "<<res2<<std::endl;
    TEST_INFO(oss.str());
    CPPUNIT_ASSERT( res1 == (MAX_IDX - 1) );
}


void  
TestAtomicCount::ac_long_like()
{
    Cdmw::OsSupport::AtomicCount ac1(0L);
    
    CPPUNIT_ASSERT(ac1==0L);
    ++ac1;
    CPPUNIT_ASSERT(ac1==1L);

    Cdmw::OsSupport::AtomicCount m( 0 );

    CPPUNIT_ASSERT( m == 0 );

    ++m;

    CPPUNIT_ASSERT( m == 1 );

    ++m;

    CPPUNIT_ASSERT( m == 2 );
    CPPUNIT_ASSERT( --m != 0 );
    CPPUNIT_ASSERT( --m == 0 );

    Cdmw::OsSupport::AtomicCount ac2(10L);
    CPPUNIT_ASSERT(ac2==10L);
    --ac2;
    CPPUNIT_ASSERT(ac2==9L);
    if(--ac2==8L)
       CPPUNIT_ASSERT(true);
    else
    {
       std::cerr<<"AtomicCounter should be 8 instead is: "<<ac2<<std::endl;
       CPPUNIT_ASSERT(false);
    }

    Cdmw::OsSupport::AtomicCount ac3(1L);
    CPPUNIT_ASSERT(ac3==1L);
    --ac3;
    CPPUNIT_ASSERT(ac3==0L);    
}

            
void  
TestAtomicCount::tearDown()
{
}
