/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/osthreads/MutexGuard.hpp"
#include <Foundation/common/Options.hpp>
#include <Foundation/common/String.hpp>
#include <Foundation/ossupport/OS.hpp>


#include <iostream>
#include <string>

 
///////////////////////////////////////////////////////////////////////////////

Cdmw::TestUtils::CORBATestManager::CORBATestManager()
:Cdmw::TestUtils::TestManager()
{
}

///////////////////////////////////////////////////////////////////////////////

Cdmw::TestUtils::CORBATestManager::~CORBATestManager()
    throw()
{
}

///////////////////////////////////////////////////////////////////////////////

CORBA::ORB_ptr
Cdmw::TestUtils::CORBATestManager::get_ORB()
{
    CDMW_MUTEX_GUARD(m_mutex_state);

    if( !m_is_initialized ) 
    {
        std::cerr<<"Orb is non initialized!"<<std::endl;
        throw Cdmw::TestUtils::CORBATestManager::InvalidObject();
    }
    
    return CORBA::ORB::_duplicate(m_orb.in());
}

///////////////////////////////////////////////////////////////////////////////

PortableServer::POA_ptr
Cdmw::TestUtils::CORBATestManager::get_POA()
{
    CDMW_MUTEX_GUARD(m_mutex_state);

    if( !m_is_initialized)
    {
        std::cerr<<"POA is non initialized!"<<std::endl;
        throw Cdmw::TestUtils::CORBATestManager::InvalidObject();
    }
    return PortableServer::POA::_duplicate(m_rootPOA.in());
}

///////////////////////////////////////////////////////////////////////////////

PortableGroup::GOA_ptr
Cdmw::TestUtils::CORBATestManager::get_GOA()
{
    CDMW_MUTEX_GUARD(m_mutex_state);

    if( !m_is_initialized)
    {
        std::cerr<<"POA is non initialized!"<<std::endl;
        throw Cdmw::TestUtils::CORBATestManager::InvalidObject();
    }
    return  PortableGroup::GOA::_narrow(m_rootPOA.in());
}

///////////////////////////////////////////////////////////////////////////////

void
Cdmw::TestUtils::CORBATestManager::init(CORBA::ORB_ptr orb,
                                        PortableServer::POA_ptr  poa,
                                        int      argc,
                                        char**  argv )
{
    this->init_impl(orb,
                    poa,
                    argc,
                    argv);
/*
    TestManager::init(argc,argv);
    m_orb =  CORBA::ORB::_duplicate(orb);
    m_rootPOA = PortableServer::POA::_duplicate(poa);
*/
}

///////////////////////////////////////////////////////////////////////////////


void 
Cdmw::TestUtils::CORBATestManager::init(CORBA::ORB_ptr orb,
                                        PortableGroup::GOA_ptr  goa,
                                        int      argc,
                                        char**  argv )
{
    this->init_impl(orb,
                    goa,
                    argc,
                    argv);
    /*     
    TestManager::init(argc,argv);    
    m_orb =  CORBA::ORB::_duplicate(orb);
    PortableGroup::GOA_ptr tmpGoa = PortableGroup::GOA::_duplicate(goa);
    m_rootPOA = PortableServer::POA::_narrow(tmpGoa); 
    */
}

///////////////////////////////////////////////////////////////////////////////

void
Cdmw::TestUtils::CORBATestManager::init(CORBA::ORB_ptr orb,
                                        int      argc,
                                        char**  argv )
{
    this->init_impl(orb,
                    0,
                    argc,
                    argv);
}

///////////////////////////////////////////////////////////////////////////////

void
Cdmw::TestUtils::CORBATestManager::init_impl(CORBA::ORB_ptr orb,
                                             PortableServer::POA_ptr  poa,
                                             int      argc,
                                             char**  argv )
{
    TestManager::init(argc,argv);
    m_orb =  CORBA::ORB::_duplicate(orb);
    if(!CORBA::is_nil(poa))
    {
      m_rootPOA = PortableServer::POA::_narrow(poa);
    }
}

///////////////////////////////////////////////////////////////////////////////

Cdmw::TestUtils::CORBATestManager* 
Cdmw::TestUtils::CORBATestManager::instance()
{
    static Cdmw::TestUtils::CORBATestManager instance; 
    return &instance; 
}

///////////////////////////////////////////////////////////////////////////////

void
Cdmw::TestUtils::CORBATestManager::summary (std::ostream& os)
{

    // thread-safe
    TestManager::summary(os);

    CDMW_MUTEX_GUARD(m_mutex_state);
    
    // Clear map, as CORBA::Objects in map may require ORB for destruction
    // (especially if those are local objects as Context in CCM),
    // and ORB will be destroyed before this singleton destruction.
    m_db.clear();
}

///////////////////////////////////////////////////////////////////////////////

CORBA::Object_ptr 
Cdmw::TestUtils::CORBATestManager::get_object(const std::string& name )
throw (Cdmw::TestUtils::CORBATestManager::InvalidObject)
{
    CDMW_MUTEX_GUARD(m_mutex_state);

    if( !m_is_initialized ) 
    {
        std::cerr<<"Orb is non initialized!"<<std::endl;
        throw Cdmw::TestUtils::CORBATestManager::InvalidObject();
    }
    
    db_iterator it = m_db.find(name);

    if (it != m_db.end())                                               
    {                                                                  
       CORBA::Object_var ret = CORBA::Object::_duplicate(it->second.in()); 
        return ret._retn();                                            
    }                                                                  
    
    throw Cdmw::TestUtils::CORBATestManager::InvalidObject();
}

///////////////////////////////////////////////////////////////////////////////

CORBA::Object_ptr 
Cdmw::TestUtils::CORBATestManager::remove_object(const std::string& name )
throw (Cdmw::TestUtils::CORBATestManager::InvalidObject)
{
    CDMW_MUTEX_GUARD(m_mutex_state);

    if( !m_is_initialized ) 
    {
        std::cerr<<"Orb is non initialized!"<<std::endl;
        throw Cdmw::TestUtils::CORBATestManager::InvalidObject();
    }
    
    db_iterator it = m_db.find(name);

    if (it != m_db.end())                                               
    {                                                                  
       CORBA::Object_var ret = CORBA::Object::_duplicate(it->second.in());
        m_db.erase(it);
        return ret._retn();                                            
    }                                                                  
    
    throw Cdmw::TestUtils::CORBATestManager::InvalidObject();
}

///////////////////////////////////////////////////////////////////////////////

void  
Cdmw::TestUtils::CORBATestManager::add(const std::string& service_alias , CORBA::Object_ptr obj)
throw (InvalidObject)
{
    CDMW_MUTEX_GUARD(m_mutex_state);

    if( !m_is_initialized ) 
    {
        std::cerr<<"Orb is non initialized!"<<std::endl;
        throw Cdmw::TestUtils::CORBATestManager::InvalidObject();
    }

    if (CORBA::is_nil(obj))
    {
        
        CORBA::Object_var obj;
        obj = m_orb->resolve_initial_references( service_alias.c_str() );
        
        if (CORBA::is_nil(obj.in() ))
            throw InvalidObject();
        m_db[service_alias] = CORBA::Object::_duplicate(obj.in());
    } else {
        m_db[service_alias] = CORBA::Object::_duplicate(obj);
    }
}

///////////////////////////////////////////////////////////////////////////////
// End Of File
///////////////////////////////////////////////////////////////////////////////
