/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.testutils.threadtestengine;

import java.lang.reflect.*;

import org.junit.internal.runners.*;
import org.junit.runner.*;
import org.junit.runner.notification.*;

/**
 * Class extends from JUnit runner for the purpose of injection in JUnit framework in order to provide
 * a warranty that all threads run inside some method of tests will have been joined before start execution
 * of any other test method of the test
 * 
 */
public class CdmwThreadedTestMethodRunner extends TestMethodRunner {
    
    protected Class testClass = null;
    protected Object testObject = null;
    
    protected ThreadedTestContext threadedContext = new ThreadedTestContext();
    
    public CdmwThreadedTestMethodRunner(Object test, Method method, RunNotifier notifier, Description description) {
        super(test, method, notifier, description);
        
        this.testClass = test.getClass();
        this.testObject = test;
    }
    
    @Override
    protected void runUnprotected() {
        this.injectThreadedContext();
        
        super.runUnprotected();
        this.join();
        
        this.resetThreadedContext();
    }
    
    protected void injectThreadedContext() {
        this.setValueToContextField(this.threadedContext);
    }
    
    protected void resetThreadedContext() {
        this.setValueToContextField(null);
    }
    
    protected void setValueToContextField(Object objToSet) {
        Field[] testClassFields = this.testClass.getDeclaredFields();
        for (int i = 0; i < testClassFields.length; i++ ) {
            Field field = testClassFields[i];
            field.setAccessible(true);
            if (field.getType().isAssignableFrom(ThreadedTestContext.class)) {
                try {
                    field.set(this.testObject, objToSet);
                }
                catch (IllegalAccessException e) {
                      addFailure(new Exception("CdmwThreadedTestRunner cannot modify field " + field.getName() + 
                      " in the class " + this.testClass.getCanonicalName() + " due to some problem in security permissions"));
                }
            }
        }
    }
    
    protected void join() {
        this.threadedContext.joinWithProcessors();
    }
}
