/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "datastoretest/TestDataStore.hpp"
#include "datastoretest/TestThread.hpp"

using namespace Cdmw::CommonSvcs::DataStore;

CPPUNIT_TEST_SUITE_REGISTRATION( TestDataStore );


// constructor
TestDataStore::TestDataStore()
    throw()
{
    if (Cdmw::TestUtils::TestManager::instance()->get_property ("realOverlap") == "yes")
        m_realOverlap = true;
    else
        m_realOverlap = false;
}

// destructor
TestDataStore::~TestDataStore()
   throw()
{
}



// do_tests
void TestDataStore::do_tests()
{
    // set number of requested successfull tests


        std::cout << "create DataStore for DataPart1" << std::endl;
        // StorageHome for DataPart1
        DataPart1StorageHome* dataPart1StorageHome = NULL;

        // StorageHome for DataPart2
        DataPart2StorageHome* dataPart2StorageHome = NULL;

        std::cout << "create DataStore for DataPart1" << std::endl;
        DataPart1StoreFactory::Create_data_store(DATA_STORE_1_IDENTIFIER);

        std::cout << "create DataStorageHome for DataPart1" << std::endl;
        dataPart1StorageHome =
           new DataPart1StorageHome(DATA_STORE_1_IDENTIFIER);

        std::cout << "create DataStore for DataPart2" << std::endl;
        DataPart2StoreFactory::Create_data_store(DATA_STORE_2_IDENTIFIER);

        std::cout << "create DataStorageHome for DataPart2" << std::endl;
        dataPart2StorageHome = 
           new DataPart2StorageHome(DATA_STORE_2_IDENTIFIER);



        // Create Threads embedding test
        TestThread testThread1(dataPart1StorageHome, dataPart2StorageHome, 1, m_realOverlap);
        TestThread testThread2(dataPart1StorageHome, dataPart2StorageHome, 2, m_realOverlap);

        // Start threads 
        testThread1.start();
        testThread2.start();

        testThread1.join();
        std::cout << "Thread1 is joined" << std::endl;
        testThread2.join();
        std::cout << "Thread2 is joined" << std::endl;
        
    
        delete dataPart1StorageHome;
        delete dataPart2StorageHome;

}



