/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <cstdlib>
#include <string>
#include <exception>
#include <sstream>
#include <iostream>
#include <fstream>

#include "Foundation/common/Options.hpp"
#include "Foundation/common/String.hpp"
#include "Foundation/orbsupport/CORBA.hpp"

#include "Foundation/ossupport/OS.hpp"
#include "Foundation/osthreads/ThreadHandle.hpp"

#include "datastoretest/DatastoreDefinition.hpp"
#include "datastoretest/TestThread.hpp"


namespace {
    
const int SUCCESS = 0;
const int FAILURE = 1;  

std::string HELP_OPTION = "--help";


static void usage( std::ostream & os, const std::string &program_name )
{
    os << "Usage:"<< std::endl
       << program_name << " options" << std::endl
       << std:: endl;
}

};  // end anonymous namespace


int main( int argc, char* argv[] )
{
    // the return code
    int ret_code = SUCCESS;

    // Check command line arguments
    std::string help_option = Cdmw::OsSupport::OS::get_option_value(
        argc, argv, HELP_OPTION);

    if (help_option == "yes")
    {
        usage (std::cout, argv[0]);
        return ret_code;
    }

    try
    {


        std::string realOverlap= Cdmw::OsSupport::OS::get_option_value( argc, argv, "--realOverlap");
        Cdmw::TestUtils::TestManager::instance()->init(argc,argv);
        Cdmw::TestUtils::TestManager::instance()->set_property ("realOverlap",realOverlap);
        Cdmw::TestUtils::TestManager::instance()->run_tests();
        Cdmw::TestUtils::TestManager::instance()->summary();




    }
    catch (const CORBA::SystemException& e)
    {
        std::cerr << e << std::endl;
        ret_code = FAILURE;
    }
    catch (const CORBA::Exception& e)
    {  
        std::cerr << e._name() << std::endl;
        ret_code = FAILURE;
    }
    catch (...)
    {
        std::cerr << "FAILURE : Unexpected exception" << std::endl;
        ret_code = FAILURE;
    }

    return ret_code;
}

