/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.lifecycle;

import cdmw.commonsvcs.naming.NamingInterface;
import cdmw.commonsvcs.naming.InvalidNameException;
import static cdmw.common.Assert.assertionFailed;

/**
 * The ObjectRegistration acts as a singleton factory for the creation
 * of Lifecycle Objects
 */
public class DefaultObjectRegistration extends ObjectRegistrationBase {

    private NamingInterface root;

    // Set bind method. TRUE is the contexts may be created. FALSE
    // else.
    private boolean createContexts;


    public DefaultObjectRegistration(org.omg.CosNaming.NamingContext root,
                                     boolean createContexts)
        throws org.omg.CORBA.SystemException
    {
        this.root = new cdmw.commonsvcs.naming.NamingInterface(root);
        this.createContexts = createContexts;
    }

    protected boolean bind(org.omg.CORBA.Object objectRef,
                           java.lang.String objectName)
        throws org.omg.CosNaming.NamingContextPackage.InvalidName,
               org.omg.CosNaming.NamingContextPackage.AlreadyBound
    {
        boolean result = false;

        try {
            org.omg.CosNaming.NameComponent[] name =
                NamingInterface.toName(objectName);

            if (name.length != 1) {
                throw new org.omg.CosNaming.NamingContextPackage.InvalidName();
            }
        }
        catch (InvalidNameException e) {
            throw new org.omg.CosNaming.NamingContextPackage.InvalidName();
        }

        if (objectRef != null) {
            try {
                root.bind(objectName, objectRef, createContexts);
                result = true;
            }
            catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
                // Should not happen as objectName length is 1..
                assertionFailed();
            }
            catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e) {
                assertionFailed();
            }
        }

        return result;
    }

    protected boolean unbind(java.lang.String objectName)
        throws org.omg.CosNaming.NamingContextPackage.InvalidName,
               org.omg.CosNaming.NamingContextPackage.NotFound
    {
        boolean result = false;

        try {
            org.omg.CosNaming.NameComponent[] name =
                NamingInterface.toName(objectName);

            if (name.length != 1) {
                throw new org.omg.CosNaming.NamingContextPackage.InvalidName();
            }

            root.unbind(objectName);
            result = true;
        }
        catch (InvalidNameException e) {
            throw new org.omg.CosNaming.NamingContextPackage.InvalidName();
        }
        catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e) {
            assertionFailed();
        }

        return result;
    }

}

