/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.lifecycle.test;

import cdmw.testutils.CORBATestManager;
import cdmw.testutils.CORBATestManager.InvalidObject;
import cdmw.lifecycle.EntityCleanupNotifier;

import org.junit.*;
import org.omg.CORBA.ORB;

import org.omg.CosPropertyService.PropertySet;
import org.omg.CosPropertyService.PropertySetHelper;
import org.omg.CosPropertyService.PropertySetDef;
import org.omg.CosPropertyService.PropertySetDefFactory;
import org.omg.CosPropertyService.InvalidPropertyName;

public class EntityCleanupNotifierTest {

    public static final String PROPERTY_NAME = "EntityData";

    public static final String OBJECT_1_NAME = "OBJECT  1";

    public static final String OBJECT_2_NAME = "OBJECT  2";

    public static final String OBJECT_3_NAME = "OBJECT  3";

    private PropertySetDefFactory factory;

    private ORB orb;

    @Before
    public void init() throws InvalidObject {
        this.factory = (PropertySetDefFactory) CORBATestManager.instance()
            .getLocalProperty(
                "cdmw.lifecycle.test.EntityCleanupNotifierTest.factory");
        this.orb = CORBATestManager.instance().getORB();
    }

    @Test
    public void doTests() {

        // This tests EntityCleanupNotifier class
        // -------------------------------------------------
        System.out.println("Testing EntityCleanupNotifier Persistence");

        try {

            // Create a PropertySet
            PropertySetDef setdef = factory.create_propertysetdef();
            PropertySet set = PropertySetHelper.unchecked_narrow(setdef);

            System.out.println("Creating an EntityCleanupNotifier object");
            EntityCleanupNotifier ecn = new EntityCleanupNotifier(null);

            System.out.println("First init()");
            Assert.assertTrue(ecn.init(set, PROPERTY_NAME));

            System.out.println("Testing that Property is not defined");
            Assert.assertTrue(!set.is_property_defined(PROPERTY_NAME));

            System.out.println("Testing that Property is not defined");
            Assert.assertTrue(ecn.entries() == 0);

            // Create 2 Entries
            System.out.println("Inserting two entries");
            byte[] oid = OBJECT_1_NAME.getBytes();
            ecn.insertEntry(oid, OBJECT_1_NAME);
            oid = OBJECT_2_NAME.getBytes();
            ecn.insertEntry(oid, OBJECT_2_NAME);

            System.out.println("Checking that entries are inserted.");
            Assert.assertTrue(ecn.entries() == 2);

            // Dump entries
            ecn.dump(System.out);

            Assert.assertTrue(ecn.finish(orb));
            System.out.println("State persisted.");

            System.out.println("Checking that Property is defined.");
            Assert.assertTrue(set.is_property_defined(PROPERTY_NAME));

            // State is persisted. Retreive it!

            System.out.println("Creating an EntityCleanupNotifier object");
            ecn = new EntityCleanupNotifier(null);

            Assert.assertTrue(ecn.entries() == 0);

            System.out.println("Retrieving previous state");
            Assert.assertTrue(ecn.init(set, PROPERTY_NAME));

            System.out.println("Checking number of entries");
            Assert.assertTrue(ecn.entries() == 2);

            // Dump entries
            ecn.dump(System.out);

            // Add 1 entry
            System.out.println("Adding another entry");
            oid = OBJECT_3_NAME.getBytes();
            ecn.insertEntry(oid, OBJECT_3_NAME);

            System.out.println("Checking number of entries.");
            Assert.assertTrue(ecn.entries() == 3);

            // Dump entries
            ecn.dump(System.out);

            Assert.assertTrue(ecn.finish(orb));
            System.out.println("New state persisted.");

            // State is persisted. Retreive it again!

            System.out.println("Creating an EntityCleanupNotifier object");
            ecn = new EntityCleanupNotifier(null);

            Assert.assertTrue(ecn.entries() == 0);

            System.out.println("Retrieving previous state");
            Assert.assertTrue(ecn.init(set, PROPERTY_NAME));

            System.out.println("Checking number of entries");
            Assert.assertTrue(ecn.entries() == 3);

            // Dump entries
            ecn.dump(System.out);

        }
        catch (InvalidPropertyName ipn) {
            System.out.println("ERROR: " + ipn.getMessage());
        }

    }

}
