/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "LifeCycle/lifecycleinit/FactoryRegistration.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/commonsvcs/naming/NamingInterface.hpp"
#include "LifeCycle/idllib/CdmwLifeCycle.stub.hpp"
#include "LifeCycle/lifecycle/ErrorMessages.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
namespace Cdmw
{
namespace LifeCycle
{
    
FactoryRegistration::FactoryRegistration(CdmwNamingAndRepository::NameDomain_ptr dom)
    throw (CORBA::SystemException)
  : m_domain(CdmwNamingAndRepository::NameDomain::_duplicate(dom))
{
}

    
FactoryRegistration::~FactoryRegistration()
    throw ()
{
}

bool
FactoryRegistration::bind(CORBA::Object_ptr object_ref,
                                 const char* object_name)
{
  bool result = false;
  using namespace Cdmw::CommonSvcs::Naming;
  using namespace Cdmw::LifeCycle;
  if (!CORBA::is_nil(object_ref)) {
    // Register the name of the factory within its NameDomain. This
    // will check validity of the name and whether it is already in use.
    std::string factory_name; 
    std::string factory_name_domain_name;
    std::string fully_qualified_factory_name(object_name);
    try {
      factory_name 
    = Cdmw::CommonSvcs::Naming::NamingInterface::basename
    (fully_qualified_factory_name);
      factory_name_domain_name
    = Cdmw::CommonSvcs::Naming::NamingInterface::dirname
    (fully_qualified_factory_name);
    } catch (const Cdmw::CommonSvcs::Naming::InvalidNameException & ) {
      std::string parameter(fully_qualified_factory_name);
      CDMW_THROW2( Cdmw::BadParameterException, parameter, ErrorMessages::InvalidNameMessage);
    }

    CdmwLifeCycle::FactoryBase_var fac_ref = 
        CdmwLifeCycle::FactoryBase::_narrow(object_ref);
    CDMW_ASSERT(!CORBA::is_nil(fac_ref.in()));
    try {
        // Register Factory's object reference
        m_domain->register_new_factory(factory_name.c_str(), fac_ref.in());
        result = true; 
        
    } catch (const CdmwNamingAndRepository::NameDomain::AlreadyExists & ) {
        // Old factory was already registered with same name.
        // Remove old one and replace with new one.
        try {
            m_domain->release_name(factory_name.c_str());
            m_domain->register_new_factory(factory_name.c_str(), fac_ref.in());
            result = true; 
            
        } catch (const CdmwNamingAndRepository::NameDomain::NotRegistered &) {
            // The name was just declared as already existing !!
            CDMW_ASSERT(false);
        } catch (const CdmwNamingAndRepository::NameDomain::AlreadyExists & ) {
            // We just released the name !!
            CDMW_ASSERT(false);
        } catch (const CdmwNamingAndRepository::InvalidName &) {
            // The name should have been valid
            CDMW_ASSERT(false);
        } catch (const CORBA::SystemException & ) {
            throw;
        }
        
    } catch (const CdmwNamingAndRepository::InvalidName &) {
        // The name should have been valid
        CDMW_ASSERT(false);
    } catch (const CORBA::SystemException & ) {
        throw;
    }
  }
  return result;
    
}

bool
FactoryRegistration::unbind(const char* object_name)
{
    bool result = false;
    using namespace Cdmw::CommonSvcs::Naming;    
    try {
        m_domain->release_name(object_name);
        result = true;
    // CdmwNamingAndRepository exceptions translated into CosNaming exceptions
    } catch (const InvalidNameException & ) {
        throw CosNaming::NamingContext::InvalidName();
    } catch (const CdmwNamingAndRepository::NameDomain::NotRegistered & ) {
        throw CosNaming::NamingContext::NotFound();
    } catch (const CdmwNamingAndRepository::InvalidName & ) {
        throw CosNaming::NamingContext::InvalidName();
    }

    return result;
}

}; // End namespace Lifecycle
}; // End namespace Cdmw

