/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "LifeCycle/lifecycleinit/InitUtils.hpp"

namespace
{    /**
     * Name of the POA root of CDMW LifeCycle framework
     */
    static const char*  FACTORIES_POA_NAME = "FactoriesPOA";   
}; // End anonymous namespace



namespace Cdmw
{
namespace LifeCycle
{
PortableServer::POA_ptr
InitUtils::create_cdmw_LifeCycle_POA(PortableServer::POA_ptr parent)
    throw (Cdmw::OrbSupport::CORBASystemExceptionWrapper)
{
    CDMW_ASSERT(!CORBA::is_nil(parent));
    
    PortableServer::POA_var factories_poa = PortableServer::POA::_nil();
    
    // Create CDMW Factories POA (persistent POA)            
    using namespace PortableServer;
    try {  

        PortableServer::POAManager_var poaManager = parent -> the_POAManager();
        
        CORBA::PolicyList policies;
        policies.length(7);
        policies[0] = parent->create_id_assignment_policy(USER_ID);
        policies[1] = parent->create_lifespan_policy(PERSISTENT);
        policies[2] = parent->create_servant_retention_policy(RETAIN);
        policies[3] = parent->create_id_uniqueness_policy(UNIQUE_ID);
        policies[4] = parent->create_request_processing_policy(USE_SERVANT_MANAGER);
        policies[5] = parent->create_implicit_activation_policy(NO_IMPLICIT_ACTIVATION);
        policies[6] = parent->create_thread_policy(ORB_CTRL_MODEL);
        
        Cdmw::OrbSupport::StrategyList poaStrategies;
        
        factories_poa = Cdmw::OrbSupport::OrbSupport::create_POA(parent,
                                                                  FACTORIES_POA_NAME,
                                                                  poaManager.in(), 
                                                                  policies,
                                                                  poaStrategies);
        //
        // Create and install servant activator
        //
        PortableServer::ServantActivator_var the_activator 
            = new CDMW_init_ServantActivator();
        
        CDMW_ASSERT(!CORBA::is_nil(the_activator.in()));
        
        factories_poa -> set_servant_manager(the_activator.in());
        
    } catch (const POA::AdapterAlreadyExists & ) {
        // Raised by create_POA
        CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                      CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMLifeCycleFrameworkPOAExists,
                                       CORBA::COMPLETED_NO) );
    } catch (const POA::InvalidPolicy & ) {
        // Raised by create_POA
        CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                      CORBA::INTERNAL(Cdmw::OrbSupport::INTERNALLifeCycleFrameworkError,
                                      CORBA::COMPLETED_NO) );
    } catch (const POA::WrongPolicy & ) {
        // Raised by set_servant_manager
        CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                      CORBA::INTERNAL(Cdmw::OrbSupport::INTERNALLifeCycleFrameworkError,
                                      CORBA::COMPLETED_NO) );
    } catch (const std::bad_alloc & ex) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                      CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                       CORBA::COMPLETED_NO),
                      ex.what() );           
    } catch (const CORBA::SystemException & ex) {
        CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,ex );
        // TODO
    }
    return factories_poa._retn();
}
 
}; // End namespace LifeCycle
}; // End namespace Cdmw


