/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "lifecycleclient/TestEntityStrategy.hpp"
#include "Repository/repositoryinterface/RepositoryInterface.hpp"
#include "Foundation/commonsvcs/naming/NamingInterface.hpp"
#include "Foundation/commonsvcs/naming/NamingUtil.hpp"
#include <string>
#include <cctype>
#include <algorithm>
#include "lifecycleclient/TestData.hpp"
#include "lifecycleclient/ResourceFactory.stub.hpp"
#include "lifecycleclient/AnonymousResourceFactory.stub.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/common/Exception.hpp"
#include "Foundation/common/Assert.hpp"


namespace Cdmw
{
namespace LifeCycle
{

/**
*Implementation notes: [if no pertinent write none ]
*<p>
*Portability issues: [if no pertinent write none ]
*<p>
*/

CPPUNIT_TEST_SUITE_REGISTRATION( TestEntityStrategy );

TestEntityStrategy::TestEntityStrategy()
    throw()

{
    m_args = Cdmw::TestUtils::CORBATestManager::instance()->get_property ("args");
    if (  Cdmw::TestUtils::CORBATestManager::instance()->get_property ("entity") == "yes") 
        m_do_test = true;
    else
        m_do_test = false;
}


TestEntityStrategy::~TestEntityStrategy()
    throw()
{
    // does nothing
}


TestEntityStrategy::TestEntityStrategy(const TestEntityStrategy& rhs)
    throw()
: m_args(rhs.m_args)
{
    // does nothing
}
        
TestEntityStrategy&
TestEntityStrategy::operator=(const TestEntityStrategy& rhs)
    throw()
{
    if (this != &rhs) {
        m_args    = rhs.m_args;
    }
    return *this;
}

void TestEntityStrategy::do_tests()        
{
    if (!m_do_test)
        return;
    //-------------------------------------------------
    // This tests Entity strategy
    //-------------------------------------------------
    
//     set_nbOfRequestedTestOK (8);
    
    TEST_INFO("Testing Entity strategy");

    using namespace Cdmw::LifeCycle;

    try {
        std::string factory_domaine_name = get_FACTORIES_DOMAIN_NAME();
        std::string factory_name         = get_ENTITY_FACTORY_NAME(); 
        std::string dev_key              = get_ENTITY_DEVICE_KEY();
        std::string dev_label            = get_ENTITY_DEVICE_LABEL();
        std::string appl_1_key           = get_ENTITY_APPLICATION_I_KEY();
        std::string appl_1_name          = get_PROGRAM_EXECUTABLE_1();
        std::string appl_2_key           = get_ENTITY_APPLICATION_II_KEY();
        std::string appl_2_name          = get_PROGRAM_EXECUTABLE_2();


        // NOTA: - Device.stop() deactivates Device Object, while
        //         Device.start() or any other operation will reactivate it.
        //       - Application.start() will launch a process, while
        //         Application.stop() will kill the process and deactivate
        //         the Application object.
        
        Cdmw::CommonSvcs::Naming::NamingInterface ni
            = Cdmw::NamingAndRepository::RepositoryInterface::get_domain_naming_interface
            (factory_domaine_name);
        
        TEST_INFO( "Testing " << factory_name << " factory.");

        typedef Cdmw::CommonSvcs::Naming::NamingUtil<CFFactories::ResourceFactory> 
            Util;
    
        CFFactories::ResourceFactory_var fac 
            = Util::resolve_name(ni,factory_name);

        CDMW_ASSERT(!CORBA::is_nil(fac.in()));

        CF::Device_var dev = CF::Device::_nil();
        try {
            TEST_INFO("Creating Device <" << dev_key << "," << dev_label << ">");
            dev = fac->create_device(dev_key.c_str(),dev_label.c_str());
            CPPUNIT_ASSERT(true);
        } catch (const CdmwLifeCycle::AlreadyExistant & ex) {
            CPPUNIT_ASSERT(false); return;
        } catch (const CdmwLifeCycle::InvalidName & ex) {
            CPPUNIT_ASSERT(false); return;
        } catch (const CF::Device::InvalidLocation & ex) {
            CPPUNIT_ASSERT(false); return;
        } catch (const CORBA::SystemException & ex) {
            CPPUNIT_ASSERT(false); return;
        }
        
        try {
            TEST_INFO("Starting it");
            dev->start();
            CPPUNIT_ASSERT(true);
        } catch (const CF::Resource::StartError & ex) {
            CPPUNIT_ASSERT(false); return;
        } catch (const CORBA::SystemException & ex) {
            CPPUNIT_ASSERT(false); return;
        }
        
        CF::Application_var appli = CF::Application::_nil();
        try {
            TEST_INFO("Creating Application I <" << appl_1_key << "," << appl_1_name << ">");
            appli = fac->create_application (appl_1_key.c_str(), appl_1_name.c_str());
            CPPUNIT_ASSERT(true);
        } catch (const CdmwLifeCycle::AlreadyExistant & ex) {
            CPPUNIT_ASSERT(false); return;
        } catch (const CdmwLifeCycle::InvalidName & ex) {
            CPPUNIT_ASSERT(false); return;
        } catch (const CF::CreateApplicationError & ex) {
            CPPUNIT_ASSERT(false); return;
        } catch (const CORBA::SystemException & ex) {
            CPPUNIT_ASSERT(false); return;
        }

        try {
            TEST_INFO("Starting it");
            appli->start();
            CPPUNIT_ASSERT(true);
        } catch (const CF::Resource::StartError & ex) {
            CPPUNIT_ASSERT(false); return;
        } catch (const CORBA::SystemException & ex) {
            CPPUNIT_ASSERT(false); return;
        }
        
        CF::Application_var appli2 = CF::Application::_nil();
        try {
            TEST_INFO("Creating Application II <" << appl_2_key << ","<< appl_2_name << ">");
            appli2 = fac->create_application (appl_2_key.c_str(), appl_2_name.c_str());
            CPPUNIT_ASSERT(true);
        } catch (const CdmwLifeCycle::AlreadyExistant & ex) {
            CPPUNIT_ASSERT(false); return;
        } catch (const CdmwLifeCycle::InvalidName & ex) {
            CPPUNIT_ASSERT(false); return;
        } catch (const CF::CreateApplicationError & ex) {
            CPPUNIT_ASSERT(false); return;
        } catch (const CORBA::SystemException & ex) {
            CPPUNIT_ASSERT(false); return;
        }
        try {
            TEST_INFO("Starting it");
            appli2->start();
            CPPUNIT_ASSERT(true);
        } catch (const CF::Resource::StartError & ex) {
            CPPUNIT_ASSERT(false); return;
        } catch (const CORBA::SystemException & ex) {
            CPPUNIT_ASSERT(false); return;
        }

        

        try {
            TEST_INFO("Stopping Application I");
            appli->stop();
            CPPUNIT_ASSERT(true);
        } catch (const CF::Resource::StopError & ex) {
            CPPUNIT_ASSERT(false); return;
        } catch (const CORBA::SystemException & ex) {
            CPPUNIT_ASSERT(false); return;
        }

        try {
            TEST_INFO("Deleting Application II");
            appli2->remove();
            CPPUNIT_ASSERT(true);
        } catch (const CosLifeCycle::NotRemovable & ex){
            CPPUNIT_ASSERT(false); return;
        } catch (const CORBA::SystemException & ex) {
          std::cerr << ex << std::endl;
            CPPUNIT_ASSERT(false); return;
        }

    } catch (const CORBA::Exception & ex) {
        TEST_INFO("ERROR:" << ex);
        CPPUNIT_ASSERT(false);
    } catch (const Cdmw::Common::TypeMismatchException & ex) {
        TEST_INFO("ERROR:" << ex.what());
        CPPUNIT_ASSERT(false);
    }
}
        
}; // End namespace LifeCyle
}; // End namespace Cdmw

