/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.validation;

import static org.junit.Assert.*;

import java.io.PrintStream;

import org.junit.*;

import cdmw.common.Assert;
import cdmw.testutils.CORBATestManager;

import cdmw.commonsvcs.naming.NamingInterface;
import cdmw.namingandrepository.RepositoryInterface;

import com.thalesgroup.CF.Application;
import com.thalesgroup.CF.Device;
import com.thalesgroup.CF.DevicePackage.InvalidLocation;
import com.thalesgroup.CF.ResourcePackage.StartError;
import com.thalesgroup.CF.ResourcePackage.StopError;
import com.thalesgroup.CF.CreateApplicationError;

import com.thalesgroup.CFFactories.ResourceFactory;
import com.thalesgroup.CFFactories.ResourceFactoryHelper;


public class UnManagedEntityStrategyTest {

    private String args;
    private PrintStream out;

    public UnManagedEntityStrategyTest ()
    {
    }

    @Before
    public void init() {
        this.out = (PrintStream) CORBATestManager.getLocalInstance()
            .getLocalProperty(
            "cdmw.validation.UnManagedEntityStrategyTest.out");
        this.args = (String) CORBATestManager.getLocalInstance()
            .getLocalProperty(
            "cdmw.validation.UnManagedEntityStrategyTest.arg");
    }

    @Test
    public void doTests() {
        
        //-------------------------------------------------
        // This tests UnManagedEntity strategy
        //-------------------------------------------------
        this.out.println("Testing UnManagedEntity strategy");

        try {
            String factory_domaine_name
                = TestData.get_FACTORIES_DOMAIN_NAME();
            String factory_name
                = TestData.get_UNMANAGED_ENTITY_FACTORY_NAME();
            String dev_key
                = TestData.get_UNMANAGED_ENTITY_DEVICE_KEY();
            String dev_label
                = TestData.get_UNMANAGED_ENTITY_DEVICE_LABEL();
            String appl_1_key
                = TestData.get_UNMANAGED_ENTITY_APPLICATION_I_KEY();
            String appl_1_name
                = TestData.get_PROGRAM_EXECUTABLE_1();
            String appl_2_key
                = TestData.get_UNMANAGED_ENTITY_APPLICATION_II_KEY();
            String appl_2_name
                = TestData.get_PROGRAM_EXECUTABLE_2();


            // NOTA: - Device.stop() deactivates Device Object, while
            //         Device.start() or any other operation will reactivate it.
            //       - Application.start() will launch a process, while
            //         Application.stop() will kill the process and deactivate
            //         the Application object.

            NamingInterface ni
                = RepositoryInterface.
                    getDomainNamingInterface(factory_domaine_name);

            this.out.println( "Testing " + factory_name + " factory.");

            org.omg.CORBA.Object obj = ni.resolve(factory_name);
            ResourceFactory fac = ResourceFactoryHelper.narrow(obj);

            Assert.check(fac!=null);

            Device dev = null;
            try {
                this.out.println("Creating Device <" + dev_key + "," + dev_label + ">");
                dev = fac.create_device(dev_key,dev_label);
            } catch (com.thalesgroup.CdmwLifeCycle.AlreadyExistant ae) {
                fail(); return;
            } catch (com.thalesgroup.CdmwLifeCycle.InvalidName in) {
                fail(); return;
            } catch (InvalidLocation il) {
                fail(); return;
            } catch (org.omg.CORBA.SystemException se) {
                fail(); return;
            }

            try {
                this.out.println("Starting it");
                dev.start();
            } catch (StartError se) {
                fail(); return;
            } catch (org.omg.CORBA.SystemException se) {
                fail(); return;
            }

            Application appli = null;
            try {
                this.out.println("Creating Application I <" + appl_1_key + "," + appl_1_name + ">");
                appli = fac.create_application (appl_1_key, appl_1_name);
            } catch (com.thalesgroup.CdmwLifeCycle.AlreadyExistant ae) {
                fail(); return;
            } catch (com.thalesgroup.CdmwLifeCycle.InvalidName in) {
                fail(); return;
            } catch (CreateApplicationError cae) {
                fail(); return;
            } catch (org.omg.CORBA.SystemException se) {
                fail(); return;
            }

            try {
                this.out.println("Starting it");
                appli.start();
            } catch (StartError se) {
                fail(); return;
            } catch (org.omg.CORBA.SystemException sex) {
                fail(); return;
            }

            Application appli2 = null;
            try {
                this.out.println("Creating Application I <" + appl_2_key + ","+ appl_2_name + ">");
                appli2 = fac.create_application (appl_2_key, appl_2_name);
            } catch (com.thalesgroup.CdmwLifeCycle.AlreadyExistant ae) {
                fail(); return;
            } catch (com.thalesgroup.CdmwLifeCycle.InvalidName in) {
                fail(); return;
            } catch (CreateApplicationError cae) {
                fail(); return;
            } catch (org.omg.CORBA.SystemException se) {
                fail(); return;
            }
            try {
                this.out.println("Starting it");
                appli2.start();
            } catch (StartError se) {
                fail(); return;
            } catch (org.omg.CORBA.SystemException sex) {
                fail(); return;
            }

            // Cdmw::OsSupport::OS::sleep(4000);

            try {
                this.out.println("Stopping Application I");
                appli.stop();
            } catch (StopError se) {
                fail(); return;
            } catch (org.omg.CORBA.SystemException sex) {
                fail(); return;
            }

            try {
                this.out.println("Deleting Application II");
                appli2.remove();
            } catch (org.omg.CosLifeCycle.NotRemovable nr){
                fail(); return;
            } catch (org.omg.CORBA.SystemException se) {
                fail(); return;
            }

        } catch (org.omg.CosNaming.NamingContextPackage.NotFound nf) {
            this.out.println("ERROR:" + nf.getMessage());
            fail();
        } catch (org.omg.CosNaming.NamingContextPackage.CannotProceed cp) {
            this.out.println("ERROR:" + cp.getMessage());
            fail();
        } catch (com.thalesgroup.CdmwNamingAndRepository.NoNameDomain nnd) {
            this.out.println("ERROR:" + nnd.getMessage());
            fail();
        } catch (org.omg.CosNaming.NamingContextPackage.InvalidName in) {
            this.out.println("ERROR:" + in.getMessage());
            fail();
        } catch (com.thalesgroup.CdmwNamingAndRepository.InvalidName in2) {
            this.out.println("ERROR:" + in2.getMessage());
            fail();
        }

    }

}

