/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @file Activate the Load Balancing Service
 *
 * @author Raffaella Panella <rpanella@amjsv.it>
 * @author Antonello Cedrone <a.cedrone@vitrociset.it>
 */

#include "lbcommon/LBConfiguration.hpp"

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>

#include "LoadBalancing/lbcommon/TaoIOGRFactory.hpp"

namespace Cdmw
{
namespace LB
{
    Cdmw::LB::LBStrategyRegistry* Cdmw::LB::LBConfiguration::M_lb_strategy_registry = 0;
    std::string  LBConfiguration::M_override_lb_policy;
    CORBA::Boolean LBConfiguration::M_override_lb_policy_default;
    CORBA::Boolean LBConfiguration::M_override_lb_policy_force;
    CORBA::Boolean LBConfiguration::M_useRepository;
    std::string LBConfiguration::miop_corbaloc;
    int LBConfiguration::M_lb_time_refresh;
    
   //Singleton
   Cdmw::LB::LBConfiguration* Cdmw::LB::LBConfiguration::lbc = 0;
   /**
     * Get the Strategy Registry containing the Load Balancing strategies
     *
     * @return The Strategy Registry
     */

    Cdmw::LB::LBStrategyRegistry* LBConfiguration::GetLbStrategyRegistry()
    {
        return M_lb_strategy_registry; 
    }

    /**
     * Activate the LB service
     *
     *@param argc The number of parameter received of the command line
     *@param argv The list of parameter received of the command line
     *
     * @return void
     */

    void LBConfiguration::Activate_LB_service(int argc,char* argv[])    
        throw(CORBA::SystemException)
    {
       if(Cdmw::LB::LBConfiguration::lbc == 0)
       {
        lbc = new Cdmw::LB::LBConfiguration();
        M_lb_strategy_registry = new Cdmw::LB::LBStrategyRegistry();
        M_override_lb_policy_default=false;
        M_override_lb_policy_force=false;
        M_useRepository=false;
        M_lb_time_refresh = 0;

        std::string str=Cdmw::OsSupport::OS::get_option_value(argc,argv,"--CdmwLBTimeRefresh");
        if (str.compare("yes")&&str.compare("no"))
        {
            M_lb_time_refresh=atoi((Cdmw::OsSupport::OS::get_option_value(argc,argv,"--CdmwLBTimeRefresh")).c_str());
        }

        str=Cdmw::OsSupport::OS::get_option_value(argc,argv,"--CdmwUseRepository");
        if (str.compare("no"))
        {
            M_useRepository = true;
        }
   
        str=Cdmw::OsSupport::OS::get_option_value(argc,argv,"--CdmwLBPolicyDefault");
        if (str.compare("yes")&&str.compare("no"))
        {
            M_override_lb_policy=(CORBA::string_dup((char *)
                                 (Cdmw::OsSupport::OS::get_option_value(argc,argv,"--CdmwLBPolicyDefault")).c_str()));
            M_override_lb_policy_default=true;
        }
        else
        {
            str=Cdmw::OsSupport::OS::get_option_value(argc,argv,"--CdmwLBPolicyForce");
            if (str.compare("yes")&&str.compare("no"))
            {
                M_override_lb_policy=(CORBA::string_dup((char *)
                                     (Cdmw::OsSupport::OS::get_option_value(argc,argv,"--CdmwLBPolicyForce")).c_str()));
                M_override_lb_policy_force=true;
            };
        };
        str="";
        str=Cdmw::OsSupport::OS::get_option_value(argc,argv,"--CdmwLBStateTransferPort");
        if ((str.compare("no")) && (str.compare("yes")) )
        {
            miop_corbaloc ="corbaloc:miop:1.0@1.0-TopicUpdateManager-1/225.1.1.225:" + str;
        }
        else
            miop_corbaloc="corbaloc:miop:1.0@1.0-TopicUpdateManager-1/225.1.1.225:19888"; 
     }
   }


   
    void LBConfiguration::Deactivate_LB_service()    
        throw(CORBA::SystemException)
    {
        delete M_lb_strategy_registry;
        delete lbc;
    }
    /**
     * Register an User Defined Load Balancing Strategy
     *
     * @param StrategyInfo The StrategyInfo, containing the name and the StrategyFactory, to add
     * in the Strategy Registry
     *
     * @return void
     */
    
    void LBConfiguration::Register_lb_policy(CdmwLB::StrategyInfo* userdef_strategy_info)
        throw(CORBA::SystemException)
    {
        M_lb_strategy_registry->set_lb_policy_factory(userdef_strategy_info);
    }
    
}; // namespace LB

}; // namespace Cdmw

