/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @brief _implementation for TestLBCommon with cppUnit library.
 *
 * @author Raffaella Panella <rpanella@amjsv.it>
 *
 */

#include <iostream>
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/common/String.hpp"
#include "Foundation/ossupport/OS.hpp"
#include <LoadBalancing/idllib/CdmwLBCommon.stub.hpp>
#include <LoadBalancing/idllib/CdmwLBIOGRFactory.stub.hpp>
#include <Foundation/logging/Logger.hpp>
#include "Foundation/testutils/CORBATestManager.hpp"
#include "testlbcommon/TestLBCommon.hpp"
#include "lbcommon/config.hpp"

#if CDMW_ORB_VDR == tao
#include <LoadBalancing/lbcommon/Tao_LB_IOGRFactory.hpp>
#else
#error "Unsupported ORB"
#endif

const char* REP_ID = "IDL:thalesgroup.com/HelloInterface1:1.0";
const ::PortableGroup::ObjectGroupId ID = 0xffffff01L;
CdmwLBCommon::HelloInterface1_var TestLBCommon::M_obj1;
CdmwLBCommon::HelloInterface1_var TestLBCommon::M_obj2;
CdmwLBCommon::HelloInterface2_var TestLBCommon::M_obj3;
Cdmw::LB::IOGRFactory* TestLBCommon::M_iogrFactory;
CORBA::ORB_var TestLBCommon::M_orb;
::PortableGroup::Location TestLBCommon::M_loc1;
::PortableGroup::Location TestLBCommon::M_loc2;
/*****************************************************************************/

              CPPUNIT_TEST_SUITE_REGISTRATION( TestLBCommon );

/*****************************************************************************/
    
 
void
TestLBCommon::setUp()
{
 
    M_orb = CORBA::ORB::_duplicate(Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());
    // create two object hello1
    obj1_impl = new Cdmw::HelloInterface1_impl();
    obj2_impl = new Cdmw::HelloInterface1_impl();

    // create one object hello2
    obj3_impl = new Cdmw::HelloInterface2_impl();

    M_obj1 = obj1_impl->_this();
    M_obj2 = obj2_impl->_this();
    M_obj3 = obj3_impl->_this();

    M_loc1.length(1);
    M_loc1[0].id="HOST1";
    M_loc1[0].kind="hostname";

    M_loc2.length(1);
    M_loc2[0].id="HOST2";
    M_loc2[0].kind="hostname";

    
#if CDMW_ORB_VDR == tao
    M_iogrFactory = new Cdmw::LB::Tao_LB_IOGRFactory(std::cerr,
                                                     M_orb.in());
#endif
    
}

/****************************************************************************/


void
TestLBCommon::tearDown()
{
}

/*****************************************************************************/

/**
 * @brief test build iogr operation.
 *
 */
void
TestLBCommon::build_iogr()
{
    ::CdmwLB::MemberInfo member_info1;
    member_info1.the_reference = CORBA::Object::_duplicate(M_obj1.in());
    member_info1.the_location = M_loc1;

    ::CdmwLB::MemberInfo member_info2;
    member_info2.the_reference = CORBA::Object::_duplicate(M_obj2.in());
    member_info2.the_location = M_loc2;

   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"\n");
   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Create MemberInfos   " );
    ::CdmwLB::MemberInfos new_members;
    new_members.length(2);
    new_members[0] = member_info1;
    new_members[1] = member_info2;

    ::CdmwLB::Objects_var no_fallBack = new ::CdmwLB::Objects;
    no_fallBack->length(0);

    ::PortableGroup::ObjectGroupRefVersion  object_group_ref_version = 1;

    ::CdmwLB::MemberInfos no_members;
    no_members.length(0);

   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Try to create an iogr without members (ObjectNotCreated exception must be raised)   " );
    CPPUNIT_ASSERT_THROW( M_iogrFactory->build_iogr(no_members,
                                                    ID,
                                                    REP_ID,
                                                    "my_domain",
                                                    "Random",
                                                    object_group_ref_version,
                                                    no_fallBack),
                          ::PortableGroup::ObjectNotCreated);

   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Create an iogr with the two previous objects (No exception must be raised)   " );
    CPPUNIT_ASSERT_NO_THROW( M_iogrFactory->build_iogr(new_members,
                                                       ID,
                                                       REP_ID,
                                                       "my_domain",
                                                       "Random",
                                                       object_group_ref_version,
                                                       no_fallBack));

    member_info2.the_reference = CORBA::Object::_duplicate(M_obj3.in());
    member_info2.the_location = M_loc2;

   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Create MemberInfos   " );
    new_members.length(2);
    new_members[0] = member_info1;
    new_members[1] = member_info2;

   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Try to create an iogr with diffrent ior type (ObjectNotCreated exception must be raised)   " );
    CPPUNIT_ASSERT_THROW( M_iogrFactory->build_iogr(new_members,
                                                    ID,
                                                    REP_ID,
                                                    "my_domain",
                                                    "Random",
                                                    object_group_ref_version,
                                                    no_fallBack),
                          ::PortableGroup::ObjectNotCreated);

   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Create MemberInfos   " );
    new_members.length(2);
    new_members[0] = member_info1;
    new_members[1] = member_info1;

   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Try to create an iogr with duplicated ior (ObjectNotCreated exception must be raised)   " );
    CPPUNIT_ASSERT_THROW( M_iogrFactory->build_iogr(new_members,
                                                    ID,
                                                    REP_ID,
                                                    "my_domain",
                                                    "Random",
                                                    object_group_ref_version,
                                                    no_fallBack),
                          ::PortableGroup::ObjectNotCreated);
}

/**
 * @brief test profile count operation.
 *
 */

void
TestLBCommon::profile_count()
{
    ::CdmwLB::MemberInfo member_info1;
    member_info1.the_reference = CORBA::Object::_duplicate(M_obj1.in());
    member_info1.the_location = M_loc1;

    ::CdmwLB::MemberInfo member_info2;
    member_info2.the_reference = CORBA::Object::_duplicate(M_obj2.in());
    member_info2.the_location = M_loc2;

   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"\n");
   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Create MemberInfos   " );
    ::CdmwLB::MemberInfos new_members;
    new_members.length(2);
    new_members[0] = member_info1;
    new_members[1] = member_info2;

    CORBA::ULong nb_network_interfaces = member_info2.the_reference->_stubobj ()->base_profiles ().profile_count();
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "==========================================");
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "|   the network interfaces up are =  " << nb_network_interfaces );
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "==========================================" );
    ::CdmwLB::Objects_var no_fallBack = new ::CdmwLB::Objects;
    no_fallBack->length(0);

    ::PortableGroup::ObjectGroupRefVersion  object_group_ref_version = 1;

    ::PortableGroup::ObjectGroup_var iogr;
   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Create an iogr with the two previous objects (No exception must be raised)   " );
    CPPUNIT_ASSERT_NO_THROW( iogr = M_iogrFactory->build_iogr(new_members,
                                                       ID,
                                                       REP_ID,
                                                       "my_domain",
                                                       "Random",
                                                       object_group_ref_version,
                                                       no_fallBack));
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Count the number of profile into the iogr just created (with two members)   ");
    CORBA::ULong nb_profile = M_iogrFactory->profile_count(iogr.in());
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),  "nb_profile" );
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),  nb_profile );
    CPPUNIT_ASSERT(nb_profile == (nb_network_interfaces*2));
}

/**
 * @brief test decode profile with group tag operation.
 *
 */

void
TestLBCommon::decode_profile_with_group_tag()
{
    ::CdmwLB::MemberInfo member_info1;
    member_info1.the_reference = CORBA::Object::_duplicate(M_obj1.in());
    member_info1.the_location = M_loc1;

    CORBA::ULong nb_network_interfaces = member_info1.the_reference->_stubobj ()->base_profiles ().profile_count();

    ::CdmwLB::MemberInfo member_info2;
    member_info2.the_reference = CORBA::Object::_duplicate(M_obj2.in());
    member_info2.the_location = M_loc2;

    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"\n");
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Create MemberInfos   " );
    ::CdmwLB::MemberInfos new_members;
    new_members.length(2);
    new_members[0] = member_info1;
    new_members[1] = member_info2;

    ::CdmwLB::Objects_var no_fallBack = new ::CdmwLB::Objects;
    no_fallBack->length(0);

    ::PortableGroup::ObjectGroupRefVersion  object_group_ref_version = 1;

    ::PortableGroup::ObjectGroup_var iogr;
   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Create an iogr with the two previous objects (No exception must be raised)   " );
    CPPUNIT_ASSERT_NO_THROW( iogr = M_iogrFactory->build_iogr(new_members,
                                                              ID,
                                                              REP_ID,
                                                              "my_domain",
                                                              "Random",
                                                              object_group_ref_version,
                                                              no_fallBack));

   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Check if the profile 1 contains the TAG_LB_GROUP (No exception must be raised)  " );
    CPPUNIT_ASSERT_NO_THROW(M_iogrFactory->decode_profile_with_group_tag (iogr.in(),0));

   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Check if the profile "<< (1*nb_network_interfaces) <<" contains the TAG_LB_GROUP (No exception must be raised)  " );
    CPPUNIT_ASSERT_NO_THROW(M_iogrFactory->decode_profile_with_group_tag (iogr.in(),(1*nb_network_interfaces)));

   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Check if the profile "<<(2*nb_network_interfaces)<< " doesn't exist (ProfileNotFound exception must be raised)  " );
    CPPUNIT_ASSERT_THROW(M_iogrFactory->decode_profile_with_group_tag (iogr.in(),(2*nb_network_interfaces)),
                            CdmwLB::ProfileNotFound);

   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Check if no TAG_LB_GROUP is present into a simple ior (TagNotFound exception must be raised)  " );
    CPPUNIT_ASSERT_THROW(M_iogrFactory->decode_profile_with_group_tag (M_obj1.in(),0),
                         CdmwLB::TagNotFound);
}

/**
 * @brief test decode profile with fallback tag operation.
 *
 */
void
TestLBCommon::decode_profile_with_fallback_tag()
{
    ::CdmwLB::MemberInfo member_info;
    member_info.the_reference = CORBA::Object::_duplicate(M_obj2.in());
    member_info.the_location = M_loc1;

   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"\n");
   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Create MemberInfos   " );
    ::CdmwLB::MemberInfos new_members;
    new_members.length(1);
    new_members[0] = member_info;

    ::CdmwLB::Objects_var fallBack = new ::CdmwLB::Objects;

    fallBack->length(0);

    ::PortableGroup::ObjectGroupRefVersion  object_group_ref_version = 1;

    ::PortableGroup::ObjectGroup_var iogr;

   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Try to create an iogr without fallback replicas (No exception must be raised)   " );
    CPPUNIT_ASSERT_NO_THROW( iogr = M_iogrFactory->build_iogr(new_members,
                                                              ID,
                                                              REP_ID,
                                                              "my_domain",
                                                              "Random",
                                                              object_group_ref_version,
                                                              fallBack));

    CORBA::ULong nb_network_interfaces = member_info.the_reference->_stubobj ()->base_profiles ().profile_count();
   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Check if the profile 1 contains the TAG_FALLBACK (TagNotFound exception must be raised)  " );
    CPPUNIT_ASSERT_THROW(M_iogrFactory->decode_profile_with_fallback_tag (iogr.in(),0),
                         CdmwLB::TagNotFound);
    
   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Check if the profile 2 doesn't exist (ProfileNotFound exception must be raised)  " );
    CPPUNIT_ASSERT_THROW(M_iogrFactory->decode_profile_with_fallback_tag (iogr.in(),(1*nb_network_interfaces)),
                         CdmwLB::ProfileNotFound);



    fallBack->length(1);
    fallBack[0L] = CORBA::Object::_duplicate(M_obj1.in());

   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Try to create an iogr with a fallback replica (No exception must be raised)   " );
    CPPUNIT_ASSERT_NO_THROW( iogr = M_iogrFactory->build_iogr(new_members,
                                                              ID,
                                                              REP_ID,
                                                              "my_domain",
                                                              "Random",
                                                              object_group_ref_version,
                                                              fallBack));

   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Check if the profile 1 contains the TAG_FALLBACK (TagNotFound exception must be raised)  " );
    CPPUNIT_ASSERT_THROW(M_iogrFactory->decode_profile_with_fallback_tag (iogr.in(),0),
                         CdmwLB::TagNotFound);

   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Check if the profile 1 contains the TAG_LB_GROUP (No exception must be raised)  " );
    CPPUNIT_ASSERT_NO_THROW(M_iogrFactory->decode_profile_with_group_tag (iogr.in(),0));

   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Check if the profile 2 (fallback replica) contains the TAG_FALLBACK (No exception must be raised)  " );
    CPPUNIT_ASSERT_NO_THROW(M_iogrFactory->decode_profile_with_fallback_tag (iogr.in(),(1*nb_network_interfaces)));

   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Check if the profile 2 (fallback replica) contains the TAG_LB_GROUP (No exception must be raised)  " );
    CPPUNIT_ASSERT_NO_THROW(M_iogrFactory->decode_profile_with_group_tag (iogr.in(),(1*nb_network_interfaces)));

   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Check if the profile 3 doesn't exist (ProfileNotFound exception must be raised)  " );
    CPPUNIT_ASSERT_THROW(M_iogrFactory->decode_profile_with_fallback_tag (iogr.in(),(2*nb_network_interfaces)),
                         CdmwLB::ProfileNotFound);
}

/**
 * @brief test get member ref operation.
 *
 */
void
TestLBCommon::get_member_ref()
{
    ::CdmwLB::MemberInfo member_info1;
    member_info1.the_reference = CORBA::Object::_duplicate(M_obj1.in());
    member_info1.the_location = M_loc1;

    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"\n");
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Create MemberInfos   " );
    ::CdmwLB::MemberInfos new_members;
    new_members.length(1);
    new_members[0] = member_info1;

    ::CdmwLB::Objects_var no_fallBack = new ::CdmwLB::Objects;
    no_fallBack->length(0);

    ::PortableGroup::ObjectGroupRefVersion  object_group_ref_version = 1;

    ::PortableGroup::ObjectGroup_var iogr;
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Create an iogr with the previous object (No exception must be raised)   " );
    CPPUNIT_ASSERT_NO_THROW( iogr = M_iogrFactory->build_iogr(new_members,
                                                       ID,
                                                       REP_ID,
                                                       "my_domain",
                                                       "Random",
                                                       object_group_ref_version,
                                                       no_fallBack));
    CORBA::Object_var member_ref;
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Try to invoke get_member_ref on the first profile (No exception must be raised)   ");
    CPPUNIT_ASSERT_NO_THROW( member_ref = M_iogrFactory->get_member_ref(iogr.in(), 0));

    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"   ...MemberReference --> " << M_orb->object_to_string(member_ref.in()));
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Try to invoke get_member_ref on the second profile ");
    CORBA::ULong nb_member_profiles = 0;
    try 
    {
        member_ref = M_iogrFactory->get_member_ref(iogr.in(), 1);
        nb_member_profiles = member_ref->_stubobj ()->base_profiles ().profile_count();
        if (nb_member_profiles >1)
        {
           CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   The current host has more than one network interface up   ");
           CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   (No exception must be raised)  ");
           CPPUNIT_ASSERT(true);
        }
    } catch(const PortableGroup::MemberNotFound& mnf)
    {
        CPPUNIT_ASSERT(true);
    }
    //CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "   Try to invoke get_member_ref on the second profile (MemberNotFound exception must be raised)   ");
    //CPPUNIT_ASSERT_THROW( member_ref = M_iogrFactory->get_member_ref(iogr.in(), 1),
                         //PortableGroup::MemberNotFound);
}
