/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @brief _implementation for TestLBCommon with cppUnit library.
 *
 * @author Raffaella Panella <rpanella@amjsv.it>
 *
 */
#ifndef LBCOMMON_PLUGIN_EXPORTS
#include <string>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/ui/text/TestRunner.h>
#include <cppunit/TestResult.h>
#include <cppunit/TestResultCollector.h>
#include <cppunit/BriefTestProgressListener.h>
#include <cppunit/XmlOutputter.h>
#include <cppunit/TextOutputter.h>
#include <iostream>
#include <fstream>
#include <stdexcept>
#include "Foundation/testutils/CORBATestManager.hpp"

using namespace std;

int main( int argc, char* argv[] )
{
  
    try
    {
        Cdmw::OrbSupport::StrategyList strategyList;
        strategyList.add_OrbThreaded(); // Mandatory in OrbSupport::ORB_init
        strategyList.add_multicast();
        // Initialises the ORB
        CORBA::ORB_var orb =
            Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, strategyList);
        
        //
        // BEGIN ADDED
        //      
        //
        // Get the root POA
        //
        CORBA::Object_var obj_rootpoa = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var rootPOA =
            PortableServer::POA::_narrow(obj_rootpoa.in());

        //
        // Run implementation (Activate the POA manager)
        //
        PortableServer::POAManager_var poaManager = rootPOA->the_POAManager();
        poaManager->activate();

        Cdmw::TestUtils::CORBATestManager::instance()->init(orb.in(),rootPOA.in(),argc,argv);
 
        //
        // END ADDED
        //    
    
    
        Cdmw::TestUtils::CORBATestManager::instance()->run_tests();
        Cdmw::TestUtils::CORBATestManager::instance()->summary();

        
#ifdef WIN32
        system("PAUSE");
#endif
    orb->shutdown(1);
    orb->destroy();

        return Cdmw::TestUtils::CORBATestManager::instance()->was_successful();
        
    }catch( const CORBA::Exception &e )
        {
            cerr <<__FILE__<<" "<<__LINE__<<" "<< e << endl;
            return -1;
        }
    catch(...)
        {
        cerr << "Uncaught exception" << endl;
        return -1;
        }       
    
}


#else

#include <cppunit/plugin/TestPlugIn.hpp>

// _implements all the plug-in stuffs, WinMain...
CPPUNIT_PLUGIN_IMPLEMENT();

#endif

///////////////////////////////////////////////////////////////////////////////
//  End Of File
///////////////////////////////////////////////////////////////////////////////
    
