/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_LB_GROUP_CREATOR_CONFIGURATIONMNGR_IMPL_HPP 
#define INCL_LB_GROUP_CREATOR_CONFIGURATIONMNGR_IMPL_HPP 

#include "Foundation/common/Exception.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "lbgroupcreator/GroupConfigurationMngr.hpp"
#include <string>
#include <list>
#include <map>

namespace Cdmw
{
namespace LB
{


/**
*Purpose:
*<p> 
*
*Features:
*<p> Thread safe
*
*@see  
*
*/
class ConfigurationMngr
{

public:

    /**
     * Purpose:
     * <p> Constructor. Sets default values.
     * 
     */ 
    ConfigurationMngr()
        throw(OutOfMemoryException);



    /**
     * Purpose:
     * <p> Initializes the configuration from an XML file.
     * 
     * @param xml_conf The XML file path
     * @param validate Validate the XML file before processing
     */ 
    void parse(const std::string& xml_conf, bool validate)
        throw(OutOfMemoryException, Cdmw::LB::GroupCreator::ConfigurationException);


    /**
     * Purpose:
     * <p> Destructor
     * 
     */ 
    virtual 
    ~ConfigurationMngr()
        throw();


    /**
     * Purpose:
     * <p> return the the host for the LBGroupManager
     */
    const char* getLBGroupManagerHost() throw();
    

    /**
     * Purpose:
     * <p> Returns the port for the LBGroupManager
     */ 
    const char* getLBGroupManagerPort() throw();
    
    
    
private:
        
    /**
     * The port for the LBGroupManager.
     */
    std::string m_lb_group_manager_port;

    /**
     * The host for the LBGroupManager 
     */
    std::string m_lb_group_manager_host;


    
}; // End class ConfigurationMngr 
} // End namespace LB
} // End namespace Cdmw
#endif // INCL_LB_GROUP_CREATOR_CONFIGURATIONMNGR_IMPL_HPP

