/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Standard Files 
#include <iostream>
#include <string>

// Cdmw Files
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>

#include <Foundation/orbsupport/StrategyList.hpp>
#include "LoadBalancing/idllib/CdmwLBGroupManager.stub.hpp"
#include "lbgroupcreator/ConfigurationMngr.hpp"
#include "lbgroupcreator/GroupConfigurationMngr.hpp"
#include <Repository/repositoryinterface/RepositoryInterface.hpp>
#include "LoadBalancing/lbcommon/StateTransferConfig.hpp"

#include <xercesc/sax/SAXParseException.hpp>
#include <xercesc/util/PlatformUtils.hpp>
using namespace Cdmw::LB::GroupCreator;
using namespace xercesc;

namespace
{
    std::string CONFIG_OPTION = "--groupConf";
};


using namespace Cdmw;

int main( int argc, char* argv[] )
{
    std::string xml_conf;
    try
    {
        XMLPlatformUtils::Initialize();

        std::string xml_init;
        bool validate = false;

        // checks the xml configuration file option
        std::string xml_init_option =
        Cdmw::OsSupport::OS::get_option_value( argc, argv, Cdmw::Common::Options::PROCESS_XML_FILE_OPTION);
        if ( xml_init_option != "no" )
        {
            if (xml_init_option != "yes")
            {
                if (Cdmw::OsSupport::OS::file_exists(xml_init_option))
                {
                    xml_init = xml_init_option;       
                }
                else
                {
                    std::cerr<<"No init configuration file present"<<std::endl;
                    ::exit(EXIT_FAILURE);
                }
                
            }
            else
            {
                std::cerr<<"No init configuration file present"<<std::endl;
                ::exit(EXIT_FAILURE);
            }
        }

        // checks the xml group configuration file option
        std::string xml_conf_option =
        Cdmw::OsSupport::OS::get_option_value( argc, argv, CONFIG_OPTION);

        if (( xml_conf_option != "no" ) && (xml_conf_option != "yes"))
    {
        if (Cdmw::OsSupport::OS::file_exists(xml_conf_option))
        {
        xml_conf = xml_conf_option;         
        
        // checks the optional validate option
        std::string validateXML_option =
                    Cdmw::OsSupport::OS::get_option_value( argc, argv, Cdmw::Common::Options::VALIDATE_XML_OPTION);
        
        if (validateXML_option != "no")
        {
            validate = true;
        }
        
        }
        else
        {
        std::cerr<<"the XML group configuration file specified doesn't exist"<<std::endl;
        ::exit(EXIT_FAILURE);
        }
        
    }
    else
    {
        std::cerr<<"No group configuration file specified"<<std::endl;
        ::exit(EXIT_FAILURE);
    }

        // Initialises the ORB
        OrbSupport::StrategyList strategyList;
            strategyList.add_OrbThreaded();
            strategyList.add_PoaThreadPerConnection();

        CORBA::ORB_var orb = Cdmw::OrbSupport::OrbSupport::ORB_init(
            argc, argv, strategyList );

        // Get the root POA
        CORBA::Object_var obj = orb->resolve_initial_references( "RootPOA" );
        PortableServer::POA_var rootPOA
            = PortableServer::POA::_narrow( obj.in() );

        Cdmw::LB::ConfigurationMngr conf;
    PortableServer::POAManager_var poaMgr = rootPOA->the_POAManager();
        poaMgr->activate();

        if (!xml_init.empty())
        {
        try{
        conf.parse(xml_init, validate);
        }catch(const ConfigurationException& ex)
        {
        std::cerr <<  "  * ConfigurationException: the XML file used is not the good type of file for the LBGroupManager \n" 
              << ex.getMessage() << std::endl;
        ::exit(EXIT_FAILURE);
        }
        std::string corbaloc_gm("corbaloc::");
        if(strcmp(conf.getLBGroupManagerPort(), "0") && (conf.getLBGroupManagerHost(), "0"))
        {
        corbaloc_gm += conf.getLBGroupManagerHost();
        corbaloc_gm += ":";
        corbaloc_gm += conf.getLBGroupManagerPort();
        corbaloc_gm += "/group_mgr";
        obj = orb->string_to_object(corbaloc_gm.c_str());
        }
        else
        {
        std::cerr <<  " ConfigurationException: the XML file used is not the good type of file for the LBGroupManager \n"  << std::endl;
        ::exit(EXIT_FAILURE);
        }
        

        }
    else
    {

        CORBA::Object_var naming_context_object = orb->resolve_initial_references ("NameService");
        CosNaming::NamingContextExt_var naming_context = CosNaming::NamingContextExt::_narrow (naming_context_object.in ());

        // get reference to Database ObjectGroup from NamingInterface
        const std::string name_full = REPOSITORY_GROUP_MANAGER_GROUP_NAME + "/" + GROUP_MANAGER_OBJECT_GROUP_NAME;
        obj = naming_context->resolve_str(name_full.c_str());
    }
    
        CdmwLB::LBGroupManager_var lb_group_mgr = CdmwLB::LBGroupManager::_narrow(obj.in());

        if (CORBA::is_nil(lb_group_mgr.in()))
        {
            std::cout<<"The LBGroupManager reference can't be resolved"<<std::endl;
            throw CORBA::OBJECT_NOT_EXIST();
        }
        
        // The default Group configuration
        Cdmw::LB::GroupCreator::GroupConfigurationMngr group_conf(lb_group_mgr.in());

        // If an XML Group configuration file has been transmit, we use it to create 
        // default group needed by the system
        if (!xml_conf.empty())
        {
            group_conf.parse(xml_conf, validate);
        }

        Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
        orb->destroy();
        
        XMLPlatformUtils::Terminate();
    }
    catch ( const Cdmw::LB::GroupCreator::ConfigurationException &ex )
    {
        if (ex.getLineNumber() != 0)
        {
            std::cerr <<__FILE__<<" "<<__LINE__<<" "<<"At Line :" << ex.getLineNumber()<<" of "<< xml_conf 
                      << "\n" << ex.getMessage()<< std::endl;
        } else {
            std::cerr <<__FILE__<<" "<<__LINE__<<" "<<ex.getMessage() << std::endl;
        }
        return EXIT_FAILURE;
    }
    catch( const CORBA::Exception &e )
    {
        std::cerr <<__FILE__<<" "<<__LINE__<<" "<< e._name() << std::endl;
        return EXIT_FAILURE;
    }
    catch( const Cdmw::Exception &e )
    {
        std::cerr <<__FILE__<<" "<<__LINE__<<" "<< e.what() << std::endl;
        return EXIT_FAILURE;
    }
    catch( ... )
    {
        std::cerr << "Unexpected exception" << std::endl;
        return EXIT_FAILURE;
    }

    return EXIT_SUCCESS;
}

