/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @brief implementation for TestLBGroupCreator with cppUnit library.
 *
 * @author Raffaella Panella <rpanella@amjsv.it>
 *
 */

// Standard Files
#include <string>

// Cdmw Files
#include <Foundation/common/Options.hpp>
#include <Foundation/common/String.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <LoadBalancing/lbinit/LBServiceInit.hpp>
#include <testlbgroupcreator/TestUtils.hpp>
#include <cppunit/extensions/HelperMacros.h>
#include "LoadBalancing/lbcommon/LBStrategyRegistry.hpp"
#include "LoadBalancing/lbcommon/LBConfiguration.hpp"
#include "Foundation/logging/Logger.hpp"
#include <LoadBalancing/idllib/CdmwLBGroupManager.stub.hpp>
#include "lbcommon/config.hpp"
#include "Foundation/logging/LogManager.hpp"

#include <iostream>
using namespace std;

using namespace Cdmw;


#define ECHO_HEADER() \
    "[Debug] "

#define ECHO_ERROR(comment) \
do {\
    std::cerr << ECHO_HEADER() << " --> " << comment << std::endl;\
} while(0)

#define LB_GROUP_MANAGER_LOGGER "LB_GROUP_MANAGER_LOGGER"

int main( int argc, char* argv[] )
{
    try
    {
    
         
        
        Cdmw::Logging::LogManager::Init(argc, argv);
          
        char** ORB_init_argv = 0;
    int ORB_init_argc = argc;
        std::string str_opt( "" );

        for (int i=0; i<argc; i++)
            str_opt += std::string( argv[i] ) + " ";
        str_opt += std::string( "--LBInterceptorSide=Client");
        
    ORB_init_argc += 1;
        ORB_init_argv = Cdmw::Common::String::to_char_array(
            Cdmw::Common::String::to_strings( str_opt ) );
       
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"   Try to start LB Service (No Exception must be throw)   ");
        CPPUNIT_ASSERT_NO_THROW(Cdmw::LB::LBServiceInit::Init(ORB_init_argc, ORB_init_argv));
        
        // Initialises the ORB
        Cdmw::OrbSupport::StrategyList strategyList;
        strategyList.add_OrbThreaded();
        strategyList.add_PoaThreadPerConnection();

    
        CORBA::ORB_var orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, strategyList);
    
    CORBA::Object_var objref = orb->resolve_initial_references("LBGroupManager");
    CdmwLB::LBGroupManager_var group_mgr = CdmwLB::LBGroupManager::_narrow(objref.in());
    
    PortableGroup::Name name_obj;
    name_obj.length(1);
    name_obj[0].id="hello_group_round_robin";

    objref = group_mgr->get_object_group_ref_from_name(name_obj);
        CdmwLBGroupCreator::HelloInterface_var hello_group = CdmwLBGroupCreator::HelloInterface::_narrow(objref.in());

        //CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER2),"   Try to call hello on Hello Object Group created with RoundRobin policy   ");
    for(int i = 0; i<16; i++)
            {
                CPPUNIT_ASSERT_NO_THROW(hello_group->hello());
            }
       
    
    name_obj[0].id="hello_group_random";

    objref = group_mgr->get_object_group_ref_from_name(name_obj);
        hello_group = CdmwLBGroupCreator::HelloInterface::_narrow(objref.in());
    //CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER2),"   Try to call hello on Hello Object Group created with Random policy   ");
    for(int i = 0; i<16; i++)
            {
                CPPUNIT_ASSERT_NO_THROW(hello_group->hello());
            }
       
    
    }
        
    catch( const CORBA::Exception &e )
    {
        cerr <<__FILE__<<" "<<__LINE__<<" "<< e._name() << endl;
        return -1;
    }
    catch( const Cdmw::Exception &e )
    {
        cerr <<__FILE__<<" "<<__LINE__<<" "<< e.what() << endl;
        return -1;
    }
    catch(...)
    {
        cerr << "Unexpected exception" << endl;
        return -1;
    }
}
    
