/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw;

import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.io.BufferedWriter;
import java.io.FileWriter;
import org.omg.PortableServer.POAHelper;

public class MainServer
{

    final static String FILE_PATH_OPTION = "--FilePath";
    final static String GM_IOR_OPTION    = "--GroupMngRef";
    final static String HOST_NAME_OPTION = "--HostName";

    private static void usage(String program_name)
    {
        System.out
                .println("Usage:\n"
                        + program_name
                        + " options\n"
                        + "\n"
                        + "Options:\n"
                        + "--help                                         Display this information.\n"
                        + "\n"
                        + "-ORBInitRef LBGroupManager=<rep_corbaloc>  ReplicationManager corbaloc\n"
                        + "["
                        + FILE_PATH_OPTION
                        + "]=path          path of the file where hello IOR should be stored.\n"
                        + "["
                        + HOST_NAME_OPTION
                        + "]=hostname      hostname where server is launched.\n");
    }

    public static void main(String[] args)
    {
        // help argument
        System.err
                .println("############################JAVA#########################################");

        if (args.length <= 2)
        {
            usage(args[0]);

        }

        if (args.length >= 2)
        {
            if (args[1].equals("--help"))
            {
                usage(args[0]);

            }
        }

        // get host name
        String host_name = cdmw.ossupport.OS.getOptionValue(args, "--HostName");

        // get Name File to save hello interface
        String file_name = cdmw.ossupport.OS.getOptionValue(args,
                FILE_PATH_OPTION);

        if (file_name.equals("no"))
        {
            file_name = "hello.ior";
        }

        try
        {
            String[] argsNew = new String[(args.length) + 2];
            for (int i = 0; i < args.length; i++)
            {
                argsNew[i] = args[i];
            }
            argsNew[args.length  ] = "--LBInterceptorSide=Server";
            argsNew[args.length+1] = "--LBGroupManager=corbaloc::localhost:5040/group_mgr";


            System.out.println("   Try to start LB Service    ");
            cdmw.lb.lbinit.LBServiceInit.init(argsNew);

            // Initialises the ORB
            cdmw.orbsupport.StrategyList orb_strategies = new cdmw.orbsupport.StrategyList();
            orb_strategies.addORBThreaded();
            orb_strategies.addPOAThreadPerConnection();

            org.omg.CORBA.ORB orb = cdmw.orbsupport.ORBSupport.ORBInit(args, orb_strategies);
     
            org.omg.PortableServer.POA rootPOA = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));
            rootPOA.the_POAManager().activate();

            cdmw.HelloInterface_impl hello_interface = new cdmw.HelloInterface_impl(host_name);
            CdmwLBGroupCreator.HelloInterface hello_interface_ref = hello_interface._this(orb);

            // export the object reference to a file
            String ref_string = orb.object_to_string(hello_interface_ref);
            System.out.println("Export to the object reference to a file OK");

            PrintWriter out = new PrintWriter(new FileWriter(file_name));
            out.println(ref_string);
            out.flush();
            out.close();

            orb.run();
        }
        catch (org.omg.CORBA.SystemException e)
        {
            System.out.println("Errore: " + e);

        }
        catch (Exception e)
        {
            System.out.println("Uncaught exception: " + e);

        }

    }
}
