/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw;

import java.util.Iterator;
import org.omg.CORBA.ORB;
import org.omg.CORBA.ORBPackage.InvalidName;
import org.omg.CosNaming.NameComponent;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;
import org.omg.PortableServer.POAManager;
import org.omg.PortableServer.POAManagerPackage.AdapterInactive;
import cdmw.clock.test.GenericCatalog;
import cdmw.common.BadParameterException;
import cdmw.testutils.Testable;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import cdmw.ossupport.ThreadedProcess;

public class MainTest
{
    public static org.omg.CORBA.ORB orb;
    public static boolean           SUCCESS = true;
    public static ThreadedProcess   tpJava1;
    public static ThreadedProcess   tpJava2;
    public static ThreadedProcess   tpJava3;
    public static ThreadedProcess   tpJava4;
    public static ThreadedProcess   tpJava5;
    public static ThreadedProcess   tp1;
    public static ThreadedProcess   tp2;
    public static ThreadedProcess   tp3;
    public static ThreadedProcess   tp4;
    public static ThreadedProcess   tp5;
    public static ThreadedProcess   tp6;
    public static ThreadedProcess   gmProcess;

    public static void main(String[] args)
    {

        System.out.println("TEST START");
        try
        {
            // Initialises the ORB
            cdmw.orbsupport.StrategyList orb_strategies = new cdmw.orbsupport.StrategyList();
            orb_strategies.addORBThreaded();
            orb_strategies.addPOAThreadPerConnection();

            orb = cdmw.orbsupport.ORBSupport.ORBInit(args, orb_strategies);
            String param = " --CdmwLBXMLFile=CdmwLBGroupManager_conf.xml";
            gmProcess = cdmw.ossupport.OS.createProcess("cdmw_lb_group_manager"
                    + param);
            cdmw.ossupport.OS.sleep(5000);

            // ---
            System.out
                    .println("Try to start LBGroupCreator specifying a not existent group configuration file ()");

            param = "--CdmwXMLFile=CdmwTestLBGroupCreatorInit_conf.xml"
                    + " --groupConf=CdmwTestLBGroupCreator_conf_not_existent.xml"
                    + " --validateXML";

            tp1 = cdmw.ossupport.OS.createProcess("cdmw_lb_group_creator "
                    + param);

            cdmw.ossupport.OS.sleep(2000);

            // ---
            System.out.println("___________________________________");
            System.out
                    .println("\nTry to start LBGroupCreator without specifying a group configuration file ()");

            param = "--CdmwXMLFile=CdmwTestLBGroupCreatorInit_conf.xml"
                    + " --validateXML";

            tp2 = cdmw.ossupport.OS.createProcess("cdmw_lb_group_creator "
                    + param);
            cdmw.ossupport.OS.sleep(2000);

            // ---
            System.out.println("___________________________________");
            System.out
                    .println("\nTry to start LBGroupCreator specifying a wrong group configuration file ()");

            param = "--CdmwXMLFile=CdmwTestLBGroupCreatorInit_conf.xml"
                    + " --groupConf=CdmwTestLBGroupCreator_err_conf.xml"
                    + " --validateXML";

            tp3 = cdmw.ossupport.OS.createProcess("cdmw_lb_group_creator "
                    + param);
            cdmw.ossupport.OS.sleep(2000);

            // ---
            System.out.println("___________________________________");
            System.out
                    .println("\nTry to start LBGroupCreator specifying a wrong init configuration file ()");
            param = "--CdmwXMLFile=CdmwTestLBGroupCreatorInit_err_conf.xml"
                    + " --groupConf=CdmwTestLBGroupCreator_conf.xml"
                    + " --validateXML";

            tp4 = cdmw.ossupport.OS.createProcess("cdmw_lb_group_creator "
                    + param);
            cdmw.ossupport.OS.sleep(2000);

            // ---
            System.out.println("___________________________________");
            System.out
                    .println("\nTry to start LBGroupCreator specifying a not existent init configuration file ()");

            param = "--CdmwXMLFile=CdmwTestLBGroupCreatorInit_not_existent_conf.xml"
                    + " --groupConf=CdmwTestLBGroupCreator_conf.xml"
                    + " --validateXML";

            tp5 = cdmw.ossupport.OS.createProcess("cdmw_lb_group_creator "
                    + param);
            cdmw.ossupport.OS.sleep(2000);

            // ---
            System.out.println("___________________________________");
            System.out
                    .println("\nTry to start LBGroupCreator specifying the correct configuration files ()");

            param = "--CdmwXMLFile=CdmwTestLBGroupCreatorInit_conf.xml"
                    + " --groupConf=CdmwTestLBGroupCreator_conf.xml"
                    + " --validateXML";

            tp6 = cdmw.ossupport.OS.createProcess("cdmw_lb_group_creator "
                    + param);
            cdmw.ossupport.OS.sleep(5000);

            System.out
                    .println("------------ Find Group Manager in the ORB -------------");
            org.omg.CORBA.Object obj = orb
                    .string_to_object("corbaloc::localhost:5040/group_mgr");
            CdmwLB.LBGroupManager group_manager = CdmwLB.LBGroupManagerHelper
                    .narrow(obj);

            if (group_manager.equals(null))
            {
                System.out.println("group_manager null");
                System.exit(0);
            }
            if (group_manager == null)
            {
                System.out.println("group_manager null");
                System.exit(0);
            }

            // Create Server process
            System.out.println("___________________________________");
            System.out.println("\nCreate Server process");

            tpJava1 = cdmw.ossupport.OS
                    .createJavaProcess("cdmw.MainServer --FilePath=hello1 --HostName=host1 -ORBInitRef "
                            + "LBGroupManager=corbaloc::localhost:5040/group_mgr");
            cdmw.ossupport.OS.sleep(1000);
            System.out.println("Add replica n 1");

            tpJava2 = cdmw.ossupport.OS
                    .createJavaProcess("cdmw.MainServer --FilePath=hello2 --HostName=host2 -ORBInitRef "
                            + "LBGroupManager=corbaloc::localhost:5040/group_mgr");
            cdmw.ossupport.OS.sleep(1000);
            System.out.println("Add replica n 2");

            tpJava3 = cdmw.ossupport.OS
                    .createJavaProcess("cdmw.MainServer --FilePath=hello3 --HostName=host3 -ORBInitRef "
                            + "LBGroupManager=corbaloc::localhost:5040/group_mgr");
            cdmw.ossupport.OS.sleep(1000);
            System.out.println("Add replica n 3");

            tpJava4 = cdmw.ossupport.OS
                    .createJavaProcess("cdmw.MainServer --FilePath=hello4 --HostName=host4 -ORBInitRef "
                            + "LBGroupManager=corbaloc::localhost:5040/group_mgr");
            cdmw.ossupport.OS.sleep(1000);
            System.out.println("Add replica n 4");

            tpJava5 = cdmw.ossupport.OS
                    .createJavaProcess("cdmw.MainServer --FilePath=hello5 --HostName=host4 -ORBInitRef "
                            + "LBGroupManager=corbaloc::localhost:5040/group_mgr");

            cdmw.ossupport.OS.sleep(1000);
            System.out.println("Add replica n 5");
            cdmw.ossupport.OS.sleep(4000);

            System.out.println("\nAdd replicas to group");

            org.omg.CORBA.Object hello_object = cdmw.lb.TestUtils.getHelloRefFromFile(orb, "hello1");

            NameComponent[] loc = new NameComponent[3];
            loc[0] = new NameComponent();
            loc[1] = new NameComponent();
            loc[2] = new NameComponent();

            loc[0].id = "HOST1";
            loc[0].kind = "hostname1";
            loc[1].id = "APPL1";
            loc[1].kind = "applicationname1";
            loc[2].id = "PROC1";
            loc[2].kind = "processname1";

            NameComponent[] name_obj_round_robin = new NameComponent[1];
            name_obj_round_robin[0] = new NameComponent();
            name_obj_round_robin[0].id = "hello_group_round_robin";
            name_obj_round_robin[0].kind = "";
            org.omg.CORBA.Object obj_round_robin = group_manager
                    .get_object_group_ref_from_name(name_obj_round_robin);

            NameComponent[] name_obj_random = new NameComponent[1];
            name_obj_random[0] = new NameComponent();
            name_obj_random[0].id = "hello_group_random";
            name_obj_random[0].kind = "";
            org.omg.CORBA.Object obj_random = group_manager
                    .get_object_group_ref_from_name(name_obj_random);

            System.out.println("***Add member to group");

            System.out.println("***Add hello_object to obj_round_robin");

            obj_round_robin = group_manager.add_member(obj_round_robin, loc,
                    hello_object);
            System.out.println("***Add hello_object to obj_random");
            obj_random = group_manager.add_member(obj_random, loc, hello_object);
            System.out.println("***Add hello_object OK");

            hello_object = cdmw.lb.TestUtils.getHelloRefFromFile(orb, "hello2");

            loc[0].id = "HOST2";
            System.out.println("***Add hello2_object to obj_round_robin");
            obj_round_robin = group_manager.add_member(obj_round_robin, loc,hello_object);
            System.out.println("***Add hello2_object to obj_random");
            obj_random = group_manager.add_member(obj_random, loc, hello_object);
            System.out.println("***Add hello2_object OK");

            hello_object = cdmw.lb.TestUtils.getHelloRefFromFile(orb, "hello3");

            loc[0].id = "HOST3";
            System.out.println("***Add hello3_object to obj_round_robin");
            obj_round_robin = group_manager.add_member(obj_round_robin, loc, hello_object);
            System.out.println("***Add hello3_object to obj_random");
            obj_random = group_manager.add_member(obj_random, loc, hello_object);
            System.out.println("***Add hello3_object OK");

            hello_object = cdmw.lb.TestUtils.getHelloRefFromFile(orb, "hello4");

            loc[0].id = "HOST4";
            System.out.println("***Add hello4_object to obj_round_robin");
            obj_round_robin = group_manager.add_member(obj_round_robin, loc, hello_object);
            System.out.println("***Add hello4_object OK");

            hello_object = cdmw.lb.TestUtils.getHelloRefFromFile(orb, "hello5");
            System.out.println("***Add hello5_object to obj_random");
            obj_random = group_manager.add_member(obj_random, loc, hello_object);
            System.out.println("***Add hello5_object OK");

            // Get the root POA
            org.omg.CORBA.Object objPOA = orb.resolve_initial_references("RootPOA");
            org.omg.PortableServer.POA rootPOA = org.omg.PortableServer.POAHelper.narrow(objPOA);

            System.out.println("Run implementation (Activate the POA manager)");
            // Run implementation (Activate the POA manager)
            rootPOA.the_POAManager().activate();

            run(orb, "testlbgroupcreator", args);
            orb.shutdown(true);
            orb.destroy();
            tpJava1.killProcess();
            tpJava2.killProcess();
            tpJava3.killProcess();
            tpJava4.killProcess();
            tpJava5.killProcess();
            tp1.killProcess();
            tp2.killProcess();
            tp3.killProcess();
            tp4.killProcess();
            tp5.killProcess();
            tp6.killProcess();
            gmProcess.killProcess();

        }// fine try

        catch (BadParameterException e)
        {
            System.out.println("BadParameterException, error: " + e);
        }
        catch (org.omg.PortableGroup.ObjectGroupNotFound e)
        {
            System.out.println("ObjectGroupNotFound, error: " + e);
            e.printStackTrace();
        }
        catch (Exception e)
        {
            System.out.println("Exception, error: " + e);
            e.printStackTrace();
        }
    }

    public static boolean run(ORB orb, String testname, String[] args)
            throws AdapterInactive, InvalidName
    {
        //
        // Resolve the Root POA
        //
        org.omg.CORBA.Object obj = orb.resolve_initial_references("RootPOA");
        POA root = POAHelper.narrow(obj);
        POAManager poaManager = root.the_POAManager();

        //
        // Initialize the server so that incoming requests are
        // handled from this point on
        //
        poaManager.activate();

        // Testing 
        runTest(orb, testname, args);
        return SUCCESS;
    }

    public static void runTest(ORB orb, String testname, String[] strArgs)
    {
        try
        {
            // create a generic catalog for tests
            GenericCatalog<Testable> testCtg = new GenericCatalog<Testable>();
            testCtg.add("TestLBGroupCreator", new TestLBGroupCreator());

            Iterator<Testable> i = testCtg.getIterator();
            while (i.hasNext())
            {
                (i.next()).start();
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }
}
