/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
* @file
* @brief Start an LBGroupManager
*
* @author Raffaella Panella <rpanella@selex-si.com>
*/
#include "Foundation/orbsupport/CORBA.hpp"
#include <Foundation/common/String.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/Codec.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/commonsvcs/datastore/StdMapDataStoreFactory.hpp>
#include <LoadBalancing/lbcommon/StateTransferConfig.hpp>
#include <LoadBalancing/lbinit/LBServiceInit.hpp>
#include <LoadBalancing/lbgroupmanager/TypeRepository_impl.hpp>
#include <LoadBalancing/lbgroupmanager/ConfigurationMngr.hpp>
#include <lbgroupcreator/GroupConfigurationMngr.hpp>
#include <iostream>
#include <string>

#include <LoadBalancing/lbcommon/TaoCosNaming.hpp>
#include "orbsvcs/CosNamingC.h"
#define SUCCESS 0;


#include "LoadBalancing/lbgroupmanager/LBGroupManager_impl.hpp"

#include "lbgroupmanager/ServiceLocator_impl.hpp"

#include <iostream>
using namespace Cdmw;
typedef Cdmw::CommonSvcs::DataStore::StdMapDataStoreFactory
< PortableGroup::ObjectGroupId,
CdmwLB::ObjectGroupData> 
DefaultDataStoreFactory;

//using namespace CdmwPlatformMngt;
using namespace Cdmw;
using namespace Common;
using namespace Cdmw::LB;

namespace
{
    std::string GROUP_CONFIG_OPTION = "--groupConf";
};


int main(int argc, char* argv[])
{
    int status = SUCCESS;
    std::string xml_conf;
    bool validate = false;
    
    
    std::string xml_group_conf;

    
    // checks the xml configuration file option
    std::string xml_conf_option =
    Cdmw::OsSupport::OS::get_option_value( argc, argv, Cdmw::Common::Options::LB_GROUP_MANAGER_CONF);

    if ( xml_conf_option != "no" )
    {
        if (xml_conf_option != "yes")
        {
            if (Cdmw::OsSupport::OS::file_exists(xml_conf_option))
            {
                xml_conf = xml_conf_option;

                // checks the optional validate option
                std::string validateXML_option =
                Cdmw::OsSupport::OS::get_option_value( argc, argv, Cdmw::Common::Options::VALIDATE_XML_OPTION);
                
                if (validateXML_option != "no")
                {
                    validate = true;
                }
                
            }
            else
            {
                std::cerr << " ERROR: the xml configuration file doesn't exist" << std::endl ;                    
                return -1;
            }
            
        }
        else
        {
            std::cerr << " ERROR: the xml configuration file must be specified" << std::endl;                    
            return -1;
        }
    }
    else
    {
        std::cerr <<  " ERROR: the xml configuration file must be specified" << std::endl;                    
        return -1;
    }

    // checks the xml group configuration file option
    xml_conf_option =
    Cdmw::OsSupport::OS::get_option_value( argc, argv, GROUP_CONFIG_OPTION);

    if ( xml_conf_option != "no" )
    {
        if (xml_conf_option != "yes")
        {
            if (Cdmw::OsSupport::OS::file_exists(xml_conf_option))
            {
                xml_group_conf = xml_conf_option;          

                // checks the optional validate option
                std::string validateXML_option =
                Cdmw::OsSupport::OS::get_option_value( argc, argv, Cdmw::Common::Options::VALIDATE_XML_OPTION);

                if (validateXML_option != "no")
                {
                    validate = true;
                }

            }
            else
            {
                std::cerr << " ERROR: the xml group configuration file doesn't exist" << std::endl ;                    
                return -1;
            }
            
        }
    }

    // The default configuration
    Cdmw::LB::ConfigurationMngr conf;

    // If an XML configuration file has been transmit, we use it
    if (!xml_conf.empty())
    {
        try{
            conf.parse(xml_conf, validate);
        }catch(const Cdmw::LB::ConfigurationException& ex)
        {
            std::cerr <<  " lbgroupmanger ConfigurationException: the XML file used is not the good type of file for the LBGroupManager  \n" 
            << ex.getMessage() << std::endl;
            return -1;
        }
    }


    Cdmw::LB::GroupCreator::GroupConfigurationMngr group_conf;
    if (!xml_group_conf.empty())
    {
        try{
            group_conf.validate(xml_group_conf);
        }catch(const Cdmw::LB::GroupCreator::ConfigurationException& ex)
        {
            std::cerr <<  " lbgroupmanger (groupconf) ConfigurationException: the XML file used is not the good type of file for the LBGroupManager  \n" 
            << ex.getMessage() << std::endl;
            return -1;
        }
    }

    CORBA::ORB_var orb = CORBA::ORB::_nil();
    
    try{
        
        // set localisation_port
        char** ORB_init_argv = 0;
        int ORB_init_argc=argc;
        std::string str_opt( "" );

        for (int i=0; i<argc; i++)
        str_opt += std::string( argv[i] ) + " ";
        str_opt += std::string(Common::Options::LOCALISATION_SERVICE_OPTION );
        str_opt += "=" ;
        str_opt += conf.getSystemPortString();
        str_opt += " ";
        str_opt += std::string( "--LBInterceptorSide=LBGroupManager");
        str_opt += " ";
        str_opt += std::string( "--CdmwLBStateTransferPort" );
        str_opt += "=" ;
        str_opt += conf.getStateTransferPort();
        ORB_init_argc += 3;
        ORB_init_argv = Cdmw::Common::String::to_char_array(
        Cdmw::Common::String::to_strings( str_opt ) );
        
        Cdmw::LB::LBServiceInit::Init(ORB_init_argc, ORB_init_argv);
        

        Cdmw::OrbSupport::StrategyList orb_strategies;
        orb_strategies.add_OrbThreaded();
        orb_strategies.add_PoaThreadPool( 30 );
        orb_strategies.add_multicast();

        

        orb =
        Cdmw::OrbSupport::OrbSupport::ORB_init(ORB_init_argc, 
        ORB_init_argv,
        orb_strategies); 

        ::Cdmw::OrbSupport::CodecBase::Init(orb.in()); 

        
        CORBA::Object_var obj=
        orb->resolve_initial_references("RootPOA");
        
        PortableGroup::GOA_var rootGOA =
        PortableGroup::GOA::_narrow(obj.in());

        PortableServer::POAManager_var poa_manager =
        rootGOA->the_POAManager();

        poa_manager->activate();

        DefaultDataStoreFactory::Create_data_store(conf.getObjectGroupDataIdentifier());

        // Create the TypeRepository object and register it
        CdmwLB::TypeRepository_var typeRepository =
        new Cdmw::LB::TypeRepository_impl(conf.getSupportedInterfaceMap());
        orb->register_initial_reference("LBTypeRepository", typeRepository.in());
        Cdmw::LB::LBGroupManager_impl*  groupManager = new Cdmw::LB::LBGroupManager_impl(orb.in(), rootGOA.in (), conf.getDomainID(), xml_group_conf, conf.getObjectGroupDataIdentifier());
        
        CdmwLB::LBGroupManager_var gm = groupManager->_this();
        std::cout << "LBGroupManager>> LB Group Manager Started." << std::endl; 

        OrbSupport::OrbSupport::bind_object_to_corbaloc(orb.in(), "group_mgr", gm.in() );
        orb->register_initial_reference("LBGroupManager", gm.in());


        std::string multicast_addr = conf.getLocalisationServiceAddress();
        
        // add multicast address
        if (multicast_addr != "")
        {
            std::cout << "LBGroupManager>> register service " << std::endl;
            std::cout << "LBGroupManager>> multicast_addr  " << multicast_addr << std::endl;

            CORBA::Object_var group = orb->string_to_object(multicast_addr.c_str());
            PortableServer::ObjectId_var  service_locator_id = rootGOA->create_id_for_reference(group.in());

            Cdmw::LB::ServiceLocator_impl* service_locator_obj
            = new  Cdmw::LB::ServiceLocator_impl(orb,
            conf.getDomainID(),
            gm.in());

            rootGOA->activate_object_with_id (service_locator_id.in (),service_locator_obj);
        } else
        std::cout << "LBGroupManager>> LocalisationService not defined  " << std::endl;
        
        
        // bind the NamingContextExt object to corbaloc
        CosNaming::NamingContextExt_var root_nc
        = CosNaming::NamingContextExt::_narrow(gm.in());
        if (!CORBA::is_nil(root_nc.in()))
        OrbSupport::OrbSupport::bind_object_to_corbaloc(orb, "NameService", root_nc.in());


        

        orb->run();
        // free the memory allocated by String::to_char_array
        for (int iArgc=0; iArgc < ORB_init_argc; ++iArgc) {
            delete[] ORB_init_argv[iArgc];
        }
        delete[] ORB_init_argv;

        delete groupManager;

    }
    catch (CORBA::SystemException& e)
    {
        std::cout << "LBGroupManager>> Unhandled CORBA::SystemException..."
        << e << std::endl;
        status = -1;
    }
    
    if(!CORBA::is_nil(orb.in()))
    {
        try{
            Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
            orb->destroy();
        }catch(const CORBA::SystemException& e)
        {
            std::cout << "LBGroupManager>> Unhandled CORBA::SystemException..."
            << e << std::endl;
            status = -1;
        }
    }    
    return status;    
}
