/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
* @brief A Supervision Observer for the LBGroupManager
*
* @author Raffaella Panella <rpanella@amjsv.it>
* @author Antonello Cedrone <a.cedrone@vitrociset.it> 
*/
#include <Foundation/common/String.hpp>
#include <iostream>
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/common/Options.hpp"
#include "Foundation/common/Exception.hpp"
#include <LoadBalancing/lbgroupmanager/TypeRepository_impl.hpp>
#include <LoadBalancing/lbgroupmanager/ConfigurationMngr.hpp>

#include "lbgroupmanager/LBGroupManager_impl.hpp"

#include "SystemMngt/platformvaluetypes/SystemModeChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/SystemStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/HostStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationAgentDeath_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationModeChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/EntityStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/UserEvent_impl.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp"
#include <Foundation/commonsvcs/datastore/StdMapDataStoreFactory.hpp>
#include <LoadBalancing/lbcommon/StateTransferConfig.hpp>
#include <LoadBalancing/lbinit/LBServiceInit.hpp>
#include <lbgroupcreator/GroupConfigurationMngr.hpp>

#include "lbgroupmanager/ServiceLocator_impl.hpp"

using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;
using namespace Cdmw::LB;
using namespace CdmwPlatformMngt;



#include "lbcommon/config.hpp"
#include <Foundation/logging/Logger.hpp>

namespace LOG=Cdmw::Logging;


namespace
{
    std::string GROUP_CONFIG_OPTION = "--groupConf";
};

std::string SYSTEM_CORBALOC_OPTION = "--system-corbaloc";

typedef Cdmw::CommonSvcs::DataStore::StdMapDataStoreFactory
< PortableGroup::ObjectGroupId,
CdmwLB::ObjectGroupData> 
DefaultDataStoreFactory;


int main( int argc, char* argv[] )
{
    int res = 0;
    std::string system_corbaloc;
    std::string observer_name;
    
    
    std::string xml_conf;
    bool validate = false;
    std::string xml_group_conf;
    
    

    // Check command line arguments

    if (argc < 2)
    {
        std::cerr << "Usage: " << "--CdmwLBXMLFile=<LBGroupManager Configuration File>" << std::endl;
        std::cerr << "       " << SYSTEM_CORBALOC_OPTION << "=system_corbaloc" << std::endl;
        std::cerr << std::endl;
        std::cerr << std::endl;
        return -1;
    }
    // checks the xml configuration file option
    std::string xml_conf_option =
    Cdmw::OsSupport::OS::get_option_value( argc, argv, Cdmw::Common::Options::LB_GROUP_MANAGER_CONF);

    if ( xml_conf_option != "no" )
    {
        if (xml_conf_option != "yes")
        {
            if (Cdmw::OsSupport::OS::file_exists(xml_conf_option))
            {
                xml_conf = xml_conf_option;          

                // checks the optional validate option
                std::string validateXML_option =
                Cdmw::OsSupport::OS::get_option_value( argc, argv, Cdmw::Common::Options::VALIDATE_XML_OPTION);

                if (validateXML_option != "no")
                {
                    validate = true;
                }

            }
            else
            {
                std::cerr << "ERROR : the xml configuration file doesn't exist" << std::endl;                    
                return -1;                }
            
        }
        else
        {
            std::cerr << "ERROR : the xml configuration file must be specified" << std::endl;                    
            return -1;
        }
    }
    else
    {
        std::cerr << "ERROR : the xml configuration file must be specified" << std::endl;                    
        return -1;
    }
    // checks the xml group configuration file option
    xml_conf_option =
    Cdmw::OsSupport::OS::get_option_value( argc, argv, GROUP_CONFIG_OPTION);

    if ( xml_conf_option != "no" )
    {
        if (xml_conf_option != "yes")
        {
            if (Cdmw::OsSupport::OS::file_exists(xml_conf_option))
            {
                xml_group_conf = xml_conf_option;          

                // checks the optional validate option
                std::string validateXML_option =
                Cdmw::OsSupport::OS::get_option_value( argc, argv, Cdmw::Common::Options::VALIDATE_XML_OPTION);

                if (validateXML_option != "no")
                {
                    validate = true;
                }

            }
            else
            {
                std::cerr << " ERROR: the xml group configuration file doesn't exist" << std::endl ;                    
                return -1;
            }
            
        }
    }
    // The default configuration
    Cdmw::LB::ConfigurationMngr conf;
    System_var system;
    PortableGroup::GOA_var goa;
    CORBA::ORB_var orb = CORBA::ORB::_nil();
    // If an XML configuration file has been transmit, we use it
    if (!xml_conf.empty())
    {
        try{
            conf.parse(xml_conf, validate);
        }catch(const ConfigurationException& ex)
        {
            std::cerr <<  " ConfigurationException: the XML file used is not the good type of file for the LBGroupManager  \n" 
            << ex.getMessage() << std::endl;
            return -1;
        }
    }
    Cdmw::LB::GroupCreator::GroupConfigurationMngr group_conf;
    if (!xml_group_conf.empty())
    {
        try{
            group_conf.validate(xml_group_conf);
        }catch(const Cdmw::LB::GroupCreator::ConfigurationException& ex)
        {
            std::cerr <<  " ConfigurationException: the XML file used is not the good type of file for the LBGroupManager  \n" 
            << ex.getMessage() << std::endl;
            return -1;
        }
    }
    
    
    try{
        char** ORB_init_argv = 0;
        int ORB_init_argc = argc;
        std::string str_opt( "" );

        for (int i=0; i<argc; i++)
        str_opt += std::string( argv[i] ) + " ";
        str_opt += std::string( "--LBInterceptorSide=LBGroupManager") + " ";
        str_opt += std::string(Common::Options::LOCALISATION_SERVICE_OPTION );
        str_opt += "="; 
        str_opt += conf.getSystemPortString();
        ORB_init_argc += 2;
        ORB_init_argv = Cdmw::Common::String::to_char_array(
        Cdmw::Common::String::to_strings( str_opt ) );

        
        // Initialise ORB
        OrbSupport::StrategyList orbStrategies;
        orbStrategies.add_OrbThreaded();
        orbStrategies.add_PoaThreadPool( 30 );
        orbStrategies.add_multicast();

        Cdmw::LB::LBServiceInit::Init(ORB_init_argc, ORB_init_argv);

        orb = OrbSupport::OrbSupport::ORB_init( ORB_init_argc, ORB_init_argv, orbStrategies );
        ::Cdmw::OrbSupport::CodecBase::Init(orb.in()); 

        // Get a reference to the root POA
        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
        goa = PortableGroup::GOA::_narrow( obj.in() );
        
        // Activate the POA manager
        PortableServer::POAManager_var mgr = goa->the_POAManager();
        mgr->activate();
        
        // Register event factories
        CORBA::ValueFactoryBase_var factory, old_factory;
        
        factory = new SystemModeChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/SystemModeChange:1.0",
                factory.in() );
        
        factory = new SystemStatusChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/SystemStatusChange:1.0",
                factory.in() );
        
        factory = new HostStatusChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/HostStatusChange:1.0",
                factory.in() );
        
        factory = new ApplicationAgentDeathFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationAgentDeath:1.0",
                factory.in() );
        
        factory = new ApplicationModeChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationModeChange:1.0",
                factory.in() );
        
        factory = new ApplicationStatusChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationStatusChange:1.0",
                factory.in() );
        
        factory = new ProcessMessageFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessMessage:1.0",
                factory.in() );
        
        factory = new ProcessStatusChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessStatusChange:1.0",
                factory.in() );
        
        factory = new EntityStatusChangeFactory;
        old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/EntityStatusChange:1.0",
            factory.in() );

        factory = new UserEventFactory;
        old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/UserEvent:1.0",
            factory.in() );
        
        // Resolve the LBGroupManager from corbaloc (ok at compile time)
        
        DefaultDataStoreFactory::Create_data_store(conf.getObjectGroupDataIdentifier());
        
        // Create the TypeRepository object and register it
        CdmwLB::TypeRepository_var typeRepository =
            new Cdmw::LB::TypeRepository_impl(conf.getSupportedInterfaceMap());
        orb->register_initial_reference("LBTypeRepository", typeRepository.in());
        
        std::string register_option =  conf.getRegisterOption();
        std::string unregister_option =  conf.getUnregisterOption();
        // Register the observer to the designated (with corbaloc) system
        if ( register_option.compare("yes") == 0 )
        {
            // Connect to the designated system
            system_corbaloc = OsSupport::OS::get_option_value( argc, argv, SYSTEM_CORBALOC_OPTION );
            observer_name = conf.getObserverName();
            
            
            CORBA::Object_var system_obj
                = orb->string_to_object( system_corbaloc.c_str() );
            
            if ( CORBA::is_nil( system_obj.in() ) )
                CDMW_THROW2(
                    Exception,
                Exception::FATAL,
                "Invalid corbaloc" );
            
            system = System::_narrow( system_obj.in() );
            
            if ( CORBA::is_nil( system.in() ) )
                CDMW_THROW2(
                    Exception,
                    Exception::FATAL,
                    "Invalid system reference" );
            // Register the observer to the system

            Cdmw::LB::LBGroupManager_impl* groupManager = new Cdmw::LB::LBGroupManager_impl(orb.in(), goa.in(), conf.getDomainID(), xml_group_conf, conf.getObjectGroupDataIdentifier(),system);
            CdmwLB::LBGroupManager_var observer  = groupManager->_this();
            orb->register_initial_reference("LBGroupManager", observer.in());
            OrbSupport::OrbSupport::bind_object_to_corbaloc(
                 orb.in(), "group_mgr", observer.in() );

            system->register_observer( observer_name.c_str(),observer.in() );
                    
        try
        {
            CORBA::Any property_any_val;
            property_any_val <<=  conf.getDomainID();

            CDMW_DEBUG(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"LBGroupManager (FT) >> set attribute Cdmw.LBGroupmanager.Configuration.LoadBalancingDomainID = " << conf.getDomainID() );
            system->define_property_with_mode ("Cdmw.LBGroupmanager.Configuration.LoadBalancingDomainID",
                             property_any_val,
                             CosPropertyService::normal
                             );

               system->set_attribute("Cdmw.LBGroupmanager.Configuration.LoadBalancingDomainID", property_any_val , CdmwPlatformMngt::USER_DEFINED);
        }
        catch( const CdmwPlatformMngt::ManagedElementNotFound& e )
        {
            CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"LBGroupManager (FT) >>Cdmw.LBGroupmanager.Configuration.LoadBalancingDomainID  ManagedElementNotFound ");
        }
        catch (const CdmwPlatformMngt::ReadOnlyAttribute& roa)
        {
            CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"LBGroupManager (FT) >> Cdmw.LBGroupmanager.Configuration.LoadBalancingDomainID ReadOnlyAttribute  ");
        }
        catch(const CosPropertyService::InvalidPropertyName &ipn )
        {
            CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"LBGroupManager (FT) >> Cdmw.LBGroupmanager.Configuration.LoadBalancingDomainID InvalidPropertyName  ");
        }
        catch(const CosPropertyService::ConflictingProperty& cp)
        {
            CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"LBGroupManager (FT) >> Cdmw.LBGroupmanager.Configuration.LoadBalancingDomainID ConflictingProperty  ");
        }
        catch(const CosPropertyService::UnsupportedTypeCode utc)
        {
            CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"LBGroupManager (FT) >> Cdmw.LBGroupmanager.Configuration.LoadBalancingDomainID UnsupportedTypeCode ");
        }
        catch (...)
        {
            CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"LBGroupManager (FT) >> Cdmw.LBGroupmanager.Configuration.LoadBalancingDomainID Exception during definition ");
        }        






            // ----- Loaclisation Service ----

            std::string multicast_addr = conf.getLocalisationServiceAddress();
            if (multicast_addr != "")
            {
                std::cout << "LBGroupManager>> register service " << std::endl;
                std::cout << "LBGroupManager>> multicast_addr  " << multicast_addr << std::endl;

                CORBA::Object_var group = orb->string_to_object(multicast_addr.c_str());
                PortableServer::ObjectId_var  service_locator_id = goa->create_id_for_reference(group.in());

                Cdmw::LB::ServiceLocator_impl* service_locator_obj
                = new  Cdmw::LB::ServiceLocator_impl(orb,
                conf.getDomainID(),
                observer.in());

                goa->activate_object_with_id (service_locator_id.in (),service_locator_obj);
            } else
                std::cout << "LBGroupManager>> LocalisationService not defined  " << std::endl;

            // bind the NamingContextExt object to corbaloc
            std::cout << " bind the NamingContextExt object to corbaloc  " << std::endl;
            CosNaming::NamingContextExt_var root_nc
               = CosNaming::NamingContextExt::_narrow(observer.in());
            if (!CORBA::is_nil(root_nc.in()))
               OrbSupport::OrbSupport::bind_object_to_corbaloc(orb, "NameService", root_nc.in());

            CORBA::Any property_any_val;
            property_any_val <<= conf.getDomainID();
            system->set_attribute("Cdmw.LBGroupmanager.Configuration.LoadBalancingDomainID", property_any_val , CdmwPlatformMngt::USER_DEFINED);
        
        






        }


        // Accept requests
        orb->run();
        if (unregister_option.compare("yes") == 0 )

        {
            // Unregister the status change event observer from the system
            system->unregister_observer( observer_name.c_str() );

        }
        // free the memory allocated by String::to_char_array
        for (int iArgc=0; iArgc < ORB_init_argc; ++iArgc) {
            delete[] ORB_init_argv[iArgc];
        }
        delete[] ORB_init_argv;
        
    }
    catch ( const CORBA::Exception &e )
    {
        std::cerr << e._name() << std::endl;
        res = 1;
    }
    catch ( ... )
    {
        res = 1;
    }


    if ( !CORBA::is_nil( orb.in() ) )
    {
        try
        {
            Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
            orb->destroy();
        }
        catch ( const CORBA::Exception &e )
        {
            std::cerr << e << std::endl;
        }
    }

    return res;
}

