/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "lbgroupmanager/ServiceLocator_impl.hpp"
#include <Foundation/common/Assert.hpp>
#include <Foundation/common/Exception.hpp>


#include "lbcommon/config.hpp"
#include <Foundation/logging/Logger.hpp>
#include <sstream>
namespace LOG=Cdmw::Logging;


namespace Cdmw {
namespace LB {

   ServiceLocator_impl::ServiceLocator_impl(CORBA::ORB_ptr orb, const char* group_domain,  CdmwLB::LBGroupManager_ptr lb_group_manager)
      : m_orb(CORBA::ORB::_duplicate(orb)),
        m_group_domain(group_domain),
        m_lb_group_manager(CdmwLB::LBGroupManager::_duplicate(lb_group_manager))
    {
        // Add  CDMW_LOG
    }
    
    
    ServiceLocator_impl::~ServiceLocator_impl()
    {
        // Add  CDMW_LOG
    }
    

    //
    // location service 
    //
    void ServiceLocator_impl::locate_service (const char * name,
                                              const char * sender)
        throw(CORBA::SystemException)
        
    {
        // Add CDMW log
        std::ostringstream oss;

        CORBA::Object_var service_registration_group = m_orb->string_to_object(sender);

        ::CdmwServiceLocator::ServiceRegistration_var sender_ = 
            CdmwServiceLocator::ServiceRegistration::_unchecked_narrow (service_registration_group.in());

        CDMW_DEBUG_3(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),
                      + "request locate_service(" << name << ", " << sender << ")");

        try {
           

           std::string sname(name);
 
           if (sname == std::string(m_group_domain) ) { 
               // Rererence to LB Group Manager
               // return the reference of the object group
               if (!CORBA::is_nil(m_lb_group_manager.in()))
                    sender_->register_service(name, m_lb_group_manager.in());
               else
                  std::cout << "LB Group Manager is NIL!!" << std::endl;    
          }        
        }
        catch(Exception ex)
        {
           //CDMW_DEBUG(FTLogger::GetLogger(),"Received exception '" << ex << "'. Ignored.");
           // do nothing
           std::cout << "LB Group Manager NOT REGISTRED!! (Received exception " 
                     << ex << "ingnored" << std::endl;
           
        }
    }


    
}
}
