/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw;

import java.io.IOException;
import java.util.Iterator;
import org.omg.CORBA.ORB;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;
import org.omg.PortableServer.POAManager;
import org.omg.PortableServer.POAManagerPackage.AdapterInactive;
import org.omg.CORBA.ORBPackage.InvalidName;
import cdmw.testutils.Testable;
import cdmw.ossupport.ThreadedProcess;
import cdmw.clock.test.GenericCatalog;

public class MainTest
{
    static boolean SUCCESS = false;

    public static void main(String[] args) throws Exception
    {
        System.out.println("------------ TEST START-------------");
        cdmw.orbsupport.StrategyList orb_strategies = new cdmw.orbsupport.StrategyList();
        orb_strategies.addORBThreaded();
        orb_strategies.addPOAThreadPerConnection();

        String arguments = " -ORBInitRef LBGroupManager=corbaloc::localhost:5040/group_mgr";
        // Aggiungo dei miei argomenti
        String[] argsNew = new String[(args.length) + 1];
        for (int i = 0; i < args.length; i++)
            argsNew[i] = args[i];
        argsNew[args.length] = arguments;
        org.omg.CORBA.ORB orb = cdmw.orbsupport.ORBSupport.ORBInit(argsNew,
                orb_strategies);

        // Get the root POA
        org.omg.CORBA.Object objPOA = orb.resolve_initial_references("RootPOA");
        org.omg.PortableServer.POA rootPOA = org.omg.PortableServer.POAHelper
                .narrow(objPOA);
        // Run implementation (Activate the POA manager)
        rootPOA.the_POAManager().activate();

        String param = "";
        try
        {
            System.out
                    .println("Try to start LBGroupManager process without specify the XML configuration file");
            System.out
                    .println("The LBGroupManager process should be stopped with an error message...");
            ThreadedProcess process = cdmw.ossupport.OS
                    .createProcess("cdmw_lb_group_manager");
            cdmw.ossupport.OS.sleep(3000);
            System.out.println("____________________________________________");

            System.out
                    .println("Try to start LBGroupManager process using the XML LB configuration file option but without specify an XML configuration file");
            System.out
                    .println("The LBGroupManager process should be stopped with an error message...");
            param = "--CdmwLBXMLFile";
            process = cdmw.ossupport.OS.createProcess("cdmw_lb_group_manager "
                    + param);
            cdmw.ossupport.OS.sleep(3000);
            System.out.println("____________________________________________");

            System.out
                    .println("Try to start LBGroupManager process specifying a not existent XML configuration file");
            System.out
                    .println("The LBGroupManager process should be stopped with an error message...");
            param = "--CdmwLBXMLFile=CdmwLBGroupManager_not_exist_conf.xml";
            process = cdmw.ossupport.OS.createProcess("cdmw_lb_group_manager "
                    + param);
            cdmw.ossupport.OS.sleep(3000);
            System.out.println("____________________________________________");

            System.out
                    .println("PCR-0659: Try to start LBGroupManager process specifying a wrong XML configuration file");
            System.out
                    .println("PCR-0659: The LBGroupManager process should raise a ConfigurationException...");
            param = "--CdmwLBXMLFile=CdmwLBGroupManager_err_test_conf.xml";
            param += " --validateXML";
            process = cdmw.ossupport.OS.createProcess("cdmw_lb_group_manager "
                    + param);
            cdmw.ossupport.OS.sleep(3000);
            System.out.println("____________________________________________");

            System.out
                    .println("Try to start LBGroupManager process specifying a right XML configuration file");
            System.out
                    .println("The LBGroupManager process should be started without errors...");
            param = "--CdmwLBXMLFile=CdmwLBGroupManager_test_conf.xml";
            process = cdmw.ossupport.OS.createProcess("cdmw_lb_group_manager "
                    + param);
            cdmw.ossupport.OS.sleep(3000);
            System.out.println("____________________________________________");

            run(orb, "testlbgroupmanager", argsNew);
            orb.shutdown(true);
            process.killProcess();

        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        catch (InterruptedException e)
        {
            e.printStackTrace();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    public static boolean run(ORB orb, String testname, String[] args)
            throws AdapterInactive, InvalidName
    {
        //
        // Resolve the Root POA
        //
        org.omg.CORBA.Object obj = orb.resolve_initial_references("RootPOA");
        POA root = POAHelper.narrow(obj);
        POAManager poaManager = root.the_POAManager();

        //
        // Initialize the server so that incoming requests are
        // handled from this point on
        //
        poaManager.activate();

        // Testing Clock
        runTest(orb, testname, args);
        return SUCCESS;
    }

    public static void runTest(ORB orb, String testname, String[] strArgs)
    {
        try
        {
            // create a generic catalog for tests
            GenericCatalog<Testable> testCtg = new GenericCatalog<Testable>();
            testCtg.add("TestGenericFactory", new TestGenericFactory(orb,
                    strArgs));
            testCtg.add("TestObjectGroupManager", new TestObjectGroupManager(
                    orb));

            Iterator<Testable> i = testCtg.getIterator();
            while (i.hasNext())
                i.next().start();
        }
        catch (InvalidName in)
        {
            in.printStackTrace();
        }
        // add all tests to the tests catalog
        catch (Exception e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }
}
