/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw;

import org.omg.CORBA.Any;
import org.omg.CORBA.AnyHolder;
import org.omg.CosNaming.NameComponent;
import org.omg.PortableGroup.ObjectNotFound;
import org.omg.PortableGroup.Property;
import org.omg.PortableGroup.PropertyHelper;

public class TestGenericFactory extends cdmw.testutils.Testable
{

    CdmwLB.LBGroupManager M_gm;
    org.omg.CORBA.ORB     M_orb;

    public TestGenericFactory(org.omg.CORBA.ORB orb, String[] args)
            throws Exception
    {
        this.name = "TestGenericFactory";
        this.out = System.out;
        M_orb = orb;
        /*
         * org.omg.CORBA.Object obj =
         * M_orb.resolve_initial_references("LBGroupManager"); M_gm =
         * CdmwLB.LBGroupManagerHelper.narrow(obj);
         */

        org.omg.CORBA.Object obj = M_orb
                .string_to_object("corbaloc::localhost:5040/group_mgr");
        M_gm = CdmwLB.LBGroupManagerHelper.narrow(obj);
        if (M_gm.equals(null)) throw new Exception();
    }

    private void stampaIntestazione(String intestazione)
    {
        System.out.println("********************************************");
        System.out.println("                 " + intestazione);
        System.out.println("********************************************");
    }

    /**
     * @brief test create object group operation.
     * 
     */
    public boolean create_object()
    {
        stampaIntestazione("create_object");
        final String rid = "IDL:CdmwGroupManager/HelloInterface1:1.0";

        AnyHolder factory_creation_id1 = new AnyHolder();
        Property[] crit = new Property[0];
        org.omg.CORBA.Object obj1;

        System.out.println("");
        System.out
                .println("   Create a first Object Group without specifying any LB Policy (default policy must be set)   ");

        // create the first object group
        try
        {

            obj1 = M_gm.create_object(rid, crit, factory_creation_id1);
            if (obj1.equals(null)) return Macro.TEST_NOK;
        }
        catch (Exception e)
        {
            System.out.println("Error in creating object group " + e);
            return Macro.TEST_NOK;
        }
        // System.out.println("obj1: " +obj1.toString());

        // set lb_policy property
        AnyHolder factory_creation_id2 = new AnyHolder();

        Property critPolicy[] = new Property[1];
        critPolicy[0] = new Property();
        String lb_policy = "default";
        Any any = M_orb.create_any();
        any.insert_string(lb_policy);
        critPolicy[0].nam = new NameComponent[1];
        critPolicy[0].nam[0] = new NameComponent("cdmw.lb.LoadBalancingPolicy",
                "");
        critPolicy[0].val = any;

        System.out
                .println(" \n  Create a second Object Group specifying Random LB Policy   ");

        try
        {
            obj1 = M_gm.create_object(rid, critPolicy, factory_creation_id2);
            if (obj1.equals(null))
            {
                System.err.println("********Error, obj1 is null");
                return Macro.TEST_NOK;
            }
        }
        catch (org.omg.CORBA.SystemException e)
        {
            System.out.print("Error in creating object group: " + e);
            e.printStackTrace();
            return Macro.TEST_NOK;
        }
        catch (Exception e)
        {
            System.out.print("Generic error: " + e);
            e.printStackTrace();
            return Macro.TEST_NOK;
        }

        System.out.print("   Delete the first Object Group");
        // Delete the first Object Group
        try
        {
            M_gm.delete_object(factory_creation_id1.value);
        }
        catch (org.omg.CORBA.SystemException e)
        {
            System.out.print("Error in deleting object group: " + e);
            e.printStackTrace();
            return Macro.TEST_NOK;
        }
        catch (Exception e)
        {
            System.out.print("Generic error in deleting object group: " + e);
            e.printStackTrace();
            return Macro.TEST_NOK;
        }

        System.out.print("   Delete the second Object Group");
        System.out.print("");
        // Delete the second Object Group
        try
        {
            M_gm.delete_object(factory_creation_id2.value);
        }
        catch (org.omg.CORBA.SystemException e)
        {
            System.out.print("Error in deleting object group: " + e);
            e.printStackTrace();
            return Macro.TEST_NOK;
        }
        catch (Exception e)
        {
            System.out.print("Generic error: " + e);
            e.printStackTrace();
            return Macro.TEST_NOK;
        }
        return Macro.TEST_OK;
    }

    /**
     * @brief test delete object group operation.
     * 
     */
    public boolean delete_object()
    {
        stampaIntestazione("delete_object");
        // set lb_policy property
        // org.omg.PortableGroup.GenericFactory factory_creation_id1=null;
        AnyHolder factory_creation_id1 = new AnyHolder();

        Property[] critPolicy = new Property[1];
        String lb_policy = "default";
        Any any = M_orb.create_any();
        any.insert_string(lb_policy);
        critPolicy[0] = new Property(new NameComponent[] { new NameComponent(
                "cdmw.lb.LoadBalancingPolicy", "") }, any);

        String rid = "IDL:CdmwGroupManager/HelloInterface1:1.0";
        org.omg.CORBA.Object obj1;
        System.out.println("   Create a  first Object Group   ");
        try
        {
            obj1 = M_gm.create_object(rid, critPolicy, factory_creation_id1);
            if (obj1.equals(null))
            {
                System.err.println("********Error, obj1 is null");
                return Macro.TEST_NOK;
            }
        }
        catch (org.omg.CORBA.SystemException e)
        {
            System.out.print("Error in creating object group: " + e);
            e.printStackTrace();
            return Macro.TEST_NOK;
        }
        catch (Exception e)
        {
            System.out.print("Generic error: " + e);
            e.printStackTrace();
            return Macro.TEST_NOK;
        }

        System.out.print("   Delete the first Object Group");
        // Delete the first Object Group
        try
        {
            M_gm.delete_object(factory_creation_id1.value);
        }
        catch (org.omg.CORBA.SystemException e)
        {
            System.out.print("Error in deleting object group: " + e);
            e.printStackTrace();
            return Macro.TEST_NOK;
        }
        catch (Exception e)
        {
            System.out.print("Generic error: " + e);
            e.printStackTrace();
            return Macro.TEST_NOK;
        }

        System.out
                .println("   Delete again the same Object Group (ObjectNotFound exception must be raised)   ");

        boolean captured = false;
        try
        {
            M_gm.delete_object(factory_creation_id1.value);
        }
        catch (ObjectNotFound e)
        {
            System.out.print("Exception ObjectNotFound captured, OK ");
            captured = true;
        }
        catch (Exception e)
        {
            System.err.print("Generic error, test failed: " + e);
            e.printStackTrace();
            return Macro.TEST_NOK;
        }

        if (!captured)
        {
            System.err
                    .print("Exception ObjectNotFound captured not captured, test failed");
            return Macro.TEST_NOK;
        }
        return Macro.TEST_OK;
    }

    public void doTests()
    {
        setNbOfRequestedTestOK(2);
        check(create_object());
        check(delete_object());

    }
}
