/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw;

import javax.naming.InvalidNameException;

import org.omg.CORBA.Any;
import org.omg.CORBA.AnyHolder;
import org.omg.CORBA.INV_OBJREF;
import org.omg.CORBA.Object;
import org.omg.CORBA.SystemException;
import org.omg.CosNaming.NameComponent;
import org.omg.PortableGroup.CannotMeetCriteria;
import org.omg.PortableGroup.InvalidCriteria;
import org.omg.PortableGroup.InvalidProperty;
import org.omg.PortableGroup.MemberAlreadyPresent;
import org.omg.PortableGroup.MemberNotFound;
import org.omg.PortableGroup.NoFactory;
import org.omg.PortableGroup.ObjectGroupIdHelper;
import org.omg.PortableGroup.ObjectGroupNotFound;
import org.omg.PortableGroup.ObjectNotAdded;
import org.omg.PortableGroup.ObjectNotCreated;
import org.omg.PortableGroup.Property;

import cdmw.commonsvcs.naming.NamingInterface;
import cdmw.testutils.Testable;

public class TestObjectGroupManager extends Testable
{

    public CdmwLB.LBGroupManager            M_gm;
    public org.omg.CORBA.ORB                M_orb;
    public CdmwGroupManager.HelloInterface1 M_obj1;
    public CdmwGroupManager.HelloInterface2 M_obj2;
    public CdmwGroupManager.HelloInterface1 M_obj3;

    public HelloInterface1_impl             obj1_impl;
    public HelloInterface1_impl             obj3_impl;
    public cdmw.HelloInterface2_impl        obj2_impl;

    public TestObjectGroupManager(org.omg.CORBA.ORB orb) throws Exception
    {
        this.name = "TestObjectGroupManager";
        this.out = System.out;
        M_orb = orb;
        /*
         * org.omg.CORBA.Object obj =
         * M_orb.resolve_initial_references("LBGroupManager"); M_gm =
         * CdmwLB.LBGroupManagerHelper.narrow(obj);
         */
        org.omg.CORBA.Object obj = M_orb
                .string_to_object("corbaloc::localhost:5040/group_mgr");
        M_gm = CdmwLB.LBGroupManagerHelper.narrow(obj);

        if (M_gm.equals(null))
        {
            System.out.println("M_gm is null");
            throw new Exception();
        }

        // create two objects hello1
        obj1_impl = new HelloInterface1_impl();
        obj3_impl = new HelloInterface1_impl();

        // create an object hello2
        obj2_impl = new cdmw.HelloInterface2_impl();

        M_obj1 = obj1_impl._this(M_orb);
        M_obj2 = obj2_impl._this(M_orb);
        M_obj3 = obj3_impl._this(M_orb);
    }

    @Override
    public void doTests()
    {
        // TODO Auto-generated method stub
        setNbOfRequestedTestOK(8);
        check(add_member());
        check(remove_member());
        check(get_object_group_id());
        check(get_object_group_ref());
        check(get_object_group_version_from_ref());
        check(get_object_group_version_from_gid());
        check(get_member_ref());
        check(locations_of_members());
    }

    private void stampaIntestazione(String intestazione)
    {
        System.out.println("********************************************");
        System.out.println("                 " + intestazione);
        System.out.println("********************************************");
    }

    /**
     * @brief test add operation.
     * 
     */
    public boolean add_member()
    {

        stampaIntestazione("add_member");
        // set lb_policy property

        String rid = "IDL:CdmwGroupManager/HelloInterface1:1.0";
        AnyHolder factory_creation_id1 = new AnyHolder();
        Property[] critPolicy = new Property[1];
        String lb_policy = "default";
        Any any = M_orb.create_any();
        any.insert_string(lb_policy);
        critPolicy[0] = new Property(new NameComponent[] { new NameComponent(
                "cdmw.lb.LoadBalancingPolicy", "") }, any);

        org.omg.CORBA.Object obj1;

        System.out.println("\nCreate a first Object Group  ");
        // CDMW_DEBUG(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),oss.str());

        // create the first object group
        try
        {
            obj1 = M_gm.create_object(rid, critPolicy, factory_creation_id1);
            if (obj1.equals(null))
            {
                System.out.println("Error obj1 null");
                return false;
            }
        }

        catch (SystemException e)
        {
            System.out.println("Error in creating object group : " + e);
            e.printStackTrace();
            return false;
        }
        catch (Exception e)
        {
            System.out.println("Generic Error in creating object group : " + e);
            e.printStackTrace();
            return false;
        }

        // add a new member on the group
        NameComponent[] loc = new NameComponent[3];
        loc[0] = new NameComponent();
        loc[1] = new NameComponent();
        loc[2] = new NameComponent();
        loc[0].id = "HOST1";
        loc[0].kind = "hostname1";
        loc[1].id = "APPL1";
        loc[1].kind = "applicationname1";
        loc[2].id = "PROC1";
        loc[2].kind = "processname1";

        try
        {
            obj1 = M_gm.add_member(obj1, loc, M_obj1);
            if (obj1.equals(null))
            {
                System.out.println("Error obj1 null");
                return false;
            }
        }
        catch (SystemException e)
        {
            System.out.println("Error in adding object group : " + e);
            e.printStackTrace();
            return false;
        }
        catch (Exception e)
        {
            System.out.println("Error in adding object group : " + e);
            e.printStackTrace();
            return false;
        }

        System.out
                .println("   Add again the same member on the group (No MemberAlreadyPresent exception must be raised --> Force Add Member)   ");
        boolean captured = false;
        try
        {
            obj1 = M_gm.add_member(obj1, loc, M_obj1);
            if (obj1.equals(null))
            {
                System.out.println("Error obj1 null");
                return false;
            }
            captured = true;
        }
        catch (MemberAlreadyPresent e)
        {
            System.out.println("Exception MemberAlreadyPresent captured, OK");
            captured = false;
        }
        catch (SystemException e)
        {
            System.out.println("Error in adding object group : " + e);
            e.printStackTrace();
            return false;
        }
        catch (Exception e)
        {
            System.out.println("Error in adding object group : " + e);
            e.printStackTrace();
            return false;
        }

        if (!captured)
        {
            System.out
                    .println("Exception MemberAlreadyPresent not captured, test failed");
            return false;
        }
        loc[0].id = "HOST2";

        captured = false;
        System.out
                .println("   Add a new member on the group with a bad type Id (ObjectNotAdded exception must be raised)   ");
        try
        {
            obj1 = M_gm.add_member(obj1, loc, M_obj2);
            if (obj1.equals(null))
            {
                System.out.println("Error obj1 null");
                return false;
            }
        }
        catch (ObjectNotAdded e)
        {
            System.out.println("Exception ObjectNotAdded captured, OK");
            captured = true;
        }
        catch (SystemException e)
        {
            System.out.println("Error in adding object group : " + e);
            e.printStackTrace();
            return false;
        }
        catch (Exception e)
        {
            System.out.println("Error in adding object group : " + e);
            e.printStackTrace();
            return false;
        }
        if (!captured)
        {
            System.out
                    .println("Exception ObjectNotAdded not captured, test failed");
            return false;
        }

        System.out.println("   Delete the Object Group   ");
        try
        {
            M_gm.delete_object(factory_creation_id1.value);
        }

        catch (SystemException e)
        {
            System.out.println("Error in adding object group : " + e);
            e.printStackTrace();
            return false;
        }
        catch (Exception e)
        {
            System.out.println("Error in adding object group : " + e);
            e.printStackTrace();
            return false;
        }

        captured = false;
        System.out
                .println("   Add a new menber on the group previously deleted (ObjectGroupNotFound exception must be raised)   ");
        try
        {
            obj1 = M_gm.add_member(obj1, loc, M_obj1);
            if (obj1.equals(null))
            {
                System.out.println("Error obj1 null");
                return false;
            }
        }
        catch (ObjectGroupNotFound e)
        {
            System.out.println("Exception ObjectGroupNotFound captured, OK");
            captured = true;
        }
        catch (SystemException e)
        {
            System.out.println("Error in adding object group : " + e);
            e.printStackTrace();
            return false;
        }
        catch (Exception e)
        {
            System.out.println("Error in adding object group : " + e);
            e.printStackTrace();
            return false;
        }
        if (!captured)
        {
            System.out
                    .println("Exception ObjectGroupNotFound not captured, test failed");
            return false;
        }
        return true;
    }

    /**
     * @brief test remove operation.
     * 
     */
    public boolean remove_member()
    {
        stampaIntestazione("remove_member");
        // set lb_policy property

        String rid = "IDL:CdmwGroupManager/HelloInterface1:1.0";
        AnyHolder factory_creation_id1 = new AnyHolder();
        Property[] critPolicy = new Property[1];
        String lb_policy = "default";
        Any any = M_orb.create_any();
        any.insert_string(lb_policy);
        critPolicy[0] = new Property(new NameComponent[] { new NameComponent(
                "cdmw.lb.LoadBalancingPolicy", "") }, any);

        org.omg.CORBA.Object obj1;

        System.out.println("   Create a first object group   ");
        // create the first object group

        try
        {
            obj1 = M_gm.create_object(rid, critPolicy, factory_creation_id1);
            if (obj1.equals(null))
            {
                System.out.println("Error obj1 null");
                return false;
            }
        }

        catch (SystemException e)
        {
            System.out.println("Error in creating object group : " + e);
            e.printStackTrace();
            return false;
        }
        catch (Exception e)
        {
            System.out.println("Error in creating object group : " + e);
            e.printStackTrace();
            return false;
        }

        // add a new member on the group
        NameComponent[] loc = new NameComponent[3];
        loc[0] = new NameComponent();
        loc[1] = new NameComponent();
        loc[2] = new NameComponent();
        loc[0].id = "HOST1";
        loc[0].kind = "hostname1";
        loc[1].id = "APPL1";
        loc[1].kind = "applicationname1";
        loc[2].id = "PROC1";
        loc[2].kind = "processname1";

        System.out.println("   Add a new member on the Object Group   ");

        try
        {
            obj1 = M_gm.add_member(obj1, loc, M_obj1);
            if (obj1.equals(null))
            {
                System.out.println("Error obj1 null");
                return false;
            }
        }
        catch (SystemException e)
        {
            System.out.println("Error in adding object group : " + e);
            e.printStackTrace();
            return false;
        }
        catch (Exception e)
        {
            System.out.println("Error in adding object group : " + e);
            e.printStackTrace();
            return false;
        }

        System.out
                .println("   Remove the new member on the group (No exception must be raised)  ");

        try
        {
            M_gm.remove_member(obj1, loc);
        }
        catch (SystemException e)
        {
            System.out.println("Error in removing object group : " + e);
            e.printStackTrace();
            return false;
        }
        catch (Exception e)
        {
            System.out.println("Error in removing object group : " + e);
            e.printStackTrace();
            return false;
        }

        System.out
                .println("   Remove again the same member on the group (No MemberNotFound exception must be raised)  ");
        boolean captured = false;
        try
        {
            M_gm.remove_member(obj1, loc);
            captured = true;
        }

        catch (MemberNotFound e)
        {
            System.out.println("Exception MemberNotFound captured, OK ");
            captured = false;
        }
        catch (Exception e)
        {
            System.out
                    .println("Error in removing object group, the error is not  (MemberNotFound exception): "
                            + e);
            e.printStackTrace();
            return false;
        }
        if (!captured)
        {
            System.err
                    .println("Exception MemberNotFound not captured, test failure ");
            return false;
        }

        NameComponent[] emptyloc = new NameComponent[1];
        emptyloc[0] = new NameComponent("", "");
        System.out
                .println("   Remove a member on the group with an empty location(No MemberNotFound exception must be raised)  ");
        captured = false;
        try
        {
            M_gm.remove_member(obj1, emptyloc);
            captured = true;
        }
        catch (MemberNotFound e)
        {
            System.out.println("Exception MemberNotFound captured, OK  ");
            captured = false;
        }
        catch (Exception e)
        {
            System.out
                    .println("Error in removing object group, the error is not  (MemberNotFound exception): "
                            + e);
            e.printStackTrace();
            return false;
        }
        if (!captured)
        {
            System.err
                    .println("Exception MemberNotFound not captured, test failure ");
            return false;
        }

        System.out.println("   Delete the Object Group   ");

        // delete the object group
        try
        {
            M_gm.delete_object(factory_creation_id1.value);
        }

        catch (Exception e)
        {
            System.out.println("Error in deleting object group: " + e);
            e.printStackTrace();
            return false;
        }

        System.out
                .println("   Remove a new member on the group previously deleted (ObjectGroupNotFound exception must be raised)   ");
        captured = false;
        try
        {
            M_gm.remove_member(obj1, loc);
        }
        catch (org.omg.PortableGroup.ObjectGroupNotFound e)
        {
            System.out.println("Exception ObjectGroupNotFound captured, OK");
            captured = true;
        }
        catch (Exception e)
        {
            System.out
                    .println("Error in removing object group, the error is not  (ObjectGroupNotFound exception): "
                            + e);
            e.printStackTrace();
            return false;
        }
        return true;
    }

    /**
     * @brief test get object group id operation.
     * 
     */
    public boolean get_object_group_id()
    {
        stampaIntestazione("get_object_group_id");
        // set lb_policy property

        String rid = "IDL:CdmwGroupManager/HelloInterface1:1.0";
        AnyHolder factory_creation_id1 = new AnyHolder();
        Property[] critPolicy = new Property[1];
        String lb_policy = "default";
        Any any = M_orb.create_any();
        any.insert_string(lb_policy);
        critPolicy[0] = new Property(new NameComponent[] { new NameComponent(
                "cdmw.lb.LoadBalancingPolicy", "") }, any);

        org.omg.CORBA.Object obj1 = null;

        // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        System.out.println("   Create a first object group   ");
        // create the first object group
        try
        {
            obj1 = M_gm.create_object(rid, critPolicy, factory_creation_id1);
            if (obj1.equals(null))
            {
                System.out.println("obj1 is null");
                return false;
            }
        }

        catch (SystemException e)
        {
            System.out.println("Error in creating object group: " + e);
            e.printStackTrace();
        }
        catch (Exception e)
        {
            System.out.println("Error in creating object group: " + e);
            e.printStackTrace();
        }

        // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        System.out.println("   Get_object_group_id on the group...   ");
        long ind = 0;
        try
        {
            ind = M_gm.get_object_group_id(obj1);
        }
        catch (Exception e)
        {
            System.out.println("Errore eccezione generica: " + e);
            return false;
        }
        System.out.println("   ...ObjectGroupId --> " + ind);

        // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        System.out.println("   Delete the first object group   ");

        try
        {
            M_gm.delete_object(factory_creation_id1.value);
        }

        catch (SystemException e)
        {
            System.out.println("Error in deleting object group: " + e);
            e.printStackTrace();
            return false;
        }
        catch (Exception e)
        {
            System.out.println("Error in deleting object group: " + e);
            e.printStackTrace();
            return false;
        }

        // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        System.out
                .println("   Get_object_group_id on the group previously deleted (ObjectGroupNotFound exception must be raised)   ");
        boolean captured = false;
        try
        {
            M_gm.get_object_group_id(obj1);
        }
        catch (ObjectGroupNotFound e)
        {
            System.out.println("Exception ObjectGroupNotFound captured, OK");
            captured = true;
        }
        catch (Exception e)
        {
            System.out
                    .println("Generic exception, exception is not ObjectGroupNotFound, test failed: "
                            + e);
            e.printStackTrace();
            return false;
        }
        if (!captured)
        {
            System.out
                    .println("Exception ObjectGroupNotFound not captured, test failed. Method: get_object_group_id");
            return false;
        }

        // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        System.out
                .println("Get_object_group_id on the Hello object (CORBA.INV_OBJREF exception must be raised)   ");
        captured = false;
        try
        {
            M_gm.get_object_group_id(M_obj1);
        }
        // Ci andrebbe INV_OBJREF
        catch (INV_OBJREF e)
        {
            System.out.println("Exception INV_OBJREF captured, OK");
            captured = true;
        }
        catch (Exception e)
        {
            System.out
                    .println("Generic exception, exception is not INV_OBJREF, test failed: "
                            + e);
            e.printStackTrace();
            return false;
        }
        if (!captured)
        {
            System.out
                    .println("Exception INV_OBJREF not captured, test failed. Method: get_object_group_id");
            return false;
        }
        return true;
    }

    /**
     * @brief test get object group reference operation.
     * 
     */
    public boolean get_object_group_ref()
    {
        stampaIntestazione("get_object_group_ref");
        // set lb_policy property

        String rid = "IDL:CdmwGroupManager/HelloInterface1:1.0";
        AnyHolder factory_creation_id1 = new AnyHolder();
        Property[] critPolicy = new Property[1];
        String lb_policy = "default";
        Any any = M_orb.create_any();
        any.insert_string(lb_policy);
        critPolicy[0] = new Property(new NameComponent[] { new NameComponent(
                "cdmw.lb.LoadBalancingPolicy", "") }, any);

        org.omg.CORBA.Object obj1;

        // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        System.out.println("   Create a first object group   ");

        try
        {
            obj1 = M_gm.create_object(rid, critPolicy, factory_creation_id1);
            if (obj1.equals(null))
            {
                System.out.println("   obj1 null   ");
                return false;
            }
        }

        catch (org.omg.CORBA.SystemException e)
        {
            System.out.println("Error in creating object group: " + e);
            e.printStackTrace();
            return false;
        }
        catch (Exception e)
        {
            System.out
                    .println("Error generic exception in creating object group: "
                            + e);
            e.printStackTrace();
            return false;
        }

        // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        // get_object_group_ref on the group created
        Object objRef;
        System.out.println("   Get_object_group_ref on the group...   ");
        try
        {
            objRef = M_gm.get_object_group_ref(obj1);
        }
        catch (org.omg.CORBA.SystemException e)
        {
            System.out.println("Error in Get_object_group_ref on the group: "
                    + e);
            e.printStackTrace();
            return false;
        }
        catch (Exception e)
        {
            System.out
                    .println("Error generic exception in Get_object_group_ref on the group: "
                            + e);
            e.printStackTrace();
            return false;
        }
        System.out.println("   ...ObjectGroupReference --> " + objRef);

        // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        // get_object_group_ref from the group id

        long objID;
        try
        {
            objID = M_gm.get_object_group_id(obj1);
        }
        catch (Exception e)
        {
            System.out.println("   Ecezione generica:   " + e);
            e.printStackTrace();
            return false;
        }
        System.out
                .println("   Get_object_group_ref_from_id on the group...   ");

        try
        {
            objRef = M_gm.get_object_group_ref_from_id(objID);
        }
        catch (Exception e)
        {
            System.out.println("   Ecezione generica:   " + e);
            e.printStackTrace();
            return false;
        }
        System.out.println("   ...ObjectGroupReference --> " + objRef);

        // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        System.out.println("   Delete the first object group   ");

        // delete the first object group
        try
        {
            M_gm.delete_object(factory_creation_id1.value);
        }

        catch (org.omg.CORBA.SystemException e)
        {
            System.out.println("Error in deleting object group: " + e);
            e.printStackTrace();
            return false;
        }
        catch (Exception e)
        {
            System.out.println("Generic Error in deleting object group: " + e);
            e.printStackTrace();
            return false;
        }

        // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        System.out
                .println("   Get_object_group_ref on the group previously deleted (ObjectGroupNotFound exception must be raised)   ");
        boolean captured = false;
        try
        {
            M_gm.get_object_group_ref(obj1);
        }
        catch (org.omg.PortableGroup.ObjectGroupNotFound e)
        {
            System.out.println("Exception ObjectGroupNotFound captured, OK");
            captured = true;
        }
        catch (Exception e)
        {
            System.err
                    .println("Generic exception, the exception is not ObjectGroupNotFound, test failed: "
                            + e);
            e.printStackTrace();
            return false;
        }
        if (!captured)
        {
            System.err
                    .println("Exception ObjectGroupNotFound not captured, test failed");
        }

        // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        System.out
                .println("   Get_object_group_ref_from_id on the group previously deleted (ObjectGroupNotFound exception must be raised)   ");
        captured = false;
        try
        {
            M_gm.get_object_group_ref_from_id(objID);
        }
        catch (org.omg.PortableGroup.ObjectGroupNotFound e)
        {
            System.out.println("Exception ObjectGroupNotFound captured, OK");
            captured = true;
        }
        catch (Exception e)
        {
            System.err
                    .println("Generic exception, the exception is not ObjectGroupNotFound, test failed: "
                            + e);
            e.printStackTrace();
            return false;
        }
        if (!captured)
        {
            System.err
                    .println("Exception ObjectGroupNotFound not captured, test failed");
        }
        return true;
    }

    /**
     * @brief test get object group version from object group reference
     *        operation.
     * 
     */

    public boolean get_object_group_version_from_ref()
    {
        stampaIntestazione("get_object_group_version_from_ref");
        // set lb_policy property

        String rid = "IDL:CdmwGroupManager/HelloInterface1:1.0";
        AnyHolder factory_creation_id1 = new AnyHolder();
        Property[] critPolicy = new Property[1];
        String lb_policy = "default";
        Any any = M_orb.create_any();
        any.insert_string(lb_policy);
        critPolicy[0] = new Property(new NameComponent[] { new NameComponent(
                "cdmw.lb.LoadBalancingPolicy", "") }, any);

        org.omg.CORBA.Object obj1;

        System.out.println("   Create a first object group   ");

        // create the first object group
        try
        {
            obj1 = M_gm.create_object(rid, critPolicy, factory_creation_id1);
            if (obj1.equals(null))
            {
                System.out.println("Object obj1 null!");
                return false;
            }
        }

        catch (org.omg.CORBA.SystemException e)
        {
            System.out.println("Error in creating object group: " + e);
            e.printStackTrace();
            return false;
        }
        catch (Exception e)
        {
            System.out.println("Error in creating object group: " + e);
            e.printStackTrace();
            return false;
        }

        NameComponent[] loc = new NameComponent[3];
        loc[0] = new NameComponent();
        loc[1] = new NameComponent();
        loc[2] = new NameComponent();
        loc[0].id = "HOST1";
        loc[0].kind = "hostname1";
        loc[1].id = "APPL1";
        loc[1].kind = "applicationname1";
        loc[2].id = "PROC1";
        loc[2].kind = "processname1";

        // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        System.out
                .println("   Get object_group_version_from_ref of the object group created...it should be 1   ");
        int og_version = 0;
        try
        {
            og_version = M_gm.get_object_group_version_from_ref(obj1);
        }
        catch (Exception e)
        {
            System.out.println(" Generic exception, test failed: " + e);
            e.printStackTrace();
            return false;
        }
        System.out.println("   Object Group Version =  " + og_version);
        if (!(og_version == 1))
        {
            System.out
                    .println("Object Group Version should be 1, otherwise Object Group Version = "
                            + og_version);
            return false;
        }

        // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        System.out.println("   Add a new member on the Object Group   ");
        try
        {
            obj1 = M_gm.add_member(obj1, loc, M_obj1);
            if (obj1.equals(null))
            {
                System.out.println("Oggetto obj1 nullo!");
                return false;
            }
        }
        catch (org.omg.CORBA.SystemException e)
        {
            System.out.println("Error in creating object group: " + e);
            e.printStackTrace();
            return false;
        }
        catch (Exception e)
        {
            System.out.println(" Generic exception, test failed: " + e);
            e.printStackTrace();
            return false;
        }

        // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        System.out
                .println("   Get object_group_version_from_ref of the object group modified...it should be 2   ");
        try
        {
            og_version = M_gm.get_object_group_version_from_ref(obj1);
        }
        catch (Exception e)
        {
            System.out.println(" Eccezione generica, test fallito: " + e);
            e.printStackTrace();
            return false;
        }
        System.out.println("   NEW Object Group Version =  " + og_version);
        if (!(og_version == 2))
        {
            System.out
                    .println("Object Group Version should be 2, otherwise Object Group Version = "
                            + og_version);
            return false;
        }

        // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        System.out.println("   Delete the first object group   ");

        // delete the first object group
        try
        {
            M_gm.delete_object(factory_creation_id1.value);
        }

        catch (org.omg.CORBA.SystemException e)
        {
            System.out.println("Error in deleting object group: " + e);
            e.printStackTrace();
            return false;
        }
        catch (Exception e)
        {
            System.out.println("Error in deleting object group: " + e);
            e.printStackTrace();
            return false;
        }

        // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        System.out
                .println("   Get object_group_version_from_ref of the object group previously deleted (ObjectGroupNotFound exception must be raised)   ");
        boolean captured = false;
        try
        {
            M_gm.get_object_group_version_from_ref(obj1);
        }
        catch (org.omg.PortableGroup.ObjectGroupNotFound e)
        {
            System.out.println("Exception ObjectGroupNotFound captured: " + e);
            captured = true;
        }
        catch (Exception e)
        {
            System.out.println(" Generic exception, test failed: " + e);
            e.printStackTrace();
            return false;
        }
        if (!captured)
        {
            System.out
                    .println(" Exception ObjectGroupNotFound not captured, test failed");
        }
        return true;
    }

    /**
     * @brief test get object group version from object group id operation.
     * 
     */

    public boolean get_object_group_version_from_gid()
    {
        stampaIntestazione("get_object_group_version_from_gid");
        // set lb_policy property

        String rid = "IDL:CdmwGroupManager/HelloInterface1:1.0";
        AnyHolder factory_creation_id1 = new AnyHolder();
        Property[] critPolicy = new Property[1];
        String lb_policy = "default";
        Any any = M_orb.create_any();
        any.insert_string(lb_policy);
        critPolicy[0] = new Property(new NameComponent[] { new NameComponent(
                "cdmw.lb.LoadBalancingPolicy", "") }, any);

        org.omg.CORBA.Object obj1;

        System.out.println("   Create a first object group   ");

        // create the first object group
        try
        {
            obj1 = M_gm.create_object(rid, critPolicy, factory_creation_id1);
            if (obj1.equals(null))
            {
                System.out.println("Object obj1 null!");
                return false;
            }
        }

        catch (org.omg.CORBA.SystemException e)
        {
            System.out.println("Error in creating object group: " + e);
            e.printStackTrace();
            return false;
        }
        catch (Exception e)
        {
            System.out.println("Error in creating object group: " + e);
            e.printStackTrace();
            return false;
        }

        NameComponent[] loc = new NameComponent[3];
        loc[0] = new NameComponent();
        loc[1] = new NameComponent();
        loc[2] = new NameComponent();
        loc[0].id = "HOST1";
        loc[0].kind = "hostname1";
        loc[1].id = "APPL1";
        loc[1].kind = "applicationname1";
        loc[2].id = "PROC1";
        loc[2].kind = "processname1";

        int og_version = 0;
        long objID = 0;

        try
        {
            objID = M_gm.get_object_group_id(obj1);
        }
        catch (Exception e)
        {
            System.out.println("Error in get_object_group_id: " + e);
            e.printStackTrace();
            return false;
        }

        System.out
                .println("   Get object_group_version_from_gid of the object group created...it should be 1   ");

        try
        {
            og_version = M_gm.get_object_group_version_from_gid(objID);
        }
        catch (Exception e)
        {
            System.out.println("Error in get_object_group_version_from_gid: "
                    + e);
            e.printStackTrace();
            return false;
        }
        if (!(og_version == 1))
        {
            System.out
                    .println("Object Group Version should be 1, otherwise Object Group Version = "
                            + og_version);
            return false;
        }

        System.out.println("   Object Group Version =  " + og_version);

        System.out.println("   Add a new member on the Object Group   ");
        // add a new member on the group
        try
        {
            obj1 = M_gm.add_member(obj1, loc, M_obj1);
            if (obj1.equals(null))
            {
                System.out.println("Object obj1 null ");
                return false;
            }
        }
        catch (SystemException e)
        {
            System.err.println("Error in adding object group " + e);
            e.printStackTrace();
            return false;
        }
        catch (Exception e)
        {
            System.err.println("Generic Error in adding object group " + e);
            e.printStackTrace();
            return false;
        }

        System.out.println("Get object_group_version_from_ref of the object "
                + "group modified...it should be 2");
        try
        {
            og_version = M_gm.get_object_group_version_from_gid(objID);
        }
        catch (Exception e)
        {
            System.out.println("Error in get_object_group_version_from_gid: "
                    + e);
            e.printStackTrace();
            return false;
        }

        System.out.println("  NEW Object Group Version =  " + og_version);
        System.out.println("   Delete the first object group   ");
        if (!(og_version == 2))
        {
            System.out
                    .println("Object Group Version should be 2, otherwise Object Group Version = "
                            + og_version);
            return false;
        }

        // delete the first object group
        try
        {
            M_gm.delete_object(factory_creation_id1.value);
        }

        catch (SystemException e)
        {
            System.err.println("Error in deleting object group " + e);
            e.printStackTrace();
            return false;
        }
        catch (Exception e)
        {
            System.err.println("Generic Error in deleting object group " + e);
            e.printStackTrace();
            return false;
        }

        boolean captured = false;
        System.out
                .println("   Get object_group_version_from_ref of the object group previously deleted (ObjectGroupNotFound exception must be raised)   ");
        try
        {
            M_gm.get_object_group_version_from_gid(objID);
        }
        catch (ObjectGroupNotFound e)
        {
            System.out.println("Exception ObjectGroupNotFound captured, OK");
            captured = true;
        }
        catch (Exception e)
        {
            System.err.println("Generic Exception captured, failed " + e);
            e.printStackTrace();
            return false;
        }
        if (!captured)
        {
            System.err
                    .println("Exception ObjectGroupNotFound not captured, failed");
            return false;
        }
        return true;
    }

    /**
     * @brief test get member reference operation.
     * 
     */

    public boolean get_member_ref()
    {
        stampaIntestazione("get_member_ref");
        // set lb_policy property

        String rid = "IDL:CdmwGroupManager/HelloInterface1:1.0";
        AnyHolder factory_creation_id1 = new AnyHolder();
        Property[] critPolicy = new Property[1];
        String lb_policy = "default";
        Any any = M_orb.create_any();
        any.insert_string(lb_policy);
        critPolicy[0] = new Property(new NameComponent[] { new NameComponent(
                "cdmw.lb.LoadBalancingPolicy", "") }, any);

        org.omg.CORBA.Object obj1;

        System.out.println("   Create a first object group   ");

        // create the first object group
        try
        {
            obj1 = M_gm.create_object(rid, critPolicy, factory_creation_id1);
            if (obj1.equals(null))
            {
                System.out.println("Object obj1 null!");
                return false;
            }
        }

        catch (org.omg.CORBA.SystemException e)
        {
            System.out.println("Error in creating object group: " + e);
            e.printStackTrace();
            return false;
        }
        catch (Exception e)
        {
            System.out.println("Error in creating object group: " + e);
            e.printStackTrace();
            return false;
        }

        NameComponent[] loc = new NameComponent[3];
        loc[0] = new NameComponent();
        loc[1] = new NameComponent();
        loc[2] = new NameComponent();
        loc[0].id = "HOST1";
        loc[0].kind = "hostname1";
        loc[1].id = "APPL1";
        loc[1].kind = "applicationname1";
        loc[2].id = "PROC1";
        loc[2].kind = "processname1";

        System.out.println("   Add a new member on the Object Group   ");

        // add a new member on the group

        try
        {
            obj1 = M_gm.add_member(obj1, loc, M_obj1);
            if (obj1.equals(null))
            {
                return false;
            }
        }
        catch (org.omg.CORBA.SystemException e)
        {
            System.out.println("Error in adding object group: " + e);
            e.printStackTrace();
            return false;
        }
        catch (Exception e)
        {
            System.out.println("Error in adding object group: " + e);
            e.printStackTrace();
            return false;
        }

        // get_member_ref on the group for the member just added
        System.out
                .println(" get_member_ref on the group for the member just added");

        org.omg.CORBA.Object objRef;

        try
        {
            objRef = M_gm.get_member_ref(obj1, loc);
        }
        catch (Exception e)
        {
            System.out.println(" Generic exception, test failed: " + e);
            e.printStackTrace();
            return false;
        }
        System.out.println("   ...MemberReference --> " + objRef);

        // get_member_ref on the group with a bad location
        // the exception ObjectNotAdded must be raised

        loc[0].id = "HOST2";
        try
        {
            M_gm.get_member_ref(obj1, loc);
        }
        catch (org.omg.PortableGroup.MemberNotFound e)
        {
            System.out.println(" Exception OK: " + e);
        }
        catch (Exception e)
        {
            System.out.println(" Eccezione generica, test fallito: " + e);
            e.printStackTrace();
            return false;
        }

        System.out.println("   Delete the Object Group   ");
        try
        {
            M_gm.delete_object(factory_creation_id1.value);
        }

        catch (org.omg.CORBA.SystemException e)
        {
            System.out.println("Error in deleting object group, test failed: "
                    + e);
            return false;
        }
        catch (Exception e)
        {
            System.out.println("Error in deleting object group, test failed: "
                    + e);
            e.printStackTrace();
            return false;
        }

        System.out
                .println("   Get_member_ref on the group previously deleted (ObjectGroupNotFound exception must be raised)   ");
        boolean captured = false;
        try
        {
            M_gm.get_member_ref(obj1, loc);
        }
        catch (org.omg.PortableGroup.ObjectGroupNotFound e)
        {
            System.out.println("  Exception ObjectGroupNotFound captured, OK: "
                    + e);
            captured = true;
        }
        catch (Exception e)
        {
            System.out.println(" Generic exception, test failed: " + e);
            e.printStackTrace();
            return false;
        }
        return true;
    }

    /**
     * @brief test locations of members operation.
     * 
     */

    public boolean locations_of_members()
    {
        stampaIntestazione("locations_of_members");
        // set lb_policy property
        NameComponent[][] locations;

        String rid = "IDL:CdmwGroupManager/HelloInterface1:1.0";
        AnyHolder factory_creation_id1 = new AnyHolder();
        Property[] critPolicy = new Property[1];
        String lb_policy = "default";
        Any any = M_orb.create_any();
        any.insert_string(lb_policy);
        critPolicy[0] = new Property(new NameComponent[] { new NameComponent(
                "cdmw.lb.LoadBalancingPolicy", "") }, any);

        org.omg.CORBA.Object obj1;

        System.out.println("   Create a first object group   ");

        // create the first object group
        try
        {
            obj1 = M_gm.create_object(rid, critPolicy, factory_creation_id1);
            if (obj1.equals(null))
            {
                System.out.println("Object obj1 null!");
                return false;
            }
        }

        catch (org.omg.CORBA.SystemException e)
        {
            System.out.println("Error in creating object group: " + e);
            e.printStackTrace();
            return false;
        }
        catch (Exception e)
        {
            System.out.println("Error in creating object group: " + e);
            e.printStackTrace();
            return false;
        }

        NameComponent[] loc = new NameComponent[3];
        loc[0] = new NameComponent();
        loc[1] = new NameComponent();
        loc[2] = new NameComponent();
        loc[0].id = "HOST1";
        loc[0].kind = "hostname1";
        loc[1].id = "APPL1";
        loc[1].kind = "applicationname1";
        loc[2].id = "PROC1";
        loc[2].kind = "processname1";

        System.out.println("   Add the first member on the Object Group    ");

        // add a new member on the group

        try
        {
            obj1 = M_gm.add_member(obj1, loc, M_obj1);
            if (obj1.equals(null))
            {
                return false;
            }
        }
        catch (org.omg.CORBA.SystemException e)
        {
            System.err.println("Error in adding object group: " + e);
            e.printStackTrace();
            return false;
        }
        catch (Exception e)
        {
            System.err.println("Error in adding object group: " + e);
            e.printStackTrace();
            return false;
        }

        loc[0].id = "HOST2";
        loc[0].kind = "hostname2";
        loc[1].id = "APPL2";
        loc[1].kind = "applicationname2";
        loc[2].id = "PROC2";
        loc[2].kind = "processname2";

        System.out.println("   Add the second member on the Object Group    ");

        try
        {
            obj1 = M_gm.add_member(obj1, loc, M_obj3);
            if (obj1.equals(null))
            {
                return false;
            }
        }
        catch (org.omg.CORBA.SystemException e)
        {
            System.err.println("Error in adding object group: " + e);
            e.printStackTrace();
            return false;
        }
        catch (Exception e)
        {
            System.err.println("Error in adding object group: " + e);
            e.printStackTrace();
            return false;
        }
        // locations_of_members on the group created,
        // no exception must be raised

        System.out
                .println("   Locations_of_members on the group (No exception must be raised)   ");

        try
        {
            locations = M_gm.locations_of_members(obj1);
        }
        catch (Exception e)
        {
            System.err
                    .println("Error in finding locations of members on the group: "
                            + e);
            e.printStackTrace();
            return false;
        }
        String location = "";
        for (int i = 0; i < locations.length; i++)
        {
            try
            {
                location = NamingInterface.toString(locations[i]);
                System.out.println("       Location Member " + i + " --> "
                        + location);
            }
            catch (Exception e)
            {
                System.out.println("Captured generic exception, test failed");
                e.printStackTrace();
                return false;
            }

        }

        System.out.println("   Delete the Object Group   ");
        // delete the object group
        try
        {
            M_gm.delete_object(factory_creation_id1.value);
        }

        catch (Exception e)
        {
            System.err.println("Error in deleting object group " + e);
            e.printStackTrace();
            return false;
        }
        System.out
                .println("   Locations_of_members on the group previously deleted (ObjectGroupNotFound exception must be raised)    ");
        // get_member_ref on the group previously deleted
        // the PortableGroup.ObjectGroupNotFound exception must be raised

        boolean captured = false;
        try
        {
            locations = M_gm.locations_of_members(obj1);
        }
        catch (ObjectGroupNotFound e)
        {
            System.out.println("Exception ObjectGroupNotFound captured, OK ");
            captured = true;
        }
        catch (Exception e)
        {
            System.err
                    .println("Captured generic exception, the exception is not ObjectGroupNotFound, test failed ");
            return false;
        }
        if (!captured)
        {
            System.err
                    .println("Exception ObjectGroupNotFound not captured, test failed");
            return false;
        }
        return true;
    }
}
