/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @brief client of testlbserviceintit
 */



// Cdmw Files
#include <Foundation/common/Options.hpp>
#include <Foundation/common/String.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <LoadBalancing/lbinit/LBServiceInit.hpp>
#include <cppunit/extensions/HelperMacros.h>
#include "LoadBalancing/lbcommon/LBStrategyRegistry.hpp"
#include "LoadBalancing/lbcommon/LBConfiguration.hpp"
#include <LoadBalancing/idllib/CdmwLBGroupManager.stub.hpp>
#include <testclient/TestHello.stub.hpp>

#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Repository/idllib/CdmwNamingAndRepository.stub.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>
#include <Foundation/orbsupport/CosNaming.stub.hpp>
#include "lbcommon/config.hpp"
#include <Foundation/logging/Logger.hpp>
#include "Foundation/logging/LogManager.hpp"

// Standard Files
#include <fstream>
#include <iostream>
#include <sstream>
#include <string>

namespace LOG=Cdmw::Logging;

using namespace std;

using namespace Cdmw;
#define CHECK "check.txt"

#define ECHO_HEADER() \
    "[Debug] "

#define ECHO_ERROR(comment) \
do {\
    std::cerr << ECHO_HEADER() << " --> " << comment << std::endl;\
} while(0)


#ifndef ENABLE_LB_DEBUG_DUMP

#   define DEBUG_DUMP(comment)
#   define DEBUG_ECHO(comment)

#else

#   define DEBUG_DUMP(comment) \
do {\
    std::cerr << ECHO_HEADER() << " --> " << comment << std::endl;\
} while(0)

#   define DEBUG_ECHO(comment) \
do {\
    std::cerr << comment ;\
} while(0)

#endif

#define SINCRO "sincro.txt"
#define FILELOG "fileLog.txt"

namespace
{
const string GM_IOR_OPTION = "--GroupMngRef";
const string NAMING_REPOS_URL_OPTION = "--namingRepos-url";
} // end anonymous namespace



void writeError(){
        CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"WRITE ERROR IN FILE fileError.txt");
        std::string fileError = "fileError.txt";
        try{
            std::ofstream os(fileError.c_str());
            os << "ERROR \n";
            os.close();
        }
        catch( ... ){
        }    
}

bool checkArrayObject(int calls[], int len){
    for(int i=0;i<len;i++){
        for(int j=i+1;j<len;j++){
            if(calls[i] == calls[j]) {
                return false;
            }    
        }
    }    
    return true;
}


void deleteSincro();
void writeRowInLog(std::string value);
void writeSINCRO();
void writeLog(std::string value);

void writeLog(std::string value){
     ifstream fin;
     fin.open( SINCRO );
     while ( fin.is_open() ) {
          std::cerr << " (cpp object) Waiting for sincronization..\n";
          sleep( 1 );
          fin.open( SINCRO );
     }
     writeSINCRO();
     writeRowInLog(value);
     deleteSincro();
}
    
void writeSINCRO(){
    try{
       std::ofstream os(SINCRO);
       os.close();
    }
    catch( ... ){
    }    
}
    
void writeRowInLog(std::string value){
    try{
       std::ofstream os(FILELOG, ios_base::app);
       std::string l = "___________________________________________________";
       if(os) {
           os << value << "\n";
           os << l << "\n";
           os.close();
       }
     }
     catch( ... ){
     }
}
    
void deleteSincro(){
    OsSupport::OS::unlink(SINCRO);
}




int main( int argc, char* argv[] )
{
    try
    {
        Cdmw::Logging::LogManager::Init(argc, argv);
        char** ORB_init_argv = 0;
        int ORB_init_argc = argc;
        std::string str_opt( "" );

        for (int i=0; i<argc; i++)
            str_opt += std::string( argv[i] ) + " ";
        str_opt += std::string( "--LBInterceptorSide=Client");
        
        ORB_init_argc += 1;
        ORB_init_argv = Cdmw::Common::String::to_char_array(
            Cdmw::Common::String::to_strings( str_opt ) );
       
        CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"   Try to start LB Service Client Side   ");
        Cdmw::LB::LBServiceInit::Init(ORB_init_argc, ORB_init_argv);
        
        // Initialises the ORB
        CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"   Initialises the ORB   ");
        Cdmw::OrbSupport::StrategyList strategyList;
        strategyList.add_OrbThreaded();
        strategyList.add_PoaThreadPerConnection();

        CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"   ORB init   ");
        CORBA::ORB_var orb = Cdmw::OrbSupport::OrbSupport::ORB_init(ORB_init_argc, ORB_init_argv, strategyList);
        
        //--------------------------------------------------------------------------------------------------------------
        
        
        // get ns corbaloc
        string ns_corbaloc = Cdmw::OsSupport::OS::get_option_value ( argc, argv, NAMING_REPOS_URL_OPTION );
        
        CdmwLB::LBGroupManager_var group_mgr = CdmwLB::LBGroupManager::_nil();
        
        if ( (ns_corbaloc == "no"  || ns_corbaloc == "yes") ) {
            CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"   Find group manager   ");
            CORBA::Object_var objrefGM = orb->string_to_object("corbaloc::localhost:5040/group_mgr");
            group_mgr = CdmwLB::LBGroupManager::_narrow(objrefGM.in());
        }
        else{
            
            //std::string namingReposURL = "corbaloc::cardamom07:42562/CdmwRepository";
            CdmwNamingAndRepository::Repository_var repository =
                CdmwNamingAndRepository::Repository::_nil();

            CORBA::Object_var obj2 = orb->string_to_object( ns_corbaloc.c_str() );

            if (CORBA::is_nil(obj2.in()))
            {
                cerr << "File : " << __FILE__ << " Line : " << __LINE__
                    << "could not convert to object NamingAndRepository from string" << endl;
                return -1;
            }
      
            repository = CdmwNamingAndRepository::Repository::_narrow(obj2.in());
            if (CORBA::is_nil(repository.in()))
            {
                cerr << "File : " << __FILE__ << " Line : " << __LINE__
                    << "could not _narrow NamingAndRepository from URL" << endl;
                return -1;
            }

            // ======================================================
            // Initialise the repository interface
            //     "CARDAMOM" has been chosen as the default domain name
            // ======================================================
            NamingAndRepository::RepositoryInterface::init ("CDMW", repository.in());

            // ======================================================
            // Get Naming Interface of PERFTEST name domain
            // ======================================================
            CommonSvcs::Naming::NamingInterface ni =
                NamingAndRepository::RepositoryInterface::get_domain_naming_interface ("lb_group_manager/object_groups");
        
            // ===================================================
            // Get the PingableInterface object from repository
            // ===================================================
  
            CORBA::Object_var objref1 = ni.resolve ("LBGroupManagerGroup");
            group_mgr = CdmwLB::LBGroupManager::_narrow(objref1.in()); 
        
            if (CORBA::is_nil(group_mgr.in()))
            {
                cerr << "File : " << __FILE__ << " Line : " << __LINE__
                    << "could not _narrow object to type LBGroupManager" << endl;
                return -1;
            }
        }
        


        PortableGroup::Name name_obj;
        name_obj.length(1);
        name_obj[0].id="hello_group_round_robin";
        name_obj[0].kind="";
       
        // ===================================================
        //Find object group
        // ===================================================
          CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"   Find object group   ");
        CORBA::Object_var objref = group_mgr->get_object_group_ref_from_name(name_obj);
           CdmwLBInit::HelloInterfaceReturn_var hello_group = CdmwLBInit::HelloInterfaceReturn::_narrow(objref.in());
        

        if(CORBA::is_nil(objref.in())){ 
            CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"   GROUP NOT FOUND  <------------------------------------- ");
            exit(1);
        }

        
        while (true)
        {
            // ===================================================
            // waiting the server process
            // ===================================================
            ifstream fin;
            fin.open( CHECK );
            while ( !fin.is_open() ) 
            {
                std::cerr << "Waiting for Test..\n";
                sleep( 1 );
                fin.open( CHECK );
            }
            std::string tmp; 
            fin >> tmp;
            
            std::ifstream is(CHECK);
            is >> tmp;
            is.close();
            CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"Number of invocation requested from Client: " + tmp);
            int num_invocation = atoi(tmp.c_str());
            sleep(3);
            
            int object_to_call[num_invocation];
            for(int i=0;i<num_invocation;i++) object_to_call[i] = 0;
            // ===================================================
            // object invocation
            // ===================================================
            
            
            
            
            for(int i=0;i<num_invocation;i++){               
                int location = hello_group->hello();
                std::ostringstream str;
                str << "host: " << location  << "  <###########   HELLO   #########> ";
                std::string stamp = str.str();
                writeLog(stamp);
                
                object_to_call[i] = location;
            }
            
            
            
            // ===================================================
            // check the call 
            // ===================================================
            if(!checkArrayObject(object_to_call, num_invocation)) 
                writeError();
 
            // ===================================================
            // remove file check 
            // ===================================================
            OsSupport::OS::unlink(CHECK);
        }
        
        
    }   
    catch( const CORBA::Exception &e )
    {
        cerr <<__FILE__<<" "<<__LINE__<<" "<< e._name() << endl;
        return -1;
    }
    catch( const Cdmw::Exception &e )
    {
        cerr <<__FILE__<<" "<<__LINE__<<" "<< e.what() << endl;
        return -1;
    }
    catch(...)
    {
        cerr << "Unexpected exception" << endl;
        return -1;
    } 
}
    
