/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.lb.test;

import java.util.Iterator;

import org.omg.CORBA.ORB;
import org.omg.CORBA.Object;
import org.omg.CORBA.ORBPackage.InvalidName;
import org.omg.CosNaming.NameComponent;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;
import org.omg.PortableServer.POAManager;
import org.omg.PortableServer.POAManagerPackage.AdapterInactive;
import cdmw.common.BadParameterException;
import cdmw.ossupport.ThreadedProcess;
import cdmw.testutils.Testable;

public class MainTest
{
    public static org.omg.CORBA.ORB    orb;
    public static boolean              SUCCESS = true;
    public static ThreadedProcess      tpJava1;
    public static ThreadedProcess      tpJava2;
    public static ThreadedProcess      tpJava3;
    public static ThreadedProcess      tpJava4;
    public static ThreadedProcess      tpJava5;
    public static ThreadedProcess      gmProcess;
    private static WriteSincronizedLog logWriter;
    private static String              sincro  = "sincro.txt";
    private static String              fileLog = "fileLog.txt";
    private static String              fileError = "fileError.txt";

    public static void main(String[] args)
    {
        cdmw.ossupport.OS.unlink(fileLog);
        delFile();
        try
        {
            cdmw.ossupport.OS.sleep(500);
        }
        catch (InterruptedException e1)
        {
            // TODO Auto-generated catch block
            e1.printStackTrace();
        }
        System.out.println("TEST START");

        logWriter = new WriteSincronizedLog(sincro, fileLog);
        // Wait the file log
        logWriter.waitFileLog();
        logWriter.writeLog("TEST START");
        try
        {
            // Initialises the ORB
            cdmw.orbsupport.StrategyList orb_strategies = new cdmw.orbsupport.StrategyList();
            orb_strategies.addORBThreaded();
            orb_strategies.addPOAThreadPerConnection();

            orb = cdmw.orbsupport.ORBSupport.ORBInit(args, orb_strategies);
            String param = " --CdmwLBXMLFile=CdmwLBGroupManager_conf.xml --groupConf=CdmwTestLBInitGroupCreator_conf.xml";
            gmProcess = cdmw.ossupport.OS.createProcess("cdmw_lb_group_manager"
                    + param);
            cdmw.ossupport.OS.sleep(20000);

            logWriter
                    .writeAsincrono("Find the group manager and create the default policy group");

            System.out
                    .println("------------ Find Group Manager in the ORB -------------");
            org.omg.CORBA.Object obj = orb
                    .string_to_object("corbaloc::localhost:5040/group_mgr");
            CdmwLB.LBGroupManager group_manager = CdmwLB.LBGroupManagerHelper
                    .narrow(obj);

            if (group_manager.equals(null))
            {
                System.out.println("group_manager null");
                System.exit(0);
            }
            if (group_manager == null)
            {
                System.out.println("group_manager null");
                System.exit(0);
            }

            logWriter.writeAsincrono("Group manager find!!!");

            // Create Server process
            System.out.println("___________________________________");
            System.out.println("\nCreate Server process");

            tpJava1 = cdmw.ossupport.OS
                    .createJavaProcess("cdmw.lb.test.MainServer --FilePath=hello1 --HostName=host1 -ORBInitRef "
                            + "LBGroupManager=corbaloc::localhost:5040/group_mgr");
            cdmw.ossupport.OS.sleep(1000);
            System.out.println("Create replica n 1");

            tpJava2 = cdmw.ossupport.OS
                    .createJavaProcess("cdmw.lb.test.MainServer --FilePath=hello2 --HostName=host2 -ORBInitRef "
                            + "LBGroupManager=corbaloc::localhost:5040/group_mgr");
            cdmw.ossupport.OS.sleep(1000);
            System.out.println("Create replica n 2");

            tpJava3 = cdmw.ossupport.OS
                    .createJavaProcess("cdmw.lb.test.MainServer --FilePath=hello3 --HostName=host3 -ORBInitRef "
                            + "LBGroupManager=corbaloc::localhost:5040/group_mgr");
            cdmw.ossupport.OS.sleep(1000);
            System.out.println("Create replica n 3");

            tpJava4 = cdmw.ossupport.OS
                    .createJavaProcess("cdmw.lb.test.MainServer --FilePath=hello4 --HostName=host4 -ORBInitRef "
                            + "LBGroupManager=corbaloc::localhost:5040/group_mgr");
            cdmw.ossupport.OS.sleep(1000);
            System.out.println("Create replica n 4");

            cdmw.ossupport.OS.sleep(4000);

            System.out.println("\nAdd replicas to group");

            org.omg.CORBA.Object hello_object = cdmw.lb.test.TestUtils
                    .getHelloRefFromFile(orb, "hello1");

            NameComponent[] loc = new NameComponent[3];
            loc[0] = new NameComponent();
            loc[1] = new NameComponent();
            loc[2] = new NameComponent();

            loc[0].id = "HOST1";
            loc[0].kind = "hostname1";
            loc[1].id = "APPL1";
            loc[1].kind = "applicationname1";
            loc[2].id = "PROC1";
            loc[2].kind = "processname1";

            NameComponent[] name_obj_round_robin = new NameComponent[1];
            name_obj_round_robin[0] = new NameComponent();
            name_obj_round_robin[0].id = "hello_group_round_robin";
            name_obj_round_robin[0].kind = "";
            org.omg.CORBA.Object obj_round_robin = group_manager
                    .get_object_group_ref_from_name(name_obj_round_robin);

            NameComponent[] name_obj_random = new NameComponent[1];
            name_obj_random[0] = new NameComponent();
            name_obj_random[0].id = "hello_group_random";
            name_obj_random[0].kind = "";
            org.omg.CORBA.Object obj_random = group_manager
                    .get_object_group_ref_from_name(name_obj_random);

            Object a = group_manager.get_object_group_ref(obj_random);
            System.out.println(+orb.object_to_string(a).length() + "GRUPPO: "
                    + orb.object_to_string(a));
            logWriter.writeAsincrono("***Add hello_object to obj_round_robin");
            System.out.println("***Add hello_object to obj_round_robin");
            System.out.println(hello_object.toString().length() + " ---> "
                    + hello_object.toString());
            obj_round_robin = group_manager.add_member(obj_round_robin, loc,
                    hello_object);
            System.out.println("***Add hello1_object OK");
            logWriter.writeAsincrono("***Add hello1_object OK");

            // Get the root POA
            org.omg.CORBA.Object objPOA = orb
                    .resolve_initial_references("RootPOA");
            org.omg.PortableServer.POA rootPOA = org.omg.PortableServer.POAHelper
                    .narrow(objPOA);

            System.out.println("Run implementation (Activate the POA manager)");
            // Run implementation (Activate the POA manager)
            rootPOA.the_POAManager().activate();

            logWriter.writeAsincrono("Start TestLBInitServer");
            // Libera il logfile
            logWriter.releaseFileLog();
            run(orb, "TestLBInitServer", args);
            orb.shutdown(true);
            orb.destroy();
            tpJava1.killProcess();
            tpJava2.killProcess();
            tpJava3.killProcess();
            tpJava4.killProcess();
            gmProcess.killProcess();
            delFile();
            logWriter.writeLog("END!!!");
            logWriter.releaseFileLog();

        }// fine try

        catch (BadParameterException e)
        {
            System.out.println("BadParameterException, error: " + e);
        }
        catch (org.omg.PortableGroup.ObjectGroupNotFound e)
        {
            System.out.println("ObjectGroupNotFound, error: " + e);
            e.printStackTrace();
        }
        catch (Exception e)
        {
            System.out.println("Exception, error: " + e);
            e.printStackTrace();
        }
    }

    
    public static void delFile(){
        cdmw.ossupport.OS.unlink("hello1");
        cdmw.ossupport.OS.unlink("hello2");
        cdmw.ossupport.OS.unlink("hello3");
        cdmw.ossupport.OS.unlink("hello4");
        cdmw.ossupport.OS.unlink("hello6");
        cdmw.ossupport.OS.unlink("hello7");
        cdmw.ossupport.OS.unlink("hello8");
        cdmw.ossupport.OS.unlink("hello9");
        cdmw.ossupport.OS.unlink(sincro);
        cdmw.ossupport.OS.unlink(fileError);
    }
    public static boolean run(ORB orb, String testname, String[] args)
            throws AdapterInactive, InvalidName
    {
        //
        // Resolve the Root POA
        //
        org.omg.CORBA.Object obj = orb.resolve_initial_references("RootPOA");
        POA root = POAHelper.narrow(obj);
        POAManager poaManager = root.the_POAManager();

        //
        // Initialize the server so that incoming requests are
        // handled from this point on
        //
        poaManager.activate();

        // Testing
        runTest(orb, testname, args);
        return SUCCESS;
    }

    public static void runTest(ORB orb, String testname, String[] strArgs)
    {
        try
        {
            // create a generic catalog for tests
            GenericCatalog<Testable> testCtg = new GenericCatalog<Testable>();
            testCtg.add("TestLBInitServer", new TestLBInitServer(orb));

            Iterator<Testable> i = testCtg.getIterator();
            while (i.hasNext())
            {
                (i.next()).start();
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }
}
