/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.lb.test;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import org.omg.CORBA.Object;
import org.omg.CosNaming.NameComponent;
import org.omg.PortableGroup.MemberAlreadyPresent;
import org.omg.PortableGroup.MemberNotFound;
import org.omg.PortableGroup.ObjectGroupNotFound;
import org.omg.PortableGroup.ObjectNotAdded;
import cdmw.ossupport.ThreadedProcess;
import cdmw.testutils.Testable;

public class TestLBInitServer extends Testable
{

    org.omg.CORBA.Object               objGroup  = null;
    org.omg.CORBA.ORB                  orb       = null;
    CdmwLBInit.HelloInterface          helloRef  = null;
    CdmwLB.LBGroupManager              group_manager;
    public String                      fileCheck = "fileCheck.txt";
    private String                     fileError = "fileError.txt";
    private String                     fileLog   = "fileLog.txt";
    private String                     sincro    = "sincro.txt";
    private int                        sleepTime = 15000;
    public static ThreadedProcess      processClient;
    public static ThreadedProcess      processCreateCppServant1;
    public static ThreadedProcess      processCreateCppServant2;
    public static ThreadedProcess      processCreateCppServant3;
    public static ThreadedProcess      processCreateCppServant4;
    private static WriteSincronizedLog logWriter;
    private static int invocation = 0;

    public TestLBInitServer(org.omg.CORBA.ORB orb)
    {
        this.orb = orb;
        this.name = "TestLBInitServer";
        this.out = System.out;
        objGroup = orb.string_to_object("corbaloc::localhost:5040/group_mgr");
        group_manager = CdmwLB.LBGroupManagerHelper.narrow(objGroup);
        logWriter = new WriteSincronizedLog(sincro, fileLog);
    }

    public void doTests()
    {
        setNbOfRequestedTestOK(4);
        check(lbInitNoParam());
        check(lbInitBadParam());
        check(lbInitClientParam());
        check(addElemGroupCheckIOGR());
    }

    /**
     * Try to start LB Service without the --LBInterceptorSide option
     * (CORBA.INTERNAL Exception should be throw) "
     * 
     * @return
     */
    public boolean lbInitNoParam()
    {
        System.out
                .println("   Try to start LB Service without the --LB Server Side options (CORBA.INTERNAL Exception should be throw)   ");

        boolean tmp = false;
        String[] args = new String[0];
        try
        {
            cdmw.lb.lbinit.LBServiceInit.init(args);
        }
        catch (org.omg.CORBA.INTERNAL i)
        {
            tmp = true;
        }
        if (!tmp)
        {
            return false;
        }
        return true;
    }

    /**
     * Try to start LB Service with the --LBInterceptorSide option with no value
     * (CORBA.INTERNAL Exception should be throw)
     */
    public boolean lbInitBadParam()
    {
        System.out
                .println("   Try to start LB Service with the --LBInterceptorSide option with no value (CORBA.INTERNAL Exception should be throw)   ");
        String[] args = new String[1];
        args[0] = "--LBInterceptorSide";
        boolean tmp = false;
        try
        {
            cdmw.lb.lbinit.LBServiceInit.init(args);
        }
        catch (org.omg.CORBA.INTERNAL i)
        {
            tmp = true;
        }
        if (!tmp)
        {
            return false;
        }
        return true;
    }

    /**
     * Try to start LB Service with the --LBInterceptorSide=Client (No
     * implement) option with no value (CORBA.INTERNAL Exception should be
     * throw)
     * 
     * @return
     */
    public boolean lbInitClientParam()
    {
        System.out
                .println("   Try to start LB Service with the --LBInterceptorSide=Client (No implement) option with no value (CORBA.INTERNAL Exception should be throw)   ");
        String[] args = new String[1];
        args[0] = "--LBInterceptorSide=Client";
        boolean tmp = false;
        try
        {
            cdmw.lb.lbinit.LBServiceInit.init(args);
        }
        catch (org.omg.CORBA.INTERNAL i)
        {
            tmp = true;
        }
        if (!tmp)
        {
            return false;
        }
        return true;
    }

    public boolean addElemGroupCheckIOGR()
    {
        cdmw.ossupport.OS.unlink(fileCheck);
        cdmw.ossupport.OS.unlink(fileError);

        logWriter.writeOne("Start the cpp client");
        // Start the cpp client
        if (!startCppClient())
        {
            return false;
        }
        // wait the execution of client cpp
        sleep(5000);
        org.omg.CORBA.Object defaultPolicyGroup = getGroup();

        NameComponent[] loc = new NameComponent[3];
        loc[0] = new NameComponent();
        loc[1] = new NameComponent();
        loc[2] = new NameComponent();

        loc[0].id = "HOST2";
        loc[0].kind = "hostname2";
        loc[1].id = "APPL1";
        loc[1].kind = "applicationname1";
        loc[2].id = "PROC1";
        loc[2].kind = "processname1";
        org.omg.CORBA.Object hello_object_tmp = cdmw.lb.test.TestUtils
                .getHelloRefFromFile(orb, "hello2");
        logWriter.waitFileLog();
        System.out.println("***Add hello2_object");
        logWriter.writeLog("***Add hello2_object");
        
       
        
        
        
        try
        {
            defaultPolicyGroup = group_manager.add_member(defaultPolicyGroup,
                    loc, hello_object_tmp);
                
            System.out.println("Added memeber HOST2.hostname2/APPL1.applicationname1/PROC1.processname1");        
        }
        catch (MemberAlreadyPresent e2)
        {
            e2.printStackTrace();
        }
        catch (ObjectGroupNotFound e2)
        {
            e2.printStackTrace();
        }
        catch (ObjectNotAdded e2)
        {
            e2.printStackTrace();
        }
        catch (org.omg.CORBA.TRANSIENT  t)
        {
            System.out.println("Error LBGroupManager not reachable!");
            System.exit(-1);
        }
        
        System.out.println("***Add hello2_object OK");
        logWriter.writeAsincrono("***Add hello2_object OK");
        logWriter.releaseFileLog();
        System.out.println("***"
                + orb.object_to_string(defaultPolicyGroup).length());
        sleep(1000);

        writeCHECK(2);

        loc[0].id = "HOST3";
        hello_object_tmp = cdmw.lb.test.TestUtils.getHelloRefFromFile(orb,
                "hello3");
        System.out.println("***Add hello3_object");
        logWriter.writeLog("***Add hello3_object");
        try
        {
            defaultPolicyGroup = group_manager.add_member(defaultPolicyGroup,
                    loc, hello_object_tmp);
        }
        catch (MemberAlreadyPresent e2)
        {
            e2.printStackTrace();
        }
        catch (ObjectGroupNotFound e2)
        {
            e2.printStackTrace();
        }
        catch (ObjectNotAdded e2)
        {
            e2.printStackTrace();
        }
        catch (org.omg.CORBA.TRANSIENT  t)
        {
             
              
                         
            System.out.println("Error LBGroupManager not reachable!");
            System.exit(-1);
        }        
        System.out.println("***Add hello3_object OK");
        logWriter.writeAsincrono("***Add hello3_object OK");
        logWriter.releaseFileLog();
        System.out.println("***"
                + orb.object_to_string(defaultPolicyGroup).length());
        // writeLog(orb.object_to_string(defaultPolicyGroup)+" \n Lenght:
        // "+orb.object_to_string(defaultPolicyGroup).length());

        sleep(1000);
        // Write the check file, sleep the server for 15 sec, so start the
        // client
        writeCHECK(3);

        loc[0].id = "HOST4";
        hello_object_tmp = cdmw.lb.test.TestUtils.getHelloRefFromFile(orb,
                "hello4");
        System.out.println("***Add hello4_object");
        logWriter.writeLog("***Add hello4_object");
        try
        {
            defaultPolicyGroup = group_manager.add_member(defaultPolicyGroup,
                    loc, hello_object_tmp);
        }
        catch (MemberAlreadyPresent e2)
        {
            e2.printStackTrace();
        }
        catch (ObjectGroupNotFound e2)
        {
            e2.printStackTrace();
        }
        catch (ObjectNotAdded e2)
        {
            e2.printStackTrace();
        }
        catch (org.omg.CORBA.TRANSIENT  t)
        {

            System.out.println("Error LBGroupManager not reachable!");
            System.exit(-1);
        }        
        System.out.println("***Add hello4_object OK");
        logWriter.writeAsincrono("***Add hello4_object OK");
        logWriter.releaseFileLog();
        System.out.println("***"
                + orb.object_to_string(defaultPolicyGroup).length());

        sleep(1000);
        writeCHECK(4);

        logWriter.writeLog("***Delete hello4_object");
        try
        {
            defaultPolicyGroup = group_manager.remove_member(
                    defaultPolicyGroup, loc);
        }
        catch (ObjectGroupNotFound onf)
        {
            onf.printStackTrace();
            return false;
        }
        catch (MemberNotFound e)
        {
            e.printStackTrace();
            return false;
        }
        catch (org.omg.CORBA.TRANSIENT  t)
        {

            System.out.println("Error LBGroupManager not reachable!");
            System.exit(-1);
        }        
        logWriter.writeAsincrono("***Delete hello4_object OK");
        logWriter.releaseFileLog();

        sleep(1000);

        writeCHECK(3);

        // Add a cpp servant
        logWriter.writeLog("Start test_server_object_cpp_creator");
        try
        {
            processCreateCppServant1 = cdmw.ossupport.OS
                    .createProcess("test_server_object_cpp_creator --FilePath=hello6 --HostName=host6 -ORBInitRef "
                            + "LBGroupManager=corbaloc::localhost:5040/group_mgr");
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }

        loc[0].id = "HOST6";
        hello_object_tmp = cdmw.lb.test.TestUtils.getHelloRefFromFile(orb,
                "hello6");
        System.out.println("***Add hello6_object cpp");
        logWriter.writeAsincrono("***Add hello6_object cpp");
        try
        {
            defaultPolicyGroup = group_manager.add_member(defaultPolicyGroup,
                    loc, hello_object_tmp);
        }
        catch (MemberAlreadyPresent e2)
        {
            e2.printStackTrace();
        }
        catch (ObjectGroupNotFound e2)
        {
            e2.printStackTrace();
        }
        catch (ObjectNotAdded e2)
        {
            e2.printStackTrace();
        }
        catch (org.omg.CORBA.TRANSIENT  t)
        {

            System.out.println("Error LBGroupManager not reachable!");
            System.exit(-1);
        }   
        logWriter.writeAsincrono("***Add hello6_object cpp OK");
        System.out.println("***Add hello6_object cpp OK");
        logWriter.releaseFileLog();
        writeCHECK(4);

        // Add a cpp servant
        logWriter.writeLog("Start test_server_object_cpp_creator");
        try
        {
            processCreateCppServant2 = cdmw.ossupport.OS
                    .createProcess("test_server_object_cpp_creator --FilePath=hello7 --HostName=host7 -ORBInitRef "
                            + "LBGroupManager=corbaloc::localhost:5040/group_mgr");
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }

        loc[0].id = "HOST7";
        hello_object_tmp = cdmw.lb.test.TestUtils.getHelloRefFromFile(orb,
                "hello7");
        System.out.println("***Add hello7_object cpp");
        logWriter.writeAsincrono("***Add hello7_object cpp");
        try
        {
            defaultPolicyGroup = group_manager.add_member(defaultPolicyGroup,
                    loc, hello_object_tmp);
        }
        catch (MemberAlreadyPresent e2)
        {
            e2.printStackTrace();
        }
        catch (ObjectGroupNotFound e2)
        {
            e2.printStackTrace();
        }
        catch (ObjectNotAdded e2)
        {
            e2.printStackTrace();
        }
        catch (org.omg.CORBA.TRANSIENT  t)
        {   
            System.out.println("Error LBGroupManager not reachable!");
            System.exit(-1);
        }
        logWriter.writeAsincrono("***Add hello7_object cpp OK");
        System.out.println("***Add hello7_object cpp OK");
        logWriter.releaseFileLog();
        writeCHECK(5);

        // Add a cpp servant
        logWriter.writeLog("Start test_server_object_cpp_creator");
        try
        {
            processCreateCppServant3 = cdmw.ossupport.OS
                    .createProcess("test_server_object_cpp_creator --FilePath=hello8 --HostName=host8 -ORBInitRef "
                            + "LBGroupManager=corbaloc::localhost:5040/group_mgr");
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }

        loc[0].id = "HOST8";
        hello_object_tmp = cdmw.lb.test.TestUtils.getHelloRefFromFile(orb,
                "hello8");
        System.out.println("***Add hello8_object cpp");
        logWriter.writeAsincrono("***Add hello8_object cpp");
        try
        {
            defaultPolicyGroup = group_manager.add_member(defaultPolicyGroup,
                    loc, hello_object_tmp);
        }
        catch (MemberAlreadyPresent e2)
        {
            e2.printStackTrace();
        }
        catch (ObjectGroupNotFound e2)
        {
            e2.printStackTrace();
        }
        catch (ObjectNotAdded e2)
        {
            e2.printStackTrace();
        }
        catch (org.omg.CORBA.TRANSIENT  t)
        {

            System.out.println("Error LBGroupManager not reachable!");
            System.exit(-1);
        }
        logWriter.writeAsincrono("***Add hello8_object cpp OK");
        System.out.println("***Add hello8_object cpp OK");
        logWriter.releaseFileLog();
        writeCHECK(6);

        // Add a cpp servant
        logWriter.writeLog("Start test_server_object_cpp_creator");
        try
        {
            processCreateCppServant4 = cdmw.ossupport.OS
                    .createProcess("test_server_object_cpp_creator --FilePath=hello9 --HostName=host9 -ORBInitRef "
                            + "LBGroupManager=corbaloc::localhost:5040/group_mgr");
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }

        loc[0].id = "HOST9";
        hello_object_tmp = cdmw.lb.test.TestUtils.getHelloRefFromFile(orb,
                "hello9");
        System.out.println("***Add hello9_object cpp");
        logWriter.writeAsincrono("***Add hello9_object cpp");
        try
        {
            defaultPolicyGroup = group_manager.add_member(defaultPolicyGroup,
                    loc, hello_object_tmp);
        }
        catch (MemberAlreadyPresent e2)
        {
            e2.printStackTrace();
        }
        catch (ObjectGroupNotFound e2)
        {
            e2.printStackTrace();
        }
        catch (ObjectNotAdded e2)
        {
            e2.printStackTrace();
        }
        catch (org.omg.CORBA.TRANSIENT  t)
        {

            System.out.println("Error LBGroupManager not reachable!");
            System.exit(-1);
        }
        logWriter.writeAsincrono("***Add hello9_object cpp OK");
        System.out.println("***Add hello9_object cpp OK");
        logWriter.releaseFileLog();
        writeCHECK(7);

        processClient.killProcess();
        processCreateCppServant1.killProcess();
        processCreateCppServant2.killProcess();
        processCreateCppServant3.killProcess();
        processCreateCppServant4.killProcess();
        
        if(checkError()){
            return false;
        }

        return true;
    }

    /**
     * Start the cpp client
     * 
     * @return
     */
    private boolean startCppClient()
    {
        System.out.println("  START THE CPP CLIENT ");
        try
        {
            processClient = cdmw.ossupport.OS.createProcess("cdmw_test_client");
            cdmw.ossupport.OS.sleep(2000);
        }
        catch (IOException e)
        {
            e.printStackTrace();
            return false;
        }
        catch (InterruptedException e)
        {
            e.printStackTrace();
        }
        return true;
    }

    public org.omg.CORBA.Object getGroup()
    {
        NameComponent[] loc = new NameComponent[1];
        loc[0] = new NameComponent();
        loc[0].id = "hello_group_round_robin";
        loc[0].kind = "";
        Object objTmp = null;
        try
        {
            objTmp = group_manager.get_object_group_ref_from_name(loc);
        }
        catch (ObjectGroupNotFound e)
        {
            e.printStackTrace();
        }
        return objTmp;
    }

    private boolean writeCHECK(int invocation)
    {
        PrintWriter out;
        String file = "check.txt";
        try
        {
            out = new PrintWriter(new FileWriter(file, false));
            out.println(invocation);
            out.flush();
            out.close();
        }
        catch (IOException e)
        {
            e.printStackTrace();
            return false;
        }
        sleep(sleepTime);
        return true;
    }

    private void sleep(int time)
    {
        try
        {
            cdmw.ossupport.OS.sleep(time);
        }
        catch (InterruptedException e)
        {
            e.printStackTrace();
        }
    }
    
    private boolean checkError(){
        File error = new File(fileError);
        if(error.exists()){
            return true;
        }
        else{
            return false;
        }
    }
}
