/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.lb.test;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

public class WriteSincronizedLog
{

    private String fileLog = "";
    private String sincro  = "";

    public WriteSincronizedLog()
    {
        fileLog = "fileLog.txt";
        sincro = "sincro.txt";
        cdmw.ossupport.OS.unlink(sincro);
    }

    public WriteSincronizedLog(String sincroFile, String logFile)
    {
        fileLog = logFile;
        sincro = sincroFile;
        cdmw.ossupport.OS.unlink(sincro);
    }

    public void writeLog(String value)
    {
        writeSincro();
        writeRowInLog(value);
    }

    // write a row in the fileLog
    private void writeRowInLog(String value)
    {
        PrintWriter out;
        try
        {
            out = new PrintWriter(new FileWriter(fileLog, true));
            out.println(value);
            out.println("___________________________________________________");
            out.flush();
            out.close();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
    }

    // stop if find the file sincro
    private void checkSincro()
    {
        File sin = new File(sincro);
        while (sin.exists())
        {
            System.out.println(" Waiting for sincronization..\n");
            try
            {
                cdmw.ossupport.OS.sleep(1000);
            }
            catch (InterruptedException e)
            {
                e.printStackTrace();
            }
        }
    }

    // Write the sincro file
    private void writeSincro()
    {
        File sin = new File(sincro);
        boolean created = false;
        try
        {
            created = sin.createNewFile();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        if (!created)
        {
            checkSincro();
            writeSincro();
        }
    }

    // Delete the file sincro
    private void deleteSincro()
    {
        cdmw.ossupport.OS.unlink(sincro);
    }

    public void releaseFileLog()
    {
        deleteSincro();
    }

    public void waitFileLog()
    {
        checkSincro();
    }

    public void writeOne(String stamp)
    {
        waitFileLog();
        writeLog(stamp);
        releaseFileLog();
    }

    // Write in asicron mode
    public void writeAsincrono(String stamp)
    {
        writeRowInLog(stamp);
    }

}
