/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @brief The main for test lbinit.
 *
 * @author Raffaella Panella <rpanella@amjsv.it>
 *
 */

#ifndef LBINIT_PLUGIN_EXPORTS
#include <string>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/ui/text/TestRunner.h>
#include <cppunit/TestResult.h>
#include <cppunit/TestResultCollector.h>
#include <cppunit/BriefTestProgressListener.h>
#include <cppunit/XmlOutputter.h>
#include <cppunit/TextOutputter.h>
#include <iostream>
#include <fstream>
#include "Foundation/testutils/CORBATestManager.hpp"
#include <LoadBalancing/lbinit/LBServiceInit.hpp>
#include <LoadBalancing/idllib/CdmwLBGroupManager.stub.hpp>
#include <testlbrequesttimeout/TestUtils.hpp>

#include <iostream>
using namespace std;

using namespace Cdmw;

namespace
{
    const std::string GROUP_MANAGER_PORT = "5040";
}

int main( int argc, char* argv[] )
{
    try
    {
        // Initialises the ORB
        Cdmw::OrbSupport::StrategyList strategyList;
        strategyList.add_OrbThreaded();
        strategyList.add_PoaThreadPerConnection();


        CORBA::ORB_var orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, strategyList);




        using namespace Cdmw::OsSupport;
        OS::ProcessId groupMngt_id
            = OS::create_process ("cdmw_lb_group_manager","--CdmwLBXMLFile=CdmwLBGroupManager_conf.xml --groupConf=CdmwTestLBInitGroupCreator_conf.xml");

        OsSupport::OS::sleep(5000);

        // Get the LB Group Manager IOR
        CORBA::Object_var obj = orb->string_to_object("corbaloc::localhost:5040/group_mgr");
        CdmwLB::LBGroupManager_var group_manager
            = CdmwLB::LBGroupManager::_narrow(obj.in());

        // Create Server process
        
        OS::ProcessId test_server_id1 = 
        OS::create_process ("test_server_init",
                                  "--FilePath=hello1 --HostName=host1 -ORBInitRef LBGroupManager=corbaloc::localhost:5040/group_mgr");
        OsSupport::OS::sleep(100);
        OS::ProcessId test_server_id2 = 
        OS::create_process ("test_server_init",
                                  "--FilePath=hello2 --HostName=host2 -ORBInitRef LBGroupManager=corbaloc::localhost:5040/group_mgr");

        OsSupport::OS::sleep(100);
        
        OS::ProcessId test_server_id3 = 
        OS::create_process ("test_server_init",
                                  "--FilePath=hello3 --HostName=host3 -ORBInitRef LBGroupManager=corbaloc::localhost:5040/group_mgr");
        
        OsSupport::OS::sleep(10000);

        PortableGroup::Name name_obj;
        name_obj.length(1);
        name_obj[0].id="hello_group_round_robin";


    obj = group_manager->get_object_group_ref_from_name(name_obj);
        

    // Add replicas
        CORBA::Object_var hello_object = Cdmw::LB::TestUtils::Get_hello_ref_from_file(orb.in(), "hello1");
        ::PortableGroup::Location loc;
        loc.length(3);
        loc[0].id="HOST1";
        loc[0].kind="hostname1";
        loc[1].id="APPL1";
        loc[1].kind="applicationname1";
        loc[2].id="PROC1";
        loc[2].kind="processname1";
        
    
        
        obj = group_manager->add_member(obj.in(),
                                        loc,
                                        hello_object.in());

        hello_object = Cdmw::LB::TestUtils::Get_hello_ref_from_file(orb.in(), "hello2");
        loc[0].id = "HOST2";
        obj = group_manager->add_member(obj.in(),
                                        loc,
                                        hello_object.in());
        
        hello_object = Cdmw::LB::TestUtils::Get_hello_ref_from_file(orb.in(), "hello3");
        loc[0].id = "HOST3";
        obj = group_manager->add_member(obj.in(),
                                        loc,
                                        hello_object.in());
    
        // export the object reference to a file
        std::string file_name = "hello_group_default";
        CORBA::String_var ref_string = orb->object_to_string(obj.in());
        std::ofstream os1(file_name.c_str());
        os1 << ref_string.in();
        os1.close();
        
        name_obj[0].id="hello_group_random";
        obj = group_manager->get_object_group_ref_from_name(name_obj);
    

        // Add replicas
        hello_object = Cdmw::LB::TestUtils::Get_hello_ref_from_file(orb.in(), "hello1");
        loc[0].id = "HOST1";
        obj = group_manager->add_member(obj.in(),
                                        loc,
                                        hello_object.in());
        
        hello_object = Cdmw::LB::TestUtils::Get_hello_ref_from_file(orb.in(), "hello2");
        loc[0].id = "HOST2";
        obj = group_manager->add_member(obj.in(),
                                        loc,
                                        hello_object.in());
        
           hello_object = Cdmw::LB::TestUtils::Get_hello_ref_from_file(orb.in(), "hello3");
        loc[0].id = "HOST3";
        obj = group_manager->add_member(obj.in(),
                                        loc,
                                        hello_object.in());
        
    // export the object reference to a file
        file_name = "hello_group_random";
        ref_string = orb->object_to_string(obj.in());
        std::ofstream os2(file_name.c_str());
        os2 << ref_string.in();
        os2.close();
  
        //
        // Get the root POA
        //
        CORBA::Object_var obj_rootpoa = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var rootPOA =
            PortableServer::POA::_narrow(obj_rootpoa.in());
        //
        // Run implementation (Activate the POA manager)
        //
        PortableServer::POAManager_var poaManager = rootPOA->the_POAManager();
        poaManager->activate();

        Cdmw::TestUtils::CORBATestManager::instance()->init(orb.in(),rootPOA.in(),argc,argv);
    
        Cdmw::TestUtils::CORBATestManager::instance()->run_tests();
        OsSupport::OS::sleep(10000);
        Cdmw::TestUtils::CORBATestManager::instance()->summary();
        
               
#ifdef WIN32
        system("PAUSE");

#endif
        OsSupport::OS::sleep(10000);

        std::cerr <<"******** unlink and kill processes******"<<std::endl;
        // clean created file
        OsSupport::OS::unlink("hello1");
        OsSupport::OS::unlink("hello2");
        OsSupport::OS::unlink("hello3");
        OsSupport::OS::unlink("hello_group_default");
        OsSupport::OS::unlink("hello_group_random");
        
        try {
        OS::kill_process(test_server_id1);
        } catch(...) {}
        try {
        OS::kill_process(test_server_id2);
        } catch(...) {}
        try {
        OS::kill_process(test_server_id3);
        } catch(...) {}
        OS::kill_process(groupMngt_id );
        orb->shutdown(1);
        orb->destroy();
        return Cdmw::TestUtils::CORBATestManager::instance()->was_successful();
        
    }
    catch( const CORBA::Exception &e )
    {
        cerr <<__FILE__<<" "<<__LINE__<<" "<< e._name() << endl;
        return -1;
    }
    catch( const Cdmw::Exception &e )
    {
        cerr <<__FILE__<<" "<<__LINE__<<" "<< e.what() << endl;
        return -1;
    }
    catch(...)
    {
        cerr << "Unexpected exception" << endl;
        return -1;
    }
    
}
#else

#include <cppunit/plugin/TestPlugIn.hpp>

// _implements all the plug-in stuffs, WinMain...
CPPUNIT_PLUGIN_IMPLEMENT();

#endif

///////////////////////////////////////////////////////////////////////////////
//  End Of File
///////////////////////////////////////////////////////////////////////////////



