/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @brief implementation for TestLBClientRequestTimeout with cppUnit library.
 *
 *
 */

// Standard Files
#include <string>

// Cdmw Files
#include <Foundation/common/Options.hpp>
#include <Foundation/common/String.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <LoadBalancing/lbinit/LBServiceInit.hpp>
#include <testlbrequesttimeout/TestUtils.hpp>
#include <cppunit/extensions/HelperMacros.h>
#include "LoadBalancing/lbcommon/LBStrategyRegistry.hpp"
#include "LoadBalancing/lbcommon/LBConfiguration.hpp"
#include "lbcommon/config.hpp"
#include <Foundation/logging/Logger.hpp>
#include "Foundation/logging/LogManager.hpp"

#include <iostream>
namespace LOG=Cdmw::Logging;
using namespace std;

using namespace Cdmw;


int main( int argc, char* argv[] )
{
    try
    {
        Cdmw::Logging::LogManager::Init(argc, argv);
        char** ORB_init_argv = 0;
        char** ORB_init_argv_err = 0;
        int ORB_init_argc = argc;
        std::string str_opt( "" );
        std::string str_opt_err( "" );

        for (int i=0; i<argc; i++)
            str_opt += std::string( argv[i] ) + " ";
        str_opt += std::string( "--LBInterceptorSide=Client ");
        for (int i=0; i<argc; i++)
            str_opt_err += std::string( argv[i] ) + " ";
        str_opt_err += std::string( "--LBInterceptorSide");
        
        ORB_init_argc += 1;
        ORB_init_argv = Cdmw::Common::String::to_char_array(
            Cdmw::Common::String::to_strings( str_opt ) );
        ORB_init_argv_err = Cdmw::Common::String::to_char_array(
            Cdmw::Common::String::to_strings( str_opt_err ) );
 
        std::cerr <<"*********************" << std::endl;
        std::cerr << str_opt << std::endl;
        std::cerr <<"*********************" << std::endl;
        Cdmw::LB::LBServiceInit::Init(ORB_init_argc, ORB_init_argv);
        
        // Initialises the ORB
        Cdmw::OrbSupport::StrategyList strategyList;
        strategyList.add_OrbThreaded();
        strategyList.add_PoaThreadPerConnection();

    
        CORBA::ORB_var orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, strategyList);

        std::cout << "policy RELATIVE_RT_TIMEOUT not setted " <<  std::endl;


        CdmwLBInit::HelloInterface_var hello_group = Cdmw::LB::TestUtils::Get_hello_ref_from_file(orb.in(), "hello_group_default");

        
        cerr <<  "  =================================================================================   " << endl;
        cerr <<  "  TEST : Execute 1 request  on Hello Object Group                        " << endl;
        cerr <<  "  Each request returns an exception: the request is re-forwarded              " << endl;
        cerr <<  "  until the timeout DEFAULT TIMEOUT for LBClientRequest is expired                          " << endl;
        cerr <<  "  =================================================================================   " << endl;
    try { 
                      hello_group->send_exception(1);
    } catch (const CORBA::TIMEOUT &e) {
                   {
                     cerr <<  "  ============================================   " << endl;
                     cerr <<"    Timeout expired : TIMEOUT exception raised     " << endl;
                     cerr <<  "  ============================================   " << endl;
                     CPPUNIT_ASSERT(true); 
                   } 
    } catch (...)
    {
       cerr <<"Unexpected exception" << endl;
       CPPUNIT_ASSERT(false); 
    }
    }
    catch( const CORBA::Exception &e )
    {
        cerr <<__FILE__<<" "<<__LINE__<<" "<< e._name() << endl;
        return -1;
    }
    catch( const Cdmw::Exception &e )
    {
        cerr <<__FILE__<<" "<<__LINE__<<" "<< e.what() << endl;
        return -1;
    }
    catch(...)
    {
        cerr << "Unexpected exception" << endl;
        return -1;
    }
}
    
