/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @brief The main for test lbinit.
 *
 * @author Raffaella Panella <rpanella@amjsv.it>
 *
 */

#ifndef LBINIT_PLUGIN_EXPORTS
#include <string>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/ui/text/TestRunner.h>
#include <cppunit/TestResult.h>
#include <cppunit/TestResultCollector.h>
#include <cppunit/BriefTestProgressListener.h>
#include <cppunit/XmlOutputter.h>
#include <cppunit/TextOutputter.h>
#include <iostream>
#include <fstream>
#include "Foundation/testutils/CORBATestManager.hpp"
#include <LoadBalancing/lbinit/LBServiceInit.hpp>
#include <LoadBalancing/idllib/CdmwLBGroupManager.stub.hpp>
#include <testsincro/TestUtils.hpp>

#include <iostream>
using namespace std;

using namespace Cdmw;

namespace
{
    const std::string GROUP_MANAGER_PORT = "5040";
}

int main( int argc, char* argv[] )
{
    try
    {
        // Initialises the ORB
        Cdmw::OrbSupport::StrategyList strategyList;
        strategyList.add_OrbThreaded();
        strategyList.add_PoaThreadPerConnection();


        CORBA::ORB_var orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, strategyList);

        using namespace Cdmw::OsSupport;
        OS::ProcessId groupMngt_id
            = OS::create_process ("cdmw_lb_group_manager","--CdmwLBXMLFile=CdmwLBGroupManager_conf.xml  --CdmwLogLevel=DEBUG+DEBUG_5+DEBUG_3 --groupConf=CdmwTestLBInitGroupCreator_conf.xml");

        std::cerr << "lb group manager started" << std::endl;
        OsSupport::OS::sleep(5000);
        // Get the LB Group Manager IOR
        CORBA::Object_var obj = orb->string_to_object("corbaloc::localhost:5040/group_mgr");
        CdmwLB::LBGroupManager_var group_manager
            = CdmwLB::LBGroupManager::_narrow(obj.in());


        std::cerr << "IOR -->" << orb->object_to_string(obj)<<std::endl;

      
        std::cerr << "crete server processes" << std::endl;

        OsSupport::OS::sleep(5000);
         
        // Create Server process
        
        OS::ProcessId test_server_id1
            = OS::create_process ("testsincro_server",
                                  "--FilePath=hello1 --HostName=host1 -ORBInitRef LBGroupManager=corbaloc::localhost:5040/group_mgr");
        std::cerr << "created server process 1" << std::endl;
    
        OsSupport::OS::sleep(1000);
        OS::ProcessId test_server_id2
            = OS::create_process ("testsincro_server",
                                  "--FilePath=hello2 --HostName=host2 -ORBInitRef LBGroupManager=corbaloc::localhost:5040/group_mgr");

        std::cerr << "created server process 2" << std::endl; 
        OsSupport::OS::sleep(1000);
        
        OS::ProcessId test_server_id3
            = OS::create_process ("testsincro_server",
                                  "--FilePath=hello3 --HostName=host3 -ORBInitRef LBGroupManager=corbaloc::localhost:5040/group_mgr");
                  
        std::cerr << "created server process 3" << std::endl;
        OsSupport::OS::sleep(1000);
        OS::ProcessId test_server_id4
            = OS::create_process ("testsincro_server",
                                  "--FilePath=hello4 --HostName=host4 -ORBInitRef LBGroupManager=corbaloc::localhost:5040/group_mgr");


OsSupport::OS::sleep(5000);
        
        

        
        //
        // Get the root POA
        //
        CORBA::Object_var obj_rootpoa = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var rootPOA =
            PortableServer::POA::_narrow(obj_rootpoa.in());
        //
        // Run implementation (Activate the POA manager)
        //
        PortableServer::POAManager_var poaManager = rootPOA->the_POAManager();
        poaManager->activate();

        

        Cdmw::TestUtils::CORBATestManager::instance()->init(orb.in(),rootPOA.in(),argc,argv);
    
        Cdmw::TestUtils::CORBATestManager::instance()->run_tests();
        Cdmw::TestUtils::CORBATestManager::instance()->summary();
        
               
#ifdef WIN32
        system("PAUSE");

#endif
        // clean created file
        OsSupport::OS::unlink("hello1");
        OsSupport::OS::unlink("hello2");
        OsSupport::OS::unlink("hello3");
        OsSupport::OS::unlink("hello4");
       
        
        OS::kill_process(test_server_id2);
        OS::kill_process(test_server_id3);
        OS::kill_process(test_server_id4);
        OS::kill_process(test_server_id1);
        
        OS::kill_process(groupMngt_id );
    orb->shutdown(1);
    orb->destroy();

        return Cdmw::TestUtils::CORBATestManager::instance()->was_successful();
        
    }
    catch( const CORBA::Exception &e )
    {
        cerr <<__FILE__<<" "<<__LINE__<<" "<< e._name() << endl;
        CPPUNIT_ASSERT(false);
    return -1;
    }
    catch( const Cdmw::Exception &e )
    {
        cerr <<__FILE__<<" "<<__LINE__<<" "<< e.what() << endl;
    CPPUNIT_ASSERT(false);
        return -1;
    }
    catch(...)
    {
        cerr << "Unexpected exception" << endl;
    CPPUNIT_ASSERT(false);
        return -1;
    }
    
}
#else

#include <cppunit/plugin/TestPlugIn.hpp>

// _implements all the plug-in stuffs, WinMain...
CPPUNIT_PLUGIN_IMPLEMENT();

#endif

///////////////////////////////////////////////////////////////////////////////
//  End Of File
///////////////////////////////////////////////////////////////////////////////



