/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @brief implementation for TestLBInit with cppUnit library.
 *
 * @author Raffaella Panella <rpanella@amjsv.it>
 *
 */

// Standard Files
#include <string>

// Cdmw Files
#include <Foundation/common/Options.hpp>
#include <Foundation/common/String.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <LoadBalancing/lbinit/LBServiceInit.hpp>
#include <testsincro/TestUtils.hpp>
#include <cppunit/extensions/HelperMacros.h>
#include "LoadBalancing/lbcommon/LBStrategyRegistry.hpp"
#include "LoadBalancing/lbcommon/LBConfiguration.hpp"
#include "lbcommon/config.hpp"
#include <Foundation/logging/Logger.hpp>
#include "Foundation/logging/LogManager.hpp"

#include <iostream>
namespace LOG=Cdmw::Logging;
using namespace std;

using namespace Cdmw;


#define ECHO_HEADER() \
    "[Debug] "

#define ECHO_ERROR(comment) \
do {\
    std::cerr << ECHO_HEADER() << " --> " << comment << std::endl;\
} while(0)


#ifndef ENABLE_LB_DEBUG_DUMP

#   define DEBUG_DUMP(comment)
#   define DEBUG_ECHO(comment)

#else

#   define DEBUG_DUMP(comment) \
do {\
    std::cerr << ECHO_HEADER() << " --> " << comment << std::endl;\
} while(0)

#   define DEBUG_ECHO(comment) \
do {\
    std::cerr << comment ;\
} while(0)

#endif

int main( int argc, char* argv[] )
{

    try
    {
        Cdmw::Logging::LogManager::Init(argc, argv);
    CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),
    "\n\n==============================="
    <<"\n=     LB CLIENT               ="
    <<"\n==============================="); 
        
        char** ORB_init_argv = 0;
    char** ORB_init_argv_err = 0;
        int ORB_init_argc = argc;
        std::string str_opt( "" );
    std::string str_opt_err( "" );

        for (int i=0; i<argc; i++)
            str_opt += std::string( argv[i] ) + " ";
        str_opt += std::string( "--LBInterceptorSide=Client");
        for (int i=0; i<argc; i++)
            str_opt_err += std::string( argv[i] ) + " ";
        str_opt_err += std::string( "--LBInterceptorSide");
        
    ORB_init_argc += 1;
        ORB_init_argv = Cdmw::Common::String::to_char_array(
            Cdmw::Common::String::to_strings( str_opt ) );
    ORB_init_argv_err = Cdmw::Common::String::to_char_array(
            Cdmw::Common::String::to_strings( str_opt_err ) );
 
        
            
        CPPUNIT_ASSERT_NO_THROW(Cdmw::LB::LBServiceInit::Init(ORB_init_argc, ORB_init_argv));
        
        // Initialises the ORB
        Cdmw::OrbSupport::StrategyList strategyList;
        strategyList.add_OrbThreaded();
        strategyList.add_PoaThreadPerConnection();

    
        CORBA::ORB_var orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, strategyList);

        CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"try to obtain LB Group manager reference (from corbaloc) "); 

        CORBA::Object_var objref = orb->string_to_object("corbaloc::localhost:5040/group_mgr");
        CdmwLB::LBGroupManager_var group_manager = CdmwLB::LBGroupManager::_narrow(objref.in());
        if(CORBA::is_nil(objref.in()))
    {
            CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"LB Group manager reference (from corbaloc) not valid");
        CPPUNIT_ASSERT(false);
        return -1;
    }        
    
     
            
        PortableGroup::Name name_obj;
        name_obj.length(1);
        name_obj[0].id="hello_group_round_robin";
        objref = group_manager->get_object_group_ref_from_name(name_obj);
        CdmwLBInit::HelloInterface_var hello_group = CdmwLBInit::HelloInterface::_narrow(objref.in());  
 
        if(CORBA::is_nil(objref.in()))
    {
            CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"LB Group reference (from get_object_group_ref_from_name) not valid");
        return -1;
        } 
        
        CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),
    "\n\n   ===================================================================== "
    <<"\n   = The Client request starts before than Server is added to LBGroup  = "
    <<"\n   = The Client calls the request on empty group a TRANSIENT exception = "
    <<"\n   = is raised.                                                        = "
    <<"\n   ===================================================================== ");
        
        for(int i = 0; i<2; i++)
        {
          CPPUNIT_ASSERT_THROW(hello_group->hello(),CORBA::TRANSIENT);
          CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),
      "\n\n   ===================================================================== "
      <<"\n   = CORBA::TRANSIENT exception received for request "<<i+1<<"          = "
      <<"\n   ===================================================================== ");
        

        }


        CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),
    "\n\n   ===================================================================== "
    <<"\n   = The LB Servers are added to LB Group                              = "
    <<"\n   ===================================================================== ");



        CORBA::Object_var obj = group_manager->get_object_group_ref_from_name(name_obj);    
        // Add replicas
        CORBA::Object_var hello_object = Cdmw::LB::TestUtils::Get_hello_ref_from_file(orb.in(), "hello1");
        ::PortableGroup::Location loc;
        loc.length(3);
        loc[0].id="HOST1";
        loc[0].kind="hostname1";
        loc[1].id="APPL1";
        loc[1].kind="applicationname1";
        loc[2].id="PROC1";
        loc[2].kind="processname1";
        
    
        
        obj = group_manager->add_member(obj.in(),
                                        loc,
                                        hello_object.in());

        hello_object = Cdmw::LB::TestUtils::Get_hello_ref_from_file(orb.in(), "hello2");
        loc[0].id = "HOST2";
        obj = group_manager->add_member(obj.in(),
                                        loc,
                                        hello_object.in());
        
        hello_object = Cdmw::LB::TestUtils::Get_hello_ref_from_file(orb.in(), "hello3");
        loc[0].id = "HOST3";
        obj = group_manager->add_member(obj.in(),
                                        loc,
                                        hello_object.in());
    
         hello_object = Cdmw::LB::TestUtils::Get_hello_ref_from_file(orb.in(), "hello4");
        loc[0].id = "HOST4";
        obj = group_manager->add_member(obj.in(),
                                        loc,
                                        hello_object.in());
        
        CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),
    "\n\n   ===================================================================== "
    <<"\n   = the Client performs the requests again.                           = "
    <<"\n   = Any exception must be raised                                      = "
    <<"\n   ===================================================================== ");    
        for(int i = 0; i<16; i++)
        {
            
           CPPUNIT_ASSERT_NO_THROW(hello_group->hello());
            
        }

    
    }
    catch( const CORBA::Exception &e )
    {
        cerr <<__FILE__<<" "<<__LINE__<<" "<< e._name() << endl;
        return -1;
    }
    catch( const Cdmw::Exception &e )
    {
        cerr <<__FILE__<<" "<<__LINE__<<" "<< e.what() << endl;
        return -1;
    }
    catch(...)
    {
        cerr << "Unexpected exception" << endl;
        return -1;
    }
}
    
