/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

// Standard Files
#include <fstream>
#include <iostream>

// Cdmw Files
#include <Foundation/common/Options.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <LoadBalancing/lbinit/LBServiceInit.hpp>
#include <Foundation/common/String.hpp>
#include <idllib/CdmwLBGroupManager.stub.hpp>
#include <Repository/idllib/CdmwNamingAndRepository.stub.hpp>


// Local Files
#include "lbinitservant/TestHello_impl.hpp"

using namespace std;

namespace
{

const string REPLICA_PATH = "--ReplicaName";
const string GM_IOR_OPTION = "--GroupMngRef";
const string NAMING_REPOS_URL_OPTION = "--namingRepos-url";
const string IOR_FILE_NAME = "server_Q-340.IOR";
} // end anonymous namespace

static void
usage( ostream & os, const string &program_name )
{
    os << "Usage:\n"
    << program_name << " options\n"
    << "\n"
    << "Options:\n"
    << "--help : Display this information.\n"
    << "\n"
    << "-ORBInitRef LBGroupManager = <rep_corbaloc>  ReplicationManager corbaloc\n"
    << "[" << REPLICA_PATH << "] = path              path of the file where interface IOR should be stored.\n"
    <<  endl;
}

int main( int argc, char* argv[] )
{
    // help argument

    if ( argc <= 2 ) {
        usage ( cerr, argv[ 0 ] );
        return 0;
    }

    if ( argc >= 2 ) {
        if ( strcmp( argv[ 1 ], "--help" ) == 0 ) {
            usage ( cerr, argv[ 0 ] );
            return 0;
        }
    }

    // get replica name 
    string file_name = Cdmw::OsSupport::OS::get_option_value ( argc, argv, REPLICA_PATH );
    std::string host_number(file_name.begin()+5 ,file_name.end());
    int number = atoi(host_number.c_str());

    if ( file_name == "no" ) {
        file_name = "interface.ior";
    }

    try {


        std::string str_opt( "" );
        char** ORB_init_argv = 0;
        int ORB_init_argc = argc;
        
        for (int i=0; i<argc; i++)
            str_opt += std::string( argv[i] ) + " ";
        str_opt += std::string( "--LBInterceptorSide=Server");
        ORB_init_argc += 1;
        ORB_init_argv = Cdmw::Common::String::to_char_array(
            Cdmw::Common::String::to_strings( str_opt ) );

        Cdmw::LB::LBServiceInit::Init(ORB_init_argc, ORB_init_argv);
        
        Cdmw::OsSupport::OS::sleep(1000);
        //Cdmw::LB::LBServiceInit::Init(argc, argv);
        Cdmw::OsSupport::OS::sleep(1000);
        
        string namingReposURL =
            Cdmw::OsSupport::OS::get_option_value ( argc, argv, NAMING_REPOS_URL_OPTION );
Cdmw::OsSupport::OS::sleep(1000);   

        if ( namingReposURL == "no" ) {
            cerr << "Error: naming and repository url undefined - " 
                      << " use " 
                      << NAMING_REPOS_URL_OPTION 
                      << "option!" 
                      << endl;
            usage( cerr, argv[ 0 ] );
            return 0;
        }

        // Initialises the ORB
        Cdmw::OrbSupport::StrategyList strategyList;
Cdmw::OsSupport::OS::sleep(1000);
        strategyList.add_OrbThreaded();
Cdmw::OsSupport::OS::sleep(1000);
        strategyList.add_PoaThreadPerConnection();
        strategyList.add_multicast();
Cdmw::OsSupport::OS::sleep(1000);


for (int i=0; i<argc; i++){
    cerr << " -Param : " << argv[i] << endl;
}

        CORBA::ORB_var orb
        = Cdmw::OrbSupport::OrbSupport::ORB_init( argc, argv, strategyList );
Cdmw::OsSupport::OS::sleep(1000);



        // Get the root POA
        CORBA::Object_var obj = orb->resolve_initial_references( "RootPOA" );

        PortableServer::POA_var rootPOA = PortableServer::POA::_narrow( obj.in() );

        PortableServer::POAManager_var poaMgr = rootPOA->the_POAManager();

        poaMgr->activate();



        Cdmw::HelloInterfaceReturn_impl * pingable_interface
        = new Cdmw::HelloInterfaceReturn_impl(number);

        CdmwLBInit::HelloInterfaceReturn_var pingable_interface_ref
        = pingable_interface->_this();

        // export the object reference to a file
        CORBA::String_var ref_string = orb->object_to_string( pingable_interface_ref.in() );

        ofstream os( file_name.c_str() );

        os << ref_string.in();

        os.close();

        // ===================================================
        // Get the naming and repository
        //     from the input URL
        // ===================================================
        CdmwNamingAndRepository::Repository_var repository =
            CdmwNamingAndRepository::Repository::_nil();

        CORBA::Object_var obj2 = orb->string_to_object( namingReposURL.c_str() );

        repository = CdmwNamingAndRepository::Repository::_narrow( obj2.in() );

        CdmwNamingAndRepository::NameDomain_var helloDomain =
            repository->resolve_name_domain ( "VALID/HELLO" );


        // ========================================================
        // Register the HelloInterface object in the HELLO_LB name domain
        // ========================================================
        // reserve the name "HelloInterface" in the domain
        CdmwNamingAndRepository::NameDomain::RegistrationId_var regId =
            helloDomain->new_name ( file_name.c_str() );

        // register the object
        helloDomain->register_object ( regId.in(), pingable_interface_ref.in() );

        cerr << "server_Q-340>> LB Server Started and Registered in the Interface Repository " << endl;

        orb->run();
    }
    catch ( const CORBA::SystemException & e ) {
        cerr << __FILE__ << " " << __LINE__ << " " << e._name() << endl;
        return -1;
    }
    catch ( const CORBA::Exception & e ) {
        cerr << __FILE__ << " " << __LINE__ << " " << e._name() << endl;
        return -1;
    }
    catch ( ... ) {
        cerr << "Uncaught exception" << endl;
        return -1;
    }

    return 0;
}

