/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.lb.test;

import java.io.DataOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.PrintWriter;
import org.omg.CORBA.ExceptionList;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;
import cdmw.lb.lbinit.LBServiceInit;
import cdmw.orbsupport.StrategyList;
import cdmw.orbsupport.ORBSupport;
import cdmw.lb.test.HelloInterfaceReturn_Impl;
import cdmw.ossupport.OS;

public class MainServer
{

    public static String   FILE_PATH_OPTION = "--ReplicaName";
    public static String   GM_IOR_OPTION    = "--GroupMngRef";
    public static String   HOST_NAME_OPTION = "--ReplicaName";
    public static String NAMING_REPOS_URL_OPTION = "--namingRepos-url";
    public static String[] ORB_init_args    = null;

    private static void usage() {
        System.out.println("Usage:\n");
        System.out.println("helloServer.sh options\n");
        System.out.println("Options:\n");
        System.out.println();
        System.out.println("--help                     Display this information.\n");
        System.out.println();
        System.out.println(NAMING_REPOS_URL_OPTION + "=<URL>  URL of the CDMW Naming and Repository service.\n");
        System.out.println();
    }

    // int main( int argc, char argv[] )
    public static void main(String args[])
    {
        {
            
            // get host name
            String host_nameS = cdmw.ossupport.OS.getOptionValue(args,
                    HOST_NAME_OPTION);
                    
                    
            String number = host_nameS.substring(5);
            int host_name = Integer.parseInt(number);
           // int host_name = Integer.parseInt(host_nameS);

            int argc = args.length;

            // get Name File to save hello interface
            String file_name = cdmw.ossupport.OS.getOptionValue(args,
                    FILE_PATH_OPTION);

            if (file_name == "no")
            {
                file_name = "hello.ior";
            }

            try
            {
            
                //
                // get Naming & Repository URL from arguments
                //
                String[] ORB_init_args = new String[(args.length) + 2];
                String GMURL = cdmw.ossupport.OS.getOptionValue (args, GM_IOR_OPTION);
                String namingReposURL = cdmw.ossupport.OS.getOptionValue (args, NAMING_REPOS_URL_OPTION);
                if(!(GMURL.equalsIgnoreCase("no") || GMURL.equalsIgnoreCase("yes"))){
                    System.out.println("*********************>   "+GMURL);
                    for (int i = 0; i < args.length; i++)
                    {
                        ORB_init_args[i] = args[i];
                    }
                    ORB_init_args[args.length] = "--LBInterceptorSide=Server";
                    ORB_init_args[args.length+1] = "--LBGroupManager="+GMURL;             
                }
                else{
                    System.out.println("*********************>   "+namingReposURL);
                    for (int i = 0; i < args.length; i++)
                    {
                        ORB_init_args[i] = args[i];
                    }
                    ORB_init_args[args.length] = "--LBInterceptorSide=Server";
                    ORB_init_args[args.length+1]= "--namingRepos-url="+namingReposURL;    
                }

                cdmw.lb.lbinit.LBServiceInit.init(ORB_init_args);

                // Initialises the ORB
                cdmw.orbsupport.StrategyList strategyList = new cdmw.orbsupport.StrategyList();

                strategyList.addORBThreaded();
                try
                {
                    strategyList.addPOAThreadPerConnection();
                }
                catch (cdmw.common.BadParameterException bpe)
                {
                    System.out.println(bpe);
                }

                org.omg.CORBA.ORB orb = cdmw.orbsupport.ORBSupport.ORBInit(
                        args, strategyList);

                // Get the root POA
                try
                {
                    org.omg.CORBA.Object obj = orb
                            .resolve_initial_references("RootPOA");
                    org.omg.PortableServer.POA rootPOA = org.omg.PortableServer.POAHelper
                            .narrow(obj);
                    org.omg.PortableServer.POAManager poaMgr = rootPOA
                            .the_POAManager();
                    poaMgr.activate();
                }
                catch (org.omg.CORBA.ORBPackage.InvalidName in)
                {
                    System.out.println(in);
                }
                catch (org.omg.PortableServer.POAManagerPackage.AdapterInactive ai)
                {
                    System.out.println(ai);
                }
                
                
                
                
                if (namingReposURL.equals("no")) 
                {
                    System.out.println("naming and repository url undefined");
                    usage();
                    return;
                }
                
                
                 cdmw.lb.test.HelloInterfaceReturn_Impl hello_interface = new cdmw.lb.test.HelloInterfaceReturn_Impl(
                        host_name);
                    CdmwLBInit.HelloInterfaceReturn hello_interface_ref = hello_interface
                        ._this(orb);
                
                
               try{ 
                    // ===================================================
                    // Get the naming and repository
                    //     from the input URL
                    // ===================================================
                    
                    System.out.println("Get the naming and repository from the input URL: "+namingReposURL);
                    com.thalesgroup.CdmwNamingAndRepository.Repository repository = null;
                                      
                    org.omg.CORBA.Object obj2 = orb.string_to_object(namingReposURL);
                    System.out.println("Narrow");
                    repository = com.thalesgroup.CdmwNamingAndRepository.RepositoryHelper.narrow(obj2);
                
                    // ======================================================
                    // Get HELLO name domain from repository
                    // ======================================================
                    System.out.println("Get HELLO name domain from repository");
                    com.thalesgroup.CdmwNamingAndRepository.NameDomain helloDomain =
                    repository.resolve_name_domain("VALID/HELLO");
 
                    // ========================================================
                    // Register the HelloInterface object in the HELLO name domain
                    // ========================================================
                    // reserve the name "HelloInterface" in the domain
                    System.out.println("Register the HelloInterface object in the HELLO name domain");
                    org.omg.CORBA.Any regId =  helloDomain.new_name (host_nameS);
            
                    // register the object
                    System.out.println("register the object");
                    helloDomain.register_object (regId, hello_interface_ref);
                }
                catch( com.thalesgroup.CdmwNamingAndRepository.NoNameDomain e) {
                     e.printStackTrace(); 
                }  
                catch( com.thalesgroup.CdmwNamingAndRepository.NameDomainPackage.AlreadyExists e) {
                     e.printStackTrace(); 
                }
                catch( com.thalesgroup.CdmwNamingAndRepository.InvalidName e) {
                     e.printStackTrace(); 
                }
                catch( com.thalesgroup.CdmwNamingAndRepository.NameDomainPackage.InvalidRegistration e) {
                     e.printStackTrace(); 
                }
               
                    
                
                // export the object reference to a file
                String ref_string = orb.object_to_string(hello_interface_ref);
                try
                {

                    PrintWriter out = new PrintWriter(new FileWriter(file_name));
                    out.println(ref_string);
                    out.flush();
                    out.close();

                }
                catch (java.io.FileNotFoundException fnfe)
                {
                    System.out.println(fnfe);
                }
                catch (java.io.IOException ioe)
                {
                    System.out.println(ioe);
                }

                orb.run();

            }
            catch (org.omg.CORBA.SystemException e)
            {
                Thread.dumpStack();
                System.out.println(e);

            }
        }
    }
}
