/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.lb.test;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import org.omg.CORBA.Object;
import org.omg.CosNaming.NameComponent;
import org.omg.PortableGroup.MemberAlreadyPresent;
import org.omg.PortableGroup.MemberNotFound;
import org.omg.PortableGroup.ObjectGroupNotFound;
import org.omg.PortableGroup.ObjectNotAdded;
import cdmw.ossupport.ThreadedProcess;
import org.omg.CORBA.ORBPackage.InvalidName;
import org.omg.CosNaming.NamingContextPackage.CannotProceed;
import org.omg.CosNaming.NamingContextPackage.NotFound;
import com.thalesgroup.CdmwNamingAndRepository.NoNameDomain;

public class TestLBInitServer
{

    static org.omg.CORBA.Object        objGroup  = null;
    static org.omg.CORBA.ORB           orb       = null;
    CdmwLBInit.HelloInterface          helloRef  = null;
    static CdmwLB.LBGroupManager       group_manager;
    static public String               fileCheck = "check.txt";
    static private String              fileError = "fileError.txt";
    static private String              fileLog   = "fileLog.txt";
    static private String              sincro    = "sincro.txt";
    static private int                 sleepTime = 15000;
    public static ThreadedProcess      processClient;
    public static ThreadedProcess      processCreateCppServant1;
    public static ThreadedProcess      processCreateCppServant2;
    public static ThreadedProcess      processCreateCppServant3;
    public static ThreadedProcess      processCreateCppServant4;
    public static ThreadedProcess      gmProcess;
    private static WriteSincronizedLog logWriter;
    private static int invocation = 0;
    private static String namingAndRepositoryURL=""; 
    private static String NAMING_REPOS_URL_OPTION = "--namingRepos-url";
    private static String GROUPMANAGER_URL_OPTION = "--GroupMngRef";
    private static boolean nsFlag = false;
    private static String cobalocNS = "";

    public TestLBInitServer(org.omg.CORBA.ORB orb)
    {
        
    }

    public static void main(String[] args){
        
        //Initialises the ORB
        try{
            cdmw.orbsupport.StrategyList orb_strategies = new cdmw.orbsupport.StrategyList();
            orb_strategies.addORBThreaded();
            orb_strategies.addPOAThreadPerConnection();
            orb = cdmw.orbsupport.ORBSupport.ORBInit(args, orb_strategies);
        }
        catch(cdmw.common.BadParameterException e){
            e.printStackTrace();
            System.exit(1);
        }
        
        
        cobalocNS = cdmw.ossupport.OS.getOptionValue(args,
                    NAMING_REPOS_URL_OPTION);
                 
        String cobalocGM = cdmw.ossupport.OS.getOptionValue(args,
                    GROUPMANAGER_URL_OPTION);         
                 
        if( !(cobalocGM.equalsIgnoreCase("no") || cobalocGM.equalsIgnoreCase("yes")) ){
            System.out.println("---------> " +cobalocGM );
            objGroup = orb.string_to_object(cobalocGM);
            group_manager = CdmwLB.LBGroupManagerHelper.narrow(objGroup);
        }
        else{ 
    
           nsFlag = true;
           // ===================================================
           // Get the naming and repository
           //     from orb
           // ===================================================
           com.thalesgroup.CdmwNamingAndRepository.Repository repository = null;
           org.omg.CORBA.Object objName = null;
           System.out.println("objName = m_orb.string_to_object(NamRapos_str);");
                  
           //String cobalocNS = "corbaloc::cardamom07:42562/CdmwRepository";
                   
           objName = orb.string_to_object(cobalocNS);                          
           if (objName == null)
           {
              System.out.println("Error: NamingRepository - null reference");
              throw new org.omg.CORBA.INTERNAL(cdmw.orbsupport.ExceptionMinorCodes.INTERNALLoadBalancingError
                           , org.omg.CORBA.CompletionStatus.COMPLETED_NO);
           }
           System.out.println("repository = com.thalesgroup.CdmwNamingAndRepository.RepositoryHelper.narrow(objName);  ");
           repository = com.thalesgroup.CdmwNamingAndRepository.RepositoryHelper.narrow(objName);            

           // ======================================================   
           // Initialise the repository interface
           //     "CDMW" has been chosen as the default domain name
           // ======================================================
           try {
                System.out.println("cdmw.namingandrepository.RepositoryInterface.init ('CDMW', repository);");
                cdmw.namingandrepository.RepositoryInterface.init ("CDMW", repository);
           } catch (NoNameDomain e1) {
                // TODO Auto-generated catch block
                e1.printStackTrace();
           }catch (com.thalesgroup.CdmwNamingAndRepository.InvalidName e1) {
                // TODO Auto-generated catch block
                e1.printStackTrace();
           }
                       
           // ======================================================
           // Get Naming Interface of LBGROUPMANAGER name domain
           // ======================================================
           cdmw.commonsvcs.naming.NamingInterface ni = null;
           try {
           System.out.println("ni = cdmw.namingandrepository.RepositoryInterface.getDomainNamingInterface('lb_group_manager/object_groups');");
                        ni = cdmw.namingandrepository.RepositoryInterface.getDomainNamingInterface("lb_group_manager/object_groups");
           } catch (NoNameDomain e1) {                                                     
               // TODO Auto-generated catch block
               e1.printStackTrace();
           } catch (com.thalesgroup.CdmwNamingAndRepository.InvalidName e1) {
               // TODO Auto-generated catch block
               e1.printStackTrace();
           }
                        
           // ===================================================
           // Get the GroupManager object from repository
           // ===================================================
           org.omg.CORBA.Object objref = null;
           try {
                System.out.println("objref = ni.resolve ('LBGroupManagerGroup');");
                objref = ni.resolve ("LBGroupManagerGroup");
           } catch (NotFound e1) {
                // TODO Auto-generated catch block
                e1.printStackTrace();
           } catch (CannotProceed e1) {
                // TODO Auto-generated catch block
                e1.printStackTrace();
           } catch (org.omg.CosNaming.NamingContextPackage.InvalidName e1) {
                // TODO Auto-generated catch block
                e1.printStackTrace();
           }
           String ss = orb.object_to_string(objref);
           System.out.println("group_manager = CdmwLB.LBGroupManagerHelper.narrow(objref);  ------> "+ ss);
           group_manager = CdmwLB.LBGroupManagerHelper.narrow(objref);
       }
       
                  
                    //***************************************************
    
        logWriter = new WriteSincronizedLog(sincro, fileLog);
        namingAndRepositoryURL = cdmw.ossupport.OS.getOptionValue (args, NAMING_REPOS_URL_OPTION);
        addElemGroupCheckIOGR();
    }

    static public boolean addElemGroupCheckIOGR()
    {
        cdmw.ossupport.OS.unlink(fileCheck);
        cdmw.ossupport.OS.unlink(fileError);
        cdmw.ossupport.OS.unlink(sincro);
        cdmw.ossupport.OS.unlink(fileLog);

        logWriter.writeOne("Start the cpp client");
        // Start the cpp client
        if (!startCppClient())
        {
            return false;
        }
        // wait the execution of client cpp
        sleep(5000);
        org.omg.CORBA.Object defaultPolicyGroup = getGroup();

        NameComponent[] loc = new NameComponent[3];
        loc[0] = new NameComponent();
        loc[1] = new NameComponent();
        loc[2] = new NameComponent();

        loc[0].id = "HOST1";
        loc[0].kind = "hostname1";
        loc[1].id = "APPL1";
        loc[1].kind = "applicationname1";
        loc[2].id = "PROC1";
        loc[2].kind = "processname1";
        
        
        String helloObj = "hello2";
        
        org.omg.CORBA.Object hello_object_tmp = cdmw.lb.test.TestUtils
                .getHelloRefFromNamingRepository(orb, helloObj, namingAndRepositoryURL);

        callHelloElement(hello_object_tmp); 
         
        logWriter.waitFileLog();
        System.out.println("***Add "+ helloObj+"_object");
        logWriter.writeLog("***Add "+ helloObj+"_object");
        try
        {
            defaultPolicyGroup = group_manager.add_member(defaultPolicyGroup,
                    loc, hello_object_tmp);
        }
        catch (MemberAlreadyPresent e2)
        {
            e2.printStackTrace();
        }
        catch (ObjectGroupNotFound e2)
        {
            e2.printStackTrace();
        }
        catch (ObjectNotAdded e2)
        {
            e2.printStackTrace();
        }
        System.out.println("***Add "+ helloObj+"_object OK");
        logWriter.writeAsincrono("***Add "+ helloObj+"_object OK");
        logWriter.releaseFileLog();
        System.out.println("***"
                + orb.object_to_string(defaultPolicyGroup).length());
                
        InfogetMemberRef(loc,hello_object_tmp);
                
        sleep(1000);

        invocation = 1;
        writeCHECK(invocation);

        helloObj = "hello6";
        
        loc[0].id = "HOST2";
        hello_object_tmp = cdmw.lb.test.TestUtils.getHelloRefFromNamingRepository(orb, helloObj, namingAndRepositoryURL);
        
        callHelloElement(hello_object_tmp); 
        
        System.out.println("***Add "+ helloObj+"_object");
        logWriter.writeLog("***Add "+ helloObj+"_object");
        try
        {
            defaultPolicyGroup = group_manager.add_member(defaultPolicyGroup,
                    loc, hello_object_tmp);
        }
        catch (MemberAlreadyPresent e2)
        {
            e2.printStackTrace();
        }
        catch (ObjectGroupNotFound e2)
        {
            e2.printStackTrace();
        }
        catch (ObjectNotAdded e2)
        {
            e2.printStackTrace();
        }
        System.out.println("***Add "+ helloObj+"_object OK");
        logWriter.writeAsincrono("***Add "+ helloObj+"_object  OK");
        logWriter.releaseFileLog();
        System.out.println("***"
                + orb.object_to_string(defaultPolicyGroup).length());
        InfogetMemberRef(loc,hello_object_tmp);

        sleep(1000);
        invocation = 2;
        writeCHECK(invocation);

        helloObj = "hello3";
        loc[0].id = "HOST3";
        hello_object_tmp = cdmw.lb.test.TestUtils.getHelloRefFromNamingRepository(orb,helloObj, namingAndRepositoryURL);
        
        callHelloElement(hello_object_tmp); 
        
        System.out.println("***Add "+ helloObj+"_object");
        logWriter.writeLog("***Add "+ helloObj+"_object");
        try
        {
            defaultPolicyGroup = group_manager.add_member(defaultPolicyGroup,
                    loc, hello_object_tmp);
        }
        catch (MemberAlreadyPresent e2)
        {
            e2.printStackTrace();
        }
        catch (ObjectGroupNotFound e2)
        {
            e2.printStackTrace();
        }
        catch (ObjectNotAdded e2)
        {
            e2.printStackTrace();
        }
        System.out.println("***Add "+ helloObj+"_object OK");
        logWriter.writeAsincrono("***Add "+ helloObj+"_object OK");
        logWriter.releaseFileLog();
        System.out.println("***"
                + orb.object_to_string(defaultPolicyGroup).length());
        InfogetMemberRef(loc,hello_object_tmp);
    
        sleep(1000);
        invocation = 3;
        writeCHECK(invocation);
        
        
        //recupero la due e la elimino
        helloObj = "hello2";
        loc[0].id = "HOST1";
        loc[0].kind = "hostname1";
        loc[1].id = "APPL1";
        loc[1].kind = "applicationname1";
        loc[2].id = "PROC1";
        loc[2].kind = "processname1";
        //................
        
        
        logWriter.writeLog("***Delete "+ helloObj+"_object");
        System.out.println("***"
                + orb.object_to_string(defaultPolicyGroup).length());
              
        try
        {
            defaultPolicyGroup = group_manager.remove_member(
                    defaultPolicyGroup, loc);
        }
        catch (ObjectGroupNotFound onf)
        {
            onf.printStackTrace();
            System.out.println("***Errore rimozione"); 
            return false;
        }
        catch (MemberNotFound e)
        {
            e.printStackTrace();
            System.out.println("***Errore rimozione"); 
            return false;
        }
        catch(Exception ee){
            System.out.println("***Errore rimozione"); 
        }        
        System.out.println("***Rimozione OK");  
        logWriter.writeAsincrono("***Delete "+ helloObj+"_object OK");
        logWriter.releaseFileLog();

        sleep(1000);
        invocation = 2;
        writeCHECK(invocation);

        helloObj = "hello7";
        loc[0].id = "HOST1";
        loc[2].id = "PROC2";
        loc[2].kind = "processname2";
        hello_object_tmp = cdmw.lb.test.TestUtils.getHelloRefFromNamingRepository(orb, helloObj , namingAndRepositoryURL);
        
        callHelloElement(hello_object_tmp); 
        
        System.out.println("***Add "+ helloObj+"_object ");
        logWriter.writeAsincrono("***Add "+ helloObj+"_object ");
        try
        {
            defaultPolicyGroup = group_manager.add_member(defaultPolicyGroup,
                    loc, hello_object_tmp);
        }
        catch (MemberAlreadyPresent e2)
        {
            e2.printStackTrace();
        }
        catch (ObjectGroupNotFound e2)
        {
            e2.printStackTrace();
        }
        catch (ObjectNotAdded e2)
        {
            e2.printStackTrace();
        }

        logWriter.writeAsincrono("***Add "+ helloObj+"_object OK");
        System.out.println("***Add "+ helloObj+"_object OK");
        logWriter.releaseFileLog();
        InfogetMemberRef(loc,hello_object_tmp);
        
        invocation = 3;
        writeCHECK(invocation);
        
        helloObj = "hello4";
        loc[0].id = "HOST2";
        hello_object_tmp = cdmw.lb.test.TestUtils.getHelloRefFromNamingRepository(orb, helloObj, namingAndRepositoryURL);
        
        callHelloElement(hello_object_tmp); 
        
        System.out.println("***Add "+ helloObj+"_object");
        logWriter.writeAsincrono("***Add "+ helloObj+"_object");
        try
        {
            defaultPolicyGroup = group_manager.add_member(defaultPolicyGroup,
                    loc, hello_object_tmp);
        }
        catch (MemberAlreadyPresent e2)
        {
            e2.printStackTrace();
        }
        catch (ObjectGroupNotFound e2)
        {
            e2.printStackTrace();
        }
        catch (ObjectNotAdded e2)
        {
            e2.printStackTrace();
        }

        logWriter.writeAsincrono("***Add "+ helloObj+"_object OK");
        System.out.println("***Add "+ helloObj+"_object OK");
        logWriter.releaseFileLog();
        InfogetMemberRef(loc,hello_object_tmp);
        invocation = 4;
        writeCHECK(invocation);

        helloObj = "hello8";
        loc[0].id = "HOST3";
        loc[2].id = "PROC3";
        loc[2].kind = "processname3";
        hello_object_tmp = cdmw.lb.test.TestUtils.getHelloRefFromNamingRepository(orb, helloObj, namingAndRepositoryURL);
        
        callHelloElement(hello_object_tmp); 
        
        System.out.println("***Add "+ helloObj+"_object");
        logWriter.writeAsincrono("***Add "+ helloObj+"_object");
        try
        {
            defaultPolicyGroup = group_manager.add_member(defaultPolicyGroup,
                    loc, hello_object_tmp);
        }
        catch (MemberAlreadyPresent e2)
        {
            e2.printStackTrace();
        }
        catch (ObjectGroupNotFound e2)
        {
            e2.printStackTrace();
        }
        catch (ObjectNotAdded e2)
        {
            e2.printStackTrace();
        }

        logWriter.writeAsincrono("***Add "+ helloObj+"_object OK");
        System.out.println("***Add "+ helloObj+"_object OK");
        logWriter.releaseFileLog();
        InfogetMemberRef(loc,hello_object_tmp);
        invocation = 5;
        writeCHECK(invocation);

        processClient.killProcess();
        
        if(checkError()){
            System.err.println("WARNING: It seems that there is an error, check the fileLog.txt for more information ");
            return false;
        }
         
        System.err.println("TEST END!!!");
         
        return true;
    }



    private static void callHelloElement(org.omg.CORBA.Object hello_object_tmp){
    /*    System.out.println("DEBUG CALL********");
        CdmwLBInit.HelloInterfaceReturn hello_element = CdmwLBInit.HelloInterfaceReturnHelper.narrow(hello_object_tmp);    
        System.out.println("******************");    
        hello_element.hello(); */
    }
    /**
    * get_member_ref
    * 
    * 
    */
    private static void InfogetMemberRef(NameComponent[] loc, org.omg.CORBA.Object objElement){
        // get_member_ref on the group for the member just added
      /*  System.out
                .println(" ------ Get member ref on the group for the member just added ------");

        org.omg.CORBA.Object objRef = null;

        try
        {
            objRef = group_manager.get_member_ref(objElement, loc);
        }
        catch (Exception e)
        {
            System.out.println(" Generic exception, test failed: " + e);
            e.printStackTrace();
        }
        System.out.println("   ...MemberReference --> " + objRef);
        System.out.println("   ------------------------------------------------------------------------");*/
    }

    /**
     * Start the cpp client
     * 
     * @return
     */
    static private boolean startCppClient()
    {
        System.out.println("  START THE CPP CLIENT ");
        try
        {
            if(!nsFlag){
                processClient = cdmw.ossupport.OS.createProcess("cdmw_test_client");
            }
            else {
                String process = "cdmw_test_client --namingRepos-url="+cobalocNS;
                processClient = cdmw.ossupport.OS.createProcess(process);
            }
            
            cdmw.ossupport.OS.sleep(2000);
        }
        catch (IOException e)
        {
            e.printStackTrace();
            return false;
        }
        catch (InterruptedException e)
        {
            e.printStackTrace();
        }
        return true;
    }

    static public org.omg.CORBA.Object getGroup()
    {
        NameComponent[] loc = new NameComponent[1];
        loc[0] = new NameComponent();
        loc[0].id = "hello_group_round_robin";
        loc[0].kind = "";
        Object objTmp = null;
        try
        {
            objTmp = group_manager.get_object_group_ref_from_name(loc);
        }
        catch (ObjectGroupNotFound e)
        {
            e.printStackTrace();
        }
        return objTmp;
    }

    static private boolean writeCHECK(int invocation)
    {
        PrintWriter out;
        try
        {
            out = new PrintWriter(new FileWriter(fileCheck, false));
            out.println(invocation);
            out.flush();
            out.close();
        }
        catch (IOException e)
        {
            e.printStackTrace();
            return false;
        }
        sleep(sleepTime);
        return true;
    }

    static private void sleep(int time)
    {
        try
        {
            cdmw.ossupport.OS.sleep(time);
        }
        catch (InterruptedException e)
        {
            e.printStackTrace();
        }
    }
    
    static private boolean checkError(){
        File error = new File(fileError);
        if(error.exists()){
            return true;
        }
        else{
            return false;
        }
    }
}
