#!/bin/sh
# =====================================================================
# This file is part of CARDAMOM (R) which is jointly developed by THALES
# and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
# Copyright (C) THALES 2000-2003. All rights reserved.
# Copyright (C) THALES 2004-2005. All rights reserved.
# Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
# All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
# License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

if test "`echo -e xxx`" = "xxx"
then
    echo="echo -e"
else
    echo=echo
fi

if [ "$1" = "--help" ]
then
    $echo "Usage: start_demo [option]"
    $echo ""
    $echo "Where option includes"
    $echo "   --killFTprim         : Use this option to kill the primary FT Manager"
    $echo "   --killSupervisionPrim: Use this option to kill the primary Supervision Manager"
    $echo "   --killDaemonPrim     : Use this option to kill the Platform daemon where is located the Primary FT Manager and Supervision"
    $echo "   --killAllPrim        : Use this option to kill the primary host, the Platform Daemon, the FTManager and Supervision"
    $echo "   --unplugNetwork      : Use this option to unplug a network cable"
    $echo "   --reboot             : Use this option to reboot a machine"
    $echo "   --help               : print this help message"
    exit 
fi


SITE_CONFIG=../site.mk

if test ! -f "$SITE_CONFIG"
then
    $echo "$SITE_CONFIG file is missing. Run '../configure.sh' and restart $0 "
    exit 1
fi

# CDMW installation folder
CDMW_HOME="`cat $SITE_CONFIG | grep CDMW_HOME |cut -d= -f2`"

LD_LIBRARY_PATH=$CDMW_HOME/lib/c++:$LD_LIBRARY_PATH
export LD_LIBRARY_PATH
PATH=$CDMW_HOME/bin:$PATH
export PATH

HOSTNAME=`hostname`
# Check HOSTNAME2 is set
if [ "$HOSTNAME2" = "" ]
then
    $echo "Export HOSTNAME2 to set the other host used by this demo. Then, run 'make data' in parent directory restart $0"
     rm ../data/Scenario.xml
    exit 1
fi

# Check HOSTNAME3 is set
if [ "$HOSTNAME3" = "" ]
then
    $echo "Export HOSTNAME3 to set the other host used by this demo. Then, run 'make data' in parent directory restart $0"
     rm ../data/Scenario.xml
    exit 1
fi

# XML scenario file
SCENARIO_FILE=../data/Scenario.xml

if test ! -f "$SCENARIO_FILE"
then
    $echo "$SCENARIO_FILE file is missing. Run 'make data' in parent directory restart $0 "
    exit 1
fi

# XML Platform Management Daemon configuration file
DAEMON_CONF_FILE=../data/CdmwPlatformMngtDaemon_conf.xml

if test ! -f "$DAEMON_CONF_FILE"
then
    $echo "$DAEMON_CONF_FILE file is missing. Run 'make data' in parent directory restart $0 "
    exit 1
fi


if test ! -x `which $TERM`
then
    TERM=xterm
fi

TERM_PATH=`which $TERM`

INIT_TIMEOUT=15
FT_MANAGER_TIMEOUT=20
START_TIMEOUT=20
EXEC_TIMEOUT=90
CLEANUP_TIMEOUT=5


#######################################################################################################
# 0) Install exe on HOSTNAME2
echo "checking if demo installed on HOSTNAME2 ..........."

BUILD_TARGET=`$CDMW_HOME/bin/config.guess`
DEMO_ALREADY_INSTALLED=`rsh $HOSTNAME2 ls $CDMW_HOME/demos/demo_crash/c++/$BUILD_TARGET 2>/dev/null` 

if [ "$DEMO_ALREADY_INSTALLED" = "" ]
then
   $echo Installing demo on $HOSTNAME2

   rsh $HOSTNAME2 mkdir -p $CDMW_HOME
    $echo "  => Copying $CDMW_HOME/bin"
   rcp -r $CDMW_HOME/bin $HOSTNAME2:$CDMW_HOME
    $echo "  => Copying $CDMW_HOME/lib"
   rcp -r $CDMW_HOME/lib $HOSTNAME2:$CDMW_HOME
    $echo "  => Copying $CDMW_HOME/share"
   rcp -r $CDMW_HOME/share $HOSTNAME2:$CDMW_HOME  

   rsh $HOSTNAME2 mkdir -p $CDMW_HOME/demos/demo_crash/c++
    $echo "  => Copying $CDMW_HOME/demos/demo_crash/c++/site.mk"
   rcp $CDMW_HOME/demos/demo_crash/c++/site.mk $HOSTNAME2:$CDMW_HOME/demos/demo_crash/c++
   rsh $HOSTNAME2 mkdir $CDMW_HOME/demos/demo_crash/c++/generated
    rcp $CDMW_HOME/demos/demo_crash/c++/generated/Server_Desc.dtd $HOSTNAME2:$CDMW_HOME/demos/demo_crash/c++/generated
    $echo "  => Copying $CDMW_HOME/demos/demo_crash/c++/data"
   rcp -r $CDMW_HOME/demos/demo_crash/c++/data $HOSTNAME2:$CDMW_HOME/demos/demo_crash/c++
    $echo "  => Copying $CDMW_HOME/demos/demo_crash/c++/run"
   rcp -r $CDMW_HOME/demos/demo_crash/c++/run $HOSTNAME2:$CDMW_HOME/demos/demo_crash/c++

   rsh $HOSTNAME2 mkdir -p $CDMW_HOME/demos/demo_crash/c++/$BUILD_TARGET
    $echo "  => Copying $CDMW_HOME/demos/demo_crash/c++/$BUILD_TARGET"
   rcp -r $CDMW_HOME/demos/demo_crash/c++/$BUILD_TARGET $HOSTNAME2:$CDMW_HOME/demos/demo_crash/c++

   rsh $HOSTNAME2 mkdir -p $CDMW_HOME/demos/demo_crash/c++/generated
    $echo "  => Copying $CDMW_HOME/demos/demo_crash/c++/generated"
   rcp -r $CDMW_HOME/demos/demo_crash/c++/generated $HOSTNAME2:$CDMW_HOME/demos/demo_crash/c++

   if [ -d  $CDMW_HOME/../tools ]
   then
    $echo "  => Copying $CDMW_HOME/../tools"
    rcp -r $CDMW_HOME/../tools $HOSTNAME2:$CDMW_HOME/..
   fi
else
   $echo Demo already installed on $HOSTNAME2
fi

#######################################################################################################
# 0) Install exe on HOSTNAME3
echo "checking if demo installed on HOSTNAME3 ..........."

BUILD_TARGET=`$CDMW_HOME/bin/config.guess`
#DEMO_ALREADY_INSTALLED=`rsh $HOSTNAME3 ls $CDMW_HOME/demos/demo_crash/c++/$BUILD_TARGET 2>/dev/null` 
DEMO_ALREADY_INSTALLED=`rsh $HOSTNAME3 ls $CDMW_HOME/demos/demo_crash/c++/$BUILD_TARGET 2>/dev/null` 

if [ "$DEMO_ALREADY_INSTALLED" = "" ]
then
   $echo Installing demo on $HOSTNAME3

   rsh $HOSTNAME3 mkdir -p $CDMW_HOME
    $echo "  => Copying $CDMW_HOME/bin"
   rcp -r $CDMW_HOME/bin $HOSTNAME3:$CDMW_HOME
    $echo "  => Copying $CDMW_HOME/lib"
   rcp -r $CDMW_HOME/lib $HOSTNAME3:$CDMW_HOME
    $echo "  => Copying $CDMW_HOME/share"
   rcp -r $CDMW_HOME/share $HOSTNAME3:$CDMW_HOME

   rsh $HOSTNAME3 mkdir -p $CDMW_HOME/demos/demo_crash/c++
    $echo "  => Copying $CDMW_HOME/demos/demo_crash/c++/site.mk"
   rcp $CDMW_HOME/demos/demo_crash/c++/site.mk $HOSTNAME3:$CDMW_HOME/demos/demo_crash/c++
   rsh $HOSTNAME3 mkdir $CDMW_HOME/demos/demo_crash/c++/generated
    rcp $CDMW_HOME/demos/demo_crash/c++/generated/Server_Desc.dtd $HOSTNAME3:$CDMW_HOME/demos/demo_crash/c++/generated
    $echo "  => Copying $CDMW_HOME/demos/demo_crash/c++/data"
   rcp -r $CDMW_HOME/demos/demo_crash/c++/data $HOSTNAME3:$CDMW_HOME/demos/demo_crash/c++
    $echo "  => Copying $CDMW_HOME/demos/demo_crash/c++/run"
   rcp -r $CDMW_HOME/demos/demo_crash/c++/run $HOSTNAME3:$CDMW_HOME/demos/demo_crash/c++

   rsh $HOSTNAME3 mkdir -p $CDMW_HOME/demos/demo_crash/c++/$BUILD_TARGET
    $echo "  => Copying $CDMW_HOME/demos/demo_crash/c++/$BUILD_TARGET"
   rcp -r $CDMW_HOME/demos/demo_crash/c++/$BUILD_TARGET $HOSTNAME3:$CDMW_HOME/demos/demo_crash/c++

   rsh $HOSTNAME3 mkdir -p $CDMW_HOME/demos/demo_crash/c++/generated
    $echo "  => Copying $CDMW_HOME/demos/demo_crash/c++/generated"
   rcp -r $CDMW_HOME/demos/demo_crash/c++/generated $HOSTNAME3:$CDMW_HOME/demos/demo_crash/c++

   if [ -d  $CDMW_HOME/../tools ]
   then
    $echo "  => Copying $CDMW_HOME/../tools"
    rcp -r $CDMW_HOME/../tools $HOSTNAME3:$CDMW_HOME/..
   fi
else
   $echo Demo already installed on $HOSTNAME3
fi

#######################################################################################################
# 1) Start daemon 
$echo Starting the Platform Management Daemons...
DAEMON_COMMAND="$CDMW_HOME/bin/cdmw_platform_daemon.sh --CdmwXMLFile=$DAEMON_CONF_FILE"
$TERM -ls -sb -sl 5000 -T cdmw_platform_$HOSTNAME -e $DAEMON_COMMAND &

DAEMON_COMMAND2="$CDMW_HOME/bin/cdmw_platform_daemon.sh --CdmwXMLFile=$CDMW_HOME/demos/demo_crash/c++/data/CdmwPlatformMngtDaemon_conf.xml"

rsh $HOSTNAME2 "(HOSTNAME2=$HOSTNAME2; export HOSTNAME2; HOSTNAME3=$HOSTNAME3; export HOSTNAME3; LD_LIBRARY_PATH=$LD_LIBRARY_PATH; export LD_LIBRARY_PATH; PATH=$PATH; export PATH; $TERM_PATH -display $DISPLAY -sb -sl 5000 -T cdmw_platform_$HOSTNAME2 -e $DAEMON_COMMAND2)" &

rsh $HOSTNAME3 "(HOSTNAME2=$HOSTNAME2; export HOSTNAME2; HOSTNAME3=$HOSTNAME3; export HOSTNAME3; LD_LIBRARY_PATH=$LD_LIBRARY_PATH; export LD_LIBRARY_PATH; PATH=$PATH; export PATH; $TERM_PATH -display $DISPLAY -sb -sl 5000 -T cdmw_platform_$HOSTNAME3 -e $DAEMON_COMMAND2)" &

trap '$DAEMON_COMMAND stop; exit' 2

sleep $INIT_TIMEOUT

#######################################################################################################
# 1b) Start FT Manager
$echo Starting the  FT Manager on $HOSTNAME2...
FTMANAGER_COMMAND2="$CDMW_HOME/bin/cdmw_ft_manager --CdmwXMLFile=$CDMW_HOME/demos/demo_crash/c++/data/CdmwFaultToleranceManager_conf.xml --groupConf=$CDMW_HOME/demos/demo_crash/c++/data/CdmwFTSystemMngtGroupCreator_conf.xml"

rsh $HOSTNAME2 "(LD_LIBRARY_PATH=$LD_LIBRARY_PATH; export LD_LIBRARY_PATH; PATH=$PATH; export PATH; $TERM_PATH -display $DISPLAY -sb -sl 5000 -T cdmw_ft_manager_$HOSTNAME2 -e ${SHELL_FORWARD} $FTMANAGER_COMMAND2)" &


sleep $FT_MANAGER_TIMEOUT


# 2) Start Platform Management Supervision
$echo Starting the Backup Platform Management Supervision on $HOSTNAME2 ........
SUPERVISION_COMMAND2="$CDMW_HOME/bin/cdmw_platform_supervision_starter --CdmwXMLFile=$CDMW_HOME/demos/demo_crash/c++/data/CdmwPlatformMngtSystemStart.xml --validate"
rsh $HOSTNAME2 "(LD_LIBRARY_PATH=$LD_LIBRARY_PATH; export LD_LIBRARY_PATH; PATH=$PATH; export PATH; $SUPERVISION_COMMAND2)" &
sleep $INIT_TIMEOUT

#######################################################################################################

# 2) Start FT Manager
$echo Starting the Backup FT Manager on $HOSTNAME3...
FTMANAGER_COMMAND3="$CDMW_HOME/bin/cdmw_ft_manager --CdmwXMLFile=$CDMW_HOME/demos/demo_crash/c++/data/CdmwFaultToleranceManager_conf.xml --groupConf=$CDMW_HOME/demos/demo_crash/c++/data/CdmwFTSystemMngtGroupCreator_conf.xml"

rsh $HOSTNAME3 "(LD_LIBRARY_PATH=$LD_LIBRARY_PATH; export LD_LIBRARY_PATH; PATH=$PATH; export PATH; $TERM_PATH -display $DISPLAY -sb -sl 5000 -T cdmw_ft_manager_$HOSTNAME3 -e ${SHELL_FORWARD} $FTMANAGER_COMMAND3)" &

# 2) Start the Supervision Backup
$echo Starting the Backup Platform Management Supervision on $HOSTNAME3 ........
SUPERVISION_COMMAND3="$CDMW_HOME/bin/cdmw_platform_supervision_starter --CdmwXMLFile=$CDMW_HOME/demos/demo_crash/c++/data/CdmwPlatformMngtSystemStart.xml --validate"
rsh $HOSTNAME3 "(LD_LIBRARY_PATH=$LD_LIBRARY_PATH; export LD_LIBRARY_PATH; PATH=$PATH; export PATH; $SUPERVISION_COMMAND3)" &

#######################################################################################################
# 3) Start Platform Management Supervision Observers

$echo Starting the Platform Management Supervision Observer...
OBSERVER_COMMAND2="$CDMW_HOME/demos/demo_crash/c++/$BUILD_TARGET/tasAgent --register --system-corbaloc=corbaloc::localhost:21376/CdmwPlatformMngtSupervision --observer-name=Observer1"
rsh $HOSTNAME "(LD_LIBRARY_PATH=$LD_LIBRARY_PATH; export LD_LIBRARY_PATH; PATH=$PATH; export PATH; $TERM_PATH -display $DISPLAY -sb -sl 5000 -T Observer$HOSTNAME -e ${SHELL_FORWARD} $OBSERVER_COMMAND2)" &

$echo Starting the Platform Management Supervision Observer...
OBSERVER_COMMAND2="$CDMW_HOME/demos/demo_crash/c++/$BUILD_TARGET/tasAgent --register --system-corbaloc=corbaloc::localhost:21376/CdmwPlatformMngtSupervision --observer-name=Observer2"
rsh $HOSTNAME2 "(LD_LIBRARY_PATH=$LD_LIBRARY_PATH; export LD_LIBRARY_PATH; PATH=$PATH; export PATH; $TERM_PATH -display $DISPLAY -sb -sl 5000 -T Observer$HOSTNAME2 -e ${SHELL_FORWARD} $OBSERVER_COMMAND2)" &

$echo Starting the Platform Management Supervision Observer...
OBSERVER_COMMAND2="$CDMW_HOME/demos/demo_crash/c++/$BUILD_TARGET/tasAgent --register --system-corbaloc=corbaloc::localhost:21376/CdmwPlatformMngtSupervision --observer-name=Observer3"
rsh $HOSTNAME3 "(LD_LIBRARY_PATH=$LD_LIBRARY_PATH; export LD_LIBRARY_PATH; PATH=$PATH; export PATH; $TERM_PATH -display $DISPLAY -sb -sl 5000 -T Observer$HOSTNAME3 -e ${SHELL_FORWARD} $OBSERVER_COMMAND2)" &

sleep $INIT_TIMEOUT

#######################################################################################################
# 4) Define system
$echo Defining the System '$SCENARIO_FILE' 
$CDMW_HOME/bin/cdmw_platform_admin --system-corbaloc=corbaloc::$HOSTNAME:21376/CdmwPlatformMngtSupervision --sys-define $SCENARIO_FILE


#######################################################################################################
# 5) Start_system
$CDMW_HOME/bin/cdmw_platform_admin --system-corbaloc=corbaloc::$HOSTNAME:21376/CdmwPlatformMngtSupervision --sys-start

sleep $START_TIMEOUT

# 6) Get Snapshot
$CDMW_HOME/bin/cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21376/CdmwPlatformMngtSupervision --sys-snapshot


#######################################################################################################
# 7) Killing steps

sleep $START_TIMEOUT

if [ "$1" = "--killAllPrim" ]
then
    $echo "Killing All CDMW processes on Primary "
    rsh $HOSTNAME2 $CDMW_HOME/demos/demo_crash/c++/run/killFTmanager.sh
    rsh $HOSTNAME2 $CDMW_HOME/demos/demo_crash/c++/run/killSupervision.sh
    rsh $HOSTNAME2 $CDMW_HOME/demos/demo_crash/c++/run/killPlatformDaemon.sh
 elif [ "$1" = "--killFTprim" ]
    then
    $echo "Killing the FT CDMW pimary option chosen"
    rsh $HOSTNAME2 $CDMW_HOME/demos/demo_crash/c++/run/killFTmanager.sh
elif [ "$1" = "--killSupervisionPrim" ]
    then
    $echo "Killing the FT CDMW pimary option chosen"
    rsh $HOSTNAME2 $CDMW_HOME/demos/demo_crash/c++/run/killSupervision.sh
elif [ "$1" = "--killDaemonPrim" ]
    then
    $echo "Killing the FT CDMW pimary option chosen"
    rsh $HOSTNAME2 $CDMW_HOME/demos/demo_crash/c++/run/killPlatformDaemon.sh
 elif [ "$1" = "--unplugNetwork" ]
    then
    $echo "Unplug the network cable then hit return ........."
    read FOO
 else
    if [ "$1" = "--reboot" ]
    then
    $echo "Reboot your host then hit return ........."
    read FOO
    fi
fi


#######################################################################################################
# 8) Get a snapshot of the system from the HOST3
$echo "Hit a key when to get the Snapshot ........................................"
read FOO
$CDMW_HOME/bin/cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21376/CdmwPlatformMngtSupervision --sys-snapshot

$echo "=============================================================="
$echo "Hit a key when ready to stop the CDMW Daemon and Supervision."
read FOO
$echo "=============================================================="

#######################################################################################################
# 9) stop_system
$CDMW_HOME/bin/cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21376/CdmwPlatformMngtSupervision --sys-stop
#sleep $CLEANUP_TIMEOUT


$DAEMON_COMMAND stop
rsh $HOSTNAME2 $DAEMON_COMMAND2 stop
rsh $HOSTNAME3  $DAEMON_COMMAND2 stop
kill -9 $FT_MANAGER_PID
rsh $HOSTNAME $CDMW_HOME/demos/demo_crash/c++/run/killObserver.sh
rsh $HOSTNAME2 $CDMW_HOME/demos/demo_crash/c++/run/killObserver.sh
rsh $HOSTNAME3 $CDMW_HOME/demos/demo_crash/c++/run/killObserver.sh
rsh $HOSTNAME2 $CDMW_HOME/demos/demo_crash/c++/run/killFTmanager.sh
rsh $HOSTNAME3 $CDMW_HOME/demos/demo_crash/c++/run/killFTmanager.sh
$echo "done."

rm -f *.ior
rm -f *.IOR

