/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Standard Files 
#include <iostream>
#include <string>

// Cdmw Files
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <ConfAndPlug/cdmwinit/CdmwInterface.hpp>
#include <FaultTolerance/ftinit/FTServiceInit.hpp>

#include <Repository/idllib/CdmwRepositoryExt.stub.hpp>
#include <FaultTolerance/idllib/CdmwFTReplicationManager.stub.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp>


using namespace Cdmw;

int main( int argc, char* argv[] )
{

    try
    {
        // Initialise FT service
        Cdmw::CdmwInit::CDMW_preORBInit(argc, argv);

        // Initialises the ORB
        OrbSupport::StrategyList strategyList;
            strategyList.add_OrbThreaded();
            strategyList.add_PoaThreadPerConnection();

        CORBA::ORB_var orb = Cdmw::OrbSupport::OrbSupport::ORB_init(
            argc, argv, strategyList );

        // Get the root POA
        CORBA::Object_var obj = orb->resolve_initial_references( "RootPOA" );
        PortableServer::POA_var rootPOA
            = PortableServer::POA::_narrow( obj.in() );

        PortableServer::POAManager_var poaMgr = rootPOA->the_POAManager();
        poaMgr->activate();

        
        Cdmw::Logging::Logger* logger =
            Cdmw::Logging::Logger::Get_Logger("DEMO.NS_CLIENT");
        

        CDMW_INFO(logger, "------- Before CDMW_init");
            
        // init Cdmw
        Cdmw::CdmwInit::CDMW_init(
            orb.in(), argc, argv, 0 );

        CDMW_INFO(logger, "------- After CDMW_init");
        
        
        obj = orb->resolve_initial_references("ReplicationManager");
        CdmwFT::ReplicationManager_var replication_manager =
            CdmwFT::ReplicationManager::_narrow(obj.in());
            
        ::FT::Name TestOGDName;
        TestOGDName.length(1);
        TestOGDName[0].id="RepositoryGroup";
        obj = replication_manager->get_object_group_ref_from_name(TestOGDName);

        CdmwNamingAndRepository::RepositoryExt_var repository =
            CdmwNamingAndRepository::RepositoryExt::_narrow(obj.in());
        
        
        int i=0;
        while (true) {
            try {
                CdmwNamingAndRepository::RootContextDecriptionSeq_var res = 
                    repository->get_root_context();
                CDMW_INFO(logger, "request [" << ++i << "] get_root_context OK");

            } catch (const CORBA::SystemException& e) {
                CDMW_INFO(logger, "request [" << i << "] exception " 
                          << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)); 
            }
            
            Cdmw::OsSupport::OS::sleep(50);
        }
        


/*
        CDMW_INFO(logger, "------- get System");
        CORBA::Object_var systemObj = 
            orb->string_to_object("corbaloc::localhost:21336/CdmwPlatformMngtSupervision");
        CdmwPlatformMngt::System_var system = 
            CdmwPlatformMngt::System::_narrow(systemObj.in());
        CDMW_INFO(logger, "------- get Application");
        CdmwPlatformMngt::Application_var application = 
            system->get_application("FTApplication");
        CORBA::String_var ior = orb->object_to_string(application.in());
        CDMW_INFO(logger, "------- Application: " << ior.in());

        int i=0;
        while (true) {
            std::cout << std::endl << std::endl;
            CDMW_INFO(logger, "======> [" << ++i << "] get_process");

            try {
                CdmwPlatformMngt::Process_var process =
                    application->get_process("hello22", "");
                CDMW_INFO(logger, "<====== [" << i << "] get_process OK");

            } catch (const CORBA::SystemException& e) {
                CDMW_INFO(logger, "<====== [" << i << "] exception " 
                          << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)); 
            }
            
            Cdmw::OsSupport::OS::sleep(50);
        }
*/
        
        

    }
    catch( const CORBA::Exception &e )
    {
        std::cerr <<__FILE__<<" "<<__LINE__<<" "<< e._name() << std::endl;
        return EXIT_FAILURE;
    }
    catch( const Cdmw::Exception &e )
    {
        std::cerr <<__FILE__<<" "<<__LINE__<<" "<< e.what() << std::endl;
        return EXIT_FAILURE;
    }
    catch( ... )
    {
        std::cerr << "Unexpected exception" << std::endl;
        return EXIT_FAILURE;
    }

    return EXIT_SUCCESS;
}

