/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include "Foundation/common/Assert.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/logging/Logger.hpp"

#include "Foundation/testutils/CORBATestManager.hpp"

#include <Command_impl.hpp>
#include <CCM_ClientDance_impl.hpp>



//
// IDL:thalesgroup.com/CcmDance1Test/ClientModule/CCM_Client:1.0
//
CcmDance1Test::CCM_ClientDance_impl::CCM_ClientDance_impl()
    : m_expectedServerHostname(""),
      m_hostnameCommandResult(""),
      m_session_context(CcmDance1Test::ClientModule::CCM_ClientDance_Context::_nil())
{
    m_hostInfo = new CcmDance1Test::FacetModule::CCM_ClientHostInfo_impl();
    m_logger = Cdmw::Logging::Logger::Get_Logger("ClientDance");
}

CcmDance1Test::CCM_ClientDance_impl::~CCM_ClientDance_impl()
{
    CDMW_INFO(m_logger, m_configuration << " : CCM_ClientDance_impl::~CCM_ClientDance_impl called!");
}



//
// attribute configuration
//
char*
CcmDance1Test::CCM_ClientDance_impl::configuration()
    throw(CORBA::SystemException)
{
    return CORBA::string_dup(m_configuration.in());
}

void
CcmDance1Test::CCM_ClientDance_impl::configuration(const char* a)
    throw(CORBA::SystemException)
{
    m_configuration =  CORBA::string_dup(a);
}


//
// attribute expectedServerHostname
//
char*
CcmDance1Test::CCM_ClientDance_impl::expectedServerHostname()
    throw(CORBA::SystemException)
{
    return CORBA::string_dup(m_expectedServerHostname.in());
}

void
CcmDance1Test::CCM_ClientDance_impl::expectedServerHostname(const char* a)
    throw(CORBA::SystemException)
{
    m_expectedServerHostname =  CORBA::string_dup(a);
}



//
// facet clientHostInfo
//
CcmDance1Test::FacetModule::CCM_ClientHostInfo_ptr 
CcmDance1Test::CCM_ClientDance_impl::get_clientHostInfo ()
    throw (CORBA::SystemException)
{
    return CcmDance1Test::FacetModule::CCM_ClientHostInfo::_duplicate(m_hostInfo.in());
}


    
//
// consumer hostnameResults
//
void 
CcmDance1Test::CCM_ClientDance_impl::push_hostnameResults (CcmDance1Test::EventTypeModule::HostnameResult * event)
    throw (CORBA::SystemException)
{
    CDMW_INFO(m_logger, m_configuration << " : CCM_ClientDance_impl::CCM_ClientDance_impl::push_hostnameResults called with hostname " << event->hostname() );
    m_hostnameCommandResult = event->hostname();
}
     



//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
CcmDance1Test::CCM_ClientDance_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, m_configuration << " : CCM_ClientDance_impl::Set_session_context called!");
   
    m_session_context = ::CcmDance1Test::ClientModule::CCM_ClientDance_Context::_narrow(ctx); 
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CcmDance1Test::CCM_ClientDance_impl::ccm_activate()
   throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, m_configuration << " : CCM_ClientDance_impl::Ccm_activate called!");
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
CcmDance1Test::CCM_ClientDance_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, m_configuration << " : CCM_ClientDance_impl::Ccm_passivate called!");
} 


//
// ccm_remove function : it destroys the component (call by the container)
//
void
CcmDance1Test::CCM_ClientDance_impl::ccm_remove()
   throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, m_configuration << " : CCM_ClientDance_impl::Ccm_remove called!");
}



//
// get result of assynchronous getHostname command
//
char* 
CcmDance1Test::CCM_ClientDance_impl::getHostnameCommandResult()
{
    char* result = CORBA::string_dup(m_hostnameCommandResult.in());
    return result;
}


//
// supported interface TestableComponent
// 
char * 
CcmDance1Test::CCM_ClientDance_impl::get_property(const char * prop_name) 
    throw (CORBA::SystemException) 
{
    CDMW_INFO(m_logger, m_configuration << " : CCM_ClientDance_impl::get_property");
    CORBA::String_var propertyValue = CORBA::string_dup("");
    
    if (strcmp(prop_name, CcmDance1Test::TestFramework::PROPERTY_GET_HOSTNAME) == 0) {
        CDMW_INFO(m_logger, m_configuration << " : PROPERTY_GET_HOSTNAME");
        propertyValue = m_expectedServerHostname;
        
    } else if (strcmp(prop_name, CcmDance1Test::TestFramework::PROPERTY_GET_SERVER_HOSTNAME) == 0) {
        CDMW_INFO(m_logger, m_configuration << " : PROPERTY_GET_SERVER_HOSTNAME");
        try { 
            CcmDance1Test::FacetModule::HostInfo_var hostInfo = 
                m_session_context->get_connection_serverHostInfo();
            
            if (CORBA::is_nil(hostInfo.in())) {
                CDMW_ERROR(m_logger, m_configuration <<  " : hostInfo is NIL !!!! ");
                throw CORBA::BAD_PARAM();
            }
            
            propertyValue = hostInfo->get_hostname();
            
        } catch (const CORBA::SystemException& e) {
            CDMW_ERROR(m_logger, m_configuration << " : Unexpected CORBA::SystemException: " << e);
            throw;
        } catch (const CORBA::Exception& e) {
            CDMW_ERROR(m_logger, m_configuration << " : Unexpected CORBA::Exception: " << e);
            throw;
        } catch (...) {
            CDMW_ERROR(m_logger, m_configuration << " : Unknown exception !! ");
            throw;
        }
    } else {
        CDMW_INFO(m_logger, m_configuration << " : Unkown property");
        propertyValue = CORBA::string_dup("UnknownProperty");
    }
    
    CDMW_INFO(m_logger, m_configuration << " : return " << propertyValue.in());
    return propertyValue._retn();
}

void 
CcmDance1Test::CCM_ClientDance_impl::reset() 
    throw (CORBA::SystemException) 
{
    CDMW_INFO(m_logger, m_configuration << " : CCM_ClientDance_impl::reset");
    m_hostnameCommandResult = CORBA::string_dup("");
}

CORBA::Boolean 
CcmDance1Test::CCM_ClientDance_impl::check(const char * ref) 
    throw (CORBA::SystemException) 
{
    CORBA::Boolean result = false;
    if (strcmp(ref, m_hostnameCommandResult.in()) == 0) {
        CDMW_INFO(m_logger, m_configuration << " : Server's hostname OK !");
        result = true;
    } else {
        CDMW_ERROR(m_logger, m_configuration << " : Server's hostname is not the one expected: " << m_expectedServerHostname.in());
    }

    return result;
}

void 
CcmDance1Test::CCM_ClientDance_impl::do_test(CORBA::Long idx) throw (CORBA::SystemException) 
{
    CDMW_INFO(m_logger, m_configuration << " : CCM_ClientDance_impl::do_test ("<< idx << ").");
    if (1 == idx) {
        CcmDance1Test::EventTypeModule::Command_var command = 
            new CcmDance1Test::Command_impl();
        command->commandName(CcmDance1Test::EventTypeModule::COMMAND_GET_HOSTNAME);

        CDMW_INFO(m_logger, m_configuration << " : --<> emits command >--" <<  command->commandName());
        m_session_context->push_commandEmits(command.in());
    } else if (2 == idx) {
        CcmDance1Test::EventTypeModule::Command_var command2 = 
            new CcmDance1Test::Command_impl();
        command2->commandName(CcmDance1Test::EventTypeModule::COMMAND_CHECK_HOSTNAME);

        CDMW_INFO(m_logger, m_configuration << " : --<> emits command >--" <<  command2->commandName());
        m_session_context->push_commandEmits(command2.in());
    }
}


