/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/logging/Logger.hpp"

#include <HostnameResult_impl.hpp>
#include <CCM_HostInfo_impl.hpp>
#include <CCM_ServerDance_impl.hpp>



using Cdmw::Logging::Logger;
//
// IDL:thalesgroup.com/CcmDance1Test/ServerModule/CCM_ServerDance:1.0
//
CcmDance1Test::CCM_ServerDance_impl::CCM_ServerDance_impl()
{
    m_bCheckResult = false;
    
    m_expectedClientHostname1 = CORBA::string_dup("");
    m_expectedClientHostname2 = CORBA::string_dup("");
    m_hostInfo = new CcmDance1Test::FacetModule::CCM_HostInfo_impl();
   
    m_logger = Cdmw::Logging::Logger::Get_Logger("ServerDance");
}

CcmDance1Test::CCM_ServerDance_impl::~CCM_ServerDance_impl()
{
    CDMW_INFO(m_logger, "CCM_ServerDance_impl::~CCM_ServerDance_impl called!");
}



//
// attribute expectedClientHostname1
//
char*
CcmDance1Test::CCM_ServerDance_impl::expectedClientHostname1()
    throw(CORBA::SystemException)
{
    return CORBA::string_dup(m_expectedClientHostname1);
}

void
CcmDance1Test::CCM_ServerDance_impl::expectedClientHostname1(const char* a)
    throw(CORBA::SystemException)
{
    m_expectedClientHostname1 =  CORBA::string_dup(a);
}

//
// attribute expectedClientHostname2
//
char*
CcmDance1Test::CCM_ServerDance_impl::expectedClientHostname2()
    throw(CORBA::SystemException)
{
    return CORBA::string_dup(m_expectedClientHostname2);
}

void
CcmDance1Test::CCM_ServerDance_impl::expectedClientHostname2(const char* a)
    throw(CORBA::SystemException)
{
    m_expectedClientHostname2 =  CORBA::string_dup(a);
}




//
// consumer commands
//
void 
CcmDance1Test::CCM_ServerDance_impl::push_commands (CcmDance1Test::EventTypeModule::Command * event)
    throw (CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_ServerDance_impl::push_commands called with command " << event->commandName() );
    
    if (strcmp(event->commandName(), CcmDance1Test::EventTypeModule::COMMAND_CHECK_HOSTNAME) == 0) {
        try {
            CcmDance1Test::FacetModule::ClientHostInfo_var hostInfo1 = 
                m_session_context->get_connection_clientHostInfo1();
            if (CORBA::is_nil(hostInfo1.in())) {
                CDMW_ERROR(m_logger, "hostInfo1 is NIL !!!!");
                return;
            }
            
            CORBA::String_var hostname1 = hostInfo1->get_hostname();
            if (strcmp(m_expectedClientHostname1, hostname1.in()) == 0) {
                CDMW_INFO(m_logger, "Client1's hostname OK !");
            } else {
                CDMW_ERROR(m_logger, "Client1's hostname is not the one expected: " << hostname1.in() );
                return;
            }
            
            CcmDance1Test::FacetModule::ClientHostInfo_var hostInfo2 = 
                m_session_context->get_connection_clientHostInfo2();
            if (CORBA::is_nil(hostInfo2.in())) {
                CDMW_ERROR(m_logger, "hostInfo2 is NIL !!!!");
                return;
            }
            
            CORBA::String_var hostname2 = hostInfo2->get_hostname();
            if (strcmp(m_expectedClientHostname2, hostname2.in()) == 0) {
                CDMW_INFO(m_logger, "Client2's hostname OK !");
            } else {
                CDMW_ERROR(m_logger, "Client2's hostname is not the one expected: " << hostname2.in() );
                return;
            }
        } catch (const CORBA::SystemException& e) {
            CDMW_ERROR(m_logger, "Unexpected CORBA::SystemException: " << e );
        } catch (const CORBA::Exception& e) {
            CDMW_ERROR(m_logger, "Unexpected CORBA::Exception: " << e );
        } catch (...) {
            CDMW_ERROR(m_logger, "Unknown exception !!");
        }
        m_bCheckResult = true;
    } else {
        if (strcmp(event->commandName(), CcmDance1Test::EventTypeModule::COMMAND_GET_HOSTNAME) == 0) {
            std::string hostname = Cdmw::OsSupport::OS::get_hostname();
            CDMW_INFO(m_logger, "CCM_ServerDance_impl:  return result " << hostname );
            
            CcmDance1Test::EventTypeModule::HostnameResult_var result = new CcmDance1Test::HostnameResult_impl();
            result->hostname(CORBA::string_dup(hostname.c_str()));
            
            CDMW_INFO(m_logger, "CCM_ServerDance_impl:  emits HostnameResult");
            m_session_context->push_hostnamePublishes(result.in());
        }
        else
            CDMW_ERROR(m_logger, "CCM_ServerDance_impl:  Unkown Command !!!!");
    }
};

//
// facet serverHostInfo1
//
CcmDance1Test::FacetModule::CCM_HostInfo_ptr 
CcmDance1Test::CCM_ServerDance_impl::get_serverHostInfo1 ()
    throw (CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_ServerDance_impl::get_serverHostInfo1");
    return CcmDance1Test::FacetModule::CCM_HostInfo::_duplicate(m_hostInfo.in());
}

//
// facet serverHostInfo2
//
CcmDance1Test::FacetModule::CCM_HostInfo_ptr 
CcmDance1Test::CCM_ServerDance_impl::get_serverHostInfo2 ()
    throw (CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_ServerDance_impl::get_serverHostInfo2");
    return CcmDance1Test::FacetModule::CCM_HostInfo::_duplicate(m_hostInfo.in());
}

//
// supported interface HelloInterface 
//
char * 
CcmDance1Test::CCM_ServerDance_impl::say_hello(const char* msg)
    throw (CORBA::SystemException)
{
    CDMW_INFO(m_logger, "Hello ! " << msg);
    return CORBA::string_dup(msg);
}


//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
CcmDance1Test::CCM_ServerDance_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_ServerDance_impl::Set_session_context called!");
   
    m_session_context = ::CcmDance1Test::ServerModule::CCM_ServerDance_Context::_narrow(ctx); 
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CcmDance1Test::CCM_ServerDance_impl::ccm_activate()
    throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_ServerDance_impl::Ccm_activate called!");
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//

void
CcmDance1Test::CCM_ServerDance_impl::ccm_passivate()
    throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_ServerDance_impl::Ccm_passivate called!");
} 



//
// ccm_remove function : it destroys the component (call by the container)
void
CcmDance1Test::CCM_ServerDance_impl::ccm_remove()
    throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_ServerDance_impl::Ccm_remove called!");
}


    
char * 
CcmDance1Test::CCM_ServerDance_impl::get_property(const char * prop_name) 
throw (CORBA::SystemException) 
{
    throw CORBA::NO_IMPLEMENT();
};

void 
CcmDance1Test::CCM_ServerDance_impl::reset() 
throw (CORBA::SystemException) 
{
    CDMW_INFO(m_logger, "CCM_ServerDance_impl::reset");
    m_bCheckResult = false;
};

CORBA::Boolean 
CcmDance1Test::CCM_ServerDance_impl::check(const char * ref) 
    throw (CORBA::SystemException) 
{
    return m_bCheckResult;
};

void 
CcmDance1Test::CCM_ServerDance_impl::do_test(CORBA::Long idx) 
    throw (CORBA::SystemException) 
{
    throw CORBA::NO_IMPLEMENT();
};
