/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include "Foundation/common/Assert.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/logging/Logger.hpp"

#include "Foundation/testutils/CORBATestManager.hpp"

#include <Command_impl.hpp>
#include <CCM_TestDriver_impl.hpp>



//
// IDL:thalesgroup.com/CcmDance1Test/DriverModule/CCM_Client:1.0
//
CcmDance1Test::CCM_TestDriver_impl::CCM_TestDriver_impl()
    : m_session_context(CcmDance1Test::DriverModule::CCM_TestDriver_Context::_nil())
{
    m_logger = Cdmw::Logging::Logger::Get_Logger("TestDriver");
}

CcmDance1Test::CCM_TestDriver_impl::~CCM_TestDriver_impl()
{
    CDMW_INFO(m_logger, "CCM_TestDriver_impl::~CCM_TestDriver_impl called!");
}



//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
CcmDance1Test::CCM_TestDriver_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_TestDriver_impl::Set_session_context called!");

   
    m_session_context = ::CcmDance1Test::DriverModule::CCM_TestDriver_Context::_narrow(ctx); 
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CcmDance1Test::CCM_TestDriver_impl::ccm_activate()
   throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_TestDriver_impl::Ccm_activate called!");
   
   CORBA::Object_var context = get_context();
   Cdmw::TestUtils::CORBATestManager::instance()->add("driver_context", context.in());
   Cdmw::TestUtils::CORBATestManager::instance()->add("ccm_testDriver", this);
   m_testThread.start();
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
CcmDance1Test::CCM_TestDriver_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_TestDriver_impl::Ccm_passivate called!");
} 



//
// ccm_remove function : it destroys the component (call by the container)
//
void
CcmDance1Test::CCM_TestDriver_impl::ccm_remove()
   throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_TestDriver_impl::Ccm_remove called!");
}



CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(CcmDance1Test::test_ccm_dance, "test_ccm_dance");


void
CcmDance1Test::test_ccm_dance::do_tests_dance()
{
    Cdmw::OsSupport::OS::sleep(2000);
    TEST_INFO("Check environement.");
    if (CORBA::is_nil(m_session_context.in())) {
        TEST_INFO("TestDriver context is NULL !!");
        CPPUNIT_ASSERT(false);
        return;
    }
    if (CORBA::is_nil(m_testDriver.in())) {
        TEST_INFO("TestDriver component is NULL !!");
        CPPUNIT_ASSERT(false);
        return;
    }
    
    TEST_INFO("Get the testable components.");
    
    CcmDance1Test::TestFramework::TestableComponent_var serverComponent; 
    CcmDance1Test::TestFramework::TestableComponent_var client1Component;
    CcmDance1Test::TestFramework::TestableComponent_var client2Component;
    
    try { 
        serverComponent = m_session_context->get_connection_server();
        client1Component = m_session_context->get_connection_client1();
        client2Component = m_session_context->get_connection_client2();
        
        if (CORBA::is_nil(serverComponent.in())) {
            TEST_INFO("serverComponent is NIL !!!! ");
            CPPUNIT_ASSERT(false);
            return;
        }
        if (CORBA::is_nil(client1Component.in())) {
            TEST_INFO("client1Component is NIL !!!! ");
            CPPUNIT_ASSERT(false);
            return;
        }
        if (CORBA::is_nil(client2Component.in())) {
            TEST_INFO("client2Component is NIL !!!! ");
            CPPUNIT_ASSERT(false);
            return;
        }
    } catch (const CORBA::SystemException& e) {
        TEST_INFO("Unexpected CORBA::SystemException: " << e);
        CPPUNIT_ASSERT(false);
    } catch (const CORBA::Exception& e) {
        TEST_INFO("Unexpected CORBA::Exception: " << e);
        CPPUNIT_ASSERT(false);
    } catch (...) {
        TEST_INFO("Unknown exception !! ");
        CPPUNIT_ASSERT(false);
    }

    TEST_INFO("First test : primaryClient is Client1, secondaryClient is Client2");
    do_tests(serverComponent.in(), client1Component.in(), client2Component.in());
    TEST_INFO("End of first test.")
    
    TEST_INFO("Second test : primaryClient is Client2, secondaryClient is Client1");
    do_tests(serverComponent.in(), client2Component.in(), client1Component.in());
    TEST_INFO("End of second test.");
    
    TEST_INFO("Demo test status is SUCCESS !");
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger("TestDriver"), "Demo test status is SUCCESS !");
}

void
CcmDance1Test::test_ccm_dance::do_tests(
    CcmDance1Test::TestFramework::TestableComponent_ptr server,
    CcmDance1Test::TestFramework::TestableComponent_ptr primaryClient,
    CcmDance1Test::TestFramework::TestableComponent_ptr secondaryClient)
{
    //input parameters are not supposed to be NULL 

    try {
        TEST_INFO("Reseting components");
        server->reset();
        primaryClient->reset();
        secondaryClient->reset();
    
        CORBA::String_var clt1Hostname = 
            primaryClient->get_property(CcmDance1Test::TestFramework::PROPERTY_GET_HOSTNAME);
        TEST_INFO("primaryClient's expected hostame is " << clt1Hostname.in());
        
        CORBA::String_var clt2Hostname = 
            secondaryClient->get_property(CcmDance1Test::TestFramework::PROPERTY_GET_HOSTNAME);
        TEST_INFO("secondaryClient's expected hostame is " << clt2Hostname.in());
        
        CORBA::String_var srvHostname = 
            primaryClient->get_property(CcmDance1Test::TestFramework::PROPERTY_GET_SERVER_HOSTNAME);
        TEST_INFO("server's hostame is " << srvHostname.in());
        
        if (strcmp(clt1Hostname.in(), srvHostname.in()) == 0) {
            TEST_INFO("hostnames are identical");
            CPPUNIT_ASSERT(true);
        } else {
            TEST_INFO("ERROR : hostname mismatch");
            CPPUNIT_ASSERT(false);
        }
        
        TEST_INFO("Test primary client's sending getHostname command to the server");
        primaryClient->do_test(1);
        
        TEST_INFO("Waiting for result...");
        Cdmw::OsSupport::OS::sleep(2000);
        TEST_INFO("Check primaryClient's result");
        
        bool bCheck = primaryClient->check(clt1Hostname.in());
        
        if (true == bCheck) {
            TEST_INFO("hostnames are identical");
            CPPUNIT_ASSERT(true);
        } else {
            TEST_INFO("ERROR : hostname mismatch");
            CPPUNIT_ASSERT(false);
        }

        TEST_INFO("Check secondaryClient's result");
        
        bCheck = secondaryClient->check(clt2Hostname.in());
        
        if (true == bCheck) {
            TEST_INFO("hostnames are identical");
            CPPUNIT_ASSERT(true);
        } else {
            TEST_INFO("ERROR : hostname mismatch");
            CPPUNIT_ASSERT(false);
        }
        
        TEST_INFO("Test primary client's sending checkHostname command to the server");
        primaryClient->do_test(2);

        TEST_INFO("Waiting for result...");
        Cdmw::OsSupport::OS::sleep(2000);
        TEST_INFO("Check server's result");
        
        bCheck = server->check(clt1Hostname.in());
        
        if (true == bCheck) {
            TEST_INFO("hostnames are identical");
            CPPUNIT_ASSERT(true);
        } else {
            TEST_INFO("ERROR : hostname mismatch");
            CPPUNIT_ASSERT(false);
        }
        
    } catch (const CORBA::SystemException& e) {
        TEST_INFO("Unexpected CORBA::SystemException: " << e);
        CPPUNIT_ASSERT(false);
    } catch (const CORBA::Exception& e) {
        TEST_INFO("Unexpected CORBA::Exception: " << e);
        CPPUNIT_ASSERT(false);
    } catch (...) {
        TEST_INFO("Unknown exception !! ");
        CPPUNIT_ASSERT(false);
    }

}
