/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef ___CLIENTDANCE_IMPL_HPP__
#define ___CLIENTDANCE_IMPL_HPP__

#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/orbsupport/RefCountLocalObject.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/osthreads/ThreadHandle.hpp"

#include <User_ccm_dance2_with_ft.skel.hpp>


namespace Cdmw {
    namespace Logging {
        class Logger;
    }
}


namespace CcmDance2WithFTTest
{
class CCM_TestDriver_impl;

class test_ccm_dance2_with_ft : public CppUnit::TestFixture
{
    CcmDance2WithFTTest::DriverModule::CCM_TestDriver_Context_var m_session_context;
    CcmDance2WithFTTest::DriverModule::CCM_TestDriver_var m_testDriver;
    
    void do_tests( CcmDance2WithFTTest::TestFramework::TestableComponent_ptr server,
                   CcmDance2WithFTTest::TestFramework::TestableComponent_ptr primaryComponent,
                   CcmDance2WithFTTest::TestFramework::TestableComponent_ptr secondaryComponent);
    
public:
    test_ccm_dance2_with_ft()
    {
        CORBA::Object_var obj1 = 
            Cdmw::TestUtils::CORBATestManager::instance()->get_object("driver_context");
        m_session_context = 
            ::CcmDance2WithFTTest::DriverModule::CCM_TestDriver_Context::_narrow(obj1.in());
            
        CORBA::Object_var obj2 =
            Cdmw::TestUtils::CORBATestManager::instance()->get_object("ccm_testDriver");
        m_testDriver = 
            ::CcmDance2WithFTTest::DriverModule::CCM_TestDriver::_narrow(obj2.in());
    };
    
    ~test_ccm_dance2_with_ft() {};
    
    void do_tests_dance();
    
    CPPUNIT_TEST_SUITE( test_ccm_dance2_with_ft );
    CPPUNIT_TEST( do_tests_dance );
    CPPUNIT_TEST_SUITE_END();
    

};

class TestThread : public Cdmw::OsSupport::Thread 
{
    public:
        TestThread()
        {};
        
    protected:
        void run() throw()
        {    
            Cdmw::TestUtils::ThreadedTestRunner runner(Cdmw::TestUtils::CORBATestManager::instance()->get_controller());
            runner.addTest( CppUnit::TestFactoryRegistry::getRegistry("test_ccm_dance2_with_ft").makeTest() );
            runner.start();
            runner.join();
        };
        
    private:
        TestThread(const TestThread&);
        TestThread& operator = (const TestThread&);
   
};



//
// IDL:thalesgroup.com/UserCcmDance2WithFTTest/CCM_Client:1.0
//
class CCM_TestDriver_impl : virtual public ::UserCcmDance2WithFTTest::User_CCM_TestDriver,
                             virtual public Cdmw::OrbSupport::RefCountLocalObject
{
    CCM_TestDriver_impl(const CCM_TestDriver_impl&);
    void operator=(const CCM_TestDriver_impl&);

    ::CcmDance2WithFTTest::DriverModule::CCM_TestDriver_Context_var m_session_context;
    
    Cdmw::Logging::Logger* m_logger;
    
public:

    CCM_TestDriver_impl();
    ~CCM_TestDriver_impl();
     
    
    //
    // Session component specific operations
    //
    void set_session_context(Components::SessionContext_ptr ctx)
        throw(CORBA::SystemException);
    void ccm_activate()
        throw(CORBA::SystemException);
    void ccm_passivate()
        throw(CORBA::SystemException);
    void ccm_remove()
        throw(CORBA::SystemException);


    ::CcmDance2WithFTTest::DriverModule::CCM_TestDriver_Context* get_context()
    {
        return ::CcmDance2WithFTTest::DriverModule::CCM_TestDriver_Context::_duplicate(m_session_context.in());
    }


private:
    
    TestThread m_testThread;

};

}; // End of namespace CcmDance2WithFTTest


#endif // ___CLIENTDANCE_IMPL_HPP__

