/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include "Foundation/common/Assert.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/logging/Logger.hpp"

#include "Foundation/testutils/CORBATestManager.hpp"

#include <Command_impl.hpp>
#include <CCM_TestDriver_impl.hpp>



//
// IDL:thalesgroup.com/CcmDance2WithFTTest/DriverModule/CCM_Client:1.0
//
CcmDance2WithFTTest::CCM_TestDriver_impl::CCM_TestDriver_impl()
    : m_session_context(CcmDance2WithFTTest::DriverModule::CCM_TestDriver_Context::_nil())
{
    m_logger = Cdmw::Logging::Logger::Get_Logger("TestDriver");
}

CcmDance2WithFTTest::CCM_TestDriver_impl::~CCM_TestDriver_impl()
{
    CDMW_INFO(m_logger, "CCM_TestDriver_impl::~CCM_TestDriver_impl called!");
}



//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
CcmDance2WithFTTest::CCM_TestDriver_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_TestDriver_impl::Set_session_context called!");

   
    m_session_context = ::CcmDance2WithFTTest::DriverModule::CCM_TestDriver_Context::_narrow(ctx); 
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CcmDance2WithFTTest::CCM_TestDriver_impl::ccm_activate()
   throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_TestDriver_impl::Ccm_activate called!");
   
   CORBA::Object_var context = get_context();
   Cdmw::TestUtils::CORBATestManager::instance()->add("driver_context", context.in());
   Cdmw::TestUtils::CORBATestManager::instance()->add("ccm_testDriver", this);
   m_testThread.start();
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
CcmDance2WithFTTest::CCM_TestDriver_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_TestDriver_impl::Ccm_passivate called!");
} 



//
// ccm_remove function : it destroys the component (call by the container)
//
void
CcmDance2WithFTTest::CCM_TestDriver_impl::ccm_remove()
   throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_TestDriver_impl::Ccm_remove called!");
}



CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(CcmDance2WithFTTest::test_ccm_dance2_with_ft, "test_ccm_dance2_with_ft");


void
CcmDance2WithFTTest::test_ccm_dance2_with_ft::do_tests_dance()
{
    Cdmw::Logging::Logger* l_logger = Cdmw::Logging::Logger::Get_Logger("TestDriver_CppUnit");

    Cdmw::OsSupport::OS::sleep(2000);
    CDMW_INFO(l_logger, "Check environement.");

    if (CORBA::is_nil(m_session_context.in())) {
        CDMW_INFO(l_logger, "TestDriver context is NULL !!");
        CPPUNIT_ASSERT(false);
        return;
    }
    if (CORBA::is_nil(m_testDriver.in())) {
        CDMW_INFO(l_logger, "TestDriver component is NULL !!");
        CPPUNIT_ASSERT(false);
        return;
    }
    
    CDMW_INFO(l_logger, "Get the testable components.");
    
    CcmDance2WithFTTest::TestFramework::TestableComponent_var serverComponent; 
    CcmDance2WithFTTest::TestFramework::TestableComponent_var client1Component;
    CcmDance2WithFTTest::TestFramework::TestableComponent_var client2Component;
    
    try { 
        serverComponent = m_session_context->get_connection_server();
        client1Component = m_session_context->get_connection_client1();
        client2Component = m_session_context->get_connection_client2();
        
        if (CORBA::is_nil(serverComponent.in())) {
            CDMW_INFO(l_logger, "serverComponent is NIL !!!! ");
            CPPUNIT_ASSERT(false);
            return;
        }
        if (CORBA::is_nil(client1Component.in())) {
            CDMW_INFO(l_logger, "client1Component is NIL !!!! ");
            CPPUNIT_ASSERT(false);
            return;
        }
        if (CORBA::is_nil(client2Component.in())) {
            CDMW_INFO(l_logger, "client2Component is NIL !!!! ");
            CPPUNIT_ASSERT(false);
            return;
        }
    } catch (const CORBA::SystemException& e) {
        CDMW_INFO(l_logger, "Unexpected CORBA::SystemException: " << e);
        CPPUNIT_ASSERT(false);
    } catch (const CORBA::Exception& e) {
        CDMW_INFO(l_logger, "Unexpected CORBA::Exception: " << e);
        CPPUNIT_ASSERT(false);
    } catch (...) {
        CDMW_INFO(l_logger, "Unknown exception !! ");
        CPPUNIT_ASSERT(false);
    }


    do_tests(serverComponent.in(), client1Component.in(), client2Component.in());    
    CDMW_INFO(l_logger, "Demo test status is SUCCESS !");
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger("TestDriver"), "Demo test status is SUCCESS !");
}

void
CcmDance2WithFTTest::test_ccm_dance2_with_ft::do_tests(
    CcmDance2WithFTTest::TestFramework::TestableComponent_ptr server,
    CcmDance2WithFTTest::TestFramework::TestableComponent_ptr primaryClient,
    CcmDance2WithFTTest::TestFramework::TestableComponent_ptr secondaryClient)
{
    //input parameters are not supposed to be NULL 
    Cdmw::Logging::Logger* l_logger = Cdmw::Logging::Logger::Get_Logger("TestDriver_CppUnit");

    try {
        CDMW_INFO(l_logger, "Reseting components");
        server->reset();
        primaryClient->reset();
        secondaryClient->reset();
    
        CDMW_INFO(l_logger, "Perform primary server auto-test");
        server->do_test(0);
        // first server
        if (server->check("")) {
            CDMW_INFO(l_logger, "First server auto-test succeeded");
            CPPUNIT_ASSERT(true);
        } else {
            CDMW_INFO(l_logger, "First server auto-test failed");
            CPPUNIT_ASSERT(false);
        }

        //ask the active server to publish his hostname
        CDMW_INFO(l_logger, "Tell primary client to make the server publish his hostname");
        primaryClient->do_test(1);
        CDMW_INFO(l_logger, "Wait for publication...");
        Cdmw::OsSupport::OS::sleep(200);
        CDMW_INFO(l_logger, "And get the result");
        
        CORBA::String_var srvPublishedHostname1 = 
            primaryClient->get_property(CcmDance2WithFTTest::TestFramework::PROPERTY_GET_PUBLISHED_HOSTNAME);
        CDMW_INFO(l_logger, "primaryClient says server's published hostame is " << srvPublishedHostname1.in());
        
        CORBA::String_var srvHostname = 
            primaryClient->get_property(CcmDance2WithFTTest::TestFramework::PROPERTY_GET_SERVER_HOSTNAME);
        CDMW_INFO(l_logger, "primaryClient says server's hostame is " << srvHostname.in());
        
        if (strcmp(srvPublishedHostname1.in(), srvHostname.in()) == 0) {
            CDMW_INFO(l_logger, "hostnames are identical");
            CPPUNIT_ASSERT(true);
        } else {
            CDMW_INFO(l_logger, "ERROR : hostname mismatch");
            CPPUNIT_ASSERT(false);
        }

        
        CDMW_INFO(l_logger, "Kill primary server");
        primaryClient->do_test(3); // kill command
        Cdmw::OsSupport::OS::sleep(2000); //wait for d347h
        
        
        srvHostname = 
            primaryClient->get_property(CcmDance2WithFTTest::TestFramework::PROPERTY_GET_SERVER_HOSTNAME);
        CDMW_INFO(l_logger, "primaryClient says server's hostame is " << srvHostname.in());
        
        if (strcmp(srvPublishedHostname1.in(), srvHostname.in()) == 0) {
            CDMW_INFO(l_logger, "ERROR : hostnames are identical : server did not change");
            CPPUNIT_ASSERT(false);
        } else {
            CDMW_INFO(l_logger, "hostnames are different : server has changed");
            CPPUNIT_ASSERT(true);
        }
        
        CDMW_INFO(l_logger, "Perform backup server auto-test");
        server->do_test(0);
        // second server
        if (server->check("")) {
            CDMW_INFO(l_logger, "Second server auto-test succeeded");
            CPPUNIT_ASSERT(true);
        } else {
            CDMW_INFO(l_logger, "Second server auto-test failed");
            CPPUNIT_ASSERT(false);
        }
        server->reset(); 


        CDMW_INFO(l_logger, "Tell secondary client to make the server publish his hostname");
        secondaryClient->do_test(1);
        CDMW_INFO(l_logger, "Wait for publication...");
        Cdmw::OsSupport::OS::sleep(200);
        CDMW_INFO(l_logger, "And get the result");

        CORBA::String_var srvPublishedHostname2 = 
            secondaryClient->get_property(CcmDance2WithFTTest::TestFramework::PROPERTY_GET_PUBLISHED_HOSTNAME);
        CDMW_INFO(l_logger, "secondaryClient says server's published hostame is " << srvPublishedHostname2.in());

        srvHostname = 
            secondaryClient->get_property(CcmDance2WithFTTest::TestFramework::PROPERTY_GET_SERVER_HOSTNAME);
        CDMW_INFO(l_logger, "secondaryClient says server's hostame is " << srvHostname.in());

        if (strcmp(srvPublishedHostname2.in(), srvHostname.in()) == 0) {
            CDMW_INFO(l_logger, "hostnames are identical");
            CPPUNIT_ASSERT(true);
        } else {
            CDMW_INFO(l_logger, "ERROR : hostname mismatch");
            CPPUNIT_ASSERT(false);
        }


        CDMW_INFO(l_logger, "Test primary client sending checkHostname command to the server");
        primaryClient->do_test(2);
        CDMW_INFO(l_logger, "Waiting for result...");
        Cdmw::OsSupport::OS::sleep(2000);
        CDMW_INFO(l_logger, "Check server result");
        if (server->check("")) {
            CDMW_INFO(l_logger, "hostnames are identical");
            CPPUNIT_ASSERT(true);
        } else {
            CDMW_INFO(l_logger, "ERROR : hostname mismatch");
            CPPUNIT_ASSERT(false);
        }

    } catch (const CORBA::SystemException& e) {
        CDMW_INFO(l_logger, "Unexpected CORBA::SystemException: " << e);
        CPPUNIT_ASSERT(false);
    } catch (const CORBA::Exception& e) {
        CDMW_INFO(l_logger, "Unexpected CORBA::Exception: " << e);
        CPPUNIT_ASSERT(false);
    } catch (...) {
        CDMW_INFO(l_logger, "Unknown exception !! ");
        CPPUNIT_ASSERT(false);
    }

}
