/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef ___CLIENTDANCE_IMPL_HPP__
#define ___CLIENTDANCE_IMPL_HPP__

#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/orbsupport/RefCountLocalObject.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/osthreads/ThreadHandle.hpp"

#include <User_ccm_dance2_with_lb.skel.hpp>
#include <CCM_ClientHostInfo_impl.hpp>


namespace Cdmw {
    namespace Logging {
        class Logger;
    }
}


namespace CcmDance2WithLBTest
{

//
// IDL:thalesgroup.com/UserCcmDance2WithLBTest/CCM_Client:1.0
//
class CCM_ClientDance_impl : virtual public ::UserCcmDance2WithLBTest::User_CCM_ClientDance,
                             virtual public Cdmw::OrbSupport::RefCountLocalObject
{
    CCM_ClientDance_impl(const CCM_ClientDance_impl&);
    void operator=(const CCM_ClientDance_impl&);

    CORBA::String_var m_configuration;
    CORBA::String_var m_expectedServerHostname;
    CORBA::String_var m_hostnameCommandResult;

    ::CcmDance2WithLBTest::ClientModule::CCM_ClientDance_Context_var m_session_context;
    ::CcmDance2WithLBTest::FacetModule::CCM_ClientHostInfo_var m_hostInfo;

    Cdmw::Logging::Logger* m_logger;
    
public:

    CCM_ClientDance_impl();
    ~CCM_ClientDance_impl();
     

    //
    // attribute configuration
    //
    virtual char * configuration ()
        throw (CORBA::SystemException);
    virtual void configuration (const char * a)
        throw (CORBA::SystemException);

    //
    // attribute expectedServerHostname
    //
    virtual char * expectedServerHostname ()
        throw (CORBA::SystemException);
    virtual void expectedServerHostname (const char * a)
        throw (CORBA::SystemException);

        
    //
    // facet clientHostInfo
    //
    virtual ::CcmDance2WithLBTest::FacetModule::CCM_ClientHostInfo_ptr get_clientHostInfo ()
        throw (CORBA::SystemException);

    
    
    //
    // consumer hostnameResults
    //
    virtual void push_hostnameResults (CcmDance2WithLBTest::EventTypeModule::HostnameResult * event)
        throw (CORBA::SystemException);
     

    //
    // supported interface TestableComponent
    // 
    virtual char * get_property(const char * prop_name) 
        throw (CORBA::SystemException);
    virtual void reset() 
        throw (CORBA::SystemException);
    virtual CORBA::Boolean check(const char * ref) 
        throw (CORBA::SystemException);
    virtual void do_test(CORBA::Long idx) 
        throw (CORBA::SystemException);


    //
    // Session component specific operations
    //
    void set_session_context(Components::SessionContext_ptr ctx)
        throw(CORBA::SystemException);
    void ccm_activate()
        throw(CORBA::SystemException);
    void ccm_passivate()
        throw(CORBA::SystemException);
    void ccm_remove()
        throw(CORBA::SystemException);


    ::CcmDance2WithLBTest::ClientModule::CCM_ClientDance_Context* get_context()
    {
        return ::CcmDance2WithLBTest::ClientModule::CCM_ClientDance_Context::_duplicate(m_session_context.in());
    }

    //
    // get result of assynchronous getHostname command
    //
    char* getHostnameCommandResult();

private:
    

};

}; // End of namespace CcmDance2WithLBTest


#endif // ___CLIENTDANCE_IMPL_HPP__

