/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include "Foundation/common/Assert.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/logging/Logger.hpp"

#include "Foundation/testutils/CORBATestManager.hpp"

#include <Command_impl.hpp>
#include <CCM_TestDriver_impl.hpp>



//
// IDL:thalesgroup.com/CcmDance2WithLBTest/DriverModule/CCM_Client:1.0
//
CcmDance2WithLBTest::CCM_TestDriver_impl::CCM_TestDriver_impl()
    : m_session_context(CcmDance2WithLBTest::DriverModule::CCM_TestDriver_Context::_nil())
{
    m_logger = Cdmw::Logging::Logger::Get_Logger("TestDriver");
}

CcmDance2WithLBTest::CCM_TestDriver_impl::~CCM_TestDriver_impl()
{
    CDMW_INFO(m_logger, "CCM_TestDriver_impl::~CCM_TestDriver_impl called!");
}



//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
CcmDance2WithLBTest::CCM_TestDriver_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_TestDriver_impl::Set_session_context called!");

   
    m_session_context = ::CcmDance2WithLBTest::DriverModule::CCM_TestDriver_Context::_narrow(ctx); 
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CcmDance2WithLBTest::CCM_TestDriver_impl::ccm_activate()
   throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_TestDriver_impl::Ccm_activate called!");
   
   CORBA::Object_var context = get_context();
   Cdmw::TestUtils::CORBATestManager::instance()->add("driver_context", context.in());
   Cdmw::TestUtils::CORBATestManager::instance()->add("ccm_testDriver", this);
   m_testThread.start();
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
CcmDance2WithLBTest::CCM_TestDriver_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_TestDriver_impl::Ccm_passivate called!");
} 



//
// ccm_remove function : it destroys the component (call by the container)
//
void
CcmDance2WithLBTest::CCM_TestDriver_impl::ccm_remove()
   throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_TestDriver_impl::Ccm_remove called!");
}



CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(CcmDance2WithLBTest::test_ccm_dance2_with_lb, "test_ccm_dance2_with_lb");


void
CcmDance2WithLBTest::test_ccm_dance2_with_lb::do_tests_dance()
{
    Cdmw::Logging::Logger* l_logger = Cdmw::Logging::Logger::Get_Logger("TestDriver_CppUnit");
    Cdmw::OsSupport::OS::sleep(2000);
    CDMW_INFO(l_logger, "Check environement.");
    if (CORBA::is_nil(m_session_context.in())) {
        CDMW_INFO(l_logger, "TestDriver context is NULL !!");
        CPPUNIT_ASSERT(false);
        return;
    }
    if (CORBA::is_nil(m_testDriver.in())) {
        CDMW_INFO(l_logger, "TestDriver component is NULL !!");
        CPPUNIT_ASSERT(false);
        return;
    }
    
    CDMW_INFO(l_logger, "Get the testable components.");
    
    CcmDance2WithLBTest::TestFramework::TestableComponent_var serverComponent; 
    CcmDance2WithLBTest::TestFramework::TestableComponent_var client1Component;
    CcmDance2WithLBTest::TestFramework::TestableComponent_var client2Component;
    
    try { 
        serverComponent = m_session_context->get_connection_server();
        client1Component = m_session_context->get_connection_client1();
        client2Component = m_session_context->get_connection_client2();
        
        if (CORBA::is_nil(serverComponent.in())) {
            CDMW_INFO(l_logger, "serverComponent is NIL !!!! ");
            CPPUNIT_ASSERT(false);
            return;
        }
        if (CORBA::is_nil(client1Component.in())) {
            CDMW_INFO(l_logger, "client1Component is NIL !!!! ");
            CPPUNIT_ASSERT(false);
            return;
        }
        if (CORBA::is_nil(client2Component.in())) {
            CDMW_INFO(l_logger, "client2Component is NIL !!!! ");
            CPPUNIT_ASSERT(false);
            return;
        }
    } catch (const CORBA::SystemException& e) {
        CDMW_INFO(l_logger, "Unexpected CORBA::SystemException: " << e);
        CPPUNIT_ASSERT(false);
    } catch (const CORBA::Exception& e) {
        CDMW_INFO(l_logger, "Unexpected CORBA::Exception: " << e);
        CPPUNIT_ASSERT(false);
    } catch (...) {
        CDMW_INFO(l_logger, "Unknown exception !! ");
        CPPUNIT_ASSERT(false);
    }

    do_tests(serverComponent.in(), client1Component.in(), client2Component.in());    
    CDMW_INFO(l_logger, "Demo test status is SUCCESS !");
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger("TestDriver"), "Demo test status is SUCCESS !");
}

void
CcmDance2WithLBTest::test_ccm_dance2_with_lb::do_tests(
    CcmDance2WithLBTest::TestFramework::TestableComponent_ptr server,
    CcmDance2WithLBTest::TestFramework::TestableComponent_ptr primaryClient,
    CcmDance2WithLBTest::TestFramework::TestableComponent_ptr secondaryClient)
{
    //input parameters are not supposed to be NULL 
    Cdmw::Logging::Logger* l_logger = Cdmw::Logging::Logger::Get_Logger("TestDriver_CppUnit");

    try {
        CDMW_INFO(l_logger, "Reseting components");
        server->reset();
        primaryClient->reset();
        secondaryClient->reset();
    
        CDMW_INFO(l_logger, "Perform server auto-test");
        server->do_test(0); // performed by first load-balanced server
        server->do_test(0); // performed by second load-balanced server

        // first server
        if (server->check(""))
        {
            CDMW_INFO(l_logger, "Primary server auto-test succeeded");
            CPPUNIT_ASSERT(true);
        }
        else
        {
            CDMW_INFO(l_logger, "Primary server auto-test failed");
            CPPUNIT_ASSERT(false);
        }
        // second server
        if (server->check(""))
        {
            CDMW_INFO(l_logger, "Secondary server auto-test succeeded");
            CPPUNIT_ASSERT(true);
        }
        else
        {
            CDMW_INFO(l_logger, "Secondary server auto-test failed");
            CPPUNIT_ASSERT(false);
        }
        server->reset(); 
        server->reset();


        CDMW_INFO(l_logger, "Client 1 : first call to server");
        CORBA::String_var clt1Hostname = 
          primaryClient->get_property(CcmDance2WithLBTest::TestFramework::PROPERTY_GET_SERVER_HOSTNAME);
        CDMW_INFO(l_logger, "server hostame is " << clt1Hostname.in());
        
        CDMW_INFO(l_logger, "Client 1 : second call to server");
        CORBA::String_var srvHostname = 
            primaryClient->get_property(CcmDance2WithLBTest::TestFramework::PROPERTY_GET_SERVER_HOSTNAME);
        CDMW_INFO(l_logger, "server's hostame is " << srvHostname.in());
        
        if (strcmp(clt1Hostname.in(), srvHostname.in()) == 0)
        {
            CDMW_INFO(l_logger, "ERROR : server did not switch");
            CPPUNIT_ASSERT(false);
        }
        else
        {
            CDMW_INFO(l_logger, "server has changed");
            CPPUNIT_ASSERT(true);
        }

        CDMW_INFO(l_logger, "Test first client's sending getHostname command to the server");
        primaryClient->do_test(1);
        CDMW_INFO(l_logger, "Waiting for result...");
        Cdmw::OsSupport::OS::sleep(2000);
        CDMW_INFO(l_logger, "Check primaryClient's result");
        if (primaryClient->check(srvHostname.in()))
        {
            CDMW_INFO(l_logger, "ERROR : server did not switch");
            CPPUNIT_ASSERT(false);
        }
        else
        {
            CDMW_INFO(l_logger, "server has changed");
            CPPUNIT_ASSERT(true);
        }
        
        CDMW_INFO(l_logger, "Test first client's sending getHostname command to the server (again)");
        primaryClient->do_test(1);
        CDMW_INFO(l_logger, "Waiting for result...");
        Cdmw::OsSupport::OS::sleep(2000);
        CDMW_INFO(l_logger, "Check primaryClient's result");
        if (primaryClient->check(srvHostname.in()))
        {
            CDMW_INFO(l_logger, "server has changed");
            CPPUNIT_ASSERT(true);
        }
        else
        {
            CPPUNIT_ASSERT(false);
            CDMW_INFO(l_logger, "ERROR : server did not switch");
        }

        
        CDMW_INFO(l_logger, "Client 2 : first call to server" );
        CORBA::String_var clt2Hostname = 
            secondaryClient->get_property(CcmDance2WithLBTest::TestFramework::PROPERTY_GET_SERVER_HOSTNAME);
        CDMW_INFO(l_logger, "server hostame is " << clt2Hostname.in());

        CDMW_INFO(l_logger, "Client 2 : second call to server");
        srvHostname = 
            secondaryClient->get_property(CcmDance2WithLBTest::TestFramework::PROPERTY_GET_SERVER_HOSTNAME);
        CDMW_INFO(l_logger, "server's hostame is " << srvHostname.in());
        if (strcmp(clt2Hostname.in(), srvHostname.in()) == 0)
        {
            CDMW_INFO(l_logger, "ERROR : server did not switch");
            CPPUNIT_ASSERT(false);
        }
        else
        {
            CDMW_INFO(l_logger, "server has changed");
            CPPUNIT_ASSERT(true);
        }


        CDMW_INFO(l_logger, "Test second client's sending getHostname command to the server");
        CDMW_INFO(l_logger, "Check secondaryClient's result");
        secondaryClient->do_test(1);
        if (secondaryClient->check(srvHostname.in()))
        {
            CDMW_INFO(l_logger, "ERROR : server did not change");
            CPPUNIT_ASSERT(false);
        }
        else
        {
            CDMW_INFO(l_logger, "server has changed");
            CPPUNIT_ASSERT(true);
        }

        CDMW_INFO(l_logger, "Test second client's sending getHostname command to the server (again)");
        CDMW_INFO(l_logger, "Check secondaryClient's result");
        secondaryClient->do_test(1);
        if (secondaryClient->check(srvHostname.in()))
        {
            CDMW_INFO(l_logger, "server has changed");
            CPPUNIT_ASSERT(true);
        }
        else
        {
            CDMW_INFO(l_logger, "ERROR : server did not change");
            CPPUNIT_ASSERT(false);
        }

        CDMW_INFO(l_logger, "Test both first and second client sending checkHostname command to the server");
        primaryClient->do_test(2);
        secondaryClient->do_test(2);
        CDMW_INFO(l_logger, "Waiting for result...");
        Cdmw::OsSupport::OS::sleep(2000);
        CDMW_INFO(l_logger, "Check servers result");
        if (server->check("")) // first load-balanced server
        {
            CDMW_INFO(l_logger, "hostnames are identical");
            CPPUNIT_ASSERT(true);
        } 
        else
        {
            CDMW_INFO(l_logger, "ERROR : hostname mismatch");
            CPPUNIT_ASSERT(false);
        }
        if (server->check("")) // second load-balanced server
        {
            CDMW_INFO(l_logger, "hostnames are identical");
            CPPUNIT_ASSERT(true);
        }
        else
        {
            CDMW_INFO(l_logger, "ERROR : hostname mismatch");
            CPPUNIT_ASSERT(false);
        }
    }

    catch (const CORBA::SystemException& e)
    {
        CDMW_INFO(l_logger, "Unexpected CORBA::SystemException: " << e);
        CPPUNIT_ASSERT(false);
    }
    catch (const CORBA::Exception& e)
    {
        CDMW_INFO(l_logger, "Unexpected CORBA::Exception: " << e);
        CPPUNIT_ASSERT(false);
    }
    catch (...)
    {
        CDMW_INFO(l_logger, "Unknown exception !! ");
        CPPUNIT_ASSERT(false);
    }

}
