/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_DEMO_FT_GROUP_CREATOR_PROCESS_BEHAVIOUR_HPP 
#define INCL_DEMO_FT_GROUP_CREATOR_PROCESS_BEHAVIOUR_HPP 

// Standard Files 
#include <string>

// Cdmw Files
#include <Foundation/common/Options.hpp>
#include <Foundation/common/String.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <FaultTolerance/ftcommon/FTLocation.hpp>
#include <FaultTolerance/ftcommon/FactoryInfos.hpp>
#include <FaultTolerance/ftinit/FTServiceInit.hpp>
#include <SystemMngt/platforminterface/PlatformInterface.hpp>
#include <ConfAndPlug/cdmwinit/ProcessControl.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>
#include <Repository/idllib/CdmwNamingAndRepository.stub.hpp>

#include "FaultTolerance/idllib/FT.stub.hpp"
#include "TestHello.stub.hpp"


class GroupCreatorProcessBehaviour : public Cdmw::CdmwInit::ProcessControl
{

public:

    // Constructor
    GroupCreatorProcessBehaviour(CORBA::ORB_ptr orb, std::string host1, std::string host2)
    {
        m_orb = CORBA::ORB::_duplicate( orb );
        m_host_name1 = host1;
        m_host_name2 = host2;
    }

    // Destructor
    virtual ~GroupCreatorProcessBehaviour() throw ()
    {
    }

    
    virtual CORBA::ULong nb_steps()
    throw( CORBA::SystemException )
    {
        return 1;
    }

    
    virtual void on_initialise(
        const CdmwPlatformMngtBase::StartupKind& startup_kind )
    throw( CORBA::SystemException )
    {

        Cdmw::OsSupport::OS::sleep(3000);

        std::cout << "\n -------- GroupCreator on_initialise --------" << std::endl;

        // Get the Replication Manager
        CORBA::Object_var replication_mgr_obj
            = m_orb->resolve_initial_references("ReplicationManager");

        m_replication_mgr = FT::ReplicationManager::_narrow(
            replication_mgr_obj.in() );

        // Create the object group properties
        std::cout << "  -> Create group..." << std::endl;
        const char* rep_id_HelloInterface
            = "IDL:thalesgroup.com/DemoFT/HelloInterface:1.0";

        std::cout << "  -> Get hostname" << std::endl;

        ::FT::Properties prop;
        prop.length(2);
        prop[0].nam.length( 1 );
        prop[0].nam[0].id = "org.omg.ft.MinimumNumberReplicas";
        prop[0].val <<= (CORBA::UShort) 1;

        ::FT::Locations locs(2);
        locs.length(2);
        locs[0].length(3);
        locs[0][0].id = m_host_name1.c_str();
        locs[0][0].kind = "hostname";
        locs[0][1].id = "FTApplication";
        locs[0][1].kind = "applicationname";
        locs[0][2].id = "hello1";
        locs[0][2].kind = "processname";

        locs[1].length(3);
        locs[1][0].id = m_host_name2.c_str();
        locs[1][0].kind = "hostname";
        locs[1][1].id = "FTApplication";
        locs[1][1].kind = "applicationname";
        locs[1][2].id = "hello2";
        locs[1][2].kind = "processname";
      
        const CORBA::ULong factory_infos_len = locs.length();
        ::FT::FactoryInfos factoryInfos;
        factoryInfos.length(factory_infos_len);
        for (CORBA::ULong i = 0; i < factory_infos_len; ++i) 
          {
            factoryInfos[i].the_factory = ::FT::GenericFactory::_nil();
            factoryInfos[i].the_location = locs[i];
            ::FT::Criteria factoryCrit;        
            factoryCrit.length(0);
            factoryInfos[i].the_criteria = factoryCrit;
          }
      
        prop[1].nam.length(1);
        prop[1].nam[0].id="org.omg.ft.Factories";
        prop[1].val <<= factoryInfos;

        ::FT::Name OGDName;
        OGDName.length(1);
        OGDName[0].id=CORBA::string_dup("hello_group");
      

        ::FT::Criteria ft_criteria;
        ft_criteria.length(2);
        ft_criteria[0].nam.length(1);
        ft_criteria[0].nam[0].id="com.thalesgroup.cdmw.ft.Name";
        ft_criteria[0].val <<= OGDName;

        ft_criteria[1].nam.length(1);
        ft_criteria[1].nam[0].id="org.omg.ft.FTProperties";
        ft_criteria[1].val <<= prop;

      
        ::FT::GenericFactory::FactoryCreationId_var factory_creation_id;
        ::FT::ObjectGroup_var obj_group;
      
        std::cout << "  ->Create the object group" << std::endl;
        try
          {
          
            obj_group =  m_replication_mgr->create_object(
                                                          rep_id_HelloInterface, ft_criteria, factory_creation_id.out() );
          }
        catch( CORBA::Exception& e )
          {
            std::cerr << e._name() << std::endl;
            throw;
          }

        std::cout << "  ->Get Repository" << std::endl;
        CdmwNamingAndRepository::Repository_var repository
          = Cdmw::NamingAndRepository::RepositoryInterface::get_repository();

        std::cout << "  ->Get NameDomain to hello_servers (for writing)" << std::endl;
        CdmwNamingAndRepository::NameDomain_var helloDomain =
          repository->resolve_name_domain ("demo_ft/hello_servers");
                 
        std::cout << "  ->Get NamingInterface to hello_servers (for reading)" << std::endl;
        Cdmw::CommonSvcs::Naming::NamingInterface helloNamingInterface =
          Cdmw::NamingAndRepository::RepositoryInterface::get_domain_naming_interface ("demo_ft/hello_servers");
        std::string full_name = "hello1_object";
      
        CORBA::Object_var obj = helloNamingInterface.resolve (full_name);
        ::DemoFT::HelloInterface_var hello1 = ::DemoFT::HelloInterface::_narrow(obj.in());
    
        std::cout << "  ->Add hello1 to object group" << std::endl;
        ::FT::Location loc;
        loc.length(3);
        loc[0].id = m_host_name1.c_str();
        loc[0].kind = "hostname";
        loc[1].id = "FTApplication";
        loc[1].kind = "applicationname";        
        loc[2].id = "hello1";
        loc[2].kind = "processname";

        m_group = DemoFT::HelloInterface::_narrow( obj_group.in() );

        try {
          obj_group = m_replication_mgr->add_member(m_group.in(), loc, hello1.in());
        }
        catch (const ::FT::ObjectGroupNotFound&)
          {
            std::cerr << __FILE__ << "  " << __LINE__ << std::endl;
            std::cerr << "add_member hello1 => FT::ObjectGroupNotFound raised!" << std::endl;
            return;
          }
        catch (const ::FT::MemberAlreadyPresent&)
          {
            std::cerr << __FILE__ << "  " << __LINE__ << std::endl;
            std::cerr << "add_member hello1 => FT::MemberAlreadyPresent raised!" << std::endl;
            return;
          }
        catch (const ::FT::ObjectNotAdded&)
          {
            std::cerr << __FILE__ << "  " << __LINE__ << std::endl;
            std::cerr << "add_member hello1 => FT::ObjectNotAdded raised!" << std::endl;
            return;
          }

        std::cout << "  ->Add hello2 to object group" << std::endl;
        full_name = "hello2_object";

        obj = helloNamingInterface.resolve (full_name);
        ::DemoFT::HelloInterface_var hello2 = ::DemoFT::HelloInterface::_narrow(obj.in());

        loc[0].id = m_host_name2.c_str();
        loc[0].kind = "hostname";
        loc[1].id = "FTApplication";
        loc[1].kind = "applicationname";        
        loc[2].id = "hello2";
        loc[2].kind = "processname";

        try {
          obj_group = m_replication_mgr->add_member(m_group.in(), loc, hello2.in());
        }
        catch (const ::FT::ObjectGroupNotFound&)
          {
            std::cerr << __FILE__ << "  " << __LINE__ << std::endl;
            std::cerr << "add_member hello2 => FT::ObjectGroupNotFound raised!" << std::endl;
            return;
          }
        catch (const ::FT::MemberAlreadyPresent&)
          {
            std::cerr << __FILE__ << "  " << __LINE__ << std::endl;
            std::cerr << "add_member hello2 => FT::MemberAlreadyPresent raised!" << std::endl;
            return;
          }
        catch (const ::FT::ObjectNotAdded&)
          {
            std::cerr << __FILE__ << "  " << __LINE__ << std::endl;
            std::cerr << "add_member hello2 => FT::ObjectNotAdded raised!" << std::endl;
            return;
          }
        
//         m_group = DemoFT::HelloInterface::_narrow( obj_group.in() );

        std::cout << "  ->Register Hello ObjectGroup into repository" << std::endl;
        std::cout << "  -> Register object group into repository to object group" << std::endl;
        CdmwNamingAndRepository::NameDomain::RegistrationId_var regId =
          helloDomain->new_name ("hello_group");
        helloDomain->register_object (regId.in(), m_group.in ());
        
        std::cout << " -------- GroupCreator on_initialise END --------" << std::endl;
    }

    
    virtual void on_run()
    throw( CdmwPlatformMngt::ProcessDelegate::NotReadyToRun,
           CORBA::SystemException )
    {
        std::cout << "\n -------- GroupCreator on_run --------" << std::endl;
        // nothing to do
    }

    
    virtual void on_stop()
    throw( CORBA::SystemException )
    {
        std::cout << "\n -------- GroupCreator on_stop --------" << std::endl;
        // nothing to do : shutdown done in ProcessControl
    }

    
private:

    virtual void add_hello_member(std::string hello_name,
                                     std::string host_location,
                                  Cdmw::CommonSvcs::Naming::NamingInterface& namingInterface,
                                  bool set_as_primary)
    {
        // Retrieve reference of member to add from NamingInterface
        DemoFT::HelloInterface_var hello;
        try
        {
            std::string full_name = hello_name + "_object";
            std::cout << "  -> Retrieve server " << hello_name << std::endl;
            CORBA::Object_var obj = namingInterface.resolve (full_name.c_str());

            // check if this is a HelloInterface
            hello =  DemoFT::HelloInterface::_narrow( obj.in() );
         }
        catch(const CosNaming::NamingContext::NotFound& e)
        {
            std::cerr << __FILE__ << "  " << __LINE__ << std::endl;
            std::cerr << "Resolve " << hello_name << " => NamingContext::NotFound raised!" << std::endl;
            return;
        }
        catch(const CosNaming::NamingContext::CannotProceed& e)
        {
            std::cerr << __FILE__ << "  " << __LINE__ << std::endl;
            std::cerr << "Resolve " << hello_name << " => NamingContext::CannotProceed raised!" << std::endl;
            return;
        }
        catch(const CosNaming::NamingContext::InvalidName& e)
        {
            std::cerr << __FILE__ << "  " << __LINE__ << std::endl;
            std::cerr << "Resolve " << hello_name << " => NamingContext::InvalidName raised!" << std::endl;
            return;
        }
        catch (const CORBA::SystemException& e)
        {
            std::cerr << __FILE__ << "  " << __LINE__ << std::endl;
            std::cerr << "Resolve " << hello_name << " => CORBA::SystemException raised!" << std::endl;
            return;
        }
        
        // create Location for this member
        Cdmw::FT::FTLocation ftLoc(host_location.c_str(), "FTApplication", hello_name.c_str());
        
        // add this reference as a new member of the object group
        ::FT::ObjectGroup_var obj_group;
        try {
            std::cout << "  -> Add " << hello_name << " to object group" << std::endl;
            obj_group = m_replication_mgr->add_member(m_group.in(), ftLoc.get_FT_Location(), hello.in());
        }
        catch (const ::FT::ObjectGroupNotFound&)
        {
            std::cerr << __FILE__ << "  " << __LINE__ << std::endl;
            std::cerr << "add_member " << hello_name << " => FT::ObjectGroupNotFound raised!" << std::endl;
            return;
        }
        catch (const ::FT::MemberAlreadyPresent&)
        {
            std::cerr << __FILE__ << "  " << __LINE__ << std::endl;
            std::cerr << "add_member " << hello_name << " => FT::MemberAlreadyPresent raised!" << std::endl;
            return;
        }
        catch (const ::FT::ObjectNotAdded&)
        {
            std::cerr << __FILE__ << "  " << __LINE__ << std::endl;
            std::cerr << "add_member " << hello_name << " => FT::ObjectNotAdded raised!" << std::endl;
            return;
        }
                
        if (set_as_primary) {
            try {
                std::cout << "  -> Set " << hello_name << " as primary" << std::endl;
                obj_group = m_replication_mgr->set_primary_member( obj_group.in(), ftLoc.get_FT_Location() );
            } 
            catch (const ::FT::ObjectGroupNotFound& e)
            {
                std::cerr << __FILE__ << "  " << __LINE__ << std::endl;
                std::cerr << "set_primary_member => FT::ObjectGroupNotFound raised!" << std::endl;
            }
            catch (const ::FT::MemberNotFound& e)
            {
                std::cerr << __FILE__ << "  " << __LINE__ << std::endl;
                std::cerr << "set_primary_member => FT::MemberNotFound raised!" << std::endl;
            }
            catch (const ::FT::PrimaryNotSet& e)
            {
                std::cerr << __FILE__ << "  " << __LINE__ << std::endl;
                std::cerr << "set_primary_member => FT::MemberNotFound raised!" << std::endl;
            }
            catch (const ::FT::BadReplicationStyle& e)
            {
                std::cerr << __FILE__ << "  " << __LINE__ << std::endl;
                std::cerr << "set_primary_member => FT::BadReplicationStyle raised!" << std::endl;
            }
        }
                
        // update group reference
        m_group = DemoFT::HelloInterface::_narrow(
            obj_group.in() );
    }

    /**
     * The ORB.
     */
    CORBA::ORB_var m_orb;

    /**
     * The object group.
     */
    DemoFT::HelloInterface_var m_group;

    /**
     * The replication manager.
     */
    FT::ReplicationManager_var m_replication_mgr;

    std::string  m_host_name1;
    std::string  m_host_name2;

};


#endif // INCL_DEMO_FT_GROUP_CREATOR_PROCESS_BEHAVIOUR_HPP

