#!/bin/sh
# =====================================================================
# This file is part of CARDAMOM (R) which is jointly developed by THALES
# and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
# Copyright (C) THALES 2000-2003. All rights reserved.
# Copyright (C) THALES 2004-2005. All rights reserved.
# Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
# All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
# License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

if test "`echo -e xxx`" = "xxx"
then
    echo="echo -e"
else
    echo=echo
fi

SITE_CONFIG=../site.mk

if test ! -f "$SITE_CONFIG"
then
    $echo "$SITE_CONFIG file is missing. Run '../configure.sh' and restart $0 "
    exit 1
fi

# CDMW installation folder
CDMW_HOME="`cat $SITE_CONFIG | grep CDMW_HOME |cut -d= -f2`"
LD_LIBRARY_PATH=$CDMW_HOME/lib/c++:$LD_LIBRARY_PATH
export LD_LIBRARY_PATH
PATH=$CDMW_HOME/bin:$PATH
export PATH

# "hostname" used for the FaultManagerRegistration because i've heard
# of problems with "localhostname"
HOSTNAME=`hostname`
# Check HOSTNAME2 is set
if [ "$HOSTNAME2" = "" ]
then
    $echo "Export HOSTNAME2 to set the other host used by this demo. Then, run 'make data' in parent directory restart $0"
     rm ../data/Scenario.xml
    exit 1
fi

# XML scenario file
SCENARIO_FILE=../data/Scenario.xml

if test ! -f "$SCENARIO_FILE"
then
    $echo "$SCENARIO_FILE file is missing. Run 'make data' in parent directory restart $0 "
    exit 1
fi

# XML Platform Management Daemon configuration file
DAEMON_CONF_FILE=$CDMW_HOME/share/CdmwPlatformMngtDaemon_conf.xml

if test ! -f "$DAEMON_CONF_FILE"
then
    $echo "$DAEMON_CONF_FILE file is missing. Run 'make data' in parent directory restart $0 "
    exit 1
fi

if test ! -x `which $TERM`
then
    TERM=xterm
fi

TERM_PATH=`which $TERM`

INIT_TIMEOUT=10
FT_MANAGER_TIMEOUT=10
EXEC_TIMEOUT=40
CLEANUP_TIMEOUT=10

$echo "========================================================="
$echo "=                       CDMW DEMO FT                    ="
$echo "========================================================="
$echo "Hit a key when ready."
read FOO
echo "==========================================================="

# 0) Install exe on HOSNAME2
BUILD_TARGET=`$CDMW_HOME/bin/config.guess`
DEMO_ALREADY_INSTALLED=`ssh -X $HOSTNAME2 "ls $CDMW_HOME/demos/demo_ft/c++/$BUILD_TARGET 2>/dev/null"` 

if [ "$DEMO_ALREADY_INSTALLED" = "" ]
then
   $echo Installing demo on $HOSTNAME2
   ssh -X $HOSTNAME2 "mkdir -p $CDMW_HOME/demos/demo_ft/c++"
    $echo "  => Copying $CDMW_HOME/demos/demo_ft/c++/site.mk"
   scp $CDMW_HOME/demos/demo_ft/c++/site.mk $HOSTNAME2:$CDMW_HOME/demos/demo_ft/c++
   ssh -X $HOSTNAME2 "mkdir $CDMW_HOME/demos/demo_ft/c++/generated"
   scp $CDMW_HOME/demos/demo_ft/c++/generated/Server_Desc.dtd $HOSTNAME2:$CDMW_HOME/demos/demo_ft/c++/generated
    $echo "  => Copying $CDMW_HOME/demos/demo_ft/c++/data"
   scp -r $CDMW_HOME/demos/demo_ft/c++/data $HOSTNAME2:$CDMW_HOME/demos/demo_ft/c++
    $echo "  => Copying $CDMW_HOME/demos/demo_ft/c++/run"
   scp -r $CDMW_HOME/demos/demo_ft/c++/run $HOSTNAME2:$CDMW_HOME/demos/demo_ft/c++

   ssh -X $HOSTNAME2 "mkdir -p $CDMW_HOME/demos/demo_ft/c++/$BUILD_TARGET"
    $echo "  => Copying $CDMW_HOME/demos/demo_ft/c++/$BUILD_TARGET"
   scp -r $CDMW_HOME/demos/demo_ft/c++/$BUILD_TARGET $HOSTNAME2:$CDMW_HOME/demos/demo_ft/c++
   if [ -d  $CDMW_HOME/../tools ]
   then
    $echo "  => Copying $CDMW_HOME/../tools"
    scp -r $CDMW_HOME/../tools $HOSTNAME2:$CDMW_HOME/..
   fi
else
   $echo Demo already installed on $HOSTNAME2
fi

# 1) Start daemon 
$echo Starting the Platform Management Daemon...
DAEMON_NAME=CdmwPlatformDaemon_on_$HOSTNAME
echo "#!/bin/sh" > start_demo_daemon.sh
echo "export PATH=$PATH" >> start_demo_daemon.sh
echo "export LD_LIBRARY_PATH=$LD_LIBRARY_PATH" >> start_demo_daemon.sh
echo "cd $CDMW_HOME/share" >> start_demo_daemon.sh
# If a daemon was badly stopped, kill it
echo "if [ -d /tmp/CdmwDaemon_21869 ]; then" >> start_demo_daemon.sh
echo "$CDMW_HOME/bin/cdmw_platform_kill_all_pids.sh /tmp/CdmwDaemon_21869" >> start_demo_daemon.sh
echo "fi" >> start_demo_daemon.sh
echo "$CDMW_HOME/demos/demo_ft/c++/data/start_daemon_verbosely.sh" >> start_demo_daemon.sh
chmod +x start_demo_daemon.sh

#redhat RHEL4 U2 bug 180353 fixed by the following variable
# echo "#!/bin/sh" > cdmw_shell_forward.sh
# echo "$*" >> cdmw_shell_forward.sh
# chmod +x cdmw_shell_forward.sh

DAEMON_COMMAND="start_demo_daemon.sh"
ssh -X $HOSTNAME "(LD_LIBRARY_PATH=$LD_LIBRARY_PATH; export LD_LIBRARY_PATH; PATH=$PATH; export PATH; export SHELL_FORWARD=$CDMW_HOME/demos/demo_ft/c++/run/cdmw_shell_forward.sh; cd $PWD; $TERM_PATH -ls -sb -sl 5000 -T $DAEMON_NAME -e start_demo_daemon.sh)" &
DAEMON_HOST1_PID=$!

DAEMON_COMMAND2=$DAEMON_COMMAND

sleep 2
DAEMON_NAME2=CdmwPlatformDaemon_on_$HOSTNAME2

ssh -X $HOSTNAME2 "(cd $CDMW_HOME/demos/demo_ft/c++/run; echo "LD_LIBRARY_PATH=$LD_LIBRARY_PATH" > start_demo_daemon.sh; echo "export LD_LIBRARY_PATH" >> start_demo_daemon.sh; echo "PATH=$PATH" >> start_demo_daemon.sh; echo "export PATH" >> start_demo_daemon.sh; echo "$CDMW_HOME/demos/demo_ft/c++/data/start_daemon_verbosely.sh" >> start_demo_daemon.sh; chmod +x start_demo_daemon.sh; $TERM_PATH -ls -sb -sl 5000 -T $DAEMON_NAME2 -e $CDMW_HOME/demos/demo_ft/c++/run/start_demo_daemon.sh)" &

# trap '$DAEMON_COMMAND stop; exit' 2
# ssh -X $HOSTNAME2 trap '$DAEMON_COMMAND2 stop; exit' 2

sleep $INIT_TIMEOUT

# 1a) Start FT Manager
if test -x "$CDMW_HOME/bin/cdmw_ft_manager"
then
    $echo Starting the FT Manager...
    LOG_NAME=FT_Manager
    FT_MANAGER_COMMAND="$CDMW_HOME/bin/cdmw_ft_manager --CdmwXMLFile=$CDMW_HOME/share/CdmwFaultToleranceManager_conf.xml --groupConf=../data/CdmwFTSystemMngtGroupCreator_conf.xml"
    $TERM -ls -sb -sl 5000 -T $LOG_NAME -e "$FT_MANAGER_COMMAND"&
    FT_MANAGER_PID=$!
    sleep $FT_MANAGER_TIMEOUT
fi

# 3) Start Platform Management Supervision
$echo Starting the Platform Management Supervision...
SUP_LOG_NAME=CdmwSupervisionDaemon

echo "export PATH=$PATH" > start_demo_supervision.sh
echo "export LD_LIBRARY_PATH=$LD_LIBRARY_PATH" >> start_demo_supervision.sh 
echo "$CDMW_HOME/demos/demo_ft/c++/data/start_supervision.sh" >> start_demo_supervision.sh
chmod +x start_demo_supervision.sh 

#redhat RHEL4 U2 bug 180353 fixed by the following variable
# scp $CDMW_HOME/demos/demo_ft/c++/run/cdmw_shell_forward.sh $HOSTNAME2:$CDMW_HOME/demos/demo_ft/c++/run

# ssh -X $HOSTNAME "(LD_LIBRARY_PATH=$LD_LIBRARY_PATH; export LD_LIBRARY_PATH; PATH=$PATH; export PATH; cd $PWD; $TERM_PATH -ls -sb -sl 5000 -T $SUP_LOG_NAME -e start_demo_supervision.sh)" &
$TERM -ls -sb -sl 5000 -T $SUP_LOG_NAME -e start_demo_supervision.sh  &


SUPERVISION_PID=$!

sleep 10

# Retain the PID of the cdmw_platform_supervision command
# SUPERVISION_PID=$(ps -ef | grep "exec cdmw_platform_supervision" | awk '{print $2}')

# 3) Define system
echo -e "Defining the System with $SCENARIO_FILE"
$CDMW_HOME/bin/cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --sys-define $SCENARIO_FILE

# 4) Start_system
$CDMW_HOME/bin/cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --sys-start
if [ $? -eq "0" ];
then
    $echo "Waiting a while for the execution of the scenario...\c"
    sleep $EXEC_TIMEOUT
    $echo done.
fi

# 5) Get a snapshot of the system
$CDMW_HOME/bin/cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --sys-snapshot

$echo "=============================================================="
$echo "Hit a key when ready to stop the System.                      "
read FOO
$echo "=============================================================="

# 6) stop_system
$CDMW_HOME/bin/cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --sys-stop
sleep $CLEANUP_TIMEOUT

$echo "=============================================================="
$echo "Hit a key when ready to stop the CDMW Daemon and Supervision."
read FOO
$echo "=============================================================="

$CDMW_HOME/bin/cdmw_platform_daemon.sh --CdmwXMLFile=$DAEMON_CONF_FILE stop
ssh -X $HOSTNAME2 "$CDMW_HOME/bin/cdmw_platform_daemon.sh --CdmwXMLFile=$DAEMON_CONF_FILE stop"
pkill cdmw_platform_supervision
pkill ft_manager
pkill start_supervision.sh
rm -f *.ior
rm -f *.IOR
rm -f start_demo_daemon.sh
rm -f start_demo_supervision.sh
rm -f *.output
ssh $HOSTNAME2 "(cd $CDMW_HOME/share); rm -f start_demo_daemon.sh"
ssh $HOSTNAME2 "(cd $CDMW_HOME/share); rm -f start_demo_supervision.sh"
ssh $HOSTNAME2 "(cd $CDMW_HOME/demos/demo_ft/c++/run); rm -f *.output"
kill -9 $DAEMON_HOST1_PID
kill -9 $SUPERVISION_PID
$echo "done."
