/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_DATABASE_IMPL_HPP
#define INCL_DATABASE_IMPL_HPP


// Cdmw Files
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountServantBase.hpp>
#include <FaultTolerance/ftstatemanager/DataStore.hpp>
#include <Foundation/commonsvcs/datastore/StorageHome.hpp>
#include <Foundation/commonsvcs/datastore/StorageObject.hpp>

// Demo Files
#include <LogCollector.skel.hpp>


// Define data_traits for fixed struct Demo::DataPart1 and Demo::DataPart2
// This is required to use a DataStore with those struct
DEFINE_VARIABLE_LENGTH_DATA_TRAITS(Demo::LogData);


class LogCollector_impl : public virtual POA_Demo::LogCollector,
                          public virtual Cdmw::OrbSupport::RefCountServantBase
{

public:

    // Constructor
    LogCollector_impl(CORBA::ORB_ptr orb,
                      PortableServer::POA_ptr parentPOA);

    // Destructor
    virtual ~LogCollector_impl()
        throw();

    // returns the POA of this servant
    virtual PortableServer::POA_ptr _default_POA();
        
        
    // IDL:thalesgroup.com/Demo/LogCollector/insert_log()
    virtual void insert_log (const Demo::LogData& data)
        throw( CORBA::SystemException );

    // IDL:thalesgroup.com/Demo/LogCollector/update_log()
    virtual void update_log (const Demo::LogData& data)
        throw( CORBA::SystemException );
        
private:

    // Hide copy constructor/assignment operator
    LogCollector_impl(
        const LogCollector_impl& rhs )
            throw();

    LogCollector_impl& operator =(
        const LogCollector_impl& rhs )
            throw();

    // Definition of Storage objects for DataPart1 type 
    typedef Cdmw::CommonSvcs::DataStore::StorageHome< Demo::Ident, Demo::LogData > LogsStorageHome;    
    typedef Cdmw::CommonSvcs::DataStore::StorageObject< Demo::Ident, Demo::LogData > LogsStorageObject;
    typedef Cdmw::FT::DataStore< Demo::Ident, Demo::LogData > LogsDataStore;
            
    // StorageHome for LogData
    LogsStorageHome m_LogsStorageHome;
    
    // orb
    CORBA::ORB_var m_orb;

    // servant's POA
    PortableServer::POA_var m_poa;

    
}; // End class Database_impl 



#endif // INCL_DATABASE_IMPL_HPP


