/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Cdmw Files
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>
#include <FaultTolerance/ftcommon/FTObjectIds.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>
#include <FaultTolerance/idllib/CdmwFTReplicationManager.stub.hpp>
#include <Foundation/common/Assert.hpp>

// Demo Files
#include "ClientProcessBehaviour.hpp"
#include "Common.hpp"

#include <iostream>


ClientProcessBehaviour::ClientProcessBehaviour(
    CORBA::ORB_ptr orb,
    PortableServer::POA_ptr rootPOA)
{
    m_orb = CORBA::ORB::_duplicate( orb );
    m_client_thread = NULL;
}

    
ClientProcessBehaviour::~ClientProcessBehaviour() throw ()
{
}

    
CORBA::ULong ClientProcessBehaviour::nb_steps()
    throw( CORBA::SystemException )
{
    return 1;
}

    
void ClientProcessBehaviour::on_initialise(
    const CdmwPlatformMngtBase::StartupKind& startup_kind )
throw( CORBA::SystemException )
{
   ECHO ("Client initialisation ...");

   CORBA::Object_var obj =
      m_orb->resolve_initial_references(Cdmw::FT::REPLICATION_MANAGER_ObjectId);
   CDMW_ASSERT(!CORBA::is_nil(obj.in()));

   CdmwFT::ReplicationManager_var rm =  CdmwFT::ReplicationManager::_narrow(obj.in());

   CosNaming::Name_var name = 
      Cdmw::CommonSvcs::Naming::NamingInterface::to_name(Demo::LOG_COLLECTOR_GROUP_NAME);
   FT::ObjectGroup_var grp = rm->get_object_group_ref_from_name(name.in());
   m_group =  Demo::LogCollector::_narrow(grp.in());
   CDMW_ASSERT(!CORBA::is_nil(m_group.in()));
   ECHO ("Client initialisation ended.");
}
        
        
void ClientProcessBehaviour::on_next_step()
    throw( CdmwPlatformMngt::ProcessDelegate::InvalidStep,
           CORBA::SystemException )
{
}

    
void ClientProcessBehaviour::on_run()
    throw( CdmwPlatformMngt::ProcessDelegate::NotReadyToRun,
           CORBA::SystemException )
{
   ECHO ("Client is runing...");


   unsigned int nbrecords = Demo::DEFAULT_NUMBER_OF_RECORDS;
   {
      int prop_val = Cdmw::OsSupport::SystemProperties::Get_Property(
         Demo::CLIENT_NUMBER_OF_RECORDS_PROPERTY_NAME,
         0);
      if (prop_val > 0) {
         nbrecords = prop_val;
      }
   }
  
   unsigned int sleeptime = 0;
   {
      int prop_val = Cdmw::OsSupport::SystemProperties::Get_Property
         (Demo::CLIENT_SLEEP_TIME_PROPERTY_NAME,
          0);
      if (prop_val > 0) {
         sleeptime = prop_val;
      }
   }

   unsigned short nbthreads = Demo::DEFAULT_NB_CLIENT_SCHEDULER_THREADS;
   {
      int prop_val = Cdmw::OsSupport::SystemProperties::Get_Property
         (Demo::CLIENT_NUMBER_OF_SCHEDULER_THREADS_PROPERTY_NAME,
          0);
      if (prop_val > 0) {
         nbthreads = prop_val;
      }
   }
   
   ECHO ("\nCurrent Configuration :\n"
         << "\t - Use of " << nbrecords << " records,\n"
         << "\t - Use of " << sleeptime << " ms between " << nbrecords << " updates, and"
         << "\t - Use of " << nbthreads << " client threads.");
   // Launch the test in a thread
   m_client_thread = new ClientThread(m_group.in(),
                                      nbthreads,
                                      nbrecords,
                                      sleeptime);    

   m_client_thread->start();
}

    
void ClientProcessBehaviour::on_stop()
    throw( CORBA::SystemException )
{
   ECHO ("Client is stopping...");
        
    m_client_thread->stop();
    m_client_thread->join();
        
    delete m_client_thread;

   ECHO ("Client stopped.");

}


