/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <ClientThread.hpp>

#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/ossupport/OS.hpp>
#include "Common.hpp"
#include "LogData.stub.hpp"
#include "LogCollector.stub.hpp"

class InsertRequest : public Cdmw::OsSupport::Runnable
{
   public:
      InsertRequest(Demo::LogCollector_ptr collector,
                    const Demo::LogData& data)
         : m_collector(Demo::LogCollector::_duplicate(collector)),
           m_data(new Demo::LogData(data))
      {}
      virtual void run() throw()
      {
         try {
            ECHO ("Inserting Log data [" << m_data->id << " / " << m_data->logs.version << "] ...");
            m_collector->insert_log(m_data.in());
         } catch (const CORBA::Exception & ex) {
            ECHO("Caught CORBA Exception for log [" << m_data->id << "] :" 
                 << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex));
         }
      }
      ~InsertRequest() throw() {}
   private:
      Demo::LogCollector_var m_collector;
      Demo::LogData_var m_data;
};


class UpdateRequest : public Cdmw::OsSupport::Runnable
{
   public:
      UpdateRequest(Demo::LogCollector_ptr collector,
                    const Demo::LogData& data)
         : m_collector(Demo::LogCollector::_duplicate(collector)),
           m_data(new Demo::LogData(data))
      {}
      virtual void run() throw()
      {
         try {
            ECHO ("Updating Log data [" << m_data->id << " / " << m_data->logs.version << "] ...");
            m_collector->update_log(m_data.in());
         } catch (const CORBA::Exception & ex) {
            ECHO("Caught CORBA Exception for log [" << m_data->id << "] :" 
                 << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex));
         }
      }
      ~UpdateRequest() throw() {}
   private:
      Demo::LogCollector_var m_collector;
      Demo::LogData_var m_data;
};


void ClientThread::run() throw()
{
   try {
      ECHO ("Starting tests ...");
      m_scheduler.start();

      Demo::Ident id = 0;
      CORBA::Long version = 0;

      std::string msg1("Hello world 1!");
      std::string msg2("Hello world 2!");
      std::string msg3("Hello world 3!");   

      // Create m_nb_records logs
      for (Demo::Ident i = 0; i < m_nb_records; ++i)
      {
         Demo::LogData_var data = new Demo::LogData;
         data->id = id++;
         data->logs.version = version++;
         data->logs.messages.length(3L);
         data->logs.messages[0L]=CORBA::string_dup(msg1.c_str());
         data->logs.messages[1L]=CORBA::string_dup(msg2.c_str());
         data->logs.messages[2L]=CORBA::string_dup(msg3.c_str());
         ECHO ("Enqueue Insert Request [" << data->id << " / " << data->logs.version << "] ...");
         m_scheduler.enqueue(new InsertRequest(m_collector.in(),
                                               data.in()));
      }

      id = 0;

      while (!m_stopped) 
      {
         if (m_sleep_time > 0)
            Cdmw::OsSupport::OS::sleep (m_sleep_time);
         Demo::LogData_var data = new Demo::LogData;
         data->id = id++;
         data->logs.version = version++;
         data->logs.messages.length(3L);
         data->logs.messages[0L]=CORBA::string_dup(msg1.c_str());
         data->logs.messages[1L]=CORBA::string_dup(msg2.c_str());
         data->logs.messages[2L]=CORBA::string_dup(msg3.c_str());
         ECHO ("Enqueue Update Request [" << data->id << " / " << data->logs.version << "] ...");
         m_scheduler.enqueue(new UpdateRequest(m_collector.in(),
                                               data.in()));
         if (id == m_nb_records) id = 0;
      }
      ECHO ("Tests ended.");
      m_scheduler.stop(true);
   } catch (const std::exception & e) {
      ECHO ("Caught exception : " << e.what() << ". Exiting thread.");
   }
}   




