/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Demo Files
#include "LogCollector_impl.hpp"
#include "Common.hpp"


LogCollector_impl::LogCollector_impl(CORBA::ORB_ptr orb,
                                     PortableServer::POA_ptr poa)
   :  m_LogsStorageHome(Demo::DATA_STORE_IDENTIFIER),
      m_orb(CORBA::ORB::_duplicate(orb)),
      m_poa(PortableServer::POA::_duplicate(poa))
{
}


LogCollector_impl::~LogCollector_impl()
throw()
{
}


PortableServer::POA_ptr LogCollector_impl::_default_POA()
{
    return PortableServer::POA::_duplicate(m_poa.in());
}




void LogCollector_impl::insert_log (const Demo::LogData& data)
    throw ( CORBA::SystemException )
{
   ECHO("Inserting log [" << data.id << " / " << data.logs.version << "] ...");

   Demo::Ident id = data.id;
    
   try {
      // part1 not found in DataStore. Create a new StorageObject.
      LogsStorageObject obj = m_LogsStorageHome.create(id, data);
   } catch (const std::exception & e ) {
      ECHO("Caught exception during insertion of log ["
           << id << " / " << data.logs.version << "] : "
           << e.what());
   }
}

void LogCollector_impl::update_log (const Demo::LogData& data)
    throw ( CORBA::SystemException )
{
   ECHO("Updating log [" << data.id << " / " << data.logs.version << "] ...");

   Demo::Ident id = data.id;
    
   try {
      // find if it is already in data store
      LogsStorageObject obj = m_LogsStorageHome.find_by_oid(id);
      obj.set(data);
   } catch (const std::exception & e ) {
      ECHO("Caught exception during update of log ["
           << data.id << " / " << data.logs.version << "] : "
           << e.what());
   }
}



