/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Cdmw Files
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <FaultTolerance/ftinit/FTServiceInit.hpp>
#include <ConfAndPlug/cdmwinit/CdmwInterface.hpp>
#include <FaultTolerance/idllib/CdmwFTReplicationManager.stub.hpp>
#include <FaultTolerance/idllib/CdmwFTGroupRepository.stub.hpp>
#include <FaultTolerance/ftcommon/FTObjectIds.hpp>
#include <Foundation/common/Assert.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Foundation/logging/LogManager.hpp>

// Demo Files
// #include <DatabaseProcessBehaviour.hpp>
// #include <StateTransferConfiguration.hpp>
#include "Common.hpp"
#include "LogCollector_impl.hpp"


namespace
{    
   const int SUCCESS = 0;
   const int FAILURE = 1;
   const int POA_THREAD_POOL_SIZE = 15;
}; // End anonymous namespace

namespace Demo
{
   Cdmw::Logging::Logger *
   GetLogger()
   {
      static Cdmw::Logging::Logger * logger__ = 
         Cdmw::Logging::Logger::Get_Logger(Demo::SERVER_LOG_DOMAINE);
      return logger__;
   }
}

int main( int argc, char* argv[] )
{
    int status = SUCCESS;

    CORBA::ORB_var orb;   
   
    try {
       (void*)Cdmw::Logging::LogManager::Init(argc,argv);
       ECHO("Initialising FT service ...");
        // Initialise FT service
        Cdmw::CdmwInit::CDMW_preORBInit(argc, argv);

        // Initialize the ORB
        Cdmw::OrbSupport::StrategyList orb_strategies;
        orb_strategies.add_OrbThreaded();
        orb_strategies.add_PoaThreadPool(POA_THREAD_POOL_SIZE);
        ECHO("ORB_init");
        orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, orb_strategies);

        //
        // Get the root POA 
        //
        CORBA::Object_var obj = orb -> resolve_initial_references("RootPOA");
        PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj.in());

        ECHO("poaManager->activate ");
        //
        // Run implementation (Activate the POA manager)
        //
        PortableServer::POAManager_var poaManager = rootPOA -> the_POAManager();
        poaManager -> activate();
        ECHO ("CDMW_init ");
        Cdmw::CdmwInit::CDMW_init(orb.in(), argc, argv);

        ECHO ("Create Server object and join object group");
        {
           CORBA::Object_var obj =
              orb->resolve_initial_references(Cdmw::FT::REPLICATION_MANAGER_ObjectId);
           CDMW_ASSERT(!CORBA::is_nil(obj.in()));
           CdmwFT::ReplicationManager_var rm =  CdmwFT::ReplicationManager::_narrow(obj.in());
           CosNaming::Name_var name = 
              Cdmw::CommonSvcs::Naming::NamingInterface::to_name(Demo::LOG_COLLECTOR_GROUP_NAME);
           FT::ObjectGroup_var grp = rm->get_object_group_ref_from_name(name.in());
           CDMW_ASSERT(!CORBA::is_nil(grp.in()));

           LogCollector_impl * collector_impl = new  LogCollector_impl(orb.in(),
                                                                       rootPOA.in());
           
           Demo::LogCollector_var ref = collector_impl->_this();
           ECHO ("Joining " << Demo::LOG_COLLECTOR_GROUP_NAME << " object group...");
           try {
              obj = orb->resolve_initial_references(Cdmw::FT::FT_GROUP_REPOSITORY_ObjectId);
              CdmwFT::Location::GroupRepository_var grp_repository =
                 CdmwFT::Location::GroupRepository::_narrow(obj.in());
              FT::Location_var location = grp_repository->the_location();

              grp = rm->add_member(grp.in(),
                                   location.in(),
                                   ref.in());
              ECHO ("Joined " << Demo::LOG_COLLECTOR_GROUP_NAME << ".");
           } catch (const CORBA::Exception & ex) {
              ECHO ("Failed to join object group " << Demo::LOG_COLLECTOR_GROUP_NAME 
                    << ". Caught excetion :"
                    << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex));
              throw;
           }
           // Join the group
        }

        ECHO ("ORB->run ");
        orb->run();
    } catch (const Cdmw::Exception & ex) {
       ECHO(ex.what());
        status = FAILURE;
    } catch(const CORBA::Exception& ex)  {  
       ECHO("Caught CORBA Exception " << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex));      
        status = FAILURE;
    }

    if(!CORBA::is_nil(orb.in()))
    {
        try {
            Cdmw::CdmwInit::CDMW_cleanup(orb.in());
            Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
            orb->destroy();
        }
        catch(const CORBA::Exception& ex)
        {           
           ECHO("Caught CORBA Exception " << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex)); 
            status = FAILURE;
        }
    }        

    return status;

}


