#!/bin/sh
# =====================================================================
# This file is part of CARDAMOM (R) which is jointly developed by THALES
# and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
# Copyright (C) THALES 2000-2003. All rights reserved.
# Copyright (C) THALES 2004-2005. All rights reserved.
# Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
# All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
# License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================


if test "`echo -e xxx`" = "xxx"
then
    echo="echo -e"
else
    echo="echo"
fi
echo="echo"

HOSTNAMES_FILE="./Hostnames"

NB_THREADS_MIN=3
FIRST_SERVER=2

PROTOCOLS="IIOP MIOP"
#PROTOCOLS="MIOP"

PROC_NAME=`basename $0`

#$echo " Usage: ./$PROC_NAME [--hosts <hostnames_file_path>] [--nbthreads <minimum_threads_number>]";\
#$echo "   ./$PROC_NAME                       : Configure State Transfer Perf with default hostnames file ($HOSTNAMES_FILE) and default minimum number of threads ($NB_THREADS_MIN).";\
#$echo "   ./$PROC_NAME --hosts <hostnames_file_path> : Configure State Transfer Perf with specified hostnames file.";\
#$echo "   ./$PROC_NAME --nbthreads <minimum_threads_number> : Configure State Transfer Perf with specified minimum number of threads .";\
#$echo ""

cat > /tmp/stp_usage <<__EOF__
Usage: ./$PROC_NAME [--hosts <hostnames_file_path>] [--nbthreads <minimum_threads_number>]
   ./$PROC_NAME                                      : Configure State Transfer Perf with default hostnames file ($HOSTNAMES_FILE) and default minimum number of threads ($NB_THREADS_MIN).
   ./$PROC_NAME --hosts <hostnames_file_path>        : Configure State Transfer Perf with specified hostnames file.
   ./$PROC_NAME --nbthreads <minimum_threads_number> : Configure State Transfer Perf with specified minimum number of threads .

__EOF__

USAGE=`cat /tmp/stp_usage`
rm -f /tmp/stp_usage

while [ "$1" != "" ] ; do
    if [ "$1" == "--help" ] || [ "$1" == "-help" ] || [ "$1" == "-h" ] ; then
        echo "$USAGE"
        exit 1
    elif [ "$1" == "--hosts" ] ; then
        HOSTNAMES_FILE="$2"
        if test -f "$HOSTNAMES_FILE"  ; then
            $echo "HOSTNAMES_FILE=$HOSTNAMES_FILE"
            shift;
        else
            $echo "Error: Hostnames file not found : $HOSTNAMES_FILE"
            echo "$USAGE"
            exit 1
        fi
    elif [ "$1" == "--nbthreads" ] ; then
        NB_THREADS_MIN="$2"
        if [ $NB_THREADS_MIN -gt 0 ]  ; then
            $echo "NB_THREADS_MIN=$NB_THREADS_MIN"
            shift;
        else
            $echo "Error: Bad minimum number of worker threads : $NB_THREADS_MIN"
            echo "$USAGE"
            exit 1
        fi
    elif [ "$1" == "--firstserver" ] ; then
        FIRST_SERVER="$2"
        if [ $FIRST_SERVER -gt 0 ]  ; then
            $echo "FIRST_SERVER=$FIRST_SERVER"
            shift;
        else
            $echo "Error: Bad first server index : $FIRST_SERVER"
            echo "$USAGE"
            exit 1
        fi
    else
        $echo "Syntax error :"
        echo "$USAGE"
        exit 1
    fi
    shift
done


PWD=`pwd`
STP_DIR=$PWD
STP_HOSTS=$STP_DIR/hosts.mk
STP_MAKEFILE=$STP_DIR/Makefile
DATA_STP_FTMGR_IIOP_XML=$STP_DIR/data/STP_FTManager_IIOP.xml
DATA_STP_FTMGR_MIOP_XML=$STP_DIR/data/STP_FTManager_MIOP.xml
DATA_STP_SUPERVISION_IIOP_XML=$STP_DIR/data/STP_Supervision_IIOP.xml
DATA_STP_SUPERVISION_MIOP_XML=$STP_DIR/data/STP_Supervision_MIOP.xml
DATA_STP_GROUP_CREATOR_XML=$STP_DIR/data/STP_GroupCreator_conf.xml
DATA_STP_SCENARIO_XML=$STP_DIR/data/STP_Scenario.xml


#
# STP HOSTS 
#

if test -f "$STP_HOSTS" ; then
    \rm -f $STP_HOSTS
fi

$echo "# Generated definition of Hostnames used by State Transfer Perf" > $STP_HOSTS
$echo "# DO NOT MODIFY THIS FILE: Modify 'Hostames' file to configure yours Hosts" >> $STP_HOSTS
$echo "" >> $STP_HOSTS

# Retrieve HOSTNAMES list from $HOSTNAMES_FILE
NB_HOSTNAMES=0
stp_makefile_hostname_sed="#"
for host in `cat $HOSTNAMES_FILE`
do
    NB_HOSTNAMES=$((${NB_HOSTNAMES}+1))
    $echo "HOSTNAME${NB_HOSTNAMES}=${host}" >> $STP_HOSTS
    stp_makefile_hostname_sed="$stp_makefile_hostname_sed\n\t@echo s%@HOSTNAME${NB_HOSTNAMES}@%\$(HOSTNAME${NB_HOSTNAMES})%g >> sed_patterns"
done

# Retrieve PROTOCOLS list from $PROTOCOLS
NB_PROTOCOLS=0
for prot in $PROTOCOLS
do
    NB_PROTOCOLS=$((${NB_PROTOCOLS}+1))
    eval PROTOCOL${NB_PROTOCOLS}=${prot}
done

if [ $NB_HOSTNAMES -le $NB_THREADS_MIN ] ; then
    NB_THREADS_MAX=${NB_THREADS_MIN}
else
    NB_THREADS_MAX=$((${NB_HOSTNAMES}-1))
fi    

$echo " NB_HOSTNAMES=${NB_HOSTNAMES}"

$echo " - configure '`basename $STP_HOSTS`' ..."
    
cat >> $STP_HOSTS <<__EOF__

NB_HOSTNAMES=${NB_HOSTNAMES}
NB_THREADS_MIN=${NB_THREADS_MIN}
NB_THREADS_MAX=${NB_THREADS_MAX}

__EOF__


#
# STP MAKEFILE
#

if test ! -f "$STP_MAKEFILE.conf"
then
    $echo "Failed: $STP_MAKEFILE.conf file is missing."
    exit 1
fi

$echo " - configure '`basename $STP_MAKEFILE`' ..."

num=${NB_THREADS_MIN}
stp_iiop="\\\\"
stp_miop="\\\\"
stp_sup_iiop="\\\\"
stp_sup_miop="\\\\"
while [ $num -le $NB_THREADS_MAX ] ; do
    stp_iiop="$stp_iiop\n    data\/STP_FTManager_IIOP_$num.xml \\\\"
    stp_miop="$stp_miop\n    data\/STP_FTManager_MIOP_$num.xml \\\\"
    stp_sup_iiop="$stp_sup_iiop\n    data\/STP_Supervision_IIOP_$num.xml \\\\"
    stp_sup_miop="$stp_sup_miop\n    data\/STP_Supervision_MIOP_$num.xml \\\\"
   
    num=$((${num}+1))
done

cat  $STP_MAKEFILE.conf | sed -e "s/@@DATA_STP_FTMGR_IIOP_XML@@/$stp_iiop/" > /tmp/stp_mk1
cat  /tmp/stp_mk1 | sed -e "s/@@DATA_STP_FTMGR_MIOP_XML@@/$stp_miop/" > /tmp/stp_mk2
cat  /tmp/stp_mk2 | sed -e "s/@@DATA_STP_SUPERVISION_IIOP_XML@@/$stp_sup_iiop/" > /tmp/stp_mk3
cat  /tmp/stp_mk3 | sed -e "s/@@DATA_STP_SUPERVISION_MIOP_XML@@/$stp_sup_miop/" > /tmp/stp_mk4
cat  /tmp/stp_mk4 | sed -e "s/@@HOSTNAME_SED@@/$stp_makefile_hostname_sed/" > $STP_MAKEFILE
rm -f /tmp/stp_mk1 /tmp/stp_mk2 /tmp/stp_mk3 /tmp/stp_mk4


#
# DATA STP FTMGR IIOP XML
#

if test ! -f "$DATA_STP_FTMGR_IIOP_XML.conf"
then
    $echo "Failed: $DATA_STP_FTMGR_IIOP_XML.conf file is missing."
    exit 1
fi

if test ! -f "$DATA_STP_FTMGR_MIOP_XML.conf"
then
    $echo "Failed: $DATA_STP_FTMGR_MIOP_XML.conf file is missing."
    exit 1
fi

if test ! -f "$DATA_STP_SUPERVISION_IIOP_XML.conf"
then
    $echo "Failed: $DATA_STP_SUPERVISION_IIOP_XML.conf file is missing."
    exit 1
fi

if test ! -f "$DATA_STP_SUPERVISION_MIOP_XML.conf"
then
    $echo "Failed: $DATA_STP_SUPERVISION_MIOP_XML.conf file is missing."
    exit 1
fi

$echo " - configure 'data/`basename $DATA_STP_FTMGR_IIOP_XML`.in' and 'data/`basename $DATA_STP_FTMGR_MIOP_XML`.in' and 'data/`basename $DATA_STP_SUPERVISION_IIOP_XML`.in' and 'data/`basename $DATA_STP_SUPERVISION_MIOP_XML`.in'..."

num=${NB_THREADS_MIN}
stp_iiop=""
stp_miop=""
stp_sup_iiop=""
stp_sup_miop=""
while [ $num -le $NB_THREADS_MAX ] ; do
    stp_iiop=$STP_DIR/data/STP_FTManager_IIOP_$num.xml.in
    cat $DATA_STP_FTMGR_IIOP_XML.conf | sed -e "s/@@NB_WORKER_THREADS@@/$num/" > $stp_iiop
    stp_miop=$STP_DIR/data/STP_FTManager_MIOP_$num.xml.in
    cat $DATA_STP_FTMGR_MIOP_XML.conf | sed -e "s/@@NB_WORKER_THREADS@@/$num/" > $stp_miop
    stp_sup_iiop=$STP_DIR/data/STP_Supervision_IIOP_$num.xml.in
    cat $DATA_STP_SUPERVISION_IIOP_XML.conf | sed -e "s/@@NB_WORKER_THREADS@@/$num/" > $stp_sup_iiop
    stp_sup_miop=$STP_DIR/data/STP_Supervision_MIOP_$num.xml.in
    cat $DATA_STP_SUPERVISION_MIOP_XML.conf | sed -e "s/@@NB_WORKER_THREADS@@/$num/" > $stp_sup_miop
   
    num=$((${num}+1))
done


#
# DATA STP GROUP CREATOR XML
#

if test ! -f "$DATA_STP_GROUP_CREATOR_XML.conf"
then
    $echo "Failed: $DATA_STP_GROUP_CREATOR_XML.conf file is missing."
    exit 1
fi

$echo " - configure 'data/`basename $DATA_STP_GROUP_CREATOR_XML`.in' ..."

nbt=${NB_THREADS_MIN}
stp_groups=""
while [ $nbt -le $NB_THREADS_MAX ] ; do
    for prot in $PROTOCOLS ; do
        stp_groups="$stp_groups\n\
    <object-group name=\\\"GRP_STP_FTManager_${prot}_${nbt}\\\" type-id=\\\"IDL:thalesgroup.com\/StateTransferPerf\/Server:1.0\\\" minimum-number-replicas=\\\"1\\\">\n\
        <locations>"
    
        nbh=1
        while [ $nbh -le $NB_HOSTNAMES ] ; do
            stp_groups="$stp_groups\n\
            <location host-name=\\\"@HOSTNAME${nbh}@\\\" application-name=\\\"STP_FTManager\\\" process-name=\\\"stp_ftmanager_${prot}_${nbt}_@HOSTNAME${nbh}@\\\"\/>"
        
            nbh=$((${nbh}+1))
        done
        stp_groups="$stp_groups\n\
        <\/locations>\n\
        <datastoreIDs>\n\
            <datastoreID id=\\\"123\\\"\/>\n\
        <\/datastoreIDs> \n\
    <\/object-group>"

    done
    nbt=$((${nbt}+1))
done



nbt=${NB_THREADS_MIN}
stp_sup_groups=""
while [ $nbt -le $NB_THREADS_MAX ] ; do
    for prot in $PROTOCOLS ; do
        stp_sup_groups="$stp_sup_groups\n\
    <object-group name=\\\"GRP_STP_Supervision_${prot}_${nbt}\\\" type-id=\\\"IDL:thalesgroup.com\/StateTransferPerf\/Server:1.0\\\" minimum-number-replicas=\\\"1\\\">\n\
        <locations>"
    
        nbh=1
        while [ $nbh -le $NB_HOSTNAMES ] ; do
            stp_sup_groups="$stp_sup_groups\n\
            <location host-name=\\\"@HOSTNAME${nbh}@\\\" application-name=\\\"STP_Supervision\\\" process-name=\\\"stp_supervision_${prot}_${nbt}_@HOSTNAME${nbh}@\\\"\/>"
        
            nbh=$((${nbh}+1))
        done
        stp_sup_groups="$stp_sup_groups\n\
        <\/locations>\n\
        <datastoreIDs>\n\
            <datastoreID id=\\\"124\\\"\/>\n\
        <\/datastoreIDs> \n\
    <\/object-group>"

    done
    nbt=$((${nbt}+1))
done

cat  $DATA_STP_GROUP_CREATOR_XML.conf | sed -e "s/@@GRP_STP_FTMGR_CONF@@/$stp_groups/" > /tmp/stp_group1
cat  /tmp/stp_group1 | sed -e "s/@@GRP_STP_SUPERVISION_CONF@@/$stp_sup_groups/" > $DATA_STP_GROUP_CREATOR_XML.in
rm -f /tmp/stp_group1


#
# DATA STP SCENARIO XML
#

if test ! -f "$DATA_STP_SCENARIO_XML.conf"
then
    $echo "Failed: $DATA_STP_SCENARIO_XML.conf file is missing."
    exit 1
fi

$echo " - configure 'data/`basename $DATA_STP_SCENARIO_XML`.in' ..."

nbh=1
stp_hosts=""
while [ $nbh -le $NB_HOSTNAMES ] ; do
        stp_hosts="$stp_hosts\n    <Host Name=\\\"@HOSTNAME${nbh}@\\\" App-exec-path=\\\"cdmw_platform_application\\\" Agent-exec-path=\\\"@CDMW_HOME@\/bin\/cdmw_platform_agent\\\"\/>"

    nbh=$((${nbh}+1))
done

stp_serv=""
stp_sup_serv=""
stp_graph1=""
stp_graph2=""
stp_graph3=""
stp_sup_graph1=""
stp_sup_graph2=""
stp_sup_graph3=""
nbp=1
while [ $nbp -le $NB_PROTOCOLS ] ; do
     prot=`eval echo '$'PROTOCOL${nbp}`
     if [ "$prot" = "IIOP" ] ; then
         argPtP="--PointToPoint"
     else
         argPtP=""
     fi
    nbt=$NB_THREADS_MIN
    while [ $nbt -le $NB_THREADS_MAX ] ; do
        nbh=$FIRST_SERVER
        # --CdmwLogLevel=ALL 
        while [ $nbh -le $NB_HOSTNAMES ] ; do
            stp_serv="$stp_serv \n\
        <Managed-process \n\
            Name=\\\"stp_ftmanager_${prot}_${nbt}_@HOSTNAME${nbh}@\\\" \n\
            Host-name=\\\"@HOSTNAME${nbh}@\\\" \n\
            Type=\\\"MANAGED-PROCESS\\\" \n\
            Autoended=\\\"FALSE\\\" \n\
            Exec-path=\\\"@PWD@\/run\/launch.sh\\\" \n\
            Args=\\\"--CdmwLogFile=STP_FTManager_${prot}_${nbt}_@HOSTNAME${nbh}@.log ${argPtP} --CdmwXMLFile=@PWD@\/data\/STP_FTManager_${prot}_${nbt}.xml --validateXML --NbWorkerThreads=${nbt} stp_ftmanager_${prot}\\\" \n\
            Port=\\\"-1\\\"> \n\
            <Init-step Name=\\\"step0\\\"\/> \n\
        <\/Managed-process>"

            stp_sup_serv="$stp_sup_serv \n\
        <Managed-process \n\
            Name=\\\"stp_supervision_${prot}_${nbt}_@HOSTNAME${nbh}@\\\" \n\
            Host-name=\\\"@HOSTNAME${nbh}@\\\" \n\
            Type=\\\"MANAGED-PROCESS\\\" \n\
            Autoended=\\\"FALSE\\\" \n\
            Exec-path=\\\"@PWD@\/run\/launch.sh\\\" \n\
            Args=\\\"--CdmwLogFile=STP_Supervision_${prot}_${nbt}_@HOSTNAME${nbh}@.log ${argPtP} --CdmwXMLFile=@PWD@\/data\/STP_Supervision_${prot}_${nbt}.xml --validateXML --NbWorkerThreads=${nbt} stp_supervision_${prot}\\\" \n\
            Port=\\\"-1\\\"> \n\
            <Init-step Name=\\\"step0\\\"\/> \n\
        <\/Managed-process>"


            # Define the successor graph elements
            nbp_next=$((${nbp}+1))
            nbt_next=$((${nbt}+1))
            nbh_next=$((${nbh}+1))
            if [ $nbh_next -le $NB_HOSTNAMES ] &&  [ $nbt_next -le $NB_THREADS_MAX ] ; then
                successsor="stp_ftmanager_${prot}_${nbt_next}_@HOSTNAME${nbh}@"
            elif [ $nbt_next -le $NB_THREADS_MAX ] ; then
                successsor="stp_ftmanager_${prot}_${nbt_next}_@HOSTNAME${nbh}@"
            elif [ $nbp_next -le $NB_PROTOCOLS ] ; then
                prot_next=`eval echo '$'PROTOCOL${nbp_next}`
                successsor="stp_ftmanager_${prot_next}_${NB_THREADS_MIN}_@HOSTNAME${nbh}@"
            else
                successsor="stp_test_driver"
            fi

            if [ $nbh_next -le $NB_HOSTNAMES ] &&  [ $nbt_next -le $NB_THREADS_MAX ] ; then
                sup_successsor="stp_supervision_${prot}_${nbt_next}_@HOSTNAME${nbh}@"
            elif [ $nbt_next -le $NB_THREADS_MAX ] ; then
                sup_successsor="stp_supervision_${prot}_${nbt_next}_@HOSTNAME${nbh}@"
            elif [ $nbp_next -le $NB_PROTOCOLS ] ; then
                prot_next=`eval echo '$'PROTOCOL${nbp_next}`
                sup_successsor="stp_supervision_${prot_next}_${NB_THREADS_MIN}_@HOSTNAME${nbh}@"
            fi

#            if [ $nbh_next -le $NB_HOSTNAMES ] &&  [ $nbt_next -le $NB_THREADS_MAX ] ; then
#                successsor="stp_ftmanager_${prot}_${nbt_next}_@HOSTNAME${nbh}@"
#            elif [ $nbt_next -le $NB_THREADS_MAX ] ; then
#                successsor="stp_ftmanager_${prot}_${nbt_next}_@HOSTNAME${nbh}@"
#            elif [ $nbp_next -le $NB_PROTOCOLS ] ; then
#                prot_next=`eval echo '$'PROTOCOL${nbp_next}`
#                successsor="stp_ftmanager_${prot_next}_${NB_THREADS_MIN}_@HOSTNAME${nbh}@"
#            else
#                successsor="stp_test_driver"
#            fi

            # Retrieve the root graph elements
            stp_graph1="$stp_graph1\n\
            <Graph-element Name=\\\"stp_ftmanager_${prot}_${nbt}_@HOSTNAME${nbh}@\\\">\n\
                <Graph-element-successor Name=\\\"$successsor\\\"\/>\n\
            <\/Graph-element>"

            if [ $nbt -le $NB_THREADS_MIN ] &&  [ $nbp -le 1 ] ; then
                stp_graph3="$stp_graph3\n\
            <Graph-element-root Name=\\\"stp_ftmanager_${prot}_${nbt}_@HOSTNAME${nbh}@\\\"\/>"
            fi

            if [ $((${nbp}+1)) -le $NB_PROTOCOLS ] ; then
                stp_sup_graph1="$stp_sup_graph1\n\
                <Graph-element Name=\\\"stp_supervision_${prot}_${nbt}_@HOSTNAME${nbh}@\\\">\n\
                    <Graph-element-successor Name=\\\"$sup_successsor\\\"\/>\n\
                <\/Graph-element>"
            else
stp_sup_graph2="$stp_sup_graph2\n\
            <Graph-element Name=\\\"stp_supervision_${prot}_${nbt}_@HOSTNAME${nbh}@\\\"\/>\n"
            fi

            if [ $nbt -le $NB_THREADS_MIN ] &&  [ $nbp -le 1 ] ; then
                stp_sup_graph3="$stp_sup_graph3\n\
            <Graph-element-root Name=\\\"stp_supervision_${prot}_${nbt}_@HOSTNAME${nbh}@\\\"\/>"
            fi

            nbh=$((${nbh}+1))
        done
        nbt=$((${nbt}+1))
    done
    nbp=$((${nbp}+1))

done

stp_graph2="$stp_graph2\n\
            <Graph-element Name=\\\"stp_test_driver\\\"\/>\n"


cat  $DATA_STP_SCENARIO_XML.conf | sed -e "s/@@STP_SCEN_HOSTS@@/$stp_hosts/" > /tmp/stp_scen1
cat  /tmp/stp_scen1 | sed -e "s/@@STP_SCEN_SERVERS@@/$stp_serv/" > /tmp/stp_scen2
cat  /tmp/stp_scen2 | sed -e "s/@@STP_SCEN_SUPERVISION_SERVERS@@/$stp_sup_serv/" > /tmp/stp_scen3
cat  /tmp/stp_scen3 | sed -e "s/@@STP_SCEN_INIT_GRAPH@@/$stp_graph3 $stp_graph1 $stp_graph2/" > /tmp/stp_scen4
cat  /tmp/stp_scen4 | sed -e "s/@@STP_SCEN_SUPERVISION_INIT_GRAPH@@/$stp_sup_graph3 $stp_sup_graph1 $stp_sup_graph2/" > $DATA_STP_SCENARIO_XML.in
rm -f /tmp/stp_scen1 /tmp/stp_scen2 /tmp/stp_scen3 /tmp/stp_scen4

    
