/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_CLIENT_THREAD_HPP
#define INCL_CLIENT_THREAD_HPP

#include <iostream>
#include <fstream>

// Cdmw Files
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/osthreads/SchedulerBase.hpp>
#include <Foundation/orbsupport/CosNaming.stub.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include "FaultTolerance/idllib/CdmwFTReplicationManager.stub.hpp"

// StateTransferPerf Files
#include <StateTransferPerf.stub.hpp>
#include "STPCommon.hpp"


class STPClientThread : public Cdmw::OsSupport::Thread
{
public:
    
      STPClientThread(CosNaming::NamingContextExt_ptr root,
                   CdmwFT::ReplicationManager_ptr replication_manager,
                   unsigned short nb_iterations)
         : m_root_context(CosNaming::NamingContextExt::_duplicate(root)),
           m_replication_manager( CdmwFT::ReplicationManager::_duplicate(replication_manager) ),
           m_stopped(false),
           m_nb_iterations(nb_iterations),
           m_application_name("")
    {
    }

    virtual ~STPClientThread()
        throw()
    {
    }
    
    void stop() throw()
    {
        m_stopped = true;
    }
    
protected:

    void run() throw();
    
    
private:
    CosNaming::NamingContextExt_var m_root_context;
    CdmwFT::ReplicationManager_var m_replication_manager;
    volatile bool m_stopped;
    unsigned short m_nb_iterations;
    std::string m_application_name;
    
    void STPClientThread::DoStateTransferPerf(Cdmw::CommonSvcs::Naming::NamingInterface & ni) throw();
    unsigned long get_duration(Cdmw::OsSupport::OS::Timeval& beg_time, Cdmw::OsSupport::OS::Timeval& end_time);
};

#endif // INCL_CLIENT_THREAD_HPP

