/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_STP_SERVER_ADMIN_IMPL_HPP
#define INCL_STP_SERVER_ADMIN_IMPL_HPP


// Cdmw Files
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountServantBase.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>

// State Transfer Perf Files
#include <StateTransferPerf.skel.hpp>


class STPServerAdmin_impl : public virtual POA_StateTransferPerf::ServerAdmin,
                            public virtual Cdmw::OrbSupport::RefCountServantBase
{

public:

    // Constructor
    STPServerAdmin_impl(CORBA::ORB_ptr orb,
                      PortableServer::POA_ptr parentPOA,
                      const StateTransferPerf::ServerConfig& config,
                      CosNaming::Name* location,
                      StateTransferPerf::Server_ptr server);

    // Destructor
    virtual ~STPServerAdmin_impl()
        throw();

    // returns the POA of this servant
    virtual PortableServer::POA_ptr _default_POA();
        
        
    // IDL:thalesgroup.com/STP/StateTransferPerf/get_configuration()
    virtual StateTransferPerf::ServerConfig* get_configuration()
        throw( CORBA::SystemException );

    // IDL:thalesgroup.com/STP/StateTransferPerf/get_server()
    virtual StateTransferPerf::Server_ptr get_server()
        throw( CORBA::SystemException );
        
    // IDL:thalesgroup.com/STP/StateTransferPerf/get_location()
    virtual CosNaming::Name* get_location()
        throw( CORBA::SystemException );
        
private:

    // Hide copy constructor/assignment operator
    STPServerAdmin_impl(
        const STPServerAdmin_impl& rhs )
            throw();

    STPServerAdmin_impl& operator =(
        const STPServerAdmin_impl& rhs )
            throw();

    // orb
    CORBA::ORB_var m_orb;

    // servant's POA
    PortableServer::POA_var m_poa;

    // Server Configuration
    StateTransferPerf::ServerConfig m_server_configuration;

    // Server pointer
    StateTransferPerf::Server_var m_server;

    // Server Location
    CosNaming::Name m_server_location;
    
    
}; // End class Database_impl 



#endif // INCL_STP_SERVER_ADMIN_IMPL_HPP



