/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_DATABASE_PROCESS_BEHAVIOUR_HPP 
#define INCL_DATABASE_PROCESS_BEHAVIOUR_HPP 


// StateTransferPerf Files
#include <STPServer_impl.hpp>
#include <STPServerAdmin_impl.hpp>

// Cdmw Files
#include <Foundation/orbsupport/CORBA.hpp>
#include <ConfAndPlug/cdmwinit/ProcessControl.hpp>
#include <FaultTolerance/idllib/CdmwFTGroupRepository.stub.hpp>




/**
* Purpose:
* <p>
* The abstract process behaviour.
*/
class STPServerProcessBehaviour : public Cdmw::CdmwInit::ProcessControl
{

public:

    STPServerProcessBehaviour(CORBA::ORB_ptr orb,
                              std::string protocol,
                              std::string nb_worker_threads);

    
    virtual ~STPServerProcessBehaviour() throw ();
    
    virtual CORBA::ULong nb_steps()
        throw( CORBA::SystemException );
    
    virtual void on_initialise(
        const CdmwPlatformMngtBase::StartupKind& startup_kind )
        throw( CORBA::SystemException );
    
    void on_next_step()
        throw( CdmwPlatformMngt::ProcessDelegate::InvalidStep,
               CORBA::SystemException );

    virtual void on_run()
        throw( CdmwPlatformMngt::ProcessDelegate::NotReadyToRun,
               CORBA::SystemException );

    virtual void on_stop()
        throw( CORBA::SystemException );

private:

    // The ORB.
    CORBA::ORB_var m_orb;
    
    // IIOP if the point-to-point State Transfer has to be used 
    // MIOP if the multicast State Transfer has to be used 
    std::string m_protocol;

    // Number of worker threads
    std::string m_nb_worker_threads;

    // The application name.
    std::string m_application_name;
    
    // The process name.
    std::string m_process_name;
    
    // The Replication Manager
    FT::ReplicationManager_var m_replication_mgr;
    
    // The STPServer servant
    STPServer_impl* m_stp_server_impl;
    
    // The STPServer object reference
    StateTransferPerf::Server_var m_stp_server_obj;
    
    // The STPServerAdmin servant
    STPServerAdmin_impl* m_stp_server_admin_impl;
    
    // The STPServerAdmin object reference
    StateTransferPerf::ServerAdmin_var m_stp_server_admin_obj;
    
};


#endif // INCL_DATABASE_PROCESS_BEHAVIOUR_HPP


