/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_STP_SUPERVISION_SERVER_IMPL_HPP
#define INCL_STP_SUPERVISION_SERVER_IMPL_HPP


// Cdmw Files
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountServantBase.hpp>
#include <FaultTolerance/ftstatemanager/DataStore.hpp>
#include <Foundation/commonsvcs/datastore/StorageHome.hpp>
#include <Foundation/commonsvcs/datastore/StorageObject.hpp>
#include "FaultTolerance/idllib/CdmwFTReplicationManager.stub.hpp"


// State Transfer Perf Files
#include <StateTransferPerf.skel.hpp>
#include <STP_SupervisionData.stub.hpp>


// Define data_traits for fixed struct StateTransferPerf::STP_SupervisionData
// This is required to use a DataStore with those struct
DEFINE_VARIABLE_LENGTH_DATA_TRAITS(StateTransferPerf::STP_SupervisionData);


class STPSupervisionServer_impl : public virtual POA_StateTransferPerf::Server,
                          public virtual Cdmw::OrbSupport::RefCountServantBase
{

public:

    // Constructor
    STPSupervisionServer_impl(CORBA::ORB_ptr orb,
                      PortableServer::POA_ptr parentPOA);

    // Destructor
    virtual ~STPSupervisionServer_impl()
        throw();

    // returns the POA of this servant
    virtual PortableServer::POA_ptr _default_POA();
        
        
    // IDL:thalesgroup.com/STP/StateTransferPerf/initialise()
    virtual void initialise (const CORBA::Long nb_records)
        throw( CORBA::SystemException );

    // IDL:thalesgroup.com/STP/StateTransferPerf/update_record()
    virtual void update_record (const CORBA::Long record_number)
        throw( CORBA::SystemException );
        
private:

    // Hide copy constructor/assignment operator
    STPSupervisionServer_impl(
        const STPSupervisionServer_impl& rhs )
            throw();

    STPSupervisionServer_impl& operator =(
        const STPSupervisionServer_impl& rhs )
            throw();

    // Definition of Storage objects for STP_FTManagerData type 
    typedef Cdmw::CommonSvcs::DataStore::StorageHome< StateTransferPerf::Ident, StateTransferPerf::STP_SupervisionData > STP_SupervisionDataStorageHome;    
    typedef Cdmw::CommonSvcs::DataStore::StorageObject< StateTransferPerf::Ident, StateTransferPerf::STP_SupervisionData > STP_SupervisionDataStorageObject;
    typedef Cdmw::FT::DataStore< StateTransferPerf::Ident, StateTransferPerf::STP_SupervisionData > STP_SupervisionDataDataStore;
            
    // StorageHome for LogData
    STP_SupervisionDataStorageHome m_SupervisionDataStorageHome;
    
    // orb
    CORBA::ORB_var m_orb;

    // servant's POA
    PortableServer::POA_var m_poa;

    // process name
    std::string m_process_name;
    
}; // End class Database_impl 



#endif // INCL_STP_SUPERVISION_SERVER_IMPL_HPP



