#!/bin/sh
# =====================================================================
# This file is part of CARDAMOM (R) which is jointly developed by THALES
# and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
# Copyright (C) THALES 2000-2003. All rights reserved.
# Copyright (C) THALES 2004-2005. All rights reserved.
# Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
# All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
# License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

if test "`echo -e xxx`" = "xxx"
then
    echo="echo -e"
else
    echo=echo
fi

#set -x

if [ "$1" != "" ]
then
    PROC_NAME=`basename $0`
    $echo "Usage: $PROC_NAME"
    $echo "   $PROC_NAME                : Start State Transfer Perf demo."
    $echo ""
    exit 1
fi

PWD=`pwd`
DEMO_DIR=`echo $PWD | sed s%/run%%g`
SITE_CONFIG=$DEMO_DIR/site.mk
HOSTS_CONFIG=$DEMO_DIR/hosts.mk

if test ! -f "$SITE_CONFIG"
then
    $echo "$SITE_CONFIG file is missing. Run  './configure.sh' and 'make' in parent directory and restart $0 "
    exit 1
fi

if test ! -f "$HOSTS_CONFIG"
then
    $echo "$HOSTS_CONFIG file is missing. Run  './configure.sh' and 'make' in parent directory and restart $0 "
    exit 1
fi

# CDMW installation folder
CDMW_HOME="`cat $SITE_CONFIG | grep CDMW_HOME= |cut -d= -f2`"
if [ "$CDMW_HOME" = "" ]
then
    $echo "CDMW_HOME is not set into $SITE_CONFIG"
    exit 1
fi

LD_LIBRARY_PATH=$CDMW_HOME/lib/c++:$LD_LIBRARY_PATH
export LD_LIBRARY_PATH
PATH=$CDMW_HOME/bin:$PATH
export PATH


# Check NB_HOSTNAMES is set
NB_HOSTNAMES="`cat $HOSTS_CONFIG | grep NB_HOSTNAMES= |cut -d= -f2`"
$echo "NB_HOSTNAMES=$NB_HOSTNAMES"
if [ "$NB_HOSTNAMES" = "" ]
then
    $echo "NB_HOSTNAMES is not set into $HOSTS_CONFIG. Run  './configure.sh' and 'make' in parent directory and restart $0"
    exit 1
fi

# "hostname" used for the FaultManagerRegistration 
#HOSTNAME1=`hostname`

nbh=1
while [ $nbh -le $NB_HOSTNAMES ] ; do
    host="`cat $HOSTS_CONFIG | grep HOSTNAME${nbh}= |cut -d= -f2`"
    eval HOSTNAME${nbh}=$host
    $echo "HOSTNAME${nbh}=$host"
    if [ "$host" = "" ]
    then
        $echo "HOSTNAME${nbh} is not set into $HOSTS_CONFIG"
        exit 1
    fi

    nbh=$((${nbh}+1))
done

# XML scenario file
SCENARIO_FILE=$DEMO_DIR/data/STP_Scenario.xml

if test ! -f "$SCENARIO_FILE"
then
    $echo "$SCENARIO_FILE file is missing. Run 'make data' in parent directory and restart $0 "
    exit 1
fi

# XML Platform Management Daemon configuration file
DAEMON_CONF_FILE=$DEMO_DIR/data/CdmwPlatformMngtDaemon_conf.xml

if test ! -f "$DAEMON_CONF_FILE"
then
    $echo "$DAEMON_CONF_FILE file is missing. Run 'make data' in parent directory and restart $0 "
    exit 1
fi


if test ! -x `which $TERM`
then
    TERM=xterm
fi

TERM_PATH=`which $TERM`

INIT_TIMEOUT=5
FT_MANAGER_TIMEOUT=10
EXEC_TIMEOUT=160
CLEANUP_TIMEOUT=10


$echo "========================================================="
$echo "=                CDMW - STATE TRANSFER PERF             ="
$echo "========================================================="

HOST_TYPE="`$CDMW_HOME/bin/config.guess`"

# 0) Check Cardamom & demo installation on all hosts

nbh=1
while [ $nbh -le $NB_HOSTNAMES ] ; do
    host=`eval echo '$HOSTNAME'${nbh}`
    CDMW_ALREADY_INSTALLED=`ssh $host ls $CDMW_HOME/bin/cdmw_platform_daemon 2>/dev/null`
    if [ "$CDMW_ALREADY_INSTALLED" = "" ]
    then
        $echo "CARDAMOM not installed on $host !"
        $echo " (it should be installed into $CDMW_HOME directory)"
        exit 1
    fi

    DEMO_ALREADY_INSTALLED=`ssh $host ls $DEMO_DIR/$HOST_TYPE/stp_test_driver 2>/dev/null`
    if [ "$DEMO_ALREADY_INSTALLED" = "" ]
    then
        $echo "State Transfer Perf is not installed on $host !"
        $echo " (it should be installed into $DEMO_DIR directory)"
        exit 1
    fi

    nbh=$((${nbh}+1))
done


# 1a) Start daemon 
DAEMON_COMMAND="$CDMW_HOME/bin/cdmw_platform_daemon.sh --CdmwXMLFile=$DAEMON_CONF_FILE"

pos=0
nbh=1
while [ $nbh -le $NB_HOSTNAMES ] ; do
    host=`eval echo '$HOSTNAME'${nbh}`

    $echo "Starting the Platform Management Daemon on ${host} ..."
    TERM_COMMAND="$TERM -sb -sl 50000 -geometry 160x25+${pos}+${pos} -title ${host}_platform_daemon"
    ssh -X $host "(LD_LIBRARY_PATH=$LD_LIBRARY_PATH; export LD_LIBRARY_PATH; PATH=$PATH; export PATH; $TERM_COMMAND -e $DAEMON_COMMAND)" &
    sleep 1

    pos=$((${pos}+75))
    nbh=$((${nbh}+1))
done

# Create a script to launch platform snapshot
$echo "LD_LIBRARY_PATH=$LD_LIBRARY_PATH ; export LD_LIBRARY_PATH ;\n PATH=$PATH ; export PATH ;\n\
      $CDMW_HOME/bin/cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21880/CdmwPlatformMngtSupervision --sys-snapshot\
     " > $DEMO_DIR/run/stp_platform_snapshot.sh
chmod +x $DEMO_DIR/run/stp_platform_snapshot.sh

trap "$DAEMON_COMMAND stop; exit" 2
#ssh $HOSTNAME2 trap "$DAEMON_COMMAND stop; exit" 2
#ssh $HOSTNAME3 trap "$DAEMON_COMMAND stop; exit" 2

#$echo "Hit a key to continue..."
#read FOO

sleep $INIT_TIMEOUT

# 1b) Start FT Manager --CdmwLogLevel=ALL
$echo Starting the FT Manager...
$CDMW_HOME/bin/cdmw_ft_manager --CdmwXMLFile=$DEMO_DIR/data/CdmwFaultToleranceManager_conf.xml --groupConf=$DEMO_DIR/data/STP_GroupCreator_conf.xml &
FT_MANAGER_PID=$!
sleep $FT_MANAGER_TIMEOUT

# 2) Start Platform Management Supervision
$CDMW_HOME/bin/cdmw_platform_supervision_starter --CdmwXMLFile=$DEMO_DIR/data/CdmwPlatformMngtSystemStart.xml --validate
sleep $INIT_TIMEOUT

# 3) Define system
$echo Defining the System '$SCENARIO_FILE'
$CDMW_HOME/bin/cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21880/CdmwPlatformMngtSupervision --sys-define $SCENARIO_FILE
sleep $INIT_TIMEOUT

# 4) Start_system
$CDMW_HOME/bin/cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21880/CdmwPlatformMngtSupervision --sys-start
if [ $? -eq "0" ];
then
    $echo "Waiting a while for the execution of the scenario..."
    #sleep $EXEC_TIMEOUT
    $echo "###############################################################"
    $echo "Hit a key when the scenario is ended..."
    $echo "###############################################################"
    read FOO
    $echo done.
fi

# 5) Get a snapshot of the system
$CDMW_HOME/bin/cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21880/CdmwPlatformMngtSupervision --sys-snapshot

# 6) stop_system
$CDMW_HOME/bin/cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21880/CdmwPlatformMngtSupervision --sys-stop
sleep $CLEANUP_TIMEOUT

$echo ""
$echo "=============================================================="
$echo "Hit a key when ready to STOP the CDMW Daemon and Supervision."
$echo "=============================================================="
read FOO
$echo ""

#$DAEMON_COMMAND stop
nbh=1
while [ $nbh -le $NB_HOSTNAMES ] ; do
    host=`eval echo '$HOSTNAME'${nbh}`

    ssh $host $DAEMON_COMMAND stop

    nbh=$((${nbh}+1))
done

kill -9 $FT_MANAGER_PID
$echo "done."

rm -f *.ior
rm -f *.IOR

