# =====================================================================
# This file is part of CARDAMOM (R) which is jointly developed by THALES
# and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
# Copyright (C) THALES 2000-2003. All rights reserved.
# Copyright (C) THALES 2004-2005. All rights reserved.
# Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
# All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
# License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

SHELL = /bin/sh

include ../site.mk
# Imported variables
#   
#   CDMW_HOME
#   RM
#   CXX
#   CXXLD
#   IDLMKDEPEND
#   IDLMKINCLUDE
#   ORB
#   IDL2CXX
#   
#   CPPFLAGS_ORB
#   CXXFLAGS_ORB
#   IDLFLAGS_ORB
#   LDFLAGS_ORB
#   LIBS_ORB
#   
#   CPPFLAGS_CDMW
#   CXXFLAGS_CDMW
#   LDFLAGS_CDMW
#   IDLFLAGS_CDMW
#   LIBS_CDMW
#   
# NOTA: The variables below includes CDMW/ORB/OS specific variables
#   CPPFLAGS
#   CXXFLAGS
#   LDFLAGS
#   IDLFLAGS
#   LIBS
include ../config.mk

#Append to CPPFLAGS and IDLFLAGS
override CPPFLAGS += -I. -I../include -I../generated \
	$(foreach d, $(shell find $(CDMW_HOME)/include/c++ -name TAO\* -prune -o -type d -print), -I$(d)) \
        -I$(CDMW_HOME)/include/c++/TAOOpenFusion
override IDLFLAGS += -I../idl -I../generated

ALL_LIBS=$(LIBS)


# Exported variables
#   CORBA_ALL_IDL,
#   CORBA_CL_ALL_SRC,
#   CORBA_SRV_ALL_SRC,
#   CORBA_CL_ALL_OBJS,
#   CORBA_SRV_ALL_OBJS,
#   CORBA_CL_ALL_HDRS, and
#   CORBA_SRV_ALL_HDRS
include idl_var.mk


vpath %.idl ../idl:../generated
vpath %.cpp ../src:../generated
vpath %.hpp ../include:../generated

# STP executable
CLIENT_NAME=stp_test_driver
SERVER1_NAME=stp_ftmanager_IIOP
SERVER2_NAME=stp_ftmanager_MIOP
SERVER3_NAME=stp_supervision_IIOP
SERVER4_NAME=stp_supervision_MIOP


# CDMW Generated Code
CDMW_GEN_CLIENT_SRC=STP_TestDriverCdmwInterface.cpp
CDMW_GEN_SERVER1_SRC=STP_IIOPFTManagerDataCdmwInterface.cpp
CDMW_GEN_SERVER2_SRC=STP_MIOPFTManagerDataCdmwInterface.cpp
CDMW_GEN_SERVER3_SRC=STP_IIOPSupervisionDataCdmwInterface.cpp
CDMW_GEN_SERVER4_SRC=STP_MIOPSupervisionDataCdmwInterface.cpp

# User Supplied Code
CLIENT_SRCS=$(CDMW_GEN_CLIENT_SRC) \
	STPClientThread.cpp \
	STPClientProcessBehaviour.cpp \
	STPClient.cpp

SERVER1_SRCS=	$(CDMW_GEN_SERVER1_SRC) \
	STPServer_impl.cpp \
	STPServerAdmin_impl.cpp \
	STPServer.cpp  \
	STPServerProcessBehaviour.cpp 

SERVER2_SRCS=	$(CDMW_GEN_SERVER2_SRC) \
	STPServer_impl.cpp \
	STPServerAdmin_impl.cpp \
	STPServerProcessBehaviour.cpp \
	STPServer.cpp 

SERVER3_SRCS=	$(CDMW_GEN_SERVER3_SRC) \
	STPSupervisionServer_impl.cpp \
	STPServerAdmin_impl.cpp \
	STPSupervisionServerProcessBehaviour.cpp \
	STPServer_impl.cpp \
	STPSupervisionServer.cpp 

SERVER4_SRCS=	$(CDMW_GEN_SERVER4_SRC) \
	STPSupervisionServer_impl.cpp \
	STPServerAdmin_impl.cpp \
	STPSupervisionServerProcessBehaviour.cpp \
	STPSupervisionServer.cpp 


.SUFFIXES:
.SUFFIXES: .idl .cpp .skel.cpp .stub.cpp .o .stub.hpp .skel.hpp .hpp .h


CLIENT_OBJ = $(CLIENT_SRCS:.cpp=.o) $(CORBA_SRV_ALL_OBJS)
SERVER1_OBJ = $(SERVER1_SRCS:.cpp=.o) $(CORBA_SRV_ALL_OBJS)
SERVER2_OBJ = $(SERVER2_SRCS:.cpp=.o) $(CORBA_SRV_ALL_OBJS)
SERVER3_OBJ = $(SERVER3_SRCS:.cpp=.o) $(CORBA_SRV_ALL_OBJS)
SERVER4_OBJ = $(SERVER4_SRCS:.cpp=.o) $(CORBA_SRV_ALL_OBJS)

# Rules to build the dependency of each source file
%.d: %.cpp
	@echo "Building dependency of $<"
	@$(CXX) -M $(CPPFLAGS) $(CXXFLAGS) $< \
	| sed 's/\($*\)\.o[ :]*/\1.o $@ : /g' > $@; \
	[ -s $@ ] || rm -f $@


SRCS=$(CLIENT_SRCS) $(SERVER1_SRCS) $(SERVER2_SRCS) $(SERVER3_SRCS) $(SERVER4_SRCS) $(CORBA_SRV_ALL_SRC)
DEPENDS=$(SRCS:.cpp=.d)



.PHONY: all
all: depend $(CLIENT_NAME) $(SERVER1_NAME) $(SERVER2_NAME) $(SERVER3_NAME) $(SERVER4_NAME)


.PHONY: depend
depend: $(DEPENDS)

.PHONY: $(CLIENT_NAME)
.PHONY: $(SERVER1_NAME)
.PHONY: $(SERVER2_NAME)
.PHONY: $(SERVER3_NAME)
.PHONY: $(SERVER4_NAME)


.cpp.o:
	$(CXX) -c $(CPPFLAGS) $(CXXFLAGS) -o $@ $<

$(CLIENT_NAME): $(CLIENT_OBJ)
	$(PURIFY) $(CXXLD) $(CXXFLAGS) $(LDFLAGS) -o $@ \
	$(CLIENT_OBJ) $(ALL_LIBS)

$(SERVER1_NAME): $(SERVER1_OBJ)
	$(PURIFY) $(CXXLD) $(CXXFLAGS) $(LDFLAGS) -o $@ \
	$(SERVER1_OBJ)  $(ALL_LIBS)

$(SERVER2_NAME): $(SERVER2_OBJ)
	$(PURIFY) $(CXXLD) $(CXXFLAGS) $(LDFLAGS) -o $@ \
	$(SERVER2_OBJ) $(ALL_LIBS)

$(SERVER3_NAME): $(SERVER3_OBJ)
	$(PURIFY) $(CXXLD) $(CXXFLAGS) $(LDFLAGS) -o $@ \
	$(SERVER3_OBJ) $(ALL_LIBS)

$(SERVER4_NAME): $(SERVER4_OBJ)
	$(PURIFY) $(CXXLD) $(CXXFLAGS) $(LDFLAGS) -o $@ \
	$(SERVER4_OBJ) $(ALL_LIBS)

-include $(DEPENDS)
#generated rules use the following variables:
# CXX,
# CPPFLAGS,
# CXXFLAGS,
# IDL2CXX,
# IDLFLAGS, AND
# IDLMKINCLUDE
include idl_rules.mk

clean::
	-$(RM) *.o core
	-$(RM) $(CORBA_CL_ALL_SRC) $(CORBA_CL_ALL_HDRS)
	-$(RM) $(CORBA_SRV_ALL_SRC) $(CORBA_SRV_ALL_HDRS)
	-$(RM) $(DEPENDS)

cleanall: clean
	-$(RM) $(CLIENT_NAME) $(SERVER1_NAME) $(SERVER2_NAME) $(SERVER3_NAME) $(SERVER4_NAME)

# -----------------------------------------------
