/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>

// Cdmw Files
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Foundation/commonsvcs/naming/NamingUtil.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>
#include <FaultTolerance/ftcommon/FTObjectIds.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>
#include <FaultTolerance/idllib/CdmwFTReplicationManager.stub.hpp>
#include <Foundation/common/Assert.hpp>
#include <Foundation/common/Locations.hpp>

// Demo Files
#include "STPClientProcessBehaviour.hpp"
#include "STPCommon.hpp"



STPClientProcessBehaviour::STPClientProcessBehaviour(
    CORBA::ORB_ptr orb,
    PortableServer::POA_ptr rootPOA)
{
    m_orb = CORBA::ORB::_duplicate( orb );
    m_client_thread = NULL;
}

    
STPClientProcessBehaviour::~STPClientProcessBehaviour() throw ()
{
}

    
CORBA::ULong STPClientProcessBehaviour::nb_steps()
    throw( CORBA::SystemException )
{
    return 1;
}

void STPClientProcessBehaviour::on_initialise(
    const CdmwPlatformMngtBase::StartupKind& startup_kind )
throw( CORBA::SystemException )
{
   using namespace Cdmw::CommonSvcs::Naming;
   typedef NamingUtil<CosNaming::NamingContext> Util;

   INFO (" --- STP_TestDriver initialisation ...");

    try
    {
         // Resolve Admin Root Context
         CdmwNamingAndRepository::Repository_var nar = Cdmw::NamingAndRepository::RepositoryInterface::get_repository();
         CosNaming::NamingContext_var admin_root = nar->resolve_root_context(Cdmw::Common::Locations::ADMIN_ROOT_CONTEXT_ID);
         CDMW_ASSERT(!CORBA::is_nil(admin_root.in()));

         // Get State Transfer Perf Naming Context
         NamingInterface admin_ni(admin_root.in());
         CosNaming::NamingContext_var nc__ = Util::resolve_name(admin_ni, StateTransferPerf::STATE_TRANSFER_PERF_NAME);
         CDMW_ASSERT(!CORBA::is_nil(nc__.in()));
         m_state_transfer_perf =  CosNaming::NamingContextExt::_narrow(nc__.in());
         CDMW_ASSERT(!CORBA::is_nil(m_state_transfer_perf.in()));

         CORBA::Object_var obj =
            m_orb->resolve_initial_references(Cdmw::FT::REPLICATION_MANAGER_ObjectId);
         CDMW_ASSERT(!CORBA::is_nil(obj.in()));
         m_replication_manager =  CdmwFT::ReplicationManager::_narrow(obj.in());
         CDMW_ASSERT(!CORBA::is_nil(m_replication_manager.in()));
    } catch (const CORBA::SystemException& e) {
        INFO("STP_TestDriver : CORBA::SystemException raised !!! " << e);
        throw;
    } catch( const CORBA::Exception &e ) {
        INFO("STP_TestDriver : CORBA::Exception raised !!! " << e);
        throw;
    } catch( const Cdmw::Exception &e ) {
        INFO("STP_TestDriver : Cdmw::Exception raised !!! " << e.what());
        throw;
    } catch( ... ) {
        INFO("STP_TestDriver : Unknown Exception raised !!! ");
        throw;
    }

   ECHO (" --- STP_TestDriver initialisation ended.");
}
        
        
void STPClientProcessBehaviour::on_next_step()
    throw( CdmwPlatformMngt::ProcessDelegate::InvalidStep,
           CORBA::SystemException )
{
}

    
void STPClientProcessBehaviour::on_run()
    throw( CdmwPlatformMngt::ProcessDelegate::NotReadyToRun,
           CORBA::SystemException )
{
    INFO (" --- STP_TestDriver is runing...");


    unsigned int nb_iterations = StateTransferPerf::DEFAULT_NUMBER_OF_ITERATIONS;
   
    ECHO ("\nCurrent Configuration : " << nb_iterations << " iterations." << std::endl);

    try
    {
        // Launch the test in a thread
        m_client_thread = new STPClientThread(m_state_transfer_perf.in(),
                                              m_replication_manager.in(),
                                              nb_iterations);    

        m_client_thread->start();
    } catch (const CORBA::SystemException& e) {
        INFO("STP_TestDriver : CORBA::SystemException raised !!! " << e);
        throw;
    } catch( const CORBA::Exception &e ) {
        INFO("STP_TestDriver : CORBA::Exception raised !!! " << e);
        throw;
    } catch( const Cdmw::Exception &e ) {
        INFO("STP_TestDriver : Cdmw::Exception raised !!! " << e.what());
        throw;
    } catch( ... ) {
        INFO("STP_TestDriver : Unknown Exception raised !!! ");
        throw;
    }

}

    
void STPClientProcessBehaviour::on_stop()
    throw( CORBA::SystemException )
{
   INFO (" --- STP_TestDriver is stopping...");
        
    m_client_thread->stop();
    m_client_thread->join();
        
    delete m_client_thread;
    m_client_thread = 0;

   ECHO (" --- STP_TestDriver stopped.");

}

