/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Cdmw Files
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/commonsvcs/naming/NamingUtil.hpp>


// StateTransferPerf Files
#include "STPCommon.hpp"
#include <STPClientThread.hpp>
#include "STPServer_impl.hpp"
#include "STPServerAdmin_impl.hpp"

/*
 *  INFO
 *
 *  Names: 
 *     StateTransferPerf/<application>/<protocol>/<nb-worker-threads>/<hostname>
 *
 *  Files:
 *     <application>_<protocol>_<nb-worker-threads>_<nb-backup>
 *
 *  (nb-replica == 1) ==> primary server only, perform inserts()
 *  (nb-replica > 1) ==> perform update (nb-records / 2)
 *
 *
 *  foreach application do {
 *      foreach protocol do {
 *          foreach nb-worker-threads do {
 *             get nb of nodes
 *             foreach node do {
 *                 get server info
 *                 openfile("<application>_<protocol>_<nb-worker-threads>_<nb-backup>.insertion");
 *                 add_member() ; // primary init then backup insertion
 *                 store time in file
 *                 close file.
 *                 foreach nb-iterations do {
 *                     server_group->update_record(nb-records / 2);
 *                 }
 *                 openfile("<application>_<protocol>_<nb-worker-threads>_<nb-backup>");
 *                 store response times
 *                 closefile();
 *             }
 *          }
 *      }
 *  }
 *
 */

void STPClientThread::run() throw()
{
    try {
        using namespace Cdmw::CommonSvcs::Naming;
        typedef NamingUtil<CosNaming::NamingContext> Util;

        //for (int i=0 ; i< 100 ; i++) sleep(100);
        sleep(30000);
        INFO("");
        system("date");
        INFO("Starting State Transfer Performance tests ...");
        sleep(5000);

        // Get State Transfer Perf Naming Interface
        NamingInterface stp_ni(m_root_context.in());

        // Get all applications
        NamingInterface::BindingList application_blist;
        stp_ni.list_context(application_blist);
        for (NamingInterface::BindingList::const_iterator application_iter = application_blist.begin();
            application_iter != application_blist.end();
            ++application_iter) {
            m_application_name = NamingInterface::to_string((*application_iter).binding_name);
            INFO("");
            INFO("*** Application \t: " << m_application_name << " (" << m_nb_iterations << ")");
            CosNaming::NamingContext_var nc__ = Util::resolve_name(stp_ni, m_application_name);
            NamingInterface application_ni(nc__.in());

            // Get all protocols
            NamingInterface::BindingList protocol_blist;
            application_ni.list_context(protocol_blist);
            for (NamingInterface::BindingList::const_iterator protocol_iter = protocol_blist.begin();
                 protocol_iter != protocol_blist.end();
                 ++protocol_iter) {
                std::string protocol_name = NamingInterface::to_string((*protocol_iter).binding_name);
                INFO("");
                INFO("*** Protocol    \t: " << protocol_name);
                CosNaming::NamingContext_var nc__ = Util::resolve_name(application_ni,protocol_name);
                NamingInterface protocol_ni(nc__.in());

                // Get all Numbers of Worker Threads
                NamingInterface::BindingList nb_worker_threads_blist;
                protocol_ni.list_context(nb_worker_threads_blist);
                for (NamingInterface::BindingList::const_iterator nb_worker_threads_iter = nb_worker_threads_blist.begin();
                     nb_worker_threads_iter != nb_worker_threads_blist.end();
                     ++nb_worker_threads_iter) {
                    std::string nb_worker_threads_name = NamingInterface::to_string((*nb_worker_threads_iter).binding_name);
                    INFO("");
                    INFO("*** Nb Threads  \t: " << nb_worker_threads_name);
                    // Server Group Interface
                    CosNaming::NamingContext_var nc__ = Util::resolve_name(protocol_ni,nb_worker_threads_name);
                    NamingInterface nb_worker_threads_ni(nc__.in());

                    // Compute State Transfer Perf
                    DoStateTransferPerf(nb_worker_threads_ni);
                }
            }
       }

       INFO("");
       INFO("State Transfer Performance tests ended.");
       system("date");
       
    } catch (const CORBA::SystemException& e) {
        INFO("STP_TestDriver : CORBA::SystemException raised !!! " << e);
        throw;
    } catch( const CORBA::Exception &e ) {
        INFO("STP_TestDriver : CORBA::Exception raised !!! " << e);
        throw;
    } catch( const Cdmw::Exception &e ) {
        INFO("STP_TestDriver : Cdmw::Exception raised !!! " << e.what());
        throw;
    } catch( ... ) {
        INFO("STP_TestDriver : Unknown Exception raised !!! ");
        throw;
    }
}   



/*
 *  For each Server Hostname, compute State Transfer Perf
 */

void STPClientThread::DoStateTransferPerf(Cdmw::CommonSvcs::Naming::NamingInterface & ni) throw()
{

    try {
       using namespace Cdmw::CommonSvcs::Naming;
       typedef NamingUtil<CosNaming::NamingContext> Util;

       int nb_backup = 0;
       unsigned int primary_update_time = 0;
       unsigned int first_backup_update_time = 0;

       FT::ObjectGroup_var ft_group;
       NamingInterface::BindingList hostname_blist;
       ni.list_context(hostname_blist);
       NamingInterface::BindingList::const_iterator host_iter = hostname_blist.begin();
       while( host_iter != hostname_blist.end() )
       {
           sleep(1000);
           // Retrieve server hostname
           std::string hostname = NamingInterface::to_string((*host_iter).binding_name);
           ECHO("");
           ECHO("*** Hostname     \t: " << hostname << " ***");
           CORBA::Object_var obj = ni.resolve(hostname);
           CDMW_ASSERT(!CORBA::is_nil(obj.in()));

           // Retrieve Server Admin
           StateTransferPerf::ServerAdmin_var sa = StateTransferPerf::ServerAdmin::_narrow(obj.in());
           CDMW_ASSERT(!CORBA::is_nil(sa.in()));

           // Retrieve Server Configuration
           ECHO("    - get_configuration()");
           StateTransferPerf::ServerConfig_var scfg = sa->get_configuration();

           std::string protocol_name = (scfg->wire_protocol == StateTransferPerf::MIOP) ? "MIOP" : "IIOP";

           ECHO("application_name    : " << m_application_name);
           ECHO("server_name         : " << scfg->server_name.in());
           ECHO("group_name          : " << scfg->server_group_name.in());
           ECHO("wire_protocol       : " << protocol_name);
           ECHO("nb_worker_threads   : " << scfg->nb_worker_threads);
           ECHO("nb_records          : " << scfg->nb_records);

           ECHO("    - get_server()");
           StateTransferPerf::Server_var server = sa->get_server();

           ECHO("    - get_location()");
           CosNaming::Name_var location = sa->get_location();
           ECHO("          " << NamingInterface::to_string(location));

           Cdmw::OsSupport::OS::Timeval beg_time;
           Cdmw::OsSupport::OS::Timeval end_time;

           // Create log file : <server_name>_<protocol_name>_<nb_worker_threads>_<nb_backup>.txt
           std::ostringstream oBuf;
           //oBuf << m_application_name << "_" << protocol_name << "_" << scfg->nb_worker_threads << "_" << nb_backup << ".txt";
           oBuf << m_application_name << "_" << protocol_name << "_" << scfg->nb_worker_threads;
           std::string file2 = oBuf.str() + ".csv";
           oBuf << "_" << nb_backup;
           std::string file1 = oBuf.str() + ".csv";

           ECHO("    - create STP file : " << file1);
           std::ofstream f1_str (file1.c_str(), std::ios_base::out);
           //f1_str << "###############################################################" << std::endl;
           //f1_str << "# STP FILE  : " << file1 << std::endl;
           //f1_str << "###############################################################" << std::endl;
           //f1_str << "#" << std::endl;

           // If first host, get group reference
           if ( host_iter == hostname_blist.begin() ) {
               std::string group = scfg->server_group_name.in();
               ECHO("    - retrieve State Transfer Perf group : " << group);
               CosNaming::Name_var group_name = NamingInterface::to_name(group);
               ft_group = m_replication_manager->get_object_group_ref_from_name(group_name.in());
               CDMW_ASSERT(!CORBA::is_nil(ft_group.in()));

               ECHO("    - create STP file : " << file2);
               std::ofstream f2_str (file2.c_str(), std::ios_base::out);
               //f2_str << "########################################################" << std::endl;
               //f2_str << "# STP FILE  : " << file2 << std::endl;
               //f2_str << "########################################################" << std::endl;
               //f2_str << "#" << std::endl;
               //f2_str << "# NumberOfHosts;AverageUpdateTime (microseconds)" << std::endl; 
               f2_str << "Average Update Time (microseconds) " << protocol_name << " " << scfg->nb_worker_threads << " Threads" << std::endl;

               f2_str.close();
           }


           // Join group
           ECHO("    - add member " << scfg->server_name.in() << " to " << scfg->server_group_name.in());
           beg_time = Cdmw::OsSupport::OS::get_time();
           ft_group = m_replication_manager->add_member(ft_group.in(), location.in(), server.in());
           end_time = Cdmw::OsSupport::OS::get_time();
           //trace_perf(f1_str, "# Add Member (microseconds) ", beg_time, end_time); 

           StateTransferPerf::Server_var server_group = StateTransferPerf::Server::_narrow(ft_group.in());
           CDMW_ASSERT(!CORBA::is_nil(server_group.in()));
           // Handle first server -> Primary Server
           if ( host_iter == hostname_blist.begin() ) {
               // Perform initialise
               ECHO("    - insert  " << scfg->nb_records << " records in datastore");
               beg_time = Cdmw::OsSupport::OS::get_time();
               server_group->initialise(scfg->nb_records);
               end_time = Cdmw::OsSupport::OS::get_time();
               //trace_perf(f1_str, "# Initialise (microseconds) ", beg_time, end_time);
               sleep(2000);
           }

           // Perform NB_ITERATIONS update on a datastore record
           f1_str << "Update Time (microseconds) " << protocol_name << " " << scfg->nb_worker_threads << " Threads " << nb_backup << " Backup" << std::endl;
           ECHO("    - update a datastore record (" << m_nb_iterations << " iterations)");
           unsigned long update_time = 0;
           for ( int i=0 ; i<m_nb_iterations ; i++) {
               beg_time = Cdmw::OsSupport::OS::get_time();
               server_group->update_record(scfg->nb_records/2);
               end_time = Cdmw::OsSupport::OS::get_time();
               f1_str << get_duration(beg_time, end_time) << std::endl; 
               update_time += get_duration(beg_time, end_time);
           }

           if ( primary_update_time == 0 ) {
               primary_update_time = update_time;
           } else if ( first_backup_update_time == 0 ) {
                first_backup_update_time = update_time;
           }

           int  pourcent1 = ( primary_update_time == 0 ) ? 0 : (100 * update_time) / primary_update_time  - 100;
           int  pourcent2 = ( first_backup_update_time == 0 ) ? 0 : (100 * update_time) / first_backup_update_time  - 100;

           ECHO("");
           ECHO(" ---> Average Update Time (microseconds) : " << update_time/m_nb_iterations << "\t( " << pourcent1 << " % )");
           INFO("  " << m_application_name << " " << protocol_name << " " << scfg->nb_worker_threads
                     << "  " << hostname << "     \t: " << update_time/m_nb_iterations
                     << "\t( " << pourcent1 << " % )   "
                     << "\t( " << pourcent2 << " % )");

           std::ofstream f2_str (file2.c_str(), std::ios_base::out|std::ios_base::app);
           f2_str << update_time/m_nb_iterations << std::endl;

           // Next host
           host_iter++;
           nb_backup++;
           f1_str.close();
           f2_str.close();
       }
    } catch (const CORBA::SystemException& e) {
        INFO("STP_TestDriver : CORBA::SystemException raised !!! " << e);
        throw;
    } catch( const CORBA::Exception &e ) {
        INFO("STP_TestDriver : CORBA::Exception raised !!! " << e);
        throw;
    } catch( const Cdmw::Exception &e ) {
        INFO("STP_TestDriver : Cdmw::Exception raised !!! " << e.what());
        throw;
    } catch( ... ) {
        INFO("STP_TestDriver : Unknown Exception raised !!! ");
        throw;
    }
}


unsigned long STPClientThread::get_duration(
        Cdmw::OsSupport::OS::Timeval& beg_time,
        Cdmw::OsSupport::OS::Timeval& end_time)
{
    // Duration in microseconds
    unsigned long duration = ((end_time.seconds - beg_time.seconds) * 1000000) + (end_time.microseconds - beg_time.microseconds);
    
    return duration;
}
