/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Cdmw Files
#include <SystemMngt/platforminterface/PlatformInterface.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
// StateTransferPerf Files
#include "STPServer_impl.hpp"
#include "STPCommon.hpp"


STPServer_impl::STPServer_impl(CORBA::ORB_ptr orb,
                                     PortableServer::POA_ptr poa)
   :  m_FTManagerDataStorageHome(StateTransferPerf::DATA_STORE_IDENTIFIER),
      m_orb(CORBA::ORB::_duplicate(orb)),
      m_poa(PortableServer::POA::_duplicate(poa))
{
    m_process_name     = Cdmw::PlatformMngt::PlatformInterface::Get_process_name();
}


STPServer_impl::~STPServer_impl()
throw()
{
}


PortableServer::POA_ptr STPServer_impl::_default_POA()
{
    return PortableServer::POA::_duplicate(m_poa.in());
}



void STPServer_impl::initialise (const CORBA::Long nb_records)
    throw ( CORBA::SystemException )
{
    ECHO(m_process_name << ": *** Inserting " << nb_records << " records in DataStore ...");
    for (int i=0; i< nb_records ; i++) {

        StateTransferPerf::STP_FTManagerData_var data = new StateTransferPerf::STP_FTManagerData;
        data->id = i;
        data->version = 0;

        try {
            data->ogd.alias = "STATE_TRANSFER_PERF FTManager";
            data->ogd.repository_id = "IDL:thalesgroup.com/StateTransferPerf/Server:1.0";
            data->ogd.stateness = StateTransferPerf::GroupStateness_STATEFULL;
            data->ogd.state = StateTransferPerf::ObjectGroupState_FULLY_REDUNDANT;
            data->ogd.fsm_internal_state = 12;
            data->ogd.group_id = 12;
            data->ogd.object_group_ref = CORBA::Object::_nil();
            data->ogd.object_group_ref_version = 12;
            data->ogd.min_nb_replicas = 2;
            CosNaming::Name_var stp_name = Cdmw::CommonSvcs::Naming::NamingInterface::to_name("dummy.location");
            data->ogd.members = StateTransferPerf::MemberOverallInfos(20);
            data->ogd.members.length(20);
            for (CORBA::ULong j=0L; j< 20; j++) {
                data->ogd.members[j].local_id = 1;
                data->ogd.members[j].the_reference = CORBA::Object::_nil();
                data->ogd.members[j].the_location = stp_name.in();
                data->ogd.members[j].the_state = StateTransferPerf::MemberState_INITIAL;
                data->ogd.members[j].the_internal_fsm_state = StateTransferPerf::MemberState_INITIAL;
            }

            data->ogd.creation_properties = STP_FT::Properties::Properties(5);
            data->ogd.dynamic_properties = STP_FT::Properties::Properties(5);
            data->ogd.creation_properties.length(5);
            data->ogd.dynamic_properties.length(5);
            for (CORBA::ULong j=0L; j< 5; j++) {
                data->ogd.creation_properties[j].nam = stp_name.in();
                data->ogd.creation_properties[j].val <<= data->ogd.creation_properties[j].nam;
                data->ogd.dynamic_properties[j].nam = stp_name.in();
                data->ogd.dynamic_properties[j].val <<= data->ogd.dynamic_properties[j].nam;
            }

            data->ogd.data_store_ids = STP_FT::StateTransfer::DataStoreIDs::DataStoreIDs (5);
            data->ogd.data_store_ids.length(5);
            for (CORBA::ULong j=0L; j< 5; j++) {
                data->ogd.data_store_ids[j] = 12;
            }

            data->ogd.last_event = StateTransferPerf::LastEventInfos(5);
            data->ogd.last_event.length(5);
            for (CORBA::ULong j=0L; j< 5; j++) {
                data->ogd.last_event[j].last_event = StateTransferPerf::EVT_CREATE_GROUP;
                data->ogd.last_event[j].the_location = stp_name.in();
                data->ogd.last_event[j].event_context.client_id = "STATE_TRANSFER_PERF";
                data->ogd.last_event[j].event_context.retention_id = 12;
                data->ogd.last_event[j].event_context.expiration_time = 12;
            }

            data->ogd.init_reply_logging = true; 

            // Create a new storageobject.
            STP_FTManagerDataStorageObject obj = m_FTManagerDataStorageHome.create(data->id, data);
        } catch (const std::exception & e) {
            INFO(m_process_name << ": Caught std::exception during insertion of DataStore ["
                                << i << "] - version = " << data->version << " : " << e.what());
            throw;
        } catch (const CORBA::Exception &e) {
            INFO(m_process_name << ": Caught CORBA::Exception during insertion of DataStore ["
                                << i << "] - version = " << data->version << " : " << e);
            throw;
        } catch (...) {
            INFO(m_process_name << ": Caught Unknown Exception during insertion of DataStore ["
                                << i << "] - version = " << data->version);
        }
        
   }
}

void STPServer_impl::update_record (const CORBA::Long record_number)
    throw ( CORBA::SystemException )
{

   StateTransferPerf::Ident id = record_number;
    
   try {
      // find if it is already in data store
      STP_FTManagerDataStorageObject obj = m_FTManagerDataStorageHome.find_by_oid(id);
      StateTransferPerf::STP_FTManagerData_var data = obj.get();
      unsigned long version = data->version;
      version++;
      data->version = version;
      ECHO(m_process_name << ": *** Updating FTManager DataStore [" << record_number << "] - version = " << data->version << " ...");
      obj.set(data);
   } catch (const std::exception & e) {
       INFO(m_process_name << ": Caught std::exception during update of DataStore ["
                           << record_number << "] : " << e.what());
       throw;
   } catch (const CORBA::Exception &e) {
       INFO(m_process_name << ": Caught CORBA::Exception during update of DataStore ["
                           << record_number << "] : " << e);
       throw;
   } catch (...) {
       INFO(m_process_name << ": Caught Unknown Exception during update of DataStore ["
                           << record_number << "]" );
   }
}



