/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Cdmw Files
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <FaultTolerance/ftinit/FTServiceInit.hpp>
#include <ConfAndPlug/cdmwinit/CdmwInterface.hpp>

// Demo Files
#include <STPSupervisionServerProcessBehaviour.hpp>
#include "STPCommon.hpp"


static const int POA_THREAD_POOL_SIZE = 10;

static const std::string POINT_TO_POINT_OPTION = "--PointToPoint";
static const std::string NB_WORKER_THREADS_OPTION = "--NbWorkerThreads";


int main( int argc, char* argv[] )
{

    CORBA::ORB_var orb;   

    // Check if 'nb-worker-threads' option is specified
    std::string nb_worker_threads_option = Cdmw::OsSupport::OS::get_option_value(
        argc, argv, NB_WORKER_THREADS_OPTION );
    std::string nb_worker_threads = (nb_worker_threads_option == "no") ? "1" : nb_worker_threads_option;

    // Check if 'point-to-point' option is specified
    std::string point_to_point_option = Cdmw::OsSupport::OS::get_option_value(
        argc, argv, POINT_TO_POINT_OPTION );
    bool point_to_point = (point_to_point_option == "no") ? false : true;
    std::string protocol = point_to_point ? "IIOP" : "MIOP";

    std::string process_name = basename(argv[0]);
    process_name +=  "_" + nb_worker_threads;
        std::cout << process_name << ": *** Nb Worker Threads = " << nb_worker_threads_option 
                  << "  -  Wire Protocol = " << protocol << " ***" << std::endl;

    try
    {
        // Initialise FT service (to be done before ORB_init)
        //std::cout << process_name << ": initialise FT service" << std::endl;
        Cdmw::CdmwInit::CDMW_preORBInit(argc, argv);

        // Initialise the ORB
        //std::cout << process_name << ": initialise the ORB" << std::endl;
        Cdmw::OrbSupport::StrategyList orb_strategies;
        orb_strategies.add_OrbThreaded();
        orb_strategies.add_PoaThreadPool(POA_THREAD_POOL_SIZE);
        //if (!point_to_point) orb_strategies.add_multicast();
        orb_strategies.add_multicast();

        orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, orb_strategies);

        // Get the root POA            
        CORBA::Object_var obj = orb -> resolve_initial_references("RootPOA");
        CDMW_ASSERT(!CORBA::is_nil(obj.in()));
        PortableServer::POA_ptr rootPOA = PortableServer::POA::_narrow(obj.in());

        // Run implementation (Activate the POA manager)
        PortableServer::POAManager_var poaManager = rootPOA -> the_POAManager();
        poaManager -> activate();
        

        // creates the supervision process behaviour
        std::auto_ptr<STPSupervisionServerProcessBehaviour> pSupervisionProcessBehaviour(
            new STPSupervisionServerProcessBehaviour( orb.in(), protocol, nb_worker_threads ) );

        // init Cdmw
        //std::cout << process_name << ": initialise CDMW" << std::endl;
        Cdmw::CdmwInit::CDMW_init( orb.in(), argc, argv, pSupervisionProcessBehaviour.get() );

        // Enter in the main loop
        //std::cout << process_name << ": orb->run()" << std::endl;
        orb->run();
    } catch( const CORBA::Exception &e ) {
        std::cerr << std::endl << process_name << " : " << __FILE__ << " " << __LINE__ 
                  << " : CORBA::Exception raised !!! " << e._name() << std::endl;
        return EXIT_FAILURE;
    } catch( const Cdmw::Exception &e ) {
        std::cerr << std::endl << process_name << " : " << __FILE__ << " " << __LINE__ 
                  << " : FAILURE : Cdmw::Exception raised !!! " << e.what() << std::endl;
        return EXIT_FAILURE;
    } catch( ... ) {
        std::cerr << std::endl << process_name << " : " << __FILE__ << " " << __LINE__ 
                  << " : Unexpected exception raised !!!  " << std::endl;
        return EXIT_FAILURE;
    }

    return EXIT_SUCCESS;
}




