/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Cdmw Files
#include <SystemMngt/platforminterface/PlatformInterface.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
// StateTransferPerf Files
#include "STPSupervisionServer_impl.hpp"
#include "STPCommon.hpp"


STPSupervisionServer_impl::STPSupervisionServer_impl(CORBA::ORB_ptr orb,
                                                     PortableServer::POA_ptr poa)
   :  m_SupervisionDataStorageHome(StateTransferPerf::DATA_STORE_IDENTIFIER_SUPERVISION),
      m_orb(CORBA::ORB::_duplicate(orb)),
      m_poa(PortableServer::POA::_duplicate(poa))
{
    m_process_name     = Cdmw::PlatformMngt::PlatformInterface::Get_process_name();
}


STPSupervisionServer_impl::~STPSupervisionServer_impl()
throw()
{
}


PortableServer::POA_ptr STPSupervisionServer_impl::_default_POA()
{
    return PortableServer::POA::_duplicate(m_poa.in());
}



void STPSupervisionServer_impl::initialise (const CORBA::Long nb_records)
    throw ( CORBA::SystemException )
{
    ECHO(m_process_name << ": *** Inserting " << nb_records << " records in Supervision DataStore ...");
    unsigned long dummy_counter=0;

    // build the ProcessInfo data for the process to isert in datastore
    STP_SMG::ProcessInfo process_info;
    process_info.process_exe = CORBA::string_dup ("corba dummy string");
    process_info.process_port = ::atoi ("123");
    process_info.working_directory = CORBA::string_dup ("corba dummy string");
    process_info.process_args = CORBA::string_dup ("corba dummy string");
    process_info.exit_autorestart_info.default_exit_action = STP_SMG::NO_ACTION;
    process_info.exit_autorestart_info.exit_actions.length(0);
    process_info.exit_autorestart_info.max_nbr_reached_action =  STP_SMG::NO_ACTION;
    process_info.exit_autorestart_info.abnormal_behaviour_action =  STP_SMG::NO_ACTION;
    process_info.requested_restart_warmth=  STP_SMG::HOT_RESTART;
    process_info.filters =  CORBA::string_dup ("");
    process_info.memory_resident =  false;
    process_info.core_dump_size =  0;
    process_info.configuration_data.run_as_root = false;
    process_info.configuration_data.generate_core = true;

    // build the processProcessInfo data for the process to isert in datastore
    STP_SMG::CompleteMonitoringInfo processMonitoringInfo;

    processMonitoringInfo.monitoring_model = STP_SMG::NO_MONITORING;
    processMonitoringInfo.pull_monitoring_interval = 2000;
    processMonitoringInfo.pull_monitoring_timeout = 6000;
    processMonitoringInfo.push_monitoring_interval = 0;
    processMonitoringInfo.push_monitoring_timeout = 0;




    for (int i=0; i< nb_records ; i++) {

        dummy_counter++;


        StateTransferPerf::STP_SupervisionData_var data = new StateTransferPerf::STP_SupervisionData;
        data->id = i;
        data->version = 0;

        try {
            data->process_record.host_name = CORBA::string_dup("corba dummy string");
            data->process_record.type = STP_SMG::MANAGED_PROCESS;
            data->process_record.auto_ending = true;
            data->process_record.process_info = process_info;


            // Set the life cycle
            STP_SMG::LifeCycleDef_var lifeCycleDef = new STP_SMG::LifeCycleDef();
            {
                // Add all the init step
                unsigned int initStepCount = 100;
                lifeCycleDef->init_steps.length( initStepCount );
                for( unsigned int initStepIndex=0;
                     initStepIndex<initStepCount;
                     initStepIndex++ )
                {
                    // Get an init step
                    STP_SMG::Step_var initStep = new STP_SMG::Step();
                        
                    initStep->step_label = CORBA::string_dup("corba dummy string");
                    initStep->step_timeout = 1000;

                    // Checks there is no step redefinition
                    for( unsigned int l_initStepIndex = 0; l_initStepIndex < initStepIndex; l_initStepIndex++ )
                    {
                        std::string initStepName = lifeCycleDef->init_steps[ l_initStepIndex ].step_label.in();
                    }

                    // Add this init step
                    lifeCycleDef->init_steps[ initStepIndex ] = initStep.in();
                }

                // Set the startup and stop timeout
                lifeCycleDef->run_timeout = 1000;
                lifeCycleDef->stop_timeout = 1000;
            }

            data->process_record.life_cycle = lifeCycleDef.in();



            data->process_record.monitoring_info = processMonitoringInfo;

            data->process_record.status = STP_SMG::PROCESS_INITIALISING;
            data->process_record.previous_state_name = STP_SMG::PROCESS_INITIAL;
            data->process_record.status_info = "dummy string";
            data->process_record.state_machine = "dummy string";

            data->process_record.startup_kind.state_retrieval = STP_SMG::SPECIFIED_STATE;
            data->process_record.startup_kind.startup_mode = 0;
            data->process_record.startup_kind.specified_state_id = "dummy string";

            data->process_record.step_counter = dummy_counter;
            data->process_record.last_request_status = STP_SMG::REQUEST_SUCCEED;

            data->process_record.last_error_header.time_stamp.year = 2008;
            data->process_record.last_error_header.time_stamp.month = 2;
            data->process_record.last_error_header.time_stamp.day = 26;
            data->process_record.last_error_header.time_stamp.hour = 12;
            data->process_record.last_error_header.time_stamp.minute = 0;
            data->process_record.last_error_header.time_stamp.second = 0;
            data->process_record.last_error_header.time_stamp.millisecond = 0;
            data->process_record.last_error_header.time_stamp.microsecond = 0;
            data->process_record.last_error_header.level = STP_SMG::INF;
            data->process_record.last_error_header.event_key.primary_key = 0;
            data->process_record.last_error_header.event_key.seconds = 0;
            data->process_record.last_error_header.event_key.microseconds = 0;
            data->process_record.last_error_header.event_key.counter_inx = 0;


            data->process_record.last_error_issuer = "dummy string";
            data->process_record.last_exit_code.exit_event = STP_SMG::EXITED;
            data->process_record.last_exit_code.event_value = 1000;
            data->process_record.last_exit_code.core_dumped = STP_SMG::NO_CORE_GENERATED;
            data->process_record.current_restart_attempt = dummy_counter;
            data->process_record.exit_autorestart_info.max_nbr_reached_action = STP_SMG::NO_ACTION;
            data->process_record.exit_autorestart_info.abnormal_behaviour_action = STP_SMG::NO_ACTION;

            data->process_record.exit_autorestart_info.exit_actions.length(2);
            STP_SMG::ExitAction ea0, ea1;
            ea0.exit_code=1;
            ea0.exit_action=STP_SMG::NO_ACTION;
            data->process_record.exit_autorestart_info.exit_actions[0]=ea0;
            ea1.exit_code=4;
            ea1.exit_action=STP_SMG::RESTART_PROCESS;
            data->process_record.exit_autorestart_info.exit_actions[1]=ea1;



            data->process_record.exit_autorestart_info.default_exit_action = STP_SMG::NO_ACTION;
            data->process_record.indicator_relevant_status = true;
            data->process_record.created_dinamically = true;
            data->process_record.pid = (CORBA::Long) 4711;
            data->process_record.previous_pid = (CORBA::Long) 4712;
            data->process_record.created_dynamically = true;
            data->process_record.initialisation_timeout = 1000;
            data->process_record.kind_start = StateTransferPerf::DEFA;


            // Create a new storageobject.
            STP_SupervisionDataStorageObject obj = m_SupervisionDataStorageHome.create(data->id, data);
        } catch (const std::exception & e) {
            INFO(m_process_name << ": Caught std::exception during insertion of Supervision DataStore ["
                                << i << "] - version = " << data->version << " : " << e.what());
            throw;
        } catch (const CORBA::Exception &e) {
            INFO(m_process_name << ": Caught CORBA::Exception during insertion of Supervision DataStore ["
                                << i << "] - version = " << data->version << " : " << e);
            throw;
        } catch (...) {
            INFO(m_process_name << ": Caught Unknown Exception during insertion of Supervision DataStore ["
                                << i << "] - version = " << data->version);
        }
        
   }
}

void STPSupervisionServer_impl::update_record (const CORBA::Long record_number)
    throw ( CORBA::SystemException )
{

   StateTransferPerf::Ident id = record_number;
    
   try {
      // find if it is already in data store
      STP_SupervisionDataStorageObject obj = m_SupervisionDataStorageHome.find_by_oid(id);
      StateTransferPerf::STP_SupervisionData_var data = obj.get();
      unsigned long version = data->version;
      version++;
      data->version = version;
      ECHO(m_process_name << ": *** Updating Supervision DataStore [" << record_number << "] - version = " << data->version << " ...");
      obj.set(data);
   } catch (const std::exception & e) {
       INFO(m_process_name << ": Caught std::exception during update of Supervision DataStore ["
                           << record_number << "] : " << e.what());
       throw;
   } catch (const CORBA::Exception &e) {
       INFO(m_process_name << ": Caught CORBA::Exception during update of Supervision DataStore ["
                           << record_number << "] : " << e);
       throw;
   } catch (...) {
       INFO(m_process_name << ": Caught Unknown Exception during update of Supervision DataStore ["
                           << record_number << "]" );
   }
}



