/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @author Raffaele Mele <rmele@progesi.it>
 * @brief TimeAdmin - Federated Controlled Clock demo
 */

//package cdmw.clock.federated;

import org.omg.CORBA.ORB;
import org.omg.CORBA.SystemException;
import org.omg.CORBA.ORBPackage.InvalidName;
import org.omg.CosClockService.*;

import org.omg.CosClockService.ClockCatalogPackage.ClockEntry;
import org.omg.CosClockService.ClockCatalogPackage.UnknownEntry;
import org.omg.CosClockService.ControlledClockHelper;

import cdmw.clock.*;
import cdmw.common.BadParameterException;
import cdmw.orbsupport.ORBSupport;
import cdmw.orbsupport.StrategyList;
import cdmw.ossupport.OS;

class ORBWorker extends Thread
{
   private ORB m_ORB;

   public ORBWorker(ORB orb)
   {
      m_ORB = orb;
   }

   public void run()
   {
      try
      {
         m_ORB.run();
      }
      catch (SystemException ex)
      {
         ex.printStackTrace();
      }
   }
}

public class TimeReader
{

   private static ORB m_orb;

   private static ClockCatalog m_catalog;

   private static ControlledClock m_fedClock, m_ctrlClock;

   public static void main(String[] args)
   {
    try 
    {
        StrategyList strategyList = new StrategyList();
      strategyList.addORBThreaded();
      try
      {
         strategyList.addPOAThreadPerConnection();
      }
      catch (BadParameterException e)
      {
         e.printStackTrace();
         System.exit(-1);
      }
      
      String[] strArgs= {
           //   "--CdmwLocalisationService=4883" ,
           //   "-ORBInitRef" ,
           //   "LocalClock=corbaloc::localhost:4883/LocalClock",
           //   "-ORBInitRef" ,
           //   "ClockService=corbaloc::localhost:4883/ClockService" ,
          //    "--CdmwFederatedCtrlClock",
              "", // reserve 2 places for Property Service arguments
              ""
          };

 //     m_orb =  org.omg.CORBA.ORB.init(args, null);
      m_orb = ORBSupport.ORBInit(args, strategyList);
      
      // init Cardamom Clock Service.
//      if (!CdmwCosClock.init(m_orb))
//      {
//         System.out.println("Error on init clock service!");
//         ORBSupport.ORBCleanup(m_orb);
//         System.exit(-1);
//      }
 
      // ORB in a thread
      ORBWorker orb_worker = new ORBWorker(m_orb);
      orb_worker.start();

      try
      {
         OS.sleep(2000);
      }
      catch (InterruptedException e)
      {
         e.printStackTrace();
         System.exit(-1);
      }
      //CdmwCosClock.init(m_orb,args);
      org.omg.CORBA.Object obj = null;
      try
      {
         //obj = m_orb.string_to_object("corbaloc::192.168.7.166:60876/ClockService");
         obj = m_orb.resolve_initial_references("ClockService");
      }
      catch (Exception e1)//(InvalidName e1)
      {
         e1.printStackTrace();
         System.exit(-1);
      }
      if (obj == null) 
         System.out.println("Error : clock catalog null ");
      //System.out.println("[IOR ] " + m_orb.object_to_string(obj));
      m_catalog = ClockCatalogHelper.narrow(obj);
      if (m_catalog == null)
      {
         System.out.println("Error : Unable to resolve clock catalog");
      }

      ClockEntry entry = null;
      try
      {

         entry = m_catalog.get_entry("FederatedControlledExecutor");
         m_fedClock = ControlledClockHelper.narrow(entry.subject);
      }
      catch (UnknownEntry ex)
      {
         System.out.println("ClockCatalog::UnknownEntry");
      }

      if (m_fedClock == null)
      {
         System.err.println("Unable to resolve FederatedControlledClock, Aborting!");
         ORBSupport.ORBCleanup(m_orb);
         System.exit(-1);
      }
/*
      try
      {
         entry = m_catalog.get_entry("FederatedControlledExecutor");
      }
      catch (UnknownEntry e)
      {
         e.printStackTrace();
         System.exit(-1);
      }

      m_ctrlClock = ControlledClockHelper.narrow(entry.subject);

      if (m_ctrlClock == null)
      {
         System.err
               .println("Unable to resolve FederatedControlledClock, Aborting!");
         System.exit(-1);
      }
*/
      while ( m_fedClock !=null)
      { 
         try
         {
            System.out.println(">> Current Time: " + m_fedClock.current_time());
         }
         catch (TimeUnavailable e)
         {
            e.printStackTrace();
            System.exit(-1);
         }
         try
         {
            OS.sleep(2000);
         }
         catch (InterruptedException e)
         {
            e.printStackTrace();
            System.exit(-1);
         }
      }
      System.exit(0);
      }   catch(org.omg.CORBA.TRANSIENT t)
         {
       System.out.println("ATTENTION !!!! Clock Service is not avaiable" );             
            System.exit(-1);
         }
   }
}
