#!/bin/sh
# =====================================================================
# This file is part of CARDAMOM (R) which is jointly developed by THALES
# and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
# Copyright (C) THALES 2000-2003. All rights reserved.
# Copyright (C) THALES 2004-2005. All rights reserved.
# Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
# All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
# License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

# define macro echo
if test "`echo -e xxx`" = "xxx"
then
    echo="echo -e"
else
    echo=echo
fi


SITE_CONFIG=../site.mk
if [[ ! -f "$SITE_CONFIG" ]]
then
    $echo "$0: $SITE_CONFIG file is missing. Run '../configure.sh' and restart $0"
    exit 1
fi

# CDMW installation folder
CDMW_HOME="`cat $SITE_CONFIG | grep CDMW_HOME |cut -d= -f2`"
LD_LIBRARY_PATH=$CDMW_HOME/lib/c++:$LD_LIBRARY_PATH
export LD_LIBRARY_PATH
PATH=$CDMW_HOME/bin:$PATH
export PATH

# Target dependent build directory name
BUILD_TARGET=`$CDMW_HOME/bin/config.guess`

# Macro defining the used terminal
if [[ ! -x `which $TERM` ]]
then
    TERM=xterm 
fi
TERM_PATH=`which $TERM`

# Macro defining the input arguments
CLOCK_NAME=$1
PORT1=18881 
PORT2=18882 
PORT3=18883 

# Check for already executing timeout handlers, time readers and time admins
PIDS_TH=`/sbin/pidof TimeoutHandler`
PIDS_TA=`/sbin/pidof TimeAdmin`
if [[ ("$PIDS_TH" != "") || ("$PIDS_TR" != "") || ("$PIDS_TA" != "") ]]
then
    $echo "$0: timeout handlers or time readers or time admins already running"
    $echo "$0: demo aborted"
    exit 1
fi

# Run the demo
$echo "========================================================="
$echo "=                  CDMW DEMO                           ="
$echo "========================================================="
$echo "Hit a key when ready."
read FOO
echo "==========================================================="

# 1) Start timeout handlers 
$echo Starting the timeout handlers...

exec $TERM -ls -sb -sl 1000 -geometry 50X10+000+000 -display $DISPLAY -title "TimeoutHandler:$PORT1" -e ${SHELL_FORWARD} ../$BUILD_TARGET/TimeoutHandler --CdmwMessageResend=1 --CdmwFederatedCtrlClock --CdmwLocalisationService=$PORT1 --CdmwClockName=$CLOCK_NAME &
exec $TERM -ls -sb -sl 1000 -geometry 50X10+330+000 -display $DISPLAY -title "TimeoutHandler:$PORT2" -e ${SHELL_FORWARD} ../$BUILD_TARGET/TimeoutHandler --CdmwMessageResend=1 --CdmwFederatedCtrlClock --CdmwLocalisationService=$PORT2 --CdmwClockName=$CLOCK_NAME &
exec $TERM -ls -sb -sl 1000 -geometry 50X10+660+000 -display $DISPLAY -title "TimeoutHandler:$PORT3" -e ${SHELL_FORWARD} ../$BUILD_TARGET/TimeoutHandler --CdmwMessageResend=1 --CdmwFederatedCtrlClock --CdmwLocalisationService=$PORT3 --CdmwClockName=$CLOCK_NAME &

sleep 5

# 3) Start the time admins
$echo Starting the time admins...

exec $TERM -ls -sb -sl 1000 -geometry 50X10+000+160 -display $DISPLAY -title "TimeAdmin:$PORT1" -e ${SHELL_FORWARD} ../$BUILD_TARGET/TimeAdmin -ORBInitRef ClockService=corbaloc::localhost:$PORT1/ClockService  &
exec $TERM -ls -sb -sl 1000 -geometry 50X10+330+160 -display $DISPLAY -title "TimeAdmin:$PORT2" -e ${SHELL_FORWARD} ../$BUILD_TARGET/TimeAdmin -ORBInitRef ClockService=corbaloc::localhost:$PORT2/ClockService  &
exec $TERM -ls -sb -sl 1000 -geometry 50X10+660+160 -display $DISPLAY -title "TimeAdmin:$PORT3" -e ${SHELL_FORWARD} ../$BUILD_TARGET/TimeAdmin -ORBInitRef ClockService=corbaloc::localhost:$PORT3/ClockService  &

sleep 1

# 4) Stop all processes when ready
$echo "=============================================================="
$echo "Hit a key when ready to stop the following processes:"
$echo "1. the timeout handlers,"
$echo "2. the time admins."
read FOO
$echo "=============================================================="

# 4.1) Stop time admins, if any
PIDS_TA=`/sbin/pidof TimeAdmin`
if [[ "$PIDS_TA" != "" ]]
then
    kill -9 $PIDS_TA
fi

# 4.3) Stop timeout handlers, if any
PIDS_TH=`/sbin/pidof TimeoutHandler`
if [[ "$PIDS_TH" != "" ]]
then
    kill -9 $PIDS_TH
fi

$echo "done."
