/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
#include <iostream>
#include <sstream>
#include <fstream>

#include <unistd.h>
#include <Time/clockservice/CdmwCosClock.hpp>
#include <Time/idllib/CdmwTime.skel.hpp>
#include <ace/Task.h>

#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/ossupport/OS.hpp>
#include "TimeoutHandler.hpp"

TimeoutHandler::TimeoutHandler(CosClockService::ControlledClock_ptr clock)
    : m_count(0),
      m_clock(CosClockService::ControlledClock::_duplicate(clock))
{
    // No-Op
}

TimeoutHandler::~TimeoutHandler()
{
    // No-Op
}

CORBA::Boolean
TimeoutHandler::do_work(const CORBA::Any&)
    throw (CORBA::SystemException)
{
    std::cerr << "[Reading " << m_count++ << "]:>> " << m_clock->current_time() << std::endl;
    return true;
}

namespace
{    
        const size_t PERIOD = 5000000; // 500 mec;
        const std::string HELP_OPTION = "--help";
        long exec_limit = 0;
        long the_period = PERIOD;

        class MyObserver 
    :   public virtual POA_Cdmw::clock::ClockObserver,
        public virtual Cdmw::clock::ClockObserver
    {
        private:
        CosClockService::PeriodicExecution::Controller_ptr  m_controller;

        public:
        MyObserver(CosClockService::PeriodicExecution::Controller_ptr controller)
        :m_controller(controller)
        {
        }
        
        void 
        handle_set (const ::CORBA::Any & params)
            throw (CORBA::SystemException)
        {
            std::cout<<"Hello from handle_set "<<std::endl;
            CORBA::Any param;
            std::cout<<"Timer has been reprogrammed and will start NOW"<<std::endl;
            m_controller->start(the_period, 0, exec_limit, param);
        }
  
        void 
        handle_set_rate (const ::CORBA::Any & params)
            throw (CORBA::SystemException)
        {
            std::cout<<"Hello from handle_set_rate "<<std::endl;
            m_controller->terminate();
        }
  
        void 
        handle_pause (const ::CORBA::Any & params)
            throw (CORBA::SystemException)
        {
            std::cout<<"Hello from handle_pause "<<std::endl;
            m_controller->terminate();
        }
  
        void 
        handle_resume (const ::CORBA::Any & params)
            throw (CORBA::SystemException)
        {
            std::cout<<"Hello from handle_resume "<<std::endl;
            m_controller->terminate();
        }
  
        void 
        handle_terminate (const ::CORBA::Any & params)
            throw (CORBA::SystemException)
        {
            std::cout<<"Hello from handle_terminate "<<std::endl;           
            m_controller->terminate();
        }
        
    };
    
    //-------------------------------------------------------------------------
    
    class ORBWorker : public Cdmw::OsSupport::Thread
    {
    public:
            ORBWorker(CORBA::ORB_ptr orb); 
            virtual ~ORBWorker() throw ();
            virtual void run() throw ();
    private:
        CORBA::ORB_var            m_ORB;
    };
    
    ORBWorker::ORBWorker(CORBA::ORB_ptr orb)
        : m_ORB(CORBA::ORB::_duplicate(orb))
    {
    }
    
    ORBWorker::~ORBWorker() throw ()
    {
        try
        {
            m_ORB->shutdown(1);
            m_ORB->destroy(); 
            this->join();     
        }
        catch (const CORBA::SystemException& e)
        {
            std::cout << ":>> CORBA system exception caught: " << e._info().rep() << std::endl;
        }
        catch (const std::exception& e)
        {
            std::cout << ":>> std exception caught: " << e.what() << std::endl;
        }
        catch (...)
        {
            std::cout << ":>> unexpected exception caught" << std::endl;
        }
    }
    
    void ORBWorker::run() throw ()
    {
        try
        {
            std::cout << ":>> ORB is running " << std::endl;            
            this->m_ORB->run();
        }
        catch (const CORBA::SystemException& e)
        {
            std::cout << ":>> CORBA system exception caught: " << e._info().rep() << std::endl;
        }
        catch (...)
        {
            std::cout << ":>> unexpected exception caught" << std::endl;
        }
    }
    
}   // end of unnamed namespace

//-----------------------------------------------------------------------------
    
void print_help() 
{
    std::cerr << "usage: TimeoutHandler <start_at_time> <options>" << std::endl;
    std::cerr << "where: <start_at_time> is the absolute time at which controller starts" << std::endl;
    std::cerr << "       <options>       indicates the following list of options:" << std::endl;
    std::cerr << "                       --CdmwLocalisationService=<port>" << std::endl;
    std::cerr << "                       --CdmwMessageResend=<send_repetitions> [default is 2]" << std::endl;
}

//-----------------------------------------------------------------------------

int
main(int argc, char* argv[])
{
    try
    {
        // this assert
        std::string help_option = Cdmw::OsSupport::OS::get_option_value(argc, argv, HELP_OPTION);
        if (help_option =="yes")
        {
            print_help();
            std::exit(-1);
        }

        TimeBase::TimeT start_time = 0;
        std::istringstream iss(argv[1]);
        iss >> start_time; 

        Cdmw::OrbSupport::StrategyList strategyList;
        strategyList.add_OrbThreaded();
        strategyList.add_PoaThreadPerConnection();
        strategyList.add_multicast();
        CORBA::ORB_var orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, strategyList);

        CORBA::Object_var CORBA_obj = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var        rootPOA     = PortableServer::POA::_narrow(CORBA_obj.in());
        PortableServer::POAManager_var poa_manager = rootPOA->the_POAManager();
        poa_manager->activate();

        // Start a thread to have ORB up
        //  better way is by using a barrier
        ORBWorker orb_worker(orb.in());
        orb_worker.start();             
        
        // Sleep to assure that ORB is up and running
        Cdmw::OsSupport::OS::sleep(2, 0);

        if (!Cdmw::clock::CosClock::init(orb.in(), argc, argv))
        {
            std::cerr << "TimeoutHandler:>> CosClock Initialization Failed" << std::endl;
            std::exit(-1);
        }

        CORBA::Object_var obj = orb->resolve_initial_references("ClockService");
        if (CORBA::is_nil(obj.in()))
        {
              std::cerr << "TimeoutHandler:>> Unable to resolve ClockService" << std::endl;
              std::exit(-1);
        }
        
        CosClockService::ClockCatalog_var clock_catalog = CosClockService::ClockCatalog::_narrow(obj.in());
        CosClockService::ClockCatalog::ClockEntry* catalog_entry = clock_catalog->get_entry("FederatedControlledExecutor");
        Cdmw::clock::ControlledExecutor_var fed_ctrl_executor =
            Cdmw::clock::ControlledExecutor::_narrow(catalog_entry->subject.in());
                           
        if (CORBA::is_nil(fed_ctrl_executor.in()))
        {
            std::cerr << "TimeoutHandler:>> Unable to resolve FederatedControlledExecutor" << std::endl;
            std::exit(-1);
        }
        
        TimeoutHandler handler(fed_ctrl_executor.in());
        CosClockService::PeriodicExecution::Periodic_var periodic = handler._this();
        
        CosClockService::PeriodicExecution::Controller_var controller; 

        // use NO_ACTION policy the behaviuor of controller on notification is under the 
        //  responsibility of programmer
        controller = fed_ctrl_executor->enable_periodic_execution(periodic.in());
        fed_ctrl_executor->set_controller_update_policy(Cdmw::clock::policy::NO_ACTION);    

        // MyObserver know controller and can use it to have a customized behaviour 
        //  (on set new start, in other case terminate).
        MyObserver* my_observer = new MyObserver(controller.in());
        Cdmw::clock::ClockObserver_var observer = my_observer->_this();
 
        CORBA::Any aparam;
            
        // attach an observer to an executor
        fed_ctrl_executor->attach(Cdmw::clock::ALL, observer.in(), aparam);    
                    
        // start controller
        controller->start(the_period, 0, exec_limit, aparam);
        
        orb_worker.join();
        
        std::exit(0);
    }
    catch (std::exception& e)
    {
        std::cerr << "TimeoutHandler:>> std exception caught: " << e.what() << std::endl;
        std::exit(-1);
    }
    catch (CORBA::Exception& e)
    {
        std::cerr << "TimeoutHandler:>> CORBA exception caught: " << e << std::endl;
        std::exit(-1);
    }
    catch (...)
    {
        std::cerr << "TimeoutHandler:>> Generic exception caught" << std::endl;
        std::exit(-1);
    }
}

//-----------------------------------------------------------------------------
//  End of File
//-----------------------------------------------------------------------------
