/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @author Raffaele Mele <rmele@progesi.it>
 * @brief TimeAdmin - Federated Controlled Clock demo
 */

//package cdmw.clock.federated;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

import org.omg.CORBA.ORB;
import org.omg.CORBA.ORBPackage.InvalidName;
import org.omg.CosClockService.*;

import org.omg.CosClockService.ClockCatalogPackage.ClockEntry;
import org.omg.CosClockService.ClockCatalogPackage.UnknownEntry;
import org.omg.CosClockService.ControlledClockPackage.NotSupported;
import org.omg.CosClockService.ControlledClockHelper;
import cdmw.orbsupport.*;
import cdmw.common.BadParameterException;


/**
 * @author rmele
 * 
 */
public class TimeAdmin {
    // local variables
    private static ORB m_orb;

    private static ClockCatalog m_catalog;

    private static ControlledClock m_ctrlClock;

    // constant values
    static final int EXIT = 5;

    public static long get_input(String echo) {
        if (echo != null) System.out.println(echo);
        try {
            BufferedReader inputUser 
                = new BufferedReader(new InputStreamReader(System.in));
            String inputStr = inputUser.readLine();
            return Long.valueOf(inputStr).longValue();
        } catch (java.io.IOException ex) {
            ex.printStackTrace();
            return 0;
        }
    }

    public static void main(String[] args) {
        String[] strArgs = { 
            "--CdmwLocalisationService=4883"
            //,"-ORBInitRef"
            //,"LocalClock=corbaloc::localhost:4883/LocalClock"
            ,"-ORBInitRef"
            ,"ClockService=corbaloc::localhost:4883/ClockService"
            //,"--CdmwFederatedCtrlClock"
            //,""
            //,""
            // reserve 2 places for Property Service arguments
             
           };

        //m_orb = org.omg.CORBA.ORB.init(strArgs, null);
        //cdmw.clock.CdmwCosClock.init(m_orb);
         try {
            StrategyList orbStrategies = new StrategyList();
                orbStrategies.addORBThreaded();
                orbStrategies.addPOAThreadPerConnection();
                
                
                m_orb = ORBSupport.ORBInit(args, orbStrategies);

                cdmw.clock.CdmwCosClock.init(m_orb,args);

        org.omg.CORBA.Object obj = null;
        try {
            obj = m_orb.resolve_initial_references("ClockService");
        } catch (InvalidName e1) {
            e1.printStackTrace();
            System.exit(-1);
        }
        if (obj == null) {
            System.out.println("Error : Unable to resolve ClockService");
        }
        m_catalog = ClockCatalogHelper.narrow(obj);
        if (m_catalog == null) {
            System.out.println("Error : Unable to resolve clock catalog");
        }

        ClockEntry entry;
        try {

            entry = m_catalog.get_entry("FederatedControlledExecutor");
            m_ctrlClock = ControlledClockHelper.narrow(entry.subject);
        } catch (UnknownEntry ex) {
            System.out.println("ClockCatalog::UnknownEntry");
        }

        if (m_ctrlClock == null) {
            System.err
                    .println("Unable to resolve FederatedControlledClock, Aborting!");
            System.exit(-1);
        }


        BufferedReader inputUser = new BufferedReader(new InputStreamReader(System.in));
        String inputStr;

            long choice = 0;

            while (choice != EXIT) {
                choice = get_input("\t Press [5] to exit: ");
            }
        cdmw.clock.CdmwCosClock.close();
                System.exit(1);
        Runtime p = Runtime.getRuntime();
        try 
        {
            p.exec("clear");
        } catch (IOException e) {
            e.printStackTrace();
        }
                
         }
                catch (cdmw.common.BadParameterException be)
                {
                  be.printStackTrace();
                }

    }

}
