/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "lifecycle/Application_impl.hpp"
//#include "orbsupport/ExceptionMinorCodes.hpp"

#include "TraceAndPerf/tracelibrary/CdmwTrace.hpp"

namespace Cdmw
{
namespace Resources
{

/**
*Implementation notes: [if no pertinent write none ]
*<p>
*Portability issues: [if no pertinent write none ]
*<p>
*/
// Application_impl::Application_impl()
//     throw(CORBA::SystemException)
//     : Cdmw::LifeCycle::LifeCycleSingleObject(),
//       m_name("/null"),
//       m_started(false),
//       m_pid(-1)
// {
// }

Application_impl::Application_impl(const Cdmw::LifeCycle::ObjectDeactivator & obj_d,
                                   const std::string &                         application_name)
    throw(CORBA::SystemException)
    : Cdmw::LifeCycle::LifeCycleSingleObject(obj_d),
      m_name(application_name),
      m_started(false),
      m_pid(-1)
{ 
}

Application_impl::~Application_impl()
    throw()
{
    if (m_started) 
        try {
            //Cdmw::OsSupport::OS::kill_process(m_pid);
            Trace::Stream stream;
            CDMW_TRACE_ST (stream, CDMW_LFCL, INF, "Stopping <" << m_name << ">");
            std::cout << "Stopping <" << m_name << ">" << std::endl;
        } catch (...) {
        }
    std::cerr << "Application_impl::~Application_impl()" << std::endl;
}


//
// IDL:thalesgroup.com/CF/Application/start:1.0
//
void Application_impl::start()
    throw(CF::Application::StartError,
          CORBA::SystemException)
{
    if (m_started)
        throw CF::Application::StartError("Application already started!");
    else {
        try {
            //m_pid = Cdmw::OsSupport::OS::create_process(m_name,"");
            //std::cerr << "create_process(" << m_name << ")" << std::endl;
            Trace::Stream stream;
            CDMW_TRACE_ST (stream, CDMW_LFCL, INF, "Starting <" << m_name << ">");
            std::cout << "Starting <" << m_name << ">" << std::endl;
            m_started = true;
        } catch (...) {
            std::string s("Could not start ");
            s += m_name;
            s += "!";
            throw CF::Application::StartError(s.c_str());
        }
    }
}

//
// IDL:thalesgroup.com/CF/Application/stop:1.0
//
void Application_impl::stop()
    throw(CF::Application::StopError,
          CORBA::SystemException)
{
    if (m_started) {    
        try {
            //Cdmw::OsSupport::OS::kill_process(m_pid);
            //std::cerr << "kill_process " << m_name << "." << std::endl;
            Trace::Stream stream;
            CDMW_TRACE_ST (stream, CDMW_LFCL, INF, "Stopping <" << m_name << ">");
            std::cout << "Stopping <" << m_name << ">" << std::endl;
            m_started = false;
            deactivate_object();
        } catch (...) {
            std::string s("Could not stop ");
            s += m_name;
            s += "!";
            throw  CF::Application::StopError(s.c_str());
        }
    } 
    else {
        throw  CF::Application::StopError("Application not started!");
    }
}

//
// IDL:thalesgroup.com/CF/Application/name:1.0
//
char* Application_impl::name()
    throw(CORBA::SystemException)
{
    return CORBA::string_dup(m_name.c_str());
}


}; // End namespace Resources
}; // End namespace Cdmw

