/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "Foundation/orbsupport/CORBA.hpp"

#include "process/Test.hpp"
#include "SystemMngt/platforminterface/PlatformInterface.hpp"

// constructor
Test::Test()
: Cdmw::TestUtils::Testable( "CdmwPlatformMngt::UserEvent" )
{
}

// destructor
Test::~Test()
{
}

// do_tests
void Test::do_tests()
{
    // set number of requested successfull tests
    set_nbOfRequestedTestOK( 2 );

    try
    {
        TEST_INFO( "Notifying a user event with data of type 'TYPE_A'" );
        CORBA::Any data;

        data <<= m_factory_type_a.create( "data type is TYPE_A" );
        Cdmw::PlatformMngt::PlatformInterface::Notify_user_event(
            "USER PROCESS", data );
        TEST_SUCCEED();  
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO( "Unexpected CORBA System Exception raised" << e );
        TEST_FAILED();  
    }

    try
    {
        TEST_INFO( "Notifying a user event with data of type 'TYPE_B'" );
        CORBA::Any data;

        data <<= m_factory_type_b.create( "data type is TYPE_B", 12345L );
        Cdmw::PlatformMngt::PlatformInterface::Notify_user_event(
            "USER PROCESS", data );
        TEST_SUCCEED();  
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO( "Unexpected CORBA System Exception raised" << e );
        TEST_FAILED();  
    }
}

