/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.perftests.CDMWDispatcher;

import org.omg.CORBA.*;

import com.thalesgroup.CdmwPlatformMngt.ProcessDelegatePackage.*;
import com.thalesgroup.CdmwPlatformMngtBase.*;
import org.omg.CosNaming.NamingContextPackage.*;

public class ClientProcessControl extends cdmw.cdmwinit.ProcessControl {
    /**
     * The orb reference.
     */
    private ORB orb;

    /**
     * The application name.
     */
    private String applicationName;

    /**
     * The process name.
     */
    private String processName;

    /**
     * Tester reference
     */
    private Tester tester;

    private int nstart;

    private int niter;

    private String fileprefix;

    public ClientProcessControl(
        ORB orb, int nstart, int niter, String fileprefix) {
        this.orb = orb;
        this.applicationName = "";
        this.processName = "";
        this.nstart = nstart;
        this.niter = niter;
        this.fileprefix = fileprefix;
    }

    /**
     * Purpose: <p> the behaviour for the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/initialise:1.0
     * operation
     */
    @Override
    public void onInitialise(StartupKind startupKind) {
        // ===================================================
        // Get the process name and application name
        // ===================================================
        this.processName = cdmw.cdmwinit.InitUtils.getCdmwProcessName();

        this.applicationName = cdmw.cdmwinit.InitUtils.getCdmwApplicationName();

        // =================================================================
        // example of using the PlatformInterface for notifying a message
        // =================================================================
        cdmw.platformmngt.PlatformInterface.notifyMessage(INF.value,
            processName,
            ">>>>>>>>>>>>>> Initialisation requested by supervision");

        // =========================================
        // retrieve Tester object from Repository
        // =========================================
        // get NamingInterface to perf_test
        cdmw.commonsvcs.naming.NamingInterface DispatcherNamingInterface = cdmw.namingandrepository.RepositoryInterface
            .getDomainNamingInterface("perf_test");

        // Retrieve Tester from NamingInterface
        String registrationName = "Tester";
        try {
            org.omg.CORBA.Object obj = DispatcherNamingInterface
                .resolve(registrationName);
            tester = Dispatcher.TesterHelper.narrow(obj);
        }
        catch (NotFound e) {
            System.err.println("Resolve " + registrationName
                + " => NamingContext.NotFound raised!");
        }
        catch (CannotProceed e) {
            System.err.println("Resolve " + registrationName
                + " => NamingContext.CannotProceed raised!");
        }
        catch (InvalidName e) {
            System.err.println("Resolve " + registrationName
                + " => NamingContext.InvalidName raised!");
        }
        catch (SystemException e) {
            System.err.println("Resolve " + registrationName
                + " => CORBA.SystemException raised!");
        }
    }

    /**
     * Purpose: <p> the behaviour for the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/run:1.0 operation
     */
    @Override
    public void onRun() throws NotReadyToRun, SystemException {
        // =================================================================
        // example of using the PlatformInterface for notifying a message
        // =================================================================
        cdmw.platformmngt.PlatformInterface.notifyMessage(INF.value,
            this.processName, ">>>>>>>>>>>>>> Run requested by supervision");

        System.out.println("Start testping");

        double[] tdelta = new double[niter + nstart];

        // object for time measurement
        long time1, time2;

        // struct for time delay
        int delay = 100;

        // start of record
        for (int i = 0; i < (nstart + niter); i++ ) {
            Thread.sleep(delay);
            time1 = (new java.util.Date()).getTime();
            tester.ping();
            time2 = (new java.util.Date()).getTime();
            // time read in micro sec.
            tdelta[i] = time2 - time1;
        }// end for
        // write file
        String fname = new String(fileprefix + "DELTA_C.dat");
        java.io.FileWriter fd = new java.io.FileWriter(fname);
        // fd.precision(6);
        for (int i = 0; i < niter; i++ )
            fd.write(String.format("%1.6f", tdelta[nstart + i]));
        fd.close();
        System.err.println("End testping");

        java.io.FileWriter feof = new java.io.FileWriter("end_of_test.log");
        feof.write("EOF");
        feof.close();

    }

    /**
     * Purpose: <p> the behaviour for the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/stop:1.0 operation
     */
    @Override
    public void onStop() {
        // =================================================================
        // example of using the PlatformInterface for notifying a message
        // =================================================================
        cdmw.platformmngt.PlatformInterface.notifyMessage(INF.value,
            this.processName, ">>>>>>>>>>>>>> Stop requested by supervision");

    }
}
