/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.perftest.CDMWInvocationC;

import java.io.*;

import org.omg.CORBA.ORB;
import org.omg.PortableServer.*;

import cdmw.orbsupport.StrategyList;

public class Client {
    
    private static final int SUCCESS = 0;
    private static final int FAILURE = 1;
    private static final int POA_THREAD_POOL_SIZE = 5;

    public static void main(String args[]) {    
        int status = SUCCESS;
        
        System.err.println("-------------------------");
        System.err.println("      Start Client");
        System.err.println("-------------------------");
        for (int n=0; n < args.length; n++)
            System.err.println("args[" + n +"]: " + args[n]);  
        System.err.println("-------------------------");

        // number of test
        int niter = 1000;
      
        // delay
        int delay = 0;
      
        // type of invocation
        String invocationType = new String("");
        // type of sent data
        String dataType = new String(""); // LONG, FLOAT, ANY, RECORD
        // size of data sent
        int dataSize = 4;

        // output file prefix
        String fileprefix = new String("");

        System.out.println("Reading Configuration File...");
        StringBuilder sb = new StringBuilder();

        char ch = 0;
        int i = 0;
        try {
            InputStreamReader isr = new FileReader("client.cfg");
            while (i != -1) {
                i = isr.read();
                ch = (char) i;
                sb.append(ch);
            }
            isr.close();
        }
        catch (IOException e) {
            e.printStackTrace();
        }
        String fData = sb.toString();

        String[] tokens = fData.split("\\s+");
        try {
            niter = Integer.parseInt(tokens[0]);
            delay = Integer.parseInt(tokens[1]);
        }
        catch (NumberFormatException e) {
            e.printStackTrace();
            System.exit(FAILURE);
        }
        invocationType = tokens[2];
        dataType = tokens[3];
        try {
            dataSize = Integer.parseInt(tokens[4]);
        }
        catch (NumberFormatException e) {
            e.printStackTrace();
            System.exit(FAILURE);
        }

        fileprefix = tokens[6];
        
        System.out.println("Sample invocation= " + niter);            
        System.out.println("Delay (ms)= " + delay);            
        System.out.println("Invocation type = " + invocationType);
        System.out.println("Prefix of output file names= " + fileprefix);            
        
         ORB orb = null;     // orb reference
        try {
            // ===================================================
            // Initialize the ORB
            // ===================================================
            cdmw.orbsupport.StrategyList orbStrategies = new StrategyList();
            orbStrategies.addORBThreaded();
            orbStrategies.addPOAThreadPool(POA_THREAD_POOL_SIZE);

            orb = cdmw.orbsupport.ORBSupport.ORBInit(args, orbStrategies);
            
            // ===================================================
            // Get the root POA
            // ===================================================
            org.omg.CORBA.Object obj = orb.resolve_initial_references("RootPOA");
            POA rootPOA = POAHelper.narrow(obj);

            // ===================================================
            // Activate the root POA manager
            // ===================================================
            POAManager poaManager = rootPOA.the_POAManager();
            poaManager.activate();
            
            // ===================================================
            // Instantiate time reading object
            // ===================================================
            ClockTime measure = new ClockTime();
            // ===================================================
            // create the process control for platform management
            // ===================================================
                cdmw.cdmwinit.ProcessControl processCtrl = new ClientProcessControl(orb,
                                           niter,
                                           delay,
                                           invocationType,
                                           fileprefix,
                                           measure);

            // ===================================================
            // Call generated CdmwInit
            // ===================================================
            cdmw.cdmwinit.CdmwInterface.cdmwInit(orb, args, processCtrl, "cdmw.generated.ClientCdmwInterface");
            // ===================================================
            // wait for CORBA message
            // ===================================================
            orb.run();

        } 
        catch (cdmw.common.Exception ex1) {
            System.err.println(ex1.getMessage());
            status = FAILURE;
        } 
        catch(org.omg.CORBA.SystemException ex2) {        
            System.err.println(ex2.getMessage());
            status = FAILURE;
        }
        catch(org.omg.CORBA.UserException ex3)  {        
            System.err.println(ex3.getMessage());
            status = FAILURE;
        }
        catch(Exception e)  {        
            System.err.println("unexpected exception");
            status = FAILURE;
            
        }

        // ========================================================
        // program stopping
        // ========================================================
        
        // ===================================================
        // Call generated Cdmw cleanup
        // ===================================================
        cdmw.cdmwinit.CdmwInterface.cdmwCleanup(orb, "cdmw.generated.ClientCdmwInterface");
        // ===================================================
        // Call ORB cleanup
        // ===================================================
        cdmw.orbsupport.ORBSupport.ORBCleanup(orb);
                
        // ===================================================
        // destroy orb
        // ===================================================
        
        if (orb != null) {
            try {
                orb.destroy();
            }
            catch(org.omg.CORBA.SystemException ex2) {        
                System.err.println(ex2.getMessage());
                status = FAILURE;
            }
        }        

        System.exit(status);    
    }
}
