/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.perftest.CDMWInvocationC;

import org.omg.CORBA.*;
import org.omg.CORBA.Object;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import org.omg.Messaging.*;

import cdmw.common.*;
import cdmw.orbsupport.*;
import cdmw.perftest.CDMWInvocationC.ClockTime;

import com.thalesgroup.CdmwNamingAndRepository.*;
import com.thalesgroup.CdmwNamingAndRepository.InvalidName;
import com.thalesgroup.CdmwNamingAndRepository.NameDomainPackage.*;
import com.thalesgroup.CdmwPlatformMngt.ProcessDelegatePackage.*;
import com.thalesgroup.CdmwPlatformMngtBase.*;

public class ClientProcessControl extends cdmw.cdmwinit.ProcessControl {
    
    public static final String CALL_RETURN = "CR";

    public static final String ONE_WAY = "OW";

    public static final String ONE_WAY_TRANSPORT = "OW_TRANSPORT";

    public static final String ONE_WAY_SERVER = "OW_SERVER";

    public static final String ONE_WAY_TARGET = "OW_TARGET";

    /**
     * The orb reference.
     */
    private ORB orb;

    /**
     * The application name.
     */
    private String applicationName;

    /**
     * The process name.
     */
    private String processName;

    /**
     * TestPing pointer
     */
    private TestPingI testpingi;

    /**
     * TestPing reference
     */
    private Invocation.TestPing testping;

    private int niter;

    private int delay;

    private String invocationType;

    private String fileprefix;

    private ClockTime measure;

    public ClientProcessControl(
        ORB orb, int niter, int delay, String invocationType,
        String fileprefix, ClockTime measureType) {
        this.orb = orb;
        this.applicationName = "";
        this.processName = "";
        this.niter = niter;
        this.delay = delay;
        this.invocationType = invocationType;
        this.fileprefix = fileprefix;
        this.measure = measureType;
    }

    /**
     * Purpose: <p> the behaviour for the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/initialise:1.0
     * operation
     */
    @Override
    public void onInitialise(StartupKind startupKind) throws SystemException {
        // ===================================================
        // Get the process name and application name
        // ===================================================
        this.processName = cdmw.cdmwinit.InitUtils.getCdmwProcessName();
        this.applicationName = cdmw.cdmwinit.InitUtils.getCdmwApplicationName();

        // =================================================================
        // example of using the PlatformInterface for notifying a message
        // =================================================================
        cdmw.platformmngt.PlatformInterface.notifyMessage(INF.value,
            this.processName,
            ">>>>>>>>>>>>>> Initialisation requested by supervision");
        // ===================================
        // register Tester into Repository
        // ===================================
        // Get the Cdmw repository
        com.thalesgroup.CdmwNamingAndRepository.Repository repository = cdmw.namingandrepository.RepositoryInterface.getRepository();

        // Get DemoPerf name domain from repository
        NameDomain demoPerfDomain = null;
        try {
            demoPerfDomain = repository.resolve_name_domain("perf_test");
        }
        catch (NoNameDomain e) {
            System.err.println("Can't find NameDomain perf_test");
            Assert.check(false);
        }
        catch (InvalidName e1) {
            System.err.println("NameDomain perf_test is invalid !!");
            Assert.check(false);
        }
        // Create a TestPing object
        testpingi = new TestPingI();
        this.testping = testpingi._this();
        // Register the TestPing object in the "perf_test" name domain
        String registrationName = "TestPing";
        try {
            // reserve the name "TestPing" in the domain
            Any regId = demoPerfDomain.new_name(registrationName);
            // register the object
            //TODO
            demoPerfDomain.register_object(regId, this.testping);

        }
        catch (AlreadyExists e1) {
            System.err.println("Name " + registrationName
                + " already exist into NameDomain perf_test");
            Assert.check(false);
        }
        catch (InvalidName e2) {
            System.err.println("Name " + registrationName
                + " is an invalid name !!");
            Assert.check(false);
        }
        catch (InvalidRegistration e3) {
            System.err.println("Invalid Registration of TestPing reference as " + registrationName);
            Assert.check(false);
        }

        // =========================================
        // retrieve Tester object from Repository
        // =========================================
        // get NamingInterface to test_perf
        cdmw.commonsvcs.naming.NamingInterface invocationNamingInterface = cdmw.namingandrepository.RepositoryInterface
            .getDomainNamingInterface("perf_test");

        // Retrieve TestPing from NamingInterface
        try {
            org.omg.CORBA.Object obj = invocationNamingInterface.resolve(registrationName);
            this.testping = TestPingHelper.narrow(obj);
        }
        catch (NotFound e) {
            System.err.println("Resolve " + registrationName + " => NamingContext.NotFound raised!");
        }
        catch (CannotProceed  e1) {
            System.err.println("Resolve " + registrationName + " => NamingContext.CannotProceed raised!");
        }
        catch (org.omg.CosNaming.NamingContextPackage.InvalidName e2) {
            System.err.println("Resolve " + registrationName + " => NamingContext.InvalidName raised!");
        }
        catch (org.omg.CORBA.SystemException e) {
            System.err.println("Resolve " + registrationName + " => CORBA::SystemException raised!");
        }

    }

    /**
     * Purpose: <p> the behaviour for the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/run:1.0 operation
     */
    @Override
    public void onRun() throws NotReadyToRun {
        // =================================================================
        // example of using the PlatformInterface for notifying a message
        // =================================================================
        cdmw.platformmngt.PlatformInterface.notifyMessage(INF.value, this.processName, ">>>>>>>>>>>>>> Run requested by supervision");

        System.out.println("Start testping");

        double tdelta[] = new double[this.niter];
        double tabs[] = new double[this.niter];

        // delay of measure
        int delays = this.delay;
        int delayn = (this.delay % 1000) * 1000000;
        // object for time measurement
        long time1;
        long time2;

        // test CR call and return
        if (this.invocationType.equals(CALL_RETURN)) {
            // start of record
            for (int i = 0; i < this.niter; i++ ) {
                // delay of measure
                if (this.delay != 0) {
                    // call to nanosleep
                    Thread.sleep(delays, delayn);
                }
                time1 = this.measure.getTime();
                this.testping.ping();
                time2 = this.measure.getTime();
                // time read in micro sec.
                tdelta[i] = time2 - time1;
            }// end for

            // write file
            String fname = new String(this.fileprefix + "DELTA_C.dat");
            java.io.FileWriter fd = new java.io.FileWriter(fname);
            // ofstream fd(fname);

            for (int i = 0; i < this.niter; i++ ) {
                fd.write(String.format("%1.6f", tdelta[i]));
            }
            fd.close();

        }// end call_return
        else {
            // case one_way measure
            org.omg.CORBA.Object obj = this.orb.resolve_initial_references("ORBPolicyManager");
            PolicyManager policyManager = PolicyManagerHelper.narrow(obj);

            obj = this.orb.resolve_initial_references("PolicyCurrent");
            PolicyCurrent policyCurrent = PolicyCurrentHelper.narrow(obj);

            short syncScope = SYNC_WITH_TRANSPORT.value;

            if (this.invocationType.equals(ONE_WAY)) {// Default SyncScope value.
                syncScope = SYNC_NONE.value;
            }
            else if (this.invocationType.equals(ONE_WAY_TRANSPORT)) {
                syncScope = SYNC_WITH_TRANSPORT.value;
            }
            else if (this.invocationType.equals(ONE_WAY_SERVER)) {
                syncScope = SYNC_WITH_SERVER.value;
            }
            else if (this.invocationType.equals(ONE_WAY_TARGET)) {
                syncScope = SYNC_WITH_TARGET.value;
            }

            // Set up the sync scope any.
            Any syncScopeAny = orb.create_any();
            syncScopeAny.insert_short(syncScope);

            // Set the length of the policy list.
            PolicyList syncScopePolicyList = new PolicyList();

            // Set up the sync scope policy.
            syncScopePolicyList.getList()[0] = this.orb.create_policy(SYNC_SCOPE_POLICY_TYPE.value, syncScopeAny);

            // Set the sync scope policy at the object level.
            obj = this.testping._set_policy_overrides(syncScopePolicyList,
                org.omg.CORBA.SetOverrideType.ADD_OVERRIDE);
            // Get the new object reference with the updated policy.
            this.testping = TestPingHelper.narrow(obj);

            // Initialize server sample collector
            this.testping.init_sdump(this.niter);

            // start of record
            for (int i = 0; i < this.niter; i++ ) {
                // delay of measure
                if (this.delay != 0) {
                    // call to nanosleep
                    Thread.sleep(delays, delayn);
                }
                time1 = this.measure.getTime();
                this.testping.pos_ping_sdump(i);
                time2 = this.measure.getTime();
                // time read in micro sec.
                tabs[i] = time1;
                tdelta[i] = time2 - time1;
            }

            // write file
            String filedeltaname = new String(this.fileprefix + "DELTA_C.dat");
            String fileabsname = new String(this.fileprefix + "ABS_C.dat");

            java.io.FileWriter fd = new java.io.FileWriter(filedeltaname);
            java.io.FileWriter fa = new java.io.FileWriter(fileabsname);

            for (int i = 0; i < this.niter; i++ ) {

                fd.write(String.format("%1.6f", tdelta[i]));
                fa.write(String.format("%1.6f", tabs[i]));
            }

            fd.close();
            fa.close();
            try {
                // Save server samples
                String fname = new String(this.fileprefix + "ABS_S.dat");
                this.testping.save_sdump(fname);
            }
            catch (SystemException e) {
                System.err.println(" => CORBA::SystemException raised!");
            }

        }

        java.io.FileWriter feof = new java.io.FileWriter("end_of_test.log");
        feof.write("EOF");
        feof.close();

        System.out.println("End testping");
    }

    /**
     * Purpose: <p> the behaviour for the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/stop:1.0 operation
     */
    @Override
    public void onStop() {
        // =================================================================
        // example of using the PlatformInterface for notifying a message
        // =================================================================
        try {
            cdmw.platformmngt.PlatformInterface.notifyMessage(INF.value,
                this.processName, ">>>>>>>>>>>>>> Stop requested by supervision");
        }
        catch (BadParameterException e) {
            e.printStackTrace();
        }
        catch (BadOrderException e) {
            e.printStackTrace();
        }

    }

}
