/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.perftest.CDMWLBinvocation;

import java.io.PrintStream;

import org.omg.CORBA.*;
import org.omg.PortableGroup.CriteriaHelper;
import org.omg.PortableGroup.GenericFactory;
import org.omg.PortableGroup.PropertiesHelper;
import org.omg.PortableGroup.Property;
import org.omg.PortableGroup.PropertyHelper;

import CdmwLB.*;

import cdmw.orbsupport.*;

public class MainBind {
    
    private static final String REPLICA_NAME1  = "--ReplicaName1";
    private static final String REPLICA_NAME2  = "--ReplicaName2";
    private static final String REPLICA_NAME3  = "--ReplicaName3";
    private static final String GROUP_MANAGER_URL  = "--GroupMng-url";

    private static final String REP_ID_PINGABLE = "IDL:CdmwLBPerf/PingableInterface:1.0";
    private static final String LB_POLICY = "default";
    private static final String IOGR_FILE_NAME = "lb_pingable_group.IOGR";
    private static final String NAMING_REPOS_URL_OPTION = "--namingRepos-url";
    private static final int SLEEP_VAL = 5000;

    private static final int FAILURE = 1;

    private static void usage(PrintStream os, String programName) {
        os.println("\nUsage:\n"
            + programName
            + "Options:\n\n"
            + GROUP_MANAGER_URL
            + "=<URL>   URL of CARDAMOM Load Balancing Group Manager.\n"
            + REPLICA_NAME1
            + "=<NAM1>   string name for Replica n.1 \n"
            + REPLICA_NAME2
            + "=<NAM2>   string name for Replica n.2 \n"
            + REPLICA_NAME3
            + "=<NAM3>   string name for Replica n.3 \n"
            + "--namingRepos-url=<URL>  URL of the CARDAMOM Naming and Repository service.\n");
    }
    
    public static void main (String args[]) {
        try {
            String namingReposURL = cdmw.ossupport.OS.getOptionValue (args,NAMING_REPOS_URL_OPTION);
            String groupManagerURL = cdmw.ossupport.OS.getOptionValue (args,GROUP_MANAGER_URL);

            if (namingReposURL == "no") {
                System.err.println("naming and repository url undefined");
                usage(System.err, args[0]);
                System.exit(FAILURE);
            }

            if (groupManagerURL == "no") {
                System.err.println("LB Group Manager url undefined");
                usage(System.err, args[0]);
                System.exit(FAILURE);
            }

            Thread.sleep(SLEEP_VAL);

            System.err.println("BindReplicas>> Start binding replicas to Group Manager...");
            // Initialises the ORB
            cdmw.orbsupport.StrategyList strategyList = new StrategyList();
            strategyList.addORBThreaded();
            strategyList.addPOAThreadPerConnection();

            ORB orb = cdmw.orbsupport.ORBSupport.ORBInit(args, strategyList);

            // Get the LB Group Manager IOR
            org.omg.CORBA.Object obj = orb.string_to_object(groupManagerURL);
            if (obj == null) {
                (new Exception("could not _narrow " + groupManagerURL)).printStackTrace();
                System.exit(-1);
            }

            LBGroupManager groupManager = LBGroupManagerHelper.narrow(obj);
            if (groupManager == null) {
                (new Exception("could not _narrow LBGroupManager")).printStackTrace();
                System.exit(-1);
            }

            ////////////////////////////////////////////////////////////////////////
            //
            // Create a group with default load balancing policy (RoundRobin)
            //
            Property[] prop = new Property[1];
            prop[0] = new Property();
            prop[0].nam[0].id="cdmw.lb.LoadBalancingPolicy";
            prop[0].val.insert_string(LB_POLICY);

            Property[] crit = new Property[1];
            crit[0] = new Property();
            crit[0].nam[0].id="cdmw.lb.LBProperties";
            PropertiesHelper.insert(crit[0].val, prop);
            
            com.thalesgroup.CdmwNamingAndRepository.Repository repository = null;
            org.omg.CORBA.Object obj2 = orb.string_to_object(namingReposURL);
            if (obj2 == null) {
                (new Exception("could not convert to object NamingAndRepository from string" )).printStackTrace();
                System.exit(-1);
            }

            repository = com.thalesgroup.CdmwNamingAndRepository.RepositoryHelper.narrow(obj2);
            if (repository == null) {
                (new Exception("could not _narrow NamingAndRepository from URL" )).printStackTrace();
                System.exit(-1);
            }

            // ======================================================
            // Initialise the repository interface
            //     "CARDAMOM" has been chosen as the default domain name
            // ======================================================
            cdmw.namingandrepository.RepositoryInterface.init("CDMW", repository);

            // ======================================================
            // Get Naming Interface of PERFTEST name domain
            // ======================================================
            cdmw.commonsvcs.naming.NamingInterface ni =
                cdmw.namingandrepository.RepositoryInterface.getDomainNamingInterface (
                "PERFTEST/HELLO_LB");

            // ===================================================
            // Get the PingableInterface object from repository
            // ===================================================
            String fileName1 = cdmw.ossupport.OS.getOptionValue (args, REPLICA_NAME1);
            if (fileName1 == "no") {
                System.err.println(REPLICA_NAME1 + " name undefined");
                usage(System.err, args[0]);
                System.exit(FAILURE);
            }

            org.omg.CORBA.Object objref1 = ni.resolve(fileName1);
            // get the PingableInterface proxy
            //TODO
            PingableInterface pingableInterface1 = PingableInterfaceHelper.narrow(objref1);

            if (pingableInterface1 == null) {
                (new Exception("could not _narrow object to type PingableInterface" )).printStackTrace();
                System.exit(-1);
            }

            String fileName2 = cdmw.ossupport.OS.getOptionValue (args, REPLICA_NAME2);
            if (fileName2 == "no") {
                System.err.println(REPLICA_NAME2 + " name undefined");
                usage(System.err, args[0]);
                System.exit(FAILURE);
            }
            org.omg.CORBA.Object objref2 = ni.resolve (fileName2);
            // get the PingableInterface proxy
            PingableInterface pingableInterface2 = PingableInterfaceHelper.narrow(objref2);

            if (pingableInterface2 == null) {
                (new Exception("could not _narrow object to type PingableInterface")).printStackTrace();
                System.exit(-1);
            }

            String fileName3 = cdmw.ossupport.OS.getOptionValue (args, REPLICA_NAME3);
            if (fileName3 == "no") {
                System.err.println(REPLICA_NAME3 + " name undefined");
                usage(System.err, args[0]);
                System.exit(FAILURE);
            }
            org.omg.CORBA.Object objref3 = ni.resolve (fileName3);
            // get the PingableInterface proxy
            PingableInterface pingableInterface3 = PingableInterfaceHelper.narrow(objref3);

            if (pingableInterface3 == null) {
                (new Exception("could not _narrow object to type PingableInterface")).printStackTrace();
                System.exit(-1);
            }

            obj = groupManager.create_object(REP_ID_PINGABLE, crit, new AnyHolder());
            ////////////////////////////////////////////////////////////////////////
            // Add replicas
            org.omg.CosNaming.NameComponent[] loc = new org.omg.CosNaming.NameComponent[3];
            loc[0].id="HOST1";
            loc[0].kind="hostname1";
            loc[1].id="APPL1";
            loc[1].kind="applicationname1";
            loc[2].id="PROC1";
            loc[2].kind="processname1";

            obj = groupManager.add_member(obj, loc, objref1);

            loc[0].id = "HOST2";
            obj = groupManager.add_member(obj, loc, objref2);
            loc[0].id = "HOST3";
            obj = groupManager.add_member(obj, loc, objref3);
            // export the group object reference to a file
            String refString = orb.object_to_string(obj);
            
            java.io.FileWriter os1 = new java.io.FileWriter(IOGR_FILE_NAME);
            os1.write(refString);
            os1.close();
            System.err.println("BindReplicas>> End binding replicas to Group Manager...");

            try {
                CORBAManager.instance().init(orb);
            }
            catch(Exception e) {
                System.err.println(e.getMessage());
                System.exit(-1);
            }
        }
        catch(org.omg.CORBA.SystemException e) {
            e.printStackTrace();
            System.exit(-1);
        }
        catch(org.omg.CORBA.UserException e0) {
          e0.printStackTrace();
          System.exit(-1);
        }
        catch(cdmw.common.Exception e1) {
            e1.printStackTrace();
            System.exit(-1);
        }
        catch(Exception e2) {
            System.err.println("Unexpected exception");
            System.exit(-1);
        }
    
    }
}
