/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.perftest.CDMWLBinvocation;

import java.io.*;
import java.util.*;

import org.omg.CORBA.*;

import cdmw.orbsupport.StrategyList;

import com.thalesgroup.CdmwNamingAndRepository.*;
import com.thalesgroup.CdmwNamingAndRepository.Repository;
import com.thalesgroup.CdmwNamingAndRepository.RepositoryHelper;

public class MainPing {
    
    private static final int FAILURE = 1;
    private static final int SLEEP_VAL = 10000;
    private static final int DEFAULT = 1024;
    private static final String IOGR_FILE_NAME = "lb_pingable_group.IOGR";
    private static final String REPLICA_NAME  = "--ReplicaName";
    private static final String NAMING_REPOS_URL_OPTION = "--namingRepos-url";

    private static void usage(PrintStream os, String programName) {
        os.println("\nUsage:\n"
            + programName
            + "Options:\n\n"
            + REPLICA_NAME + "=<NAM1>   string name for a remote Replica. \n"
            + NAMING_REPOS_URL_OPTION + "=<URL>  URL of the CARDAMOM Naming and Repository service.\n");
    }

    public static void main (String args[]) {
        try {
          Thread.sleep(SLEEP_VAL);
          java.io.FileWriter osPerformanceLb = new java.io.FileWriter("DELTA_C_LB.dat");
          java.io.FileWriter osPerformance = new java.io.FileWriter("DELTA_C.dat");

          //number of Invocation
          int niter=0;
          //delay of Invocation
          int delay=0;

          System.err.println("-------------------------");
          System.err.println("      Start Client");
          System.err.println("-------------------------");
          for (int n=0; n < args.length; n++)
              System.err.println("args[" +n +"]: " + args[n]);
          System.err.println("-------------------------");
          System.err.println("Reading Configuration File...");

          //read configuration file
          StringBuilder sb = new StringBuilder();
          char ch = 0;
          int i = 0;
          InputStreamReader fd = new FileReader("client.cfg");
          
          if(fd == null) {
              System.err.println("Error: client.cfg don't exist!\n");
              niter = DEFAULT;
          }
          else {    
              while (i != -1) {
                  i = fd.read();
                  ch = (char) i;
                  sb.append(ch);
              }
              fd.close();
              String fData = sb.toString();

              String[] tokens = fData.split("\\s+");
              try {
                  niter = Integer.parseInt(tokens[0]);
                  delay = Integer.parseInt(tokens[1]);
              }
              catch (NumberFormatException e) {
                  e.printStackTrace();
                  System.exit(FAILURE);
              }
          }
          
          String ORBInitArgs[] = new String[args.length+1];

          for (int i1 = 0; i < args.length; i1++) {
              ORBInitArgs[i] = args[i1] + " ";
          }
          ORBInitArgs[args.length] = "--LBInterceptorSide=Client";

          //TODO - LBServiceInit is not converted
          LBServiceInit.Init(ORBInitArgs);

          // Initialise the ORB
          cdmw.orbsupport.StrategyList strategyList = new StrategyList();
          strategyList.addORBThreaded();
          strategyList.addPOAThreadPerConnection();

          ORB orb = cdmw.orbsupport.ORBSupport.ORBInit(args, strategyList);

          PingableInterface pingableGroup = TestUtils.getPingableRefFromFile(orb, IOGR_FILE_NAME);
          System.err.println("PingClient>> calling a Load Balanced server..");
          List<Long> data = new ArrayList<Long>();
          List<Long> datalb = new ArrayList<Long>();
          //delay of measure
          long sec  = delay / 1000;
          int nsec = (delay % 1000)  * 1000000;

          for(int i1 = 0; i1 < niter; i1++) {
              if( delay != 0 ) {
                  // call to nanosleep
                  Thread.sleep(sec, nsec);
                }
             
              long t1 = System.nanoTime();
              pingableGroup.ping();
              datalb.add(new Long(System.nanoTime() - t1));
         }

          // Get replica reference from the repository
          String namingReposURL = cdmw.ossupport.OS.getOptionValue (args,NAMING_REPOS_URL_OPTION);
          if (namingReposURL == "no") {
              System.err.println("naming and repository url undefined");
              usage(System.err, args[0]);
              System.exit(FAILURE);
          }

          Repository repository = null;

          org.omg.CORBA.Object obj2 = orb.string_to_object(namingReposURL);
          if (obj2 == null) {
              (new Exception("could not convert to object NamingAndRepository from string")).printStackTrace();
              System.exit(-1);
          }

          repository = RepositoryHelper.narrow(obj2);
          if (repository == null) {
              (new Exception("could not _narrow NamingAndRepository from URL")).printStackTrace();
              System.exit(-1);
          }

          // ======================================================
          // Initialise the repository interface
          //     "CARDAMOM" has been chosen as the default domain name
          // ======================================================
          cdmw.namingandrepository.RepositoryInterface.init("CDMW", repository);

          // ======================================================
          // Get Naming Interface of PERFTEST name domain
          // ======================================================
          cdmw.commonsvcs.naming.NamingInterface ni =
                 cdmw.namingandrepository.RepositoryInterface.getDomainNamingInterface ("PERFTEST/HELLO_LB");

          // ===================================================
          // Get the PingableInterface object from repository
          // ===================================================
          String replicaName = cdmw.ossupport.OS.getOptionValue (args, REPLICA_NAME);
          if (replicaName == "no") {
              System.err.println("Replica url undefined:");
              usage(System.err, args[0]);
              System.exit(FAILURE);
          }

          org.omg.CORBA.Object replicaRef = ni.resolve(replicaName);

          // get the PingableInterface proxy
          PingableInterface pingableObject = PingableInterfaceHelper.narrow(replicaRef);

          System.err.println("PingClient>> calling a single normal server..(id: " + replicaName + ")");

          for(int i1 = 0; i1<niter; i1++) {
              if (delay != 0 ) {
                  // call to nanosleep
                  Thread.sleep(sec, nsec);
                }
              long t1 = System.nanoTime();
              pingableObject.ping();
              data.add(new Long(System.nanoTime() - t1));
          }

          for (int i1 = 0; i1 < niter; ++i1) {
              osPerformanceLb.write(datalb.get(i).intValue());
              osPerformance.write(data.get(i).intValue());
            }
          osPerformanceLb.close();
          osPerformance.close();
          // write file for shell script execution control
          java.io.FileWriter feof = new  java.io.FileWriter("end_of_test.log");
          feof.write("EOF");
          feof.close();
      }
      catch(SystemException e) {
          e.printStackTrace();
          System.exit(-1);
      }
      catch(cdmw.common.Exception e0) {
          e0.printStackTrace();
          System.exit(-1);
      }
      catch(Exception e1) {
          System.err.println("Unexpected exception");
          System.exit(-1);
        }
    }
}
