/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.perftest.CDMWLBinvocation;

import java.io.PrintStream;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;

import cdmw.orbsupport.StrategyList;

public class MainServer {

    private static final String REPLICA_NAME = "--ReplicaName";

    private static final String GM_IOR_OPTION = "--GroupMngRef";

    private static final String HOST_NAME_OPTION = "--HostName";

    private static final String NAMING_REPOS_URL_OPTION = "--namingRepos-url";

    private static final String IOR_FILE_NAME = "hello_server.IOR";

    private static void usage(PrintStream os, String programName) {
    {
        os.println("\nUsage:\n"
        + programName + " options\n"
        + "\n"
        + "Options:\n"
        + "--help                                         Display this information.\n"
        + "\n"
        + "-ORBInitRef LBGroupManager=<rep_corbaloc>  ReplicationManager corbaloc\n"
        + "[" + REPLICA_NAME + "]=path          path of the file where interface IOR should be stored.\n"
        + "[" + HOST_NAME_OPTION +"]=hostname      hostname where server is launched.\n");
    }

    public static void main(String[] args) {
        // help argument
        if (args.length <= 2) {
            usage(System.err, args[0]);
            System.exit(0);
        }

        if (args.length >= 2) {
            if (args[1].equals("--help")) {
                usage(System.err, args[0]);
                System.exit(0);
            }
        }

        // get host name
        String hostName = cdmw.ossupport.OS.getOptionValue(args, "--HostName");
        // get Name File to save interface interface
        String replicaName = cdmw.ossupport.OS.getOptionValue(args, REPLICA_NAME);

        if (replicaName.equals("no")) {
            replicaName = new String("interface.ior");
        }

        try {
            String namingReposURL = cdmw.ossupport.OS.getOptionValue(args, NAMING_REPOS_URL_OPTION);

            if (namingReposURL.equals("no")) {
                System.err.println("naming and repository url undefined");
                usage(System.err, args[0]);
                System.exit(0);
            }

            // Initialises the ORB
            StrategyList strategyList = new StrategyList();
            strategyList.addORBThreaded();
            strategyList.addPOAThreadPerConnection();

            ORB orb = cdmw.orbsupport.ORBSupport.ORBInit(args, strategyList);

            // Get the root POA
            org.omg.CORBA.Object obj = orb.resolve_initial_references("RootPOA");
            POA rootPOA = POAHelper.narrow(obj);
            POAManager poaMgr = rootPOA.the_POAManager();
            poaMgr.activate();

            PingableInterfaceImpl pingableInterface = new PingableInterfaceImpl();
            PingableInterface pingableInterfaceRef = pingableInterface._this();
            // ===================================================
            // Get the naming and repository
            // from the input URL
            // ===================================================
            com.thalesgroup.CdmwNamingAndRepository.Repository repository = null;

            org.omg.CORBA.Object obj2 = orb.string_to_object(namingReposURL);
            repository = com.thalesgroup.CdmwNamingAndRepository.RepositoryHelper.narrow(obj2);
            com.thalesgroup.CdmwNamingAndRepository.NameDomain helloDomain = repository.resolve_name_domain("PERFTEST/HELLO_LB");

            // ========================================================
            // Register the HelloInterface object in the HELLO_LB name domain
            // ========================================================

            // reserve the name "HelloInterface" in the domain
            Any regId = helloDomain.new_name(replicaName);
            // register the object
            helloDomain.register_object(regId, pingableInterfaceRef);
            System.err.println("HelloServer>> Hello Server Started and Registered in the Interface Repository with name: "
                    + replicaName);

            orb.run();
        }
        catch (SystemException e) {
            e.printStackTrace();
            System.exit(-1);
        }
        catch (UserException e1) {
            e1.printStackTrace();
            System.exit(-1);
        }
        catch (Exception e3) {
            System.err.println("Uncaught exception");
            System.exit(-1);
        }

        System.exit(0);
    }

}
