/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.perftest.CDMWLBinvocation;

import java.io.FileNotFoundException;

import org.omg.CORBA.*;

import CdmwLB.LBGroupManager;

import cdmw.orbsupport.ExceptionMinorCodes;

public class TestUtils {
    public static final String[] locations = {"host1", "host2", "host3"};
    public static final String[] files = {"server1", "server2", "server3"};
    
    public static PingableInterface getPingableRefFromFile(ORB orb, String fileName) {
        org.omg.CORBA.Object objref = null;
        PingableInterface pingableRef = null;

        // import the object reference from the file
        try {
            java.io.FileReader is = new java.io.FileReader(fileName);
            char [] objrefStr;
            is.read(objrefStr);
            try {
                objref = orb.string_to_object(objrefStr.toString());
                pingableRef = PingableInterfaceHelper.narrow(objref);
              }
            catch(SystemException e) {
                System.err.println(" CORBA System Exception raised: " + e.getMessage());
                e.printStackTrace();
                throw e;
            }
            catch (Exception e1) {
                throw new INTERNAL(ExceptionMinorCodes.INTERNALLoadBalancingError,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }

            is.close();
        }
        catch(FileNotFoundException e) {
            System.err.println("Error opening reference filename: " + fileName);
            throw new INTERNAL(ExceptionMinorCodes.INTERNALLoadBalancingError,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        
        }
        return pingableRef;
    }
    
    public static org.omg.CORBA.Object createGroup(ORB orbTmp,
                            LBGroupManager groupManager,
                            final String repId,
                            final String policy)
    throws UserException
    {
        // set default properties
        System.err.println("set_lb_policy_property (default = RoundRobin)");
        org.omg.PortableGroup.Property[] prop = new org.omg.PortableGroup.Property[1];
        ORB orb = orbTmp;
        prop[0].nam = new org.omg.CosNaming.NameComponent[1];
        prop[0].nam[0].id="cdmw.lb.LoadBalancingPolicy";
        String lbPolicy = policy;
        prop[0].val.insert_string(lbPolicy);

        String repositoryId = repId;

        AnyHolder factoryCreationId = new AnyHolder();
        org.omg.PortableGroup.Property[] crit = new org.omg.PortableGroup.Property[1];
        crit[0].nam = new org.omg.CosNaming.NameComponent[1];
        crit[0].nam[0].id="cdmw.lb.LBProperties";
        org.omg.PortableGroup.PropertiesHelper.insert(crit[0].val,prop);

        org.omg.CORBA.Object objGroup = null;

        // create the object group
        System.err.println("Create an object group");
        try {
            objGroup =  groupManager.create_object(repId, crit, factoryCreationId);
          }
        catch(UserException e) {
            System.err.println(e.getMessage());
            throw e;
        }

        org.omg.CosNaming.NameComponent[] loc = new org.omg.CosNaming.NameComponent[3];
        loc[0].id = locations[0];
        loc[0].kind = "hostname";

        loc[1].id = "APPL";
        loc[1].kind = "application";
        loc[2].id = "PROC";
        loc[2].kind = "process";

        // add a new menber
        try {
            if (repositoryId.equals("IDL:CdmwLBPerf/PingableInterface:1.0")) {
                org.omg.CORBA.Object pingable = getPingableRefFromfile(orb, files[0]);

                System.err.println("Add new member on the group");
                objGroup = groupManager.add_member(objGroup, loc, pingable);

                loc[0].id = locations[1];
                pingable = getPingableRefFromFile(orb, files[1]);

                System.err.println("Add new member on the group"); 
                objGroup = groupManager.add_member(objGroup, loc, pingable);

                loc[0].id = locations[2];
                pingable = getPingableRefFromFile(orb, files[2]);

                System.err.println("Add new member on the group");
                objGroup = groupManager.add_member(objGroup, loc, pingable);
            }
        }
        catch( org.omg.PortableGroup.ObjectGroupNotFound e ) {
            System.err.println(" ObjectGroupNotFound Exception raised! ");
            (new Exception(" **** TEST FAILED AT LINE ")).printStackTrace();
            throw e;
        }
        catch( org.omg.PortableGroup.ObjectNotAdded e1) {
            System.err.println(" ObjectNotAdded Exception raised! ");
            (new Exception(" **** TEST FAILED AT LINE ")).printStackTrace();
            throw e1;
        }
        catch(UserException e2) {
            System.err.println(e2.getMessage());
            throw e2;
        }
        catch(SystemException e3) {
            System.err.println(e3.getMessage());
            throw e3;
        }
        return objGroup;
    }

}
