/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.perftest.CDMWMarshallingC;

import org.omg.CORBA.*;
import org.omg.CORBA.Object;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import org.omg.Messaging.*;

import cdmw.common.*;
import cdmw.orbsupport.*;
import cdmw.perftest.CDMWMarshallingC.ClockTime;

import com.thalesgroup.CdmwNamingAndRepository.*;
import com.thalesgroup.CdmwNamingAndRepository.InvalidName;
import com.thalesgroup.CdmwNamingAndRepository.NameDomainPackage.*;
import com.thalesgroup.CdmwPlatformMngt.ProcessDelegatePackage.*;
import com.thalesgroup.CdmwPlatformMngtBase.*;

public class ClientProcessControl extends cdmw.cdmwinit.ProcessControl {

    public static final String CALL_RETURN = "CR";

    public static final String ONE_WAY = "OW";

    public static final String ONE_WAY_TRANSPORT = "OW_TRANSPORT";

    public static final String ONE_WAY_SERVER = "OW_SERVER";

    public static final String ONE_WAY_TARGET = "OW_TARGET";

    public static final String TYPE_LONG = "LONG";

    public static final String TYPE_FLOAT = "FLOAT";

    public static final String TYPE_ANY = "ANY";

    public static final String TYPE_RECORD = "RECORD";

    public static final String TYPE_NONE = "NONE";

    /**
     * The orb reference.
     */
    private ORB orb;

    /**
     * The application name.
     */
    private String applicationName;

    /**
     * The process name.
     */
    private String processName;

    /**
     * TestPing reference
     */
    private Marshalling.TestPing testping;

    private int niter;

    private int delay;

    private String invocationType;

    private String fileprefix;

    private String dataType;

    private int dataSize;

    private ClockTime measure;

    public ClientProcessControl(
        ORB orb, int niter, int delay, String invocationType,
        String fileprefix, String dataType, int dataSize, ClockTime measureType) {

        this.orb = orb;
        this.applicationName = "";
        this.processName = "";
        this.niter = niter;
        this.delay = delay;
        this.invocationType = invocationType;
        this.dataType = dataType;
        this.dataSize = dataSize;
        this.fileprefix = fileprefix;
        this.measure = measureType;
    }

    /**
     * Purpose: <p> the behaviour for the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/initialise:1.0
     * operation
     */
    @Override
    public void onInitialise(StartupKind startupKind) throws SystemException {
        // ===================================================
        // Get the process name and application name
        // ===================================================
        this.processName = cdmw.cdmwinit.InitUtils.getCdmwProcessName();
        this.applicationName = cdmw.cdmwinit.InitUtils.getCdmwApplicationName();

        // =================================================================
        // example of using the PlatformInterface for notifying a message
        // =================================================================
        cdmw.platformmngt.PlatformInterface.notifyMessage(INF.value,
            this.processName,
            ">>>>>>>>>>>>>> Initialisation requested by supervision");

        // =========================================
        // retrieve Tester object from Repository
        // =========================================
        // get NamingInterface to test_perf
        cdmw.commonsvcs.naming.NamingInterface marshallingNamingInterface = cdmw.namingandrepository.RepositoryInterface
            .getDomainNamingInterface("perf_test");

        // Retrieve TestPing from NamingInterface
        String registrationName = "TestPing";
        try {
            org.omg.CORBA.Object obj = marshallingNamingInterface
                .resolve(registrationName);
            this.testping = Marshalling.TestPingHelper.narrow(obj);
        }
        catch (NotFound e) {
            System.err.println("Resolve " + registrationName
                + " => NamingContext.NotFound raised!");
        }
        catch (CannotProceed e1) {
            System.err.println("Resolve " + registrationName
                + " => NamingContext.CannotProceed raised!");
        }
        catch (org.omg.CosNaming.NamingContextPackage.InvalidName e2) {
            System.err.println("Resolve " + registrationName
                + " => NamingContext.InvalidName raised!");
        }
        catch (org.omg.CORBA.SystemException e) {
            System.err.println("Resolve " + registrationName
                + " => CORBA::SystemException raised!");
        }

    }

    /**
     * Purpose: <p> the behaviour for the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/run:1.0 operation
     */
    @Override
    public void onRun() throws NotReadyToRun {
        // =================================================================
        // example of using the PlatformInterface for notifying a message
        // =================================================================
        cdmw.platformmngt.PlatformInterface.notifyMessage(INF.value,
            this.processName, ">>>>>>>>>>>>>> Run requested by supervision");

        System.out.println("Start testping");

        double tdelta[] = new double[this.niter];

        // delay of measure
        int delays = this.delay;
        int delayn = (this.delay % 1000) * 1000000;

        Marshalling.LongSeq lseq;
        Marshalling.FloatSeq fseq;
        Any any = new Any();
        Marshalling.RecordSeq rseq;

        // creating object
        if (this.dataType.equals(TYPE_LONG)) {
            int lSeqSize = this.dataSize / 4;
            long[] lSeqBuffer;
            lSeqBuffer = new long[lSeqSize];
            lseq = new Marshalling.LongSeq(lSeqSize, lSeqSize, lSeqBuffer, 0);
            for (int i = 0; i < lSeqSize; i++ ) {
                lseq[i] = i;
            }
        }
        else if (this.dataType.equals(TYPE_FLOAT)) {
            int fSeqSize = this.dataSize / 4;
            float fSeqBuffer[] = new float[fSeqSize];
            fseq = new Marshalling.FloatSeq(fSeqSize, fSeqSize, fSeqBuffer, 0);
            for (int i = 0; i < fSeqSize; i++ ) {
                fseq[i] = i + 0.6;
            }
        }
        else if (this.dataType.equals(TYPE_ANY)) {
            char[] txt = new char[this.dataSize + 1];
            for (int i = 0; i < this.dataSize; i++ ) {
                txt[i] = 'A';
            }
            // txt[this.dataSize] = '\0';
            String str = txt.toString();
            any.insert_string(str);
            txt = null;
        }
        else if (this.dataType.equals(TYPE_RECORD)) {
            int rSeq_size = this.dataSize / 8;
            Marshalling.RecordStruct rSeq_buffer = new Marshalling.RecordStruct[rSeq_size];
            rseq = new Marshalling.RecordSeq(rSeq_size, rSeq_size, rSeq_buffer,
                0);
            for (int i = 0; i < rSeq_size; i++ ) {
                rseq[i].l = i;
                rseq[i].f = i + 0.9;
            }
        }

        // result of time measurement
        long time1, time2;

        // test CR: call and return
        if (this.invocationType.equals(CALL_RETURN)) {

            // select for data type
            if (this.dataType.equals(TYPE_LONG)) {
                // start of record
                for (int i = 0; i < this.niter; i++ ) {

                    // delay of measure
                    if (this.delay != 0) {
                        // call to nanosleep
                        Thread.sleep(delays, delayn);
                    }
                    time1 = this.measure.getTime();
                    this.testping.long_ping(lseq);
                    time2 = this.measure.getTime();
                    tdelta[i] = time2 - time1;
                }// end for
            }// end type data: LONG
            else if (this.dataType.equals(TYPE_FLOAT)) {
                // start of record
                for (int i = 0; i < this.niter; i++ ) {

                    // delay of measure
                    if (this.delay != 0) {
                        // call to nanosleep
                        Thread.sleep(delays, delayn);
                    }
                    time1 = this.measure.getTime();
                    this.testping.float_ping(fseq);
                    time2 = this.measure.getTime();
                    tdelta[i] = time2 - time1;

                }// end for
            } // end type data: FLOAT
            else if (this.dataType.equals(TYPE_ANY)) {
                // start of record
                for (int i = 0; i < this.niter; i++ ) {

                    // delay of measure
                    if (this.delay != 0) {
                        // call to nanosleep
                        Thread.sleep(delays, delayn);
                    }
                    time1 = this.measure.getTime();
                    this.testping.any_ping(any);
                    time2 = this.measure.getTime();
                    tdelta[i] = time2 - time1;

                }// end for
            } // end type data: ANY
            else if (this.dataType.equals(TYPE_RECORD)) {
                // start of record
                for (int i = 0; i < this.niter; i++ ) {

                    // delay of measure
                    if (this.delay != 0) {
                        // call to nanosleep
                        Thread.sleep(delays, delayn);
                    }
                    time1 = this.measure.getTime();
                    this.testping.record_ping(rseq);
                    time2 = measure.getTime();
                    tdelta[i] = time2 - time1;

                }// end for
            } // end type data: RECORD
            else if (this.dataType.equals(TYPE_NONE)) {
                // start of Invocation test
                for (int i = 0; i < this.niter; i++ ) {

                    // delay of measure
                    if (this.delay != 0) {
                        // call to nanosleep
                        Thread.sleep(delays, delayn);
                    }
                    time1 = this.measure.getTime();
                    this.testping.invocation_ping();
                    time2 = this.measure.getTime();
                    tdelta[i] = time2 - time1;

                }// end for
            } // end type data: NONE

        } // end case Call and Return

        else {
            org.omg.CORBA.Object obj = this.orb
                .resolve_initial_references("ORBPolicyManager");
            PolicyManager policyManager = PolicyManagerHelper.narrow(obj);

            obj = this.orb.resolve_initial_references("PolicyCurrent");
            PolicyCurrent policyCurrent = PolicyCurrentHelper.narrow(obj);

            short syncScope = SYNC_WITH_TRANSPORT.value;

            if (this.invocationType.equals(ONE_WAY)) {// Default SyncScope
                                                        // value.
                syncScope = SYNC_NONE.value;
            }
            else if (this.invocationType.equals(ONE_WAY_TRANSPORT)) {
                syncScope = SYNC_WITH_TRANSPORT.value;
            }
            else if (this.invocationType.equals(ONE_WAY_SERVER)) {
                syncScope = SYNC_WITH_SERVER.value;
            }
            else if (this.invocationType.equals(ONE_WAY_TARGET)) {
                syncScope = SYNC_WITH_TARGET.value;
            }

            // Set up the sync scope any.
            Any syncScopeAny = orb.create_any();
            syncScopeAny.insert_short(syncScope);

            // Set the length of the policy list.
            PolicyList syncScopePolicyList = new PolicyList();

            // Set up the sync scope policy.
            syncScopePolicyList.getList()[0] = this.orb.create_policy(
                SYNC_SCOPE_POLICY_TYPE.value, syncScopeAny);

            // Set the sync scope policy at the object level.
            obj = this.testping._set_policy_overrides(syncScopePolicyList,
                org.omg.CORBA.SetOverrideType.ADD_OVERRIDE);
            // Get the new object reference with the updated policy.
            this.testping = TestPingHelper.narrow(obj);

            // select for type of data
            if (this.dataType.equals(TYPE_LONG)) {
                // start of record
                for (int i = 0; i < this.niter; i++ ) {
                    // delay of measure
                    if (this.delay != 0) {
                        // call to nanosleep
                        Thread.sleep(delays, delayn);
                    }
                    time1 = this.measure.getTime();
                    this.testping.ow_long_ping(lseq.in());
                    time2 = this.measure.getTime();
                    // time read in micro sec.
                    tdelta[i] = time2 - time1;

                }// end for
            }// end data type: long

            else if (this.dataType.equals(TYPE_FLOAT)) {
                // start of record

                for (int i = 0; i < this.niter; i++ ) {
                    // delay of measure
                    if (this.delay != 0) {
                        // call to nanosleep
                        Thread.sleep(delays, delayn);
                    }
                    time1 = this.measure.getTime();
                    this.testping.ow_float_ping(fseq);
                    time2 = this.measure.getTime();
                    // time read in micro sec.
                    tdelta[i] = time2 - time1;

                }// end for
            }// end data type: float

            else if (this.dataType.equals(TYPE_ANY)) {
                // start of record

                for (int i = 0; i < this.niter; i++ ) {
                    // delay of measure
                    if (this.delay != 0) {
                        // call to nanosleep
                        Thread.sleep(delays, delayn);
                    }
                    time1 = this.measure.getTime();
                    this.testping.ow_any_ping(any);
                    time2 = this.measure.getTime();
                    // time read in micro sec.
                    tdelta[i] = time2 - time1;

                }// end for
            }// end data type: any

            else if (this.dataType.equals(TYPE_RECORD)) {
                // start of record

                for (int i = 0; i < this.niter; i++ ) {
                    // delay of measure
                    if (this.delay != 0) {
                        // call to nanosleep
                        Thread.sleep(delays, delayn);
                    }
                    time1 = this.measure.getTime();
                    this.testping.ow_record_ping(rseq);
                    time2 = this.measure.getTime();
                    // time read in micro sec.
                    tdelta[i] = time2 - time1;

                }// end for
            }// end data type: RECORD
            else if (this.dataType.equals(TYPE_NONE)) {
                // start of Invocation test

                for (int i = 0; i < this.niter; i++ ) {
                    // delay of measure
                    if (this.delay != 0) {
                        // call to nanosleep
                        Thread.sleep(delays, delayn);
                    }
                    time1 = this.measure.getTime();
                    this.testping.ow_invocation_ping();
                    time2 = this.measure.getTime();
                    // time read in micro sec.
                    tdelta[i] = time2 - time1;

                }// end for
            }// end data type: NONE

        }

        // write file
        java.io.FileWriter fd = new java.io.FileWriter(fileprefix
            + "DELTA_C.dat");

        for (int i = 0; i < this.niter; i++ ) {
            fd.write(String.format("%1.6f", tdelta[i]));
        }

        fd.close();

        java.io.FileWriter feof = new java.io.FileWriter("end_of_test.log");
        feof.write("EOF");
        feof.close();
    }

    /**
     * Purpose: <p> the behaviour for the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/stop:1.0 operation
     */
    @Override
    public void onStop() {
        // =================================================================
        // example of using the PlatformInterface for notifying a message
        // =================================================================
        try {
            cdmw.platformmngt.PlatformInterface.notifyMessage(INF.value,
                this.processName,
                ">>>>>>>>>>>>>> Stop requested by supervision");
        }
        catch (BadParameterException e) {
            e.printStackTrace();
        }
        catch (BadOrderException e) {
            e.printStackTrace();
        }

    }

}
