/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.perftest.CDMWMarshallingCCM;

import org.omg.CORBA.*;

import cdmw.common.BadOrderException;
import cdmw.common.BadParameterException;
import cdmw.orbsupport.CORBASystemExceptionWrapper;

import com.thalesgroup.CdmwPlatformMngt.ProcessDelegatePackage.NotReadyToRun;
import com.thalesgroup.CdmwPlatformMngtBase.StartupKind;

public class ServerProcessControl extends cdmw.cdmwinit.ProcessControl {

    /**
     * The orb reference.
     */
    private ORB orb;

    /**
     * The application name.
     */
    private String applicationName;

    /**
     * The process name.
     */
    private String processName;

    public ServerProcessControl(
        ORB orb) {
        this.orb = orb;
        this.applicationName = "";
        this.processName = "";
    }

    @Override
    public void onInitialise(StartupKind startupKind) {
        // ===================================================
        // Get the process name and application name
        // ===================================================
        try {
            this.processName = cdmw.cdmwinit.InitUtils.getCdmwProcessName();
            this.applicationName = cdmw.cdmwinit.InitUtils
                .getCdmwApplicationName();
        }
        catch (CORBASystemExceptionWrapper e2) {
            e2.printStackTrace();
        }

        // =================================================================
        // example of using the PlatformInterface for notifying a message
        // =================================================================
        try {
            cdmw.platformmngt.PlatformInterface.notifyMessage(
                com.thalesgroup.CdmwPlatformMngtBase.INF.value,
                this.processName,
                ">>>>>>>>>>>>>> Initialisation requested by supervision");
        }
        catch (BadOrderException e1) {
            e1.printStackTrace();
        }
        catch (BadParameterException e1) {
            e1.printStackTrace();
        }

        // We wait that assembly call ccm activate
        // (which must be done before the init timout specified in Scenario.xml)
        synchronized (this) {
            System.out.println("Wait for ccm_activate..");
            try {
                this.wait();
            }
            catch (InterruptedException e) {
                e.printStackTrace();
            }
            System.out.println("ccm_activate called..");
        }
    }

    @Override
    public void onRun() throws NotReadyToRun {
        // =================================================================
        // example of using the PlatformInterface for notifying a message
        // =================================================================
        try {
            cdmw.platformmngt.PlatformInterface
                .notifyMessage(com.thalesgroup.CdmwPlatformMngtBase.INF.value,
                    this.processName,
                    ">>>>>>>>>>>>>> Run requested by supervision");
        }
        catch (BadOrderException e) {
            e.printStackTrace();
        }
        catch (BadParameterException e) {
            e.printStackTrace();
        }

        // do nothing
    }

    @Override
    public void onStop() {
        // =================================================================
        // example of using the PlatformInterface for notifying a message
        // =================================================================
        try {
            cdmw.platformmngt.PlatformInterface.notifyMessage(
                com.thalesgroup.CdmwPlatformMngtBase.INF.value,
                this.processName,
                ">>>>>>>>>>>>>> Stop requested by supervision");
        }
        catch (BadOrderException e) {
            e.printStackTrace();
        }
        catch (BadParameterException e) {
            e.printStackTrace();
        }
    }

    // call by server executor when ccm_activate received
    public synchronized void ccmActivated() {
        System.out
            .println("Assembly ready: unlock init -> PFMNG will trigger client init");
        this.notify();
    }

}
