/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.perftests.CDMWParallelism;

import java.io.*;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;

import cdmw.orbsupport.*;
import cdmw.cdmwinit.*;

public class Client {

    private static final int SUCCESS = 0;

    private static final int FAILURE = 1;

    private static final int POA_THREAD_POOL_SIZE = 5;

    public static void main(String[] args) {    
        int status = SUCCESS;

        System.err.println("-------------------------");
        System.err.println("      Start Client");
        System.err.println("-------------------------");
        for (int n = 0; n < args.length; n++)
            System.err.println("args[" + n +"]: " + args[n]);  
        System.err.println("-------------------------");
        
        // number of start-up ping
        int nstart = 10;

        // number of test
        int niter = 1000;
        
        //number of thread
        int nthread = 5;

        // output file prefix
        String fileprefix ="";

        System.out.println("Reading Configuration File...");
        // read configuration file
        
        StringBuilder sb = new StringBuilder();
        char ch = 0;
        int i = 0;
        try {
            InputStreamReader isr = new FileReader("client.cfg");
            while (i != -1) {
                i = isr.read();
                ch = (char) i;
                sb.append(ch);
            }
            isr.close();
        }
        catch (FileNotFoundException e) {
            System.err.println("Error: client.cfg don't exist!\n");
            System.exit(-1);
        }
        catch (IOException e) {
            e.printStackTrace();
        }
        String fData = sb.toString();

        String[] tokens = fData.split("\\s+");
        try {
            nstart = Integer.parseInt(tokens[0]);
            niter = Integer.parseInt(tokens[1]);
            nthread = Integer.parseInt(tokens[2]);
            fileprefix = tokens[3];            
        }
        catch (NumberFormatException e) {
            e.printStackTrace();
            System.exit(FAILURE);
        }
          
        System.out.println("Thread startup invocations= " + nstart);            
        System.out.println("Total invocations= " + niter);   
        System.out.println("Test invocation per thread= " + (niter / nthread));
        System.out.println("Prefix of output file names= " + fileprefix);            
        
        ORB orb = null;                        // orb reference
        try {
            // ===================================================
            // Initialize the ORB
            // ===================================================
            StrategyList orbStrategies = new StrategyList();
            orbStrategies.addORBThreaded();
            orbStrategies.addPOAThreadPool(POA_THREAD_POOL_SIZE);

            orb = ORBSupport.ORBInit(args, orbStrategies);
            
            // ===================================================
            // Get the root POA
            // ===================================================
            org.omg.CORBA.Object obj = orb.resolve_initial_references("RootPOA");
            POA rootPOA = POAHelper.narrow(obj);

            // ===================================================
            // Activate the root POA manager
            // ===================================================
            POAManager poaManager = rootPOA.the_POAManager();
            poaManager.activate();
            
            // ===================================================
            // create the process control for platform management
            // ===================================================
            ProcessControl processCtrl = new ClientProcessControl(orb, nstart, niter, nthread, fileprefix);

            // ===================================================
            // Call generated CdmwInit
            // ===================================================
            cdmw.cdmwinit.CdmwInterface.cdmwInit(orb, args, processCtrl, "cdmw.generated.ClientCdmwInterface");
            // ===================================================
            // wait for CORBA message
            // ===================================================
            orb.run();

        } 
        catch (cdmw.common.Exception ex1) {
            System.err.println(ex1.getMessage());
            status = FAILURE;
        } 
        catch(UserException ex2) {        
            System.err.println(ex2.getMessage());
            status = FAILURE;
        }
        catch(SystemException ex4) {        
            System.err.println(ex4.getMessage());
            status = FAILURE;
        }
        catch(Exception ex3) {        
            System.err.println("unexpected exception");
            status = FAILURE;
        }


        // ========================================================
        // program stopping
        // ========================================================
        
        // ===================================================
        // Call generated Cdmw cleanup
        // ===================================================
        cdmw.cdmwinit.CdmwInterface.cdmwCleanup(orb, "cdmw.generated.ClientCdmwInterface");
        // ===================================================
        // Call ORB cleanup
        // ===================================================
        cdmw.orbsupport.ORBSupport.ORBCleanup(orb);
                
        // ===================================================
        // destroy orb
        // ===================================================
        
        if (orb != null) {
            try {
                orb.destroy();
            }
            catch(org.omg.CORBA.SystemException ex) {           
                ex.printStackTrace();
                status = FAILURE;
            }
        }        

        System.exit(status);    
    }
}
