/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.perftests.CDMWParallelism;

import java.util.Calendar;
import java.util.Timer;

import org.omg.CORBA.*;
import org.omg.Messaging.*;

import com.thalesgroup.CdmwPlatformMngt.ProcessDelegatePackage.NotReadyToRun;
import com.thalesgroup.CdmwPlatformMngtBase.StartupKind;

public class ClientProcessControl extends cdmw.cdmwinit.ProcessControl {
    
    private static short syncScope;
    /**
     * The orb reference.
     */
    private ORB orb;

    /**
     * The application name.
     */
    private String applicationName;

    /**
     * The process name.
     */
    private String processName;

    /**
     * Tester reference
     */
    private Parallelism.Tester tester;

    /**
     * Array of Parallelism thread
     */
    private ParallelismThread[] benchThread;

    private int nstart;

    private int niter;

    private int nthread;

    private String fileprefix;
    
    public ClientProcessControl (ORB orb,
                                 int nstart, 
                                 int niter,
                                 int nthread,
                                 String fileprefix) {
        this.orb = orb;
        this.applicationName = "";
        this.processName =""; 
        this.nstart = nstart;
        this.niter = niter;
        this.nthread = nthread;
        this.fileprefix = fileprefix;
}

    @Override
    public void onInitialise(StartupKind startupKind) {
//      ===================================================
        // Get the process name and application name
        // ===================================================
        this.processName = cdmw.cdmwinit.InitUtils.getCdmwProcessName();
            
        this.applicationName = cdmw.cdmwinit.InitUtils.getCdmwApplicationName();
        
        // =================================================================
        // example of using the PlatformInterface for notifying a message
        // =================================================================
        cdmw.platformmngt.PlatformInterface.notifyMessage(com.thalesgroup.CdmwPlatformMngtBase.INF.value,
                                 this.processName, 
                                 ">>>>>>>>>>>>>> Initialisation requested by supervision");
        
        // =========================================
        // retrieve Tester object from Repository
        // =========================================
        // get NamingInterface to perf_test
        cdmw.commonsvcs.naming.NamingInterface ParallelismNamingInterface =
          cdmw.namingandrepository.RepositoryInterface.getDomainNamingInterface ("perf_test");
        
        // Retrieve Tester from NamingInterface
        String registrationName = "Tester";
        try {
          org.omg.CORBA.Object obj = ParallelismNamingInterface.resolve (registrationName);
          this.tester = Parallelism.TesterHelper.narrow(obj);
          
          //Set the policy to Oneway SYNC_NONE 
          obj =
            this.orb.resolve_initial_references ("ORBPolicyManager");
          PolicyManager policyManager =
            PolicyManagerHelper.narrow(obj);
          
          obj = this.orb.resolve_initial_references ("PolicyCurrent");
          PolicyCurrent policyCurrent =
            PolicyCurrentHelper.narrow(obj);
          
          syncScope = SYNC_WITH_TRANSPORT.value;
          
          // Set up the sync scope any.
          Any syncScopeAny;
          syncScopeAny.insert_short(syncScope);
          
          // Set the length of the policy list.
          org.omg.CORBA.Policy[] syncScopePolicylist = new org.omg.CORBA.Policy[1];
          
          // Set up the sync scope policy.
          syncScopePolicyList[0] =
              this.orb.create_policy (SYNC_SCOPE_POLICY_TYPE.value,
                      syncScopeAny);
          
          // Set the sync scope policy at the object level.
          obj = this.tester._set_policy_overrides (syncScopePolicylist,
              SetOverrideType.ADD_OVERRIDE);
          // Get the new object reference with the updated policy.
          this.tester = Parallelism.TesterHelper.narrow(obj);
          
        }
        catch(org.omg.CosNaming.NamingContextPackage.NotFound e) {
            System.err.println("Resolve " + registrationName 
              + " => NamingContext::NotFound raised!");
        }
        catch(org.omg.CosNaming.NamingContextPackage.CannotProceed e1) {
            System.err.println("Resolve " + registrationName 
              + " => NamingContext::CannotProceed raised!");
          }
        catch(org.omg.CosNaming.NamingContextPackage.InvalidName e2) {
            System.err.println("Resolve " + registrationName 
              + " => NamingContext::InvalidName raised!");
          }
        catch (SystemException e) {
            System.err.println("Resolve " + registrationName 
              + " => CORBA::SystemException raised!");
          }                
    }

    @Override
    public void onRun() throws NotReadyToRun {
        // =================================================================
        // example of using the PlatformInterface for notifying a message
        // =================================================================
        cdmw.platformmngt.PlatformInterface.notifyMessage(com.thalesgroup.CdmwPlatformMngtBase.INF.value,
                                 this.processName, 
                                 ">>>>>>>>>>>>>> Run requested by supervision");
        
        // ======================
        // run Parallelism thread
        // ======================
        int n_iter_x_thread = this.niter / this.nthread;  // number of sample invocation x thread
        
        
        //    long double tdelta[m_nthread];

        // Thread initialization
        this.benchThread = new ParallelismThread[this.nthread];
        for (int i = 0; i< this.nthread; i++) {
            this.benchThread[i].setTester(this.tester);
            this.benchThread[i].setNIter(n_iter_x_thread);
            this.benchThread[i].setNStart(this.nstart);
        }


        //    int status1 = ::gettimeofday(& timeval1, NULL);
        long time1 = Calendar.getInstance().getTimeInMillis();
        
        // Start threads
        for (int i = 0; i< this.nthread; i++) {
            this.benchThread[i].start();
        }
        
        
        // Wait threads
        for (int i = 0; i< this.nthread; i++) {
            this.benchThread[i].join();
        }
        
        long time2 = Calendar.getInstance().getTimeInMillis();

        long time_start_min = this.benchThread[0].getStartTime();
        long time_end_max = this.benchThread[0].getEndTime();

        //Read time of invocation of each thread
        for (int i = 1; i< this.nthread; i++) {
          if (time_start_min > this.benchThread[i].getStartTime()) {
        time_start_min = this.benchThread[i].getStartTime();
          };
          if (time_end_max < this.benchtThread[i].getEndTime()) {
        time_end_max = this.benchThread[i].getEndTime();
          };
        }

        long tdeltatot = time2 - time1;
        long tdeltatot2 = time_end_max - time_start_min;

        System.err.println("timestart main= " + time1);
        System.err.println("timestart 1st thread= " + time_start_min );
        System.err.println("timeend main= " + time2);
        System.err.println("timeend last thread= " + time_end_max);

        System.err.println("Total time of work main= " + tdeltatot);
        System.err.println("Total time of work thread= " + tdeltatot2);
        System.err.println("Mean time for invocation= " + tdeltatot2 * 1.0 / (n_iter_x_thread * this.nthread));

        //write file
        
        String fname = new String(fileprefix + "DELTA_C.dat");
        java.io.FileWriter fd = new java.io.FileWriter(fname);
        fd.write(String.format("%1.6f", (( tdeltatot2 * 1.0 ) / (n_iter_x_thread * this.nthread))));
        fd.close();
        System.err.println("End testping");

        java.io.FileWriter feof = new java.io.FileWriter("end_of_test.log");
        feof.write("EOF");
        feof.close();
    }

    @Override
    public void onStop() {
        // =================================================================
        // example of using the PlatformInterface for notifying a message
        // =================================================================
        cdmw.platformmngt.PlatformInterface.notifyMessage(com.thalesgroup.CdmwPlatformMngtBase.INF.value,
                                 this.processName, 
                                 ">>>>>>>>>>>>>> Stop requested by supervision");
        for (int i = 0; i< this.nthread; i++) {
          this.benchThread[i].stop();
          this.benchThread[i].join();
        }
        benchThread = null;
    }

}
