/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.perftests.CDMWParallelism;

import java.util.Calendar;

import org.omg.CORBA.SystemException;

public class ParallelismThread extends Thread {
    /**
     * Tester reference
     */
    private Parallelism.Tester tester;
    
    /**
     * thread stop flag
     */
    private boolean stopped = false;
    
    /**
     * Number of Parallelism call for each type
     */
    private int nbPingTest;
    
    /**
     * Number of Parallelism call for each type
     */
    private int nbPingStart;
    
    /**
     * Start and end time
     */
    private long timeval1, timeval2;
    
    public ParallelismThread() {};
    
    public ParallelismThread(Parallelism.Tester tester,
                             int niter, 
                             int nstart) {
        this.tester = tester;
        this.nbPingTest = niter;
        this.nbPingStart = nstart;
    }

    public void setNIter(int niter) {
        this.nbPingTest = niter;
    } 

    public void setNStart(int nstart) {
        this.nbPingStart = nstart;
    } 

    public void setTester(Parallelism.Tester tester) {
        this.tester = tester;
    } 

    public long getTimeOfTest() {
        return this.timeval2 - this.timeval1;
    }

    public long getStartTime() {
        return this.timeval1;
    }

    public long getEndTime() {
        return this.timeval2;
    }

    public void run() {
        try {
            int i;
            // Startup samples
            for (i = 0; !this.stopped && i < this.nbPingStart; i++) {
                testPing();        
            }
            // Test samples
            this.timeval1 = Calendar.getInstance().getTimeInMillis();
     
            for (i = 0; !this.stopped && i < this.nbPingTest; i++) {
                testPing();
        
            }
            this.timeval2 = Calendar.getInstance().getTimeInMillis();
        }
        catch (SystemException e) 
          {
            System.err.println("SystemException: " 
                      + cdmw.orbsupport.ORBSupport.exceptionToString(e));
          }

    }

    public void testPing () {
        this.tester.ping();
    }    

    public void onStop() {
        this.stopped = true;
    }
}
