/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.perftests.CDMWParallelism;

import java.io.*;

import org.omg.CORBA.ORB;
import org.omg.CORBA.SystemException;
import org.omg.CORBA.UserException;
import org.omg.PortableServer.*;

import cdmw.orbsupport.StrategyList;

public class Server {
    private static final int SUCCESS = 0;

    private static final int FAILURE = 1;
    
    private static final int POA_THREAD_POOL_SIZE = 5;

    public static void main(String args[]) {
        int status = SUCCESS;

        System.err.println("-------------------------");
        System.err.println("      Start Server");
        System.err.println("-------------------------");
        for (int n = 0; n < args.length; n++ )
            System.err.println("args[" + n + "]: " + args[n]);
        System.err.println("-------------------------");

        ORB orb = null; // orb reference

        try {
            // ===================================================
            // Initialize the ORB
            // ===================================================
            StrategyList orbStrategies = new StrategyList();
            orbStrategies.addORBThreaded();
            orbStrategies.addPOAThreadPool(POA_THREAD_POOL_SIZE);

            orb = cdmw.orbsupport.ORBSupport.ORBInit(args, orbStrategies);

            // ===================================================
            // Get the root POA
            // ===================================================
            org.omg.CORBA.Object obj = orb.resolve_initial_references("RootPOA");
            POA rootPOA = POAHelper.narrow(obj);

            // ===================================================
            // Activate the root POA manager
            // ===================================================
            POAManager poaManager = rootPOA.the_POAManager();
            poaManager.activate();

            // ===================================================
            // create the process control for platform management
            // ===================================================
            cdmw.cdmwinit.ProcessControl processCtrl = new ServerProcessControl(orb);

            // ===================================================
            // Call generated CdmwInit
            // ===================================================
            cdmw.cdmwinit.CdmwInterface.cdmwInit(orb, args, processCtrl,  "cdmw.generated.ServerCdmwInterface");

            // ===================================================
            // wait for CORBA message
            // ===================================================
            orb.run();

        }
        catch (cdmw.common.Exception ex1) {
            ex1.printStackTrace();
            status = FAILURE;
        }
        catch (SystemException ex2) {
            ex2.printStackTrace();
            status = FAILURE;
        }
        catch (UserException ex3) {
            ex3.printStackTrace();
            status = FAILURE;
        }
        catch (Exception ex4) {
            System.err.println("unexpected exception");
            status = FAILURE;
        }

        // ========================================================
        // program stopping
        // ========================================================

        // ===================================================
        // Call generated Cdmw cleanup
        // ===================================================
        cdmw.cdmwinit.CdmwInterface.cdmwCleanup(orb, "cdmw.generated.ServerCdmwInterface");

        // ===================================================
        // Call ORB cleanup
        // ===================================================
        cdmw.orbsupport.ORBSupport.ORBCleanup(orb);

        // ===================================================
        // destroy orb
        // ===================================================
        if (orb != null) {
            try {
                orb.destroy();
            }
            catch (SystemException ex1) {
                ex1.printStackTrace();
                status = FAILURE;
            }
        }

        System.exit(status);
    }
}
