/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.perftests.CDMWParallelism;

import org.omg.CORBA.Any;
import org.omg.CORBA.ORB;

import cdmw.common.Assert;
import cdmw.common.BadOrderException;
import cdmw.common.BadParameterException;
import cdmw.namingandrepository.RepositoryInterface;

import com.thalesgroup.CdmwPlatformMngt.ProcessDelegatePackage.*;
import com.thalesgroup.CdmwPlatformMngtBase.*;
import com.thalesgroup.CdmwNamingAndRepository.*;
import com.thalesgroup.CdmwNamingAndRepository.NameDomainPackage.*;

public class ServerProcessControl extends cdmw.cdmwinit.ProcessControl {
    /**
     * The orb reference.
     */
    private ORB orb;

    /**
     * The application name.
     */
    private String applicationName;

    /**
     * The process name.
     */
    private String processName;
    
    /**
     * The Tester servant
     */
    private TesterImpl testerImpl;

    public ServerProcessControl(ORB orb) {
        this.orb = orb;
        applicationName = new String("");
        processName = new String("");
    }

    /**
     * Purpose: <p> the behaviour for the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/initialise:1.0
     * operation
     */
    @Override
    public void onInitialise(StartupKind startupKind) {
        // ===================================================
        // Get the process name and application name
        // ===================================================
        this.processName = cdmw.cdmwinit.InitUtils.getCdmwProcessName();
        this.applicationName = cdmw.cdmwinit.InitUtils.getCdmwApplicationName();

        // =================================================================
        // example of using the PlatformInterface for notifying a message
        // =================================================================
        cdmw.platformmngt.PlatformInterface.notifyMessage(INF.value,
            this.processName,
            ">>>>>>>>>>>>>> Initialisation requested by supervision");

        // ===============================
        // create I and Tester objects
        // ===============================
        
        this.testerImpl = new TesterImpl();
        Parallelism.Tester tester = this.TesterImpl._this();

        // ===================================
        // register Tester into Repository
        // ===================================
        // Get the Cdmw repository
        Repository repository = RepositoryInterface.getRepository();

        // Get Parallelism name domain from repository
        NameDomain parallelismDomain;
        try {
            parallelismDomain = repository.resolve_name_domain("perf_test");
        }
        catch (NoNameDomain e) {
            System.err.println("Can't find NameDomain perf_test");
            Assert.check(false);
        }
        catch (InvalidName e1) {
            System.err.println("NameDomain perf_test is invalid !!");
            Assert.check(false);
        }

        // Register the TestPing object in the "perf_test" name domain
        String registrationName = "TestPing";
        try {
            // reserve the name "TestPing" in the domain
            Any regId = parallelismDomain.new_name(registrationName);

            // register the object
            parallelismDomain.register_object(regId, tester);

        }
        catch (AlreadyExists e) {
            System.err.println("Name " + registrationName
                + " already exist into NameDomain perf_test");
            Assert.check(false);
        }
        catch (InvalidName e1) {
            System.err.println("Name " + registrationName
                + " is an invalid name !!");
            Assert.check(false);
        }
        catch (InvalidRegistration e2) {
            System.err.println("Invalid Registration of Tester reference as "
                + registrationName);
            Assert.check(false);
        }

    }

    /**
     * Purpose: <p> the behaviour for the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/run:1.0 operation
     */
    @Override
    public void onRun() throws NotReadyToRun {
        // =================================================================
        // example of using the PlatformInterface for notifying a message
        // =================================================================
        try {
            cdmw.platformmngt.PlatformInterface.notifyMessage(INF.value, this.processName,
                    ">>>>>>>>>>>>>> Run requested by supervision");
        }
        catch (BadOrderException e) {
            e.printStackTrace();
        }
        catch (BadParameterException e) {
            e.printStackTrace();
        }

    }

    /**
     * Purpose: <p> the behaviour for the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/stop:1.0 operation
     */
    @Override
    public void onStop() {
        // =================================================================
        // example of using the PlatformInterface for notifying a message
        // =================================================================
        try {
            cdmw.platformmngt.PlatformInterface.notifyMessage(INF.value, this.processName,
                ">>>>>>>>>>>>>> Stop requested by supervision");
        }
        catch (BadOrderException e) {
            e.printStackTrace();
        }
        catch (BadParameterException e) {
            e.printStackTrace();
        }
    }

}
