/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.perftests.CDMWSMGnotification;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;

public class HelloProcess {
    private static final int SUCCESS = 0;
    private static final int FAILURE = 1;

    private static final int POA_THREAD_POOL_SIZE = 5;

    public static void main(String[] args) {
        int status = SUCCESS;

        // wait for actviation of other process
        try {
            Thread.sleep(1);
        }
        catch (InterruptedException e) {
            e.printStackTrace();
        }


        System.err.println("-------------------------");
        System.err.println("      Start Process");
        System.err.println("-------------------------");
        
        for (int n=0; n < args.length; n++)
          System.err.println("args[" + n +"]: " + args[n]);
        System.err.println("-------------------------");

        ORB orb = null;                        // orb reference

        try {
            // ===================================================
            // Initialize the ORB
            // ===================================================
            cdmw.orbsupport.StrategyList orbStrategies = new cdmw.orbsupport.StrategyList();
            orbStrategies.addORBThreaded();
            orbStrategies.addPOAThreadPool(POA_THREAD_POOL_SIZE);

            orb = cdmw.orbsupport.ORBSupport.ORBInit(args, orbStrategies);

            // ===================================================
            // Get the root POA
            // ===================================================
            org.omg.CORBA.Object obj = orb.resolve_initial_references("RootPOA");
            POA rootPOA = POAHelper.narrow(obj);

            // ===================================================
            // Activate the root POA manager
            // ===================================================
            POAManager poaManager = rootPOA.the_POAManager();
            poaManager.activate();


            // ==============================================
            // Initialise the platform management interface
            // ==============================================

            // Call platform interface setup
            cdmw.platformmngt.PlatformInterface.setup(orb, args);

            // creates the hello process behaviour
            HelloProcessBehaviour pHelloProcess =  new HelloProcessBehaviour(orb);

            // acknowledge the creation of the process
            cdmw.platformmngt.PlatformInterface.acknowledgeCreation(pHelloProcess);

            // ===================================================
            // wait for CORBA message
            // ===================================================
            orb.run();
        }
        catch (cdmw.common.BadParameterException ex) {
            System.err.println("BAD PARAMETER " + ex.getMessage());
            status = FAILURE;
        }
        catch (cdmw.common.Exception ex1) {
            System.err.println(" generic exception " + ex1.getMessage());
            status = FAILURE;
        }
        catch(UserException ex2) {
            System.err.println(ex2.toString());
            status = FAILURE;
        }
        catch(SystemException ex3) {
            System.err.println(ex3.toString());
            status = FAILURE;
        }

        // ========================================================
        // program stopping
        // ========================================================

        //
        // Call ORB cleanup
        // ===================================================
        cdmw.orbsupport.ORBSupport.ORBCleanup(orb);

        // =====================================================
        // Destroy orb
        // =====================================================
        if (orb != null) {
            try {
                orb.destroy();
            }
            catch(SystemException ex3) {
                System.err.println(ex3.toString());
                status = FAILURE;
            }
        }

        System.exit(status);
    }
}
