/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.perftests.CDMWSMGnotification;

import org.omg.CORBA.*;

import cdmw.common.BadOrderException;

import com.thalesgroup.CdmwPlatformMngt.ProcessDelegatePackage.NotReadyToRun;
import com.thalesgroup.CdmwPlatformMngtBase.StartupKind;

/**
 * Purpose: <p> The abstract process behaviour.
 */
public class HelloProcessBehaviour extends cdmw.platformmngt.ProcessBehaviour {

    /**
     * The ORB.
     */
    private ORB orb;

    /**
     * The application name.
     */
    private String applicationName;

    /**
     * The process name.
     */
    private String processName;

    /**
     * Purpose: <p> The constructor.
     */
    public HelloProcessBehaviour(
        ORB orb) {
        this.orb = orb;
    }

    /**
     * Purpose: <p> the behaviour for the
     * IDL:thalesgroup.com/CdmwSystemMngt/platformlibrary/Process/nb_steps:1.0
     * attribute
     */
    public int nbSteps() {
        return 1;
    }

    /**
     * Purpose: <p> the behaviour for the
     * IDL:thalesgroup.com/CdmwSystemMngt/platformlibrary/Process/get_service:1.0
     * operation
     */
    public org.omg.CORBA.Object getService() {
        return null;
    }

    /**
     * Purpose: <p> the behaviour for the
     * IDL:thalesgroup.com/CdmwSystemMngt/platformlibrary/Process/initialise:1.0
     * operation
     */
    @Override
    public void initialise(StartupKind startupKind) {
        // ==============================================================
        // get application and process names
        // ==============================================================
        try {
            this.applicationName = cdmw.platformmngt.PlatformInterface
                .getApplicationName();
            this.processName = cdmw.platformmngt.PlatformInterface
                .getProcessName();
        }
        catch (BadOrderException e) {
            e.printStackTrace();
        }

    }

    /**
     * Purpose: <p> the behaviour for the
     * IDL:thalesgroup.com/CdmwSystemMngt/platformlibrary/Process/run:1.0
     * operation
     */
    @Override
    public void run() throws NotReadyToRun {
    }

    /**
     * Purpose: <p> the behaviour for the
     * IDL:thalesgroup.com/CdmwSystemMngt/platformlibrary/Process/stop:1.0
     * operation
     */
    @Override
    public void stop() {
        // ==============================================================
        // Shut down the orb
        // ==============================================================
        this.orb.shutdown(false);
    }

    /**
     * The behaviour for the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/next_step:1.0
     * operation
     */
    public void nextStep()
        throws com.thalesgroup.CdmwPlatformMngt.ProcessDelegatePackage.InvalidStep {
    }

}
