/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef ___CCM_ALL_CAPABILITIES_TEST_CCM_CLIENT_IMPL_HPP__
#define ___CCM_ALL_CAPABILITIES_TEST_CCM_CLIENT_IMPL_HPP__

#include <Foundation/logging/Logger.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <Foundation/osthreads/ThreadHandle.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/osthreads/Mutex.hpp>
#include <CCMContainer/ccmcif/SessionContextExt_impl.hpp>
#include <Time/idllib/CosClockService.stub.hpp>
#include <TraceAndPerf/recordingidl/CdmwRecordingData.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtExternalSystem.stub.hpp>

#include "ccm_all_capabilities/User_ccm_all_capabilities.skel.hpp"
#include <vector>
#include <queue>


namespace CcmAllCapabilitiesTest
{

    class CCM_Client_impl;
    

    class CcmAllCapabilitiesTester : public CppUnit::TestFixture
    {
        CCM_Client_impl * m_user_ccm_client;
        ::CcmAllCapabilitiesTest::CCM_Client_Context_var m_session_context;
        ::CcmAllCapabilitiesTest::Client_var m_component_ref;

    public:
        CcmAllCapabilitiesTester()
        {
            CORBA::Object_var obj1 = 
                Cdmw::TestUtils::CORBATestManager::instance()->get_object(
                    "client_context");
            m_session_context =
                ::CcmAllCapabilitiesTest::CCM_Client_Context::_narrow(
                    obj1.in());
        };


        ~CcmAllCapabilitiesTester()
        {
        };

        //
        //
        void do_tests();

        void tearDown()
        {
            //TEST_INFO("tearDown()");
            m_session_context = ::CcmAllCapabilitiesTest::CCM_Client_Context::_nil();
            m_component_ref = ::CcmAllCapabilitiesTest::Client::_nil();
        }

    protected:
        // Check the connections 
        // with the currently attached fault tolerant server
        void test_ft_connections(const char* expectedHostname);

        // Check the connections 
        // with the currently attached load balanced server
        char* test_lb_connections(void);

        // find in record list a "parse" request 
        // from server_id containing specified component_name
        // and with specified message
        bool find_parse_record(const char* component_name,
                               const char* message);

        CPPUNIT_TEST_SUITE( CcmAllCapabilitiesTester );
        CPPUNIT_TEST( do_tests );
        CPPUNIT_TEST_SUITE_END();

    };


    class TestThread : public Cdmw::OsSupport::Thread 
    {
    public:
        TestThread()
        {
        };

    protected:
        void run() throw()
        {
            Cdmw::TestUtils::CORBATestManager* testManager =
                Cdmw::TestUtils::CORBATestManager::instance();
            Cdmw::TestUtils::ThreadedTestRunner runner(
                testManager->get_controller());
            runner.addTest(
                CppUnit::TestFactoryRegistry::getRegistry(
                    "CcmAllCapabilitiesTester").makeTest());
            runner.start();
            runner.join();
        };

    private:
        TestThread(const TestThread&);
        TestThread& operator = (const TestThread&);
    };


    //
    // IDL:thalesgroup.com/UserCcmAllCapabilitiesTest/CCM_Client:1.0
    //
    class CCM_Client_impl : 
        virtual public ::UserCcmAllCapabilitiesTest::User_CCM_Client,
        virtual public Cdmw::OrbSupport::RefCountLocalObject
    {

    public:

        CCM_Client_impl();
        ~CCM_Client_impl();


        //
        // attribute expectedPrimaryHostname
        //
        virtual char * expectedPrimaryHostname ()
            throw (CORBA::SystemException);
        virtual void expectedPrimaryHostname (const char * a)
            throw (CORBA::SystemException);

        //
        // attribute expectedBackupHostname
        //
        virtual char * expectedBackupHostname ()
            throw (CORBA::SystemException);
        virtual void expectedBackupHostname (const char * a)
            throw (CORBA::SystemException);

        //
        // attribute fileName
        //
        virtual
        char*
        fileName()
            throw(CORBA::SystemException);

        virtual
        void
        fileName(const char* name)
            throw(CORBA::SystemException);


        //
        // facet clientHostInfo
        //
        virtual ::CcmAllCapabilitiesTest::CCM_HostInfo_ptr get_clientHostInfo()
            throw (CORBA::SystemException);

        virtual char* get_hostname()
            throw (CORBA::SystemException);

        virtual TimeBase::TimeT get_local_time()
            throw (CORBA::SystemException);


        //
        // consumer hostnameResults
        //
        virtual void push_hostnameResults(
            CcmAllCapabilitiesTest::HostnameResult * event
            )
            throw (CORBA::SystemException);


        //
        // Session component specific operations
        //
        void set_session_context(Components::SessionContext_ptr ctx)
            throw(CORBA::SystemException);
        void cdmw_initialise_component(const ::CdmwPlatformMngtBase::StartupKind& startup_kind)
            throw(CdmwPlatformMngt::ManagedComponent::BadOrder, CORBA::SystemException);
        void ccm_activate()
            throw(CORBA::SystemException);
        void ccm_passivate()
            throw(CORBA::SystemException);
        void ccm_remove()
            throw(CORBA::SystemException);


        //
        // CdmwRecording::Recorder supported interface operations
        //
        void record(const CdmwRecording::GroupHeader& header,
                    const CdmwRecording::GIOP_Buffers& data)
            throw(CORBA::SystemException);


        ::CcmAllCapabilitiesTest::CCM_Client_Context* get_context()
        {
            return ::CcmAllCapabilitiesTest::CCM_Client_Context::_duplicate(
                m_session_context.in());
        }

        //
        // SupervisionObserver supported interface operations
        //
        void notify(CdmwPlatformMngt::Event*)
            throw(CORBA::SystemException);

        CdmwPlatformMngt::Event* take_supervision_event();

        //
        // get result of asynchronous getHostname command
        //
        char* getHostnameCommandResult();


    protected:

        CCM_Client_impl(const CCM_Client_impl&);
        void operator=(const CCM_Client_impl&);

    private:
        TestThread m_testThread;
        Cdmw::Logging::Logger* m_logger;
        CosClockService::Clock_var m_clock;

        CORBA::String_var m_expectedPrimaryHostname;
        CORBA::String_var m_expectedBackupHostname;
        CORBA::String_var m_hostnameCommandResult;
        CORBA::String_var m_fileName;

        ::CcmAllCapabilitiesTest::CCM_Client_Context_var m_session_context;
        // extended context with CDMW services
        CdmwComponents::SessionContextExt_var m_session_context_ext;

        Cdmw::OsSupport::Mutex m_mutex;

        typedef std::vector<CdmwRecording::RequestReplyPair_var> RecordList;
        RecordList m_record_list;

        std::queue<CdmwPlatformMngt::Event*> m_event_queue;

        friend class CcmAllCapabilitiesTester;
    };

}; // End of namespace CcmAllCapabilitiesTest


#endif // ___CCM_ALL_CAPABILITIES_TEST_CCM_CLIENT_IMPL_HPP__
