/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <SystemMngt/platforminterface/PlatformInterface.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtManagedElementContexts.stub.hpp>

#include "ccm_all_capabilities/CCM_Client_impl.hpp"
#include "ccm_all_capabilities/Command_impl.hpp"


const std::string APPLICATION_NAME      = "CcmAllCapabilitiesApplication";
const std::string CLIENT_PROCESS_NAME   = "ClientProcess";
const std::string CLIENT_COMPONENT_NAME = "Client";


//
// IDL:thalesgroup.com/CcmAllCapabilitiesTest/CCM_Client:1.0
//
CcmAllCapabilitiesTest::CCM_Client_impl::CCM_Client_impl()
    :
    m_testThread(),
    m_logger(Cdmw::Logging::Logger::Get_Logger("CCM_Client",
                                               "QUALIF_CCM_ALL_CAPABILITIES")),
    m_clock(CosClockService::Clock::_nil()),
    m_expectedPrimaryHostname(""),
    m_expectedBackupHostname(""),
    m_hostnameCommandResult(""),
    m_fileName(""),
    m_session_context(CcmAllCapabilitiesTest::CCM_Client_Context::_nil()),
    m_session_context_ext(CdmwComponents::SessionContextExt::_nil()),
    m_mutex(),
    m_record_list(),
    m_event_queue()
{
    CDMW_INFO(m_logger, "CCM_Client_impl constructor");
}

CcmAllCapabilitiesTest::CCM_Client_impl::~CCM_Client_impl()
{
    CDMW_INFO(m_logger, "CCM_Client_impl destructor");
}




//
// attribute expectedPrimaryHostname
//
char*
CcmAllCapabilitiesTest::CCM_Client_impl::expectedPrimaryHostname()
    throw(CORBA::SystemException)
{
    return CORBA::string_dup(m_expectedPrimaryHostname);
}

void
CcmAllCapabilitiesTest::CCM_Client_impl::expectedPrimaryHostname(const char* a)
    throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "expectedPrimaryHostname [" << a << "]");
    m_expectedPrimaryHostname =  CORBA::string_dup(a);
}


//
// attribute expectedBackupHostname
//
char*
CcmAllCapabilitiesTest::CCM_Client_impl::expectedBackupHostname()
    throw(CORBA::SystemException)
{
    return CORBA::string_dup(m_expectedBackupHostname);
}

void
CcmAllCapabilitiesTest::CCM_Client_impl::expectedBackupHostname(const char* a)
    throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "expectedBackupHostname [" << a << "]");
    m_expectedBackupHostname =  CORBA::string_dup(a);
}


//
// attribute fileName
//
char*
CcmAllCapabilitiesTest::CCM_Client_impl::fileName()
    throw(CORBA::SystemException)
{
    return CORBA::string_dup(m_fileName.in());
}

void
CcmAllCapabilitiesTest::CCM_Client_impl::fileName (const char* name)
    throw(CORBA::SystemException)
{
    m_fileName = CORBA::string_dup(name);
}

//
// facet clientHostInfo
//
CcmAllCapabilitiesTest::CCM_HostInfo_ptr
CcmAllCapabilitiesTest::CCM_Client_impl::get_clientHostInfo ()
    throw (CORBA::SystemException)
{
    return CcmAllCapabilitiesTest::CCM_HostInfo::_duplicate(this);
}

char*
CcmAllCapabilitiesTest::CCM_Client_impl::get_hostname()
    throw (CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_Client_impl::get_hostname()");
    std::string hostname = Cdmw::OsSupport::OS::get_hostname();
    CDMW_INFO(m_logger, "hostname = " << hostname);
    return CORBA::string_dup(hostname.c_str());
}

TimeBase::TimeT
CcmAllCapabilitiesTest::CCM_Client_impl::get_local_time()
    throw (CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_Client_impl::get_local_time()");
    TimeBase::TimeT now = m_clock->current_time();
    CDMW_INFO(m_logger, "time = " << now);
    return now;
}


//
// consumer hostnameResults
//
void
CcmAllCapabilitiesTest::CCM_Client_impl::push_hostnameResults (
    CcmAllCapabilitiesTest::HostnameResult * event
    )
    throw (CORBA::SystemException)
{
    CDMW_INFO(m_logger,
              "CCM_Client_impl::push_hostnameResults() " << event->hostname());

    std::cout << "### CCM_Client_impl received hostnameResult("
              << event->hostname()
              << std::endl;
    m_hostnameCommandResult = event->hostname();
}


//
// set_session_context function : it fixes the session context of the component
// (call by the container)
//
void
CcmAllCapabilitiesTest::CCM_Client_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_Client_impl::set_session_context()");

    m_session_context =
        ::CcmAllCapabilitiesTest::CCM_Client_Context::_narrow(ctx);

    // get context for services
    m_session_context_ext =
        CdmwComponents::SessionContextExt::_narrow(m_session_context.in());

    // get the Time service
    CosClockService::ClockCatalog_var clock_catalog =
        CosClockService::ClockCatalog::_narrow(
            m_session_context_ext->resolve_service_reference("ClockService"));
    CosClockService::ClockCatalog::ClockEntry_var clockEntry =
        clock_catalog->get_entry("LocalExecutor");
    m_clock = clockEntry->subject;
}


//
// cdmw_initialise_component : it fixes to the component to initialize itself
// (called by the supervision )
//
void CcmAllCapabilitiesTest::CCM_Client_impl::cdmw_initialise_component(
    const ::CdmwPlatformMngtBase::StartupKind& startup_kind)
    throw(CdmwPlatformMngt::ManagedComponent::BadOrder, CORBA::SystemException)
{
    CDMW_INFO(m_logger, " *** Component Client initialisation ****  ");
    Cdmw::OsSupport::OS::sleep(5000);
    CDMW_INFO(m_logger, " *** Component Client initialised ****  ");
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CcmAllCapabilitiesTest::CCM_Client_impl::ccm_activate()
    throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_Client_impl::ccm_activate()");

    try {
        // narrow context in SessionContext for get_CCM_objet() access
        Components::SessionContext_var ctx =
            Components::SessionContext::_narrow(m_session_context.in());

        Cdmw::TestUtils::CORBATestManager::instance()->add(
           "client_context",
           get_context()
           );
        Cdmw::TestUtils::CORBATestManager::instance()->add(
           "user_ccm_client",
           static_cast< ::UserCcmAllCapabilitiesTest::User_CCM_Client* >(this)
           );

        // call SessionContext::get_CCM_objet()
        CORBA::Object_var obj = ctx->get_CCM_object();
        ::CcmAllCapabilitiesTest::Client_var the_component =
              ::CcmAllCapabilitiesTest::Client::_narrow(obj.in());
        Cdmw::TestUtils::CORBATestManager::instance()->add(
            "the_component",
            the_component.in());
    }

    catch (const CORBA::UserException& e) {
        std::cerr << "UserException during get_CCM_Object() :"
                  << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
                  << std::endl;
        CDMW_ASSERT(false);
    }
    catch (const CORBA::SystemException& e) {
        std::cerr << "SystemException during get_CCM_Object() :"
                  << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
                  << std::endl;
        CDMW_ASSERT(false);
    }

    m_testThread.start();
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
CcmAllCapabilitiesTest::CCM_Client_impl::ccm_passivate()
    throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_Client_impl::ccm_passivate()");
}



//
// ccm_remove function : it destroys the component (call by the container)
//
void
CcmAllCapabilitiesTest::CCM_Client_impl::ccm_remove()
    throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_Client_impl::ccm_remove()");
}



//
// get result of asynchronous getHostname command
//
char*
CcmAllCapabilitiesTest::CCM_Client_impl::getHostnameCommandResult()
{
    char* result = CORBA::string_dup(m_hostnameCommandResult.in());
    m_hostnameCommandResult = "";
    return result;
}


//
// CdmwRecording::Recorder::record() implementation
//
void
CcmAllCapabilitiesTest::CCM_Client_impl::record(
    const CdmwRecording::GroupHeader& header,
    const CdmwRecording::GIOP_Buffers& data
    )
    throw(CORBA::SystemException)
{
    CDMW_MUTEX_GUARD(m_mutex);

    TEST_INFO("" << header.num_pairs << " new Records received from "
              << header.hostname << " at " << header.base_time);

    // Use TAO_InputCDR for unmarshalling
    CORBA::ULong len = data.length();
    const CORBA::Octet* buffer = data.get_buffer();
    size_t char_len = len * sizeof(CORBA::Octet) / sizeof(char);
    TAO_InputCDR input((const char*) buffer, char_len);
    input.reset_byte_order(ACE_static_cast (int, header.flags));

    // Unmarshall while TAO_InputCDR is not empty
    CdmwRecording::RequestReplyPair pair;
    int i=0;
    while (input.length() > 0)
    {

        // number of octets still to read
        size_t left = input.length();

        // Unmarshall a RequestReplyPair
        if (input >> pair)
        {
            TEST_INFO("  - " << (++i) << ": \"" <<
                      pair.request.operation <<
                      "\" from " <<
                      pair.request.server_id <<
                      "\" from interface " <<
                      pair.request.interface_rep_id);

            // save copy of RequestReplyPair in list
            m_record_list.push_back(new CdmwRecording::RequestReplyPair(pair));
        }

        else
        {
            // number of read octets
            size_t nb_read = left-input.length();
            TEST_INFO("UNMARSHALL ERROR (only " << nb_read <<
                      " octets read)");
            ACE_HEX_DUMP ((LM_INFO,
                           (const char *) input.rd_ptr() - nb_read,
                           nb_read,
                           ACE_TEXT ("read buffer")));
            TEST_INFO("- Content of pair.request.head:");
            ACE_HEX_DUMP ((LM_INFO,
                           (const char *) &(pair.request.head),
                           12,
                           ACE_TEXT ("pair.request.head")));
            TEST_INFO("- Content of pair.reply.head:");
            ACE_HEX_DUMP ((LM_INFO,
                           (const char *) &(pair.reply.head),
                           12,
                           ACE_TEXT ("pair.reply.head")));
            // It's useless to try to continue unmarshalling... break loop
            break;
        }
    }
}

//
// return first unconsumed event
//
CdmwPlatformMngt::Event*
CcmAllCapabilitiesTest::CCM_Client_impl::take_supervision_event()
{
    CdmwPlatformMngt::Event* event = 0;
    if (! m_event_queue.empty())
    {
        event = m_event_queue.front();
        m_event_queue.pop();
    }
    return event;
}

const char* event_kind_to_string(CdmwPlatformMngt::Event* ev)
{
    switch (ev->event_kind())
    {
    case CdmwPlatformMngt::SYSTEM_MODE_CHANGE:
        return "SYSTEM_MODE_CHANGE";
    case CdmwPlatformMngt::SYSTEM_STATUS_CHANGE:
        return "SYSTEM_STATUS_CHANGE";
    case CdmwPlatformMngt::HOST_STATUS_CHANGE:
        return "HOST_STATUS_CHANGE";
    case CdmwPlatformMngt::APPLICATION_AGENT_DEATH:
        return "APPLICATION_AGENT_DEATH";
    case CdmwPlatformMngt::APPLICATION_MODE_CHANGE:
        return "APPLICATION_MODE_CHANGE";
    case CdmwPlatformMngt::APPLICATION_STATUS_CHANGE:
        return "APPLICATION_STATUS_CHANGE";
    case CdmwPlatformMngt::PROCESS_MESSAGE:
        return "PROCESS_MESSAGE";
    case CdmwPlatformMngt::PROCESS_STATUS_CHANGE:
        return "PROCESS_STATUS_CHANGE";
    case CdmwPlatformMngt::ENTITY_STATUS_CHANGE:
        return "ENTITY_STATUS_CHANGE";
    case CdmwPlatformMngt::CONFIGURATION_CHANGE:
        return "CONFIGURATION_CHANGE";
    case CdmwPlatformMngt::USER_EVENT:
        return "USER_EVENT";
    case CdmwPlatformMngt::PROPERTY_CHANGE:
        return "PROPERTY_CHANGE";
    case CdmwPlatformMngt::RESTART_REQUESTED:
        return "RESTART_REQUESTED";
    case CdmwPlatformMngt::REBOOT_REQUESTED:
        return "REBOOT_REQUESTED";
    case CdmwPlatformMngt::STOP_REQUESTED:
        return "STOP_REQUESTED";
    default:
        return "unknown event kind !";
    }
}

//
// SupervisionObserver::notify() implementation
//
void
CcmAllCapabilitiesTest::CCM_Client_impl::notify(
    CdmwPlatformMngt::Event* event
    )
    throw(CORBA::SystemException)
{
    CORBA::String_var msg = event->to_string();
    TEST_INFO(" - " << event_kind_to_string(event)
              << " notification received from Supervision : " << msg.in());
    CDMW_INFO(m_logger, event_kind_to_string(event)
              << " notification received from Supervision : " << msg.in());

    // save event in event queue
    event->_add_ref();
    m_event_queue.push(event);
}

//
//

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(\
    CcmAllCapabilitiesTest::CcmAllCapabilitiesTester,\
    "CcmAllCapabilitiesTester"\
    );

void
CcmAllCapabilitiesTest::CcmAllCapabilitiesTester::do_tests()
{

//     try
//     {
//         TEST_INFO("Test that a request on a facet where its component is in cdmw_initialise_component succeeds");

//         CcmAllCapabilitiesTest::HostInfo_var hostInfo =
//             m_session_context->get_connection_serverHostInfo();

//         CORBA::String_var hostname = hostInfo->get_hostname();
//         TEST_INFO("Request on facet succeed !! hostname: "
//                   << hostname.in());
//         CPPUNIT_ASSERT(true);
//     }
//     catch (const CORBA::SystemException& e)
//     {
//         TEST_INFO("Unexpected CORBA::SystemException: " << e);
//         CPPUNIT_ASSERT(false);
//     }
//     catch (const CORBA::Exception& e)
//     {
//         TEST_INFO("Unexpected CORBA::Exception: " << e);
//         CPPUNIT_ASSERT(false);
//     }


    TEST_INFO("Wait for the system is ready before running the tests.");
    Cdmw::OsSupport::OS::sleep(5000);

    CORBA::Object_var obj =
        Cdmw::TestUtils::CORBATestManager::instance()->get_object(
            "user_ccm_client");
    ::UserCcmAllCapabilitiesTest::User_CCM_Client_var user_ccm_client =
          ::UserCcmAllCapabilitiesTest::User_CCM_Client::_narrow(obj.in());
    m_user_ccm_client =
        reinterpret_cast< CcmAllCapabilitiesTest::CCM_Client_impl* >(
            user_ccm_client.in());
    obj =
       Cdmw::TestUtils::CORBATestManager::instance()->get_object(
          "the_component");
    m_component_ref =
       CcmAllCapabilitiesTest::Client::_narrow(obj.in());

    CORBA::String_var status_info;
    CdmwPlatformMngt::SupervisionObserver_var old_observer;
    CdmwPlatformMngt::Event_var event;
    const std::string observer_name = "TestCcmAllCapabilities_Client";

    try
    {
        // get ManagedComponentContext
        obj = m_session_context->resolve_service_reference(
            "ManagedComponentContext");
        CdmwPlatformMngt::ManagedElementContexts::ComponentContext_var supContext =
            CdmwPlatformMngt::ManagedElementContexts::ComponentContext::_narrow(obj.in());
        CPPUNIT_ASSERT(! CORBA::is_nil(supContext.in()));

        // get System supervision object
        TEST_INFO("Get System supervision object");
        CdmwPlatformMngt::External::System_var system =
            supContext->the_system();
        TEST_INFO("Test System status");
        CPPUNIT_ASSERT(
            system->get_status(status_info.out()) == CdmwPlatformMngt::SYSTEM_STARTING ||
            system->get_status(status_info.out()) == CdmwPlatformMngt::SYSTEM_STARTED ||
            system->get_status(status_info.out()) == CdmwPlatformMngt::SYSTEM_PARTIAL_STARTED);
        TEST_INFO("      status info: " << system->get_status(status_info.out()));

        // get component, process, application names
        CORBA::String_var component_name = supContext->managed_element_name();
        TEST_INFO("      component_name = '"
                  << (component_name.in() == 0 ? "null" : component_name.in())
                  << "'");
        CPPUNIT_ASSERT(component_name.in() != 0);
        CPPUNIT_ASSERT(CLIENT_COMPONENT_NAME == component_name.in());

        CORBA::String_var process_name = supContext->process_name();
        TEST_INFO("      process_name = '"
                  << (process_name.in() == 0 ? "null" : process_name.in())
                  << "'");
        CPPUNIT_ASSERT(process_name.in() != 0);
        CPPUNIT_ASSERT(CLIENT_PROCESS_NAME == process_name.in());

        CORBA::String_var appli_name = supContext->application_name();
        TEST_INFO("      appli_name = '"
                 << (appli_name.in() == 0 ? "null" : appli_name.in())
                 << "'");
        CPPUNIT_ASSERT(appli_name.in() != 0);
        CPPUNIT_ASSERT(APPLICATION_NAME == appli_name.in());

        // test Application supervision object
        TEST_INFO("Test Application status");
        CdmwPlatformMngt::External::Application_var application;
        {
            int nbtryremain = 5;
            while (nbtryremain > 0) {
                application = supContext->the_application();
                if (application->get_status(status_info.out()) ==
                        CdmwPlatformMngt::APPLICATION_RUNNING) {
                    break;
                }
                TEST_INFO("get application status failed, retry in 2 seconds. (" << nbtryremain << " attempts left)");
                Cdmw::OsSupport::OS::sleep(2000);
                nbtryremain--;
            }
            if (nbtryremain == 0)
                CPPUNIT_ASSERT(false);
        }
        TEST_INFO("      status info: " << status_info.in());

        // test Process supervision object
        TEST_INFO("Test Process status");
        CdmwPlatformMngt::External::Process_var process =
            supContext->the_process();
        CPPUNIT_ASSERT(process->get_status(status_info.out()) ==
                       CdmwPlatformMngt::PROCESS_RUNNING);
        TEST_INFO("      status info: " << status_info.in());

        // test Host supervision object
        TEST_INFO("Test Host status");
        CdmwPlatformMngt::External::Host_var host = supContext->the_host();
        CPPUNIT_ASSERT(host->status() == CdmwPlatformMngt::HOST_STARTED);
        TEST_INFO("      status info: HOST_STARTED");

        // test RedundancyGroup supervision object
        TEST_INFO("Test RedundancyGroup status");
        CdmwPlatformMngt::External::RedundancyGroup_var group =
            supContext->the_redundancy_group();
        CPPUNIT_ASSERT(group.in()
                       == CdmwPlatformMngt::External::RedundancyGroup::_nil());


#ifndef SKIP_TESTS_FAULT_TOLERANCE

        // test connections to group : primary hostname is expected
        test_ft_connections(m_user_ccm_client->expectedPrimaryHostname());

        try
        {
            TEST_INFO("Register this component as System Observer");
            old_observer =
               system->register_observer(observer_name.c_str(),
                                         m_component_ref.in());
            TEST_INFO("Registered this component as System Observer");

            TEST_INFO("Send kill command to group (only primary will stop)");
            CcmAllCapabilitiesTest::Command_var command =
                new CcmAllCapabilitiesTest::Command_impl();
            command->name(CcmAllCapabilitiesTest::COMMAND_KILL);

            TEST_INFO("Emits command " <<  command->name());
            m_session_context->push_commandEmits(command.in());

            TEST_INFO("Wait new primary election");
            Cdmw::OsSupport::OS::sleep(5000);

            TEST_INFO("Unregister this component as System Observer");
            old_observer = system->unregister_observer(observer_name.c_str());
        }

        catch (const CORBA::SystemException& e)
        {
            TEST_INFO("Unexpected CORBA::SystemException: " << e);
            CPPUNIT_ASSERT(false);
        }

        // test connections to group : backup hostname is expected
        test_ft_connections(m_user_ccm_client->expectedBackupHostname());

        try
        {
            TEST_INFO("Send kill command to group (only backup will stop)");

            CcmAllCapabilitiesTest::Command_var command =
                new CcmAllCapabilitiesTest::Command_impl();
            command->name(CcmAllCapabilitiesTest::COMMAND_KILL);

            TEST_INFO("Emits command " <<  command->name());
            m_session_context->push_commandEmits(command.in());

            TEST_INFO("Wait new primary election");
            Cdmw::OsSupport::OS::sleep(5000);
        }
        catch (const CORBA::SystemException& e)
        {
            TEST_INFO("Unexpected CORBA::SystemException: " << e);
            CPPUNIT_ASSERT(false);
        }

        try
        {
            TEST_INFO("Test that FT group is now empty");

            CcmAllCapabilitiesTest::HostInfo_var hostInfo =
                m_session_context->get_connection_serverHostInfo();

            CORBA::String_var hostname = hostInfo->get_hostname();
            TEST_INFO("Request on empty group succeed !! hostname: "
                      << hostname.in());
            CPPUNIT_ASSERT(false);
        }
        catch (const CORBA::SystemException& e)
        {
            CPPUNIT_ASSERT(true);
        }
        catch (const CORBA::Exception& e)
        {
            TEST_INFO("Unexpected CORBA::Exception: " << e);
            CPPUNIT_ASSERT(false);
        }


        TEST_INFO("****************************");
        TEST_INFO("Re run a FT Server on HOST2 ");
        TEST_INFO("Press any key and Enter");
        TEST_INFO("****************************");
        std::string dummy;
        std::cin >> dummy;

        // Test cdmw_initialise_component on FT Server (PCR-1983)
        // Facets musn't be reachable before cdmw_initialise_component is completed
        // A boolean set in facets is cheched at the end of the cdmw_initialise_compoent
        int max_retry = 10, retry = 0;
        while (retry < max_retry)
        {
            try {
                TEST_INFO("Try to request the FT Server");
                CcmAllCapabilitiesTest::HostInfo_var hostInfo =
                    m_session_context->get_connection_serverHostInfo();
                CORBA::String_var hostname = hostInfo->get_hostname();
                TEST_INFO("Request on re started ServerFT Succeed !! hostname: "
                          << hostname.in());
                retry = -1; // OK
                break;
            }
            catch (const CORBA::SystemException& e)
            {
                TEST_INFO("CORBA::Exception: " << e);
            }
            catch (const CORBA::Exception& e)
            {
                TEST_INFO("CORBA::Exception: " << e);
            }

            TEST_INFO("Client sleeps 1 sec.");
            Cdmw::OsSupport::OS::sleep(1000);
        }
        if ( retry != -1 )
        {
            TEST_INFO("Max tentative exceed (" << max_retry << ")");
            CPPUNIT_ASSERT(false);
        }




#endif // SKIP_TESTS_FAULT_TOLERANCE


        // perform 10 request with the RoundRobin policy;
        // each request is expected to be carried out by a LB server
        // different from the previous one
        //
        Cdmw::OsSupport::OS::sleep(3000);
        CORBA::String_var previous_process_name = CORBA::string_dup("");

        for (int i = 0; i < 10; i++)
        {
            CORBA::String_var current_process_name =
                CORBA::string_dup(test_lb_connections());
            CPPUNIT_ASSERT(strcmp(current_process_name.in(), ""));

            TEST_INFO("REQUEST " << i << "...check the RoundRobin policy...") ;
            TEST_INFO("the process called is ==> "
                      << current_process_name.in());

            if (strcmp(current_process_name, previous_process_name))
            {
                previous_process_name = current_process_name;
                CPPUNIT_ASSERT(true);
            }
            else
            {
                CPPUNIT_ASSERT_MESSAGE(
                    "The RoundRobin policy has not been respected", false);
            }
        }


#ifndef SKIP_TESTS_RECORDING

        TEST_INFO("Now check that all calls to \"parse\" operation "
                  "have been recorded");
        // now checks the recording service works;
        // 10 records are expected
        //
        Cdmw::OsSupport::OS::sleep(3000);
        CORBA::String_var fileName = m_user_ccm_client->fileName();

        for (int i = 0; i < 5; i++)
        {
            CPPUNIT_ASSERT_MESSAGE(
                "\"parse\" was not recorded on ServerLB1",
                find_parse_record("ServerLB1", fileName.in()));
        }
        TEST_INFO("Requests to the first LB server recorded successfully");
        for (int i = 0; i < 5; i++)
        {
            CPPUNIT_ASSERT_MESSAGE(
                "\"parse\" was not recorded on ServerLB2",
                find_parse_record("ServerLB2", fileName.in()));
        }
        TEST_INFO("Requests to the second LB server recorded successfully");

        //
        // !!! WARNING !!!
        // the next assert could be set to false in the future,
        // more exactly when the useless CORBA requests (e.g. _is_a)
        // won't be recorded anymore.
        // But at present we consider that having additionnal records
        // is normal.
        //
        // Check if there is non-expected records
//         if (! m_user_ccm_client->m_record_list.empty()) {
//             TEST_INFO("Following records were not expected:");
//             for (CORBA::ULong i = 0;
//                  i < m_user_ccm_client->m_record_list.size();
//                  i++)
//             {
//                 TEST_INFO(
//                     "  - " << (i+1) << ": \"" <<
//                     m_user_ccm_client->m_record_list[i]->request.operation
//                     << "\" from " <<
//                     m_user_ccm_client->m_record_list[i]->request.server_id);
//             }

//             CPPUNIT_ASSERT_MESSAGE("Some records were not expected", false);
//         }

#endif // SKIP_TESTS_RECORDING


#ifndef SKIP_TESTS_FAULT_TOLERANCE

        // test System supervision object
        TEST_INFO("Test System status");
        CPPUNIT_ASSERT(system->get_status(status_info.out()) ==
                       CdmwPlatformMngt::SYSTEM_PARTIAL_STARTED);
        TEST_INFO("      status info: " << status_info.in());

        // test Application supervision object
        TEST_INFO("Test Application status");
        CPPUNIT_ASSERT(application->get_status(status_info.out()) ==
                       CdmwPlatformMngt::APPLICATION_PARTIAL_RUNNING);
        TEST_INFO("      status info: " << status_info.in());

        // test Process supervision object
        TEST_INFO("Test Process status");
        CPPUNIT_ASSERT(process->get_status(status_info.out()) ==
                       CdmwPlatformMngt::PROCESS_RUNNING);
        TEST_INFO("      status info: " << status_info.in());

        // test Host supervision object
        TEST_INFO("Test Host status");
        CPPUNIT_ASSERT(host->status()
                       == CdmwPlatformMngt::HOST_PARTIAL_STARTED);
        TEST_INFO("      status info: HOST_PARTIAL_STARTED");

        // test observer
        //
        TEST_INFO("Check supervision notifications can be observed.");

        //  - Test process MODE_CHANGE to FAILED_DEATH
        TEST_INFO("check process mode changed to FAILED_DEATH");
        event = m_user_ccm_client->take_supervision_event();
        CPPUNIT_ASSERT(event.in() != 0);
        CPPUNIT_ASSERT(event->event_kind()
                       == CdmwPlatformMngt::PROCESS_STATUS_CHANGE);
        CdmwPlatformMngt::ProcessStatusChange* death_event =
            CdmwPlatformMngt::ProcessStatusChange::_downcast(event.in());
        CPPUNIT_ASSERT(death_event->process_status()
                       == CdmwPlatformMngt::PROCESS_FAILED_DEATH);

        //  - Test application MODE_CHANGE to DEGRADED
        TEST_INFO("check application mode changed to DEGRADED");
        event = m_user_ccm_client->take_supervision_event();
        CPPUNIT_ASSERT(event.in() != 0);
        CPPUNIT_ASSERT(event->event_kind()
                       == CdmwPlatformMngt::APPLICATION_MODE_CHANGE);
        CdmwPlatformMngt::ApplicationModeChange* app_mode_change =
            CdmwPlatformMngt::ApplicationModeChange::_downcast(event.in());
        CPPUNIT_ASSERT(APPLICATION_NAME
                       == app_mode_change->application_name());
        CPPUNIT_ASSERT(app_mode_change->application_mode()
                       == CdmwPlatformMngt::DEGRADED_MODE);

        //  - Test system MODE_CHANGE to DEGRADED
        TEST_INFO("      check system mode changed to DEGRADED");
        event = m_user_ccm_client->take_supervision_event();
        CPPUNIT_ASSERT(event.in() != 0);
        CPPUNIT_ASSERT(event->event_kind()
                       == CdmwPlatformMngt::SYSTEM_MODE_CHANGE);
        CdmwPlatformMngt::SystemModeChange* sys_mode_change =
            CdmwPlatformMngt::SystemModeChange::_downcast(event.in());
        CPPUNIT_ASSERT(sys_mode_change->system_mode()
                       == CdmwPlatformMngt::DEGRADED_MODE);

        //  - Test application STATUS_CHANGE to COMPONENT_FAILED_ERROR
        TEST_INFO("      check application status changed to COMPONENT_STATUS_CHANGE");
        event = m_user_ccm_client->take_supervision_event();
        CPPUNIT_ASSERT(event.in() != 0);
        CPPUNIT_ASSERT(event->event_kind()
                       == CdmwPlatformMngt::COMPONENT_STATUS_CHANGE);
        CdmwPlatformMngt::ComponentStatusChange* component_status_change =
            CdmwPlatformMngt::ComponentStatusChange::_downcast(event.in());
        CPPUNIT_ASSERT(component_status_change->component_status()
                       == CdmwPlatformMngt::COMPONENT_FAILED_ERROR);

        //  - Test application STATUS_CHANGE to PARTIAL_RUNNING
        TEST_INFO("      check application status changed to PARTIAL_RUNNING");
        event = m_user_ccm_client->take_supervision_event();
        CPPUNIT_ASSERT(event.in() != 0);
        CPPUNIT_ASSERT(event->event_kind()
                       == CdmwPlatformMngt::APPLICATION_STATUS_CHANGE);

        CdmwPlatformMngt::ApplicationStatusChange* app_status_change =
            CdmwPlatformMngt::ApplicationStatusChange::_downcast(event.in());
        CPPUNIT_ASSERT(APPLICATION_NAME
                       == app_status_change->application_name());
        CPPUNIT_ASSERT(app_status_change->application_status()
                       == CdmwPlatformMngt::APPLICATION_PARTIAL_RUNNING);

        //  - Test system STATUS_CHANGE to PARTIAL_STARTED
        TEST_INFO("      check system status changed to PARTIAL_STARTED");
        event = m_user_ccm_client->take_supervision_event();
        CPPUNIT_ASSERT(event.in() != 0);
        CPPUNIT_ASSERT(event->event_kind()
                       == CdmwPlatformMngt::SYSTEM_STATUS_CHANGE);
        CdmwPlatformMngt::SystemStatusChange* sys_status_change =
            CdmwPlatformMngt::SystemStatusChange::_downcast(event.in());
        CPPUNIT_ASSERT(sys_status_change->system_status()
                       == CdmwPlatformMngt::SYSTEM_PARTIAL_STARTED);

        //  - Test host STATUS_CHANGE to PARTIAL_STARTED
        TEST_INFO("      check host status changed to PARTIAL_STARTED");
        event = m_user_ccm_client->take_supervision_event();
        CPPUNIT_ASSERT(event.in() != 0);
        CPPUNIT_ASSERT(event->event_kind()
                       == CdmwPlatformMngt::HOST_STATUS_CHANGE);
        CdmwPlatformMngt::HostStatusChange* host_status_change =
            CdmwPlatformMngt::HostStatusChange::_downcast(event.in());
        CPPUNIT_ASSERT(host_status_change->host_status()
                       == CdmwPlatformMngt::HOST_PARTIAL_STARTED);

#endif // SKIP_TESTS_FAULT_TOLERANCE

        // register this component as System observer
        TEST_INFO("Register this component as System Observer");
        old_observer =
            system->register_observer(observer_name.c_str(),
                                      m_component_ref.in());

        // test message notification to Supervision
        supContext->notify_message(CdmwPlatformMngtBase::INF,
                                   "Test supervision observer");
        CORBA::String_var Process =
            Cdmw::PlatformMngt::PlatformInterface::Get_process_name();
        Cdmw::PlatformMngt::PlatformInterface::Notify_message(
            CdmwPlatformMngtBase::INF, process_name,
            "Test supervision observer");
        Cdmw::OsSupport::OS::sleep(2000);

        TEST_INFO("Unregister this component as System Observer");
        old_observer = system->unregister_observer(observer_name.c_str());
        event = m_user_ccm_client->take_supervision_event();
        CPPUNIT_ASSERT(event.in() != 0);
        CPPUNIT_ASSERT(event->event_kind()
                       == CdmwPlatformMngt::PROCESS_MESSAGE);
        event = m_user_ccm_client->take_supervision_event();
        CPPUNIT_ASSERT(event.in() != 0);
        CPPUNIT_ASSERT(event->event_kind()
                       == CdmwPlatformMngt::PROCESS_MESSAGE);
        event = m_user_ccm_client->take_supervision_event();
        CPPUNIT_ASSERT(event.in() == 0);
    }
    catch (const std::exception& e)
    {
        // catch any CPPUnit exception, print it and throw it
        TEST_INFO(e.what());
        throw e;
    }

    // print test summary
    Cdmw::TestUtils::CORBATestManager::instance()->summary();
}


void
CcmAllCapabilitiesTest::CcmAllCapabilitiesTester::test_ft_connections(
    const char* expectedHostname
    )
{

    // First test the facet connection
    try
    {
        TEST_INFO("Test synchronous connections to ServerFT group"
                  << " (expected primary host: " << expectedHostname << ")");

        CcmAllCapabilitiesTest::HostInfo_var hostInfo =
            m_session_context->get_connection_serverHostInfo();

        if (CORBA::is_nil(hostInfo.in()))
        {
            TEST_INFO("hostInfo is NIL !!!! ");
            CPPUNIT_ASSERT(false);
        }

        CORBA::String_var hostname = hostInfo->get_hostname();
        if (strcmp(expectedHostname, hostname.in()) == 0)
        {
            CPPUNIT_ASSERT(true);
        }
        else
        {
            TEST_INFO("Server hostname is not the one expected: "
                      << hostname.in());
            CPPUNIT_ASSERT(false);
        }
    }

    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("Unexpected CORBA::SystemException: " << e);
        CPPUNIT_ASSERT(false);
    }
    catch (const CORBA::Exception& e)
    {
        TEST_INFO("Unexpected CORBA::Exception: " << e);
        CPPUNIT_ASSERT(false);
    }
    catch (...)
    {
        TEST_INFO("Unknown exception !! ");
        CPPUNIT_ASSERT(false);
    }


    // Then test the connection with events
    try
    {
        TEST_INFO("Test asynchronous connections to ServerFT group"
                  << " (expected primary host: "
                  << expectedHostname << ")");

        CcmAllCapabilitiesTest::Command_var command =
            new CcmAllCapabilitiesTest::Command_impl();
        command->name(CcmAllCapabilitiesTest::COMMAND_GET_HOSTNAME);

        TEST_INFO("Emits command " << command->name());
        m_session_context->push_commandEmits(command.in());

        TEST_INFO("Wait result");
        Cdmw::OsSupport::OS::sleep(2000);

        CORBA::String_var hostname =
            m_user_ccm_client->getHostnameCommandResult();

        if (strcmp(expectedHostname, hostname.in()) == 0)
        {
            CPPUNIT_ASSERT(true);
        }
        else
        {
            TEST_INFO("Server hostname is not the one expected: "
                      << hostname.in());
            CPPUNIT_ASSERT(false);
        }


        TEST_INFO("Publishes command " <<  command->name());
        m_session_context->push_commandPublishes(command.in());

        TEST_INFO("Wait result");
        Cdmw::OsSupport::OS::sleep(2000);

        hostname = m_user_ccm_client->getHostnameCommandResult();
        if (strcmp(expectedHostname, hostname.in()) == 0)
        {
            CPPUNIT_ASSERT(true);
        }
        else
        {
            TEST_INFO("Server hostname is not the one expected: "
                      << hostname.in());
            CPPUNIT_ASSERT(false);
        }
    }

    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("Unexpected CORBA::SystemException: " << e);
        CPPUNIT_ASSERT(false);
    }
}



char *
CcmAllCapabilitiesTest::CcmAllCapabilitiesTester::test_lb_connections()
{
    CORBA::String_var process_name = CORBA::string_dup("");

    try
    {
        TEST_INFO("Testing connections to ServerLB group");

        // get the server HostInfo
        CcmAllCapabilitiesTest::XMLValidator_var validator =
            m_session_context->get_connection_serverXMLValidator();

        if (CORBA::is_nil(validator.in()))
        {
            CPPUNIT_ASSERT_MESSAGE("Cannot get the XML validator", false);
            return 0;
        }

        // read the hostname from the server HostInfo
        CORBA::String_var fileName = m_user_ccm_client->fileName();
        bool res = validator->parse(fileName.in(), process_name);

        if (! res)
        {
            TEST_INFO("# Error: Parsing of XML file " << fileName<< " failed");
            CPPUNIT_ASSERT(false);
        }
        else
        {
            CPPUNIT_ASSERT(true);
        }
    }

    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("Unexpected CORBA::SystemException: " << e);
        CPPUNIT_ASSERT(false);
    }
    catch (const CORBA::Exception& e)
    {
        TEST_INFO("Unexpected CORBA::Exception: " << e);
        CPPUNIT_ASSERT(false);
    }
    catch (...)
    {
        TEST_INFO("Unknown exception!");
        CPPUNIT_ASSERT(false);
    }

    return CORBA::string_dup(process_name);
}


bool
CcmAllCapabilitiesTest::CcmAllCapabilitiesTester::find_parse_record(
    const char* component_name,
    const char* message
    )
{
    CCM_Client_impl::RecordList::iterator it;
    for (it = m_user_ccm_client->m_record_list.begin();
         it != m_user_ccm_client->m_record_list.end();
         it++)
    {
        // test is record is for a "parse" operation
        if (strcmp((*it)->request.operation, "parse") == 0)
        {
            // check if server_id ends with component_name
            std::string server_id((*it)->request.server_id);
            // get last chars of server_id with same lenght as component_name
            std::string substr =
                server_id.substr(server_id.length() - strlen(component_name));

            if (substr == component_name)
            {
                // try to unmarshall parameters as a string
                CORBA::ULong len = (*it)->request.data.length();
                const CORBA::Octet* buffer = (*it)->request.data.get_buffer();
                size_t char_len = len * sizeof(CORBA::Octet) / sizeof(char);
                TAO_InputCDR input((const char*) buffer, char_len);
                input.reset_byte_order(
                    ACE_static_cast (int, (*it)->request.head.flags));

                CORBA::String_var str;
                if (input >> str.out())
                {
                    // unmarshall OK. Test if it's expected message
                    if (strcmp(str.in(), message) == 0)
                    {
                        // message found, remove it from list
                        m_user_ccm_client->m_record_list.erase(it);
                        return true;
                    }
                }
                else
                {
                    TEST_INFO("record for \"parse\" found, but parameter"
                              " is not a string!");
                }
            }
        }
    }

    return false;
}
