/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>
#include "ccm_all_capabilities/HostnameResult_impl.hpp"
#include "ccm_all_capabilities/CCM_ServerFT_impl.hpp"

#include <iostream>


class SuicideThread : public Cdmw::OsSupport::Thread 
{
public: 
    SuicideThread(unsigned int timeToKill, Cdmw::Logging::Logger* logger)
        :
        m_timeToKill(timeToKill),
        m_logger(logger)
    {
    };
        
protected:
    void run() throw()
    {
            Cdmw::OsSupport::OS::sleep(m_timeToKill);
            CDMW_INFO(m_logger, "CCM_ServerFT_impl: SUICIDE");
            exit(0);
    };
    
private:
    SuicideThread();
    SuicideThread(const SuicideThread&);
    SuicideThread& operator = (const SuicideThread&);
   
    unsigned int m_timeToKill;    
    Cdmw::Logging::Logger* m_logger;
};


//
// IDL:thalesgroup.com/CcmAllCapabilitiesTest/CCM_ServerFT:1.0
//
CcmAllCapabilitiesTest::CCM_ServerFT_impl::CCM_ServerFT_impl()
    :
    m_session_context(CcmAllCapabilitiesTest::CCM_ServerFT_Context::_nil()),
    m_session_context_ext(CdmwComponents::SessionContextExt::_nil()),
    m_clock(CosClockService::Clock::_nil()),
    m_logger(Cdmw::Logging::Logger::Get_Logger("CCM_ServerFT",
                                               "QUALIF_CCM_ALL_CAPABILITIES")),
    m_is_facet_called(false)
{
}

CcmAllCapabilitiesTest::CCM_ServerFT_impl::~CCM_ServerFT_impl()
{
    CDMW_INFO(m_logger, "CCM_ServerFT_impl destructor");
}


//
// facet serverHostInfo
//
CcmAllCapabilitiesTest::CCM_HostInfo_ptr 
CcmAllCapabilitiesTest::CCM_ServerFT_impl::get_serverHostInfo()
    throw (CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_ServerFT_impl::get_serverHostInfo()");
    m_is_facet_called = true;
    return CcmAllCapabilitiesTest::CCM_HostInfo::_duplicate(this);
}

char*
CcmAllCapabilitiesTest::CCM_ServerFT_impl::get_hostname()
    throw (CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_ServerFT_impl::get_hostname()");
    std::string hostname = Cdmw::OsSupport::OS::get_hostname();
    CDMW_INFO(m_logger, "hostname = " << hostname);
    return CORBA::string_dup(hostname.c_str());
}

TimeBase::TimeT
CcmAllCapabilitiesTest::CCM_ServerFT_impl::get_local_time()
    throw (CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_ServerFT_impl::get_local_time()");
    TimeBase::TimeT now = m_clock->current_time();
    CDMW_INFO(m_logger, "time = " << now);
    return now;
}


//
// consumer commands
//
void 
CcmAllCapabilitiesTest::CCM_ServerFT_impl::push_commands(
    CcmAllCapabilitiesTest::Command * event
    )
    throw (CORBA::SystemException)
{
    CDMW_INFO(m_logger,
              "CCM_ServerFT_impl::push_commands() " << event->name());

    std::cout << "### CCM_ServerFT_impl received command "
              << event->name()
              << std::endl;

    if (! strcmp(event->name(), CcmAllCapabilitiesTest::COMMAND_KILL))
    {
        std::cout << "### CCM_ServerFT_impl: SUICIDE in 500 ms !"
                  << std::endl;
        SuicideThread* thread = new SuicideThread(500, m_logger);
        thread->start();
    }
    else if (! strcmp(event->name(), 
                      CcmAllCapabilitiesTest::COMMAND_GET_HOSTNAME))
    {
        std::string hostname = Cdmw::OsSupport::OS::get_hostname();
        CcmAllCapabilitiesTest::HostnameResult_var result =
            new CcmAllCapabilitiesTest::HostnameResult_impl();
        result->hostname(CORBA::string_dup(hostname.c_str()));

        std::cout << "### CCM_ServerFT_impl: emits HostnameResult("
                  << hostname << ")"
                  << std::endl;
        m_session_context->push_hostnameEmits(result.in());
    }
    else
    {
        CDMW_WARN(m_logger, "CCM_ServerFT_impl: unkown command");
    }
};


//
// set_session_context function : it fixes the session context of the component
// (call by the container)
//
void
CcmAllCapabilitiesTest::CCM_ServerFT_impl::set_session_context(
    Components::SessionContext_ptr ctx
    )
    throw (CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_ServerFT_impl::set_session_context()");

    m_session_context = 
        ::CcmAllCapabilitiesTest::CCM_ServerFT_Context::_narrow(ctx); 

    // get context for services
    m_session_context_ext = 
        CdmwComponents::SessionContextExt::_narrow(m_session_context.in());

    // get the Time service
    CosClockService::ClockCatalog_var clock_catalog = 
        CosClockService::ClockCatalog::_narrow(
            m_session_context_ext->resolve_service_reference("ClockService"));
    CosClockService::ClockCatalog::ClockEntry_var clockEntry =
        clock_catalog->get_entry("LocalExecutor");
    m_clock = clockEntry->subject;
}


//
// cdmw_initialise_component : it fixes to the component to initialize itself
// (called by the supervision )
//
void CcmAllCapabilitiesTest::CCM_ServerFT_impl::cdmw_initialise_component(
    const ::CdmwPlatformMngtBase::StartupKind& startup_kind)
    throw(CdmwPlatformMngt::ManagedComponent::BadOrder, CORBA::SystemException)
{
    CDMW_INFO(m_logger, " *** Component ServerFT initialisation ****  ");
    Cdmw::OsSupport::OS::sleep(5000);
    CDMW_ASSERT(! m_is_facet_called);
    CDMW_INFO(m_logger, " *** Component ServerFT initialised ****  ");
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CcmAllCapabilitiesTest::CCM_ServerFT_impl::ccm_activate()
    throw (CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_ServerFT_impl::ccm_activate()");
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
CcmAllCapabilitiesTest::CCM_ServerFT_impl::ccm_passivate()
    throw (CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_ServerFT_impl::ccm_passivate()");
}


//
// ccm_remove function : it destroys the component (call by the container)
//
void
CcmAllCapabilitiesTest::CCM_ServerFT_impl::ccm_remove()
    throw (CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_ServerFT_impl::ccm_remove()");
}
