/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>

#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <XMLServer/idllib/CdmwXML.stub.hpp>
#include <SystemMngt/platforminterface/PlatformInterface.hpp>

#include "ccm_all_capabilities/CCM_ServerLB_impl.hpp"


static const char* XML_SERVICE_NAME = "XMLService";

//
// IDL:thalesgroup.com/CcmAllCapabilitiesTest/CCM_ServerLB:1.0
//
CcmAllCapabilitiesTest::CCM_ServerLB_impl::CCM_ServerLB_impl()
    :
    m_parser(CdmwXML::XMLParser::_nil()),
    m_logger(Cdmw::Logging::Logger::Get_Logger("CCM_ServerLB",
                                               "QUALIF_CCM_ALL_CAPABILITIES")),
    m_expectedClientHostname(),
    m_is_facet_called(false)
{
    // get the XML Server reference
    try
    {
        CORBA::Object_ptr obj =
            Cdmw::PlatformMngt::PlatformInterface::Get_service(
                XML_SERVICE_NAME);
        m_parser = CdmwXML::XMLParser::_narrow(obj);
    }
    catch (...)
    {
        CDMW_ERROR(m_logger, "### Test Failed: XML Service not found.");
    }
}


CcmAllCapabilitiesTest::CCM_ServerLB_impl::~CCM_ServerLB_impl()
{
    CDMW_INFO(m_logger, "CCM_ServerLB_impl destructor");
}


//
// attribute expectedClientHostname
//
char*
CcmAllCapabilitiesTest::CCM_ServerLB_impl::expectedClientHostname()
    throw(CORBA::SystemException)
{
    return CORBA::string_dup(m_expectedClientHostname);
}

void
CcmAllCapabilitiesTest::CCM_ServerLB_impl::expectedClientHostname(const char* a)
    throw(CORBA::SystemException)
{
    m_expectedClientHostname =  CORBA::string_dup(a);
}


//
// facet serverHostInfo
//
CcmAllCapabilitiesTest::CCM_XMLValidator_ptr 
CcmAllCapabilitiesTest::CCM_ServerLB_impl::get_serverXMLValidator ()
    throw (CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_ServerLB_impl::get_serverXMLValidator()"); 
    return CcmAllCapabilitiesTest::CCM_XMLValidator::_duplicate(this);
}

bool
CcmAllCapabilitiesTest::CCM_ServerLB_impl::parse(
    const char* file_name,
    ::CORBA::String_out process_name
    )
    throw (CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_ServerLB_impl::parse(" << file_name << ")" );

    process_name = Cdmw::PlatformMngt::PlatformInterface::Get_process_name();

    bool res = false;
    try
    {
        m_parser->parse(file_name);
        res = true;
    }
    catch (const CdmwXML::XMLParser::ParsingException& pe)
    {
        CDMW_ERROR(m_logger, "Exception raised in parsing: " << pe);
        std::cout << "#### Failure in parsing " << file_name
             << ": " << pe.reason
             << std::endl;
    }
    catch (const CORBA::SystemException& e)
    {        
        CDMW_ERROR(m_logger, "Unexpected CORBA::SystemException: " << e);
    }
    catch (const CORBA::Exception& e)
    {
        CDMW_ERROR(m_logger, "Unexpected CORBA::Exception: " << e);
    }
    catch (...)
    {
        CDMW_ERROR(m_logger, "Unknown exception!");
    }

    return res;
}


//
// set_session_context() sets the session context of the component
// (called by the container)
//
void
CcmAllCapabilitiesTest::CCM_ServerLB_impl::set_session_context(
    Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_ServerLB_impl::set_session_context()");

    m_session_context =
        ::CcmAllCapabilitiesTest::CCM_ServerLB_Context::_narrow(ctx);
}


//
// cdmw_initialise_component : it fixes to the component to initialize itself
// (called by the supervision )
//
void CcmAllCapabilitiesTest::CCM_ServerLB_impl::cdmw_initialise_component(
    const ::CdmwPlatformMngtBase::StartupKind& startup_kind)
    throw(CdmwPlatformMngt::ManagedComponent::BadOrder, CORBA::SystemException)
{
    CDMW_INFO(m_logger, " *** Component ServerLB initialisation ****  ");
    Cdmw::OsSupport::OS::sleep(10000);
    // CDMW_ASSERT(m_is_facet_called);
    CDMW_INFO(m_logger, " *** Component ServerLB initialised ****  ");
}


//
// ccm_activate() activates the component (called by the container)
//
void
CcmAllCapabilitiesTest::CCM_ServerLB_impl::ccm_activate()
    throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_ServerLB_impl::ccm_activate()");
}


//
// ccm_passivate() stops the component (called by the container)
//
void
CcmAllCapabilitiesTest::CCM_ServerLB_impl::ccm_passivate()
    throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_ServerLB_impl::ccm_passivate()");
}


//
// ccm_remove function : it destroys the component (called by the container)
void
CcmAllCapabilitiesTest::CCM_ServerLB_impl::ccm_remove()
    throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "CCM_ServerLB_impl::ccm_remove()");
}
