/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef _CDMW_PIQ_CCM_AND_LB_CLIENTEXECUTOR_IMPL_HPP_
#define _CDMW_PIQ_CCM_AND_LB_CLIENTEXECUTOR_IMPL_HPP_


#include <Foundation/logging/Logger.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/osthreads/ThreadHandle.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>

#include "UserCcmAndLB.skel.hpp"


namespace CcmAndLB
{
    class TestThread : public Cdmw::OsSupport::Thread
    {
       public:
           TestThread() {};

       protected:
           void run() throw()
        {
            Cdmw::TestUtils::ThreadedTestRunner runner(Cdmw::TestUtils::CORBATestManager::instance()->get_controller());
            runner.addTest( CppUnit::TestFactoryRegistry::getRegistry("test_ccmlb_send_requests").makeTest() );
            runner.start();
            runner.join();
            Cdmw::TestUtils::CORBATestManager::instance()->summary();
        };

        private:
            TestThread(const TestThread&);
            TestThread& operator = (const TestThread&);

     };
    /**
     * IDL:thalesgroup.com/CcmAndLB/CCM_Client:1.0
     */
    class ClientExecutor_impl
            : virtual public CcmAndLB::User_Client,
              virtual public Cdmw::OrbSupport::RefCountLocalObject
    {
        public:
            ClientExecutor_impl();

            ~ClientExecutor_impl();

            void
            set_session_context(Components::SessionContext_ptr ctx)
                throw(CORBA::SystemException);

            void
            ccm_activate()
                throw(CORBA::SystemException);

            void
            ccm_passivate()
                throw(CORBA::SystemException);

            void
            ccm_remove()
                throw(CORBA::SystemException);

  void cdmw_initialise_component(const ::CdmwPlatformMngtBase::StartupKind& startup_kind)
      throw(CdmwPlatformMngt::ManagedComponent::BadOrder, CORBA::SystemException);
  void handle_set(const CORBA::Any& params)
    throw (CORBA::SystemException) ;

  void handle_set_rate(const CORBA::Any& params)
    throw (CORBA::SystemException) ;

  void handle_pause(const CORBA::Any& params)
    throw (CORBA::SystemException) ;

  void handle_resume(const CORBA::Any& params)
    throw (CORBA::SystemException) ;

  void handle_terminate(const CORBA::Any& params)
    throw (CORBA::SystemException) ;

    ::CcmAndLB::CCM_Client_Context* get_context()
    {
        return ::CcmAndLB::CCM_Client_Context::_duplicate(m_session_context.in());
    }
/*
  CcmAndLB::CCM_HelloWorld_ptr get_client_facet()  throw (CORBA::SystemException) ;
*/


        private:
            ClientExecutor_impl(const ClientExecutor_impl&);

            void
            operator=(const ClientExecutor_impl&);


        private:
            CCM_Client_Context_var m_session_context;

            Cdmw::Logging::Logger* m_logger;
            
            TestThread m_testThread;
 

    };
class test_ccmlb_send_requests : public CppUnit::TestFixture
{
    ClientExecutor_impl * m_user_ccm_client;

    ::CcmAndLB::CCM_Client_Context_var m_session_context;

public:
    test_ccmlb_send_requests()
    {
        CORBA::Object_var obj1 = Cdmw::TestUtils::CORBATestManager::instance()->get_object("client_context");
        m_session_context = ::CcmAndLB::CCM_Client_Context::_narrow(obj1.in());
    };


    ~test_ccmlb_send_requests()
    {
    };


    //
    void do_tests();
    void manageProcess(std::string command, std::string application_name, std::string process_name, const char* host_name);
    void executeConnectionToLBGroup();

    CPPUNIT_TEST_SUITE( test_ccmlb_send_requests );
    CPPUNIT_TEST( do_tests );
    CPPUNIT_TEST_SUITE_END();

};


} // namespace CcmAndLB


#endif // _CDMW_PIQ_CCM_AND_LB_CLIENTEXECUTOR_IMPL_HPP_
