/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/logging/LogManager.hpp>

#include "ccm_and_lb/ClientExecutor_impl.hpp"

#include <SystemMngt/idllib/CdmwPlatformMngtManagedElementContexts.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp>


#include <Foundation/orbsupport/OrbSupport.hpp>



namespace CcmAndLB
{
    const std::string CLIENT_APPLICATION_NAME = " CcmAndLBClientApplication ";
    const std::string SERVER_APPLICATION_NAME = " CcmAndLBApplication ";
    const std::string CLIENT_PROCESS_NAME     = " ClientProcess ";
    const std::string SERVER_PROCESS1_NAME    = " ServerProcess1 ";
    const std::string LB_APPLICATION_NAME     = " LBGroupManagerApplication ";
    const std::string LB_PROCESS_NAME         = " LBGroupManager_host1 ";

/**
 * IDL:thalesgroup.com/CcmAndLB/CCM_Client:1.0
 */


    /**
     * Constructor.
     */
    ClientExecutor_impl::ClientExecutor_impl()
        : m_session_context(CCM_Client_Context::_nil()),
          m_logger(Cdmw::Logging::Logger::Get_Logger("CCM_Client",
                                                     "QUALIF_CCM_AND_LB")),
          m_testThread()
    {
    }


    /**
     * Destructor.
     */
    ClientExecutor_impl::~ClientExecutor_impl()
    {
        CDMW_INFO(m_logger, "ClientExecutor_impl::~ClientExecutor_impl() called.");
    }


    /**
     * Set the session context of the component (called by the container).
     */
    void
    ClientExecutor_impl::set_session_context(
        Components::SessionContext_ptr ctx
        )
        throw(CORBA::SystemException)
    {
        CDMW_INFO(m_logger,
                  "ClientExecutor_impl::set_session_context() called.");

        m_session_context = CCM_Client_Context::_narrow(ctx);
    }


    /**
     * Activate the component (called by the container).
     */
    void
    ClientExecutor_impl::ccm_activate()
        throw(CORBA::SystemException)
    {
        CDMW_INFO(m_logger, "ClientExecutor_impl::ccm_activate() called.");
   Cdmw::TestUtils::CORBATestManager::instance()->add("client_context", get_context());
   Cdmw::TestUtils::CORBATestManager::instance()->add("user_ccm_client", static_cast< ::CcmAndLB::User_Client_ptr > (this));
   m_testThread.start();

/*
        int numOfLBGroupConnections = 15000;   
        int numOfClientServerConnections = 10;
        CORBA::String_var process_name = CORBA::string_dup("");
        try {
            // get the display connection of the server.
            CDMW_INFO(m_logger, "connect to the display facet...");

            for (int ji = 0 ; ji<numOfLBGroupConnections; ji++) 
            {
               std::cerr << " *** Component Client (c++) connection n. " << ji+1 << " to LB Compoments Group.. " << std::endl; 
               HelloWorld_var display =
                   m_session_context->get_connection_client_receptacle();

               // call the server display.
               for (int i = 0 ; i<numOfClientServerConnections; i++) 
               {
               
                  std::cerr << " *** Component Client (c++) call sayHello " << std::endl; 
                  Cdmw::OsSupport::OS::sleep(2000);
                  display->sayHello();
                  
               }
               CORBA::String_var out_;
               CORBA::String_var ret_;
               //ret_ = display->printName(out_.out());
            }


        }
        catch (const CORBA::Exception& e) {
            CDMW_ERROR(m_logger, "unexpected exception raised: " << e);
        }
        catch (...) {
            CDMW_ERROR(m_logger, "unexpected exception raised.");
        }
*/
    }

    /**
     * Stop the component (called by the container).
     */
    void
    ClientExecutor_impl::ccm_passivate()
        throw(CORBA::SystemException)
    {
        CDMW_INFO(m_logger, "ClientExecutor_impl::ccm_passivate() called.");
    }


    /**
     * Destroy the component (called by the container).
     */
    void
    ClientExecutor_impl::ccm_remove()
        throw(CORBA::SystemException)
    {
        CDMW_INFO(m_logger, "ClientExecutor_impl::ccm_remove() called.");
    }



  void ClientExecutor_impl::cdmw_initialise_component(const ::CdmwPlatformMngtBase::StartupKind& startup_kind)
      throw(CdmwPlatformMngt::ManagedComponent::BadOrder, CORBA::SystemException)
  {
     std::cerr << " *** Component Client initialisation ****  " << std::endl;
        
     Cdmw::OsSupport::OS::sleep(5000);
     std::cerr << " *** Component Client initialised ****  " << std::endl;
  }
  void ClientExecutor_impl::handle_set(const CORBA::Any& params)
    throw (CORBA::SystemException) {
  }

  void ClientExecutor_impl::handle_set_rate(const CORBA::Any& params)
    throw (CORBA::SystemException) {
  }


  void ClientExecutor_impl::handle_pause(const CORBA::Any& params)
    throw (CORBA::SystemException) {
  }

  void ClientExecutor_impl::handle_resume(const CORBA::Any& params)
    throw (CORBA::SystemException) {
  }

  void ClientExecutor_impl::handle_terminate(const CORBA::Any& params)
    throw (CORBA::SystemException) {
  }
/*  CcmAndLB::CCM_HelloWorld_ptr ClientExecutor_impl::get_client_facet()  
     throw (CORBA::SystemException) {
      return 0;
  };
*/

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(CcmAndLB::test_ccmlb_send_requests, "test_ccmlb_send_requests");


     

void
CcmAndLB::test_ccmlb_send_requests::manageProcess(  std::string command, std::string application_name, std::string process_name, const char* host_name)
{
    try {
    std::string admin_args =  application_name + " " + process_name + " " + host_name;
    Cdmw::OsSupport::OS::sleep(2000);
    std::cerr << "platform_admin.sh " << command << admin_args << std::endl;
    
    Cdmw::OsSupport::OS::create_process("platform_admin.sh" , command + admin_args);
    Cdmw::OsSupport::OS::sleep(2000);
    } catch (const CORBA::SystemException& e) {
            TEST_INFO("Exception calling platform_admin commands: " <<
                      Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            CPPUNIT_ASSERT(false);

      }
 
    
}

void 
CcmAndLB::test_ccmlb_send_requests::do_tests()
{
        // get ManagedComponentContext
    CORBA::Object_var obj =
        m_session_context->resolve_service_reference("ManagedComponentContext");
    CdmwPlatformMngt::ManagedElementContexts::ComponentContext_var component_context =
        CdmwPlatformMngt::ManagedElementContexts::ComponentContext::_narrow(obj.in());
    CPPUNIT_ASSERT(!CORBA::is_nil(component_context.in()));
    CdmwPlatformMngt::System_var system2 = 
        CdmwPlatformMngt::System::_narrow(component_context->the_system());
    // get number of hosts
    CORBA::ULong nbr_of_host = system2->get_number_of_hosts();
       
    CdmwPlatformMngt::ElementNames_var elems;
    CdmwPlatformMngt::ElementNamesIterator_var it;

    // get the name of hosts
    system2->get_all_host_names(nbr_of_host, elems.out(), it.out());

    const char* host_name1 = elems[0U];    
    //const char* host_name2 = elems[1U];    

    std::cerr << "Connect to LBGroup" << std::endl;
    executeConnectionToLBGroup();
    Cdmw::OsSupport::OS::sleep(2000);
    
    Cdmw::OsSupport::OS::sleep(2000);
    std::cerr << "******************************************" << std::endl;
    std::cerr << "*     TEST AVAIABILITY OF LB SERVICE     *" << std::endl;
    std::cerr << "*   After the crash of Primary LBGrmgr   *" << std::endl;
    std::cerr << "*   the Client continues to receive      *" << std::endl;
    std::cerr << "*   the replies from LBServer1           *" << std::endl;
    std::cerr << "*   and LBServer2 in Load Balanced mode  *" << std::endl;
    std::cerr << "******************************************" << std::endl;
    std::cerr << "(1) stop the LBGroupManager  " << std::endl;
    manageProcess(" --proc-stop ",LB_APPLICATION_NAME,LB_PROCESS_NAME,host_name1);
    std::cerr << "(2) waiting for reconnection to LBGroup" << std::endl;
    executeConnectionToLBGroup();  
    CPPUNIT_ASSERT(true);
  

    Cdmw::OsSupport::OS::sleep(5000);
    std::cerr << "**************************************" << std::endl;
    std::cerr << "*     TEST CRASH OF LB REPLICA       *" << std::endl;
    std::cerr << "*  After the crash of LB Server1     *" << std::endl;
    std::cerr << "*  the Client receives the reply     *" << std::endl;
    std::cerr << "*  only from alive LB Server2        *" << std::endl;
    std::cerr << "**************************************" << std::endl;
    std::cerr << "(1) stop the LB Server 1 " << std::endl;
    manageProcess(" --proc-stop ", SERVER_APPLICATION_NAME,SERVER_PROCESS1_NAME,host_name1);
    std::cerr << "(2) waiting for reconnection to LBGroup" << std::endl;
    executeConnectionToLBGroup();  
    CPPUNIT_ASSERT(true);
 
    std::cerr << "******************************************" << std::endl;
    std::cerr << "*    TEST RESTART OF LB REPLICA          *" << std::endl;
    std::cerr << "*   After the restart of LBServer1       *" << std::endl;
    std::cerr << "*   the Client continues to receive      *" << std::endl;
    std::cerr << "*   the replies from LBServer1           *" << std::endl;
    std::cerr << "*   and LBServer2 in Load Balanced mode  *" << std::endl;
    std::cerr << "******************************************" << std::endl;
    std::cerr << "(1) restart the LB Server 1 " << std::endl;
    manageProcess(" --proc-hot-restart ", SERVER_APPLICATION_NAME,SERVER_PROCESS1_NAME,host_name1);
    Cdmw::OsSupport::OS::sleep(10000);
    std::cerr << "(2) waiting for reconnection to LBGroup" << std::endl;
    std::cerr << "reconnect to LBGroup" << std::endl;
    executeConnectionToLBGroup();  
    CPPUNIT_ASSERT(true);

    
}

void
CcmAndLB::test_ccmlb_send_requests::executeConnectionToLBGroup()
{
    for (int sec=0; sec < 31 ; sec++)
    {
       std::cerr << "##";
       Cdmw::OsSupport::OS::sleep(500);
       if (sec == 30) std::cerr << std::endl;
    }

    CORBA::Object_var obj2 = Cdmw::TestUtils::CORBATestManager::instance()->get_object("user_ccm_client");
    ::CcmAndLB::User_Client_var user_ccm_client = ::CcmAndLB::User_Client::_narrow(obj2.in());
    m_user_ccm_client = reinterpret_cast<CcmAndLB::ClientExecutor_impl* > (user_ccm_client.in());

        int numOfLBGroupConnections = 10;
        int numOfClientServerConnections = 10;
        CORBA::String_var process_name = CORBA::string_dup("");
        try {
            // get the display connection of the server.
            //CDMW_INFO(m_logger, "connect to the display facet...");

            std::cerr << " *** Component LB Client (c++) received createSFPL ******   " << std::endl;
            for (int ji = 0 ; ji<numOfLBGroupConnections; ji++)
            {
               std::cerr << " *** Component Client (c++) connection n. " << ji+1 << " to LB Compoments Group.. " << std::endl;
               HelloWorld_var display =
                   m_session_context->get_connection_client_receptacle();

               // call the server display.
               for (int i = 0 ; i<numOfClientServerConnections; i++)
               {

                  std::cerr << " *** Component Client (c++) call createSFPL to server  " << std::endl;
                  Cdmw::OsSupport::OS::sleep(50);
                  display->sayHello();

               }
               Cdmw::OsSupport::OS::sleep(50);
               CORBA::String_var out_;
               CORBA::String_var ret_;
               //ret_ = display->printName(out_.out());
            }


        }
        catch (const CORBA::Exception& e) {
            //CDMW_ERROR(m_logger, "unexpected exception raised: " << e);
            std::cerr << "unexpected exception raised: "  << std::endl;
            CPPUNIT_ASSERT(false);
        }
        catch (...) {
            std::cerr << "unknown exception raised: "  << std::endl;
            CPPUNIT_ASSERT(false);
            //CDMW_ERROR(m_logger, "unexpected exception raised.");
        }
}


}; // End of namespace CcmAndLB
