/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <ccm_auto_reconnect/CCM_Server_impl.hpp>
#include <ccm_auto_reconnect/CCM_EventChecking_impl.hpp>
#include <Foundation/ossupport/OS.hpp>


AutoReconnectDemo::SuicideThread::SuicideThread(
    unsigned int timeToKill,
    Cdmw::Logging::Logger* logger)
    :
    m_timeToKill(timeToKill),
    m_logger(logger)
{
};

void AutoReconnectDemo::SuicideThread::run() throw()
{
    CDMW_INFO(m_logger,
              "CCM_Server_impl: SUICIDE in " << m_timeToKill << " msec.");
    Cdmw::OsSupport::OS::sleep(m_timeToKill);
    CDMW_INFO(m_logger, "CCM_Server_impl: SUICIDE");
    exit(0);
};
    


//
// IDL:thalesgroup.com/AutoReconnectDemo/CCM_Server:1.0
//
AutoReconnectDemo::CCM_Server_impl::CCM_Server_impl()
    : m_session_context(AutoReconnectDemo::CCM_Server_Context::_nil()),
      m_logger(Cdmw::Logging::Logger::Get_Logger("CCM_Server",
                                                 "QUALIF_CCM_AUTO_RECONNECT"))
{
    CDMW_INFO(m_logger,
              "CCM_Server_impl::CCM_Server_impl called!");
    m_facet = new AutoReconnectDemo::CCM_EventChecking_impl();
}

AutoReconnectDemo::CCM_Server_impl::~CCM_Server_impl()
{
    CDMW_INFO(m_logger,
              "CCM_Server_impl::~CCM_Server_impl called!");
}

//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
AutoReconnectDemo::CCM_Server_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "Set_session_context called!");
   
    m_session_context = ::AutoReconnectDemo::CCM_Server_Context::_narrow(ctx); 
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
AutoReconnectDemo::CCM_Server_impl::ccm_activate()
   throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "Ccm_activate called!");
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//

void
AutoReconnectDemo::CCM_Server_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "Ccm_passivate called!");
} 



//
// ccm_remove function : it destroys the component (call by the container)
void
AutoReconnectDemo::CCM_Server_impl::ccm_remove()
   throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "Ccm_remove called!");
}

//
// IDL:thalesgroup.com/AutoReconnectDemo/CCM_Server/get_for_clients:1.0
//
::AutoReconnectDemo::CCM_EventChecking_ptr
AutoReconnectDemo::CCM_Server_impl::get_for_clients()
    throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "get_for_clients called!");
    return AutoReconnectDemo::CCM_EventChecking::_duplicate(m_facet.in());
}


//
// IDL:thalesgroup.com/AutoReconnectDemo/CCM_Server/push_event_sink:1.0
//
void 
AutoReconnectDemo::CCM_Server_impl::push_event_sink(
    AutoReconnectDemo::TestEvent* event)
    throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "push_event_sink called!");
    CCM_EventChecking_impl* facet =
        dynamic_cast<CCM_EventChecking_impl*>(m_facet.in());

    if (! strcmp(event->name(), "SUICIDE ME"))
    {
        SuicideThread* thread = new SuicideThread(500, m_logger);
        thread->start();
    }

    // set flag that event has been received 
    facet->_was_event_received = true;
    
    // send the same event back to Client
    m_session_context->push_event_publish(event);
}


//
// IDL:thalesgroup.com/AutoReconnectDemo/Registration/register:1.0
//
void
AutoReconnectDemo::CCM_Server_impl::test_supported()
    throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "test_supported called !");
    // nothing to do
}

