#!/bin/sh
# =====================================================================
# This file is part of CARDAMOM (R) which is jointly developed by THALES
# and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
# Copyright (C) THALES 2000-2003. All rights reserved.
# Copyright (C) THALES 2004-2009. All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
# License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

SCRIPT_NAME=$0
DANCE_PORT_NUMBER=21666

echo "------------------------------------------"
echo " -----------BUGGY DANCE SCRIPT----------"
echo "------------------------------------------"

#usage : give hosts where there are node managers
usage()
{
    echo "Usage : $0.sh [-h | --help] "
    echo "        [--repository=<host>:<port> | --ft-domain=<faultToleranceDomain>]"
    echo "        --system-port=<systemPort> [--dance-port=<dancePort>]"
    echo "        [--nodes='host1 host2 ... hostn']"
    echo "Where "
    echo "-h, --help                : display this help"
    echo "--repository              : <host>:<port> how to access the repository"
    echo "                            if not specified, will try to get the local repository"
    echo "                            registered into local Daemon"
    echo "--ft-domain               : <faultToleranceDomain> Name of the FaultTolerance domain"
    echo "                            for the discovery of the FT repository"
    echo "--system-port (mandatory) : <systemPort> is the port of the local CARDAMOM "
    echo "                            daemon"
    echo "--dance-port              : <dancePort> internal port for the node manager"
    echo "                            if not specified, will use default value $DANCE_PORT_NUMBER"
    echo "--nodes                   : host1, host2, ...n hostn are the hosts where a node manager is running"
    echo "Note that localhost should not be passed as an argument with the --node option"
    exit 0
}

HOST=`hostname`

while [ -n "$1" ]; do
    case $1 in
        -h) usage;;
        --help) usage;;
        --repository=*)
            CDMW_REPOSITORY=`echo $1 | sed 's/[-_a-zA-Z0-9]*=//'`;;
        --system-port=*)
            DAEMON_PORT=`echo $1 | sed 's/[-_a-zA-Z0-9]*=//'`;;
        --dance-port=*)
            DANCE_PORT_NUMBER=`echo $1 | sed 's/[-_a-zA-Z0-9]*=//'`;;
        --nodes=*) 
            Nodes=`echo $1 | sed 's/[-_a-zA-Z0-9]*=//'`;;
        --ft-domain=*) 
            FTDomain=`echo $1 | sed 's/[-_a-zA-Z0-9]*=//'`;;
        *)
            BadArgEncounter=1;break;;
    esac
    shift
done


if [ -z "$DAEMON_PORT" ] ; then
    echo "Error : Missing system port"
    echo "$SCRIPT_NAME -h for help"
    exit 1
fi



EM_Args="--log-level 1 -e em.$HOST.ior -n $HOST"

for Node in $Nodes ; do
    EM_Args=${EM_Args}" -n $Node=corbaname::$Node:$DAEMON_PORT/NameService#$Node.NodeManager"
done

EM_Args=${EM_Args}" -p -r -c -i -ORBListenEndpoints iiop://$HOST:$DANCE_PORT_NUMBER"

if [ -n "$CDMW_REPOSITORY" ] ; then
    # use specified Repository
    echo "  get ComponentServers from Repository:" 
    echo "     corbaname::$CDMW_REPOSITORY/NameService#CDMW/SERVICES/ASSEMBLYANDDEPLOYMENT"
    EM_Args=${EM_Args}" -ORBInitRef ProcessDestinationNC=corbaname::$CDMW_REPOSITORY/NameService#CDMW/SERVICES/ASSEMBLYANDDEPLOYMENT"
else
if [ -n "$FTDomain" ] ; then
    # use FT Repository group registered in FTManager (get it via Daemon)
    echo "  get ComponentServers from FT Repository:"
    echo "     corbaname::localhost:$DAEMON_PORT#$FTDomain/DefaultRootContextGroup/CDMW/SERVICES/ASSEMBLYANDDEPLOYMENT"
    EM_Args=${EM_Args}" -ORBInitRef ProcessDestinationNC=corbaname::localhost:$DAEMON_PORT#$FTDomain/DefaultRootContextGroup/CDMW/SERVICES/ASSEMBLYANDDEPLOYMENT"
else 
    # use local Repository registered in Daemon
    echo "  get ComponentServers from local Repository:"
    echo "     corbaname::localhost:$DAEMON_PORT#CdmwRootContext/CDMW/SERVICES/ASSEMBLYANDDEPLOYMENT"
    EM_Args=${EM_Args}" -ORBInitRef ProcessDestinationNC=corbaname::localhost:$DAEMON_PORT#CdmwRootContext/CDMW/SERVICES/ASSEMBLYANDDEPLOYMENT"
fi
fi

EM_Args=${EM_Args}" -ORBInitRef DomainNC=corbaloc::localhost:$DAEMON_PORT/NameService"


if [ -z "$BadArgEncounter" ] ; then
    #set the environment for dance
    export PATH=${ACE_ROOT}/bin:$PATH
    export LD_LIBRARY_PATH=${ACE_ROOT}/lib:$LD_LIBRARY_PATH
    tmpDir=$(dirname $(dirname $0))/lib/c++
    if [ -d $tmpDir ] ; then
        cd $tmpDir
        export LD_LIBRARY_PATH=$(pwd):$LD_LIBRARY_PATH
        cd -
    fi


    # DAnCE will use this directive to redirect its logs to the test library libcdmwdanceFailerLoggingLibCcmDeploymentFailure.so, 
    # This library is used to simulate a crash of DAnCE

    ORBSvcConfDirective="dynamic DAnCE_Logger_Backend_Factory Service_Object * libcdmwdanceFailerLoggingLibCcmDeploymentFailure:_make_MyFactoryImpl() \"$(basename $0) --CdmwLogLevel=ALL --DaemonPort=${DAEMON_PORT}\""
 
    
    exec ${ACE_ROOT}/bin/dance ${EM_Args} -ORBSvcConfDirective "$ORBSvcConfDirective"
else
    echo "Error bad argument passed. "
    echo "$SCRIPT_NAME -h for help"
fi

