/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "ccm_deployment_failure/TaoDAnCELogger.hpp"

#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <Foundation/common/Assert.hpp>


namespace Cdmw
{
namespace CMD
{
namespace CcmDeploymentTool
{
    const char* DANCE_LOG_DOMAIN = "OFTAO.DAnCE";
    const char* DANCE_LOG_COMPONENT = "DAnCE";

    // This flag is needed to prevent this process from causing core on exit.
    // When the process exits the destructors of global objects are executed, which in turn call back 
    // the log method of this class, which then accesses already destroyed data. To enforce a gracefull exit
    // the log method just returns in case exit has been called already.
    static bool exitCalled = false;

    /**
     *Purpose:
     *<p>    This class allows collection of All DAnCE logs
     *
     *Features:
     *<p> Thread safe
     *
     */
    TaoDAnCELogger::TaoDAnCELogger() :
        m_logger(Cdmw::Logging::Logger::Get_Logger(DANCE_LOG_COMPONENT, DANCE_LOG_DOMAIN))
    {
        CDMW_ASSERT(m_logger != 0);
        // CDMW_LOG_FUNCTION ( m_logger, "TaoDAnCELogger::TaoDAnCELogger");
    }
      
      
    /// Destructor
    TaoDAnCELogger::~TaoDAnCELogger() throw()
    {
        // CDMW_LOG_FUNCTION ( m_logger, "TaoDAnCELogger::~TaoDAnCELogger"); // Will core if uncommented
        (void) this->close ();
        // m_logger will be deleted by LogManager at destruction
        m_logger = 0;
    }


    /// Open a new event log.
    /**
     * Initialize the event logging facility.
     * @param logger_key The name of the calling program. This name is
     *                   used as the @arg ident in the syslog entries. If
     *                   it is 0 (no name), the application name as
     *                   returned from ACE_Log_Msg::program_name() is used.
     */
    int TaoDAnCELogger::open(const ACE_TCHAR *logger_key)
    {
        // CDMW_LOG_FUNCTION ( m_logger, "TaoDAnCELogger::open");
        // CDMW_LOG_FUNCTION_RETURN(0);
        return 0;
    }
        
    /// Reset the backend.
    int TaoDAnCELogger::reset(void)
    {
        // CDMW_LOG_FUNCTION ( m_logger, "TaoDAnCELogger::reset");
        int result = this->close();
        // CDMW_LOG_FUNCTION_RETURN(result);
        return result;
    }

    /// Close the backend completely.
    int TaoDAnCELogger::close(void)
    {
        // CDMW_LOG_FUNCTION ( m_logger, "TaoDAnCELogger::close");
        // CDMW_LOG_FUNCTION_RETURN(0);
        return 0;
    }

    // This is called when we want to log a message.
    int TaoDAnCELogger::log(ACE_Log_Record &log_record) 
    {
    
        if(exitCalled) {
        return 0;
        }
    
        // CDMW_LOG_FUNCTION ( m_logger, "TaoDAnCELogger::log");
        
        //std::cout << log_record.msg_data() << std::endl;
        if (strstr(log_record.msg_data(),"createComponent")!=NULL) {
         exitCalled = true;
         std::cout << "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" << std::endl;
         std::cout << "!!!!!!!!DAnCE will exit immediately!!!!!!!!!" << std::endl;
         std::cout << "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" << std::endl;         
                 exit(0);      
        }
        
        else{
        long level = (long) this->convert_level (log_record.type ());
        level |= Cdmw::Logging::ORB_TRACE;
        if (m_logger->is_enabled_for(level)) {
            /// Get the message data of the <Log_Record>.
            const ACE_TCHAR * msg_data = log_record.msg_data ();
            // NOTA: ACE_TCHAR is a typedef to char
            m_logger->log(level,msg_data,"",-1);
        }
        }
        
        // CDMW_LOG_FUNCTION_RETURN(0);
        return 0;
    }
            
    int TaoDAnCELogger::convert_level(int acelevel)
    {
        // CDMW_LOG_FUNCTION ( m_logger, "TaoDAnCELogger::log");

        int level = acelevel;
  
        switch (acelevel) {
        case LM_STARTUP:
        case LM_SHUTDOWN:
        case LM_TRACE:
        case LM_INFO:
            level = Cdmw::Logging::INFO;
            break;
        case LM_DEBUG:
            level = Cdmw::Logging::DEBUG;
            break;
        case LM_NOTICE:
        case LM_WARNING:
            level = Cdmw::Logging::WARN;
            break;
        case LM_ERROR:
            level = Cdmw::Logging::ERROR;
            break;
        case LM_CRITICAL:
        case LM_ALERT:
        case LM_EMERGENCY:
        default:
            level = Cdmw::Logging::FATAL;
            break;
        }

        // CDMW_LOG_FUNCTION_RETURN(level);
        return level;
    }
} // End namespace CcmDeploymentTool
} // End namespace CMD
} // End namespace Cdmw
