/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef ___CCM_FT_TEST_CCM_SERVER_IMPL_HPP__
#define ___CCM_FT_TEST_CCM_SERVER_IMPL_HPP__

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <Foundation/osthreads/ThreadHandle.hpp>
#include <Foundation/osthreads/Mutex.hpp>
#include <Foundation/osthreads/Condition.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>

#include "ccm_ft_events/User_ccm_ft_events.skel.hpp"
#include "ccm_ft_events/EventInfo.hpp"
#include "ccm_ft_events/CCM_HostInfo_impl.hpp"

namespace CcmFtEventsTest
{

 class CCM_Server_impl;
    

class test_ccm_ft_activate : public CppUnit::TestFixture
{
    CCM_Server_impl * m_ccm_server;

  ::CcmFtEventsTest::CCM_Server_Context_var m_session_context;
    
public:
    test_ccm_ft_activate()
    {
        CORBA::Object_var obj1 = Cdmw::TestUtils::CORBATestManager::instance()->get_object("server_context");
        m_session_context = ::CcmFtEventsTest::CCM_Server_Context::_narrow(obj1.in());
    };
    

    ~test_ccm_ft_activate()
    {
    };

    //
    void do_tests();
    
    void tearDown()
    {
        //TEST_INFO("tearDown()");
        m_session_context = ::CcmFtEventsTest::CCM_Server_Context::_nil();
    }

    CPPUNIT_TEST_SUITE( test_ccm_ft_activate );
    CPPUNIT_TEST( do_tests );
    CPPUNIT_TEST_SUITE_END();

};
       
    
class TestThread : public Cdmw::OsSupport::Thread 
{
    public:
        TestThread()
         {};
        
    protected:
        void run() throw()
        {
            Cdmw::TestUtils::ThreadedTestRunner runner(Cdmw::TestUtils::CORBATestManager::instance()->get_controller());
            runner.addTest( CppUnit::TestFactoryRegistry::getRegistry("test_ccm_ft_activate").makeTest() );
            runner.start();
            runner.join();
        };
        
    private:
        TestThread(const TestThread&);
        TestThread& operator = (const TestThread&);
};

    
//
// IDL:thalesgroup.com/CcmFtEventsTest/CCM_Server:1.0
//
class CCM_Server_impl : virtual public ::UserCcmFtEventsTest::User_CCM_Server,
                        virtual public Cdmw::OrbSupport::RefCountLocalObject
{
    CCM_Server_impl(const CCM_Server_impl&);
    void operator=(const CCM_Server_impl&);

    ::CcmFtEventsTest::CCM_Server_Context_var m_session_context;
    ::CcmFtEventsTest::CCM_HostInfo_var  m_host_info;
    TestThread m_testThread;

public:

    CCM_Server_impl();
    ~CCM_Server_impl();
     
    // Session component specific operations
    //
    void set_session_context(Components::SessionContext_ptr ctx)
        throw(CORBA::SystemException);
    void ccm_activate()
        throw(CORBA::SystemException);
    void ccm_passivate()
        throw(CORBA::SystemException);
    void ccm_remove()
        throw(CORBA::SystemException);


    ::CcmFtEventsTest::CCM_Server_Context* get_context()
    {
        return ::CcmFtEventsTest::CCM_Server_Context::_duplicate(m_session_context.in());
    }

    CcmFtEventsTest::CCM_HostInfo_ptr
    CcmFtEventsTest::CCM_Server_impl::get_serverHostInfo ()
      throw ( CORBA::SystemException );

      // Synchronization mutex.
      static Cdmw::OsSupport::Mutex  M_mutex;
      static bool M_ack_evts;
      static Cdmw::OsSupport::Condition M_condition;

      static std::string M_host1;
      static std::string M_host2;
      static struct EventInfos M_event_infos;
 };

}; // End of namespace CcmFtEventsTest


#endif // ___CCM_FT_TEST_CCM_SERVER_IMPL_HPP__

