/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <ccm_ft_events/CCM_Client_impl.hpp>

#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>


//
// IDL:thalesgroup.com/CcmFtEventsTest/CCM_Client:1.0
//
CcmFtEventsTest::CCM_Client_impl::CCM_Client_impl()
     : m_session_context(CcmFtEventsTest::CCM_Client_Context::_nil()),
       m_ccm_activate_was_called(false)
{
}

CcmFtEventsTest::CCM_Client_impl::~CCM_Client_impl()
{
    std::cout << "   ... CCM_Client_impl::~CCM_Client_impl called!" << std::endl;
}

//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
CcmFtEventsTest::CCM_Client_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_Client_impl::Set_session_context called!" << std::endl;
   
    m_session_context = ::CcmFtEventsTest::CCM_Client_Context::_narrow(ctx); 
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CcmFtEventsTest::CCM_Client_impl::ccm_activate()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_Client_impl::ccm_activate called!" << std::endl;
   std::cout << "   ... CCM_Client_impl::ccm_activate: sleep 5 s." << std::endl;
   sleep(5);
    // PCR-1957 - No need of a mutex, push_<EventType> must be called after 'ccm_activate'
    m_ccm_activate_was_called = true;
    std::cout << "   ... CCM_Client_impl::ccm_activate: end" << std::endl;
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
CcmFtEventsTest::CCM_Client_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_Client_impl::Ccm_passivate called!" << std::endl;
} 



//
// ccm_remove function : it destroys the component (call by the container)
//
void
CcmFtEventsTest::CCM_Client_impl::ccm_remove()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_Client_impl::Ccm_remove called!" << std::endl;
}

void
CcmFtEventsTest::CCM_Client_impl::push_eventConsumes (CcmFtEventsTest::EventTest * event)
        throw (CORBA::SystemException)
{
    static int event_counter = 0;
  {
    CDMW_MUTEX_GUARD(m_mutex);
    event_counter++;
  }

  // PCR-1957: ccm_activate should be called at first
  CDMW_ASSERT(m_ccm_activate_was_called == true);

    std::cout << "Client: Event #"  << event->numOfEvent() << " received"
          << std::endl;
    
    ::CcmFtEventsTest::HostInfo_var server_facet;

     try {
       server_facet = m_session_context->get_connection_serverHostInfo();
       
       if (CORBA::is_nil(server_facet.in()) != 0)
     {
      CDMW_ASSERT(false);
     }
       
       //get the connection to the server 
       server_facet->display_info( Cdmw::OsSupport::OS::get_hostname().c_str(), 
                   event->numOfEvent() );

       // Only the primary client component may exit
       if( event_counter ==  MAX_NB_EVENTS / 2 
       && event->numOfEvent() < MAX_NB_EVENTS / 2 + 1 ) {
     
     exit(0);

       }
       
    } catch (const CORBA::SystemException& e) {
        TEST_INFO("Unexpected CORBA::SystemException: " << e);
    CDMW_ASSERT(false);
    }
}







