/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef ___CCM_FT_TEST_CCM_CLIENT_IMPL_HPP__
#define ___CCM_FT_TEST_CCM_CLIENT_IMPL_HPP__

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <Foundation/osthreads/ThreadHandle.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/logging/Logger.hpp>
#include <ccm_ft_perf/User_ccm_ft_perf.skel.hpp>

namespace CcmFtTest
{

    class CCM_Client_impl;
    

    class test_ccm_ft_activate : public CppUnit::TestFixture
    {
        CCM_Client_impl * m_user_ccm_client;
        ::CcmFtTest::ClientModule::CCM_Client_Context_var m_session_context;
        std::string m_expectedPrimaryHostname;
        std::string m_expectedBackupHostname;
    
    public:

        test_ccm_ft_activate();
    
        ~test_ccm_ft_activate();

        void do_tests();
    
        void tearDown();

        CPPUNIT_TEST_SUITE( test_ccm_ft_activate );
        CPPUNIT_TEST( do_tests );
        CPPUNIT_TEST_SUITE_END();


    private:

        Cdmw::Logging::Logger* m_logger;
    };
   
    
    class TestThread : public Cdmw::OsSupport::Thread 
    {

    public:

        TestThread();
    
        virtual
        ~TestThread()
        throw();
    
    protected:

        void run()
            throw();

    
    private:

        TestThread(const TestThread&);
        TestThread& operator = (const TestThread&);
        Cdmw::Logging::Logger* m_logger;

    };

    
    //
    // IDL:thalesgroup.com/UserCcmFtTest/CCM_Client:1.0
    //
    class CCM_Client_impl : virtual public ::UserCcmFtTest::User_CCM_Client,
                            virtual public Cdmw::OrbSupport::RefCountLocalObject
    {
        CCM_Client_impl(const CCM_Client_impl&);
        void operator=(const CCM_Client_impl&);


        ::CcmFtTest::ClientModule::CCM_Client_Context_var m_session_context;
        CORBA::String_var m_expectedPrimaryHostname;
        CORBA::String_var m_expectedBackupHostname;

    public:

        CCM_Client_impl();
        ~CCM_Client_impl();
     

        virtual void kill_primary()
            throw(CORBA::SystemException);
    
        virtual void is_primary(const char*)
            throw(CORBA::SystemException);

        virtual CcmFtTest::FacetModule::CCM_FacetClientModule_ptr get_clientModule()
            throw (CORBA::SystemException);
    
        //
        // attribute expectedPrimaryHostname
        //
        virtual char * expectedPrimaryHostname ()
            throw (CORBA::SystemException);
        virtual void expectedPrimaryHostname (const char * a)
            throw (CORBA::SystemException);

        
        //
        // attribute expectedBackupHostname
        //
        virtual char * expectedBackupHostname ()
            throw (CORBA::SystemException);
        virtual void expectedBackupHostname (const char * a)
            throw (CORBA::SystemException);

        //
        // Session component specific operations
        //
        void set_session_context(Components::SessionContext_ptr ctx)
            throw(CORBA::SystemException);
        void ccm_activate()
            throw(CORBA::SystemException);
        void ccm_passivate()
            throw(CORBA::SystemException);
        void ccm_remove()
            throw(CORBA::SystemException);


        ::CcmFtTest::ClientModule::CCM_Client_Context* get_context();

        unsigned long get_elapse();

    private:

        TestThread m_testThread;
        Cdmw::OsSupport::OS::Timeval m_time_exit;
        Cdmw::OsSupport::OS::Timeval m_time_primary;
        unsigned long m_elapse_time;
        Cdmw::Logging::Logger* m_logger;

    };

}; // End of namespace CcmFtTest


#endif // ___CCM_FT_TEST_CCM_CLIENT_IMPL_HPP__

