/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>

#include <ccm_ft_perf/CCM_ServerWithFacet_impl.hpp>

#include <Foundation/ossupport/OS.hpp>
#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/FunctionLogger.hpp>

//
// IDL:thalesgroup.com/CcmFtTest/CCM_ServerWithFacet:1.0
//
CcmFtTest::CCM_ServerWithFacet_impl::CCM_ServerWithFacet_impl()
        : m_logger(Cdmw::Logging::Logger::Get_Logger(
                   Cdmw::Logging::LogManager::Instance()->get_process_name(),
                   "CDMW.QUALIF.CcmFtPerf"))
{
    CDMW_LOG_FUNCTION(m_logger, "");
}

CcmFtTest::CCM_ServerWithFacet_impl::~CCM_ServerWithFacet_impl()
{
    CDMW_LOG_FUNCTION(m_logger, "");
}


CcmFtTest::FacetModule::CCM_FacetServerModule_ptr 
CcmFtTest::CCM_ServerWithFacet_impl::get_serverModule ()
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger, "");
    return CcmFtTest::FacetModule::CCM_FacetServerModule::_duplicate(this);
}


//
// attribute expectedPrimaryHostname
//
char*
CcmFtTest::CCM_ServerWithFacet_impl::expectedPrimaryHostname()
    throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger, "");
    CDMW_LOG_FUNCTION_RETURN("expectedPrimaryHostname [" << m_expectedPrimaryHostname << "]");
    return CORBA::string_dup(m_expectedPrimaryHostname);
}

void
CcmFtTest::CCM_ServerWithFacet_impl::expectedPrimaryHostname(const char* a)
    throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger, "expectedPrimaryHostname [" << a << "]");
    m_expectedPrimaryHostname =  CORBA::string_dup(a);
}


//
// attribute expectedBackup1Hostname
//
char*
CcmFtTest::CCM_ServerWithFacet_impl::expectedBackupHostname()
    throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger, "");
    CDMW_LOG_FUNCTION_RETURN("expectedBackupHostname [" << m_expectedBackupHostname << "]");
    return CORBA::string_dup(m_expectedBackupHostname);
}

void
CcmFtTest::CCM_ServerWithFacet_impl::expectedBackupHostname(const char* a)
    throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger, "expectedBackupHostname [" << a << "]");
    m_expectedBackupHostname =  CORBA::string_dup(a);
}

//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
CcmFtTest::CCM_ServerWithFacet_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger, "");
    m_session_context = ::CcmFtTest::ServerModule::CCM_ServerWithFacet_Context::_narrow(ctx); 
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CcmFtTest::CCM_ServerWithFacet_impl::ccm_activate()
   throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger, "");
    if (!strcmp(m_expectedBackupHostname, Cdmw::OsSupport::OS::get_hostname().c_str())) {
        ::CcmFtTest::FacetModule::FacetClientModule_var client = m_session_context->get_connection_clientModule();
        client->is_primary(Cdmw::OsSupport::OS::get_hostname().c_str());
    }
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//

void
CcmFtTest::CCM_ServerWithFacet_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger, "");
} 



//
// ccm_remove function : it destroys the component (call by the container)
void
CcmFtTest::CCM_ServerWithFacet_impl::ccm_remove()
   throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger, "");
}


::CcmFtTest::ServerModule::CCM_ServerWithFacet_Context* CcmFtTest::CCM_ServerWithFacet_impl::get_context()
{
    CDMW_LOG_FUNCTION(m_logger, "");
    return ::CcmFtTest::ServerModule::CCM_ServerWithFacet_Context::_duplicate(m_session_context.in());
}

void
CcmFtTest::CCM_ServerWithFacet_impl:: kill_primary()
        throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger, "");
    if (!strcmp(m_expectedPrimaryHostname, Cdmw::OsSupport::OS::get_hostname().c_str())) {
        ::CcmFtTest::FacetModule::FacetClientModule_var client = m_session_context->get_connection_clientModule();
        client->kill_primary();
        exit(0);
    }
}
