/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef ___CCM_FT_PUBLISHER_CRASH_CCM_CLIENT_IMPL_HPP__
#define ___CCM_FT_PUBLISHER_CRASH_CCM_CLIENT_IMPL_HPP__

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/osthreads/Mutex.hpp>

#include <ccm_ft_publisher_crash/CcmFtPublisherCrashLocal.skel.hpp>

#include <queue>


namespace CcmFtPublisherCrash
{


//
// IDL:thalesgroup.com/CcmFtPublisherCrash/Session_CCM_Client:1.0
//
class CCM_Client_impl : virtual public ::CcmFtPublisherCrash::Session_CCM_Client,
                        virtual public Cdmw::OrbSupport::RefCountLocalObject
{
    CCM_Client_impl(const CCM_Client_impl&);
    void operator=(const CCM_Client_impl&);


    ::CcmFtPublisherCrash::CCM_Client_Context_var m_session_context;
     
public:

    CCM_Client_impl();
    ~CCM_Client_impl();

    //
    // Session component specific operations
    //
    void set_session_context(Components::SessionContext_ptr ctx)
        throw(CORBA::SystemException);
    void ccm_activate()
        throw(CORBA::SystemException);
    void ccm_passivate()
        throw(CORBA::SystemException);
    void ccm_remove()
        throw(CORBA::SystemException);


    //
    // from_publisher consumer
    //
    virtual void push_from_publisher (CcmFtPublisherCrash::PublishedEvent * event)
        throw (CORBA::SystemException);
    
    //
    // from_emitter consumer
    //
    virtual void push_from_emitter (CcmFtPublisherCrash::EmittedEvent * event)
        throw (CORBA::SystemException);
    

private:

    typedef std::queue<Components::EventBase_var> EventList;
    EventList m_event_list;
    
    friend class test_ccm_ft_publisher_crash;
};



class test_ccm_ft_publisher_crash : public CppUnit::TestFixture
{
    CCM_Client_impl * m_ccm_client;
    
public:
    test_ccm_ft_publisher_crash();

    //
    void do_tests();

private:
    
    void check_published_event(CCM_Client_impl::EventList& event_list,
                               const std::string& expected_name);
                             
    void check_emitted_event(CCM_Client_impl::EventList& event_list,
                             const std::string& expected_name);

    
    // register test suite and test operation
    CPPUNIT_TEST_SUITE( test_ccm_ft_publisher_crash );
    CPPUNIT_TEST( do_tests );
    CPPUNIT_TEST_SUITE_END();

};
    
}; // End of namespace CcmFtPublisherCrash


#endif // ___CCM_FT_PUBLISHER_CRASH_CCM_CLIENT_IMPL_HPP__

