/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <ccm_ft_publisher_crash/CCM_Server_impl.hpp>
#include <ccm_ft_publisher_crash/PublishedEvent_impl.hpp>
#include <ccm_ft_publisher_crash/EmittedEvent_impl.hpp>

#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtManagedElementContexts.stub.hpp>


namespace CcmFtPublisherCrash
{

//
// IDL:thalesgroup.com/CcmFtTest/ServerModule/CCM_Server:1.0
//
CCM_Server_impl::CCM_Server_impl()
{
    m_session_context = CcmFtPublisherCrash::CCM_Server_Context::_nil();
}

CCM_Server_impl::~CCM_Server_impl()
{
    std::cout << "   ... CCM_Server_impl::~CCM_Server_impl called!" << std::endl;
}


void CCM_Server_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_Server_impl::set_session_context called!" << std::endl;
    m_session_context = ::CcmFtPublisherCrash::CCM_Server_Context::_narrow(ctx);
}



//
// ccm_activate function : it actives the component (call by the container)
//
void
CCM_Server_impl::ccm_activate()
   throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_Server_impl::ccm_activate called!" << std::endl;
    
    // get component name from ManagedComponentContext
    CORBA::String_var component_name;
    try {
        // get ManagedComponentContext
        CORBA::Object_var obj = 
            m_session_context->resolve_service_reference("ManagedComponentContext");
        CdmwPlatformMngt::ManagedElementContexts::ComponentContext_var supContext =
            CdmwPlatformMngt::ManagedElementContexts::ComponentContext::_narrow(obj.in());
        CDMW_ASSERT(!CORBA::is_nil(supContext.in()));

        component_name = supContext->managed_element_name();
        
    } catch (const Components::CCMException& e) {
        std::cout << "ManagedComponentContext no found !!" << std::endl;
        CDMW_ASSERT(false);
    } catch (const CORBA::SystemException& e) {
        std::cout << "SystemException during ManagedComponentContext test : " <<
                  Cdmw::OrbSupport::OrbSupport::exception_to_string(e) << std::endl;
        CDMW_ASSERT(false);
    }
    
    
    try {    
        // publish event to client
        std::cout << "   ... CCM_Server_impl: publish event to Client" << std::endl;
        PublishedEvent_var event1 = new PublishedEvent_impl(component_name.in());
        m_session_context->push_event_publisher(event1.in());
        
        
        // wait 1 second
        Cdmw::OsSupport::OS::sleep(1000);
        
        // emit event to client
        std::cout << "   ... CCM_Server_impl: emit event to Client" << std::endl;
        EmittedEvent_var event2 = new EmittedEvent_impl(component_name.in());
        m_session_context->push_event_emitter(event2.in());
    } catch (const CORBA::SystemException& e) {
        std::cout << "SystemException sending events to Client : " <<
                  Cdmw::OrbSupport::OrbSupport::exception_to_string(e) << std::endl;
        CDMW_ASSERT(false);
    }
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
CCM_Server_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_Server_impl::ccm_passivate called!" << std::endl;
} 


//
// ccm_remove function : it destroys the component (call by the container)
//
void
CCM_Server_impl::ccm_remove()
   throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_Server_impl::ccm_remove called!" << std::endl;
}


};  // namespace CcmFtPublisherCrash

