/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009. 
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */

#ifndef ___CCM_TEST_CCM_RECORDING_CCM_CLIENT_IMPL_HPP__
#define ___CCM_TEST_CCM_RECORDING_CCM_CLIENT_IMPL_HPP__

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/osthreads/Mutex.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>

#include<CCMContainer/ccmcommon/CCMUtils.hpp>

#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/TestResult.h>
#include <cppunit/XmlOutputter.h>
#include <cppunit/TextOutputter.h>
#include <cppunit/ui/text/TestRunner.h>

#include <TraceAndPerf/recordingidl/CdmwRecordingData.stub.hpp>

#include "ccm_ft_recorder/TestCcmFtRecorderLocal.skel.hpp"
#include "ccm_ft_recorder/RecordedOperationEvent_impl.hpp"


namespace 
{
  class test_ccm_ft_recorder;
}


namespace TestCcmFtRecorder
{

class CCM_Client_impl;
    

//
// IDL:thalesgroup.com/TestCcmFtRecorder/Session_CCM_Client:1.0
//
class CCM_Client_impl : virtual public ::TestCcmFtRecorder::Session_CCM_Client,
                        virtual public Cdmw::OrbSupport::RefCountLocalObject
{
    // need to access context and mutex
    friend class test_ccm_ft_recorder;
public:

    CCM_Client_impl();
    ~CCM_Client_impl();
     
    void
    set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException);

    void
    ccm_activate()
    throw(CORBA::SystemException);

    void
    ccm_passivate()
    throw(CORBA::SystemException);

    void
    ccm_remove()
    throw(CORBA::SystemException);
    
    void 
    push_event_consumer(RecordedOperationEvent*)
    throw(CORBA::SystemException);

    static void
    set_nb_invocation( unsigned int nb_invocation )
      throw();

    static unsigned int
    get_nb_invocation()
      throw();

    CCM_Client_Context_ptr get_context()
    {
      return CCM_Client_Context::_duplicate(m_session_context.in());
    }

    std::string get_time();
        
    //TestCcmFtRecorder::RecordedOperationEvent* lastReceivedEvent;
    
    int m_nbTestOperation;
    int m_nbGenericOperation;
    
    typedef std::vector<RecordedOperationEvent_var> RecordList;
    RecordList m_sendList;
    RecordList m_recordList;
    

private:
    static unsigned int m_nb_invocation;      

    CCM_Client_impl(const CCM_Client_impl&);

    void operator=(const CCM_Client_impl&);

    Cdmw::TestUtils::ThreadedTestRunner* runner;    

    Cdmw::OsSupport::Mutex m_mtx;

    CCM_Client_Context_var m_session_context; 
    
};

}; // End of namespace TestCcmFtRecorder


#endif // ___CCM_TEST_CCM_RECORDING_CCM_CLIENT_IMPL_HPP__

