/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009. 
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "ccm_ft_recorder/CCM_Client_impl.hpp"
#include "ccm_ft_recorder/RecordedOperationEvent_impl.hpp"

#include "tao/CDR.h"
/*#include "ace/CDR_Stream.h"*/

#include <Foundation/common/Assert.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/CosNaming.stub.hpp>

#include <recordingidl/CdmwRecordingGenericRecorder.skel.hpp>





static std::string CCM_CLIENT_IMPL_NAME = "CCMClientImpl_object";
static std::string SESSION_CONTEXT_NAME = "CCM_Client_Context_object";  

//
// IDL:thalesgroup.com/TestPerf/CCM_Client:1.0
//
TestCcmFtRecorder::CCM_Client_impl::CCM_Client_impl()
    : m_session_context(TestCcmFtRecorder::CCM_Client_Context::_nil())
{
}

TestCcmFtRecorder::CCM_Client_impl::~CCM_Client_impl()
{
   if(runner != 0) delete runner;
}

//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
TestCcmFtRecorder::CCM_Client_impl::set_session_context(Components::SessionContext_ptr ctx)
  throw(CORBA::SystemException)
{
  std::cout << "Set_session_context called!" << std::endl;
   
  m_session_context = ::TestCcmFtRecorder::CCM_Client_Context::_narrow(ctx); 
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
TestCcmFtRecorder::CCM_Client_impl::ccm_activate()
  throw(CORBA::SystemException)
{

    Cdmw::TestUtils::CORBATestManager::instance()->add (CCM_CLIENT_IMPL_NAME, dynamic_cast< ::TestCcmFtRecorder::CCM_Client_ptr > (this) );
    //Cdmw::TestUtils::CORBATestManager::instance()->add (SESSION_CONTEXT_NAME, this->m_session_context);
    
  Cdmw::TestUtils::CORBATestManager::instance()->add("client_context", get_context());
  

  //  ::CommonDemo::Registration_var register_server;
  //     register_server = m_session_context->get_connection_to_register();
  //     CORBA::String_var client_name = register_server->_cxx_register();
  //     set_name(client_name.in());

  runner = new Cdmw::TestUtils::ThreadedTestRunner(Cdmw::TestUtils::CORBATestManager::instance()->get_controller());
  runner->addTest( CppUnit::TestFactoryRegistry::getRegistry("test_ccm_ft_recorder").makeTest() );
  runner->start();
    
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//

void
TestCcmFtRecorder::CCM_Client_impl::ccm_passivate()
  throw(CORBA::SystemException)
{
  std::cout << "Ccm_passivate called!" << std::endl;
  if(runner != 0)
     runner->join();
} 

//unsigned int TestPerf::CCM_Client_impl::m_nb_invocation = 1000;

//
// ccm_remove function : it destroys the component (call by the container)
void
TestCcmFtRecorder::CCM_Client_impl::ccm_remove()
  throw(CORBA::SystemException)
{
  std::cout << "Ccm_remove called!" << std::endl;
}


void 
TestCcmFtRecorder::CCM_Client_impl::push_event_consumer(RecordedOperationEvent* evt)
  throw(CORBA::SystemException)
{
    std::cout << "*** CCMClientImpl.push_event_consumer() called! -  Received : "
              << evt->recorder_name() << " / " << evt->operation_name() << std::endl;

    CDMW_MUTEX_GUARD(m_mtx);

    if (std::string(evt->operation_name()) == std::string("test_operation")) {
        m_nbTestOperation++;
    } else if (std::string(evt->operation_name()) == std::string("record_data")) {
        m_nbGenericOperation++;
    } else {
        std::cout << "Unknown Event received !!!" << std::endl;
        assert(false);
    }

    // Add the event to the list of recorded operations
    TestCcmFtRecorder::RecordedOperationEvent_var lastEvent = new RecordedOperationEvent_impl( dynamic_cast< RecordedOperationEvent_impl& > (*evt));
    m_recordList.push_back(lastEvent);

}

std::string TestCcmFtRecorder::CCM_Client_impl::get_time()
{
    Cdmw::OsSupport::OS::Timeval t = Cdmw::OsSupport::OS::get_time();
    std::ostringstream ostr;
    ostr << t.seconds << "." << t.microseconds;
    return (ostr.str());
}



namespace 
{

  /**
   *  This class provides an object to test the ccm_activate.
   *
   **/

  static  std::string MY_RECORDER_PRIMARY = "MyRecorder1";
  static  std::string MY_RECORDER_BACKUP = "MyRecorder2"; 
  static  std::string OPERATION_NAME = "test_operation";        
  static  std::string GENERIC_OPERATION_NAME = "record_data";        
  static  std::string PARAM_A_S_VALUE = "value for 'a' param";
  static  std::string PARAM_A_S_SUCIDE_VALUE = "MyRecorder: kill yourself !!";
  static  short       PARAM_A_I_VALUE = 1;
  static  std::string PARAM_B1_S_VALUE = "value for 'b' param  = in";
  static  short       PARAM_B1_I_VALUE = 1;
  static  std::string PARAM_B2_S_VALUE = "value for 'b' param  = out";
  static  short       PARAM_B2_I_VALUE = 1;
  static  std::string PARAM_C_S_VALUE = "value for 'c' param";
  static  short       PARAM_C_I_VALUE = 1;
  static  std::string PARAM_RET_S_VALUE = "value for return";
  static  short       PARAM_RET_I_VALUE = 1;    
  static  unsigned long REQUEST_MAX_DURATION = 150;    
   

 
    
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(test_ccm_ft_recorder, "test_ccm_ft_recorder");


class test_ccm_ft_recorder : public CppUnit::TestFixture
{
  private:
    TestCcmFtRecorder::CCM_Client_impl* m_client;


    // issues request to server component
    bool send_request_to_server_component(std::string recorder_name, bool suicide, std::string msg)
    {
        std::cout << "CCMSClientImpl::send_request_to_server_component() at " << m_client->get_time()
                  << " - " << msg << std::endl;

        CORBA::Object_var obj = Cdmw::TestUtils::CORBATestManager::instance()->get_object("client_context");
        ::TestCcmFtRecorder::CCM_Client_Context_var client_context =
                 ::TestCcmFtRecorder::CCM_Client_Context::_narrow(obj.in());
        CPPUNIT_ASSERT(!CORBA::is_nil(client_context.in()));
        std::ostringstream message;

        try {
            ::TestCcmFtRecorder::Test_var test
                  = client_context->get_connection_test_receptacle();

            ::TestCcmFtRecorder::TestStruct_var a
                  = new ::TestCcmFtRecorder::TestStruct();
            if (suicide)
                a->s =  ::CORBA::string_dup (PARAM_A_S_SUCIDE_VALUE.c_str());
            else
                a->s =  ::CORBA::string_dup (PARAM_A_S_VALUE.c_str());
            a->i = PARAM_A_I_VALUE;

            ::TestCcmFtRecorder::TestStruct_var b
                  = new ::TestCcmFtRecorder::TestStruct();
            b->s =  ::CORBA::string_dup (PARAM_B1_S_VALUE.c_str());
            b->i = PARAM_B1_I_VALUE;          

            ::TestCcmFtRecorder::TestStruct_var c
                  = new ::TestCcmFtRecorder::TestStruct();       

            TestCcmFtRecorder::RecordedOperationEvent_var sendOpe = new TestCcmFtRecorder::RecordedOperationEvent_impl();
            sendOpe->recorder_name(::CORBA::string_dup(recorder_name.c_str()));
            sendOpe->operation_name(::CORBA::string_dup(OPERATION_NAME.c_str()));
            sendOpe->a_value(a.in());
            sendOpe->b1_value(b.in());

            ::TestCcmFtRecorder::TestStruct_var ret 
                  = test->test_operation(a.in(),  b.inout(),  c.out());
        
            message << "Out value of b.s is '" << std::string(b->s) <<
                "' instead of '" << PARAM_B2_S_VALUE << "'";
            CPPUNIT_ASSERT_MESSAGE(message.str(),
                                   (std::string(b->s) == (PARAM_B2_S_VALUE)));     


            message.str("");
            message << "Out value of b.i is '" << b->i <<
                "' instead of '" << PARAM_B2_I_VALUE << "'";
            CPPUNIT_ASSERT_MESSAGE(message.str(),
                                   (b->i == PARAM_B2_I_VALUE));


            message.str("");
            message << "Out value of c.s is '" << c->s << 
                             "' instead of '" << PARAM_C_S_VALUE << "'";
            CPPUNIT_ASSERT_MESSAGE(message.str(),
                                   (std::string(c->s) == (PARAM_C_S_VALUE)));

            message.str("");
            message << "Out value of c.i is '" << c->i << 
                             "' instead of '" << PARAM_C_I_VALUE << "'";
            CPPUNIT_ASSERT_MESSAGE(message.str(),
                                   (c->i == PARAM_C_I_VALUE));


            message.str("");
            message << "Return.s value is '" << ret->s << 
                "' instead of '" << PARAM_RET_S_VALUE << "'";
            CPPUNIT_ASSERT_MESSAGE(message.str(),
                                   (std::string(ret->s) == (PARAM_RET_S_VALUE)));


            message.str("");
            message << "Return.s value is '" << ret->i << 
                "' instead of '" << PARAM_RET_I_VALUE << "'";
            CPPUNIT_ASSERT_MESSAGE(message.str(),
                                   (ret->i == PARAM_RET_I_VALUE));

            sendOpe->b2_value(b.in());
            sendOpe->c_value(c.in());
            sendOpe->ret_value(ret.in());
            m_client->m_sendList.push_back(sendOpe);

        } catch (const CORBA::Exception & e) {
            message.str("");
            message << "Exception calling Server1's facet : " << e;
            TEST_INFO(message.str());
            CPPUNIT_ASSERT(false);          
        }

        return true;
    }


    bool send_data_to_generic_recorder(CdmwRecording::GenericRecorder_var genRecorder, std::string recorder_name, std::string msg)
    {

        std::cout << "CCMSClientImpl::send_data_to_generic_recorder() at " << m_client->get_time() << " - " << msg << std::endl;

//             cdmw.recording.tools.ApplicationRecorder appRecorder
//                 = new cdmw.recording.tools.ApplicationRecorder(
//                         genRecorder, "ccm_ft_recorder_test", "CCMClientImpl", null);
// 
//             // My NamedValue elements
//             //System.out.println("CCMServerImpl::test_operation(): Create  'MyContext'...");
//             NamedValue[] myContext = new NamedValue[1];
//             myContext[0] = new NamedValue();
//             myContext[0].name = "MyContext";
//             myContext[0].value = org.omg.CORBA.ORB.init().create_any();
//             myContext[0].value.insert_string("TestContext1");
// 
//             //System.out.println("CCMServerImpl::test_operation(): Create  'MyData'...");
//             NamedValue[] myData = new NamedValue[1];
//             myData[0] = new NamedValue();
//             myData[0].name = "MyData";
//             myData[0].value = org.omg.CORBA.ORB.init().create_any();
//             myData[0].value.insert_string(msg);
// 
//             //System.out.println("Call to 'ApplicationRecorder::Record(myContext, myData) ...");
//             appRecorder.Record(myContext, myData);
//             
//             RecordedOperationEvent sendOpe = new RecordedOperationEventImpl();
//             sendOpe.recorder_name = recorder_name;
//             sendOpe.operation_name = GENERIC_OPERATION_NAME;
//             this.ccmClientImpl.sendList.add(sendOpe);

        ::CdmwRecording::NamedValueSeq context;
        ::CdmwRecording::NamedValueSeq data;

        genRecorder->record_data(context, data);
            
        TestCcmFtRecorder::RecordedOperationEvent_var sendOpe = new TestCcmFtRecorder::RecordedOperationEvent_impl();
        sendOpe->recorder_name(::CORBA::string_dup(recorder_name.c_str()));
        sendOpe->operation_name(::CORBA::string_dup(GENERIC_OPERATION_NAME.c_str()));
        m_client->m_sendList.push_back(sendOpe);

        return true;
    }


    bool check_recorded_operations()
    {

        std::cout << "*** Check recorded operations" << std::endl;
        std::cout << " - Sent operations      : " << m_client->m_sendList.size() << std::endl;
        std::cout << " - Recorded operations  : " << m_client->m_recordList.size() << std::endl;
        
        std::ostringstream message;

        TestCcmFtRecorder::CCM_Client_impl::RecordList::iterator it1;
        TestCcmFtRecorder::CCM_Client_impl::RecordList::iterator it2;
            
//         std::cout << "*** Sent operations" << std::endl;
//         for (it1 = m_client->m_sendList.begin();
//              it1 != m_client->m_sendList.end();
//              it1++)
//         {
//             std::cout << " - " << (*it1)->recorder_name() <<  " / " <<(*it1)->operation_name() << std::endl;
//         }
// 
//         std::cout << "*** Recorded operations" << std::endl;
//         for (it1 = m_client->m_recordList.begin();
//              it1 != m_client->m_recordList.end();
//              it1++)
//         {
//             std::cout << " - " << (*it1)->recorder_name() <<  " / " <<(*it1)->operation_name() << std::endl;
//         }


        for (it1 = m_client->m_sendList.begin();
             it1 != m_client->m_sendList.end();
             it1++)
        {
            for (it2 = m_client->m_recordList.begin();
                 it2 != m_client->m_recordList.end();
                 it2++)
            {
                if (std::string((*it2)->operation_name()) == std::string((*it1)->operation_name())) {
                    if ( (std::string((*it2)->operation_name()) == OPERATION_NAME)
                      && (std::string((*it2)->recorder_name()) != std::string((*it1)->recorder_name())) ) {
                        continue;
                    }
                    break;
                }
            }

            if (it2 == m_client->m_recordList.end()) {
                std::cout << " - expected record not found : "
                          << (*it1)->recorder_name() << " / " << (*it1)->operation_name() << " !!!" << std::endl;
            }
            message.str("");
            message << "Record not found : " << (*it1)->recorder_name() << " / " << (*it1)->operation_name();            
            CPPUNIT_ASSERT_MESSAGE(message.str(), it2 != m_client->m_recordList.end());

//             std::cout << " - check recorded operation : "
//                       << (*it2)->recorder_name() << " / " << (*it2)->operation_name() << std::endl;

            if (std::string((*it2)->operation_name()) == OPERATION_NAME) {
                  message.str("");
                  message << "Value of a.s is '" << (*it2)->a_value().s << "' instead of '" << (*it1)->a_value().s << "'";
                  CPPUNIT_ASSERT_MESSAGE(message.str(), std::string((*it2)->a_value().s) == std::string((*it1)->a_value().s));   

                  message.str("");
                  message << "Value of a.i is '" << (*it2)->a_value().i << "' instead of '" << (*it1)->a_value().i << "'";
                  CPPUNIT_ASSERT_MESSAGE(message.str(), (*it2)->a_value().i == (*it1)->a_value().i);  

                  message.str("");
                  message << "Value of b1.s is '" << (*it2)->b1_value().s << "' instead of '" << (*it1)->b1_value().s << "'";
                  CPPUNIT_ASSERT_MESSAGE(message.str(), std::string((*it2)->b1_value().s) == std::string((*it1)->b1_value().s));   

                  message.str("");
                  message << "Value of b1.i is '" << (*it2)->b1_value().i << "' instead of '" << (*it1)->b1_value().i << "'";
                  CPPUNIT_ASSERT_MESSAGE(message.str(), (*it2)->b1_value().i == (*it1)->b1_value().i);  

                  message.str("");
                  message << "Value of b2.s is '" << (*it2)->b2_value().s << "' instead of '" << (*it1)->b2_value().s << "'";
                  CPPUNIT_ASSERT_MESSAGE(message.str(), std::string((*it2)->b2_value().s) == std::string((*it1)->b2_value().s));   

                  message.str("");
                  message << "Value of b2.i is '" << (*it2)->b2_value().i << "' instead of '" << (*it1)->b2_value().i << "'";
                  CPPUNIT_ASSERT_MESSAGE(message.str(), (*it2)->b2_value().i == (*it1)->b2_value().i);  

                  message.str("");
                  message << "Value of c.s is '" << (*it2)->c_value().s << "' instead of '" << (*it1)->c_value().s << "'";
                  CPPUNIT_ASSERT_MESSAGE(message.str(), std::string((*it2)->c_value().s) == std::string((*it1)->c_value().s));   

                  message.str("");
                  message << "Value of c.i is '" << (*it2)->c_value().i << "' instead of '" << (*it1)->c_value().i << "'";
                  CPPUNIT_ASSERT_MESSAGE(message.str(), (*it2)->c_value().i == (*it1)->c_value().i);  

                  message.str("");
                  message << "Value of ret.s is '" << (*it2)->ret_value().s << "' instead of '" << (*it1)->ret_value().s << "'";
                  CPPUNIT_ASSERT_MESSAGE(message.str(), std::string((*it2)->ret_value().s) == std::string((*it1)->ret_value().s));   

                  message.str("");
                  message << "Value of ret.i is '" << (*it2)->ret_value().i << "' instead of '" << (*it1)->ret_value().i << "'";
                  CPPUNIT_ASSERT_MESSAGE(message.str(), (*it2)->ret_value().i == (*it1)->ret_value().i);  
            }

            m_client->m_recordList.erase(it2);

        }

        // Check if there is non-expected records
        if (! m_client->m_recordList.empty()) {
            std::cout << "Following records were not expected !!! :" << std::endl;
            for (unsigned int i=0; i < m_client->m_recordList.size(); i++) {
                std::cout << "  - " << (i+1) << " : "
                          << m_client->m_recordList[i]->recorder_name() << " / " 
                          << m_client->m_recordList[i]->operation_name() << std::endl;
            }
        }

        CPPUNIT_ASSERT_MESSAGE("Some records were not expected !!!",
                               m_client->m_recordList.empty());

        return true;
    }


    bool check_duration(
            Cdmw::OsSupport::OS::Timeval t1, Cdmw::OsSupport::OS::Timeval t2, unsigned long max_duration)
    {
        if (t2 < t1)
            return false;
        unsigned long d = (t2.seconds - t1.seconds) * 1000000 + (t2.microseconds - t1.microseconds);
        if (d > max_duration * 1000)
            return false;

        return true;
    }



  public:
    test_ccm_ft_recorder()
    {    
    };


    ~test_ccm_ft_recorder() {};



    void do_tests()
    {

        TEST_INFO("********************************************");
        TEST_INFO("            ccm_activate start");
        TEST_INFO("********************************************");

        bool passed = false;

        CORBA::Object_var  obj = Cdmw::TestUtils::CORBATestManager::instance()->get_object(CCM_CLIENT_IMPL_NAME);
        CPPUNIT_ASSERT(!CORBA::is_nil(obj.in()));

        m_client = dynamic_cast< TestCcmFtRecorder::CCM_Client_impl* > (obj.in());
        CPPUNIT_ASSERT(m_client != 0);

        m_client->m_nbTestOperation = 0;
        m_client->m_nbGenericOperation =0;

        // get Recorder
        ::CdmwRecording::Recorder_var recorder = 0;
        TEST_INFO("Get Recorder");
        try {
            CORBA::Object_var myObj = m_client->m_session_context->resolve_service_reference("RecordingServer");
            recorder = ::CdmwRecording::Recorder::_narrow(myObj);
        } catch (...) {
            TEST_INFO("Exception getting recorder connection");
            CPPUNIT_ASSERT(false);
        }
        CPPUNIT_ASSERT(!CORBA::is_nil(recorder));

        // get NamingContext
        CosNaming::NamingContextExt_var nc = 0;
        std::cout << "CCMClientImpl::doTests(): Retrieve 'NamingContext'..." << std::endl;
        try {
            CORBA::Object_var myObj = m_client->m_session_context->resolve_service_reference("LocalNameService");
            nc = CosNaming::NamingContextExt::_narrow(myObj);
        }
        catch (...) {
            TEST_INFO("Exception Retrieving 'NamingContext' !!!");
            CPPUNIT_ASSERT(false);
        }
        CPPUNIT_ASSERT(!CORBA::is_nil(recorder));

        // get GenericRecorder
        ::CdmwRecording::GenericRecorder_var genRecorder = 0;
        std::cout << "CCMClientImpl::doTests(): Retrieve 'GenericRecorder'..." << std::endl;
        try {
            CORBA::Object_var myObj = nc->resolve_str("GenericRecorderDaemon");
            genRecorder = CdmwRecording::GenericRecorder::_narrow(myObj);
        }
        catch (...) {
            TEST_INFO("Exception Retrieving 'GenericRecorder' !!!");
            CPPUNIT_ASSERT(false);
        }
        CPPUNIT_ASSERT(!CORBA::is_nil(genRecorder));


        Cdmw::OsSupport::OS::Timeval t1, t2;
        short i = 0; short iMax = 5;
        short j = 0; short jMax = 10;
        bool suicide = false;
        short periode = 39;
        std::string recorder_name = MY_RECORDER_PRIMARY;

        for (i = 0 ;  i < iMax ; ++i)
        {
            std::ostringstream message;
            // Suicide of the primary recorder requested at midle time
            if ( i == iMax/2 + 1 ) {
                suicide = true;
                recorder_name = MY_RECORDER_BACKUP;
            } else {
                suicide = false;
            }

            // send request to server component, recording deamon should record the request/reply
            // then write the recorded into the recording server group, the recording server (the primary one)     
            // will push an event back the client component
            t1 = Cdmw::OsSupport::OS::get_time();
            message.str("");
            message << "Send request to Server Component : " << (i+1);
            passed = send_request_to_server_component(recorder_name, suicide, message.str());
            CPPUNIT_ASSERT(passed);
            t2 = Cdmw::OsSupport::OS::get_time();
            passed = check_duration(t1, t2, REQUEST_MAX_DURATION);
            CPPUNIT_ASSERT(passed);

            Cdmw::OsSupport::OS::sleep(periode);

            for (j = 0 ;  j < jMax ; ++j)
            {
                // Send data to Generic Recorder
                t1 = Cdmw::OsSupport::OS::get_time();
                message.str("");
                message << "Send data to Generic Recorder : " << ((i*jMax)+j+1);
                passed = send_data_to_generic_recorder(genRecorder, recorder_name, message.str());
                CPPUNIT_ASSERT(passed);
                t2 = Cdmw::OsSupport::OS::get_time();
                passed = check_duration(t1, t2, REQUEST_MAX_DURATION);
                CPPUNIT_ASSERT(passed);
                Cdmw::OsSupport::OS::sleep(periode);
            }
        }

        // waiting for all requests to be recorded
        short k = 0;
        while ( m_client->m_nbTestOperation < iMax  ||  m_client->m_nbGenericOperation < (iMax*jMax) )
        {
            Cdmw::OsSupport::OS::sleep(2000);

//             std::cout << "CCMServerImpl::doTests(): [" << k << "] Number of received 'test_operation' : "
//                               << m_client->m_nbTestOperation << " / " << (iMax) << std::endl;
//             std::cout << "CCMServerImpl::doTests(): [" << k << "] Number of received 'record_data' : "
//                               << m_client->m_nbGenericOperation << " / " << (iMax*jMax) << std::endl;

            if ( k++ > (((iMax*jMax)+iMax)/5) ) break;
        }

        std::cout << "CCMServerImpl::doTests(): Number of received event for 'test_operation' : "
                          << m_client->m_nbTestOperation << " / " << (iMax) << std::endl;
        std::cout << "CCMServerImpl::doTests(): Number of received event for 'record_data'    : "
                          << m_client->m_nbGenericOperation << " / " << (iMax*jMax) << std::endl;

        // Waiting end of test
        Cdmw::OsSupport::OS::sleep(2000);

        // check content of recorded requests
        passed = check_recorded_operations();
        CPPUNIT_ASSERT(passed);
            
        Cdmw::TestUtils::CORBATestManager::instance()->summary();      
    }


    CPPUNIT_TEST_SUITE( test_ccm_ft_recorder );
    CPPUNIT_TEST( do_tests );
    CPPUNIT_TEST_SUITE_END();


}; // End of class test_ccm_ft_recorder
    
} // End of anonymous namespace

