/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <ccm_ft_recorder/CCM_Server_impl.hpp>
#include <ccm_ft_recorder/RecordedOperationEvent_impl.hpp>

#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtManagedElementContexts.stub.hpp>

namespace {

  const  std::string PARAM_A_S_VALUE = "value for 'a' param";
  const  std::string PARAM_A_S_SUCIDE_VALUE = "MyRecorder: kill yourself !!";
  const  short       PARAM_A_I_VALUE = 1;
  const  std::string PARAM_B1_S_VALUE = "value for 'b' param  = in";
  const  short       PARAM_B1_I_VALUE = 1;
  const  std::string PARAM_B2_S_VALUE = "value for 'b' param  = out";
  const  short       PARAM_B2_I_VALUE = 1;
  const  std::string PARAM_C_S_VALUE = "value for 'c' param";
  const  short       PARAM_C_I_VALUE = 1;
  const  std::string PARAM_RET_S_VALUE = "value for return";
  const  short       PARAM_RET_I_VALUE = 1;       
       
}

namespace TestCcmFtRecorder
{

//
// IDL:thalesgroup.com/CcmFtTest/MyRecorderModule/CCM_MyRecorder:1.0
//
CCM_Server_impl::CCM_Server_impl()
{
    m_session_context = TestCcmFtRecorder::CCM_Server_Context::_nil();
}

CCM_Server_impl::~CCM_Server_impl()
{
}


void CCM_Server_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_Server_impl::set_session_context called!" << std::endl;
    m_session_context = ::TestCcmFtRecorder::CCM_Server_Context::_narrow(ctx);
}

::TestCcmFtRecorder::CCM_Server_Context* CCM_Server_impl::get_context()
{
    return ::TestCcmFtRecorder::CCM_Server_Context::_duplicate(m_session_context.in());
}


::TestCcmFtRecorder::CCM_Test_ptr  CCM_Server_impl::get_test_facet()
        throw(CORBA::SystemException)
{
    return TestCcmFtRecorder::CCM_Test::_duplicate(this);
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CCM_Server_impl::ccm_activate()
   throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_Server_impl::ccm_activate called!" << std::endl;
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
CCM_Server_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_Server_impl::ccm_passivate called!" << std::endl;
} 


//
// ccm_remove function : it destroys the component (call by the container)
//
void
CCM_Server_impl::ccm_remove()
   throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_Server_impl::ccm_remove called!" << std::endl;
}


::TestCcmFtRecorder::TestStruct * CCM_Server_impl::test_operation(
        const ::TestCcmFtRecorder::TestStruct & a,
        ::TestCcmFtRecorder::TestStruct & b,
        ::TestCcmFtRecorder::TestStruct_out c)
        throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_Server_impl::test_operation called!" << std::endl;
        b.s = PARAM_B2_S_VALUE.c_str();
        b.i = PARAM_B2_I_VALUE;
        
        c = new ::TestCcmFtRecorder::TestStruct();
        c->s = PARAM_C_S_VALUE.c_str();
        c->i = PARAM_C_I_VALUE;

        TestStruct *ret = new TestStruct();
        ret->s = PARAM_RET_S_VALUE.c_str();
        ret->i = PARAM_RET_I_VALUE;
        
        return ret;
}

}
