/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include "ccm_ft_request_identification/CCM_Client_impl.hpp"
#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <set>
#include "ccm_ft_request_identification/User_ccm_ft_request_identification.skel.hpp"


//
// IDL:thalesgroup.com/CcmFtTest/ClientModule/CCM_Client:1.0
//
CcmFtTest::CCM_Client_impl::CCM_Client_impl()
    : m_session_context(CcmFtTest::ClientModule::CCM_Client_Context::_nil()),
      m_testThread()
{
}

CcmFtTest::CCM_Client_impl::~CCM_Client_impl()
{
    std::cout << "   ... CCM_Client_impl::~CCM_Client_impl called!" << std::endl;
}





//
// facet clientRequestInfo
//
CcmFtTest::FacetModule::CCM_RequestInfo_ptr 
CcmFtTest::CCM_Client_impl::get_clientRequestInfo ()
    throw (CORBA::SystemException)
{
    return CcmFtTest::FacetModule::CCM_RequestInfo::_duplicate(this);
}

char*
CcmFtTest::CCM_Client_impl::get_request_id()
    throw (CORBA::SystemException)
{
    std::string request = "request";
    return CORBA::string_dup(request.c_str());
}    






//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
CcmFtTest::CCM_Client_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_Client_impl::Set_session_context called!" << std::endl;
   
    m_session_context = ::CcmFtTest::ClientModule::CCM_Client_Context::_narrow(ctx); 
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CcmFtTest::CCM_Client_impl::ccm_activate()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_Client_impl::Ccm_activate called!" << std::endl;
   ::CcmFtTest::ClientModule::CCM_Client_Context_var ctx = get_context();
   Cdmw::TestUtils::CORBATestManager::instance()->add("client_context", ctx.in());
   Cdmw::TestUtils::CORBATestManager::instance()->add("user_ccm_client", static_cast< ::UserCcmFtTest::User_CCM_Client_ptr > (this));
   m_testThread.start();
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
CcmFtTest::CCM_Client_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_Client_impl::Ccm_passivate called!" << std::endl;
} 



//
// ccm_remove function : it destroys the component (call by the container)
//
void
CcmFtTest::CCM_Client_impl::ccm_remove()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_Client_impl::Ccm_remove called!" << std::endl;
   Cdmw::TestUtils::CORBATestManager::instance()->remove_object("user_ccm_client");
   Cdmw::TestUtils::CORBATestManager::instance()->remove_object("client_context");
}


//
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(CcmFtTest::test_ccm_ft_activate, "test_ccm_ft_activate");

void
CcmFtTest::test_ccm_ft_activate::do_tests()
{
    Cdmw::OsSupport::OS::sleep(2000);
    
    CORBA::Object_var obj2 = Cdmw::TestUtils::CORBATestManager::instance()->get_object("user_ccm_client");
    ::UserCcmFtTest::User_CCM_Client_var user_ccm_client = ::UserCcmFtTest::User_CCM_Client::_narrow(obj2.in());
    m_user_ccm_client = reinterpret_cast< CcmFtTest::CCM_Client_impl* > (user_ccm_client.in());

    std::set< std::string > setstring;
    std::set< std::string >::size_type size = 1000;

    for (std::set< std::string >::size_type i = 0; i<size; i++)
    {

        try {
            TEST_INFO("Test that group is empty");
            //         add_nbOfRequestedTestOK(1);
            
            CcmFtTest::FacetModule::RequestInfo_var requestInfo = m_session_context->get_connection_serverRequestInfo();
            
            std::string requestId = requestInfo->get_request_id();
            setstring.insert(requestId);
            TEST_INFO("Request on the group succeed !! request id: " << requestId);
            CPPUNIT_ASSERT(true);
            
        } catch (const CORBA::SystemException& e) {
            CPPUNIT_ASSERT(false);
        } catch (const CORBA::Exception& e) {
            TEST_INFO("Unexpected CORBA::Exception: " << e);
            CPPUNIT_ASSERT(false);
        }
    }

    TEST_INFO("setstring.size()= "<<setstring.size()<<" size="<<size)
    if (setstring.size() == size)
        CPPUNIT_ASSERT(true);
    else
        CPPUNIT_ASSERT(false);
    
}





