/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <ccm_generic_recording/CCM_Server_impl.hpp>

#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>


namespace TestCcmRecording
{

//
// IDL:thalesgroup.com/CcmFtTest/ServerModule/CCM_Server:1.0
//
CCM_Server_impl::CCM_Server_impl()
{
    m_session_context = TestCcmRecording::CCM_Server_Context::_nil();
}

CCM_Server_impl::~CCM_Server_impl()
{
    std::cout << "   ... CCM_Server_impl::~CCM_Server_impl called!" << std::endl;
}


void CCM_Server_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_Server_impl::set_session_context called!" << std::endl;
    m_session_context = ::TestCcmRecording::CCM_Server_Context::_narrow(ctx);
}

::TestCcmRecording::CCM_Server_Context* CCM_Server_impl::get_context()
{
    return ::TestCcmRecording::CCM_Server_Context::_duplicate(m_session_context.in());
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CCM_Server_impl::ccm_activate()
   throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_Server_impl::ccm_activate called!" << std::endl;
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
CCM_Server_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_Server_impl::ccm_passivate called!" << std::endl;
} 


//
// ccm_remove function : it destroys the component (call by the container)
//
void
CCM_Server_impl::ccm_remove()
   throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_Server_impl::ccm_remove called!" << std::endl;
}


//
// facet test_recording
//
::TestCcmRecording::CCM_TestRecording_ptr 
CCM_Server_impl::get_test_recording()
    throw (CORBA::SystemException)
{
    // component itself implements the test_recording facet
    return TestCcmRecording::CCM_TestRecording::_duplicate(this);
}


void 
CCM_Server_impl::test_operation(const char * message)
  throw (CORBA::SystemException)
{
    std::cout << "CCM_Server_impl::test_operation(): " << message << std::endl;
}
  

void 
CCM_Server_impl::push_test_event(TestEvent* event)
  throw (CORBA::SystemException)
{
    std::cout << "CCM_Server_impl::push_test_event(): " 
              << event->message() << std::endl;
}
  


};  // namespace TestCcmRecording

