/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
#include <iostream>
#include <fstream>
#include <unistd.h>
#include <sys/types.h>
#include <sstream>

#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>

#include <ccm_generic_recording/Recorder_impl.hpp>

using namespace std;

int main(int argc, char * argv[])
{
    std::cout << "*** Recording Server : initialising .." << std::endl;
    int id = 0;
    
    if( argc > 0 && argv )
    {
        string str = Cdmw::OsSupport::OS::get_option_value(argc, argv, "--id");
        
        if (str != "no")  id = atoi(str.c_str());  
        else
        {
            std::cerr << "There is no id argument ( -id=<value> ) " << std::endl;
            return 1;
        }          
    }  
    
  try
  {
    // Initialize orb                      
    Cdmw::OrbSupport::StrategyList orb_strategies;
    orb_strategies.add_OrbThreaded();
    orb_strategies.add_PoaThreadPool(5);
    CORBA::ORB_var orb = 
        Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, orb_strategies);      

    // Get reference to Root POA.
    CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
    PortableServer::POA_var root_poa = PortableServer::POA::_narrow(obj);

    //Create IdAssignmentPolicy        
    PortableServer::IdAssignmentPolicy_var assignment_policy = root_poa->create_id_assignment_policy(PortableServer::USER_ID);
    // Create PolicyList
    CORBA::PolicyList policy_list;
    policy_list.length(1);
    policy_list[0] = PortableServer::IdAssignmentPolicy::_duplicate(assignment_policy);
    // Create the child POA
    PortableServer::POA_var poa = root_poa->create_POA("poa", PortableServer::POAManager::_nil(), policy_list);
    // Destroy object
    assignment_policy->destroy();
        
    // Activate POA manager
    PortableServer::POAManager_var mgr = poa->the_POAManager();
    mgr->activate();
    
    // Create an object
    POA_CdmwRecording::Recorder_impl recorder_servant;
    
    recorder_servant.set_id(id);
    
    stringstream strm2;   
    strm2 << "TestRecorder" << id ;
    const std::string corbaloc_name = strm2.str();
    
    // Create an object
    PortableServer::ObjectId_var oid = PortableServer::string_to_ObjectId(corbaloc_name.c_str());
    
    // Activate our Controller.
    poa->activate_object_with_id(oid, &recorder_servant);
    CORBA::Object_ptr object = poa->servant_to_reference (&recorder_servant);

    Cdmw::OrbSupport::OrbSupport::bind_object_to_corbaloc (orb.in(), corbaloc_name, object);        
        
    // Accept requests
    std::cout << "*** Recording Server ( "<< corbaloc_name << ") : waiting for requests.." << std::endl;
    orb->run();
  }
  catch (const CORBA::Exception& ex)
  {
    std::cout << "*** Recording Server : Exception caught in ORBThread" << std::endl;
    return -1;
  }

  return 0;
}
