/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include "ccm_local_event_manager/CCM_ClientEvents_impl.hpp"

#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>


//
// IDL:thalesgroup.com/CcmLocalEventManagerTest/CCM_ClientEvents:1.0
//
CcmLocalEventManagerTest::CCM_ClientEvents_impl::CCM_ClientEvents_impl()
     : m_session_context(CcmLocalEventManagerTest::CCM_ClientEvents_Context::_nil())
{
}

CcmLocalEventManagerTest::CCM_ClientEvents_impl::~CCM_ClientEvents_impl()
{
    std::cout << "   ... CCM_ClientEvents_impl::~CCM_ClientEvents_impl called!" << std::endl;
}

//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
CcmLocalEventManagerTest::CCM_ClientEvents_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_ClientEvents_impl::Set_session_context called!" << std::endl;
   
    m_session_context = ::CcmLocalEventManagerTest::CCM_ClientEvents_Context::_narrow(ctx); 
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CcmLocalEventManagerTest::CCM_ClientEvents_impl::ccm_activate()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_ClientEvents_impl::Ccm_activate called!" << std::endl;
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
CcmLocalEventManagerTest::CCM_ClientEvents_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_ClientEvents_impl::Ccm_passivate called!" << std::endl;
} 



//
// ccm_remove function : it destroys the component (call by the container)
//
void
CcmLocalEventManagerTest::CCM_ClientEvents_impl::ccm_remove()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_ClientEvents_impl::Ccm_remove called!" << std::endl;
}

void
CcmLocalEventManagerTest::CCM_ClientEvents_impl::push_eventServ1Consumes (CcmLocalEventManagerTest::EventTest * event)
        throw (CORBA::SystemException)
{
    static int event_counter = 0;
    
    event_counter++;

    std::cout << "ClientCPP: EventTest #"  << event->numOfEvent() << " received"
          << std::endl;
    
    ::CcmLocalEventManagerTest::HostInfo_var server_facet;

     try {
       server_facet = m_session_context->get_connection_server1HostInfo();
       
       if (CORBA::is_nil(server_facet.in()) != 0)
     {
      CDMW_ASSERT(false);
     }
       
       //get the connection to the server 
       server_facet->display_info( Cdmw::OsSupport::OS::get_hostname().c_str(), 
                   event->numOfEvent() );

    } catch (const CORBA::SystemException& e) {
        TEST_INFO("Unexpected CORBA::SystemException: " << e);
    CDMW_ASSERT(false);
    }
}

void
CcmLocalEventManagerTest::CCM_ClientEvents_impl::push_eventServ2Consumes (CcmLocalEventManagerTest::SecondEventTest * event)
        throw (CORBA::SystemException)
{
    static int event_counter = 0;
    
    event_counter++;

    std::cout << "ClientCPP: SecondEventTest #"  << event->numOfEvent() << " received"
          << std::endl;
    
    ::CcmLocalEventManagerTest::HostInfo_var server_facet;

     try {
       server_facet = m_session_context->get_connection_server2HostInfo();
       
       if (CORBA::is_nil(server_facet.in()) != 0)
     {
      CDMW_ASSERT(false);
     }
       
       //get the connection to the server 
       server_facet->display_info( Cdmw::OsSupport::OS::get_hostname().c_str(), 
                   event->numOfEvent() );

    } catch (const CORBA::SystemException& e) {
        TEST_INFO("Unexpected CORBA::SystemException: " << e);
    CDMW_ASSERT(false);
    }
}






