/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef _CDMW_PIQ_CCM_LOGGING_FATALCLIENTEXECUTOR_IMPL_HPP_
#define _CDMW_PIQ_CCM_LOGGING_FATALCLIENTEXECUTOR_IMPL_HPP_


#include <Foundation/logging/Logger.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/osthreads/ThreadHandle.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include "Foundation/ossupport/Signals.hpp"
#include "Foundation/ossupport/SignalHandler.hpp"

#include "UserHello.skel.hpp"


namespace Hello
{


    class FatalClientExecutor_impl;
    

    class CcmLoggingTester : public CppUnit::TestFixture
    {
        FatalClientExecutor_impl * m_user_ccm_client;
        CCM_Client_Context_var m_session_context;

    public:
        CcmLoggingTester()
        {
            CORBA::Object_var obj1 = 
                Cdmw::TestUtils::CORBATestManager::instance()->get_object(
                    "client_context");
            m_session_context =
                CCM_Client_Context::_narrow(obj1.in());
        };


        ~CcmLoggingTester()
        {
        };

        //
        void do_tests();

        void tearDown()
        {
            //TEST_INFO("tearDown()");
            m_session_context = CCM_Client_Context::_nil();
        }

    protected:

        CPPUNIT_TEST_SUITE( CcmLoggingTester );
        CPPUNIT_TEST( do_tests );
        CPPUNIT_TEST_SUITE_END();

    };


    class TestThread : public Cdmw::OsSupport::Thread 
    {
    public:
        TestThread()
        {
        };

    protected:
        void run() throw()
        {
            Cdmw::TestUtils::CORBATestManager* testManager =
                Cdmw::TestUtils::CORBATestManager::instance();
            Cdmw::TestUtils::ThreadedTestRunner runner(
                testManager->get_controller());
            runner.addTest(
                CppUnit::TestFactoryRegistry::getRegistry(
                    "CcmLoggingTester").makeTest());
            runner.start();
            runner.join();
        };

    private:
        TestThread(const TestThread&);
        TestThread& operator = (const TestThread&);
    };


    class EventHandler : public Cdmw::OsSupport::SignalHandler
    {
    public:
        EventHandler()
            : 
            m_call(false)
        {
        }
    
        virtual ~EventHandler() throw () {}
    
        int handle_signal(int sig) throw ();    

        bool handler_called();
    
    private:
        bool m_call;
    };


    /**
     * IDL:thalesgroup.com/Hello/CCM_Client:1.0
     */
    class FatalClientExecutor_impl
            : virtual public User_Client,
              virtual public Cdmw::OrbSupport::RefCountLocalObject
    {
        public:
            FatalClientExecutor_impl();

            ~FatalClientExecutor_impl();

            void
            set_session_context(Components::SessionContext_ptr ctx)
                throw(CORBA::SystemException);

            void
            ccm_activate()
                throw(CORBA::SystemException);

            void
            ccm_passivate()
                throw(CORBA::SystemException);

            void
            ccm_remove()
                throw(CORBA::SystemException);

            CCM_Client_Context* get_context()
            {
                return CCM_Client_Context::_duplicate(m_session_context.in());
            }


        private:
            FatalClientExecutor_impl(const FatalClientExecutor_impl&);

            void
            operator=(const FatalClientExecutor_impl&);


        private:
            CCM_Client_Context_var m_session_context;
            Cdmw::Logging::Logger* m_logger;
            TestThread m_testThread;

        friend class CcmLoggingTester;
    };
} // namespace Hello


#endif // _CDMW_PIQ_CCM_LOGGING_FATALCLIENTEXECUTOR_IMPL_HPP_
