/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <Foundation/logging/LogManager.hpp>
#include <Foundation/ossupport/OS.hpp>

#include "ccm_logging/FatalClientExecutor_impl.hpp"


namespace Hello
{

/**
 * IDL:thalesgroup.com/Hello/CCM_Client:1.0
 */


/**
 * Constructor.
 */
FatalClientExecutor_impl::FatalClientExecutor_impl()
    : m_session_context(CCM_Client_Context::_nil()),
      m_logger(Cdmw::Logging::Logger::Get_Logger("CCM_Client",
                                                 "QUALIF_CCM_LOGGING")),
      m_testThread()
{
}


/**
 * Destructor.
 */
FatalClientExecutor_impl::~FatalClientExecutor_impl()
{
    CDMW_INFO(m_logger, "FatalClientExecutor_impl::~FatalClientExecutor_impl() called.");
}


/**
 * Set the session context of the component (called by the container).
 */
void
FatalClientExecutor_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "FatalClientExecutor_impl::set_session_context() called.");

    m_session_context = CCM_Client_Context::_narrow(ctx);
}


/**
 * Activate the component (called by the container).
 */
void
FatalClientExecutor_impl::ccm_activate()
   throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "FatalClientExecutor_impl::ccm_activate() called.");

     Cdmw::TestUtils::CORBATestManager::instance()->add(
         "client_context",
         get_context()
         );
     Cdmw::TestUtils::CORBATestManager::instance()->add(
         "user_ccm_client",
         static_cast< User_Client* >(this)
         );
     m_testThread.start();
}


/**
 * Stop the component (called by the container).
 */
void
FatalClientExecutor_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "FatalClientExecutor_impl::ccm_passivate() called.");
}


/**
 * Destroy the component (called by the container).
 */
void
FatalClientExecutor_impl::ccm_remove()
   throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "FatalClientExecutor_impl::ccm_remove() called.");
}


//
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(\
    CcmLoggingTester,\
    "CcmLoggingTester"\
    );

void
CcmLoggingTester::do_tests()
{
    CORBA::Object_var obj = 
        Cdmw::TestUtils::CORBATestManager::instance()->get_object(
            "user_ccm_client");
    User_Client_var user_ccm_client = User_Client::_narrow(obj.in());
    m_user_ccm_client =
        reinterpret_cast< FatalClientExecutor_impl* >(
            user_ccm_client.in());

    EventHandler* eh = new EventHandler();
    Cdmw::OsSupport::Signals::Instance()->register_handler(15, eh);

    try
    {
        Cdmw::OsSupport::OS::sleep(2000);

        // get the display connection of the server.
        CDMW_INFO(m_user_ccm_client->m_logger, "connect to the display facet...");

        Display_var display =
            m_session_context->get_connection_client_receptacle();

        // call the server display.
        CDMW_INFO(m_user_ccm_client->m_logger, "call the server display...");
        display->display_hello();

        TEST_INFO("Check that a fatal log makes the supervision kill "
                  "the calling process");
        CDMW_FATAL(m_user_ccm_client->m_logger, "Fatal error detected!");

        Cdmw::OsSupport::OS::sleep(2000);
        CPPUNIT_ASSERT_MESSAGE("Signal handler received no kill signal",
                               eh->handler_called());
    }

    catch (const std::exception& e)
    {
        // catch any CPPUnit exception, print it and throw it
        TEST_INFO(e.what());
        throw e;
    }

    // print test summary
    Cdmw::TestUtils::CORBATestManager::instance()->summary();
}

int
EventHandler::handle_signal(int sig) throw ()
{
    m_call = true;
    std::cout << "Process killed with signal 15" << std::endl;

    return sig;
}

bool
EventHandler::handler_called()
{
    return m_call;
}

}; // End of namespace Hello
