/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "ccm_perf_invocations/CCM_Client_impl.hpp"

#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "CCMContainer/ccmcommon/CCMUtils.hpp"

#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/ui/text/TestRunner.h>
#include <cppunit/TestResult.h>
#include <cppunit/XmlOutputter.h>
#include <cppunit/TextOutputter.h>

namespace 
{

  /**
   *  This class provides an object to test the ccm_activate.
   *
   **/

  class test_ccm_client_activate;
    
  CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(test_ccm_client_activate, "test_ccm_client_activate");

  class test_ccm_client_activate : public CppUnit::TestFixture
  {
    TestPerf::CCM_Client_impl* m_client;
  
  public:
    test_ccm_client_activate()
    {
    };


    ~test_ccm_client_activate() {};
  
    void do_tests()
    {

      TEST_INFO("********************************************");
      TEST_INFO("            ccm_activate start");
      TEST_INFO("********************************************");
     

     ::TestPerf::Ping_var server_facet;

      double local_elapsed_micro_sec = 0; 
      double distant_elapsed_micro_sec = 0; 

      CORBA::Object_var obj1 = Cdmw::TestUtils::CORBATestManager::instance()->get_object("client_context");
      ::TestPerf::CCM_Client_Context_var client_context = ::TestPerf::CCM_Client_Context::_narrow(obj1.in());
      
      CPPUNIT_ASSERT(!CORBA::is_nil(client_context.in()));

      TEST_INFO("Get_connection_local_ping...");
      
      bool test_ok = false;
      try
        {
          server_facet = client_context->get_connection_local_ping();

          if (!CORBA::is_nil(server_facet.in()))
            {
              test_ok = true;
            }
        }
      catch (...)
        {
          TEST_INFO("Unexpected exception raised!");
        }
      CPPUNIT_ASSERT(test_ok);

      TestPerf::CCM_Client_impl::set_nb_invocation(1000);

      unsigned int nb_invocation =
        TestPerf::CCM_Client_impl::get_nb_invocation();

      //get the connection to the server 
      TEST_INFO("Connect to the local server...");

      test_ok = false;
      try
        {
          if (!CORBA::is_nil(server_facet.in()))
            {

              struct Cdmw::OsSupport::OS::Timeval timeval_start;
              struct Cdmw::OsSupport::OS::Timeval timeval_end;

              TEST_INFO("Perform " << nb_invocation 
                        << " invocations to local server...");
              for(unsigned int i = 0; i < nb_invocation; i++){
                timeval_start = Cdmw::OsSupport::OS::get_time();
                server_facet->synchronous_ping();
                timeval_end = Cdmw::OsSupport::OS::get_time();
                
                local_elapsed_micro_sec += 
                    (timeval_end.seconds - timeval_start.seconds) * 1000000 +
                    (timeval_end.microseconds - timeval_start.microseconds);
              }
              TEST_INFO("Invocations to local server done.");
              test_ok = true;
            }
        }
      catch (const CORBA::Exception& e)
        {
          TEST_INFO("Exception raised : " 
                    << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        }
      catch (...)
        {
          TEST_INFO("Unexpected exception raised!");
        }
      CPPUNIT_ASSERT(test_ok);

      /*
       * Test of the specific receptacle
       */

      TEST_INFO("Get_connection_distant_ping...");
      test_ok = false;
      try
        {
          server_facet = client_context->get_connection_distant_ping();

          if (!CORBA::is_nil(server_facet.in()))
            {
              test_ok = true;
            }
        }
      catch (const CORBA::Exception& e)
        {
          TEST_INFO("Exception raised : " 
                    << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        }
       catch (...)
        {
          TEST_INFO("Unexpected exception raised!");
        }
      CPPUNIT_ASSERT(test_ok);

      //get the connection to the server 
      TEST_INFO("Connect to the distant server");
      test_ok = false;
      try
        {
          if (!CORBA::is_nil(server_facet.in()))
            {

              struct Cdmw::OsSupport::OS::Timeval timeval_start;
              struct Cdmw::OsSupport::OS::Timeval timeval_end;

              TEST_INFO("Perform " << nb_invocation 
                        << " invocations to distant server...");
              for(unsigned int i = 0; i < nb_invocation; i++){
                timeval_start = Cdmw::OsSupport::OS::get_time();
                server_facet->synchronous_ping();
                timeval_end = Cdmw::OsSupport::OS::get_time();
                
                distant_elapsed_micro_sec += 
                    (timeval_end.seconds - timeval_start.seconds) * 1000000 +
                    (timeval_end.microseconds - timeval_start.microseconds);
              }
              TEST_INFO("Invocations to distant server done.");
              test_ok = true;
            }
        }
      catch (const CORBA::Exception& e)
        {
          TEST_INFO("Exception raised : " 
                    << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        }
      catch (...)
        {
          TEST_INFO("Unexpected exception raised!");
        }
      CPPUNIT_ASSERT(test_ok);

      test_ok = false;

      std::cout << "Time spent for " << nb_invocation << " local invocations:"
                << std::endl;
      std::cout << local_elapsed_micro_sec << " microseconds" << std::endl;
      std::cout << "Average time for a local invocation = "
                << local_elapsed_micro_sec / nb_invocation
                << " microsecond(s)" << std::endl;
      
      std::cout << "Time spent for " << nb_invocation << " distant invocations:"
                << std::endl;
      std::cout << distant_elapsed_micro_sec << " microseconds" << std::endl;
      std::cout << "Average time for a distant invocation = "
                << distant_elapsed_micro_sec / nb_invocation
                << " microsecond(s)" << std::endl;
      
      double ratio = (local_elapsed_micro_sec*100)/distant_elapsed_micro_sec;

      std::cout << "The ratio (percentage) between the two average elapsed times = "
                << ratio << "%" << std::endl;

      if( ratio < 20 ) test_ok = true;

      CPPUNIT_ASSERT(test_ok);
      
      TEST_INFO("********************************************");
      TEST_INFO("            ccm_activate stop");
      TEST_INFO("********************************************");
      
    };

      CPPUNIT_TEST_SUITE( test_ccm_client_activate );
      CPPUNIT_TEST( do_tests );
      CPPUNIT_TEST_SUITE_END();

  }; // End of class test_ccm_client_activate
    
} // End of anonymous namespace


//
// IDL:thalesgroup.com/TestPerf/CCM_Client:1.0
//
TestPerf::CCM_Client_impl::CCM_Client_impl()
  : m_session_context(TestPerf::CCM_Client_Context::_nil())
{
}

TestPerf::CCM_Client_impl::~CCM_Client_impl()
{
  std::cout << "CCM_Client_impl::~CCM_Client_impl called!" << std::endl;
}

//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
TestPerf::CCM_Client_impl::set_session_context(Components::SessionContext_ptr ctx)
  throw(CORBA::SystemException)
{
  std::cout << "Set_session_context called!" << std::endl;
   
  m_session_context = ::TestPerf::CCM_Client_Context::_narrow(ctx); 
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
TestPerf::CCM_Client_impl::ccm_activate()
  throw(CORBA::SystemException)
{

  Cdmw::TestUtils::CORBATestManager::instance()->add("client_context", get_context());
  //  Cdmw::TestUtils::CORBATestManager::instance()->set_property ("text",get_text());

  //  ::CommonDemo::Registration_var register_server;
  //     register_server = m_session_context->get_connection_to_register();
  //     CORBA::String_var client_name = register_server->_cxx_register();
  //     set_name(client_name.in());

  Cdmw::TestUtils::ThreadedTestRunner runner(Cdmw::TestUtils::CORBATestManager::instance()->get_controller());
  runner.addTest( CppUnit::TestFactoryRegistry::getRegistry("test_ccm_client_activate").makeTest() );
  runner.start();
  runner.join();

  sleep(5);
    
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//

void
TestPerf::CCM_Client_impl::ccm_passivate()
  throw(CORBA::SystemException)
{
  std::cout << "Ccm_passivate called!" << std::endl;
} 

unsigned int TestPerf::CCM_Client_impl::m_nb_invocation = 1000;

//
// ccm_remove function : it destroys the component (call by the container)
void
TestPerf::CCM_Client_impl::ccm_remove()
  throw(CORBA::SystemException)
{
  std::cout << "Ccm_remove called!" << std::endl;
}

unsigned int
TestPerf::CCM_Client_impl::get_nb_invocation()
  throw()
{
  return TestPerf::CCM_Client_impl::m_nb_invocation;
}

void
TestPerf::CCM_Client_impl::set_nb_invocation( unsigned int nb_invocation )
  throw()
{
  TestPerf::CCM_Client_impl::m_nb_invocation = nb_invocation;
}
