/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.qualif.ccmperfinvocations;

import org.junit.*;
import org.omg.CORBA.*;

public class TestCCMClientActivate {

    private CCMClientImpl client;

    public TestCCMClientActivate ()
    {
    }

    @Test
    public void do_tests() throws cdmw.testutils.CORBATestManager.InvalidObject {
        System.out.println("********************************************");
        System.out.println("            ccm_activate start");
        System.out.println("********************************************");

        com.acme.TestPerf.Ping server_facet = null;

        double localElapsedMicroSec = 0;
        double distantElapsedMicroSec = 0;

        com.acme.TestPerf.CCM_Client_Context client_context = 
            (com.acme.TestPerf.CCM_Client_Context) 
            cdmw.testutils.CORBATestManager.instance().getLocalProperty("client_context");

        Assert.assertNotNull(client_context);

        System.out.println("Get_connection_local_ping...");

        boolean testOk = false;
        try {
            server_facet = client_context.get_connection_local_ping();

            if (server_facet != null) {
                testOk = true;
            }
        }
        catch (Exception e) {
            System.out.println("Unexpected exception raised!");
        }
        Assert.assertTrue(testOk);

        CCMClientImpl.set_nb_invocation(1000);

        int nb_invocation = CCMClientImpl.get_nb_invocation();

        // get the connection to the server
        System.out.println("Connect to the local server...");

        testOk = false;
        try {
            if (server_facet != null) {
                cdmw.ossupport.Timeval timeval_start;
                cdmw.ossupport.Timeval timeval_end;

                System.out.println("Perform " + nb_invocation
                    + " invocations to local server...");
                for (int i = 0; i < nb_invocation; i++ ) {
                    timeval_start = cdmw.ossupport.OS.getTime();
                    server_facet.synchronous_ping();
                    timeval_end = cdmw.ossupport.OS.getTime();

                    localElapsedMicroSec += (timeval_end.getSeconds() - timeval_start
                        .getSeconds())
                        * 1000000
                        + (timeval_end.getMicroseconds() - timeval_start
                            .getMicroseconds());
                }
                System.out.println("Invocations to local server done.");
                testOk = true;
            }
        }
        catch (SystemException e) {
            System.out.println("Exception raised : "
                + cdmw.orbsupport.ORBSupport.exceptionToString(e));
        }
        catch (Exception e1) {
            System.out.println("Unexpected exception raised!");
        }
        Assert.assertTrue(testOk);

        /*
         * Test of the specific receptacle
         */

        System.out.println("Get_connection_distant_ping...");
        testOk = false;
        try {
            server_facet = client_context.get_connection_distant_ping();

            if (server_facet != null) {
                testOk = true;
            }
        }
        catch (SystemException e) {
            System.out.println("Exception raised : "
                + cdmw.orbsupport.ORBSupport.exceptionToString(e));
        }
        catch (Exception e1) {
            System.out.println("Unexpected exception raised!");
        }
        Assert.assertTrue(testOk);

        // get the connection to the server
        System.out.println("Connect to the distant server");
        testOk = false;
        try {
            if (server_facet != null) {

                cdmw.ossupport.Timeval timeval_start;
                cdmw.ossupport.Timeval timeval_end;

                System.out.println("Perform " + nb_invocation
                    + " invocations to distant server...");
                for (int i = 0; i < nb_invocation; i++ ) {
                    timeval_start = cdmw.ossupport.OS.getTime();
                    server_facet.synchronous_ping();
                    timeval_end = cdmw.ossupport.OS.getTime();

                    distantElapsedMicroSec += (timeval_end.getSeconds() - timeval_start
                        .getSeconds())
                        * 1000000
                        + (timeval_end.getMicroseconds() - timeval_start
                            .getMicroseconds());
                }
                System.out.println("Invocations to distant server done.");
                testOk = true;
            }
        }
        catch (SystemException e) {
            System.out.println("Exception raised : "
                + cdmw.orbsupport.ORBSupport.exceptionToString(e));
        }
        catch (Exception e1) {
            System.out.println("Unexpected exception raised!");
        }
        Assert.assertTrue(testOk);

        testOk = false;

        System.out.println("Time spent for " + nb_invocation
            + " local invocations:");
        System.out.println(localElapsedMicroSec + " microseconds");
        System.out.println("Average time for a local invocation = "
            + localElapsedMicroSec / nb_invocation + " microsecond(s)");

        System.out.println("Time spent for " + nb_invocation
            + " distant invocations:");
        System.out.println(distantElapsedMicroSec + " microseconds");
        System.out.println("Average time for a distant invocation = "
            + distantElapsedMicroSec / nb_invocation + " microsecond(s)");

        double ratio = (localElapsedMicroSec * 100) / distantElapsedMicroSec;

        System.out
            .println("The ratio (percentage) between the two average elapsed times = "
                + ratio + "%");

        if (ratio < 20)
            testOk = true;

        Assert.assertTrue(testOk);

        System.out.println("********************************************");
        System.out.println("            ccm_activate stop");
        System.out.println("********************************************");

    }

}
