/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

// Unix Files
#include <stdlib.h>
#include <iostream>
#include <sstream>


// Cdmw Files
#include <Foundation/common/Assert.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>

#include <SystemMngt/platforminterface/PlatformInterface.hpp>


#include <ccm_philo/CCM_Philosopher_impl.hpp>
#include <ccm_philo/StatusInfo_impl.hpp>


namespace 
{
    
void MySrand ()
{
  static bool initialized = false;

  if (!initialized) 
  {
      Cdmw::OsSupport::OS::Timeval tv 
          = Cdmw::OsSupport::OS::get_time ();
      Cdmw::OsSupport::OS::ProcessId pid 
          = Cdmw::OsSupport::OS::get_processId();
    ::srand ((unsigned int) (tv.microseconds + pid) % 991);
    initialized = true;
  }
}

int MyRand ()
{
  MySrand ();
  return (int) rand ();
}

} // End of anonymous namespace

/**
*  This class provides an object to test the component registration.
*
**/
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(Dinner::test_component_registration, "test_component_registration");

int Dinner::test_ccm_philosopher_activate::M_counter = 0;
int Dinner::PhiloActivationThread::M_counter = 0;
int Dinner::test_component_registration::M_counter = 0;

void
    Dinner::test_component_registration::do_tests()
    {
     bool testOk = false;
     CDMW_ASSERT(component_ref);
     
     TEST_INFO("***********************************************************");
     TEST_INFO("  Component and Facets Registration Test for component");
     TEST_INFO("***********************************************************");

     std::string system_port =  Cdmw::OsSupport::SystemProperties::Get_Property(Cdmw::OrbSupport::SYSTEM_PORT_PROPERTY_NAME);
     CDMW_ASSERT(!system_port.empty());
                
     std::string corbaloc = "corbaloc::";
               corbaloc += "localhost";
               corbaloc += ":";
               corbaloc += system_port;
               corbaloc +="/NameService";


           // hard coded default testId
           std::string testId = "Q-406-1";

               if (!Cdmw::TestUtils::CORBATestManager::instance()->get_property("testId").empty())
         { 
           testId = Cdmw::TestUtils::CORBATestManager::instance()->get_property("testId");
         }

              
              try{
             
                   CORBA::Object_var o = Cdmw::TestUtils::CORBATestManager::instance()->get_ORB()->string_to_object(corbaloc.c_str()) ;
                   CDMW_ASSERT(!CORBA::is_nil(o));
                   CosNaming::NamingContext_var nc = CosNaming::NamingContext::_narrow(o.in());
                   Cdmw::CommonSvcs::Naming::NamingInterface ni(nc.in());
                   
               //checking facets registration
                    Components::FacetDescriptions_var facets=component_ref->get_all_facets();
                   if(facets->length()==0) {
                           testOk=true;
                           TEST_INFO("Philo: No Facets for component "+instance_name+"...");
                   }
                   
               for(unsigned int i=0;i<facets->length();i++){
          std::string name="DeploymentPlans/";
           name+=testId;
                   name+=".DeploymentPlan/";
                   name+=instance_name;
                   name+="/";
                   name+=facets[i]->name();
                   name+=".Port";
                   
                   TEST_INFO("Philo: Looking for facet "+name+" in the Naming Service...");
                   o=ni.resolve(name);
                   
                   
                   if(!CORBA::is_nil(o))
             {
                           std::ostringstream s;
                        s <<"Philo: Facet "<<facets[i]->name()<<" has been registered as "<< name <<"...";
                       TEST_INFO(s.str());
                       testOk=true;
                   }
                   else {
                        std::ostringstream s;
                        s << "Philo: Facet " << facets[i]->name() <<" has not been registered !!!";
                           TEST_INFO(s.str());
                   }
              
               }
               CPPUNIT_ASSERT(testOk);
             

               //checking components registration
                
           std::string name="DeploymentPlans/";
               name+=testId;
               name+=".DeploymentPlan/";
           name+=instance_name;
           name+=".Component";
           
                   
                   TEST_INFO("Philo: looking for Component "+instance_name+" in the Naming Service...");
                   TEST_INFO("Resolving: " + name);
                   o=ni.resolve(name);
                   if(CORBA::is_nil(o)) {
                           TEST_INFO("Philo: "+instance_name+" has not been registered !!!");
                           testOk=false;
                               }
                    else TEST_INFO("Philo: "+instance_name+" has been registred as "+name+"...");

               CPPUNIT_ASSERT(testOk);
               
               }
               catch(const CORBA::Exception& e){
                std::cout << "Exception raised during names resolution : " << e << std::endl;
                testOk = false;   
               }
               

              CPPUNIT_ASSERT(testOk);
    }

/**
*  This class provides an object to test the ccm_activate.
*
**/
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(Dinner::test_ccm_philosopher_activate, "test_ccm_philosopher_activate");

void 
Dinner::test_ccm_philosopher_activate::do_tests()
  {
      CDMW_ASSERT(m_philosopher);

      TEST_INFO("********************************************");
      TEST_INFO("            ccm_activate start");
      TEST_INFO("********************************************");

     ::Dinner::CCM_Philosopher_Context_var philosopher_context = m_philosopher->get_context();
     
      TEST_INFO("Philo: Wait all Forks to be created...");
      bool fork_ready = false;
      while (!fork_ready) {
          try {
              Dinner::Fork_var left = 
                  philosopher_context->get_connection_left();
              Dinner::Fork_var right = 
                  philosopher_context->get_connection_right();
              
              fork_ready = true;
          } catch (const CORBA::TRANSIENT& e) {
              TEST_INFO(" ... fork not yet ready ...");
              Cdmw::OsSupport::OS::sleep(3000);
          }
      }
      
      ::Dinner::Registration_var register_server;
      
     TEST_INFO("Philo: Get_connection_to_register...");
      bool test_ok = false;
      try
      {
          register_server = philosopher_context->get_connection_register();

            if (!CORBA::is_nil(register_server.in()))
            {
                test_ok = true;
            }
      }
      catch (...)
      {
          TEST_INFO("Philo: Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);

      // register philosopher
      TEST_INFO("Philo: Register philosopher...");
     test_ok = false;
      try
      {

          if (!CORBA::is_nil(register_server.in()))
          {
              CORBA::String_var philosopher_name;

              bool isNameSet = false;
              while (!isNameSet) {
                  try {
                      philosopher_name = register_server->_cxx_register();
                      isNameSet = true;
                  }
                  catch (const CORBA::TRANSIENT& e) {
                      std::cout << "Exception raised: " << e << std::endl;
                      TEST_INFO("register_server->_cxx_register not ready yet, retrying...");
                      Cdmw::OsSupport::OS::sleep(3000);
                  }
              }

              m_philosopher->set_speed(philosopher_name.in());
              m_philosopher->set_name(philosopher_name.in());
              std::cout << "Philo: "  << philosopher_name.in() << " is active, speed is " << m_philosopher->m_speed << std::endl;
              test_ok = true;
          }

      }
      catch (const CORBA::SystemException& e)
      {
          std::cout << "System Exception raised: " << e << std::endl;
      }
      catch (const CORBA::Exception& e)
      {
          std::cout << "CORBA Exception raised: " << e << std::endl;
      }
      catch (...)
      {
            TEST_INFO("Philo: Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);

      // push config from emitter port 
      TEST_INFO("Philo: push_config using emitter port...");
     test_ok = false;
      try
      {
            Dinner::StatusInfo_var si = new StatusInfo_impl;
          si->name (m_philosopher->m_name.in());
           si->state (Dinner::THINKING);
          si->secs_since_last_meal (m_philosopher->m_state);
          si->has_left_fork (m_philosopher->m_have_left_fork);
          si->has_right_fork (m_philosopher->m_have_right_fork);

          m_philosopher->m_context->push_config (si.in());
             test_ok = true;
      }
      catch (...)
      {
            TEST_INFO("Philo: Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);

     
     //get the connection to the server 
      TEST_INFO("Philo: Connect to the fork facet...");
      Dinner::Fork_var left;
      Dinner::Fork_var right;
     test_ok = false;
      try
      {
         left = philosopher_context->get_connection_left();
         right = philosopher_context->get_connection_right();

            if ( (!CORBA::is_nil(left.in())) && (!CORBA::is_nil(right.in())) )
            {
                test_ok = true;
            }
            else
            {
              TEST_INFO("Philo: Facet reference is nil!");
            }
      }
      catch (...)
      {
            TEST_INFO("Philo: Unexpected exception raised!");
      }
      CPPUNIT_ASSERT(test_ok);

      // Let philosopher try to eat
      try
      {
         m_philosopher->m_mutex.lock();
         for (int i=0; i<3; i++)
         {
            m_philosopher->m_mutex.unlock();
             int timescale = Cdmw::TestUtils::get_timescale();
             Cdmw::OsSupport::OS::sleep(timescale*m_philosopher->m_speed);

            m_philosopher->m_mutex.lock();
              if (!m_philosopher->m_is_active)
              {
                  // philosopher has been desactivated: get out from loop
                  break;
              }
            m_philosopher->m_mutex.unlock();

             if (m_philosopher->m_have_left_fork && m_philosopher->m_have_right_fork) 
             {
                /*
                 * Eating
                 */
                if (m_philosopher->m_state > 3) 
                {
                    m_philosopher->m_state -= 3;
                    m_philosopher->push_state ();
                    m_philosopher->m_mutex.lock();
                     continue;
                }
                m_philosopher->m_state = 0;

                try 
                {
                    left->release ();
                    m_philosopher->m_have_left_fork = false;
                    right->release ();
                    m_philosopher->m_have_right_fork = false;
                } 
                catch (...) 
                {
                }
                std::cout << "Philo: " << m_philosopher->m_name.in() << " has eaten." << std::endl;
                m_philosopher->push_state ();
                m_philosopher->m_mutex.lock();
                continue;
           }

           m_philosopher->m_state++;

           if (m_philosopher->m_state < 3) 
             {
               std::cout << "Philo: " << m_philosopher->m_name.in() << " is thinking." << std::endl;
                m_philosopher->push_state ();
                m_philosopher->m_mutex.lock();
                continue;
           }

           if (m_philosopher->m_state == 3) 
             {
              std::cout  << "Philo: "<< m_philosopher->m_name.in() << " is becoming hungry." << std::endl;
           }
           else if (m_philosopher->m_state >= 10) 
             {
                 std::cout  << "Philo: "<< m_philosopher->m_name.in() << " is starving." << std::endl;
           }

           if (!m_philosopher->m_have_left_fork) 
             {
                try 
                {
                    left->get ();
                    m_philosopher->m_have_left_fork = true;
                }
                catch (const Dinner::InUse& ) 
                {
                    std::cout  << "Philo: "<< m_philosopher->m_name.in() << " has its left fork used." << std::endl;
                }
                    catch (const CORBA::Exception&)
                    {
                    }

                if (m_philosopher->m_have_left_fork) 
                {
                    std::cout  << "Philo: "<< m_philosopher->m_name.in() << " has got the left fork." << std::endl;
                    m_philosopher->push_state ();
                    m_philosopher->m_mutex.lock();
                    continue;
                }
                else 
                {
                    // std::cout << _name << " did not get the left fork." << std::endl;
                }
           }


           if (!m_philosopher->m_have_right_fork) 
             {
                try 
                {
                    right->get ();
                    m_philosopher->m_have_right_fork = true;
                }
                catch (const Dinner::InUse& ) 
                {
                    std::cout  << "Philo: "<< m_philosopher->m_name.in() << " has its right fork used." << std::endl;
                }
                    catch (const CORBA::Exception&)
                {
                }

                if (m_philosopher->m_have_right_fork) 
                {
                    std::cout  << "Philo: "<< m_philosopher->m_name.in() << " has got the right fork." << std::endl;
                    m_philosopher->push_state ();
                    m_philosopher->m_mutex.lock();
                    continue;
                }
                else 
                {
                    // std::cout << m_philosopher->m_name.in() << " did not get the right fork." << std::endl;
                }
           }

           if (((m_philosopher->m_have_left_fork && !m_philosopher->m_have_right_fork) ||
                (!m_philosopher->m_have_left_fork && m_philosopher->m_have_right_fork)) && ((rand() % 3) == 0)) 
             {
                if (m_philosopher->m_have_left_fork) 
                {
                    try 
                     {
                        left->release ();
                        m_philosopher->m_have_left_fork = false;
                        std::cout  << "Philo: "<< "Deadlock prevention! " << m_philosopher->m_name.in()
                                   << " drops the left fork." << std::endl;
                    }
                    catch (...) 
                     {
                    }
                }
                if (m_philosopher->m_have_right_fork) 
                {
                    try 
                     {
                        right->release ();
                        m_philosopher->m_have_right_fork = false;
                          std::cout  << "Philo: "<< "Deadlock prevention! " << m_philosopher->m_name.in()
                                   << " drops the right fork." << std::endl;
                    }
                    catch (...) 
                     {
                    }
                }
           }

           m_philosopher->push_state ();
            m_philosopher->m_mutex.lock();
       } // End of while (is_active)
        m_philosopher->m_mutex.unlock();
        std::cout << "End of dinner for " << m_philosopher->m_name.in() << std::endl;
      }
      catch (const CORBA::SystemException& e)
      {
          std::cout << "System Exception raised: " << e << std::endl;
      }
      catch (const CORBA::Exception& e)
      {
          std::cout << "CORBA Exception raised: " << e << std::endl;
      }
      

  } // End of do_test()

void Dinner::CCM_Philosopher_impl::set_speed(const char* name){

  const unsigned int number_philosophers = 12; 
  enum {NAME = 0, SPEED = 1};


  m_mutex.lock();

  const char * philosophers[2][number_philosophers] = {
    {"Socrates", "Descartes", "Pascal", "Plato", "Aristotle", "Pythagoras", "Zeno",
     "Heraclitus", "Democritus", "Descartes", "Kant", "Protagoras"},
    {
      "2700" /*Socrates*/, "3600" /*Descartes*/, "1800"/*Pascal*/, "4800"/*Plato*/, 
      "2400" /*Aristotle*/, "3200" /*Pythagoras*/, "1000"/*Zeno*/,"4200" /*Heraclitus*/,
      "1400"/*Democritus*/, "3500"/*Descartes*/, "5100"/*Kant*/, "2200"/*Protagoras*/
    }
      };

  m_speed = 0;
  for(unsigned long i = 0; i < number_philosophers; i++){
    if(!strcmp(philosophers[NAME][i], name)){
    m_speed = atol(philosophers[SPEED][i]);
    break;
    }
  }
  
  if(!m_speed)
    std::cerr << __FILE__ << "Speed of " << name << " " << "speed not found !" << __LINE__ << std::endl;
  m_mutex.unlock();

}
    

//
// IDL:acme.com/Dinner/CCM_Philosopher:1.0
//
Dinner::CCM_Philosopher_impl::CCM_Philosopher_impl()
    : m_name(""),
      m_is_active(false),
      m_state(0),
      m_have_left_fork(false),
      m_have_right_fork(false),
      m_context(Dinner::CCM_Philosopher_Context::_nil())
{
//    m_speed = 1500 + (MyRand() % 13) * 300;
      m_activation_thread = new PhiloActivationThread(this);
}

Dinner::CCM_Philosopher_impl::~CCM_Philosopher_impl()
{
    // ->summary is called in dtor because due to the concurrent nature of this test 
    // calls of summary at the end of each test will yield a unpredictable results (depending on race  2, 3 or  4 succeeded tests are reported) 
    // making it hard to determine whether this test passed or not.
  try {
    Cdmw::TestUtils::CORBATestManager::instance()->summary();
    std::cout << "CCM_Philosopher_impl::~CCM_Philosopher_impl called!" << std::endl;
  } catch (...) {
  }
 
}


//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
Dinner::CCM_Philosopher_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    std::cout  << "Philo: "<< "Set_session_context called!" << std::endl;
   
    m_context = ::Dinner::CCM_Philosopher_Context::_narrow(ctx); 
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
Dinner::CCM_Philosopher_impl::ccm_activate()
   throw(CORBA::SystemException)
{
   if (! m_is_active)
    {
        m_is_active = true;

        Dinner::test_component_registration::M_counter++;

        std::ostringstream component_ref,component_instance_name; 
        component_ref << "ComponentRef" << Dinner::test_component_registration::M_counter; 
        component_instance_name << "ComponentInstanceName" << Dinner::test_component_registration::M_counter;
        
         if (!Cdmw::TestUtils::CORBATestManager::instance()->get_property("testRegistration").empty()){
           try{
                   Components::SessionContext_var ctx = Components::SessionContext::_narrow(m_context.in());

                   CORBA::Object_var obj=ctx->get_CCM_object();
                
                if(CORBA::is_nil(obj)) {
                        std::cout << "ERROR: NULL reference of the component !!!" << std::endl;
                        return;
                        }
                
                Cdmw::TestUtils::CORBATestManager::instance()->set_property(component_ref.str(),
                        Cdmw::TestUtils::CORBATestManager::instance()->get_ORB()->object_to_string(obj.in()) );

                obj=m_context->resolve_service_reference("ManagedComponentContext");
                CdmwPlatformMngt::ManagedElementContexts::ComponentContext_var comp_ctx=CdmwPlatformMngt::ManagedElementContexts::ComponentContext::_narrow(obj.in());
                Cdmw::TestUtils::CORBATestManager::instance()->set_property(component_instance_name.str(),comp_ctx->managed_element_name());
                }
                
                catch(const Components::CCMException &ex){
                std::cout << "Enable to get reference to ManagedComponentContext service: " << ex << std::endl;
                }
                catch(const CORBA::Exception& ex){
                std::cout << "Enable to get the component reference: " << ex << std::endl;
                }
        
        }   
        
        m_activation_thread->start();
    }
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//

void
Dinner::CCM_Philosopher_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
   std::cout  << "Philo: "<< "Ccm_passivate called!" << std::endl;

    m_mutex.lock();
   if (!m_is_active) 
    {
        m_mutex.unlock();
        return;
   }

   m_is_active = false;
    m_mutex.unlock();

    std::cout  << "Philo: "<< m_name.in() << " is inactive." << std::endl;

    try
    {
       m_activation_thread->join();
    }
    catch (const Cdmw::OsSupport::DeadlockException& )
    {
        std::cout  << "Deadlock Exception raised!" << std::endl;
    }
    catch (const Cdmw::InternalErrorException& )
    {
        std::cout  << "InternalError Exception raised!" << std::endl;
    }

    std::cout  << "Philo: "<< m_name.in() << " thread has been joined" << std::endl;

   if (m_have_left_fork) 
    {
        try 
         {
            std::cout  << "Philo: "<< m_name.in() << " release left fork!" << std::endl;
            Dinner::Fork_var left = m_context->get_connection_left ();
              if (!CORBA::is_nil(left.in()))
              {
                left->release ();
                std::cout  << "Philo: "<< m_name.in() << " has released left fork!" << std::endl;
                  m_have_left_fork = false;
              }
        } 
         catch (...) 
         {
                std::cout  << "Philo: "<< m_name.in() << " Exception raised" << std::endl;
        }
   }

   if (m_have_right_fork) 
    {
        try 
         {
            std::cout  << "Philo: "<< m_name.in() << " release right fork!" << std::endl;
            Dinner::Fork_var right = m_context->get_connection_right ();
              if (!CORBA::is_nil(right.in()))
              {
                right->release ();
                std::cout  << "Philo: "<< m_name.in() << " has released right fork!" << std::endl;
                m_have_right_fork = false;
              }
        } 
         catch (...) 
         {
                std::cout  << "Philo: "<< m_name.in() << " Exception raised" << std::endl;
        }
   }

   
} 



//
// ccm_remove function : it destroys the component (call by the container)
void
Dinner::CCM_Philosopher_impl::ccm_remove()
   throw(CORBA::SystemException)
{
   std::cout  << "Philo: "<< "Ccm_remove called!" << std::endl;
}

// push status info event
void 
Dinner::CCM_Philosopher_impl::push_state ()
{
    Dinner::StatusInfo_var si = new StatusInfo_impl;
    si->name (m_name.in());

    if (m_have_left_fork && m_have_right_fork) 
     {
         si->state (Dinner::EATING);
    }
    else if (m_state < 3) 
     {
         si->state (Dinner::THINKING);
    }
    else if (m_state < 10) 
     {
         si->state (Dinner::HUNGRY);
    }
    else if (m_state < 40) 
     {
         si->state (Dinner::STARVING);
    }
    else 
     {
         si->state (Dinner::DEAD);
    }

    si->secs_since_last_meal (m_state);
    si->has_left_fork (m_have_left_fork);
    si->has_right_fork (m_have_right_fork);

    m_context->push_info (si.in());
}


//
// IDL:acme.com/Dinner/CCM_Philosopher_Executor/name:1.0
//
char*
Dinner::CCM_Philosopher_impl::name()
    throw(CORBA::SystemException)
{
    char* name = CORBA::string_dup(m_name);
    return name;
}

