/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <ccm_reconnect/CCM_Client_impl.hpp>
#include <ccm_reconnect/TestEvent_impl.hpp>

#include <Foundation/common/Assert.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>


namespace
{

/**
*  This class provides an object to test the ccm_activate.
*
**/
class test_ccm_client_activate;
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(test_ccm_client_activate, "test_ccm_client_activate");

class test_ccm_client_activate : public CppUnit::TestFixture
{
    ::ReconnectDemo::CCM_Client_impl* m_client_impl;
    ::ReconnectDemo::CCM_Client_Context_var m_context;
    ::ReconnectDemo::Supported_var m_supported;
    ::ReconnectDemo::EventChecking_var m_event_checking;


public:
    test_ccm_client_activate()
    {
    };

    ~test_ccm_client_activate()
    {
    };


    void perform_test(bool success_expected)
    {
        TEST_INFO("Perform calls on Server component");
        if (! success_expected)
            TEST_INFO("exceptions are expected");

        TEST_INFO("Test calling supported interface");
        try {
            m_supported->test_supported();
            CPPUNIT_ASSERT(success_expected);
        } catch (const CORBA::SystemException& e) {
            TEST_INFO("Exception received: "
                      << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            CPPUNIT_ASSERT(!success_expected);
        }

        TEST_INFO("Test sending event");
        try {
            ReconnectDemo::TestEvent_var event = new ReconnectDemo::TestEvent_impl;
            event->name("TEST");
            m_context->push_event_publish(event.in());
            // event sending should always be succesfull
            CPPUNIT_ASSERT(true);

            // Wait event to be received back
            Cdmw::OsSupport::OS::sleep(2000);
        } catch (const CORBA::SystemException& e) {
            TEST_INFO("Exception received: "
                      << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            // event sending should always be succesfull
            CPPUNIT_ASSERT(false);
        }

        TEST_INFO("Test calling facet and test if event was received");
        try {
            bool event_received = m_event_checking->was_event_received();
            if (success_expected) {
                CPPUNIT_ASSERT(event_received);
            } else {
                // if success wasn't expected, SystemException should
                // have been received
                CPPUNIT_ASSERT(false);
            }
        } catch (const CORBA::SystemException& e) {
            TEST_INFO("Exception received: "
                      << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            CPPUNIT_ASSERT(!success_expected);
        }

        // Wait TestEvent to be received back
        Cdmw::OsSupport::OS::sleep(500);

        if (success_expected) {
            TEST_INFO("Check that event has been received back");
            CPPUNIT_ASSERT(m_client_impl->has_event_been_received());
        } else {
            TEST_INFO("Check that event has'nt been received back");
            CPPUNIT_ASSERT(!m_client_impl->has_event_been_received());
        }

        TEST_INFO("Calls on Server component perfomed");
        if (! success_expected)
            TEST_INFO("exceptions were expected");
    }


    void do_tests()
    {
        TEST_INFO("do_tests()");
        
        CORBA::Object_var obj =
            Cdmw::TestUtils::CORBATestManager::instance()->get_object("user_ccm_client");
        ::UserReconnectDemo::User_CCM_Client_var user_ccm_client =
            ::UserReconnectDemo::User_CCM_Client::_narrow(obj.in());
        m_client_impl =
            reinterpret_cast< ReconnectDemo::CCM_Client_impl* > (user_ccm_client.in());

        m_context = m_client_impl->get_context();

        try {
        m_supported = m_context->get_connection_to_supported();
        CPPUNIT_ASSERT(!CORBA::is_nil(m_supported.in()));
    
//      CORBA::PolicyTypeSeq policy_types(1);
//      policy_types.length(1);
//      policy_types[0] = Messaging::RELATIVE_RT_TIMEOUT_POLICY_TYPE;

//      CORBA::PolicyList_var policies =
//          m_supported->_get_policy_overrides(policy_types);
//      if(policies->length() > 0)
//        {
//      Messaging::RelativeRoundtripTimeoutPolicy_var rrtp =
//        Messaging::RelativeRoundtripTimeoutPolicy::_narrow(policies->operator[](0));
//      if(!CORBA::is_nil(rrtp.in())) {
//        TEST_INFO("Policy value : " << rrtp->relative_expiry());
//      }
//      else
//        TEST_INFO("No policy value");
//        }
//      else
//        {
//      TEST_INFO("PolicyList is empty");
//        }

        CORBA::Policy_var policy =
          m_supported->_get_policy(Messaging::RELATIVE_RT_TIMEOUT_POLICY_TYPE);
        if(! CORBA::is_nil(policy.in()))
          {
        Messaging::RelativeRoundtripTimeoutPolicy_var rrtp =
          Messaging::RelativeRoundtripTimeoutPolicy::_narrow(policy.in());
        if(!CORBA::is_nil(rrtp.in())) {
          TEST_INFO("Policy value : " << rrtp->relative_expiry());
        }
        else
          TEST_INFO("No policy value");
          }
        else
          {
        TEST_INFO("Policy is empty");
          }
        } catch (const CORBA::SystemException& e) {
            TEST_INFO("Exception received: "
                      << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            CPPUNIT_ASSERT(false);
        }

        try {
            m_event_checking = m_context->get_connection_to_facet();
            CPPUNIT_ASSERT(!CORBA::is_nil(m_event_checking.in()));
        } catch (const CORBA::SystemException& e) {
            TEST_INFO("Exception received: "
                      << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            CPPUNIT_ASSERT(false);
        }

        // perform test invocations.
        // Server is present, success is expected
        perform_test(true);


        char enter;
        std::cout << "----------------------------------------" << std::endl;
        std::cout << "Terminate the server and press any key and enter ..." << std::endl;
        std::cin >> enter;


        // perform test invocations.
        // Server is no more present, failure is expected
        perform_test(false);


        // wait for server restart
        std::cout << "----------------------------------------" << std::endl;
        std::cout << "Restart the server, rebuild server assembly and press any key and enter ..." << std::endl;
        std::cin >> enter;

        // perform test invocations.
        // Server is present again, success is expected
        perform_test(true);
    };


    void tearDown()
    {
        //TEST_INFO("tearDown()");
        m_context = ::ReconnectDemo::CCM_Client_Context::_nil();
        m_supported = ::ReconnectDemo::Supported::_nil();
        m_event_checking = ::ReconnectDemo::EventChecking::_nil();
    }

    CPPUNIT_TEST_SUITE( test_ccm_client_activate );
    CPPUNIT_TEST( do_tests );
    CPPUNIT_TEST_SUITE_END();
}; // End of class test_ccm_client_activate

} // End of anonymous namespace



//
// IDL:thalesgroup.com/ReconnectDemo/CCM_Client:1.0
//
ReconnectDemo::CCM_Client_impl::CCM_Client_impl()
    : m_session_context(ReconnectDemo::CCM_Client_Context::_nil()),
      m_activationThread(0),
      m_has_event_been_received(false)
{
}

ReconnectDemo::CCM_Client_impl::~CCM_Client_impl()
{
    std::cout << "ReconnectDemo::CCM_Client_impl::~CCM_Client_impl called!" << std::endl;
    m_session_context = ReconnectDemo::CCM_Client_Context::_nil();
}


//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
ReconnectDemo::CCM_Client_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    std::cout << "ReconnectDemo::CCM_Client_impl::set_session_context called!" << std::endl;

    m_session_context = ::ReconnectDemo::CCM_Client_Context::_narrow(ctx);
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
ReconnectDemo::CCM_Client_impl::ccm_activate()
   throw(CORBA::SystemException)
{
    std::cout << "ReconnectDemo::CCM_Client_impl::ccm_activate called!" << std::endl;

    m_activationThread = new ClientActivationThread(this);
    m_activationThread->start();
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//

void
ReconnectDemo::CCM_Client_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
   std::cout << "ReconnectDemo::CCM_Client_impl::ccm_passivate called!" << std::endl;

   if (m_activationThread != 0)
       m_activationThread->join();
}



//
// ccm_remove function : it destroys the component (call by the container)
void
ReconnectDemo::CCM_Client_impl::ccm_remove()
   throw(CORBA::SystemException)
{
    std::cout << "ReconnectDemo::CCM_Client_impl::ccm_remove called!" << std::endl;
}


//
// IDL:thalesgroup.com/ReconnectDemo/CCM_Client/push_event_sink:1.0
//
void
ReconnectDemo::CCM_Client_impl::push_event_sink(ReconnectDemo::TestEvent* event)
    throw(CORBA::SystemException)
{
    m_has_event_been_received = true;
}


bool ReconnectDemo::CCM_Client_impl::has_event_been_received()
{
    bool result = m_has_event_been_received;
    m_has_event_been_received = false;
    return result;
}


