/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccmreconnect;

import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import cdmw.testutils.TestManager;

import com.acme.ReconnectDemo.*;



public class ClientActivationThread
{
    public static final String CLIENT_EXECUTOR_IMPL
        = "cdmw.ccmreconnect.ClientActivationThread";
    
    private ClientExecutorImpl client;
    private CCM_Client_Context context;
    private Supported supported;
    private EventChecking eventChecking;

    public ClientActivationThread ()
    {
    }

    @Before
    public void init() {
        TestManager testManager = TestManager.getLocalInstance();
        
        this.client
            = (ClientExecutorImpl)testManager.getLocalProperty(CLIENT_EXECUTOR_IMPL);
        this.context = null;
        this.supported = null;
        this.eventChecking = null;        
    }

    @Test
    public void doTests()
    {
        context = client.getContext();
        
        try {
            supported = context.get_connection_to_supported();
            assertTrue(supported != null);
        } catch (org.omg.CORBA.SystemException e) {
            e.printStackTrace();
            fail("Exception received: " + e);
        }
        
        try {
            eventChecking = context.get_connection_to_facet();
            assertTrue(eventChecking != null);
        } catch (org.omg.CORBA.SystemException e) {
            e.printStackTrace();
            fail("Exception received: " + e);
        }
        
        java.io.BufferedReader inputReader = 
            new java.io.BufferedReader(
                new java.io.InputStreamReader(System.in));
        
        // perform test invocations. 
        // Server is present, success is expected
        performTest(true);
        
        
        System.out.println("----------------------------------------");
        System.out.println("Terminate the server and press Enter ...");
        System.out.println("");
        try {
            inputReader.readLine();
        } catch (java.io.IOException e) {
            e.printStackTrace();
            fail(e.getMessage());
        }
        
        // perform test invocations. 
        // Server is no more present, failure is expected
        performTest(false);
        
        
        // wait for server restart
        System.out.println("----------------------------------------");
        System.out.println("Restart the server, rebuild server assembly and press Enter ...");
        System.out.println("");
        try {
            inputReader.readLine();
        } catch (java.io.IOException e) {
            e.printStackTrace();
            fail(e.getMessage());
        }

        // perform test invocations. 
        // Server is present again, success is expected
        performTest(true);
    }
    
    
    private void performTest(boolean successExpected)
    {
        System.out.println("Perform calls on Server component");
        if (! successExpected)
            System.out.println("exceptions are expected");
        
        System.out.println("Test calling supported interface");
        try {
            supported.test_supported();
            assertTrue(successExpected);
        } catch (org.omg.CORBA.SystemException e) {
            System.out.println("Exception received: " + e);
            assertTrue(!successExpected);
        }
        
        System.out.println("Test sending event");
        try {
            TestEvent event = new TestEventImpl();
            event.name = "TEST";
            context.push_event_publish(event);
            // event sending should always be succesfull

            // Wait event to be received back
            try {
                cdmw.ossupport.OS.sleep(2000);
            } catch (InterruptedException e) {
                // ignore
            }
        } catch (org.omg.CORBA.SystemException e) {
            // event sending should always be succesfull
            fail("Exception received: " + e);
        }
        
        System.out.println("Test calling facet and test if event was received");
        try {
            boolean eventReceived = eventChecking.was_event_received();
            if (successExpected) {
                assertTrue(eventReceived);
            } else {
                // if success wasn't expected, SystemException should 
                // have been received
                fail(); 
            }
        } catch (org.omg.CORBA.SystemException e) {
            System.out.println("Exception received: " + e);
            assertTrue(!successExpected);
        }

        // Wait TestEvent to be received back
        try {
            cdmw.ossupport.OS.sleep(1000);
        } catch (InterruptedException e) {
            // ignore
        }
        
        if (successExpected) {
            System.out.println("Check that event has been received back");
            assertTrue(this.client.hasEventBeenReceived());
        } else {
            System.out.println("Check that event has'nt been received back");
            assertTrue(!this.client.hasEventBeenReceived());
        }
        
        System.out.println("Calls on Server component perfomed");
        if (! successExpected)
            System.out.println("exceptions were expected");
    }
}
