/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef ___CCM_TEST_CCM_RECORDING_CCM_CLIENT_IMPL_HPP__
#define ___CCM_TEST_CCM_RECORDING_CCM_CLIENT_IMPL_HPP__

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/osthreads/Mutex.hpp>

#include <ccm_recording/TestCcmRecordingLocal.skel.hpp>
#include <TraceAndPerf/recordingidl/CdmwRecordingData.stub.hpp>

#include <vector>


namespace TestCcmRecording
{

class CCM_Client_impl;
    

class test_ccm_recording : public CppUnit::TestFixture
{
    CCM_Client_impl * m_ccm_client;
    ::TestCcmRecording::CCM_Client_Context_var m_session_context;
    
public:
    test_ccm_recording();

    //
    //
    void do_tests();

    void tearDown()
    {
        //TEST_INFO("tearDown()");
        m_session_context = ::TestCcmRecording::CCM_Client_Context::_nil();
    }
    
    // register test suite and test operation
    CPPUNIT_TEST_SUITE( test_ccm_recording );
    CPPUNIT_TEST( do_tests );
    CPPUNIT_TEST_SUITE_END();

private:

    // find in record list a "test_operation" request 
    // from server_id containing specified component_name
    // and with specified message
    bool find_test_operation_record(const char* component_name,
                                    const char* message);
    
    // find in record list a "push_event" request 
    // from server_id containing specified component_name
    // and with specified message
    bool find_push_event_record(const char* component_name,
                                const char* message);
};
    
    
    
//
// IDL:thalesgroup.com/TestCcmRecording/Session_CCM_Client:1.0
//
class CCM_Client_impl : virtual public ::TestCcmRecording::Session_CCM_Client,
                        virtual public Cdmw::OrbSupport::RefCountLocalObject
{
    CCM_Client_impl(const CCM_Client_impl&);
    void operator=(const CCM_Client_impl&);


    ::TestCcmRecording::CCM_Client_Context_var m_session_context;
     
public:

    CCM_Client_impl();
    ~CCM_Client_impl();
     

    //
    // Session component specific operations
    //
    void set_session_context(Components::SessionContext_ptr ctx)
        throw(CORBA::SystemException);
    void ccm_activate()
        throw(CORBA::SystemException);
    void ccm_passivate()
        throw(CORBA::SystemException);
    void ccm_remove()
        throw(CORBA::SystemException);


    //
    // MyRecorder supported interface operations
    //
    void record(const CdmwRecording::GroupHeader& header,
                const CdmwRecording::GIOP_Buffers& data)
        throw(CORBA::SystemException);
    
    void my_record(const char* record)
        throw(CORBA::SystemException);
    

private:

    ::TestCcmRecording::CCM_Client_Context* get_context();
    
    Cdmw::TestUtils::ThreadedTestRunner* m_test_thread;
    
    typedef std::vector<CdmwRecording::RequestReplyPair_var> RecordList;
    RecordList m_record_list;
    
    Cdmw::OsSupport::Mutex                   m_mutex;
    
    bool m_my_record_has_been_called;

    // True if configuration_complete has been received by the recorder
    // otherwise false. 
    // configuration_complete is not recorded into recordList because
    // recordedList is cleaned at the beginning of do_tests(). A boolean is
    // enough to know if configuration_complete has been recorded.
    bool m_configuration_complete_has_been_recorded;
    
    friend class test_ccm_recording;
};

}; // End of namespace TestCcmRecording


#endif // ___CCM_TEST_CCM_RECORDING_CCM_CLIENT_IMPL_HPP__

