/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/logging/FunctionLogger.hpp>
#include <Time/idllib/CosClockService.stub.hpp>
#include <Time/clockservice/Util.hpp>
#include <SystemMngt/platforminterface/PlatformInterface.hpp>

#include "ccm_time/DisplayFacetExecutor_impl.hpp"

#include <iostream>

namespace CcmTime
{
    /**
     * IDL:thalesgroup.com/CcmTime/CCM_Display:1.0
     */


    /**
     * Default constructor.
     */
    DisplayFacetExecutor_impl::DisplayFacetExecutor_impl(
        ServerExecutor_impl *server
        )
        : 
        m_logger(Cdmw::Logging::Logger::Get_Logger("CCM_Display",
                                                   "QUALIF_CCM_TIME")),
        m_server(server)
    {
    }


    /**
     * Destructor.
     */
    DisplayFacetExecutor_impl::~DisplayFacetExecutor_impl()
    {
        CDMW_INFO(m_logger, "DisplayFacetExecutor_impl::~DisplayFacetExecutor_impl() called.");
    }


    /**
     * IDL:thalesgroup.com/CcmTime/Display/print:1.0
     */
    void
    DisplayFacetExecutor_impl::display_hello(timingData_out outputData)
        throw(CORBA::SystemException)
    {
        CDMW_INFO(m_logger, "DisplayFacetExecutor_impl::display_hello() called.");

        using namespace std;

        // get process name 
        CORBA::String_var Process =
            Cdmw::PlatformMngt::PlatformInterface::Get_process_name();

        // inform supervision  
        Cdmw::PlatformMngt::PlatformInterface::Notify_message(
            CdmwPlatformMngtBase::INF, Process, "!!!! in display_hello !!!!");

        // get date
        /************/
        // ask server the ClockCatalog
        CosClockService::ClockCatalog* catalog = m_server->get_clock_catalog();
        // get catalag entry named "LocalExecutor"
        CosClockService::ClockCatalog::ClockEntry_var clockEntry =
            catalog->get_entry("LocalExecutor");
        // get associated clock
        CosClockService::Clock_var clock = clockEntry->subject;
        // ask it date
        TimeBase::TimeT now = clock->current_time();

        cout << endl << "<------------------------------>"
             << endl << "<### Hello! (time returned) ###>"
             << endl << "<------------------------------>" << endl << endl;

        // fill output data
        outputData = new timingData();
        outputData->server = Process.in();
        outputData->time = now;
        outputData->operational_time = now; 
    }

}; // namespace CcmTime
