/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "ccm_time/DisplayFacetExecutor_impl.hpp"
#include "ccm_time/ServerExecutor_impl.hpp"
#include <iostream>

namespace CcmTime
{
    /**
     * IDL:thalesgroup.com/CcmTime/CCM_Server:1.0
     */


    /**
     * Default constructor.
     */
    ServerExecutor_impl::ServerExecutor_impl()
        : m_display(new DisplayFacetExecutor_impl(this)),
          m_logger(Cdmw::Logging::Logger::Get_Logger("CCM_Server",
                                                     "QUALIF_CCM_TIME"))
    {
    }

    /**
     * Destructor.
     */
    ServerExecutor_impl::~ServerExecutor_impl()
    {
        CDMW_INFO(m_logger,
                  "ServerExecutor_impl::~ServerExecutor_impl() called.");
    }


    /**
     * IDL:thalesgroup.com/CcmTime/CCM_Server/get_server_facet:1.0
     */
    CCM_Display_ptr
    ServerExecutor_impl::get_server_facet()
        throw(CORBA::SystemException)
    {
        CDMW_INFO(m_logger, "ServerExecutor_impl::get_server_facet() called.");

        return CCM_Display::_duplicate(m_display.in());
    }


    /** 
     * Get access to time service.
     */
    CosClockService::ClockCatalog* ServerExecutor_impl::get_clock_catalog()
    {
        CDMW_INFO(m_logger,"ServerExecutor_impl::get_clock_catalog() called.");

        return CosClockService::ClockCatalog::_duplicate(m_clock_catalog.in());
    }


   CORBA::Boolean ServerExecutor_impl::do_work(const CORBA::Any& args)
      throw (CORBA::SystemException)
   {
       return true;
   }

    /**
     * Set the session context of the component (called by the container).
     */
    void
    ServerExecutor_impl::set_session_context(
        Components::SessionContext_ptr ctx
        )
        throw(CORBA::SystemException)
    {
        CDMW_INFO(m_logger,
                  "ServerExecutor_impl::set_session_context() called.");
        
        // get context
        m_session_context = CCM_Server_Context::_narrow(ctx);

        // get context for services
        m_session_context_ext = 
            CdmwComponents::SessionContextExt::_narrow(m_session_context.in());

        // get Time Service
        m_clock_catalog = 
            CosClockService::ClockCatalog::_narrow(
                m_session_context_ext->resolve_service_reference("ClockService"));
    }


    /**
     * Activate the component (called by the container).
     */
    void
    ServerExecutor_impl::ccm_activate()
        throw(CORBA::SystemException)
    {
        CDMW_INFO(m_logger, "ServerExecutor_impl::ccm_activate() called.");
    }


    /**
     * Stop the component (called by the container).
     */
    void
    ServerExecutor_impl::ccm_passivate()
        throw(CORBA::SystemException)
    {
        CDMW_INFO(m_logger, "ServerExecutor_impl::ccm_passivate() called.");
    }


    /**
     * Destroy the component (called by the container).
     */
    void
    ServerExecutor_impl::ccm_remove()
        throw(CORBA::SystemException)
    {
        CDMW_INFO(m_logger, "ServerExecutor_impl::ccm_remove() called.");
    }

}; // End of namespace CcmTime
