/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/common/Exception.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <SystemMngt/platforminterface/PlatformInterface.hpp>
#include <ConfAndPlug/cdmwinit/InitUtils.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Repository/idllib/CdmwNamingAndRepository.stub.hpp>


#include "ccm_unaware/ClientProcessControl.hpp"
#include "ccm_unaware/SimpleInterfaces.stub.hpp"



namespace SimpleCcmServer
{


class test_ccm_unaware;
CPPUNIT_TEST_SUITE_REGISTRATION(test_ccm_unaware);


class test_ccm_unaware : public CppUnit::TestFixture
{

    
public:
    test_ccm_unaware()
    {

    };
    
    ~test_ccm_unaware() {};
    
    
    void do_tests()   
    {
        //     set_nbOfRequestedTestOK(4);    

        CORBA::Object_var obj = Cdmw::TestUtils::CORBATestManager::instance()->get_object("rooot");
        std::string pathToServer = Cdmw::TestUtils::CORBATestManager::instance()->get_property("pathToServer");
        CosNaming::NamingContext_var root = CosNaming::NamingContext::_narrow(obj.in());
        
        Cdmw::CommonSvcs::Naming::NamingInterface ni(root.in());
        
        // 
        // Retrive NamedServer
        //
        TEST_INFO("Retrive NamedServer object from Repository");
        bool namedServerFound = false;
        CORBA::Object_var serverObj;
        while (!namedServerFound) {
            try {
                TEST_INFO("Try to resolve server");
                std::string pathToComponent = pathToServer + std::string(".Component");
                TEST_INFO(pathToComponent);
                serverObj = ni.resolve (pathToComponent);

                namedServerFound = true;
            } catch (const CosNaming::NamingContext::NotFound& e) {
                TEST_INFO("   waiting for ccm deployment to finish...");
                // retry in 3 sec
                Cdmw::OsSupport::OS::sleep(3000);
            }
        }
    
        // get the Server proxy
        SimpleCcmServer::Named_var namedServer = SimpleCcmServer::Named::_narrow(serverObj.in());

        if (CORBA::is_nil(namedServer.in()))
        {
            std::cerr << "File : " << __FILE__ << " Line : " << __LINE__
                      << "could not _narrow object to type Named" << std::endl;
            CPPUNIT_ASSERT(false);
            return;
        }
        CPPUNIT_ASSERT(true);


        // 
        // Retrive NamedServer
        //
        TEST_INFO("Retrive Display object from Repository");
        bool displayFound = false;
        CORBA::Object_var displayObj;
        while (!displayFound) {
            try {
                TEST_INFO("Try to resolve facet");
                std::string pathToFacet = pathToServer + std::string("/for_clients.Port");
                TEST_INFO(pathToFacet);
                displayObj = ni.resolve (pathToFacet);

                displayFound = true;
            } catch (const CosNaming::NamingContext::NotFound& e) {
                TEST_INFO("   waiting for ccm deployment to finish...");
                // retry in 3 sec
                Cdmw::OsSupport::OS::sleep(3000);
            }
        }
    
        // get the Server proxy
        SimpleCcmServer::Display_var display = SimpleCcmServer::Display::_narrow(displayObj.in());

        if (CORBA::is_nil(display.in()))
        {
            std::cerr << "File : " << __FILE__ << " Line : " << __LINE__
                      << "could not _narrow object to type Display" << std::endl;
            CPPUNIT_ASSERT(false);
            return;
        }
        CPPUNIT_ASSERT(true);


        //
        // Test request to Named
        //
        TEST_INFO("Test request to Named object");
        try {
            CORBA::String_var name = namedServer->get_name();
        
            if (strcmp("TestName", name.in()) == 0) {
                CPPUNIT_ASSERT(true);
            } else {
                std::cout << "Name is not the one expected: " 
                          << name.in() << std::endl;           
                CPPUNIT_ASSERT(false);
            }
        
        } catch (const CORBA::Exception& e) {
            std::cerr << "File : " << __FILE__ << " Line : " << __LINE__
                      << "Exception during namedServer->get_name():\n" 
                      << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
                      << std::endl;
            CPPUNIT_ASSERT(false);
        }

        //
        // Test request to Display
        //
        TEST_INFO("Test request to Display object");
        try {
            display->print("Hello World...");
        
            CPPUNIT_ASSERT(true);
        
        } catch (const CORBA::Exception& e) {
            std::cerr << "File : " << __FILE__ << " Line : " << __LINE__
                      << "Exception during display->print():\n" 
                      << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
                      << std::endl;
            CPPUNIT_ASSERT(false);
        }

    
    }


    CPPUNIT_TEST_SUITE( test_ccm_unaware );
    CPPUNIT_TEST( do_tests );
    CPPUNIT_TEST_SUITE_END();
    
};



ClientProcessControl::ClientProcessControl (CORBA::ORB_ptr orb, std::string& serverHost, std::string& pathToServer)
    throw(CORBA::SystemException)
    :  m_orb (CORBA::ORB::_duplicate(orb))
{
    Cdmw::TestUtils::CORBATestManager::instance()->set_property("pathToServer", pathToServer);
    std::string serverRootContext = std::string("corbaname::") + serverHost + std::string("/NameService#DeploymentPlans");
    CORBA::Object_var obj = m_orb->string_to_object(serverRootContext.c_str());
    CosNaming::NamingContext_var root = CosNaming::NamingContext::_narrow(obj.in());
    
    Cdmw::TestUtils::CORBATestManager::instance()->add( "rooot", root.in());

}

ClientProcessControl::~ClientProcessControl()
    throw()
{
}
  


// process initialisation called by platformmngt     
void ClientProcessControl::on_initialise (
        const CdmwPlatformMngtBase::StartupKind& startup_kind)
    throw(CORBA::SystemException)
{
}
  
// process to run called by platformmngt    
void ClientProcessControl::on_run()
    throw(CdmwPlatformMngt::ProcessDelegate::NotReadyToRun, 
          CORBA::SystemException)
{
    Cdmw::TestUtils::CORBATestManager::instance()->run_tests();
    Cdmw::TestUtils::CORBATestManager::instance()->summary();

}


// process to stop called by platformmngt
void ClientProcessControl::on_stop()
    throw(CORBA::SystemException)
{
}



}; // End namespace SimpleCcmServer
