/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.simpleccmserver;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;

import cdmw.orbsupport.StrategyList;

public class Main {

    private static final int SUCCESS = 0;

    private static final int FAILURE = 1;

    private static final int POA_THREAD_POOL_SIZE = 5;

    public static void main(String[] args) {
        int status = SUCCESS;

        ORB orb = null;

        try {
            // Initialize the ORB
            cdmw.orbsupport.StrategyList orbStrategies = new StrategyList();
            orbStrategies.addORBThreaded();
            orbStrategies.addPOAThreadPool(POA_THREAD_POOL_SIZE);

            orb = cdmw.orbsupport.ORBSupport.ORBInit(args, orbStrategies);

            System.out.println(".... Call cdmwInit");
            cdmw.cdmwinit.CdmwInterface.cdmwInit(
                orb, args, null, "cdmw.generated.SimpleCcmServerCdmwInterface");

            System.out.println(".... get RootPOA");
            //
            // Get the root POA
            //
            org.omg.CORBA.Object obj = orb.resolve_initial_references("RootPOA");
            POA rootPOA = POAHelper.narrow(obj);

            System.out.println(".... activate POAMgr");
            //
            // Run implementation (Activate the POA manager)
            //
            POAManager poaManager = rootPOA.the_POAManager();
            poaManager.activate();

            System.out.println(".... ORB run");
            orb.run();
        }
        catch (cdmw.common.Exception ex0) {
            System.err.println(ex0.toString());;
            status = FAILURE;
        }
        catch (SystemException ex1) {
            System.err.println(ex1.toString());;
            status = FAILURE;
        }
        catch (UserException ex2) {
            System.err.println(ex2.toString());;
            status = FAILURE;
        }

        if (orb != null) {
            try {
                cdmw.cdmwinit.CdmwInterface.cdmwCleanup(orb, "cdmw.generated.SimpleCcmServerCdmwInterface");
                cdmw.orbsupport.ORBSupport.ORBCleanup(orb);
                orb.destroy();
            }
            catch (SystemException ex1) {
                System.err.println(ex1.toString());;
                status = FAILURE;
            }
        }
        System.exit(status);
    }
}
