/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.simpleccmserver;

import org.junit.*;
import org.omg.CORBA.*;

public class TestCcmUnaware {

    public TestCcmUnaware ()
    {
    }

    @Test
    public void doTests() {
        try {
            // Get Naming Interface of root name domain
            org.omg.CosNaming.NamingContext root = (org.omg.CosNaming.NamingContext)cdmw.testutils.CORBATestManager.getLocalInstance().getLocalProperty("cdmw.simpleccmserver.rooot");
            cdmw.commonsvcs.naming.NamingInterface ni = new cdmw.commonsvcs.naming.NamingInterface(root);
            
            String pathToServer = cdmw.testutils.CORBATestManager.getLocalInstance().getProperty("cdmw.simpleccmserver.pathToServer");

            // 
            // Retrieve NamedServer
            //
            System.out.println("Retrieve NamedServer object from Repository : " + pathToServer + ".Component");
            boolean namedServerFound = false;
            org.omg.CORBA.Object serverObj = null;
            while (!namedServerFound) {
                try {
                    serverObj = ni.resolve (pathToServer + ".Component");
                    namedServerFound = true;
                } catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
                    System.out.println("   waiting for ccm deployment to finish...");
                    // retry in 3 sec
                    cdmw.ossupport.OS.sleep(3000);
                }
            }
        
            // get the Server proxy
            com.acme.SimpleCcmServer.Named namedServer = com.acme.SimpleCcmServer.NamedHelper.narrow(serverObj);
        
            if (namedServer == null) {
                (new Throwable()).printStackTrace();
                System.err.println("could not _narrow object to type Named");
                Assert.assertTrue(false);
                return;
            }
            
            // 
            // Retrieve NamedServer
            //
            System.out.println("Retrieve Display object from Repository : " + pathToServer + "/for_clients.Port");
            boolean displayFound = false;
            org.omg.CORBA.Object displayObj = null;
            while (!displayFound) {
                try {
                    displayObj = ni.resolve (pathToServer + "/for_clients.Port");
                    displayFound = true;
                } catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
                    System.out.println("   waiting for ccm deployment to finish...");
                    // retry in 3 sec
                    cdmw.ossupport.OS.sleep(3000);
                }
            }
        
            // get the Server proxy
            com.acme.SimpleCcmServer.Display display = com.acme.SimpleCcmServer.DisplayHelper.narrow(displayObj);
        
            if (display == null) {
                (new Throwable()).printStackTrace();
                System.err.println("could not _narrow object to type Display");
                Assert.assertTrue(false);
                return;
            }
        
            //
            // Test request to Named
            //
            System.out.println("Test request to Named object");
            try {
                String name = namedServer.get_name();
            
                if (!("TestName".equals(name))) {
                    System.out.println("Name is not the one expected: " + name);           
                    Assert.assertTrue(false);
                }
            
            } catch (SystemException e) {
                (new Throwable()).printStackTrace();
                System.err.println("Exception during namedServer->get_name():\n" 
                          + cdmw.orbsupport.ORBSupport.exceptionToString(e));
                Assert.assertTrue(false);
            }
            //
            // Test request to Display
            //
            System.out.println("Test request to Display object");
            try {
                display.print("Hello World...");
            
            } catch (SystemException e) {
                (new Throwable()).printStackTrace();
                System.err.println("Exception during display->print():\n" 
                    + cdmw.orbsupport.ORBSupport.exceptionToString(e));
                Assert.assertTrue(false);
            }
        }
        catch (Exception e) {
            Assert.assertTrue(false);
        }
   }
}
