/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef _CDMW_PIQ_CCM_WITH_FEDERATED_CLOCKS_SERVEREXECUTOR_IMPL_HPP_
#define _CDMW_PIQ_CCM_WITH_FEDERATED_CLOCKS_SERVEREXECUTOR_IMPL_HPP_


#include <Foundation/logging/Logger.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Time/idllib/CosClockService.stub.hpp>
#include <Time/clockservice/Util.hpp>
#include <CCMContainer/ccmcif/SessionContextExt_impl.hpp>

#include "UserCcmWithFederatedClocks.skel.hpp"


namespace CcmWithFederatedClocks
{
    /**
     * IDL:thalesgroup.com/CcmWithFederatedClocks/CCM_Server:1.0
     */
    class ServerExecutor_impl
        : virtual public User_Server,
          virtual public Cdmw::OrbSupport::RefCountLocalObject
    {
    public:
        ServerExecutor_impl();
        
        ~ServerExecutor_impl();

        /**
         * IDL:thalesgroup.com/CcmWithFederatedClocks/CCM_Server/get_server_facet:1.0
         */
        virtual CCM_Display_ptr
        get_server_facet()
            throw(CORBA::SystemException);

        // get access to time service
        CosClockService::ClockCatalog* get_clock_catalog();

        //
        // Session component specific operations
        //
        void
        set_session_context(Components::SessionContext_ptr ctx)
            throw(CORBA::SystemException);

        void
        ccm_activate () throw (CORBA::SystemException);

        void
        ccm_passivate () throw (CORBA::SystemException);

        void
        ccm_remove () throw (CORBA::SystemException);

        // implement Periodic interface
        CORBA::Boolean do_work(const CORBA::Any& args)
            throw (CORBA::SystemException);


    private:
        ServerExecutor_impl(const ServerExecutor_impl&);

        void
        operator=(const ServerExecutor_impl&);


    private:
        // basic context
        CCM_Server_Context_var m_session_context;
        // extended context with CDMW services
        CdmwComponents::SessionContextExt_var m_session_context_ext;

        CCM_Display_var m_display;
        Cdmw::Logging::Logger* m_logger;

        // clock service    
        CosClockService::ClockCatalog_var m_clock_catalog;
    };
}; // namespace CcmWithFederatedClocks


#endif // _CDMW_PIQ_CCM_WITH_FEDERATED_CLOCKS_SERVEREXECUTOR_IMPL_HPP_
