/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <Foundation/logging/LogManager.hpp>

#include "ccm_xms/ClientExecutor_impl.hpp"


namespace XMLValidate
{

//
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(XMLValidateTester, "XMLValidateTester");


void
XMLValidateTester::do_tests()
{
  bool res = false;
  try {
    // get the Validator connection of the server.
    TEST_INFO("connect to the validator facet...");
    
    Validator_var validator =
      m_session_context->get_connection_client_receptacle();
    
    // call the server validator.
    TEST_INFO("call the server validator...");
    
    res = validator->parse(m_file_name.in());

  }
  catch (const CORBA::Exception& e) {
    TEST_INFO("unexpected exception raised:" << e);
    res = false;
  }
  CPPUNIT_ASSERT(res);  

  Cdmw::TestUtils::CORBATestManager::instance()->summary();
}


/**
 * IDL:thalesgroup.com/XMLValidate/CCM_Client:1.0
 */


/**
 * Constructor.
 */
ClientExecutor_impl::ClientExecutor_impl()
    : m_session_context(CCM_Client_Context::_nil()),
      m_logger(Cdmw::Logging::Logger::Get_Logger("CCM_Client",
                                                 "QUALIF_CCM_XMS"))
{
}


/**
 * Destructor.
 */
ClientExecutor_impl::~ClientExecutor_impl()
{
    CDMW_INFO(m_logger, "ClientExecutor_impl::~ClientExecutor_impl() called.");
}


/**
 * Set the session context of the component (called by the container).
 */
void
ClientExecutor_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "ClientExecutor_impl::set_session_context() called.");

    m_session_context = CCM_Client_Context::_narrow(ctx);
}


/**
 * Activate the component (called by the container).
 */
void
ClientExecutor_impl::ccm_activate()
   throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "ClientExecutor_impl::ccm_activate() called.");
    Cdmw::TestUtils::CORBATestManager::instance()->add("client_context", m_session_context.in());
    Cdmw::TestUtils::CORBATestManager::instance()->set_property("file_name", m_file_name.in());
    m_testThread.start();
}


/**
 * Stop the component (called by the container).
 */
void
ClientExecutor_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
    
    CDMW_INFO(m_logger, "ClientExecutor_impl::ccm_passivate() called.");
}


/**
 * Destroy the component (called by the container).
 */
void
ClientExecutor_impl::ccm_remove()
   throw(CORBA::SystemException)
{
    CDMW_INFO(m_logger, "ClientExecutor_impl::ccm_remove() called.");
}

char*
ClientExecutor_impl::file_name()
  throw(CORBA::SystemException)
{
    return CORBA::string_dup(m_file_name.in());
}

void
ClientExecutor_impl::file_name (const char* file_name)
  throw(CORBA::SystemException)
{
    m_file_name = CORBA::string_dup(file_name);
}


}; // End of namespace XMLValidate
