/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>

#include <SystemMngt/platforminterface/PlatformInterface.hpp>
#include <XMLServer/idllib/CdmwXML.stub.hpp>
#include "ccm_xms/ValidatorFacetExecutor_impl.hpp"


static const char* XML_SERVICE_NAME = "XMLService";


namespace XMLValidate
{
    /**
     * IDL:thalesgroup.com/XMLValidate/CCM_Validator:1.0
     */


    /**
     * Default constructor.
     */
    ValidatorFacetExecutor_impl::ValidatorFacetExecutor_impl()
        : 
        m_logger(Cdmw::Logging::Logger::Get_Logger("CCM_Validator",
                                                   "QUALIF_CCM_XMS"))
    {
        // get the XML Server reference
        try
        {
            CORBA::Object_ptr obj = 
                Cdmw::PlatformMngt::PlatformInterface::Get_service(
                    XML_SERVICE_NAME);
            m_parser = CdmwXML::XMLParser::_narrow(obj);
        }
        catch (...)
        {
            CDMW_ERROR(m_logger, "Test Failed: XML Service not found.");
        }
    }


    /**
     * Destructor.
     */
    ValidatorFacetExecutor_impl::~ValidatorFacetExecutor_impl()
    {
        CDMW_INFO(m_logger,
                  "ValidatorFacetExecutor_impl::~ValidatorFacetExecutor_impl() called.");
    }


    /**
     * IDL:thalesgroup.com/XMLValidate/Validator/print:1.0
     */
    bool
    ValidatorFacetExecutor_impl::parse(const char* file)
        throw(CORBA::SystemException)
    {
        CDMW_INFO(m_logger,
                  "ValidatorFacetExecutor_impl::parse() called.");

        using namespace std;

        cout << endl << "<-------------------------------------->"
             << endl << "<#### PARSING " << file << " ####>"
             << endl << "<-------------------------------------->" << endl
             << endl;

        bool res = false;
        try
        {
            m_parser->parse(file);
            res = true;
        }
        catch (const CdmwXML::XMLParser::ParsingException& pe) {
            CDMW_ERROR(m_logger, "Exception raised in parsing");
            cout << "<#### Failure in parsing: " << pe.reason
                 << endl << "<-------------------------------------->"
                 << endl;
        }
        catch (const CORBA::SystemException& e)
        {
            CDMW_ERROR(m_logger, "Unexpected CORBA::SystemException: " << e);
        }
        catch (const CORBA::Exception& e)
        {
            CDMW_ERROR(m_logger, "Unexpected CORBA::Exception: " << e);
        }
        catch (...)
        {
            CDMW_ERROR(m_logger, "Unknown exception!");
        }
        cout << endl << "<-------------------------------------->"
             << endl << "<####        END OF PARSING        ####>"
             << endl << "<-------------------------------------->" << endl 
             << endl;


        return res;
    }

}; // namespace XMLValidate
