#!/bin/sh
# =====================================================================
# This file is part of CARDAMOM (R) which is jointly developed by THALES
# and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
# Copyright (C) THALES 2000-2003. All rights reserved.
# Copyright (C) THALES 2004-2005. All rights reserved.
# Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
# All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
# License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================


# Qualification test name
TEST=Q-009

# ORB Name
ORBNAME=OB

# List of files including ORBACUS API calls
RESULTFILE=$TEST.$ORBNAME.txt

# CARDAMOM ROOT Directory
ROOT="../../../../.."
ALLOWEDKEYWORDSFILE=./OB-allowed-keywords.txt

echo " "
echo "Processing data for test $TEST ($ORBNAME)"
echo " "

echo "List of files including references to $ORBNAME API" > $RESULTFILE
echo "=================================================" >> $RESULTFILE
echo " "  >> $RESULTFILE

    KEYWORDS=""

    if [ -s $ALLOWEDKEYWORDSFILE ]
    then
      while read keyword 
      do
        if [ ! -n "$KEYWORDS" ]
        then
          KEYWORDS=$keyword
        else
          KEYWORDS="$keyword|$KEYWORDS"
        fi
      done < $ALLOWEDKEYWORDSFILE
      echo "reading $ALLOWEDKEYWORDSFILE done"
    else
      echo "**WARNING: $ALLOWEDKEYWORDSFILE does not exist"
      # must assign a keyword what does not exist (for grep -v)
      KEYWORDS="714832f1-f8d4-4dba-872e-4e4aeb25fa1c"
    fi
  
    # mask ( " * + in $KEYWORDS 
    # $KEYWORDS serves as a pattern for grep
 
    KEYWORDS=`echo $KEYWORDS | sed 's%(%\\\(%g' | sed 's%)%\\\)%g' `    
    KEYWORDS=`echo $KEYWORDS | sed 's%"%\\\"%g' ` 
    KEYWORDS=`echo $KEYWORDS | sed 's%*%\\\*%g' `
    KEYWORDS=`echo $KEYWORDS | sed 's%+%\\\+%g' `

    # if $ALLOWEDKEYWORDSFILE contained empty lines this
    # test would always pass
    echo $KEYWORDS |  grep -q  "||"    
    if [ $? -eq 0 ]
    then 
        echo "$ALLOWEDKEYWORDSFILE contains empty lines. Aborting..."      
        exit
    fi


for i in `find $ROOT -name prod -prune -o -name delivery -prune -o -name generated -prune -o -name test -prune -o -name valid -prune -o -name ProdTests -prune -o -name orbsupport -prune -o -name Tutorial -prune -o -name CodeGen -prune -o -name \*.cpp -o -name \*.hpp -print  | grep -v -i "orbacus" | grep -v Foundation | grep -v CodeGen | grep -v Build`; do 

    result1=`grep OB $i | \
                grep -v OBJECT | \
                grep -v GLOBAL | \
                grep -v OBSERVER | \
                grep -v INV_OBJREF | \
                grep -v TAO_OBJID | \
                grep -v PROBE_ | \
                grep -v _PROBE | \
                grep -v PROBLEM | \
                grep -v OBSERVABLE_ | \
                grep -v TAOBound | \
                grep -v OBJ_ADAPTER | \
                grep -v " consumer to/from an untyped event channel" | \
                grep -v OBV_ | \
        grep -v -E "$KEYWORDS" | \
         sed "s%\/\/%\/\/%g;T;s%\(.*OB.*\/\/.*\)%\1%g;t;d"`;




    result2=`grep -i orbacus $i  | \
        grep -v -E "$KEYWORDS"`;

    if [ -n "$result1" ]; then
        file_str=`echo $i | sed -e 's/\.\.\///g'`
        echo "==========================" >> $RESULTFILE;
        echo "file $file_str :" >> $RESULTFILE;
        echo "--------------------------" >> $RESULTFILE;
        echo "$result1" >> $RESULTFILE;
        echo "==========================" >> $RESULTFILE;
        echo " " >> $RESULTFILE;
    fi

    if [ -n "$result2" ]; then
        file_str=`echo $i | sed -e 's/\.\.\///g'`
        echo "==========================" >> $RESULTFILE;
        echo "file $file_str :" >> $RESULTFILE;
        echo "--------------------------" >> $RESULTFILE;
        echo "$result2" >> $RESULTFILE;
        echo "==========================" >> $RESULTFILE;
        echo " " >> $RESULTFILE;
    fi
done

echo "Processing data for test $TEST ($ORBNAME) completed";
