#!/bin/ksh
# =====================================================================
# This file is part of CARDAMOM (R) which is jointly developed by THALES
# and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
# Copyright (C) THALES 2000-2003. All rights reserved.
# Copyright (C) THALES 2004-2005. All rights reserved.
# Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
# All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
# License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

INDENT="  "
VALIDATION_HOME=@config.install.dir@/share
TEST_STATUS=OK
TEST_ID=`basename ${0%.sh}`
RESULTS_DIR=${VALIDATION_HOME}/${TEST_ID}-results
LOG_FILE=${RESULTS_DIR}/log

function convert_csci 
{
  case ${1} in
    ccmcontainer)
      echo "CMC";;
    ccmdeployment)
      echo "CMD";;
    lb)
      echo "LDB";;
    ft)
      echo "FLT";;
    smg|platform)
      echo "SMG";;
    event)
      echo "EVT";;
    lifecycle)
      echo "LFC";;
    repository)
      echo "RPS";;
    orbsupport)
      echo "FDS";;
    clock)
      echo "TMG";;
    trace)
      echo "TRC";;
  esac
}

if [ -d "${RESULTS_DIR}" ]; then
   rm -rf ${RESULTS_DIR}
fi
mkdir -p ${RESULTS_DIR}

echo "######################" >${LOG_FILE}
echo "#${INDENT}${TEST_ID}" >>${LOG_FILE}
echo "######################" >>${LOG_FILE}

cd ${VALIDATION_HOME}

for CSCI in CMD EVT RPS TMG; 
do
  echo "${INDENT}${CSCI} c++ : OK (exempted)">>${LOG_FILE}
done

for CSCI in ccmcontainer lb ft platform lifecycle orbsupport trace; 
do
  Message="${INDENT}`convert_csci ${CSCI}` c++ : "
  nm ../lib/c++/libcdmw*${CSCI}*.so |c++filt|grep "Cdmw::Exception::~Exception" > /dev/null 2>/tmp/__${!}
  case ${?} in 
    0)
      echo "${Message}OK">>${LOG_FILE}
      ;;
    1)
      TEST_STATUS=FAILED
      echo "${Message}FAILED">>${LOG_FILE}
      echo "${INDENT}${INDENT} FAILED ==> Cdmw::Exception not used in the following libraries :" >>${LOG_FILE}
      ls ../lib/c++/libcdmw*${CSCI}*.so | awk '{ printf "                "$1"\n" }'>>${LOG_FILE}
      ;;
    2)
      TEST_STATUS=FAILED
      echo "${Message}FAILED">>${LOG_FILE}
      echo "${INDENT}${INDENT} FAILED ==> \c">>${LOG_FILE}
      cat /tmp/__${!}>>${LOG_FILE}
      ;;
  esac
  rm -f /tmp/__${!}
done
echo "######################">>${LOG_FILE}
echo "#${INDENT}${TEST_ID} : ${TEST_STATUS}">>${LOG_FILE}
echo "######################">>${LOG_FILE}

cat ${LOG_FILE}

