# =====================================================================
# This file is part of CARDAMOM (R) which is jointly developed by THALES
# and SELEX SISTEMI INTEGRATI.
# Copyright (C) SELEX SISTEMI INTEGRATI 2004-2007. All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
# License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

#!/bin/sh

# Qualification test name
TEST=Q-164

# List of external interfaces to check
INPUTFILE=$TEST'_ext_int_list.txt'

if [ ! -f $INPUTFILE ]; then
    echo "Error: '$INPUTFILE' not found."
    echo ""
    print_usage
fi
    
ret_ok=0;
ret_nok=1;

my_script=`basename $0`
current_dir=`pwd` 

## Usage
print_usage() {
    echo "$my_script --CdmwRunTestLog=RunTestLogFile"
    echo "       parse the provided runtest execution log RunTestLogFile"
    echo "       to extract the coverage of inbound and outbound"
    echo "       parameters checks on the external (user) interfaces"
    echo " "
    exit 1
}

arg= 
run_test_log_file=


while [ -n "$1" ]; do
    case $1 in
        --help)
            print_usage
            exit $ret_ok
            ;;        
        --CdmwRunTestLog=?*)
            run_test_log_file=`echo $1 | sed -e "s%--CdmwRunTestLog=%%"`
            ;;
#        --CdmwExtIntList=?*)
#            ext_int_list_file=`echo $1 | sed -e "s%--CdmwExtIntList=%%"`
#            ;;
        *)
            echo "Error: unknown option '$1'"
            print_usage
            exit $ret_ok
            ;;
    esac
        shift
done


## Check mandatory arguments.
if [ -z "$run_test_log_file" ]; then
    print_usage
fi

if [ -n "$run_test_log_file" ]; then
    if [ ! -f $run_test_log_file ]; then
        echo "Error: RunTest log file '$run_test_log_file' not found."
        echo ""
        print_usage
    fi
fi



RUNTEST=$run_test_log_file
LISTFILE=`mktemp /tmp/${my_script}_files_to_copy.XXXXXX`
echo $LISTFILE

   
cat $INPUTFILE | grep . | awk -v file="$LISTFILE" '
    # BEGIN OF AWK PROGRAM
    BEGIN {RS = "\n" ; FS = "\n"}
    {
        line=$1
        #print "LINE-  "line
        #print "NR- "length($i)   
        split(line, array, "::")   
        before_comment = array[1]
        after_comment = array[2]
        if (length(after_comment) != 0 ) {
            text = before_comment"_impl::"after_comment       
            #print "BEFORE - "before_comment
            #print "AFTER - "after_comment
            #print "      text - "text
            print text > file
        }
    }
    
    # END OF AWK PROGRAM
'
       

TEMPRESULTFILE='results_file.txt'
GLOBAL_RESULT_FILE=$TEST'_results_file.txt'
echo " ---------------removing "$TEMPRESULTFILE" files on local machine ------------"
echo " ---------------removing "$GLOBAL_RESULT_FILE" files on local machine ------------"
rm -f $GLOBAL_RESULT_FILE $TEMPRESULTFILE

exec < $LISTFILE
echo "INBOUND OUTBOUND CHECK BEGINS..."
while read LINE
do
   INOCCURRENCES=`grep -c -E "$LINE.*INBOUND PARAMETER CHECK.*]" $RUNTEST`
   OUTOCCURRENCES=`grep -c -E "$LINE.*OUTBOUND PARAMETER CHECK.*]" $RUNTEST`
   echo "Looking for INBOUND checks on " $LINE
   if [ "$INOCCURRENCES" == "0" ]; then
       echo "### NO INBOUND CHECK FOR FUNCTION "$LINE" ###"
   else
       echo "||| INBOUND CHECK FOR FUNCTION "$LINE" |||"      
   fi
   echo $LINE" INBOUND:  "$INOCCURRENCES >> $TEMPRESULTFILE
   
   echo "Looking for OUTBOUND checks on " $LINE
   if [ "$OUTOCCURRENCES" == "0" ]; then
       echo "### NO OUTBOUND CHECK FOR FUNCTION "$LINE" ###"
   else
       echo "||| OUTBOUND CHECK FOR FUNCTION "$LINE" |||"      
   fi
   echo $LINE" OUTBOUND:  "$OUTOCCURRENCES >> $TEMPRESULTFILE
done   


##store results
#if [ ! -d $SCENARIO-results ];
#then
#    mkdir  $SCENARIO-results
#fi


NB_ERR_IN=`grep -c -E "$LINE.*INBOUND.*0$" $TEMPRESULTFILE`
NB_ERR_OUT=`grep -c -E "$LINE.*OUTBOUND.*0$" $TEMPRESULTFILE`
NB_OK_IN=`grep -c -E "$LINE.*INBOUND.*[1-9]+$" $TEMPRESULTFILE`
NB_OK_OUT=`grep -c -E "$LINE.*OUTBOUND.*[1-9]+$" $TEMPRESULTFILE`
#echo $NB_ERR1_IN
#echo $NB_ERR1_OUT
#NB_ERR=$NB_ERR1


echo "" >> $GLOBAL_RESULT_FILE
echo "=============================================================================">> $GLOBAL_RESULT_FILE
echo "Ending Test $TEST for INBOUND PARAMETERS CHECK"                               >> $GLOBAL_RESULT_FILE
echo "=============================================================================">> $GLOBAL_RESULT_FILE
echo "Summary:"                                                                     >> $GLOBAL_RESULT_FILE
echo "      the qualification scenario has produced:"                               >> $GLOBAL_RESULT_FILE
echo "          $NB_ERR_IN not checked functions "                                  >> $GLOBAL_RESULT_FILE
echo "          $NB_OK_IN checked functions      "                                  >> $GLOBAL_RESULT_FILE
    
if [ "$NB_ERR_IN" != "0" ]; then
    echo "      >>>> the qualification scenario is NOK <<<<                        ">> $GLOBAL_RESULT_FILE
else 
    echo "      >>>> the qualification scenario is OK <<<<                         ">> $GLOBAL_RESULT_FILE
fi

echo "" >> $GLOBAL_RESULT_FILE
echo "=============================================================================">> $GLOBAL_RESULT_FILE
echo "Ending Test $TEST for OUTBOUND PARAMETERS CHECK"                              >> $GLOBAL_RESULT_FILE
echo "=============================================================================">> $GLOBAL_RESULT_FILE
echo "Summary:"                                                                     >> $GLOBAL_RESULT_FILE
echo "      the qualification scenario has produced:"                               >> $GLOBAL_RESULT_FILE
echo "          $NB_ERR_OUT not checked functions "                                 >> $GLOBAL_RESULT_FILE
echo "          $NB_OK_OUT checked functions      "                                 >> $GLOBAL_RESULT_FILE
    
if [ "$NB_ERR_OUT" != "0" ]; then
    echo "      >>>> the qualification scenario is NOK <<<<                        ">> $GLOBAL_RESULT_FILE
else 
    echo "      >>>> the qualification scenario is OK <<<<                         ">> $GLOBAL_RESULT_FILE
fi

# Now we can display all the results
echo "==========================================================="
cat $GLOBAL_RESULT_FILE
echo "==========================================================="

#mv $RESULTS_FILE $SCENARIO-results/scenario.result

echo "  done."
