#!/bin/ksh
# =====================================================================
# This file is part of CARDAMOM (R) which is jointly developed by THALES
# and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
# Copyright (C) THALES 2000-2003. All rights reserved.
# Copyright (C) THALES 2004-2005. All rights reserved.
# Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
# All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
# License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

INDENT="  "
VALIDATION_HOME=@config.install.dir@/share
TEST_STATUS=OK
TEST_ID=`basename ${0%.sh}`
RESULTS_DIR=${VALIDATION_HOME}/${TEST_ID}-results
TMP_LOG_FILE=/tmp/__${$}
LOG_FILE=${RESULTS_DIR}/log
STRING="Cdmw::Logging::Logger::Get_Logger"
function convert_csci_to_libs 
{
  case ${1} in
    CMC)
      # PCR-0979: for V3 only Repository has to be checked
      #           all future check for V4 are in comment
      echo "ccmcontainer ccmcif";;  
      #echo "";;
    CMD)
      #echo "libcdmwccmdeploymenttool";;
      echo "";;
    COG)
      echo "";;
    EVT)
      echo "eventinterface eventsupport";;
      #echo "";;
    FDS)
      echo "common commonsvcsdatastore commonsvcsfederation commonsvcsnaming commonsvcsserviceregistration commonsvcstime jniossupport logging orbsupport ossupport osthreads testutils";;
      #echo "";;
    FLT)
      echo "ftccmcif ftcommon ftgroupcreator ftinit ftlocationmanager ftreplicationmanager ftrequestlogging ftstatemanager ftultimatefallback monitoringlibrary mutualmonitoring simplemonitoring";;
      #echo "";;
    LDB)
      echo "lbcommon lbgroupcreator lbgroupmanager lbinit lbstrategy";;
      #echo "";;
    LFC)
      # echo "lifecycle lifecycleinit";;
      echo "";;
    RPS)
      echo "naming_and_repository_proxy_lib processrepositorytools repositoryinterface";;
    SMG)
      echo "platformagentlibrary platformapplicationlibrary platformdaemonlibrary platformelementlibrary platformfaultdetector platformftaccessorlibrary platformftproxy platforminterface platforminternallibrary platformlibrary platformsystemlibrary platformvaluetypes processadmintools supervisioncommon";;
      #echo "";;
    TMG)
      # echo "cdmwclockservice";;
      echo "";;
    TRC)
      echo "performancelibrary tracelibrary";;
      #echo "";;
    XML)
      # echo "xmllibrary";;
      echo "";;
  esac
}
function convert_csci_to_exes 
{
  case ${1} in
    CMC)
      echo "";;
    CMD)
      echo "cdmw_node_manager";;
      #echo "";;
    COG)
      echo "";;
    EVT)
      # echo "cdmw_event_admin  cdmw_event_channel_manager";;
      echo "";;
    FDS)
      echo "";;
    FLT)
      echo "cdmw_ft_group_creator  cdmw_ft_naming_and_repository cdmw_ft_manager start_ft_group_manager_observer";;
      #echo "";;
    LDB)
      echo "cdmw_lb_group_creator  cdmw_lb_group_manager  start_lb_group_manager_observer";;
      #echo "";;
    LFC)
      echo "";;
    RPS)
      echo "cdmw_naming_and_repository cdmw_repository_admin cdmw_naming_and_repository_proxy cdmw_simulated_repository";;
    SMG)
      echo "cdmw_platform_admin cdmw_platform_supervision_observer cdmw_platform_daemon cdmw_platform_supervision_starter cdmw_platform_supervision cdmw_process_admin";;
      #echo "";;
    TMG)
      echo "";;
    TRC)
      echo "cdmw_global_trace_collector cdmw_trace_admin cdmw_trace_collector cdmw_performance_admin";;
      #echo "";;
    XML)
      echo "";;
  esac
}

if [ -d "${RESULTS_DIR}" ]; then
   rm -rf ${RESULTS_DIR}
fi
mkdir -p ${RESULTS_DIR}

echo "######################" >${LOG_FILE}
echo "#${INDENT}${TEST_ID}" >>${LOG_FILE}
echo "######################" >>${LOG_FILE}

cd ${VALIDATION_HOME}

for CSCI in CMC CMD COG EVT FDS FLT LDB LFC RPS SMG TMG TRC XML;
do
  Message="${INDENT} ${CSCI} c++ : "
  RESULT_LIB=0
  ALL_LIBS=`convert_csci_to_libs ${CSCI}`
  if [ -z "${ALL_LIBS}" ]; then
     RESULT_LIB=3
  else
     ALL_LIBS=`echo ${ALL_LIBS}|awk 'BEGIN {RS=" "} {printf "../lib/c++/libcdmw" $1 ".so "}'`
     nm ${ALL_LIBS} 2>/dev/null|c++filt|grep ${STRING} >/dev/null 2>${TMP_LOG_FILE}
     RESULT_LIB=${?}
  fi

  # C++ exe part
  RESULT_EXE=0
  ALL_EXES=`convert_csci_to_exes ${CSCI}`
  if [ -z "${ALL_EXES}" ]; then
     RESULT_EXE=3
  else
     ALL_EXES=`echo ${ALL_EXES}|awk 'BEGIN {RS=" "} {printf "../bin/" $1 " "}'`
     nm ${ALL_EXES} 2>/dev/null|c++filt|grep ${STRING} >/dev/null 2>${TMP_LOG_FILE}
     RESULT_EXE=${?}
  fi

  if [ ${RESULT_LIB} = 0 -o ${RESULT_EXE} = 0 ]; then
      echo "${Message}OK">>${LOG_FILE}
  elif [ ${RESULT_LIB} = 3 -a ${RESULT_EXE} = 3 ]; then
      echo "${Message}Exempted">>${LOG_FILE}
  else
      TEST_STATUS=FAILED
      echo "${Message}FAILED">>${LOG_FILE}
      if [ ${RESULT_LIB} = 1 ]; then
          echo "${INDENT}${INDENT} FAILED ==> ${STRING} not used in the following libraries :" >>${LOG_FILE}
          echo ${ALL_LIBS} | awk 'BEGIN {RS=" "} { printf "                "$1"\n" }'>>${LOG_FILE}
      fi 
      if [ ${RESULT_EXE} = 1 ]; then
          echo "${INDENT}${INDENT} FAILED ==> ${STRING} not used in the following executables :" >>${LOG_FILE}
          echo ${ALL_EXES} | awk 'BEGIN {RS=" "} { printf "                "$1"\n" }'>>${LOG_FILE}
      fi 
      if [ ${RESULT_LIB} = 2 -o ${RESULT_EXE} = 2 ]; then
          echo "${INDENT}${INDENT} FAILED ==> \c">>${LOG_FILE}
          cat ${TMP_LOG_FILE}>>${LOG_FILE}
      fi
  fi
  rm -f ${TMP_LOG_FILE}

done

echo "######################">>${LOG_FILE}
echo "#${INDENT}${TEST_ID} : ${TEST_STATUS}">>${LOG_FILE}
echo "######################">>${LOG_FILE}

cat ${LOG_FILE}

