#!/bin/ksh
# =====================================================================
# This file is part of CARDAMOM (R) which is jointly developed by THALES
# and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
# Copyright (C) THALES 2000-2003. All rights reserved.
# Copyright (C) THALES 2004-2005. All rights reserved.
# Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
# All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
# License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

INDENT="  "
VALIDATION_HOME=@config.install.dir@/share
TEST_STATUS=OK
JAVA_TEST_STATUS=OK
TEST_ID=`basename ${0%.sh}`
RESULTS_DIR=${VALIDATION_HOME}/${TEST_ID}-results
TMP_LOG=/tmp/__${$}
LOG_FILE=${RESULTS_DIR}/log
STRING="Get_Logger"

function convert_csci_to_jars 
{
  case ${1} in
    CMC)
      # PCR-0979: for V3 only Repository has to be checked
      #           all future check for V4 are in comment
      echo "ccmcontainer ccmcif ccmcommon ccmcomponentserver";;
      #echo "";;
    CMD)
      #echo "ccmassembly ccmdeploymenttool ccmpackagemanager";;
      echo "";;
    COG)
      echo "cdmwinit cgengine";;
      #echo "";;
    EVT)
      #echo "eventinterface";;
      echo "";;
    FDS)
      echo "common commonsvcsnaming logging orbsupport ossupport testutils";;
      #echo "";;
    FLT)
      echo "";;
    LDB)
      echo "";;
    LFC)
      #echo "lifecycle";;
      echo "";;
    RPS)
      echo "repositoryinterface";;
    SMG)
      echo "platforminterface processadmintools";;
      #echo "";;
    TMG)
      echo "";;
    TRC)
      #echo "performancelibrary tracelibrary";;
      echo "";;
    XML)
      #echo "xmllibrary xmlserverlib";;
      echo "";;
  esac
}

if [ -d "${RESULTS_DIR}" ]; then
   rm -rf ${RESULTS_DIR}
fi
mkdir -p ${RESULTS_DIR}

echo "######################" >${LOG_FILE}
echo "#${INDENT}${TEST_ID}" >>${LOG_FILE}
echo "######################" >>${LOG_FILE}

cd ${VALIDATION_HOME}

for CSCI in CMC CMD COG EVT FDS FLT LDB LFC RPS SMG TMG TRC XML;
do
  Message="${INDENT} ${CSCI} java : "
  RESULT_JAR=0
  ALL_JARS=`convert_csci_to_jars ${CSCI}`
  if [ -z "${ALL_JARS}" ]; then
     RESULT_JAR=3
  else
     ALL_JARS=`echo ${ALL_JARS}|awk 'BEGIN {RS=" "} {printf "../lib/java/libcdmw" $1 ".jar "}'`
     mkdir ${TMP_LOG}_dir
     cp ${ALL_JARS} ${TMP_LOG}_dir
     cd ${TMP_LOG}_dir
     for jar in *.jar;do
          jar xf ${jar}
     done
     javap -c -v `find cdmw -name "*.class" |sed -e "s/.class//g"`|grep ${STRING} >/dev/null 2>${TMP_LOG}.log
     RESULT_JAR=${?}
  fi

  case ${RESULT_JAR} in
   0)
      echo "${Message}OK">>${LOG_FILE};;
   1)
      TEST_STATUS=FAILED
      echo "${Message}FAILED">>${LOG_FILE}
      echo "${INDENT}${INDENT} FAILED ==> ${STRING} not used in the following jar :" >>${LOG_FILE}
      echo ${ALL_JARS} | awk 'BEGIN {RS=" "} { printf "                "$1"\n" }'>>${LOG_FILE};;
   2)
      echo "${INDENT}${INDENT} FAILED ==> \c">>${LOG_FILE}
      cat ${TMP_LOG}.log>>${LOG_FILE};;
   3)
      echo "${Message}Exempted">>${LOG_FILE};;
  esac
  cd ${VALIDATION_HOME}
  rm -rf ${TMP_LOG}*

done

echo "######################">>${LOG_FILE}
echo "#${INDENT}${TEST_ID} : ${TEST_STATUS}">>${LOG_FILE}
echo "######################">>${LOG_FILE}

cat ${LOG_FILE}

