#!/bin/ksh
# =====================================================================
# This file is part of CARDAMOM (R) which is jointly developed by THALES
# and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
# Copyright (C) THALES 2000-2003. All rights reserved.
# Copyright (C) THALES 2004-2005. All rights reserved.
# Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
# All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
# License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

INSTALL_DIR=@config.install.dir@
HOSTS="@hostname1val@ @hostname2val@ @hostname3val@ @hostname4val@ @hostname5val@"
HOST1=@hostname1val@

CDMW_TEST_OUT=${CDMW_TEST_OUT:-${INSTALL_DIR}/TEST-RESULTS}
NEW_TEST=${1}

TODAY=$(date +%Y%m%d)
NOW=$(date +%H%M)

#get old test id
TEST_ID=$(grep "export CDMW_TEST_ID" ${INSTALL_DIR}/bin/log.sh | sed "s%export CDMW_TEST_ID=%%")
SAVE_DIR="${CDMW_TEST_OUT}/${TEST_ID}-${TODAY}-${NOW}"

OLD_DATE=`ls -l --time-style=+%y%m%d%H%M ${INSTALL_DIR}/bin/log.sh|awk '{print $6}'`

print_root_error()
{
    echo " "
    echo "Error: you should be root to run this script !"
    exit -1
}

print_host_error()
{
    echo " "
    echo "Error: you should be on host ${HOST1} to run this script !"
    exit -1
}

if [ `whoami` != root ]; then
    print_root_error
fi

if [ `hostname` != ${HOST1} ]; then
    print_host_error
fi

# no copy if no log has been performed before
if [ ${TEST_ID} == "Q-XXX" ]; then
    echo "No test performed before : old logs not gathered."
else
    echo "Gathering log files for ${TEST_ID}."
    echo "Creating directory ${SAVE_DIR}."
    mkdir -p ${SAVE_DIR}
    chmod 777 ${SAVE_DIR}

    for host_i in ${HOSTS}; do
       if (ping -c 1 ${host_i} 1>/dev/null 2>/dev/null); then
           echo "Saving ${TEST_ID} log files from ${host_i} to ${HOST1}"
           # saving all files from log.sh
           scp -r -p ${host_i}:${CDMW_TEST_OUT}/${TEST_ID}_*/* ${SAVE_DIR} 2>/dev/null
           ssh ${host_i} "rm -rf ${CDMW_TEST_OUT}/${TEST_ID}_*"
           # saving all modified files from share
           for file in `ssh ${host_i} "ls -lt --time-style=+%y%m%d%H%M ${INSTALL_DIR}/share" |head -10|awk -v OLD_DATE=$OLD_DATE '($6 >= OLD_DATE){print $7}'`;do
              scp -rp ${host_i}:${INSTALL_DIR}/share/${file} ${SAVE_DIR}/${host_i}_${file}
           done
       fi  
    done
fi

for host_i in ${HOSTS}; do
    if (ping -c 1 ${host_i} 1>/dev/null 2>/dev/null); then
       echo "updating test id ${TEST_ID} in log.sh on ${host_i}"
       ssh ${host_i} sed -i "s%CDMW_TEST_ID=.*%CDMW_TEST_ID=${NEW_TEST}%" ${INSTALL_DIR}/bin/log.sh
    fi
done

