#!/bin/sh
# =====================================================================
# This file is part of CARDAMOM (R) which is jointly developed by THALES
# and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
# Copyright (C) THALES 2000-2003. All rights reserved.
# Copyright (C) THALES 2004-2009. All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
# License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

#
# This script checks that all domains contained in the variable DOMAINS are
# found in files generated by the Global Trace collector
# refer to Q-426 test sheet of QTPDR
#

################################################################################

# SCRIPT STARTS HERE

# set -x

DOMAINS="CDMW.TRC CDMW.CCM CDMW.INIT CDMW.FLT CDMW.LDB \
CDMW.PFMMNG CDMW.EVT CDMW.NAR"

# Usage if parameter length different from 1
if [ $# -lt 1 ] ; then
    echo "Usage : `basename $0` [GTRC result file to process]"
    echo "Abort ..."
    exit 1
fi

# Check that the given file is readable
if [ $# -eq 1 ] ; then
    FILES=`ls $1`
else
    while [ $# -ne 0 ] ; do
        [ -r $1 ] && FILES="$FILES $1"
        shift 1
    done
fi

# Check if all mentionned domains are present
OK=0
for DOMAIN in $DOMAINS ; do 
    TMP_OK=1
    for FILE in $FILES ; do
        grep -q $DOMAIN $FILE
        [ $? -eq 0 ] && TMP_OK=0
    done
    if [ ${TMP_OK} -eq 0 ] ; then
        echo "Domain [$DOMAIN] Found !"
    else
        echo "Domain [$DOMAIN] Not found !"
        OK=1
    fi
done

# Set the test result
if [ $OK -eq 0 ] ; then
    TEST_RESULT="TESTABLE_OK"
else
    TEST_RESULT="NOT OK"
fi

# Display the test result
echo "==================================================="
echo "Ending test CDMW services using Trace service"
echo "==================================================="
echo "Summary:"
if [ "$TEST_RESULT" = "TESTABLE_OK" ] ; then
    echo "   Failed    : 0"
    echo "   Succeeded : 1"
else
    echo "   Failed    : 1"
    echo "   Succeeded : 0"
fi
echo "   Expected successfull tests (CPP or Java) :1"
echo "   >>>> Test result is ${TEST_RESULT} <<<<"
