#!/bin/sh
# =====================================================================
# This file is part of CARDAMOM (R) which is jointly developed by THALES
# and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
# Copyright (C) THALES 2000-2003. All rights reserved.
# Copyright (C) THALES 2004-2005. All rights reserved.
# Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
# All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
# License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

declare -i CDMW_VIRTUAL_SIZE=250000
declare -a DONE_LIBS

unset DONE_LIBS
test -f lib_already_in_charge.txt

if [ $? -ne 0 ]
then
   echo "Start the pmap_base.sh before the test"
   exit -1
fi

DONE_LIBS=`cat lib_already_in_charge.txt`
rm -rf lib_already_in_charge.txt
# echo $DONE_LIBS
declare -i VERBOSE=false

declare -i TOTAL_TEXT_KBS=0
declare -i TOTAL_DATA_KBS=0
declare -i TOTAL_ANON_KBS=0

FIRST_TIME_SEEN_RESULT=false

function first_time_seen ()
{
   if [ `echo ${DONE_LIBS[*]} |grep -c $1` -ne 0 ]
   then
      FIRST_TIME_SEEN_RESULT=false
   else
      DONE_LIBS[${#DONE_LIBS[*]}]=$1
      FIRST_TIME_SEEN_RESULT=true
   fi

   return 
}

function process_pid ()
{
   declare -a TEXT_LIBS=(`pmap $1 |grep r-x-- |grep -o /.*$`)
   declare -a TEXT_SIZES=(`pmap $1 |grep r-x-- |grep -o [0-9]*K |grep -o ^[0-9]*`)

   declare -i TEMP_TOTAL=$TOTAL_TEXT_KBS

   for (( I=0; I < ${#TEXT_LIBS[*]}; I++ )) 
   do
      first_time_seen ${TEXT_LIBS[$I]}
      if [ $FIRST_TIME_SEEN_RESULT == true ]
      then
          if [ $VERBOSE == true ]
          then
            echo "Recording text size of ${TEXT_SIZES[$I]} for lib ${TEXT_LIBS[$I]}"
          fi
          TOTAL_TEXT_KBS=$TOTAL_TEXT_KBS+${TEXT_SIZES[$I]}
      else
          if [ $VERBOSE == true ]
          then
              echo "Disregarding duplicate lib  ${TEXT_LIBS[$I]}"
          fi
      fi
   done

   TEMP_TOTAL=$TOTAL_TEXT_KBS-$TEMP_TOTAL

   echo "   New text mapped for this process : $TEMP_TOTAL"

   declare -a DATA_SIZES=(`pmap $1 |grep -v r-x-- |grep / |grep -o [0-9]*K |grep -o ^[0-9]*`)

   TEMP_TOTAL=$TOTAL_DATA_KBS

   for (( I=0; I < ${#DATA_SIZES[*]}; I++ ))
   do
      TOTAL_DATA_KBS=$TOTAL_DATA_KBS+${DATA_SIZES[$I]}
   done

   TEMP_TOTAL=$TOTAL_DATA_KBS-$TEMP_TOTAL

   echo "   New data mapped for this process : $TEMP_TOTAL"

   declare -a ANON_SIZES=(`pmap $1 |grep -v r-x-- |grep -v / | grep -v total |grep -o [0-9]*K |grep -o ^[0-9]*`)

   TEMP_TOTAL=$TOTAL_ANON_KBS

   for (( I=0; I < ${#ANON_SIZES[*]}; I++ ))
   do
      TOTAL_ANON_KBS=$TOTAL_ANON_KBS+${ANON_SIZES[$I]}
   done

   TEMP_TOTAL=$TOTAL_ANON_KBS-$TEMP_TOTAL

   echo "   New anon mapped for this process : $TEMP_TOTAL"
}

#
declare -a CDMW_PIDS=(`ps -f -ww -e |grep '[c]dmw_' | grep -v grep | awk '{ print $2}'`)
for (( N=0; N < ${#CDMW_PIDS[*]}; N++ )) ; do
    PID=${CDMW_PIDS[$N]}
    echo "Processing with PID $PID"
    process_pid $PID
done


declare -a DANCE_PIDS=(`ps -f -ww -e |grep 'dance' | grep -v grep | awk '{ print $2}'`)
for (( N=0; N < ${#DANCE_PIDS[*]}; N++ )) ; do
    echo "Processing with PID "${DANCE_PIDS[$N]}
    process_pid $PID
done

# echo "libraries already into the memory"
# echo ${DONE_LIBS[*]}
# echo ${DONE_LIBS[*]} >> lib_already_in_charge.txt

arg= 
file=
for arg in $@; do
    case $arg in
        --help)
            print_usage
            exit $ret_ok
            ;;
        
        --test=?*)
            file=`echo $arg | sed -e "s%--test=%%"`
            ;;
    esac
done

declare -i TOTAL=$TOTAL_TEXT_KBS
TOTAL=$TOTAL+$TOTAL_DATA_KBS
TOTAL=$TOTAL+$TOTAL_ANON_KBS

if [ ${TOTAL} -le ${CDMW_VIRTUAL_SIZE} ] ; then
    TEST_RESULT="TESTABLE_OK"
else
    TEST_RESULT="NOT OK"
fi

echo "" | tee $file
echo "Memory used by the config" | tee -a $file
echo "Total text memory used : $TOTAL_TEXT_KBS KB" | tee -a $file
echo "Total data memory used : $TOTAL_DATA_KBS KB" | tee -a $file
echo "Total anonymous memory : $TOTAL_ANON_KBS KB" | tee -a $file
echo "" | tee -a $file
echo "Maximum memory allowed : $CDMW_VIRTUAL_SIZE KB" | tee -a $file
echo "Total memory in use    : $TOTAL KB" | tee -a $file
echo "" | tee -a $file
if [ -n "$file" ] ; then
    echo "   >>>> Test result is ${TEST_RESULT} <<<<">>$file
    echo "Results stored in $file"
fi

echo ""
echo "==================================================="
echo "Ending test Q-157"
echo "==================================================="
echo "Summary:"
if [ ${TOTAL} -le ${CDMW_VIRTUAL_SIZE} ] ; then
echo "   Failed    : 0"
echo "   Succeeded : 1"
else
echo "   Failed    : 1"
echo "   Succeeded : 0"
fi
echo "   Expected successfull tests : 1"
echo "   >>>> Test result is ${TEST_RESULT} <<<<"
