/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.faulttoleranceclient1;

import java.io.IOException;

import org.junit.Assert;
import org.junit.Test;
import org.omg.CORBA.*;
import org.omg.CosNaming.NameComponent;
import org.omg.CosNotifyComm.StructuredPushConsumer;
import org.omg.FT.PropertiesHelper;
import org.omg.FT.REQUEST_DURATION_POLICY;
import org.omg.PortableServer.*;

import cdmw.commonsvcs.naming.InvalidNameException;
import cdmw.ossupport.OS;
import cdmw.testutils.CORBATestManager.InvalidObject;

public class TestFTInit {
    private ORB orb;

    private POA rootPOA;

    private com.thalesgroup.CdmwFT.ReplicationManager replicationManager;

    private String hostName1;

    private String hostName2;

    private String hostName3;

    public TestFTInit() {
        try {
            this.orb = cdmw.testutils.CORBATestManager.instance().getORB();

            this.rootPOA = cdmw.testutils.CORBATestManager.instance().getPOA();

            org.omg.CORBA.Object obj2 = cdmw.testutils.CORBATestManager
                .instance().getObject("replication_manager");
            this.replicationManager = com.thalesgroup.CdmwFT.ReplicationManagerHelper
                .narrow(obj2);
            this.hostName1 = cdmw.testutils.CORBATestManager.instance()
                .getProperty("host_name1");
            this.hostName2 = cdmw.testutils.CORBATestManager.instance()
                .getProperty("host_name2");
            this.hostName3 = cdmw.testutils.CORBATestManager.instance()
                .getProperty("host_name3");
        }
        catch (InvalidObject e) {
            e.printStackTrace();
        }
    }

    private void readLine() {
        try {
            java.io.BufferedReader inputReader = 
                new java.io.BufferedReader(
                    new java.io.InputStreamReader(System.in));
            inputReader.readLine();
        } catch (java.io.IOException e) {
            e.printStackTrace();
        }
    }

    @Test
    public void doTests() throws InterruptedException, InvalidNameException,
        UserException {

        // set_nbOfRequestedTestOK (132);
        String rep_id_HelloInterface1 = "IDL:thalesgroup.com/Cdmw/Test/HelloInterface1:1.0";
        String rep_id_HelloInterface2 = "IDL:thalesgroup.com/Cdmw/Test/HelloInterface2:1.0";
        String rep_id_HelloInterface3 = "IDL:thalesgroup.com/Cdmw/Test/HelloInterface3:1.0";

        System.out.println("define fault tolerance properties");
        //
        // define fault tolerance properties
        //
        org.omg.FT.Property[] prop = new org.omg.FT.Property[4];
        prop[0] = new org.omg.FT.Property();
        prop[1] = new org.omg.FT.Property();
        prop[2] = new org.omg.FT.Property();
        prop[3] = new org.omg.FT.Property();
        prop[0].val = this.orb.create_any();
        prop[1].val = this.orb.create_any();
        prop[2].val = this.orb.create_any();
        prop[3].val = this.orb.create_any();
        
        prop[0].nam = new NameComponent[1];
        prop[0].nam[0] = new NameComponent();
        prop[0].nam[0].id = "org.omg.ft.MinimumNumberReplicas";
        prop[0].nam[0].kind = new String();
        prop[0].val.insert_ushort((short) 2);

        prop[1].nam = new NameComponent[1];
        prop[1].nam[0] = new NameComponent();
        prop[1].nam[0].id = "org.omg.ft.ReplicationStyle";
        prop[1].nam[0].kind = new String();
        prop[1].val.insert_ushort((short) 2);

        prop[2].nam = new NameComponent[1];
        prop[2].nam[0] = new NameComponent();
        prop[2].nam[0].id = "org.omg.ft.MembershipStyle";
        prop[2].nam[0].kind = new String();
        prop[2].val.insert_ushort((short) 0);

        prop[3].nam = new NameComponent[1];
        prop[3].nam[0] = new NameComponent();
        prop[3].nam[0].id = "org.omg.ft.Factories";
        prop[3].nam[0].kind = new String();

        cdmw.ft.FactoryInfos factoryInfos1 = new cdmw.ft.FactoryInfos();
        factoryInfos1.add_ftLocation(this.hostName1, "APPL1", "P11");
        factoryInfos1.add_ftLocation(this.hostName2, "APPL3", "P31");
        factoryInfos1.add_ftLocation(this.hostName3, "APPL5", "P51");
        org.omg.FT.FactoryInfosHelper.insert(prop[3].val, factoryInfos1
            .get_factoryInfos());

        Any factory_creation_id1 = this.orb.create_any();
        org.omg.FT.Property[] ftCriteria = new org.omg.FT.Property[1];
        ftCriteria[0] = new org.omg.FT.Property();
        ftCriteria[0].val = this.orb.create_any();
        
        ftCriteria[0].nam = new NameComponent[1];
        ftCriteria[0].nam[0] = new NameComponent();
        ftCriteria[0].nam[0].id = "org.omg.ft.FTProperties";
        ftCriteria[0].nam[0].kind = new String();
        PropertiesHelper.insert(ftCriteria[0].val, prop);

        //
        // Create group HelloInterface1
        //
        System.out.println("Create group for HelloInterface1");
        org.omg.CORBA.Object obj_group_HelloInterface1 = null;

        try {
            AnyHolder holder = new AnyHolder(factory_creation_id1);
            obj_group_HelloInterface1 = this.replicationManager.create_object(
                rep_id_HelloInterface1, ftCriteria, holder);
            factory_creation_id1 = holder.value;
        }
        catch (SystemException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e0) {
            e0.printStackTrace();
            Assert.assertTrue(false);
        }

        Cdmw.Test.HelloInterface1 hello_group_HelloInterface1 = Cdmw.Test.HelloInterface1Helper
            .narrow(obj_group_HelloInterface1);

        int timescale = cdmw.testutils.CORBATestManager.getTimescale();
        cdmw.ossupport.OS.sleep(timescale * 1000);

        // REM : FLT-0210 TRANSIENT TEST could be done with other FLT-0210 test
        // in the corresponding independent test for FLT-120,130,210
        // call the method on an empty object group
        // (REQ 160)
        try {
            System.out
                .println("Calling hello on HelloInterface on an empty object group...");
            hello_group_HelloInterface1.hello();
            Assert.assertTrue(false);
        }
        catch (org.omg.CORBA.TRANSIENT e) {
            System.out.println("Expected CORBA TRANSIENT Exception raised"
                + e.toString());
            if (e.completed != org.omg.CORBA.CompletionStatus.COMPLETED_NO)
                Assert.assertTrue(false);
        }
        catch (SystemException e0) {
            System.out.println(e0.toString());
            Assert.assertTrue(false);
        }

        cdmw.ossupport.OS.sleep(timescale * 1000);

        com.thalesgroup.CdmwNamingAndRepository.Repository repository = cdmw.namingandrepository.RepositoryInterface
            .getRepository();

        cdmw.namingandrepository.RepositoryInterface.init("CDMW", repository);

        cdmw.commonsvcs.naming.NamingInterface ni = cdmw.namingandrepository.RepositoryInterface
            .getDomainNamingInterface("dom1/dom2");

        NameComponent[] loc = new NameComponent[3];
        loc[0] = new NameComponent();
        loc[1] = new NameComponent();
        loc[2] = new NameComponent();
        
        loc[0].id = this.hostName3;
        loc[0].kind = "hostname";
        loc[1].id = "APPL5";
        loc[1].kind = "applicationname";
        loc[2].id = "P51";
        loc[2].kind = "processname";

        String s_name = cdmw.commonsvcs.naming.NamingInterface.toString(loc);

        String full_name = "dom3/P51HelloInterface1";
        org.omg.CORBA.Object obj = null;

        System.out.println("resolving dom3/P51HelloInterface1...");
        try {
            obj = ni.resolve(full_name);
        }
        catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
            System.out.println(e.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e0) {
            System.out.println(e0.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.InvalidName e1) {
            System.out.println(e1.toString());
            Assert.assertTrue(false);
        }
        catch (SystemException e2) {
            System.out.println(e2.toString());
            Assert.assertTrue(false);
        }
        catch (UserException e3) {
            System.out.println(e3.toString());
            Assert.assertTrue(false);
        }

        Cdmw.Test.HelloInterface1 hello_APPL5_PROC51_INTER1 = Cdmw.Test.HelloInterface1Helper
            .narrow(obj);
        Assert.assertNotNull(hello_APPL5_PROC51_INTER1);

        // add a new member in the object group
        // (REQ 159)
        System.out.println("Add new member on the group");

        System.out.println("resolve_initial_references( ReplicationManager )");
        org.omg.CORBA.Object replication_mgr_obj = this.orb
            .resolve_initial_references("ReplicationManager");

        this.replicationManager = com.thalesgroup.CdmwFT.ReplicationManagerHelper
            .narrow(replication_mgr_obj);
        Assert.assertNotNull(this.replicationManager);

        try {
            obj_group_HelloInterface1 = this.replicationManager.add_member(
                hello_group_HelloInterface1, loc, hello_APPL5_PROC51_INTER1);
        }
        catch (SystemException e2) {
            e2.printStackTrace();
            Assert.assertTrue(false);
        }

        hello_group_HelloInterface1 = Cdmw.Test.HelloInterface1Helper
            .narrow(obj_group_HelloInterface1);
        cdmw.ossupport.OS.sleep(timescale * 1500);

        loc[0].id = this.hostName2;
        loc[0].kind = "hostname";
        loc[1].id = "APPL3";
        loc[1].kind = "applicationname";
        loc[2].id = "P31";
        loc[2].kind = "processname";

        s_name = cdmw.commonsvcs.naming.NamingInterface.toString(loc);

        full_name = "dom3/P31HelloInterface1";

        // recover the reference of the APPL3/PROC31/HelloInterface1 object
        System.out.println("resolving dom3/P31HelloInterface1...");
        try {
            obj = ni.resolve(full_name);
        }
        catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
            System.out.println(e.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e0) {
            System.out.println(e0.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.InvalidName e1) {
            System.out.println(e1.toString());
            Assert.assertTrue(false);
        }
        catch (SystemException e2) {
            System.out.println(e2.toString());
            Assert.assertTrue(false);
        }
        catch (UserException e3) {
            System.out.println(e3.toString());
            Assert.assertTrue(false);
        }

        System.out.println("narrowing hello_APPL3_PROC31_INTER1... ");
        Cdmw.Test.HelloInterface1 hello_APPL3_PROC31_INTER1 = Cdmw.Test.HelloInterface1Helper
            .narrow(obj);
        Assert.assertNotNull(hello_APPL3_PROC31_INTER1);

        // add a new member in the object group
        // (REQ 159)
        System.out.println("Add new member on the group");
        obj_group_HelloInterface1 = this.replicationManager.add_member(
            hello_group_HelloInterface1, loc, hello_APPL3_PROC31_INTER1);
        hello_group_HelloInterface1 = Cdmw.Test.HelloInterface1Helper
            .narrow(obj_group_HelloInterface1);
        cdmw.ossupport.OS.sleep(timescale * 1500);

        System.out.println("get client information");
        Cdmw.Test.process_info hello_APPL3_PROC31_INTER1_info = hello_APPL3_PROC31_INTER1
            .who_is_called();
        Cdmw.Test.process_info hello_APPL5_PROC51_INTER1_info = hello_APPL5_PROC51_INTER1
            .who_is_called();

        System.out
            .println("----------------------------------------------------------------");
        System.out
            .println("----SYS                                                       ");
        System.out
            .println("     |                                                        ");
        System.out
            .println("     |                                                        ");
        System.out
            .println("     |-------APPL3------P31------HelloInterface1------backup ");
        System.out
            .println("     |                                                   PID : "
                + hello_APPL3_PROC31_INTER1_info.processID);
        System.out
            .println("     |-------APPL5------P51------HelloInterface1------primary");
        System.out
            .println("                                                         PID : "
                + hello_APPL5_PROC51_INTER1_info.processID);
        System.out
            .println("----------------------------------------------------------------");

        System.out.println("Press any key and Enter");
        readLine();

        // call the method on an object group with primary
        // and get the group version
        // (REQ 160)
        int version_HelloInterface1 = 0;

        try {
            System.out.println("Calling hello on HelloInterface1...");
            Cdmw.Test.process_info temp_info = hello_group_HelloInterface1
                .who_is_called();

            System.out.println("the process called is the " + temp_info.name);
            if (temp_info.processID != hello_APPL5_PROC51_INTER1_info.processID)
                Assert.assertTrue(false);

            version_HelloInterface1 = hello_group_HelloInterface1
                .group_version_identification();

        }
        catch (SystemException e) {
            System.out.println(e.toString());
            Assert.assertTrue(false);
        }

        // add new member

        //loc = new NameComponent[3];
        loc[0].id = this.hostName1;
        loc[0].kind = "hostname";
        loc[1].id = "APPL1";
        loc[1].kind = "applicationname";
        loc[2].id = "P11";
        loc[2].kind = "processname";

        s_name = cdmw.commonsvcs.naming.NamingInterface.toString(loc);

        full_name = "dom3/P11HelloInterface1";

        // recover the reference of the APPL1/PROC11/HelloInterface1 object
        try {
            obj = ni.resolve(full_name);
        }
        catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
            System.out.println(e.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e0) {
            System.out.println(e0.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.InvalidName e1) {
            System.out.println(e1.toString());
            Assert.assertTrue(false);
        }
        catch (SystemException e2) {
            System.out.println(e2.toString());
            Assert.assertTrue(false);
        }

        Cdmw.Test.HelloInterface1 hello_APPL1_PROC11_INTER1 = Cdmw.Test.HelloInterface1Helper
            .narrow(obj);

        System.out.println("get client information");
        Cdmw.Test.process_info hello_APPL1_PROC11_INTER1_info = hello_APPL1_PROC11_INTER1
            .who_is_called();

        System.out
            .println("----------------------------------------------------------------");
        System.out
            .println("----SYS------APPL1------P11------HelloInterface1------backup ");
        System.out
            .println("     |                                                   PID : "
                + hello_APPL1_PROC11_INTER1_info.processID);
        System.out
            .println("     |-------APPL3------P31------HelloInterface1------backup");
        System.out
            .println("     |                                                   PID : "
                + hello_APPL3_PROC31_INTER1_info.processID);
        System.out
            .println("     |-------APPL5------P51------HelloInterface1------primary ");
        System.out
            .println("                                                         PID : "
                + hello_APPL5_PROC51_INTER1_info.processID);
        System.out
            .println("----------------------------------------------------------------");

        // add a new member in the object group
        // (REQ 159)
        System.out.println("Add new member on the group");
        obj_group_HelloInterface1 = this.replicationManager.add_member(
            hello_group_HelloInterface1, loc, hello_APPL1_PROC11_INTER1);
        hello_group_HelloInterface1 = Cdmw.Test.HelloInterface1Helper
            .narrow(obj_group_HelloInterface1);
        cdmw.ossupport.OS.sleep(timescale * 1500);

        // call the method on an object group with primary
        // and get the group version
        // the group version must be higher than the previous group version
        // stored
        // (REQ 160)
        try {
            System.out.println("Calling hello on HelloInterface1...");
            Cdmw.Test.process_info temp_info = hello_group_HelloInterface1
                .who_is_called();
            System.out.println("the process called is the " + temp_info.name);
            if (temp_info.processID != hello_APPL5_PROC51_INTER1_info.processID)
                Assert.assertTrue(false);

            System.out.println("check the version");
            if (version_HelloInterface1 < hello_group_HelloInterface1
                .group_version_identification()) {
                version_HelloInterface1 = hello_group_HelloInterface1
                    .group_version_identification();
            }
            else {
                Assert.assertTrue(false);
            }

        }
        catch (SystemException e) {
            Assert.assertTrue(false);
        }

        // get_object_group_id
        // (REQ 178)
        long object_group_id = 0;
        Cdmw.Test.HelloInterface1 temp = null;
        System.out
            .println("(REQ 178) Calling the get_object_group_id on the replication manager");
        try {
            object_group_id = this.replicationManager
                .get_object_group_id(hello_group_HelloInterface1);
        }
        catch (SystemException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e0) {
            e0.printStackTrace();
            Assert.assertTrue(false);
        }

        // use replication manager method to recover the most recent object
        // group reference
        System.out
            .println("(REQ 178) Calling the hello interface from the reference coming from the replication manager (get_object_group_ref_from_gid)");

        try {
            org.omg.CORBA.Object object_group = this.replicationManager
                .get_object_group_ref_from_gid(object_group_id);
            temp = Cdmw.Test.HelloInterface1Helper.narrow(object_group);

            Cdmw.Test.process_info temp_info = temp.who_is_called();
            System.out.println("the process called is the " + temp_info.name);
            if (temp_info.processID != hello_APPL5_PROC51_INTER1_info.processID)
                Assert.assertTrue(false);
        }
        catch (SystemException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e0) {
            e0.printStackTrace();
            Assert.assertTrue(false);
        }

        System.out
            .println("Calling the hello interface from the reference coming from the replication manager (get_object_group_ref)");

        try {
            org.omg.CORBA.Object object_group = this.replicationManager
                .get_object_group_ref(hello_group_HelloInterface1);
            temp = Cdmw.Test.HelloInterface1Helper.narrow(object_group);

            Cdmw.Test.process_info temp_info = temp.who_is_called();
            System.out.println("the process called is the " + temp_info.name);
            if (temp_info.processID != hello_APPL5_PROC51_INTER1_info.processID)
                Assert.assertTrue(false);

        }
        catch (SystemException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e0) {
            e0.printStackTrace();
            Assert.assertTrue(false);
        }

        // set_primary on the P11 member
        // (REQ 010)
        System.out
            .println("REQ 010: select the P11 member as the next primary, remove the primary and add it again");
        try {
            this.replicationManager.set_first_backup(loc);
        }
        catch (com.thalesgroup.CdmwFT.AlreadyPrimary e) {
            System.out.println("Unexpected AlreadyPrimary exception raised!");
            Assert.assertTrue(false);
        }
        catch (org.omg.FT.MemberNotFound e0) {
            System.out.println("Unexpected MemberNotFound exception raised!");
            Assert.assertTrue(false);
        }
        catch (org.omg.FT.BadReplicationStyle e1) {
            System.out
                .println("Unexpected BadReplicationStyle exception raised!");
            Assert.assertTrue(false);
        }
        catch (SystemException e2) {
            System.out.println(e2.toString());
            Assert.assertTrue(false);
        }

        //NameComponent[] loc1 = new NameComponent[3];
//        loc1[0] = new NameComponent();
//        loc1[1] = new NameComponent();
//        loc1[2] = new NameComponent();
        
        loc[0].id = this.hostName3;
        loc[0].kind = "hostname";
        loc[1].id = "APPL5";
        loc[1].kind = "applicationname";
        loc[2].id = "P51";
        loc[2].kind = "processname";

        int helloInterface1_version = TestUtils.get_object_group_version(
            this.orb, hello_group_HelloInterface1);
        try {
            obj_group_HelloInterface1 = this.replicationManager.remove_member(
                hello_group_HelloInterface1, loc);
        }
        catch (SystemException e) {
            Assert.assertTrue(false);
        }

        try {
            obj_group_HelloInterface1 = this.replicationManager.add_member(
                hello_group_HelloInterface1, loc, hello_APPL5_PROC51_INTER1);
        }
        catch (SystemException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e0) {
            e0.printStackTrace();
            Assert.assertTrue(false);
        }

        hello_group_HelloInterface1 = Cdmw.Test.HelloInterface1Helper
            .narrow(obj_group_HelloInterface1);
        cdmw.ossupport.OS.sleep(timescale * 1500);

        System.out
            .println("----------------------------------------------------------------");
        System.out
            .println("----SYS------APPL1------P11------HelloInterface1------primary");
        System.out
            .println("     |                                                   PID : "
                + hello_APPL1_PROC11_INTER1_info.processID);
        System.out
            .println("     |-------APPL3------P31------HelloInterface1------backup ");
        System.out
            .println("     |                                                   PID : "
                + hello_APPL3_PROC31_INTER1_info.processID);
        System.out
            .println("     |-------APPL5------P51------HelloInterface1------backup ");
        System.out
            .println("                                                         PID : "
                + hello_APPL5_PROC51_INTER1_info.processID);
        System.out
            .println("----------------------------------------------------------------");

        System.out.println("Press any key and Enter");
        readLine();

        // call the method on an object group with primary
        // (REQ 160)
        try {
            System.out.println("Calling hello on HelloInterface1...");

            Cdmw.Test.process_info temp_info = hello_group_HelloInterface1
                .who_is_called();
            System.out.println("the process called is the " + temp_info.name);
            if (temp_info.processID != hello_APPL1_PROC11_INTER1_info.processID)
                Assert.assertTrue(false);

            if (version_HelloInterface1 < hello_group_HelloInterface1
                .group_version_identification()) {
                version_HelloInterface1 = hello_group_HelloInterface1
                    .group_version_identification();
            }
            else
                Assert.assertTrue(false);

        }
        catch (SystemException e) {
            Assert.assertTrue(false);
        }

        try {
            System.out.println("Calling hello on HelloInterface1...");
            hello_group_HelloInterface1.hello();
            Assert.assertTrue(true);
        }
        catch (SystemException e) {
            Assert.assertTrue(false);
        }

        System.out.println("Press any key and Enter");
        readLine();

        // calling the server to recover the reference of the server object
        // group
        // the object group (on client side) must be equivalent of the object
        // group on replication manager side
        // (REQ 181)
        System.out
            .println("the object group (on client side) must be equivalent of the object group on replication manager side ");

        try {
            long id = this.replicationManager
                .get_object_group_id(hello_group_HelloInterface1);
            org.omg.CORBA.Object obj1 = hello_group_HelloInterface1
                .get_objectgroup(id);

            Assert.assertTrue(obj1._is_equivalent(obj_group_HelloInterface1));
        }
        catch (SystemException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e0) {
            e0.printStackTrace();
            Assert.assertTrue(false);
        }

        // use replication manager method to recover the next object group
        // reference
        // (REQ 178)
        System.out
            .println("use replication manager method to recover the next object group reference");
        try {
            if (TestUtils.get_object_group_version(this.orb,
                hello_group_HelloInterface1) < TestUtils
                .get_object_group_version(this.orb, temp))
                Assert.assertTrue(false);

            org.omg.CORBA.Object object_group = this.replicationManager
                .get_object_group_ref(hello_group_HelloInterface1);
            temp = Cdmw.Test.HelloInterface1Helper.narrow(object_group);

            if (TestUtils.get_object_group_version(this.orb,
                hello_group_HelloInterface1) != TestUtils
                .get_object_group_version(this.orb, temp))
                Assert.assertTrue(false);

            Cdmw.Test.process_info temp_info = temp.who_is_called();
            System.out.println("the process called is the " + temp_info.name);
            if (temp_info.processID != hello_APPL1_PROC11_INTER1_info.processID)
                Assert.assertTrue(false);

            object_group = this.replicationManager
                .get_object_group_ref_from_gid(object_group_id);
            temp = Cdmw.Test.HelloInterface1Helper.narrow(object_group);

            if (TestUtils.get_object_group_version(this.orb,
                hello_group_HelloInterface1) != TestUtils
                .get_object_group_version(this.orb, temp))
                Assert.assertTrue(false);

            temp_info = temp.who_is_called();
            System.out.println("the process called is the " + temp_info.name);
            if (temp_info.processID != hello_APPL1_PROC11_INTER1_info.processID)
                Assert.assertTrue(false);

        }
        catch (SystemException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }

        // call the method on an object group with primary
        // the method called is the send_exseption which generated exception
        // (REQ 162 & 164)
        System.out
            .println("(REQ 162 & 164) Calling the send_exception interface...");
        for (int i = 1; i < 9; i++ ) {
            try {

                switch (i){
                    case 1: {
                        System.out
                            .println("throw CORBA.COMM_FAILURE(minor_code, CORBACOMPLETED_NO)");
                        break;
                    }
                    case 2: {
                        System.out
                            .println("throw CORBA.COMM_FAILURE(minor_code, CORBA.COMPLETED_MAYBE)");
                        break;
                    }
                    case 3: {
                        System.out
                            .println("throw CORBA.TRANSIENT(minor_code, CORBA.COMPLETED_NO)");
                        break;
                    }
                    case 4: {
                        System.out
                            .println("throw CORBA.TRANSIENT(minor_code, CORBA.COMPLETED_MAYBE)");
                        break;
                    }
                    case 5: {
                        System.out
                            .println("throw CORBA.NO_RESPONSE(minor_code, CORBA.COMPLETED_NO)");
                        break;
                    }
                    case 6: {
                        System.out
                            .println("throw CORBA.NO_RESPONSE(minor_code, CORBA.COMPLETED_MAYBE)");
                        break;
                    }
                    case 7: {
                        System.out
                            .println("throw CORBA.OBJ_ADAPTER(minor_code, CORBA.COMPLETED_NO)");
                        break;
                    }
                    case 8: {
                        System.out
                            .println("throw CORBA.OBJ_ADAPTER(minor_code, CORBA.COMPLETED_MAYBE)");
                        break;
                    }
                }

                if (hello_group_HelloInterface1.send_exception(i) == 2)
                    Assert.assertTrue(true);  
            }
            catch (SystemException e) {
                Assert.assertTrue(false);
            }
        }

        // Call the send_exception with 9 in parameter
        // the method return always TRANSIENT
        // this allow to check the Request Duration Time
        // (REQ-163)
        // Obtain the ORB�s PolicyManager
        System.out.println("Check the .FT.REQUEST_DURATION_POLICY");

        System.out.println("Obtain the ORBs PolicyManager ");
        org.omg.CORBA.Object objPM = this.orb
            .resolve_initial_references("ORBPolicyManager");

        org.omg.CORBA.PolicyManager policy_manager = PolicyManagerHelper
            .narrow(objPM);

        int[] policyTypes = new int[1];
        policyTypes[0] = REQUEST_DURATION_POLICY.value;

        org.omg.CORBA.Policy[] set_policies = policy_manager
            .get_policy_overrides(policyTypes);
        System.out.println("Number of set policies is: " + set_policies.length);
        org.omg.FT.RequestDurationPolicy durationPolicy = org.omg.FT.RequestDurationPolicyHelper
            .narrow(set_policies[0]);
        long duration_time = durationPolicy.request_duration_value();

        System.out.println("Request duration time : " + duration_time);
        cdmw.ossupport.Timeval time_start = cdmw.ossupport.OS.getTime();
        try {
            hello_group_HelloInterface1.send_exception(9);
            Assert.assertTrue(false);
        }
        catch (OBJ_ADAPTER e) {
            System.out.println(e);
        }
        catch (org.omg.CORBA.TRANSIENT e) {
            Assert.assertTrue(true);
        }
        catch (SystemException e) {
            System.out.println(e);
            Assert.assertTrue(false);
        }

        cdmw.ossupport.Timeval time_end = cdmw.ossupport.OS.getTime();

        long start = time_start.getSeconds() * 10000000
            + time_start.getMicroseconds() * 10;
        long end = time_end.getSeconds() * 10000000
            + time_end.getMicroseconds() * 10;

        // the test is OK if the end-start is contained between the
        // duration_time and the duration_time + 10%
        System.out
            .println("the test is OK if the end-start is contained between the duration_time and the duration_time + 10% ");
        if (!((duration_time < (end - start)) && ((duration_time + 0.1 * duration_time) > (end - start))))
            Assert.assertTrue(false);

        // call the method request_identification
        // check if the client_id, retention_id are in the request and
        // they are OK with the configuration
        System.out
            .println("check if the client_id, retention_id are in the request ");
        org.omg.FT.FTRequestServiceContext serviceContext = null;
        try {
            serviceContext = hello_group_HelloInterface1
                .request_identification();
        }
        catch (SystemException e) {
            Assert.assertTrue(false);
        }

        // check if the retention id is increase for each request
        System.out
            .println("check if the retention id is increase for each request");
        long retention_id = serviceContext.retention_id;
        try {
            for (int i = 0; i < 10; i++ ) {
                org.omg.FT.FTRequestServiceContext serviceContext2 = hello_group_HelloInterface1
                    .request_identification();
                retention_id++ ;
                if (serviceContext2.retention_id != retention_id)
                    Assert.assertTrue(false);
            }
        }
        catch (SystemException e) {
            Assert.assertTrue(false);
        }

        // REM : FLT-0130 INV OBJ REF TEST could be done with other FLT-0130
        // test
        // in the corresponding independant test for FLT-120,130,210
        // the Replication Manager Version is lower than the object group
        // reference
        // create a new object group reference with a higher group version
        // System.out.println("the Replication Manager Object Version is lower
        // than the object group reference ");
        //        
        // org.omg.CORBA.Object object_group_with_bad_version =
        // TestUtils.update_group_with_primary_outside_cdmw_ft(this.orb,
        // hello_group_HelloInterface1,
        // this.replicationManager,
        // loc);
        //
        // temp =
        // Cdmw.Test.HelloInterface1Helper.narrow(object_group_with_bad_version);
        // try {
        // temp.hello();
        // Assert.assertTrue(false);
        // }
        // catch (INV_OBJREF e) {
        // System.out.println("Expected CORBA.INV_OBJREF Exception raised");
        // }
        // catch (SystemException e) {
        // System.out.println("Unexpected CORBA System Exception raised" + e);
        // Assert.assertTrue(false);
        // }

        //
        // Create group HelloInterface2
        //
        System.out.println("Create group HelloInterface2");

        Any factory_creation_id2 = this.orb.create_any();
        org.omg.CORBA.Object obj_group_HelloInterface2;

        cdmw.ft.FactoryInfos factoryInfos2 = new cdmw.ft.FactoryInfos();
        factoryInfos2.add_ftLocation(this.hostName1, "APPL2", "P21");
        factoryInfos2.add_ftLocation(this.hostName2, "APPL4", "P41");
        factoryInfos2.add_ftLocation(this.hostName3, "APPL6", "P61");
        org.omg.FT.FactoryInfosHelper.insert(prop[3].val, factoryInfos2
            .get_factoryInfos());

        PropertiesHelper.insert(ftCriteria[0].val, prop);

        try {
            AnyHolder holder = new AnyHolder(factory_creation_id2);
            obj_group_HelloInterface2 = this.replicationManager.create_object(
                rep_id_HelloInterface2, ftCriteria, holder);
            factory_creation_id2 = holder.value;    
        }
        catch (SystemException e) {
            e.printStackTrace();
            throw e;
        }

        Cdmw.Test.HelloInterface2 hello_group_HelloInterface2 = Cdmw.Test.HelloInterface2Helper
            .narrow(obj_group_HelloInterface2);

        cdmw.ossupport.OS.sleep(timescale * 1000);

        // Create and initialize the name
        //NameComponent[] loc2 = new NameComponent[3];
        loc[0].id = this.hostName1;
        loc[0].kind = "hostname";
        loc[1].id = "APPL2";
        loc[1].kind = "applicationname";
        loc[2].id = "P21";
        loc[2].kind = "processname";

        s_name = cdmw.commonsvcs.naming.NamingInterface.toString(loc);

        full_name = "dom3/P21HelloInterface2";
        // recover the reference of the APPL2/PROC21/HelloInterface2 object
        try {
            obj = ni.resolve(full_name);
        }
        catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
            System.out.println(e.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e0) {
            System.out.println(e0.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.InvalidName e1) {
            System.out.println(e1.toString());
            Assert.assertTrue(false);
        }
        catch (SystemException e2) {
            System.out.println(e2.toString());
            Assert.assertTrue(false);
        }

        Cdmw.Test.HelloInterface2 hello_APPL2_PROC21_INTER2 = Cdmw.Test.HelloInterface2Helper
            .narrow(obj);

        // add a new member in the object group
        // (REQ 159)
        System.out.println("Add new member on the group");
        obj_group_HelloInterface2 = this.replicationManager.add_member(
            hello_group_HelloInterface2, loc, hello_APPL2_PROC21_INTER2);

        hello_group_HelloInterface2 = Cdmw.Test.HelloInterface2Helper
            .narrow(obj_group_HelloInterface2);
        cdmw.ossupport.OS.sleep(timescale * 1500);

        loc[0].id = this.hostName2;
        loc[0].kind = "hostname";
        loc[1].id = "APPL4";
        loc[1].kind = "applicationname";
        loc[2].id = "P41";
        loc[2].kind = "processname";

        s_name = cdmw.commonsvcs.naming.NamingInterface.toString(loc);

        full_name = "dom3/P41HelloInterface2";
        // recover the reference of the APPL4/PROC41/HelloInterface2 object
        try {
            obj = ni.resolve(full_name);
        }
        catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
            System.out.println(e.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e0) {
            System.out.println(e0.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.InvalidName e1) {
            System.out.println(e1.toString());
            Assert.assertTrue(false);
        }
        catch (SystemException e2) {
            System.out.println(e2.toString());
            Assert.assertTrue(false);
        }

        Cdmw.Test.HelloInterface2 hello_APPL4_PROC41_INTER2 = Cdmw.Test.HelloInterface2Helper
            .narrow(obj);

        // add a new member in the object group
        // (REQ 159)
        System.out.println("Add new member on the group");
        obj_group_HelloInterface2 = this.replicationManager.add_member(
            hello_group_HelloInterface2, loc, hello_APPL4_PROC41_INTER2);
        hello_group_HelloInterface2 = Cdmw.Test.HelloInterface2Helper
            .narrow(obj_group_HelloInterface2);
        cdmw.ossupport.OS.sleep(timescale * 1500);

        loc[0].id = this.hostName3;
        loc[0].kind = "hostname";
        loc[1].id = "APPL6";
        loc[1].kind = "applicationname";
        loc[2].id = "P61";
        loc[2].kind = "processname";

        s_name = cdmw.commonsvcs.naming.NamingInterface.toString(loc);

        full_name = "dom3/P61HelloInterface2";

        // recover the reference of the APPL6/PROC61/HelloInterface2 object
        try {
            obj = ni.resolve(full_name);
        }
        catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
            System.out.println(e.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e0) {
            System.out.println(e0.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.InvalidName e1) {
            System.out.println(e1.toString());
            Assert.assertTrue(false);
        }
        catch (SystemException e2) {
            System.out.println(e2.toString());
            Assert.assertTrue(false);
        }

        Cdmw.Test.HelloInterface2 hello_APPL6_PROC61_INTER2 = Cdmw.Test.HelloInterface2Helper
            .narrow(obj);

        int helloInterface2_version = TestUtils.get_object_group_version(
            this.orb, hello_group_HelloInterface2);

        // add a new member in the object group
        // (REQ 159)
        System.out.println("Add new member on the group");
        obj_group_HelloInterface2 = this.replicationManager.add_member(
            hello_group_HelloInterface2, loc, hello_APPL6_PROC61_INTER2);
        hello_group_HelloInterface2 = Cdmw.Test.HelloInterface2Helper
            .narrow(obj_group_HelloInterface2);
        OS.sleep(timescale * 1500);

        //
        // Create group HelloInterface3
        //
        System.out.println("Create group");

        Any factory_creation_id3 = this.orb.create_any();
        org.omg.CORBA.Object obj_group_HelloInterface3 = null;

        cdmw.ft.FactoryInfos factoryInfos3 = new cdmw.ft.FactoryInfos();
        factoryInfos3.add_ftLocation(this.hostName1, "APPL2", "P22");
        factoryInfos3.add_ftLocation(this.hostName2, "APPL4", "P42");
        factoryInfos3.add_ftLocation(this.hostName3, "APPL6", "P62");
        org.omg.FT.FactoryInfosHelper.insert(prop[3].val, factoryInfos3
            .get_factoryInfos());
        PropertiesHelper.insert(ftCriteria[0].val, prop);

        try {
            AnyHolder holder = new AnyHolder(factory_creation_id3);
            obj_group_HelloInterface3 = this.replicationManager.create_object(
                rep_id_HelloInterface3, ftCriteria, holder);
            factory_creation_id3 = holder.value;    
        }
        catch (SystemException e) {
            e.printStackTrace();
            throw e;
        }
        catch (UserException e0) {
            e0.printStackTrace();
            throw e0;
        }

        Cdmw.Test.HelloInterface3 hello_group_HelloInterface3 = Cdmw.Test.HelloInterface3Helper
            .narrow(obj_group_HelloInterface3);

        cdmw.ossupport.OS.sleep(timescale * 1000);

        // Create and initialize the name
        //NameComponent[] loc3 = new NameComponent[3];
        loc[0].id = this.hostName1;
        loc[0].kind = "hostname";
        loc[1].id = "APPL2";
        loc[1].kind = "applicationname";
        loc[2].id = "P22";
        loc[2].kind = "processname";

        s_name = cdmw.commonsvcs.naming.NamingInterface.toString(loc);

        full_name = "dom3/P22HelloInterface3";
        // recover the reference of the APPL2/PROC22/HelloInterface3 object
        try {
            obj = ni.resolve(full_name);
        }
        catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
            System.out.println(e.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e0) {
            System.out.println(e0.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.InvalidName e1) {
            System.out.println(e1.toString());
            Assert.assertTrue(false);
        }
        catch (SystemException e2) {
            System.out.println(e2.toString());
            Assert.assertTrue(false);
        }

        Cdmw.Test.HelloInterface3 hello_APPL2_PROC22_INTER3 = Cdmw.Test.HelloInterface3Helper
            .narrow(obj);

        // add a new member in the object group
        // (REQ 159)
        System.out.println("Add new member on the group");
        obj_group_HelloInterface3 = this.replicationManager.add_member(
            hello_group_HelloInterface3, loc, hello_APPL2_PROC22_INTER3);
        hello_group_HelloInterface3 = Cdmw.Test.HelloInterface3Helper
            .narrow(obj_group_HelloInterface3);
        cdmw.ossupport.OS.sleep(timescale * 1500);

        loc[0].id = this.hostName2;
        loc[0].kind = "hostname";
        loc[1].id = "APPL4";
        loc[1].kind = "applicationname";
        loc[2].id = "P42";
        loc[2].kind = "processname";

        s_name = cdmw.commonsvcs.naming.NamingInterface.toString(loc);

        full_name = "dom3/P42HelloInterface3";

        // recover the reference of the APPL4/PROC42/HelloInterface3 object
        try {
            obj = ni.resolve(full_name);
        }
        catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
            System.out.println(e.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e0) {
            System.out.println(e0.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.InvalidName e1) {
            System.out.println(e1.toString());
            Assert.assertTrue(false);
        }
        catch (SystemException e2) {
            System.out.println(e2.toString());
            Assert.assertTrue(false);
        }

        Cdmw.Test.HelloInterface3 hello_APPL4_PROC42_INTER3 = Cdmw.Test.HelloInterface3Helper
            .narrow(obj);

        // add a new member in the object group
        // (REQ 159)
        System.out.println("Add new member on the group");
        obj_group_HelloInterface3 = this.replicationManager.add_member(
            hello_group_HelloInterface3, loc, hello_APPL4_PROC42_INTER3);
        hello_group_HelloInterface3 = Cdmw.Test.HelloInterface3Helper
            .narrow(obj_group_HelloInterface3);
        cdmw.ossupport.OS.sleep(timescale * 1500);

        loc[0].id = this.hostName3;
        loc[0].kind = "hostname";
        loc[1].id = "APPL6";
        loc[1].kind = "applicationname";
        loc[2].id = "P62";
        loc[2].kind = "processname";

        s_name = cdmw.commonsvcs.naming.NamingInterface.toString(loc);

        full_name = "dom3/P62HelloInterface3";

        // recover the reference of the APPL6/PROC62/HelloInterface3 object
        try {
            obj = ni.resolve(full_name);
        }
        catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
            System.out.println(e.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e0) {
            System.out.println(e0.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.InvalidName e1) {
            System.out.println(e1.toString());
            Assert.assertTrue(false);
        }
        catch (SystemException e2) {
            System.out.println(e2.toString());
            Assert.assertTrue(false);
        }

        Cdmw.Test.HelloInterface3 hello_APPL6_PROC62_INTER3 = Cdmw.Test.HelloInterface3Helper
            .narrow(obj);

        int helloInterface3_version = TestUtils.get_object_group_version(
            this.orb, hello_group_HelloInterface3);

        // add a new member in the object group
        // (REQ 159)
        System.out.println("Add new member on the group");
        obj_group_HelloInterface3 = this.replicationManager.add_member(
            hello_group_HelloInterface3, loc, hello_APPL6_PROC62_INTER3);
        hello_group_HelloInterface3 = Cdmw.Test.HelloInterface3Helper
            .narrow(obj_group_HelloInterface3);
        cdmw.ossupport.OS.sleep(timescale * 1500);

        // call the method on an object group with primary
        try {
            System.out.println("Calling hello on HelloInterface1...");
            hello_group_HelloInterface1.who_is_called();
        }
        catch (SystemException e) {
            Assert.assertTrue(false);
        }

        // call the method on an object group with primary
        try {
            System.out.println("Calling hello on HelloInterface2...");
            hello_group_HelloInterface2.who_is_called();
        }
        catch (SystemException e) {
            Assert.assertTrue(false);
        }

        // call the method on an object group with primary
        try {
            System.out.println("Calling hello on HelloInterface3...");
            hello_group_HelloInterface3.who_is_called();
        }
        catch (SystemException e) {
            Assert.assertTrue(false);
        }

        System.out.println("Create 2 fault notifier consumer");

        SIMConsumer consumer_obj1 = new SIMConsumer();
        SIMConsumer consumer_obj2 = new SIMConsumer();
        consumer_obj1._this_object(orb);
        consumer_obj2._this_object(orb);

        StructuredPushConsumer consumer1 = consumer_obj1._this();
        StructuredPushConsumer consumer2 = consumer_obj2._this();
        org.omg.FT.FaultNotifier fault_notifier = this.replicationManager
            .get_fault_notifier();

        long id1 = 0;
        long id2 = 0;

        System.out.println("register the first consumer");
        id1 = fault_notifier.connect_structured_fault_consumer(consumer1);
        System.out.println("id1:" + id1);

        System.out.println("register the second consumer");
        id2 = fault_notifier.connect_structured_fault_consumer(consumer2);
        System.out.println("id2:" + id2);

        com.thalesgroup.CdmwFT.FaultManagement.FaultDetector fault_detector = this.replicationManager
            .get_fault_detector();

        System.out.println("disconnect one of the previous consumer");
        try {
            fault_notifier.disconnect_consumer(id2);
        }
        catch (Exception e) {
            Assert.assertTrue(false);
        }

        System.out.println("disconnect an unknown consumer");
        try {
            fault_notifier.disconnect_consumer(1000);
            Assert.assertTrue(false);
        }
        catch (org.omg.CosEventComm.Disconnected e0) {
            System.out.println("Expected exception org.omg.CosEventComm.Disconnected");
        }
        catch (Exception e) {
            Assert.assertTrue(false);
        }

        if (helloInterface1_version > TestUtils.get_object_group_version(
            this.orb, hello_group_HelloInterface1))
            Assert.assertTrue(false);

        if (helloInterface2_version > TestUtils.get_object_group_version(
            this.orb, hello_group_HelloInterface2))
            Assert.assertTrue(false);

        if (helloInterface3_version > TestUtils.get_object_group_version(
            this.orb, hello_group_HelloInterface3))
            Assert.assertTrue(false);

        System.out.println("calling all object without the object group ref");
        Cdmw.Test.process_info hello_APPL2_PROC21_INTER2_info = hello_APPL2_PROC21_INTER2
            .who_is_called();
        Cdmw.Test.process_info hello_APPL4_PROC41_INTER2_info = hello_APPL4_PROC41_INTER2
            .who_is_called();
        Cdmw.Test.process_info hello_APPL6_PROC61_INTER2_info = hello_APPL6_PROC61_INTER2
            .who_is_called();

        Cdmw.Test.process_info hello_APPL2_PROC22_INTER3_info = hello_APPL2_PROC22_INTER3
            .who_is_called();
        Cdmw.Test.process_info hello_APPL4_PROC42_INTER3_info = hello_APPL4_PROC42_INTER3
            .who_is_called();
        Cdmw.Test.process_info hello_APPL6_PROC62_INTER3_info = hello_APPL6_PROC62_INTER3
            .who_is_called();

        System.out
            .println("----------------------------------------------------------------");
        System.out
            .println("----SYS------APPL1------P11------HelloInterface1------primary");
        System.out
            .println("     |                                                   PID : "
                + hello_APPL1_PROC11_INTER1_info.processID);
        System.out
            .println("     |-------APPL2------P21------HelloInterface2------primary ");
        System.out
            .println("     |         |                                         PID : "
                + hello_APPL2_PROC21_INTER2_info.processID);
        System.out
            .println("     |         ---------P22------HelloInterface3------primary ");
        System.out
            .println("     |                                                   PID : "
                + hello_APPL2_PROC22_INTER3_info.processID);
        System.out
            .println("     |-------APPL3------P31------HelloInterface1------backup ");
        System.out
            .println("     |                                                   PID : "
                + hello_APPL3_PROC31_INTER1_info.processID);
        System.out
            .println("     |-------APPL4------P41------HelloInterface2------backup ");
        System.out
            .println("     |         |                                         PID : "
                + hello_APPL4_PROC41_INTER2_info.processID);
        System.out
            .println("     |         ---------P42------HelloInterface3------backup");
        System.out
            .println("     |                                                   PID : "
                + hello_APPL4_PROC42_INTER3_info.processID);
        System.out
            .println("     |-------APPL5------P51------HelloInterface1------backup ");
        System.out
            .println("     |                                                   PID : "
                + hello_APPL5_PROC51_INTER1_info.processID);
        System.out
            .println("     |-------APPL6------P61------HelloInterface2------backup");
        System.out
            .println("     |         |                                         PID : "
                + hello_APPL6_PROC61_INTER2_info.processID);
        System.out
            .println("     |         ---------P62------HelloInterface3------backup ");
        System.out
            .println("                                                         PID : "
                + hello_APPL6_PROC62_INTER3_info.processID);
        System.out
            .println("----------------------------------------------------------------");

        // call the method on an object group with primary
        try {
            System.out.println("Calling hello on HelloInterface1...");
            Cdmw.Test.process_info temp_info = hello_group_HelloInterface1
                .who_is_called();

            System.out.println("the process called is the " + temp_info.name);
            if (temp_info.processID != hello_APPL1_PROC11_INTER1_info.processID)
                Assert.assertTrue(false);
        }
        catch (SystemException e) {
            Assert.assertTrue(false);
        }

        // call the method on an object group with primary
        try {
            System.out.println("Calling hello on HelloInterface2...");
            Cdmw.Test.process_info temp_info = hello_group_HelloInterface2
                .who_is_called();
            Assert.assertTrue(true);

            System.out.println("the process called is the " + temp_info.name);
            if (temp_info.processID != hello_APPL2_PROC21_INTER2_info.processID)
                Assert.assertTrue(false);
        }
        catch (SystemException e) {
            Assert.assertTrue(false);
        }

        // call the method on an object group with primary
        try {
            System.out.println("Calling hello on HelloInterface3...");
            Cdmw.Test.process_info temp_info = hello_group_HelloInterface3
                .who_is_called();

            System.out.println("the process called is the " + temp_info.name);
            if (temp_info.processID != hello_APPL2_PROC22_INTER3_info.processID)
                Assert.assertTrue(false);

        }
        catch (SystemException e) {
            Assert.assertTrue(false);
        }

        helloInterface1_version = TestUtils.get_object_group_version(this.orb,
            hello_group_HelloInterface1);
        helloInterface2_version = TestUtils.get_object_group_version(this.orb,
            hello_group_HelloInterface2);
        helloInterface3_version = TestUtils.get_object_group_version(this.orb,
            hello_group_HelloInterface3);

        cdmw.ossupport.OS.sleep(timescale * 2000);
        System.out.println("kill the process P11");
        System.out.println("Press any key and Enter");
        readLine();

        cdmw.ossupport.OS.sleep(timescale * 2000);
        System.out.println("NEW SYSTEM");
        System.out
            .println("----------------------------------------------------------------");
        System.out
            .println("----SYS------APPL1                                              ");
        System.out
            .println("     |                                                   PID : "
                + hello_APPL1_PROC11_INTER1_info.processID);
        System.out
            .println("     |-------APPL2------P21------HelloInterface2------primary ");
        System.out
            .println("     |         |                                         PID : "
                + hello_APPL2_PROC21_INTER2_info.processID);
        System.out
            .println("     |         ---------P22------HelloInterface3------primary ");
        System.out
            .println("     |                                                   PID : "
                + hello_APPL2_PROC22_INTER3_info.processID);
        System.out
            .println("     |-------APPL3------P31------HelloInterface1------primary ");
        System.out
            .println("     |                                                   PID : "
                + hello_APPL3_PROC31_INTER1_info.processID);
        System.out
            .println("     |-------APPL4------P41------HelloInterface2------backup ");
        System.out
            .println("     |         |                                         PID : "
                + hello_APPL4_PROC41_INTER2_info.processID);
        System.out
            .println("     |         ---------P42------HelloInterface3------backup");
        System.out
            .println("     |                                                   PID : "
                + hello_APPL4_PROC42_INTER3_info.processID);
        System.out
            .println("     |-------APPL5------P51------HelloInterface1------backup ");
        System.out
            .println("     |                                                   PID : "
                + hello_APPL5_PROC51_INTER1_info.processID);
        System.out
            .println("     |-------APPL6------P61------HelloInterface2------backup");
        System.out
            .println("     |         |                                         PID : "
                + hello_APPL6_PROC61_INTER2_info.processID);
        System.out
            .println("     |         ---------P62------HelloInterface3------backup ");
        System.out
            .println("                                                         PID : "
                + hello_APPL6_PROC62_INTER3_info.processID);
        System.out
            .println("----------------------------------------------------------------");

        System.out.println("Press any key and Enter");
        readLine();

        // call the method on an object group with primary
        try {
            System.out.println("Calling hello on HelloInterface1...");

            Cdmw.Test.process_info temp_info = hello_group_HelloInterface1
                .who_is_called();

            System.out.println("the process called is the " + temp_info.name);
            if (temp_info.processID != hello_APPL3_PROC31_INTER1_info.processID)
                Assert.assertTrue(false);

        }
        catch (SystemException e) {
            Assert.assertTrue(false);
        }

        // call the method on an object group with primary
        try {
            System.out.println("Calling hello on HelloInterface2...");

            Cdmw.Test.process_info temp_info = hello_group_HelloInterface2
                .who_is_called();

            System.out.println("the process called is the " + temp_info.name);
            if (temp_info.processID != hello_APPL2_PROC21_INTER2_info.processID)
                Assert.assertTrue(false);

        }
        catch (SystemException e) {
            System.out.println(e);
            Assert.assertTrue(false);
        }

        // call the method on an object group with primary
        try {
            System.out.println("Calling hello on HelloInterface3...");

            Cdmw.Test.process_info temp_info = hello_group_HelloInterface3
                .who_is_called();

            System.out.println("the process called is the " + temp_info.name);
            if (temp_info.processID != hello_APPL2_PROC22_INTER3_info.processID)
                Assert.assertTrue(false);

        }
        catch (SystemException e) {
            Assert.assertTrue(false);
        }

        // This flag could be used when PCR-0243 will be solved by Prismtech
        // In this case, REQ-SRS-FLT-120 is also tested in this test
        // #ifndef TESTFWDPERM
        long object_group_id1 = 0, object_group_id2 = 0, object_group_id3 = 0;
        Cdmw.Test.HelloInterface1 hello1 = null;
        Cdmw.Test.HelloInterface2 hello2 = null;
        Cdmw.Test.HelloInterface3 hello3 = null;
        try {
            object_group_id1 = this.replicationManager
                .get_object_group_id(hello_group_HelloInterface1);
            object_group_id2 = this.replicationManager
                .get_object_group_id(hello_group_HelloInterface2);
            object_group_id3 = this.replicationManager
                .get_object_group_id(hello_group_HelloInterface3);
        }
        catch (SystemException e) {
            e.printStackTrace();
        }
        catch (UserException e1) {
            e1.printStackTrace();
        }
        // #endif

        int hello1_version_sav, hello2_version_sav, hello3_version_sav;
        System.out.println("Check the good version of each object group");
        // #ifdef TESTFWDPERM
        // hello1_version_sav = TestUtils.get_object_group_version(this.orb,
        // hello_group_HelloInterface1);
        // #else
        try {
            org.omg.CORBA.Object object_group = this.replicationManager
                .get_object_group_ref_from_gid(object_group_id1);
            hello1 = Cdmw.Test.HelloInterface1Helper.narrow(object_group);
        }
        catch (SystemException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e1) {
            e1.printStackTrace();
            Assert.assertTrue(false);
        }

        hello1_version_sav = TestUtils.get_object_group_version(this.orb,
            hello1);
        // #endif
        if (helloInterface1_version > hello1_version_sav)
            Assert.assertTrue(false);

        // #ifdef TESTFWDPERM
        // hello2_version_sav = TestUtils.get_object_group_version(this.orb,
        // hello_group_HelloInterface2);
        // #else
        try {
            org.omg.CORBA.Object object_group = this.replicationManager
                .get_object_group_ref_from_gid(object_group_id2);
            hello2 = Cdmw.Test.HelloInterface2Helper.narrow(object_group);
        }
        catch (SystemException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e0) {
            e0.printStackTrace();
            Assert.assertTrue(false);
        }

        hello2_version_sav = TestUtils.get_object_group_version(this.orb,
            hello2);
        // #endif
        if (helloInterface2_version != hello2_version_sav)
            Assert.assertTrue(false);

        // #ifdef TESTFWDPERM
        // hello3_version_sav = TestUtils.get_object_group_version(this.orb,
        // hello_group_HelloInterface3);
        // #else
        try {
            org.omg.CORBA.Object object_group = this.replicationManager
                .get_object_group_ref_from_gid(object_group_id3);
            hello3 = Cdmw.Test.HelloInterface3Helper.narrow(object_group);
        }
        catch (SystemException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e0) {
            e0.printStackTrace();
            Assert.assertTrue(false);
        }
        hello3_version_sav = TestUtils.get_object_group_version(this.orb,
            hello3);
        // #endif
        if (helloInterface3_version != hello3_version_sav)
            Assert.assertTrue(false);

        helloInterface1_version = hello1_version_sav;
        helloInterface2_version = hello2_version_sav;
        helloInterface3_version = hello3_version_sav;

        System.out
            .println("Check the good number of member in each object group");
        // #ifdef TESTFWDPERM
        // if (TestUtils.get_number_of_member(this.orb,
        // hello_group_HelloInterface1) == 2)
        // #else
        // no need to take ref again
        if (!(TestUtils.get_number_of_member(this.orb, hello1) == 2))
            // #endif
            // CPPUNIT_ASSERT(true);
            // else
            Assert.assertTrue(false);

        // #ifdef TESTFWDPERM
        // if ( TestUtils.get_number_of_member(this.orb,
        // hello_group_HelloInterface2) == 3)
        // #else
        // no need to take ref again
        if (!(TestUtils.get_number_of_member(this.orb, hello2) == 3))
            // #endif
            // CPPUNIT_ASSERT(true);
            // else
            Assert.assertTrue(false);

        // #ifdef TESTFWDPERM
        // if (TestUtils.get_number_of_member(this.orb,
        // hello_group_HelloInterface3) == 3)
        // #else
        // // no need to take ref again)
        if (!(TestUtils.get_number_of_member(this.orb, hello3) == 3))
            // #endif
            // CPPUNIT_ASSERT(true);
            // else
            Assert.assertTrue(false);

        cdmw.ossupport.OS.sleep(timescale * 2000);
        System.out.println("kill the agent of the APPL2");
        System.out.println("Press any key and Enter");
        readLine();

        cdmw.ossupport.OS.sleep(timescale * 2000);

        System.out.println("NEW SYSTEM");
        System.out
            .println("----------------------------------------------------------------");
        System.out
            .println("----SYS------APPL1                                              ");
        System.out
            .println("     |                                                          ");
        System.out
            .println("     |                                                          ");
        System.out
            .println("     |                                                          ");
        System.out
            .println("     |                                                          ");
        System.out
            .println("     |                                                          ");
        System.out
            .println("     |-------APPL3------P31------HelloInterface1------primary ");
        System.out
            .println("     |                                                   PID : "
                + hello_APPL3_PROC31_INTER1_info.processID);
        System.out
            .println("     |-------APPL4------P41------HelloInterface2------primary ");
        System.out
            .println("     |         |                                         PID : "
                + hello_APPL4_PROC41_INTER2_info.processID);
        System.out
            .println("     |         ---------P42------HelloInterface3------primary");
        System.out
            .println("     |                                                   PID : "
                + hello_APPL4_PROC42_INTER3_info.processID);
        System.out
            .println("     |-------APPL5------P51------HelloInterface1------backup ");
        System.out
            .println("     |                                                   PID : "
                + hello_APPL5_PROC51_INTER1_info.processID);
        System.out
            .println("     |-------APPL6------P61------HelloInterface2------backup");
        System.out
            .println("     |         |                                         PID : "
                + hello_APPL6_PROC61_INTER2_info.processID);
        System.out
            .println("     |         ---------P62------HelloInterface3------backup ");
        System.out
            .println("                                                         PID : "
                + hello_APPL6_PROC62_INTER3_info.processID);
        System.out
            .println("----------------------------------------------------------------");
        System.out.println("Press any key and Enter");
        readLine();

        // call the method on an object group with primary
        try {
            System.out.println("Calling hello on HelloInterface1...");

            Cdmw.Test.process_info temp_info = hello_group_HelloInterface1
                .who_is_called();

            System.out.println("the process called is the " + temp_info.name);
            if (temp_info.processID != hello_APPL3_PROC31_INTER1_info.processID)
                Assert.assertTrue(false);
        }
        catch (SystemException e) {
            Assert.assertTrue(false);
        }

        // call the method on an object group with primary
        try {
            System.out.println("Calling hello on HelloInterface2...");

            Cdmw.Test.process_info temp_info = hello_group_HelloInterface2
                .who_is_called();

            System.out.println("the process called is the " + temp_info.name);
            if (temp_info.processID != hello_APPL4_PROC41_INTER2_info.processID)
                Assert.assertTrue(false);
        }
        catch (SystemException e) {
            Assert.assertTrue(false);
        }

        // call the method on an object group with primary
        try {
            System.out.println("Calling hello on HelloInterface3...");

            Cdmw.Test.process_info temp_info = hello_group_HelloInterface3
                .who_is_called();

            System.out.println("the process called is the " + temp_info.name);
            if (temp_info.processID != hello_APPL4_PROC42_INTER3_info.processID)
                Assert.assertTrue(false);
        }
        catch (SystemException e) {
            Assert.assertTrue(false);
        }

        System.out.println("Check the good version of each object group");
        System.out.println("helloInterface1_version :"
            + helloInterface1_version);
        // #ifdef TESTFWDPERM
        // hello1_version_sav = TestUtils.get_object_group_version(this.orb,
        // hello_group_HelloInterface1);
        // #else
        try {
            org.omg.CORBA.Object object_group = this.replicationManager
                .get_object_group_ref_from_gid(object_group_id1);
            hello1 = Cdmw.Test.HelloInterface1Helper.narrow(object_group);
        }
        catch (SystemException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e0) {
            e0.printStackTrace();
            Assert.assertTrue(false);
        }
        hello1_version_sav = TestUtils.get_object_group_version(this.orb,
            hello1);
        // #endif
        System.out.println(" Cdmw.FT.TestUtils.get_object_group_version :"
            + hello1_version_sav);
        if (helloInterface1_version != hello1_version_sav)
            Assert.assertTrue(false);

        System.out.println("helloInterface2_version :"
            + helloInterface2_version);
        // #ifdef TESTFWDPERM
        // hello2_version_sav = TestUtils.get_object_group_version(this.orb,
        // hello_group_HelloInterface2);
        // #else
        try {
            org.omg.CORBA.Object object_group = this.replicationManager
                .get_object_group_ref_from_gid(object_group_id2);
            hello2 = Cdmw.Test.HelloInterface2Helper.narrow(object_group);
        }
        catch (SystemException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e0) {
            e0.printStackTrace();
            Assert.assertTrue(false);
        }
        hello2_version_sav = TestUtils.get_object_group_version(this.orb,
            hello2);
        // #endif
        System.out.println(" Cdmw.FT.TestUtils.get_object_group_version :"
            + hello2_version_sav);
        if (helloInterface2_version > hello2_version_sav)
            Assert.assertTrue(false);

        System.out.println("helloInterface3_version :"
            + helloInterface3_version);
        // #ifdef TESTFWDPERM
        // hello3_version_sav = TestUtils.get_object_group_version(this.orb,
        // hello_group_HelloInterface3);
        // #else
        try {
            org.omg.CORBA.Object object_group = this.replicationManager
                .get_object_group_ref_from_gid(object_group_id3);
            hello3 = Cdmw.Test.HelloInterface3Helper.narrow(object_group);
        }
        catch (SystemException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e0) {
            e0.printStackTrace();
            Assert.assertTrue(false);
        }

        hello3_version_sav = TestUtils.get_object_group_version(this.orb,
            hello3);
        // #endif
        System.out.println(" Cdmw.FT.TestUtils.get_object_group_version :"
            + hello3_version_sav);
        if (helloInterface3_version > hello3_version_sav)
            Assert.assertTrue(false);

        helloInterface1_version = hello1_version_sav;
        helloInterface2_version = hello2_version_sav;
        helloInterface3_version = hello3_version_sav;
        System.out
            .println("Check the good number of member in each object group");
        // #ifdef TESTFWDPERM
        // if (TestUtils.get_number_of_member(this.orb,
        // hello_group_HelloInterface1) == 2)
        // #else
        if (TestUtils.get_number_of_member(this.orb, hello1) != 2)
            // #endif
            // CPPUNIT_ASSERT(true);
            // else
            Assert.assertTrue(false);

        // #ifdef TESTFWDPERM
        // if (TestUtils.get_number_of_member(this.orb,
        // hello_group_HelloInterface2) == 2)
        // #else
        if (TestUtils.get_number_of_member(this.orb, hello2) != 2)
            // #endif
            // CPPUNIT_ASSERT(true);
            // else
            Assert.assertTrue(false);

        // #ifdef TESTFWDPERM
        // if (TestUtils.get_number_of_member(this.orb,
        // hello_group_HelloInterface3) == 2)
        // #else
        if (TestUtils.get_number_of_member(this.orb, hello3) != 2)
            // #endif
            // CPPUNIT_ASSERT(true);
            // else
            Assert.assertTrue(false);

        cdmw.ossupport.OS.sleep(timescale * 2000);
        System.out.println("kill the daemon on the host2");
        System.out.println("Press any key and Enter");
        readLine();
        cdmw.ossupport.OS.sleep(timescale * 2000);

        System.out.println("NEW SYSTEM");
        System.out
            .println("----------------------------------------------------------------");
        System.out
            .println("----SYS------APPL1                                              ");
        System.out
            .println("     |                                                          ");
        System.out
            .println("     |                                                          ");
        System.out
            .println("     |                                                          ");
        System.out
            .println("     |                                                          ");
        System.out
            .println("     |                                                          ");
        System.out
            .println("     |-------APPL5------P51------HelloInterface1------primary ");
        System.out
            .println("     |                                                   PID : "
                + hello_APPL5_PROC51_INTER1_info.processID);
        System.out
            .println("     |-------APPL6------P61------HelloInterface2------primary");
        System.out
            .println("     |         |                                         PID : "
                + hello_APPL6_PROC61_INTER2_info.processID);
        System.out
            .println("     |         ---------P62------HelloInterface3------primary ");
        System.out
            .println("                                                         PID : "
                + hello_APPL6_PROC62_INTER3_info.processID);
        System.out
            .println("----------------------------------------------------------------");
        System.out.println("Press any key and Enter");
        readLine();

        // call the method on an object group with primary
        try {
            System.out.println("Calling hello on HelloInterface1...");

            Cdmw.Test.process_info temp_info = hello_group_HelloInterface1
                .who_is_called();

            System.out.println("the process called is the " + temp_info.name);
            if (temp_info.processID != hello_APPL5_PROC51_INTER1_info.processID)
                Assert.assertTrue(false);
        }
        catch (SystemException e) {
            Assert.assertTrue(false);
        }

        // call the method on an object group with primary
        try {
            System.out.println("Calling hello on HelloInterface2...");
            Cdmw.Test.process_info temp_info = hello_group_HelloInterface2
                .who_is_called();

            System.out.println("the process called is the " + temp_info.name);
            if (temp_info.processID != hello_APPL6_PROC61_INTER2_info.processID)
                Assert.assertTrue(false);

        }
        catch (SystemException e) {
            System.out.println(e);
            Assert.assertTrue(false);
        }

        // call the method on an object group with primary
        try {
            System.out.println("Calling hello on HelloInterface3...");
            hello_group_HelloInterface3.hello();

            Cdmw.Test.process_info temp_info = hello_group_HelloInterface3
                .who_is_called();

            System.out.println("the process called is the " + temp_info.name);
            if (temp_info.processID != hello_APPL6_PROC62_INTER3_info.processID)
                Assert.assertTrue(false);

        }
        catch (SystemException e) {
            Assert.assertTrue(false);
        }
        System.out.println("Check the good version of each object group");
        System.out.println("helloInterface1_version :"
            + helloInterface1_version);
        // #ifdef TESTFWDPERM
        // hello1_version_sav = TestUtils.get_object_group_version(this.orb,
        // hello_group_HelloInterface1);
        // #else
        try {
            org.omg.CORBA.Object object_group = this.replicationManager
                .get_object_group_ref_from_gid(object_group_id1);
            hello1 = Cdmw.Test.HelloInterface1Helper.narrow(object_group);
        }
        catch (SystemException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e0) {
            e0.printStackTrace();
            Assert.assertTrue(false);
        }

        hello1_version_sav = TestUtils.get_object_group_version(this.orb,
            hello1);
        // #endif
        System.out.println(" Cdmw.FT.TestUtils.get_object_group_version :"
            + hello1_version_sav);
        if (helloInterface1_version > hello1_version_sav)
            Assert.assertTrue(false);

        System.out.println("helloInterface2_version :"
            + helloInterface2_version);
        // #ifdef TESTFWDPERM
        // hello2_version_sav = TestUtils.get_object_group_version(this.orb,
        // hello_group_HelloInterface2);
        // #else
        try {
            org.omg.CORBA.Object object_group = this.replicationManager
                .get_object_group_ref_from_gid(object_group_id2);
            hello2 = Cdmw.Test.HelloInterface2Helper.narrow(object_group);
        }
        catch (SystemException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e0) {
            e0.printStackTrace();
            Assert.assertTrue(false);
        }
        hello2_version_sav = TestUtils.get_object_group_version(this.orb,
            hello2);
        // #endif
        System.out.println(" Cdmw.FT.TestUtils.get_object_group_version :"
            + hello2_version_sav);
        if (helloInterface2_version > hello2_version_sav)
            Assert.assertTrue(false);

        System.out.println("helloInterface3_version :"
            + helloInterface3_version);
        // #ifdef TESTFWDPERM
        // hello3_version_sav = TestUtils.get_object_group_version(this.orb,
        // hello_group_HelloInterface3);
        // #else
        try {
            org.omg.CORBA.Object object_group = this.replicationManager
                .get_object_group_ref_from_gid(object_group_id3);
            hello3 = Cdmw.Test.HelloInterface3Helper.narrow(object_group);
        }
        catch (SystemException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e0) {
            e0.printStackTrace();
            Assert.assertTrue(false);
        }
        hello3_version_sav = TestUtils.get_object_group_version(this.orb,
            hello3);
        // #endif

        System.out.println(" Cdmw.FT.TestUtils.get_object_group_version :"
            + hello3_version_sav);
        if (helloInterface3_version > hello3_version_sav)
            Assert.assertTrue(false);

        helloInterface1_version = hello1_version_sav;
        helloInterface2_version = hello2_version_sav;
        helloInterface3_version = hello3_version_sav;

        System.out
            .println("Check the good number of member in each object group");

        // #ifdef TESTFWDPERM
        // System.out.println("get_number_of_member" +
        // TestUtils.get_number_of_member(this.orb,
        // hello_group_HelloInterface1));
        // if (TestUtils.get_number_of_member(this.orb,
        // hello_group_HelloInterface1) == 1)
        // #else
        System.out.println("get_number_of_member"
            + TestUtils.get_number_of_member(this.orb, hello1));
        if (TestUtils.get_number_of_member(this.orb, hello1) != 1)
            // #endif
            // CPPUNIT_ASSERT(true);
            // else
            Assert.assertTrue(false);

        // #ifdef TESTFWDPERM
        // System.out.println("get_number_of_member" +
        // TestUtils.get_number_of_member(this.orb,
        // hello_group_HelloInterface2));
        // if (TestUtils.get_number_of_member(this.orb,
        // hello_group_HelloInterface2) == 1)
        // #else
        System.out.println("get_number_of_member"
            + TestUtils.get_number_of_member(this.orb, hello2));
        if (TestUtils.get_number_of_member(this.orb, hello2) != 1)
            // #endif
            // CPPUNIT_ASSERT(true);
            // else
            Assert.assertTrue(false);

        // #ifdef TESTFWDPERM
        // System.out.println("get_number_of_member" +
        // TestUtils.get_number_of_member(this.orb,
        // hello_group_HelloInterface3));
        // if (TestUtils.get_number_of_member(this.orb,
        // hello_group_HelloInterface3) == 1)
        // #else
        System.out.println("get_number_of_member"
            + TestUtils.get_number_of_member(this.orb, hello3));
        if (TestUtils.get_number_of_member(this.orb, hello3) != 1)
            // #endif
            // CPPUNIT_ASSERT(true);
            // else
            Assert.assertTrue(false);

        System.out
            .println("calling send_exception for the emission of a notifyFatalError from the server to the supervision");
        try {
            hello_group_HelloInterface1.send_exception(11);
        }
        catch (SystemException e) {
            Assert.assertTrue(false);
        }

        cdmw.ossupport.OS.sleep(timescale * 2000);

        System.out.println("NEW SYSTEM");
        System.out
            .println("----------------------------------------------------------------");
        System.out
            .println("----SYS------APPL1                                              ");
        System.out
            .println("     |                                                          ");
        System.out
            .println("     |                                                          ");
        System.out
            .println("     |                                                          ");
        System.out
            .println("     |-------APPL6------P61------HelloInterface2------primary");
        System.out
            .println("     |         |                                         PID : "
                + hello_APPL6_PROC61_INTER2_info.processID);
        System.out
            .println("     |         ---------P62------HelloInterface3------primary ");
        System.out
            .println("                                                         PID : "
                + hello_APPL6_PROC62_INTER3_info.processID);
        System.out
            .println("----------------------------------------------------------------");
        System.out.println("Press any key and Enter");
        System.out.println("Press enter to stop the process");
        readLine();

        // call the method on an object group with primary
        try {
            System.out.println("Calling hello on HelloInterface1...");
            Cdmw.Test.process_info temp_info = hello_group_HelloInterface1
                .who_is_called();
            Assert.assertTrue(false);
        }
        catch (TRANSIENT e) {
            System.out.println("Expected CORBA.TRANSIENT Exception raised" + e);
            if (e.completed.value() != CompletionStatus.COMPLETED_NO.value())
                Assert.assertTrue(false);
        }
        catch (SystemException e) {
            System.out.println(e);
            Assert.assertTrue(false);
        }

        // REM : FLT-0210 TRANSIENT TEST could be done with other FLT-0210 test
        // in the corresponding independent test for FLT-120,130,210
        // call the method on an object group with primary
        try {
            System.out.println("Calling hello on HelloInterface2...");
            Cdmw.Test.process_info temp_info = hello_group_HelloInterface2
                .who_is_called();
        }
        catch (TRANSIENT e) {
            System.out.println("Unexpected CORBA.TRANSIENT Exception raised"
                + e);
            Assert.assertTrue(false);
        }
        catch (SystemException e) {
            System.out.println(e);
            Assert.assertTrue(false);
        }

        // call the method on an object group with primary
        try {
            System.out.println("Calling hello on HelloInterface3...");
            hello_group_HelloInterface3.hello();

            Cdmw.Test.process_info temp_info = hello_group_HelloInterface3
                .who_is_called();

            System.out.println("the process called is the " + temp_info.name);
            if (temp_info.processID != hello_APPL6_PROC62_INTER3_info.processID)
                Assert.assertTrue(false);
        }
        catch (SystemException e) {
            Assert.assertTrue(false);
        }

        // remove a new menber on the group previously deleted
        System.out.println("Remove a member P61 from the helloInterface1");

        //loc = new NameComponent[3];
        loc[0].id = this.hostName3;
        loc[0].kind = "hostname";
        loc[1].id = "APPL6";
        loc[1].kind = "applicationname";
        loc[2].id = "P61";
        loc[2].kind = "processname";
        try {
            obj_group_HelloInterface2 = this.replicationManager.remove_member(
                hello_group_HelloInterface2, loc);
        }
        catch (SystemException e) {
            Assert.assertTrue(false);
        }

        System.out.println("Remove a member P62 from the helloInterface3");

        //loc = new NameComponent[3];
        loc[0].id = this.hostName3;
        loc[0].kind = "hostname";
        loc[1].id = "APPL6";
        loc[1].kind = "applicationname";
        loc[2].id = "P62";
        loc[2].kind = "processname";
        try {
            obj_group_HelloInterface3 = this.replicationManager.remove_member(
                hello_group_HelloInterface3, loc);
        }
        catch (SystemException e) {
            Assert.assertTrue(false);
        }

        // remove the object group
        System.out.println("Delete the object group helloInterface1");
        try {
            this.replicationManager.delete_object(factory_creation_id1);
        }
        catch (SystemException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e0) {
            e0.printStackTrace();
            Assert.assertTrue(false);
        }

        // remove the object group
        System.out.println("Delete the object group helloInterface2");
        try {
            this.replicationManager.delete_object(factory_creation_id2);
        }
        catch (SystemException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e0) {
            e0.printStackTrace();
            Assert.assertTrue(false);
        }

        // remove the object group
        System.out.println("Delete the object group helloInterface3");
        try {
            this.replicationManager.delete_object(factory_creation_id3);
        }
        catch (SystemException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e0) {
            e0.printStackTrace();
            Assert.assertTrue(false);
        }

    }
}
