/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.faulttoleranceclient2;

import java.io.IOException;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;

public class MainTest {

    private static final String REPLICATION_MANAGER_PORT = "5040";

    public static void main(String[] args) {
        // get host name
        String hostName1 = cdmw.ossupport.OS.getOptionValue(args, "--host1");
        // get host name
        String hostName2 = cdmw.ossupport.OS.getOptionValue(args, "--host2");
        // get host name
        String hostName3 = cdmw.ossupport.OS.getOptionValue(args, "--host3");


        try
        {
            // Initialise FT service
            //cdmw.ftinit.FTServiceInit.init(args, true );
            cdmw.cdmwinit.CdmwInterface.cdmwPreORBInit(args,"cdmw.generated.FTClient2ClientCdmwInterface");
            // Initialises the ORB
            cdmw.orbsupport.StrategyList strategyList = new cdmw.orbsupport.StrategyList();
            strategyList.addORBThreaded();
            strategyList.addPOAThreadPerConnection();

            ORB orb =  cdmw.orbsupport.ORBSupport.ORBInit(args, strategyList); 

            // Get the root POA
            org.omg.CORBA.Object obj = orb.resolve_initial_references( "RootPOA" );
            POA rootPOA = POAHelper.narrow(obj);

            POAManager poaMgr = rootPOA.the_POAManager();
            poaMgr.activate();

            cdmw.testutils.CORBATestManager.instance().init(
                (new Throwable()).getStackTrace()[0].getClassName(),
                orb,
                rootPOA);
            cdmw.testutils.CORBATestManager.instance().setProperty ("host_name1",hostName1);
            cdmw.testutils.CORBATestManager.instance().setProperty ("host_name2",hostName2);
            cdmw.testutils.CORBATestManager.instance().setProperty ("host_name3",hostName3);


            // creates the process behaviour
            MyProcessBehaviour pProcessBehaviour = new MyProcessBehaviour(orb);

            // init Cdmw
            cdmw.cdmwinit.CdmwInterface.cdmwInit(orb, args,  pProcessBehaviour, "cdmw.generated.FTClient2ClientCdmwInterface");

            // Enter in the main loop
            orb.run();

            // orb->shutdown() is done by the stop() of the ProcessBehaviour
            cdmw.cdmwinit.CdmwInterface.cdmwCleanup(orb, "cdmw.generated.FTClient2ClientCdmwInterface");
            cdmw.orbsupport.ORBSupport.ORBCleanup(orb);
            orb.destroy();
        }
        catch(org.omg.CORBA.SystemException e) {
            e.printStackTrace();
            System.exit(1);
        }
        catch(org.omg.CORBA.UserException e1) {
            e1.printStackTrace();
            System.exit(1);
        }
        catch(cdmw.common.Exception e2) {
            e2.printStackTrace();
            System.exit(1);
        }
        catch( Exception e3) {
            System.err.println("Unexpected exception");
            System.exit(1);
        }


        System.out.println("Press enter to stop the process");
        try {
            System.in.read();
        }
        catch (IOException e) {
            e.printStackTrace();
        }
        System.exit(0);
    }
}
