/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.faulttoleranceclient2;

import org.omg.CORBA.*;
import org.omg.CORBA.ORBPackage.InvalidName;

import cdmw.testutils.CORBATestManager.InvalidObject;

import com.thalesgroup.CdmwPlatformMngt.ProcessDelegatePackage.NotReadyToRun;
import com.thalesgroup.CdmwPlatformMngtBase.StartupKind;

/**
 * Purpose: <p> The abstract process behaviour.
 */
public class MyProcessBehaviour extends cdmw.cdmwinit.ProcessControl {

    /**
     * The ORB.
     */
    private ORB orb;

    /**
     * The replication manager.
     */
    private com.thalesgroup.CdmwFT.ReplicationManager replication_mgr;

    /**
     * The test thread.
     */
    private TestFTInitThread test_thread;

    /**
     * Purpose: <p> The constructor.
     */
    public MyProcessBehaviour(
        ORB orb) {
        this.orb = orb;
    }

    /**
     * Purpose: <p> the behaviour for the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/initialise:1.0
     * operation
     */
    @Override
    public void onInitialise(StartupKind startupKind) {

    }

    /**
     * Purpose: <p> the behaviour for the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/run:1.0 operation
     */
    @Override
    public void onRun() throws NotReadyToRun {
        org.omg.CORBA.Object replication_mgr_obj = null;
        try {
            replication_mgr_obj = this.orb.resolve_initial_references("ReplicationManager");

            this.replication_mgr = com.thalesgroup.CdmwFT.ReplicationManagerHelper.narrow(replication_mgr_obj);

            // Launch the test in a thread
            this.test_thread = new TestFTInitThread();
            cdmw.testutils.CORBATestManager.instance().add("replication_manager", this.replication_mgr);
            this.test_thread.start();

        }
        catch (InvalidName e) {
            e.printStackTrace();
        }
        catch (InvalidObject e) {
            e.printStackTrace();
        }
    }

    /**
     * Purpose: <p> the behaviour for the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/stop:1.0 operation
     */
    @Override
    public void onStop() {
        try {
            this.test_thread.join();
        }
        catch (InterruptedException e) {
            e.printStackTrace();
        }
    }
}
