/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.faulttoleranceclient2;

import java.io.IOException;

import org.junit.Assert;
import org.junit.Test;
import org.omg.CORBA.*;
import org.omg.CosNaming.NameComponent;
import org.omg.FT.*;
import org.omg.FT.GenericFactoryPackage.FactoryCreationIdHelper;
import org.omg.PortableServer.*;

import cdmw.testutils.CORBATestManager.InvalidObject;

public class TestFTInit {
    private ORB orb;

    private POA rootPOA;

    private com.thalesgroup.CdmwFT.ReplicationManager replicationManager;

    private String hostName1;

    private String hostName2;

    private String hostName3;

    private class AssertCounterImpl extends Cdmw.Test.AssertCounterPOA {

        private int nb_assert;

        public int nb_assert() {
            return this.nb_assert;
        }

        public void nb_assert(int nbAssert) {
            this.nb_assert = nbAssert;
        }
    }

    private void readLine() {
        try {
            java.io.BufferedReader inputReader = 
                new java.io.BufferedReader(
                    new java.io.InputStreamReader(System.in));
            inputReader.readLine();
        } catch (java.io.IOException e) {
            e.printStackTrace();
        }
    }

    // constructor
    public TestFTInit() {
        try {
            this.orb = cdmw.testutils.CORBATestManager.instance().getORB();
            this.rootPOA = cdmw.testutils.CORBATestManager.instance().getPOA();

            org.omg.CORBA.Object obj2 = cdmw.testutils.CORBATestManager.instance().getObject("replication_manager");
            this.replicationManager = com.thalesgroup.CdmwFT.ReplicationManagerHelper.narrow(obj2);
        }
        catch (InvalidObject e) {
            e.printStackTrace();
        }

        this.hostName1 = cdmw.testutils.CORBATestManager.instance().getProperty("host_name1");
        this.hostName2 = cdmw.testutils.CORBATestManager.instance().getProperty("host_name2");
        this.hostName3 = cdmw.testutils.CORBATestManager.instance().getProperty("host_name3");
    }

    // do_tests
    @Test
    public void doTests() 
        throws UserException,
            com.thalesgroup.CdmwNamingAndRepository.NoNameDomain,
            cdmw.commonsvcs.naming.InvalidNameException,
            org.omg.FT.MemberAlreadyPresent,
            InterruptedException 
    
    {
        // set_nbOfRequestedTestOK (28);

        String rep_id_HelloInterface1 = "IDL:thalesgroup.com/Cdmw/Test/HelloInterface1:1.0";
        String rep_id_HelloInterface2 = "IDL:thalesgroup.com/Cdmw/Test/HelloInterface2:1.0";

        //
        // define fault tolerance properties
        //
        org.omg.FT.Property[] prop = new org.omg.FT.Property[4];
        prop[0] = new org.omg.FT.Property();
        prop[1] = new org.omg.FT.Property();
        prop[2] = new org.omg.FT.Property();
        prop[3] = new org.omg.FT.Property();
        prop[0].val = this.orb.create_any();
        prop[1].val = this.orb.create_any();
        prop[2].val = this.orb.create_any();
        prop[3].val = this.orb.create_any();

        prop[0].nam = new NameComponent[1];
        prop[0].nam[0] = new NameComponent();
        prop[0].nam[0].id = "org.omg.ft.MinimumNumberReplicas";
        prop[0].nam[0].kind = "";
        prop[0].val.insert_ushort((short) 2);

        prop[1].nam = new NameComponent[1];
        prop[1].nam[0] = new NameComponent();
        prop[1].nam[0].id = "org.omg.ft.ReplicationStyle";
        prop[1].nam[0].kind = "";
        prop[1].val.insert_ushort((short) 2);

        prop[2].nam = new NameComponent[1];
        prop[2].nam[0] = new NameComponent();
        prop[2].nam[0].id = "org.omg.ft.MembershipStyle";
        prop[2].nam[0].kind = "";
        prop[2].val.insert_ushort((short) 0);

        prop[3].nam = new NameComponent[1];
        prop[3].nam[0] = new NameComponent();
        prop[3].nam[0].id = "org.omg.ft.Factories";
        prop[3].nam[0].kind = "";
        cdmw.ft.FactoryInfos factoryInfos1 = new cdmw.ft.FactoryInfos();
        factoryInfos1.add_ftLocation(this.hostName1, "APPL1", "P11");
        factoryInfos1.add_ftLocation(this.hostName2, "APPL2", "P21");
        factoryInfos1.add_ftLocation(this.hostName3, "APPL3", "P31");
        org.omg.FT.FactoryInfosHelper.insert(prop[3].val, factoryInfos1.get_factoryInfos());

        AnyHolder factoryCreationId1 = new AnyHolder();
        org.omg.FT.Property[] ftCriteria = new org.omg.FT.Property[1];
        ftCriteria[0] = new org.omg.FT.Property();
        ftCriteria[0].val = this.orb.create_any();
        

        ftCriteria[0].nam = new NameComponent[1];
        ftCriteria[0].nam[0] = new NameComponent();
        ftCriteria[0].nam[0].id = "org.omg.ft.FTProperties";
        ftCriteria[0].nam[0].kind = "";
        PropertiesHelper.insert(ftCriteria[0].val, prop);

        //
        // Create group HelloInterface1
        //
        System.out.println("Create group");
        org.omg.CORBA.Object obj_group_HelloInterface1 = null;

        try {
            obj_group_HelloInterface1 = this.replicationManager.create_object(
                rep_id_HelloInterface1, ftCriteria, factoryCreationId1);
        
        }
        catch (SystemException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e0) {
            e0.printStackTrace();
            Assert.assertTrue(false);
        }

        Cdmw.Test.HelloInterface1 hello_group_HelloInterface1 = Cdmw.Test.HelloInterface1Helper
            .narrow(obj_group_HelloInterface1);

        int timescale = cdmw.testutils.CORBATestManager.getTimescale();
        cdmw.ossupport.OS.sleep(timescale * 1000);

        // call the method on an empty object group
        // (REQ 160)
        try {
            System.out
                .println("Calling hello on HelloInterface on an empty object group...");
            hello_group_HelloInterface1.hello();
            Assert.assertTrue(false);
        }
        catch (org.omg.CORBA.TRANSIENT e) {
            System.out.println(e.toString());
        }
        catch (SystemException e0) {
            System.out.println(e0.toString());
            Assert.assertTrue(false);
        }

        cdmw.ossupport.OS.sleep(timescale * 1000);

        AssertCounterImpl assert_counter_servant = new AssertCounterImpl();
        Cdmw.Test.AssertCounter assert_counter_ref = assert_counter_servant._this(this.orb);

        com.thalesgroup.CdmwNamingAndRepository.Repository repository = cdmw.namingandrepository.RepositoryInterface
            .getRepository();

        cdmw.namingandrepository.RepositoryInterface.init("CDMW", repository);

        cdmw.commonsvcs.naming.NamingInterface ni = cdmw.namingandrepository.RepositoryInterface
            .getDomainNamingInterface("dom1/dom2");

        NameComponent[] loc = new NameComponent[3];
        loc[0] = new NameComponent();
        loc[1] = new NameComponent();
        loc[2] = new NameComponent();
        
        loc[0].id = this.hostName1;
        loc[0].kind = "hostname";
        loc[1].id = "APPL1";
        loc[1].kind = "applicationname";
        loc[2].id = "P11";
        loc[2].kind = "processname";

        String s_name = cdmw.commonsvcs.naming.NamingInterface.toString(loc);

        String fullName = "dom3/P11HelloInterface1";
        org.omg.CORBA.Object obj = null;

        try {
            obj = ni.resolve(fullName);
        }
        catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
            System.out.println(e.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e0) {
            System.out.println(e0.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.InvalidName e1) {
            System.out.println(e1.toString());
            Assert.assertTrue(false);
        }
        catch (SystemException e2) {
            System.out.println(e2.toString());
            Assert.assertTrue(false);
        }

        Cdmw.Test.HelloInterface1 hello_APPL1_PROC11_INTER1 = Cdmw.Test.HelloInterface1Helper
            .narrow(obj);

        // set assert_counter
        hello_APPL1_PROC11_INTER1.set_assert_counter(assert_counter_ref);

        // add a new member in the object group
        // (REQ 159)
        System.out.println("Add new member on the group");
        try {
            obj_group_HelloInterface1 = this.replicationManager.add_member(
                hello_group_HelloInterface1, loc, hello_APPL1_PROC11_INTER1);
            hello_group_HelloInterface1 = Cdmw.Test.HelloInterface1Helper
                .narrow(obj_group_HelloInterface1);
            cdmw.ossupport.OS.sleep(timescale * 1500);
        }
        catch (SystemException e) {
            Assert.assertTrue(false);
        }

        // add new member

        loc[0].id = this.hostName2;
        loc[0].kind = "hostname";
        loc[1].id = "APPL2";
        loc[1].kind = "applicationname";
        loc[2].id = "P21";
        loc[2].kind = "processname";

        s_name = cdmw.commonsvcs.naming.NamingInterface.toString(loc);

        fullName = "dom3/P21HelloInterface1";

        // recover the reference of the APPL2/PROC21/HelloInterface1 object
        try {
            obj = ni.resolve(fullName);
        }
        catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
            System.out.println(e.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e0) {
            System.out.println(e0.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.InvalidName e1) {
            System.out.println(e1.toString());
            Assert.assertTrue(false);
        }
        catch (SystemException e2) {
            System.out.println(e2.toString());
            Assert.assertTrue(false);
        }

        Cdmw.Test.HelloInterface1 hello_APPL2_PROC21_INTER1 = Cdmw.Test.HelloInterface1Helper
            .narrow(obj);

        // set assert_counter
        hello_APPL2_PROC21_INTER1.set_assert_counter(assert_counter_ref);

        // add a new member in the object group
        // (REQ 159)
        System.out.println("Add new member on the group");
        try {
            obj_group_HelloInterface1 = this.replicationManager.add_member(
                hello_group_HelloInterface1, loc, hello_APPL2_PROC21_INTER1);
            hello_group_HelloInterface1 = Cdmw.Test.HelloInterface1Helper
                .narrow(obj_group_HelloInterface1);
            cdmw.ossupport.OS.sleep(timescale * 1500);
        }
        catch (SystemException e) {
            Assert.assertTrue(false);
        }

        Cdmw.Test.HelloInterface1 hello_group_HelloInterface1_bis = Cdmw.Test.HelloInterface1Helper
            .narrow(obj_group_HelloInterface1);

        System.out.println("calling all object without the object group ref");
        Cdmw.Test.process_info hello_APPL1_PROC11_INTER1_info = hello_APPL1_PROC11_INTER1
            .who_is_called();
        Cdmw.Test.process_info hello_APPL2_PROC21_INTER1_info = hello_APPL2_PROC21_INTER1
            .who_is_called();

        System.out
            .println("----------------------------------------------------------------");
        System.out
            .println("----SYS------APPL1------P11------HelloInterface1------primary");
        System.out
            .println("     |                                                   PID : "
                + hello_APPL1_PROC11_INTER1_info.processID);
        System.out
            .println("     |-------APPL2------P21------HelloInterface1------backup ");
        System.out
            .println("                                                         PID : "
                + hello_APPL2_PROC21_INTER1_info.processID);
        System.out
            .println("----------------------------------------------------------------");

        // call the method on an object group with primary
        try {
            System.out.println("Calling hello on HelloInterface1...");
            hello_group_HelloInterface1.hello();
        }
        catch (SystemException e) {
            Assert.assertTrue(false);
        }

        System.out.println("kill the process P11 and P21");
        System.out.println("Press any key and Enter to continue");
        System.out.println("Press enter to stop the process");
        readLine();

        System.out.println("process P11 and P21 killed");
        cdmw.ossupport.OS.sleep(timescale * 2000);
        // add new member

        //loc = new NameComponent[3];
        loc[0].id = this.hostName3;
        loc[0].kind = "hostname";
        loc[1].id = "APPL3";
        loc[1].kind = "applicationname";
        loc[2].id = "P31";
        loc[2].kind = "processname";

        s_name = cdmw.commonsvcs.naming.NamingInterface.toString(loc);
        fullName = "dom3/P31HelloInterface1";
        // recover the reference of the APPL3/PROC31/HelloInterface1 object
        try {
            obj = ni.resolve(fullName);
        }
        catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
            System.out.println(e.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e0) {
            System.out.println(e0.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.InvalidName e1) {
            System.out.println(e1.toString());
            Assert.assertTrue(false);
        }
        catch (SystemException e2) {
            System.out.println(e2.toString());
            Assert.assertTrue(false);
        }

        Cdmw.Test.HelloInterface1 hello_APPL3_PROC31_INTER1 = Cdmw.Test.HelloInterface1Helper
            .narrow(obj);

        // set assert_counter
        hello_APPL3_PROC31_INTER1.set_assert_counter(assert_counter_ref);

        Cdmw.Test.process_info hello_APPL3_PROC31_INTER1_info = hello_APPL3_PROC31_INTER1
            .who_is_called();

        // add a new member in the object group
        // (REQ 159)
        System.out.println("Add new member on the group");
        try {
            obj_group_HelloInterface1 = this.replicationManager.add_member(
                hello_group_HelloInterface1, loc, hello_APPL3_PROC31_INTER1);
            cdmw.ossupport.OS.sleep(timescale * 1500);
        }
        catch (SystemException e) {
            Assert.assertTrue(false);
        }


        if (TestUtils.getObjectGroupVersion(this.orb, hello_group_HelloInterface1_bis) < 
            TestUtils.getObjectGroupVersion(this.orb, obj_group_HelloInterface1)) 
            Assert.assertTrue(true);
        else 
            Assert.assertTrue(false);


        // call the method on an object group with primary
        try {
            System.out.println("Calling hello on HelloInterface1... (on the old object)");
            Cdmw.Test.process_info temp = hello_group_HelloInterface1_bis
                .who_is_called();

            if (temp.processID == hello_APPL3_PROC31_INTER1_info.processID)
                Assert.assertTrue(true);
            else
                Assert.assertTrue(false);
        }
        catch (SystemException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }

        System.out.println("start again the process P11 and P21");

        System.out.println("Press any key and Enter to continue");
        readLine();

        System.out.println("process P11 and P21 started");
        cdmw.ossupport.OS.sleep(timescale * 4000);
        // add new member

        // loc = new NameComponent[3];
        loc[0].id = this.hostName1;
        loc[0].kind = "hostname";
        loc[1].id = "APPL1";
        loc[1].kind = "applicationname";
        loc[2].id = "P11";
        loc[2].kind = "processname";

        s_name = cdmw.commonsvcs.naming.NamingInterface.toString(loc);

        fullName = "dom3/P11HelloInterface1";

        // recover the reference of the APPL1/PROC11/HelloInterface1 object
        try {
            obj = ni.resolve(fullName);
        }
        catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
            System.out.println(e.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e0) {
            System.out.println(e0.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.InvalidName e1) {
            System.out.println(e1.toString());
            Assert.assertTrue(false);
        }
        catch (SystemException e2) {
            System.out.println(e2.toString());
            Assert.assertTrue(false);
        }

        hello_APPL1_PROC11_INTER1 = Cdmw.Test.HelloInterface1Helper
            .narrow(obj);

        // set assert_counter
        hello_APPL1_PROC11_INTER1.set_assert_counter(assert_counter_ref);

        // add a new member in the object group
        // (REQ 159)
        System.out.println("Add new member on the group");
        try {
            obj_group_HelloInterface1 = this.replicationManager.add_member(
                hello_group_HelloInterface1, loc, hello_APPL1_PROC11_INTER1);
            cdmw.ossupport.OS.sleep(timescale * 1500);
        }
        catch (org.omg.FT.ObjectGroupNotFound e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (org.omg.FT.MemberAlreadyPresent e0) {
            e0.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (org.omg.FT.ObjectNotAdded e1) {
            e1.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (SystemException e2) {
            e2.printStackTrace();
            Assert.assertTrue(false);
        }

        //loc = new NameComponent[3];
        loc[0].id = this.hostName2;
        loc[0].kind = "hostname";
        loc[1].id = "APPL2";
        loc[1].kind = "applicationname";
        loc[2].id = "P21";
        loc[2].kind = "processname";

        s_name = cdmw.commonsvcs.naming.NamingInterface.toString(loc);

        fullName = "dom3/P21HelloInterface1";

        // recover the reference of the APPL1/PROC21/HelloInterface1 object
        try {
            obj = ni.resolve(fullName);
        }
        catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
            System.out.println(e.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e0) {
            System.out.println(e0.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.InvalidName e1) {
            System.out.println(e1.toString());
            Assert.assertTrue(false);
        }
        catch (SystemException e2) {
            System.out.println(e2.toString());
            Assert.assertTrue(false);
        }

        hello_APPL2_PROC21_INTER1 = Cdmw.Test.HelloInterface1Helper
            .narrow(obj);

        // set assert_counter
        hello_APPL2_PROC21_INTER1.set_assert_counter(assert_counter_ref);

        // add a new member in the object group
        // (REQ 159)
        System.out.println("Add new member on the group");
        try {
            obj_group_HelloInterface1 = this.replicationManager.add_member(
                hello_group_HelloInterface1, loc, hello_APPL2_PROC21_INTER1);
            hello_group_HelloInterface1 = Cdmw.Test.HelloInterface1Helper
                .narrow(obj_group_HelloInterface1);
            cdmw.ossupport.OS.sleep(timescale * 1500);
        }
        catch (org.omg.FT.ObjectGroupNotFound e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (org.omg.FT.MemberAlreadyPresent e0) {
            e0.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (org.omg.FT.ObjectNotAdded e1) {
            e1.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (SystemException e2) {
            e2.printStackTrace();
            Assert.assertTrue(false);
        }

        // call the method on an object group with primary
        System.out
            .println("Check the COMM_FAILURE exception thrown by the ORB(the process server is aborted");
        try {
            System.out.println("Calling send_exception");
            hello_group_HelloInterface1.send_exception(10);
        }
        catch (SystemException e) {
            System.out.println(e.toString());
            Assert.assertTrue(false);
        }

        System.out.println("Press any key and Enter to continue");
        readLine();

        // remove a new menber on the group previously deleted
        System.out.println("Remove a member P21 from the helloInterface1");

        //loc = new NameComponent[3];
        loc[0].id = this.hostName2;
        loc[0].kind = "hostname";
        loc[1].id = "APPL2";
        loc[1].kind = "applicationname";
        loc[2].id = "P21";
        loc[2].kind = "processname";
        try {
            obj_group_HelloInterface1 = this.replicationManager.remove_member(
                hello_group_HelloInterface1, loc);
        }
        catch (SystemException e0) {
            System.out.println(e0.toString());
            Assert.assertTrue(false);
        }
        catch (UserException e1) {
            System.out.println(e1.toString());
            Assert.assertTrue(false);
        }

        // remove the object group
        System.out.println("Delete the object group helloInterface1");
        try {
            this.replicationManager.delete_object(factoryCreationId1.value);
        }
        catch (SystemException e0) {
            System.out.println(e0.toString());
            Assert.assertTrue(false);
        }
        catch (UserException e1) {
            System.out.println(e1.toString());
            Assert.assertTrue(false);
        }

        System.out.println("TEST 2");
        cdmw.ossupport.OS.sleep(timescale * 1000);

        System.out.println("Press any key and Enter to continue");
        readLine();

        //
        // Create group HelloInterface2
        //
        System.out.println("Create group");
        org.omg.CORBA.Object obj_group_HelloInterface2;
        AnyHolder factory_creation_id2 = new AnyHolder();

        cdmw.ft.FactoryInfos factoryInfos2 = new cdmw.ft.FactoryInfos();
        factoryInfos2.add_ftLocation(this.hostName1, "APPL1", "P12");
        factoryInfos2.add_ftLocation(this.hostName2, "APPL2", "P22");
        factoryInfos2.add_ftLocation(this.hostName3, "APPL3", "P32");
        org.omg.FT.FactoryInfosHelper.insert(prop[3].val, factoryInfos2.get_factoryInfos());

        PropertiesHelper.insert(ftCriteria[0].val, prop);

        try {
            obj_group_HelloInterface2 = this.replicationManager.create_object(
                rep_id_HelloInterface2, ftCriteria, factory_creation_id2);
        }
        catch (SystemException e) {
            e.printStackTrace();
            throw e;
        }
        catch (UserException e) {
            e.printStackTrace();
            throw e;
        }

        Cdmw.Test.HelloInterface2 hello_group_HelloInterface2 = Cdmw.Test.HelloInterface2Helper
            .narrow(obj_group_HelloInterface2);

        //loc = new NameComponent[3];
        loc[0].id = this.hostName2;
        loc[0].kind = "hostname";
        loc[1].id = "APPL2";
        loc[1].kind = "applicationname";
        loc[2].id = "P22";
        loc[2].kind = "processname";

        s_name = cdmw.commonsvcs.naming.NamingInterface.toString(loc);

        fullName = "dom3/P22HelloInterface2";

        // recover the reference of the APPL2/PROC22/HelloInterface2 object
        try {
            obj = ni.resolve(fullName);
        }
        catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
            System.out.println(e.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e0) {
            System.out.println(e0.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.InvalidName e1) {
            System.out.println(e1.toString());
            Assert.assertTrue(false);
        }
        catch (SystemException e2) {
            System.out.println(e2.toString());
            Assert.assertTrue(false);
        }

        Cdmw.Test.HelloInterface2 hello_APPL2_PROC22_INTER2 = Cdmw.Test.HelloInterface2Helper
            .narrow(obj);

        // add a new member in the object group
        // (REQ 159)
        System.out.println("Add new member on the group");
        cdmw.ossupport.OS.sleep(timescale * 1500);
        try {
            obj_group_HelloInterface2 = this.replicationManager.add_member(
                hello_group_HelloInterface2, loc, hello_APPL2_PROC22_INTER2);
            hello_group_HelloInterface2 = Cdmw.Test.HelloInterface2Helper
                .narrow(obj_group_HelloInterface2);
        }

        catch (SystemException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        cdmw.ossupport.OS.sleep(timescale * 1500);

        // add new member

        loc[0].id = this.hostName3;
        loc[0].kind = "hostname";
        loc[1].id = "APPL3";
        loc[1].kind = "applicationname";
        loc[2].id = "P32";
        loc[2].kind = "processname";

        s_name = cdmw.commonsvcs.naming.NamingInterface.toString(loc);

        fullName = "dom3/P32HelloInterface2";

        // recover the reference of the APPL2/PROC22/HelloInterface2 object
        try {
            obj = ni.resolve(fullName);
        }
        catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
            System.out.println(e.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e0) {
            System.out.println(e0.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.InvalidName e1) {
            System.out.println(e1.toString());
            Assert.assertTrue(false);
        }
        catch (SystemException e2) {
            System.out.println(e2.toString());
            Assert.assertTrue(false);
        }

        Cdmw.Test.HelloInterface2 hello_APPL3_PROC32_INTER2 = Cdmw.Test.HelloInterface2Helper
            .narrow(obj);

        // add a new member in the object group
        cdmw.ossupport.OS.sleep(timescale * 1500);
        // (REQ 159)
        System.out.println("Add new member on the group");
        try {
            obj_group_HelloInterface2 = this.replicationManager.add_member(
                hello_group_HelloInterface2, loc, hello_APPL3_PROC32_INTER2);
            hello_group_HelloInterface2 = Cdmw.Test.HelloInterface2Helper
                .narrow(obj_group_HelloInterface2);
        }
        catch (SystemException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }

        cdmw.ossupport.OS.sleep(timescale * 1500);

        // add new member

        loc[0].id = this.hostName1;
        loc[0].kind = "hostname";
        loc[1].id = "APPL1";
        loc[1].kind = "applicationname";
        loc[2].id = "P12";
        loc[2].kind = "processname";

        s_name = cdmw.commonsvcs.naming.NamingInterface.toString(loc);

        fullName = "dom3/P12HelloInterface2";

        // recover the reference of the APPL1/PROC12/HelloInterface2 object

        try {
            obj = ni.resolve(fullName);
        }
        catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
            System.out.println(e.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e0) {
            System.out.println(e0.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.InvalidName e1) {
            System.out.println(e1.toString());
            Assert.assertTrue(false);
        }
        catch (SystemException e2) {
            System.out.println(e2.toString());
            Assert.assertTrue(false);
        }

        Cdmw.Test.HelloInterface2 hello_APPL1_PROC12_INTER2 = Cdmw.Test.HelloInterface2Helper
            .narrow(obj);

        // add a new member in the object group
        // (REQ 159)
        System.out.println("Add new member on the group");
        cdmw.ossupport.OS.sleep(timescale * 1500);
        try {
            obj_group_HelloInterface2 = this.replicationManager.add_member(
                hello_group_HelloInterface2, loc, hello_APPL1_PROC12_INTER2);
            hello_group_HelloInterface2 = Cdmw.Test.HelloInterface2Helper
                .narrow(obj_group_HelloInterface2);
        }
        catch (SystemException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }

        cdmw.ossupport.OS.sleep(timescale * 1500);
        Cdmw.Test.process_info hello_APPL1_PROC12_INTER2_info = hello_APPL1_PROC12_INTER2
            .who_is_called();
        Cdmw.Test.process_info hello_APPL2_PROC22_INTER2_info = hello_APPL2_PROC22_INTER2
            .who_is_called();
        Cdmw.Test.process_info hello_APPL3_PROC32_INTER2_info = hello_APPL3_PROC32_INTER2
            .who_is_called();

        System.out
            .println("----------------------------------------------------------------");
        System.out
            .println("----SYS------APPL1------P12------HelloInterface2------backup ");
        System.out
            .println("     |                                                   PID : "
                + hello_APPL1_PROC12_INTER2_info.processID);
        System.out
            .println("     |-------APPL2------P22------HelloInterface2------primary ");
        System.out
            .println("     |                                                   PID : "
                + hello_APPL2_PROC22_INTER2_info.processID);
        System.out
            .println("     |-------APPL3------P32------HelloInterface2------backup");
        System.out
            .println("                                                         PID : "
                + hello_APPL3_PROC32_INTER2_info.processID);
        System.out
            .println("----------------------------------------------------------------");

        // (REQ 158)
        System.out
            .println("remove_member P12 (this request must be intercepted by an interceptor on the server side");
        System.out
            .println("=> the version on the location manager is lower than the Replication Manager");

        // remove the first member
        //loc = new NameComponent[3];
        loc[0].id = this.hostName1;
        loc[0].kind = "hostname";
        loc[1].id = "APPL1";
        loc[1].kind = "applicationname";
        loc[2].id = "P12";
        loc[2].kind = "processname";

        try {
            obj_group_HelloInterface2 = this.replicationManager.remove_member(
                hello_group_HelloInterface2, loc);
        }
        catch (SystemException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }

        System.out.println("calling all object without the object group ref");

        System.out
            .println("----------------------------------------------------------------");
        System.out
            .println("----SYS                                                         ");
        System.out
            .println("     |                                                          ");
        System.out
            .println("     |-------APPL2------P22------HelloInterface2------primary ");
        System.out
            .println("     |                                                   PID : "
                + hello_APPL2_PROC22_INTER2_info.processID);
        System.out
            .println("     |-------APPL3------P32------HelloInterface2------backup");
        System.out
            .println("                                                         PID : "
                + hello_APPL3_PROC32_INTER2_info.processID);
        System.out
            .println("----------------------------------------------------------------");

        // call the method on an object group with primary
        // the object group version in the location manager of the process P22
        // is lower than the
        // version of the replication manager
        try {
            System.out.println("Calling hello on HelloInterface2...");
            hello_group_HelloInterface2.hello();
            Assert.assertTrue(false);
        }
        catch (org.omg.CORBA.TRANSIENT e) {
            System.out.println(e.toString());
        }
        catch (SystemException e0) {
            e0.printStackTrace();
            Assert.assertTrue(false);
        }

        // remove the third member
        //loc = new NameComponent[3];
        loc[0].id = this.hostName3;
        loc[0].kind = "hostname";
        loc[1].id = "APPL3";
        loc[1].kind = "applicationname";
        loc[2].id = "P32";
        loc[2].kind = "processname";

        try {
            obj_group_HelloInterface2 = this.replicationManager.remove_member(
                hello_group_HelloInterface2, loc);
            hello_group_HelloInterface2 = Cdmw.Test.HelloInterface2Helper
                .narrow(obj_group_HelloInterface2);
        }
        catch (SystemException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }

        // remove the third member
        //loc = new NameComponent[3];
        loc[0].id = this.hostName2;
        loc[0].kind = "hostname";
        loc[1].id = "APPL2";
        loc[1].kind = "applicationname";
        loc[2].id = "P22";
        loc[2].kind = "processname";

        try {
            obj_group_HelloInterface2 = this.replicationManager.remove_member(
                hello_group_HelloInterface2, loc);
            hello_group_HelloInterface2 = Cdmw.Test.HelloInterface2Helper
                .narrow(obj_group_HelloInterface2);
        }
        catch (SystemException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }

        // remove the object group
        System.out.println("Delete the object group helloInterface2");
        try {
            this.replicationManager.delete_object(factory_creation_id2.value);
        }
        catch (SystemException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
    }
}
