/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.faulttoleranceclient2;

import org.omg.CORBA.*;
import org.omg.CosNaming.NameComponent;

public class TestUtils {
    private static int NO_PRIMARY = -1;
    
    
    private TestUtils() {}
    
    public static org.omg.CORBA.Object createGroup(ORB orb,
                                             com.thalesgroup.CdmwFT.ReplicationManager replicationManager,
                                             String repId)
        throws UserException
    {
        // set default properties
        System.out.println("set_default_properties");
        
        org.omg.FT.Property[] prop = new org.omg.FT.Property[4];

        prop[0].nam = new NameComponent[1];
        prop[0].nam[0].id="MinimumNumberReplicas";
        prop[0].val.insert_ushort((short)2);
        
        prop[1].nam = new NameComponent[1];
        prop[1].nam[0].id="ReplicationStyle";
        prop[1].val.insert_ushort((short)2);
        
        prop[2].nam = new NameComponent[1];
        prop[2].nam[0].id="MemberShipStyle";
        prop[2].val.insert_ushort((short)0);
        
        prop[3].nam = new NameComponent[1];
        prop[3].nam[0].id="FaultMonitoringStyle";
        prop[3].val.insert_ushort((short)0);
        

        Any factoryCreationId = null;
        org.omg.FT.Property[] ftCriteria = prop;
        org.omg.CORBA.Object objGroup;
        
        // create the object group 
        System.out.println("Create an object group");
        try {
            objGroup =  replicationManager.create_object(repId, ftCriteria,
                                                           new AnyHolder(factoryCreationId));
        }    
        catch( SystemException e ) {
            e.printStackTrace();
            throw e;
        }
        catch( UserException e0) {
            e0.printStackTrace();
            throw e0;
        }

        return objGroup;
    }

    public static int getObjectGroupVersion(ORB orb, org.omg.CORBA.Object object_group)
        throws UserException {

        //FT/ftcommon/IOGRFactory.cpp - NC!
        cdmw.ft.JacORBIOGRFactory iogrFactory = new cdmw.ft.JacORBIOGRFactory(orb);
        
        org.omg.FT.TagFTGroupTaggedComponent tag = iogrFactory.decode_profile_with_group_tag (object_group,0);
        
          return tag.object_group_ref_version;
                 
    }

    public static int getNumberOfMember(ORB orb, org.omg.CORBA.Object object_group)
        throws UserException {

        //      FT/ftcommon/IOGRFactory.cpp - NC!
        cdmw.ft.JacORBIOGRFactory iogrFactory = new cdmw.ft.JacORBIOGRFactory(orb);
        int  nbProfile = iogrFactory.profile_count(object_group);
        
        return nbProfile - 1;
    }

//    public static org.omg.CORBA.Object updateGroupWithPrimaryOutsideCdmwFt(ORB orb, 
//                              org.omg.CORBA.Object objectGroup,
//                              com.thalesgroup.CdmwFT.ReplicationManager replicationManager,
//                              NameComponent[] primaryLocation) throws UserException {
//        // get members of the group
//        GroupInfo groupInfo = replicationManager.get_object_group_info(objectGroup); 
//
//        // create object group
//        org.omg.CORBA.Object newObjectGroup = null;
//
//        org.omg.CORBA.Object[] noFallBack = new org.omg.CORBA.Object[8];     
//
//        //FT/ftcommon/IOGRFactory.cpp - NC!
//        com.thalesgroup.Cdmw.FT.IOGRFactory iogrFactory = new com.thalesgroup.Cdmw.FT.IOGRFactory();
//        
//        newObjectGroup = iogrFactory.build_iogr(groupInfo.members, primaryLocation, groupInfo.object_group_id,
//                                  "type_id", "My_FTdomain", groupInfo.object_group_ref_version, noFallBack);
//
//        return newObjectGroup;
//    }
}
