/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <exception>
#include <iostream>
#include <sstream>


#include "idllib/CdmwFTReplicationManager.stub.hpp"

#include "testftbackupinsertion/TestHello_impl.hpp"

#include "testftbackupinsertion/TestBackupInsertion.hpp"
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <FaultTolerance/ftlocationmanager/StatefullPrimaryBackupAdmin_impl.hpp>
#include "Repository/repositoryinterface/RepositoryInterface.hpp"
#include "Repository/idllib/CdmwNamingAndRepository.stub.hpp"

namespace Cdmw {
namespace FT {

class Modifier: public Cdmw::OsSupport::Thread
{
    public:
        Modifier(CdmwReplicationManager::HelloInterface_ptr hello)
            throw (OutOfMemoryException,
                   InternalErrorException,
                   BadParameterException);

        virtual ~Modifier() throw() {};

        virtual void run() throw();

    private:
        CdmwReplicationManager::HelloInterface_var m_hello;
};

CPPUNIT_TEST_SUITE_REGISTRATION( TestBackupInsertion );
// constructor
TestBackupInsertion::TestBackupInsertion()
{
    m_orb =  CORBA::ORB::_duplicate( Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());
    m_rootPOA = PortableServer::POA::_duplicate( Cdmw::TestUtils::CORBATestManager::instance()->get_POA());
    m_process_name = Cdmw::TestUtils::CORBATestManager::instance()->get_property ("process_name");
    m_host1 = Cdmw::TestUtils::CORBATestManager::instance()->get_property ("host_name1");
    m_host2 = Cdmw::TestUtils::CORBATestManager::instance()->get_property ("host_name2");
    m_host3 = Cdmw::TestUtils::CORBATestManager::instance()->get_property ("host_name3");
    
}

//destructor
TestBackupInsertion::~TestBackupInsertion()
{
}

// do_tests
void TestBackupInsertion::do_tests()
{
//     set_nbOfRequestedTestOK (12);

    try
    {
        TEST_INFO( "Resolve the Replication Manager reference" );

        CORBA::Object_var objref = m_orb->resolve_initial_references("ReplicationManager");
        CdmwFT::ReplicationManager_var rm = CdmwFT::ReplicationManager::_narrow(objref.in());
        
        if (CORBA::is_nil(rm.in()))
        {
            std::cout << "File : " << __FILE__ << " Line : " << __LINE__
                      << "could not _narrow object to type HelloInterface" << std::endl;
            abort();
        }
        
        CPPUNIT_ASSERT(true);


        
        TEST_INFO( "Resolve the Fault Notifier reference" );
        ::FT::FaultNotifier_var fault_notifier
            = rm->get_fault_notifier(); 
        
        CPPUNIT_ASSERT(true);


        
        // create the object groups
        TEST_INFO("Create objects groups");

        const char * rid1 = "IDL:thalesgroup.com/CdmwReplicationManager/HelloInterface:1.0";

        ::FT::GenericFactory::FactoryCreationId_var factory_creation_id1;

        CORBA::Object_var obj1;
        
        // set default properties
        TEST_INFO( "set_default_properties" );
    
        ::FT::Properties prop;
    
        prop.length(2);
        prop[0].nam.length(1);
        prop[0].nam[0].id="org.omg.ft.MinimumNumberReplicas";
        prop[0].val <<= (CORBA::UShort)1;
    

        std::vector<std::string> slocvect(2);
        slocvect[0] =  m_host1 +".hostname/APPL1.applicationname/P11.processname";
        slocvect[1] =  m_host2 +".hostname/APPL2.applicationname/P21.processname";

        ::FT::Locations locs(2);
        locs.length(2);
        locs[0].length(3);
        locs[0][0].id = m_host1.c_str();
        locs[0][0].kind = "hostname";
        locs[0][1].id = "APPL1";
        locs[0][1].kind = "applicationname";
        locs[0][2].id = "P11";
        locs[0][2].kind = "processname";

        locs[1].length(3);
        locs[1][0].id = m_host2.c_str();
        locs[1][0].kind = "hostname";
        locs[1][1].id = "APPL2";
        locs[1][1].kind = "applicationname";
        locs[1][2].id = "P21";
        locs[1][2].kind = "processname";

        const CORBA::ULong factory_infos_len = locs.length();
        ::FT::FactoryInfos factoryInfos;
        factoryInfos.length(factory_infos_len);
        for (CORBA::ULong i = 0; i < factory_infos_len; ++i) 
        {
            factoryInfos[i].the_factory = ::FT::GenericFactory::_nil();
            std::cout << '[' << i << "] " << " --- " 
                      << Cdmw::CommonSvcs::Naming::NamingInterface::to_string
                           (locs[i]) << std::endl;

            factoryInfos[i].the_location = locs[i];
            ::FT::Criteria factoryCrit;        
            factoryCrit.length(0);
            factoryInfos[i].the_criteria = factoryCrit;
        }

        prop[1].nam.length(1);
        prop[1].nam[0].id="org.omg.ft.Factories";
        prop[1].val <<= factoryInfos;

        ::CdmwFT::StateTransfer::DataStoreIDs dsids;
        dsids.length(1);
        dsids[0] = 1;

        ::FT::Criteria crit1;        
        crit1.length(2);
        crit1[0].nam.length(1);
        crit1[0].nam[0].id="org.omg.ft.FTProperties";
        crit1[0].val <<= prop;

        crit1[1].nam.length(1);
        crit1[1].nam[0].id="com.thalesgroup.cdmw.ft.DataStoreIdentifiers";
        crit1[1].val <<= dsids;


        try {
            obj1 =  rm->create_object(rid1,
                                      crit1,
                                      factory_creation_id1.out());

            CPPUNIT_ASSERT(true); 
        }
        catch( CORBA::Exception& e ) {
            std::cerr << e._name() << std::endl;
            CPPUNIT_ASSERT(false);
        }
        
        ::FT::ObjectGroupId ogid = 0;
        try {
            ogid = rm->get_object_group_id(obj1.in());
            CPPUNIT_ASSERT(true); 
        }
        catch( CORBA::Exception& e ) {
            std::cerr << e._name() << std::endl;
            CPPUNIT_ASSERT(false);
        }
        

        TEST_INFO("start processes");

        //
        // This validation test uses interceptors to induce a freeze timeout.
        // The interceptor implementation causes a sleep to occur after a given
        // number of next_n calls.
        //

        std::cout << "//" << std::endl;
        std::cout << "// ###########################################################" << std::endl;
        std::cout << "// ##### FIRST STEP" << std::endl;
        std::cout << "//" << std::endl;

        int timescale = Cdmw::TestUtils::get_timescale();
        OsSupport::OS::sleep(timescale*1000);

        CdmwNamingAndRepository::Repository_var repository =
                                  CdmwNamingAndRepository::Repository::_nil();

        CORBA::Object_var obj2 = m_orb->string_to_object("corbaloc::localhost:42562/CdmwRepository");
        repository = CdmwNamingAndRepository::Repository::_narrow(obj2.in());

        
        Cdmw::NamingAndRepository::RepositoryInterface::init ("CDMW",
                                                              repository.in());
        
        Cdmw::CommonSvcs::Naming::NamingInterface ni =
        Cdmw::NamingAndRepository::RepositoryInterface::get_domain_naming_interface ("dom1/dom2");
    

        std::string full_name = "dom3/P11HelloInterface";
        
        CORBA::Object_var obj;
        
        try
        {
            obj = ni.resolve (full_name);
            CPPUNIT_ASSERT(true);
        }
        catch(const CosNaming::NamingContext::NotFound& e)
        {
            std::cout<< e << std::endl;
            CPPUNIT_ASSERT(false);
        }
        catch(const CosNaming::NamingContext::CannotProceed& e)
        {
            std::cout<< e << std::endl;
            CPPUNIT_ASSERT(false);
        }
        catch(const CosNaming::NamingContext::InvalidName& e)
        {
            std::cout<< e << std::endl;
            CPPUNIT_ASSERT(false);
        }
        catch (const CORBA::SystemException& e)
        {
            std::cout<< e << std::endl;
            CPPUNIT_ASSERT(false);
        }

        TEST_INFO("resolved P11HelloInterface")        
        
        CdmwReplicationManager::HelloInterface_var helloPROC11 = CdmwReplicationManager::HelloInterface::_narrow(obj.in());
        CPPUNIT_ASSERT( ! CORBA::is_nil(helloPROC11.in()) );

        full_name = "dom3/P21HelloInterface";
        
        try
        {
            obj = ni.resolve (full_name);
            CPPUNIT_ASSERT(true);
        }
        catch(const CosNaming::NamingContext::NotFound& e)
        {
            std::cout<< e << std::endl;
            CPPUNIT_ASSERT(false);
        }
        catch(const CosNaming::NamingContext::CannotProceed& e)
        {
            std::cout<< e << std::endl;
            CPPUNIT_ASSERT(false);
        }
        catch(const CosNaming::NamingContext::InvalidName& e)
        {
            std::cout<< e << std::endl;
            CPPUNIT_ASSERT(false);
        }
        catch (const CORBA::SystemException& e)
        {
            std::cout<< e << std::endl;
            CPPUNIT_ASSERT(false);
        }
        
        TEST_INFO("resolved P21HelloInterface")

        CdmwReplicationManager::HelloInterface_var helloPROC21 = CdmwReplicationManager::HelloInterface::_narrow(obj.in());
        CPPUNIT_ASSERT( ! CORBA::is_nil(helloPROC21.in()) );

        // Add group members: PROC1 first, then PROC2 and 3. PROC1 become the primary process.
        TEST_INFO("[---- TestBackupInsertion::do_tests] add the member "<<m_host1.c_str()<<"/APPL1/P11 in group 1");
        try  {
            TEST_INFO("about to add_member helloPROC11")
            obj1 = rm->add_member(obj1.in(),
                                  locs[0],
                                  helloPROC11.in());
            TEST_INFO("after add_member")
            CPPUNIT_ASSERT(true);
        } catch( CORBA::Exception& e ) {
            std::cerr << e._name() << std::endl;
            std::cerr << " **** TEST FAILED AT LINE " << __LINE__ 
                      << " IN FILE: " << __FILE__ << std::endl; 
            CPPUNIT_ASSERT(false);
        }

        TEST_INFO("Set info into data store");
        try
        {
            for (int i = 0; i < 100; i++)
            {
                helloPROC11->insert(1, i, "one");
            }
            CPPUNIT_ASSERT(true);
        }
        catch(...)
        {
            CPPUNIT_ASSERT(false);
        }

        TEST_INFO("About to create Modifier")
        Modifier modifier(helloPROC11.in());
        TEST_INFO("About to start Modifier")
        modifier.start();
        TEST_INFO("After start Modifier")

        TEST_INFO("[---- TestBackupInsertion::do_tests] add the member "<<m_host2.c_str()<<"/APPL2/P21 in group 1");
        try  {
            TEST_INFO("about to add_member helloPROC21")
            obj1 = rm->add_member(obj1.in(),
                                  locs[1],
                                  helloPROC21.in());
            TEST_INFO("after add_member")
            CPPUNIT_ASSERT(false);
        } catch( CORBA::Exception& e ) {
            TEST_INFO("after add_member Exception")
            CPPUNIT_ASSERT(true);
        }

        OsSupport::OS::sleep(timescale*1000);

        TEST_INFO("[---- TestBackupInsertion::do_tests] store 11/\"one/one\": should failed");
        try {
            helloPROC21->insert(1, 11, "one/one");
            CPPUNIT_ASSERT(false);
        }
        catch(...) {
            CPPUNIT_ASSERT(true);
        }

        modifier.join();

        //
        // END FIRST STEP
        // ######################################################################
        
        Cdmw::NamingAndRepository::RepositoryInterface::finish();
        
    }
    catch(const CORBA::Exception& e ) {
        std::cerr << e._name() << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch( const Cdmw::Exception& e ) {
        std::cerr << e.what() << std::endl;
        CPPUNIT_ASSERT(false);
    }
}


Modifier::Modifier(CdmwReplicationManager::HelloInterface_ptr hello)
    throw (OutOfMemoryException,
           InternalErrorException,
           BadParameterException)
    : m_hello(CdmwReplicationManager::HelloInterface::_duplicate(hello))
{
}

void Modifier::run() throw()
{
    for (int i = 0; i < 100; i++)
    {
        m_hello->insert(1, i + 1000, "hello world");
        Cdmw::OsSupport::OS::sleep(500);
    }
}

}; // namespace FT
}; // namespace Cdmw

