/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <exception>
#include <iostream>
#include <sstream>


#include "idllib/CdmwFTReplicationManager.stub.hpp"

#include "testftperffailure/TestHello_impl.hpp"
//#include "testftlocationmanager2/TestFTActivationHandler_impl.hpp"
#include "FaultTolerance/idllib/CdmwFTManager.stub.hpp"

#include "testftperffailure/TestFTPerfFailure.hpp"
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include "Repository/repositoryinterface/RepositoryInterface.hpp"
#include "Repository/idllib/CdmwNamingAndRepository.stub.hpp"

namespace Cdmw {
namespace FT {


// constructor
CPPUNIT_TEST_SUITE_REGISTRATION( TestFTPerfFailure );

TestFTPerfFailure::TestFTPerfFailure()    
{
    m_orb =  CORBA::ORB::_duplicate( Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());
    m_rootPOA = PortableServer::POA::_duplicate( Cdmw::TestUtils::CORBATestManager::instance()->get_POA());
    m_process_name = Cdmw::TestUtils::CORBATestManager::instance()->get_property ("process_name");
    m_host1 = Cdmw::TestUtils::CORBATestManager::instance()->get_property ("host_name1");
    m_host2 = Cdmw::TestUtils::CORBATestManager::instance()->get_property ("host_name2");
    m_host3 = Cdmw::TestUtils::CORBATestManager::instance()->get_property ("host_name3");
    
}

//destructor
TestFTPerfFailure::~TestFTPerfFailure()
{
}




// do_tests
void TestFTPerfFailure::do_tests()
{
//     set_nbOfRequestedTestOK (212);

    try
    {

        CdmwNamingAndRepository::Repository_var repository =
        CdmwNamingAndRepository::Repository::_nil();
        
        CORBA::Object_var obj_repository = m_orb->string_to_object("corbaloc::localhost:42562/CdmwRepository");
        repository = CdmwNamingAndRepository::Repository::_narrow(obj_repository.in());
                
        
        Cdmw::NamingAndRepository::RepositoryInterface::init ("CDMW",
                                                                      repository.in());
        
        CdmwNamingAndRepository::NameDomain_var helloDomain =
        repository->resolve_name_domain ("dom1/dom2/dom3");
        
        CdmwNamingAndRepository::NameDomain::RegistrationId_var regId = helloDomain->new_name ("client");
        // register the object
        Cdmw::client_impl* client_obj = new Cdmw::client_impl ();
        ::FT::client_var client = client_obj->_this();
        helloDomain->register_object (regId.in(), client.in ());


        TEST_INFO( "Resolve the Replication Manager reference" );

        CORBA::Object_var objref = m_orb->resolve_initial_references("ReplicationManager");
        CdmwFT::ReplicationManager_var rm = CdmwFT::ReplicationManager::_narrow(objref.in());
        
        if (CORBA::is_nil(rm.in()))
        {
            std::cout << "File : " << __FILE__ << " Line : " << __LINE__
                      << "could not _narrow object to type HelloInterface" << std::endl;
            abort();
        }
        
        CPPUNIT_ASSERT(true);

        // create a first object with an invalid criteria
        // the invalid criteria is the FaultMonitoringStyle 
        TEST_INFO("Create objects groups");

        const char * rid = "IDL:thalesgroup.com/FT/server:1.0";

        ::FT::GenericFactory::FactoryCreationId_var factory_creation_id;

        CORBA::Object_var obj1;
        CORBA::Object_var obj2;
        
        // set default properties
        TEST_INFO( "set_default_properties" );
    
        ::FT::Properties prop;
    
        prop.length(2);
        prop[0].nam.length(1);
        prop[0].nam[0].id="org.omg.ft.MinimumNumberReplicas";
        prop[0].val <<= (CORBA::UShort)2;
    
        ::FT::Locations locs(2);
        locs.length(2);
        locs[0].length(3);
        locs[0][0].id = m_host1.c_str();
        locs[0][0].kind = "hostname";
        locs[0][1].id = "APPL1";
        locs[0][1].kind = "applicationname";
        locs[0][2].id = "P11";
        locs[0][2].kind = "processname";

        locs[1].length(3);
        locs[1][0].id = m_host2.c_str();
        locs[1][0].kind = "hostname";
        locs[1][1].id = "APPL2";
        locs[1][1].kind = "applicationname";
        locs[1][2].id = "P21";
        locs[1][2].kind = "processname";


        const CORBA::ULong factory_infos_len = locs.length();
        ::FT::FactoryInfos factoryInfos;
        factoryInfos.length(factory_infos_len);
        for (CORBA::ULong i = 0; i < factory_infos_len; ++i) 
        {
            factoryInfos[i].the_factory = ::FT::GenericFactory::_nil();
            std::cout << '[' << i << "] " << " --- " 
                      << Cdmw::CommonSvcs::Naming::NamingInterface::to_string
                           (locs[i]) << std::endl;

            factoryInfos[i].the_location = locs[i];
            ::FT::Criteria factoryCrit;        
            factoryCrit.length(0);
            factoryInfos[i].the_criteria = factoryCrit;
        }

        prop[1].nam.length(1);
        prop[1].nam[0].id="org.omg.ft.Factories";
        prop[1].val <<= factoryInfos;

       ::FT::Criteria crit;        
        crit.length(2);
        crit[0].nam.length(1);
        crit[0].nam[0].id="org.omg.ft.FTProperties";
        crit[0].val <<= prop;

        ::FT::Name OGName;
        OGName.length(1);
        OGName[0].id=CORBA::string_dup("ObjectTest");
                 
        crit[1].nam.length(1);
        crit[1].nam[0].id="com.thalesgroup.cdmw.ft.Name";
        crit[1].val <<= OGName;
     
        TEST_INFO( "create the group" );
        try
        {
            obj1 =  rm->create_object(rid,
                                      crit,
                                      factory_creation_id.out());

            CPPUNIT_ASSERT(true); 
        }
        catch( CORBA::Exception& e )
        {
            std::cerr << e._name() << std::endl;
            CPPUNIT_ASSERT(false);
        }


        ::FT::server_var server = ::FT::server::_narrow(obj1.in());
        int timescale = Cdmw::TestUtils::get_timescale();
        OsSupport::OS::sleep(timescale*1000);
       

        typedef std::vector< unsigned long > TimeVector;
        TimeVector time_vec;
        
        unsigned int nb_test = 100;
        
        for (unsigned int i = 0 ; i<nb_test; i++)
        {
            TEST_INFO("Start server processes");
            
            std::ostringstream proc_init11;
            std::ostringstream proc_init21;
            proc_init11<< " --proc-initialise APPL1 P11 " << m_host1;
            proc_init21<< " --proc-initialise APPL2 P21 " << m_host2;
            
            OsSupport::OS::create_process( "platform_admin.sh" , proc_init11.str());   
            OsSupport::OS::sleep(timescale*1000);
            OsSupport::OS::create_process( "platform_admin.sh" , proc_init21.str());   
            OsSupport::OS::sleep(timescale*1000);
            
            std::ostringstream proc_run11;
            std::ostringstream proc_run21;
            proc_run11<< " --proc-run APPL1 P11 " << m_host1;
            proc_run21<< " --proc-run APPL2 P21 " << m_host2;
            
            OsSupport::OS::create_process( "platform_admin.sh" , proc_run11.str());   
            OsSupport::OS::create_process( "platform_admin.sh" , proc_run21.str());
            OsSupport::OS::sleep(timescale*2000);
            
            TEST_INFO("Call kill primary on the primary")
            try
            {
                server->kill_primary();
            }
            catch(...)
            {
            }

            std::ostringstream proc_stop11;
            std::ostringstream proc_stop21;
            proc_stop11<< " --proc-stop APPL1 P11 " << m_host1;
            proc_stop21<< " --proc-stop APPL2 P21 " << m_host2;
            OsSupport::OS::create_process( "platform_admin.sh" , proc_stop11.str());   
            OsSupport::OS::create_process( "platform_admin.sh" , proc_stop21.str());
            OsSupport::OS::sleep(timescale*2000);
            
            TEST_INFO("test n°"<<i);
            TEST_INFO("elapse time : "<<client_obj->get_elapse()<<" ms");
            time_vec.push_back(client_obj->get_elapse());
            
            
        }

        if (time_vec.size() != nb_test)
            CPPUNIT_ASSERT(false);

        unsigned long max = time_vec[0];
        unsigned long min = time_vec[0];
        unsigned long ave = 0;
        int nm_test_over_the_limit = 0;
        
        for(unsigned int i = 0; i<nb_test; i++)
        {
            ave = time_vec[i] + ave;
            if (time_vec[i] > max)
                max = time_vec[i];
            if (time_vec[i] < min)
                min = time_vec[i];
            if (time_vec[i] > 500)
                nm_test_over_the_limit++;
        }
        
        ave = ave/nb_test;

        TEST_INFO("------------------------------");
        TEST_INFO("result : ");
        TEST_INFO("max : " <<max<< " ms");
        TEST_INFO("min : " <<min<< " ms");
        TEST_INFO("average : " <<ave<< " ms");
        TEST_INFO("nb case under 500 ms :"<<((nb_test - nm_test_over_the_limit)/nb_test)*100<<"%");
        TEST_INFO("------------------------------");
    
    }
    catch(const CORBA::Exception& e )
    {
        std::cerr << e._name() << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch( const Cdmw::Exception& e )
    {
        std::cerr << e.what() << std::endl;
        CPPUNIT_ASSERT(false);
    }
} // end do_test operation

}; // namespace FT
}; // namespace Cdmw

