/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>
#include <Repository/idllib/CdmwNamingAndRepository.stub.hpp>

#include <testfttimeoutrequest/TestTimeoutRequest.hpp>
#include <testfttimeoutrequest/HelloThread.hpp>

#include <testfttimeoutrequest/TestHello.stub.hpp>


#include <fstream>

namespace Cdmw 
{

namespace FT 
{
CPPUNIT_TEST_SUITE_REGISTRATION( TestTimeoutRequest );

// constructor
TestTimeoutRequest::TestTimeoutRequest()
{    
   
    m_orb =  
       CORBA::ORB::_duplicate( 
             Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());
    m_rootPOA = 
       PortableServer::POA::_duplicate( 
             Cdmw::TestUtils::CORBATestManager::instance()->get_POA());
    
    CORBA::Object_var obj4 = Cdmw::TestUtils::CORBATestManager::instance()->get_object("replication_manager");
    m_replication_manager = CdmwFT::ReplicationManager::_narrow(obj4.in());

    m_host1 = Cdmw::TestUtils::CORBATestManager::instance()->get_property ("host_name1");
    m_host2 = Cdmw::TestUtils::CORBATestManager::instance()->get_property ("host_name2");

}

// destructor
TestTimeoutRequest::~TestTimeoutRequest()
{
}

// do_tests
void TestTimeoutRequest::do_tests()
{
    int timescale = Cdmw::TestUtils::get_timescale();

    try 
    {
        Cdmw::OsSupport::OS::unlink("hello.stt");
        Cdmw::OsSupport::OS::unlink("hello.act");
        Cdmw::OsSupport::OS::unlink("hello.dct");
        Cdmw::OsSupport::OS::unlink("comput.stt");
        Cdmw::OsSupport::OS::unlink("comput.act");
        Cdmw::OsSupport::OS::unlink("comput.dct");
    }
    catch(...)
    {
    }

    // get the object groups
    CdmwFT::Test::HelloInterface_var hello_group;
    CdmwFT::Test::ComputInterface_var comput_group;
    try {
        CosNaming::Name_var hello_group_name =
            Cdmw::CommonSvcs::Naming::NamingInterface::to_name("hello_group");
        CORBA::Object_var obj = 
            m_replication_manager->get_object_group_ref_from_name(hello_group_name.in());
        hello_group = CdmwFT::Test::HelloInterface::_narrow(obj.in());

        CosNaming::Name_var comput_group_name =
            Cdmw::CommonSvcs::Naming::NamingInterface::to_name("comput_group");
        obj = 
            m_replication_manager->get_object_group_ref_from_name(comput_group_name.in());
        comput_group = CdmwFT::Test::ComputInterface::_narrow(obj.in());
    } catch (const CORBA::Exception& e) {
        std::cerr << Cdmw::OrbSupport::OrbSupport::exception_to_string(e) << std::endl;
        CPPUNIT_ASSERT(false);
    }

    // Locations for hello_group
    ::FT::Locations locs(2);
    locs.length(2);
    locs[0].length(3);
    locs[0][0].id = m_host1.c_str();
    locs[0][0].kind = "hostname";
    locs[0][1].id = "APPL1";
    locs[0][1].kind = "applicationname";
    locs[0][2].id = "P11";
    locs[0][2].kind = "processname";

    locs[1].length(3);
    locs[1][0].id = m_host2.c_str();
    locs[1][0].kind = "hostname";
    locs[1][1].id = "APPL3";
    locs[1][1].kind = "applicationname";
    locs[1][2].id = "P31";
    locs[1][2].kind = "processname";

    // Locations for comput_group
    ::FT::Locations locs1(2);
    locs1.length(2);

    locs1[0].length(3);
    locs1[0][0].id = m_host1.c_str();
    locs1[0][0].kind = "hostname";
    locs1[0][1].id = "APPL2";
    locs1[0][1].kind = "applicationname";
    locs1[0][2].id = "P21";
    locs1[0][2].kind = "processname";
    
    locs1[1].length(3);
    locs1[1][0].id = m_host2.c_str();
    locs1[1][0].kind = "hostname";
    locs1[1][1].id = "APPL4";
    locs1[1][1].kind = "applicationname";
    locs1[1][2].id = "P41";
    locs1[1][2].kind = "processname";

    std::cout << "// ###########################################################" << std::endl;
    TEST_INFO("client started");
    std::cout << "// ###########################################################" << std::endl;
    
    TEST_INFO("get repository");
    CdmwNamingAndRepository::Repository_var repository 
        = Cdmw::NamingAndRepository::RepositoryInterface::get_repository();
    
    TEST_INFO("init repository");
    Cdmw::NamingAndRepository::RepositoryInterface::init ("CDMW",
                                                          repository.in());
    
    TEST_INFO("get_domain_naming_interface dom1/dom2 ");
    Cdmw::CommonSvcs::Naming::NamingInterface ni =
    Cdmw::NamingAndRepository::RepositoryInterface::get_domain_naming_interface ("dom1/dom2");


    // Getting P11HelloInterface
    std::string full_name = "dom3/P11HelloInterface";

    CORBA::Object_var obj;
        
    try
    {
        TEST_INFO("resolve " << full_name);
        obj = ni.resolve (full_name);
        CPPUNIT_ASSERT(true);
    }
    catch(const CosNaming::NamingContext::NotFound& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch(const CosNaming::NamingContext::CannotProceed& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch(const CosNaming::NamingContext::InvalidName& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch (const CORBA::SystemException& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }

    CdmwFT::Test::HelloInterface_var helloP11 
        = CdmwFT::Test::HelloInterface::_narrow(obj.in());

    // Getting P31HelloInterface
    full_name = "dom3/P31HelloInterface";  // BH ca serait pas plutot ca ? : 
    try
    {
        TEST_INFO("resolve " << full_name);
        obj = ni.resolve (full_name);
        CPPUNIT_ASSERT(true);
    }
    catch(const CosNaming::NamingContext::NotFound& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch(const CosNaming::NamingContext::CannotProceed& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch(const CosNaming::NamingContext::InvalidName& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch (const CORBA::SystemException& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    
    CdmwFT::Test::HelloInterface_var helloP31 
        = CdmwFT::Test::HelloInterface::_narrow(obj.in());


    // Getting P21ComputInterface
    full_name = "dom3/P21ComputInterface"; 
    try
    {
        TEST_INFO("resolve " << full_name);
        obj = ni.resolve (full_name);
        CPPUNIT_ASSERT(true);
    }
    catch(const CosNaming::NamingContext::NotFound& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch(const CosNaming::NamingContext::CannotProceed& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch(const CosNaming::NamingContext::InvalidName& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch (const CORBA::SystemException& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    
    CdmwFT::Test::ComputInterface_var computP21
      = CdmwFT::Test::ComputInterface::_narrow(obj.in());

    // Getting P41ComputInterface
    full_name = "dom3/P41ComputInterface";
    try
    {
        TEST_INFO("resolve " << full_name);
        obj = ni.resolve (full_name);
        CPPUNIT_ASSERT(true);
    }
    catch(const CosNaming::NamingContext::NotFound& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch(const CosNaming::NamingContext::CannotProceed& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch(const CosNaming::NamingContext::InvalidName& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch (const CORBA::SystemException& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    
    CdmwFT::Test::ComputInterface_var computP41 
        = CdmwFT::Test::ComputInterface::_narrow(obj.in());

    // Add group members: P11 first, then P21, P11 is the primary 
    TEST_INFO("add the member "<<m_host1.c_str()<<"/APPL1/P11 in group hello");

    try  {
        CORBA::Object_var group 
            = m_replication_manager->add_member(hello_group.in(),
                                                locs[0],
                                                helloP11.in());
        hello_group = CdmwFT::Test::HelloInterface::_narrow(group.in());
        CPPUNIT_ASSERT(true);
    } catch( CORBA::Exception& e ) {
        std::cerr << e._name() << std::endl;
        std::cerr << " **** TEST FAILED AT LINE " << __LINE__ 
                  << " IN FILE: " << __FILE__ << std::endl; 
        CPPUNIT_ASSERT(false);
    }

    // Add group members: P11 first, then P21, P11 is the primary 
    TEST_INFO("add the member "<<m_host1.c_str()<<"/APPL2/P21 in group comput");

    try  {
        CORBA::Object_var group 
            = m_replication_manager->add_member(comput_group.in(),
                                                locs1[0],
                                                computP21.in());
        comput_group = CdmwFT::Test::ComputInterface::_narrow(group.in()); 
        CPPUNIT_ASSERT(true);
    } catch( CORBA::Exception& e ) {
        std::cerr << e._name() << std::endl;
        std::cerr << " **** TEST FAILED AT LINE " << __LINE__ 
                  << " IN FILE: " << __FILE__ << std::endl; 
        CPPUNIT_ASSERT(false);
    }

    TEST_INFO("add the member "<<m_host2.c_str()<<"/APPL3/P31 in group hello");
    try  {
        CORBA::Object_var group 
            = m_replication_manager->add_member(hello_group.in(),
                                                locs[1],
                                                helloP31.in());
        hello_group = CdmwFT::Test::HelloInterface::_narrow(group.in());
        CPPUNIT_ASSERT(true);
    } catch( CORBA::Exception& e ) {
        std::cerr << e._name() << std::endl;
        std::cerr << " **** TEST FAILED AT LINE " << __LINE__ 
                  << " IN FILE: " << __FILE__ << std::endl; 
        CPPUNIT_ASSERT(false);
    }

        
    TEST_INFO("add the member "<<m_host2.c_str()<<"/APPL4/P41 in group comput");
    try  {
        CORBA::Object_var group 
            = m_replication_manager->add_member(comput_group.in(),
                                                locs1[1],
                                                computP41.in());
        comput_group = CdmwFT::Test::ComputInterface::_narrow(group.in()); 
        CPPUNIT_ASSERT(true);
    } catch( CORBA::Exception& e ) {
        std::cerr << e._name() << std::endl;
        std::cerr << " **** TEST FAILED AT LINE " << __LINE__ 
                  << " IN FILE: " << __FILE__ << std::endl; 
        CPPUNIT_ASSERT(false);
    }

   /* Cdmw::TestUtils::CORBATestManager::instance()->add("hello_object_group", 
                                                       hello_group.in());
    Cdmw::TestUtils::CORBATestManager::instance()->add("comput_object_group",
                                                       comput_group.in());
    Cdmw::TestUtils::CORBATestManager::instance()->add("comput_object", 
                                                       computP21.in());*/

    OsSupport::OS::sleep(timescale*1000);

    // Start the HelloThread to make concurrent request
    std::cout << "Start the HelloThread to make concurrent request" << std::endl;
    Cdmw::Test::HelloThread hello_thread;
    hello_thread.start();


    try
    {
        TEST_INFO("Calling hello on HelloInterface on P11...");
        helloP11->hello(computP21.in(), 100);
        TEST_INFO("hello on HelloInterface on P11 OK...");
        CPPUNIT_ASSERT(true);  
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);
    }
    
    try
    {
        TEST_INFO("Calling send_TRANSIENT_exception on HelloInterface on P11...");
        helloP11->send_TRANSIENT_exception();
        CPPUNIT_ASSERT(false);  
    }
    catch (const CORBA::TRANSIENT& e)
    {
        TEST_INFO("Expected CORBA::TRANSIENT Exception raised");
        CPPUNIT_ASSERT(true);
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("Unexpected CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);  
    }

    try
    {
        TEST_INFO("Calling hello on HelloInterface group (primary on P11) and comput obj on P31 ...");
        hello_group->hello(computP21.in(), 100);
        CPPUNIT_ASSERT(true);  
    }
    catch (const CORBA::TRANSIENT& e)
    {
        TEST_INFO("Unexpected CORBA::TRANSIENT Exception raised " << e);
        CPPUNIT_ASSERT(false);
    }
    catch (const CORBA::COMM_FAILURE& e)
    {
        TEST_INFO("Unexpected CORBA::COMM_FAILURE Exception raised");
        CPPUNIT_ASSERT(false);  
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("Unexpected CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);  
    }

    try
    {
        TEST_INFO("Calling hello on HelloInterface on P11 ...");
        helloP11->hello(computP21.in(), 100);
        TEST_INFO("hello on HelloInterface on P11 OK...");
        CPPUNIT_ASSERT(true);  
    }
    catch (const CORBA::TRANSIENT& e)
    {
        TEST_INFO("Unexpected CORBA::TRANSIENT Exception raised " << e);
        CPPUNIT_ASSERT(false);
    }
    catch (const CORBA::COMM_FAILURE& e)
    {
        TEST_INFO("Unexpected CORBA::COMM_FAILURE Exception raised");
        CPPUNIT_ASSERT(false);  
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);
    }

    try
     {
         TEST_INFO("Calling hello on primary HelloInterface (primary on P11) and comput group (primary on P31) ...");
         helloP11->hello(comput_group.in(), 100);
         CPPUNIT_ASSERT(true);  
     }
     catch (const CORBA::TRANSIENT& e)
     {
         TEST_INFO("Unexpected CORBA::TRANSIENT Exception raised " << e);
         CPPUNIT_ASSERT(false);
     }
     catch (const CORBA::COMM_FAILURE& e)
     {
         TEST_INFO("Unexpected CORBA::COMM_FAILURE Exception raised");
         CPPUNIT_ASSERT(false);  
     }
     catch (const CORBA::SystemException& e)
     {
         TEST_INFO("Unexpected CORBA System Exception raised " << e);
         CPPUNIT_ASSERT(false);  
     }

     OsSupport::OS::sleep(timescale*1000);

     std::cout<<"Check P11, P21, P31 and P41 are alive"<<std::endl;
     std::cout<<"Then press any key and Enter"<<std::endl;
     std::string dummy;
     std::cin >> dummy;


     try
     {
         TEST_INFO("Calling hello on primary HelloInterface on P11 which take a long time to process...");
         helloP11->hello(computP21.in(), 3000);
         CPPUNIT_ASSERT(false);
     }
     catch (const CORBA::TRANSIENT& e)
     {
         TEST_INFO("Unexpected CORBA::TRANSIENT Exception raised " << e);
         CPPUNIT_ASSERT(false);
     }
     catch (const CORBA::COMM_FAILURE& e)
     {
         TEST_INFO("Expected CORBA::COMM_FAILURE Exception raised");
         CPPUNIT_ASSERT(true);  
     }
     catch (const CORBA::SystemException& e)
     {
         TEST_INFO("Unexpected CORBA System Exception raised " << e);
         CPPUNIT_ASSERT(false);  
     }
     
    std::cout<<"Check P21 is dead, P11, P31 and P41 are alive"<<std::endl;
    std::cout<<"Then press any key and Enter"<<std::endl;
    std::cin >> dummy;

    std::cout<<"Changing of primary location: remove member and add again P11"<<std::endl;
    std::cout<<"Start again P21"<<std::endl;
    std::cout<<"Then press any key and Enter"<<std::endl;
    std::cin >> dummy;

    // Add group members: P11 first, then P21, P11 is the primary 
    TEST_INFO("add the member "<<m_host1.c_str()<<"/APPL1/P11 in group hello");

    CdmwFT::Test::HelloInterface_var old_hello_group = hello_group;
    try  {
        CORBA::Object_var group 
            = m_replication_manager->remove_member(hello_group.in(),
                                                locs[0]);
        hello_group = CdmwFT::Test::HelloInterface::_narrow(group.in());
        CPPUNIT_ASSERT(true);
    } catch( CORBA::Exception& e ) {
        std::cerr << e._name() << std::endl;
        std::cerr << " **** TEST FAILED AT LINE " << __LINE__ 
                  << " IN FILE: " << __FILE__ << std::endl; 
        CPPUNIT_ASSERT(false);
    }

    try  {
        CORBA::Object_var group 
            = m_replication_manager->add_member(hello_group.in(),
                                                locs[0],
                                                helloP11.in());
        hello_group = CdmwFT::Test::HelloInterface::_narrow(group.in());
        CPPUNIT_ASSERT(true);
    } catch( CORBA::Exception& e ) {
        std::cerr << e._name() << std::endl;
        std::cerr << " **** TEST FAILED AT LINE " << __LINE__ 
                  << " IN FILE: " << __FILE__ << std::endl; 
        CPPUNIT_ASSERT(false);
    }
    OsSupport::OS::sleep(timescale*2000);

    try
    {
        TEST_INFO("Calling init on HelloInterface on P31...");

        helloP31->init(hello_group.in());
        CPPUNIT_ASSERT(true);  
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);
    }

    try
    {
        TEST_INFO("Calling hello on HelloInterface on old group ref: send_other is called...");
        old_hello_group->hello(comput_group.in(), 100);
        CPPUNIT_ASSERT(true);
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);
    }

    OsSupport::OS::sleep(timescale*2000);

    try
    {
        TEST_INFO("Calling hello on HelloInterface (primary on P31) and comput on P41...");
        hello_group->hello(computP41.in(), 100);
        CPPUNIT_ASSERT(true);
    }
    catch (const CORBA::TRANSIENT& e)
    {
        TEST_INFO("Unexpected CORBA::TRANSIENT Exception raised");
        CPPUNIT_ASSERT(false);  
    }
    catch (const CORBA::COMM_FAILURE& e)
    {
        TEST_INFO("Unxpected CORBA::COMM_FAILURE Exception raised");
        CPPUNIT_ASSERT(false);  
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);
    }

    try
    {
        TEST_INFO("Calling hello on HelloInterface (primary on P31) and comput on P41 which takes a long time to process...");
        hello_group->hello(computP41.in(), 3000);
        CPPUNIT_ASSERT(false);
    }
    catch (const CORBA::TRANSIENT& e)
    {
        TEST_INFO("Expected CORBA::TRANSIENT Exception raised");
        CPPUNIT_ASSERT(true);  
    }
    catch (const CORBA::COMM_FAILURE& e)
    {
        TEST_INFO("Unxpected CORBA::COMM_FAILURE Exception raised");
        CPPUNIT_ASSERT(false);  
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);
    }

    std::cout<<"Check P41 is dead, P11, P21 and P31 are alive"<<std::endl;
    std::cout<<"Then press any key and Enter"<<std::endl;
    std::cin >> dummy;

    // Getting P21ComputInterface again
    full_name = "dom3/P21ComputInterface"; 
    try
    {
        TEST_INFO("resolve " << full_name);
        obj = ni.resolve (full_name);
        CPPUNIT_ASSERT(true);
    }
    catch(const CosNaming::NamingContext::NotFound& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch(const CosNaming::NamingContext::CannotProceed& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch(const CosNaming::NamingContext::InvalidName& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch (const CORBA::SystemException& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    
    computP21
      = CdmwFT::Test::ComputInterface::_narrow(obj.in());

    try
    {
        TEST_INFO("Calling hello on HelloInterface on P11 ...");
        helloP11->hello(computP21.in(), 100);
        CPPUNIT_ASSERT(true);
    }
    catch (const CORBA::TRANSIENT& e)
    {
        TEST_INFO("Unexpected CORBA::TRANSIENT Exception raised");
        CPPUNIT_ASSERT(false);  
    }
    catch (const CORBA::COMM_FAILURE& e)
    {
        TEST_INFO("Unpected CORBA::COMM_FAILURE Exception raised");
        CPPUNIT_ASSERT(false);  
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("Unexpected CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);  
    }

    try
    {
        TEST_INFO("Calling hello on HelloInterface on P11 ...");
        helloP11->hello(computP21.in(), 100);
        CPPUNIT_ASSERT(true);
    }
    catch (const CORBA::TRANSIENT& e)
    {
        TEST_INFO("Unexpected CORBA::TRANSIENT Exception raised");
        CPPUNIT_ASSERT(false);
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("Unexpected CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);  
    }

    OsSupport::OS::sleep(timescale*2000);

    std::cout<<"Check P41 is dead, P11, P21 and P31 are alive"<<std::endl;
    std::cout<<"Then press any key and Enter"<<std::endl;
    std::cin >> dummy;


    try
    {
        Cdmw::OsSupport::OS::unlink("hello.stt");
        Cdmw::OsSupport::OS::unlink("hello.act");
        Cdmw::OsSupport::OS::unlink("hello.dct");
        Cdmw::OsSupport::OS::unlink("comput.stt");
    }
    catch (...)
    {
    }
}


} // end namespace FT

} // end namespace Cdmw

