/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package qualif.ft.testfttimeoutrequest;

import org.omg.CORBA.*;
import org.omg.CORBA.ORBPackage.InvalidName;
import cdmw.testutils.CORBATestManager.InvalidObject;

public class ClientProcessBehaviour extends cdmw.cdmwinit.ProcessControl {

    /**
     * The ORB.
     */
    private ORB orb;

    /**
     * The replication manager.
     */
    private com.thalesgroup.CdmwFT.ReplicationManager replicationMgr;

    /**
     * The test thread.
     */
    private TestTimeoutRequestThread testThread;

    /**
     * Purpose: <p> The constructor.
     */
    public ClientProcessBehaviour(
        ORB orb) {
        this.orb = orb;
    }

    /**
     * Purpose: <p> the behaviour for the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/initialise:1.0
     * operation
     */
    public void onInitialise(
        com.thalesgroup.CdmwPlatformMngtBase.StartupKind startup_kind) {
    }

    /**
     * Purpose: <p> the behaviour for the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/run:1.0 operation
     */
    @Override
    public void onRun()
        throws com.thalesgroup.CdmwPlatformMngt.ProcessDelegatePackage.NotReadyToRun {
        System.out.println("resolve_initial_references( ReplicationManager )");

        try {
            org.omg.CORBA.Object replication_mgr_obj = this.orb
                .resolve_initial_references("ReplicationManager");

            System.out.println("edcation_mgr_obj is " + replication_mgr_obj);
            this.replicationMgr = com.thalesgroup.CdmwFT.ReplicationManagerHelper
                .narrow(replication_mgr_obj);
            System.out.println("replicationMgr is " + this.replicationMgr);
        }
        catch (InvalidName e) {
            System.out.println("resolve_initial_references failed");
            e.printStackTrace();
        }
        // Launch the test in a thread
        this.testThread = new TestTimeoutRequestThread();

        try {
            cdmw.testutils.CORBATestManager.instance().add(
                "replication_manager", this.replicationMgr);
        }
        catch (InvalidObject ex) {
            System.out.println("add Failed");
            ex.printStackTrace();
        }

        this.testThread.start();
    }

    @Override
    public void onStop() {

        try {
            this.testThread.join();
        }
        catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

}
