/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package qualif.ft.testfttimeoutrequest;

import java.io.*;

import org.omg.CORBA.*;
import org.omg.FT.*;

import cdmw.orbsupport.ExceptionMinorCodes;
import cdmw.ossupport.OS;

import com.thalesgroup.CdmwFT.*;
import com.thalesgroup.CdmwFT.Test.*;
import com.thalesgroup.CdmwFT.Location.*;

public class ComputInterfaceImpl extends ComputInterfacePOA {

    private ORB orb;

    private String location;

    private ActivationHandler handler;

    /**
     * Purpose: <p> [Constructor description if necessary]
     * 
     * @param
     */
    public ComputInterfaceImpl(
        ORB orb, String location, ActivationHandler handler) {
        this.orb = orb;
        this.location = location;
        this.handler = handler;
    }

    //
    // IDL:thalesgroup.com/CdmwFT/Test/Comput/init:1.0
    //
    public void init(org.omg.CORBA.Object objectGroup) throws SystemException {
        try {
            org.omg.CORBA.Object obj = this.orb
                .resolve_initial_references("FTActivationManager");
            ActivationManager activManager = ActivationManagerHelper
                .narrow(obj);
            activManager.register_activation_handler(objectGroup, this.handler);
        }
        catch (ObjectGroupNotFound e) {
            System.err
                .println("ERROR: Impossible to register FTActivationHandler! Object Group not found!");
            throw new INTERNAL(ExceptionMinorCodes.INTERNAL,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
        catch (HandlerAlreadyExist e) {
            System.err
                .println("ERROR: Impossible to register FTActivationHandler! Handler already exists!");
            throw new INTERNAL(ExceptionMinorCodes.INTERNAL,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
        catch (org.omg.CORBA.ORBPackage.InvalidName e) {
            System.err
                .println("ERROR: Impossible to register FTActivationManager! Name is invalid!");
            throw new INTERNAL(ExceptionMinorCodes.INTERNAL,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
        catch (org.omg.CORBA.SystemException e) {
            System.err
                .println("ERROR: Impossible to register FTActivationHandler!");
            throw new INTERNAL(ExceptionMinorCodes.INTERNAL,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

    }

    //
    // IDL:thalesgroup.com/CdmwFT/Test/Comput/hello:1.0
    //
    public int get_result(int timeMs) throws SystemException {
        FileWriter fw = null;
        try {
            fw = new FileWriter("comput.stt");
            fw.write(this.location);
            fw.close();
        }
        catch (IOException e) {
            e.printStackTrace();
        }

        System.out.println(this.location + " <####### Comput " + timeMs
            + "  ######>");
        try {
            OS.sleep((int) timeMs);
        }
        catch (InterruptedException e) {
            e.printStackTrace();
        }
        System.out.println(this.location + " <####### End of Comput " + timeMs
            + "  ######>");

        try {
            System.out
                .println(this.location
                    + "ComputInterface_impl::get_result => resolve_initial_references FTCurrent");
            org.omg.CORBA.Object obj = this.orb
                .resolve_initial_references("FTCurrent");
            com.thalesgroup.CdmwFT.Current ftCurrent = com.thalesgroup.CdmwFT.CurrentHelper
                .narrow(obj);
            String ftDomainId = ftCurrent.get_ft_domain_id();
            System.out.println("    ft_domain_id = " + ftDomainId);

            long groupId = ftCurrent.get_object_group_id();
            System.out.println("    group_id = " + groupId);

            int groupRefVersion = ftCurrent.get_object_group_ref_version();
            System.out.println("    group_ref_version = " + groupRefVersion);

            String clientId = ftCurrent.get_client_id();
            System.out.println("    client_id = " + clientId);

            long retentionId = ftCurrent.get_retention_id();
            System.out.println("retention_id = " + retentionId);

            long expirationTime = ftCurrent.get_expiration_time();
            System.out.println("    expiration_time = " + expirationTime);
        }
        catch (org.omg.CORBA.ORBPackage.InvalidName e) {
            System.err
                .println("ComputInterfaceImpl.get_result => FTCurrent is an unknown ObjectID!");
            throw new INTERNAL(ExceptionMinorCodes.INTERNAL,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
        catch (com.thalesgroup.CdmwFT.CurrentPackage.NoContext e) {
            System.err
                .println("ComputInterfaceImpl.get_result => NoContext because the call is done on an IOR not an IOGR.");
            throw new INTERNAL(ExceptionMinorCodes.INTERNAL,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
        catch (SystemException e) {
            System.err
                .println("ComputInterfaceImpl.get_result => CORBA.SystemException raised : ");
            throw new INTERNAL(ExceptionMinorCodes.INTERNAL,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        return timeMs;
    }
}
