/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package qualif.ft.testfttimeoutrequest;

import java.io.FileWriter;
import java.io.IOException;

import org.omg.CORBA.*;

import cdmw.orbsupport.*;

import com.thalesgroup.CdmwFT.Location.*;
import com.thalesgroup.CdmwFT.Test.ComputInterface;
import com.thalesgroup.CdmwFT.Test.HelloInterfacePOA;

public class HelloInterfaceImpl extends HelloInterfacePOA {

    private ORB orb;

    private String location;

    private ActivationHandler activHandler;

    private int hello;

    public HelloInterfaceImpl(
        ORB orb, String location, ActivationHandler activHandler) {
        this.orb = orb;
        this.location = location;
        this.activHandler = activHandler;
        this.hello = 0;
    }

    //
    // IDL:thalesgroup.com/CdmwFT/Test/Hello/hello:1.0
    //
    public void hello(ComputInterface comput, int timeMS) {
        try {
            FileWriter fw = new FileWriter("hello.stt");
            fw.write(this.location);
            fw.close();
        }
        catch (IOException e) {
            e.printStackTrace();
        }
        this.hello++ ;
        System.out.println(" <####### HELLO " + this.hello + "  ######>");

        try {
            long result = comput.get_result(timeMS);
            System.out.println("result = " + result);
        }
        catch (SystemException e) {
            System.err.println(this.location
                + "get_result on comput raises a CORBA::SystemException: " + e);
        }

    }

    public void send_TRANSIENT_exception() {
        // TODO Auto-generated method stub
        throw new TRANSIENT(0, CompletionStatus.COMPLETED_MAYBE);
    }

    //
    // IDL:thalesgroup.com/CdmwFT/Test/Hello/init:1.0
    //
    public void init(org.omg.CORBA.Object objectGroup) throws SystemException {
        try {
            org.omg.CORBA.Object obj = this.orb
                .resolve_initial_references("FTActivationManager");
            ActivationManager activManager = ActivationManagerHelper
                .narrow(obj);
            activManager.register_activation_handler(objectGroup,
                this.activHandler);
        }
        catch (org.omg.FT.ObjectGroupNotFound e) {
            System.err
                .println("ERROR: Impossible to register FTActivationHandler! Object Group not found!");
            throw new INTERNAL(ExceptionMinorCodes.INTERNAL,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
        catch (HandlerAlreadyExist e) {
            System.err
                .println("ERROR: Impossible to register FTActivationHandler! Handler already exists!");
            throw new INTERNAL(ExceptionMinorCodes.INTERNAL,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
        catch (org.omg.CORBA.ORBPackage.InvalidName e) {
            System.err
                .println("ERROR: Impossible to register FTActivationManager! Name is invalid!");
            throw new INTERNAL(ExceptionMinorCodes.INTERNAL,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
        catch (org.omg.CORBA.SystemException e) {
            System.err
                .println("ERROR: Impossible to register FTActivationHandler!");
            throw new INTERNAL(ExceptionMinorCodes.INTERNAL,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
    }

}
