/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package qualif.ft.testfttimeoutrequest;

import org.junit.*;
import org.omg.CORBA.*;

import com.thalesgroup.CdmwNamingAndRepository.InvalidName;
import com.thalesgroup.CdmwNamingAndRepository.NoNameDomain;

import cdmw.testutils.CORBATestManager.InvalidObject;

public class HelloTest {
    private ORB orb;

    private com.thalesgroup.CdmwFT.Test.HelloInterface hello_obj;

    private com.thalesgroup.CdmwFT.Test.ComputInterface comput_obj;

    private com.thalesgroup.CdmwFT.Test.ComputInterface comput_group;

    public HelloTest() {
        try {
            this.orb = cdmw.testutils.CORBATestManager.instance().getORB();
        }
        catch (InvalidObject e3) {
            e3.printStackTrace();
        }

        System.out.println("get repository");
        com.thalesgroup.CdmwNamingAndRepository.Repository repository = cdmw.namingandrepository.RepositoryInterface
            .getRepository();

        cdmw.commonsvcs.naming.NamingInterface ni = null;
        try {
            System.out.println("init repository");
            cdmw.namingandrepository.RepositoryInterface.init("CDMW",
                repository);

            System.out.println("get_domain_naming_interface dom1/dom2 ");
            ni = cdmw.namingandrepository.RepositoryInterface
                .getDomainNamingInterface("dom1/dom2");
        }
        catch (NoNameDomain e3) {
            e3.printStackTrace();
        }
        catch (InvalidName e3) {
            e3.printStackTrace();
        }
        catch (SystemException e3) {
            e3.printStackTrace();
        }

        // Getting P11HelloInterface
        String fullName = "dom3/P11HelloInterface";

        org.omg.CORBA.Object obj = null;

        try {
            System.out.println("resolve " + fullName);
            obj = ni.resolve(fullName);
            Assert.assertTrue(true);
        }
        catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
            System.out.println(e);
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e0) {
            System.out.println(e0);
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.InvalidName e1) {
            System.out.println(e1);
            Assert.assertTrue(false);
        }
        catch (SystemException e2) {
            System.out.println(e2);
            Assert.assertTrue(false);
        }

        this.hello_obj = com.thalesgroup.CdmwFT.Test.HelloInterfaceHelper
            .narrow(obj);

        // Getting P31ComputInterface
        fullName = "dom3/P31ComputInterface";
        try {
            System.out.println("resolve " + fullName);
            obj = ni.resolve(fullName);
            Assert.assertTrue(true);
        }
        catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
            System.out.println(e);
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e0) {
            System.out.println(e0);
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.InvalidName e1) {
            System.out.println(e1);
            Assert.assertTrue(false);
        }
        catch (SystemException e2) {
            System.out.println(e2);
            Assert.assertTrue(false);
        }

        this.comput_obj = com.thalesgroup.CdmwFT.Test.ComputInterfaceHelper
            .narrow(obj);
        this.comput_group = this.comput_obj;

    }

    @Test
    public void doTests() {
        for (int i = 0; i < 5; i++ ) {
            try {
                System.out
                    .println("Calling hello on HelloInterface in Thread...");
                this.hello_obj.hello(this.comput_obj, 100);
                System.out.println("hello on HelloInterface in Thread OK...");
                Assert.assertTrue(true);
            }
            catch (SystemException e) {
                System.out.println("CORBA System Exception raised " + e);
                Assert.assertTrue(false);
            }
        }

        System.out.println("********************************************");
        System.out.println("            HelloTest stop");
        System.out.println("********************************************");

    } // End of do_test()
}
