/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package qualif.ft.testfttimeoutrequest;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;

import cdmw.orbsupport.StrategyList;

public class MainTest {

    public static void main(String[] args) {
        try {
            // Initialise FT service
            //cdmw.ftinit.FTServiceInit.init(args, true);
            //cdmw.ft.watchdog.WatchdogServiceInit.init(args);
            cdmw.cdmwinit.CdmwInterface.cdmwPreORBInit(args, "cdmw.generated.FTTimeoutClientCdmwInterface");
            // get host name
            String host_name1 = cdmw.ossupport.OS.getOptionValue(args,
                "--host1");
            // get host name
            String host_name2 = cdmw.ossupport.OS.getOptionValue(args,
                "--host2");

            // Initialises the ORB
            StrategyList strategyList = new StrategyList();
            strategyList.addORBThreaded();
            strategyList.addPOAThreadPerConnection();

            System.out.println(" cdmw_test_timeout_request_client: ORB_init");
            ORB orb = cdmw.orbsupport.ORBSupport.ORBInit(args, strategyList);

            // Get the root POA
            org.omg.CORBA.Object obj = orb
                .resolve_initial_references("RootPOA");
            POA rootPOA = POAHelper.narrow(obj);

            POAManager poaMgr = rootPOA.the_POAManager();
            System.out
                .println(" cdmw_test_timeout_request_client: activate POA");
            poaMgr.activate();

            // Do test
            cdmw.testutils.CORBATestManager.instance().init(args[0], orb,
                rootPOA);
            cdmw.testutils.CORBATestManager.instance().setProperty(
                "host_name1", host_name1);
            cdmw.testutils.CORBATestManager.instance().setProperty(
                "host_name2", host_name2);

            // creates the process behaviour
            ClientProcessBehaviour pProcessBehaviour = new ClientProcessBehaviour(
                orb);

            // init Cdmw
            System.out.println(" faulttoleraceclient: init Cdmw");
            cdmw.cdmwinit.CdmwInterface.cdmwInit(orb, args, pProcessBehaviour, "cdmw.generated.FTTimeoutClientCdmwInterface");
            // Enter in the main loop
            System.out.println("ORB run");
            orb.run();

            // orb->shutdown() is done by the stop() of the ProcessBehaviour
            cdmw.cdmwinit.CdmwInterface.cdmwCleanup(orb, "");
            cdmw.orbsupport.ORBSupport.ORBCleanup(orb);
            orb.destroy();
        }
        catch (SystemException e) {
            e.printStackTrace();
            System.exit(1);
        }
        catch (UserException e0) {
            e0.printStackTrace();
            System.exit(1);
        }
        catch (cdmw.common.Exception e1) {
            e1.printStackTrace();
            System.exit(1);
        }
        catch (Exception e2) {
            System.err.println("Unexpected exception");
            System.exit(1);
        }

        System.exit(0);
    }
}
