/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package qualif.ft.testfttimeoutrequest;

import java.io.IOException;

import org.junit.*;
import org.junit.runner.RunWith;
import org.omg.CORBA.*;
import org.omg.CosNaming.NameComponent;
import org.omg.PortableServer.*;

// import cdmw.ftwatchdog.test.testfttimeoutrequest.HelloTest;
import cdmw.testutils.CORBATestManager.InvalidObject;
import cdmw.testutils.threadtestengine.CdmwThreadedTestClassRunner;
import cdmw.testutils.threadtestengine.TestContext;
import cdmw.testutils.threadtestengine.TestProcessor;

@RunWith( CdmwThreadedTestClassRunner.class)
public class TestTimeoutRequest {
    /**
     * Field is used for obtaining TestProcessor class instance
     */
    private TestContext textContext;

    public static String CLIENT_ID = "TestTimeoutRequestClient";

    private ORB orb;

    private POA rootPOA;

    private com.thalesgroup.CdmwFT.ReplicationManager replication_manager;

    private String host1;

    private String host2;

    // constructor
    public TestTimeoutRequest() {
        try {
            this.orb = cdmw.testutils.CORBATestManager.instance().getORB();
            this.rootPOA = cdmw.testutils.CORBATestManager.instance().getPOA();

            org.omg.CORBA.Object obj4 = cdmw.testutils.CORBATestManager
                .instance().getObject("replication_manager");
            this.replication_manager = com.thalesgroup.CdmwFT.ReplicationManagerHelper
                .narrow(obj4);
        }
        catch (InvalidObject e) {
            e.printStackTrace();
        }

        this.host1 = cdmw.testutils.CORBATestManager.instance().getProperty(
            "host_name1");
        this.host2 = cdmw.testutils.CORBATestManager.instance().getProperty(
            "host_name2");
    }

    // do_tests
    @Test
    public void doTests() {
        int timescale = cdmw.testutils.CORBATestManager.getTimescale();

        try {
            cdmw.ossupport.OS.unlink("hello.stt");
            cdmw.ossupport.OS.unlink("hello.act");
            cdmw.ossupport.OS.unlink("hello.dct");
            cdmw.ossupport.OS.unlink("comput.stt");
            cdmw.ossupport.OS.unlink("comput.act");
            cdmw.ossupport.OS.unlink("comput.dct");
        }
        catch (Exception ex) {}

        // get the object groups
        com.thalesgroup.CdmwFT.Test.HelloInterface hello_group = null;
        com.thalesgroup.CdmwFT.Test.ComputInterface comput_group = null;
        try {
            org.omg.CosNaming.NameComponent[] hello_group_name =
                cdmw.commonsvcs.naming.NamingInterface.toName("hello_group");
            org.omg.CORBA.Object obj = 
                replication_manager.get_object_group_ref_from_name(hello_group_name);
            hello_group = 
                com.thalesgroup.CdmwFT.Test.HelloInterfaceHelper.narrow(obj);

            org.omg.CosNaming.NameComponent[] comput_group_name =
                cdmw.commonsvcs.naming.NamingInterface.toName("comput_group");
            obj = 
                replication_manager.get_object_group_ref_from_name(comput_group_name);
            comput_group = 
                com.thalesgroup.CdmwFT.Test.ComputInterfaceHelper.narrow(obj);
        } catch (Exception e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }

        // Locations for hello_group
        NameComponent[][] locs = new NameComponent[2][];
        locs[0] = new NameComponent[3];
        locs[0][0] = new NameComponent();
        locs[0][1] = new NameComponent();
        locs[0][2] = new NameComponent();
        
        locs[0][0].id = this.host1;
        locs[0][0].kind = "hostname";
        locs[0][1].id = "APPL1";
        locs[0][1].kind = "applicationname";
        locs[0][2].id = "P11";
        locs[0][2].kind = "processname";

        locs[1] = new NameComponent[3];
        locs[1][0] = new NameComponent();
        locs[1][1] = new NameComponent();
        locs[1][2] = new NameComponent();

        locs[1][0].id = this.host2;
        locs[1][0].kind = "hostname";
        locs[1][1].id = "APPL3";
        locs[1][1].kind = "applicationname";
        locs[1][2].id = "P31";
        locs[1][2].kind = "processname";
        
        // Locations for comput_group
        NameComponent[][] locs1 = new NameComponent[2][];
        locs1[0] = new NameComponent[3];
        locs1[0][0] = new NameComponent();
        locs1[0][1] = new NameComponent();
        locs1[0][2] = new NameComponent();
        
        locs1[0][0].id = this.host1;
        locs1[0][0].kind = "hostname";
        locs1[0][1].id = "APPL2";
        locs1[0][1].kind = "applicationname";
        locs1[0][2].id = "P21";
        locs1[0][2].kind = "processname";

        locs1[1] = new NameComponent[3];
        locs1[1][0] = new NameComponent();
        locs1[1][1] = new NameComponent();
        locs1[1][2] = new NameComponent();
        
        locs1[1][0].id = this.host2;
        locs1[1][0].kind = "hostname";
        locs1[1][1].id = "APPL4";
        locs1[1][1].kind = "applicationname";
        locs1[1][2].id = "P41";
        locs1[1][2].kind = "processname";
        
        System.out
            .println("// ###########################################################");
        System.out.println("client started");
        System.out
            .println("// ###########################################################");

        cdmw.commonsvcs.naming.NamingInterface ni = null;
        try {
            System.out.println("get repository");
            com.thalesgroup.CdmwNamingAndRepository.Repository repository = cdmw.namingandrepository.RepositoryInterface
                .getRepository();

            System.out.println("init repository");
            cdmw.namingandrepository.RepositoryInterface.init("CDMW",
                repository);

            System.out.println("get_domain_naming_interface dom1/dom2 ");
            ni = cdmw.namingandrepository.RepositoryInterface
                .getDomainNamingInterface("dom1/dom2");

        }
        catch (Exception e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }

        // Getting P11HelloInterface
        String full_name = "dom3/P11HelloInterface";

        org.omg.CORBA.Object obj = null;

        try {
            System.out.println("resolve " + full_name);
            obj = ni.resolve(full_name);
            Assert.assertTrue(true);
        }
        catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
            System.out.println(e.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e0) {
            System.out.println(e0.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.InvalidName e1) {
            System.out.println(e1.toString());
            Assert.assertTrue(false);
        }
        catch (SystemException e2) {
            System.out.println(e2.toString());
            Assert.assertTrue(false);
        }
        catch (UserException e3) {
            System.out.println(e3.toString());
            Assert.assertTrue(false);
        }

        com.thalesgroup.CdmwFT.Test.HelloInterface helloP11 = com.thalesgroup.CdmwFT.Test.HelloInterfaceHelper
            .narrow(obj);

        // Getting P31HelloInterface
        full_name = "dom3/P31HelloInterface"; // BH ca serait pas plutot ca ?
        // :
        try {
            System.out.println("resolve " + full_name);
            obj = ni.resolve(full_name);
            Assert.assertTrue(true);
        }
        catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
            System.out.println(e.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e0) {
            System.out.println(e0.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.InvalidName e1) {
            System.out.println(e1.toString());
            Assert.assertTrue(false);
        }
        catch (SystemException e2) {
            System.out.println(e2.toString());
            Assert.assertTrue(false);
        }
        catch (UserException e3) {
            System.out.println(e3.toString());
            Assert.assertTrue(false);
        }

        com.thalesgroup.CdmwFT.Test.HelloInterface helloP31 = com.thalesgroup.CdmwFT.Test.HelloInterfaceHelper
            .narrow(obj);

        // Getting P21ComputInterface
        full_name = "dom3/P21ComputInterface";
        try {
            System.out.println("resolve " + full_name);
            obj = ni.resolve(full_name);
            Assert.assertTrue(true);
        }
        catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
            System.out.println(e.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e0) {
            System.out.println(e0.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.InvalidName e1) {
            System.out.println(e1.toString());
            Assert.assertTrue(false);
        }
        catch (SystemException e2) {
            System.out.println(e2.toString());
            Assert.assertTrue(false);
        }
        catch (UserException e3) {
            System.out.println(e3.toString());
            Assert.assertTrue(false);
        }

        com.thalesgroup.CdmwFT.Test.ComputInterface computP21 = com.thalesgroup.CdmwFT.Test.ComputInterfaceHelper
            .narrow(obj);

        // Getting P41ComputInterface
        full_name = "dom3/P41ComputInterface";
        try {
            System.out.println("resolve " + full_name);
            obj = ni.resolve(full_name);
            Assert.assertTrue(true);
        }
        catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
            System.out.println(e.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e0) {
            System.out.println(e0.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.InvalidName e1) {
            System.out.println(e1.toString());
            Assert.assertTrue(false);
        }
        catch (SystemException e2) {
            System.out.println(e2.toString());
            Assert.assertTrue(false);
        }

        com.thalesgroup.CdmwFT.Test.ComputInterface computP41 = com.thalesgroup.CdmwFT.Test.ComputInterfaceHelper
            .narrow(obj);

        // Add group members: P11 first, then P21, P11 is the primary
        System.out.println("add the member " + this.host1
            + "/APPL1/P11 in group hello");

        try {
            org.omg.CORBA.Object group1 = this.replication_manager.add_member(
                hello_group, locs[0], helloP11);
            hello_group = com.thalesgroup.CdmwFT.Test.HelloInterfaceHelper
                .narrow(group1);
            Assert.assertTrue(true);
        }
        catch (SystemException e) {
            System.err.println(" **** TEST FAILED AT LINE ");
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e0) {
            System.err.println(" **** TEST FAILED AT LINE ");
            e0.printStackTrace();
            Assert.assertTrue(false);
        }

        // Add group members: P11 first, then P21, P11 is the primary
        System.out.println("add the member " + this.host1
            + "/APPL2/P21 in group comput");

        try {
            org.omg.CORBA.Object group2 = this.replication_manager.add_member(
                comput_group, locs1[0], computP21);
            comput_group = com.thalesgroup.CdmwFT.Test.ComputInterfaceHelper
                .narrow(group2);
            Assert.assertTrue(true);
        }
        catch (SystemException e) {
            System.err.println(" **** TEST FAILED AT LINE ");
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e0) {
            System.err.println(" **** TEST FAILED AT LINE ");
            e0.printStackTrace();
            Assert.assertTrue(false);
        }

        System.out.println("add the member " + this.host2
            + "/APPL3/P31 in group hello");
        try {
            org.omg.CORBA.Object group3 = this.replication_manager.add_member(
                hello_group, locs[1], helloP31);
            hello_group = com.thalesgroup.CdmwFT.Test.HelloInterfaceHelper
                .narrow(group3);
            Assert.assertTrue(true);
        }
        catch (SystemException e) {
            System.err.println(" **** TEST FAILED AT LINE ");
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e0) {
            System.err.println(" **** TEST FAILED AT LINE ");
            e0.printStackTrace();
            Assert.assertTrue(false);
        }

        System.out.println("add the member " + this.host2
            + "/APPL4/P41 in group comput");
        try {
            org.omg.CORBA.Object group4 = this.replication_manager.add_member(
                comput_group, locs1[1], computP41);
            comput_group = com.thalesgroup.CdmwFT.Test.ComputInterfaceHelper
                .narrow(group4);
            Assert.assertTrue(true);
        }
        catch (SystemException e) {
            System.err.println(" **** TEST FAILED AT LINE ");
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e0) {
            System.err.println(" **** TEST FAILED AT LINE ");
            e0.printStackTrace();
            Assert.assertTrue(false);
        }

        /*
         * Cdmw::TestUtils::CORBATestManager::instance()->add("hello_object_group",
         * hello_group.in());
         * Cdmw::TestUtils::CORBATestManager::instance()->add("comput_object_group",
         * comput_group.in());
         * Cdmw::TestUtils::CORBATestManager::instance()->add("comput_object",
         * computP21.in());
         */
        try {
            cdmw.ossupport.OS.sleep(timescale * 1000);
        }
        catch (InterruptedException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }

        // Start the HelloThread to make concurrent request
        System.out.println("Start the HelloThread to make concurrent request");

        // HelloThread hello_thread;
        // hello_thread.start();

        TestProcessor tpHello = this.textContext.getTestProcessor();
        tpHello.addTest(HelloTest.class);
        tpHello.runTests();

        try {
            System.out.println("Calling hello on HelloInterface on P11...");
            helloP11.hello(computP21, 100);
            System.out.println("hello on HelloInterface on P11 OK...");
            Assert.assertTrue(true);
        }
        catch (SystemException e) {
            System.out.println("CORBA System Exception raised " + e);
            Assert.assertTrue(false);
        }

        try {
            System.out
                .println("Calling send_TRANSIENT_exception on HelloInterface on P11...");
            helloP11.send_TRANSIENT_exception();
            Assert.assertTrue(false);
        }
        catch (TRANSIENT e) {
            System.out.println("Expected CORBA::TRANSIENT Exception raised");
            Assert.assertTrue(true);
        }
        catch (SystemException e0) {
            System.out
                .println("Unexpected CORBA System Exception raised " + e0);
            Assert.assertTrue(false);
        }

        try {
            System.out
                .println("Calling hello on HelloInterface group (primary on P11) and comput obj on P31 ...");
            hello_group.hello(computP21, 100);
            Assert.assertTrue(true);
        }
        catch (TRANSIENT e) {
            System.out.println("Unexpected CORBA::TRANSIENT Exception raised "
                + e);
            Assert.assertTrue(false);
        }
        catch (COMM_FAILURE e0) {
            System.out
                .println("Unexpected CORBA::COMM_FAILURE Exception raised");
            Assert.assertTrue(false);
        }
        catch (SystemException e1) {
            System.out
                .println("Unexpected CORBA System Exception raised " + e1);
            Assert.assertTrue(false);
        }

        try {
            System.out.println("Calling hello on HelloInterface on P11 ...");
            helloP11.hello(computP21, 100);
            System.out.println("hello on HelloInterface on P11 OK...");
            Assert.assertTrue(true);
        }
        catch (TRANSIENT e) {
            System.out.println("Unexpected CORBA::TRANSIENT Exception raised "
                + e);
            Assert.assertTrue(false);
        }
        catch (COMM_FAILURE e0) {
            System.out
                .println("Unexpected CORBA::COMM_FAILURE Exception raised");
            Assert.assertTrue(false);
        }
        catch (SystemException e1) {
            System.out.println("CORBA System Exception raised " + e1);
            Assert.assertTrue(false);
        }

        try {
            System.out
                .println("Calling hello on primary HelloInterface (primary on P11) and comput group (primary on P31) ...");
            helloP11.hello(comput_group, 100);
            Assert.assertTrue(true);
        }
        catch (TRANSIENT e) {
            System.out.println("Unexpected CORBA::TRANSIENT Exception raised "
                + e);
            Assert.assertTrue(false);
        }
        catch (COMM_FAILURE e0) {
            System.out
                .println("Unexpected CORBA::COMM_FAILURE Exception raised");
            Assert.assertTrue(false);
        }
        catch (SystemException e1) {
            System.out
                .println("Unexpected CORBA System Exception raised " + e1);
            Assert.assertTrue(false);
        }

        try {
            cdmw.ossupport.OS.sleep(timescale * 1000);
        }
        catch (InterruptedException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        System.out.println("Check P11, P21, P31 and P41 are alive");
        System.out.println("Then press any key and Enter");
        System.out.println("Press enter to stop the process");

        try {
            System.in.read();
        }
        catch (IOException e) {
            e.printStackTrace();
        }

        try {
            System.out
                .println("Calling hello on primary HelloInterface on P11 which take a long time to process...");
            helloP11.hello(computP21, 3000);
            Assert.assertTrue(false);
        }
        catch (TRANSIENT e) {
            System.out.println("Unexpected CORBA::TRANSIENT Exception raised "
                + e);
            Assert.assertTrue(false);
        }
        catch (COMM_FAILURE e0) {
            System.out.println("Expected CORBA::COMM_FAILURE Exception raised");
            Assert.assertTrue(true);
        }
        catch (SystemException e1) {
            System.out
                .println("Unexpected CORBA System Exception raised " + e1);
            Assert.assertTrue(false);
        }

        System.out.println("Check P21 is dead, P11, P31 and P41 are alive");
        System.out.println("Then press any key and Enter");
        try {
            System.in.read();
        }
        catch (IOException e) {
            e.printStackTrace();
        }

        System.out
            .println("Changing of primary location: remove member and add again P11");
        System.out.println("Start again P21");
        System.out.println("Then press any key and Enter");
        try {
            System.in.read();
        }
        catch (IOException e) {
            e.printStackTrace();
        }

        // Add group members: P11 first, then P21, P11 is the primary
        System.out.println("add the member " + this.host1
            + "/APPL1/P11 in group hello");

        com.thalesgroup.CdmwFT.Test.HelloInterface old_hello_group = hello_group;
        try {
            org.omg.CORBA.Object group5 = this.replication_manager
                .remove_member(hello_group, locs[0]);
            hello_group = com.thalesgroup.CdmwFT.Test.HelloInterfaceHelper
                .narrow(group5);
            Assert.assertTrue(true);
        }
        catch (SystemException e) {
            System.err.println(" **** TEST FAILED AT LINE ");
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e0) {
            System.err.println(" **** TEST FAILED AT LINE ");
            e0.printStackTrace();
            Assert.assertTrue(false);
        }

        try {
            org.omg.CORBA.Object group6 = this.replication_manager.add_member(
                hello_group, locs[0], helloP11);
            hello_group = com.thalesgroup.CdmwFT.Test.HelloInterfaceHelper
                .narrow(group6);
            Assert.assertTrue(true);
        }
        catch (SystemException e) {
            System.err.println(" **** TEST FAILED AT LINE ");
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (UserException e0) {
            System.err.println(" **** TEST FAILED AT LINE ");
            e0.printStackTrace();
            Assert.assertTrue(false);
        }

        try {
            cdmw.ossupport.OS.sleep(timescale * 2000);
        }
        catch (InterruptedException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        try {
            System.out.println("Calling init on HelloInterface on P31...");

            helloP31.init(hello_group);
            Assert.assertTrue(true);
        }
        catch (SystemException e) {
            System.out.println("CORBA System Exception raised " + e);
            Assert.assertTrue(false);
        }

        try {
            System.out
                .println("Calling hello on HelloInterface on old group ref: send_other is called...");
            old_hello_group.hello(comput_group, 100);
            Assert.assertTrue(true);
        }
        catch (SystemException e) {
            System.out.println("CORBA System Exception raised " + e);
            Assert.assertTrue(false);
        }

        try {
            cdmw.ossupport.OS.sleep(timescale * 2000);
        }
        catch (InterruptedException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }

        try {
            System.out
                .println("Calling hello on HelloInterface (primary on P31) and comput on P41...");
            hello_group.hello(computP41, 100);
            Assert.assertTrue(true);
        }
        catch (TRANSIENT e) {
            System.out.println("Unexpected CORBA::TRANSIENT Exception raised");
            Assert.assertTrue(false);
        }
        catch (COMM_FAILURE e0) {
            System.out
                .println("Unxpected CORBA::COMM_FAILURE Exception raised");
            Assert.assertTrue(false);
        }
        catch (SystemException e1) {
            System.out.println("CORBA System Exception raised " + e1);
            Assert.assertTrue(false);
        }

        try {
            System.out
                .println("Calling hello on HelloInterface (primary on P31) and comput on P41 which takes a long time to process...");
            hello_group.hello(computP41, 3000);
            Assert.assertTrue(false);
        }
        catch (TRANSIENT e) {
            System.out.println("Expected CORBA::TRANSIENT Exception raised");
            Assert.assertTrue(true);
        }
        catch (COMM_FAILURE e0) {
            System.out
                .println("Unxpected CORBA::COMM_FAILURE Exception raised");
            Assert.assertTrue(false);
        }
        catch (SystemException e1) {
            System.out.println("CORBA System Exception raised " + e1);
            Assert.assertTrue(false);
        }

        System.out.println("Check P41 is dead, P11, P21 and P31 are alive");
        System.out.println("Then press any key and Enter");
        try {
            System.in.read();
        }
        catch (IOException e) {
            e.printStackTrace();
        }

        // Getting P21ComputInterface again
        full_name = "dom3/P21ComputInterface";
        try {
            System.out.println("resolve " + full_name);
            obj = ni.resolve(full_name);
            Assert.assertTrue(true);
        }
        catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
            System.out.println(e.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e0) {
            System.out.println(e0.toString());
            Assert.assertTrue(false);
        }
        catch (org.omg.CosNaming.NamingContextPackage.InvalidName e1) {
            System.out.println(e1.toString());
            Assert.assertTrue(false);
        }
        catch (SystemException e2) {
            System.out.println(e2.toString());
            Assert.assertTrue(false);
        }

        computP21 = com.thalesgroup.CdmwFT.Test.ComputInterfaceHelper
            .narrow(obj);

        try {
            System.out.println("Calling hello on HelloInterface on P11 ...");
            helloP11.hello(computP21, 100);
            Assert.assertTrue(true);
        }
        catch (TRANSIENT e) {
            System.out.println("Unexpected CORBA::TRANSIENT Exception raised");
            System.out.println(e.toString());
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (COMM_FAILURE e0) {
            System.out.println("Unpected CORBA::COMM_FAILURE Exception raised");
            Assert.assertTrue(false);
        }
        catch (SystemException e1) {
            System.out
                .println("Unexpected CORBA System Exception raised " + e1);
            Assert.assertTrue(false);
        }

        try {
            System.out.println("Calling hello on HelloInterface on P11 ...");
            helloP11.hello(computP21, 100);
            Assert.assertTrue(true);
        }
        catch (TRANSIENT e) {
            System.out.println("Unexpected CORBA::TRANSIENT Exception raised");
            Assert.assertTrue(false);
        }
        catch (SystemException e0) {
            System.out
                .println("Unexpected CORBA System Exception raised " + e0);
            Assert.assertTrue(false);
        }

        try {
            cdmw.ossupport.OS.sleep(timescale * 2000);
        }
        catch (InterruptedException e) {
            e.printStackTrace();
            Assert.assertTrue(false);
        }

        System.out.println("Check P41 is dead, P11, P21 and P31 are alive");
        System.out.println("Then press any key and Enter");
        System.out.println("Press enter to stop the process");
        try {
            System.in.read();
        }
        catch (IOException e) {
            e.printStackTrace();
        }

        try {
            cdmw.ossupport.OS.unlink("hello.stt");
            cdmw.ossupport.OS.unlink("hello.act");
            cdmw.ossupport.OS.unlink("hello.dct");
            cdmw.ossupport.OS.unlink("comput.stt");
        }
        catch (Exception e) {}
    }
}