/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef _HELLO_SERVANT_LOCATOR_HPP
#define _HELLO_SERVANT_LOCATOR_HPP


#include <string>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <testtimeoutrequest/TestHello_impl.hpp>
#include <testtimeoutrequest/TestComput_impl.hpp>

#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>
#include <Repository/idllib/CdmwNamingAndRepository.stub.hpp>



class HelloServantLocator:
    public virtual PortableServer::ServantLocator
{
    public: 
    
        // Constructor
        HelloServantLocator(Cdmw::Test::HelloInterface_impl * hello_servant, 
                        Cdmw::Test::ComputInterface_impl * comput_servant, 
                        std::string process_name, 
                Cdmw::CommonSvcs::Naming::NamingInterface naming_interface) 
        : m_hello_servant(hello_servant),
          m_comput_servant(comput_servant),
          m_process_name(process_name),
          m_naming_interface(naming_interface)
          {};
       
        // Destructor
        virtual ~HelloServantLocator() {};
    
        // IDL:omg.org/PortableServer/ServantLocator/preinvoke()
        virtual PortableServer::Servant 
        preinvoke(const PortableServer::ObjectId& oid,
                  PortableServer::POA_ptr poa,
                  const char* operation,
                  PortableServer::ServantLocator::Cookie& the_cookie)
            throw (CORBA::SystemException, PortableServer::ForwardRequest);
        
        
        // IDL:omg.org/PortableServer/ServantLocator/postinvoke()
        virtual void 
        postinvoke(const PortableServer::ObjectId& oid,
                   PortableServer::POA_ptr poa,
                   const char* operation,
                   PortableServer::ServantLocator::Cookie the_cookie,
                   PortableServer::Servant the_servant)
            throw (CORBA::SystemException);
    
    
    private: 
        Cdmw::Test::HelloInterface_impl * m_hello_servant;
        Cdmw::Test::ComputInterface_impl * m_comput_servant;
        std::string m_process_name;
    Cdmw::CommonSvcs::Naming::NamingInterface m_naming_interface;
};



#endif // _HELLO_SERVANT_LOCATOR_HPP

