/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>
#include <Repository/idllib/CdmwNamingAndRepository.stub.hpp>

#include <testtimeoutrequest/TestTimeoutRequest.hpp>
#include <testtimeoutrequest/HelloThread.hpp>
#include <testtimeoutrequest/TestHelloNonFT.stub.hpp>


#include <fstream>

namespace Cdmw 
{

namespace FT 
{
CPPUNIT_TEST_SUITE_REGISTRATION( TestTimeoutRequest );

// constructor
TestTimeoutRequest::TestTimeoutRequest()
{    
    m_orb =  
       CORBA::ORB::_duplicate( 
             Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());
    m_rootPOA = 
       PortableServer::POA::_duplicate( 
             Cdmw::TestUtils::CORBATestManager::instance()->get_POA());
}

// destructor
TestTimeoutRequest::~TestTimeoutRequest()
{
}

// do_tests
void TestTimeoutRequest::do_tests()
{

    try 
    {
        Cdmw::OsSupport::OS::unlink("hello.stt");
        Cdmw::OsSupport::OS::unlink("comput.stt");
    }
    catch(...)
    {
    }

    // set default properties
    TEST_INFO( "set_default_properties" );


    std::cout << "// ###########################################################" << std::endl;
    TEST_INFO("client started");
    std::cout << "// ###########################################################" << std::endl;
    
    TEST_INFO("get repository");
    CdmwNamingAndRepository::Repository_var repository 
        = Cdmw::NamingAndRepository::RepositoryInterface::get_repository();
    
    TEST_INFO("init repository");
    Cdmw::NamingAndRepository::RepositoryInterface::init ("CDMW",
                                                          repository.in());
    
    TEST_INFO("get_domain_naming_interface dom1/dom2 ");
    Cdmw::CommonSvcs::Naming::NamingInterface ni =
    Cdmw::NamingAndRepository::RepositoryInterface::get_domain_naming_interface ("dom1/dom2");


    // Getting P11HelloInterface
    std::string full_name = "dom3/P11HelloInterface";

    CORBA::Object_var obj;
        
    try
    {
        TEST_INFO("resolve " << full_name);
        obj = ni.resolve (full_name);
        CPPUNIT_ASSERT(true);
    }
    catch(const CosNaming::NamingContext::NotFound& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch(const CosNaming::NamingContext::CannotProceed& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch(const CosNaming::NamingContext::InvalidName& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch (const CORBA::SystemException& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }

    CdmwFT::Test::HelloInterface_var helloP11 
        = CdmwFT::Test::HelloInterface::_narrow(obj.in());

    // Getting P31HelloInterface
    full_name = "dom3/P31HelloInterface";
    try
    {
        TEST_INFO("resolve " << full_name);
        obj = ni.resolve (full_name);
        CPPUNIT_ASSERT(true);
    }
    catch(const CosNaming::NamingContext::NotFound& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch(const CosNaming::NamingContext::CannotProceed& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch(const CosNaming::NamingContext::InvalidName& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch (const CORBA::SystemException& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    
    CdmwFT::Test::HelloInterface_var helloP31 
        = CdmwFT::Test::HelloInterface::_narrow(obj.in());


    // Getting P21ComputInterface
    full_name = "dom3/P21ComputInterface"; 
    try
    {
        TEST_INFO("resolve " << full_name);
        obj = ni.resolve (full_name);
        CPPUNIT_ASSERT(true);
    }
    catch(const CosNaming::NamingContext::NotFound& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch(const CosNaming::NamingContext::CannotProceed& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch(const CosNaming::NamingContext::InvalidName& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch (const CORBA::SystemException& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    
    CdmwFT::Test::ComputInterface_var computP21
      = CdmwFT::Test::ComputInterface::_narrow(obj.in());

    // Getting P41ComputInterface
    full_name = "dom3/P41ComputInterface";
    try
    {
        TEST_INFO("resolve " << full_name);
        obj = ni.resolve (full_name);
        CPPUNIT_ASSERT(true);
    }
    catch(const CosNaming::NamingContext::NotFound& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch(const CosNaming::NamingContext::CannotProceed& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch(const CosNaming::NamingContext::InvalidName& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch (const CORBA::SystemException& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    
    CdmwFT::Test::ComputInterface_var computP41 
        = CdmwFT::Test::ComputInterface::_narrow(obj.in());
    
    // Start the HelloThread to make concurrent request
    std::cout << "Start the HelloThread to make concurrent request" << std::endl;
    Cdmw::Test::HelloThread hello_thread;
    hello_thread.start();   

    try
    {
        TEST_INFO("Calling Hello obj on P11 and Comput obj on P21...");
        helloP11->hello(computP21.in(), 100);
        TEST_INFO("hello on P11 OK...");
        CPPUNIT_ASSERT(true);  
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);
    }
    
    try
    {
        TEST_INFO("Calling send_TRANSIENT_exception on HelloInterface on P11...");
        helloP11->send_TRANSIENT_exception();
        CPPUNIT_ASSERT(false);  
    }
    catch (const CORBA::TRANSIENT& e)
    {
        TEST_INFO("Expected CORBA::TRANSIENT Exception raised");
        CPPUNIT_ASSERT(true);
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("Unexpected CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);  
    }

    try
    {
        TEST_INFO("Calling Hello obj on P11 and Comput obj on P21 ...");
        helloP11->hello(computP21.in(), 200);
    TEST_INFO("hello on P11 OK...");
        CPPUNIT_ASSERT(true);  
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("Unexpected CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);  
    }

    
    try
    {
      TEST_INFO("Calling hello obj on P11 and Comput on P21. P11 should forward to P41.");
      helloP11->hello_forward(computP21.in(), 100);
      CPPUNIT_ASSERT(true);
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("Unexpected CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);  
    }
    


    try
    {
      TEST_INFO("Calling hello obj on P11 and comput obj on P21. An exception should be raised, the computation time of Comput obj is too long...");
      helloP11->hello(computP21.in(), 3000);
      CPPUNIT_ASSERT(false);
    }
    catch (const CORBA::TRANSIENT& e)
    {
      TEST_INFO("Unexpected CORBA::TRANSIENT Exception raised " << e);
      CPPUNIT_ASSERT(false);  
    }
    catch (const CORBA::COMM_FAILURE& e)
    {
      TEST_INFO("Expected CORBA::COMM_FAILURE Exception raised");
      std::cout<<"Check P21 is dead, P11, P31 and P41 are alive"<<std::endl;
      CPPUNIT_ASSERT(true);  
    }
    catch (const CORBA::SystemException& e)
    {
      TEST_INFO("Unexpected CORBA System Exception raised " << e);
      CPPUNIT_ASSERT(false);  
    }
    

    try
    {
        TEST_INFO("Calling hello obj on P11 and Comput on P41. This should prove that P11 is always alive! ");
        helloP11->hello(computP41.in(), 300);
    TEST_INFO("hello on P11 OK...");
    CPPUNIT_ASSERT(true); 
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);
    }

    try
    {
        TEST_INFO("Calling hello obj on 31 and Comput obj on P41...");
        helloP31->hello(computP41.in(), 300);
    TEST_INFO("hello on P31 OK...");
    CPPUNIT_ASSERT(true); 
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);
    }

    try
    {
        
        TEST_INFO("Calling hello obj on P31 and Comput obj on P41. An exception should be raised, the computation time of Comput obj is too long...");
        helloP31->hello(computP41.in(), 3000);
        CPPUNIT_ASSERT(false);
    }
    catch (const CORBA::TRANSIENT& e)
     {
         TEST_INFO("Unexpected CORBA::TRANSIENT Exception raised " << e);
         CPPUNIT_ASSERT(false);  
     }
     catch (const CORBA::COMM_FAILURE& e)
     {
         TEST_INFO("Expected CORBA::COMM_FAILURE Exception raised");
         std::cout<<"Check P41 has just been dead. P11 and P31 are the only one to be alive"<<std::endl;
         CPPUNIT_ASSERT(true);  
     }
     catch (const CORBA::SystemException& e)
     {
         TEST_INFO("Unexpected CORBA System Exception raised " << e);
         CPPUNIT_ASSERT(false);  
     }

    try
    {
        Cdmw::OsSupport::OS::unlink("hello.stt");
        Cdmw::OsSupport::OS::unlink("comput.stt");
    }
    catch (...)
    {
    }

}


} // end namespace FT

} // end namespace Cdmw



