/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <fstream>

// Cdmw Files
#include <Foundation/common/Options.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>
#include <Repository/idllib/CdmwNamingAndRepository.stub.hpp>
#include <SystemMngt/platforminterface/PlatformInterface.hpp>

#include <FaultTolerance/ftinit/FTServiceInit.hpp>

#include <ConfAndPlug/cdmwinit/CdmwInterface.hpp>

#include <testtimeoutrequest/HelloServantLocator.hpp>
#include <testtimeoutrequest/TestHello_impl.hpp>
#include <testtimeoutrequest/TestComput_impl.hpp>



using namespace std;

namespace 
{

    const int POA_THREAD_POOL_SIZE = 5;   

} // end anonymous namespace


int main( int argc, char* argv[] )
{
    std::string host_name = Cdmw::OsSupport::OS::get_hostname();

    CORBA::ORB_var orb = CORBA::ORB::_nil();

    try
    {
        // Initialise FT service
        std::cout <<  host_name 
                  <<  "test_server: initialise FT services" << std::endl;
        Cdmw::CdmwInit::CDMW_preORBInit(argc, argv);

        // Initialises the ORB
        Cdmw::OrbSupport::StrategyList strategyList;
        strategyList.add_OrbThreaded();
        strategyList.add_PoaThreadPool(POA_THREAD_POOL_SIZE);

        std::cout << host_name 
                  << " test_server: initialises the ORB" 
                  << std::endl;
        orb= Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, strategyList);
  
  
          // get root POA            
        std::cout << host_name << " test_server: get the root POA" << std::endl;
        CORBA::Object_var obj = orb->resolve_initial_references( "RootPOA" );
        PortableServer::POA_var rootPOA
            = PortableServer::POA::_narrow( obj.in() );
        PortableServer::POAManager_var poaMgr = rootPOA->the_POAManager();
            
        // create a POA with USE_SERVANT_MANAGER & NON_RETAIN policy (to use ServantLocator)
        CORBA::PolicyList policies;
        policies.length(3);
        policies[0] = rootPOA->create_request_processing_policy(PortableServer::USE_SERVANT_MANAGER); 
        policies[1] = rootPOA->create_servant_retention_policy(PortableServer::NON_RETAIN); 
        policies[2] = rootPOA->create_id_assignment_policy(PortableServer::USER_ID); 
        
        PortableServer::POA_var myPOA = 
            Cdmw::OrbSupport::OrbSupport::create_POA(rootPOA.in(),
                                                      "helloPOA",
                                                      poaMgr.in(),
                                                      policies,
                                                      strategyList);
        


        int timescale = Cdmw::TestUtils::get_timescale();
        Cdmw::OsSupport::OS::sleep(timescale*1000);

        // init Cdmw
        std::cout << host_name << " test_server: init Cdmw"<<std::endl;
        Cdmw::CdmwInit::CDMW_init(orb.in(), argc, argv);


        // get process name
        std::string process_name = Cdmw::PlatformMngt::PlatformInterface::Get_process_name();

    
        // create HelloInterface object     
        std::cout << host_name << " test_server: create HelloInterface_impl"<<std::endl;
        Cdmw::Test::HelloInterface_impl * hello_interface 
            = new Cdmw::Test::HelloInterface_impl(orb.in(), host_name);
        PortableServer::ObjectId_var hello_oid = PortableServer::string_to_ObjectId("Hello");
        CORBA::Object_var hello_obj = 
            myPOA->create_reference_with_id(hello_oid, "IDL:thalesgroup.com/CdmwFT/Test/HelloInterface:1.0");
        CdmwFT::Test::HelloInterface_var hello_interface_ref = CdmwFT::Test::HelloInterface::_narrow(hello_obj.in());
                    

        // create ComputInterface object    
        std::cout << host_name << " test_server: create ComputInterface_impl"<<std::endl;   
        Cdmw::Test::ComputInterface_impl * comput_interface 
            = new Cdmw::Test::ComputInterface_impl(orb.in(), host_name);
        PortableServer::ObjectId_var comput_oid = PortableServer::string_to_ObjectId("Comput");
        CORBA::Object_var comput_obj = 
            myPOA->create_reference_with_id(comput_oid, "IDL:thalesgroup.com/CdmwFT/Test/ComputInterface:1.0");
        CdmwFT::Test::ComputInterface_var comput_interface_ref = CdmwFT::Test::ComputInterface::_narrow(comput_obj.in());
    
        
        // export the object reference to the CDMW Repository
        std::cout << host_name << " test_server: export the hello object reference to CDMW Repository"
                  << std::endl;
        CdmwNamingAndRepository::Repository_var repository
            = Cdmw::NamingAndRepository::RepositoryInterface::get_repository();
     
        Cdmw::NamingAndRepository::RepositoryInterface::init ("CDMW",
                                                              repository.in());
                                  
    
       Cdmw::CommonSvcs::Naming::NamingInterface ni =
       Cdmw::NamingAndRepository::RepositoryInterface::get_domain_naming_interface ("dom1/dom2");   
    
        // create ServantLocator
        std::cout << host_name << " test_server: create ServantLocator"<<std::endl; 
        HelloServantLocator* m_servant_locator = new HelloServantLocator(hello_interface,
                            comput_interface,
                            process_name,
                            ni);

        // set ServantLocator to helloPOA
        try {
            myPOA->set_servant_manager(m_servant_locator);
        } catch (const PortableServer::POA::WrongPolicy& e) {
            // shouldn't happen
            cerr <<__FILE__<<" "<<__LINE__<<" "<< e._name() << endl;
           return EXIT_FAILURE;     
        }       
            
    // activate POAManager
        std::cout << "  -> Activate POAManager " << std::endl;
        poaMgr->activate();                           
                                  
    
        CdmwNamingAndRepository::NameDomain_var helloDomain
            = repository->resolve_name_domain ("dom1/dom2/dom3");
     

        std::string hello_name = process_name + "HelloInterface";
        std::cout << host_name << " test_server: register " + hello_name << std::endl;

        try
        {
            helloDomain->release_name(hello_name.c_str());
        }
        catch (const CdmwNamingAndRepository::NameDomain::NotRegistered& e)
        {
            // not a problem it's the first time the server is launched
            cerr <<__FILE__<<" "<<__LINE__<<" "<< e._name() << " OK" << endl;
        }
        catch (const CdmwNamingAndRepository::InvalidName& e)
        {
            cerr <<__FILE__<<" "<<__LINE__<<" "<< e._name() << endl;
            return EXIT_FAILURE;

        }
        catch (const CORBA::SystemException& e)
        {
            std::cerr <<__FILE__<<" "<<__LINE__ 
                      << "ERROR: ------------------------"
                      << std::endl;
            std::cerr << e << std::endl;
            return EXIT_FAILURE;
        }

        try
        {
            CdmwNamingAndRepository::NameDomain::RegistrationId_var regId 
                = helloDomain->new_name (hello_name.c_str());
    
            // register the object
            helloDomain->register_object (regId.in(), hello_interface_ref.in ());
        }
        catch (const CORBA::SystemException& e)
        {
            std::cerr <<__FILE__<<" "<<__LINE__ 
                      <<"ERROR: ------------------------"
                      << std::endl;
            std::cerr << e << std::endl;
            return EXIT_FAILURE;
        }
        catch( const CdmwNamingAndRepository::NameDomain::AlreadyExists &e )
        {
           cerr <<__FILE__<<" "<<__LINE__<<" "<< e._name() << endl;
           return EXIT_FAILURE;
        }

        // export the object reference to a CDMW Repository
        std::cout << host_name 
                  << " test_server: export the comput object reference to a CDMW Repository"
                  << std::endl;
        std::string comput_name = process_name + "ComputInterface";
        std::cout << host_name << " test_server: register " + comput_name << std::endl;

        try 
        {
            helloDomain->release_name(comput_name.c_str());
        }
        catch (const CdmwNamingAndRepository::NameDomain::NotRegistered& e)
        {
            // not a problem it's the first time the server is launched
            cerr <<__FILE__<<" "<<__LINE__<<" "<< e._name() << " OK" << endl;

        }
        catch (const CdmwNamingAndRepository::InvalidName& e)
        {
            cerr <<__FILE__<<" "<<__LINE__<<" "<< e._name() << endl;
            return EXIT_FAILURE;

        }
        catch (const CORBA::SystemException& e)
        {
            std::cerr <<__FILE__<<" "<<__LINE__ 
                      << "ERROR: ------------------------"
                      << std::endl;
            std::cerr << e << std::endl;
            return EXIT_FAILURE;
        }

        try
        {
            CdmwNamingAndRepository::NameDomain::RegistrationId_var regId 
                = helloDomain->new_name (comput_name.c_str());
    
            // register the object
            helloDomain->register_object (regId.in(), comput_interface_ref.in ());
        }
        catch (const CORBA::SystemException& e)
        {
                
            std::cerr << "ERROR: ------------------------"
                      << std::endl;
            std::cerr << e << std::endl;
            return EXIT_FAILURE;
        }
        catch( const CdmwNamingAndRepository::NameDomain::AlreadyExists &e )
        {
            cerr <<__FILE__<<" "<<__LINE__<<" "<< e._name() << endl;
            return EXIT_FAILURE;
        }


        // Setup signal handling for ORB to be shut down
        Cdmw::OrbSupport::OrbSupport::install_orb_shutdown_signal_handlers(orb.in());

        std::cout<< host_name << " test_server: orb run"<<std::endl;
        orb->run();

        Cdmw::OrbSupport::OrbSupport::uninstall_orb_shutdown_signal_handlers(orb.in());

    }
    catch (const CORBA::SystemException &e )
    {
        cerr <<__FILE__<<" "<<__LINE__<<" "<< e << endl;
        return EXIT_FAILURE;
    }
    catch( const CORBA::Exception &e )
    {
        cerr <<__FILE__<<" "<<__LINE__<<" "<< e._name() << endl;
        return EXIT_FAILURE;
    }
    catch( const Cdmw::Exception &e )
    {
        cerr <<__FILE__<<" "<<__LINE__<<" "<< e.what() << endl;
        return EXIT_FAILURE;
    }
    catch(...)
    {
        cerr << "Uncaught exception" << endl;
        return EXIT_FAILURE;
    }

    if (!CORBA::is_nil(orb.in()))
    {
         try
         {
             Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
             orb->destroy();
         }
         catch(const CORBA::Exception& ex)
         {
             std::cerr << ex << std::endl;
             std::cerr << " **** TEST FAILED AT LINE " << __LINE__ 
                       << " IN FILE: " << __FILE__ << std::endl;
             return EXIT_FAILURE;
         }
    }        
    
    return EXIT_SUCCESS;
}

