/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package qualif.timeoutrequest.testtimeoutrequest;   

import java.io.IOException;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;

import cdmw.orbsupport.PolicyList;
import cdmw.orbsupport.OrbShutdownSignalHandler;
import cdmw.orbsupport.StrategyList;
import cdmw.platformmngt.PlatformInterface;

import qualif.timeoutrequest.testtimeoutrequest.HelloServantLocator;

public class MainServer {
    private static final int POA_THREAD_POOL_SIZE = 5;


    public static void main(String[] args) {
        String hostName = cdmw.ossupport.OS.getHostname();

        ORB orb = null;

        try {
            // Initialise FT service
            System.out.println(hostName + "test_server: initialise FT services");
            cdmw.cdmwinit.CdmwInterface.cdmwPreORBInit(args, "cdmw.generated.TimeoutServerCdmwInterface");

            // Initialises the ORB
            StrategyList strategyList = new StrategyList();
            strategyList.addORBThreaded();
            strategyList.addPOAThreadPool(POA_THREAD_POOL_SIZE);

            System.out.println(hostName + " test_server: initialises the ORB");
            orb = cdmw.orbsupport.ORBSupport.ORBInit(args, strategyList);

            // Get the root POA
            System.out.println(hostName + " test_server: get the root POA");
            org.omg.CORBA.Object obj = orb
                .resolve_initial_references("RootPOA");
            POA rootPOA = POAHelper.narrow(obj);
        
        
            POAManager poaMgr = rootPOA.the_POAManager();
        
            // create a POA with USE_SERVANT_MANAGER & NON_RETAIN policy (to use ServantLocator)
            cdmw.orbsupport.PolicyList policies = new cdmw.orbsupport.PolicyList();
            policies.insert(
            rootPOA.create_request_processing_policy(
                    org.omg.PortableServer.RequestProcessingPolicyValue.USE_SERVANT_MANAGER));
            policies.insert(
            rootPOA.create_servant_retention_policy(
                    org.omg.PortableServer.ServantRetentionPolicyValue.NON_RETAIN));
            policies.insert(
            rootPOA.create_id_assignment_policy(
                    org.omg.PortableServer.IdAssignmentPolicyValue.USER_ID));
            StrategyList anotherStrategyList = new StrategyList();

            POA myPOA =
                    cdmw.orbsupport.ORBSupport.createPOA(
                    rootPOA,
                    "helloPOA",
                    poaMgr,
                    policies,
                        anotherStrategyList);
        
            int timescale = cdmw.testutils.CORBATestManager.getTimescale();
            cdmw.ossupport.OS.sleep(timescale * 1000);

            // init Cdmw
            System.out.println(hostName + " test_server: init Cdmw");
            cdmw.cdmwinit.CdmwInterface.cdmwInit(orb, args, null, "cdmw.generated.TimeoutServerCdmwInterface");

            // get process name
            String processName = cdmw.platformmngt.PlatformInterface.getProcessName();

            System.out.println(hostName + " test_server: create HelloInterface_impl");
            HelloInterfaceImpl helloInterface = new HelloInterfaceImpl(orb, hostName);
            byte[] hello_oid = new String("Hello").getBytes();
            org.omg.CORBA.Object hello_interface_ref = 
            myPOA.create_reference_with_id(hello_oid, "IDL:thalesgroup.com/CdmwFT/Test/ComputInterface:1.0");


            System.out.println(hostName + " test_server: create ComputInterface_impl");
            ComputInterfaceImpl computInterface = new ComputInterfaceImpl(orb, hostName);
            byte[] comput_oid = new String("Comput").getBytes();
            org.omg.CORBA.Object comput_interface_ref = 
                myPOA.create_reference_with_id(comput_oid, "IDL:thalesgroup.com/CdmwFT/Test/ComputInterface:1.0");
        

            // export the object reference to the CDMW Repository
            System.out.println(hostName
                    + " test_server: export the hello object reference to CDMW Repository");
            com.thalesgroup.CdmwNamingAndRepository.Repository repository = cdmw.namingandrepository.RepositoryInterface
                .getRepository();

            cdmw.namingandrepository.RepositoryInterface.init("CDMW",
                repository);
        
            cdmw.commonsvcs.naming.NamingInterface ni =
            cdmw.namingandrepository.RepositoryInterface.getDomainNamingInterface ("dom1/dom2");        
        
            // create ServantLocator
            System.out.println(hostName + " test_server: create ServantLocator");
            HelloServantLocator m_servant_locator = new HelloServantLocator(helloInterface,
                            computInterface,
                            processName,
                            ni);

            // set ServantLocator to helloPOA
            try {
            myPOA.set_servant_manager(m_servant_locator);
        } catch (org.omg.PortableServer.POAPackage.WrongPolicy wp) {
            // shouldn't happen
            wp.printStackTrace();
                System.exit(1);    
            }                                   
        

            System.out
                .println(hostName + " test_server: activate the root POA");
            poaMgr.activate();      
        
        

            com.thalesgroup.CdmwNamingAndRepository.NameDomain helloDomain = repository
                .resolve_name_domain("dom1/dom2/dom3");


            String helloName = processName + "HelloInterface";
            System.out
                .println(hostName + " test_server: register " + helloName);

            try {
                helloDomain.release_name(helloName);
            }
            catch (com.thalesgroup.CdmwNamingAndRepository.NameDomainPackage.NotRegistered e) {
                // not a problem it's the first time the server is launched
                e.printStackTrace();
                System.err.println("OK");
            }
            catch (com.thalesgroup.CdmwNamingAndRepository.InvalidName e0) {
                e0.printStackTrace();
                System.exit(1);
            }
            catch (SystemException e1) {
                e1.printStackTrace();
                System.err.println("ERROR: ------------------------");
                System.exit(1);
            }

            try {
                Any regId = helloDomain.new_name(helloName);

                // register the object
                helloDomain.register_object(regId, hello_interface_ref);
            }
            catch (SystemException e) {
                e.printStackTrace();
                System.err.println("ERROR: ------------------------");
                System.exit(1);
            }
            catch (com.thalesgroup.CdmwNamingAndRepository.NameDomainPackage.AlreadyExists e0) {
                e0.printStackTrace();
                System.exit(1);
            }

            // export the object reference to a CDMW Repository
            System.out
                .println(hostName
                    + " test_server: export the comput object reference to a CDMW Repository");
            String computName = processName + "ComputInterface";
            System.out.println(hostName + " test_server: register "
                + computName);

            try {
                helloDomain.release_name(computName);
            }
            catch (com.thalesgroup.CdmwNamingAndRepository.NameDomainPackage.NotRegistered e) {
                // not a problem it's the first time the server is launched
                e.printStackTrace();
                System.err.println("OK");
            }
            catch (com.thalesgroup.CdmwNamingAndRepository.InvalidName e0) {
                e0.printStackTrace();
                System.exit(1);

            }
            catch (SystemException e1) {
                e1.printStackTrace();
                System.err.println("ERROR: ------------------------");
                System.exit(1);
            }

            try {
                Any regId = helloDomain.new_name(computName);

                // register the object
                helloDomain.register_object(regId, comput_interface_ref);
            }
            catch (SystemException e) {
                e.printStackTrace();
                System.err.println("ERROR: ------------------------");
                System.exit(1);
            }
            catch (com.thalesgroup.CdmwNamingAndRepository.NameDomainPackage.AlreadyExists e0) {
                e0.printStackTrace();
                System.exit(1);
            }

            // Setup signal handling for ORB to be shut down
            OrbShutdownSignalHandler signalHandler = new OrbShutdownSignalHandler(
                orb);
            cdmw.orbsupport.ORBSupport.InstallORBShutdownSignalHandler(orb,
                signalHandler);

            System.out.println(hostName + " test_server: orb run");
            orb.run();

            cdmw.orbsupport.ORBSupport
                .UninstallORBShutdownSignalHandler(signalHandler);

        }
        catch (SystemException e) {
            e.printStackTrace();
            System.exit(1);
        }
        catch (UserException e0) {
            e0.printStackTrace();
            System.exit(1);
        }
        catch (cdmw.common.Exception e1) {
            e1.printStackTrace();
            System.exit(1);
        }
        catch (Exception e2) {
            System.err.println("Uncaught exception");
            System.exit(1);
        }

        if (orb != null) {
            try {
                cdmw.orbsupport.ORBSupport.ORBCleanup(orb);
                orb.destroy();
            }
            catch (SystemException ex) {
                System.err.println(" **** TEST FAILED AT LINE ");
                ex.printStackTrace();
                System.exit(1);
            }
        }

        System.exit(0);
    }
}
