/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package qualif.timeoutrequest.testtimeoutrequest;

import java.io.IOException;

import org.junit.*;
import org.junit.runner.RunWith;
import org.omg.CORBA.*;
import org.omg.CosNaming.NameComponent;
import org.omg.PortableServer.*;

import qualif.timeoutrequest.testtimeoutrequest.HelloTest;
import cdmw.testutils.*;
import cdmw.testutils.CORBATestManager.InvalidObject;
import cdmw.testutils.threadtestengine.*;


import static org.junit.Assert.*;

//import com.thalesgroup.CdmwFT.*;
//import com.thalesgroup.CdmwFT.Test.*;
import cdmw.commonsvcs.naming.NamingInterface;
import cdmw.namingandrepository.RepositoryInterface;

@RunWith( CdmwThreadedTestClassRunner.class)
public class TestTimeoutRequest {

    /**
     * Field is used for obtaining TestProcessor class instance
     */
    private TestContext testContext;

    private org.omg.CORBA.ORB orb = null;

    private POA rootPOA;

    public TestTimeoutRequest() {

        try {
            this.orb = CORBATestManager.instance().getORB();

            this.rootPOA = CORBATestManager.instance().getPOA();

        }
        catch (InvalidObject e) {
            e.printStackTrace();
        }
    }

    @Test
    public void doTests() {
        int timescale = CORBATestManager.getTimescale();

        try {
            cdmw.ossupport.OS.unlink("hello.stt");
            cdmw.ossupport.OS.unlink("comput.stt");
        }
        catch (Exception e) {}

        // set default properties
        System.out.println( "set_default_properties" );


        System.out.println( "// ###########################################################" );
        System.out.println("client started");
        System.out.println( "// ###########################################################" );
    
        System.out.println("get repository");
        com.thalesgroup.CdmwNamingAndRepository.Repository repository
            = RepositoryInterface.getRepository();

        System.out.println("init repository");

        try{
            RepositoryInterface.init ("CDMW",repository);
        }catch(Exception e) {
            e.printStackTrace();
            System.exit(1);   
        }

        System.out.println("get_domain_naming_interface dom1/dom2 ");

        NamingInterface ni = null;
        try {
            ni = RepositoryInterface
                .getDomainNamingInterface("dom1/dom2");
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(1);   
        }

        // Getting P11HelloInterface
        String fullName = "dom3/P11HelloInterface";

        org.omg.CORBA.Object obj = null;
        
        try {
                System.out.println("resolve " + fullName);
                obj = ni.resolve (fullName);
                assertTrue(true);
            }
        catch( org.omg.CosNaming.NamingContextPackage.NotFound e)
            {
                e.printStackTrace();
                assertTrue(false);
            }
        catch( org.omg.CosNaming.NamingContextPackage.CannotProceed e)
            {
                e.printStackTrace();
                assertTrue(false);
            }
        catch( org.omg.CosNaming.NamingContextPackage.InvalidName e)
            {
                e.printStackTrace();
                assertTrue(false);
            }
        catch ( org.omg.CORBA.SystemException e)
            {
                e.printStackTrace();
                assertTrue(false);
            }

        com.thalesgroup.CdmwFT.Test.HelloInterface helloP11 
            = com.thalesgroup.CdmwFT.Test.HelloInterfaceHelper.narrow(obj);

        // Getting P31HelloInterface
        fullName = "dom3/P31HelloInterface";
        try
            {
                System.out.println("resolve " + fullName);
                obj = ni.resolve (fullName);
                assertTrue(true);
            }
        catch( org.omg.CosNaming.NamingContextPackage.NotFound e)
            {
                e.printStackTrace();
                assertTrue(false);
            }
        catch( org.omg.CosNaming.NamingContextPackage.CannotProceed e)
            {
                e.printStackTrace();
                assertTrue(false);
            }
        catch( org.omg.CosNaming.NamingContextPackage.InvalidName e)
            {
                e.printStackTrace();
                assertTrue(false);
            }
        catch ( org.omg.CORBA.SystemException e)
            {
                e.printStackTrace();
                assertTrue(false);
            }
    
        com.thalesgroup.CdmwFT.Test.HelloInterface helloP31 
            = com.thalesgroup.CdmwFT.Test.HelloInterfaceHelper.narrow(obj);


        // Getting P21ComputInterface
        fullName = "dom3/P21ComputInterface"; 
        try
            {
                System.out.println("resolve " + fullName);
                obj = ni.resolve (fullName);
                assertTrue(true);

            }
        catch( org.omg.CosNaming.NamingContextPackage.NotFound e)
            {
                e.printStackTrace();
                assertTrue(false);
            }
        catch( org.omg.CosNaming.NamingContextPackage.CannotProceed e)
            {
                e.printStackTrace();
                assertTrue(false);
            }
        catch( org.omg.CosNaming.NamingContextPackage.InvalidName e)
            {
                e.printStackTrace();
                assertTrue(false);
            }
        catch ( org.omg.CORBA.SystemException e)
            {
                e.printStackTrace();
                assertTrue(false);
            }
    
        com.thalesgroup.CdmwFT.Test.ComputInterface computP21
            = com.thalesgroup.CdmwFT.Test.ComputInterfaceHelper.narrow(obj);

        // Getting P41ComputInterface
        fullName = "dom3/P41ComputInterface";
        try
            {
                System.out.println("resolve " + fullName);
                obj = ni.resolve (fullName);
                assertTrue(true);
            }
        catch( org.omg.CosNaming.NamingContextPackage.NotFound e)
            {
                e.printStackTrace();
                assertTrue(true);
            }
        catch( org.omg.CosNaming.NamingContextPackage.CannotProceed e)
            {
                e.printStackTrace();
                assertTrue(false);
            }
        catch( org.omg.CosNaming.NamingContextPackage.InvalidName e)
            {
                e.printStackTrace();
                assertTrue(false);

            }
        catch ( org.omg.CORBA.SystemException e)
            {
                e.printStackTrace();
                assertTrue(false);
            }

        com.thalesgroup.CdmwFT.Test.ComputInterface computP41 
            = com.thalesgroup.CdmwFT.Test.ComputInterfaceHelper.narrow(obj);
        
        

        TestProcessor tpHello = this.testContext.getTestProcessor();
        tpHello.addTest(HelloTest.class);
        tpHello.runTests();
            
   
        try {
            System.out.println("Calling Hello obj on P11 and Comput obj on P21...");
            helloP11.hello(computP21, 100);
            System.out.println("hello on P11 OK...");
            assertTrue(true);
        }
        catch (SystemException e) {
            System.out.println("CORBA System Exception raised " + e.getMessage());
            assertTrue(false);
        }

        try {
            System.out.println("Calling send_TRANSIENT_exception on HelloInterface on P11...");
            helloP11.send_TRANSIENT_exception();
            assertTrue(false);
        }
        catch (TRANSIENT e) {
            System.out.println("Expected CORBA.TRANSIENT Exception raised");
            assertTrue(true);
        }
        catch (SystemException e1) {
            System.out.println("Unexpected CORBA System Exception raised "
                               + e1.getMessage());
            assertTrue(false);
        }

        try {
            System.out.println("Calling Hello obj on P11 and Comput obj on P21 ...");
            helloP11.hello(computP21, 200);
            System.out.println("hello on P11 OK...");
            assertTrue(true);
        }
        catch (SystemException e) {
            System.out.println("Unexpected CORBA System Exception raised "
                               + e.getMessage());
            assertTrue(false);
        }
    
        try {
            System.out.println("Calling Hello obj on P11 and Comput obj on P21. P21 should forward to P41.");
            helloP11.hello_forward(computP21, 100);
            assertTrue(true);
        }
        catch (SystemException e) {
            System.out.println("Unexpected CORBA System Exception raised "
                               + e.getMessage());
            assertTrue(false);
        }

        try {
            System.out.println("Calling hello obj on P11 and comput obj on P21. An exception should be raised, the computation time of Comput obj is too long...");
            helloP11.hello(computP21, 3000);
            assertTrue(false);
        }
        catch (TRANSIENT e) {
            System.out.println("Unexpected CORBA::TRANSIENT Exception raised " + e.getMessage());
            assertTrue(false);
        }
        catch (org.omg.CORBA.COMM_FAILURE e) {
            System.out.println("Expected CORBA::COMM_FAILURE Exception raised" + e.getMessage());
            System.out.println("Check P21 is dead, P11, P31 and P41 are alive");
            assertTrue(true);
        }
        catch (SystemException e) {
            System.out.println("Unexpected CORBA System Exception raised " + e.getMessage());
            assertTrue(false);
        }

        try {
            System.out.println("Calling hello obj on 31 and Comput obj on P41...");
            helloP31.hello(computP41, 300);
            System.out.println("hello on P31 OK...");
            assertTrue(true);
        }
        catch (SystemException e) {
            System.out.println("CORBA System Exception raised "
                               + e.getMessage());
            assertTrue(false);
        }

        try {
            System.out.println("Calling hello obj on P31 and Comput obj on P41. An exception should be raised, the computation time of Comput obj is too long...");
            helloP31.hello(computP41, 3000);
            assertTrue(false);
        }
        catch (org.omg.CORBA.TRANSIENT e) {
            System.out.println("Unexpected CORBA.TRANSIENT Exception raised "
                               + e.getMessage());
            assertTrue(false);
        }
        catch (org.omg.CORBA.COMM_FAILURE e)
            {
                System.out.println("Expected CORBA::COMM_FAILURE Exception raised");
                System.out.println("Check P41 has just been dead. P11 and P31 are the only one to be alive");
                assertTrue(true);
            }
        catch (SystemException e)
            {
                System.out.println("Unexpected CORBA System Exception raised " + e.getMessage());
                assertTrue(false);
            }

        try {
            cdmw.ossupport.OS.unlink("hello.stt");
            cdmw.ossupport.OS.unlink("comput.stt");
        }
        catch (Exception e) {}
    }
}

