/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.testAMI;

import org.omg.CORBA.*;

public class ServerHandlerImpl extends Cdmw.TestAMI.AMI_ServerHandlerPOA {
    private static final int RESULT_NOT_RECEIVED = 0;

    private static final int RESULT_NOT_CORRECT = -1;

    private static final int RESULT_CORRECT = 1;

    private static final short EXPECTED_RETURN_VALUE = 10;

    private static final short EXPECTED_A_VALUE = 20;

    private static final short EXPECTED_B_VALUE = 30;

    private ORB orb;

    private int requestSleepingResult;

    private int requestSendingExceptionResult;

    public ServerHandlerImpl(
        ORB orb) {
        this.orb = orb;
        this.requestSleepingResult = RESULT_NOT_RECEIVED;
        this.requestSendingExceptionResult = RESULT_NOT_RECEIVED;
    }

    public void request_sleeping(short ami_return_val, short a, short b) {
        System.out.println("request_sleeping() result received.");
        this.requestSleepingResult = RESULT_CORRECT;

        if (ami_return_val != EXPECTED_RETURN_VALUE) {
            System.err
                .println("Return value of request_sleeping() is not the one expected: "
                    + ami_return_val);
            this.requestSleepingResult = RESULT_NOT_CORRECT;
        }

        if (a != EXPECTED_A_VALUE) {
            System.err
                .println("intout 'a' value of request_sleeping() is not the one expected: "
                    + a);
            this.requestSleepingResult = RESULT_NOT_CORRECT;
        }

        if (b != EXPECTED_B_VALUE) {
            System.err
                .println("out 'b' value of request_sleeping() is not the one expected: "
                    + b);
            this.requestSleepingResult = RESULT_NOT_CORRECT;
        }
    }

    public void request_sleeping_excep(
        org.omg.Messaging.ExceptionHolder excep_holder) {
        System.out
            .println("request_sleeping() result received: it is an Exception !!");
        this.requestSleepingResult = RESULT_NOT_CORRECT;
    }

    public void request_sending_exception() {
        System.out
            .println("request_sending_exception() result received: it is not an Exception !!");
        this.requestSendingExceptionResult = RESULT_NOT_CORRECT;
    }

    public void request_sending_exception_excep(
        org.omg.Messaging.ExceptionHolder excep_holder) {
        System.out.println("request_sending_exception() result received.");

        try {
            String[] exc_id = excep_holder._truncatable_ids();

            for (int i=0; i<exc_id.length; i++)
        {
            System.out.println("Exception: "+exc_id[i]);
        }
            excep_holder.raise_exception();

            System.err
                .println("request_sending_exception() has'nt send an exception !!");
            this.requestSendingExceptionResult = RESULT_NOT_CORRECT;

        }
        catch (Cdmw.TestAMI.TestException e) {
            System.err.println ("*request_sending_exception_excep* got the correct exception");
            this.requestSendingExceptionResult = RESULT_CORRECT;
        }
        catch (SystemException e0) {
            System.err
                .println("request_sending_exception() e0 has'nt send the expected exception: \n"
                    + cdmw.orbsupport.ORBSupport.exceptionToString(e0));
            this.requestSendingExceptionResult = RESULT_NOT_CORRECT;
        }
        catch (UserException e1) {
            System.err
                .println("request_sending_exception() e1 has'nt send the expected exception: \n"
                    + cdmw.orbsupport.ORBSupport.exceptionToString(e1));
            this.requestSendingExceptionResult = RESULT_NOT_CORRECT;
        }
        catch (Exception e2) {
            System.err
                .println("request_sending_exception() e2 has'nt send the expected exception: \n"
                    + "Unknown non-CORBA exception");
            this.requestSendingExceptionResult = RESULT_NOT_CORRECT;
        }
    }

    public int checkRequestSleepingResult() {
        return this.requestSleepingResult;
    }

    public int checkRequestSendingExceptionResult() {
        return this.requestSendingExceptionResult;
    }
}
