/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.testcodec;

import org.omg.CORBA.*;
import Cdmw.TestCodec.*;

public class DataManager {
    private ORB orb;
    
    private static final String TEST_UNION_STR_VALUE = "String for 'str' member of TestUnion";
    private static final short TEST_UNION_S_VALUE = -123;
    private static final long TEST_UNION_ULL_VALUE = 123456;
    private static final short TEST_STR_SEQ_LENGTH = 2;
    private static final String TEST_STR_SEQ_0_VALUE = "String for member number 0 of TestStrSeq";
    private static final String TEST_STR_SEQ_1_VALUE = "String for member number 1 of TestStrSeq";
    private static final String TEST_ANY_VALUE = "String for 'a' member of TestStruct";
    
    public DataManager(ORB orb) { 
        this.orb = orb;
    }
    
    public TestStruct createTestStruct(TestEnum e) {
        //Create TestUnion
        TestUnion u = new TestUnion();
        //Create Any
        Any a = this.orb.create_any();
        a.insert_string(TEST_ANY_VALUE);
        //Create seq object
        String[] seq =  new String[TEST_STR_SEQ_LENGTH];
        seq[0] = TEST_STR_SEQ_0_VALUE;
        seq[1] = TEST_STR_SEQ_1_VALUE;
        
        TestStruct testStruct = new TestStruct(u, seq, a);
        
        // fill u member
        switch (e.value()) {
            case TestEnum._VAL1:
                testStruct.u.str(TEST_UNION_STR_VALUE);
                break;               
            case TestEnum._VAL2:   
                testStruct.u.s(TEST_UNION_S_VALUE);
                break;
            case TestEnum._VAL3:   
                testStruct.u.ull(TEST_UNION_ULL_VALUE);
                break;
        }
        
        
//        // fill seq member
//        testStruct.seq = new String[TEST_STR_SEQ_LENGTH];
//        testStruct.seq[0] = TEST_STR_SEQ_0_VALUE;
//        testStruct.seq[1] = TEST_STR_SEQ_1_VALUE;
//        
//        // fill a member
//        testStruct.a.insert_string(TEST_ANY_VALUE);
        
        return testStruct;
    }
    
    public void checkTestStruct(TestStruct testStruct)
        throws DecodingFailure
    {
        try {   
        
            // check u member
            switch (testStruct.u.discriminator().value()) {
                case TestEnum._VAL1:
                    if (!testStruct.u.str().equals(TEST_UNION_STR_VALUE)) {
                        System.err.println("Unexpected value of TestStruct.u.str() : " 
                                  + testStruct.u.str());
                        throw new DecodingFailure();
                    }
                    break;
                    
                case TestEnum._VAL2:   
                    if (testStruct.u.s() != TEST_UNION_S_VALUE) {
                        System.err.println("Unexpected value of TestStruct.u.s() : " 
                                  + testStruct.u.s());
                        throw new DecodingFailure();
                    }
                    break;
        
                case TestEnum._VAL3:   
                    if (testStruct.u.ull() != TEST_UNION_ULL_VALUE) {
                        System.err.println("Unexpected value of TestStruct.u.ull() : " 
                                  + testStruct.u.ull());
                        throw new DecodingFailure();
                    }
                    break;
                    
                default:
                    System.err.println("Unexpected value of TestStruct.u.discriminator() : " 
                              + testStruct.u.discriminator().value());
                    throw new DecodingFailure();
            }
            
            // check seq member
            if (testStruct.seq.length != TEST_STR_SEQ_LENGTH) {
                System.err.println("Unexpected length for TestStruct.seq : " 
                          + testStruct.seq.length);
                throw new DecodingFailure();
            }
            if (!testStruct.seq[0].equals(TEST_STR_SEQ_0_VALUE)) {
                System.err.println("Unexpected value of TestStruct.seq[0] : " 
                          + testStruct.seq[0]);
                throw new DecodingFailure();
            }
            if (!testStruct.seq[1].equals(TEST_STR_SEQ_1_VALUE)) {
                System.err.println("Unexpected value of TestStruct.seq[1] : " 
                          + testStruct.seq[1]);
                throw new DecodingFailure();
            }
        
            // check a member
            String aValue = null;
            if (testStruct.a.type().kind().value() != org.omg.CORBA.TCKind._tk_string) {
                System.err.println("Unexpected type in TestStruct.a : ");
                org.omg.CORBA.TypeCode tc = testStruct.a.type();
                System.err.println(tc.name());
                throw new DecodingFailure();
            }
            else {
                aValue = testStruct.a.extract_string();
            }
             
            if (!aValue.equals(TEST_ANY_VALUE)) {
                System.err.println("Unexpected value of content of TestStruct.a : " 
                          + aValue);
                throw new DecodingFailure();
            }
        
        
        } catch (DecodingFailure e) {
            throw e;
        } catch (SystemException e0) {
            System.err.println("ERROR while checking TestStruct value:\n"
                      + cdmw.orbsupport.ORBSupport.exceptionToString(e0));
            throw new DecodingFailure();
        } 
        catch (UserException e1) {
            System.err.println("ERROR while checking TestStruct value:\n"
                      + cdmw.orbsupport.ORBSupport.exceptionToString(e1));
            throw new DecodingFailure();
        }
        catch (Exception e2) {
            System.err.println("ERROR while checking TestStruct value:\n"
                      + "Unknown exception !!" 
                      + cdmw.orbsupport.ORBSupport.exceptionToString(e2));
            
            throw new DecodingFailure();
        }
    }
}
