/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.testcodec;

import org.omg.CORBA.*;
import org.omg.CORBA.ORBPackage.InvalidName;
import org.omg.PortableServer.*;

import com.thalesgroup.CdmwNamingAndRepository.*;
import com.thalesgroup.CdmwPlatformMngt.ProcessDelegatePackage.NotReadyToRun;
import com.thalesgroup.CdmwPlatformMngtBase.StartupKind;

public class DecoderProcessControl extends cdmw.cdmwinit.ProcessControl {
    /**
     * The orb reference.
     */
    private ORB orb = null;

    public DecoderProcessControl(ORB orb) {
        this.orb = orb;
    }

    // process initialisation called by platformmngt
    public void onInitialise(StartupKind startupKind) {
        //
        // Create and register DataTransfer
        //
        System.out.println("Create and Register DataTransfer into the Repository");

        // Get the naming and repository from Repository interface
        com.thalesgroup.CdmwNamingAndRepository.Repository repository = 
            cdmw.namingandrepository.RepositoryInterface.getRepository();        

        DataTransferImpl pDataTransferServant = new DataTransferImpl(this.orb);

        try {
            // Get the root POA
            org.omg.CORBA.Object rootPOAobj = this.orb.resolve_initial_references("RootPOA");
            POA rootPOA = POAHelper.narrow(rootPOAobj);

            byte[] oid = rootPOA.activate_object(pDataTransferServant);

            org.omg.CORBA.Object object = rootPOA.id_to_reference(oid);
            //create the DataTransfer servant
            Cdmw.TestCodec.DataTransfer dataTransfer = 
                            Cdmw.TestCodec.DataTransferHelper.narrow(object);

            // Get dom1 name domain from repository
            com.thalesgroup.CdmwNamingAndRepository.NameDomain domain = repository.resolve_name_domain("dom1");

            // Register the DataTransfer object in the dom1 name domain
            Any regId = domain.new_name("DataTransfer");
            domain.register_object(regId, dataTransfer);
        } catch (UserException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public void onRun() throws NotReadyToRun {

    }

    @Override
    public void onStop() {

    }

}
