/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.testcodec;

import org.omg.CORBA.*;
import org.omg.CosNaming.NamingContextPackage.CannotProceed;
import org.omg.CosNaming.NamingContextPackage.NotFound;

import Cdmw.TestCodec.TestEnum;
import cdmw.orbsupport.Codec;
import cdmw.testutils.CORBATestManager;
import cdmw.testutils.CORBATestManager.InvalidObject;

import org.junit.*;

import com.thalesgroup.CdmwNamingAndRepository.InvalidName;
import com.thalesgroup.CdmwNamingAndRepository.NoNameDomain;

public class EncoderTest {
    private Cdmw.TestCodec.DataTransfer dataTransfer;

    /**
     * The orb reference.
     */
    private ORB orb;

    // constructor
    public EncoderTest() throws InvalidObject {
        this.orb = CORBATestManager.instance().getORB();
    }

    @Test
    public void doTests() throws NoNameDomain, InvalidName, NotFound, CannotProceed, org.omg.CosNaming.NamingContextPackage.InvalidName {
        // set_nbOfRequestedTestOK(3);

        // 
        // Retrive DataTransfer
        //
        System.out.println("Retrive DataTransfer object from Repository");

        // Get the naming and repository from Repository interface
        com.thalesgroup.CdmwNamingAndRepository.Repository repository = cdmw.namingandrepository.RepositoryInterface
            .getRepository();

        // Get Naming Interface of dom1 name domain
        cdmw.commonsvcs.naming.NamingInterface ni = cdmw.namingandrepository.RepositoryInterface
            .getDomainNamingInterface("dom1");

        // Get the DataTransfer object from repository
        org.omg.CORBA.Object objref = ni.resolve("DataTransfer");

        // get the DataTransfer proxy
        this.dataTransfer = Cdmw.TestCodec.DataTransferHelper.narrow(objref);

        Assert.assertNotNull(this.dataTransfer);

        DataManager dataMgr = new DataManager(this.orb);
        cdmw.orbsupport.Codec codec = new Codec();

        try {
            System.out.println("Sending encoded data with enum set to VAL1");

            // construct data
            Cdmw.TestCodec.TestStruct testStruct = dataMgr.createTestStruct(TestEnum.VAL1);


            // encode data
            byte[] encoded_data = OctetSeqHelper.extract(codec.encode(testStruct));

            // send data
            this.dataTransfer.send_data(encoded_data);
        }
        catch (Cdmw.TestCodec.DecodingFailure e) {
            System.err.println("DecodingFailure exception send by decoder");
            e.printStackTrace();
            Assert.assertTrue(false);
        }
        catch (SystemException e1) {
            System.err.println("CORBA Exception :\n"
                + cdmw.orbsupport.ORBSupport.exceptionToString(e1));
            Assert.assertTrue(false);
        }
        catch (UserException e2) {
            System.err.println("CORBA Exception :\n"
                + cdmw.orbsupport.ORBSupport.exceptionToString(e2));
            Assert.assertTrue(false);
        }

        try {
            System.out.println("Sending encoded data with enum set to VAL2");

            // construct data
            Cdmw.TestCodec.TestStruct testStruct = dataMgr
                .createTestStruct(TestEnum.VAL2);

            // encode data
            byte[] encoded_data = OctetSeqHelper.extract(codec.encode(testStruct));

            // send data
            this.dataTransfer.send_data(encoded_data);
        }
        catch (Cdmw.TestCodec.DecodingFailure e) {
            System.err.println("DecodingFailure exception send by decoder");
            Assert.assertTrue(false);
        }
        catch (SystemException e1) {
            System.err.println("CORBA Exception :\n"
                + cdmw.orbsupport.ORBSupport.exceptionToString(e1));
            Assert.assertTrue(false);
        }
        catch (UserException e2) {
            System.err.println("CORBA Exception :\n"
                + cdmw.orbsupport.ORBSupport.exceptionToString(e2));
            Assert.assertTrue(false);
        }

        try {
            System.out.println("Sending encoded data with enum set to VAL3");

            // construct data
            Cdmw.TestCodec.TestStruct testStruct = dataMgr.createTestStruct(TestEnum.VAL3);

            // encode data
            byte[] encoded_data = OctetSeqHelper.extract(codec.encode(testStruct));

            // send data
            this.dataTransfer.send_data(encoded_data);
        }
        catch (Cdmw.TestCodec.DecodingFailure e) {
            System.err.println("DecodingFailure exception send by decoder");
            Assert.assertTrue(false);
        }
        catch (SystemException e1) {
            System.err.println("CORBA Exception :\n"
                + cdmw.orbsupport.ORBSupport.exceptionToString(e1));
            Assert.assertTrue(false);
        }
        catch (UserException e2) {
            System.err.println("CORBA Exception :\n"
                + cdmw.orbsupport.ORBSupport.exceptionToString(e2));
            Assert.assertTrue(false);
        }
    }
}
