/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef ___FT_COMP_WITH_TIMERS_TEST_CCM_SERVER_WITH_FACET_IMPL_HPP__
#define ___FT_COMP_WITH_TIMERS_TEST_CCM_SERVER_WITH_FACET_IMPL_HPP__

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>

#include <ft_comp_with_timers/User_ft_comp_with_timers.skel.hpp>
#include <CCMContainer/ccmcif/SessionContextExt_impl.hpp>



namespace FTCompWithTimersTest
{
//
// IDL:thalesgroup.com/FTCompWithTimersTest/CCM_ServerWithFacet:1.0
//
class CCM_ServerWithFacet_impl : 
    virtual public ::UserFTCompWithTimersTest::User_CCM_ServerWithFacet,
    virtual public Cdmw::OrbSupport::RefCountLocalObject
{
    
    CCM_ServerWithFacet_impl(const CCM_ServerWithFacet_impl&);
    void operator=(const CCM_ServerWithFacet_impl&);


    ::FTCompWithTimersTest::CCM_ServerWithFacet_Context_var m_session_context;
    // extended context with CDMW services
    CdmwComponents::SessionContextExt_var m_session_context_ext;
    // clock service
    CosClockService::ClockCatalog_var m_clock_catalog;
    Cdmw::clock::ControlledExecutor_var m_executor;
    CosClockService::PeriodicExecution::Controller_var m_controller;
    CosClockService::PeriodicExecution::Periodic_var m_periodic;
    
    unsigned long m_num_of_execution_done;


public:

    CCM_ServerWithFacet_impl();
    ~CCM_ServerWithFacet_impl();

    
    //
    // facet serverHostInfo
    //
    virtual ::FTCompWithTimersTest::CCM_HostInfo_ptr get_serverHostInfo ()
        throw (CORBA::SystemException);
    
    virtual char* get_hostname()
        throw (CORBA::SystemException);
    

    //
    // Session component specific operations
    //
    void set_session_context(Components::SessionContext_ptr ctx)
        throw(CORBA::SystemException);
    void ccm_activate()
        throw(CORBA::SystemException);
    void ccm_passivate()
        throw(CORBA::SystemException);
    void ccm_remove()
        throw(CORBA::SystemException);


    ::FTCompWithTimersTest::CCM_ServerWithFacet_Context* get_context()
    {
        return ::FTCompWithTimersTest::CCM_ServerWithFacet_Context::_duplicate(m_session_context.in());
    }
      
    // implement Periodic interface
    CORBA::Boolean do_work(const CORBA::Any& args)
        throw (CORBA::SystemException);
    void cdmw_initialise_component(const ::CdmwPlatformMngtBase::StartupKind& startup_kind)
      throw(CdmwPlatformMngt::ManagedComponent::BadOrder, CORBA::SystemException);


  
    
    // getter
    CosClockService::ClockCatalog_var getClockCatalog() 
    {  return m_clock_catalog;  }   
    Cdmw::clock::ControlledExecutor_var getExecutor()
    {  return m_executor; }
    CosClockService::PeriodicExecution::Controller_var getController()
    {  return m_controller; }
    CosClockService::PeriodicExecution::Periodic_var getPeriodic() 
    {  return m_periodic; }
    // setter    
    void setClockCatalog(); 
    void setExecutor();
    void setPeriodic();
};


}; // End of namespace FTCompWithTimersTest

#endif // ___FT_COMP_WITH_TIMERS_TEST_CCM_SERVER_WITH_FACET_IMPL_HPP__

