/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <ft_comp_with_timers/CCM_Client_impl.hpp>

#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>


//
// IDL:thalesgroup.com/FTCompWithTimersTest/CCM_Client:1.0
//
FTCompWithTimersTest::CCM_Client_impl::CCM_Client_impl()
    : 
      m_session_context(FTCompWithTimersTest::CCM_Client_Context::_nil()),
      m_testThread()
{
}

FTCompWithTimersTest::CCM_Client_impl::~CCM_Client_impl()
{
    std::cout << "   ... CCM_Client_impl::~CCM_Client_impl called!" << std::endl;
}


char*
FTCompWithTimersTest::CCM_Client_impl::get_hostname()
    throw (CORBA::SystemException)
{
    std::string hostname = Cdmw::OsSupport::OS::get_hostname();
    return CORBA::string_dup(hostname.c_str());
}    


    
//
// consumer hostnameResults
//
void 
FTCompWithTimersTest::CCM_Client_impl::push_hostnameResults (FTCompWithTimersTest::HostnameResult * event)
    throw (CORBA::SystemException)
{
    std::cout << "   ... CCM_Client_impl::CCM_Client_impl::push_hostnameResults called with event " << event->hostname() << std::endl;
}
     



//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
FTCompWithTimersTest::CCM_Client_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_Client_impl::Set_session_context called!" << std::endl;
   
    m_session_context = ::FTCompWithTimersTest::CCM_Client_Context::_narrow(ctx); 
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
FTCompWithTimersTest::CCM_Client_impl::ccm_activate()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_Client_impl::Ccm_activate called!" << std::endl;
   Cdmw::TestUtils::CORBATestManager::instance()->add("client_context", get_context());
   Cdmw::TestUtils::CORBATestManager::instance()->add("user_ccm_client", static_cast< ::UserFTCompWithTimersTest::User_CCM_Client_ptr > (this));
   m_testThread.start();
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
FTCompWithTimersTest::CCM_Client_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_Client_impl::Ccm_passivate called!" << std::endl;
} 



//
// ccm_remove function : it destroys the component (call by the container)
//
void
FTCompWithTimersTest::CCM_Client_impl::ccm_remove()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_Client_impl::Ccm_remove called!" << std::endl;
}



//
// Testable do_tests() operation
//

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(FTCompWithTimersTest::test_ft_comp_with_timers_activate, "test_ft_comp_with_timers_activate");

void
FTCompWithTimersTest::test_ft_comp_with_timers_activate::do_tests()
{
    Cdmw::OsSupport::OS::sleep(2000);
    
    CORBA::Object_var obj2 = Cdmw::TestUtils::CORBATestManager::instance()->get_object("user_ccm_client");
    ::UserFTCompWithTimersTest::User_CCM_Client_var user_ccm_client = 
        ::UserFTCompWithTimersTest::User_CCM_Client::_narrow(obj2.in());
    m_user_ccm_client = reinterpret_cast< FTCompWithTimersTest::CCM_Client_impl* > (user_ccm_client.in());

//     set_nbOfRequestedTestOK(0);
    
    // test connections to group (primary hostname is expected)
    test_connections();
 
    TEST_INFO("\n=========================================================\n"
             << "You can see the Periodic Execution Output displayed on HOST1 \n"
             << "=========================================================");
    std::string dummy;
    std::cout<<"Press any key and Enter"<<std::endl;
    std::cin >> dummy;
    try {
        TEST_INFO(   "\n=========================================================\n"
                  << "    Simulate the crash of primary FT Component Server\n"
                  << "    On HOST1 execute following command :\n"
                  << "    killall ft_comp_with_timers_server_comp_srv :\n"
                  << "=========================================================");

        //std::string dummy;
        std::cout<<"Press any key and Enter"<<std::endl;
        std::cin >> dummy;

        TEST_INFO("\n=========================================================\n"
                  << "You can see the Periodic Execution Output displayed on HOST2 \n"
                  << "=========================================================");
        

        CPPUNIT_ASSERT(true);
        Cdmw::OsSupport::OS::sleep(1000);
        //std::string dummy;
        std::cout<<"Press any key and Enter"<<std::endl;
        std::cin >> dummy;
    } catch (const CORBA::SystemException& e) {
        TEST_INFO("Unexpected CORBA::SystemException: " << e);
        CPPUNIT_ASSERT(false);
    }
    
    // test connections to group (backup1 hostname is expected)
    test_connections();


    try {
        TEST_INFO(   "\n=========================================================\n"
                  << "    Simulate the crash/restart of FT Component Server\n"
                  << "    On HOST2 execute following command :\n"
                  << "    platform_admin.sh --proc-hot-restart CcmFtApplication ServerProcess2 <HOST2>\n"
                  << "=========================================================");
    
        //std::string dummy;
        std::cout<<"Press any key and Enter"<<std::endl;
        std::cin >> dummy;

        
        TEST_INFO("\n=========================================================\n"
                  << "You can see the Periodic Execution Output displayed on HOST3 \n"
                  << "=========================================================");
                  
        CPPUNIT_ASSERT(true);
        Cdmw::OsSupport::OS::sleep(1000);
        std::string dummy;
        std::cout<<"Press any key and Enter"<<std::endl;
        std::cin >> dummy;
    } catch (const CORBA::SystemException& e) {
        TEST_INFO("Unexpected CORBA::SystemException: " << e);
        CPPUNIT_ASSERT(false);
    }
    
    // test connections to group (backup2 hostname is expected)
    test_connections();

    try {
        TEST_INFO(   "\n=========================================================\n"
                  << "    Stop of FT Component Server\n"
                  << "    On HOST3 execute following command :\n"
                  << "    platform_admin.sh --proc-stop CcmFtApplication ServerProcess3 <HOST3>\n"
                  << "=========================================================");

    
        TEST_INFO("\n=========================================================\n"
                  << "You can see the Periodic Execution Output displayed on HOST2 \n"
                  << "=========================================================");
        

        CPPUNIT_ASSERT(true);
        Cdmw::OsSupport::OS::sleep(1000);
        std::string dummy;
        std::cout<<"Press any key and Enter"<<std::endl;
        std::cin >> dummy;

        TEST_INFO("\n=========================================================\n"
                  << "TEST COMPLETED. \n"
                  << "=========================================================");
        
        Cdmw::TestUtils::CORBATestManager::instance()->summary();

    } catch (const CORBA::SystemException& e) {
        TEST_INFO("Unexpected CORBA::SystemException: " << e);
        CPPUNIT_ASSERT(false);
    }
    
    /*try {
        TEST_INFO("Test that group is empty");   
        FTCompWithTimersTest::HostInfo_var hostInfo = m_session_context->get_connection_serverHostInfo();      
        if (CORBA::is_nil(hostInfo.in())) {
            TEST_INFO("Now that group is empty, hostInfo is NIL !!!! ");
            CPPUNIT_ASSERT(false);
            return;
        }        
    
        CORBA::String_var hostname = hostInfo->get_hostname();
        TEST_INFO("Request on empty group succeed !! hostname: " << hostname.in());
        CPPUNIT_ASSERT(false);
         
    } catch (const CORBA::SystemException& e) {
        CPPUNIT_ASSERT(true);
    } catch (const CORBA::Exception& e) {
        TEST_INFO("Unexpected CORBA::Exception: " << e);
        CPPUNIT_ASSERT(false);
    }*/
    
}


void
FTCompWithTimersTest::test_ft_comp_with_timers_activate::test_connections()
{
//     add_nbOfRequestedTestOK(3);
    
    try {
        TEST_INFO("Test connections to ServerWithFacet group " 
                  <<  ")");    
    
        FTCompWithTimersTest::HostInfo_var hostInfo = m_session_context->get_connection_serverHostInfo();

        if (CORBA::is_nil(hostInfo.in())) {
            TEST_INFO("hostInfo is NIL !!!! ");
            CPPUNIT_ASSERT(false);
            return;
        }
        
        
    } catch (const CORBA::SystemException& e) {
        TEST_INFO("Unexpected CORBA::SystemException: " << e);
        CPPUNIT_ASSERT(false);
    } catch (const CORBA::Exception& e) {
        TEST_INFO("Unexpected CORBA::Exception: " << e);
        CPPUNIT_ASSERT(false);
    } catch (...) {
        TEST_INFO("Unknown exception !! ");
        CPPUNIT_ASSERT(false);
    }
    

    
}




