/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <ft_comp_with_timers/CCM_ServerWithFacet_impl.hpp>

#include <Foundation/ossupport/OS.hpp>

#include <Time/clockservice/Util.hpp>
#include <Time/clockservice/Time.hpp>

#include <Foundation/common/Assert.hpp>


//
// IDL:thalesgroup.com/FTCompWithTimersTest/CCM_ServerWithFacet:1.0
//
FTCompWithTimersTest::CCM_ServerWithFacet_impl::CCM_ServerWithFacet_impl()
{
   m_num_of_execution_done = 0;
}

FTCompWithTimersTest::CCM_ServerWithFacet_impl::~CCM_ServerWithFacet_impl()
{
    std::cout << "   ... CCM_ServerWithFacet_impl::~CCM_ServerWithFacet_impl called!" << std::endl;
}




//
// facet serverHostInfo
//
FTCompWithTimersTest::CCM_HostInfo_ptr 
FTCompWithTimersTest::CCM_ServerWithFacet_impl::get_serverHostInfo ()
    throw (CORBA::SystemException)
{
    std::cout << "   ... CCM_ServerWithFacet_impl::get_serverHostInfo()" << std::endl; 
    return FTCompWithTimersTest::CCM_HostInfo::_duplicate(this);
}

char*
FTCompWithTimersTest::CCM_ServerWithFacet_impl::get_hostname()
    throw (CORBA::SystemException)
{
    std::string hostname = Cdmw::OsSupport::OS::get_hostname();
    std::cout << "   ... CCM_ServerWithFacet_impl::get_hostname()  " << hostname << std::endl; 
    return CORBA::string_dup(hostname.c_str());
}    


//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
FTCompWithTimersTest::CCM_ServerWithFacet_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    
    
    std::cout << "   ... CCM_ServerWithFacet_impl::Set_session_context called!" << std::endl;
 
    m_session_context = ::FTCompWithTimersTest::CCM_ServerWithFacet_Context::_narrow(ctx); 
    
    // get context for services
    m_session_context_ext =
        CdmwComponents::SessionContextExt::_narrow(m_session_context.in());
    
}

void FTCompWithTimersTest::CCM_ServerWithFacet_impl::setClockCatalog()
{
    // get Time Service
    m_clock_catalog =
        CosClockService::ClockCatalog::_narrow(
            m_session_context_ext->resolve_service_reference("ClockService"));
}

void FTCompWithTimersTest::CCM_ServerWithFacet_impl::setExecutor()
{

    CosClockService::ClockCatalog::ClockEntry_var clockEntry =
        m_clock_catalog->get_entry("ControlledExecutor");
    // get associated clock
    
    CosClockService::Clock_var clock = 
        clockEntry->subject;
    CDMW_ASSERT(!CORBA::is_nil( clock.in()));
    m_executor = Cdmw::clock::ControlledExecutor::_narrow(clock.in());
}    
void FTCompWithTimersTest::CCM_ServerWithFacet_impl::setPeriodic()
{
    ::FTCompWithTimersTest::ServerWithFacet_var  ccm_component_ref = 
                ::FTCompWithTimersTest::ServerWithFacet::_nil();  
    try {
        std::cout << "   ... CCM_ServerWithFacet_impl:: get_CCM_object called!" << std::endl;
        CORBA::Object_var obj = 
                m_session_context_ext->get_CCM_object();
        ccm_component_ref = 
                FTCompWithTimersTest::ServerWithFacet::_narrow(obj.in());
    } catch (const Components::IllegalState& e) {
      // shoudn't happen
      std::cerr << "Error narrowing component to ServerWithFacet" << std::endl;
      CDMW_ASSERT(false);
    } catch (const CORBA::BAD_PARAM& e) {
      // shouldn't happen
      std::cerr << "Failed to narrow Component to Serveur! Is this call performed on Component itself ?" << std::endl;
      CDMW_ASSERT(false);
    }
    
    m_periodic =
        CosClockService::PeriodicExecution::Periodic::_nil();
    try {
        // narrow component reference as Periodic since it supports this interface
        m_periodic =
            CosClockService::PeriodicExecution::Periodic::_narrow(ccm_component_ref.in());
    }
    catch (const CORBA::BAD_PARAM& e) {
        std::cerr << "Failed to narrow Component in CosClockService.Periodic. " 
                  << "Does it support this interface ?" << std::endl;
        CDMW_ASSERT(false);
    }

}

//
// ccm_activate function : it actives the component (call by the container)
//
void
FTCompWithTimersTest::CCM_ServerWithFacet_impl::ccm_activate()
   throw(CORBA::SystemException)
{
    int          step         = 1; 
    unsigned long execution_limit = 30;
    const int    period_sec   = 4;
    // prepare args
    CORBA::Any_var params = new CORBA::Any();
    TimeBase::TimeT period_tmt =  Cdmw::clock::util::seconds2TimeT(period_sec);
    
    // get controller
    m_controller =
        m_executor->enable_periodic_execution(m_periodic.in());
    std::cout << "   ... Step "<< step++ <<" ... Set the timer ... " << std::endl;
    // set 2 expirations
    
    std::cout << "   ... Set the timer ... " << std::endl;
    m_controller->start(period_tmt, 
                      Cdmw::clock::util::seconds2TimeT(0), 
                      (CORBA::ULong)      execution_limit, 
                      params.in());

}


//
// ccm_passivate function : it makes stop the component (call by the container)
//

void
FTCompWithTimersTest::CCM_ServerWithFacet_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_ServerWithFacet_impl::Ccm_passivate called!" << std::endl;
} 



//
// ccm_remove function : it destroys the component (call by the container)
void
FTCompWithTimersTest::CCM_ServerWithFacet_impl::ccm_remove()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_ServerWithFacet_impl::Ccm_remove called!" << std::endl;
}


CORBA::Boolean FTCompWithTimersTest::CCM_ServerWithFacet_impl::do_work(const CORBA::Any& args)
      throw (CORBA::SystemException)
{
    std::cout << "   ... CCM_ServerWithFacet_impl::do_work called!" << std::endl;
    std::cout << "   ====== BEGIN OF PERIODIC EXECUTION ====="  << std::endl
              << "   = The CCMServer called is running on " << Cdmw::OsSupport::OS::get_hostname() << std::endl
              << "   = execution # " << ++m_num_of_execution_done <<  std::endl
              << "   ========================================"  << std::endl;
    Cdmw::OsSupport::OS::sleep(2000);
    

    return true;
}

void  
FTCompWithTimersTest::CCM_ServerWithFacet_impl::cdmw_initialise_component(const ::CdmwPlatformMngtBase::StartupKind& startup_kind)
      throw(CdmwPlatformMngt::ManagedComponent::BadOrder, CORBA::SystemException)
{

    int          step         = 1;
    std::cout << "   ... CCM_ServerWithFacet_impl::cdmw_initialise_component called!" << std::endl;
    std::cout << "   ... Step "<< step++ <<" ... Contact Clock Service ... " << std::endl;  
    std::cout << "   ... Step "<< step++ <<" ... Contact ClockCatalog  ... " << std::endl;
    
    setClockCatalog();
    std::cout << "   ... Step "<< step++ <<" ... get ControlledExecutor ... " << std::endl;
    setExecutor();
     
    std::cout << "   ... Step "<< step++ <<" ... Set Periodic  ... " << std::endl;
    setPeriodic();

    if (m_periodic == NULL)
       std::cout << "   ... Periodic Object is null  ... " << std::endl;

}

