/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef __INCL_CDWM_PRODTESTS_CCMCLIENTIMPL_HPP__
#define __INCL_CDWM_PRODTESTS_CCMCLIENTIMPL_HPP__

#include <Foundation/logging/Logger.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <CCMContainer/ccmcif/SessionContextExt_impl.hpp>
#include <Time/idllib/CosClockService.stub.hpp>

#include "test_ccm_container_virtual_size/User_container_virtual_size.skel.hpp"


namespace ContainerVirtualSize
{

    //
    // IDL:thalesgroup.com/UserContainerVirtualSize/CCM_Client:1.0
    //
    class CCM_Client_impl : 
        virtual public UserContainerVirtualSize::User_CCM_Client,
        virtual public Cdmw::OrbSupport::RefCountLocalObject
    {

    public:
      
        /**
         * Purpose:
         * <p> Constructor
         */
        CCM_Client_impl();

        /**
         * Purpose:
         * <p> Destructor
         */
        virtual ~CCM_Client_impl();

        /**
         * Purpose:
         * <p> Method called when a method is called in a recorded component
         *
         * @param header 
         * @param data
         *
         * @exception CORBA::SystemException when unmarshalling data
         */
        virtual void record(
                            const ::CdmwRecording::GroupHeader & header,
                            const ::CdmwRecording::GIOP_Buffers & data
                            )
          throw(CORBA::SystemException);

        //
        // Session component specific operations
        //
        /**
         * Purpose:
         * <p> Get reference on implementation of launcher interface
         *
         * @param ctx the new session context
         * @exception CORBA::SystemException because of a call on 
         */
        virtual void set_session_context(Components::SessionContext_ptr ctx)
            throw(CORBA::SystemException, Components::CCMException);
        
        /**
         * Purpose:
         * <p> Activate the component
         */
        void ccm_activate()
            throw(CORBA::SystemException, Components::CCMException);
        
        /**
         * Purpose:
         * <p> Deactivate the component
         */
        void ccm_passivate()
            throw(CORBA::SystemException, Components::CCMException);

        /**
         * Purpose:
         * <p> Remove the component
         */
        void ccm_remove()
            throw(CORBA::SystemException, Components::CCMException);

    protected:
        /**
         * Purpose:
         * <p> Copy Constructor
         */
        CCM_Client_impl(const CCM_Client_impl&);

        /**
         * Purpose:
         * <p> Assignment operator
         */ 
        void operator=(const CCM_Client_impl&);

    private:
        Cdmw::Logging::Logger* m_logger;
        ContainerVirtualSize::CCM_Client_Context_var m_session_context;
        CdmwComponents::SessionContextExt_var m_session_context_ext;
    };

}; // End of namespace ContainerVirtualSize


#endif // __INCL_CDWM_PRODTESTS_CCMCLIENTIMPL_HPP__
