/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef __INCL_CDWM_PRODTESTS_CCMSERVERFTCLIENTIMPL_HPP__
#define __INCL_CDWM_PRODTESTS_CCMSERVERFTCLIENTIMPL_HPP__

#include <Foundation/logging/Logger.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Time/idllib/CosClockService.stub.hpp>
#include <Time/clockservice/Util.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <Foundation/osthreads/ThreadHandle.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/osthreads/Mutex.hpp>
#include <CCMContainer/ccmcif/SessionContextExt_impl.hpp>

#include "test_ccm_container_virtual_size/container_virtual_size_cif.skel.hpp"
#include "test_ccm_container_virtual_size/User_container_virtual_size.skel.hpp"


namespace ContainerVirtualSize
{
    class CCM_ServerFTClientLB_impl;
    
    /**
     * Purpose:
     * <p> Define the tests to pass in order to validate the test
     */
    class ContainerVirtualSizeTester : public CppUnit::TestFixture
    {
        CCM_ServerFTClientLB_impl * m_serverFTclientLB;
        CCM_ServerFTClientLB_Context_var m_session_context;
        ServerFTClientLB_var m_component_ref;

    public:
        
        /**
         * Constructor
         */
        ContainerVirtualSizeTester()
        {
            std::cout
               << "ContainerVirtualSizeTester::ContainerVirtualSizeTester"
               << std::endl;

            CORBA::Object_var obj = 
                Cdmw::TestUtils::CORBATestManager::instance()->get_object(
                    "ft_context");
            m_session_context =
                CCM_ServerFTClientLB_Context::_narrow(
                    obj.in());
        };

        /**
         * Destructor
         */
        ~ContainerVirtualSizeTester()
        {
            std::cout
               << "ContainerVirtualSizeTester::~ContainerVirtualSizeTester"
               << std::endl;
        };

        /**
         * Testable operations
         */
        void do_tests();

    protected:
        CPPUNIT_TEST_SUITE( ContainerVirtualSizeTester );
        CPPUNIT_TEST( do_tests );
        CPPUNIT_TEST_SUITE_END();
    };


    /**
     * Purpose:
     * <p> Thread that run the test
     */
    class TestThread : public Cdmw::OsSupport::Thread 
    {
    public:
        /**
         * Constructor
         */
        TestThread()
        {
            std::cout << "TestThread::TestThread" << std::endl;
        };

    protected:
        /**
         * Method called to run the test
         */
        void run() throw()
        {
            Cdmw::TestUtils::CORBATestManager* testManager =
                Cdmw::TestUtils::CORBATestManager::instance();
            Cdmw::TestUtils::ThreadedTestRunner runner(
                testManager->get_controller());
            runner.addTest(
                CppUnit::TestFactoryRegistry::getRegistry(
                    "ContainerVirtualSizeTester").makeTest());
            runner.start();
            runner.join();

            // print test summary
            Cdmw::TestUtils::CORBATestManager::instance()->summary();

        };

    private:
        // Copy Constructor
        TestThread(const TestThread&);
        // Assignmemt operator
        TestThread& operator = (const TestThread&);
    };


    //
    // IDL:thalesgroup.com/ContainerVirtualSize/CCM_ServerFTClientLB:1.0
    //
    /**
     * Purpose:
     * <p> Component implementation of a ServerFTClientLB
     */
    class CCM_ServerFTClientLB_impl : 
        virtual public UserContainerVirtualSize::User_CCM_ServerFTClientLB,
        virtual public Cdmw::OrbSupport::RefCountLocalObject
    {

    public:
        /**
         * Purpose:
         * <p> Constructor
         */
        CCM_ServerFTClientLB_impl();

        /**
         * Purpose:
         * <p> Destructor
         */
        virtual ~CCM_ServerFTClientLB_impl();

        /**
         * Purpose:
         * <p> Method called by the supervision
         */
        virtual void notify(CdmwPlatformMngt::Event * an_event)
          throw(CORBA::SystemException);

        /**
         * Purpose:
         * <p> Accessor on hostname
         *
         * @exception CORBA::SystemException because CORBA::string_dup
         */
        virtual char* get_hostname()
          throw(CORBA::SystemException);
    
        /**
         * Purpose:
         * <p> FT Server asks LB Server
         *
         * @exception CORBA::SystemException because of a call on 
         */
        virtual void launch()
          throw(CORBA::SystemException);

        /**
         * Purpose:
         * <p> Get reference on implementation of hostinfo interface
         *
         * @exception CORBA::SystemException because of a call on 
         */
        virtual ContainerVirtualSize::CCM_HostInfo_ptr get_hostInfo ()
           throw(CORBA::SystemException);

        /**
         * Purpose:
         * <p> Get reference on implementation of launcher interface
         *
         * @exception CORBA::SystemException because of a call on 
         */
        virtual ContainerVirtualSize::CCM_Launcher_ptr get_launcher ()
          throw(CORBA::SystemException);
          
        /**
         * Purpose:
         * <p> Get reference on implementation of launcher interface
         *
         * @param ctx the new session context
         * @exception CORBA::SystemException because of a call on 
         */
        virtual void set_session_context (::Components::SessionContext_ptr ctx)
          throw(CORBA::SystemException, Components::CCMException);

        /**
         * Purpose:
         * <p> Activate the component
         */
        virtual void ccm_activate()
          throw(CORBA::SystemException, Components::CCMException);

        /**
         * Purpose:
         * <p> Deactivate the component
         */
        virtual void ccm_passivate()
          throw(CORBA::SystemException, Components::CCMException);
    
        /**
         * Purpose:
         * <p> Remove the component
         */
        virtual void ccm_remove()
          throw(CORBA::SystemException, Components::CCMException);

        /**
         * Purpose:
         * <p> Activate the component
         *
         * @return a pointer to the context
         */
        CCM_ServerFTClientLB_Context* get_context();

    protected:
        /**
         * Purpose:
         * <p> Copy constructor
         */
        CCM_ServerFTClientLB_impl(const CCM_ServerFTClientLB_impl&);
        
        /**
         * Purpose:
         * <p> Assignment operator
         */
         void operator=(const CCM_ServerFTClientLB_impl&);


    private:
        ContainerVirtualSize::CCM_ServerFTClientLB_Context_var m_session_context;
        CdmwComponents::SessionContextExt_var m_session_context_ext;
        CosClockService::Clock_var m_clock;
        Cdmw::Logging::Logger* m_logger;
        TestThread m_testThread;

        friend class ContainerVirtualSizeTester;
    };

}; // End of namespace ContainerVirtualSize

#endif // __INCL_CDWM_PRODTESTS_CCMSERVERFTCLIENTIMPL_HPP__
