/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef __INCL_CDWM_PRODTESTS_CCMSERVERFTCLIENTIMPL_HPP__
#define __INCL_CDWM_PRODTESTS_CCMSERVERFTCLIENTIMPL_HPP__

#include <Foundation/logging/Logger.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Time/idllib/CosClockService.stub.hpp>
#include <Time/clockservice/Util.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <Foundation/osthreads/ThreadHandle.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/osthreads/Mutex.hpp>
#include <CCMContainer/ccmcif/SessionContextExt_impl.hpp>

#include "test_ccm_container_virtual_size/container_virtual_size_cif.skel.hpp"
#include "test_ccm_container_virtual_size/User_container_virtual_size.skel.hpp"


namespace ContainerVirtualSize
{
//
// IDL:thalesgroup.com/ContainerVirtualSize/CCM_ServerLBClientFT:1.0
//
/**
 * Purpose:
 * <p> Component implementation of a ServerLBClientFT
 */
class CCM_ServerLBClientFT_impl 
   : virtual public ::UserContainerVirtualSize::User_CCM_ServerLBClientFT,
     virtual public Cdmw::OrbSupport::RefCountLocalObject
{
      
public:
    /**
     * Purpose:
     * <p> Constructor
     */
    CCM_ServerLBClientFT_impl();
      
    /**
     * Purpose:
     * <p> Destructor
     */
    virtual ~CCM_ServerLBClientFT_impl();
        

    /**
     * Purpose:
     * <p> Get names of a LB server and a FT server
     *
     * @param lbserver lb server name
     * @param ftserver ft server name
     *
     * @exception CORBA::SystemException because of a call on a servant
     */
    virtual void get_names(CORBA::String_out lbserver,
                           CORBA::String_out ftserver)
       throw(CORBA::SystemException);

    /**
     * Purpose:
     * <p> Get reference on servant of interface HostsName
     *
     * @exception CORBA::SystemException because of a call on a servant
     */
    virtual ContainerVirtualSize::CCM_HostsName_ptr get_hostsName()
       throw(CORBA::SystemException);

    /**
     * Purpose:
     * <p> Get reference on implementation of launcher interface
     *
     * @param ctx the new session context
     * @exception CORBA::SystemException because of a call on 
     */
    virtual void set_session_context (::Components::SessionContext_ptr ctx)
       throw(CORBA::SystemException, Components::CCMException);
      
    /**
     * Purpose:
     * <p> Activate the component
     */
    virtual void ccm_activate()
       throw(CORBA::SystemException, Components::CCMException);

    /**
     * Purpose:
     * <p> Deactivate the component
     */
    virtual void ccm_passivate()
       throw(CORBA::SystemException, Components::CCMException);
    
    /**
     * Purpose:
     * <p> Remove the component
     */
    virtual void ccm_remove()
       throw(CORBA::SystemException, Components::CCMException);


protected:
    /**
     * Purpose:
     * <p> Copy constructor
     */
    CCM_ServerLBClientFT_impl(const CCM_ServerLBClientFT_impl&);
        
    /**
     * Purpose:
     * <p> Assignment operator
     */
    void operator=(const CCM_ServerLBClientFT_impl&);


private:
        ContainerVirtualSize::CCM_ServerLBClientFT_Context_var m_session_context;
        CdmwComponents::SessionContextExt_var m_session_context_ext;
        Cdmw::Logging::Logger* m_logger;
    };

}; // End of namespace ContainerVirtualSize

#endif // __INCL_CDWM_PRODTESTS_CCMSERVERFTCLIENTIMPL_HPP__
