/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <SystemMngt/platforminterface/PlatformInterface.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtManagedElementContexts.stub.hpp>

#include "test_ccm_container_virtual_size/CCM_Client_impl.hpp"


namespace ContainerVirtualSize
{

CCM_Client_impl::CCM_Client_impl(const CCM_Client_impl& impl)
{
    CDMW_LOG_FUNCTION(m_logger, "CCM_Client_impl::CCM_Client_impl Copy Constructor");
}

void CCM_Client_impl::operator=(const CCM_Client_impl& impl)
{
   CDMW_LOG_FUNCTION(m_logger, "CCM_Client_impl::operator=");
}

//
// IDL:thalesgroup.com/ContainerVirtualSize/CCM_Client:1.0
//
CCM_Client_impl::CCM_Client_impl()
    :
    m_logger(Cdmw::Logging::Logger::Get_Logger("CCM_Client",
                                               "QUALIF_CCM_CONTAINER_VIRTUAL_SIZE")),
    m_session_context(CCM_Client_Context::_nil()),
    m_session_context_ext(CdmwComponents::SessionContextExt::_nil())
{
    CDMW_LOG_FUNCTION(m_logger, "CCM_Client_impl::CCM_Client_impl Constructor");
}

CCM_Client_impl::~CCM_Client_impl()
{
    CDMW_LOG_FUNCTION(m_logger, "CCM_Client_impl::~CCM_Client_impl Destructor");
}


void 
CCM_Client_impl::record( const ::CdmwRecording::GroupHeader & header,
                         const ::CdmwRecording::GIOP_Buffers & data)
  throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger, "CCM_Client_impl::record");
}

//
// set_session_context function : it fixes the session context of the component
// (call by the container)
//
void
CCM_Client_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException, Components::CCMException)
{
    CDMW_LOG_FUNCTION(m_logger, "CCM_Client_impl::set_session_context");

    m_session_context =
        ContainerVirtualSize::CCM_Client_Context::_narrow(ctx); 

    // get context for services
    m_session_context_ext = 
        CdmwComponents::SessionContextExt::_narrow(m_session_context.in());

}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CCM_Client_impl::ccm_activate()
    throw(CORBA::SystemException, Components::CCMException)
{
    CDMW_LOG_FUNCTION(m_logger, "CCM_Client_impl::ccm_activate");

    try {
        // Get reference on FT Server Client LB
        ContainerVirtualSize::Launcher_var launcher = 
          this->m_session_context->get_connection_launcher();
        
        // Ask it to run
        launcher->launch();
    }
    catch (const CORBA::SystemException& e)
    {
        std::cerr << "SystemException :" 
                  << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
                  << std::endl;
        CDMW_ASSERT(false);
    }
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
CCM_Client_impl::ccm_passivate()
    throw(CORBA::SystemException, Components::CCMException)
{
    CDMW_LOG_FUNCTION(m_logger, "CCM_Client_impl::ccm_passivate");
} 



//
// ccm_remove function : it destroys the component (call by the container)
//
void
CCM_Client_impl::ccm_remove()
    throw(CORBA::SystemException, Components::CCMException)
{
    CDMW_LOG_FUNCTION(m_logger, "CCM_Client_impl::ccm_remove");
}

} // End of namespace ContainerVirtualSize
