/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/common/Assert.hpp>
#include <Foundation/common/Exception.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/logging/FunctionLogger.hpp>

#include "test_ccm_container_virtual_size/CCM_ServerFTClientLB_impl.hpp"

namespace ContainerVirtualSize
{

CCM_ServerFTClientLB_impl::CCM_ServerFTClientLB_impl(const CCM_ServerFTClientLB_impl& impl)
{
    CDMW_LOG_FUNCTION(m_logger,"CCM_ServerFTClientLB_impl::CCM_ServerFTClientLB_impl Copy Constructor");
}
        
void CCM_ServerFTClientLB_impl::operator=(const CCM_ServerFTClientLB_impl& impl)
{
   CDMW_LOG_FUNCTION(m_logger,"CCM_ServerFTClientLB_impl::operator=");
}

//
// IDL:thalesgroup.com/ContainerVirtualSize/CCM_ServerFTClientLB:1.0
//
CCM_ServerFTClientLB_impl::CCM_ServerFTClientLB_impl()
    :
    m_session_context(CCM_ServerFTClientLB_Context::_nil()),
    m_session_context_ext(CdmwComponents::SessionContextExt::_nil()),
    m_clock(CosClockService::Clock::_nil()),
    m_logger(Cdmw::Logging::Logger::Get_Logger("CCM_ServerFTClientLB",
                                               "QUALIF_CONTAINER_VIRTUAL_SIZE")),
    m_testThread()
{
    CDMW_LOG_FUNCTION(m_logger,"CCM_ServerFTClientLB_impl::CCM_ServerFTClientLB_impl Constructor");
}

CCM_ServerFTClientLB_impl::~CCM_ServerFTClientLB_impl()
{
    CDMW_LOG_FUNCTION(m_logger,"CCM_ServerFTClientLB_impl::~CCM_ServerFTClientLB_impl Destructor");
}
      
void
CCM_ServerFTClientLB_impl::notify(CdmwPlatformMngt::Event * an_event)
  throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger, "CCM_ServerFTClientLB_impl::notify");
}

char*
CCM_ServerFTClientLB_impl::get_hostname()
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger, "CCM_ServerFTClientLB_impl::get_hostname");
    
    TimeBase::TimeT now = m_clock->current_time();
    CDMW_INFO(m_logger, "time = " << now);

    std::string hostname = Cdmw::OsSupport::OS::get_hostname();
    CDMW_INFO(m_logger, "hostname = " << hostname);
    
    return CORBA::string_dup(hostname.c_str());
}

void CCM_ServerFTClientLB_impl::launch()
  throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger, "CCM_ServerFTClientLB_impl::launch");

    // Start the test
    m_testThread.start();
}


ContainerVirtualSize::CCM_HostInfo_ptr 
CCM_ServerFTClientLB_impl::get_hostInfo()
  throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger, "CCM_ServerFTClientLB_impl::get_hostInfo");

    return CCM_HostInfo::_duplicate(this);
}

ContainerVirtualSize::CCM_Launcher_ptr
CCM_ServerFTClientLB_impl::get_launcher()
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger, "CCM_ServerFTClientLB_impl::get_launcher");

    return CCM_Launcher::_duplicate(this);
}

//
// set_session_context function : it fixes the session context of the component
// (call by the container)
//
void
CCM_ServerFTClientLB_impl::set_session_context(
    Components::SessionContext_ptr ctx
    )
  throw (CORBA::SystemException, Components::CCMException)
{
    CDMW_LOG_FUNCTION(m_logger, "CCM_ServerFTClientLB_impl::set_session_context");
    
    m_session_context = 
      ContainerVirtualSize::CCM_ServerFTClientLB_Context::_narrow(ctx); 

    // get context for services
    m_session_context_ext = 
        CdmwComponents::SessionContextExt::_narrow(m_session_context.in());

    // get the Time service
    std::cout << "get the time..." << std::endl;
    CosClockService::ClockCatalog_var clock_catalog = 
        CosClockService::ClockCatalog::_narrow(
            m_session_context_ext->resolve_service_reference("ClockService"));
    CosClockService::ClockCatalog::ClockEntry_var clockEntry =
        clock_catalog->get_entry("LocalExecutor");
    m_clock = clockEntry->subject;
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CCM_ServerFTClientLB_impl::ccm_activate()
    throw (CORBA::SystemException, Components::CCMException)
{
    CDMW_LOG_FUNCTION(m_logger, "CCM_ServerFTClientLB_impl::ccm_activate");

    try 
    {
       CDMW_DEBUG(m_logger, "Register the FT component");
    
        Cdmw::TestUtils::CORBATestManager::instance()->add(
           "ft_context",
           get_context()
           );
        Cdmw::TestUtils::CORBATestManager::instance()->add(
           "user_ccm_serverftclientlb",
           static_cast< UserContainerVirtualSize::User_CCM_ServerFTClientLB* >
           (this)

           );

        // call SessionContext::get_CCM_objet()
        CORBA::Object_var obj = m_session_context_ext->get_CCM_object();            
        ContainerVirtualSize::ServerFTClientLB_var the_component = 
              ContainerVirtualSize::ServerFTClientLB::_narrow(obj.in());
        Cdmw::TestUtils::CORBATestManager::instance()->add(
            "the_ftcomponent", 
            the_component.in());
    }
    catch (const CORBA::UserException& e) {
        std::cerr << "UserException during get_CCM_Object() :" 
                  << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
                  << std::endl;
        CDMW_ASSERT(false);
    }
    catch (const CORBA::SystemException& e) {
        std::cerr << "SystemException during get_CCM_Object() :" 
                  << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
                  << std::endl;
        CDMW_ASSERT(false);
    }
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
CCM_ServerFTClientLB_impl::ccm_passivate()
    throw (CORBA::SystemException, Components::CCMException)
{
    CDMW_LOG_FUNCTION(m_logger, "CCM_ServerFTClientLB_impl::ccm_passivate");
}


//
// ccm_remove function : it destroys the component (call by the container)
//
void
CCM_ServerFTClientLB_impl::ccm_remove()
    throw (CORBA::SystemException, Components::CCMException)
{
    CDMW_LOG_FUNCTION(m_logger, "CCM_ServerFTClientLB_impl::ccm_remove");
}

CCM_ServerFTClientLB_Context* CCM_ServerFTClientLB_impl::get_context()
{
    CDMW_LOG_FUNCTION(m_logger, "CCM_ServerFTClientLB_impl::get_context");

    return CCM_ServerFTClientLB_Context::_duplicate(m_session_context.in());
}

//
// Testable do_tests() operation
//

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(
    ContainerVirtualSizeTester,
    "ContainerVirtualSizeTester"
    );

void
ContainerVirtualSizeTester::do_tests()
{
    TEST_INFO("ContainerVirtualSizeTester::do_tests");
    TEST_INFO("Sleep 2 seconds");
    Cdmw::OsSupport::OS::sleep(2000);

    TEST_INFO("Get user_ccm_serverftclientlb");
    CORBA::Object_var obj = 
        Cdmw::TestUtils::CORBATestManager::instance()->get_object(
            "user_ccm_serverftclientlb");
    UserContainerVirtualSize::User_CCM_ServerFTClientLB_var 
      user_ccm_serverFTclientLB =
      UserContainerVirtualSize::User_CCM_ServerFTClientLB::_narrow(obj.in());
    this->m_serverFTclientLB =
        reinterpret_cast< ContainerVirtualSize::CCM_ServerFTClientLB_impl* >(
            user_ccm_serverFTclientLB.in());
    
    TEST_INFO("Get the_ftcomponent");
    obj = 
       Cdmw::TestUtils::CORBATestManager::instance()->get_object(
          "the_ftcomponent");
    this->m_component_ref =
      ServerFTClientLB::_narrow(obj.in());

    try
    {
        // perform 10 requests with the RoundRobin policy;
        // each request is expected to be carried out by a LB server
        // different from the previous one
        //
        TEST_INFO("Sleep 3 seconds");
        Cdmw::OsSupport::OS::sleep(3000);

        ContainerVirtualSize::HostsName_var serverLB =
          this->m_session_context->get_connection_hostsName();
        
        CORBA::String_var previous_serverlb_name = CORBA::string_dup("");
        for (int i = 0; i < 11; i++)
        {
            CORBA::String_var serverlb_name = CORBA::string_dup("");
            CORBA::String_var serverft_name = CORBA::string_dup("");
            serverLB->get_names ( serverlb_name, serverft_name );

            CPPUNIT_ASSERT(strcmp(serverlb_name.in(), ""));
            CPPUNIT_ASSERT(strcmp(serverft_name.in(), ""));

            if ( i > 0 )
              {
                TEST_INFO("REQUEST [" << i << "] ...check the RoundRobin policy...") ;
                TEST_INFO("The server LB called is ==> "
                          << serverlb_name.in());
                
                
                
                TEST_INFO("The primary server FT is ==> "
                          << serverft_name.in());

                int result = 
                  strcmp(serverlb_name.in(), previous_serverlb_name.in());

                TEST_INFO("Comparing " << serverlb_name.in() 
                          << " to " << previous_serverlb_name.in() 
                          << "=" << result);

                if ( result == 0 )
                  {
                    previous_serverlb_name = serverlb_name;
                    CPPUNIT_ASSERT(true);
                    
                    std::string hostname;
                    try
                      {
                        hostname = Cdmw::OsSupport::OS::get_hostname();
                      }
                    catch (const Cdmw::InternalErrorException& iee)
                      {
                        std::stringstream stream;
                        stream << "[EXCEPTION] InternalErrorException:";
                        stream << iee.what();
                        TEST_INFO(stream.str());
                      }

                    result = strcmp(serverft_name.in(), hostname.c_str());

                    TEST_INFO("Comparing " << serverft_name.in()
                          << " to " << hostname.c_str() 
                          << "=" << result);

                    CPPUNIT_ASSERT(result == 0);
                  }
                else
                  {
                    CPPUNIT_ASSERT_MESSAGE(
                                           "The RoundRobin policy has not been respected", false);
                  }
              }
            else
              {
                previous_serverlb_name = serverlb_name;
                TEST_INFO("Previous set to " << previous_serverlb_name.in());
              }
        }
        
    }
    catch (const std::exception& e)
    {
        // catch any CPPUnit exception, print it and throw it
        TEST_INFO(e.what());
        throw e;
    }
}

} // End of namespace ContainerVirtualSize

