/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/common/Assert.hpp>
#include <Foundation/common/Exception.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/logging/FunctionLogger.hpp>

#include "test_ccm_container_virtual_size/CCM_ServerLBClientFT_impl.hpp"

namespace ContainerVirtualSize
{

CCM_ServerLBClientFT_impl::CCM_ServerLBClientFT_impl(const CCM_ServerLBClientFT_impl& impl)
{
    CDMW_LOG_FUNCTION(m_logger,"CCM_ServerLBClientFT_impl::CCM_ServerLBClientFT_impl copy contructor");
}
        
void CCM_ServerLBClientFT_impl::operator=(const CCM_ServerLBClientFT_impl& impl)
{
    CDMW_LOG_FUNCTION(m_logger,"CCM_ServerLBClientFT_impl::operator=");
}

//
// IDL:thalesgroup.com/ContainerVirtualSize/CCM_ServerLBClientFT:1.0
//
CCM_ServerLBClientFT_impl::CCM_ServerLBClientFT_impl()
    :
    m_session_context(CCM_ServerLBClientFT_Context::_nil()),
    m_session_context_ext(CdmwComponents::SessionContextExt::_nil()),
    m_logger(Cdmw::Logging::Logger::Get_Logger("CCM_ServerLBClientFT",
                                               "QUALIF_CONTAINER_VIRTUAL_SIZE"))
{
    CDMW_LOG_FUNCTION(m_logger,"CCM_ServerLBClientFT_impl::CCM_ServerLBClientFT_impl");
}

CCM_ServerLBClientFT_impl::~CCM_ServerLBClientFT_impl()
{
    CDMW_LOG_FUNCTION(m_logger, "CCM_ServerLBClientFT_impl::~CCM_ServerLBClientFT_impl destructor");
}
      
void CCM_ServerLBClientFT_impl::get_names ( CORBA::String_out lbserver,
                                            CORBA::String_out ftserver )
  throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger, "CCM_ServerLBClientFT_impl::get_names");

    std::string hostname = Cdmw::OsSupport::OS::get_hostname();
    CDMW_INFO(m_logger, "hostname = " << hostname);

    ContainerVirtualSize::HostInfo_var serverFT =
          this->m_session_context->get_connection_hostInfo();

    lbserver = CORBA::string_dup(hostname.c_str());
    ftserver = CORBA::string_dup(serverFT->get_hostname());
}

ContainerVirtualSize::CCM_HostsName_ptr CCM_ServerLBClientFT_impl::get_hostsName()
  throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger, "CCM_ServerLBClientFT_impl::get_hostsName");

    return CCM_HostsName::_duplicate(this);
}


//
// set_session_context function : it fixes the session context of the component
// (call by the container)
//
void
CCM_ServerLBClientFT_impl::set_session_context(
    Components::SessionContext_ptr ctx
    )
  throw (CORBA::SystemException, Components::CCMException)
{
    CDMW_LOG_FUNCTION(m_logger, "CCM_ServerLBClientFT_impl::set_session_context");

    m_session_context = 
      ContainerVirtualSize::CCM_ServerLBClientFT_Context::_narrow(ctx); 

    // get context for services
    m_session_context_ext = 
        CdmwComponents::SessionContextExt::_narrow(m_session_context.in());
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CCM_ServerLBClientFT_impl::ccm_activate()
    throw (CORBA::SystemException, Components::CCMException)
{
    CDMW_LOG_FUNCTION(m_logger, "CCM_ServerLBClientFT_impl::ccm_activate");
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
CCM_ServerLBClientFT_impl::ccm_passivate()
    throw (CORBA::SystemException, Components::CCMException)
{
    CDMW_LOG_FUNCTION(m_logger, "CCM_ServerLBClientFT_impl::ccm_passivate");
}


//
// ccm_remove function : it destroys the component (call by the container)
//
void
CCM_ServerLBClientFT_impl::ccm_remove()
    throw (CORBA::SystemException, Components::CCMException)
{
    CDMW_LOG_FUNCTION(m_logger, "CCM_ServerLBClientFT_impl::ccm_remove");
}

} // End of namespace ContainerVirtualSize

