/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>

#include <Foundation/common/System.hpp>
#include <Foundation/orbsupport/Codec.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <FaultTolerance/ftinit/FTServiceInit.hpp>
#include <ConfAndPlug/cdmwinit/CdmwInterface.hpp>
#include <LoadBalancing/lbinit/LBServiceInit.hpp>

#include <SystemMngt/platformvaluetypes/SystemModeChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/SystemStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/HostStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/ApplicationAgentDeath_impl.hpp>
#include <SystemMngt/platformvaluetypes/ApplicationModeChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/ApplicationStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/EntityStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/PropertyChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/UserEvent_impl.hpp>

#include "test_ccm_container_virtual_size/CCM_ServerFTClientLBHome_impl.hpp"

namespace
{
    const int SUCCESS = 0;
    const int FAILURE = 1;

    const int POA_THREAD_POOL_SIZE = 5;    

} // End anonymous namespace


void register_supervision_events_factories(CORBA::ORB_ptr orb);


/**
 * Method main called to launch the server
 *
 * @param argc number of arguments
 * @param args arguments of the server
 */
int main(int argc, char* argv[])
{    
    int status = SUCCESS;

    CORBA::ORB_var orb = CORBA::ORB::_nil();

    try
    {
        // Initialize LB service and FT service
        std::cout 
           << "********** Cdmw::CdmwInit::CDMW_preORBInit (ServerFTClientLB) **********" 
           << std::endl;
        Cdmw::CdmwInit::CDMW_preORBInit(argc, argv);

        // Initialize the ORB
        Cdmw::OrbSupport::StrategyList orb_strategies;
        orb_strategies.add_OrbThreaded();
        orb_strategies.add_PoaThreadPool(POA_THREAD_POOL_SIZE);
        orb_strategies.add_multicast();

        std::cout 
          << "********** Cdmw::OrbSupport::OrbSupport::ORB_init (ServerFTClientLB) **********" 
          << std::endl;
        orb = Cdmw::OrbSupport::OrbSupport::ORB_init(
            argc, argv, orb_strategies);

        // Get the root POA 
        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(
            obj.in());

        //
        // Run implementation (Activate the POA manager)
        //
        std::cout 
           << "********** Activate POA Manager (ServerFTClientLB) **********"
           << std::endl;
        PortableServer::POAManager_var poaManager = rootPOA->the_POAManager();
        poaManager->activate();
        Cdmw::TestUtils::CORBATestManager::instance()->init(
            orb.in(),rootPOA.in(),argc,argv);

        std::cout 
           << "********** Cdmw::CdmwInit::CDMW_init (ServerFTClientLB) **********" 
           << std::endl;
        Cdmw::CdmwInit::CDMW_init(orb.in(), argc, argv);

        // create all Value factories relative to supervision
        register_supervision_events_factories(orb.in());

        // Setup signal handling for ORB to be shut down
        Cdmw::OrbSupport::OrbSupport::install_orb_shutdown_signal_handlers(
            orb.in());

        std::cout 
           << "********** ORB Run (ServerFTClientLB) **********"
           << std::endl;
        orb->run();
        Cdmw::OrbSupport::OrbSupport::uninstall_orb_shutdown_signal_handlers(
            orb.in());

    }

    catch (const Cdmw::Exception & ex)
    {
        std::cerr << ex.what() << std::endl;
        status = FAILURE;
    }
    catch(const CORBA::Exception& ex)
    {
        std::cerr << ex << std::endl;
        status = FAILURE;
    }

    if(! CORBA::is_nil(orb.in()))
    {
        try
        {
            std::cout 
               << "********** cleanup (ServerFTClientLB) **********"
               << std::endl;
            Cdmw::CdmwInit::CDMW_cleanup(orb.in());
            Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
            std::cout 
               << "********** ORB destroy (ServerFTClientLB) **********"
               << std::endl;
            orb->destroy();
        }
        catch(const CORBA::Exception& ex)
        {           
            std::cerr << ex << std::endl;
            status = FAILURE;
        }
    }

    return status;
}

void register_supervision_events_factories(CORBA::ORB_ptr orb)
{
    using namespace Cdmw::PlatformMngt;
    // Register event factories
    CORBA::ValueFactoryBase_var factory, old_factory;
    
    factory = new SystemModeChangeFactory;
    old_factory = orb->register_value_factory(
        "IDL:thalesgroup.com/CdmwPlatformMngt/SystemModeChange:1.0",
        factory.in() );
    
    factory = new SystemStatusChangeFactory;
    old_factory = orb->register_value_factory(
        "IDL:thalesgroup.com/CdmwPlatformMngt/SystemStatusChange:1.0",
        factory.in() );
    
    factory = new HostStatusChangeFactory;
    old_factory = orb->register_value_factory(
        "IDL:thalesgroup.com/CdmwPlatformMngt/HostStatusChange:1.0",
        factory.in() );
    
    factory = new ApplicationAgentDeathFactory;
    old_factory = orb->register_value_factory(
        "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationAgentDeath:1.0",
        factory.in() );
    
    factory = new ApplicationModeChangeFactory;
    old_factory = orb->register_value_factory(
        "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationModeChange:1.0",
        factory.in() );
    
    factory = new ApplicationStatusChangeFactory;
    old_factory = orb->register_value_factory(
        "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationStatusChange:1.0",
        factory.in() );
    
    factory = new ProcessMessageFactory;
    old_factory = orb->register_value_factory(
        "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessMessage:1.0",
        factory.in() );
    
    factory = new ProcessStatusChangeFactory;
    old_factory = orb->register_value_factory(
        "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessStatusChange:1.0",
        factory.in() );
    
    factory = new EntityStatusChangeFactory;
    old_factory = orb->register_value_factory(
        "IDL:thalesgroup.com/CdmwPlatformMngt/EntityStatusChange:1.0",
        factory.in() );
    
    factory = new UserEventFactory;
    old_factory = orb->register_value_factory(
        "IDL:thalesgroup.com/CdmwPlatformMngt/UserEvent:1.0",
        factory.in() );
    
    factory = new PropertyChangeFactory;
    old_factory = orb->register_value_factory(
        "IDL:thalesgroup.com/CdmwPlatformMngt/PropertyChange:1.0",
        factory.in() );
}
