/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.containervirtualsize;

import cdmw.orbsupport.ORBSupport;
import cdmw.orbsupport.StrategyList;
import cdmw.cdmwinit.CdmwInterface;


/**
 * This class is the main class to launch a ServerLBClientFT
 */
public class ServerLBClientFT 
{
    /**
     * Size of the orb thread pool
     */
    public final static int POA_THREAD_POOL_SIZE = 5;


    /**
     * Purpose:
     * <p> Method main called to launch the server
     *
     * @param args arguments of the server
     */
    public static void main(String[] args) {
        
        org.omg.CORBA.ORB orb = null;
       
        try {
            System.out.println("********** CdmwInterface.cdmwPreORBInit (ServerLBClientFT) **********");
            CdmwInterface.cdmwPreORBInit(
                args,
                "cdmw.generated.ServerLBClientFTCdmwInterface");

            // Initialize the ORB
            StrategyList orbStrategies = new StrategyList();
            orbStrategies.addORBThreaded();
            orbStrategies.addPOAThreadPool(POA_THREAD_POOL_SIZE);
            
            System.out.println("********** ORBSupport.ORBInit (ServerLBClientFT) **********");
            orb = ORBSupport.ORBInit(args, orbStrategies);

            System.out.println("********** CdmwInterface.cdmwInit (ServerLBClientFT) **********");
            CdmwInterface.cdmwInit(
                orb, args, null,
                "cdmw.generated.ServerLBClientFTCdmwInterface");

            // Get the root POA 
            org.omg.CORBA.Object obj = 
                orb.resolve_initial_references("RootPOA");
            org.omg.PortableServer.POA rootPOA = 
                org.omg.PortableServer.POAHelper.narrow(obj);

            // Run implementation (Activate the POA manager)
            System.out.println("********** Activation of POA Manager (ServerLBClientFT) **********");
            rootPOA.the_POAManager().activate();
    
            System.out.println("********** ORB run (ServerLBClientFT) **********");
            orb.run();
        }
        catch (Exception ex) {
            System.err.println(ex.getMessage());
        }
    
        if (orb != null) {
            try
            {
                System.out.println("********** CdmwInterface.cdmwCleanup (ServerLBClientFT) **********");
                CdmwInterface.cdmwCleanup(
                    orb,
                    "cdmw.generated.ServerLBClientFTCdmwInterface");

                System.out.println("********** ORB destroy (ServerLBClientFT) **********");
                orb.destroy();
            }
            catch(Exception ex) {
                System.err.println(ex.getMessage());
            }
        }
    } /* End of method main */
} /* End of class */
