/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "test_ccm_invocation/ClientProcessControl.hpp"
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/idllib/Components.stub.hpp>


using namespace utils;

namespace Invocation
{
    
ClientProcessControl::ClientProcessControl( CORBA::ORB_ptr orb, cfgData& cfg, const std::string& serverHostName)
    throw(CORBA::SystemException)
    : m_orb( CORBA::ORB::_duplicate( orb))
    , m_applicationName( "")
    , m_processName( "")
    , m_serverHostName( serverHostName)
    , m_cfg( cfg) {
    // ===================================================
    // Instantiate time reading object
    // ===================================================
    std::string measure_str = cfg.measure_type[0];

    if ( measure_str.compare( MEASURE_TH ) == 0 ) {
        m_measure = new ClockCpuTime;
    } else if ( measure_str.compare( MEASURE_TK ) == 0 ) {
        m_measure = new ClockCpuTick;
    } else { // using default 
        m_measure = new ClockHighResTime;
    }
}
  
ClientProcessControl::~ClientProcessControl()
    throw() {
}
  
  
  
// process initialisation called by platformmngt     
void ClientProcessControl::on_initialise( const CdmwPlatformMngtBase::StartupKind& startup_kind)
    throw(CORBA::SystemException) {
    // ===================================================
    // Get the process name and application name
    // ===================================================
    m_processName = Cdmw::CdmwInit::InitUtils::get_cdmw_process_name();
    
    m_applicationName = Cdmw::CdmwInit::InitUtils::get_cdmw_application_name();
    
    // =================================================================
    // example of using the PlatformInterface for notifying a message
    // =================================================================
    Cdmw::PlatformMngt::PlatformInterface::Notify_message( CdmwPlatformMngtBase::INF,
                                                           m_processName.c_str(), 
                                                           ">>>>>>>>>>>>>> Initialisation requested by supervision");
    
    // =========================================
    // retrieve Tester object from Repository
    // =========================================
        
    // retrieve "ServerInst" component from Repository
    try {
        std::string admin_ctx_cobaloc = 
            "corbaname::localhost:22456#CDMW/ADMIN/GlobalRepository";
            
        std::cout << "Try to get AdminRoot context" << std::endl;
        CORBA::Object_var obj = m_orb->string_to_object(admin_ctx_cobaloc.c_str());
        
        std::cout << "Try to narrow as NameContext" << std::endl;
        CosNaming::NamingContext_var ctx = CosNaming::NamingContext::_narrow(obj.in());
        if (CORBA::is_nil(ctx.in())) {
            std::cerr << "ERROR: Retrieved object is not a NamingContext" << std::endl;
            return;
        }
        
        Cdmw::CommonSvcs::Naming::NamingInterface ni(ctx.in());
        
        std::cout << "Try to get Server component from AdminRoot context" << std::endl;
        std::string name = m_serverHostName + "/CcmPerfApplication/ServerProcess/CCM/Components/ServerInst";
        
        obj = ni.resolve(name);
        
        std::cout << "Try to narrow as CCMObject" << std::endl;
        Components::CCMObject_var comp = 
            Components::CCMObject::_narrow(obj.in());
        if (CORBA::is_nil(comp.in())) {
            std::cerr << "ERROR: Retrieved object is not a CCMObject" << std::endl;
            return;
        }
        
        std::cout << "Try to get server_facet" << std::endl;
        obj = comp->provide_facet("server_facet");
        std::cout << "Try to narrow as Invocation::TestPing" << std::endl;
        m_testping = Invocation::TestPing::_narrow(obj.in());
        if (CORBA::is_nil(m_testping.in())) {
            std::cerr << "ERROR: Retrieved object is not a Invocation::TestPing" << std::endl;
            return;
        }
        
        std::cout << "Succeed to get server_facet !" << std::endl;
        
    } catch (const CORBA::Exception& e) {
        std::cerr << "ERROR: " 
                  << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
                  << std::endl;
    }
    
    
}
  
  
// process to run called by platformmngt    
void ClientProcessControl::on_run()
    throw(CdmwPlatformMngt::ProcessDelegate::NotReadyToRun, CORBA::SystemException) {
    // =================================================================
    // example of using the PlatformInterface for notifying a message
    // =================================================================
    Cdmw::PlatformMngt::PlatformInterface::Notify_message( CdmwPlatformMngtBase::INF,
                                                           m_processName.c_str(), 
                                                           ">>>>>>>>>>>>>> Run requested by supervision");
    
    std::cout << "Start testing" << std::endl;
    
    m_niter = m_cfg.niter[0];
    m_fileprefix = m_cfg.fileprefix[0];
    
    long double tdelta[m_niter];

    for ( size_t i_del = 0; i_del < m_cfg.delay.size(); i_del++ ) {
        // assign current delay
        m_delay = m_cfg.delay[ i_del ];
        //delay of measure
        timespec t_delay;
        t_delay.tv_sec  = m_delay / 1000;
        t_delay.tv_nsec = (m_delay % 1000) * 1000000;

        for ( size_t i_inv = 0; i_inv < m_cfg.inv_type.size(); i_inv++ ) { //for each invocation type (CR, OW)
            //result of time measurement    
            long double time1, time2;
            
            m_invocation_type = m_cfg.inv_type[i_inv];
            
            if ( m_invocation_type.compare( CALL_RETURN ) == 0 ) { //test CR: call and return
                std::cout << "Test Call&Return calls" << std::endl;
                
                //start of Invocation test
                for(int i = 0; i <  m_niter; i++){
                    //delay of measure
                    if ( m_delay != 0) {
                        // call to nanosleep
                        nanosleep( &t_delay, NULL);
                    }

                    time1 = m_measure->getTime();
                    m_testping->invocation_ping();
                    time2 = m_measure->getTime();
                    tdelta[i] = time2 - time1;

                }
            } else { // ONE_WAY
                std::cout << "Test OneWay calls" << std::endl;
                CORBA::Object_var obj =
                    m_orb->resolve_initial_references ("ORBPolicyManager");
                
                CORBA::PolicyManager_var policy_manager =
                    CORBA::PolicyManager::_narrow (obj.in ());
      
                obj = m_orb->resolve_initial_references ("PolicyCurrent");
                CORBA::PolicyCurrent_var policy_current =
                    CORBA::PolicyCurrent::_narrow (obj.in());
      
                // Default SyncScope value (ONE_WAY).
                static Messaging::SyncScope sync_scope = Messaging::SYNC_NONE;
      
                // Set up the sync scope any.
                CORBA::Any sync_scope_any;
                sync_scope_any <<= sync_scope;
      
                // Set the length of the policy list.
                CORBA::PolicyList sync_scope_policy_list (1);
                sync_scope_policy_list.length (1);
      
                // Set up the sync scope policy.
                sync_scope_policy_list[0] =
                    m_orb->create_policy( Messaging::SYNC_SCOPE_POLICY_TYPE,
                                          sync_scope_any);
      
                // Set the sync scope policy at the object level.
                obj = m_testping->_set_policy_overrides( sync_scope_policy_list,
                                                         CORBA::ADD_OVERRIDE);

                // Get the new object reference with the updated policy.
                m_testping = Invocation::TestPing::_narrow( obj.in());

                //start of Invocation test
                for ( int i = 0; i <  m_niter; i++) {
                    //delay of measure
                    if ( m_delay!=0) {
                        // call to nanosleep
                        nanosleep( &t_delay, NULL);
                    }

                    time1 = m_measure->getTime();
                    m_testping->ow_invocation_ping();
                    time2 = m_measure->getTime();
                    //time read in micro sec.   
                    tdelta[i] = time2 - time1;
                }
            }
      
            //write file
            std::ostringstream nameStream;
            nameStream << m_cfg.inv_type[i_inv] << "_" << m_cfg.delay[i_del]  << "_" << "DELTA_C.dat";
            
            std::string fileDeltaName( m_fileprefix + nameStream.str());
            
            ofstream fd( fileDeltaName.c_str());
            fd.precision(6);
            
            for ( int i = 0; i < m_niter; i++) {
                fd << tdelta[i] << std::endl;
            }
    
            fd.close();
        } // end for i_inv
    } // end for i_del

    std::cerr << "** test execution successfull terminated : please run the data anaylsis script" << std::endl;
}
  
    
// process to stop called by platformmngt
void ClientProcessControl::on_stop()
    throw(CORBA::SystemException) {
    // =================================================================
    // example of using the PlatformInterface for notifying a message
    // =================================================================
    Cdmw::PlatformMngt::PlatformInterface::Notify_message( CdmwPlatformMngtBase::INF,
                                                           m_processName.c_str(), 
                                                           ">>>>>>>>>>>>>> Stop requested by supervision");
    m_orb->shutdown( true );
}

} // End namespace Invocation

  
  
