/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.qualification.perftest.testccminvocation;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;

import cdmw.qualification.perftest.testperfcommon.CfgData;
import cdmw.qualification.perftest.testperfcommon.ParseCfg;

public class Client {

    private static final int SUCCESS = 0;

    private static final int FAILURE = 1;

    private static final int POA_THREAD_POOL_SIZE = 5;

    private static final String CFG_CLIENT_NAME = "--configFile";
    
    private static final String SERVER_HOSTNAME = "--serverHost";

    public static void main(String[] args) {
        int status = SUCCESS;

        System.err.println("-------------------------"
            + System.getProperty("line.separator")
            + "      Start Client       "
            + System.getProperty("line.separator")
            + "-------------------------");

        for (int n = 0; n < args.length; n++ )
            System.err.println("argv[" + n + "]: " + args[n]);

        System.err.println("-------------------------");

        String serverHostName = cdmw.ossupport.OS.getOptionValue(args,
            SERVER_HOSTNAME);

        if (serverHostName == "no") {
            System.err.println("Error: Option" + SERVER_HOSTNAME + " is mandatory!\n");
            System.exit(FAILURE);
        }
            
        String configFile = cdmw.ossupport.OS.getOptionValue(args,
            CFG_CLIENT_NAME);

        if (configFile == "no") {
            System.err.println("Error: Option "+ CFG_CLIENT_NAME +" is mandatory!\n");
            System.exit(FAILURE);
        }

        System.out.println("Parsing Configuration File...");

        // client configuration container
        CfgData clientcfg = new CfgData();

        // read configuration file

        if (ParseCfg.parseCfg(configFile, clientcfg) == FAILURE) {
            System.err.println("Error: when reading " + configFile);
            System.exit(FAILURE);
        }

        ORB orb = null; // orb reference

        try {
            // ===================================================
            // Initialize the ORB
            // ===================================================
            cdmw.orbsupport.StrategyList orb_strategies = new cdmw.orbsupport.StrategyList();
            orb_strategies.addORBThreaded();
            orb_strategies.addPOAThreadPool(POA_THREAD_POOL_SIZE);

            orb = cdmw.orbsupport.ORBSupport.ORBInit(args, orb_strategies);

            // ===================================================
            // Get the root POA
            // ===================================================
            org.omg.CORBA.Object obj = orb
                .resolve_initial_references("RootPOA");
            POA rootPOA = POAHelper.narrow(obj);

            // ===================================================
            // Activate the root POA manager
            // ===================================================
            POAManager poaManager = rootPOA.the_POAManager();
            poaManager.activate();

            // ===================================================
            // create the process control for platform management
            // ===================================================
            cdmw.cdmwinit.ProcessControl process_ctrl = new ClientProcessControl(
                orb, clientcfg, serverHostName);

            // ===================================================
            // Call generated CdmwInit
            // ===================================================
            cdmw.cdmwinit.CdmwInterface.cdmwInit(orb, args, process_ctrl, "cdmw.generated.CCMInvocationClientCdmwInterface");

            // ===================================================
            // wait for CORBA message
            // ===================================================
            orb.run();

        }
        catch (cdmw.common.Exception ex0) {
            System.err.println(ex0.what());
            status = FAILURE;
        }
        catch (SystemException ex1) {
            ex1.printStackTrace();
            status = FAILURE;
        }
        catch (UserException ex2) {
            ex2.printStackTrace();
            status = FAILURE;
        }
        catch (Exception ex3) {
            System.err.println("unexpected exception");
            status = FAILURE;
        }

        // ========================================================
        // program stopping
        // ========================================================

        // ===================================================
        // Call generated Cdmw cleanup
        // ===================================================
        cdmw.cdmwinit.CdmwInterface.cdmwCleanup(orb, "");

        // ===================================================
        // Call ORB cleanup
        // ===================================================
        cdmw.orbsupport.ORBSupport.ORBCleanup(orb);

        // ===================================================
        // destroy orb
        // ===================================================

        if (orb != null) {
            try {
                orb.destroy();
            }
            catch (SystemException ex) {
                System.err.println(ex);
                status = FAILURE;
            }
        }

        System.exit(status);
    }

}
