/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.qualification.perftest.testccminvocation;

import java.io.IOException;

import org.omg.CORBA.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import org.omg.CosNaming.NamingContextPackage.InvalidName;

import com.thalesgroup.CdmwNamingAndRepository.*;
import com.thalesgroup.CdmwNamingAndRepository.Repository;
import com.thalesgroup.CdmwPlatformMngt.ProcessDelegatePackage.NotReadyToRun;
import com.thalesgroup.CdmwPlatformMngtBase.StartupKind;

import cdmw.cdmwinit.ProcessControl;
import cdmw.common.BadOrderException;
import cdmw.common.BadParameterException;
import cdmw.orbsupport.CORBASystemExceptionWrapper;
import cdmw.qualification.perftest.testperfcommon.CfgData;
import cdmw.qualification.perftest.testperfcommon.ClockTime;
import cdmw.qualification.perftest.testperfcommon.ClockCPUTime;
import cdmw.qualification.perftest.testperfcommon.ClockHighResTime;
import cdmw.qualification.perftest.testperfcommon.ClockCPUTick;

public class ClientProcessControl extends ProcessControl {

    public static final String CALL_RETURN = "CR";

    public static final String ONE_WAY = "OW";

    public static final String ONE_WAY_TRANSPORT = "OW_TRANSPORT";

    public static final String ONE_WAY_SERVER = "OW_SERVER";

    public static final String ONE_WAY_TARGET = "OW_TARGET";

    public static final String TYPE_LONG = "LONG";

    public static final String TYPE_FLOAT = "FLOAT";

    public static final String TYPE_ANY = "ANY";

    public static final String TYPE_RECORD = "RECORD";

    public static final String TYPE_NONE = "NONE"; // for invocation CCM

    // benchmark

    public static final String MEASURE_HR = "TIME_HR"; // cTools HighRes Time

    public static final String MEASURE_TH = "TIME_TH"; // clock thread cpu time

    public static final String MEASURE_TK = "TIME_TK"; // cTools HighRes clock

    // tick count

    private static short sync_scope;

    /**
     * The orb reference.
     */
    private ORB orb;

    /**
     * The application name.
     */
    private String applicationName;

    /**
     * The process name.
     */
    private String processName;

    /**
     * TestPing reference
     */
    private com.acme.Invocation.TestPing testping;

    private int niter;

    private int delay;

    private String invocation_type;

    private String fileprefix;

    private String data_type;

    private int data_size;

    private ClockTime measure;

    private CfgData cfg;
    
    private String serverHostName;

    public ClientProcessControl(
        ORB orb, CfgData cfg, String hostName) {
        this.orb = orb;
        this.applicationName = "";
        this.processName = "";
        this.cfg = cfg;
        this.serverHostName = hostName;
        
        // ===================================================
        // Instantiate time reading object
        // ===================================================
        String measureStr = cfg.measure_type[0];
        
        if (measureStr.equals(MEASURE_TH)) {
            this.measure = new ClockCPUTime();
        }
        else if (measureStr.equals(MEASURE_TK)) {
            this.measure = new ClockCPUTick();
        } 
        else {  // using default 
            this.measure = new ClockHighResTime();
        }
        
    }

    // process initialisation called by platformmngt
    public void onInitialise(StartupKind startupKind) {
        try {
            // ===================================================
            // Get the process name and application name
            // ===================================================
            this.processName = cdmw.cdmwinit.InitUtils.getCdmwProcessName();

            this.applicationName = cdmw.cdmwinit.InitUtils
                .getCdmwApplicationName();

            // =================================================================
            // example of using the PlatformInterface for notifying a message
            // =================================================================
            cdmw.platformmngt.PlatformInterface.notifyMessage(
                com.thalesgroup.CdmwPlatformMngtBase.INF.value,
                this.processName,
                ">>>>>>>>>>>>>> Initialisation requested by supervision");

            // =========================================
            // retrieve Tester object from Repository
            // =========================================

            // retrieve "TestPing" registered via CcmPerf.cad
            // in the naming interface (which is not domain naming..)
            System.out.println("CLIENT ### GET  DEFAULT_ROOT_CONTEXT ###");
            Repository repository = cdmw.namingandrepository.RepositoryInterface
                .getRepository();

            // get the default root context
            NamingContext def_root_context = repository
                .resolve_root_context(com.thalesgroup.CdmwNamingAndRepository.DEFAULT_ROOT_CONTEXT.value);

            // create a naming interface on default root context
            cdmw.commonsvcs.naming.NamingInterface invocationNamingInterface = new cdmw.commonsvcs.naming.NamingInterface(
                def_root_context);

            // retrieve "ServerInst" component from Repository
            try {
                String admin_ctx_cobaloc = 
                    "corbaname::localhost:22456#CDMW/ADMIN/GlobalRepository";
                    
                System.out.println("Try to get AdminRoot context");
                org.omg.CORBA.Object obj = this.orb.string_to_object(admin_ctx_cobaloc);
                
                System.out.println("Try to narrow as NameContext");
                org.omg.CosNaming.NamingContext ctx 
                    = org.omg.CosNaming.NamingContextHelper.narrow(obj);
                if (ctx == null) {
                    System.err.println("ERROR: Retrieved object is not a NamingContext");
                    return;
                }
                
                cdmw.commonsvcs.naming.NamingInterface ni =
                    new cdmw.commonsvcs.naming.NamingInterface(ctx);
                
                System.out.println("Try to get Server component from AdminRoot context");
                String name = serverHostName + "/CcmPerfApplication/ServerProcess/CCM/Components/ServerInst";
                
                obj = ni.resolve(name);
                
                System.out.println("Try to narrow as CCMObject");
                org.omg.Components.CCMObject comp = 
                    org.omg.Components.CCMObjectHelper.narrow(obj);
                if (comp == null) {
                    System.err.println("ERROR: Retrieved object is not a CCMObject");
                    return;
                }
                
                System.out.println("Try to get server_facet");
                obj = comp.provide_facet("server_facet");
                System.out.println("Try to narrow as Invocation::TestPing");
                this.testping = com.acme.Invocation.TestPingHelper.narrow(obj);
                if (this.testping == null) {
                    System.err.println("ERROR: Retrieved object is not a Invocation::TestPing");
                    return;
                }
                
                System.out.println("Succeed to get server_facet !");
                
            } catch (Exception e) {
                System.err.println("ERROR: ");
                e.printStackTrace();
            }
            
            
        }
        catch (UserException e) {
            e.printStackTrace();
        }
        catch (BadOrderException e) {
            e.printStackTrace();
        }
        catch (BadParameterException e) {
            e.printStackTrace();
        }
        catch (CORBASystemExceptionWrapper e) {
            e.printStackTrace();
        }
    }

    public void onRun() throws NotReadyToRun {
        try {
            // =================================================================
            // example of using the PlatformInterface for notifying a message
            // =================================================================
            cdmw.platformmngt.PlatformInterface
                .notifyMessage(com.thalesgroup.CdmwPlatformMngtBase.INF.value,
                    this.processName,
                    ">>>>>>>>>>>>>> Run requested by supervision");

            System.out.println("Start testing");

            this.niter = this.cfg.niter[0];
            this.fileprefix = this.cfg.fileprefix[0];

            double[] tdelta = new double[this.niter];

            for (int i_del = 0; i_del < this.cfg.delay.length; i_del++ ) {
                // assign current delay
                this.delay = this.cfg.delay[i_del];
                // delay of measure

                int tv_msec = this.delay;
                int tv_nsec = 0;           // in c++, this code is stupid: (this.delay % 1000) * 1000000;

                for (int i_inv = 0; i_inv < this.cfg.inv_type.length; i_inv++ ) { // for
                    // each
                    // invocation
                    // type
                    // (CR,
                    // OW)
                    // result of time measurement
                    double time1, time2;

                    this.invocation_type = this.cfg.inv_type[i_inv];

                    if (this.invocation_type.compareTo(CALL_RETURN) == 0) { // test
                        // CR:
                        // call
                        // and
                        // return
                        System.out.println("Test Call&Return calls");

                        // start of Invocation test
                        for (int i = 0; i < this.niter; i++ ) {
                            // delay of measure
                            if (this.delay != 0) {
                                // call to nanosleep
                                Thread.sleep(tv_msec, tv_nsec);
                            }

                            time1 = this.measure.getTime();
                            this.testping.invocation_ping();
                            time2 = this.measure.getTime();
                            tdelta[i] = time2 - time1;

                        }
                    }
                    else { // ONE_WAY
                        System.out.println("Test OneWay calls");
                        org.omg.CORBA.Object obj = this.orb
                            .resolve_initial_references("ORBPolicyManager");

                        org.omg.CORBA.PolicyManager policy_manager = org.omg.CORBA.PolicyManagerHelper
                            .narrow(obj);

                        // obj = this.orb
                        //     .resolve_initial_references("PolicyCurrent");
                        // org.omg.CORBA.PolicyCurrent policy_current = org.omg.CORBA.PolicyCurrentHelper
                        //     .narrow(obj);

                        // Default SyncScope value (ONE_WAY).
                        sync_scope = org.omg.Messaging.SYNC_NONE.value;

                        // Set up the sync scope any.
                        org.omg.CORBA.Any sync_scope_any = this.orb
                            .create_any();
                        org.omg.Messaging.SyncScopeHelper.insert(
                            sync_scope_any, sync_scope);

                        // Set the length of the policy list.
                        org.omg.CORBA.Policy[] sync_scope_policy_list = new org.omg.CORBA.Policy[1];

                        // Set up the sync scope policy.
                        sync_scope_policy_list[0] = this.orb.create_policy(
                            org.omg.Messaging.SYNC_SCOPE_POLICY_TYPE.value,
                            sync_scope_any);

                        // Set the sync scope policy at the object level.
                        obj = this.testping._set_policy_override(
                            sync_scope_policy_list,
                            org.omg.CORBA.SetOverrideType.ADD_OVERRIDE);

                        // Get the new object reference with the updated policy.
                        this.testping = com.acme.Invocation.TestPingHelper
                            .narrow(obj);

                        // start of Invocation test
                        for (int i = 0; i < this.niter; i++ ) {
                            // delay of measure
                            if (this.delay != 0) {
                                // call to nanosleep
                                Thread.sleep(tv_msec, tv_nsec);
                            }

                            time1 = this.measure.getTime();
                            this.testping.ow_invocation_ping();
                            time2 = this.measure.getTime();
                            // time read in micro sec.
                            tdelta[i] = time2 - time1;
                        }
                    }

                    // write file
                    String nameStream = this.cfg.inv_type[i_inv] + "_"
                        + this.cfg.delay[i_del] + "_" + "DELTA_C.dat";

                    String fileDeltaName = new String(this.fileprefix
                        + nameStream);

                    java.io.FileWriter fd = new java.io.FileWriter(
                        fileDeltaName);

                    for (int i = 0; i < niter; i++ )
                        fd.write(String.format("%1.6f\n", tdelta[i]));
                    fd.close();
                } // end for i_inv
            } // end for i_del

            System.err
                .println("** test execution successfull terminated : please run the data anaylsis script");
        }
        catch (UserException e) {
            e.printStackTrace();
        }
        catch (BadOrderException e) {
            e.printStackTrace();
        }
        catch (BadParameterException e) {
            e.printStackTrace();
        }
        catch (InterruptedException e) {
            e.printStackTrace();
        }
        catch (IOException e) {
            e.printStackTrace();
        }
        catch (Throwable e) {
            e.printStackTrace();
        }
    }

    public void onStop() {
        // =================================================================
        // example of using the PlatformInterface for notifying a message
        // =================================================================
        try {
            cdmw.platformmngt.PlatformInterface.notifyMessage(
                com.thalesgroup.CdmwPlatformMngtBase.INF.value,
                this.processName,
                ">>>>>>>>>>>>>> Stop requested by supervision");
        }
        catch (BadOrderException e) {
            e.printStackTrace();
        }
        catch (BadParameterException e) {
            e.printStackTrace();
        }
        this.orb.shutdown(true);
    }
}
