/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.qualification.perftest.testccminvocation;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;

public class MainComponentServer {

    private static final int SUCCESS = 0;

    private static final int FAILURE = 1;

    private static final int POA_THREAD_POOL_SIZE = 5;

    public static void main(String[] args) {
        int status = SUCCESS;

        System.err.println("-------------------------"
            + System.getProperty("line.separator")
            + "    Component Server     "
            + System.getProperty("line.separator")
            + "-------------------------");

        for (int n = 0; n < args.length; n++ )
            System.err.println("argv[" + n + "]: " + args[n]);

        System.err.println("-------------------------");

        ORB orb = null;

        try {
            // ===================================================
            // Initialize the ORB
            // ===================================================
            cdmw.orbsupport.StrategyList orb_strategies = new cdmw.orbsupport.StrategyList();
            orb_strategies.addORBThreaded();
            orb_strategies.addPOAThreadPool(POA_THREAD_POOL_SIZE);

            orb = cdmw.orbsupport.ORBSupport.ORBInit(args, orb_strategies);

            // ===================================================
            // create the process control for platform management
            // ===================================================
            cdmw.cdmwinit.ProcessControl process_ctrl = new ServerProcessControl(
                orb);

            // ===================================================
            // Call generated cdmwinit
            // ===================================================
            cdmw.cdmwinit.CdmwInterface.cdmwInit(orb, args, process_ctrl, 
                "cdmw.generated.CCMInvocationServerCdmwInterface");

            // ===================================================
            // Get the root POA
            // ===================================================
            org.omg.CORBA.Object obj = orb
                .resolve_initial_references("RootPOA");
            POA rootPOA = POAHelper.narrow(obj);

            // ===================================================
            // Activate the root POA manager
            // ===================================================
            POAManager poaManager = rootPOA.the_POAManager();
            poaManager.activate();

            // ===================================================
            // Get the process name and application name
            // ===================================================
            String processName = cdmw.cdmwinit.InitUtils.getCdmwProcessName();

            String applicationName = cdmw.cdmwinit.InitUtils
                .getCdmwApplicationName();

            System.out.println("===> main component server of "
                + applicationName + "/" + processName);

            // ===================================================
            // wait for CORBA message
            // ===================================================
            orb.run();
        }
        catch (cdmw.common.Exception ex) {
            System.err.println(ex.what());
            status = FAILURE;
        }
        catch (SystemException ex) {
            System.err.println(ex);
            status = FAILURE;
        }
        catch (UserException ex) {
            System.err.println(ex);
            status = FAILURE;
        }

        // ========================================================
        // program stopping
        // ========================================================

        // ===================================================
        // Call generated Cardamom cleanup
        // ===================================================
        cdmw.cdmwinit.CdmwInterface.cdmwCleanup(orb, 
            "cdmw.generated.CCMInvocationServerCdmwInterface");

        // ===================================================
        // Call ORB cleanup
        // ===================================================
        cdmw.orbsupport.ORBSupport.ORBCleanup(orb);

        // =====================================================
        // Destroy orb
        // =====================================================
        if (orb != null) {
            try {
                orb.destroy();
            }
            catch (SystemException ex) {
                System.err.println(ex);
                status = FAILURE;
            }
        }

        System.exit(status);
    }

}
