/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.qualification.perftest.testccminvocation;

import org.omg.CORBA.*;

import com.thalesgroup.CdmwPlatformMngt.ProcessDelegatePackage.NotReadyToRun;
import com.thalesgroup.CdmwPlatformMngtBase.INF;
import com.thalesgroup.CdmwPlatformMngtBase.StartupKind;

import cdmw.cdmwinit.ProcessControl;

public class ServerProcessControl extends ProcessControl {

    /**
     * The orb reference.
     */
    private ORB orb;

    /**
     * The application name.
     */
    private String applicationName;

    /**
     * The process name.
     */
    private String processName;

    public ServerProcessControl(
        ORB orb) {
        this.orb = orb;
        this.applicationName = "";
        this.processName = "";
    }

    public void onInitialise(StartupKind startupKind) {
        try {
            // ===================================================
            // Get the process name and application name
            // ===================================================
            this.processName = cdmw.cdmwinit.InitUtils.getCdmwProcessName();

            this.applicationName = cdmw.cdmwinit.InitUtils
                .getCdmwApplicationName();

            // =================================================================
            // example of using the PlatformInterface for notifying a message
            // =================================================================
            cdmw.platformmngt.PlatformInterface.notifyMessage(INF.value,
                this.processName,
                ">>>>>>>>>>>>>> Initialisation requested by supervision");
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void onRun() throws NotReadyToRun {
        try {
            // =================================================================
            // example of using the PlatformInterface for notifying a message
            // =================================================================
            cdmw.platformmngt.PlatformInterface
                .notifyMessage(INF.value, this.processName,
                    ">>>>>>>>>>>>>> Run requested by supervision");

            // do nothing
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void onStop() {
        try {
            // =================================================================
            // example of using the PlatformInterface for notifying a message
            // =================================================================
            cdmw.platformmngt.PlatformInterface.notifyMessage(INF.value,
                this.processName,
                ">>>>>>>>>>>>>> Stop requested by supervision");
            this.orb.shutdown(true);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }
}
