
/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef ___CCM_TEST_CCM_NOTIFICATION_CCM_CLIENT_IMPL_HPP__
#define ___CCM_TEST_CCM_NOTIFICATION_CCM_CLIENT_IMPL_HPP__

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/osthreads/Mutex.hpp>
#include <Foundation/osthreads/Condition.hpp>
#include <queue>

#include <test_ccm_notification/TestComponentNotificationLocal.skel.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtManagedElementContexts.stub.hpp>

namespace TestCcmNotification
{

class CCM_Client_impl;
    

class test_ccm_notification : public CppUnit::TestFixture
{
public:
    CCM_Client_impl * m_ccm_client;
    ::TestCcmNotification::CCM_Client_Context_var m_session_context;
    TestCcmNotification::Client_var m_component_ref;
    CORBA::ORB_var orb;
    
private:
    Cdmw::OsSupport::Mutex  *             m_test_mutex;
    Cdmw::OsSupport::Condition *          m_test_condition;


public:
    test_ccm_notification();
    void register_observer(CdmwPlatformMngt::External::System_var );

    //
    //
    void times_gathering();
    void first_req_test();
    void second_req_test();

    
    // register test suite and test operation
    CPPUNIT_TEST_SUITE( test_ccm_notification );
    CPPUNIT_TEST( times_gathering );
    CPPUNIT_TEST( first_req_test );
    CPPUNIT_TEST( second_req_test );
    CPPUNIT_TEST_SUITE_END();

};
    
    
    
//
// IDL:thalesgroup.com/TestComponentNotification/Session_CCM_Client:1.0
//
class CCM_Client_impl : virtual public ::TestCcmNotification::Session_CCM_Client,
                        virtual public Cdmw::OrbSupport::RefCountLocalObject
{
    CCM_Client_impl(const CCM_Client_impl&);
    void operator=(const CCM_Client_impl&);


    ::TestCcmNotification::CCM_Client_Context_var m_session_context;
     
public:

    CCM_Client_impl();
    ~CCM_Client_impl();
     

    //
    // Session component specific operations
    //
    void set_session_context(Components::SessionContext_ptr ctx)
        throw(CORBA::SystemException);
    void ccm_activate()
        throw(CORBA::SystemException);
    void ccm_passivate()
        throw(CORBA::SystemException);
    void ccm_remove()
        throw(CORBA::SystemException);

    void CCM_Client_impl::notify(CdmwPlatformMngt::Event* event)
       throw(CORBA::SystemException);
    
    bool timestamps_analisys(const float percentage, const unsigned long delay);


private:
    
    
    ::TestCcmNotification::CCM_Client_Context* get_context();
    
    Cdmw::TestUtils::ThreadedTestRunner* m_test_thread;
    
    Cdmw::OsSupport::Mutex  *             m_mutex;
    Cdmw::OsSupport::Condition *          m_condition;

    std::vector <unsigned long> elapsedTimes;
    Cdmw::OsSupport::OS::Timeval               lastKillTime;
    
    friend class test_ccm_notification;
};

}; // End of namespace TestCcmNotification


#endif // ___CCM_TEST_CCM_NOTIFICATION_CCM_CLIENT_IMPL_HPP__

