/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <test_ccm_notification/CCM_Client_impl.hpp>
#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/logging/CdmwLogging.stub.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include "SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/EntityStatusChange_impl.hpp"
#include <SystemMngt/idllib/CdmwPlatformMngtManagedElementContexts.stub.hpp>




namespace TestCcmNotification
{

//
// IDL:thalesgroup.com/CcmFtTest/ClientModule/CCM_Client:1.0
//
CCM_Client_impl::CCM_Client_impl()
    : m_test_thread(0)
{
    //Creating a mutex to sinchronize the test's "kill and restart" procedure. 
    m_mutex=new Cdmw::OsSupport::Mutex();
    m_condition=new Cdmw::OsSupport::Condition(*m_mutex);
}

CCM_Client_impl::~CCM_Client_impl()
{
    std::cerr << "   ... CCM_Client_impl::~CCM_Client_impl called!" << std::endl;
    delete m_mutex;
    delete m_condition;
 
}


void CCM_Client_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    m_session_context = ::TestCcmNotification::CCM_Client_Context::_narrow(ctx);
}
        
::TestCcmNotification::CCM_Client_Context* CCM_Client_impl::get_context()
{
    return ::TestCcmNotification::CCM_Client_Context::_duplicate(m_session_context.in());
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CCM_Client_impl::ccm_activate()
   throw(CORBA::SystemException)
{
    std::cerr << "   ... CCM_Client_impl::ccm_activate called!" << std::endl;

    // Get this component CORBA object
    // Note: this has to be performed in a CORBA request thread; thus
    //       not in test which is a specific thread.
    TestCcmNotification::Client_var this_component =
        TestCcmNotification::Client::_nil();
     
    try 
    {
        // narrow Context in SessionContext  for get_CCM_objet() access
        Components::SessionContext_var ctx =
            Components::SessionContext::_narrow(m_session_context.in());
        
        // call SessionContext::get_CCM_objet()
        CORBA::Object_var obj =
            ctx->get_CCM_object();
            
        this_component =
            TestCcmNotification::Client::_narrow(obj.in());
            
        CDMW_ASSERT(!CORBA::is_nil(this_component.in()));
            
    } catch (const CORBA::UserException& e) {
        std::cerr << "UserException during get_CCM_Object() :" 
                  << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
                  << std::endl;
        CDMW_ASSERT(false);
    } catch (const CORBA::SystemException& e) {
        std::cerr << "SystemException during get_CCM_Object() :" 
                  << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
                  << std::endl;
        CDMW_ASSERT(false);
    }


    try {   
        // save Context in CORBATestManager for usage in test
        Cdmw::TestUtils::CORBATestManager::instance()->add(
                "client_context", 
                get_context());
        // save this component in CORBATestManager for usage in test
        Cdmw::TestUtils::CORBATestManager::instance()->add(
                "this_component", 
                this_component.in());


        // save this executor in CORBATestManager for usage in test
        Cdmw::TestUtils::CORBATestManager::instance()->add(
                "this_executor", 
                dynamic_cast< Session_CCM_Client_ptr > (this));
        
        // create test runner thread
        m_test_thread = new Cdmw::TestUtils::ThreadedTestRunner(
                Cdmw::TestUtils::CORBATestManager::instance()->get_controller());
        // add test to be run
        m_test_thread->addTest( 
                CppUnit::TestFactoryRegistry::getRegistry("test_ccm_notification").makeTest());
        // start test
        m_test_thread->start();


   } catch (const CORBA::Exception& e) {
        std::cerr << "CORBA Exception exception during test: "
                  << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
                  << std::endl;
    } catch (const std::exception& e) {
        std::cerr << "std::exception exception during test: "
                  << e.what()
                  << std::endl;
    } catch (...) {
        std::cerr << "unknown exception during test !!" << std::endl;
    }
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
CCM_Client_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
    std::cerr << "   ... CCM_Client_impl::ccm_passivate called!" << std::endl;
    
    if (m_test_thread != 0) {
        // wait test to be finished
        m_test_thread->join();
        delete m_test_thread;
        m_test_thread = 0;
        // print test summary
        Cdmw::TestUtils::CORBATestManager::instance()->summary();
    }

} 


//
// ccm_remove function : it destroys the component (call by the container)
//
void
CCM_Client_impl::ccm_remove()
   throw(CORBA::SystemException)
{
    std::cerr << "   ... CCM_Client_impl::ccm_remove called!" << std::endl;
}

//
//
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(test_ccm_notification, "test_ccm_notification");

test_ccm_notification::test_ccm_notification()
{
    orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::instance ()->get_ORB ());
    
    // retrieve notification object
    CORBA::Object_var obj = 
        Cdmw::TestUtils::CORBATestManager::instance()->get_object("client_context");        
    m_session_context = ::TestCcmNotification::CCM_Client_Context::_narrow(obj.in());
    
    obj = Cdmw::TestUtils::CORBATestManager::instance()->get_object("this_executor");       
    m_ccm_client = dynamic_cast< CCM_Client_impl* > (obj.in());
    
    obj = Cdmw::TestUtils::CORBATestManager::instance()->get_object("this_component");
    m_component_ref = TestCcmNotification::Client::_narrow(obj.in());

}

void test_ccm_notification::register_observer(CdmwPlatformMngt::External::System_var system)
{
    // register this component as System observer
        TEST_INFO("   register this component as System Observer");
                
        // Register event factories
        CORBA::ValueFactoryBase_var factory, old_factory;

        factory = new Cdmw::PlatformMngt::ProcessMessageFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessMessage:1.0",
                factory.in() );

        factory = new Cdmw::PlatformMngt::ProcessStatusChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessStatusChange:1.0",
                factory.in() );

        //PCR-1257 commented code to be deleted
        /*factory = new ProcessEntityStatusChangeFactory;
         old_factory = orb->register_value_factory(
                           "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessEntityStatusChange:1.0",
                           factory.in() );*/

        factory = new Cdmw::PlatformMngt::EntityStatusChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/EntityStatusChange:1.0",
                factory.in() );
             
        CdmwPlatformMngt::SupervisionObserver_var ob =
            system->register_observer("TestCcmNotification_Client",m_component_ref.in());
                
}

 void test_ccm_notification::first_req_test()
 {   
     TEST_INFO("   Executing first requirement test PERCENTAGE: 99 DELAY: 750 ms");
     if (m_ccm_client->timestamps_analisys(0.99,750))
     {
         TEST_INFO("   TEST CASE SUCCESSFULL");
         CPPUNIT_ASSERT(true);
     }
     else 
     {
         TEST_INFO("   TEST CASE FAILED");
         CPPUNIT_ASSERT(false);
     }
 }
 
 void test_ccm_notification::second_req_test()
 {
     TEST_INFO("   Executing second requirement test PERCENTAGE: 95 DELAY: 600 ms");
     if ((m_ccm_client->timestamps_analisys(0.95,600)))
     {
         TEST_INFO("   TEST CASE SUCCESSFULL");
         CPPUNIT_ASSERT(true);
     }
     else 
     {
         TEST_INFO("   TEST CASE FAILED");
         CPPUNIT_ASSERT(false);
     }
 }

void test_ccm_notification::times_gathering()
{
    std::cerr << "   ... test_ccm_notification::do_tests called!" << std::endl;
    int nb=100;
    try
    {
        // get ManagedComponentContext
        CORBA::Object_var obj_ctx = 
            m_session_context->resolve_service_reference("ManagedComponentContext");
        CdmwPlatformMngt::ManagedElementContexts::ComponentContext_var component_context =
            CdmwPlatformMngt::ManagedElementContexts::ComponentContext::_narrow(obj_ctx.in());
        
        CPPUNIT_ASSERT(!CORBA::is_nil(component_context.in()));

        // get System supervision object
        TEST_INFO("   Get System supervision object");
        CdmwPlatformMngt::External::System_var system = component_context->the_system();

        register_observer(system);
        
        TestComponentNotification_var test_ccm_notification =
            m_session_context->get_connection_test_server();
            
        TEST_INFO("   Starting component killing...");
        sleep(2);
                
        int counter = 0;
        Cdmw::OsSupport::OS::Timeval time_before ;
        Cdmw::OsSupport::OS::Timeval time_after;
        
        while (counter < nb )
        {
            try
            {   
                time_before=Cdmw::OsSupport::OS::get_time();
                std::cerr<< "---------------------------------------------"<<std::endl;
                std::cerr<<" \tKILLING SERVER COMPONENT "<<"\n\t"<< (nb-counter-1)<<" remaining  measures"<<std::endl;
                std::cerr<< "---------------------------------------------"<<std::endl;
                test_ccm_notification->kill_you("TestCcmNotification_Client");
            }
            catch (const CORBA::COMM_FAILURE & e)
            {      
                TEST_INFO("   Expected COMM_FAILURE EXCEPTION after component failure...");
                CPPUNIT_ASSERT(true);
                std::cout << e<< std::endl ; 
            }
            {
                CDMW_MUTEX_GUARD(*(m_ccm_client->m_mutex));
                //  Considering the half calling time for function kill_you...
                time_after = Cdmw::OsSupport::OS::get_time();
                unsigned long diff_sec  = ( abs(time_after.seconds - time_before.seconds) );
                unsigned long diff_usec = ( abs(time_after.microseconds - time_before.microseconds) );
                m_ccm_client->lastKillTime.seconds = abs(time_after.seconds - diff_sec/2);
                m_ccm_client->lastKillTime.microseconds = abs((time_after.microseconds - diff_usec/2));
                TEST_INFO("   Waiting for the Server Component Restart");
                m_ccm_client->m_condition->wait();
            }
            sleep(2);
            counter++;
        }
    }catch (const CORBA::Exception & ex)
    {
        std::cerr<<ex;
        CPPUNIT_ASSERT(false);
    }
    
}

bool CCM_Client_impl::timestamps_analisys(const float percentage, const unsigned long delay)
{
    bool ret=false;
    float success=0;
    float size=elapsedTimes.size();
    
    if( size!=0 )
    {
        for( unsigned long i=0 ; i< size; i++ )
        {
            if(elapsedTimes[i] < (delay*1000)) success ++;
        }
        if ( (success/size) >= percentage ) ret = true;
    }
    return ret;
}
    
void CCM_Client_impl::notify(CdmwPlatformMngt::Event* an_event)
        throw(CORBA::SystemException)
{
    try
    {
        CdmwPlatformMngt::ProcessStatusChange* prc_chg = dynamic_cast<CdmwPlatformMngt::ProcessStatusChange*> (an_event);
        if (((prc_chg!=0))&&(prc_chg->process_status()==CdmwPlatformMngt::PROCESS_FAILED_DEATH))
        {
            CDMW_MUTEX_GUARD(*m_mutex);
            Cdmw::OsSupport::OS::Timeval current= Cdmw::OsSupport::OS::get_time();
            TEST_INFO("         notified PROCESS_FAILED_DEATH");
                        
            unsigned long diff = abs((1000000*(current.seconds - lastKillTime.seconds))) 
                               + abs((current.microseconds - lastKillTime.microseconds));
            //Saving the elapsed Delay.
            elapsedTimes.push_back( diff );
            std::cout<<"\t\t\t\t\t **** Elapsed Delay : "<< diff/1000 << "(ms) ****"<<std::endl;
            lastKillTime.seconds = 0;
            lastKillTime.microseconds = 0;
        }
        if (((prc_chg!=0))&&(prc_chg->process_status()==CdmwPlatformMngt::PROCESS_RUNNING))
        {
            TEST_INFO("         notified PROCESS_RUNNING");
            m_condition->broadcast();  
        }
    }
    catch (const std::exception& ex)
    {
        std::cerr << " TestCcmNotification_Client observer exception  " << ex.what();
    }
}

};  // namespace TestCcmNotification


