/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
#include <iostream>

// get size of arrays in definition.idl
// (definitionC.h is generated from definition.idl)
#include "test_lb_invocation/definition.stub.hpp"
#include "test_lb_invocation/fill_struct.hpp"


// for each OPT... we select if it's filled or not
// true => filled
// false => empty
#define OPTCOVARIANCE true
#define OPTCSTRIDLIST true
#define OPTWAYPOINTANDID true
#define OPTAIRPORTNAME true
#define OPTSTAR true
#define OPTSID true
#define OPTSPEED false
#define SPEED AvBact::NO_SPEEDKIND
#define OPTFLS true
#define WAYPOINT AvBamt::NAVIGATION_POINT
#define BASICWAYPOINT AvBamt::NAVIGATION_POINT
#define OPTDATEDAY true
#define OPTTIME true
#define OPTFPINITIALCOND true
#define SPEEDCASMACH AvBact::SPEED_CAS
#define BASICROUTEITEM AvFlplt::AIRWAY

void fill_Time(AvTim::Time& in);
void fill_OptTime(AvTim::OptTime& in);
void fill_BasicWaypoint(AvBamt::BasicWaypoint& in);
void fill_Waypoint(AvBamt::Waypoint& in);
void fill_OptFPInitialCond(AvFlplt::OptFPInitialCond& in);
void fill_OptDateDay(AvTim::OptDateDay& in);
void fill_OptFls(AvBsut::OptFls& in);
void fill_Speed(AvBact::Speed& in);
void fill_OptSpeed(AvBact::OptSpeed& in);
void fill_OptStar(AvFlplt::OptStar& in);
void fill_OptSid(AvFlplt::OptSid& in);
void fill_OptAirportName(AvBamt::OptAirportName& in);
void fill_OptCovariance(AvTrajt::OptCovariance& in);
void fill_OptCstrIdList(AvCflgt::OptCstrIdList& in);
void fill_OptWaypointAndId(AvTrajt::OptWaypointAndId& in);
void fill_SpeedCasMach(AvBact::SpeedCasMach& in);
void fill_BasicRouteItem(AvFlplt::BasicRouteItem& in);

void
fill_BasicRouteItem(AvFlplt::BasicRouteItem& in)
{
  //if (BASICROUTEITEM) {
    AvFlplt::AirwayItem myAirwayItem;
    for (uint i28 = 0 ; i28 < 7 ; i28++) {
      myAirwayItem.airway_name[i28]='a';
    }
    for (int i = 0 ; i < POINT_INFO_LIST ; i++) {
      fill_Waypoint(myAirwayItem.points[i].waypoint);
      fill_OptFls(myAirwayItem.points[i].flight_level);
      fill_OptSpeed(myAirwayItem.points[i].speed);
      fill_OptTime(myAirwayItem.points[i].time);
      myAirwayItem.points[i].route_centre_change=AvFlplt::CENTRE_NO_CHANGE;
      myAirwayItem.points[i].route_rule_change=AvFlplt::RULE_NO_CHANGE;
      myAirwayItem.points[i].route_control_change=AvFlplt::CONTROL_NO_CHANGE;
    }
    in.airway_item = myAirwayItem;
    //}
    //else {
    AvFlplt::PointInfo myPointInfo;
    fill_Waypoint(myPointInfo.waypoint);
    fill_OptFls(myPointInfo.flight_level);
    fill_OptSpeed(myPointInfo.speed);
    fill_OptTime(myPointInfo.time);
    myPointInfo.route_centre_change=AvFlplt::CENTRE_NO_CHANGE;
    myPointInfo.route_rule_change=AvFlplt::RULE_NO_CHANGE;
    myPointInfo.route_control_change=AvFlplt::CONTROL_NO_CHANGE;
    in.pt_info = myPointInfo;
    //}
}

void
fill_SpeedCasMach(AvBact::SpeedCasMach& in)
{
  switch (SPEEDCASMACH) {
  case AvBact::SPEED_CAS: {
    AvBsut::Cas myCas = 4.0;
    in.cas_value(myCas);
  } break;
  case AvBact::SPEED_MACH: {
    AvBsut::MachNumber myMachNumber = 4.0;
    in.mach_value(myMachNumber);
  } break;
  }
}

void
fill_Time(AvTim::Time& in)
{
  in.sec = 4;
  in.usec = 4;
}

void
fill_OptTime(AvTim::OptTime& in)
{
  if (OPTTIME) {
    AvTim::Time myTime;
    fill_Time(myTime);
    in.value(myTime);
  }
  else {
    AvStd::Empty myField = 0;
    in.field(myField);
  }
}

void
fill_BasicWaypoint(AvBamt::BasicWaypoint& in)
{
  switch (BASICWAYPOINT) {
  case AvBamt::NAVIGATION_POINT : {
    AvBamt::NavigationPointName myNavigationPointName;
    for (int i = 0 ; i < 5 ; i++) {
      myNavigationPointName[i] = 'a';
    }
    in.waypoint_name(myNavigationPointName);
  } break;
  case AvBamt::GEOGRAPHICAL : {
    AvBsut::LatLong2DPosition myLatLong2DPosition;
    myLatLong2DPosition.latit.lat_hemis = AvBsut::NORTH;
    myLatLong2DPosition.latit.lat_azim = 4.0;
    myLatLong2DPosition.longit.long_dir = AvBsut::EAST;
    myLatLong2DPosition.longit.long_azim = 4.0;
    in.waypoint_pos(myLatLong2DPosition);
  } break;
  case AvBamt::RANGE_AND_BEARING : {
    AvStd::Empty myField = 0;
    in.empty_range_and_bearing(myField);
  } break;
  case AvBamt::NULL_WAYPOINT : {
    AvStd::Empty myField = 0;
    in.empty_null_waypoint(myField);
  } break;
  }
}

void
fill_Waypoint(AvBamt::Waypoint& in)
{
  switch (WAYPOINT) {
  case AvBamt::NAVIGATION_POINT : {
    AvBamt::NavigationPointName myNavigationPointName;
    for (int i = 0 ; i < 5 ; i++) {
      myNavigationPointName[i] = 'a';
    }
    in.waypoint_name(myNavigationPointName);
  } break;
  case AvBamt::GEOGRAPHICAL : {
    AvBsut::LatLong2DPosition myLatLong2DPosition;
    myLatLong2DPosition.latit.lat_hemis = AvBsut::NORTH;
    myLatLong2DPosition.latit.lat_azim = 4.0;
    myLatLong2DPosition.longit.long_dir = AvBsut::EAST;
    myLatLong2DPosition.longit.long_azim = 4.0;
    in.waypoint_pos(myLatLong2DPosition);
  } break;
  case AvBamt::RANGE_AND_BEARING : {
    AvBamt::WaypointRangeAndBearingCase myWaypointRangeAndBearingCase;
    fill_BasicWaypoint(myWaypointRangeAndBearingCase.waypoint_from);
    myWaypointRangeAndBearingCase.bearing = 4.0;
    myWaypointRangeAndBearingCase.distance = 4.0;
    in.range_and_bearing_desc(myWaypointRangeAndBearingCase);
  } break;
  case AvBamt::NULL_WAYPOINT : {
    AvStd::Empty myField = 0;
    in.field(myField);
  } break;
  }
}

void
fill_OptFPInitialCond(AvFlplt::OptFPInitialCond& in)
{
  if (OPTFPINITIALCOND) {
    AvFlplt::FPInitialCond myFPInitialCond;
    fill_OptTime(myFPInitialCond.time);
    fill_Waypoint(myFPInitialCond.position);
    AvBsut::Feet myFeet = 4;
    myFPInitialCond.level.feet_val(myFeet);
    AvBsut::Kilograms myKilograms = 4.0;
    myFPInitialCond.weight.kilog_val(myKilograms);
    AvBsut::Azimuths myAzimuths = 4.0;
    myFPInitialCond.heading.azim_val(myAzimuths);
    fill_OptSpeed(myFPInitialCond.speed);
    AvBsut::FeetPerMinute myFeetPerMinute = 4.0;
    myFPInitialCond.rocd.feet_mn_val(myFeetPerMinute);
    AvBsut::DegreesPerSecond myDegrresPerSecond = 4.0;
    myFPInitialCond.rot.deg_sec_val(myDegrresPerSecond);
    in.initial_cond(myFPInitialCond);
  }
  else {
    AvStd::Empty myField = 0;
    in.field(myField);
  }
}

void
fill_OptDateDay(AvTim::OptDateDay& in)
{
  if (OPTDATEDAY) {
    AvTim::DateDay myDateDay;
    myDateDay.year = 2;
    myDateDay.month = 2;
    myDateDay.day = 2;
    in.value(myDateDay);
  }
  else {
    AvStd::Empty myField = 0;
    in.field(myField);
  }
}

void
fill_OptFls(AvBsut::OptFls& in)
{
  if (OPTFLS) {
    // we want a Fls
    AvBsut::Fls myFls = 4;
    in.fls_val(myFls);
  }
  else {
    // we want nothing
    AvStd::Empty myField = 0;
    in.field(myField);
  }
}

void
fill_Speed(AvBact::Speed& in)
{
  switch (SPEED) {
  case AvBact::NO_SPEEDKIND: {
    AvStd::Empty myField = 0;
    in.field(myField);
  } break;
  case AvBact::TAS: {
    AvBsut::Knots myKnots = 4.0;
    in.tas_value(myKnots);
  } break;
  case AvBact::GS: {
    AvBsut::Knots myKnots = 4.0;
    in.gs_value(myKnots);
  } break;
  case AvBact::CAS: {
    AvBsut::Cas myCas = 4.0;
    in.cas_value(myCas);
  } break;
  case AvBact::IAS: {
    AvBsut::Cas myCas = 4.0;
    in.ias_value(myCas);
  } break;
  case AvBact::MACH: {
    AvBsut::MachNumber myMachNumber = 4.0;
    in.mach_value(myMachNumber);
  } break;
  }
}

void
fill_OptSpeed(AvBact::OptSpeed& in)
{
  if (OPTSPEED) {
    AvBact::Speed mySpeed;
    fill_Speed(mySpeed);
    in.value(mySpeed);
  }
  else {
    // we want nothing
    AvStd::Empty myField = 0;
    in.field(myField);
  }
}

void
fill_OptStar(AvFlplt::OptStar& in)
{
  // struct => all fields got to be filled
  //if (OPTSTAR) {
    AvFlplt::StarInfo myStarInfo;
    for (uint i28 = 0 ; i28 < 7 ; i28++) {
      myStarInfo.star_name[i28]='a';
    }
    for (int i = 0 ; i < POINT_INFO_LIST ; i++) {
      fill_Waypoint(myStarInfo.points[i].waypoint);

      fill_OptFls(myStarInfo.points[i].flight_level);
      fill_OptSpeed(myStarInfo.points[i].speed);
      fill_OptTime(myStarInfo.points[i].time);
      myStarInfo.points[i].route_centre_change=AvFlplt::CENTRE_NO_CHANGE;
      myStarInfo.points[i].route_rule_change=AvFlplt::RULE_NO_CHANGE;
      myStarInfo.points[i].route_control_change=AvFlplt::CONTROL_NO_CHANGE;
    }
  //}
  // struct => all fields got to be filled
  //else {
    // we want nothing
    AvStd::Empty myField = 0;
    in.field = myField;
  //}
}
void
fill_OptSid(AvFlplt::OptSid& in)
{
  // struct => all fields got to be filled
  //if (OPTSTAR) {
    AvFlplt::SidInfo mySidInfo;
    for (uint i28 = 0 ; i28 < 7 ; i28++) {
      mySidInfo.sid_name[i28]='a';
    }
    for (int i = 0 ; i < POINT_INFO_LIST ; i++) {
      fill_Waypoint(mySidInfo.points[i].waypoint);

      fill_OptFls(mySidInfo.points[i].flight_level);
      fill_OptSpeed(mySidInfo.points[i].speed);
      fill_OptTime(mySidInfo.points[i].time);
      mySidInfo.points[i].route_centre_change=AvFlplt::CENTRE_NO_CHANGE;
      mySidInfo.points[i].route_rule_change=AvFlplt::RULE_NO_CHANGE;
      mySidInfo.points[i].route_control_change=AvFlplt::CONTROL_NO_CHANGE;
    }
  //}
  // struct => all fields got to be filled
  //else {
    // we want nothing
    AvStd::Empty myField = 0;
    in.field = myField;
  //}
}

void
fill_OptAirportName(AvBamt::OptAirportName& in)
{
  if (OPTAIRPORTNAME) {
    // we want an AirportName
    AvBamt::AirportName myAirportName;
    for (uint i5 = 0 ; i5 < 4 ; i5++) {
      myAirportName[i5]='a';
    }
    in.airport(myAirportName);
  }
  else {
    // we want nothing
    AvStd::Empty myField = 0;
    in.field(myField);
  }
}

void
fill_OptCovariance(AvTrajt::OptCovariance& in)
{
  if (OPTCOVARIANCE) {
    // we want a Covariance
    AvTrajt::Covariance myMatrix;
    for (uint i1 = 0 ; i1 < 2 ; i1++) {
      for (uint i2 = 0 ; i2 < 2 ; i2++) {
    myMatrix[i1][i2]=4.0;
      }
    }
    in.matrix(myMatrix);
  }
  else {
    // we want nothing
    AvStd::Empty myField = 0;
    in.field(myField);
  }
}

void
fill_OptCstrIdList(AvCflgt::OptCstrIdList& in)
{
  if (OPTCSTRIDLIST) {
    // we want a CstrIdList
    AvCflgt::CstrIdList myCstrIdList;
    for (int i3 = 0 ; i3 < CSTRIDLIST ; i3++) {
      myCstrIdList[i3].kind=AvCflgt::STRATEGICAL;
      myCstrIdList[i3].nb=4;
    }
    in.cstr_id_list(myCstrIdList);
  }
  else {
    // we want nothing
    AvStd::Empty myField = 0;
    in.spare(myField);
  }
}

void
fill_OptWaypointAndId(AvTrajt::OptWaypointAndId& in)
{
  if (OPTWAYPOINTANDID) {
    // we want a WaypointAndId
    AvTrajt::WaypointAndId myWaypointAndId;

    fill_Waypoint(myWaypointAndId.waypoint);

    myWaypointAndId.key=4;
    in.pointandid(myWaypointAndId);
  }
  else {
    // we want nothing
    AvStd::Empty myField = 0;
    in.spare(myField);
  }
}

void
Mockup::fill_FlightPlanData(::AvAbsTypes::FlightPlanData& impl)
{
  for (uint i0 = 0 ; i0 < 8 ; i0++) {
    impl.ifp.call_sign[i0]='0';
  }
  impl.ifp.state=AvFlplt::INITIAL_STATE;
  impl.ifp.fp_completion=AvCflgt::FULL_FP;
  impl.ifp.flight_rules=AvBfpt::VFR;
  impl.ifp.acr_info.nb_of_aircraft=2;
  for (uint i1 = 0 ; i1 < 4 ; i1++) {
    impl.ifp.acr_info.icao_ac_type[i1]='a';
  }
  impl.ifp.acr_info.turbulence=AvBact::HEAVY;

  for (uint i2 = 0 ; i2 < 6 ; i2++) {
    impl.ifp.acr_info.model[i2]='a';
  }
  for (uint i3 = 0 ; i3 < 3 ; i3++) {
    impl.ifp.acr_info.company[i3]='a';
  }
  for (uint i4 = 0 ; i4 < 7 ; i4++) {
    impl.ifp.acr_info.engine[i4]='a';
  }
  impl.ifp.acr_info.ac_type=AvBact::JET;
  impl.ifp.acr_info.flight_type=AvBact::SHORT_RANGE;

  impl.ifp.acr_info.icao_flight_type='a';
  impl.ifp.acr_info.raw_ac_typz="string_10";
  impl.ifp.acr_info.avionics.rt_frequency=4.0;
  impl.ifp.acr_info.avionics.selected_ils_frequency=4.0;
  impl.ifp.acr_info.avionics.mode_a_state=AvBact::MODE_A_ON;
  for (uint i5 = 0 ; i5 < 4 ; i5++) {
    impl.ifp.acr_info.avionics.ssr_code_used[i5]=0;
  }
  impl.ifp.acr_info.avionics.mode_c_state=AvBact::MODE_C_ON;
  impl.ifp.acr_info.avionics.altimeter=AvBact::QNH;
  impl.ifp.acr_info.avionics.mode_c_fl=4;
  impl.ifp.acr_info.avionics.mode_s_address=4;
  impl.ifp.acr_info.avionics.mode_s_state=AvBact::MODE_S_ON;
  impl.ifp.acr_info.avionics.fms_equipment=AvBact::NOT_EQUIPED;
  impl.ifp.acr_info.avionics.fms_state=AvBact::FMS_ON;
  for (uint i6 = 0 ; i6 < 8 ; i6++) {
    impl.ifp.acr_info.avionics.dl_aircraft_id[i6]='a';
  }

    impl.ifp.acr_info.avionics.cma_ctx.cma_desc.version_id=4;
    impl.ifp.acr_info.avionics.cma_ctx.cma_desc.communication_class=AvBact::NO_DLCOMMUNICATION;
    impl.ifp.acr_info.avionics.cma_ctx.cma_desc.security_level=AvBact::LOW;
    impl.ifp.acr_info.avionics.cma_ctx.status=AvBact::CMA_ON;
    impl.ifp.acr_info.avionics.cpdlc_ctx.cpdlc_desc.version_id=4;
    impl.ifp.acr_info.avionics.cpdlc_ctx.cpdlc_desc.communication_class=AvBact::NO_DLCOMMUNICATION;
    impl.ifp.acr_info.avionics.cpdlc_ctx.cpdlc_desc.security_level=AvBact::LOW;
    impl.ifp.acr_info.avionics.cpdlc_ctx.status=AvBact::CPDLC_ON;
    impl.ifp.acr_info.avionics.dap_ctx.dap_desc.version_id=4;
    impl.ifp.acr_info.avionics.dap_ctx.dap_desc.communication_class=AvBact::NO_DLCOMMUNICATION;
    impl.ifp.acr_info.avionics.dap_ctx.dap_desc.security_level=AvBact::LOW;
    impl.ifp.acr_info.avionics.dap_ctx.status=AvBact::DAP_ON;
    impl.ifp.acr_info.avionics.tn_ctx.tn_desc.version_id=4;
    impl.ifp.acr_info.avionics.tn_ctx.tn_desc.communication_class=AvBact::NO_DLCOMMUNICATION;
    impl.ifp.acr_info.avionics.tn_ctx.tn_desc.security_level=AvBact::LOW;
    impl.ifp.acr_info.avionics.tn_ctx.status=AvBact::TN_ON;
    impl.ifp.acr_info.avionics.wns_ctx.wns_desc.version_id=4;
    impl.ifp.acr_info.avionics.wns_ctx.wns_desc.communication_class=AvBact::NO_DLCOMMUNICATION;
    impl.ifp.acr_info.avionics.wns_ctx.wns_desc.security_level=AvBact::LOW;
    impl.ifp.acr_info.avionics.wns_ctx.status=AvBact::WNS_ON;
    impl.ifp.acr_info.avionics.ads_ctx.ads_desc.version_id=4;
    impl.ifp.acr_info.avionics.ads_ctx.ads_desc.communication_class=AvBact::NO_DLCOMMUNICATION;
    impl.ifp.acr_info.avionics.ads_ctx.ads_desc.security_level=AvBact::LOW;
    impl.ifp.acr_info.avionics.ads_ctx.status=AvBact::ADS_ON;
    impl.ifp.acr_info.avionics.acas_ctx.acas_desc.version_id=4;
    impl.ifp.acr_info.avionics.acas_ctx.acas_desc.communication_class=AvBact::NO_DLCOMMUNICATION;
    impl.ifp.acr_info.avionics.acas_ctx.acas_desc.security_level=AvBact::LOW;
    for (uint i7 = 0 ; i7 < 10 ; i7++) {
       impl.ifp.acr_info.avionics.acas_ctx.acas_equipment[i7]='a';
    }
    impl.ifp.acr_info.avionics.acas_ctx.status=AvBact::ACAS_ON;
    impl.ifp.acr_info.avionics.asas_ctx.asas_desc.version_id=4;
    impl.ifp.acr_info.avionics.asas_ctx.asas_desc.communication_class=AvBact::NO_DLCOMMUNICATION;
    impl.ifp.acr_info.avionics.asas_ctx.asas_desc.security_level=AvBact::LOW;
    for (uint i8 = 0 ; i8 < 10 ; i8++) {
       impl.ifp.acr_info.avionics.asas_ctx.asas_equipment[i8]='a';
    }
    impl.ifp.acr_info.avionics.asas_ctx.status=AvBact::ASAS_ON;

  for (uint i9 = 0 ; i9 < 4 ; i9++) {
    impl.ifp.departure_info.adep[i9]='a';
  }
  impl.ifp.departure_info.raw_depz='a';

  fill_OptDateDay(impl.ifp.departure_info.eobd);

  fill_OptTime(impl.ifp.departure_info.eobt);

  fill_OptTime(impl.ifp.departure_info.etd);

  fill_OptTime(impl.ifp.departure_info.atd);

  fill_OptTime(impl.ifp.departure_info.dep_time_slot);

  for (uint i10 = 0 ; i10 < 4 ; i10++) {
    impl.ifp.arrival_info.ades[i10]='a';
  }
  impl.ifp.arrival_info.raw_destz='a';

  fill_OptTime(impl.ifp.arrival_info.eet);

  fill_OptTime(impl.ifp.arrival_info.ata);

  for (uint i11 = 0 ; i11 < 2 ; i11++) {
    for (uint i12 = 0 ; i12 < 4 ; i12++) {
      impl.ifp.arrival_info.alt_airports[i11][i12]='a';
    }
  }

  fill_Time(impl.ifp.fir_time);

  impl.ifp.rfl=4;

  fill_Speed(impl.ifp.cruise_speed);

  impl.ifp.raw_route="string_10";
  for (uint i13 = 0 ; i13 < 8 ; i13++) {
    impl.ifp.suppl_info.registration[i13]='a';
  }
  for (uint i14 = 0 ; i14 < 8 ; i14++) {
    impl.ifp.suppl_info.flplt_rem[i14]='a';
  }
  for (uint i15 = 0 ; i15 < 8 ; i15++) {
    impl.ifp.suppl_info.sts[i15]='a';
  }
  impl.ifp.suppl_info.flight_region=AvBamt::UIR;
  for (uint i16 = 0 ; i16 < 4 ; i16++) {
    impl.ifp.suppl_info.selcal_key[i16]='a';
  }
  impl.ifp.suppl_info.oper="string_10";
  impl.ifp.suppl_info.performance="string_10";
  impl.ifp.suppl_info.nca_eq="string_10";
  impl.ifp.suppl_info.com_eq="string_10";
  impl.ifp.suppl_info.nav_eq="string_10";
  impl.ifp.suppl_info.ssr_eq='a';

  fill_Waypoint(impl.ifp.suppl_info.fr_changing_at);

  fill_OptFPInitialCond(impl.ifp.opt_initial_cond);

  impl.route.rfl=4;
  for (uint i19 = 0 ; i19 < 4 ; i19++) {
    impl.route.departure_info.adep[i19]='a';
  }
  AvFlplt::SidInfo temp20;
  for (uint i20 = 0 ; i20 < 7 ; i20++) {
    temp20.sid_name[i20]='a';
  }
  for (int i21 = 0 ; i21 < POINT_INFO_LIST ; i21++) {

    fill_Waypoint(temp20.points[i21].waypoint);

    fill_OptFls(temp20.points[i21].flight_level);

    fill_OptSpeed(temp20.points[i21].speed);

    fill_OptTime(temp20.points[i21].time);

    temp20.points[i21].route_centre_change=AvFlplt::CENTRE_NO_CHANGE;
    temp20.points[i21].route_rule_change=AvFlplt::RULE_NO_CHANGE;
    temp20.points[i21].route_control_change=AvFlplt::CONTROL_NO_CHANGE;
  }
  impl.route.departure_info.sid.sid_info = temp20;
  for (int i23 = 0 ; i23 < ROUTE_LENGTH ; i23++) {
    fill_BasicRouteItem(impl.route.basic_route_list[i23]);
  }
  for (uint i27 = 0 ; i27 < 4 ; i27++) {
    impl.route.arrival_info.ades[i27]='a';
  }
  AvFlplt::StarInfo temp32;
  for (uint i28 = 0 ; i28 < 7 ; i28++) {
    temp32.star_name[i28]='a';
  }
  for (int i29 = 0 ; i29 < POINT_INFO_LIST ; i29++) {

    fill_Waypoint(temp32.points[i29].waypoint);

    fill_OptFls(temp32.points[i29].flight_level);

    fill_OptSpeed(temp32.points[i29].speed);

    fill_OptTime(temp32.points[i29].time);

    temp32.points[i29].route_centre_change=AvFlplt::CENTRE_NO_CHANGE;
    temp32.points[i29].route_rule_change=AvFlplt::RULE_NO_CHANGE;
    temp32.points[i29].route_control_change=AvFlplt::CONTROL_NO_CHANGE;
  }
  impl.route.arrival_info.star.star_info = temp32;
}

void
Mockup::fill_Trajectory(::AvTrajt::TrajectoryFP& impl)
{
  impl.trajectory.traj_granularity=AvTrajt::SIGNIFICANT_SAMPLES_ONLY;
  for (int i0 = 0 ; i0 < TRAJE_LENGTH ; i0++) {

    fill_Time(impl.trajectory.detailed_traj[i0].eto);

    impl.trajectory.detailed_traj[i0].nav_dta.pos.latit.lat_hemis=AvBsut::NORTH;
    impl.trajectory.detailed_traj[i0].nav_dta.pos.latit.lat_azim=4.0;
    impl.trajectory.detailed_traj[i0].nav_dta.pos.longit.long_dir=AvBsut::EAST;
    impl.trajectory.detailed_traj[i0].nav_dta.pos.longit.long_azim=4.0;
    impl.trajectory.detailed_traj[i0].nav_dta.pos.altit=4;
    impl.trajectory.detailed_traj[i0].nav_dta.track=4.0;
    impl.trajectory.detailed_traj[i0].nav_dta.g_speed=4.0;
    impl.trajectory.detailed_traj[i0].kin_dta.heading=4.0;
    impl.trajectory.detailed_traj[i0].kin_dta.tas=4.0;
    fill_SpeedCasMach(impl.trajectory.detailed_traj[i0].kin_dta.speed);
    impl.trajectory.detailed_traj[i0].kin_dta.rocd=4.0;
    impl.trajectory.detailed_traj[i0].kin_dta.rot=4.0;

    impl.trajectory.detailed_traj[i0].uncertainty_zone.geo_tube.lat_accuracy.left=4.0;
    impl.trajectory.detailed_traj[i0].uncertainty_zone.geo_tube.lat_accuracy.right=4.0;
    impl.trajectory.detailed_traj[i0].uncertainty_zone.geo_tube.long_accuracy.forward=4.0;
    impl.trajectory.detailed_traj[i0].uncertainty_zone.geo_tube.long_accuracy.backward=4.0;
    impl.trajectory.detailed_traj[i0].uncertainty_zone.geo_tube.level_accuracy.above=4;
    impl.trajectory.detailed_traj[i0].uncertainty_zone.geo_tube.level_accuracy.below=4;
    impl.trajectory.detailed_traj[i0].uncertainty_zone.geo_tube.time_accuracy.early=4.0;
    impl.trajectory.detailed_traj[i0].uncertainty_zone.geo_tube.time_accuracy.late=4.0;

    fill_OptCovariance(impl.trajectory.detailed_traj[i0].uncertainty_zone.cov_matrix);

    impl.trajectory.detailed_traj[i0].mass=4.0;
    impl.trajectory.detailed_traj[i0].elapsed_distance=4.0;
    impl.trajectory.detailed_traj[i0].phase=AvBfpt::UNKNOWN_ATC_FLIGHT;
    impl.trajectory.detailed_traj[i0].attributes.overfly=1;
    impl.trajectory.detailed_traj[i0].attributes.across=1;
    impl.trajectory.detailed_traj[i0].attributes.cstr=1;
    impl.trajectory.detailed_traj[i0].attributes.airport=1;
    impl.trajectory.detailed_traj[i0].attributes.toc=1;
    impl.trajectory.detailed_traj[i0].attributes.tod=1;
    impl.trajectory.detailed_traj[i0].attributes.boc=1;
    impl.trajectory.detailed_traj[i0].attributes.bod=1;
    impl.trajectory.detailed_traj[i0].attributes.tip=1;
    impl.trajectory.detailed_traj[i0].attributes.eot=1;
    impl.trajectory.detailed_traj[i0].attributes.ccr=1;
    impl.trajectory.detailed_traj[i0].attributes.cc=1;
    impl.trajectory.detailed_traj[i0].attributes.cd=1;
    impl.trajectory.detailed_traj[i0].attributes.soa=1;
    impl.trajectory.detailed_traj[i0].attributes.eoa=1;
    impl.trajectory.detailed_traj[i0].attributes.sod=1;
    impl.trajectory.detailed_traj[i0].attributes.eod=1;
    impl.trajectory.detailed_traj[i0].attributes.info=1;

    fill_OptCstrIdList(impl.trajectory.detailed_traj[i0].cstr_opt);

    fill_OptWaypointAndId(impl.trajectory.detailed_traj[i0].waypoint_opt);

    fill_OptAirportName(impl.trajectory.detailed_traj[i0].airport_opt);
  }
  for (int i6 = 0 ; i6 < TRAJE_LENGTH ; i6++) {
    fill_Time(impl.forecast_manoeuvre[i6].eto);
    impl.forecast_manoeuvre[i6].nav_dta.pos.latit.lat_hemis=AvBsut::NORTH;
    impl.forecast_manoeuvre[i6].nav_dta.pos.latit.lat_azim=4.0;
    impl.forecast_manoeuvre[i6].nav_dta.pos.longit.long_dir=AvBsut::EAST;
    impl.forecast_manoeuvre[i6].nav_dta.pos.longit.long_azim=4.0;
    impl.forecast_manoeuvre[i6].nav_dta.pos.altit=4;
    impl.forecast_manoeuvre[i6].nav_dta.track=4.0;
    impl.forecast_manoeuvre[i6].nav_dta.g_speed=4.0;
    impl.forecast_manoeuvre[i6].kin_dta.heading=4.0;
    impl.forecast_manoeuvre[i6].kin_dta.tas=4.0;
    fill_SpeedCasMach(impl.forecast_manoeuvre[i6].kin_dta.speed);
    impl.forecast_manoeuvre[i6].kin_dta.rocd=4.0;
    impl.forecast_manoeuvre[i6].kin_dta.rot=4.0;
    impl.forecast_manoeuvre[i6].uncertainty_zone.geo_tube.lat_accuracy.left=4.0;
    impl.forecast_manoeuvre[i6].uncertainty_zone.geo_tube.lat_accuracy.right=4.0;
    impl.forecast_manoeuvre[i6].uncertainty_zone.geo_tube.long_accuracy.forward=4.0;
    impl.forecast_manoeuvre[i6].uncertainty_zone.geo_tube.long_accuracy.backward=4.0;
    impl.forecast_manoeuvre[i6].uncertainty_zone.geo_tube.level_accuracy.above=4;
    impl.forecast_manoeuvre[i6].uncertainty_zone.geo_tube.level_accuracy.below=4;
    impl.forecast_manoeuvre[i6].uncertainty_zone.geo_tube.time_accuracy.early=4.0;
    impl.forecast_manoeuvre[i6].uncertainty_zone.geo_tube.time_accuracy.late=4.0;

    fill_OptCovariance(impl.forecast_manoeuvre[i6].uncertainty_zone.cov_matrix);

    impl.forecast_manoeuvre[i6].mass=4.0;
    impl.forecast_manoeuvre[i6].elapsed_distance=4.0;
    impl.forecast_manoeuvre[i6].phase=AvBfpt::UNKNOWN_ATC_FLIGHT;
    impl.forecast_manoeuvre[i6].attributes.overfly=1;
    impl.forecast_manoeuvre[i6].attributes.across=1;
    impl.forecast_manoeuvre[i6].attributes.cstr=1;
    impl.forecast_manoeuvre[i6].attributes.airport=1;
    impl.forecast_manoeuvre[i6].attributes.toc=1;
    impl.forecast_manoeuvre[i6].attributes.tod=1;
    impl.forecast_manoeuvre[i6].attributes.boc=1;
    impl.forecast_manoeuvre[i6].attributes.bod=1;
    impl.forecast_manoeuvre[i6].attributes.tip=1;
    impl.forecast_manoeuvre[i6].attributes.eot=1;
    impl.forecast_manoeuvre[i6].attributes.ccr=1;
    impl.forecast_manoeuvre[i6].attributes.cc=1;
    impl.forecast_manoeuvre[i6].attributes.cd=1;
    impl.forecast_manoeuvre[i6].attributes.soa=1;
    impl.forecast_manoeuvre[i6].attributes.eoa=1;
    impl.forecast_manoeuvre[i6].attributes.sod=1;
    impl.forecast_manoeuvre[i6].attributes.eod=1;
    impl.forecast_manoeuvre[i6].attributes.info=1;

    fill_OptCstrIdList(impl.forecast_manoeuvre[i6].cstr_opt);

    fill_OptWaypointAndId(impl.forecast_manoeuvre[i6].waypoint_opt);

    fill_OptAirportName(impl.forecast_manoeuvre[i6].airport_opt);
  } 
 }
