/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

import cdmw.common.BadParameterException;
import cdmw.orbsupport.StrategyList;
import cdmw.ossupport.OS;

import java.io.FileWriter;
import java.io.PrintWriter;

//import cdmw.logging.Logger;
import cdmw.logging.LogManager;

import CdmwLBPerf.PingableInterface;
import org.omg.CORBA.ORB;
import org.omg.CORBA.ORBPackage.InvalidName;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;
import org.omg.PortableServer.POAManager;
import org.omg.PortableServer.POAManagerPackage.AdapterInactive;

public class server_Q340
{

    private static final String LB_SERVER_OPT = "--LBInterceptorSide=Server";
    private static final String REPLICA_NAME_OPT = "--ReplicaName";
    private static final String NAMING_REPOS_URL_OPT = "--namingRepos-url";
    private static final String GM_URL_OPT = "--GroupMng-url";
    private static final String IOR_FILE_NAME = "server_Q340";

    private static ORB m_orb;
    private static String replica_name;
    private static String naming_repos_url;
    private static String lb_gm_url;
    private static String file_name;
    
    public static void main(String[] args)
    {
        // init log manager
        LogManager.Init(args);
        // add POA strategy
        StrategyList strategyList = new StrategyList();
        strategyList.addORBThreaded();
        try
        {
            strategyList.addPOAThreadPerConnection();
        }
        catch (BadParameterException e)
        {
            e.printStackTrace();
            System.exit(-1);
        }

        try {
            String opt = OS.getOptionValue( args, "--help");
            if (opt.compareTo("yes") == 0) {
                usage();
                System.exit(0);
            }

            replica_name = OS.getOptionValue(args, REPLICA_NAME_OPT);
            if (replica_name.compareTo("no") == 0)
            {
                replica_name=IOR_FILE_NAME;
            }
            file_name = replica_name + ".IOR";

            lb_gm_url = OS.getOptionValue(args, GM_URL_OPT);
            if (lb_gm_url.equals("no") ||
                lb_gm_url.equals("yes"))
            {
                System.out.println("ERROR: LBGroupManager URL it's missed!\n");
                usage();
                System.exit(0);
            }

            naming_repos_url = OS.getOptionValue(args, NAMING_REPOS_URL_OPT);
            if (naming_repos_url.equals("no") ||
                naming_repos_url.equals("yes"))
            {
                System.out.println("ERROR: Repository URL it's missed!\n");
                usage();
                System.exit(0);
            }
        }
        catch (NumberFormatException e) {
        }
        // parse argument
        // prepare arguments for lbinit()
        String[] ORB_init_args = new String[(args.length) + 2];
        for (int i = 0; i < args.length; i++)
        {
            ORB_init_args[i] = args[i];
        }
        // lb server mode
        ORB_init_args[args.length] = LB_SERVER_OPT;
        ORB_init_args[args.length+1]= "--LBGroupManager=" + lb_gm_url;

        // initialise lb service
        cdmw.lb.lbinit.LBServiceInit.init(ORB_init_args);

        m_orb =  org.omg.CORBA.ORB.init(args, null);
        //m_orb = cdmw.orbsupport.ORBSupport.ORBInit(args, null);

        POAManager poaManager = null;
        org.omg.CORBA.Object obj = null;

        try
        {
            obj = m_orb.resolve_initial_references("RootPOA");
            POA root = POAHelper.narrow(obj);
            poaManager = root.the_POAManager();
            poaManager.activate();
        }
        catch (AdapterInactive ai)
        {
            ai.printStackTrace();
        }
        catch (InvalidName in)
        {
            in.printStackTrace();
        }

        PingableInterfaceImpl pi =  new PingableInterfaceImpl(replica_name);
        PingableInterface pi_ref = pi._this(m_orb);
        String ref_string = m_orb.object_to_string(pi_ref);

        try
        {

            PrintWriter out = new PrintWriter(new FileWriter(file_name));
            out.println(ref_string);
            out.flush();
            out.close();
        }

        catch (java.io.FileNotFoundException fnfe)
        {
            System.out.println(fnfe);
        }
        catch (java.io.IOException ioe)
        {
            System.out.println(ioe);
        }
        // ===================================================
        // Get the naming and repository
        //     from the input URL
        // ===================================================
        try
        {
            com.thalesgroup.CdmwNamingAndRepository.Repository repository = null;
            org.omg.CORBA.Object obj2 = m_orb.string_to_object(naming_repos_url);
            repository = com.thalesgroup.CdmwNamingAndRepository.RepositoryHelper.narrow(obj2);

            // ======================================================
            // Get HELLO_LB name domain from repository
            // ======================================================
            com.thalesgroup.CdmwNamingAndRepository.NameDomain helloDomain =
            repository.resolve_name_domain("PERFTEST/HELLO_LB");
            // ========================================================
            // Register the HelloInterface object in the HELLO name domain
            // ========================================================
            // reserve the name "HelloInterface" in the domain
            System.out.println("Register the HelloInterface object in the HELLO name domain");
            org.omg.CORBA.Any regId =  helloDomain.new_name (replica_name);

            // register the object
            System.out.println("register the object");
            helloDomain.register_object (regId, pi_ref);
            System.out.println("server_Q-340>> LB Server (Java) started and registered in the Repository\n");
        }
        catch( com.thalesgroup.CdmwNamingAndRepository.NoNameDomain e) {
             e.printStackTrace();
        }
        catch( com.thalesgroup.CdmwNamingAndRepository.NameDomainPackage.AlreadyExists e) {
             e.printStackTrace();
        }
        catch( com.thalesgroup.CdmwNamingAndRepository.InvalidName e) {
             e.printStackTrace();
        }
        catch( com.thalesgroup.CdmwNamingAndRepository.NameDomainPackage.InvalidRegistration e) {
             e.printStackTrace();
        }
        m_orb.run();
    }

    private static void usage()
    {
        System.out.println("Usage:\n"
        + " Options\n"
        + "--help : Display this information.\n"
        + REPLICA_NAME_OPT + " = <path/name>, path/name of the file where IOR should be stored.\n"
        + GM_URL_OPT + " = <LBGroupManager url>\n"
        + NAMING_REPOS_URL_OPT + "= <naming repos URL>");
    }


}
