/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <fstream>

// Cdmw Files
#include <Foundation/common/Options.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <LoadBalancing/lbinit/LBServiceInit.hpp>
#include <Foundation/common/String.hpp>
#include <idllib/CdmwLBGroupManager.stub.hpp>
#include <time.h>

// cTools include
#include "qualif/perftests/testperfcommon/HighResTimer.hpp"
#include "qualif/perftests/testperfcommon/HighResTime.hpp"
#include "qualif/perftests/testperfcommon/HighResClock.hpp"

namespace
{

    const std::string FILE_PREFIX  = "--FilePrefix";
    const std::string GM_IOR_OPTION = "--LBGroupManager";
    const std::string LB_INTERCEPTOR_SIDE = "--LBInterceptorSide";
    const std::string NUM_ITERATION = "--Iteration";
    std::string FILE_CLIENT = "LBInitClientStartTime.txt";
    std::string FILE_SERVER = "LBInitServerStartTime.txt";
    std::string FILE_= "";
 
}// end anonymous namespace

static void
usage(std::ostream & os, const std::string &program_name)
{
    os << "Usage:\n"
       << program_name << " options\n"
       << "\n"
       << "Options:\n"
       << "--help                                         Display this information.\n"
       << "\n"
       << "-ORBInitRef LBGroupManager=<rep_corbaloc>  ReplicationManager corbaloc\n"
       << "[" << FILE_PREFIX <<"]=path          path of the file where hello IOR should be stored.\n"
       << "[" << GM_IOR_OPTION <<"]=LBGroupManager      \n"
       << std::endl;
}

int main( int argc, char* argv[] )
{
    // help argument
    if (argc <=2)
    {
        usage (std::cerr, argv[0]);
        return 0;
    }

    if (argc >= 2)
    {
        if (strcmp(argv[1], "--help") == 0)
        {
            usage (std::cerr, argv[0]);
            return 0;
        }
    }
    std::cout << "START TEST!!!" << std::endl;
    // get Name File prefix
    std::string file_prefix = Cdmw::OsSupport::OS::get_option_value (argc, argv, FILE_PREFIX);
    std::string interceptor_side  = Cdmw::OsSupport::OS::get_option_value (argc, argv, LB_INTERCEPTOR_SIDE);
    std::string s_iteration = Cdmw::OsSupport::OS::get_option_value (argc, argv, NUM_ITERATION);
    int iteration = atoi(s_iteration.c_str());

    if (interceptor_side.compare("yes")&&interceptor_side.compare("no"))
    { 
        std::cout << "TEST " << interceptor_side << " side " << std::endl;
    }
    else
    {   
        std::cout << "I need --LBInterceptorSide option to run" << std::endl;
        exit(1);
    }
    
    if (!interceptor_side.compare("Client"))
    { 
        FILE_ = FILE_CLIENT;
    }
    if (!interceptor_side.compare("Server"))
    { 
        FILE_ = FILE_SERVER;
    }
    
    if (file_prefix.compare("yes")&&file_prefix.compare("no"))
    { 
        
    }
    else
    {   
        std::cout << "I need --FilePrefix option to run " << std::endl;
        exit(1);
    }
    
    if (s_iteration.compare("yes")&&s_iteration.compare("no"))
    { 
        std::cout << "Num invocation: " << iteration << std::endl;
    }
    else
    {   
        std::cout << "I need --FilePrefix option to run " << std::endl;
        exit(1);
    }
    FILE_ = file_prefix + FILE_;
     
    try
    {
        char** ORB_init_argv = 0;
        int ORB_init_argc = argc;
        std::string str_opt( "" );

        for (int i=0; i<argc; i++)
            str_opt += std::string( argv[i] ) + " ";
        str_opt += std::string( "--LBInterceptorSide="+interceptor_side);
        ORB_init_argc += 1;
        ORB_init_argv = Cdmw::Common::String::to_char_array(
            Cdmw::Common::String::to_strings( str_opt ) );
    
        long double start, end, temp_elapsed;
        std::cout << "LBInit "<< interceptor_side <<" side!!!" << std::endl; 
        std::ofstream os(FILE_.c_str());
        for(int i=0;i<iteration;i++){
            start = ctools::time::HighResClock::getTime().toMicrosec();
            Cdmw::LB::LBServiceInit::Init(ORB_init_argc, ORB_init_argv);
            end = ctools::time::HighResClock::getTime().toMicrosec();
            temp_elapsed = end-start; 
            os << temp_elapsed << std::endl;
        }
        os.close();
        std::cout << "Generated file: "<<  FILE_ << std::endl;
        
    }catch (const CORBA::SystemException &e )
        {
            std::cerr <<__FILE__<<" "<<__LINE__<<" "<< e << std::endl;
            return -1;
        }
    catch( const CORBA::Exception &e )
        {
            std::cerr <<__FILE__<<" "<<__LINE__<<" "<< e._name() << std::endl;
            return -1;
        }
    catch(...)
        {
            std::cerr << "Uncaught exception" << std::endl;
            return -1;
        }

    return 0;
}
        
