##################################################
## QTPDR requirements
##################################################
DISP_TIME_CR <- 0.1 # (10 %)
DISP_TIME_OW <- 0.3 # (30 %)

##################################################
## local definitions
##################################################
results <- "test_summary.result.Q-304"
warmup <- 10      # if warmup=N then the first N samples are filtered out (warm up stage)
threshold <- 1    # if threshold=N then N% of samples is filtered out

##################################################
## filter functions: exclude outliers and/or 
## warm up stage samples from the data set
##################################################
filter_warmup <- function(x) x[warmup:length(x)]
filter_top <- function(x) sort(x)[0:ceiling((length(x)*(100-(threshold%%100)))/100)]
filter <- function(x) filter_top(filter_warmup(x))


## main

write("****** PERFORMANCE TEST Q-304 ********", results)
write("*** please note: unit is (1 msec) ***", results, append=TRUE)

## ACQUISITION OF DELTA_C.dat
DC_CR_0<-scan("Q-304_CR_0_DELTA_C.dat")
DC_OW_0<-scan("Q-304_OW_0_DELTA_C.dat")

## INVOCATION COMPARISON CR-0 OW-0

### DELTA_C FILTERING (note: while filtering, time unit is also converted from usec to msec)
DC_CR_0f<-filter(DC_CR_0/1000);
DC_OW_0f<-filter(DC_OW_0/1000);

write("", results, append=TRUE)

##################################################
## calculate dispersion around mean value 
##################################################
write("CR-0 results", results, append=TRUE)
write("------------", results, append=TRUE)
avg <- mean(DC_CR_0f)
std <- sqrt(var(DC_CR_0f))
DISP_CR <- std/avg
write(ncolumns=2, c("AVG =", avg), results, append=TRUE)
write(ncolumns=2, c("STD =", std), results, append=TRUE)
write(ncolumns=2, c("STD/AVG =", DISP_CR), results, append=TRUE)
write("", results, append=TRUE)

##################################################
## calculate dispersion around mean value 
##################################################
write("OW-0 results", results, append=TRUE)
write("------------", results, append=TRUE)
avg <- mean(DC_OW_0f)
std <- sqrt(var(DC_OW_0f))
DISP_OW <- std/avg
write(ncolumns=2, c("AVG =", avg), results, append=TRUE)
write(ncolumns=2, c("STD =", std), results, append=TRUE)
write(ncolumns=2, c("STD/AVG =", DISP_OW), results, append=TRUE)

##################################################
## write test OK/NOK to a file 
##################################################

if ((DISP_CR <= DISP_TIME_CR) && (DISP_OW <= DISP_TIME_OW)) {
    write("########################################", results, append=TRUE)
    write("## The qualification scenario is: OK  ##", results, append=TRUE)
    write("########################################", results, append=TRUE)
} else {
    write("########################################", results, append=TRUE)
    write("## The qualification scenario is: NOK ##", results, append=TRUE)
    write("########################################", results, append=TRUE)
}
##################################################
## display the file with SO primitive
##################################################
system("cat test_summary.result.Q-304")
