/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_CLIENT_THREAD_HPP
#define INCL_CLIENT_THREAD_HPP

#include <iostream>
#include <fstream>
#include <sstream>

#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/osthreads/Thread.hpp"
#include "Repository/repositoryinterface/RepositoryInterface.hpp"
#include "tao/Messaging/Messaging.h"

#include "qualif/perftests/testperfcommon/HighResTimer.hpp"
#include "qualif/perftests/testperfcommon/HighResTime.hpp"
#include "qualif/perftests/testperfcommon/HighResClock.hpp"
#include "qualif/perftests/testperfcommon/ClockTime.hpp"
#include "qualif/perftests/testperfcommon/ParseCfg.hpp"
#include "Invocation.stub.hpp"

#define CALL_RETURN       "CR"
#define ONE_WAY           "OW"

#define MEASURE_HR "TIME_HR" // cTools HighRes Time
#define MEASURE_TH "TIME_TH" // clock thread cpu time 
#define MEASURE_TK "TIME_TK" // cTools HighRes clock tick count

namespace PerfThread {

class ClientThread : public Cdmw::OsSupport::Thread
{
public:
    
    ClientThread(CORBA::ORB_ptr orb, utils::cfgData& cfg)
        throw( CORBA::SystemException );

    virtual ~ClientThread()
        throw()
    {
    
    }
    
    void stop() throw()
    {
        delete m_measure;
        m_stopped = true;
    }
    
protected:

    void run() throw();
    
private:
    /**
     * The orb reference.
     */
    CORBA::ORB_var m_orb;

    /**
     * Client configuration data
     */
    utils::cfgData m_cfg;

    /**
     * TestPing reference
     */
    Invocation::TestPing_var m_testping;

    /**
     * time measure type
     */
    ClockTime *m_measure;

    /**
     * configuration data
     */
    std::string m_invocation_type;
    std::string m_fileprefix;
    std::string m_data_type;
    int m_niter;
    int m_delay;
    int m_data_size;

    /**
     * Thread info
     */
    bool m_stopped;
   
};

} // namespace PerfThread

#endif // INCL_CLIENT_THREAD_HPP
