/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_MY_PROCESS_BEHAVIOUR_HPP 
#define INCL_MY_PROCESS_BEHAVIOUR_HPP 

#include "SystemMngt/platforminterface/PlatformInterface.hpp"
#include <ConfAndPlug/cdmwinit/ProcessControl.hpp>
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/orbsupport/CORBA.hpp"

#include "Foundation/orbsupport/RefCountLocalObject.hpp"
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include "Repository/repositoryinterface/RepositoryInterface.hpp"
#include "Repository/idllib/CdmwNamingAndRepository.stub.hpp"
#include "test_repository_bind/TestHello_impl.hpp"
#include "test_repository_bind/TestRepositoryBinder.hpp"
#include <string>
#include <sstream>
#include <iostream>
#include <fstream>

/**
* Purpose:
* <p>
* The abstract process behaviour.
*/
class MyProcessBehaviour : public Cdmw::CdmwInit::ProcessControl
{

public:

    MyProcessBehaviour  (CORBA::ORB_ptr orb, PortableServer::POA_ptr rootPOA, std::string bind_timeout)
        : m_bind_timeout( bind_timeout )
    {
          m_orb = CORBA::ORB::_duplicate(orb);
          m_rootPOA = PortableServer::POA::_duplicate(rootPOA);
    }

    virtual ~MyProcessBehaviour() throw ()
    {
     
    }

    
    /**
    * Purpose:
    * <p>
    * the behaviour for the
    * IDL:thalesgroup.com/CdmwPlatformMngt/Process/nb_steps:1.0
    * attribute
    */
    virtual CORBA::ULong nb_steps() throw(CORBA::SystemException)
    {
        return 1;
    }
    
    
    /**
    * Purpose:
    * <p>
    * the behaviour for the
    * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_service:1.0
    * operation
    */
    virtual CORBA::Object_ptr get_service() throw(CORBA::SystemException)
    {
        return CORBA::Object::_nil();
    }
    
    
    /**
    * Purpose:
    * <p>
    * the behaviour for the
    * IDL:thalesgroup.com/CdmwPlatformMngt/Process/initialise:1.0
    * operation
    */
    virtual void on_initialise(const CdmwPlatformMngtBase::StartupKind& startup_kind)
        throw( CORBA::SystemException)
    {

    }
    
    /**
    * Purpose:
    * <p>
    * the behaviour for the
    * IDL:thalesgroup.com/CdmwPlatformMngt/Process/run:1.0
    * operation
    * @param
    * @return
    * @exception CdmwPlatformMngt::ProcessDelegate::NotReadyToRun
    */
    virtual void on_run()
    throw( CORBA::SystemException,
           CdmwPlatformMngt::ProcessDelegate::NotReadyToRun )
    {    
        
        Cdmw::TestUtils::CORBATestManager::instance()->set_property ("bind_timeout", m_bind_timeout);
        
        Cdmw::TestUtils::CORBATestManager::instance()->run_tests();
        // no summary: R-analysis prints the test result  
        //Cdmw::TestUtils::CORBATestManager::instance()->summary();
    }

    /**
    * Purpose:
    * <p>
    * the behaviour for the
    * IDL:thalesgroup.com/CdmwPlatformMngt/Process/stop:1.0
    * operation
    */
    virtual void on_stop() throw(CORBA::SystemException)
    {
    }


    
private:

    /**
    * The ORB.
    */
    CORBA::ORB_var m_orb;

    /**
     * The poa
     */
    PortableServer::POA_var m_rootPOA;

    /**
     * bind_timeout
     */
    std::string m_bind_timeout;
    
    /**
     * test
     */
    Cdmw::TestRepositoryBinder* m_test;
    
};


#endif // INCL_MY_PROCESS_BEHAVIOUR_HPP

