/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "test_repository_bind/TestHello_impl.hpp"
#include "test_repository_bind/BindReaderThread.hpp"
#include "Repository/repositoryinterface/RepositoryInterface.hpp"
#include "Foundation/commonsvcs/naming/NamingInterface.hpp"
#include "Foundation/commonsvcs/naming/NamingUtil.hpp"
#include <string>
#include <cctype>
#include <algorithm>

#include "Foundation/common/System.hpp"
#include "Foundation/common/Locations.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/common/Exception.hpp"
#include "Foundation/common/Assert.hpp"

#include "Foundation/osthreads/Mutex.hpp"
#include "Foundation/osthreads/Condition.hpp"
#include "SystemMngt/platforminterface/PlatformInterface.hpp"
#include "SystemMngt/platforminterface/ServiceNames.hpp"

namespace
{


}; // End anonymous namespace

namespace Cdmw
{
//----------------------------------------------------------------------------
BindReaderThread::BindReaderThread(Cdmw::OsSupport::Condition *cond) 
    throw (CORBA::SystemException)
  : m_terminated(true),
    m_is_binded(false),
    m_bindAvailable(cond)
{
}

//----------------------------------------------------------------------------
BindReaderThread::BindReaderThread() throw()
{
}

//----------------------------------------------------------------------------
void BindReaderThread::run() throw()
{

    m_terminated = false;

    CORBA::Object_var obj =
    PlatformMngt::PlatformInterface::Get_service(PlatformMngt::ServiceNames::NAMING_AND_REPOSITORY_SERVICE);

    CdmwNamingAndRepository::Repository_var repository = CdmwNamingAndRepository::Repository::_narrow(obj.in());


    CosNaming::NamingContext_var tmp_nc =
    repository->resolve_root_context(CdmwNamingAndRepository::DEFAULT_ROOT_CONTEXT);

    CosNaming::NamingContextExt_var defaultRootCtx;
    defaultRootCtx = CosNaming::NamingContextExt::_narrow(tmp_nc.in());

    CosNaming::Name_var name = new CosNaming::Name;
    name->length(1);
    CosNaming::NameComponent_var nc = new CosNaming::NameComponent;
    nc->id = CORBA::string_dup(BIND_OBJECT_ID);
    nc->kind = CORBA::string_dup("object");
    name->operator[](0) = nc.in();
    
    //main thread cycle
    while(!m_terminated)
    {
        if(m_is_binded)
        {
            try
            {
                obj = defaultRootCtx->resolve(name.in());
                Cdmw::HelloInterface1_var res_obj1 = Cdmw::HelloInterface1::_narrow(obj.in());
                
                std::string res = res_obj1->hello();
                if (strcmp(res.c_str(), BIND_OBJECT_ID) == 0)
                {
                    m_bindAvailable->broadcast();
                    m_is_binded = false; //to prevent multiple broadcasts on the same bind
                }            
            }
            catch(CORBA::Exception &e)
            {
            }
            catch(...)
            {
                std::cout << "Unknown exception caught." << std::endl;
            }
        } //if m_is_binded
    }
}

void BindReaderThread::terminate() throw()
{
    m_terminated = true;
}

void BindReaderThread::set_binded()
{
    m_is_binded = true;
}

}; // namespace Cdmw


