##################################################
## QTPDR requirements
##################################################
REP_UPD_TIM <- 10 # 10 msec

##################################################
## local definitions
##################################################
results <- "PTest_Q-429-2.txt"
warmup <- 10      # if warmup=N then the first N samples are filtered out (warm up stage)
threshold <- 1    # if threshold=N then N% of samples is filtered out

##################################################
## filter functions: exclude outliers and/or 
## warm up stage samples from the data set
##################################################
filter_warmup <- function(x) x[warmup:length(x)]
filter_top <- function(x) sort(x)[0:ceiling((length(x)*(100-(threshold%%100)))/100)]
filter <- function(x) filter_top(filter_warmup(x))


## main

write("****** PERFORMANCE TEST Q-429-2 ********", results)

## ACQUISITION OF DELTA_C.dat
DELTA_C<-scan("Q-429-2_DELTA_C.dat")

### DELTA_C FILTERING (note: time unit is msec)
DELTA_C_filtered<-filter(DELTA_C);

write("", results, append=TRUE)

##################################################
## calculate dispersion around mean value 
##################################################
write("results", results, append=TRUE)
write("------------", results, append=TRUE)
avg <- mean(DELTA_C_filtered)
write(ncolumns=3, c("AVG =", avg, " microseconds"), results, append=TRUE)
avg <- avg / 1000
write(ncolumns=3, c("    =", avg, " milliseconds"), results, append=TRUE)
write("", results, append=TRUE)

write(ncolumns=3, c("[REQUIREMENT] less than ", REP_UPD_TIM, " ms"), results, append=TRUE)
write(ncolumns=3, c("[ACHIEVEMENT] ", avg, " ms"), results, append=TRUE)
write("", results, append=TRUE)

##################################################
## write test OK/NOK to a file 
##################################################

write("########################################", results, append=TRUE)

if (avg <= REP_UPD_TIM) {
    write("## The qualification scenario is: OK  ##", results, append=TRUE)
} else {
    write("## The qualification scenario is: NOK ##", results, append=TRUE)
}
write("########################################", results, append=TRUE)

##################################################
## display the file with SO primitive
##################################################
system("cat PTest_Q-429-2.txt")
