/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.qualification.perftest.testperfbind;

import cdmw.ossupport.OS;
import cdmw.testutils.CORBATestManager;
import cdmw.testutils.TestManager;

import org.omg.CORBA.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import org.omg.CosNaming.NamingContextPackage.InvalidName;
import com.thalesgroup.CdmwNamingAndRepository.Repository;

import Cdmw.*;

public class BindReaderThread extends Thread
{
    public static final String BIND_OBJECT_ID = "bindTestObject";

    private ORB orb;

    private boolean m_terminated;

    private boolean m_is_binded;

    private java.lang.Object m_parent_thread;

    public BindReaderThread(ORB orb, java.lang.Object parent)
    {    
        this.orb = orb;
        m_terminated = true;
        m_is_binded = false;
        m_parent_thread = parent;
    }

    public void terminate()
    {
        m_terminated = true;
    }

    public void set_binded()
    {
        m_is_binded = true;
    }

    public void run()
    {
        m_terminated = false;

        //get Repository
        Repository repository = cdmw.namingandrepository.RepositoryInterface.getRepository();

        // get the default root context
        NamingContext def_root_context = null;
        try
        {
            def_root_context = repository
                .resolve_root_context(com.thalesgroup.CdmwNamingAndRepository.DEFAULT_ROOT_CONTEXT.value);
        }
        catch(com.thalesgroup.CdmwNamingAndRepository.RepositoryPackage.NoRootContext e)
        {
            System.out.println(e);
        }

        org.omg.CosNaming.NameComponent[] nc_array = new org.omg.CosNaming.NameComponent[1];
        nc_array[0] = new org.omg.CosNaming.NameComponent(BIND_OBJECT_ID, "object");

        //main thread loop
        System.out.println("BindReaderThread >> entering main loop");
        while(! m_terminated)
        {
            if(m_is_binded)
            {
                try
                {
                    org.omg.CORBA.Object obj = def_root_context.resolve(nc_array);
                    Cdmw.HelloInterface1 res_obj1 = Cdmw.HelloInterface1Helper.narrow(obj);
                    
                    String res = res_obj1.hello();
                    if (res.equals(BIND_OBJECT_ID))
                    {
                        m_is_binded = false; //to prevent multiple notifies on the same bind

                        synchronized(m_parent_thread)
                        {
                            m_parent_thread.notify();
                        }                        
                    }
                }
                catch(Exception e)
                {
                }
            } //if m_is_binded
            this.yield();
        }
        System.out.println("BindReaderThread >> leaving main loop");
    }
}

