/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.qualification.perftest.testperfbind;

import java.io.IOException;

import org.omg.CORBA.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import org.omg.CosNaming.NamingContextPackage.InvalidName;

import com.thalesgroup.CdmwNamingAndRepository.*;
import com.thalesgroup.CdmwNamingAndRepository.Repository;
import com.thalesgroup.CdmwPlatformMngt.ProcessDelegatePackage.NotReadyToRun;
import com.thalesgroup.CdmwPlatformMngtBase.StartupKind;

import cdmw.cdmwinit.ProcessControl;
import cdmw.common.BadOrderException;
import cdmw.common.BadParameterException;
import cdmw.orbsupport.CORBASystemExceptionWrapper;

import cdmw.qualification.perftest.testperfbind.BindReaderThread;
import cdmw.qualification.perftest.testperfbind.HelloInterface1_impl;


public class TestPerfBind extends ProcessControl
{
    public static final int N_ITERATION = 1000;

    public static final int BIND_TIMEOUT = 5000;

    private static short sync_scope;

    /**
     * The orb reference.
     */
    private ORB m_orb;


    public TestPerfBind(ORB orb)
    {
        this.m_orb = orb;
    }

    // process initialisation called by platformmngt
    public void onInitialise(StartupKind startupKind) 
    {
    }

    public void onRun() throws NotReadyToRun
    {
        try 
        {
            System.out.println("Getting Repository..");
            //get Repository
            Repository repository = cdmw.namingandrepository.RepositoryInterface.getRepository();

            // get the default root context
            NamingContext def_root_context = repository
                .resolve_root_context(com.thalesgroup.CdmwNamingAndRepository.DEFAULT_ROOT_CONTEXT.value);

            System.out.println("Starting BindReaderThread..");
            BindReaderThread bindReader = new BindReaderThread(m_orb, this);
            bindReader.start();

            HelloInterface1_impl   helloInterfaceImpl = new HelloInterface1_impl(BindReaderThread.BIND_OBJECT_ID);
            Cdmw.HelloInterface1   helloInterface = helloInterfaceImpl._this(m_orb);

            org.omg.CosNaming.NameComponent[] nc_array = new org.omg.CosNaming.NameComponent[1];
            nc_array[0] = new org.omg.CosNaming.NameComponent(BindReaderThread.BIND_OBJECT_ID, "object");

            long[] deltas = new long[N_ITERATION];

            long time1; 

            System.out.println("Starting the bind/unbind loop " + N_ITERATION);
            // start of Invocation test
            for (int i = 0; i < N_ITERATION; ++i )
            {
                time1 = System.nanoTime();

                try
                {
                    def_root_context.bind(nc_array, helloInterface);
                }
                catch(Exception e)
                {
                    System.out.println(e);
                }

                //Notify bindReader's thread that bind was done.
                bindReader.set_binded();

                synchronized(this)
                {
                    wait(BIND_TIMEOUT);
                }

                //microseconds 
                long elapsedTime = (System.nanoTime() - time1)/1000 ;

                try
                {
                    def_root_context.unbind(nc_array);
                }
                catch(Exception e)
                {
                    System.out.println(e);
                }

                //check whether it was timeout or notify();
                if(elapsedTime/1000 > BIND_TIMEOUT)
                {
                    //TIMEOUT
                    System.out.println("Bind " + i + " timedout. Delay " + elapsedTime + " microsec.");
                }

                deltas[i] = elapsedTime;

            } //for i < N_ITERATION
            
            //terminate bind reader thread and wait for it's finish
            System.out.println("Terminating BindReaderThread..");
            bindReader.terminate();
            bindReader.join();

            // write measured time data to file
            System.out.println("Writing data to file Q-429-2_DELTA_C.dat");
            java.io.FileWriter fd = new java.io.FileWriter("Q-429-2_DELTA_C.dat");

            for (int i = 0; i < N_ITERATION; ++i)
            {
                fd.write(String.format("%d\n", deltas[i]));
            }

            fd.close();
            System.out.println("Writing finished");
        }
        catch (UserException e)
        {
            e.printStackTrace();
        }
        catch (BAD_PARAM e)
        {
            e.printStackTrace();
        }
        catch (InterruptedException e)
        {
            e.printStackTrace();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        catch (Throwable e)
        {
            e.printStackTrace();
        }
    }

    public void onStop()
    {
        m_orb.shutdown(false);
    }
}
