###################################################
# crash time thresholds (from the QTPDR document)
###################################################
CRASH_DET_TIME1 <- 120
CRASH_DET_TIME2 <- 200
###################################################
## percentage thresholds (from the QTPDR document)
###################################################
PERCENT1 <- 95
PERCENT2 <- 99
###################################################

############## function definitions ###############

## return numember x with dec decimal
num_dec <- function(x,dec) x<- round(x*(10^dec))/(10^dec)

###################################################

### read process crash time
prc_time<-scan("Q-342_process.dat",list("",0,"",0));
### extract seconds from the table
prc_sec<-prc_time[[2]];
### extract usec from the table
prc_usec<-prc_time[[4]];

### read observer notification time
obs_time<-scan("Q-342_observer.dat",list("",0,"",0));
### extract seconds from the table
obs_sec<-obs_time[[2]];
### extract usec from the table
obs_usec<-obs_time[[4]];

### calculate elapsed seconds
elapsed_sec<-obs_sec-prc_sec;
### calculate elapsed microseconds
elapsed_usec<-obs_usec-prc_usec;
## total elapsed microseconds (10-6 sec)
elapsed_tot=(elapsed_sec*1000000)+(elapsed_usec)
## total number of element
len_tot = length(elapsed_tot)
## number of element with elapsed time <= CRASH_DET_TIME1 (in usec)
len_percent1 <- length(elapsed_tot[elapsed_tot <= (CRASH_DET_TIME1*1000)])
## number of element with elapsed time <= CRASH_DET_TIME1 (in usec)
len_percent2 <- length(elapsed_tot[elapsed_tot <= (CRASH_DET_TIME2*1000)])
## percentage of element with elapsed time <= CRASH_DET_TIME1
DET_PERCENT1 <- (len_percent1 / len_tot) * 100
## percentage of element with elapsed time <= CRASH_DET_TIME1
DET_PERCENT2 <- (len_percent2 / len_tot) * 100

## write results
results<-"test_summary.result.Q-342"
write("               Q-342 result",results);
write("-------------------------------------------",results,append=TRUE);
write(ncolumns=7,c("Percentage with notification time <=",
                   CRASH_DET_TIME1,
                   "ms:\n",
                   num_dec(DET_PERCENT1,1),
                   "%  (threshold:",
                   PERCENT1,
                   "%)"),
                   results,append=TRUE);
write(ncolumns=7,c("Percentage with notification time <=",
                   CRASH_DET_TIME2,
                   "ms:\n",
                   num_dec(DET_PERCENT2,1),
                   "%  (threshold:",
                   PERCENT2,
                   "%)"),
                   results,append=TRUE);

write("-------------------------------------------",results,append=TRUE);
##################################################
## write test OK/NOK to a file 
##################################################

if ((DET_PERCENT1 >= PERCENT1) && (DET_PERCENT2 >= PERCENT2)) {
    write("########################################", results, append=TRUE)
    write("## The qualification scenario is: OK  ##", results, append=TRUE)
    write("########################################", results, append=TRUE)
} else {
    write("########################################", results, append=TRUE)
    write("## The qualification scenario is: NOK ##", results, append=TRUE)
    write("########################################", results, append=TRUE)
}
##################################################
## display the file with SO primitive
##################################################
system("cat test_summary.result.Q-342")

