/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <time.h>
#include <math.h>
#include <sstream>
#include "qualif/perftests/testperfcommon/ParseCfg.hpp"
#include "Foundation/ossupport/OS.hpp"


const std::string FNAME_PFX = "--outfile";
using namespace std;

ostream& operator<<(ostream& os, const timespec& ts)
{
  double d = ts.tv_nsec;
  d=ceil(d/1000.0);

  return os << ts.tv_sec 
            << endl 
            << d;   // usec
}

int main(int argc, char **argv)
{
    std::cerr   << "-------------------------" << std::endl
                << "      Start Client" << std::endl
                << "-------------------------" << std::endl;
    for (int n=0; n < argc; n++)
        std::cerr << "argv[" << n <<"]: " << argv[n] << std::endl;
    std::cerr   << "-------------------------" << std::endl;

    std::string namePrefix = Cdmw::OsSupport::OS::get_option_value ( argc, argv, FNAME_PFX );

    if ( namePrefix == "no" ) {
        std::cerr << "Error: Option '"<< FNAME_PFX << "' is mandatory!\n";
        exit( 1 );
    }

  timespec ts;

  if(!clock_getres(CLOCK_REALTIME,&ts))
  {
    cout<<"CLOCK_REALTIME res is:\n" << ts << endl;
  }
  else
  {
    cerr << "FATAL: error in getting clock resolution" << endl;
    exit(1);
  }
  
  std::ostringstream nameStream;
  nameStream  << namePrefix << ".dat";
  std::string filename ( nameStream.str() );
  std::ofstream fp( filename.c_str() );
  fp << ts << endl;
  fp.close();
  return 0;
}
