/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <sstream>
#include <fstream>

#include <time.h>
#include <math.h>

#include "Foundation/ossupport/OS.hpp"

///////////////////////////////////////////////////////////////////////////////

const std::string FNAME_PFX = "--outfile";
const std::string NUM_ITER = "--iterations";
const int DEFAULT_ITER = 1000;
using namespace std;

///////////////////////////////////////////////////////////////////////////////

std::ostream& operator<< ( std::ostream& os, const timespec& ts )
{
    double d = ts.tv_nsec;
    d = ceil ( d / 1000.0 );

    return os << ts.tv_sec
           << std::endl
           << d;   // usec
}

///////////////////////////////////////////////////////////////////////////////

long long timeval2microsec ( const Cdmw::OsSupport::OS::Timeval& os_timeval )
{
    long long res = 0;

    res = ( os_timeval.seconds / 1000 ) + os_timeval.microseconds;
    return res;
}

///////////////////////////////////////////////////////////////////////////////

int main ( int argc, char **argv )
{
    std::cerr   << "-------------------------" << std::endl
    << "      Start Client" << std::endl
    << "-------------------------" << std::endl;

    for ( int n = 0; n < argc; n++ )
        std::cerr << "argv[" << n << "]: " << argv[n] << std::endl;

    std::cerr   << "-------------------------" << std::endl;

    std::string fileName = Cdmw::OsSupport::OS::get_option_value ( argc, argv, FNAME_PFX );

    if ( fileName == "no" ) {
        std::cerr << "Error: Option '" << FNAME_PFX << "' is mandatory!\n";
        exit ( 1 );
    }

    std::string iterStr = Cdmw::OsSupport::OS::get_option_value ( argc, argv, NUM_ITER );

    int iterations = DEFAULT_ITER;

    if ( iterStr == "no" ) {
        iterations = atoi ( iterStr.c_str() );
    }

    std::ostringstream nameStream;

    nameStream << fileName << "_" << iterations << "_DELTA_C.dat";

    std::string filedeltaname ( nameStream.str() );

    std::ofstream fd ( filedeltaname.c_str() );


    if ( !fd ) {
        std::cerr << "Error opening file: " << fileName << std::endl;
        exit ( 1 );
    }

    fd.precision ( 6 );

    long long val_from, val_to;

    for ( int n=0; n < iterations; n++ ) {
        val_from = timeval2microsec ( Cdmw::OsSupport::OS::get_time() );
        val_to = timeval2microsec ( Cdmw::OsSupport::OS::get_time() );
        // Foundation time difference
        fd << val_to - val_from << std::endl;
    }

    fd.close();

    return 0;
}
