/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @author Raffaele Mele <rmele@progesi.it>
 */

import cdmw.ossupport.OS;
import cdmw.ossupport.Timeval;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.text.DecimalFormat;
import java.io.IOException;
import java.io.PrintStream;
import java.lang.Math;

public class client_Q331
{

    private static final String niter_option = "--iterations";
    private static final String file_option = "--outfile";
    private static final int DEFAULT_NITER = 1000;
    private static final int ERROR = -1;

    public static void main(String[] args)
    {
        int niter;
        String filePrefix = new String();

        for (int n=0; n < args.length; n++)
            System.out.println("args["+n+"]: " + args[n]);

        try
        {
            filePrefix = OS.getOptionValue( args, file_option);
            if (filePrefix.equals("no"))
            {
                System.err.println("Error: output file name is undefined");
                System.exit (ERROR);
            }

            niter = Integer.valueOf(OS.getOptionValue( args, niter_option)).intValue();
            if (niter <= 0)
            {
                niter = DEFAULT_NITER;
            }
        }
        catch (NumberFormatException e)
        {
            niter = DEFAULT_NITER;
        }

        long[] time_delta = new long[niter];
        long time_pre, time_post;

            System.out.println("***> start " + niter +" invocations");
            for (int i = 0; i < niter; i++) {
                time_pre = timeval2microseconds( OS.getTime() );
                time_post = timeval2microseconds( OS.getTime() );
                time_delta[i] = (time_post - time_pre);
            }

        System.out.println("***> writing results");
        String result_filename = filePrefix + "_" + niter + "_DELTA_C.dat";
        try {
            writeResultsToFile(result_filename,time_delta);
        } catch (FileNotFoundException e) {
            System.err.println("Error while opening the output file " + result_filename);
        } catch (IOException e) {
            System.err.println("Generic io error while writing on " + result_filename + " file");
        }
    }

    private static long timeval2microseconds( Timeval val )
    {
        return ( (val.getMilliseconds()/1000) + (long)val.getMicroseconds());
    }

    /**
    * writeResultsToFile: write the delays on the a file
    * @param filename: file which will contain the delays
    * @throws FileNotFoundException
    * @throws IOException
    */
    private static void writeResultsToFile(String filename, long [] data ) throws FileNotFoundException, IOException
    {
        FileOutputStream fos = new FileOutputStream(filename);
        PrintStream ps = new PrintStream(fos);
        for (int i = 0; i < data.length; i++) {
            ps.println(data[i]);
        }
        ps.close();
        fos.close();
    }

    /**
     * computeStatistics: computes the average and the std of the delays
     *
     */
    private static void computeStatistics(long [] values){
        double average = 0.0;
        double std = 0.0;

        for (int i = 0; i < values.length; i++) {
            average += values[i];
        }

        // Compute average and standard deviation
        average = average / values.length;

        for (int i = 0; i < values.length; i++) {
            std += Math.pow(values[i] - average, 2.0);
        }
        std = (1.0 / values.length) * Math.sqrt(std);
        DecimalFormat df = new DecimalFormat("####.00");
        System.out.println("Average: " + df.format((average/1000)) + " usec");
        System.out.println("Standard deviation: " + df.format((std/1000)) + " usec");
    }

}
