/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @author Raffaele Mele <rmele@progesi.it>
 */

package javaclock;

import cdmw.common.BadParameterException;
import cdmw.orbsupport.ORBSupport;
import cdmw.orbsupport.StrategyList;
import cdmw.ossupport.OS;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.lang.Math;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Timer;

import org.omg.CORBA.ORB;
import org.omg.CORBA.ORBPackage.InvalidName;
import org.omg.CORBA.SystemException;
import org.omg.CosClockService.*;
import org.omg.CosClockService.ClockCatalogPackage.ClockEntry;
import org.omg.CosClockService.ClockCatalogPackage.UnknownEntry;
import org.omg.CosClockService.ControlledClockHelper;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;
import org.omg.PortableServer.POAManager;
import org.omg.PortableServer.POAManagerPackage.AdapterInactive;


class ORBWorker extends Thread
{
    private ORB m_ORB;

    public ORBWorker(ORB orb)
    {
        m_ORB = orb;
    }

    public void run()
    {
        try
        {
            m_ORB.run();
        }
        catch (SystemException ex)
        {
            ex.printStackTrace();
        }
    }
}

public class server_Q330
{

    private static final String niter_option = "--iterations";
    private static final String delay_option = "--delay";
    private static final String server_option = "--server";
    private static final String IORfile = "--IORfile";

    private static final int DEFAULT_NITER = 2000;
    private static final int DEFAULT_DELAY = 0;
    
    private static ORB m_orb;
    private static int niter;
    private static int delay;
    private static String file_name;
    private static boolean server_mode = false;
    private static org.omg.CosClockService.Clock m_clock;

    public static void main(String[] args)
    {
        StrategyList strategyList = new StrategyList();
        strategyList.addORBThreaded();
        try
        {
            strategyList.addPOAThreadPerConnection();
        }
        catch (BadParameterException e)
        {
            e.printStackTrace();
            System.exit(-1);
        }

        try {
            String tmp = OS.getOptionValue( args, server_option);
            if (tmp.compareTo("yes") == 0) {
                server_mode = true;
            }
        }

        catch (NumberFormatException e) 
        {
            server_mode = false;
        }

        try 
        {
            niter = Integer.valueOf(OS.getOptionValue( args, niter_option)).intValue();
            if (niter <= 0) 
            {
                niter = DEFAULT_NITER;
            }
        }
        catch (NumberFormatException e) {
            niter = DEFAULT_NITER;
        }
        
        try 
        {
            delay = Integer.valueOf(OS.getOptionValue( args, delay_option)).intValue();
            if (delay < 0) 
            {
                delay = DEFAULT_DELAY;
            }
        }
        catch (NumberFormatException e) {
            delay = DEFAULT_DELAY;
        }
        
        String file_name = OS.getOptionValue( args, IORfile);
           if (file_name.compareTo("no") == 0) {
               file_name = "server_Q330.IOR";
           }
        System.out.println("***> IOR File: " + file_name);
        POAManager poaManager = null;
        org.omg.CORBA.Object obj = null;

        System.out.println("***> ORB Init");
        m_orb =  org.omg.CORBA.ORB.init(args, null);
        //m_orb = cdmw.orbsupport.ORBSupport.ORBInit(args, null);
        cdmw.clock.CdmwCosClock.init(m_orb,args);


        try 
      {
        obj = m_orb.resolve_initial_references("LocalClock");
        m_clock =  ClockHelper.narrow(obj);
        if (m_clock == null)
            {
                System.out.println("Error : Unable to resolve LocalClock");
                System.exit(1);
            }
      } 
      catch (InvalidName ex)
      {
        ex.printStackTrace();
      }
      catch (org.omg.CORBA.BAD_PARAM ex)
      {
        ex.printStackTrace();
      }
        try
        {
            obj = m_orb.resolve_initial_references("RootPOA");
            POA root = POAHelper.narrow(obj);
            poaManager = root.the_POAManager();
            poaManager.activate();
        }
        catch (AdapterInactive ai)
        {
            ai.printStackTrace();
        }
        catch (InvalidName in)
        {
            in.printStackTrace();
        }

        String ref_string = m_orb.object_to_string(m_clock);
        try
        {

            PrintWriter out = new PrintWriter(new FileWriter(file_name));
            out.println(ref_string);
            out.flush();
            out.close();

        }
        catch (java.io.FileNotFoundException fnfe)
        {
            System.out.println(fnfe);
        }
        catch (java.io.IOException ioe)
        {
            System.out.println(ioe);
        }

        ORBWorker orb_worker = null;
        if (server_mode) 
        {
            m_orb.run();
            try
            {
                OS.sleep(2000);
            }
            catch (InterruptedException e)
            {
                e.printStackTrace();
                System.exit(-1);
            }
            m_orb.destroy();
            return;
        }
        else 
        {
            orb_worker = new ORBWorker(m_orb);
            orb_worker.start();
        }

        long time_before, time_after;
        long[] time_delta = new long[niter];

        try {

            for (int i = 0; i < niter; i++) {
                // Measure the orb-call delay in nanoseconds
                time_before = System.nanoTime();
                m_clock.current_time();
                time_after = System.nanoTime();

                time_delta[i] = (time_after - time_before);
                if (delay>0)
                    Thread.sleep(delay);
            }
        } catch (InterruptedException e) {
            System.err.println("Thread was interrupted");
            e.printStackTrace();
        } catch (TimeUnavailable e) {
            System.err.println("Thread was interrupted");
            e.printStackTrace();
        }

        String result_filename = "Q_330_sr_" + delay + "_" + niter + ".dat";
        try {
            writeResultsToFile(result_filename,time_delta);
        } catch (FileNotFoundException e) {
            System.err.println("Error while opening the output file " + result_filename);
        } catch (IOException e) {
            System.err.println("Generic io error while writing on " + result_filename + " file");
        }

        computeStatistics(time_delta);
        if (!server_mode) orb_worker.stop();
        cdmw.clock.CdmwCosClock.close();
        m_orb.destroy();
    }

    /**
    * writeResultsToFile: write the delays on the a file
    * @param filename: file which will contain the delays
    * @throws FileNotFoundException
    * @throws IOException
    */
    private static void writeResultsToFile(String filename, long [] data ) throws FileNotFoundException, IOException
    {
        FileOutputStream fos = new FileOutputStream(filename);
        PrintStream ps = new PrintStream(fos);
        for (int i = 0; i < data.length; i++) {
            ps.println(data[i]);
        }
        ps.close();
        fos.close();
    }


    /**
     * computeStatistics: computes the average and the std of the delays
     *
     */
    private static void computeStatistics(long [] values){
        double average = 0.0;
        double std = 0.0;

        for (int i = 0; i < values.length; i++) {
            average += values[i];
        }

        // Compute average and standard deviation
        average = average / values.length;

        for (int i = 0; i < values.length; i++) {
            std += Math.pow(values[i] - average, 2.0);
        }
        std = (1.0 / values.length) * Math.sqrt(std);
        DecimalFormat df = new DecimalFormat("####.00");
        System.out.println("Average: " + df.format((average/1000)) + " usec");
        System.out.println("Standard deviation: " + df.format((std/1000)) + " usec");
    }

}
