##################################################
## QTPDR requirements
##################################################
DEL_TIME <- 10

##################################################
## filter functions: exclude outliers and/or 
## warm up stage samples from the data set
##################################################
filter_warmup <- function(x) x[warmup:length(x)]
filter_top <- function(x) sort(x)[0:ceiling((length(x)*(100-(threshold%%100)))/100)]
filter_down <- function(x) sort(x)[ceiling((length(x)*(threshold%%100))/100):length(x)]
filter_top_and_down <- function(x) sort(x)[ceiling((length(x)*(threshold%%100))/100):floor((length(x)*(100-(threshold%%100)))/100)]
filter <- function(x) filter_top_and_down(filter_warmup(x))

##################################################
## local definitions
##################################################
results <- "test_summary.result.Q-334"
warmup <- 10        # if warmup=N then the first N samples are filtered out (warm up stage)
threshold <- 0.5    # if threshold=N then 2N% of samples is filtered out (with the "filter_top_and_down" function)

###ACQUISITION OF DELTA_C.dat
DC_CR_10<-scan("Q-334_TR_10_DELTA_C.dat")
DC_CR_50<-scan("Q-334_TR_50_DELTA_C.dat")
DC_CR_100<-scan("Q-334_TR_100_DELTA_C.dat")
DC_CR_500<-scan("Q-334_TR_500_DELTA_C.dat")

write("******** PERFORMANCE TEST Q-334 ******", results)
write("*** please note:  unit is (1 msec) ***", results, append=TRUE)

### DELTA_C FILTERING
DC_CR_10f<-filter(DC_CR_10/1000);
DC_CR_50f<-filter(DC_CR_50/1000);
DC_CR_100f<-filter(DC_CR_100/1000);
DC_CR_500f<-filter(DC_CR_500/1000);


write("", results, append=TRUE)
write("CR-10 results", results, append=TRUE)
write("-------------", results, append=TRUE)
T <- 10
DEL10 <- min(T, DEL_TIME/2)
RES_CR_10 <- max(abs(DC_CR_10f-T))
write(ncolumns=2, c("max|Ts-T| =", RES_CR_10), results, append=TRUE)
write("", results, append=TRUE)
write("CR-50 results", results, append=TRUE)
write("-------------", results, append=TRUE)
T <- 50
DEL50 <- min(T, DEL_TIME/2)
RES_CR_50 <- max(abs(DC_CR_50f-T))
write(ncolumns=2, c("max|Ts-T| =", RES_CR_50), results, append=TRUE)
write("", results, append=TRUE)
write("CR-100 results", results, append=TRUE)
write("--------------", results, append=TRUE)
T <- 100
DEL100 <- min(T, DEL_TIME/2)
RES_CR_100 <- max(abs(DC_CR_100f-T))
write(ncolumns=2, c("max|Ts-T| =", RES_CR_100), results, append=TRUE)
write("", results, append=TRUE)
write("CR-500 results", results, append=TRUE)
write("--------------", results, append=TRUE)
T <- 500
DEL500 <- min(T, DEL_TIME/2)
RES_CR_500 <- max(abs(DC_CR_500f-T))
write(ncolumns=2, c("max|Ts-T| =", RES_CR_500), results, append=TRUE)
write("", results, append=TRUE)

if ((RES_CR_10 <= DEL10) && (RES_CR_50 <= DEL50) && (RES_CR_100 <= DEL100) && (RES_CR_500 <= DEL500)) {
    write("########################################", results, append=TRUE)
    write("## The qualification scenario is: OK  ##", results, append=TRUE)
    write("########################################", results, append=TRUE)
} else {
    write("########################################", results, append=TRUE)
    write("## The qualification scenario is: NOK ##", results, append=TRUE)
    write("########################################", results, append=TRUE)
}
##################################################
## display the file with SO primitive
##################################################
system("cat test_summary.result.Q-334")
