##################################################
## QTPDR requirements
##################################################
DEL_TIME <- 10
DEL_TIME2 <- (DEL_TIME/2)*1000 # usec

##################################################
## local definitions
##################################################
results <- "test_summary.result.Q-336"
warmup <- 2      # if warmup=N then the first N samples are filtered out (warm up stage)

##################################################
## filter functions: exclude outliers and/or 
## warm up stage samples from the data set
##################################################
filter_warmup <- function(x) x[warmup:length(x)]
filter_top <- function(x) sort(x)[0:ceiling((length(x)*(100-(threshold%%100)))/100)]
filter <- function(x) filter_top(filter_warmup(x))
num_dec <- function(x,dec) x<- round(x*(10^dec))/(10^dec)

## Acquisition of configuration file
client_cfg <- scan("client_Q-336.cfg")

## Acquisition of DELTA_C.dat
DATASET<-scan("Q-336_CR_50_DELTA_C.dat")

## exclude warm up stage samples from the data set
DS_f<-filter_warmup(DATASET);
length(DS_f)

## calculate difference from measures and required interval for timer expiration
diff_int <- DS_f - client_cfg[2]*1000

## calculate how many timer expires in advance
lt_zero <- diff_int[diff_int < 0]
num_lt_zero <- length(lt_zero)

## calculate maximum delay of timer expiration
max_delay <- max(diff_int)

write("******** PERFORMANCE TEST Q-336 ******", results)
write("", results, append=TRUE)
write("Results:", results, append=TRUE)
write("------------------------------------------------------------------------", results, append=TRUE)
write(ncolumns=2, c("Number of Timers expirated in advance:",num_lt_zero),results, append=TRUE)
write(ncolumns=3, c("Maximum delay of expiration:",max_delay,"usec"),results, append=TRUE)
write("------------------------------------------------------------------------", results, append=TRUE)

##################################################
## write test OK/NOK to a file 
##################################################

if ((num_lt_zero==0) && (max_delay <= DEL_TIME2)) {
    write("########################################", results, append=TRUE)
    write("## The qualification scenario is: OK  ##", results, append=TRUE)
    write("########################################", results, append=TRUE)
} else {
    write("########################################", results, append=TRUE)
    write("## The qualification scenario is: NOK ##", results, append=TRUE)
    write("########################################", results, append=TRUE)
}
##################################################
## display the file with SO primitive
##################################################
system("cat test_summary.result.Q-336")
