/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "test_timer_precision/Periodic.hpp"

using namespace std;

using namespace perfPeriodic;

myPeriodic::myPeriodic( std::valarray<long double> &va,
                        std::valarray<long double> &vb,
                        size_t &count,
                        IExecutor *ref )
        : m_t0( va ),
        m_t1( vb ),
        m_count( count ),
        m_ProcCtrlRef( ref )
{}

CORBA::Boolean myPeriodic::do_work( const CORBA::Any& par )
throw ( CORBA::SystemException )
{
    m_t1[ m_count++ ] = ctools::time::HighResClock::getTime().toMicrosec();

    if ( m_count == m_t1.size() ) { // no other sample to get
        // write file for script control
        writeRes();
        std::ofstream testeof( TEST_EOF );
        testeof.close();
        std::cerr << "** test execution successfull terminated: Please STOP the system!" << std::endl;
        return false;
    }

    m_ProcCtrlRef->startAt();
    return true;
}

void myPeriodic::writeRes()
{
#ifdef DEBUG_ON
    std::cerr << "myPeriodic::writeRes()..\n";
#endif

    //write file
    std::ostringstream nameStream;

    nameStream  << "Q-336_CR_50_DELTA_C.dat";
//                 "DAT_FILE_PREFIX 
//                 << ( m_deltaT / 10000 )
//                 << "_DELTA_C.dat";

    std::string filedeltaname ( nameStream.str() );


    std::ofstream fp( filedeltaname.c_str() );

    fp << std::ios_base::fixed ;

    for ( size_t n = 0; n < m_count; n++ ) {
        // write absolute deltas between consecutive waking ups.
        // Pay attention: delta is expressed in *** usec *** units (10^-6 sec).
        CDMW_ASSERT( m_t1[ n ] >= m_t0[ n ] );
        fp << m_t1[ n ] - m_t0[ n ] << std::endl;
    }

    fp.close();
}
