/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
#ifndef INCL_CLIENT_PROCESS_CONTROL_HPP
#define INCL_CLIENT_PROCESS_CONTROL_HPP

// Cardamom Platform include
#include "Foundation/common/Exception.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "Foundation/orbsupport/CORBA.hpp"

#include "ConfAndPlug/cdmwinit/InitUtils.hpp"
#include "ConfAndPlug/cdmwinit/ProcessControl.hpp"

#include "SystemMngt/platforminterface/PlatformInterface.hpp"
#include "Repository/repositoryinterface/RepositoryInterface.hpp"
#include "tao/Messaging/Messaging.h"

// OMG EVoT include
#include "Time/clockservice/CdmwCosClock.hpp"

// Std C++ include
#include <iostream>
#include <fstream>
#include <sstream>

#include "qualif/perftests/testperfcommon/HighResTimer.hpp"
#include "qualif/perftests/testperfcommon/HighResTime.hpp"
#include "qualif/perftests/testperfcommon/HighResClock.hpp"
#include "qualif/perftests/testperfcommon/ClockTime.hpp"
#include "qualif/perftests/testperfcommon/ParseCfg.hpp"

#define MEASURE_HR "TIME_HR" // cTools HighRes Time
#define MEASURE_TH "TIME_TH" // clock thread cpu time 
#define MEASURE_TK "TIME_TK" // cTools HighRes clock tick count

#define TEST_EOF "end_of_test.log"

namespace ControlledInvocation
{

class ClientProcessControl : public Cdmw::CdmwInit::ProcessControl
{
    
  public:

    ClientProcessControl (CORBA::ORB_ptr orb, utils::cfgData& cfg)
    throw(CORBA::SystemException);  

    ~ClientProcessControl()
        throw();
    
    /**
     * Purpose:
     * <p>
     * the behaviour for the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/initialise:1.0
     * operation
     */
    virtual  
    void on_initialise (const CdmwPlatformMngtBase::StartupKind& startup_kind)
        throw(CORBA::SystemException);
    
    
    /**
     * Purpose:
     * <p>
     * the behaviour for the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/run:1.0
     * operation
     */
    virtual 
    void on_run()
        throw(CdmwPlatformMngt::ProcessDelegate::NotReadyToRun, 
                  CORBA::SystemException);
    
    /**
     * Purpose:
     * <p>
     * the behaviour for the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/stop:1.0
     * operation
     */
    virtual 
    void on_stop()
        throw(CORBA::SystemException);

  private:
  
    /**
    * The orb reference.
    */
    CORBA::ORB_var m_orb;
    
    /**
    * The application name.
    */
    std::string m_applicationName;
      
    /**
    * The process name.
    */
    std::string m_processName;
    
    /**
    * Clock reference
    */
    CosClockService::ControlledClock_var m_controlledClock;

  int m_niter;
  int m_delay;
    std::string m_fileprefix;
    ClockTime *m_measure;
    utils::cfgData m_cfg;

}; // End class ClientProcessControl

}; // End namespace ControlledInvocation

#endif // INCL_CLIENT_PROCESS_CONTROL_HPP
