#!/bin/sh
# =====================================================================
# This file is part of CARDAMOM (R) which is jointly developed by THALES
# and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
# Copyright (C) THALES 2000-2003. All rights reserved.
# Copyright (C) THALES 2004-2005. All rights reserved.
# Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
# All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
# License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

# ===================================== #
#  @name R_analysis.sh
#  @brief execute data analysis with R.
# ===================================== #

function echoc(){
  local str=$*
  if [ ! "$str" == "" ]; then
    echo -e "\e[32;40m$str\e[0m"
  fi
}

function echos(){
  local str=$*
  if test "`echo -e xxx`" = "xxx"
  then
      echoc $str
  else
      echo $str
  fi
}

function usage(){
  cat >&2 <<EOF
* Usage:
  $SCRIPTNAME <R_script_name[.R]> <trigger_file> <time_step_10s>
* where:
  R_script_name[.R]: The name of R file.
  trigger_file.....: Waiting file for trigger before R execution.
  time_step_10s....: Waiting step (1 step = 10 seconds) before timeout.
EOF
}

## this shell script
SCRIPTNAME=`basename $0`
## Qualification tests name
TESTNAME=`basename $1 .R`
## R Script file name
R_FILENAME=$TESTNAME.R
if [ "$#" -lt 3 ]; then
  usage
  exit 1
fi
if [ ! -f $R_FILENAME ]; then
  echos "\n($SCRIPTNAME) ERROR: $R_FILENAME not found\n"
  usage
  exit 1
fi
## trigger file
EOF_FILE=$2
## number of step
EXEC_STEP=$3
EXEC_TIMEOUT=10
## default R options
R_ARGUMENT="--slave --no-save"
## R executable location (see SCOM for installation)
R_EXEC=`which R`
## test R path 
if [ ! -x $R_EXEC ]; then
    echos "($SCRIPTNAME) ERROR:"
    echo "$R_EXEC path not found!"
    exit 1
fi
## test argument
if [ -f $EOF_FILE ]; then
    echos "######################## INFO ########################"
    echos "($SCRIPTNAME) WARNING: trigger file \"$EOF_FILE\" already exist!"
    echos "######################################################"
    NEW_SUFFIX=`date -r $EOF_FILE | sed 's/ /_/g'`
    NEW_NAME="$EOF_FILE"_"$NEW_SUFFIX"
    echo "==> Copying file $EOF_FILE to $NEW_NAME"
    cp $EOF_FILE $NEW_NAME
fi
## waiting for a file or a timeout
istep=1
while [ ! -f "$EOF_FILE" ] && [ $istep -le $EXEC_STEP ];
do
  if [ $istep == 1 ]; then 
    echos "Waiting a while (timeout at $(($EXEC_STEP*$EXEC_TIMEOUT)) sec) for the execution of the scenario..."
  fi
  echos "Waiting step $istep of $EXEC_STEP ..."  
  sleep $EXEC_TIMEOUT;
  istep=$(($istep+1))
done
## test executions
if [ ! -f "$EOF_FILE" ] && [ $istep -ge $EXEC_STEP ]; then
  cat >&2 <<EOF
##############################
$SCRIPTNAME: TIMEOUT ERROR!
##############################
EOF
  exit 1
else
  ## execute R with argument
  $R_EXEC $R_ARGUMENT < $R_FILENAME
  echos "Data analysis and the execution scenario are terminated."
  echos "Hit a key when ready."
  ## move file in results directory
  TESTCPDIR=$TESTNAME-results
  mkdir -p $TESTCPDIR 2>/dev/null
  cp -f $TESTNAME\_*.dat $TESTCPDIR 2>/dev/null
  mv -f test_summary.result.$TESTNAME $TESTCPDIR/test_summary.result 2>/dev/null
  cp -f $R_FILENAME $TESTCPDIR 2>/dev/null
  exit 0
fi
