/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/*
 * @author Raffaele Mele <rmele@progesi.it>
*/

#include "GenSysInfo.hpp"

// local functions
void errorPrint( std::string );
bool readLines( const std::string , std::vector<std::string>& );


namespace
{
// local variables
const int MAXLEN = 500;
//const std::string ux_cpuinfo = "/proc/cpuinfo";
//const std::string ux_theadmax = "/proc/sys/kernel/threads-max";
}

/* ========================================================================= */

GenSysInfo* GenSysInfo::pinstance = 0; // initialize static pointer

/* ========================================================================= */

GenSysInfo* GenSysInfo::Instance ()
{
    if ( pinstance == 0 )    // is it the first call?
    {
        pinstance = new GenSysInfo; // create sole instance
    }

    return pinstance; // address of sole instance
}

/* ========================================================================= */

GenSysInfo::GenSysInfo()
{
    const std::string ux_cpuinfo = "/proc/cpuinfo";
    const std::string ux_theadmax = "/proc/sys/kernel/threads-max";
    std::vector <std::string> fileLines;
    long procNum = 0;
    double readValue = 0;

    if ( !readLines( ux_cpuinfo, fileLines ) )
        errorPrint( ux_cpuinfo );

    while ( !fileLines.empty() ) {
        if ( ( fileLines.back() ).find( "cpu MHz" ) != std::string::npos ) {
            procNum += 1;
            parseValue( fileLines.back(), readValue );
        }

        fileLines.pop_back();
    }

    PROC_NUM = procNum;
    CLOCK_FREQUENCY = static_cast<double>(readValue);
    CLOCK_PERIOD_NS = static_cast<double>(1000.0 / readValue);
    CLOCK_PERIOD_US = static_cast<double>(1.0 / readValue);

    if ( !readLines( ux_theadmax, fileLines ) )
        errorPrint( ux_theadmax );

    THREAD_MAX = atol( fileLines.back().c_str() );
}

/* ========================================================================= */

GenSysInfo::~GenSysInfo()
{
    delete pinstance;
}

/* ========================================================================= */

void errorPrint( std::string fname )
{
    std::cerr << "FATAL: " << std:: endl
    << "cTools inizialisation error!" << std::endl
    << "can't open file: " << fname << std::endl;
    exit( 1 );

}

/* ========================================================================= */

bool readLines( const std::string fileName, std::vector<std::string>& text )
{
    char line[ MAXLEN ];
    std::ifstream inputFile( fileName.c_str() );

    if ( !inputFile ) {
        return false;
    }

    while ( inputFile.getline( line, MAXLEN ) ) {
        text.push_back( line );
    }

    inputFile.close();
    return true;
}
