/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef CDMW_GENSYSINFO_HPP
#define CDMW_GENSYSINFO_HPP

#include <iostream>
#include <fstream>
#include <vector>
#include <iterator>
#include <sstream>

/**
 * @brief store into value the content of line string after the separator
 * @param line string formatted properly with separator
 * @param value output of value represented into string line
 * @param sepCh serarator character
 */

template <class T>
bool parseValue( const std::string& line, T& value, char sepCh = ':' )
{
    std::string::size_type pos = line.rfind ( sepCh );

    if ( pos == std::string::npos )
        return false; // separator character not found

    std::istringstream dataStream( line.substr( pos + 1 ) ); // the remainder of string

    dataStream >> value; // store the value

    return true;
}

/* ========================================================================= */

/**
 * @brief Storing information about machine CPU characteristics
 */

class GenSysInfo
{

public:
    static GenSysInfo* Instance();
    double CLOCK_FREQUENCY;
    double CLOCK_PERIOD_NS;
    double CLOCK_PERIOD_US;
    long PROC_NUM;
    long THREAD_MAX;

    static const long POW_3_TEN = 1000;
    static const long POW_6_TEN = 1000000;
    ~GenSysInfo();

protected:
    GenSysInfo();
    GenSysInfo( const GenSysInfo& );
    GenSysInfo& operator= ( const GenSysInfo& );

private:
    /**
     * @brief 
     * @param 
     * @param 
     */
    static GenSysInfo* pinstance;
};

#endif // CDMW_GENSYSINFO_HPP
