/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.qualification.perftest.testperfcommon;

import java.io.*;

public class ParseCfg {
    
    private static final int SUCCESS = 0;
    private static final int FAILURE = 1;

    /**
     * @brief parse performance tests configuration file.
     * @param filename name of the configuration file.
     * @param cfg contain the results of config file parsing.
     */
    public static int parseCfg(String fileName, CfgData cfg) {
        String buf = "";
        BufferedReader bufReader = null;
        try {
            FileReader fr = new FileReader(fileName);
            bufReader = new BufferedReader(fr);
            if (!bufReader.ready()) {
                System.err.println("Error: in reading " + fileName);
              return FAILURE; 
            }
        }     
        catch (FileNotFoundException ex) {
            ex.printStackTrace();
            return FAILURE; 
        }
        catch (IOException e) {
            e.printStackTrace();
            return FAILURE; 
        }
        
        try {    
            // number of iteration
            buf = bufReader.readLine();
            cfg.niter = Tokenizer.intToken( buf, null);
            // delay
            buf = bufReader.readLine();
            cfg.delay = Tokenizer.intToken( buf, null);
            // Invocation type
            buf = bufReader.readLine();
            cfg.inv_type = Tokenizer.stringToken( buf, null);
            // Data type
            buf = bufReader.readLine();
            cfg.data_type = Tokenizer.stringToken( buf, null);
            // Data size
            buf = bufReader.readLine();
            cfg.data_size = Tokenizer.intToken( buf, null);
           // Measure type
            buf = bufReader.readLine();
            cfg.measure_type = Tokenizer.stringToken( buf, null);
            // File prefix
            buf = bufReader.readLine();
            cfg.fileprefix = Tokenizer.stringToken( buf, null);
        }
        catch (Exception e) {
            System.err.println("Error: cfg data error in " + fileName);
            e.printStackTrace();
            return FAILURE;
        }

        return SUCCESS;
    }
}
