/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>
#include <Repository/idllib/CdmwNamingAndRepository.stub.hpp>

#include <testprocessblock/TestTimeoutRequest.hpp>
#include <testprocessblock/TestHello.stub.hpp>


#include <fstream>

namespace Cdmw 
{

namespace FT 
{
CPPUNIT_TEST_SUITE_REGISTRATION( TestTimeoutRequest );

// constructor
TestTimeoutRequest::TestTimeoutRequest()
{    
   
    m_orb =  
       CORBA::ORB::_duplicate( 
             Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());
    m_rootPOA = 
       PortableServer::POA::_duplicate( 
             Cdmw::TestUtils::CORBATestManager::instance()->get_POA());
    
    CORBA::Object_var obj4 = Cdmw::TestUtils::CORBATestManager::instance()->get_object("replication_manager");
    m_replication_manager = CdmwFT::ReplicationManager::_narrow(obj4.in());

    m_host1 = Cdmw::TestUtils::CORBATestManager::instance()->get_property ("host_name1");
    m_host2 = Cdmw::TestUtils::CORBATestManager::instance()->get_property ("host_name2");

}

// destructor
TestTimeoutRequest::~TestTimeoutRequest()
{
}

// do_tests
void TestTimeoutRequest::do_tests()
{

    int timescale = Cdmw::TestUtils::get_timescale();

    try 
    {
        Cdmw::OsSupport::OS::unlink("hello.stt");
        Cdmw::OsSupport::OS::unlink("hello.act");
        Cdmw::OsSupport::OS::unlink("hello.dct");
        Cdmw::OsSupport::OS::unlink("comput.stt");
        Cdmw::OsSupport::OS::unlink("comput.act");
        Cdmw::OsSupport::OS::unlink("comput.dct");
    }
    catch(...)
    {
    }


    std::cout << "// ###########################################################" << std::endl;
    TEST_INFO("client started");
    std::cout << "// ###########################################################" << std::endl;
    
    TEST_INFO("get repository");
    CdmwNamingAndRepository::Repository_var repository 
        = Cdmw::NamingAndRepository::RepositoryInterface::get_repository();
    
    TEST_INFO("init repository");
    Cdmw::NamingAndRepository::RepositoryInterface::init ("CDMW",
                                                          repository.in());
    
    TEST_INFO("get_domain_naming_interface dom1/dom2 ");
    Cdmw::CommonSvcs::Naming::NamingInterface ni =
    Cdmw::NamingAndRepository::RepositoryInterface::get_domain_naming_interface ("dom1/dom2");


    // Getting P11HelloInterface
    std::string full_name = "dom3/P11HelloInterface";

    CORBA::Object_var obj;
        
    try
    {
        TEST_INFO("resolve " << full_name);
        obj = ni.resolve (full_name);
        CPPUNIT_ASSERT(true);
    }
    catch(const CosNaming::NamingContext::NotFound& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch(const CosNaming::NamingContext::CannotProceed& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch(const CosNaming::NamingContext::InvalidName& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch (const CORBA::SystemException& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }

    CdmwFT::Test::HelloInterface_var helloP11 
        = CdmwFT::Test::HelloInterface::_narrow(obj.in());


    // Getting P21ComputInterface
    full_name = "dom3/P21ComputInterface"; 
    try
    {
        TEST_INFO("resolve " << full_name);
        obj = ni.resolve (full_name);
        CPPUNIT_ASSERT(true);
    }
    catch(const CosNaming::NamingContext::NotFound& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch(const CosNaming::NamingContext::CannotProceed& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch(const CosNaming::NamingContext::InvalidName& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch (const CORBA::SystemException& e)
    {
        std::cout<< e << std::endl;
        CPPUNIT_ASSERT(false);
    }
    
    CdmwFT::Test::ComputInterface_var computP21
      = CdmwFT::Test::ComputInterface::_narrow(obj.in());


    OsSupport::OS::sleep(timescale*1000);


    try
    {
        TEST_INFO("Calling hello on HelloInterface on P11...");
        helloP11->hello(computP21.in(), 100);
        TEST_INFO("hello on HelloInterface on P11 OK...");
        CPPUNIT_ASSERT(true);  
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);
    }
    
    try
    {
        TEST_INFO("Calling send_TRANSIENT_exception on HelloInterface on P11...");
        helloP11->send_TRANSIENT_exception();
        CPPUNIT_ASSERT(false);  
    }
    catch (const CORBA::TRANSIENT& e)
    {
        TEST_INFO("Expected CORBA::TRANSIENT Exception raised");
        CPPUNIT_ASSERT(true);
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("Unexpected CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);  
    }


    try
    {
        TEST_INFO("Calling hello on HelloInterface on P11 ...");
        helloP11->hello(computP21.in(), 100);
        TEST_INFO("hello on HelloInterface on P11 OK...");
        CPPUNIT_ASSERT(true);  
    }
    catch (const CORBA::TRANSIENT& e)
    {
        TEST_INFO("Unexpected CORBA::TRANSIENT Exception raised " << e);
        CPPUNIT_ASSERT(false);
    }
    catch (const CORBA::COMM_FAILURE& e)
    {
        TEST_INFO("Unexpected CORBA::COMM_FAILURE Exception raised");
        CPPUNIT_ASSERT(false);  
    }
    catch (const CORBA::SystemException& e)
    {
        TEST_INFO("CORBA System Exception raised " << e);
        CPPUNIT_ASSERT(false);
    }

    std::cout<< "\nVerifing watchdog machanism !\n\n";
     try
     {
         TEST_INFO("Calling hello on primary HelloInterface on P11 which take a long time to process...");
         struct timeval tv;
         struct timezone tz;
              gettimeofday(&tv,&tz);
              std::ofstream crash_obs("Q-153_detect.dat",std::ios::out|std::ios::app);
              std::cout<<"WRITING ON Q-153_detect.dat";
              if (!crash_obs) 
                std::cerr << "*** ERROR IN WRITING FILE Q-153_detect.dat ***" << std::endl;
              else
                std::cerr << "*** APPEND TO FILE Q-153_detect.dat ***" << std::endl;

              crash_obs <<"Sec " << tv.tv_sec <<" Usec " << tv.tv_usec <<std::endl;
              crash_obs.close();
        
         
         helloP11->hello(computP21.in(), 3000);
         CPPUNIT_ASSERT(false);
     }
     catch (const CORBA::TRANSIENT& e)
     {
         TEST_INFO("Expected CORBA::TRANSIENT Exception raised " << e);
         CPPUNIT_ASSERT(true);
     }
     catch (const CORBA::COMM_FAILURE& e)
     {
         TEST_INFO("Expected CORBA::COMM_FAILURE Exception raised");
         CPPUNIT_ASSERT(true);  
     }
     catch (const CORBA::SystemException& e)
     {
         TEST_INFO("Unexpected CORBA System Exception raised " << e);
         CPPUNIT_ASSERT(false);  
     }
     
    std::cout<<"Check P21 is dead, P11, P31 and P41 are alive"<<std::endl;
    
    


    try
    {
        Cdmw::OsSupport::OS::unlink("hello.stt");
        Cdmw::OsSupport::OS::unlink("hello.act");
        Cdmw::OsSupport::OS::unlink("hello.dct");
        Cdmw::OsSupport::OS::unlink("comput.stt");
    }
    catch (...)
    {
    }
    
    
}


} // end namespace FT

} // end namespace Cdmw

