/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_RECORDER_PROCESS_BEHAVIOUR_HPP 
#define INCL_RECORDER_PROCESS_BEHAVIOUR_HPP 

#include <SystemMngt/platforminterface/PlatformInterface.hpp>
#include <ConfAndPlug/cdmwinit/ProcessControl.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>
#include <Repository/idllib/CdmwNamingAndRepository.stub.hpp>
#include <recording_operations/Basic_Q_411.skel.hpp>
#include <string>
#include <sstream>
#include <iostream>
#include <fstream>
#include "recording_operations/MyProcessBehaviour.hpp"
#include "recording_operations/CdmwRecordingRecorder_impl.hpp"

/**
* Purpose:
* <p>
* The abstract process behaviour.
*/
class RecorderProcessBehaviour : virtual public MyProcessBehaviour
{

public:

    /**
    * Purpose:
    * <p> The constructor.
    */ 
    RecorderProcessBehaviour  (CORBA::ORB_ptr orb, PortableServer::POA_ptr rootPOA, bool filter_activated)
      : MyProcessBehaviour (orb, rootPOA), m_filter_activated (filter_activated)
    {
       
    }


    /**
    * Purpose:
    * <p> The destructor.
    */ 
    virtual ~RecorderProcessBehaviour() throw ()
    {
     
    }

    /**
    * Purpose:
    * <p>
    * the behaviour for the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/initialise:1.0
    * operation
    */
    virtual void on_initialise(const CdmwPlatformMngtBase::StartupKind& startup_kind)
        throw( CORBA::SystemException)
    {
      Cdmw::OsSupport::OS::sleep(100);
        
      // get process name
      std::string process_name = Cdmw::PlatformMngt::PlatformInterface::Get_process_name();

      std::cout << "Process name = " << process_name << std::endl;

      // Get Recorder name domain from repository
      CdmwNamingAndRepository::Repository_var repository
        = Cdmw::NamingAndRepository::RepositoryInterface::get_repository();
      CdmwNamingAndRepository::NameDomain_var recorderDomain =
        repository->resolve_name_domain ("dom1/dom2/dom3");

      std::cout << "create Recorder_impl"<<std::endl;
      // create object recorder
      ::CdmwRecording::Recorder_impl * recorder_impl
          = new ::CdmwRecording::Recorder_impl (m_filter_activated);
      ::CdmwRecording::Recorder_var recorderInterface
          =  recorder_impl->_this();

      // Register the Server object in the Server name domain
      // reserve the name "Server" in the domain
      std::string full_name = process_name;

      std::cout << "full_name = " << full_name << std::endl;

      try
        {
          CdmwNamingAndRepository::NameDomain::RegistrationId_var regId = recorderDomain->new_name (full_name.c_str());
          // register the object
          recorderDomain->register_object (regId.in(), recorderInterface.in ());

        }catch (const CORBA::SystemException& e) {
          std::cerr << "Recorder: ERROR ------------------------"
                    << std::endl;
          std::cerr << e << std::endl;
        }
    }
    /**
    * Purpose:
    * <p>
    * the behaviour for the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/run:1.0
    * operation
    */
    virtual void on_run(const CdmwPlatformMngtBase::StartupKind& startup_kind)
        throw( CORBA::SystemException)
    {
//       CdmwRecording::Recorder_impl::start_test_thread();
    }

    /**
    * Purpose:
    * <p>
    * the behaviour for the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/stop:1.0
    * operation
    */
    virtual void on_stop() throw(CORBA::SystemException)
    {    
      CdmwRecording::Recorder_impl::check_test_result();
    }

    

private:
  bool m_filter_activated;

};


#endif // INCL_RECORDER_PROCESS_BEHAVIOUR_HPP

