/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_SERVER_PROCESS_BEHAVIOUR_HPP 
#define INCL_SERVER_PROCESS_BEHAVIOUR_HPP 

#include <SystemMngt/platforminterface/PlatformInterface.hpp>
#include <ConfAndPlug/cdmwinit/ProcessControl.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>
#include <Repository/idllib/CdmwNamingAndRepository.stub.hpp>
#include <recording_operations/Basic_Q_411.skel.hpp>
#include <string>
#include <sstream>
#include <iostream>
#include <fstream>
#include <Foundation/idllib/CdmwRecordingContainerManager.skel.hpp>
#include "recording_operations/MyProcessBehaviour.hpp"
#include "recording_operations/Server.hpp"

/**
* Purpose:
* <p>
* The abstract process behaviour.
*/
class ServerProcessBehaviour : virtual public ::MyProcessBehaviour
{

public:

    /**
    * Purpose:
    * <p> The constructor.
    */ 
    ServerProcessBehaviour  (CORBA::ORB_ptr orb, PortableServer::POA_ptr rootPOA):
      MyProcessBehaviour (orb, rootPOA)
    {
    }


    /**
    * Purpose:
    * <p> The destructor.
    */ 
    virtual ~ServerProcessBehaviour() throw ()
    {
    }

    /**
    * Purpose:
    * <p>
    * the behaviour for the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/initialise:1.0
    * operation
    */
    virtual void on_initialise(const CdmwPlatformMngtBase::StartupKind& startup_kind)
        throw( CORBA::SystemException)
    {
      // get process name
      std::string process_name = Cdmw::PlatformMngt::PlatformInterface::Get_process_name();

      std::cout << "Process name = " << process_name << std::endl;

      // Get Recorder name domain from repository
      CdmwNamingAndRepository::Repository_var repository
        = Cdmw::NamingAndRepository::RepositoryInterface::get_repository();
      CdmwNamingAndRepository::NameDomain_var recorderDomain =
        repository->resolve_name_domain ("dom1/dom2/dom3");

      std::cout << "create Server"<<std::endl;
      // create object Server
      BasicDemo::Server* server = new BasicDemo::Server();

      ::BasicDemo::Display_var display_interface =  server->_this();

      // Register the Server object in the Server name domain
      // reserve the name "Server" in the domain
      std::string full_name = process_name;

      std::cout << "full_name = " << full_name << std::endl;

      try
        {
          std::cout << "Server: Creation of a new NameDomain " << full_name << std::endl; 
          CdmwNamingAndRepository::NameDomain::RegistrationId_var regId = recorderDomain->new_name (full_name.c_str());
          // register the object
          std::cout << "Server: Registration of Server object in the new NameDomain " << std::endl; 
          recorderDomain->register_object (regId.in(), display_interface.in ());
          
          std::string nameserv_s = std::string ("corbaloc::localhost:21869/NameService" );
          CORBA::Object_var nameserv_obj = m_orb->string_to_object (nameserv_s.c_str());
          CosNaming::NamingContextExt_var naming_ctx = CosNaming::NamingContextExt::_narrow(nameserv_obj.in());
          if(CORBA::is_nil(naming_ctx.in()))
            {
              std::cerr << "Server: Naming Context not found!!! " << std::endl; 
            }

          ::Cdmw::CommonSvcs::Naming::NamingInterface ni(naming_ctx.in());
          CORBA::Object_var o_contMgr  = ni.resolve("RecordingDaemonContainerMgr");
          std::cout << "Server: ContainerManager Narrow " << std::endl; 
          CdmwRecording::ContainerManager_var contMgr = CdmwRecording::ContainerManager::_narrow(o_contMgr.in());

          if(CORBA::is_nil(contMgr.in()))
            std::cout << "Server: ## could not find RecordingDaemonContainerMgr" << std::endl; 
          else 
            {
              CdmwRecording::Object_List Obj_List  ;
              Obj_List.length(1);
              Obj_List[0] = display_interface._retn();
              
              contMgr->record_objects ("Server", Obj_List); 
              std::cout << "Server: record_objects - ok" << std::endl;
            }
          
        } catch (const CORBA::SystemException& e)
          {
            std::cerr << "Server: ERROR ------------------------"
                      << std::endl;
            std::cerr << e << std::endl;
          }

    }

};


#endif // INCL_SERVER_PROCESS_BEHAVIOUR_HPP

