/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <exception>
#include <iostream>
#include <sstream>


#include "recording_operations/Client.hpp"
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>
#include <Repository/idllib/CdmwNamingAndRepository.stub.hpp>
#include <Foundation/common/Assert.hpp>
#include "recording_operations/Basic_Q_411.skel.hpp"

namespace BasicDemo {

// constructor
Client::Client()
{
    
}

//destructor
Client::~Client()
{
}

void Client::run_client()
{

    try
    {
      std::cout << "---- Client: running ----" << std::endl;

      Cdmw::OsSupport::OS::sleep(7 * 1000); // Wait that the server finishes to register itself on the
                                             // repository

      // get NamingInterface 
      Cdmw::CommonSvcs::Naming::NamingInterface ni =
        Cdmw::NamingAndRepository::RepositoryInterface::get_domain_naming_interface ("dom1/dom2/dom3");

      std::string full_name = "P11";

      CORBA::Object_var obj;

      try
      {
        obj = ni.resolve (full_name);
        CDMW_ASSERT(true);
      }
      catch(const CosNaming::NamingContext::NotFound& e)
      {
        std::cerr<< e << std::endl;
        CDMW_ASSERT(false);
      }
      catch(const CosNaming::NamingContext::CannotProceed& e)
      {
        std::cerr << e << std::endl;
        CDMW_ASSERT(false);
      }
      catch(const CosNaming::NamingContext::InvalidName& e)
      {
        std::cerr << e << std::endl;
        CDMW_ASSERT(false);
      }
      catch (const CORBA::SystemException& e)
      {
        std::cerr << e << std::endl;
        CDMW_ASSERT(false);
      }

      ::BasicDemo::Display_var display_interface= ::BasicDemo::Display::_narrow(obj.in());
      CDMW_ASSERT( ! CORBA::is_nil(display_interface.in()) );

      ::CORBA::String_var str_in = "IN";
      ::CORBA::String_var str_inout = "IN"; // it will be INOUT after the processing of server
      ::CORBA::String_var str_out = ""; // it will be INOUT after processing
      ::CORBA::String_var str_rtn = ""; // it will be RETURN after processing
      str_rtn = display_interface->test_string(str_in.in(),
                                               str_inout.inout(),
                                               str_out.out());

      if (strcmp(str_rtn.in(),"RETURN") == 0) {
        std::cout <<  "Client : expected return value received" << std::endl;
      } else {
        std::cout <<  "Client : unexpected return value received!!!" << std::endl;
      }

      try {
        display_interface->test_exception();
      } catch(ExceptionTestRecording){
        std::cout <<  "Client : test_exception raised ExceptionTestRecording - OK" << std::endl;
      }

    }
    catch(const CORBA::Exception& e ) {
        std::cerr << e._name() << std::endl;
        CDMW_ASSERT(false);
    }
    catch( const Cdmw::Exception& e ) {
        std::cerr << e.what() << std::endl;
        CDMW_ASSERT(false);
    }
}

}; // namespace BasicDemo

