/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <ccm_context/CCM_Client_impl.hpp>
#include <ccm_context/SupervisionObserver_impl.hpp>

#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/logging/CdmwLogging.stub.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtManagedElementContexts.stub.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp>



namespace TestCcmContext
{
    const std::string SYSTEM_NAME = "S1";
    const std::string APPLICATION_NAME = "CcmTestComponentContextApplication";
    const std::string CLIENT_PROCESS_NAME = "TestComponentContextProcessClient1";
    const std::string SERVER_PROCESS1_NAME = "TestComponentContextProcessServer";
    const std::string SERVER_PROCESS2_NAME = "TestComponentContextProcessServer";
    const std::string SERVER_PROCESS3_NAME = "TestComponentContextProcessServer";
    const std::string CLIENT_COMPONENT_NAME = "Client1";
    const std::string SERVER_COMPONENT1_NAME = "Server1";
    const std::string SERVER_COMPONENT2_NAME = "Server2";
    const std::string SERVER_COMPONENT3_NAME = "Server3";
    const std::string CLIENT_ISSUER_NAME = "TestComponentContextProcessClient1/Client1";
    const std::string SERVER_ISSUER1_NAME = "TestComponentContextProcessServer/Server1";
    const std::string SUPERVISION_ISSUER_NAME = "CDMW.PFMMNG";
    const std::string CLIENT_NOTIFICATION_MESSAGE = "Notification message from Client";
    const CORBA::ULong CLIENT_USER_EVENT = 33;
    const std::string SERVER_NOTIFICATION_MESSAGE = "Notification message from Server";
    const std::string SERVER_NOTIFICATION_FATAL = "Fatal error from Server";
    const CORBA::ULong SERVER_USER_EVENT = 44;
    const CORBA::Long SERVER_EXIT_CODE = 10;




  const char* event_kind_to_string(CdmwPlatformMngt::Event* ev) {
        switch (ev->event_kind()) {
            case CdmwPlatformMngt::SYSTEM_MODE_CHANGE:
                return "SYSTEM_MODE_CHANGE";
            case CdmwPlatformMngt::SYSTEM_STATUS_CHANGE:
                return "SYSTEM_STATUS_CHANGE";
            case CdmwPlatformMngt::HOST_STATUS_CHANGE:
                return "HOST_STATUS_CHANGE";
            case CdmwPlatformMngt::APPLICATION_AGENT_DEATH:
                return "APPLICATION_AGENT_DEATH";
            case CdmwPlatformMngt::APPLICATION_MODE_CHANGE:
                return "APPLICATION_MODE_CHANGE";
            case CdmwPlatformMngt::APPLICATION_STATUS_CHANGE:
                return "APPLICATION_STATUS_CHANGE";
            case CdmwPlatformMngt::PROCESS_MESSAGE:
                return "PROCESS_MESSAGE";
            case CdmwPlatformMngt::PROCESS_STATUS_CHANGE:
                return "PROCESS_STATUS_CHANGE";
            case CdmwPlatformMngt::ENTITY_STATUS_CHANGE:
                return "ENTITY_STATUS_CHANGE";
            case CdmwPlatformMngt::CONFIGURATION_CHANGE:
                return "CONFIGURATION_CHANGE";
            case CdmwPlatformMngt::USER_EVENT:
                return "USER_EVENT";
            case CdmwPlatformMngt::PROPERTY_CHANGE:
                return "PROPERTY_CHANGE";
            case CdmwPlatformMngt::RESTART_REQUESTED:
                return "RESTART_REQUESTED";
            case CdmwPlatformMngt::REBOOT_REQUESTED:
                return "REBOOT_REQUESTED";
            case CdmwPlatformMngt::STOP_REQUESTED:
                return "STOP_REQUESTED";
            case CdmwPlatformMngt::COMPONENT_STATUS_CHANGE:
                return "COMPONENT_STATUS_CHANGE";
            case CdmwPlatformMngt::REDUNDANCY_GROUP_STATUS_CHANGE:
                return "REDUNDANCY_GROUP_STATUS_CHANGE";
            default:
                return "unknown event kind !";
        }
    }



//
// IDL:thalesgroup.com/CcmFtTest/ClientModule/CCM_Client:1.0
//
CCM_Client_impl::CCM_Client_impl()
{
}

CCM_Client_impl::~CCM_Client_impl()
{
    std::cerr << "   ... CCM_Client_impl::~CCM_Client_impl called!" << std::endl;
}


void CCM_Client_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    m_session_context = ::TestCcmContext::CCM_Client_Context::_narrow(ctx);
}
        
::TestCcmContext::CCM_Client_Context* CCM_Client_impl::get_context()
{
    return ::TestCcmContext::CCM_Client_Context::_duplicate(m_session_context.in());
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CCM_Client_impl::ccm_activate()
   throw(CORBA::SystemException)
{
    std::cerr << "   ... CCM_Client_impl::ccm_activate called!" << std::endl;

    // Get this component CORBA object
    // Note: this has to be performed in a CORBA request thread; thus
    //       not in test which is a specific thread.
    TestCcmContext::Client_var this_component =
        TestCcmContext::Client::_nil();
    try {
        // narrow context in SessionContext for get_CCM_object() access
        Components::SessionContext_var ctx =
            Components::SessionContext::_narrow(m_session_context.in());
        
        // call SessionContext::get_CCM_object()
        CORBA::Object_var obj =
            ctx->get_CCM_object();
            
        this_component =
            TestCcmContext::Client::_narrow(obj.in());
            
        CDMW_ASSERT(!CORBA::is_nil(this_component.in()));
            
    } catch (const CORBA::UserException& e) {
        std::cerr << "UserException during get_CCM_Object() :" 
                  << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
                  << std::endl;
        CDMW_ASSERT(false);
    } catch (const CORBA::SystemException& e) {
        std::cerr << "SystemException during get_CCM_Object() :" 
                  << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
                  << std::endl;
        CDMW_ASSERT(false);
    }


    try {   
        // save context in CORBATestManager for usage in test
        Cdmw::TestUtils::CORBATestManager::instance()->add(
            "client_context", 
            get_context());
    
        // save this component in CORBATestManager for usage in test
        Cdmw::TestUtils::CORBATestManager::instance()->add(
            "this_component", 
            this_component.in());
    
        // save this executor in CORBATestManager for usage in test
        Cdmw::TestUtils::CORBATestManager::instance()->add(
            "this_executor", 
            dynamic_cast< Session_CCM_Client_ptr > (this));
    
        // create test runner thread
        m_test_thread = new Cdmw::TestUtils::ThreadedTestRunner(
            Cdmw::TestUtils::CORBATestManager::instance()->get_controller());
        // add test to be run
        m_test_thread->addTest( 
            CppUnit::TestFactoryRegistry::getRegistry("test_ccm_context").makeTest());
        // start test
        m_test_thread->start();

    } catch (const CORBA::Exception& e) {
        std::cerr << "CORBA Exception exception during test: "
                  << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
                  << std::endl;
    } catch (const std::exception& e) {
        std::cerr << "std::exception exception during test: "
                  << e.what()
                  << std::endl;
    } catch (...) {
        std::cerr << "unknown exception during test !!" << std::endl;
    }
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
CCM_Client_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
    std::cerr << "   ... CCM_Client_impl::ccm_passivate called!" << std::endl;
    
    if (m_test_thread != 0) {
        // wait test to be finished
        m_test_thread->join();
        delete m_test_thread;
        m_test_thread = 0;
        // print test summary
        Cdmw::TestUtils::CORBATestManager::instance()->summary();
    }

} 


//
// ccm_remove function : it destroys the component (call by the container)
//
void
CCM_Client_impl::ccm_remove()
   throw(CORBA::SystemException)
{
    std::cerr << "   ... CCM_Client_impl::ccm_remove called!" << std::endl;
}


CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(test_ccm_context, "test_ccm_context");


test_ccm_context::test_ccm_context()
{
    // retrieve context object
    CORBA::Object_var obj = 
        Cdmw::TestUtils::CORBATestManager::instance()->get_object("client_context");
    m_session_context = 
        ::TestCcmContext::CCM_Client_Context::_narrow(obj.in());
        
    obj = 
        Cdmw::TestUtils::CORBATestManager::instance()->get_object("this_executor");
    m_ccm_client = dynamic_cast< CCM_Client_impl* > (obj.in());
    
    obj = 
        Cdmw::TestUtils::CORBATestManager::instance()->get_object("this_component");
    m_component_ref =
        TestCcmContext::Client::_narrow(obj.in());
}

void
test_ccm_context::do_tests()
{
    std::cerr << "   ... test_ccm_context::do_tests called!" << std::endl;
    
    
    try {

        TestComponentContext_var test_ccm_context =
            m_session_context->get_connection_test_server();

       Cdmw::OsSupport::OS::sleep(3000);
        
       TEST_INFO("Testing ManagedComponentContext");

       CORBA::String_var real_component_name;
       try {
        // get ManagedComponentContext
        CORBA::Object_var obj = 
            m_session_context->resolve_service_reference("ManagedComponentContext");
        CdmwPlatformMngt::ManagedElementContexts::ComponentContext_var component_context =
            CdmwPlatformMngt::ManagedElementContexts::ComponentContext::_narrow(obj.in());
        CPPUNIT_ASSERT(!CORBA::is_nil(component_context.in()));
     
        TEST_INFO("check supervised object names");
        // get component, process, application names
        CORBA::String_var component_name = component_context->managed_element_name();
        TEST_INFO(" component_name = '" 
                  << (component_name.in() == 0 ? "null" : component_name.in())
                  << "'");
        CPPUNIT_ASSERT(component_name.in() != 0);
        CPPUNIT_ASSERT(CLIENT_COMPONENT_NAME == component_name.in());        
        
        CORBA::String_var process_name = component_context->process_name();
        TEST_INFO("      process_name = '" 
                  << (process_name.in() == 0 ? "null" : process_name.in())
                  << "'");
        CPPUNIT_ASSERT(process_name.in() != 0);
        CPPUNIT_ASSERT(CLIENT_PROCESS_NAME == process_name.in());        

        CORBA::String_var appli_name = component_context->application_name();
        TEST_INFO("      appli_name = '" 
                 << (appli_name.in() == 0 ? "null" : appli_name.in())
                 << "'");
        CPPUNIT_ASSERT(appli_name.in() != 0);
        CPPUNIT_ASSERT(APPLICATION_NAME == appli_name.in());        

        // get System supervision object
        TEST_INFO("   get System supervision object");
        CdmwPlatformMngt::External::System_var system = component_context->the_system();

        // register this component as System observer
        TEST_INFO("   register this component as System Observer");
        CdmwPlatformMngt::SupervisionObserver_var old_one =
            system->register_observer("TestCcmContext_Client", 
                    m_component_ref.in());
        
        SupervisionObserver_impl* obsSystem = new SupervisionObserver_impl();
        std::string SysObsNameExt ="SystemObserverExt";
        CdmwPlatformMngt::SupervisionObserver_var systemObsExt = obsSystem->_this();
        CdmwPlatformMngt::RegistrationDepth depth=CdmwPlatformMngt::FULL_HIERARCHY;
        system->register_observer_ext(SysObsNameExt.c_str(),
                                       depth,
                                       systemObsExt.in());
 
        /*********************************************
        * Register observers for component and group 
        * status change notification
        ********************************************/
            
        //Getting Component supervision object
        CdmwPlatformMngt::System_var System = CdmwPlatformMngt::System::_narrow (system.in());
        
        TEST_INFO("   get Component supervision object");
        std::string remote_host_name = Cdmw::TestUtils::CORBATestManager::instance()->get_aux_info();
        
        CdmwPlatformMngt::CcmComponent_var component = 
            ((System->get_application(APPLICATION_NAME.c_str()))->
            get_process(SERVER_PROCESS2_NAME.c_str(), remote_host_name.c_str()))->
            get_component(SERVER_COMPONENT2_NAME.c_str());

        TEST_INFO(" Register_observer_ext() for Component :"<<SERVER_COMPONENT2_NAME);
        SupervisionObserver_impl* ObsComp1 = new SupervisionObserver_impl();
        CdmwPlatformMngt::SupervisionObserver_var compObsExt = ObsComp1->_this();
        depth=CdmwPlatformMngt::OBJECT_ONLY;
        std::string CompObsNameExt = "ComponentObserverExt";
        component->register_observer_ext(CompObsNameExt.c_str(),
                                          depth,
                                          compObsExt.in());
        TEST_INFO(" Register_observer() for Component :"<<SERVER_COMPONENT2_NAME);
        SupervisionObserver_impl* ObsComp2 = new SupervisionObserver_impl();
        CdmwPlatformMngt::SupervisionObserver_var compObs = ObsComp2->_this();
        std::string CompObsName = "ComponentObserver";
        component->register_observer(CompObsName.c_str(), compObs.in());

        
        // Getting RedundancyGroup supervision object
        TEST_INFO("   get RedundancyGroup supervision object");
        CdmwPlatformMngt::External::RedundancyGroup_var redgroup = test_ccm_context->get_server_group();
        CPPUNIT_ASSERT(!CORBA::is_nil(redgroup.in()));
        
        TEST_INFO(" Register_observer_ext() for RedundancyGroup :"<<redgroup->name());
        SupervisionObserver_impl* ObsGroup1 = new SupervisionObserver_impl();
        CdmwPlatformMngt::SupervisionObserver_var grObsExt = ObsGroup1->_this();
        depth=CdmwPlatformMngt::OBJECT_ONLY;
        std::string GrObsNameExt = "RedGroupObserverExt";
        redgroup->register_observer_ext(GrObsNameExt.c_str(),
                                       depth,
                                       grObsExt.in());
        
        TEST_INFO(" Register_observer() for RedundancyGroup :"<<redgroup->name());
        SupervisionObserver_impl* ObsGroup2 = new SupervisionObserver_impl();
        CdmwPlatformMngt::SupervisionObserver_var grObs = ObsGroup2->_this();
        std::string GrObsName = "RedGroupObserver";
        redgroup->register_observer(GrObsName.c_str(),
                                       grObs.in());





        
        // test System supervision object
        TEST_INFO("test System status");
        CORBA::String_var status_info;
        CPPUNIT_ASSERT(system->get_status(status_info.out()) == 
                       CdmwPlatformMngt::SYSTEM_STARTED);
                                      
        // test Application supervision object
        TEST_INFO("   test Application status");
        CdmwPlatformMngt::External::Application_var application =
            component_context->the_application();
        CPPUNIT_ASSERT(application->get_status(status_info.out()) == 
                       CdmwPlatformMngt::APPLICATION_RUNNING);
                                      
        // test Process supervision object
        TEST_INFO("   test Process status");
        CdmwPlatformMngt::External::Process_var process =
            component_context->the_process();
        CPPUNIT_ASSERT(process->get_status(status_info.out()) == 
                       CdmwPlatformMngt::PROCESS_RUNNING);
                                      
        // test Host supervision object
        TEST_INFO("   test Host status");
        CdmwPlatformMngt::External::Host_var host =
            component_context->the_host();
        TEST_INFO(host->status());
        CPPUNIT_ASSERT(host->status() == CdmwPlatformMngt::HOST_STARTED);                         
        
        
        // test RedundancyGroup supervision object
        TEST_INFO("   test RedundancyGroup status");
        //CdmwPlatformMngt::External::RedundancyGroup_var group = component_context->the_redundancy_group();
        CdmwPlatformMngt::External::RedundancyGroup_var group = test_ccm_context->get_server_group();
        CPPUNIT_ASSERT(!CORBA::is_nil(group.in()));

        //CPPUNIT_ASSERT(group->get_status(status_info.out()) == CdmwPlatformMngt::INITIAL);

        // test message notification to Supervision
        TEST_INFO("   notify message to supervision");
        component_context->notify_message(CdmwPlatformMngtBase::INF,
                                   CLIENT_NOTIFICATION_MESSAGE.c_str());
        Cdmw::OsSupport::OS::sleep(5000);
                                   
        TEST_INFO("      check that notification has been received back");
       
        CdmwPlatformMngt::Event_var event = m_ccm_client->get_last_event();
        CPPUNIT_ASSERT(event.in() != 0);
        CPPUNIT_ASSERT(event->event_kind() == CdmwPlatformMngt::PROCESS_MESSAGE);
        CdmwPlatformMngt::ProcessMessage* message_event =
            CdmwPlatformMngt::ProcessMessage::_downcast(event.in());
        CPPUNIT_ASSERT(CLIENT_ISSUER_NAME == message_event->issuer());
        CPPUNIT_ASSERT(CLIENT_NOTIFICATION_MESSAGE == message_event->info());
        
  
        // test user event notification to Supervision
        TEST_INFO("   notify user event to supervision");
        CORBA::Any_var any = new CORBA::Any();
        *any <<= CLIENT_USER_EVENT;
        component_context->notify_user_event(any.in());
        Cdmw::OsSupport::OS::sleep(10000);
                                   
        TEST_INFO("      check that notification has been received back");
        event = m_ccm_client->get_last_event();
        CPPUNIT_ASSERT(event.in() != 0);
        CPPUNIT_ASSERT(event->event_kind() == CdmwPlatformMngt::USER_EVENT);
        CdmwPlatformMngt::UserEvent* user_event =
            CdmwPlatformMngt::UserEvent::_downcast(event.in());
        CORBA::ULong value;
        if (user_event->data() >>= value) {
            CPPUNIT_ASSERT(value == CLIENT_USER_EVENT);
        } else {
            CPPUNIT_FAIL("UserEvent's data doesn't contain a CORBA::ULong");
        }
    
        CPPUNIT_ASSERT(CLIENT_ISSUER_NAME == user_event->issuer());
        CPPUNIT_ASSERT(APPLICATION_NAME == user_event->application_name());
        CPPUNIT_ASSERT(CLIENT_PROCESS_NAME == user_event->process_name());
        CPPUNIT_ASSERT(Cdmw::OsSupport::OS::get_hostname()  == user_event->host_name());

        
        TEST_INFO("testing group status is FULLY_REDUNDANT");   
        CPPUNIT_ASSERT(CdmwPlatformMngt::FULLY_REDUNDANT == group->get_status (status_info.out()));
        
          
        Cdmw::OsSupport::OS::sleep(5000);

        
        CdmwPlatformMngt::System_var system2 = CdmwPlatformMngt::System::_narrow(component_context->the_system());
        
        CdmwPlatformMngt::ElementNames_var elems;
        CdmwPlatformMngt::ElementNamesIterator_var it;

        // get number of hosts
        CORBA::ULong nbr_of_host = system2->get_number_of_hosts();
       
       // get the name of hosts
       system2->get_all_host_names(nbr_of_host, elems.out(), it.out());

       const char* host_name1 = elems[0U];

        CdmwPlatformMngt::CcmComponent_var component1 = 
            ((System->get_application(APPLICATION_NAME.c_str()))->
            get_process(SERVER_PROCESS1_NAME.c_str(), host_name1))->
            get_component(SERVER_COMPONENT1_NAME.c_str());
        
        CORBA::Any_var value_to_get =new CORBA::Any();
        CdmwPlatformMngt::RedundancyEltState property_value;
        CdmwPlatformMngt::PropertyChange_var property_change;

        component1->get_attribute (CdmwPlatformMngt::REPLICATION_ROLE,
                                  value_to_get.out(), 
                                  CdmwPlatformMngt::PRE_DEFINED);
                         
                                  
        value_to_get.in() >>= property_value;    
 
        TEST_INFO("checking if component1 is REDSTATE_PRIMARY");
        CPPUNIT_ASSERT(CdmwPlatformMngt::REDSTATE_PRIMARY == property_value);
        
        CdmwPlatformMngt::Process_var process1 = 
            (System->get_application(APPLICATION_NAME.c_str()))->
            get_process(SERVER_PROCESS1_NAME.c_str(), host_name1);
        
        process1->get_attribute(CdmwPlatformMngt::REPLICATION_ROLE, value_to_get.out(), CdmwPlatformMngt::PRE_DEFINED);
        value_to_get.in() >>= property_value;
        
        TEST_INFO("checking if process1 is REDSTATE_PRIMARY");
        CPPUNIT_ASSERT(CdmwPlatformMngt::REDSTATE_PRIMARY == property_value);  
        
 
        TEST_INFO("call test_operation on ServerGroup");       
        Cdmw::OsSupport::OS::sleep(10000);
             
        int number_of_requests = 1;

        while (number_of_requests < 4)
        {
        TEST_INFO("killing server" + (number_of_requests)); 
             try
             {
                  test_ccm_context->test_operation("ServerGroup");     
                  
             }
             catch(const CORBA::SystemException& ex)
             {
                   Cdmw::OsSupport::OS::sleep(10000);
                   if (1 == number_of_requests)
                   {
                        TEST_INFO("group status should be DEGRADED");
                        CPPUNIT_ASSERT(CdmwPlatformMngt::DEGRADED_REDUNDANCY == group->get_status (status_info.out ()));

                   }     
                   if (2 == number_of_requests)
                   {
                        TEST_INFO("group status should be LOST_REDUNDANCY");
                        CPPUNIT_ASSERT(CdmwPlatformMngt::LOST_REDUNDANCY == group->get_status (status_info.out ()));
                        
                   }
                   if (3 == number_of_requests)
                   {
                        TEST_INFO("group status should be COMPLETE_FAILURE");
                        CPPUNIT_ASSERT(CdmwPlatformMngt::COMPLETE_FAILURE == group->get_status (status_info.out ()));
                   }     
             }
             Cdmw::OsSupport::OS::sleep(10000);
             
            if (1 == number_of_requests){
             
                const char* host_name2 = elems[1U];


                CdmwPlatformMngt::CcmComponent_var component2 = 
                ((System->get_application(APPLICATION_NAME.c_str()))->
                get_process(SERVER_PROCESS2_NAME.c_str(), host_name2))->
                get_component(SERVER_COMPONENT2_NAME.c_str());

                component2->get_attribute (CdmwPlatformMngt::REPLICATION_ROLE,
                                  value_to_get.out(), 
                                  CdmwPlatformMngt::PRE_DEFINED);
                         
       
                value_to_get.in() >>= property_value;    
 
                TEST_INFO("checking if component2 is REDSTATE_PRIMARY");
                CPPUNIT_ASSERT(CdmwPlatformMngt::REDSTATE_PRIMARY == property_value);
          
                CdmwPlatformMngt::Process_var process2 = 
                    (System->get_application(APPLICATION_NAME.c_str()))->
                        get_process(SERVER_PROCESS2_NAME.c_str(), host_name2);
            

                process2->get_attribute(CdmwPlatformMngt::REPLICATION_ROLE, value_to_get.out(), CdmwPlatformMngt::PRE_DEFINED);
                value_to_get.in() >>= property_value;
        
                TEST_INFO("checking if process2 is REDSTATE_PRIMARY");
                CPPUNIT_ASSERT(CdmwPlatformMngt::REDSTATE_PRIMARY == property_value);  
             
             }
             
              if (2 == number_of_requests){
             
                  const char* host_name3 = elems[2U];

                  CdmwPlatformMngt::CcmComponent_var component3 = 
                      ((System->get_application(APPLICATION_NAME.c_str()))->
                          get_process(SERVER_PROCESS3_NAME.c_str(), host_name3))->
                              get_component(SERVER_COMPONENT3_NAME.c_str());

                  component3->get_attribute (CdmwPlatformMngt::REPLICATION_ROLE,
                                  value_to_get.out(), 
                                  CdmwPlatformMngt::PRE_DEFINED);
                         
       
                  value_to_get.in() >>= property_value;    
 
                  TEST_INFO("checking if component3 is REDSTATE_PRIMARY");
                  CPPUNIT_ASSERT(CdmwPlatformMngt::REDSTATE_PRIMARY == property_value);

        
                  CdmwPlatformMngt::Process_var process3 = 
                      (System->get_application(APPLICATION_NAME.c_str()))->
                          get_process(SERVER_PROCESS3_NAME.c_str(), host_name3);
            

                  process3->get_attribute(CdmwPlatformMngt::REPLICATION_ROLE, value_to_get.out(), CdmwPlatformMngt::PRE_DEFINED);
                  value_to_get.in() >>= property_value;
        
                  TEST_INFO("checking if process3 is REDSTATE_PRIMARY");
                  CPPUNIT_ASSERT(CdmwPlatformMngt::REDSTATE_PRIMARY == property_value);  
             
             }            
               
             number_of_requests++;
            
        }
        
        Cdmw::OsSupport::OS::sleep(15000);
        TEST_INFO("testing group status is COMPLETE_FAILURE");   
        CPPUNIT_ASSERT(CdmwPlatformMngt::COMPLETE_FAILURE == group->get_status (status_info.out()));
        
        
        // test Component Status Change 
        TEST_INFO(" test Component Status Change: ");
        while( ! ((obsSystem->is_component_test_ok()) && 
                         (ObsComp1->is_component_test_ok()) && 
                         (ObsComp2->is_component_test_ok())))
        {
            TEST_INFO(" Waiting ComponentStatusChange notification ...");
            sleep(5000);
        }
        CPPUNIT_ASSERT (true);

        TEST_INFO(" test Component Status Change: OK ");

        //test RedundancyGroup Status Change 
        TEST_INFO(" test RedundancyGroup Status Change: " );
        
        while( !((obsSystem->is_group_test_ok()) &&
                         (ObsGroup1->is_group_test_ok()) && 
                         (ObsGroup2->is_group_test_ok())))
        {
            TEST_INFO(" Waiting GroupStatusChange notification ...");
            sleep(5000);
        }
        CPPUNIT_ASSERT (true);
        TEST_INFO(" test RedundancyGroup Status Change: OK" );
        
        TEST_INFO("  Unregister Observers...");
        system->unregister_observer_ext(SysObsNameExt.c_str());
        redgroup->unregister_observer_ext(GrObsNameExt.c_str());
        component->unregister_observer_ext(CompObsNameExt.c_str());
        
        delete obsSystem;
        
           TEST_INFO("  Test is finished");
     
        } catch (const CORBA::SystemException& e) {
            TEST_INFO("Exception calling Server1's facet: " << 
                      Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            CPPUNIT_ASSERT(false);
             }
    } catch(CdmwPlatformMngt::ObserverNotFound)
    {
        std::cerr << "Exception Observer Not Found " << std::endl;
    }catch (...) {
        std::cerr << "unknown exception during test " << std::endl;
    }

}

    
void CCM_Client_impl::notify(CdmwPlatformMngt::Event* event)
        throw(CORBA::SystemException)
{

    CORBA::String_var msg = event->to_string();
    std::cout << "   ... " << event_kind_to_string(event)
              << " notification received from Supervision : \n"
              << msg.in() << std::endl;

    // save event in event queue
    event->_add_ref();
    m_event_queue.push(event);
    
}



//
// return first unconsumed event
//
CdmwPlatformMngt::Event*
CCM_Client_impl::get_last_event()
{
    // if no event in queue, return null
    if (m_event_queue.size() == 0)
        return 0;

    // get first event from event queue
    CdmwPlatformMngt::Event* event = m_event_queue.front();
    m_event_queue.pop();

    return event;
}


};  // namespace TestCcmContext

