/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <string>
#include "ccm_context/SupervisionObserver_impl.hpp"

#include "SystemMngt/platformvaluetypes/ComponentStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/RedundancyGroupStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/Event_impl.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtComponent.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtApplication.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtProcess.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp"


namespace TestCcmContext
{
        SupervisionObserver_impl::SupervisionObserver_impl()
        {

            std::string appl_name= "CcmTestComponentContextApplication";
            std::string proc_name= "TestComponentContextProcessServer";
            std::string comp_name= "Server2";
            std::string group_name= "ServerGroup";
            std::string remote_host_name = Cdmw::TestUtils::CORBATestManager::instance()->get_aux_info();
            std::string system_corbaloc = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";
            
            //Getting the system and component reference 
            CORBA::ORB_var orb;
            orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager:: instance ()->get_ORB ());
            CORBA::Object_var system_obj = orb->string_to_object (system_corbaloc.c_str());
            
            m_sys = CdmwPlatformMngt::System::_narrow (system_obj.in());
            CdmwPlatformMngt::Application_var appl= m_sys->get_application(appl_name.c_str());
            CdmwPlatformMngt::Process_var proc=appl->get_process(proc_name.c_str(),
                                                                 remote_host_name.c_str());
            m_comp = proc->get_component(comp_name.c_str());
            m_group = appl->get_ft_group(group_name.c_str());
            
            m_group_test=false;
            m_component_test=false;
        }

        void
        SupervisionObserver_impl::notify( CdmwPlatformMngt::Event* an_event )
        throw( CORBA::SystemException )
        {

            CORBA::String_var message = an_event->to_string();
            CORBA::String_var status_info = CORBA::string_dup ("");
            try 
            {

            if  ( an_event->event_kind() == CdmwPlatformMngt::COMPONENT_STATUS_CHANGE ) 
            {         
                    set_component_test_ok(true);
                    CdmwPlatformMngt::ComponentStatusChange* comp_chg = 
                        dynamic_cast<CdmwPlatformMngt::ComponentStatusChange*> (an_event);
                        
                        CORBA::String_var message = comp_chg->to_string();
                        TEST_INFO(" COMPONENT_STATUS_CHANGE NOTIFICATION : "<< message.in()<< "for component = "<<comp_chg->component_name()<<std::endl);
                        
                        if ( m_comp->get_status(status_info) == comp_chg->component_status() )
                        {
                            set_component_test_ok(true);
                        }

            }
            else if  ( an_event->event_kind() == CdmwPlatformMngt::REDUNDANCY_GROUP_STATUS_CHANGE  ) 
            {
                    set_group_test_ok(true);
                    CdmwPlatformMngt::RedundancyGroupStatusChange* grp_chg = 
                        dynamic_cast<CdmwPlatformMngt::RedundancyGroupStatusChange*> (an_event);
                        
                        CORBA::String_var message = grp_chg->to_string();
                        TEST_INFO(" GROUP_STATUS_CHANGE NOTIFICATION : "<< message.in()<< "for group = "<<grp_chg->group_name()<<std::endl);
                        if ( m_group->get_status(status_info) == grp_chg->group_status() )
                        {
                            set_group_test_ok(true);
                        }
                }
            }
            
            catch (const std::exception& ex)
            {
                std::cerr << " Process observer exception  " << ex.what();
            }
        }
            

} // End namespace 

