/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "starttime/DeploymentStarter.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include <iostream>

namespace
{
    const std::string SUPERVISION_CORBALOC = 
    "corbaloc::localhost:21871/CdmwPlatformMngtSupervision";
    const std::string NODEMANAGER_CORBALOC =
    "corbaloc::localhost:6677/CdmwNodeManager"; 
};

namespace Cdmw
{

    DeploymentStarter::DeploymentStarter(CORBA::ORB_ptr orb)
    : ProcessStarter("", ""),
      m_orb(CORBA::ORB::_duplicate(orb))
{         
}


void DeploymentStarter::start()
    throw (ProcessError)    
{

    try {
        // retrieve System object from Supervision
        CORBA::Object_var obj = m_orb->string_to_object(SUPERVISION_CORBALOC.c_str());
        m_system = CdmwPlatformMngt::System::_narrow(obj.in());
        
        CdmwPlatformMngtBase::StartupKind_var startupKind
            = new CdmwPlatformMngtBase::StartupKind();
        startupKind->state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
        startupKind->startup_mode = 0;
        startupKind->specified_state_id = CORBA::string_dup("");

        // start time
        m_start_time = Cdmw::OsSupport::OS::get_time();
        m_system->start(startupKind.in());

        //std::cout<<"start_time: "<<m_start_time.seconds<<"."<<m_start_time.microseconds<<std::endl;
        m_is_created = true;


    } catch (const CdmwPlatformMngt::IncompatibleStatus& e) {
        std::string msg = "Cannot initialise or run process (IncompatibleStatus !)";
        throw ProcessError(msg);
    } catch (const CdmwPlatformMngt::NotReadyToRun& e) {
        std::string msg = "Cannot run process (NotReadyToRun !)";
        throw ProcessError(msg);
    } catch (const CdmwPlatformMngt::IncompatibleType& e) {
        std::string msg = "Cannot run process (IncompatibleType !)";
        throw ProcessError(msg);
    } catch (const CORBA::SystemException& e) {
        std::string msg = "Error while contacting Supervision on ";
        msg += SUPERVISION_CORBALOC;
        throw ProcessError(msg);
    }

}

void DeploymentStarter::stop()
{
    if (!CORBA::is_nil(m_system.in())) {
        try {
            m_system->stop(false);
            m_is_started = false;

        } catch (const CORBA::SystemException& e) {
            std::cout << "Error while trying to stop system!" << std::endl;
        }
    }   
}
void DeploymentStarter::loop_until_stopped(unsigned int sleep_time)
{
    bool stopped = false ;

    while (!stopped) 
    {             
        try 
        {
            Cdmw::OsSupport::OS::sleep(sleep_time);
            CORBA::Object_var obj = m_orb->string_to_object("corbaloc::localhost:22456/NameService");
            CosNaming::NamingContextExt * ns = CosNaming::NamingContextExt::_narrow(obj);
            ns->resolve_str("foo");
           
        } catch (const Cdmw::OsSupport::InterruptedException& e) {
            // ignore
        } catch (const Cdmw::InternalErrorException& e) {
            // ignore
        } catch (CosNaming::NamingContext::NotFound ) {
            // ignore 
        } catch (CORBA::TRANSIENT) {
           //std::cout << "stopped" << std::endl;
           stopped = true;
        }                  
    }
}

bool DeploymentStarter::check_if_started() 
    throw (ProcessError)
{
    bool started = false;
   
    try 
    {        
        CORBA::Object_var obj = m_orb->string_to_object(NODEMANAGER_CORBALOC.c_str());
        CdmwDnC::NodeManager_var nodemngr = CdmwDnC::NodeManager::_narrow(obj.in());
        nodemngr->prepare_plan("");
    }
    catch (CdmwDnC::PlanError e)
    {
        set_as_started();
        started = true;
    }
    catch (const CORBA::SystemException& ex)
    {
        //ignore
    }
    catch (...)
    {
        std::cout << "unexpected exception" << std::endl;
    }
    return started ;    
}

}

