/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/Exception.hpp>
#include "SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtApplication.stub.hpp"
#include <FaultTolerance/ftreplicationmanager/InitUtils.hpp>
#include <FaultTolerance/idllib/FT.stub.hpp>
#include <SystemMngt/platformvaluetypes/PropertyChange_impl.hpp>
#include "Foundation/common/System.hpp"



using namespace Cdmw;
using namespace Common;
using namespace PlatformMngt;
using namespace CdmwPlatformMngt;
using namespace std;

int main( int argc, char* argv[] )
{
    std::cerr << " Main Started " << std::endl;

    CORBA::ValueFactoryBase_var factory;
    CdmwPlatformMngt::Application_ptr application_test  = 0;
    CdmwPlatformMngt::Process_ptr     process_test      = 0;

    try {

    Cdmw::OrbSupport::StrategyList orb_strategies;
    orb_strategies.add_OrbThreaded();
    orb_strategies.add_PoaThreadPool( 10 );

    CORBA::ORB_var orb;
    orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv,orb_strategies);

    CORBA::Object_var poa = orb->resolve_initial_references("RootPOA");
    PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(poa.in());

    // activate the POA manager
    PortableServer::POAManager_var manager =  PortableServer::POAManager::_duplicate(rootPOA->the_POAManager());
    manager->activate();

    CdmwPlatformMngt::System_var system_test; 
    /*Create an Instance to Client Observer */
            
     CORBA::Object_var objref;

     //define the corbaloc
     const char *corbaloc_str = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";
     objref = orb->string_to_object(corbaloc_str);
     //resolve the System_ptr
                    
     system_test = CdmwPlatformMngt::System::_narrow(objref.in() );


    if (CORBA::is_nil (system_test.in ()))
    {
            // TEST_INFO ("Error in narrowiPropertyChangeGenerator.cppng CdmwPlatformMngt::System");
            exit (-1);
    }

        CdmwPlatformMngt::Children_var children = new CdmwPlatformMngt::Children;
        children = system_test->get_children();
        CORBA::ULong size_of_children = children->length();

    // recoveer the Application APPL1
        application_test = system_test->get_application("APPL1");
        char * hostname  = getenv("HOSTNAME");
        process_test     = application_test->get_process("P11",hostname);

    // Prepare a Any value to set into define property
        CORBA::Any anyp_val;
        string sprop;
        int val=10;
        anyp_val <<= val;
          
        // begin a loop 
        try
        {
            for (CORBA::ULong j = 0; j< size_of_children; j++)
            {
                children[j]->define_property("Property1",anyp_val);
                sleep(1);
                children[j]->define_property("Property2",anyp_val);
                sleep(1);
                children[j]->define_property("Property3",anyp_val);
                sleep(1);
            }
            process_test->define_property("Property1",anyp_val);
            sleep(1);
            system_test ->define_property("Property1",anyp_val);
            sleep(1);
            process_test->define_property("Property2",anyp_val);
            sleep(1);
            system_test->define_property("Property2",anyp_val);
            sleep(1);         
            process_test->define_property("Property3",anyp_val);
            sleep(1);         
            system_test->define_property("Property3",anyp_val);
        sleep(1);
        }
        catch(const CORBA::SystemException & e)
        {
             // if exception catched recover Children object 
            std::cerr << e._name()<< std::endl;
            children = new CdmwPlatformMngt::Children;
            children = system_test->get_children();
            size_of_children = children->length();
            process_test     = application_test->get_process("P11",hostname);
        }
    
        cout << "RESTARTING APPL1" << std::endl;
        sleep(2);
        CdmwPlatformMngtBase::StartupKind_var startupKind
            = new CdmwPlatformMngtBase::StartupKind();
        startupKind->state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
        startupKind->startup_mode = 0;
        startupKind->specified_state_id = CORBA::string_dup("");
        application_test->cold_restart(startupKind.in());
        cout << "RESTARTING APPL1 DONE" << std::endl;
    sleep(5);
    
        cout << std::endl;
        while(true)
        {
          try
          {
            for (CORBA::ULong j = 0; j< size_of_children; j++)
            {
                children[j]->define_property("Property1",anyp_val);
                sleep(1);
                children[j]->define_property("Property2",anyp_val);
                sleep(1);
                children[j]->define_property("Property3",anyp_val);
                sleep(1);
            }
            process_test->define_property("Property1",anyp_val);
            sleep(1);
            system_test ->define_property("Property1",anyp_val);
            sleep(1);
            process_test->define_property("Property2",anyp_val);
            sleep(1);
            system_test->define_property("Property2",anyp_val);
            sleep(1);         
            process_test->define_property("Property3",anyp_val);
            sleep(1);         
            system_test->define_property("Property3",anyp_val);
        sleep(1);
          }
            catch(const CORBA::SystemException & e)
            {
                // if exception catched recover Children object 
                std::cerr << e._name()<< std::endl;
                children = new CdmwPlatformMngt::Children;
                children = system_test->get_children();
                size_of_children = children->length();
                process_test     = application_test->get_process("P11",hostname);
            }
        }
    }
    catch (const CORBA::Exception& eCBex)
    {
    cout << "CORBA Exception :" << eCBex._name() << endl;
    return -1;
    }
    CORBA::release(application_test);
    CORBA::release(process_test);

    cout << "Exit process..." << endl;
}
