/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/Exception.hpp>
#include "SystemMngt/idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtApplication.stub.hpp"
#include <SystemMngt/idllib/CdmwPlatformMngtSupervisionObserver.stub.hpp>
#include <FaultTolerance/ftreplicationmanager/InitUtils.hpp>
#include <FaultTolerance/idllib/FT.stub.hpp>

#include <SystemMngt/platformvaluetypes/PropertyChange_impl.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtSupervisionObserver.stub.hpp>
#include "Foundation/common/System.hpp"

#include "testFTpropertychange/SupervisionObserver_impl.hpp"





using namespace Cdmw;
using namespace Common;
using namespace PlatformMngt;
using namespace CdmwPlatformMngt;
using namespace std;

string REGISTER_OPTION = "--register";
string UNREGISTER_OPTION = "--unregister";
string SYSTEM_CORBALOC_OPTION = "--system_test-corbaloc";
string SYSTEM_CORBALOC = "--system-corbaloc";
string OBSERVER_PORT_OPTION = Options::LOCALISATION_SERVICE_OPTION;
string OBSERVER_NAME_OPTION = "--observer-name";
string OBSERVER_FOR_ALL     = "--observer-for-all";
string HOST_NAME     = "--host-name";

int main( int argc, char* argv[] )
{
    CORBA::ValueFactoryBase_var factory;
    CdmwPlatformMngt::Application_ptr application_test  = 0;
    CdmwPlatformMngt::Process_ptr     process_test      = 0;
    string observer_name, observer_port, observer_for_all, observer_unregister;


    try {
    // Check command line arguments
        if ( (argc != 5) && (argc != 6) )
        {
            cerr << "Usage: " << REGISTER_OPTION << " or " << UNREGISTER_OPTION << endl;
            cerr << "       " << SYSTEM_CORBALOC_OPTION << "=system_test_corbaloc" << endl;
            cerr << "       " << OBSERVER_NAME_OPTION << "=observer_name" << endl;
            cerr << "       " << OBSERVER_FOR_ALL     << "=observer-for-all" << endl;
            cerr << endl;
            cerr << "   Or: " << OBSERVER_PORT_OPTION << "=observer_port" << endl;
            cerr << "       " << OBSERVER_NAME_OPTION << "=observer_name" << endl;
            cerr << "       " << OBSERVER_FOR_ALL     << "=observer-for-all" << endl;
            cerr << endl;
            throw 0;
        }

    Cdmw::OrbSupport::StrategyList orb_strategies;
    orb_strategies.add_OrbThreaded();
    orb_strategies.add_PoaThreadPool( 10 );

    CORBA::ORB_var orb;
    orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv,orb_strategies);

    CORBA::Object_var poa = orb->resolve_initial_references("RootPOA");
            
    PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(poa.in());
    // activate the POA manager
    PortableServer::POAManager_var manager =  PortableServer::POAManager::_duplicate(rootPOA->the_POAManager());
                                    
    manager->activate();

    CORBA::ValueFactoryBase_var factory,old_factory;
    factory = new PropertyChangeFactory;

    old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/PropertyChange:1.0",factory.in() );

    CdmwPlatformMngt::System_var system_test; 
    /*Create an Instance to Client Observer */
            
    SupervisionObserver_impl obstoreg;
            
    SupervisionObserver_var observer = obstoreg._this();
    
    observer_unregister = OsSupport::OS::get_option_value( argc, argv, UNREGISTER_OPTION );
    observer_name       = OsSupport::OS::get_option_value(argc, argv, OBSERVER_NAME_OPTION );
    observer_port       = OsSupport::OS::get_option_value( argc, argv, OBSERVER_PORT_OPTION );
    observer_for_all    = OsSupport::OS::get_option_value( argc, argv, OBSERVER_FOR_ALL );
    std::string sys_corbaloc = OsSupport::OS::get_option_value( argc, argv, SYSTEM_CORBALOC );
    std::string host_name = OsSupport::OS::get_option_value( argc, argv, HOST_NAME );
                            
    
    std::cerr<<std::endl;
    std::cerr << "<<<<<      OBSERVER NAME: "<< observer_name << "                    >>>>>" << std::endl;
    if (observer_for_all == "yes")
    {
        if (observer_unregister == "no")
            std::cerr << "<<<<<   Property Observer Registered   >>>>>" << std::endl;
        else
            std::cerr << "<<<<<  Property Observer Unregistered  >>>>>" << std::endl;
        std::cerr << "<<<<<     for all Properties           >>>>>" << std::endl;
    }
    if (observer_for_all != "yes")
    {
        if (observer_unregister == "no")
            std::cerr << "<<<<<   Property Observer Registered   >>>>>" << std::endl;
        else
            std::cerr << "<<<<<  Property Observer Unegistered   >>>>>" << std::endl;
            std::cerr << "<<<<<     for Properties with names    >>>>>" << std::endl;
        if (observer_unregister == "no")
            std::cerr << "<<<<<      Property1  Property2        >>>>>" << std::endl;
        else
            std::cerr << "<<<<<            Property1             >>>>>" << std::endl;
    }
    std::cerr << "<<<<<                                  >>>>>" << std::endl;                            
                    


///////////////////


                    CORBA::Object_var objref;


                    //define the corbaloc
                    //const char *corbaloc_str = "corbaloc::localhost:21871/CdmwPlatformMngtSupervision";
                    std::cout << "Supervision: " << sys_corbaloc.c_str() << std::endl;
                    objref = orb->string_to_object(sys_corbaloc.c_str());


                    //resolve the System_ptr
                    system_test = CdmwPlatformMngt::System::_narrow(objref.in() );


///////////////////



    if (CORBA::is_nil (system_test.in ()))
    {
            // TEST_INFO ("Error in narrowing CdmwPlatformMngt::System");
            exit (-1);
    }
                                                                    
        /*select application supervised object */
        //char * hostname  = getenv("HOSTNAME");
        
        application_test = system_test->get_application("APPL1");
        process_test     = application_test->get_process("P11",host_name.c_str());
       
        CdmwPlatformMngt::Children_var children = new CdmwPlatformMngt::Children;
        children = system_test->get_children();
        CORBA::ULong size_of_children = children->length();
                                    
        CosPropertyService::PropertyNames property_names;
        property_names.length(2);
        property_names[0] = "Property1";
        property_names[1] = "Property2";

    if ( OsSupport::OS::get_option_value( argc, argv, REGISTER_OPTION ) == "yes" )
    {
        /*Register observer for all property*/
        for (CORBA::ULong i = 0; i< size_of_children; i++)
        {
            if (observer_for_all != "yes")
            {
                 cout << "register_property_observer..." << endl;

                children[i]->register_property_observer(observer_name.c_str(),observer.in(),property_names);
                
            }
            if (observer_for_all == "yes")
            {
                children[i]->register_all_properties_observer(observer_name.c_str(),observer.in(),false);
            }
        }
        if (observer_for_all != "yes")
            {
                cout << "process_test register_property_observer..." << endl;
                process_test->register_property_observer(observer_name.c_str(),observer.in(),property_names);
                cout << "system_test register_property_observer..." << endl;
                system_test->register_property_observer(observer_name.c_str(),observer.in(),property_names);
            }
        if (observer_for_all == "yes")
            {
                process_test->register_all_properties_observer(observer_name.c_str(),observer.in(),false); 
                system_test->register_all_properties_observer(observer_name.c_str(),observer.in(),false);
            }
    }

    if ( OsSupport::OS::get_option_value( argc, argv, UNREGISTER_OPTION ) == "yes" )
    {
        property_names.length(1);
        property_names[0] = "Property1";
        
        /*Register observer for all property*/
        for (CORBA::ULong i = 0; i< size_of_children; i++)
        {
            if (observer_for_all != "yes")
            {
             cout << "unregister_property_observer..." << endl;
                children[i]->unregister_property_observer(observer_name.c_str(), property_names);
            }
            if (observer_for_all == "yes")
            {
                children[i]->unregister_all_properties_observer(observer_name.c_str());
            }
        }
        if (observer_for_all != "yes")
            {
                cout << "process_test unregister_property_observer..." << endl;
                process_test->unregister_property_observer(observer_name.c_str(), property_names);
                cout << "system_test unregister_property_observer..." << endl;
                system_test->unregister_property_observer(observer_name.c_str(), property_names);
                cout << "END..." << endl;
            }
        if (observer_for_all == "yes")
            {
                process_test->unregister_all_properties_observer(observer_name.c_str());
                system_test-> unregister_all_properties_observer(observer_name.c_str());
            }
        }

        //orb run only if observer registration is required 
        if ( OsSupport::OS::get_option_value( argc, argv, REGISTER_OPTION ) == "yes" )        
            orb->run();

    }
    catch (const CORBA::Exception& eCBex)
    {
    cout << "CORBA Exception :" << eCBex._name() << endl;
    return -1;
    }

    cout << "Exit process..." << endl;




}
