/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "test_automatic_host_stop/MyProcessBehaviour.hpp"
#include <Foundation/orbsupport/OrbSupport.hpp>

#include "Foundation/testutils/CORBATestManager.hpp"

#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"

#include <Foundation/common/Options.hpp>
#include <Foundation/common/Exception.hpp>
#include <Foundation/common/Assert.hpp>
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"
#include "idllib/CdmwPlatformMngtHost.stub.hpp"

#include "Foundation/common/System.hpp"
#include "idllib/CdmwPlatformMngtDaemon.stub.hpp"


#include <iostream>

using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;


MyProcessBehaviour::MyProcessBehaviour(CORBA::ORB_ptr orb)
{
    m_orb = CORBA::ORB::_duplicate(orb);
}


MyProcessBehaviour::~MyProcessBehaviour() throw ()
{
}

void MyProcessBehaviour::initialise(const CdmwPlatformMngtBase::StartupKind& startup_kind)
        throw(CdmwPlatformMngt::ProcessDelegate::BadOrder, CORBA::SystemException)
{
    std::cerr << "******* initialise *******" << std::endl;
    std::cerr << "******* stop current host " << std::endl;

    CdmwPlatformMngt::ElementNames_var elems;
    CdmwPlatformMngt::ElementNamesIterator_var it;

    CdmwPlatformMngt::System_var system ;

    std::cerr << " Getting system var " << std::endl;
    std::string system_corbaloc = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";

    CORBA::Object_var system_obj = m_orb->string_to_object (system_corbaloc.c_str());

    if (CORBA::is_nil(system_obj)) 
    {
        std::cerr << " System_obj nil " << std::endl;
        CPPUNIT_ASSERT(false);
    }

    system = CdmwPlatformMngt::System::_narrow (system_obj.in ());

    if (CORBA::is_nil(system)) 
    {
       std::cerr << " System nil " << std::endl;
       CPPUNIT_ASSERT(false);
    }

    CORBA::ULong nbr_of_host;

    try {

          nbr_of_host = system->get_number_of_hosts();

     } catch (const CORBA::TRANSIENT& excTrans) {

          std::cerr << "  CORBA::TRANSIENT " << std::endl;
          std::cerr << "  CORBA::TRANSIENT: " << excTrans << std::endl;
          throw;

     } catch (const CORBA::SystemException& excS) {

          std::cerr << "  CORBA::SystemException: " << std::endl;
          std::cerr << "  Exception: "+std::string(excS._name()) << std::endl;
          throw;

     }  catch (const Exception& exc) {

          std::cerr << "  C++ Exception: " << std::endl;
          std::cerr << "  Exception: "+std::string(exc.what()) << std::endl;
          throw;

     } catch (...) {
           std::cerr << "  Generic Exception " << std::endl;
           throw;
     }

     try {
     
     // get the name of hosts
     system->get_all_host_names(nbr_of_host, elems.out(), it.out());

//     const char* host_name1 = elems[0U];
     const char* host_name2 = elems[1U];

     std::string path_string = std::string(system->name())+".sys/"+std::string(host_name2)+".host";

     CdmwPlatformMngt::ElementPath_var host_path (path_string.c_str());

     ManagedElement_var element_host (system->get_element (host_path.in()));

     CdmwPlatformMngt::Host_var second_host =
             CdmwPlatformMngt::Host::_narrow(element_host);

     second_host->stop_host();

     } catch (...) {

         std::cerr << "  Generic Exception during host operations" << std::endl;
         throw;

     }

}

void MyProcessBehaviour::run()
        throw(CdmwPlatformMngt::ProcessDelegate::NotReadyToRun,
              CdmwPlatformMngt::ProcessDelegate::AlreadyDone,
              CORBA::SystemException)
{
}

void MyProcessBehaviour::stop() throw(CORBA::SystemException)
{
}
