
/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef ___CCM_TEST_COMMAND_NOTIFICATION_CCM_CLIENT_IMPL_HPP__
#define ___CCM_TEST_COMMAND_NOTIFICATION_CCM_CLIENT_IMPL_HPP__

#include <map>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/osthreads/Mutex.hpp>
#include <Foundation/osthreads/Condition.hpp>
#include <queue>

#include <testcommandnotification/TestCommandNotificationLocal.skel.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtManagedElementContexts.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp>

namespace TestCmdNotification
{

class CCM_Client_impl;
    

class test_command_notif : public CppUnit::TestFixture
{
public:
    
    
    CCM_Client_impl * m_ccm_client;
    ::TestCmdNotification::CCM_Client_Context_var m_session_context;
    TestCmdNotification::Client_var m_component_ref;
    CORBA::ORB_var orb;
    
private:
    Cdmw::OsSupport::Mutex  *             m_test_mutex;
    Cdmw::OsSupport::Condition *          m_test_condition;
    CdmwPlatformMngt::External::System_var m_system;
    CdmwPlatformMngt::System_var m_System ;
   
public:
    test_command_notif();
    void register_observer();

    //
    //
    void test_cmd();
    
    // register test suite and test operation
    CPPUNIT_TEST_SUITE( test_command_notif );
    CPPUNIT_TEST( test_cmd );
    CPPUNIT_TEST_SUITE_END();

};
    
    
    
//
// IDL:thalesgroup.com/TestCommandNotification/Session_CCM_Client:1.0
//
class CCM_Client_impl : virtual public ::TestCmdNotification::Session_CCM_Client,
                        virtual public Cdmw::OrbSupport::RefCountLocalObject
{
    CCM_Client_impl(const CCM_Client_impl&);
    void operator=(const CCM_Client_impl&);
    ::TestCmdNotification::CCM_Client_Context_var m_session_context;
     
public:
    enum CommandIndex 
    {
        ACK_SYS,
        ACK_APPL,
        COLD_REST_APPL,
        HOT_REST_APPL,
        COLD_REST_PROC,
        HOT_REST_PROC,
        SYS_SNAPSHOT,
        STOP_PROC,
        INIT_PROC,
        RUN_PROC,
        STOP_APPL,
        INIT_APPL,
        RUN_APPL,
        HOT_REST_HOST,
        COLD_REST_HOST,
        COLD_REB_HOST,
        HOT_REB_HOST,
        EXT_TEST
    };
    
    std::string host_name1; 
    std::string host_name2;

    typedef struct {

        CommandIndex  nextcommand;

        bool result;

    } Command;


    typedef std::map < std::string, Command> MapCmd ;

    // command map to be performed
    MapCmd MapCommand;


public:

    CCM_Client_impl();
    ~CCM_Client_impl();
    
    //
    // Session component specific operations
    //
    void set_session_context(Components::SessionContext_ptr ctx)
        throw(CORBA::SystemException);
    void ccm_activate()
        throw(CORBA::SystemException);
    void ccm_passivate()
        throw(CORBA::SystemException);
    void ccm_remove()
        throw(CORBA::SystemException);

    void notify(CdmwPlatformMngt::Event* event)
       throw(CORBA::SystemException);
    
    void set_hostnames(const char* host1,const char* host2) ;
    
    void set_sup_host (const char* host_path);   
    
    void next_command(CdmwPlatformMngt::CdmwCommand* event,std::string pre_key)
    throw(CdmwPlatformMngt::ManagedElementNotFound);

    const char* getCommand(CommandIndex cmd) throw ();
    

private:
    
    std::string m_adminArgs;
    std::string m_sup_host_path;
    
    bool m_restart_requested;
    bool m_stop_proc_requested;
    bool m_stop_appl_requested;
    bool m_disable_notify;  
    bool m_ext_test; 
    bool m_automatic_action;
    
    ::TestCmdNotification::CCM_Client_Context* get_context();
    
    Cdmw::TestUtils::ThreadedTestRunner* m_test_thread;
    
    Cdmw::OsSupport::Mutex  *             m_mutex;
    Cdmw::OsSupport::Condition *          m_condition;

    std::vector <unsigned long> elapsedTimes;
    Cdmw::OsSupport::OS::Timeval               lastKillTime;
    
    friend class test_command_notif;
    
    void check_result();
    
public:   
    bool test_automatic_action() {return m_disable_notify;}
};

}; // End of namespace TestCmdNotification


#endif // ___CCM_TEST_COMMAND_NOTIFICATION_CCM_CLIENT_IMPL_HPP__

