/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/common/System.hpp"
#include "SystemMngt/platformvaluetypes/Restart_impl.hpp"
#include "SystemMngt/platformvaluetypes/Stop_impl.hpp"
#include "SystemMngt/platformvaluetypes/Shutdown_impl.hpp"
#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include <Foundation/testutils/CORBATestManager.hpp>

namespace Cdmw
{        // Begin namespace Cdmw

namespace PlatformMngt
{ 
// Begin namespace PlatformMngt


/**
 *Purpose:
 *<p> This class is used to perform the restart (hot and cold) 
 *    commands on System by external process.   
 */
class ExtSystemObserver_impl: public virtual POA_CdmwPlatformMngt::SupervisionObserver,
                              public virtual PortableServer::RefCountServantBase, 
                              public CppUnit::TestFixture
{

public:

    //Constructor
    ExtSystemObserver_impl ();

    //Destructor                 
    ~ExtSystemObserver_impl();
    
    void setUp ();

    //the notify function
    void notify( CdmwPlatformMngt::Event* an_event )
    throw( CORBA::SystemException );

    void test_system_restart();

private:

    /**
     * Indicates the restart type  :
     *    
     *    0  =>  cold_restart
     *    1  =>  host_restart
     */
    bool restart;
    // result of cold restart command
    bool cold_restart_result;    
    // result of hot restart command
    bool hot_restart_result;
    // result of stop command
    bool stop_sys_result;
    bool stop_host_result;
    bool shutdown_host_result;
    std::string host_name1; 
    std::string host_name2;
    
    CdmwPlatformMngt::System_var m_system;
    
    
    // register test suite and test operation
    CPPUNIT_TEST_SUITE( ExtSystemObserver_impl );
    CPPUNIT_TEST( test_system_restart );
    CPPUNIT_TEST_SUITE_END();

};

} // End namespace PlatformMngt
} // End namespace Cdmw


