/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <testcommandnotification/CCM_Client_impl.hpp>
#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/logging/CdmwLogging.stub.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include "SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/EntityStatusChange_impl.hpp"
#include <SystemMngt/idllib/CdmwPlatformMngtManagedElementContexts.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp>
#include <SystemMngt/platformvaluetypes/CdmwCommand_impl.hpp>
#include <SystemMngt/platformvaluetypes/Restart_impl.hpp>
#include <SystemMngt/platformvaluetypes/Acknowledge_impl.hpp>
#include <SystemMngt/platformvaluetypes/Stop_impl.hpp>
#include <SystemMngt/platformvaluetypes/Reboot_impl.hpp>
#include <SystemMngt/platformvaluetypes/Shutdown_impl.hpp>
#include <SystemMngt/tools/ManagedElementKey.hpp>

namespace TestCmdNotification {

//
// IDL:thalesgroup.com/CcmFtTest/ClientModule/CCM_Client:1.0
//
CCM_Client_impl::CCM_Client_impl() :
    m_test_thread(0) {
    //Creating a mutex to sinchronize the test's "kill and restart" procedure. 
    m_mutex=new Cdmw::OsSupport::Mutex();
    m_condition=new Cdmw::OsSupport::Condition(*m_mutex);

    m_adminArgs = "";
    m_restart_requested = false;
    m_stop_proc_requested=false;
    m_stop_appl_requested=false; 
    
    int def_test = atol((Cdmw::TestUtils::CORBATestManager::instance()->get_property("--default_test")).c_str());
    if (def_test==0)
    {
        m_disable_notify=false;
        m_automatic_action = false;
    }else
    {
        m_disable_notify=true;
        m_automatic_action = true; 
    }
    m_ext_test=false;
    host_name1="";
    host_name2="";   
    
}


void CCM_Client_impl::set_hostnames(const char* host1,const char* host2) 
{
    host_name1=host1;
    host_name2=host2;
}

void CCM_Client_impl::set_sup_host (const char* host_path)
{
    m_sup_host_path = host_path;
}


CCM_Client_impl::~CCM_Client_impl() {
    std::cerr << "   ... CCM_Client_impl::~CCM_Client_impl called!"<< std::endl;
    delete m_mutex;
    delete m_condition;

}

void CCM_Client_impl::set_session_context(Components::SessionContext_ptr ctx) throw(
        CORBA::SystemException) {
    m_session_context = ::TestCmdNotification::CCM_Client_Context::_narrow(ctx);
}

::TestCmdNotification::CCM_Client_Context* CCM_Client_impl::get_context() {
    return ::TestCmdNotification::CCM_Client_Context::_duplicate(m_session_context.in());
}

//
// ccm_activate function : it actives the component (call by the container)
//
void CCM_Client_impl::ccm_activate() throw(CORBA::SystemException) {
    std::cerr << "   ... CCM_Client_impl::ccm_activate called!"<< std::endl;

    // Get this component CORBA object
    // Note: this has to be performed in a CORBA request thread; thus
    //       not in test which is a specific thread.
    TestCmdNotification::Client_var
            this_component =TestCmdNotification::Client::_nil();

    try
    {
        // narrow Context in SessionContext  for get_CCM_objet() access
        Components::SessionContext_var ctx =
        Components::SessionContext::_narrow(m_session_context.in());

        // call SessionContext::get_CCM_objet()
        CORBA::Object_var obj =
        ctx->get_CCM_object();

        this_component =
        TestCmdNotification::Client::_narrow(obj.in());

        CDMW_ASSERT(!CORBA::is_nil(this_component.in()));

    } catch (const CORBA::UserException& e) {
        std::cerr << "UserException during get_CCM_Object() :"
        << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
        << std::endl;
        CDMW_ASSERT(false);
    } catch (const CORBA::SystemException& e) {
        std::cerr << "SystemException during get_CCM_Object() :"
        << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
        << std::endl;
        CDMW_ASSERT(false);
    }

    try {
        // save Context in CORBATestManager for usage in test
        Cdmw::TestUtils::CORBATestManager::instance()->add(
                "client_context",
                get_context());
        // save this component in CORBATestManager for usage in test
        Cdmw::TestUtils::CORBATestManager::instance()->add(
                "this_component",
                this_component.in());

        // save this executor in CORBATestManager for usage in test
        Cdmw::TestUtils::CORBATestManager::instance()->add(
                "this_executor",
                dynamic_cast< Session_CCM_Client_ptr > (this));

        // create test runner thread
        m_test_thread = new Cdmw::TestUtils::ThreadedTestRunner(
                Cdmw::TestUtils::CORBATestManager::instance()->get_controller());
        // add test to be run
        m_test_thread->addTest(
                CppUnit::TestFactoryRegistry::getRegistry("test_command_notif").makeTest());
        // start test
        m_test_thread->start();

    } catch (const CORBA::Exception& e) {
        std::cerr << "CORBA Exception exception during test: "
        << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
        << std::endl;
    } catch (const std::exception& e) {
        std::cerr << "std::exception exception during test: "
        << e.what()
        << std::endl;
    } catch (...) {
        std::cerr << "unknown exception during test !!" << std::endl;
    }
}

//
// ccm_passivate function : it makes stop the component (call by the container)
//
void CCM_Client_impl::ccm_passivate() throw(CORBA::SystemException) {
    std::cerr << "   ... CCM_Client_impl::ccm_passivate called!"<< std::endl;

    if (m_test_thread != 0) {
        // wait test to be finished
        m_test_thread->join();
        delete m_test_thread;
        m_test_thread = 0;
        
    }

}

//
// ccm_remove function : it destroys the component (call by the container)
//
void CCM_Client_impl::ccm_remove() throw(CORBA::SystemException) {
    std::cerr << "   ... CCM_Client_impl::ccm_remove called!"<< std::endl;
}

//
//
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(test_command_notif, "test_command_notif");

test_command_notif::test_command_notif() {
    orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::instance ()->get_ORB());

    // retrieve notification object
    CORBA::Object_var obj = Cdmw::TestUtils::CORBATestManager::instance()->get_object("client_context");

    m_session_context
            = ::TestCmdNotification::CCM_Client_Context::_narrow(obj.in());

    obj = Cdmw::TestUtils::CORBATestManager::instance()->get_object("this_executor");
    m_ccm_client = dynamic_cast< CCM_Client_impl* > (obj.in());

    obj = Cdmw::TestUtils::CORBATestManager::instance()->get_object("this_component");
    m_component_ref = TestCmdNotification::Client::_narrow(obj.in());

    // get ManagedComponentContext
    CORBA::Object_var
            obj_ctx =m_session_context->resolve_service_reference("ManagedComponentContext");
    CdmwPlatformMngt::ManagedElementContexts::ComponentContext_var
            component_context =CdmwPlatformMngt::ManagedElementContexts::ComponentContext::_narrow(obj_ctx.in());

    CPPUNIT_ASSERT(!CORBA::is_nil(component_context.in()));

    // get System supervision object
    TEST_INFO("   Get System supervision object");
    m_system = component_context->the_system();
    
    /*****************************************************************************
     * 
     *  Get host element path where the primary supervision is active
     * 
     *****************************************************************************/
    
     const char* result = 0 ;
     CORBA::Any_var property_any = new CORBA::Any();
     CdmwPlatformMngt::System_var m_System = CdmwPlatformMngt::System::_narrow (m_system.in());
     m_System->get_attribute (CdmwPlatformMngt::GLOBAL_SUPERVISION_HOST, property_any.out(), CdmwPlatformMngt::PRE_DEFINED);
     property_any.in() >>= result;
     std::string sup_host_path = Cdmw::PlatformMngt::ManagedElementKey::Get_host_path(m_System->name (),result);
    m_ccm_client->set_sup_host(sup_host_path.c_str());                 
                        

    /*****************************************************************************
     * 
     *  CCM_Client in this test has a map containing a key 'command+element_name' 
     *  associated to a struct of type 'Command'.
     * 
     *  Filling this map creating an activation sequence of commands for specific 
     *  managed elements (system,applications and processes).
     * 
     *****************************************************************************/

    CCM_Client_impl::Command cmd;
    
    CdmwPlatformMngt::ElementNames_var elems;
    CdmwPlatformMngt::ElementNamesIterator_var it;
    CORBA::ULong nbr_of_host = m_System->get_number_of_hosts();
    m_System->get_all_host_names(nbr_of_host, elems.out(), it.out());

    //The Scenario has two different hosts
    std::string host_name1 = CORBA::string_dup ((elems.in())[0]);
    std::string host_name2 = CORBA::string_dup ((elems.in())[1]);
    m_ccm_client->set_hostnames(host_name1.c_str(),host_name2.c_str());
    std::string application_name="CcmTestCommandNotificationApplicationServer";
    std::string proc_name="TestCommandNotificationProcessServer1";
    std::string proc_no_comp="Process_test";
    std::string key;
    // System Acknowledge -> Application Acknowledge
    key="ack";
    key.append(m_system->name());
    cmd.nextcommand = CCM_Client_impl::ACK_APPL;
    // This false value will be modified by the notify().
    cmd.result = false;
    m_ccm_client->MapCommand.insert(CCM_Client_impl::MapCmd::value_type (key,cmd));

    // Application Acknowledge --> Application cold restart
    key="ack";
    key.append(application_name);
    cmd.nextcommand = CCM_Client_impl::COLD_REST_APPL;
    cmd.result = false;
    m_ccm_client->MapCommand.insert(CCM_Client_impl::MapCmd::value_type (key,cmd));

    // Application cold restart --> Application hot restart
    key="c_rest";
    key.append(application_name);
    cmd.nextcommand = CCM_Client_impl::HOT_REST_APPL;
    cmd.result = false;
    m_ccm_client->MapCommand.insert(CCM_Client_impl::MapCmd::value_type (key,cmd));

    // Application hot restart --> Process cold restart
    key="h_rest";
    key.append(application_name);
    cmd.nextcommand = CCM_Client_impl::COLD_REST_PROC;
    cmd.result = false;
    m_ccm_client->MapCommand.insert(CCM_Client_impl::MapCmd::value_type (key,cmd));

    // Process cold restart --> Process hot restart
    key="c_rest";
    key.append(proc_name);
    cmd.nextcommand = CCM_Client_impl::HOT_REST_PROC;
    cmd.result = false;
    m_ccm_client->MapCommand.insert(CCM_Client_impl::MapCmd::value_type (key,cmd));

    // Process hot restart --> Application stop
    key="h_rest";
    key.append(proc_name);
    cmd.nextcommand = CCM_Client_impl::STOP_APPL;
    cmd.result = false;
    m_ccm_client->MapCommand.insert(CCM_Client_impl::MapCmd::value_type (key,cmd));

    // Application stop --> Process stop
    key="stop";
    key.append(application_name);
    cmd.nextcommand = CCM_Client_impl::STOP_PROC;
    cmd.result = false;
    m_ccm_client->MapCommand.insert(CCM_Client_impl::MapCmd::value_type (key,cmd));
    
    // Process stop --> host hot_restart
    key="stop";
    key.append(proc_no_comp);
    cmd.nextcommand = CCM_Client_impl::HOT_REST_HOST;
    cmd.result = false;
    m_ccm_client->MapCommand.insert(CCM_Client_impl::MapCmd::value_type (key,cmd));
    
    
    // host_hot_restart stop --> host_cold_restart 
    key="h_rest";
    key.append(host_name2);
    cmd.nextcommand = CCM_Client_impl::COLD_REST_HOST;
    cmd.result = false;
    m_ccm_client->MapCommand.insert(CCM_Client_impl::MapCmd::value_type (key,cmd));

    // host_cold_restart  --> host_hot_reboot
    key="c_rest";
    key.append(host_name2);
    cmd.nextcommand = CCM_Client_impl::HOT_REB_HOST;
    cmd.result = false;
    m_ccm_client->MapCommand.insert(CCM_Client_impl::MapCmd::value_type (key,cmd));
    
    // host_hot_reboot  --> host_cold_reboot
    key="c_reb";
    key.append(host_name2);
    cmd.nextcommand = CCM_Client_impl::SYS_SNAPSHOT;
    cmd.result = false;
    m_ccm_client->MapCommand.insert(CCM_Client_impl::MapCmd::value_type (key,cmd));
    
}

void test_command_notif::register_observer() {
    // register this component as System observer
    TEST_INFO("   register this component as System Observer");

    // Register event factories
    CORBA::ValueFactoryBase_var factory, old_factory;

    factory = new Cdmw::PlatformMngt::ProcessStatusChangeFactory;
    old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessStatusChange:1.0",
            factory.in() );

    factory = new Cdmw::PlatformMngt::RestartFactory;
    old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/Restart:1.0", factory.in() );

    factory = new Cdmw::PlatformMngt::RebootFactory;
    old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/Reboot:1.0", factory.in() );

    factory = new Cdmw::PlatformMngt::ShutdownFactory;
    old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/Shutdown:1.0", factory.in() );

    factory = new Cdmw::PlatformMngt::StopFactory;
    old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/Stop:1.0", factory.in() );

    factory = new Cdmw::PlatformMngt::AcknowledgeFactory;
    old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/Acknowledge:1.0",
            factory.in() );

    // register to command notification
    CdmwPlatformMngt::RegistrationDepth depth=CdmwPlatformMngt::FULL_HIERARCHY;
    m_system->register_observer_ext("TestCmdNotification_Client", depth,
            m_component_ref.in());

}

void CCM_Client_impl::check_result( )
{
    for (MapCmd::iterator it = MapCommand.begin(); it!=MapCommand.end();it++ )    
    {    
        cerr << "[ KEY = " << "]"<< it->first << "command result ="<< (it->second).result<<std::endl; 
        if (!((it->second).result))
        {
            CPPUNIT_ASSERT (false);
        }
    }   
    CPPUNIT_ASSERT (true);
}

void test_command_notif::test_cmd() {

    std::cerr << "   ... test_command_notif::do_tests called!"<< std::endl;
    try
    {
        register_observer();
        TestCommandNotification_var test_command_notif =
            m_session_context->get_connection_test_server();
        sleep(2);
       
        if  (! m_ccm_client->test_automatic_action())
        {
            TEST_INFO("\n\n ###  Starting the test with the first command requested: system acknowledge ###  \n\n"); 
        }
          
    } catch (const CORBA::Exception & ex)
    {
        std::cerr<<ex;
    }
}

const char* CCM_Client_impl::getCommand(CommandIndex cmd) throw() {
    switch (cmd) {
    case INIT_PROC:
    {
        std::string command = "cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --proc-initialise CcmTestCommandNotificationApplicationServer Process_test ";
        command.append (host_name1);
        return command.c_str();
    }
    case ACK_SYS:
        return "cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --sys-acknowledge";
    case ACK_APPL:
        return "cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --app-acknowledge CcmTestCommandNotificationApplicationServer";
    case COLD_REST_APPL:
        return "cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --app-cold-restart CcmTestCommandNotificationApplicationServer";
    case HOT_REST_APPL:
        return "cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --app-hot-restart CcmTestCommandNotificationApplicationServer";
    case COLD_REST_PROC:
    {
        std::string command = "cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --proc-cold-restart CcmTestCommandNotificationApplicationServer TestCommandNotificationProcessServer1 ";
        command.append (host_name2);
        return command.c_str();
    }
    case HOT_REST_PROC:
    {
        std::string command = "cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --proc-hot-restart CcmTestCommandNotificationApplicationServer TestCommandNotificationProcessServer1 ";
        command.append (host_name2);
        return command.c_str();
    }
    case SYS_SNAPSHOT:
        return "cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --sys-snapshot";
    case STOP_PROC:
    {
        std::string command = "cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --proc-stop CcmTestCommandNotificationApplicationServer Process_test ";
        command.append (host_name1);
        return command.c_str();
    }
    case RUN_PROC:
    {
        std::string command = "cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --proc-run CcmTestCommandNotificationApplicationServer Process_test ";
        command.append (host_name1);
        return command.c_str();
    }

    case STOP_APPL:
        return "cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --app-stop CcmTestCommandNotificationApplicationServer";
    case INIT_APPL:
        return "cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --app-initialise CcmTestCommandNotificationApplicationServer";
    case RUN_APPL:
        return "cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --app-run CcmTestCommandNotificationApplicationServer";
    case HOT_REST_HOST:
    {
        std::string command = "cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --host-hot-restart ";
        command.append (host_name2);
        return command.c_str();
    }
    case COLD_REST_HOST:
    {
        std::string command = "cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --host-cold-restart ";
        command.append (host_name2);
        return command.c_str();
    }
    case COLD_REB_HOST:
    {
        std::string command = "cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --host-cold-reboot ";
        command.append (host_name2);
        return command.c_str();
    }     
    case HOT_REB_HOST:
    {
        std::string command = "cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --host-hot-reboot ";
        command.append (host_name2);
        return command.c_str();
    } default:
        // Unknown test modus!  Add to list abough!!
        CDMW_NEVER_HERE();
    }
}

void CCM_Client_impl::next_command(CdmwPlatformMngt::CdmwCommand* event,std::string pre_key) 
     throw(CdmwPlatformMngt::ManagedElementNotFound) 
{
    //Setting true the 'result' in the map of commands
    std::string key=pre_key;
    key.append(event->element_name());
    MapCmd::iterator commandIt = (MapCommand).find(key);
    if (commandIt == MapCommand.end())
        throw CdmwPlatformMngt::ManagedElementNotFound();
    (commandIt->second).result = true;
    // command to perform
    m_adminArgs = getCommand ((commandIt->second).nextcommand);
    
    std::cerr<< "\n=================================================="
            "========================================================\n";
    std::cerr<< event->to_string();
    std::cerr<< " \n Next command incoming = "<<m_adminArgs <<"\n";
    std::cerr<< "\n================================================="
            "=======================================================\n";
    
}


void CCM_Client_impl::notify(CdmwPlatformMngt::Event* an_event) throw(
        CORBA::SystemException) {
 
    if  (m_automatic_action)
    {
        std::string sup = "cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision";
        //This test waits the system status STARTED to check notification for automatic action
        CdmwPlatformMngt::SystemStatusChange* sys_chg = dynamic_cast<CdmwPlatformMngt::SystemStatusChange*>(an_event);
        if( (sys_chg !=0) && ( CdmwPlatformMngt::SYSTEM_STARTED == sys_chg->system_status()))
        {
                std::cerr<<"\n---------------SYSTEM STARTED---------------\n";
                std::string command = sup + " --proc-initialise CcmTestCommandNotificationApplicationServer Process_autom "+host_name1;
                Cdmw::OsSupport::OS::create_process("cdmw_platform_admin", command);
        }
        CdmwPlatformMngt::CdmwCommand* Ccmd = dynamic_cast <CdmwPlatformMngt::CdmwCommand*>(an_event);
        if(Ccmd !=0 )
        {
            // CdmwCommand::Restart Events
            CdmwPlatformMngt::Restart* rest=dynamic_cast<CdmwPlatformMngt::Restart*>(Ccmd);
            if(rest !=0 )
            {
                // All the platform admin command will be notified with an empty issuer 
                if (strcmp (Ccmd->issuer(), m_sup_host_path.c_str()) == 0)
                {
                    TEST_INFO( "\n\n ###  ISSUER is  OK for AUTOMATIC ACTION ### \n\n");                  
                }
                else 
                {
                    TEST_INFO(" ERROR: Issuer NOT VALID for automatic action!!! ");
                    CPPUNIT_ASSERT (false);
                }
                // print test summary
                Cdmw::TestUtils::CORBATestManager::instance()->summary();                    // print test summary
                return;
            }  
        }
    }
    
    if  (!m_disable_notify)
    {
        try
        {

            //This test waits the system status STARTED after each kind of restart before the next command
            CdmwPlatformMngt::SystemStatusChange* sys_chg = dynamic_cast<CdmwPlatformMngt::SystemStatusChange*>(an_event);
            if( (sys_chg !=0) && ( CdmwPlatformMngt::SYSTEM_STARTED == sys_chg->system_status()))
            {
                if (m_restart_requested)
                {
                    std::cerr<<"\n---------------SYSTEM STARTED---------------\n";
                    Cdmw::OsSupport::OS::create_process("cdmw_platform_admin", m_adminArgs);
                    m_restart_requested = false;

                }
                if(m_ext_test)
                {
                    check_result();
                    std::cerr<< "\n===================================\n";
                    std::cerr<< "\nFIRST SUMMARY PRINTED \n";
                    std::cerr<< "\n===================================\n";
                    // print test summary
                    Cdmw::TestUtils::CORBATestManager::instance()->summary();
                    m_disable_notify=true;
                    sleep(10);
                    std::cerr<< "\n===================================\n";
                    std::cerr<< "\n SECOND TEST PHASE STARTING...\n";
                    std::cerr<< "\n===================================\n";
                    Cdmw::OsSupport::OS::create_process("start_ext_command",""); 


                    m_ext_test=false;
                    return;
                }
            } //SystemStatusChange


            // CdmwCommand Events 
            CdmwPlatformMngt::CdmwCommand* Ccmd = dynamic_cast <CdmwPlatformMngt::CdmwCommand*>(an_event);
            if(Ccmd !=0 )
            {
                // All the platform admin command will be notified with an empty issuer 
                std::cerr << " \n\n\n Ccmd->issuer() : "<< Ccmd->issuer()<<"\n\n\n";
                if (strcmp (Ccmd->issuer(), "") == 0)
                {

                    // CdmwCommand::Acknowledge Events
                    CdmwPlatformMngt::Acknowledge* ack = dynamic_cast <CdmwPlatformMngt::Acknowledge*>(Ccmd);
                    if(ack !=0 )
                    {
                        next_command(ack,"ack");
                        Cdmw::OsSupport::OS::create_process("cdmw_platform_admin", m_adminArgs);
                        return;
                    }

                    // CdmwCommand::Restart Events
                    CdmwPlatformMngt::Restart* rest=dynamic_cast<CdmwPlatformMngt::Restart*>(Ccmd);
                    if(rest !=0 )
                    {
                        if( CdmwPlatformMngt::COLD_RESTART==rest->restart_warmth())
                        {
                            m_restart_requested=true;
                        next_command(rest,"c_rest");
                            return;
                        }

                        if( CdmwPlatformMngt::HOT_RESTART==rest->restart_warmth())
                        {
                            m_restart_requested=true;//false di default
                        next_command(rest,"h_rest");
                            return;
                        }
                    }
                    // CdmwCommand::Reboot Events
                    CdmwPlatformMngt::Reboot* reb=dynamic_cast<CdmwPlatformMngt::Reboot*>(Ccmd);
                    if(reb !=0 )
                    {
                        if( CdmwPlatformMngt::COLD_RESTART==reb->restart_warmth())
                        {
                            m_restart_requested=true;
                            next_command(reb,"c_reb");
                            return;
                        }

                        if( CdmwPlatformMngt::HOT_RESTART==reb->restart_warmth())
                        {
                            m_restart_requested=true;//false di default
                            next_command(reb,"c_reb");// 25-07-08 GD
                            m_ext_test=true;
                            return;
                        }

                    }
                    // CdmwCommand::Shutdown Events
                    CdmwPlatformMngt::Shutdown* shutd=dynamic_cast<CdmwPlatformMngt::Shutdown*>(Ccmd);
                    if(shutd !=0 )
                    {

                    }
                    // CdmwCommand::Stop Events
                    CdmwPlatformMngt::Stop* stop_cmd=dynamic_cast<CdmwPlatformMngt::Stop*>(Ccmd);
                    if(stop_cmd !=0 )
                    {
                        const char* elem_name=stop_cmd->element_name();
                        if(0 == strcmp(elem_name,"CcmTestCommandNotificationApplicationServer"))
                        {
                            m_stop_appl_requested=true;
                            next_command(stop_cmd,"stop");
                        } else if(0 == strcmp(elem_name,"Process_test"))
                        {
                            m_stop_proc_requested=true;
                            next_command(stop_cmd,"stop");
                        }

                        return;
                    }

                } 
                else 
                {
                    std::cerr << " \n\n\n #################### IF issuer InValid  "<<Ccmd->issuer()<<"\n\n\n";
                    TEST_INFO(" ERROR: Issuer NOT VALID!!! ");
                    CPPUNIT_ASSERT (false);
                }
            }// CdmwCommand

            //After an application stop we need to restart it.

            CdmwPlatformMngt::ApplicationStatusChange* app_chg = dynamic_cast<CdmwPlatformMngt::ApplicationStatusChange*>(an_event);
            {
                if( (app_chg !=0) && ( CdmwPlatformMngt::APPLICATION_STOPPED == app_chg->application_status()))
                {
                    if (m_stop_appl_requested)
                    {
                        std::string app_init_args=getCommand(INIT_APPL);
                        Cdmw::OsSupport::OS::create_process("cdmw_platform_admin", app_init_args);
                        return;
                    }
                }

                if( (app_chg !=0) && ( CdmwPlatformMngt::APPLICATION_INITIALISED == app_chg->application_status()))
                {
                    if (m_stop_appl_requested)
                    {
                        std::string app_run_args=getCommand(RUN_APPL);
                        Cdmw::OsSupport::OS::create_process("cdmw_platform_admin", app_run_args);
                        return;

                    }
                }

                if( (app_chg !=0) && ( CdmwPlatformMngt::APPLICATION_RUNNING == app_chg->application_status()))
                {
                    if (m_stop_appl_requested)
                    {
                        //m_adminArgs is modified by STOP_PROCESS command event
                        Cdmw::OsSupport::OS::create_process("cdmw_platform_admin", m_adminArgs);
                        m_stop_appl_requested = false;
                        return;
                    }
                }

            }  //ApplicationStatusChange

            CdmwPlatformMngt::ProcessStatusChange* pro_chg = dynamic_cast<CdmwPlatformMngt::ProcessStatusChange*>(an_event);
            {
                if( (pro_chg !=0) && ( CdmwPlatformMngt::PROCESS_STOPPED == pro_chg->process_status()))
                {
                    if (m_stop_proc_requested)
                    {
                        std::string proc_init_args=getCommand(INIT_PROC);
                        Cdmw::OsSupport::OS::create_process("cdmw_platform_admin", proc_init_args);
                        return;
                    }
                }

                if( (pro_chg !=0) && ( CdmwPlatformMngt::PROCESS_INITIALISED == pro_chg->process_status()))
                {
                    if (m_stop_proc_requested)
                    {
                        std::string proc_run_args=getCommand(RUN_PROC);
                        Cdmw::OsSupport::OS::create_process("cdmw_platform_admin", proc_run_args);
                        return;

                    }
                }        

                if( (pro_chg !=0) && ( CdmwPlatformMngt::PROCESS_RUNNING == pro_chg->process_status()))
                {
                    if (m_stop_proc_requested)
                    {
                        //m_adminArgs is modified by STOP_PROCESS command event
                        Cdmw::OsSupport::OS::create_process("cdmw_platform_admin", m_adminArgs);
                        m_stop_proc_requested = false;
                        return;
                    }
                }
            }// ProcessStatusChange



        }
        catch (const std::exception& ex)
        {
            std::cerr << " TestCmdNotification_Client observer exception  " << ex.what();
        }
    }

}

}
; // namespace TestCmdNotification


