/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <iomanip>
#include <typeinfo>
#include <fstream>
#include <sys/time.h>
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/common/Options.hpp"
#include "Foundation/common/Exception.hpp"
#include "testcommandnotification/PropertyStatusChangeObserver_impl.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "SystemMngt/platformvaluetypes/SystemStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/HostStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/EntityStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ComponentStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/RedundancyGroupStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/PropertyChange_impl.hpp"

#include "SystemMngt/idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp"
#include <tao/AnyTypeCode/TypeCode.h>


CPPUNIT_TEST_SUITE_REGISTRATION(Cdmw::PlatformMngt::PropertyStatusChange_impl);

namespace Cdmw
{

namespace PlatformMngt
{

void PropertyStatusChange_impl::setUp ()
{
    try
    {
        //Getting the orb reference
        CORBA::ORB_var orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::instance ()->get_ORB ());

        //Getting the system reference
        std::string system_corbaloc = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";
        CORBA::Object_var system_obj = orb->string_to_object (system_corbaloc.c_str());
        m_system = CdmwPlatformMngt::System::_narrow (system_obj.in ());
        
        // Get a reference to the root POA
        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var poa = PortableServer::POA::_narrow( obj.in() );

        // Activate the POA manager
        PortableServer::POAManager_var mgr = poa->the_POAManager();
        mgr->activate();
        
        
        CdmwPlatformMngt::ElementNames_var elems;
        CdmwPlatformMngt::ElementNamesIterator_var it;
        //CdmwPlatformMngt::System_var System = CdmwPlatformMngt::System::_narrow (m_system.in());
        CORBA::ULong nbr_of_host = m_system->get_number_of_hosts();
        m_system->get_all_host_names(nbr_of_host, elems.out(), it.out());

        //The Scenario has two different hosts
        host_name1 = CORBA::string_dup ((elems.in())[0]);
        host_name2 = CORBA::string_dup ((elems.in())[1]);
        
        try
        {
            //Getting the application, process and component references
            app_managed_proc = "CcmTestCommandNotificationApplicationClient";
            managed_process = "TestCommandNotificationProcessClient1";
            m_application = m_system->get_application(app_managed_proc.c_str());
            m_managed_process = m_application->get_process(managed_process.c_str(), host_name1.c_str());
            comp = "Client1";
            m_component = m_managed_process->get_component(comp.c_str());
            
            app_unmanaged_proc = "ExecutionManagers";
            unmanaged_process = "ExecutionManager1";
            CdmwPlatformMngt::Application_var application = m_system->get_application(app_unmanaged_proc.c_str());
            m_unmanaged_process = application->get_process(unmanaged_process.c_str(), host_name1.c_str());
           
            m_host1= m_system->get_host(host_name2.c_str());
            
            m_entity= m_system ->get_entity("CdmwSystemSynchronization");
        }
        catch ( const  CdmwPlatformMngt::EntityNotFound& e)
        {
            cerr << "Exception !!! "<<e;
        }
        catch ( const  CdmwPlatformMngt::ApplicationNotFound& e)
        {
            cerr << "Exception !!! "<<e;
        }
        catch ( const  CdmwPlatformMngt::HostNotFound& e)
        {
            cerr << "Exception !!! "<<e;
        }
        catch (  const  CdmwPlatformMngt::ProcessNotFound& e)
        {
            cerr << "Exception !!! "<<e;
        }
        catch (  const  CdmwPlatformMngt::ComponentNotFound& e)
        {
            cerr << "Exception !!! "<<e;
        }
        catch (...)
        {
            cerr << "Exception !!! ";
        }

        // Register event factories
        CORBA::ValueFactoryBase_var factory, old_factory;
        factory = new HostStatusChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/HostStatusChange:1.0",
                factory.in() );
        
        factory = new SystemStatusChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/SystemStatusChange:1.0",
                factory.in() );

        factory = new ApplicationStatusChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationStatusChange:1.0",
                factory.in() );

        factory = new ProcessStatusChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessStatusChange:1.0",
                factory.in() );

        factory = new EntityStatusChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/EntityStatusChange:1.0",
                factory.in() );

        factory = new ComponentStatusChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/ComponentStatusChange:1.0",
                factory.in() );

        factory = new RedundancyGroupStatusChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/RedundancyGroupStatusChange:1.0",
                factory.in() );
        
        factory = new PropertyChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/PropertyChange:1.0",
                factory.in() );


        // Register the observer to the m_system
        CdmwPlatformMngt::SupervisionObserver_var observer = this->_this();

        std::string observer_name = "TestObserver";
        CdmwPlatformMngt::RegistrationDepth depth=CdmwPlatformMngt::FULL_HIERARCHY;
        m_system->register_observer_ext( observer_name.c_str(), depth, observer.in() );
    }
    catch (const CORBA::SystemException& ex) 
    {
        std::cerr<< ex._name()<<std::endl;
        throw;
    }
    catch(...)
    {
    }
}



PropertyStatusChange_impl::PropertyStatusChange_impl ()
{
}

PropertyStatusChange_impl::~PropertyStatusChange_impl()
{
}

void  PropertyStatusChange_impl::notify(CdmwPlatformMngt::Event* an_event)
throw( CORBA::SystemException )
{
    try
    {
        /**********************************************************
         * Check that TECH_STATUS and TECH_STATUS_INFO properties
         * are not notified with property change notifications 
         * ********************************************************/
        if (an_event->event_kind() == CdmwPlatformMngt::PROPERTY_CHANGE )
        {
            CdmwPlatformMngt::PropertyChange* prp_chg = dynamic_cast<CdmwPlatformMngt::PropertyChange*> (an_event);
            if (prp_chg!=0)
            {
                CORBA::ULong len  = prp_chg->property_infos().length ();
                const char* value;
                
                for (CORBA::ULong j = 0; j < len ; ++j)
                {
                    if (strcmp(prp_chg->property_infos()[j].property_name, "Cdmw.TechStatus") == 0) 
                    {
                        try
                        {                           
                            prp_chg->property_infos()[j].property_new_value >>= CORBA::Any::to_string(value,0);                           
                            CORBA::String_var message = an_event->to_string();
                            std::cout <<"##### TECH STATUS PROPERTY = " << value << " ####"<<std::endl;
                            CPPUNIT_ASSERT (false);
                        }
                        catch (const std::exception& ex)
                        {
                            std::cerr << " observer exception !!! " << ex.what();
                        }
                    }
                    else if  (strcmp(prp_chg->property_infos()[j].property_name, "Cdmw.TechStatusInfo") == 0)
                    {
                        try
                        {
                            prp_chg->property_infos()[j].property_new_value >>= CORBA::Any::to_string(value,0);                           
                            CORBA::String_var message = an_event->to_string();
                            std::cout <<"##### TECH STATUS INFO = " <<value <<"  ###"<< std::endl;
                            CPPUNIT_ASSERT (false);
                        }
                        catch (const std::exception& ex)
                        {
                            std::cerr << " observer exception !!! " << ex.what();
                        }
                    }
                }
            }
        }
                
        /**********************************************************
         * Compare the actual entity status with the status value 
         * notified with entity status changhe notifications 
         * ********************************************************/
        if (an_event->event_kind() == CdmwPlatformMngt::ENTITY_STATUS_CHANGE )
        {
            CdmwPlatformMngt::EntityStatusChange* entity_chg = CdmwPlatformMngt::EntityStatusChange::_downcast( an_event );
            std::string entity_name = std::string ( entity_chg->entity_name());                 
            if (entity_name == "CdmwSystemSynchronization")
            {
                CdmwPlatformMngt::EntityStatus entity_status = m_entity->get_status(status_result.out());
                if (strcmp(entity_chg->entity_status(), entity_status) == 0 )
                {
                    std::cout << "\n\nget_status()... OK for entity = "<<entity_name<<std::endl;
                }
                else
                {
                    std::cout << "\n\n #### NOT OK !!! for entity = "<<entity_name<<""
                            "received status = "<<entity_chg->entity_status()<< " get_status()= "<<entity_status<<std::endl;
                    CPPUNIT_ASSERT(false);
                } 
            }
        }
        
        /**********************************************************
         * Compare the actual host status with the status value 
         * notified with host status changhe notifications 
         * ********************************************************/
        CdmwPlatformMngt::HostStatusChange* host_chg = 
             dynamic_cast<CdmwPlatformMngt::HostStatusChange*> (an_event);
        if ( host_chg!=0 ) 
        {
            if ((host_chg->host_status()==CdmwPlatformMngt::HOST_STARTED) ||
                    (host_chg->host_status()==CdmwPlatformMngt::HOST_RESPONDING))
            {
                CdmwPlatformMngt::HostStatus host_status = m_host1->status();
                if ((host_chg->host_status()==host_status) || (host_chg->host_status()<host_status))
                {
                    std::cout << "\n\n get_status()... OK for host = "<<host_chg->host_name()<<std::endl;
                }
                else
                {
                    std::cout << "\n\n #### NOT OK !!!  for host = "<<host_chg->host_name()<<" "
                            "received status = "<<host_chg->host_status()<< " get_status()= "<<host_status<<std::endl;
                    CPPUNIT_ASSERT(false);
                }
            }  
        }        
        
        /**********************************************************
         * Compare the actual system status with the status value 
         * notified with system status changhe notifications 
         * ********************************************************/
        CdmwPlatformMngt::SystemStatusChange* sys_chg = 
             dynamic_cast<CdmwPlatformMngt::SystemStatusChange*> (an_event);             
        if ( sys_chg!=0 ) 
        {
            if ((sys_chg->system_status()== CdmwPlatformMngt::SYSTEM_STARTED) ||
                    (sys_chg->system_status()== CdmwPlatformMngt::SYSTEM_STOPPED))
            {
                CdmwPlatformMngt::SystemStatus sys_status = m_system->get_status(status_result.out());
                if ((sys_chg->system_status()==sys_status) || (sys_chg->system_status()<sys_status))
                {
                    std::cout << "\n\n get_status()... OK  for system = "<<sys_chg->system_name()<<std::endl;
                }
                else
                {
                    std::cout << "\n\n #### NOT OK !!! for system  "
                            "received status = "<<sys_chg->system_status()<< " get_status()= "<<sys_status<<std::endl;
                    CPPUNIT_ASSERT(false);
                }               
            }
        }

        /**********************************************************
         * Compare the actual application status with the status value 
         * notified with application status changhe notifications 
         * ********************************************************/
        CdmwPlatformMngt::ApplicationStatusChange* app_chg = 
             dynamic_cast<CdmwPlatformMngt::ApplicationStatusChange*> (an_event);
             
        if ((app_chg!=0) && (strcmp(app_chg->application_name(), app_managed_proc.c_str()) == 0))
        {
            if ((app_chg->application_status()==CdmwPlatformMngt::APPLICATION_RUNNING) ||
                    (app_chg->application_status()==CdmwPlatformMngt::APPLICATION_STOPPED))
            {
                CdmwPlatformMngt::ApplicationStatus appl_status = m_application->get_status(status_result.out());
                if ( (app_chg->application_status()==appl_status) || (app_chg->application_status()<appl_status))
                {
                    std::cout << "\n\n get_status()... OK  for application = "<<app_chg->application_name()<<std::endl;
                }
                else
                {
                    std::cout << "\n\n #### NOT OK !!! for application "
                            "received status = "<<app_chg->application_status()<< " get_status()= "<<appl_status<<std::endl;
                    CPPUNIT_ASSERT(false);
                }
            }       
        }
        
        /**********************************************************
         * Compare the actual component status with the status value 
         * notified with component status changhe notifications 
         * ********************************************************/
        CdmwPlatformMngt::ComponentStatusChange* comp_chg = 
             dynamic_cast<CdmwPlatformMngt::ComponentStatusChange*> (an_event);
             
        if ((comp_chg!=0) && (strcmp(comp_chg->component_name(), comp.c_str()) == 0))
        {
            if ((comp_chg->component_status()==CdmwPlatformMngt::COMPONENT_INITIALISED) ||
                    (comp_chg->component_status()==CdmwPlatformMngt::COMPONENT_STOPPED))
            {
                CdmwPlatformMngt::ComponentStatus comp_status = m_component->get_status(status_result.out());
                if ( (comp_chg->component_status()==comp_status) || (comp_chg->component_status()<comp_status) )
                {
                    std::cout << "\n\n get_status()... OK for component = "<<comp_chg->component_name()<<std::endl;
                }
                else
                {
                    std::cout << "\n\n ####NOT OK !!!  for component "
                            "received status = "<<comp_chg->component_status()<< " get_status()= "<<comp_status<<std::endl;
                    CPPUNIT_ASSERT(false);
                }
                  
            }     
        }
         
        /**********************************************************
         * Compare the actual process status with the status value 
         * notified with process status changhe notifications 
         * ********************************************************/
        CdmwPlatformMngt::ProcessStatusChange* prc_chg = dynamic_cast<CdmwPlatformMngt::ProcessStatusChange*> (an_event);
        if (prc_chg!=0)
        {
            if ((prc_chg->process_status()==CdmwPlatformMngt::PROCESS_RUNNING) ||
                    (prc_chg->process_status()==CdmwPlatformMngt::PROCESS_STOPPED))
            {
                if (strcmp(prc_chg->process_id().process_name, managed_process.c_str()) == 0)
                {
                    CdmwPlatformMngt::ProcessStatus proc_status = m_managed_process->get_status(status_result.out());
                    if ( (prc_chg->process_status()==proc_status) || (prc_chg->process_status()<proc_status) )
                    {
                        std::cout << "\n\n  get_status()... OK  for managed process = "<<prc_chg->process_id().process_name<<std::endl;
                    }
                    else
                    {
                        std::cout << "\n\n #### NOT OK !!! for managed process "
                                "received status = "<<prc_chg->process_status()<< " get_status()= "<<proc_status<<std::endl;
                        CPPUNIT_ASSERT(false);
                    }
                }
                
                if (strcmp(prc_chg->process_id().process_name, unmanaged_process.c_str()) == 0)
                {
                    CdmwPlatformMngt::ProcessStatus proc_status = m_unmanaged_process->get_status(status_result.out());
                    if  ( (prc_chg->process_status()==proc_status) || (prc_chg->process_status()<proc_status) )
                    {
                        std::cout << "\n\n get_status()... OK  for unmanaged process = "<<prc_chg->process_id().process_name<<std::endl;
                    }
                    else
                    {
                        std::cout << "\n\n #### NOT OK !!! for unmanaged process "
                                "received status = "<<prc_chg->process_status()<< " get_status()= "<<proc_status<<std::endl;
                        CPPUNIT_ASSERT(false);
                    }
                }
            }
        }    
    }
//    catch (const std::exception& ex)
//    {
//        std::cerr << " Process observer exception  " << ex.what();
//    }
    catch (const CORBA::SystemException& ex)
    {
        std::cerr << " EXCEPTION  "<< ex ;
    }
    catch (...)
    {
        std::cerr << " EXCEPTION  " ;
    }

}

void PropertyStatusChange_impl::test_system_restart()
{
    std::cerr<< "\n======================================================================================================\n";
    TEST_INFO("System START");
    std::cerr<< "\n======================================================================================================\n";
    Cdmw::OsSupport::OS::create_process("cdmw_platform_admin",
            "cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --sys-start"); 
    if (!CORBA::is_nil(m_entity))
    {
        m_entity->set_status("OK","in order to test get_satus of entity");
    }
    Cdmw::OsSupport::OS::sleep (5000);
    std::cerr<< "\n======================================================================================================\n";
    TEST_INFO("Kill primary supervision");
    std::cerr<< "\n======================================================================================================\n";
    std::string adminArgs ="--system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --proc-initialise "
            "CcmTestCommandNotificationApplicationClient Process_sup_killer " + host_name1;
    OsSupport::OS::create_process("cdmw_platform_admin", adminArgs);    
    Cdmw::OsSupport::OS::sleep (15000);
    std::cerr<< "\n======================================================================================================\n";
    TEST_INFO("System STOP");
    std::cerr<< "\n======================================================================================================\n";
    Cdmw::OsSupport::OS::create_process("cdmw_platform_admin",
            "cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --sys-stop"); 
    Cdmw::OsSupport::OS::sleep (20000);
}

} // PlatformMngt
} // Cdmw

