/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef ___CCM_TEST_EXIT_STOP_CCM_CLIENT_IMPL_HPP__
#define ___CCM_TEST_EXIT_STOP_CCM_CLIENT_IMPL_HPP__

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/osthreads/Mutex.hpp>
#include <queue>

#include "testgracefulexitstop/TestGracefulExitStopLocal.skel.hpp"

namespace TestExitStop
{

class CCM_Client_impl;
    

class test_graceful_exit_stop : public CppUnit::TestFixture
{
    CCM_Client_impl * m_ccm_client;
    ::TestExitStop::CCM_Client_Context_var m_session_context;
    TestExitStop::Client_var m_component_ref;
    
public:
    test_graceful_exit_stop();

    //
    // Testable operations
    //
    void do_tests();

    
    // register test suite and test operation
    CPPUNIT_TEST_SUITE( test_graceful_exit_stop );
    CPPUNIT_TEST( do_tests );
    CPPUNIT_TEST_SUITE_END();

};
    
    
    
//
// IDL:thalesgroup.com/TestComponentContext/Session_CCM_Client:1.0
//
class CCM_Client_impl : virtual public ::TestExitStop::Session_CCM_Client,
                        virtual public Cdmw::OrbSupport::RefCountLocalObject
{
    CCM_Client_impl(const CCM_Client_impl&);
    void operator=(const CCM_Client_impl&);


    ::TestExitStop::CCM_Client_Context_var m_session_context;
     
public:

    CCM_Client_impl();
    ~CCM_Client_impl();
     

    //
    // Session component specific operations
    //
    void set_session_context(Components::SessionContext_ptr ctx)
        throw(CORBA::SystemException);
    void ccm_activate()
        throw(CORBA::SystemException);
    void ccm_passivate()
        throw(CORBA::SystemException);
    void ccm_remove()
        throw(CORBA::SystemException);

    //
    // return last received event (or nil if already returned)
    //
    CdmwPlatformMngt::Event* get_last_event();


    void CCM_Client_impl::notify(CdmwPlatformMngt::Event* event)
       throw(CORBA::SystemException);

private:

    ::TestExitStop::CCM_Client_Context* get_context();
    
    Cdmw::TestUtils::ThreadedTestRunner* m_test_thread;
    
    Cdmw::OsSupport::Mutex                   m_mutex;

    std::queue<CdmwPlatformMngt::Event*> m_event_queue;
    
    friend class test_graceful_exit_stop;
};

}; // End of namespace TestExitStop


#endif // ___CCM_TEST_EXIT_STOP_CCM_CLIENT_IMPL_HPP__

