/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef ___CCM_TEST_EXIT_STOP_CCM_SERVER_IMPL_HPP__
#define ___CCM_TEST_EXIT_STOP_CCM_SERVER_IMPL_HPP__

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <testgracefulexitstop/TestGracefulExitStopLocal.skel.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/osthreads/Mutex.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtManagedElementContexts.stub.hpp>


namespace TestExitStop
{

class CCM_Server_impl;

class test_process_context : public CppUnit::TestFixture
{
    CCM_Server_impl * m_ccm_server;
    ::TestExitStop::CCM_Server_Context_var m_session_context;
    TestExitStop::Server_var m_component_ref;

public:
    test_process_context();

    //
    // Testable operations
    //
    void do_tests();


    // register test suite and test operation
    CPPUNIT_TEST_SUITE( test_process_context );
    CPPUNIT_TEST( do_tests );
    CPPUNIT_TEST_SUITE_END();

};

    

//
// IDL:thalesgroup.com/TestComponentContext/Session_CCM_Server:1.0
//
class CCM_Server_impl : virtual public ::TestExitStop::Session_CCM_Server,
                        virtual public Cdmw::OrbSupport::RefCountLocalObject
{
public:

    CCM_Server_impl();
    ~CCM_Server_impl();
     
    //
    // facet test_component_context
    //
    virtual ::TestExitStop::CCM_TestGracefulExitStop_ptr get_test_graceful_exit_stop ()
        throw (CORBA::SystemException);
    

    //
    // TestComponentContext facet
    //
    virtual CORBA::Boolean test_exit()
      throw (CORBA::SystemException);
    
    virtual CORBA::Boolean test_sleep()
      throw (CORBA::SystemException);
    
  
    //
    // Session component specific operations
    //
    void set_session_context(Components::SessionContext_ptr ctx)
        throw(CORBA::SystemException);
    void ccm_activate()
        throw(CORBA::SystemException);
    void ccm_passivate()
        throw(CORBA::SystemException);
    void ccm_remove()
        throw(CORBA::SystemException);


    ::TestExitStop::CCM_Server_Context* get_context();

    void cdmw_initialise_component(const CdmwPlatformMngtBase::StartupKind&) throw (CdmwPlatformMngt::ManagedComponent::BadOrder);

    std::string m_message;
    std::string m_component_name;

private:
    
    CCM_Server_impl(const CCM_Server_impl&);
    void operator=(const CCM_Server_impl&);

    // session context
    ::TestExitStop::CCM_Server_Context_var m_session_context;
    
    friend class test_process_context;

    Cdmw::TestUtils::ThreadedTestRunner* m_test_thread;

    bool  m_cdmw_init_comp_called;
 
    CdmwPlatformMngt::ManagedElementContexts::ProcessContext_var m_process_context;
 
};

}; // End of namespace TestCcmContext


#endif // ___CCM_TEST_EXIT_STOP_CCM_SERVER_IMPL_HPP__

