/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CORBA.hpp>

#include <testhostisolation/TestHostIsolation.hpp>
#include "Foundation/common/Options.hpp"
#include "Foundation/common/String.hpp"
#include "Foundation/ossupport/OS.hpp"
#include <string>
#include <sstream>
#include <iostream>
#include <fstream>
#include "orbsvcs/orbsvcs/PortableGroupC.h"
#include "SystemMngt/idllib/CdmwPlatformMngtDaemonController.skel.hpp"
#include <SystemMngt/idllib/CdmwPlatformMngtDaemon.stub.hpp>

#define NB_TEST_THREADS 10
#define NB_CALLS 10
namespace
{
    const std::string NAME_DAEMON_CORBALOC = "CdmwPlatformMngtDaemon";
}

namespace CdmwFT 
{

namespace Test 
{

    
CPPUNIT_TEST_SUITE_REGISTRATION( TestHostIsolation );
 
int findDaemonPid(std::string daemonport);
   
// constructor
TestHostIsolation::TestHostIsolation()
{ 
    m_orb =  CORBA::ORB::_duplicate( Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());
    platform_daemon_port = Cdmw::TestUtils::CORBATestManager::instance()->get_property ("platform_daemon_port");
}

// destructor
TestHostIsolation::~TestHostIsolation()
{
}

void TestHostIsolation::test_single_controller_isolation()
{
    if(0 == strcmp("",platform_daemon_port.c_str())){
        std::cerr << std::endl << "Test error: port of platform daemon is null!!!" << std::endl;
    CPPUNIT_ASSERT(false);
    }
    std::string daemon_corbaloc =  "corbaloc::localhost:"+ platform_daemon_port + "/" + NAME_DAEMON_CORBALOC;
    CORBA::Object_var obj_daemon = m_orb->string_to_object(daemon_corbaloc.c_str());
    if ( CORBA::is_nil( obj_daemon.in() )){
            std::cerr << std::endl << "Test error: can't find the platform daemon!!!" << std::endl;
        CPPUNIT_ASSERT(false);
    }
    CdmwPlatformMngt::Daemon_var m_daemon = CdmwPlatformMngt::Daemon::_narrow(obj_daemon.in());
    CORBA::Object_var object_mon = m_daemon->resolve_local_service("DaemonController");
    if ( CORBA::is_nil( object_mon.in() )){
            std::cerr << std::endl << "Test error: can't find the daemon controller!!!" << std::endl;
        CPPUNIT_ASSERT(false);
    }
    CdmwPlatformMngt::DaemonController_var dm = CdmwPlatformMngt::DaemonController::_narrow(object_mon.in());
    
    if (1 == dm->is_isolated()){
        //if the host is isolated
        CPPUNIT_ASSERT(false);
    }
    
    findDaemonPid(platform_daemon_port);
    std::cerr << std::endl << "Remove the network cable and wait the isolation detection!!!" << std::endl;
 
    while(dm->is_isolated() != 1){
        Cdmw::OsSupport::OS::sleep(100);    
    }
    Cdmw::OsSupport::OS::Timeval timeend = Cdmw::OsSupport::OS::get_time();
    std::ostringstream os;
    os << "Isolated at time " << std::endl << " - Second: " << timeend.seconds << std::endl << " - microsecond: " << timeend.microseconds  << std::endl;
    std::cerr << std::endl << os.str() << std::endl;
    
    std::string file_name = "/tmp/CdmwDaemon_" + platform_daemon_port + "/__CDMW_DAEMON__.pid";
    int check = 11;
    while(Cdmw::OsSupport::OS::file_exists(file_name) && check>0 ){
        check--;
    }
    Cdmw::OsSupport::OS::Timeval timeend2 = Cdmw::OsSupport::OS::get_time();
    std::ostringstream os2;
    os2 << "End of check at  " << std::endl << " - Second: " << timeend2.seconds << std::endl << " - microsecond: " << timeend2.microseconds  << std::endl;
    std::cerr << std::endl << os2.str() << std::endl;
    unsigned long elapsed_sec, elapsed_microsec = 0;
    elapsed_sec = timeend2.seconds - timeend.seconds;
    elapsed_microsec = timeend2.microseconds - timeend.microseconds;
    std::ostringstream os3;
    os3 << "Elapsed time  " << std::endl << " - Second: " << elapsed_sec << std::endl << " - microsecond: " << elapsed_microsec  << std::endl;
    std::cerr << std::endl << os3.str() << std::endl;
    if (elapsed_sec > 0 && elapsed_microsec  > 0){
        CPPUNIT_ASSERT(false);
    }
    if (elapsed_sec > 1){
        CPPUNIT_ASSERT(false);
    }
   
    CPPUNIT_ASSERT(true);
}

//Find the platform daemon in file /tmp/CdmwDaemon_<port>/ __CDMW_DAEMON__.pid
int findDaemonPid(std::string daemonport){
    int daemon_pid = -1;  
    std::string file_name = "/tmp/CdmwDaemon_" + daemonport + "/__CDMW_DAEMON__.pid";
    std::ifstream fin; 
    fin.open(file_name.c_str());
    while(!fin.is_open()) {
        sleep(1);
        fin.open(file_name.c_str());
    }
    std::ifstream is(file_name.c_str());
    std::string objref_str;
    is >> objref_str;
    daemon_pid = atoi(objref_str.c_str());
    return daemon_pid;
} 

} // end namespace Test

} // end namespace CdmwFT

