/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <fstream>
#include <limits>
#include <vector>

#include "testhostmonitor/TestObjs.hpp"
#include "testhostmonitor/ProcObserver_impl.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "SystemMngt/platformlibrary/Iterator.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtHost.stub.hpp"
#include "idllib/CdmwPlatformMngtEntity.stub.hpp"

using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestObjs);

namespace Cdmw
{
  namespace PlatformMngt
  {
    
    TestObjs::TestObjs()
    {
    

    TestObjs::m_initial = true;
    }

    void TestObjs::setUp ()
    {
    TestObjs::one_shot_init();
    
    }    


    void TestObjs::one_shot_init()
    {

        if (m_initial)
        {
            if ((timescale = Cdmw::TestUtils::get_timescale()) == 0) timescale = 1;
            m_initial = false;
            try
            {
                //Getting system var
                std::string system_corbaloc = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";
                CORBA::ORB_var orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::instance ()->get_ORB ());
                CORBA::Object_var system_obj = orb->string_to_object (system_corbaloc.c_str());
                m_system = CdmwPlatformMngt::System::_narrow (system_obj.in ());
                        
                CORBA::ULong nb_hosts = m_system -> get_number_of_hosts();
                
                if(nb_hosts == 3)
                {
                    m_system -> get_all_hosts(nb_hosts,hosts.out(),rest.out());
                }
                else 
                {
                    TEST_INFO ("Unexpected number of hosts" );
                    CPPUNIT_ASSERT(false);
                }
            }
            catch (const CORBA::SystemException& ex) 
            {
                std::cerr<< ex._name()<<std::endl;
                throw;
            }
    
        }
    }
    
    void TestObjs::test_starting()
    {
        TEST_INFO("test_starting");
        OsSupport::OS::sleep(timescale*1000);
        bool result = check_status ("OK","STARTING","STARTING","STARTING");
        if (!result) CPPUNIT_ASSERT(false);
        TEST_INFO("test_starting successfully ended\n\n");
        OsSupport::OS::sleep(timescale*11000);
    }
    
    void TestObjs::test_synchronised()
    {
        TEST_INFO("test_synchronised");
        TEST_INFO("Please execute the following command on ExternalReference1 :> sudo /sbin/service ntpd start");
        TEST_INFO("Please execute the following command on HOST1              :> sudo /sbin/service ntpd restart");
        TEST_INFO("Please execute the following command on HOST2              :> sudo /sbin/service ntpd restart");
        TEST_INFO("Please execute the following command on HOST3              :> sudo /sbin/service ntpd restart");    
        TEST_INFO("Check with command check_ntp_status.sh until all hosts are synchronized (*)\n");
        
        TEST_INFO("After, press any key to go on...");
        char c; while (!(std::cin >> c)) { OsSupport::OS::sleep(timescale*1000); }    
        
        bool result = check_status ("OK","OK","OK","OK");
        if (!result) CPPUNIT_ASSERT(false);
        TEST_INFO("test_synchronised successfully ended\n\n");
    }

    void TestObjs::test_no_external_reference()
    {
        TEST_INFO("test_no_external_reference");
        TEST_INFO("Please execute the following command on ExternalReference1 :> sudo /sbin/service ntpd stop");
        TEST_INFO("Please execute the following command on HOST1              :> sudo /sbin/service ntpd restart");
        TEST_INFO("Please execute the following command on HOST2              :> sudo /sbin/service ntpd restart");
        TEST_INFO("Please execute the following command on HOST3              :> sudo /sbin/service ntpd restart");
        
        TEST_INFO("Check with command check_ntp_status.sh until all hosts are synchronized (*)\n");
        
        TEST_INFO("After, press any key to go on...");
        char c; while (!(std::cin >> c)) { OsSupport::OS::sleep(timescale*1000); }
        
        bool result = check_status ("FAILED","FAILED","FAILED","FAILED");
        if (!result) CPPUNIT_ASSERT(false);
        TEST_INFO("test_no_external_reference successfully ended\n\n");
    }
    
    void TestObjs::test_stratum_degraded()
    {
        TEST_INFO("test_stratum_degraded");
        CdmwPlatformMngt::Application_var appl = m_system->get_application("CdmwServices");
        CORBA::ULong hc = 1;
        const char * hostname = hosts[hc]->name();
        const std::string proc_name = std::string("HostMonitor3");
        TEST_INFO("Now restarting HostMonitor on HOST2 with Q-142-1-2.xml");
        CdmwPlatformMngt::Process_var proc = appl -> get_process (proc_name.c_str(), hostname);
        proc -> stop(false);
        OsSupport::OS::sleep(timescale*5000);
        appl -> remove_process (proc_name.c_str(), hostname );
        std::string adminArgs("cdmw_platform_admin --system-corbaloc=corbaloc::localhost:22869/CdmwPlatformMngtSupervision --app-add-process CdmwServices Q-142-1-2.xml");
        OsSupport::OS::create_process("cdmw_platform_admin", adminArgs);  
        OsSupport::OS::sleep(timescale*3000);
        CdmwPlatformMngt::Process_var proc2= appl -> get_process (proc_name.c_str(), hostname);
        CdmwPlatformMngtBase::StartupKind_var startupKind = new CdmwPlatformMngtBase::StartupKind(); 
        startupKind->state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
        startupKind->startup_mode = 0;
        startupKind->specified_state_id  = CORBA::string_dup( "" );
        proc2 -> hot_restart(startupKind.in());
        OsSupport::OS::sleep(timescale*3000);
    
        bool result = check_status ("DEGRADED","OK","DEGRADED","OK");
        if (!result) CPPUNIT_ASSERT(false);
        TEST_INFO("test_stratum_degraded successfully ended\n\n");
    }
    
    void TestObjs::test_stratum_failed()
    {
        TEST_INFO("test_stratum_failed");
        const std::string proc_name = std::string("HostMonitor3");
        CdmwPlatformMngt::Application_var appl = m_system->get_application("CdmwServices");
        CORBA::ULong hc = 1;
        const std::string hostname = std::string(hosts[hc]->name());
        TEST_INFO("Now restarting HostMonitor on HOST2 with Q-142-1-3.xml");
        CdmwPlatformMngt::Process_var proc = appl -> get_process (proc_name.c_str(), hostname.c_str());
        proc -> stop(false);
        OsSupport::OS::sleep(timescale*5000);
        appl -> remove_process (proc_name.c_str(), hostname.c_str() );
        std::string adminArgs = std::string("cdmw_platform_admin --system-corbaloc=corbaloc::localhost:22869/CdmwPlatformMngtSupervision --app-add-process CdmwServices Q-142-1-3.xml");
        OsSupport::OS::create_process("cdmw_platform_admin", adminArgs);  
        OsSupport::OS::sleep(timescale*3000);
        CdmwPlatformMngt::Process_var proc2= appl -> get_process (proc_name.c_str(), hostname.c_str());
        CdmwPlatformMngtBase::StartupKind_var startupKind = new CdmwPlatformMngtBase::StartupKind(); 
        startupKind->state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
        startupKind->startup_mode = 0;
        startupKind->specified_state_id  = CORBA::string_dup( "" );
        proc2 -> hot_restart(startupKind.in());
        OsSupport::OS::sleep(timescale*3000);
        
        bool result = check_status ("FAILED","OK","FAILED","OK");
        if (!result) CPPUNIT_ASSERT(false);
        TEST_INFO("test_stratum_failed successfully ended\n\n");
    }

    void TestObjs::test_no_main_server()
    {
        TEST_INFO("test_no_main_server");
        TEST_INFO("Please execute the following command on HOST1              :> sudo /sbin/service ntpd stop");
        TEST_INFO("then execute the following command on HOST2                :> sudo /sbin/service ntpd restart");
        TEST_INFO("Please execute the following command on HOST3              :> sudo /sbin/service ntpd restart");
        TEST_INFO("After, press any key to go on...");
        char c; while (!(std::cin >> c)) { OsSupport::OS::sleep(timescale*1000); }
        
        bool result = check_status ("FAILED","FAILED","FAILED","FAILED");
        if (!result) CPPUNIT_ASSERT(false);
        TEST_INFO("test_no_main_server successfully ended\n\n");
    }    
    
    bool TestObjs::check_status (const std::string& system,
                                 const std::string& host1,
                                 const std::string& host2,
                                 const std::string& host3 )
    {
        bool result = false;
        std::vector <std::string> entity_statuses(3,"");
        char * host_entity_status_info = 0;
        for (CORBA::ULong count = 0; count<3; count++)
        {
            //Get the Host_impl* reference  
            //current_host = m_system -> get_host( hosts[count]->name() ) ;
            
            try 
            {
                    entity_statuses [count] = 
                        std::string ( hosts[count] -> get_entity ("CdmwHostSynchronization") 
                                                    -> get_status (host_entity_status_info) );
            }
            catch (const CdmwPlatformMngt::EntityNotFound &)
            {
                TEST_INFO ("The following host has no CdmwHostSynchronization entity :" << hosts[count]->name()) ;
                CPPUNIT_ASSERT(false);
            }
            catch (const CORBA::SystemException & ex)
            {
                TEST_INFO ("Unexpected exception has occurred" << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex) ) ;
                CPPUNIT_ASSERT(false);
            }
        }
        char * sys_entity_status_info=0;
            
        std::string sys_entity_status = std::string ( m_system -> get_entity ("CdmwSystemSynchronization") -> get_status (sys_entity_status_info) );
        if ( ( system == sys_entity_status ) &&
            ( host1 == entity_statuses[0] ) &&
            ( host2 == entity_statuses[1] ) &&
            ( host3 == entity_statuses[2] ) )
        {
            result = true;
        }
        
            TEST_INFO ("Expected statuses (system, host1, host2, host3): " << system <<" " << host1 << " " << host2 << " " << host3);
            TEST_INFO ("Actual statuses (system, host1, host2, host3): " << sys_entity_status << " "
                                                                        << entity_statuses[0] << " " 
                                                                        << entity_statuses[1] << " "
                                                                        << entity_statuses[2] );  
            return result;
    }
    
        
    
}/* End namespace PlatformMngt */
}/* End namespace Cdmw */





