/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/Exception.hpp>
#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"
#include <SystemMngt/idllib/CdmwPlatformMngtSupervisionObserver.stub.hpp>
#include <FaultTolerance/ftreplicationmanager/InitUtils.hpp>
#include <FaultTolerance/idllib/FT.stub.hpp>

#include <SystemMngt/platformvaluetypes/ConfigurationChange_impl.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtSupervisionObserver.stub.hpp>
#include "Foundation/common/System.hpp"

#include "testmultiplenotifications/SupervisionObserver_impl.hpp"


// ADDED
#include <SystemMngt/platformvaluetypes/HostStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/ApplicationStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/EntityStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/SystemStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/SystemModeChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/ApplicationModeChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/PropertyChange_impl.hpp>
// ADDED

using namespace Cdmw;
using namespace Common;
using namespace PlatformMngt;
using namespace CdmwPlatformMngt;
using namespace std;

string REGISTER_OPTION = "--register";
string UNREGISTER_OPTION = "--unregister";
string OBSERVER_NAME_OPTION = "--observer-name";
string OBSERVER_FOR_ALL = "--observer-for-all";
string DEPTH = "--depth";
string OBJ_TYPE = "--so-type";

int
main (int argc, char *argv[])
{
  CORBA::ValueFactoryBase_var factory;
  CdmwPlatformMngt::Application_ptr application_test = 0;
  CdmwPlatformMngt::Process_ptr process_test = 0;
  CdmwPlatformMngt::Host_var host_test = 0;
  CdmwPlatformMngt::Entity_var system_entity_test = 0;
  CdmwPlatformMngt::Entity_var application_entity_test = 0;
  CdmwPlatformMngt::Entity_var process_entity_test = 0;
  CdmwPlatformMngt::Entity_var host_entity_test = 0;
  string observer_name, observer_unregister;


  try
  {
    // Check command line arguments
    if ((argc != 5)&&(argc != 4))
      {
    cerr << "Usage: " << REGISTER_OPTION <<
      endl;
    cerr << "       " << OBSERVER_NAME_OPTION << "=observer_name" << endl;
    cerr << "       " << DEPTH << "=yes/no" << endl;
    cerr << "       " << OBJ_TYPE <<
    "s-System, a-Application, p-Process, h-Host, e-Entity (System)"
    << endl<< " or " << endl<<
        UNREGISTER_OPTION<< endl;
        cerr << "       " << OBSERVER_NAME_OPTION << "=observer_name" << endl;
        cerr << "       " << OBJ_TYPE <<
        "s-System, a-Application, p-Process, h-Host, e-Entity (System)";
    cerr << endl;
    throw 0;
      }

    Cdmw::OrbSupport::StrategyList orb_strategies;
    orb_strategies.add_OrbThreaded ();
    orb_strategies.add_PoaThreadPool (10);

    CORBA::ORB_var orb;
    orb = Cdmw::OrbSupport::OrbSupport::ORB_init (argc, argv, orb_strategies);

    CORBA::Object_var poa = orb->resolve_initial_references ("RootPOA");

    PortableServer::POA_var rootPOA =
      PortableServer::POA::_narrow (poa.in ());
    // activate the POA manager
    PortableServer::POAManager_var manager =
      PortableServer::POAManager::_duplicate (rootPOA->the_POAManager ());

    manager->activate ();

    CORBA::ValueFactoryBase_var factory, old_factory;
    factory = new ConfigurationChangeFactory;

    old_factory =
      orb->
      register_value_factory
      ("IDL:thalesgroup.com/CdmwPlatformMngt/ConfigurationChange:1.0",
       factory.in ());

// ADDED to test also Property Events and Status Events Api 
    
    factory = new PropertyChangeFactory;
    old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/PropertyChange:1.0",factory.in() );

    factory = new ApplicationStatusChangeFactory;
    old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationStatusChange:1.0",factory.in
() );

    factory = new ApplicationModeChangeFactory;
    old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationModeChange:1.0",factory.in()
 );

    factory = new ProcessStatusChangeFactory;
    old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/ProcessStatusChange:1.0",factory.in() )
;

    factory = new SystemStatusChangeFactory;
    old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/SystemStatusChange:1.0",factory.in() );

    factory = new SystemModeChangeFactory;
    old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/SystemModeChange:1.0",factory.in() );

    factory = new HostStatusChangeFactory;
    old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/HostStatusChange:1.0",factory.in() );

    factory = new EntityStatusChangeFactory;
    old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/EntityStatusChange:1.0",factory.in() );
    /*factory = new SystemEntityStatusChangeFactory();
    old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/SystemEntityStatusChange:1.0",factory.in() );
    factory = new ProcessEntityStatusChangeFactory();
    old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/ProcessEntityStatusChange:1.0",factory.in() );
    factory = new ApplicationEntityStatusChangeFactory();
    old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationEntityStatusChange:1.0",factory.in() );
    factory = new HostEntityStatusChangeFactory();
    old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/HostEntityStatusChange:1.0",factory.in() );*/

// ADDED



    CdmwPlatformMngt::System_var system_test;
    /*Create an Instance to Client Observer */

    SupervisionObserver_impl obstoreg;

    SupervisionObserver_var observer = obstoreg._this ();

    observer_unregister =
      OsSupport::OS::get_option_value (argc, argv, UNREGISTER_OPTION);
    observer_name =
      OsSupport::OS::get_option_value (argc, argv, OBSERVER_NAME_OPTION);


    std::cerr << std::endl;
    std::cerr << "<<<<<                                  >>>>>" << std::endl;

    if (observer_unregister == "no")
      std::
    cerr << "<<<<<   ObserverExtended Registered   >>>>>" << std::
    endl;
    else
      std::
    cerr << "<<<<<   ObserverExtended Unregistered  >>>>>" << std::
    endl;

    CORBA::Object_var objref;


    //define the corbaloc
    const char *corbaloc_str = "corbaloc::localhost:21871/CdmwPlatformMngtSupervision";
    objref = orb->string_to_object(corbaloc_str);

    //resolve the System_ptr
    system_test = CdmwPlatformMngt::System::_narrow(objref.in() );

    char *hostname = getenv ("HOSTNAME");

    host_test = system_test->get_host (hostname);


    if (CORBA::is_nil (system_test.in ()))
      {
    // TEST_INFO ("Error in narrowing CdmwPlatformMngt::System");
    exit (-1);
      }

    /*select application supervised object */

    application_test = system_test->get_application ("APPL1");
    process_test = application_test->get_process ("P11", hostname);
    system_entity_test = system_test->get_entity ("SYSTEM_ENTITY");
/*    application_entity_test = system_test->get_entity ("APPLICATION_ENTITY");
    process_entity_test = system_test->get_entity ("PROCESS_ENTITY");
    host_entity_test = system_test->get_entity ("HOST_ENTITY"); 
*/

    CdmwPlatformMngt::Children_var children = new CdmwPlatformMngt::Children;

    CdmwPlatformMngt::RegistrationDepth depth = CdmwPlatformMngt::OBJECT_ONLY;
    if (OsSupport::OS::get_option_value (argc, argv, DEPTH) == "yes")
      depth = CdmwPlatformMngt::FULL_HIERARCHY;

    if (OsSupport::OS::get_option_value (argc, argv, REGISTER_OPTION) ==
    "yes")
      {
    string str = OsSupport::OS::get_option_value (argc, argv, OBJ_TYPE);
    char ch = str[0];
    if (ch == 's')
      {
        system_test->register_observer_ext (observer_name.c_str (),
                        depth, observer.in ());
            std::cerr << " Observer registered on System " << std::endl;
      }
    if (ch == 'a')
          { 
        application_test->register_observer_ext (observer_name.c_str (),
                             depth, observer.in ());
            std::cerr << " Observer registered on Application " << std::endl;
      }
    if (ch == 'p')
          {
        process_test->register_observer_ext (observer_name.c_str (),
                         depth, observer.in ());
            std::cerr << " Observer registered on Process " << std::endl;
          }
    if (ch == 'e')
          {
        system_entity_test->register_observer_ext (observer_name.c_str (),
                        depth, observer.in ());
            /*application_entity_test->register_observer_ext (observer_name.c_str (),
                                                depth, observer.in ());
            process_entity_test->register_observer_ext (observer_name.c_str (),
                                                depth, observer.in ());
            host_entity_test->register_observer_ext (observer_name.c_str (),
                                                depth, observer.in ());
            std::cerr << " Observer registered on all Entities of the Supervised Object " << std::endl;*/
          }
         if (ch =='h')
      {
        host_test->register_observer_ext (observer_name.c_str (), depth,
                          observer.in ());
            std::cerr << " Observer registered on Host " << std::endl;
      }
      }


    if (OsSupport::OS::get_option_value (argc, argv, UNREGISTER_OPTION) ==
    "yes")
      {
    string str = OsSupport::OS::get_option_value (argc, argv, OBJ_TYPE);
    char ch = str[0];
      if (ch == 's')
      {
        system_test->unregister_observer_ext (observer_name.c_str ());
            std::cerr << " Observer unregisted on System " << std::endl;
          }
      if (ch== 'a')
      { 
        application_test->unregister_observer_ext (observer_name.
                               c_str ());
            std::cerr << " Observer unregisted on Application " << std::endl;
          }  
      if (ch == 'p')
      {
        process_test->unregister_observer_ext (observer_name.c_str ());
            std::cerr << " Observer unregisted on Process " << std::endl;
          }
      if (ch == 'e')
          { 
        system_entity_test->unregister_observer_ext (observer_name.c_str ());
            /*application_entity_test->unregister_observer_ext (observer_name.c_str ());
            process_entity_test->unregister_observer_ext (observer_name.c_str ());
            host_entity_test->unregister_observer_ext (observer_name.c_str ());
            std::cerr << " Observer unregisted on all Entities " << std::endl;*/
          }
      if (ch== 'h')
      {
        host_test->unregister_observer_ext (observer_name.c_str ());
            std::cerr << " Observer unregisted on Host " << std::endl;
          } 
      }

      
    
    //orb run only if observer registration is required 
    if (OsSupport::OS::get_option_value (argc, argv, REGISTER_OPTION) ==
    "yes")
      orb->run ();

  }
  catch (const CORBA::Exception & eCBex)
  {
    cout << "CORBA Exception :" << eCBex._name () << endl;
    return -1;
  }

  cout << "Exit process..." << endl;




}
