/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "testpreconhostgraphs/HostGraphObserver_impl.hpp"

#include <iostream>
#include <iomanip>
#include <typeinfo>
#include <fstream>

#include "Foundation/ossupport/OS.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/common/Options.hpp"
#include "Foundation/common/Exception.hpp"

#include "SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/SystemStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/HostStatusChange_impl.hpp"

#include "idllib/CdmwPlatformMngtSystem.stub.hpp"


namespace Cdmw
  {

  namespace PlatformMngt
    {

    HostGraphObserver_impl::HostGraphObserver_impl(std::string host_name)
    {
        m_host_name = host_name;
        m_checker = new HostGraphChecker();
    }

    HostGraphObserver_impl::~HostGraphObserver_impl()
    {
        delete m_checker;
    }


    void  HostGraphObserver_impl::notify(CdmwPlatformMngt::Event* an_event)
    throw( CORBA::SystemException )
    {
       try
       {

           CdmwPlatformMngt::ProcessStatusChange* prc_chg = dynamic_cast<CdmwPlatformMngt::ProcessStatusChange*> (an_event);

           if ((prc_chg!=0))
           {
                   analyse_event (prc_chg);
           }
/*
           CdmwPlatformMngt::SystemStatusChange* sys_chg = dynamic_cast<CdmwPlatformMngt::SystemStatusChange*> (an_event);

           if ((sys_chg!=0))
           {
               if ( CdmwPlatformMngt::SYSTEM_STARTED == sys_chg->system_status())
//               || CdmwPlatformMngt::SYSTEM_PARTIAL_STARTED == sys_chg->system_status())
               {
                  print_test_results();
               }
           }
*/
           CdmwPlatformMngt::HostStatusChange* host_chg = dynamic_cast<CdmwPlatformMngt::HostStatusChange*> (an_event);

           if ((host_chg!=0))
           {

              CdmwPlatformMngt::ElementPath_var element_path = host_chg->element_path();

              std::string elementPath (element_path);

              std::string partials = "";

              std::string process_name = "";

              std::string host_name = "";

              size_t position = elementPath.find("/");

              if ( position != std::string::npos)
              {
                 partials = elementPath.substr(position+1, elementPath.length());

                 elementPath = partials;

                 position = elementPath.find ("/");

              }

              host_name = elementPath.substr(0, elementPath.find (".") );

              if (host_name == m_host_name)
              {
                  if ( CdmwPlatformMngt::HOST_STARTED == host_chg->host_status())
                  {
                      print_test_results();
                  }
              }
           }

        }
        catch (const std::exception& ex)
        {
            std::cerr << " Process observer exception  " << ex.what();
        }
    }

    void HostGraphObserver_impl::analyse_event (CdmwPlatformMngt::ProcessStatusChange* processStatusChange)
    {
         CdmwPlatformMngt::ElementPath_var element_path = processStatusChange->element_path();

         std::string elementPath (element_path);

         std::string partials = "";

         std::string process_name = "";

         std::string host_name = "";

         size_t position = elementPath.find("/");

         for (int index = 0; index < 2; index++)
         {
             if ( position != std::string::npos)
             {
                 partials = elementPath.substr(position+1, elementPath.length());

                 elementPath = partials;

                 position = elementPath.find ("/");

             }
         }

         host_name = elementPath.substr(0, position - 5 );

         partials = elementPath.substr(position+1, elementPath.length());

         process_name = partials.substr(0, partials.find(".") );

         if (host_name == m_host_name)
         {
             CdmwPlatformMngt::ProcessStatus process_status =  processStatusChange->process_status();

             CdmwPlatformMngtBase::EventHeader event_header = processStatusChange->header();

             TimeStampInfo time_stamp = {event_header.time_stamp.year, event_header.time_stamp.month,
                 event_header.time_stamp.day, event_header.time_stamp.hour, event_header.time_stamp.minute,
                 event_header.time_stamp.second, event_header.time_stamp.millisecond, 
                 event_header.time_stamp.microsecond};

             ProcessEventInfo other_event_info =
                 { process_name, process_status, time_stamp };

             m_checker->submit_event(other_event_info);
         }
    }

    void HostGraphObserver_impl::print_test_results()
    {
        std::list<std::string> results = m_checker->get_streaming_result();

        std::list<std::string>::iterator results_it = results.begin();

        for (; results_it != results.end(); ++results_it)
        {
            std::cerr << "****** "<< *results_it << " ******" <<std::endl;
        }

    }

  } // PlatformMngt
} // Cdmw

