/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "testpreconhostgraphs/HostGraphsChecker.hpp"

namespace Cdmw
  {

  namespace PlatformMngt
    {

        HostGraphChecker::HostGraphChecker()
        {

           WAITING_PROCESS_NAME = "WaitingProcess";

           SUCCESSOR_PROCESS_NAME = "ProcessSuccessor";

           FIRST_WAITED_PROCESS_NAME = "WaitedProcess1";

           SECOND_WAITED_PROCESS_NAME = "WaitedProcess2";

           MSG_TITLE = " Test Results ";

           MSG_SUCCESSORS_TEST = " Test Process successor constrain test :  ";

           MSG_PRECONS_TEST = " Test Precondition contrain test : ";

           RESULT_OK = " OK ";

           RESULT_NOT_OK = " NOT OK ";

           threshold_status = CdmwPlatformMngt::PROCESS_INITIALISING;

           successor_test_result = false;

           first_precon_test_result = false;

           second_precon_test_result = false;

            for (int index = 0; index < 4; index++)
            {
                TimeStampInfo time_stamp = {0, 0, 0, 0, 0, 0, 0, 0};
                time_stamps[index] = time_stamp;
            }
        }

        HostGraphChecker::~HostGraphChecker()
        {
        }

       void HostGraphChecker::submit_event (ProcessEventInfo processEventInfo)
       {

            if (threshold_status == processEventInfo.process_status)
            {

                if (WAITING_PROCESS_NAME == processEventInfo.process_name)
                {

                    time_stamps [0] = processEventInfo.time_stamp;
/*
                    std::cerr << " ######## First waited process " << 
                    time_stamps[1].year << " " << 
                    time_stamps[1].month << " " << 
                    time_stamps[1].day << " " << 
                    time_stamps[1].hour << " " << 
                    time_stamps[1].minute << " " << 
                    time_stamps[1].second << " " << 
                    time_stamps[1].millisecond << " " << 
                    time_stamps[1].microsecond << " " 
                    << std::endl;

                    std::cerr << " ######## Second waited process " << 
                    time_stamps[2].year << " " << 
                    time_stamps[2].month << " " << 
                    time_stamps[2].day << " " << 
                    time_stamps[2].hour << " " << 
                    time_stamps[2].minute << " " << 
                    time_stamps[2].second << " " << 
                    time_stamps[2].millisecond << " " << 
                    time_stamps[2].microsecond << " " 
                    << std::endl;

                    std::cerr << " ######## Waiting process " << 
                    time_stamps[0].year << " " << 
                    time_stamps[0].month << " " << 
                    time_stamps[0].day << " " << 
                    time_stamps[0].hour << " " << 
                    time_stamps[0].minute << " " << 
                    time_stamps[0].second << " " << 
                    time_stamps[0].millisecond << " " << 
                    time_stamps[0].microsecond << " " 
                    << std::endl;

                    std::cerr << " ######## Successor process " << 
                    time_stamps[3].year << " " << 
                    time_stamps[3].month << " " << 
                    time_stamps[3].day << " " << 
                    time_stamps[3].hour << " " << 
                    time_stamps[3].minute << " " << 
                    time_stamps[3].second << " " << 
                    time_stamps[3].millisecond << " " << 
                    time_stamps[3].microsecond << " "
                    << std::endl;
*/
                    if (0 != time_stamps[1].hour)
                    {
                       first_precon_test_result = 
                          compareTimeStamps(processEventInfo.time_stamp, time_stamps[1]);
                    }
                    if (0 != time_stamps[2].hour)
                    {
                       second_precon_test_result = 
                          compareTimeStamps(processEventInfo.time_stamp, time_stamps[2]);
                    }
                    if (0 != time_stamps[3].hour)
                    {
                       successor_test_result = 
                          compareTimeStamps(time_stamps[3], processEventInfo.time_stamp);
                    }
                }
                else if (SUCCESSOR_PROCESS_NAME == processEventInfo.process_name)
                {
                    time_stamps [3] = processEventInfo.time_stamp;

                    if (0 != time_stamps[0].hour)
                    {
                       successor_test_result = 
                          compareTimeStamps(processEventInfo.time_stamp, time_stamps[0]);
                    }

                }
                else if (FIRST_WAITED_PROCESS_NAME == processEventInfo.process_name)
                {
                    time_stamps [1] = processEventInfo.time_stamp;

                    if (0 != time_stamps[0].hour)
                    {
                       first_precon_test_result = 
                          compareTimeStamps(time_stamps[0], processEventInfo.time_stamp);
                    }

                }
                else if (SECOND_WAITED_PROCESS_NAME == processEventInfo.process_name)
                {
                    time_stamps [2] = processEventInfo.time_stamp;

                    if (0 != time_stamps[0].hour)
                    {
                       second_precon_test_result = 
                          compareTimeStamps(time_stamps[0], processEventInfo.time_stamp);
                    }
                }

            }
       }

       std::list<std::string> HostGraphChecker::get_streaming_result()
       {
           std::list<std::string> results;
           results.push_back(MSG_TITLE);
           results.push_back(MSG_PRECONS_TEST);

           if (first_precon_test_result)
           {
               results.push_back(RESULT_OK);
           }
           else
           {
               results.push_back(RESULT_NOT_OK);
           }

           if (second_precon_test_result)
           {
               results.push_back(RESULT_OK);
           }
           else
           {
               results.push_back(RESULT_NOT_OK);
           }

           results.push_back(MSG_SUCCESSORS_TEST);

           if (successor_test_result)
           {
               results.push_back(RESULT_OK);
           }
           else
           {
               results.push_back(RESULT_NOT_OK);
           }

           return results;
       }

        bool HostGraphChecker::compareTimeStamps (TimeStampInfo first, TimeStampInfo second)
        {
             if (first.year > second.year)
             {
                 return true;
             }
             else
             {
                 if (first.month > second.month)
                 {
                     return true;
                 }
                 else
                 {
                     if (first.day > second.day)
                     {
                         return true;
                     }
                 }
             }

             if(first.hour > second.hour)
             {
                   return true;
             }
             else
             {
                  if(first.minute > second.minute)
                  {
                      return true;
                  }
                  else
                  {
                      if(first.second > second.second)
                      {
                         return true;
                      }
                  }
             }

             if (first.millisecond > second.millisecond)
             {
                 return true;
             }
             else
             {

                 if(first.microsecond >= second.microsecond)
                 {
                    return true;
                 }

             }


             return false;
        }

    } // PlatformMngt
   } // Cdmw
