/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/common/Options.hpp"
#include "Foundation/common/String.hpp"
#include "Foundation/orbsupport/CORBA.hpp"

#include "Foundation/ossupport/OS.hpp"
#include "Foundation/osthreads/ThreadHandle.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"

#include "testpreconhostgraphs/HostGraphObserver_impl.hpp"
#include "testpreconhostgraphs/HostGraphsChecker.hpp"

#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"

#include <Foundation/common/Exception.hpp>
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtHost.stub.hpp"

#include "idllib/CdmwPlatformMngtDaemon.stub.hpp"

#include "SystemMngt/platformvaluetypes/SystemModeChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/SystemStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/HostStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationAgentDeath_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationModeChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/EntityStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/PropertyChange_impl.hpp"
#include <SystemMngt/platformvaluetypes/Restart_impl.hpp>
#include <SystemMngt/platformvaluetypes/Acknowledge_impl.hpp>
#include <SystemMngt/platformvaluetypes/Restart_impl.hpp>
#include <SystemMngt/platformvaluetypes/Reboot_impl.hpp>
#include <SystemMngt/platformvaluetypes/Shutdown_impl.hpp>
#include <SystemMngt/platformvaluetypes/Stop_impl.hpp>
#include "SystemMngt/platformvaluetypes/UserEvent_impl.hpp"
#include "SystemMngt/platformvaluetypes/ComponentStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/RedundancyGroupStatusChange_impl.hpp"

#include <cstdlib>
#include <string>
#include <exception>
#include <sstream>
#include <iostream>
#include <fstream>

const int POA_THREAD_POOL_SIZE = 5;

namespace
{
    using namespace CdmwPlatformMngt;
    using namespace Cdmw::OsSupport;
    using namespace Cdmw::OrbSupport;
    using namespace Cdmw::PlatformMngt;

    std::string applicationName;    
    std::string processName;

    const int time_before_main_thread_dies = 3000; 
}


void exceptionHandler(void)
{
    std::cout << "Unexpected Exception : Test failed" << std::endl;
}


void exitHandler(void)
{
    std::cout << "Process '" << processName << "' exited" << std::endl;
}

int main(int argc, char* argv[])
{  
    int ret_code = 0;

    try
    {
        std::set_unexpected(&exceptionHandler);

        ::atexit(&exitHandler);

        StrategyList strategy;
        strategy.add_OrbThreaded();
        strategy.add_PoaThreadPool(POA_THREAD_POOL_SIZE);

        CORBA::ORB_var orb =  Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, strategy); 

        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");

        PortableServer::POA_var poa = PortableServer::POA::_narrow(obj.in());

        PortableServer::POAManager_var poaMgr = poa->the_POAManager();

        poaMgr->activate();

        try
        {

            sleep(1);

            std::cerr << "******* initialise the process *******" << std::endl;

            CdmwPlatformMngt::ElementNames_var elems;
            CdmwPlatformMngt::ElementNamesIterator_var it;

            std::cerr << " Getting system var " << std::endl;
            std::string system_corbaloc = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";

            CORBA::Object_var system_obj = orb->string_to_object (system_corbaloc.c_str());

            if (CORBA::is_nil(system_obj)) 
            {
                std::cerr << " System_obj nil " << std::endl;
                return -1;
            }

            std::cerr << "******* retrieving system reference *******" << std::endl;
            CdmwPlatformMngt::System_var system = CdmwPlatformMngt::System::_narrow (system_obj.in ());

            if (CORBA::is_nil(system)) 
            {
                std::cerr << " System nil " << std::endl;
                return -2;
            }

            CORBA::ULong nbr_of_host;

            try {

                 nbr_of_host = system->get_number_of_hosts();

            } catch (const CORBA::TRANSIENT& excTrans) {

                 std::cerr << "  CORBA::TRANSIENT " << std::endl;
                 std::cerr << "  CORBA::TRANSIENT: " << excTrans << std::endl;
                 throw;

            } catch (const CORBA::SystemException& excS) {

                 std::cerr << "  CORBA::SystemException: " << std::endl;
                 std::cerr << "  Exception: "+std::string(excS._name()) << std::endl;
                 throw;

            }  catch (const Cdmw::Exception& exc) {

                 std::cerr << "  C++ Exception: " << std::endl;
                 std::cerr << "  Exception: "+std::string(exc.what()) << std::endl;
                 throw;

            } catch (...) {

                 std::cerr << "  Generic Exception " << std::endl;
                 throw;
            }

            // get the name of hosts
            system->get_all_host_names(nbr_of_host, elems.out(), it.out());

            //     const char* host_name1 = elems[0U];
            const char* host_name2 = elems[1U];

            std::string path_string = std::string(system->name())+".sys/"+std::string(host_name2)+".host";

            CdmwPlatformMngt::ElementPath_var host_path (path_string.c_str());

            ManagedElement_var element_host (system->get_element (host_path.in()));

            std::cerr << "******* retrieving "<< std::string(host_name2) <<" host reference *******" << std::endl;
            CdmwPlatformMngt::Host_var second_host = CdmwPlatformMngt::Host::_narrow(element_host);

            if (CORBA::is_nil(second_host)) 
            {
                std::cerr << " Second host nil " << std::endl;
                return -3;
            }

            // Register event factories
            CORBA::ValueFactoryBase_var factory, old_factory;

            factory = new SystemModeChangeFactory;
            old_factory = orb->register_value_factory(
                    "IDL:thalesgroup.com/CdmwPlatformMngt/SystemModeChange:1.0",
                    factory.in() );

            factory = new SystemStatusChangeFactory;
            old_factory = orb->register_value_factory(
                    "IDL:thalesgroup.com/CdmwPlatformMngt/SystemStatusChange:1.0",
                    factory.in() );

            factory = new HostStatusChangeFactory;
            old_factory = orb->register_value_factory(
                    "IDL:thalesgroup.com/CdmwPlatformMngt/HostStatusChange:1.0",
                    factory.in() );
/*
            factory = new ApplicationAgentDeathFactory;
            old_factory = orb->register_value_factory(
                    "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationAgentDeath:1.0",
                    factory.in() );

            factory = new ApplicationModeChangeFactory;
            old_factory = orb->register_value_factory(
                    "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationModeChange:1.0",
                    factory.in() );

            factory = new ApplicationStatusChangeFactory;
            old_factory = orb->register_value_factory(
                    "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationStatusChange:1.0",
                    factory.in() );
*/
            factory = new ProcessMessageFactory;
            old_factory = orb->register_value_factory(
                    "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessMessage:1.0",
                    factory.in() );

            factory = new ProcessStatusChangeFactory;
            old_factory = orb->register_value_factory(
                    "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessStatusChange:1.0",
                    factory.in() );
/*
            factory = new EntityStatusChangeFactory;
            old_factory = orb->register_value_factory(
                    "IDL:thalesgroup.com/CdmwPlatformMngt/EntityStatusChange:1.0",
                    factory.in() );

            factory = new UserEventFactory;
            old_factory = orb->register_value_factory(
                    "IDL:thalesgroup.com/CdmwPlatformMngt/UserEvent:1.0",
                    factory.in() );

            factory = new PropertyChangeFactory;
            old_factory = orb->register_value_factory(
                    "IDL:thalesgroup.com/CdmwPlatformMngt/PropertyChange:1.0",
                    factory.in() );

            factory = new ComponentStatusChangeFactory;
            old_factory = orb->register_value_factory(
                    "IDL:thalesgroup.com/CdmwPlatformMngt/ComponentStatusChange:1.0",
                    factory.in() );

            factory = new RedundancyGroupStatusChangeFactory;
            old_factory = orb->register_value_factory(
                    "IDL:thalesgroup.com/CdmwPlatformMngt/RedundancyGroupStatusChange:1.0",
                    factory.in() );

            factory = new RestartFactory;
            old_factory = orb->register_value_factory(
                    "IDL:thalesgroup.com/CdmwPlatformMngt/Restart:1.0",
                    factory.in() );

            factory = new StopFactory;
            old_factory = orb->register_value_factory(
                    "IDL:thalesgroup.com/CdmwPlatformMngt/Stop:1.0",
                    factory.in() );

            factory = new AcknowledgeFactory;
            old_factory = orb->register_value_factory(
                    "IDL:thalesgroup.com/CdmwPlatformMngt/Acknowledge:1.0",
                    factory.in() );

            factory = new ShutdownFactory;
            old_factory = orb->register_value_factory(
                    "IDL:thalesgroup.com/CdmwPlatformMngt/Shutdown:1.0",
                    factory.in() );

            factory = new RebootFactory;
            old_factory = orb->register_value_factory(
                    "IDL:thalesgroup.com/CdmwPlatformMngt/Reboot:1.0",
                    factory.in() );
*/
            HostGraphObserver_impl* observerImpl = new HostGraphObserver_impl(std::string(host_name2));

            CdmwPlatformMngt::SupervisionObserver_var observer = observerImpl->_this();

            std::cerr << "******* registering obsever *******" << std::endl;

            system->register_observer("HostGraphObserver", observer.in() );

             // start orb
            orb->run();

        }
        catch (const CORBA::SystemException& sysExc)
        {
            std::cerr << " SystemException during process initialising " << std::endl;
            throw;
        }
        catch (const CdmwPlatformMngt::ElementAlreadyExists& alreadyExists)
        {
            std::cerr << " ElementAlreadyExists Exception during process initialising " << std::endl;
            throw;
        }
        catch (...) 
        {
            orb->shutdown(false);
            std::cerr << "  Generic Exception during process initialising " << std::endl;
            orb->destroy();
            throw;
        }

        Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
        orb->destroy();

    }
    catch (const CORBA::Exception& e)
    {  
        std::cerr << "FAILURE : CORBA exception raised :" << std::endl
                  << e._name() << std::endl;
        ret_code = EXIT_FAILURE;
    }
    catch(const std::exception &e)
    {
        std::cerr << "FAILURE : Exception raised : " << std::endl
                  << e.what() << std::endl;
        ret_code = EXIT_FAILURE;
    }
    catch(...)
    {
        std::cerr << "FAILURE : Unexpected exception" << std::endl;
        ret_code = EXIT_FAILURE;
    }

    return ret_code;

}
