/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <testproccompconsistency/CCM_Server_impl.hpp>

#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp>

#include <SystemMngt/platformlibrary/EventHeaderFactory.hpp>


namespace TestProcCompConsistency
{

const std::string SYSTEM_NAME = "S1";
const std::string SYSTEM_NAME_INIT_EXCEPTION = "S2";
const std::string SYSTEM_NAME_ACTIVATE_EXCEPTION = "S3";
const std::string SYSTEM_NAME_CHECK_SERIALIZATION = "S4";
const std::string APPLICATION_NAME = "CcmTestProcessComponentConsistencyApplication";
const std::string CLIENT_PROCESS_NAME = "TestProcessComponentConsistencyProcessClient1";
const std::string SERVER_PROCESS1_NAME = "TestProcessComponentConsistencyProcessServer";

const std::string CLIENT_COMPONENT_NAME = "Client1";
const std::string SERVER_COMPONENT1_NAME = "Server1";

const std::string CLIENT_ISSUER_NAME = "TestProcessComponentConsistencyProcessClient1/Client1";
const std::string SERVER_ISSUER1_NAME = "TestProcessComponentConsistencyProcessServer/Server1";
const std::string SUPERVISION_ISSUER_NAME = "CDMW.PFMMNG";

const CORBA::Long SERVER_EXIT_CODE = 10;

//
// IDL:thalesgroup.com/CcmFtTest/ServerModule/CCM_Server:1.0
//
CCM_Server_impl::CCM_Server_impl(): m_message(""),m_component_name(""), m_test_thread(0), m_cdmw_init_comp_called(0)
{
    m_session_context = TestProcCompConsistency::CCM_Server_Context::_nil();
    m_redundancy_group = CdmwPlatformMngt::External::RedundancyGroup::_nil();
}

CCM_Server_impl::~CCM_Server_impl()
{
    std::cout << "   ... CCM_Server_impl::~CCM_Server_impl called!" << std::endl;
}


void CCM_Server_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_Server_impl::set_session_context called!" << std::endl;
    m_session_context = ::TestProcCompConsistency::CCM_Server_Context::_narrow(ctx);
}

::TestProcCompConsistency::CCM_Server_Context* CCM_Server_impl::get_context()
{
    return ::TestProcCompConsistency::CCM_Server_Context::_duplicate(m_session_context.in());
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CCM_Server_impl::ccm_activate()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_Server_impl::ccm_activate called!" << std::endl;

   m_process_context = CdmwPlatformMngt::ManagedElementContexts::ProcessContext::_nil();
   
   CORBA::Object_var obj =
                m_session_context->resolve_service_reference("ManagedProcessContext");
                m_process_context =
                CdmwPlatformMngt::ManagedElementContexts::ProcessContext::_narrow(obj.in());
            CPPUNIT_ASSERT(!CORBA::is_nil(m_process_context.in()));

    try {
        // save context in CORBATestManager for usage in test
        Cdmw::TestUtils::CORBATestManager::instance()->add(
            "server_context",
            get_context());


        // save this executor in CORBATestManager for usage in test
        Cdmw::TestUtils::CORBATestManager::instance()->add(
            "this_executor",
            dynamic_cast< Session_CCM_Server_ptr > (this));

        // create test runner thread
        m_test_thread = new Cdmw::TestUtils::ThreadedTestRunner(
            Cdmw::TestUtils::CORBATestManager::instance()->get_controller());
        // add test to be run
        m_test_thread->addTest(
            CppUnit::TestFactoryRegistry::getRegistry("test_process_context").makeTest());
        // start test

        m_test_thread->start();
    
    
      
        CORBA::Object_var obj = m_session_context->resolve_service_reference("ManagedComponentContext");

        CdmwPlatformMngt::ManagedElementContexts::ComponentContext_var component_context =
            CdmwPlatformMngt::ManagedElementContexts::ComponentContext::_narrow(obj.in());
            CPPUNIT_ASSERT(!CORBA::is_nil(component_context.in()));

 
         std::string system_corbaloc = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";
         CORBA::ORB_var orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::instance ()->get_ORB ());
         CORBA::Object_var system_obj = orb->string_to_object (system_corbaloc.c_str());
         CdmwPlatformMngt::System_var system = CdmwPlatformMngt::System::_narrow (system_obj.in ());
     
         if(SYSTEM_NAME_ACTIVATE_EXCEPTION == system->name()){

             CdmwPlatformMngt::ManagedElement_var managed_element = component_context->the_managed_element();
             CORBA::String_var managed_element_name=managed_element->name();
        
             std::string server3_name = "Server3noft";
             std::string server2_name = "Server2noft";
       
             if (!strcmp (server3_name.c_str (), managed_element_name.in ())){
       
                 CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, CORBA::COMPLETED_NO);

                 std::cerr<<server3_name<<": Throw Exception during ccm_activate --------"<<std::endl;
                 throw ex;
             }
             else if (!strcmp (server2_name.c_str (), managed_element_name.in ())){
             std::cerr<<server2_name<<": Sleep 15 seconds during ccm_activate -------"<<std::endl;
                 Cdmw::OsSupport::OS::sleep (15000);
             }
         } 

    } 
    catch (const CORBA::SystemException& e) 
    {
        std::cerr << "CORBA Exception exception during test: "
                  << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
                  << std::endl;
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_NO);
        throw ex;
    } 
    catch (const std::exception& e)
    {
        std::cerr << "std::exception exception during test: "
                  << e.what()
                  << std::endl;
    } 
    catch (...) {
        std::cerr << "unknown exception during test !!" << std::endl;
    }

}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
CCM_Server_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_Server_impl::ccm_passivate called!" << std::endl;

} 


//
// ccm_remove function : it destroys the component (call by the container)
//
void
CCM_Server_impl::ccm_remove()
   throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_Server_impl::ccm_remove called!" << std::endl;
}


//
// facet test_processcomponent_consistency
//
::TestProcCompConsistency::CCM_TestProcessComponentConsistency_ptr 
CCM_Server_impl::get_test_processcomponent_consistency()
    throw (CORBA::SystemException)
{
    // component itself implements the test_processcomponent_consistency facet
    return TestProcCompConsistency::CCM_TestProcessComponentConsistency::_duplicate(this);
}


void 
CCM_Server_impl::test_operation(const char * message)
  throw (CORBA::SystemException)
{
    m_message = message;
    std::cout << "CCM_Server_impl::test_operation(): checking CCM_Server_impl::cdmw_initialise_component  on " << message << std::endl;
    sleep(3);

    if (m_test_thread != 0) {
        // wait test to be finished
        m_test_thread->join();
        delete m_test_thread;
        m_test_thread = 0;
        // print test summary
        Cdmw::TestUtils::CORBATestManager::instance()->summary();
    }
    sleep(3);
    std::cout << "**** going to call exit with code "<< SERVER_EXIT_CODE << " ***"<< std::endl;
    try
     {
    //      if (m_message == m_component_name)
              m_process_context->exit(SERVER_EXIT_CODE);
     } catch (const CORBA::SystemException& e) {
        std::cerr << "ERROR during call_notify_user_event() : " <<
                  Cdmw::OrbSupport::OrbSupport::exception_to_string(e) << std::endl;
        std::cerr << "Throw CORBA::INTERNAL" << std::endl;
        throw CORBA::INTERNAL();
    }
   
}


CdmwPlatformMngt::External::RedundancyGroup_ptr CCM_Server_impl::get_server_group() throw (CORBA::SystemException)
{
     return CdmwPlatformMngt::External::RedundancyGroup::_duplicate(m_redundancy_group.in());
}  

void
CCM_Server_impl::cdmw_initialise_component(const CdmwPlatformMngtBase::StartupKind&) throw (CdmwPlatformMngt::ManagedComponent::BadOrder, CORBA::SystemException)
{
    std::cout<<"*** CCM_Server_impl::cdmw_initialise_component() ***" << std::endl;
    m_cdmw_init_comp_called = true;
    std::cout<<"*** CCM_Server_impl::cdmw_initialise_component() testing get_snapshot() ***" << std::endl;

    //retrieve the ProcessContext to perform exit operation
    CORBA::Object_var obj = m_session_context->resolve_service_reference("ManagedComponentContext");

    CdmwPlatformMngt::ManagedElementContexts::ComponentContext_var component_context =
            CdmwPlatformMngt::ManagedElementContexts::ComponentContext::_narrow(obj.in());
            CPPUNIT_ASSERT(!CORBA::is_nil(component_context.in()));

 
    std::string system_corbaloc = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";
    CORBA::ORB_var orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::instance ()->get_ORB ());
    CORBA::Object_var system_obj = orb->string_to_object (system_corbaloc.c_str());
    CdmwPlatformMngt::System_var system = CdmwPlatformMngt::System::_narrow (system_obj.in ());

  
    if(SYSTEM_NAME_INIT_EXCEPTION == system->name()){

        CdmwPlatformMngt::ManagedElement_var managed_element = component_context->the_managed_element();
            CORBA::String_var managed_element_name=managed_element->name();
        
        std::string server3_name = "Server3";
        std::string server2_name = "Server2";
       
        if (!strcmp (server3_name.c_str (), managed_element_name.in ())){
       
            CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, CORBA::COMPLETED_NO);

            std::cerr<<server3_name<<": Throw Exception during cdmw_initialise_component --------"<<std::endl;
            throw ex;
        }
        else if (!strcmp (server2_name.c_str (), managed_element_name.in ())){
            std::cerr<<server2_name<<": Sleep 15 seconds during cdmw_initialise_component -------"<<std::endl;
            Cdmw::OsSupport::OS::sleep (15000);
        }
   } 
   
   if(SYSTEM_NAME_CHECK_SERIALIZATION == system->name()){
    
       Cdmw::OsSupport::OS::Timeval time_val = Cdmw::OsSupport::OS::get_time();
       CdmwPlatformMngtBase::TimeStamp time_stamp = Cdmw::PlatformMngt::EventHeaderFactory::TimeVal_2_TimeStamp(time_val);
       
       CdmwPlatformMngt::ManagedElement_var managed_element = component_context->the_managed_element();
       CORBA::String_var managed_element_name=managed_element->name();
        
       std::string server3_name = "Server3noft";
       std::string server2_name = "Server2noft";
       
       if (!strcmp (server3_name.c_str (), managed_element_name.in ())||!strcmp (server2_name.c_str (), managed_element_name.in ())){
                   
       std::cerr<<"------ cdmw_initialise_component Start time"<<time_stamp.hour <<":"<<time_stamp.minute <<":"<<time_stamp.second <<std::endl;
       std::cerr<<"------ SLEEP 30 seconds";
           Cdmw::OsSupport::OS::sleep (30000);
       }
   } 
    
}  


CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(test_process_context, "test_process_context");


test_process_context::test_process_context()
{
    // retrieve context object
    CORBA::Object_var obj =
        Cdmw::TestUtils::CORBATestManager::instance()->get_object("server_context");
    m_session_context =
        ::TestProcCompConsistency::CCM_Server_Context::_narrow(obj.in());

    obj =
        Cdmw::TestUtils::CORBATestManager::instance()->get_object("this_executor");
    m_ccm_server= dynamic_cast< CCM_Server_impl* > (obj.in());
}


void
test_process_context::do_tests()
{
    std::cerr << "   ... test_process_context::do_tests called!" << std::endl;

    try {   
    
            TEST_INFO("Test cdmw_initialise_component");
            CPPUNIT_ASSERT(m_ccm_server->m_cdmw_init_comp_called); 
            

            //retrieve the ProcessContext to perform exit operation
            CORBA::Object_var obj =
                m_session_context->resolve_service_reference("ManagedProcessContext");
            CdmwPlatformMngt::ManagedElementContexts::ProcessContext_var m_process_context =
                CdmwPlatformMngt::ManagedElementContexts::ProcessContext::_narrow(obj.in());
       
            CPPUNIT_ASSERT(!CORBA::is_nil(m_process_context.in()));
            

            obj = m_session_context->resolve_service_reference("ManagedComponentContext");
        
            CdmwPlatformMngt::ManagedElementContexts::ComponentContext_var component_context =
                 CdmwPlatformMngt::ManagedElementContexts::ComponentContext::_narrow(obj.in());
            CPPUNIT_ASSERT(!CORBA::is_nil(component_context.in()));

            CdmwPlatformMngt::ManagedElement_var managed_element = component_context->the_managed_element();
            
            m_ccm_server->m_component_name = managed_element->name();
            
            if (m_ccm_server->m_component_name!="Server1noft" && m_ccm_server->m_component_name!="Server2noft" && m_ccm_server->m_component_name!="Server3noft") {

                CdmwPlatformMngt::External::RedundancyGroup_var group = component_context->the_redundancy_group();

                CPPUNIT_ASSERT(!CORBA::is_nil(group.in()));
                m_ccm_server->m_redundancy_group = CdmwPlatformMngt::External::RedundancyGroup::_duplicate(group.in()); 
            
            }

            TEST_INFO("end of Server do_tests");
        } catch (...) {
        std::cerr << "unknown exception during test !!" << std::endl;
    }

}

};  // namespace TestProcCompConsistency

