/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <iostream>
#include <fstream>
#include <limits>
#include <string>
#include "testprocessinitprecondition/TestPrecondition.hpp"
#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "SystemMngt/platformlibrary/Iterator.hpp"
#include "SystemMngt/platformvaluetypes/SystemModeChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/SystemStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/HostStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationAgentDeath_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationModeChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/EntityStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/PropertyChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/UserEvent_impl.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"


using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestProcessPrecondition);

namespace Cdmw
{
namespace PlatformMngt
{

TestProcessPrecondition::TestProcessPrecondition()
{
}

void TestProcessPrecondition::setUp ()
{
    try
    {
        //Getting the orb reference
        std::string system_corbaloc = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";

        orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::instance ()->get_ORB ());

        CORBA::Object_var system_obj = orb->string_to_object (system_corbaloc.c_str());

        //Getting the system reference
        m_system = CdmwPlatformMngt::System::_narrow (system_obj.in ());

        //Setting up the startupKind parameters
        state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
        startup_mode = 0;
        specified_state_id = "";
        startupKind = new CdmwPlatformMngtBase::StartupKind();
        startupKind->state_retrieval = state_retrieval;
        startupKind->startup_mode = startup_mode;
        startupKind->specified_state_id  = CORBA::string_dup( specified_state_id.c_str() );

        //Initialising the variables..
        obName = "T_Observer";          
        m_sleeping = 200;
    }
    catch (const CORBA::SystemException& ex) 
    {
        std::cerr<< ex._name()<<std::endl;
        throw;
    }
    catch(...)
    {
    }
}

void TestProcessPrecondition::test_start_process ()
{
    try 
    {

        TEST_INFO("Test Starting...");

        //Getting the application vector.
        m_nbApps = m_system->get_number_of_applications();
        m_apps   = m_system->get_applications(m_nbApps,appIter.out());

        for (CORBA::ULong appl_count = 0; appl_count<m_nbApps; appl_count++)
        {
            std::string ApplName = m_apps[appl_count]->name();
            if(0 == strcmp("FDPS",ApplName.c_str()))
            {
                
                m_nbProcs = m_apps[appl_count]->get_number_of_processes();
                m_procs   = m_apps[appl_count]->get_processes(m_nbProcs, procIter.out());       
            }
        } 
        //Creating and registering the observer
        TimeObserver_impl* sysOb = register_observer();

        //Waiting for the system started notification
        int count = 0;
        TEST_INFO("Waiting for the system start...");
        //If the system doesn't start by the sleeping time, the test fails
        while (!sysOb->isStarted())
        {
            if (m_sleeping == count) {CPPUNIT_ASSERT(false);}
            sleep (1);
        }
        TEST_INFO("Controlling if the processes are started with the right delay ... ");
        
        OsSupport::OS::Timeval DDS_host1_time;
        OsSupport::OS::Timeval DDS_host2_time;
        OsSupport::OS::Timeval FDPS_FE_CS_1_time;
        OsSupport::OS::Timeval FDPS_FE_CS_2_time;
        OsSupport::OS::Timeval FDPS_FE_CS_ROOT_time;
        //The delay deviation (ms) to control the process initialisation
        int delay_deviation=450;  
        
        for (CORBA::ULong appl_count = 0; appl_count<m_nbApps; appl_count++)
        {
            std::string ApplName = m_apps[appl_count]->name();
            
            if(0 == strcmp("DDS_host1",ApplName.c_str()))
            {
                DDS_host1_time = sysOb->getInformation(ApplName,"initialised");

                std::cout<<"\n____________________________________________________"<<std::endl;
                std::cout<<"The application "<<ApplName<<": became initialised  at time (s)"<< DDS_host1_time.seconds 
                <<", (usec) "<<DDS_host1_time.microseconds<<std::endl;
            }
            if(0 == strcmp("DDS_host2",ApplName.c_str()))
            {
                DDS_host2_time = sysOb->getInformation(ApplName,"initialised");
                std::cout<<"\n____________________________________________________"<<std::endl;
                std::cout<<"The application "<<ApplName<<": became initialised  at time (s)"<< DDS_host2_time.seconds 
                <<", (usec) "<<DDS_host2_time.microseconds<<std::endl;
            }
        }

            
            //The processes of FDPS application are controlled by the applications DDS_host1 DDS_host2  
            for (CORBA::ULong proc_count = 0; proc_count<m_nbProcs; proc_count++)
            {
                std::string ProcName = m_procs[proc_count]->name();
                if (  strcmp(ProcName.c_str(),"FDPS_FE_CS_1") == 0)
                {
                    FDPS_FE_CS_1_time = sysOb->getInformation(ProcName,"initialising");
                    std::cout<<"\n____________________________________________________"<<std::endl;
                    std::cout<<"The process "<<ProcName<<"controlled by DDS_host1 became initialising at time (s)"<< FDPS_FE_CS_1_time.seconds 
                    <<", (usec) "<<FDPS_FE_CS_1_time.microseconds<<std::endl;
                }
                if (  strcmp(ProcName.c_str(),"FDPS_FE_CS_2") == 0)
                {
                    FDPS_FE_CS_2_time = sysOb->getInformation(ProcName,"initialising");
                    std::cout<<"\n____________________________________________________"<<std::endl;
                    std::cout<<"The process "<<ProcName<<" controlled by DDS_host2 became initialising at time (s)"<< FDPS_FE_CS_2_time.seconds 
                    <<", (usec) "<<FDPS_FE_CS_2_time.microseconds<<std::endl;
                }
                if (  strcmp(ProcName.c_str(),"FDPS_FE_CS_ROOT") == 0)
                {
                    FDPS_FE_CS_ROOT_time = sysOb->getInformation(ProcName,"initialising");
                    std::cout<<"\n____________________________________________________"<<std::endl;
                    std::cout<<"The process "<<ProcName<<" root in the init graph became initialising at time (s)"<< FDPS_FE_CS_ROOT_time.seconds 
                    <<", (usec) "<<FDPS_FE_CS_ROOT_time.microseconds<<std::endl;
                }
            }
            
            
            //Calculating the difference between currApp and its last predecessor
            long diff_host2 = (1000000*(FDPS_FE_CS_2_time.seconds - DDS_host2_time.seconds)) 
            + (FDPS_FE_CS_2_time.microseconds - DDS_host2_time.microseconds);
            //Printing the value 
            std::cout<<"\n____________________________________________________"<<std::endl;
            std::cerr<<"\nDDS_host2  -->  FDPS_FE_CS_2  start delay  (ms):"<< (diff_host2/1000);
            std::cout<<"\n____________________________________________________"<<std::endl;
            
            
            long diff_host1 = (1000000*(FDPS_FE_CS_1_time.seconds - DDS_host1_time.seconds)) 
            + (FDPS_FE_CS_1_time.microseconds - DDS_host1_time.microseconds);
            std::cout<<"\n____________________________________________________"<<std::endl;
            std::cerr<<"\nDDS_host1  -->  FDPS_FE_CS_1 start delay (ms):"<< (diff_host1/1000);
            std::cout<<"\n____________________________________________________"<<std::endl;
            //Printing the value   

            CPPUNIT_ASSERT( ( (diff_host2 > 0) && (diff_host2 < (delay_deviation*1000)) ) );
            CPPUNIT_ASSERT( ( (diff_host1 > 0) && (diff_host1 < (delay_deviation*1000)) ) );
            
            
        //Destroying and unregistering the observer
        std::cout<<"\n"<<std::endl;
        unregister_observer(sysOb);
        std::cout<<"\n"<<std::endl;
        CPPUNIT_ASSERT(true);
    }
    catch (const CORBA::SystemException& e)
    {
        std::cout<<e<<std::endl;
        CPPUNIT_ASSERT (false);
        throw;
    }
}

TimeObserver_impl* TestProcessPrecondition::register_observer () throw () 
{
    // Get a reference to the root POA
    CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
    PortableServer::POA_var poa = PortableServer::POA::_narrow( obj.in() );

    // Activate the POA manager
    PortableServer::POAManager_var mgr = poa->the_POAManager();
    mgr->activate();

    // Register event factories
    CORBA::ValueFactoryBase_var factory, old_factory;

    factory = new SystemModeChangeFactory;
    old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/SystemModeChange:1.0",
            factory.in() );

    factory = new SystemStatusChangeFactory;
    old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/SystemStatusChange:1.0",
            factory.in() );

    factory = new HostStatusChangeFactory;
    old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/HostStatusChange:1.0",
            factory.in() );

    factory = new ApplicationAgentDeathFactory;
    old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationAgentDeath:1.0",
            factory.in() );

    factory = new ApplicationModeChangeFactory;
    old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationModeChange:1.0",
            factory.in() );

    factory = new ApplicationStatusChangeFactory;
    old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationStatusChange:1.0",
            factory.in() );

    factory = new ProcessMessageFactory;
    old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessMessage:1.0",
            factory.in() );

    factory = new ProcessStatusChangeFactory;
    old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessStatusChange:1.0",
            factory.in() );

    factory = new EntityStatusChangeFactory;
    old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/EntityStatusChange:1.0",
            factory.in() );

    factory = new UserEventFactory;
    old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/UserEvent:1.0",
            factory.in() );

    factory = new PropertyChangeFactory;
    old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/PropertyChange:1.0",
            factory.in() );

    TEST_INFO("Registering an observer to the system... ");

    //there are the applications used to create the map
    //TimeObserver_impl* sysOb= new TimeObserver_impl(m_apps.in(),m_nbApps,m_procs.in(), m_nbProcs);
    TimeObserver_impl* sysOb= new TimeObserver_impl(m_apps.in(),m_nbApps);
    observer = sysOb->_this();
    CdmwPlatformMngt::RegistrationDepth depth=FULL_HIERARCHY;

    //Registering the observer
    m_system->register_observer_ext (obName.c_str(),
            depth, 
            observer.in ());

    std::cerr << "Observer registered on System "<< " with depth = " << depth << std::endl;

    return sysOb;
}

void TestProcessPrecondition::unregister_observer (TimeObserver_impl* sysOb) throw () 
{
    //Unregistering the observers
    m_system->unregister_observer_ext (obName.c_str());
    std::cerr << "Unregistering " << obName <<std::endl;
}  



}/* End namespace PlatformMngt */
}/* End namespace Cdmw */





