/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */

package cdmw.qualification;

import cdmw.ossupport.OS;
import cdmw.orbsupport.StrategyList;
import cdmw.orbsupport.*;
import com.thalesgroup.CdmwPlatformMngt.*;

public class TestTracing {

    public static final int POA_THREAD_POOL_SIZE = 5;

    public static final int EXIT_FAILURE = -1;

    public static final String UNMANAGED_OPTION = "--unmanaged";
    public static final String SLEEP_OPTION = "--Sleep";
    public static final String MONITORING_REQUEST_OPTION = "--Request";
    public static final String MONITORING_EXCEPT_OPTION = "--Exception";

    public static void usage() {
        java.lang.System.out.println("Usage:\n"
           + "cdmw.qualification.TestTracing options\n"
           + "\n"
           + "Options:\n"
           + "--help                             Display this information.\n"
           + "[--unmanaged]                      Process is not managed.\n"
           + "[--Sleep=<time>]                   Time before is_alive return.\n"
           + "[--Request=<bool>]                 bool returned by is_alive\n"
           + "[--Exception=<bool>]               exception raised by is_alive (after a certain number of times)\n");
    }

    public static void main(String[] args) {

        if (run(args) == EXIT_FAILURE) {
            java.lang.System.out.println("Test failed.");
        } else {
            java.lang.System.out.println("Test successful.");
        }

    }

    public static int run(String[] args) {
    
        // the return code
        int retCode = 0;
        org.omg.CORBA.ORB orb = null;

        //
        // help argument
        //
        if (args.length > 1) {
            if(args[0].equals("--help")) {
                usage();
                return retCode;
            }
        }

        try {
    
            String unmanagedOption = OS.getOptionValue(args, UNMANAGED_OPTION);
            boolean managedProcess = true;
            if (unmanagedOption.equals("yes")) {
                managedProcess = false;
            }

            String sleepTimeOption = OS.getOptionValue(args, SLEEP_OPTION);
            int sleepTime = 0;
            if (sleepTimeOption.equals("no")) {
                sleepTime = 100;
            } else if (sleepTimeOption.equals("yes")) {
                java.lang.System.out.println("bad value for "
                    + SLEEP_OPTION + "=<value> option\n");
                return EXIT_FAILURE;
            } else if (sleepTimeOption != "no") {
                try {
                    sleepTime = Short.valueOf(sleepTimeOption).intValue();
                } catch(NumberFormatException nfe) {
                    java.lang.System.out.println("bad value for "
                        + SLEEP_OPTION + "=<value> option\n");
                    return EXIT_FAILURE;
                }
            }

            String monitoringRequestOption = OS.getOptionValue(args, MONITORING_REQUEST_OPTION);
            boolean monitoringRequest = true;
            if (monitoringRequestOption.equals("no")) {
                monitoringRequest = true;
            } else if (monitoringRequestOption.equals("true")) {
                monitoringRequest = true;
            } else if (monitoringRequestOption.equals("false")) {
                monitoringRequest = false;
            } else monitoringRequest = true;

            String monitoringExceptOption = OS.getOptionValue(args, MONITORING_EXCEPT_OPTION);
            boolean monitoringExcept = true;
            if (monitoringExceptOption.equals("no")) {
                monitoringExcept = false;
            } else if (monitoringExceptOption.equals("true")) {
                monitoringExcept = true;
            } else if (monitoringExceptOption.equals("false")) {
                monitoringExcept = false;
            } else monitoringExcept = false;

            java.lang.System.out.println("test cdmw process, unmanagedOption:" + unmanagedOption + " sleepTimeOption:" + sleepTimeOption + " monitoringRequestOption:" + monitoringRequestOption + " monitoringExceptOption:" + monitoringExceptOption);

            // Initialize the ORB
            StrategyList orbStrategies = new StrategyList();
            orbStrategies.addORBThreaded();
            orbStrategies.addPOAThreadPool(POA_THREAD_POOL_SIZE);

            orb = ORBSupport.ORBInit(args, orbStrategies);

            // Get the root POA
            org.omg.PortableServer.POA rootPOA =
                org.omg.PortableServer.POAHelper.narrow(
                    orb.resolve_initial_references("RootPOA"));

            // Run implementation (Activate the POA manager)
            rootPOA.the_POAManager().activate();
       
            try {  

                MyProcessControl ctrl = new MyProcessControl(orb, java.lang.System.out, sleepTime, monitoringRequest, monitoringExcept);    

                cdmw.cdmwinit.CdmwInterface.cdmwInit(orb, args, ctrl,
                                                         "cdmw.generated.CdmwInterface");

                // resolve system ref
                org.omg.CORBA.Object objref2;

                /* define the corbaloc */
                String corbalocStr = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";

                objref2 = orb.string_to_object(corbalocStr);

                /* resolve the System */
                com.thalesgroup.CdmwPlatformMngt.System mySystem = com.thalesgroup.CdmwPlatformMngt.SystemHelper.narrow(objref2);

                if (mySystem == null) {
                    throw new Exception("Error in narrowing CdmwPlatformMngt::System");
                }

                Application app = mySystem.get_application("HELLO_WORLD");

                String procName = "TraceClient";
                String hostname = java.lang.System.getenv("HOSTNAME");

                com.thalesgroup.CdmwPlatformMngt.Process proc = app.get_process(procName, hostname);

                if (proc == null){
                    throw new Exception("*******ERROR in proc_name, is nil *************");
                }

                //try a getting of filters
                org.omg.CORBA.Any myAny = org.omg.CORBA.ORB.init().create_any();

                org.omg.CORBA.AnyHolder myAnyHolder = new org.omg.CORBA.AnyHolder(myAny);

                proc.get_attribute(TRACE_FILTERS.value, myAnyHolder, Owner.PRE_DEFINED);

                String initFilters = myAnyHolder.value.extract_string();  

                if (initFilters.compareTo("TraceClient,HELLO_WORLD,0x80") == 0) {
                    java.lang.System.out.println("TEST NUM. S1 -->  OK: scenario filters ="+initFilters);
                } else {
                    java.lang.System.out.println("TEST NUM. S1 -->  Error: scenario filters ="+initFilters);
                }

                //try a parsing exception

                char flagTest = 0;
                try {
                    myAny.insert_string("Trace.Client,HELLO_WORLD,0x0");
                    proc.set_attribute(TRACE_FILTERS.value, myAny, Owner.PRE_DEFINED);
                } catch(org.omg.CORBA.BAD_PARAM bd) {
                    java.lang.System.out.println("TEST NUM. C1 -->  Error: allowed . character test in domain CORBA::BAD_PARAM exception ");
                    flagTest = 1; 
                } catch ( Exception exc) {
                    java.lang.System.out.println("TEST NUM. C1 -->  Error: allowed . character test in domain unknown exception ");
                    flagTest = 1; 
                }

                if (flagTest == 0) {
                    java.lang.System.out.println("TEST NUM. C1 -->  OK: allowed . character test in domain ");
                }

                flagTest = 0;
                try {
                    myAny.insert_string("Trace+Client,HELLO_WORLD,0x0");
                    proc.set_attribute(TRACE_FILTERS.value, myAny, Owner.PRE_DEFINED);
                } catch(org.omg.CORBA.BAD_PARAM bd) {
                    java.lang.System.out.println("TEST NUM. C2 -->  Error: allowed + character test in domain CORBA::BAD_PARAM exception ");
                    flagTest = 1; 
                } catch ( Exception exc) {
                    java.lang.System.out.println("TEST NUM. C2 -->  Error: allowed + character test in domain unknown exception ");
                    flagTest = 1; 
                }

                if (flagTest == 0) {

                    java.lang.System.out.println("TEST NUM. C2 -->  OK: allowed + character test in domain ");
                }

                flagTest = 0;
                try {
                    myAny.insert_string("Trace-Client,HELLO_WORLD,0x0");
                    proc.set_attribute(TRACE_FILTERS.value, myAny, Owner.PRE_DEFINED);
                } catch(org.omg.CORBA.BAD_PARAM bd) {
                    java.lang.System.out.println("TEST NUM. C3 -->  Error: allowed - character test in domain CORBA::BAD_PARAM exception ");
                    flagTest = 1; 
                } catch ( Exception exc) {
                    java.lang.System.out.println("TEST NUM. C3 -->  Error: allowed - character test in domain unknown exception ");
                    flagTest = 1; 
                }

                if (flagTest == 0) {

                    java.lang.System.out.println("TEST NUM. C3 -->  OK: allowed - character test in domain ");
                }

                flagTest = 0;
                try {
                    myAny.insert_string("Trace_Client,HELLO_WORLD,0x0");
                    proc.set_attribute(TRACE_FILTERS.value, myAny, Owner.PRE_DEFINED);
                } catch(org.omg.CORBA.BAD_PARAM bd) {
                    java.lang.System.out.println("TEST NUM. C4 -->  Error: allowed _ character test in domain CORBA::BAD_PARAM exception ");
                    flagTest = 1; 
                } catch ( Exception exc) {
                    java.lang.System.out.println("TEST NUM. C4 -->  Error: allowed _ character test in domain unknown exception ");
                    flagTest = 1; 
                }

                if (flagTest == 0) {

                    java.lang.System.out.println("TEST NUM. C4 -->  OK: allowed _ character test in domain ");
                }

                flagTest = 0;
                try {
                    myAny.insert_string("Trace,Client,HELLO_WORLD,0x0");
                    proc.set_attribute(TRACE_FILTERS.value, myAny, Owner.PRE_DEFINED);
                } catch(org.omg.CORBA.BAD_PARAM bd) {
                    java.lang.System.out.println("TEST NUM. E1 -->  OK: not allowed , character in domain CORBA::BAD_PARAM exception "+
                    "Caught ");
                    flagTest = 1; 
                } catch ( Exception exc) {
                    java.lang.System.out.println("TEST NUM. E1 -->  Error: not allowed , character in domain unknown exception ");
                }

                if (flagTest == 0) {

                    java.lang.System.out.println("TEST NUM. E1 -->  NOT OK: not allowed , character in domain CORBA::BAD_PARAM exception "+
                    "not reached ! ");
                }


                //try a parsing exception
                flagTest = 0;
                try {
                    myAny.insert_string("TraceClient,!,11");
                    proc.set_attribute(TRACE_FILTERS.value, myAny, Owner.PRE_DEFINED);
                } catch(org.omg.CORBA.BAD_PARAM bd) {
                    java.lang.System.out.println("TEST NUM. E2 -->  OK: reached a CORBA::BAD_PARAM exception");
                    flagTest = 1; 
                }

                if (flagTest == 0) {
                    java.lang.System.out.println("TEST NUM. E2 -->  Error: not reached a CORBA::BAD_PARAM exception ");
                }
                
                
                //try a parsing exception
                flagTest = 0;
                try {
                    myAny.insert_string("TraceClient,HELLO.WORLD,-1");
                    proc.set_attribute(TRACE_FILTERS.value, myAny, Owner.PRE_DEFINED);
                } catch(org.omg.CORBA.BAD_PARAM bd) {
                    java.lang.System.out.println("TEST NUM. E3 -->  OK: reached a CORBA::BAD_PARAM exception");
                    flagTest = 1; 
                }

                if (flagTest == 0) {
                    java.lang.System.out.println("TEST NUM. E3 -->  Error: not reached a CORBA::BAD_PARAM exception ");
                }
                
                
                //try a parsing exception
                flagTest = 0;
                try {
                    myAny.insert_string("TraceClient,HELLO.WORLD,32");
                    proc.set_attribute(TRACE_FILTERS.value, myAny, Owner.PRE_DEFINED);
                } catch(org.omg.CORBA.BAD_PARAM bd) {
                    java.lang.System.out.println("TEST NUM. E4 -->  OK: reached a CORBA::BAD_PARAM exception");
                    flagTest = 1; 
                }

                if (flagTest == 0) {
                    java.lang.System.out.println("TEST NUM. E4 -->  Error: not reached a CORBA::BAD_PARAM exception ");
                }


                // retrieve an unmanaged process reference to test
                char flagTestU1 = 0;

                String procNameUnmanaged = "HelloProcess1";
                com.thalesgroup.CdmwPlatformMngt.Process procUnmanaged = app.get_process(procNameUnmanaged, hostname);

                try {
                    myAny.insert_string("TraceClient,HELLO_WORLD,19/TraceClient,HELLO_WORLD,22");
                    procUnmanaged.set_attribute(TRACE_FILTERS.value, myAny, Owner.PRE_DEFINED);
                } catch(Exception e) {
                    java.lang.System.out.println("TEST NUM. U1 -->  Error: reached unexpected exception of trace filters setting to an unmanaged process ");
                    java.lang.System.out.println(e.toString());
                    flagTestU1 = 1 ;
                }

                if (flagTestU1 == 0) {
                    java.lang.System.out.println("TEST NUM. U1 -->  OK: setting (without effect) of trace filters to an unmanaged process ");
                }


                // test: adding a setting of filters ...
                flagTest = 0;

                try {
                    myAny.insert_string("TraceClient,HELLO_WORLD,*");
                    proc.set_attribute(TRACE_FILTERS.value, myAny, Owner.PRE_DEFINED);
                } catch (Exception e) {
                    java.lang.System.out.println("TEST NUM. S2 -->  Error: Failed to set trace filters to managed process");
                    flagTest = 1;
                }

                if (flagTest == 0) {
                    java.lang.System.out.println("TEST NUM. S2 -->  OK: setting of other trace filters to the managed process"); 
                }


                // test: adding a setting of filters ...
                flagTest = 0;

                try {
                    myAny.insert_string("TraceClient,HELLO_WORLD,!");
                    proc.set_attribute(TRACE_FILTERS.value, myAny, Owner.PRE_DEFINED);
                } catch (Exception e) {
                    java.lang.System.out.println("TEST NUM. S3 -->  Error: Failed to set trace filters to managed process");
                    flagTest = 1;
                }

                if (flagTest == 0) {
                    java.lang.System.out.println("TEST NUM. S3 -->  OK: setting of other trace filters to the managed process"); 
                }

                // test: getting of these new filters ...
                proc.get_attribute(TRACE_FILTERS.value, myAnyHolder, Owner.PRE_DEFINED);
                String  newFilters_g1 = myAnyHolder.value.extract_string();

                if ( newFilters_g1.compareTo("") == 0) {
                    java.lang.System.out.println("TEST NUM. G1 -->  OK: getting of actual (in managed process) filters="+newFilters_g1);
                } else {
                    java.lang.System.out.println("TEST NUM. G1 -->  Error: getting of actual (in managed process) filters="+newFilters_g1);
                }


                // test: adding a setting of filters ...
                flagTest = 0;

                try {
                    myAny.insert_string("TraceClient,HELLO_WORLD,0x1280");
                    proc.set_attribute(TRACE_FILTERS.value, myAny, Owner.PRE_DEFINED);
                } catch (Exception e) {
                    java.lang.System.out.println("TEST NUM. S4 -->  Error: Failed to set trace filters to managed process");
                    flagTest = 1;
                }

                if (flagTest == 0) {
                    java.lang.System.out.println("TEST NUM. S4 -->  OK: setting of other trace filters to the managed process"); 
                }

                // test: getting of these new filters ...
                proc.get_attribute(TRACE_FILTERS.value, myAnyHolder, Owner.PRE_DEFINED);
                String  newFilters_g2 = myAnyHolder.value.extract_string();

                if ( newFilters_g2.compareTo("TraceClient,HELLO_WORLD,7/TraceClient,HELLO_WORLD,9/TraceClient,HELLO_WORLD,12") == 0) {
                    java.lang.System.out.println("TEST NUM. G2 -->  OK: getting of actual (in managed process) filters="+newFilters_g2);
                } else {
                    java.lang.System.out.println("TEST NUM. G2 -->  Error: getting of actual (in managed process) filters="+newFilters_g2);
                }


                // test: setting of filters at host level ...
                flagTest = 0;

                Host myHost = mySystem.get_host(hostname);

                try {
                    myAny.insert_string("TraceClient,HELLO_WORLD,14") ; 
                    myHost.set_attribute(TRACE_FILTERS.value, myAny, Owner.PRE_DEFINED);
                } catch ( AttributeNotFound e) {
                    java.lang.System.out.println("REACHED ATTR. NOT FOUND EXCEPTION");  
                } catch(Exception e) {
                    java.lang.System.out.println("TEST NUM. H1 --> Error: setting of other trace filters at host level");
                    java.lang.System.out.println(e.toString());
                    flagTest = 1 ;
                }

                if (flagTest == 0) {
                    java.lang.System.out.println("TEST NUM. H1 -->  OK: setting of other trace filters at host level");
                }
    
                // advise behavior that init is done
                ctrl.set_init_done();
                synchronized (ctrl) {
                    ctrl.notify();
                }

                // start orb
                orb.run();
            } catch(Exception e) { java.lang.System.err.println("*****ERROR: reached exception in managed process *****");

                // cleanup the ORB to exit completely
                orb.shutdown(false);
                orb.destroy();
                return EXIT_FAILURE;
            }

        } catch (Exception ex) {
            ex.printStackTrace();
            return EXIT_FAILURE;
        }

        if (orb != null) {
            try {
                cdmw.cdmwinit.CdmwInterface.cdmwCleanup(orb, "cdmw.generated.CdmwInterface");
                java.lang.System.out.println("Destroying the orb...");
                orb.destroy();
            } catch(Exception ex) {
                ex.printStackTrace();
                return EXIT_FAILURE;
            }
        }
        
        return retCode;
    }

}

