/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <testunknown/CCM_Client_impl.hpp>
#include <testunknown/SupervisionObserver_impl.hpp>

#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/logging/CdmwLogging.stub.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtManagedElementContexts.stub.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp>



namespace TestUnknown
{
    const std::string SYSTEM_NAME = "S1";
    const std::string APPLICATION_NAME = "CcmTestUnknownStateApplication";
    const std::string CLIENT_PROCESS_NAME = "TestUnknownStateProcessClient1";
    const std::string SERVER_PROCESS1_NAME = "TestUnknownStateProcessServer";
    const std::string SERVER_PROCESS2_NAME = "TestUnknownStateProcessServer";
    const std::string SERVER_PROCESS3_NAME = "TestUnknownStateProcessServer";
    const std::string CLIENT_COMPONENT_NAME = "Client1";
    const std::string SERVER_COMPONENT1_NAME = "Server1";
    const std::string SERVER_COMPONENT2_NAME = "Server2";
    const std::string SERVER_COMPONENT3_NAME = "Server3";
    const std::string CLIENT_ISSUER_NAME = "TestUnknownStateProcessClient1/Client1";
    const std::string SERVER_ISSUER1_NAME = "TestUnknownStateProcessServer/Server1";
    const std::string SUPERVISION_ISSUER_NAME = "CDMW.PFMMNG";
    const std::string CLIENT_NOTIFICATION_MESSAGE = "Notification message from Client";
    const CORBA::ULong CLIENT_USER_EVENT = 33;
    const std::string SERVER_NOTIFICATION_MESSAGE = "Notification message from Server";
    const std::string SERVER_NOTIFICATION_FATAL = "Fatal error from Server";
    const CORBA::ULong SERVER_USER_EVENT = 44;
    const CORBA::Long SERVER_EXIT_CODE = 10;




  const char* event_kind_to_string(CdmwPlatformMngt::Event* ev) {
        switch (ev->event_kind()) {
            case CdmwPlatformMngt::SYSTEM_MODE_CHANGE:
                return "SYSTEM_MODE_CHANGE";
            case CdmwPlatformMngt::SYSTEM_STATUS_CHANGE:
                return "SYSTEM_STATUS_CHANGE";
            case CdmwPlatformMngt::HOST_STATUS_CHANGE:
                return "HOST_STATUS_CHANGE";
            case CdmwPlatformMngt::APPLICATION_AGENT_DEATH:
                return "APPLICATION_AGENT_DEATH";
            case CdmwPlatformMngt::APPLICATION_MODE_CHANGE:
                return "APPLICATION_MODE_CHANGE";
            case CdmwPlatformMngt::APPLICATION_STATUS_CHANGE:
                return "APPLICATION_STATUS_CHANGE";
            case CdmwPlatformMngt::PROCESS_MESSAGE:
                return "PROCESS_MESSAGE";
            case CdmwPlatformMngt::PROCESS_STATUS_CHANGE:
                return "PROCESS_STATUS_CHANGE";
            case CdmwPlatformMngt::ENTITY_STATUS_CHANGE:
                return "ENTITY_STATUS_CHANGE";
            case CdmwPlatformMngt::CONFIGURATION_CHANGE:
                return "CONFIGURATION_CHANGE";
            case CdmwPlatformMngt::USER_EVENT:
                return "USER_EVENT";
            case CdmwPlatformMngt::PROPERTY_CHANGE:
                return "PROPERTY_CHANGE";
            case CdmwPlatformMngt::RESTART_REQUESTED:
                return "RESTART_REQUESTED";
            case CdmwPlatformMngt::REBOOT_REQUESTED:
                return "REBOOT_REQUESTED";
            case CdmwPlatformMngt::STOP_REQUESTED:
                return "STOP_REQUESTED";
            case CdmwPlatformMngt::COMPONENT_STATUS_CHANGE:
                return "COMPONENT_STATUS_CHANGE";
            case CdmwPlatformMngt::REDUNDANCY_GROUP_STATUS_CHANGE:
                return "REDUNDANCY_GROUP_STATUS_CHANGE";
            default:
                return "unknown event kind !";
        }
    }



//
// IDL:thalesgroup.com/CcmFtTest/ClientModule/CCM_Client:1.0
//
CCM_Client_impl::CCM_Client_impl()
{
}

CCM_Client_impl::~CCM_Client_impl()
{
    std::cerr << "   ... CCM_Client_impl::~CCM_Client_impl called!" << std::endl;
}


void CCM_Client_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    m_session_context = ::TestUnknown::CCM_Client_Context::_narrow(ctx);
}
        
::TestUnknown::CCM_Client_Context* CCM_Client_impl::get_context()
{
    return ::TestUnknown::CCM_Client_Context::_duplicate(m_session_context.in());
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CCM_Client_impl::ccm_activate()
   throw(CORBA::SystemException)
{
    std::cerr << "   ... CCM_Client_impl::ccm_activate called!" << std::endl;

    // Get this component CORBA object
    // Note: this has to be performed in a CORBA request thread; thus
    //       not in test which is a specific thread.
    TestUnknown::Client_var this_component =
        TestUnknown::Client::_nil();
    try {
        // narrow context in SessionContext for get_CCM_object() access
        Components::SessionContext_var ctx =
            Components::SessionContext::_narrow(m_session_context.in());
        
        // call SessionContext::get_CCM_object()
        CORBA::Object_var obj =
            ctx->get_CCM_object();
            
        this_component =
            TestUnknown::Client::_narrow(obj.in());
            
        CDMW_ASSERT(!CORBA::is_nil(this_component.in()));
            
    } catch (const CORBA::UserException& e) {
        std::cerr << "UserException during get_CCM_Object() :" 
                  << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
                  << std::endl;
        CDMW_ASSERT(false);
    } catch (const CORBA::SystemException& e) {
        std::cerr << "SystemException during get_CCM_Object() :" 
                  << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
                  << std::endl;
        CDMW_ASSERT(false);
    }


    try {   
        // save context in CORBATestManager for usage in test
        Cdmw::TestUtils::CORBATestManager::instance()->add(
            "client_context", 
            get_context());
    
        // save this component in CORBATestManager for usage in test
        Cdmw::TestUtils::CORBATestManager::instance()->add(
            "this_component", 
            this_component.in());
    
        // save this executor in CORBATestManager for usage in test
        Cdmw::TestUtils::CORBATestManager::instance()->add(
            "this_executor", 
            dynamic_cast< Session_CCM_Client_ptr > (this));
    
        // create test runner thread
        m_test_thread = new Cdmw::TestUtils::ThreadedTestRunner(
            Cdmw::TestUtils::CORBATestManager::instance()->get_controller());
        // add test to be run
        m_test_thread->addTest( 
            CppUnit::TestFactoryRegistry::getRegistry("test_unknown").makeTest());
        // start test
        m_test_thread->start();

    } catch (const CORBA::Exception& e) {
        std::cerr << "CORBA Exception exception during test: "
                  << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
                  << std::endl;
    } catch (const std::exception& e) {
        std::cerr << "std::exception exception during test: "
                  << e.what()
                  << std::endl;
    } catch (...) {
        std::cerr << "unknown exception during test !!" << std::endl;
    }
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
CCM_Client_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
    std::cerr << "   ... CCM_Client_impl::ccm_passivate called!" << std::endl;
    
    if (m_test_thread != 0) {
        // wait test to be finished
        m_test_thread->join();
        delete m_test_thread;
        m_test_thread = 0;
        // print test summary
        Cdmw::TestUtils::CORBATestManager::instance()->summary();
    }

} 


//
// ccm_remove function : it destroys the component (call by the container)
//
void
CCM_Client_impl::ccm_remove()
   throw(CORBA::SystemException)
{
    std::cerr << "   ... CCM_Client_impl::ccm_remove called!" << std::endl;
}


CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(test_unknown, "test_unknown");


test_unknown::test_unknown()
{
    // retrieve context object
    CORBA::Object_var obj = 
        Cdmw::TestUtils::CORBATestManager::instance()->get_object("client_context");
    m_session_context = 
        ::TestUnknown::CCM_Client_Context::_narrow(obj.in());
        
    obj = 
        Cdmw::TestUtils::CORBATestManager::instance()->get_object("this_executor");
    m_ccm_client = dynamic_cast< CCM_Client_impl* > (obj.in());
    
    obj = 
        Cdmw::TestUtils::CORBATestManager::instance()->get_object("this_component");
    m_component_ref =
        TestUnknown::Client::_narrow(obj.in());
}

void
test_unknown::do_tests()
{
    std::cerr << "   ... test_unknown::do_tests called!" << std::endl;
    
    
    try {

        TestUnknownState_var test_unknown =
            m_session_context->get_connection_test_server();

       Cdmw::OsSupport::OS::sleep(3000);
        
       TEST_INFO("Testing ManagedUnknownState");

       CORBA::String_var real_component_name;
       try {
        // get ManagedComponentContext
        CORBA::Object_var obj = 
            m_session_context->resolve_service_reference("ManagedComponentContext");
        CdmwPlatformMngt::ManagedElementContexts::ComponentContext_var componet_context =
            CdmwPlatformMngt::ManagedElementContexts::ComponentContext::_narrow(obj.in());
        CPPUNIT_ASSERT(!CORBA::is_nil(componet_context.in()));
     
        TEST_INFO("check supervised object names");
        // get component, process, application names
        CORBA::String_var component_name = componet_context->managed_element_name();
        TEST_INFO(" component_name = '" 
                  << (component_name.in() == 0 ? "null" : component_name.in())
                  << "'");
        CPPUNIT_ASSERT(component_name.in() != 0);
        CPPUNIT_ASSERT(CLIENT_COMPONENT_NAME == component_name.in());        
        
        CORBA::String_var process_name = componet_context->process_name();
        TEST_INFO("      process_name = '" 
                  << (process_name.in() == 0 ? "null" : process_name.in())
                  << "'");
        CPPUNIT_ASSERT(process_name.in() != 0);
        CPPUNIT_ASSERT(CLIENT_PROCESS_NAME == process_name.in());        

        CORBA::String_var appli_name = componet_context->application_name();
        TEST_INFO("      appli_name = '" 
                 << (appli_name.in() == 0 ? "null" : appli_name.in())
                 << "'");
        CPPUNIT_ASSERT(appli_name.in() != 0);
        CPPUNIT_ASSERT(APPLICATION_NAME == appli_name.in());        

        // get System supervision object
        TEST_INFO("   get System supervision object");
        CdmwPlatformMngt::External::System_var system = componet_context->the_system();

        // register this component as System observer
        TEST_INFO("   register this component as System Observer");
        CdmwPlatformMngt::SupervisionObserver_var old_one =
            system->register_observer("TestUnknown_Client", 
                    m_component_ref.in());
        
        SupervisionObserver_impl* obsSystem = new SupervisionObserver_impl();
        std::string SysObsNameExt ="SystemObserverExt";
        CdmwPlatformMngt::SupervisionObserver_var systemObsExt = obsSystem->_this();
        CdmwPlatformMngt::RegistrationDepth depth=CdmwPlatformMngt::FULL_HIERARCHY;
        system->register_observer_ext(SysObsNameExt.c_str(),
                                       depth,
                                       systemObsExt.in());
 
        /*********************************************
        * Register observers for component and group 
        * status change notification
        ********************************************/
            
        //Getting Component supervision object
        CdmwPlatformMngt::System_var System = CdmwPlatformMngt::System::_narrow (system.in());
        
        TEST_INFO("   get Component supervision object");
        std::string remote_host_name = Cdmw::TestUtils::CORBATestManager::instance()->get_aux_info();
        
        CdmwPlatformMngt::CcmComponent_var component = 
            ((System->get_application(APPLICATION_NAME.c_str()))->
            get_process(SERVER_PROCESS2_NAME.c_str(), remote_host_name.c_str()))->
            get_component(SERVER_COMPONENT2_NAME.c_str());

        TEST_INFO(" Register_observer_ext() for Component :"<<SERVER_COMPONENT2_NAME);
        SupervisionObserver_impl* ObsComp1 = new SupervisionObserver_impl();
        CdmwPlatformMngt::SupervisionObserver_var compObsExt = ObsComp1->_this();
        depth=CdmwPlatformMngt::OBJECT_ONLY;
        std::string CompObsNameExt = "ComponentObserverExt";
        component->register_observer_ext(CompObsNameExt.c_str(),
                                          depth,
                                          compObsExt.in());
        TEST_INFO(" Register_observer() for Component :"<<SERVER_COMPONENT2_NAME);
        SupervisionObserver_impl* ObsComp2 = new SupervisionObserver_impl();
        CdmwPlatformMngt::SupervisionObserver_var compObs = ObsComp2->_this();
        std::string CompObsName = "ComponentObserver";
        component->register_observer(CompObsName.c_str(), compObs.in());

        
        // Getting RedundancyGroup supervision object
        TEST_INFO("   get RedundancyGroup supervision object");
        CdmwPlatformMngt::External::RedundancyGroup_var redgroup = test_unknown->get_server_group();
        CPPUNIT_ASSERT(!CORBA::is_nil(redgroup.in()));
        
        TEST_INFO(" Register_observer_ext() for RedundancyGroup :"<<redgroup->name());
        SupervisionObserver_impl* ObsGroup1 = new SupervisionObserver_impl();
        CdmwPlatformMngt::SupervisionObserver_var grObsExt = ObsGroup1->_this();
        depth=CdmwPlatformMngt::OBJECT_ONLY;
        std::string GrObsNameExt = "RedGroupObserverExt";
        redgroup->register_observer_ext(GrObsNameExt.c_str(),
                                       depth,
                                       grObsExt.in());
        
        TEST_INFO(" Register_observer() for RedundancyGroup :"<<redgroup->name());
        SupervisionObserver_impl* ObsGroup2 = new SupervisionObserver_impl();
        CdmwPlatformMngt::SupervisionObserver_var grObs = ObsGroup2->_this();
        std::string GrObsName = "RedGroupObserver";
        redgroup->register_observer(GrObsName.c_str(),
                                       grObs.in());

         TEST_INFO("  Test is finished");
     
        } catch (const CORBA::SystemException& e) {
            TEST_INFO("Exception calling Server1's facet: " << 
                      Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            CPPUNIT_ASSERT(false);
             }
    } catch(CdmwPlatformMngt::ObserverNotFound)
    {
        std::cerr << "Exception Observer Not Found " << std::endl;
    }catch (...) {
        std::cerr << "unknown exception during test " << std::endl;
    }

}

    
void CCM_Client_impl::notify(CdmwPlatformMngt::Event* event)
        throw(CORBA::SystemException)
{

    CORBA::String_var msg = event->to_string();
    std::cout << "   ... " << event_kind_to_string(event)
              << " notification received from Supervision : \n"
              << msg.in() << std::endl;

    // save event in event queue
    event->_add_ref();
    m_event_queue.push(event);
    
}



//
// return first unconsumed event
//
CdmwPlatformMngt::Event*
CCM_Client_impl::get_last_event()
{
    // if no event in queue, return null
    if (m_event_queue.size() == 0)
        return 0;

    // get first event from event queue
    CdmwPlatformMngt::Event* event = m_event_queue.front();
    m_event_queue.pop();

    return event;
}


};  // namespace TestUnknown

