/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <string>
#include <cctype>
#include <algorithm>
#include <sstream>


#include "Foundation/common/Locations.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/common/Exception.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/logging/LogManager.hpp"
#include "Foundation/logging/LogLevel.hpp"

#include "TraceAndPerf/tracelibrary/Trace.hpp"
#include "TraceAndPerf/performancelibrary/Probe.hpp"

#include "traceclient/TraceClientThread.hpp"





TraceClientThread::TraceClientThread(CORBA::ORB_ptr orb,
                                     std::string& applicationName,
                                     std::string& processName)
   throw (CORBA::SystemException)
   :  m_orb(CORBA::ORB::_duplicate(orb)),
      m_applicationName(applicationName),
      m_processName(processName),
      m_exit_required(false),
      m_logger (Cdmw::Logging::Logger::Get_Logger(m_applicationName, m_processName))
{
    std::cout << "TraceClientThread::TraceClientThread" << std::endl;
}

TraceClientThread::~TraceClientThread() throw()
{
    std::cout << "TraceClientThread::~TraceClientThread" << std::endl;
    this->stop();
}

void TraceClientThread::stop()
{
    std::cout << "TraceClientThread::stop" << std::endl;
    this->m_exit_required = true; 
}

void TraceClientThread::run () throw()
{
    std::cout << "TraceClientThread::run" << std::endl;
    using namespace Cdmw;
    using namespace Cdmw::Logging;
    
    try
    {        
        std::string baseMsg = "process <" + m_applicationName + "/" + m_processName + ">" ;
        std::cout << "Trace Client Thread Started :" << baseMsg << std::endl;
        int msg_cycle=0;

        // ====================================================
        // thread loop
        // ====================================================
        
        // trace message or probe call main loop
        while (!m_exit_required) 
        {
            std::cout << "[" << msg_cycle << "] New cycle !" << std::endl;
            msg_cycle++;

            CDMW_LOG(m_logger, Cdmw::Logging::FATAL, "A first FATAL message in cycle [" << msg_cycle << "]");
            CDMW_LOG(m_logger, Cdmw::Logging::ERROR, "A first ERROR message in cycle [" << msg_cycle << "]");
            CDMW_LOG(m_logger, Cdmw::Logging::WARN, "A first WARN message in cycle [" << msg_cycle << "]");
            CDMW_LOG(m_logger, Cdmw::Logging::INFO, "A first INFO message in cycle [" << msg_cycle << "]");
            CDMW_LOG(m_logger, Cdmw::Logging::DEBUG, "A first DEBUG message in cycle [" << msg_cycle << "]");

            CDMW_LOG(m_logger, Cdmw::Logging::FATAL, "A second FATAL message in cycle [" << msg_cycle << "]");
            CDMW_LOG(m_logger, Cdmw::Logging::ERROR, "A second ERROR message in cycle [" << msg_cycle << "]");
            CDMW_LOG(m_logger, Cdmw::Logging::WARN, "A second WARN message in cycle [" << msg_cycle << "]");
            CDMW_LOG(m_logger, Cdmw::Logging::INFO, "A second INFO message in cycle [" << msg_cycle << "]");
            CDMW_LOG(m_logger, Cdmw::Logging::DEBUG, "A second DEBUG message in cycle [" << msg_cycle << "]");

            std::cout << "Thread sleeps 1 sec !" << std::endl;
            Cdmw::OsSupport::OS::sleep (1000);
        } 

        CDMW_LOG(m_logger, Cdmw::Logging::FATAL, "A last FATAL message !");
    }
    catch (...)
    {
        std::cout << "File : " << __FILE__ << " Line : " << __LINE__
                  << "Unexpected exception in thread loop" << std::endl;
    }
}
