/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/common/Options.hpp"
#include "Foundation/common/String.hpp"
#include "Foundation/common/Exception.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/common/Locations.hpp"

#include "Foundation/orbsupport/CORBA.hpp"

#include "Foundation/ossupport/OS.hpp"
#include "Foundation/osthreads/ThreadHandle.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/logging/LogManager.hpp"

#include <ConfAndPlug/cdmwinit/CdmwInterface.hpp>
#include <ConfAndPlug/cdmwinit/InitUtils.hpp>

#include <Time/clockservice/CdmwCosClock.hpp>
#include <Foundation/commonsvcs/time/Util.hpp>

#include "Foundation/logging/Logger.hpp"

#include "SystemMngt/platforminterface/PlatformInterface.hpp"
#include "SystemMngt/platforminterface/ServiceNames.hpp"

#include "Repository/repositoryinterface/RepositoryInterface.hpp"

#include "TraceAndPerf/tracelibrary/InitUtils.hpp"
#include "TraceAndPerf/tracelibrary/Trace.hpp"

#include "traceclient/MyProcessBehaviour.hpp"

#include <cstdlib>
#include <string>
#include <exception>
#include <sstream>
#include <iostream>
#include <fstream>




const int POA_THREAD_POOL_SIZE = 5;

namespace
{
    using namespace Cdmw::OsSupport;
    using namespace Cdmw::OrbSupport;
    using namespace Cdmw::PlatformMngt;    
    
    const int SUCCESS = 0;
    const int FAILURE = 1;
    
    std::string process_name;
    
}


void exceptionHandler(void)
{
    std::cout << "Unexpected Exception : Test failed" << std::endl;
}


void exitHandler(void)
{
    std::cout << "Process '" << process_name << "' exited" << std::endl;
}

int main(int argc, char* argv[])
{
   // the return code 
   int ret_code = 0;

   std::set_unexpected(&exceptionHandler);
   ::atexit(&exitHandler);

   CORBA::ORB_var orb = CORBA::ORB::_nil();
   try
   {
      // initialize the ORB.
      Cdmw::OrbSupport::StrategyList orb_strategies;
      orb_strategies.add_OrbThreaded();
      orb_strategies.add_PoaThreadPool(POA_THREAD_POOL_SIZE);
      std::cout << "Cdmw::OrbSupport::OrbSupport::ORB_init" << std::endl;
      orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, orb_strategies);

      // get the root POA.
      CORBA::Object_var obj = orb -> resolve_initial_references("RootPOA");
      PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj.in());

      // activate the root POA manager.
      PortableServer::POAManager_var poaManager = rootPOA -> the_POAManager();
      poaManager -> activate();

      // creates the process behaviour
      MyProcessBehaviour * pProcBehaviour = new MyProcessBehaviour(orb);
 

      // call generated CDMW_init().
      std::cout << "Cdmw::CdmwInit::CDMW_init" << std::endl;
      Cdmw::CdmwInit::CDMW_init(orb.in(), argc, argv, pProcBehaviour);

      // Sleep for debug
      std::cout << "Sleep 30 sec" << std::endl;
      Cdmw::OsSupport::OS::sleep (30000);

      // start orb
      std::cout << "orb run" << std::endl;
      orb->run();            
   }
            
   catch (const Cdmw::Exception & ex) 
   {
      std::cerr << ex.what() << std::endl;
      ret_code = FAILURE;
   }
   catch (const CORBA::Exception& e)
   {  
      std::cerr << "FAILURE : CORBA exception raised :" << std::endl
                << e._name() << std::endl;
      ret_code = FAILURE;
   }
   catch(const std::exception &e)
   {
      std::cerr << "FAILURE : Exception raised : " << std::endl
                << e.what() << std::endl;
      ret_code = FAILURE;
   }
 

   if(!CORBA::is_nil(orb.in()))
   {
      try 
      {
         Cdmw::CdmwInit::CDMW_cleanup(orb.in());
         Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
         orb->destroy();
      }
      catch(const CORBA::Exception& ex)
      {           
         std::cerr << ex << std::endl;
         ret_code = FAILURE;
      } 
   }
        
   return ret_code;
}


