/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.qualif.traceclient;

import java.io.*;
import java.util.*;

import org.omg.CORBA.*;
import org.omg.CORBA.Object;
import org.omg.PortableServer.*;

import cdmw.common.*;
import cdmw.logging.*;
import static cdmw.logging.Logger.*;
import cdmw.namingandrepository.*;
import cdmw.orbsupport.*;
import cdmw.ossupport.*;
import cdmw.platformmngt.*;
import cdmw.trace.*;

import com.thalesgroup.CdmwNamingAndRepository.*;

public class MainTest
{
    private static final int POA_THREAD_POOL_SIZE = 5;
        
    private static final int FAILURE = 1;
    
    private static String processName;
    
    
    public static void main(String[] args)
    {
        String applicationName = "";
        int retCode = 0;
        
        StrategyList strategy = new StrategyList();
        strategy.addORBThreaded();
        try
        {
            strategy.addPOAThreadPool(MainTest.POA_THREAD_POOL_SIZE);
        }
        catch (BadParameterException e)
        {
            e.printStackTrace();
        }
        
        ORB orb = null;        
        
        try
        {
            // initialise the orb    
            orb = ORBSupport.ORBInit(args, strategy);
            Object obj = orb.resolve_initial_references("RootPOA");
            POA rootPOA = POAHelper.narrow(obj);
            POAManager poaMng = rootPOA.the_POAManager();
            poaMng.activate();
                
            System.out.println("~+~+~+~ Making sure that the TraceCollector is fully initialized : Sleeping for 4 sec...");
            OS.sleep (4000);      
            System.out.println("~+~+~+~ done.");
            
            // The logger
            Logger logger = Logger.Get_Logger("TEST", "TEST");
            
            try
            {
                System.out.println("Init PlatformInterface");
          
                String fooStr = "";
                // creates the process behaviour
                MyProcessBehaviour procBehaviour = new MyProcessBehaviour(orb);
                                                               
                cdmw.cdmwinit.CdmwInterface.cdmwInit(orb, args, procBehaviour, 
                    "cdmw.generated.TraceClientCdmwInterface");   

                // PCR-1875
                final String PCR1875Option = "--PCR-1875";
                String option = 
                    cdmw.ossupport.OS.getOptionValue(args,
                         PCR1875Option);
                
                if (option.compareTo("no") == 0) {
                    System.out.println("TEST INFO: register a null fatal handler to avoid" 
                                       + " the process to be killed by the supervision" 
                                       + " following a call to a CDMW_FATAL message.");
                    cdmw.logging.LogManager.Instance().registerFatalHandler(null);
                } else {
                    // This test is used to test the changes done for the PCR-1875
                    System.out.println("TEST INFO: this test validates also the changes of PCR-1875");
                }

                            
                // start orb
                orb.run();            
            }
            catch(java.lang.Exception e)
            {
                e.printStackTrace();
            }
            
            if (orb != null) {
                cdmw.cdmwinit.CdmwInterface.cdmwCleanup(orb, 
                    "cdmw.generated.TraceClientCdmwInterface");                                                 
                //orb->shutdown(false) is done by the ProcessBehaviour
                ORBSupport.ORBCleanup(orb);
                orb.destroy();
            }
            
        }
        catch (SystemException e)
        {  
            System.err.println( "FAILURE : CORBA exception raised :" + e.getMessage());
            e.printStackTrace();
            retCode = FAILURE;
        }
        catch(java.lang.Exception e)
        {
            System.err.println("FAILURE : Exception raised" + "\r\n" + e.getMessage());
            e.printStackTrace();
            retCode = FAILURE;
        }

        System.exit(retCode);
    }

    private void exceptionHandler()
    {
        System.out.println("Unexpected Exception : Test failed");
    }

    private void exitHandler()
    {
        System.out.println("Process '" + MainTest.processName + "' exited");
    }
    
}
