/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.qualif.traceclient;

import cdmw.common.BadOrderException;
import cdmw.common.BadParameterException;
import static cdmw.logging.Logger.*;
import cdmw.logging.Logger;
import cdmw.platformmngt.*;
import org.omg.CORBA.ORB;

public class MyProcessBehaviour extends cdmw.cdmwinit.ProcessControl {
    /**
     * The ORB.
     */
    private org.omg.CORBA.ORB orb;

    /**
     * The trace message.
     */
    private String traceMessage;

    /**
     * The logger.
     */
    private Logger logger;

    /**
     * init done flag
     */
    private boolean initDone;

    /**
     * The application name.
     */
    private String applicationName;

    /**
     * The process name.
     */
    private String processName;

    /**
     * The client thread reference
     */
    private TraceClientThread traceClientThread;



    public MyProcessBehaviour(ORB orb)
    {
        System.out.println("MyProcessBehaviour.MyProcessBehaviour");

        this.orb = orb;
        this.traceClientThread = null;
        this.logger = Logger.Get_Logger("TEST", "TEST");
        this.initDone = false;
    }

    public void onRun()  
        throws com.thalesgroup.CdmwPlatformMngt.ProcessDelegatePackage.NotReadyToRun 
    {
        System.out.println("MyProcessBehaviour.onRun");

        try {
            PlatformInterface.notifyMessage(
                com.thalesgroup.CdmwPlatformMngtBase.INF.value,
                this.processName, "Run requested");
        }
        catch (BadOrderException e) {
            CDMW_ERROR(this.logger, "BadOrderException", null);
            e.printStackTrace();
        }
        catch (BadParameterException e) {
            CDMW_ERROR(this.logger, "BadParameterException", null);
            e.printStackTrace();
        }

        // start the this.traceClientThread
        if (this.traceClientThread != null) {
            this.traceClientThread.start();
        }
    }

    public void onStop()
    {
        System.out.println("MyProcessBehaviour.onStop");

        TraceClientThread clientThread = this.traceClientThread;
        this.traceClientThread = null;

        try {
            PlatformInterface.notifyMessage(
                com.thalesgroup.CdmwPlatformMngtBase.INF.value,
                this.processName, "Stop requested");
        }
        catch (BadOrderException e) {
            CDMW_ERROR(this.logger, "BadOrderException", null);
            e.printStackTrace();
        }
        catch (BadParameterException e) {
            CDMW_ERROR(this.logger, "BadParameterException", null);
            e.printStackTrace();
        }

        if (clientThread != null
            && Thread.State.RUNNABLE == clientThread.getState()) {
            try {
                clientThread.join();
            }
            catch (InterruptedException ie) {
                CDMW_ERROR(this.logger, "InterruptedException", null);
                ie.printStackTrace();
            }
        }

        this.orb.shutdown(false);
    }

    public void onInitialise(com.thalesgroup.CdmwPlatformMngtBase.StartupKind startupKind) 
    {
        System.out.println("MyProcessBehaviour.onInitialise");

        try {
            // get the name of the application
            this.applicationName = PlatformInterface.getApplicationName();
            // get the name of the process
            this.processName = PlatformInterface.getProcessName();
            PlatformInterface.notifyMessage(com.thalesgroup.CdmwPlatformMngtBase.INF.value,
                this.processName, "Initialisation requested");
            this.traceMessage = "A trace from " + this.processName + "@" 
               + this.applicationName;
        }
        catch (BadOrderException e) {
            CDMW_ERROR(this.logger, "BadOrderException", null);
            e.printStackTrace();
        }
        catch (BadParameterException e) {
            CDMW_ERROR(this.logger, "BadParameterException", null);
            e.printStackTrace();
        }

        // create the this.traceClientThread
        this.traceClientThread = 
           new TraceClientThread(this.orb, this.applicationName, 
                                 this.processName, this.logger);
    }

    public org.omg.CORBA.Object getService()
        throws org.omg.CORBA.SystemException 
    {
        System.out.println("MyProcessBehaviour.getService");

        try {
            PlatformInterface.notifyMessage(
                com.thalesgroup.CdmwPlatformMngtBase.INF.value,
                this.processName, "Service requested -> NIL returned");
        }
        catch (BadOrderException e) {
            CDMW_ERROR(this.logger, "BadOrderException", null);
            e.printStackTrace();
        }
        catch (BadParameterException e) {
            CDMW_ERROR(this.logger, "BadParameterException", null);
            e.printStackTrace();
        }

        return null;
    }

    public void onNextStep()
        throws com.thalesgroup.CdmwPlatformMngt.ProcessDelegatePackage.InvalidStep
    {
       System.out.println("MyProcessBehaviour.onNextStep");
    }

    public int nbSteps() throws org.omg.CORBA.SystemException 
    {
        System.out.println("MyProcessBehaviour.nbSteps");
        return 1;
    }

}
