/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.qualif.traceclient;

import org.omg.CORBA.ORB;

import cdmw.common.BadParameterException;
import cdmw.logging.*;
import static cdmw.logging.Logger.*;

public class TraceClientThread extends Thread
{
      public  int CDMW_TRACE_LEVEL = 6;
        
      /**
       * The orb reference.
       */
      private ORB orb;
            
      /**
       * The application name.
       */
      private String applicationName;
              
      /**
       * The process name.
       */
      private String processName;
            
      /**
       * The trace message.
       */
      private String traceMessage;
            
      /**
       * The flag to break the thread loop.
       */
      private boolean exitRequired;
            
      /**
       * The logger.
       */
      private Logger logger;
            

      public TraceClientThread(ORB orb, 
                               String applicationName, 
                               String processName,
                               Logger logger) 
         throws org.omg.CORBA.SystemException
      {
         System.out.println("TraceClientThread.TraceClientThread");

         this.orb=orb;
         this.applicationName = applicationName;
         this.processName = processName;
         this.traceMessage = "A Trace message from " + this.processName + "@" 
            + this.applicationName;
         this.exitRequired = false;
         this.logger = logger;
      }
        
      public void run()
      {    
         System.out.println("TraceClientThread.run");

         try
         {        
            long msg_cycle = 0;
            long count = 0;
            long msgCount = 0;


            if (CDMW_TRACE_LEVEL == 0)
            {
               System.out.println("Sending a message to the Trace collector.");
            }

            CDMW_INFO(this.logger, this.traceMessage, new Throwable());

            // trace message call main loop
            while (!this.exitRequired) 
            {
               msg_cycle++;
               System.out.println("["+msg_cycle+"] "+"New cycle !");

               CDMW_FATAL(this.logger,"A first FATAL message in cycle ["+msg_cycle+"]", new Throwable());
               CDMW_ERROR(this.logger,"A first ERROR message in cycle ["+msg_cycle+"]", new Throwable());
               CDMW_WARN(this.logger,"A first WARN message in cycle ["+msg_cycle+"]", new Throwable());
               CDMW_INFO(this.logger,"A first INFO message in cycle ["+msg_cycle+"]", new Throwable());
               CDMW_DEBUG(this.logger,"A first DEBUG message in cycle ["+msg_cycle+"]", new Throwable());
               
               CDMW_FATAL(this.logger,"A second FATAL message in cycle ["+msg_cycle+"]", new Throwable());
               CDMW_ERROR(this.logger,"A second ERROR message in cycle ["+msg_cycle+"]", new Throwable());
               CDMW_WARN(this.logger,"A second WARN message in cycle ["+msg_cycle+"]", new Throwable());
               CDMW_INFO(this.logger,"A second INFO message in cycle ["+msg_cycle+"]", new Throwable());
               CDMW_DEBUG(this.logger,"A second DEBUG message in cycle ["+msg_cycle+"]", new Throwable());

               System.out.println("Thread sleeps 1 sec !");
               cdmw.ossupport.OS.sleep (1000);
            }
            
            CDMW_FATAL(this.logger, "A last FATAL message !", new Throwable());
         }
         catch (Exception e)
         {
            System.err.println(e.toString()+" Unexpected exception in thread loop");
         }
                
      }
        
      public void stopImpl() 
      {
         System.out.println("TraceClientThread.stopImpl");
         exitRequired = true; 
      }
}
