/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_HELLO_CLIENT_THREAD_HPP 
#define INCL_HELLO_CLIENT_THREAD_HPP 

#include <iostream>
#include <string>

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/osthreads/ThreadHandle.hpp>



class TraceClientThread : public Cdmw::OsSupport::Thread 
{
    
  public:
  
    TraceClientThread (CORBA::ORB_ptr,
                       std::string& application_name,
                       std::string& process_name,
                       bool multi_domain_level,
                       std::string& trace_message,
                       long trace_count,
                       long perf_count)
                 throw (CORBA::SystemException);



    ~TraceClientThread() throw();




    void stop();


    void target_api (long count_for_perf);


  protected:

    void run() throw();
      


  private:
    
    TraceClientThread();
    
    TraceClientThread(const TraceClientThread&);
    
    TraceClientThread& operator = (const TraceClientThread&);
    
    
    
    /**
    * The orb reference.
    */
    CORBA::ORB_var m_orb;
    
    /**
    * The application name.
    */
    std::string m_application_name;
      
    /**
    * The process name.
    */
    std::string m_process_name;
    
    /**
    * The multi domain-level option to trace.
    */
    bool m_multi_domain_level;
    
    /**
    * The trace message.
    */
    std::string m_trace_message;
    
    /**
    * The number of trace message.
    */
    long m_trace_count;
    
    /**
    * The number of performance probe cycle.
    */
    long m_perf_count;
       
    /**
    * The flag to break the thread loop.
    */
    bool m_exit_required;
};



#endif // INCL_HELLO_CLIENT_THREAD_HPP 
