/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.validatexmlfromxsd;


import java.io.File;
import java.io.IOException;
import javax.xml.XMLConstants;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Schema;
import javax.xml.validation.Validator;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

/**
 * Validate an XML document against an XML schema.
 */
public class XMLValidator {


    protected static final String JAXP_SCHEMA_LANGUAGE =
        "http://java.sun.com/xml/jaxp/properties/schemaLanguage";

    protected static final String W3C_XML_SCHEMA =
        "http://www.w3.org/2001/XMLSchema";

    protected static final String JAXP_SCHEMA_SOURCE =
        "http://java.sun.com/xml/jaxp/properties/schemaSource";

    protected static final String VALIDATION_FEATURE =
        "http://xml.org/sax/features/validation";

    protected static final String INIT_FAILED_MSG =
        "Could not start XML validation: ";

    // The file to be validated.
    private File m_xml = null;

    // The XML schema file.
    private File m_xsd = null;


    /**
     * Default constructor.
     */
    public XMLValidator() {
        
    }

    /**
     * Specify the file to be checked.
     *
     * @param path The path of the file to be checked.
     */
    public void setXMLFile(String path) {
        System.out.println("setXMLFile [" +path+ "]");
        m_xml = new File(path);
    }


    /**
     * Specify the XML schema to be used for the validation.
     *
     * @param path The path of the XML schema file.
     */
    public void setXSDFile(String path) {
        System.out.println("setXSDFile [" +path+ "]");
        m_xsd = new File(path);
    }


    /**
     * Run the validator.
     */
    public void execute() throws ParserConfigurationException, IOException, SAXException {
        checkAttributes();
        doValidate();
    }


    /**
     * Parse the file.
     *
     */
    protected void doValidate() throws ParserConfigurationException, IOException, SAXException {
        System.out.println("Validating [" + m_xml.getName() + "]");

        DocumentBuilder parser = DocumentBuilderFactory.newInstance().newDocumentBuilder();
        Document document = parser.parse(m_xml);

        SchemaFactory factory = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);

        Source schemaFile = new StreamSource(m_xsd);
        Schema schema = factory.newSchema(schemaFile);

        Validator validator = schema.newValidator();

        validator.validate(new DOMSource(document));

        System.out.println("Validation OK");
    }


    /**
     * Check the attributes.
     */
    protected void checkAttributes() {
        if (!(m_xml.exists() && m_xml.canRead() && m_xml.isFile())) {
            System.out.println("XML File ["+m_xml.getPath()+"] doesn't exist or is not reading.");
            System.out.println("Aborting ...");
            System.exit(1);
        }
        if (!(m_xsd.exists() && m_xsd.canRead() && m_xsd.isFile())) {
            System.out.println("XSD File ["+m_xsd.getPath()+"] doesn't exist or is not reading.");
            System.out.println("Aborting ...");
            System.exit(1);
        }
    }


    /**
     * Main
     */
    public static void main(String[] args) {
        XMLValidator validator = new XMLValidator();
        validator.setXMLFile(args[0]);
        validator.setXSDFile(args[1]);

        try {
            // might abord
            validator.execute();
        } catch (ParserConfigurationException pce) {
            System.out.println("Validation KO");
            pce.printStackTrace();
            // exit code 1 when cmdln args invalid
            System.exit(2);
        } catch (IOException ioe) {
            System.out.println("Validation KO");
            ioe.printStackTrace();
            System.exit(3);
        } catch (SAXException se) {
            System.out.println("Validation KO");
            se.printStackTrace();
            System.exit(4);
        }
    }
}

