/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.xmlutils.test;


import org.junit.*;

import cdmw.testutils.CORBATestManager;
import cdmw.xmlutils.XMLHelper;


import com.thalesgroup.CdmwXML.*;
import com.thalesgroup.CdmwXML.DocumentPackage.SaveException;
import cdmw.xmlutils.XMLHelper;

public class TestClient {
    private static final int POA_THREAD_POOL_SIZE = 10;

    private static final String XML_FILE = "Q-030-file.xml";
    private static String FTP_HOME_DIR;

    private static String remoteHostName;
    
    private static com.thalesgroup.CdmwXML.XMLParser xmlParser;
    
//    @Before
//    public void init() {
//        FTP_HOME_DIR = (String) CORBATestManager.getLocalInstance().getLocalProperty(
//            "cdmw.xmlutils.test.TestClient.ftpDirPath");
//    }
   

    public TestClient ()
    {
    }

    void setXmlParser(com.thalesgroup.CdmwXML.XMLParser xmlParser) {
        this.xmlParser = xmlParser;
    }
   
    
    public void run(String[] args) {

        org.omg.CORBA.ORB orb = null;
        
        // parse the args to get the remote hostname
        if (args.length >= 2) {
            this.remoteHostName = args[0];
        } else {
            System.out.println("TestClient must have a remote hostname and the ftp directory " 
                + "as first argument !");
            System.exit(-1);
        }

        try {
            // init orb
            cdmw.orbsupport.StrategyList orbStrategies = 
                new cdmw.orbsupport.StrategyList();
            orbStrategies.addORBThreaded();
            orbStrategies.addPOAThreadPool(POA_THREAD_POOL_SIZE);
            orb = cdmw.orbsupport.ORBSupport.ORBInit(args, orbStrategies);
            // register XML value factories
            XMLHelper.registerValueFactories(orb);
            
            // get rootPOA and activate its POA Manager
            org.omg.PortableServer.POA rootPOA =
                org.omg.PortableServer.POAHelper.narrow(
                    orb.resolve_initial_references("RootPOA"));
            rootPOA.the_POAManager().activate();
            
            // initialise the platform interface
            cdmw.platformmngt.PlatformInterface.setup(orb, args);

            // create TestClientBehaviour
            TestClientBehaviour clientBehaviour =
                new TestClientBehaviour(this, orb);

            // acknowledge the creation of the process
            cdmw.platformmngt.PlatformInterface.acknowledgeCreation(clientBehaviour);

            orb.run();
            //orb.shutdown() is done by the ProcessBehaviour
            orb.destroy();
        
        } catch (Exception ex) {
            if (orb != null) {
                orb.shutdown(false);
                orb.destroy();
            }
            ex.printStackTrace();
        }
    }    
    
    @Test
    public void doTests() {    
        System.out.println("Testing Cdmw XML Service");

        java.util.Properties props = System.getProperties();
        String userDir = props.getProperty("user.dir");

        try {
            // load remote file from URL on localhost ftp (xmlServer is remote)
            // PCR-1369: Remove FTP while it is no more used for CMD and has been
            // removed from CARDAMOM configuration value.
            //java.io.File xmlFile = new java.io.File(FTP_HOME_DIR + "/" + XML_FILE);
            String xmlFileName = userDir + "/Q-030-data/" + XML_FILE;
            
            java.io.File xmlFile = new java.io.File(xmlFileName);
            if (!xmlFile.canRead()) {
                System.out.println("Can't find \"" + xmlFile.toString() + "\". " +
                        "Cdmw XML Service cannot be tested.");
                return;
            }
            
            //String xmlUrl = "ftp://" + cdmw.ossupport.OS.getHostname() + 
            //                "/" + XML_FILE;
            
            // --- REQ 120 & 123 ---
            //System.out.println("Load remote XML file: " + xmlUrl);
            //DocumentDescription docDescr = xmlParser.parse(xmlUrl);
            
            // --- REQ 120 & 123 ---
            System.out.println("Load local XML file");
            DocumentDescription docDescr = xmlParser.parse(xmlFileName);
            Document doc = DocumentHelper.narrow(docDescr.reference());
            
            // --- REQ 122 ---
            System.out.println("Get root element");
            ElementDescription rootEltDescr = doc.get_document_element();
            Element rootElt = ElementHelper.narrow(rootEltDescr.reference());
            String rootEltName = XMLHelper.toString(rootEltDescr.name());
            Assert.assertTrue("Q-030-validation".equals(rootEltName));
        
            System.out.println("Get first child element");
            ElementDescription[] childEltsDescr = rootElt.get_child_elements();
            Assert.assertTrue(childEltsDescr.length == 1);
            Element childElt = ElementHelper.narrow(childEltsDescr[0].reference());
            String childEltName =  XMLHelper.toString(childEltsDescr[0].name());
            Assert.assertTrue("childElement".equals(childEltName));
            
            System.out.println("Get attribute of first child element");
            AttrDescription attrDescr = childElt.get_attribute(
                XMLHelper.toShortArray("test_attr"));
            Attr attr = AttrHelper.narrow(attrDescr.reference());
            String attrValue = XMLHelper.toString(attrDescr.value());
            Assert.assertTrue("AttrValue".equals(attrValue));
            
            System.out.println("Get text of first child element");
            TextDescription[] textsDescr = childElt.get_texts();
            Assert.assertTrue(textsDescr.length == 1);
            Text text = TextHelper.narrow(textsDescr[0].reference());
            String textValue = XMLHelper.toString(textsDescr[0].data());
            Assert.assertTrue("TextValue".equals(textValue));
            
            System.out.println("Get CDATASection of root element");
            textsDescr = rootElt.get_texts();
            Assert.assertTrue(textsDescr.length > 0);
            Assert.assertTrue(textsDescr[0] instanceof CDATASectionDescription);
            CDATASection cdata = null;
            if (textsDescr[0] instanceof CDATASectionDescription) {
                cdata = CDATASectionHelper.narrow((textsDescr[0].reference()));
                String cdataValue = XMLHelper.toString(textsDescr[0].data());
                Assert.assertTrue("CDATASectionValue".equals(cdataValue));
           }
            
            // --- REQ 125 ---
            System.out.println("Get first child element by XPath");
            NodeDescription[] nodes1 = doc.select_nodes_from_root(
                XMLHelper.toShortArray("/Q-030-validation/childElement"));
            Assert.assertTrue(nodes1.length == 1);
            Element childElt2 = ElementHelper.narrow(
                ((ElementDescription) nodes1[0]).reference());
            Assert.assertTrue(childElt2._is_equivalent(childElt));
            
            System.out.println("Get attribute of first child element by XPath");
            NodeDescription[] nodes2 = doc.select_nodes_from_root(
                XMLHelper.toShortArray("/Q-030-validation/childElement/@test_attr"));
            Assert.assertTrue(nodes2.length == 1);
            Attr attr2 = AttrHelper.narrow(
                ((AttrDescription) nodes2[0]).reference());
            Assert.assertTrue(attr2._is_equivalent(attr));

            System.out.println("Get text of first child element by XPath");
            NodeDescription[] nodes3 = doc.select_nodes_from_root(
                XMLHelper.toShortArray("/Q-030-validation/childElement/text()"));
            Assert.assertTrue(nodes3.length == 1);
            Text text2 = TextHelper.narrow(
                ((TextDescription) nodes3[0]).reference());
            Assert.assertTrue(text2._is_equivalent(text));
            
            System.out.println("Get CDATASection of root element by XPath");
            NodeDescription[] nodes4 = doc.select_nodes_from_root(
                XMLHelper.toShortArray("/Q-030-validation/text()"));
            Assert.assertTrue(nodes4.length > 0);
            Assert.assertTrue(nodes4[0] instanceof CDATASectionDescription);
            CDATASection cdata2 = null;
            if (nodes4[0] instanceof CDATASectionDescription) {
                cdata2 = CDATASectionHelper.narrow(
                    ((CDATASectionDescription) nodes4[0]).reference());
                Assert.assertTrue(cdata2._is_equivalent(cdata));
            }
            
            // --- REQ 124 ---
            System.out.println("Close Document");
            doc.close();
            //Assert.assertTrue(xmlFile.delete());

        } catch (DOMException e) {
            e.printStackTrace();
            System.err.println("" + e.code + ": " + e.reason);
        } catch (XPathFormatException e) {
            e.printStackTrace();
            System.err.println(e.reason);
        } catch (Exception e) {
            e.printStackTrace();
            System.err.println("Unexpected exception");
        }

        
        try {
            // Load local file (on xmlServer host)
            // We assumes that the validation dir has the same path on both hosts,
            // and that the TestClient and the XMLServer have the same working 
            // directory: /*/validation/data
            String xmlFile = userDir + "/Q-030-data/" + XML_FILE;
            
            // --- REQ 120 & 123 ---
            System.out.println("Load local XML file");
            DocumentDescription docDescr = xmlParser.parse(xmlFile);
            Document doc = DocumentHelper.narrow(docDescr.reference());
            
            // --- REQ 129 ---
            System.out.println("Modify value of an attribute");
            NodeDescription[] nodes = doc.select_nodes_from_root(
                XMLHelper.toShortArray("/Q-030-validation/childElement/@test_attr"));
            Attr attr = AttrHelper.narrow(
                ((AttrDescription) nodes[0]).reference());
            attr.set_value(XMLHelper.toShortArray("NewAttrValue"));
            NodeDescription[] newNodes = doc.select_nodes_from_root(
                XMLHelper.toShortArray("/Q-030-validation/childElement/@test_attr"));
            String attrValue = XMLHelper.toString(
                ((AttrDescription) newNodes[0]).value());
            Assert.assertTrue("NewAttrValue".equals(attrValue));
            
            System.out.println("Modify value of text");
            nodes = doc.select_nodes_from_root(
                XMLHelper.toShortArray("/Q-030-validation/childElement/text()"));
            Text text = TextHelper.narrow(
                ((TextDescription) nodes[0]).reference());
            text.set_data(XMLHelper.toShortArray("NewTextValue"));
            newNodes = doc.select_nodes_from_root(
                 XMLHelper.toShortArray("/Q-030-validation/childElement/text()"));
            String textValue = XMLHelper.toString(
                ((TextDescription) newNodes[0]).data());
            Assert.assertTrue("NewTextValue".equals(textValue));
                            
            System.out.println("Modify value of CDATASection");
            nodes = doc.select_nodes_from_root(
                XMLHelper.toShortArray("/Q-030-validation/text()"));
            CDATASection cdata = CDATASectionHelper.narrow(
                ((CDATASectionDescription) nodes[0]).reference());
            cdata.set_data(XMLHelper.toShortArray("NewCDATAValue"));
            newNodes = doc.select_nodes_from_root(
                 XMLHelper.toShortArray("/Q-030-validation/text()"));
            String cdataValue = XMLHelper.toString(
                ((CDATASectionDescription) newNodes[0]).data());
            Assert.assertTrue("NewCDATAValue".equals(cdataValue));
            
            // --- REQ 127 ---
            System.out.println("Create a new element");
            Element newElt = doc.create_element(XMLHelper.toShortArray("newElement"));
            
            System.out.println("Create a new attribute");
            Attr newAttr = doc.create_attribute(
                XMLHelper.toShortArray("newAttr"), XMLHelper.toShortArray("newAttrValue"));
            
            System.out.println("Create a new text");
            Text newText = doc.create_text_node(XMLHelper.toShortArray("newText"));
            
            System.out.println("Create a new CDATASection");
            CDATASection newCdata = doc.create_CDATASection(
                XMLHelper.toShortArray("newCDATA"));
            
            // --- REQ 128 ---
            System.out.println("Insert new Element before first child of root element");
            ElementDescription rootEltDescr = doc.get_document_element();
            Element rootElt = ElementHelper.narrow(rootEltDescr.reference());
            ElementDescription[] childEltsDescr = rootElt.get_child_elements();
            int nbElt = childEltsDescr.length;
            Element childElt = ElementHelper.narrow(childEltsDescr[0].reference());
            Node newNode = rootElt.insert_before(newElt, childElt);
            newElt = ElementHelper.narrow(newNode);
            childEltsDescr = rootElt.get_child_elements();
            Assert.assertTrue(childEltsDescr.length == nbElt+1);
            Assert.assertTrue("newElement".equals(XMLHelper.toString(childEltsDescr[0].name())));
            
            System.out.println("Append new Attribute to new element");
            newNode = newElt.append_child(newAttr);
            //newAttr = AttrHelper.narrow(newNode);
            nodes = doc.select_nodes_from_root(
                XMLHelper.toShortArray("/Q-030-validation/newElement/@newAttr"));
            Assert.assertTrue("newAttrValue".equals(XMLHelper.toString(
                ((AttrDescription) nodes[0]).value())));

            System.out.println("Replace new element by new Text");
            newNode = rootElt.replace_child(newText, newElt);
            newText = TextHelper.narrow(newNode);
            nodes = doc.select_nodes_from_root(
                XMLHelper.toShortArray("/Q-030-validation/text()"));
            Node txtNode = nodes[0].reference();
            Assert.assertTrue("newText".equals(XMLHelper.toString(
                 ((TextDescription) nodes[0]).data())));
            
            System.out.println("Remove new Text");
            newNode = rootElt.remove_child(txtNode);
            childEltsDescr = rootElt.get_child_elements();
            Assert.assertTrue(childEltsDescr.length == nbElt);
            Assert.assertTrue("childElement".equals(XMLHelper.toString(childEltsDescr[0].name())));

            // --- REQ 121 ---
            String modifiedFilename = userDir + "/Q-030-data/modified.xml";
            System.out.println("Save document as " + modifiedFilename);
            java.io.File modifiedFile = new java.io.File(modifiedFilename);
            if (modifiedFile.exists())
                modifiedFile.delete();
            doc.save_as(modifiedFilename);
            
            System.out.println("Save document as " + modifiedFilename);
            java.io.File tmpFile = 
                java.io.File.createTempFile("xml_validation_modifiedFile", "xml");
            tmpFile.delete();
            Assert.assertTrue(!tmpFile.exists());
            System.out.println("rcopy " + remoteHostName + ":" + modifiedFilename + " on " + tmpFile.getAbsolutePath());
            rcopyFile(remoteHostName + ":" + modifiedFilename, 
                tmpFile.getAbsolutePath());
            System.out.println("Check file existence " + tmpFile.getAbsolutePath());
            Assert.assertTrue(tmpFile.isFile());
            System.out.println("delete tmpFile");
            tmpFile.delete();
            
            doc.close();
            
        } catch (DOMException e) {
            e.printStackTrace();
            System.err.println("" + e.code + ": " + e.reason);
        } catch (XPathFormatException e) {
            e.printStackTrace();
            System.err.println(e.reason);
        } catch (Exception e) {
            e.printStackTrace();
            System.err.println("Unexpected exception");
        }
        
        try {
            // --- REQ 126 ---
            String newXmlFile = userDir + "/Q-030-data/newDoc.xml";
            System.out.println("Create an empty document: " + newXmlFile);
            DocumentDescription docDescr = 
                xmlParser.create_empty_document(newXmlFile,
                    XMLHelper.toShortArray("newXmlFile"), "");
            Document doc = DocumentHelper.narrow(docDescr.reference());
            
            // --- REQ 121 ---
            System.out.println("Save new document");
            doc.save();
            java.io.File xmlFile = new java.io.File(newXmlFile);
            
            java.io.File tmpFile = 
                java.io.File.createTempFile("xml_validation_newFile", "xml");
            tmpFile.delete();
            Assert.assertTrue(!tmpFile.exists());
            rcopyFile(remoteHostName + ":" + newXmlFile, 
                tmpFile.getAbsolutePath());
            Assert.assertTrue(tmpFile.isFile());
            tmpFile.delete();

            // --- REQ 124 ---
            doc.close();

        } catch (SaveException e) {
            e.printStackTrace();
            System.err.println(e.reason);
        } catch (Exception e) {
            e.printStackTrace();
            System.err.println("Unexpected exception");
        }

    }
    
    
    private void rcopyFile(String source, String destination) 
        throws java.io.IOException, InterruptedException
    {
        cdmw.ossupport.ThreadedProcess tp =
            cdmw.ossupport.OS.createProcess(
                "/usr/bin/scp " + source + " " + destination);
        tp.waitProcess();
    }
    
    
    public static void main(String[] args) {
        String ftpDirPath = null;
        // get path of FTP dir if specified

        if (args.length > 2)
            ftpDirPath = args[1];

        if (cdmw.platformmngt.PlatformInterface.
                isLaunchedByPlatformManagement(args)) 
        {        
            
            TestClient testClient = new TestClient();
            FTP_HOME_DIR = ftpDirPath;
            testClient.run(args);

        } else {
            System.err.println("TestClient should be launched by PlatformManagement.");
        }
    }
    
    
    
}

