/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_ACCESSOR_PROXY_HPP
#define INCL_ACCESSOR_PROXY_HPP

#include "RepositoryProxy.hpp"
#include "Foundation/ossupport/OS.hpp"

namespace Cdmw
{
namespace NamingAndRepository
{

/**
 * Purpose:
 * <p>
 * This static class is used by the classes NameDomainProxy and
 * NamingcontextProxy to perform bindings just before the first client
 * request on the repository.
 */
class AccessorProxy
{
public:

  /**
   * Purpose:
   * <p>
   * This method binds the default root context and the admin root context
   * <p>
   * <b><u>NOTA: </u><b><br>
   * All these bindings are performed just before the first client 
   * request on the repository. 
   */
  static bool initialise() 
    throw (CORBA::TRANSIENT, CORBA::SystemException);

  /**
   * Sets the attribut m_repository.
   */
  static void set_repository (CdmwNamingAndRepository::RepositoryExt_ptr proxy_repository)
    throw ();

  /**
   * Sets the attribut m_orb
   */
  static void set_orb (CORBA::ORB_ptr orb)
    throw ();
  
  /**
   * Sets the attribut m_port
   */
  static void set_port (const std::string& orb)
    throw ();

  /**
   * Sets the attribut m_context_holder_name
   */
  static void set_context_holder_name (const std::string& holder_name)
    throw ();

private:
  
  AccessorProxy(){}
  ~AccessorProxy(){}

  /** Purpose: <p> 
     *
     * This mutex allows to avoid parallel executions of the initiliase method.
     * Thus the bindings are perfomed only once and this avoids to manage
     * CosNaming::NamingContext::AlreadyBound exceptions.
     */
  static Cdmw::OsSupport::Mutex m_accessor_mutex;
  
  /**
   * The proxy repository
   */
  static CdmwNamingAndRepository::RepositoryExt_var m_repository;

  /**
   * The ORB.
   */
  static CORBA::ORB_var m_orb;

  /**
    * The port of the repository.
    */
  static std::string m_port;

  // The name of the holder context
  // Pattern is :
  //    hostname/appli_name/process_name/CONTEXTS
  static std::string m_context_holder_name;

};

} // End of namespace NamingAndRepository
} // End of namespace Cdmw



#endif
