/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_NAMINGANDREPOSITORY_PROXY_BEHAVIOUR_HPP 
#define INCL_NAMINGANDREPOSITORY_PROXY_BEHAVIOUR_HPP 


#include <ConfAndPlug/cdmwinit/ProcessControl.hpp>
#include <Repository/idllib/CdmwNamingAndRepository.stub.hpp>
#include <FaultTolerance/idllib/CdmwFTReplicationManager.stub.hpp>

#include "namingandrepositoryproxy/RepositoryProxy.hpp"

namespace Cdmw
{
    namespace NamingAndRepository
    {

        /**
         * Purpose:
         * <p>
         * The abstract process behaviour.
         */
        class NamingAndRepositoryProxyBehaviour : public Cdmw::CdmwInit::ProcessControl
        {

        public:

            /**
             * Purpose:
             * <p> The constructor.
             */ 
            NamingAndRepositoryProxyBehaviour(CORBA::ORB_ptr orb, 
                                              const std::string& port,
                                              const std::string& daemonPort,
                                              const std::string& processName);


            /**
             * Purpose:
             * <p> The destructor.
             */ 
            virtual ~NamingAndRepositoryProxyBehaviour()
                throw ();


            /**
             * Purpose:
             * <p>
             * the behaviour for the
             * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/get_service:1.0
             * operation
             */
            virtual CORBA::Object_ptr get_service()
                throw(CORBA::SystemException);


            /**
             * Purpose:
             * <p>
             * the behaviour for the
             * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/initialise:1.0
             * operation
             */
            virtual void on_initialise(const CdmwPlatformMngtBase::StartupKind& startup_kind)
                throw(CORBA::SystemException);


            /**
             * Purpose:
             * <p>
             * the behaviour for the
             * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/run:1.0
             * operation
             */
            virtual void on_run()
                throw(CdmwPlatformMngt::ProcessDelegate::NotReadyToRun,
                      CORBA::SystemException);

            /**
             * Purpose:
             * <p>
             * the behaviour for the
             * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/stop:1.0
             * operation
             */
            virtual void on_stop() 
                throw(CORBA::SystemException);


        private:

            /**
             * Notifies the fatal error via the platform interface or
             * output in cerr.
             */ 
            void notifyFatalError(const char* issuer, const char* message)
                throw(CORBA::SystemException);

            /**
             * The ORB.
             */
            CORBA::ORB_var m_orb;

            /**
             * Indicates whether the repository is managed by the platform management.
             */
            bool m_platformManaged;

            /**
             * The port of the daemon.
             */
            std::string m_daemonPort;

            /**
             * The name of the process.
             */
            std::string m_processName;
    
            /**
             * The repository implementation.
             */
            RepositoryProxy *m_repository_impl;

            /**
             * The naming and repository object;
             */
            CdmwNamingAndRepository::Repository_var m_repository;

            /**
             * The replication manager.
             */
            CdmwFT::ReplicationManager_var m_replication_mgr;
        };

    } // End namespace NamingAndRepository
} // End namespace Cdmw

#endif // INCL_NAMINGANDREPOSITORY_PROXY_BEHAVIOUR_HPP

