/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_TOOLS_NAME_SERVICE_ADMIN_HPP
#define INCL_TOOLS_NAME_SERVICE_ADMIN_HPP


#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/CosNaming.stub.hpp>
#include <iostream>
#include <string>

namespace Cdmw
{
namespace Tools
{
    
//--------------------------------------------------------------------------------------//
class NameServiceAdmin
{
public:
    
    NameServiceAdmin(CORBA::ORB_ptr, CosNaming::NamingContext_ptr root)
        throw (CORBA::SystemException);
    
    ~NameServiceAdmin() throw();

    void run(std::istream&,std::ostream&) throw();
    
protected:
    
    
    // Available commands
    CORBA::Long print_help (const std::string &, std::ostream &) throw();
    CORBA::Long exit (const std::string &, std::ostream &) throw();
    CORBA::Long reset(const std::string &, std::ostream &) throw();
    CORBA::Long chroot(const std::string &, std::ostream &) throw();

    CORBA::Long bind(const std::string &, std::ostream &) throw();
    CORBA::Long bind_context(const std::string &, std::ostream &) throw();
    CORBA::Long bind_new_context(const std::string &, std::ostream &) throw();
    CORBA::Long destroy(const std::string &, std::ostream &) throw();
    CORBA::Long destroy_all(const std::string &, std::ostream &) throw();
    CORBA::Long list(const std::string &, std::ostream &) throw();
    CORBA::Long rebind(const std::string &, std::ostream &) throw();
    CORBA::Long rebind_context(const std::string &, std::ostream &) throw();
    CORBA::Long resolve(const std::string &, std::ostream &) throw();
    CORBA::Long unbind(const std::string &, std::ostream &) throw();

    CORBA::Long toggle_force(const std::string &, std::ostream &) throw();
    CORBA::Long toggle_verbose(const std::string &, std::ostream &) throw();
    CORBA::Long load_file(const std::string &, std::ostream &) throw();

      /** some CORBA::Object operations ...
       * is_nil, is_a, non_existent, is_equivalent, hash, repository_id, get_component,
       */
      CORBA::Long obj_get_component(const std::string &, std::ostream &) throw();
      CORBA::Long obj_hash(const std::string &, std::ostream &) throw();
      CORBA::Long obj_is_a(const std::string &, std::ostream &) throw();
      CORBA::Long obj_is_equivalent(const std::string &, std::ostream &) throw();
      CORBA::Long obj_is_nil(const std::string &, std::ostream &) throw();
      CORBA::Long obj_non_existent(const std::string &, std::ostream &) throw();
      CORBA::Long obj_repository_id(const std::string &, std::ostream &) throw();

      /**
       * To start an external program. 
       * Appends -ORBInitRef NameService=m_root ior
       */
      CORBA::Long system(const std::string &, std::ostream &) throw();

private:
    
    typedef CORBA::Long  (NameServiceAdmin::* operation)(const std::string &, std::ostream &);
    
    struct command_def
    {
        const char* name;
        operation   op;
    };
    
    static const command_def commands[];
    static const CORBA::ULong nb_commands;
    
    CORBA::Long runOperation(const std::string &,const std::string &, std::ostream &) throw();
    
    NameServiceAdmin();
    NameServiceAdmin(const NameServiceAdmin&);
    NameServiceAdmin& operator = (const NameServiceAdmin&);
    
    void unknown_command(std::ostream &) const throw();    
    void print_prompt(std::ostream&) const throw(); 
    void print_info(std::ostream&) const throw();
    
    CORBA::ORB_var                           m_orb;
    CosNaming::NamingContext_var             m_root;
    CosNaming::NamingContext_var             m_initial_root;
    bool                                     m_verbose;
    bool                                     m_force;
        
};


} // namespace Tools
} // namespace Cdmw

#endif // INCL_TOOLS_NAME_SERVICE_ADMIN_HPP

