/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "tools/NSUtils.hpp"
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Foundation/commonsvcs/naming/NamingUtil.hpp>
#include <string>
#include <iostream>


namespace
{

    const char * TAB_VALUE = "    ";

    std::string tab_string(int level)
    {
        std::string tab("");

        for (int i = 0; i < level; ++i)
            tab += TAB_VALUE;
        
        return tab;
    }
}; // End anonymous namespace

namespace Cdmw
{
namespace Tools
{

//----------------------------------------------------------------------------
   bool 
   NSUtils::Show_context(CosNaming::NamingContext_ptr nc, 
                         std::ostream&                os, 
                         unsigned int                 level, 
                         unsigned int                 max)
      throw()
   {
      bool all_done = false;
      CosNaming::BindingIterator_var it;                      // Iterator reference
      CosNaming::BindingList_var     bl;                      // Binding list
      const CORBA::ULong LIST_CHUNK_SIZE = 500;               // Chunk size
      
      try {
         nc->list(LIST_CHUNK_SIZE,bl,it);                    // Get first chunk
         bool chunk_done = Show_chunk(nc, bl.in(),os,level, max);      // Show first chunk
         
         if (!CORBA::is_nil(it.in())) {                           // More bindings?
            CORBA::Boolean more;
            do {
               more = it->next_n(LIST_CHUNK_SIZE,bl);      // Get next chunk
               chunk_done &= Show_chunk(nc, bl.in(), os, level, max);// Show chunk
            } while (more);                                 // While not done
            it->destroy();                                  // Clean up iterator
         }
         all_done = chunk_done;
      } catch (const CORBA::SystemException &) {
         // keep silent (ignore)
      }
      return all_done;
   }

//----------------------------------------------------------------------------
   bool NSUtils::Show_chunk(CosNaming::NamingContext_ptr   nc, 
                            const CosNaming::BindingList & bl, 
                            std::ostream&                  os, 
                            unsigned int                   level, 
                            unsigned int                   max)
      throw()
   {
      bool done = true;
      
      for (CORBA::ULong i = 0; i < bl.length(); ++i) {
         // Note from CosNaming specification: 
         //   In struct Binding, binding_name is incorrectly defined
         //   as a Name instead of a NameComponent. This definition is
         //   unchanged for compatibility reasons.
         try {
            std::string tab = tab_string(level);
            
            os << tab 
               << Cdmw::CommonSvcs::Naming::NamingInterface::to_string(bl[i].binding_name);
            
            if (bl[i].binding_type == CosNaming::ncontext) {
               if (max == 0 || (level + 1)< max) {
                  os << "/" << std::endl; // for contexts
                  // For contexts, first destroy then unbind
                  CORBA::Object_var obj = nc->resolve(bl[i].binding_name);
                  CosNaming::NamingContext_var __nc__
                     = CosNaming::NamingContext::_narrow(obj.in());
                  done &= Show_context(__nc__.in(), os, level + 1, max);
               }
            } else // object
               os << std::endl;
         } catch (const CORBA::Exception & ) {
            // keep silent (ignore)
            done = false;
         } catch (const Cdmw::CommonSvcs::Naming::InvalidNameException &) {
            // This is impossible ! keep silent (ignore)
            done = false;
         }
      }
      return done;
   }

//----------------------------------------------------------------------------

} // namespace Tools
} // namespace Cdmw

