/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <cstdlib>
#include <string>
#include "Foundation/common/System.hpp"
#include "Foundation/common/Exception.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "Repository/tools/NameServiceAdmin.hpp"


namespace {
    const int SUCCESS = 0;
    const int FAILURE = 1;
}; // End anonymous namespace


int run(CORBA::ORB_ptr orb, CosNaming::NamingContext_ptr root)
{
   Cdmw::Tools::NameServiceAdmin adm(orb,root);
   
   adm.run(std::cin,std::cout);
   
   return SUCCESS;
}

int main(int argc, char* argv[])
{
    int status = SUCCESS;
    CORBA::ORB_var orb;

    try
    {
        orb = CORBA::ORB_init(argc, argv);

        CORBA::Object_var obj = CORBA::Object::_nil();
        //
        // Get naming service
        //
        try
        {
           obj = orb -> resolve_initial_references("NameService");
        }
        catch(const CORBA::ORB::InvalidName&)
        {
           std::cerr << argv[0] << ": can't resolve `NameService'" << std::endl;
           return FAILURE;
        }
        
        if(CORBA::is_nil(obj.in()))
        {
           std::cerr << argv[0] << ": `NameService' is a nil object reference" << std::endl;
           return FAILURE;
        }
        
        CosNaming::NamingContext_var nc =
           CosNaming::NamingContext::_narrow(obj.in());
        
        if(CORBA::is_nil(nc.in()))
        {
           std::cerr << argv[0] << ": `NameService' is not a NamingContextExt object "
                     << "reference" << std::endl;
           return FAILURE;
        }
        status = run(orb.in(), nc.in());
    }
    catch(const CORBA::Exception& ex)
    {
        std::cerr << ex << std::endl;
        status = FAILURE;
    }

    if(!CORBA::is_nil(orb.in()))
    {
        try {
            orb -> destroy();
        }
        catch(const CORBA::Exception& ex)
        {
            std::cerr << ex << std::endl;
            status = FAILURE;
        }
    }        

    return status;    
}



