/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef _CDMW_SYSTEM_MNGT_ConfigurationDC_CONTROLLERSYNCRO_H_
#define _CDMW_SYSTEM_MNGT_ConfigurationDC_CONTROLLERSYNCRO_H_

#include <Foundation/common/Exception.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <string>
#include <list>
#include <map>
#include <vector>

namespace Cdmw
{
namespace PlatformMngt
{   

class ConfigurationException : public Exception
{
public:

    ConfigurationException(const std::string& message, size_t lineNumber)
        : Exception( Exception::FATAL, "Invalid configuration file" ),
          m_message(message), m_lineNumber(lineNumber)
    {
    }

    ~ConfigurationException() throw() {};

    const char* getMessage() const
    {
        return m_message.c_str();
    }

    size_t getLineNumber() const
    {
        return m_lineNumber;
    }

private:
    std::string m_message;
    size_t m_lineNumber;
};

class PDaemon{
public:

    /**
     * Purpose:
     * <p> Constructor. Sets default values.
     * 
     */ 
    PDaemon(int port_number){
                   m_port_number = port_number;
    };
    PDaemon(){
    }
    ;
    /**
    *Setter
    */
    void setPort_number(int port_number){
        m_port_number = port_number;
    }
    void setControllerPolicy(std::string policy){
        controlled_policy = policy;
    }
    void setHostStartFile(std::string host_start_file){
        m_host_start_file = host_start_file;
    }
    /**
    *Getter
    */
    int getPort_number(){
        return m_port_number;
    }
    std::string getControllerPolicy(){
        return controlled_policy;
    }
    std::string getHostStartFile(){
        return m_host_start_file;
    }
    
private:
    int m_port_number;
    std::string controlled_policy;
    std::string m_host_start_file;
};
/**
*Purpose:
*<p> 
*
*Features:
*<p> Thread safe
*
*@see  
*
*/
class ConfigurationDC
{

public:

    /**
     * Purpose:
     * <p> Constructor. Sets default values.
     * 
     */ 
    ConfigurationDC();


    /**
     * Purpose:
     * <p> Initializes the configuration from an XML file.
     * 
     * @param xml_conf The XML file path
     * @param validate Validate the XML file before processing
     */ 
    void parse(const std::string& xml_conf, bool validate)
        throw(OutOfMemoryException, ConfigurationException);

    /**
     * Purpose:
     * <p> Destructor
     * 
     */ 
    virtual 
    ~ConfigurationDC()
        throw();

    /**
     * Purpose:
     * <p> Returns the Rate_of_daemon_check optinion.
     */ 
    const int getRate_of_daemon_check() const throw();
    /**
     * Purpose:
     * <p> Returns the MaxNumberOfMissingEchoRequest optinion.
     */ 
    const int getMaxNumberOfMissingEchoRequest() const throw();
    /**
     * Purpose:
     * <p> Returns the MaxNumberOfMissingEchoRequest on startup.
     */ 
    const int getMaxNumberOfMissingEchoRequestInit() const throw();
    /**
     * Purpose:
     * <p> Returns the Platform daemon option of the host.
     */ 
    const std::vector<PDaemon> GetPlatformDaemons() const throw();
    /**
     * Purpose:
     * <p> Returns the Localisation option of the daemon controller.
     */ 
    const int getLocalisation_service() const throw();
    /**
     * Purpose:
     * <p> Returns the isolation-policy option of the daemon controller.
     */ 
    const std::string getIsolation_Policy() const throw();
    /**
     * Purpose:
     * <p> Returns the suspension-timeout option of the daemon controller.
     */ 
    const int getSuspension_Timeout() const throw();    
    /**
     * Purpose:
     * <p> Returns the suspension-timeout option of the daemon controller.
     */ 
    const int getMulticast_Request_Rate() const throw();
    /**
     * Purpose:
     * <p> Returns the  isolation_detection_activated option of the daemon controller.
     */ 
    const bool getIsolation_detection_activated() const throw();
    /**
     * Purpose:
     * <p> Returns the  miop_corbaloc option of the daemon controller.
     */ 
    const std::string getMiop_corbaloc() const throw();


private:   

    std::vector<std::string> eth_interface;
    int rate_of_daemon_check;
    int localisation_service;
    std::vector<PDaemon> platformDaemon;
    int max_number_missing_requests;
    int max_number_missing_requests_init;
    std::string miop_corbaloc;
    int suspension_timeout;
    int multicast_request_rate;
    bool isolation_detection_activated;
}; // End class ConfigurationDC
} // End namespace PlatformMngt
} // End namespace Cdmw

#endif //_CDMW_SYSTEM_MNGT_ConfigurationDC_CONTROLLERSYNCRO_H_
