/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef _CDMW_SYSTEM_MNGT_DaemonController_CONTROLLERSYNCRO_H_
#define _CDMW_SYSTEM_MNGT_DaemonController_CONTROLLERSYNCRO_H_

#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtDaemonController.skel.hpp"
#include "daemoncontroller/DaemonController_impl.hpp"
#include "Foundation/osthreads/Mutex.hpp"
#include "Foundation/osthreads/MutexGuard.hpp"
#include <Foundation/osthreads/Condition.hpp>
#include "daemoncontroller/ConfigurationDC.hpp"
#include <iostream>
#include <string>
#include <time.h> 

namespace Cdmw
{   class DaemonController_impl;
    class  ControllerSyncro
    {
    public:
        
        /**
         * Constructor
         */
        ControllerSyncro(CdmwPlatformMngt::DaemonController_ptr dm, std::string miop_group, CORBA::ORB_ptr orb,int time, 
                         int number_of_req, int number_of_req_init, std::string host_name, int sospension_timeout, 
                         std::vector<Cdmw::PlatformMngt::PDaemon> vpd, bool isolation_detection_activated);
        
        /**
         * Destructor
         */
        virtual
        ~ControllerSyncro()
            throw();
        
        /**
        * send the miop message
        */
        void notify(std::string host_name); 
        
        /**
        * receive a notify 
        */
        void receive(std::string host_name); 
        
        void go() throw ();
        
        /**
        * Get is my invocation
        */        
        bool is_received_invocation();
        
        /**
        * Set is received invocation
        */
        void set_received_invocation(bool value);
        
        /**
        * Set is  isolated
        */
        void set_isolation(short value);
        
        /**
        * notify to DaemonController
        */
        void notify_to_DC(short code, long port_number);
        
        /**
        * Get the DaemonController
        */
        CdmwPlatformMngt::DaemonController_var get_DaemonController();
        
        
    private:
        /*
        * Get random number
        */
        int random_time(int max);
        
        
        
    public:    
        CdmwPlatformMngt::DaemonController_var m_dm;
        CdmwPlatformMngt::DaemonController_var no_multicast_dm;
        std::string m_miop_addres;
        Cdmw::DaemonController_impl*  daemoncontroller;
        CORBA::ORB_var m_orb;
        std::string m_miop_group;
        
    
    private:
        unsigned long w_time; //10^-7 sec to wait
        unsigned long max_time; //10^-7 max sec to wait
        unsigned long time_random; //10^-7 random sec to wait
        unsigned long total_time; //10^-7 total sec to wait total_time=w_time+time_random or total_time=max_time
        Cdmw::OsSupport::Mutex m_mutex;
        bool invocation_received; // tell my if i have received the notification, if is my invocation me don't notify again
        int num_of_my_invocation; // numeber of my consecutiv invocation
        int try_miop_request;
        int try_miop_request_init;
        std::string m_host_name;
        Cdmw::OsSupport::Condition m_condition;
        bool isolated;  
        std::string m_host_policy; 
        std::vector<Cdmw::PlatformMngt::PDaemon> m_vpd;
        clock_t now, prew;
        bool first_time_isolation;
        bool start_up;
        long long first_time_millisec;
        unsigned int m_sospension_timeout;
        bool m_isolation_detection_activated;
        bool startup;
    }; // End class ControllerSyncro
    
}; // End namespace Cdmw

#endif //_CDMW_SYSTEM_MNGT_DaemonController_CONTROLLERSYNCRO_H_
