/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
#ifndef _CDMW_SYSTEM_MNGT_DaemonController_DAEMONCONTROLLER_H_
#define _CDMW_SYSTEM_MNGT_DaemonController_DAEMONCONTROLLER_H_

#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtDaemonController.skel.hpp"
#include "idllib/CdmwPlatformMngtPeriodicObserver.skel.hpp"
#include "daemoncontroller/ControllerSyncro.hpp"
#include "daemoncontroller/ConfigurationDC.hpp"
#include <iostream>
#include <string>
#ifdef CDMW_HAS_BOOST
#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>
#endif

namespace Cdmw
{
    class ControllerSyncro;
    class DaemonController_impl :
        public virtual POA_CdmwPlatformMngt::DaemonController,
        virtual public PortableServer::RefCountServantBase
    {
    public:
        
        /**
         * Constructor
         */
        DaemonController_impl(Cdmw::ControllerSyncro* cs, int sospension_timeout, std::vector<Cdmw::PlatformMngt::PDaemon> vpd)
            throw(CORBA::SystemException);
        
        /**
         * Destructor
         */
        virtual
        ~DaemonController_impl()
            throw();
        
        /**
        * utility function 
        */
        void execute_shell_command(std::string command)
        throw (CORBA::NO_PERMISSION); 
        
        void my_notification()
            throw (CORBA::NO_PERMISSION);
            
        void set_isolated(short value)
            throw (CORBA::NO_PERMISSION);
        
        std::string get_platform_daemon_policy(CORBA::Long port_number);
        std::string get_host_start_file(CORBA::Long port_number);
        bool kill_sons_pid_of_daemon_policy(CORBA::Long port_number);
        bool recursive_process_kill(std::string);
        /**
         * IDL interface implementation
         */
        virtual void notify (short exitcode, CORBA::Long port_number)
                throw(CORBA::SystemException);

        virtual void miop_request (const char* host_name)
            throw(CORBA::SystemException);
        
        virtual short is_isolated () throw(CORBA::SystemException);
        /**
         * End nterface implementation    
         */
        
        Cdmw::ControllerSyncro* m_cs;
        int m_sospension_timeout; 
        std::vector<Cdmw::PlatformMngt::PDaemon> m_vpd;
        
    private:
        short isolated;
        bool know_status;
    }; // End class daemoncontroller
    
}; // End namespace Cdmw

#endif //_CDMW_SYSTEM_MNGT_DaemonController_DAEMONCONTROLLER_H_
