/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <cstdlib>

#include "daemoncontroller/ConfigurationDC.hpp"
#include "Foundation/common/Assert.hpp"

#include <sstream>
#include <xercesc/dom/deprecated/DOM.hpp>
#include <xercesc/dom/deprecated/DOMParser.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/sax/ErrorHandler.hpp>
#include <xercesc/sax/SAXParseException.hpp>

#include "Foundation/orbsupport/CORBA.hpp"
#include "idllib/CdmwPlatformMngtCommon.stub.hpp"

using namespace xercesc;

namespace Cdmw
{
namespace PlatformMngt
{


class  ErrorReporter : public ErrorHandler
{
public:
    // -----------------------------------------------------------------------
    //  Constructors and Destructor
    // -----------------------------------------------------------------------
    ErrorReporter()
    {
    }

    ~ErrorReporter()
    {
    } 


    // Each error reported by the parsing is translated into an exception to be
    //  catch in the main procedure
    void warning(const SAXParseException& ex) { 
        throw ex; 
    }

    void error(const SAXParseException& ex)
    {
        throw ex;
    }

    void fatalError(const SAXParseException& ex)
    {
        throw ex;

    }

    void resetErrors() { 
    }
};




//
// Convert a Xerces string into a C++ string
std::string to_string(const DOMString &s)
{
    char *c_str = s.transcode();

    std::string str(c_str);

    delete c_str;
    
    return str;

}



//
// Returns the value associated to an Element node
//  
inline 
std::string
get_value(const DOM_Node& node)
{
    DOMString domString = node.getFirstChild().getNodeValue();
    return to_string( domString );

}


ConfigurationDC::ConfigurationDC()

{       
        localisation_service = 0;
        rate_of_daemon_check = 0;
        max_number_missing_requests = 0;
    max_number_missing_requests_init = -1;
        isolation_detection_activated=false;
    miop_corbaloc = "";
}


void ConfigurationDC::parse(const std::string& xml_conf, bool validate)
    throw(OutOfMemoryException, ConfigurationException)
{
    using namespace std;            
    
    try {
    
        // Xerces initialisation
        XMLPlatformUtils::Initialize();
    
        // We parse the configuration file
        DOMParser *parser = new DOMParser;
        if (validate)
        {
            parser->setValidationScheme(DOMParser::Val_Auto);
            parser->setDoNamespaces(false);
        }
        ErrorReporter errorReporter;
        parser->setErrorHandler( &errorReporter );

        parser->parse( xml_conf.c_str() );

        DOM_Document doc = parser->getDocument();
        // test if the dtd used for the xml file is the good
        if (!doc.getDoctype().getName().equals("CdmwDaemonController"))
        {
            CDMW_THROW2(ConfigurationException,
                        "The XML file used is not the good type of file for the Daemon Controller ",
                        0);
        }
        

        // We build a TreeWalker to skip all comment and automatically resolve
        // the entities
        DOM_TreeWalker walker = doc.createTreeWalker(doc,
        DOM_NodeFilter::SHOW_ELEMENT |
        DOM_NodeFilter::SHOW_ATTRIBUTE |
        DOM_NodeFilter::SHOW_TEXT |
        DOM_NodeFilter::SHOW_DOCUMENT_TYPE,
        NULL, true);

        // The first child is the DTD definition of the XML data, we skip it
        walker.firstChild();
      
        // We obtain the "CdmwDaemonController" node
        DOM_Node node_root = walker.nextSibling();
        DOM_Element m_element_root = (xercesc::DOM_Element &) node_root;
        rate_of_daemon_check = atoi(to_string(m_element_root.getAttribute(DOMString::transcode( "rate-daemon-check" ) ) ).c_str());
        localisation_service = atoi(to_string(m_element_root.getAttribute(DOMString::transcode( "localisation-service" ) ) ).c_str());
        // Each element configuration is under the root node,
        // so we get the first one
        DOM_Node node = walker.firstChild();

        // We can iter on each configuration element
        while (node != NULL) { 
        
            if (node.getNodeName().equals("IsolationDetection"))
            {
                DOM_Element m_element = (xercesc::DOM_Element &) node;                
                max_number_missing_requests = atoi(to_string(m_element.getAttribute(DOMString::transcode("max-number-missing-requests"))).c_str());
                max_number_missing_requests_init =  atoi(to_string(m_element.getAttribute(DOMString::transcode("max-number-missing-requests-init"))).c_str()); 
                suspension_timeout = atoi(to_string(m_element.getAttribute(DOMString::transcode("suspension-timeout"))).c_str());   
                multicast_request_rate =  atoi(to_string(m_element.getAttribute(DOMString::transcode("multicast-request-rate"))).c_str()); 
                std::string isolation = to_string(m_element.getAttribute(DOMString::transcode("isolation-detection-activated")));
                miop_corbaloc = to_string(m_element.getAttribute(DOMString::transcode("miop-corbaloc")));
                if(isolation=="true"){
                    isolation_detection_activated = true;
                }
            } 
            if (node.getNodeName().equals("PlatformDaemon"))
            {
                PDaemon pd;
                DOM_Element m_element = (xercesc::DOM_Element &) node;
                std::string port_number = to_string(m_element.getAttribute(DOMString::transcode( "port-number" ) ) );
                pd.setPort_number(atoi(port_number.c_str()));
                std::string ControllerPolicy = to_string(m_element.getAttribute(DOMString::transcode( "controller-policy" ) ) );
                pd.setControllerPolicy(ControllerPolicy);
                std::string HostStartFile = to_string(m_element.getAttribute(DOMString::transcode( "host-start-file" ) ) );
                pd.setHostStartFile(HostStartFile);      
                platformDaemon.push_back(pd);
            }
            // Next Element to be processed
            //walker.parentNode();
            node = walker.nextSibling();
        }

        delete parser;
    }
    catch(const SAXParseException& ex1)
    {
        CDMW_THROW2(ConfigurationException,
            to_string(DOMString(ex1.getMessage())), ex1.getLineNumber());
    }
    catch(const XMLException& ex2)
    {
        CDMW_THROW2(ConfigurationException,
           to_string(DOMString(ex2.getMessage())), ex2.getSrcLine());

    }
    catch ( std::bad_alloc& )
    {
        CDMW_THROW(OutOfMemoryException);
    }

}


ConfigurationDC::~ConfigurationDC()
    throw()
{
};

const int ConfigurationDC::getRate_of_daemon_check() const throw()
{
    return rate_of_daemon_check;
}

const int ConfigurationDC::getMaxNumberOfMissingEchoRequest() const throw()
{
    return max_number_missing_requests;
}

const int ConfigurationDC::getMaxNumberOfMissingEchoRequestInit() const throw()
{
    return max_number_missing_requests_init;
}


const int ConfigurationDC::getLocalisation_service() const throw()
{
    return localisation_service;
}

const int ConfigurationDC::getSuspension_Timeout() const throw()
{
    return suspension_timeout;
}

const int ConfigurationDC::getMulticast_Request_Rate() const throw()
{
    return multicast_request_rate;
}

const bool ConfigurationDC::getIsolation_detection_activated() const throw()
{
    return isolation_detection_activated;
}

const std::vector<PDaemon> ConfigurationDC::GetPlatformDaemons() const throw()
{
    return platformDaemon;
};

const std::string ConfigurationDC::getMiop_corbaloc() const throw()
{
    return miop_corbaloc;
};

} // End namespace PlatformMngt
} // End namespace Cdmw


