/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "daemoncontroller/ControllerLogMngr.hpp"
#include "SystemMngt/platformlibrary/CommonLogMessageId.hpp"
#include "daemoncontroller/ControllerLogMessageId.hpp"
#include "Foundation/logging/LogManager.hpp"


namespace Cdmw
{
namespace PlatformMngt
{

    
const char* messages[] = {
            "---------DAEMON CONTROLLER START-------",
            "Waiting platform daemon with port %i",
            "platform daemon with port %i find!!!",
            "Register daemon controller in daemon's (port = %i local services with name DaemonController",
            "Start controller for platform daemon with port %i",
            "Platform daemon with pid %i and port %i dies",
            "Invalid pid process: %i",
            "Notify exit action %i for platform daemon with port %i and policy %s",
            "Stop request for system with platform daemon port %i, stop host in progress",
            "Shutdown request , shutdown in progress",
            "Restart request, restart of the system with platform daemon port %i using start file %s",
            "Host isolated, stop all systems",
            "Request to monitor unknown",
            "Error in register_local_service, BAD PARAM for platform daemon with port %i",
            "Daemon with port %i dies",
            "Reboot request, reboot in progress",
            "Restart of the system with platform daemon port %i using start file %s",
            "No action for the dies of the daemon with port %i",
            "Killing system with daemon port %i, and clean the platform daemon file %s",
            "Opening: %s",
            "Deleting %s",
            "Path not found",
            "Killing process with pid %s",
        "Change status of host, new status = %s"
};

void ControllerLogMngr::initialise(int& argc, char** argv)
     throw (OutOfMemoryException, InternalErrorException)
{
    // if init not already done
    if (M_pSingleton == NULL)
    {
        try 
         {
            // create the concrete Log Manager singleton
            M_pSingleton = new ControllerLogMngr(argc, argv);
        }
        catch (const std::bad_alloc&)
         {
            CDMW_THROW (OutOfMemoryException);
        }
    }
}


ControllerLogMngr::ControllerLogMngr(int& argc, char** argv)
        throw(OutOfMemoryException, InternalErrorException)
   : LogMngr(argc, argv)
{
}
            
             
ControllerLogMngr::~ControllerLogMngr()
        throw()
{
}


const char* ControllerLogMngr::getService() const
{
    return "CDMW.PFMCTR";
}

    
const char* ControllerLogMngr::getMessage(long messageId) const
{
    if (messageId >= MSG_ID_COMMON_START && messageId < MSG_ID_COMMON_END)
    {
        return CommonLogMessages::M_commonMessages[messageId - MSG_ID_COMMON_START];
    }
    else if (messageId >= 0 && messageId < MSG_ID_CONTROLLER_END)
    {
        return messages[messageId];
    }
    else
    {
        return NULL;
    }
}


} // End namespace PlatformMngt
} // End namespace Cdmw


