/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "daemoncontroller/ControllerSyncro.hpp"
#include "SystemMngt/tools/SingleShotTimer.hpp"
#include <Foundation/commonsvcs/time/Util.hpp>
#include "Foundation/orbsupport/OrbSupport.hpp"
// Standard Files
#include <fstream>
#include <ctime>

namespace
{
    const int PERC_RANDOM_TIME = 5; // random time < time/PERC_RANDOM_TIME
    const int B_TIME = 10; //min multicast request time, 10^-7 min sec to wait (w_time>=b_time)
}

namespace Cdmw
{
    ControllerSyncro::ControllerSyncro(CdmwPlatformMngt::DaemonController_ptr dm, 
                                       std::string miop_group, 
                                       CORBA::ORB_ptr orb, 
                                       int time,       
                                       int number_of_req,
                       int number_of_req_init,
                                       std::string host_name,
                                       int sospension_timeout, 
                                       std::vector<Cdmw::PlatformMngt::PDaemon> vpd,
                                       bool isolation_detection_activated
                                        ): m_condition(m_mutex)
    {    
        set_received_invocation(false);
    startup = true;
        m_isolation_detection_activated = isolation_detection_activated;
        isolated = 0;
        num_of_my_invocation=0;
        max_time = time*2;
        try_miop_request = number_of_req;
    try_miop_request_init = number_of_req_init;
        m_host_name = host_name;
        m_vpd = vpd;
        first_time_isolation= true;
        first_time_millisec=0;
        m_sospension_timeout = sospension_timeout;
        if(time>B_TIME){
            w_time=time;
        } 
        else w_time = B_TIME;
        time_random = random_time(w_time/PERC_RANDOM_TIME);
        total_time = time_random+w_time;
        m_orb = CORBA::ORB::_duplicate(orb);
        m_miop_group = miop_group;
        daemoncontroller = new Cdmw::DaemonController_impl(this, m_sospension_timeout, m_vpd);
        
        CORBA::Object_var obj = m_orb->resolve_initial_references("RootPOA");
        //id_to_reference on the GOA
        PortableGroup::GOA_var rootGOA = PortableGroup::GOA::_narrow(obj.in());
        PortableServer::ObjectId_var obj_id = rootGOA->activate_object(daemoncontroller);
        //id_to_reference on the POA
        PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj.in());
        CORBA::Object_var daeV = rootPOA->id_to_reference(obj_id);
        CORBA::Object_var miop_ref = m_orb->string_to_object(m_miop_group.c_str());
        rootGOA->associate_reference_with_id (miop_ref.in(), obj_id);
        m_dm = CdmwPlatformMngt::DaemonController::_unchecked_narrow (miop_ref.in());
        no_multicast_dm = CdmwPlatformMngt::DaemonController::_narrow (daeV.in());
        OrbSupport::OrbSupport::bind_object_to_corbaloc(m_orb.in(), "daemon_controller", no_multicast_dm.in() );;
    }
    
    void ControllerSyncro::notify(std::string host_name)
    {   
        total_time = max_time;
        num_of_my_invocation++;
        //std::cout << m_host_name << ": Invocatin number " << num_of_my_invocation << std::endl;
        m_dm->miop_request(host_name.c_str());
    } 
    
    void ControllerSyncro::go() throw (){
        while(true){
            CDMW_MUTEX_GUARD(m_mutex);
            m_condition.wait(total_time);
            if(!is_received_invocation()){
                notify(m_host_name);
            }
            set_received_invocation(false);
        }
    }
    
    void ControllerSyncro::receive(std::string host_name)
    {
         // std::cout << std::endl << m_host_name << ": INVOCATION BY " << host_name << std::endl; ;
         if(host_name == m_host_name){ // if is my invocation
     // std::cout << m_host_name << ": AUTOINVOCATION NUM " << num_of_my_invocation  << std::endl;
             if( (num_of_my_invocation>try_miop_request && !startup) || (num_of_my_invocation>try_miop_request_init && startup)){
                 if(first_time_isolation){
                     first_time_isolation = false;
                     first_time_millisec = Cdmw::CommonSvcs::Time::Util::get_current_time()/10000;
                     //std::cout << m_host_name << ": first_time_millisec: " << first_time_millisec  << std::endl; 
                 }
                 else{
                     if(m_sospension_timeout < ((Cdmw::CommonSvcs::Time::Util::get_current_time()/10000) - first_time_millisec)){
                       //  std::cout << m_host_name << ": m_sospension_timeout: " << m_sospension_timeout  <<  " < " 
                                 //  << ((Cdmw::CommonSvcs::Time::Util::get_current_time()/10000) - first_time_millisec) << std::endl; 
                         set_isolation(1);
                     }         
                 }
             } 
         }  
         else{ 
         startup = false;   
             first_time_isolation = true;    
             set_received_invocation(true);   
             set_isolation(2);
             CDMW_MUTEX_GUARD(m_mutex);  
             m_condition.broadcast();
             total_time = w_time + random_time(w_time/PERC_RANDOM_TIME);        
             num_of_my_invocation = 0;   
         }              
         // std::cout << m_host_name << ": Wait time: " << total_time << std::endl;
         // std::cout << m_host_name << ": Isolation state:  " << no_multicast_dm->is_isolated() << std::endl;
    } 
    
    int ControllerSyncro::random_time(int max){
        srand(Cdmw::CommonSvcs::Time::Util::get_current_time());
        return (rand() % max);
    }
    ControllerSyncro::~ControllerSyncro() throw()
    {}
    
    void ControllerSyncro::set_received_invocation(bool value)
    {
        //CDMW_MUTEX_GUARD(m_mutex_invocation_received);
        invocation_received = value;
    }
            
    bool ControllerSyncro::is_received_invocation()
    {
        //CDMW_MUTEX_GUARD(m_mutex_invocation_received);
        return invocation_received;
    }
    
    void ControllerSyncro::set_isolation(short value)
    {
        if(m_isolation_detection_activated){
            //CDMW_MUTEX_GUARD(m_mutex_invocation_received);
            isolated = value;
            if(value==1){
                notify_to_DC(6,0);
            }
    }
        no_multicast_dm->set_isolated(value);
    }
    
    void ControllerSyncro::notify_to_DC(short code, long port_number){
        no_multicast_dm->notify(code, port_number);
    }
    
    CdmwPlatformMngt::DaemonController_var ControllerSyncro::get_DaemonController()
    {
        return no_multicast_dm;
    }
    
}
