/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/common/Options.hpp"
#include "Foundation/common/String.hpp"
#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/osthreads/ThreadHandle.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include <SystemMngt/idllib/CdmwPlatformMngtDaemon.stub.hpp>
#include <cstdlib>
#include <string>
#include <exception>
#include <sstream>
#include <iostream>
#include <fstream>
//#include "daemoncontroller/DaemonController_impl.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtDaemonController.skel.hpp"
#include "daemoncontroller/ConfigurationDC.hpp"
#include "daemoncontroller/ControllerSyncro.hpp"
#include "platformperiodicnotifier/Notifier_impl.hpp"
#include "daemoncontroller/ControllerLogMngr.hpp"
#include "daemoncontroller/ControllerLogMessageId.hpp"

using namespace Cdmw::OsSupport;
using namespace Cdmw::OrbSupport;  
using namespace Cdmw::PlatformMngt;
using namespace Cdmw::Common;  

namespace
{
    const std::string DAEMON_CONTROLLER_FILE_OPTION = "--CdmwXMLFile";
    const std::string DAEMON_PORT_OPTION = "--daemonport-number";
    const std::string BEXITACTION = "--exit-action";
    const std::string NAME_DAEMON_CORBALOC = "CdmwPlatformMngtDaemon";
    const std::string FILE_LOG = "/tmp/daemoncontrollerlog.txt";
    const int POA_THREAD_POOL_SIZE = 5;
    const int MIN_RATE_DAEMON_CHECK = 50;
}
 
//Find the platforl daemon controller in file /tmp/CdmwDaemon_<port>/ __CDMW_DAEMON__.pid
int findDaemonPid(std::string daemonport){
    int daemon_pid = -1;
    
    std::string file_name = "/tmp/CdmwDaemon_" + daemonport + "/__CDMW_DAEMON__.pid";
    std::ifstream fin; 
    fin.open(file_name.c_str());
    LogMngr::logMessage(INF,MSG_ID_CONTROLLER_WAINTING_DAEMON_WITH_PORT,atoi(daemonport.c_str()));
    while(!fin.is_open()) {
        OS::sleep(1);
        fin.open(file_name.c_str());
    }
    std::ifstream is(file_name.c_str());
    LogMngr::logMessage(INF,MSG_ID_CONTROLLER_DAEMON_WITH_PORT_FOUND,atoi(daemonport.c_str()));
    std::string objref_str;
    is >> objref_str;
    daemon_pid = atoi(objref_str.c_str());
    return daemon_pid;
}   

class Pinger : public Cdmw::OsSupport::Thread
{
    public:
       Pinger::Pinger (Cdmw::ControllerSyncro* cs) 
       {
           m_cs = cs;
       }
       
       Pinger::Pinger () 
       {
       }
        Pinger::~Pinger()
                throw(){};
        void Pinger::run() throw(){
              m_cs->go();
        }
        
        Cdmw::ControllerSyncro* m_cs;
};

class DaemonChecker : public Cdmw::OsSupport::Thread
{
    public:
        int localDaemonPid;
        CdmwPlatformMngt::DaemonController_var localdm;
        std::string localplatform_daemon_port;
        CORBA::ORB_var localorb;
        Pinger* m_p;
        int m_rate_of_daemon_check;
        /**
        * Purpose:
        * <p> Constructor
        * 
        *@param DaemonPid
        *
        */ 
        DaemonChecker::DaemonChecker (Pinger* p,CdmwPlatformMngt::DaemonController_ptr dm, std::string platform_daemon_port, 
        CORBA::ORB_ptr orb, int rate_of_daemon_check){
        localDaemonPid = 0;
        localdm = CdmwPlatformMngt::DaemonController::_duplicate(dm);
        localplatform_daemon_port = platform_daemon_port;
        localorb = CORBA::ORB::_duplicate(orb);
        m_p = p;
        if(MIN_RATE_DAEMON_CHECK < rate_of_daemon_check){
            m_rate_of_daemon_check = rate_of_daemon_check;
        }
        else{
            m_rate_of_daemon_check = MIN_RATE_DAEMON_CHECK;
        }
            
    };
        /**
        * Purpose:
        * <p> Destructor
        * 
        */  
        DaemonChecker::~DaemonChecker()
                throw(){};

        void DaemonChecker::run() throw(){
        while(true){
            int localDaemonPid = findDaemonPid(localplatform_daemon_port);
        int localDaemonPort = atoi(localplatform_daemon_port.c_str());
            sleep(2000);     
            try {
                LogMngr::logMessage(INF,MSG_ID_CONTROLLER_REGISTER_IN_DAEMON, localDaemonPort);
                std::string daemon_corbaloc =  "corbaloc::localhost:"+ localplatform_daemon_port + "/" + NAME_DAEMON_CORBALOC;
                CORBA::Object_var obj_daemon = localorb->string_to_object(daemon_corbaloc.c_str());
                CdmwPlatformMngt::Daemon_var daemon = CdmwPlatformMngt::Daemon::_narrow(obj_daemon.in());
                daemon->register_local_service("DaemonController", ((m_p->m_cs)->get_DaemonController()).in());
                LogMngr::logMessage(INF,MSG_ID_CONTROLLER_START_FOR_DAEMON,localDaemonPort);
            }   
            catch (const CORBA::BAD_PARAM& e) {
                LogMngr::logMessage(ERR,MSG_ID_CONTROLLER_REGISTER_LOCAL_SERVICE_ERROR,atoi(localplatform_daemon_port.c_str()));
            }
            catch (const CORBA::SystemException& e) {
                // not yet started        
            }  
            if(localDaemonPid != -1){
                try {
                    while(true) {
                        OS::kill_process(localDaemonPid, 0);
                        OS::sleep(m_rate_of_daemon_check);
                    } 
                }
                catch(...) {
                    LogMngr::logMessage(INF,MSG_ID_CONTROLLER_DAEMON_WITH_PID_PORT_DIES, localDaemonPid, localDaemonPort);           
                }
            }
            else {
                LogMngr::logMessage(INF,MSG_ID_CONTROLLER_INVALID_PID_PROCESS,localDaemonPid);   
            }
            (m_p->m_cs)->notify_to_DC(0,atoi(localplatform_daemon_port.c_str()));
        }
    }
};

static void
usage(std::ostream & os, const std::string &program_name)
{
    os << "Usage:\n"
       << program_name << " options\n"
       << "\n"
       << "Options:\n"
       << "--help                                   Display this information.\n"
       << "[--daemonport-number=<number>]           The port number of the platform daemon.\n"
       << "[--CdmwLocalisationService=<number>]     The port number of the Controller.\n"
       << std:: endl;
}
  
class OrbThread : public Cdmw::OsSupport::Thread 
{

    public:
        
        OrbThread(CORBA::ORB_ptr orb)
            : m_orb(CORBA::ORB::_duplicate(orb))
        {};
         
            
        ~OrbThread() throw()
        {};
    
    
        void shutdown()
        {
            m_orb->shutdown(true);
        };
    
    protected:

        void run() throw()
        {
            m_orb->run();
        };
        
    private:
        CORBA::ORB_var  m_orb;
};
  
    
int main(int argc, char* argv[])
{    
    std::string XML_file_option = Cdmw::OsSupport::OS::get_option_value( argc, argv, DAEMON_CONTROLLER_FILE_OPTION);
    std::string log_file = Cdmw::OsSupport::OS::get_option_value( argc, argv, Cdmw::Common::Options::LOG_FILE_OPTION);
    Cdmw::PlatformMngt::ConfigurationDC opt;
    opt.parse(XML_file_option, true);  
    std::vector<Cdmw::PlatformMngt::PDaemon> v = opt.GetPlatformDaemons();
    std::string miop_group (opt.getMiop_corbaloc());
    int MaxNumberOfMissingEchoRequestInit (opt.getMaxNumberOfMissingEchoRequestInit());
    if("" == miop_group  ){
        miop_group = "corbaloc:miop:1.0@1.0-cdmwdaemoncontroller-10/225.1.1.8:5555";
    }  
    if(-1 == MaxNumberOfMissingEchoRequestInit  || 0 == MaxNumberOfMissingEchoRequestInit){
        MaxNumberOfMissingEchoRequestInit  = opt.getMaxNumberOfMissingEchoRequest();
    }    
    /*
    std::cerr <<  "Miop corbaloc: " << miop_group << std::endl;  
    std::cerr <<  "Rate_of_daemon_check: " << opt.getRate_of_daemon_check() << std::endl;  
    std::cerr << "Number of daemon: " << v.size() << std::endl;
    std::cerr << "Localisation service:  " << opt.getLocalisation_service() << std::endl;
    std::cerr << "MaxNumberOfMissingEchoRequest:  " << opt.getMaxNumberOfMissingEchoRequest() << std::endl;
    std::cerr << "MaxNumberOfMissingEchoRequest on init :  " << MaxNumberOfMissingEchoRequestInit << std::endl;
    std::cerr << "suspension-timeout:  " << opt.getSuspension_Timeout() << std::endl;
    std::cerr << "multicast_request_rate: " << opt.getMulticast_Request_Rate() << std::endl;
    std::cerr <<  "Log file: " << log_file << std::endl;  
    if(opt.getIsolation_detection_activated()){
        std::cerr << "Isolation detection activated!!!" << std::endl;
    }
    else{
        std::cerr << "Isolation detection NOT activated!!!" << std::endl;
    }
    for (std::vector<Cdmw::PlatformMngt::PDaemon>::iterator itrCollector = v.begin(); 
         itrCollector != v.end(); itrCollector++)
    {
        std::cerr << std::endl << "------------------------------------------------"<< std::endl;
        std::cerr << "port-number:                    " << (*itrCollector).getPort_number() << std::endl;
        std::cerr << "Controller-Policy:                    " << (*itrCollector).getControllerPolicy() << std::endl;
        std::cerr << "Host-start-file:                    " << (*itrCollector).getHostStartFile() << std::endl;
        std::cerr << "------------------------------------------------"<< std::endl;
    }
    */
    // the return code 
    int ret_code = 0;
    StrategyList strategy;
    strategy.add_OrbThreaded();
    strategy.add_PoaThreadPool(30);
    strategy.add_multicast();
    CORBA::ORB_var orb;

    //
    // help argument
    //
    if (argc > 1)
    {
        if(strcmp(argv[1], "--help") == 0)
        {
            usage(std::cout, argv[0]);
            return ret_code;
        }
    }
    if (argc == 1)
    {
        usage(std::cout, argv[0]);
        return ret_code;   
    }
   
    // set localisation_port
    char** ORB_init_argv = 0;
    int ORB_init_argc=argc;
    
    if (opt.getLocalisation_service() == 0)
    {
        std::string str_opt( "" );
        for (int i=0; i<argc; i++) str_opt += std::string( argv[i] ) + " ";
        str_opt += std::string(Cdmw::Common::Options::LOCALISATION_SERVICE_OPTION );
        str_opt += "=" ;
        str_opt += "5051";
        if(""==log_file || "no"==log_file){
            str_opt += Cdmw::Common::Options::LOG_FILE_OPTION;
            str_opt += "=" ;
            str_opt += FILE_LOG;
            std::cout << "Parametri: "<< str_opt << std::endl;
            ORB_init_argc += 2;
        }
        else{
        ORB_init_argc += 1;
        }
        ORB_init_argv = Cdmw::Common::String::to_char_array(
        Cdmw::Common::String::to_strings( str_opt ) );
    }
    else{
        std::string str_opt( "" );
        for (int i=0; i<argc; i++) str_opt += std::string( argv[i] ) + " ";
        str_opt += std::string(Cdmw::Common::Options::LOCALISATION_SERVICE_OPTION );
        str_opt += "=" ;
        
        std::ostringstream ls;
        ls << opt.getLocalisation_service();
        str_opt += ls.str().c_str();
        str_opt += " " ;
        if(""==log_file || "no"==log_file){
            str_opt += Cdmw::Common::Options::LOG_FILE_OPTION;
            str_opt += "=" ;
            str_opt += FILE_LOG;
            std::cout << "Parametri: "<< str_opt << std::endl;
            ORB_init_argc += 2;
    }
    else{
        ORB_init_argc += 1;
    }
        ORB_init_argv = Cdmw::Common::String::to_char_array(
        Cdmw::Common::String::to_strings( str_opt ) );
    } 
    Cdmw::PlatformMngt::ControllerLogMngr::initialise(ORB_init_argc, ORB_init_argv);
    LogMngr::logMessage(INF,MSG_ID_CONTROLLER_START);
    
    orb =  Cdmw::OrbSupport::OrbSupport::ORB_init(ORB_init_argc, ORB_init_argv, strategy);     
    CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
    PortableServer::POA_var poa = PortableServer::POA::_narrow(obj.in());
    PortableServer::POAManager_var poaMgr = poa->the_POAManager();
    poaMgr->activate();  
    
    CdmwPlatformMngt::DaemonController_var dm;
    Cdmw::ControllerSyncro* cs = new Cdmw::ControllerSyncro(dm.in(),
                                                            miop_group, 
                                                            orb.in(),
                                                            opt.getMulticast_Request_Rate(),
                                                            opt.getMaxNumberOfMissingEchoRequest(),
                                                            MaxNumberOfMissingEchoRequestInit,
                                                            Cdmw::OsSupport::OS::get_hostname(), 
                                                            opt.getSuspension_Timeout(), 
                                                            opt.GetPlatformDaemons(),
                                                            opt.getIsolation_detection_activated());
    Pinger* p = new Pinger(cs);
    p->start();
    
    std::vector<DaemonChecker*> DC;
    for (std::vector<Cdmw::PlatformMngt::PDaemon>::iterator itrCollector = v.begin(); 
         itrCollector != v.end(); itrCollector++)
    {
        std::ostringstream s1;
        s1 << (*itrCollector).getPort_number();
        DaemonChecker* dmc = new DaemonChecker(p, dm.in(),s1.str(), orb.in(), opt.getRate_of_daemon_check());
        DC.push_back(dmc);
    }
    for (std::vector<DaemonChecker*>::iterator itrCollector = DC.begin(); 
         itrCollector != DC.end(); itrCollector++)
    {
         (*itrCollector)->start();
    }
    orb->run();
}






