/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
 
#include "daemoncontroller/DaemonController_impl.hpp"
#include "daemoncontroller/ControllerSyncro.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/common/Options.hpp"
#include "Foundation/common/String.hpp"
#include "daemoncontroller/ControllerLogMngr.hpp"
#include "daemoncontroller/ControllerLogMessageId.hpp"
#include <sstream>
#include <fstream>
using namespace Cdmw::OsSupport;
using namespace Cdmw::OrbSupport;  
using namespace Cdmw::PlatformMngt;
using namespace Cdmw::Common;  
#ifdef CDMW_HAS_BOOST
namespace fs = boost::filesystem;
#endif
namespace{
    const std::string FILE_LOG = "/tmp/daemoncontrollerlog.txt";
    const std::string m_start_host_script = "cdmw_host_start.sh";
    const std::string m_reboot_host_script = "cdmw_host_reboot.sh";
    const std::string m_shutdown_host_script = "cdmw_host_shutdown.sh";
}


namespace Cdmw
{      
    bool oldNotification = false;
    DaemonController_impl::DaemonController_impl(Cdmw::ControllerSyncro* cs, int sospension_timeout, std::vector<Cdmw::PlatformMngt::PDaemon> vpd)
        throw(CORBA::SystemException)
    {
       m_cs = cs;
       m_vpd = vpd;
       m_sospension_timeout = sospension_timeout;
       isolated = 0;
       know_status = false;
    }
    
    DaemonController_impl::~DaemonController_impl()
        throw()
    {
    }
    
    std::string DaemonController_impl::get_platform_daemon_policy(CORBA::Long port_number){
        std::string my_policy="NONE";
        for (std::vector<Cdmw::PlatformMngt::PDaemon>::iterator itrCollector = m_vpd.begin(); itrCollector != m_vpd.end(); itrCollector++)
        {
           if(port_number == (*itrCollector).getPort_number()){
               my_policy = (*itrCollector).getControllerPolicy();
           } 
        }
        return my_policy;
    }
    
    std::string DaemonController_impl::get_host_start_file(CORBA::Long port_number){
        std::string start_file="NONE";
        for (std::vector<Cdmw::PlatformMngt::PDaemon>::iterator itrCollector = m_vpd.begin(); itrCollector != m_vpd.end(); itrCollector++)
        {
           if(port_number == (*itrCollector).getPort_number()){
               start_file = (*itrCollector).getHostStartFile();
           } 
        }
        return start_file;
    }
    
    bool DaemonController_impl::kill_sons_pid_of_daemon_policy(CORBA::Long port_number){     
        std::ostringstream mex;
        mex << "/tmp/CdmwDaemon_" << port_number;
        if(isolated!=1){
            LogMngr::logMessage(INF,MSG_ID_KILLING_AND_CLEANING, port_number, mex.str());   
        }
    recursive_process_kill(mex.str());
    return true;
    }
    
    bool DaemonController_impl::recursive_process_kill(std::string path){
        #ifdef CDMW_HAS_BOOST
        try{ 
        fs::path full_path = fs::system_complete(fs::path(path, fs::native));
        if ( fs::is_directory( full_path ) )
        {
            fs::directory_iterator end_iter;
            for ( fs::directory_iterator dir_itr( full_path );
                  dir_itr != end_iter;
                  ++dir_itr )
            {    
                try
                {
                    std::ostringstream mex;
                    mex << full_path.native_file_string() << "/" << dir_itr->leaf(); 
                    //std::cerr << " - " << mex.str() << std::endl;
                    fs::path full_path = fs::system_complete(fs::path(mex.str(), fs::native));
                    if ( fs::is_directory( full_path ) )
                    {
                       recursive_process_kill(mex.str());    
                    }
                    else
                    {
                        std::ifstream is((mex.str().c_str()));
                        std::string objref_str;
                        is >> objref_str;
                        LogMngr::logMessage(INF,MSG_ID_KILLING_PROCESS, objref_str.c_str());  
                        int pid = atoi(objref_str.c_str());
                        // 9 it's a SIGKILL
                        OS::kill_process(pid);
                    }
                }
                catch ( const std::exception & ex )
                {
                    // return false;
                }
            }
        }
        //LogMngr::logMessage(INF,MSG_ID_OPENING, mex.str());  
        fs::remove_all(full_path.native_file_string());
        
        }
        catch(...){
            //LogMngr::logMessage(INF,MSG_ID_PATH_NOT_FOUND);  
        }
    #endif
        return true;
    }     
  
    void execute_shell_command(std::string command)
    throw (CORBA::NO_PERMISSION)
    {
        try
        {
             OsSupport::OS::create_process("/bin/sh" , command , "", NULL, NULL);
        }
        catch (...)
        {
             throw CORBA::NO_PERMISSION(OrbSupport::NO_PERMISSION, CORBA::COMPLETED_NO);
        }
    }
    void DaemonController_impl::notify (short exitcode, CORBA::Long port_number) throw(CORBA::SystemException)
    {
       std::string pd_policy = get_platform_daemon_policy(port_number);  
       if(6 != exitcode || 1 != isolated){
           LogMngr::logMessage(INF,MSG_ID_CONTROLLER_NOTIFY_EXIT_ACTION_PORT_POLICY, exitcode, port_number, pd_policy);   
       }
       switch (exitcode){
           case(0):
                if(!oldNotification){                  
                    if(pd_policy=="restart"){
                        //For restart
                        LogMngr::logMessage(INF,MSG_ID_CONTROLLER_RESTART_IN_PROGRESS_OF_THE_SYSTEM, port_number,get_host_start_file(port_number));  
                        kill_sons_pid_of_daemon_policy(port_number);
                        //OsSupport::OS::set_process_attributes(0, 0);
                        OsSupport::OS::create_process(get_host_start_file(port_number).c_str(), "", "", NULL, NULL);                        
                    }
                    else if(pd_policy=="reboot"){  
                        LogMngr::logMessage(INF,MSG_ID_CONTROLLER_REBOOT_IN_PROGRESS);  
                        kill_sons_pid_of_daemon_policy(port_number);
                        OsSupport::OS::create_process(m_reboot_host_script, "", "", NULL, NULL);
                    }         
                    else if(pd_policy == "exit-only"){
                       LogMngr::logMessage(INF,MSG_ID_CONTROLLER_NO_ACTION, port_number);  
                       kill_sons_pid_of_daemon_policy(port_number);
                    }
                }
                else oldNotification = false;
            break;
            case(2)://stop 
                LogMngr::logMessage(INF,MSG_ID_CONTROLLER_STOP_IN_PROGRESS, port_number);   
                oldNotification = true;
                kill_sons_pid_of_daemon_policy(port_number);
            break;
            case(3)://reboot
                LogMngr::logMessage(INF,MSG_ID_CONTROLLER_REBOOT_IN_PROGRESS);   
                oldNotification = true;
                for (std::vector<Cdmw::PlatformMngt::PDaemon>::iterator itrCollector = m_vpd.begin(); itrCollector != m_vpd.end(); itrCollector++)
                {
                    kill_sons_pid_of_daemon_policy((*itrCollector).getPort_number());
                }
                OsSupport::OS::create_process(m_reboot_host_script , "" , "", NULL, NULL);
            break;
            case(4):
                //For shutdown 
                LogMngr::logMessage(INF,MSG_ID_CONTROLLER_SHUTDOWN_IN_PROGRESS); 
                oldNotification = true;
                for (std::vector<Cdmw::PlatformMngt::PDaemon>::iterator itrCollector = m_vpd.begin(); itrCollector != m_vpd.end(); itrCollector++)
                {
                    kill_sons_pid_of_daemon_policy((*itrCollector).getPort_number());
                }
                OsSupport::OS::set_process_attributes(0, 0);
                OsSupport::OS::create_process(m_shutdown_host_script, "", "", NULL, NULL);
                break;
            case(5):
                //For restart 
                LogMngr::logMessage(INF,MSG_ID_CONTROLLER_RESTART_IN_PROGRESS, port_number, get_host_start_file(port_number).c_str()); 
                oldNotification = true;
                OsSupport::OS::set_process_attributes(0, 0);
                //kill system with port "port_number" and delete the tmp file
                kill_sons_pid_of_daemon_policy(port_number);   
                //restart system
                OsSupport::OS::create_process(get_host_start_file(port_number).c_str(), "", "", NULL, NULL);
                break;
            case(6):
                //For stop al PD, when host became isolated
        if(isolated!=1){ 
            LogMngr::logMessage(INF,MSG_ID_CONTROLLER_STOP_ALL_SYSTEM); 
        }
                oldNotification = true;
                for (std::vector<Cdmw::PlatformMngt::PDaemon>::iterator itrCollector = m_vpd.begin(); itrCollector != m_vpd.end(); itrCollector++)
                {
                    kill_sons_pid_of_daemon_policy((*itrCollector).getPort_number()); 
                }
                break;
            default:
                LogMngr::logMessage(INF,MSG_ID_CONTROLLER_UNKNOW); 
           } 
    } 
    
    void DaemonController_impl::miop_request (const char* host_name) throw(CORBA::SystemException){
        std::string tmp(host_name);
        m_cs->receive(tmp);
    }
    
    short DaemonController_impl::is_isolated() throw(CORBA::SystemException){
        return isolated;
    }
    
    void DaemonController_impl::set_isolated(short value) throw (CORBA::NO_PERMISSION){
        if(value!=isolated){
        //new status 
        if(value==1){
            LogMngr::logMessage(INF,MSG_ID_HOST_CHANGE_STATUS, "Isolated"); 
        }
        else if(value==2){
            LogMngr::logMessage(INF,MSG_ID_HOST_CHANGE_STATUS, "Not isolated"); 
        }
    }
        isolated = value;
    }
};
