/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CORBA.hpp>

#include <testcontrollersyncro/TestControllerSyncro.hpp>
#include "Foundation/common/Options.hpp"
#include "Foundation/common/String.hpp"
#include "daemoncontroller/ControllerLogMngr.hpp"
#include "daemoncontroller/ControllerLogMessageId.hpp"
#include <string>
#include <sstream>

#include "orbsvcs/orbsvcs/PortableGroupC.h"
#include "daemoncontroller/ControllerSyncro.hpp"
#include "daemoncontroller/ConfigurationDC.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtDaemonController.skel.hpp"

#define NB_TEST_THREADS 10
#define NB_CALLS 10

class Pinger : public Cdmw::OsSupport::Thread
{
    public:
       Pinger::Pinger (Cdmw::ControllerSyncro* cs) 
       {
           m_cs = cs;
       }
        Pinger::~Pinger()
                throw(){};
        void Pinger::run() throw(){
              m_cs->go();
        }
        
        Cdmw::ControllerSyncro* m_cs;
};

namespace CdmwFT 
{

namespace Test 
{

    
CPPUNIT_TEST_SUITE_REGISTRATION( TestControllerSyncro );
    
// constructor
TestControllerSyncro::TestControllerSyncro()
{ 
    m_orb =  CORBA::ORB::_duplicate( Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());
    char** ORB_init_argv = 0;
    int ORB_init_argc= 0;
    std::string str_opt( "" );
    str_opt += Cdmw::Common::Options::LOG_FILE_OPTION;
    str_opt += "=" ;
    str_opt += "daemoncontrollerlog.txt";;
    ORB_init_argc += 1;
    ORB_init_argv = Cdmw::Common::String::to_char_array(
    Cdmw::Common::String::to_strings( str_opt ) );
    Cdmw::PlatformMngt::ControllerLogMngr::initialise(ORB_init_argc, ORB_init_argv);
}

// destructor
TestControllerSyncro::~TestControllerSyncro()
{
}

void TestControllerSyncro::test_single_controller_isolation()
{
    CORBA::Object_var obj = m_orb->resolve_initial_references("RootPOA");
    std::string miop_group ("corbaloc:miop:1.0@1.0-cdmwdaemoncontrollerunik-11/225.1.1.8:5555");
    CdmwPlatformMngt::DaemonController_var dm;
    std::vector<Cdmw::PlatformMngt::PDaemon> vpd;
    Cdmw::PlatformMngt::PDaemon pd;
    pd.setPort_number(1);
    pd.setControllerPolicy("exit-only");
    pd.setHostStartFile("startfile.sh"); 
    vpd.push_back(pd);
    Cdmw::ControllerSyncro* cs = new Cdmw::ControllerSyncro(dm.in(),miop_group, m_orb, 1000, 4, 4, "H0",100, vpd, true);
    Pinger* ping = new Pinger(cs);
    ping->start();
    if((cs->get_DaemonController())->is_isolated() != 0){
        CPPUNIT_ASSERT(false);
    }     
    sleep(15);
    if((cs->get_DaemonController())->is_isolated()){
        CPPUNIT_ASSERT(true);
    } 
    else{
        CPPUNIT_ASSERT(false);
    } 
    CPPUNIT_ASSERT(true);
}


} // end namespace Test

} // end namespace CdmwFT

