/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_MONITOR_THREAD_HPP
#define INCL_PLATFORMMNGT_MONITOR_THREAD_HPP

#include "Foundation/common/Options.hpp"
#include "Foundation/common/String.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/osthreads/Barrier.hpp"
#include "Foundation/commonsvcs/time/Util.hpp"
#include "idllib/CdmwPlatformMngtEntity.stub.hpp"
#include "Foundation/logging/Logger.hpp"
#include "Foundation/osthreads/Mutex.hpp"
#include "Foundation/osthreads/MutexGuard.hpp"

using namespace CdmwPlatformMngt;

namespace Cdmw
{        // Begin namespace Cdmw

    namespace PlatformMngt
    { // Begin namespace PlatformMngt

        class MonitorThread : public Cdmw::OsSupport::Thread
        {

            public:
                 
                MonitorThread(  Entity_ptr host_entity,
                                int degraded_threshold,
                                int failed_threshold,
                                long polling_interval,
                                long starting_timeout  );
                virtual ~MonitorThread() throw ();
                
                void set_up();
                
                void stop();
                
                bool is_stopped();
                
                
                virtual void run() throw ();

            private:
            Entity_var m_host_entity;
            int m_degraded_threshold;
            int m_failed_threshold;
            long m_polling_interval;
            long m_starting_timeout;
            bool m_stopped;
            Cdmw::OsSupport::Mutex        m_mutex;
            
            Cdmw::Logging::Logger* m_logger; 
            
            Cdmw::CommonSvcs::Time::Util::NTP_Status m_sync_status;

         };

    } // End namespace PlatformMngt
} // End namespace Cdmw

#endif //INCL_PLATFORMMNGT_MONITOR_THREAD_HPP

