
/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_MYPROCESSBEHAVIOUR_HPP 
#define INCL_MYPROCESSBEHAVIOUR_HPP 

#include "SystemMngt/platforminterface/PlatformInterface.hpp"
#include "SystemMngt/platforminterface/ProcessBehaviour.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "idllib/CdmwPlatformMngtManagedElementContexts.stub.hpp"
#include "idllib/CdmwPlatformMngtHost.stub.hpp"
#include "idllib/CdmwPlatformMngtEntity.stub.hpp"

#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/common/Exception.hpp"
#include "Foundation/logging/Logger.hpp"

#include "hostmonitor/MonitorThread.hpp"

#include <string>
#include <sstream>
#include <iostream>
#include <fstream>

using namespace Cdmw::PlatformMngt;
using namespace Cdmw::OsSupport;
using namespace CdmwPlatformMngt;

const std::string HOST_ENTITY_NAME = "CdmwHostSynchronization";
/**
* Purpose:
* <p>
* The abstract process behaviour.
*/
class MyProcessBehaviour : public Cdmw::PlatformMngt::ProcessBehaviour
{

public:

    /**
    * Purpose:
    * <p> The constructor.
    */ 
    MyProcessBehaviour(CORBA::ORB_ptr orb, 
                       int t_degraded, 
                       int t_failed,
                       long p_interval,
                       long s_timeout )
    {
        m_orb = CORBA::ORB::_duplicate(orb);
        
        m_t_degraded = t_degraded;
        m_t_failed = t_failed;
        m_p_interval = p_interval;
        m_s_timeout = s_timeout;
        m_monitor_thread = 0;
   
    }


    /**
    * Purpose:
    * <p> The destructor.
    */ 
    virtual ~MyProcessBehaviour() throw ()
    {
    
    if (0!=m_monitor_thread)
    {
        m_monitor_thread->stop();
        Cdmw::OsSupport::ThreadStatus t_status = m_monitor_thread->get_status();
        switch (t_status)  
        {
            case Cdmw::OsSupport::THREAD_RUN_REQUEST:
            case Cdmw::OsSupport::THREAD_RUNNING:
            
                m_monitor_thread -> join();
            break;
            case Cdmw::OsSupport::THREAD_ENDED:
            case Cdmw::OsSupport::THREAD_CREATED:              
            break;   
        } 
        delete m_monitor_thread;
        m_monitor_thread = 0;
    }
    
 
    }

    /**
    * Purpose:
    * <p>
    * the behaviour for the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/nb_steps:1.0
    * attribute
    */
    virtual CORBA::ULong nb_steps() throw(CORBA::SystemException)
    {
        return 1;
    }
    
    
    /**
    * Purpose:
    * <p>
    * the behaviour for the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/get_service:1.0
    * operation
    */
    virtual CORBA::Object_ptr get_service() throw(CORBA::SystemException)
    {       
        return CORBA::Object::_nil();
    }
    
    
    /**
    * Purpose:
    * <p>
    * the behaviour for the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/initialise:1.0
    * operation
    */
    virtual void initialise(const CdmwPlatformMngtBase::StartupKind& startup_kind)
        throw(CdmwPlatformMngt::ProcessDelegate::BadOrder, CORBA::SystemException)
    {

    const int MAX_RETRY=7;
    const int RETRY_TIMEOUT_MS=500;
    int counter=0;
    CORBA::Object_var obj;
    try
    {
        try
        {
            obj = PlatformInterface::Get_service(PlatformInterface::MANAGED_PROCESS_CONTEXT_NAME);
        }
        catch (const CORBA::SystemException& ex)
        {
            CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger("CDMW.PFMHMN"),"Unexpected exception " << ex );
        }

        while( CORBA::is_nil(obj.in()))
        {
            if(MAX_RETRY==counter)
            {
                CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger("CDMW.PFMHMN"),"Unexpected invalid MANAGED_PROCESS_CONTEXT. This process is exiting"  );
                exit(1);
            }
            CDMW_WARN(Cdmw::Logging::Logger::Get_Logger("CDMW.PFMHMN"), "MANAGED_PROCESS_CONTEXT not yet ready, retrying in (ms)"<< RETRY_TIMEOUT_MS);
            Cdmw::OsSupport::OS::sleep (RETRY_TIMEOUT_MS);
            try
            {
                obj = PlatformInterface::Get_service(PlatformInterface::MANAGED_PROCESS_CONTEXT_NAME);
            }
            catch (const CORBA::SystemException& ex)
            {
                CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger("CDMW.PFMHMN"),"Unexpected exception " << ex );
            }
            counter++;
        }

        ManagedElementContexts::ProcessContext_var ctx = ManagedElementContexts::ProcessContext::_narrow(obj.in());

        External::Host_var host = ctx->the_host();
        counter=0; 
        while(CORBA::is_nil(host.in()))
        {
            if(MAX_RETRY==counter)
            {
                CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger("CDMW.PFMHMN"),"Invalid MANAGED_PROCESS_CONTEXT . This process is exiting"  );
                exit(1);
            }
            CDMW_WARN(Cdmw::Logging::Logger::Get_Logger("CDMW.PFMHMN"), "MANAGED_PROCESS_CONTEXT not yet ready retry in (ms)"<< RETRY_TIMEOUT_MS);
            Cdmw::OsSupport::OS::sleep (1000);
            host = ctx->the_host();
            counter++;
        }
        
        EntityDef ent_def;
        
        ent_def.entity_name = HOST_ENTITY_NAME.c_str();
        ent_def.status_values.length(0);
        ent_def.properties.length(0);    
        Entity_var host_entity = CdmwPlatformMngt::Entity::_nil();

        try 
        {
            host_entity = host -> get_entity(HOST_ENTITY_NAME.c_str());
        }
        catch (const CdmwPlatformMngt::EntityNotFound & ex)
        {
            host_entity = host -> add_entity(ent_def);
        }
        
        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger("CDMW.PFMHMN"),"Starting monitor thread with degraded threshold " << m_t_degraded << " and failed threshold " << m_t_failed );
        m_monitor_thread = new MonitorThread( host_entity._retn(), m_t_degraded, m_t_failed, m_p_interval, m_s_timeout );
        m_monitor_thread -> set_up();

    }
    catch (const CORBA::SystemException& ex)
    {
        CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger("CDMW.PFMHMN"),"Unexpected exception " << ex );
    }
}
        
        
 
    /**
    * Purpose:
    * <p>
    * the behaviour for the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/next_step:1.0
    * operation
    */
    virtual void next_step()
        throw(CdmwPlatformMngt::ProcessDelegate::InvalidStep, CORBA::SystemException)
    {
      
    }
    
    
    /**
    * Purpose:
    * <p>
    * the behaviour for the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/run:1.0
    * operation
    */
    virtual void run()
        throw(CdmwPlatformMngt::ProcessDelegate::NotReadyToRun,
              CdmwPlatformMngt::ProcessDelegate::AlreadyDone,
              CORBA::SystemException)
    {
            if ( m_monitor_thread ) m_monitor_thread -> start();
         
    }

    /**
    * Purpose:
    * <p>
    * the behaviour for the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/stop:1.0
    * operation
    */
    virtual void stop() throw(CORBA::SystemException)
    {
        m_orb->shutdown(false);       
        m_monitor_thread -> stop();

  
    }

    /**
        * Purpose:
        * <p>
        * the behaviour for the
        * IDL:FT/PullMonitorable/is_alive:1.0
        * operation
        */
    virtual bool is_alive()
        throw(CORBA::SystemException)
    {

         return true;
    }

    
private:


   MonitorThread * m_monitor_thread ;

    /**
    * The ORB.
    */
    CORBA::ORB_var m_orb;
    
    int m_t_degraded;
    int m_t_failed; 
    int m_p_interval;
    int m_s_timeout;

};


#endif // INCL_MYPROCESSBEHAVIOUR_HPP

