/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <string>
#include <sstream>
#include <xercesc/sax/SAXParseException.hpp>
#include <xercesc/util/PlatformUtils.hpp>

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/Options.hpp"
#include "Foundation/orbsupport/CORBA.hpp"
#include "platformadminlibrary/AdminLogMngr.hpp"

#include "SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtDaemon.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"

#include "platformadminlibrary/ConfigElementParser.hpp"
#include "platformadminlibrary/System.hpp"
#include "platformadminlibrary/Application.hpp"
#include "platformadminlibrary/Process.hpp"
#include "platformadminlibrary/CcmComponent.hpp"
#include "platformadminlibrary/FTGroup.hpp"
#include "platformadminlibrary/LBGroup.hpp"
#include "platformadminlibrary/Host.hpp"
#include "platformadminlibrary/Entity.hpp"
#include "SystemMngt/tools/AdminLogMessageId.hpp"

#include "platformadmin/PlatformAdmin.hpp"
#include "platformadmin/PlatformAdminUsage.hpp"
#include "platformadminlibrary/PlatformAdminSnapshot.hpp"

#include <SystemMngt/platformlibrary/Iterator.hpp>


using namespace std;
using namespace Cdmw;
using namespace PlatformAdmin;
using namespace OsSupport;


//------------------
// Global variables 
//------------------
extern bool check_only;
bool set_filters_string_one_process=false; //this in case the use choose to set trace filters
                                          //only for one process
bool get_filters_string_one_process=false; //this in case the use choose to get trace filters only for one process
bool set_filters_string_host=false; //this in case the use choose to set trace filters for all managed processes in an host

extern bool show_properties; //used to enable/disable properties in snapshot

namespace{
//---------------------//
// Function prototypes //
//---------------------//
int getCmdPos( int argc, char** );

void performCmd( CORBA::ORB_ptr orb, int, char** );
void performSysCmd( System*, string, int, char** );
void performAppCmd( Application*, string, int, char** );
void performProcCmd( Process*, string, int, char** );
void performHostCmd( CORBA::ORB_ptr, string, int, char** );
void performEntCmd(Entity*, string, int, char**);
void performComponentCmd( CcmComponent*, string, int, char** );
void performFTGroupCmd( FTGroup*, string, int, char** );
void performLBGroupCmd( LBGroup*, string, int, char** );


CdmwPlatformMngtBase::StartupKind* getStartupKind( int, char** );

CdmwPlatformMngt::Daemon_ptr getDaemon(
    CORBA::ORB_ptr,
    const char*,
    const CdmwPlatformMngt::Port )
    throw( CdmwPlatformMngt::HostNotFound,
       CdmwPlatformMngt::HostNotReachable,
       CORBA::SystemException );

//CdmwPlatformMngt::System_ptr retrieveSystem( CORBA::ORB_ptr orb, PortableServer::POA_ptr poa, const string& ftmanager_corbaloc);

//-----------------//
// Function bodies //
//-----------------//
int getCmdPos( int argc, char** argv )
{
    for( int argIndex = argc-1; argIndex > -1; argIndex-- )
    {
        string arg = argv[ argIndex ];

        if( arg.find( "--" ) == 0 )
            return argIndex;
    }

    return -1;
}

void performCmd( CORBA::ORB_ptr orb, int argc, char** argv )
{
    int    cmdPos  = getCmdPos( argc, argv );
    string cmdName;
    if (cmdPos == -1) {
        // no option, equivalent to --help option
        cmdName = HELP_OPTION;
    } else {
    cmdName = argv[ cmdPos ];
    }
     
    // Look for the existence of the command
    if( usage( cmdName ) == CmdUsage::M_cmdUsage )
    throw CmdUsageException( cmdName );

    // Perform an other command
    else
    {
    // Get the system specified with the corbaloc
    CdmwPlatformMngt::System_var system;
    
    if( !check_only )
    {
            if( OS::get_option_value( argc, argv, SYSTEM_CORBALOC_OPTION ) == "no" )
                        throw CmdUsageException( cmdName );
          
        string system_corbaloc
        = OS::get_option_value( argc, argv, SYSTEM_CORBALOC_OPTION );
        
        CORBA::Object_var obj_system
        = orb->string_to_object( system_corbaloc.c_str() );
        if( CORBA::is_nil( obj_system.in() ) )
        CDMW_THROW2(
            Cdmw::Exception,
            Cdmw::Exception::FATAL,
            "Invalid system reference" );
          
        system = CdmwPlatformMngt::System::_narrow( obj_system.in() );
        if( CORBA::is_nil( system.in() ) )
        CDMW_THROW2(
            Cdmw::Exception,
            Cdmw::Exception::FATAL,
            "Invalid system" );
    }
    
    System admSystem( system.in() );

    //perform Host command
    if( cmdName.find( "--host-" ) != string::npos )
    { 
        if( ((cmdPos+1) >= argc)|| ((cmdPos+6) <= argc) )
        throw CmdUsageException( cmdName );

        std::string hostName = argv[cmdPos + 1];
        CdmwPlatformMngt::Host_var host = system -> get_host(hostName.c_str());
        Host*  admHost = new Host( host.in() );
        
        
        
        if ( cmdName == HOST_ADD_PROPERTY )
        {
        admHost -> add_property (argv[cmdPos + 2]);
        }
        else if (cmdName == HOST_REMOVE_PROPERTY)
        {
        admHost -> remove_property (argv[cmdPos + 2]);
        }

        else if (cmdName == HOST_COLD_REBOOT)
        {
        admHost -> cold_reboot ();
        }
        else if (cmdName == HOST_HOT_REBOOT)
        {
        admHost -> hot_reboot ();
        }
        else if(cmdName == HOST_PROPERTY )
        {
            if( (argc - (cmdPos + 1) < 3)||(argc - (cmdPos + 1) > 4) )
            {
                throw CmdUsageException( cmdName );
            }
            else if (argc - (cmdPos + 1) == 3)
            {
                admHost->get_property( argv[cmdPos + 2], argv[cmdPos + 3] );
            } 
            else if (argc - (cmdPos + 1) == 4) 
            {
                admHost->set_property( argv[cmdPos + 2], argv[cmdPos + 3], argv [cmdPos + 4]);
            }
        }
        else if (cmdName == HOST_STOP)
        {
            //!!!
            //std::cerr << "####### Requested command HOST_STOP" << std::endl;
            
            admHost->stop_host();
        }
    else if( cmdName == HOST_SHUTDOWN )
        {
             admHost->shutdown();
        }
        else if (cmdName == HOST_HOT_RESTART)
        {
            //!!!
            //std::cerr << "####### Requested command HOST_HOT_RESTART" << std::endl;            
            admHost->hot_restart();
        }
        else if (cmdName == HOST_COLD_RESTART)
        {
            //!!!
            //std::cerr << "####### Requested command HOST_COLD_RESTART" << std::endl;            
            
            admHost->cold_restart();
        }
        else if( cmdName == HOST_SNAPSHOT )
        {
            if ( (argc - (cmdPos +1 ) > 2) || ( ( 2 == argc - (cmdPos +1) ) && (SHOW_ATTR_PARAM != string( argv[ cmdPos + 2 ] ) ) ) ) throw CmdUsageException( cmdName );
            if( 2 == argc - (cmdPos +1) ) show_properties = true;
             
            CdmwPlatformMngt::HostSnapshot_var result = admHost->get_snapshot();        
        CdmwPlatformMngt::HostSnapshot admSnapshot(result.in());
        cout << snapShotToString( &admSnapshot, "" ).c_str();       
        
            //cout << snapShotToString( admHost->get_snapshot(), "" ).c_str();
        }
        else 
        performHostCmd(
            orb,
            cmdName,
            argc - (cmdPos + 1),
            &argv[ cmdPos + 1 ] );
    }
    // Perform a system command
    else if( cmdName.find( "--sys-" ) != string::npos )
    {
        performSysCmd(
        &admSystem,
        cmdName,
        argc - (cmdPos + 1),
        &argv[ cmdPos + 1 ] );
    }
    // Perform an application command
    else if( cmdName.find( "--app-" ) != string::npos ) 
    {
        if( (cmdPos+1) >= argc )
        throw CmdUsageException( cmdName );

        string applicationName = argv[ cmdPos + 1 ];
        CdmwPlatformMngt::Application_var application;

        if( !check_only )
        application = system->get_application( applicationName.c_str() );

        Application admApplication( application.in() );

        performAppCmd(
        &admApplication,
        cmdName,
        argc - (cmdPos + 2),
        &argv[ cmdPos + 2 ] );
    }
    // Perform a process command
    else if( (cmdName.find( "--proc-" ) != string::npos) ) //&& (!(cmdName.find( "--proc-trace") != string::npos) ))
    {
        if( (cmdPos+3) >= argc )
        throw CmdUsageException( cmdName );

        string applicationName = argv[ cmdPos + 1 ];
        string processName = argv[ cmdPos + 2 ];
        string hostName = argv[ cmdPos + 3 ];
        CdmwPlatformMngt::Application_var application;
        CdmwPlatformMngt::Process_var process;

        if( !check_only )
        {
        application = system->get_application (applicationName.c_str());
        process = application->get_process (processName.c_str(),
                            hostName.c_str());
        }

        Process admProcess( process.in() );

        performProcCmd(
        &admProcess,
        cmdName,
        argc - (cmdPos + 4),
        &argv[ cmdPos + 4 ] );
    }
    //perform a ftgroup command
    else if( (cmdName.find( "--ftgroup-" ) != string::npos) )
    {
        if( (cmdPos+2) >= argc )
        throw CmdUsageException( cmdName );

        string applicationName = argv[ cmdPos + 1 ];
        string groupName = argv[ cmdPos + 2 ];
        CdmwPlatformMngt::Application_var application;
        CdmwPlatformMngt::FTGroup_var group;

        if( !check_only )
        {
        application = system->get_application (applicationName.c_str());
        group = application->get_ft_group (groupName.c_str());
        
        }
        
        FTGroup admFTGroup( group.in() );

        performFTGroupCmd(
        &admFTGroup,
        cmdName,
        argc - (cmdPos + 3),
        &argv[ cmdPos + 3] );
    }
    
    //perform a lbgroup command
    else if( (cmdName.find( "--lbgroup-" ) != string::npos) )
    {
        if( (cmdPos+2) >= argc )
        throw CmdUsageException( cmdName );

        string applicationName = argv[ cmdPos + 1 ];
        string groupName = argv[ cmdPos + 2 ];
        CdmwPlatformMngt::Application_var application;
        CdmwPlatformMngt::LBGroup_var group;

        if( !check_only )
        {
        application = system->get_application (applicationName.c_str());
        group = application->get_lb_group (groupName.c_str());
        }
        
        LBGroup admLBGroup( group.in() );

        performLBGroupCmd(
        &admLBGroup,
        cmdName,
        argc - (cmdPos + 3),
        &argv[ cmdPos + 3] );
    }
    
    //perform a component command
    else if( (cmdName.find( "--comp-" ) != string::npos) )
    {
        if( (cmdPos+4) >= argc )
        throw CmdUsageException( cmdName );

        string applicationName = argv[ cmdPos + 1 ];
        string processName = argv[ cmdPos + 2 ];
        string hostName = argv[ cmdPos + 3 ];
        string compName = argv[ cmdPos + 4 ];
        CdmwPlatformMngt::Application_var application;
        CdmwPlatformMngt::Process_var process;
        CdmwPlatformMngt::Host_var host;
        CdmwPlatformMngt::CcmComponent_var comp;

        if( !check_only )
        {
        application = system->get_application (applicationName.c_str());
        process = application->get_process (processName.c_str(),
                            hostName.c_str());

        comp = process->get_component (compName.c_str());
        }
        
        CcmComponent admComponent( comp.in() );

        performComponentCmd(
        &admComponent,
        cmdName,
        argc - (cmdPos + 5),
        &argv[ cmdPos + 5] );
    }
     
    //perform entity command
    else if((cmdName.find("--ent-") != string::npos) )
    
    {
     //case of system ...
        if ( strcmp( argv[ cmdPos + 1 ] , "system" ) == 0)
        {
                CdmwPlatformMngt::Entity_var entity;
                //retrieve entity from system ...
                string entityName = argv[cmdPos+2];
                entity = system->get_entity(entityName.c_str() );
                Entity admEntity(entity.in());
                performEntCmd( &admEntity, cmdName, argc - (cmdPos +2 ) , &argv[ cmdPos + 2 ] );
        }
               
        //case of application ...
        else if ( strcmp( argv[ cmdPos + 1 ] , "application" ) == 0)
        {
                CdmwPlatformMngt::Entity_var entity;
                CdmwPlatformMngt::Application_var application;
        
                //case to set status of Entity (of Application) ...
                //retrieve entity from application...
                string app_name = argv[cmdPos+2];
                string entityName = argv[cmdPos+3];
        
                application = system->get_application(app_name.c_str() );
                entity = application->get_entity(entityName.c_str() );
                Entity admEntity(entity.in());
                performEntCmd( &admEntity, cmdName, argc - (cmdPos+3) , &argv[ cmdPos + 3 ] );
         }
       
        else if ( strcmp( argv[ cmdPos + 1 ] , "host" ) == 0)
        {
            CdmwPlatformMngt::Entity_var entity;
            CdmwPlatformMngt::Host_var host;
                
                string host_name = argv[cmdPos+2];
                string entityName = argv[cmdPos+3];
                
                host = system->get_host(host_name.c_str() ) ;
                entity = host->get_entity(entityName.c_str() );
                Entity admEntity(entity.in());
                performEntCmd( &admEntity, cmdName, argc - (cmdPos + 3), &argv[ cmdPos + 3 ] );
         }
        
        else if ( strcmp( argv[ cmdPos + 1 ] , "process" ) == 0)
        {
                CdmwPlatformMngt::Entity_var entity;
                CdmwPlatformMngt::Application_var application;
                CdmwPlatformMngt::Host_var host;
                CdmwPlatformMngt::Process_var process;
        
                //retrieve entity from process ...
                string app_name = argv[cmdPos+2];
                string proc_name = argv[cmdPos+3];
                string host_name = argv[cmdPos+4];
                string entityName = argv[cmdPos+5];

                application = system->get_application (app_name.c_str());
                process = application->get_process (proc_name.c_str(), host_name.c_str());

                entity = process->get_entity(entityName.c_str() );

               Entity admEntity(entity.in());
               performEntCmd( &admEntity, cmdName, argc - (cmdPos +5), &argv[ cmdPos + 5 ] );
          }
          
          else if ( strcmp( argv[ cmdPos + 1 ] , "component" ) == 0)
          {
                CdmwPlatformMngt::Entity_var entity;
                CdmwPlatformMngt::Application_var application;
                CdmwPlatformMngt::Host_var host;
                CdmwPlatformMngt::Process_var process;
                CdmwPlatformMngt::CcmComponent_var comp;
        
                //retrieve entity from process ...
                string app_name = argv[cmdPos+2];
                string proc_name = argv[cmdPos+3];
                string host_name = argv[cmdPos+4];
                string comp_name = argv[cmdPos+5];
                string entityName = argv[cmdPos+6];

                application = system->get_application (app_name.c_str());
                process = application->get_process (proc_name.c_str(), host_name.c_str());
                comp = process->get_component(comp_name.c_str());

                entity = comp->get_entity(entityName.c_str() );

               Entity admEntity(entity.in());
               performEntCmd( &admEntity, cmdName, argc - (cmdPos +6), &argv[ cmdPos + 6 ] );
          }
          
          else if ( strcmp( argv[ cmdPos + 1 ] , "ftgroup" ) == 0)
          {
                CdmwPlatformMngt::Entity_var entity;
                CdmwPlatformMngt::Application_var application;
                CdmwPlatformMngt::FTGroup_var group;
        
                //retrieve entity from process ...
                string app_name = argv[cmdPos+2];
                string group_name = argv[cmdPos+3];
                string entityName = argv[cmdPos+4];

                application = system->get_application (app_name.c_str());
                group = application->get_ft_group (group_name.c_str());

                entity = group -> get_entity(entityName.c_str() );

               Entity admEntity(entity.in());
               performEntCmd( &admEntity, cmdName, argc - (cmdPos +4), &argv[ cmdPos + 4 ] );
          }
          
          else if ( strcmp( argv[ cmdPos + 1 ] , "lbgroup" ) == 0)
          {
                CdmwPlatformMngt::Entity_var entity;
                CdmwPlatformMngt::Application_var application;
                CdmwPlatformMngt::LBGroup_var group;
        
                //retrieve entity from process ...
                string app_name = argv[cmdPos+2];
                string group_name = argv[cmdPos+3];
                string entityName = argv[cmdPos+4];

                application = system->get_application (app_name.c_str());
                group = application->get_lb_group (group_name.c_str());

                entity = group -> get_entity(entityName.c_str() );

               Entity admEntity(entity.in());
               performEntCmd( &admEntity, cmdName, argc - (cmdPos +4), &argv[ cmdPos + 4 ] );
          }
          else throw CmdUsageException( cmdName );
          
          
          
      }

      // Not existing command
        else throw CmdUsageException( cmdName );
    }
}

void performSysCmd(
    System* admSystem,
    string cmdName,
    int paramCnt,
    char** param )
{
    if( cmdName == SYS_SNAPSHOT )
    {
        if ( ( paramCnt >1 ) || ( (1 == paramCnt) && (SHOW_ATTR_PARAM != string( param[0] ) ) ) ) throw CmdUsageException( cmdName );
        if (1 == paramCnt)  show_properties = true;
 
        //cout << snapShotToString( admSystem->get_snapshot() ).c_str();
        CdmwPlatformMngt::SystemSnapshot_var result = admSystem->get_snapshot();
    CdmwPlatformMngt::SystemSnapshot admSnapshot(result.in());
    cout << snapShotToString( &admSnapshot ).c_str();
    }
    else if( cmdName == SYS_DEFINE )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admSystem->define( param[0] );
    }
    else if( cmdName == SYS_ADD_HOST )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admSystem->add_host( param[0] );
    }
    else if( cmdName == SYS_REMOVE_HOST )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admSystem->remove_host( param[0] );
    }
    else if( cmdName == SYS_ADD_APPLICATION )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admSystem->add_application( param[0] );
    }
    else if( cmdName == SYS_REMOVE_APPLICATION )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admSystem->remove_application( param[0] );
    }
    else if( cmdName == SYS_ADD_ENTITY )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admSystem->add_entity( param[0] );
    }
    else if( cmdName == SYS_REMOVE_ENTITY )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admSystem->remove_entity( param [0] );
    }
    else if( cmdName == SYS_ADD_SERVICE )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admSystem->add_service( param[0] );
    }
    else if( cmdName == SYS_REMOVE_SERVICE )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admSystem->remove_service( param[0] );
    }
    else if( cmdName == SYS_SET_INIT_GRAPH )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admSystem->set_init_graph( param[0] );
    }
    else if( cmdName == SYS_SET_STOP_GRAPH )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admSystem->set_stop_graph( param[0] );
    }
    else if( cmdName == SYS_START )
    {
    if( (paramCnt != 0) && (paramCnt != 3) )
        throw CmdUsageException( cmdName );

    CdmwPlatformMngtBase::StartupKind_var startupKind
        = getStartupKind( paramCnt, param );

    admSystem->start( startupKind.in() );
    }
    else if( cmdName == SYS_RESUME )
    {
    if( paramCnt != 0 ) throw CmdUsageException( cmdName );
    admSystem->resume();
    }
    else if( cmdName == SYS_ACKNOWLEDGE )
    {
    if( paramCnt != 0 ) throw CmdUsageException( cmdName );
    admSystem->acknowledge();
    }
    else if( cmdName == SYS_STOP )
    {
    CORBA::Boolean emergencyStop;

    if( paramCnt == 0)
        emergencyStop = false;
    else if( (paramCnt == 1) && (string( param[0] ) == "EMERGENCY") )
        emergencyStop = true;
    else
        throw CmdUsageException( cmdName );

    admSystem->stop( emergencyStop );
    }
    else if( cmdName == SYS_HOT_RESTART )
    {
    if( (paramCnt != 0) && (paramCnt != 3) )
        throw CmdUsageException( cmdName );

    CdmwPlatformMngtBase::StartupKind_var startupKind
        = getStartupKind( paramCnt, param );

    admSystem->hot_restart( startupKind.in() );
    }
        else if( cmdName == SYS_COLD_RESTART )
    {
    if( (paramCnt != 0) && (paramCnt != 3) )
        throw CmdUsageException( cmdName );

    CdmwPlatformMngtBase::StartupKind_var startupKind
        = getStartupKind( paramCnt, param );

    admSystem->cold_restart( startupKind.in() );
    }
    else if(cmdName == SYS_ADD_PROPERTY )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admSystem->add_property( param[0] );
    }
    else if(cmdName == SYS_REMOVE_PROPERTY )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admSystem->remove_property( param[0] );
    }
    else if(cmdName == SYS_PROPERTY )
    {
    if( (paramCnt < 1)||(paramCnt > 3) ) throw CmdUsageException( cmdName );
    else if (paramCnt == 2) admSystem->get_property( param[0], param[1] );
    else if (paramCnt == 3) admSystem->set_property( param[0], param[1], param [2]);
    }      
    else throw CmdUsageException( cmdName );
}
 
void performAppCmd(
    Application* admApplication,
    string cmdName,
    int paramCnt,
    char** param )
{
    if( cmdName == APP_SNAPSHOT )
    {
        if ( ( paramCnt >1 ) || ( (1 == paramCnt) && (SHOW_ATTR_PARAM != string( param[0] ) ) ) ) throw CmdUsageException( cmdName );
        if (1 == paramCnt)  show_properties = true;
 
        cout << snapShotToString( admApplication->get_snapshot(), "" ).c_str();
    }
    else if( cmdName == APP_ADD_PROCESS )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admApplication->add_process( param[0] );
    }
    else if( cmdName == APP_REMOVE_PROCESS )
    {
    if( paramCnt != 2 ) throw CmdUsageException( cmdName );
    admApplication->remove_process( param[0],  param[1]);
    }
    else if( cmdName == APP_ADD_SERVICE )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admApplication->add_service( param[0] );
    }
    else if( cmdName == APP_REMOVE_SERVICE )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admApplication->remove_service( param[0] );
    }
    else if( cmdName == APP_ADD_ENTITY )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admApplication->add_entity( param[0] );
    }
    else if( cmdName == APP_REMOVE_ENTITY )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admApplication->remove_entity( param [0] );
    }
    else if( cmdName == APP_SET_INIT_GRAPH )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admApplication->set_init_graph( param[0] );
    }
    else if( cmdName == APP_SET_STOP_GRAPH )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admApplication->set_stop_graph( param[0] );
    }
    else if( cmdName == APP_INITIALISE )
    {
    if( (paramCnt != 0) && (paramCnt != 3) )
        throw CmdUsageException( cmdName );

    CdmwPlatformMngtBase::StartupKind_var startupKind
        = getStartupKind( paramCnt, param );

    admApplication->initialise( startupKind.in() );
    }
    else if( cmdName == APP_INITIALISE_AND_WAIT )
    {
        if( (paramCnt != 0) && (paramCnt != 3) )
            throw CmdUsageException( cmdName );
    
        CdmwPlatformMngtBase::StartupKind_var startupKind
            = getStartupKind( paramCnt, param );
    
        CdmwPlatformMngt::LongRequestStatus status =
            admApplication->initialise_and_wait( startupKind.in() );
    
        switch (status) 
        {
            case CdmwPlatformMngt::LONG_REQUEST_COMPLETED:
                break;
            case CdmwPlatformMngt::LONG_REQUEST_NOT_COMPLETED:
                cout << "WARNING: Application initialisation not completed." << endl;
                break;
            case CdmwPlatformMngt::LONG_REQUEST_SUSPENDED:
                cout << "WARNING: Application initialisation suspended." << endl;
                break;
            case CdmwPlatformMngt::LONG_REQUEST_ABORTED:
                cout << "WARNING: Application initialisation aborted." << endl;
                break;
        }
    }
    else if( cmdName == APP_RESUME )
    {
    if( paramCnt != 0 ) throw CmdUsageException( cmdName );
    admApplication->resume();
    }
    else if( cmdName == APP_ACKNOWLEDGE )
    {
    if( paramCnt != 0 ) throw CmdUsageException( cmdName );
    admApplication->acknowledge();
    }
    else if( cmdName == APP_RUN )
    {
    if( paramCnt != 0 ) throw CmdUsageException( cmdName );
    admApplication->run();
    }

    else if( cmdName == APP_COLD_RESTART )
    {
    if( (paramCnt != 0) && (paramCnt != 3) )
        throw CmdUsageException( cmdName );

    CdmwPlatformMngtBase::StartupKind_var startupKind
        = getStartupKind( paramCnt, param );
    admApplication->cold_restart( startupKind.in() );

    }
    else if( cmdName == APP_HOT_RESTART )
    {
    if( (paramCnt != 0) && (paramCnt != 3) )
        throw CmdUsageException( cmdName );

    CdmwPlatformMngtBase::StartupKind_var startupKind
        = getStartupKind( paramCnt, param );
    admApplication->hot_restart( startupKind.in() );

    }
    else if( cmdName == APP_STOP )
    {
    CORBA::Boolean emergencyStop;

    if( paramCnt == 0)
        emergencyStop = false;
    else if( (paramCnt == 1) && (string( param[0] ) == "EMERGENCY") )
        emergencyStop = true;
    else
        throw CmdUsageException( cmdName );

    admApplication->stop( emergencyStop );
    }
    else if(cmdName == APP_ADD_PROPERTY)
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admApplication->add_property( param[0] );
    }
    else if(cmdName == APP_REMOVE_PROPERTY)
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admApplication->remove_property( param[0] );
    }
    else if(cmdName == APP_PROPERTY )
    {
        //CGi The function property is now being devided in two function:  get_property and set_property. 
        //What method is called, it depends of the input argument number
        if( (paramCnt < 1)||(paramCnt > 3) ) throw CmdUsageException( cmdName );
        else if (paramCnt == 2) admApplication->get_property( param[0], param[1] );
        else if (paramCnt == 3) admApplication->set_property( param[0], param[1], param [2]);
    }
    else throw CmdUsageException( cmdName );
}


void performProcCmd(
    Process* admProcess,
    string cmdName,
    int paramCnt,
    char** param )
{
    if( cmdName == PROC_SNAPSHOT )
    {
        if ( ( paramCnt >1 ) || ( (1 == paramCnt) && (SHOW_ATTR_PARAM != string( param[0] ) ) ) ) throw CmdUsageException( cmdName );
        if (1 == paramCnt)  show_properties = true;
 
               
        //cout << snapShotToString( admProcess->get_snapshot(), "" ).c_str();
        CdmwPlatformMngt::ProcessSnapshot_var result = admProcess->get_snapshot();
    CdmwPlatformMngt::ProcessSnapshot admSnapshot(result.in());
    cout << snapShotToString( &admSnapshot, "" ).c_str();   
    }
    else if( cmdName == PROC_ADD_SERVICE )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admProcess->add_service( param[0] );
    }
    else if( cmdName == PROC_REMOVE_SERVICE )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admProcess->remove_service( param[0] );
    }
    else if( cmdName == PROC_ADD_ENTITY )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admProcess->add_entity( param[0] );
    }
    else if( cmdName == PROC_REMOVE_ENTITY )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admProcess->remove_entity( param [0] );
    }
    else if( cmdName == PROC_INITIALISE )
    {
    if( (paramCnt != 0) && (paramCnt != 3) )
        throw CmdUsageException( cmdName );

    CdmwPlatformMngtBase::StartupKind_var startupKind
        = getStartupKind( paramCnt, param );

    admProcess->initialise( startupKind.in() );
    }
    else if( cmdName == PROC_NEXT_STEP )
    {
    if( paramCnt != 0 ) throw CmdUsageException( cmdName );
    admProcess->next_step();
    }
    else if( cmdName == PROC_RUN )
    {
    if( paramCnt != 0 ) throw CmdUsageException( cmdName );
    admProcess->run();
    }
    else if (cmdName == PROC_STOP )
    {
    CORBA::Boolean emergencyStop;

    if( paramCnt == 0)
        emergencyStop = false;
    else if( (paramCnt == 1) && (string( param[0] ) == "EMERGENCY") )
        emergencyStop = true;
    else
        throw CmdUsageException( cmdName );

    admProcess->stop( emergencyStop );
    }
    else if( cmdName == PROC_SET_INFO )
    {
    if( paramCnt < 2 || paramCnt > 4) throw CmdUsageException( cmdName );

    typedef char* char_ptr;
    char_ptr paramTmp[5];
        
    paramTmp[0] = (char*)"";
        
    for (int i=0 ; i < 4 ; i++)
    {
        if (i < paramCnt) 
        {
        paramTmp[i+1] = param[i];
        }
        else
        {
        paramTmp[i+1] = (char*)"";
        } 
    }

    string exec = OS::get_option_value( 5, paramTmp, "EXEC" );
    string port = OS::get_option_value( 5, paramTmp, "PORT" );
    string wdir = OS::get_option_value( 5, paramTmp, "WDIR" );
    string args = OS::get_option_value( 5, paramTmp, "ARGS" );
        
    if (exec == "no" || exec == "yes")
    {
        throw CmdUsageException( cmdName );
    }
        
    int portNbr;
    if (port != "no") 
    {           
        std::istringstream portNbr_strm(port);
        portNbr_strm >> portNbr;
        if (portNbr_strm.fail()) 
        {
        throw CmdUsageException( cmdName );
        }
    }
        
    if (wdir == "no" || wdir == "yes")
    {
        wdir = "";
    }
        
    if (args == "no" || args == "yes")
    {
        args = "";
    }
 
    admProcess->set_info(
        exec.c_str(),
        portNbr,
        wdir.c_str(),
        args.c_str() );
    }
    else if( cmdName == PROC_SET_AUTOENDING )
    {
    if( paramCnt != 0 ) throw CmdUsageException( cmdName );
    admProcess->set_autoending(true);
    }
    else if( cmdName == PROC_RESET_AUTOENDING )
    {
    if( paramCnt != 0 ) throw CmdUsageException( cmdName );
    admProcess->set_autoending(false);
    } 
     
    else if( cmdName == PROC_COLD_RESTART )
    {
    if( (paramCnt != 0) && (paramCnt != 3) )
        throw CmdUsageException( cmdName );

    CdmwPlatformMngtBase::StartupKind_var startupKind
        = getStartupKind( paramCnt, param );

    admProcess->cold_restart( startupKind.in() );
    }
    
    else if( cmdName == PROC_HOT_RESTART )
    {
    if( (paramCnt != 0) && (paramCnt != 3) )
        throw CmdUsageException( cmdName );

    CdmwPlatformMngtBase::StartupKind_var startupKind
        = getStartupKind( paramCnt, param );

    admProcess->hot_restart( startupKind.in() );
    }
    else if( cmdName == PROC_SET_LIFE_CYLE )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admProcess->set_life_cycle( param[0] );
    }
    else if( cmdName == PROC_ADD_PROPERTY )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admProcess->add_property( param[0] );
    }
    else if( cmdName == PROC_REMOVE_PROPERTY )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admProcess->remove_property( param[0] );
    }
    /*else if( cmdName == PROC_TRACE)
    {
    if ( set_filters_string_one_process==true)
      {
      std::cout<<"filters string = "<<param[0]<<std::endl;
          admProcess->set_trace_filters( param[0])  ;
      }

        if ( get_filters_string_one_process==true)
      { //case to get the trace filters
      admProcess->get_trace_filters() ;
      }

    }*/
    else if(cmdName == PROC_PROPERTY )
    {
        //CGi The function property is now being devided in two function:  get_property and set_property. 
        //What method is called, it depends of the input argument number
        if( (paramCnt < 1)||(paramCnt > 3) ) throw CmdUsageException( cmdName );
        else if (paramCnt == 2) admProcess->get_property( param[0], param[1] );
        else if (paramCnt == 3) admProcess->set_property( param[0], param[1], param [2]);
    }
    else throw CmdUsageException( cmdName );
}

void performFTGroupCmd(
    FTGroup* admFTGroup,
    string cmdName,
    int paramCnt,
    char** param )
{

    if( cmdName == FTGROUP_SNAPSHOT )
    {
        if ( ( paramCnt >1 ) || ( (1 == paramCnt) && (SHOW_ATTR_PARAM != string( param[0] ) ) ) ) throw CmdUsageException( cmdName );
        if (1 == paramCnt)  show_properties = true;
            
        //cout << snapShotToString( admFTGroup->get_snapshot(), "" ).c_str();
        CdmwPlatformMngt::FTGroupSnapshot_var result = admFTGroup->get_snapshot();
    CdmwPlatformMngt::FTGroupSnapshot admSnapshot(result.in());
    cout << snapShotToString( &admSnapshot, "" ).c_str();
    
    }
    else if( cmdName == FTGROUP_COMPONENTS )
    {
        if( paramCnt != 0 ) throw CmdUsageException( cmdName );
        admFTGroup->print_components();
    }
    else if( cmdName == FTGROUP_COMPONENTS_PROPERTY )
    {
        if( paramCnt != 2 ) throw CmdUsageException( cmdName );
        admFTGroup->print_components_property(param[0], param[1]);
    }
    /*else if( cmdName == PROC_ADD_PROPERTY )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admProcess->add_property( param[0] );
    }
    else if( cmdName == PROC_REMOVE_PROPERTY )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admProcess->remove_property( param[0] );
    }*/
    else if(cmdName == FTGROUP_PROPERTY )
    {
        //What method is called, it depends of the input argument number
        if( (paramCnt < 2)||(paramCnt > 3) ) throw CmdUsageException( cmdName );
        else if (paramCnt == 2) admFTGroup->get_property( param[0], param[1] );
        else if (paramCnt == 3) admFTGroup->set_property( param[0], param[1], param [2]);
    }
    else throw CmdUsageException( cmdName );
}

void performLBGroupCmd(
    LBGroup* admLBGroup,
    string cmdName,
    int paramCnt,
    char** param )
{

    if( cmdName == LBGROUP_SNAPSHOT )
    {
        if ( ( paramCnt >1 ) || ( (1 == paramCnt) && (SHOW_ATTR_PARAM != string( param[0] ) ) ) ) throw CmdUsageException( cmdName );
        if (1 == paramCnt)  show_properties = true;
        
        cout << snapShotToString( admLBGroup->get_snapshot(), "" ).c_str();
    }
    else if( cmdName == LBGROUP_COMPONENTS )
    {
    if( paramCnt != 0 ) throw CmdUsageException( cmdName );
        admLBGroup->print_components();
    }
    else if( cmdName == LBGROUP_COMPONENTS_PROPERTY )
    {
        if( paramCnt != 2 ) throw CmdUsageException( cmdName );
        admLBGroup->print_components_property(param[0], param[1]);
    }
    /*else if( cmdName == PROC_ADD_PROPERTY )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admProcess->add_property( param[0] );
    }
    else if( cmdName == PROC_REMOVE_PROPERTY )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admProcess->remove_property( param[0] );
    }*/
    else if(cmdName == LBGROUP_PROPERTY )
    {
        //What method is called, it depends of the input argument number
        if( (paramCnt < 1)||(paramCnt > 3) ) throw CmdUsageException( cmdName );
        else if (paramCnt == 2) admLBGroup->get_property( param[0], param[1] );
        else if (paramCnt == 3) admLBGroup->set_property( param[0], param[1], param [2]);
    }
    else throw CmdUsageException( cmdName );
}

void performComponentCmd(
    CcmComponent* admCcmComponent,
    string cmdName,
    int paramCnt,
    char** param )
{
    if( cmdName == COMP_SNAPSHOT )
    {
        if ( ( paramCnt >1 ) || ( (1 == paramCnt) && (SHOW_ATTR_PARAM != string( param[0] ) ) ) ) throw CmdUsageException( cmdName );
        if (1 == paramCnt)  show_properties = true;
     
        cout << snapShotToString( admCcmComponent->get_snapshot(), "" ).c_str();
    }

    else if(cmdName == COMP_PROPERTY )
    {
        //What method is called, it depends of the input argument number
        if( (paramCnt < 1)||(paramCnt > 3) ) throw CmdUsageException( cmdName );
        else if (paramCnt == 2) admCcmComponent->get_property( param[0], param[1] );
        else if (paramCnt == 3) admCcmComponent->set_property( param[0], param[1], param [2]);
    }
    else throw CmdUsageException( cmdName );
}


void performEntCmd(
    Entity* admEntity,
    string cmdName,
    int paramCnt,
    char** param )
{
    if( cmdName == ENTITY_ADD_PROPERTY )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admEntity->add_property( param[0] );
    }
    else if( cmdName == ENTITY_REMOVE_PROPERTY )
    {
    if( paramCnt != 1 ) throw CmdUsageException( cmdName );
    admEntity->remove_property( param[0] );
    }
    else if( cmdName == ENTITY_STATUS ) 
    {
        //if ( strcmp(param[0] , "system") == 0)
        {
            switch (paramCnt)
            {
                case 3:
                //case of : --ent-status system entity_name entity_status entity_status_info 
                //set status and status info of the Entity ...
                admEntity->set_status( param[1], param[2] ) ;
                break;
                case 2:
                //if status_info is not provided, default is ""
                admEntity->set_status( param[1], "" ) ;
                break;
                case 1:
                admEntity->get_status() ;
                break;
                default:
                throw CmdUsageException( cmdName );
            }
        }
        /*else if ( ( strcmp(param[0] , "application") == 0) ||
                ( strcmp(param[0] , "host") == 0) )
        {
            switch (paramCnt)
            {
            case 6:
            //case of : --ent-status application application_name entity_name entity_status entity_status_info 
            // or case of : --ent-status host host_name entity_name entity_status entity_status_info 
           //set status and status info of the Entity ...
            admEntity->set_status( param[3], param[4] ) ;
            break;
            case 4:
            admEntity->get_status() ;
            break;
            default:
            throw CmdUsageException( cmdName );
            }
        }
      
        else if ( ( strcmp(param[0] , "process") == 0) )
        {
            switch (paramCnt)
            {
            case 8:
            //case of : --ent-status process application_name process_name host_name entity_name entity_status entity_status_info 

            admEntity->set_status( param[5], param[6] ) ;
            break;
            case 6:
            admEntity->get_status() ;
            break;
            default:
            throw CmdUsageException( cmdName );
            }
        }*/

    }
    else if(cmdName == ENTITY_PROPERTY )
    {
    //cout << "Entity property command, param:" << paramCnt << endl;    
        //if ( strcmp(param[0] , "system") == 0)
        {
            switch (paramCnt)
            {
                case 4:
                //case of : --ent-property system entity_name property_name P|U value  
                //set status and status info of the Entity ...
               // cout << param[1] << param[2] << param [3]<< endl;
                admEntity->set_property( param[1], param[2], param[3] ) ;
                break;
                case 3:
                //cout << param[1] << param[2] << endl;
                admEntity->get_property( param[1], param[2]) ;
                break;
                default:
                throw CmdUsageException( cmdName );
            }
        }
  
    }
    else throw CmdUsageException( cmdName );
}


CdmwPlatformMngtBase::StartupKind* getStartupKind( int paramCnt, char** param )
{
    CdmwPlatformMngtBase::StateRetrieval state_retrieval
    = CdmwPlatformMngtBase::LAST_STATE;
    unsigned long startup_mode = 0;
    string specified_state_id = "";

    if( paramCnt == 3 )
    {
    // Get the retrieval state
    if( string( param[0] ) == "LAST_STATE" )
        state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
    else if( string( param[0] ) == "RECOVER_STATE" )
        state_retrieval = CdmwPlatformMngtBase::RECOVER_STATE;
    else if( string( param[0] ) == "SPECIFIED_STATE" )
        state_retrieval = CdmwPlatformMngtBase::SPECIFIED_STATE;
    else
        throw ;

    // Get the startup mode
    startup_mode = atol( param[ 1 ] );
    
    // Get the state id
    specified_state_id = param[ 2 ];
    }
    
    CdmwPlatformMngtBase::StartupKind_var startupKind
    = new CdmwPlatformMngtBase::StartupKind();

    startupKind->state_retrieval = state_retrieval;
    startupKind->startup_mode = startup_mode;
    startupKind->specified_state_id
    = CORBA::string_dup( specified_state_id.c_str() );

    return startupKind._retn();
}


//!!!
void performHostCmd(
    CORBA::ORB_ptr orb,
    string cmdName,
    int paramCnt,
    char** param )
{
    // Get the parameters
    CdmwPlatformMngt::Port systemPort = -1;
    string hostName;

    if( paramCnt == 1)
    {
    hostName = param[0];
    systemPort = -1;
    }
    else if( paramCnt == 2 )
    {
    hostName = param[0];
    systemPort = ::atoi( param[1] );
    }
    else
    throw CmdUsageException( cmdName );

    if( check_only )
    return;

    // Get the daemon reference
    CdmwPlatformMngt::Daemon_var daemon
    = getDaemon( orb, hostName.c_str(), systemPort );

    // Perform the command on the daemon
    if( cmdName == HOST_REBOOT )
    {
    daemon->cold_reboot();
    }
    else throw CmdUsageException( cmdName );
}


CdmwPlatformMngt::Daemon_ptr getDaemon(
    CORBA::ORB_ptr orb,
    const char* hostName,
    const CdmwPlatformMngt::Port systemPort )
    throw( CdmwPlatformMngt::HostNotFound,
       CdmwPlatformMngt::HostNotReachable,
       CORBA::SystemException )
{
    try
    {

    CdmwPlatformMngt::Port daemonPort; 

    if( systemPort < 0 )
    {
        daemonPort = CdmwPlatformMngt::DEFAULT_SYSTEM_PORT;
    }
    else
    {
        daemonPort = systemPort;
    }

    std::ostringstream daemonCorbaloc;

    daemonCorbaloc 
        << "corbaloc::" << hostName
        <<  ":" << daemonPort
        <<  "/" << CdmwPlatformMngt::DAEMON_NAME;

    CORBA::Object_var obj =
        orb->string_to_object( daemonCorbaloc.str().c_str() );
            
    if( CORBA::is_nil( obj.in() ) )
    {
        throw CdmwPlatformMngt::HostNotFound();
    }
        
    CdmwPlatformMngt::Daemon_var daemon =
        CdmwPlatformMngt::Daemon::_narrow( obj.in() );
            
    if( CORBA::is_nil( daemon.in() ) )
    {
        throw CdmwPlatformMngt::HostNotFound();
    }

    return daemon._retn();
    }
    catch( const CORBA::SystemException& )
    {
    throw CdmwPlatformMngt::HostNotReachable();
    }
}

}//end of anonimous namespace
//----------------------------------------//
// Main loop (entry point of the process) //
//----------------------------------------//
int main( int argc, char* argv[] )
{
    CORBA::ORB_var orb;
    int ret_code = 0;

    XMLPlatformUtils::Initialize();
    
    AdminLogMngr::initialise(argc, argv);

    try
    {
    if( OS::get_option_value( argc, argv, CHECK_ONLY_OPTION ) == "yes" )
        check_only = true;

    // Log the command line
    ostringstream message;

    message << "Requested command \"";
    for( int i=1; i<argc; i++ )
        message << argv[i] << " ";
    message << "\"";

    PlatformMngt::LogMngr::logMessage(
        PlatformMngt::INF,
        message.str().c_str() );

    // Initialize the ORB
    if( !check_only )
        orb = CORBA::ORB_init( argc, argv );

    // Perform the command
    performCmd( orb.in(), argc, argv );

    if( check_only )
    {
        PlatformMngt::LogMngr::logMessage(
        PlatformMngt::INF,
        "Check successful" );
    }
    }
    catch( const LoggedException& )
    {
    ret_code = -1;
    }
    catch( const CmdUsageException& e )
    {
    if (e.m_cmdName != HELP_OPTION) {
        PlatformMngt::LogMngr::logMessage(
        PlatformMngt::ERR,
        MSG_ID_ADMIN_COMMAND_USAGE );
    }

    cout << usage( e.m_cmdName ) << endl;
    ret_code = -1;
    }
    catch( const SAXParseException& e )
    {
    ostringstream message;
        
    message << to_string( e.getMessage() ) << endl;
    message << "in file: " << to_string( e.getSystemId() ) << endl;
    message << "   line: " << e.getLineNumber() << endl;
    message << "   char: " << e.getColumnNumber() << endl;

    PlatformMngt::LogMngr::logMessage( PlatformMngt::ERR, message.str().c_str() );
    ret_code = -1;
    }
    catch( const Cdmw::Exception& e)
    {
    PlatformMngt::LogMngr::logMessage( PlatformMngt::ERR, e.what() );
    ret_code = -1;
    }
    catch( const CORBA::Exception& e )
    {
    PlatformMngt::LogMngr::logMessage( PlatformMngt::ERR, e._name() );
    ret_code = -1;
    }
    catch( ... )
    {
    PlatformMngt::LogMngr::logMessage( PlatformMngt::ERR, MSG_ID_ADMIN_UNKNOWN_ERROR );
    ret_code = -1;
    }

    PlatformMngt::LogMngr::destroy();

    // XMLPlatformUtils::Terminate();

    if( !CORBA::is_nil( orb.in() ) )
    {
    orb->destroy();
    }
  
    return ret_code;
}

