/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMADMIN_ADMINLOGMNGR_HPP 
#define INCL_PLATFORMADMIN_ADMINLOGMNGR_HPP


#include "SystemMngt/platformlibrary/LogMngr.hpp"

#include "Foundation/common/Exception.hpp"

namespace Cdmw
{
namespace PlatformAdmin
{

using namespace Cdmw::PlatformMngt;

    
class AdminLogMngr : public LogMngr
{
    public:

        /**
        * Purpose:
        * <p> Destructor
        * 
        */ 
        virtual 
        ~AdminLogMngr()
            throw();
            
        /**
         *Purpose:
         *<p>
         * Initialises the logging manager with standard output.
         * Command-line arguments will be used to initialise the default logger.
         *
         *@exception OutOfMemoryException
         */
        static void initialise(int& argc, char** argv)
               throw (OutOfMemoryException, InternalErrorException);
                   
        
        /**
         *Purpose:
         *<p>
         * Returns the identifier of the service.
         */
        virtual const char* getService() const;


        /**
         *Purpose:
         *<p>
         * Returns the specified message string or NULL if messageId is invalid.
         *
         * @param messageId the identifier of the message.
         */
        virtual const char* getMessage(long messageId) const;
        
        

    private:
    
        /**
        * Purpose:
        * <p> Constructor
        * Command-line arguments will be used to initialise the default logger.
        * 
        */ 
        AdminLogMngr(int& argc, char** argv)
            throw(OutOfMemoryException, InternalErrorException);
 
            

}; // End class AdminLogMngr 

} // End namespace PlatformAdmin
} // End namespace Cdmw

#endif // INCL_PLATFORMADMIN_ADMINLOGMNGR_HPP


