/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMADMIN_ENTITY_HPP
#define INCL_PLATFORMADMIN_ENTITY_HPP

#include "Foundation/ossupport/OS.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"

#include "SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtEntity.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtEntityContainer.stub.hpp"
#include "platformadminlibrary/PropertyParser.hpp"

#include <map>
#include <vector>

namespace Cdmw {         // Begin namespace Cdmw
namespace PlatformAdmin { // Begin namespace PlatformAdmin

typedef std::vector<PropertyParser> PropList;
typedef std::map<std::string, std::string> ent_types;
typedef std::map<std::string, PropList> ent_type_properties;

class ConfigException;
class EntityParser;

class Entity
{
public:
    static CdmwPlatformMngt::EntityDef* get_definition(
        const EntityParser& entityParser );
    
    Entity(CdmwPlatformMngt::Entity_ptr entity);
    ~Entity();
    void add_property(const char* propertyConfigFileName) ;
    void remove_property(const char* propertyName);
    void set_status( const char* status, const char* status_info ) ;
    void get_status();

    void get_property ( const char* propertyName, 
                        const char* owner);
    
    void set_property ( const char* propertyName, 
                        const char* owner, 
                        const char* propertyValue );
                        

    static ent_types entity_types;
    static ent_type_properties entity_type_properties;
    
private:
    CdmwPlatformMngt::Entity_var m_entity;  

    //map of entity types

};

class EntityContainer
{
private:
    CdmwPlatformMngt::RWEntityContainer_var m_entity_container;

protected:
    EntityContainer(
        CdmwPlatformMngt::RWEntityContainer_ptr entity_container );

public:
    void add_entity(
        const char* entityConfigFileName );

    void remove_entity(
        const char* entityName );
};

} // End namespace PlatformAdmin
} // End namespace Cdmw

#endif //INCL_PLATFORMADMIN_ENTITY_HPP

