/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMADMIN_HOST_HPP
#define INCL_PLATFORMADMIN_HOST_HPP

#include "Foundation/ossupport/OS.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/common/Exception.hpp"

#include "SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtHost.stub.hpp"
#include "platformadminlibrary/Entity.hpp"
#include "SystemMngt/tools/AdminLogMessageId.hpp"


namespace Cdmw {         // Begin namespace Cdmw
namespace PlatformAdmin { // Begin namespace PlatformAdmin

class ConfigException;
class HostParser;

class Host 
{
private:
    CdmwPlatformMngt::Host_var m_host;
    
public:
    Host(
        CdmwPlatformMngt::Host_ptr host );
    
    ~Host();

    CdmwPlatformMngt::HostSnapshot* get_snapshot ();
    void add_property(const char* propertyConfigFileName);
    void remove_property(const char* propertyName);
    void cold_reboot();
    void hot_reboot();
    void cold_restart() throw (LoggedException);
    void hot_restart() throw (LoggedException);
    void stop_host() throw (LoggedException);
    void shutdown() throw (LoggedException);
        
    
    void get_property (
        const char* propertyName, 
        const char* owner );
    
    void set_property (
        const char* propertyName, 
        const char* owner, 
        const char* propertyValue );

    static
    CdmwPlatformMngt::HostDef* get_definition(
        const HostParser& hostParser );
    
};

} // End namespace PlatformAdmin
} // End namespace Cdmw

#endif //INCL_PLATFORMADMIN_HOST_HPP

