/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "platformadminlibrary/AdminLogMngr.hpp"
#include "SystemMngt/tools/AdminLogMessageId.hpp"
#include "SystemMngt/tools/MessageMapManager.hpp"


using namespace  Cdmw::PlatformAdmin;

void AdminLogMngr::initialise(int& argc, char** argv)
     throw (OutOfMemoryException, InternalErrorException)
{
    // if init not already done
    if (0 == M_pSingleton )
    {
        try 
        {
            // create the concrete Log Manager singleton
            M_pSingleton = new AdminLogMngr (argc, argv);
        }
        catch (const std::bad_alloc&)
        {
          CDMW_THROW (OutOfMemoryException);
        }
    }
}


AdminLogMngr::AdminLogMngr(int& argc, char** argv)
        throw(OutOfMemoryException, InternalErrorException)
   : LogMngr (argc, argv)
{
}
            
             
AdminLogMngr::~AdminLogMngr()
        throw()
{
}


const char* AdminLogMngr::getService() const
{
    return "CDMW.PFMADM";
}

    
const char* AdminLogMngr::getMessage(long messageId) const
{
  std::string error_message=MessageMapManager::instance()->find(messageId);
  
  if(error_message.empty())
    error_message = "Not Valid messageId";

  return error_message.c_str();
}



