/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "platformadminlibrary/Entity.hpp"
#include "platformadminlibrary/EntityParser.hpp"
#include "platformadminlibrary/ConfigFileParser.hpp"
#include "SystemMngt/tools/AdminLogMessageId.hpp"
#include "platformadminlibrary/Property.hpp"
#include "platformadminlibrary/PropertyParser.hpp"

#include "platformadminlibrary/AdminLogMngr.hpp"
#include <map>
#include <vector>

extern bool check_only;

namespace Cdmw {         // Begin namespace Cdmw
    namespace PlatformAdmin { // Begin namespace PlatformAdmin

/**
 * class Entity
 */
    
    Entity::Entity( CdmwPlatformMngt::Entity_ptr entity )
        {
            if( !check_only )
                m_entity = CdmwPlatformMngt::Entity::_duplicate( entity);
        }
                                                                                                                             
        Entity::~Entity()
        {
        }

    CdmwPlatformMngt::EntityDef* Entity::get_definition(
        const EntityParser& entityParser )
    {
        try
        {
    
        CdmwPlatformMngt::EntityDef_var entityDef
            = new CdmwPlatformMngt::EntityDef();
        // Get the entity's name
        entityDef->entity_name = CORBA::string_dup( entityParser.name().c_str() );
        //Add Properties to Entity  
    //Property in Entity                                                                                                                            
        unsigned int propertyCount = entityParser.get_property_count();
    
    //Property in entity type
    unsigned int propNum=0;
    std::string type_str = entityParser.get_type();
        /*check if the tag Type is present */
    bool EntityTypeExist = false;
        if ( type_str.size() > 0)
        {  
        std::vector<Cdmw::PlatformAdmin::PropertyParser> PropertiesOfEntity;
        //find the Entitytype in the map Entity::ent_type_properties.
        std::map<std::string,PropList>::iterator iter;  
        //find the type in the map
        for( iter = entity_type_properties.begin(); iter != entity_type_properties.end(); iter++ ) {
        if(type_str == iter->first){ 
            //the entity type exist
            EntityTypeExist = true;
            propNum = (iter->second).size();
            bool isIn = false;
            //For all entitytype property 
            for( unsigned int propTypeIndex=0; propTypeIndex<propNum; propTypeIndex++)
                    {
                //For all entity property
                for( unsigned int propIndex=0; propIndex<propertyCount; propIndex++)
                        {
                 //Check if there are a property redefinition in entity
                 if (entityParser.get_property(propIndex).name() == iter->second[propTypeIndex].name())
                 {
                     isIn= true;
                 PlatformMngt::LogMngr::logMessage( PlatformMngt::WRN, MSG_ID_ENTITY_REDEFINES_PROPERTY,
                 entityParser.name(),iter->second[propTypeIndex].name(), type_str);
                 }   
            }
            //if the property is not redefinited i put it in my vector
            if(!isIn){
                PropertiesOfEntity.push_back(iter->second[propTypeIndex]);
               //
            } 
            isIn=false;
            } 
                    //Put the personal property in my vector
                for( unsigned int propIndex=0; propIndex<propertyCount; propIndex++)
                    {
            PropertiesOfEntity.push_back(entityParser.get_property(propIndex));
                    }
            //Now i now the number of the entity property, i put them in entityDef->properties
            entityDef->properties.length( PropertiesOfEntity.size());
            for( unsigned int propIndex=0; propIndex<PropertiesOfEntity.size(); propIndex++)
                    {
            CdmwPlatformMngt::PropertyDef_var prop = Property::get_definition(PropertiesOfEntity[propIndex]);
                        entityDef->properties[propIndex] = prop; 
                    } 
        }
            }
        if(!EntityTypeExist){
            //type is not present
                LogMngr::logMessage( PlatformMngt::ERR, MSG_ID_ENTITY_INCOMPATIBLE_TYPE, entityDef->entity_name);
                throw LoggedException();
                exit(-1);
        }
    }
    else{
        for( unsigned int propIndex=0; propIndex<propertyCount; propIndex++)
            {
            entityDef->properties.length(propertyCount);
                CdmwPlatformMngt::PropertyDef_var prop = Property::get_definition(entityParser.get_property(propIndex));  
                entityDef->properties[propIndex+propNum] = prop;
            }
    }
    
    
                //read the new attribute value
                //(from scenario) EntityStatusValues .

                std::string stat_val_str = entityParser.get_status_values();

    //parse the stat_val_str and extract the sequence of
        //EntityStatusValues (of EntityDef).

                CdmwPlatformMngt::EntityStatusValues_var my_status_values;

                //entityDef->status_values = entityParser.parse_status_values(stat_val_str);
        if ( stat_val_str.size() > 0)
          {
                  //verify that the tag Type is not present ...
          //infact, is this tag is present, an exception must be throwed
          std::string type_str = entityParser.get_type();

          if ( type_str.size() > 0)
           {
                     PlatformMngt::LogMngr::logMessage( PlatformMngt::ERR, MSG_ID_ENTITY_INCOMPATIBLE_TAGS, entityParser.name() );

                   throw LoggedException();
           exit(-1);
           }

                  my_status_values = entityParser.parse_status_values(stat_val_str);

                  entityDef->status_values = (*my_status_values);

                  }
                else
          { //see if Type tag is present and perform suitable checks ...
          std::string type_entity_str = entityParser.get_type(); 

                  if ( type_entity_str.size() > 0)
            {
             std::string type_content = (entity_types[type_entity_str]);
                     //see if the type is present in map types ...
             if ( type_content.size() <= 0)
               {
                       //type is not present or it is empty 
                       LogMngr::logMessage( PlatformMngt::ERR, MSG_ID_ENTITY_INCOMPATIBLE_TYPE, entityParser.name());

                       throw LoggedException();
               exit(-1);

               }

                    //fill the map of status values ...
                    my_status_values = entityParser.parse_status_values(entity_types[type_entity_str]);

                    entityDef->status_values = (*my_status_values);
            }

          }

        return entityDef._retn();
        }
        catch( const std::bad_alloc& )
        {
        CDMW_THROW( OutOfMemoryException );
        }
    }
      
      void Entity::add_property(const char* propertyConfigFileName) 
      {
    try
      {
        ConfigFileParser parser( propertyConfigFileName );
        PropertyParser propertyparser = parser.get_property();
        CosPropertyService::PropertyDef_var cos_prop = new CosPropertyService::PropertyDef;
        CosPropertyService::PropertyModeType prop_mode_type = CosPropertyService::PropertyModeType(0);
        CdmwPlatformMngt::PropertyDef_var prop = Property::get_definition(propertyparser);
        cos_prop->property_name = prop->property_name;
        cos_prop->property_value = prop->initial_value;
        if (prop->mode_type==1) prop_mode_type = CosPropertyService::PropertyModeType(1);
        cos_prop->property_mode = prop_mode_type;
        m_entity -> define_property_with_mode(prop->property_name.in(),prop->initial_value, prop_mode_type);
      }
    catch( const CdmwPlatformMngt::IncompatibleStatus& e )
      {
        PlatformMngt::LogMngr::logMessage(
                          PlatformMngt::ERR,
                          MSG_ID_ADMIN_INCOMPATIBLE_STATUS );
        
        throw LoggedException();
      }
      }
      
      
      void Entity::remove_property(const char* propertyName)
      {
    try
      {
            m_entity -> delete_property (propertyName);
      }
    catch( const CdmwPlatformMngt::IncompatibleStatus& e )
      {
        PlatformMngt::LogMngr::logMessage(
                          PlatformMngt::ERR,
                          MSG_ID_ADMIN_INCOMPATIBLE_STATUS );
        
        throw LoggedException();
      }
    
      }
      
      
      
      void Entity::set_status(const char* status, const char* status_info )
      {
    try
      {
            m_entity -> set_status(status, status_info );
      }
    catch( const CdmwPlatformMngt::EntityStatusNotAllowed & e )
      {
        PlatformMngt::LogMngr::logMessage(
                          PlatformMngt::ERR,
                          MSG_ID_ADMIN_INCOMPATIBLE_STATUS );
        
        throw LoggedException();
      }
    
      }


      void Entity::get_status()
      {
          CORBA::String_var entity_info;
          std::string mystatus = m_entity->get_status(entity_info);

          std::string info_str = CORBA::string_dup( entity_info);

          std::cout << "Entity Status = \""<<mystatus<<"\" ; Entity Status info = \""<<info_str<<"\""<<std::endl; 
    
      }

void Entity::get_property (const char* propertyName, const char* owner)
    {
    if( !check_only )
        {
            CORBA::Any_var prop_value = new CORBA::Any();

            try
            {
                m_entity -> get_attribute (propertyName, prop_value.out() , ( ('P'== *owner) ? CdmwPlatformMngt::PRE_DEFINED : CdmwPlatformMngt::USER_DEFINED ));
                            
            }
            catch( const CdmwPlatformMngt::AttributeNotFound& e )
            {
                PlatformMngt::LogMngr::logMessage(
                        PlatformMngt::ERR,
                        "Attribute Not Found" );
    
                throw LoggedException();
            }

            CORBA::TypeCode_var tc;
            tc = prop_value->type();

            if (tc->equal(CdmwPlatformMngt::_tc_EntityStatus) )
            {
                //CdmwPlatformMngt::EntityStatus value;
                const char * value;
                prop_value >>= value;
                //std::string output("");
                //output = value;
                std::cout << propertyName << " : " << value << std::endl;
            }
            else
            {
                Property::print_standard_property_type( propertyName, prop_value.in() );
            }
        }//End checkonly
    }//End

void Entity::set_property (const char* propertyName, const char* owner, const char* propertyValue)
    {
        if( !check_only )
        {
            CORBA::Any_var any_var = new CORBA::Any();
            try
            {
                if ('P'== *owner)
                {
                    *any_var = Property::prepare_predefined_any (propertyName, propertyValue);
                }
                else // User defined property
                {
                    CORBA::Any_var prop_value = new CORBA::Any();
                    try
                    {
                        m_entity -> get_attribute (propertyName, prop_value.out() ,
                                       ( ('P'== *owner) ? CdmwPlatformMngt::PRE_DEFINED : CdmwPlatformMngt::USER_DEFINED ));
                    }
                    catch( const CdmwPlatformMngt::AttributeNotFound& e )
                    {
                        PlatformMngt::LogMngr::logMessage(PlatformMngt::ERR, "Attribute Not Found" );
                        throw LoggedException();
                    }
                    CORBA::TypeCode_var tc = prop_value->type();
                    *any_var = Property::prepare_userdefined_any (tc.in(), propertyValue); 
                 }
                 
                 //finally set the attribute
                 m_entity -> set_attribute (propertyName, any_var.in(),
                                           ( ('P'== *owner) ? CdmwPlatformMngt::PRE_DEFINED : CdmwPlatformMngt::USER_DEFINED));
                
            }
            catch( const CdmwPlatformMngt::AttributeNotFound& e )
            {
                PlatformMngt::LogMngr::logMessage(
                        PlatformMngt::ERR,
                        "Attribute Not Found" );
                throw LoggedException();
            }
            catch (const CdmwPlatformMngt::ReadOnlyAttribute& e)
            {
                PlatformMngt::LogMngr::logMessage(
                        PlatformMngt::ERR, 
                        "Read Only Property");
                throw LoggedException();                      
            }
        }//End checkonly
    }//End      

/**
 * class EntityContainer
 */
    EntityContainer::EntityContainer(
        CdmwPlatformMngt::RWEntityContainer_ptr entityContainer )
    {
        if( !check_only )
        m_entity_container = CdmwPlatformMngt::RWEntityContainer::_duplicate(
            entityContainer );
    }

    void EntityContainer::add_entity(
        const char* entityConfigFileName )
    {
        // Open the entity configuration file
        ConfigFileParser parser( entityConfigFileName );

        // Get a entity definition
        CdmwPlatformMngt::EntityDef_var entityDef
        = Entity::get_definition( parser.get_entity() );

        try
        {
        if( !check_only )
            m_entity_container->add_entity( entityDef.in() );
        }
        catch( const CdmwPlatformMngt::EntityAlreadyExists& e )
        {
        PlatformMngt::LogMngr::logMessage(
            PlatformMngt::ERR,
            MSG_ID_ADMIN_ENTITY_ALREADY_EXISTS,
            entityDef->entity_name.in() );

        throw LoggedException();
        }
        catch( const CdmwPlatformMngt::IncompatibleStatus& e )
        {
        PlatformMngt::LogMngr::logMessage(
            PlatformMngt::ERR,
            MSG_ID_ADMIN_INCOMPATIBLE_STATUS );

        throw LoggedException();
        }
    }

    void EntityContainer::remove_entity(
        const char* entityName )
    {
        if( !check_only )
        m_entity_container->remove_entity( entityName );
    }

    } // End namespace PlatformAdmin
} // End namespace Cdmw

