/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <vector>

#include "platformadminlibrary/FTGroupParser.hpp"
#include "platformadminlibrary/EntityParser.hpp"

namespace Cdmw {         // Begin namespace Cdmw
namespace PlatformAdmin { // Begin namespace PlatformAdmin

FTGroupContainerParser::FTGroupContainerParser(
    const DOM_Node& entityContainerParser )
: ConfigElementParser( entityContainerParser ),
  EntityContainerParser( entityContainerParser)
{
}

unsigned int FTGroupContainerParser::get_ftgroup_count() const
{
    return get_child_count( "FT-group" );
}

FTGroupParser FTGroupContainerParser::get_ftgroup(
    unsigned int ftgroupIndex ) const
{
    DOM_Node FTGroup_Parser(
        get_child_element( "FT-group", ftgroupIndex ) );

    return FTGroupParser( FTGroup_Parser );
}

FTGroupParser::FTGroupParser( const DOM_Node& ftgroupParser )
: ConfigElementParser( ftgroupParser ),
EntityContainerParser( ftgroupParser )
{
}

std::string FTGroupParser::name() const
{
    return get_attribute_value( "Name" );
}

unsigned long FTGroupParser::minimum_nbr_replicas() const
{
    std::string minimun_nbr_replicas = get_attribute_value( "Minimum-nbr-replicas" );

    if( minimun_nbr_replicas == "" )
        throw NotFoundException();
        
    return atol( minimun_nbr_replicas.c_str() );
}
CdmwPlatformMngt::ReplicationStyle FTGroupParser::style() const
{
    CdmwPlatformMngt::ReplicationStyle m_style = CdmwPlatformMngt::RG_WARM_PASSIVE ;

    if ( "STATELESS" == get_attribute_value( "Style") )
    {
        m_style = CdmwPlatformMngt::RG_STATELESS;
    }
    else if ( "COLD_PASSIVE" == get_attribute_value( "Style"))
    {
        m_style = CdmwPlatformMngt::RG_COLD_PASSIVE;
    }
    else if ( "WARM_PASSIVE" == get_attribute_value( "Style") )
    {
        m_style = CdmwPlatformMngt::RG_WARM_PASSIVE;
    }
    else if ( "ACTIVE" == get_attribute_value( "Style"))
    {
        m_style = CdmwPlatformMngt::RG_ACTIVE;
    }
    else if ( "IMPL_DEFINED" == get_attribute_value( "Style") )
    {
        m_style = CdmwPlatformMngt::RG_IMPL_DEFINED;
    }
    return m_style;
    
}



PropertyParser FTGroupParser::get_property(unsigned int propertyIndex) const
{
    DOM_Node property_parser(get_child_element("Property", propertyIndex ));
    return PropertyParser(property_parser);
}

                                                                                                                 
unsigned int FTGroupParser::get_property_count() const
{
     return get_child_count("Property");
}


} // End namespace PlatformAdmin
} // End namespace Cdmw

