/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "platformadminlibrary/Host.hpp"
#include "platformadminlibrary/ConfigFileParser.hpp"
#include "platformadminlibrary/SystemParser.hpp"
#include "platformadminlibrary/MonitoringParser.hpp"
#include "platformadminlibrary/HostParser.hpp"
#include "platformadminlibrary/EntityParser.hpp"
#include "platformadminlibrary/Property.hpp"
#include "platformadminlibrary/PropertyParser.hpp"
#include "SystemMngt/platformlibrary/LogMngr.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"

extern bool check_only;

namespace Cdmw {         // Begin namespace Cdmw
    namespace PlatformAdmin { // Begin namespace PlatformAdmin

    Host::Host(
        CdmwPlatformMngt::Host_ptr host )
    {
    if( !check_only )       
        
        m_host = CdmwPlatformMngt::Host::_duplicate( host );
    }

    Host::~Host()
    {
    }

    CdmwPlatformMngt::HostSnapshot* Host::get_snapshot ()
    {
        CdmwPlatformMngt::HostSnapshot_var snapshot;
    
        if( !check_only )
        
            snapshot = m_host->get_snapshot();

        return snapshot._retn();

    }   
        
    void Host::cold_reboot()
    {
    try {
    m_host->cold_reboot ();
    }
    catch( ...) 
            {
                PlatformMngt::LogMngr::logMessage(
                    PlatformMngt::ERR,
                    MSG_ID_ADMIN_UNKNOWN_ERROR );
                throw LoggedException();
            }
    }
    
    void Host::hot_reboot()
    {
    try{
    m_host->hot_reboot ();
    }
    catch( ...) 
            {
                PlatformMngt::LogMngr::logMessage(
                    PlatformMngt::ERR,
                    MSG_ID_ADMIN_UNKNOWN_ERROR );
                throw LoggedException();
            }

    }
    
    void Host::cold_restart()
        throw (LoggedException) 
    {
        try {
            m_host->cold_restart();
        }
        catch( ... ) 
        {
            PlatformMngt::LogMngr::logMessage(PlatformMngt::ERR,
                    MSG_ID_ADMIN_UNKNOWN_ERROR );
            throw LoggedException();
        }
    }

    void Host::hot_restart()
        throw (LoggedException)
    {
        try{
            m_host->hot_restart();
        }
        catch( ... ) 
        {
            PlatformMngt::LogMngr::logMessage(
                    PlatformMngt::ERR,
                    MSG_ID_ADMIN_UNKNOWN_ERROR );
            throw LoggedException();
        }

    }

    void Host::stop_host()
        throw (LoggedException)
    {
        try{
            //!!!
           // std::cerr << "########## Calling m_host->stop_host();" << std::endl;
            m_host->stop_host();

            //!!!
           // std::cerr << "########## Called m_host->stop_host();" << std::endl;

        }
        catch( ::CORBA::SystemException& ex ) 
        {
            //!!!
            std::cerr << "Host::stop_host() CORBA::SystemException: " << ex._info() << std::endl;
            throw LoggedException();
        }
        catch( ... ) 
        {
            PlatformMngt::LogMngr::logMessage(
                    PlatformMngt::ERR,
                    MSG_ID_ADMIN_UNKNOWN_ERROR );
            throw LoggedException();
        }

    }
    
    void Host::shutdown()
        throw (LoggedException)
    {
        try
    {
            m_host->shutdown();
        }
        catch( ::CORBA::SystemException& ex ) 
        {
            //!!!
            std::cerr << "Host::shutdown() CORBA::SystemException: " << ex._info() << std::endl;
            throw LoggedException();
        }
        catch( ... ) 
        {
            PlatformMngt::LogMngr::logMessage(
                    PlatformMngt::ERR,
                    MSG_ID_ADMIN_UNKNOWN_ERROR );
            throw LoggedException();
        }

    }

    CdmwPlatformMngt::HostDef* Host::get_definition(
        const HostParser& hostParser )
    {
        try
        {
        CdmwPlatformMngt::HostDef_var hostDef
            = new CdmwPlatformMngt::HostDef();

        // Get the host's name
        hostDef->host_name = CORBA::string_dup( hostParser.name().c_str() );

        // Set the application process's info
        hostDef->platform_application_exe
            = CORBA::string_dup( hostParser.app_exec_path().c_str() );

        // Set the monitoring's info
        hostDef->daemon_monitoring_info.monitoring_interval
            = hostParser.get_monitoring().pull_interval();
        hostDef->daemon_monitoring_info.monitoring_timeout
            = hostParser.get_monitoring().pull_timeout();
         
        CdmwPlatformMngt::CompleteMonitoringInfo_var monitoring_info = new
        CdmwPlatformMngt::CompleteMonitoringInfo ();
        monitoring_info->monitoring_model = CdmwPlatformMngt::PULL_MONITORING_MODEL;
        monitoring_info->pull_monitoring_interval = hostParser.get_monitoring().pull_interval();
        monitoring_info->pull_monitoring_timeout = hostParser.get_monitoring().pull_timeout();
        monitoring_info->push_monitoring_interval = 0;
        monitoring_info->push_monitoring_timeout = 0;

        hostDef->monitoring_parameters = monitoring_info.in(); 
            
        // Add all the entity definitions to the host definition
        unsigned int entityCount = hostParser.get_entity_count();
        hostDef->entities.length( entityCount );
        for( unsigned int entityIndex=0; entityIndex<entityCount; entityIndex++ )
        {
            // Create an EntityDef
            CdmwPlatformMngt::EntityDef_var entityDef = Entity::get_definition(
            hostParser.get_entity( entityIndex ) );
            
            std::string newEntityName = entityDef->entity_name.in();

            // Checks there is no entity redefinition
            for( unsigned int l_entityIndex = 0;
             l_entityIndex < entityIndex;
             l_entityIndex++ )
            {
            std::string entityName
                = hostDef->entities[ l_entityIndex ].entity_name.in();
                
            if( newEntityName == entityName )
                throw CdmwPlatformMngt::DuplicateEntity(
                newEntityName.c_str() );
            }
            hostDef->entities[ entityIndex ] =  entityDef.in();
        }
    
        // Add Properties to host
        unsigned int propertyCount = hostParser.get_property_count();
        hostDef->properties.length( propertyCount );
        for( unsigned int propIndex=0; propIndex<propertyCount; propIndex++)
        {
            CdmwPlatformMngt::PropertyDef_var prop = Property::get_definition(hostParser.get_property(propIndex));
            hostDef->properties[propIndex] = prop;
        }
        
        //PRE_DEFINED properties
        std::ostringstream property_name;
        
        //AUTORESTART_INFO
        propertyCount =  hostDef->properties.length();
        hostDef->properties.length( propertyCount +1 );
    
        CdmwPlatformMngt::AutoRestartInfo * my_auto_restart_info = new (CdmwPlatformMngt::AutoRestartInfo);
        my_auto_restart_info->max_nbr_of_restart_attempt = hostParser.get_max_nbr_of_restart_attempt();
        my_auto_restart_info->auto_restart_control = hostParser.get_auto_restart_control();

        hostDef->properties[propertyCount].initial_value <<= *my_auto_restart_info;
        property_name.str("");
        property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::AUTORESTART_INFO << std::ends;
        hostDef->properties[propertyCount].property_name = property_name.str().c_str();
        hostDef->properties[propertyCount].mode_type = CosPropertyService::fixed_normal;

        return hostDef._retn();
        }
        catch( const std::bad_alloc& )
        {
        CDMW_THROW( OutOfMemoryException );
        }
    }

      
    void Host::add_property(const char* propertyConfigFileName) 
    {
        try
        {
                ConfigFileParser parser( propertyConfigFileName );
        PropertyParser propertyparser = parser.get_property();
        CdmwPlatformMngt::PropertyDefs_var propertydefs ;
        CosPropertyService::PropertyDef_var cos_prop = new CosPropertyService::PropertyDef;
        CosPropertyService::PropertyModeType prop_mode_type = CosPropertyService::PropertyModeType(0);
        CdmwPlatformMngt::PropertyDef_var prop = Property::get_definition(propertyparser);
                cos_prop->property_name = prop->property_name;
        cos_prop->property_value = prop->initial_value;
        if (prop->mode_type==1) prop_mode_type = CosPropertyService::PropertyModeType(1);
        cos_prop->property_mode = prop_mode_type;
        m_host -> define_property_with_mode(prop->property_name.in(),prop->initial_value, prop_mode_type);
        }
        catch( const CdmwPlatformMngt::IncompatibleStatus& e )
        {
        PlatformMngt::LogMngr::logMessage(
            PlatformMngt::ERR,
            MSG_ID_ADMIN_INCOMPATIBLE_STATUS );
        
        throw LoggedException();
        }
    }
    
    void Host::remove_property(const char* propertyName)
    {
           try
            {
                m_host -> delete_property (propertyName);
            }
            catch( const CdmwPlatformMngt::IncompatibleStatus& e )
            {
                PlatformMngt::LogMngr::logMessage(
                    PlatformMngt::ERR,
                    MSG_ID_ADMIN_INCOMPATIBLE_STATUS );

                throw LoggedException();
            }
    }
    
void Host::get_property (const char* propertyName, const char* owner)
{
    if( !check_only )
    {
        CORBA::Any_var prop_value = new CORBA::Any();
        try
        {
            m_host -> get_attribute (propertyName, prop_value.out() ,( ('P'== *owner) ? CdmwPlatformMngt::PRE_DEFINED : CdmwPlatformMngt::USER_DEFINED ));
                            
        }
        catch( const CdmwPlatformMngt::AttributeNotFound& e )
        {
            PlatformMngt::LogMngr::logMessage(
                    PlatformMngt::ERR,
                    "Attribute Not Found" );
            throw LoggedException();
        }

        CORBA::TypeCode_var tc;
        tc = prop_value->type();

        if (tc->equal(CdmwPlatformMngt::_tc_HostStatus) )
        {
            CdmwPlatformMngt::HostStatus value;
            *prop_value >>= value;
            std::string output("");
            switch( value )
            {
    
                case (CdmwPlatformMngt::HOST_RESPONDING) :
                output = "HOST_RESPONDING";
                break;
                case (CdmwPlatformMngt::HOST_UNREACHABLE) :
                output = "HOST_UNREACHABLE";
                break;
                case (CdmwPlatformMngt::HOST_STARTED) :
                output = "HOST_STARTED";
                break;
                case (CdmwPlatformMngt::HOST_PARTIAL_STARTED) :
                output = "HOST_PARTIAL_STARTED";
                break;
                case (CdmwPlatformMngt::HOST_STARTING) :
                output = "HOST_STARTING";
                break;
                case (CdmwPlatformMngt::HOST_STOPPING) :
                output = "HOST_STOPPING";
                break;
                case (CdmwPlatformMngt::HOST_REINSERTING) :
                output = "HOST_REINSERTING";
                break;
            }
            std::cout << propertyName << " : " << output << std::endl;

        } 
        else
        {
          Property::print_standard_property_type( propertyName, prop_value.in() );
        }
    }//End checkonly
}//End


void Host::set_property (const char* propertyName, const char* owner, const char* propertyValue)
{
    if( !check_only )
    {
        CORBA::Any_var any_var = new CORBA::Any();

        try
        {
            if ('P'== *owner)
            {
              *any_var = Property::prepare_predefined_any (propertyName, propertyValue);

            }
            else //user defined properties
            {
                    CORBA::Any_var prop_value = new CORBA::Any();
                    try
                    {
                        m_host -> get_attribute (propertyName, prop_value.out() ,
                                  ( ('P'== *owner) ? CdmwPlatformMngt::PRE_DEFINED : CdmwPlatformMngt::USER_DEFINED ));
                    }
                    catch( const CdmwPlatformMngt::AttributeNotFound& e )
                    {
                        PlatformMngt::LogMngr::logMessage(PlatformMngt::ERR, "Attribute Not Found" );
                        throw LoggedException();
                    }
                    CORBA::TypeCode_var tc = prop_value->type();
                    *any_var = Property::prepare_userdefined_any (tc.in(), propertyValue); 

            }
            //finally set the attribute
            m_host -> set_attribute (propertyName, any_var.in(),
                                    ( ('P'== *owner) ? CdmwPlatformMngt::PRE_DEFINED : CdmwPlatformMngt::USER_DEFINED));
            
        }
        catch( const CdmwPlatformMngt::AttributeNotFound& e )
        {
            PlatformMngt::LogMngr::logMessage( PlatformMngt::ERR,
                    "Attribute Not Found" );
            throw LoggedException();
        }
        catch (const CdmwPlatformMngt::ReadOnlyAttribute& e)
        {
            PlatformMngt::LogMngr::logMessage(
                    PlatformMngt::ERR, 
                    "Read Only Property");
            throw LoggedException();                      
        }
    }//End checkonly
}//End
    
    } // End namespace PlatformAdmin
} // End namespace Cdmw

