/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "platformadminlibrary/LBGroup.hpp"
#include "platformadminlibrary/LBGroupParser.hpp"
#include "platformadminlibrary/ConfigFileParser.hpp"
#include "SystemMngt/tools/AdminLogMessageId.hpp"
#include "platformadminlibrary/Property.hpp"
#include "platformadminlibrary/PropertyParser.hpp"
#include "platformadminlibrary/Entity.hpp"
#include "platformadminlibrary/EntityParser.hpp"

#include "SystemMngt/idllib/CdmwPlatformMngtComponent.stub.hpp"
#include "SystemMngt/platformlibrary/LogMngr.hpp"

extern bool check_only;

namespace Cdmw
{                               // Begin namespace Cdmw
  namespace PlatformAdmin
  {                             // Begin namespace PlatformAdmin

/**
 * class LBGroup
 */

    LBGroup::LBGroup (CdmwPlatformMngt::LBGroup_ptr lbgroup)
    {
      if (!check_only)
        m_lbgroup =
          CdmwPlatformMngt::LBGroup::_duplicate (lbgroup);
    }

    LBGroup::~LBGroup ()
    {
    }

    CdmwPlatformMngt::LBGroupSnapshot* LBGroup::get_snapshot()
    {
        CdmwPlatformMngt::LBGroupSnapshot_var snapshot;
    
        if( !check_only )
        snapshot = m_lbgroup->get_snapshot();

        return snapshot._retn();
    }
    
    void LBGroup::print_components()
    {
        // Gets all the components
        CdmwPlatformMngt::CcmComponents_var components;
        CdmwPlatformMngt::ComponentsIterator_var componentsIt;
        CORBA::ULong nbr_of_components;
                   
        nbr_of_components = m_lbgroup->get_number_of_components ();
        m_lbgroup->get_all_components (nbr_of_components, components.out (), componentsIt.out ());
                    
        for (unsigned int i = 0; i < nbr_of_components; i++)
        {
            std::cout << components[i]->name() << std::endl;
        }
    }
    void LBGroup::print_components_property (const char* propertyName, 
                                    const char* owner)
    {
            // Gets all the components
        CdmwPlatformMngt::CcmComponents_var components;
        CdmwPlatformMngt::ComponentsIterator_var componentsIt;
        CORBA::ULong nbr_of_components;
                   
        nbr_of_components = m_lbgroup->get_number_of_components ();
        m_lbgroup->get_all_components (nbr_of_components, components.out (), componentsIt.out ());
                    
        for (unsigned int i = 0; i < nbr_of_components; i++)
        {
            CORBA::Any_var prop_value = new CORBA::Any();

            try
            {
                components[i]-> get_attribute (propertyName, prop_value.out() , ( ('P'== *owner) ? CdmwPlatformMngt::PRE_DEFINED : CdmwPlatformMngt::USER_DEFINED ));
                            
            }
            catch( const CdmwPlatformMngt::AttributeNotFound& e )
            {
                PlatformMngt::LogMngr::logMessage(
                        PlatformMngt::ERR,
                        "Attribute Not Found" );
    
                throw LoggedException();
            }

            CORBA::TypeCode_var tc;
            tc = prop_value->type();

            if (tc->equal(CdmwPlatformMngt::_tc_ComponentStatus) )
            {
                CdmwPlatformMngt::ComponentStatus value;
                *prop_value >>= value;
                std::string output("");
                switch( value )
                {
                    case (CdmwPlatformMngt::COMPONENT_INITIAL) :
                    output = "COMPONENT_INITIAL";
                    break;
                    case (CdmwPlatformMngt::COMPONENT_INITIALISING) :
                    output = "COMPONENT_INITIALISING";
                    break;
                    case (CdmwPlatformMngt::COMPONENT_CONFIGURING) :
                    output = "COMPONENT_CONFIGURING";
                    break;
                    case (CdmwPlatformMngt::COMPONENT_INITIALISED) :
                    output = "COMPONENT_INITIALISED";
                    break;
                    case (CdmwPlatformMngt::COMPONENT_CONFIGURED) :
                    output = "COMPONENT_CONFIGURED";
                    break;
                    case (CdmwPlatformMngt::COMPONENT_STOPPING) :
                    output = "COMPONENT_STOPPING";
                    break;
                    case (CdmwPlatformMngt::COMPONENT_STOPPED) :
                    output = "COMPONENT_STOPPED";
                    break;
                    case (CdmwPlatformMngt::COMPONENT_FAILED_ERROR) :
                    output = "COMPONENT_FAILED_ERROR";
                    break;
                    case (CdmwPlatformMngt::COMPONENT_FAILED_TIMEOUT) :
                    output = "COMPONENT_FAILED_TIMEOUT";
                    break;
                    case (CdmwPlatformMngt::COMPONENT_FAILED_INVALID) :
                    output = "COMPONENT_FAILED_INVALID";
                    break;
                    }
                    std::cout << components[i]->name() << " , " <<propertyName << " : " << output << std::endl;
            }
            else
            {
                std::cout << components[i]->name() << " , "; Property::print_standard_property_type( propertyName, prop_value.in() );
            }
        }
    }
                                                                    
    CdmwPlatformMngt::LBGroupDef *
      LBGroup::
      get_definition (const LBGroupParser & lbgroupParser)
    {
      try
      {
        CdmwPlatformMngt::LBGroupDef_var lbgroupDef
          = new CdmwPlatformMngt::LBGroupDef ();

        // Get the lbgroup's name
        lbgroupDef->name =
          CORBA::string_dup (lbgroupParser.name ().c_str ());

        // Get the minimum number of replicas 
        lbgroupDef->minimum_nbr_replicas =
            (lbgroupParser.minimum_nbr_replicas());
        
        // Get the group style 
        lbgroupDef->style =
            (lbgroupParser.style());
        

        //Add Properties to LBGroup                                                                                                                              
        unsigned int propertyCount = lbgroupParser.get_property_count ();
        lbgroupDef->properties.length (propertyCount);

        for (unsigned int propIndex = 0; propIndex < propertyCount; propIndex++)
          {
            CdmwPlatformMngt::PropertyDef_var prop =
              Property::get_definition (lbgroupParser.
                                        get_property (propIndex));
            lbgroupDef->properties[propIndex] = prop;
          }


        // Add all the entity definitions to the lbgroup definition
        unsigned int entityCount = lbgroupParser.get_entity_count();
        lbgroupDef->entities.length( entityCount );
        for( unsigned int entityIndex=0; entityIndex<entityCount; entityIndex++ )
        {
            // Create an EntityDef
            CdmwPlatformMngt::EntityDef_var entityDef = Entity::get_definition(
                lbgroupParser.get_entity( entityIndex ) );
         
            std::string newEntityName = entityDef->entity_name.in();


            std:: stringstream str;
            //str<<"*** OK READ Entity (of LBGroup="<<lbgroupDef->name<<") ...Entity name="<<newEntityName<<std::endl; 

            std::string buffer(str.str());

            //CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG) , buffer) ;


            // Checks there is no entity redefinition
            for( unsigned int l_entityIndex = 0;
                 l_entityIndex < entityIndex;
                 l_entityIndex++ )
            {
                std::string entityName
                    = lbgroupDef->entities[ l_entityIndex ].entity_name.in();
                                         
                if( newEntityName == entityName )
                    throw CdmwPlatformMngt::DuplicateEntity(
                        newEntityName.c_str() );
            }

            lbgroupDef->entities[ entityIndex ] =  entityDef.in();
          }

        return lbgroupDef._retn ();
      }
      catch (const std::bad_alloc &)
      {
        CDMW_THROW (OutOfMemoryException);
      }
    }

    void LBGroup::add_property (const char *propertyConfigFileName)
    {
      try
      {
        ConfigFileParser parser (propertyConfigFileName);
        PropertyParser propertyparser = parser.get_property ();

        CosPropertyService::PropertyDef_var cos_prop =
          new CosPropertyService::PropertyDef;

        CosPropertyService::PropertyModeType prop_mode_type =
          CosPropertyService::PropertyModeType (0);

        CdmwPlatformMngt::PropertyDef_var prop =
          Property::get_definition (propertyparser);

        cos_prop->property_name = prop->property_name;

        cos_prop->property_value = prop->initial_value;

        if (prop->mode_type == 1)
          prop_mode_type = CosPropertyService::PropertyModeType (1);

        cos_prop->property_mode = prop_mode_type;

        m_lbgroup->define_property_with_mode (prop->property_name.in (),
                                                   prop->initial_value,
                                                   prop_mode_type);
      }
      catch (const CdmwPlatformMngt::IncompatibleStatus & e)
      {
        PlatformMngt::LogMngr::logMessage (PlatformMngt::ERR,
                                           MSG_ID_ADMIN_INCOMPATIBLE_STATUS);

        throw LoggedException ();
      }
    }


    void LBGroup::remove_property (const char *propertyName)
    {
      try
      {
        m_lbgroup->delete_property (propertyName);
      }
      catch (const CdmwPlatformMngt::IncompatibleStatus & e)
      {
        PlatformMngt::LogMngr::logMessage (PlatformMngt::ERR,
                                           MSG_ID_ADMIN_INCOMPATIBLE_STATUS);

        throw LoggedException ();
      }

    }

void LBGroup::get_property (const char* propertyName, const char* owner)
    {
    if( !check_only )
        {
            CORBA::Any_var prop_value = new CORBA::Any();

            try
            {
                m_lbgroup -> get_attribute (propertyName, prop_value.out() , ( ('P'== *owner) ? CdmwPlatformMngt::PRE_DEFINED : CdmwPlatformMngt::USER_DEFINED ));
                            
            }
            catch( const CdmwPlatformMngt::AttributeNotFound& e )
            {
                PlatformMngt::LogMngr::logMessage(
                        PlatformMngt::ERR,
                        "Attribute Not Found" );
    
                throw LoggedException();
            }

            CORBA::TypeCode_var tc;
            tc = prop_value->type();

            if (tc->equal(CdmwPlatformMngt::_tc_TechnicalStatus) )
            {
                CdmwPlatformMngt::TechnicalStatus value;
                *prop_value >>= value;
                std::string output("");
                switch( value )
                {
                    case (CdmwPlatformMngt::INITIAL) :
                    output = "INITIAL";
                    break;
                    case (CdmwPlatformMngt::FULLY_REDUNDANT) :
                    output = "FULLY_REDUNDANT";
                    break;
                    case (CdmwPlatformMngt::DEGRADED_REDUNDANCY) :
                    output = "DEGRADED_REDUNDANCY";
                    break;
                    case (CdmwPlatformMngt::LOST_REDUNDANCY) :
                    output = "LOST_REDUNDANCY";
                    break;
                    case (CdmwPlatformMngt::COMPLETE_FAILURE) :
                    output = "COMPLETE_FAILURE";
                    break;
                }
                std::cout << propertyName << " : " << output << std::endl;
            }
            else
            {
                Property::print_standard_property_type( propertyName, prop_value.in() );
            }
        }//End checkonly
    }//End

void LBGroup::set_property (const char* propertyName, const char* owner, const char* propertyValue)
    {
        if( !check_only )
        {
            CORBA::Any_var any_var = new CORBA::Any();
            try
            {
                if ('P'== *owner)
                {
                    *any_var = Property::prepare_predefined_any (propertyName, propertyValue);
                }
                else // User defined property
                {
                    CORBA::Any_var prop_value = new CORBA::Any();
                    try
                    {
                        m_lbgroup -> get_attribute (propertyName, prop_value.out() ,
                                       ( ('P'== *owner) ? CdmwPlatformMngt::PRE_DEFINED : CdmwPlatformMngt::USER_DEFINED ));
                    }
                    catch( const CdmwPlatformMngt::AttributeNotFound& e )
                    {
                        PlatformMngt::LogMngr::logMessage(PlatformMngt::ERR, "Attribute Not Found" );
                        throw LoggedException();
                    }
                    CORBA::TypeCode_var tc = prop_value->type();
                    *any_var = Property::prepare_userdefined_any (tc.in(), propertyValue); 
                 }
                 
                 //finally set the attribute
                 m_lbgroup -> set_attribute (propertyName, any_var.in(),
                                           ( ('P'== *owner) ? CdmwPlatformMngt::PRE_DEFINED : CdmwPlatformMngt::USER_DEFINED));
                
            }
            catch( const CdmwPlatformMngt::AttributeNotFound& e )
            {
                PlatformMngt::LogMngr::logMessage(
                        PlatformMngt::ERR,
                        "Attribute Not Found" );
                throw LoggedException();
            }
            catch (const CdmwPlatformMngt::ReadOnlyAttribute& e)
            {
                PlatformMngt::LogMngr::logMessage(
                        PlatformMngt::ERR, 
                        "Read Only Property");
                throw LoggedException();                      
            }
        }//End checkonly
    }//End
    
  }                             // End namespace PlatformAdmin
}                               // End namespace Cdmw
