/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <vector>

#include "platformadminlibrary/LBGroupParser.hpp"
#include "platformadminlibrary/EntityParser.hpp"

namespace Cdmw {         // Begin namespace Cdmw
namespace PlatformAdmin { // Begin namespace PlatformAdmin

LBGroupContainerParser::LBGroupContainerParser(
    const DOM_Node& entityContainerParser )
: ConfigElementParser( entityContainerParser ),
  EntityContainerParser( entityContainerParser)
{
}

unsigned int LBGroupContainerParser::get_lbgroup_count() const
{
    return get_child_count( "LB-group" );
}

LBGroupParser LBGroupContainerParser::get_lbgroup(
    unsigned int lbgroupIndex ) const
{
    DOM_Node LBGroup_Parser(
        get_child_element( "LB-group", lbgroupIndex ) );

    return LBGroupParser( LBGroup_Parser );
}

LBGroupParser::LBGroupParser( const DOM_Node& lbgroupParser )
: ConfigElementParser( lbgroupParser ),
EntityContainerParser( lbgroupParser )
{
}

std::string LBGroupParser::name() const
{
    return get_attribute_value( "Name" );
}

CdmwPlatformMngt::BalancingStyle LBGroupParser::style() const
{
    CdmwPlatformMngt::BalancingStyle m_style =  CdmwPlatformMngt::LB_ROUND_ROBIN ;

    if ( "ROUND_ROBIN" == get_attribute_value( "Style") )
    {
        m_style = CdmwPlatformMngt::LB_ROUND_ROBIN;
    }
    else if ( "RANDOM" == get_attribute_value( "Style"))
    {
        m_style = CdmwPlatformMngt::LB_RANDOM;
    }
    else if ( "IMPL_DEFINED" == get_attribute_value( "Style") )
    {
        m_style = CdmwPlatformMngt::LB_IMPL_DEFINED;
    }
    
    return m_style;
    
}

unsigned long LBGroupParser::minimum_nbr_replicas() const
{
    std::string minimum_nbr_replicas = get_attribute_value( "Minimum-nbr-replicas" );

    if( minimum_nbr_replicas == "" )
        throw NotFoundException();
        
    return atol( minimum_nbr_replicas.c_str() );
}


PropertyParser LBGroupParser::get_property(unsigned int propertyIndex) const
{
    DOM_Node property_parser(get_child_element("Property", propertyIndex ));
    return PropertyParser(property_parser);
}

                                                                                                                 
unsigned int LBGroupParser::get_property_count() const
{
     return get_child_count("Property");
}


} // End namespace PlatformAdmin
} // End namespace Cdmw

