/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "tao/AnyTypeCode/TypeCode.h"

#include <iostream>
#include <sstream>
#include "platformadminlibrary/PlatformAdminSnapshot.hpp"

using namespace std;
bool show_properties = false; //global variable to enable/disable properties in shapshots
//----------------------//
// Function definitions //
//----------------------//
string snapShotToString( CdmwPlatformMngt::SystemSnapshot* snapshot )
{
    ostringstream strSnapshot;

    strSnapshot << endl;
    strSnapshot << "|  " << endl;
    strSnapshot << "+- System" << endl;
    switch( snapshot->system_status )
    {
    case (CdmwPlatformMngt::SYSTEM_INITIAL) :
    strSnapshot << "|  status:      SYSTEM_INITIAL" << endl;
    break;
    case (CdmwPlatformMngt::SYSTEM_STARTING) :
    strSnapshot << "|  status:      SYSTEM_STARTING" << endl;
    break;
    case (CdmwPlatformMngt::SYSTEM_STARTUP_SUSPENDED) :
    strSnapshot << "|  status:      SYSTEM_STARTUP_SUSPENDED" << endl;
    break;
    case (CdmwPlatformMngt::SYSTEM_STARTED) :
    strSnapshot << "|  status:      SYSTEM_STARTED" << endl;
    break;
    case (CdmwPlatformMngt::SYSTEM_STOPPING) :
    strSnapshot << "|  status:      SYSTEM_STOPPING" << endl;
    break;
    case (CdmwPlatformMngt::SYSTEM_STOPPED) :
    strSnapshot << "|  status:      SYSTEM_STOPPED" << endl;
    break;
    case (CdmwPlatformMngt::SYSTEM_PARTIAL_STARTED) :
        strSnapshot << "|  status:      SYSTEM_PARTIAL_STARTED" << endl;
    break;
    }
    strSnapshot << "|  status info: " << snapshot->status_info.in() << endl;
    switch( snapshot->system_mode )
    {
    case (CdmwPlatformMngt::NORMAL_MODE) :
    strSnapshot << "|  mode:        NORMAL_MODE" << endl;
    break;
    case (CdmwPlatformMngt::DEGRADED_MODE) :
    strSnapshot << "|  mode:        DEGRADED_MODE" << endl;
    break;
    default:
    strSnapshot << "|  mode:        " << endl;
    }
    strSnapshot << "|  mode info:   " << snapshot->mode_info.in() << endl;
    strSnapshot << "|  element path:   " << snapshot->element_path.in() << endl;
    // Get the host snapshots
    int hostCount = snapshot->system_hosts.length();

    for( int hostIndex = 0;
         hostIndex < hostCount;
         hostIndex++ )
    {
        CdmwPlatformMngt::HostSnapshot hostSnapshot
            = snapshot->system_hosts[ hostIndex ];

        strSnapshot << snapShotToString( &hostSnapshot, "|  " ).c_str();
    }

    // Get the application snapshots
    int applicationCount = snapshot->system_applications.length();

    for( int applicationIndex = 0;
         applicationIndex < applicationCount;
         applicationIndex++ )
    {
        CdmwPlatformMngt::ApplicationSnapshot applicationSnapshot
            = snapshot->system_applications[ applicationIndex ];

        strSnapshot << snapShotToString( &applicationSnapshot, "|  " ).c_str();
    }

    // Get the entities snapshots
    int entityCount = snapshot->system_entities.length();

    for( int entityIndex = 0;
         entityIndex < entityCount;
         entityIndex++ )
    {
        CdmwPlatformMngt::EntitySnapshot entitySnapshot
            = snapshot->system_entities[ entityIndex ];

        strSnapshot << snapShotToString( &entitySnapshot, "|  " ).c_str();
    }

    // Get the properties snapshots
    int propertyCount = snapshot->system_properties.length();
     
    for( int propIndex = 0;
    propIndex < propertyCount;
    propIndex++ )
    {
        CdmwPlatformMngt::PropertySnapshot propertySnapshot
        = snapshot->system_properties[propIndex];

    strSnapshot << snapShotToString(&propertySnapshot,"|  ").c_str();
    }       

    return string( strSnapshot.str() );
}


string snapShotToString(
    CdmwPlatformMngt::HostSnapshot* snapshot,
    const char* sIndent )
{
    ostringstream strSnapshot;

    strSnapshot << sIndent << "|  " << endl;
    strSnapshot << sIndent << "+- Host" << endl;
    strSnapshot << sIndent << "|  name:        " << snapshot->host_name.in() << endl;
    
    switch( snapshot->host_status )
    {
    case (CdmwPlatformMngt::HOST_RESPONDING) :
    strSnapshot << sIndent << "|  status: HOST_RESPONDING" << endl;
    break;
    case (CdmwPlatformMngt::HOST_UNREACHABLE) :
    strSnapshot << sIndent << "|  status: HOST_UNREACHABLE" << endl;
    break;
    case (CdmwPlatformMngt::HOST_STARTED) :
        strSnapshot << sIndent << "|  status: HOST_STARTED" << endl;
        break;
    case (CdmwPlatformMngt::HOST_PARTIAL_STARTED) :
        strSnapshot << sIndent << "|  status: HOST_PARTIAL_STARTED" << endl;
        break;
    case (CdmwPlatformMngt::HOST_STARTING) :
        strSnapshot << sIndent << "|  status: HOST_STARTING" << endl;
        break;
    case (CdmwPlatformMngt::HOST_STOPPING) :
        strSnapshot << sIndent << "|  status: HOST_STOPPING" << endl;
        break;
    case (CdmwPlatformMngt::HOST_REINSERTING) :
        strSnapshot << sIndent << "|  status: HOST_REINSERTING" << endl;
        break;
    }

    strSnapshot << sIndent << "|  element path:   " << snapshot->element_path.in() << endl;
    // Get the entities snapshots
    int entityCount = snapshot->host_entities.length();

    for( int entityIndex = 0;
         entityIndex < entityCount;
         entityIndex++ )
    {
        CdmwPlatformMngt::EntitySnapshot entitySnapshot
            = snapshot->host_entities[ entityIndex ];

        strSnapshot << snapShotToString(&entitySnapshot, (string( sIndent ) + "|  ").c_str()).c_str();
    }

                                                                                                                                 
    // Get the properties snapshots
    int propertyCount = snapshot->host_properties.length();
                                                                                                                             
    for(int propIndex = 0;
    propIndex < propertyCount;
    propIndex++ )
    {
        CdmwPlatformMngt::PropertySnapshot propertySnapshot
        = snapshot->host_properties[propIndex];
        strSnapshot << snapShotToString(&propertySnapshot,(string( sIndent ) + "|  ").c_str()).c_str();
    }
    

    return string( strSnapshot.str() );
}

string snapShotToString(
    CdmwPlatformMngt::ApplicationSnapshot* snapshot,
    const char* sIndent )
{
    ostringstream strSnapshot;

    strSnapshot << sIndent << "|  " << endl;
    strSnapshot << sIndent << "+- Application" << endl;
    strSnapshot << sIndent << "|  name:        " << snapshot->application_name.in() << endl;
    switch( snapshot->application_status )
    {
    case (CdmwPlatformMngt::APPLICATION_INITIAL) :
    strSnapshot << sIndent << "|  status:      APPLICATION_INITIAL" << endl;
    break;
    case (CdmwPlatformMngt::APPLICATION_INITIALISING) :
    strSnapshot << sIndent << "|  status:      APPLICATION_INITIALISING" << endl;
    break;
    case (CdmwPlatformMngt::APPLICATION_INITIALISATION_SUSPENDED) :
    strSnapshot << sIndent << "|  status:      APPLICATION_INITIALISATION_SUSPENDED" << endl;
    break;
    case (CdmwPlatformMngt::APPLICATION_INITIALISATION_INCOMPLETE) :
    strSnapshot << sIndent << "|  status:      APPLICATION_INITIALISATION_INCOMPLETE" << endl;
    break;
    case (CdmwPlatformMngt::APPLICATION_INITIALISED) :
    strSnapshot << sIndent << "|  status:      APPLICATION_INITIALISED" << endl;
    break;
    case (CdmwPlatformMngt::APPLICATION_RUN_REQUEST) :
    strSnapshot << sIndent << "|  status:      APPLICATION_RUN_REQUEST" << endl;
    break;
    case (CdmwPlatformMngt::APPLICATION_RUNNING) :
    strSnapshot << sIndent << "|  status:      APPLICATION_RUNNING" << endl;
    break;
    case (CdmwPlatformMngt::APPLICATION_STOPPING) :
    strSnapshot << sIndent << "|  status:      APPLICATION_STOPPING" << endl;
    break;
    case (CdmwPlatformMngt::APPLICATION_STOPPED) :
    strSnapshot << sIndent << "|  status:      APPLICATION_STOPPED" << endl;
    break;
    //case (CdmwPlatformMngt::APPLICATION_FAILED) :
    //strSnapshot << sIndent << "|  status:      APPLICATION_FAILED" << endl;
    //break;
    case (CdmwPlatformMngt::APPLICATION_PARTIAL_RUNNING) :
        strSnapshot << sIndent << "|  status:      APPLICATION_PARTIAL_RUNNING" << endl;
    break;
    }
    strSnapshot << sIndent << "|  status info: " << snapshot->status_info.in() << endl;
    switch( snapshot->application_mode )
    {
    case (CdmwPlatformMngt::NORMAL_MODE) :
    strSnapshot << sIndent << "|  mode:        NORMAL_MODE" << endl;
    break;
    case (CdmwPlatformMngt::DEGRADED_MODE) :
    strSnapshot << sIndent << "|  mode:        DEGRADED_MODE" << endl;
    break;
    default:
    strSnapshot << sIndent << "|  mode:" << endl;
    }
    strSnapshot << sIndent << "|  mode info:   " << snapshot->mode_info.in() << endl;

    strSnapshot << sIndent << "|  element path:   " << snapshot->element_path.in() << endl;
    // Get the process snapshots
    int processCount = snapshot->application_processes.length();

    for( int processIndex = 0;
         processIndex < processCount;
         processIndex++ )
    {
        CdmwPlatformMngt::ProcessSnapshot processSnapshot
            = snapshot->application_processes[ processIndex ];

        strSnapshot << snapShotToString(&processSnapshot, (string( sIndent ) + "|  ").c_str()).c_str();
    }

    // Get the entities snapshots
    int entityCount = snapshot->application_entities.length();

    for( int entityIndex = 0;
         entityIndex < entityCount;
         entityIndex++ )
    {
        CdmwPlatformMngt::EntitySnapshot entitySnapshot
            = snapshot->application_entities[ entityIndex ];

        strSnapshot << snapShotToString(&entitySnapshot, (string( sIndent ) + "|  ").c_str()).c_str();
    }

    // Get the properties snapshots
    int propertyCount = snapshot->application_properties.length();
    for(int propIndex = 0;
    propIndex < propertyCount;
    propIndex++ )
    {
        CdmwPlatformMngt::PropertySnapshot propertySnapshot
        = snapshot->application_properties[propIndex];
    strSnapshot << snapShotToString(&propertySnapshot,(string( sIndent ) + "|  ").c_str()).c_str();
    }       
 
    //Get the ftgroups snapshots ...
    int ftgroupCount = snapshot->application_ft_component_groups.length();

     for(int ftgroupIndex = 0;
     ftgroupIndex < ftgroupCount;
     ftgroupIndex++ )
     {
         CdmwPlatformMngt::FTGroupSnapshot ftgroupSnapshot
        = snapshot->application_ft_component_groups[ftgroupIndex];
        strSnapshot << snapShotToString(&ftgroupSnapshot,(string( sIndent ) + "|  ").c_str()).c_str();
     }      
    


    //Get the lbgroups snapshots ...
    
    int lbgroupCount = snapshot->application_lb_component_groups.length();

     for(int lbgroupIndex = 0;
     lbgroupIndex < lbgroupCount;
     lbgroupIndex++ )
     {
         CdmwPlatformMngt::LBGroupSnapshot lbgroupSnapshot
        = snapshot->application_lb_component_groups[lbgroupIndex];
        strSnapshot << snapShotToString(&lbgroupSnapshot,(string( sIndent ) + "|  ").c_str()).c_str();
     }      
    
    
    return string( strSnapshot.str() );
}

string snapShotToString(
    CdmwPlatformMngt::ProcessSnapshot* snapshot,
    const char* sIndent )
{
    ostringstream strSnapshot;

    strSnapshot << sIndent << "|  " << endl;
    strSnapshot << sIndent << "+- Process" << endl;
    strSnapshot << sIndent << "|  name:        " << snapshot->process_name.in() << endl;
    strSnapshot << sIndent << "|  host:        " << snapshot->host_name.in() << endl;
    switch( snapshot->process_status )
    {
    case (CdmwPlatformMngt::PROCESS_INITIAL) :
    strSnapshot << sIndent << "|  status:      PROCESS_INITIAL" << endl;
    break;
    case (CdmwPlatformMngt::PROCESS_INITIALISING) :
    strSnapshot << sIndent << "|  status:      PROCESS_INITIALISING" << endl;
    break;
    case (CdmwPlatformMngt::PROCESS_STEP_PERFORMED) :
    strSnapshot << sIndent << "|  status:      PROCESS_STEP_PERFORMED" << endl;
    break;
    case (CdmwPlatformMngt::PROCESS_INITIALISED) :
    strSnapshot << sIndent << "|  status:      PROCESS_INITIALISED" << endl;
    break;
    case (CdmwPlatformMngt::PROCESS_RUN_REQUEST) :
    strSnapshot << sIndent << "|  status:      PROCESS_RUN_REQUEST" << endl;
    break;
    case (CdmwPlatformMngt::PROCESS_RUNNING) :
    strSnapshot << sIndent << "|  status:      PROCESS_RUNNING" << endl;
    break;
    case (CdmwPlatformMngt::PROCESS_STOPPING) :
    strSnapshot << sIndent << "|  status:      PROCESS_STOPPING" << endl;
    break;
    case (CdmwPlatformMngt::PROCESS_STOPPED) :
    strSnapshot << sIndent << "|  status:      PROCESS_STOPPED" << endl;
    break;
    case (CdmwPlatformMngt::PROCESS_ENDED) :
    strSnapshot << sIndent << "|  status:      PROCESS_ENDED" << endl;
    break;
    case (CdmwPlatformMngt::PROCESS_FAILED_ERROR) :
    strSnapshot << sIndent << "|  status:      PROCESS_FAILED_ERROR" << endl;
    break;
    case (CdmwPlatformMngt::PROCESS_FAILED_TIMEOUT) :
    strSnapshot << sIndent << "|  status:      PROCESS_FAILED_TIMEOUT" << endl;
    break;
    case (CdmwPlatformMngt::PROCESS_FAILED_DISCREPANCY) :
    strSnapshot << sIndent << "|  status:      PROCESS_FAILED_DISCREPANCY" << endl;
    break;
    case (CdmwPlatformMngt::PROCESS_FAILED_INVALID) :
    strSnapshot << sIndent << "|  status:      PROCESS_FAILED_INVALID" << endl;
    break;
    case (CdmwPlatformMngt::PROCESS_FAILED_NO_RESPONSE) :
    strSnapshot << sIndent << "|  status:      PROCESS_FAILED_NO_RESPONSE" << endl;
    break;
    case (CdmwPlatformMngt::PROCESS_FAILED_DEATH) :
    strSnapshot << sIndent << "|  status:      PROCESS_FAILED_DEATH" << endl;
    break;
    }
    strSnapshot << sIndent << "|  status info: " << snapshot->process_status_info.in() << endl;
    strSnapshot << sIndent << "|  element path:   " << snapshot->element_path.in() << endl;
    // Get the entities snapshots
    int entityCount = snapshot->process_entities.length();

    for( int entityIndex = 0;
         entityIndex < entityCount;
         entityIndex++ )
    {
        CdmwPlatformMngt::EntitySnapshot entitySnapshot
            = snapshot->process_entities[ entityIndex ];

        strSnapshot << snapShotToString(&entitySnapshot, (string( sIndent )  + "|  ").c_str()).c_str();
    }
    
    // Get the properties snapshots
    int propertyCount = snapshot->process_properties.length();
    
    for(int propIndex = 0;
    propIndex < propertyCount;
    propIndex++ )
    {
        CdmwPlatformMngt::PropertySnapshot propertySnapshot
        = snapshot->process_properties[propIndex];
        strSnapshot << snapShotToString(&propertySnapshot,(string( sIndent ) + "|  ").c_str()).c_str();
    }       

    // Get the components snapshots
    int componentCount = snapshot->process_components.length();
    for( int componentIndex = 0;
         componentIndex < componentCount;
         componentIndex++ )
    {
        CdmwPlatformMngt::ComponentSnapshot componentSnapshot
            = snapshot->process_components[ componentIndex ];

        strSnapshot << snapShotToString(&componentSnapshot, (string( sIndent ) + "|  ").c_str()).c_str();
    }
    
    return string( strSnapshot.str() );
}

string snapShotToString(
    CdmwPlatformMngt::EntitySnapshot* snapshot,
    const char* sIndent )
{
    ostringstream strSnapshot;
    
    strSnapshot << sIndent << "|  " << endl;
    strSnapshot << sIndent << "+- Entity" << endl;
    strSnapshot << sIndent << "|  name  :  " << snapshot->entity_name.in() << endl;
    strSnapshot << sIndent << "|  status:  " << snapshot->entity_status.in() << endl;
    strSnapshot << sIndent << "|  info  :  " << snapshot->entity_info.in() << endl;
    strSnapshot << sIndent << "|  element path:   " << snapshot->element_path.in() << endl;
    // Get the properties snapshots
    int propertyCount = snapshot->entity_properties.length();
    
    for(int propIndex = 0;
    propIndex < propertyCount;
    propIndex++ )
    {
        CdmwPlatformMngt::PropertySnapshot propertySnapshot
        = snapshot->entity_properties[propIndex];
    strSnapshot << snapShotToString(&propertySnapshot,(string( sIndent ) + "|  ").c_str()).c_str();
    }       
    
    return string( strSnapshot.str() );
}

string snapShotToString(
    CdmwPlatformMngt::FTGroupSnapshot *snapshot,
    const char* sIndent )
{
    ostringstream strSnapshot;
    strSnapshot << sIndent << "|  " << endl;
    strSnapshot << sIndent << "+- FT-group" << endl;
    strSnapshot << sIndent << "|  name  :  " << snapshot->ft_group_name.in() << endl;


    switch( snapshot->ft_group_status)
    {
    case (CdmwPlatformMngt::INITIAL) :
        strSnapshot << sIndent << "|  status:      INITIAL" << endl;
        break;
    case (CdmwPlatformMngt::FULLY_REDUNDANT) :
        strSnapshot << sIndent << "|  status:      FULLY_REDUNDANT" << endl;
        break;
    case (CdmwPlatformMngt::DEGRADED_REDUNDANCY) :
        strSnapshot << sIndent << "|  status:      DEGRADED_REDUNDANCY" << endl;
        break;
    case (CdmwPlatformMngt::LOST_REDUNDANCY) :
        strSnapshot << sIndent << "|  status:      LOST_REDUNDANCY" << endl;
        break;
    case (CdmwPlatformMngt::COMPLETE_FAILURE) :
        strSnapshot << sIndent << "|  status:      COMPLETE_FAILURE" << endl;
        break;  
    }
    strSnapshot << sIndent << "|  status info  :  " << snapshot->status_info.in() << endl;

    strSnapshot << sIndent << "|  element path:   " << snapshot->element_path.in() << endl;
    

    // Get the entities snapshots
    int entityCount = snapshot->ft_group_entities.length();

    for( int entityIndex = 0; 
         entityIndex < entityCount;
         entityIndex++ )
    {
        CdmwPlatformMngt::EntitySnapshot entitySnapshot
            = snapshot->ft_group_entities[ entityIndex ];



        strSnapshot << snapShotToString(
            &entitySnapshot,
            (string( sIndent )  + "|  ").c_str()
            ).c_str();

    } 

    // Get the properties snapshots
    int propertyCount = snapshot->ft_group_properties.length();
    
    for(int propIndex = 0;
        propIndex < propertyCount;
        propIndex++ )
    {
        CdmwPlatformMngt::PropertySnapshot propertySnapshot
            = snapshot->ft_group_properties[propIndex];
        strSnapshot << snapShotToString(&propertySnapshot,(string( sIndent ) + "| ").c_str()).c_str();
    }       
    
    return string( strSnapshot.str() );
}


string snapShotToString(
    CdmwPlatformMngt::LBGroupSnapshot *snapshot,
    const char* sIndent )
{
    ostringstream strSnapshot;
    
    strSnapshot << sIndent << "|  " << endl;
    strSnapshot << sIndent << "+- LB-group" << endl;
    strSnapshot << sIndent << "|  name  :  " << snapshot->lb_group_name.in() << endl;

    switch( snapshot->lb_group_status)
    {
    case (CdmwPlatformMngt::INITIAL) :
        strSnapshot << sIndent << "|  status:      INITIAL" << endl;
        break;
    case (CdmwPlatformMngt::FULLY_REDUNDANT) :
        strSnapshot << sIndent << "|  status:      FULLY_REDUNDANT" << endl;
        break;
    case (CdmwPlatformMngt::DEGRADED_REDUNDANCY) :
        strSnapshot << sIndent << "|  status:      DEGRADED_REDUNDANCY" << endl;
        break;
    case (CdmwPlatformMngt::LOST_REDUNDANCY) :
        strSnapshot << sIndent << "|  status:      LOST_REDUNDANCY" << endl;
        break;
    case (CdmwPlatformMngt::COMPLETE_FAILURE) :
        strSnapshot << sIndent << "|  status:      COMPLETE_FAILURE" << endl;
        break;  
    }
    strSnapshot << sIndent << "|  status info  :  " << snapshot->status_info.in() << endl;

    strSnapshot << sIndent << "|  element path:   " << snapshot->element_path.in() << endl;
    
    // Get the entities snapshots
    int entityCount = snapshot->lb_group_entities.length();

    for( int entityIndex = 0; 
         entityIndex < entityCount;
         entityIndex++ )
    {
        CdmwPlatformMngt::EntitySnapshot entitySnapshot
            = snapshot->lb_group_entities[ entityIndex ];



        strSnapshot << snapShotToString(
            &entitySnapshot,
            (string( sIndent )  + "|  ").c_str()
            ).c_str();

    } 


    // Get the properties snapshots
    int propertyCount = snapshot->lb_group_properties.length();
    
    for(int propIndex = 0;
        propIndex < propertyCount;
        propIndex++ )
    {
        CdmwPlatformMngt::PropertySnapshot propertySnapshot
            = snapshot->lb_group_properties[propIndex];
        strSnapshot << snapShotToString(&propertySnapshot,(string( sIndent ) + "| ").c_str()).c_str();
    }      


    return string( strSnapshot.str() );
}


string snapShotToString(
    CdmwPlatformMngt::ComponentSnapshot* snapshot,
    const char* sIndent )
{
    ostringstream strSnapshot;
    
    strSnapshot << sIndent << "|  " << endl;
    strSnapshot << sIndent << "+- Component" << endl;
    strSnapshot << sIndent << "|  name  :  " << snapshot->component_name.in() << endl;
    switch( snapshot->component_status )
    {
    case (CdmwPlatformMngt::COMPONENT_INITIAL) :
    strSnapshot << sIndent << "|  status:      COMPONENT_INITIAL" << endl;
    break;
    case (CdmwPlatformMngt::COMPONENT_INITIALISING) :
    strSnapshot << sIndent << "|  status:      COMPONENT_INITIALISING" << endl;
    break;
    case (CdmwPlatformMngt::COMPONENT_CONFIGURING) :
    strSnapshot << sIndent << "|  status:      COMPONENT_CONFIGURING" << endl;
    break;
    case (CdmwPlatformMngt::COMPONENT_INITIALISED) :
    strSnapshot << sIndent << "|  status:      COMPONENT_INITIALISED" << endl;
    break;
    case (CdmwPlatformMngt::COMPONENT_CONFIGURED) :
    strSnapshot << sIndent << "|  status:      COMPONENT_CONFIGURED" << endl;
    break;
    case (CdmwPlatformMngt::COMPONENT_STOPPING) :
    strSnapshot << sIndent << "|  status:      COMPONENT_STOPPING" << endl;
    break;
    case (CdmwPlatformMngt::COMPONENT_STOPPED) :
    strSnapshot << sIndent << "|  status:      COMPONENT_STOPPED" << endl;
    break;
    case (CdmwPlatformMngt::COMPONENT_FAILED_ERROR) :
    strSnapshot << sIndent << "|  status:      COMPONENT_FAILED_ERROR" << endl;
    break;
    case (CdmwPlatformMngt::COMPONENT_FAILED_TIMEOUT) :
    strSnapshot << sIndent << "|  status:      COMPONENT_FAILED_TIMEOUT" << endl;
    break;
    case (CdmwPlatformMngt::COMPONENT_FAILED_INVALID) :
    strSnapshot << sIndent << "|  status:      COMPONENT_FAILED_INVALID" << endl;
    break;
    }
    strSnapshot << sIndent << "|  status info  :  " << snapshot->component_status_info.in() << endl;
    strSnapshot << sIndent << "|  element path :  " << snapshot->element_path.in() << endl;
    strSnapshot << sIndent << "|  deployment plan:" << snapshot->component_deployment_plan.in() << endl;


    // Get the entities snapshots
    int entityCount = snapshot->component_entities.length();

    for( int entityIndex = 0;
         entityIndex < entityCount;
         entityIndex++ )
    {
        CdmwPlatformMngt::EntitySnapshot entitySnapshot
            = snapshot->component_entities[ entityIndex ];



        strSnapshot << snapShotToString(
            &entitySnapshot,
            (string( sIndent )  + "|  ").c_str()
            ).c_str();

    } 

    // Get the properties snapshots
    int propertyCount = snapshot->component_properties.length();
    
    for(int propIndex = 0;
    propIndex < propertyCount;
    propIndex++ )
    {
        CdmwPlatformMngt::PropertySnapshot propertySnapshot
        = snapshot->component_properties[propIndex];
//  strSnapshot << snapShotToString(&propertySnapshot,"|  ").c_str();
    strSnapshot << snapShotToString(&propertySnapshot,(string( sIndent ) + "|  ").c_str()).c_str();
    }       
    
    return string( strSnapshot.str() );
}

string snapShotToString( 
    CdmwPlatformMngt::PropertySnapshot* snapshot, 
    const char* sIndent)
{
    ostringstream strSnapshot("");

    if (true == show_properties) 
    {
    strSnapshot << sIndent << "|  " << endl;
    strSnapshot << sIndent << "+- Property" << endl;
    strSnapshot << sIndent << "|  name  :  " << snapshot->property_name.in() << endl;
    //strSnapshot << sIndent << "|  type  :  " << snapshot->property_type.in() << endl;
    strSnapshot << sIndent << "|  mode  :  " << snapshot->property_mode.in() << endl;
    
    // Preparing the output for the Properties  
    CORBA::Any prop_value = snapshot -> property_value;      
    CORBA::TypeCode_var tc;
    tc = prop_value.type();

    if (tc->equal(CdmwPlatformMngt::_tc_RestartWarmth) )
    {
        strSnapshot << sIndent << "|  type  :  RestartWarmth" << endl;
        CdmwPlatformMngt::RestartWarmth value;
    prop_value >>= value;
        strSnapshot << sIndent << "|  value :  "<< ( (CdmwPlatformMngt::HOT_RESTART == value)?"Hot":"Cold" ) <<endl;   
    } else if (tc->equal(CdmwPlatformMngt::_tc_AutoRestartInfo) )
    {
        strSnapshot << sIndent << "|  type  :  AutoRestartInfo" << endl;
        CdmwPlatformMngt::AutoRestartInfo *value = new (CdmwPlatformMngt::AutoRestartInfo);
        prop_value >>= value;
        string control("");
    switch (value->auto_restart_control)
        {
        case (CdmwPlatformMngt::IGNORE_COUNTER):
        control="IGNORE_COUNTER";
        break;
        case (CdmwPlatformMngt::IGNORE_ACTION):
        control="IGNORE_ACTION";
        break;
        default:
        control="USE_COUNTER";
        break;
        }
        strSnapshot << sIndent << "|  value :  "<< ( value->max_nbr_of_restart_attempt)<<":"<<control.c_str()<< endl;
    } else if (tc->equal(CdmwPlatformMngt::_tc_RedundancyEltState) )
    {
        strSnapshot << sIndent << "|  type  :  RedundancyEltState" << endl;
        CdmwPlatformMngt::RedundancyEltState value;
        prop_value >>= value;

    switch (value)
     {
         case CdmwPlatformMngt::REDSTATE_NORG:
           strSnapshot << sIndent << "|  value :  "<< "REDSTATE_NORG"  <<endl;   
       break;

         case CdmwPlatformMngt::REDSTATE_PRIMARY:
           strSnapshot << sIndent << "|  value :  "<< "REDSTATE_PRIMARY"  <<endl;   
       break;

         case CdmwPlatformMngt::REDSTATE_PASSIVE:
           strSnapshot << sIndent << "|  value :  "<< "REDSTATE_PASSIVE"  <<endl;   
       break;
       
         case CdmwPlatformMngt::REDSTATE_ACTIVE:
           strSnapshot << sIndent << "|  value :  "<< "REDSTATE_ACTIVE"  <<endl;   
       break;

     }
        
        //strSnapshot << sIndent << "|  value :  NO_IMPL"<< endl;
    } else 
    {
        switch (tc->kind())
        {
    
        case (CORBA::tk_string):  
        {
    //snapshot -> property_type = CORBA::string_dup("String");
    strSnapshot << sIndent << "|  type  :  String" << endl;
        const char* value;
    prop_value >>= value ;
        strSnapshot << sIndent << "|  value :  " << value << endl;
        }
        break;  
    
        case (CORBA::tk_float): 
        { 
    //snapshot -> property_type = CORBA::string_dup("Float");
    strSnapshot << sIndent << "|  type  :  Float" << endl;
        CORBA::Float value;
    prop_value >>= value;
        strSnapshot << sIndent << "|  value :  " << value << endl;
        }
        break;

        case (CORBA::tk_short):
        {  
    //snapshot -> property_type = CORBA::string_dup("Short");
    strSnapshot << sIndent << "|  type  :  Short" << endl;
        short value;
    prop_value >>= value;
        strSnapshot << sIndent << "|  value :  " << value << endl;
        }   
        break;
        
        case (CORBA::tk_boolean):
        {
        strSnapshot << sIndent << "|  type  :  Boolean" << endl;
        CORBA::Boolean value;
        prop_value >>= CORBA::Any::to_boolean(value);
        strSnapshot << sIndent << "|  value :  " << value << endl;
        }
        break;

        case (CORBA::tk_long):
        {
        strSnapshot << sIndent << "|  type  :  Long" << endl;
        CORBA::Long value;
        prop_value >>= value;
        strSnapshot << sIndent << "|  value :  " << value << endl;
        }
        break;

        default:
        {
        strSnapshot << sIndent << "|  type  :  UNKNOWN" << endl;
        strSnapshot << sIndent << "|  value :  N/A" <<  endl;
        }
        break;
        } //End switch              
    
    }//End else
     
    }//End if (true==show_properties)
            
    //TO DO OTHER FIELD OF THE PROPERTY (UAD)
    return string(strSnapshot.str() );      
}





