/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <cstdlib>

#include "Foundation/orbsupport/OrbSupport.hpp"

#include "SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"

#include "SystemMngt/platformlibrary/LogMngr.hpp"
#include "SystemMngt/platformvaluetypes/ProcessDef_impl.hpp"

#include "platformadminlibrary/Process.hpp"
#include "platformadminlibrary/Service.hpp"
#include "platformadminlibrary/ConfigFileParser.hpp"
#include "platformadminlibrary/TimeoutParser.hpp"
#include "platformadminlibrary/ServiceParser.hpp"
#include "platformadminlibrary/ProcessParser.hpp"
#include "platformadminlibrary/EntityParser.hpp"
#include "platformadminlibrary/LifeCycleParser.hpp"
#include "SystemMngt/tools/AdminLogMessageId.hpp"
#include "platformadminlibrary/Property.hpp"
#include "platformadminlibrary/PropertyParser.hpp"
#include "platformadminlibrary/CcmComponent.hpp"
#include "platformadminlibrary/Graph.hpp"
#include "platformadminlibrary/GraphParser.hpp"



extern bool check_only;

namespace Cdmw {         // Begin namespace Cdmw
    namespace PlatformAdmin { // Begin namespace PlatformAdmin

    Process::Process( CdmwPlatformMngt::Process_ptr process )
        : ServiceContainer( process ),
          EntityContainer( process )
    {
        if( !check_only )
        m_process = CdmwPlatformMngt::Process::_duplicate( process );
    }

    Process::~Process()
    {
    }

    CdmwPlatformMngt::ProcessSnapshot* Process::get_snapshot()
    {
        CdmwPlatformMngt::ProcessSnapshot_var snapshot;
    
        if( !check_only )
        snapshot = m_process->get_snapshot();

        return snapshot._retn();
    }

    void Process::initialise( const CdmwPlatformMngtBase::StartupKind& startup_kind )
    {
        try
        {
        if( !check_only )
            m_process->initialise( startup_kind );
        }
        catch( const CdmwPlatformMngt::IncompatibleStatus& e )
        {
        PlatformMngt::LogMngr::logMessage(
            PlatformMngt::ERR,
            MSG_ID_ADMIN_INCOMPATIBLE_STATUS );

        throw LoggedException();
        }
    }

    void Process::next_step()
    {
        try
        {
        if( !check_only )
            m_process->next_step();
        }
        catch( const CdmwPlatformMngt::InvalidStep& e )
        {
        PlatformMngt::LogMngr::logMessage(
            PlatformMngt::ERR,
            MSG_ID_ADMIN_INVALID_STEP );

        throw LoggedException();
        }
        catch( const CdmwPlatformMngt::IncompatibleType& e )
        {
        PlatformMngt::LogMngr::logMessage(
            PlatformMngt::ERR,
            MSG_ID_ADMIN_INCOMPATIBLE_TYPE );

        throw LoggedException();
        }
        catch( const CdmwPlatformMngt::IncompatibleStatus& e )
        {
        PlatformMngt::LogMngr::logMessage(
            PlatformMngt::ERR,
            MSG_ID_ADMIN_INCOMPATIBLE_STATUS );

        throw LoggedException();
        }
    }

    void Process::run()
    {
        try
        {
        if( !check_only )
            m_process->run();
        }
        catch( const CdmwPlatformMngt::NotReadyToRun& e )
        {
        PlatformMngt::LogMngr::logMessage(
            PlatformMngt::ERR,
            MSG_ID_ADMIN_NOT_READY_TO_RUN,
            m_process->name() );

        throw LoggedException();
        }
        catch( const CdmwPlatformMngt::IncompatibleType& )
        {
        PlatformMngt::LogMngr::logMessage(
            PlatformMngt::ERR,
            MSG_ID_ADMIN_INCOMPATIBLE_TYPE );

        throw LoggedException();
        }
        catch( const CdmwPlatformMngt::IncompatibleStatus& )
        {
        PlatformMngt::LogMngr::logMessage(
            PlatformMngt::ERR,
            MSG_ID_ADMIN_INCOMPATIBLE_STATUS );

        throw LoggedException();
        }
    }

    CdmwPlatformMngt::RequestStatus Process::stop( bool emergency )
    {
        CdmwPlatformMngt::RequestStatus req;
        if( !check_only )
        {
            try
            {
                req = m_process->stop_process( emergency );
            }
            catch( ::CORBA::SystemException& ex ) 
            {
                throw LoggedException();
            }
            catch( ... ) 
            {
                PlatformMngt::LogMngr::logMessage(
                        PlatformMngt::ERR,
                        MSG_ID_ADMIN_UNKNOWN_ERROR );
                throw LoggedException();
            }
        }
        return req;
    }

    CdmwPlatformMngt::ProcessDef* Process::get_definition(
        const ProcessParser& processParser )
    {
        try
        {
        // Create the process definition
        CdmwPlatformMngt::ProcessDef_var processDef;

        if( processParser.type() == CdmwPlatformMngt::UNMANAGED_PROCESS )
        {
#if CDMW_ORB_VDR == tao
            PlatformMngt::ProcessDefFactory processDefFactory;
            processDef = processDefFactory.create(
            processParser.name().c_str() );
#else
            CORBA::ValueFactoryBase_var processDefFactory
                            = new PlatformMngt::ProcessDefFactory();
            processDef = dynamic_cast< PlatformMngt::ProcessDefFactory* >(
            processDefFactory.in() )->create(
                processParser.name().c_str() ); 
#endif
        }
        else
        {
#if CDMW_ORB_VDR == tao
            PlatformMngt::ManagedProcessDefFactory managedProcessDefFactory;
            processDef
            = managedProcessDefFactory.create(
                processParser.name().c_str(),
                processParser.type() );
#else
            CORBA::ValueFactoryBase_var managedProcessDefFactory
            = new PlatformMngt::ManagedProcessDefFactory();

            processDef
            = dynamic_cast< PlatformMngt::ManagedProcessDefFactory* >(
                managedProcessDefFactory.in() )->create(
                processParser.name().c_str(),
                processParser.type() );
#endif
        }
        // Set the process definition
                  
        processDef->auto_ending( processParser.is_autoended() ); 
        
        //This value is used only by UNMANAGED PROCESSES 
        if( CdmwPlatformMngt::UNMANAGED_PROCESS == processParser.type() )
        {
            processDef->initialisation_timeout( processParser.initialisation_timeout() ); 
        }
        else 
            processDef->initialisation_timeout ( 0 );
        
        processDef->host_name( processParser.host_name().c_str() );
               
        // Set the process info
        CdmwPlatformMngt::ProcessInfo_var processInfo = new CdmwPlatformMngt::ProcessInfo();
        processInfo->process_exe = CORBA::string_dup( processParser.executable_path().c_str() );
        processInfo->process_port = processParser.port();
        processInfo->working_directory = CORBA::string_dup( processParser.run_directory().c_str() );
        processInfo->process_args = CORBA::string_dup( processParser.arguments().c_str() );
        processInfo->filters = CORBA::string_dup(processParser.filters().c_str() ) ;
        processInfo->requested_restart_warmth=CdmwPlatformMngt::HOT_RESTART;
        processInfo->process_env=CORBA::string_dup( processParser.get_process_env().c_str() );
        processInfo->memory_resident= ( processParser.get_memory_resident() );
        processInfo->core_dump_size= ( processParser.get_core_size() ) ;
        processInfo->run_as_root= ( processParser.get_run_as_root() );
        
        processDef->process_info( processInfo.in() );
                
        //Add ExitAutoRestartInfo to Process

        //CdmwPlatformMngt::ExitAutoRestartInfo_var exitAutorestartInfo
        //    = new CdmwPlatformMngt::ExitAutoRestartInfo ();

        //exitAutorestartInfo->default_exit_action=processParser.get_default_exit_action();
        //exitAutorestartInfo->exit_actions.length(processParser.get_exit_action_count());

        (processInfo->exit_autorestart_info).exit_actions.length(processParser.get_exit_action_count());

        for (unsigned int i=0; i<processParser.get_exit_action_count(); i++)
        {
            //exitAutorestartInfo->exit_actions[i].exit_action=processParser.get_exit_action(i).get_type();
            //exitAutorestartInfo->exit_actions[i].exit_code=processParser.get_exit_action(i).get_exit_code();
            (processInfo->exit_autorestart_info).exit_actions[i].exit_action=processParser.get_exit_action(i).get_type();
            (processInfo->exit_autorestart_info).exit_actions[i].exit_code=processParser.get_exit_action(i).get_exit_code();
        }

        //load other fields of processInfo->exit_autorestart_info
        (processInfo->exit_autorestart_info).default_exit_action=processParser.get_exit_action("Default-automatic-action");
        (processInfo->exit_autorestart_info).max_nbr_reached_action=processParser.get_exit_action("Max-nbr-reached-action");
        (processInfo->exit_autorestart_info).abnormal_behaviour_action=processParser.get_exit_action("Abnormal-behaviour-action");
        processDef->process_info( processInfo.in() );

        //processDef->exit_autorestart_info( exitAutorestartInfo.in() );  
    
        //Add Properties to Process                                                                                                                               
        unsigned int propertyCount = processParser.get_property_count();
        processDef->properties().length( propertyCount );
                                                                                                                             
        for( unsigned int propIndex=0; propIndex<propertyCount; propIndex++)
        {
            CdmwPlatformMngt::PropertyDef_var prop = Property::get_definition(processParser.get_property(propIndex));
            processDef->properties()[propIndex] = prop.in();
        }
                   

        //PRE_DEFINED properties
        std::ostringstream property_name;

        //DEFAULT_RESTART_WARMTH
        propertyCount =  processDef->properties().length();
        processDef->properties().length( propertyCount +1 );

        std::string default_warmth = processParser.get_default_restart_warmth();
        if (default_warmth == "Cold")
        {
            processDef->properties()[propertyCount].initial_value <<= CdmwPlatformMngt::COLD_RESTART;
        }
        else
        {
            processDef->properties()[propertyCount].initial_value <<= CdmwPlatformMngt::HOT_RESTART;
        }

        property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::DEFAULT_RESTART_WARMTH << std::ends;
        processDef->properties()[propertyCount].property_name = property_name.str().c_str();
        processDef->properties()[propertyCount].mode_type = CosPropertyService::fixed_normal;

        //AUTORESTART_INFO
        propertyCount =  processDef->properties().length();
        processDef->properties().length( propertyCount +1 );
    
        CdmwPlatformMngt::AutoRestartInfo * my_auto_restart_info = new (CdmwPlatformMngt::AutoRestartInfo);
        my_auto_restart_info->max_nbr_of_restart_attempt = processParser.get_max_nbr_of_restart_attempt();
        my_auto_restart_info->auto_restart_control = processParser.get_auto_restart_control();

        processDef->properties()[propertyCount].initial_value <<= *my_auto_restart_info;
        property_name.str("");
        property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::AUTORESTART_INFO << std::ends;
        processDef->properties()[propertyCount].property_name = property_name.str().c_str();
        processDef->properties()[propertyCount].mode_type = CosPropertyService::fixed_normal;

        //FORCE_MANUAL_START
        propertyCount =  processDef->properties().length();
        processDef->properties().length( propertyCount +1 );

        std::string force_manual = processParser.get_force_manual_start();
        if (force_manual == "Manual")
        {
            processDef->properties()[propertyCount].initial_value <<= CORBA::Any::from_boolean(true);
        }
        else
        {
            processDef->properties()[propertyCount].initial_value <<= CORBA::Any::from_boolean(false);
        }
        property_name.str("");
        property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::FORCE_MANUAL_START << std::ends;
        processDef->properties()[propertyCount].property_name = property_name.str().c_str();
        processDef->properties()[propertyCount].mode_type = CosPropertyService::fixed_normal;

        // set relevant status indicator in processDef
        std::string relevant_indicator = processParser.get_relevant_indicator();
        if (relevant_indicator == "true")
        {
            processDef->indicator_relevant_status(true);
        }
        else
            processDef->indicator_relevant_status(false);


        // Set the managed process definition
        if( processDef->type() != CdmwPlatformMngt::UNMANAGED_PROCESS )
        {
            CdmwPlatformMngt::ManagedProcessDef* managedProcessDef
            = CdmwPlatformMngt::ManagedProcessDef::_downcast(processDef.in());

            //TRACE_FILTERS
            propertyCount =  managedProcessDef->properties().length();
            managedProcessDef->properties().length( propertyCount +1 );
            std::string trace_filters = processParser.filters();
            property_name.str("");
            property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::TRACE_FILTERS << std::ends;
            managedProcessDef->properties()[propertyCount].property_name = property_name.str().c_str();
            managedProcessDef->properties()[propertyCount].mode_type = CosPropertyService::fixed_normal;
            managedProcessDef->properties()[propertyCount].initial_value <<= trace_filters.c_str();
            
            // Add all the entity definitions to the process definition
            unsigned int entityCount = processParser.get_entity_count();
            managedProcessDef->entities().length( entityCount );
            for( unsigned int entityIndex=0;
             entityIndex<entityCount;
             entityIndex++ )
            {
            // Create an EntityDef
            CdmwPlatformMngt::EntityDef_var entityDef = Entity::get_definition(
                processParser.get_entity( entityIndex ) );
                       
            std::string newEntityName = entityDef->entity_name.in();

            // Checks there is no entity redefinition
            for( unsigned int l_entityIndex = 0;
                 l_entityIndex < entityIndex;
                 l_entityIndex++ )
            {
                std::string entityName
                = managedProcessDef->entities()[ l_entityIndex ].entity_name.in();
                    
                if( newEntityName == entityName )
                throw CdmwPlatformMngt::DuplicateProcessEntity(
                    managedProcessDef->name(),
                    managedProcessDef->host_name(),
                    newEntityName.c_str() );
            }

            managedProcessDef->entities()[ entityIndex ] =  entityDef.in();
            }


            if ( (processParser.type() != CdmwPlatformMngt::COMPONENT_SERVER) &&
           (processParser.get_ccmcomponent_count() > 0 ) )
             {

                 CDMW_ERROR(Cdmw::Logging::Logger::
                  Get_Logger(CdmwPlatformMngtBase::PFMMNG) , 
                  "Process hosting component is not of COMPONENT_SERVER type") ;
         }


        //Add the ccm component definition ...
            if( processParser.type() == CdmwPlatformMngt::COMPONENT_SERVER)
             {

              unsigned int ccmcomponentCount = processParser.get_ccmcomponent_count();
              managedProcessDef->components().length( ccmcomponentCount );

              for( unsigned int ccmcomponentIndex=0;
               ccmcomponentIndex<ccmcomponentCount;
               ccmcomponentIndex++ )
               {

               CdmwPlatformMngt::ComponentDef_var ccmcomponentDef = 
                CcmComponent::get_definition(
                    processParser.get_ccmcomponent( ccmcomponentIndex ) );

                std::string newCcmComponentName = ccmcomponentDef->name.in();
                std::string newDeploymentPlanName = ccmcomponentDef->deployment_plan.in();

                // Checks there is no ccmcomponent redefinition
                for( unsigned int l_ccmcomponentIndex = 0;
                 l_ccmcomponentIndex < ccmcomponentIndex;
                 l_ccmcomponentIndex++ )
                {
                 
                std::string ccmcomponentName 
                = managedProcessDef->components()[ l_ccmcomponentIndex ].name.in();

                /*print for check*/
         
                if( newCcmComponentName == ccmcomponentName )
                throw CdmwPlatformMngt::DuplicateComponent(
                    managedProcessDef->name(),
                    managedProcessDef->host_name(),
                    newCcmComponentName.c_str() );
            }

               managedProcessDef->components()[ ccmcomponentIndex ] =  ccmcomponentDef.in();

               }
           
              try
           {
               // Set the process component init process graph
               CdmwPlatformMngt::GraphElements_var rootInitGraphElements;
               ComponentGraph initComponentGraph( processParser, processParser.get_component_init_graph());
               rootInitGraphElements = initComponentGraph.get_roots();
           
           // get the defined componentGraph length
           unsigned int rootInitComponentGraphCount = rootInitGraphElements->length();

                   // set the defined componentGraph length into managedProcessDef
           managedProcessDef->component_init_graph().length(rootInitComponentGraphCount);
           
               for( unsigned int rootIndex=0; rootIndex< rootInitComponentGraphCount; rootIndex++)
           {
               managedProcessDef->component_init_graph()[rootIndex] = rootInitGraphElements[rootIndex];
           }
                                          
           }
           catch( const NotFoundException& )
           {
           }

           
          } 

            // Add all the service definitions to the process definition
            unsigned int serviceCount = processParser.get_service_count();
            managedProcessDef->service_definitions().length( serviceCount );
            for( unsigned int serviceIndex=0;
             serviceIndex<serviceCount;
             serviceIndex++ )
            {
            CdmwPlatformMngtService::ServiceDef_var serviceDef
                = Service::get_definition(
                processParser.get_service( serviceIndex ) );

            // Checks there is no service redefinition
            for( unsigned int l_serviceIndex = 0;
                 l_serviceIndex < serviceIndex;
                 l_serviceIndex++ )
            {
                std::string serviceName =
                managedProcessDef->service_definitions()[ l_serviceIndex ].service_name.in();

                if( serviceName == serviceDef->service_name.in() )
                throw CdmwPlatformMngt::DuplicateProcessService(
                    managedProcessDef->name(),
                    managedProcessDef->host_name(),
                    serviceDef->service_name.in() );
            }

            managedProcessDef->service_definitions()[ serviceIndex ]
                = serviceDef.in();
            }

            // Set the life cycle
            CdmwPlatformMngt::LifeCycleDef_var lifeCycleDef
            = new CdmwPlatformMngt::LifeCycleDef();
            {
            // Add all the init step
            unsigned int initStepCount
                = processParser.get_init_step_count();
            lifeCycleDef->init_steps.length( initStepCount );
            for( unsigned int initStepIndex=0;
                 initStepIndex<initStepCount;
                 initStepIndex++ )
            {
                // Get an init step
                CdmwPlatformMngt::Step_var initStep
                = new CdmwPlatformMngt::Step();
                        
                initStep->step_label = CORBA::string_dup(
                processParser.get_init_step_name(
                    initStepIndex ).c_str() );
                initStep->step_timeout
                = processParser.get_init_step_timeout( initStepIndex );

                // Checks there is no step redefinition
                for( unsigned int l_initStepIndex = 0;
                 l_initStepIndex < initStepIndex;
                 l_initStepIndex++ )
                {
                std::string initStepName
                    = lifeCycleDef->init_steps[ l_initStepIndex ].step_label.in();
                        
                if( initStepName == initStep->step_label.in() )
                    throw CdmwPlatformMngt::DuplicateProcessStep(
                    managedProcessDef->name(),
                    managedProcessDef->host_name(),
                    initStep->step_label.in() );
                }

                // Add this init step
                lifeCycleDef->init_steps[ initStepIndex ] = initStep.in();
            }

            // Set the startup and stop timeout
            lifeCycleDef->run_timeout
                = processParser.get_run_timeout();
            lifeCycleDef->stop_timeout
                = processParser.get_stop_timeout();
            }
            
            managedProcessDef->life_cycle( lifeCycleDef.in() );

            // Set the monitoring info
            CdmwPlatformMngt::CompleteMonitoringInfo_var monitoringInfo
            = new CdmwPlatformMngt::CompleteMonitoringInfo();
            
            try
            {
            // get the monitoring parser
            // if not found exception has been raised
            ProcessMonitoringParser monitor_parser = 
                processParser.get_monitoring();
                              
            monitoringInfo->monitoring_model = 
                monitor_parser.model();
            monitoringInfo->pull_monitoring_interval =
                monitor_parser.pull_interval();
            monitoringInfo->pull_monitoring_timeout =
                monitor_parser.pull_timeout();
            monitoringInfo->push_monitoring_interval =
                monitor_parser.push_interval();
            monitoringInfo->push_monitoring_timeout =
                monitor_parser.push_timeout();
            }
            catch (const NotFoundException&)
            {
            monitoringInfo->monitoring_model = CdmwPlatformMngt::NO_MONITORING;
            monitoringInfo->pull_monitoring_interval = 0;
            monitoringInfo->pull_monitoring_timeout  = 0;
            monitoringInfo->push_monitoring_interval = 0;
            monitoringInfo->push_monitoring_timeout  = 0;
            }

            managedProcessDef->monitoring_info( monitoringInfo.in() ); 

        }
           return processDef._retn();
        }
        catch( const std::bad_alloc& )
        {
        CDMW_THROW( OutOfMemoryException );
        }
        catch( const CdmwPlatformMngt::DuplicateProcessEntity& e )
        {
        PlatformMngt::LogMngr::logMessage(
            PlatformMngt::ERR,
            MSG_ID_ADMIN_DUPLICATE_PROCESS_ENTITY,
            e.entity_name.in(),
            e.process_name.in(),
            e.host_name.in() );

        throw LoggedException();
        }
        catch( const CdmwPlatformMngt::DuplicateProcessService& e )
        {
        PlatformMngt::LogMngr::logMessage(
            PlatformMngt::ERR,
            MSG_ID_ADMIN_DUPLICATE_PROCESS_SERVICE,
            e.service_name.in(),
            e.process_name.in(),
            e.host_name.in() );

        throw LoggedException();
        }
        catch( const CdmwPlatformMngt::DuplicateProcessStep& e )
        {
        PlatformMngt::LogMngr::logMessage(
            PlatformMngt::ERR,
            MSG_ID_ADMIN_DUPLICATE_PROCESS_STEP,
            e.step_label.in(),
            e.process_name.in(),
            e.host_name.in() );

        throw LoggedException();
        }
    }

    void Process::set_info(
        const char* process_exe,
        CdmwPlatformMngt::Port process_port,
        const char* working_directory,
        const char* process_args )
    {
        try
        {
        // Set the process info
        CdmwPlatformMngt::ProcessInfo_var processInfo
            = new CdmwPlatformMngt::ProcessInfo();

        processInfo->process_exe = CORBA::string_dup( process_exe );
        processInfo->process_port = process_port;
        processInfo->working_directory = CORBA::string_dup( working_directory);
        processInfo->process_args = CORBA::string_dup( process_args );

        if( !check_only )
            m_process->set_process_info( processInfo.in() );
        }
        catch( const CdmwPlatformMngt::IncompatibleStatus& )
        {
        PlatformMngt::LogMngr::logMessage(
            PlatformMngt::ERR,
            MSG_ID_ADMIN_INCOMPATIBLE_STATUS );

        throw LoggedException();
        }
    }

    void Process::set_autoending(bool autoending)
    {
        try
        {
        if( !check_only )
        {
            CORBA::Boolean flag;
            
            if (autoending)
            {
                flag = 1;
            }
            else
            {
            flag = 0;
            }
            
            m_process->set_auto_ending(flag);
        }
        }
        catch( const CdmwPlatformMngt::IncompatibleStatus& )
        {
        PlatformMngt::LogMngr::logMessage(
            PlatformMngt::ERR,
            MSG_ID_ADMIN_INCOMPATIBLE_STATUS );

        throw LoggedException();
        }
    }

    //CGi Force Manual Start removed from Process

    void Process::hot_restart( const CdmwPlatformMngtBase::StartupKind& startup_kind )
    {
        try
        {
            if( !check_only ) 
            {
                //std::cout << "Admin: Process Hot_restart" << std::endl;
                m_process->hot_restart( startup_kind );
            }
        }
        catch( const CdmwPlatformMngt::IncompatibleStatus& e )
        {
            PlatformMngt::LogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_INCOMPATIBLE_STATUS );

        throw LoggedException();
        }
    }

    void Process::cold_restart( const CdmwPlatformMngtBase::StartupKind& startup_kind )
    {
        try
        {
            if( !check_only )
            {
                //std::cout << "Admin: Process Cold_restart" << std::endl;
                m_process->cold_restart( startup_kind );
            }
        }
        catch( const CdmwPlatformMngt::IncompatibleStatus& e )
        {
        PlatformMngt::LogMngr::logMessage(
            PlatformMngt::ERR,
            MSG_ID_ADMIN_INCOMPATIBLE_STATUS );

        throw LoggedException();
        }
    }

    void Process::set_life_cycle( const char* lifeCycleFileName )
    {
        ConfigFileParser parser( lifeCycleFileName );
        LifeCycleParser lifeCycleParser( parser.get_life_cycle() );

        try
        {

        CdmwPlatformMngt::LifeCycleDef_var lifeCycleDef
            = new CdmwPlatformMngt::LifeCycleDef();

        // Add all the init step
        unsigned int initStepCount
            = lifeCycleParser.get_init_step_count();

        lifeCycleDef->init_steps.length( initStepCount );
        for( unsigned int initStepIndex=0;
             initStepIndex<initStepCount;
             initStepIndex++ )
        {
            // Get an init step
            CdmwPlatformMngt::Step_var initStep
            = new CdmwPlatformMngt::Step();
                
            initStep->step_label = CORBA::string_dup(
            lifeCycleParser.get_init_step_name(
                initStepIndex ).c_str() );
            initStep->step_timeout
            = lifeCycleParser.get_init_step_timeout( initStepIndex );

            // Checks there is no step redefinition
            for( unsigned int l_initStepIndex = 0;
             l_initStepIndex < initStepIndex;
             l_initStepIndex++ )
            {
            std::string initStepName
                = lifeCycleDef->init_steps[ l_initStepIndex ].step_label.in();
                
            if( initStepName == initStep->step_label.in() )
                throw CdmwPlatformMngt::DuplicateStep(
                initStep->step_label.in() );
            }

            // Add this init step
            lifeCycleDef->init_steps[ initStepIndex ] = initStep.in();
        }

        // Set the startup and stop timeout
        lifeCycleDef->run_timeout = lifeCycleParser.get_run_timeout();
        lifeCycleDef->stop_timeout = lifeCycleParser.get_stop_timeout();

        // Set the life cycle
        if( !check_only )
            m_process->set_life_cycle( lifeCycleDef.in() );
        }
        catch( const CdmwPlatformMngt::DuplicateStep& e )
        {
        PlatformMngt::LogMngr::logMessage(
            PlatformMngt::ERR,
            MSG_ID_ADMIN_DUPLICATE_STEP,
            e.step_label.in() );

        throw LoggedException();
        }
        catch( const CdmwPlatformMngt::IncompatibleType& )
        {
        PlatformMngt::LogMngr::logMessage(
            PlatformMngt::ERR,
            MSG_ID_ADMIN_INCOMPATIBLE_TYPE );

        throw LoggedException();
        }
        catch( const CdmwPlatformMngt::IncompatibleStatus& e )
        {
        PlatformMngt::LogMngr::logMessage(
            PlatformMngt::ERR,
            MSG_ID_ADMIN_INCOMPATIBLE_STATUS );

        throw LoggedException();
        }
    }

    void Process::add_property(const char* propertyConfigFileName) 
    {
        try
        {
        ConfigFileParser parser( propertyConfigFileName );
        PropertyParser propertyparser = parser.get_property();
        CosPropertyService::PropertyDef_var cos_prop = new CosPropertyService::PropertyDef;
        CosPropertyService::PropertyModeType prop_mode_type = CosPropertyService::PropertyModeType(0);
        CdmwPlatformMngt::PropertyDef_var prop = Property::get_definition(propertyparser);
        cos_prop->property_name = prop->property_name;
        cos_prop->property_value = prop->initial_value;
        if (prop->mode_type==1) prop_mode_type = CosPropertyService::PropertyModeType(1);
        cos_prop->property_mode = prop_mode_type;
        m_process -> define_property_with_mode(prop->property_name.in(),prop->initial_value, prop_mode_type);
        }
        catch( const CdmwPlatformMngt::IncompatibleStatus& e )
        {
        PlatformMngt::LogMngr::logMessage(
            PlatformMngt::ERR,
            MSG_ID_ADMIN_INCOMPATIBLE_STATUS );
        
        throw LoggedException();
        }
    }

    void Process::remove_property(const char* propertyName)
    {
        try
            {
                m_process -> delete_property (propertyName);
            }
            catch( const CdmwPlatformMngt::IncompatibleStatus& e )
            {
                PlatformMngt::LogMngr::logMessage(
                    PlatformMngt::ERR,
                    MSG_ID_ADMIN_INCOMPATIBLE_STATUS );

                throw LoggedException();
            }

    }
    
    void Process::get_property (const char* propertyName, const char* owner)
    {
    if( !check_only )
        {
            CORBA::Any_var prop_value = new CORBA::Any();

            try
            {
                m_process -> get_attribute (propertyName, prop_value.out() , ( ('P'== *owner) ? CdmwPlatformMngt::PRE_DEFINED : CdmwPlatformMngt::USER_DEFINED ));
                            
            }
            catch( const CdmwPlatformMngt::AttributeNotFound& e )
            {
                PlatformMngt::LogMngr::logMessage(
                        PlatformMngt::ERR,
                        "Attribute Not Found" );
    
                throw LoggedException();
            }

            CORBA::TypeCode_var tc;
            tc = prop_value->type();

            if (tc->equal(CdmwPlatformMngt::_tc_ProcessStatus) )
            {
                CdmwPlatformMngt::ProcessStatus value;
                *prop_value >>= value;
                std::string output("");
                switch( value )
                {
                    case (CdmwPlatformMngt::PROCESS_INITIAL) :
                    output = "PROCESS_INITIAL";
                    break;
                    case (CdmwPlatformMngt::PROCESS_INITIALISING) :
                    output = "PROCESS_INITIALISING";
                    break;
                    case (CdmwPlatformMngt::PROCESS_STEP_PERFORMED) :
                    output = "PROCESS_STEP_PERFORMED";
                    break;
                    case (CdmwPlatformMngt::PROCESS_INITIALISED) :
                    output = "PROCESS_INITIALISED";
                    break;
                    case (CdmwPlatformMngt::PROCESS_RUN_REQUEST) :
                    output = "PROCESS_RUN_REQUEST";
                    break;
                    case (CdmwPlatformMngt::PROCESS_RUNNING) :
                    output = "PROCESS_RUNNING";
                    break;
                    case (CdmwPlatformMngt::PROCESS_STOPPING) :
                    output = "PROCESS_STOPPING";
                    break;
                    case (CdmwPlatformMngt::PROCESS_STOPPED) :
                    output = "PROCESS_STOPPED";
                    break;
                    case (CdmwPlatformMngt::PROCESS_ENDED) :
                    output = "PROCESS_ENDED";
                    break;
                    case (CdmwPlatformMngt::PROCESS_FAILED_ERROR) :
                    output = "PROCESS_FAILED_ERROR";
                    break;
                    case (CdmwPlatformMngt::PROCESS_FAILED_TIMEOUT) :
                    output = "PROCESS_FAILED_TIMEOUT";
                    break;
                    case (CdmwPlatformMngt::PROCESS_FAILED_DISCREPANCY) :
                    output = "PROCESS_FAILED_DISCREPANCY";
                    break;
                    case (CdmwPlatformMngt::PROCESS_FAILED_INVALID) :
                    output = "PROCESS_FAILED_INVALID";
                    break;
                    case (CdmwPlatformMngt::PROCESS_FAILED_NO_RESPONSE) :
                    output = "PROCESS_FAILED_NO_RESPONSE";
                    break;
                    case (CdmwPlatformMngt::PROCESS_FAILED_DEATH) :
                    output = "PROCESS_FAILED_DEATH";
                    break;
                    }
                std::cout << propertyName << " : " << output << std::endl;
            }
            else
            {
                Property::print_standard_property_type( propertyName, prop_value.in() );
            }
        }//End checkonly
    }//End

   void Process::set_property (const char* propertyName, const char* owner, const char* propertyValue)
    {
        if( !check_only )
        {
            CORBA::Any_var any_var = new CORBA::Any();
            try
            {
                if ('P'== *owner)
                {
                    *any_var = Property::prepare_predefined_any (propertyName, propertyValue);
                }
                else // User defined property
                {
                    CORBA::Any_var prop_value = new CORBA::Any();
                    try
                    {
                        m_process -> get_attribute (propertyName, prop_value.out() ,
                                       ( ('P'== *owner) ? CdmwPlatformMngt::PRE_DEFINED : CdmwPlatformMngt::USER_DEFINED ));
                    }
                    catch( const CdmwPlatformMngt::AttributeNotFound& e )
                    {
                        PlatformMngt::LogMngr::logMessage(PlatformMngt::ERR, "Attribute Not Found" );
                        throw LoggedException();
                    }
                    CORBA::TypeCode_var tc = prop_value->type();
                    *any_var = Property::prepare_userdefined_any (tc.in(), propertyValue); 
                 }
                 
                 //finally set the attribute
                 m_process -> set_attribute (propertyName, any_var.in(),
                                           ( ('P'== *owner) ? CdmwPlatformMngt::PRE_DEFINED : CdmwPlatformMngt::USER_DEFINED));
                
            }
            catch( const CdmwPlatformMngt::AttributeNotFound& e )
            {
                PlatformMngt::LogMngr::logMessage(
                        PlatformMngt::ERR,
                        "Attribute Not Found" );
                throw LoggedException();
            }
            catch (const CdmwPlatformMngt::ReadOnlyAttribute& e)
            {
                PlatformMngt::LogMngr::logMessage(
                        PlatformMngt::ERR, 
                        "Read Only Property");
                throw LoggedException();                      
            }
        }//End checkonly
    }//End

       /* to set the trace filters associated to a managed process */
    /*
    void Process::set_trace_filters(const char* filters)
    {
        CORBA::Any_var myany = new CORBA::Any();
        myany <<= filters;
        m_process->set_attribute ( CdmwPlatformMngt::TRACE_FILTERS, myany.in(), CdmwPlatformMngt::PRE_DEFINED );
    }*/

       /* to get the trace filters associated to a managed process */
   /* void Process::get_trace_filters()
    {
        const char* trace_filters ;
        CORBA::Any_var myany = new CORBA::Any();
        m_process->get_attribute ( CdmwPlatformMngt::TRACE_FILTERS, myany.out(), CdmwPlatformMngt::PRE_DEFINED );
        myany >>= trace_filters;
        std::cout<<"TRACE FILTERS=\""<< trace_filters<<"\""<<std::endl;
    }*/

    } // End namespace PlatformAdmin
} // End namespace Cdmw

