/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "platformadminlibrary/Property.hpp"
#include "platformadminlibrary/PropertyParser.hpp"
#include "platformadminlibrary/ConfigFileParser.hpp"
#include "SystemMngt/tools/AdminLogMessageId.hpp"

#include "SystemMngt/platformlibrary/LogMngr.hpp"

extern bool check_only;

namespace Cdmw {         // Begin namespace Cdmw
namespace PlatformAdmin { // Begin namespace PlatformAdmin


/**
 * class Property 
 */
CdmwPlatformMngt::PropertyDef* Property::get_definition(
    const PropertyParser& propertyParser)
{
    try
    {
        CdmwPlatformMngt::PropertyDef_var propertyDef
            = new CdmwPlatformMngt::PropertyDef();

        // Get the property's name
    propertyDef->property_name = CORBA::string_dup(propertyParser.get_name().c_str());
    
    std::string tmp = CORBA::string_dup(propertyParser.get_mode().c_str());
        propertyDef->mode_type = CdmwPlatformMngt::PropertyModeType(0);
    if (tmp=="RO")  propertyDef->mode_type = CdmwPlatformMngt::PropertyModeType(1);

    std::string value_type = propertyParser.get_type();
    
    CORBA::Any any_value;
    
    
    if (value_type=="String") propertyDef->value_type = CdmwPlatformMngt::PropertyValueType(2);
    if (value_type=="Float") propertyDef->value_type = CdmwPlatformMngt::PropertyValueType(1);
    if (value_type=="Short") propertyDef->value_type = CdmwPlatformMngt::PropertyValueType(0);

    if (propertyDef->value_type==2)
            {
                    std::string tmp_str_value = propertyParser.get_value();
                any_value <<= tmp_str_value.c_str();
                propertyDef->initial_value=any_value;
            }
            
    if (propertyDef->value_type==1)
            {
                float tmp_float_value = (float)atof(propertyParser.get_value().c_str());
                any_value <<= tmp_float_value;
                propertyDef->initial_value=any_value;
            }
        
    if (propertyDef->value_type==0)
            {
                short tmp_short_value = (short)atoi(propertyParser.get_value().c_str());
                any_value <<= tmp_short_value;
                propertyDef->initial_value=any_value;
            }

    return propertyDef._retn();
    }
    catch( const std::bad_alloc& )
    {
        CDMW_THROW( OutOfMemoryException );
    }
}

CdmwPlatformMngt::PropertyDef* Property::get_definition(
        CosPropertyService::PropertyName name, 
        CdmwPlatformMngt::Owner owner, 
        CdmwPlatformMngt::PropertyModeType mode, 
        CdmwPlatformMngt::PropertyValueType value )
{
    //TODO:
    return NULL;
}  

CORBA::Any Property::prepare_predefined_any (const char* propertyName, const char* propertyValue)
{
            CORBA::Any any;
            std::string string_value(propertyValue);
            std::string string_name (propertyName);
          
            if ( (std::string(CdmwPlatformMngt::GLOBAL_SUPERVISION_HOST) == string_name) ||
                 (std::string(CdmwPlatformMngt::TRACE_FILTERS) == string_name ) )  
                {
                    any <<= propertyValue;
                }

            else if ( (std::string(CdmwPlatformMngt::FORCE_MANUAL_START) == string_name) ||
                      (std::string(CdmwPlatformMngt::GLOBAL_SUPERVISION) == string_name)  )
            {
                if ( ("True" != string_value ) && ("False" != string_value ) )
                {
                 PlatformMngt::LogMngr::logMessage(PlatformMngt::ERR,
                                                 "Syntax error - Boolean value is not valid - Correct values are  True|False" );
                    throw CORBA::BAD_PARAM();   
                }
                any <<= CORBA::Any::from_boolean(("True"== string_value)? true : false);
            }
            else if (std::string(CdmwPlatformMngt::GROUP_SIZE) == string_name )
            {
            CORBA::ULong size = 0;
            any <<= size;
            }
                        
            else if ( std::string(CdmwPlatformMngt::AUTORESTART_INFO) == string_name )
            {
                        CdmwPlatformMngt::AutoRestartInfo * autorestart_info = new (CdmwPlatformMngt::AutoRestartInfo);
                        std::string restart_attempt = "";
                        std::string auto_restart_control = "";
    
                        std::string::size_type pos_sep1;
                        std::string::size_type key_size = string_value.size();
    
                        pos_sep1 = string_value.find(":");
    
                        // check if max_nbr_of_restart_attempt found
                        if (pos_sep1 != std::string::npos)
                        {
                            if (pos_sep1 >= 1)
                            {
                                restart_attempt = string_value.substr(0, pos_sep1);
                            }
                        pos_sep1 += 1;
                        }
                        else
                        { 
                            // max_nbr_of_restart_attempt is not valid
                            PlatformMngt::LogMngr::logMessage(PlatformMngt::ERR,"Syntax error - max_nbr_of_restart_attempt is not valid" );
                            throw CORBA::BAD_PARAM();
                        }
                        // check if auto_restart_control found
                        if (pos_sep1 < key_size)
                        {
                            auto_restart_control = string_value.substr(pos_sep1);
                        }
                        else
                        {
                            // auto_restart_control is not valid
                            PlatformMngt::LogMngr::logMessage(PlatformMngt::ERR,"Syntax error - auto_restart_control is not present - Correct values are  USE_COUNTER|IGNORE_COUNTER|IGNORE_ACTION" );
                            throw CORBA::BAD_PARAM();
                        }
                        if (atoi(restart_attempt.c_str()) != 0 || restart_attempt == "0")
                        {
                            autorestart_info->max_nbr_of_restart_attempt = atoi(restart_attempt.c_str());
                        }
                        else 
                        {
                            // max_nbr_of_restart_attempt is not valid
                            PlatformMngt::LogMngr::logMessage(PlatformMngt::ERR,"Syntax error - max_nbr_of_restart_attempt is not valid" );
                            throw CORBA::BAD_PARAM();
                        }
                        CdmwPlatformMngt::AutoRestartControl auto_restart_control_enum;
                        if (auto_restart_control == "USE_COUNTER")
                            auto_restart_control_enum = CdmwPlatformMngt::USE_COUNTER;
                        else if (auto_restart_control == "IGNORE_COUNTER")
                            auto_restart_control_enum = CdmwPlatformMngt::IGNORE_COUNTER;
                        else if (auto_restart_control == "IGNORE_ACTION")
                            auto_restart_control_enum = CdmwPlatformMngt::IGNORE_ACTION;
                        else 
                        {
                            // auto_restart_control is not valid
                            PlatformMngt::LogMngr::logMessage(PlatformMngt::ERR,
                                        "Syntax error - auto_restart_control is not valid - Correct values are  USE_COUNTER|IGNORE_COUNTER|IGNORE_ACTION" );
                            throw CORBA::BAD_PARAM();
                        }
                        autorestart_info->auto_restart_control = auto_restart_control_enum;
                        any <<= *autorestart_info;
            }
            else if ( (std::string(CdmwPlatformMngt::DEFAULT_RESTART_WARMTH) == string_name) ||
                       // attribute is readonly , throws exception 
                      (std::string(CdmwPlatformMngt::REQUESTED_RESTART_WARMTH) == string_name) )
            {
                if ( ("Hot" != string_value ) && ("Cold" != string_value ) )
                {
                    PlatformMngt::LogMngr::logMessage(PlatformMngt::ERR,
                                        "Syntax error - restart_warmth is not valid - Correct values are  Hot|Cold" );
                    throw CORBA::BAD_PARAM();   
                }
                any <<= ( ("Hot"== string_value)? CdmwPlatformMngt::HOT_RESTART : CdmwPlatformMngt::COLD_RESTART );
            }
                    
            else if ( CdmwPlatformMngt::REPLICATION_ROLE == string_name )
            {
            // attribute is readonly , throws exception
            CdmwPlatformMngt::RedundancyEltState elt_state = CdmwPlatformMngt::REDSTATE_NORG; 
            any <<= elt_state;
            }
            else
            {
                        PlatformMngt::LogMngr::logMessage( PlatformMngt::ERR, "Attribute Not Found" );
                        throw LoggedException();
            }
                
    
     return any;                
}      

CORBA::Any Property::prepare_userdefined_any (const CORBA::TypeCode* tc, const char* propertyValue)
{

                  CORBA::Any any;
                  switch (tc->kind())
                  {
                        case (CORBA::tk_string):
                        any <<= propertyValue;
                        break;
                        
                        case (CORBA::tk_float):
                        CORBA::Float fvalue = atof (propertyValue);
                        any <<= fvalue;
                        break;
                        
                        case (CORBA::tk_short):
                        CORBA::Short svalue = atoi (propertyValue);
                        any <<= svalue;
                        break;
                        
                        default:
                        std::cout << propertyValue << " : Unknowm type" << std::endl;
                        throw LoggedException();
                }
   return any;
                    
}

void Property::print_standard_property_type(const char* propertyName, const CORBA::Any& prop_value)
{
            CORBA::TypeCode_var tc;
            tc = prop_value.type();

            if (tc->equal(CdmwPlatformMngt::_tc_RestartWarmth) )
            {
                CdmwPlatformMngt::RestartWarmth value;
                prop_value >>= value;
                std::cout << propertyName << " : " << ( (CdmwPlatformMngt::HOT_RESTART == value)?"Hot":"Cold" ) <<std::endl;
            } 
            else if (tc->equal(CdmwPlatformMngt::_tc_FunctioningMode) )
            {
                CdmwPlatformMngt::FunctioningMode value;
                prop_value >>= value;
                std::string output("");
                switch (value)
                {
                case (CdmwPlatformMngt::NORMAL_MODE):
                output="NORMAL_MODE";
                break;
                case (CdmwPlatformMngt::DEGRADED_MODE):
                output="DEGRADED_MODE";
                break;
                }
            std::cout << propertyName << " : " << output << std::endl;
            }
            else if (tc->equal(CdmwPlatformMngt::_tc_AutoRestartInfo) )
            {
                CdmwPlatformMngt::AutoRestartInfo *value = new (CdmwPlatformMngt::AutoRestartInfo);
                prop_value >>= value;
                std::string control("");
                switch (value->auto_restart_control)
                {
                case (CdmwPlatformMngt::IGNORE_COUNTER):
                control="IGNORE_COUNTER";
                break;
                case (CdmwPlatformMngt::IGNORE_ACTION):
                control="IGNORE_ACTION";
                break;
                default:
                control="USE_COUNTER";
                break;
                }
                std::cout << propertyName << " : " << ( value->max_nbr_of_restart_attempt)<<":"<<control.c_str()<< std::endl;
            }
            else if (tc->equal(CdmwPlatformMngt::_tc_RedundancyEltState) )
            {
                CdmwPlatformMngt::RedundancyEltState value;
                prop_value >>= value;
                std::string output("");
                switch (value)
                {
                    case (CdmwPlatformMngt::REDSTATE_NORG):
                    output="REDSTATE_NORG";
                    break;
                            case (CdmwPlatformMngt::REDSTATE_PRIMARY):
                    output="REDSTATE_PRIMARY";
                    break;
                    case (CdmwPlatformMngt::REDSTATE_PASSIVE):
                    output="REDSTATE_PASSIVE";
                    break;
                    case (CdmwPlatformMngt::REDSTATE_ACTIVE):
                        output="REDSTATE_ACTIVE";   
                  break;

                }
                std::cout << propertyName << " : " << output << std::endl;
            }

            else 
            {
                switch (tc->kind())
                {
                    case (CORBA::tk_string):
                    {
                    const char* value;
                    prop_value >>= value ;
                    std::cout << propertyName << " : " << value << std::endl;
                    }
                    break;

                    case (CORBA::tk_float):
                    {
                    CORBA::Float value;
                    prop_value >>= value;
                    std::cout << propertyName << " : " << value << std::endl;
                    }
                    break;
            
                    case (CORBA::tk_short):
                    {
                    short value;
                    prop_value >>= value;
                    std::cout << propertyName << " : " << value << std::endl;
                    }
                    break;
            
                    case (CORBA::tk_boolean):
                    {
                    CORBA::Boolean value;
                    prop_value >>= CORBA::Any::to_boolean(value);
                    std::cout << propertyName << " : " << ((value)?"True":"False") << std::endl;
                    }
                    break;
            
                    case (CORBA::tk_ulong):
                    {
                    CORBA::ULong value;
                    prop_value >>= value;
                    std::cout << propertyName << " : " << value << std::endl;
                    }
                    break;
                    
                    case (CORBA::tk_long):
                    {
                    CORBA::Long value;
                    prop_value >>= value;
                    std::cout << propertyName << " : " << value << std::endl;
                    }
                    break;

            
                    default:
                    std::cout << propertyName << " : Unknowm type" << std::endl;
                    break;
               }
        }
}

} // End namespace PlatformAdmin
} // End namespace Cdmw

