/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/orbsupport/OrbSupport.hpp"

#include "SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"

#include "platformadminlibrary/AdminLogMngr.hpp"
#include "platformadminlibrary/ConfigFileParser.hpp"
#include "platformadminlibrary/SystemParser.hpp"
#include "platformadminlibrary/HostParser.hpp"
#include "platformadminlibrary/ApplicationParser.hpp"
#include "platformadminlibrary/ServiceParser.hpp"
#include "platformadminlibrary/RunSetParser.hpp"
#include "platformadminlibrary/GraphParser.hpp"
#include "platformadminlibrary/Host.hpp"
#include "platformadminlibrary/Application.hpp"
#include "platformadminlibrary/Service.hpp"
#include "platformadminlibrary/RunSet.hpp"
#include "platformadminlibrary/Graph.hpp"
#include "platformadminlibrary/System.hpp"
#include "SystemMngt/tools/AdminLogMessageId.hpp"
#include "platformadminlibrary/Property.hpp"
#include "platformadminlibrary/PropertyParser.hpp"
#include <map>
#include <sstream>
extern bool check_only;


namespace Cdmw {         // Begin namespace Cdmw
namespace PlatformAdmin { // Begin namespace PlatformAdmin

ent_types Entity::entity_types;
ent_type_properties Entity::entity_type_properties;

System::System( CdmwPlatformMngt::System_ptr system )
: ServiceContainer( system ),
EntityContainer( system )

{
    if( !check_only )
        m_system = CdmwPlatformMngt::System::_duplicate( system );
}

System::~System()
{
}

CdmwPlatformMngt::SystemSnapshot* System::get_snapshot()
{
    CdmwPlatformMngt::SystemSnapshot_var snapshot;

    if( !check_only )
        snapshot = m_system->get_snapshot();

        return snapshot._retn();
}

void System::start(
        const CdmwPlatformMngtBase::StartupKind& startup_kind )
{
    try
    {
        if( !check_only )
            m_system->start( startup_kind );
    }
    catch( const CdmwPlatformMngt::IncompatibleStatus& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_INCOMPATIBLE_STATUS );

        throw LoggedException();
    }
}

//This method substitutes stop() cause of command  
void System::stop( bool emergency )
{

    if( !check_only )
    {
        try
        {
            m_system->stop_system( emergency );
        }
        catch( ::CORBA::SystemException& ex ) 
        {
            throw LoggedException();
        }
        catch( ... ) 
        {
            PlatformMngt::LogMngr::logMessage(
                    PlatformMngt::ERR,
                    MSG_ID_ADMIN_UNKNOWN_ERROR );
            throw LoggedException();
        }
    }            
}

void System::resume()
{
    try
    {
        if( !check_only )
            m_system->resume();
    }
    catch( const CdmwPlatformMngt::IncompatibleStatus& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_INCOMPATIBLE_STATUS );

        throw LoggedException();
    }
}

void System::acknowledge()
{
    try
    {
        if( !check_only )
            m_system->acknowledge();
    }
    catch( const CdmwPlatformMngt::IncompatibleStatus& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_INCOMPATIBLE_STATUS );

        throw LoggedException();
    }
}

CdmwPlatformMngt::SystemDef* System::get_definition(
        const SystemParser& systemParser )
{
    try
    {
    // Get a system definition
        CdmwPlatformMngt::SystemDef_var systemDef
        = new CdmwPlatformMngt::SystemDef();
        systemDef->system_name
        = CORBA::string_dup( systemParser.name().c_str() );
        systemDef->system_port = systemParser.port();
        // Add all the entity definitions to the system definition
        unsigned int entityCount = systemParser.get_entity_count();
        systemDef->entities.length( entityCount );
        //num. of entity types ...
        unsigned int entityTypesCount = systemParser.get_entity_type_count();
        for(  unsigned int entityTypeIndex=0; entityTypeIndex<entityTypesCount ; entityTypeIndex++ )
        {
            //get the entity type parser ...
            EntityTypeParser entityTypeParser = systemParser.get_entity_type(entityTypeIndex);
            //store all the entries of Entity-type in
            //a suitable map ...

            std::string entity_type_name = entityTypeParser.get_entity_type_name() ;
            std::string status_values_for_type = entityTypeParser.get_status_values_for_type();

            if(status_values_for_type==""){
            //Entitytype status values is empty
                AdminLogMngr::logMessage( PlatformMngt::ERR, MSG_ID_TYPE_ENTITY_EMPTY, entityTypeParser.get_entity_type_name());
                throw LoggedException();
                exit(-1);
        }
            //Insert a new element to map of Entity Types ...
            Entity::entity_types[entity_type_name] = status_values_for_type;
        
        /*Find all the properties of the EntityType and add them into vector entity_type_properties, then  
        i add the vector to the static map typedef std::map<std::string, PropList> ent_type_properties of Entity */
        unsigned int entityTypesPropertiesCount = entityTypeParser.get_property_count();  ;
        for(  unsigned int entityTypeProperties=0; entityTypeProperties<entityTypesPropertiesCount ; entityTypeProperties++ )
            {
            PropertyParser PropParsElement =  entityTypeParser.get_property(entityTypeProperties);
        (entityTypeParser.entity_type_properties).push_back(PropParsElement);
        }
            Entity::entity_type_properties[entity_type_name] = entityTypeParser.entity_type_properties;
         
        }       


        //print all the filled map entity_types for a test ...
        /*
                ent_types::const_iterator it  = Entity::entity_types.begin();
                ent_types::const_iterator end = Entity::entity_types.end(); 

                for(;it!=end;++it)
                    {       
                      std::cout<<"**** entity_types["<< it->first<<"]="<<it->second<<std::endl;
                    }
         */


        for( unsigned int entityIndex=0; entityIndex<entityCount; entityIndex++ )
        {
            // Create an EntityDef
            CdmwPlatformMngt::EntityDef_var entityDef = Entity::get_definition(
                    systemParser.get_entity( entityIndex ) );
            std::string newEntityName = entityDef->entity_name.in();
            // Checks there is no entity redefinition
            for( unsigned int l_entityIndex = 0;
            l_entityIndex < entityIndex;
            l_entityIndex++ )
            {
                std::string entityName
                = systemDef->entities[ l_entityIndex ].entity_name.in();

                if( newEntityName == entityName )
                    throw CdmwPlatformMngt::DuplicateEntity(
                            newEntityName.c_str() );
            }
            systemDef->entities[ entityIndex ] =  entityDef.in();
        }

        // Add all the service definitions to the system definition
        unsigned int serviceCount = systemParser.get_service_count();

        systemDef->service_definitions.length( serviceCount );
        for( unsigned int serviceIndex=0;
        serviceIndex<serviceCount;
        serviceIndex++ )
        {
            CdmwPlatformMngtService::ServiceDef_var serviceDef
            = Service::get_definition(
                    systemParser.get_service( serviceIndex ) );

            // Checks there is no service redefinition
            std::string newServiceName = serviceDef->service_name.in();

            for( unsigned int l_serviceIndex = 0;
            l_serviceIndex < serviceIndex;
            l_serviceIndex++ )
            {
                std::string serviceName =
                    systemDef->service_definitions[ l_serviceIndex ].service_name.in();

                    if( newServiceName == serviceName )
                        throw CdmwPlatformMngt::DuplicateService(
                                newServiceName.c_str() );
            }

            systemDef->service_definitions[ serviceIndex ] = serviceDef.in();
        }

        // Add all the host definitions to the system definition
        unsigned int hostCount = systemParser.get_host_count();

        systemDef->hosts.length( hostCount );
        for( unsigned int hostIndex=0; hostIndex<hostCount; hostIndex++ )
        {
            CdmwPlatformMngt::HostDef_var hostDef = Host::get_definition(
                    systemParser.get_host( hostIndex ) );

            // Checks there is no host redefinition
            std::string newHostName = hostDef->host_name.in();

            for( unsigned int l_hostIndex = 0;
            l_hostIndex < hostIndex;
            l_hostIndex++ )
            {
                std::string hostName =
                    systemDef->hosts[ l_hostIndex ].host_name.in();

                    if( newHostName == hostName )
                        throw CdmwPlatformMngt::DuplicateHost(
                                newHostName.c_str() );
            }

            systemDef->hosts[ hostIndex ] = hostDef.in();
        }

        unsigned int propertyCount = systemParser.get_property_count();
        systemDef->properties.length( propertyCount );

        for( unsigned int propIndex=0; propIndex<propertyCount; propIndex++)
        {
            CdmwPlatformMngt::PropertyDef_var prop = Property::get_definition(systemParser.get_property(propIndex));
            systemDef->properties[propIndex] = prop;        
        }

        //PRE_DEFINED properties
        std::ostringstream property_name;

        //DEFAULT_RESTART_WARMTH
        propertyCount =  systemDef->properties.length();
        systemDef->properties.length( propertyCount +1 );

        std::string default_warmth = systemParser.get_default_restart_warmth();
        if (default_warmth == "Cold")
        {
            systemDef->properties[propertyCount].initial_value <<= CdmwPlatformMngt::COLD_RESTART;
        }
        else
        {
            systemDef->properties[propertyCount].initial_value <<= CdmwPlatformMngt::HOT_RESTART;
        }

        property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::DEFAULT_RESTART_WARMTH << std::ends;
        systemDef->properties[propertyCount].property_name = property_name.str().c_str();
        systemDef->properties[propertyCount].mode_type = CosPropertyService::fixed_normal;

        return systemDef._retn();
    }
    catch( const std::bad_alloc& )
    {
        CDMW_THROW( OutOfMemoryException );
    }
    catch( const CdmwPlatformMngt::DuplicateEntity& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_DUPLICATE_ENTITY,
                e.entity_name.in() );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::DuplicateService& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_DUPLICATE_SERVICE,
                e.service_name.in() );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::DuplicateHost& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_DUPLICATE_HOST,
                e.host_name.in() );

        throw LoggedException();
    }
}

void System::define( const char* systemConfigFileName )
{
    // Open the system configuration file
    ConfigFileParser  parser( systemConfigFileName );
    SystemParser      systemParser = parser.get_system();
    
    try
    {
        // define the system onto the platform supervision
        CdmwPlatformMngt::SystemDef_var systemDef = System::get_definition( systemParser );
        if( !check_only ) {
            m_system->define( systemDef.in() );
        }

        // Add all the applications to the system
        unsigned int appCount = systemParser.get_application_count();
        for( unsigned int appIndex=0; appIndex<appCount; appIndex++ )
        {
            // Create and add each application to the system onto the supervision
            ApplicationParser appParser = systemParser.get_application( appIndex );
            CdmwPlatformMngt::PlatformApplicationDef_var platformApplicationDef;
            CdmwPlatformMngt::ApplicationDef_var applicationDef
            = Application::get_definition(
                    appParser,
                    platformApplicationDef.out() );
            // Checks there is no application redefinition
            for( unsigned int l_appIndex = 0;
            l_appIndex < appIndex;
            l_appIndex++ )
            {
                std::string applicationName
                = systemParser.get_application( l_appIndex ).name();

                if( applicationName == platformApplicationDef->application_name.in() )
                {
                     PlatformMngt::LogMngr::logMessage(
                            PlatformMngt::ERR,
                            MSG_ID_ADMIN_DUPLICATE_APPLICATION,
                            platformApplicationDef->application_name.in() );

                    throw LoggedException();
                }
            }
            
            // Actualy add the application to the system
            CdmwPlatformMngt::Application_var application;
            try
            {
                if( !check_only ) {
                    application = m_system->add_application(
                            platformApplicationDef.in(),
                            applicationDef.in() );
                }
            }
            catch( const CdmwPlatformMngt::ApplicationAlreadyExists& e )
            {
                AdminLogMngr::logMessage(
                        PlatformMngt::ERR,
                        MSG_ID_ADMIN_APPLICATION_ALREADY_EXISTS,
                        platformApplicationDef->application_name.in() );

                throw LoggedException();
            }

            try
            {
                // Set each application's init process graph
                ProcessGraph initProcessGraph(
                        appParser,
                        appParser.get_init_graph(),
                        false );
                CdmwPlatformMngt::GraphElements_var rootGraphElements
                = initProcessGraph.get_roots();

                if( !check_only ) {
                    application->set_init_graph( rootGraphElements.in() );
                }
            }
            catch( const NotFoundException& )
            {
            }

            try
            {
                // Set each application's stop process graph
                ProcessGraph stopProcessGraph(
                        appParser,
                        appParser.get_stop_graph(),
                        true );
                CdmwPlatformMngt::GraphElements_var rootGraphElements
                = stopProcessGraph.get_roots();

                if( !check_only ) {
                    application->set_stop_graph( rootGraphElements.in() );
                }
            }
            catch( const NotFoundException& )
            {
            }
        }

        // Set the application run sets
        unsigned int runSetCount = systemParser.get_run_set_count();
        for( unsigned int runSetIndex = 0;
        runSetIndex < runSetCount;
        runSetIndex++ )
        {
            RunSetParser runSetParser = systemParser.get_run_set( runSetIndex );
            CdmwPlatformMngt::RunSetDef_var runSetDef
            = RunSet::get_definition( runSetParser );

            // Checks there is no run set redefinition
            for( unsigned int l_runSetIndex = 0;
            l_runSetIndex < runSetIndex;
            l_runSetIndex++ )
            {
                std::string runSetName
                = systemParser.get_run_set( l_runSetIndex ).name();

                if( runSetName == runSetDef->set_name.in() )
                {
                    AdminLogMngr::logMessage(
                            PlatformMngt::ERR,
                            MSG_ID_ADMIN_DUPLICATE_RUNSET,
                            runSetDef->set_name.in() );

                    throw LoggedException();
                }
            }
            if( !check_only ) {
                m_system->set_run_set( runSetDef.in() );
            }
        }
        
        try
        {
            // Set the system's application init graph onto the platform supervision
            CdmwPlatformMngt::GraphElements_var rootGraphElements;
            ApplicationGraph initApplicationGraph(
                    systemParser,
                    systemParser.get_init_graph() );
            rootGraphElements = initApplicationGraph.get_roots();

            if( !check_only ) {
                m_system->set_init_graph( rootGraphElements.in() );
            }
        }
        catch( const NotFoundException& )
        {
        }

        try
        {
            // Set the system's application stop graph onto the platform supervision
            ApplicationGraph stopApplicationGraph(
                    systemParser,
                    systemParser.get_stop_graph() );
            CdmwPlatformMngt::GraphElements_var rootGraphElements
            = stopApplicationGraph.get_roots();

            if( !check_only ) {
                m_system->set_stop_graph( rootGraphElements.in() );
            }
        }
        catch( const NotFoundException& )
        {
        }
    }
    catch( const CdmwPlatformMngt::HostNotFound& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_HOST_NOT_FOUND );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::DuplicateEntity& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_DUPLICATE_ENTITY,
                e.entity_name.in() );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::DuplicateService& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_DUPLICATE_SERVICE,
                e.service_name.in() );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::DuplicateProcess& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_DUPLICATE_PROCESS,
                e.process_name.in(),
                e.host_name.in() );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::ProcessHostNotFound& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_PROCESS_HOST_NOT_FOUND,
                e.host_name.in(),
                e.process_name.in() );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::DuplicateProcessEntity& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_DUPLICATE_PROCESS_ENTITY,
                e.entity_name.in(),
                e.process_name.in(),
                e.host_name.in() );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::DuplicateProcessService& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_DUPLICATE_PROCESS_SERVICE,
                e.service_name.in(),
                e.process_name.in(),
                e.host_name.in() );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::DuplicateProcessStep& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_DUPLICATE_PROCESS_STEP,
                e.step_label.in(),
                e.process_name.in(),
                e.host_name.in() );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::ReferenceNotFound& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_REFERENCE_NOT_FOUND,
                e.name.in() );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::CircularReference& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_CIRCULAR_REFERENCE,
                e.name.in(),
                e.step );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::StepOutOfBound& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_STEP_OUT_OF_BOUND,
                e.step,
                e.name.in() );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::StepJump& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_STEP_JUMP,
                e.name.in(),
                e.origin_step,
                e.target_step );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::IncompatibleStatus& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_INCOMPATIBLE_STATUS );

        throw LoggedException();
    }
}

void System::add_host( const char* hostConfigFileName )
{
    // Open the host configuration file
    ConfigFileParser parser( hostConfigFileName );

    // Get a host definition
    CdmwPlatformMngt::HostDef_var hostDef
    = Host::get_definition( parser.get_host() );

    try
    {
        if( !check_only )
            m_system->add_host( hostDef.in() );
    }
    catch( const CdmwPlatformMngt::HostAlreadyExists& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_HOST_ALREADY_EXISTS,
                hostDef->host_name.in() );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::IncompatibleStatus& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_INCOMPATIBLE_STATUS );

        throw LoggedException();
    }
}

void System::remove_host( const char* hostConfigFileName )
{
    try
    {
        if( !check_only )
            m_system->remove_host( hostConfigFileName );
    }
    catch( const CdmwPlatformMngt::HostNotFound& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_HOST_NOT_FOUND,
                hostConfigFileName );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::IncompatibleStatus& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_INCOMPATIBLE_STATUS );

        throw LoggedException();
    }
}

void System::add_application( const char* applicationConfigFileName )
{
    // Open the application's configuration file
    ConfigFileParser  parser( applicationConfigFileName );
    ApplicationParser appParser = parser.get_application();

    // Add the application's definition
    CdmwPlatformMngt::PlatformApplicationDef_var platformApplicationDef;
    CdmwPlatformMngt::ApplicationDef_var applicationDef
    = Application::get_definition(
            appParser,
            platformApplicationDef.out() );
    CdmwPlatformMngt::Application_var application;

    try
    {
        if( !check_only )
            application = m_system->add_application(
                    platformApplicationDef.in(),
                    applicationDef.in() );

            try
            {
                // Set the application's init process graph
                CdmwPlatformMngt::GraphElements_var rootGraphElements;
                ProcessGraph initProcessGraph(
                        appParser,
                        appParser.get_init_graph(),
                        false );
                rootGraphElements = initProcessGraph.get_roots();

                if( !check_only )
                    application->set_init_graph( rootGraphElements.in() );
            }
            catch( const NotFoundException& )
            {
            }

            try
            {
                // Set the application's stop process graph
                ProcessGraph stopProcessGraph(
                        appParser,
                        appParser.get_stop_graph(),
                        true );
                CdmwPlatformMngt::GraphElements_var rootGraphElements
                = stopProcessGraph.get_roots();

                if( !check_only )
                    application->set_stop_graph( rootGraphElements.in() );
            }
            catch( const NotFoundException& )
            {
            }
    }
    catch( const CdmwPlatformMngt::HostNotFound& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_HOST_NOT_FOUND );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::DuplicateEntity& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_DUPLICATE_ENTITY,
                e.entity_name.in() );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::DuplicateService& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_DUPLICATE_SERVICE,
                e.service_name.in() );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::DuplicateProcess& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_DUPLICATE_PROCESS,
                e.process_name.in(),
                e.host_name.in() );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::ProcessHostNotFound& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_PROCESS_HOST_NOT_FOUND,
                e.host_name.in(),
                e.process_name.in() );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::DuplicateProcessEntity& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_DUPLICATE_PROCESS_ENTITY,
                e.process_name.in(),
                e.entity_name.in(),
                e.host_name.in() );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::DuplicateProcessService& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_DUPLICATE_PROCESS_SERVICE,
                e.service_name.in(),
                e.process_name.in(),
                e.host_name.in() );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::DuplicateProcessStep& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_DUPLICATE_PROCESS_STEP,
                e.step_label.in(),
                e.process_name.in(),
                e.host_name.in() );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::ReferenceNotFound& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_REFERENCE_NOT_FOUND,
                e.name.in() );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::CircularReference& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_CIRCULAR_REFERENCE,
                e.name.in(),
                e.step );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::StepOutOfBound& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_STEP_OUT_OF_BOUND,
                e.step,
                e.name.in() );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::StepJump& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_STEP_JUMP,
                e.name.in(),
                e.origin_step,
                e.target_step );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::IncompatibleStatus& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_INCOMPATIBLE_STATUS );

        throw LoggedException();
    }
}

void System::remove_application( const char* applicationName )
{
    try
    {
        if( !check_only )
            m_system->remove_application( applicationName );
    }
    catch( const CdmwPlatformMngt::ApplicationNotFound& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_APPLICATION_NOT_FOUND,
                applicationName );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::ApplicationStillActive& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_APPLICATION_STILL_ACTIVE,
                applicationName );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::IncompatibleStatus& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_INCOMPATIBLE_STATUS );

        throw LoggedException();
    }
}

void System::set_init_graph( const char* graphConfigFileName )
{
    // Open the graph configuration file
    ConfigFileParser parser( graphConfigFileName );

    try
    {
        // Set the init graph sequence to the system onto the platform supervision
        ApplicationGraph applicationGraph( parser.get_init_graph() );
        CdmwPlatformMngt::GraphElements_var rootGraphElements
        = applicationGraph.get_roots();

        if( !check_only )
            m_system->set_init_graph( rootGraphElements.in() );
    }
    catch( const CdmwPlatformMngt::ReferenceNotFound& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_REFERENCE_NOT_FOUND,
                e.name.in() );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::CircularReference& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_CIRCULAR_REFERENCE,
                e.name.in(),
                e.step );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::IncompatibleStatus& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_INCOMPATIBLE_STATUS );

        throw LoggedException();
    }
}

void System::set_stop_graph( const char* graphConfigFileName )
{
    // Open the graph configuration file
    ConfigFileParser parser( graphConfigFileName );

    try
    {
        // Set the stop graph sequence to the system onto the platform supervision
        ApplicationGraph applicationGraph( parser.get_stop_graph() );
        CdmwPlatformMngt::GraphElements_var rootGraphElements
        = applicationGraph.get_roots();

        if( !check_only )
            m_system->set_stop_graph( rootGraphElements.in() );
    }
    catch( const CdmwPlatformMngt::ReferenceNotFound& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_REFERENCE_NOT_FOUND,
                e.name.in() );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::CircularReference& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_CIRCULAR_REFERENCE,
                e.name.in(),
                e.step );

        throw LoggedException();
    }
    catch( const CdmwPlatformMngt::IncompatibleStatus& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_INCOMPATIBLE_STATUS );

        throw LoggedException();
    }
}

void System::add_property(const char* propertyConfigFileName) 
{
    try
    {
        ConfigFileParser parser( propertyConfigFileName );
        PropertyParser propertyparser = parser.get_property();
        CdmwPlatformMngt::PropertyDefs_var propertydefs ;
        CosPropertyService::PropertyDef_var cos_prop = new CosPropertyService::PropertyDef;
        CosPropertyService::PropertyModeType prop_mode_type = CosPropertyService::PropertyModeType(0);
        CdmwPlatformMngt::PropertyDef_var prop = Property::get_definition(propertyparser);
        cos_prop->property_name = prop->property_name;
        cos_prop->property_value = prop->initial_value;
        if (prop->mode_type==1) prop_mode_type = CosPropertyService::PropertyModeType(1);
        cos_prop->property_mode = prop_mode_type;
        m_system -> define_property_with_mode(prop->property_name.in(),prop->initial_value, prop_mode_type);
    }
    catch( const CdmwPlatformMngt::IncompatibleStatus& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_INCOMPATIBLE_STATUS );

        throw LoggedException();
    }
}

void System::remove_property(const char* propertyName)
{
    try
    {
        m_system -> delete_property (propertyName); 
    }
    catch( const CdmwPlatformMngt::IncompatibleStatus& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_INCOMPATIBLE_STATUS );

        throw LoggedException();
    }

}
void System::hot_restart(
        const CdmwPlatformMngtBase::StartupKind& startup_kind )
{
    try
    {
        if( !check_only )
            m_system->hot_restart( startup_kind );
    }
    catch( const CdmwPlatformMngt::IncompatibleStatus& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_INCOMPATIBLE_STATUS );
        throw LoggedException();
    }
}

void System::cold_restart(
        const CdmwPlatformMngtBase::StartupKind& startup_kind )
{
    try
    {
        if( !check_only )
            m_system->cold_restart( startup_kind );
    }
    catch( const CdmwPlatformMngt::IncompatibleStatus& e )
    {
        AdminLogMngr::logMessage(
                PlatformMngt::ERR,
                MSG_ID_ADMIN_INCOMPATIBLE_STATUS );
        throw LoggedException();
    }
}

void System::set_property (const char* propertyName, const char* owner, const char* propertyValue)
{
    if( !check_only )
    {
            CORBA::Any_var any_var = new CORBA::Any();

            try
            {
                if ('P'== *owner)
                {
                      *any_var = Property::prepare_predefined_any (propertyName, propertyValue);

                }
                else // User defined property
                {
                    CORBA::Any_var prop_value = new CORBA::Any();
                    try
                    {
                     // get the attribute to check if exists and which type is
                     m_system -> get_attribute (propertyName, prop_value.out() ,
                                       ( ('P'== *owner) ? CdmwPlatformMngt::PRE_DEFINED : CdmwPlatformMngt::USER_DEFINED ));
                    }
                    catch( const CdmwPlatformMngt::AttributeNotFound& e )
                    {
                    AdminLogMngr::logMessage(PlatformMngt::ERR, "Attribute Not Found" );
                    throw LoggedException();
                    }

                    CORBA::TypeCode_var tc = prop_value->type();
                    *any_var = Property::prepare_userdefined_any (tc.in(), propertyValue);
                    
                    
                }
              //finally set the attribute
              m_system -> set_attribute (propertyName,  
                                           any_var.in(),
                                           ( ('P'== *owner) ? CdmwPlatformMngt::PRE_DEFINED : CdmwPlatformMngt::USER_DEFINED));
                
            }
            catch( const CdmwPlatformMngt::AttributeNotFound& e )
            {
                AdminLogMngr::logMessage(
                        PlatformMngt::ERR,
                        "Attribute Not Found" );
                throw LoggedException();
            }
            catch (const CdmwPlatformMngt::ReadOnlyAttribute& e)
            {
                AdminLogMngr::logMessage(
                        PlatformMngt::ERR, 
                        "Read Only Property");
                throw LoggedException();                      
            }
    }
}

void System::get_property (const char* propertyName, const char* owner)
{
    if( !check_only )
    {
            CORBA::Any_var prop_value = new CORBA::Any();

            try
            {
                m_system -> get_attribute (propertyName, prop_value.out() , ( ('P'== *owner) ? CdmwPlatformMngt::PRE_DEFINED : CdmwPlatformMngt::USER_DEFINED ));
                            
            }
            catch( const CdmwPlatformMngt::AttributeNotFound& e )
            {
                AdminLogMngr::logMessage(
                        PlatformMngt::ERR,
                        "Attribute Not Found" );
    
                throw LoggedException();
            }

            CORBA::TypeCode_var tc = prop_value->type();

            if (tc->equal(CdmwPlatformMngt::_tc_SystemStatus) )
            {
                CdmwPlatformMngt::SystemStatus value;
                *prop_value >>= value;
                std::string output("");
                switch( value )
                {   
    
                    case (CdmwPlatformMngt::SYSTEM_INITIAL) :
                    output = "SYSTEM_INITIAL";
                    break;
                    case (CdmwPlatformMngt::SYSTEM_STARTING) :
                    output = "SYSTEM_STARTING";
                    break;
                    case (CdmwPlatformMngt::SYSTEM_STARTUP_SUSPENDED) :
                    output = "SYSTEM_STARTUP_SUSPENDED";
                    break;
                    case (CdmwPlatformMngt::SYSTEM_STARTED) :
                    output = "SYSTEM_STARTED";
                    break;
                    case (CdmwPlatformMngt::SYSTEM_STOPPING) :
                    output = "SYSTEM_STOPPING";
                    break;
                    case (CdmwPlatformMngt::SYSTEM_STOPPED) :
                    output = "SYSTEM_STOPPED";
                    break;
                    case (CdmwPlatformMngt::SYSTEM_PARTIAL_STARTED) :
                    output = "SYSTEM_PARTIAL_STARTED";
                    break;
                    }
            std::cout << propertyName << " : " << output << std::endl;
            }
            else
            {
                Property::print_standard_property_type( propertyName, prop_value.in() );
            }

        }// end checkonly
    }


} // End namespace PlatformAdmin
} // End namespace Cdmw

