/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_MANAGED_COMPONENT_WRAPPER_IMPL_HPP 
#define INCL_PLATFORMMNGT_MANAGED_COMPONENT_WRAPPER_IMPL_HPP 

#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/osthreads/Mutex.hpp"
#include "Foundation/ossupport/ProcessCallback.hpp"

#include "SystemMngt/idllib/CdmwPlatformMngtManagedComponentWrapper.skel.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp"
#include "platformagent/ApplicationAgent_impl.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtProcessDelegate.stub.hpp"

namespace Cdmw
{
namespace PlatformMngt
{

class ApplicationAgent_impl;

/**
* Purpose:
* <p>
* The wrapper of managed component.
*/
class ManagedComponentWrapper_impl : virtual public POA_CdmwPlatformMngt::ManagedComponentWrapper,
    virtual public PortableServer::RefCountServantBase
{

public:


    /**
    * Purpose:
    * <p> Constructor.
    */ 
    ManagedComponentWrapper_impl(const std::string& component_name, 
                                 const std::string& process_name, 
                                 Cdmw::PlatformMngt::ApplicationAgent_impl* agent,
                 CdmwPlatformMngt::ProcessDelegate_ptr process_delegate);
    
    /**
    * Purpose:
    * <p> Destructor.
    */ 
    virtual ~ManagedComponentWrapper_impl() throw (){};
       
    CdmwPlatformMngt::ComponentCommandResult stop_component_wrapper (CdmwPlatformMngt::Timeout stop_timeout)
                                                                        throw(CORBA::SystemException);
    
    char*  get_command_result_info()  throw(CORBA::SystemException);


    CdmwPlatformMngt::ComponentCommandResult initialise_component_wrapper (const CdmwPlatformMngtBase::StartupKind& startup_kind, CdmwPlatformMngt::Timeout init_timeout) throw(CORBA::SystemException);
    
                                  
    void cdmw_initialise_component(const CdmwPlatformMngtBase::StartupKind& startup_kind) throw 
                                                                          (CORBA::SystemException, CdmwPlatformMngt::ManagedComponent::BadOrder);

    CORBA::Object_ptr get_ccm_object();
    
    private:
    // Reference to the corresponding ManagedComponent 
    //CdmwPlatformMngt::ManagedComponent_var m_managed_component;
    
    CORBA::Object_var m_component_obj;

       

    // The mutex protecting concurrent access changing component status
    //OsSupport::Mutex* m_pMutex;

    // The component name 
    std::string m_component_name;
    
    // 
    ApplicationAgent_impl* m_agent;
};


} // End namespace PlatformMngt
} // End namespace Cdmw

#endif // INCL_PLATFORMMNGT_MANAGED_COMPONENT_WRAPPER_IMPL_HPP

