/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_PROCESS_DELEGATE_WRAPPER_IMPL_HPP 
#define INCL_PLATFORMMNGT_PROCESS_DELEGATE_WRAPPER_IMPL_HPP 

#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/osthreads/Mutex.hpp"
#include "Foundation/ossupport/ProcessCallback.hpp"

#include "SystemMngt/idllib/CdmwPlatformMngtProcessDelegateWrapper.skel.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp"
#include "platformagent/ManagedComponentWrapper_impl.hpp"



namespace Cdmw
{
namespace PlatformMngt
{

class ApplicationAgent_impl;

/**
* Purpose:
* <p>
* The wrapper of process delegate.
*/
class ProcessDelegateWrapper_impl : virtual public POA_CdmwPlatformMngt::ProcessDelegateWrapper,
    virtual public PortableServer::RefCountServantBase
{

public:


    /**
    * Purpose:
    * <p> Constructor.
    */ 
    ProcessDelegateWrapper_impl( CdmwPlatformMngt::ProcessDelegate_ptr process,
                                 const std::string processName,
                                 //OsSupport::OS::ProcessId processId );
                                 ApplicationAgent_impl* pAgent );

    /**
    * Purpose:
    * <p> Destructor.
    */ 
    virtual ~ProcessDelegateWrapper_impl() throw ();

    /**
    * Purpose:
    * <p>
    * implements the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/nb_steps:1.0
    * attribute
    */
    virtual CORBA::ULong nb_steps()
        throw(CORBA::SystemException);

    /**
    * Purpose:
    * <p>
    * implements the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/get_service:1.0
    * operation
    */
    virtual CORBA::Object_ptr get_service()
        throw(CORBA::SystemException);

    /**
    * Purpose:
    * <p>
    * implements the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/get_pull_monitorable:1.0
    * operation
    */
    virtual FT::PullMonitorable_ptr get_pull_monitorable()
        throw(CORBA::SystemException);

    /**
    * Purpose:
    * <p>
    * implements the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/get_push_monitorable:1.0
    * operation
    */
    virtual CdmwPlatformMngt::PushMonitorable_ptr get_push_monitorable()
        throw(CORBA::SystemException);

    /**
    * Purpose:
    * <p>
    * implements the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/initialise:1.0
    * operation
    */
    virtual void initialise(const CdmwPlatformMngtBase::StartupKind& startup_kind)
        throw(CdmwPlatformMngt::ProcessDelegate::BadOrder, CORBA::SystemException);

    /**
    * Purpose:
    * <p>
    * implements the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/next_step:1.0
    * operation
    */
    virtual void next_step()
        throw(CdmwPlatformMngt::ProcessDelegate::BadOrder,
              CdmwPlatformMngt::ProcessDelegate::InvalidStep,
              CORBA::SystemException);

    /**
    * Purpose:
    * <p>
    * implements the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/run:1.0
    * operation
    */
    virtual void run()
        throw(CdmwPlatformMngt::ProcessDelegate::NotReadyToRun,
              CdmwPlatformMngt::ProcessDelegate::AlreadyDone,
              CORBA::SystemException);

    /**
    * Purpose:
    * <p>
    * implements the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/stop:1.0
    * operation
    */
    virtual void stop()
        throw(CORBA::SystemException);
    
              
    /**
    * Purpose:
    * <p>
    * implements the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegateWrapper/initialise:1.0
    * operation
    */
    virtual CdmwPlatformMngt::ProcessCommandResult initialise_process(const CdmwPlatformMngtBase::StartupKind& startup_kind,
                                    CdmwPlatformMngt::Timeout init_timeout)
        throw(CORBA::SystemException);

    /**
    * Purpose:
    * <p>
    * implements the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegateWrapper/next_step:1.0
    * operation
    */
    virtual CdmwPlatformMngt::ProcessCommandResult next_step_process(CORBA::ULong step_number,
                                   CdmwPlatformMngt::Timeout next_step_timeout)
        throw(CORBA::SystemException);

    /**
    * Purpose:
    * <p>
    * implements the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegateWrapper/run:1.0
    * operation
    */
    virtual CdmwPlatformMngt::ProcessCommandResult run_process(CdmwPlatformMngt::Timeout run_timeout)
        throw(CORBA::SystemException);

    /**
    * Purpose:
    * <p>
    * implements the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegateWrapper/stop:1.0
    * operation
    */
    virtual CdmwPlatformMngt::ProcessCommandResult stop_process(CdmwPlatformMngt::Timeout stop_timeout)
        throw(CORBA::SystemException);
        
    /**
    * Purpose:
    * <p>
    * implements the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegateWrapper/get_command_result_info:1.0
    * operation
    */
    virtual char* get_command_result_info()
        throw(CORBA::SystemException);
        
    /**
    * Purpose:
    * <p>
    * implements the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegateWrapper/activate_level:1.0
    * operation
    */
    void activate_level(const char *comp, const char *dom, CORBA::ULong level_value) throw(CORBA::SystemException);


    /**
    * Purpose:
    * <p>
    * implements the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegateWrapper/deactivate_level:1.0
    * operation
    */
    void deactivate_level(const char *comp, const char *dom, CORBA::ULong level_value) throw(CORBA::SystemException);


    /**
     * Purpose:
     * <p>
     * implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/get_component_redundancy_state:1.0
     * operation
     */

     virtual CdmwPlatformMngt::RedundancyEltState get_component_redundancy_state(const char* component_name)
          throw(CORBA::SystemException);

    /**
     * Purpose:
     * <p>
     * implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/get_process_redundancy_state:1.0
     * operation
     */
     virtual CdmwPlatformMngt::RedundancyEltState get_process_redundancy_state()  throw(CORBA::SystemException);

    /**
     * Purpose:
     * implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/get_managed_component_wrapper:1.0
     * operation
     */
          
     virtual CdmwPlatformMngt::ManagedComponentWrapper_ptr get_managed_component_wrapper(const char*
     managed_component_wrapper_name,  CORBA::Object_out ccm_object_out)  throw(CORBA::SystemException);

     /**
     * Purpose:
     * implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/get_managed_component:1.0
     * operation
     */
                  
     virtual CdmwPlatformMngt::ManagedComponent_ptr get_managed_component(const char* component_name) throw(CORBA::SystemException);            
    /**
    * Purpose:
    * <p>
    * implements the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegateWrapper/get_process_message_broker:1.0
    * operation
    */
    CdmwPlatformMngt::ProcessMessageBroker_ptr 
    get_process_message_broker()
      throw(CORBA::SystemException);
    void set_process_message_broker(CdmwPlatformMngt::ProcessMessageBroker_ptr pmb) 
      throw();
private:

    /**
    * Purpose:
    * <p> Constructor.
    */ 
    ProcessDelegateWrapper_impl();

    // Reference to the corresponding  ProcessDelegate
    CdmwPlatformMngt::ProcessDelegate_var m_process;

    // status to verify there is no duplicated calls 
    CdmwPlatformMngt::ProcessStatus m_status;

    // The mutex protecting concurrent access changing process status
    OsSupport::Mutex* m_pMutex;
    
    // The mutex protecting concurrent access to the process_message_broker_ref
    OsSupport::Mutex* m_pmbMutex;

    // The last exception name
    std::string m_exceptionName;

    // The current step of the process
    CORBA::ULong m_currentStep;

    //OsSupport::OS::ProcessId m_processID;
    // The process name
    std::string m_processName;
    // The application agent
    ApplicationAgent_impl* m_pAgent;
    
    // the ManagedComponentWrapperMap type
    typedef std::map <std::string, CdmwPlatformMngt::ManagedComponentWrapper_var> ManagedComponentWrapperMap;
    
    // the managed component wrapper class instance
    ManagedComponentWrapperMap m_managed_component_wrapper_map;
    CdmwPlatformMngt::ProcessMessageBroker_var m_process_message_broker_ref;
};


} // End namespace PlatformMngt
} // End namespace Cdmw

#endif // INCL_PLATFORMMNGT_PROCESS_DELEGATE_WRAPPER_IMPL_HPP

