/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "platformagent/ManagedComponentWrapper_impl.hpp"

#include "SystemMngt/idllib/CdmwPlatformMngtManagedComponent.stub.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "Foundation/common/Locations.hpp"
#include "SystemMngt/platformlibrary/LogMngr.hpp"
#include "SystemMngt/platformlibrary/BoundSyncCall.hpp"

#include <Foundation/logging/LogManager.hpp>


namespace Cdmw
{
namespace PlatformMngt
{


    /**
    * Purpose:
    * <p> Constructor.
    */
                        
    ManagedComponentWrapper_impl::ManagedComponentWrapper_impl(const std::string& component_name, 
                                                               const std::string& process_name,
                                   Cdmw::PlatformMngt::ApplicationAgent_impl* agent,
                                   CdmwPlatformMngt::ProcessDelegate_ptr process_delegate): m_agent (agent)
    {
    std::string message = "";
    m_component_name = component_name;
    m_component_obj = CORBA::Object::_nil();
    try 
    {
        if (!CORBA::is_nil(process_delegate))
        {
        // m_component_obj = CORBA::Object::_duplicate(process_delegate->get_managed_component(component_name.c_str()));
        m_component_obj = process_delegate->get_managed_component(component_name.c_str());
        if (CORBA::is_nil(m_component_obj)) 
           LogMngr::logMessage(INF, "managed component is nil");
        else 
            { 
               message = " recovered ManagedComponent from ProcessDelegate on " + component_name; 
               LogMngr::logMessage( INF, message.c_str());
            } 
        }
    }
        catch (const CORBA::SystemException& ex)    
    {
        LogMngr::logMessage( ERR, "exception while contacting ManagedComponent from ProcessDelegate ");
    } 
    };

    CdmwPlatformMngt::ComponentCommandResult ManagedComponentWrapper_impl::stop_component_wrapper (CdmwPlatformMngt::Timeout stop_timeout)  throw(CORBA::SystemException) 
    {
        CdmwPlatformMngt::ComponentCommandResult component_command_result = CdmwPlatformMngt::COMPONENT_CMD_SUCCEED;
        return component_command_result;
    }; 
    
    char* ManagedComponentWrapper_impl::get_command_result_info () throw(CORBA::SystemException)
    {
       //return 0;
       CORBA::String_var info=CORBA::string_dup("");
       return info._retn();
    };


    CdmwPlatformMngt::ComponentCommandResult ManagedComponentWrapper_impl::initialise_component_wrapper(const CdmwPlatformMngtBase::StartupKind& startup_kind, CdmwPlatformMngt::Timeout init_timeout)  throw(CORBA::SystemException)
    {
      
        CdmwPlatformMngt::ComponentCommandResult component_command_result = CdmwPlatformMngt::COMPONENT_CMD_EXCEPTION;
        std::string message("");
    
        CdmwPlatformMngt::ManagedComponent_var m_managed_component = CdmwPlatformMngt::ManagedComponent::_nil();
        try
    {
        m_managed_component = CdmwPlatformMngt::ManagedComponent::_narrow(m_component_obj.in()); 
    }
    catch(const CORBA::SystemException& ex)
    {
         LogMngr::logMessage( ERR, ex._name());
         component_command_result = CdmwPlatformMngt::COMPONENT_CMD_EXCEPTION;
    }
    
        
    if (!CORBA::is_nil(m_managed_component.in()))
    {
            /*MRIstd::string message = "calling cdmw_initialise_component on ManagedComponent on : " + m_component_name ;
         LogMngr::logMessage( INF, message.c_str());
         try 
         {  
              m_managed_component->cdmw_initialise_component(startup_kind);
         }
         catch(const CORBA::SystemException& ex)
         {
              LogMngr::logMessage( ERR, ex._name());
            }*/
            // Call initialise of Component with timeout
            try
            { 
                BoundSyncCallReturn result = EXCEPTION;
                BoundSyncCallFactory* factory = BoundSyncCallFactory::getFactory();
    
                // Initialise it within the timeout
                std::auto_ptr< ComponentInitCall > initCall(
                    factory->createComponentInitCall(
                        m_managed_component.in(),
                        startup_kind,
                        init_timeout ) );
    
                result = initCall->execute();
                switch (result)
                {
                    case EXCEPTION:
            
                        component_command_result = CdmwPlatformMngt::COMPONENT_CMD_EXCEPTION; 

                       message="Exception in component initialisation call: " +
                                        m_component_name + ", " + initCall->getExceptionName();
                        LogMngr::logMessage(ERR, message.c_str());
                        break;
    
                    case TIMEOUT:
                        component_command_result = CdmwPlatformMngt::COMPONENT_CMD_TIMEOUT;
                        message="Timeout expired in component initialisation call: " +
                                        m_component_name;
                        LogMngr::logMessage(ERR,  message.c_str());
                        break;
    
                    case SUCCEED:
                        component_command_result = CdmwPlatformMngt::COMPONENT_CMD_SUCCEED;
                        message="Component successfully initialised: " +
                                        m_component_name;
                        LogMngr::logMessage(INF, message.c_str());
                        break;
                        
                    default:
                        CDMW_ASSERT(false);
         }
    }
            catch( const OutOfMemoryException& e )
            {
                // Log the error
                LogMngr::logMessage( ERR, "Run out of memory" );
                // TODO: do an assert violation till a policy is decided
                CDMW_ASSERT( false );
            }
            
        }
    else 
    {  
    
        // case of nil reference the CCMObject doesn't support ManagedComponent interface
            message = "The CCMObject doesn't support ManagedComponent interface: " +
                                        m_component_name;
            LogMngr::logMessage( ERR , message.c_str()); 
            component_command_result = CdmwPlatformMngt::COMPONENT_CMD_SUCCEED;
    }
    
        return component_command_result;
    };
                                  
    void ManagedComponentWrapper_impl::cdmw_initialise_component(const CdmwPlatformMngtBase::StartupKind& startup_kind)
                      throw  (CdmwPlatformMngt::ManagedComponent::BadOrder, CORBA::SystemException)
    {
        // in Wrapper it is cdmw_initialise_component that is called
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    };


   CORBA::Object_ptr ManagedComponentWrapper_impl::get_ccm_object()
   {
//       return m_component_obj._retn();
        return CORBA::Object::_duplicate(m_component_obj.in());
   }
   
} // End namespace PlatformMngt
} // End namespace Cdmw


