/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_INITTIMEOUTOBSERVER_HPP 
#define INCL_PLATFORMMNGT_INITTIMEOUTOBSERVER_HPP 

#include <SystemMngt/tools/SingleShotTimer.hpp>
#include <SystemMngt/tools/TimeoutObserver.hpp>
#include <Foundation/osthreads/Mutex.hpp>
#include <Foundation/osthreads/Condition.hpp>
#include "platformapplicationlibrary/UnmanagedProcess_impl.hpp"
#include "platformapplicationlibrary/UnmanagedProcessStates.hpp"





namespace Cdmw
{
    namespace PlatformMngt
    {
    
        class InitTimeoutObserver: public TimeoutObserver
        {        
            public:
        
                /**
                *Purpose:
                *<p> Constructor.
                */
                InitTimeoutObserver(Cdmw::PlatformMngt::UnmanagedProcessStateMachine* stateM,
                                    Cdmw::PlatformMngt::UnmanagedProcess_impl* process,
                                    Cdmw::OsSupport::Mutex* mutex,
                                    Cdmw::OsSupport::Condition* condition);  
                        
                 
                /**
                *Purpose:
                *<p> Destructor.
                */
                virtual ~InitTimeoutObserver() { } ;
                
                // callback method called at timeout expiration.
                virtual void wake_up() throw (); 
             

            private:   
                
                /*
                 * Flag used to know the timer status 
                 */
                bool expired ;      
                
                /**
                * Mutex protecting stop flag and associated to the following
                * two conditions
                */
                Cdmw::OsSupport::Mutex* m_mutex;


                /**
                * Condition on which the thread is sleeping to wait 
                * either the start of the timer, either the end of the timer
                * (with a timeout or a cancel timer).
                */
                Cdmw::OsSupport::Condition* m_condition;
                
                /*
                 * Unmanaged Process state machine associated to this timer
                 */
                Cdmw::PlatformMngt::UnmanagedProcessStateMachine* m_stateMachine;
                
                /*
                 * Unmanaged process reference
                 */
                Cdmw::PlatformMngt::UnmanagedProcess_impl* m_process; 
        }; //End class 

    } //End namespace PlatformMngt 
} //End namespace Cdmw
#endif // INCL_PLATFORMMNGT_INITTIMEOUTOBSERVER_HPP

