/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_PLATFORMMNGT_NODEDEPLOYMENTEXECUTOR_HPP
#define INCL_PLATFORMMNGT_NODEDEPLOYMENTEXECUTOR_HPP

#include <platformapplicationlibrary/DeploymentInfo.hpp>
#include <Foundation/idllib/CdmwDnC.stub.hpp>
#include <SystemMngt/platformapplicationlibrary/Application_impl.hpp>
#include <Foundation/osthreads/Thread.hpp>
//#include <Foundation/osthreads/Runnable.hpp>

namespace Cdmw
{
namespace PlatformMngt
{

 /**
  *Purpose:
  *<p> execute startLaunch, finishLaunch operations
  *  on NodeApplications Object
  */
class NodeDeploymentExecutor : public OsSupport::Thread
//public OsSupport::Runnable
{
private:

       CdmwDnC::NodeApplicationManager_var m_node_application_manager;

       CdmwDnC::NodeApplication_var m_node_application;
       
       DeploymentInfo m_deployment_info;
       
       Cdmw::PlatformMngt::Application_impl * m_application;

       OsSupport::Mutex m_mutex;
       
       OsSupport::Mutex m_mutex_to_start;

       OsSupport::Mutex m_mutex_run_stop;

       size_t  m_all_completion_event;

       OsSupport::Condition m_all_completion_event_condition;

       // allows to wait the condition the first time the thread starts
       bool m_init; 
       // true if suspension event on ManagedProcess_impl by the start of ComponentSequencer
       bool m_suspended;

       // true if the thread has been stopped
       bool m_stopped;
       
       // true if the thread has to start
       bool m_to_start;
       
       OsSupport::Condition m_deployment_ended_condition;

       /**
        *  Deployment plan stop timeout.
        */
       unsigned long m_stop_timeout;
       /**
        *  Deployment plan init timeout.
        */
       CdmwPlatformMngt::Timeout m_init_timeout;

public:
    
        NodeDeploymentExecutor(DeploymentInfo deployment_info, 
                               CdmwDnC::NodeApplicationManager_ptr node_application_manager,
                               Cdmw::PlatformMngt::Application_impl *application);

        virtual ~NodeDeploymentExecutor() throw();
    
       /**
        *Purpose:
        *<p> execute startLaunch, finishLaunch operations
        */
        void execute() throw (CdmwDnC::StartError, CdmwDnC::StopError, CORBA::SystemException);

       /**
        * Purpose:
        * <p>run the thread
        */
        virtual void run() throw();

       /**
        * Purpose:
        * <p>stop the thread
        */
        virtual void stop() throw();

       /**
        *Purpose:
        *<p> retrieve the NodeApplicationManager_ptr
        */
        CdmwDnC::NodeApplicationManager_ptr get_node_application_manager();

       /**
        *Purpose:
        *<p> increase a local counter associateted to not INITIALISED components
        * when 0 the the local condition is unlocked
        */

        void increase_counter() throw();

       /**
        *Purpose:
        *<p> decrease a local counter associateted to not INITIALISED components 
        * when 0 the the local condition is unlocked
        */
        void decrease_counter() throw();


       /**
        *Purpose:
        *<p> force to zero local counter associateted to not INITIALISED components
        *  when  0 the local condition is unlocked
        */
        void force_counter_to_zero() throw();
    
    /**
        *Purpose:
        *<p> set to start flag to true
        */
        void set_to_start() throw();
    

    /**
       *Purpose:
       *<p> wait all node_deployment_executors related to an application are ended
       */
    void wait_deployment(const std::string& application_name) throw();
    
    /**
      *Purpose:
      *<p> Set the deployment plan init Timeout
      */
    void set_start_deploy_timeout(CdmwPlatformMngt::Timeout init_tm);

    /**
      *Purpose:
      *<p> Set the deployment plan stop Timeout
      */
    void set_stop_deploy_timeout(unsigned long stop_tm);

    /**
      *Purpose:
      *<p> Get the deployment plan stop Timeout
      */
    unsigned long get_stop_deploy_timeout();

    /**
      *Purpose:
      *<p> wait all node_deployment_executors related to a process are ended
      */
    void wait_deployment(const std::string& application_name, const std::string& process_name) throw();

        inline bool is_stopped() throw() { return m_stopped;}
};

}// closing namespaces
}

#endif /*INCL_PLATFORMMNGT_NODEDEPLOYMENTEXECUTOR_HPP*/
