/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_CCMCOMPONENT_HPP
#define INCL_PLATFORMMNGT_CCMCOMPONENT_HPP
#include "SystemMngt/platformelementlibrary/EntityContainer_impl.hpp"
#include "SystemMngt/platformelementlibrary/RWEntityContainer_impl.hpp"
#include "SystemMngt/platformelementlibrary/EntityContainer.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/logging/Logger.hpp"

#include "SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtComponent.skel.hpp"

#include "SystemMngt/platformlibrary/ServiceDefContainer_impl.hpp"

#include "SystemMngt/platformelementlibrary/ManagedElement_impl.hpp"
#include "SystemMngt/platformelementlibrary/DataStoreDefinition.hpp"

#include "SystemMngt/idllib/CdmwPlatformMngtElementRepository.stub.hpp"

#include "Foundation/osthreads/Condition.hpp"
#include "Foundation/osthreads/Mutex.hpp"
#include "Foundation/osthreads/ReaderWriterLock.hpp"

#include "SystemMngt/idllib/CdmwPlatformMngtElementRepository.stub.hpp"

#include "platformapplicationlibrary/ComponentStates.hpp"
#include "SystemMngt/platformlibrary/StateMachineTemplate.hpp"
#include "SystemMngt/platformelementlibrary/ManagedElement_impl.hpp"
#include "SystemMngt/platformelementlibrary/EntityContainer.hpp"
#include "SystemMngt/platformelementlibrary/DataStoreDefinition.hpp"
#include "platformapplicationlibrary/Process_impl.hpp"


#include <map>
#include <iostream>

namespace Cdmw
{
  namespace PlatformMngt
  {

    class RedundancyGroup_impl;
    class FTGroup_impl;
    class ComponentStatusChangeFactory;
    
    class CcmComponent_impl:virtual public POA_CdmwPlatformMngt::CcmComponent,
      public ManagedElement_impl
    {
        friend class ComponentConfiguring;
        friend class ComponentInitialising;
        friend class ComponentStopping;
        friend class ComponentStopped;
        friend class ComponentFailedError;
        friend class ComponentFailedInvalid;
        friend class ComponentFailedTimeout;
        
    public:
    static const string CcmComponent_impl::M_initialStateName;              
    static const string CcmComponent_impl::M_initialisingStateName;             
    static const string CcmComponent_impl::M_configuringStateName;      
    static const string CcmComponent_impl::M_configuredStateName;             
    static const string CcmComponent_impl::M_initialisedStateName;           
    static const string CcmComponent_impl::M_stoppingStateName;             
    static const string CcmComponent_impl::M_stoppedStateName;                
    static const string CcmComponent_impl::M_failedErrorStateName;              
    static const string CcmComponent_impl::M_failedTimeoutStateName;           
    static const string CcmComponent_impl::M_failedInvalidStateName; 
    
    /**
     * The startup kind.
     */
    CdmwPlatformMngtBase::StartupKind_var m_startup_kind;
    
    protected:

        /**
         * Component Datastore
         */
        ComponentDataStore* m_component_ds;
        
        /**
         * The name indicating the RedundancyGroup membership.
         */
        CORBA::String_var m_redundancy_group_name;
        
        /**
         * The type of the RedundancyGroup membership ( FT_GROUP_TYPE / LB_GROUP_TYPE / COMPONENT_TYPE )
         */
        CdmwPlatformMngt::ElementType m_redundancy_group_type;
    
        /**
         * The  Component name.
         */
        CORBA::String_var m_name;
        
        /**
         * The  Component init timeout.
         */
        CdmwPlatformMngt::Timeout m_init_timeout;

    private:
        
        /**
         * The group managing this component.
         */
        Cdmw::PlatformMngt::RedundancyGroup_impl* m_redundancy_group;
        
        /**
         *Purpose:
         *<p> convert TECH_STATUS property to process status
         *
         */
        CdmwPlatformMngt::ComponentStatus property_to_status (const char* status)
        throw (CdmwPlatformMngt::AttributeNotFound);
        
    protected:
        /**
         *Purpose:
         *<p> Gets the component's states.
         * This method must exclusively be used by the state machine.
         */
        CdmwPlatformMngt::ComponentStatus getStatus()
            throw( CORBA::SystemException );

     public:
        /**
         *Purpose:
         *<p> Set the component record in data store
         *
         */
        void set_record_in_datastore();

        /**
         *Purpose:
         *<p> Remove the component record in datastore
         *
         */
        void remove_record_in_datastore();

        /**
         *Purpose:
         *<p> Get the component record from data store
         *
         */
        CdmwPlatformMngt::ComponentRecord* get_record_from_datastore();
        
        bool is_ft_group();
         
        bool is_lb_group();
        
    public:


      CcmComponent_impl (PortableServer::POA_ptr poa,
                         const CdmwPlatformMngt::ComponentDef& ccmcomponent_def,
                         CdmwPlatformMngt::
                         RWElementRepository_ptr element_repository,
             const char * component_key,
             const char * component_path,
             Cdmw::PlatformMngt::Process_impl* process_impl);
      
      /**
       *Purpose:
       *<p> Default destructor.
       */
      virtual ~CcmComponent_impl();
      
      void set_indicator_relevant_status (CORBA::Boolean indicator_relevant_status)
                throw (CORBA::SystemException);
      
      CORBA::ULong get_number_of_entities () throw (CORBA::SystemException);

      void get_all_entity_names (CORBA::ULong,
                                 CdmwPlatformMngt::ElementNames_out,
                                 CdmwPlatformMngt::ElementNamesIterator_out) throw (CORBA::SystemException);

      CdmwPlatformMngt::Entity *
        get_entity (const char *) throw ( CdmwPlatformMngt::EntityNotFound,CORBA::SystemException);
      
      CORBA::Boolean get_entities (const CdmwPlatformMngt::ElementNames &,
                                     CdmwPlatformMngt::Entities_out) throw (CORBA::SystemException);

      void get_all_entities (CORBA::ULong how_many,
                             CdmwPlatformMngt::Entities_out ent_out,
                             CdmwPlatformMngt::
                             EntitiesIterator_out ent_iter_out) throw (CORBA::
                                                                       SystemException);

      CdmwPlatformMngt::Entity *
        add_entity (const CdmwPlatformMngt::EntityDef &) throw (CORBA::
                                                                SystemException);

      void get_attribute (const char *, CORBA::Any_out,
                          CdmwPlatformMngt::Owner) 
                          throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound);


      void set_attribute (const char *, const CORBA::Any &,
                          CdmwPlatformMngt::Owner) throw (CORBA::SystemException,
                                                  CdmwPlatformMngt::AttributeNotFound,
                                                  CdmwPlatformMngt::ReadOnlyAttribute);



      CdmwPlatformMngt::ElementType get_element_type () throw (CORBA::
                                                                 SystemException);


      CdmwPlatformMngt::Children *
        get_children () throw (CORBA::SystemException);


      CdmwPlatformMngt::ComponentSnapshot *
        get_snapshot () throw (CORBA::SystemException);


      CdmwPlatformMngt::ComponentStatus get_status (CORBA::String_out) 
                                          throw (CORBA::SystemException);


      char *instantiation_id () throw (CORBA::SystemException);


      CdmwPlatformMngtBase::TimeStamp start_time () throw (CORBA::
                                                             SystemException);


      CdmwPlatformMngtBase::TimeStamp stop_time () throw (CORBA::
                                                            SystemException);


      char *get_component_server_name () throw (CORBA::SystemException);


      CORBA::Boolean get_indicator_relevant_status () throw (CORBA::
                                                               SystemException);

      Cdmw::PlatformMngt::RedundancyGroup_impl* get_group()
          throw (CORBA::SystemException,  CdmwPlatformMngt::GroupNotFound);

      CdmwPlatformMngt::RedundancyGroup *
        get_component_group () throw (CORBA::SystemException);

      void remove_entity (const char *) throw (CdmwPlatformMngt::EntityNotFound, CORBA::SystemException);

      char* deployment_plan () throw (CORBA::SystemException);

      /**
       *Purpose:
       *<p> Synchronise with datastore
       *
       */
      virtual
      void synchro_with_datastore();
      
      /**
       *Purpose:
       *<p> Creates all the state instances (states are singleton).
       *
       *@exception OutOfMemoryException Lack of memory.
       */
      static void createStates()
          throw( OutOfMemoryException );

      /**
       *Purpose:
       *<p> Destroys all the states.
       */
      static void destroyStates();

      /**
       * Moves the state machine to specified state.
       */
      void setState( const char* stateName );


          
      void initialise( const CdmwPlatformMngtBase::StartupKind& startup_kind, const std::string& requested_deploymnet_plan)
              throw( CdmwPlatformMngt::IncompatibleStatus );
        
      void stopping() throw( CdmwPlatformMngt::IncompatibleStatus );  
      
      void stop() throw( CdmwPlatformMngt::IncompatibleStatus );  
      
      void failed_error() throw( CdmwPlatformMngt::IncompatibleStatus );
      
      /**
       *Purpose:
       *<p> Notifies the component's status has changed. This method is
       * designed to be used inside the state machine.
       */
      void notifyStatusChangeEvent();
      
     /**
     *Purpose:
     *<p> Get a Entity implementation.
     *
     *@param entity_name the name identifying the entity.
     *
     */              
     Entity_impl* getEntity (const char* entity_name)
           throw (CORBA::SystemException);
      
      /**
      *Purpose:
      *<p> Synchronise the entities of CcmComponent
      *
      */
     
     void synchro_entities_with_datastore();
      /**
       *Purpose:
       *<p> Ends the creation of the ccmcomponet
       *
       *@exception DuplicateEntity if there is a duplicate entity name among
       * the entities of the process
       */
      void end_creation(const CdmwPlatformMngt::ComponentDef& ccmcomponent_def)
            throw (CdmwPlatformMngt::DuplicateEntity,
                   OutOfMemoryException,
                   CORBA::SystemException);
      
      private:
          
      bool doInitialisation() throw (CORBA::SystemException); 
          
      Cdmw::PlatformMngt::Process_impl* m_process_impl;
          
      CORBA::String_var m_deployment_plan;
     
      CORBA::Object_var m_ccm_object;
      
      /**
       * The associated state machine.
       */
      ComponentStateMachine* m_stateMachine;
      
      static ComponentInitial* M_initial;
      static ComponentInitialising* M_initialising;
      static ComponentConfiguring* M_configuring;
      static ComponentInitialised* M_initialised;
      static ComponentConfigured* M_configured;
      static ComponentStopping* M_stopping;
      static ComponentStopped* M_stopped;
      static ComponentFailedError* M_failedError;
      static ComponentFailedTimeout* M_failedTimeout;
      static ComponentFailedInvalid* M_failedInvalid;
      
private:

    Cdmw::Logging::Logger* m_logger; 

    /**
     * The RW Entity Container object var to take pointer ownership
     */
    PortableServer::ServantBase_var m_rwEntityContainerServant;
    /**
     * The read/write entity container of this process.
     */
    RWEntityContainer_impl* m_rwEntityContainer;

    PlatformMngt::EntityContainer* m_actualEntityContainer;
    
    /**
     * The component status change factory.
     */
    ComponentStatusChangeFactory* m_compStatusChangeFactory;
    
    /**
     * The supervision event notifier.
     */
    SupervisionEventNotifier* m_eventNotifier;
    
    };

  }// End namespace PlatformMngt
}// End namespace Cdmw



#endif                          //INCL_PLATFORMMNGT_CCMCOMPONENT_HPP
