/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_COMPONENT_STATES_HPP
#define INCL_PLATFORMMNGT_COMPONENT_STATES_HPP

#include "Foundation/common/Exception.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtProcess.stub.hpp"
#include "SystemMngt/platformlibrary/StateMachineTemplate.hpp"

namespace Cdmw
{
namespace PlatformMngt
{

class CcmComponent_impl;
class ComponentState;

typedef StateMachine< CcmComponent_impl, ComponentState > ComponentStateMachine;

/**
 *Purpose:
 *<p> The abstract state of the state machine.
 * Provides the default implementation of an application state.
 */ 
class ComponentState : public virtual ComponentStateMachine::AbstractState
{
    
private:
    ComponentState(
        const ComponentState& rhs );

    ComponentState& operator=(
        const ComponentState& rhs );

protected:
    ComponentState() {};

public:
    virtual ~ComponentState() {};

public:

    virtual CdmwPlatformMngt::ComponentStatus status() = 0;
    
    virtual
     void entryAction(
         ComponentStateMachine* stateMachine )
     {};

     virtual
     void exitAction(
         ComponentStateMachine* stateMachine )
     {};

     virtual
     void doActivity(
         ComponentStateMachine* stateMachine )
     {};

     virtual
     void completionTransition(
         ComponentStateMachine* stateMachine )
     {};

     virtual
     void initialise(ComponentStateMachine* stateMachine, const CdmwPlatformMngtBase::StartupKind& startup_kind )
             throw( CdmwPlatformMngt::IncompatibleStatus )
     {
         throw CdmwPlatformMngt::IncompatibleStatus();
     };
     
     virtual
     void set_stopping(ComponentStateMachine* stateMachine )
             throw( CdmwPlatformMngt::IncompatibleStatus )
     {
         throw CdmwPlatformMngt::IncompatibleStatus();
     };
     
     virtual
     void set_stop(ComponentStateMachine* stateMachine )
             throw( CdmwPlatformMngt::IncompatibleStatus )
     {
         throw CdmwPlatformMngt::IncompatibleStatus();
     };

     virtual
     void set_failed_error(ComponentStateMachine* stateMachine )
             throw( CdmwPlatformMngt::IncompatibleStatus )
     {
         throw CdmwPlatformMngt::IncompatibleStatus();
     };

};

class ComponentInitial : public virtual ComponentState
{
private:
    ComponentInitial(
        const ComponentInitial& rhs );

    ComponentInitial& operator=(
        const ComponentInitial& rhs );

public:
    ComponentInitial();

    CdmwPlatformMngt::ComponentStatus status();
    
    void entryAction(ComponentStateMachine* stateMachine );    
    
    void initialise(ComponentStateMachine* stateMachine,  const CdmwPlatformMngtBase::StartupKind& startup_kind )
            throw( CdmwPlatformMngt::IncompatibleStatus);

    void set_failed_error(ComponentStateMachine* stateMachine )
                 throw( CdmwPlatformMngt::IncompatibleStatus );
                 
    void set_stopping(ComponentStateMachine* stateMachine )
             throw( CdmwPlatformMngt::IncompatibleStatus );                 

};

class ComponentConfiguring : public virtual ComponentState
{
private:
    ComponentConfiguring(
        const ComponentConfiguring& rhs );

    ComponentConfiguring& operator=(
        const ComponentConfiguring& rhs );

public:
    ComponentConfiguring();

    CdmwPlatformMngt::ComponentStatus status();

    void entryAction(ComponentStateMachine* stateMachine ); 

};


class ComponentConfigured : public virtual ComponentState
{
private:
    ComponentConfigured(
        const ComponentConfigured& rhs );

    ComponentConfigured& operator=(
        const ComponentConfigured& rhs );

public:
    ComponentConfigured();

    CdmwPlatformMngt::ComponentStatus status();
    
    void entryAction(ComponentStateMachine* stateMachine ); 
};

class ComponentInitialising : public virtual ComponentState
{
private:
    ComponentInitialising(
        const ComponentInitialising& rhs );

    ComponentInitialising& operator=(
        const ComponentInitialising& rhs );

public:
    ComponentInitialising();

    CdmwPlatformMngt::ComponentStatus status();
    
    void entryAction(ComponentStateMachine* stateMachine );

    void exitAction(ComponentStateMachine* stateMachine );

    void doActivity(ComponentStateMachine* stateMachine ) ;
    
    void set_failed_error(ComponentStateMachine* stateMachine )
             throw( CdmwPlatformMngt::IncompatibleStatus );

   void set_stopping(ComponentStateMachine* stateMachine )
             throw( CdmwPlatformMngt::IncompatibleStatus );
             
   void set_stop(ComponentStateMachine* stateMachine )
             throw( CdmwPlatformMngt::IncompatibleStatus );

};


class ComponentInitialised : public virtual ComponentState
{
private:
    ComponentInitialised(
        const ComponentInitialised& rhs );

    ComponentInitialised& operator=(
        const ComponentInitialised& rhs );

public:
    ComponentInitialised();

    CdmwPlatformMngt::ComponentStatus status();
    
    void entryAction(ComponentStateMachine* stateMachine );
    
    void set_stopping(ComponentStateMachine* stateMachine )
             throw( CdmwPlatformMngt::IncompatibleStatus );
             
    void set_stop(ComponentStateMachine* stateMachine )
             throw( CdmwPlatformMngt::IncompatibleStatus );
    
    void set_failed_error(ComponentStateMachine* stateMachine )
                 throw( CdmwPlatformMngt::IncompatibleStatus );
  
};



class ComponentStopping : public virtual ComponentState
{
private:
    ComponentStopping(
        const ComponentStopping& rhs );

    ComponentStopping& operator=(
        const ComponentStopping& rhs );

public:
    ComponentStopping();

    CdmwPlatformMngt::ComponentStatus status();
    
    void entryAction(ComponentStateMachine* stateMachine );
    
    void set_stop(ComponentStateMachine* stateMachine )
             throw( CdmwPlatformMngt::IncompatibleStatus );
};



class ComponentStopped : public virtual ComponentState
{
private:
    ComponentStopped(
        const ComponentStopped& rhs );

    ComponentStopped& operator=(
        const ComponentStopped& rhs );

public:
    ComponentStopped();

    CdmwPlatformMngt::ComponentStatus status();
    
    void entryAction(ComponentStateMachine* stateMachine );
    
    void initialise(ComponentStateMachine* stateMachine,  const CdmwPlatformMngtBase::StartupKind& startup_kind )
            throw( CdmwPlatformMngt::IncompatibleStatus);
};


class ComponentFailedError : public virtual ComponentState
{
private:
    ComponentFailedError(
        const ComponentFailedError& rhs );

    ComponentFailedError& operator=(
        const ComponentFailedError& rhs );

public:
    ComponentFailedError();

    CdmwPlatformMngt::ComponentStatus status();
    
    void entryAction(ComponentStateMachine* stateMachine );    
    
    void initialise(ComponentStateMachine* stateMachine,  const CdmwPlatformMngtBase::StartupKind& startup_kind )
                throw( CdmwPlatformMngt::IncompatibleStatus);
                
    void set_failed_error(ComponentStateMachine* stateMachine )
        throw( CdmwPlatformMngt::IncompatibleStatus );
};


class ComponentFailedTimeout : public virtual ComponentState
{
private:
    ComponentFailedTimeout(
        const ComponentFailedTimeout& rhs );

    ComponentFailedTimeout& operator=(
        const ComponentFailedTimeout& rhs );

public:
    ComponentFailedTimeout();

    CdmwPlatformMngt::ComponentStatus status();
    
    void entryAction(ComponentStateMachine* stateMachine ); 
    
    void initialise(ComponentStateMachine* stateMachine,  const CdmwPlatformMngtBase::StartupKind& startup_kind )
                throw( CdmwPlatformMngt::IncompatibleStatus);
                
    void set_failed_error(ComponentStateMachine* stateMachine )
        throw( CdmwPlatformMngt::IncompatibleStatus );
};


class ComponentFailedInvalid : public virtual ComponentState
{
private:
    ComponentFailedInvalid(
        const ComponentFailedInvalid& rhs );

    ComponentFailedInvalid& operator=(
        const ComponentFailedInvalid& rhs );

public:
    ComponentFailedInvalid();

    CdmwPlatformMngt::ComponentStatus status();
    
    void entryAction(ComponentStateMachine* stateMachine ); 
};

} // End namespace PlatformMngt
} // End namespace Cdmw

#endif //INCL_PLATFORMMNGT_COMPONENT_STATES_HPP

