/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_FTGROUP_HPP
#define INCL_PLATFORMMNGT_FTGROUP_HPP
#include "SystemMngt/platformelementlibrary/EntityContainer_impl.hpp"
#include "SystemMngt/platformelementlibrary/RWEntityContainer_impl.hpp"
#include "SystemMngt/platformelementlibrary/EntityContainer.hpp"
#include "Foundation/common/Exception.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtManagedElement.skel.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtRedundancyGroup.skel.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtFTGroup.skel.hpp"
#include "SystemMngt/platformlibrary/ServiceDefContainer_impl.hpp"
#include "SystemMngt/platformelementlibrary/ManagedElement_impl.hpp"
#include "SystemMngt/platformelementlibrary/DataStoreDefinition.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtElementRepository.stub.hpp"
#include "Foundation/osthreads/Condition.hpp"
#include "Foundation/osthreads/Mutex.hpp"
#include "Foundation/osthreads/ReaderWriterLock.hpp"
#include "SystemMngt/platformlibrary/StateMachineTemplate.hpp"
#include "SystemMngt/platformelementlibrary/ManagedElement_impl.hpp"
#include "SystemMngt/platformelementlibrary/EntityContainer.hpp"
#include "SystemMngt/platformelementlibrary/DataStoreDefinition.hpp"
#include "platformapplicationlibrary/RedundancyGroup_impl.hpp"

#include <iostream>

namespace Cdmw
{
namespace PlatformMngt
{

class FTGroup_impl : virtual public POA_CdmwPlatformMngt::FTGroup, public Cdmw::PlatformMngt::RedundancyGroup_impl
{
    

public:


    FTGroup_impl( PortableServer::POA_ptr poa, const CdmwPlatformMngt::FTGroupDef& ftgroup_def,
       CdmwPlatformMngt::RWElementRepository_ptr element_repository,
       const char *ftgroup_key,
       const char *ftgroup_path) ;


    CdmwPlatformMngt::FTGroupSnapshot* get_snapshot() throw (CORBA::SystemException);


    CdmwPlatformMngt::ReplicationStyle get_style() throw (CORBA::SystemException);


    void set_style(CdmwPlatformMngt::ReplicationStyle) throw (CORBA::SystemException);
    
     /**
      * Purpose:
      * <p>
      * Implements the
      * IDL:thalesgroup.com/CdmwPlatformMngt/ManagedElement/get_element_type:1.0
      * operation
      */

    virtual CdmwPlatformMngt::ElementType get_element_type()
                       throw (CORBA::SystemException);

     /**
     *Purpose:
     *<p> Get a Entity implementation.
     *
     *@param entity_name the name identifying the entity.
     *
     */              
     Entity_impl* getEntity (const char* entity_name)
           throw (CORBA::SystemException);

    /**
     *Purpose:
     *<p> Synchronise the entities of FTGroup
     *
     */
     void synchro_entities_with_datastore();
     
     /**
      *Purpose:
      *<p> Ends the creation of the FTGroup
      *
      *@exception DuplicateEntity if there is a duplicate entity name among
      * the entities of the process
      */
     void end_creation(const CdmwPlatformMngt::FTGroupDef& ftgroup_def)
            throw (CdmwPlatformMngt::DuplicateEntity,
                   OutOfMemoryException,
                   CORBA::SystemException);

private:

    /**
     * The RW Entity Container object var to take pointer ownership
     */
    PortableServer::ServantBase_var m_rwEntityContainerServant;
    /**
     * The read/write entity container of this process.
     */
    RWEntityContainer_impl* m_rwEntityContainer;

    PlatformMngt::EntityContainer* m_actualEntityContainer;

};

    //try to intance it only to check compilation ...
     
    /*
    PortableServer::POA_ptr poa;
    CdmwPlatformMngt::RWElementRepository_ptr element_repository;
    FTGroup_impl *ftgroup = new FTGroup_impl(poa, element_repository);  
    */
    



} // End namespace PlatformMngt
} // End namespace Cdmw



#endif //INCL_PLATFORMMNGT_FTGROUP_HPP

