/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_REDUNDANCYGROUP_HPP
#define INCL_PLATFORMMNGT_REDUNDANCYGROUP_HPP
#include "Foundation/common/Exception.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/logging/Logger.hpp"
#include "Foundation/osthreads/Condition.hpp"
#include "Foundation/osthreads/Mutex.hpp"
#include "Foundation/osthreads/ReaderWriterLock.hpp"

#include "SystemMngt/platformelementlibrary/EntityContainer_impl.hpp"
#include "SystemMngt/platformelementlibrary/RWEntityContainer_impl.hpp"
#include "SystemMngt/platformelementlibrary/EntityContainer.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtManagedElement.skel.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtRedundancyGroup.skel.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtElementRepository.stub.hpp"
#include "SystemMngt/platformlibrary/ServiceDefContainer_impl.hpp"
#include "SystemMngt/platformelementlibrary/ManagedElement_impl.hpp"
#include "SystemMngt/platformelementlibrary/DataStoreDefinition.hpp"
#include "SystemMngt/platformlibrary/StateMachineTemplate.hpp"
#include "SystemMngt/platformlibrary/SupervisionEventNotifier.hpp"

#include <iostream>

namespace Cdmw
{
  namespace PlatformMngt
  {

  class CcmComponent_impl;
  class RedundancyGroupStatusChangeFactory;
  
    
    class RedundancyGroup_impl:virtual public POA_CdmwPlatformMngt::
      RedundancyGroup, public ManagedElement_impl
    {


    public:


      RedundancyGroup_impl (PortableServer::POA_ptr poa,
                            CdmwPlatformMngt::
                            RWElementRepository_ptr element_repository);
      
      /**
       *Purpose:
       *<p> Default destructor.
       */
      virtual ~RedundancyGroup_impl();


      CORBA::ULong get_number_of_entities () throw (CORBA::SystemException);


      void get_all_entity_names (CORBA::ULong how_many,
                                 CdmwPlatformMngt::ElementNames_out entity_names,
                                 CdmwPlatformMngt::
                                 ElementNamesIterator_out rest) throw (CORBA::
                                                                  SystemException);


      CdmwPlatformMngt::Entity *
        get_entity (const char * entity_name) throw ( CdmwPlatformMngt::EntityNotFound, CORBA::SystemException);

      CORBA::Boolean get_entities (const CdmwPlatformMngt::ElementNames & entity_names,
                                     CdmwPlatformMngt::
                                     Entities_out nentities) throw (CORBA::
                                                          SystemException);

      void get_all_entities (CORBA::ULong how_many, CdmwPlatformMngt::Entities_out nentities,
                             CdmwPlatformMngt::
                             EntitiesIterator_out rest ) throw (CORBA::
                                                          SystemException);


      CdmwPlatformMngt::Entity *
        add_entity (const CdmwPlatformMngt::EntityDef &) throw (CORBA::
                                                                SystemException);

      void remove_entity (const char *) throw (CdmwPlatformMngt::EntityNotFound, CORBA::SystemException);

      void get_attribute (const char *, CORBA::Any_out,
                          CdmwPlatformMngt::Owner) throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound);

      void set_attribute (const char *, const CORBA::Any &,
                          CdmwPlatformMngt::Owner)
                           throw (CORBA::SystemException,
                                                  CdmwPlatformMngt::AttributeNotFound,
                                                  CdmwPlatformMngt::ReadOnlyAttribute);


      CdmwPlatformMngt::ElementType get_element_type () throw (CORBA::
                                                                 SystemException);


      CdmwPlatformMngt::Children *
        get_children () throw (CORBA::SystemException);

      CORBA::Long get_minimum_size () throw (CORBA::SystemException);


      char *group_domain_id () throw (CORBA::SystemException);

      CdmwPlatformMngt::TechnicalStatus get_status(CORBA::String_out) throw (CORBA::SystemException);

      void set_minimum_size (CORBA::Long) throw (CORBA::SystemException);

      void start_up () throw (CORBA::SystemException);

      void shut_down () throw (CORBA::SystemException);

      void insert (CdmwPlatformMngt::ManagedElement *) throw (CORBA::
                                                              SystemException);

      void get_all_components (CORBA::ULong,
                               CdmwPlatformMngt::CcmComponents_out,
                               CdmwPlatformMngt::
                               ComponentsIterator_out) throw (CORBA::
                                                              SystemException);


      CdmwPlatformMngt::CcmComponent_ptr get_component (const char *,
                                                        const char *)
        throw (CORBA::SystemException,CdmwPlatformMngt::ComponentNotFound);      

      CORBA::ULong get_number_of_components () throw (CORBA::SystemException);

      void _cxx_delete (CdmwPlatformMngt::ManagedElement *) throw (CORBA::
                                                                   SystemException);

      /*
       * Purpose :
       * <p> Add the component to the component map.
       * 
       * @param full_component_name "componentName/componentHostName"
       * @param component pointer
       * @param component Def
       */
      void add_component(std::string full_component_name, 
                         Cdmw::PlatformMngt::CcmComponent_impl*,
                         const CdmwPlatformMngt::ComponentDef& ccmcomponent_def ) 
          throw (CORBA::SystemException,CdmwPlatformMngt::ComponentAlreadyExists);

      /*
       * Purpose :
       * <p> update the component status in to the component map.
       * 
       * @param full_component_name "componentName/componentHostName"
       * @param the current component status
       */
      void update_component_status(const std::string& component_name,const char* component_status) 
      throw (CORBA::SystemException,CdmwPlatformMngt::ComponentNotFound);

      /*
       * Purpose :
       * <p> update the group status
       */
      void update_group_status();
      
      /**
       *Purpose:
       *<p> Synchronise with datastore
       *
       */
      void synchro_with_datastore();
      /**
       *Purpose:
       *<p> Synchronise the entities of redundancygroup
       *
       */
      virtual void synchro_entities_with_datastore();

protected:
      /* to set the var. m_rwEntityContainer */
      void set_rw_entity_container( RWEntityContainer_impl* rwEntityContainer);     

      /* to set  minimum number of replicas */
      void set_minimum_nbr_of_replicas( const long & minimum_nb_of_replicas);
      
      /* to set the ReplicationStyle */
      void set_replication_style( const CdmwPlatformMngt::ReplicationStyle& replication_style);
      
      /* to set BalancingStyle */
      void set_balancing_style( const CdmwPlatformMngt::BalancingStyle& balancing_style);

      /* to get the ReplicationStyle */
      CdmwPlatformMngt::ReplicationStyle get_replication_style();

      /* to get BalancingStyle */
      CdmwPlatformMngt::BalancingStyle get_balancing_style();
      
      /* to set the m_rwEntityContainer */
      void set_group_type(  const CdmwPlatformMngt::ElementType& group_type);
      
private:

     /* this var. will be set from a child every time it needs */
     RWEntityContainer_impl* m_rwEntityContainer;

    /**
     * The group type ( FT_GROUP_TYPE / LB_GROUP_TYPE )
     */
     CdmwPlatformMngt::ElementType m_group_type;
     
    /**
     * The minimum number of replicas
     */
    long m_minimum_nb_of_replicas;
    
    /**
     * The RedundancyGroup style
     * only one of the following Style is used, depending on the group_type
     */
     CdmwPlatformMngt::ReplicationStyle m_replication_style;

     CdmwPlatformMngt::BalancingStyle m_balancing_style;
     
    /**
     * The read/write lock to protect access to the internal data.
     */
    OsSupport::ReaderWriterLock m_rwGroupStatusLock;    
    
    /**
     * The group status change factory.
     */
    RedundancyGroupStatusChangeFactory* m_redgrStatusChangeFactory;
    
    Cdmw::Logging::Logger* m_logger; 




protected:
    
    /**
     * Component Datastore
     */
    RedundancyGroupDataStore* m_redundancy_group_ds;
    
    /**
     * The group status 
     */
     CdmwPlatformMngt::TechnicalStatus m_group_status;
    
    /**
     * The Components making up the group 
     */
     CdmwPlatformMngt::ComponentDefs m_components;
     
     /**
      * The map which contains the componentData
      * data: pointer , status 
      * ( When the status is 1, the component is considered active,
      * instead when is 0, the component is considered down) 
      */
     typedef struct ComponentData
     {
         CcmComponent_impl* component;       
         bool status;
     };
     typedef std::map <std::string, ComponentData*> ComponentsMap;
     ComponentsMap m_component_map ;
    
public:
     
    /**
     *Purpose:
     *<p> Set the component record in data store
     *
     */
    void set_record_in_datastore();

    /**
     *Purpose:
     *<p> Remove the redundancy_group_record in datastore
     *
     */
    void remove_record_in_datastore();

    /**
     *Purpose:
     *<p> Get the redundancy_group_record  from datastore
     *
     */
    CdmwPlatformMngt::RedundancyGroupRecord* get_record_from_datastore();
    
    /**
     *Purpose:
     *<p> Notifies the group's status has changed.
     */
    void notifyStatusChangeEvent();
    
    SupervisionEventNotifier* m_eventNotifier;

    };

  }// End namespace PlatformMngt
}// End namespace Cdmw

#endif                          //INCL_PLATFORMMNGT_REDUNDANCYGROUP_HPP
