/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "SystemMngt/platformapplicationlibrary/ApplicationConsistencyStatusSolver.hpp"
#include "SystemMngt/platformapplicationlibrary/Application_impl.hpp"
#include "SystemMngt/platformlibrary/LogMngr.hpp"
#include "SystemMngt/platformapplicationlibrary/ApplicationLogMessageId.hpp"

namespace Cdmw
{
namespace PlatformMngt
{

using namespace std;

  

//////////////////////////////////////////////////////////
//   Application Consistency Status Solver Implementation
/////////////////////////////////////////////////////////

         
ApplicationConsistencyStatusSolver::ApplicationConsistencyStatusSolver()
{
}
         
ApplicationConsistencyStatusSolver::~ApplicationConsistencyStatusSolver()
{
}


std::string ApplicationConsistencyStatusSolver::get_consistent_status(ApplicationStateMachine* state_machine,  const string& status)
{
    std::string state = status;

    Cdmw::PlatformMngt::Application_impl* application = state_machine->getContext();
    // log message
    std::ostringstream log_message;
    log_message << " Application status = " << state;

    // get all processes, all running processes, stopped processes, failed processes  to check Application consistent status
    CORBA::ULong all_processes     = application ->get_number_of_processes_with_relevant_indicator();
     
    log_message << "number_of_processes_with_relevant_indicator = "<< all_processes;

    CORBA::ULong running_processes = 
               application->get_number_of_processes_with_status(CdmwPlatformMngt::PROCESS_RUNNING);

    CORBA::ULong stopped_processes = 
               application->get_number_of_processes_with_status(CdmwPlatformMngt::PROCESS_STOPPED);

    CORBA::ULong failed_processes =
                        application->get_number_of_processes_with_status(CdmwPlatformMngt::PROCESS_FAILED_ERROR);
    failed_processes += application->get_number_of_processes_with_status(CdmwPlatformMngt::PROCESS_FAILED_TIMEOUT);
    failed_processes += application->get_number_of_processes_with_status(CdmwPlatformMngt::PROCESS_FAILED_DISCREPANCY); 
    failed_processes += application->get_number_of_processes_with_status(CdmwPlatformMngt::PROCESS_FAILED_INVALID);
    failed_processes += application->get_number_of_processes_with_status(CdmwPlatformMngt::PROCESS_FAILED_NO_RESPONSE);
    failed_processes += application->get_number_of_processes_with_status(CdmwPlatformMngt::PROCESS_FAILED_DEATH);

    CORBA::ULong ended_processes = application->get_number_of_processes_with_status(CdmwPlatformMngt::PROCESS_ENDED);

    //  if all processes are ENDED application is RUNNING
    if (ended_processes == all_processes)
    {
         state = Application_impl::M_runningStateName;
    }
    // if all processes are RUNNING application is RUNNING
    else if (running_processes == (all_processes-ended_processes))
    {
        state = Application_impl::M_runningStateName;
    }
    //  if all processes are STOPPED application is STOPPED
    else  if (stopped_processes == (all_processes-ended_processes)){
        state = Application_impl::M_stoppedStateName;
    }
    // if any process is not RUNNING application is PARTIAL_RUNNING
    else if (running_processes < (all_processes-ended_processes))
    {
        state = Application_impl::M_partialRunningStateName;
    }
    


    log_message <<" ApplicationStatusSolver: running_processes:" << running_processes << " stopped processes:"<< 
                stopped_processes<<" failed processes:"<< failed_processes << " all processes:"<< all_processes<< " " << " => consistent status:"<< state <<ends;

    LogMngr::logMessage(INF, MSG_ID_CHECK_CONSISTENCY_APPLICATION, log_message.str().c_str());

    return state;
}
    

} // End namespace PlatformMngt
} // End namespace Cdmw
