/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtEvent.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "SystemMngt/platformlibrary/EventHeaderFactory.hpp"
#include "SystemMngt/platformlibrary/GraphUtility.hpp"
#include "SystemMngt/platformlibrary/LogMngr.hpp"
#include "SystemMngt/platformlibrary/SupervisionEventNotifier.hpp"
#include "SystemMngt/platformlibrary/SynchronisableTaskDef.hpp"
#include "SystemMngt/platformlibrary/Sequencer.hpp"
#include "SystemMngt/platformlibrary/Starter.hpp"
#include "SystemMngt/platformvaluetypes/GraphElement_impl.hpp"
#include "platformapplicationlibrary/ComponentStates.hpp"
#include "platformapplicationlibrary/CcmComponent_impl.hpp"

namespace Cdmw
{
namespace PlatformMngt
{

ComponentInitial::ComponentInitial()
{
}

CdmwPlatformMngt::ComponentStatus ComponentInitial::status()
{
    return CdmwPlatformMngt::COMPONENT_INITIAL;
}

void ComponentInitial::entryAction(
    ComponentStateMachine* stateMachine )
{
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
               "COMPONENT_INITIAL " << stateMachine->getContext()->name());
    
    CcmComponent_impl* component = stateMachine->getContext();
    
    // Log and notify the change of status
    component->set_status_info ( CORBA::string_dup( "" ));
    component->notifyStatusChangeEvent();
}

void ComponentInitial::initialise(ComponentStateMachine* stateMachine,  const CdmwPlatformMngtBase::StartupKind& startup_kind )
throw( CdmwPlatformMngt::IncompatibleStatus )
{
    try
    {
    //stateMachine->getContext()->m_newInitGraph = true;
        stateMachine->getContext()->m_startup_kind = new CdmwPlatformMngtBase::StartupKind( startup_kind );

        // Change to the state Initialising
        stateMachine->getContext()->setState( CcmComponent_impl::M_initialisingStateName.c_str() );
    }
    catch( const std::bad_alloc& )
    {
        throw CORBA::NO_MEMORY( OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO );
    }
}

void ComponentInitial::set_failed_error(ComponentStateMachine* stateMachine )
        throw( CdmwPlatformMngt::IncompatibleStatus )
{
           // Change to the state Stopping
          stateMachine->setState( CcmComponent_impl::M_failedErrorStateName );
}

void ComponentInitial::set_stopping(ComponentStateMachine* stateMachine )
             throw( CdmwPlatformMngt::IncompatibleStatus )
{
};




ComponentConfiguring::ComponentConfiguring()
{
}

CdmwPlatformMngt::ComponentStatus ComponentConfiguring::status()
{
    return CdmwPlatformMngt::COMPONENT_CONFIGURING;
}

void ComponentConfiguring::entryAction(
    ComponentStateMachine* stateMachine )
{
    // NEVER HERE
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
               "COMPONENT_CONFIGURING " << stateMachine->getContext()->name());
    
    CcmComponent_impl* component = stateMachine->getContext();
    
    // Log and notify the change of status
    component->set_status_info ( CORBA::string_dup( "" ));
    component->notifyStatusChangeEvent();
}


ComponentConfigured::ComponentConfigured()
{
}

CdmwPlatformMngt::ComponentStatus ComponentConfigured::status()
{
    return CdmwPlatformMngt::COMPONENT_CONFIGURED;
}

void ComponentConfigured::entryAction(
    ComponentStateMachine* stateMachine )
{
    // NEVER HERE
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
               "COMPONENT_CONFIGURED " << stateMachine->getContext()->name());
    
    CcmComponent_impl* component = stateMachine->getContext();
    
    // Log and notify the change of status
    component->set_status_info ( CORBA::string_dup( "" ));
    component->notifyStatusChangeEvent();
}


ComponentInitialising::ComponentInitialising()
{
}

CdmwPlatformMngt::ComponentStatus ComponentInitialising::status()
{
    return CdmwPlatformMngt::COMPONENT_INITIALISING;
}

void ComponentInitialising::entryAction(
    ComponentStateMachine* stateMachine )
{
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
               "COMPONENT_INITIALISING "  << stateMachine->getContext()->name());

    CcmComponent_impl* component = stateMachine->getContext();

    // Log and notify the change of status
    component->set_status_info ( CORBA::string_dup( "" ));
    component->notifyStatusChangeEvent();
}

void ComponentInitialising::exitAction(
    ComponentStateMachine* stateMachine )
{
    //stateMachine->getContext()->m_newInitGraph = false;
}

void ComponentInitialising::doActivity(
    ComponentStateMachine* stateMachine )
{
    //Do component initialisation
    bool result = false;
    try
    {
        result = stateMachine->getContext()->doInitialisation();
    
        switch (result)
        {
            case true:
        // Change to the state Initialised
        stateMachine->getContext()->setState( CcmComponent_impl::M_initialisedStateName.c_str() );
            break;
            
            case false:
            // Change to the state FailedTimeout
            stateMachine->getContext()->setState( CcmComponent_impl::M_failedTimeoutStateName.c_str() );
            CDMW_WARN(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
               "Timeout expired on Component Initialisation : "  << stateMachine->getContext()->name());

            break;
        }
    }
    catch(const CORBA::SystemException& ex)
    {
        // Change to the state FailedError
        stateMachine->getContext()->setState( CcmComponent_impl::M_failedErrorStateName.c_str() );
    } 
    
}

void ComponentInitialising::set_failed_error(ComponentStateMachine* stateMachine )
        throw( CdmwPlatformMngt::IncompatibleStatus )
{
           // Change to the state FailedError
          stateMachine->setState( CcmComponent_impl::M_failedErrorStateName );
}

void ComponentInitialising::set_stopping(ComponentStateMachine* stateMachine )
             throw( CdmwPlatformMngt::IncompatibleStatus )
{
   stateMachine->getContext()->setState( CcmComponent_impl::M_stoppingStateName.c_str());
}

void ComponentInitialising::set_stop(ComponentStateMachine* stateMachine )
             throw( CdmwPlatformMngt::IncompatibleStatus )
{
   stateMachine->getContext()->setState( CcmComponent_impl::M_stoppedStateName.c_str());
};             

ComponentInitialised::ComponentInitialised()
{
}

CdmwPlatformMngt::ComponentStatus ComponentInitialised::status()
{
    return CdmwPlatformMngt::COMPONENT_INITIALISED;
}

void ComponentInitialised::entryAction(
    ComponentStateMachine* stateMachine )
{
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
               "COMPONENT_INITIALISED " << stateMachine->getContext()->name());
    
    CcmComponent_impl* component = stateMachine->getContext();
    
    // Log and notify the change of status
    component->set_status_info ( CORBA::string_dup( "" ));
    component->notifyStatusChangeEvent();
}

void ComponentInitialised::set_stopping(ComponentStateMachine* stateMachine )
         throw( CdmwPlatformMngt::IncompatibleStatus )
 {
            // Change to the state Stopping
           stateMachine->getContext()->setState( CcmComponent_impl::M_stoppingStateName.c_str() );
 };

void ComponentInitialised::set_stop(ComponentStateMachine* stateMachine )
             throw( CdmwPlatformMngt::IncompatibleStatus )
{
   stateMachine->getContext()->setState( CcmComponent_impl::M_stoppedStateName.c_str());
};             
  
 void ComponentInitialised::set_failed_error(ComponentStateMachine* stateMachine )
         throw( CdmwPlatformMngt::IncompatibleStatus )
 {
            // Change to the state Stopping
           stateMachine->getContext()->setState( CcmComponent_impl::M_failedErrorStateName.c_str() );
 };
 

ComponentStopping::ComponentStopping()
{
}

CdmwPlatformMngt::ComponentStatus ComponentStopping::status()
{
    return CdmwPlatformMngt::COMPONENT_STOPPING;
}

void ComponentStopping::entryAction(
    ComponentStateMachine* stateMachine )
{
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
             "COMPONENT_STOPPING "<< stateMachine->getContext()->name());
    
    CcmComponent_impl* component = stateMachine->getContext();
    
    // Log and notify the change of status
    component->set_status_info ( CORBA::string_dup( "" ));
    component->notifyStatusChangeEvent();
}

void ComponentStopping::set_stop(ComponentStateMachine* stateMachine )
         throw( CdmwPlatformMngt::IncompatibleStatus )
{
    // Change to the state Stopping
    stateMachine->getContext()->setState( CcmComponent_impl::M_stoppedStateName.c_str() );
}

ComponentStopped::ComponentStopped()
{
}

CdmwPlatformMngt::ComponentStatus ComponentStopped::status()
{
    return CdmwPlatformMngt::COMPONENT_STOPPED;
}

void ComponentStopped::entryAction(
    ComponentStateMachine* stateMachine )
{
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
             "COMPONENT_STOPPED " << stateMachine->getContext()->name());
    
    CcmComponent_impl* component = stateMachine->getContext();
    
    // Log and notify the change of status
    component->set_status_info ( CORBA::string_dup( "" ));
    component->notifyStatusChangeEvent();
}

void ComponentStopped::initialise(ComponentStateMachine* stateMachine,  const CdmwPlatformMngtBase::StartupKind& startup_kind )
throw( CdmwPlatformMngt::IncompatibleStatus )
{
    try
    {
        //stateMachine->getContext()->m_newInitGraph = true;
        stateMachine->getContext()->m_startup_kind = new CdmwPlatformMngtBase::StartupKind( startup_kind );

        // Change to the state Initialising
        stateMachine->getContext()->setState( CcmComponent_impl::M_initialisingStateName.c_str() );
    }
    catch( const std::bad_alloc& )
    {
        throw CORBA::NO_MEMORY( OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO );
    }
}

ComponentFailedError::ComponentFailedError()
{
}

CdmwPlatformMngt::ComponentStatus ComponentFailedError::status()
{
    return CdmwPlatformMngt::COMPONENT_FAILED_ERROR;
}

void ComponentFailedError::entryAction(
    ComponentStateMachine* stateMachine )
{
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
               "COMPONENT_FAILED_ERROR " << stateMachine->getContext()->name());
    
    CcmComponent_impl* component = stateMachine->getContext();
    
    // Log and notify the change of status
    component->set_status_info ( CORBA::string_dup( "" ));
    component->notifyStatusChangeEvent();
}

void ComponentFailedError::initialise(ComponentStateMachine* stateMachine,  const CdmwPlatformMngtBase::StartupKind& startup_kind )
throw( CdmwPlatformMngt::IncompatibleStatus )
{
    try
    {
        //stateMachine->getContext()->m_newInitGraph = true;
        stateMachine->getContext()->m_startup_kind = new CdmwPlatformMngtBase::StartupKind( startup_kind );

        // Change to the state Initialising
        stateMachine->getContext()->setState( CcmComponent_impl::M_initialisingStateName.c_str() );
    }
    catch( const std::bad_alloc& )
    {
        throw CORBA::NO_MEMORY( OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO );
    }
}

void ComponentFailedError::set_failed_error(ComponentStateMachine* stateMachine )
        throw( CdmwPlatformMngt::IncompatibleStatus )
{
           // Nothing to do
}

ComponentFailedTimeout::ComponentFailedTimeout()
{
}

CdmwPlatformMngt::ComponentStatus ComponentFailedTimeout::status()
{
    return CdmwPlatformMngt::COMPONENT_FAILED_TIMEOUT;
}

void ComponentFailedTimeout::entryAction(
    ComponentStateMachine* stateMachine )
{
    // NEVER HERE
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
               "COMPONENT_FAILED_TIMEOUT " << stateMachine->getContext()->name());
    
    CcmComponent_impl* component = stateMachine->getContext();
    
    // Log and notify the change of status
    component->set_status_info ( CORBA::string_dup( "" ));
    component->notifyStatusChangeEvent();
}

void ComponentFailedTimeout::initialise(ComponentStateMachine* stateMachine,  const CdmwPlatformMngtBase::StartupKind& startup_kind )
throw( CdmwPlatformMngt::IncompatibleStatus )
{
    try
    {
        //stateMachine->getContext()->m_newInitGraph = true;
        stateMachine->getContext()->m_startup_kind = new CdmwPlatformMngtBase::StartupKind( startup_kind );

        // Change to the state Initialising
        stateMachine->getContext()->setState( CcmComponent_impl::M_initialisingStateName.c_str() );
    }
    catch( const std::bad_alloc& )
    {
        throw CORBA::NO_MEMORY( OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO );
    }
}

void ComponentFailedTimeout::set_failed_error(ComponentStateMachine* stateMachine )
        throw( CdmwPlatformMngt::IncompatibleStatus )
{
           // Nothing to do
}

ComponentFailedInvalid::ComponentFailedInvalid()
{
}

CdmwPlatformMngt::ComponentStatus ComponentFailedInvalid::status()
{
    return CdmwPlatformMngt::COMPONENT_FAILED_INVALID;
}

void ComponentFailedInvalid::entryAction(
    ComponentStateMachine* stateMachine )
{
    // NEVER HERE
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
               "COMPONENT_FAILED_INVALID " << stateMachine->getContext()->name());
    
    CcmComponent_impl* component = stateMachine->getContext();
    
    // Log and notify the change of status
    component->set_status_info ( CORBA::string_dup( "" ));
    component->notifyStatusChangeEvent();
}


} // End namespace PlatformMngt
} // End namespace Cdmw


