/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "Foundation/common/Options.hpp"
#include <Foundation/common/Locations.hpp>
#include "Foundation/common/String.hpp"
#include "Foundation/common/Exception.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/common/Locations.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"

#include "Foundation/commonsvcs/datastore/StorageErrorException.hpp"
#include <Foundation/logging/FunctionLogger.hpp>
#include <Foundation/logging/Logger.hpp>

#include "tao/AnyTypeCode/TypeCode.h"
#include "platformapplicationlibrary/Process_impl.hpp"
#include "platformapplicationlibrary/ManagedProcess_impl.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtDaemon.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtEvent.stub.hpp"
#include "SystemMngt/platforminterface/PlatformInterface.hpp"
#include "SystemMngt/platforminterface/ServiceNames.hpp"
#include "SystemMngt/platformlibrary/Configuration.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "SystemMngt/platformlibrary/Configuration.hpp"
#include "SystemMngt/platformlibrary/EventHeaderFactory.hpp"
#include "SystemMngt/platformlibrary/LogMngr.hpp"
#include "SystemMngt/platformlibrary/SupervisionEventNotifier.hpp"
#include "SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp"
#include "SystemMngt/platformapplicationlibrary/Application_impl.hpp"
#include "SystemMngt/platformapplicationlibrary/ApplicationLogMessageId.hpp"
#include "SystemMngt/platformapplicationlibrary/HostProcessInitSequencer.hpp"
#include "platformapplicationlibrary/CcmComponent_impl.hpp"
#include "SystemMngt/platformlibrary/EventNotifierManager.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"
#include "SystemMngt/platformlibrary/ComponentsIterator_impl.hpp"
#include "SystemMngt/tools/ManagedElementKey.hpp"

#include "platformapplicationlibrary/CcmComponent_impl.hpp"
#include "platformapplicationlibrary/CommandCallback.hpp"
#include "SystemMngt/tools/HostIssuerManager.hpp"
#include "SystemMngt/tools/ApplicationIssuerManager.hpp"
#include "SystemMngt/tools/ProcessIssuerManager.hpp"

#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"
#include "Repository/repositoryinterface/RepositoryInterface.hpp"
#include <set>
#include <string>
#include <iostream>
#include <sstream>
#include <map>
#include <math.h>
#include <ctype.h>
#include <bitset>

using namespace Cdmw::OsSupport;
using namespace Cdmw::OrbSupport;
using namespace Cdmw::PlatformMngt;
using namespace Cdmw;
using namespace Common;
using namespace PlatformMngt;
using namespace CdmwPlatformMngt;


typedef unsigned long ulong;

//define multi-map for the filtering to apply
typedef std::pair<std::string,std::string> filter_key;



namespace
{
    // some useful MACRO to set all levels
    const ulong FULL_ONE = (ulong)(-1);
    const ulong FULL_ZERO = 0x0;

    /*
      Only for DEBUG scope
     */
    void dump_map(const filters_map& tmp_map,std::ostream& os)
    {
        filters_map::const_iterator
            cit  = tmp_map.begin(),
            cend = tmp_map.end();

        for( ;cit != cend ; ++cit )
        {
            filter_key k = cit->first;
            os<<"["<<k.first<<","<<k.second<<"]= "<<std::hex<<cit->second<<std::dec<<endl;
        }
    }

    void print_map(filters_map& map_f)
    {
        std::string str = " , " ;
        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),  "[ STARTMAP ]" );
        std::cerr << "[ STARTMAP ]";
        for (filters_map::iterator it = map_f.begin(); it!=map_f.end();it++ )
        {
        filter_key k = it->first;
            CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
        "[ KEY ] = " <<  k.first << " , " <<  k.second << "]= " <<  std::hex<<it->second<<std::dec );
//            CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
//        " , " <<  k.second );
//            CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
//        "]= " <<  std::hex<<it->second<<std::dec );
//            std::cerr << "[ KEY ] = "<<  k.first<<std::endl;
        }
        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),  "[ ENDMAP ]\n" );
        std::cerr << "[ ENDMAP ]\n";
    }




    bool my_isalnum(const std::string& s,
                  std::vector<char> chtoaccept)
    {
        int size = s.length();
        const char* p = s.c_str();
        bool accepted_char;
        for(int i=0;i<size;++i)
        {
            const char &c = *p;
            if( !isalnum((int)c) )
            {
                accepted_char = false;
                for( unsigned int j = 0; j < chtoaccept.size(); j++ ) {
                    if( c==chtoaccept[j] )
                    accepted_char = true;
                    ++p;
                }
                if ( !(accepted_char) )
                        return false;
             }
        }
        return true;
    }

    bool  my_isdigit(const std::string& s)
    {
        int size = s.length();
        const char* p = s.c_str();
        for(int i=0;i<size;++i)
        {
            const char &c = *p;

            if(!isdigit(c))
                return false;
            ++p;
        }
        return true;
    }

    bool  my_isxdigit(const std::string& s)
    {
        int size = s.length();
        const char* p = s.c_str();
        for(int i=0;i<size;++i)
        {
            const char &c = *p;

            if(!isxdigit(c))
                return false;
            ++p;
        }
        return true;
    }



    ulong
    pow2(int exp)
    {
        ulong ref = 0x1;

        if(exp < 0 )
            return 0;

        ref<<=exp;
        return ref;
    }


    string strip_leading_and_trailing_blanks(const string& s)
    {
        string out_s;

        if(!s.empty())
        {
            string::size_type startIndex = s.find_first_not_of(" ");
            string::size_type endIndex = s.find_last_not_of(" ");

            if(startIndex != string::npos )
                out_s = s.substr(startIndex, (endIndex-startIndex+ 1) );
            else
                throw CdmwPlatformMngt::ParsingException("Error: a field is totally blank!");
        }
        return out_s;
    }


    void extract(filter_key& fk,
                 ulong& level,
                 const string& s)
    {
        // delimiter to identify the subfields
        const char *delimiter = ",";

        // max size of used buffer
        const int max_size = 1024;

        if( s.size() > (max_size -1 ))
            throw CdmwPlatformMngt::ParsingException("Error: the string is too long!");

        char
            buff[max_size],
            tmp_buff[max_size];

        strcpy(buff,s.c_str());

        // two useful alias
        string
            &component = fk.first,
            &domain    = fk.second;
        string
            stringified_lev_value;

        char* trace_filter_tokenized = buff;
        char* trace_filter_token = strtok_r( trace_filter_tokenized, delimiter, (char**)&tmp_buff);

        std::vector<char> char_to_accept;
        char_to_accept.push_back( '_' );
        char_to_accept.push_back( '-' );
        char_to_accept.push_back( '.' );
        char_to_accept.push_back( '+' );

        ulong
            &lev_value = level;

        std::vector <string> tokens;

        //insert the token inside a vector
        while( trace_filter_token != NULL )
        {
            string str_trace_filter_token( trace_filter_token );

            str_trace_filter_token = strip_leading_and_trailing_blanks( str_trace_filter_token );

            if(!str_trace_filter_token.empty())
            {
                tokens.push_back(  str_trace_filter_token );
            }
            trace_filter_token = strtok_r( NULL, delimiter, (char**)&tmp_buff );
        }

        int tokens_size = tokens.size();

        if ( tokens_size != 3)
            throw  CdmwPlatformMngt::ParsingException("Error: no enough fields filled...");

        //insert the element in tha internal map ...
        string level_indicator = tokens.back();
        std::istringstream istr_level_indicator( level_indicator );

        // if there is a ! reset of levels
        if ( 0 == level_indicator.compare("!") )
        {
            lev_value=FULL_ZERO;
        }
        else
        {
            // if there is a * set all levels
            if ( 0 == level_indicator.compare("*") )
            {
                lev_value=FULL_ONE;
            }
            else
            {
                // is the value hex?
                if (0 == level_indicator.compare(0, 2, "0x"))
                {
                    istr_level_indicator.str(level_indicator.substr(2));
                    const string &tmp = istr_level_indicator.str();
                    if(!my_isxdigit(tmp))
                          throw CdmwPlatformMngt::ParsingException("Error: input is not an hexadecimal number!");

                    // insert it in lev_value
                    istr_level_indicator >> std::hex >> lev_value;
                    if (istr_level_indicator.fail())
                    {
                        throw CdmwPlatformMngt::ParsingException(
                            "Error in a input level value, there is an invalid character!");
                    }
                }
                else
                {
                    // if it is not
                    if(!my_isdigit(istr_level_indicator.str()))
                          throw CdmwPlatformMngt::ParsingException("Error: input is not an decimal number!");

                    int exponent=0;
                    istr_level_indicator >> std::dec >> exponent;
                    if (istr_level_indicator.fail())
                    {
                        throw CdmwPlatformMngt::ParsingException(
                            "Error in a input level value, there is an invalid character!");
                    }

                    //here exponent is decimal
                    if ( ( 0 > exponent) || (exponent >31))
                    {
                        throw CdmwPlatformMngt::ParsingException("Error: a level must be inside the range [0,31]");
                    }
                    lev_value = pow2 ( exponent );
                }
            }
        }

        // I'm sure size is 3
        domain =  tokens[ tokens_size-2 ];

        bool v1,v2,v3;

        v1 = (0 == domain.compare("*"));
        v2 = my_isalnum(domain, char_to_accept);
//        v4 = my_isalnum(domain,'-');
        // check for wrong parameters in domain
        if( !( v1 || v2 ) )
            throw CdmwPlatformMngt::ParsingException("Error: the domain has wrong name!");;

        component =  tokens[ tokens_size-3 ] ;

        v1 = (0 == component.compare("!"));
        v2 = (0 == component.compare("*"));
        v3 = my_isalnum(component, char_to_accept);
//        v5 = my_isalnum(domain,'-');
        // check for wrong parameters in component
        if (!( v3  || (v1 || v2) ) )
            throw CdmwPlatformMngt::ParsingException("Error: the component has wrong name!");;
    }
}




namespace Cdmw
{
    namespace PlatformMngt
    {

        Process_impl::Process_impl (PortableServer::POA_ptr poa,
                    CdmwPlatformMngt::ProcessDef* process_def,
                    const char *process_key,
                    const char *process_path,
                    Application_impl * application,
                    CdmwPlatformMngt::RWElementRepository_ptr element_repository)

            :DeactivableServant_impl(poa),
             ManagedElement_impl(element_repository, poa),
             m_current_restart_attempt(0),
             m_pid(0),
             m_previous_pid(0)
        {
            // only one logger is used
            m_logger = Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG);

            m_poa = PortableServer::POA::_duplicate (poa);

            if (application == NULL)
                CDMW_THROW2 (BadParameterException, "application", "NULL");

            m_procStatusChangeFactory = NULL;
            m_procRestartFactory = 0;
            m_procStopFactory = 0;

            // set in Managed Element the instance of Property Event Notifier Thread
            set_property_event_notifier(EventNotifierManager::Instance()->get_property_notifier());
            // set in Managed Element the instance of Status Event Notifier Thread
            set_status_event_notifier(EventNotifierManager::Instance()->get_status_notifier());

            try
            {
                // Sets the process's internal data
                std::auto_ptr < ProcessStatusChangeFactory >
                procStatusChangeFactory (new ProcessStatusChangeFactory ());
                m_procStatusChangeFactory = procStatusChangeFactory.get ();

                std::auto_ptr < RestartFactory >
                procRestartFactory (new RestartFactory ());
                m_procRestartFactory = procRestartFactory.get ();

                std::auto_ptr < StopFactory >
                procStopFactory (new StopFactory ());
                m_procStopFactory = procStopFactory.get ();

                m_init_mutex=0;
                m_init_condition=0;
                m_timer=0;
                m_component_init_graph= new CdmwPlatformMngt::GraphElements();
                m_application = application;
                m_type = process_def->type ();
                m_autoending = process_def->auto_ending ();
                m_initialisation_timeout = process_def->initialisation_timeout ();
                //set_requested_restart_warmth ( process_def->default_restart_warmth() );

                m_last_exit_info.exit_event=CdmwPlatformMngt::UNKNOWN;
                m_last_exit_info.event_value=0;
                m_last_exit_info.core_dumped=CdmwPlatformMngt::CORE_UNKNOWN;
                m_last_exit_info.process_pid=0;

                m_process_info = new CdmwPlatformMngt::ProcessInfo ();

                CdmwPlatformMngt::ProcessInfo_var m_process_info = new CdmwPlatformMngt::ProcessInfo(process_def->process_info()) ;


                m_lifeCycle = new CdmwPlatformMngt::LifeCycleDef ();
                m_lifeCycle->init_steps.length (0);
                m_lifeCycle->run_timeout = 0;
                m_lifeCycle->stop_timeout = 0;

                m_monitoring_info = new CdmwPlatformMngt::CompleteMonitoringInfo ();
                m_monitoring_info->monitoring_model = CdmwPlatformMngt::NO_MONITORING;
                m_monitoring_info->pull_monitoring_interval = 0;
                m_monitoring_info->pull_monitoring_timeout = 0;
                m_monitoring_info->push_monitoring_interval = 0;
                m_monitoring_info->push_monitoring_timeout = 0;

                set_status_info ( CORBA::string_dup (""));
                m_stepCounter = 0;

                m_host = CdmwPlatformMngt::Host::_nil ();
                // store element path in managed element
                set_element_path (process_path, process_def->name ());

                // memo the different names : process, host and process key
                m_proc_syst_name = CORBA::string_dup (m_application->get_system_name ());
                m_proc_appli_name = CORBA::string_dup (m_application->get_element_name ());
                m_proc_host_name = CORBA::string_dup (process_def->host_name ());
                m_proc_key = CORBA::string_dup (process_key);

                m_startup_kind = new CdmwPlatformMngtBase::StartupKind ();

                m_created_dynamically = true;
                m_indicator_relevant_status = true;

                m_comp_primary = 0;
                m_comp_passive = 0;
                m_comp_active = 0;
                // Release the ownership of the temporary objects
                procStatusChangeFactory.release ();
                procRestartFactory.release ();
                procStopFactory.release ();

                m_stop_emergency = false;                
        
        std::string element_path = std::string(get_element_path());
                Cdmw::PlatformMngt::ProcessIssuerManager::Instance()->attach_element(element_path);
        
                //host element path for supervision commands
                m_host_issuer = get_sup_host_path();

            }
            catch (const std::bad_alloc &)
            {
                CDMW_THROW (OutOfMemoryException);
            }

            try
            {
                Cdmw::CommonSvcs::DataStore::DataStoreBase * ds =
                Cdmw::CommonSvcs::DataStore::DataStoreRegistry::
                Get_data_store (Configuration::Get_process_datastore_id ());
                m_process_ds =
                dynamic_cast < Cdmw::PlatformMngt::ProcessDataStore * >(ds);

            }
            catch (const NotExistException & e)
            {
                CDMW_ASSERT (false);
            }
        }


        Process_impl::~Process_impl ()
        {
            if (m_procStatusChangeFactory != NULL)
            {
                m_procStatusChangeFactory->_remove_ref ();
            }

            if (m_procRestartFactory != NULL)
            {
                m_procRestartFactory->_remove_ref ();
            }

            if (m_procStopFactory != NULL)
            {
                m_procStopFactory->_remove_ref ();
            }

        }

            void Process_impl::remove()
            {
                std::string element_path = std::string (get_element_path());
                Cdmw::PlatformMngt::ProcessIssuerManager::Instance()->detach_element(element_path);
        
                // first increment counter reference to avoid the destructor calling
                // by servant deactivation
                this->_add_ref();

                // deactivate the Corba object
                this->deactivate();

                // remove associated process record in datastore
                remove_record_in_datastore();

                // remove the associated managed element
                this->ManagedElement_impl::remove();

                // then decrement the counter reference to call destructor
                this->_remove_ref();
        }

        void Process_impl::set_record_in_datastore ()
        {
            // test if the supervision is not fault tolerant or if it is, is it the primary?
            if (!Configuration::Is_fault_tolerant ()
                || Configuration::Is_primary ())
            {
                if (Configuration::Is_synchronising ())
                {
                    CDMW_INFO(m_logger,
                    "<<Process_impl::set_record_in_datastore() start >> "
                    << m_element_name.in () );
                }
                
                CdmwPlatformMngt::ProcessRecord record;
                record.host_name = m_proc_host_name;
                record.type = m_type;
                record.initialisation_timeout = m_initialisation_timeout;
                record.auto_ending = m_autoending;
                record.process_info = m_process_info.in ();
        
                record.life_cycle = m_lifeCycle.in ();
                record.monitoring_info = m_monitoring_info.in ();
                record.status = this->getStatus ();
        
                record.previous_state_name =  getPreviousStatus() ;
                record.status_info = get_status_info();
                        record.state_machine = CORBA::string_dup (this->get_state_machine_state_name ());
                record.step_counter = m_stepCounter;
                record.last_request_status = m_lastRequestStatus;
                record.last_error_header = m_lastErrorHeader;
                record.last_error_issuer =
                CORBA::string_dup (m_lastErrorIssuer.c_str ());
                record.last_exit_code = m_last_exit_info;
                record.current_restart_attempt = m_current_restart_attempt;
                record.startup_kind = m_startup_kind.in ();
                record.created_dynamically = m_created_dynamically;
                record.indicator_relevant_status =  m_indicator_relevant_status;
                record.component_init_graph = m_component_init_graph;
        
                switch(m_kind_of_start)
                {
                    case DEFAULT :
                    record.kind_start = CdmwPlatformMngt::DEFA;
                    break;
        
                    case ADDED:
                    record.kind_start = CdmwPlatformMngt::ADDED;
                    break;
        
                    case DEFAULT_REMOVED:
                    record.kind_start = CdmwPlatformMngt::DEFAULT_REMOVED;
                    break;
        
                }
    
                string ds_operation("contains");            
                try
                {
        
                    if (m_process_ds->contains (m_element_path.in ()))
                    {
                        ds_operation = "update";
                        m_process_ds->update (m_element_path.in (), record);
                    }
                    else
                    {
                        set_element_in_datastore ();
                        ds_operation = "insert";
                        m_process_ds->insert (m_element_path.in (), record);
                    }
                }
                catch (const Common::AlreadyExistException&)
                {
                    CDMW_WARN(m_logger,
                    "record already exists exception in " 
                    << ds_operation << " : " 
                    << m_element_path.in () << ":"
                    << (const char*) record.status);
                }
                catch (const CommonSvcs::DataStore::StorageErrorException &exc_storage)
                {
                    CDMW_FATAL(m_logger,
                    "record storage error exception in "
                    << ds_operation <<" : " 
                    << m_element_path.in() << ":"
                    << (const char*) record.status << ":" 
                    << exc_storage.what());
                    CDMW_ASSERT (false);
                }
                catch (const Common::NotFoundException&)
                {
                CDMW_FATAL(m_logger,
                "record cannot be found in datastore in "
                << ds_operation <<" : " 
                << m_element_path.in());
                CDMW_ASSERT (false);
                }
                catch (...)
                {
                    CDMW_FATAL(m_logger,
                    "record unknown exception in "
                    << ds_operation << " : " 
                    << m_element_path.in() << ":"
                    << (const char*) record.status);
                    CDMW_ASSERT (false);
                }
                    
                if (Configuration::Is_synchronising ())
                {
                    CDMW_INFO(m_logger,
                    "<<Process_impl::set_record_in_datastore() done >> "
                    << m_element_name.in () );
                }
            }
    
            m_status_in_ds = this->getStatus ();
        }


        void Process_impl::remove_record_in_datastore ()
        {
            // inhibition if backup
            if (!Configuration::Is_fault_tolerant ()
                || Configuration::Is_primary ())
            {
                try
                {
                m_process_ds->remove (m_element_path.in ());
                }
                catch (const Common::NotFoundException &)
                {
                }
                catch (const CommonSvcs::DataStore::StorageErrorException &)
                {   
                }
            }
        }

        CdmwPlatformMngt::ProcessRecord *
        Process_impl::get_record_from_datastore ()
        {
        CdmwPlatformMngt::ProcessRecord * process_record = NULL;

        try
        {
        process_record = m_process_ds->select (m_element_path.in ());
        }
        catch (const Common::NotFoundException &)
        {
            CDMW_FATAL(m_logger,
                        "process record cannot be found in datastore : "
                        << m_element_path.in());
            CDMW_ASSERT (false);
        }

        return process_record;
        }


        void Process_impl::synchro_with_datastore ()
        {

        Cdmw::PlatformMngt::ManagedElement_impl::synchro_with_datastore ();
        CdmwPlatformMngt::ProcessRecord_var process_record = get_record_from_datastore ();

        //map from process_record->previous_stat to string

        string prev_state_str;

        if ( process_record->previous_state_name == CdmwPlatformMngt::PROCESS_INITIAL )
            prev_state_str = "Initial";

        if ( process_record->previous_state_name == CdmwPlatformMngt::PROCESS_INITIALISING)
            prev_state_str = "Initialising";

        if ( process_record->previous_state_name == CdmwPlatformMngt::PROCESS_STEP_PERFORMED)
            prev_state_str = "StepPerformed";

        if ( process_record->previous_state_name == CdmwPlatformMngt::PROCESS_INITIALISED)
            prev_state_str = "Initialised";

        if ( process_record->previous_state_name == CdmwPlatformMngt::PROCESS_RUN_REQUEST)
            prev_state_str = "RunRequest";

        if ( process_record->previous_state_name == CdmwPlatformMngt::PROCESS_RUNNING)
            prev_state_str = "Running";

        if ( process_record->previous_state_name == CdmwPlatformMngt::PROCESS_STOPPING)
            prev_state_str = "Stopping";

        if ( process_record->previous_state_name == CdmwPlatformMngt::PROCESS_STOPPED)
            prev_state_str = "Stopped";

        if ( process_record->previous_state_name == CdmwPlatformMngt::PROCESS_ENDED)
            prev_state_str = "Ended";

        if ( process_record->previous_state_name == CdmwPlatformMngt::PROCESS_FAILED_ERROR)
            prev_state_str = "FailedError";

        if ( process_record->previous_state_name == CdmwPlatformMngt::PROCESS_FAILED_NO_RESPONSE)
            prev_state_str = "FailedMonitoringFailure";

        if ( process_record->previous_state_name == CdmwPlatformMngt::PROCESS_FAILED_DEATH)
            prev_state_str = "FailedDeath";

        if ( process_record->previous_state_name == CdmwPlatformMngt::PROCESS_FAILED_DISCREPANCY)
            prev_state_str = "FailedDiscrepancy";

        if ( process_record->previous_state_name == CdmwPlatformMngt::PROCESS_FAILED_TIMEOUT)
            prev_state_str = "FailedTimeout";

        if ( process_record->previous_state_name == CdmwPlatformMngt::PROCESS_FAILED_INVALID)
            prev_state_str = "FailedInvalid";

        //to set the machine previous state .
        set_state_machine_previous_state_name( (prev_state_str).c_str() ) ;

        m_autoending = process_record->auto_ending;
        m_process_info = new CdmwPlatformMngt::ProcessInfo (process_record->process_info);
        m_lifeCycle = new CdmwPlatformMngt::LifeCycleDef (process_record->life_cycle);
        m_monitoring_info = new CdmwPlatformMngt::CompleteMonitoringInfo (process_record->monitoring_info);
        set_status_info ( process_record->status_info);
        m_stepCounter = process_record->step_counter;
        m_lastRequestStatus = process_record->last_request_status;
        m_lastErrorHeader = process_record->last_error_header;
        m_lastErrorIssuer = (process_record->last_error_issuer).in ();
        m_startup_kind = new CdmwPlatformMngtBase::StartupKind (process_record->startup_kind);
        m_created_dynamically = process_record->created_dynamically;


        m_status_in_ds = process_record->status;
        m_last_exit_info = process_record->last_exit_code;
        m_current_restart_attempt = process_record->current_restart_attempt;
        m_indicator_relevant_status = process_record->indicator_relevant_status;
        *m_component_init_graph = process_record->component_init_graph;
        
        this->set_state_machine_state_name (process_record->state_machine.
                          in ());

        //Synchronize the entities of process
        synchro_entities_with_datastore();

          //update the map of processes ...

          Cdmw::PlatformMngt::ProcessHostInfo info;
          info.process_name = CORBA::string_dup (name());
          info.process_impl  = this;
          switch(process_record->kind_start)
            {
        case CdmwPlatformMngt::DEFA:
            info.kind_of_start=Cdmw::PlatformMngt::DEFAULT;
            break;
        case CdmwPlatformMngt::ADDED:
            info.kind_of_start=Cdmw::PlatformMngt::ADDED;
            break;
        case CdmwPlatformMngt::DEFAULT_REMOVED:
            info.kind_of_start=Cdmw::PlatformMngt::DEFAULT_REMOVED;
            break;
        }


          ManagedProcess_impl *manag = dynamic_cast<ManagedProcess_impl *> (this);

          if ( !manag)
            { //case of unmanaged process
             set_proc_info(info, 2);
        }
          else
            { //case of managed process
              set_proc_info(info, 0);
        }
        }

        void Process_impl::synchro_components_with_datastore ()
        {
         Cdmw::PlatformMngt::ManagedElement_impl::synchro_with_datastore ();

         //Synchronising all the components.
         for (ComponentsMap::iterator it = m_component_map.begin(); it !=m_component_map.end(); ++it)
             {
                (it->second)->synchro_with_datastore();
             }

        }

        void Process_impl::synchro_processing ()
        {
        }

        void Process_impl::synchro_entities_with_datastore()
    {
        }
    
        void Process_impl::monitoringFailure_event (const CdmwPlatformMngtBase::
                            EventHeader & header)
        throw (CdmwPlatformMngt::IncompatibleType, CORBA::SystemException)
        {
        throw CdmwPlatformMngt::IncompatibleType ();
        }


        void Process_impl::property_change_event ( const char* attr_name,
                const CORBA::Any& prop_value,
                const char* component_name)
        throw (CORBA::SystemException, CdmwPlatformMngt::ComponentNotFound)
        {
            string property_name (attr_name);
            string comp_name (component_name);

            CdmwPlatformMngt::RedundancyEltState value;
            prop_value >>= value;

            if (!property_name.compare(CdmwPlatformMngt::REPLICATION_ROLE))
            {
                CdmwPlatformMngt::Owner flag = CdmwPlatformMngt::PRE_DEFINED;

                /******************************
                 * Get the component reference
                 *****************************/
                std::string component_key =
                    ManagedElementKey::Get_component_rel_key(get_process_host_name(),
                            name(),
                            component_name);

                ComponentsMap::iterator componentIt = m_component_map.find (component_key);

                //if component is found
                if (componentIt != m_component_map.end ())
                {
                    try
                    {

                        // set a component property
                        (componentIt->second)-> set_attribute_internal(CdmwPlatformMngt::REPLICATION_ROLE,
                                prop_value, flag);
                    }
                    catch (const Cdmw::BadOrderException& ex)
                    {
                        throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
                    }
                    catch (...)
                    {
                        throw;
                    }
                }
                else
                {
                    throw CdmwPlatformMngt::ComponentNotFound();
                }


                /**********************************************
                 * Check if need to set also process property
                 * ********************************************/
                if (CdmwPlatformMngt::REDSTATE_PRIMARY == value)
                {
                    CDMW_INFO(m_logger,
                            "ReplicationRole Property Changed to REDSTATE_PRIMARY for component : "<< comp_name);
                    m_comp_primary++;

                    //check if all ft components are primary
                   if (get_number_of_ft_components() == m_comp_primary )
                    {
                        CDMW_INFO(m_logger,
                                "ReplicationRole Property Changed to REDSTATE_PRIMARY for process : "<< name());
                        try
                        {

                            // set a process property only if all its
                            // componets have the same value of property
                            set_attribute_internal (CdmwPlatformMngt::REPLICATION_ROLE, prop_value, flag);
                            m_comp_primary = 0;
                        }
                        catch (const Cdmw::BadOrderException& ex)
                        {
                            throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
                        }
                        catch (...)
                        {
                            throw;
                        }
                    }
                }
                else if (CdmwPlatformMngt::REDSTATE_PASSIVE == value)
                {
                    CDMW_INFO(m_logger,
                            "ReplicationRole Property Changed to REDSTATE_PASSIVE for component : "<< comp_name);
                    m_comp_passive++;

                    //check if all ft components are passive
                    if (get_number_of_ft_components() == m_comp_passive )
                    {
                        try
                        {
                            CDMW_INFO(m_logger,
                                    "ReplicationRole Property Changed to REDSTATE_PASSIVE for process : "<< name());
                            // set a process property only if all its
                            // componets have the same value of property
                            set_attribute_internal (CdmwPlatformMngt::REPLICATION_ROLE, prop_value, flag);
                            m_comp_passive = 0;
                        }
                        catch (const Cdmw::BadOrderException& ex)
                        {
                            throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
                        }
                        catch (...)
                        {
                            throw;
                        }
                    }
                }
                else if (CdmwPlatformMngt::REDSTATE_ACTIVE  == value)
                {
                    CDMW_INFO(m_logger,
                            "ReplicationRole Property Changed to REDSTATE_ACTIVE for component : "<< comp_name);
                    m_comp_active++;

                    //check if all lb component's properties are set
                    if (get_number_of_lb_components() == m_comp_active )
                    {
                        try
                        {
                            CDMW_INFO(m_logger,
                                    "ReplicationRole Property Changed to REDSTATE_ACTIVE for process : "<< name());
                            // set a process property only if all its
                            // componets have the same value of property
                            set_attribute_internal (CdmwPlatformMngt::REPLICATION_ROLE, prop_value, flag);
                            m_comp_active = 0;
                        }
                        catch (const Cdmw::BadOrderException& ex)
                        {
                            throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
                        }
                        catch (...)
                        {
                            throw;
                        }
                    }
                }
            }
        }

        /*
        *Purpose:
        *<p> reset a replication role property for process
        *    and for its components ;
        */
        void Process_impl::reset_replication_role()
        throw(CORBA::SystemException)
        {
            CORBA::Any prop_value;
            CdmwPlatformMngt::RedundancyEltState value = CdmwPlatformMngt::REDSTATE_NORG;
            prop_value <<= value;

            CDMW_INFO(m_logger,
                    "ReplicationRole Property Changed to REDSTATE_NORG for process : "<< name()<<" and its components");
            try
            {
                //reset a replication role property for the process
                set_attribute_internal (CdmwPlatformMngt::REPLICATION_ROLE,
                                        prop_value, CdmwPlatformMngt::PRE_DEFINED);

                //reset  a replication role property for all component
                for (ComponentsMap::iterator it = m_component_map.begin(); it !=m_component_map.end(); ++it)
                {
                    (it->second)-> set_attribute_internal(CdmwPlatformMngt::REPLICATION_ROLE,
                                                          prop_value, CdmwPlatformMngt::PRE_DEFINED);
                }
            }
            catch (const Cdmw::BadOrderException& ex)
            {
                throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
            }
            catch (...)
            {
                throw;
            }
        }




        int Process_impl::get_number_of_ft_components()
        throw(CORBA::SystemException)
        {

            int counter = 0;

            try
            {

                for (ComponentsMap::iterator it = m_component_map.begin(); it !=m_component_map.end(); ++it)
                {
                    if ((it->second)-> is_ft_group()) {
                        counter++;
                    }
                }
            }

            catch (...)
            {
                throw;
            }

            return counter;
        }

        int Process_impl::get_number_of_lb_components()
        throw(CORBA::SystemException)
        {

            int counter = 0;

            try
            {

                for (ComponentsMap::iterator it = m_component_map.begin(); it !=m_component_map.end(); ++it)
                {
                    if ((it->second)-> is_lb_group()) {
                        counter++;
                    }
                }
            }

            catch (...)
            {
                throw;
            }

            return counter;
        }


        /*
        *Purpose:
        *<p> it returns the string of trace filters ;
        * Implements the
        * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_status:1.0
        * operation
        */
        char * Process_impl::get_status() throw(CORBA::SystemException)
        {

            return NULL;
        }


        CdmwPlatformMngt::ProcessStatus Process_impl::get_status_in_ds ()
        {
        // return status stored in datastore
        return m_status_in_ds;
        }

        CdmwPlatformMngtBase::StartupKind* Process_impl::get_startup_kind_from_ds ()
        {
        return m_startup_kind;
        }

        CdmwPlatformMngt::ProcessType Process_impl::type ()throw (CORBA::
                                 SystemException)
        {
        return m_type;
        }


        CORBA::Object_ptr Process_impl::embedded_service ()throw (CORBA::SystemException)
        {
        return CORBA::Object::_nil ();
        }


        CdmwPlatformMngt::ProcessPID Process_impl::pid ()
            throw (CORBA::SystemException)
        {
        return m_pid;

        }

        CdmwPlatformMngt::ProcessPID Process_impl::previous_pid ()
            throw (CORBA::SystemException)
        {
        return m_previous_pid;
        }

        CORBA::Boolean Process_impl::created_dynamically ()throw (CORBA::SystemException)
        {
        return m_created_dynamically;
        }


        CORBA::ULong Process_impl::restart_attempt ()throw (CORBA::SystemException)
        {
            return m_current_restart_attempt;
        }

        void Process_impl::set_restart_attempt (CORBA::Long nrestart)
                throw(CORBA::SystemException)
        {
            m_current_restart_attempt = nrestart;
        }


        bool Process_impl::get_run_as_root () throw (CORBA::SystemException)
        {
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
                    CORBA::COMPLETED_NO);
        }

        CdmwPlatformMngtBase::TimeStamp Process_impl::start_time ()throw (CORBA::
                                        SystemException)
        {
        // TODO
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
                    CORBA::COMPLETED_NO);
        }

        CdmwPlatformMngtBase::TimeStamp Process_impl::stop_time ()throw (CORBA::
                                        SystemException)
        {
        // TODO
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
                    CORBA::COMPLETED_NO);
        }


        /**
        *Purpose:
        *<p> facade to get an attribute
        *
        */

        void Process_impl::get_attribute (const char *attr_name, CORBA::Any_out ret_attr, CdmwPlatformMngt::Owner flag)
        throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound)
        {
            std::string tmp1 (attr_name);
            if ( CdmwPlatformMngt::PRE_DEFINED == flag )
            {
                // Case of predefined property
                std::ostringstream property_name;
                property_name << CdmwPlatformMngt::CDMW_PREFIX << attr_name << std::ends;
                try
                {
                    CORBA::Any_var outany = get_property_value ( property_name.str().c_str() );
                    ret_attr = outany._retn();
                }
                catch (const CosPropertyService::PropertyNotFound&)
                {
                    throw CdmwPlatformMngt::AttributeNotFound ();
                }
            }
            else
            {
                /*case of user defined attribute */
                try
                {
                    CORBA::Any * outany = get_property_value (attr_name);
                    ret_attr = outany;
                }
                catch (const CosPropertyService::PropertyNotFound&)
                {
                    throw CdmwPlatformMngt::AttributeNotFound ();
                }
            }
        }



        /**
        *Purpose:
        *<p> facade to set an attribute
        *
        */

        void Process_impl::set_attribute (const char *attr_name, const CORBA::Any & attr, CdmwPlatformMngt::Owner flag)
            throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound, CdmwPlatformMngt::ReadOnlyAttribute)
        {
            CORBA::Any_var myany = new CORBA::Any();
            *myany = attr;

            if ( flag == CdmwPlatformMngt::PRE_DEFINED )
            {
                std::string tmp1 (attr_name);
                // check if the predefined property is readonly
                if ((tmp1 == CdmwPlatformMngt::REQUESTED_RESTART_WARMTH) ||
                    (tmp1 == CdmwPlatformMngt::REPLICATION_ROLE )||
                    (tmp1 == CdmwPlatformMngt::TECH_STATUS) ||
                    (tmp1 == CdmwPlatformMngt::TECH_STATUS_INFO))
                {
                    throw CdmwPlatformMngt::ReadOnlyAttribute();
                }

                if (tmp1 == CdmwPlatformMngt::TRACE_FILTERS)
                {
                    const char * tmp2;
                    attr >>= tmp2;

                    try
                    {
                        set_trace_filtering (tmp2);
                    }
                    catch (const CdmwPlatformMngt::CastException& e)
                    {
                        //throw CdmwPlatformMngt::AttributeNotFound();
                        return;
                    }
                    catch (const CdmwPlatformMngt::ParsingException& e)
                    {
                        CDMW_ERROR(m_logger,
                                  "Exception: " <<e.message );
                        throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
                    }

                    tmp2 = get_trace_filtering ();
                    *myany <<= CORBA::string_dup (tmp2);
                }
            }

            try
            {
                set_attribute_internal ( attr_name, myany.in(), flag );
            }
        catch (const Cdmw::BadOrderException& ex)
            {
            throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
            }
            catch (...)
        {
                throw;
            }
        }


        /*
        CdmwPlatformMngt::ProcessMode Process_impl::
        get_process_mode ()throw (CORBA::SystemException)
        {
        // TODO
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
                    CORBA::COMPLETED_NO);
        }
        */

        bool Process_impl::get_memory_resident () throw (CORBA::SystemException)
        {
        // TODO
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
                    CORBA::COMPLETED_NO);
        }

        void Process_impl::set_memory_resident (bool memory_resident)
        throw (CORBA::SystemException)
        {
        // TODO
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
                    CORBA::COMPLETED_NO);

        }

        CORBA::Long Process_impl::get_core_dump_size ()throw (CORBA::
                                SystemException)
        {
        // TODO
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
                    CORBA::COMPLETED_NO);
        }




        CORBA::Boolean Process_impl::is_auto_ending ()throw (CORBA::
                                SystemException)
        {
        return m_autoending;
        }

        CdmwPlatformMngt::ManagedElement_ptr Process_impl::
        get_parent ()throw (CORBA::SystemException)
        {
        return m_application->_this ();
        }

        CdmwPlatformMngt::Host_ptr Process_impl::get_host ()throw (CORBA::
                                 SystemException)
        {
        CdmwPlatformMngt::Host_var host = m_host;

        return host._retn ();
        }

        CdmwPlatformMngt::ProcessInfo *
        Process_impl::get_process_info ()throw (CORBA::SystemException)
        {
        try
        {
        CdmwPlatformMngt::ProcessInfo_var processInfo
        = new CdmwPlatformMngt::ProcessInfo (m_process_info.in ());

        return processInfo._retn ();
        }
        catch (const std::bad_alloc &)
        {
        throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
        }

        CdmwPlatformMngt::LifeCycleDef *
        Process_impl::get_life_cycle ()throw (CdmwPlatformMngt::
                           IncompatibleType,
                           CORBA::SystemException)
        {
        throw CdmwPlatformMngt::IncompatibleType ();
        }

            void Process_impl::set_life_cycle (const CdmwPlatformMngt::
                LifeCycleDef & life_cycle)
                throw (CdmwPlatformMngt::DuplicateStep,
                CdmwPlatformMngt::IncompatibleType,
                CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
            {
                throw CdmwPlatformMngt::IncompatibleType ();
            }

            CdmwPlatformMngt::CompleteMonitoringInfo Process_impl::
                get_monitoring_info ()throw (CdmwPlatformMngt::IncompatibleType,
                CORBA::SystemException)
            {
                throw CdmwPlatformMngt::IncompatibleType ();
            }

            void Process_impl::set_monitoring_info (const CdmwPlatformMngt::
                CompleteMonitoringInfo &
                monitoring_info)
                throw (CdmwPlatformMngt::IncompatibleType,
                CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
            {
                throw CdmwPlatformMngt::IncompatibleType ();
            }

            CdmwPlatformMngt::ExitAutoRestartInfo *
                Process_impl::get_exit_autorestart_info ()throw (CORBA::SystemException)
            {
                try
                {

                    CdmwPlatformMngt::ExitAutoRestartInfo_var exit_autorestart_info = new CdmwPlatformMngt::ExitAutoRestartInfo (m_process_info->exit_autorestart_info );

                    return exit_autorestart_info._retn ();
                }
                catch (const std::bad_alloc &)
                {
                    throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
                }
            }



            void Process_impl::set_exit_autorestart_info (const CdmwPlatformMngt::
                ExitAutoRestartInfo & exit_autorestart_info)
                throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
            {

                try
                {
                    m_process_info->exit_autorestart_info = exit_autorestart_info;

                    // update record in datastore
                    set_record_in_datastore ();
                }
                catch (const std::bad_alloc &)
                {
                    throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
                }
            }






        int Process_impl::perform_max_nbr_reached_action ()
                throw (CdmwPlatformMngt::IncompatibleStatus,
                        CORBA::SystemException)
            {

               CDMW_LOG_FUNCTION(m_logger,"");
               try
               {
                set_restart_attempt (0);

                //recover the exit action to perform
                CdmwPlatformMngt::ExitActionType current_exit_action=(m_process_info->exit_autorestart_info).max_nbr_reached_action;

                switch (current_exit_action)
                {
                    case CdmwPlatformMngt::NO_ACTION:
                    CDMW_INFO(m_logger,
                            "perform_max_nbr_reached_action NO_ACTION case : "<< m_element_path.in());
                    break;
                    case CdmwPlatformMngt::RESTART_PROCESS:
                    {
                    CDMW_INFO(m_logger,
                            "perform_max_nbr_reached_action RESTART_PROCESS case : "<< m_element_path.in());
                    CDMW_INFO(m_logger,
                                    "perform_max_nbr_reached_action : restart_attempt ()= "<<restart_attempt ());

                    // Get auto_restart_control
                    CORBA::Any_var any_var;
                    CdmwPlatformMngt::AutoRestartInfo* ari;
                    try
                    {
                        get_attribute (CdmwPlatformMngt::AUTORESTART_INFO, any_var.out(), CdmwPlatformMngt::PRE_DEFINED);
                        any_var.in() >>= ari;
                    }
                    catch(...)
                    {
                        CDMW_FATAL(m_logger,
                                           "Predefined property not found: " << CdmwPlatformMngt::AUTORESTART_INFO);
                        throw;
                    }

            CdmwPlatformMngt::AutoRestartControl auto_restart_control = static_cast<CdmwPlatformMngt::AutoRestartControl>(ari->auto_restart_control);

                    // Get max_nbr_of_restart_attempt
                    CORBA::ULong max_nbr_of_restart_attempt = static_cast<CORBA::ULong>(ari->max_nbr_of_restart_attempt);

                    CDMW_INFO(m_logger,
                                    "perform_max_nbr_reached_action : max_nbr_of_restart_attempt = "
                                    << max_nbr_of_restart_attempt);
                    if ( restart_attempt() < max_nbr_of_restart_attempt || CdmwPlatformMngt::IGNORE_COUNTER==auto_restart_control)
                    {
                     if ( CdmwPlatformMngt::IGNORE_COUNTER==auto_restart_control)
                     {
                         CDMW_INFO(m_logger,
                                    "perform_max_nbr_reached_action : max_nbr_of_restart_attempt ignored");
                     }

                        CdmwPlatformMngtBase::StartupKind* startup_kind=get_startup_kind_from_ds();
                        set_restart_attempt (restart_attempt ()+1);
                        //Set issuer command to the host element path 
                        // Done in hot_restart: set_record_in_datastore ();
                        std::string element_path = get_element_path ();
                        //LC: Attention: call an issuer mutex guard before calling this method.                        
                        Cdmw::PlatformMngt::ProcessIssuerManager::Instance()->get_command_issuer(element_path);

                        {
                            CDMW_MUTEX_GUARD (*(Cdmw::PlatformMngt::ProcessIssuerManager::Instance()->get_process_issuer_mutex(element_path)));
                            Cdmw::PlatformMngt::ProcessIssuerManager::Instance()->set_issuer(element_path, m_host_issuer);
                            hot_restart(*startup_kind);
                        }
                    }
            else if (CdmwPlatformMngt::IGNORE_ACTION!=auto_restart_control)
                    {
                return perform_max_nbr_reached_action();
                }
                    else
                            {
                                CDMW_INFO(m_logger,
                                    "perform_max_nbr_reached_action : max_nbr_reached_action ignored");
                            }
                }
                break;
                case CdmwPlatformMngt::SHUTDOWN_NODE:
                {
                    CDMW_INFO(m_logger,
                            "perform_max_nbr_reached_action SHUTDOWN_NODE case : "<< m_element_path.in());
                    
                    //Set issuer command to the host element path 
                    CdmwPlatformMngt::Host_var host_reference = get_host();
                    CdmwPlatformMngt::HostSnapshot_var host_snapshot = host_reference->get_snapshot();
                    
                    CdmwPlatformMngt::ElementPath_var host_element_path (host_snapshot->element_path);

                    std::string host_path (host_element_path.in());

                   {
                         CDMW_MUTEX_GUARD (*(Cdmw::PlatformMngt::HostIssuerManager::Instance()->get_host_issuer_mutex(host_path)));
                         
                         Cdmw::PlatformMngt::HostIssuerManager::Instance()->set_issuer (host_path, m_host_issuer);
                         host_reference->shutdown();             
                     }
                }
                break;
                case CdmwPlatformMngt::REBOOT_NODE:
                {
                    CDMW_INFO(m_logger,
                            "perform_max_nbr_reached_action REBOOT_NODE case : "<< m_element_path.in());

                     /*manage to max. number of host restarts */                                  
             CORBA::Long current_host_restart = get_host()->restart_attempt();
             

                     // Get max_host_restart_attempt
                     CORBA::Any_var any_var;
                     CdmwPlatformMngt::AutoRestartInfo* ari;
                     try
                     {
             get_host()->get_attribute (CdmwPlatformMngt::AUTORESTART_INFO,
                                                           any_var.out(), CdmwPlatformMngt::PRE_DEFINED);
                         any_var.in() >>= ari;
                     }
                     catch(...)
                     {
                         CDMW_FATAL(m_logger,
                                           "Predefined property not found: " << CdmwPlatformMngt::AUTORESTART_INFO);
                         throw;
                     }
                     CORBA::Long max_host_restart_attempt = ari->max_nbr_of_restart_attempt;

                     current_host_restart++;
                     if ( current_host_restart <= max_host_restart_attempt)
                     {
                         //get_host()->set_restart_attempt( current_host_restart);
                         //Set issuer command to the host element path 
                         CdmwPlatformMngt::Host_var host_reference = get_host();
                         CdmwPlatformMngt::HostSnapshot_var host_snapshot = host_reference->get_snapshot();

                         CdmwPlatformMngt::ElementPath_var host_element_path (host_snapshot->element_path);

                         std::string host_path (host_element_path.in());

                        {
                             CDMW_MUTEX_GUARD (*(Cdmw::PlatformMngt::HostIssuerManager::Instance()->get_host_issuer_mutex(host_path)));
                         
                             Cdmw::PlatformMngt::HostIssuerManager::Instance()->set_issuer (host_path, m_host_issuer);

                             host_reference->hot_reboot();
                           }

                         return CdmwPlatformMngt::REBOOT_NODE; //case of reboot by the notifier
                     }

                     return 0;
                 }
                 break;

                 case CdmwPlatformMngt::STOP_APPLICATION:
                 {
                     CDMW_INFO(m_logger,
                            "perform_max_nbr_reached_action STOP_APPLICATION case : "<< m_element_path.in());
                     //Set issuer command to the host element path 
                     std::string appl_path = std::string (get_application()->get_element_path());
                     {
                         CDMW_MUTEX_GUARD (*(Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->get_application_issuer_mutex(appl_path)));
             Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->set_issuer(appl_path, m_host_issuer);
                         get_application()->stop_application_wait(false);
             }
                 }
                 break;
                 case CdmwPlatformMngt::RESTART_APPLICATION:
                 {
                     CDMW_INFO(m_logger,
                            "perform_max_nbr_reached_action RESTART_APPLICATION case : "<< m_element_path.in());
                     CDMW_INFO(m_logger,
                                    "perform_max_nbr_reached_action : Application restart_attempt ()= "
                                    << get_application()->restart_attempt ());
                     CDMW_INFO(m_logger,
                                    "perform_max_nbr_reached_action : Application max_nbr_of_restart_attempt ()= "
                                    << get_application()->get_max_restart_attempt ());

                     if ( get_application()->restart_attempt () < get_application()->get_max_restart_attempt ())
                     {
                         const CdmwPlatformMngtBase::StartupKind startup_kind;
                         get_application()->set_restart_attempt (get_application()->restart_attempt ()+1);
                         //Set issuer command to the host element path 
                         // Done in hot_restart: get_application()->set_record_in_datastore ();
                         std::string appl_path = std::string (get_application()->get_element_path());
                        {
                            CDMW_MUTEX_GUARD (*(Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->get_application_issuer_mutex(appl_path)));
                        Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->set_issuer(appl_path, m_host_issuer);
                            get_application()->hot_restart(startup_kind);
             }

                     }
                     else if ( get_application()->restart_attempt () >= get_application()->get_max_restart_attempt ())
                     {
                         get_application()->set_restart_attempt (0);
                         CDMW_INFO(m_logger,
                                        "Stop Application max_restart_attempt reached: STOP Application : "<< m_element_path.in());
                         //Set issuer command to the host element path 
                         std::string appl_path = std::string (get_application()->get_element_path());
                         {
                             CDMW_MUTEX_GUARD (*(Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->get_application_issuer_mutex(appl_path)));
                         Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->set_issuer(appl_path, m_host_issuer);
                             get_application()->stop_application_wait(false);
             }
                     }
                 }
                 break;
                 default:
                     CDMW_INFO(m_logger,
                            "perform_max_nbr_reached_action NO_ACTION case : "<< m_element_path.in());
                 };
                 
                 }
                 catch(const CORBA::SystemException& ex_sys)
                 {
                     CDMW_ERROR(m_logger,
                          "Process_impl::perform_max_nbr_reached_action execution failed, CORBA::SystemException caught ");

                     CDMW_LOG_FUNCTION_EXCEPTION(ex_sys);

                     return (-1);
                 }
                 catch(const Cdmw::Exception& ex_cdmw)
                 {

                     CDMW_ERROR(m_logger,
                          "Process_impl::perform_max_nbr_reached_action execution failed, Cdmw::Exception caught ");

                     CDMW_LOG_FUNCTION_EXCEPTION(ex_cdmw);

                     return (-1);
                 }
                 catch (...)
                 {

                     CDMW_ERROR(m_logger,
                          "Process_impl::perform_max_nbr_reached_action execution failed, unknown type exception caught ");

                     return (-1);
                 }
                 return 0;
        }







       int Process_impl::perform_exit_autorestart_action (ProcBehaviour proc_behaviour)
                throw (CdmwPlatformMngt::IncompatibleStatus,
                        CORBA::SystemException)
       {

                try
                {
                    CORBA::String_var status_info;
            
                    if ( (get_host()->status() != CdmwPlatformMngt::HOST_STOPPING)
                 &&  (get_host()->status() != CdmwPlatformMngt::HOST_UNREACHABLE)
                 &&  !(get_host()->get_reboot_in_progress())
                     &&  !(get_host()->get_stop_in_progress())
                 &&  (CdmwPlatformMngt::System::_narrow(( get_application()->get_parent()))->get_status(status_info)!=CdmwPlatformMngt::SYSTEM_STOPPING)
                 &&  (get_application()->get_internal_status ()!= CdmwPlatformMngt::APPLICATION_STOPPING)
                 &&  (getPreviousStatus()!=CdmwPlatformMngt::PROCESS_STOPPING)
                 // &&  (get_application()->get_internal_status ()!= CdmwPlatformMngt::APPLICATION_STOPPED) // COF-MCR893
                       )
                    {

                        //recover the exit action to perform
                        CdmwPlatformMngt::ExitActionType current_exit_action=(m_process_info->exit_autorestart_info).default_exit_action;

                        if (Cdmw::PlatformMngt::FAILED_DEATH==proc_behaviour){

                            CDMW_INFO(m_logger,
                            "perform_exit_autorestart_action NON-ABNORMAL_BEHAVIOUR case "
                            << m_element_path.in());

                            CORBA::Long current_exit_code=0;
                            if (m_last_exit_info.exit_event==CdmwPlatformMngt::EXITED)
                            {

                                current_exit_code = m_last_exit_info.event_value;

                                CdmwPlatformMngt::ExitActions exit_actions=(m_process_info->exit_autorestart_info).exit_actions;

                                for (CORBA::ULong i = 0; i < exit_actions.length (); i++)
                                {
                                    CDMW_DEBUG(m_logger,
                                            "perform_exit_autorestart_action : "<< m_element_path.in());
                                    if (exit_actions[i].exit_code==current_exit_code)
                                   {
                                        current_exit_action=exit_actions[i].exit_action;
                                   }
                               }

                           }

                    }
                    else{
                        CDMW_INFO(m_logger,
                        "perform_exit_autorestart_action ABNORMAL_BEHAVIOUR case "
                        << m_element_path.in());
                        current_exit_action=(m_process_info->exit_autorestart_info).abnormal_behaviour_action;
                   }


                    switch (current_exit_action)
                    {
                        case CdmwPlatformMngt::NO_ACTION:
                            CDMW_INFO(m_logger,
                            "perform_exit_autorestart_action NO_ACTION case : "<< m_element_path.in());
                            break;
                        case CdmwPlatformMngt::ABORT_PROCESS:
                            CDMW_INFO(m_logger,
                            "perform_exit_autorestart_action ABORT_PROCESS case : "<< m_element_path.in());
                            break;
                        case CdmwPlatformMngt::RESTART_PROCESS:
                        {
                            //PCR-685 : Check force_manual_start tag
                            if(!get_force_manual_start())
                            {

                                CDMW_INFO(m_logger,
                                        "perform_exit_autorestart_action RESTART_PROCESS case : "<< m_element_path.in());
                                CDMW_INFO(m_logger,
                                        "perform_exit_autorestart_action : restart_attempt ()= "<<restart_attempt ());

                                // Get auto_restart_control
                                CORBA::Any_var any_var;
                                CdmwPlatformMngt::AutoRestartInfo* ari;
                                try {
                                    get_attribute (CdmwPlatformMngt::AUTORESTART_INFO, any_var.out(), CdmwPlatformMngt::PRE_DEFINED);
                                    any_var.in() >>= ari;
                                }
                                catch(...) {
                                    CDMW_FATAL(m_logger,
                                            "Predefined property not found: " << CdmwPlatformMngt::AUTORESTART_INFO);
                                    throw;
                                }

                                CdmwPlatformMngt::AutoRestartControl auto_restart_control = static_cast<CdmwPlatformMngt::AutoRestartControl>(ari->auto_restart_control);

                                // Get max_nbr_of_restart_attempt

                                CORBA::ULong max_nbr_of_restart_attempt = static_cast<CORBA::ULong>(ari->max_nbr_of_restart_attempt);


                                CDMW_INFO(m_logger,
                                        "perform_exit_autorestart_action : max_nbr_of_restart_attempt = "
                                        << max_nbr_of_restart_attempt);

                                if ( restart_attempt() < max_nbr_of_restart_attempt  || CdmwPlatformMngt::IGNORE_COUNTER==auto_restart_control )
                                {
                                    if (CdmwPlatformMngt::IGNORE_COUNTER==auto_restart_control)
                                    {
                                        CDMW_INFO(m_logger,
                                        "perform_exit_autorestart_action : max_nbr_of_restart_attempt ignored");
                                    }

                                    CdmwPlatformMngtBase::StartupKind* startup_kind=get_startup_kind_from_ds();
                                    set_restart_attempt (restart_attempt ()+1);
                                    //Set issuer command to the host element path 
                                    // Done in hot_restart: set_record_in_datastore ();
                                    std::string element_path = get_element_path();

                                    {
                                        CDMW_MUTEX_GUARD
                                        (*(Cdmw::PlatformMngt::ProcessIssuerManager::Instance()->get_process_issuer_mutex(element_path)));
                                        Cdmw::PlatformMngt::ProcessIssuerManager::Instance()->set_issuer(element_path, m_host_issuer);
                                        hot_restart(*startup_kind);
                                    }                        
                                }
                                else if (CdmwPlatformMngt::IGNORE_ACTION!=auto_restart_control)
                                {
                                    return perform_max_nbr_reached_action();
                                }
                                else
                                {
                                    CDMW_INFO(m_logger,
                                    "perform_exit_autorestart_action : max_nbr_reached_action ignored");
                                }


                            } // check force manual start tag 
                            else
                            {
                                CDMW_INFO(m_logger,
                                        "perform_exit_autorestart_action RESTART_PROCESS case : "<< m_element_path.in() <<
                                " Force manual start set to MANUAL, no action executed "); 
                            } 
                        }
                        break;
                        case CdmwPlatformMngt::SHUTDOWN_NODE:
                        {
                            CDMW_INFO(m_logger,
                            "perform_exit_autorestart_action SHUTDOWN_NODE case : "<< m_element_path.in());
                            //Set issuer command to the host element path 
                           CdmwPlatformMngt::Host_var host_reference = get_host();
                           CdmwPlatformMngt::HostSnapshot_var host_snapshot = host_reference->get_snapshot();
                    
                           CdmwPlatformMngt::ElementPath_var host_element_path (host_snapshot->element_path);

                           std::string host_path = std::string ((char *) host_element_path);

                           {
                             CDMW_MUTEX_GUARD (*(Cdmw::PlatformMngt::HostIssuerManager::Instance()->get_host_issuer_mutex(host_path)));
                         
                             Cdmw::PlatformMngt::HostIssuerManager::Instance()->set_issuer (host_path, m_host_issuer);
                             host_reference->shutdown();
                           }

                        }
                        break;
                        case CdmwPlatformMngt::REBOOT_NODE:
                        {
                            CDMW_INFO(m_logger,
                            "perform_exit_autorestart_action REBOOT_NODE case : "<< m_element_path.in());
                            /*manage to max. number of host restarts */
                            CORBA::Long current_host_restart = get_host()->restart_attempt();

                            // Get max_host_restart_attempt
                            CORBA::Any_var any_var;
                            CdmwPlatformMngt::AutoRestartInfo* ari;
                            try {
                get_host()->get_attribute (CdmwPlatformMngt::AUTORESTART_INFO,
                                                           any_var.out(), CdmwPlatformMngt::PRE_DEFINED);
                                any_var.in() >>= ari;
                            }
                            catch(...) {
                                CDMW_FATAL(m_logger,
                                           "Predefined property not found: " << CdmwPlatformMngt::AUTORESTART_INFO);
                                throw;
                            }
                            CORBA::Long max_host_restart_attempt = ari->max_nbr_of_restart_attempt;

                            current_host_restart++;
                            if ( current_host_restart <= max_host_restart_attempt)
                            {
                                //get_host()->set_restart_attempt( current_host_restart);
                                //Set issuer command to the host element path 
                                CdmwPlatformMngt::Host_var host_reference = get_host();
                                CdmwPlatformMngt::HostSnapshot_var host_snapshot = host_reference->get_snapshot();

                                CdmwPlatformMngt::ElementPath_var host_element_path (host_snapshot->element_path);

                                std::string host_path = std::string ((char *) host_element_path);

                                 {
                                    CDMW_MUTEX_GUARD (*(Cdmw::PlatformMngt::HostIssuerManager::Instance()->get_host_issuer_mutex(host_path)));
                         
                                    Cdmw::PlatformMngt::HostIssuerManager::Instance()->set_issuer (host_path, m_host_issuer);
                                    host_reference->hot_reboot();
                                 }

                               return CdmwPlatformMngt::REBOOT_NODE; //case of reboot by the notifier
                            }

                            return 0;
                        }
                        break;

                        case CdmwPlatformMngt::STOP_APPLICATION:
                        {
                            CDMW_INFO(m_logger,
                            "perform_exit_autorestart_action STOP_APPLICATION case : "<< m_element_path.in());
                            //Set issuer command to the host element path 
                            std::string appl_path = std::string (get_application()->get_element_path());
                            {
                               CDMW_MUTEX_GUARD (*(Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->get_application_issuer_mutex(appl_path)));
                       Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->set_issuer(appl_path, m_host_issuer);
                               get_application()->stop_application_wait(false);
                            }
                
                        }
                        break;
                        case CdmwPlatformMngt::RESTART_APPLICATION:
                        {

                            CORBA::Any_var any_var = new CORBA::Any();
                            CORBA::Boolean force_manual_start_tag;
                            try
                            {
                                get_application()->get_attribute (CdmwPlatformMngt::FORCE_MANUAL_START, any_var.out(), CdmwPlatformMngt::PRE_DEFINED);
                                any_var >>= CORBA::Any::to_boolean (force_manual_start_tag);
                            }
                            catch(const CdmwPlatformMngt::AttributeNotFound &)
                            {

                                CDMW_FATAL(m_logger,
                                        "Property not found: " << CdmwPlatformMngt::FORCE_MANUAL_START);
                            }

                            if(!force_manual_start_tag)                         
                            {
                                CDMW_INFO(m_logger,
                                        "perform_exit_autorestart_action RESTART_APPLICATION case : "<< m_element_path.in());
                                CDMW_INFO(m_logger,
                                        "perform_exit_autorestart_action : Application restart_attempt ()= "
                                        << get_application()->restart_attempt ());
                                CDMW_INFO(m_logger,
                                        "perform_exit_autorestart_action : Application  max_nbr_of_restart_attempt ()= "
                                        << get_application()->get_max_restart_attempt ());

                                if ( get_application()->restart_attempt () < get_application()->get_max_restart_attempt ())
                                {
                                    const CdmwPlatformMngtBase::StartupKind startup_kind;
                                    get_application()->set_restart_attempt (get_application()->restart_attempt ()+1);
                                    // Done in hot_restart: get_application()->set_record_in_datastore ();
                                    //Set issuer command to the host element path 
                                    std::string appl_path = std::string (get_application()->get_element_path());
                                    {
                                        CDMW_MUTEX_GUARD (*(Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->get_application_issuer_mutex(appl_path)));
                                        Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->set_issuer(appl_path, m_host_issuer);
                                        get_application()->hot_restart(startup_kind);
                                    }

                                }
                                else if ( get_application()->restart_attempt () >= get_application()->get_max_restart_attempt ())
                                {
                                    get_application()->set_restart_attempt (0);
                                    CDMW_INFO(m_logger,
                                            "Stop Application max_restart_attempt reached: STOP Application : "<< m_element_path.in());
                                    //Set issuer command to the host element path 
                                    std::string appl_path = std::string (get_application()->get_element_path());
                                    {
                                        CDMW_MUTEX_GUARD (*(Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->get_application_issuer_mutex(appl_path)));
                                        Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->set_issuer(appl_path, m_host_issuer);
                                        get_application()->stop_application_wait(false);
                                    }
                                }

                            } //if force_manual_start
                            else
                            {
                                CDMW_INFO(m_logger,
                                        "perform_exit_autorestart_action RESTART_APPLICATION case : "<< m_element_path.in() <<
                                " Force manual start set to MANUAL for this application, no action executed "); 
                            }
                        }
                            break;
                        default:
                            CDMW_INFO(m_logger,
                            "perform_exit_autorestart_action NO_ACTION case : "<< m_element_path.in());
                        };
                    }
                    else
                    {
                        CDMW_INFO(m_logger,
                        "Process_impl : NO AUTO ACTION PERFORMED : STOPPING or HOST UNREACHABLE : "
                        << m_element_path.in());
                    }

                }
                catch (...)
                {
                    return (-1);
                }

            return 0;
            }




            CdmwPlatformMngt::ProcessSnapshot *
                Process_impl::get_snapshot ()throw (CORBA::SystemException)
            {
                try
                {

                    //MRCDMW_MUTEX_GUARD(*m_application->get_check_consistency_status_recursive_mutex());

                    unsigned int i;

                    // Get the snapshot of all components of Process
                    // CdmwPlatformMngt::COMPONENT_SERVER

                    CdmwPlatformMngt::ComponentsSnapshot_var componentsSnapshot
                        = new CdmwPlatformMngt::ComponentsSnapshot;

                    // If the process is managed
                    if (type () != CdmwPlatformMngt::UNMANAGED_PROCESS)
                    {
                        // Gets all the components
                        CdmwPlatformMngt::CcmComponents_var components;
                        CdmwPlatformMngt::ComponentsIterator_var componentsIt;
                        CORBA::ULong nbr_of_components;

                        nbr_of_components = get_number_of_components ();

                        get_all_components (nbr_of_components, //
                            components.out (), componentsIt.out ());

                        componentsSnapshot->length (nbr_of_components);
                        for (i = 0; i < nbr_of_components; i++)
                        {
                            (*componentsSnapshot)[i] = *(components[i]->get_snapshot ());
                        }
                    }
                    else
                    {
                        componentsSnapshot->length (0);
                    }

                    //////////////

                    // Get the snapshot of all entities of application
                    CdmwPlatformMngt::EntitiesSnapshot_var entitiesSnapshot
                        = new CdmwPlatformMngt::EntitiesSnapshot ();

                    // If the process is managed
                    if (type () != CdmwPlatformMngt::UNMANAGED_PROCESS)
                    {
                        // Gets all the entities
                        CdmwPlatformMngt::Entities_var entities;
                        CdmwPlatformMngt::EntitiesIterator_var entitiesIt;
                        CORBA::ULong nbr_of_entities;

                        nbr_of_entities = get_number_of_entities ();

                        get_all_entities (nbr_of_entities,
                            entities.out (), entitiesIt.out ());

                        entitiesSnapshot->length (nbr_of_entities);
                        for (i = 0; i < nbr_of_entities; i++)
                        {
                            (*entitiesSnapshot)[i] = *(entities[i]->get_snapshot ());
                        }
                    }
                    else
                    {
                        entitiesSnapshot->length (0);
                    }

                    //Gets all the components ...



                    //Gets all the properties
                    CosPropertyService::PropertiesIterator_var propertiesIt;
                    CORBA::ULong nbr_of_properties;
                    CdmwPlatformMngt::PropertiesSnapshot_var propertiesSnapshot =
                        new CdmwPlatformMngt::PropertiesSnapshot;
                    nbr_of_properties = get_number_of_properties ();
                    CosPropertyService::PropertyNames_var prop_names;
                    CosPropertyService::PropertyNamesIterator_var prop_namesIt;
                    get_all_property_names (nbr_of_properties, prop_names.out (),
                        prop_namesIt.out ());
                    propertiesSnapshot->length (nbr_of_properties-2);
                    CORBA::ULong snap_prop=0;
                    for (i = 0; i < nbr_of_properties; i++)
                    {
                        if ((strcmp (prop_names[i], "Cdmw.TechStatus")==0) ||
                                (strcmp (prop_names[i], "Cdmw.TechStatusInfo")==0))
                        {
                            // do not add the TECH_STATUS and TECH_STATUS_INFO properties to the snapshot 
                        }
                        else
                        {
                          CdmwPlatformMngt::PropertySnapshot_var propertySnapshot =
                                new CdmwPlatformMngt::PropertySnapshot ();
                
                            propertySnapshot->property_name =
                                CORBA::string_dup (prop_names[i]);
                            CORBA::Any prop_value;
                            prop_value = *(get_property_value (prop_names[i]));
                            CORBA::TypeCode_var tc;
                            tc = prop_value.type ();
                            switch (tc->kind ())
                            {
                            default:
                                break;
                            case CORBA::tk_string:
                                propertySnapshot->property_type =
                                    CORBA::string_dup ("String");
                                break;
                            case CORBA::tk_short:
                                propertySnapshot->property_type = CORBA::string_dup ("Short");
                                break;
                            case CORBA::tk_float:
                                propertySnapshot->property_type = CORBA::string_dup ("Float");
                                break;
                            };
                            CosPropertyService::PropertyModeType mode =
                                get_property_mode (prop_names[i]);
                            propertySnapshot->property_value = prop_value;

                            if (mode == CosPropertyService::normal) propertySnapshot->property_mode = CORBA::string_dup ("Read/Write");
                            else if (mode == CosPropertyService::read_only) propertySnapshot->property_mode = CORBA::string_dup ("Read/Only");
                            else propertySnapshot->property_mode = CORBA::string_dup ("Pre-defined");

                            propertySnapshot->property_value = prop_value;                            
                (*propertiesSnapshot)[snap_prop] = *(propertySnapshot);
                            snap_prop++;
                        }
                    }


                    // Create a process snapshot
                    CORBA::String_var statusInfo;
                    CdmwPlatformMngt::ProcessSnapshot_var snapshot
                        = new CdmwPlatformMngt::ProcessSnapshot();  

                    // note : the copy of a snapshot sequence (= operator)
                    //        make a deeply copy with a new memory allocation
                    //        for the sequence

                    snapshot->process_name = CORBA::string_dup (m_element_name.in ());
                    snapshot->host_name = CORBA::string_dup (m_proc_host_name.in ());
                    snapshot->process_status = get_status (statusInfo.out ());
                    snapshot->process_status_info = statusInfo;
                    snapshot->element_path = element_path();
                    snapshot->process_entities = entitiesSnapshot.in ();
                    snapshot->process_properties = propertiesSnapshot.in ();
                    snapshot->process_components = componentsSnapshot.in();

                    return snapshot._retn ();
                }
                catch (const std::bad_alloc &)
                {
                    throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
                }
        }

        CdmwPlatformMngt::RequestStatus Process_impl::
            next_step ()throw (CdmwPlatformMngt::InvalidStep,
            CdmwPlatformMngt::IncompatibleType,
            CdmwPlatformMngt::IncompatibleStatus,
            CORBA::SystemException)
        {
            throw CdmwPlatformMngt::IncompatibleType ();
        }

        CdmwPlatformMngt::RequestStatus Process_impl::
            run ()throw (CdmwPlatformMngt::NotReadyToRun,
            CdmwPlatformMngt::IncompatibleType,
            CdmwPlatformMngt::IncompatibleStatus,
            CORBA::SystemException)
        {
            throw CdmwPlatformMngt::IncompatibleType ();
        }

        void Process_impl::notifyCommandEvent(ProcessCommand proc_command,
                const char* issuer,
                CORBA::Boolean emergency)
        {
            CdmwPlatformMngt::CdmwCommand_var event;
            CdmwPlatformMngtBase::EventHeader header
            = EventHeaderFactory::createHeader (CdmwPlatformMngtBase::INF);

            if ((COLD_RESTART == proc_command) || (HOT_RESTART == proc_command))
            {
                //Setting up the startupKind parameters
                CdmwPlatformMngtBase::StateRetrieval state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
                unsigned long startup_mode = 0;
                std::string specified_state_id = "";
                CdmwPlatformMngtBase::StartupKind_var startupKind = new CdmwPlatformMngtBase::StartupKind();
                startupKind->state_retrieval = state_retrieval;
                startupKind->startup_mode = startup_mode;
                startupKind->specified_state_id  = CORBA::string_dup( specified_state_id.c_str() );

                CdmwPlatformMngt::RestartWarmth restart_warmth;

                if (COLD_RESTART == proc_command)
                {
                    restart_warmth = CdmwPlatformMngt::COLD_RESTART;
                }
                else
                {
                    restart_warmth = CdmwPlatformMngt::HOT_RESTART;
                }

                // Create the event
                event = m_procRestartFactory->create(
                        header,
                        m_application->get_system_name(),
                        get_element_path(),
                        name(),
                        issuer,
                        startupKind.in(),
                        restart_warmth);

            }

            if (STOP == proc_command)
            {
                event = m_procStopFactory->create (header,
                        m_application->get_system_name(),
                        get_element_path(),
                        name(),
                        issuer,
                        emergency);

            }

            //Log event
            CORBA::String_var eventStr = event->to_string ();
            LogMngr::logMessage (eventStr.in ());

            // Perform the command notification to supervision event notifier
            event->_add_ref();
            (EventNotifierManager::Instance()->get_sup_notifier())->addEvent (event);

            // Perform the command notification to command event notifier
            event->_add_ref();
            (EventNotifierManager::Instance()->get_command_notifier())->add_event(event);
            event->_remove_ref();

            event._retn ();

        }

        CdmwPlatformMngt::RequestStatus Process_impl::
            cold_restart (const CdmwPlatformMngtBase::
            StartupKind & startup_kind) throw (CORBA::SystemException, CdmwPlatformMngt::IncompatibleStatus)
        {

            // Create a command event to notify
            std::string element_path = get_element_path ();
            std::string issuer = Cdmw::PlatformMngt::ProcessIssuerManager::Instance()->get_command_issuer(element_path);
            notifyCommandEvent(COLD_RESTART, issuer.c_str());

            set_restart_attempt (0);
            bool emergency = false;
            try {

                if (CdmwPlatformMngt::LONG_REQUEST_COMPLETED !=
                    restart ( CdmwPlatformMngt::COLD_RESTART, startup_kind, emergency ) ){
                    return CdmwPlatformMngt::REQUEST_ERROR;
                }
            }
            catch (...) {
                throw;
            }

            return CdmwPlatformMngt::REQUEST_SUCCEED;
        }


        CdmwPlatformMngt::RequestStatus Process_impl::
            hot_restart (const CdmwPlatformMngtBase::
            StartupKind & startup_kind) throw (CORBA::SystemException,CdmwPlatformMngt::IncompatibleStatus)
        {

            // Create a command event to notify
            std::string element_path = get_element_path ();
            //LC: Attention: call an issuer mutex guard before calling this method.            
            std::string issuer = Cdmw::PlatformMngt::ProcessIssuerManager::Instance()->get_command_issuer(element_path);

            notifyCommandEvent(HOT_RESTART, issuer.c_str());

            bool emergency = false;
            try {

                if (CdmwPlatformMngt::LONG_REQUEST_COMPLETED !=
                    restart ( CdmwPlatformMngt::HOT_RESTART, startup_kind, emergency ) ){
                    return CdmwPlatformMngt::REQUEST_ERROR;
                }
            }
            catch (...) {
                throw;
            }

            return CdmwPlatformMngt::REQUEST_SUCCEED;
        }

        CdmwPlatformMngt::LongRequestStatus Process_impl::restart ( const CdmwPlatformMngt::RestartWarmth restartWarmth,
            const CdmwPlatformMngtBase::StartupKind & startup_kind,
            bool emergency)
            throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)

        {

        //TODO add control on current status
        //TODO add return values


        set_requested_restart_warmth ( restartWarmth);


        try{
    
                    if(CdmwPlatformMngt::LONG_REQUEST_COMPLETED != stop_and_wait( emergency ))
                    {
                            return CdmwPlatformMngt::LONG_REQUEST_NOT_COMPLETED;
                    }
        }
        catch (...) {
        throw;
            }

        try
            {
                    //FLI - To avoid deadlock on application initialise with processes failed timeout 
                    CDMW_MUTEX_GUARD(*m_application->get_check_consistency_status_recursive_mutex());
        
                    if(CdmwPlatformMngt::REQUEST_SUCCEED !=  initialise ( startup_kind ))
                    {
                            return CdmwPlatformMngt::LONG_REQUEST_NOT_COMPLETED;
                    }
        }
            catch (...) {
            throw;
            }

        try
            {
        
           if ((0 != dynamic_cast<ManagedProcess_impl*> (this))&&(CdmwPlatformMngt::REQUEST_SUCCEED !=  run ()))
                {
                        return CdmwPlatformMngt::LONG_REQUEST_NOT_COMPLETED;
                }

            }
            catch (...) {
            throw;
            }

       /* Reset the process default_restart_warmth */
       //MR
       //set_requested_restart_warmth ( get_default_restart_warmth () );


        return CdmwPlatformMngt::LONG_REQUEST_COMPLETED;
        }

        CORBA::ULong Process_impl::get_number_of_entities ()throw (CORBA::
            SystemException)
        {
            throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
                CORBA::COMPLETED_NO);
        }


        void Process_impl::get_all_entity_names (CORBA::ULong how_many,
            CdmwPlatformMngt::
            ElementNames_out entity_names,
            CdmwPlatformMngt::
            ElementNamesIterator_out rest)
            throw (CORBA::SystemException)
        {
            throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
                CORBA::COMPLETED_NO);
        }

        CdmwPlatformMngt::Entity_ptr Process_impl::
            get_entity (const char *entity_name) throw (CdmwPlatformMngt::
                            EntityNotFound,
                            CORBA::SystemException)
        {
            throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
                CORBA::COMPLETED_NO);
        }

        CORBA::Boolean Process_impl::get_entities (const CdmwPlatformMngt::
            ElementNames & entity_names,
            CdmwPlatformMngt::
            Entities_out nentities)
            throw (CORBA::SystemException)
        {
            throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
                CORBA::COMPLETED_NO);
        }

        void Process_impl::get_all_entities (CORBA::ULong how_many,
            CdmwPlatformMngt::
            Entities_out nentities,
            CdmwPlatformMngt::
            EntitiesIterator_out rest)
            throw (CORBA::SystemException)
        {
            throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
                CORBA::COMPLETED_NO);
        }

        CdmwPlatformMngt::Entity_ptr Process_impl::
            add_entity (const CdmwPlatformMngt::
            EntityDef & entity_def) throw (CdmwPlatformMngt::
            EntityAlreadyExists,
            CORBA::SystemException)
        {
            throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
                CORBA::COMPLETED_NO);
        }

        void Process_impl::remove_entity (const char *entity_name)
            throw (CdmwPlatformMngt::EntityNotFound, CORBA::SystemException)
        {
            throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
                CORBA::COMPLETED_NO);
        }

        CdmwPlatformMngtService::ServiceProviderID *
            Process_impl::get_service_def (const char *service_name)
            throw (CdmwPlatformMngtService::ServiceNotFound, CORBA::SystemException)
        {
            throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
                CORBA::COMPLETED_NO);
        }

        void Process_impl::add_service_def (const CdmwPlatformMngtService::
            ServiceDef & service_def)
            throw (CdmwPlatformMngtService::ServiceAlreadyExists,
            CORBA::SystemException)
        {
            throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
                CORBA::COMPLETED_NO);
        }

        void Process_impl::remove_service_def (const char *service_name)
            throw (CdmwPlatformMngtService::ServiceNotFound, CORBA::SystemException)
        {
            throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
                CORBA::COMPLETED_NO);
        }

        void Process_impl::remove_all_service_defs ()
            throw (CORBA::SystemException)
        {
            throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
                CORBA::COMPLETED_NO);
        }

        void Process_impl::is_alive ()
        {
            // TODO
            throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
                CORBA::COMPLETED_NO);
        }

        CdmwPlatformMngt::ApplicationAgent_ptr
            Process_impl::getApplicationAgent ()throw (CdmwPlatformMngt::
            HostNotFound,
            CdmwPlatformMngt::
            HostNotReachable,
            CdmwPlatformMngt::
            CreationError,
            CdmwPlatformMngt::
            ApplicationAgentAlreadyExists,
            CdmwPlatformMngt::
            InvalidInfo,
            CdmwPlatformMngt::
            CreationTimeout,
            AgentNotFoundException,
            CORBA::SystemException)
        {
            CdmwPlatformMngt::ApplicationAgent_var agent;

            // Get the application agent on the host of this process
            // if does not exist start a new one
            agent = m_application->getAgent (m_proc_host_name.in ());

            return agent._retn ();
        }

        CdmwPlatformMngt::ApplicationAgent_ptr
            Process_impl::searchApplicationAgent ()throw (CdmwPlatformMngt::
            HostNotFound,
            CdmwPlatformMngt::
            HostNotReachable,
            CORBA::SystemException)
        {
            CdmwPlatformMngt::ApplicationAgent_var agent;

            // Get the application agent on the host of this process
            // if does not exist nil is returned
            agent = m_application->searchAgentFromDaemon (m_proc_host_name.in ());

            return agent._retn ();
        }




        void Process_impl::setAutoending (CORBA::Boolean auto_ending)
        {
            m_autoending = auto_ending;

            // update record in datastore
            set_record_in_datastore ();
        }


        void Process_impl::setHost (const char *host_name)
            throw (CdmwPlatformMngt::HostNotFound, CORBA::SystemException)
        {
            CdmwPlatformMngt::HostContainer_var host_container =
                m_application->get_host_container ();
            m_host = host_container->get_host (host_name);

            m_proc_host_name = CORBA::string_dup (host_name);

            // update record in datastore
            set_record_in_datastore ();
        }

        void Process_impl::setProcessInfo (const CdmwPlatformMngt::
            ProcessInfo & process_info)
        {
            try
            {
                m_process_info = new CdmwPlatformMngt::ProcessInfo (process_info);

                // update record in datastore
                set_record_in_datastore ();
            }
            catch (const std::bad_alloc &)
            {
                throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
            }
        }

        void Process_impl::setMonitoringInfo (const CdmwPlatformMngt::
            CompleteMonitoringInfo &
            monitoring_info)
        {
            try
            {
                m_monitoring_info =
                    new CdmwPlatformMngt::CompleteMonitoringInfo (monitoring_info);

                // update record in datastore
                set_record_in_datastore ();
            }
            catch (const std::bad_alloc &)
            {
                throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
            }
        }

        void Process_impl::errorEvent (const CdmwPlatformMngtBase::
            EventHeader & header, const char *issuer,
            const char *error_info)
        {

            // Do nothing if in a failed status
            CdmwPlatformMngt::ProcessStatus status = getStatus ();

            if ((status == CdmwPlatformMngt::PROCESS_FAILED_ERROR) ||
                (status == CdmwPlatformMngt::PROCESS_FAILED_TIMEOUT) ||
                (status == CdmwPlatformMngt::PROCESS_FAILED_DISCREPANCY) ||
                (status == CdmwPlatformMngt::PROCESS_FAILED_INVALID) ||
                (status == CdmwPlatformMngt::PROCESS_FAILED_NO_RESPONSE) ||
                (status == CdmwPlatformMngt::PROCESS_FAILED_DEATH))
                return;

            // Set the status info's message
            set_status_info ( CORBA::string_dup (error_info));

            // Store the info about the issuance of this error event.
            m_lastErrorHeader = header;
            m_lastErrorIssuer = issuer;

            // Change to the state FailedError
            setState ("FailedError");

            // update record in datastore
            set_record_in_datastore ();
        }


        void Process_impl::endingEvent (const CdmwPlatformMngtBase::
                        EventHeader & header)
        {
            // Do nothing if in a failed status
            CdmwPlatformMngt::ProcessStatus status = getStatus ();

            if ((status == CdmwPlatformMngt::PROCESS_FAILED_ERROR) ||
                (status == CdmwPlatformMngt::PROCESS_FAILED_TIMEOUT) ||
                (status == CdmwPlatformMngt::PROCESS_FAILED_DISCREPANCY) ||
                (status == CdmwPlatformMngt::PROCESS_FAILED_INVALID) ||
                (status == CdmwPlatformMngt::PROCESS_FAILED_NO_RESPONSE) ||
                (status == CdmwPlatformMngt::PROCESS_FAILED_DEATH))
                return;

            // if process is authorised to end itself
            if (is_auto_ending ())
            {
                // change to the status Ended
                setState ("Ended");
            }

            else
            {
                // Set the status info's message
                set_status_info (
                    CORBA::string_dup (LogMngr::
                    getFormattedMessage
                    (MSG_ID_PROCESS_ENDING_ERROR).c_str ()));

                // Store the info about the issuance of this event.
                m_lastErrorHeader = header;

                // Change to the state FailedDeath
                setState ("FailedDeath");
            }

            // update record in datastore
            set_record_in_datastore ();
        }

        void Process_impl::invalidateEvent (const CdmwPlatformMngtBase::
            EventHeader & header,
            const char *reason)
        {
            // Do nothing if in a failed status
            CdmwPlatformMngt::ProcessStatus status = getStatus ();

            if ((status == CdmwPlatformMngt::PROCESS_FAILED_ERROR) ||
                (status == CdmwPlatformMngt::PROCESS_FAILED_TIMEOUT) ||
                (status == CdmwPlatformMngt::PROCESS_FAILED_DISCREPANCY) ||
                (status == CdmwPlatformMngt::PROCESS_FAILED_INVALID) ||
                (status == CdmwPlatformMngt::PROCESS_FAILED_NO_RESPONSE) ||
                (status == CdmwPlatformMngt::PROCESS_FAILED_DEATH))
                return;

            // Set the status info's message
            set_status_info ( CORBA::string_dup (reason));

            // Store the info about the issuance of this event.
            m_lastErrorHeader = header;

            // Change to the state FailedInvalid
            setState ("FailedInvalid");

            // update record in datastore
            set_record_in_datastore ();
        }


        Application_impl *Process_impl::get_application ()
        {
            return m_application;
        }

        const char *Process_impl::get_process_host_name ()
        {
            return CORBA::string_dup(m_proc_host_name);
        }


        const char *Process_impl::get_process_key ()
        {
            return m_proc_key.in ();
        }

        void Process_impl::set_create_process_result (CdmwPlatformMngt::
                            ProcessCommandResult result,
                            const char *error_info,
                            const char *error_issuer,
                            CdmwPlatformMngt::
                            ProcessDelegateWrapper_ptr
                            process_delegate_wrap)
                            throw (CORBA::SystemException)
        {
            // the process delegate wrapper object is not used in this case

            m_starting_result = result;

            m_creation_error_info = CORBA::string_dup (error_info);
            m_creation_error_issuer = CORBA::string_dup (error_issuer);

            // signals the acknowledgement
            if (!m_process_start_cbk->signal ())
            {
                // The acknowledgement has been performed too late
                throw CORBA::NO_PERMISSION (OrbSupport::NO_PERMISSION,
                    CORBA::COMPLETED_NO);
            }
        }



        void Process_impl::set_initialisation_result (CdmwPlatformMngt::
                            ProcessCommandResult result)
                            throw (CORBA::SystemException)
        {
            throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
                CORBA::COMPLETED_NO);
        }


        void Process_impl::set_next_step_result (CdmwPlatformMngt::
            ProcessCommandResult result,
            CORBA::
            ULong step_nbr) throw (CORBA::
            SystemException)
        {
            throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
                CORBA::COMPLETED_NO);
        }


        void Process_impl::set_run_result (CdmwPlatformMngt::
            ProcessCommandResult result)
            throw (CORBA::SystemException)
        {
            throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
                CORBA::COMPLETED_NO);
        }


        void Process_impl::set_stop_result (CdmwPlatformMngt::
            ProcessCommandResult result)
            throw (CORBA::SystemException)
        {
            throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
                CORBA::COMPLETED_NO);
        }

        void Process_impl::set_force_manual_start( const bool force_manual_start )
        throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound)
        {
            CORBA::Any_var any_var = new CORBA::Any();
            try{
            *any_var <<= CORBA::Any::from_boolean(force_manual_start);
            set_attribute_internal (CdmwPlatformMngt::FORCE_MANUAL_START, any_var.in(), CdmwPlatformMngt::PRE_DEFINED);
            }
            catch(const CdmwPlatformMngt::AttributeNotFound&)
            {
            CDMW_FATAL(m_logger,
                       "Property not found: " << CdmwPlatformMngt::FORCE_MANUAL_START);
            }
        }

        bool Process_impl::get_force_manual_start()
        throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound)
        {
            CORBA::Any_var any_var = new CORBA::Any();
            CORBA::Boolean b;
            try{
            get_attribute (CdmwPlatformMngt::FORCE_MANUAL_START, any_var.out(), CdmwPlatformMngt::PRE_DEFINED);
            any_var >>= CORBA::Any::to_boolean (b);
            }
            catch(const CdmwPlatformMngt::AttributeNotFound &)
            {
            CDMW_FATAL(m_logger,
                       "Property not found: " << CdmwPlatformMngt::FORCE_MANUAL_START);
            }
            return b;

        }

        void Process_impl::set_last_exit_code( const CdmwPlatformMngt::ProcessExitInfo & last_exit_code )
        throw (CORBA::SystemException)
        {
            m_last_exit_info=last_exit_code;
        }

        CdmwPlatformMngt::ProcessExitInfo Process_impl::get_last_exit_code()
        throw (CORBA::SystemException)
        {
            return m_last_exit_info;
        }

        void Process_impl::set_process_arguments( const char * process_arguments )
        throw (CORBA::SystemException)
        {
            // TODO
            throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
        }

        char *  Process_impl::get_process_arguments()
        throw (CORBA::SystemException)
        {
            // TODO
            throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
        }

        void Process_impl::set_indicator_relevant_status( const bool indicator_relevant_status)
        throw (CORBA::SystemException)
        {
            m_indicator_relevant_status = indicator_relevant_status;
        }

        bool  Process_impl::get_indicator_relevant_status()
        throw (CORBA::SystemException)
        {
            return m_indicator_relevant_status;
        }







        void Process_impl::set_trace_filtering(const char* trace_filtering)
            throw (CORBA::SystemException, CdmwPlatformMngt::ParsingException , CdmwPlatformMngt::CastException )
        {
            std::vector <string> items;
            if(0 == trace_filtering)
                throw CdmwPlatformMngt::ParsingException("Error: trace in null");

            // avoid too long string usage
            const int max_size = 1024;

            if(strlen(trace_filtering) > (max_size - 1))
                throw CdmwPlatformMngt::ParsingException("Error: trace in null");

            char buff[max_size];
            char tmp_buff[max_size];
            strcpy( buff,trace_filtering );

            char* t_f_work = buff;

            const char* delimiter = "/";

            char* t_f_token = strtok_r( t_f_work ,delimiter , (char**)&tmp_buff);
            std::vector <string> final_tokens;
            while( t_f_token != NULL )
            {
                //insert the token inside a vector
                items.push_back( t_f_token );
                t_f_token = strtok_r( NULL, delimiter , (char**)&tmp_buff);
            }

            // perform a deep copy of the old map before to change it
            filters_map fm(m_filt_map);

            // fill the map with token strings
            std::vector <string>::iterator
                it=items.begin(),
                end=items.end();

            for ( ; it!=end; ++it )
            {
                filter_key k;
                ulong l;
                // extract the key and the filter value
                extract( k, l, *it );

                // update the map
                filters_map::iterator
                    it = fm.find( k );

                if ( it != fm.end() )
                {
                    //pair is found, perform the necessary operations ...
                    if ( l > 0 )
                    {
                        //perform the OR operation on the bits ...
                        it->second |= l;
                    }
                    else
                    {
                        // lev_value = 0 , so reset all the levels ...
                        string
                            &component = k.first,
                            &domain    = k.second;
                        if ( 0 == component.compare("*") )
                        {
                            //[*,DOM]=level or [*,*]=level
                            //is '*' the domain value?
                            if ( 0 == domain.compare("*") )
                            {
                                // [*,*]=!
                                filters_map::iterator
                                cit  = fm.begin(),
                                cend = fm.end();
                                for( ;cit != cend ; ++cit )
                                {
                                    filter_key h = cit->first;
                                    fm[ h ] = 0;
                                }
                            }
                            else
                            {
                                //[*,DOM]=!
                                //so reset all levels for domain DOM ...
                                filters_map::iterator
                                cit  = fm.begin(),
                                cend = fm.end();
                                for( ;cit != cend ; ++cit )
                                {
                                    filter_key h = cit->first;
                                    if ( 0 == h.second.compare(k.second) )
                                    {
                                        fm[ h ] = 0;
                                    }
                                }
                            }
                        }
                        else
                        {
                            //[COMP,DOM]=! or [COMP,*]=!
                            //is '*' the domain value?
                            if ( 0 == domain.compare("*") )
                            {
                                // [COMP,*]=!
                                //perform the necessary operations ...
                                //so reset all levels for component COMP ...
                                filters_map::iterator
                                cit  = fm.begin(),
                                cend = fm.end();
                                for( ;cit != cend ; ++cit )
                                {
                                    filter_key h = cit->first;
                                    if ( 0 == h.first.compare(component) )
                                    {
                                        fm[ h ] = 0;
                                    }
                                }
                            }
                            else
                            {
                                //[COMP,DOM]=!
                                fm[ k ] = 0;
                            }
                        }
                    }
                }
                else
                {
                    //pair is not found, so:
                    //is '*' the component value?
                    string
                        &component = k.first,
                        &domain    = k.second;
                    if ( 0 == component.compare("*") )
                    {
                        //[*,DOM]=level or [*,*]=level
                        //is '*' the domain value?
                        if ( 0 == domain.compare("*") )
                        {
                            // [*,*]=level
                            //perform the necessary operations ...
                            if ( l > 0 )
                            {
                                // first of all perform the OR for all the entries in the map ...
                                filters_map::iterator
                                cit  = fm.begin(),
                                cend = fm.end();
                                for( ;cit != cend ; ++cit )
                                {
                                    cit->second |= l;
                                }
                                //add at the end of the map ...
                                fm[ k ] = l;        
                            }
                            else
                            {
                                // [*,*]=0 , so reset all levels ...
                                filters_map::iterator
                                cit  = fm.begin(),
                                cend = fm.end();
                                for( ;cit != cend ; ++cit )
                                {
                                    filter_key h = cit->first;
                                    fm[ h ] = 0;
                                }
                                //add at the end of the map ...
                                fm[ k ] = l;
                            }
                        }
                        else
                        {
                            //[*,DOM]=level
                            //perform the necessary operations ...
                            if ( l > 0 )
                            {
                                // first of all perform the OR for all the entries whith DOM1 in the map ...
                                filters_map::iterator
                                cit  = fm.begin(),
                                cend = fm.end();
                                for( ;cit != cend ; ++cit )
                                {
                                    filter_key h = cit->first;
                                    if ( 0 == h.second.compare(domain) )
                                    {
                                        cit->second |= l;
                                    }
                                }
                                //add at the end of the map ...
                                fm[ k ] = l;            
                            }
                            else
                            {
                                // [*,DOM1]=0
                                //so reset all levels for domain DOM ...
                                filters_map::iterator
                                cit  = fm.begin(),
                                cend = fm.end();
                                for( ;cit != cend ; ++cit )
                                {
                                    filter_key h = cit->first;
                                    if ( 0 == h.second.compare(k.second) )
                                    {
                                        fm[ h ] = 0;
                                    }
                                }
                            }
                        }
                    }
                    else
                    {
                        //[COMP,DOM]=level or [COMP,*]=level
                        //is '*' the domain value?
                        if ( 0 == domain.compare("*") )
                        {
                            // [COMP,*]=level
                            //perform the necessary operations ...
                            if ( l > 0 )
                            {
                                // first of all perform the OR for all the entries whith COMP_A in the map ...
                                filters_map::iterator
                                cit  = fm.begin(),
                                cend = fm.end();
                                for( ;cit != cend ; ++cit )
                                {
                                    filter_key h = cit->first;
                                    if ( 0 == h.first.compare(component) )
                                    {
                                        cit->second |= l;
                                    }
                                }
                                //add at the end of the map ...
                                fm[ k ] = l;
                            }
                            else
                            {
                                // [COMP_A,*]=0
                                //so reset all levels for component COMP ...
                                filters_map::iterator
                                cit  = fm.begin(),
                                cend = fm.end();
                                for( ;cit != cend ; ++cit )
                                {
                                    filter_key h = cit->first;
                                    if ( 0 == h.first.compare(component) )
                                    {
                                        fm[ h ] = 0;
                                    }
                                }
                            }
                        }
                        else
                        {
                            //[COMP,DOM]=level
                            fm[ k ] = l;
                        }
                    }
                }
            }
            
            //so supress all the Level = 0 entries ...
            filters_map::iterator
            cit  = fm.begin(),
            cend = fm.end();
            while ( !fm.empty() && cit != cend)
            {
                filter_key h = cit->first;
                if ( 0 == cit->second )
                {
                    filters_map::iterator tmp_it = cit;
                    cit++;
                    fm.erase( tmp_it );
                }
                else 
                {
                    cit++;                
                }
            }
            
            if ( fm.empty() ) 
            {              
                filter_key fk;  
                fk.first = "*";
                fk.second = "*";  
                fm.insert( std::make_pair( fk , 0 ) );
            }
            // commit change only if all is ok
            fm.swap(m_filt_map);

            //update the filters_ds in datastore
            set_record_in_datastore() ;

            //apply the filters...
            int ret_code = apply_trace_filters() ;

            switch(ret_code)
            {
                case -1:
                    throw CdmwPlatformMngt::CastException( "Error in dynamic_cast<ManagedProcess_impl *>" );
                    //return;
                    break;

                case -2: //case of ProcessDelegate not ready, so only the filters map was filled
                    return;
                    break;

                case 0: //all ok
                    break;
            }
        }



        int Process_impl::apply_trace_filters() throw (CORBA::SystemException)
        {
            //example
            //m_processDelegateWrap->activate_level("com1","dom1",8);
            ManagedProcess_impl *manag = dynamic_cast<ManagedProcess_impl *> (this);
            if (!manag)
            {
                return(-1);
            }

            /*test if m_processDelegateWrap is nil*/
            if ( CORBA::is_nil(manag->m_processDelegateWrap.in() ) )
            {
                //in apply_trace_filters m_processDelegateWrap is NIL
                return (-2);
            }

           //if filters map is empty, it returns
           if ( m_filt_map.size() == 0)
                 return (0);

            //analyzing the map to discover the activate to apply...

            //first of all: deactivate all levels in all components
            //and in all domains...
            manag->m_processDelegateWrap->deactivate_level(
                    Cdmw::Logging::ALL_COMPONENT_NAMES.c_str(),
                    Cdmw::Logging::ALL_DOMAINS.c_str(),
                    Cdmw::Logging::ALL_LEVELS_SET );

            //analyzing the map of filters to apply ...
            filters_map::const_iterator it = m_filt_map.begin();
            filters_map::const_iterator end = m_filt_map.end();
            my_key tmp;

            for(;it!=end;++it)
            {
                tmp = it->first;
                if ( (it->second) > 0)
                {
                    manag->m_processDelegateWrap->activate_level( tmp.first.c_str(), tmp.second.c_str(), it->second );
                }
                else
                { //deactivate all levels in the specific comp. and domain
                    manag->m_processDelegateWrap->deactivate_level( tmp.first.c_str(),
                                                                    tmp.second.c_str(),
                                                                    Cdmw::Logging::ALL_LEVELS_SET);
                }
            }
            return 0;
        }




        void Process_impl::stripLeadingAndTrailingBlanks(string& StringToModify)
        {
            if(StringToModify.empty()) return;

            int startIndex = StringToModify.find_first_not_of(" ");
            int endIndex = StringToModify.find_last_not_of(" ");
            string tempString = StringToModify;
            StringToModify.erase();

            StringToModify = tempString.substr(startIndex, (endIndex-startIndex+ 1) );
        }


        char * Process_impl::get_trace_filtering() throw (CORBA::SystemException)
        {
            if ( m_filt_map.size() == 0)
            {
                return NULL;
            }

            string final_str("") , tmp_str("") , cur1_str("");
            filters_map::const_iterator it  = m_filt_map.begin();
            filters_map::const_iterator end = m_filt_map.end();
            my_key tmp = it->first;
            char level_str[255+1];
            int ii;
            //reading actual filters from suitable map ...
            for(;it!=end;++it)
            {
                my_key tmp = it->first;
                cur1_str = tmp.first + "," + tmp.second;
                //see wich bits are up
                bitset<32>  bs (  it->second ) ;
                sprintf(&level_str[0],"%ld", it->second );
                for (ii=0; ii<32; ii++ )
                {
                    if ( bs.test(ii) == true )
                    {
                        sprintf(&level_str[0],"%d", ii );
                        final_str = final_str + cur1_str +","+ level_str+"/" ;
                    }
                }
            }

            size_t found = final_str.find_last_of( "/" );
            tmp_str = final_str.substr( 0,found );

            //end to fill string ...
            //return a suitable char pointer ...
            CORBA::String_var filt_str = CORBA::string_dup(tmp_str.c_str() );
            return (filt_str._retn());
        }

        void Process_impl::set_init_graph(const CdmwPlatformMngt::GraphElements& component_graph)
        throw (CORBA::SystemException)
        {
            // TODO
            throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
        }


        void Process_impl::set_stop_graph(const CdmwPlatformMngt::GraphElements& component_graph)
        throw (CORBA::SystemException)
        {
            // TODO
            throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
        }


        CdmwPlatformMngt::ElementType Process_impl::get_element_type()
            throw (CORBA::SystemException)
        {
                return CdmwPlatformMngt::PROCESS_TYPE;
        }

        CdmwPlatformMngt::Children* Process_impl::get_children()
            throw (CORBA::SystemException)
        {
                //return 0;

            //ARE_E_COFLIGHT_8_001
                Children_var children = new Children(0);
            return children._retn();
        }

        void Process_impl::set_created_dynamically(bool created_dynamically)
        {
            m_created_dynamically = created_dynamically;
        set_record_in_datastore();
        }


        CdmwPlatformMngt::CcmComponent_ptr Process_impl::add_component(const CdmwPlatformMngt::ComponentDef& component_def)
        throw (CORBA::SystemException, CdmwPlatformMngt::ComponentAlreadyExists, CdmwPlatformMngt::ComponentServerNotFound,
                CdmwPlatformMngt::DuplicateEntity, CosPropertyService::MultipleExceptions,
                CdmwPlatformMngt::IncompatibleStatus)
        {
           std::string mystr(component_def.name) ;
           //build the component path to pass
           //to CcmComponent_impl contructor ...
           std::string proc_name = name() ;

           Application_impl * myapp = get_application() ;

           std::string application_name = myapp->name() ;

           std::string host_name = get_process_host_name() ;

           std::string system_name = myapp->get_system_name() ;

           std::string component_path = ManagedElementKey::Get_component_path(
                                          system_name,
                                          application_name,
                                          host_name,
                                          proc_name,
                                          mystr) ;

           std::string component_key =  ManagedElementKey::Get_component_rel_key(
                                           host_name,
                                           proc_name,
                           mystr);


           //check if the name is already filled with
           //a component with the same name

           ComponentsMap::iterator componentIt =
                    m_component_map.find (component_key);

           if (componentIt != m_component_map.end ())
                    throw CdmwPlatformMngt::ComponentAlreadyExists();

           //check
           //std::cout<<"***********ELEMENT KEY OF COMPONENT="<<component_key<<std::endl;
           //std::cout<<"***********ELEMENT PATH OF COMPONENT="<<component_path<<std::endl;

           //try to instance component
           CcmComponent_impl *component =
                    new  CcmComponent_impl(m_poa.in (),
                          component_def,
                            m_element_repository.in(),
                component_key.c_str(),
                component_path.c_str(),
                this );

           //std::cout<<"***COMPONENT PTR = "<<component<<std::endl;

           CdmwPlatformMngt::CcmComponent_var mycomponent = component->_this ();

           // add in repository the process object

           m_element_repository->add_element (component_path.c_str (),
                    CdmwPlatformMngt::COMPONENT_TYPE,
                    mycomponent.in() );
       
           //fill the map of <std::string component_key, CcmComponent*>
           //with component key

           m_component_map[ component_key ] = component;
           //m_component_map->insert (mystr, component);

           component->set_record_in_datastore();
           component->end_creation(component_def);
           return mycomponent._retn() ;
    //       CdmwPlatformMngt::CcmComponent_var comp_var = component->_this() ;
    //       return comp_var._retn() ;
        }

        void Process_impl::remove_component(const char* component_name, const char* host_name)
            throw(CORBA::SystemException, CdmwPlatformMngt::ComponentNotFound,
                CdmwPlatformMngt::ComponentStillActive, CdmwPlatformMngt::IncompatibleStatus)
        {
        // TODO
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
        }

        void Process_impl::get_all_components(CORBA::ULong how_many,
                                        CdmwPlatformMngt::CcmComponents_out ncomponents,
                                        CdmwPlatformMngt::ComponentsIterator_out rest)
            throw(CORBA::SystemException,
                CdmwPlatformMngt::NotComponentServer)
        {


             // create the requested sequence of components
             CdmwPlatformMngt::CcmComponents_var requested_components =
                new CdmwPlatformMngt::CcmComponents;

             std::list <CdmwPlatformMngt::CcmComponent_var> rest_of_components;

             // get length of list
             size_t len = m_component_map.size();

             if (len > 0)
             {
                     // set length of sequence
                 //requested_components->length(len);

                 // initialize list iterator
                 ComponentsMap::iterator it =
                              m_component_map.begin();


                 for (CORBA::ULong i = 0; i < len; i++)
                 {
                    // get entity from container list using iterator
                    // and convert to Component object reference
                    CdmwPlatformMngt::CcmComponent_var component = (it->second)->_this();

                    // store entity in sequence
                    if( i < how_many)
              {
              requested_components->length(i+1) ;
                      requested_components[i] = component._retn();
              }
                    else
                      rest_of_components.push_back(component) ;



                    // increment iterator
                    it++;
                 }
             }

            // create the EntityIterator with the rest of list
             CdmwPlatformMngt::ComponentsIterator_var components_iter =
                                ComponentsIterator_impl::create (rest_of_components);

             ncomponents = requested_components._retn() ;
             rest = components_iter._retn();

        }

        CdmwPlatformMngt::CcmComponent_ptr Process_impl::get_component(const char* component_name)
            throw(CORBA::SystemException,
                CdmwPlatformMngt::ComponentNotFound,
                CdmwPlatformMngt::NotComponentServer)
        {
           std::string mystr(component_name) ;
           //get component key
           std::string proc_name = name() ;
           std::string host_name = get_process_host_name() ;

           std::string component_key =  ManagedElementKey::Get_component_rel_key(
                                           host_name,
                                           proc_name,
                           mystr);

           ComponentsMap::iterator componentIt =
                    m_component_map.find (component_key);

           //check if component does not exists
           if (componentIt == m_component_map.end ())
                    throw CdmwPlatformMngt::ComponentNotFound();

           CcmComponent_var component = (componentIt->second)->_this()  ;

           return component._retn() ;

        }

        CcmComponent_impl* Process_impl::get_component_impl(const char* component_name)
                throw(CORBA::SystemException,
                      CdmwPlatformMngt::ComponentNotFound,
              CdmwPlatformMngt::NotComponentServer)
       {
            std::string mystr(component_name);
            std::string proc_name = name();
        std::string host_name = get_process_host_name() ;
        std::string component_key =  ManagedElementKey::Get_component_rel_key(host_name,
                                                                              proc_name,
                                              mystr);

            ComponentsMap::iterator componentIt =
                m_component_map.find (component_key);

             //check if component does not exists
            if (componentIt == m_component_map.end ())
                   throw CdmwPlatformMngt::ComponentNotFound();

            CcmComponent_impl* component = componentIt->second;
            return component;
       }


        CORBA::ULong Process_impl::get_number_of_components()
            throw(CORBA::SystemException, CdmwPlatformMngt::NotComponentServer)
        {
            CORBA::ULong len = m_component_map.size();
            return len;
        }

         void Process_impl::set_proc_info(Cdmw::PlatformMngt::ProcessHostInfo info, char flag)
        {
             m_kind_of_start = info.kind_of_start;

             set_record_in_datastore();

             switch (flag)
           {
               case 0:
             {
         //case of set_init_entry and set_run_entry                      
                 HostProcessInitSequencer::get_instance()->set_init_entry(get_host()->name(),info);
                 HostProcessInitSequencer::get_instance()->set_run_entry(get_host()->name(),info);
                 }                 
                 break;

               case 1: 
             {
         //case of remove_init_entry and remove_run_entry
                 HostProcessInitSequencer::get_instance()->remove_init_entry(get_host()->name(),info);
                 HostProcessInitSequencer::get_instance()->remove_run_entry(get_host()->name(),info);
                 
         }
         break;

               case 2: //case of only set_init_entry                             
             {
         HostProcessInitSequencer::get_instance()->set_init_entry(get_host()->name(),info);                           
                 }
         break;
           }
                      
        }

    std::vector<std::string>*  Process_impl::get_deployment_plans()
    {
          ComponentsMap::iterator component_map_it  = m_component_map.begin();
          ComponentsMap::iterator component_map_it_end = m_component_map.end();

          set<std::string>  deployment_plans;

          while(component_map_it!=component_map_it_end)
          {
              CcmComponent_impl* component = component_map_it->second;
              //get the deployment plan
              std::string deployment_plan = component->deployment_plan();

              set<std::string>::iterator deployment_plans_it_found  = deployment_plans.find(deployment_plan);
              set<std::string>::iterator deployment_plans_it_end  = deployment_plans.find(deployment_plan);

              if (deployment_plans_it_found == deployment_plans_it_end)
              {
                  deployment_plans.insert(deployment_plan);
              }
              ++component_map_it;
          }


          std::vector<std::string>* vector_of_deployment_plans = 0;
          set<std::string>::iterator set_iterator = deployment_plans.begin();
          set<std::string>::iterator set_iterator_end = deployment_plans.end();
          if (set_iterator!= set_iterator_end)
             vector_of_deployment_plans = new std::vector<std::string>;
          for (unsigned int i=0; i<deployment_plans.size(); i++)
          {
               std::string deployment_plan = *set_iterator;
               vector_of_deployment_plans->push_back(deployment_plan);
               ++set_iterator;
           }
          return vector_of_deployment_plans;

    //      return deployment_plans;
        }


    /*
    void Process_impl::set_requested_deployment_plans (const char * deployment_plan)
    {

        CDMW_DEBUG(m_logger, " set_requested_deploymnet_plans " << name() << " deployment_plan:" << deployment_plan);
        ComponentsMap::iterator component_map_it  = m_component_map.begin();
        ComponentsMap::iterator component_map_it_end = m_component_map.end();

        while(component_map_it!=component_map_it_end)
        {
             CcmComponent_impl* component = component_map_it->second;

             //std::string deployment_plan_found = component->deployment_plan();

             //if ( deployment_plan_found == string(deployment_plan) )
             //{
             component -> set_requested_deployment_plan( deployment_plan );
             //}
             ++component_map_it;
         }
    }
    */
        void Process_impl::stopping_components()
        {
         // CDMW_INFO(m_logger, " stopping_components  on " << name());

         ComponentsMap::iterator component_map_it  = m_component_map.begin();
         ComponentsMap::iterator component_map_it_end = m_component_map.end();

         while(component_map_it!=component_map_it_end)
         {
              CcmComponent_impl* component = component_map_it->second;

              try
              {
                   component -> stopping ();
              }
              catch(const CdmwPlatformMngt::IncompatibleStatus& ex)
              {
                  CDMW_INFO(m_logger, " IncompatibleStatus  on " << name());
              }

              ++component_map_it;
          }
        }

        void Process_impl::stop_components()
        {
         // CDMW_INFO(m_logger, " stop_components on " << name());
         ComponentsMap::iterator component_map_it  = m_component_map.begin();
         ComponentsMap::iterator component_map_it_end = m_component_map.end();
         while(component_map_it!=component_map_it_end)
         {
              CcmComponent_impl* component = component_map_it->second;
              CORBA::String_var statusInfo;
              std::string deployment_plan_found = component->deployment_plan();
              try
              {
                component -> stop ();
              }
              catch(const CdmwPlatformMngt::IncompatibleStatus& e)
              {
                    CDMW_INFO(m_logger, e._name());
              }
              ++component_map_it;
          }
        }

         void Process_impl::failed_error_components()
        {
          CDMW_INFO(m_logger, " failed_error_components on " << name());
         ComponentsMap::iterator component_map_it  = m_component_map.begin();
         ComponentsMap::iterator component_map_it_end = m_component_map.end();

         while(component_map_it!=component_map_it_end)
         {
              CcmComponent_impl* component = component_map_it->second;

              std::string deployment_plan_found = component->deployment_plan();

                component -> failed_error ();

                ++component_map_it;
          }
        }

        void Process_impl::set_pid(CdmwPlatformMngt::ProcessPID pid)
        {
            m_previous_pid = m_pid;
            m_pid = pid;
        }

        // implemented in ManagedProcess_impl
        void Process_impl::initialise_components (const char * deployment_plan)
        {
            throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
        }

        // implemented in ManagedProcess_impl
        void Process_impl::set_state(const std::string& state_name) throw()
        {
            throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
        }

        void Process_impl::set_requested_restart_warmth(CdmwPlatformMngt::RestartWarmth requested_restart_warmth)
        {

            ComponentsMap::iterator component_map_it  = m_component_map.begin();
            ComponentsMap::iterator component_map_it_end = m_component_map.end();

            while(component_map_it!=component_map_it_end)
            {
                CcmComponent_impl* component = component_map_it->second;
                component -> set_requested_restart_warmth(requested_restart_warmth);
                ++component_map_it;
            }

        this->ManagedElement_impl::set_requested_restart_warmth(requested_restart_warmth);

        }

        const char* Process_impl::get_sup_host_path ()
        {
            const char* result = 0 ;
            CORBA::Any_var property_any = new CORBA::Any();
        
    //CdmwPlatformMngt::ManagedElement_var parent_var = CdmwPlatformMngt::System::_narrow(m_application->get_parent());
    CdmwPlatformMngt::ManagedElement_var p_var = m_application->get_parent();
        CdmwPlatformMngt::ManagedElement_var parent_var = CdmwPlatformMngt::System::_narrow(p_var.in());
    
        parent_var->get_attribute (CdmwPlatformMngt::GLOBAL_SUPERVISION_HOST, property_any.out(),             CdmwPlatformMngt::PRE_DEFINED);       

            property_any.in() >>= result;
            return  (ManagedElementKey::Get_host_path(m_application->get_system_name (),
                    result)).c_str();
        }

    } // End namespace PlatformMngt
} // End namespace Cdmw
