/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "platformapplicationlibrary/UnmanagedProcess_impl.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtEvent.stub.hpp"
#include "SystemMngt/platformlibrary/EventHeaderFactory.hpp"
#include "SystemMngt/platformlibrary/LogMngr.hpp"
#include "SystemMngt/platformapplicationlibrary/Application_impl.hpp"
#include "SystemMngt/platformapplicationlibrary/ApplicationLogMessageId.hpp"
#include "platformapplicationlibrary/InitTimeoutObserver.hpp"
#include <sstream>
#include <string>



namespace Cdmw
{
namespace PlatformMngt
{

// ----------------------------------------------------------------------
// UnmanagedProcessState class.
// ----------------------------------------------------------------------
char* UnmanagedProcessState::status_info(
    UnmanagedProcessStateMachine* stateMachine )
{
    CORBA::String_var statusInfo = CORBA::string_dup
        (stateMachine->getContext()->get_status_info());

    return statusInfo._retn();
}

void UnmanagedProcessState::invalidate_event(
        UnmanagedProcessStateMachine* stateMachine,
    const CdmwPlatformMngtBase::EventHeader& header,
    const char* reason )
{
    stateMachine->getContext()->invalidateEvent(header, reason );
}

// ----------------------------------------------------------------------
// UnmanagedProcessInitial class.
// ----------------------------------------------------------------------
UnmanagedProcessInitial::UnmanagedProcessInitial()
{
}

CdmwPlatformMngt::ProcessStatus UnmanagedProcessInitial::status()
{
    return CdmwPlatformMngt::PROCESS_INITIAL;
}

void UnmanagedProcessInitial::set_autoending(
    UnmanagedProcessStateMachine* stateMachine,
    CORBA::Boolean auto_ending )
throw( CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException )
{
    stateMachine->getContext()->setAutoending(auto_ending);

    // TODO: Notify the configuration change

    // TODO: Trace to the log manager
}

void UnmanagedProcessInitial::set_host(
    UnmanagedProcessStateMachine* stateMachine,
    const char* host_name )
throw( CdmwPlatformMngt::HostNotFound,
       CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException )
{
    stateMachine->getContext()->setHost(host_name );

    // TODO: Notify the configuration change

    // TODO: Trace to the log manager
}

void UnmanagedProcessInitial::set_process_info(
        UnmanagedProcessStateMachine* stateMachine,
    const CdmwPlatformMngt::ProcessInfo& process_info )
throw( CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException )
{
    stateMachine->getContext()->setProcessInfo(process_info );

    // TODO: Notify the configuration change

    // TODO: Trace to the log manager
}

CdmwPlatformMngt::RequestStatus UnmanagedProcessInitial::initialise(
        UnmanagedProcessStateMachine* stateMachine,
    const CdmwPlatformMngtBase::StartupKind& startup_kind )
throw( CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException )
{
    // Change to the state Initialising
    stateMachine->setState("Initialising");

    return stateMachine->getContext()->m_lastRequestStatus;
}

// ----------------------------------------------------------------------
// UnmanagedProcessInitialising class.
// ----------------------------------------------------------------------
UnmanagedProcessInitialising::UnmanagedProcessInitialising()
{
}

void UnmanagedProcessInitialising::entryAction(
    UnmanagedProcessStateMachine* stateMachine )
{
    UnmanagedProcess_impl* process = stateMachine->getContext();

    // Log and notify the change of status
    process->set_status_info ( CORBA::string_dup(""));

    CdmwPlatformMngtBase::EventHeader header
        = EventHeaderFactory::createHeader( CdmwPlatformMngtBase::INF );

    process->notifyStatusChangeEvent(
            header,
            CdmwPlatformMngtBase::PFMMNG );

}

void UnmanagedProcessInitialising::doActivity(
        UnmanagedProcessStateMachine* stateMachine) 
{
    UnmanagedProcess_impl* process = stateMachine->getContext();

    process->m_lastRequestStatus = CdmwPlatformMngt::REQUEST_ERROR;

    const char* p_applicationName = process->get_application()->get_element_name();
    const char* p_processName = process->get_element_name();
    CdmwPlatformMngt::Host_var host = process->get_host();
    CORBA::String_var processHostName = host->name();
    
    try
    {
        // Create the unmanaged process
        CdmwPlatformMngt::ApplicationAgent_var agent
        = process->getApplicationAgent();

        CdmwPlatformMngt::ProcessInfo_var processInfo =
        process->get_process_info();
        processInfo->requested_restart_warmth= process->requested_restart_warmth();
        CORBA::String_var result_info;

        CdmwPlatformMngt::ProcessCommandResult creation_result =
        agent->create_process (p_processName, processInfo.in(), result_info.out());
            try
        {
            process -> set_pid ( agent->get_pid (p_processName) ); 
        }
            catch(const CdmwPlatformMngt::ProcessNotFound& p_not_f)
        {
            std::string message = "Can't set pid for an invalid Process: ";
        message += p_processName;
        LogMngr::logMessage (WRN, message.c_str());
        }

        unsigned long init_timeout=process->get_initialisation_timeout();       

        if (creation_result == CdmwPlatformMngt::PROCESS_CMD_SUCCEED ||
                creation_result == CdmwPlatformMngt::PROCESS_ALREADY_EXIST)
        {
            if(0!=init_timeout) 
            {
                // Change to state RUNNING only at timer expiration             
                process->m_init_mutex=new OsSupport::Mutex();
                process->m_init_condition=new OsSupport::Condition(*process->m_init_mutex);
                                
                //Creating the timer Observer
                std::auto_ptr <InitTimeoutObserver>
                initObserver ( new InitTimeoutObserver(stateMachine,process,process->m_init_mutex,process->m_init_condition));
                            //m_observer = initObserver.release();
                             InitTimeoutObserver* m_observer = initObserver.release();
                
                //Creating the initialisation Timer 
                std::auto_ptr <SingleShotTimer>
                initTimer ( new SingleShotTimer(m_observer,init_timeout) );
                process->m_timer = initTimer.release();

                //Starting the Timer
                process->m_timer->start();
                process->m_timer->start_timer();
                process->m_lastRequestStatus = CdmwPlatformMngt::REQUEST_TIMEOUT;
                std::string msg= "A:";
                msg.append(process->get_application()->get_element_name());
                msg.append(" P:");
                msg.append(process->name());                
                msg.append(" waiting for the initialisation-timeout... ");              
                LogMngr::logMessage (INF,msg.c_str());
                

            }
            // If the initialisation_timeout is null is set the Running state
            else  
            {               
            // Change to state RUNNING
                //stateMachine->setState( "Running" );
                            process->setState("Running");
                process->m_lastRequestStatus = CdmwPlatformMngt::REQUEST_SUCCEED;
            }

        }
        else if ( creation_result == CdmwPlatformMngt::PROCESS_INVALID_INFO )
        {
            // Set the info about the issuance of this error event.
            process->set_status_info ( CORBA::string_dup (result_info.in()));

            // Store the info about the issuance of this error event.
            process->m_lastErrorHeader =
            EventHeaderFactory::createHeader (CdmwPlatformMngtBase::ERR);

            // Change to state INVALID
            stateMachine->setState( "FailedInvalid" );
        }

    }
    catch( const CdmwPlatformMngt::HostNotFound& e )
    {
        // Set the info about the issuance of this error event.
        std::string error_info =
        LogMngr::getFormattedMessage(
                MSG_ID_HOST_NOT_FOUND,
                processHostName.in() );
        process->set_status_info ( CORBA::string_dup( error_info.c_str() ));

        // Store the info about the issuance of this error event.
        process->m_lastErrorHeader =
        EventHeaderFactory::createHeader( CdmwPlatformMngtBase::ERR );

        // Change to state INVALID
        stateMachine->setState( "FailedInvalid" );
    }
    catch( const CdmwPlatformMngt::HostNotReachable& e )
    {
        // Set the info about the issuance of this error event.
        std::string error_info =
        LogMngr::getFormattedMessage(
                MSG_ID_HOST_UNREACHABLE,
                processHostName.in() );
        process->set_status_info ( CORBA::string_dup( error_info.c_str() ));

        // Store the info about the issuance of this error event.
        process->m_lastErrorHeader =
        EventHeaderFactory::createHeader( CdmwPlatformMngtBase::ERR );

        // Change to state INVALID
        stateMachine->setState( "FailedInvalid" );
    }
    catch( const CdmwPlatformMngt::CreationError& e )
    {
        // Set the info about the issuance of this error event.
        process->set_status_info ( CORBA::string_dup( e.error_info.in() ));

        // Store the info about the issuance of this error event.
        process->m_lastErrorHeader =
        EventHeaderFactory::createHeader( CdmwPlatformMngtBase::ERR );

        // Change to state INVALID
        stateMachine->setState( "FailedInvalid" );
    }
    catch( const CdmwPlatformMngt::ApplicationAgentAlreadyExists& e )
    {
        // Set the info about the issuance of this error event.
        std::string error_info =
        LogMngr::getFormattedMessage(
                MSG_ID_AGENT_EXISTS,
                p_applicationName,
                processHostName.in() );
        process->set_status_info ( CORBA::string_dup( error_info.c_str() ));

        // Store the info about the issuance of this error event.
        process->m_lastErrorHeader =
        EventHeaderFactory::createHeader( CdmwPlatformMngtBase::ERR );

        // Change to state INVALID
        stateMachine->setState( "FailedInvalid" );
    }
    catch( const AgentNotFoundException& e )
    {
        // Set the info about the issue of this error event.
        process->set_status_info ( CORBA::string_dup( e.what() ));

        // Store the info about the issue of this error event.
        process->m_lastErrorHeader =
        EventHeaderFactory::createHeader( CdmwPlatformMngtBase::ERR );

        // Change to state INVALID
        stateMachine->setState( "FailedInvalid" );
    }
    catch( const CORBA::SystemException& e )
    {
        std::ostringstream exceptionInfo;
        exceptionInfo << e;

        // Set the info about the issue of this error event.
        process->set_status_info ( CORBA::string_dup( exceptionInfo.str().c_str() ));

        // Store the info about the issue of this error event.
        process->m_lastErrorHeader =
        EventHeaderFactory::createHeader( CdmwPlatformMngtBase::ERR );

        // Change to state INVALID
        stateMachine->setState( "FailedInvalid" );
    }
}

CdmwPlatformMngt::ProcessStatus UnmanagedProcessInitialising::status() 
{
    return CdmwPlatformMngt::PROCESS_INITIALISING;
}

void UnmanagedProcessInitialising::ending_event(
        UnmanagedProcessStateMachine* stateMachine,
        const CdmwPlatformMngtBase::EventHeader& header) 
{
    UnmanagedProcess_impl* process = stateMachine->getContext();
    process->m_timer->stop();
    process->endingEvent(header);

        CDMW_MUTEX_GUARD( *process->m_init_mutex );
        process->m_lastRequestStatus = CdmwPlatformMngt::REQUEST_SUCCEED; 

        process->m_init_condition->broadcast();

}

void UnmanagedProcessInitialising::exitAction( UnmanagedProcessStateMachine* stateMachine ){

}


// ----------------------------------------------------------------------
// UnmanagedProcessRunning class.
// ----------------------------------------------------------------------
UnmanagedProcessRunning::UnmanagedProcessRunning() 
{
}

void UnmanagedProcessRunning::entryAction(
         UnmanagedProcessStateMachine* stateMachine) 
{
    UnmanagedProcess_impl* process = stateMachine->getContext();
    //The number of restart attempts is set to zero after the initialisation timeout 
    process->set_restart_attempt(0);
    
    // Log and notify the change of status
    process->set_status_info ( CORBA::string_dup( "" ));
    
    CdmwPlatformMngtBase::EventHeader header
        = EventHeaderFactory::createHeader( CdmwPlatformMngtBase::INF );

    process->notifyStatusChangeEvent(
            header, 
            CdmwPlatformMngtBase::PFMMNG );

    process->get_application()->process_status_change_event(process->name(),  process->get_process_host_name(), status());
}

CdmwPlatformMngt::ProcessStatus
UnmanagedProcessRunning::status()
{
    return CdmwPlatformMngt::PROCESS_RUNNING;
}


void UnmanagedProcessRunning::doActivity(
        UnmanagedProcessStateMachine* stateMachine ){
        UnmanagedProcess_impl* process = stateMachine->getContext();
    process->set_restart_attempt(0);
        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                            "<<UnmanagedProcessStates::UnmanagedProcessRunning::doActivity >> The number of restart attempts is set to zero ");
}


CdmwPlatformMngt::RequestStatus UnmanagedProcessRunning::stop(
    UnmanagedProcessStateMachine* stateMachine,
    CORBA::Boolean emergency  )
throw( CORBA::SystemException )
{
    UnmanagedProcess_impl* process = stateMachine->getContext();

    // Change to the state Stopped
    stateMachine->getContext()->m_stop_emergency = emergency;
    stateMachine->setState( "Stopped" );
  
    return process->m_lastRequestStatus;
}

void UnmanagedProcessRunning::ending_event(
    UnmanagedProcessStateMachine* stateMachine,
    const CdmwPlatformMngtBase::EventHeader& header )
{
    UnmanagedProcess_impl* process = stateMachine->getContext();
    process->endingEvent(header);
}

// ----------------------------------------------------------------------
// UnmanagedProcessStopped class.
// ----------------------------------------------------------------------
UnmanagedProcessStopped::UnmanagedProcessStopped()
{
}

void UnmanagedProcessStopped::entryAction(
    UnmanagedProcessStateMachine* stateMachine) 
{
    UnmanagedProcess_impl* process = stateMachine->getContext();

    // Log and notify the change of status
    process->set_status_info ( CORBA::string_dup(""));

    CdmwPlatformMngtBase::EventHeader header
        = EventHeaderFactory::createHeader( CdmwPlatformMngtBase::INF );

    process->notifyStatusChangeEvent(
            header,
            CdmwPlatformMngtBase::PFMMNG );

    process->m_lastRequestStatus = CdmwPlatformMngt::REQUEST_SUCCEED;

    CdmwPlatformMngt::Host_var host = process->get_host();
    CORBA::String_var processHostName = host->name();

    try
    {
        // Kill the OS process
        CdmwPlatformMngt::ApplicationAgent_var agent
        = process->getApplicationAgent();

        int signal = SIGTERM;

        if (stateMachine->getContext()->m_stop_emergency){
            signal = SIGABRT;
        }

        agent->kill_process( process->get_element_name(), signal);
    }
    catch( const CdmwPlatformMngt::HostNotFound& )
    {
    }
    catch( const CdmwPlatformMngt::HostNotReachable& )
    {
    }
    catch( const CdmwPlatformMngt::ProcessNotFound& )
    {
    }
    catch( ... )
    {
        // Set the info about the issuance of this error event.
        std::string error_info =
        LogMngr::getFormattedMessage(
                MSG_ID_PROCESS_KILL_ERROR,
                process->get_element_name(),
                processHostName.in() );

        // Log the error
        LogMngr::logMessage( ERR, error_info.c_str() );
    }
    process->get_application()->process_status_change_event(process->name(),  process->get_process_host_name(), status());

}

CdmwPlatformMngt::ProcessStatus
UnmanagedProcessStopped::status()
{
    return CdmwPlatformMngt::PROCESS_STOPPED;
}

void UnmanagedProcessStopped::set_autoending(
    UnmanagedProcessStateMachine* stateMachine,
    CORBA::Boolean auto_ending )
throw( CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException )
{
    stateMachine->getContext()->setAutoending(auto_ending);

    // TODO: Notify the configuration change

    // TODO: Trace to the log manager
}

void UnmanagedProcessStopped::set_host(
    UnmanagedProcessStateMachine* stateMachine,
    const char* host_name )
throw( CdmwPlatformMngt::HostNotFound,
       CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException )
{
    stateMachine->getContext()->setHost(host_name );

    // TODO: Notify the configuration change

    // TODO: Trace to the log manager
}

void UnmanagedProcessStopped::set_process_info(
        UnmanagedProcessStateMachine* stateMachine,
    const CdmwPlatformMngt::ProcessInfo& process_info )
throw( CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException )
{
    stateMachine->getContext()->setProcessInfo(process_info );

    // TODO: Notify the configuration change

    // TODO: Trace to the log manager
}

CdmwPlatformMngt::RequestStatus UnmanagedProcessStopped::initialise(
        UnmanagedProcessStateMachine* stateMachine,
    const CdmwPlatformMngtBase::StartupKind& startup_kind )
throw( CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException )
{
    // Change to the state Initialising
    stateMachine->setState("Initialising");

    return stateMachine->getContext()->m_lastRequestStatus;
}

void UnmanagedProcessStopped::ending_event(
        UnmanagedProcessStateMachine* stateMachine,
        const CdmwPlatformMngtBase::EventHeader& header) 
{
    UnmanagedProcess_impl* process = stateMachine->getContext();
    process->setStopStatus(CdmwPlatformMngt::LONG_REQUEST_COMPLETED );
}

// ----------------------------------------------------------------------
// UnmanagedProcessEnded class.
// ----------------------------------------------------------------------
UnmanagedProcessEnded::UnmanagedProcessEnded() 
{
}

void UnmanagedProcessEnded::entryAction(
        UnmanagedProcessStateMachine* stateMachine) 
{
    UnmanagedProcess_impl* process = stateMachine->getContext();

    // Log and notify the change of status
    process->set_status_info ( CORBA::string_dup(""));

    CdmwPlatformMngtBase::EventHeader header
        = EventHeaderFactory::createHeader( CdmwPlatformMngtBase::INF );

    process->notifyStatusChangeEvent(
            header,
            CdmwPlatformMngtBase::PFMMNG );
}

CdmwPlatformMngt::ProcessStatus
UnmanagedProcessEnded::status()
{
    return CdmwPlatformMngt::PROCESS_ENDED;
}

void UnmanagedProcessEnded::set_autoending(
    UnmanagedProcessStateMachine* stateMachine,
    CORBA::Boolean auto_ending )
throw( CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException )
{
    stateMachine->getContext()->setAutoending(auto_ending);

    // TODO: Notify the configuration change

    // TODO: Trace to the log manager
}

void UnmanagedProcessEnded::set_host(
    UnmanagedProcessStateMachine* stateMachine,
    const char* host_name )
throw( CdmwPlatformMngt::HostNotFound,
       CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException )
{
    stateMachine->getContext()->setHost(host_name );

    // TODO: Notify the configuration change

    // TODO: Trace to the log manager
}

void UnmanagedProcessEnded::set_process_info(
        UnmanagedProcessStateMachine* stateMachine,
    const CdmwPlatformMngt::ProcessInfo& process_info )
throw( CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException )
{
    stateMachine->getContext()->setProcessInfo(process_info );

    // TODO: Notify the configuration change

    // TODO: Trace to the log manager
}

CdmwPlatformMngt::RequestStatus UnmanagedProcessEnded::stop(
    UnmanagedProcessStateMachine* stateMachine,
    CORBA::Boolean emergency )
throw( CORBA::SystemException )
{
    UnmanagedProcess_impl* process = stateMachine->getContext();

    process->m_lastRequestStatus = CdmwPlatformMngt::REQUEST_SUCCEED;

    try
    {
        // call kill_process to clean all process info in agent
        CdmwPlatformMngt::ApplicationAgent_var agent
        = process->getApplicationAgent();
        
        int signal = SIGTERM;
        
        CdmwPlatformMngt::ProcessInfo_var processInfo = process->get_process_info();
                               
        if  (CdmwPlatformMngt::ABORT_PROCESS==(processInfo->exit_autorestart_info).abnormal_behaviour_action){
            signal = SIGABRT;
        }


        agent->kill_process(process->get_element_name(), signal);
    }
    catch( const CdmwPlatformMngt::HostNotFound& )
    {
    }
    catch( const CdmwPlatformMngt::HostNotReachable& )
    {
    }
    catch( const CdmwPlatformMngt::ProcessNotFound& )
    {
    }
    catch( ... )
    {
    }

    return CdmwPlatformMngt::REQUEST_SUCCEED;
}

CdmwPlatformMngt::RequestStatus UnmanagedProcessEnded::initialise(
        UnmanagedProcessStateMachine* stateMachine,
    const CdmwPlatformMngtBase::StartupKind& startup_kind )
throw( CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException )
{
    // Change to the state Initialising
    stateMachine->setState("Initialising");

    return stateMachine->getContext()->m_lastRequestStatus;
}

// ----------------------------------------------------------------------
// UnmanagedProcessFailedDeath class.
// ----------------------------------------------------------------------
UnmanagedProcessFailedDeath::UnmanagedProcessFailedDeath()
{
}

void UnmanagedProcessFailedDeath::entryAction(
    UnmanagedProcessStateMachine* stateMachine )
{
    UnmanagedProcess_impl* process = stateMachine->getContext();

    // Log and notify the change of status
    process->notifyStatusChangeEvent(
        process->m_lastErrorHeader,
            CdmwPlatformMngtBase::PFMMNG );

    // Change the application's mode to degraded
    process->get_application()->degradation_event();

    try
    {
        // call kill_process to clean all process info in agent
        CdmwPlatformMngt::ApplicationAgent_var agent
        = process->getApplicationAgent();
        
        int signal = SIGTERM;

        agent->kill_process(process->get_element_name(), signal);
    }
    catch( const CdmwPlatformMngt::HostNotFound& )
    {
    }
    catch( const CdmwPlatformMngt::HostNotReachable& )
    {
    }
    catch( const CdmwPlatformMngt::ProcessNotFound& )
    {
    }
    catch( ... )
    {
    }
    process->get_application()->process_status_change_event(process->name(), process->get_process_host_name(), status());
}

void UnmanagedProcessFailedDeath::completionTransition(
        UnmanagedProcessStateMachine* stateMachine )
{
    //stateMachine->getContext()->perform_exit_autorestart_action();
}


CdmwPlatformMngt::ProcessStatus
UnmanagedProcessFailedDeath::status()
{
    return CdmwPlatformMngt::PROCESS_FAILED_DEATH;
}

void UnmanagedProcessFailedDeath::set_autoending(
    UnmanagedProcessStateMachine* stateMachine,
    CORBA::Boolean auto_ending )
throw( CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException )
{
    stateMachine->getContext()->setAutoending(auto_ending);

    // TODO: Notify the configuration change

    // TODO: Trace to the log manager
}

void UnmanagedProcessFailedDeath::set_host(
    UnmanagedProcessStateMachine* stateMachine,
    const char* host_name )
throw( CdmwPlatformMngt::HostNotFound,
       CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException )
{
    stateMachine->getContext()->setHost(host_name );

    // TODO: Notify the configuration change

    // TODO: Trace to the log manager
}

void UnmanagedProcessFailedDeath::set_process_info(
        UnmanagedProcessStateMachine* stateMachine,
    const CdmwPlatformMngt::ProcessInfo& process_info )
throw( CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException )
{
    stateMachine->getContext()->setProcessInfo(process_info );

    // TODO: Notify the configuration change

    // TODO: Trace to the log manager
}

CdmwPlatformMngt::RequestStatus
UnmanagedProcessFailedDeath::initialise(
        UnmanagedProcessStateMachine* stateMachine,
    const CdmwPlatformMngtBase::StartupKind& startup_kind )
throw( CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException )
{
    // Change to the state Initialising
    stateMachine->setState("Initialising");

    return stateMachine->getContext()->m_lastRequestStatus;
}

// ----------------------------------------------------------------------
// UnmanagedProcessFailedInvalid class.
// ----------------------------------------------------------------------
UnmanagedProcessFailedInvalid::UnmanagedProcessFailedInvalid()
{
}

void UnmanagedProcessFailedInvalid::entryAction(
    UnmanagedProcessStateMachine* stateMachine )
{
    UnmanagedProcess_impl* process = stateMachine->getContext();

    // Log and notify the change of status
    process->notifyStatusChangeEvent(
        process->m_lastErrorHeader,
            CdmwPlatformMngtBase::PFMMNG );

    // Change the application's mode to degraded
    process->get_application()->degradation_event();

    process->get_application()->process_status_change_event(process->name(), process->get_process_host_name(), status());
}

void UnmanagedProcessFailedInvalid::completionTransition(
        UnmanagedProcessStateMachine* stateMachine )
{
    //stateMachine->getContext()->perform_exit_autorestart_action();
}

CdmwPlatformMngt::ProcessStatus
UnmanagedProcessFailedInvalid::status()
{
    return CdmwPlatformMngt::PROCESS_FAILED_INVALID;
}

void UnmanagedProcessFailedInvalid::set_autoending(
    UnmanagedProcessStateMachine* stateMachine,
    CORBA::Boolean auto_ending )
throw( CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException )
{
    stateMachine->getContext()->setAutoending(auto_ending);

    // TODO: Notify the configuration change

    // TODO: Trace to the log manager
}

void UnmanagedProcessFailedInvalid::set_host(
    UnmanagedProcessStateMachine* stateMachine,
    const char* host_name )
throw( CdmwPlatformMngt::HostNotFound,
       CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException )
{
    stateMachine->getContext()->setHost(host_name );

    // TODO: Notify the configuration change

    // TODO: Trace to the log manager
}

void UnmanagedProcessFailedInvalid::set_process_info(
        UnmanagedProcessStateMachine* stateMachine,
    const CdmwPlatformMngt::ProcessInfo& process_info )
throw( CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException )
{
    stateMachine->getContext()->setProcessInfo(process_info );

    // TODO: Notify the configuration change

    // TODO: Trace to the log manager
}

CdmwPlatformMngt::RequestStatus UnmanagedProcessFailedInvalid::initialise(
        UnmanagedProcessStateMachine* stateMachine,
    const CdmwPlatformMngtBase::StartupKind& startup_kind )
throw( CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException )
{
    stateMachine->setState("Initialising");

    return stateMachine->getContext()->m_lastRequestStatus;
}

} // End namespace PlatformMngt
} // End namespace Cdmw

