/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_TestComponentsAndGroups_HPP 
#define INCL_PLATFORMMNGT_TestComponentsAndGroups_HPP 

#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/common/System.hpp"
#include <Foundation/orbsupport/CORBA.hpp>
#include <platformsystemlibrary/System_impl.hpp>
#include <platformapplicationlibrary/Application_impl.hpp>

namespace Cdmw
{
namespace PlatformMngt
{

/**
*Purpose:
*<p>   
*
*Features:
*<p> Thread safe
*
*
*/
class TestComponentsAndGroups : public CppUnit::TestFixture
{

    public:
        CORBA::ORB_var m_orb;

        /**
        * Purpose:
        * <p> Constructor
        * 
        */ 
        TestComponentsAndGroups() ;



        /**
        * Purpose:
        * <p> Destructor 
        * 
        */ 
        virtual 
        ~TestComponentsAndGroups() ;

        void setUp() ;

        
        /**
        * Purpose:
        * <p> Call from the testable object 
        * 
        */ 
        virtual void get_component() ;
        virtual void get_all_components() ;
        virtual void get_number_of_components() ;
        virtual void get_component_status() ;
        virtual void get_process_from_component() ;
        virtual void get_entities_from_component() ;
        virtual void get_component_from_entity() ;
        virtual void get_number_of_component_entities() ;
        virtual void get_component_snapshot() ;
        virtual void get_component_properties() ;
        virtual void set_and_get_component_properties() ;

        virtual void get_ft_group() ;
        virtual void get_all_ft_groups() ;
        virtual void get_number_of_ft_groups() ;
        virtual void get_application_from_ftgroup() ;
        virtual void get_entities_from_ftgroup() ;
        virtual void get_ftgroup_from_entity() ;
        virtual void get_number_of_ftgroup_entities() ;
        virtual void get_ftgroup_snapshot() ;
        virtual void get_ftgroup_properties() ;
        virtual void get_ftgroup_status() ;
        virtual void set_and_get_ftgroup_properties() ;

        virtual void get_lb_group() ;
        virtual void get_lbgroup_status() ;
        virtual void get_all_lb_groups() ;
        virtual void get_number_of_lb_groups() ;
        virtual void get_application_from_lbgroup() ;
        virtual void get_entities_from_lbgroup() ;
        virtual void get_lbgroup_from_entity() ;
        virtual void get_number_of_lbgroup_entities() ;
        virtual void get_lbgroup_snapshot() ;
        virtual void get_lbgroup_properties() ;
        virtual void set_and_get_lbgroup_properties() ;


    protected:


    private:

        /**
        * Purpose:
        * <p> Assignment operator
        * 
        */ 
        TestComponentsAndGroups& operator=(const TestComponentsAndGroups& rhs) throw();
        /**
        * Purpose:
        * <p>  Copy constructor
        * 
        */ 
        TestComponentsAndGroups(const TestComponentsAndGroups& rhs) throw();

        static CdmwPlatformMngt::System_var mysystem;
        static CdmwPlatformMngt::Process_var myprocess;
        static CdmwPlatformMngt::Application_var app;
        static CdmwPlatformMngt::CcmComponent_var comp;
        static bool m_is_one_shot_initialized;
                 
        static void one_shot_init();

CPPUNIT_TEST_SUITE(TestComponentsAndGroups);
CPPUNIT_TEST( get_component );
CPPUNIT_TEST( get_all_components );
CPPUNIT_TEST( get_number_of_components );
CPPUNIT_TEST( get_component_status );
CPPUNIT_TEST( get_process_from_component );
CPPUNIT_TEST( get_entities_from_component );
CPPUNIT_TEST( get_component_from_entity );
CPPUNIT_TEST( get_number_of_component_entities );
CPPUNIT_TEST( get_component_snapshot );
CPPUNIT_TEST( get_component_properties );
CPPUNIT_TEST( set_and_get_component_properties );

CPPUNIT_TEST( get_ft_group );
CPPUNIT_TEST( get_all_ft_groups );
CPPUNIT_TEST( get_ftgroup_status );
CPPUNIT_TEST( get_number_of_ft_groups );
CPPUNIT_TEST( get_application_from_ftgroup );
CPPUNIT_TEST( get_entities_from_ftgroup );
CPPUNIT_TEST( get_ftgroup_from_entity );
CPPUNIT_TEST( get_number_of_ftgroup_entities);
CPPUNIT_TEST( get_ftgroup_snapshot );
CPPUNIT_TEST( get_ftgroup_properties );
CPPUNIT_TEST( set_and_get_ftgroup_properties );

CPPUNIT_TEST( get_lb_group );
CPPUNIT_TEST( get_all_lb_groups );
CPPUNIT_TEST( get_lbgroup_status );
CPPUNIT_TEST( get_number_of_lb_groups );
CPPUNIT_TEST( get_application_from_lbgroup );
CPPUNIT_TEST( get_entities_from_lbgroup );
CPPUNIT_TEST( get_lbgroup_from_entity );
CPPUNIT_TEST( get_number_of_lbgroup_entities);
CPPUNIT_TEST( get_lbgroup_snapshot );
CPPUNIT_TEST( get_lbgroup_properties );
CPPUNIT_TEST( set_and_get_lbgroup_properties );


CPPUNIT_TEST_SUITE_END();
        


};

} /* End namespace PlatformMngt */
} /* End namespace Cdmw */
#endif 

