/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/* @author Fabrizio Lorenna <f.lorenna@vitrociset.it> */

#include <iostream>
#include <sstream>
#include <limits>

#include "test_components_and_groups/TestComponentsAndGroups.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"

#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"
#include "idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"

#include <platformftproxy/InitUtils.hpp>
#include <platformftproxy/SystemProxy.hpp>
#include <SystemMngt/platformftproxy/AccessorLookupService.hpp>
#include <SystemMngt/platformftproxy/DefaultAccessorLookupService.hpp>
#include <SystemMngt/platformlibrary/Iterator.hpp>

#include <FaultTolerance/ftreplicationmanager/InitUtils.hpp>
#include <FaultTolerance/idllib/FT.stub.hpp>
#include <platformftaccessorlibrary/SystemAccessor_impl.hpp>
#include "tao/AnyTypeCode/TypeCode.h"


using namespace std;

using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestComponentsAndGroups);

namespace Cdmw
{
  namespace PlatformMngt
  {



    TestComponentsAndGroups::TestComponentsAndGroups ()
    {

    }


    TestComponentsAndGroups::~TestComponentsAndGroups ()
    {

    }
    bool TestComponentsAndGroups::m_is_one_shot_initialized = false;

    CdmwPlatformMngt::System_var TestComponentsAndGroups::mysystem;
    CdmwPlatformMngt::Process_var TestComponentsAndGroups::myprocess;
    CdmwPlatformMngt::Application_var TestComponentsAndGroups::app;
    CdmwPlatformMngt::CcmComponent_var TestComponentsAndGroups::comp;

    void TestComponentsAndGroups::setUp ()
    {
      TestComponentsAndGroups::one_shot_init ();
    }

    void TestComponentsAndGroups::one_shot_init ()
    {

      if (!m_is_one_shot_initialized)
        {
          m_is_one_shot_initialized = true;

          try
          {
            CORBA::ORB_var orb;
            orb =
              CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::
                                      instance ()->get_ORB ());

            PortableServer::POA_var poa =
              PortableServer::POA::_duplicate (Cdmw::TestUtils::
                                               CORBATestManager::instance ()->
                                               get_POA ());

            PortableServer::POAManager_var poaMgr =
              PortableServer::POAManager::_duplicate (poa->the_POAManager ());
            poaMgr->activate ();

        CORBA::Object_var objref;

        //define the corbaloc
        const char *corbaloc_str = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";

        objref = orb->string_to_object(corbaloc_str);

        //resolve the System_ptr
        mysystem = CdmwPlatformMngt::System::_narrow(objref.in() );

            if (CORBA::is_nil (mysystem.in ()))
              {
                exit (-1);
              }

            /*from System, use a method to retrieve an application */
            app = mysystem->get_application ("HELLO_WORLD");

            /*from the application, use a method to retrieve a Process */

            /*retrieve the current hostname */
            char *hostname = getenv ("HOSTNAME");

            myprocess = app->get_process ("P12", hostname);

          }
          catch (const Cdmw::Exception & ex)
          {
            std::cerr << ex.what () << std::endl;
            exit (-1);
          }
          catch (const CORBA::Exception & ex)
          {
            std::cerr << ex << std::endl;
            exit (-1);
          }
          catch (...)
          {
            std::cerr << "unexpected exception" << std::endl;
            exit (-1);
          }
        }
    }

    void TestComponentsAndGroups::get_component ()
    {

      //test to retrieve the component ptr ...
      try
        {
         comp = myprocess->get_component("FABCOMPONENT2");

         std::string compname ( comp->name() );

         if ( compname.compare("FABCOMPONENT2") == 0 )
             CPPUNIT_ASSERT(1);  /*test ok*/
         else
             CPPUNIT_ASSERT (0); /*test not ok */
    }
      catch(...)
        {
        //error
        CPPUNIT_ASSERT(0);
    }
      

    }


    void TestComponentsAndGroups::get_all_components ()
    {

      //test to retrieve the component ptr ...
      try
        {
     CdmwPlatformMngt::CcmComponents_var ncomponents ;
     CdmwPlatformMngt::ComponentsIterator_var rest ;

     //try to get 2 references of the 3 components ...
         myprocess->get_all_components(2,
                                      ncomponents,
                                      rest);

        CORBA::String_var compname1 = ( (*ncomponents)[0])->name() ;
        CORBA::String_var compname2 = ( (*ncomponents)[1])->name() ;

        std::string compstr1(compname1) ;
    std::string compstr2(compname2) ;


        CdmwPlatformMngt::CcmComponent_var comp3 ;

        rest->next_one(comp3) ;

        CORBA::String_var compname3 = comp3->name() ;

        std::string compstr3(compname3) ;


    if ( (compstr1.compare("FABCOMPONENT1") == 0 ) &&
             (compstr2.compare("FABCOMPONENT2") == 0 ) &&
             (compstr3.compare("FABCOMPONENT3") == 0 ) )
         {

             CPPUNIT_ASSERT (1); /*test ok */

         }
        else
       CPPUNIT_ASSERT (0); //test not ok


    }
      catch(...)
        {
        //error
        CPPUNIT_ASSERT(0);
    }
      

    }


    void TestComponentsAndGroups::get_number_of_components ()
    {

      try
        {
         CORBA::ULong num_comp =  myprocess->get_number_of_components();

         if ( num_comp == 3 )
         CPPUNIT_ASSERT(1); //test ok
         else
         CPPUNIT_ASSERT(0); //test not ok
         
    }
      catch(...)
        {
        //error
        CPPUNIT_ASSERT(0);
    }
      

    }

    void TestComponentsAndGroups::get_component_status ()
    {
    CORBA::String_var info ;

      try
      {

      CdmwPlatformMngt::ComponentStatus stat = comp->get_status(info) ;
      std::string info_str ( info ) ;

      if ( (stat == CdmwPlatformMngt::COMPONENT_INITIAL) &&
            (info_str.compare("") == 0) )
          CPPUNIT_ASSERT (1); /*test ok */
      else
          CPPUNIT_ASSERT(0);  /*test not ok*/

      }
      catch (...)
      {
    //error
        CPPUNIT_ASSERT (0);
      }

    }


    void TestComponentsAndGroups::get_process_from_component ()
    {
    CORBA::String_var info ;

      try
      {


      CdmwPlatformMngt::Process_var proc_local =
        CdmwPlatformMngt::Process::_narrow(comp->get_parent() ) ;


      CORBA::String_var procname = proc_local->name() ;

      std::string procname_str(procname) ;

      if ( procname_str.compare("P12") == 0)
         CPPUNIT_ASSERT(1); //test ok
      else
         CPPUNIT_ASSERT(0); //test not ok


      }
      catch (...)
      {
    //error
        CPPUNIT_ASSERT (0);
      }

    } 


    void TestComponentsAndGroups::get_entities_from_component ()
    {
    CORBA::String_var info ;

      try
      {

      CdmwPlatformMngt::CcmComponent_var comp3 = 
              myprocess->get_component("FABCOMPONENT3");





      CdmwPlatformMngt::Entities_var entities;
      CdmwPlatformMngt::EntitiesIterator_var ent_iter;

      comp3->get_all_entities(2, entities, ent_iter) ;

      CORBA::String_var entname1 = ( (*entities)[0])->name() ;
      CORBA::String_var entname2 = ( (*entities)[1])->name() ;

      std::string entstr1(entname1) ;
      std::string entstr2(entname2) ;


      //also try with 1 and the rest ...
      comp3->get_all_entities(1, entities, ent_iter) ;

      CORBA::String_var newentname1 = ( (*entities)[0])->name() ;

      
      CdmwPlatformMngt::Entity_var ent2;

      ent_iter->next_one(ent2) ;

      CORBA::String_var newentname2 = ent2->name() ;

      std::string newentstr1(newentname1) ;
      std::string newentstr2(newentname2) ;



      if ( (entstr1.compare("MyEntity1") == 0 ) &&
             (entstr2.compare("MyEntity2") == 0 ) &&
             (newentstr1.compare("MyEntity1") == 0 ) &&
             (newentstr2.compare("MyEntity2") == 0 ) )
           {
               CPPUNIT_ASSERT (1); //test ok
           }
      else
         CPPUNIT_ASSERT (0); //error


      }
      catch (...)
      {
    //error
        CPPUNIT_ASSERT (0);
      }

    } 


    void TestComponentsAndGroups::get_component_from_entity()
    {
    CORBA::String_var info ;

      try
      {

      CdmwPlatformMngt::CcmComponent_var comp3 = 
              myprocess->get_component("FABCOMPONENT3");


      CdmwPlatformMngt::Entities_var entities;
      CdmwPlatformMngt::EntitiesIterator_var ent_iter;

      comp3->get_all_entities(3, entities, ent_iter) ;


      //get component from entity ...


      CdmwPlatformMngt::Entity_var ent1 = (*entities)[0] ;

      CdmwPlatformMngt::CcmComponent_var comptest = 
            CcmComponent::_narrow(ent1->get_parent()) ;


      CORBA::String_var compname = comptest->name() ;

      std::string testname( compname );

      if ( testname.compare("FABCOMPONENT3") == 0)
          CPPUNIT_ASSERT (1); //test ok
      else
          CPPUNIT_ASSERT (0); //test not ok



      }
      catch (...)
      {
    //error
        CPPUNIT_ASSERT (0);
      }

    } 

    void TestComponentsAndGroups::get_number_of_component_entities()
    {
    CORBA::String_var info ;

      try
      {

      CdmwPlatformMngt::CcmComponent_var comp3 = 
              myprocess->get_component("FABCOMPONENT3");



      //also try with 1 and the rest ...
      CORBA::ULong num = comp3->get_number_of_entities();

      if ( num == 2)
         CPPUNIT_ASSERT (1); //test ok
      else
         CPPUNIT_ASSERT (0); //test not ok



      }
      catch (...)
      {
    //error
        CPPUNIT_ASSERT (0);
      }

    } 

    //get the snapshot ...
    void TestComponentsAndGroups::get_component_snapshot()
    {
    CORBA::String_var info ;

      try
      {

      CdmwPlatformMngt::CcmComponent_var comp2 = 
              myprocess->get_component("FABCOMPONENT2");


      CdmwPlatformMngt::ComponentSnapshot_var comp_snap =
        comp2->get_snapshot () ;

      std::string comp_name( comp_snap->component_name ) ;

      CdmwPlatformMngt::ComponentStatus
          comp_stat = comp_snap->component_status;

      std::string comp_stat_info(comp_snap->component_status_info);
      std::string deploy_plan(comp_snap->component_deployment_plan);

      std::string ele_orig_path_str( comp_snap->element_path ) ;


      //example elem. path.
      //MySystem.sys/HELLO_WORLD.appl/CARDAMOM14.host/P12.proc/FABCOMPONENT2.comp

      /*retrieve the current hostname */
      char *local_hostname = getenv ("HOSTNAME");

      std::stringstream stre;

      stre<<"MySystem.sys/HELLO_WORLD.appl/"<<local_hostname<<".host/P12.proc/FABCOMPONENT2.comp";

      std::string ele_path_str(stre.str()) ;

      //get the entities ...
      CdmwPlatformMngt::EntitiesSnapshot ents_snap;
      ents_snap = comp_snap->component_entities ;

      CORBA::ULong num_ents = ents_snap.length() ;

      CdmwPlatformMngt::EntitySnapshot ent1 = ents_snap[0] ;
      CdmwPlatformMngt::EntitySnapshot ent2 = ents_snap[1] ;

      std::string name_ent1( ent1.entity_name );
      std::string name_ent2( ent2.entity_name );


      std::string ent_status1( ent1.entity_status);
      std::string ent_status2( ent2.entity_status);
     
      std::string ent_status_info1( ent1.entity_info);
      std::string ent_status_info2( ent2.entity_info);

      //get the properties ...
      CdmwPlatformMngt::PropertiesSnapshot props_snap =
          comp_snap->component_properties ;

      CORBA::ULong num_props = props_snap.length() ;

      CdmwPlatformMngt::PropertySnapshot prop1 = props_snap[0] ;
      CdmwPlatformMngt::PropertySnapshot prop2 = props_snap[1] ;
      CdmwPlatformMngt::PropertySnapshot prop3 = props_snap[2] ;
      CdmwPlatformMngt::PropertySnapshot prop4 = props_snap[3] ;
      CdmwPlatformMngt::PropertySnapshot prop5 = props_snap[4] ;
      CdmwPlatformMngt::PropertySnapshot prop6 = props_snap[5] ;
      
      CORBA::Any prop_val1 = prop1.property_value;
      CORBA::Any prop_val2 = prop2.property_value;
      CORBA::Any prop_val3 = prop3.property_value;
      CORBA::Any prop_val4 = prop4.property_value;
      CORBA::Any prop_val5 = prop5.property_value;
      CORBA::Any prop_val6 = prop6.property_value;

      //prop. val. 1  
      const char* str1;
      prop_val1 >>= str1;
      std::string property_val1( str1 ) ;

      //prop. val. 2
      short property_val2;
      prop_val2 >>= property_val2;

      //prop. val. 3
      CdmwPlatformMngt::RedundancyEltState property_val3;
      prop_val3 >>= property_val3;

      //prop. val. 4
      CdmwPlatformMngt::RestartWarmth property_val4;
      prop_val4 >>= property_val4;

      //prop. val. 5
      CORBA::Float property_val5;
      prop_val5 >>= property_val5;

      //prop. val. 6
      CdmwPlatformMngt::RestartWarmth property_val6;
      prop_val6 >>= property_val6;

      CORBA::TypeCode_var tc1 = prop_val1.type();
      CORBA::TypeCode_var tc2 = prop_val2.type();
      CORBA::TypeCode_var tc3 = prop_val3.type();

      CORBA::TypeCode_var tc4 = prop_val4.type();
      CORBA::TypeCode_var tc5 = prop_val5.type();
      CORBA::TypeCode_var tc6 = prop_val6.type();

      std::string name_prop1( prop1.property_name );
      std::string name_prop2( prop2.property_name );
      std::string name_prop3( prop3.property_name );
      std::string name_prop4( prop4.property_name );
      std::string name_prop5( prop5.property_name );
      std::string name_prop6( prop6.property_name );
      

      if ( 
      (num_ents == 2) &&
       (name_ent1.compare("FabEntity1") == 0) && 
       (name_ent2.compare("FabEntity2") == 0) && 
       (ent_status1.compare("UNKNOWN") == 0) && 
       (ent_status2.compare("UNKNOWN") == 0) && 
       (ent_status_info1.compare("Default entity status") == 0) && 
       (ent_status_info2.compare("Default entity status") == 0) && 

      (num_props == 6) &&
      (property_val1.compare("MYSTRING") == 0 ) &&
      (property_val2 == 5 ) &&
      (property_val3 == CdmwPlatformMngt::REDSTATE_NORG) &&
      (property_val4 == CdmwPlatformMngt::HOT_RESTART) &&
      (property_val5 == ( (CORBA::Float) 2.4)) &&
      (property_val6 == CdmwPlatformMngt::HOT_RESTART) &&

      (tc1->kind() == CORBA::tk_string) &&
      (tc2->kind() == CORBA::tk_short) &&
      (tc3->equal(CdmwPlatformMngt::_tc_RedundancyEltState)) &&
      (tc4->equal(CdmwPlatformMngt::_tc_RestartWarmth)) &&
      (tc5->kind() == CORBA::tk_float) &&
      (tc6->equal(CdmwPlatformMngt::_tc_RestartWarmth)) &&

      (name_prop1.compare("COMP_PROP_STR") == 0) &&
      (name_prop2.compare("COMP_PROP_SHORT") == 0) &&
      (name_prop3.compare("Cdmw.ReplicationRole") == CdmwPlatformMngt::REDSTATE_NORG) &&
      (name_prop4.compare("Cdmw.DefaultRestartWarmth") == 0) &&
      (name_prop5.compare("COMP_PROP_FLOAT") == 0) &&
      (name_prop6.compare("Cdmw.RequestedRestartWarmth") == 0) &&

      (comp_name.compare("FABCOMPONENT2") == 0) &&
      (comp_stat == CdmwPlatformMngt::COMPONENT_INITIAL) &&
          ( comp_stat_info.compare("") == 0) &&
      ( deploy_plan.compare("TRUE") == 0) &&
      ( ele_orig_path_str.compare(ele_path_str) == 0) )

        {
        //ok
            CPPUNIT_ASSERT (1);
        }
       else
        CPPUNIT_ASSERT (0); //error
         

      }
      catch (...)
      {
    //error
        CPPUNIT_ASSERT (0);
      }


    }


    void TestComponentsAndGroups::get_component_properties()
    {
    CORBA::String_var info ;

      try
      {

      CdmwPlatformMngt::CcmComponent_var comp2 = 
              myprocess->get_component("FABCOMPONENT2");


      //get user properties ...
      CORBA::Any_var prop_value1;
      comp2->get_attribute("COMP_PROP_SHORT", prop_value1,
         CdmwPlatformMngt::USER_DEFINED );

      CORBA::Any_var prop_value2;
      comp2->get_attribute("COMP_PROP_FLOAT", prop_value2,
         CdmwPlatformMngt::USER_DEFINED );

      CORBA::Any_var prop_value3;
      comp2->get_attribute("COMP_PROP_STR", prop_value3,
         CdmwPlatformMngt::USER_DEFINED );

      //get pre-defined properties ...
      CORBA::Any_var prop_value4;
      comp2->get_attribute("ReplicationRole", prop_value4,
         CdmwPlatformMngt::PRE_DEFINED );

      CORBA::Any_var prop_value5;
      comp2->get_attribute("DefaultRestartWarmth", prop_value5,
         CdmwPlatformMngt::PRE_DEFINED );

      CORBA::Any_var prop_value6;
      comp2->get_attribute("RequestedRestartWarmth", prop_value6,
         CdmwPlatformMngt::PRE_DEFINED );

      //see number of properties ...

      CORBA::ULong num_props = comp2->get_number_of_properties();

      CORBA::TypeCode_var tc1 = prop_value1->type();
      CORBA::TypeCode_var tc2 = prop_value2->type();
      CORBA::TypeCode_var tc3 = prop_value3->type();

      CORBA::TypeCode_var tc4 = prop_value4->type();
      CORBA::TypeCode_var tc5 = prop_value5->type();
      CORBA::TypeCode_var tc6 = prop_value6->type();


      CORBA::Short value1;
      (*prop_value1) >>= value1 ;

      CORBA::Float value2;
      (*prop_value2) >>= value2 ;


      const char* value3_tmp;
      prop_value3 >>= value3_tmp;


      std::string value3( value3_tmp ) ;

      CdmwPlatformMngt::RedundancyEltState value4;
      (*prop_value4) >>= value4;

      CdmwPlatformMngt::RestartWarmth value5;
      (*prop_value5) >>= value5;

      CdmwPlatformMngt::RestartWarmth value6;
      (*prop_value6) >>= value6;


      if ( (num_props == 8) && 
       (tc1->kind() == CORBA::tk_short) &&
       (tc2->kind() == CORBA::tk_float) &&
       (tc3->kind() == CORBA::tk_string) &&
       (tc4->equal(CdmwPlatformMngt::_tc_RedundancyEltState)) &&
       (tc5->equal(CdmwPlatformMngt::_tc_RestartWarmth)) &&
       (tc6->equal(CdmwPlatformMngt::_tc_RestartWarmth)) &&
       (value1 == 5) &&
       (value2 == ( (CORBA::Float) 2.4) ) &&
       (value3.compare("MYSTRING") == 0)  &&
       (value4 == CdmwPlatformMngt::REDSTATE_NORG) &&
       (value5 == CdmwPlatformMngt::HOT_RESTART) &&
       (value6 == CdmwPlatformMngt::HOT_RESTART) )
        {
    //ok
        CPPUNIT_ASSERT (1);
    }
      else
        CPPUNIT_ASSERT (0); //test not ok 

      }
      catch (...)
      {
    //error
        CPPUNIT_ASSERT (0);
      }

    } 



    void TestComponentsAndGroups::set_and_get_component_properties()
    {
    CORBA::String_var info ;

      try
      {

      CdmwPlatformMngt::CcmComponent_var comp2 = 
              myprocess->get_component("FABCOMPONENT2");

      CORBA::Any_var prop_value2 = new CORBA::Any() ;
      CORBA::Any_var prop_value3 = new CORBA::Any() ;
      CORBA::Any_var prop_value4 = new CORBA::Any() ;


      CdmwPlatformMngt::RestartWarmth value2;
      value2 = CdmwPlatformMngt::COLD_RESTART ;
      (*prop_value2) <<= value2;
      comp2->set_attribute("DefaultRestartWarmth", (*prop_value2),
         CdmwPlatformMngt::PRE_DEFINED ); 


      CORBA::Short value4 = 32 ;
      (*prop_value4) <<= value4;
      comp2->set_attribute("COMP_PROP_SHORT", (*prop_value4),
         CdmwPlatformMngt::USER_DEFINED ); 


      //get the values ...


      CORBA::Any_var prop_value6;
      comp2->get_attribute("DefaultRestartWarmth", prop_value6,
         CdmwPlatformMngt::PRE_DEFINED );
      CdmwPlatformMngt::RestartWarmth value6;
      (*prop_value6) >>= value6;


      CORBA::Any_var prop_value8;
      comp2->get_attribute("COMP_PROP_SHORT", prop_value8,
         CdmwPlatformMngt::USER_DEFINED );
      CORBA::Short value8;
      (*prop_value8) >>= value8 ;

      CORBA::TypeCode_var tc6 = prop_value6->type();
      CORBA::TypeCode_var tc8 = prop_value8->type();

      //see number of properties ...

      CORBA::ULong num_props = comp2->get_number_of_properties();

      if ( (num_props == 8) && 
       (tc6->equal(CdmwPlatformMngt::_tc_RestartWarmth) ) &&
       (tc8->kind() == CORBA::tk_short) &&
       (value6 == CdmwPlatformMngt::COLD_RESTART ) &&
       (value8 == 32 )  )
        {
    //ok
        CPPUNIT_ASSERT (1);
    }
      else
        CPPUNIT_ASSERT (0); //test not ok 

      }
      catch (...)
      {
    //error
        CPPUNIT_ASSERT (0);
      }

    } 

    void TestComponentsAndGroups::get_ft_group()
    {

      CdmwPlatformMngt::FTGroup_var ftg ;

      //test to retrieve the ftgroup ptr ...
      try
        {
         ftg = app->get_ft_group("FABFTGROUP1");

         char *ftg_name = ftg->name() ;

     std::string ftgroup_name( ftg_name)  ;

     if ( ftgroup_name.compare("FABFTGROUP1") == 0)
             CPPUNIT_ASSERT(1);  /*test ok*/
         else
             CPPUNIT_ASSERT(0); /*test not ok */
    }
      catch(...)
        {
        //error
        CPPUNIT_ASSERT(0);
    }
      

    }


    void TestComponentsAndGroups::get_all_ft_groups()
    {

    CdmwPlatformMngt::FTGroups_var ftgroups;

    CdmwPlatformMngt::FTGroupsIterator_var rest ;


    CdmwPlatformMngt::FTGroup_var ftg3 ;

    try
        {
         app->get_all_ft_groups(2, ftgroups, rest );

     CORBA::ULong len_subset1 = ftgroups->length() ;
         
         std::string ftg_name1( ( (*ftgroups)[0])->name() );
         std::string ftg_name2( ( (*ftgroups)[1])->name() );

         rest->next_one(ftg3) ;

         std::string ftg_name3(  ftg3->name() );

         if ( ( len_subset1 == 2) &&
         (ftg_name1.compare("FABFTGROUP1") == 0) &&
         (ftg_name2.compare("FABFTGROUP2") == 0) &&
         (ftg_name3.compare("FABFTGROUP3") == 0) )
           {
           CPPUNIT_ASSERT(1); //ok
           }
             else
           CPPUNIT_ASSERT(0); //not ok


    }
    catch(...)
        {
        //error
        CPPUNIT_ASSERT(0);
    }
      

    }

    void TestComponentsAndGroups::get_number_of_ft_groups()
    {
    
    try
    {
    CORBA::ULong num_ftgroups = app->get_number_of_ft_groups() ;


    if ( num_ftgroups == 3)
      CPPUNIT_ASSERT(1); //ok
    else
      CPPUNIT_ASSERT(0); //not ok

    }
    catch(...)
      {
        CPPUNIT_ASSERT(0);
      }

    }

    void TestComponentsAndGroups::get_application_from_ftgroup()
    {


      CdmwPlatformMngt::FTGroup_var ftg ;

      //test to retrieve the appl. form ftgroup ptr ...
      try
        {
         ftg = app->get_ft_group("FABFTGROUP1");


         CdmwPlatformMngt::Application_var app_local = 
            Application::_narrow(ftg->get_parent()) ;


     char *app_name =  app_local->name()  ;

     std::string appname_str ( app_name ) ;

         if ( (appname_str.compare("HELLO_WORLD") == 0))
            CPPUNIT_ASSERT(1); //test ok
         else
            CPPUNIT_ASSERT(0); //test not ok


    }
      catch(...)
        {
        //error
        CPPUNIT_ASSERT(0);
    }

    } 

    void TestComponentsAndGroups::get_entities_from_ftgroup()
    {
      CdmwPlatformMngt::FTGroup_var ftg ;

      try
        {
         ftg = app->get_ft_group("FABFTGROUP2");

     CdmwPlatformMngt::Entities_var ents_var ;
         CdmwPlatformMngt::EntitiesIterator_var rest ;

         //get 1 entity and 1 in the rest ...
     ftg->get_all_entities(1, ents_var, rest) ;

         
         std::string entname1( ( (*ents_var)[0])->name() );

         CdmwPlatformMngt::Entity_var ent2;
         rest->next_one(ent2) ;

         std::string entname2( ent2->name() );

         if ( entname1.compare("FABFTENTITY1") != 0)
       {
           CPPUNIT_ASSERT(0);
       return;
       }

         if ( entname2.compare("FABFTENTITY2") != 0)
       {
           CPPUNIT_ASSERT(0);
       return;
       }

     CdmwPlatformMngt::Entities_var newents_var ;
         CdmwPlatformMngt::EntitiesIterator_var newrest ;

         //get all the 2 entities ...
     ftg->get_all_entities(2, newents_var, newrest) ;

         std::string newentname1( ( (*newents_var)[0])->name() );
         std::string newentname2( ( (*newents_var)[1])->name() );

         CORBA::ULong num_ents = newents_var->length() ;


         if ( ( newentname1.compare("FABFTENTITY1") == 0) &&
     ( newentname2.compare("FABFTENTITY2") == 0) &&
     ( num_ents == (( CORBA::ULong) 2) ) )
        {
            CPPUNIT_ASSERT(1); //test ok
        }
     else
        {
            CPPUNIT_ASSERT(0); // test not ok
        }

        }
      catch(...)
        {
        //error
        CPPUNIT_ASSERT(0);
    }

    }

    void TestComponentsAndGroups::get_ftgroup_from_entity()
    {
      CdmwPlatformMngt::FTGroup_var ftg ;

      try
        {
     CdmwPlatformMngt::Entities_var ents_var ;
         CdmwPlatformMngt::EntitiesIterator_var rest ;


         ftg = app->get_ft_group("FABFTGROUP2");

         //get 1 entity and 1 in the rest ...
     ftg->get_all_entities(1, ents_var, rest) ;

         CdmwPlatformMngt::Entity_var ent1 = (*ents_var)[0] ;

         CdmwPlatformMngt::FTGroup_var ftgtest = 
            FTGroup::_narrow(ent1->get_parent()) ;
         


         std::string ftgname2( ftgtest->name() );


         if ( ftgname2.compare("FABFTGROUP2") == 0)
           CPPUNIT_ASSERT(1); //test ok
         else
           CPPUNIT_ASSERT(0);





        }
      catch(...)
        {
        //error
        CPPUNIT_ASSERT(0);
    }

    }

    void TestComponentsAndGroups::get_number_of_ftgroup_entities()
    {
      CdmwPlatformMngt::FTGroup_var ftg ;

      try
        {
         ftg = app->get_ft_group("FABFTGROUP2");

         CORBA::ULong num_entities = ftg->get_number_of_entities() ;

     if ( num_entities == 2)
            CPPUNIT_ASSERT(1); //test ok
         else
            CPPUNIT_ASSERT(0); // test not ok


        }
      catch(...)
        {
        //error
        CPPUNIT_ASSERT(0);
    }

    }


    void TestComponentsAndGroups::get_ftgroup_properties()
    {
      CdmwPlatformMngt::FTGroup_var ftg ;

      try
        {
         ftg = app->get_ft_group("FABFTGROUP1");

         //get the properties ...
     CORBA::Any_var prop_value1;
         ftg->get_attribute("FTG_PROP_SHORT", prop_value1,
          CdmwPlatformMngt::USER_DEFINED );

         CORBA::Any_var prop_value2;
         ftg->get_attribute("FTG_PROP_FLOAT", prop_value2,
          CdmwPlatformMngt::USER_DEFINED );

         CORBA::Any_var prop_value3;
         ftg->get_attribute("FTG_PROP_STR", prop_value3,
          CdmwPlatformMngt::USER_DEFINED );
 
         CORBA::Any_var prop_value4;
         ftg->get_attribute("GroupSize", prop_value4,
          CdmwPlatformMngt::PRE_DEFINED );

         //get number of properties ...
         CORBA::ULong num_props = ftg->get_number_of_properties();

         CORBA::TypeCode_var tc1 = prop_value1->type();
         CORBA::TypeCode_var tc2 = prop_value2->type();
         CORBA::TypeCode_var tc3 = prop_value3->type();
         CORBA::TypeCode_var tc4 = prop_value4->type();

         CORBA::Short value1; 
         (*prop_value1) >>= value1 ;

         CORBA::Float value2; 
         (*prop_value2) >>= value2 ;

         const char* value3_tmp;
         prop_value3 >>= value3_tmp;
         std::string value3( value3_tmp ) ;


         CORBA::Long value4; 
         (*prop_value4) >>= value4 ;


     if ( (num_props = 4) &&
              ( value1 == 10) &&
              (value2 == ((CORBA::Float) 11.1) ) &&
              ( value3.compare("MYSTRING_FT") == 0) &&
              (value4 == 0) &&
              (tc1->kind() == CORBA::tk_short) &&
              (tc2->kind() == CORBA::tk_float) &&
              (tc3->kind() == CORBA::tk_string) &&
              (tc4->kind() == CORBA::tk_long ))
                  CPPUNIT_ASSERT(1); //ok
         else
                  CPPUNIT_ASSERT(0); //not ok


        }
      catch(...)
        {
        //error
        CPPUNIT_ASSERT(0);
    }


    }


    void TestComponentsAndGroups::get_ftgroup_status ()
    {
    CORBA::String_var info ;
    CdmwPlatformMngt::FTGroup_var ftg ;

    try
      {
      ftg = app->get_ft_group("FABFTGROUP1");

      CdmwPlatformMngt::TechnicalStatus stat = ftg->get_status(info) ;
      std::string info_str ( info ) ;

      if ( (stat == CdmwPlatformMngt::INITIAL) &&
            (info_str.compare("") == 0) )
          CPPUNIT_ASSERT (1); /*test ok */
      else
          CPPUNIT_ASSERT(0);  /*test not ok*/

      }
      catch (...)
      {
    //error
        CPPUNIT_ASSERT (0);
      }

    }

    void TestComponentsAndGroups::set_and_get_ftgroup_properties()
    {
    CORBA::String_var info ;

    CdmwPlatformMngt::FTGroup_var ftg ;
    CORBA::Any_var prop_value1 = new CORBA::Any() ;
    CORBA::Any_var prop_value2 = new CORBA::Any() ;
    CORBA::Any_var prop_value1_get;
    CORBA::Short newvalue1; 

    try
     {
      ftg = app->get_ft_group("FABFTGROUP1");



      CORBA::Short value1 = 23 ;
      (*prop_value1) <<= value1 ;
      ftg->set_attribute("FTG_PROP_SHORT", (*prop_value1),
         CdmwPlatformMngt::USER_DEFINED );


      ftg->get_attribute("FTG_PROP_SHORT", prop_value1_get,
         CdmwPlatformMngt::USER_DEFINED );

      (*prop_value1_get) >>= newvalue1 ;
      }
      catch (...)
      {
    //error
        CPPUNIT_ASSERT (0);
      }


      try
      {
      CORBA::TypeCode_var tc1_get = prop_value1_get->type();

      //see number of properties ...

      CORBA::ULong num_props = ftg->get_number_of_properties();

      if ( (num_props == 6) && 
       (tc1_get->kind() == CORBA::tk_short) &&
           ( newvalue1 == 23 ) ) 
        {
    //test ok
        CPPUNIT_ASSERT (1);
    }
      else
        CPPUNIT_ASSERT (0); //test not ok 

      }
      catch (...)
      {
    //error
        CPPUNIT_ASSERT (0);
      }

    } 



    void TestComponentsAndGroups::get_ftgroup_snapshot()
    {
    CORBA::String_var info ;

      try
      {

      CdmwPlatformMngt::FTGroup_var ftg2 = 
              app->get_ft_group("FABFTGROUP2");


      CdmwPlatformMngt::FTGroupSnapshot_var ftg_snap =
        ftg2->get_snapshot () ;

      std::string ftg_name( ftg_snap->ft_group_name) ;

      CdmwPlatformMngt::TechnicalStatus
          ftg_stat = ftg_snap->ft_group_status;

      std::string ftg_stat_info(ftg_snap->status_info);
      

      std::string ele_orig_path_str( ftg_snap->element_path ) ;

      std::string comp1 = string (ftg_snap->component_element_paths[0]);
      std::string comp2 = string (ftg_snap->component_element_paths[1]);
       
      //std::cerr <<  comp1 << std::endl;
      //std::cerr <<  comp2 << std::endl;

      std::string sub_comp1 = string (comp1, comp1.length()-28, 28 );
      std::string sub_comp2 = string (comp2, comp2.length()-28, 28 );
      
      //std::cerr << sub_comp1 << "   " << sub_comp2 << std::endl;
      
      CPPUNIT_ASSERT ("/P12.proc/FABCOMPONENT1.comp" == sub_comp1);
      CPPUNIT_ASSERT ("/P12.proc/FABCOMPONENT2.comp" == sub_comp2);

      std::stringstream stre;

      stre<<"MySystem.sys/HELLO_WORLD.appl/FABFTGROUP2.ftgroup" ;

      std::string ele_path_str(stre.str()) ;

      //get the entities ...
      CdmwPlatformMngt::EntitiesSnapshot ents_snap;

      ents_snap = ftg_snap->ft_group_entities ;

      CORBA::ULong num_ents = ents_snap.length() ;

      CdmwPlatformMngt::EntitySnapshot ent1 = ents_snap[0] ;
      CdmwPlatformMngt::EntitySnapshot ent2 = ents_snap[1] ;

      std::string name_ent1( ent1.entity_name );
      std::string name_ent2( ent2.entity_name );

      std::string ent_status1( ent1.entity_status);
      std::string ent_status2( ent2.entity_status);
     
      std::string ent_status_info1( ent1.entity_info);
      std::string ent_status_info2( ent2.entity_info);

      //get the properties ...
      CdmwPlatformMngt::PropertiesSnapshot props_snap =
          ftg_snap->ft_group_properties;

      CORBA::ULong num_props = props_snap.length() ;

      CdmwPlatformMngt::PropertySnapshot prop1 = props_snap[0] ;
      CdmwPlatformMngt::PropertySnapshot prop2 = props_snap[1] ;
      CdmwPlatformMngt::PropertySnapshot prop3 = props_snap[2] ;
      CdmwPlatformMngt::PropertySnapshot prop4 = props_snap[3] ;
      
      CORBA::Any prop_val1 = prop1.property_value;
      CORBA::Any prop_val2 = prop2.property_value;
      CORBA::Any prop_val3 = prop3.property_value;
      CORBA::Any prop_val4 = prop4.property_value;

      std::string name_prop1( prop1.property_name );
      std::string name_prop2( prop2.property_name );
      std::string name_prop3( prop3.property_name );
      std::string name_prop4( prop4.property_name );


      //prop. val. 1  
      CORBA::Long property_val1;
      prop_val1 >>= property_val1;

      //prop. val. 2
      CORBA::Float property_val2;
      prop_val2 >>= property_val2;



      //prop. val. 3
      const char *str3 ;
      prop_val3 >>= str3;
      std::string property_val3( str3 ) ;

      //prop. val. 4
      short property_val4;
      prop_val4 >>= property_val4;

      CORBA::TypeCode_var tc1 = prop_val1.type();
      CORBA::TypeCode_var tc2 = prop_val2.type();
      CORBA::TypeCode_var tc3 = prop_val3.type();
      CORBA::TypeCode_var tc4 = prop_val4.type();


      if ( 
      (num_ents == 2) &&
       (name_ent1.compare("FABFTENTITY1") == 0) && 
       (name_ent2.compare("FABFTENTITY2") == 0) && 
       (ent_status1.compare("UNKNOWN") == 0) && 
       (ent_status2.compare("UNKNOWN") == 0) && 
       (ent_status_info1.compare("Default entity status") == 0) && 
       (ent_status_info2.compare("Default entity status") == 0) && 

      (num_props == 4) &&
      (property_val1 == 0 ) &&
      (property_val2 == ((CORBA::Float) 13.3) ) &&
      (property_val3.compare("NEWMYSTRING_FT") == 0) &&
      (property_val4 == 12 ) &&

      (tc1->kind() == CORBA::tk_long) &&
      (tc2->kind() == CORBA::tk_float) &&
      (tc3->kind() == CORBA::tk_string) &&
      (tc4->kind() == CORBA::tk_short) &&

      (name_prop1.compare("Cdmw.GroupSize") == 0) &&
      (name_prop2.compare("MYFTG_PROP_FLOAT") == 0) &&
      (name_prop3.compare("MYFTG_PROP_STR") == 0) &&
      (name_prop4.compare("MYFTG_PROP_SHORT") == 0) &&

      (ftg_name.compare("FABFTGROUP2") == 0) &&
      (ftg_stat == CdmwPlatformMngt::INITIAL) &&
          ( ftg_stat_info.compare("") == 0) &&
      ( ele_orig_path_str.compare(ele_path_str) == 0) )

        {
        //ok
            CPPUNIT_ASSERT (1);
        }
       else
        CPPUNIT_ASSERT (0); //error
         

      }
      catch (...)
      {
    //error
        CPPUNIT_ASSERT (0);
      }


    }

    void TestComponentsAndGroups::get_lb_group()
    {

      CdmwPlatformMngt::LBGroup_var lbg ;

      //test to retrieve the lbgroup ptr ...
      try
        {
         lbg = app->get_lb_group("FABLBGROUP1");

         char *lbg_name = lbg->name() ;

     std::string lbgroup_name( lbg_name)  ;

     if ( lbgroup_name.compare("FABLBGROUP1") == 0)
             CPPUNIT_ASSERT(1);  /*test ok*/
         else
             CPPUNIT_ASSERT(0); /*test not ok */
    }
      catch(...)
        {
        //error
        CPPUNIT_ASSERT(0);
    }
      

    }


    void TestComponentsAndGroups::get_all_lb_groups()
    {

    CdmwPlatformMngt::LBGroups_var lbgroups;

    CdmwPlatformMngt::LBGroupsIterator_var rest ;


    CdmwPlatformMngt::LBGroup_var lbg3 ;

    try
        {
         app->get_all_lb_groups(2, lbgroups, rest );

     CORBA::ULong len_subset1 = lbgroups->length() ;
         
         std::string lbg_name1( ( (*lbgroups)[0])->name() );
         std::string lbg_name2( ( (*lbgroups)[1])->name() );

         rest->next_one(lbg3) ;

         std::string lbg_name3(  lbg3->name() );

         if ( ( len_subset1 == 2) &&
         (lbg_name1.compare("FABLBGROUP1") == 0) &&
         (lbg_name2.compare("FABLBGROUP2") == 0) &&
         (lbg_name3.compare("FABLBGROUP3") == 0) )
           {
           CPPUNIT_ASSERT(1); //ok
           }
             else
           CPPUNIT_ASSERT(0); //not ok


    }
    catch(...)
        {
        //error
        CPPUNIT_ASSERT(0);
    }
      

    }


    void TestComponentsAndGroups::get_number_of_lb_groups()
    {
    
    try
    {
    CORBA::ULong num_lbgroups = app->get_number_of_lb_groups() ;


    if ( num_lbgroups == 3)
      CPPUNIT_ASSERT(1); //ok
    else
      CPPUNIT_ASSERT(0); //not ok

    }
    catch(...)
      {
        CPPUNIT_ASSERT(0);
      }

    }


    void TestComponentsAndGroups::get_application_from_lbgroup()
    {


      CdmwPlatformMngt::LBGroup_var lbg ;

      //test to retrieve the appl. form lbgroup ptr ...
      try
        {
         lbg = app->get_lb_group("FABLBGROUP1");


         CdmwPlatformMngt::Application_var app_local = 
            Application::_narrow(lbg->get_parent()) ;


     char *app_name =  app_local->name()  ;

     std::string appname_str ( app_name ) ;

         if ( (appname_str.compare("HELLO_WORLD") == 0))
            CPPUNIT_ASSERT(1); //test ok
         else
            CPPUNIT_ASSERT(0); //test not ok


    }
      catch(...)
        {
        //error
        CPPUNIT_ASSERT(0);
    }

    } 


    void TestComponentsAndGroups::get_entities_from_lbgroup()
    {
      CdmwPlatformMngt::LBGroup_var lbg ;

      try
        {
         lbg = app->get_lb_group("FABLBGROUP1");

     CdmwPlatformMngt::Entities_var ents_var ;
         CdmwPlatformMngt::EntitiesIterator_var rest ;

         //get 1 entity and 1 in the rest ...
     lbg->get_all_entities(1, ents_var, rest) ;

         
         std::string entname1( ( (*ents_var)[0])->name() );

         CdmwPlatformMngt::Entity_var ent2;
         rest->next_one(ent2) ;

         std::string entname2( ent2->name() );

         if ( entname1.compare("FABLBENTITY1") != 0)
       {
           CPPUNIT_ASSERT(0);
       return;
       }

         if ( entname2.compare("FABLBENTITY2") != 0)
       {
           CPPUNIT_ASSERT(0);
       return;
       }

     CdmwPlatformMngt::Entities_var newents_var ;
         CdmwPlatformMngt::EntitiesIterator_var newrest ;

         //get all the 2 entities ...
     lbg->get_all_entities(2, newents_var, newrest) ;

         std::string newentname1( ( (*newents_var)[0])->name() );
         std::string newentname2( ( (*newents_var)[1])->name() );

         CORBA::ULong num_ents = newents_var->length() ;


         if ( ( newentname1.compare("FABLBENTITY1") == 0) &&
     ( newentname2.compare("FABLBENTITY2") == 0) &&
     ( num_ents == (( CORBA::ULong) 2) ) )
        {
            CPPUNIT_ASSERT(1); //test ok
        }
     else
        {
            CPPUNIT_ASSERT(0); // test not ok
        }

        }
      catch(...)
        {
        //error
        CPPUNIT_ASSERT(0);
    }

    }

    void TestComponentsAndGroups::get_lbgroup_from_entity()
    {
      CdmwPlatformMngt::LBGroup_var lbg ;

      try
        {
     CdmwPlatformMngt::Entities_var ents_var ;
         CdmwPlatformMngt::EntitiesIterator_var rest ;


         lbg = app->get_lb_group("FABLBGROUP1");

     lbg->get_all_entities(3, ents_var, rest) ;

         CdmwPlatformMngt::Entity_var ent1 = (*ents_var)[0] ;


         CdmwPlatformMngt::LBGroup_var lbgtest = 
            LBGroup::_narrow(ent1->get_parent()) ;

         

         std::string lbgname1( lbgtest->name() );


         if ( lbgname1.compare("FABLBGROUP1") == 0)
           CPPUNIT_ASSERT(1); //test ok
         else
           CPPUNIT_ASSERT(0);





        }
      catch(...)
        {
        //error
        CPPUNIT_ASSERT(0);
    }

    }


    void TestComponentsAndGroups::get_lbgroup_status ()
    {
    CORBA::String_var info ;
    CdmwPlatformMngt::LBGroup_var lbg ;

    try
      {
      lbg = app->get_lb_group("FABLBGROUP1");

      CdmwPlatformMngt::TechnicalStatus stat = lbg->get_status(info) ;
      std::string info_str ( info ) ;

      if ( (stat == CdmwPlatformMngt::INITIAL) &&
            (info_str.compare("") == 0) )
          CPPUNIT_ASSERT (1); /*test ok */
      else
          CPPUNIT_ASSERT(0);  /*test not ok*/

      }
      catch (...)
      {
    //error
        CPPUNIT_ASSERT (0);
      }

    }


    void TestComponentsAndGroups::get_number_of_lbgroup_entities()
    {
      CdmwPlatformMngt::LBGroup_var lbg ;

      try
        {
         lbg = app->get_lb_group("FABLBGROUP1");

         CORBA::ULong num_entities = lbg->get_number_of_entities() ;

     if ( num_entities == 2)
            CPPUNIT_ASSERT(1); //test ok
         else
            CPPUNIT_ASSERT(0); // test not ok


        }
      catch(...)
        {
        //error
        CPPUNIT_ASSERT(0);
    }

    }


    void TestComponentsAndGroups::get_lbgroup_snapshot()
    {
    CORBA::String_var info ;

      try
      {

      CdmwPlatformMngt::LBGroup_var lbg2 = 
              app->get_lb_group("FABLBGROUP3");


      CdmwPlatformMngt::LBGroupSnapshot_var lbg_snap =
        lbg2->get_snapshot () ;

      std::string lbg_name( lbg_snap->lb_group_name) ;

      CdmwPlatformMngt::TechnicalStatus
          lbg_stat = lbg_snap->lb_group_status;

      std::string lbg_stat_info(lbg_snap->status_info);
      

      std::string ele_orig_path_str( lbg_snap->element_path ) ;

      std::string comp1 = string (lbg_snap->component_element_paths[0]);
       
      //std::cerr <<  comp1 << std::endl;

      std::string sub_comp1 = string (comp1, comp1.length()-28, 28 );
      
      //std::cerr << sub_comp1 << std::endl;
      
      CPPUNIT_ASSERT ("/P12.proc/FABCOMPONENT3.comp" == sub_comp1);


      std::stringstream stre;

      stre<<"MySystem.sys/HELLO_WORLD.appl/FABLBGROUP3.lbgroup" ;

      std::string ele_path_str(stre.str()) ;

      //get the entities ...
      CdmwPlatformMngt::EntitiesSnapshot ents_snap;

      ents_snap = lbg_snap->lb_group_entities ;

      CORBA::ULong num_ents = ents_snap.length() ;

      CdmwPlatformMngt::EntitySnapshot ent1 = ents_snap[0] ;
      CdmwPlatformMngt::EntitySnapshot ent2 = ents_snap[1] ;

      std::string name_ent1( ent1.entity_name );
      std::string name_ent2( ent2.entity_name );

      std::string ent_status1( ent1.entity_status);
      std::string ent_status2( ent2.entity_status);
     
      std::string ent_status_info1( ent1.entity_info);
      std::string ent_status_info2( ent2.entity_info);

      //get the properties ...
      CdmwPlatformMngt::PropertiesSnapshot props_snap =
          lbg_snap->lb_group_properties;

      CORBA::ULong num_props = props_snap.length() ;

      CdmwPlatformMngt::PropertySnapshot prop1 = props_snap[0] ;
      CdmwPlatformMngt::PropertySnapshot prop2 = props_snap[1] ;
      CdmwPlatformMngt::PropertySnapshot prop3 = props_snap[2] ;
      CdmwPlatformMngt::PropertySnapshot prop4 = props_snap[3] ;
      
      CORBA::Any prop_val1 = prop1.property_value;
      CORBA::Any prop_val2 = prop2.property_value;
      CORBA::Any prop_val3 = prop3.property_value;
      CORBA::Any prop_val4 = prop4.property_value;

      std::string name_prop1( prop1.property_name );
      std::string name_prop2( prop2.property_name );
      std::string name_prop3( prop3.property_name );
      std::string name_prop4( prop4.property_name );



      //prop. val. 1  
      CORBA::Long property_val1;
      prop_val1 >>= property_val1;

      //prop. val. 2
      CORBA::Float property_val2;
      prop_val2 >>= property_val2;

      //prop. val. 3
      short property_val3;
      prop_val3 >>= property_val3;


      //prop. val. 4
      const char *str4 ;
      prop_val4 >>= str4;
      std::string property_val4( str4 ) ;


      CORBA::TypeCode_var tc1 = prop_val1.type();
      CORBA::TypeCode_var tc2 = prop_val2.type();
      CORBA::TypeCode_var tc3 = prop_val3.type();
      CORBA::TypeCode_var tc4 = prop_val4.type();


      if ( 
      (num_ents == 2) &&
       (name_ent1.compare("NEWFABLBENTITY1") == 0) && 
       (name_ent2.compare("NEWFABLBENTITY2") == 0) && 
       (ent_status1.compare("UNKNOWN") == 0) && 
       (ent_status2.compare("UNKNOWN") == 0) && 
       (ent_status_info1.compare("Default entity status") == 0) && 
       (ent_status_info2.compare("Default entity status") == 0) && 

      (num_props == 4) &&
      (property_val1 == 0 ) &&
      (property_val2 == ((CORBA::Float) 14.4) ) &&
      (property_val3 == 13 ) &&
      (property_val4.compare("MYSTRING_LB") == 0) &&

      (tc1->kind() == CORBA::tk_long) &&
      (tc2->kind() == CORBA::tk_float) &&
      (tc3->kind() == CORBA::tk_short) &&
      (tc4->kind() == CORBA::tk_string) &&

      (name_prop1.compare("Cdmw.GroupSize") == 0) &&
      (name_prop2.compare("LBG_PROP_FLOAT") == 0) &&
      (name_prop3.compare("LBG_PROP_SHORT") == 0) &&
      (name_prop4.compare("LBG_PROP_STR") == 0) &&

      (lbg_name.compare("FABLBGROUP3") == 0) &&
      (lbg_stat == CdmwPlatformMngt::INITIAL) &&
          ( lbg_stat_info.compare("") == 0) &&
      ( ele_orig_path_str.compare(ele_path_str) == 0) )

        {
        //ok
            CPPUNIT_ASSERT (1);
        }
       else
        CPPUNIT_ASSERT (0); //error
         

      }
      catch (...)
      {
      //error
        CPPUNIT_ASSERT (0);
      }


    }


    void TestComponentsAndGroups::get_lbgroup_properties()
    {
      CdmwPlatformMngt::LBGroup_var lbg ;

      try
        {
         lbg = app->get_lb_group("FABLBGROUP3");

         //get the properties ...
         CORBA::Any_var prop_value1;
         lbg->get_attribute("LBG_PROP_SHORT", prop_value1,
          CdmwPlatformMngt::USER_DEFINED );

         CORBA::Any_var prop_value2;
         lbg->get_attribute("LBG_PROP_FLOAT", prop_value2,
          CdmwPlatformMngt::USER_DEFINED );

         CORBA::Any_var prop_value3;
         lbg->get_attribute("LBG_PROP_STR", prop_value3,
          CdmwPlatformMngt::USER_DEFINED );
 
         CORBA::Any_var prop_value4;
         lbg->get_attribute("GroupSize", prop_value4,
          CdmwPlatformMngt::PRE_DEFINED );

         //get number of properties ...
         CORBA::ULong num_props = lbg->get_number_of_properties();

         CORBA::TypeCode_var tc1 = prop_value1->type();
         CORBA::TypeCode_var tc2 = prop_value2->type();
         CORBA::TypeCode_var tc3 = prop_value3->type();
         CORBA::TypeCode_var tc4 = prop_value4->type();

         CORBA::Short value1; 
         (*prop_value1) >>= value1 ;

         CORBA::Float value2; 
         (*prop_value2) >>= value2 ;

         const char* value3_tmp;
         prop_value3 >>= value3_tmp;
         std::string value3( value3_tmp ) ;


         CORBA::Long value4; 
         (*prop_value4) >>= value4 ;


     if ( (num_props = 4) &&
              ( value1 == 13) &&
              (value2 == ((CORBA::Float) 14.4) ) &&
              ( value3.compare("MYSTRING_LB") == 0) &&
              (value4 == 0) &&
              (tc1->kind() == CORBA::tk_short) &&
              (tc2->kind() == CORBA::tk_float) &&
              (tc3->kind() == CORBA::tk_string) &&
              (tc4->kind() == CORBA::tk_long ))
                  CPPUNIT_ASSERT(1); //ok
         else
                  CPPUNIT_ASSERT(0); //not ok


        }
      catch(...)
        {
        //error
        CPPUNIT_ASSERT(0);
    }


    }


    void TestComponentsAndGroups::set_and_get_lbgroup_properties()
    {
    CORBA::String_var info ;

    CdmwPlatformMngt::LBGroup_var lbg ;
    CORBA::Any_var prop_value1 = new CORBA::Any() ;
    CORBA::Any_var prop_value1_get;
    CORBA::Short newvalue1;

    try
     {
      lbg = app->get_lb_group("FABLBGROUP3");


      CORBA::Short value1 = 23 ;
      (*prop_value1) <<= value1 ;
      lbg->set_attribute("LBG_PROP_SHORT", (*prop_value1),
         CdmwPlatformMngt::USER_DEFINED );


      lbg->get_attribute("LBG_PROP_SHORT", prop_value1_get,
         CdmwPlatformMngt::USER_DEFINED );

      (*prop_value1_get) >>= newvalue1 ;
      }
      catch (...)
      {
    //error
        CPPUNIT_ASSERT (0);
      }


      try
      {
      CORBA::TypeCode_var tc1_get = prop_value1_get->type();

      //see number of properties ...

      CORBA::ULong num_props = lbg->get_number_of_properties();

      if ( (num_props == 6) && 
       (tc1_get->kind() == CORBA::tk_short) &&
           ( newvalue1 == 23) ) 
        {
    //test ok
        CPPUNIT_ASSERT (1);
    }
      else
        CPPUNIT_ASSERT (0); //test not ok 

      }
      catch (...)
      {
    //error
        CPPUNIT_ASSERT (0);
      }

    } 

  }                             /* End namespace PlatformMngt */
}                               /* End namespace Cdmw */
