/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include "testdeploymentmanager/TestDeploymentManager.hpp"
#include "platformapplicationlibrary/Process_impl.hpp"
#include "platformapplicationlibrary/DeploymentManager.hpp"
#include "platformapplicationlibrary/DeploymentInfo.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"



using namespace std;

using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestDeploymentManager);

namespace Cdmw
{
  namespace PlatformMngt
  {
    TestDeploymentManager::TestDeploymentManager () throw ()
    {
    }

    TestDeploymentManager::~TestDeploymentManager () throw ()
    {
    }

    void TestDeploymentManager::setUp ()
    {
      CORBA::ORB_var orb =  CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::instance ()->get_ORB ());
      DeploymentManager::get_instance()->init(orb.in());
      DeploymentManager::get_instance()->set_node_manager_port_name("21869");
      char* host_name = getenv ("HOSTNAME");
      DeploymentManager::get_instance()->set_node_manager_host_name(host_name);
    }


    void TestDeploymentManager::tearDown ()
    {
    }
    

    void TestDeploymentManager::test_set_remove_node_deployment_info()
    {
      DeploymentManager* deployment_manager = DeploymentManager::get_instance();
      char* host_name = getenv ("HOSTNAME");
      
      DeploymentInfo deployment_info1, deployment_info2, deployment_info3;
      deployment_info1.deployment_plan = "deployment_plan1";
      deployment_info2.deployment_plan = "deployment_plan2";
      deployment_info3.deployment_plan = "deployment_plan3";
          
      Application_impl* application = 0;
      TEST_INFO("set_node_deployment_info with deployment_plan1");
      TEST_INFO("set_node_deployment_info with deployment_plan2");
      TEST_INFO("set_node_deployment_info with deployment_plan3");

      deployment_manager->set_node_deployment_info(host_name,"deployment_plan1", application);
      deployment_manager->set_node_deployment_info(host_name,"deployment_plan2", application);
      deployment_manager->set_node_deployment_info(host_name,"deployment_plan3", application);

      deployment_manager->print();
      TEST_INFO("remove_node_deployment_info with deployment_plan1");
      //deployment_manager->remove_node_deployment_info(deployment_info1);
      deployment_manager->remove_node_deployment_info(host_name,"deployment_plan1");
      deployment_manager->print();
      TEST_INFO("remove_node_deployment_info with deployment_plan2");
      //deployment_manager->remove_node_deployment_info(deployment_info2);
      deployment_manager->remove_node_deployment_info(host_name,"deployment_plan2");
      deployment_manager->print();
      TEST_INFO("remove_node_deployment_info with deployment_plan3");
      //deployment_manager->remove_node_deployment_info(deployment_info3);
      deployment_manager->remove_node_deployment_info(host_name,"deployment_plan3");

      deployment_manager->print();
 

    }
    
    void TestDeploymentManager::test_set_remove_node_deployment_infos()
    {
      DeploymentManager* deployment_manager = DeploymentManager::get_instance();
      char* host_name = getenv ("HOSTNAME");
      
      Application_impl* application = 0;

      std::vector<std::string>* vect = new std::vector<std::string>();

      TEST_INFO("prepare a vector with deployment plans {deployment_plan1,deployment_plan2,deployment_plan3,deployment_plan4, deployment_plan5, deployment_plan6}");         
      vect->push_back("deployment_plan1");  
      vect->push_back("deployment_plan2");  
      vect->push_back("deployment_plan3");  
      vect->push_back("deployment_plan4");  
      vect->push_back("deployment_plan5");  
      vect->push_back("deployment_plan6");  

      TEST_INFO("set_node_deployment_infos");
      deployment_manager->set_node_deployment_infos(host_name, vect, application);
      deployment_manager->print();

      std::vector<std::string>* vect_to_remove = new std::vector<std::string>();

      TEST_INFO("prepare a vector with deployment plans to remove {deployment_plan1,deployment_plan2,deployment_plan5}");         
      vect_to_remove->push_back("deployment_plan1");    
      vect_to_remove->push_back("deployment_plan2");
      vect_to_remove->push_back("deployment_plan5");
      TEST_INFO("remove_deployment_infos the vector ");
      deployment_manager->remove_node_deployment_infos(host_name, vect_to_remove);
      deployment_manager->print();
    }


 void TestDeploymentManager::test_notify()
    {
      DeploymentManager* deployment_manager = DeploymentManager::get_instance();
      char* host_name = getenv ("HOSTNAME");
      
      DeploymentInfo deployment_info1, deployment_info2, deployment_info3;
      deployment_info1.deployment_plan = "deployment_plan1";
          
      Application_impl* application = 0;
      TEST_INFO("set_node_deployment_info with deployment_plan1");

      deployment_manager->set_node_deployment_info(host_name,"deployment_plan1", application);
      TEST_INFO("Testing DeploymentManager notify");

      try
      {
      deployment_manager->notify(host_name, "deployment_plan1");
      }
      catch(...)
      {
      std::cout<<"Exception of notify "<<std::endl; 
      }
     
      std::cout<<" Waiting for the notify ..."<<std::endl;
      sleep (10);  
       
      deployment_manager->remove_node_deployment_info(host_name,"deployment_plan1");

      deployment_manager->print();
 
    }


    
  } /* End namespace PlatformMngt */
}/* End namespace Cdmw */





