/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.platformmngt.test;


import cdmw.testutils.Testable;
import cdmw.common.Options;

import cdmw.ossupport.OS;
import cdmw.orbsupport.StrategyList;
import cdmw.orbsupport.ORBSupport;
import cdmw.common.BadParameterException;
import cdmw.common.BadOrderException;
import java.io.FileWriter;
import java.io.File;
import com.thalesgroup.CdmwPlatformMngt.*;

import cdmw.commonsvcs.naming.*;

public class MainTestSIM {

    private final static int SUCCESS = 0;
    private final static int FAILURE = 1;


    /**
     * Thread pool size of the Root POA
     */
    public static final int POA_THREAD_POOL_SIZE = 5;


    private static class  MyTestThread extends Testable 
    {
        // Implementation references to check if a method has been called
        private NodeManagerImpl nodeManagerImpl;
        private NodeApplicationManagerImpl nodeApplicationManagerImpl;
        private NodeApplicationImpl nodeApplicationImpl;

        //the ORB
        private org.omg.CORBA.ORB orb;
        // NodeManager Stringfied reference
        private String  nodeManagerString;
        private Boolean mode;

    MyTestThread (org.omg.CORBA.ORB orb, String nodeManagerString, NodeManagerImpl nodeManagerImpl)
    {
        this.orb = orb; 
        this.nodeManagerString = nodeManagerString;
        this.nodeManagerImpl   = nodeManagerImpl;
        this.out = java.lang.System.out;
        this.name = "FakeNodeManger Test";
    }    

    /*
     *  Check if a method has been called
     */
        void doCheck()
        {
            org.omg.CORBA.Object obj = null;
            out.println("\n    Begin the check");

            out.println("check on NodeManagerImpl");            

            // test prepare plan
            try
        {
            OS.sleep(Testable.getTimescale() * 10000);
        }
            catch(Exception e)
        {
        }

            loopAndCheck(nodeManagerImpl, "/prepare_plan/prepare_plan");
            NodeApplicationManagerImpl nodeApplicationManagerImpl = nodeManagerImpl.getNodeApplicationManagerImpl();  

            out.println("check nodeApplicationManagerImpl");  
            loopAndCheck(nodeApplicationManagerImpl, "/start_launch/start_launch");
            try
        {
            OS.sleep(Testable.getTimescale() * 20000);
        }
            catch(Exception e)
        {
        }
            out.println("check nodeApplicationImpl");
            NodeApplicationImpl nodeApplicationImpl = nodeApplicationManagerImpl.getNodeApplicationImpl();
            loopAndCheck(nodeApplicationImpl, "/finish_launch/start");

            out.println("<<< ready for stop sytem  >>>");
//            loopAndCheck(nodeApplicationManagerImpl, "/start_launch/start_launch/destroy_application_manager/destroy_application_manager");   

//            loopAndCheck(nodeManagerImpl, "/prepare_plan/prepare_plan/destroy_manager/destroy_manager");
          
            if(orb != null) 
                orb.destroy();

    }   



    void loopAndCheck(Object object, String status_desired)
    {
        int counter = 0;
         
        while((counter < 20)&&((!object.toString().equals(status_desired)))) 
        {
//          out.println("waiting .... "+ status_desired);
            out.println("waiting SMG calls on NodeManager ...");
            try
            {
                OS.sleep(Testable.getTimescale() * 1000);
            }
            catch(Exception e)
            {
            }
            counter++;
        }
        if (object.toString().equals(status_desired))
        succeed();
        else
        fail();
    } 
        
    public void doTests()
    {
        try
        {
            
//            setNbOfRequestedTestOK(5);
            setNbOfRequestedTestOK(3);

            org.omg.CORBA.Object obj = null;
               
            //            OS.sleep(Testable.getTimescale() * 10000); 
            obj = orb.string_to_object(nodeManagerString);
            //    obj = orb.resolve_initial_references("CdmwDnCNodeManager");     
            com.thalesgroup.CdmwDnC.NodeManager nodeManager 
            = com.thalesgroup.CdmwDnC.NodeManagerHelper.narrow(obj);                   
            /*
              out.println("Thread started");

              com.thalesgroup.CdmwDnC.NodeApplicationManager nodeApplicationManager = 
              nodeManager.prepare_plan("local_plan_name");

              out.println("client: nodeApplicationManager recovered by nodeManager, called" +
              "nodeManager.prepare_plan(local_plan_name)");
              com.thalesgroup.CdmwDnC.NodeApplication nodeApplication = nodeApplicationManager.start_launch();

              out.println("client: nodeApplication recovered by nodeApplicationManager, called nodeApplicationManager.start_launch");

              nodeApplication.finish_launch();

              out.println("client: called nodeApplication.finish_launch");
            
            */
                    doCheck();
        }
        catch(Exception ex)
        {
            out.println(ex);
        }

    }
    };
    

    public static void main(String[] args) {
    
        org.omg.CORBA.ORB orb = null;
        try{
 
        String exceptionAt = OS.getOptionValue (args, "--exception-at");
        String testMode = OS.getOptionValue(args, "--test-mode");
 
        String status = "Begin";
        // Init ORB
        StrategyList orbStrategies = new StrategyList();
        orbStrategies.addORBThreaded();
        orbStrategies.addPOAThreadPool(POA_THREAD_POOL_SIZE);
        orb = ORBSupport.ORBInit(args, orbStrategies);

        //orb = org.omg.CORBA.ORB.init(args, ORBUtils.getProperties());
        org.omg.PortableServer.POA root = null;
        org.omg.CORBA.Object obj = null;

        // Resolve the Root POA
        obj = orb.resolve_initial_references("RootPOA");
        root = org.omg.PortableServer.POAHelper.narrow(obj);
        // Initialize the server so that incoming requests are
        // handled from this point on
        root.the_POAManager().activate();

        NodeManagerImpl nodeManagerImpl = new NodeManagerImpl(orb, exceptionAt);

        // activates the servant and creates the CORBA object
        com.thalesgroup.CdmwDnC.NodeManager nodeManager = nodeManagerImpl._this(orb);

            //cdmw.orbsupport.ORBSupport.registerInitialReference(orb, "CdmwDnCNodeManager", nodeManager);

            cdmw.orbsupport.ORBSupport.bindObjectToCorbaloc(orb,"CdmwNodeManager", nodeManager);
 
        //start the first thread to check object registration and to call methods
        String localhost = OS.getHostname();


        java.lang.System.out.println("NodeManager has been started on: " + localhost);

        String corbaname = "corbaname:iiop:localhost:21869/NameService#CdmwNodeManager"; 
            obj = orb.resolve_initial_references("NameService");
            org.omg.CosNaming.NamingContext nc = org.omg.CosNaming.NamingContextHelper.narrow(obj);

            NamingInterface ni = new NamingInterface(nc);
            ni.rebind("CdmwNodeManager", nodeManager, true) ; 
        //Start the thread
        if ((testMode.equals("yes"))&&(exceptionAt.equals("no")))
        {  
            java.lang.System.out.println("start the Thread test"); 
            MyTestThread t = new MyTestThread(orb, corbaname, nodeManagerImpl);
            t.start();
        } 
            
        // ORB run
        orb.run();
        }
        catch (org.omg.CORBA.ORBPackage.InvalidName in) {
        java.lang.System.out.println("org.omg.CORBA.ORBPackage.InvalidName");
        } catch(org.omg.PortableServer.POAManagerPackage.AdapterInactive ai) {
        java.lang.System.out.println("Can't activate POA Manager");
        }
        catch(Exception e) {
        // cleanup the ORB to exit completely
        if (orb!=null) {
            orb.shutdown(false);
            orb.destroy();
        }
        }

    }

    }

