/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/* @author Fabrizio Lorenna <card03@selex-si.com> */

#include <iostream>
#include <limits>

#include "testfacade_Application/TestFacade_Application.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"


#include <platformftproxy/InitUtils.hpp>
#include <platformftproxy/SystemProxy.hpp>
#include <SystemMngt/platformlibrary/Iterator.hpp>
#include "idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"

#include <FaultTolerance/ftreplicationmanager/InitUtils.hpp>
#include <FaultTolerance/idllib/FT.stub.hpp>
#include <platformftaccessorlibrary/SystemAccessor_impl.hpp>


using namespace std;

using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION(TestFacade_Application);

namespace Cdmw
{
namespace PlatformMngt
{



TestFacade_Application::TestFacade_Application() 
{

}


TestFacade_Application::~TestFacade_Application()
{

}

    bool    TestFacade_Application::m_is_one_shot_initialized=false;
    CdmwPlatformMngt::System_var TestFacade_Application::mysystem;
    CdmwPlatformMngt::Application_var TestFacade_Application::myapplication;


    void TestFacade_Application::setUp()
    {
      TestFacade_Application::one_shot_init();
    }


    void TestFacade_Application::one_shot_init()
    {
      if(!m_is_one_shot_initialized)
      {
        m_is_one_shot_initialized = true;
   
      try
      {
        CORBA::ORB_var orb;
        orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager:: instance ()->get_ORB ());

        PortableServer::POA_var poa = PortableServer::POA::_duplicate (Cdmw::TestUtils::CORBATestManager:: instance ()->get_POA ());


        PortableServer::POAManager_var poaMgr = PortableServer::POAManager::_duplicate(poa->the_POAManager());
        poaMgr->activate();


        CORBA::Object_var objref;

        //define the corbaloc
        const char *corbaloc_str = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";

        objref = orb->string_to_object(corbaloc_str);

        //resolve the System_ptr
        mysystem = CdmwPlatformMngt::System::_narrow(objref.in() );
        
        if (CORBA::is_nil (mysystem.in ()))
          {
           // TEST_INFO ("Error in narrowing CdmwPlatformMngt::System");
            exit (-1);
          }

        /*get the Application ptr */
    myapplication = mysystem->get_application("HELLO_WORLD");

      }
      catch (const Cdmw::Exception & ex)
      {
        std::cerr << ex.what () << std::endl;
        exit (-1);
      }
      catch (const CORBA::Exception & ex)
      {
        std::cerr << ex << std::endl;
        exit (-1);
      }
      catch (...)
      {
        std::cerr << "unexpected exception" << std::endl;
        exit (-1);
      }
      }
    }

    void TestFacade_Application::get_attribute_status()
       {
       try
    {
        //TEST_INFO("Test in get_attribute of 'status' :");
    CORBA::Any_var outany = new CORBA::Any(); 
        myapplication->get_attribute(CdmwPlatformMngt::TECH_STATUS, outany, CdmwPlatformMngt::PRE_DEFINED);
        /*extract outany value;*/
    const char* stat = "Stopped" ;
        outany.in() >>= CORBA::Any::to_string(stat,0);

        cout << "Appl. status=" << stat << " " << CdmwPlatformMngt::APPLICATION_INITIAL << endl;
        std::string status(stat);
        if ( status == "Initial")
      {
      CPPUNIT_ASSERT(1); /*test ok*/
      }
        else
      {
      CPPUNIT_ASSERT(0); /*test not ok*/
      }

    }
    catch(...)
      {
          cout <<"Error, exception in get_attribute with 'status' "<<endl;
      CPPUNIT_ASSERT(0);
      }

       }

    void TestFacade_Application::get_attribute_status_info()
       {
    try
    {
        //TEST_INFO("Test in get_attribute of 'status_info' :");
    CORBA::Any_var outany; 
        myapplication->get_attribute(CdmwPlatformMngt::TECH_STATUS_INFO, outany, CdmwPlatformMngt::PRE_DEFINED);
        /*extract outany value;*/
    char * status_info2 = "HELLO"; 
    /*cout <<"status_info2 before="<<status_info2<<endl;*/
        (*outany) >>= CORBA::Any::to_string(status_info2, 0);

    /*cout <<"status_info2 after="<<status_info2<<endl;*/

    string tmp1(status_info2);
    if (tmp1 == "")
      {
          CPPUNIT_ASSERT(1); /*test ok*/
      }
        else
      {
          CPPUNIT_ASSERT(0); /*test not ok*/
      }


    }
    catch(...)
      {
          cout <<"Error, exception in get_attribute with 'status_info' "<<endl;
      CPPUNIT_ASSERT(0);
      }
       }


    void TestFacade_Application::get_attribute_mode()
       {
    try
    {
        //TEST_INFO("Test in get_attribute of 'mode' :");
    CORBA::Any_var outany; 
    CdmwPlatformMngt::FunctioningMode func_mode;
        myapplication->get_attribute(CdmwPlatformMngt::MODE, outany, CdmwPlatformMngt::PRE_DEFINED);
    func_mode = CdmwPlatformMngt::DEGRADED_MODE;
        /*extract outany value;*/
        (*outany) >>= func_mode ;

    if (func_mode == CdmwPlatformMngt::NORMAL_MODE)
      {
          CPPUNIT_ASSERT(1); /*test ok*/
      }
        else
      {
          CPPUNIT_ASSERT(0); /*test not ok*/
      }


    }
    catch(...)
      {
          cout <<"Error, exception in get_attribute with 'mode' "<<endl;
      CPPUNIT_ASSERT(0);
      }
       }


    void TestFacade_Application::get_attribute_mode_info()
       {
    try
    {
        //TEST_INFO("Test in get_attribute of 'mode_info' :");
    CORBA::Any_var outany; 
        myapplication->get_attribute(CdmwPlatformMngt::MODE_INFO, outany, CdmwPlatformMngt::PRE_DEFINED);
    char * mode_info = "HELLO";
    (*outany) >>= CORBA::Any::to_string(mode_info, 0);
        string tmp1(mode_info) ;

    if (tmp1 == "" )
      {
          CPPUNIT_ASSERT(1); /*test ok*/
      }
        else
      {
          CPPUNIT_ASSERT(0); /*test not ok*/
      }

    }
    catch(...)
      {
          cout <<"Error, exception in get_attribute with 'mode_info' "<<endl;
      CPPUNIT_ASSERT(0);
      }
       }

    void TestFacade_Application::get_attribute_exception()
       {
        CORBA::Any_var outany;

       //  TEST_INFO("Test in get_attribute to throw AttributeNotFound :");

        myapplication->get_attribute("dummy", outany, CdmwPlatformMngt::PRE_DEFINED);

       }

    void TestFacade_Application::set_attribute_uda()
       {
       
      CORBA::Any inany;
      CORBA::Short proshort1 = 99 ;
        
     try
     {
     myapplication->set_attribute("CIAO", inany , CdmwPlatformMngt::USER_DEFINED);
     CPPUNIT_ASSERT(0);
     
     }
     
     catch (const CdmwPlatformMngt::AttributeNotFound &)
     {
     CPPUNIT_ASSERT(1);
     }
        
       try
        {


        //TEST_INFO("Test in set_attribute of user defined attribute :");

        inany <<= proshort1;

        myapplication->define_property("CIAO", inany );

        CORBA::Any_var outany;
        myapplication->get_attribute("CIAO", outany, CdmwPlatformMngt::USER_DEFINED);
        CORBA::Short proshort2 = 0 ;

        (*outany) >>= proshort2;
        if (proshort2 == 99 )
          {
          CPPUNIT_ASSERT(true);
          }
        else
          {
          CPPUNIT_ASSERT(false);
          }
        }
       catch(...)
          {
          CPPUNIT_ASSERT(false); /*test not ok*/
          }
       }


    void TestFacade_Application::get_attribute_uda()
       {
       try
        {
        CORBA::Any_var outany;
        //TEST_INFO("Test in get_attribute of user defined attribute :");

        myapplication->get_attribute("CIAO", outany, CdmwPlatformMngt::USER_DEFINED);
        CORBA::Short proshort = 0 ;

        (*outany) >>= proshort;
        if (proshort == 99 )
          {
          CPPUNIT_ASSERT(true);
          }
        else
          {
          CPPUNIT_ASSERT(false);
          }
        }
        catch(...)
          {
          CPPUNIT_ASSERT(false); /*test not ok*/
          }

       }



} /* End namespace PlatformMngt*/
} /* End namespace Cdmw*/


