/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/* @author Fabrizio Lorenna <card03@selex-si.com> */

#include <iostream>
#include <limits>
#include <stdlib.h>
#include <string.h>

#include "teststdattributes_Process/TestStdAttributes_Process.hpp"
#include "idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"

#include <platformftproxy/InitUtils.hpp>
#include <platformftproxy/SystemProxy.hpp>
#include <SystemMngt/platformftproxy/AccessorLookupService.hpp>
#include <SystemMngt/platformftproxy/DefaultAccessorLookupService.hpp>
#include <SystemMngt/platformlibrary/Iterator.hpp>

#include <FaultTolerance/ftreplicationmanager/InitUtils.hpp>
#include <FaultTolerance/idllib/FT.stub.hpp>
#include <platformftaccessorlibrary/SystemAccessor_impl.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp>


using namespace std;

using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestStdAttributes_Process);

namespace Cdmw
{
  namespace PlatformMngt
  {



    TestStdAttributes_Process::TestStdAttributes_Process () throw ()
    {

    }


    TestStdAttributes_Process::~TestStdAttributes_Process () throw ()
    {

    }

/*
@brief   do_tests method for new navigability methods of IBB2
@brief   developments  .
*/


// //  set_nbOfRequestedTestOK (10);

    void TestStdAttributes_Process::setUp()
    {
      TestStdAttributes_Process::one_shot_init();
    }

    bool    
    TestStdAttributes_Process::m_is_one_shot_initialized=false;
  
    CdmwPlatformMngt::System_var TestStdAttributes_Process::mysystem;
    CdmwPlatformMngt::Process_var TestStdAttributes_Process::proc;
    CdmwPlatformMngt::Application_var TestStdAttributes_Process::app;
    CdmwPlatformMngt::ProcessInfo_var TestStdAttributes_Process::proc_info;

    void    
    TestStdAttributes_Process::one_shot_init()
    {
      if(!m_is_one_shot_initialized)
      {
        m_is_one_shot_initialized = true; 
     
      try     
      {
        CORBA::ORB_var orb;
        orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager:: instance ()->get_ORB ());
        /*
           ===================================================
           Get the HelloInterface object
           ===================================================

           import the object reference from the ior file
         */

        PortableServer::POA_var poa = PortableServer::POA::_duplicate (Cdmw::TestUtils::CORBATestManager:: instance ()->get_POA ());

        PortableServer::POAManager_var poaMgr = PortableServer::POAManager::_duplicate(poa->the_POAManager());
        poaMgr->activate();

        CORBA::Object_var objref;

        //define the corbaloc
        const char *corbaloc_str = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";

        objref = orb->string_to_object(corbaloc_str);

        //resolve the System_ptr
        mysystem = CdmwPlatformMngt::System::_narrow(objref.in() );

        if (CORBA::is_nil (mysystem.in ()))
          {
           // TEST_INFO ("Error in narrowing CdmwPlatformMngt::System");
            exit (-1);
          }

    /* from System, use a method to retrieve an application */
    app = mysystem->get_application ("HELLO_WORLD");


    /* from the application, use a method to retrieve a Process */

    /* retrieve the current hostname */
    char *hostname = getenv ("HOSTNAME");

    /* name of process to test */
    std::string proc_name = "HelloProcess1";


    proc = app->get_process (proc_name.c_str (), hostname);
       }
      catch (const Cdmw::Exception & ex)
      {
        std::cerr << ex.what () << std::endl;
        exit (-1);
      }
     catch (const CORBA::Exception & ex)
      {
        std::cerr << ex << std::endl;
        exit (-1);
      }
      catch (...)
      {
        std::cerr << "unexpected exception" << std::endl;
        exit (-1);
      }
      }
    }



    void TestStdAttributes_Process::name()
        {
    std::string proc_name = "HelloProcess1";
    CORBA::String_var name ;
    //TEST_INFO ("Test to get process name");
    name = proc->name ();

    if (!strcmp (proc_name.c_str (), name.in ()))
      {
        CPPUNIT_ASSERT (1); /* test ok */
      }
    else
      {
        CPPUNIT_ASSERT (0); /* test not ok */
      }
        }

    void TestStdAttributes_Process::get_status()
        {
    //TEST_INFO ("Test to get process status");

    CORBA::String_var status_info;

    CdmwPlatformMngt::ProcessStatus proc_status =
      proc->get_status (status_info.out ());

    if (proc_status == CdmwPlatformMngt::PROCESS_INITIAL ||
        proc_status == CdmwPlatformMngt::PROCESS_RUNNING)
      {
        CPPUNIT_ASSERT (1);
      }
    else
      {
        CPPUNIT_ASSERT (0);
      }
        }

    void TestStdAttributes_Process::get_process_info()
        {
    //TEST_INFO ("Test to get process info: exe, port, args, ...");

    //CdmwPlatformMngt::ProcessInfo_var proc_info;
        char flag_get_proc_info=0;

    try
    {
      proc_info = proc->get_process_info ();
      flag_get_proc_info = 1;
    }
    catch (...)
    {
      flag_get_proc_info = 0;
    }

    if (flag_get_proc_info == 0)
      {
        CPPUNIT_ASSERT (0); /* test not ok */
      }
    else
      {
        CPPUNIT_ASSERT (1); /* test ok */
      }
        }

    void TestStdAttributes_Process::set_process_info()
        {
    //TEST_INFO ("Test to set process info: exe, port, args, ...");
        char flag_set_proc_info=0;

    try
    {
      proc->set_process_info (proc_info.in ());
      flag_set_proc_info = 1;
    }
    catch (...)
    {
      flag_set_proc_info = 0;
    }

    if (flag_set_proc_info == 0)
      {
        CPPUNIT_ASSERT (0); /* test not ok */
      }
    else
      {
        CPPUNIT_ASSERT (1); /* test ok */
      }
        }


    /* test on set_exit_autorestart_info( .... ) */
    void TestStdAttributes_Process::set_exit_autorestart_info()
    {
        CdmwPlatformMngt::ExitAutoRestartInfo exit_autorestart_info;
        exit_autorestart_info.default_exit_action=CdmwPlatformMngt::RESTART_APPLICATION;
        exit_autorestart_info.exit_actions.length(2);
        CdmwPlatformMngt::ExitAction ea0, ea1;
        ea0.exit_code=1;
        ea0.exit_action=CdmwPlatformMngt::NO_ACTION;
        exit_autorestart_info.exit_actions[0]=ea0;
        ea1.exit_code=4;
        ea1.exit_action=CdmwPlatformMngt::RESTART_PROCESS;
        exit_autorestart_info.exit_actions[1]=ea1;
        
        CdmwPlatformMngt::AutoRestartInfo autorestart_info;
        autorestart_info.max_nbr_of_restart_attempt = 3;
        autorestart_info.auto_restart_control = CdmwPlatformMngt::USE_COUNTER;
        CORBA::Any_var myany = new CORBA::Any();
        *myany <<= autorestart_info;
        try {
            proc->set_attribute(CdmwPlatformMngt::AUTORESTART_INFO, myany.in(), CdmwPlatformMngt::PRE_DEFINED);
        }
        catch(...) {
            CPPUNIT_ASSERT(false);    
        }
        
        proc->set_exit_autorestart_info (exit_autorestart_info);
    }

    /*test get_exit_autorestart_info( ... ) */
    void TestStdAttributes_Process::get_exit_autorestart_info()
    {
        CdmwPlatformMngt::ExitAutoRestartInfo_var exit_info;
        exit_info=proc->get_exit_autorestart_info();
        CPPUNIT_ASSERT_EQUAL( CdmwPlatformMngt::RESTART_APPLICATION,
                                exit_info->default_exit_action);
        CPPUNIT_ASSERT_EQUAL( CORBA::ULong(2), exit_info->exit_actions.length());
        CPPUNIT_ASSERT_EQUAL( CdmwPlatformMngt::NO_ACTION,
                                exit_info->exit_actions[0].exit_action); 
        CPPUNIT_ASSERT_EQUAL(  CORBA::Long(1), exit_info->exit_actions[0].exit_code);
        CPPUNIT_ASSERT_EQUAL( CdmwPlatformMngt::RESTART_PROCESS,
                                exit_info->exit_actions[1].exit_action);
        CPPUNIT_ASSERT_EQUAL( CORBA::Long(4), exit_info->exit_actions[1].exit_code);

        CORBA::Any_var any_var;
        CdmwPlatformMngt::AutoRestartInfo* ari;

        try {
        proc->get_attribute (CdmwPlatformMngt::AUTORESTART_INFO, any_var.out(), CdmwPlatformMngt::PRE_DEFINED);
       
        }
        catch(...) {
            CPPUNIT_ASSERT(false);
        }

    CORBA::Boolean extract_succeeds = (any_var.in() >>= ari);
        CPPUNIT_ASSERT (extract_succeeds);
        CPPUNIT_ASSERT_EQUAL (CORBA::ULong(3), ari->max_nbr_of_restart_attempt) ;
        CPPUNIT_ASSERT_EQUAL (CdmwPlatformMngt::USE_COUNTER , ari->auto_restart_control) ;
     
    }

    void TestStdAttributes_Process::restart_attempt()
        {
      //TEST_INFO ("Test on Process::restart_attempt: ");
      CORBA::Long varlong = proc->restart_attempt ();
      varlong = 0;
     }


    /*
    void TestStdAttributes_Process::get_process_mode_exception()
    {
        //TEST_INFO ("Test on Process::get_process_mode: ");
        CdmwPlatformMngt::ProcessMode proc_mode = proc->get_process_mode ();
        proc_mode = CdmwPlatformMngt::ProcessMode (BACKUP);
    }
    */


    void TestStdAttributes_Process::get_memory_resident_exception()
        {
     // TEST_INFO ("Test on Process::get_memory_resident: ");
      bool memory_bool = proc->get_memory_resident ();
      memory_bool = true;
     }


    void TestStdAttributes_Process::set_memory_resident_exception()
        {
     // TEST_INFO ("Test on Process::set_memory_resident: ");
      bool memory_bool = true;
      proc->set_memory_resident (memory_bool);
     }

    void TestStdAttributes_Process::get_core_dump_size_exception()
        {
    //  TEST_INFO ("Test on Process::get_core_dump_size: ");
      CORBA::Long mylong;
      mylong = proc->get_core_dump_size ();

        }


    void TestStdAttributes_Process::get_name_by_snapshot()
        {
    //TEST_INFO ("Test to get process_name by get_snapshot: ");
    std::string proc_name = "HelloProcess1";
    CdmwPlatformMngt::ProcessSnapshot_var snap;
    snap = proc->get_snapshot ();

    string conf1 (snap->process_name);

    if (conf1 == proc_name)
      {
        CPPUNIT_ASSERT (1); /* test ok */
      }
    else
      {
        CPPUNIT_ASSERT (0); /* test not ok */
      }
        }

    void TestStdAttributes_Process::get_status_by_snapshot()
        {
    //TEST_INFO ("Test to get status by get_snapshot: ");

    CdmwPlatformMngt::ProcessSnapshot_var snap;
    snap = proc->get_snapshot ();
    if ((snap->process_status == CdmwPlatformMngt::PROCESS_INITIAL) ||
        (snap->process_status == CdmwPlatformMngt::PROCESS_RUNNING))
      {
        CPPUNIT_ASSERT (1);
      }
    else
      {
        CPPUNIT_ASSERT (0);
      }
        }


    void TestStdAttributes_Process::get_run_as_root_exception()
        {
      //TEST_INFO ("Test on Process::get_run_as_root: ");
      bool mybool = proc->get_run_as_root ();
      mybool = true;
        }
    
    void TestStdAttributes_Process::force_manual_start()
        {
      CORBA::Boolean mybool = true;
      CORBA::Boolean mybool_retn;

     CORBA::Any_var any_var = new CORBA::Any();     
     try
     {
        *any_var <<= CORBA::Any::from_boolean(mybool);    
        proc->set_attribute (CdmwPlatformMngt::FORCE_MANUAL_START, any_var.in(), CdmwPlatformMngt::PRE_DEFINED);
        proc->get_attribute (CdmwPlatformMngt::FORCE_MANUAL_START, any_var.out() , CdmwPlatformMngt::PRE_DEFINED);
    }
    catch (const CdmwPlatformMngt::AttributeNotFound&)
    {
           CPPUNIT_ASSERT(0) ; /* test not ok */
    }
    CORBA::Boolean extract_succeeds = any_var >>= CORBA::Any::to_boolean(mybool_retn); 
    CPPUNIT_ASSERT(true==extract_succeeds) ; 
    CPPUNIT_ASSERT(mybool == mybool_retn);
        }

    void TestStdAttributes_Process::get_last_exit_code()
        {
          CdmwPlatformMngt::ProcessExitInfo last_exit_code;
          last_exit_code.exit_event=CdmwPlatformMngt::UNKNOWN;
          last_exit_code.event_value=-15;
          last_exit_code.core_dumped=CdmwPlatformMngt::CORE_UNKNOWN;
          last_exit_code = proc->get_last_exit_code();
          CPPUNIT_ASSERT(last_exit_code.event_value!=-15);
        }

    /*
    void TestStdAttributes_Process::set_process_arguments()
    {
        proc->set_process_arguments("TEST");
    }
    */

    /*
    void TestStdAttributes_Process::get_process_arguments()
    {
        char * str = 0;
        str = proc->get_process_arguments();
    }
    */
    
    void TestStdAttributes_Process::set_indicator_relevant_status()
        {
      bool indicator_relevant_status = true ;
      proc->set_indicator_relevant_status(indicator_relevant_status);
        }

    void TestStdAttributes_Process::get_indicator_relevant_status()
        {
      bool indicator_relevant_status = false ;
      indicator_relevant_status = proc->get_indicator_relevant_status();
        }

    void TestStdAttributes_Process::trace_filters()

        {    
            CORBA::Any_var myany = new CORBA::Any();
            *myany <<= "*,*,!/c1,d1,1/";
            
            try {
                proc->set_attribute (CdmwPlatformMngt::TRACE_FILTERS, myany.in(), CdmwPlatformMngt::PRE_DEFINED);
                CPPUNIT_ASSERT(true); /* trace filters is only for unmanaged processes*/
                              /* for unmanaged process there is not
                       * effect, and no exception is thrown */
            }
        /*
            catch (const CdmwPlatformMngt::AttributeNotFound& e)
            {
                CPPUNIT_ASSERT(true);
            }
        */
        
            catch(...) {
                CPPUNIT_ASSERT(false);    
            }         
            
            try {
                proc->get_attribute(CdmwPlatformMngt::TRACE_FILTERS, myany.out(), CdmwPlatformMngt::PRE_DEFINED);
                CPPUNIT_ASSERT(false); /* trace filters is only for unmanaged processes*/
                              /*  for get operation AttributeNotFound must
                       *  be thrown */
            }
            catch (const CdmwPlatformMngt::AttributeNotFound& e)
            {
                CPPUNIT_ASSERT(true);
            }

            catch(...) {
                CPPUNIT_ASSERT(false);    
            }         
            
        }

    void TestStdAttributes_Process::restart_warmth()
    {
        // TEST_INFO ("Test on Process::restart_warmth: ");
        
        // Get requested restart warmth
        CdmwPlatformMngt::RestartWarmth myrestart = CdmwPlatformMngt::COLD_RESTART;
        CORBA::Any_var myany;// = new CORBA::Any();
        try {
            proc->get_attribute(CdmwPlatformMngt::REQUESTED_RESTART_WARMTH, myany.out(), CdmwPlatformMngt::PRE_DEFINED);
            CPPUNIT_ASSERT((myany.in()) >>= myrestart);
            CPPUNIT_ASSERT ( myrestart == CdmwPlatformMngt::HOT_RESTART);
        }
        catch(...) {
            CPPUNIT_ASSERT(false);    
        }         
        
        // Set default restart warmth
        myrestart = CdmwPlatformMngt::COLD_RESTART;          
        *myany <<= myrestart;
        try {
            proc->set_attribute (CdmwPlatformMngt::DEFAULT_RESTART_WARMTH, myany.in(), CdmwPlatformMngt::PRE_DEFINED);
        }
        catch(...) {
            CPPUNIT_ASSERT(false);    
        }         
        
        // Get default restart warmth
        try {
            proc->get_attribute (CdmwPlatformMngt::DEFAULT_RESTART_WARMTH, myany.out(), CdmwPlatformMngt::PRE_DEFINED);
            CPPUNIT_ASSERT(myany.in() >>= myrestart);
            CPPUNIT_ASSERT ( myrestart == CdmwPlatformMngt::COLD_RESTART );
        }
        catch(...) {
            CPPUNIT_ASSERT(false);    
        }

        // Get requested restart warmth
        try {
            proc->get_attribute (CdmwPlatformMngt::REQUESTED_RESTART_WARMTH, myany.out(), CdmwPlatformMngt::PRE_DEFINED);
            CPPUNIT_ASSERT(myany.in() >>= myrestart);
            CPPUNIT_ASSERT ( myrestart == CdmwPlatformMngt::HOT_RESTART );
        }
        catch(...) {
            CPPUNIT_ASSERT(false);    
        }
    }
    
    void TestStdAttributes_Process::get_pid()
    {
 
      try
      {
      CPPUNIT_ASSERT(0 == (proc->pid ()) );
      }
      catch (...)
      {
      CPPUNIT_ASSERT(0);
      }

     }
     
    void TestStdAttributes_Process::get_previous_pid()
    {
 
      try
      {
      CPPUNIT_ASSERT(0 == (proc->previous_pid ()) );
      }
      catch (...)
      {
      CPPUNIT_ASSERT(0);
      }

     }
                
    void  TestStdAttributes_Process::get_predefined_attributes()
    {
        // Get predefined attributes of an unmanaged process
        CORBA::Any_var any_var = new CORBA::Any();
        CORBA::Boolean extract_succeeds = false;
        try {
            
            //The following properties are already tested in other unit tests
            //any_var = new CORBA::Any();
            //proc->get_attribute(CdmwPlatformMngt::FORCE_MANUAL_START, any_var.out(), CdmwPlatformMngt::PRE_DEFINED);
            
            //any_var = new CORBA::Any();
            //proc->get_attribute(CdmwPlatformMngt::AUTORESTART_INFO, any_var.out(), CdmwPlatformMngt::PRE_DEFINED);
            
            //any_var = new CORBA::Any();
            //proc->get_attribute(CdmwPlatformMngt::DEFAULT_RESTART_WARMTH, any_var.out(), CdmwPlatformMngt::PRE_DEFINED);            

            // TRACE_FILTERS property is only for managed processes
            
            proc->get_attribute(CdmwPlatformMngt::REPLICATION_ROLE, any_var.out(), CdmwPlatformMngt::PRE_DEFINED);
            CdmwPlatformMngt::RedundancyEltState rr;
            extract_succeeds = (any_var.in() >>= rr);            
            CPPUNIT_ASSERT( extract_succeeds );
            
            proc->get_attribute(CdmwPlatformMngt::REQUESTED_RESTART_WARMTH, any_var.out(), CdmwPlatformMngt::PRE_DEFINED);
            
            CdmwPlatformMngt::RestartWarmth rw;
            extract_succeeds = (any_var.in() >>= rw);            
            CPPUNIT_ASSERT( extract_succeeds );
            
        }
        catch(...) {
            CPPUNIT_ASSERT(false);    
        }
    }

  }             /* End namespace PlatformMngt */
}               /* End namespace Cdmw */
