/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/OrbSpecificProperties.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/String.hpp>
#include <Foundation/commonsvcs/serviceregistration/ServiceRegistration_impl.hpp>
#include "platformdaemonlibrary/DaemonLogMngr.hpp"
#include "platformdaemonlibrary/DaemonLogMessageId.hpp"

#include "platformdaemonlibrary/ConfigurationMngr.hpp"

#include "platformdaemon/Daemon_impl.hpp"

#include "SystemMngt/plugin/PluginManager.hpp"

#include <SystemMngt/platformlibrary/ProcessLauncher.hpp>
#include <SystemMngt/platformlibrary/ProcessKillMngr.hpp>
#include <sstream>
#ifdef CDMW_USE_FAULTTOLERANCE
#include "platformdaemon/AccessorLookupService_impl.hpp"
#include <SystemMngt/platformlibrary/Configuration.hpp>
#include <SystemMngt/platformlibrary/ClientIDGenerator.hpp>
#include <SystemMngt/platformftproxy/InitUtils.hpp>
#include <SystemMngt/platformftproxy/SystemProxy.hpp>
#include <SystemMngt/platformvaluetypes/GraphElement_impl.hpp>
#include <SystemMngt/platformvaluetypes/ProcessDef_impl.hpp>
#include "SystemMngt/idllib/CdmwPlatformMngtDaemonController.skel.hpp"

#include <FaultTolerance/ftcommon/FTConfiguration.hpp>
#include <FaultTolerance/ftmonitoring/InitUtils.hpp>
#include <FaultTolerance/idllib/FT.stub.hpp>
#include <FaultTolerance/idllib/CdmwFTReplicationManager.stub.hpp>
#include <FaultTolerance/idllib/CdmwFTManager.stub.hpp>
#include <FaultTolerance/ftmonitoring/MonitoringLibraryInterface_impl.hpp>
#include <FaultTolerance/ftultimatefallback/FTUltimateFallbackInit.hpp>
#include <tao/ORB_Core.h>
#endif


#if CDMW_ORB_VDR == tao && CDMW_ORB_VER > 13
#include <tao/ORBInitializer_Registry.h>
#endif

#include <Foundation/orbsupport/TypeTraits.hpp>
#include <Foundation/orbsupport/Codec.hpp>
#include <SystemMngt/platformftproxy/ServerOrbInitializer.hpp>


extern std::string AUTOMATICACTION;
const int POA_THREAD_POOL_SIZE = 10;    

// Daemon must be launched by the following script.
const char* CDMW_START_DAEMON_SCRIPT="cdmw_platform_daemon.sh";
const char * CDMW_SYSTEM_MNGT_PROXY = "SystemMngtProxy";
const int MAX_RETRY_ON_FTMGR = 4;
const int SLEEP_RETRY_ON_FTMGR_MS = 1000;


// This option is to check if daemon has been started by the script
// (only the script should use it !!) 
const char* STARTED_BY_SCRIPT_CHECKING_OPTION = "--isStartedByScript";

const char* FAULT_MANAGER_CORBALOC_OPTION = "--FaultManagerRegistration";

void print_syntax(char* exe)
{
    std::cout << "Syntax is :" << std::endl << exe;
    std::cout << " [ " 
              << Cdmw::Common::Options::LOCALISATION_SERVICE_OPTION 
              << "=PORT";
    std::cout << " | " 
              << Cdmw::Common::Options::PROCESS_XML_FILE_OPTION 
              << "=XMLConfigFile";
    std::cout << " [" 
              << Cdmw::Common::Options::VALIDATE_XML_OPTION 
              << "] ]";
    std::cout << " [" 
              << FAULT_MANAGER_CORBALOC_OPTION 
              << "=fault_manager_corbaloc]";
    std::cout << " [" 
              << Cdmw::Common::Options::ORB_THREADS 
              << "=NUMBER]";
    std::cout << std::endl;
}


#ifdef CDMW_USE_FAULTTOLERANCE
// Workaround to be corrected with PCR-xxxx
Cdmw::PlatformMngt::Proxy::SystemProxy *the_system_proxy_handle = NULL;
#endif



class ExitHandler : public Cdmw::OsSupport::SignalHandler
{
public:
    ExitHandler(CORBA::ORB_ptr orb)
        : m_orb(CORBA::ORB::_duplicate(orb))
    {
    }
    
    virtual ~ExitHandler() 
    {
    }
    
    int handle_signal(int sig) throw()
    {
        std::cerr<<"ExitHandler : public Cdmw::OsSupport::SignalHandler"<<std::endl;
        Cdmw::PlatformMngt::LogMngr::logMessage(
            Cdmw::PlatformMngt::WRN, 
            Cdmw::PlatformMngt::MSG_ID_SIGNAL_TRAPPED, sig);
#ifdef CDMW_USE_FAULTTOLERANCE
        // Workaround to be corrected with PCR-xxxx
        if (the_system_proxy_handle != NULL) {
            delete the_system_proxy_handle;
            the_system_proxy_handle = NULL;
        }
#endif

        // unregister all process ending callbacks
        // before ORB shutdown, because ending callbacks
        // use the Daemon_impl object (which will be deleted)
        Cdmw::OsSupport::OS::clear_process_ending_callbacks();
    
        if (!CORBA::is_nil(m_orb.in())) {
            try {
                m_orb->shutdown(false);
            } catch (const CORBA::SystemException& e) {
                Cdmw::PlatformMngt::LogMngr::logMessage(
                    Cdmw::PlatformMngt::WRN, 
                    Cdmw::PlatformMngt::MSG_ID_ORB_SUTDOWN_ERROR, 
                    Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            }
        }
        
        Cdmw::PlatformMngt::LogMngr::logMessage(
            Cdmw::PlatformMngt::WRN, 
            Cdmw::PlatformMngt::MSG_ID_END_OF_HANDLER);
        
        return 0;
    };
    
private :
    CORBA::ORB_var  m_orb;
};

namespace
{
   // Default value for ORB connection timeout
   std::string CONNECTION_TIMEOUT_PROPERTY_VALUE="100"; // 100 ms
}; // End of anonymous namespace

int main(int argc, char* argv[])
{
    using namespace Cdmw::PlatformMngt;
    using namespace Cdmw::OsSupport;
    using namespace Cdmw::Common;

    // the return code 
    int ret_code = 0;

    // checks the arguments
    std::string port_option;
    long specific_port = -1;
    long orb_thread_nb = POA_THREAD_POOL_SIZE;
    std::string xml_conf;
    bool validate = false;
    std::string fault_manager_corbaloc = "";
    bool ProcessKillMngr_init = false;
    
    // check if daemon has been started by cdmw_platform_start_daemon.sh
    std::string isStartedByScript_value =
        OS::get_option_value(argc, argv, STARTED_BY_SCRIPT_CHECKING_OPTION);
    if (isStartedByScript_value == "no") {
        std::cout << "You should not directly use " 
                  << argv[0] << "." 
                  << std::endl;
        std::cout << "Use " 
                  << CDMW_START_DAEMON_SCRIPT 
                  << " instead." 
                  << std::endl;
        ::exit(EXIT_FAILURE);
    }
 
    if (argc > 2)
    {
        // checks the port option
        port_option =
            OS::get_option_value(argc, 
                                 argv, 
                                 Options::LOCALISATION_SERVICE_OPTION);

        if (port_option != "no")
        {
            if (port_option != "yes")
            {
                specific_port = ::atoi(port_option.c_str());

                if (specific_port <= 0)
                {
                    // initialise log manager
                    DaemonLogMngr::initialise(argc, argv);
    
                    LogMngr::logMessage(FTL, 
                                        MSG_ID_DAEMON_INVALID_PORT, 
                                        port_option);
                    ::exit(EXIT_FAILURE);
                }
            }
            else
            {
                // initialise log manager
                DaemonLogMngr::initialise(argc, argv);
    
                LogMngr::logMessage(FTL, MSG_ID_DAEMON_PORT_NOT_SPECIFIED);
                ::exit(EXIT_FAILURE);
            }

        }

        // checks the orb threads option
        std::string orb_threads_option =
            OS::get_option_value(argc, 
                                 argv, 
                                 Options::ORB_THREADS);

        if (orb_threads_option != "no")
        {
            if (orb_threads_option != "yes")
            {
                orb_thread_nb = ::atoi(orb_threads_option.c_str());

                if (orb_thread_nb <= 0)
                {
                    // initialise log manager
                    DaemonLogMngr::initialise(argc, argv);

                    LogMngr::logMessage(FTL, 
                                        MSG_ID_DAEMON_INVALID_OPTIONS, 
                                        port_option);
                    ::exit(EXIT_FAILURE);
                }
            }
        }

        // checks the xml configuration file option
        std::string xml_conf_option =
            OS::get_option_value( argc, 
                                  argv, 
                                  Options::PROCESS_XML_FILE_OPTION);

        if (xml_conf_option != "no")
        {
            // if the port option is used, xml option is not allowed
            if (port_option != "no")
            {
                // initialise log manager
                DaemonLogMngr::initialise(argc, argv);

                LogMngr::logMessage(FTL, MSG_ID_DAEMON_INVALID_OPTIONS);
                print_syntax(argv[0]);

                ::exit(EXIT_FAILURE);
            }
            
            if (xml_conf_option != "yes")
            {
                if (OS::file_exists(xml_conf_option))
                {
                    xml_conf = xml_conf_option;         

                    // checks the optional validate option
                    std::string validateXML_option =
                        OS::get_option_value( 
                              argc, 
                              argv, 
                              Cdmw::Common::Options::VALIDATE_XML_OPTION);

                    if (validateXML_option != "no")
                    {
                        validate = true;
                    }

                }
                else
                {    
                    // initialise log manager
                    DaemonLogMngr::initialise(argc, argv);

                    LogMngr::logMessage(FTL, 
                                        MSG_ID_DAEMON_CONF_FILE_NOT_FOUND,
                                        xml_conf_option);
                    ::exit(EXIT_FAILURE);
                }
                
            }
            else
            {
                // initialise log manager
                DaemonLogMngr::initialise(argc, argv);

                LogMngr::logMessage(FTL, 
                                    MSG_ID_DAEMON_CONF_FILE_NOT_SPECIFIED);
                ::exit(EXIT_FAILURE);
            }
        }

        // checks the FaultManager corbaloc
        std::string FaultManCorbaloc_option =
              OS::get_option_value( argc, argv, FAULT_MANAGER_CORBALOC_OPTION);
        
        if (FaultManCorbaloc_option!= "no")
        {
            if (FaultManCorbaloc_option != "yes")
            {
                fault_manager_corbaloc = FaultManCorbaloc_option;           
            }
//             else
//             {
//                 LogMngr::logMessage(FTL, 
//                                     MSG_ID_DAEMON_FAULT_MANAGER_NOT_SPECIFIED);
//                 ::exit(EXIT_FAILURE);
//             }
//         }
//         else
//         {
//             LogMngr::logMessage(FTL, 
//                                 MSG_ID_DAEMON_FAULT_MANAGER_NOT_SPECIFIED);
//             ::exit(EXIT_FAILURE);
        }
    }


    // The default configuration
    ConfigurationMngr conf(specific_port);

    CORBA::ORB_var orb = CORBA::ORB::_nil();
    try
    {

 
        // If an XML configuration file has been transmit, we use it
        if (!xml_conf.empty())
        {
            conf.parse(xml_conf, validate);
        }


#ifdef CDMW_POSIX         


        if (conf.getUID() > 0 && conf.getGID() > 0)
        {

            uid_t uid = conf.getUID();
            gid_t gid = conf.getGID();

            OS::set_process_attributes(uid, gid);

        }

#endif
        if (conf.getPolicyLowLevelDaemonExit() != "")
        {
            AUTOMATICACTION = conf.getPolicyLowLevelDaemonExit();
        }
        
        
        // Use a copy of argv, possibly adding 2 options
        char** argv__ = new char*[argc + 3];
        for (int i = 0; i < argc; ++i) 
        {
            argv__[i] = argv[i];
        }
        
        // set optionnal args to 0
        argv__[argc] = 0;
        argv__[argc + 1] = 0;
        argv__[argc + 2] = 0;
        
        int argc__ = argc;
        int argc_x = argc;
        
        // if port option undefined in main arguments, add it
        if (port_option == "no") 
        {
            std::string service_port_option 
               = Options::LOCALISATION_SERVICE_OPTION;
            service_port_option += "=";
            service_port_option += conf.getSystemPortString();
            argv__[argc__] = new char[service_port_option.size() + 1];
            ::strcpy(argv__[argc__], service_port_option.c_str());
            argc__++;
        }

        // If a log file has been specified we create a file to log data
        if (conf.getLogFile() != NULL )
        {
            std::string log_option = Cdmw::Common::Options::LOG_FILE_OPTION;
            log_option += "=";
            log_option += conf.getLogFile();
            argv__[argc__] = new char[log_option.size() + 1];
            ::strcpy(argv__[argc__], log_option.c_str());
            argc__++;
        }

        // initialise log manager
        DaemonLogMngr::initialise(argc__, argv__);

#ifdef CDMW_USE_FAULTTOLERANCE  
              
        // Initialise FT service (client side only)
        Cdmw::FT::FTConfiguration::Activate_FT_service(argc__, argv__);

        // Pre_Initialise FT UltimateFallback service
        Cdmw::FT::UltimateFallback::UltimateFallbackInit::Pre_init(
              argc__, argv__);
#endif

        // Create a server interceptor and register it within an orb initializer
        ServerOrbInitializer* initializer_impl= new ServerOrbInitializer();
        PortableInterceptor::ORBInitializer_var initializer = initializer_impl;
        PortableInterceptor::register_orb_initializer(initializer.in());        
        
        // Allow specification of a value for the ORB connection timeout from
        // command-line arguments or a property file
        std::string dummy_prop = 
           SystemProperties::Get_Property(Cdmw::OrbSupport::CDMW_ORB_CONNECTION_TIMEOUT_PROPERTY_NAME);
        if (dummy_prop.empty())
           SystemProperties::Set_Property(Cdmw::OrbSupport::CDMW_ORB_CONNECTION_TIMEOUT_PROPERTY_NAME,
                                          CONNECTION_TIMEOUT_PROPERTY_VALUE);
        // initialises the ORB
        Cdmw::OrbSupport::StrategyList strategyList;
        strategyList.add_OrbThreaded();
        //strategyList.add_PoaThreadPerRequest();
        strategyList.add_PoaThreadPool(orb_thread_nb);
        strategyList.add_multicast();
        
        // Hack to TAO's dispatch proble
        std::string ORBClientConnectionHandlerPName(Cdmw::OrbSupport::CDMW_ORB_PROPERTY_NAME_PREFIX);
        ORBClientConnectionHandlerPName += "." + std::string(Cdmw::OrbSupport::TAO_ORB_CLIENT_CONNECTION_HANDLER_PROPERTY_NAME);        
        SystemProperties::Set_Property(ORBClientConnectionHandlerPName,"MT");
        
        // initialise the orb        
        orb = Cdmw::OrbSupport::OrbSupport::ORB_init (argc__, argv__, strategyList);
        
        if(conf.isControlledMode()){
            int time_to_wait = conf.getTimeToWaitController();
            int time_wait = 0;
            bool daemon_controller_found = false;
            bool time_elapsed = false;
            std::ostringstream dc_corbaloc;
            dc_corbaloc << "corbaloc::" << Cdmw::OsSupport::OS::get_hostname() << ":" << conf.getControllerPort() << "/daemon_controller";
            LogMngr::logMessage(INF, MSG_ID_CONTROLLER_CORBALOC, dc_corbaloc.str());
            CdmwPlatformMngt::DaemonController_var  m_dm;        
            while(!daemon_controller_found && !time_elapsed){
                try{
                    CORBA::Object_var obj = orb->string_to_object(dc_corbaloc.str().c_str());
                    m_dm = CdmwPlatformMngt::DaemonController::_narrow (obj.in());    
                    if (!CORBA::is_nil(m_dm.in()))
                    { 
                        while(m_dm->is_isolated() == 0) Cdmw::OsSupport::OS::sleep(100);
                        daemon_controller_found = true;
                    }
                    else 
                    {
                        LogMngr::logMessage(INF, MSG_ID_ERROR_NULL_POITER_FROM_CORBALOC);
                        Cdmw::OsSupport::OS::sleep(100);
                        if (time_wait>=time_to_wait) time_elapsed= true;
                        time_wait += 100;
                    }    
                }
                catch (const CORBA::SystemException& e) 
                {
                    LogMngr::logMessage(INF, MSG_ID_ERROR_IN_CONTROLLER_SEARCH);
                    Cdmw::OsSupport::OS::sleep(100);
                    if (time_wait>=time_to_wait) time_elapsed= true;
                    time_wait += 100;
                }
                catch (...) 
                {
                    LogMngr::logMessage(INF, MSG_ID_ERROR_UNKNOW_IN_CONTROLLER_SEARCH);
                    Cdmw::OsSupport::OS::sleep(100);
                    if (time_wait>=time_to_wait) time_elapsed= true;
                    time_wait += 100;
                }
            } 
            LogMngr::logMessage(INF, MSG_ID_ELAPSED_TIME_WAITING_CONTROLLER, time_wait); 
            if(!daemon_controller_found){
                try {
                   // kill(getpid(), 2);
                   Cdmw::OsSupport::OS::kill_process(getpid(), SIGINT );
                }
                catch(...) {
                }
            } 
            else{
                if (m_dm->is_isolated() == 1){
                    LogMngr::logMessage(INF, MSG_ID_ISOLATION_STATUS, "ISOLATED");
                    m_dm->notify(2,atoi(conf.getSystemPortString())); 
                }
                LogMngr::logMessage(INF, MSG_ID_ISOLATION_STATUS, "NOT ISOLATED");
            } 
        }
    
        CORBA::Object_var poaObj = 
           orb->resolve_initial_references("RootPOA");
        PortableGroup::GOA_var poa = 
           PortableGroup::GOA::_narrow(poaObj.in());
         
        ::Cdmw::OrbSupport::CodecBase::Init(orb.in());
        // set the orb reference to the server interceptor
        initializer_impl->set_orb(orb.in());

#ifdef CDMW_USE_FAULTTOLERANCE

        TimeBase::TimeT duration_time = 0;

        TAO_ORB_Core::set_endpoint_selector_factory (
              "FT_Endpoint_Selector_Factory");
        
        orb->_tao_ft_client_id(ClientIDGenerator::generate_daemon_client_id(
                 OS::get_hostname()).c_str());

        CORBA::Object_var obj;
        obj = orb->resolve_initial_references("ORBPolicyManager");

        CORBA::PolicyManager_var policy_manager 
            = CORBA::PolicyManager::_narrow(obj.in());

        // Create and apply an ORB-wide Routed policy        
        duration_time = conf.getRequestDurationTime();   
        
        if (duration_time == 0)
        {
            CDMW_THROW2(ConfigurationException,
                        "no request duration time defined", 
                        __LINE__);
        }     

        CORBA::Any any;
        any <<= duration_time;
        CORBA::PolicyList policies(1);
        policies.length(1);
        policies[0] =
            orb->create_policy(::FT::REQUEST_DURATION_POLICY, any);
        
        policy_manager->set_policy_overrides(policies, CORBA::ADD_OVERRIDE);
        policies[0]->destroy(); 
        
#else
        size_t duration_time = 0;
#endif

        // Activate the POA manager
        PortableServer::POAManager_var manager = poa->the_POAManager();
        manager->activate();


        //create service registration implementation
        Cdmw::CommonSvcs::ServiceRegistration::ServiceRegistration_impl* 
           service_registration_impl 
          = Cdmw::CommonSvcs::ServiceRegistration::ServiceRegistration_impl::Init(
                                           orb.in(),
                                           poa.in(),
                                           conf.getLocationServiceAddress(),
                                           conf.getRegistrationServiceAddress(),
                                           conf.getLocationServiceRetries(),
                                           conf.getLocationServiceTimeInterval());

        
#ifdef CDMW_USE_FAULTTOLERANCE
        // Initialise FT UltimateFallback service
        Cdmw::FT::UltimateFallback::UltimateFallbackInit::Init(orb.in());

        // set orb arguments for the monitoring library
         if (conf.getPullCorbaloc() == "no_corbaloc")
        {
            CDMW_THROW2(ConfigurationException,
                        "no simple pull corbaloc defined", 
                        __LINE__);
        }
         if (conf.getPushCorbaloc() == "no_corbaloc")
        {
            CDMW_THROW2(ConfigurationException,
                        "no simple pull corbaloc defined", 
                        __LINE__);
        }
        
        std::string str_opt_mt("");
        str_opt_mt += "--SIMPLE_PULL_CORBALOC ";
        str_opt_mt += conf.getPullCorbaloc();        
        str_opt_mt += " --SIMPLE_PUSH_CORBALOC ";
        str_opt_mt += conf.getPushCorbaloc();        

        char** argv_mt = Cdmw::Common::String::to_char_array
        (Cdmw::Common::String::to_strings(str_opt_mt));
        int argc_mt(4);


        // create monitoring library
        CdmwFT::MonitoringLibrary::MonitoringLibraryInterface_var 
           monitoringLibraryInterface
            = Cdmw::FT::InitUtils::Create_monitoring_library(
                  argc_mt, argv_mt, orb.in());  
    
        // create acknowledgingSimpleMonitoring
        CdmwFT::MonitoringLibrary::SimpleMonitoring::AcknowledgingSimpleMonitoring_var acknowledgingSimpleMonitoring 
            = monitoringLibraryInterface->create_acknowledging_simple_monitoring();
        // activate the monitoring 
        acknowledgingSimpleMonitoring->start_monitoring();

        // create SystemMngt proxy

        CORBA::Object_var objref;
             
        CdmwPlatformMngt::System_var the_system_proxy = 
           CdmwPlatformMngt::System::_nil();
           
        try
        {
            auto_ptr<Cdmw::PlatformMngt::Proxy::AccessorLookupService> 
               lookup_service
              (new Cdmw::PlatformMngt::Proxy::AccessorLookupService_impl(service_registration_impl)); 
        
            the_system_proxy_handle= 
              Cdmw::PlatformMngt::Proxy::InitUtils::create_platformmngt_proxy
                (orb.in(),
                 poa.in(),
                 CDMW_SYSTEM_MNGT_PROXY,
                 lookup_service.get()); /* proxy takes over responsibility
                                          for lookup service object */ 

            lookup_service.release(); 
 
            the_system_proxy = the_system_proxy_handle->getReference();

        }
        catch (const Cdmw::BadParameterException &ex) 
        {
            ret_code = EXIT_FAILURE;

            LogMngr::logMessage(FTL, 
                                MSG_ID_DAEMON_CANNOT_CREATE_SYSTEM_MNGT_PROXY,
                                ex.get_info());

            return ret_code;
        }
        catch (const Cdmw::OutOfMemoryException &ex) 
        {
            ret_code = EXIT_FAILURE;

            LogMngr::logMessage(FTL, 
                                MSG_ID_DAEMON_CANNOT_CREATE_SYSTEM_MNGT_PROXY,
                                ex.get_info());

            return ret_code;
        }

        // publishes the supervision system proxy corbaloc
        Cdmw::OrbSupport::OrbSupport::bind_object_to_corbaloc (
                     orb.in(), 
                     CdmwPlatformMngt::SUPERVISION_NAME, 
                     the_system_proxy.in());

        // register event factories (value type factory) needed by the proxy
        CORBA::ValueFactoryBase_var factory, old_factory;

        factory = new GraphElementFactory;
        old_factory = orb->register_value_factory (
            "IDL:thalesgroup.com/CdmwPlatformMngt/GraphElement:1.0",
            factory.in() );
                
        factory = new ProcessDefFactory;
        old_factory = orb->register_value_factory (
            "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDef:1.0",
            factory.in() );
                
        factory = new ManagedProcessDefFactory;
        old_factory = orb->register_value_factory (
            "IDL:thalesgroup.com/CdmwPlatformMngt/ManagedProcessDef:1.0",
            factory.in() );

#endif        
        
        // create ProcessesLauncher
        ProcessLauncher* processLauncher = 
            new ProcessLauncher(conf.getProcessesIdDirectory());

        // creates the daemon servant
        PortableServer::ServantBase_var daemon_servant = 
           new Daemon_impl(orb.in(),
                           conf.getSystemPortString(), 
                           conf.getNotificationCallTimeout(), 
                           conf.getPullInterval(), 
                           conf.getPullTimeout(),
                           processLauncher, 
                           service_registration_impl,
                           duration_time, 
                           fault_manager_corbaloc);
                           
        // create the ProcessKillMngr singleton
        ProcessKillMngr::Init (argc, argv);
        ProcessKillMngr_init = true;
        
        
    
    
    
        // creates the daemon CORBA object and activates it
        CdmwPlatformMngt::Daemon_var daemon =
            (dynamic_cast<Daemon_impl*>(daemon_servant.in()))->activate();

        // publishes the daemon corbaloc
        Cdmw::OrbSupport::OrbSupport::bind_object_to_corbaloc
            (orb.in(), CdmwPlatformMngt::DAEMON_NAME, daemon.in() );
    
        // writes the daemon IOR in the IOR file
        {
            std::ofstream iorFile(conf.getIorFile());
            std::string ior = orb->object_to_string(daemon.in());
            iorFile << ior;
        }

        // logs the startup info
        LogMngr::logMessage(INF, MSG_ID_DAEMON_STARTUP,
            conf.getSystemPortString(), conf.getNotificationCallTimeout());

        LogMngr::logMessage(INF, MSG_ID_DAEMON_CORBALOC, OS::get_hostname(),
                conf.getSystemPortString(), CdmwPlatformMngt::DAEMON_NAME);
        
#ifdef CDMW_USE_FAULTTOLERANCE
        LogMngr::logMessage(INF, MSG_ID_DAEMON_CORBALOC, OS::get_hostname(),
                conf.getSystemPortString(), CdmwPlatformMngt::SUPERVISION_NAME);
#endif
        LogMngr::logMessage(INF, MSG_ID_DAEMON_IOR_FILE, conf.getIorFile());

        LogMngr::logMessage(INF, MSG_ID_DAEMON_PID_DIR_CREATED,
            conf.getProcessesIdDirectory());
        

        // Setup signal handling for ORB to be shut down
        ExitHandler * exitHandler = new ExitHandler(orb.in());
        Cdmw::OrbSupport::OrbSupport::install_orb_shutdown_signal_handlers(orb.in(), exitHandler);

        // Plugin initialization
        std::string plugin_conf_filename = OS::get_option_value( argc, argv, Options::PLUGIN_CONFIGURATION );
        PluginManager* plugin_manager = 0;
        if (plugin_conf_filename != "no")
        {
            if (plugin_conf_filename != "yes")
            {
               plugin_manager = new PluginManager( ::CdmwPlatformMngt::System::_nil(), conf.getSystemPortString() );  
                if (!(plugin_manager->set_up_all( plugin_conf_filename ))) {
                    std::vector<std::string> errors = plugin_manager->get_error_descriptions();
                    if (errors.size() > 0) {
                        std::vector<std::string>::iterator i;
                        for (i = errors.begin(); i != errors.end() ; i++) {
                            LogMngr::logMessage (WRN, (*i).c_str());
                        }
                    }
                    else {
                        LogMngr::logMessage (WRN, "There was a problem during plugin loading");
                    }
                }
                else {
                    LogMngr::logMessage (INF, "Plugin successfully loaded");
                }
            }
            else
            {
                LogMngr::logMessage (WRN, "Plugin option is set but no configuration file is specified!");
            }
        }
        else {
            LogMngr::logMessage (INF, "No plugin was specified");
        }

        if (plugin_manager!=0) {
          if ( !(plugin_manager->run_all()) ) {
            LogMngr::logMessage (WRN, "Failed to run loaded plugins!");
          }    
        } 

        // runs the ORB loop
        orb->run();

        if (plugin_manager!=0) {
          if ( !(plugin_manager->tear_down_all()) ) {
            LogMngr::logMessage (WRN, "Failed to tear down loaded plugins!");
          }    
          delete plugin_manager;
          plugin_manager=0;
        }

            
        Cdmw::OrbSupport::OrbSupport::uninstall_orb_shutdown_signal_handlers(orb.in());
        delete exitHandler;




        // delete added arguments if exist
        for (int i=argc_x; i < argc__; ++i)
        {
            delete[] argv__[i];
        }
        delete[] argv__;

    }
    catch (const CannotCreatePidDirectory& ex)
    {
        ret_code = EXIT_FAILURE;

        LogMngr::logMessage(FTL, MSG_ID_DAEMON_CANNOT_CREATE_DIR,
            conf.getProcessesIdDirectory(), ex.get_info());

    }
    catch (const CORBA::COMM_FAILURE& ex)
    {
        ret_code = EXIT_FAILURE;

        LogMngr::logMessage(FTL, MSG_ID_DAEMON_UNAVAILABLE_PORT,
            conf.getSystemPortString());

    }
    catch (const ConfigurationException& ex)
    {
        ret_code = EXIT_FAILURE;

        LogMngr::logMessage(FTL, MSG_ID_DAEMON_INVALID_CONF_FILE,
            xml_conf, ex.getLineNumber(), ex.getMessage()); 
    }
    catch (const CORBA::Exception& ex) {
        std::cerr << "!!!! CORBA::Exception: " << ex << std::endl;
        ret_code = EXIT_FAILURE;
    }
    catch (const std::exception& ex) {
        std::cerr << "!!!! std::exception: " << ex.what() << std::endl;
        ret_code = EXIT_FAILURE;
    }    
    catch (...)
    {
        std::cerr << "!!!! unknown exception. " << std::endl;
        ret_code = EXIT_FAILURE;
    }

    if (!CORBA::is_nil(orb.in()))
    {
         try
         {
            // stops the ORB
             Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
             orb->destroy();
         }
         catch(const CORBA::Exception& ex)
         {
             std::cerr << ex << std::endl;
             return EXIT_FAILURE;
         }
     }
 
 
    // destroy log manager
    LogMngr::destroy();
    
    // cleanup the ProcessKillMngr singleton
    if (ProcessKillMngr_init)
        ProcessKillMngr::Cleanup();

    return ret_code;

}

