/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_MANAGED_ELEMENT_IMPL_HPP
#define INCL_PLATFORMMNGT_MANAGED_ELEMENT_IMPL_HPP


#include <Foundation/common/Exception.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include "Foundation/logging/Logger.hpp"
#include <Foundation/osthreads/ReaderRecursiveLockGuard.hpp>
#include <Foundation/osthreads/WriterRecursiveLockGuard.hpp>
#include <Foundation/osthreads/ReaderWriterRecursiveMutex.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtManagedElement.skel.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtPropertyManager.skel.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtElementRepository.stub.hpp>
#include <SystemMngt/platformelementlibrary/PropertySet_impl.hpp>

#include <SystemMngt/platformelementlibrary/DataStoreDefinition.hpp>

#include "platformlibrary/PropertyEventNotifier.hpp"
#include "platformlibrary/SupervisionEventNotifier.hpp"
#include "platformlibrary/StatusEventNotifier.hpp"

#include <string>
#include <memory>

#include <SystemMngt/platformvaluetypes/Event_impl.hpp>
#include <SystemMngt/platformvaluetypes/PropertyChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/ConfigurationChange_impl.hpp>

/**
 * Root namespace for CDMW runtime.
 */
namespace Cdmw
{
/**
 * Namespace for CDMW PlatformMngt
 */
    namespace PlatformMngt
    {

        class RWEntityContainer_impl;

        /**
         *Purpose:
         *<p>
          */
    class ManagedElement_impl : virtual public POA_CdmwPlatformMngt::ManagedElement,
                    virtual public PortableServer::RefCountServantBase
    {

    public:


        /**
         *Purpose:
         *<p> Constructor.
         *
         * @param element_repository the element repository object reference
         *
         */
        ManagedElement_impl (
        CdmwPlatformMngt::RWElementRepository_ptr element_repository, PortableServer::POA_ptr poa)
        throw ();

        /**
         *Purpose:
         *<p> Default destructor.
         */
        virtual ~ManagedElement_impl();


           
             /**
              *Purpose:
              *<p>
              * Implements the
              * IDL:thalesgroup.com/CdmwPlatformMngt/ManagedElement/get_children:1.0
              * operation
              */
             void remove();


             /**
              *Purpose:
              *<p>
              * Implements the
              * IDL:thalesgroup.com/CdmwPlatformMngt/ManagedElement/get_children:1.0
              * operation
              */
            
            
            virtual CdmwPlatformMngt::Children* get_children()
                throw (CORBA::SystemException) = 0; 
            
            
        /**
         *Purpose:
         *<p>
         * Implements the
         * IDL:thalesgroup.com/CdmwPlatformMngt/ManagedElement/name:1.0
         * operation
         */
        char* name()
        throw (CORBA::SystemException);
          
                     
        /**
         *Purpose:
         *<p>
         * Implements the
         * IDL:thalesgroup.com/CdmwPlatformMngt/ManagedElement/element_path:1.0
         * operation
         */
        char* element_path()
        throw (CORBA::SystemException);
        

        
        /**
         *Purpose:
         *<p>
         * Implements the
         * IDL:thalesgroup.com/CdmwPlatformMngt/ManagedElement/get_parent:1.0
         * operation
         */
        CdmwPlatformMngt::ManagedElement_ptr get_parent()
        throw (CORBA::SystemException);

        /**
         *Purpose:
         *<p>
         *  Get the name of element
         *
         * @return the element name (!! do not desallocate !!)
         */
        const char* get_element_name();

        /**
         *Purpose:
         *<p>
         *  Get the path of element
         *
         * @return the element path (!! do not desallocate !!)
         */
        const char* get_element_path();



        /**
         *Purpose:
         *<p>
         * Implements the
         * IDL:omg.org/CosPropertyService/PropertySet/define_property:1.0
         * operation
         */
        void define_property(const char* property_name,
                 const CORBA::Any& property_value)
        throw (CosPropertyService::InvalidPropertyName,
               CosPropertyService::ConflictingProperty,
               CosPropertyService::UnsupportedTypeCode,
               CosPropertyService::UnsupportedProperty,
               CosPropertyService::ReadOnlyProperty,
               CORBA::SystemException);

        /**
         *Purpose:
         *<p>
         * Implements the
         * IDL:omg.org/CosPropertyService/PropertySet/define_properties:1.0
         * operation
         */
        void define_properties(const CosPropertyService::Properties & nproperties)
        throw (CosPropertyService::MultipleExceptions,
               CORBA::SystemException);


        /**
         *Purpose:
         *<p>
         * Implements the
         * IDL:omg.org/CosPropertyService/PropertySet/get_number_of_properties:1.0
         * operation
         */
        CORBA::ULong get_number_of_properties()
        throw (CORBA::SystemException);


        /**
         *Purpose:
         *<p>
         * Implements the
         * IDL:omg.org/CosPropertyService/PropertySet/get_all_property_names:1.0
         * operation
         */
        void get_all_property_names (CORBA::ULong how_many,
                     CosPropertyService::PropertyNames_out property_names,
                     CosPropertyService::PropertyNamesIterator_out rest)
        throw (CORBA::SystemException);


        /**
         *Purpose:
         *<p>
         * Implements the
         * IDL:omg.org/CosPropertyService/PropertySet/get_property_value:1.0
         * operation
         */
        CORBA::Any* get_property_value (const char* property_name)
        throw (CosPropertyService::PropertyNotFound,
               CosPropertyService::InvalidPropertyName,
               CORBA::SystemException);

        /**
         *Purpose:
         *<p>
         * Implements the
         * IDL:omg.org/CosPropertyService/PropertySet//get_properties:1.0
         * operation
         */
        CORBA::Boolean get_properties (
        const CosPropertyService::PropertyNames& property_names,
        CosPropertyService::Properties_out nproperties)
        throw (CORBA::SystemException);

        /**
         *Purpose:
         *<p>
         * Implements the
         * IDL:omg.org/CosPropertyService/PropertySet/get_all_properties:1.0
         * operation
         */
        void get_all_properties (CORBA::ULong how_many,      
                     CosPropertyService::Properties_out nproperties,
                     CosPropertyService::PropertiesIterator_out rest)
        throw (CORBA::SystemException);
      
      
        /**
         *Purpose:
         *<p>
         * Implements the
         * IDL:omg.org/CosPropertyService/PropertySet/delete_property:1.0
         * operation
         */  
        void delete_property(const char* property_name)
        throw (CosPropertyService::PropertyNotFound,
               CosPropertyService::InvalidPropertyName,
               CosPropertyService::FixedProperty,
               CORBA::SystemException);
           
     
        /**
         *Purpose:
         *<p>
     * Delete propriety without notify event
         */  
        void delete_property_without_notify(const char* property_name)
        throw (CosPropertyService::PropertyNotFound,
               CosPropertyService::InvalidPropertyName,
               CosPropertyService::FixedProperty,
               CORBA::SystemException);

        /**
         *Purpose:
         *<p>
         * Implements the
         * IDL:omg.org/CosPropertyService/PropertySet/delete_properties:1.0
         * operation
         */              
        void delete_properties(const CosPropertyService::PropertyNames& property_names)
        throw (CosPropertyService::MultipleExceptions,
               CORBA::SystemException);

        /**
         *Purpose:
         *<p>
         * Implements the
         * IDL:omg.org/CosPropertyService/PropertySet/delete_all_properties:1.0
         * operation
         */
        CORBA::Boolean delete_all_properties()
        throw (CORBA::SystemException);
    

    
        /**
         *Purpose:
         *<p>
         * Implements the
         * IDL:omg.org/CosPropertyService/PropertySet/is_property_defined:1.0
         * operation
         */
        CORBA::Boolean is_property_defined(const char* property_name)
        throw (CosPropertyService::InvalidPropertyName,
               CORBA::SystemException);



        /**
         *Purpose:
         *<p>
         * Implements the
         * IDL:omg.org/CosPropertyService/PropertySet/get_allowed_property_types:1.0
         * operation
         */
        void get_allowed_property_types(CosPropertyService::PropertyTypes_out property_types)
        throw (CORBA::SystemException);
          

        /**
         *Purpose:
         *<p>
         * Implements the
         * IDL:omg.org/CosPropertyService/PropertySet/get_allowed_properties:1.0
         * operation
         */
        void get_allowed_properties(CosPropertyService::PropertyDefs_out property_defs)
        throw (CORBA::SystemException);


        /**
         *Purpose:
         *<p>
         * Implements the
         * IDL:omg.org/CosPropertyService/PropertySet/define_property_with_mode:1.0
         * operation
         */
        void define_property_with_mode(const char* property_name,
                       const CORBA::Any& property_value,
                       CosPropertyService::PropertyModeType property_mode)
        throw (CosPropertyService::InvalidPropertyName,
               CosPropertyService::ConflictingProperty,
               CosPropertyService::UnsupportedTypeCode,
               CosPropertyService::UnsupportedProperty,
               CosPropertyService::UnsupportedMode,
               CosPropertyService::ReadOnlyProperty,
               CORBA::SystemException);
    

        /**
         *Purpose:
         *<p>
         * Implements the
         * IDL:omg.org/CosPropertyService/PropertySet/define_properties_with_mode:1.0
         * operation
         */
        void define_properties_with_modes(const CosPropertyService::PropertyDefs & property_defs)
        throw (CosPropertyService::MultipleExceptions,
               CORBA::SystemException);


        /**
         *Purpose:
         *<p>
         * Implements the
         * IDL:omg.org/CosPropertyService/PropertySet/get_property_mode:1.0
         * operation
         */
        CosPropertyService::PropertyModeType get_property_mode(const char* property_name)
        throw (CosPropertyService::PropertyNotFound,
               CosPropertyService::InvalidPropertyName,
               CORBA::SystemException);


        /**
         *Purpose:
         *<p>
         * Implements the
         * IDL:omg.org/CosPropertyService/PropertySet/get_property_modes:1.0
         * operation
         */
        CORBA::Boolean get_property_modes(const CosPropertyService::PropertyNames& property_names,
                          CosPropertyService::PropertyModes_out property_modes)
        throw (CORBA::SystemException);

        /**
         *Purpose:
         *<p>
         * Implements the
         * IDL:omg.org/CosPropertyService/PropertySet/set_property_mode:1.0
         * operation
         */
        void set_property_mode(const char* property_name,
                   CosPropertyService::PropertyModeType property_mode)
        throw (CosPropertyService::InvalidPropertyName,
               CosPropertyService::PropertyNotFound,
               CosPropertyService::UnsupportedMode,
               CORBA::SystemException);


        /**
         *Purpose:
         *<p>
         * Implements the
         * IDL:omg.org/CosPropertyService/PropertySet/set_property_modes:1.0
         * operation
         */
        void set_property_modes(const CosPropertyService::PropertyModes & property_modes)
        throw (CosPropertyService::MultipleExceptions,
               CORBA::SystemException);
    

        /**
         *Purpose:
         *<p>
         * Implements the
         * IDL:thalesgroup.com/CdmwPlatformMngt/PropertyManager/register_all_properties_observer:1.0
         * operation
         */
        void register_all_properties_observer (
        const char* observer_name,
        CdmwPlatformMngt::SupervisionObserver_ptr observer,
        CORBA::Boolean deeply)
        throw (CORBA::SystemException, CdmwPlatformMngt::ElementAlreadyExists);

        /**
         *Purpose:
         *<p>
         * Implements the
         * IDL:thalesgroup.com/CdmwPlatformMngt/PropertyManager/unregister_all_properties_observer:1.0
         * operation
         */
        void unregister_all_properties_observer (
        const char* observer_name)
        throw (CdmwPlatformMngt::ObserverNotFound,CORBA::SystemException);


        /**
         *Purpose:
         *<p>
         * Implements the
         * IDL:thalesgroup.com/CdmwPlatformMngt/PropertyManager/register_property_observer:1.0
         * operation
         */
        void register_property_observer (
        const char* observer_name,
        CdmwPlatformMngt::SupervisionObserver_ptr observer,
        const CosPropertyService::PropertyNames& property_names)
        throw (CdmwPlatformMngt::PropertyInListNotFound,CdmwPlatformMngt::ElementAlreadyExists,
               CORBA::SystemException);


        /**
         *Purpose:
         *<p>
         * Implements the
         * IDL:thalesgroup.com/CdmwPlatformMngt/PropertyManager/unregister_property_observer:1.0
         * operation
         */
        void unregister_property_observer (
        const char* observer_name,
        const CosPropertyService::PropertyNames& property_names)
        throw (CdmwPlatformMngt::PropertyInListNotFound,
               CdmwPlatformMngt::ObserverNotFound, 
               CORBA::SystemException);


        /**
         * Purpose :
         * <p>
         * Implements the recovering of userdefined attributes default values
         *
         */
        void recover_default_properties();


        /*
         * Purpose :
         * <p>
         * Implements the operation to store userdefined attributes in a class variable to allow correct restore of default values
         * after restart of Supervise Object
         */
        void store_default_properties();

        /*
         * Purpose :
         * <p>
         * Implements the operation to store userdefined attributes in a class variable to allow correct restore of default values
         * after restart of Supervise Object
         */
        void define_status_property (const char* init_value) 
        throw (CosPropertyService::MultipleExceptions, CORBA::SystemException);

        /**
         * Purpose:
         *<p>
         *Implements the setting of an attribute by CDMW, bypassing the check on read-only mode done on users set_attribute()
         *
         */
        void set_attribute_internal (const char* attr_name, const CORBA::Any& attr, CdmwPlatformMngt::Owner flag)  
        throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound, Cdmw::BadOrderException, CdmwPlatformMngt::ReadOnlyAttribute);

        /**
         * Purpose:
         *<p>
         *Implements the operation to get the status info of managed element.
         *
         */
        char * get_status_info();
        
        /**
         * Purpose:
         *<p>
         *Implements the operation to set the status info of managed element.
         *
         */
        void set_status_info (const char *);        

    protected:

        /**
         * The name of the element.
         */
        CdmwPlatformMngt::ElementName_var m_element_name;

        /**
         * The full name of the element.
         */
        CdmwPlatformMngt::ElementPath_var m_element_path;

        /**
         * The repository of the managed elements.
         */
        CdmwPlatformMngt::RWElementRepository_var m_element_repository; 
        
        /**
         * The information about the status of managed element.
         */
        CORBA::String_var m_status_info;  

    private:
        /*
         * The PropertySet used for representing User Defined Attributes
         */

        Cdmw::PlatformMngt::PropertySet_impl* m_property_set;

        CosPropertyService::PropertySetDef_var  m_property_set_var;

        /*
         * The PropertyDefs var used as the container of default properties
         */

        CosPropertyService::PropertyDefs* m_user_defined_attributes;


        /*
         * The PropertyDefs var used as the container of default properties
         */
        CosPropertyService::PropertyDefs* m_default_user_defined_attributes;
 
        /*
         * The PropertyEventRecord_var  used to store infos about property
         * observer registration
         */
        CdmwPlatformMngt::PropertyEventRecords_var m_property_event_infos;

        /*
         * The EventRecordExt_var  used to store infos about 
         * ObserverExt(for Status and Configuration Event Change) registration
         */
        CdmwPlatformMngt::EventRecordExts_var m_event_ext_infos;
 
        /*
         * Poa instance needed for DeactivableServant class
         */
        PortableServer::POA_var m_poa;

        /*
         *  update the PropertyEventRecord entry from m_property_event_infos 
         */
        void update_properties_record(const CdmwPlatformMngt::PropertyEventRecord&  record_to_update);

        /*
         *  update the EventRecordExt entry from m_event_infos_ext
         */
        void update_record_ext(const CdmwPlatformMngt::EventRecordExt&  record_to_update);

        /*
         *  remove the PropertyEventRecord entry from m_property_event_infos
         */
        void remove_properties_record(const CdmwPlatformMngt::PropertyEventRecord&  record_to_remove);

        /*
         *  update default Supervised Object Record
         */
//        virtual void update_default_record();                     
                
    protected:

        /**
         *Purpose:
         *<p> Copy constructor is not allowed.
         */
        ManagedElement_impl (ManagedElement_impl& rhs);

        /**
         * Purpose:
         * <p> Assignment operator is not allowed.
         */
        ManagedElement_impl& operator = (const ManagedElement_impl& rhs);


        /**
         * Purpose:
         * <p> Set the managed element full pathname
         *
         * @param element_path         the full path name of element
         * @param element_name         the name of element
         *
         */
        void set_element_path (const char* element_path,
                   const char* element_name);

        /**
         *Purpose:
         *<p> Set the managed element record in data store
         *
         */
        void set_element_in_datastore();


        /**
         *Purpose:
         *<p> Get the managed element record from data store
         *
         */
        CdmwPlatformMngt::ManagedElementRecord* get_managed_element_record_from_datastore()
        throw (Common::NotFoundException);


        /**
         *Purpose:
         *<p> Synchronise with datastore
         *
         */
        void synchro_with_datastore();
        
public:
    /**
        *Purpose:
        *<p>
        * Implements the
        * IDL:thalesgroup.com/CdmwPlatformMngt/Process/restart_warmth:1.0
        * operation
        */    
        virtual CdmwPlatformMngt::RestartWarmth requested_restart_warmth()
            throw(CORBA::SystemException);
        
        //Non-IDL method
        void set_requested_restart_warmth (CdmwPlatformMngt::RestartWarmth requested_restart_warmth);


        void set_default_restart_warmth ( 
     CdmwPlatformMngt::RestartWarmth default_restart_warmth )
     throw (CORBA::SystemException, CdmwPlatformMngt::IncompatibleStatus);
     
        CdmwPlatformMngt::RestartWarmth get_default_restart_warmth ()
         throw(CORBA::SystemException);



    private:

        /*
         * The datastore where m_element_name is stored with a m_element_path oid
         */
        ManagedElementDataStore* m_managed_element_ds;


        /**
         *Purpose:
         *<p> Remove the host record in data store
         *
         */
        void remove_element_in_datastore();

        /**
         * The mutex protecting concurrent access to operations on properties
         */
        OsSupport::ReaderWriterRecursiveMutex m_rw_property_mutex;

        /*
         * Property Change Factory
         */
        std::auto_ptr <PropertyChangeFactory > m_propertyChangeFactory;

        /*
         * Property Change Factory
         */
        std::auto_ptr <ConfigurationChangeFactory > m_configurationChangeFactory;

        
       /**
        * The entityDef to restore entity default values
        */
        CdmwPlatformMngt::EntityDefs* m_default_entitydefs;

        Cdmw::Logging::Logger* m_logger; 
     // Property Notifier Facilities
     protected:
        /*
         * The pointer to Property Event Notifier thread
         */
         PropertyEventNotifier* m_property_event_notifier;
         
        /*
         * The pointer to Status Event Notifier thread
         */
         StatusEventNotifier* m_status_event_notifier;

        /*
         * The pointer to Supervision Event Notifier thread
         */
//         SupervisionEventNotifier* m_sup_event_notifier;

        /*
         * Indicator flag for ownership of the thread
         */
         bool m_bownerPropertyEventNotifier;

         /*
         * Indicator flag for ownership of the thread
         */
         bool m_bownerStatusEventNotifier;

         /*
         * Indicator flag for ownership of the thread
         */
         bool m_bownerSupEventNotifier;


     // virtual access to notifier to allowing at the derived class if want
     // use parent or owner notifier
     public:
        /**
         *Purpose:
         *<p> Return The pointer of the  Property Event Notifier thread.
         */
         virtual PropertyEventNotifier* get_property_event_notifier(){return m_property_event_notifier;}
         /**
         *Purpose:
         *<p> set the pointer to the Property Event notifier thread ( The engine is initialized by Supervision process )
         *@param PropertyEventNotifier* property_event_notifier
         */
         void set_property_event_notifier(PropertyEventNotifier* property_event_notifier);

         /**
         *Purpose:
         *<p> Return The pointer of the  Status Event Notifier thread.
         */
         virtual StatusEventNotifier* get_status_event_notifier(){return m_status_event_notifier;}

        /**
         *Purpose:
         *<p> Return The pointer of the  Supervision Event Notifier thread.
         */
//         virtual SupervisionEventNotifier* get_sup_event_notifier(){return m_sup_event_notifier;}

         /**
         *Purpose:
         *<p> set the pointer to the Status Event notifier thread ( The engine is initialized by Supervision process )
         *@param StatusEventNotifier* status_Event_notifier
         */
         void set_status_event_notifier(StatusEventNotifier* status_event_notifier);


        /**
         *Purpose:
         *<p> Notify to PropertyEventNotifier that children have been added/removed
         *@param char* additional info
         *@param CdmwPlatformMngt::PropertyInfos* Infos about Properties change
         */
        void notify_property_change(const char* info,const CdmwPlatformMngt::PropertyInfos* p_infos);
        

        
        /**
         *Purpose:
         *<p> Notify to PropertyEventNotifier that children have been added/removed
         *@param char* additional info about the operation on Properties
         */
          
         void notify_configuration_change(const char* info_operation);
         
         /**
         *Purpose: Register Observer to Property notifier , internal methods base on
         *         PropertyChangeAspect class
         *<p>
         */
         void register_observer_to_property_event_notifier(PropertyChangeAspect& prop_aspect,
                                CdmwPlatformMngt::SupervisionObserver_ptr observer)  
        throw( BadParameterException,OutOfMemoryException,                                                                                CdmwPlatformMngt::ElementAlreadyExists);
         /**
         *Purpose: UnRegister Observer to Property notifier , internal methods base on
         *         PropertyChangeAspect class
         *<p>
         */
         void unregister_observer_to_property_event_notifier(PropertyChangeAspect& prop_aspect) 
        throw( BadParameterException, OutOfMemoryException, 
                       CdmwPlatformMngt::PropertyObserverNotFound); 

        /**
         *Purpose: Register Observer to register also to configuration events
         *@param CORBA::boolean deeply information to manage notification to parents
         *<p>
         */

         void   register_observer_ext(const char* observer_name, 
                                      CdmwPlatformMngt::RegistrationDepth depth,
                                      CdmwPlatformMngt::SupervisionObserver_ptr observer)
                                          throw (CORBA::SystemException, 
                                                 CdmwPlatformMngt::ElementAlreadyExists);
        
        /**
         *Purpose: De-Register Observer 
         *<p>
         */
         void unregister_observer_ext(const char* observer_name)
        throw (CORBA::SystemException, CdmwPlatformMngt::ObserverNotFound);
                                                                    
         
         /**
         *Purpose: Register Observer to register also to configuration events
         *@param const char* observer name
         *@param CdmwPlatformMngt::SupervisionObserver_ptr Observer Ptr 
         *<p>
         */
 
         CdmwPlatformMngt::SupervisionObserver_ptr  register_observer(const char* observer_name,
                                                                      CdmwPlatformMngt::SupervisionObserver_ptr observer)
                                                                           throw (CORBA::SystemException);

        /**
         *Purpose: Unregister Observer to deregister also to configuration events
         *@param const char* observer name
         */

         CdmwPlatformMngt::SupervisionObserver_ptr   unregister_observer(const char* observer_name)   
                                                                 throw (CdmwPlatformMngt::ObserverNotFound,
                                                                        CORBA::SystemException);

        /**
         *  TO-DO Check if Proxy Obserrver is needed by ManagedElement
         */


        CdmwPlatformMngt::SupervisionObserver_ptr register_proxy_observer(const char* observer_name,
                                                                          const char* host_name,
                                                                          CdmwPlatformMngt::SupervisionObserver_ptr observer)
                                                                               throw (CORBA::SystemException);

        /**
         *Purpose:
         *<p> Recover the default entities
         */

       virtual  void  recover_default_entities (Cdmw::PlatformMngt::RWEntityContainer_impl* rwEntityContainer, CdmwPlatformMngt::EntityDefs* default_entitydefs );

    };
    } // End namespace PlatformMngt
} // End namespace Cdmw

#endif //INCL_PLATFORMMNGT_MANAGED_ELEMENT_IMPL_HPP


