/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_PROPERTY_SERVICE_BRIDGE_HPP
#define INCL_PLATFORMMNGT_PROPERTY_SERVICE_BRIDGE_HPP


#include <Foundation/common/Exception.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/CosPropertyService.skel.hpp>

namespace Cdmw
{
/**
 * Namespace for CDMW PlatformMngt
 */
    namespace PlatformMngt
    {

    class PropertySetBridge 
    {
        private:
        PropertySetBridge(const PropertySetBridge& );
        PropertySetBridge& operator=(const PropertySetBridge& );

    public:


        /**
         *Purpose:
         *<p> Constructor.
         *
         * @param element_repository the element repository object reference
         *     
         */
        PropertySetBridge() 
        {}            

        /**
         *Purpose:
         *<p> Default destructor.
         */
        virtual ~PropertySetBridge() 
        {}
        
        /**
         *Purpose:
         *<p>
         * Abstract method for the 
         * IDL:omg.org/CosPropertyService/PropertySet/define_property:1.0
         * operation
         */
        virtual  void define_property(const char* property_name,
                      const CORBA::Any& property_value) 
        throw (CosPropertyService::InvalidPropertyName,
               CosPropertyService::ConflictingProperty,
               CosPropertyService::UnsupportedTypeCode,
               CosPropertyService::UnsupportedProperty,
               CosPropertyService::ReadOnlyProperty,
               CORBA::SystemException) = 0;
                 
        /**
         *Purpose:
         *<p>
         * Abstract method for the 
         * IDL:omg.org/CosPropertyService/PropertySet/define_properties:1.0
         * operation
         */ 
        virtual void define_properties(const CosPropertyService::Properties & nproperties)
        throw (CosPropertyService::MultipleExceptions,
               CORBA::SystemException) = 0;
 

        /**
         *Purpose:
         *<p>
         * Abstract method for the 
         * IDL:omg.org/CosPropertyService/PropertySet/get_number_of_properties:1.0
         * operation
         */
        virtual  CORBA::ULong get_number_of_properties()
        throw (CORBA::SystemException) = 0;
      
      
        /**
         *Purpose:
         *<p>
         * Abstract method for the 
         * IDL:omg.org/CosPropertyService/PropertySet/get_all_property_names:1.0
         * operation
         */
        virtual void get_all_property_names (CORBA::ULong how_many,       
                         CosPropertyService::PropertyNames_out property_names,
                         CosPropertyService::PropertyNamesIterator_out rest)
        throw (CORBA::SystemException) = 0;
      
      
        /**
         *Purpose:
         *<p>
         * Abstract method for the 
         * IDL:omg.org/CosPropertyService/PropertySet/get_property_value:1.0
         * operation
         */
        virtual  CORBA::Any* get_property_value (const char* property_name)
        throw (CosPropertyService::PropertyNotFound,
               CosPropertyService::InvalidPropertyName,
               CORBA::SystemException) = 0;

        /**
         *Purpose:
         *<p>
         * Abstract method for the 
         * IDL:omg.org/CosPropertyService/PropertySet//get_properties:1.0
         * operation
         */
        virtual  CORBA::Boolean get_properties (
        const CosPropertyService::PropertyNames& property_names,
        CosPropertyService::Properties_out nproperties)
        throw (CORBA::SystemException) = 0;

        /**
         *Purpose:
         *<p>
         * Abstract method for the 
         * IDL:omg.org/CosPropertyService/PropertySet/get_all_properties:1.0
         * operation
         */
        virtual  void get_all_properties (CORBA::ULong how_many,      
                          CosPropertyService::Properties_out nproperties,
                          CosPropertyService::PropertiesIterator_out rest)
        throw (CORBA::SystemException) = 0;
      
      
        /**
         *Purpose:
         *<p>
         * Abstract method for the 
         * IDL:omg.org/CosPropertyService/PropertySet/delete_property:1.0
         * operation
         */  
        virtual   void delete_property(const char* property_name)
        throw (CosPropertyService::PropertyNotFound,
               CosPropertyService::InvalidPropertyName,
               CosPropertyService::FixedProperty,
               CORBA::SystemException) = 0;
    
        /**
         *Purpose:
         *<p>
         * Abstract method for the 
         * IDL:omg.org/CosPropertyService/PropertySet/delete_properties:1.0
         * operation
         */              
        virtual  void delete_properties(const CosPropertyService::PropertyNames& property_names)
        throw (CosPropertyService::MultipleExceptions,
               CORBA::SystemException) = 0;

        /**
         *Purpose:
         *<p>
         * Abstract method for the 
         * IDL:omg.org/CosPropertyService/PropertySet/delete_all_properties:1.0
         * operation
         */
        virtual  CORBA::Boolean delete_all_properties()
        throw (CORBA::SystemException) = 0;
    
    
    
        /**
         *Purpose:
         *<p>
         * Abstract method for the 
         * IDL:omg.org/CosPropertyService/PropertySet/is_property_defined:1.0
         * operation
         */
        virtual   CORBA::Boolean is_property_defined(const char* property_name)
        throw (CosPropertyService::InvalidPropertyName,
               CORBA::SystemException) = 0;
    
    
    
        /**
         *Purpose:
         *<p>
         * Abstract method for the 
         * IDL:omg.org/CosPropertyService/PropertySet/get_allowed_property_types:1.0
         * operation
         */
        virtual  void get_allowed_property_types(CosPropertyService::PropertyTypes_out property_types)
        throw (CORBA::SystemException) = 0;
          
          
        /**
         *Purpose:
         *<p>
         * Abstract method for the 
         * IDL:omg.org/CosPropertyService/PropertySet/get_allowed_properties:1.0
         * operation
         */
        virtual  void get_allowed_properties(CosPropertyService::PropertyDefs_out property_defs)
        throw (CORBA::SystemException) = 0;


        /**
         *Purpose:
         *<p>
         * Abstract method for the 
         * IDL:omg.org/CosPropertyService/PropertySet/define_property_with_mode:1.0
         * operation
         */
        virtual  void define_property_with_mode(const char* property_name,
                            const CORBA::Any& property_value,
                            CosPropertyService::PropertyModeType property_mode)
        throw (CosPropertyService::InvalidPropertyName,
               CosPropertyService::ConflictingProperty,
               CosPropertyService::UnsupportedTypeCode,
               CosPropertyService::UnsupportedProperty,
               CosPropertyService::UnsupportedMode,
               CosPropertyService::ReadOnlyProperty,
               CORBA::SystemException) = 0;
    

        /**
         *Purpose:
         *<p>
         * Abstract method for the 
         * IDL:omg.org/CosPropertyService/PropertySet/define_properties_with_mode:1.0
         * operation
         */
        virtual  void define_properties_with_modes(const CosPropertyService::PropertyDefs & property_defs)
        throw (CosPropertyService::MultipleExceptions,
               CORBA::SystemException) = 0;


        /**
         *Purpose:
         *<p>
         * Abstract method for the 
         * IDL:omg.org/CosPropertyService/PropertySet/get_property_mode:1.0
         * operation
         */
        virtual  CosPropertyService::PropertyModeType get_property_mode(const char* property_name)
        throw (CosPropertyService::PropertyNotFound,
               CosPropertyService::InvalidPropertyName,
               CORBA::SystemException) = 0;


        /**
         *Purpose:
         *<p>
         * Abstract method for the 
         * IDL:omg.org/CosPropertyService/PropertySet/get_property_modes:1.0
         * operation
         */
        virtual  CORBA::Boolean get_property_modes(const CosPropertyService::PropertyNames& property_names,
                               CosPropertyService::PropertyModes_out property_modes)
        throw (CORBA::SystemException) = 0;

        /**
         *Purpose:
         *<p>
         * Abstract method for the 
         * IDL:omg.org/CosPropertyService/PropertySet/set_property_mode:1.0
         * operation
         */
        virtual void set_property_mode(const char* property_name,
                       CosPropertyService::PropertyModeType property_mode)
        throw (CosPropertyService::InvalidPropertyName,
               CosPropertyService::PropertyNotFound,
               CosPropertyService::UnsupportedMode,
               CORBA::SystemException) = 0;


        /**
         *Purpose:
         *<p>
         * Abstract method for the 
         * IDL:omg.org/CosPropertyService/PropertySet/set_property_modes:1.0
         * operation
         */
        virtual void set_property_modes(const CosPropertyService::PropertyModes & property_modes)
        throw (CosPropertyService::MultipleExceptions,
               CORBA::SystemException) = 0;
        
        virtual PropertySetBridge* clone() = 0 ;
      
    };
    } // End namespace PlatformMngt
} // End namespace Cdmw

#endif //INCL_PLATFORMMNGT_PROPERTY_SERVICE_BRIDGE_HPP

