/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_PROPERTY_SET_HPP
#define INCL_PLATFORMMNGT_PROPERTY_SET_HPP


#include <Foundation/common/Exception.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/CosPropertyService.skel.hpp>

#include <SystemMngt/platformelementlibrary/PropertySetBridge.hpp>
#include <SystemMngt/platformelementlibrary/PropertyFactory.hpp>

/**
 * Root namespace for CDMW runtime.
 */
namespace Cdmw
{
  /**
   * Namespace for CDMW PlatformMngt 
   */
  namespace PlatformMngt
  {
    class PropertySet_impl 
      : virtual public POA_CosPropertyService::PropertySet,
    virtual public POA_CosPropertyService::PropertySetDef,
    virtual public PortableServer::RefCountServantBase
              
    {
      public:
      
      /**
       * Purpose:
       * <p> Constructor.
       *
       * @param poaName the name identifying the POA that will be created.
       */
      PropertySet_impl(PortableServer::POA_ptr poa);
                                                                                                                                    
      /**
       *Purpose:
       *<p> Default destructor.
       */
      virtual ~PropertySet_impl();


      /**
       * Purpose:
       * <p> Activates this servant.
       *
       * @exception AlreadyDoneException if the servant is already activated.
       */ 
      void activate() throw (AlreadyDoneException, InternalErrorException);

      /**
       * Purpose:
       * <p> Activates this servant with the specified object id.
       *
       * @param objectId the object id string of the servant.
       * @exception AlreadyDoneException if the servant is already activated.
       */ 
      void activateWithId(const std::string& objectId)
        throw (AlreadyDoneException, InternalErrorException);
 
      /**
       * Purpose:
       * <p> Deactivates this servant.
       *
       * @exception BadOrderException if the servant was not activated.
       */ 
      void deactivate()
        throw (BadOrderException, InternalErrorException);

      /**
       * Purpose:
       * <p> Redefines the PortableServer::ServantBase method
       * to return the associated POA.
       *
       * @return The m_poa member
       */
      PortableServer::POA_ptr _default_POA();


    private:

      /**
       * The specific POA in charge of this servant.
       */
      PortableServer::POA_var m_poa;


      /**
       * The object id of this servant once it has been activated.
       */
      PortableServer::ObjectId_var m_oid;


    public:


    
      /**
       *Purpose:
       *<p>
       * Implements the
       * IDL:omg.org/CosPropertyService/PropertySet/define_property:1.0
       * operation independently from ORB used
       */
      void  define_property(const char* property_name,const CORBA::Any& property_value)
    throw (CosPropertyService::InvalidPropertyName,
           CosPropertyService::ConflictingProperty,
           CosPropertyService::UnsupportedTypeCode,
           CosPropertyService::UnsupportedProperty,
           CosPropertyService::ReadOnlyProperty,
           CORBA::SystemException);
                 
      /**
       *Purpose:
       *<p>
       * Implements the
       * IDL:omg.org/CosPropertyService/PropertySet/define_properties:1.0
       * operation independently from ORB used
       */ 
      void define_properties(const CosPropertyService::Properties & nproperties)
    throw (CosPropertyService::MultipleExceptions,
           CORBA::SystemException);
 

      /**
       *Purpose:
       *<p>
       * Implements the
       * IDL:omg.org/CosPropertyService/PropertySet/get_number_of_properties:1.0
       * operation independently from ORB used
       */
      CORBA::ULong get_number_of_properties()
    throw (CORBA::SystemException);
      
      
      /**
       *Purpose:
       *<p>
       * Implements the
       * IDL:omg.org/CosPropertyService/PropertySet/get_all_property_names:1.0
       * operation independently from ORB used
       */
      void get_all_property_names (CORBA::ULong how_many,       
                   CosPropertyService::PropertyNames_out property_names,
                   CosPropertyService::PropertyNamesIterator_out rest)
    throw (CORBA::SystemException);
      
      
      /**
       *Purpose:
       *<p>
       * Implements the
       * IDL:omg.org/CosPropertyService/PropertySet/get_property_value:1.0
       * operation independently from ORB used
       */
      CORBA::Any* get_property_value (const char* property_name)
    throw (CosPropertyService::PropertyNotFound,
           CosPropertyService::InvalidPropertyName,
           CORBA::SystemException);

      /**
       *Purpose:
       *<p>
       * Implements the
       * IDL:omg.org/CosPropertyService/PropertySet//get_properties:1.0
       * operation independently from ORB used
       */
      CORBA::Boolean get_properties (
                     const CosPropertyService::PropertyNames& property_names,
                     CosPropertyService::Properties_out nproperties)
    throw (CORBA::SystemException);

      /**
       *Purpose:
       *<p>
       * Implements the
       * IDL:omg.org/CosPropertyService/PropertySet/get_all_properties:1.0
       * operation independently from ORB used
       */
      void get_all_properties (CORBA::ULong how_many,      
                   CosPropertyService::Properties_out nproperties,
                   CosPropertyService::PropertiesIterator_out rest)
    throw (CORBA::SystemException);
      
      
      /**
       *Purpose:
       *<p>
       * Implements the
       * IDL:omg.org/CosPropertyService/PropertySet/delete_property:1.0
       * operation independently from ORB used
       */  
      void delete_property(const char* property_name)
    throw (CosPropertyService::PropertyNotFound,
           CosPropertyService::InvalidPropertyName,
           CosPropertyService::FixedProperty,
           CORBA::SystemException);
    
      /**
       *Purpose:
       *<p>
       * Implements the
       * IDL:omg.org/CosPropertyService/PropertySet/delete_properties:1.0
       * operation independently from ORB used
       */              
      void delete_properties(const CosPropertyService::PropertyNames& property_names)
    throw (CosPropertyService::MultipleExceptions,
           CORBA::SystemException);

      /**
       *Purpose:
       *<p>
       * Implements the
       * IDL:omg.org/CosPropertyService/PropertySet/delete_all_properties:1.0
       * operation independently from ORB used
       */
      CORBA::Boolean delete_all_properties()
    throw (CORBA::SystemException);
    
    
    
      /**
       *Purpose:
       *<p>
       * Implements the
       * IDL:omg.org/CosPropertyService/PropertySet/is_property_defined:1.0
       * operation independently from ORB used
       */
      CORBA::Boolean is_property_defined(const char* property_name)
    throw (CosPropertyService::InvalidPropertyName,
           CORBA::SystemException);
    
    
    
      /**
       *Purpose:
       *<p>
       * Implements the
       * IDL:omg.org/CosPropertyService/PropertySet/get_allowed_property_types:1.0
       * operation independently from ORB used
       */
      void get_allowed_property_types(CosPropertyService::PropertyTypes_out property_types)
    throw (CORBA::SystemException);
          
          
      /**
       *Purpose:
       *<p>
       * Implements the
       * IDL:omg.org/CosPropertyService/PropertySet/get_allowed_properties:1.0
       * operation independently from ORB used
       */
      void get_allowed_properties(CosPropertyService::PropertyDefs_out property_defs)
    throw (CORBA::SystemException);


      /**
       *Purpose:
       *<p>
       * Implements the
       * IDL:omg.org/CosPropertyService/PropertySet/define_property_with_mode:1.0
       * operation independently from ORB used
       */
      void define_property_with_mode(const char* property_name,
                     const CORBA::Any& property_value,
                     CosPropertyService::PropertyModeType property_mode)
    throw (CosPropertyService::InvalidPropertyName,
           CosPropertyService::ConflictingProperty,
           CosPropertyService::UnsupportedTypeCode,
           CosPropertyService::UnsupportedProperty,
           CosPropertyService::UnsupportedMode,
           CosPropertyService::ReadOnlyProperty,
           CORBA::SystemException);
    

      /**
       *Purpose:
       *<p>
       * Implements the
       * IDL:omg.org/CosPropertyService/PropertySet/define_properties_with_mode:1.0
       * operation independently from ORB used
       */
      void define_properties_with_modes(const CosPropertyService::PropertyDefs & property_defs)
    throw (CosPropertyService::MultipleExceptions,
           CORBA::SystemException);


      /**
       *Purpose:
       *<p>
       * Implements the
       * IDL:omg.org/CosPropertyService/PropertySet/get_property_mode:1.0
       * operation independently from ORB used
       */
      CosPropertyService::PropertyModeType get_property_mode(const char* property_name)
    throw (CosPropertyService::PropertyNotFound,
           CosPropertyService::InvalidPropertyName,
           CORBA::SystemException);


      /**
       *Purpose:
       *<p>
       * Implements the
       * IDL:omg.org/CosPropertyService/PropertySet/get_property_modes:1.0
       * operation independently from ORB used
       */
      CORBA::Boolean get_property_modes(const CosPropertyService::PropertyNames& property_names,
                    CosPropertyService::PropertyModes_out property_modes)
    throw (CORBA::SystemException);

      /**
       *Purpose:
       *<p>
       * Implements the
       * IDL:omg.org/CosPropertyService/PropertySet/set_property_mode:1.0
       * operation independently from ORB used
       */
      void set_property_mode(const char* property_name,
                 CosPropertyService::PropertyModeType property_mode)
    throw (CosPropertyService::InvalidPropertyName,
           CosPropertyService::PropertyNotFound,
           CosPropertyService::UnsupportedMode,
           CORBA::SystemException);


      /**
       *Purpose:
       *<p>
       * Implements the
       * IDL:omg.org/CosPropertyService/PropertySet/set_property_modes:1.0
       * operation independently from ORB used
       */
      void set_property_modes(const CosPropertyService::PropertyModes & property_modes)
    throw (CosPropertyService::MultipleExceptions,
           CORBA::SystemException);



      PropertySetBridge*  get_impl();

    private:    
        
      PropertySetBridge* m_property_set_bridge;

    };
  } // End namespace PlatformMngt
} // End namespace Cdmw

#endif //INCL_PLATFORMMNGT_PROPERTY_SET_HPP

