/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "tao/AnyTypeCode/TypeCode.h"
#include "SystemMngt/platformelementlibrary/Entity_impl.hpp"
#include "SystemMngt/platformlibrary/Configuration.hpp"
#include "SystemMngt/tools/ManagedElementKey.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "Foundation/commonsvcs/datastore/StorageErrorException.hpp"
#include "SystemMngt/platformlibrary/SupervisionEventNotifier.hpp"
#include "SystemMngt/platformlibrary/PropertyEventNotifier.hpp"
#include "SystemMngt/platformlibrary/EventNotifierManager.hpp"

#include "SystemMngt/platformlibrary/EventHeaderFactory.hpp"

#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"
#include "idllib/CdmwPlatformMngtHost.stub.hpp"
#include "idllib/CdmwPlatformMngtProcess.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtManagedElement.skel.hpp"

#include "SystemMngt/platformlibrary/SupervisionEventNotifier.hpp"
#include "SystemMngt/platformlibrary/PropertyEventNotifier.hpp"
#include "SystemMngt/platformlibrary/EventNotifierManager.hpp"

#include "SystemMngt/platformlibrary/EventHeaderFactory.hpp"

#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"
#include "idllib/CdmwPlatformMngtHost.stub.hpp"
#include "idllib/CdmwPlatformMngtProcess.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtManagedElement.skel.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"


namespace Cdmw
{
    namespace PlatformMngt
    {
        
        
// ----------------------------------------------------------------------
// Entity_impl class.
// ----------------------------------------------------------------------
        
        
        Entity_impl::Entity_impl (PortableServer::POA_ptr poa,
            const CdmwPlatformMngt::EntityDef& entity_def,
            const char* entity_path,
            CdmwPlatformMngt::RWElementRepository_ptr element_repository)
            throw (BadParameterException,
            OutOfMemoryException,CosPropertyService::MultipleExceptions)
            : DeactivableServant_impl (poa),
          ManagedElement_impl(element_repository, poa)                             
        {
        m_logger = Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG);
        m_ent_status_change_factory    = 0;

        //try to printing the entity_def->status_values
        int num_seq = (entity_def.status_values).length() ;

        if (num_seq > 0)
       {
       //fill internal sequence of status_values ...
       //add the default status 
       std::string UNKNOWN_status="UNKNOWN";
       m_status_values = entity_def.status_values ;
       m_status_values.length(entity_def.status_values.length()+1);
       m_status_values[entity_def.status_values.length()]= CORBA::string_dup(UNKNOWN_status.c_str());
       }


            try
            {  
                try
                {
                    Cdmw::CommonSvcs::DataStore::DataStoreBase* ds =
                        Cdmw::CommonSvcs::DataStore::DataStoreRegistry::Get_data_store(
                        Configuration::Get_entity_datastore_id());
                    m_entity_ds = 
                        dynamic_cast<Cdmw::PlatformMngt::EntityDataStore*>(ds);
                }
                catch (const NotExistException& e)
                {
                    CDMW_ASSERT(false);
                }
                
        // set in Managed Element the instance of Property Event Notifier Thread
        set_property_event_notifier(EventNotifierManager::Instance()->get_property_notifier());
        
        // Allocate memory for Status Event Factory
        m_ent_status_change_factory      =   new EntityStatusChangeFactory();
         
        // store element path in managed element
        set_element_path (entity_path,
                  entity_def.entity_name.in());
                         
        unsigned int propCount = entity_def.properties.length();
        for (unsigned int propIndex=0 ; propIndex < propCount ; propIndex++)
        {
            const CdmwPlatformMngt::PropertyDef& prop_def =
            entity_def.properties[propIndex];
            try
            {
            std::string propertyName(entity_def.properties[propIndex].property_name);
            CosPropertyService::PropertyModeType propertyMode = CosPropertyService::PropertyModeType(0);
            CdmwPlatformMngt::PropertyModeType propertyModeCdmw = entity_def.properties[propIndex].mode_type;
            if (propertyModeCdmw == 1) propertyMode = CosPropertyService::PropertyModeType(1);
            define_property_with_mode(propertyName.c_str() , prop_def.initial_value, propertyMode);
            }
            catch (const CosPropertyService::InvalidPropertyName&)
            {
                        throw  CosPropertyService::MultipleExceptions(); /* TO-DO */
            }
            catch (const CosPropertyService::ConflictingProperty&)
            {
                        throw  CosPropertyService::MultipleExceptions(); /* TO-DO */
            }
            catch (const CosPropertyService::UnsupportedTypeCode&)
            {
                        throw  CosPropertyService::MultipleExceptions(); /* TO-DO */
            }
            catch (const CosPropertyService::UnsupportedProperty&)
            {
                        throw  CosPropertyService::MultipleExceptions(); /* TO-DO */
            }
            catch (const CosPropertyService::UnsupportedMode&)
            {
                        throw  CosPropertyService::MultipleExceptions(); /* TO-DO */
            }
            catch (const CosPropertyService::ReadOnlyProperty&)
            {
                        throw  CosPropertyService::MultipleExceptions(); /* TO-DO */
            }
    
        }
        
        try{
            // define TECH_STATUS and TECH_STATUS_INFO properties
            const char* init_value = "";
            define_status_property(init_value);
        }
        catch(...)
        {
            throw  CosPropertyService::MultipleExceptions();
        }
                                                                                                                           store_default_properties();  
        // initialise data store of the object
        create_record_in_datastore(entity_def);
        
        }
        catch (const std::bad_alloc&)
        {
        CDMW_THROW (OutOfMemoryException);
        } 

    }


    Entity_impl::~Entity_impl ()
    {       
    if(m_ent_status_change_factory != NULL)
         delete m_ent_status_change_factory;
    }
        
        void Entity_impl::remove()
        {
            // first increment counter reference to avoid the destructor calling
            // by servant deactivation
            this->_add_ref();
            
            // deactivate the Corba object
            this->deactivate();
            
            // remove associated entity record in datastore
            remove_record_in_datastore();
            
            // remove the associated managed element
            this->ManagedElement_impl::remove();
            
            // then decrement the counter reference to call destructor
            this->_remove_ref();        
    }


    void 
    Entity_impl::create_record_in_datastore(const CdmwPlatformMngt::EntityDef& entity_def)
        throw (OutOfMemoryException)
    {  
        // test if the supervision is not fault tolerant or if it is, is it the primary?
        if (!Configuration::Is_fault_tolerant() || Configuration::Is_primary())
        { 
            // set managed element
            set_element_in_datastore();
        
            CdmwPlatformMngt::EntityRecord record;
            
            record.entity_status = CORBA::string_dup ("UNKNOWN");
            record.entity_info = CORBA::string_dup ("Default entity status");
            //record.status_values.length(0);
    
            //add to datastore the range of status values ...
            record.status_values = m_status_values;
           
            string ds_operation("insert");
            try
            {
            m_entity_ds->insert(m_element_path.in(), record);
            }
            catch (const Common::AlreadyExistException&)
            {
                CDMW_WARN(m_logger,
                "record already exists exception in " 
                << ds_operation << " : " 
                << m_element_path.in () << ":"
                << (const char*) record.entity_status);
            }
            catch (const CommonSvcs::DataStore::StorageErrorException &exc_storage)
            {
                CDMW_FATAL(m_logger,
                "record storage error exception in "
                << ds_operation <<" : " 
                << m_element_path.in() << ":"
                << (const char*) record.entity_status << ":" 
                << exc_storage.what());
                CDMW_ASSERT (false);
            }
            catch (...)
            {
                CDMW_FATAL(m_logger,
                "record unknown exception in "
                << ds_operation << " : " 
                << m_element_path.in() << ":"
                << (const char*) record.entity_status);
                CDMW_ASSERT (false);
            }
        }
    }


    void 
    Entity_impl::remove_record_in_datastore()
    {
        // inhibition if backup
        if (!Configuration::Is_fault_tolerant() || Configuration::Is_primary())
        {
            try
            {
                m_entity_ds->remove(m_element_path.in());
            }
            catch (const Common::NotFoundException&)
            {
            }
            catch (const CommonSvcs::DataStore::StorageErrorException &)
            {
            }
        }
    }

    CdmwPlatformMngt::EntityRecord* 
    Entity_impl::get_record_from_datastore()
    {
        CdmwPlatformMngt::EntityRecord* entity_record = NULL;
    
        try
        {
            entity_record = m_entity_ds->select(m_element_path.in());
        }
        catch (const Common::NotFoundException&)
        {
        CDMW_FATAL(m_logger, "entity record cannot be found in datastore : " 
              << m_element_path.in());
              
        CDMW_ASSERT(false);
        }   
    
        return entity_record;
    }

    

    void 
    Entity_impl::update_record_in_datastore(const CdmwPlatformMngt::EntityRecord& entity_record)
    {
        try
        {
        m_entity_ds->update(m_element_path.in(), entity_record);
        }
        catch (const Common::NotFoundException&)
        {
        CDMW_FATAL(m_logger, "entity record cannot be found in datastore : " 
              << m_element_path.in());
        CDMW_ASSERT(false);
        } 
        catch (const CommonSvcs::DataStore::StorageErrorException&)
        {
        CDMW_FATAL(m_logger, "entity record storage error : " 
              << m_element_path.in());
        CDMW_ASSERT(false);
        } 
    }

    /**
     *Purpose:
     *<p> facade to get an attribute
     *
     */

    void Entity_impl::get_attribute(const char * attr_name, CORBA::Any_out ret_attr  , CdmwPlatformMngt::Owner flag)  
        throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound)
    {

            
    std::string tmp1(attr_name);

    if (CdmwPlatformMngt::PRE_DEFINED == flag )
    { 
        // Case of predefined property
        std::ostringstream property_name;
        property_name << CdmwPlatformMngt::CDMW_PREFIX << attr_name << std::ends;
        try
        {
            CORBA::Any_var outany =
                get_property_value ( property_name.str().c_str() );
            ret_attr = outany._retn();
        }
        catch (const CosPropertyService::PropertyNotFound&)
        {
            throw CdmwPlatformMngt::AttributeNotFound ();

        }
         
    }
    //case of user defined attribute
    else
    { 
        try
        {       
            CORBA::Any* outany = get_property_value( attr_name );
            //CORBA::Any_var tempany = new CORBA::Any;
            ret_attr = outany; 
        }
        catch (const CosPropertyService::PropertyNotFound& )
        {
            throw CdmwPlatformMngt::AttributeNotFound();
        }       
    }
    }
        
    /**
    *Purpose:
    *<p> facade to set an attribute
    *
    */
    
    void Entity_impl::set_attribute(const char * attr_name, const CORBA::Any & attr  , CdmwPlatformMngt::Owner flag)  
        throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound, CdmwPlatformMngt::ReadOnlyAttribute)
    {

    if (CdmwPlatformMngt::PRE_DEFINED == flag )
    {
        std::string tmp1 (attr_name);
        // check if the predefined property is readonly 
        if ((tmp1 == CdmwPlatformMngt::TECH_STATUS) ||
                (tmp1 == CdmwPlatformMngt::TECH_STATUS_INFO))
        {
            throw CdmwPlatformMngt::ReadOnlyAttribute(); 
        }
    }
    try
    {
        set_attribute_internal ( attr_name, attr, flag );
    }
    catch (const Cdmw::BadOrderException& ex)
    {
        throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
    }
    catch (...) 
    {
        throw;
    }

            
    }
        
        
/**
        *Purpose:
        *<p>
        * Implements the
        * IDL:thalesgroup.com/CdmwPlatformMngt/Entity/get_status:1.0
        * operation
        */
        char* 
            Entity_impl::get_status(CORBA::String_out entity_info)
            throw (CORBA::SystemException)
        {
            // lock for concurent access
            CDMW_MUTEX_GUARD(m_mutex_entity_record);
            
            // get record                
            CdmwPlatformMngt::EntityRecord_var entity_record = get_record_from_datastore();
            
            // get entity status
            CORBA::String_var entity_status = 
                CORBA::string_dup ((entity_record->entity_status).in());
            
            // get info associated to status
            entity_info = CORBA::string_dup ((entity_record->entity_info).in());
            
            return entity_status._retn();
        }
        
/**
        *Purpose:
        *<p>
        * Implements the
        * IDL:thalesgroup.com/CdmwPlatformMngt/Entity/set_status:1.0
        * operation
        */
        void 
            Entity_impl::set_status(const char* entity_status, const char* entity_info)
            throw (CdmwPlatformMngt::EntityStatusNotAllowed, 
            CORBA::SystemException)
        {    
            CdmwPlatformMngt::EntityRecord_var entity_record;
            CdmwPlatformMngt::EntityStatus current_status;
            
            {
                // lock for concurent access
                CDMW_MUTEX_GUARD(m_mutex_entity_record);

                //for safety, get from datastore
        //the range of status values ...

                // get record                
                CdmwPlatformMngt::EntityRecord_var entity_record = get_record_from_datastore();

                m_status_values = entity_record->status_values;

                //check is status is in the range of allowed status
                long int num_status_val = m_status_values.length() ;
                long int k;
        std::string comp_str ;
            char flag_found = 0;
                if ( num_status_val > 0)
                 {
                 for (k=0; k<num_status_val; k++)
                    {
                    comp_str = (m_status_values)[k] ;

                    if ( comp_str.compare(entity_status) == 0)
                        flag_found = 1 ;
                    } 

                 if (flag_found == 0)
              {
                      throw( CdmwPlatformMngt::EntityStatusNotAllowed() ) ;
              return;
              }
                 }
                
                
                //Set TECH_STATUS and TECH_STATUS_INFO property
                CORBA::Any_var status_value = new CORBA::Any();
                *status_value <<= CORBA::string_dup(entity_status);
                CORBA::Any_var status_info_value = new CORBA::Any();
                *status_info_value <<= CORBA::string_dup(entity_info);
                try
                {
                    set_attribute_internal(CdmwPlatformMngt::TECH_STATUS,
                            status_value.in(), CdmwPlatformMngt::PRE_DEFINED);
                    set_attribute_internal(CdmwPlatformMngt::TECH_STATUS_INFO,
                            status_info_value.in(), CdmwPlatformMngt::PRE_DEFINED);
                }
                catch(const CdmwPlatformMngt::AttributeNotFound&) {}
                catch (...){}
                
                // get current status
                entity_record = get_record_from_datastore();
                current_status = CORBA::string_dup ((entity_record->entity_status).in());
                
        
                // if status change
                if (strcmp(entity_status, current_status))
                {
                    // store new status
                    entity_record->entity_status = CORBA::string_dup (entity_status);
                    entity_record->entity_info = CORBA::string_dup (entity_info);
                    
                    update_record_in_datastore(entity_record.in());
                    notify_entity_status_change(entity_record->entity_status,entity_record->entity_info);
                }
            }

            
        }
        
        
        
        /**
        *Purpose:
        *<p>
        * Implements the
        * IDL:thalesgroup.com/CdmwPlatformMngt/Entity/get_status_values:1.0
        * operation
        */
        CdmwPlatformMngt::EntityStatusValues* Entity_impl::get_status_values() throw (CORBA::SystemException)
        {
        CdmwPlatformMngt::EntityStatusValues_var stat_values = new CdmwPlatformMngt::EntityStatusValues() ;

    long int num_seq = m_status_values.length() ;
        stat_values->length(num_seq) ;

        (*stat_values) = m_status_values;
        return stat_values._retn() ;
            
        }
        
        
/**
 *Purpose:
 *<p>
 * Implements the
 * IDL:thalesgroup.com/CdmwPlatformMngt/Entity/get_snapshot:1.0
 * operation
 */
    CdmwPlatformMngt::EntitySnapshot* 
    Entity_impl::get_snapshot()
        throw (CORBA::SystemException)
    {
        try
        {
        CdmwPlatformMngt::EntitySnapshot_var snapshot =
            new CdmwPlatformMngt::EntitySnapshot;
        
        snapshot->entity_name = name();
        snapshot->entity_status = get_status ((snapshot->entity_info).out());
        snapshot->element_path = element_path();
        // managing properties 

         CosPropertyService::PropertiesIterator_var propertiesIt;
        CORBA::ULong nbr_of_properties;
        CdmwPlatformMngt::PropertiesSnapshot_var propertiesSnapshot = new CdmwPlatformMngt::PropertiesSnapshot;
        nbr_of_properties = get_number_of_properties();
        CosPropertyService::PropertyNames_var prop_names;
        CosPropertyService::PropertyNamesIterator_var prop_namesIt;
        get_all_property_names(nbr_of_properties, prop_names.out(), prop_namesIt.out());
        propertiesSnapshot->length(nbr_of_properties-2);
        CORBA::ULong snap_prop=0;
        for (unsigned int j = 0; j < nbr_of_properties ; j++)
        {
            if ((strcmp (prop_names[j], "Cdmw.TechStatus")==0) ||
                    (strcmp (prop_names[j], "Cdmw.TechStatusInfo")==0))
            {
                // do not add the TECH_STATUS and TECH_STATUS_INFO properties to the snapshot 
            }
            else
            {
                //CdmwPlatformMngt::PropertySnapshot* propertySnapshot = new CdmwPlatformMngt::PropertySnapshot();
        CdmwPlatformMngt::PropertySnapshot_var propertySnapshot = new CdmwPlatformMngt::PropertySnapshot();
        
                propertySnapshot -> property_name = CORBA::string_dup(prop_names[j]);
                CORBA::Any prop_value;
                prop_value = *(get_property_value(prop_names[j]));
                CORBA::TypeCode_var tc;
                tc = prop_value.type();
                switch (tc->kind())    
                {
                default:
                    break;
                case CORBA::tk_string:
                    propertySnapshot -> property_type = CORBA::string_dup("String");
                    break;
                case CORBA::tk_short:
                    propertySnapshot -> property_type = CORBA::string_dup("Short");
                    break;
                case CORBA::tk_float:
                    propertySnapshot -> property_type = CORBA::string_dup("Float");
                    break;
                };


                CosPropertyService::PropertyModeType mode= get_property_mode(prop_names[j]);
                propertySnapshot->property_value = prop_value;

                if (mode == CosPropertyService::normal) propertySnapshot->property_mode = CORBA::string_dup ("Read/Write");
                else if (mode == CosPropertyService::read_only) propertySnapshot->property_mode = CORBA::string_dup ("Read/Only");
                else propertySnapshot->property_mode = CORBA::string_dup ("Pre-defined");

                propertySnapshot->property_value = prop_value;
                (*propertiesSnapshot)[snap_prop] = *(propertySnapshot);
                snap_prop++;
            }
        }        
        snapshot->entity_properties = propertiesSnapshot.in();
    
        // return snapshot sequence
        return snapshot._retn();
        }
        catch (const std::bad_alloc&)
        {
        throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
        catch (const Cdmw::OutOfResourcesException&)
        {
        throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
    }

        /**
        *Purpose:
        *<p> facade to get an element type
        *
        */

        CdmwPlatformMngt::ElementType Entity_impl::get_element_type()
                              throw (CORBA::SystemException)
        {
            return ::CdmwPlatformMngt::ENTITY_TYPE;
        };


        CdmwPlatformMngt::Children* Entity_impl::get_children()
                              throw (CORBA::SystemException)
        {
            ::CdmwPlatformMngt::Children_var children = new ::CdmwPlatformMngt::Children(0);
            return children._retn();
        };
 
  
    void Entity_impl::notify_entity_status_change(const char* status, const char* mode) 
                                throw (CdmwPlatformMngt::EntityStatusNotAllowed, CORBA::SystemException)

    {
        // recover the StatusEventNotifier by EventNotifierManager
        Cdmw::PlatformMngt::StatusEventNotifier* 
            status_event_notifier = EventNotifierManager::Instance()->get_status_notifier();
            
        //recover the SupervisionEventNotifier by EventNotifierManager
        Cdmw::PlatformMngt::SupervisionEventNotifier* 
            supervision_event_notifier = EventNotifierManager::Instance()->get_sup_notifier();

        CdmwPlatformMngtBase::EventHeader header
            = EventHeaderFactory::createHeader (CdmwPlatformMngtBase::INF);

        CdmwPlatformMngt::ManagedElement_var obj = CdmwPlatformMngt::ManagedElement::_duplicate(this->get_parent());
        
        CdmwPlatformMngt::EntityStatusChange_var event;
       
        //////////////////////////////////////////
        //        check the type_id
        //////////////////////////////////////////        
        if (obj->_is_a("IDL:thalesgroup.com/CdmwPlatformMngt/System:1.0"))
        {   
            CdmwPlatformMngt::System_var system_var = CdmwPlatformMngt::System::_narrow(obj.in() );
               
            event = m_ent_status_change_factory->create ( 
                        header,
                        system_var->name(),
                        name(),
                        element_path(),
                        status,
                        mode);
        }
        
         
        else if (obj->_is_a("IDL:thalesgroup.com/CdmwPlatformMngt/Host:1.0"))
        {
            CdmwPlatformMngt::Host_var host_var = CdmwPlatformMngt::Host::_narrow(obj.in());
               
            event = m_ent_status_change_factory->create ( 
                        header,
                        host_var->get_parent()->name(),
                        name(),
                        element_path(),
                        status,
                        mode);
            
        }

                
        else if (obj->_is_a("IDL:thalesgroup.com/CdmwPlatformMngt/Application:1.0"))
        {
            CdmwPlatformMngt::Application_var application_var = CdmwPlatformMngt::Application::_narrow(obj.in());
                
            event = m_ent_status_change_factory->create ( 
                        header,
                        application_var->get_parent()->name(),
                        name(),
                        element_path(),
                        status,
                        mode);
        }
       
        
        else if (obj->_is_a("IDL:thalesgroup.com/CdmwPlatformMngt/Process:1.0"))
        {
            CdmwPlatformMngt::Process_var process_var = CdmwPlatformMngt::Process::_narrow(obj.in());
            event = m_ent_status_change_factory->create ( 
                    header,
                    process_var->get_parent()->get_parent()->name(),
                    name(),
                    element_path(),
                    status,
                    mode);
        }
        
        else if (obj->_is_a("IDL:thalesgroup.com/CdmwPlatformMngt/RedundancyGroup:1.0"))
        {   
            CdmwPlatformMngt::RedundancyGroup_var group = CdmwPlatformMngt::RedundancyGroup::_narrow(obj.in() );
            
            event = m_ent_status_change_factory->create ( 
                        header,
                        group->get_parent()->get_parent()->name(),
                        name(),
                        element_path(),
                        status,
                        mode);
                
        }
        else if (obj->_is_a("IDL:thalesgroup.com/CdmwPlatformMngt/CcmComponent:1.0"))
        {   
            CdmwPlatformMngt::CcmComponent_var component = CdmwPlatformMngt::CcmComponent::_narrow(obj.in() );
            
            event = m_ent_status_change_factory->create ( 
                    header,
                    component->get_parent()->get_parent()->get_parent()->name(),
                    name(),
                    element_path(),
                    status,
                    mode);

        }
        
            event->_add_ref();
            if (status_event_notifier!=0) status_event_notifier -> add_event (event.in());
            
            event->_add_ref();
            if (supervision_event_notifier!=0) supervision_event_notifier -> addEvent(event.in());
            event->_remove_ref();
            
            event._retn ();
    }
    void Entity_impl::synchro_with_datastore ()
    {
        Cdmw::PlatformMngt::ManagedElement_impl::synchro_with_datastore ();
    }


    } // End namespace PlatformMngt
} // End namespace Cdmw


