/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/common/Exception.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"

#include "SystemMngt/platformelementlibrary/PropertySet_impl.hpp"


/**
 * Root namespace for CDMW runtime.
 */
namespace Cdmw
{
/**
 * Namespace for CDMW PlatformMngt
 */
    namespace PlatformMngt
    {
    
    PropertySet_impl::PropertySet_impl(PortableServer::POA_ptr poa)
    {
        m_property_set_bridge = 0;
        if (CORBA::is_nil(poa))
        CDMW_THROW2(BadParameterException, "poa", "nil");
        m_poa = PortableServer::POA::_duplicate(poa);

           }
    
    PropertySet_impl::~PropertySet_impl()
    {
    }
 
        PortableServer::POA_ptr PropertySet_impl::_default_POA()
    {

        return PortableServer::POA::_duplicate(m_poa.in());

    }
 

       
    void PropertySet_impl::activate()
        throw (AlreadyDoneException, InternalErrorException)
    {

        try
        {
        m_oid = m_poa->activate_object(this);
        }
        catch(const PortableServer::POA::ServantAlreadyActive&)
        {
        CDMW_THROW(AlreadyDoneException);
        }
        catch(...)
        {
        CDMW_THROW(InternalErrorException);
        }


    }


    void PropertySet_impl::activateWithId(const std::string& objectId)
        throw (AlreadyDoneException, InternalErrorException)
    {

        try
        {

        m_oid = PortableServer::string_to_ObjectId(objectId.c_str());

        m_poa->activate_object_with_id(m_oid.in(), this);

        }
        catch(const PortableServer::POA::ObjectAlreadyActive&)
        {
        CDMW_THROW(AlreadyDoneException);
        }
        catch(const PortableServer::POA::ServantAlreadyActive&)
        {
        CDMW_THROW(AlreadyDoneException);
        }
        catch(...)
        {
        CDMW_THROW(InternalErrorException);
        }


    }


    void PropertySet_impl::deactivate()
        throw (BadOrderException, InternalErrorException)
    {

        try
        {

        m_poa->deactivate_object(m_oid.in());

        }
        catch(const PortableServer::POA::ObjectNotActive&)
        {
        CDMW_THROW(BadOrderException);
        }
        catch(...)
        {
        CDMW_THROW(InternalErrorException);
        }

    }


     void  PropertySet_impl::define_property(const char* property_name,
                        const CORBA::Any& property_value)
        throw (CosPropertyService::InvalidPropertyName,
           CosPropertyService::ConflictingProperty,
           CosPropertyService::UnsupportedTypeCode,
           CosPropertyService::UnsupportedProperty,
           CosPropertyService::ReadOnlyProperty,
           CORBA::SystemException)
    {
        this->get_impl() -> define_property(property_name,property_value);
    }
    
    void  PropertySet_impl::define_properties(const CosPropertyService::Properties & nproperties)
        throw (CosPropertyService::MultipleExceptions,
           CORBA::SystemException)
    {
        this->get_impl() -> define_properties(nproperties);
    }
       
    CORBA::ULong PropertySet_impl::get_number_of_properties()
        throw (CORBA::SystemException)
          {
        return  this->get_impl() ->get_number_of_properties();
    }
      
      
    void  PropertySet_impl::get_all_property_names (CORBA::ULong how_many,       
                            CosPropertyService::PropertyNames_out property_names,
                            CosPropertyService::PropertyNamesIterator_out rest)
        throw (CORBA::SystemException)
      
          {
        this->get_impl() -> get_all_property_names (how_many,       
                            property_names,
                            rest);
    }
    
    
    CORBA::Any*  PropertySet_impl::get_property_value (const char* property_name)
        throw (CosPropertyService::PropertyNotFound,
           CosPropertyService::InvalidPropertyName,
           CORBA::SystemException)

    {
        return this->get_impl() -> get_property_value(property_name) ;
    }
      
    CORBA::Boolean  PropertySet_impl::get_properties (
        const CosPropertyService::PropertyNames& property_names,
        CosPropertyService::Properties_out nproperties)
        throw (CORBA::SystemException)
    
    {
        return  this->get_impl() -> get_properties ( property_names, nproperties);
    }
    
    void  PropertySet_impl::get_all_properties (CORBA::ULong how_many,      
                            CosPropertyService::Properties_out nproperties,
                            CosPropertyService::PropertiesIterator_out rest)
        throw (CORBA::SystemException)
      
    {
        this->get_impl() -> get_all_properties ( how_many, nproperties, rest);
    }
      
    void  PropertySet_impl::delete_property(const char* property_name)
        throw (CosPropertyService::PropertyNotFound,
           CosPropertyService::InvalidPropertyName,
           CosPropertyService::FixedProperty,
           CORBA::SystemException)
          {
        this->get_impl() -> delete_property(property_name);
    }
        
    void  PropertySet_impl::delete_properties(const CosPropertyService::PropertyNames& property_names)
        throw (CosPropertyService::MultipleExceptions,
           CORBA::SystemException)
    {
        this->get_impl() -> delete_properties(property_names);
    }
      
    CORBA::Boolean  PropertySet_impl::delete_all_properties()
        throw (CORBA::SystemException)
    {
        return  this->get_impl() -> delete_all_properties();
    }
    
    CORBA::Boolean  PropertySet_impl::is_property_defined(const char* property_name)
        throw (CosPropertyService::InvalidPropertyName,
           CORBA::SystemException)
        {
        return  this->get_impl() -> is_property_defined(property_name);
    }
    
    void PropertySet_impl::get_allowed_property_types(CosPropertyService::PropertyTypes_out property_types)
        throw (CORBA::SystemException)
    {
        this->get_impl() -> get_allowed_property_types(property_types);
    }
      
    void  PropertySet_impl::get_allowed_properties(CosPropertyService::PropertyDefs_out property_defs)
        throw (CORBA::SystemException)
    {
        this->get_impl() -> get_allowed_properties(property_defs);
    }
      
    void  PropertySet_impl::define_property_with_mode(const char* property_name,
                              const CORBA::Any& property_value,
                              CosPropertyService::PropertyModeType property_mode)
        throw (CosPropertyService::InvalidPropertyName,
           CosPropertyService::ConflictingProperty,
           CosPropertyService::UnsupportedTypeCode,
           CosPropertyService::UnsupportedProperty,
           CosPropertyService::UnsupportedMode,
           CosPropertyService::ReadOnlyProperty,
           CORBA::SystemException)
    
    {
        this->get_impl() -> define_property_with_mode(property_name,
                              property_value,
                              property_mode);
    }
    
    void PropertySet_impl::define_properties_with_modes(const CosPropertyService::PropertyDefs & property_defs)
        throw (CosPropertyService::MultipleExceptions,
           CORBA::SystemException)
    {
        this->get_impl() -> define_properties_with_modes(property_defs);
    }
    
    
    CosPropertyService::PropertyModeType  PropertySet_impl::get_property_mode(const char* property_name)
        throw (CosPropertyService::PropertyNotFound,
           CosPropertyService::InvalidPropertyName,
           CORBA::SystemException)
    {
        return this->get_impl() -> get_property_mode(property_name);
    }
      
    CORBA::Boolean  PropertySet_impl::get_property_modes(const CosPropertyService::PropertyNames& property_names,
                                 CosPropertyService::PropertyModes_out property_modes)
        throw (CORBA::SystemException)
    {
        return this->get_impl() -> get_property_modes(property_names, property_modes);
    }
      
    void  PropertySet_impl::set_property_mode(const char* property_name,
                          CosPropertyService::PropertyModeType property_mode)
        throw (CosPropertyService::InvalidPropertyName,
           CosPropertyService::PropertyNotFound,
           CosPropertyService::UnsupportedMode,
           CORBA::SystemException)
    {
        this->get_impl() -> set_property_mode(property_name,property_mode);
    }
      
    void  PropertySet_impl::set_property_modes(const CosPropertyService::PropertyModes & property_modes)
        throw (CosPropertyService::MultipleExceptions,
           CORBA::SystemException)
    {
        this->get_impl() -> set_property_modes(property_modes);
    }

      
        PropertySetBridge*  PropertySet_impl::get_impl()
    {
        if( m_property_set_bridge == 0 ) 
        m_property_set_bridge = PropertyFactory::instance() -> get_property_set_implementation();
            return m_property_set_bridge;
    }

    } // End namespace PlatformMngt
} // End namespace Cdmw

