/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#if CDMW_ORB_VDR == tao

#include "SystemMngt/platformelementlibrary/TAO_PropertySet_impl.hpp"
#include "SystemMngt/platformelementlibrary/PropertyFactory.hpp"


/**
 * Root namespace for CDMW runtime.
 */

namespace 
{

 bool init_factory()
 {
    Cdmw::PlatformMngt::PropertyFactory::instance()->register_property_set_implementation(new Cdmw::PlatformMngt::TAO_PropertySet_impl);
     return true;
 }

 bool init = init_factory();

}

// add define TAO

namespace Cdmw
{
    namespace PlatformMngt
    {
    
    
      /*
      */
    TAO_PropertySet_impl::TAO_PropertySet_impl()
    {
        m_property_set = new TAO_PropertySetDef();                     
    }
    
    
    /**
     *Purpose:
     *<p> Default destructor.
     */
    TAO_PropertySet_impl::~TAO_PropertySet_impl()
    {
        delete m_property_set;
    }
    
          void  TAO_PropertySet_impl::define_property(const char* property_name,
                           const CORBA::Any& property_value)
        throw (CosPropertyService::InvalidPropertyName,
           CosPropertyService::ConflictingProperty,
           CosPropertyService::UnsupportedTypeCode,
           CosPropertyService::UnsupportedProperty,
           CosPropertyService::ReadOnlyProperty,
           CORBA::SystemException)
    {
        m_property_set -> define_property(property_name,property_value);
    }
    
    void  TAO_PropertySet_impl::define_properties(const CosPropertyService::Properties & nproperties)
        throw (CosPropertyService::MultipleExceptions,
           CORBA::SystemException)
    {
        m_property_set -> define_properties(nproperties);
    }
      
    CORBA::ULong TAO_PropertySet_impl::get_number_of_properties()
        throw (CORBA::SystemException)
      
          {
        return  m_property_set ->get_number_of_properties() ;
    }
      
      
    void  TAO_PropertySet_impl::get_all_property_names (CORBA::ULong how_many,       
                               CosPropertyService::PropertyNames_out property_names,
                               CosPropertyService::PropertyNamesIterator_out rest)
        throw (CORBA::SystemException)
      
          {
        m_property_set -> get_all_property_names (how_many,       
                              property_names,
                              rest);
    }
    
    
    CORBA::Any* TAO_PropertySet_impl::get_property_value (const char* property_name)
        throw (CosPropertyService::PropertyNotFound,
           CosPropertyService::InvalidPropertyName,
           CORBA::SystemException)
    {
        return  m_property_set -> get_property_value(property_name) ;
    }
      
    CORBA::Boolean  TAO_PropertySet_impl::get_properties (
        const CosPropertyService::PropertyNames& property_names,
        CosPropertyService::Properties_out nproperties)
        throw (CORBA::SystemException)
    
    {
        return  m_property_set -> get_properties ( property_names, nproperties);
    }
    
    void  TAO_PropertySet_impl::get_all_properties (CORBA::ULong how_many,      
                               CosPropertyService::Properties_out nproperties,
                               CosPropertyService::PropertiesIterator_out rest)
        throw (CORBA::SystemException)
      
    {
        m_property_set -> get_all_properties ( how_many, nproperties, rest);
    }
      
    void  TAO_PropertySet_impl::delete_property(const char* property_name)
        throw (CosPropertyService::PropertyNotFound,
           CosPropertyService::InvalidPropertyName,
           CosPropertyService::FixedProperty,
           CORBA::SystemException)
          {
        m_property_set -> delete_property(property_name);
    }
        
    void  TAO_PropertySet_impl::delete_properties(const CosPropertyService::PropertyNames& property_names)
        throw (CosPropertyService::MultipleExceptions,
           CORBA::SystemException)

    {
        m_property_set -> delete_properties(property_names);
    }
 
    
    CORBA::Boolean  TAO_PropertySet_impl::delete_all_properties()
        throw (CORBA::SystemException)
     {
        return  m_property_set -> delete_all_properties();
    }
    
    CORBA::Boolean  TAO_PropertySet_impl::is_property_defined(const char* property_name)
        throw (CosPropertyService::InvalidPropertyName,
           CORBA::SystemException)
        {
        return m_property_set -> is_property_defined(property_name);
    }
    
    void  TAO_PropertySet_impl::get_allowed_property_types(CosPropertyService::PropertyTypes_out property_types)
        throw (CORBA::SystemException)
    {
        m_property_set -> get_allowed_property_types(property_types);
    }
      
    void  TAO_PropertySet_impl::get_allowed_properties(CosPropertyService::PropertyDefs_out property_defs)
        throw (CORBA::SystemException)
    {
        m_property_set -> get_allowed_properties(property_defs);
    }
      
    void  TAO_PropertySet_impl::define_property_with_mode(const char* property_name,
                                 const CORBA::Any& property_value,
                                 CosPropertyService::PropertyModeType property_mode)
        throw (CosPropertyService::InvalidPropertyName,
           CosPropertyService::ConflictingProperty,
           CosPropertyService::UnsupportedTypeCode,
           CosPropertyService::UnsupportedProperty,
           CosPropertyService::UnsupportedMode,
           CosPropertyService::ReadOnlyProperty,
           CORBA::SystemException)
    
    {
        m_property_set -> define_property_with_mode(property_name,
                            property_value,
                            property_mode);
        
    }
    
    void  TAO_PropertySet_impl::define_properties_with_modes(const CosPropertyService::PropertyDefs & property_defs)
        throw (CosPropertyService::MultipleExceptions,
           CORBA::SystemException)
    {
        m_property_set -> define_properties_with_modes(property_defs);
    }
    
    
        CosPropertyService::PropertyModeType  TAO_PropertySet_impl::get_property_mode(const char* property_name)
        throw (CosPropertyService::PropertyNotFound,
           CosPropertyService::InvalidPropertyName,
           CORBA::SystemException)
    {
        return  m_property_set -> get_property_mode(property_name);
    }
      
    CORBA::Boolean  TAO_PropertySet_impl::get_property_modes(const CosPropertyService::PropertyNames& property_names,
                                CosPropertyService::PropertyModes_out property_modes)
        throw (CORBA::SystemException)
    {
        return  m_property_set -> get_property_modes(property_names, property_modes);
    }
      
    void  TAO_PropertySet_impl::set_property_mode(const char* property_name,
                             CosPropertyService::PropertyModeType property_mode)
        throw (CosPropertyService::InvalidPropertyName,
           CosPropertyService::PropertyNotFound,
           CosPropertyService::UnsupportedMode,
           CORBA::SystemException)
    {
        m_property_set -> set_property_mode(property_name,property_mode);
    }
      
    void  TAO_PropertySet_impl::set_property_modes(const CosPropertyService::PropertyModes & property_modes)
        throw (CosPropertyService::MultipleExceptions,
           CORBA::SystemException)
    
    {
        m_property_set -> set_property_modes(property_modes);
    }
      
      TAO_PropertySet_impl* TAO_PropertySet_impl::clone()
      {
      return new TAO_PropertySet_impl();
      }
     
    } // End namespace PlatformMngt
} // End namespace Cdmw

#else
#error "Unsupported ORB"
#endif


    
