/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <iostream>
#include <limits>
#include "testchildobjects/TestChildObjects.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"

#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"

 
using namespace std;

using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION(TestChildObjects);

std::string str_row("-------------------");
std::string str_col("|");
#define Graph_parent(P) std::cerr<<std::endl<<P<<"\n";
#define Graph_child(INFO,F) std::cerr<<str_col<<(str_row+INFO+" "+F+"\n");
#define Graph_end std::cerr<<"\n"<<"\n";

namespace Cdmw
{
namespace PlatformMngt
{



TestChildObjects::TestChildObjects() throw() 
{

}


TestChildObjects::~TestChildObjects()
    throw()
{

}


void
TestChildObjects::do_tests()                
{
 //new for test
// //  set_nbOfRequestedTestOK (7);


 CdmwPlatformMngt::System_ptr mysystem = CdmwPlatformMngt::System::_nil();
 CdmwPlatformMngt::Application_var myapp;
 CdmwPlatformMngt::Process_var myproc1;
 CdmwPlatformMngt::Process_var myproc2;
 CdmwPlatformMngt::Process_var myproc3;
 CdmwPlatformMngt::Host_var myhost;
 CdmwPlatformMngt::Entity_var myent1;
 CdmwPlatformMngt::Entity_var myent2;


 try
    {
        /*get the orb*/
        m_orb = CORBA::ORB::_duplicate( Cdmw::TestUtils::CORBATestManager::instance()->get_ORB() );

        // ===================================================
        // Get the HelloInterface object
        // ===================================================

        // import the object reference from the ior file
        CORBA::Object_var objref;


        //define the corbaloc
        const char *corbaloc_str = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";

        objref = m_orb->string_to_object(corbaloc_str);

    char * hostname  = getenv("HOSTNAME");

        //resolve the System_ptr
        mysystem = CdmwPlatformMngt::System::_narrow(objref.in() );
        
    myapp = mysystem -> get_application("HELLO_WORLD");
    
    myhost = mysystem -> get_host(hostname);
    
    myproc1 = myhost->get_process("HelloProcess1", "HELLO_WORLD");
    myproc2 = myhost->get_process("HelloProcess2", "HELLO_WORLD");
    myproc3 = myhost->get_process("HelloProcess3", "HELLO_WORLD");
    
    if (CORBA::is_nil(mysystem) )
      {
        TEST_INFO("Error in narrowing CdmwPlatformMngt::System_ptr");
        exit(-1);
      }

    try
        { 
              CORBA::ULong res = 0; 
              CdmwPlatformMngt::Children_var children = new CdmwPlatformMngt::Children;
              children = mysystem->get_children();      
              CORBA::ULong size_of_children = children->length();
              CdmwPlatformMngt::Application_var     app_child;
              CdmwPlatformMngt::Entity_var          ent_child;
              CdmwPlatformMngt::Host_var            host_child;
              
              myent1 = mysystem -> get_entity("EntitySys1");
              myent2 = mysystem -> get_entity("EntitySys2");
              
              Graph_parent("SYSTEM_CHILDREN");  
              if (size_of_children != 0)
              {
                  for (CORBA::ULong count = 0; count < size_of_children ; count ++ )
                  {
                      if (((children[count])->get_element_type()) == CdmwPlatformMngt::APPLICATION_TYPE )
                      {
                        app_child = CdmwPlatformMngt::Application::_narrow(children[count]);
                        CdmwPlatformMngt::Application_ptr app_ptr = app_child.in();
                        if ((myapp->_is_equivalent(app_ptr))) 
                        {
                            res++;
                            Graph_child(" APPLICATION ",app_ptr->name());
                        }
                      } 
                      else if (((children[count])->get_element_type()) == CdmwPlatformMngt::ENTITY_TYPE )
                      {
                   ent_child = CdmwPlatformMngt::Entity::_narrow(children[count]);
                   CdmwPlatformMngt::Entity_ptr ent_ptr = ent_child.in();
                   if (((myent1->_is_equivalent(ent_ptr)))||((myent2->_is_equivalent((ent_ptr)))))
                   {
                      res++;
                      Graph_child(" ENTITY ",ent_ptr->name());
                   }
                    }
                      else if (((children[count])->get_element_type()) == CdmwPlatformMngt::HOST_TYPE )
                            {
                              host_child = CdmwPlatformMngt::Host::_narrow(children[count]);
                              CdmwPlatformMngt::Host_ptr host_ptr = host_child.in();
                              if ((myhost->_is_equivalent(host_ptr)))//||((myhost->_is_equivalent((host_ptr)))))
                               {
                                 res++;
                                 Graph_child(" HOST ",host_ptr->name());
                               }
                            }
                  }
                  if (res== (size_of_children -1 )) // CdmwSystemSynchronization is excluded
                  { 
                    CPPUNIT_ASSERT(1);
                  }
                  else 
                  {
                    CPPUNIT_ASSERT(0);
                  }
              }
              
              Graph_end;
              std::cerr<<std::endl;
              Graph_parent("APPLICATION_CHILDREN"); 
              res = 0;  
              children = myapp->get_children();     
              size_of_children = children->length();
              CdmwPlatformMngt::Process_var     proc_child;
              myent1 = myapp-> get_entity("EntityApp1");
              myent2 = myapp-> get_entity("EntityApp2");
              if (size_of_children != 0)
              {
                  for (CORBA::ULong count = 0; count < size_of_children ; count ++ )
                  {
                      if (((children[count])->get_element_type()) == CdmwPlatformMngt::PROCESS_TYPE )
                      {
                        proc_child = CdmwPlatformMngt::Process::_narrow(children[count]);
                        CdmwPlatformMngt::Process_ptr proc_ptr = proc_child.in();
                        if (((myproc1->_is_equivalent(proc_ptr)))||((myproc2->_is_equivalent(proc_ptr)))||((myproc3->_is_equivalent(proc_ptr))))
                        {
                            res++;
                            Graph_child(" PROCESS ",proc_ptr->name());
                        }
                      }
                      if (((children[count])->get_element_type()) == CdmwPlatformMngt::ENTITY_TYPE )
                      {
                          ent_child = CdmwPlatformMngt::Entity::_narrow(children[count]);
                          CdmwPlatformMngt::Entity_ptr ent_ptr = ent_child.in();
                          if (((myent1->_is_equivalent(ent_ptr)))||((myent2->_is_equivalent((ent_ptr)))))
                          {
                              res++;
                              Graph_child(" ENTITY ",ent_ptr->name());
                          }
                      }
                  }
                  if (res == size_of_children) CPPUNIT_ASSERT(1);
                  else CPPUNIT_ASSERT(0);
              }
              
              Graph_end;  
              Graph_parent("HOST_CHILDREN");
              res = 0;  
              children = myhost->get_children();        
              size_of_children = children->length();
              myent1 = myhost-> get_entity("EntityHost1");
              myent2 = myhost-> get_entity("EntityHost2");
              
          if (size_of_children != 0)
              {
                  for (CORBA::ULong count = 0; count < size_of_children ; count ++ )
                  {
                      if (((children[count])->get_element_type()) == CdmwPlatformMngt::ENTITY_TYPE )
                      {
                          ent_child = CdmwPlatformMngt::Entity::_narrow(children[count]);
                          CdmwPlatformMngt::Entity_ptr ent_ptr = ent_child.in();
                          if (((myent1->_is_equivalent(ent_ptr)))||((myent2->_is_equivalent((ent_ptr)))))
                          {
                              res++;
                              Graph_child(" ENTIY ",ent_ptr->name());
                          }
                      }
                  }
                  if (res == size_of_children) CPPUNIT_ASSERT(1);
                  else CPPUNIT_ASSERT(0);
              }
             
              Graph_end;
              Graph_parent("MANAGED_PROCESS_CHILDREN");
              res = 0;  
              children = myproc3->get_children();       
              size_of_children = children->length();
              myent1 = myproc3-> get_entity("EntityProc1");
              myent2 = myproc3-> get_entity("EntityProc2");
              
          if (size_of_children != 0)
              {
                  for (CORBA::ULong count = 0; count < size_of_children ; count ++ )
                  {
                      if (((children[count])->get_element_type()) == CdmwPlatformMngt::ENTITY_TYPE )
                      {
                          ent_child = CdmwPlatformMngt::Entity::_narrow(children[count]);
                          CdmwPlatformMngt::Entity_ptr ent_ptr = ent_child.in();
                          if (((myent1->_is_equivalent(ent_ptr)))||((myent2->_is_equivalent((ent_ptr)))))
                          {
                              res++;
                              Graph_child(" ENTITY ",ent_ptr->name());
                          }
                      }
                  }
                  if (res == size_of_children) CPPUNIT_ASSERT(1);
                  else CPPUNIT_ASSERT(0);
              }
  
                Graph_end;  
                Graph_parent("UNMANAGED_PROCESS_CHILDREN");
                children = myproc1->get_children();
                    
                if ( 0 == children)
                {
                    CPPUNIT_ASSERT(0); //test nok             
                }
                if (0 == children-> length())
                {
                    CPPUNIT_ASSERT(1); //test ok
                }
                
                
                Graph_end;  
                Graph_parent("ENTITY_CHILDREN");
                children = myent1->get_children();
                
                if ( 0 == children)
                {
                    CPPUNIT_ASSERT(0); //test nok             
                }
                if (0 == children-> length())
                {
                    CPPUNIT_ASSERT(1); //test ok
                }
                Graph_end; 
        
        }
        catch( const CORBA::Exception  & ex )
        {
            CPPUNIT_ASSERT(0); //test nok
            std::cerr << ex._name() << std::endl;
        }
        catch (const Cdmw::Exception & ex)
        {
            CPPUNIT_ASSERT(0); //test nok
            std::cerr << ex.what() << std::endl;
            exit(-1);
        }

    }
 catch(...)
 {
     std::cerr << "unexpected exception" << std::endl;
     exit(-1);
 }
 
 // ========================================================
 // program stopping
 // ========================================================
 
 // ===================================================
 // Call ORB cleanup
 // ===================================================
 Cdmw::OrbSupport::OrbSupport::ORB_cleanup(m_orb.in());
 
 // =====================================================
 // Destroy orb
 // =====================================================
 
 if (!CORBA::is_nil(m_orb.in()))
   {
     try
       {     
     m_orb->destroy();
       }
     catch(const CORBA::Exception& ex)
       {
     std::cerr << ex << std::endl;
     exit(-1);
       }
   }
}    

} // End namespace PlatformMngt
} // End namespace Cdmw




