/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_TESTCONFIGURATIONCHANGE_HPP
#define INCL_PLATFORMMNGT_TESTCONFIGURATIONCHANGE_HPP

#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/common/System.hpp"
#include <Foundation/orbsupport/CORBA.hpp>

#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
//#include "idllib/CdmwPlatformMngtApplication.stub.hpp"
#include <SystemMngt/idllib/CdmwPlatformMngtSupervisionObserver.stub.hpp>
#include "SupervisionObserver_impl.hpp"
#include "testconfigurationchange/OneShotInitFactory.hpp"

namespace Cdmw
{
  namespace PlatformMngt
  {

/**
*Purpose:
*<p>
*
*Features:
*<p> Thread safe
*
*
*/
    class TestConfigurationChange:public CppUnit::TestFixture
    {

    public:

    /**
        * Purpose:
        * <p> Constructor
        *
        */
      TestConfigurationChange () throw ();

    /**
        * Purpose:
        * <p> Destructor
        *
        */
      virtual ~TestConfigurationChange () throw ();

      void setUp ();

      CdmwPlatformMngt::System_var  m_system;
      CdmwPlatformMngt::Host_var    m_host;
      CdmwPlatformMngt::Process_var m_process;
      CdmwPlatformMngt::SupervisionObserver_var m_observer;

      SupervisionObserver_impl *m_observer_impl;
      OneShotInitFactory*  one_shot_init_factory;

    /**
        * Purpose:
        * <p> Call from the testable object
        *
        */
 
      virtual void test_register_observer_deeply ();
      virtual void test_register_observer_not_deeply ();
      virtual void test_unregister_observer ();
      virtual void test_register_host_observer_deeply ();
      virtual void test_register_host_observer_not_deeply (); 
      virtual void test_add_process ();
      virtual void test_remove_process ();
      virtual void test_add_application();
      virtual void test_remove_application ();
      virtual void test_add_entity_to_system ();
      virtual void test_remove_entity_to_system ();
      virtual void test_add_host();
      virtual void test_remove_host();
      virtual void notification_received();
      virtual void notification_not_received();
      virtual void test_unregister_host_observer(); 
      virtual void test_define_property_and_entity_to_process();
      virtual void test_remove_property_and_entity_to_process();
    private:

    /**
        * Purpose:
        * <p> Assignment operator
        *
        */
        TestConfigurationChange & operator= (const TestConfigurationChange & rhs)
    throw ();
    /**
        * Purpose:
        * <p>  Copy constructor
        *
        */
        TestConfigurationChange (const TestConfigurationChange & rhs) throw ();


    
        CPPUNIT_TEST_SUITE (TestConfigurationChange);
        // Testing Host 
        CPPUNIT_TEST (test_register_host_observer_not_deeply);
        CPPUNIT_TEST (test_define_property_and_entity_to_process);
        CPPUNIT_TEST (notification_not_received);
        CPPUNIT_TEST (test_remove_property_and_entity_to_process);
        CPPUNIT_TEST (notification_not_received);
        CPPUNIT_TEST (test_unregister_host_observer);
        CPPUNIT_TEST (test_define_property_and_entity_to_process); 
        CPPUNIT_TEST (notification_not_received);
        CPPUNIT_TEST (test_remove_property_and_entity_to_process);
        CPPUNIT_TEST (notification_not_received);
 
        CPPUNIT_TEST (test_register_host_observer_deeply);
        CPPUNIT_TEST (test_define_property_and_entity_to_process);
        CPPUNIT_TEST (notification_received);
        CPPUNIT_TEST (test_remove_property_and_entity_to_process);
        CPPUNIT_TEST (notification_received);
        CPPUNIT_TEST (test_unregister_host_observer);
        CPPUNIT_TEST (test_define_property_and_entity_to_process);
        CPPUNIT_TEST (notification_not_received);
        CPPUNIT_TEST (test_remove_property_and_entity_to_process);
        CPPUNIT_TEST (notification_not_received);
        // end Testing Host

        CPPUNIT_TEST (test_register_observer_not_deeply);
        CPPUNIT_TEST (test_add_application);
        CPPUNIT_TEST (notification_received);
        CPPUNIT_TEST (test_add_process);
        CPPUNIT_TEST (notification_not_received);
        CPPUNIT_TEST (test_add_entity_to_system);
        CPPUNIT_TEST (notification_received);  
        CPPUNIT_TEST (test_add_host);
        CPPUNIT_TEST (notification_received);
        CPPUNIT_TEST (test_remove_application);
        CPPUNIT_TEST (notification_received);
        CPPUNIT_TEST (test_remove_process);
        CPPUNIT_TEST (notification_not_received);
        CPPUNIT_TEST (test_remove_host);
        CPPUNIT_TEST (notification_received);
        CPPUNIT_TEST (test_remove_entity_to_system);
        CPPUNIT_TEST (notification_received);
        // De-registration 
        CPPUNIT_TEST (test_unregister_observer);
        CPPUNIT_TEST (test_add_application);
        CPPUNIT_TEST (notification_not_received);
        CPPUNIT_TEST (test_add_entity_to_system);
        CPPUNIT_TEST (notification_not_received);
        CPPUNIT_TEST (test_add_host);
        CPPUNIT_TEST (notification_not_received);
        CPPUNIT_TEST (test_remove_application);
        CPPUNIT_TEST (notification_not_received);
        CPPUNIT_TEST (test_remove_host);
        CPPUNIT_TEST (notification_not_received);
        CPPUNIT_TEST (test_remove_entity_to_system);
        CPPUNIT_TEST (notification_not_received);
       
        // test register for all
        CPPUNIT_TEST (test_register_observer_deeply);
        //CPPUNIT_TEST (test_unregister_observer);
        CPPUNIT_TEST (test_add_application);
        CPPUNIT_TEST (notification_received);
        CPPUNIT_TEST (test_add_process);
        CPPUNIT_TEST (notification_received);
        CPPUNIT_TEST (test_add_entity_to_system);
        CPPUNIT_TEST (notification_received);
        CPPUNIT_TEST (test_add_host);
        CPPUNIT_TEST (notification_received);
        CPPUNIT_TEST (test_remove_application);
        CPPUNIT_TEST (notification_received);
        CPPUNIT_TEST (test_remove_process);
        CPPUNIT_TEST (notification_received);
        CPPUNIT_TEST (test_remove_host);    
        CPPUNIT_TEST (notification_received);
        CPPUNIT_TEST (test_remove_entity_to_system);
        CPPUNIT_TEST (notification_received);

        //De-registration Deeply 
        CPPUNIT_TEST (test_unregister_observer);
        CPPUNIT_TEST (test_add_application);
        CPPUNIT_TEST (notification_not_received);
        CPPUNIT_TEST (test_add_process);
        CPPUNIT_TEST (notification_not_received);
        CPPUNIT_TEST (test_add_entity_to_system);
        CPPUNIT_TEST (notification_not_received);
        CPPUNIT_TEST (test_add_host);
        CPPUNIT_TEST (notification_not_received);
        CPPUNIT_TEST (test_remove_application);
        CPPUNIT_TEST (notification_not_received);
        CPPUNIT_TEST (test_remove_process);
        CPPUNIT_TEST (notification_not_received);
        CPPUNIT_TEST (test_remove_host);
        CPPUNIT_TEST (notification_not_received);
        CPPUNIT_TEST (test_remove_entity_to_system);
        CPPUNIT_TEST (notification_not_received);
 
            
        CPPUNIT_TEST_SUITE_END ();
    };

  }             /* End namespace PlatformMngt */
}               /* End namespace Cdmw */
#endif              /* INCL_PLATFORMMNGT_TESTCONFIGURATIONCHANGE_HPP */
