/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#ifdef CDMW_USE_FAULTTOLERANCE


#include "testconfigurationchange/OneShotInitFT.hpp"
#include "testconfigurationchange/OneShotInitFactory.hpp"
#include "testconfigurationchange/TestConfigurationChange.hpp"
#include "testconfigurationchange/SupervisionObserver_impl.hpp"
#include "SystemMngt/platformvaluetypes/ConfigurationChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/PropertyChange_impl.hpp"

namespace
{

  bool init_factory ()
  {
    Cdmw::PlatformMngt::OneShotInitFactory::get_instance()->register_one_shot_kind (new Cdmw::PlatformMngt::OneShotInitFT ());
    return true;
  }

  bool booly = init_factory ();
}

namespace Cdmw
{
  namespace PlatformMngt
  {

    void OneShotInitFT::one_shot_init ()
    {
      try
      {
        CORBA::ORB_var orb;
        orb =
          CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::
                      instance ()->get_ORB ());

        PortableServer::POA_var poa =
          PortableServer::POA::_duplicate (Cdmw::TestUtils::
                           CORBATestManager::instance ()->
                           get_POA ());
        PortableServer::POAManager_var poaMgr =
          PortableServer::POAManager::_duplicate (poa->the_POAManager ());
        poaMgr->activate ();

        CORBA::ValueFactoryBase_var factory, old_factory;
        factory = new ConfigurationChangeFactory;
        old_factory =
          orb->
          register_value_factory
          ("IDL:thalesgroup.com/CdmwPlatformMngt/ConfigurationChange:1.0",
           factory.in ());

            factory = new PropertyChangeFactory;
            old_factory =
              orb->
              register_value_factory
              ("IDL:thalesgroup.com/CdmwPlatformMngt/PropertyChange:1.0",
               factory.in ());


            m_observer_impl =  new SupervisionObserver_impl ();

        m_observer = m_observer_impl->_this ();

        CORBA::Object_var objref;

        //define the corbaloc
        const char *corbaloc_str = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";

        objref = orb->string_to_object(corbaloc_str);

        //resolve the System_ptr
        m_system = CdmwPlatformMngt::System::_narrow(objref.in() );


        if (CORBA::is_nil (m_system.in ()))
          {
        // TEST_INFO ("Error in narrowing CdmwPlatformMngt::System");
        exit (-1);
          }

      }
      catch (const Cdmw::Exception & ex)
      {
        std::cerr << ex.what () << std::endl;
        exit (-1);
      }
      catch (const CORBA::Exception & ex)
      {
        std::cerr << ex << std::endl;
        exit (-1);
      }
      catch (...)
      {
        std::cerr << "unexpected exception" << std::endl;
        exit (-1);
      }
    }

  }
}
#endif
