/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <limits>

#include "testconfigurationchange/TestConfigurationChange.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include <SystemMngt/platformlibrary/Iterator.hpp>

#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "SystemMngt/platformvaluetypes/ProcessDef_impl.hpp"

#include "testconfigurationchange/OneShotInitFactory.hpp"

using namespace std;

using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestConfigurationChange);

namespace Cdmw
{
  namespace PlatformMngt
  {
    TestConfigurationChange::TestConfigurationChange () throw ()
    {
    }

    TestConfigurationChange::~TestConfigurationChange () throw ()
    {
    }

    void TestConfigurationChange::setUp ()
    {
       one_shot_init_factory =
    OneShotInitFactory::get_instance ();
    
         m_observer_impl = one_shot_init_factory->get_observer_impl();
         m_system    = one_shot_init_factory->get_system();
         m_observer  = one_shot_init_factory->get_observer();  
         char *hostname  = getenv ("HOSTNAME");
         m_host      = m_system->get_host (hostname);
         m_process       = m_host->get_process("HelloProcess1","HELLO_WORLD");
     }

    void TestConfigurationChange::test_register_observer_deeply ()
    {
        CdmwPlatformMngt::RegistrationDepth depth = CdmwPlatformMngt::FULL_HIERARCHY;
    m_system->register_observer_ext("Observer", depth, m_observer.in());
     CPPUNIT_ASSERT(true);
    }
    
    void TestConfigurationChange::test_unregister_observer() 
    {
    m_system->unregister_observer_ext("Observer");
    CPPUNIT_ASSERT(true);
    }
   
    void TestConfigurationChange::test_register_observer_not_deeply ()
    {
        CdmwPlatformMngt::RegistrationDepth depth = CdmwPlatformMngt::OBJECT_ONLY;
        m_system->register_observer_ext("Observer", depth, m_observer.in());
        CPPUNIT_ASSERT(true);
    }

 
    void TestConfigurationChange::test_register_host_observer_deeply()
    {
        CdmwPlatformMngt::RegistrationDepth depth = FULL_HIERARCHY;
        m_host->register_observer_ext("HostObserver", depth, m_observer.in());
        CPPUNIT_ASSERT(true);
    }

   
    void TestConfigurationChange::test_unregister_host_observer()
    {
        try{
        m_host->unregister_observer_ext("HostObserver");
        CPPUNIT_ASSERT(true);
    } 
    catch(const CdmwPlatformMngt::ObserverNotFound& ex)
    {
    TEST_INFO("EXCEPTION Observer not found");
        CPPUNIT_ASSERT(false);      
    }   
    catch (...)
    {
    TEST_INFO("EXCEPTION UNKNOWN");
        CPPUNIT_ASSERT(false);  
    }
    }

    void TestConfigurationChange::test_register_host_observer_not_deeply()
    {
        CdmwPlatformMngt::RegistrationDepth depth = OBJECT_ONLY;
        m_host->register_observer_ext("HostObserver", depth, m_observer.in());
        CPPUNIT_ASSERT(true);
    }

    void TestConfigurationChange::test_define_property_and_entity_to_process()
    {
     m_observer_impl->set_notification_done (false);
     m_observer_impl->set_test_ok (false);
     CdmwPlatformMngt::EntityDef ent_def;
     ent_def.entity_name = "NewProcessEntity";
     //defining Properties
     CORBA::Any anyp_val;
     string sprop;
     int val=10;
     anyp_val <<= val;
     m_process->define_property("Property1",anyp_val);
     m_process->add_entity(ent_def);
     ent_def.entity_name = "NewHostEntity"; 
     //m_host->add_entity(ent_def);
     //m_host->define_property("Property1",anyp_val);
   }

   

   void TestConfigurationChange::test_remove_property_and_entity_to_process()
   {
     m_observer_impl->set_notification_done (false);
     m_observer_impl->set_test_ok (false);
     //defining Properties
     CORBA::Any anyp_val;
     string sprop;
     int val=10;
     anyp_val <<= val;
     m_process->delete_property("Property1");
     m_process->remove_entity("NewProcessEntity");
     //m_host->remove_entity("NewHostEntity");
     //m_host->delete_property("Property1");
   }

 
    void TestConfigurationChange::test_add_process ()
    {
        m_observer_impl->set_notification_done (false);
        m_observer_impl->set_test_ok (false);
    CdmwPlatformMngt::Application_var app_var  = m_system->get_application("HELLO_WORLD");
    std::cerr << " get Application HELLO_WORLD " << std::endl;
        Cdmw::PlatformMngt::ManagedProcessDefFactory process_def_factory;
        CdmwPlatformMngt::ProcessType process_type = MANAGED_PROCESS;
        CdmwPlatformMngt::ProcessDef_var process_def =  process_def_factory.create("NewProcess", process_type );
        char * hostname  = getenv("HOSTNAME");
        process_def->host_name(CORBA::string_dup (hostname));
    app_var -> add_process(process_def);
        CPPUNIT_ASSERT(true);
    }
    
    void TestConfigurationChange::test_remove_process ()
    {
        m_observer_impl->set_notification_done (false);
        m_observer_impl->set_test_ok (false);
        CdmwPlatformMngt::Application_var app_var  = m_system->get_application("HELLO_WORLD");
        char * hostname  = getenv("HOSTNAME");
    app_var -> remove_process("NewProcess",CORBA::string_dup ((hostname)));
        CPPUNIT_ASSERT(true);
    }
        
    void TestConfigurationChange::test_add_application()
    {   
        m_observer_impl->set_notification_done (false);
        m_observer_impl->set_test_ok (false);
    CdmwPlatformMngt::ApplicationDef application_def;
        CdmwPlatformMngt::PlatformApplicationDef platform_application_def;
    platform_application_def.application_name = "NewApp";
        m_system->add_application(platform_application_def, application_def);
        CPPUNIT_ASSERT(true);
    }
    
    void TestConfigurationChange::test_remove_application ()
    {
        m_observer_impl->set_notification_done (false);
        m_observer_impl->set_test_ok (false);
    m_system -> remove_application("NewApp");
        CPPUNIT_ASSERT(true);
    }

    void TestConfigurationChange::test_add_entity_to_system ()
    {
        m_observer_impl->set_notification_done (false);
        m_observer_impl->set_test_ok (false);
    CdmwPlatformMngt::EntityDef ent_def;
    ent_def.entity_name = "NewEntity";
    m_system->add_entity(ent_def);
        CPPUNIT_ASSERT(true);
    }
    
    void TestConfigurationChange::test_remove_entity_to_system ()
    {
        m_observer_impl->set_notification_done (false);
        m_observer_impl->set_test_ok (false);
    m_system-> remove_entity("NewEntity");
//      CPPUNIT_ASSERT(notification_received());
        CPPUNIT_ASSERT(true);
    }
    
    void TestConfigurationChange::test_add_host()
    {
        m_observer_impl->set_notification_done (false);
        m_observer_impl->set_test_ok (false);
    CdmwPlatformMngt::HostDef hostdef;
    hostdef.host_name = "newHost";
    m_system->add_host(hostdef); 
        CPPUNIT_ASSERT(true);
    }
    
    
    void TestConfigurationChange::test_remove_host()
    {
        m_observer_impl->set_notification_done (false);
        m_observer_impl->set_test_ok (false);
        m_system->remove_host("newHost");
        CPPUNIT_ASSERT(true);
    }
    

    /*wait notification from cardamom */
    void TestConfigurationChange::notification_not_received()
    {
     int rtries = 0;
     while (!m_observer_impl->is_notification_done ())
        {
          Cdmw::OsSupport::OS::sleep (1000);
          rtries++;
          if (rtries > 2)
            break;
        }
     if (!m_observer_impl->test_ok())
     CPPUNIT_ASSERT(true);
     else
     CPPUNIT_ASSERT(false);
    }

 
    /*wait notification from cardamom */ 
    void TestConfigurationChange::notification_received()
    {
     //bool bool_ret = false;
     int rtries = 0;
     while (!m_observer_impl->is_notification_done ())
        {
          Cdmw::OsSupport::OS::sleep (1000);
          rtries++;
          if (rtries > 2)
            break;
        }
     if (m_observer_impl->test_ok()) 
     CPPUNIT_ASSERT(true);
     else
     CPPUNIT_ASSERT(false);

    }
  }             /* End namespace PlatformMngt */
}               /* End namespace Cdmw */





