/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/* @author Fabrizio Lorenna <f.lorenna@vitrociset.it> */

#include <iostream>
#include <limits>

#include "testentitystatus/TestEntityStatus.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"

#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"

#include <platformftproxy/InitUtils.hpp>
#include <platformftproxy/SystemProxy.hpp>
#include <SystemMngt/platformftproxy/AccessorLookupService.hpp>
#include <SystemMngt/platformftproxy/DefaultAccessorLookupService.hpp>
#include <SystemMngt/platformlibrary/Iterator.hpp>

#include <FaultTolerance/ftreplicationmanager/InitUtils.hpp>
#include <FaultTolerance/idllib/FT.stub.hpp>
#include <platformftaccessorlibrary/SystemAccessor_impl.hpp>

using namespace std;

using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestEntityStatus);

namespace Cdmw
{
  namespace PlatformMngt
  {



    TestEntityStatus::TestEntityStatus () throw ()
    {

    }


    TestEntityStatus::~TestEntityStatus () throw ()
    {

    }

    void TestEntityStatus::setUp()
    {
      TestEntityStatus::one_shot_init();
    }


    bool TestEntityStatus::m_is_one_shot_initialized=false;

    CdmwPlatformMngt::Application_var TestEntityStatus::myapp;
    CdmwPlatformMngt::Entity_var TestEntityStatus::myentity;
    CdmwPlatformMngt::Entity_var TestEntityStatus::myentity2;
    CdmwPlatformMngt::System_var TestEntityStatus::mysystem;
    
    CdmwPlatformMngt::Entity_var TestEntityStatus::myentity3;
    CdmwPlatformMngt::Entity_var TestEntityStatus::myentity4;
    CdmwPlatformMngt::Entity_var TestEntityStatus::myentity5;

    void
    TestEntityStatus::one_shot_init()
    {
      if(!m_is_one_shot_initialized)
      {
        m_is_one_shot_initialized = true;



      PortableServer::POA_var poa ;

      try
      {

        CORBA::ORB_var orb;
        orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager:: instance ()->get_ORB ());

        poa = PortableServer::POA::_duplicate (Cdmw::TestUtils::CORBATestManager:: instance ()->get_POA ());
        
        PortableServer::POAManager_var poaMgr = PortableServer::POAManager::_duplicate(poa->the_POAManager());
        poaMgr->activate();

        CORBA::Object_var objref;

        //define the corbaloc
        const char *corbaloc_str = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";

        objref = orb->string_to_object(corbaloc_str);

        //resolve the System_ptr
        mysystem = CdmwPlatformMngt::System::_narrow(objref.in() );

    if (CORBA::is_nil (mysystem.in ()))
      {
        //TEST_INFO ("Error in narrowing CdmwPlatformMngt::System");
        exit (-1);
      }

        /* retrieve application*/

    myapp = mysystem->get_application("HELLO_WORLD");

    myentity = myapp->get_entity("MyEntity");

    myentity2 = myapp->get_entity("MyEntity2");
    
    myentity3 = myapp->get_entity("MyEntity3");
    
    myentity4 = myapp->get_entity("MyEntity4");
    
    myentity5 = myapp->get_entity("MyEntity5");
        }
      catch (const Cdmw::Exception & ex)
      {
        std::cerr << ex.what () << std::endl;
        exit (-1);
      }
      catch (const CORBA::Exception & ex)
      {
        std::cerr << ex << std::endl;
        exit (-1);
      }
      catch (...)
      {
        std::cerr << "unexpected exception" << std::endl;
        exit (-1);
      }
     }
    }

    void TestEntityStatus::get_name()
    {
    CORBA::String_var name = myentity->name();

    if (!strcmp ("MyEntity", name.in () ) )
        CPPUNIT_ASSERT (1); /* test ok */
    else
      {
        CPPUNIT_ASSERT (0); /* test not ok */
      }

        }


    void TestEntityStatus::get_name2()
    {

    CORBA::String_var name2 = myentity2->name();

    if (!strcmp ("MyEntity2", name2.in () ) )
        CPPUNIT_ASSERT (1); /* test ok */
    else
      {
        CPPUNIT_ASSERT (0); /* test not ok */
      }
        }


    /*test to set status aout of allowed range*/
    void TestEntityStatus::set_status_out_range()
    {
    myentity->set_status("DUMMY","MY_INFO");

        }

    void TestEntityStatus::set_status_out_range2()
    {
    myentity2->set_status("DUMMY2","MY_INFO");

        }

    void TestEntityStatus::set_status()
      {

    char flag=0;
    try
    {
    myentity->set_status("SLEEPING","STATUS_INFO");
    }
    catch(...)
      {
          CPPUNIT_ASSERT (0); /* test not ok */
      flag=1;
      }
        
    if (flag == 0)
     {
         CPPUNIT_ASSERT (1); /*test ok*/
     }


      }



    void TestEntityStatus::set_status2()
      {

      char flag=0;
      try
      {
    myentity2->set_status("ENT_STAT3","ENT_INFO");
      }
      catch(...)
      {
          CPPUNIT_ASSERT (0); /* test not ok */
      flag=1;
      }
        
    if (flag == 0)
     {
         CPPUNIT_ASSERT (1); /*test ok*/
     }

      }

    void TestEntityStatus::get_status()
      {
      CORBA::String_var entity_info;
      std::string mystatus = myentity->get_status( entity_info);
      std::string info_str = CORBA::string_dup( entity_info);

      if ( (mystatus.compare("SLEEPING") == 0) && ( info_str.compare("STATUS_INFO") == 0 ))
         {
          CPPUNIT_ASSERT (1); /*test ok*/
     }
      else
        {
          CPPUNIT_ASSERT (0); /* test not ok */
    }
          

      }

    void TestEntityStatus::get_status2()
      {
      CORBA::String_var entity_info2;
      std::string mystatus2 = myentity2->get_status( entity_info2);
      std::string info_str2 = CORBA::string_dup( entity_info2);

      if ( (mystatus2.compare("ENT_STAT3") == 0) && ( info_str2.compare("ENT_INFO") == 0 ))
         {
          CPPUNIT_ASSERT (1); /*test ok*/
     }
      else
        {
          CPPUNIT_ASSERT (0); /* test not ok */
    }

      }
      
      void TestEntityStatus::get_default_status()
      {
      CORBA::String_var entity_info;
      std::string mystatus = myentity4->get_status( entity_info);
      std::string info_str = CORBA::string_dup( entity_info);
      if ( (mystatus.compare("UNKNOWN") == 0) )
         {
          CPPUNIT_ASSERT (1); /*test ok*/
     }
      else
        {
          CPPUNIT_ASSERT (0); /* test not ok */
    }
          

      }
      
      void TestEntityStatus::check_propertyOfEntity3()
      {
       CORBA::Any* return_any_val;
       return_any_val = myentity3-> get_property_value("Property1");
           const char* str_out_sys ;
           (*return_any_val) >>=  str_out_sys;
       
       string tmp = "";
       tmp.append(str_out_sys);
           if(tmp!="valueprop1") CPPUNIT_ASSERT (0);
           CPPUNIT_ASSERT (1);
      }
      
      void TestEntityStatus::check_propertyOfEntity4()
      {
           CORBA::Any* return_any_val;
       return_any_val = myentity4-> get_property_value("Property1");
           const char* str_out_sys ;
           (*return_any_val) >>=  str_out_sys;
       
       string tmp = "";
       tmp.append(str_out_sys);
           if(tmp!="valueprop1") CPPUNIT_ASSERT (0);

       return_any_val = myentity4-> get_property_value("Property2");
       short value2;
       (*return_any_val) >>=  value2;
       if(value2!=2) CPPUNIT_ASSERT (0);

       return_any_val = myentity4-> get_property_value("Property3");
       short value3;
       (*return_any_val) >>=  value3;
       if(value3!=3) CPPUNIT_ASSERT (0);

           CPPUNIT_ASSERT (1);
      }
      
      void TestEntityStatus::check_propertyOfEntity5()
      {
           CORBA::Any* return_any_val;
       return_any_val = myentity5-> get_property_value("Property1");
           const char* str_out_sys ;
           (*return_any_val) >>=  str_out_sys;
       
       string tmp = "";
       tmp.append(str_out_sys);
           if(tmp!="valueprop1") CPPUNIT_ASSERT (0);

       return_any_val = myentity5-> get_property_value("Property2");
       short value2;
       (*return_any_val) >>=  value2;
       if(value2!=6) CPPUNIT_ASSERT (0);

       return_any_val = myentity5-> get_property_value("Property3");
       short value3;
       (*return_any_val) >>=  value3;
       if(value3!=5) CPPUNIT_ASSERT (0);

       CPPUNIT_ASSERT (1);
      }

  }

}

