/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <limits>

#include "testentitystatuschange/TestEntityStatusChange.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"

#include <platformftproxy/InitUtils.hpp>
#include <platformftproxy/SystemProxy.hpp>
#include <SystemMngt/platformftproxy/AccessorLookupService.hpp>
#include <SystemMngt/platformftproxy/DefaultAccessorLookupService.hpp>
#include <SystemMngt/platformlibrary/Iterator.hpp>

#include <FaultTolerance/ftreplicationmanager/InitUtils.hpp>
#include <FaultTolerance/idllib/FT.stub.hpp>
#include <platformftaccessorlibrary/SystemAccessor_impl.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtSupervisionObserver.stub.hpp>

#include "testentitystatuschange/SupervisionObserver_impl.hpp"

#include "SystemMngt/platformvaluetypes/ApplicationStatusChange_impl.hpp"

using namespace std;

using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestEntityStatusChange);

namespace Cdmw
{
    namespace PlatformMngt
    {
        TestEntityStatusChange::TestEntityStatusChange () throw ()
        {


        }

        TestEntityStatusChange::~TestEntityStatusChange () throw ()
        {
        }


        CORBA::ORB_var TestEntityStatusChange::m_orb;

        CdmwPlatformMngt::System_var TestEntityStatusChange::m_system;
        CdmwPlatformMngt::Application_var TestEntityStatusChange::m_app;
        CdmwPlatformMngt::Host_var TestEntityStatusChange::m_host;
        CdmwPlatformMngt::Process_var TestEntityStatusChange::m_proc;
        CdmwPlatformMngt::CcmComponent_var TestEntityStatusChange::m_component;
        CdmwPlatformMngt::FTGroup_var TestEntityStatusChange::m_ftgroup;
        CdmwPlatformMngt::LBGroup_var TestEntityStatusChange::m_lbgroup;
        
        CdmwPlatformMngt::Entity_var TestEntityStatusChange::m_sys_entity;
        CdmwPlatformMngt::Entity_var TestEntityStatusChange::m_app_entity;
        CdmwPlatformMngt::Entity_var TestEntityStatusChange::m_proc_entity;
        CdmwPlatformMngt::Entity_var TestEntityStatusChange::m_host_entity;
        CdmwPlatformMngt::Entity_var TestEntityStatusChange::m_component_entity;
        CdmwPlatformMngt::Entity_var TestEntityStatusChange::m_ftgroup_entity;
        CdmwPlatformMngt::Entity_var TestEntityStatusChange::m_lbgroup_entity;
        
        CORBA::ULong TestEntityStatusChange::size1;
        
        bool TestEntityStatusChange::m_is_one_shot_initialized=false;
        SupervisionObserver_impl* TestEntityStatusChange::m_observer_impl= new SupervisionObserver_impl();;
        CdmwPlatformMngt::SupervisionObserver_var TestEntityStatusChange::m_observer;

        void TestEntityStatusChange::setUp()
        {
          TestEntityStatusChange::one_shot_init();
        }

        void TestEntityStatusChange::one_shot_init()
        {
        if(!m_is_one_shot_initialized)
        {
          m_is_one_shot_initialized = true;

        try
        {
        CORBA::ORB_var orb;
        orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager:: instance ()->get_ORB ());

        PortableServer::POA_var poa = PortableServer::POA::_duplicate (Cdmw::TestUtils::CORBATestManager:: instance ()->get_POA ());
        PortableServer::POAManager_var poaMgr = PortableServer::POAManager::_duplicate(poa->the_POAManager());
        poaMgr->activate();

        CORBA::ValueFactoryBase_var factory,old_factory;
        factory = new EntityStatusChangeFactory();
        old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/EntityStatusChange:1.0",factory.in() );
        
        CdmwPlatformMngt::SupervisionObserver_var observer;
        observer = m_observer_impl->_this();

        CORBA::Object_var objref;

        //define the corbaloc
        const char *corbaloc_str = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";

        objref = orb->string_to_object(corbaloc_str);

        //resolve the System_ptr
        m_system = CdmwPlatformMngt::System::_narrow(objref.in() );

        if (CORBA::is_nil (m_system.in ()))
          {
           // TEST_INFO ("Error in narrowing CdmwPlatformMngt::System");
            exit (-1);
          }

        char *hostname = getenv ("HOSTNAME");

        m_host = m_system->get_host (hostname);
        m_app = m_system -> get_application("HELLO_WORLD");
        m_proc = m_host->get_process("HelloProcess3", "HELLO_WORLD");
        m_ftgroup = m_app->get_ft_group("FTGroup");
        m_lbgroup = m_app->get_lb_group("LBGroup");
        m_component = m_proc->get_component("Comp1");
        
        // Register observer to Supervision
        std::string observer_name("ObserverName");
        
        //m_system -> register_observer(observer_name.c_str(), observer.in());
        m_sys_entity = m_system->get_entity("SystemEntity");  
        m_app_entity = m_app->get_entity("ApplicationEntity");
        m_host_entity = m_host->get_entity("HostEntity");
        m_proc_entity = m_proc->get_entity("ProcessEntity");
        m_component_entity = m_component->get_entity("ComponentEntity");
        m_ftgroup_entity = m_ftgroup->get_entity("FTGroupEntity");
        m_lbgroup_entity = m_lbgroup->get_entity("LBGroupEntity");
        
        m_sys_entity -> register_observer(observer_name.c_str(),observer.in());
        m_app_entity -> register_observer(observer_name.c_str(),observer.in());
        m_proc_entity -> register_observer(observer_name.c_str(),observer.in());
        m_host_entity -> register_observer(observer_name.c_str(),observer.in());
        m_component_entity -> register_observer(observer_name.c_str(),observer.in());
        m_ftgroup_entity -> register_observer(observer_name.c_str(),observer.in());
        m_lbgroup_entity -> register_observer(observer_name.c_str(),observer.in());


        if (CORBA::is_nil (m_host.in ()))
                {
                    TEST_INFO ("Error in return of get_host function from System to Host");
                    exit (-1);
                }
        }
        catch (const Cdmw::Exception & ex)
          {
          std::cerr << ex.what () << std::endl;
          exit (-1);
          }
        catch (const CORBA::Exception & ex)
          {
          std::cerr << ex << std::endl;
          exit (-1);
          }
        catch (...)
          {
          std::cerr << "unexpected exception" << std::endl;
          exit (-1);
          }
        }
        }
        
        /*
         * Test the set_status operation on System Entity
         */
        void TestEntityStatusChange::test_set_system_entity_status_change()
        {
            Entity_var sys_entity = m_system->get_entity("SystemEntity");
            m_sys_entity->set_status("NewEntitySystemStatus","NewSystemEntityMode");  
        }

        /*
         * Test the set_status operation on Application Entity
         */
        void TestEntityStatusChange::test_set_application_entity_status_change()
        {
            Entity_var app_entity = m_app->get_entity("ApplicationEntity");
            m_app_entity->set_status("NewEntityApplicationStatus","NewApplicationEntityMode");
        }
          
        /*
         * Test the set_status operation on Host Entity
         */
        void TestEntityStatusChange::test_set_host_entity_status_change()
        {
            Entity_var host_entity = m_host->get_entity("HostEntity");
            m_host_entity->set_status("NewEntityHostStatus","NewHostEntityMode");
        }

        /*
         * Test the set_status operation on Process Entity
         */
        void TestEntityStatusChange::test_set_process_entity_status_change()
        {
            Entity_var proc_entity = m_proc->get_entity("ProcessEntity");
            m_proc_entity->set_status("NewProcessSystemStatus","NewProcessEntityMode");
        }
         
        void TestEntityStatusChange::test_set_component_entity_status_change()
        {
            Entity_var comp_entity = m_component->get_entity("ComponentEntity");
            m_component_entity->set_status("NewComponentSystemStatus","NewComponentEntityMode");
        }

        void TestEntityStatusChange::test_set_ftgroup_entity_status_change()
        {
            Entity_var ft_entity = m_ftgroup->get_entity("FTGroupEntity");
            m_ftgroup_entity->set_status("NewFTGroupSystemStatus","NewFTGroupEntityMode");
        }

        void TestEntityStatusChange::test_set_lbgroup_entity_status_change()
        {
            Entity_var lb_entity = m_lbgroup->get_entity("LBGroupEntity");
            m_lbgroup_entity->set_status("NewLBGroupSystemStatus","NewLBGroupEntityMode");
        }
       
        void TestEntityStatusChange::test_notify()
        {
            m_observer_impl->set_notification_done(false);

            int rtries=0;
            while (!m_observer_impl->is_notification_done()){
                Cdmw::OsSupport::OS::sleep (1000);
                rtries++;
                if (rtries>2)
                    break;
            }
            CPPUNIT_ASSERT(m_observer_impl->test_ok());
        }

    }               /* End namespace PlatformMngt */
}               /* End namespace Cdmw */
