/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_TESTPROPERTYCHANGE_HPP
#define INCL_PLATFORMMNGT_TESTPROPERTYCHANGE_HPP

#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/common/System.hpp"
#include <Foundation/orbsupport/CORBA.hpp>

#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
//#include "idllib/CdmwPlatformMngtApplication.stub.hpp"
#include <SystemMngt/idllib/CdmwPlatformMngtSupervisionObserver.stub.hpp>
#include "SupervisionObserver_impl.hpp"
#include "testpropertychange/OneShotInitFactory.hpp"

namespace Cdmw
{
  namespace PlatformMngt
  {

/**
*Purpose:
*<p>
*
*Features:
*<p> Thread safe
*
*
*/
    class TestPropertyChange:public CppUnit::TestFixture
    {

    public:

    /**
        * Purpose:
        * <p> Constructor
        *
        */
      TestPropertyChange () throw ();

    /**
        * Purpose:
        * <p> Destructor
        *
        */
      virtual ~ TestPropertyChange () throw ();

      void setUp ();

      CdmwPlatformMngt::System_var m_system;
      CdmwPlatformMngt::SupervisionObserver_var m_observer;

      SupervisionObserver_impl *m_observer_impl;
      OneShotInitFactory*  one_shot_init_factory;

    /**
        * Purpose:
        * <p> Call from the testable object
        *
        */
      virtual bool notification_received();
 
      virtual void test_register_all_properties_observer ();
      virtual void test_unregister_all_properties_observer ();
      virtual void test_calldefine_attribute ();
      virtual void test_register_property_observer ();
      virtual void test_unregister_property_observer ();
      virtual void test_unregister_property_observer_2_props ();
      virtual void test_calldefine_attribute_no_notify ();
      virtual void test_register_all_properties_observer_already_exist();   
      virtual void test_register_property_observer_already_exist();
      virtual void test_unregister_property_observer_2_props_just_done();
      virtual void test_unregister_all_properties_observer_just_done();
      virtual void test_calldefine_attribute_one_reception();
      virtual void test_calldefine_attributes ();
      virtual void test_calldelete_attribute();
      virtual void test_calldelete_attributes();
      virtual void test_calldefine_attribute_with_mode();
      virtual void test_calldefine_attributes_with_mode();
      virtual void test_register_all_properties_observer_deeply(); 
      virtual void the_notification_is_received();
      virtual void test_calldefine_attribute_to_all_children();

    protected:


    private:

    /**
        * Purpose:
        * <p> Assignment operator
        *
        */
        TestPropertyChange & operator= (const TestPropertyChange & rhs)
    throw ();
    /**
        * Purpose:
        * <p>  Copy constructor
        *
        */
        TestPropertyChange (const TestPropertyChange & rhs) throw ();


    
        CPPUNIT_TEST_SUITE (TestPropertyChange);
        // test register for all
    CPPUNIT_TEST (test_register_all_properties_observer);
        CPPUNIT_TEST (test_calldefine_attribute);
        CPPUNIT_TEST (test_unregister_all_properties_observer);
        CPPUNIT_TEST (test_calldefine_attribute_no_notify);
        // test register for properties change
        CPPUNIT_TEST (test_register_property_observer);
        CPPUNIT_TEST (test_calldefine_attribute);
        CPPUNIT_TEST (test_unregister_property_observer);
        CPPUNIT_TEST (test_calldefine_attribute_one_reception);
        CPPUNIT_TEST (test_unregister_property_observer_2_props);
        CPPUNIT_TEST (test_calldefine_attribute_no_notify);
        // testing other use cases
        CPPUNIT_TEST (test_register_all_properties_observer);
    // test another registration
    CPPUNIT_TEST (test_register_all_properties_observer_already_exist);
    // test multiple de-registration
    CPPUNIT_TEST (test_unregister_all_properties_observer);
    CPPUNIT_TEST (test_unregister_all_properties_observer_just_done);
    // test multiple registration
    CPPUNIT_TEST (test_register_property_observer);
    CPPUNIT_TEST (test_register_property_observer_already_exist);
    // test multiple de-registration
    CPPUNIT_TEST (test_unregister_property_observer_2_props);
    CPPUNIT_TEST (test_unregister_property_observer_2_props_just_done);
        
        CPPUNIT_TEST (test_register_property_observer); 
        CPPUNIT_TEST (test_calldefine_attributes);
        CPPUNIT_TEST (test_calldelete_attribute);
    
    CPPUNIT_TEST (test_calldefine_attributes);
    CPPUNIT_TEST (test_calldelete_attributes);
    CPPUNIT_TEST (test_calldefine_attribute_with_mode);
    CPPUNIT_TEST (test_calldefine_attributes_with_mode);
    CPPUNIT_TEST (test_register_all_properties_observer_deeply);
        CPPUNIT_TEST (test_calldefine_attribute_to_all_children); 
        CPPUNIT_TEST (the_notification_is_received);
        CPPUNIT_TEST_SUITE_END ();
    };

  }             /* End namespace PlatformMngt */
}               /* End namespace Cdmw */
#endif              /* INCL_PLATFORMMNGT_TESTPROPERTYCHANGE_HPP */
