/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <limits>

#include "testpropertychange/TestPropertyChange.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include <SystemMngt/platformlibrary/Iterator.hpp>

#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"


#include "testpropertychange/OneShotInitFactory.hpp"

using namespace std;

using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestPropertyChange);

namespace Cdmw
{
  namespace PlatformMngt
  {
    TestPropertyChange::TestPropertyChange () throw ()
    {


    }

    TestPropertyChange::~TestPropertyChange () throw ()
    {
    }

    void TestPropertyChange::setUp ()
    {
       one_shot_init_factory =
    OneShotInitFactory::get_instance ();
    
         m_observer_impl = one_shot_init_factory->get_observer_impl();
         m_system    = one_shot_init_factory->get_system();
         m_observer  = one_shot_init_factory->get_observer();   
    }

    void TestPropertyChange::test_register_all_properties_observer ()
    {

      std::string observer_name = "observer_for_system_all";
      m_system->register_all_properties_observer (observer_name.c_str (),
                          m_observer.in (), false);
      CPPUNIT_ASSERT (true);
    }
    
    void TestPropertyChange::test_unregister_all_properties_observer ()
    {

      std::string observer_name = "observer_for_system_all";
      m_system->unregister_all_properties_observer (observer_name.c_str ());
      CPPUNIT_ASSERT (true);
    }
 
    void TestPropertyChange::test_unregister_all_properties_observer_just_done ()
    {
        
      std::string observer_name = "observer_for_system_all";
      try
      { 
      m_system->unregister_all_properties_observer (observer_name.c_str ());
      CPPUNIT_ASSERT (true);
      }
      catch(const CdmwPlatformMngt::ObserverNotFound& e)
      {
      std::cerr << e._name()<<std::endl;
          CPPUNIT_ASSERT (true);
          return;
      }
      catch (...)
      {
        std::cerr <<"<<< e._name()"<<std::endl;
      }
      CPPUNIT_ASSERT (false);

    }
   

    void TestPropertyChange::test_register_all_properties_observer_already_exist ()
    {   
    try 
    {
            std::string observer_name = "observer_for_system_all";
            m_system->register_all_properties_observer (observer_name.c_str (),
                                                        m_observer.in (), false);
    }   
        catch (const CdmwPlatformMngt::ElementAlreadyExists& e) 
    {
        std::cerr << e._name() << std::endl;
        CPPUNIT_ASSERT (true);
        return; 
    }
    CPPUNIT_ASSERT (false);
    }

   void TestPropertyChange::test_register_property_observer_already_exist()
    {
    CosPropertyService::PropertyNames properties;
        try
        {
             std::string observer_name = "observer_for_system_SysProp";
         properties.length (2);
             properties[0] = "SysProp";
             properties[1] = "SysProp2";
             m_system->register_property_observer (observer_name.c_str (),
                                            m_observer.in (), properties);
    }
        catch (const CdmwPlatformMngt::ElementAlreadyExists& e)
        {
            CPPUNIT_ASSERT (true);
            return;
        }
        CPPUNIT_ASSERT (false);
    }

    void TestPropertyChange::test_register_property_observer ()
    {
      CosPropertyService::PropertyNames properties;
      std::string observer_name = "observer_for_system_SysProp";

      properties.length (2);
      properties[0] = "SysProp";
      properties[1] = "SysProp2";
      m_system->register_property_observer (observer_name.c_str (),
                        m_observer.in (), properties);
      CPPUNIT_ASSERT (true);
    }

    void TestPropertyChange::test_unregister_property_observer ()
    {
      CosPropertyService::PropertyNames properties;
      std::string observer_name = "observer_for_system_SysProp";

      properties.length (1);
      properties[0] = "SysProp";
      m_system->unregister_property_observer (observer_name.c_str (),
                          properties);
      CPPUNIT_ASSERT (true);
    }

    void TestPropertyChange::test_unregister_property_observer_2_props ()
    {
      CosPropertyService::PropertyNames properties;
      std::string observer_name = "observer_for_system_SysProp";

      properties.length (2);
      properties[0] = "SysProp";
      properties[1] = "SysProp2";
      m_system->unregister_property_observer (observer_name.c_str (),
                          properties);
      CPPUNIT_ASSERT (true);
    }

    void TestPropertyChange::test_unregister_property_observer_2_props_just_done ()
    {
      CosPropertyService::PropertyNames properties;
      std::string observer_name = "observer_for_system_SysProp";
      try 
      { 
      properties.length (2);
      properties[0] = "SysProp";
      properties[1] = "SysProp2";
      m_system->unregister_property_observer (observer_name.c_str (),
                                               properties);
      }
      catch(const CdmwPlatformMngt::ObserverNotFound& )
      {
          CPPUNIT_ASSERT (true);
          return;
      }
      CPPUNIT_ASSERT (false);
    }


    void TestPropertyChange::test_calldefine_attribute ()
    {
      CORBA::Any sys_value;
      int value = 10;
      sys_value <<= value;
      m_observer_impl->set_notification_done (false);
      m_observer_impl->set_test_ok (false);
      m_system->define_property ("SysProp", sys_value);
      CPPUNIT_ASSERT (notification_received());
      m_observer_impl->set_notification_done (false);
      m_observer_impl->set_test_ok (false);
      m_system->define_property ("SysProp2", sys_value);
      CPPUNIT_ASSERT (notification_received());
    }

    void TestPropertyChange::test_calldefine_attributes ()
    {
      CosPropertyService::Properties properties;
      CORBA::Any sys_value;
      int value = 10;
      sys_value <<= value;
      properties.length(2);
      properties[0].property_name =  "SysProp";
      properties[1].property_name =  "SysProp2";    
      properties[0].property_value = sys_value;
      properties[1].property_value = sys_value;
      m_observer_impl->set_notification_done (false);
      m_observer_impl->set_test_ok (false);
      m_system->define_properties (properties);
      CPPUNIT_ASSERT (notification_received());
      //m_observer_impl->set_notification_done (false);
      //m_observer_impl->set_test_ok (false);
      CPPUNIT_ASSERT (notification_received());
    }
  
    void TestPropertyChange::test_calldefine_attribute_one_reception()
    {
      CORBA::Any sys_value;
      int value = 10;
      sys_value <<= value;
      m_observer_impl->set_notification_done (false);
      m_observer_impl->set_test_ok (false);
      m_system->define_property ("SysProp", sys_value);
      CPPUNIT_ASSERT (!notification_received());
      m_observer_impl->set_notification_done (false);
      m_observer_impl->set_test_ok (false);
      m_system->define_property ("SysProp2", sys_value);
      CPPUNIT_ASSERT (notification_received());
    }

    void TestPropertyChange::test_calldefine_attribute_no_notify ()
    {
      CORBA::Any sys_value;
      int value = 10;
      sys_value <<= value;
      m_observer_impl->set_notification_done (false);
      m_observer_impl->set_test_ok (false);
      m_system->define_property ("SysProp", sys_value);
      CPPUNIT_ASSERT (!notification_received());
      m_observer_impl->set_notification_done (false);
      m_observer_impl->set_test_ok (false);
      m_system->define_property ("SysProp2", sys_value);
      CPPUNIT_ASSERT (!notification_received());
    }
 
    /*wait notification from cardamom */ 
    bool TestPropertyChange::notification_received()
    {
     bool bool_ret = false;
     int rtries = 0;
     while (!m_observer_impl->is_notification_done ())
        {
          std::cerr << " waiting for notification" << std::endl;
          Cdmw::OsSupport::OS::sleep (1000);
          rtries++;
          if (rtries > 2)
            break;
        }
     if (m_observer_impl->test_ok()) 
         bool_ret=true;
     return bool_ret;
    }
   
    void TestPropertyChange::test_calldelete_attribute()
    {
      CORBA::Any sys_value;
      int value = 10;
      sys_value <<= value;
      m_observer_impl->set_notification_done (false);
      m_observer_impl->set_test_ok (false);
      Cdmw::OsSupport::OS::sleep (2000);
      m_system->delete_property ("SysProp");
      CPPUNIT_ASSERT (notification_received());
      m_observer_impl->set_notification_done (false);
      m_observer_impl->set_test_ok (false);
      m_system->delete_property ("SysProp2");
      CPPUNIT_ASSERT (notification_received());
    }
 
    void TestPropertyChange::test_calldelete_attributes()
    {
      CosPropertyService::PropertyNames properties;
      properties.length (2);
      properties[0] = "SysProp";
      properties[1] = "SysProp2";
      m_system->delete_properties (properties);
      CPPUNIT_ASSERT (notification_received());
      CPPUNIT_ASSERT (notification_received());
    }

    void TestPropertyChange::test_calldefine_attribute_with_mode ()
    {
      CosPropertyService::PropertyModeType prop_mode_type;
      prop_mode_type = CosPropertyService::normal;
      CORBA::Any sys_value;
      int value = 10;
      sys_value <<= value;
      m_observer_impl->set_notification_done (false);
      m_observer_impl->set_test_ok (false);
      m_system->define_property_with_mode ("SysProp",sys_value, prop_mode_type);
      
      CPPUNIT_ASSERT (notification_received());
      m_observer_impl->set_notification_done (false);
      m_observer_impl->set_test_ok (false);
      m_system->define_property_with_mode ("SysProp2",sys_value, prop_mode_type);
      CPPUNIT_ASSERT (notification_received());
    }
     
    void TestPropertyChange::test_calldefine_attributes_with_mode ()
    {
  
      CosPropertyService::PropertyModeType prop_mode_type;
      prop_mode_type = CosPropertyService::normal;
       
      CORBA::Any sys_value;
      int value = 10;
      sys_value <<= value;
      CosPropertyService::PropertyDefs prop_defs;
      prop_defs.length(2L);
      
      CosPropertyService::PropertyDef prop_def;
      prop_def.property_mode = CosPropertyService::normal;
      prop_def.property_name = "SysProp2";
      prop_def.property_value = sys_value;
      prop_defs[0L] = prop_def;
            
      prop_def.property_mode = CosPropertyService::read_only;
      prop_def.property_name = "SysProp";
      prop_def.property_value = sys_value;
      
      prop_defs[1L] = prop_def;
      
      m_observer_impl->set_notification_done (false);
      m_observer_impl->set_test_ok (false);
      
      m_system->define_properties_with_modes (prop_defs);
      
      CPPUNIT_ASSERT (notification_received());
      CPPUNIT_ASSERT (notification_received());
    }
 
    /*wait notification from cardamom */
    void TestPropertyChange::the_notification_is_received()
    {
     //bool bool_ret = false;
     int rtries = 0;
     while (!m_observer_impl->is_notification_done ())
        {
          std::cerr << " waiting for notification" << std::endl;
          Cdmw::OsSupport::OS::sleep (1000);
          rtries++;
          if (rtries > 2)
            break;
        }
     if (m_observer_impl->test_ok())
     CPPUNIT_ASSERT(true);
     else
     CPPUNIT_ASSERT(false);
     }  

    void TestPropertyChange::test_register_all_properties_observer_deeply()
    {
        std::string observer_name = "observer_for_system_all_deeply";
        m_system->register_all_properties_observer (observer_name.c_str (),
                                                  m_observer.in (), true);
        CPPUNIT_ASSERT (true);
 
    }

    void  TestPropertyChange::test_calldefine_attribute_to_all_children()       
    {
        CORBA::Any sys_value;
        int value = 10;
        sys_value <<= value;
        m_observer_impl->set_notification_done (false);
        m_observer_impl->set_test_ok (false);

        CdmwPlatformMngt::Children_var children = 0;
        children = m_system->get_children();

        if (children!=0)
        for (CORBA::ULong i=0; i < children->length(); i++)
        {
                children[i]->define_property ("SysProp", sys_value);
        }
 
    }    


        
  }             /* End namespace PlatformMngt */
}               /* End namespace Cdmw */





