/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_CCMCOMPONENT_ACCESSOR_IMPL_HPP
#define INCL_PLATFORMMNGT_CCMCOMPONENT_ACCESSOR_IMPL_HPP


#include <Foundation/common/Exception.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>

#include <SystemMngt/platformftaccessorlibrary/ManagedElementAccessor_impl.hpp>

#include <SystemMngt/idllib/CdmwFTSystMngtComponent.stub.hpp>
#include <SystemMngt/idllib/CdmwFTSystMngtComponent.skel.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtElementRepository.stub.hpp>
#include <SystemMngt/platformftaccessorlibrary/RWEntityContainerAccessor_impl.hpp>
#include <SystemMngt/platformftaccessorlibrary/RWServiceDefContainerAccessor_impl.hpp>


namespace Cdmw
{
  namespace PlatformMngt
  {

    class CcmComponentAccessor_impl:public virtual POA_CdmwFTSystMngt::
      CcmComponentAccessor, public ManagedElementAccessor_impl,
      public RWEntityContainerAccessor_impl
    {

    public:

      CcmComponentAccessor_impl ();
      CcmComponentAccessor_impl (CdmwPlatformMngt::
                                 ElementRepository_ptr element_repository)
        throw ();

      ~CcmComponentAccessor_impl ();

      CdmwPlatformMngt::
        CcmComponent_ptr get_ccmcomponent (const char *target_element_path)
        throw (CdmwPlatformMngt::ManagedElementNotFound);

      void set_indicator_relevant_status (const char *target_element_path,
                                          CORBA::
                                          Boolean indicator_relevant_status)
        throw (CORBA::SystemException);

      CORBA::
        ULong get_number_of_entities (const char *target_element_path)
        throw (CORBA::SystemException);

      void get_all_entity_names (const char *target_element_path, CORBA::ULong,
                                 CdmwPlatformMngt::ElementNames_out,
                                 CdmwPlatformMngt::
                                 ElementNamesIterator_out) throw (CORBA::
                                                                  SystemException);

      char *get_instantiation_id (const char *) throw (CORBA::SystemException,
                                                       CdmwPlatformMngt::
                                                       ManagedElementNotFound);

      char *get_deployment_plan (const char *) throw (CORBA::SystemException,
                                                      CdmwPlatformMngt::
                                                      ManagedElementNotFound);

      CORBA::Boolean get_created_dynamically (const char *) throw (CORBA::
                                                                     SystemException,
                                                                     CdmwPlatformMngt::
                                                                     ManagedElementNotFound);


      CdmwPlatformMngtBase::
        TimeStamp get_start_time (const char *) throw (CORBA::SystemException,
                                                       CdmwPlatformMngt::
                                                       ManagedElementNotFound);

      CdmwPlatformMngtBase::
        TimeStamp get_stop_time (const char *) throw (CORBA::SystemException,
                                                      CdmwPlatformMngt::
                                                      ManagedElementNotFound);

      CdmwPlatformMngt::
        RedundancyEltState get_redundancy_elt_state (const char *)
        throw (CORBA::SystemException,
               CdmwPlatformMngt::ManagedElementNotFound);

      CdmwPlatformMngt::
        RequestStatus initialise (const char *target_element_path,
                                  const CdmwPlatformMngtBase::
                                  StartupKind & start) throw (CORBA::
                                                              SystemException,
                                                              CdmwPlatformMngt::
                                                              IncompatibleStatus,
                                                              CdmwPlatformMngt::
                                                              ManagedElementNotFound);


      CORBA::Boolean get_entities (const char *target_element_path,
                                     const CdmwPlatformMngt::ElementNames &,
                                     CdmwPlatformMngt::
                                     Entities_out) throw (CORBA::
                                                          SystemException);

      //void get_all_entities( const char * target_element_path, CORBA::ULong how_many, CdmwPlatformMngt::Entities_out ent_out, CdmwPlatformMngt::EntitiesIterator_out ent_iter_out)  throw (CORBA::SystemException);


      void get_attribute (const char *target_element_path, const char *,
                          CORBA::Any_out,
                          CdmwPlatformMngt::Owner) throw (CORBA::SystemException,CdmwPlatformMngt::AttributeNotFound, CdmwPlatformMngt::ManagedElementNotFound );


      void set_attribute (const char *target_element_path, const char *,
                          const CORBA::Any &,
                          CdmwPlatformMngt::Owner) 
                          throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound, CdmwPlatformMngt::ReadOnlyAttribute, CdmwPlatformMngt::ManagedElementNotFound);


      CdmwPlatformMngt::
        ElementType get_element_type (const char *target_element_path)
        throw (CORBA::SystemException);


      /*CdmwPlatformMngt::Children *
        get_children (const char *target_element_path) throw (CORBA::
                                                              SystemException);*/


      CdmwPlatformMngt::ComponentSnapshot *
        get_snapshot (const char *target_element_path) throw (CORBA::
                                                              SystemException);


        CdmwPlatformMngt::
        ComponentStatus get_status (const char *target_element_path,
                                    CORBA::String_out) throw (CORBA::
                                                              SystemException);


      char *instantiation_id (const char *target_element_path) throw (CORBA::
                                                                      SystemException);

        CdmwPlatformMngtBase::
        TimeStamp start_time (const char *target_element_path) throw (CORBA::
                                                                      SystemException);


        CdmwPlatformMngtBase::
        TimeStamp stop_time (const char *target_element_path) throw (CORBA::
                                                                     SystemException);


      char *get_component_server_name (const char *target_element_path)
        throw (CORBA::SystemException);


        CORBA::
        Boolean get_indicator_relevant_status (const char *target_element_path)
        throw (CORBA::SystemException);

        CdmwPlatformMngt::RedundancyGroup *
        get_component_group (const char *target_element_path) throw (CORBA::
                                                                     SystemException);

      void remove_entity (const char *target_element_path,
                          const char *) throw (CORBA::SystemException);

      char *deployment_plan (const char *target_element_path) throw (CORBA::
                                                                     SystemException);

    };

/* try to instance it, only to check compilation ... */
    //CcmComponentAccessor_impl   *ccm_comp_accessor = new CcmComponentAccessor_impl() ; 

  }                             // End namespace PlatformMngt
}                               // End namespace Cdmw

#endif                          //INCL_PLATFORMMNGT_CCMCOMPONENT_ACCESSOR_IMPL_HPP
