/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_FTSYSTMNGT_NOTIFIER_ACCESSOR_IMPL_HPP
#define INCL_FTSYSTMNGT_NOTIFIER_ACCESSOR_IMPL_HPP

#include <SystemMngt/idllib/CdmwFTSystMngtPeriodicNotifier.skel.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtPeriodicNotifier.stub.hpp>


namespace Cdmw
{ // Begin namespace Cdmw
  
namespace PlatformMngt
{ // Begin namespace PlatformMngt 
   
namespace Periodic
{ // Begin namespace Periodic 

class NotifierAccessor_impl: public virtual POA_CdmwFTSystMngt::Periodic::NotifierAccessor,
                             public virtual PortableServer::RefCountServantBase
{

public:
                
    /**
    * Purpose:
    * <p> Constructor.
    */
     NotifierAccessor_impl(CdmwPlatformMngt::Periodic::Notifier_ptr periodic_notifier);

    /**
    * Purpose:
    * <p> Destructor.
    */
    ~NotifierAccessor_impl() throw();
    
    
    /**
    *Purpose:
    *<p>
    * Implements the
    * IDL:thalesgroup.com/CdmwFTSystMngt/Periodic/NotifierAccessor/shut_down:1.0
    * operation
    */     
    void shut_down (const char * target_element_path)
        throw (CORBA::SystemException, 
               CdmwPlatformMngt::ManagedElementNotFound);


    /**
    *Purpose:
    *<p>
    * Implements the
    * IDL:thalesgroup.com/CdmwFTSystMngt/Periodic/NotifierAccessor/register_observer:1.0
    * operation
    */ 
    void register_observer (const char * target_element_path, const char * observer_name, 
                            const CosPropertyService::PropertyNames & property_names, 
                            CdmwPlatformMngt::Periodic::Observer_ptr observer_ref,
                            TimeBase::TimeT period)
        throw (CORBA::SystemException,
               CdmwPlatformMngt::ManagedElementNotFound,
               CdmwPlatformMngt::ElementAlreadyExists);
     
              
    /**
    *Purpose:
    *<p>
    * Implements the
    * IDL:thalesgroup.com/CdmwFTSystMngt/Periodic/NotifierAccessor/register_observer_group:1.0
    * operation
    */  
    void register_observer_group (const char * target_element_path, const char * observer_name, 
                                  const CosPropertyService::PropertyNames & property_names,
                                  const char * m_miop_group, TimeBase::TimeT period)
        throw (CORBA::SystemException,
               CdmwPlatformMngt::ManagedElementNotFound,
               CdmwPlatformMngt::ElementAlreadyExists);  
        
                
    /**
    *Purpose:
    *<p>
    * Implements the
    * IDL:thalesgroup.com/CdmwFTSystMngt/Periodic/NotifierAccessor/unregister_observer:1.0
    * operation
    */                 
    CdmwPlatformMngt::Periodic::Observer_ptr unregister_observer (const char * target_element_path, 
                                                                  const char * observer_name)
        throw (CORBA::SystemException,
               CdmwPlatformMngt::ObserverNotFound);
           
    /**
    *Purpose:
    *<p>
    * Implements the
    * IDL:thalesgroup.com/CdmwFTSystMngt/Periodic/NotifierAccessor/unregister_observer_group:1.0
    * operation
    */           
    char * unregister_observer_group (const char * target_element_path,
                                      const char * observer_name)
        throw (CORBA::SystemException, 
               CdmwPlatformMngt::ObserverNotFound);
                    
private:

    /**
     * The system managed element
     */
     CdmwPlatformMngt::Periodic::Notifier_var m_periodic_notifier;

};


} // End namespace Periodic
} // End namespace PlatformMngt
} // End namespace Cdmw

#endif //INCL_FTSYSTMNGT_NOTIFIER_ACCESSOR_IMPL_HPP
