/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_REDUNDANCYGROUP_ACCESSOR_IMPL_HPP
#define INCL_PLATFORMMNGT_REDUNDANCYGROUP_ACCESSOR_IMPL_HPP


#include <Foundation/common/Exception.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>

#include <SystemMngt/platformftaccessorlibrary/ManagedElementAccessor_impl.hpp>

#include <SystemMngt/idllib/CdmwFTSystMngtComponent.stub.hpp>
#include <SystemMngt/idllib/CdmwFTSystMngtComponent.skel.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtElementRepository.stub.hpp>
#include <SystemMngt/platformftaccessorlibrary/RWEntityContainerAccessor_impl.hpp>
#include <SystemMngt/platformftaccessorlibrary/RedundancyGroupAccessor_impl.hpp>


namespace Cdmw
{
  namespace PlatformMngt
  {

    class RedundancyGroupAccessor_impl:public virtual POA_CdmwFTSystMngt::RedundancyGroupAccessor, public ManagedElementAccessor_impl,
      public RWEntityContainerAccessor_impl
    {

    public:

      //RedundancyGroupAccessor_impl();

      RedundancyGroupAccessor_impl 
         ( CdmwPlatformMngt::ElementRepository_ptr element_repository ) 
         throw();

      ~RedundancyGroupAccessor_impl ();

      CdmwPlatformMngt::
        RedundancyGroup_ptr get_redundancy_group (const char
                                                  *target_element_path)
        throw (CdmwPlatformMngt::ManagedElementNotFound);

      CORBA::
        ULong get_number_of_entities (const char *target_element_path)
        throw (CORBA::SystemException);

      char *get_group_domain_id (const char *) throw (CORBA::SystemException);


      void get_all_entity_names (const char *target_element_path, CORBA::ULong,
                                 CdmwPlatformMngt::ElementNames_out,
                                 CdmwPlatformMngt::
                                 ElementNamesIterator_out) throw (CORBA::
                                                                  SystemException);



      CORBA::Boolean get_entities (const char *target_element_path,
                                     const CdmwPlatformMngt::ElementNames &,
                                     CdmwPlatformMngt::
                                     Entities_out) throw (CORBA::
                                                          SystemException);

      void get_all_entities (const char *target_element_path, CORBA::ULong,
                             CdmwPlatformMngt::Entities_out,
                             CdmwPlatformMngt::
                             EntitiesIterator_out) throw (CORBA::
                                                          SystemException);



      void remove_entity (const char *target_element_path,
                          const char *) throw (CORBA::SystemException);

      void get_attribute (const char *target_element_path, const char *,
                          CORBA::Any_out,
                          CdmwPlatformMngt::Owner)
       throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound, CdmwPlatformMngt::ManagedElementNotFound);
      
      void set_attribute (const char *target_element_path, const char *,
                          const CORBA::Any &,
                          CdmwPlatformMngt::Owner)
      throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound, CdmwPlatformMngt::ReadOnlyAttribute, CdmwPlatformMngt::ManagedElementNotFound);
       
      CdmwPlatformMngt::
        ElementType get_element_type (const char *target_element_path)
        throw (CORBA::SystemException);


      /*CdmwPlatformMngt::Children *
        get_children (const char *target_element_path) throw (CORBA::
                                                              SystemException);*/

      CORBA::
        Long get_minimum_size (const char *target_element_path) throw (CORBA::
                                                                       SystemException);



      
      CdmwPlatformMngt::
        TechnicalStatus get_status (const char *target_element_path,
                                    CORBA::String_out) throw (CORBA::
                                                              SystemException); 
      
      void set_minimum_size (const char *target_element_path,
                             CORBA::Long) throw (CORBA::SystemException);

      void start_up (const char *target_element_path) throw (CORBA::
                                                             SystemException);

      void shut_down (const char *target_element_path) throw (CORBA::
                                                              SystemException);

      void insert (const char *target_element_path,
                   CdmwPlatformMngt::ManagedElement *) throw (CORBA::
                                                              SystemException);


      void get_all_components (const char *, CORBA::ULong,
                               CdmwPlatformMngt::ElementPaths_out,
                               CdmwPlatformMngt::
                               ComponentsIterator_out) throw (CORBA::
                                                              SystemException);



      char *get_component (const char *target_element_path,
                           const char *component_name,
                           const char *host_name) throw (CORBA::SystemException,
                                                         CdmwPlatformMngt::
                                                         ComponentNotFound,
                                                         CdmwPlatformMngt::
                                                         ManagedElementNotFound);

      CORBA::
        ULong get_number_of_components (const char *target_element_path)
        throw (CORBA::SystemException);

      void _cxx_delete (const char *target_element_path,
                        CdmwPlatformMngt::ManagedElement *) throw (CORBA::
                                                                   SystemException);


    };

/* try to instance it, only to check compilation ... */
    //RedundancyGroupAccessor_impl   *redu_group_accessor = new RedundancyGroupAccessor_impl() ; 

  }                             // End namespace PlatformMngt
}                               // End namespace Cdmw

#endif                          //INCL_PLATFORMMNGT_REDUNDANCYGROUP_ACCESSOR_IMPL_HPP
