/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"

#include "platformftaccessorlibrary/ApplicationAccessor_impl.hpp"

#include "SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtApplication.stub.hpp"

#include "SystemMngt/tools/ManagedElementKey.hpp"

#include "SystemMngt/platformlibrary/ElementPathsIterator_impl.hpp"

#include "SystemMngt/tools/ApplicationIssuerManager.hpp"

namespace Cdmw
{
namespace PlatformMngt
{

// ----------------------------------------------------------------------
// ApplicationAccessor_impl class.
// ----------------------------------------------------------------------

ApplicationAccessor_impl::ApplicationAccessor_impl(
    CdmwPlatformMngt::ElementRepository_ptr element_repository)
    throw()
    : ManagedElementAccessor_impl(element_repository),
      RWEntityContainerAccessor_impl(element_repository),
      RWServiceDefContainerAccessor_impl(element_repository)
{   
}

ApplicationAccessor_impl::~ApplicationAccessor_impl()
{
}


void ApplicationAccessor_impl::get_attribute( const char * target_element_path, const char * attr_name, CORBA::Any_out ret_attr, CdmwPlatformMngt::Owner flag  )  throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound, CdmwPlatformMngt::ManagedElementNotFound)
{

    CdmwPlatformMngt::Application_var application_obj = 
                        get_application (target_element_path);
                        
    application_obj->get_attribute(attr_name, ret_attr, flag);

}

void ApplicationAccessor_impl::set_attribute( const char * target_element_path, const char * attr_name, const CORBA::Any & attr, CdmwPlatformMngt::Owner flag  )  throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound, CdmwPlatformMngt::ReadOnlyAttribute, CdmwPlatformMngt::ManagedElementNotFound)
{

    CdmwPlatformMngt::Application_var application_obj = 
                        get_application (target_element_path);
                        
    application_obj->set_attribute(attr_name, attr, flag);

}


CdmwPlatformMngt::Application_ptr ApplicationAccessor_impl::get_application (
          const char* target_element_path)
              throw(CdmwPlatformMngt::ManagedElementNotFound)
{
    CdmwPlatformMngt::Application_var application_obj;
    
    try
    {
        CdmwPlatformMngt::ManagedElement_var element_obj =
              m_element_repository->get_element(target_element_path);
              
        application_obj = CdmwPlatformMngt::Application::_narrow (element_obj.in());
    }
    catch (const CdmwPlatformMngt::ElementPathInvalid&)
    {
        throw CdmwPlatformMngt::ManagedElementNotFound(target_element_path);
    }
    catch(const CORBA::SystemException&)
    {
        throw CdmwPlatformMngt::ManagedElementNotFound(target_element_path);
    }
    
    return application_obj._retn();                   
}


   
CORBA::Long ApplicationAccessor_impl::get_restart_attempt (
     const char* target_element_path)
throw(CdmwPlatformMngt::ManagedElementNotFound,
      CORBA::SystemException)
{
    CdmwPlatformMngt::Application_var application_obj = 
                        get_application (target_element_path);
                        
    return application_obj->restart_attempt();
}
        
CdmwPlatformMngt::ApplicationStatus ApplicationAccessor_impl::get_status(
    const char* target_element_path,
    CORBA::String_out status_info)
throw(CdmwPlatformMngt::ManagedElementNotFound,
      CORBA::SystemException)    
{
    CdmwPlatformMngt::Application_var application_obj = 
                        get_application (target_element_path);
                        
    return application_obj->get_status(status_info);
}

CdmwPlatformMngt::FunctioningMode ApplicationAccessor_impl::get_mode(
    const char* target_element_path,
    CORBA::String_out mode_info)
throw(CdmwPlatformMngt::ManagedElementNotFound,
      CORBA::SystemException)
{
    CdmwPlatformMngt::Application_var application_obj = 
                        get_application (target_element_path);
                        
    return application_obj->get_mode(mode_info);
}

CdmwPlatformMngt::ApplicationSnapshot* ApplicationAccessor_impl::get_snapshot(
    const char* target_element_path)
throw(CdmwPlatformMngt::ManagedElementNotFound,
      CORBA::SystemException)
{
    CdmwPlatformMngt::Application_var application_obj = 
                        get_application (target_element_path);
                        
    return application_obj->get_snapshot();
}


 CORBA::ULong ApplicationAccessor_impl::get_number_of_processes( const char* target_element_path) 
throw(CdmwPlatformMngt::ManagedElementNotFound,
      CORBA::SystemException)
{
    CdmwPlatformMngt::Application_var application_obj = 
                        get_application (target_element_path);
                        
    return application_obj->get_number_of_processes();
}

CdmwPlatformMngt::ElementPaths* ApplicationAccessor_impl::get_processes(
    const char* target_element_path,
    CORBA::ULong how_many,
    CdmwPlatformMngt::ElementPathsIterator_out processes_iterator)
throw(CdmwPlatformMngt::ManagedElementNotFound,
      CORBA::SystemException)
{
    CdmwPlatformMngt::Application_var application_obj = 
                        get_application (target_element_path);
                        
    // get the list of process references
    CdmwPlatformMngt::ProcessesIterator_var processesRemainingIt;
    
    CdmwPlatformMngt::Processes_var processes_seq =
                application_obj->get_processes (how_many, processesRemainingIt.out());
                
    // convert each process reference in element path
    
    // create sequence of element path
    CdmwPlatformMngt::ElementPaths_var elements =
                          new CdmwPlatformMngt::ElementPaths();
    
    // set element sequence length 
    size_t seq_len = processes_seq->length();
    elements->length(seq_len);
    
    // fill element sequence
    unsigned int count;
    for (count=0 ; count < seq_len ; count++)
    {
        // Store process path in sequence
        // path is allocated by element_path()
        elements[count] = (processes_seq[count])->element_path();                                           
    }
        
    // Get an iterator on the rest of the elements
    std::list<CdmwPlatformMngt::ElementPath_var> elementsRemaining;
    CdmwPlatformMngt::Process_var process;
    
    // fore each element contained in reference iterator
    while (processesRemainingIt->next_one (process.out()))
    { 
        elementsRemaining.push_back (process->element_path());
    }
    
    // create the path iterator
    CdmwPlatformMngt::ElementPathsIterator_var elementsRemainingIt =
              ElementPathsIterator_impl::create (elementsRemaining);

    // Assign the iterator to the processes_iterator
    processes_iterator = elementsRemainingIt._retn();

    return elements._retn();
}

char* ApplicationAccessor_impl::get_process(
    const char* target_element_path,
    const char* process_name,
    const char* host_name)
throw(CdmwPlatformMngt::ManagedElementNotFound,
      CdmwPlatformMngt::ProcessNotFound,
      CORBA::SystemException)
{
    CdmwPlatformMngt::Application_var application_obj = 
                        get_application (target_element_path);
                        
    CdmwPlatformMngt::Process_var process =
             application_obj->get_process(process_name, host_name);
             
    return process->element_path();
}

void ApplicationAccessor_impl::define(
    const char* target_element_path,
    const CdmwPlatformMngt::ApplicationDef& application_def)
throw(CdmwPlatformMngt::ManagedElementNotFound,
      CdmwPlatformMngt::DuplicateEntity,
      CdmwPlatformMngt::DuplicateService,
      CdmwPlatformMngt::DuplicateProcess,
      CdmwPlatformMngt::ProcessHostNotFound,
      CdmwPlatformMngt::DuplicateProcessEntity,
      CdmwPlatformMngt::DuplicateProcessService,
      CdmwPlatformMngt::DuplicateProcessStep,
      CdmwPlatformMngt::AlreadyDone,
      CdmwPlatformMngt::IncompatibleStatus,
      CosPropertyService::MultipleExceptions,
      CORBA::SystemException)
{
    CdmwPlatformMngt::Application_var application_obj = 
                        get_application (target_element_path);
                        
    application_obj->define(application_def);
}

char* ApplicationAccessor_impl::add_process(
    const char* target_element_path,
    CdmwPlatformMngt::ProcessDef* process_def)
throw(CdmwPlatformMngt::ManagedElementNotFound,
      CdmwPlatformMngt::ProcessAlreadyExists,
      CdmwPlatformMngt::HostNotFound,
      CdmwPlatformMngt::DuplicateEntity,
      CdmwPlatformMngt::DuplicateService,
      CdmwPlatformMngt::DuplicateStep,
      CdmwPlatformMngt::IncompatibleStatus,
      CosPropertyService::MultipleExceptions, 
      CORBA::SystemException)
{
    CdmwPlatformMngt::Application_var application_obj = 
                        get_application (target_element_path);
                        
    CdmwPlatformMngt::Process_var process =
             application_obj->add_process(process_def);
             
    return process->element_path();
}

void ApplicationAccessor_impl::remove_process(
    const char* target_element_path,
    const char* process_name,
    const char* host_name)
throw(CdmwPlatformMngt::ManagedElementNotFound,
      CdmwPlatformMngt::ProcessNotFound,
      CdmwPlatformMngt::ProcessStillActive,
      CdmwPlatformMngt::IncompatibleStatus,
      CORBA::SystemException)
{
    CdmwPlatformMngt::Application_var application_obj = 
                        get_application (target_element_path);
                        
    application_obj->remove_process(process_name, host_name);
}

void ApplicationAccessor_impl::set_init_graph(
    const char* target_element_path,
    const CdmwPlatformMngt::GraphElements& process_graph)
throw(CdmwPlatformMngt::ManagedElementNotFound,
      CdmwPlatformMngt::ReferenceNotFound,
      CdmwPlatformMngt::CircularReference,
      CdmwPlatformMngt::StepOutOfBound,
      CdmwPlatformMngt::StepJump,
      CdmwPlatformMngt::IncompatibleStatus,
      CORBA::SystemException)
{
    CdmwPlatformMngt::Application_var application_obj = 
                        get_application (target_element_path);
                        
    application_obj->set_init_graph(process_graph);
}

void ApplicationAccessor_impl::set_stop_graph(
    const char* target_element_path,
    const CdmwPlatformMngt::GraphElements& process_graph)
throw(CdmwPlatformMngt::ManagedElementNotFound,
      CdmwPlatformMngt::ReferenceNotFound,
      CdmwPlatformMngt::CircularReference,
      CdmwPlatformMngt::IncompatibleStatus,
      CORBA::SystemException)
{
    CdmwPlatformMngt::Application_var application_obj = 
                        get_application (target_element_path);
                        
    application_obj->set_stop_graph(process_graph);
}


void ApplicationAccessor_impl::initialise(
    const char* target_element_path,
    const CdmwPlatformMngtBase::StartupKind& startup_kind)
throw(CdmwPlatformMngt::ManagedElementNotFound,
      CdmwPlatformMngt::IncompatibleStatus,
      CORBA::SystemException)
{
    CdmwPlatformMngt::Application_var application_obj = 
                        get_application (target_element_path);
                        
    application_obj->initialise(startup_kind);
}

CdmwPlatformMngt::LongRequestStatus ApplicationAccessor_impl::initialise_and_wait(
    const char* target_element_path,
    const CdmwPlatformMngtBase::StartupKind& startup_kind)
throw(CdmwPlatformMngt::ManagedElementNotFound,
      CdmwPlatformMngt::IncompatibleStatus,
      CORBA::SystemException)
{
    CdmwPlatformMngt::Application_var application_obj = 
                        get_application (target_element_path);
                        
    return application_obj->initialise_and_wait(startup_kind);
}

void ApplicationAccessor_impl::resume(const char* target_element_path)
throw(CdmwPlatformMngt::ManagedElementNotFound,
      CdmwPlatformMngt::IncompatibleStatus,
      CORBA::SystemException)
{
    CdmwPlatformMngt::Application_var application_obj = 
                        get_application (target_element_path);
                        
    application_obj->resume();
}

CdmwPlatformMngt::LongRequestStatus ApplicationAccessor_impl::resume_and_wait(
     const char* target_element_path)
throw(CdmwPlatformMngt::ManagedElementNotFound,
      CdmwPlatformMngt::IncompatibleStatus,
      CORBA::SystemException)
{
    CdmwPlatformMngt::Application_var application_obj = 
                        get_application (target_element_path);
                        
    return application_obj->resume_and_wait();
}

void ApplicationAccessor_impl::acknowledge(const char* target_element_path,
                                                    const char* issuer)
throw(CdmwPlatformMngt::ManagedElementNotFound,
      CdmwPlatformMngt::IncompatibleStatus,
      CORBA::SystemException)
{
    try 
    {
        CdmwPlatformMngt::Application_var application_obj = 
                            get_application (target_element_path);              

        {
            CDMW_MUTEX_GUARD(*(Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->get_application_issuer_mutex(target_element_path)));
        
        Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->set_issuer(target_element_path, issuer);
                
            application_obj->acknowledge();
    
        }
    }
    catch (const CORBA::SystemException & ex) 
    {
        throw;
    }
    catch (...) 
    {
        throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL, 
                CORBA::COMPLETED_MAYBE);
    } 
}

void ApplicationAccessor_impl::run(const char* target_element_path)
throw(CdmwPlatformMngt::ManagedElementNotFound,
      CdmwPlatformMngt::IncompatibleStatus,
      CORBA::SystemException)
{
    CdmwPlatformMngt::Application_var application_obj = 
                        get_application (target_element_path);
                        
    application_obj->run();
}

void ApplicationAccessor_impl::stop(
     const char* target_element_path,
     CORBA::Boolean emergency)
throw(CdmwPlatformMngt::ManagedElementNotFound,
      CORBA::SystemException)
{
    CdmwPlatformMngt::Application_var application_obj = 
                        get_application (target_element_path);
                        
    application_obj->stop(emergency);
}


void ApplicationAccessor_impl:: stop_application(const char* target_element_path,
                                                          CORBA::Boolean emergency,
                                                          const char* issuer)
throw(CdmwPlatformMngt::ManagedElementNotFound,
       CORBA::SystemException)
{  
    try 
    {
        CdmwPlatformMngt::Application_var application_obj = 
            get_application (target_element_path);

        {
            CDMW_MUTEX_GUARD(*(Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->get_application_issuer_mutex(target_element_path)));
        
        Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->set_issuer(target_element_path, issuer);
        
        application_obj->stop_application(emergency);   
    }
        
    }
    catch (const CORBA::SystemException & ex) 
    {
        throw;
    }
    catch (...) 
    {
        throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL, 
                CORBA::COMPLETED_MAYBE);
    } 
}

CdmwPlatformMngt::LongRequestStatus ApplicationAccessor_impl::stop_and_wait(
    const char* target_element_path,
    CORBA::Boolean emergency)
throw(CdmwPlatformMngt::ManagedElementNotFound,
      CORBA::SystemException)
{
    CdmwPlatformMngt::Application_var application_obj = 
                        get_application (target_element_path);
                        
    return application_obj->stop_and_wait(emergency);
}
      
            
void ApplicationAccessor_impl::cold_restart(
    const char* target_element_path,
    const CdmwPlatformMngtBase::StartupKind& startup_kind,
    const char* issuer)
throw(CdmwPlatformMngt::ManagedElementNotFound,
      CdmwPlatformMngt::IncompatibleStatus,
      CORBA::SystemException)
{
    try 
    {
        CdmwPlatformMngt::Application_var application_obj = 
                            get_application (target_element_path);

        {
            CDMW_MUTEX_GUARD(*(Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->get_application_issuer_mutex(target_element_path)));
        
        Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->set_issuer(target_element_path, issuer);
        
            application_obj->cold_restart(startup_kind);
    }           
    }
    catch (const CORBA::SystemException & ex) 
    {
        throw;
    }
    catch (...) 
    {
        throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL, 
                CORBA::COMPLETED_MAYBE);
    } 
}


void ApplicationAccessor_impl::hot_restart(
    const char* target_element_path,
    const CdmwPlatformMngtBase::StartupKind& startup_kind,
    const char* issuer)
throw(CdmwPlatformMngt::ManagedElementNotFound,
      CdmwPlatformMngt::IncompatibleStatus,
      CORBA::SystemException)
{
    try 
    {
        CdmwPlatformMngt::Application_var application_obj = 
                            get_application (target_element_path);              

        {
            CDMW_MUTEX_GUARD(*(Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->get_application_issuer_mutex(target_element_path)));
        
        Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->set_issuer(target_element_path, issuer);
        
        application_obj->hot_restart(startup_kind);
    }                            
    }
    catch (const CORBA::SystemException & ex) 
    {
        throw;
    }
    catch (...) 
    {
        throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL, 
                CORBA::COMPLETED_MAYBE);
    } 
}
 

/*CdmwPlatformMngt::ElementPaths* ApplicationAccessor_impl::get_children (const char* target_element_path)
                                 throw (CORBA::SystemException)
{
    try{
        CdmwPlatformMngt::Application_var application_obj =
                        get_application (target_element_path);
        return application_obj->get_children();
     }
     catch (const CORBA::SystemException & ex) {
         throw;
    }
    catch (...) {
        throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
    }

}*/


void ApplicationAccessor_impl::set_indicator_relevant_status( const char* target_element_path,  const bool indicator_relevant_status) throw( CORBA::SystemException)
{
    CdmwPlatformMngt::Application_var application_obj = get_application (target_element_path);
    application_obj->set_indicator_relevant_status(indicator_relevant_status);
}

bool ApplicationAccessor_impl::get_indicator_relevant_status( const char* target_element_path) throw( CORBA::SystemException)
{
    CdmwPlatformMngt::Application_var application_obj = get_application (target_element_path);
    return application_obj->get_indicator_relevant_status();
}

CdmwPlatformMngt::ElementType ApplicationAccessor_impl::get_element_type(const char* target_element_path) throw     (CORBA::SystemException,CdmwPlatformMngt::ManagedElementNotFound)
{
      CdmwPlatformMngt::Application_var appl_obj =
        get_application (target_element_path);

      return (appl_obj->get_element_type ());
}
    


void ApplicationAccessor_impl::get_all_ft_groups(const char* target_element_path,
          CORBA::ULong how_many, 
          CdmwPlatformMngt::ElementPaths_out paths, 
            CdmwPlatformMngt::FTGroupsIterator_out rest)
             throw (CORBA::SystemException,CdmwPlatformMngt::ManagedElementNotFound)
{
   
   try{
        CdmwPlatformMngt::Application_var application_obj =
                        get_application (target_element_path);
        
        CdmwPlatformMngt::FTGroups_var ftgroups;
        
    
        application_obj->get_all_ft_groups(how_many, ftgroups.out(), 
                        rest );

        
        CdmwPlatformMngt::ElementPaths_var ftgroup_paths =  
         new CdmwPlatformMngt::ElementPaths();
        
    
        CORBA::ULong num_ftgroups = ftgroups->length() ;
        
    
        ftgroup_paths->length( num_ftgroups ) ;
      
    
        CORBA::ULong i=0;
          
        for ( i=0; i< (num_ftgroups); i++)
                {
                ftgroup_paths[i] = ((*ftgroups)[i])->element_path() ;
        }
    
    
        paths = ftgroup_paths._retn();
       
       }
       catch (const CORBA::SystemException & ex) {
         throw;
       }
       catch (...) {
        throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
   
       }
}
   


 void ApplicationAccessor_impl::get_all_lb_groups(
      const char* target_element_path, 
      CORBA::ULong how_many,
           CdmwPlatformMngt::ElementPaths_out paths,
           CdmwPlatformMngt::LBGroupsIterator_out rest)
 throw (CORBA::SystemException,CdmwPlatformMngt::ManagedElementNotFound)
 {
       
    try{    
        CdmwPlatformMngt::Application_var application_obj =
                        get_application (target_element_path);
        
        CdmwPlatformMngt::LBGroups_var lbgroups;
        
        
        application_obj->get_all_lb_groups(how_many, lbgroups.out(), 
                        rest );

        
        CdmwPlatformMngt::ElementPaths_var lbgroup_paths =  
         new CdmwPlatformMngt::ElementPaths();
        
        
        CORBA::ULong num_lbgroups = lbgroups->length() ;
        
        
        lbgroup_paths->length( num_lbgroups ) ;
      
        
        CORBA::ULong i=0;
         
        for ( i=0; i< (num_lbgroups); i++)
                {
                lbgroup_paths[i] = ((*lbgroups)[i])->element_path() ;
                }
        
        
        paths = lbgroup_paths._retn();


       }
       catch (const CORBA::SystemException & ex) {
         throw;
       }
       catch (...) {
        throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);

       }


 }
    
    


char* ApplicationAccessor_impl::add_ftgroup(
           const char* target_element_path, const CdmwPlatformMngt::FTGroupDef& ftgroup_def)
            throw(CdmwPlatformMngt::ManagedElementNotFound,
                     CdmwPlatformMngt::GroupAlreadyExists, 
                     CdmwPlatformMngt::DuplicateEntity, 
                     CosPropertyService::MultipleExceptions,
                     CdmwPlatformMngt::IncompatibleStatus)
{


    CdmwPlatformMngt::Application_var application_obj = 
                        get_application (target_element_path);
                        
    CdmwPlatformMngt::FTGroup_var ftgroup =
             application_obj->add_ftgroup(ftgroup_def);
             
    return ftgroup->element_path();

    
}

char* ApplicationAccessor_impl::add_lbgroup(const char*,
             const CdmwPlatformMngt::LBGroupDef&)
              throw(CdmwPlatformMngt::ManagedElementNotFound,
                     CdmwPlatformMngt::GroupAlreadyExists,
                     CdmwPlatformMngt::DuplicateEntity, 
                     CosPropertyService::MultipleExceptions,
                     CdmwPlatformMngt::IncompatibleStatus)
{
    throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
}

    char* ApplicationAccessor_impl::get_ft_group(
                                   const char* target_element_path, 
                                   const char* ft_group_name)
      throw(CdmwPlatformMngt::GroupNotFound, 
            CdmwPlatformMngt::ManagedElementNotFound,
            CORBA::SystemException)
{
        CdmwPlatformMngt::Application_var application_obj = 
                            get_application (target_element_path);
                            
        
        CdmwPlatformMngt::FTGroup_var ftgroup =
                   application_obj->get_ft_group(ft_group_name);
                 
        return (  ftgroup->element_path());
}

    char* ApplicationAccessor_impl::get_lb_group(const char* target_element_path, 
     const char* lb_group_name)
      throw(CdmwPlatformMngt::GroupNotFound, 
            CdmwPlatformMngt::ManagedElementNotFound,
            CORBA::SystemException)
{

        CdmwPlatformMngt::Application_var application_obj = 
                            get_application (target_element_path);
        CdmwPlatformMngt::LBGroup_var lbgroup =
                   application_obj->get_lb_group(lb_group_name);
                 
        return (  lbgroup->element_path());

}

    CORBA::ULong ApplicationAccessor_impl::get_number_of_ft_groups(const char*
        target_element_path)
       throw(CORBA::SystemException,
             CdmwPlatformMngt::ManagedElementNotFound)
{
        CdmwPlatformMngt::Application_var application_obj = 
                            get_application (target_element_path);

        return( application_obj->get_number_of_ft_groups() ) ;
}

    CORBA::ULong ApplicationAccessor_impl::get_number_of_lb_groups(const char*
         target_element_path)
       throw(CORBA::SystemException,
             CdmwPlatformMngt::ManagedElementNotFound)
{
        CdmwPlatformMngt::Application_var application_obj = 
                            get_application (target_element_path);
        return( application_obj->get_number_of_lb_groups() ) ;
}



} // End namespace PlatformMngt
} // End namespace Cdmw

