/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"

#include "platformftaccessorlibrary/CcmComponentAccessor_impl.hpp"

#include "SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp"

#include "SystemMngt/tools/ManagedElementKey.hpp"

#include "SystemMngt/platformlibrary/ElementPathsIterator_impl.hpp"



namespace Cdmw
{
  namespace PlatformMngt
  {

// ----------------------------------------------------------------------
// CcmComponentAccessor_impl class.
// ----------------------------------------------------------------------

    CcmComponentAccessor_impl::CcmComponentAccessor_impl (CdmwPlatformMngt::
                                                          ElementRepository_ptr
                                                          element_repository)
      throw ():ManagedElementAccessor_impl (element_repository),
      RWEntityContainerAccessor_impl (element_repository)
    {
    }

    CcmComponentAccessor_impl::~CcmComponentAccessor_impl ()
    {
    }

    CdmwPlatformMngt::CcmComponent_ptr CcmComponentAccessor_impl::
      get_ccmcomponent (const char *target_element_path)
      throw (CdmwPlatformMngt::ManagedElementNotFound)
    {
      CdmwPlatformMngt::CcmComponent_var ccmcomponent_obj;

      try
      {
        CdmwPlatformMngt::ManagedElement_var element_obj =
          m_element_repository->get_element (target_element_path);

        ccmcomponent_obj =
          CdmwPlatformMngt::CcmComponent::_narrow (element_obj.in ());
      }
      catch (const CdmwPlatformMngt::ElementPathInvalid &)
      {
        throw CdmwPlatformMngt::ManagedElementNotFound (target_element_path);
      }
      catch (const CORBA::SystemException &)
      {
        throw CdmwPlatformMngt::ManagedElementNotFound (target_element_path);
      }

      return ccmcomponent_obj._retn ();
    }


    char *CcmComponentAccessor_impl::
      get_deployment_plan (const char *target_element_path) throw (CORBA::
                                                                   SystemException,
                                                                   CdmwPlatformMngt::
                                                                   ManagedElementNotFound)
    {

      CdmwPlatformMngt::CcmComponent_var ccmcomponent_obj =
        get_ccmcomponent (target_element_path);

      return (ccmcomponent_obj->deployment_plan ());

    }

    CORBA::Boolean CcmComponentAccessor_impl::
      get_created_dynamically (const char *target_element_path) throw (CORBA::
                                                                       SystemException,
                                                                       CdmwPlatformMngt::
                                                                       ManagedElementNotFound)
    {

      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);

    }

    CdmwPlatformMngt::RequestStatus CcmComponentAccessor_impl::
      initialise (const char *target_element_path,
                  const CdmwPlatformMngtBase::
                  StartupKind & start) throw (CORBA::SystemException,
                                              CdmwPlatformMngt::
                                              IncompatibleStatus,
                                              CdmwPlatformMngt::
                                              ManagedElementNotFound)
    {

      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);

    }


    CdmwPlatformMngt::RedundancyEltState CcmComponentAccessor_impl::
      get_redundancy_elt_state (const char *target_element_path) throw (CORBA::
                                                                        SystemException,
                                                                        CdmwPlatformMngt::
                                                                        ManagedElementNotFound)
    {

      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);

    }

    CdmwPlatformMngtBase::TimeStamp CcmComponentAccessor_impl::
      get_start_time (const char *target_element_path) throw (CORBA::
                                                              SystemException,
                                                              CdmwPlatformMngt::
                                                              ManagedElementNotFound)
    {

      CdmwPlatformMngt::CcmComponent_var ccmcomponent_obj =
        get_ccmcomponent (target_element_path);

      return (ccmcomponent_obj->start_time ());

    }

    CdmwPlatformMngtBase::TimeStamp CcmComponentAccessor_impl::
      get_stop_time (const char *target_element_path) throw (CORBA::
                                                             SystemException,
                                                             CdmwPlatformMngt::
                                                             ManagedElementNotFound)
    {

      CdmwPlatformMngt::CcmComponent_var ccmcomponent_obj =
        get_ccmcomponent (target_element_path);

      return (ccmcomponent_obj->stop_time ());

    }


    char *CcmComponentAccessor_impl::
      get_instantiation_id (const char *target_element_path) throw (CORBA::
                                                                    SystemException,
                                                                    CdmwPlatformMngt::
                                                                    ManagedElementNotFound)
    {

      CdmwPlatformMngt::CcmComponent_var ccmcomponent_obj =
        get_ccmcomponent (target_element_path);

      return (ccmcomponent_obj->instantiation_id ());

    }


    void CcmComponentAccessor_impl::
      set_indicator_relevant_status (const char *target_element_path,
                                     CORBA::
                                     Boolean indicator_relevant_status)
      throw (CORBA::SystemException)
    {

      CdmwPlatformMngt::CcmComponent_var ccmcomponent_obj =
        get_ccmcomponent (target_element_path);

      ccmcomponent_obj->
        set_indicator_relevant_status (indicator_relevant_status);

    }

    CORBA::ULong CcmComponentAccessor_impl::
      get_number_of_entities (const char *target_element_path) throw (CORBA::
                                                                      SystemException)
    {

      CdmwPlatformMngt::CcmComponent_var ccmcomponent_obj =
        get_ccmcomponent (target_element_path);

      return (ccmcomponent_obj->get_number_of_entities ());

    }

    void CcmComponentAccessor_impl::
      get_all_entity_names (const char *target_element_path, CORBA::ULong num,
                            CdmwPlatformMngt::ElementNames_out ele_out,
                            CdmwPlatformMngt::
                            ElementNamesIterator_out ele_iter_out)
      throw (CORBA::SystemException)
    {

      CdmwPlatformMngt::CcmComponent_var ccmcomponent_obj =
        get_ccmcomponent (target_element_path);

      ccmcomponent_obj->get_all_entity_names (num, ele_out, ele_iter_out);

    }


    CORBA::Boolean CcmComponentAccessor_impl::
      get_entities (const char *target_element_path,
                    const CdmwPlatformMngt::ElementNames & element_names,
                    CdmwPlatformMngt::Entities_out entities_out) throw (CORBA::
                                                                        SystemException)
    {

      CdmwPlatformMngt::CcmComponent_var ccmcomponent_obj =
        get_ccmcomponent (target_element_path);

      return (ccmcomponent_obj->get_entities (element_names, entities_out));

    }

    /*
    void CcmComponentAccessor_impl::
      get_all_entities (const char *target_element_path, CORBA::ULong how_many,
                        CdmwPlatformMngt::Entities_out ent_out,
                        CdmwPlatformMngt::
                        EntitiesIterator_out ent_iter_out) throw (CORBA::
                                                                  SystemException)
    {

      CdmwPlatformMngt::CcmComponent_var ccmcomponent_obj =
        get_ccmcomponent (target_element_path);

      ccmcomponent_obj->get_all_entities (how_many, ent_out, ent_iter_out);

    }
    */


    void CcmComponentAccessor_impl::
      get_attribute (const char *target_element_path, const char *name,
                     CORBA::Any_out out_any,
                     CdmwPlatformMngt::Owner owner) 
 throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound, CdmwPlatformMngt::ManagedElementNotFound)
    {

      CdmwPlatformMngt::CcmComponent_var ccmcomponent_obj =
        get_ccmcomponent (target_element_path);

      ccmcomponent_obj->get_attribute (name, out_any, owner);

    }


    void CcmComponentAccessor_impl::
      set_attribute (const char *target_element_path, const char *name,
                     const CORBA::Any & out_any,
                     CdmwPlatformMngt::Owner owner) 
    throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound, CdmwPlatformMngt::ReadOnlyAttribute, CdmwPlatformMngt::ManagedElementNotFound)
    {

      CdmwPlatformMngt::CcmComponent_var ccmcomponent_obj =
        get_ccmcomponent (target_element_path);

      ccmcomponent_obj->set_attribute (name, out_any, owner);

    }


    CdmwPlatformMngt::ElementType CcmComponentAccessor_impl::
      get_element_type (const char *target_element_path) throw (CORBA::
                                                                SystemException)
    {

      CdmwPlatformMngt::CcmComponent_var ccmcomponent_obj =
        get_ccmcomponent (target_element_path);

      return (ccmcomponent_obj->get_element_type ());

    }


/*   CdmwPlatformMngt::ElementPaths* CcmComponentAccessor_impl::get_children (const char *target_element_path) 
                                                                           throw (CORBA::SystemException)
    {

      CdmwPlatformMngt::CcmComponent_var ccmcomponent_obj =
        get_ccmcomponent (target_element_path);

      return (ccmcomponent_obj->get_children ());

    }*/


    CdmwPlatformMngt::ComponentSnapshot *
      CcmComponentAccessor_impl::
      get_snapshot (const char *target_element_path) throw (CORBA::
                                                            SystemException)
    {

      CdmwPlatformMngt::CcmComponent_var ccmcomponent_obj =
        get_ccmcomponent (target_element_path);

      return (ccmcomponent_obj->get_snapshot ());

    }


    CdmwPlatformMngt::ComponentStatus CcmComponentAccessor_impl::
      get_status (const char *target_element_path,
                  CORBA::String_out str_out) throw (CORBA::SystemException)
    {

      CdmwPlatformMngt::CcmComponent_var ccmcomponent_obj =
        get_ccmcomponent (target_element_path);

      return (ccmcomponent_obj->get_status (str_out));

    }


    char *CcmComponentAccessor_impl::
      instantiation_id (const char *target_element_path) throw (CORBA::
                                                                SystemException)
    {

      CdmwPlatformMngt::CcmComponent_var ccmcomponent_obj =
        get_ccmcomponent (target_element_path);

      return (ccmcomponent_obj->instantiation_id ());

    }

    CdmwPlatformMngtBase::TimeStamp CcmComponentAccessor_impl::
      start_time (const char *target_element_path) throw (CORBA::
                                                          SystemException)
    {

      CdmwPlatformMngt::CcmComponent_var ccmcomponent_obj =
        get_ccmcomponent (target_element_path);

      return (ccmcomponent_obj->start_time ());

    }


    CdmwPlatformMngtBase::TimeStamp CcmComponentAccessor_impl::
      stop_time (const char *target_element_path) throw (CORBA::SystemException)
    {

      CdmwPlatformMngt::CcmComponent_var ccmcomponent_obj =
        get_ccmcomponent (target_element_path);

      return (ccmcomponent_obj->stop_time ());

    }


    char *CcmComponentAccessor_impl::
      get_component_server_name (const char *target_element_path) throw (CORBA::
                                                                         SystemException)
    {

      CdmwPlatformMngt::CcmComponent_var ccmcomponent_obj =
        get_ccmcomponent (target_element_path);

      return (ccmcomponent_obj->get_component_server_name ());

    }

    CORBA::Boolean CcmComponentAccessor_impl::
      get_indicator_relevant_status (const char *target_element_path)
      throw (CORBA::SystemException)
    {

      CdmwPlatformMngt::CcmComponent_var ccmcomponent_obj =
        get_ccmcomponent (target_element_path);

      return (ccmcomponent_obj->get_indicator_relevant_status ());

    }

    CdmwPlatformMngt::RedundancyGroup *
      CcmComponentAccessor_impl::
      get_component_group (const char *target_element_path) throw (CORBA::
                                                                   SystemException)
    {

      CdmwPlatformMngt::CcmComponent_var ccmcomponent_obj =
        get_ccmcomponent (target_element_path);

      return (ccmcomponent_obj->get_component_group ());

    }

    void CcmComponentAccessor_impl::
      remove_entity (const char *target_element_path,
                     const char *name) throw (CORBA::SystemException)
    {

      CdmwPlatformMngt::CcmComponent_var ccmcomponent_obj =
        get_ccmcomponent (target_element_path);

      ccmcomponent_obj->remove_entity (name);

    }

    char *CcmComponentAccessor_impl::
      deployment_plan (const char *target_element_path) throw (CORBA::
                                                               SystemException)
    {

      CdmwPlatformMngt::CcmComponent_var ccmcomponent_obj =
        get_ccmcomponent (target_element_path);

      return (ccmcomponent_obj->deployment_plan ());

    }


  }                             // End namespace PlatformMngt
}                               // End namespace Cdmw
