/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"

#include "platformftaccessorlibrary/RedundancyGroupAccessor_impl.hpp"
#include "platformftaccessorlibrary/FTGroupAccessor_impl.hpp"

#include "SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp"
#include "SystemMngt/idllib/CdmwFTSystMngtFTGroup.stub.hpp"

#include "SystemMngt/tools/ManagedElementKey.hpp"

#include "SystemMngt/platformlibrary/ElementPathsIterator_impl.hpp"



namespace Cdmw
{
namespace PlatformMngt
{

// ----------------------------------------------------------------------
// FTGroupAccessor_impl class.
// ----------------------------------------------------------------------


FTGroupAccessor_impl::FTGroupAccessor_impl( CdmwPlatformMngt::ElementRepository_ptr
element_repository) throw() : RedundancyGroupAccessor_impl(element_repository)
{
}

FTGroupAccessor_impl::~FTGroupAccessor_impl()
{
}

CdmwPlatformMngt::FTGroup_ptr FTGroupAccessor_impl::
     get_ftgroup ( const char* target_element_path) 
            throw(CdmwPlatformMngt::ManagedElementNotFound)
{
    CdmwPlatformMngt::FTGroup_var ftgroup_obj;
    
    try
    {
        CdmwPlatformMngt::ManagedElement_var element_obj = m_element_repository->get_element(target_element_path);
         
        ftgroup_obj = CdmwPlatformMngt::FTGroup::_narrow (element_obj.in());
    }
    catch (const CdmwPlatformMngt::ElementPathInvalid&)
    {
        throw CdmwPlatformMngt::ManagedElementNotFound(target_element_path);
    }
    catch(const CORBA::SystemException&)
    {
        throw CdmwPlatformMngt::ManagedElementNotFound(target_element_path);
    }
    
    return ftgroup_obj._retn();        
}


CdmwPlatformMngt::FTGroupSnapshot* 
    FTGroupAccessor_impl::get_snapshot(const char* target_element_path) 
           throw (CORBA::SystemException, 
                  CdmwPlatformMngt::ManagedElementNotFound)
{
    CdmwPlatformMngt::FTGroup_var ftgroup_obj = get_ftgroup (target_element_path);
    return(ftgroup_obj->get_snapshot( )); 

}


CdmwPlatformMngt::ReplicationStyle 
    FTGroupAccessor_impl::get_style(const char* target_element_path) 
           throw (CORBA::SystemException, 
                  CdmwPlatformMngt::ManagedElementNotFound)
{
    CdmwPlatformMngt::FTGroup_var ftgroup_obj = get_ftgroup (target_element_path);
         
    return(ftgroup_obj->get_style( )); 
}


void FTGroupAccessor_impl::set_style(const char* target_element_path, 
                                     CdmwPlatformMngt::ReplicationStyle rep) 
                                        throw (CORBA::SystemException, 
                                               CdmwPlatformMngt::ManagedElementNotFound)
{
    CdmwPlatformMngt::FTGroup_var ftgroup_obj = get_ftgroup (target_element_path);
         
    return(ftgroup_obj->set_style( rep )); 
}


CdmwPlatformMngt::ElementType FTGroupAccessor_impl::get_element_type(const char* target_element_path) 
                                        throw (CORBA::SystemException)
{
    CdmwPlatformMngt::FTGroup_var ftgroup_obj = get_ftgroup (target_element_path);
         
    return(ftgroup_obj->get_element_type()); 
}



} // End namespace PlatformMngt
} // End namespace Cdmw

