/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"

#include "platformsystemlibrary/System_impl.hpp"
#include "platformsystemlibrary/Host_impl.hpp"
#include "SystemMngt/platformapplicationlibrary/Application_impl.hpp"
#include "platformapplicationlibrary/Process_impl.hpp"
#include "platformapplicationlibrary/ManagedProcess_impl.hpp"

#include "SystemMngt/platformelementlibrary/Entity_impl.hpp"
#include "SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/UserEvent_impl.hpp"
#include "SystemMngt/platformlibrary/SupervisionEventNotifier.hpp"
#include "SystemMngt/platformlibrary/EventHeaderFactory.hpp"
#include "SystemMngt/tools/ManagedElementKey.hpp"
#include "SystemMngt/platformlibrary/ElementRepository_impl.hpp"

#include "platformftaccessorlibrary/FTProcessObserver_impl.hpp"

namespace Cdmw
{
namespace PlatformMngt
{

// ----------------------------------------------------------------------
// FTProcessObserver_impl class.
// ----------------------------------------------------------------------
FTProcessObserver_impl::FTProcessObserver_impl(
    PortableServer::POA_ptr poa,
    System_impl* system,
    CdmwPlatformMngt::ElementRepository_ptr element_repository)
: DeactivableServant_impl (poa)
{
    m_system = system;
    m_element_repository = 
           CdmwPlatformMngt::ElementRepository::_duplicate(element_repository);
}

void FTProcessObserver_impl::notify_message(
    const char* application_name,
    const char* process_name,
    const char* host_name,
    const CdmwPlatformMngtBase::EventHeader& header,
    const char* issuer,
    const char* message)
throw (CdmwPlatformMngt::ProcessNotFound,
       CORBA::SystemException)
{
    try
    {
        // search the application process observer and relay the command
        CdmwPlatformMngt::ProcessObserver_var app_proc_observer = 
                               m_system->get_process_observer (application_name);
                               
        app_proc_observer->notify_message(application_name, process_name,
                                          host_name, header, issuer, message);
    }
    catch (const CdmwPlatformMngt::ApplicationNotFound&)
    {
        throw CdmwPlatformMngt::ProcessNotFound();
    }    
    catch (...)
    {
        throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
    }
}

void FTProcessObserver_impl::notify_error(
    const char* application_name,
    const char* process_name,
    const char* host_name,
    const CdmwPlatformMngtBase::TimeStamp& time_stamp,
    const char* issuer,
    const char* error_info)
throw (CdmwPlatformMngt::ProcessNotFound,
       CORBA::SystemException)
{
    try
    {
        // search the application process observer and relay the command
        CdmwPlatformMngt::ProcessObserver_var app_proc_observer = 
                               m_system->get_process_observer (application_name);
                               
        app_proc_observer->notify_error (application_name, process_name,
                                         host_name, time_stamp, issuer, error_info);
    }
    catch (const CdmwPlatformMngt::ApplicationNotFound&)
    {
        throw CdmwPlatformMngt::ProcessNotFound();
    }
    catch(...)
    {
        throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
    }
}


void FTProcessObserver_impl::notify_starting(
    const char* application_name,
    const char* process_name,
    const char* host_name,
    CdmwPlatformMngt::ProcessPID pid,
    const CdmwPlatformMngtBase::TimeStamp & time_stamp,
    CORBA::Long restart_attempt)
            throw (CdmwPlatformMngt::ProcessNotFound,
                   CORBA::SystemException)
{
    // TODO
    throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
}



void FTProcessObserver_impl::notify_ending(
    const char* application_name,
    const char* process_name,
    const char* host_name,
    const CdmwPlatformMngtBase::TimeStamp& time_stamp,
    const CdmwPlatformMngt::ProcessExitInfo & exit_info)
throw (CdmwPlatformMngt::ProcessNotFound,
       CORBA::SystemException)
{
    try
    {
        // search the application process observer and relay the command
        CdmwPlatformMngt::ProcessObserver_var app_proc_observer = 
                               m_system->get_process_observer (application_name);
                               
        app_proc_observer->notify_ending (application_name, process_name,
                                          host_name, time_stamp, exit_info);
    }
    catch (const CdmwPlatformMngt::ApplicationNotFound&)
    {
        throw CdmwPlatformMngt::ProcessNotFound();
    }
    catch (...)
    {
        throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
    }
}

void FTProcessObserver_impl::notify_monitoring_failure(
    const char* application_name,
    const char* process_name,
    const char* host_name,
    const CdmwPlatformMngtBase::TimeStamp& time_stamp)
throw (CdmwPlatformMngt::ProcessNotFound,
       CORBA::SystemException)
{
    try
    {
        // search the application process observer and relay the command
        CdmwPlatformMngt::ProcessObserver_var app_proc_observer = 
                               m_system->get_process_observer (application_name);
                               
        app_proc_observer->notify_monitoring_failure (application_name, process_name,
                                                      host_name, time_stamp);
    }
    catch (const CdmwPlatformMngt::ApplicationNotFound&)
    {
        throw CdmwPlatformMngt::ProcessNotFound();
    }
    catch (...)
    {
        throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
    }
}


void FTProcessObserver_impl::notify_user_event(
    const char* application_name,
    const char* process_name,
    const char* host_name,
    const char* issuer,
    const CORBA::Any& data)
throw (CdmwPlatformMngt::ProcessNotFound,
       CORBA::SystemException)
{
    try
    {
        // search the application process observer and relay the command
        CdmwPlatformMngt::ProcessObserver_var app_proc_observer = 
                               m_system->get_process_observer (application_name);
                               
        app_proc_observer->notify_user_event (application_name, process_name,
                                              host_name, issuer, data);
    }
    catch (const CdmwPlatformMngt::ApplicationNotFound&)
    {
        throw CdmwPlatformMngt::ProcessNotFound();
    }
}

void FTProcessObserver_impl::notify_property_change (
    CdmwPlatformMngt::PropertyChange * property_change_event )
throw (CORBA::SystemException)
{
    try
    {
        string system_name, 
        application_name, 
        host_name,process_name,
        component_name,
        entity_name,
        attribute_name, 
        ftgroup_name,lbgroup_name;
        
        try
        {
            Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(
                    property_change_event->element_path(),
                    system_name,
                    application_name, 
                    host_name,
                    process_name, 
                    component_name,
                    entity_name,
                    attribute_name,
                    ftgroup_name,
                    lbgroup_name);
        }
        catch (const CdmwPlatformMngt::ElementPathInvalid&)
        {
            LogMngr::logMessage(ERR,"FTProcessObserver_impl:: notify_property_change- ElementPathInvalid ");  
            throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
        }
        
        // search the application process observer and relay the command
        CdmwPlatformMngt::ProcessObserver_var app_proc_observer = 
                               m_system->get_process_observer (application_name.c_str());
                               
        app_proc_observer->notify_property_change (property_change_event);
    }
    catch (const CdmwPlatformMngt::ApplicationNotFound&)
    {
        throw CdmwPlatformMngt::ProcessNotFound();
    }
}

CdmwPlatformMngt::ManagedElement_ptr FTProcessObserver_impl::get_managed_element (
     const char * element_path)
throw (CdmwPlatformMngt::ElementPathInvalid,
       CdmwPlatformMngt::ManagedElementNotFound,
       CORBA::SystemException)
{
    // search the element in repository
    CdmwPlatformMngt::ManagedElement_var element = 
                 m_element_repository->get_element(element_path);
                 
    return element._retn();
}


void FTProcessObserver_impl::set_create_process_result (
        const char * application_name,
        const char * process_name,
        const char * host_name,
        CdmwPlatformMngt::ProcessCommandResult result,
        const char* error_info,
        const char* error_issuer,
        CdmwPlatformMngt::ProcessDelegateWrapper_ptr process_delegate_wrap)
            throw (CdmwPlatformMngt::ProcessNotFound,
                   CORBA::SystemException)
{
    
    // build the process path
    std::string process_path = ManagedElementKey::Get_process_path (m_system->name(),
                                    application_name, host_name, process_name);
    try
    {
        // search the process in repository and relay the command
        CdmwPlatformMngt::ManagedElement_var element = 
                 m_element_repository->get_element(process_path.c_str());
                 
        CdmwPlatformMngt::Process_var process = 
                 CdmwPlatformMngt::Process::_narrow(element.in());

        process->set_create_process_result(result, error_info, error_issuer,
                                           process_delegate_wrap);
    }
    catch (const CdmwPlatformMngt::ElementPathInvalid&)
    {
        throw CdmwPlatformMngt::ProcessNotFound();
    }
    catch (const CdmwPlatformMngt::ManagedElementNotFound&)
    {
        throw CdmwPlatformMngt::ProcessNotFound();
    }      
}


     

void FTProcessObserver_impl::set_initialisation_result (
        const char * application_name,
        const char * process_name,
        const char * host_name,
        CdmwPlatformMngt::ProcessCommandResult result)
            throw (CdmwPlatformMngt::ProcessNotFound,
                   CORBA::SystemException)
{
    
    // build the process path
    std::string process_path = ManagedElementKey::Get_process_path (m_system->name(),
                            application_name, host_name, process_name);
    try
    {
        // search the process in repository and relay the command
        CdmwPlatformMngt::ManagedElement_var element = 
                 m_element_repository->get_element(process_path.c_str());
                 
        CdmwPlatformMngt::Process_var process = 
                 CdmwPlatformMngt::Process::_narrow(element.in());

        process->set_initialisation_result(result);
    }
    catch (const CdmwPlatformMngt::ElementPathInvalid&)
    {
        throw CdmwPlatformMngt::ProcessNotFound();
    }
    catch (const CdmwPlatformMngt::ManagedElementNotFound&)
    {
        throw CdmwPlatformMngt::ProcessNotFound();
    }        
}
     

void FTProcessObserver_impl::set_next_step_result (
        const char * application_name,
        const char * process_name,
        const char * host_name,
        CdmwPlatformMngt::ProcessCommandResult result,
        CORBA::ULong step_nbr)
            throw (CdmwPlatformMngt::ProcessNotFound,
                   CORBA::SystemException)
{
    
    // build the process path
    std::string process_path = ManagedElementKey::Get_process_path (m_system->name(),
                            application_name, host_name, process_name);
    try
    {
        // search the process in repository and relay the command
        CdmwPlatformMngt::ManagedElement_var element = 
                 m_element_repository->get_element(process_path.c_str());
                 
        CdmwPlatformMngt::Process_var process = 
                 CdmwPlatformMngt::Process::_narrow(element.in());

        process->set_next_step_result(result, step_nbr);
    }
    catch (const CdmwPlatformMngt::ElementPathInvalid&)
    {
        throw CdmwPlatformMngt::ProcessNotFound();
    }
    catch (const CdmwPlatformMngt::ManagedElementNotFound&)
    {
        throw CdmwPlatformMngt::ProcessNotFound();
    } 
}
     

void FTProcessObserver_impl::set_run_result (
        const char * application_name,
        const char * process_name,
        const char * host_name,
        CdmwPlatformMngt::ProcessCommandResult result)
            throw (CdmwPlatformMngt::ProcessNotFound,
                   CORBA::SystemException)
{
    
    // build the process path
    std::string process_path = ManagedElementKey::Get_process_path (m_system->name(),
                            application_name, host_name, process_name);
    try
    {
        // search the process in repository and relay the command
        CdmwPlatformMngt::ManagedElement_var element = 
                 m_element_repository->get_element(process_path.c_str());
                 
        CdmwPlatformMngt::Process_var process = 
                 CdmwPlatformMngt::Process::_narrow(element.in());

        process->set_run_result(result);
    }
    catch (const CdmwPlatformMngt::ElementPathInvalid&)
    {
        throw CdmwPlatformMngt::ProcessNotFound();
    }
    catch (const CdmwPlatformMngt::ManagedElementNotFound&)
    {
        throw CdmwPlatformMngt::ProcessNotFound();
    } 
}
     

void FTProcessObserver_impl::set_stop_result (
        const char * application_name,
        const char * process_name,
        const char * host_name,
        CdmwPlatformMngt::ProcessCommandResult result)
            throw (CdmwPlatformMngt::ProcessNotFound,
                   CORBA::SystemException)
{
    
    // build the process path
    std::string process_path = ManagedElementKey::Get_process_path (m_system->name(),
                            application_name, host_name, process_name);
    try
    {
        // search the process in repository and relay the command
        CdmwPlatformMngt::ManagedElement_var element = 
                 m_element_repository->get_element(process_path.c_str());
                 
        CdmwPlatformMngt::Process_var process = 
                 CdmwPlatformMngt::Process::_narrow(element.in());

        process->set_stop_result(result);
    }
    catch (const CdmwPlatformMngt::ElementPathInvalid&)
    {
        throw CdmwPlatformMngt::ProcessNotFound();
    }
    catch (const CdmwPlatformMngt::ManagedElementNotFound&)
    {
        throw CdmwPlatformMngt::ProcessNotFound();
    } 
}




} // End namespace PlatformMngt
} // End namespace Cdmw

