/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"

#include "platformftaccessorlibrary/RedundancyGroupAccessor_impl.hpp"
#include "platformftaccessorlibrary/LBGroupAccessor_impl.hpp"

#include "SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp"
#include "SystemMngt/idllib/CdmwFTSystMngtLBGroup.stub.hpp"

#include "SystemMngt/tools/ManagedElementKey.hpp"

#include "SystemMngt/platformlibrary/ElementPathsIterator_impl.hpp"



namespace Cdmw
{
  namespace PlatformMngt
  {

// ----------------------------------------------------------------------
// LBGroupAccessor_impl class.
// ----------------------------------------------------------------------

    LBGroupAccessor_impl::LBGroupAccessor_impl (CdmwPlatformMngt::
                                                ElementRepository_ptr
                                                element_repository)
      throw ():RedundancyGroupAccessor_impl (element_repository)
    {
    }

    LBGroupAccessor_impl::~LBGroupAccessor_impl ()
    {
    }

CdmwPlatformMngt::LBGroup_ptr LBGroupAccessor_impl::
     get_lbgroup ( const char* target_element_path) 
            throw(CdmwPlatformMngt::ManagedElementNotFound)
{
    CdmwPlatformMngt::LBGroup_var lbgroup_obj;
    
    try
    {
        CdmwPlatformMngt::ManagedElement_var element_obj = 
              m_element_repository->get_element(target_element_path);
         
        lbgroup_obj = CdmwPlatformMngt::LBGroup::_narrow (element_obj.in());
    }
    catch (const CdmwPlatformMngt::ElementPathInvalid&)
    {
        throw CdmwPlatformMngt::ManagedElementNotFound(target_element_path);
    }
    catch(const CORBA::SystemException&)
    {
        throw CdmwPlatformMngt::ManagedElementNotFound(target_element_path);
    }
    
    return lbgroup_obj._retn();        
}


    CdmwPlatformMngt::LBGroupSnapshot *
      LBGroupAccessor_impl::get_snapshot (const char *target_element_path)
      throw (CORBA::SystemException, CdmwPlatformMngt::ManagedElementNotFound)
    {
     CdmwPlatformMngt::LBGroup_var lbgroup_obj = get_lbgroup (target_element_path);

     return(lbgroup_obj->get_snapshot( ));

    }


    CdmwPlatformMngt::BalancingStyle LBGroupAccessor_impl::
      get_style (const char *target_element_path)
      throw (CORBA::SystemException, CdmwPlatformMngt::ManagedElementNotFound)
    {
     
     CdmwPlatformMngt::LBGroup_var lbgroup_obj = get_lbgroup (target_element_path);

     return(lbgroup_obj->get_style( ));
    }


    void LBGroupAccessor_impl::set_style (const char *target_element_path,
                                          CdmwPlatformMngt::BalancingStyle bal_style)
      throw (CORBA::SystemException, CdmwPlatformMngt::ManagedElementNotFound)
    {
     CdmwPlatformMngt::LBGroup_var lbgroup_obj = get_lbgroup (target_element_path);

     lbgroup_obj->set_style( bal_style);
    }
    
    CdmwPlatformMngt::ElementType LBGroupAccessor_impl::get_element_type (const char *target_element_path)
    throw (CORBA::SystemException)
    {
        CdmwPlatformMngt::LBGroup_var lbgroup_obj = get_lbgroup (target_element_path);

        return (lbgroup_obj->get_element_type());
    }

    

  }                             // End namespace PlatformMngt
}                               // End namespace Cdmw
