/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include "platformftaccessorlibrary/NotifierAccessor_impl.hpp"
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>

namespace Cdmw
{        // Begin namespace Cdmw

namespace PlatformMngt
{ // Begin namespace PlatformMngt
     
namespace Periodic
{ // Begin namespace Periodic


NotifierAccessor_impl::NotifierAccessor_impl(CdmwPlatformMngt::Periodic::Notifier_ptr periodic_notifier)     
    : m_periodic_notifier(periodic_notifier)     
{                     
}
  
            
NotifierAccessor_impl::~NotifierAccessor_impl () throw ()
{        
}


void NotifierAccessor_impl::shut_down (const char * target_element_path)
    throw (CORBA::SystemException, 
           CdmwPlatformMngt::ManagedElementNotFound)
{
    throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
}

           
void NotifierAccessor_impl::register_observer (const char * target_element_path,
                                               const char * observer_name, 
                                               const CosPropertyService::PropertyNames & property_names,
                                               CdmwPlatformMngt::Periodic::Observer_ptr observer_ref, 
                                               TimeBase::TimeT period)
    throw (CORBA::SystemException, 
           CdmwPlatformMngt::ManagedElementNotFound,
           CdmwPlatformMngt::ElementAlreadyExists)
{
    try
    {
        m_periodic_notifier->register_observer(observer_name, property_names,observer_ref, period);
        
    }catch(const CdmwPlatformMngt::PropertyNameInvalid & e)
    {

        throw CdmwPlatformMngt::ManagedElementNotFound();
        
    }catch(const CdmwPlatformMngt::ElementAlreadyExists & e)
    {
        throw e;
    }
    catch(const CORBA::SystemException & ex)
    {
        throw ;
    }
}
    
    
void NotifierAccessor_impl::register_observer_group (const char * target_element_path,
                                                     const char * observer_name, 
                                                     const CosPropertyService::PropertyNames &  property_names,
                                                     const char * m_miop_group,
                                                     TimeBase::TimeT period)
    throw (CORBA::SystemException,
           CdmwPlatformMngt::ManagedElementNotFound,
           CdmwPlatformMngt::ElementAlreadyExists)
{
    try
    {
        m_periodic_notifier->register_observer_group(observer_name, property_names, m_miop_group, period);

    }catch(const CdmwPlatformMngt::PropertyNameInvalid & e)
    {

        throw CdmwPlatformMngt::ManagedElementNotFound();

    }catch(const CdmwPlatformMngt::ElementAlreadyExists & e)
    {
        throw e;
    }
    catch(const CORBA::SystemException & e)
    {
        throw ;
    }
} 


CdmwPlatformMngt::Periodic::Observer_ptr NotifierAccessor_impl::unregister_observer (const char * target_element_path,
                                                                                     const char * observer_name)
    throw (CORBA::SystemException,CdmwPlatformMngt::ObserverNotFound)
{
    CdmwPlatformMngt::Periodic::Observer_var periodic_observer_obj;
    
    try
    {  
        periodic_observer_obj = m_periodic_notifier->unregister_observer (observer_name);
    }
    catch (const CdmwPlatformMngt::ObserverNotFound&)
    {
        throw CdmwPlatformMngt::ObserverNotFound();
    }
    catch(const CORBA::SystemException & e)
    {
        throw ;
    }
    
    return periodic_observer_obj._retn();                   
}
       
           
char * NotifierAccessor_impl::unregister_observer_group (const char * target_element_path, 
                                                         const char * observer_name)
    throw (CORBA::SystemException,CdmwPlatformMngt::ObserverNotFound)
{
    CORBA::String_var miop_group;
    
    try
    {  
        miop_group = CORBA::string_dup (m_periodic_notifier->unregister_observer_group (observer_name));
    }
    catch (const CdmwPlatformMngt::ObserverNotFound&)
    {
        throw CdmwPlatformMngt::ObserverNotFound();
    }
    catch(const CORBA::SystemException & e)
    {
        throw ;
    }
    
    return miop_group._retn();    
}


} // End namespace Periodic
} // End namespace PlatformMngt
} // End namespace Cdmw


