/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "Foundation/osthreads/MutexGuard.hpp"

#include "platformftaccessorlibrary/ProcessAccessor_impl.hpp"

#include "SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtProcess.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtHost.stub.hpp"

#include "SystemMngt/tools/ProcessIssuerManager.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp"

#include "SystemMngt/tools/ManagedElementKey.hpp"


namespace Cdmw
{
  namespace PlatformMngt
  {

    ProcessAccessor_impl::ProcessAccessor_impl (CdmwPlatformMngt::
                                                ElementRepository_ptr
                                                element_repository)
      throw ():ManagedElementAccessor_impl (element_repository),
      RWEntityContainerAccessor_impl (element_repository),
      RWServiceDefContainerAccessor_impl (element_repository)
    {
    }


    ProcessAccessor_impl::~ProcessAccessor_impl ()
    {
    }


    CdmwPlatformMngt::Process_ptr ProcessAccessor_impl::
      get_process (const char *target_element_path) throw (CdmwPlatformMngt::
                                                           ManagedElementNotFound)
    {
      CdmwPlatformMngt::Process_var process_obj;

      try
      {
        CdmwPlatformMngt::ManagedElement_var element_obj =
          m_element_repository->get_element (target_element_path);

        process_obj = CdmwPlatformMngt::Process::_narrow (element_obj.in ());
      }
      catch (const CdmwPlatformMngt::ElementPathInvalid &)
      {
        throw CdmwPlatformMngt::ManagedElementNotFound (target_element_path);
      }
      catch (const CORBA::SystemException &)
      {
        throw CdmwPlatformMngt::ManagedElementNotFound (target_element_path);
      }

      return process_obj._retn ();
    }

    CdmwPlatformMngt::ProcessType ProcessAccessor_impl::
      get_type (const char *target_element_path) throw (CdmwPlatformMngt::
                                                        ManagedElementNotFound,
                                                        CORBA::SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      return process_obj->type ();
    }

    CORBA::Object_ptr ProcessAccessor_impl::
      get_embedded_service (const char *target_element_path)
      throw (CdmwPlatformMngt::ManagedElementNotFound, CORBA::SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      return process_obj->embedded_service ();
    }

    CdmwPlatformMngt::ProcessPID ProcessAccessor_impl::
      get_pid (const char *target_element_path) throw (CdmwPlatformMngt::
                                                       ManagedElementNotFound,
                                                       CORBA::SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      return process_obj->pid ();
    }

    CORBA::Boolean ProcessAccessor_impl::
      get_created_dynamically (const char *target_element_path)
      throw (CdmwPlatformMngt::ManagedElementNotFound, CORBA::SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      return process_obj->created_dynamically ();
    }



    CORBA::Long ProcessAccessor_impl::
      get_restart_attempt (const char *target_element_path)
      throw (CdmwPlatformMngt::ManagedElementNotFound, CORBA::SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      return process_obj->restart_attempt ();
    }

    void ProcessAccessor_impl::get_attribute (const char *target_element_path,
                                              const char *attr_name,
                                              CORBA::Any_out ret_attr,
                                              CdmwPlatformMngt::
                                              Owner flag) throw (CORBA::
                                                                 SystemException,
                                                                 CdmwPlatformMngt::
                                                                 AttributeNotFound, 
                                 CdmwPlatformMngt::ManagedElementNotFound)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      process_obj->get_attribute (attr_name, ret_attr, flag);
    }

    void ProcessAccessor_impl::set_attribute (const char *target_element_path,
                                              const char *attr_name,
                                              const CORBA::Any & attr,
                                              CdmwPlatformMngt::
                                              Owner flag) throw (CORBA::
                                                                 SystemException,
                                                                 CdmwPlatformMngt::
                                                                 AttributeNotFound,
                                                                 CdmwPlatformMngt::
                                                                 ReadOnlyAttribute, 
                                                                 CdmwPlatformMngt::ManagedElementNotFound)
    {

      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      process_obj->set_attribute (attr_name, attr, flag);

    }

    bool ProcessAccessor_impl::get_run_as_root (const char *target_element_path)
      throw (CdmwPlatformMngt::ManagedElementNotFound, CORBA::SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      return process_obj->get_run_as_root ();
    }

////
/*
CdmwPlatformMngt::ProcessMode ProcessAccessor_impl::get_process_mode(
                             const char* target_element_path)
throw(CdmwPlatformMngt::ManagedElementNotFound,
      CORBA::SystemException)
{
    CdmwPlatformMngt::Process_var process_obj = get_process (target_element_path);
                        
    return process_obj->get_process_mode();
}
*/
///
    bool ProcessAccessor_impl::
      get_memory_resident (const char *target_element_path)
      throw (CdmwPlatformMngt::ManagedElementNotFound, CORBA::SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      return process_obj->get_memory_resident ();
    }

///
    void ProcessAccessor_impl::
      set_memory_resident (const char *target_element_path,
                           bool memory_resident) throw (CdmwPlatformMngt::
                                                        ManagedElementNotFound,
                                                        CORBA::SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      process_obj->set_memory_resident (memory_resident);
    }
///
    CORBA::Long ProcessAccessor_impl::
      get_core_dump_size (const char *target_element_path)
      throw (CdmwPlatformMngt::ManagedElementNotFound, CORBA::SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      return (process_obj->get_core_dump_size ());
    }



    CdmwPlatformMngtBase::TimeStamp ProcessAccessor_impl::
      get_start_time (const char *target_element_path) throw (CdmwPlatformMngt::
                                                              ManagedElementNotFound,
                                                              CORBA::
                                                              SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      return process_obj->start_time ();
    }

    CdmwPlatformMngtBase::TimeStamp ProcessAccessor_impl::
      get_stop_time (const char *target_element_path) throw (CdmwPlatformMngt::
                                                             ManagedElementNotFound,
                                                             CORBA::
                                                             SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      return process_obj->stop_time ();
    }


    CdmwPlatformMngt::ProcessStatus ProcessAccessor_impl::
      get_status (const char *target_element_path,
                  CORBA::String_out status_info) throw (CdmwPlatformMngt::
                                                        ManagedElementNotFound,
                                                        CORBA::SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      return process_obj->get_status (status_info);
    }



    CORBA::Boolean ProcessAccessor_impl::
      is_auto_ending (const char *target_element_path) throw (CdmwPlatformMngt::
                                                              ManagedElementNotFound,
                                                              CORBA::
                                                              SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      return process_obj->is_auto_ending ();
    }

    void ProcessAccessor_impl::set_auto_ending (const char *target_element_path,
                                                CORBA::Boolean auto_ending)
      throw (CdmwPlatformMngt::ManagedElementNotFound,
             CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      process_obj->set_auto_ending (auto_ending);
    }

    char *ProcessAccessor_impl::get_host (const char *target_element_path)
      throw (CdmwPlatformMngt::ManagedElementNotFound, CORBA::SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      CdmwPlatformMngt::Host_var host = process_obj->get_host ();

      return host->element_path ();
    }


    CdmwPlatformMngt::ProcessInfo *
      ProcessAccessor_impl::
      get_process_info (const char *target_element_path)
      throw (CdmwPlatformMngt::ManagedElementNotFound, CORBA::SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      return process_obj->get_process_info ();
    }

    void ProcessAccessor_impl::
      set_process_info (const char *target_element_path,
                        const CdmwPlatformMngt::
                        ProcessInfo & process_info) throw (CdmwPlatformMngt::
                                                           ManagedElementNotFound,
                                                           CdmwPlatformMngt::
                                                           IncompatibleStatus,
                                                           CORBA::
                                                           SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      process_obj->set_process_info (process_info);
    }


    CdmwPlatformMngt::LifeCycleDef *
      ProcessAccessor_impl::
      get_life_cycle (const char *target_element_path) throw (CdmwPlatformMngt::
                                                              ManagedElementNotFound,
                                                              CdmwPlatformMngt::
                                                              IncompatibleType,
                                                              CORBA::
                                                              SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      return process_obj->get_life_cycle ();
    }

    void ProcessAccessor_impl::set_life_cycle (const char *target_element_path,
                                               const CdmwPlatformMngt::
                                               LifeCycleDef & life_cycle)
      throw (CdmwPlatformMngt::ManagedElementNotFound,
             CdmwPlatformMngt::DuplicateStep,
             CdmwPlatformMngt::IncompatibleType,
             CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      process_obj->set_life_cycle (life_cycle);
    }

    CdmwPlatformMngt::CompleteMonitoringInfo ProcessAccessor_impl::
      get_monitoring_info (const char *target_element_path)
      throw (CdmwPlatformMngt::ManagedElementNotFound,
             CdmwPlatformMngt::IncompatibleType, CORBA::SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      return process_obj->get_monitoring_info ();
    }

    void ProcessAccessor_impl::
      set_monitoring_info (const char *target_element_path,
                           const CdmwPlatformMngt::
                           CompleteMonitoringInfo & monitoring_info)
      throw (CdmwPlatformMngt::ManagedElementNotFound,
             CdmwPlatformMngt::IncompatibleType,
             CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      process_obj->set_monitoring_info (monitoring_info);
    }

    CdmwPlatformMngt::ExitAutoRestartInfo *
      ProcessAccessor_impl::
      get_exit_autorestart_info (const char *target_element_path)
      throw (CdmwPlatformMngt::ManagedElementNotFound, CORBA::SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      return process_obj->get_exit_autorestart_info ();
    }

    void ProcessAccessor_impl::
      set_exit_autorestart_info (const char *target_element_path,
                                 const CdmwPlatformMngt::
                                 ExitAutoRestartInfo & exit_autorestart_info)
      throw (CdmwPlatformMngt::ManagedElementNotFound,
             CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      process_obj->set_exit_autorestart_info (exit_autorestart_info);
    }



    CdmwPlatformMngt::ProcessSnapshot *
      ProcessAccessor_impl::
      get_snapshot (const char *target_element_path) throw (CdmwPlatformMngt::
                                                            ManagedElementNotFound,
                                                            CORBA::
                                                            SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      return process_obj->get_snapshot ();
    }

    CdmwPlatformMngt::RequestStatus ProcessAccessor_impl::
      initialise (const char *target_element_path,
                  const CdmwPlatformMngtBase::
                  StartupKind & startup_kind) throw (CdmwPlatformMngt::
                                                     ManagedElementNotFound,
                                                     CdmwPlatformMngt::
                                                     IncompatibleStatus,
                                                     CORBA::SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      return process_obj->initialise (startup_kind);
    }



    CdmwPlatformMngt::RequestStatus ProcessAccessor_impl::
      next_step (const char *target_element_path) throw (CdmwPlatformMngt::
                                                         ManagedElementNotFound,
                                                         CdmwPlatformMngt::
                                                         InvalidStep,
                                                         CdmwPlatformMngt::
                                                         IncompatibleType,
                                                         CdmwPlatformMngt::
                                                         IncompatibleStatus,
                                                         CORBA::SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      return process_obj->next_step ();
    }

    CdmwPlatformMngt::RequestStatus ProcessAccessor_impl::
      run (const char *target_element_path) throw (CdmwPlatformMngt::
                                                   ManagedElementNotFound,
                                                   CdmwPlatformMngt::
                                                   NotReadyToRun,
                                                   CdmwPlatformMngt::
                                                   IncompatibleType,
                                                   CdmwPlatformMngt::
                                                   IncompatibleStatus,
                                                   CORBA::SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      return process_obj->run ();
    }

    CdmwPlatformMngt::RequestStatus ProcessAccessor_impl::
      stop (const char *target_element_path,
            CORBA::Boolean emergency) throw (CdmwPlatformMngt::
                                             ManagedElementNotFound,
                                             CORBA::SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      return process_obj->stop (emergency);
    }

    
    CdmwPlatformMngt::RequestStatus ProcessAccessor_impl::
        stop_process (const char *target_element_path,
                               CORBA::Boolean emergency,
                               const char* issuer) 
    throw (CdmwPlatformMngt::ManagedElementNotFound,
                        CORBA::SystemException)
    {
        try 
        {
            CdmwPlatformMngt::Process_var process_obj = get_process (target_element_path);
            
            CdmwPlatformMngt::RequestStatus request_status = CdmwPlatformMngt::REQUEST_ERROR;
        
            CDMW_MUTEX_GUARD(*(Cdmw::PlatformMngt::ProcessIssuerManager::Instance()->get_process_issuer_mutex(target_element_path)));                        
        
            Cdmw::PlatformMngt::ProcessIssuerManager::Instance()->set_issuer(target_element_path, issuer);

            request_status = process_obj->stop_process (emergency);           

            return request_status;
        }
        catch (const CORBA::SystemException & ex) 
        {
            throw;
        }
        catch (...) 
        {
            throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL, 
                    CORBA::COMPLETED_MAYBE);
        } 
    }
    
    CdmwPlatformMngt::LongRequestStatus ProcessAccessor_impl::
      stop_and_wait (const char *target_element_path,
                     CORBA::Boolean emergency) throw (CdmwPlatformMngt::
                                                      ManagedElementNotFound,
                                                      CORBA::SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      return process_obj->stop_and_wait (emergency);
    }


    
    CdmwPlatformMngt::RequestStatus ProcessAccessor_impl::
      cold_restart (const char *target_element_path,
                    const CdmwPlatformMngtBase::
                    StartupKind & startup_kind,
                    const char* issuer) throw (CdmwPlatformMngt::
                                               ManagedElementNotFound,
                                               CdmwPlatformMngt::
                                               IncompatibleStatus,
                                               CORBA::SystemException)
    {
        try 
        {
            CdmwPlatformMngt::Process_var process_obj =
              get_process (target_element_path);        
            
    CdmwPlatformMngt::RequestStatus request_status = CdmwPlatformMngt::REQUEST_ERROR;

        {
            CDMW_MUTEX_GUARD(*(Cdmw::PlatformMngt::ProcessIssuerManager::Instance()->get_process_issuer_mutex(target_element_path)));
        
            Cdmw::PlatformMngt::ProcessIssuerManager::Instance()->set_issuer(target_element_path, issuer);
        
            request_status = process_obj->cold_restart (startup_kind);
    
        }

        return request_status;
  
    }
        catch (const CORBA::SystemException & ex) 
        {
            throw;
        }
        catch (...) 
        {
            throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL, 
                    CORBA::COMPLETED_MAYBE);
        } 
    }

    
    CdmwPlatformMngt::RequestStatus ProcessAccessor_impl::
      hot_restart(const char *target_element_path,
                   const CdmwPlatformMngtBase::
                   StartupKind & startup_kind,
                   const char* issuer) throw (CdmwPlatformMngt::
                                              ManagedElementNotFound,
                                              CdmwPlatformMngt::
                                              IncompatibleStatus,
                                              CORBA::SystemException)
    {
        try 
        {
            CdmwPlatformMngt::Process_var process_obj =
              get_process (target_element_path);
            
    CdmwPlatformMngt::RequestStatus request_status = CdmwPlatformMngt::REQUEST_ERROR;

        {
            CDMW_MUTEX_GUARD(*(Cdmw::PlatformMngt::ProcessIssuerManager::Instance()->get_process_issuer_mutex(target_element_path)));
        
            Cdmw::PlatformMngt::ProcessIssuerManager::Instance()->set_issuer(target_element_path, issuer);
                
            request_status = process_obj->hot_restart (startup_kind);
    
        }

        return request_status;
            
        }
        catch (const CORBA::SystemException & ex) 
        {
            throw;
        }
        catch (...) 
        {
            throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL, 
                    CORBA::COMPLETED_MAYBE);
        } 
    }


  /*  CdmwPlatformMngt::Children *
      ProcessAccessor_impl::
      get_children (const char *target_element_path) throw (CORBA::
                                                            SystemException)
    {
      try
      {
        CdmwPlatformMngt::Process_var process_obj =
          get_process (target_element_path);
        return process_obj->get_children ();
      }
      catch (const CORBA::SystemException & ex)
      {
        throw;
      }
      catch (...)
      {
        throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                               CORBA::COMPLETED_MAYBE);
      }

    }*/

    CdmwPlatformMngt::ProcessExitInfo ProcessAccessor_impl::
      get_last_exit_code (const char *target_element_path) throw (CORBA::
                                                                  SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      return process_obj->get_last_exit_code ();
    }

    void ProcessAccessor_impl::
      set_indicator_relevant_status (const char *target_element_path,
                                     const bool indicator_relevant_status)
      throw (CORBA::SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      process_obj->set_indicator_relevant_status (indicator_relevant_status);
    }

    bool ProcessAccessor_impl::
      get_indicator_relevant_status (const char *target_element_path)
      throw (CORBA::SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      return process_obj->get_indicator_relevant_status ();
    }


    void ProcessAccessor_impl::set_init_graph (const char *target_element_path,
                                               const CdmwPlatformMngt::
                                               GraphElements & component_graph)
      throw (CORBA::SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      process_obj->set_init_graph (component_graph);
    }

    void ProcessAccessor_impl::set_stop_graph (const char *target_element_path,
                                               const CdmwPlatformMngt::
                                               GraphElements & component_graph)
      throw (CORBA::SystemException)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      process_obj->set_stop_graph (component_graph);
    }

    CdmwPlatformMngt::CcmComponent_ptr ProcessAccessor_impl::
      add_component (const char *target_element_path,
                     CdmwPlatformMngt::
                     ComponentDef & component_def) throw (CORBA::
                                                          SystemException,
                                                          CdmwPlatformMngt::
                                                          ComponentAlreadyExists,
                                                          CdmwPlatformMngt::
                                                          ComponentServerNotFound,
                                                          CdmwPlatformMngt::
                                                          DuplicateEntity,
                                                          CosPropertyService::
                                                          MultipleExceptions,
                                                          CdmwPlatformMngt::
                                                          DuplicateService,
                                                          CdmwPlatformMngt::
                                                          DuplicateStep,
                                                          CdmwPlatformMngt::
                                                          IncompatibleStatus)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      return process_obj->add_component (component_def);
    }

    void ProcessAccessor_impl::
      remove_component (const char *target_element_path,
                        const char *component_name,
                        const char *host_name) throw (CORBA::SystemException,
                                                      CdmwPlatformMngt::
                                                      ComponentNotFound,
                                                      CdmwPlatformMngt::
                                                      ComponentStillActive,
                                                      CdmwPlatformMngt::
                                                      IncompatibleStatus)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);
      process_obj->remove_component (component_name, host_name);
    }

    CdmwPlatformMngt::ElementType ProcessAccessor_impl::
      get_element_type (const char *target_element_path) throw (CORBA::
                                                                SystemException,
                                                                CdmwPlatformMngt::
                                                                ManagedElementNotFound)
    {
      CdmwPlatformMngt::Process_var proc_obj =
        get_process (target_element_path);

      return (proc_obj->get_element_type ());
    }

    CdmwPlatformMngt::ProcessPID ProcessAccessor_impl::
      get_previous_pid (const char *target_element_path) throw (CORBA::
                                                                SystemException,
                                                                CdmwPlatformMngt::
                                                                ManagedElementNotFound)
    {
      CdmwPlatformMngt::Process_var process_obj = get_process (target_element_path);

      return process_obj->previous_pid ();
    
    }

    void ProcessAccessor_impl::
      get_all_components (const char *target_element_path,
                          CORBA::ULong how_many,
                          CdmwPlatformMngt::ElementPaths_out paths,
                          CdmwPlatformMngt::
                          ComponentsIterator_out rest) throw (CORBA::
                                                              SystemException,
                                                              CdmwPlatformMngt::
                                                              NotComponentServer,
                                                              CdmwPlatformMngt::
                                                              ManagedElementNotFound)
    {

      try
      {
        CdmwPlatformMngt::Process_var process_obj =
          get_process (target_element_path);

        CdmwPlatformMngt::CcmComponents_var components;


        process_obj->get_all_components (how_many, components.out (), rest);


        CdmwPlatformMngt::ElementPaths_var component_paths =
          new CdmwPlatformMngt::ElementPaths ();


        CORBA::ULong num_components = components->length ();


        component_paths->length (num_components);


        CORBA::ULong i = 0;

        for (i = 0; i < (num_components); i++)
          {

            component_paths[i] = ((*components)[i])->element_path ();
          }


        paths = component_paths._retn ();

      }
      catch (const CORBA::SystemException & ex)
      {
        throw;
      }
      catch (...)
      {
        throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                               CORBA::COMPLETED_MAYBE);

      }
    }






    char *ProcessAccessor_impl::get_component (const char *target_element_path,
                                               const char *component_name)
      throw (CORBA::SystemException, CdmwPlatformMngt::ComponentNotFound,
             CdmwPlatformMngt::NotComponentServer,
             CdmwPlatformMngt::ManagedElementNotFound)
    {
      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      CdmwPlatformMngt::CcmComponent_var comp =
        process_obj->get_component (component_name);

      return comp->element_path ();


    }

    CORBA::ULong ProcessAccessor_impl::
      get_number_of_components (const char *target_element_path) throw (CORBA::
                                                                        SystemException,
                                                                        CdmwPlatformMngt::
                                                                        NotComponentServer,
                                                                        CdmwPlatformMngt::
                                                                        ManagedElementNotFound)
    {

      CdmwPlatformMngt::Process_var process_obj =
        get_process (target_element_path);

      return(process_obj->get_number_of_components());

    }

    CdmwPlatformMngt::CcmComponent *
      ProcessAccessor_impl::add_component (const char *target_element_path,
                                           const CdmwPlatformMngt::
                                           ComponentDef & component_def)
      throw (CORBA::SystemException, CdmwPlatformMngt::ComponentAlreadyExists,
             CdmwPlatformMngt::ComponentServerNotFound,
             CdmwPlatformMngt::DuplicateEntity,
             CosPropertyService::MultipleExceptions,
             CdmwPlatformMngt::DuplicateService,
             CdmwPlatformMngt::DuplicateStep,
             CdmwPlatformMngt::IncompatibleStatus)
    {
      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    }

  }                             // End namespace PlatformMngt
}                               // End namespace Cdmw
