/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"

#include "platformftaccessorlibrary/RedundancyGroupAccessor_impl.hpp"

#include "SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp"

#include "SystemMngt/tools/ManagedElementKey.hpp"

#include "SystemMngt/platformlibrary/ElementPathsIterator_impl.hpp"



namespace Cdmw
{
  namespace PlatformMngt
  {

// ----------------------------------------------------------------------
// RedundancyGroupAccessor_impl class.
// ----------------------------------------------------------------------
    /*
    RedundancyGroupAccessor_impl::RedundancyGroupAccessor_impl()
       {
       }
    */

    RedundancyGroupAccessor_impl::RedundancyGroupAccessor_impl(
    CdmwPlatformMngt::ElementRepository_ptr element_repository)
    throw() 
    : ManagedElementAccessor_impl(element_repository),
      RWEntityContainerAccessor_impl(element_repository)
         {   
         }



    RedundancyGroupAccessor_impl::~RedundancyGroupAccessor_impl ()
    {
    }


    CdmwPlatformMngt::RedundancyGroup_ptr RedundancyGroupAccessor_impl::
      get_redundancy_group (const char *target_element_path)
      throw (CdmwPlatformMngt::ManagedElementNotFound)
    {
      CdmwPlatformMngt::RedundancyGroup_var redu_group_obj;

      try
      {
        CdmwPlatformMngt::ManagedElement_var element_obj =
          m_element_repository->get_element (target_element_path);

        redu_group_obj =
          CdmwPlatformMngt::RedundancyGroup::_narrow (element_obj.in ());
      }
      catch (const CdmwPlatformMngt::ElementPathInvalid &)
      {
        throw CdmwPlatformMngt::ManagedElementNotFound (target_element_path);
      }
      catch (const CORBA::SystemException &)
      {
        throw CdmwPlatformMngt::ManagedElementNotFound (target_element_path);
      }

      return redu_group_obj._retn ();
    }

    CORBA::ULong RedundancyGroupAccessor_impl::
      get_number_of_entities (const char *target_element_path) throw (CORBA::
                                                                      SystemException)
    {

      CdmwPlatformMngt::RedundancyGroup_var redundancy_group_obj =
        get_redundancy_group (target_element_path);

      return (redundancy_group_obj->get_number_of_entities ());

    }


    void RedundancyGroupAccessor_impl::
      get_all_entity_names (const char *target_element_path,
                            CORBA::ULong how_many,
                            CdmwPlatformMngt::ElementNames_out ele_out,
                            CdmwPlatformMngt::
                            ElementNamesIterator_out iter_out) throw (CORBA::
                                                                      SystemException)
    {

      CdmwPlatformMngt::RedundancyGroup_var redundancy_group_obj =
        get_redundancy_group (target_element_path);

      return (redundancy_group_obj->
              get_all_entity_names (how_many, ele_out, iter_out));

    }

    char *RedundancyGroupAccessor_impl::
      get_group_domain_id (const char *target_element_path) throw (CORBA::
                                                                   SystemException)
    {

      CdmwPlatformMngt::RedundancyGroup_var redundancy_group_obj =
        get_redundancy_group (target_element_path);

      return (redundancy_group_obj->group_domain_id ());
    }


    CORBA::Boolean RedundancyGroupAccessor_impl::
      get_entities (const char *target_element_path,
                    const CdmwPlatformMngt::ElementNames & ele_names,
                    CdmwPlatformMngt::Entities_out ent_out) throw (CORBA::
                                                                   SystemException)
    {

      CdmwPlatformMngt::RedundancyGroup_var redundancy_group_obj =
        get_redundancy_group (target_element_path);

      return (redundancy_group_obj->get_entities (ele_names, ent_out));
    }

    void RedundancyGroupAccessor_impl::
      get_all_entities (const char *target_element_path, CORBA::ULong how_many,
                        CdmwPlatformMngt::Entities_out ent_out,
                        CdmwPlatformMngt::
                        EntitiesIterator_out iter_out) throw (CORBA::
                                                              SystemException)
    {

      CdmwPlatformMngt::RedundancyGroup_var redundancy_group_obj =
        get_redundancy_group (target_element_path);

      redundancy_group_obj->get_all_entities (how_many, ent_out, iter_out);
    }

    void RedundancyGroupAccessor_impl::
      remove_entity (const char *target_element_path,
                     const char *name) throw (CORBA::SystemException)
    {

      CdmwPlatformMngt::RedundancyGroup_var redundancy_group_obj =
        get_redundancy_group (target_element_path);

      redundancy_group_obj->remove_entity (name);
    }

    void RedundancyGroupAccessor_impl::
      get_attribute (const char *target_element_path, const char *name,
                     CORBA::Any_out out_any,
                     CdmwPlatformMngt::Owner owner) 
    throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound, CdmwPlatformMngt::ManagedElementNotFound)
    {

      CdmwPlatformMngt::RedundancyGroup_var redundancy_group_obj =
        get_redundancy_group (target_element_path);

      redundancy_group_obj->get_attribute (name, out_any, owner);
    }


    void RedundancyGroupAccessor_impl::
      set_attribute (const char *target_element_path, const char *name,
                     const CORBA::Any & out_any,
                     CdmwPlatformMngt::Owner own) 
 throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound, CdmwPlatformMngt::ReadOnlyAttribute, CdmwPlatformMngt::ManagedElementNotFound)
    {

      CdmwPlatformMngt::RedundancyGroup_var redundancy_group_obj =
        get_redundancy_group (target_element_path);

      redundancy_group_obj->set_attribute (name, out_any, own);
    }


    CdmwPlatformMngt::ElementType RedundancyGroupAccessor_impl::
      get_element_type (const char *target_element_path) throw (CORBA::
                                                                SystemException)
    {

      CdmwPlatformMngt::RedundancyGroup_var redundancy_group_obj =
        get_redundancy_group (target_element_path);

      return (redundancy_group_obj->get_element_type ());
    }


 /*   CdmwPlatformMngt::Children *
      RedundancyGroupAccessor_impl::
      get_children (const char *target_element_path) throw (CORBA::
                                                            SystemException)
    {

      CdmwPlatformMngt::RedundancyGroup_var redundancy_group_obj =
        get_redundancy_group (target_element_path);

      return (redundancy_group_obj->get_children ());
    }*/

    CORBA::Long RedundancyGroupAccessor_impl::
      get_minimum_size (const char *target_element_path) throw (CORBA::
                                                                SystemException)
    {

      CdmwPlatformMngt::RedundancyGroup_var redundancy_group_obj =
        get_redundancy_group (target_element_path);

      return (redundancy_group_obj->get_minimum_size ());
    }

     CdmwPlatformMngt::TechnicalStatus RedundancyGroupAccessor_impl::
      get_status (const char *target_element_path,
                  CORBA::String_out str_out) throw (CORBA::SystemException)
    {

      CdmwPlatformMngt::RedundancyGroup_var redundancy_group_obj =
        get_redundancy_group (target_element_path);

      return (redundancy_group_obj->get_status (str_out));
    }
    
    void RedundancyGroupAccessor_impl::
      set_minimum_size (const char *target_element_path,
                        CORBA::Long number) throw (CORBA::SystemException)
    {

      CdmwPlatformMngt::RedundancyGroup_var redundancy_group_obj =
        get_redundancy_group (target_element_path);

      redundancy_group_obj->set_minimum_size (number);
    }

    void RedundancyGroupAccessor_impl::
      start_up (const char *target_element_path) throw (CORBA::SystemException)
    {

      CdmwPlatformMngt::RedundancyGroup_var redundancy_group_obj =
        get_redundancy_group (target_element_path);

      redundancy_group_obj->start_up ();
    }

    void RedundancyGroupAccessor_impl::
      shut_down (const char *target_element_path) throw (CORBA::SystemException)
    {

      CdmwPlatformMngt::RedundancyGroup_var redundancy_group_obj =
        get_redundancy_group (target_element_path);

      redundancy_group_obj->shut_down ();
    }

    void RedundancyGroupAccessor_impl::insert (const char *target_element_path,
                                               CdmwPlatformMngt::
                                               ManagedElement *
                                               managed_element) throw (CORBA::
                                                                       SystemException)
    {

      CdmwPlatformMngt::RedundancyGroup_var redundancy_group_obj =
        get_redundancy_group (target_element_path);

      redundancy_group_obj->insert (managed_element);
    }


    void RedundancyGroupAccessor_impl::get_all_components (
                          const char *target_element_path,
                          CORBA::ULong how_many,
                          CdmwPlatformMngt::ElementPaths_out paths,
                          CdmwPlatformMngt::
                          ComponentsIterator_out rest)
      throw (CORBA::SystemException)
    {

      try
      {
        CdmwPlatformMngt::RedundancyGroup_var redundancy_group_obj =
          get_redundancy_group (target_element_path);

        CdmwPlatformMngt::CcmComponents_var components;

        redundancy_group_obj->get_all_components (how_many, components.out (), rest);

        CdmwPlatformMngt::ElementPaths_var component_paths =
          new CdmwPlatformMngt::ElementPaths ();

       CORBA::ULong num_components = components->length ();

       component_paths->length (num_components);

       CORBA::ULong i = 0;

        for (i = 0; i < (num_components); i++)
          {
            component_paths[i] = ((*components)[i])->element_path ();
          }

        paths = component_paths._retn ();

      }
      catch (const CORBA::SystemException & ex)
      {
        throw;
      }
      catch (...)
      {
        throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                               CORBA::COMPLETED_MAYBE);
      }
    }


    char *RedundancyGroupAccessor_impl::
      get_component (const char *target_element_path,
                     const char *component_name,
                     const char *host_name) throw (CORBA::SystemException,
                                                   CdmwPlatformMngt::
                                                   ComponentNotFound,
                                                   CdmwPlatformMngt::
                                                   ManagedElementNotFound)
    {
      CdmwPlatformMngt::RedundancyGroup_var redundancy_group_obj =
         get_redundancy_group (target_element_path);
      CdmwPlatformMngt::CcmComponent_var ccmcomponent;

     try
     {
         ccmcomponent = redundancy_group_obj->get_component (component_name, host_name);
     }
     catch(const CdmwPlatformMngt::ComponentNotFound&)
     {
         throw;
     }

      return ccmcomponent->element_path ();

    }

    CORBA::ULong RedundancyGroupAccessor_impl::
      get_number_of_components (const char *target_element_path) throw (CORBA::
                                                                        SystemException)
    {

      CdmwPlatformMngt::RedundancyGroup_var redundancy_group_obj =
        get_redundancy_group (target_element_path);

      return (redundancy_group_obj->get_number_of_components ());
    }

    void RedundancyGroupAccessor_impl::
      _cxx_delete (const char *target_element_path,
                   CdmwPlatformMngt::ManagedElement *
                   managed_element) throw (CORBA::SystemException)
    {

      CdmwPlatformMngt::RedundancyGroup_var redundancy_group_obj =
        get_redundancy_group (target_element_path);

      redundancy_group_obj->_cxx_delete (managed_element);
    }



  }                             // End namespace PlatformMngt
}                               // End namespace Cdmw
