/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_SERVER_INTERCEPTOR_HPP 
#define INCL_SERVER_INTERCEPTOR_HPP 

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/orbsupport/PortableInterceptor.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/orbsupport/TypeTraits.hpp>
#include <Foundation/orbsupport/Codec.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtCurrent.stub.hpp>

#include <string>
#include <iostream>

namespace Cdmw
{
namespace PlatformMngt
{

    class ServerInterceptor : public PortableInterceptor::ServerRequestInterceptor,
                              public Cdmw::OrbSupport::RefCountLocalObject
    {

    public:
        
        typedef Cdmw::OrbSupport::Codec <std::string> IssuerCodec;

        ServerInterceptor ();
        virtual ~ServerInterceptor ();

        //
        // IDL to C++ Mapping
        //
        virtual char*
        name() throw (CORBA::SystemException);

        virtual void
        destroy() throw (CORBA::SystemException);

        virtual void
        receive_request_service_contexts(
            PortableInterceptor::ServerRequestInfo_ptr ri)
            throw (CORBA::SystemException, PortableInterceptor::ForwardRequest);

        virtual void
        receive_request(PortableInterceptor::ServerRequestInfo_ptr ri)
            throw (CORBA::SystemException, PortableInterceptor::ForwardRequest);

        virtual void
        send_reply(PortableInterceptor::ServerRequestInfo_ptr ri)
            throw (CORBA::SystemException);

        virtual void
        send_other(PortableInterceptor::ServerRequestInfo_ptr ri)
            throw (CORBA::SystemException, PortableInterceptor::ForwardRequest);

        virtual void
        send_exception(PortableInterceptor::ServerRequestInfo_ptr ri)
            throw (CORBA::SystemException, PortableInterceptor::ForwardRequest);               
        
        void
        set_orb(CORBA::ORB_ptr orb);
  
    private:
        
        ServerInterceptor(const ServerInterceptor&);
        void operator=(const ServerInterceptor&);
        
        /*
         *  check if notification is required for the command
        */
        bool check_command(const char* command);
        
        
        IOP::ServiceId m_service_id;
        CORBA::ORB_var m_orb_server;       
        IssuerCodec* m_codec;              

        CdmwPlatformMngt::Current_var m_current;

    }; // End class

} // End namespace Cdmw
}

#endif // INCL_ONINIT_INTERFACE_IMPL_HPP 

