/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_PROXY_CCMCOMPONENT_IMPL_HPP
#define INCL_PLATFORMMNGT_PROXY_CCMCOMPONENT_IMPL_HPP

#include <Foundation/common/Exception.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtComponent.skel.hpp>
//#include <SystemMngt/idllib/CdmwPlatformMngtComponentAgent.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtApplicationAgent.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtEntity.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtEvent.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtHost.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtService.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtDaemon.stub.hpp>

#include "platformftproxy/ManagedElement_impl.hpp"
#include "platformftproxy/ProxyUtil.hpp"
#include "platformftproxy/AccessorLookupService.hpp"
#include "platformftproxy/ProxyCollection.hpp"

#include <string>
#include <set>
#include <map>

namespace Cdmw
{
  namespace PlatformMngt
  {
    namespace Proxy
    {

      class System_impl;

/**
 *Purpose:
 *<p>
 */
      class CcmComponent_impl:public virtual POA_CdmwPlatformMngt::CcmComponent,
        virtual public PortableServer::RefCountServantBase,
        virtual public ManagedElement_impl
      {
      private:
    /**
     * Purpose: <p>
     *
     * The accessor interface provides all operations to be mapped onto.
     */
        CdmwFTSystMngt::CcmComponentAccessor_var m_accessor;

    /** Purpose: <p> protects access to  m_system_accessor */
        Cdmw::OsSupport::Mutex m_accessor_mutex;

    /** Purpose: <p> True if m_system_target_path is valid obejct
    reference, otherwise false */
        bool m_accessor_ok;

    /**
     * Purpose: <p> 
     *
     * The proxy collection
     */
          Cdmw::PlatformMngt::Proxy::ProxyCollection * m_proxy_collection;

    /**
     * Purpose: <p> 
     */
        AccessorLookupService *m_lookup_service;

      private:


      CdmwFTSystMngt::ManagedElementAccessor_ptr 
        accessor_managed_element () throw (CORBA::TRANSIENT);

    /**
     * Purpose: <p> 
     *
     @return Fetches the accessor from lokoup service and returns the object reference
     *@exception CORBA::TRANSIENT Raised if accessor not defined. 
     */
        const CdmwFTSystMngt::CcmComponentAccessor_ptr accessor ()
          throw (CORBA::TRANSIENT);

          Cdmw::Logging::Logger * Get_Logger ();

      private:

    /**
     *Purpose:
     *<p> Copy constructor is not allowed.
     */
          CcmComponent_impl (CcmComponent_impl & rhs);

    /**
     * Purpose:
     * <p> Assignment operator is not allowed.
     */
          CcmComponent_impl & operator= (const CcmComponent_impl & rhs);

      /**
       *Purpose:
       *<p> Default constructor is not allowed.
       *  
       *
       *@exception BadParameterException
       *@exception OutOfMemoryException
       *
       */
          CcmComponent_impl ()
          throw (Cdmw::BadParameterException, Cdmw::OutOfMemoryException);

      public:

        /*
           CcmComponent_impl (CORBA::ORB_ptr orb,
           ProxyCollection * proxy_collection,
           AccessorLookupService * lookup_service,
           CdmwPlatformMngt::RWElementRepository* rep,
           PortableServer::POA* poa)
           throw (Cdmw::BadParameterException, Cdmw::OutOfMemoryException);
         */

          CcmComponent_impl (CORBA::ORB_ptr orb,
                             ProxyCollection * proxy_collection,
                             AccessorLookupService * lookup_service)
          throw (Cdmw::BadParameterException, Cdmw::OutOfMemoryException);

      /**
       *Purpose:
       *<p> Default destructor
       */
        ~CcmComponent_impl () throw ();



        CORBA::ULong get_number_of_entities () throw (CORBA::SystemException);


        void get_all_entity_names (CORBA::ULong,
                                   CdmwPlatformMngt::ElementNames_out,
                                   CdmwPlatformMngt::
                                   ElementNamesIterator_out) throw (CORBA::
                                                                    SystemException);

        CdmwPlatformMngt::Entity *
          get_entity (const char *) throw ( CdmwPlatformMngt::EntityNotFound, CORBA::SystemException);

        CORBA::Boolean get_entities (const CdmwPlatformMngt::ElementNames &,
                                       CdmwPlatformMngt::
                                       Entities_out) throw (CORBA::
                                                            SystemException);

        void get_all_entities (CORBA::ULong how_many,
                               CdmwPlatformMngt::Entities_out ent_out,
                               CdmwPlatformMngt::
                               EntitiesIterator_out ent_iter_out) throw (CORBA::
                                                                         SystemException);

        CdmwPlatformMngt::Entity *
          add_entity (const CdmwPlatformMngt::EntityDef &) throw (CORBA::
                                                                  SystemException);

        void remove_entity (const char *) throw (CdmwPlatformMngt::EntityNotFound, CORBA::SystemException);

        CdmwPlatformMngt::ComponentSnapshot *
          get_snapshot () throw (CORBA::SystemException);

        CdmwPlatformMngt::ComponentStatus get_status (CORBA::
                                                        String_out)
          throw (CORBA::SystemException);

        char *instantiation_id () throw (CORBA::SystemException);

        char *deployment_plan () throw (CORBA::SystemException);

        CdmwPlatformMngtBase::TimeStamp start_time () throw (CORBA::
                                                               SystemException);

        CdmwPlatformMngtBase::TimeStamp stop_time () throw (CORBA::
                                                              SystemException);

        char *get_component_server_name () throw (CORBA::SystemException);

        CORBA::Boolean get_indicator_relevant_status () throw (CORBA::
                                                                 SystemException);

        void set_indicator_relevant_status (CORBA::Boolean) throw (CORBA::
                                                                   SystemException);

        CdmwPlatformMngt::RedundancyGroup *
          get_component_group () throw (CORBA::SystemException);




      };


      //instance it only to try the compilation ...

      /*
         CORBA::ORB_ptr orb;
         ProxyCollection *proxy_collection;
         AccessorLookupService *lookup_service;
         CdmwPlatformMngt::RWElementRepository* rep;
         PortableServer::POA* poa;

         CcmComponent_impl *ccm_comp =
         new CcmComponent_impl (orb, proxy_collection,
         lookup_service, rep, poa); */


    }                           // End namespace Proxy
  }                             // End namespace PlatformMngt
}                               // End namespace Cdmw

#endif                          //INCL_PLATFORMMNGT_CCMCOMPONENT_IMPL_HPP
