/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_PROXY_ENTITY_IMPL_HPP
#define INCL_PLATFORMMNGT_PROXY_ENTITY_IMPL_HPP


#include <Foundation/orbsupport/OrbSupport.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtElementRepository.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtEntity.skel.hpp>

#include "platformftproxy/ProxyUtil.hpp"
#include "platformftproxy/AccessorLookupService.hpp"
#include "platformftproxy/ProxyCollection.hpp"
#include "platformftproxy/ManagedElement_impl.hpp"

#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>

namespace Cdmw
{
namespace PlatformMngt
{
namespace Proxy
{

class System_impl;

/**
 *Purpose:
 *<p> Provides the Entity.
 */
class Entity_impl : virtual public POA_CdmwPlatformMngt::Entity,
             virtual public PortableServer::RefCountServantBase,
            //public ProxyUtil,
            virtual public ManagedElement_impl
                    
{

private:
   /**
     * Purpose: <p>
     *
     * The accessor interface provides all operations to be mapped onto.
     */
    CdmwFTSystMngt::EntityAccessor_var           m_accessor;

    /** Purpose: <p> 
     *
     * Protects access to  m_system_accessor */
    Cdmw::OsSupport::Mutex                       m_accessor_mutex; 

    /** Purpose: <p> True if m_system_target_path is valid obejct
    reference, otherwise false */
    bool                                         m_accessor_ok;

    /**
     * Purpose: <p> 
     *
     * The proxy collection
     */ 
    Cdmw::PlatformMngt::Proxy::ProxyCollection *m_proxy_collection;

    /**
     * Purpose: <p>
     *
     * The lookup service 
     */ 
    AccessorLookupService *m_lookup_service;

private:
    
    /**
     * Purpose: <p> 
     *
     @return Fetches the accessor from lokoup service and returns the object reference
     *@exception CORBA::TRANSIENT Raised if accessor not defined. 
     */ 
    const CdmwFTSystMngt::EntityAccessor_ptr accessor() 
    throw(CORBA::TRANSIENT);
    
    
    CdmwFTSystMngt::ManagedElementAccessor_ptr 
        accessor_managed_element () throw (CORBA::TRANSIENT);
    
    Cdmw::Logging::Logger * Get_Logger ();
    
public:

    /**
     *Purpose:
     *<p> Constructor.
     *  
     *@param orb                The ORB 
     *@param m_proxy_collection The proxy collection provides operations for object reference creation
     *@param lookup_service     The lookup service provides access to accessor object references
     *
     *@exception BadParameterException One parameter is nil
     *@exception OutOfMemoryException Lack of memory.
     *
     */
     Entity_impl (CORBA::ORB_ptr orb,
          ProxyCollection *proxy_collection,
          AccessorLookupService *lookup_service)
     throw (Cdmw::BadParameterException, 
        Cdmw::OutOfMemoryException);

private:
     /**
     *Purpose:
     *<p> Default constructor is not allowed.
     *  
     *@exception BadParameterException
     *@exception OutOfMemoryException
     *
     */
     Entity_impl ()     
     throw (Cdmw::BadParameterException, 
        Cdmw::OutOfMemoryException)   ;
                    
public:           

    /**
     *Purpose:
     *<p> Destructor.
     */
    ~Entity_impl() throw();

 
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Entity/get_status:1.0
     * operation
     */
    char* get_status(CORBA::String_out entity_info)
             throw (CORBA::SystemException);
    
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Entity/set_status:1.0
     * operation
     */
    void set_status(const char* entity_status, const char* entity_info)
             throw (CdmwPlatformMngt::EntityStatusNotAllowed, 
                    CORBA::SystemException);
        

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Entity/set_status_values:1.0
     * operation
     */
    CdmwPlatformMngt::EntityStatusValues* get_status_values()
             throw (CORBA::SystemException);


    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Entity/get_snapshot:1.0
     * operation
     */
     CdmwPlatformMngt::EntitySnapshot* get_snapshot()
             throw (CORBA::SystemException);

};   



} // End namespace Proxy
} // End namespace PlatformMngt
} // End namespace Cdmw

#endif //INCL_PLATFORMMNGT_ENTITY_IMPL_HPP

