/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_PROXY_LBGROUP_IMPL_HPP
#define INCL_PLATFORMMNGT_PROXY_LBGROUP_IMPL_HPP

#include <Foundation/common/Exception.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtLBGroup.skel.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtLBGroup.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtRedundancyGroup.skel.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtRedundancyGroup.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtApplicationAgent.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtEntity.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtEvent.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtHost.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtService.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtDaemon.stub.hpp>

#include "platformftproxy/ProxyUtil.hpp"
#include "platformftproxy/AccessorLookupService.hpp"
#include "platformftproxy/ProxyCollection.hpp"
#include "platformftproxy/RedundancyGroup_impl.hpp"

#include <string>
#include <set>
#include <map>

namespace Cdmw
{
  namespace PlatformMngt
  {
    namespace Proxy
    {

      //class System_impl;

     /**
     *Purpose:
     *<p>
     */
      class LBGroup_impl:public virtual POA_CdmwPlatformMngt::LBGroup,
        public RedundancyGroup_impl
      {

      private:

    /**
     * Purpose: <p> 
     *
     @return Fetches the accessor from lokoup service and returns the object reference
     *@exception CORBA::TRANSIENT Raised if accessor not defined. 
     */
        const CdmwFTSystMngt::LBGroupAccessor_ptr accessor ()
          throw (CORBA::TRANSIENT);

      private:

    /**
     *Purpose:
     *<p> Copy constructor is not allowed.
     */
          LBGroup_impl (LBGroup_impl & rhs);

    /**
     * Purpose:
     * <p> Assignment operator is not allowed.
     */
          LBGroup_impl & operator= (const LBGroup_impl & rhs);

    /**
     *Purpose:
     *<p> Default constructor is not allowed.
     *  
     *
     *@exception BadParameterException
     *@exception OutOfMemoryException
     *
     */
          LBGroup_impl ()
          throw (Cdmw::BadParameterException, Cdmw::OutOfMemoryException);

      public:

          LBGroup_impl (CORBA::ORB_ptr orb,
                  
                        ProxyCollection * proxy_collection,
                        AccessorLookupService * lookup_service)
          throw (Cdmw::BadParameterException, Cdmw::OutOfMemoryException);


          /**
           *Purpose:
           *<p> Default destructor.
           */
          virtual ~LBGroup_impl () throw ();


          CdmwFTSystMngt::ManagedElementAccessor_ptr
                accessor_managed_group ()throw (CORBA::TRANSIENT);

          CdmwFTSystMngt::RedundancyGroupAccessor_ptr
                accessor_redu_group ()throw (CORBA::SystemException);

          CdmwPlatformMngt::LBGroupSnapshot *
           get_snapshot () throw (CORBA::SystemException);

          CdmwPlatformMngt::BalancingStyle get_style () throw (CORBA::
                                                               SystemException);

          void set_style (CdmwPlatformMngt::BalancingStyle) throw (CORBA::
                                                               SystemException);

          Cdmw::Logging::Logger * Get_Logger ();


      private:
          /**
          * Purpose: <p>
          *
          * The accessor interface provides all operations to be mapped onto.
          */
          CdmwFTSystMngt::LBGroupAccessor_var m_accessor;

          /** Purpose: <p> protects access to  m_system_accessor */
          Cdmw::OsSupport::Mutex m_accessor_mutex;

          /** Purpose: <p> True if m_system_target_path is valid obejct
          reference, otherwise false */
          bool m_accessor_ok;

         /**
          * Purpose: <p> 
          *
          * The proxy collection
          */

          Cdmw::PlatformMngt::Proxy::ProxyCollection * m_proxy_collection;

        /**
         * Purpose: <p> 
         */
        AccessorLookupService *m_lookup_service;

      };


      //instance it only to try the compilation ...
    /*
        CORBA::ORB_ptr orb;
      ProxyCollection *proxy_collection;
      AccessorLookupService *lookup_service;

      LBGroup_impl *lbgroup =
        new LBGroup_impl (orb, proxy_collection, lookup_service); */


    }                           // End namespace Proxy
  }                             // End namespace PlatformMngt
}                               // End namespace Cdmw

#endif                          //INCL_PLATFORMMNGT_LBGROUP_IMPL_HPP
