/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_PROXY_MANAGED_ELEMENT_IMPL_HPP
#define INCL_PLATFORMMNGT_PROXY_MANAGED_ELEMENT_IMPL_HPP



#include "platformftproxy/ProxyUtil.hpp"
#include "platformftproxy/AccessorLookupService.hpp"
#include "platformftproxy/ProxyCollection.hpp"

#include <Foundation/orbsupport/OrbSupport.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtEntity.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtElementRepository.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtCurrent.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtManagedElement.skel.hpp>

#include <Foundation/logging/FunctionLogger.hpp>


namespace Cdmw
{
  namespace PlatformMngt
  {
    namespace Proxy
    {

    //class RWEntityContainer_impl;

        /**
         *Purpose:
         *<p>
          */
    class ManagedElement_impl:
      virtual public POA_CdmwPlatformMngt::ManagedElement,
      virtual public PortableServer::RefCountServantBase,
      public ProxyUtil
      
    {

    protected:

          /**
           *Purpose:
           *<p> Copy constructor is not allowed.
           */
      ManagedElement_impl (ManagedElement_impl & rhs);

          /**
           * Purpose:
           * <p> Assignment operator is not allowed.
           */
      ManagedElement_impl & operator = (const ManagedElement_impl & rhs);

    public:


         /**
         *Purpose:
         *<p> Constructor.
         *
         * @param element_repository the element repository object reference
         *
         */
        ManagedElement_impl (CORBA::ORB_ptr orb) throw ();
 
         /**
         *Purpose:
         *<p> Default destructor.
         */
      virtual ~ManagedElement_impl () throw();

    
      virtual Cdmw::Logging::Logger* Get_Logger () = 0;


      virtual CdmwFTSystMngt::ManagedElementAccessor_ptr 
                accessor_managed_element () throw (CORBA::TRANSIENT) = 0;
                
     /**
     *Purpose:
     *<p> facade to get an attribute
     *
     */
     virtual void get_attribute(const char* attr_name, CORBA::Any_out ret_attr, CdmwPlatformMngt::Owner flag)  
        throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound);

    /**
     *Purpose:
     *<p> facade to set an attribute
     *
     */
     virtual void set_attribute(const char* attr_name, const CORBA::Any& attr, CdmwPlatformMngt::Owner flag)  
        throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound,  CdmwPlatformMngt::ReadOnlyAttribute);


      virtual CdmwPlatformMngt::SupervisionObserver *
        register_observer (const char *,
                           CdmwPlatformMngt::
                           SupervisionObserver *) throw (CORBA::
                                                         SystemException);


      virtual CdmwPlatformMngt::SupervisionObserver *
        register_proxy_observer (const char *, const char *,
                                 CdmwPlatformMngt::
                                 SupervisionObserver *) throw (CORBA::
                                                               SystemException);

      virtual CdmwPlatformMngt::SupervisionObserver *
        unregister_observer (const char *) throw (CdmwPlatformMngt::ObserverNotFound, 
                                                  CORBA::SystemException);

      virtual void register_observer_ext (const char *,
                                  CdmwPlatformMngt::RegistrationDepth,
                                  CdmwPlatformMngt::
                                  SupervisionObserver *) throw (CORBA::SystemException,  CdmwPlatformMngt::ElementAlreadyExists);


      virtual void unregister_observer_ext (const char *) throw (CORBA::SystemException, CdmwPlatformMngt::ObserverNotFound);
      
      virtual void define_property (const char *,
                            const CORBA::Any &)           
           throw (CosPropertyService::InvalidPropertyName,
           CosPropertyService::ConflictingProperty,
           CosPropertyService::UnsupportedTypeCode,
           CosPropertyService::UnsupportedProperty,
           CosPropertyService::ReadOnlyProperty,
           CORBA::SystemException) ;

      virtual void define_properties (const CosPropertyService::
                              Properties &)           
           throw (CosPropertyService::MultipleExceptions,
           CORBA::SystemException) ; 

      virtual CORBA::ULong get_number_of_properties () throw (CORBA::SystemException);

      virtual void get_all_property_names (CORBA::ULong,
                                   CosPropertyService::PropertyNames_out,
                                   CosPropertyService::
                                   PropertyNamesIterator_out) throw (CORBA::
                                                                     SystemException);

      virtual CORBA::Any *
        get_property_value (const char *)
                 throw (CosPropertyService::PropertyNotFound,
                 CosPropertyService::InvalidPropertyName,
                 CORBA::SystemException);

      virtual CORBA::Boolean get_properties (const CosPropertyService::
                                       PropertyNames &,
                                       CosPropertyService::
                                       Properties_out) throw (CORBA::
                                                              SystemException);

      virtual void get_all_properties (CORBA::ULong, CosPropertyService::Properties_out,
                               CosPropertyService::
                               PropertiesIterator_out) throw (CORBA::
                                                              SystemException);

      virtual void delete_property (const char *) 
           throw (CosPropertyService::PropertyNotFound,
           CosPropertyService::InvalidPropertyName,
           CosPropertyService::FixedProperty,
           CORBA::SystemException) ;

      virtual void delete_properties (const CosPropertyService::
                              PropertyNames &) 
       throw (CosPropertyService::MultipleExceptions,
           CORBA::SystemException);

      virtual CORBA::Boolean delete_all_properties () throw (CORBA::SystemException);

      virtual CORBA::Boolean is_property_defined (const char *)
         throw (CosPropertyService::InvalidPropertyName, CORBA::SystemException) ;
                                                         

      virtual void get_allowed_property_types (CosPropertyService::
                                       PropertyTypes_out) throw (CORBA::
                                                                 SystemException);


      virtual void get_allowed_properties (CosPropertyService::
                                   PropertyDefs_out) throw (CORBA::
                                                            SystemException);


      virtual void define_property_with_mode (const char *, const CORBA::Any &,
                                      CosPropertyService::
                                      PropertyModeType) 
       throw (CosPropertyService::InvalidPropertyName,
           CosPropertyService::ConflictingProperty,
           CosPropertyService::UnsupportedTypeCode,
           CosPropertyService::UnsupportedProperty,
           CosPropertyService::UnsupportedMode,
           CosPropertyService::ReadOnlyProperty,
           CORBA::SystemException);
                                      


     virtual void define_properties_with_modes (const CosPropertyService::
                                         PropertyDefs &) 
       throw (CosPropertyService::MultipleExceptions,
           CORBA::SystemException);
                                         

      /************************/

      virtual CosPropertyService::PropertyModeType 
        get_property_mode (const char *) 
         throw (CosPropertyService::PropertyNotFound,
           CosPropertyService::InvalidPropertyName,
           CORBA::SystemException) ;


      virtual CORBA::Boolean 
        get_property_modes (const CosPropertyService::PropertyNames &,
                            CosPropertyService::
                            PropertyModes_out) throw (CORBA::SystemException);


      virtual void 
        set_property_mode (const char *,
                           CosPropertyService::PropertyModeType) 
          throw (CosPropertyService::InvalidPropertyName,
           CosPropertyService::PropertyNotFound,
           CosPropertyService::UnsupportedMode,
           CORBA::SystemException) ;


      virtual void 
        set_property_modes (const CosPropertyService::
                            PropertyModes &)
       throw (CosPropertyService::MultipleExceptions,
           CORBA::SystemException) ;


     virtual  void 
        register_all_properties_observer (const char *,
                                          CdmwPlatformMngt::
                                          SupervisionObserver *,
                                          CORBA::Boolean) 
           throw (CORBA::SystemException, CdmwPlatformMngt::ElementAlreadyExists);                                                         


     virtual void 
        unregister_all_properties_observer (const char *) 
         throw (CdmwPlatformMngt::ObserverNotFound, CORBA::SystemException);                                                         


     virtual void 
        register_property_observer (const char *,
                                    CdmwPlatformMngt::SupervisionObserver *,
                                    const CosPropertyService::
                                    PropertyNames &) 
         throw (CdmwPlatformMngt::PropertyInListNotFound,
           CdmwPlatformMngt::ElementAlreadyExists,
           CORBA::SystemException) ;


      virtual void 
        unregister_property_observer (const char *,
                                      const CosPropertyService::
                                      PropertyNames &) 
        throw (CdmwPlatformMngt::PropertyInListNotFound,
           CdmwPlatformMngt::ObserverNotFound,
           CORBA::SystemException);                                                      


     virtual char *name () throw (CORBA::SystemException);


     virtual char *
        element_path () throw (CORBA::SystemException);


     virtual CdmwPlatformMngt::ManagedElement *
        get_parent () throw (CORBA::SystemException);


     virtual CdmwPlatformMngt::ElementType 
        get_element_type () throw (CORBA::SystemException);


     virtual CdmwPlatformMngt::Children*
        get_children () throw (CORBA::SystemException);

      protected:
        //no-corba method to set the logger
    void set_logger( Cdmw::Logging::Logger* logger  );
     
        //void set_accessor(
          //CdmwFTSystMngt::ManagedElementAccessor_ptr accessor  );

        //no-corba method to set the ptr of the son
        void set_managed_element(
          ManagedElement_impl * manag  );

        //no-corba method to set the proxy collection
        void set_proxy_collection( ProxyCollection * proxy_collection);

      protected:

    /**
     * Purpose:
     *<p> Custom SMG PICurrent object reference.
     */
         CdmwPlatformMngt::Current_var m_current;

      private:
         Cdmw::Logging::Logger* m_logger ;

         CdmwFTSystMngt::ManagedElementAccessor_ptr m_accessor ;
         ManagedElement_impl *m_managed_element;
         ProxyCollection *m_proxy_collection;

    };

    //to try compilation ...
      /*
      CdmwPlatformMngt::RWElementRepository_ptr element_repository;
      PortableServer::POA_ptr poa;

    ManagedElement_impl *manag =
      new ManagedElement_impl (element_repository, poa);
      */


   }
  } 
}

#endif                          //INCL_PLATFORMMNGT_PROXY_MANAGED_ELEMENT_IMPL_HPP
