/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_PROXY_NOTIFIER_IMPL_HPP
#define INCL_PLATFORMMNGT_PROXY_NOTIFIER_IMPL_HPP

#include <idllib/CdmwPlatformMngtPeriodicNotifier.skel.hpp>
#include "platformftproxy/ProxyUtil.hpp"
#include "platformftproxy/ProxyCollection.hpp"
#include "platformftproxy/AccessorLookupService.hpp"



namespace Cdmw
{ // Begin namespace Cdmw

namespace PlatformMngt
{ // Begin namespace PlatformMngt
        
namespace Proxy
{ //namespace Proxy
        
namespace Periodic
{ // Begin namespace Periodic 
    
    
class Notifier_impl: public virtual POA_CdmwPlatformMngt::Periodic::Notifier,
                     public virtual PortableServer::RefCountServantBase,
                     public ProxyUtil
{
 
private:
                
    /**
    * Purpose: <p>
    *
    * The accessor interface provides all operations to be mapped onto.
    */
    CdmwFTSystMngt::Periodic::NotifierAccessor_var m_accessor;   
                
    /** Purpose: <p> True if periodic notifier is valid obejct
    reference, otherwise false */
    bool    m_accessor_ok;   
                
    /**
    *Purpose:
    *<p> The proxy collection
    */
    Cdmw::PlatformMngt::Proxy::ProxyCollection *m_proxy_collection;
                
    /**
    * Purpose: <p> 
    */ 
    AccessorLookupService *m_lookup_service; 
                         

public:     
        
        
    /**
     *Purpose:
     *<p> Constructor.
     *     
     *@param orb           the orb reference
     *@param m_proxy_collection The proxy collection provides operations for object reference creation
     *@param lookup_service     The lookup service provides access to accessor object references
     *
     *@exception BadParameterException if the parameter is invalid.
     *@exception OutOfMemoryException Lack of resource.
     */                
    Notifier_impl(CORBA::ORB_ptr orb, 
                  ProxyCollection *proxy_collection, 
                  AccessorLookupService *lookup_service)
    throw (Cdmw::BadParameterException, 
           Cdmw::OutOfMemoryException);
           
           
    /**
     *Purpose:
     *<p> Default destructor.
     */                    
    virtual ~Notifier_impl() throw();
    
          
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Periodic/Notifier/register_observer:1.0
     * operation
     */             
    void register_observer (const char * observer_name, const CosPropertyService::PropertyNames & property_names, 
                            CdmwPlatformMngt::Periodic::Observer_ptr observer_ref,TimeBase::TimeT period)
    throw (CORBA::SystemException,
           CdmwPlatformMngt::PropertyNameInvalid,
           CdmwPlatformMngt::ElementAlreadyExists);
    
    
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Periodic/Notifier/register_observer_group:1.0
     * operation
     */      
    void register_observer_group (const char * observer_name, const CosPropertyService::PropertyNames & 
                                  property_names, const char * m_miop_group, TimeBase::TimeT period)
    throw (CORBA::SystemException,
           CdmwPlatformMngt::PropertyNameInvalid,
           CdmwPlatformMngt::ElementAlreadyExists);
    
         
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Periodic/Notifier/unregister_observer:1.0
     * operation
     */              
    CdmwPlatformMngt::Periodic::Observer_ptr unregister_observer (const char * observer_name)
    throw (CORBA::SystemException, CdmwPlatformMngt::ObserverNotFound);
    
    
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Periodic/Notifier/unregister_observer_group:1.0
     * operation
     */         
    char * unregister_observer_group (const char * observer_name)
    throw (CORBA::SystemException, CdmwPlatformMngt::ObserverNotFound);
     
     
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Periodic/Notifier/periodic_notifier_id:1.0
     * operation
     */                  
    char * periodic_notifier_id() throw (CORBA::SystemException);
    
    
private:

    /**
     * Purpose: <p> 
     *
     @return Fetches the accessor from lokoup service and returns the object reference
     *@exception CORBA::TRANSIENT Raised if accessor not defined. 
     */     
    const CdmwFTSystMngt::Periodic::NotifierAccessor_ptr accessor() 
    throw(CORBA::TRANSIENT);
        
};


} // End namespace Proxy
} // End namespace Periodic
} // End namespace PlatformMngt
} // End namespace Cdmw
#endif /*INCL_PLATFORMMNGT_PROXY_NOTIFIER_IMPL_HPP*/
