/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_PROXY_PROCESS_HPP
#define INCL_PLATFORMMNGT_PROXY_PROCESS_HPP

#include <Foundation/common/Exception.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtEvent.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtProcess.skel.hpp>

#include "platformftproxy/ProxyUtil.hpp"
#include "platformftproxy/ManagedElement_impl.hpp"
// #include "platformftproxy/System_impl.hpp"
#include "platformftproxy/AccessorLookupService.hpp"
#include "platformftproxy/ProxyCollection.hpp"

namespace Cdmw
{
namespace PlatformMngt
{
namespace Proxy
{

class System_impl;

/**
 *Purpose:
 *<p>
 */
class Process_impl : virtual public POA_CdmwPlatformMngt::Process,
                     virtual public PortableServer::RefCountServantBase,
                     //public ProxyUtil,
                     virtual public ManagedElement_impl
{
private:
    /**
     * Purpose: <p>
     *
     * The accessor interface provides all operations to be mapped onto.
     */
    CdmwFTSystMngt::ProcessAccessor_var m_accessor;

    /** Purpose: <p> 
     *
     * Protects access to  m_system_accessor */
    Cdmw::OsSupport::Mutex m_accessor_mutex; 

    /** Purpose: <p> 
     *
     * True if m_system_target_path is valid obejct reference,
     * otherwise false */
    bool m_accessor_ok;

    /**
     * Purpose: <p> 
     *
     * The proxy collection
     */ 
    Cdmw::PlatformMngt::Proxy::ProxyCollection *m_proxy_collection;

    /**
     * Purpose: <p> 
     */ 
    AccessorLookupService *m_lookup_service;

private:
        
    /**
     * Purpose: <p> 
     *
     @return Fetches the accessor from lokoup service and returns the object reference
     *@exception CORBA::TRANSIENT Raised if accessor not defined. 
     */ 
    const CdmwFTSystMngt::ProcessAccessor_ptr accessor() 
    throw(CORBA::TRANSIENT);
    
    
    CdmwFTSystMngt::ManagedElementAccessor_ptr 
        accessor_managed_element () throw (CORBA::TRANSIENT);
   
    Cdmw::Logging::Logger * Get_Logger (); 


    /**
     *Purpose:
     *<p> Copy constructor is not allowed.
     */
    Process_impl(const Process_impl& rhs);
    
     /**
     *Purpose:
     *<p> Default constructor is not allowed.
     *
     *@exception BadParameterException
     *@exception OutOfMemoryException
     */
     Process_impl ()     
          throw (Cdmw::BadParameterException,
         Cdmw::OutOfMemoryException);

    /**
     * Purpose:
     * <p> Assignment operator is not allowed.
     */ 
    Process_impl& operator=(const Process_impl& rhs);

 
public:

    /**
     *Purpose:
     *<p> Constructor.
     *  
     *@param orb                The ORB 
     *@param m_proxy_collection The proxy collection provides operations for object reference creation
     *@param lookup_service     The lookup service provides access to accessor object references
     *
     *@exception BadParameterException One parameter is nil.
     *@exception OutOfMemoryException  Lack of memory.
     *
     */
    Process_impl (CORBA::ORB_ptr orb,
          ProxyCollection *proxy_collection,
          AccessorLookupService *lookup_service)
    throw (Cdmw::BadParameterException, 
           Cdmw::OutOfMemoryException);


    /**
     *Purpose:
     *<p> Default destructor.
     */
    virtual
    ~Process_impl() throw();

public:


    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/type:1.0
     * operation
     */
    virtual
    CdmwPlatformMngt::ProcessType type()
            throw(CORBA::SystemException);

  
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_status:1.0
     * operation
     */
    virtual
    CdmwPlatformMngt::ProcessStatus get_status(
        CORBA::String_out status_info)
            throw(CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_memory_resident:1.0
     * operation
     */
    virtual
    bool get_memory_resident()
            throw(CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_memory_resident:1.0
     * operation
     */
    virtual
    void set_memory_resident(bool memory_resident)
            throw(CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_core_dump_size:1.0
     * operation
     */
    virtual
    CORBA::Long get_core_dump_size()
            throw(CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_run_as_root:1.0
     * operation
     */
    virtual
    bool get_run_as_root()
            throw(CORBA::SystemException);




    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/embedded_service:1.0
     * operation
     */
    virtual CORBA::Object_ptr embedded_service()
            throw(CORBA::SystemException);
     
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/pid:1.0
     * operation
     */       
    virtual CdmwPlatformMngt::ProcessPID pid()
            throw(CORBA::SystemException);
                
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/created_dynamically:1.0
     * operation
     */
    virtual CORBA::Boolean created_dynamically()
            throw(CORBA::SystemException);
    
   
    
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/restart_attempt:1.0
     * operation
     */      
    virtual CORBA::ULong restart_attempt()
            throw(CORBA::SystemException);
     
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/start_time:1.0
     * operation
     */     
    virtual CdmwPlatformMngtBase::TimeStamp start_time()
            throw(CORBA::SystemException);
    
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/stop_time:1.0
     * operation
     */    
    virtual CdmwPlatformMngtBase::TimeStamp stop_time()
            throw(CORBA::SystemException);
   

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/is_auto_ending:1.0
     * operation
     */
    virtual
    CORBA::Boolean is_auto_ending()
      throw(CORBA::SystemException);
      
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_auto_ending:1.0
     * operation
     */
    virtual
    void set_auto_ending(CORBA::Boolean auto_ending)
        throw (CdmwPlatformMngt::IncompatibleStatus,
               CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_host:1.0
     * operation
     */
    virtual
    CdmwPlatformMngt::Host_ptr get_host()
      throw (CORBA::SystemException);

   /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_create_process_result:1.0
     * operation
     */
    virtual 
    void set_create_process_result (
     ::CdmwPlatformMngt::ProcessCommandResult result,
     const char * error_info,
     const char * error_issuer,
     ::CdmwPlatformMngt::ProcessDelegateWrapper_ptr process_delegate_wrap)
    throw (CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_process_info:1.0
     * operation
     */
    virtual
    CdmwPlatformMngt::ProcessInfo* get_process_info()
      throw (CORBA::SystemException);
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_process_info:1.0
     * operation
     */
    virtual
    void set_process_info(const CdmwPlatformMngt::ProcessInfo& process_info)
            throw (CdmwPlatformMngt::IncompatibleStatus,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_life_cycle:1.0
     * operation
     */
    virtual
    CdmwPlatformMngt::LifeCycleDef* get_life_cycle()
            throw (CdmwPlatformMngt::IncompatibleType,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_life_cycle:1.0
     * operation
     */
    virtual
    void set_life_cycle (const CdmwPlatformMngt::LifeCycleDef& life_cycle)
            throw (CdmwPlatformMngt::DuplicateStep,
                   CdmwPlatformMngt::IncompatibleType,
                   CdmwPlatformMngt::IncompatibleStatus,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_monitoring_info:1.0
     * operation
     */
    virtual
    CdmwPlatformMngt::CompleteMonitoringInfo get_monitoring_info()
            throw (CdmwPlatformMngt::IncompatibleType,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_monitoring_info:1.0
     * operation
     */
    virtual
    void set_monitoring_info(
        const CdmwPlatformMngt::CompleteMonitoringInfo& monitoring_info)
            throw (CdmwPlatformMngt::IncompatibleType,
                   CdmwPlatformMngt::IncompatibleStatus,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_exit_autorestart_info:1.0
     * operation
     */
    virtual CdmwPlatformMngt::ExitAutoRestartInfo* get_exit_autorestart_info ()
             throw (CORBA::SystemException);
             
             
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_exit_autorestart_info:1.0
     * operation
     */
    virtual void set_exit_autorestart_info (
        const CdmwPlatformMngt::ExitAutoRestartInfo & exit_autorestart_info)
             throw (CdmwPlatformMngt::IncompatibleStatus,
                    CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_snapshot:1.0
     * operation
     */
    virtual
    CdmwPlatformMngt::ProcessSnapshot* get_snapshot()
      throw (CORBA::SystemException);
      
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/initialise:1.0
     * operation
     */
    virtual
    CdmwPlatformMngt::RequestStatus initialise(
        const CdmwPlatformMngtBase::StartupKind& startup_kind)
            throw (CdmwPlatformMngt::IncompatibleStatus,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/next_step:1.0
     * operation
     */
    virtual
    CdmwPlatformMngt::RequestStatus next_step()
            throw (CdmwPlatformMngt::InvalidStep,
                   CdmwPlatformMngt::IncompatibleType,
                   CdmwPlatformMngt::IncompatibleStatus,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/run:1.0
     * operation
     */
    virtual
    CdmwPlatformMngt::RequestStatus run()
            throw (CdmwPlatformMngt::NotReadyToRun,
                   CdmwPlatformMngt::IncompatibleType,
                   CdmwPlatformMngt::IncompatibleStatus,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/stop:1.0
     * operation
     */
    virtual
    CdmwPlatformMngt::RequestStatus stop(CORBA::Boolean emergency)
      throw (CORBA::SystemException);

    
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/stop_and_wait:1.0
     * operation
     */
    virtual
    CdmwPlatformMngt::LongRequestStatus stop_and_wait(CORBA::Boolean emergency)
      throw (CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/cold_restart:1.0
     * operation
     */  
    virtual CdmwPlatformMngt::RequestStatus cold_restart (
        const CdmwPlatformMngtBase::StartupKind & startup_kind)
      throw (CORBA::SystemException, CdmwPlatformMngt::IncompatibleStatus);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/hot_restart:1.0
     * operation
     */
    virtual CdmwPlatformMngt::RequestStatus hot_restart (
        const CdmwPlatformMngtBase::StartupKind & startup_kind)
      throw (CORBA::SystemException, CdmwPlatformMngt::IncompatibleStatus);
      

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/EntityContainer/get_number_of_entities:1.0
     * operation
     */
    virtual
    CORBA::ULong get_number_of_entities()
        throw (CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/EntityContainer/get_all_entity_names:1.0
     * operation
     */
    virtual
    void get_all_entity_names(
        CORBA::ULong how_many,
        CdmwPlatformMngt::ElementNames_out entity_names,
        CdmwPlatformMngt::ElementNamesIterator_out rest)
            throw (CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/EntityContainer/get_entity:1.0
     * operation
     */
    virtual
    CdmwPlatformMngt::Entity_ptr get_entity(const char* entity_name)
            throw (CdmwPlatformMngt::EntityNotFound,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/EntityContainer/get_entities:1.0
     * operation
     */
    virtual
    CORBA::Boolean get_entities(
        const CdmwPlatformMngt::ElementNames& entity_names,
        CdmwPlatformMngt::Entities_out nentities)
            throw (CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/EntityContainer/get_all_entities:1.0
     * operation
     */
    virtual
    void get_all_entities(
        CORBA::ULong how_many,
        CdmwPlatformMngt::Entities_out nentities,
        CdmwPlatformMngt::EntitiesIterator_out rest)
            throw (CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/RWEntityContainer/add_entity:1.0
     * operation
     */
    virtual
    CdmwPlatformMngt::Entity_ptr add_entity(const CdmwPlatformMngt::EntityDef & entity_def)
            throw (CdmwPlatformMngt::EntityAlreadyExists,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/RWEntityContainer/remove_entity:1.0
     * operation
     */
    virtual
    void remove_entity(const char* entity_name)
            throw (CdmwPlatformMngt::EntityNotFound,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngtService/ServiceDefContainer/get_service_def:1.0
     * operation
     */
    virtual
    CdmwPlatformMngtService::ServiceProviderID* get_service_def(
        const char* service_name)
            throw (CdmwPlatformMngtService::ServiceNotFound,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngtService/RWServiceDefContainer/add_service_def:1.0
     * operation
     */
    virtual
    void add_service_def(
        const CdmwPlatformMngtService::ServiceDef& service_def)
            throw (CdmwPlatformMngtService::ServiceAlreadyExists,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngtService/RWServiceDefContainer/remove_service_def:1.0
     * operation
     */
    virtual
    void remove_service_def(const char* service_name)
            throw (CdmwPlatformMngtService::ServiceNotFound,
                   CORBA::SystemException);

    
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/RWServiceDefContainer/remove_all_service_defs:1.0
     * operation
     */  
    virtual 
    void remove_all_service_defs ()
     throw (CORBA::SystemException);
    
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/PullMonitorable/is_alive:1.0
     * operation
     */
    virtual 
     void is_alive() throw (CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_create_process_result:1.0
     * operator
     */
     virtual
     void set_create_process_result (
        CdmwPlatformMngt::ProcessCommandResult result,
        const char* result_info,
        CdmwPlatformMngt::ProcessDelegateWrapper_ptr process_delegate_wrap)
            throw (CORBA::SystemException);

     
     /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_initialisation_result:1.0
     * operator
     *
     * <p> Not iplemented - raises NO_IMPLEMENT exception. The
     * operation is for internal use in SystemMngt application only.
     */
     virtual
     void set_initialisation_result (CdmwPlatformMngt::ProcessCommandResult result)
            throw (CORBA::SystemException);
     
     /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_next_step_result:1.0
     * operator
     *
     * <p> Not iplemented - raises NO_IMPLEMENT exception. The
     * operation is for internal use in SystemMngt application only.
     */
     virtual
     void set_next_step_result (CdmwPlatformMngt::ProcessCommandResult result,
                                CORBA::ULong step_nbr)
            throw (CORBA::SystemException);
     
     /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_run_result:1.0
     * operator
     *
     * <p> Not iplemented - raises NO_IMPLEMENT exception. The
     * operation is for internal use in SystemMngt application only.
     */
     virtual
     void set_run_result (CdmwPlatformMngt::ProcessCommandResult result)
            throw (CORBA::SystemException);
     
     /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_stop_result:1.0
     * operator
     *
     * <p> Not iplemented - raises NO_IMPLEMENT exception. The
     * operation is for internal use in SystemMngt application only.
     */
     virtual
     void set_stop_result (CdmwPlatformMngt::ProcessCommandResult result)
            throw (CORBA::SystemException);


     /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_last_exit_code:1.0
     * operation
     * @return last_exit_code attribute 
     */
     CdmwPlatformMngt::ProcessExitInfo get_last_exit_code()throw (CORBA::SystemException);

     /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_indicator_relevant_status:1.0
     * operation
     * @param set_indicator_relevant_status attribute 
     */
     virtual void set_indicator_relevant_status(const bool indicator_relevant_status) throw (CORBA::SystemException);

     /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_indicator_relevant_status:1.0
     * operation
     * @return get_indicator_relevant_status attribute 
     */
     virtual bool get_indicator_relevant_status()  throw (CORBA::SystemException);

        
    /**
     *Purpose:
     *<p> add component to process
     * @param component_def Component Definition
     * @exception CdmwPlatformMngt::ComponentAlreadyExists
     * @exception ComponentAlreadyExists
     * @exception ComponentServerNotFound
     * @exception DuplicateEntity
     * @exception DuplicateStep
     * @exception IncompatibleStatus
     */

    virtual CdmwPlatformMngt::CcmComponent_ptr add_component(CdmwPlatformMngt::ComponentDef& component_def)
       throw (CORBA::SystemException, CdmwPlatformMngt::ComponentAlreadyExists, CdmwPlatformMngt::ComponentServerNotFound,
              CdmwPlatformMngt::DuplicateEntity, CosPropertyService::MultipleExceptions,
              CdmwPlatformMngt::DuplicateService, CdmwPlatformMngt::DuplicateStep,
              CdmwPlatformMngt::IncompatibleStatus);

   /**
    *Purpose:
    *<p> remove component from process
    * @param component_name name of component
    * @param host_name host name of host
    * @exception IncompatibleStatus
    * @exception ComponentNotFound
    * @exception ComponentStillActive
    */
    virtual void remove_component(const char* component_name, const char* host_name)
       throw(CORBA::SystemException, CdmwPlatformMngt::ComponentNotFound, CdmwPlatformMngt::ComponentStillActive, CdmwPlatformMngt::IncompatibleStatus);



    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_init_graph:1.0
     * operation
     * @param component init graph attribute
     */
     virtual void set_init_graph(const CdmwPlatformMngt::GraphElements& component_graph) throw (CORBA::SystemException);


     /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_stop_graph:1.0
     * operation
     * @param component stop graph attribute
     */
     virtual void set_stop_graph(const CdmwPlatformMngt::GraphElements& component_graph) throw (CORBA::SystemException);

     
     virtual CdmwPlatformMngt::ProcessPID previous_pid()
        throw (CORBA::SystemException);
        
     virtual void get_all_components(CORBA::ULong how_many, 
                        CdmwPlatformMngt::CcmComponents_out ncomponents, 
                        CdmwPlatformMngt::ComponentsIterator_out rest)
        throw (CORBA::SystemException);

     virtual CdmwPlatformMngt::CcmComponent* get_component(const char* component_name)
        throw ( CdmwPlatformMngt::ComponentNotFound,
                CdmwPlatformMngt::NotComponentServer,
                CORBA::SystemException);
        
     virtual CORBA::ULong get_number_of_components()
        throw (CORBA::SystemException);
        
     virtual CdmwPlatformMngt::RequestStatus stop_process(CORBA::Boolean emergency)   
        throw (CORBA::SystemException);
        
     virtual CdmwPlatformMngt::CcmComponent* add_component(const CdmwPlatformMngt::ComponentDef& component_def)
        throw (CORBA::SystemException);
     
};

} // End namespace Proxy
} // End namespace PlatformMngt
} // End namespace Cdmw

#endif //INCL_PLATFORMMNGT_PROCESS_HPP

