/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtDaemon.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtEvent.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp>
#include <SystemMngt/idllib/CdmwFTSystMngtComponent.stub.hpp>
#include <Foundation/logging/FunctionLogger.hpp>

#include "platformftproxy/CcmComponent_impl.hpp"


const char *const LOGGER_NAME = "CDMW.PFMDMN.SupProxy.CcmComponent";

namespace Cdmw
{
  namespace PlatformMngt
  {
    namespace Proxy
    {


      /*
         CcmComponent_impl::CcmComponent_impl (CORBA::ORB_ptr orb,
         ProxyCollection * proxy_collection,
         AccessorLookupService *
         lookup_service,
         CdmwPlatformMngt::RWElementRepository *rep,
         PortableServer::POA *poa) throw (Cdmw::
         BadParameterException,
         Cdmw::OutOfMemoryException):
         ManagedElement_impl(rep, poa,
         orb),
         m_accessor (CdmwFTSystMngt::CcmComponentAccessor::_nil ()),
         m_accessor_ok (false), m_proxy_collection (proxy_collection),
         m_lookup_service (lookup_service)

       */


      CcmComponent_impl::CcmComponent_impl (CORBA::ORB_ptr orb,
                                            ProxyCollection * proxy_collection,
                                            AccessorLookupService *
                                            lookup_service) throw (Cdmw::
                                                                   BadParameterException,
                                                                   Cdmw::
                                                                   OutOfMemoryException):
                                   ManagedElement_impl(orb),
        m_accessor (CdmwFTSystMngt::CcmComponentAccessor::_nil ()),
        m_accessor_ok (false), m_proxy_collection (proxy_collection),
        m_lookup_service (lookup_service)
      {
        m_proxy_collection->add_ref ();
        set_logger (Get_Logger ());
        set_managed_element (this);
        set_proxy_collection (proxy_collection);
      }

      CcmComponent_impl::~CcmComponent_impl () throw ()
      {
      m_proxy_collection->remove_ref();
      }

      Cdmw::Logging::Logger * CcmComponent_impl::Get_Logger ()
      {
        //const char *const LOGGER_NAME = "CDMW.PFMDMN.SupProxy.CcmComponent";

        static Cdmw::Logging::Logger *  logger =
          Cdmw::Logging::Logger::Get_Logger (LOGGER_NAME);
          return logger ;
      }


      CdmwFTSystMngt::ManagedElementAccessor_ptr CcmComponent_impl::
        accessor_managed_element () throw (CORBA::TRANSIENT)
      {
      /*  if (!m_accessor_ok)
          {

            CDMW_MUTEX_GUARD (m_accessor_mutex);
            // leaving mutex other thread might have fetched the data
            // already

            try
            {

              if (CORBA::is_nil (m_accessor.in ()))
                {

                  m_accessor = m_lookup_service->getCcmComponentAccessor ();    // raises AccessorUndefined

                  // update the flag
                  m_accessor_ok = !CORBA::is_nil (m_accessor.in ());
                }

            }
            catch (...)
            {
              throw CORBA::TRANSIENT (::Cdmw::OrbSupport::TRANSIENT,
                                      CORBA::COMPLETED_NO);
            }

            // paranoid testing
            if (CORBA::is_nil (m_accessor.in ()))
              {
                throw CORBA::TRANSIENT (::Cdmw::OrbSupport::TRANSIENT,
                                        CORBA::COMPLETED_NO);
              }

            // releasing mutex here
          }

        CDMW_ASSERT (m_accessor_ok && !CORBA::is_nil (m_accessor.in ()));

        return m_accessor.ptr ();       // return const ptr
       */
      
      return this->accessor();
      }

      const CdmwFTSystMngt::CcmComponentAccessor_ptr CcmComponent_impl::
        accessor () throw (CORBA::TRANSIENT)
      {
        if (!m_accessor_ok)
          {

            CDMW_MUTEX_GUARD (m_accessor_mutex);
            // leaving mutex other thread might have fetched the data
            // already

            try
            {

              if (CORBA::is_nil (m_accessor.in ()))
                {

                  m_accessor = m_lookup_service->getCcmComponentAccessor ();    // raises AccessorUndefined

                  // update the flag
                  m_accessor_ok = !CORBA::is_nil (m_accessor.in ());
                }

            }
            catch (...)
            {
              throw CORBA::TRANSIENT (::Cdmw::OrbSupport::TRANSIENT,
                                      CORBA::COMPLETED_NO);
            }

            // paranoid testing
            if (CORBA::is_nil (m_accessor.in ()))
              {
                throw CORBA::TRANSIENT (::Cdmw::OrbSupport::TRANSIENT,
                                        CORBA::COMPLETED_NO);
              }

            // releasing mutex here
          }

        CDMW_ASSERT (m_accessor_ok && !CORBA::is_nil (m_accessor.in ()));

        return m_accessor.ptr ();       // return const ptr
      }

      CORBA::ULong CcmComponent_impl::get_number_of_entities ()throw (CORBA::
                                                                      SystemException)
      {

        try
        {
          std::string target = this->getObjectId ();

          return (accessor ()->get_number_of_entities (target.c_str ()));
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          throw CORBA::OBJECT_NOT_EXIST (OrbSupport::OBJECT_NOT_EXIST,
                                         CORBA::COMPLETED_NO);
        }
        catch (const CORBA::SystemException & ex)
        {
          throw;
        }
        catch (...)
        {
          throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_MAYBE);
        }

      }


      void CcmComponent_impl::get_all_entity_names (CORBA::ULong how_many,
                                                    CdmwPlatformMngt::
                                                    ElementNames_out
                                                    elem_names_out,
                                                    CdmwPlatformMngt::
                                                    ElementNamesIterator_out
                                                    elem_iter_out)
        throw (CORBA::SystemException)
      {
        try
        {
          std::string target = this->getObjectId ();

          accessor ()->get_all_entity_names (target.c_str (),
                                             how_many,
                                             elem_names_out, elem_iter_out);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          throw CORBA::OBJECT_NOT_EXIST (OrbSupport::OBJECT_NOT_EXIST,
                                         CORBA::COMPLETED_NO);
        }
        catch (const CORBA::SystemException & ex)
        {
          throw;
        }
        catch (...)
        {
          throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_MAYBE);
        }

      }

      CdmwPlatformMngt::Entity *
        CcmComponent_impl::get_entity (const char *name) throw (  CdmwPlatformMngt::EntityNotFound,
                                                                    CORBA::SystemException)
      {
        try
        {
          std::string target = this->getObjectId ();

          //return(accessor()->get_entity(target.c_str(), name));

          CORBA::String_var entity_target =
            accessor ()->get_entity (target.c_str (), name);

          return this->m_proxy_collection->createEntityRef (entity_target);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          throw CORBA::OBJECT_NOT_EXIST (OrbSupport::OBJECT_NOT_EXIST,
                                         CORBA::COMPLETED_NO);
        }
        catch (const CORBA::SystemException & ex)
        {
          throw;
        }
        catch (...)
        {
          throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_MAYBE);
        }

      }

      CORBA::Boolean CcmComponent_impl::get_entities (const CdmwPlatformMngt::
                                                      ElementNames & elem_names,
                                                      CdmwPlatformMngt::
                                                      Entities_out ent_out)
        throw (CORBA::SystemException)
      {
        try
        {

          std::string target = this->getObjectId ();

          ::CdmwPlatformMngt::ElementPaths_var paths =
            new::CdmwPlatformMngt::ElementPaths ();

          CdmwPlatformMngt::ElementNames_var tmp_entity_names;

          CORBA::Boolean succeed =
            accessor ()->get_entities (target.c_str (),
                                       elem_names, paths.out ());

          if (succeed)
            {
              ent_out =
                this->m_proxy_collection->createEntitiesFromSeq (paths.ptr ());
            }

          return succeed;

        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          throw CORBA::OBJECT_NOT_EXIST (OrbSupport::OBJECT_NOT_EXIST,
                                         CORBA::COMPLETED_NO);
        }
        catch (const CORBA::SystemException & ex)
        {
          throw;
        }
        catch (...)
        {
          throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_MAYBE);
        }

      }

      void CcmComponent_impl::get_all_entities (CORBA::ULong how_many,
                                                CdmwPlatformMngt::
                                                Entities_out ent_out,
                                                CdmwPlatformMngt::
                                                EntitiesIterator_out
                                                ent_iter_out) throw (CORBA::
                                                                     SystemException)
      {
        try
        {

          std::string target = this->getObjectId ();

          ::CdmwPlatformMngt::ElementPaths_var paths;
          ::CdmwPlatformMngt::ElementPathsIterator_var iter;





           accessor ()->get_all_entities (target.c_str (),
                                         how_many, paths.out (), iter.out ());


          CdmwPlatformMngt::Entities_var tmp_nentities =
            this->m_proxy_collection->createEntitiesFromSeq (paths.ptr ());

          CdmwPlatformMngt::EntitiesIterator_var tmp_rest =
            this->m_proxy_collection->createEntitiesIteratorRef (iter.in ());

          ent_out = tmp_nentities._retn ();
          ent_iter_out = tmp_rest._retn ();
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          throw CORBA::OBJECT_NOT_EXIST (OrbSupport::OBJECT_NOT_EXIST,
                                         CORBA::COMPLETED_NO);
        }
        catch (const CORBA::SystemException & ex)
        {
          throw;
        }
        catch (...)
        {
          throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_MAYBE);
        }

      }

      CdmwPlatformMngt::Entity *
        CcmComponent_impl::add_entity (const CdmwPlatformMngt::
                                       EntityDef & ent_def) throw (CORBA::
                                                                   SystemException)
      {
        try
        {

          //return (accessor()->add_entity(target.c_str(), ent_def ));

          std::string target = this->getObjectId ();

          CORBA::String_var uid =
            accessor ()->add_entity (target.c_str (), ent_def);

          return this->m_proxy_collection->createEntityRef (uid);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          throw CORBA::OBJECT_NOT_EXIST (OrbSupport::OBJECT_NOT_EXIST,
                                         CORBA::COMPLETED_NO);
        }
        catch (const CORBA::SystemException & ex)
        {
          throw;
        }
        catch (...)
        {
          throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_MAYBE);
        }

      }

      void CcmComponent_impl::remove_entity (const char *name) throw (CdmwPlatformMngt::EntityNotFound, CORBA::
                                                                      SystemException)
      {
        try
        {
          std::string target = this->getObjectId ();

          accessor ()->remove_entity (target.c_str (), name);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          throw CORBA::OBJECT_NOT_EXIST (OrbSupport::OBJECT_NOT_EXIST,
                                         CORBA::COMPLETED_NO);
        }
        catch (const CORBA::SystemException & ex)
        {
          throw;
        }
        catch (...)
        {
          throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_MAYBE);
        }

      }

      CdmwPlatformMngt::ComponentSnapshot *
        CcmComponent_impl::get_snapshot ()throw (CORBA::SystemException)
      {
        try
        {
          std::string target = this->getObjectId ();

          return (accessor ()->get_snapshot (target.c_str ()));
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          throw CORBA::OBJECT_NOT_EXIST (OrbSupport::OBJECT_NOT_EXIST,
                                         CORBA::COMPLETED_NO);
        }
        catch (const CORBA::SystemException & ex)
        {
          throw;
        }
        catch (...)
        {
          throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_MAYBE);
        }

      }

      CdmwPlatformMngt::ComponentStatus CcmComponent_impl::get_status (CORBA::
                                                                       String_out
                                                                       str_out)
        throw (CORBA::SystemException)
      {
        try
        {
          std::string target = this->getObjectId ();

          return (accessor ()->get_status (target.c_str (), str_out));
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          throw CORBA::OBJECT_NOT_EXIST (OrbSupport::OBJECT_NOT_EXIST,
                                         CORBA::COMPLETED_NO);
        }
        catch (const CORBA::SystemException & ex)
        {
          throw;
        }
        catch (...)
        {
          throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_MAYBE);
        }

      }

      char *CcmComponent_impl::instantiation_id () throw (CORBA::
                                                          SystemException)
      {
        try
        {
          std::string target = this->getObjectId ();

          return (accessor ()->get_instantiation_id (target.c_str ()));
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          throw CORBA::OBJECT_NOT_EXIST (OrbSupport::OBJECT_NOT_EXIST,
                                         CORBA::COMPLETED_NO);
        }
        catch (const CORBA::SystemException & ex)
        {
          throw;
        }
        catch (...)
        {
          throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_MAYBE);
        }

      }

      char *CcmComponent_impl::deployment_plan () throw (CORBA::SystemException)
      {
        try
        {
          std::string target = this->getObjectId ();

          return (accessor ()->get_deployment_plan (target.c_str ()));
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          throw CORBA::OBJECT_NOT_EXIST (OrbSupport::OBJECT_NOT_EXIST,
                                         CORBA::COMPLETED_NO);
        }
        catch (const CORBA::SystemException & ex)
        {
          throw;
        }
        catch (...)
        {
          throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_MAYBE);
        }

      }


      CdmwPlatformMngtBase::TimeStamp CcmComponent_impl::
        start_time ()throw (CORBA::SystemException)
      {
        try
        {
          std::string target = this->getObjectId ();

          return (accessor ()->get_start_time (target.c_str ()));
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          throw CORBA::OBJECT_NOT_EXIST (OrbSupport::OBJECT_NOT_EXIST,
                                         CORBA::COMPLETED_NO);
        }
        catch (const CORBA::SystemException & ex)
        {
          throw;
        }
        catch (...)
        {
          throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_MAYBE);
        }

      }

      CdmwPlatformMngtBase::TimeStamp CcmComponent_impl::
        stop_time ()throw (CORBA::SystemException)
      {
        try
        {
          std::string target = this->getObjectId ();

          return (accessor ()->get_stop_time (target.c_str ()));
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          throw CORBA::OBJECT_NOT_EXIST (OrbSupport::OBJECT_NOT_EXIST,
                                         CORBA::COMPLETED_NO);
        }
        catch (const CORBA::SystemException & ex)
        {
          throw;
        }
        catch (...)
        {
          throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_MAYBE);
        }
      }

      char *CcmComponent_impl::get_component_server_name () throw (CORBA::
                                                                   SystemException)
      {
        try
        {
          std::string target = this->getObjectId ();

          return (accessor ()->get_component_server_name (target.c_str ()));
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          throw CORBA::OBJECT_NOT_EXIST (OrbSupport::OBJECT_NOT_EXIST,
                                         CORBA::COMPLETED_NO);
        }
        catch (const CORBA::SystemException & ex)
        {
          throw;
        }
        catch (...)
        {
          throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_MAYBE);
        }
      }

      CORBA::Boolean CcmComponent_impl::
        get_indicator_relevant_status ()throw (CORBA::SystemException)
      {
        try
        {
          std::string target = this->getObjectId ();

          return (accessor ()->get_indicator_relevant_status (target.c_str ()));
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          throw CORBA::OBJECT_NOT_EXIST (OrbSupport::OBJECT_NOT_EXIST,
                                         CORBA::COMPLETED_NO);
        }
        catch (const CORBA::SystemException & ex)
        {
          throw;
        }
        catch (...)
        {
          throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_MAYBE);
        }
      }



      void CcmComponent_impl::set_indicator_relevant_status (CORBA::
                                                             Boolean
                                                             indicator_relevant_status)
        throw (CORBA::SystemException)
      {
        try
        {
          std::string target = this->getObjectId ();

          accessor ()->set_indicator_relevant_status (target.c_str (),
                                                      indicator_relevant_status);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          throw CORBA::OBJECT_NOT_EXIST (OrbSupport::OBJECT_NOT_EXIST,
                                         CORBA::COMPLETED_NO);
        }
        catch (const CORBA::SystemException & ex)
        {
          throw;
        }
        catch (...)
        {
          throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_MAYBE);
        }

      }


      CdmwPlatformMngt::RedundancyGroup *
        CcmComponent_impl::get_component_group ()throw (CORBA::SystemException)
      {
        try
        {
          std::string target = this->getObjectId ();

          return (accessor ()->get_component_group (target.c_str ()));
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          throw CORBA::OBJECT_NOT_EXIST (OrbSupport::OBJECT_NOT_EXIST,
                                         CORBA::COMPLETED_NO);
        }
        catch (const CORBA::SystemException & ex)
        {
          throw;
        }
        catch (...)
        {
          throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_MAYBE);
        }
      }




    }                           // End namespace Proxy
  }                             // End namespace PlatformMngt
}                               // End namespace Cdmw
