/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtDaemon.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtEvent.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp>
#include <SystemMngt/idllib/CdmwFTSystMngtComponent.stub.hpp>
#include <Foundation/logging/FunctionLogger.hpp>

#include "platformftproxy/LBGroup_impl.hpp"

const char *const LOGGER_NAME = "CDMW.PFMDMN.SupProxy.LBGroup";

namespace Cdmw
{
  namespace PlatformMngt
  {
    namespace Proxy
    {

      LBGroup_impl::LBGroup_impl (CORBA::ORB_ptr orb,
                                  ProxyCollection * proxy_collection,
                                  AccessorLookupService * lookup_service) 
                                  throw (Cdmw::BadParameterException,
                                         Cdmw::OutOfMemoryException):
        ManagedElement_impl(orb),
        RedundancyGroup_impl( orb, proxy_collection, lookup_service), 
        m_accessor (CdmwFTSystMngt::LBGroupAccessor::_nil ()),
        m_accessor_ok (false), m_proxy_collection (proxy_collection),
        m_lookup_service (lookup_service)
      {
        m_proxy_collection->add_ref ();

        set_logger (Get_Logger ());

        set_managed_element (this);

        set_proxy_collection (proxy_collection);

      }

      LBGroup_impl::~LBGroup_impl () throw ()
        {
         m_proxy_collection->remove_ref();
        }

      Cdmw::Logging::Logger * LBGroup_impl::Get_Logger ()
      {

        static Cdmw::Logging::Logger *  logger =
          Cdmw::Logging::Logger::Get_Logger (LOGGER_NAME);
          return logger ;
      }


      const CdmwFTSystMngt::LBGroupAccessor_ptr LBGroup_impl::
        accessor () throw (CORBA::TRANSIENT)
      {
        if (!m_accessor_ok)
          {

            CDMW_MUTEX_GUARD (m_accessor_mutex);
            // leaving mutex other thread might have fetched the data
            // already

            try
            {

              if (CORBA::is_nil (m_accessor.in ()))
                {

                  m_accessor = m_lookup_service->getLBGroupAccessor (); // raises AccessorUndefined

                  // update the flag
                  m_accessor_ok = !CORBA::is_nil (m_accessor.in ());
                }

            }
            catch (...)
            {
              throw CORBA::TRANSIENT (::Cdmw::OrbSupport::TRANSIENT,
                                      CORBA::COMPLETED_NO);
            }

            // paranoid testing
            if (CORBA::is_nil (m_accessor.in ()))
              {
                throw CORBA::TRANSIENT (::Cdmw::OrbSupport::TRANSIENT,
                                        CORBA::COMPLETED_NO);
              }

            // releasing mutex here
          }
        CDMW_ASSERT (m_accessor_ok && !CORBA::is_nil (m_accessor.in ()));

        return m_accessor.ptr ();       // return const ptr
      }

      CdmwFTSystMngt::ManagedElementAccessor_ptr
        LBGroup_impl::accessor_managed_group ()throw (CORBA::TRANSIENT)
      {
        if (!m_accessor_ok)
          {       

            CDMW_MUTEX_GUARD (m_accessor_mutex);
            // leaving mutex other thread might have fetched the data
            // already

            try     
            {       

              if (CORBA::is_nil (m_accessor.in ()))
                {       

                  m_accessor = m_lookup_service->getLBGroupAccessor ();    // raises AccessorUndefined

                  // update the flag
                  m_accessor_ok = !CORBA::is_nil (m_accessor.in ());
                }         

            }
            catch (...)
            {
              throw CORBA::TRANSIENT (::Cdmw::OrbSupport::TRANSIENT,
                                      CORBA::COMPLETED_NO);
            }

            // paranoid testing
            if (CORBA::is_nil (m_accessor.in ()))
              {
                throw CORBA::TRANSIENT (::Cdmw::OrbSupport::TRANSIENT,
                                        CORBA::COMPLETED_NO);
              }

            // releasing mutex here
          }
        CDMW_ASSERT (m_accessor_ok && !CORBA::is_nil (m_accessor.in ()));

        return m_accessor.ptr ();       // return const ptr
      }


       CdmwFTSystMngt::RedundancyGroupAccessor_ptr
               LBGroup_impl::accessor_redu_group ()
                                            throw (CORBA::SystemException)

      {
        if (!m_accessor_ok)
          {

            CDMW_MUTEX_GUARD (m_accessor_mutex);
            // leaving mutex other thread might have fetched the data
            // already

            try
            {

              if (CORBA::is_nil (m_accessor.in ()))
                {

                  m_accessor = m_lookup_service->getLBGroupAccessor ();    // raises AccessorUndefined

                  // update the flag
                  m_accessor_ok = !CORBA::is_nil (m_accessor.in ());
                }

            }
            catch (...)
            {
              throw CORBA::TRANSIENT (::Cdmw::OrbSupport::TRANSIENT,
                                      CORBA::COMPLETED_NO);
            }

            // paranoid testing
            if (CORBA::is_nil (m_accessor.in ()))
              {
                throw CORBA::TRANSIENT (::Cdmw::OrbSupport::TRANSIENT,
                                        CORBA::COMPLETED_NO);
              }

            // releasing mutex here
         }

        CDMW_ASSERT (m_accessor_ok && !CORBA::is_nil (m_accessor.in ()));

        return m_accessor.ptr ();       // return const ptr
      }



      CdmwPlatformMngt::LBGroupSnapshot *
       LBGroup_impl::get_snapshot () throw (CORBA::SystemException)
       {

       try
        {
          std::string target = this->getObjectId ();

          return (accessor ()->get_snapshot(target.c_str ()));
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          throw CORBA::OBJECT_NOT_EXIST (OrbSupport::OBJECT_NOT_EXIST,
                                         CORBA::COMPLETED_NO);
        }
        catch (const CORBA::SystemException & ex)
        {
          throw;
        }
        catch (...)
        {
          throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_MAYBE);
        }
       }

      CdmwPlatformMngt::BalancingStyle LBGroup_impl::get_style () throw (CORBA::
                                                               SystemException)
       {
       //std::cout<<"********* Inside LBGroup_impl::get_style of platformftproxy"<<std::endl;

       try
        {
          std::string target = this->getObjectId ();

          return (accessor ()->get_style (target.c_str ()));
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          throw CORBA::OBJECT_NOT_EXIST (OrbSupport::OBJECT_NOT_EXIST,
                                         CORBA::COMPLETED_NO);
        }
        catch (const CORBA::SystemException & ex)
        {
          throw;
        }
        catch (...)
        {
          throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_MAYBE);
        }


       }

      void LBGroup_impl::set_style (CdmwPlatformMngt::BalancingStyle) throw (CORBA::
                                                                 SystemException)
       {
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
                                   CORBA::COMPLETED_NO);
       }


    }                           // End namespace Proxy
  }                             // End namespace PlatformMngt
}                               // End namespace Cdmw
