/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtDaemon.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtEvent.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp>
#include <SystemMngt/idllib/CdmwFTSystMngtComponent.stub.hpp>
#include <Foundation/logging/FunctionLogger.hpp>

#include "platformftproxy/ManagedElement_impl.hpp"

namespace Cdmw
{
  namespace PlatformMngt
  {
    namespace Proxy
    {





      // ----------------------------------------------------------------------
      // ManagedElement_impl class.
      // ----------------------------------------------------------------------

      /*
      ManagedElement_impl::ManagedElement_impl (CdmwPlatformMngt::
                                                RWElementRepository_ptr
                                                element_repository,
                                                PortableServer::
                                                POA_ptr poa,
                                                CORBA::
                                                ORB_ptr orb)
        throw ():ProxyUtil (orb)
      {
      }
      */

      ManagedElement_impl::ManagedElement_impl (CORBA::ORB_ptr orb) throw ():ProxyUtil(orb)
      {

         CORBA::Object_var obj = m_orb->resolve_initial_references(CdmwPlatformMngt::Current::CURRENT_NAME);

          m_current = CdmwPlatformMngt::Current::_narrow(obj.in());

      }

      ManagedElement_impl::~ManagedElement_impl () throw ()
      {
      }


      CdmwPlatformMngt::SupervisionObserver *
        ManagedElement_impl::register_observer (const char *observer_name,
                                                CdmwPlatformMngt::
                                                SupervisionObserver *
                                                observer) throw (CORBA::
                                                                 SystemException)
      {
        CDMW_LOG_FUNCTION (m_logger, "observer_name=" << observer_name);
        try
        { 
          std::string target = m_managed_element->getObjectId ();
          return accessor_managed_element()->register_observer (target.c_str (), observer_name,
                                                observer);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }


      }


      void ManagedElement_impl::set_logger (Cdmw::Logging::Logger * logger)
      {
        ManagedElement_impl::m_logger = logger;
      }

      /***
      void ManagedElement_impl::set_accessor (CdmwFTSystMngt::
                                              ManagedElementAccessor_ptr
                                              accessor)
      {
        accessor_managed_element() = accessor;
      } ***/

      void ManagedElement_impl::set_managed_element (ManagedElement_impl *
                                                     manag)
      {
        m_managed_element = manag;
      }

     void ManagedElement_impl::set_proxy_collection( ProxyCollection * proxy_collection)
      {
        m_proxy_collection = proxy_collection;
      }

      CdmwPlatformMngt::SupervisionObserver *
        ManagedElement_impl::register_proxy_observer (const char *,
                                                      const char *,
                                                      CdmwPlatformMngt::
                                                      SupervisionObserver *)
        throw (CORBA::SystemException)
      {
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
                                   CORBA::COMPLETED_NO);
      }

      CdmwPlatformMngt::SupervisionObserver *
        ManagedElement_impl::
        unregister_observer (const char *observer_name) throw (CdmwPlatformMngt::ObserverNotFound, 
                                                               CORBA::SystemException)
      {
        CDMW_LOG_FUNCTION (m_logger, "observer_name=" << observer_name);
        try
        {
          std::string target = m_managed_element->getObjectId ();

          return accessor_managed_element()->unregister_observer (target.c_str (),
                                                  observer_name);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }

      }

      void ManagedElement_impl::
        register_observer_ext (const char *observer_name,
                               CdmwPlatformMngt::RegistrationDepth depth,
                               CdmwPlatformMngt::SupervisionObserver *
                               observer) throw (CORBA::SystemException,  CdmwPlatformMngt::ElementAlreadyExists)
      {
        CDMW_LOG_FUNCTION (m_logger, "observer_name=" << observer_name);
        try
        {
            std::string target = m_managed_element->getObjectId ();

            return accessor_managed_element()->register_observer_ext (target.c_str (),
                                                    observer_name, depth,
                                                    observer);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
            CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                        CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION (e);
            throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          throw;
        }
        catch (...)
        {
          throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_MAYBE);
        }
      }

      void ManagedElement_impl::
        unregister_observer_ext (const char *observer_name) throw (CORBA::SystemException, CdmwPlatformMngt::ObserverNotFound)
      {
        CDMW_LOG_FUNCTION (m_logger, "observer_name=" << observer_name);
        try
        {
            std::string target = m_managed_element->getObjectId ();
            return accessor_managed_element()->unregister_observer_ext (target.c_str (),
                                                      observer_name);
        
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          throw;
        }
        catch (...)
        {
          throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_MAYBE);
        }
      }

      void ManagedElement_impl::define_property (const char *property_name,
                                                 const CORBA::
                                                 Any & property_value)
        throw (CosPropertyService::InvalidPropertyName,
               CosPropertyService::ConflictingProperty,
               CosPropertyService::UnsupportedTypeCode,
               CosPropertyService::UnsupportedProperty,
               CosPropertyService::ReadOnlyProperty, CORBA::SystemException)
      {
        CDMW_LOG_FUNCTION (m_logger, "property_name=" << property_name);
        try
        {
          std::string target = m_managed_element->getObjectId ();
          accessor_managed_element()->define_property (target.c_str (),
                                       property_name, property_value);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }

      }

      void ManagedElement_impl::define_properties (const CosPropertyService::
                                                   Properties & nproperties)
        throw (CosPropertyService::MultipleExceptions, CORBA::SystemException)
      {
        CDMW_LOG_FUNCTION (m_logger, "");
        try
        {
          std::string target = m_managed_element->getObjectId ();

          accessor_managed_element()->define_properties (target.c_str (), nproperties);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }

      }

      CORBA::ULong ManagedElement_impl::
        get_number_of_properties ()throw (CORBA::SystemException)
      {
        CDMW_LOG_FUNCTION (m_logger, "");
        try
        {
          std::string target = m_managed_element->getObjectId ();

          return accessor_managed_element()->get_number_of_properties (target.c_str ());
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }

      }

      void ManagedElement_impl::get_all_property_names (CORBA::ULong how_many,
                                                        CosPropertyService::
                                                        PropertyNames_out
                                                        property_names,
                                                        CosPropertyService::
                                                        PropertyNamesIterator_out
                                                        rest) throw (CORBA::
                                                                     SystemException)
      {
        CDMW_LOG_FUNCTION (m_logger, "");
        try
        {
          std::string target = m_managed_element->getObjectId ();

          accessor_managed_element()->get_all_property_names (target.c_str (),
                                              how_many, property_names, rest);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }

      }

      CORBA::Any *
        ManagedElement_impl::get_property_value (const char *property_name) throw (CosPropertyService::PropertyNotFound,
                                                                    CosPropertyService::InvalidPropertyName,
                                                                    CORBA::SystemException)
      {
        CDMW_LOG_FUNCTION (m_logger, "property_name=" << property_name);
        try
        {
          std::string target = m_managed_element->getObjectId ();

          return accessor_managed_element()->get_property_value (target.c_str (),
                                                 property_name);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }

      }

      CORBA::Boolean ManagedElement_impl::
        get_properties (const CosPropertyService::
                        PropertyNames & property_names,
                        CosPropertyService::
                        Properties_out nproperties) throw (CORBA::
                                                           SystemException)
      {
        CDMW_LOG_FUNCTION (m_logger, "");
        try
        {
          std::string target = m_managed_element->getObjectId ();

          return accessor_managed_element()->get_properties (target.c_str (),
                                             property_names, nproperties);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }

      }

      void ManagedElement_impl::get_all_properties (CORBA::ULong how_many,
                                                    CosPropertyService::
                                                    Properties_out nproperties,
                                                    CosPropertyService::
                                                    PropertiesIterator_out rest)
        throw (CORBA::SystemException)
      {
        CDMW_LOG_FUNCTION (m_logger, "");
        try
        {
          std::string target = m_managed_element->getObjectId ();

          accessor_managed_element()->get_all_properties (target.c_str (),
                                          how_many, nproperties, rest);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }

      }

      void ManagedElement_impl::delete_property (const char *property_name)
        throw (CosPropertyService::PropertyNotFound,
               CosPropertyService::InvalidPropertyName,
               CosPropertyService::FixedProperty, CORBA::SystemException)
      {
        CDMW_LOG_FUNCTION (m_logger, "property_name=" << property_name);
        try
        {
          std::string target = m_managed_element->getObjectId ();

          accessor_managed_element()->delete_property (target.c_str (), property_name);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }


      }

      void ManagedElement_impl::delete_properties (const CosPropertyService::
                                                   PropertyNames &
                                                   property_names)
          throw (CosPropertyService::MultipleExceptions,
           CORBA::SystemException) 
      {
        CDMW_LOG_FUNCTION (m_logger, "");
        try
        {
          std::string target = m_managed_element->getObjectId ();

          accessor_managed_element()->delete_properties (target.c_str (), property_names);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }

      }

      CORBA::Boolean ManagedElement_impl::delete_all_properties ()throw (CORBA::
                                                                         SystemException)
      {
        CDMW_LOG_FUNCTION (m_logger, "");
        try
        {
          std::string target = m_managed_element->getObjectId ();

          return accessor_managed_element()->delete_all_properties (target.c_str ());
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }

      }

      CORBA::Boolean ManagedElement_impl::
        is_property_defined (const char *property_name)
        throw (CosPropertyService::InvalidPropertyName, CORBA::SystemException)
      {
        CDMW_LOG_FUNCTION (m_logger, "property_name=" << property_name);
        try
        {
          std::string target = m_managed_element->getObjectId ();

          return accessor_managed_element()->is_property_defined (target.c_str (),
                                                  property_name);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }

      }

      void ManagedElement_impl::get_allowed_property_types (CosPropertyService::
                                                            PropertyTypes_out
                                                            property_types)
        throw (CORBA::SystemException)
      {
        CDMW_LOG_FUNCTION (m_logger, "");
        try
        {
          std::string target = m_managed_element->getObjectId ();

          accessor_managed_element()->get_allowed_property_types (target.c_str (),
                                                  property_types);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }

      }


      void ManagedElement_impl::get_allowed_properties (CosPropertyService::
                                                        PropertyDefs_out
                                                        property_defs)
        throw (CORBA::SystemException)
      {
        CDMW_LOG_FUNCTION (m_logger, "");
        try
        {
          std::string target = m_managed_element->getObjectId ();

          accessor_managed_element()->get_allowed_properties (target.c_str (), property_defs);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }

      }

      void ManagedElement_impl::
        define_property_with_mode (const char *property_name,
                                   const CORBA::Any & property_value,
                                   CosPropertyService::
                                   PropertyModeType property_mode)
        throw (CosPropertyService::InvalidPropertyName,
               CosPropertyService::ConflictingProperty,
               CosPropertyService::UnsupportedTypeCode,
               CosPropertyService::UnsupportedProperty,
               CosPropertyService::UnsupportedMode,
               CosPropertyService::ReadOnlyProperty, CORBA::SystemException)
      {
        CDMW_LOG_FUNCTION (m_logger, "property_name=" << property_name);
        try
        {
          std::string target = m_managed_element->getObjectId ();

          accessor_managed_element()->define_property_with_mode (target.c_str (),
                                                 property_name,
                                                 property_value, property_mode);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }

      }


      void ManagedElement_impl::
        define_properties_with_modes (const CosPropertyService::
                                      PropertyDefs & property_defs)
        throw (CosPropertyService::MultipleExceptions, CORBA::SystemException)
      {
        CDMW_LOG_FUNCTION (m_logger, "");
        try
        {
          std::string target = m_managed_element->getObjectId ();

          accessor_managed_element()->define_properties_with_modes (target.c_str (),
                                                    property_defs);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }

      }

      CosPropertyService::PropertyModeType
        ManagedElement_impl::get_property_mode (const char *property_name)
        throw (CosPropertyService::PropertyNotFound,
               CosPropertyService::InvalidPropertyName, CORBA::SystemException)
      {
        CDMW_LOG_FUNCTION (m_logger, "property_name=" << property_name);
        try
        {
          std::string target = m_managed_element->getObjectId ();

          return accessor_managed_element()->get_property_mode (target.c_str (), property_name);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }

      }


      CORBA::Boolean
        ManagedElement_impl::get_property_modes (const CosPropertyService::
                                                 PropertyNames & property_names,
                                                 CosPropertyService::
                                                 PropertyModes_out
                                                 property_modes) throw (CORBA::
                                                                        SystemException)
      {
        CDMW_LOG_FUNCTION (m_logger, "");
        try
        {
          std::string target = m_managed_element->getObjectId ();

          return accessor_managed_element()->get_property_modes (target.c_str (),
                                                 property_names,
                                                 property_modes);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }

      }


      void
        ManagedElement_impl::set_property_mode (const char *property_name,
                                                CosPropertyService::
                                                PropertyModeType property_mode)
        throw (CosPropertyService::InvalidPropertyName,
               CosPropertyService::PropertyNotFound,
               CosPropertyService::UnsupportedMode, CORBA::SystemException)
      {
        CDMW_LOG_FUNCTION (m_logger, "property_name=" << property_name);
        try
        {
          std::string target = m_managed_element->getObjectId ();

          accessor_managed_element()->set_property_mode (target.c_str (),
                                         property_name, property_mode);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }

      }


      void
        ManagedElement_impl::set_property_modes (const CosPropertyService::
                                                 PropertyModes & property_modes)
        throw (CosPropertyService::MultipleExceptions, CORBA::SystemException)
      {
        CDMW_LOG_FUNCTION (m_logger, "");
        try
        {
          std::string target = m_managed_element->getObjectId ();

          accessor_managed_element()->set_property_modes (target.c_str (), property_modes);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }

      }



      void
        ManagedElement_impl::get_attribute (const char *attr_name,
                                            CORBA::Any_out ret_attr,
                                            CdmwPlatformMngt::
                                            Owner flag) throw (CORBA::
                                                               SystemException,
                                                               CdmwPlatformMngt::
                                                               AttributeNotFound)
      {
        CDMW_LOG_FUNCTION (m_logger, "attr_name=" << attr_name);
        try
        {
          std::string target = m_managed_element->getObjectId ();

          accessor_managed_element()->get_attribute (target.c_str (),
                                     attr_name, ret_attr, flag);
        }
        catch (const CdmwPlatformMngt::AttributeNotFound& e)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (e); 
          throw e;
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }

      }



      void
        ManagedElement_impl::set_attribute (const char *attr_name,
                                            const CORBA::Any & attr,
                                            CdmwPlatformMngt::
                                            Owner flag) throw (CORBA::SystemException,
                                                               CdmwPlatformMngt::AttributeNotFound,
                                                               CdmwPlatformMngt::ReadOnlyAttribute)
      {
        CDMW_LOG_FUNCTION (m_logger, "attr_name=" << attr_name);
        try
        {
          std::string target = m_managed_element->getObjectId ();

          accessor_managed_element()->set_attribute (target.c_str (), attr_name, attr, flag);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }

      }


      void
        ManagedElement_impl::
        register_all_properties_observer (const char *observer_name,
                                          CdmwPlatformMngt::
                                          SupervisionObserver * observer,
                                          CORBA::Boolean deeply) throw (CORBA::
                                                                        SystemException,
                                                                        CdmwPlatformMngt::
                                                                        ElementAlreadyExists)
      {
        CDMW_LOG_FUNCTION (m_logger, "observer_name=" << observer_name);
        try
        {
          std::string target = m_managed_element->getObjectId ();

          accessor_managed_element()->register_all_properties_observer (target.c_str (),
                                                        observer_name,
                                                        observer, deeply);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }


      }


      void
        ManagedElement_impl::
        unregister_all_properties_observer (const char *observer_name)
        throw (CdmwPlatformMngt::ObserverNotFound, CORBA::SystemException)
      {
        CDMW_LOG_FUNCTION (m_logger, "observer_name=" << observer_name);
        try
        {
          std::string target = m_managed_element->getObjectId ();

          accessor_managed_element()->unregister_all_properties_observer (target.c_str (),
                                                          observer_name);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }

      }


      void
        ManagedElement_impl::
        register_property_observer (const char *observer_name,
                                    CdmwPlatformMngt::SupervisionObserver *
                                    observer,
                                    const CosPropertyService::
                                    PropertyNames & property_names)
        throw (CdmwPlatformMngt::PropertyInListNotFound,
               CdmwPlatformMngt::ElementAlreadyExists, CORBA::SystemException)
      {
        CDMW_LOG_FUNCTION (m_logger, "observer_name=" << observer_name);
        try
        {
          std::string target = m_managed_element->getObjectId ();

          accessor_managed_element()->register_property_observer (target.c_str (),
                                                  observer_name,
                                                  observer, property_names);

        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }

      }


      void
        ManagedElement_impl::
        unregister_property_observer (const char *observer_name,
                                      const CosPropertyService::
                                      PropertyNames & property_names)
        throw (CdmwPlatformMngt::PropertyInListNotFound,
               CdmwPlatformMngt::ObserverNotFound, CORBA::SystemException)
      {
        CDMW_LOG_FUNCTION (m_logger, "observer_name=" << observer_name);
        try
        {
          std::string target = m_managed_element->getObjectId ();

          accessor_managed_element()->unregister_property_observer (target.c_str (),
                                                    observer_name,
                                                    property_names);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }

      }


      char *ManagedElement_impl::name () throw (CORBA::SystemException)
      {


        CDMW_LOG_FUNCTION (m_logger, "");
        try
        {


          std::string target = m_managed_element->getObjectId ();

          return accessor_managed_element()->get_name (target.c_str ());
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }

      }


      char *ManagedElement_impl::element_path () throw (CORBA::SystemException)
      {
        CDMW_LOG_FUNCTION (m_logger, "");
        try
        {
          std::string target = m_managed_element->getObjectId ();

          // verify the target-object is accessible
          CORBA::String_var name = accessor_managed_element()->get_name (target.c_str ());

          return CORBA::string_dup (target.c_str ());
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }

      }



      CdmwPlatformMngt::ManagedElement *
        ManagedElement_impl::get_parent ()throw (CORBA::SystemException)
      {
        CDMW_LOG_FUNCTION (m_logger, "");
        try
        {
          std::string target = m_managed_element->getObjectId ();

          CORBA::String_var uid;

          CdmwPlatformMngt::ElementType type =
            accessor_managed_element()->get_parent (target.c_str (), uid.out ());

          return this->m_proxy_collection->createManagedElement (uid.in (),
                                                                 type);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          CORBA::OBJECT_NOT_EXIST e (OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION (e);
          throw e;
        }
        catch (const CORBA::Exception & ex)
        {
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw;
        }
        catch (...)
        {
          CORBA::INTERNAL ex (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
          CDMW_LOG_FUNCTION_EXCEPTION (ex);
          throw ex;
        }

      }


      CdmwPlatformMngt::ElementType
        ManagedElement_impl::get_element_type ()throw (CORBA::SystemException)
      {
        std::string target = m_managed_element->getObjectId ();
        try
        {
          return accessor_managed_element()->get_element_type (target.c_str ());
        }
        catch (const CORBA::SystemException & ex)
        {
          throw;
        }
        catch (...)
        {
          throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_MAYBE);
        }

      }


    CdmwPlatformMngt::Children* ManagedElement_impl::get_children ()
                                             throw (CORBA::SystemException)
    {
        CDMW_LOG_FUNCTION (m_logger, "");
        try
        { 
            std::string target = m_managed_element->getObjectId();
            CdmwPlatformMngt::ElementPaths_var paths = 
                    accessor_managed_element()->get_children(target.c_str());
                    
            CdmwPlatformMngt::Children_var tmp_managed_elements =
                this->m_proxy_collection->createManagedElementsFromSeq(paths.ptr());
            return tmp_managed_elements._retn();
       } 
       catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) 
       {
             CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                    CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(e);
            throw e;
       }
       catch (const CORBA::Exception & ex)
       {
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw;
       }
       catch (...)
       {
            CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                            CORBA::COMPLETED_MAYBE);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
       }
    }


    }
  }
}
