/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtDaemon.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtEvent.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp>
#include <SystemMngt/idllib/CdmwFTSystMngtComponent.stub.hpp>
#include <Foundation/logging/FunctionLogger.hpp>

#include "platformftproxy/RedundancyGroup_impl.hpp"

namespace {
    
    const char* const LOGGER_NAME = "CDMW.PFMDMN.SupProxy.RedundancyGroup";
    
    Cdmw::Logging::Logger* Get_Logger () {
        static Cdmw::Logging::Logger* logger =
            Cdmw::Logging::Logger::Get_Logger(LOGGER_NAME);
        return logger;
    }
}

namespace Cdmw
{
  namespace PlatformMngt
  {
    namespace Proxy
    {


      /*
      RedundancyGroup_impl::RedundancyGroup_impl () 
       {
       } */

      RedundancyGroup_impl::RedundancyGroup_impl (CORBA::ORB_ptr orb,
           ProxyCollection * proxy_collection,
           AccessorLookupService * lookup_service)
           throw (Cdmw::BadParameterException, Cdmw::OutOfMemoryException)
             : ManagedElement_impl(orb) 
         {
              m_proxy_collection = proxy_collection;

         }

      RedundancyGroup_impl::~RedundancyGroup_impl ()  throw ()
       {
       }

     CdmwPlatformMngt::Entity *
        RedundancyGroup_impl::add_entity (const CdmwPlatformMngt::
                                       EntityDef & ent_def) throw (CORBA::
                                                                   SystemException)
      {
        try
        {


          std::string target = this->getObjectId ();

      /*
          CORBA::String_var uid =
            accessor_managed_group()->add_entity (target.c_str (), ent_def);*/
      
          CORBA::String_var uid =
            accessor_redu_group()->add_entity (target.c_str (), ent_def);

          return this->m_proxy_collection->createEntityRef (uid);
        }       
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {       
          throw CORBA::OBJECT_NOT_EXIST (OrbSupport::OBJECT_NOT_EXIST,
                                         CORBA::COMPLETED_NO);
        }
        catch (const CORBA::SystemException & ex)
        {
          throw;
        }
        catch (...)
        {
          throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_MAYBE);
        }

      }

     void RedundancyGroup_impl::remove_entity (const char *name) throw ( CdmwPlatformMngt::EntityNotFound, CORBA::
                                                                      SystemException)
      {
        try
        {
          std::string target = this->getObjectId ();

          accessor_redu_group ()->remove_entity (target.c_str (), name);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          throw CORBA::OBJECT_NOT_EXIST (OrbSupport::OBJECT_NOT_EXIST,
                                         CORBA::COMPLETED_NO);
        }       
        catch (const CORBA::SystemException & ex)
        {       
          throw;  
        }
        catch (...)
        {
          throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_MAYBE);
        }

      }

      char *RedundancyGroup_impl::group_domain_id () throw (CORBA::SystemException)
       {       
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
                                   CORBA::COMPLETED_NO);
       } 

      CdmwPlatformMngt::TechnicalStatus RedundancyGroup_impl::get_status (CORBA::String_out status_info)
      throw (CORBA::SystemException)
      { 
          try
      {
              std::string target = this->getObjectId ();

              return (accessor_redu_group ()->get_status (target.c_str (), status_info));
          }
          catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
          {
              throw CORBA::OBJECT_NOT_EXIST (OrbSupport::OBJECT_NOT_EXIST,
                      CORBA::COMPLETED_NO);
          }
          catch (const CORBA::SystemException & ex)
          {
              throw;
          }
          catch (...)
          {
              throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                      CORBA::COMPLETED_MAYBE);
          }
}
      
    void RedundancyGroup_impl::get_all_entities (CORBA::ULong how_many,
                                                CdmwPlatformMngt::
                                                Entities_out ent_out,
                                                CdmwPlatformMngt::
                                                EntitiesIterator_out
                                                ent_iter_out) throw (CORBA::
                                                                     SystemException)
      {
        try
        {

          std::string target = this->getObjectId ();

          ::CdmwPlatformMngt::ElementPaths_var paths;
          ::CdmwPlatformMngt::ElementPathsIterator_var iter;

          accessor_redu_group()->get_all_entities (target.c_str (),
                                         how_many, paths.out (), iter.out ());

          CdmwPlatformMngt::Entities_var tmp_nentities =
            this->m_proxy_collection->createEntitiesFromSeq (paths.ptr ());

          CdmwPlatformMngt::EntitiesIterator_var tmp_rest =
            this->m_proxy_collection->createEntitiesIteratorRef (iter.in ());

          ent_out = tmp_nentities._retn ();
          ent_iter_out = tmp_rest._retn ();
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          throw CORBA::OBJECT_NOT_EXIST (OrbSupport::OBJECT_NOT_EXIST,
                                         CORBA::COMPLETED_NO);
        }
        catch (const CORBA::SystemException & ex)
        {
          throw;
        }
        catch (...)
       {
          throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_MAYBE);
        }

      }


       void RedundancyGroup_impl::start_up () throw (CORBA::SystemException)
       {
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
                                   CORBA::COMPLETED_NO);
       }

       void RedundancyGroup_impl::shut_down () throw (CORBA::SystemException)
       {
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
                                   CORBA::COMPLETED_NO);
       }

       void RedundancyGroup_impl::insert (CdmwPlatformMngt::ManagedElement *) throw (CORBA::
                                                                SystemException)
       {
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
                                   CORBA::COMPLETED_NO);
       }

       void RedundancyGroup_impl::_cxx_delete (CdmwPlatformMngt::ManagedElement *) throw (CORBA::
                                                                     SystemException)
       {
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
                                   CORBA::COMPLETED_NO);
       }
       void RedundancyGroup_impl::get_all_components (
               CORBA::ULong how_many, 
               CdmwPlatformMngt::CcmComponents_out ncomponents, 
               CdmwPlatformMngt::ComponentsIterator_out rest)
       throw (CORBA::SystemException)
       {
          
           CDMW_LOG_FUNCTION(::Get_Logger(), "");
           try {   
               std::string target = this->getObjectId();

               CdmwPlatformMngt::ElementPaths_var paths ; 
               CdmwPlatformMngt::ComponentsIterator_var myrest; 

               accessor_redu_group()->get_all_components(target.c_str(), 
                       how_many,
                       paths.out(),
                       myrest.out());

               rest = myrest._retn() ;

               ncomponents =
                   this->m_proxy_collection->createCcmComponentsFromSeq(paths.ptr());

           }
           catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) { 
               CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                       CORBA::COMPLETED_NO);
               CDMW_LOG_FUNCTION_EXCEPTION(e);
               throw e;
           }
           catch (const CORBA::Exception & ex) { 
               CDMW_LOG_FUNCTION_EXCEPTION(ex);
               throw;
           }
           catch (...) { 
               CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                       CORBA::COMPLETED_MAYBE);
               CDMW_LOG_FUNCTION_EXCEPTION(ex);
               throw ex;
           }

       }

       CdmwPlatformMngt::CcmComponent* RedundancyGroup_impl::get_component (const char * comp_name,
                                                          const char * host_name)
          throw (CORBA::SystemException,CdmwPlatformMngt::ComponentNotFound)
       {
           std::string target = this->getObjectId ();
           CORBA::String_var comp_target;
           try
           {
               comp_target = accessor_redu_group()->get_component(target.c_str (),comp_name,host_name);
           }
           catch (const CdmwPlatformMngt::ComponentNotFound&)
           {
               throw;
           }
       
       return this->m_proxy_collection->createCcmComponentRef(comp_target);
       }

       CORBA::ULong RedundancyGroup_impl::get_number_of_components () throw (CORBA::
                                                          SystemException)
       {
           std::string target = this->getObjectId ();
           return accessor_redu_group()->get_number_of_components(target.c_str ());
       }


        CORBA::ULong RedundancyGroup_impl::get_number_of_entities () throw
           (CORBA::SystemException)
        {

        try
        {

          std::string target = this->getObjectId ();

          CORBA::ULong num_ents ;

          num_ents = accessor_redu_group()->get_number_of_entities(target.c_str () ) ;

      return num_ents ;
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
        {
          throw CORBA::OBJECT_NOT_EXIST (OrbSupport::OBJECT_NOT_EXIST,
                                         CORBA::COMPLETED_NO);
        }
        catch (const CORBA::SystemException & ex)
         {
          throw;
         }
        catch (...)
         {
          throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_MAYBE);
         }


        }

        void RedundancyGroup_impl::get_all_entity_names (CORBA::ULong,
                                   CdmwPlatformMngt::ElementNames_out,
                                   CdmwPlatformMngt::
                                   ElementNamesIterator_out) throw (CORBA::
                                                                    SystemException)
        {                                                          
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
                                   CORBA::COMPLETED_NO);
        }                                  

        CdmwPlatformMngt::Entity *
          RedundancyGroup_impl::get_entity (const char * name) throw ( CdmwPlatformMngt::EntityNotFound, CORBA::SystemException)
        {         
            
            //std::cout << "proxy : get_entity " << std::endl;
            try
            {
            std::string target = this->getObjectId ();
    
            CORBA::String_var entity_target =
                accessor_redu_group ()->get_entity (target.c_str (), name);
            
            return this->m_proxy_collection->createEntityRef (entity_target);

            }
            catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
            {
            throw CORBA::OBJECT_NOT_EXIST (OrbSupport::OBJECT_NOT_EXIST,
                                            CORBA::COMPLETED_NO);
            }       
            catch (const CORBA::SystemException & ex)
            {       
            throw;  
            }
            catch (...)
            {
            throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                                    CORBA::COMPLETED_MAYBE);
            }
        
        }

        CORBA::Boolean RedundancyGroup_impl::get_entities (const CdmwPlatformMngt::ElementNames &,
                                       CdmwPlatformMngt::
                                       Entities_out) throw (CORBA::
                                                            SystemException)
        {
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
                                   CORBA::COMPLETED_NO);
        }


       CORBA::Long RedundancyGroup_impl::get_minimum_size () throw (CORBA::
                                                                  SystemException)
       {  
           try
           {
               std::string target = this->getObjectId ();

               return (accessor_redu_group ()-> get_minimum_size (target.c_str ()));
           }
           catch (const CdmwPlatformMngt::ManagedElementNotFound & ex)
           {
               throw CORBA::OBJECT_NOT_EXIST (OrbSupport::OBJECT_NOT_EXIST,
                       CORBA::COMPLETED_NO);
           }
           catch (const CORBA::SystemException & ex)
           {
               throw;
           }
           catch (...)
           {
               throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                       CORBA::COMPLETED_MAYBE);
           }       
 }   

      void RedundancyGroup_impl::set_minimum_size (CORBA::Long) throw
         (CORBA::SystemException)
       {
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
                                   CORBA::COMPLETED_NO);
       }

      CdmwFTSystMngt::ManagedElementAccessor_ptr 
        RedundancyGroup_impl::accessor_managed_element () throw (CORBA::TRANSIENT)
       {
       return ( accessor_managed_group() ) ;

       }

      Cdmw::Logging::Logger * RedundancyGroup_impl::Get_Logger ()
      {
           /*
        static Cdmw::Logging::Logger *  logger =
          Cdmw::Logging::Logger::Get_Logger (LOGGER_NAME);
          return logger ;
          */

        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
                                   CORBA::COMPLETED_NO);

      }


    }                           // End namespace Proxy
  }                             // End namespace PlatformMngt
}                               // End namespace Cdmw
