/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <SystemMngt/platformftproxy/ServerInterceptor.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp>
#include <Foundation/orbsupport/TypeTraits.hpp>
#include <Foundation/orbsupport/Codec.hpp>

#if CDMW_ORB_VDR == tao
#include "tao/PI_Server/ServerRequestInfo.h"
#endif

#include <iostream>
#include <string>

#include "SystemMngt/platformlibrary/LogMngr.hpp"

namespace Cdmw
{
namespace PlatformMngt
{

    ServerInterceptor::ServerInterceptor ()
    {
        m_orb_server = CORBA::ORB::_nil();
        m_service_id = CdmwPlatformMngt::SMG_INTERCEPTOR;
        m_current = CdmwPlatformMngt::Current::_nil();

    }

    ServerInterceptor::~ServerInterceptor (){}
    
    //
    // IDL to C++ Mapping
    //
    char*
    ServerInterceptor::name() throw (CORBA::SystemException)
    {
      return CORBA::string_dup("ServerInterceptor");
    }
    
    
    void
    ServerInterceptor::set_orb(CORBA::ORB_ptr orb)
    {
        if ( !CORBA::is_nil (orb))
        {    
            m_orb_server =  CORBA::ORB::_duplicate (orb);   
            // create a codec 
            m_codec= new IssuerCodec(m_orb_server);

            CORBA::Object_var obj = m_orb_server->resolve_initial_references(CdmwPlatformMngt::Current::CURRENT_NAME);
            m_current = CdmwPlatformMngt::Current::_narrow(obj.in());

        }
    }
    
    void
    ServerInterceptor::destroy() throw (CORBA::SystemException)
    {
  
    }

    void
    ServerInterceptor::receive_request_service_contexts(
        PortableInterceptor::ServerRequestInfo_ptr ri)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
    {

    }
    
    bool ServerInterceptor::check_command(const char* command)
    {
        bool result=false;
        
        if (strcmp(command, "acknowledge") == 0 
                || strcmp(command, "cold_restart") == 0 
                || strcmp(command, "hot_restart") == 0 
                || strcmp(command, "stop_process") == 0 
                || strcmp(command, "stop_application") == 0 
                || strcmp(command, "stop_system") == 0 
                || strcmp(command, "shutdown") == 0 
                || strcmp(command, "hot_reboot") == 0 
                || strcmp(command, "cold_reboot") == 0 
                || strcmp(command, "stop_host") == 0 ) 
        {
            result = true;
        }
        return result;
    }
    
    void
    ServerInterceptor::receive_request(PortableInterceptor::ServerRequestInfo_ptr ri)
    throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
    {

    CDMW_LOG_FUNCTION(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),"");

        std::string issuer = "";
        IOP::ServiceContext_var sc ;      
        
        if ( !CORBA::is_nil (m_orb_server))
        {
            CORBA::String_var actual_method = ri->operation();              
            if (check_command(actual_method)) 
            { 
                try
                {
                    // get Service Context
                    sc = ri->get_request_service_context(m_service_id);
                    if (sc->context_id == m_service_id)
                    {
                        CORBA::OctetSeq data(sc->context_data.length());
                        data.length(sc->context_data.length());
                        
                        for (unsigned int i=0; i < sc->context_data.length(); i++)
                        {
                            data[i] = sc->context_data[i];
                        }
                        // decode info
                        m_codec->decode(data, issuer);
                    }
                }catch (const CORBA::BAD_PARAM& ex) 
                { 
                    // the context service is empty in case of unmanaged process 
                    // do nothing 
                }catch (IOP::Codec::FormatMismatch ex) 
                {
                    // data format is not match 
                }catch (...) 
                { 
                    // the context service is empty in case of unmanaged process 
                    // do nothing 
                }

                 try 
                 {
                     m_current->set_issuer(issuer.c_str());
                     
                 }catch (const CORBA::SystemException& ex)
                 {
                      CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                           "ServerInterceptor::receive_request : issuer can't be got, CORBA::SystemException caught");

                      CDMW_LOG_FUNCTION_EXCEPTION(ex);
                      
                 }catch (...)
                 {
                      CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                           "ServerInterceptor::receive_request : issuer can't be got, unknown exception caught");
                 }

            }
            }
    }

    void
    ServerInterceptor::send_reply(PortableInterceptor::ServerRequestInfo_ptr ri)
        throw (CORBA::SystemException)  
    {
      
    }

    void
    ServerInterceptor::send_other(PortableInterceptor::ServerRequestInfo_ptr ri)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
    {
      
    }

    void
    ServerInterceptor::send_exception(PortableInterceptor::ServerRequestInfo_ptr ri)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
    {
      
    }


} // End namespace Cdmw
}

