/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <SystemMngt/platformftproxy/ServerOrbInitializer.hpp>
#include <SystemMngt/platformftproxy/ServerInterceptor.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>

#include <Foundation/orbsupport/TypeTraits.hpp>
#include <Foundation/orbsupport/Codec.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>

#include <SystemMngt/tools/Current_impl.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtCurrent.stub.hpp>

#include <iostream>
#include <string>

namespace Cdmw
{
namespace PlatformMngt
{

    // constructor
   ServerOrbInitializer::ServerOrbInitializer()
   { 
       m_server=0;
       post_done=false;
    }

    //destructor
    ServerOrbInitializer::~ServerOrbInitializer() { }

    
    void
    ServerOrbInitializer::set_orb(CORBA::ORB_ptr orb)
    {
        m_orb =  CORBA::ORB::_duplicate (orb);
        if (post_done)
        {        
            m_server->set_orb(m_orb);
        }
    }

    //
    // IDL to C++ Mapping
    //
    void
    ServerOrbInitializer::pre_init(PortableInterceptor::ORBInitInfo_ptr info) 
    throw (CORBA::SystemException) {}


    //
    // Note: the interceptors are registered in post_init()
    //       if their constructors require
    //       ORBInitInfo::resolve_initial_reference(), which XXX
    //       cannot be called in pre_init().
    //
    void
    ServerOrbInitializer::post_init(PortableInterceptor::ORBInitInfo_ptr info) 
    throw (CORBA::SystemException)
    {       
        m_server=new ServerInterceptor();
        PortableInterceptor::ServerRequestInterceptor_var server_interceptor = m_server;
        post_done=true;
        
        try {
            info->add_server_request_interceptor(server_interceptor.in());
        }
        catch(const PortableInterceptor::ORBInitInfo::DuplicateName&) {
            // can't happened while interceptor is not named!
            CDMW_ASSERT(false);
        }
  
         PortableInterceptor::SlotId issuer = info->allocate_slot_id();
  
         CORBA::Object_var obj_current = info->resolve_initial_references("PICurrent"); 
             
         PortableInterceptor::Current_var corba_current = PortableInterceptor::Current::_narrow(obj_current.in());
  
         CdmwPlatformMngt::Current_var pi_current = new Cdmw::PlatformMngt::Current_impl (corba_current, issuer);
         
         info->register_initial_reference(CdmwPlatformMngt::Current::CURRENT_NAME,pi_current.in());

    }

} // End namespace Cdmw
} // End namespace PlatformMngt
