/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_PROXY_SIM_PROCESS_ACCESSOR_HPP
#define INCL_PLATFORMMNGT_PROXY_SIM_PROCESS_ACCESSOR_HPP

#include <Foundation/common/Exception.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>

#include "testbasic/SIM_ManagedElementAccessor_impl.hpp"
#include "testbasic/SIM_RWEntityContainerAccessor_impl.hpp"
#include "testbasic/SIM_RWServiceDefContainerAccessor_impl.hpp"




#include <SystemMngt/idllib/CdmwFTSystMngtProcess.skel.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtElementRepository.stub.hpp>


namespace Cdmw
{
namespace PlatformMngt
{
namespace Proxy
{


/**
 *Purpose: Provides the process access.
 *<p>
 */
class SIM_ProcessAccessor_impl 
    : public virtual POA_CdmwFTSystMngt::ProcessAccessor,
      public SIM_ManagedElementAccessor_impl,
      public SIM_RWEntityContainerAccessor_impl,
      public SIM_RWServiceDefContainerAccessor_impl
{
    
private:
    /**
     *Purpose:
     *<p> Copy constructor is not allowed.
     */
    SIM_ProcessAccessor_impl(const SIM_ProcessAccessor_impl& rhs);

    /**
     * Purpose:
     * <p> Assignment operator is not allowed.
     */ 
    SIM_ProcessAccessor_impl& operator=(const SIM_ProcessAccessor_impl& rhs);

public:
    /**
     *Purpose:
     *<p> constructor.
     */
    SIM_ProcessAccessor_impl()
            throw();

    /**
     *Purpose:
     *<p> Default destructor.
     */
    virtual
    ~SIM_ProcessAccessor_impl();
    
    //CdmwPlatformMngt::ProcessMode get_process_mode(const char*)throw(CORBA::SystemException);
    CORBA::Boolean get_memory_resident(const char*)throw(CORBA::SystemException);

    CORBA::Long get_core_dump_size(const char*)throw(CORBA::SystemException);

    
    void set_memory_resident(const char*, bool) throw(CORBA::SystemException);
    
    CdmwPlatformMngt::RequestStatus stop_process(const char*, CORBA::Boolean emergency, const char* issuer)   
       throw (CORBA::SystemException);



    /**
     *Purpose:
     *<p> facade to get an attribute
     *
     */

     void get_attribute( const char * target_element_path, 
                         const char * attr_name, 
                         CORBA::Any_out ret_attr, 
                         CdmwPlatformMngt::Owner flag)  
        throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound, CdmwPlatformMngt::ManagedElementNotFound);

    /**
     *Purpose:
     *<p> facade to set an attribute
     *
     */

     void set_attribute( const char * target_element_path, 
                         const char * attr_name, 
                         const CORBA::Any & attr, 
                         CdmwPlatformMngt::Owner flag)  
        throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound, CdmwPlatformMngt::ReadOnlyAttribute, CdmwPlatformMngt::ManagedElementNotFound);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/get_type:1.0
     * operation
     */
    CdmwPlatformMngt::ProcessType get_type (const char* target_element_path)
              throw(CdmwPlatformMngt::ManagedElementNotFound,
                    CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/get_embedded_service:1.0
     * operation
     */
    CORBA::Object_ptr get_embedded_service(const char* target_element_path)
    throw(CdmwPlatformMngt::ManagedElementNotFound,
          CORBA::SystemException);
              
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/get_pid:1.0
     * operation
     */          
    CdmwPlatformMngt::ProcessPID get_pid(const char* target_element_path)
    throw(CdmwPlatformMngt::ManagedElementNotFound,
          CORBA::SystemException);
    
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/get_created_dynamically:1.0
     * operation
     */
    CORBA::Boolean get_created_dynamically(const char* target_element_path)
    throw(CdmwPlatformMngt::ManagedElementNotFound,
          CORBA::SystemException);
     
     
  
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/get_restart_attempt:1.0
     * operation
     */     
    CORBA::Long get_restart_attempt(const char* target_element_path)
              throw(CdmwPlatformMngt::ManagedElementNotFound,
                    CORBA::SystemException);
    
    
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/get_start_time:1.0
     * operation
     */     
    CdmwPlatformMngtBase::TimeStamp get_start_time(const char* target_element_path)
              throw(CdmwPlatformMngt::ManagedElementNotFound,
                    CORBA::SystemException);
    
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/get_stop_time:1.0
     * operation
     */     
    CdmwPlatformMngtBase::TimeStamp get_stop_time(const char* target_element_path)
              throw(CdmwPlatformMngt::ManagedElementNotFound,
                    CORBA::SystemException);
        
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessAccessor/get_last_exit_code:1.0
     * operation
     */
     CdmwPlatformMngt::ProcessExitInfo get_last_exit_code(const char* target_element_path)throw (CORBA::SystemException );

     /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessAccessor/set_process_arguments:1.0
     * operation
     */
     //virtual void set_process_arguments(const char* target_element_path, const char * process_arguments)throw (CORBA::SystemException);

     /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessAccessor/get_process_arguments:1.0
     * operation
     */
     //char * get_process_arguments(const char* target_element_path)throw (CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessAccessor/set_indicator_relevant_status:1.0
     * operation
     */
     void set_indicator_relevant_status(const char* target_element_path, const bool indicator_relevant_status)throw (CORBA::SystemException);

     /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessAccessor/get_indicator_relevant_status:1.0
     * operation
     */
     bool get_indicator_relevant_status(const char* target_element_path)throw (CORBA::SystemException);


     /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessAccessor/set_component_start_graph:1.0
     * operation
     */
     void set_component_start_graph(const char* target_element_path, const CdmwPlatformMngt::GraphElements& component_graph)throw (CORBA::SystemException);

     /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessAccessor/get_component_start_graph:1.0
     * operation
     */
     CdmwPlatformMngt::GraphElements * get_component_start_graph(const char* target_element_path)throw (CORBA::SystemException);
    

     /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessAccessor/set_component_stop_graph:1.0
     * operation
     */
     void set_component_stop_graph(const char* target_element_path, const CdmwPlatformMngt::GraphElements& component_graph)throw (CORBA::SystemException);

     /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessAccessor/get_component_stop_graph:1.0
     * operation
     */
      CdmwPlatformMngt::GraphElements * get_component_stop_graph(const char* target_element_path)throw (CORBA::SystemException);
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/get_status:1.0
     * operation
     */
    CdmwPlatformMngt::ProcessStatus get_status(
                              const char* target_element_path,
                              CORBA::String_out status_info)
              throw(CdmwPlatformMngt::ManagedElementNotFound,
                    CORBA::SystemException);

    //
    virtual bool get_run_as_root(const char* target_element_path)
              throw(CdmwPlatformMngt::ManagedElementNotFound,
                    CORBA::SystemException)
            {
            //no implemented
            return true;
            }
       

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/get_host:1.0
     * operation
     */          
    char* get_host(const char* target_element_path)
              throw(CdmwPlatformMngt::ManagedElementNotFound,
                    CORBA::SystemException);
              
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/is_auto_ending:1.0
     * operation
     */
    CORBA::Boolean is_auto_ending(const char* target_element_path)
              throw(CdmwPlatformMngt::ManagedElementNotFound,
                    CORBA::SystemException);
              
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/set_auto_ending:1.0
     * operation
     */
    void set_auto_ending(const char* target_element_path,
             CORBA::Boolean auto_ending)
              throw(CdmwPlatformMngt::ManagedElementNotFound,
                    CdmwPlatformMngt::IncompatibleStatus,
                    CORBA::SystemException);


    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/get_process_info:1.0
     * operation
     */
    CdmwPlatformMngt::ProcessInfo* get_process_info(const char* target_element_path)
              throw(CdmwPlatformMngt::ManagedElementNotFound,
                    CORBA::SystemException);
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/set_process_info:1.0
     * operation
     */
    void set_process_info(const char* target_element_path,
                          const CdmwPlatformMngt::ProcessInfo& process_info )
             throw(CdmwPlatformMngt::ManagedElementNotFound,
                   CdmwPlatformMngt::IncompatibleStatus,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/get_life_cycle:1.0
     * operation
     */
    CdmwPlatformMngt::LifeCycleDef* get_life_cycle(const char* target_element_path)
             throw(CdmwPlatformMngt::ManagedElementNotFound,
                   CdmwPlatformMngt::IncompatibleType,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/set_life_cycle:1.0
     * operation
     */
    void set_life_cycle(const char* target_element_path,
                        const CdmwPlatformMngt::LifeCycleDef& life_cycle)
             throw(CdmwPlatformMngt::ManagedElementNotFound,
                   CdmwPlatformMngt::DuplicateStep,
                   CdmwPlatformMngt::IncompatibleType,
                   CdmwPlatformMngt::IncompatibleStatus,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/get_monitoring_info:1.0
     * operation
     */
    CdmwPlatformMngt::CompleteMonitoringInfo get_monitoring_info(
                         const char* target_element_path)
             throw(CdmwPlatformMngt::ManagedElementNotFound,
                   CdmwPlatformMngt::IncompatibleType,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/set_monitoring_info:1.0
     * operation
     */
    void set_monitoring_info(
                         const char* target_element_path,
                         const CdmwPlatformMngt::CompleteMonitoringInfo& monitoring_info)
             throw(CdmwPlatformMngt::ManagedElementNotFound,
                   CdmwPlatformMngt::IncompatibleType,
                   CdmwPlatformMngt::IncompatibleStatus,
                   CORBA::SystemException);
                   
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/get_exit_autorestart_info:1.0
     * operation
     */               
    CdmwPlatformMngt::ExitAutoRestartInfo* get_exit_autorestart_info(
                         const char* target_element_path)
             throw(CdmwPlatformMngt::ManagedElementNotFound,
                   CORBA::SystemException);


    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/set_exit_autorestart_info:1.0
     * operation
     */ 
    void set_exit_autorestart_info(
                         const char* target_element_path,
                         const CdmwPlatformMngt::ExitAutoRestartInfo& exit_autorestart_info)
             throw(CdmwPlatformMngt::ManagedElementNotFound,
                   CdmwPlatformMngt::IncompatibleStatus,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/get_snapshot:1.0
     * operation
     */
    CdmwPlatformMngt::ProcessSnapshot* get_snapshot(const char* target_element_path)
             throw(CdmwPlatformMngt::ManagedElementNotFound,
                   CORBA::SystemException);
             
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/initialise:1.0
     * operation
     */
    CdmwPlatformMngt::RequestStatus initialise(
                          const char* target_element_path,
                          const CdmwPlatformMngtBase::StartupKind& startup_kind)
             throw (CdmwPlatformMngt::ManagedElementNotFound,
                    CdmwPlatformMngt::IncompatibleStatus,
                    CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/next_step:1.0
     * operation
     */
    CdmwPlatformMngt::RequestStatus next_step(const char* target_element_path)
             throw(CdmwPlatformMngt::ManagedElementNotFound,
                   CdmwPlatformMngt::InvalidStep,
                   CdmwPlatformMngt::IncompatibleType,
                   CdmwPlatformMngt::IncompatibleStatus,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/run:1.0
     * operation
     */
    CdmwPlatformMngt::RequestStatus run(const char* target_element_path)
             throw(CdmwPlatformMngt::ManagedElementNotFound,
                   CdmwPlatformMngt::NotReadyToRun,
                   CdmwPlatformMngt::IncompatibleType,
                   CdmwPlatformMngt::IncompatibleStatus,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/stop:1.0
     * operation
     */
    CdmwPlatformMngt::RequestStatus stop(const char* target_element_path,
                                         CORBA::Boolean emergency)
             throw(CdmwPlatformMngt::ManagedElementNotFound,
                   CORBA::SystemException);
             
             
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/cold_restart:1.0
     * operation
     */
    CdmwPlatformMngt::RequestStatus cold_restart(
                          const char* target_element_path,
                          const CdmwPlatformMngtBase::StartupKind& startup_kind,
                          const char* issuer)
             throw (CdmwPlatformMngt::ManagedElementNotFound,
                    CdmwPlatformMngt::IncompatibleStatus,
                    CORBA::SystemException);
                    
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/hot_restart:1.0
     * operation
     */
    CdmwPlatformMngt::RequestStatus hot_restart(
                          const char* target_element_path,
                          const CdmwPlatformMngtBase::StartupKind& startup_kind,
                          const char* issuer)
             throw (CdmwPlatformMngt::ManagedElementNotFound,
                    CdmwPlatformMngt::IncompatibleStatus,
                    CORBA::SystemException);
    
 
    /**
     * Returns the type of the Managed Element
     */
    CdmwPlatformMngt::ElementType get_element_type(const char * target_element_path) throw (CORBA::SystemException);


    /**
     * Returns the children of the Managed Element
     */
    CdmwPlatformMngt::ElementPaths* get_children(const char * target_element_path) throw (CORBA::SystemException);


                  
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/stop_and_wait:1.0
     * operation
     */
    CdmwPlatformMngt::LongRequestStatus stop_and_wait (const char* target_element_path,
                                         CORBA::Boolean emergency)
             throw(CdmwPlatformMngt::ManagedElementNotFound,
                   CORBA::SystemException);

     /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/set_init_graph:1.0
     * operation
     */
     void set_init_graph (const char* target_element_path, const CdmwPlatformMngt::GraphElements& graph_element)
      throw(CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/set_stop_graph:1.0
     * operation
     */
     void set_stop_graph (const char* target_element_path, const CdmwPlatformMngt::GraphElements& graph_element)  
             throw(CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/add_component:1.0
     * operation
     */
     CdmwPlatformMngt::CcmComponent* add_component (const char* target_element_path,
                                                    CdmwPlatformMngt::ComponentDef* comp_def)
       throw (CORBA::SystemException, CdmwPlatformMngt::ComponentAlreadyExists, CdmwPlatformMngt::ComponentServerNotFound,
              CdmwPlatformMngt::DuplicateEntity, CosPropertyService::MultipleExceptions,
              CdmwPlatformMngt::DuplicateService, CdmwPlatformMngt::DuplicateStep,
              CdmwPlatformMngt::IncompatibleStatus);
                              
     /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwFTSystMngt/ProcessAccessor/remove_component:1.0
     * operation
     */
     void  remove_component (const char* target_element_path, const char* component_name, const char* host_name)
         throw(CORBA::SystemException, CdmwPlatformMngt::ComponentNotFound, CdmwPlatformMngt::ComponentStillActive, CdmwPlatformMngt::IncompatibleStatus);

     
     CdmwPlatformMngt::ProcessPID get_previous_pid(const char* target_element_path) 
        throw(CORBA::SystemException,CdmwPlatformMngt::ManagedElementNotFound);
    
    void get_all_components(const char* target_element_path, CORBA::ULong how_many, CdmwPlatformMngt::ElementPaths_out element_paths, CdmwPlatformMngt::ComponentsIterator_out components_iterator) 
        throw(CORBA::SystemException,CdmwPlatformMngt::NotComponentServer,                        CdmwPlatformMngt::ManagedElementNotFound);
    
    char* get_component(const char* target_element_path, const char* component_name) 
    throw(CORBA::SystemException,CdmwPlatformMngt::ComponentNotFound, CdmwPlatformMngt::NotComponentServer,
                    CdmwPlatformMngt::ManagedElementNotFound);
    
    CORBA::ULong get_number_of_components(const char* target_element_path)
        throw(CORBA::SystemException,CdmwPlatformMngt::NotComponentServer,         CdmwPlatformMngt::ManagedElementNotFound);
    
    CdmwPlatformMngt::CcmComponent* add_component(const char* target_element_path, const CdmwPlatformMngt::ComponentDef& component_def)
        throw(CORBA::SystemException,CdmwPlatformMngt::ComponentAlreadyExists, CdmwPlatformMngt::ComponentServerNotFound,
            CdmwPlatformMngt::DuplicateEntity, CosPropertyService::MultipleExceptions,
            CdmwPlatformMngt::DuplicateService, CdmwPlatformMngt::DuplicateStep,
            CdmwPlatformMngt::IncompatibleStatus);       
         
         
};

} // End namespace Proxy
} // End namespace PlatformMngt
} // End namespace Cdmw

#endif //INCL_PLATFORMMNGT_PROXY_SIM_PROCESS_ACCESSOR_HPP

