/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_CDMW_PLATFORMMNGT_CDMW_INTERFACE_HPP 
#define INCL_CDMW_PLATFORMMNGT_CDMW_INTERFACE_HPP

#include "Foundation/orbsupport/CORBA.hpp"

namespace Cdmw {         // Begin namespace Cdmw
namespace PlatformMngt { // Begin namespace PlatformMngt

/**
* Purpose:
* <p> Initialise the CDMW runtime and services within the current process.
* Implementation is generated by CDMW code generation tools according to user
* provided description.
* 
*@param orb  A reference to the ORB pseudo-object
*@param argc The number of arguments passed to the program
*@param argv The list of parameters received of the command line
*@param supervisionProcessName The name of the supervision process
*@param fault_manager_corbaloc The corbaloc of the FT manager
*
*@exception CORBA::BAD_PARAM Invalid parameter. The following CDMW minor codes 
* detail the reason: 
* <DL>
* <DT><STRONG>BAD_PARAMProcessXMLFileNameMissing</STRONG>
* <DD>The CDMW Process XML initialisation file name is missing from the command
* line arguments.
* <DT><STRONG>BAD_PARAMInvalidXMLInitData</STRONG>
* <DD>Inivalid XML initialisation data.
* </DL>
*@exception CORBA::INTERNAL Internal failure.The following CDMW minor codes 
* detail the reason: 
* <DL>
* <DT><STRONG>INTERNALXMLLibraryError</STRONG>
* <DD>Failure in XML library.
* <DT><STRONG>INTERNAL</STRONG>
* <DD>Internal CDMW runtime failure.
* <DT><STRONG>INTERNALORBAdapterInactive</STRONG>
* <DD>The POA Manager is in the <I>inactive</I> state.
* </DL>
*@exception CORBA::NO_MEMORY CDMW runtime could not allocate memory
*@exception CORBA::SystemException Any CORBA system exception that can be
* raised by the undelying ORB.
*/ 
extern void CDMW_init(CORBA::ORB_ptr orb, 
                      int & argc, 
                      char** argv,
                      const char* supervision_process_name,
                      const char* fault_manager_corbaloc)
    throw (CORBA::SystemException);

/**
* Purpose:
* <p> Clean any static data allocated/initialised by CDMW_init.
* Implementation is generated by CDMW code generation tools according to user
* provided description.
* 
*@param orb  A reference to the ORB pseudo-object   
*
*/
extern void CDMW_cleanup(CORBA::ORB_ptr orb) throw();


} // End namespace PlatformMngt
} // End namespace Cdmw

#endif // INCL_CDMW_PLATFORMMNGT_CDMW_INTERFACE_HPP
