/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <Foundation/common/Exception.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <string>
#include <list>
#include <map>

namespace Cdmw
{
namespace PlatformMngt
{   

class ConfigurationException : public Exception
{
public:

    ConfigurationException(const std::string& message, size_t lineNumber)
        : Exception( Exception::FATAL, "Invalid configuration file" ),
          m_message(message), m_lineNumber(lineNumber)
    {
    }

    ~ConfigurationException() throw() {};

    const char* getMessage() const
    {
        return m_message.c_str();
    }

    size_t getLineNumber() const
    {
        return m_lineNumber;
    }

private:
    std::string m_message;
    size_t m_lineNumber;

};
/**
*Purpose:
*<p> 
*
*Features:
*<p> Thread safe
*
*@see  
*
*/
class ConfigurationSup
{

public:

    /**
     * Purpose:
     * <p> Constructor. Sets default values.
     * 
     */ 
    ConfigurationSup();


    /**
     * Purpose:
     * <p> Initializes the configuration from an XML file.
     * 
     * @param xml_conf The XML file path
     * @param validate Validate the XML file before processing
     */ 
    void parse(const std::string& xml_conf, bool validate)
        throw(OutOfMemoryException, ConfigurationException);


    /**
     * Purpose:
     * <p> Destructor
     * 
     */ 
    virtual 
    ~ConfigurationSup()
        throw();


    /**
     * Purpose:
     * <p> Returns the system definition file.
     */ 
    const char* getSystem_definition_file() const throw();

    /**
     * Purpose:
     * <p> Returns the Host_start_control_auto_start optinion.
     */ 
    const char* getSystem_auto_start() const throw();
    /**
     * Purpose:
     * <p> Returns the Host_start_control_auto_start options.
     */ 
    const char* getHost_auto_start() const throw();
    /**
     * Purpose:
     * <p> Returns the daemon_controller_port_number.
     */ 
    std::string getDaemon_controller_port_number() const throw();

private:   
    std::string System_start_control_system_definition_file;
    std::string System_start_control_auto_start;
    std::string Host_start_control_auto_start;
    std::string Daemon_controller_port_number;
    
    
}; // End class ConfigurationSup 
} // End namespace LB
} // End namespace PlatformMngt


