/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "platformftsystem/PeriodicNotifierDataStoreObserver.hpp"
#include "platformlibrary/Configuration.hpp"
#include "Foundation/logging/Logger.hpp"
#include <iostream>

namespace Cdmw
{
namespace PlatformMngt
{


// ----------------------------------------------------------------------
// PeriodicNotifierDataStoreObserver class.
// ----------------------------------------------------------------------

PeriodicNotifierDataStoreObserver::PeriodicNotifierDataStoreObserver (Cdmw::PlatformMngt::Periodic::Notifier_impl* p_notifier)
throw()
:m_notifier (p_notifier)
{     
    
}


PeriodicNotifierDataStoreObserver::~PeriodicNotifierDataStoreObserver()
throw()
{
}


void PeriodicNotifierDataStoreObserver::on_insert(_Oid_in_type oid, _Data_in_type data)
throw ()
{
    m_notifier->define_ds();
    
    if (!Cdmw::PlatformMngt::Configuration::Is_primary())
    {
           CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                      "<<PeriodicNotifierDataStoreObserver::on_insert (" << oid <<")>>");
           
        m_notifier->update_observers_infos(data.periodic_observers);

    }
    
    // else nothing to do because we are the primary and we are inserting the element 
    // into the datastore
}

void PeriodicNotifierDataStoreObserver::on_update(_Oid_in_type oid, _Data_in_type data)
throw ()
{

    if (!Cdmw::PlatformMngt::Configuration::Is_primary())
    {
        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                "<<PeriodicNotifierDataStoreObserver::on_update (" << oid <<")>>");
        
        m_notifier->update_observers_infos(data.periodic_observers);
    }
    // else nothing to do because we are the primary and we are inserting the element 
    // into the datastore
}

void PeriodicNotifierDataStoreObserver::on_remove(_Oid_in_type oid) 
throw ()
{       

}


} // End namespace PlatformMngt
} // End namespace Cdmw



