/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#if CDMW_ORB_VDR == tao
#include "tao/PI/ORBInitInfo.h"
#endif

#include "platformftsystem/PrimaryActivationHandler_impl.hpp"

#include "SystemMngt/platformlibrary/Configuration.hpp"
#include "SystemMngt/platformlibrary/SupervisionEventNotifier.hpp"
#include "platformsystemlibrary/Host_impl.hpp"
#include "platformsystemlibrary/CentralisedSupervisionSystem_impl.hpp"

#include "SystemMngt/platformapplicationlibrary/Application_impl.hpp"
#include "SystemMngt/platformapplicationlibrary/ApplicationControl_impl.hpp"
#include "platformapplicationlibrary/ProcessObserver_impl.hpp"

#include "Foundation/ossupport/OS.hpp"
#include "Foundation/osthreads/ThreadHandle.hpp"
#include "Foundation/osthreads/MutexGuard.hpp"
#include "Foundation/common/Exception.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/logging/Logger.hpp"

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>

#include "FaultTolerance/idllib/CdmwFTGroupRepository.stub.hpp"

#include "idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"

#include <iostream>

namespace Cdmw 
{
namespace PlatformMngt
{
    //    
    // Dumper of event notification from datastore
    // operator ()
    //    
    void EventNotifDumper::operator()(const PairType &elt)
    {
        // for the extracted record from datastore
        // store it in temporary list in synchro processing
        m_synchro_proc->add_event_notif ((elt.second).in());
    }    
    
    //    
    // Dumper of event report from datastore
    // operator ()
    //    
    void EventReportDumper::operator()(const PairType &elt)
    {
        // for the extracted record from datastore
        // store it in temporary list in synchro processing
        
        // make a deep copy (elt returns a var)
        CdmwPlatformMngtBase::EventKey_var event_key = elt.first;
        CdmwPlatformMngt::EventReportRecord_var event_report = elt.second;
        
        // and store in synchro processing
        m_synchro_proc->add_event_report (event_key._retn(),event_report._retn());
    }   
    
    

    SynchroProcessing::SynchroProcessing (CORBA::ORB_ptr orb,
                                          CdmwFTSystMngt::SystemAccessor_ptr system_accessor_ref,
                                          CentralisedSupervisionSystem_impl* system,
                                          EventNotifierDataStore* event_notif_ds,
                                          EventNotifierDataStore* event_notif_ext_ds,
                                          EventReportDataStore* event_report_ds,
                                          PrimaryActivationHandler_impl* handler)
            : m_system (system),
              m_event_notif_ds (event_notif_ds),
              m_event_notif_ext_ds (event_notif_ext_ds),
              m_event_report_ds (event_report_ds),
              m_handler (handler)
    {
        m_orb = CORBA::ORB::_duplicate(orb);
        m_system_accessor_ref = CdmwFTSystMngt::SystemAccessor::_duplicate(system_accessor_ref);
    }


    SynchroProcessing::~SynchroProcessing()
            throw()
    {
    }

    void SynchroProcessing::add_event_notif(CdmwPlatformMngt::Event * event_notif)
    {
        // duplicate the reference
        CORBA::add_ref(event_notif);
        CdmwPlatformMngt::Event_var event_notif_var = event_notif;
        
        // store in the list
        m_event_notif_list.push_back (event_notif_var);
    }
    
    void SynchroProcessing::add_event_report(CdmwPlatformMngtBase::EventKey event_key,
                                             CdmwPlatformMngt::EventReportRecord * event_report)
    {
        m_event_key_list.push_back (event_key);
        m_event_report_list.push_back (event_report);
    }
    

    void SynchroProcessing::run()
            throw()
    {
        
        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                  "<<SynchroProcessing::run() starting>>");
        
        // set synchronising in progress
        Configuration::Set_synchronising();
                            
        // set process as primary
        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                  "<<SynchroProcessing::run() !!set process as primary !!>>");
        
        //////
        // get the hostname
            std::string hostname=OsSupport::OS::get_hostname();

            const char* result = 0 ;
            CORBA::Any_var property_any = new CORBA::Any();

            //Reading the property DEFAULT value of GLOBAL_SUPERVISION_HOST
            try 
            {
                m_system->get_attribute (CdmwPlatformMngt::GLOBAL_SUPERVISION_HOST, property_any.out(), CdmwPlatformMngt::PRE_DEFINED);
                property_any.in() >>= result;
            }
            catch(...)
            {
                CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                     "\nError reading the property: " << CdmwPlatformMngt::GLOBAL_SUPERVISION);
            }

            // Updating the system property GLOBAL_SUPERVISION_HOST with the current value of hostname
            CORBA::Any_var any_hostname = new CORBA::Any();
            try
            {
                *any_hostname <<= hostname.c_str();
                m_system -> set_attribute_internal (CdmwPlatformMngt::GLOBAL_SUPERVISION_HOST, 
                                                    any_hostname.in(),
                                                    CdmwPlatformMngt::PRE_DEFINED);
            }
            catch(const CdmwPlatformMngt::AttributeNotFound&)
            {
                CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                        "\nProperty not found: " << CdmwPlatformMngt::GLOBAL_SUPERVISION);
            }
            
            
            //Reading the property GLOBAL_SUPERVISION_HOST
            try {

                m_system->get_attribute (CdmwPlatformMngt::GLOBAL_SUPERVISION_HOST,
                                         property_any.out(),
                                         CdmwPlatformMngt::PRE_DEFINED);
                property_any.in() >>= (result);
            }
            catch(...)
            {
                CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                     "\nError reading the property: " << CdmwPlatformMngt::GLOBAL_SUPERVISION);
            }
            
            
            // Check if system is defined 
            CORBA::ULong nb_hosts=m_system->get_number_of_hosts();
            if(nb_hosts != 0)
            {
                CdmwPlatformMngt::Hosts_var hosts;
                CdmwPlatformMngt::HostsIterator_var rest;
                m_system->get_all_hosts(nb_hosts,hosts.out(),rest.out());
                
                CORBA::Any_var property_host_any = new CORBA::Any();
                Cdmw::PlatformMngt::Host_impl* current_host = 0;
                
                for (CORBA::ULong count = 0; count<nb_hosts; count++)
                {
                    //Get the Host_impl* reference  
                    current_host = m_system->getHost( hosts[count]->name() ) ;
                    //Update the property GLOBAL_SUPERVISION of all the hosts in the system
                    if( 0 == strcmp ( hostname.c_str(),( current_host->name() ) )  )
                    {
                        *property_host_any <<= CORBA::Any::from_boolean(true);
                    }else
                    {
                        *property_host_any <<= CORBA::Any::from_boolean(false); 
                    }
                    try
                    {
                        current_host-> set_attribute_internal (CdmwPlatformMngt::GLOBAL_SUPERVISION, 
                                                               property_host_any.in(), 
                                                               CdmwPlatformMngt::PRE_DEFINED);
                    }
                    catch(const CdmwPlatformMngt::AttributeNotFound&)
                    {
                        CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                                "\nCannot Proceed setting the Property : " << CdmwPlatformMngt::GLOBAL_SUPERVISION);
                    }
                }
                //Reading the property GLOBAL_SUPERVISION
                CORBA::Boolean property_val;
                try 
                {
                    current_host->get_attribute (CdmwPlatformMngt::GLOBAL_SUPERVISION, property_any.out(), CdmwPlatformMngt::PRE_DEFINED);
                    property_any.in() >>=CORBA::Any::to_boolean(property_val);
                }
                catch(...)
                {
                    CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                            "\nCannot Proceed setting the Property : " << CdmwPlatformMngt::GLOBAL_SUPERVISION);
                }
            }else
            {
                 CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                            "System is not defined");
            }
        
        Configuration::Set_primary();
           
        // if it is the first primary
        if (Configuration::Is_first_primary())
        { 
            // indicate to handler it can return to set primary in FT group
            m_handler->set_synchro_ready();
        }
        
        // else it is not the first primary (backup becomes primary)
        else
        {  
            // synchro the event notification
            CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                      "<<SynchroProcessing::run() send event notifications>>");
            m_event_notif_ds->for_each(EventNotifDumper(this));
            m_event_notif_ext_ds->for_each(EventNotifDumper(this));
            
            // get the system event notifier
            SupervisionEventNotifier* event_notifier = m_system->get_event_notifier();
            
            // execute the appropriate event notif
            EventNotifList::iterator event_notif_it;
            
            int i;
    
            // for each event of the list
            for (i=1, event_notif_it = m_event_notif_list.begin();
                 event_notif_it != m_event_notif_list.end();
                 i++, event_notif_it++)
            {
                try
                {    
                    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                              "<<SynchroProcessing::run() send notification " << i << ">>");
                                   
                    // notify the event (event reference is removed in notifier)                  
                    event_notifier->pushEvent ((*event_notif_it)._retn());
                }
                catch (...)
                {
                    // nothing to do
                }
            }
            
            // clear the list of notifications
            m_event_notif_list.clear();
                


            // get the event reports stored in datastore
            CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                      "<<SynchroProcessing::run() get event reports>>");
            m_event_report_ds->for_each(EventReportDumper(this));

            
            // freeze all event handlers to not postpone the received event
            CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                      "<<SynchroProcessing::run() freeze all event handlers>>");
            m_system->freeze_event_handlers();                     
             


            // indicate to handler it can return to set primary in FT group
            m_handler->set_synchro_ready();
            
            
            // wait objects are set to primary            
            CdmwFT::Location::PrimaryBackupGroupRepository_var ft_group_repository;
                        
            try
            {
                CORBA::Object_var obj = m_orb->resolve_initial_references("FTGroupRepository");

                ft_group_repository = 
                     CdmwFT::Location::PrimaryBackupGroupRepository::_narrow(obj.in());
            }
            catch (const PortableInterceptor::ORBInitInfo::InvalidName& )
            {
                CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                           "<<SynchroProcessing::run() Impossible to resolve initial reference : InvalidName>>");
                CDMW_ASSERT(false);
            }
            catch (const CORBA::SystemException& ex)
            {
                CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                           "<<SynchroProcessing::run() Impossible to resolve initial reference : SystemException>>"
                           << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex));
                CDMW_ASSERT(false);
            }
            
            try
            {
                bool is_primary = false;
                
                while (!is_primary)
                {
                    // wait the setting of primary
                    OsSupport::OS::sleep(20);
                    
                    is_primary = ft_group_repository->is_primary(m_system_accessor_ref.in());                                
                }
            }
            catch (const ::FT::MemberNotFound &)
            {
                CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                           "<<SynchroProcessing::run() Member not Found exception raised while checking the primary>>");
                CDMW_ASSERT(false);
            }
            catch (const CORBA::SystemException& ex)
            {
                CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                           "<<SynchroProcessing::run() System exception raised while checking the primary>>"
                           << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex));
                CDMW_ASSERT(false);
            }

            // now the FT objects are set to primary
            // FT request will be received and processed
            // and coordinator of datastore is set
            
                    
            // synchronise the processing of system processes
            CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                      "<<SynchroProcessing::run() processes sync>>");
                        
            try
            {
                m_system->synchro_processing_of_processes();
            }
            catch (...)
            {
                CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                           "<<SynchroProcessing::run() processes sync exception>>");
            }
                
            // synchronise the processing of the applications
            CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                      "<<SynchroProcessing::run() applications sync>>");
            
            try
            {
                m_system->synchro_processing_of_applications();  
            }
            catch (...)
            {
                CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                           "<<SynchroProcessing::run() applications sync exception>>");
            }
            
            // synchronise the system processing
            CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                      "<<SynchroProcessing::run() system sync>>");
            
            try
            {
                m_system->synchro_processing();
            }
            catch (...)
            {
                CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                           "<<SynchroProcessing::run() system sync exception>>");
            }
            
            // synchronise the event reports
            CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                      "<<SynchroProcessing::run() send event reports>>");

            // execute the appropriate event report got from datastore
            // and registered in the list
            EventReportList::iterator event_report_it;
    
            // for each event of the list
            for (i=1, event_report_it = m_event_report_list.begin();
                 event_report_it != m_event_report_list.end();
                 i++, event_report_it++)
            {
                CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                          "<<SynchroProcessing::run() send report " << i << ">>");
                
                CdmwPlatformMngt::EventReportRecord * event_report = *event_report_it;
                    
                // take owner ship of the data structure                                              
                CdmwPlatformMngt::EventReportRecord_var event_report_var = event_report;
                    
                try
                {                                            
                    if (strcmp(event_report->application_name.in(),"") != 0)
                    {     
                        Application_impl* application;
                        ApplicationControl_impl* app_control;
                  
                        // get the appropriate application control
                        m_system->getApplicationServants (event_report->application_name.in(),
                                                          application, app_control);
            
                        // if application control event type
                        if (event_report->event_type == CdmwPlatformMngt::INVALIDATE_PROCESS_EVENT)
                        {
                            CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                                      "<<SynchroProcessing::run() ApplicationControl synchro_event calling: invalidate process event>>");
                            
                            // execute the event report                  
                            app_control->synchro_event (*event_report);
                        }
                
                        // else it is a process event type
                        else
                        {
                            // get the process observer of the application
                            ProcessObserver_impl* process_observer =
                            app_control->get_process_observer_servant();
                   
                            CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                                      "<<SynchroProcessing::run() ProcessObserver synchro_event calling>>");
                            
                            // execute the event report                  
                            process_observer->synchro_event (*event_report);  
                        }
                    }  
                    
                    else
                    {
                        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                                  "<<SynchroProcessing::run() send report not processed>>");
                    }
                                      
                }
                catch (...)
                {
                    // nothing to do
                }
            }
            
            // clear the list
            m_event_report_list.clear();
            
            
            // delete event records in datastore
            // stored by previous primary
            EventKeyList::iterator event_key_it;
    
            // for each key of the list
            for (event_key_it = m_event_key_list.begin();
                 event_key_it != m_event_key_list.end();
                 event_key_it++)
            {
                // remove event from datastore
                try
                {
                    m_event_report_ds->remove(*event_key_it);
                }
                catch (const Common::NotFoundException&)
                {
                    // nothing to do
                }
                catch (const CommonSvcs::DataStore::StorageErrorException &)
                {
                }
            }
            
            // clear the list
            m_event_key_list.clear(); 
            
            // unfreeze all event handlers to process received event
            // during synchronisation
            
            CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                      "<<SynchroProcessing::run() unfreeze all event handlers>>");
            m_system->unfreeze_event_handlers();                                            
        }
    
        // align status status     
        m_system->synchro_consistent_status(); 
        
        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                  "<<Periodic Notifier::notify_registered_observers() ");
        //notify all registered observer
        m_handler->get_periodic_notifier()->notify_registered_observers();   
        
        // set synchronising done
        Configuration::Set_synchronised();

        
        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                  "<<SynchroProcessing::run() done>>");
    }
        


    PrimaryActivationHandler_impl::PrimaryActivationHandler_impl(
             CORBA::ORB_ptr orb,
             CdmwFTSystMngt::SystemAccessor_ptr system_accessor_ref,
             CentralisedSupervisionSystem_impl* system,
             PluginManager* plugin_manager)
        : m_system (system),
          m_sync_ready_condition(m_sync_ready_mutex),
          m_plugin_manager( plugin_manager )
    {
        m_synchro_processing = NULL;
        
        try
        {
            // get the event notification datastore interface
            Cdmw::CommonSvcs::DataStore::DataStoreBase* ds =
            Cdmw::CommonSvcs::DataStore::DataStoreRegistry::Get_data_store(
                   Configuration::Get_event_notifier_datastore_id());
            m_event_notif_ds = 
                 dynamic_cast<Cdmw::PlatformMngt::EventNotifierDataStore*>(ds);
                 
            // get the event report datastore interface
            Cdmw::CommonSvcs::DataStore::DataStoreBase* ds2 =
                Cdmw::CommonSvcs::DataStore::DataStoreRegistry::Get_data_store(
                   Configuration::Get_event_report_datastore_id());
            m_event_report_ds = 
                 dynamic_cast<Cdmw::PlatformMngt::EventReportDataStore*>(ds2);

            
           // get the event notification datastore interface for configuration change
            Cdmw::CommonSvcs::DataStore::DataStoreBase* ds3 =
            Cdmw::CommonSvcs::DataStore::DataStoreRegistry::Get_data_store(
                   Configuration::Get_event_notifier_ext_datastore_id());
            m_event_notif_ext_ds =
                 dynamic_cast<Cdmw::PlatformMngt::EventNotifierDataStore*>(ds3);

           
            // create the synchro processing class    
            m_synchro_processing = new SynchroProcessing (
                        orb, system_accessor_ref, m_system, 
                        m_event_notif_ds, m_event_notif_ext_ds, m_event_report_ds, this);
        }
        catch (const NotExistException& e)
        {
            CDMW_ASSERT(false);
        }
    }
    
    PrimaryActivationHandler_impl::~PrimaryActivationHandler_impl()
    {
        if (m_synchro_processing != NULL)
        {
            delete m_synchro_processing;
        }
    }
    
    //
    // IDL:thalesgroup.com/CdmwFT/Location/ActivationHandler/activate:1.0
    //
    void PrimaryActivationHandler_impl::activate()
        throw(CORBA::SystemException)
    {
        // The server will be set as primary by the FT layer just after
        // the return of this handler, then all FT requests received
        // by this new primary will be accepted
        
        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                  "<<PrimaryActivationHandler_impl::activate()>>");
        
        // synchronise the system with its datastore
        m_system->synchro_with_datastore();
        
        // synchronise the applications and processes with their datastore
        m_system->synchro_applications_with_datastore();
       
        // synchronise the hosts with their datastore
        m_system->synchro_hosts_with_datastore();
        
        // synchronise the entity with their datastore
        m_system->synchro_entities_with_datastore();
    
        // activate local plugin
        if (m_plugin_manager != NULL) {
            if ( !(m_plugin_manager->run_all()) ) {
                CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                  "<<PrimaryActivationHandler_impl::activate() Error during plugin activation>>");
            }    
        }
 
        // lock the mutex of ready flag
        CDMW_MUTEX_GUARD(m_sync_ready_mutex);
        
        // reset the ready flag
        m_sync_ready = false;
        
        // start the synchro processing thread
        m_synchro_processing->start();
                
        /*****
        // wait the termination of thread
        m_synchro_processing->join();
        *****/
        
        // wait the ready flag
        while (!m_sync_ready)
        {
            m_sync_ready_condition.wait();
        }

        
                //MRI start all the host consistency checkers
        CORBA::ULong how_many = m_system->get_number_of_hosts ();
    
        CdmwPlatformMngt::ElementNames_var host_names;
        CdmwPlatformMngt::ElementNamesIterator_var rest;

        m_system->get_all_host_names (how_many, host_names, rest);

        for (CORBA::ULong count = 0U; count< how_many; count++)
        {
        //synchro the host with its datastore 
        m_system->getHost(host_names[count])->start_consistency_checker();
        }
        
        // now the FT group can be updated to receive the FT request
        
        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                  "########## Supervision is active.");
    }
    

    //
    // IDL:thalesgroup.com/CdmwFT/Location/ActivationHandler/deactivate:1.0
    //
    void PrimaryActivationHandler_impl::deactivate()
        throw(CORBA::SystemException)
    {
        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                  "PrimaryActivationHandler_impl::deactivate()");
       
        // set process as backup
        Configuration::Set_backup();
        
    }
    
    //
    // set synchro ready to start
    //
    void PrimaryActivationHandler_impl::set_synchro_ready()
    {
        // first lock the mutex of ready flag
        CDMW_MUTEX_GUARD(m_sync_ready_mutex);
        
        // set the ready flag
        m_sync_ready = true;
        
        // broadcast the condition
        m_sync_ready_condition.broadcast();
    }
    
    
    void PrimaryActivationHandler_impl::set_periodic_notifier(Periodic::Notifier_impl* periodic_notifier)
    {
        m_periodic_notifier = periodic_notifier;
    }
    
    
    Periodic::Notifier_impl* PrimaryActivationHandler_impl::get_periodic_notifier()
    {
        Periodic::Notifier_impl* periodic_notifier = m_periodic_notifier;
        
        return periodic_notifier;
    }

} // End of namespace PlatformMngt
} // End of namespace Cdmw

