/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
 
#include <SystemMngt/tools/ManagedElementKey.hpp>
#include <SystemMngt/platformlibrary/Configuration.hpp>

#include <Foundation/logging/Logger.hpp>

#include "platformftsystem/RedundancyGroupDataStoreObserver.hpp"

#include <SystemMngt/idllib/CdmwPlatformMngtApplicationExt.stub.hpp>

namespace Cdmw
{
namespace PlatformMngt
{


// ----------------------------------------------------------------------
// RedundancyGroupDataStoreObserver class.
// ----------------------------------------------------------------------

RedundancyGroupDataStoreObserver::RedundancyGroupDataStoreObserver (CdmwPlatformMngt::ElementRepository_ptr element_repository)
  throw()
{  
    m_element_repository = 
           CdmwPlatformMngt::ElementRepository::_duplicate(element_repository);
              
    // create the managed element storage home
    m_storage_home = new ManagedElementStorageHome(
          Configuration::Get_managed_element_datastore_id());   
}


RedundancyGroupDataStoreObserver::~RedundancyGroupDataStoreObserver()
  throw()
{
    delete m_storage_home;
}


void RedundancyGroupDataStoreObserver::on_insert(_Oid_in_type oid, _Data_in_type data)
  throw ()
{
    if (!Cdmw::PlatformMngt::Configuration::Is_primary())
    {
         try
         {
            CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                      "<<RedundancyGroupDataStoreObserver::on_insert (" << oid <<")>>");
            
             // get the application parent of process           
             std::string group_name ("");
             std::string parent_path ("");
             
             // get the element name and parent path
             ManagedElementKey::Get_parent_path (oid, group_name, 
                                                 parent_path);
         
             // get the application object reference
             CdmwPlatformMngt::ManagedElement_var element_obj =
                         m_element_repository->get_element(parent_path.c_str());
                         
             CdmwPlatformMngt::Application_var application_obj =
                         CdmwPlatformMngt::Application::_narrow (element_obj.in());
                    
             if ( CdmwPlatformMngt::FT_GROUP_TYPE == data.group_type )
         {
             // Create the group definition
                 CdmwPlatformMngt::FTGroupDef_var ftgroup_def = new CdmwPlatformMngt::FTGroupDef();
         
         // Set the group definition
         ftgroup_def->name  = CORBA::string_dup(group_name.c_str());
         ftgroup_def->style = data.replication_style;
                 //ftgroup_def->entities ();
         ftgroup_def->minimum_nbr_replicas = data.minimum_nb_of_replicas;
         
                 application_obj->add_ftgroup (ftgroup_def.in()); 

             }
             else if ( CdmwPlatformMngt::LB_GROUP_TYPE == data.group_type )
             {
                 // Create the group definition
                 CdmwPlatformMngt::LBGroupDef_var lbgroup_def = new CdmwPlatformMngt::LBGroupDef();

         // Set the group definition
         lbgroup_def->name = CORBA::string_dup(group_name.c_str());
         lbgroup_def->style = data.balancing_style;
                 //ftgroup_def->entities ();
         lbgroup_def->minimum_nbr_replicas = data.minimum_nb_of_replicas;
         
                 application_obj->add_lbgroup (lbgroup_def.in()); 

             }

         
        }
        catch (...)
        {
            // nothing to do
        }    
    }
    // else nothing to do because we are the primary and we are inserting the element 
    // into the datastore
}

void RedundancyGroupDataStoreObserver::on_update(_Oid_in_type oid, _Data_in_type data)
  throw ()
{

}

void RedundancyGroupDataStoreObserver::on_remove(_Oid_in_type oid) 
  throw ()
{
    if (!Cdmw::PlatformMngt::Configuration::Is_primary())
    {
        CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                      "<<RedundancyGroupDataStoreObserver::on_remove NOT IMPLEMENTED>>");

        /*try
        {
            // get the application parent of process           
            std::string process_name;
            std::string host_name;
            std::string parent_path;
            
            // get the element name and parent path
            ManagedElementKey::Get_process_parent_path (oid, process_name, 
                                                        host_name, parent_path);
        
            // get the application object reference
            CdmwPlatformMngt::ManagedElement_var element_obj =
                        m_element_repository->get_element(parent_path.c_str());
                        
            CdmwPlatformMngt::ApplicationExt_var application_obj =
                        CdmwPlatformMngt::ApplicationExt::_narrow (element_obj.in());
            
    
            application_obj->on_remove_process (process_name.c_str(), host_name.c_str());
        }
        catch (...)
        {
            // nothing to do
        }*/
    }
    // else nothing to do because we are the primary and we are inserting the element 
    // into the datastore
}

     

} // End namespace PlatformMngt
} // End namespace Cdmw

