/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <cstdlib>
#include <string>
#include <sstream>

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/String.hpp>
#include <Foundation/common/Exception.hpp>
#include <Foundation/commonsvcs/initutils/InitUtils.hpp>

#include <Foundation/logging/LogManager.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>


#include <Foundation/idllib/FTPullMonitorable.skel.hpp>


#include <SystemMngt/platformlibrary/Configuration.hpp>
#include <SystemMngt/platformlibrary/BoundSyncCall.hpp>
#include <SystemMngt/platformlibrary/Iterator.hpp>
#include <SystemMngt/platformlibrary/RWElementRepository_impl.hpp>


#include "Foundation/idllib/FTPullMonitorable.skel.hpp"


#include "SystemMngt/platformlibrary/Configuration.hpp"
#include "SystemMngt/platformlibrary/SupervisionEventNotifier.hpp"
#include "SystemMngt/platformlibrary/BoundSyncCall.hpp"
#include "SystemMngt/platformlibrary/Iterator.hpp"
#include "SystemMngt/platformlibrary/RWElementRepository_impl.hpp"
#include "SystemMngt/platformlibrary/EventNotifierManager.hpp"
#include "SystemMngt/platformlibrary/CommandEventNotifier.hpp"
#include "SystemMngt/platformlibrary/PropertyEventNotifier.hpp"
#include "SystemMngt/platformlibrary/StatusEventNotifier.hpp"
#include "SystemMngt/platformapplicationlibrary/Application_impl.hpp"

#include "platformsystemlibrary/SystemLogMngr.hpp"
#include "platformsystemlibrary/SystemLogMessageId.hpp"
#include "platformsystemlibrary/CentralisedSupervisionSystem_impl.hpp"

#include "SystemMngt/platformvaluetypes/SystemStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/SystemModeChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/HostStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationAgentDeath_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationModeChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/EntityStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/GraphElement_impl.hpp"
#include "SystemMngt/platformvaluetypes/ProcessDef_impl.hpp"
#include "SystemMngt/platformvaluetypes/UserEvent_impl.hpp"
#include "SystemMngt/platformvaluetypes/PropertyChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ConfigurationChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ComponentStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/RedundancyGroupStatusChange_impl.hpp"
#include <SystemMngt/platformvaluetypes/CdmwCommand_impl.hpp>
#include <SystemMngt/platformvaluetypes/Restart_impl.hpp>
#include <SystemMngt/platformvaluetypes/Acknowledge_impl.hpp>
#include <SystemMngt/platformvaluetypes/Stop_impl.hpp>
#include <SystemMngt/platformvaluetypes/Reboot_impl.hpp>
#include <SystemMngt/platformvaluetypes/Shutdown_impl.hpp>


#include "SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp"
#include "SystemMngt/idllib/CdmwFTSystMngtSystem.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtPullMonitorableRegistration.stub.hpp"

#include "SystemMngt/platformlibrary/ClientIDGenerator.hpp"
#include "FaultTolerance/ftinit/FTServiceInit.hpp"
#include "FaultTolerance/ftstatemanager/DataStoreBase.hpp"

#include "platformftsystem/CdmwInterface.hpp"
#include "platformftsystem/ApplicationDataStoreObserver.hpp"
#include "platformftsystem/EntityDataStoreObserver.hpp"
#include "platformftsystem/HostDataStoreObserver.hpp"
#include "platformftsystem/ProcessDataStoreObserver.hpp"
#include "platformftsystem/RunSetDataStoreObserver.hpp"
#include "platformftsystem/ServiceDataStoreObserver.hpp"
#include "platformftsystem/SystemDataStoreObserver.hpp"
#include "platformftsystem/SystemObsvDataStoreObserver.hpp"
#include "platformftsystem/SystemDataStoreOnInit.hpp"
#include "platformftsystem/PeriodicNotifierDataStoreObserver.hpp"
#include "platformftsystem/RedundancyGroupDataStoreObserver.hpp"
#include "platformftsystem/ComponentDataStoreObserver.hpp"
#include "platformftsystem/PrimaryActivationHandler_impl.hpp"
#include "platformftsystem/ConfigurationSup.hpp"


#include "SystemMngt/platformftaccessorlibrary/SystemAccessor_impl.hpp"


#include "SystemMngt/platformapplicationlibrary/ConsistencyStatusSolver.hpp"
#include "SystemMngt/platformapplicationlibrary/ApplicationConsistencyStatusSolver.hpp"
#include "platformsystemlibrary/SystemConsistencyStatusSolver.hpp"

#include "SystemMngt/plugin/PluginManager.hpp"
#include "platformapplicationlibrary/DeploymentManager.hpp"

#include "platformadminlibrary/ConfigElementParser.hpp"
#include <xercesc/sax/SAXParseException.hpp>
#include "platformadminlibrary/ConfigElementParser.hpp"
#include "platformadminlibrary/System.hpp"
#include "platformadminlibrary/Application.hpp"
#include "platformadminlibrary/Process.hpp"
#include "platformadminlibrary/CcmComponent.hpp"
#include "platformadminlibrary/FTGroup.hpp"
#include "platformadminlibrary/LBGroup.hpp"
#include "platformadminlibrary/Host.hpp"
#include "platformadminlibrary/Entity.hpp"
#include "SystemMngt/tools/AdminLogMessageId.hpp"

#include "SystemMngt/tools/TimeoutInserter.hpp"


namespace Cdmw
{
namespace PlatformMngt
{

/**
*Purpose: SystemMngtPullMonitorable implementation
*<p>
*
*
*/
class SystemMngtPullMonitorable_impl : virtual public POA_FT::PullMonitorable,
                                    virtual public PortableServer::RefCountServantBase
{

    public:

        /**
        * Purpose:
        * <p> Constructor
        *
        */
        SystemMngtPullMonitorable_impl()
                throw()
        {
        }

        /**
        * Purpose:
        * <p> Destructor
        *
        */
        virtual
        ~SystemMngtPullMonitorable_impl()
                throw()
        {
        }

        /**
        * Purpose:
        * <p> to call to check if object is still living
        *
        *@exception CORBA::SystemException
        *
        */
        virtual CORBA::Boolean is_alive ()
                throw(CORBA::SystemException)
        {
             return true;
        }

}; // End class SystemMngtPullMonitorable_impl

} // End namespace PlatformMngt
} // End namespace Cdmw



namespace
{
    const std::string CENTRALISED_OPTION = "--centralised";
    const std::string DISTRIBUTED_OPTION = "--distributed";
    const std::string EVENT_TIMEOUT_OPTION = "--event-timeout";
    const std::string CREATION_TIMEOUT_OPTION = "--creation-timeout";
    const std::string CONNECTION_TIMEOUT_OPTION = "--connection-timeout";
    const std::string DAEMON_PORT_OPTION = "--DaemonPort";
    const std::string PULLMON_REGISTER_IOR_OPTION = "--PullMonRegisterIor";
    const std::string FAULT_MANAGER_CORBALOC = "--FaultManagerRegistration";
    const std::string HOST_STATUS_CONSISTENCY_OPTION = "--hostStatusCheckPeriod";
    const std::string ENABLE_LOG_DELEGATE_OPTION = "--enableLogDelegate";
    
}; // End anonymous namespace


enum ExitType
{
    ORB_DESTROY,                // the ORB will be destroyed
    ORB_SHUTDOWN_AND_DESTROY    // the ORB will be shutdown and destroyed
};


const int POA_THREAD_POOL_SIZE = 200;   


void print_syntax(char* program_name)
{
    std::cout << "Syntax :\n";
    std::cout << program_name 
              << Cdmw::Common::Options::PROCESS_XML_FILE_OPTION << "=XMLConfigFile\n";
    std::cout << "\n";
    std::cout << "Options:\n";
    std::cout << " [" << Cdmw::Common::Options::LOCALISATION_SERVICE_OPTION << "=PORT]";
    std::cout << " [" << CENTRALISED_OPTION << "]";
    std::cout << " [" << EVENT_TIMEOUT_OPTION << "=time_out]";
    std::cout << " [" << CREATION_TIMEOUT_OPTION << "=time_out]";
    std::cout << " [" << CONNECTION_TIMEOUT_OPTION << "=time_out]";
    std::cout << " [" << Cdmw::Common::Options::LOG_FILE_OPTION << "=full_path_to_log_file]";
    std::cout << " [" << Cdmw::Common::Options::PROCESS_NAME_OPTION << "=process_name]";
    std::cout << " [" << PULLMON_REGISTER_IOR_OPTION << "=pullmonitor_registration_callback_ior]";
    std::cout << " [" << DAEMON_PORT_OPTION << "=daemon_port]";
    std::cout << " [" << FAULT_MANAGER_CORBALOC << "=fault_manager_corbaloc]";
    std::cout << " [" << HOST_STATUS_CONSISTENCY_OPTION << "=ms_period]";
    std::cout << " [" << ENABLE_LOG_DELEGATE_OPTION << "]";
    std::cout << std::endl;
}


void exceptionHandler(void)
{
    std::cout << "Platform Supervision : UNEXPECTED EXCEPTION HANDLER" << std::endl;
}

class ExitHandler : public Cdmw::OsSupport::SignalHandler
{
public:
    ExitHandler(CORBA::ORB_ptr orb)
        : m_orb(CORBA::ORB::_duplicate(orb))
    {
    }
    
    virtual ~ExitHandler() 
    {
    }
    
    int handle_signal(int sig) throw()
    {
        Cdmw::PlatformMngt::LogMngr::logMessage(
            Cdmw::PlatformMngt::WRN, 
            Cdmw::PlatformMngt::MSG_ID_SIGNAL_TRAPPED, sig);
        if (!CORBA::is_nil(m_orb.in())) {
            try {
                m_orb->shutdown(false);
            } catch (const CORBA::SystemException& e) {
                Cdmw::PlatformMngt::LogMngr::logMessage(
                    Cdmw::PlatformMngt::WRN, 
                    Cdmw::PlatformMngt::MSG_ID_ORB_SUTDOWN_ERROR, 
                    Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            }
        }

// ---------------------------------------------------------------------------------------------
// Workaround in order to fix the PCR-1418: commented in order to exit soon after orb->shutdown;
// 13-Nov-07 : mr
        //Cdmw::PlatformMngt::LogMngr::logMessage(
        //    Cdmw::PlatformMngt::WRN, 
        //    Cdmw::PlatformMngt::MSG_ID_END_OF_HANDLER);
// ---------------------------------------------------------------------------------------------
        return 0;
    };
    
private :
    CORBA::ORB_var  m_orb;
};




int main(int argc, char* argv[])
{
    using namespace Cdmw::PlatformMngt;
    using namespace Cdmw::OsSupport;
    using namespace Cdmw::OrbSupport;
    using namespace Cdmw::Common;
    
    // the return code 
    int ret_code = 0;
    ExitType exitType = ORB_SHUTDOWN_AND_DESTROY;
    
    // initialise log manager
    // Init the Logging library
    SystemLogMngr::initialise(argc, argv);

    // checks the arguments
    long specificPort = -1;
    std::string specificPortStr;
    long daemonPort = -1;
    bool centralised = true;
    std::string logFile;
    unsigned long eventTimeout;
    unsigned long creationTimeout;
    std::string faultManagerCorbaloc = "";
    std::string supervisionProcessName = "";
    std::string pullMonitorRegistrationIOR = "";
//    std::string nodeManagerPortName = "";

#   ifndef _MSC_VER    

    std::set_unexpected(&exceptionHandler);

    // VC++ contrary to the clause 18.6 of the C++ standard, 
    //  set_unexpected, is not inside the std:: namespace
#   else

    set_unexpected(&exceptionHandler);

#   endif 


    /*
    // handle interupt
    signal(SIGINT, sighandler);
    */


    //
    // help argument
    //
    if (argc > 1)
    {
        if (strcmp(argv[1], "--help") == 0)
       {
            print_syntax (argv[0]);
            return ret_code;
        }
    }

    if (argc < 3)
    {
       print_syntax (argv[0]);
       return ret_code;
    }

    // checks the platform management supervision port option
    std::string port_option =
        OS::get_option_value(argc, argv, Options::LOCALISATION_SERVICE_OPTION);

    if (port_option != "no")
    {
        if (port_option != "yes")
        {
            specificPort = ::atoi(port_option.c_str());

            if (specificPort <= 0)
            {
               LogMngr::logMessage (FTL, MSG_ID_SUPERVISION_INVALID_PORT, port_option);
               ::exit(EXIT_FAILURE);
            }
        }
        else
        {
            LogMngr::logMessage (FTL, MSG_ID_SUPERVISION_PORT_NOT_SPECIFIED);
            print_syntax(argv[0]);
            ::exit(EXIT_FAILURE);
        }
    }
    
    // get default port for supervision system
    else
    {
        specificPort = CdmwPlatformMngt::DEFAULT_SUPERVISION_PORT;
    }
    
    // checks the centralised/distributed supervision option
    std::string centralised_option =
        OS::get_option_value(argc, argv, CENTRALISED_OPTION);

    std::string distributed_option =
        OS::get_option_value(argc, argv, DISTRIBUTED_OPTION);

    if (centralised_option == "yes" && distributed_option == "yes")
    {
        LogMngr::logMessage (FTL, MSG_ID_SUPERVISION_INVALID_OPTIONS);
        print_syntax(argv[0]);
        ::exit(EXIT_FAILURE);
    }
        
    if (distributed_option == "yes")
    {
        // !! distributed option is not compatible with FT supervision !!
        // centralised = false;
        LogMngr::logMessage (FTL, MSG_ID_SUPERVISION_INVALID_OPTIONS);
        print_syntax(argv[0]);
        ::exit(EXIT_FAILURE);
    }
        
        
    // checks the event notifier timeout option
    std::string event_timeout_option =
        OS::get_option_value(argc, argv, EVENT_TIMEOUT_OPTION);

    if (event_timeout_option != "no")
    {
        if (event_timeout_option != "yes")
        {
            eventTimeout = ::atoi(event_timeout_option.c_str());
            if (eventTimeout <= 0)
            {
               LogMngr::logMessage (FTL, MSG_ID_SUPERVISION_INVALID_EVNT_TIMEOUT, 
                                    event_timeout_option);
               ::exit(EXIT_FAILURE);
            }
        }
        else
        {
            LogMngr::logMessage (FTL, MSG_ID_SUPERVISION_EVNT_TIMEOUT_NOT_SPECIFIED);
            print_syntax(argv[0]);
            ::exit(EXIT_FAILURE);
        }
    }
    
    // set default event notifier timeout in msec
    else
    {
        eventTimeout = 5000;
    }

    // checks the application, agent, process creation timeout option
    std::string creation_timeout_option =
        OS::get_option_value(argc, argv, CREATION_TIMEOUT_OPTION);

    if (creation_timeout_option != "no")
    {
        if (creation_timeout_option != "yes")
        {
            creationTimeout = ::atoi(creation_timeout_option.c_str());

            if (creationTimeout <= 0)
            {
               LogMngr::logMessage (FTL, 
                                    MSG_ID_SUPERVISION_INVALID_CREAT_TIMEOUT,
                                    creation_timeout_option);
               ::exit(EXIT_FAILURE);
            }
        }
        else
        {
            LogMngr::logMessage(FTL, 
                                MSG_ID_SUPERVISION_CREAT_TIMEOUT_NOT_SPECIFIED);
            print_syntax(argv[0]);
            ::exit(EXIT_FAILURE);
        }
    }
    
    // set default creation timeout in msec
    else
    {
        creationTimeout = 5000;
    }

    // checks the log file option
    std::string logFile_option =
        OS::get_option_value( argc, argv, Options::LOG_FILE_OPTION);

    if (logFile_option != "no")
    {
        if (logFile_option != "yes")
        {
            logFile = logFile_option; 
        }
        else
        {
            LogMngr::logMessage(FTL, MSG_ID_SUPERVISION_LOG_FILE_NOT_SPECIFIED);
            print_syntax(argv[0]);
            ::exit(EXIT_FAILURE);
        }
    }

    // checks the daemon port option (port of the system)
    std::string daemon_port_option =
        OS::get_option_value(argc, argv, DAEMON_PORT_OPTION);

    if (daemon_port_option != "no")
    {
        if (daemon_port_option != "yes")
        {
            daemonPort = ::atoi(daemon_port_option.c_str());

            if (daemonPort <= 0)
            {
                std::string msg = "Daemon port option invalid ";
                msg += daemon_port_option;
                LogMngr::logMessage (FTL, msg.c_str());
                ::exit(EXIT_FAILURE);
            }
        }
        else
        {
            LogMngr::logMessage (FTL, "Daemon port option not specified");
            print_syntax(argv[0]);
            ::exit(EXIT_FAILURE);
        }
    }
    
    // get daemon default port
    else
    {
        daemonPort = CdmwPlatformMngt::DEFAULT_SYSTEM_PORT;
    }
    
    // store the daemon port
    Cdmw::PlatformMngt::Configuration::Set_daemon_port(daemonPort);

    // checks the FaultManager corbaloc
    std::string faultManagerCorbaloc_option =
        OS::get_option_value( argc, argv, FAULT_MANAGER_CORBALOC);
        
    if (faultManagerCorbaloc_option!= "no")
    {
        if (faultManagerCorbaloc_option != "yes")
        {
            faultManagerCorbaloc = faultManagerCorbaloc_option;           
        }
        else
        {
            LogMngr::logMessage (FTL, "Fault manager corbaloc not specified");
            print_syntax(argv[0]);
            ::exit(EXIT_FAILURE);
        }
    }

    // checks the application, agent, process creation timeout option
    std::string connection_timeout_option =
        OS::get_option_value(argc, argv, CONNECTION_TIMEOUT_OPTION);

    if (connection_timeout_option != "no")
    {
       if (connection_timeout_option != "yes")
       {
          // connection_timeout_option is in msec
          int connectionTimeout = ::atoi(connection_timeout_option.c_str());

          if (connectionTimeout <= 0)
          {
             LogMngr::logMessage (FTL, 
                                  MSG_ID_SUPERVISION_INVALID_CONNECT_TIMEOUT,
                                  connection_timeout_option);
             ::exit(EXIT_FAILURE);
          }
          LogMngr::logMessage(INF, 
                              MSG_ID_SUPERVISION_CONNECT_TIMEOUT_SPECIFIED, 
                              connection_timeout_option);

          SystemProperties::Set_Property(CDMW_ORB_CONNECTION_TIMEOUT_PROPERTY_NAME,
                                         connection_timeout_option);
       }
       else
       {
          LogMngr::logMessage(FTL, 
                              MSG_ID_SUPERVISION_CONNECT_TIMEOUT_NOT_SPECIFIED);
          print_syntax(argv[0]);
          ::exit(EXIT_FAILURE);
       }
    }
    // set default connection timeout
    else
    {
        connection_timeout_option = "500";        

        // Allow specification of a value for the ORB connection timeout from
        // command-line arguments or a property file
        std::string dummy_prop = 
           SystemProperties::Get_Property(CDMW_ORB_CONNECTION_TIMEOUT_PROPERTY_NAME);
        if (dummy_prop.empty())
           SystemProperties::Set_Property(CDMW_ORB_CONNECTION_TIMEOUT_PROPERTY_NAME,
                                          connection_timeout_option);
    }

    
    // checks the supervision process name option
    std::string processName_option =
        OS::get_option_value( argc, argv, Cdmw::Common::Options::PROCESS_NAME_OPTION);

    supervisionProcessName = "SMG_SUPERVISION_PROCESS";
    
    if (processName_option != "no")
    {
        if (processName_option != "yes")
        {
            supervisionProcessName = processName_option; 
        }
        else
        {
            LogMngr::logMessage (FTL, 
                                 MSG_ID_SUPERVISION_PROCESS_NAME_NOT_SPECIFIED);
            print_syntax(argv[0]);
            ::exit(EXIT_FAILURE);
        }
    }  
    
    // checks the pullmonitoring register callback ior option
    std::string callbackIor_option =
        OS::get_option_value( argc, argv, PULLMON_REGISTER_IOR_OPTION);

    if (callbackIor_option != "no")
    {
        if (callbackIor_option != "yes")
        {
            pullMonitorRegistrationIOR = callbackIor_option; 
        }
        else
        {
            LogMngr::logMessage (FTL, MSG_ID_SUPERVISION_CALLBACK_IOR_NOT_SPECIFIED);
            print_syntax(argv[0]);
            ::exit(EXIT_FAILURE);
        }
    }
    
    // init configuration static members
    Configuration::Set_timeout(creationTimeout);
    Configuration::Set_fault_tolerant();

    // Initialise FT service
    Cdmw::FT::FTServiceInit::Init(argc, argv);

    // checks host status consistency compute period 
    std::string host_status_consistency_period =
        OS::get_option_value(argc, argv, HOST_STATUS_CONSISTENCY_OPTION);

    long host_status_period = 1500;
    
    if (host_status_consistency_period != "no")
    {
       if (host_status_consistency_period != "yes")
       {

          // host_status_consistency_period is in msec
          long probe_host_status_period = ::atol(host_status_consistency_period.c_str());

          if (probe_host_status_period > 0)
          {
             host_status_period = probe_host_status_period;
          }
       }

    }

 
    CORBA::ORB_var orb = CORBA::ORB::_nil();
    try
    {
        // initialises the ORB
        StrategyList strategyList;
        strategyList.add_OrbThreaded();
        //strategyList.add_PoaThreadPerRequest();
        strategyList.add_PoaThreadPool(POA_THREAD_POOL_SIZE);
        strategyList.add_multicast();
        
        // set the port to string
        std::ostringstream buffer;
        buffer << specificPort;
        specificPortStr = buffer.str();

        // set orb arguments by adding service port option if undefined
        char** argv__ = new char*[argc + 2];

        for (int i = 0; i < argc; ++i) 
        {
            argv__[i] = argv[i];
        }
        
        argv__[argc] = 0;
        argv__[argc + 1] = 0;
        
        int argc__ = argc;
        int argc_x = argc;
        
        // if port option indefined in main arguments
        if (port_option == "no") 
        {
            std::string service_port_option = Options::LOCALISATION_SERVICE_OPTION;
            service_port_option += "=" + specificPortStr;
            argv__[argc__] = new char[service_port_option.size() + 1];
            ::strcpy(argv__[argc__], service_port_option.c_str());
            argc__++;
        }    
               
        // init ORB
        orb = OrbSupport::OrbSupport::ORB_init(argc__, argv__, strategyList);

        // init configuration static members      
        Configuration::Set_orb(orb.in());
             
        // delete added arguments if exist
        for (int i=argc_x; i < argc__; ++i)
        {
            delete[] argv__[i];
        }
        delete[] argv__;
        
       // redirects the log to the PlatformDaemon log collector 
       std::string  enable_log_delegate_option =
        OS::get_option_value(argc, argv, ENABLE_LOG_DELEGATE_OPTION);

    if (enable_log_delegate_option == "yes")
    {
         Cdmw::CommonSvcs::InitUtils* initUtils =
                Cdmw::CommonSvcs::InitUtils::Instance();
         try {
                initUtils->activate_LogCollector(orb, daemonPort);
            }
            catch (const Cdmw::Exception& ex) {
              LogMngr::logMessage (WRN, "Trace logs will not be sent to Trace Collector.");
            }

    }     
    
       LogMngr::logMessage(INF, MSG_ID_HOST_STATUS_CHK_PERIOD, host_status_period);
   
        // get root poa
        CORBA::Object_var poaObj = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var rootPOA = 
           PortableServer::POA::_narrow(poaObj.in());
       
        // activate the POA manager
        PortableServer::POAManager_var manager = rootPOA->the_POAManager();
        manager->activate();

        // init Cdmw
        Cdmw::PlatformMngt::CDMW_init(orb.in(), argc, argv, 
                                      supervisionProcessName.c_str(),
                                      faultManagerCorbaloc.c_str());
        
        
        // init iterator manager with root poa
        IteratorManager::initialize(rootPOA.in());
 
        // init BoundSyncCall factory
        BoundSyncCallFactory::initialise(orb.in());
        
        //PCR-770: BoundSyncCall substituted for SupervisionEventNotifier, StatusEventNotifier
        TimeoutInserter* timeOutInserter = TimeoutInserter::Instance();

        timeOutInserter->attachOrb(orb.in());

        // register event factories (value type factory)
        CORBA::ValueFactoryBase_var factory, old_factory;

        factory = new PropertyChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/PropertyChange:1.0",
                factory.in() );

        factory = new ConfigurationChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/ConfigurationChange:1.0",
                factory.in() );

        factory = new SystemModeChangeFactory;
        old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/SystemModeChange:1.0",
            factory.in() );

        factory = new SystemStatusChangeFactory;
        old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/SystemStatusChange:1.0",
            factory.in() );

        factory = new HostStatusChangeFactory;
        old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/HostStatusChange:1.0",
            factory.in() );

        factory = new ApplicationAgentDeathFactory;
        old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationAgentDeath:1.0",
            factory.in() );

        factory = new ApplicationModeChangeFactory;
        old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationModeChange:1.0",
            factory.in() );

        factory = new ApplicationStatusChangeFactory;
        old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationStatusChange:1.0",
            factory.in() );

        factory = new ProcessMessageFactory;
        old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessMessage:1.0",
            factory.in() );

        factory = new ProcessStatusChangeFactory;
        old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessStatusChange:1.0",
            factory.in() );

        factory = new EntityStatusChangeFactory;
        old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/EntityStatusChange:1.0",
            factory.in() );

        factory = new GraphElementFactory;
        old_factory = orb->register_value_factory (
            "IDL:thalesgroup.com/CdmwPlatformMngt/GraphElement:1.0",
            factory.in() );
                
        factory = new ProcessDefFactory;
        old_factory = orb->register_value_factory (
            "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDef:1.0",
            factory.in() );
                
        factory = new ManagedProcessDefFactory;
        old_factory = orb->register_value_factory (
                "IDL:thalesgroup.com/CdmwPlatformMngt/ManagedProcessDef:1.0",
                factory.in() );

        factory = new UserEventFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/UserEvent:1.0",
                factory.in() );

        factory = new ComponentStatusChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/ComponentStatusChange:1.0",
                factory.in() );

        factory = new RestartFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/Restart:1.0",
                factory.in() );
        factory = new RebootFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/Reboot:1.0",
                factory.in() );
        factory = new ShutdownFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/Shutdown:1.0",
                factory.in() );
        factory = new StopFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/Stop:1.0",
                factory.in() ); 
        factory = new AcknowledgeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/Acknowledge:1.0",
                factory.in() );

        factory = new RedundancyGroupStatusChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/RedundancyGroupStatusChange:1.0",
                factory.in() );
            
        // and start it
        EventNotifierManager::Instance()->set_sup_notifier(new SupervisionEventNotifier (eventTimeout));

        SupervisionEventNotifier* eventNotifier = EventNotifierManager::Instance()->get_sup_notifier();
        
        // create the supervision property change event notifier
        LogMngr::logMessage (INF, "create the supervision property change event notifier");

        // start it
        EventNotifierManager::Instance() -> set_property_notifier(new PropertyEventNotifier (eventTimeout,orb.in()));
        // and start it
        EventNotifierManager::Instance()->set_status_notifier(new StatusEventNotifier (eventTimeout,orb.in()));
        // start it
        EventNotifierManager::Instance() -> set_command_notifier(new CommandEventNotifier (eventTimeout,orb.in()));
        
        // create element repository servant
        PortableServer::ServantBase_var elementRepositoryServant;
        CdmwPlatformMngt::RWElementRepository_var elementRepositoryObject;
        
        RWElementRepository_impl* p_elementRepository =
               new RWElementRepository_impl();
               
        // create an object var to take pointer ownership
        elementRepositoryServant = p_elementRepository;
        
        // activate object
        elementRepositoryObject = p_elementRepository->_this();

        // create system states
        System_impl::createStates();
            
        // create system servant
        PortableServer::ServantBase_var systemServant;
        CdmwPlatformMngt::System_var systemObject;

        CentralisedSupervisionSystem_impl* p_system = 0;

        // create application states
        Application_impl::createStates();
        
        // create the centralised system servant
        p_system = 
                new CentralisedSupervisionSystem_impl(eventNotifier,
                                                      orb.in(), 
                                                      rootPOA.in(), 
                                                      elementRepositoryObject.in());

        p_system->set_host_status_check_period(host_status_period);

        LogMngr::logMessage (INF, " create the centralised system servant");

        // initialise the  DeploymentManager
        DeploymentManager::get_instance()->init(orb.in());

        std::stringstream daemonPortStr;
        daemonPortStr << daemonPort;

        DeploymentManager::get_instance()->set_node_manager_port_name (daemonPortStr.str().c_str());

        // create an object var to take pointer ownership
        systemServant = p_system;

        // activate object
        systemObject = p_system->_this();

        // publishes the supervision system corbaloc
        OrbSupport::OrbSupport::bind_object_to_corbaloc (
                     orb.in(), CdmwPlatformMngt::SUPERVISION_NAME, systemObject.in());
                     
        // Plugin initialization
        std::string plugin_conf_filename = OS::get_option_value( argc, argv, Options::PLUGIN_CONFIGURATION );
        PluginManager* plugin_manager = NULL;
        if (plugin_conf_filename != "no")
        {
            if (plugin_conf_filename != "yes")
            {
                plugin_manager = new PluginManager( systemObject.in() );  
                if (!(plugin_manager->set_up_all( plugin_conf_filename ))) {
                    std::vector<std::string> errors = plugin_manager->get_error_descriptions();
                    if (errors.size() > 0) {
                        std::vector<std::string>::iterator i;
                        for (i = errors.begin(); i != errors.end() ; i++) {
                            LogMngr::logMessage (WRN, (*i).c_str());
                        }
                    }
                    else {
                        LogMngr::logMessage (WRN, "There was a problem during plugin loading");
                    }
                }
                else {
                    LogMngr::logMessage (INF, "Plugin successfully loaded");
                }
            }
            else
            {
                LogMngr::logMessage (WRN, "Plugin option is set but no configuration file is specified!");
            }
        }
        else {
            LogMngr::logMessage (INF, "No plugin was specified");
        }
        
        
        /*********************************************
         * Periodic Notifier 
         * ******************************************/   
        CdmwPlatformMngt::Periodic::Notifier_var periodicNotifierObject;
        PortableServer::ServantBase_var periodicNotifierServant;
        Periodic::Notifier_impl* periodic_notifier;
 
        periodic_notifier = new Periodic::Notifier_impl(orb.in(), systemObject.in());
            
        // create an object var to take pointer ownership
        periodicNotifierServant = periodic_notifier;
        
        //active object
        periodicNotifierObject = periodic_notifier->_this();
        
        //set periodic notifier reference into system
        p_system->set_periodic_notifier(periodicNotifierObject.in());
        
        
        //
        // Register a datastore observer to create SystemMngt tree in backup process
        // 
        // 1) Entity DS
        EntityDataStoreObserver entity_ds_observer(elementRepositoryObject.in());
        CORBA::ULong ds_id = 
           Cdmw::PlatformMngt::Configuration::Get_entity_datastore_id();
        Cdmw::FT::DataStoreBase* dsBase = 
           Cdmw::FT::DataStoreBase::Get_dataStore(ds_id);
        EntityDataStore* entity_datastore = 
           dynamic_cast<EntityDataStore*>(dsBase);
        entity_datastore->register_observer(&entity_ds_observer);

        // 2) Application DS
        ApplicationDataStoreObserver application_ds_observer(p_system);
        ds_id = 
           Cdmw::PlatformMngt::Configuration::Get_application_datastore_id();
        dsBase = Cdmw::FT::DataStoreBase::Get_dataStore(ds_id);
        ApplicationDataStore* application_datastore = 
           dynamic_cast<ApplicationDataStore*>(dsBase);
        application_datastore->register_observer(&application_ds_observer);

        // 3) Host DS
        HostDataStoreObserver host_ds_observer(p_system);
        ds_id = 
           Cdmw::PlatformMngt::Configuration::Get_host_datastore_id();
        dsBase = Cdmw::FT::DataStoreBase::Get_dataStore(ds_id);
        HostDataStore* host_datastore = 
           dynamic_cast<HostDataStore*>(dsBase);
        host_datastore->register_observer(&host_ds_observer);

        // 4)  Process DS
        ProcessDataStoreObserver process_ds_observer(elementRepositoryObject.in());
        ds_id = 
           Cdmw::PlatformMngt::Configuration::Get_process_datastore_id();
        dsBase = Cdmw::FT::DataStoreBase::Get_dataStore(ds_id);
        ProcessDataStore* process_datastore = 
           dynamic_cast<ProcessDataStore*>(dsBase);
        process_datastore->register_observer(&process_ds_observer);

        // 5) RunSet DS
        RunSetDataStoreObserver runset_ds_observer(p_system);
        ds_id = 
           Cdmw::PlatformMngt::Configuration::Get_runset_datastore_id();
        dsBase = Cdmw::FT::DataStoreBase::Get_dataStore(ds_id);
        RunSetDataStore* runset_datastore = 
           dynamic_cast<RunSetDataStore*>(dsBase);
        runset_datastore->register_observer(&runset_ds_observer);

        // 6) Service DS
        ServiceDataStoreObserver service_ds_observer(elementRepositoryObject.in());
        ds_id = 
           Cdmw::PlatformMngt::Configuration::Get_service_defs_datastore_id();
        dsBase = Cdmw::FT::DataStoreBase::Get_dataStore(ds_id);
        ServiceDefsDataStore* service_datastore = 
           dynamic_cast<ServiceDefsDataStore*>(dsBase);
        service_datastore->register_observer(&service_ds_observer);

        // 7) System DS
        SystemDataStoreObserver system_ds_observer(p_system);
        ds_id = 
           Cdmw::PlatformMngt::Configuration::Get_system_datastore_id();
        dsBase = Cdmw::FT::DataStoreBase::Get_dataStore(ds_id);
        SystemDataStore* system_datastore = 
           dynamic_cast<SystemDataStore*>(dsBase);
        system_datastore->register_observer(&system_ds_observer);
        
        // 8) System Observer DS
        SystemObsvDataStoreObserver system_obsv_ds_observer(p_system);
        ds_id = 
           Cdmw::PlatformMngt::Configuration::Get_system_observer_registration_datastore_id();
        dsBase = Cdmw::FT::DataStoreBase::Get_dataStore(ds_id);
        SystemObserverRegistrationDataStore* system_obsv_datastore = 
           dynamic_cast<SystemObserverRegistrationDataStore*>(dsBase);
        system_obsv_datastore->register_observer(&system_obsv_ds_observer);
                    
        // 9) Periodic Notifier DS
        PeriodicNotifierDataStoreObserver periodic_obsv_ds_observer(periodic_notifier);
        ds_id = 
        Cdmw::PlatformMngt::Configuration::Get_periodic_observer_datastore_id();
        dsBase = Cdmw::FT::DataStoreBase::Get_dataStore(ds_id);
        PeriodicObserverDataStore* periodic_obsv_datastore = 
          dynamic_cast<PeriodicObserverDataStore*>(dsBase);
        periodic_obsv_datastore->register_observer(&periodic_obsv_ds_observer);
    
     // 10) RedundancyGroup DS
        RedundancyGroupDataStoreObserver redundancy_group_ds_observer(elementRepositoryObject.in());
        ds_id = 
           Cdmw::PlatformMngt::Configuration::Get_redundancy_group_datastore_id();
        dsBase = Cdmw::FT::DataStoreBase::Get_dataStore(ds_id);
        RedundancyGroupDataStore* redundancy_group_datastore = 
           dynamic_cast<RedundancyGroupDataStore*>(dsBase);
        redundancy_group_datastore->register_observer(&redundancy_group_ds_observer);
    
     // 11) Component DS
        ComponentDataStoreObserver component_ds_observer(elementRepositoryObject.in());
        ds_id = 
           Cdmw::PlatformMngt::Configuration::Get_component_datastore_id();
        dsBase = Cdmw::FT::DataStoreBase::Get_dataStore(ds_id);
        ComponentDataStore* component_datastore = 
           dynamic_cast<ComponentDataStore*>(dsBase);
        component_datastore->register_observer(&component_ds_observer);


    LogMngr::logMessage (INF, "created DS");

        //    
        // Create the system accessor
        //
                  
        // Create the system accessor servant
        SystemAccessor_impl* system_accessor = 
           new SystemAccessor_impl(orb.in(),
                                   rootPOA.in(), 
                                   p_system,
                                   elementRepositoryObject.in(),
                                   supervisionProcessName);   
        
        LogMngr::logMessage (INF, "created SystemAccessor");

        // create an object var to take pointer ownership
        PortableServer::ServantBase_var system_accessor_servant = 
           system_accessor;        
        
        // activate object
        CdmwFTSystMngt::SystemAccessor_var system_accessor_ref = 
           system_accessor->_this();
           

        //
        // Register a datastore initialiser observer for system datastore
        // 
        SystemDataStoreOnInit system_ds_oninit (system_accessor);
        SystemFTDataStore* system_ft_datastore = 
           dynamic_cast<SystemFTDataStore*>(system_datastore);
        system_ft_datastore->register_initialiser(&system_ds_oninit);

        LogMngr::logMessage (INF, "Registered Datastore");
            
        // Set System Accessor object in ft group
        ::FT::ObjectGroup_var system_accessor_group;

        // get replication manager
        CORBA::Object_var replication_mngr_obj = 
           orb->resolve_initial_references("ReplicationManager");
        CdmwFT::ReplicationManager_var replication_mngr =
           CdmwFT::ReplicationManager::_narrow(replication_mngr_obj.in());

                
        // get System Accessor Group Id
        try
        {
            ::FT::Name group_name;
            group_name.length(1);
            group_name[0].id =
               Cdmw::PlatformMngt::Configuration::Get_SystemAccessor_group_name(
                     ).c_str();
                
            system_accessor_group = 
               replication_mngr->get_object_group_ref_from_name(group_name);
        }
        catch(const ::FT::ObjectGroupNotFound&)
        {
            CDMW_THROW2 (Cdmw::BadParameterException, 
                         "SystemAccessor", 
                         "group undefined");
        }
                
        //
        // Register the primary activation handler
        //
        
        // resolve the Activation Manager        
        CORBA::Object_var activation_mngr_obj = orb->resolve_initial_references("FTActivationManager");

        CdmwFT::Location::ActivationManager_var activation_manager = 
                  CdmwFT::Location::ActivationManager::_narrow(activation_mngr_obj.in());
        
        PrimaryActivationHandler_impl* activation_handler =
                          new PrimaryActivationHandler_impl(orb.in(), 
                                                            system_accessor_ref.in(), 
                                                            p_system,
                                                            plugin_manager);
        //Set periodic notifier
        activation_handler->set_periodic_notifier(periodic_notifier);
        
        // register the handler into the Activation Manager
        LogMngr::logMessage (INF, "register the handler into the Activation Manager");

        ::CdmwFT::Location::HandlerId handler_id;
        
        handler_id = activation_manager->register_activation_handler(
                      system_accessor_group.in(), activation_handler);

        // set System Management location
        ::FT::Location smg_location;
        smg_location.length(2);
            
        smg_location[0].id = Cdmw::OsSupport::OS::get_hostname().c_str();
        smg_location[0].kind = "hostname";    
        smg_location[1].id = supervisionProcessName.c_str();
        smg_location[1].kind = "processname"; 

        // add the System Accessor object in group
        system_accessor_group =  replication_mngr->add_member(system_accessor_group.in(),
                                                                smg_location,
                                                                system_accessor_ref.in());
        
        // create the Pull Monitorable Object to be polled by daemon
        SystemMngtPullMonitorable_impl* pull_monitorable =
             new SystemMngtPullMonitorable_impl();

        // create an object var to take pointer ownership
        PortableServer::ServantBase_var pull_monitorable_servant = pull_monitorable;

        // create the CORBA reference
        FT::PullMonitorable_var pull_monitorable_ref = 
             pull_monitorable->_this();

        // register Pull Monitorable in daemon if callback is defined
        if (!pullMonitorRegistrationIOR.empty())
        {
            // gets the daemon callback for pull monitorable registration
            CORBA::Object_var callback_obj = orb->string_to_object(pullMonitorRegistrationIOR.c_str());

            CdmwPlatformMngt::PullMonitorableRegistration_var callback = 
                CdmwPlatformMngt::PullMonitorableRegistration::_narrow(callback_obj.in());

            if (CORBA::is_nil(callback.in()))
            {
                CDMW_THROW2(Cdmw::BadParameterException, "argv", "doesn't contain a valid callback IOR");
            }
            
            // register the pull monitorable 
            callback->register_pull_monitorable (pull_monitorable_ref.in());
        }

        // logs the startup info
        LogMngr::logMessage (INF, MSG_ID_SUPERVISION_STARTUP, specificPortStr);

        LogMngr::logMessage (INF, MSG_ID_SUPERVISION_CORBALOC, OS::get_hostname(),
                             specificPortStr, CdmwPlatformMngt::SUPERVISION_NAME);

        // Setup signal handling for ORB to be shut down
        ExitHandler * exitHandler = new ExitHandler(orb.in());
        Cdmw::OrbSupport::OrbSupport::install_orb_shutdown_signal_handlers(orb.in(), exitHandler);


        if(Cdmw::PlatformMngt::Configuration::Is_primary())
        {
            //ADD
            // checks the optional validate option
            
            try
            {
                
                std::string XML_file_option =
                    Cdmw::OsSupport::OS::get_option_value( argc, argv, Cdmw::Common::Options::SUPERVISION_CONF);
                if (XML_file_option != "yes" && XML_file_option != "no")
                {
                    Cdmw::PlatformMngt::ConfigurationSup opt;
                    opt.parse(XML_file_option, true);
                                               
                    //if we have the daemon controller port number else set to default
                    if(opt.getDaemon_controller_port_number() != "")
                        Cdmw::PlatformMngt::Configuration::Set_DaemonController_port_number(opt.getDaemon_controller_port_number());
                    
                    //if we have the define file
                    if(strcmp(opt.getSystem_definition_file(), ""))
                    {
                        Cdmw::PlatformAdmin::System admSystem( systemObject );

                        admSystem.define(opt.getSystem_definition_file());

                        //Check the restart on responding option
                        if(!strcmp(opt.getHost_auto_start(), "yes"))
                        {
                            Cdmw::PlatformMngt::Configuration::Set_restart_process_on_responding_option(true); 
                        }    
                        if(!strcmp(opt.getSystem_auto_start(), "yes"))
                        {
                            CdmwPlatformMngtBase::StartupKind_var startupKind
                                = new CdmwPlatformMngtBase::StartupKind();

                            startupKind->state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
                            startupKind->startup_mode = 0;
                            startupKind->specified_state_id = CORBA::string_dup( "" );

                            admSystem.start( startupKind.in() );

                        }
                    }
                }
            }
            catch( const Cdmw::PlatformAdmin::LoggedException& )
            {
            ::exit(EXIT_FAILURE);
            }
            catch( const SAXParseException& e )
            {
            ostringstream message;
                
            message << Cdmw::PlatformAdmin::to_string( e.getMessage() ) << endl;
            message << "in file: " << Cdmw::PlatformAdmin::to_string( e.getSystemId() ) << endl;
            message << "   line: " << e.getLineNumber() << endl;
            message << "   char: " << e.getColumnNumber() << endl;
        
            Cdmw::PlatformMngt::LogMngr::logMessage( ERR, message.str().c_str() );
            ::exit(EXIT_FAILURE);
            }
            catch( const Cdmw::Exception& e)
            {
            Cdmw::PlatformMngt::LogMngr::logMessage( ERR, e.what() );
            ::exit(EXIT_FAILURE);
            }
            catch( const CORBA::Exception& e )
            {
            Cdmw::PlatformMngt::LogMngr::logMessage( ERR, e._name() );
            ::exit(EXIT_FAILURE);
            }
            catch( ... )
            {
            Cdmw::PlatformMngt::LogMngr::logMessage( ERR, "Unknown exception" );
            ::exit(EXIT_FAILURE);
            }

        
        
        
        
        
        }
        

    //END ADD
    
    
    // runs the ORB loop
        orb->run();

//
// -----------------------------------------------------------------------------------------
// Workaround in order to fix the PCR-1418 because the operation "m_orb->shutdown(false);"
// performed in the handle_signal() of Exit_handler doesn't return.
// 9-Nov-07 : amx - cgi - gma     
// -----------------------------------------------------------------------------------------
//        Cdmw::OrbSupport::OrbSupport::uninstall_orb_shutdown_signal_handlers(orb.in());
// -----------------------------------------------------------------------------------------
     

        LogMngr::logMessage (INF, "Platform Supervision : stop in progess");

// -----------------------------------------------------------------------------------------
// Workaround in order to fix the PCR-1418: delete exitHandler; postponed after
// LogMngr::logMessage (INF, "Platform Supervision : stop in progess");
// in order to give time to the exitHandler to finish.
// Commented in ExitHanlder::handle_signal the last log in order to exit soon after orb->shutdown;
// 13-Nov-07 : mr
// -----------------------------------------------------------------------------------------
     
        delete exitHandler;
          
        exitType = ORB_DESTROY;
        delete plugin_manager;
   
    }
    catch (const CORBA::COMM_FAILURE& ex)
    {
        ret_code = EXIT_FAILURE;

        LogMngr::logMessage (FTL, MSG_ID_SUPERVISION_UNAVAILABLE_PORT,
                             specificPortStr);
    }    
    catch (const CORBA::Exception& e) {
        CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                   "!!!! CORBA::Exception: )"
                   << OrbSupport::OrbSupport::exception_to_string(e));
        ret_code = EXIT_FAILURE;
    }
    catch (const std::exception& e) {
        CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                   "!!!! std::exception: )" << e.what());
        ret_code = EXIT_FAILURE;
    }    
    catch (...)
    {
        CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                   "!!!! unknown exception. ");
        ret_code = EXIT_FAILURE;
    }
   
    
    try
    {
        if (!CORBA::is_nil(orb.in()))
        {
            // unregister the value factories
            #if CDMW_ORB_VDR != tao
            orb->unregister_value_factory (
                "IDL:thalesgroup.com/CdmwPlatformMngt/HostStatusChange:1.0");
            orb->unregister_value_factory (
                "IDL:thalesgroup.com/CdmwPlatformMngt/SystemStatusChange:1.0");
            orb->unregister_value_factory (
                "IDL:thalesgroup.com/CdmwPlatformMngt/SystemModeChange:1.0");
            orb->unregister_value_factory (
                "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationAgentDeath:1.0");
            orb->unregister_value_factory (
                "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationStatusChange:1.0");        
            orb->unregister_value_factory (
                "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationModeChange:1.0");                        
            orb->unregister_value_factory (
                "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessStatusChange:1.0");
            orb->unregister_value_factory (
                "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessMessage:1.0");
            orb->unregister_value_factory (
                "IDL:thalesgroup.com/CdmwPlatformMngt/SystemEntityStatusChange:1.0");
            orb->unregister_value_factory (
                "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationEntityStatusChange:1.0");
            orb->unregister_value_factory (
                "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessEntityStatusChange:1.0");               
            orb->unregister_value_factory (
                "IDL:thalesgroup.com/CdmwPlatformMngt/GraphElement:1.0");
            orb->unregister_value_factory (
                "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDef:1.0");                
            orb->unregister_value_factory (
                "IDL:thalesgroup.com/CdmwPlatformMngt/ManagedProcessDef:1.0");
            orb->unregister_value_factory (
                "IDL:thalesgroup.com/CdmwPlatformMngt/PropertyChange:1.0");
            orb->unregister_value_factory (
                "IDL:thalesgroup.com/CdmwPlatformMngt/ComponentStatusChange:1.0");
            orb->unregister_value_factory (
                "IDL:thalesgroup.com/CdmwPlatformMngt/RedundancyGroupStatusChange:1.0");
            orb->unregister_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/Restart:1.0");
            orb->unregister_value_factory(
                    "IDL:thalesgroup.com/CdmwPlatformMngt/Reboot:1.0");
            orb->unregister_value_factory(
                    "IDL:thalesgroup.com/CdmwPlatformMngt/Shutdown:1.0);
            orb->unregister_value_factory(
                    "IDL:thalesgroup.com/CdmwPlatformMngt/Stop:1.0");
            orb->unregister_value_factory(
                    "IDL:thalesgroup.com/CdmwPlatformMngt/Acknowledge:1.0");
            #endif
            
            
            if (exitType == ORB_SHUTDOWN_AND_DESTROY)
            {
                orb->shutdown(false);
            }
                               
            // stops the ORB
            OrbSupport::OrbSupport::ORB_cleanup(orb.in());
            orb->destroy();
        }
    }
    catch (...)
    {
    }

    // destroy states
    System_impl::destroyStates();
    
    if (centralised)
    {
        // destroy application states
        Application_impl::destroyStates();
    }

    // destroy factory
    BoundSyncCallFactory::destroy();
    
    LogMngr::logMessage (INF, "Platform Supervision : stopped and exited");
    
    // destroy log manager
    LogMngr::destroy();

    return ret_code;

}

