/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_PROCESS_CONTEXTIMPL_HPP 
#define INCL_PLATFORMMNGT_PROCESS_CONTEXTIMPL_HPP 

#include <SystemMngt/idllib/CdmwPlatformMngtManagedElementContexts.skel.hpp>

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include "platforminterface/ManagedElementContext_impl.hpp"

#include "Foundation/osthreads/Mutex.hpp"

//added
#include <SystemMngt/idllib/CdmwPlatformMngtProcessCallback.stub.hpp>

namespace Cdmw
{
namespace PlatformMngt
{

/**
* Purpose:
* <p>
* The default implementation of a ManagedElementContexts::ProcessContext.
*/
class ProcessContext_impl : 
    virtual public CdmwPlatformMngt::ManagedElementContexts::ProcessContext,
    virtual public Cdmw::OrbSupport::RefCountLocalObject,
    virtual public ManagedElementContext_impl
{

public:

    /**
     * Purpose:
     * <p> Constructor.
     */ 
    ProcessContext_impl(CORBA::ORB_ptr orb);


    /**
     * Purpose:
     * <p> Destructor.
     */ 
    virtual ~ProcessContext_impl() throw ();
    
    /**
    * The message broker that forwards messages and fatal error to the Platform
    * Management service.
    */
    static CdmwPlatformMngt::ProcessMessageBroker_var M_proc_mess_broker;

    /**
     * Purpose:
     * <p>
     * implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ManagedElementContexts/ManagedElementContext/managed_element_name:1.0
     * attribute
     */
    char* managed_element_name()
        throw(CORBA::SystemException);

    /**
     * Get ManagedElement Proxy.
     */
     CdmwPlatformMngt::ManagedElement_ptr the_managed_element() throw (CORBA::SystemException);

        
    /**
     * Purpose:
     * <p>
     * implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ManagedElementContexts/ProcessContext/exit:1.0
     * operation
     */
    void exit(CORBA::Long exit_code)
        throw(CORBA::SystemException);
    
        
    /**
     * Purpose:
     * <p>
     * implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ManagedElementContexts/ProcessContext/get_process_redundancy_state:1.0
     * operation
     */
    CdmwPlatformMngt::RedundancyEltState get_process_redundancy_state()
        throw(CORBA::SystemException);
    
    /**
     * Purpose: notifies a property change event to process message broker
     * 
     */
    void notify_property_change (
        CdmwPlatformMngt::PropertyChange * property_change_event);
    
    // get system name
    static std::string getSystemName();
    
        
protected:
        
    /**
     * Purpose:
     * <p>
     * returns the issuer name to be used in notifications.
     *
     * @return the issuer name
     */
    virtual std::string get_issuer_name();
    
    OsSupport::Mutex m_mutex_exit;

};

} // End namespace PlatformMngt
} // End namespace Cdmw

#endif // INCL_PLATFORMMNGT_PROCESS_CONTEXTIMPL_HPP

