/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_CLIENT_INTERCEPTOR_HPP 
#define INCL_CLIENT_INTERCEPTOR_HPP 

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/orbsupport/PortableInterceptor.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>

#include <Foundation/orbsupport/TypeTraits.hpp>
#include <Foundation/orbsupport/Codec.hpp>

#include <iostream>
#include <string>

namespace Cdmw
{
namespace PlatformMngt
{

    class ClientInterceptor : public PortableInterceptor::ClientRequestInterceptor,
                                  public Cdmw::OrbSupport::RefCountLocalObject
    {

    public:
        
        ClientInterceptor ();
        virtual ~ClientInterceptor ();
        
        typedef Cdmw::OrbSupport::Codec<std::string> IssuerCodec;

        //
        // IDL to C++ Mapping
        //
        virtual char*
        name() throw (CORBA::SystemException);

        virtual void
        destroy() throw (CORBA::SystemException);

        virtual void
        send_request (PortableInterceptor::ClientRequestInfo_ptr ri)
            throw (CORBA::SystemException, PortableInterceptor::ForwardRequest);

        virtual void
        send_poll (PortableInterceptor::ClientRequestInfo_ptr ri)
            throw (CORBA::SystemException);

        virtual void
        receive_reply (PortableInterceptor::ClientRequestInfo_ptr ri)
            throw (CORBA::SystemException);

        virtual void
        receive_exception(PortableInterceptor::ClientRequestInfo_ptr ri)
            throw (CORBA::SystemException, PortableInterceptor::ForwardRequest);

        virtual void
        receive_other(PortableInterceptor::ClientRequestInfo_ptr ri)
            throw (CORBA::SystemException, PortableInterceptor::ForwardRequest);

        void
          set_orb(CORBA::ORB_ptr orb);

        void
        add_issuer(std::string issuer);


      
    private:

        IssuerCodec* m_codec; 
        IOP::ServiceContext_var m_context_service;       
        CORBA::ORB_var m_orb_client;
        std::string m_issuer;

        ClientInterceptor(const ClientInterceptor&);
        void operator=(const ClientInterceptor&);

    }; // End class OnInit_impl


} // End namespace Cdmw
} // End namespace PlatformMngt

#endif // INCL_CLIENT_INTERCEPTORS_HPP

