/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_COMPONENT_CONTEXTIMPL_HPP 
#define INCL_PLATFORMMNGT_COMPONENT_CONTEXTIMPL_HPP 

#include "SystemMngt/idllib/CdmwPlatformMngtManagedElementContexts.skel.hpp"

#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/orbsupport/RefCountLocalObject.hpp"
#include "platforminterface/ManagedElementContext_impl.hpp"



namespace Cdmw
{
namespace PlatformMngt
{


/**
* Purpose:
* <p>
* The default implementation of a ManagedElementContexts::ComponentContext.
*/
class ComponentContext_impl : 
    virtual public CdmwPlatformMngt::ManagedElementContexts::ComponentContext,
    virtual public Cdmw::OrbSupport::RefCountLocalObject,
    virtual public MessageNotifier_impl
{

public:

    /**
     * Purpose:
     * <p> Constructor.
     */ 
    ComponentContext_impl(const char* component_name, 
            CdmwPlatformMngt::ManagedElementContexts::ProcessContext_ptr process_ctx);


    /**
     * Purpose:
     * <p> Destructor.
     */ 
    virtual ~ComponentContext_impl() throw ();
   
    /**
     * Get ManagedElement Proxy.
     */
    CdmwPlatformMngt::ManagedElement_ptr the_managed_element() throw (CORBA::SystemException);
 
    
    /**
     * Purpose:
     * <p>
     * implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ManagedElementContexts/ManagedElementContext/managed_element_name:1.0
     * attribute
     */
    char* managed_element_name()
        throw(CORBA::SystemException);
    
    
    CdmwPlatformMngt::External::System_ptr the_system() throw ( CORBA::SystemException );
    
    /**
     * Get Appllication Proxy.
     */      
    CdmwPlatformMngt::External::Application_ptr the_application() throw (CORBA::SystemException);
    
    /**
     * Get Process Proxy.
     */     
    CdmwPlatformMngt::External::Process_ptr the_process() throw (CORBA::SystemException);
    
    /**
     * Get Host Proxy.
     */  
    CdmwPlatformMngt::External::Host_ptr the_host() throw (CORBA::SystemException);
    
    
    /**
     * Get Redundancy Group Name.
     */
    CdmwPlatformMngt::External::RedundancyGroup_ptr the_redundancy_group() throw (CORBA::SystemException);
       
    /**
     * get Supervised Object name method
     */         
    char* system_name()throw (CORBA::SystemException);
    
    /**
     * get Supervised Object name method
     */         
    char* application_name() throw (CORBA::SystemException);
    
    /**
     * get Supervised Object name method
     */         
    char* process_name() throw (CORBA::SystemException);
    
    /**
     * get Supervised Object name method
     */         
    char* host_name() throw (CORBA::SystemException);
        
    /**
     * get Managed Element Name name method
     */               
    char* redundancy_group_name() throw (CORBA::SystemException);
    

protected:

    /**
     * Purpose:
     * <p>
     * returns the issuer name to be used in notifications.
     *
     * @return the issuer name
     */
    virtual std::string get_issuer_name();
    
    /*
     * Process Context instance
     */
    CdmwPlatformMngt::ManagedElementContexts::ProcessContext_var m_process_ctx;

private:

    /**
     * Component name
     */
    std::string m_component_name;
    
    /*
     * Ccm Component instance
     */
    CdmwPlatformMngt::CcmComponent_var  m_ccm_component_var;
};


} // End namespace PlatformMngt
} // End namespace Cdmw

#endif // INCL_PLATFORMMNGT_COMPONENT_CONTEXTIMPL_HPP

