/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_MANAGED_COMPONENT_CONTEXT_FACTORY_IMPL_HPP 
#define INCL_PLATFORMMNGT_MANAGED_COMPONENT_CONTEXT_FACTORY_IMPL_HPP 

#include "SystemMngt/idllib/CdmwPlatformMngtManagedComponentContextFactory.skel.hpp"

#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/orbsupport/RefCountLocalObject.hpp"
#include <map>



namespace Cdmw
{
namespace PlatformMngt
{


/**
* Purpose:
* <p>
* The default implementation of a ManagedComponentContextFactory.
*/
class ManagedComponentContextFactory_impl : 
    virtual public CdmwPlatformMngt::ManagedComponentContextFactory,
    virtual public Cdmw::OrbSupport::RefCountLocalObject
{

public:

    /**
     * Purpose:
     * <p> Constructor.
     */ 
    ManagedComponentContextFactory_impl();


    /**
     * Purpose:
     * <p> Destructor.
     */ 
    virtual ~ManagedComponentContextFactory_impl() throw ();
    

    /**
     * Purpose:
     * <p>
     * implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/
     *   ManagedComponentContextFactory/find_managed_component_context:1.0
     * operation
     */
    CdmwPlatformMngt::ManagedElementContexts::ComponentContextExt_ptr  
    find_managed_component_context(const char* component_name)
        throw (CORBA::SystemException, CdmwPlatformMngt::InvalidName);

    void set_managed_process_context (CdmwPlatformMngt::ManagedElementContexts::ProcessContext_ptr managed_process_context) throw (CORBA::SystemException, CdmwPlatformMngt::ContextAlreadyExists);
private:
    
    /** 
     * Map of created ManagedComponentContexts
     */
    typedef std::map<std::string, 
                     CdmwPlatformMngt::ManagedElementContexts::ComponentContextExt_var> 
         ComponentContextExtMap;
    
    ComponentContextExtMap m_component_contexts;
    CdmwPlatformMngt::ManagedElementContexts::ProcessContext_ptr m_process_context_ptr;
    
    
    
};


} // End namespace PlatformMngt
} // End namespace Cdmw

#endif // INCL_PLATFORMMNGT_MANAGED_COMPONENT_CONTEXT_FACTORY_IMPL_HPP

