/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_MANAGED_ELEMENT_CONTEXTIMPL_HPP
#define INCL_PLATFORMMNGT_MANAGED_ELEMENT_CONTEXTIMPL_HPP

#include "idllib/CdmwPlatformMngtManagedElementContexts.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtProcess.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"
#include "idllib/CdmwPlatformMngtProcess.stub.hpp"
#include "idllib/CdmwPlatformMngtCommon.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtProcessCallback.stub.hpp"
#include "Foundation/orbsupport/RefCountServantBase.hpp"
#include "Foundation/orbsupport/RefCountLocalObject.hpp"
#include "platforminterface/MessageNotifier_impl.hpp"

#include <iostream>

namespace Cdmw
{
  namespace PlatformMngt
  {

      struct ContextData 
      {
    std::string process_name;
    std::string system_name;
    std::string host_name;
    std::string application_name;
    CdmwPlatformMngt::ProcessMessageBroker_ptr process_message_broker;
      };

      /**
       * Purpose:
       * <p>
       * The abstract Managed Process Context
       */
      class ManagedElementContext_impl: virtual public CdmwPlatformMngt::ManagedElementContexts::ManagedElementContext,
                                        virtual public Cdmw::OrbSupport::RefCountLocalObject,
                            virtual public MessageNotifier_impl
      {
      
    public:


        /**
     * Constructor
     */
    ManagedElementContext_impl();

                  
        /**
         * Constructor 
         */
        ManagedElementContext_impl(CORBA::ORB_ptr orb);

        /**
         * Constructor
     */
    virtual ~ManagedElementContext_impl() throw();
                  
    /**
     * Get System Proxy.
     */ 
        CdmwPlatformMngt::External::System_ptr the_system() throw (CORBA::SystemException);

    /**
     * Get Appllication Proxy.
     */      
        CdmwPlatformMngt::External::Application_ptr the_application() throw (CORBA::SystemException);
    
    /**
     * Get Process Proxy.
     */     
        CdmwPlatformMngt::External::Process_ptr the_process() throw (CORBA::SystemException);
    
    /**
     * Get Host Proxy.
     */  
        CdmwPlatformMngt::External::Host_ptr the_host() throw (CORBA::SystemException);
    
    /**
     * Get Redundancy Group.
     */
    virtual CdmwPlatformMngt::External::RedundancyGroup_ptr the_redundancy_group() throw (CORBA::SystemException);
       
        /**
     * get Supervised Object name method
     */         
    char* system_name()throw (CORBA::SystemException);

    /**
     * get Supervised Object name method
     */         
    char* application_name() throw (CORBA::SystemException);

    /**
     * get Supervised Object name method
     */         
    char* process_name() throw (CORBA::SystemException);

    /**
     * get Supervised Object name method
     */         
    char* host_name() throw (CORBA::SystemException);
        
        /**
     * get Managed Element Name name method
     */               
    virtual char* redundancy_group_name() throw (CORBA::SystemException);

    /**
     * Set Supervised Object Name.
     *
     * @param name the Name of System.
     */ 
        void set_system_name(const char* system_name)throw (CORBA::SystemException);

    /**
     * Set Supervised Object Name.
     *
     * @param name the Name of Application.
     */ 
        void set_application_name(const char* application_name) throw (CORBA::SystemException);

    /**
     * Set Supervised Object Name.
     *
     * @param name the Name of Host.
     */ 
        void set_host_name(const char* host_name) throw (CORBA::SystemException);

    /**
     * Set Supervised Object Name.
     *
     * @param name the Name of Process.
     */
    void set_process_name (const char* process_name) throw (CORBA::SystemException);

    /**
     * Set Supervised Object Name.
     *
     * @param name the Name of Supervised Object.
     */ 
        void set_managed_element_name(const char* managed_element_name) throw (CORBA::SystemException);
   
    /**
     * Set Supervised Object Name.
     *
     * @param name the Name of Supervised Object.
     */ 
        void set_redundancy_group_name(const char* redundancy_group_name) throw (CORBA::SystemException);
     
    /**
     * Set Supervised Object Reference.
     *
     * @param name Object_ptr of Supervised Object.
     */
        void set_system(CORBA::Object_ptr system_ptr) throw (CORBA::SystemException);

    /**
     * Set Supervised Object Reference.
     *
     * @param name Object_ptr of Supervised Object.
     */
        void set_application(CORBA::Object_ptr application_ptr) throw (CORBA::SystemException);

    /**
     * Set Supervised Object Reference.
     *
     * @param name Object_ptr of Supervised Object.
     */
        void set_host(CORBA::Object_ptr host_ptr) throw (CORBA::SystemException);
    
    /**
     * Set Supervised Object Reference.
     *
     * @param name Object_ptr of Supervised Object.
     */ 
        void set_process(CORBA::Object_ptr process_ptr) throw (CORBA::SystemException);
        
        /**
     * Set Redundancy Group Object Reference.
     *
     * @param name Object_ptr of Redundancy Group.
     */ 
        void set_redundancy_group(CORBA::Object_ptr process_ptr) throw (CORBA::SystemException);

        /**
     * Set managedElement Object Reference.
     *
     * @param name Object_ptr of Managed Element
     */
    void set_managed_element ( CORBA::Object_ptr element_ptr ) throw ( CORBA::SystemException );
    
    
    /**
     * Set ManagedProcess Context Data.
     *
     * @param name ContextData of ManagedProcess.
     */
    void set_data(const ContextData & context_data){};


    /*
     * acknowledge method to parse ContextData
     */
    void ack(const ContextData& ctx_data)  throw ( CdmwPlatformMngtService::ServiceNotFound, CdmwPlatformMngtService::ServiceUnavailable, CORBA::SystemException );

    /**
     * recover Supervised Object proxies 
     */
    void recover_proxies()  throw ( CdmwPlatformMngtService::ServiceNotFound, CdmwPlatformMngtService::ServiceUnavailable, CORBA::SystemException );

       
      protected:
       
        ////////////////////////////////
        //  Supervised object proxies
        //  Orb and ProcessMessageBroker
        ////////////////////////////////
         
        CORBA::ORB_var                                m_orb;
        CdmwPlatformMngt::System_var                  m_system_var; 
        CdmwPlatformMngt::Application_var             m_application_var;
    CdmwPlatformMngt::Process_var                 m_process_var;
    CdmwPlatformMngt::Host_var                    m_host_var;
        CdmwPlatformMngt::ProcessMessageBroker_var    m_process_message_broker;    
    CdmwPlatformMngt::ManagedElement_var          m_managed_element_var;
        CdmwPlatformMngt::RedundancyGroup_var         m_redundancy_group_var;
        
    ////////////////////////////
    // Supervised object names
    ////////////////////////////
    
    CORBA::String_var m_system_name;
    CORBA::String_var m_application_name;
    CORBA::String_var m_host_name;
    CORBA::String_var m_process_name;
    CORBA::String_var m_element_name;
        CORBA::String_var m_redundancy_group_name;
    CORBA::String_var m_managed_element_name;
        
      };


  } // End namespace PlatformMngt
} // End namespace Cdmw

#endif // INCL_PLATFORMMNGT_MANAGED_ELEMENT_CONTEXTIMPL_HPP
