/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_MESSAGE_NOTIFIER_HPP 
#define INCL_PLATFORMMNGT_MESSAGE_NOTIFIER_HPP 

#include "SystemMngt/idllib/CdmwPlatformMngtManagedElementContexts.skel.hpp"

#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/orbsupport/RefCountLocalObject.hpp"

#include <string>



namespace Cdmw
{
namespace PlatformMngt
{


/**
* Purpose:
* <p>
* The default implementation of a ManagedElementContexts::MessageNotifier.
*/
class MessageNotifier_impl : 
    virtual public CdmwPlatformMngt::ManagedElementContexts::MessageNotifier,
    virtual public Cdmw::OrbSupport::RefCountLocalObject
{

public:

    /**
     * Purpose:
     * <p> Constructor.
     */ 
    MessageNotifier_impl();


    /**
     * Purpose:
     * <p> Destructor.
     */ 
    virtual ~MessageNotifier_impl() throw ();
    

    /**
     * Purpose:
     * <p>
     * implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ManagedElementContexts/MessageNotifier/notify_message:1.0
     * operation
     */
    void notify_message(CORBA::Long level, const char* message)
        throw(CORBA::SystemException);
    
        
    /**
     * Purpose:
     * <p>
     * implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ManagedElementContexts/MessageNotifier/notify_fatal_error:1.0
     * operation
     */
    void notify_fatal_error(const char* error_info)
        throw(CORBA::SystemException);
    
        
    /**
     * Purpose:
     * <p>
     * implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ManagedElementContexts/MessageNotifier/notify_user_event:1.0
     * operation
     */
    void notify_user_event(const CORBA::Any& data)
        throw(CORBA::SystemException);
        

protected:

    /**
     * Purpose:
     * <p>
     * returns the issuer name to be used in notifications.
     * This operation has to be implemented by inheriting classes
     *
     * @return the issuer name
     */
    virtual std::string get_issuer_name() = 0;
        

};


} // End namespace PlatformMngt
} // End namespace Cdmw

#endif // INCL_PLATFORMMNGT_MESSAGE_NOTIFIER_HPP

