/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <platforminterface/ClientInterceptor.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp>

#if CDMW_ORB_VDR == tao
#include "tao/PI/ClientRequestInfo.h"
#endif

#include <iostream>
#include <string>

namespace Cdmw
{
namespace PlatformMngt
{
ClientInterceptor::ClientInterceptor ()
{       
    m_orb_client = CORBA::ORB::_nil();
    m_issuer = "";
    //Create a Context Service 
    m_context_service = new IOP::ServiceContext();
    m_context_service->context_id = CdmwPlatformMngt::SMG_INTERCEPTOR;
}


ClientInterceptor::~ClientInterceptor(){}

//
// IDL to C++ Mapping
//
char*
ClientInterceptor::name() throw (CORBA::SystemException)
{
    return CORBA::string_dup("ClientInterceptor");
}

void
ClientInterceptor::destroy() throw (CORBA::SystemException)
{

}

void
ClientInterceptor::add_issuer(std::string issuer)
{
    m_issuer = issuer;
}

void
ClientInterceptor::set_orb(CORBA::ORB_ptr orb)
{
    if ( !CORBA::is_nil (orb))
    {
        m_orb_client =  CORBA::ORB::_duplicate (orb);   
        //Create a Codec 
        m_codec = new IssuerCodec(m_orb_client.in());

    }
}

void
ClientInterceptor::send_request (PortableInterceptor::ClientRequestInfo_ptr ri)
throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{                    
    if ( !CORBA::is_nil (m_orb_client))
    {                      
        // Encode some data in ServiceContext
        CORBA::OctetSeq_var seq_data;               
        seq_data = m_codec->encode(m_issuer); 
        
        m_context_service->context_data.length(seq_data->length());
        for (unsigned int i=0; i < m_context_service->context_data.length(); i++)
        {
            m_context_service->context_data[i] = seq_data[i];
        }

        // Add Context Service
        ri->add_request_service_context (m_context_service, true);

    }

}


void
ClientInterceptor::send_poll (PortableInterceptor::ClientRequestInfo_ptr ri)
throw (CORBA::SystemException)
{

}


void
ClientInterceptor::receive_reply (PortableInterceptor::ClientRequestInfo_ptr ri)
throw (CORBA::SystemException)
{

}


void
ClientInterceptor::receive_exception(PortableInterceptor::ClientRequestInfo_ptr ri)
throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{

}


void
ClientInterceptor::receive_other(PortableInterceptor::ClientRequestInfo_ptr ri)
throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{

}

} // End namespace Cdmw
} // End namespace PlatformMngt


