/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <platforminterface/ClientInterceptor.hpp>
#include <platforminterface/ClientOrbInitializer.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>

#include <Foundation/orbsupport/TypeTraits.hpp>
#include <Foundation/orbsupport/Codec.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>


#include <iostream>
#include <string>

namespace Cdmw
{
namespace PlatformMngt
{

    // constructor
   ClientOrbInitializer::ClientOrbInitializer( )
   { 
       m_client=0;
       post_done=false;
    }

    //destructor
    ClientOrbInitializer::~ClientOrbInitializer() { }

    // added GD
    void
    ClientOrbInitializer::set_orb(CORBA::ORB_ptr orb)
    {
        m_orb =  CORBA::ORB::_duplicate (orb);
        if (post_done)
        {        
            m_client->set_orb(m_orb);
        }
    }
    
    // added GD
    void
    ClientOrbInitializer::add_issuer(std::string issuer)
    {
        if (post_done)
        {        
            m_client->add_issuer(issuer);
        }
    }

    //
    // IDL to C++ Mapping
    //
    void
    ClientOrbInitializer::pre_init(PortableInterceptor::ORBInitInfo_ptr info) 
    throw (CORBA::SystemException) {}


    //
    // Note: the interceptors are registered in post_init()
    //       if their constructors require
    //       ORBInitInfo::resolve_initial_reference(), which XXX
    //       cannot be called in pre_init().
    //
    void
    ClientOrbInitializer::post_init(PortableInterceptor::ORBInitInfo_ptr info) 
    throw (CORBA::SystemException)
    {
        //
        // Create and register the interceptors
        //
        m_client = new ClientInterceptor();

        PortableInterceptor::ClientRequestInterceptor_var client_interceptor = m_client;

        post_done= true;
        try {
            info->add_client_request_interceptor(client_interceptor.in());
        }
        catch(const PortableInterceptor::ORBInitInfo::DuplicateName&) {
            // can't happened while interceptor is not named!
            CDMW_ASSERT(false);
        }
    }

} // End namespace Cdmw
} // End namespace PlatformMngt
