/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "platforminterface/ComponentContext_impl.hpp"
#include "platforminterface/PlatformInterface.hpp"


namespace Cdmw
{
namespace PlatformMngt
{

//
// Constructor
//
ComponentContext_impl::ComponentContext_impl(const char* component_name, CdmwPlatformMngt::ManagedElementContexts::ProcessContext_ptr process_ctx)
    : m_process_ctx(CdmwPlatformMngt::ManagedElementContexts::ProcessContext::_duplicate(process_ctx)), 
      m_component_name(component_name), 
      m_ccm_component_var(CdmwPlatformMngt::CcmComponent::_nil())
{
   CdmwPlatformMngt::Process_var  m_process_var = CdmwPlatformMngt::Process::_narrow(the_process());
   try 
   {
       if (!CORBA::is_nil(m_process_var.in()))
           m_ccm_component_var = m_process_var->get_component(m_component_name.c_str());
    else
       CDMW_WARN(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),"CdmwPlatformMngt::Process has a nil reference");
   }
   catch (const CdmwPlatformMngt::ComponentNotFound&)
   {
       CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),                         " CcmComponent " << component_name << " is not defined" );
   }
   catch (const CdmwPlatformMngt::NotComponentServer&)
   {
       CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),                        " Process hosting component " << component_name << " is not a component server" );
   }
   catch (const CORBA::SystemException&)
   {
       CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),"CORBA::SystemException");
   }
   catch(...)
   {
        CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),"unexpected exception in ComponentContext_impl C'tor");
   }
}


//
// Destructor
//
ComponentContext_impl::~ComponentContext_impl()
    throw()
{
}



//
// IDL:thalesgroup.com/CdmwPlatformMngt/ManagedElementContexts/ManagedElementContext/the_managed_element:1.0
//

CdmwPlatformMngt::ManagedElement_ptr ComponentContext_impl::the_managed_element() throw (CORBA::SystemException)
{
    return CdmwPlatformMngt::CcmComponent::_duplicate(m_ccm_component_var.in());
}


//
// IDL:thalesgroup.com/CdmwPlatformMngt/ManagedElementContexts/ManagedElementContext/managed_element_name:1.0
//
char* ComponentContext_impl::managed_element_name()
    throw(CORBA::SystemException)
{
    // return component name
    CORBA::String_var name(m_component_name.c_str());
    return name._retn();
}
    
        

std::string ComponentContext_impl::get_issuer_name()
{
    // TODO: Define how to use process_name and component_name in message:
    //       Both in issuer ? add component_name in message ?
    //       Temporary solution: issuer = process_name/component_name
    std::string issuer(PlatformInterface::Get_process_name());
    issuer += "/";
    issuer += m_component_name;
    return issuer;
}

CdmwPlatformMngt::External::System_ptr ComponentContext_impl::the_system() throw ( CORBA::SystemException )
{
    return m_process_ctx->the_system ();     
}

CdmwPlatformMngt::External::Application_ptr ComponentContext_impl::the_application() throw ( CORBA::SystemException )
{
    return m_process_ctx->the_application ();
}

CdmwPlatformMngt::External::Process_ptr ComponentContext_impl::the_process() throw ( CORBA::SystemException )
{
    return m_process_ctx->the_process ();
}

CdmwPlatformMngt::External::Host_ptr ComponentContext_impl::the_host() throw ( CORBA::SystemException )
{
    return m_process_ctx->the_host ();
}

CdmwPlatformMngt::External::RedundancyGroup_ptr ComponentContext_impl::the_redundancy_group() throw ( CORBA::SystemException )
{
     if ( !CORBA::is_nil (m_ccm_component_var.in() ) ) 
     {
     return m_ccm_component_var->get_component_group();
     }
     else 
     {
     return CdmwPlatformMngt::External::RedundancyGroup::_nil();
     }     
}

char* ComponentContext_impl::system_name()throw (CORBA::SystemException)
{
    return m_process_ctx->system_name();
}

char* ComponentContext_impl::application_name() throw (CORBA::SystemException)
{ 
    return m_process_ctx->application_name();
}

char* ComponentContext_impl::process_name() throw (CORBA::SystemException)
{ 
    return m_process_ctx->process_name();
}

char* ComponentContext_impl::host_name() throw (CORBA::SystemException)
{ 
    return m_process_ctx->host_name();
}
    
char* ComponentContext_impl::redundancy_group_name() throw ( CORBA::SystemException )
{
    
     if ( !CORBA::is_nil (m_ccm_component_var.in() ) )return m_ccm_component_var->get_component_group()->name();
     else return CORBA::string_dup("");
}

} // End namespace PlatformMngt
} // End namespace Cdmw


