/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "platforminterface/ManagedComponentContextFactory_impl.hpp"
#include "platforminterface/ComponentContextExt_impl.hpp"


namespace Cdmw
{
namespace PlatformMngt
{

//
// Constructor
//
ManagedComponentContextFactory_impl::ManagedComponentContextFactory_impl()
{
    m_process_context_ptr = CdmwPlatformMngt::ManagedElementContexts::ProcessContext::_nil();
}


//
// Destructor
//
ManagedComponentContextFactory_impl::~ManagedComponentContextFactory_impl()
    throw()
{
    CORBA::release ( m_process_context_ptr );
}


//
// IDL:thalesgroup.com/CdmwPlatformMngt/ManagedComponentContextFactory/find_managed_component_context:1.0
//
CdmwPlatformMngt::ManagedElementContexts::ComponentContextExt_ptr 
ManagedComponentContextFactory_impl::find_managed_component_context(const char* component_name)
    throw (CORBA::SystemException, CdmwPlatformMngt::InvalidName)
{
    using namespace CdmwPlatformMngt::ManagedElementContexts;
    
    // try to find service in ComponentContextMap first
    ComponentContextExtMap::iterator it =  
        m_component_contexts.find(component_name);
    if (it != m_component_contexts.end())
    {
        // ManagedComponentContext found, return it
        return ComponentContextExt::_duplicate(
            it->second.in());
    }
    
    // create a new ManagedComponentContext
    ComponentContextExt_var component_context_ext =
        new ComponentContextExt_impl(component_name, m_process_context_ptr );
        
    // store it in ComponentContextMap
    m_component_contexts.insert(
        ComponentContextExtMap::value_type(component_name, component_context_ext));
        
    return component_context_ext._retn();
}

void ManagedComponentContextFactory_impl::set_managed_process_context (CdmwPlatformMngt::ManagedElementContexts::ProcessContext_ptr managed_process_context) 
    throw (CORBA::SystemException, CdmwPlatformMngt::ContextAlreadyExists) 
{
    using namespace CdmwPlatformMngt::ManagedElementContexts;
    if (CORBA::is_nil ( m_process_context_ptr ) )
    {
        m_process_context_ptr = ProcessContext::_duplicate ( managed_process_context );
    }
    else 
    {
        throw CdmwPlatformMngt::ContextAlreadyExists ();
    }
}


} // End namespace PlatformMngt
} // End namespace Cdmw


