/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "platforminterface/ManagedElementContext_impl.hpp"
#include "platforminterface/PlatformInterface.hpp"

#include "Foundation/idllib/CdmwFTProcess.stub.hpp"
#include "Foundation/orbsupport/CdmwObjectIds.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "tools/ManagedElementKey.hpp"

namespace Cdmw
{

  namespace PlatformMngt
  {

              
      ManagedElementContext_impl::ManagedElementContext_impl()
      {
      }
          
      ManagedElementContext_impl::ManagedElementContext_impl ( CORBA::ORB_ptr orb ):
    m_orb(CORBA::ORB::_duplicate(orb)), m_system_var(CdmwPlatformMngt::System::_nil()),
        m_application_var(CdmwPlatformMngt::Application::_nil()),m_process_var(CdmwPlatformMngt::Process::_nil()),
        m_host_var(CdmwPlatformMngt::Host::_nil()), m_process_message_broker(CdmwPlatformMngt::ProcessMessageBroker::_nil()),        m_managed_element_var(CdmwPlatformMngt::ManagedElement::_nil()),m_redundancy_group_var(CdmwPlatformMngt::RedundancyGroup::_nil())

      {
    m_system_name = CORBA::string_dup ( "" );
    m_application_name = CORBA::string_dup ( "" );
    m_host_name = CORBA::string_dup ( "" );
    m_process_name = CORBA::string_dup ( "" );
    m_process_message_broker = CdmwPlatformMngt::ProcessMessageBroker::_nil(); 
      }

      ManagedElementContext_impl::~ManagedElementContext_impl() throw() 
      {
      }

      CdmwPlatformMngt::External::System_ptr ManagedElementContext_impl::the_system() throw ( CORBA::SystemException )
      {
    return CdmwPlatformMngt::System::_duplicate(m_system_var.in());
      }

      CdmwPlatformMngt::External::Application_ptr ManagedElementContext_impl::the_application() throw ( CORBA::SystemException )
      {
    return CdmwPlatformMngt::External::Application::_duplicate(m_application_var.in());
      }

      CdmwPlatformMngt::External::Process_ptr ManagedElementContext_impl::the_process() throw ( CORBA::SystemException )
      {
    return CdmwPlatformMngt::Process::_duplicate(m_process_var.in());
      }

      CdmwPlatformMngt::External::Host_ptr ManagedElementContext_impl::the_host() throw ( CORBA::SystemException )
      {
    return CdmwPlatformMngt::Host::_duplicate(m_host_var.in());
      }

      CdmwPlatformMngt::External::RedundancyGroup_ptr ManagedElementContext_impl::the_redundancy_group() throw ( CORBA::SystemException )
      {
     //Implemented in ComponentConext
     throw CORBA::NO_IMPLEMENT ( OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO );
      }

      char* ManagedElementContext_impl::system_name()throw (CORBA::SystemException)
      {
    CORBA::String_var name = m_system_name;
    return name._retn();
      }

      char* ManagedElementContext_impl::application_name() throw (CORBA::SystemException)
      { 
    CORBA::String_var name = m_application_name;
    return name._retn();
      }

      char* ManagedElementContext_impl::process_name() throw (CORBA::SystemException)
      { 
    CORBA::String_var name = m_process_name;
    return name._retn();
      }

      char* ManagedElementContext_impl::host_name() throw (CORBA::SystemException)
      { 
    CORBA::String_var name = m_host_name;
    return name._retn();
      }
      
      char* ManagedElementContext_impl::redundancy_group_name() throw ( CORBA::SystemException )
      {
    throw CORBA::NO_IMPLEMENT ( OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO );
      }

      void ManagedElementContext_impl::set_system_name(const char* system_name)throw (CORBA::SystemException) 
      {  
    m_system_name = CORBA::string_dup(system_name);
      }

      void ManagedElementContext_impl::set_application_name(const char* application_name) throw (CORBA::SystemException){ m_application_name = CORBA::string_dup(application_name);}

      void ManagedElementContext_impl::set_host_name(const char* host_name) throw (CORBA::SystemException)
      { 
    m_host_name = CORBA::string_dup(host_name);
      } 

      void ManagedElementContext_impl::set_process_name (const char* process_name) throw (CORBA::SystemException)
      { 
    m_process_name = CORBA::string_dup(process_name);
      }   
         
      
      void ManagedElementContext_impl::set_system ( CORBA::Object_ptr system_ptr ) throw ( CORBA::SystemException )
      {
    m_system_var = CdmwPlatformMngt::System::_narrow ( system_ptr );
    if (!CORBA::is_nil(m_system_var.in()))
      {
        set_system_name(m_system_var->name());
      }
      }

      void ManagedElementContext_impl::set_application ( CORBA::Object_ptr application_ptr ) throw ( CORBA::SystemException )
      {
    m_application_var =  CdmwPlatformMngt::Application::_narrow ( application_ptr );
      }

      void ManagedElementContext_impl::set_host ( CORBA::Object_ptr host_ptr ) throw ( CORBA::SystemException )
      {
    m_host_var =  CdmwPlatformMngt::Host::_duplicate ( CdmwPlatformMngt::Host::_narrow (host_ptr ) );
      }

      void ManagedElementContext_impl::set_process ( CORBA::Object_ptr process_ptr ) throw ( CORBA::SystemException )
      {
    m_process_var = CdmwPlatformMngt::Process::_narrow ( process_ptr );
      }
      
      void ManagedElementContext_impl::set_redundancy_group( CORBA::Object_ptr redundancy_ptr ) throw ( CORBA::SystemException )
      {
    m_redundancy_group_var = CdmwPlatformMngt::RedundancyGroup::_narrow( redundancy_ptr );
      }
      
      /*
       *  Acknowledge the Process Context data
       */

      void ManagedElementContext_impl::ack ( const ContextData& ctx_data ) throw ( CdmwPlatformMngtService::ServiceNotFound, CdmwPlatformMngtService::ServiceUnavailable, CORBA::SystemException )
      { 
    set_system_name (ctx_data.system_name.c_str() ) ;
    set_process_name ( ctx_data.process_name.c_str() );
    set_application_name ( ctx_data.application_name.c_str() );
    set_host_name ( ctx_data.host_name.c_str() );
    m_process_message_broker = CdmwPlatformMngt::ProcessMessageBroker::_duplicate (ctx_data.process_message_broker);
    recover_proxies();
      }

      /*
       *   Recover proxies of all supervised objects 
       */
      void ManagedElementContext_impl::recover_proxies() throw ( CdmwPlatformMngtService::ServiceNotFound, CdmwPlatformMngtService::ServiceUnavailable, CORBA::SystemException)
      {
    set_system ( m_process_message_broker->get_service ( "singleton" ) );
    set_process ( m_process_message_broker->get_service ( ManagedElementKey::Get_process_path ( m_system_name.in(), m_application_name.in(), m_host_name.in(), m_process_name.in() ).c_str() ) );
    set_application ( m_process_message_broker->get_service ( ManagedElementKey::Get_application_path (m_system_name.in(), m_application_name.in() ).c_str() ) );
    set_host ( m_process_message_broker->get_service ( ManagedElementKey::Get_host_path ( m_system_name.in(), m_host_name.in() ).c_str() ) );    }

  } // End namespace PlatformMngt
} // End namespace Cdmw


  
