/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "platforminterface/MessageNotifier_impl.hpp"
#include "platforminterface/PlatformInterface.hpp"
#include "Foundation/common/Assert.hpp"


namespace Cdmw
{
namespace PlatformMngt
{

//
// Constructor
//
MessageNotifier_impl::MessageNotifier_impl()
{
}


//
// Destructor
//
MessageNotifier_impl::~MessageNotifier_impl()
    throw()
{
}


//
// IDL:thalesgroup.com/CdmwPlatformMngt/ManagedElementContexts/MessageNotifier/notify_message:1.0
//
void MessageNotifier_impl::notify_message(CORBA::Long level, 
                                          const char* message)
    throw(CORBA::SystemException)
{
    try {
        PlatformInterface::Notify_message(level, 
                                          get_issuer_name().c_str(),
                                          message);
    } catch (const Cdmw::BadOrderException& e) {
        // Shouldn't happen since PlatformInterface has been initialised
        // at process startup
        CDMW_NEVER_HERE();
    } catch (const Cdmw::BadParameterException& e) {
        // TODO: throw an exception ?? Log an ERROR ?
        std::cerr << "ERROR calling notify_message: " << e << std::endl; 
    }
}

  
//
// IDL:thalesgroup.com/CdmwPlatformMngt/ManagedElementContexts/MessageNotifier/notify_fatal_error:1.0
//
void MessageNotifier_impl::notify_fatal_error(const char* error_info)
    throw(CORBA::SystemException)
{
    try {
        PlatformInterface::Notify_fatal_error(get_issuer_name().c_str(),
                                              error_info);
    } catch (const Cdmw::BadOrderException& e) {
        // Shouldn't happen since PlatformInterface has been initialised
        // at process startup
        CDMW_NEVER_HERE();
    } catch (const Cdmw::BadParameterException& e) {
        // TODO: throw an exception ?? Log an ERROR ?
        std::cerr << "ERROR calling notify_fatal_error: " << e << std::endl; 
    }
}


//
// IDL:thalesgroup.com/CdmwPlatformMngt/ManagedElementContexts/MessageNotifier/notify_user_event:1.0
//
void MessageNotifier_impl::notify_user_event(const CORBA::Any& data)
    throw(CORBA::SystemException)
{
    try  {
        PlatformInterface::Notify_user_event(get_issuer_name().c_str(),
                                             data);
    } catch (const Cdmw::BadOrderException& e) {
        // Shouldn't happen since PlatformInterface has been initialised
        // at process startup
        CDMW_NEVER_HERE();
    } catch (const Cdmw::BadParameterException& e) {
        // TODO: throw an exception ?? Log an ERROR ?
        std::cerr << "ERROR calling notify_user_event: " << e << std::endl; 
    }
}

  
} // End namespace PlatformMngt
} // End namespace Cdmw


