/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <cstring>
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/common/Options.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/osthreads/Mutex.hpp"
#include "Foundation/osthreads/MutexGuard.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "SystemMngt/platforminterface/PlatformInterface.hpp"
#include "SystemMngt/platformlibrary/EventHeaderFactory.hpp"
#include "SystemMngt/tools/ManagedElementKey.hpp"

#include "idllib/CdmwPlatformMngtManagedElementContexts.skel.hpp"
#include "platforminterface/ProcessContext_impl.hpp"
#include "platforminterface/ManagedElementContext_impl.hpp"

#include "platforminterface/ProcessDelegate_impl.hpp"
#include "platforminterface/ManagedComponentContextFactory_impl.hpp"
#include "platforminterface/SupervisionFatalHandler.hpp"

#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/logging/LogManager.hpp"
#include <Foundation/ossupport/SystemProperties.hpp>

#include "SystemMngt/platformlibrary/LogMngr.hpp"

#include <sys/mman.h>
#if CDMW_ORB_VDR == tao && CDMW_ORB_VER > 13
#include <tao/ORBInitializer_Registry.h>
#endif
#include "platforminterface/ClientOrbInitializer.hpp"

namespace Cdmw
{
  namespace PlatformMngt
  {
      const char* PlatformInterface::MANAGED_PROCESS_CONTEXT_NAME =
      "ManagedProcessContext";
    
       


      const char* PlatformInterface::MANAGED_COMPONENT_CONTEXT_FACTORY_NAME =
      "ManagedComponentContextFactory";    

      
      PlatformInterface::InterfaceStatus PlatformInterface::M_status =
      PlatformInterface::INITIAL;

      OsSupport::Mutex PlatformInterface::M_status_mtx;

      CdmwPlatformMngt::ProcessCallback_var PlatformInterface::
      M_process_callback;

      std::string PlatformInterface::M_application_name;
      std::string PlatformInterface::M_process_name;
      
      CORBA::Long PlatformInterface::M_exit_code;
      OsSupport::Mutex PlatformInterface::M_mutex_exit_code;
      
      CdmwPlatformMngt::ProcessConfigurationData PlatformInterface::
      M_configuration_data;

      CdmwPlatformMngt::ProcessMessageBroker_var PlatformInterface::
      M_process_message_broker;

      CdmwPlatformMngt::ManagedElementContexts::ProcessContext_var 
      PlatformInterface::M_managed_process_context;

      CdmwPlatformMngt::ManagedComponentContextFactory_var PlatformInterface::
      M_component_context_finder;

      CORBA::ORB_var PlatformInterface::M_orb;
      
      SupervisionFatalHandler* M_supervision_fatal_handler = 0;
      
      ClientOrbInitializer* PlatformInterface::M_initializer_impl = 0;
      
      bool PlatformInterface::is_interceptor_registered = false;

      void PlatformInterface::Check_status (InterfaceStatus requiredStatus)
    throw (BadOrderException)
      {
        CDMW_MUTEX_GUARD (M_status_mtx);

        if (M_status != requiredStatus)
      {
            std::cout << "M_status=" << M_status << " requiredStatus " <<
          requiredStatus << std::endl;
            CDMW_THROW (BadOrderException);
      }
      }


      void PlatformInterface::Check_status (InterfaceStatus requiredStatus1,
                        InterfaceStatus requiredStatus2)
    throw (BadOrderException)
      {
        CDMW_MUTEX_GUARD (M_status_mtx);

        if ((M_status != requiredStatus1) && 
            (M_status != requiredStatus2))
      {
            std::cout << "M_status=" << M_status << " requiredStatus " <<
          requiredStatus1 << " or " << requiredStatus2 << std::endl;
            CDMW_THROW (BadOrderException);
      }
      }

    
      bool PlatformInterface::Is_launched_by_PlatformManagement (int argc,
                                 char *argv[])
      {
        bool ret = true;

        std::string callbackIor_option =
      OsSupport::OS::get_option_value (argc, argv,
                       Common::Options::CALLBACK_IOR_OPTION);

        if (callbackIor_option == "no" || callbackIor_option == "yes")
      {
            ret = false;
      }

        return ret;
      }


      void PlatformInterface::Setup (CORBA::ORB_ptr orb, int argc, char *argv[])
    throw (BadOrderException, BadParameterException)
      {
        // initialize LogManager (in case this has not yet be done)
        Cdmw::Logging::LogManager::Init(argc, argv);
        
        Check_status (INITIAL);

        if (CORBA::is_nil (orb))
      CDMW_THROW2 (BadParameterException, "orb", "nil");

        M_orb = CORBA::ORB::_duplicate (orb);

        std::string callbackIor_option =
      OsSupport::OS::get_option_value (argc, argv,
                       Common::Options::CALLBACK_IOR_OPTION);

        if (callbackIor_option == "no" || callbackIor_option == "yes")
      {
            CDMW_THROW2 (BadParameterException, "argv",
                         "doesn't contain the callback IOR");
      }
        else
      {
            try
          {
                // gets the acknowledgement callback
                CORBA::Object_var callback_obj =
          orb->string_to_object (callbackIor_option.c_str ());

                CdmwPlatformMngt::ProcessCallback_var callback =
          CdmwPlatformMngt::ProcessCallback::_narrow (callback_obj.in ());

                if (CORBA::is_nil (callback.in ()))
          {
                    CDMW_THROW2 (BadParameterException, "argv",
                                 "doesn't contain a valid callback IOR");
          }

                M_process_callback = callback;

          }
            catch (const CORBA::SystemException &)
          {
                CDMW_THROW2 (BadParameterException, "argv",
                             "contains an invalid IOR");
          }
      }
        
        /* Environment variable CDMW_MEMORY_RESIDENT is modified following the value
         * of memory-resident process tag, a boolean flag to fobid the OS to swap 
         * the process memory          
         */
        char* memory_resident_str= getenv("CDMW_MEMORY_RESIDENT");

        //Locking all of the pages become mapped into the address space of the process.
        if (0!= memory_resident_str)
        {        

            if(0 == strcmp(memory_resident_str,"TRUE"))
            {
                try
                {
                    if ( mlockall(MCL_CURRENT) < 0)
                    {
                        ::perror(" \n Error mlockall(MCL_FUTURE) Cdmw::OsSupport::OS::create_process()");
                    }
                    if ( mlockall(MCL_FUTURE) < 0)
                    {
                        ::perror(" \n Error mlockall(MCL_CURRENT) Cdmw::OsSupport::OS::create_process()");
                    }
                }   
                catch(...)
                {
                    ::perror(" \n Error mlockall(MCL_FUTURE) Cdmw::OsSupport::OS::create_process()");
                }
            }
       }
       // create ManagedComponentContextFactory object
        M_component_context_finder = 
      new ManagedComponentContextFactory_impl();

        // register ManagedComponentContextFactory object in ORB
        Cdmw::OrbSupport::OrbSupport::register_initial_reference(
            orb,
                                 MANAGED_COMPONENT_CONTEXT_FACTORY_NAME,
            M_component_context_finder.in());
                
        std::string register_fatal_handler =
          Cdmw::OsSupport::SystemProperties::Get_Property(
            Cdmw::OrbSupport::SYSTEM_MANAGEMENT_FATAL_HANDLER_PROPERTY_NAME,
            "true"); // by default return true

        if(register_fatal_handler == "true") {
          // No logger used in PlatformInterface therefore std::cout is used
          std::cout << "PlatformInterface registers SupervisionFatalHandler in LogManager" << std::endl;
          M_supervision_fatal_handler = new SupervisionFatalHandler();
          Cdmw::Logging::LogManager::Instance()->register_fatal_handler(
            M_supervision_fatal_handler);
        } else {
          std::cout << "PlatformInterface don't register SupervisionFatalHandler in LogManager" << std::endl;
        }
                                 
        CDMW_MUTEX_GUARD (M_status_mtx);

        M_status = SETUP_PERFORMED;

      }

      void PlatformInterface::Cleanup ()
      {
        CDMW_MUTEX_GUARD (M_status_mtx);
        M_status = PlatformInterface::INITIAL;

        // unregister SupervisionFatalHandler from LogManager
        Cdmw::Logging::LogManager::Instance()->unregister_fatal_handler(
            M_supervision_fatal_handler);
        delete M_supervision_fatal_handler;
        M_supervision_fatal_handler = 0;
        
        M_managed_process_context = CdmwPlatformMngt::ManagedElementContexts::ProcessContext::_nil(); 
        M_process_callback = CdmwPlatformMngt::ProcessCallback::_nil ();
        M_process_message_broker =
      CdmwPlatformMngt::ProcessMessageBroker::_nil ();
    
        M_orb = CORBA::ORB::_nil ();
      }


      void PlatformInterface::Acknowledge_creation (CdmwPlatformMngt::
                            ProcessDelegate_ptr process)
    throw (CORBA::SystemException, BadOrderException, BadParameterException)
      {

        // Notify_creation_starting should have been called before
        // If not (case of some unit tests directly calling Acknowledge_creation)
        // call Notify_creation_starting first
        if (M_status == SETUP_PERFORMED) {
      Notify_creation_starting(process);
        }

        Check_status (CREATION_STARTING_PERFORMED);

        if (CORBA::is_nil (process)) {
      CDMW_THROW2 (BadParameterException, "process", "nil");
        }

        CDMW_MUTEX_GUARD (M_status_mtx);

        CdmwPlatformMngt::ProcessCallback::ProcessStartingData_var start_data =
      M_process_callback->set_ready (process);

        // don't use returned start_data since it has already been
        // received in Notify_creation_starting calling get_starting_info

        M_status = CREATION_DONE_PERFORMED;
      }


      void PlatformInterface::Acknowledge_creation (ProcessBehaviour * behaviour) 
        throw (CORBA::SystemException,
               BadOrderException,
               BadParameterException)
      {
        Check_status (SETUP_PERFORMED, CREATION_STARTING_PERFORMED);

        if (behaviour == NULL) {
      CDMW_THROW2 (BadParameterException, "behaviour", "NULL");
        }

        // creates the default managed process servant
        PortableServer::ServantBase_var process_servant =
      new ProcessDelegate_impl (M_orb.in (), behaviour);

        ProcessDelegate_impl *processDelegate_impl =
      dynamic_cast < ProcessDelegate_impl * >(process_servant.in ());

        // activates the servant and creates the CORBA object
        CdmwPlatformMngt::ProcessDelegate_var processDelegate =
      processDelegate_impl->_this ();

        Acknowledge_creation (processDelegate.in ());
      }


      void PlatformInterface::Notify_creation_starting (CdmwPlatformMngt::ProcessDelegate_ptr process) 
        throw (CORBA::SystemException,
               BadOrderException,
               BadParameterException)
      {

        Check_status (SETUP_PERFORMED);

        if (CORBA::is_nil (process)) {
      CDMW_THROW2 (BadParameterException, "process", "nil");
        }

        CDMW_MUTEX_GUARD (M_status_mtx);

        CdmwPlatformMngt::ProcessCallback::ProcessStartingData_var start_data =
      M_process_callback->get_starting_info (process);

        M_application_name = start_data->application_name;

        M_process_name = start_data->process_name;

        M_configuration_data = start_data->process_configuration_data;

        CDMW_ASSERT (!CORBA::is_nil ((start_data->process_message_broker).in()));
      
        M_process_message_broker =
      CdmwPlatformMngt::ProcessMessageBroker::
      _duplicate ((start_data->process_message_broker).in ());

        // Store process infos in ProcessContext
    ContextData ctx_data;

    ctx_data.system_name = start_data->system_name;
    ctx_data.process_name = M_process_name;
    ctx_data.application_name = M_application_name;
    ctx_data.host_name = OsSupport::OS::get_hostname();
        ctx_data.process_message_broker = start_data->process_message_broker.in();
    
    //instatiate the ProcessContext_impl
    ProcessContext_impl* M_managed_process_context_impl = 
      new ProcessContext_impl(M_orb);
    //set the Process Context Information and recover proxies

        try
        {
        M_managed_process_context_impl->ack(ctx_data);
        }
        catch(const CdmwPlatformMngtService::ServiceNotFound& ex)
        {
        std::cout << ex << std::endl;
        }
        catch(const CdmwPlatformMngtService::ServiceUnavailable& ex)
        {
        std::cout << ex << std::endl;
        }
        catch(const CORBA::SystemException & ex)
        {
        std::cout << ex << std::endl;
        }
    //associate Impl to CORBA Local Oject
    M_managed_process_context = M_managed_process_context_impl;
    //register reference as orb service
    Cdmw::OrbSupport::OrbSupport::register_initial_reference( M_orb, MANAGED_PROCESS_CONTEXT_NAME, M_managed_process_context);
    M_status = CREATION_STARTING_PERFORMED;
    try 
    {
        M_component_context_finder ->  set_managed_process_context ( M_managed_process_context ) ; 
    }
    catch (const CdmwPlatformMngt::ContextAlreadyExists& ex)
    {
        std::cerr << ex._name() << std::endl;
    }
    if (is_interceptor_registered)
    {
        // create interceptor 
        std::string element_path = ManagedElementKey::Get_process_path ( ctx_data.system_name, 
                                                                         ctx_data.application_name,
                                                                         ctx_data.host_name,
                                                                         ctx_data.process_name);     
        M_initializer_impl->set_orb(M_orb);
        M_initializer_impl->add_issuer(element_path);
    }
    
}


      void PlatformInterface::Notify_creation_starting (ProcessBehaviour * behaviour)
        throw (CORBA::SystemException, BadOrderException, BadParameterException)
      {

        Check_status (SETUP_PERFORMED);

        if (behaviour == NULL) {
      CDMW_THROW2 (BadParameterException, "behaviour", "NULL");
        }

        // creates the default managed process servant
        PortableServer::ServantBase_var process_servant =
      new ProcessDelegate_impl (M_orb.in (), behaviour);

        ProcessDelegate_impl *processDelegate_impl =
      dynamic_cast < ProcessDelegate_impl * >(process_servant.in ());

        // activates the servant and creates the CORBA object
        CdmwPlatformMngt::ProcessDelegate_var processDelegate =
      processDelegate_impl->_this ();

        Notify_creation_starting (processDelegate.in ());
      }


      void PlatformInterface::Notify_creation_done ()
    throw (CORBA::SystemException, BadOrderException)
      {

        Check_status (CREATION_STARTING_PERFORMED);

        CDMW_MUTEX_GUARD (M_status_mtx);

        M_process_callback->set_creation_done ();

        M_status = CREATION_DONE_PERFORMED;
      }



      void PlatformInterface::Check_issuer (const char *issuer) 
        throw (BadParameterException)
      {

        if (issuer == NULL) {
      CDMW_THROW2 (BadParameterException, "issuer", "NULL");
        }

        size_t len =::strlen (CdmwPlatformMngtBase::PFMMNG);

        if (::strncmp (CdmwPlatformMngtBase::PFMMNG, issuer, len) == 0) {
      CDMW_THROW2 (BadParameterException, "issuer", "PFMMNG");
        }
      }


      void PlatformInterface::Notify_message (CdmwPlatformMngtBase::EventLevel level,
                          const char *issuer,
                          const char *message)
        throw (CORBA::SystemException, BadOrderException, BadParameterException)
      {
        Check_status (CREATION_STARTING_PERFORMED, CREATION_DONE_PERFORMED);

        if (level == CdmwPlatformMngtBase::FTL) {
      CDMW_THROW2 (BadParameterException, "level", "FTL");
        }

        Check_issuer (issuer);

        if (message == NULL) {
      CDMW_THROW2 (BadParameterException, "message", "NULL");
        }

        CdmwPlatformMngtBase::EventHeader header =
      Cdmw::PlatformMngt::EventHeaderFactory::createHeader (level);

        M_process_message_broker->notify_message (header, issuer, message);
      }



      void PlatformInterface::Notify_fatal_error (const char *issuer,
                          const char *errorInfo)
    throw (CORBA::SystemException, BadOrderException, BadParameterException)
      {
        CDMW_MUTEX_GUARD (M_status_mtx);

        if (M_status == INITIAL) {
      std::cout << "Cannot notify fatal error \"" << errorInfo
            << "\" from " << issuer << ". Throw BadOrderException."
            << std::endl;
      CDMW_THROW (BadOrderException);
        }

        Check_issuer (issuer);

        if (errorInfo == NULL) {
      CDMW_THROW2 (BadParameterException, "errorInfo", "NULL");
        }

        if (M_status == SETUP_PERFORMED)
      {
            // the acknowledgement has not been performed yet
            M_process_callback->notify_error (issuer, errorInfo);
      }
        else
      {
            // the acknowledgement has already been performed

            CdmwPlatformMngtBase::TimeStamp timeStamp =
          Cdmw::PlatformMngt::EventHeaderFactory::createTimeStamp ();

            M_process_message_broker->notify_error (timeStamp, issuer,
                                                    errorInfo);

      }
      }

      void PlatformInterface::Notify_process_fatal_error (const char *process_name,
                              const char *issuer,
                              const char *errorInfo) 
        throw (CORBA::SystemException,
               BadOrderException,
               BadParameterException,
               CdmwPlatformMngt::ProcessNotFound)
      {
        // TODO
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
                                   CORBA::COMPLETED_NO);
      }


      void PlatformInterface::Notify_user_event (const char *issuer,
                         const CORBA::Any & data)
    throw (CORBA::SystemException, BadOrderException, BadParameterException)
      {
        Check_status (CREATION_STARTING_PERFORMED, CREATION_DONE_PERFORMED);

        M_process_message_broker->notify_user_event (issuer, data);

      }



      const char *PlatformInterface::Get_application_name ()
    throw (BadOrderException)
      {
        Check_status (CREATION_STARTING_PERFORMED, CREATION_DONE_PERFORMED);

        return M_application_name.c_str ();
      }


      const char *PlatformInterface::Get_process_name ()
    throw (BadOrderException)
      {
        Check_status (CREATION_STARTING_PERFORMED, CREATION_DONE_PERFORMED);

        return M_process_name.c_str ();
      }
      
           CORBA::Long PlatformInterface::Get_exit_code()
    throw (BadOrderException)
      {
        CDMW_MUTEX_GUARD (M_mutex_exit_code);
        return M_exit_code;
      }

      CdmwPlatformMngt::ProcessConfigurationData PlatformInterface::
      Get_process_config_data () throw (BadOrderException)
      {
        return M_configuration_data;
      }



      CORBA::Object_ptr PlatformInterface::Get_service (const char *serviceName)
    throw (CdmwPlatformMngtService::ServiceNotFound,
           CdmwPlatformMngtService::ServiceUnavailable,
           CORBA::SystemException, BadOrderException, BadParameterException)
      {
        Check_status (CREATION_STARTING_PERFORMED, CREATION_DONE_PERFORMED);

        if (serviceName == NULL) {
      CDMW_THROW2 (BadParameterException, "serviceName", "NULL");
        }
        
        CORBA::Object_var obj = 0;
        std::string serviceNameStr =   serviceName; 
        if (!strcmp(serviceName, MANAGED_PROCESS_CONTEXT_NAME))
      {
        try
          {
            obj = M_orb->resolve_initial_references(MANAGED_PROCESS_CONTEXT_NAME);
        return obj._retn();
          }
        catch (const CORBA::SystemException& ex)
          {
            std::cerr << ex._name() << std::endl;
          }    
      }
        return M_process_message_broker->get_service (serviceName);

      }


      CdmwPlatformMngt::ManagedElement_ptr PlatformInterface::
      Get_managed_element (const char *element_path) throw (CdmwPlatformMngt::
                                ManagedElementNotFound,
                                CORBA::
                                SystemException,
                                BadOrderException,
                                BadParameterException)
      {
        Check_status (CREATION_STARTING_PERFORMED, CREATION_DONE_PERFORMED);

        if (element_path == NULL) {
      CDMW_THROW2 (BadParameterException, "element_path", "NULL");
        }

        CdmwPlatformMngt::ManagedElement_var element;

        try
      {
            element =
          M_process_message_broker->get_managed_element (element_path);
      }
        catch (const CdmwPlatformMngt::ElementPathInvalid&)
      {
            CDMW_THROW2 (BadParameterException, "element_path", "invalid");
      }

        return element._retn ();
      }




      void PlatformInterface::Check_entity_name (const char *entity_name)
    throw (BadOrderException, BadParameterException)
      {

        Check_status (CREATION_STARTING_PERFORMED, CREATION_DONE_PERFORMED);

        if (entity_name == NULL)
      CDMW_THROW2 (BadParameterException, "entity_name", "NULL");

        if (!strcmp (entity_name, ""))
      CDMW_THROW2 (BadParameterException, "entity_name", "NULL");
      }

      void PlatformInterface::
      Check_entity_status_parameters (const char *entity_name,
                                      const char *entity_status,
                                      const char *entity_info)
    throw (BadOrderException, BadParameterException)
      {

        Check_status (CREATION_STARTING_PERFORMED, CREATION_DONE_PERFORMED);

        if (entity_name == NULL)
      CDMW_THROW2 (BadParameterException, "entity_name", "NULL");

        if (!strcmp (entity_name, ""))
      CDMW_THROW2 (BadParameterException, "entity_name", "NULL");
        
        if (entity_status == NULL)
      CDMW_THROW2 (BadParameterException, "entity_status", "NULL");
        
        if (entity_info == NULL)
      CDMW_THROW2 (BadParameterException, "entity_info", "NULL");
      }


      void PlatformInterface::Get_property_value (const char *element_path,
                          const char *property_name,
                          CORBA::Any * property_value)
    throw (CosPropertyService::PropertyNotFound,
           CORBA::SystemException, BadOrderException)
      {

    CdmwPlatformMngt::ManagedElement_var managed_element =
      PlatformInterface::Get_managed_element (element_path);
    CORBA::Any * return_value =
      managed_element->get_property_value (property_name);
    *property_value = *return_value;
      }


    void PlatformInterface::Set_property_value (const char *element_path,
                          const char *property_name,
                          const CORBA::
                          Any & property_value)
    throw (CosPropertyService::PropertyNotFound,
           CosPropertyService::ReadOnlyProperty, CORBA::SystemException,
           BadOrderException)
      {
    CdmwPlatformMngt::ManagedElement_var managed_element =
      PlatformInterface::Get_managed_element (element_path);
    managed_element->define_property (property_name, property_value);
      }
  
    void PlatformInterface::Set_exit_code(CORBA::Long exit_code){
        CDMW_MUTEX_GUARD (M_mutex_exit_code);
        M_exit_code = exit_code;
    }

    void PlatformInterface::Create_interceptor () throw (BadParameterException)
    {
        // Register the ORB initializer
        M_initializer_impl = new ClientOrbInitializer();
            PortableInterceptor::ORBInitializer_var initializer = M_initializer_impl;
            PortableInterceptor::register_orb_initializer(initializer.in());

            is_interceptor_registered = true;
    }

  }                // End namespace PlatformMngt
}                // End namespace Cdmw
