/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "platforminterface/ProcessContext_impl.hpp"
#include "platforminterface/PlatformInterface.hpp"

#include "Foundation/idllib/CdmwFTProcess.stub.hpp"
#include "Foundation/orbsupport/CdmwObjectIds.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"

#include "SystemMngt/idllib/CdmwPlatformMngtEvent.stub.hpp"
#include "Foundation/logging/Logger.hpp"
#include "SystemMngt/platformlibrary/LogMngr.hpp"
#include "Foundation/ossupport/Signals.hpp"
#include "Foundation/ossupport/SystemProperties.hpp"


namespace 
{
    /**
     * Property name for the process exit hold time
     */
    const std::string CDMW_PROCESS_EXIT_HOLDTIME="Cdmw.Process.ExitHoldTime";

    int DEFAULT_CDMW_PROCESS_EXIT_HOLDTIME_VALUE=5000;

/**    
class ProcessExitHandler : public Cdmw::OsSupport::SignalHandler
{
public:
    ProcessExitHandler(CORBA::ORB_ptr orb, CORBA::Long exit_code)
        : m_orb(CORBA::ORB::_duplicate(orb)), m_exit_code(exit_code)
    { 
    }
    
    virtual ~ProcessExitHandler() 
    {
    }
    
    int handle_signal(int sig) throw()
    {
    
        if (!CORBA::is_nil(m_orb.in())) {
            try {
                m_orb->shutdown(true);
            } catch (const CORBA::SystemException& e) {
                    std::cerr<<"ProcessExitHandler : ORB shutdown error "<<Cdmw::OrbSupport::OrbSupport::exception_to_string(e)<<std::endl;
            }
        }

        Cdmw::OsSupport::OS::sleep(5000);

        ::exit(m_exit_code);
        
        return 0;
    };
    
private :
    CORBA::ORB_var m_orb;
    
    CORBA::Long m_exit_code;
    
};
**/

}



namespace Cdmw
{
namespace PlatformMngt
{


CdmwPlatformMngt::ProcessMessageBroker_var ProcessContext_impl::M_proc_mess_broker; 
//
// Constructor
//
ProcessContext_impl::ProcessContext_impl(CORBA::ORB_ptr orb):ManagedElementContext_impl(orb)
{
}


//
// Destructor
//
ProcessContext_impl::~ProcessContext_impl()
    throw()
{
}



//
// IDL:thalesgroup.com/CdmwPlatformMngt/ManagedElementContexts/ManagedElementContext/the_managed_element:1.0
//

CdmwPlatformMngt::ManagedElement_ptr ProcessContext_impl::the_managed_element() throw (CORBA::SystemException)
{
    return CdmwPlatformMngt::ManagedElement::_duplicate(m_process_var.in());
}


//
// IDL:thalesgroup.com/CdmwPlatformMngt/ManagedElementContexts/ManagedElementContext/managed_element_name:1.0
//
char* ProcessContext_impl::managed_element_name()
    throw(CORBA::SystemException)
{
    // return process name
    return this->process_name();
}


//
// IDL:thalesgroup.com/CdmwPlatformMngt/ManagedProcessContext/exit:1.0
//
void ProcessContext_impl::exit(CORBA::Long exit_code)
    throw(CORBA::SystemException)
{

    CDMW_MUTEX_GUARD (m_mutex_exit);

    /** PCR 1915
    ProcessExitHandler * exitHandler = new ProcessExitHandler(m_orb, exit_code);

    Cdmw::OrbSupport::OrbSupport::install_orb_shutdown_signal_handlers(m_orb.in(), exitHandler);
   
    PlatformInterface::Set_exit_code(exit_code);

    int pid = Cdmw::OsSupport::OS::get_processId();
   
    Cdmw::OsSupport::OS::kill_process (pid, SIGTERM);
    **/
    
            
    if (!CORBA::is_nil(m_orb.in())) m_orb->shutdown(false);
    
    //Cdmw::OrbSupport::OrbSupport::uninstall_orb_shutdown_signal_handlers(m_orb.in());
    
    int timeout=Cdmw::OsSupport::SystemProperties::Get_Property(CDMW_PROCESS_EXIT_HOLDTIME, DEFAULT_CDMW_PROCESS_EXIT_HOLDTIME_VALUE);
    
    Cdmw::OsSupport::OS::sleep(timeout);
    
    ::exit(exit_code);
}

  
//
// IDL:thalesgroup.com/CdmwPlatformMngt/ManagedProcessContext/get_process_redundancy_state:1.0
//
CdmwPlatformMngt::RedundancyEltState 
ProcessContext_impl::get_process_redundancy_state()
    throw(CORBA::SystemException)
{
    try {
        // Get PrimaryBackupGroupRepository object
        CORBA::Object_var obj = 
            m_orb->resolve_initial_references(
                Cdmw::OrbSupport::FT_GROUP_REPOSITORY_ObjectId);
                
        if (CORBA::is_nil(obj.in())) {
            // Registered PrimaryBackupGroupRepository is null !!
            throw CORBA::INTERNAL(::Cdmw::OrbSupport::INTERNAL,
                                  CORBA::COMPLETED_NO);
        }

        CdmwFT::PrimaryBackupProcess_var primary_backup_process =
            CdmwFT::PrimaryBackupProcess::_narrow(obj.in());
            
        if (CORBA::is_nil(primary_backup_process.in())) {
            // Registered object is not a PrimaryBackupProcess !! 
            throw CORBA::INTERNAL(::Cdmw::OrbSupport::INTERNAL,
                                  CORBA::COMPLETED_NO);
        }
        
        // Check if this process is primary
        CdmwFT::LocationState state = primary_backup_process->status();
        switch (state) {
            case CdmwFT::LocationState_PRIMARY:
                return CdmwPlatformMngt::REDSTATE_PRIMARY;
            case CdmwFT::LocationState_BACKUP:
                return CdmwPlatformMngt::REDSTATE_PASSIVE;
                
            case CdmwFT::LocationState_INITIAL:
            case CdmwFT::LocationState_INSERTING_PRIMARY:
            case CdmwFT::LocationState_INSERTING_BACKUP:
            case CdmwFT::LocationState_FAILED:     
            default:
                // Behaviour to be defined in those case
                throw CORBA::INTERNAL(::Cdmw::OrbSupport::INTERNAL,
                                      CORBA::COMPLETED_NO);
        }
        
    } catch (const CORBA::ORB::InvalidName& e) {
        // No PrimaryBackupGroupRepository, thus, non-FT process
        return CdmwPlatformMngt::REDSTATE_NORG;
    }
    
}

void ProcessContext_impl::notify_property_change (
    CdmwPlatformMngt::PropertyChange * property_change_event)
{
    CDMW_ASSERT(!CORBA::is_nil(m_process_message_broker.in()));
    
    try
    {
        m_process_message_broker->notify_property_change(property_change_event);
    }
    catch (const CORBA::SystemException &e)
    {
        std::cerr<< " ProcessContext_impl::notify_property_change - ProcessContext_impl:: Exception is  " << e <<std::endl;
    }

}

std::string ProcessContext_impl::get_issuer_name()
{
    // issuer name is process name
    std::string issuer(PlatformInterface::Get_process_name());
    return issuer;
}

} // End namespace PlatformMngt
} // End namespace Cdmw



