/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "platforminterface/SupervisionFatalHandler.hpp"

#include <SystemMngt/platforminterface/PlatformInterface.hpp>
#include <string.h>

namespace Cdmw
{
namespace PlatformMngt
{

    
void 
SupervisionFatalHandler::on_fatal_log(const CdmwLogging::LogHeader& header, 
                                      const char* msg)
    throw()
{
    try {
        
        // Issuer format is <process_name>[/<component_name]
        // Same solution than in ComponentContext_impl::get_issuer_name()
        std::string issuer(PlatformInterface::Get_process_name());
        if (strlen(header.component_name) > 0) {
            issuer += "/";
            issuer += header.component_name;
        }
        
        PlatformInterface::Notify_fatal_error(issuer.c_str(), msg);
        
    } catch (...) {
        // Error notifying Supervision.
        // No other way to notify Supervision.
        // Give up, since fatal error is alredy logged 
    }
    
}
    
    
} // End namespace PlatformMngt
} // End namespace Cdmw


