/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.platformmngt;

import com.thalesgroup.CdmwPlatformMngt.ManagedElementContexts.ComponentContext;
import com.thalesgroup.CdmwPlatformMngt.ManagedElementContexts.ProcessContext;
import com.thalesgroup.CdmwPlatformMngt.*;

import  cdmw.logging.*;
import static cdmw.logging.Logger.*;
import  com.thalesgroup.CdmwPlatformMngtBase.PFMMNG;


/**
 * The default implementation of a ManagedElementContexts::ComponentContext.
 *
 */
class ComponentContextImpl 
    //extends ManagedElementContextImpl
    extends MessageNotifierImpl
    implements ComponentContext 
{

    private String m_componentName;
    protected ProcessContext m_processContext;
    private CcmComponent m_ccm_component;
    
    
    //
    // Default constructor
    //
    
    public ComponentContextImpl()
    {
    }
    
    //
    // Constructor
    //
    public ComponentContextImpl(String componentName, ProcessContext processContext)
    {
        m_componentName = componentName;
        m_processContext = processContext;
        
        com.thalesgroup.CdmwPlatformMngt.Process process = com.thalesgroup.CdmwPlatformMngt.ProcessHelper.narrow(the_process());
        try
        {
            m_ccm_component = process.get_component(m_componentName);
        }
        catch (Exception e)
        {
         CDMW_WARN(Logger.Get_Logger(PFMMNG.value), "CcmComponent "+ m_componentName + " is not defined" , e);
        }
    }
    
    
    //
    // IDL:thalesgroup.com/CdmwPlatformMngt/ManagedComponentContext/managed_element_name:1.0
    //
    public String managed_element_name()
    {
        return m_componentName;
    }
        
    
    /**
     * Purpose:
     * <p>
     * returns the issuer name to be used in notifications.
     * This operation has to be implemented by inheriting classes
     *
     * @return the issuer name
     */
    protected String getIssuerName()
    {
        // TODO: Define how to use process_name and component_name in message:
        //       Both in issuer ? add component_name in message ?
        //       Temporary solution: issuer = process_name/component_name
        String issuer = this.process_name() + "/" + m_componentName;
        return issuer;
    }
    
    public com.thalesgroup.CdmwPlatformMngt.External.System the_system()
    {
        return m_processContext.the_system ();     
    }

    public com.thalesgroup.CdmwPlatformMngt.External.Application the_application() 
    {
        return m_processContext.the_application ();
    }

    public com.thalesgroup.CdmwPlatformMngt.External.Process the_process()
    {
        return m_processContext.the_process ();
    }

    public com.thalesgroup.CdmwPlatformMngt.External.Host the_host() 
    {
        return m_processContext.the_host ();
    }

    public com.thalesgroup.CdmwPlatformMngt.External.RedundancyGroup the_redundancy_group()
    {
        if (null != m_ccm_component) return  m_ccm_component.get_component_group();
    else return null;
    }

    public com.thalesgroup.CdmwPlatformMngt.ManagedElement the_managed_element() 
    {
    //  throw new org.omg.CORBA.NO_IMPLEMENT(
    //            cdmw.orbsupport.ExceptionMinorCodes.NO_IMPLEMENT, 
    //            org.omg.CORBA.CompletionStatus.COMPLECTED_NO);
    
     return m_ccm_component;

    }
      
    public String system_name()
    {
        return m_processContext.system_name();
    }

    public String application_name()
    { 
        return m_processContext.application_name();
    }

    public String process_name()
    { 
        return m_processContext.process_name();
    }

    public String host_name()
    { 
        return m_processContext.host_name();
    }
        
    public String redundancy_group_name() 
    {
        if (null != m_ccm_component) return m_ccm_component.get_component_group().name();
    else {
    String null_string = "";
    return null_string;
    }
    }
    
}

