/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.platformmngt;

import com.thalesgroup.CdmwPlatformMngt.*;
import cdmw.ossupport.OS;
import cdmw.common.BadOrderException;
import cdmw.common.BadParameterException;
import com.thalesgroup.CdmwPlatformMngtService.*;

import org.omg.CORBA.portable.*;
import org.omg.CORBA.*;
import cdmw.tools.*;

/**
* Purpose:
* <p>
* The abstract Managed Process Context
*/

abstract class ManagedElementContextImpl   extends MessageNotifierImpl
    implements com.thalesgroup.CdmwPlatformMngt.ManagedElementContexts.ManagedElementContext
{


    protected org.omg.CORBA.ORB m_orb;
    
    /**
     * Purpose: 
     * <p> Default Constructor.
     */
    public ManagedElementContextImpl()
    {
    }

    /**
     * Purpose:
     * <p> Constructor.
     */ 
    public ManagedElementContextImpl(org.omg.CORBA.ORB orb)
    {
        m_orb = orb;
    }
    
      /**
     * Get System Proxy.
     */
    public com.thalesgroup.CdmwPlatformMngt.External.System the_system() throws org.omg.CORBA.SystemException
    {
        return m_system;
    }

    /**
     * Get Appllication Proxy.
     */
    public com.thalesgroup.CdmwPlatformMngt.External.Application the_application() throws org.omg.CORBA.SystemException
    {
        return m_application;
    }

    /**
    * Get Process Proxy.
    */
    public com.thalesgroup.CdmwPlatformMngt.External.Process the_process() throws org.omg.CORBA.SystemException
    {
        return m_process;
    }

    /**
    * Get Host Proxy.
    */
    public com.thalesgroup.CdmwPlatformMngt.External.Host the_host() throws org.omg.CORBA.SystemException
    {
        return m_host;
    }


        
   /**
    * Get Managed Element Proxy.
    */
    public com.thalesgroup.CdmwPlatformMngt.ManagedElement the_managed_element() throws org.omg.CORBA.SystemException
    {
         return m_process;
    }

   /**
    * Get Redundancy Group Proxy
    */
    public com.thalesgroup.CdmwPlatformMngt.RedundancyGroup the_redundancy_group() throws org.omg.CORBA.SystemException
    {
         //Implemented in ComponentContextImpl
     
     throw new org.omg.CORBA.NO_IMPLEMENT(
            cdmw.orbsupport.ExceptionMinorCodes.NO_IMPLEMENT, 
            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
    }   

     
    /**
     * IDL:thalesgroup.com/CdmwPlatformMngt/ManagedProcessContext/system_name:1.0
     */
    public java.lang.String system_name() throws org.omg.CORBA.SystemException
    {
        return m_system_name;
    }

    /**
     * IDL:thalesgroup.com/CdmwPlatformMngt/ManagedProcessContext/application_name:1.0
     */
    public java.lang.String application_name() throws org.omg.CORBA.SystemException
    {
        return m_application_name;
    }

    /**
     * IDL:thalesgroup.com/CdmwPlatformMngt/ManagedProcessContext/process_name:1.0
     */
    public java.lang.String process_name() throws org.omg.CORBA.SystemException
    {
        return m_process_name;
    }

    /**
     * IDL:thalesgroup.com/CdmwPlatformMngt/ManagedProcessContext/host_name:1.0
     */
    public java.lang.String host_name() throws org.omg.CORBA.SystemException
    {
        return m_host_name;
    }

    /**
     * IDL:thalesgroup.com/CdmwPlatformMngt/ManagedProcessContext/managed_element_name/1.0
     */        
    public java.lang.String managed_element_name() throws org.omg.CORBA.SystemException
    {
        return m_process_name;
    }

    /**
     * IDL:thalesgroup.com/CdmwPlatformMngt/ManagedProcessContext/redundancy_group_name:1.0
     */        
    public String redundancy_group_name() throws org.omg.CORBA.SystemException
    {
        throw new org.omg.CORBA.NO_IMPLEMENT(
            cdmw.orbsupport.ExceptionMinorCodes.NO_IMPLEMENT, 
            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
    }

    
    /**
     * get the process redundancy group state
     */
      
    /**
     * IDL:thalesgroup.com/CdmwPlatformMngt/ManagedProcessContext/get_process_redundancy_state:1.0
     */
    public com.thalesgroup.CdmwPlatformMngt.RedundancyEltState 
    get_process_redundancy_state()
    {
        // no FT in Java
        return com.thalesgroup.CdmwPlatformMngt.RedundancyEltState.REDSTATE_NORG;
    }
        

    /**
     * set the system name
     */ 
    public void set_system_name(String system_name)
    {
        m_system_name = system_name;
    }

    /**
     * set the application name
     */ 
    public void set_application_name(String application_name)
    {
        m_application_name = application_name;
    }

    /**
     * set the process name
     */ 
    public void set_process_name(String process_name)
    {
        m_process_name = process_name;
    }

    /**
     * set the host name
     */ 
    public void set_host_name(String host_name)
    {
        m_host_name = host_name;
    }

    /**
     * set the system reference
     */ 
    public void set_system(org.omg.CORBA.Object system)  throws org.omg.CORBA.SystemException
    {
        m_system =  com.thalesgroup.CdmwPlatformMngt.SystemHelper.narrow(system);
    if (m_system !=null)
        set_system_name(m_system.name());
    }

    /**
     * set the application reference
     */ 
    public void set_application(org.omg.CORBA.Object application)  throws org.omg.CORBA.SystemException
    {
        m_application =  com.thalesgroup.CdmwPlatformMngt.ApplicationHelper.narrow(application);
    }

    /**
     * set the host reference
     */ 
    public void set_host(org.omg.CORBA.Object host)  throws org.omg.CORBA.SystemException
    {
        m_host =  com.thalesgroup.CdmwPlatformMngt.HostHelper.narrow(host);
    }

    /**
     * set the process reference
     */ 
    public void set_process(org.omg.CORBA.Object process)  throws org.omg.CORBA.SystemException
    {
        m_process =  com.thalesgroup.CdmwPlatformMngt.ProcessHelper.narrow(process);
    }

    /**
     * set the redundancy group reference
     */ 
    public void set_redundancy_group(org.omg.CORBA.Object redundancy_group)  throws org.omg.CORBA.SystemException
    {
         throw new org.omg.CORBA.NO_IMPLEMENT(
            cdmw.orbsupport.ExceptionMinorCodes.NO_IMPLEMENT, 
            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
    }

    /**
     * set the redundancy group name
     */ 
    public void  set_redundancy_group_name(String redundancy_group_name) throws org.omg.CORBA.SystemException
    {
         throw new org.omg.CORBA.NO_IMPLEMENT(
            cdmw.orbsupport.ExceptionMinorCodes.NO_IMPLEMENT, 
            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
    }
 
    /**
     * acknowledge the ContextData
     */ 
    void ack(ContextData ctx_data) throws ServiceNotFound, ServiceUnavailable, org.omg.CORBA.SystemException
    {
        set_system_name (ctx_data.system_name);
    set_process_name(ctx_data.process_name);
        set_application_name(ctx_data.application_name);
        set_host_name(ctx_data.host_name);
        m_process_message_broker =  ctx_data.process_message_broker;
        recover_proxies();
    }

    /**
     * recover proxies supervised objects
     */ 
    void recover_proxies() throws ServiceNotFound, ServiceUnavailable, org.omg.CORBA.SystemException
    {
    // set locally the supervised object 
    // after it has been recovered from message broker
        set_system(m_process_message_broker.get_service ("singleton"));
        set_process(m_process_message_broker.get_service(ManagedElementKey.getProcessPath(m_system_name, m_application_name, m_host_name, m_process_name)));
        set_application(m_process_message_broker.get_service(ManagedElementKey.getApplicationPath(m_system_name, m_application_name)));
        set_host(m_process_message_broker.get_service(ManagedElementKey.getHostPath(m_system_name, m_host_name)));
    }

    public static class ContextData
    {
        public static String process_name;
        public static String system_name;
        public static String host_name;
        public static String application_name;
        public static com.thalesgroup.CdmwPlatformMngt.ProcessMessageBroker process_message_broker;
    }


    /**
     * Purpose:
     * <p>
     * returns the issuer name to be used in notifications.
     * This operation has to be implemented by inheriting classes
     *
     * @return the issuer name
     */
    protected String getIssuerName()
    {
        // issuer name is process name
        return this.process_name();
    }
    

    /**
     * Purpose:
     * <p>
     * returns a process message broker istance.
     *
     * @return the issuer name
     */
    protected com.thalesgroup.CdmwPlatformMngt.ProcessMessageBroker getProcessMessageBroker()
    {
        return m_process_message_broker;
    }
    

    /////////////////////////////////////
    // Managed Element Proxies References
    /////////////////////////////////////
    
    private com.thalesgroup.CdmwPlatformMngt.External.System            m_system;
    private com.thalesgroup.CdmwPlatformMngt.External.Application       m_application;
    private com.thalesgroup.CdmwPlatformMngt.External.Process           m_process;
    private com.thalesgroup.CdmwPlatformMngt.External.Host          m_host;
    private com.thalesgroup.CdmwPlatformMngt.ProcessMessageBroker       m_process_message_broker;

    //////////////////////////
    //  Managed Element  names
    //////////////////////////
    private String m_system_name;
    private String m_application_name;
    private String m_host_name;
    private String m_process_name;




}



