/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.platformmngt;

import com.thalesgroup.CdmwPlatformMngt.*;
import cdmw.ossupport.OS;
import cdmw.orbsupport.*;
import cdmw.common.BadOrderException;
import cdmw.common.BadParameterException;
import com.thalesgroup.CdmwPlatformMngtService.*;

import org.omg.CORBA.portable.*;
import org.omg.CORBA.*;
import cdmw.tools.*;

import  cdmw.logging.*;
import static cdmw.logging.Logger.*;
import  com.thalesgroup.CdmwPlatformMngtBase.PFMMNG;




class ExitThread extends Thread
{
    protected int m_exitCode;

    public ExitThread(int exitCode)
    {
        m_exitCode = exitCode;
    }

    public void run()
    { 
       java.lang.System.exit(m_exitCode);
    }
}





/**
* Purpose:
* <p>
* The abstract Managed Process Context
*/

class ProcessContextImpl   extends ManagedElementContextImpl
                           implements com.thalesgroup.CdmwPlatformMngt.ManagedElementContexts.ProcessContext
{
     /**
     * Property name for the process exit hold time
     */
     
    static protected String CDMW_PROCESS_EXIT_HOLDTIME_PROPERTY="Cdmw.Process.ExitHoldTime";
     
     /**
     * The default exit hold time value in milliseconds
     */
    static protected String DEFAULT_CDMW_PROCESS_EXIT_HOLDTIME_VALUE="5000";


     
    /**
     * Purpose:
     * <p> Constructor.
     */ 
    public ProcessContextImpl(org.omg.CORBA.ORB orb)
    {
    super(orb);
    }
    
      
    /*
     * IDL:thalesgroup.com/CdmwPlatformMngt/ManagedProcessContext/exit:1.0
    */
    public void exit(int exit_code)
    {
    
      /*  OrbGracefulShutdownSignalHandler sighandler = null;
       
        // ===================================================
        // Add ORB shutdown hooks
        // ===================================================
        sighandler = new OrbGracefulShutdownSignalHandler (m_orb);
                
        ORBSupport.InstallORBShutdownSignalHandler (m_orb, sighandler);
         
        ExitThread exitThread = new ExitThread(exit_code);
    
        exitThread.start();*/

       CDMW_DEBUG(Logger.Get_Logger(PFMMNG.value),"Call orb shutdown");
        
       m_orb.shutdown(false);
    
       int timeout=new Integer(java.lang.System.getProperty(CDMW_PROCESS_EXIT_HOLDTIME_PROPERTY, DEFAULT_CDMW_PROCESS_EXIT_HOLDTIME_VALUE)).intValue();
    
       try{
        cdmw.ossupport.OS.sleep(timeout);
       }
       catch(java.lang.InterruptedException ex){
        CDMW_WARN(Logger.Get_Logger(PFMMNG.value),"Interrupted sleep while exiting !",ex); //FIXME: is it just a warn or Error ?
       }
    
       java.lang.System.exit(timeout);

    }
    
    
     /**
     * IDL:thalesgroup.com/CdmwPlatformMngt/ManagedProcessContext/get_process_redundancy_state:1.0
     */
    public com.thalesgroup.CdmwPlatformMngt.RedundancyEltState 
    get_process_redundancy_state()
    {
        // no FT in Java
        return com.thalesgroup.CdmwPlatformMngt.RedundancyEltState.REDSTATE_NORG;
    }
        
  
    /**
     * Purpose:
     * <p>
     * returns the issuer name to be used in notifications.
     * This operation has to be implemented by inheriting classes
     *
     * @return the issuer name
     */
    protected String getIssuerName()
    {
        // issuer name is process name
        return this.process_name();
    }
    
    public void notify_property_change (PropertyChange property_change_event)
    {
        getProcessMessageBroker().notify_property_change(property_change_event);
    } 
 
}



