/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.platformmngt;

/**
 * Implementation of FatalHandler notifying supervision when a 
 * FATAL message is logged.
 */
public class SupervisionFatalHandler 
    implements cdmw.logging.FatalHandler 
{

    /**
     * This operation is called by LogManager when a FATAL message
     * is logged. A notification of fatal error will be send to Supervision.
     *
     * @param header the message header.
     * @param msg the message.
     */
    public void onFatalLog(com.thalesgroup.CdmwLogging.LogHeader header,
                           String msg)
    {
        try {
            
            // Issuer format is <process_name>[/<component_name]
            // Same solution than in ComponentContext_impl::get_issuer_name()
            String issuer = PlatformInterface.getProcessName();
            if (header.component_name.length() != 0) {
                issuer += "/" + header.component_name;
            }
            
            PlatformInterface.notifyFatalError(issuer, msg);
            
        } catch (Throwable e) {
            // Error notifying Supervision.
            // No other way to notify Supervision.
            // Give up, since fatal error is alredy logged 
        }
    }

}

