/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <testmanagedprocess/ManagedContextUtil.hpp>
#include <platforminterface/PlatformInterface.hpp>

#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtProcess.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"

#include "Foundation/testutils/CORBATestManager.hpp"

#include <SystemMngt/platformvaluetypes/HostStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/ApplicationStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/EntityStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/SystemStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/SystemModeChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/ApplicationModeChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/PropertyChange_impl.hpp>

#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"
#include "idllib/CdmwPlatformMngtProcess.stub.hpp"
#include "idllib/CdmwPlatformMngtHost.stub.hpp"




using namespace CdmwPlatformMngt;
 

namespace Cdmw
{

namespace PlatformMngt
{

ManagedContextUtil::ManagedContextUtil(CORBA::ORB_ptr orb)
{
    m_orb = CORBA::ORB::_duplicate(orb);
    // register a SupervisionObserver to get the notification of user messages
    m_supervision_observer_impl = new SupervisionObserver_impl();
    CORBA::ValueFactoryBase_var factory, old_factory;
    factory = new ProcessMessageFactory;
    old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/ProcessMessage:1.0",factory.in() );
    m_supervision_observer_var = m_supervision_observer_impl->_this();
    ctx = CdmwPlatformMngt::ManagedElementContexts::ProcessContext::_nil();
}


void ManagedContextUtil::perform_operation() 
{

    CORBA::Object_var obj;
    try
    {
           obj = PlatformInterface::Get_service(PlatformInterface::MANAGED_PROCESS_CONTEXT_NAME);
    }
    catch (const CORBA::SystemException& ex)
    {
            std::cerr << "  PlatformInterface::Get_service " << ex << std::endl;
    }
    
    
     CdmwPlatformMngt::System_var       mysystem;
     CdmwPlatformMngt::Application_var  myapp;
     CdmwPlatformMngt::Process_var      myproc;
     CdmwPlatformMngt::Host_var         myhost;

     // import the object reference from the ior file
     CORBA::Object_var objref;

     //define the corbaloc
     const char *corbaloc_str = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";
     objref = m_orb->string_to_object(corbaloc_str);
     char * hostname  = getenv("HOSTNAME");

     //resolve the System_ptr
     mysystem = CdmwPlatformMngt::System::_narrow(objref.in() );
     myapp = mysystem -> get_application(PlatformInterface::Get_application_name());
     myhost = mysystem -> get_host(hostname);
     myproc = myhost->get_process(PlatformInterface::Get_process_name(), PlatformInterface::Get_application_name());

     std::string system_name = mysystem->name();
     std::string application_name =  myapp->name();
     std::string process_name = myproc->name();
     std::string host_name = myhost->name();

     ctx = CdmwPlatformMngt::ManagedElementContexts::ProcessContext::_narrow(obj.in());
     CdmwPlatformMngt::External::System_var system = ctx->the_system();  
     CdmwPlatformMngt::External::Application_var app = ctx->the_application();
     CdmwPlatformMngt::External::Process_var proc = ctx->the_process();
     CdmwPlatformMngt::External::Host_var host = ctx->the_host();
     CdmwPlatformMngt::ManagedElement_var managed_element = ctx->the_managed_element();

     std::cerr <<"Check on Supervised Object names"<<std::endl;
     assert(system_name == system->name());
     assert(application_name == app->name());    
     assert(process_name == proc->name());
     assert(host_name == host->name());
     assert(process_name == managed_element->name());
     system->register_observer("MyTestObserver", m_supervision_observer_var.in());
     //sleep(2);
     ctx->notify_message ( 1 , "MessageFromProcessContext");
     size_t count = 0;
     while ((!m_supervision_observer_impl->is_ok())&&(count < 3))
     {
        count++;
        std::cerr << "waiting the ProcessMessage notification "<< std::endl;
        //sleep(1);
     }
     //  if (m_supervision_observer_impl->is_ok())
     /*
     ctx->notify_fatal_error ("Message from ComponentContext");
     CORBA::Any any_data;
     any_data <<="any_dummy";
     ctx->notify_user_event (any_data);
     */       
}

ManagedContextUtil::~ManagedContextUtil() throw()
{
     if (!CORBA::is_nil(ctx.in()) )
     {
         std::cerr << "All verification on ProcessContext are OK" << std::endl;
         std::cerr << "going to test ProcessContext::exit()"<<std::endl;
         ctx->exit(0);
     }
}


} //end PlatformMngt

} //end Cdmw


















