/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.platformmngt.test;
import org.omg.CORBA.StringHolder;

import cdmw.common.BadOrderException;
import cdmw.common.BadParameterException;

import cdmw.platformmngt.PlatformInterface;

import com.thalesgroup.CdmwPlatformMngt.Entity;
import com.thalesgroup.CdmwPlatformMngtBase.ERR;

/**
 * SystemEntityToUpdate.
 */
public class ApplicationEntityToUpdate extends EntityToUpdate {

    /**
     * Constructor.
     */ 
    public ApplicationEntityToUpdate(String entityName, 
                                     String hostName,
                                     String applicationName,
                                     String processName) {
        super(entityName, hostName, applicationName, processName);
    }

    /**
     * Executes the updating.
     */
    public void update() throws BadParameterException, BadOrderException {

        if (entityName.length() != 0 && !errorSet) {
            String infoMsg = "";

            try {
        
            org.omg.CORBA.Object obj_mng_context = null;
            try{   
               obj_mng_context = PlatformInterface.getService("ManagedProcessContext");
            }
            catch (com.thalesgroup.CdmwPlatformMngtService.ServiceNotFound ex)
            { 
            }
            catch (com.thalesgroup.CdmwPlatformMngtService.ServiceUnavailable ex2)
            { 
            }
            com.thalesgroup.CdmwPlatformMngt.ManagedElementContexts.ProcessContext ctx = com.thalesgroup.CdmwPlatformMngt.ManagedElementContexts.ProcessContextHelper.narrow(obj_mng_context);
            com.thalesgroup.CdmwPlatformMngt.System sys = com.thalesgroup.CdmwPlatformMngt.SystemHelper.narrow(ctx.the_system());
            com.thalesgroup.CdmwPlatformMngt.Application application = sys.get_application(applicationName);
            com.thalesgroup.CdmwPlatformMngt.Entity entity = application.get_entity(entityName);
        
            updatingCounter++;

            if (updatingCounter == ENTITY_UPDATE_MAX_COUNTER) {
            System.out.println(processName+ " " + "notify");
                    PlatformInterface.notifyFatalError(
                        processName, "Fatal Error");
                }
                else if (updatingCounter == 1) {
                    entity.set_status("functioning","the application entity status is : functioning");
                    StringHolder info = new StringHolder();
                    String status;   
                    status = entity.get_status(info);
                }
                else if (updatingCounter < ENTITY_UPDATE_MAX_COUNTER) {

                    StringHolder info = new StringHolder();
                    String status;   
                    status = entity.get_status(info);
                    
                    if (status.equals("disfunction"))
                        {
                            entity.set_status("functioning","the application entity status is : functioning");
                        } else {
                        entity.set_status("disfunction","the application entity status is : disfunction");;                      
                    }
                }
            } catch (com.thalesgroup.CdmwPlatformMngt.ApplicationNotFound anf) {
                errorSet = true;
                infoMsg = "Entity not found : " + entityName;
            } catch (com.thalesgroup.CdmwPlatformMngt.EntityNotFound enf) {
                errorSet = true;
                infoMsg = "Entity not found : " + entityName;
            } catch (org.omg.CORBA.SystemException se) {
                errorSet = true;
                infoMsg = "setApplicationEntityStatus system exception : "
                    + entityName + "\n" + se.toString();
            } catch (cdmw.common.BadOrderException boe) {
                errorSet = true;
                infoMsg = "setApplicationEntityStatus bad order exception";
            } catch (cdmw.common.BadParameterException bpe) {
                errorSet = true;
                infoMsg = "setApplicationEntityStatus bad parameter exception";
            } catch (com.thalesgroup.CdmwPlatformMngt.EntityStatusNotAllowed esna) {
                errorSet = true;
                infoMsg = "setProcessEntityStatus entity status not allowed";
            } 

            if (errorSet) {
                PlatformInterface.notifyMessage(ERR.value, processName, infoMsg);
            }
        }
        
    }

}
