/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.platformmngt.test;

import cdmw.testutils.Testable;
import cdmw.common.BadOrderException;
import cdmw.common.BadParameterException;

import cdmw.platformmngt.ProcessDelegateImpl;
import org.omg.CORBA.*;
import org.omg.PortableServer.*;
import cdmw.platformmngt.*;
import com.thalesgroup.CdmwPlatformMngt.ProcessCallbackPackage.*;
import cdmw.ossupport.OS;
import com.thalesgroup.CdmwPlatformMngt.*;

import cdmw.common.Assert;



public class ManagedContextUtil
{

  public ManagedContextUtil(org.omg.CORBA.ORB orb)
  {
      this.orb = orb;

      org.omg.CORBA.Object obj = null; 
      supervisionObserverImpl = new SupervisionObserverImpl(orb);
      try{
      obj =  orb.resolve_initial_references("RootPOA");
      org.omg.PortableServer.POA poa = org.omg.PortableServer.POAHelper.narrow(obj);
      poa.the_POAManager().activate();
      }catch(Exception e)
      {
      }
      
      // valuetype registration
      //((org.omg.CORBA_2_3.ORB)orb).register_value_factory(ProcessMessageHelper.id(), new    ProcessMessageFactoryImpl());
      InitUtils.registerValuetypeFactories(orb);
      
      supervisionObserver = null;
      try{
          supervisionObserver = supervisionObserverImpl._this(orb);
          new Thread(supervisionObserverImpl).start();
      }
      catch(Exception ex)
      {
      java.lang.System.out.println(ex.toString());
      }
  }

  public void performOperation(){

    org.omg.CORBA.Object obj = null;

    obj = orb.string_to_object("corbaloc::localhost:21869/CdmwPlatformMngtSupervision");

    // get the supervised object proxies
    com.thalesgroup.CdmwPlatformMngt.System mySystem = null;
    com.thalesgroup.CdmwPlatformMngt.Application myApplication = null;
    com.thalesgroup.CdmwPlatformMngt.Host myHost = null;
    com.thalesgroup.CdmwPlatformMngt.Process myProcess = null;
    String processName = "";
    String applicationName = "";
    String systemName = "";
    String hostName = "";
                   
    try 
    {
       mySystem  = com.thalesgroup.CdmwPlatformMngt.SystemHelper.narrow(obj);
       myApplication = mySystem.get_application(PlatformInterface.getApplicationName());
       
       myHost = mySystem.get_host(OS.getHostname());
       myProcess = myHost.get_process(PlatformInterface.getProcessName(), PlatformInterface.getApplicationName());
       processName = myProcess.name();
       applicationName = myApplication.name();
       systemName = mySystem.name();
       hostName = OS.getHostname();
                      
        }
    catch(Exception ex)
    {
        java.lang.System.out.println(ex.toString());
    }
    
        // recover ManagedProcessContext    
        try
        {
            obj = cdmw.platformmngt.PlatformInterface.getService(cdmw.platformmngt.PlatformInterface.MANAGED_PROCESS_CONTEXT_NAME);
        }
        catch (Exception ex)
        {
            java.lang.System.out.println(ex.toString());
        }

        ctx = com.thalesgroup.CdmwPlatformMngt.ManagedElementContexts.ProcessContextHelper.narrow(obj);
        com.thalesgroup.CdmwPlatformMngt.External.System sys = ctx.the_system();
        com.thalesgroup.CdmwPlatformMngt.External.Process proc = ctx.the_process();
        com.thalesgroup.CdmwPlatformMngt.External.Host host = ctx.the_host();
        com.thalesgroup.CdmwPlatformMngt.External.Application app = ctx.the_application();
        com.thalesgroup.CdmwPlatformMngt.ManagedElement managed_element = ctx.the_managed_element();
        sys.register_observer("MyTestObserver", supervisionObserver);
        java.lang.System.out.println("Check on Supervised Object names");
    
        Assert.check(sys.name().equals(systemName));
        Assert.check(app.name().equals(applicationName));
        Assert.check(proc.name().equals(processName));
        Assert.check(managed_element.name().equals(processName));
        Assert.check(host.name().equals(hostName));
        
        try{OS.sleep(2000);}catch(Exception e){};
        ctx.notify_message ( 1 , "MessageFromProcessContext");

        int iteration = 0;
        int max_iteration = 3;
        while ((!supervisionObserverImpl.is_ok())&&(iteration < max_iteration))
        {
            try
            {
                OS.sleep(1000);
                java.lang.System.out.println("waiting the ProcessMessage notification ");
            }
            catch(Exception ex)
            {
            }
            iteration++;
        }
        //if (supervisionObserverImpl.is_ok())
        performExitOperation();
        //ctx.exit(1);
        //ctx.notify_message ( 1 , "MessageFromProcessContext");
         
        //java.lang.System.out.println("ProcessContext notify_fatal_error");
        //ctx.notify_fatal_error ("Message from ProcessContext");
        //java.lang.System.out.println("ProcessContext notify_user_event");
    
    }
    
    public void performExitOperation(){
         java.lang.System.out.println("All verification on ProcessContext are OK, going to test ProcessContext::exit()");
         ctx.exit(1);
    }
    private  org.omg.CORBA.ORB orb;   

    private String sysName;
    private String appName;
    private String procName; 
    private com.thalesgroup.CdmwPlatformMngt.SupervisionObserver supervisionObserver;
    private SupervisionObserverImpl supervisionObserverImpl;
    private com.thalesgroup.CdmwPlatformMngt.ManagedElementContexts.ProcessContext ctx;
}
