/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMSIMU_SIM_PROCESS_MESSAGE_BROKER_IMPL_HPP
#define INCL_PLATFORMSIMU_SIM_PROCESS_MESSAGE_BROKER_IMPL_HPP

#include "Foundation/common/System.hpp"
#include "Foundation/orbsupport/CORBA.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtService.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtManagedElement.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtProcessCallback.skel.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp"



/**
 *Purpose:
 *<p> 
 *
 *Features:
 *<p> Thread safe
 *
 *@see  
 *
 */
class SimProcessMessageBroker_impl : 
  public POA_CdmwPlatformMngt::ProcessMessageBroker,
  public virtual PortableServer::RefCountServantBase
{

public:

  /**
   * Purpose:
   * <p> Constructor
   * 
   */ 
  SimProcessMessageBroker_impl()
    throw (CORBA::SystemException)
  {
  }
  
  /**
   * Purpose:
   * <p> Destructor
   * 
   */ 
  virtual 
  ~SimProcessMessageBroker_impl()
    throw()
  {
  };
  

  //
  // IDL:thalesgroup.com/CdmwPlatformMngt/ProcessMessageBroker/notify_user_event:1.0
  //
  virtual
  void
  notify_user_event(const char* issuer,
            const CORBA::Any& data)
    throw(CORBA::SystemException)
  {
    std::cout<<"ProcessMessageBroker::notify_user_event"<<"\n"<< "( " << issuer << ","
         << "data" << ")"<<std::endl;
  };
  
  //
  // IDL:thalesgroup.com/CdmwPlatformMngt/ProcessMessageBroker/notify_message:1.0
  //
  virtual
  void
  notify_message(const CdmwPlatformMngtBase::EventHeader& header,
         const char* issuer,
         const char* message)
    throw(CORBA::SystemException)
  {
    std::cout<<"ProcessMessageBroker::notify_message"<<"\n"<< "( {" << header.level << "},"
         << issuer << "," << message << ")"<<std::endl;
  };
  
  //
  // IDL:thalesgroup.com/CdmwPlatformMngt/ProcessMessageBroker/notify_error:1.0
  //
  virtual
  void
  notify_error(const CdmwPlatformMngtBase::TimeStamp& time_stamp,
           const char* issuer,
           const char* error_info)
    throw(CORBA::SystemException)
  {        
    std::cout<<"ProcessMessageBroker::notify_error"<<"\n"<< "("<< issuer << "," << error_info << ")"<<std::endl;
  };
    
  //
  // IDL:thalesgroup.com/CdmwPlatformMngt/ProcessMessageBroker/notify_process_error:1.0
  //
  virtual 
  void 
  notify_process_error (const char * failed_process_name,
            const CdmwPlatformMngtBase::TimeStamp & time_stamp,
            const char * issuer,
            const char * error_info)
    throw(CdmwPlatformMngt::ProcessNotFound,
      CORBA::SystemException)
  {
    std::cout<<"ProcessMessageBroker::notify_process_error"<<"\n"<<"("<< failed_process_name << "," 
         << issuer << "," << error_info << ")"<<std::endl;
  };
    
    
  //
  // IDL:thalesgroup.com/CdmwPlatformMngt/ProcessMessageBroker/get_service:1.0
  //
  virtual
  CORBA::Object_ptr 
  get_service (const char* service_name)
    throw (CdmwPlatformMngtService::ServiceNotFound,
       CdmwPlatformMngtService::ServiceUnavailable,
       CORBA::SystemException)
  {
    std::cerr<<"ProcessMessageBroker::get_service"<<"\n"<<"("<< service_name << ")"<<std::endl;

    std::string service_name_str =      service_name;
    CdmwPlatformMngt::System_ptr        system_ptr = CdmwPlatformMngt::System::_nil();
    CdmwPlatformMngt::Application_ptr   application_ptr = CdmwPlatformMngt::Application::_nil();
    CdmwPlatformMngt::Process_ptr       process_ptr = CdmwPlatformMngt::Process::_nil();
    CdmwPlatformMngt::Host_ptr          host_ptr = CdmwPlatformMngt::Host::_nil(); 
    
            
    if (service_name == "singleton")
      return system_ptr;
    
    unsigned int pos = service_name_str.find("PROC1.proc");
    if (pos!=std::string::npos)
      return process_ptr;

    pos = service_name_str.find("APPL1.appl");
    if (pos!=std::string::npos)
      return application_ptr;
        
    //std::string hostname = OsSupport::OS::get_hostname();
    pos = service_name_str.find(".host");
    if (pos!=std::string::npos)
      return host_ptr;  
        
    CORBA::Object_var obj = CORBA::Object::_nil();
    
    return obj._retn();
  }
    
    
  //
  // IDL:thalesgroup.com/CdmwPlatformMngt/ProcessMessageBroker/get_managed_element:1.0
  //
  virtual
  CdmwPlatformMngt::ManagedElement_ptr 
  get_managed_element (const char * element_path)
    throw (CdmwPlatformMngt::ElementPathInvalid,
       CdmwPlatformMngt::ManagedElementNotFound,
       CORBA::SystemException)
  {
    std::cout<<"ProcessMessageBroker::get_managed_element"<<"\n"<<"("<< element_path << ")"<<std::endl;
        
    CdmwPlatformMngt::ManagedElement_var element;
    
    return element._retn();
  }
  
  
  void notify_property_change (
      CdmwPlatformMngt::PropertyChange * property_change_event )
  throw (CORBA::SystemException)
  {
    std::cerr<< "SImMessageBroker_impl::notify_property_change CALLED!!!" <<std::endl;    
  }
    
protected:
    
private:
    
  /**
   * Purpose:
   * <p>  Copy constructor
   * 
   */ 
  SimProcessMessageBroker_impl (const SimProcessMessageBroker_impl& rhs)
    throw();
    
    
  /**
   * Purpose:
   * <p> Assignment operator
   * 
   */ 
  SimProcessMessageBroker_impl&
  operator=(const SimProcessMessageBroker_impl& rhs)
    throw();


  
}; // End class SimProcessMessageBroker_impl 


#endif // INCL_PLATFORMSIMU__SIM_PROCESS_MESSAGE_BROKER_IMPL_HPP

