/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.platformmngt.test;

import cdmw.testutils.Testable;
import cdmw.common.BadOrderException;
import cdmw.common.BadParameterException;

import cdmw.platformmngt.ProcessDelegateImpl;
import org.omg.CORBA.*;
import org.omg.PortableServer.*;
import cdmw.platformmngt.*;
import com.thalesgroup.CdmwPlatformMngt.ProcessCallbackPackage.*;
import cdmw.ossupport.OS;
import com.thalesgroup.CdmwPlatformMngt.PropertyChange;
import com.thalesgroup.CdmwPlatformMngt.RedundancyEltState;
import com.thalesgroup.CdmwPlatformMngt.RedundancyEltStateHelper;
//import com.thalesgroup.CdmwPlatformMngt.PropertyInfosHolder;
import com.thalesgroup.CdmwPlatformMngt.PropertyInfo;

import com.thalesgroup.CdmwPlatformMngt.REPLICATION_ROLE;
import com.thalesgroup.CdmwPlatformMngt.DEFAULT_RESTART_WARMTH;
import com.thalesgroup.CdmwPlatformMngt.TRACE_FILTERS ;
import com.thalesgroup.CdmwPlatformMngt.Owner;
import com.thalesgroup.CdmwPlatformMngt.RestartWarmth;
//import com.thalesgroup.CdmwPlatformMngt.*;
import com.thalesgroup.CdmwPlatformMngtBase.*;
//import cdmw.platformmngt.PropertyChangeFactoryImpl;

/**
 * The ProcessContext test.
 *
 */
public class ProcessContextTest extends Testable {

    private String[] args;

    public static int INITIAL = 0;
    public static int SETUP_PERFORMED = 1;
    public static int ACKNOWLEDGEMENT_PERFORMED = 2;

    public ProcessContextTest(String[] args) {
        this.out = System.out;
        this.name = "ProcessContext Test";
        this.args = args;
    }

    public void doTests() {

        // set number of requested successfull tests
        setNbOfRequestedTestOK(28);

        String eventIssuer = "An issuer";
        String entityName = "An entity name";
        UserProcessBehaviour behaviour = null;
    //  ProcessDelegateImpl process = null;
    
        out.println("Testing ProcessContext...");

        org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(
                               args, cdmw.orbsupport.ORBUtils.getProperties());


        org.omg.CORBA.Object obj = null;
        org.omg.PortableServer.POA poa = null ;

        try {
            obj = orb.resolve_initial_references("RootPOA");

            poa = org.omg.PortableServer.POAHelper.narrow(obj);
            poa.create_request_processing_policy(RequestProcessingPolicyValue.USE_ACTIVE_OBJECT_MAP_ONLY);
            poa.the_POAManager().activate();
        } catch (Exception e) {
            e.printStackTrace();
        }

        //com.thalesgroup.CdmwPlatformMngt.ProcessMessageBroker  message_broker_ref = null;
        org.omg.CORBA.Object obj_message_broker_ref = null;  

    // Create a ProcessMessageBroker Servant Object
        SimProcessMessageBrokerImpl message_broker = new SimProcessMessageBrokerImpl();
    com.thalesgroup.CdmwPlatformMngt.ProcessMessageBroker  message_broker_ref = null;

        try
        {

        String str_oid = new String ("processmessagebrokersimulated");
        byte[] oid = str_oid.getBytes();

        // create ior for ProcessMessageBroker object
        obj_message_broker_ref = poa.servant_to_reference(message_broker);
        System.out.println(orb.object_to_string(obj_message_broker_ref));
        
        message_broker_ref = com.thalesgroup.CdmwPlatformMngt.ProcessMessageBrokerHelper.narrow(obj_message_broker_ref);  
        // create ProcessStartingData
        ProcessStartingData process_start_data = new ProcessStartingData();
        
        process_start_data.system_name = "SYS1";
        process_start_data.application_name = "APPL1";
        process_start_data.process_name = "PROC1";
        process_start_data.process_message_broker = message_broker_ref;

        // Create ProcessCallback
        SimProcessCallbackImpl callbackServant_impl =
            new  SimProcessCallbackImpl(process_start_data);

        // create ior for ProcessCallback object
        org.omg.CORBA.Object obj_call_back = poa.servant_to_reference(callbackServant_impl);
        com.thalesgroup.CdmwPlatformMngt.ProcessCallback process_callback =
            com.thalesgroup.CdmwPlatformMngt.ProcessCallbackHelper.narrow(obj_call_back);
 
                // create ProcessBehaviour object
        behaviour = new UserProcessBehaviour();

        int length = args.length + 1;
        String[] tmp = new String[args.length +1];
        String[] args =  tmp;
        
        String callbackopt = new String("--CdmwCallbackIor=");
        String callbackIOR = orb.object_to_string (process_callback);
        callbackopt = callbackopt+callbackIOR;
        args[args.length - 1] = callbackopt;

        
        out.println("Calling acknowledgeCreation with a behaviour before setup...");

        out.println("Calling setup with callback IOR argument...");
        try {
            cdmw.platformmngt.PlatformInterface.setup(orb, args);
            succeed();
        } catch (BadParameterException bpe) {
            fail(); 
        } catch (BadOrderException boe) {
            fail();
        }

        try {
            cdmw.platformmngt.PlatformInterface.acknowledgeCreation(behaviour);
            succeed();
        } catch (BadOrderException boe) {
            fail(); 
        } catch (BadParameterException bpe) {
            fail();
        }
        }
    catch (Exception e)
        {
        System.out.println(e.toString());
        fail();
        }

        out.println("ProcessContext getService()");
        obj = null;
        try
        {
            obj = cdmw.platformmngt.PlatformInterface.getService(cdmw.platformmngt.PlatformInterface.MANAGED_PROCESS_CONTEXT_NAME);
        }
        catch (Exception ex)
        {
        out.println(ex.toString());
        fail();
        }
    
    out.println("ProcessContext resolve_initial_references()");
        obj = null;
        try
        {
            obj = orb.resolve_initial_references(cdmw.platformmngt.PlatformInterface.MANAGED_PROCESS_CONTEXT_NAME);
        }
        catch (Exception ex)
        {
        out.println(ex.toString());
        fail();
        }

        com.thalesgroup.CdmwPlatformMngt.ManagedElementContexts.ProcessContext ctx = com.thalesgroup.CdmwPlatformMngt.ManagedElementContexts.ProcessContextHelper.narrow(obj);
        ctx.notify_message(3,"*** MESSAGE OK ***");
        com.thalesgroup.CdmwPlatformMngt.External.System sys = ctx.the_system();
        out.println("ProcessContext check system proxy and name");
        succeed();
    if (ctx.system_name().equals("SYS1"))
        succeed(); else fail(); 
    out.println("ProcessContext check process proxy and name");    
        com.thalesgroup.CdmwPlatformMngt.External.Process proc = ctx.the_process();
        if (proc == null)
        succeed(); else fail();
        if (ctx.process_name().equals("PROC1"))
        succeed(); else fail();
    out.println("ProcessContext check host proxy and name");
        com.thalesgroup.CdmwPlatformMngt.External.Host host = ctx.the_host();
    if (host == null)
            succeed(); else fail();
    if (ctx.host_name().equals(OS.getHostname()))
        succeed(); else fail();
     out.println("ProcessContext check application proxy and name");    
        com.thalesgroup.CdmwPlatformMngt.External.Application app = ctx.the_application();
        if (app == null)
        succeed(); else fail();
    if (ctx.application_name().equals("APPL1"))
        succeed(); else fail();
        out.println("ProcessContext check managed element proxy and name");    
        com.thalesgroup.CdmwPlatformMngt.ManagedElement managed_element = ctx.the_managed_element();
    if (managed_element == null)
        succeed(); else fail();
    if (ctx.managed_element_name().equals("PROC1"))
            succeed(); else fail();   
        out.println("ProcessContext notify_message");
        ctx.notify_message ( 1 , "Message from ProcessContext");
        succeed();
    out.println("ProcessContext notify_fatal_error");
        ctx.notify_fatal_error ("Message from ProcessContext");
        succeed();
    out.println("ProcessContext notify_user_event");
        org.omg.CORBA.Any any_data = orb.create_any();
        any_data.insert_string("any_dummy");
        ctx.notify_user_event (any_data);
        succeed();
        out.println("ProcessContext the_redundancy_group");    
    try
        {
            ctx.the_redundancy_group();
        }
    catch(org.omg.CORBA.NO_IMPLEMENT ex)
        { 
        System.out.println(ex.toString());    
            succeed();
        }
        out.println("ProcessContext redundancy_group_name");    
        try
        {
        ctx.redundancy_group_name();
        }
        catch(org.omg.CORBA.NO_IMPLEMENT ex)
        {
            System.out.println(ex.toString());
            succeed();
        }

        out.println("ProcessContext get_process_redundancy_state()");   
        try
        {
        ctx.get_process_redundancy_state();
        }
        catch ( org.omg.CORBA.NO_IMPLEMENT ex)
        {
        System.out.println(ex.toString());
        succeed();
        }
        
        /////////////////////////////////////
        /// start with ComponentContext Test
        /////////////////////////////////////
        //out.println("check the ComponentContext");
        try
        {
           obj = orb.resolve_initial_references(cdmw.platformmngt.PlatformInterface.MANAGED_COMPONENT_CONTEXT_FACTORY_NAME);

        }
        catch (Exception e) {
            e.printStackTrace();
            fail();
        }
    
        com.thalesgroup.CdmwPlatformMngt.ManagedComponentContextFactory component_ctx_factory = com.thalesgroup.CdmwPlatformMngt.ManagedComponentContextFactoryHelper.narrow(obj);
        if (null == component_ctx_factory) fail();
        
        com.thalesgroup.CdmwPlatformMngt.ManagedElementContexts.ComponentContext component_ctx;
        try
        {
            component_ctx = component_ctx_factory.find_managed_component_context( "COMP1" ); 
            if (null == component_ctx) fail();
        
            sys = component_ctx.the_system();
            out.println("ComponentContext check system proxy and name");
            succeed();
            if (component_ctx.system_name().equals("SYS1"))
                succeed(); else fail(); 
            out.println("ComponentContext check process proxy and name");    
                proc = component_ctx.the_process();
                if (proc == null)
                    succeed(); else fail();
                if (component_ctx.process_name().equals("PROC1"))
                succeed(); else fail();
            out.println("ComponentContext check host proxy and name");
                host = component_ctx.the_host();
            if (host == null)
                    succeed(); else fail();
            if (component_ctx.host_name().equals(OS.getHostname()))
                succeed(); else fail();
             out.println("ComponentContext check application proxy and name");    
                app = component_ctx.the_application();
                if (app == null)
                succeed(); else fail();
            if (component_ctx.application_name().equals("APPL1"))
                succeed(); else fail();
                out.println("ComponentContext check managed element name");    
                /*managed_element = component_ctx.the_managed_element();
            if (managed_element == null)
                succeed(); else fail();*/
            if (component_ctx.managed_element_name().equals("COMP1"))
                    succeed(); else fail();
            

               out.println("ComponentContext the_redundancy_group");    
              if (component_ctx.the_redundancy_group() == null)
                succeed(); else fail();
              out.println("ComponentContext redundancy_group_name");    
              if (component_ctx.redundancy_group_name().equals(""))
                succeed(); else fail();

    
            component_ctx.notify_message ( 1 , "Message from ComponentContext");
            component_ctx.notify_fatal_error ("Message from ComponentContext");
            
            //NotifyUserEvent not implemented in platforminterface java
            //any_data.insert_string("any_dummy");
            //component_ctx.notify_user_event (any_data);        
            //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
            out.println("Check the ComponentContextExt");
            com.thalesgroup.CdmwPlatformMngt.ManagedElementContexts.ComponentContextExt component_ctx_ext;
            component_ctx_ext = component_ctx_factory.find_managed_component_context( "COMP1" );
            if (null == component_ctx_ext) fail();
            
            EventHeader header = cdmw.platformmngt.PlatformInterface.createHeader(INF.value);
            String element_path = component_ctx_ext.system_name() + 
                                  ".sys/"+component_ctx_ext.application_name()+
                                  ".appl/"+component_ctx_ext.host_name()+
                                  ".host/"+component_ctx_ext.process_name()+
                                  ".proc/"+component_ctx_ext.managed_element_name()+
                                  ".comp";
            
            org.omg.CORBA.Any old_value = org.omg.CORBA.ORB.init().create_any();
            RedundancyEltStateHelper.insert(old_value, RedundancyEltState.REDSTATE_PASSIVE);
            org.omg.CORBA.Any new_value = org.omg.CORBA.ORB.init().create_any();            
            RedundancyEltStateHelper.insert(new_value, RedundancyEltState.REDSTATE_PRIMARY);
            PropertyInfo property_info = new PropertyInfo( REPLICATION_ROLE.value, new_value, old_value );
            PropertyInfo[] property_infos = new PropertyInfo[1]; 
            property_infos[0] =  property_info;
            
            String info= "The redundancy state has changed";
            PropertyChangeFactoryImpl m_propertyChangeFactory = new cdmw.platformmngt.PropertyChangeFactoryImpl();
            com.thalesgroup.CdmwPlatformMngt.PropertyChange property_change =  
                m_propertyChangeFactory.create (header,element_path, property_infos, info);

            component_ctx_ext.notify_property_change (property_change);
            //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        }
        catch (com.thalesgroup.CdmwPlatformMngt.InvalidName e) {
                   e.printStackTrace();
                   fail();
        }
              
    }

}

