/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_EVENT_NOTIFIER_MANAGER_HPP
#define INCL_PLATFORMMNGT_EVENT_NOTIFIER_MANAGER_HPP

#include "platformlibrary/SupervisionEventNotifier.hpp"
#include "platformlibrary/StatusEventNotifier.hpp"
#include "platformlibrary/PropertyEventNotifier.hpp"
#include "platformlibrary/CommandEventNotifier.hpp"
#include <Foundation/common/Exception.hpp>

namespace Cdmw
{
namespace PlatformMngt
{
        /**
          *Purpose: EventNotifierManager implementation
          *<p>
          * Singleton class to provide set and get method for the 
          * SupervisionEventNotifier, StatusEventNotifier and PropertyEventNotifier.
          * EventNotifierManager has the ownership SupervisionEventNotifier StatusEventNotifier 
          * and PropertyEventNotifier threads 
          * EventNotifierManager starts and stops SupervisionEventNotifier StatusEventNotifier 
          * and PropertyEventNotifier threads
          */
    
        class EventNotifierManager{
        
            public:

                /**
                 * Provide the singleton instance 
                 */
                static EventNotifierManager*    Instance();
                
                /**
                 * Get the instance of CommandEventNotifier
                 * @return: Cdmw::PlatformMngt::CommandEventNotifier*
                 */
                CommandEventNotifier*          get_command_notifier() const;

                /**
                 * Get the instance of PropertyEventNotifier
                 * @return: Cdmw::PlatformMngt::PropertyEventNotifier*
                 */
                PropertyEventNotifier*          get_property_notifier() const;
                
                /**
                 * Get the instance of StatusEventNotifier
                 * @return: Cdmw::PlatformMngt::StatusEventNotifier*
                 */
                StatusEventNotifier*       get_status_notifier() const;

                /**
                 * Get the instance of SupervisionEventNotifier
                 * @return: Cdmw::PlatformMngt::SupervisionEventNotifier*
                 */
                SupervisionEventNotifier*       get_sup_notifier() const;

                
                /**
                 * Set the CommandEventNotifier instance in the CommandNotifierManager
                 * @parameter : CommandEventNotifier*
                 */
                void set_command_notifier(CommandEventNotifier* command_notifier) throw (BadParameterException);
                
                /**
                 * Set the PropertyEventNotifier instance in the EventNotifierManager
                 * @parameter : PropertyEventNotifier*
                 */
                void set_property_notifier(PropertyEventNotifier* property_notifier) throw (BadParameterException);

                /**
                 * Set the StatusEventNotifier instance in the EventNotifierManager
                 * @parameter : StatusEventNotifier*
                 */
                void set_status_notifier(StatusEventNotifier* status_notifier) throw (BadParameterException);


                /**
                 * Set the SupervisionEventNotifier instance in the EventNotifierManager
                 * @parameter : SupervisionEventNotifier*
                 */
                void set_sup_notifier(SupervisionEventNotifier* sup_notifier) throw (BadParameterException);

            private:

                /**
                 * Private Constructor 
                 */
                EventNotifierManager();
                
                /**
                 * Copy Constructor
                 */
                EventNotifierManager(const EventNotifierManager& );

                /**
                 * Assignement Operator
                 */
                EventNotifierManager& operator=(const EventNotifierManager& );
    
                /**
                 * Destructor
                 */
                ~EventNotifierManager();

            private:
                // CommandEventNotifier thread instance
                CommandEventNotifier*          m_command_notifier;
                // PropertyEventNotifier thread instance
                PropertyEventNotifier*          m_property_notifier;
                // SupervisionEventNotifier thread instance
                SupervisionEventNotifier*       m_sup_notifier;
                // StatusEventNotifier thread instance
                StatusEventNotifier*       m_status_notifier;

        };
};
};

#endif



