/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_LBGROUPSITERATOR_IMPL_HPP
#define INCL_PLATFORMMNGT_LBGROUPSITERATOR_IMPL_HPP

#include <Foundation/orbsupport/CORBA.hpp>
//#include "SystemMngt/idllib/CdmwPlatformMngtEntityContainer.skel.hpp"
#include <SystemMngt/platformlibrary/Iterator.hpp>


namespace Cdmw
{
namespace PlatformMngt
{


/**
*Purpose:
*<p>
*Implementation of IDL:thalesgroup.com/CdmwPlatformMngt/LBGroupsIterator:1.0 interface.
*<p>
*Features:
*<p>
*Thread safe.
*
*@see Iterator.
*/
class LBGroupsIterator_impl : virtual public POA_CdmwPlatformMngt::LBGroupsIterator,
                              virtual public PortableServer::RefCountServantBase
{

public:

    /**
    * Purpose:
    * <p>
    * implements the
    * IDL:thalesgroup.com/CdmwPlatformMngt/LBGroupsIterator/reset:1.0
    * operation
    */
    virtual void reset()
        throw(CORBA::SystemException)
    {
    //throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
        m_delegate.reset();
    }


    /**
    * Purpose:
    * <p>
    * implements the
    * IDL:thalesgroup.com/CdmwPlatformMngt/LBGroupsIterator/next_one:1.0
    * operation
    */
    virtual CORBA::Boolean next_one(CdmwPlatformMngt::LBGroup_out lbgroup)
        throw(CORBA::SystemException)
    {
    //throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
        return m_delegate.next_one(lbgroup);
    }


    /**
    * Purpose:
    * <p>
    * implements the
    * IDL:thalesgroup.com/CdmwPlatformMngt/LBGroupsIterator/next_n:1.0
    * operation
    */
    virtual CORBA::Boolean next_n(CORBA::ULong how_many,
                                  CdmwPlatformMngt::LBGroups_out lbgroups)
        throw(CORBA::SystemException)
    {
    //throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
        return m_delegate.next_n(how_many, lbgroups);
    }


    /**
    * Purpose:
    * <p>
    * implements the
    * IDL:thalesgroup.com/CdmwPlatformMngt/LBGroupsIterator/destroy:1.0
    * operation
    */
    virtual void destroy()
        throw(CORBA::SystemException)
    {
    //throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
        m_delegate.destroy();
    }


    /**
    * The destructor.
    */
    virtual ~LBGroupsIterator_impl() {}


public:

    /**
    * Purpose:
    * <p>
    * Creates the iterator servant, activates this servant, and returns the corresponding
    * CORBA object.
    *
    * @param elementList The list of elements referenced by the created iterator.
    * @return   The reference to the CORBA object
    */
    static CdmwPlatformMngt::LBGroupsIterator_ptr create(
        const std::list<CdmwPlatformMngt::LBGroup_var> elementList)
        throw (OutOfMemoryException, InternalErrorException)
    {

        try
        {
            PortableServer::ServantBase_var servant = new LBGroupsIterator_impl(elementList);
            LBGroupsIterator_impl *servant_impl =
                dynamic_cast<LBGroupsIterator_impl*>(servant.in());

            CORBA::Object_var object =
                IteratorManager::createIterator(servant.in(), &(servant_impl->m_delegate));

            return CdmwPlatformMngt::LBGroupsIterator::_narrow(object.in());

        }
        catch(const std::bad_alloc &)
        {
            CDMW_THROW(OutOfMemoryException);
        }
        catch(const CORBA::SystemException &)
        {
            CDMW_THROW(InternalErrorException);
        }

    }


private:

    /**
    * The actual implementation of the iterator.
    */
    Iterator<CdmwPlatformMngt::LBGroup,
        CdmwPlatformMngt::LBGroup_var,
        CdmwPlatformMngt::LBGroup_out,
        CdmwPlatformMngt::LBGroups,
        CdmwPlatformMngt::LBGroups_var,
        CdmwPlatformMngt::LBGroups_out,
        InterfaceElementAllocator<CdmwPlatformMngt::LBGroup,
                                  CdmwPlatformMngt::LBGroup_out>
                                 > m_delegate;

private:

    /**
    * Purpose:
    * <p>
    * Constructor.
    */
    LBGroupsIterator_impl(const std::list<CdmwPlatformMngt::LBGroup_var> elementList)
        : m_delegate(elementList)
    {
    }



}; //End of LBGroupsIterator_impl

} // End of namespace PlatformMngt
} // End of namespace Cdmw

#endif

