/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_PROPERTY_CHANGE_ASPECT_HPP
#define INCL_PLATFORMMNGT_PROPERTY_CHANGE_ASPECT_HPP

#include <Foundation/common/Exception.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtEvent.stub.hpp>

#include <string>
#include <map>
#include <vector>


namespace Cdmw
{
 namespace PlatformMngt
 {
// internal usage
 class PropertyChangeAspect
 {
  std::string m_sObservername,m_target_element_path;
  std::vector<std::string>  m_setproperty;
  bool m_bregforall;
    
 public:
  // constructors
  PropertyChangeAspect();
 
  PropertyChangeAspect(const std::string& sobname,CdmwPlatformMngt::PropertyInfos& propertyset);
   
  ~PropertyChangeAspect();
 

  void  set_obsname(char* lpszobsname);
  const char* get_obsname() const;

  void  set_targetelement_path(char* lpsztargetelement_path);
  const char* get_targetelement_path() const;

  void  set_propertyset(const CdmwPlatformMngt::PropertyInfos& propertyset);
  void  set_propertyset(const CosPropertyService::PropertyNames& property_names);

  std::vector<std::string>* get_propertyset();

  void  add_property(std::string& sname);

  // operators
  bool operator==(const PropertyChangeAspect& sright) const;
  bool operator>(const PropertyChangeAspect& sright) const;
  bool operator<(const PropertyChangeAspect& sright) const;
  PropertyChangeAspect& operator=(const PropertyChangeAspect& rhs);
  PropertyChangeAspect& operator-=(const PropertyChangeAspect& rhs);
     
  // register for all properties methods
  inline void set_register_for_all_properties(bool bset);
  inline bool is_register_for_all_properties() const;
  // control if property event is present in defined set
  bool check(const CdmwPlatformMngt::PropertyInfos& control_sequence_properties);
 };


 inline std::vector<std::string>* PropertyChangeAspect::get_propertyset()
 {
  return &m_setproperty;
 }

 inline  void  PropertyChangeAspect::add_property(std::string& sname)
 {
    m_setproperty.push_back(sname);
 }


 inline void  PropertyChangeAspect::set_register_for_all_properties(bool bset)
 {
  m_bregforall=bset;
 }

 inline  bool  PropertyChangeAspect::is_register_for_all_properties() const 
 {
  return m_bregforall;
 }

 inline void  PropertyChangeAspect::set_obsname(char* lpszobsname)
 {
  m_sObservername=lpszobsname;
 }

 inline const char* PropertyChangeAspect::get_obsname() const
 {
  return m_sObservername.c_str();
 }

 inline void  PropertyChangeAspect::set_targetelement_path(char* lpsztargetelement_path)
 {
  m_target_element_path=lpsztargetelement_path;
 }

 inline const char* PropertyChangeAspect::get_targetelement_path() const
 {
  return m_target_element_path.c_str();
 }

 inline void PropertyChangeAspect::set_propertyset(const CdmwPlatformMngt::PropertyInfos& propertyset)
 {
  // transfer CORBA sequence into vector<string> of class
  if(!m_setproperty.empty())
    m_setproperty.clear();

  for (unsigned int i=0; i<propertyset.length();i++)
    m_setproperty.push_back( propertyset[i].property_name.in());
 }

 inline void PropertyChangeAspect::set_propertyset(const CosPropertyService::PropertyNames& property_names)
 {
  // transfer CORBA sequence into vector<string> of class
  if(!m_setproperty.empty())
    m_setproperty.clear();

  for (unsigned int i=0; i<property_names.length();i++)
    m_setproperty.push_back( property_names[i].in());
 }

 
 inline  PropertyChangeAspect& PropertyChangeAspect::operator-=(const PropertyChangeAspect& rhs)
 {
    
    std::vector<std::string>::iterator iter;
    
    for (unsigned int i=0; i<rhs.m_setproperty.size();i++)
    { 
        iter = find(m_setproperty.begin(),m_setproperty.end(),rhs.m_setproperty[i]);
        if (iter!=m_setproperty.end())
              m_setproperty.erase(iter);
            
    }
    return (*this);
        
 
 
 }
 } // End namespace PlatformMngt
} // End namespace Cdmw

#endif // !INCL_PLATFORMMNGT_PROPERTY_CHANGE_ASPECT_HPP
