/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_SYNCHRONISABLETIMEDTASK_HPP 
#define INCL_PLATFORMMNGT_SYNCHRONISABLETIMEDTASK_HPP 

#include <SystemMngt/platformlibrary/SynchronisableTaskFactory.hpp>
#include <SystemMngt/platformlibrary/Sequencer.hpp>
#include <SystemMngt/platformlibrary/Starter.hpp>
#include <SystemMngt/tools/SingleShotTimer.hpp>
#include <SystemMngt/tools/TimeoutObserver.hpp>


namespace Cdmw
{
namespace PlatformMngt
{

class SynchronisableTimedTaskObserver;


/**
*Purpose:
*<p> extends the Synchronisable task, making the task timed too  
*/

class SynchronisableTimedTask : public SynchronisableTask
{


    public:

        /**
        * Purpose:
        * @param timeout           the timeout of the Timer. 
        * @param taskId            the identifier of the task.
        * @param  observer         the task observer
        * @param  predecessorCount the number of predecessor tasks
        * @param sequencer         the sequencer of the created task.
        * @param syncTaskFactory   the factory used to create the task.
        * <p> Constructor 
        */ 
        SynchronisableTimedTask(unsigned long timeout,
                                     const std::string& id,
                                     Sequencer* sequencer,
                                     SynchronisableTaskFactory* synchTaskFactory,
                                     TaskObserver* observer,
                                     size_t predecessorCount);

        /**
        * Purpose:
        * <p> Destructor.
        */ 
        virtual ~SynchronisableTimedTask() throw();
              
         /**
         *Purpose:
         *<p> Decrements the number of predecessors to wait.
         * Starts actually the task if there is no more predecessors
         * and a timer and its observer, which calls a callback when the timer is expired.
         */  
        virtual void startTask() throw();
              
        /**
         *Purpose:
         *<p> Usefull to know if the timer is already expired
         */
        bool isExpired() throw();

        
    private:
          
        /**
        * Purpose:
        * <p>
        * Copy constructor is not allowed.
        */ 
        SynchronisableTimedTask(const SynchronisableTimedTask& rhs);

        /**
        * Purpose:
        * <p>
        * Assignment operator is not allowed.
        */ 
        SynchronisableTimedTask& operator=(const SynchronisableTimedTask& rhs);
        

    private:

        /**
         * The TimeoutObserver
         */
         SynchronisableTimedTaskObserver* m_observer ;
         
        /**
         * The timer 
         */
        SingleShotTimer* m_timer;
        
        /**
         * The timeout value
         */
        unsigned long  m_timeout;
        


}; // End SynchronisableTaskTemporiser

class SynchronisableTimedTaskObserver : public TimeoutObserver
{ 
    
    friend class SynchronisableTimedTask;
    
    public:
    
        /**
        *Purpose:
        *<p> Constructor.
        */
        SynchronisableTimedTaskObserver (const std::string& id,
                                           Sequencer* sequencer,
                                           SynchronisableTaskFactory* syncTaskFactory);                
         
        //destructor
        virtual ~SynchronisableTimedTaskObserver() { } ;
        
        // callback method to call when the Timeout will be expired.
        virtual void wake_up() throw (); 
     

    private:
        
        /*
         * a flag used to know if the timer it's expired 
         */
        bool expired ;

         /**
         * The taskId
         */
        std::string m_taskId;
        
        /**
         * The sequencer.
         */
        Sequencer* m_sequencer;
        
        /**
         * The TaskFactory
         */
        SynchronisableTaskFactory* m_taskFactory;
        
        
}; // End SynchronisableTaskTimeoutObserver

}//End Namespacing
}

#endif // INCL_PLATFORMMNGT_SYNCHRONISABLETIMEDTASK_HPP





