
/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "platformlibrary/EventNotifierManager.hpp"

namespace Cdmw
{
namespace PlatformMngt
{
    EventNotifierManager::EventNotifierManager(){
        m_command_notifier = 0;
        m_property_notifier = 0;
        m_status_notifier   = 0;
        m_sup_notifier   = 0;
    }    

    EventNotifierManager* EventNotifierManager::Instance()
    {
        static EventNotifierManager event_notifier_manager;
        return &event_notifier_manager;
    }

    CommandEventNotifier* EventNotifierManager::get_command_notifier() const
    {
        return m_command_notifier;
    }
    
    PropertyEventNotifier* EventNotifierManager::get_property_notifier() const
    {
        return m_property_notifier;
    }

    SupervisionEventNotifier* EventNotifierManager::get_sup_notifier() const
    {
        return m_sup_notifier;
    }

    StatusEventNotifier* EventNotifierManager::get_status_notifier() const
    {
        return m_status_notifier;
    }

    void EventNotifierManager::set_command_notifier(CommandEventNotifier* command_notifier) throw (BadParameterException)
    {
        // if m_command_notifier is already assigned throw BadParameterException 
        // avoid the assignement to more than once 
        if (m_command_notifier)
             CDMW_THROW2(BadParameterException,"m_command_notifier","Already assigned");
        // if m_property_notifier is 0 throw BadParameterException
        if (0 == command_notifier ) 
             CDMW_THROW2(BadParameterException, "property_notifier","0");
        m_command_notifier = command_notifier;
        m_command_notifier->start();
    }
    
    void EventNotifierManager::set_property_notifier(PropertyEventNotifier* property_notifier) throw (BadParameterException)
    {
        // if m_property_notifier is already assigned throw BadParameterException 
        // avoid the assignement to more than once 
        if (m_property_notifier)
             CDMW_THROW2(BadParameterException,"m_property_notifier","Already assigned");
        // if m_property_notifier is 0 throw BadParameterException
        if (0 == property_notifier ) 
             CDMW_THROW2(BadParameterException, "property_notifier","0");
        m_property_notifier = property_notifier;
        m_property_notifier->start();
    }


    void EventNotifierManager::set_sup_notifier(SupervisionEventNotifier* sup_notifier) throw (BadParameterException)
    {
        // if m_status_notifier is already assigned throw BadParameterException
        // avoid the assignement to more than once
        if (m_sup_notifier)
            CDMW_THROW2(BadParameterException,"m_status_notifier","Already assigned");
        if (0 == sup_notifier )
            CDMW_THROW2(BadParameterException, "status_notifier","0");
        m_sup_notifier = sup_notifier;
        m_sup_notifier->start();
    }

   void EventNotifierManager::set_status_notifier(StatusEventNotifier* status_notifier) throw (BadParameterException)
    {
        // if m_status_notifier is already assigned throw BadParameterException
        // avoid the assignement to more than once
        if (m_status_notifier)
            CDMW_THROW2(BadParameterException,"m_status_notifier","Already assigned");
        if (0 == status_notifier )
            CDMW_THROW2(BadParameterException, "status_notifier","0");
        m_status_notifier = status_notifier;
        m_status_notifier->start();
    }

    EventNotifierManager::~EventNotifierManager(){
        
        // delete threads (destructors stop the threads)   
        delete m_command_notifier;
        delete m_property_notifier;
        delete m_status_notifier;
        delete m_sup_notifier;
    }

}   //End of namespace PlatformMngt
}   //End of namespace Cdmw


