/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "platformlibrary/PropUtils.hpp"
#include <vector>

namespace Cdmw
{
  namespace PlatformMngt
  {

    PropUtils::PropUtils()
    {
    }

    PropUtils::~PropUtils()
    {
    }

    PropUtils& PropUtils::operator-= (const PropUtils & rhs)
    {

      std::vector < std::string >::iterator iter;

      for (unsigned int i = 0; i < rhs.m_setproperty.size (); i++)
    {
      iter =
        find (m_setproperty.begin (), m_setproperty.end (),
          rhs.m_setproperty[i]);
      if (iter != m_setproperty.end ())
        m_setproperty.erase (iter);

    }
      return (*this);
    }


    void PropUtils::set_propertyset (const CosPropertyService::
                        PropertyNames & property_names)
    {
      // transfer CORBA sequence into vector<string> of class
      if (!m_setproperty.empty ())
    m_setproperty.clear ();

      for (unsigned int i = 0; i < property_names.length (); i++)
    m_setproperty.push_back (property_names[i].in ());
    }

    
    void PropUtils::set_propertyset (const std::vector<std::string>& property_names)
    {
      // transfer CORBA sequence into vector<string> of class
      if (!m_setproperty.empty ())
        m_setproperty.clear ();

      for (unsigned int i = 0; i < property_names.size (); i++)
        m_setproperty.push_back (property_names[i]);
    }


    inline std::vector < std::string >
      *PropUtils::get_propertyset (const CosPropertyService::
                   PropertyNames & property_names)
    {
      set_propertyset (property_names);
      return &m_setproperty;
    }

    std::vector < std::string >
      *PropUtils::get_propertyset ()
    {
      return &m_setproperty;
    }
    

    void PropUtils::update_keys (MapObsForProp& map_obs_for_prop, const std::string& element_path,
                 const std::vector<std::string>* prop_names, const ObserverInfo & obs_info) 
        throw (CdmwPlatformMngt::ElementAlreadyExists)
    {
        PropObsKey prop_key;
        prop_key.set_element_path(element_path);
        for (CORBA::ULong i = 0; i < prop_names->size (); i++)
        { 
            prop_key.set_property ((*prop_names)[i]);
            CdmwPlatformMngt::SupervisionObserver::_duplicate(obs_info.observer_ptr); 

            // check if the pair <key,value> already exists, if not insert it into the multimap
            bool element_exists = false;
            std::pair<MapObsForProp::iterator, MapObsForProp::iterator> itp = 
                map_obs_for_prop.equal_range(prop_key);
            for (MapObsForProp::iterator it = itp.first; it != itp.second; ++it) {
                if ((*it).second.observer_name == obs_info.observer_name) {
                    element_exists = true;
                    break;    
                }
            }
            if (!element_exists) {
                map_obs_for_prop.insert (MapObsForProp::value_type (prop_key, obs_info));    
            }
            else
                throw CdmwPlatformMngt::ElementAlreadyExists();
        }
    }
    
    std::vector <std::string>* PropUtils::get_property_names_in_map (MapObsForProp & map_obs_for_prop,
                                   const std::string & element_path)
    {
      std::vector < std::string >* vect_prop_names_in_map =   new std::vector < std::string >;

      PropObsKey prop_key;
      for (MapObsForProp::iterator  it_prop =  map_obs_for_prop.begin(); 
        it_prop != map_obs_for_prop.end(); 
        it_prop++)         
      {
       prop_key=it_prop->first;     
       std::cerr <<" prop_key.get_element_path"  <<prop_key.get_element_path() << std::endl;
       if (prop_key.get_element_path() ==  element_path)
        {
            std::cerr << "PropUtils::get_property_names_in_map"<< prop_key.get_property() << std::endl; 
            std::vector <std::string>::iterator it_vect = find (vect_prop_names_in_map->begin(),
                                                                            vect_prop_names_in_map->end(),
                                                                            prop_key.get_property ());
                        if (it_vect == vect_prop_names_in_map->end())
                {
                                vect_prop_names_in_map->push_back (prop_key.get_property ());
                }
        } 
      }
      return vect_prop_names_in_map;
    }

    void PropUtils::print_map_for_all(MapObsForAll & map_obs_for_all)
    {
    std::cerr <<std::endl <<"["<< std::endl;
    std::string str = " , " ;
    for (MapObsForAll::iterator it = map_obs_for_all.begin(); it!= map_obs_for_all.end(); it++)
    {
        std::cerr << "KEY = ("<<it->first << ") VALUE = (" << it->second.observer_name << str << it->second.observer_ptr<< ")" <<std::endl;
    }    
        std::cerr << "]" << std::endl;
    }

    void PropUtils::print_map_for_prop(MapObsForProp & map_obs_for_prop)
    {
    std::cerr << std::endl <<"["<< std::endl;
    std::string str = " , " ;
    PropObsKey prop_obs_key;
    for (MapObsForProp::iterator it = map_obs_for_prop.begin(); it!=map_obs_for_prop.end();it++ )    
    {    
        prop_obs_key=it->first;
        std::cerr << "KEY = ("<< prop_obs_key.get_element_path()<< ","<<prop_obs_key.get_property()<< ") VALUE = (" << it->second.observer_name << str << it->second.observer_ptr<< ")" <<std::endl;
    }    
    std::cerr << "]" << std::endl;
    }

    bool PropUtils::check_obs_name_in_map_for_prop(MapObsForProp& map_for_prop ,const std::string& observer_name,const PropObsKey& key)
    {
       bool bool_ret = false;
           
       MapObsForProp::iterator it;
       it = map_for_prop.find(key);
       int count = map_for_prop.count(key);     
       for (int i=0; i<count; i++ )
       {     
           std::string observer_name_in_map =it->second.observer_name;
       if (observer_name == observer_name_in_map)
            bool_ret=true;    
        it++;
        }
    return bool_ret;
    }

    bool PropUtils::check_obs_name_in_map_for_all(MapObsForAll& map_for_all,
        const std::string& observer_name, const std::string& element_path)
//    {
//        bool bool_ret = false;
//        
//        MapObsForAll::iterator it;
//        it = map_for_all.find(element_path);
//        int count = map_for_all.count(element_path);
//        for (int i=0; i<count; i++)
//        {
//            std::string observer_name_in_map = it->second.observer_name;
//            if (observer_name == observer_name_in_map ) {
//                bool_ret = true;
//                break;
//            }
//            it ++;
//        }
//        return bool_ret;
//    }   
    {
        bool bool_ret = false;
        
        MapObsForAll::iterator it;
        std::pair<MapObsForAll::iterator, MapObsForAll::iterator> itp = 
                map_for_all.equal_range(element_path);
        for (MapObsForAll::iterator it = itp.first; it != itp.second; ++it) {
            if ((*it).second.observer_name == observer_name) {
                bool_ret = true;
                break;    
            }
        }
        return bool_ret;
    }   

}
}
