/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "platformlibrary/PropertyChangeAspect.hpp"

// STL SUPPORT
#include <string>
#include <map>

using namespace std;
using namespace Cdmw::PlatformMngt;

// constructors
PropertyChangeAspect::PropertyChangeAspect()
{
    m_bregforall=false;
    m_sObservername="";
    m_target_element_path="";
}
       
PropertyChangeAspect::PropertyChangeAspect(const string& sobname,CdmwPlatformMngt::PropertyInfos& propertyset)
{
    m_bregforall=false;
    m_sObservername=sobname;
    set_propertyset(propertyset);
}
      
PropertyChangeAspect::~PropertyChangeAspect()
{

}

bool 
PropertyChangeAspect::operator==(const PropertyChangeAspect& sright) const
{
    bool bisegual=false;
    if ( m_sObservername == sright.m_sObservername)
    bisegual=true;
    return bisegual;
}

bool PropertyChangeAspect::operator>(const PropertyChangeAspect& sright) const
{
    return ( m_sObservername > sright.m_sObservername) ? true : false;

}

bool PropertyChangeAspect::operator<(const PropertyChangeAspect& sright) const
{
    return ( m_sObservername < sright.m_sObservername) ? true : false;
}

PropertyChangeAspect& PropertyChangeAspect::operator=(const PropertyChangeAspect& rhs)
{
    m_setproperty.clear();
    m_sObservername= rhs.m_sObservername;
    m_bregforall= rhs.m_bregforall;
    m_target_element_path=rhs.m_target_element_path;


    for ( unsigned int i=0;i<rhs.m_setproperty.size();i++)
    m_setproperty.push_back(rhs.m_setproperty.at(i));

    return *this;
}

bool PropertyChangeAspect::check(const CdmwPlatformMngt::PropertyInfos& control_sequence_properties)
{
    bool bret=false;
    string sproperty_name,sgetpropertyname;

    CdmwPlatformMngt::PropertyInfo popProperty;

    if(m_bregforall)
    return true; // register for all properties ? i don't check it !

    // extract each property and verify if it content in the registered list
    for ( unsigned long li=0;li<control_sequence_properties.length();li++)
    {
    popProperty = control_sequence_properties[li];
    sproperty_name = popProperty.property_name.in();

    for (unsigned int i=0; i< m_setproperty.size();i++)
    {

        sgetpropertyname = m_setproperty[i];
        if(sproperty_name == sgetpropertyname)
        {
        bret=true;
        break;
        }
    }

    if(bret) // need only one match case
        break;
    }
    return bret;
}
