/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_PERIODIC_NOTIFIER_HPP
#define INCL_PLATFORMMNGT_PERIODIC_NOTIFIER_HPP

#include <string>
#include <map>
#include <cstdlib>
#include <vector>

#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/osthreads/Mutex.hpp"

#include "SystemMngt/idllib/CdmwPlatformMngtPeriodicNotifier.skel.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtPeriodicObserver.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "platformperiodicnotifier/TimerObserver.hpp"
#include "SystemMngt/platformlibrary/LogMngr.hpp"
#include "SystemMngt/platformlibrary/Configuration.hpp"
#include "SystemMngt/platformelementlibrary/DataStoreDefinition.hpp"
#include "SystemMngt/tools/ManagedElementKey.hpp"


namespace Cdmw
{ // Begin namespace Cdmw
    
namespace PlatformMngt
{ // Begin namespace PlatformMngt
    
namespace Periodic
{ // Begin namespace Periodic 
    

class Notifier_impl: public virtual POA_CdmwPlatformMngt::Periodic::Notifier,
                    public virtual PortableServer::RefCountServantBase
{

    
public:
    
    typedef std::multimap < std::string, TimerObserver* > MapObs ;
    
    
    /**
    * Purpose:
    * <p> Constructor.
    */
    Notifier_impl(CORBA::ORB_ptr orb,
                CdmwPlatformMngt::System_ptr system)
    throw (Cdmw::OutOfResourcesException);
    
    /**
    * Purpose:
    * <p> Destructor.
    */
    ~Notifier_impl() throw();
    
    
    /**
    *Purpose:
    *<p>
    * Implements the
    * IDL:thalesgroup.com/CdmwPlatformMngt/Periodic/Notifier/register_observer:1.0
    * operation
    */    
    void register_observer (const char * observer_name, 
                            const CosPropertyService::PropertyNames & property_names, 
                            CdmwPlatformMngt::Periodic::Observer_ptr observer,
                            TimeBase::TimeT period)
        throw (CORBA::SystemException,
            CdmwPlatformMngt::ElementAlreadyExists, 
            CdmwPlatformMngt::PropertyNameInvalid);
    
    
    /**
    *Purpose:
    *<p>
    * Implements the
    * IDL:thalesgroup.com/CdmwPlatformMngt/Periodic/Notifier/register_observer_group:1.0
    * operation
    */    
    void register_observer_group (const char * observer_name, 
                                const CosPropertyService::PropertyNames & property_names,
                                const char * miop_group, 
                                TimeBase::TimeT period)
        throw (CORBA::SystemException, 
            CdmwPlatformMngt::ElementAlreadyExists, 
            CdmwPlatformMngt::PropertyNameInvalid);  
    
    
    /**
    *Purpose:
    *<p>
    * Implements the
    * IDL:thalesgroup.com/CdmwPlatformMngt/Periodic/Notifier/unregister_observer:1.0
    * operation
    */                
    CdmwPlatformMngt::Periodic::Observer_ptr unregister_observer (const char * observer_name)
        throw (CORBA::SystemException, 
            CdmwPlatformMngt::ObserverNotFound);
    
    
    /**
    *Purpose:
    *<p>
    * Implements the
    * IDL:thalesgroup.com/CdmwPlatformMngt/Periodic/Notifier/unregister_observer_group:1.0
    * operation
    */           
    char * unregister_observer_group (const char * observer_name)
        throw (CORBA::SystemException, 
            CdmwPlatformMngt::ObserverNotFound);
    
    
    /**
    *Purpose:
    *<p>
    * Implements the
    * IDL:thalesgroup.com/CdmwPlatformMngt/Periodic/Notifier/periodic_notifier_id:1.0
    * operation
    */                
    char * periodic_notifier_id() throw (CORBA::SystemException);
    
    /**
    *Purpose:
    *<p> Synchronise with datastore
    *
    */
    void notify_registered_observers();
    
    /**
    *Purpose:
    *<p> Set true the boolean variable which means that 
    * at least one record is inserted in datastore 
    *
    */
    void define_ds();
    
    /**
    *Purpose:
    *<p> Update the sequence of all registered periodic observer into memory  
    *  @param pd_observers_infos the reference of periodic observer's sequence
    */
    void update_observers_infos(const CdmwPlatformMngt::PeriodicObserverRecords & pd_observers_infos);
    
    /**
    *Purpose:
    *<p> Set the periodic observer record
    * 
    * @param observer_name the name of observer
    * @param property_names the sequence of property names
    * @param observer_ptr the observer reference
    * @param period the notification period
    * @param miop_group the multicast reference (present only for periodic observer group) 
    */
    void 
    set_observer_info(const char * observer_name, 
                    const CosPropertyService::PropertyNames & property_names,
                    CdmwPlatformMngt::Periodic::Observer_ptr observer_ptr,
                    TimeBase::TimeT period, 
                    const char * miop_group);

    void 
    set_observer_info(const char * observer_name, 
                    const CosPropertyService::PropertyNames & property_names,
                    CdmwPlatformMngt::Periodic::Observer_ptr observer_ptr,
                    TimeBase::TimeT period);
                    
private:

    /**
    *Purpose:
    *<p> Copy constructor is not allowed.
    */
    Notifier_impl(
        Notifier_impl& rhs );

    /**
    * Purpose:
    * <p> Assignment operator is not allowed.
    */ 
    Notifier_impl& operator=(
        const Notifier_impl& rhs );
    
    /**
    * The Logger pointer
    */
    Cdmw::Logging::Logger * m_logger_pointer;    
    
    /**
    * Purpose:
    * <p> The ORB
    */
    CORBA::ORB_var          m_orb;

    /*
    * Unique ID for periodic notifier
    */
    static
    std::string M_periodic_notifier_id ;

    /**
    * Map of the registered observer 
    */
    MapObs m_periodic_observers_map;

    /*
    * System object
    */
    CdmwPlatformMngt::System_var m_system;
    
    /*
    * The observer of timer
    */
    //TimerObserver* m_timer_obs;
                    
    /**
    * Periodic observer datastore
    */
    PeriodicObserverDataStore* m_periodic_observer_ds;  

    
    /**
    * The periodic datastore is created
    */
    bool m_ds_defined;
    
    /**
    * The mutex used register and unregister observer guard.
    */
    OsSupport::Mutex m_mutex_condition;
    
    /**
    * The PeriodicObserverRecords_var  used to store infos about periodic
    * observer registration
    */
    CdmwPlatformMngt::PeriodicObserverRecords_var m_pd_observers_infos;
    
    /**
    *Purpose:
    *<p> Set the periodic observer record in data store
    */
    void set_observer_record_in_datastore();

    /**
    *Purpose:
    *<p> Remove the periodic observer record in datastore
    *
    */
    void remove_record_in_datastore();

    
    /**
    *Purpose:
    *<p> Check if the periodic observer is already registered (present in map)
    *
    * @param observer_name the name of observer
    * @param map_periodic_observer the map of registered observer
    */
    bool check_periodic_obs_name_in_map (const std::string& observer_name,
                                        MapObs& map_periodic_observer);
    
    /**
    *Purpose:
    *<p> Print map of registered observer
    *
    * @param map_periodic_observer the map of registered observer
    */                                      
    void print_map (MapObs & map_periodic_observer);
    
    
    /**
    *Purpose:
    *<p> Remove the periodic observer record in datastore
    *
    * @param observer_name the name of observer
    */                           
    void remove_observer_record_in_datastore (const char* observer_name);
    
    
    /**
    *Purpose:
    *<p> Get the periodic notifier record from data store
    *
    */
    CdmwPlatformMngt::PeriodicNotifierRecord * get_record_from_datastore();
    
    /**
    *Purpose:
    *<p> Convert the string to miop corbaloc 
    *
    */
    CdmwPlatformMngt::Periodic::ObserverMIOPGroup_ptr create_miop_ref (const char* miop_group)
    throw (BadParameterException);
    
protected:


};
} // End namespace Periodic
} // End namespace PlatformMngt
} // End namespace Cdmw

#endif //INCL_PLATFORMMNGT_PERIODIC_NOTIFIER_HPP
