/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_PERIODIC_TIMER_OBSERVER_HPP
#define INCL_PLATFORMMNGT_PERIODIC_TIMER_OBSERVER_HPP

#include <vector>
#include <string>

#include "tools/SingleShotTimer.hpp"
#include "tools/TimeoutObserver.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtPeriodicObserver.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "SystemMngt/tools/ManagedElementKey.hpp"
#include "SystemMngt/platformlibrary/LogMngr.hpp"

using namespace std;

namespace Cdmw
{// Begin namespace Cdmw
    
namespace PlatformMngt
{// Begin namespace PlatformMngt
    
namespace Periodic
{ // Begin namespace Periodic 
    

/**
* Purpose:
* <p>
* Implementation of a single shot timer timeout observer.
*/
class TimerObserver : public Cdmw::PlatformMngt::TimeoutObserver
{
    
public:


    struct PropertyInfo
    {      
        CosPropertyService::PropertyName property_name;
                
        CdmwPlatformMngt::ElementType element_type;
        
        std::string system_name;
        
        std::string application_name;
        
        std::string host_name;
        
        std::string process_name;
        
        std::string entity_name;
        
        std::string component_name;
        
        std::string ftgroup_name;
        
        std::string lbgroup_name;
        
        std::string attribute_name;
        
        CdmwPlatformMngt::Owner flag;
    };
    
    // Map for periodic observer registered for properties
    typedef std::vector < PropertyInfo > PropertyVector;

    //costructor
    TimerObserver(CdmwPlatformMngt::Periodic::ObserverMIOPGroup_ptr miop_obj,
                CdmwPlatformMngt::Periodic::Observer_ptr observer,
                CdmwPlatformMngt::System_ptr system,
                TimeBase::TimeT period, 
                const CosPropertyService::PropertyNames &  property_names) 
    throw (CdmwPlatformMngt::PropertyNameInvalid);          
    
    //destructor
    virtual ~TimerObserver();
    
    // callback method to call when the timeout will be expired.
    virtual void wake_up() throw ();
    
    //return the periodic observer reference 
    CdmwPlatformMngt::Periodic::Observer_ptr get_observer_ref();
    
    
    //return the miop group reference
    CdmwPlatformMngt::Periodic::ObserverMIOPGroup_ptr get_group_ref();
    
    //start the timer
    void start_timer()
    throw(Cdmw::AlreadyDoneException);

private:

    /**
    * The periodic observer reference
    */   
    CdmwPlatformMngt::Periodic::Observer_var m_observer;
    
    /**
    * The periodic observer reference
    */   
    CdmwPlatformMngt::Periodic::ObserverMIOPGroup_var m_observers;
    
    /**
    * The system reference
    */            
    CdmwPlatformMngt::System_var m_system;

    /**
    * The multicast reference
    */                
    CdmwPlatformMngt::Periodic::ObserverMIOPGroup_var m_miop_obj;
    
    /**
    * The timer
    */         
    SingleShotTimer* m_timer;
    
    /**
    * The timer interval.
    */
    TimeBase::TimeT m_period;

    /**
    *  The sequence of property names
    */
    PropertyVector  m_property_vector;

    /**
    * Property info registration 
    */
    PropertyInfo m_property_info;
    
    /**
    * The Logger pointer
    */
    Cdmw::Logging::Logger * m_logger_pointer;     
    
    /**
    *Purpose:
    *<p> Check if property names syntax are correct
    *
    * @param property_names the sequence of property names
    */ 
    bool TimerObserver::check_property_names(const CosPropertyService::PropertyNames & property_names)
    throw (CdmwPlatformMngt::PropertyNameInvalid); 
    
};

    
} // End namespace Periodic 
} // End namespace PlatformMngt
} // End namespace Cdmw 

#endif //INCL_PLATFORMMNGT_PERIODIC_TIMER_OBSERVER_HPP
