/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_TESTNAVIGABILITY_HPP 
#define INCL_PLATFORMMNGT_TESTNAVIGABILITY_HPP 

#include "Foundation/testutils/CORBATestManager.hpp"
#include "platformperiodicnotifier/Notifier_impl.hpp"
#include "platformperiodicnotifier/TimerObserver.hpp"

#include "testperiodicnotifier/TestObserver_impl.hpp"
#include "testperiodicnotifier/TestGroupObserver_impl.hpp"
#include "testperiodicnotifier/FakeSystem.hpp"

#include <iostream>

namespace Cdmw
{
namespace PlatformMngt
{

class TestPeriodicNotifier : public CppUnit::TestFixture
{

public:
    
    void setUp ();
    
    TestPeriodicNotifier() throw();
    virtual ~TestPeriodicNotifier() throw();

    virtual void single_periodic_obs_registeration() throw();
    virtual void group_periodic_obs_registeration() throw();
    virtual void test_notify_single_registered_observers() throw();
    virtual void test_notify_group_registered_observers() throw();
    virtual void test_periodic_notification() throw();
    virtual void periodic_notifier_id() throw();

private:
    
    virtual void oneShotInit();
    /**
    * Initialise the property sequence to be registered to
    */
    CosPropertyService::PropertyNames* initialise_property();
    
    bool oneShotinitdone;    
    CORBA::ORB_var orb;
    PortableGroup::GOA_var rootGOA;
    
    CdmwPlatformMngtBase::StartupKind_var startupKind;
    CdmwPlatformMngtBase::StateRetrieval state_retrieval;
    CORBA::String_var statusInfo;

    //Startupkind parameter
    unsigned long startup_mode;
    std::string specified_state_id;
    

    TestPeriodicNotifier& operator=(const TestPeriodicNotifier& rhs) throw();
    TestPeriodicNotifier(const TestPeriodicNotifier& rhs) throw();

    CPPUNIT_TEST_SUITE(TestPeriodicNotifier);
    CPPUNIT_TEST(periodic_notifier_id);
    CPPUNIT_TEST(single_periodic_obs_registeration);
    CPPUNIT_TEST(group_periodic_obs_registeration);
    CPPUNIT_TEST(test_periodic_notification);  
    CPPUNIT_TEST(test_notify_single_registered_observers);
    CPPUNIT_TEST(test_notify_group_registered_observers); 
//    CPPUNIT_TEST(test_periodic_notification);  
    CPPUNIT_TEST_SUITE_END();
};

}
}
#endif /* INCL_PLATFORMMNGT_PILOT_SYSTEM_HPP*/

