#include "testperiodicnotifier/FakeApplication.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"

namespace Cdmw
{
namespace PlatformMngt
{

FakeApplication::FakeApplication()
{   

    //fake system elements
    m_entity = new FakeEntity();
    m_lb_group = new FakeLBGroup();
    m_ft_group = new FakeFTGroup();
    m_proc = new FakeProcess();

    m_attr_value = 2 ;
   
}

FakeApplication::~FakeApplication() throw()
{   

    //fake system elements
    delete m_entity;
    delete m_lb_group;
    delete m_ft_group;
    delete m_proc;
    m_entity =0;
    m_lb_group=0;
    m_ft_group=0;
    m_proc=0;

}


void FakeApplication::get_attribute(const char * attr_name, CORBA::Any_out ret_attr , CdmwPlatformMngt::Owner flag)  
throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound)
{ 
    TEST_INFO("FakeApplication::get_attribute"<<std::endl);
    
    if (!strcmp(attr_name, "ApplicationAttribute"))
    {
        TEST_INFO("FakeApplication::get_attribute - HostAttribute"<<std::endl);
        CORBA::Any_var tempany = new CORBA::Any;
        tempany.inout () <<= m_attr_value;
        ret_attr = tempany._retn ();
    }
    else if (!strcmp(attr_name, "AttributeNotFound"))
    {
        TEST_INFO("FakeApplication::get_attribute - AttributeNotFound"<<std::endl);
        throw CdmwPlatformMngt::AttributeNotFound();
    }
    else if (!strcmp(attr_name, "AttributeException"))
    {
        TEST_INFO("FakeApplication::get_attribute - AttributeException"<<std::endl);
        CDMW_THROW2 (Cdmw::Exception, Cdmw::Exception::UNKNOWN,
                     "FakeApplication::get_attribute - AttributeException");

    }
}

CdmwPlatformMngt::Entity_ptr FakeApplication::get_entity (const char* entity_name)
throw( CdmwPlatformMngt::EntityNotFound,CORBA::SystemException )
{
    TEST_INFO("FakeApplication::get_entity"<<std::endl);
    if (!strcmp(entity_name, "ENTNotFound"))
    {
        TEST_INFO("FakeApplication::get_entity - EntityNotFound"<<std::endl);
        throw CdmwPlatformMngt::EntityNotFound();
    }
    return m_entity;
} 
  

CdmwPlatformMngt::Process_ptr FakeApplication::get_process(const char* proc_name,const char* host_name)
throw (CdmwPlatformMngt::ProcessNotFound,CORBA::SystemException)
{
    TEST_INFO("FakeApplication::get_process"<<std::endl);
    if (!strcmp(proc_name, "PROCNotFound") || !strcmp(host_name, "HOSTNotFound"))
    {
        TEST_INFO("FakeApplication::get_process - ProcessNotFound"<<std::endl);
        throw CdmwPlatformMngt::ProcessNotFound();
    }
    return m_proc;
}

CdmwPlatformMngt::LBGroup_ptr FakeApplication::get_lb_group(const char* group_name)
throw (CdmwPlatformMngt::GroupNotFound,CORBA::SystemException)
{
    TEST_INFO("FakeApplication::get_lb_group"<<std::endl);
    if (!strcmp(group_name, "LBGRNotFound"))
    {
        TEST_INFO("FakeApplication::get_lb_group - LBGRNotFound"<<std::endl);
        throw CdmwPlatformMngt::GroupNotFound();
    }
    return m_lb_group;
}

CdmwPlatformMngt::FTGroup_ptr FakeApplication::get_ft_group(const char* group_name)
throw (CdmwPlatformMngt::GroupNotFound,CORBA::SystemException)
{
    TEST_INFO("FakeApplication::get_ft_group"<<std::endl);
    if (!strcmp(group_name, "FTGRNotFound"))
    {
        TEST_INFO("FakeApplication::get_ft_group - FTGRNotFound"<<std::endl);
        throw CdmwPlatformMngt::GroupNotFound();
    }
    return m_ft_group;
}

            
}
}









