/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "testperiodicnotifier/TestPeriodicNotifier.hpp"

#include "Foundation/testutils/CORBATestManager.hpp"

#include "platformperiodicnotifier/Notifier_impl.hpp" //class to test
#include "platformperiodicnotifier/TimerObserver.hpp" //class to test
#include "testperiodicnotifier/FakeSystem.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp" 
#include "testperiodicnotifier/TestObserver_impl.hpp"
#include "testperiodicnotifier/TestGroupObserver_impl.hpp"

#include "orbsvcs/orbsvcs/PortableGroupC.h"
#include <SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp>
#include "Foundation/orbsupport/StrategyList.hpp"


#include <string>
#include <iostream>

using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestPeriodicNotifier);

namespace Cdmw
{
namespace PlatformMngt
{

TestPeriodicNotifier::TestPeriodicNotifier () throw ()
{
        oneShotinitdone = false;
}

TestPeriodicNotifier::~TestPeriodicNotifier () throw ()
{

}



void TestPeriodicNotifier::setUp ()
{
        if (!oneShotinitdone) oneShotInit ();
        
}

void TestPeriodicNotifier::oneShotInit()
{

        oneShotinitdone = true;

        //Getting orb
        orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::instance ()->get_ORB ());
        //Get the root GOA
        CORBA::Object_var obj = orb->resolve_initial_references ("RootPOA");
        rootGOA = PortableGroup::GOA::_narrow(obj.in());
        
        //Setting up the startupKind parameters
        state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
        startup_mode = 0;
        specified_state_id = "";
        startupKind = new CdmwPlatformMngtBase::StartupKind();
        startupKind->state_retrieval = state_retrieval;
        startupKind->startup_mode = startup_mode;
        startupKind->specified_state_id  = CORBA::string_dup( specified_state_id.c_str() );

}

    
void TestPeriodicNotifier::single_periodic_obs_registeration () throw ()
{

    TEST_INFO("==========================================================="<<std::endl);
    TEST_INFO(" START SINGLE PERIODIC OBSERVER REGISTRATION"<<std::endl);
    TEST_INFO("==========================================================="<<std::endl);

    //fake system       
    FakeSystem* system = new Cdmw::PlatformMngt::FakeSystem();

    //inizialise an instance of periodic notifier
    Cdmw::PlatformMngt::Periodic::Notifier_impl* pd_notifier = new Cdmw::PlatformMngt::Periodic::Notifier_impl(orb.in(), system);

    //activate periodic observer
     Cdmw::PlatformMngt::Periodic::TestObserver_impl* obs_impl = new Cdmw::PlatformMngt::Periodic::TestObserver_impl();   
    
    //observer = obs_impl->_this();
    PortableServer::ObjectId_var  oid = rootGOA->activate_object(obs_impl); 
    CORBA::Object_var object = rootGOA->id_to_reference(oid.in());
    CdmwPlatformMngt::Periodic::Observer_var observer = CdmwPlatformMngt::Periodic::Observer::_narrow(object.in());
    
    //initialise the property to be registered to
    CORBA::ULong len = 1;
    CosPropertyService::PropertyNames_var property_names = new CosPropertyService::PropertyNames(len);
    property_names->length(len);
    std::string property_name("S1.sys/SystemAttribute.attr");
    property_names[len-1] = CORBA::string_dup( property_name.c_str() );
    TimeBase::TimeT period = 1000;
    std::string obName("TestPeriodicObserver");

   TEST_INFO(" Register operation with valid property_name"<<std::endl);
    try
    {
        //register periodic observer 
        pd_notifier->register_observer(obName.c_str(), property_names.in(), observer.in(), period);          
        CPPUNIT_ASSERT (true);

    } catch (const CdmwPlatformMngt::PropertyNameInvalid &e)
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);

    } catch (const CdmwPlatformMngt::ElementAlreadyExists & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    }   


   TEST_INFO(" Register operation with the same observer name"<<std::endl);
    try
    {
        //register periodic observer 
        pd_notifier->register_observer(obName.c_str(), property_names.in(), observer.in(), period);          
        CPPUNIT_ASSERT (false);

    } catch (const CdmwPlatformMngt::PropertyNameInvalid &e)
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);

    } catch (const CdmwPlatformMngt::ElementAlreadyExists & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (true);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    }    


    Cdmw::OsSupport::OS::sleep(2000);

    TEST_INFO(" Unregistering periodic observer previuosly registered..."<<std::endl);
    try
    {
        observer = pd_notifier->unregister_observer(obName.c_str());
        CPPUNIT_ASSERT (true);

    }catch (const CdmwPlatformMngt::ObserverNotFound & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    }


    TEST_INFO(" Register operation with the null observer reference "<<std::endl);
    CdmwPlatformMngt::Periodic::Observer_var observer2 = CdmwPlatformMngt::Periodic::Observer::_nil();
    try
    {
        //register periodic observer 
        pd_notifier->register_observer(obName.c_str(), property_names.in(), observer2.in(), period);          
        CPPUNIT_ASSERT (false);

    } catch (const CdmwPlatformMngt::PropertyNameInvalid &e)
    {
        TEST_INFO(" Exception "<<e);
        CPPUNIT_ASSERT (false);

    } catch (const CdmwPlatformMngt::ElementAlreadyExists & e) 
    {
        TEST_INFO(" Exception "<<e);
        CPPUNIT_ASSERT (false);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception "<<e);
        CPPUNIT_ASSERT (true);
    } 


    
    TEST_INFO(" Register operation with invalid property_name"<<std::endl);
    std::string property_temp("S1.sys/SystemAttribute.at");
    property_names[len-1] = CORBA::string_dup( property_temp.c_str() ); 
    try
    {
        //register periodic observer 
        pd_notifier->register_observer(obName.c_str(), property_names.in(), observer.in(), period);          
        CPPUNIT_ASSERT (false);

    } catch (const CdmwPlatformMngt::PropertyNameInvalid &e)
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (true);

    } catch (const CdmwPlatformMngt::ElementAlreadyExists & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);

    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    }      

    TEST_INFO(" Two register operation with valid property_name"<<std::endl);
    property_names[len-1] = CORBA::string_dup( property_name.c_str() ); 
    std::string obName2("TestPeriodicObserver2");
    observer2 = obs_impl->_this();
    try
    {
        //observer = CdmwPlatformMngt::Periodic::Observer::nil();
        //register periodic observer 
        pd_notifier->register_observer(obName.c_str(), property_names.in(), observer.in(), period);
        pd_notifier->register_observer(obName2.c_str(), property_names.in(), observer2.in(), period);
        CPPUNIT_ASSERT (true);

    } catch (const CdmwPlatformMngt::PropertyNameInvalid &e)
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);

    } catch (const CdmwPlatformMngt::ElementAlreadyExists & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    }  

    Cdmw::OsSupport::OS::sleep(2000);

    TEST_INFO(" Unregistering periodic observers previuosly registered..."<<std::endl);
    try
    {
        observer = pd_notifier->unregister_observer(obName.c_str());
        observer2 = pd_notifier->unregister_observer(obName2.c_str());
        CPPUNIT_ASSERT (true);

    }catch (const CdmwPlatformMngt::ObserverNotFound & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    }

        
    TEST_INFO(" Unregistering not registered  observer ..."<<std::endl);
    obName = "FakePeriodicObserver";
    try
    {
        observer = pd_notifier->unregister_observer(obName.c_str());
        CPPUNIT_ASSERT (false);

    }catch (const CdmwPlatformMngt::ObserverNotFound & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (true);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    }

        delete pd_notifier;
        pd_notifier = 0;
        delete system;
        system = 0;
        rootGOA->deactivate_object (oid.in());
        delete obs_impl;
        obs_impl = 0;
        //property_names->length(0);

}

void TestPeriodicNotifier::group_periodic_obs_registeration () throw ()
{

    TEST_INFO("==========================================================="<<std::endl);
    TEST_INFO(" START GROUP PERIODIC OBSERVER REGISTRATION"<<std::endl);
    TEST_INFO("==========================================================="<<std::endl);

    //fake system       
    FakeSystem* system = new Cdmw::PlatformMngt::FakeSystem();

    //inizialise an instance of periodic notifier
    Cdmw::PlatformMngt::Periodic::Notifier_impl* pd_notifier = new Cdmw::PlatformMngt::Periodic::Notifier_impl(orb.in(), system);

    //activate periodic observer group
    // create servant
    Cdmw::PlatformMngt::Periodic::TestGroupObserver_impl* group_obs1 =
        new Cdmw::PlatformMngt::Periodic::TestGroupObserver_impl();
    Cdmw::PlatformMngt::Periodic::TestGroupObserver_impl* group_obs2 =
        new Cdmw::PlatformMngt::Periodic::TestGroupObserver_impl();

    // activate Miop Object
    PortableServer::ObjectId_var obj_id1  = rootGOA->activate_object(group_obs1);
    PortableServer::ObjectId_var obj_id2  = rootGOA->activate_object(group_obs2);

    std::string miop_group ("corbaloc:miop:1.0@1.0-cdmwftdomain-10/225.1.1.8:5555");
    CORBA::Object_var miop_ref = orb->string_to_object(miop_group.c_str());
    rootGOA->associate_reference_with_id (miop_ref.in(), obj_id1.in());
    rootGOA->associate_reference_with_id (miop_ref.in(), obj_id2.in());

    //initialise the property to be registered to
    CORBA::ULong len = 1;
    CosPropertyService::PropertyNames_var property_names = new CosPropertyService::PropertyNames(len);
    property_names->length(len);
    std::string property_name ("S1.sys/SystemAttribute.attr");
    property_names[len-1] = CORBA::string_dup( property_name.c_str() );
    TimeBase::TimeT period = 3000;
    std::string obGroupName("TestPeriodicObserverGroup");

    TEST_INFO(" Register operation with valid property_name"<<std::endl);
    try
    {
        //register periodic observer 
        pd_notifier->register_observer_group(obGroupName.c_str(), property_names.in(), miop_group.c_str(), period);          
        CPPUNIT_ASSERT (true);

    } catch (const CdmwPlatformMngt::PropertyNameInvalid &e)
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);

    } catch (const CdmwPlatformMngt::ElementAlreadyExists & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    }   

    TEST_INFO(" Register operation with the same observer name"<<std::endl);
    try
    {
        //register periodic observer 
        pd_notifier->register_observer_group(obGroupName.c_str(), property_names.in(), miop_group.c_str(), period);          
        CPPUNIT_ASSERT (false);

    } catch (const CdmwPlatformMngt::PropertyNameInvalid &e)
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);

    } catch (const CdmwPlatformMngt::ElementAlreadyExists & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (true);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    }    

    Cdmw::OsSupport::OS::sleep(2000);
 
    TEST_INFO(" Unregistering periodic observer previuosly registered..."<<std::endl);
    try
    {
        miop_group = pd_notifier->unregister_observer_group(obGroupName.c_str());
        CPPUNIT_ASSERT (true);

    }catch (const CdmwPlatformMngt::ObserverNotFound & e) 
    {
        TEST_INFO(" Exception"<<e);        
        CPPUNIT_ASSERT (false);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    }

    TEST_INFO(" Register operation with null group reference "<<std::endl);
    string null_str ="";
    try
    {
        //register periodic observer 
        pd_notifier->register_observer_group(obGroupName.c_str(), property_names.in(), null_str.c_str(), period);          
        CPPUNIT_ASSERT (false);

    } catch (const CdmwPlatformMngt::PropertyNameInvalid &e)
    {
        TEST_INFO(" Exception "<<e);
        CPPUNIT_ASSERT (false);

    } catch (const CdmwPlatformMngt::ElementAlreadyExists & e) 
    {
        TEST_INFO(" Exception "<<e);
        CPPUNIT_ASSERT (false);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception "<<e);
        CPPUNIT_ASSERT (true);
    }



    TEST_INFO(" Register operation with invalid property_name"<<std::endl);
    property_name = "S1.sys/SystemAttribute.at";
    property_names[len-1] = CORBA::string_dup( property_name.c_str() ); 
    try
    {
        //register periodic observer 
        pd_notifier->register_observer_group(obGroupName.c_str(), property_names.in(), miop_group.c_str(), period);          
        CPPUNIT_ASSERT (false);

    } catch (const CdmwPlatformMngt::PropertyNameInvalid &e)
    {
        TEST_INFO(" Exception "<<e);
        CPPUNIT_ASSERT (true);

    } catch (const CdmwPlatformMngt::ElementAlreadyExists & e) 
    {
        TEST_INFO(" Exception "<<e);
        CPPUNIT_ASSERT (false);

    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception "<<e);
        CPPUNIT_ASSERT (false);
    }      

    TEST_INFO(" Two register operation with valid property_name"<<std::endl);
    property_names[len-1] = CORBA::string_dup( "S1.sys/SystemAttribute.attr" ); 
    std::string obGroupName2("SecondPeriodicGroup");
    std::string miop_group2("corbaloc:miop:1.0@1.0-cdmwtestdomain-10/225.1.1.8:5555");
    //associate the periodic observers to the same miop group
    miop_ref = orb->string_to_object(miop_group2.c_str());
    rootGOA->associate_reference_with_id (miop_ref.in(), obj_id1.in());
    rootGOA->associate_reference_with_id (miop_ref.in(), obj_id2.in());
    try
    {
        //register periodic observer 
        pd_notifier->register_observer_group(obGroupName.c_str(), property_names.in(), miop_group.c_str(), period);
        pd_notifier->register_observer_group(obGroupName2.c_str(), property_names.in(), miop_group2.c_str(), period);
        CPPUNIT_ASSERT (true);

    } catch (const CdmwPlatformMngt::PropertyNameInvalid &e)
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);

    } catch (const CdmwPlatformMngt::ElementAlreadyExists & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    }  

    Cdmw::OsSupport::OS::sleep(1000);
    
    TEST_INFO(" Unregistering periodic observers previuosly registered..."<<std::endl);
    try
    {
        miop_group = pd_notifier->unregister_observer_group(obGroupName.c_str());
        miop_group = pd_notifier->unregister_observer_group(obGroupName2.c_str());
        CPPUNIT_ASSERT (true);

    }catch (const CdmwPlatformMngt::ObserverNotFound & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    }

    //sleep (3);
    
    TEST_INFO(" Unregistering not registered  observer (with invalid observer name)..."<<std::endl);
    obGroupName = "FakePeriodicObserverName";
    try
    {
        miop_group = pd_notifier->unregister_observer_group(obGroupName.c_str());
        CPPUNIT_ASSERT (false);

    }catch (const CdmwPlatformMngt::ObserverNotFound & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (true);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    }
    
    
        delete pd_notifier;
        pd_notifier = 0;
        delete system;
        system = 0;
        
        rootGOA->deactivate_object (obj_id1.in());
        rootGOA->deactivate_object (obj_id2.in());
        delete group_obs1;
        group_obs1 = 0;
        delete group_obs2;
        group_obs2 = 0;
        property_names->length(0);
}


void TestPeriodicNotifier::test_notify_single_registered_observers () throw ()
{

    TEST_INFO("==========================================================="<<std::endl);
    TEST_INFO(" START NOTIFY ALL REGISTERED OBSERVER TEST"<<std::endl);
    TEST_INFO("==========================================================="<<std::endl);
    
    //fake system       
    FakeSystem* system = new Cdmw::PlatformMngt::FakeSystem();

    //inizialise an instance of periodic notifier
    Cdmw::PlatformMngt::Periodic::Notifier_impl* pd_notifier = new Cdmw::PlatformMngt::Periodic::Notifier_impl(orb.in(), system);
    
    TEST_INFO(" Notify all registered observers before any registration operation...expect doing nothing "<<std::endl);
    pd_notifier->notify_registered_observers();

    //activate periodic observer
     Cdmw::PlatformMngt::Periodic::TestObserver_impl* obs_impl = new Cdmw::PlatformMngt::Periodic::TestObserver_impl();   
    //observer = obs_impl->_this();
    
    PortableServer::ObjectId_var  oid = rootGOA->activate_object(obs_impl); 
    CORBA::Object_var object = rootGOA->id_to_reference(oid.in());
    CdmwPlatformMngt::Periodic::Observer_var observer = CdmwPlatformMngt::Periodic::Observer::_narrow(object.in()); 

    //initialise the property to be registered to
    CORBA::ULong len = 1;
    CosPropertyService::PropertyNames_var property_names = new CosPropertyService::PropertyNames(len);
    property_names->length(len);
    property_names[len-1] = CORBA::string_dup( "S1.sys/SystemAttribute.attr" );
    TimeBase::TimeT period = 1000;
    std::string obName("TestPeriodicObserver");

    TEST_INFO(" Register operation with valid property_name"<<std::endl);
    try
    {
        //register periodic observer 
        pd_notifier->set_observer_info(obName.c_str(),property_names.in(),observer.in(), period);
        
        //simulate defining the datastore
        pd_notifier->define_ds();
        TEST_INFO(" Notify a single periodic observer "<<std::endl);
        pd_notifier->notify_registered_observers();

        CPPUNIT_ASSERT (true);

    } catch (const CdmwPlatformMngt::PropertyNameInvalid &e)
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);

    } catch (const CdmwPlatformMngt::ElementAlreadyExists & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } 
    catch (...)
    {
    TEST_INFO(" Unexpected!");
    }

    Cdmw::OsSupport::OS::sleep(2000);
     
        delete pd_notifier;
        pd_notifier = 0;
        delete system;
        system = 0;
        rootGOA->deactivate_object(oid.in());
        delete obs_impl;
        obs_impl = 0;
        property_names->length(0);
}


void TestPeriodicNotifier::test_notify_group_registered_observers () throw ()
{

    TEST_INFO("==========================================================="<<std::endl);
    TEST_INFO(" START NOTIFY ALL REGISTERED OBSERVER TEST"<<std::endl);
    TEST_INFO("==========================================================="<<std::endl);

    //fake system       
    FakeSystem* system = new Cdmw::PlatformMngt::FakeSystem();

    //inizialise an instance of periodic notifier
    Cdmw::PlatformMngt::Periodic::Notifier_impl* pd_notifier = new Cdmw::PlatformMngt::Periodic::Notifier_impl(orb.in(), system);
    
    TEST_INFO(" Notify all registered observers before any registration operation...expect doing nothing "<<std::endl);
    pd_notifier->notify_registered_observers();

    //activate periodic observer group
    // create servant
    Cdmw::PlatformMngt::Periodic::TestGroupObserver_impl* group_obs1 =
        new Cdmw::PlatformMngt::Periodic::TestGroupObserver_impl();
    Cdmw::PlatformMngt::Periodic::TestGroupObserver_impl* group_obs2 =
        new Cdmw::PlatformMngt::Periodic::TestGroupObserver_impl();

    // activate Miop Object
    PortableServer::ObjectId_var obj_id1  = rootGOA->activate_object(group_obs1);
    PortableServer::ObjectId_var obj_id2  = rootGOA->activate_object(group_obs2);

    std::string miop_group("corbaloc:miop:1.0@1.0-cdmwftdomain-10/225.1.1.8:5555");
    CORBA::Object_var miop_ref = orb->string_to_object(miop_group.c_str());
    rootGOA->associate_reference_with_id (miop_ref.in(), obj_id1.in());
    rootGOA->associate_reference_with_id (miop_ref.in(), obj_id2.in());
    
    //initialise the property to be registered to
    CORBA::ULong len = 1;
    CosPropertyService::PropertyNames_var property_names = new CosPropertyService::PropertyNames(len);
    property_names->length(len);
    property_names[len-1] = CORBA::string_dup( "S.sys/SystemAttribute.attr" );
    TimeBase::TimeT period = 3000;
    std::string obGroupName("TestPeriodicObserverGroup");
    
    TEST_INFO(" Register operation with valid property_name"<<std::endl);
    try
    {
        //register periodic observer 
        pd_notifier->register_observer_group(obGroupName.c_str(), property_names.in(), miop_group.c_str(), period);   
        
        //simulate defining the datastore
        pd_notifier->define_ds();
        TEST_INFO(" Notify a group of periodic observer "<<std::endl);
        pd_notifier->notify_registered_observers();

        CPPUNIT_ASSERT (true);

    } catch (const CdmwPlatformMngt::PropertyNameInvalid &e)
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);

    } catch (const CdmwPlatformMngt::ElementAlreadyExists & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } 
    
   Cdmw::OsSupport::OS::sleep(2000);
    
    TEST_INFO(" Unregistering periodic observer previuosly registered..."<<std::endl);
    try
    {
        miop_group = pd_notifier->unregister_observer_group(obGroupName.c_str());
        CPPUNIT_ASSERT (true);

    }catch (const CdmwPlatformMngt::ObserverNotFound & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    }
        
        delete pd_notifier;
        pd_notifier = 0;
        delete system;
        system = 0;
        rootGOA->deactivate_object(obj_id1.in());
        rootGOA->deactivate_object(obj_id2.in());
        delete group_obs1;
        group_obs1 = 0;
        delete group_obs2;
        group_obs2 = 0;
        //property_names->length(0);
}


void TestPeriodicNotifier::test_periodic_notification () throw ()
{

    TEST_INFO("==========================================================="<<std::endl);
    TEST_INFO(" START TEST PERIODIC NOTIFICATION"<<std::endl);
    TEST_INFO("==========================================================="<<std::endl);
    
    //fake system       
    FakeSystem* system = new Cdmw::PlatformMngt::FakeSystem();

    //activate periodic observer
    Cdmw::PlatformMngt::Periodic::TestObserver_impl* obs_impl = new Cdmw::PlatformMngt::Periodic::TestObserver_impl();   
    //observer = obs_impl->_this(); 
    PortableServer::ObjectId_var  oid = rootGOA->activate_object(obs_impl); 
    CORBA::Object_var object = rootGOA->id_to_reference(oid.in());
    CdmwPlatformMngt::Periodic::Observer_var observer = CdmwPlatformMngt::Periodic::Observer::_narrow(object.in());
    
    CosPropertyService::PropertyNames_var property_names = initialise_property();
    TimeBase::TimeT period = 600000;
    CdmwPlatformMngt::Periodic::ObserverMIOPGroup_var group_obs = CdmwPlatformMngt::Periodic::ObserverMIOPGroup::_nil();
    
    Cdmw::PlatformMngt::Periodic::TimerObserver* timer_observer_wakeup = 
        new Cdmw::PlatformMngt::Periodic::TimerObserver(group_obs.in(), observer.in(), system, period, property_names.in());
    
    timer_observer_wakeup->wake_up();
    
    Cdmw::OsSupport::OS::sleep(1000);
    
    CORBA::ULong len = 14;
    
    CORBA::ULong prop_num = obs_impl->get_num_property_ok();
    
    if (prop_num == len)
    {
        CPPUNIT_ASSERT (true);
    }
    else 
    {
        CPPUNIT_ASSERT (false);
    }
    
        delete timer_observer_wakeup;
        timer_observer_wakeup = 0;
        delete system;
        system = 0;
        rootGOA->deactivate_object(oid.in());
        delete obs_impl;
        obs_impl = 0;
        property_names->length(0);

}

void TestPeriodicNotifier::periodic_notifier_id() throw ()
{

    TEST_INFO("==========================================================="<<std::endl);
    TEST_INFO(" VERIFY PERIODIC NOTIFIER ID"<<std::endl);
    TEST_INFO("==========================================================="<<std::endl);
    
    //fake system       
    FakeSystem* system = new Cdmw::PlatformMngt::FakeSystem();

    //inizialise an instance of periodic notifier
    Cdmw::PlatformMngt::Periodic::Notifier_impl* pd_notifier = new Cdmw::PlatformMngt::Periodic::Notifier_impl(orb.in(), system);
    
    
    std::string pd_id = "CdmwPlatformMngtPeriodicNotifier";
    CPPUNIT_ASSERT (!strcmp(pd_notifier->periodic_notifier_id(), pd_id.c_str()));
    
        delete pd_notifier;
        pd_notifier = 0;
        delete system;
        system = 0;
}

CosPropertyService::PropertyNames*  TestPeriodicNotifier::initialise_property()
{
    
    CORBA::ULong len = 48 ;
    //initialise the property to be registered to
    CosPropertyService::PropertyNames_var property_names = new CosPropertyService::PropertyNames(len);

    //initialise the property to be registered to

    property_names->length(len);

    /*
     * The elements of property names are syntax correct.
     */
    std::string property_name = string (256, '0');
    property_name = "S1.sys/SystemAttribute.attr";
    len = 0 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[0] = "<<property_name.c_str()<<std::endl);
    
    property_name = "S1.sys/ENT1.ent/EntityAttribute.attr";
    len = 1 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[1] = "<<property_name.c_str()<<std::endl);
    
    property_name = "S1.sys/APPL1.appl/ApplicationAttribute.attr";
    len = 2 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[2] = "<<property_name.c_str()<<std::endl);
    
    property_name = "S1.sys/APPL1.appl/FTGR1.ftgroup/ENT1.ent/EntityAttribute.attr";
    len = 3 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[3] = "<<property_name.c_str()<<std::endl);   
    
    property_name = "S1.sys/APPL1.appl/LBGR1.lbgroup/ENT1.ent/EntityAttribute.attr";
    len = 4 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[4] = "<<property_name.c_str()<<std::endl);
    
    property_name = "S1.sys/APPL1.appl/FTGR1.ftgroup/FTAttribute.attr";
    len = 5 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[5] = "<<property_name.c_str()<<std::endl);   
    
    property_name = "S1.sys/APPL1.appl/LBGR1.lbgroup/LBAttribute.attr";
    len = 6 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[6] = "<<property_name.c_str()<<std::endl);
 
    property_name = "S1.sys/APPL1.appl/HOST1.host/PROC1.proc/ProcessAttribute.attr";
    len = 7 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[7] = "<<property_name.c_str()<<std::endl);

    property_name = "S1.sys/APPL1.appl/HOST1.host/PROC1.proc/COMP1.comp/ComponentAttribute.attr";
    len = 8 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[8] = "<<property_name.c_str()<<std::endl);
    
    property_name = "S1.sys/APPL1.appl/HOST1.host/PROC1.proc/COMP1.comp/ENT1.ent/EntityAttribute.attr";
    len = 9 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[9] = "<<property_name.c_str()<<std::endl);
    
    property_name = "S1.sys/APPL1.appl/HOST1.host/PROC1.proc/ENT1.ent/EntityAttribute.attr";
    len = 10 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[10] = "<<property_name.c_str()<<std::endl);
    
    property_name = "S1.sys/APPL1.appl/ENT1.ent/EntityAttribute.attr";
    len = 11 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[11] = "<<property_name.c_str()<<std::endl);
    
    property_name = "S1.sys/HOST1.host/HostAttribute.attr";
    len = 12 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[12] = "<<property_name.c_str()<<std::endl);
    
    property_name = "S1.sys/HOST1.host/ENT1.ent/EntityAttribute.attr";
    len = 13 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[13] = "<<property_name.c_str()<<std::endl);
     

    /*
     * The error elements of property names.
     */
     //for system
    property_name = "S1.sys/AttributeNotFound.attr";
    len = 14 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[14] = "<<property_name.c_str()<<std::endl);  
    property_name = "S1.sys/AttributeException.attr";
    len = 15 ;
    property_names[len] = CORBA::string_dup(property_name.c_str() );
    TEST_INFO("user-defined property_names[15] = "<<property_name.c_str()<<std::endl);

    //for host
    property_name = "S1.sys/HOSTNotFound.host/HostAttribute.attr";
    len = 16 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[16] = "<<property_name.c_str()<<std::endl); 
    property_name = "S1.sys/HOST1.host/AttributeNotFound.attr";
    len = 17 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[17] = "<<property_name.c_str()<<std::endl);
    property_name = "S1.sys/HOST1.host/AttributeException.attr";
    len = 18 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[18] = "<<property_name.c_str()<<std::endl);
 
    //for application
    property_name = "S1.sys/APPLNotFound.appl/ApplicationAttribute.attr";
    len = 19 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[19] = "<<property_name.c_str()<<std::endl);
    property_name = "S1.sys/APPL1.appl/AttributeNotFound.attr";
    len = 20;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[20] = "<<property_name.c_str()<<std::endl);
    property_name = "S1.sys/APPL1.appl/AttributeException.attr";
    len = 21 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[21] = "<<property_name.c_str()<<std::endl);

    //for ft_group
    property_name = "S1.sys/APPLNotFound.appl/FTGR1.ftgroup/FTAttribute.attr";
    len = 22 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[22] = "<<property_name.c_str()<<std::endl);
    property_name = "S1.sys/APPL1.appl/FTGRNotFound.ftgroup/FTAttribute.attr";
    len = 23 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[23] = "<<property_name.c_str()<<std::endl);
    property_name = "S1.sys/APPL1.appl/FTGR1.ftgroup/AttributeNotFound.attr";
    len = 24 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[24] = "<<property_name.c_str()<<std::endl);
    property_name = "S1.sys/APPL1.appl/FTGR1.ftgroup/AttributeException.attr";
    len = 25 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[25] = "<<property_name.c_str()<<std::endl);
    
    //for lb_group
    property_name = "S1.sys/APPLNotFound.appl/LBGR1.lbgroup/LBAttribute.attr";
    len = 26 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() ); 
    TEST_INFO("user-defined property_names[26] = "<<property_name.c_str()<<std::endl);
    property_name = "S1.sys/APPL1.appl/LBGRNotFound.lbgroup/LBAttribute.attr";
    len = 27 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() ); 
    TEST_INFO("user-defined property_names[27] = "<<property_name.c_str()<<std::endl);
    property_name = "S1.sys/APPL1.appl/LBGR1.lbgroup/AttributeNotFound.attr";
    len = 28 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[28] = "<<property_name.c_str()<<std::endl);
    property_name = "S1.sys/APPL1.appl/LBGR1.lbgroup/AttributeException.attr";
    len = 29 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[29] = "<<property_name.c_str()<<std::endl);
    
    //for process
    property_name = "S1.sys/APPLNotFound.appl/HOST1.host/PROC1.proc/ProcessAttribute.attr";
    len = 30 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[30] = "<<property_name.c_str()<<std::endl);
    property_name = "S1.sys/APPL1.appl/HOST1.host/PROCNotFound.proc/ProcessAttribute.attr";
    len = 31 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[31] = "<<property_name.c_str()<<std::endl);
    property_name = "S1.sys/APPL1.appl/HOST1.host/PROC1.proc/AttributeNotFound.attr";
    len = 32 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[32] = "<<property_name.c_str()<<std::endl);
    property_name = "S1.sys/APPL1.appl/HOST1.host/PROC1.proc/AttributeException.attr";
    len = 33 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[33] = "<<property_name.c_str()<<std::endl);
    
    //for component
    property_name = "S1.sys/APPLNotFound.appl/HOST1.host/PROC1.proc/COMP1.comp/ComponentAttribute.attr";
    len = 34 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[34] = "<<property_name.c_str()<<std::endl);
    property_name = "S1.sys/APPL1.appl/HOST1.host/PROCNotFound.proc/COMP1.comp/ComponentAttribute.attr";
    len = 35 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[35] = "<<property_name.c_str()<<std::endl);
    property_name = "S1.sys/APPL1.appl/HOST1.host/PROC1.proc/COMPNotFound.comp/ComponentAttribute.attr";
    len = 36 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[36] = "<<property_name.c_str()<<std::endl);
    property_name = "S1.sys/APPL1.appl/HOST1.host/PROC1.proc/COMP1.comp/AttributeNotFound.attr";
    len = 37 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[37] = "<<property_name.c_str()<<std::endl);
    property_name = "S1.sys/APPL1.appl/HOST1.host/PROC1.proc/COMP1.comp/AttributeException.attr";
    len = 38 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[38] = "<<property_name.c_str()<<std::endl);
    
    //for entity
    property_name = "S1.sys/HOSTNotFound.host/ENT1.ent/EntityAttribute.attr";
    len = 39 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[39] = "<<property_name.c_str()<<std::endl);
    property_name = "S1.sys/ENTNotFound.ent/EntityAttribute.attr";
    len = 40 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[40] = "<<property_name.c_str()<<std::endl);
    property_name = "S1.sys/APPLNotFound.appl/ENT1.ent/EntityAttribute.attr";
    len = 41 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[41] = "<<property_name.c_str()<<std::endl);
    property_name = "S1.sys/APPL1.appl/HOST1.host/PROCNotFound.proc/ENT1.ent/EntityAttribute.attr";
    len = 42 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[42] = "<<property_name.c_str()<<std::endl);
    property_name = "S1.sys/APPL1.appl/HOST1.host/PROC1.proc/COMPNotFound.comp/ENT1.ent/EntityAttribute.attr";
    len = 43 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[43] = "<<property_name.c_str()<<std::endl);
    property_name = "S1.sys/HOST1.host/ENT1.ent/AttributeNotFound.attr";
    len = 44 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[44] = "<<property_name.c_str()<<std::endl);
    property_name = "S1.sys/HOST1.host/ENT1.ent/AttributeException.attr";
    len = 45 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[45] = "<<property_name.c_str()<<std::endl);
    property_name = "S1.sys/APPL1.appl/FTGRNotFound.ftgroup/ENT1.ent/EntityAttribute.attr";
    len = 46 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[46] = "<<property_name.c_str()<<std::endl);
    property_name = "S1.sys/APPL1.appl/LBGRNotFound.lbgroup/ENT1.ent/EntityAttribute.attr";
    len = 47 ;
    property_names[len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("user-defined property_names[47] = "<<property_name.c_str()<<std::endl);

    return property_names._retn();
}

}
}





