/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef HOSTSTATES_HPP_
#define HOSTSTATES_HPP_

#include "Foundation/common/Exception.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "SystemMngt/platformlibrary/StateMachineTemplate.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp"
#include "SystemMngt/platformapplicationlibrary/ConsistencyStatusSolver.hpp"
#include "Foundation/logging/Logger.hpp"
#include <string>

#ifdef USE_FAKE_TEST_OBJECTS
#include "testhoststatemachine/FakeHost_impl.hpp"
#else
#include "platformsystemlibrary/Host_impl.hpp"
#endif

namespace Cdmw
{
namespace PlatformMngt
{

#define HOST_RESPONDING_NAME "Responding"
#define HOST_REINSERTING_NAME "Reinserting"
#define HOST_STARTING_NAME "Starting"
#define HOST_STARTED_NAME "Started"
#define HOST_PARTIAL_STARTED_NAME "PartialStarted"
#define HOST_STOPPING_NAME "Stopping"
#define HOST_UNREACHABLE_NAME "Unreachable"
#define HOST_STATEMACHINE_NAME "StateMachine"

class Host_impl;

//typedef StateMachine <Host_impl, HostState> HostStateMachine;
class HostState;
class HostResponding;
class HostStarting;
class HostStarted;
class HostPartialStarted;
class HostStopping;
class HostUnreachable;
//class HostStateMachine : public StateMachine <Host_impl, HostState>;

typedef StateMachine <Host_impl, HostState> BaseHostStateMachine;

//class HostStateMachine : public virtual StateMachine <Host_impl, HostState>
class HostStateMachine : public virtual BaseHostStateMachine
{
public:
    //HostStateMachine(Host_impl*, const std::string&) throw (OutOfMemoryException);
    HostStateMachine(Host_impl*) throw (OutOfMemoryException);
    virtual ~HostStateMachine();
    void setState(CdmwPlatformMngt::HostStatus);            
    void setState(const std::string& stateName);            

    const static std::string m_HostRespondingName;
    const static std::string m_HostStartingName;
    const static std::string m_HostStartedName;
    const static std::string m_HostPartialStartedName;
    const static std::string m_HostStoppingName;
    const static std::string m_HostUnreachableName;

private: 
    
    // machine states
    HostResponding* m_HostResponding;
    HostStarting* m_HostStarting;
    HostStarted* m_HostStarted;
    HostPartialStarted* m_HostPartialStarted;
    HostStopping* m_HostStopping;
    HostUnreachable* m_HostUnreachable;
    
    
};


/**
 *Purpose:
 *<p> The abstract state of the state machine.
 *    Provides the default implementation of a host state.
 */ 
class HostState : public virtual HostStateMachine::AbstractState
{
protected:
    // the solver of the consistent status
    //ConsistencyStatusSolver<HostStateMachine>* m_consistency_status_resolver;
    
public:

    enum DaemonStatus { DAEMON_RUNNING, DAEMON_NOT_RUNNING };
    enum ProcessesStatus { NO_PROCESS_RUNNING, NOT_ALL_PROCESSES_RUNNING, ALL_PROCESSES_RUNNING };
            
    virtual ~HostState() {};

    // Methods inherited from AbstractState
    virtual void entryAction (BaseHostStateMachine* stateMachine);
    //virtual void exitAction (BaseHostStateMachine* stateMachine) {};
    //virtual void doActivity (BaseHostStateMachine* stateMachine) {};
    //virtual void completionTransition (BaseHostStateMachine* stateMachine) {};

    virtual CdmwPlatformMngt::HostStatus status() = 0;
    
    // State machine commands
    virtual void start(HostStateMachine* )
        throw (CdmwPlatformMngt::IncompatibleStatus);
    virtual void stop(HostStateMachine* )
        throw (CdmwPlatformMngt::IncompatibleStatus);
    virtual void reinsert(HostStateMachine* )                               // for future use
        throw (CdmwPlatformMngt::IncompatibleStatus);
    virtual void notify_daemon_status(HostStateMachine* , DaemonStatus)     // if NOT_RUNNING set HostUnreachable 
        throw (CdmwPlatformMngt::IncompatibleStatus);
    virtual void notify_processes_status(HostStateMachine* , ProcessesStatus) // do nothing
        throw (CdmwPlatformMngt::IncompatibleStatus) {};
            


};

class HostResponding : public virtual HostState
{
public:
    //HostResponding();
    
    virtual CdmwPlatformMngt::HostStatus status();

    virtual void start(HostStateMachine* )
        throw (CdmwPlatformMngt::IncompatibleStatus);
    virtual void stop(HostStateMachine* )
        throw (CdmwPlatformMngt::IncompatibleStatus);
    virtual void notify_processes_status(HostStateMachine* , ProcessesStatus) 
        throw (CdmwPlatformMngt::IncompatibleStatus);
        
};


class HostStarting : public virtual HostState
{
public:
    //HostStarting();

    virtual CdmwPlatformMngt::HostStatus status();

    virtual void doActivity (BaseHostStateMachine* stateMachine);
    
    virtual void stop(HostStateMachine* )
        throw (CdmwPlatformMngt::IncompatibleStatus);
    virtual void notify_processes_status(HostStateMachine* , ProcessesStatus)
        throw (CdmwPlatformMngt::IncompatibleStatus);
};

class HostReinserting : public virtual HostState
{
public:
    //HostReinserting();

    virtual CdmwPlatformMngt::HostStatus status();

};

class HostStarted : public virtual HostState
{
public:
    //HostStarted();

    virtual CdmwPlatformMngt::HostStatus status();

    virtual void stop(HostStateMachine* )
        throw (CdmwPlatformMngt::IncompatibleStatus);
    virtual void notify_processes_status(HostStateMachine* , ProcessesStatus) 
        throw (CdmwPlatformMngt::IncompatibleStatus);
    virtual void doActivity(BaseHostStateMachine* stateMachine);

};

class HostPartialStarted : public virtual HostState
{
public:
    //HostPartialStarted();

    virtual CdmwPlatformMngt::HostStatus status();

    virtual void stop(HostStateMachine* )
        throw (CdmwPlatformMngt::IncompatibleStatus);
    virtual void notify_processes_status(HostStateMachine* , ProcessesStatus) 
        throw (CdmwPlatformMngt::IncompatibleStatus);
};

class HostStopping : public virtual HostState
{
public:
    //HostStopping();

    virtual CdmwPlatformMngt::HostStatus status();

    virtual void doActivity (BaseHostStateMachine* stateMachine);

    virtual void entryAction (BaseHostStateMachine* stateMachine);        
    
    virtual void stop(HostStateMachine* )
        throw (CdmwPlatformMngt::IncompatibleStatus);
    virtual void notify_processes_status(HostStateMachine* , ProcessesStatus) 
        throw (CdmwPlatformMngt::IncompatibleStatus);
};

class HostUnreachable : public virtual HostState
{
public:
    //HostUnreachable();

    virtual CdmwPlatformMngt::HostStatus status();
    
    virtual void exitAction (BaseHostStateMachine* stateMachine);        

    virtual void notify_daemon_status(HostStateMachine* , DaemonStatus)
            throw (CdmwPlatformMngt::IncompatibleStatus);
//    virtual void stop(HostStateMachine* )
//            throw (CdmwPlatformMngt::IncompatibleStatus);
        
};


};
};

#endif /*HOSTSTATES_HPP_*/
