/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_PLATFORMMNGT_HOSTSTATUSCHECKER_HPP
#define INCL_PLATFORMMNGT_HOSTSTATUSCHECKER_HPP

#include <Foundation/idllib/CdmwDnC.stub.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/osthreads/Mutex.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtProcess.stub.hpp>


namespace Cdmw
{
namespace PlatformMngt
{

class HostStateMachine;
class Host_impl;

 /**
  *Purpose:
  *<p> execute host status consistency periodically
  */
class HostStatusConsistencyChecker : public OsSupport::Thread
{

private:

       /**
        * Period (in ms) between different host status computings
        */
        long m_sleeping_period;

       /**
        * Pointer to status checked host
        */
        Host_impl* m_attached_host;

       /**
        * Pointer to host state machine
        */
        HostStateMachine* m_state_machine;

       /**
        * boolean to stop this thread
        */
        bool m_stop_flag;

       /**
        * Mutex for setter/getter boolean above
        */
        OsSupport::Mutex m_stop_flag_mutex;

       /**
        * Mutex for setter/getter for m_sleeping_period
        */
        OsSupport::Mutex m_sleeping_period_mutex;

       /**
        *Purpose:
        *<p> getter stop flag value
        */
        bool getStopFlag ();

       /**
        *Purpose:
        *<p> inits stop flag value to false
        */
        void initStopFlag();

       /**
        *Purpose:
        *<p> getter sleeping period
        */
        long getSleepingPeriod();

       /**
        *Purpose:
        *<p> inits sleeping period to 1500ms
        */
        void initSleepingPeriod();

       /**
        *Purpose:
        *<p> computes host status consistency
        */
        void computeStatusConsistency ()
             throw (CORBA::SystemException);

       /**
        * Processes pointer sequence
        */
        CdmwPlatformMngt::Processes_var m_all_processes;

       /**
        * Processes sequence length
        */
        CORBA::ULong m_seq_size;

public:

        HostStatusConsistencyChecker(Host_impl* attaching_host, 
            HostStateMachine* state_machine);

        virtual ~HostStatusConsistencyChecker() throw();

       /**
        *Purpose:
        *<p> execute status consistency checking
        */
        void execute() throw ();

       /**
        * Purpose:
        * <p>run the thread
        */
        virtual void run() throw();

       /**
        * Purpose:
        * <p>stop the thread
        */
        virtual void stop() throw();

       /**
        *Purpose:
        *<p> getter method of the sleeping period
        */
        long get_sleeping_period();

       /**
        *Purpose:
        *<p> setter method of the sleeping period
        */
        void set_sleeping_period(long sleeping_period);

};

}// closing namespaces
}

#endif /*INCL_PLATFORMMNGT_HOSTSTATUSCHECKER_HPP*/
