/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_HOST_TREES_HPP
#define INCL_PLATFORMMNGT_HOST_TREES_HPP

#include "Foundation/common/Exception.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "platformsystemlibrary/System_impl.hpp"
#include "platformsystemlibrary/CentralisedSupervisionSystem_impl.hpp"
#include "platformapplicationlibrary/HostProcessInitSequencer.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtApplication.stub.hpp"
#include <list>
#include <map>

namespace Cdmw
{
    namespace PlatformMngt
    {
        class HostTrees
        {

            public:

            /**
             * Purpose:
             * <p> obtain a HostTree object reference
             */

            static HostTrees* getInstance();

            /**
             * Purpose:
             * <p> compose HostInitProcessSequencer process map
             */

            void composeMaps (System_impl* system);

            private:

            HostTrees();

            /**
             * Purpose:
             * <p> Analyse init graphs
             */

            std::list<std::string> composeListFromGraph (const CdmwPlatformMngt::GraphElements& elements_graph);

            /**
             * Purpose:
             * <p> compose HostInitProcessSequencer process map from application sequence
             */

            void compose_map (std::list<std::string> applications_sequence, 
                       CentralisedSupervisionSystem_impl* system, MapHostProcess result_map []);

            MapStopSequences compose_flat_map (std::list<std::string> applications_sequence,
                                           CentralisedSupervisionSystem_impl* system);

            std::list<std::string> composeFlatListFromGraph (const CdmwPlatformMngt::GraphElements& elements_graph);

            void getTaskInfo(CdmwPlatformMngt::GraphElement* graphElement,
            std::string tsk_info [2]);

            std::map <std::string, std::string> composePrecondMapFromGraph (const CdmwPlatformMngt::GraphElements& elements_graph);

            void apply_preconditions_correction (MapHostProcess result_map [], const CdmwPlatformMngt::GraphElements& elements_graph, 
                 CentralisedSupervisionSystem_impl* system);

        };

    } // End namespace PlatformMngt
} // End namespace Cdmw

#endif //INCL_PLATFORMMNGT_HOST_TREES_HPP

