/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_HOST_IMPL_HPP
#define INCL_PLATFORMMNGT_HOST_IMPL_HPP

#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/osthreads/Mutex.hpp"
#include <Foundation/osthreads/Thread.hpp>
#include "SystemMngt/platformlibrary/Callback.hpp"
#include "SystemMngt/platformlibrary/DeactivableServant_impl.hpp"
#include "SystemMngt/platformelementlibrary/ManagedElement_impl.hpp"
#include "SystemMngt/platformelementlibrary/EntityContainer.hpp"
#include "SystemMngt/platformelementlibrary/DataStoreDefinition.hpp"

#include "SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtEntity.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtEvent.stub.hpp"

#include "SystemMngt/idllib/CdmwPlatformMngtElementRepository.stub.hpp"

#include "SystemMngt/idllib/CdmwPlatformMngtHost.skel.hpp"
#include "SystemMngt/platformlibrary/EventHeaderFactory.hpp"
#include "SystemMngt/platformapplicationlibrary/HostProcessInitSequencer.hpp"
#include "platformvaluetypes/HostStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/Restart_impl.hpp"
#include "SystemMngt/platformvaluetypes/Stop_impl.hpp"
#include "SystemMngt/platformvaluetypes/Reboot_impl.hpp"
#include "SystemMngt/platformvaluetypes/Shutdown_impl.hpp"
#include "platformsystemlibrary/HostStatusConsistencyChecker.hpp"


#ifdef CDMW_USE_FAULTTOLERANCE
#include "FaultTolerance/idllib/CdmwFTFaultDetector.stub.hpp"
#include "Foundation/commonsvcs/naming/NamingInterface.hpp"
#endif

namespace Cdmw
{
namespace PlatformMngt
{

class Host_impl;
class System_impl;
class RWEntityContainer_impl;
class HostStateMachine;
class DaemonMonitor;
class HostStateMachine;

/**
 * The callback for the responding host.
 */
class HostRespondingCallback : public virtual Callback
{
public:

    /**
     *Purpose:
     *<p> Constructor.
     *
     *@param  host  the host associated to callback.
     */
    HostRespondingCallback (Host_impl* host);
 
    /**
     *Purpose:
     *<p> Destructor.
     */
    virtual ~HostRespondingCallback ();
        
    /**
     *Purpose:
     *<p> Callback execution. notifies the host is responding
     */
    void execute() throw();
    
private:

    /**
     * The host.
     */
    Host_impl* m_host;

};


/**
 * The callback for the not responding host.
 */
class HostNotRespondingCallback : public virtual Callback
{
public:

    /**
     *Purpose:
     *<p> Constructor.
     *
     *@param  host  the host associated to callback.
     */
    HostNotRespondingCallback (Host_impl* host);
   
    /**
     *Purpose:
     *<p> Destructor.
     */
    virtual ~HostNotRespondingCallback ();
    
    /**
     *Purpose:
     *<p> Callback execution. notifies the host is not responding
     */
    void execute() throw();
    
private:

    /**
     * The host.
     */
    Host_impl* m_host;

};



/**
 *Purpose:
 *<p> Provides the host monitoring.
 */
class Host_impl : virtual public POA_CdmwPlatformMngt::Host,
                  public DeactivableServant_impl,
                  public ManagedElement_impl//,
//                  virtual public POA_CdmwPlatformMngt::SupervisionObserver,
{

public:
    
    /**
     * The type of host command to notify.
     */
   /* enum HostCommand
    {
        COLD_RESTART,
        HOT_RESTART,
        COLD_REBOOT,
        HOT_REBOOT,
        SHUTDOWN,
        ACKNOWLEDGE,
        STOP
    };*/
    
    /**
     *Purpose:
     *<p> Constructor.
     * 
     *@param poa       the poa in charge of servants.
     *@param host_def  the host definition.
     *@param host_path the path of the Host.
     *@param system    the system responsible for the host.        
     *@param element_repository the element repository   
     *
     *@exception BadParameterException
     *@exception OutOfMemoryException
     *
     */
#ifdef CDMW_USE_FAULTTOLERANCE
    Host_impl (PortableServer::POA_ptr poa,
               const CdmwPlatformMngt::HostDef& host_def,
               const char* host_path,               
               System_impl* system,
               CdmwPlatformMngt::RWElementRepository_ptr element_repository,
               CdmwFT::FaultManagement::FaultDetector_ptr faultDetector)
#else
    Host_impl (PortableServer::POA_ptr poa,
               const CdmwPlatformMngt::HostDef& host_def,
               const char* host_path,
               System_impl* system,
               CdmwPlatformMngt::RWElementRepository_ptr element_repository)      
#endif
          throw (BadParameterException,
                 OutOfMemoryException,
         CosPropertyService::MultipleExceptions
        );
                    



    /**
     *Purpose:
     *<p> Destructor.
     */
    virtual ~Host_impl();
    
    /**
     *Purpose:
     *<p> Remove the host object
     */
    void remove();

    /** method to get the current number
    of restart attempt **/
    CORBA::Long restart_attempt() throw (CORBA::SystemException);

    /** method to set the current number
    of restart attempt **/
    void set_restart_attempt( CORBA::Long ) throw (CORBA::SystemException);
    

    /** method to get the current number
    of restart attempt **/
    CORBA::Long get_max_restart_attempt() throw (CORBA::SystemException);

    
    
    /**
     *Purpose:
     *<p> Ends the creation of the host servant
     *
     *@exception DuplicateEntity if there is a duplicate entity name among
     * the entities of the process
     */
    void end_creation(const CdmwPlatformMngt::HostDef& host_def)
            throw (CdmwPlatformMngt::DuplicateEntity,
                   OutOfMemoryException,
                   CORBA::SystemException);
    
    /**
     *Purpose:
     *<p> facade to get an attribute
     *
     */
     void get_attribute(const char * attr_name, CORBA::Any_out ret_attr , CdmwPlatformMngt::Owner flag)  
         throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound);

    /**
     *Purpose:
     *<p> facade to set an attribute
     *
     */
     void set_attribute(const char * attr_name, const CORBA::Any & attr ,CdmwPlatformMngt::Owner flag) 
         throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound, CdmwPlatformMngt::ReadOnlyAttribute);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Host/status:1.0
     * operation
     */
    CdmwPlatformMngt::HostStatus status()
         throw (CORBA::SystemException);

    /**
     * to get the host_status_info attribute 
     */
    char* host_status_info()
         throw (CORBA::SystemException);

    CdmwPlatformMngt::CompleteMonitoringInfo get_monitoring_parameters() 
        throw (CORBA::SystemException);    

    void set_monitoring_parameters(const CdmwPlatformMngt::CompleteMonitoringInfo& monitoring_parameters) 
        throw (CORBA::SystemException);

    /**  
    *Purpose:
    *<p> 
    * to get the Host autorestart info.
    */
    CdmwPlatformMngt::AutoRestartInfo get_autorestart_info() 
        throw(CORBA::SystemException); 

    /**  
     *Purpose:
     *<p> 
     * to set the Host autorestart info.
     */
    void set_autorestart_info( const CdmwPlatformMngt::AutoRestartInfo& autorestart_info) 
        throw(CORBA::SystemException); 

    /**
     *Purpose:
     *<p>
     * Implements the
     * get_parent() function from Host to System
     * required for navigability
     */
    CdmwPlatformMngt::ManagedElement_ptr  get_parent()  throw (CORBA::SystemException);
 
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Host/get_processes:1.0
     * operation
     */
    CdmwPlatformMngt::Processes * get_processes (
        CORBA::ULong how_many,
        CdmwPlatformMngt::ProcessesIterator_out processes_iterator)
              throw (CORBA::SystemException);

    void set_trace_filtering(const char *trace_filtering) 
        throw (CORBA::SystemException, CdmwPlatformMngt::ParsingException , CdmwPlatformMngt::CastException ) ;

    /**
     *Purpose:
     *<p>
     * Implements the
     * get_process(const char * , const char *) function from Host to Process
     * required for navigability
     */
    CdmwPlatformMngt::Process_ptr get_process(const char *process_name, const char *application_name) 
         throw (CdmwPlatformMngt::ProcessNotFound, CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * getprocesses() function from Host to Process
     * required for navigability
     */
    CdmwPlatformMngt::Processes * getprocesses() 
        throw (CORBA::SystemException);
   
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Host/shutdown:1.0
     * operation
     */        
    void shutdown ()
        throw (CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Host/cold_reboot:1.0
     * operation
     */ 
    void cold_reboot ()
        throw (CORBA::SystemException);

    /**
     * The mutex used for changing the reboot_in_progress.
     */
    OsSupport::Mutex m_mutexRebootInProgress;

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Host/get_reboot_in_progress:1.0
     * operation
     */
    CORBA::Boolean get_reboot_in_progress()
         throw (CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     */
    void set_reboot_in_progress(CORBA::Boolean reboot_in_progress) ;
        
    /**
     * The mutex used for changing the stop_in_progress.
     */
    OsSupport::Mutex m_mutexStopInProgress;
            
    /**
     *Purpose:
     *<p>
     * get the flag's value that indicates if a stop of the host is in progress.
     */
    CORBA::Boolean get_stop_in_progress() 
         throw (CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * set the flag's value that indicates if a stop of the host is in progress.
     */
    void set_stop_in_progress(CORBA::Boolean stop_in_progress);
 
    /**
     *Purpose:
     *<p>
     * Starts the processes after remote host is REACHABLE again after reboot. 
     */
    void start_processes();

    /**
     *Purpose:
     *<p>
     * Stops the processes before host reboot (restart)
     */
    void stop_processes();
              
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Host/hot_reboot:1.0
     * operation
     */ 
    void hot_reboot ()
         throw (CORBA::SystemException);

     /**
     *Purpose:
     *<p> change host status
     *
     *@param  status  the new status of host.
     */
    void setStatus(CdmwPlatformMngt::HostStatus status, CdmwPlatformMngtBase::TimeStamp time_stamp);
    
    /**
     *Purpose:
     *<p> start the pull monitoring
     *
     */
    void start();
    
    /**
     *Purpose:
     *<p> stop the pull monitoring
     *
     */
    void stop() 
        throw (CORBA::SystemException);
    
    
    /**
     *<p> Notifies the supervision observer that the host status changed.
     *
     */
    void notify_status_change();   
        
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/EntityContainer/get_number_of_entities:1.0
     * operation
     */
    CORBA::ULong get_number_of_entities()
        throw( CORBA::SystemException );

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/EntityContainer/get_all_entity_names:1.0
     * operation
     */
    void get_all_entity_names(
        CORBA::ULong how_many,
        CdmwPlatformMngt::ElementNames_out entity_names,
        CdmwPlatformMngt::ElementNamesIterator_out rest )
            throw( CORBA::SystemException );

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/EntityContainer/get_entity:1.0
     * operation
     */
    CdmwPlatformMngt::Entity_ptr get_entity (const char* entity_name)
        throw( CdmwPlatformMngt::EntityNotFound,
               CORBA::SystemException );

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/EntityContainer/get_entities:1.0
     * operation
     */
    CORBA::Boolean get_entities(const CdmwPlatformMngt::ElementNames& entity_names,
        CdmwPlatformMngt::Entities_out nentities )
            throw( CORBA::SystemException );

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/EntityContainer/get_all_entities:1.0
     * operation
     */
    void get_all_entities(
        CORBA::ULong how_many,
        CdmwPlatformMngt::Entities_out nentities,
        CdmwPlatformMngt::EntitiesIterator_out rest )
            throw( CORBA::SystemException );

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/RWEntityContainer/add_entity:1.0
     * operation
     */
    CdmwPlatformMngt::Entity_ptr add_entity (const CdmwPlatformMngt::EntityDef & entity_def)
        throw(CdmwPlatformMngt::EntityAlreadyExists, CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/RWEntityContainer/remove_entity:1.0
     * operation
     */
    void remove_entity(const char* entity_name )
        throw( CdmwPlatformMngt::EntityNotFound, CORBA::SystemException );

    /**
     *Purpose:
     *<p> Get a Entity implementation.
     *
     *@param entity_name the name identifying the entity.
     *
     */              
     Entity_impl* getEntity (const char* entity_name)
           throw (CORBA::SystemException);
    
    /**
     *Purpose:
     *<p> Synchronise the entities of host
     *
     */
     void synchro_entities_with_datastore();    
     
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ManagedElement/get_children:1.0
     * operation
     */
    virtual CdmwPlatformMngt::Children* get_children ()
     throw (CORBA::SystemException);

    /**
     *Purpose: Register Observer to register also to configuration events
     *@param CORBA::boolean depth information to manage notification to parents
     *<p>
     */
     void register_observer_ext(const char* observer_name,
                                CdmwPlatformMngt::RegistrationDepth depth,
                                CdmwPlatformMngt::SupervisionObserver_ptr observer)
                                    throw (CORBA::SystemException, CdmwPlatformMngt::ElementAlreadyExists);

    /**
     *Purpose: De-Register Observer
     *<p>
     */
    void unregister_observer_ext(const char* observer_name)
        throw (CORBA::SystemException, CdmwPlatformMngt::ObserverNotFound);

    /**
     *Purpose:
     *<p> Synchronise with datastore
     *
     */
    void synchro_with_datastore();
        
    CdmwPlatformMngt::ElementType get_element_type()
        throw (CORBA::SystemException);
        
    void get_all_processes(CORBA::ULong, CdmwPlatformMngt::Processes_out,
                           CdmwPlatformMngt::ProcessesIterator_out)
        throw (CORBA::SystemException);
        
    CORBA::ULong get_number_of_processes()
        throw (CORBA::SystemException);
        
    void cold_restart()
        throw (CORBA::SystemException);
        
    void hot_restart()
        throw (CORBA::SystemException);

    CdmwPlatformMngt::HostSnapshot * get_snapshot ()
        throw (CORBA::SystemException);
    
    virtual void stop_host()
        throw (CORBA::SystemException);
    
    void notify (CdmwPlatformMngt::Event* an_event)
        throw (CORBA::SystemException);

    //void notify_property_change (CdmwPlatformMngt::PropertyChange* an_property_event)
    //    throw (CORBA::SystemException);
    void start_consistency_checker();
    
    friend class HostRespondingCallback;
    friend class HostNotRespondingCallback;

    Cdmw::OsSupport::ThreadStatus get_consistency_checker_status();

protected:

    /**
     * The host state machine
     */
    HostStateMachine* m_state_machine;
    
    /**
     * Daemon last status 
     */
    //HostState::DaemonStatus m_last_daemon_status;

    /**
     * Host Status checker
     */
     HostStatusConsistencyChecker* m_host_status_checker;

private:
    /**
     *Purpose:
     *<p> Copy constructor is not allowed.
     */
    Host_impl (Host_impl& rhs);

    /**
     * Purpose:
     * <p> Assignment operator is not allowed.
     */ 
    Host_impl& operator= (const Host_impl& rhs);
    
    /**
     *Purpose:
     *<p> Create the host record in data store
     *
     *@param hostDef the data for host to start.
     *
     *@exception OutOfMemoryException Lack of memory.
     */
    void create_record_in_datastore(const CdmwPlatformMngt::HostDef& hostDef)
        throw (OutOfMemoryException);
            
    /**
     *Purpose:
     *<p> Remove the host record in data store
     *
     */
    void remove_record_in_datastore();

    /**
     *Purpose:
     *<p> Get the host record from data store
     *
     */
    CdmwPlatformMngt::HostRecord* get_record_from_datastore();
    
    /**
     *Purpose:
     *<p> Update the host record in data store
     *
     */
    void update_record_in_datastore(const CdmwPlatformMngt::HostRecord& host_record);

    /**
     *Purpose:
     *<p> Helper function to start a remote host
     *
     */
    void start_remote_host(std::string hostname)
        throw (CORBA::NO_PERMISSION);

    /**
     *Purpose:
     *<p> Helper function to execute a shell command
     *
     */
    void execute_shell_command(std::string command)
        throw (CORBA::NO_PERMISSION);    
    
    /**
     *Purpose:
     *<p> Helper function to execute a host command
     *
     */
    void execute_host_command(std::string command)
        throw (CORBA::NO_PERMISSION); 
        
    /**
     *Purpose:
     *<p> Helper function to execute a host command
     *
     */
    void execute_host_command_without_daemoncontroller(std::string command)
        throw (CORBA::NO_PERMISSION);
        
    /**
     *Purpose:
     *<p> Helper function to cold/hot reboot an host
     *
     */
    void reboot (bool is_hot_reboot)
        throw (CORBA::SystemException);


    /**
     * The system responsible for the entity.
     */
    System_impl* m_system;
    
    /**
     * The Daemon Pull Monitor.
     */
    DaemonMonitor* m_monitor;
    

#ifdef CDMW_USE_FAULTTOLERANCE

    CdmwFT::FaultManagement::FaultDetector_var m_faultDetector;
    CdmwFT::FaultManagement::MonitoringId m_monitoringID; 
       
#endif

    /**
     * The RW Entity Container object var to take pointer ownership
     */
    PortableServer::ServantBase_var m_rwEntityContainerServant;
    
    /**
     * The actual entity container.
     */
    PlatformMngt::EntityContainer* m_actualEntityContainer;
    
    /**
     * The read/write entity container.
     */
    RWEntityContainer_impl* m_rwEntityContainer;
    

    /**
     * The mutex used for changing the host record.
     */
    OsSupport::Mutex m_mutex_host_record;
    
    /**
     * The host datastore.
     */
    HostDataStore* m_host_ds;

    /**
     * This flag indicates if a reboot of the host is in progress.
     */
    CORBA::Boolean m_reboot_in_progress;

    /**
     * This flag indicates if a stop of the host is in progress.
     */
    CORBA::Boolean m_stop_in_progress;

    /**
     * The entityDef to restore entity default values
     */
    CdmwPlatformMngt::EntityDefs* m_default_entitydefs;

    /**
     * The current restart attempt sequenc. number
     */
    CORBA::Long m_current_restart_attempt;
   
    /**
     * The host status change factory.
     */
    HostStatusChangeFactory* m_host_status_change_factory;
    
    /**
     * The host restart command factory.
     */
    RestartFactory* m_hostRestartFactory;
        
    /**
     * The host stop command factory.
     */
    StopFactory* m_hostStopFactory;
    
    /**
     * The host restart command factory.
     */
    RebootFactory* m_hostRebootFactory;
        
    /**
     * The host stop command factory.
     */
    ShutdownFactory* m_hostShutdownFactory; 
       
    /**
     * The supervision event notifier (notify to external observers).
     */
    SupervisionEventNotifier* m_event_notifier;
    
    /**
     * Current command being performed
     */
    typedef enum {NO_COMMAND, SHUTDOWN, COLD_REBOOT, HOT_REBOOT, STOP, HOT_RESTART, COLD_RESTART} HostCommand;
    HostCommand m_current_host_command;
    
    /**
     * Setter and getter for m_current_host_command
     */
    bool set_current_host_command(HostCommand);
    HostCommand get_current_host_command();
    
    /**
     * Mutex for setter/getter above
     */
    OsSupport::Mutex m_command_mutex;
    
    /**
     * Script names for remote operations
     */
    std::string m_stop_host_script;
    std::string m_start_host_script;
    std::string m_reboot_host_script;
    std::string m_shutdown_host_script;
    
    
    OsSupport::Mutex m_preemption_flag_mutex;
    
    bool m_preemption_flag;
    
    /**
     *<p> Notify process command event  
     *
     */
    void notifyCommandEvent(HostCommand host_command, const char* issuer,
                            CORBA::Boolean emergency=false);
    
    /**
     *Purpose:
     *<p> convert TECH_STATUS property to process status
     *
     */
    CdmwPlatformMngt::HostStatus property_to_status (const char* status)
     throw (CdmwPlatformMngt::AttributeNotFound);

    /**
     *Purpose:
     *<p> init preemption flag to false
     *
     */
    void init_preemption ();
    
   /**
     *Purpose:
     *<p> chck preemption flag value
     *
     */
    
    bool is_preemption_active ();

    /**
     *Purpose:
     *<p> try to activate preemption flag
     *
     */
                
    bool activate_preemption ();

    /**
     *Purpose:
     *<p> try to deactivate preemption flag
     *
     */
                
    bool de_activate_preemption();
};


} // End namespace PlatformMngt
} // End namespace Cdmw

#endif //INCL_PLATFORMMNGT_HOST_IMPL_HPP

