/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <sstream>
#include "platformsystemlibrary/CentralisedSupervisionObserver_impl.hpp"
#include "Foundation/logging/LogManager.hpp"
#include "platformapplicationlibrary/Application_impl.hpp"
#include "platformapplicationlibrary/Process_impl.hpp"
#include "platformsystemlibrary/Host_impl.hpp"
#include "platformsystemlibrary/ConsistentSynchroSolver.hpp"


namespace Cdmw 
{
namespace PlatformMngt 
{

CentralisedSupervisionObserver_impl::CentralisedSupervisionObserver_impl (CentralisedSupervisionSystem_impl* system)
{
    m_system = system;
}
    
void CentralisedSupervisionObserver_impl::notify (CdmwPlatformMngt::Event* an_event)
    throw (CORBA::SystemException)
{
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
            "notify event : "<< an_event->to_string()); 

        switch (an_event->event_kind())
        {
            case CdmwPlatformMngt::HOST_STATUS_CHANGE :
            {
                CdmwPlatformMngt::HostStatusChange* event = CdmwPlatformMngt::HostStatusChange::_downcast( an_event );

                std::string hostname = (const char*)(event->host_name());
                Host_impl* host = m_system->getHost (hostname.c_str());
                host->notify (an_event);
            }
            break;

           case CdmwPlatformMngt::ENTITY_STATUS_CHANGE :
           {
                CdmwPlatformMngt::EntityStatusChange* event = CdmwPlatformMngt::EntityStatusChange::_downcast( an_event );

                std::string entity_name = std::string ( event->entity_name());

                if (entity_name == "CdmwHostSynchronization")
                {
                    Cdmw::PlatformMngt::ConsistentSynchroSolver * consistency_synchro_solver = m_system -> get_consistent_synchro_solver();        

                    Cdmw::OsSupport::ThreadStatus t_status = consistency_synchro_solver->get_status();
                    switch (t_status)  
                    {
                        case Cdmw::OsSupport::THREAD_RUN_REQUEST:
                        case Cdmw::OsSupport::THREAD_RUNNING:

                                consistency_synchro_solver -> join();
                        break;
                        case Cdmw::OsSupport::THREAD_ENDED:
                        case Cdmw::OsSupport::THREAD_CREATED:
                        break;
                    } 

                    consistency_synchro_solver -> start(); 
                }
           }
            break;
            case CdmwPlatformMngt::PROCESS_STATUS_CHANGE :
            {
                CdmwPlatformMngt::ProcessStatusChange* event = CdmwPlatformMngt::ProcessStatusChange::_downcast( an_event );
                std::string hostname = (const char*)(event->process_id().host_name);
                std::string appname = (const char*)(event->process_id().application_name);
                std::string process_name = (const char*)event->process_id().process_name;
                try
                {
                    switch (event->process_status()) 
                    {
                        case CdmwPlatformMngt::PROCESS_FAILED_DEATH:
                        {
                        Application_impl* application_servant=0;
                                ApplicationControl_impl* app_control_servant=0;
                            m_system->getApplicationServants (appname.c_str(),
                                                            application_servant,
                                                            app_control_servant);
                        Process_impl* process_servant=0;
                        process_servant = application_servant->getProcess(process_name.c_str(), hostname.c_str());
                        process_servant->reset_replication_role();
                        
                                int ret_code = application_servant->getProcess(process_name.c_str(), hostname.c_str())
                                    ->perform_exit_autorestart_action(Cdmw::PlatformMngt::FAILED_DEATH);

                                // if ret_code == CdmwPlatformMngt::REBOOT_NODE then host reboot has been performed and current_restart_attempt must be incremented

                                if (ret_code == CdmwPlatformMngt::REBOOT_NODE)
                            {
                            //retrieve host name
                                    string host_name_str = application_servant->getProcess(process_name.c_str(), hostname.c_str())
                                        ->get_process_host_name() ;
                        
                            //retrieve Host_impl
                            Host_impl *host_impl = m_system->getHost(host_name_str.c_str() );
                        
                            CORBA::Long current_host_restart = host_impl->restart_attempt();
                            current_host_restart++;
                        host_impl->set_restart_attempt(current_host_restart);
                        
                            }
                        }
                        break;
                
                        case CdmwPlatformMngt::PROCESS_FAILED_TIMEOUT:
                        case CdmwPlatformMngt::PROCESS_FAILED_INVALID:
                        case CdmwPlatformMngt::PROCESS_FAILED_NO_RESPONSE:
                        case CdmwPlatformMngt::PROCESS_FAILED_DISCREPANCY:
                        case CdmwPlatformMngt::PROCESS_FAILED_ERROR:
                        {
                            //Application_impl*  app_impl=NULL;
                            //app_impl=m_system->get_application_impl(appname.c_str());
                            Application_impl* application_servant=0;
                            ApplicationControl_impl* app_control_servant=0;
                            m_system->getApplicationServants (appname.c_str(),
                                                            application_servant,
                                                            app_control_servant);
                            Process_impl* process_servant=0;
                            process_servant = application_servant->getProcess(process_name.c_str(), hostname.c_str());
                            process_servant->reset_replication_role();
                            
                            int ret_code = application_servant->getProcess(process_name.c_str(), hostname.c_str())
                                    ->perform_exit_autorestart_action(Cdmw::PlatformMngt::ABNORMAL);
                
                            // if ret_code == CdmwPlatformMngt::REBOOT_NODE then host reboot has been performed and current_restart_attempt must be incremented
        
                            if (ret_code == CdmwPlatformMngt::REBOOT_NODE)
                        {
                            //retrieve host name
                                    string host_name_str = application_servant->getProcess(process_name.c_str(), hostname.c_str())
                                        ->get_process_host_name() ;
        
                            //retrieve Host_impl
                            Host_impl *host_impl = m_system->getHost(host_name_str.c_str() );
        
                                    CORBA::Long current_host_restart = host_impl->restart_attempt();
                                    current_host_restart++;
                            host_impl->set_restart_attempt(current_host_restart);
        
                        }
                        }
                        break;
                        case CdmwPlatformMngt::PROCESS_STOPPED:
                        case CdmwPlatformMngt::PROCESS_ENDED:
                        {
                            Application_impl* application_servant=0;
                            ApplicationControl_impl* app_control_servant=0;
                            m_system->getApplicationServants (appname.c_str(),
                                                            application_servant,
                                                            app_control_servant);
                            Process_impl* process_servant=0;
                            process_servant = application_servant->getProcess(process_name.c_str(), hostname.c_str());
                            process_servant->reset_replication_role();
                        }
                        break;    
                        default:
                        break; 
                    }
                }
                catch (const CORBA::SystemException& ) {
                    //rethrow this exception
                    CDMW_WARN(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                    "CentralisedSupervisionObserver_impl::notify CORBA::SystemException" );
        
                    throw;
                }
                catch (...) {
        
                    CDMW_WARN(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),"CentralisedSupervisionObserver_impl::notify Exception" );
                } 
        }
        break;
        default:
        break;
    }          
}

} // End namespace PlatformMngt
} // End namespace Cdmw

