/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "platformsystemlibrary/ConsistentSynchroSolver.hpp"
#include "platformsystemlibrary/Host_impl.hpp"
#include "SystemMngt/platformlibrary/LogMngr.hpp"

#include "SystemMngt/idllib/CdmwPlatformMngtHost.stub.hpp"
#include "platformsystemlibrary/System_impl.hpp"

#include "SystemMngt/platformelementlibrary/EntityContainer_impl.hpp"

#include <string>

namespace Cdmw
{
namespace PlatformMngt
{

using namespace std;


ConsistentSynchroSolver::ConsistentSynchroSolver(Cdmw::PlatformMngt::System_impl* system) 
                                             
{
   m_system = system;
}

ConsistentSynchroSolver::~ConsistentSynchroSolver() throw()
{
}

void ConsistentSynchroSolver::run() throw()
{ 
      
    CDMW_MUTEX_GUARD (m_mutex); 
   
    CDMW_INFO (Cdmw::Logging::Logger::Get_Logger (CdmwPlatformMngtBase::PFMMNG), 
                   "Consolidating system synchronization status X1") ;

    std::string consolidated_status("OK");
    std::string consolidated_status_info("Synchronization status of all hosts is UNKNOWN or STARTING or OK"); 

    CORBA::ULong nb_hosts = m_system -> get_number_of_hosts();
    if(nb_hosts != 0)
    {
        CdmwPlatformMngt::Hosts_var hosts;
        CdmwPlatformMngt::HostsIterator_var rest;
        m_system -> get_all_hosts(nb_hosts,hosts.out(),rest.out());
        
        std::string host_entity_status = "";
        CORBA::String_var host_entity_status_info = CORBA::string_dup("");
        
    //X1
    CdmwPlatformMngt::Entity_var entity_var;
     
        for (CORBA::ULong count = 0; count<nb_hosts; count++)
        {
            try 
            {
                //host_entity_status = hosts[count]->get_entity ("CdmwHostSynchronization") -> get_status (host_entity_status_info.out());
        
        entity_var        = hosts[count]->get_entity ("CdmwHostSynchronization");
        host_entity_status = entity_var->get_status (host_entity_status_info.out());
            }
            catch (const CdmwPlatformMngt::EntityNotFound &)
            {
        CORBA::String_var host_name = hosts[count]->name();
                CDMW_WARN (Cdmw::Logging::Logger::Get_Logger (CdmwPlatformMngtBase::PFMMNG), 
                   "The following host has no CdmwHostSynchronization entity :" << host_name) ;
                   //"The following host has no CdmwHostSynchronization entity :" << hosts[count]->name()) ;
            
            }
            catch (const CORBA::SystemException &)
            {
                CDMW_ERROR (Cdmw::Logging::Logger::Get_Logger (CdmwPlatformMngtBase::PFMMNG), 
                   "Unexpected exception has occurred") ;
            }
            
            if ( host_entity_status == "FAILED" )
            {
                consolidated_status ="FAILED";
                consolidated_status_info ="At least one host has synchronization status FAILED";
                break;
            }
            if ( host_entity_status == "DEGRADED" )
            {
                consolidated_status ="DEGRADED";
                consolidated_status_info ="At least one host has synchronization status DEGRADED";
                break;
            }
        }
        
        //m_system -> get_entity ("CdmwSystemSynchronization") -> set_status (consolidated_status.c_str(), consolidated_status_info.c_str());
    entity_var = m_system -> get_entity ("CdmwSystemSynchronization");
    entity_var -> set_status (consolidated_status.c_str(), consolidated_status_info.c_str());

        CDMW_INFO (Cdmw::Logging::Logger::Get_Logger (CdmwPlatformMngtBase::PFMMNG), 
                   "Consolidating system synchronization status X1 2") ;
               
    }

}

} // End namespace PlatformMngt
} // End namespace Cdmw


