/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "platformsystemlibrary/HostTrees.hpp"
#include "platformsystemlibrary/System_impl.hpp"
#include "platformsystemlibrary/CentralisedSupervisionSystem_impl.hpp"
#include "SystemMngt/platformapplicationlibrary/Application_impl.hpp"
#include "SystemMngt/platformlibrary/GraphUtility.hpp"

#include <list>
#include <vector>
#include <map>
#include <set>
#include <algorithm>

#include <iostream>


namespace Cdmw
{
   namespace PlatformMngt
   {
   
   HostTrees* HostTrees::getInstance()
   {
   static HostTrees host_trees;
   return &host_trees;
   }

   HostTrees::HostTrees ()
   {
   }

   void HostTrees::composeMaps (System_impl* system)
   {

    CentralisedSupervisionSystem_impl* m_system = dynamic_cast<CentralisedSupervisionSystem_impl*>(system);

    if (m_system != 0)
    {
        MapHostProcess init_map [2];

        init_map[0] = MapHostProcess ();
        init_map[1] = MapHostProcess ();

        CdmwPlatformMngt::GraphElements_var appl_graph (m_system->m_applicationInitGraph);
        CdmwPlatformMngt::GraphElements_var appl_stop_graph (m_system->m_applicationStopGraph);

        bool stop_graph_defined = false;

        if (appl_stop_graph->length() > 0)
        {
            stop_graph_defined = true;
        }

        std::list<std::string> applications_sequence = composeListFromGraph (appl_graph.in());

        compose_map (applications_sequence, m_system, init_map);

        apply_preconditions_correction (init_map, appl_graph.in(), m_system);

        MapStopSequences* process_stop_map;

        if (stop_graph_defined)
        {
            std::list<std::string> applications_flat_sequence = composeFlatListFromGraph (appl_stop_graph.in());
            process_stop_map = new MapStopSequences(compose_flat_map (applications_flat_sequence, m_system));
        }
        else
        {
            process_stop_map = new MapStopSequences();
        }

        HostProcessInitSequencer::get_instance()->set_sequencer_map(init_map);
        HostProcessInitSequencer::get_instance()->set_stop_sequences_map(process_stop_map);
    }
    else
    {
        MapHostProcess empty_map [2];

        empty_map[0] = MapHostProcess ();
        empty_map[1] = MapHostProcess ();

        MapStopSequences* process_stop_map = new MapStopSequences();
        process_stop_map->clear();

        HostProcessInitSequencer::get_instance()->set_stop_sequences_map(process_stop_map);

        HostProcessInitSequencer::get_instance()->set_sequencer_map(empty_map);
    }

   }

   void HostTrees::compose_map (std::list<std::string> applications_sequence, 
                                           CentralisedSupervisionSystem_impl* system,
                                           MapHostProcess result_map [])
   {

    try
    {
        CORBA::ULong host_names_number = system->get_number_of_hosts();

        CdmwPlatformMngt::ElementNames_var host_names;

        CdmwPlatformMngt::ElementNamesIterator_var rest_iterator;

        system->get_all_host_names(host_names_number, host_names.out(), rest_iterator.out());

        if (host_names_number > 0)
        {

            for (CORBA::ULong host_index = 0; host_index < host_names_number; host_index++)
            {
                std::string host_string_name (CORBA::string_dup(((*host_names)[host_index])));
            
                ProcessInfos* process_infos_unmanaged = new ProcessInfos ();
                ProcessInfos* process_infos = new ProcessInfos ();
            
                (result_map[0])[host_string_name] = process_infos;
                (result_map[1])[host_string_name] = process_infos_unmanaged;
           }

           std::vector<Application_impl*> application_impls = system->get_application_impls();
           std::list<std::string>::iterator appl_sequence_iterator;

           for (appl_sequence_iterator = applications_sequence.begin(); 
                   appl_sequence_iterator != applications_sequence.end(); appl_sequence_iterator++)
           {
                std::string appl_name = *appl_sequence_iterator;

                CdmwPlatformMngt::Application_var application = system->get_application (appl_name.c_str());

                CORBA::ULong nbr_of_processes = application->get_number_of_processes();

                CdmwPlatformMngt::ProcessesIterator_var rest_of_processes;

                CdmwPlatformMngt::Processes_var application_processes_seq;

                application_processes_seq = application->get_processes (nbr_of_processes, rest_of_processes.out());

                std::vector<CdmwPlatformMngt::Process_var> application_processes;

                for (CORBA::ULong index = 0; index < application_processes_seq->length(); index++)
                {
                    application_processes.push_back(CdmwPlatformMngt::Process::_duplicate (application_processes_seq[index]));
                }

                CdmwPlatformMngt::GraphElements_var graph;

                Application_impl* current_impl;

                for (unsigned int appl_index = 0; appl_index < application_impls.size(); appl_index++)
                {           
                    std::string string_name = std::string(CORBA::string_dup((application_impls[appl_index])->name()));
                       
                    if (string_name == appl_name)
                    {
                        current_impl = application_impls[appl_index];
                        graph = current_impl->m_processInitGraph;
                    }
               }

                std::list <std::string> process_sequence  = composeListFromGraph (graph);

                std::list<std::string>::iterator process_sequence_iterator;

                for (process_sequence_iterator = process_sequence.begin();
                         process_sequence_iterator != process_sequence.end(); process_sequence_iterator++)
                {
                     std::string sequence_process = *process_sequence_iterator;

                     bool is_Id = false;

                     std::string proc_name = "";
                     std::string host_string_name = "";

                     size_t position = sequence_process.find("/");

                     if ( position == std::string::npos)
                     {
                         proc_name = sequence_process;
                     }
                     else
                     {
                         is_Id = true;

                         host_string_name = sequence_process.substr (0, position);
                         proc_name = sequence_process.substr(position+1, sequence_process.length());

                     }

                     CdmwPlatformMngt::Process_var corba_process;

                     unsigned int proc_cursor = 0;

                     while (proc_cursor < application_processes.size())
                     {
                          std::string comparison_name (CORBA::string_dup((application_processes.at(proc_cursor))->name()));

                          if (!is_Id)
                          {

                              if ( comparison_name == proc_name)
                              {
                                  CdmwPlatformMngt::Process_var process = application_processes[proc_cursor];
                                  corba_process = process;

                                  CdmwPlatformMngt::Host_var corba_host = corba_process->get_host();

                                  host_string_name = std::string(CORBA::string_dup(corba_host->name()));
                              }
                          }
                          else
                          {

                             CdmwPlatformMngt::Host_var corba_host = application_processes.at(proc_cursor)->get_host();

                             if (comparison_name == proc_name && 
                                std::string(CORBA::string_dup(corba_host->name())) == host_string_name)
                             {
                                  CdmwPlatformMngt::Process_var process = application_processes[proc_cursor];
                                  corba_process = process;
                             }
                           }

                           proc_cursor = proc_cursor + 1;
                    }

                    bool managed_or_not = true;

                    try 
                    {
                        corba_process->get_life_cycle();
                    }
                    catch (const CdmwPlatformMngt::IncompatibleType&)
                    {
                        managed_or_not = false;
                    }
                    catch (...)
                    {
                        CDMW_WARN(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), 
                            "HostTrees::compose_map: Unknown Exception Caught");
                        throw;
                    }

                    MapIterator proc_iterator = (result_map[0]).find (host_string_name);

                    if (proc_iterator != (result_map[0]).end())
                    {
                        ProcessInfos proc_infos = *proc_iterator->second;

                        std::string process_id = host_string_name+"/"+proc_name;

                        Cdmw::PlatformMngt::ProcessHostInfo info = {
                                               proc_name, 
                                               current_impl->getProcess (process_id.c_str()),
                                               Cdmw::PlatformMngt::DEFAULT}; 

                        proc_infos.push_back(info);

                       *proc_iterator->second = proc_infos;

                        if (managed_or_not)
                        {
                           MapIterator managed_proc_iterator = (result_map[1]).find (host_string_name);

                           if (managed_proc_iterator != (result_map[1]).end())
                           {
                              ProcessInfos managed_proc_infos = *managed_proc_iterator->second;

                              managed_proc_infos.push_back(info);

                              *managed_proc_iterator->second = proc_infos;
                           }

                        }

                   }
                }
              }
           }
       }
       catch (...)
       {
          CDMW_WARN(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "HostTrees::compose_map: Unknown Exception Caught");
       }
   }

   std::list<std::string> HostTrees::composeListFromGraph (const CdmwPlatformMngt::GraphElements& elements_graph)
   {
      std::list <std::string> element_sequence;

      try 
      {
      std::set <std::string> names_set;
      std::set <std::string> cyclic_names;

      std::list<CdmwPlatformMngt::GraphElement_var> elements;

      for (CORBA::ULong alfa = 0; alfa < elements_graph.length(); alfa++)
      {
          elements.push_back(CdmwPlatformMngt::GraphElement::_downcast((elements_graph[alfa])->_copy_value()));
      }

      bool no_stop_flag = true;

      while (no_stop_flag)
      {
        // is a string set 
        GraphElementNames names;

       for( std::list<CdmwPlatformMngt::GraphElement_var>::iterator iterator = elements.begin();
            iterator != elements.end(); iterator++ )
        {
            std::string newName (CORBA::string_dup((*iterator)->name()));

            if( names.find( newName ) == names.end() )
            {
                names.insert( newName );
            }

        }

        GraphElementNames::iterator names_iterator;
    
        bool is_cyclic = false;
    
        if (!names_set.empty())
          {
    
              std::set<std::string>::iterator cyclic_names_iterator;
   
              for (names_iterator = names.begin(); names_iterator != names.end(); names_iterator++)
              {
                  cyclic_names_iterator = find (names_set.begin(), names_set.end(), *names_iterator);
            
                  if (cyclic_names_iterator != names_set.end())
                  {
                      cyclic_names.insert(*cyclic_names_iterator);
                      is_cyclic = true;
                  }
            
             }
    
          }

          for (names_iterator = names.begin(); names_iterator != names.end(); names_iterator++)
          {    
              names_set.insert(*names_iterator);
          }
    
          for (names_iterator = names.begin(); names_iterator != names.end(); names_iterator++)
          {    
              std::string current_element = *names_iterator;

              std::set<std::string>::iterator cyclic_names_iterator;
        
              if (! cyclic_names.empty())
              {
                  cyclic_names_iterator = find (cyclic_names.begin(), cyclic_names.end(), current_element);

                  if (cyclic_names_iterator == cyclic_names.end())
                  {
                      element_sequence.push_back(*names_iterator);
                  }
                  else
                  {
                      std::list<std::string>::iterator cyclic_occurrence = find (element_sequence.begin(), element_sequence.end(), current_element);

                      element_sequence.remove(*cyclic_occurrence);

                      element_sequence.push_back(*names_iterator);
                  }
              }
              else
              {
                  element_sequence.push_back(*names_iterator);
              }
          }

          std::list<CdmwPlatformMngt::GraphElement_var> successors;
                        
          for (std::list<CdmwPlatformMngt::GraphElement_var>::iterator iterator = elements.begin(); 
            iterator != elements.end(); iterator++)
          {
              CdmwPlatformMngt::GraphElement_var element = *iterator;

              CdmwPlatformMngt::GraphElements other_successors = element->successors();

              for (CORBA::ULong succ = 0; succ < other_successors.length(); succ++)
              {
                  CdmwPlatformMngt::GraphElement_var successor_element = other_successors[succ];

                  successors.push_back(CdmwPlatformMngt::GraphElement::_downcast(successor_element->_copy_value()));

              }
          }

         if (successors.empty())
         {
            no_stop_flag = false;
         }
         else
         {
            elements.clear();
            
            for (std::list<CdmwPlatformMngt::GraphElement_var>::iterator iterator = successors.begin(); 
               iterator != successors.end(); iterator++)
            {
               elements.push_back(CdmwPlatformMngt::GraphElement::_downcast((*iterator)->_copy_value()));
            }
         }

      }

      }
      catch (...)
      {
          CDMW_WARN(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "HostTrees::composeListFromGraph: Unknown Exception Caught");  
      }
      return element_sequence;
   }

   std::list<std::string> HostTrees::composeFlatListFromGraph (const CdmwPlatformMngt::GraphElements& elements_graph)
   {

      std::list <std::string> element_sequence;

      try 
      {
      std::set <std::string> names_set;
      std::set <std::string> cyclic_names;

      std::list<CdmwPlatformMngt::GraphElement_var> elements;

      for (CORBA::ULong alfa = 0; alfa < elements_graph.length(); alfa++)
      {
          elements.push_back(CdmwPlatformMngt::GraphElement::_downcast((elements_graph[alfa])->_copy_value()));
      }

      bool no_stop_flag = true;

      while (no_stop_flag)
      {
        // is a string set 
        GraphElementNames names;

       for( std::list<CdmwPlatformMngt::GraphElement_var>::iterator iterator = elements.begin();
            iterator != elements.end(); iterator++ )
        {
            std::string newName (CORBA::string_dup((*iterator)->name()));

            if( names.find( newName ) == names.end() )
            {
                names.insert( newName );
            }

        }

        GraphElementNames::iterator names_iterator;

        bool is_cyclic = false;

        if (!names_set.empty())
          {

              std::set<std::string>::iterator cyclic_names_iterator;

              for (names_iterator = names.begin(); names_iterator != names.end(); names_iterator++)    
              {
                  cyclic_names_iterator = find (names_set.begin(), names_set.end(), *names_iterator);

                  if (cyclic_names_iterator != names_set.end())
                  {
                      cyclic_names.insert(*cyclic_names_iterator);
                      is_cyclic = true;
                  }

             }

          }

          for (names_iterator = names.begin(); names_iterator != names.end(); names_iterator++)
          {
              names_set.insert(*names_iterator);
          }

          for (names_iterator = names.begin(); names_iterator != names.end(); names_iterator++)
          {    
              std::string current_element = *names_iterator; 

              std::set<std::string>::iterator cyclic_names_iterator;

              if (! cyclic_names.empty())
              {
                  cyclic_names_iterator = find (cyclic_names.begin(), cyclic_names.end(), current_element);

                  if (cyclic_names_iterator == cyclic_names.end())
                  {
                      element_sequence.push_back(*names_iterator);
                  }
                  else
                  {
                  // Skips insertion
                  }
              }
              else
              {
                  element_sequence.push_back(*names_iterator);
              }
          }

          std::list<CdmwPlatformMngt::GraphElement_var> successors;

          for (std::list<CdmwPlatformMngt::GraphElement_var>::iterator iterator = elements.begin(); 
            iterator != elements.end(); iterator++)
          {
              CdmwPlatformMngt::GraphElement_var element = *iterator;

              CdmwPlatformMngt::GraphElements other_successors = element->successors();

              for (CORBA::ULong succ = 0; succ < other_successors.length(); succ++)
              {
                  CdmwPlatformMngt::GraphElement_var successor_element = other_successors[succ];

                  successors.push_back(CdmwPlatformMngt::GraphElement::_downcast(successor_element->_copy_value()));

              }
          }
          
         if (successors.empty())
         {
            no_stop_flag = false;
         }
         else
         {
            elements.clear();

            for (std::list<CdmwPlatformMngt::GraphElement_var>::iterator iterator = successors.begin(); 
               iterator != successors.end(); iterator++)
            {
               elements.push_back(CdmwPlatformMngt::GraphElement::_downcast((*iterator)->_copy_value()));
            }
         }
        
      }

      }
      catch (...)
      {
          CDMW_WARN(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "HostTrees::composeFlatListFromGraph: Unknown Exception Caught");  
      }
      return element_sequence;
   }

   MapStopSequences HostTrees::compose_flat_map (std::list<std::string> applications_sequence,
                                           CentralisedSupervisionSystem_impl* system)
   {

     MapStopSequences resultMap;
     resultMap.clear();

    try
    {
        CORBA::ULong host_names_number = system->get_number_of_hosts();

        CdmwPlatformMngt::ElementNames_var host_names;

        CdmwPlatformMngt::ElementNamesIterator_var rest_iterator;

        system->get_all_host_names(host_names_number, host_names.out(), rest_iterator.out());

        if (host_names_number > 0)
        {

           for (unsigned int init_index = 0; init_index < host_names->length(); init_index++)
           {
                CdmwPlatformMngt::ElementName host_name = CORBA::string_dup(host_names[init_index]);

                std::string key_host = std::string ((char *) host_name);

                std::list<std::string> list;

                resultMap.insert (std::pair<std::string, std::list<std::string> >(key_host, list));
           }

           std::vector<Application_impl*> application_impls = system->get_application_impls();

           std::list<std::string>::iterator appl_sequence_iterator;

           for (appl_sequence_iterator = applications_sequence.begin(); 
                   appl_sequence_iterator != applications_sequence.end(); appl_sequence_iterator++)
           {
                std::string appl_name = *appl_sequence_iterator;

                CdmwPlatformMngt::GraphElements_var graph;

                Application_impl* current_impl;

                for (unsigned int appl_index = 0; appl_index < application_impls.size(); appl_index++)
                {
                    std::string string_name = std::string(CORBA::string_dup((application_impls[appl_index])->name()));

                    if (string_name == appl_name)
                    {
                        current_impl = application_impls[appl_index];
                        graph = current_impl->m_processInitGraph;

                   }
               }

                if (graph->length() > 0)
                {
                    std::list <std::string> process_sequence = composeFlatListFromGraph (graph.in());

                    std::list<std::string>::iterator process_sequence_iterator;

                    for (process_sequence_iterator = process_sequence.begin();
                             process_sequence_iterator != process_sequence.end(); 
                                 process_sequence_iterator++)
                    {
                         std::string sequence_process = *process_sequence_iterator;

                         bool is_Id = false;

                         std::string proc_name = "";
                         std::string host_string_name = "";

                         size_t position = sequence_process.find("/");

                         if ( position == std::string::npos)
                         {
                             proc_name = std::string (sequence_process);

                             CORBA::ULong nbr_of_processes = current_impl->get_number_of_processes();

                             CdmwPlatformMngt::ProcessesIterator_var rest_of_processes;

                             CdmwPlatformMngt::Processes_var processes;

                             processes = current_impl->get_processes (nbr_of_processes, rest_of_processes.out());

                             unsigned int proc_cursor = 0;

                             while (proc_cursor < processes->length())
                             {

                                  CdmwPlatformMngt::Process_var process = processes [proc_cursor];

                                  std::string comparison_name (CORBA::string_dup(process->name()));

                                  if ( comparison_name == proc_name)
                                  {

                                     CdmwPlatformMngt::Host_var process_host = process->get_host();

                                     host_string_name = std::string(CORBA::string_dup(process_host->name()));

                                     (resultMap[host_string_name]).push_back(proc_name);
                                  }

                                  proc_cursor = proc_cursor + 1;
                             }
                         }
                         else
                         {
                             is_Id = true;
                             host_string_name = sequence_process.substr (0, position);
                             proc_name = std::string (sequence_process.substr(position+1, sequence_process.length()));

                             (resultMap[host_string_name]).push_back(proc_name);
                         }
                    } //processes for
                }// application stop_graph defined
              }
           }
       }
       catch (...)
       {
          CDMW_WARN(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "HostTrees::compose_flat_map: Unknown Exception Caught");
       }

       return resultMap;

   }

void HostTrees::apply_preconditions_correction (MapHostProcess result_map [], 
    const CdmwPlatformMngt::GraphElements& elements_graph, 
    CentralisedSupervisionSystem_impl* system)
{
    std::multimap <std::string, std::string> filtered_preconditions_map;
    filtered_preconditions_map.clear();

    std::map<std::string, std::list<std::string> > successors_constraints;
    successors_constraints.clear();

    std::list<std::string> applications_flat_sequence = composeFlatListFromGraph (elements_graph);

    std::vector<Application_impl*> application_impls = system->get_application_impls();

    CdmwPlatformMngt::GraphElements_var graph;

    Application_impl* current_impl;

    for (unsigned int appl_index = 0; appl_index < application_impls.size(); appl_index++)
    {
        current_impl = application_impls[appl_index];
        graph = current_impl->m_processInitGraph;

        // Map filled with waiting-element-name -  waited-element-name
        std::map <std::string, std::string> precondition_map;
        precondition_map.clear();

        precondition_map = composePrecondMapFromGraph (graph.in());

        if (!precondition_map.empty ())
        {

            std::map <std::string, std::string>::iterator preconditions_it = precondition_map.begin();

            for (; preconditions_it != precondition_map.end(); ++preconditions_it)
            {

                 std::string waiting_element = preconditions_it->first;

                 std::string proc_name = "";
                 std::string host_string_name = "";

                 size_t position = waiting_element.find("/");

                 if ( position == std::string::npos)
                 {
                     proc_name = waiting_element;

                     CORBA::ULong nbr_of_processes = current_impl->get_number_of_processes();

                     CdmwPlatformMngt::ProcessesIterator_var rest_of_processes;

                     CdmwPlatformMngt::Processes_var application_processes_seq;

                     application_processes_seq = current_impl->get_processes (nbr_of_processes, rest_of_processes.out());

                     for (CORBA::ULong index = 0; index < application_processes_seq->length(); 
                        index++)
                     {
                         CORBA::String_var process_name ( (application_processes_seq[index])->name());

                         std::string std_process_name (process_name);

                         if (std_process_name == proc_name)
                         {
                             CdmwPlatformMngt::Host_var process_host = (application_processes_seq[index])->get_host();
                             host_string_name = std::string(CORBA::string_dup(process_host->name()));
                         }

                     }

                 }
                 else
                 {
                     host_string_name = waiting_element.substr (0, position);
                     proc_name = waiting_element.substr(position+1, waiting_element.length());
                 }

                 std::string waited_element = preconditions_it->second;

                 std::list<std::string>::iterator is_app_element = 
                     find( applications_flat_sequence.begin(), applications_flat_sequence.end(),
                         waited_element);

                 if (is_app_element == applications_flat_sequence.end())
                 {
                     bool found_process  = false;

                     CORBA::ULong nbr_of_processes = current_impl->get_number_of_processes();

                     CdmwPlatformMngt::ProcessesIterator_var rest_of_processes;

                     CdmwPlatformMngt::Processes_var application_processes_seq;

                     application_processes_seq = current_impl->get_processes (nbr_of_processes, rest_of_processes.out());

                     for (CORBA::ULong index = 0; index < application_processes_seq->length(); 
                        index++)
                     {
                         CORBA::String_var process_name ( (application_processes_seq[index])->name());

                         std::string std_process_name (process_name);

                         if (std_process_name == waited_element)
                         {
                             CdmwPlatformMngt::Host_var process_succ_host = (application_processes_seq[index])->get_host();
                             std::string element_host_string_name = 
                                 std::string(CORBA::string_dup(process_succ_host->name()));

                             if ( element_host_string_name == host_string_name)
                             {

                               found_process = true;

                               //Map swaps elements order waited-element-key - waiting-element-key
                               std::string wed_proc_name (application_processes_seq[index]->name());

                               std::string wed_key = wed_proc_name + std::string ("/");
                               wed_key += host_string_name;

                               std::string win_value = proc_name + std::string ("/");
                               win_value += host_string_name;

                               filtered_preconditions_map.insert(
                                   std::pair<std::string, std::string>( wed_key, win_value ));

                             }

                         }

                     }

                     if (!found_process)
                     {
                        CDMW_WARN(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), 
                          "HostTrees::apply_preconditions_correction: Unable to apply one correction");
                     }

                 }
                 else
                 {
                     CdmwPlatformMngt::Application_var application = 
                           system->get_application ((*is_app_element).c_str());

                     CORBA::ULong nbr_of_processes = application->get_number_of_processes();

                     CdmwPlatformMngt::ProcessesIterator_var rest_of_processes;

                     CdmwPlatformMngt::Processes_var application_processes_seq;

                     application_processes_seq = application->get_processes (nbr_of_processes, rest_of_processes.out());

                     for (CORBA::ULong index = 0; index < application_processes_seq->length(); 
                        index++)
                     {
                         CdmwPlatformMngt::Host_var process_host = application_processes_seq[index]->get_host();

                         std::string element_host_string_name = std::string(CORBA::string_dup(process_host->name()));

                         if (host_string_name == element_host_string_name)
                         {
                             //Map swaps elements order waited-element-key - waiting-element-key
                             std::string wed_proc_name (application_processes_seq[index]->name());

                             std::string wed_key = wed_proc_name + std::string ("/");
                             wed_key += host_string_name;

                             std::string win_value = proc_name + std::string ("/");
                             win_value += host_string_name;

                             filtered_preconditions_map.insert(
                                std::pair<std::string, std::string>( wed_key, win_value ));

                         }
                     }

                     CORBA::ULong nbr_of_processes_for_successors = current_impl->get_number_of_processes();

                     CdmwPlatformMngt::ProcessesIterator_var rest_of_processes_succ;

                     CdmwPlatformMngt::Processes_var application_processes_succ;

                     application_processes_succ = current_impl->get_processes (nbr_of_processes_for_successors, rest_of_processes_succ.out());

                     for (CORBA::ULong alfa = 0; alfa < graph->length(); alfa++)
                     {
                          std::string graph_element_name = graph[alfa]->name();

                          std::string element_name = "";
                          std::string element_host_name = "";

                          size_t slash_position = graph_element_name.find("/");

                          if ( slash_position == std::string::npos)
                          {
                              element_name = graph_element_name;

                              for (CORBA::ULong index = 0; index < nbr_of_processes_for_successors; 
                                index++)
                              {
                                  CORBA::String_var process_name ( (application_processes_succ[index])->name());

                                  std::string std_process_name (process_name);

                                  if (std_process_name == element_name)
                                  {
                                      CdmwPlatformMngt::Host_var process_host = (application_processes_succ[index])->get_host();
                                      element_host_name = std::string(CORBA::string_dup(process_host->name()));
                                  }

                              }

                         }
                         else
                         {
                              element_host_name = graph_element_name.substr (0, slash_position);
                              element_name = graph_element_name.substr(slash_position+1, graph_element_name.length());
                         }

                         if (proc_name == element_name && host_string_name == element_host_name)
                         {

                             if (((graph[alfa])->successors()).length() > 0)
                             {

                                std::list<std::string> useful_successors;

                                std::list<std::string> waiting_successors = 
                                   composeFlatListFromGraph((graph[alfa])->successors());

                                std::list<std::string>::iterator successor_it = waiting_successors.begin();

                                for (; successor_it != waiting_successors.end();
                                   ++successor_it)
                                {
                                    std::string successor_graph_element_name = *successor_it;

                                    std::string successor_element_name = "";
                                    std::string successor_element_host_name = "";

                                    size_t slash_position = graph_element_name.find("/");

                                   if ( slash_position == std::string::npos)
                                   {
                                      successor_element_name = successor_graph_element_name;

                                      for (CORBA::ULong index = 0; index < nbr_of_processes_for_successors; 
                                         index++)
                                      {
                                           CORBA::String_var process_name ( (application_processes_succ[index])->name());

                                           std::string std_process_name (process_name);

                                           if (std_process_name == successor_element_name)
                                           {
                                              CdmwPlatformMngt::Host_var process_host = (application_processes_succ[index])->get_host();
                                              successor_element_host_name = std::string(CORBA::string_dup(process_host->name()));
                                           }

                                      }
                                   }
                                   else
                                   {
                                       successor_element_host_name = successor_graph_element_name.substr (0, position);
                                       successor_element_name = successor_graph_element_name.substr(position+1, waiting_element.length());
                                   }

                                   if (successor_element_host_name == element_host_name)
                                   {
                                       useful_successors.push_back(successor_element_name);
                                   }

                                }

                                 if (useful_successors.size()> 0)
                                 {
                                      std::string win_value = element_name + std::string ("/");
                                      win_value += element_host_name;

                                      successors_constraints[win_value] = useful_successors;
                                 }

                             }

                         }

                    }
                 }
            }
        }
    }

    if (!filtered_preconditions_map.empty())
    {
        std::multimap<std::string, std::string>::iterator preconditions_it 
           = filtered_preconditions_map.begin();

        std::string host_name = "";

        for (; preconditions_it != filtered_preconditions_map.end(); ++preconditions_it)
        {
           std::string waited_process = preconditions_it->first;

           std::string wed_proc_name = "";
           std::string wed_host_string_name = "";

           size_t position = waited_process.find("/");

           if ( position == std::string::npos)
           {
               CDMW_WARN(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), 
                      "HostTrees::apply_preconditions_correction: Unable to apply one correction");
           }
           else
           {
                wed_host_string_name = waited_process.substr (position+1, waited_process.length());
                wed_proc_name = waited_process.substr(0, position);

                std::string waiting_process = preconditions_it->second;

                std::string win_proc_name = "";

                position = std::string::npos;

                position = waiting_process.find("/");

                if (position == std::string::npos)
                {
                    CDMW_WARN(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), 
                        "HostTrees::apply_preconditions_correction: Unable to apply one correction");
                }
                else
                {
                    win_proc_name = waiting_process.substr(0, position);

                    MapIterator proc_iterator = (result_map[0]).find (wed_host_string_name);

                    if (proc_iterator != (result_map[0]).end())
                    {
                        ProcessInfos proc_infos = *proc_iterator->second;

                        int waiting_process_idx = 0;
                        int waited_process_idx = 0;

                        int cursor_index = 0;

                        int infos_size = proc_infos.size();

                        for (; cursor_index < infos_size; ++cursor_index)
                        {
                             std::string element_name ((proc_infos.at(cursor_index)).process_name);

                             if (element_name == win_proc_name)
                             {
                                 waiting_process_idx = cursor_index;
                             }
                             else if (element_name == wed_proc_name)
                             {
                                 waited_process_idx = cursor_index;
                             }
                             else
                             {
                             }
                        }

                         if (waited_process_idx > waiting_process_idx)
                         {
                             int idx_difference = waited_process_idx - waiting_process_idx;

                             for (int swap_idx = 0; swap_idx < idx_difference; swap_idx++)
                             {
                                 swap (proc_infos[waiting_process_idx + swap_idx], proc_infos[waiting_process_idx + swap_idx + 1]);
                             }

                             MapIterator changing_proc_iterator = (result_map[0]).find (wed_host_string_name);

                             *changing_proc_iterator->second = proc_infos;
                         }

                   }
                   else
                   {
                        CDMW_WARN(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), 
                         "HostTrees::apply_preconditions_correction: Unable to apply one correction");
                   }
               }
            }
       }

       if (!successors_constraints.empty())
       {
          std::map<std::string, std::list<std::string> >::iterator useful_successors_it =
              successors_constraints.begin();

          for (; useful_successors_it != successors_constraints.end(); ++useful_successors_it)
          {
              std::string first_element = useful_successors_it->first;

              std::string first_element_name = "";

              std::string first_element_host_name = "";

              size_t seed_position = first_element.find("/");

              if ( seed_position == std::string::npos)
              {
                 CDMW_WARN(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), 
                    "HostTrees::apply_preconditions_correction: Unable to apply one correction");
              }
              else
              {
                 first_element_host_name = first_element.substr (seed_position+1, first_element.length());
                 first_element_name = first_element.substr(0, seed_position);
              }

              MapIterator proc_iterator = (result_map[0]).find (first_element_host_name);

              if (proc_iterator != (result_map[0]).end())
              {
                  ProcessInfos proc_infos = *proc_iterator->second;

                  int infos_size = proc_infos.size();

                  std::list<std::string>::iterator us_it = (useful_successors_it->second).begin();

                  for (; us_it != (useful_successors_it->second).end(); ++us_it)
                  {
                      int cursor_index = 0;

                      int first_process_idx = 0;
                      int successor_process_idx = 0;

                      for (; cursor_index < infos_size; ++cursor_index)
                      {

                          std::string element_name ((proc_infos.at(cursor_index)).process_name);

                          if (element_name == first_element_name)
                          {
                                 first_process_idx = cursor_index;
                          }
                          else if (element_name == *us_it)
                          {
                                 successor_process_idx = cursor_index;
                          }

                      }

                      if (first_process_idx > successor_process_idx)
                      {
                           int idx_difference = first_process_idx - successor_process_idx;

                             for (int swap_idx = 0; swap_idx < idx_difference; swap_idx++)
                             {
                                 swap (proc_infos[successor_process_idx + swap_idx], proc_infos[successor_process_idx + swap_idx + 1]);
                             }

                             MapIterator changing_proc_iterator = (result_map[0]).find (first_element_host_name);

                             *changing_proc_iterator->second = proc_infos;
                      }

                  }

              }

          }

       }

    }

}

   std::map <std::string, std::string> HostTrees::composePrecondMapFromGraph (const CdmwPlatformMngt::GraphElements& elements_graph)
   {

      std::map <std::string, std::string> precondition_map;

      precondition_map.clear();

      try 
      {
      std::set <std::string> names_set;

      std::list<CdmwPlatformMngt::GraphElement_var> elements;

      for (CORBA::ULong alfa = 0; alfa < elements_graph.length(); alfa++)
      {
          elements.push_back(CdmwPlatformMngt::GraphElement::_downcast((elements_graph[alfa])->_copy_value()));
      }

      bool no_stop_flag = true;

      while (no_stop_flag)
      {
          std::list<CdmwPlatformMngt::GraphElement_var> successors;

          for (std::list<CdmwPlatformMngt::GraphElement_var>::iterator iterator = elements.begin(); 
            iterator != elements.end(); iterator++)
          {
              CdmwPlatformMngt::GraphElement_var element = *iterator;

              std::string element_name = element->name();

              std::set<std::string>::iterator names_iterator;

              names_iterator = find (names_set.begin(), names_set.end(), element_name);

              if (names_iterator == names_set.end())
              {
                  names_set.insert(element_name);

                  std::string precondition_info[2];

                  getTaskInfo(element.in(), precondition_info);

                  if ("NONE" != precondition_info[1])
                  {
                      precondition_map.insert( 
                         std::pair<std::string, std::string>(
                         precondition_info[0], precondition_info[1]));
                  }

              }

              CdmwPlatformMngt::GraphElements other_successors = element->successors();

              for (CORBA::ULong succ = 0; succ < other_successors.length(); succ++)
              {
                  CdmwPlatformMngt::GraphElement_var successor_element = other_successors[succ];

                  successors.push_back(CdmwPlatformMngt::GraphElement::_downcast(successor_element->_copy_value()));

              }
          }

         if (successors.empty())
         {
            no_stop_flag = false;
         }
         else
         {
            elements.clear();

            for (std::list<CdmwPlatformMngt::GraphElement_var>::iterator iterator = successors.begin(); 
               iterator != successors.end(); iterator++)
            {
               elements.push_back(CdmwPlatformMngt::GraphElement::_downcast((*iterator)->_copy_value()));
            }
         }

      }

      }
      catch (...)
      {
          CDMW_WARN(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), 
            "HostTrees::composePrecondMapFromGraph: Unknown Exception Caught");  
      }

      return precondition_map;
   }

   void HostTrees::getTaskInfo(CdmwPlatformMngt::GraphElement* graphElement, std::string tsk_info [2])
   {
       tsk_info [0] = "";
       tsk_info [1] = "";

       tsk_info [0] = graphElement->name();

       std::string s= graphElement->precond_managed_element(); 

       //If precond_managed_element is empty means that the Precondition is not used
       if(!s.empty())
       {
           tsk_info[1] =  graphElement->precond_managed_element();
       }
       else 
       {
           tsk_info[1] = "NONE";
       }

   }

   } // End namespace PlatformMngt
} // End namespace Cdmw
