/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "platformsystemlibrary/SystemConsistencyStatusSolver.hpp"
#include "platformsystemlibrary/System_impl.hpp"
#include "platformsystemlibrary/SystemLogMessageId.hpp"

#include "SystemMngt/platformlibrary/LogMngr.hpp"

namespace Cdmw
{
namespace PlatformMngt
{

using namespace std;

  

/////////////////////////////////////////////////////
//   System Consistency Status Solver implementation
/////////////////////////////////////////////////////



SystemConsistencyStatusSolver::SystemConsistencyStatusSolver()
{
}

SystemConsistencyStatusSolver::~SystemConsistencyStatusSolver()
{
}

std::string SystemConsistencyStatusSolver::get_consistent_status(SystemStateMachine* state_machine, const string& status)
{ 
      
    Cdmw::PlatformMngt::System_impl* system =  state_machine->getContext();
    std::string state = status;

    // log message
    std::ostringstream log_message;

    log_message << " System status = "<< state ;

    // get all applications, all running application, all stopped application to check System consistent status
    CORBA::ULong running_applications = system->get_number_of_running_applications();
    CORBA::ULong all_applications = system->get_number_of_application_with_relevant_indicator();

    CORBA::ULong stopped_applications = system->get_number_of_applications_with_status(CdmwPlatformMngt::APPLICATION_STOPPED);

    CORBA::ULong  partial_running_apps = system->get_number_of_applications_with_status(CdmwPlatformMngt::APPLICATION_PARTIAL_RUNNING);

    // if all applications are STOPPED system is STOPPED
    if (stopped_applications == all_applications)
    {
        state =  System_impl::M_stoppedStateName;
    }

    // if all applications are RUNNING system is STARTED
    if (running_applications == all_applications)
    {
        state = System_impl::M_startedStateName;
    }

    // if any application is PARTIAL_RUNNING or there are some application RUNNING
    // System is PARTIAL_STARTED
    if ((partial_running_apps > 0)||((running_applications < all_applications)&&(running_applications > 0)))
    {
        state = System_impl::M_partialStartedStateName;
    }

    
    log_message << " SystemStartedStatusSolver: running_applications:" << running_applications << " stopped_applications:"                   << stopped_applications << " all applications:"<< all_applications << " number of PARTIAL_RUNNING applications:"<< partial_running_apps <<" => consistent status = "<< state <<ends;

    LogMngr::logMessage(INF, MSG_ID_CONSISTENCY_OF_STATUS, log_message.str().c_str());
    return state;
}

} // End namespace PlatformMngt
} // End namespace Cdmw


