/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <iostream>
#include <limits>

#include "testelemnavigability/TestElemNav.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"

#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"

using namespace std;

using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestEntityNav);


namespace Cdmw
{
namespace PlatformMngt
{

TestEntityNav::TestEntityNav() 
{

}


TestEntityNav::~TestEntityNav()
{

}


const char* TestEntityNav::getElementName(int counter) throw ()
{
    switch (counter)
    {
    case 1: return "SYS_ENT1";
    case 2: return "FTGR_ENT1";
    case 3: return "LBGR_ENT1";
    case 4: return "PROC_ENT1";
    case 5: return "APPL_ENT1";
    case 6: return "HOST_ENT1";
    case 7: return "APPL1";
    case 8: return (OsSupport::OS::get_hostname()).c_str();
    case 9: return "FTGR1";
    case 10: return "LBGR1";
    case 11: return "PROC1";
    case 12: return "COMP1";
    case 13: return "COMP_ENT1";
    case 14: return "S1";
    default:
        // Unknown test modus!  Add to list abough!!
        CDMW_NEVER_HERE();
    }
}



void
TestEntityNav::do_tests()                
{
    try
    {
      /*get the orb */
        m_orb =
          CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::
                                  instance ()->get_ORB ());

        CORBA::Object_var objref;

        /*define the corbaloc*/
        const char *corbaloc_str = 
             "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";

        objref = m_orb->string_to_object(corbaloc_str);

        /* resolve the System */
        CdmwPlatformMngt::System_var system = 
             CdmwPlatformMngt::System::_narrow(objref.in());

        if (CORBA::is_nil(system.in()))
        {
            TEST_INFO("Error in narrowing CdmwPlatformMngt::System_var");
            exit(-1);
        }

        /***********************************************************
         * Check get_element() passing null element_path
         * *********************************************************/
        string null_elem_path = "";
        CdmwPlatformMngt::ManagedElement_var managed_element; 
        try
        {
            TEST_INFO("Checking null element_path ");
            managed_element = system->get_element (null_elem_path.c_str());               
            CPPUNIT_ASSERT(false) ;

        }
        catch (const CORBA::SystemException & ex)
        {
            TEST_INFO("OK!");
            CPPUNIT_ASSERT (true);
        }
        

        /*****************************************
         * inizialise the list of element paths. 
         ******************************************/       
        ElementList managed_elements;
        std::string host_name = OsSupport::OS::get_hostname();
        
        //inserting the correct element paths.
        managed_elements.push_back("S1.sys/SYS_ENT1.ent");
        managed_elements.push_back("S1.sys/APPL1.appl/FTGR1.ftgroup/FTGR_ENT1.ent");
        managed_elements.push_back("S1.sys/APPL1.appl/LBGR1.lbgroup/LBGR_ENT1.ent");
        managed_elements.push_back("S1.sys/APPL1.appl/"+ host_name + ".host/PROC1.proc/PROC_ENT1.ent");
        managed_elements.push_back("S1.sys/APPL1.appl/APPL_ENT1.ent");
        managed_elements.push_back("S1.sys/"+ host_name + ".host/HOST_ENT1.ent");
        managed_elements.push_back("S1.sys/APPL1.appl");
        managed_elements.push_back("S1.sys/"+ host_name + ".host");
        managed_elements.push_back("S1.sys/APPL1.appl/FTGR1.ftgroup");
        managed_elements.push_back("S1.sys/APPL1.appl/LBGR1.lbgroup");
        managed_elements.push_back("S1.sys/APPL1.appl/"+ host_name + ".host/PROC1.proc");
        managed_elements.push_back("S1.sys/APPL1.appl/"+ host_name + ".host/PROC1.proc/COMP1.comp");
        managed_elements.push_back("S1.sys/APPL1.appl/"+ host_name + ".host/PROC1.proc/COMP1.comp/COMP_ENT1.ent");
        managed_elements.push_back("S1.sys");
        //inserting not existing element paths in order to test the exeptions throwing.
        managed_elements.push_back("S1.sys/HOSTNotFound.host");
        managed_elements.push_back("S1.sys/HOSTNotFound.host/ENT1.ent");
        managed_elements.push_back("S1.sys/APPLNotFound.appl/LBGR1.lbgroup");
        managed_elements.push_back("S1.sys/APPLNotFound.appl");
        managed_elements.push_back("S1.sys/APPLNotFound.appl/FTGR1.ftgroup");
        managed_elements.push_back("S1.sys/APPLNotFound.appl/ENT1.ent");
        managed_elements.push_back("S1.sys/APPLNotFound.appl/"+ host_name + ".host/PROC1.proc");
        managed_elements.push_back("S1.sys/APPLNotFound.appl/"+ host_name + ".host/PROC1.proc/COMP1.comp");        
        managed_elements.push_back("S1.sys/APPL1.appl/FTGRNotFound.ftgroup");
        managed_elements.push_back("S1.sys/APPL1.appl/FTGRNotFound.ftgroup/ENT1.ent");
        managed_elements.push_back("S1.sys/APPL1.appl/LBGRNotFound.lbgroup");
        managed_elements.push_back("S1.sys/APPL1.appl/LBGRNotFound.lbgroup/ENT1.ent");
        managed_elements.push_back("S1.sys/APPL1.appl/"+ host_name + ".host/PROCNotFound.proc");
        managed_elements.push_back("S1.sys/APPL1.appl/"+ host_name + ".host/PROCNotFound.proc/COMP1.comp");
        managed_elements.push_back("S1.sys/APPL1.appl/"+ host_name + ".host/PROCNotFound.proc/ENT1.ent");
        managed_elements.push_back("S1.sys/APPL1.appl/"+ host_name + ".host/PROC1.proc/COMPNotFound.comp");
        managed_elements.push_back("S1.sys/APPL1.appl/"+ host_name + ".host/PROC1.proc/COMPNotFound.comp/ComponentAttribute.attribute/ENT1.ent");
        managed_elements.push_back("S1.sys/"+ host_name + ".host/ENTNotFound.ent");
        managed_elements.push_back("S1.sys/ENTNotFound.ent");
        managed_elements.push_back("S1.sys/APPL1.appl/FTGR1.ftgroup/ENTNotFound.ent");
        managed_elements.push_back("S1.sys/APPL1.appl/LBGR1.lbgroup/ENTNotFound.ent");
        managed_elements.push_back("S1.sys/APPL1.appl/"+ host_name + ".host/PROC1.proc/ENTNotFound.ent");
        managed_elements.push_back("S1.sys/APPL1.appl/ENTNotFound.ent");  
        //inserting syntaxly wrong element path (.entity suffix instead of .ent)
        managed_elements.push_back("S1.sys/APPL1.appl/LBGR1.lbgroup/LBGR_ENT1.entity");
        
        
        /***********************************************************
         * Check get_element() passing correct and wrong element_path
         * *********************************************************/ 
        std::string element_name;
        int num_correct_path = 14;
        int num_total_path=38;
        ElementList::iterator it = managed_elements.begin();
        
        //Checking the right element_path
        for (int counter=1; counter <= num_correct_path; counter++)
        {               
            try
            {
                managed_element = system->get_element ((*it).c_str());               
                element_name = managed_element->name();
                TEST_INFO("Checking the right element_path :"<<(*it).c_str());
                CPPUNIT_ASSERT( element_name.compare(getElementName(counter)) ==0 ) ;

            }
            catch (const CORBA::SystemException & ex)
            {
                CPPUNIT_ASSERT (false);
            }
            it++;
        }
        
        //Checking the wrong element_path
        for (int counter = num_correct_path+1 ; counter <= num_total_path ; counter++ )
        {               
            try
            {
                TEST_INFO("Checking the wrong element_path : "<<(*it).c_str());
                CdmwPlatformMngt::ManagedElement_var managed_element = system->get_element ((*it).c_str());             
                CPPUNIT_ASSERT(false);
            }
            catch (const CORBA::SystemException & ex) {
                TEST_INFO("\t\t\t\t\t Exception raised :: "<<ex<<std::endl);
                CPPUNIT_ASSERT(true);
            }
            catch (...)
            {
                TEST_INFO("Unexpected exception raised :: ");
                CPPUNIT_ASSERT(false);
            }
            it++;
        }
    }
    catch (const Cdmw::Exception & ex)
    {
        std::cerr << ex.what() << std::endl;
        exit(-1);
    }
    catch(...)
    {
        std::cerr << "Unexpected exception" << std::endl;
        exit(-1);
    }    

    /*
    ========================================================
    program stopping
    ========================================================
    */

    Cdmw::OrbSupport::OrbSupport::ORB_cleanup(m_orb.in());

    if (!CORBA::is_nil(m_orb.in()))
    {
        try
        {     
            m_orb->destroy();
        }
        catch(const CORBA::Exception& ex)
        {
            std::cerr << ex << std::endl;
            exit(-1);
        }
    }
}    



} /* End namespace PlatformMngt*/
} /* End namespace Cdmw*/


