/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/* @author Fabrizio Lorenna <card03@selex-si.com> */

#include <iostream>
#include <limits>

#include "testfacade_Host/TestFacade_Host.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"

#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"
#include "idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"

#include <platformftproxy/InitUtils.hpp>
#include <platformftproxy/SystemProxy.hpp>
#include <SystemMngt/platformlibrary/Iterator.hpp>

#include <FaultTolerance/ftreplicationmanager/InitUtils.hpp>
#include <FaultTolerance/idllib/FT.stub.hpp>
#include <platformftaccessorlibrary/SystemAccessor_impl.hpp>

using namespace std;

using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION( TestFacade_Host );

namespace Cdmw
{
namespace PlatformMngt
{



TestFacade_Host::TestFacade_Host() 
{

}


TestFacade_Host::~TestFacade_Host()
{

}


void TestFacade_Host::setUp()
    {
      TestFacade_Host::one_shot_init();
    }

   bool    TestFacade_Host::m_is_one_shot_initialized=false;
  
   CdmwPlatformMngt::System_var TestFacade_Host::mysystem;
   CdmwPlatformMngt::Host_var TestFacade_Host::myhost;

void TestFacade_Host::one_shot_init()        
{

      if(!m_is_one_shot_initialized)
      {
        m_is_one_shot_initialized = true; 
     
      try     
      {
        CORBA::ORB_var orb;
        orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager:: instance ()->get_ORB ());

        PortableServer::POA_var poa = PortableServer::POA::_duplicate (Cdmw::TestUtils::CORBATestManager:: instance ()->get_POA ());

        PortableServer::POAManager_var poaMgr = PortableServer::POAManager::_duplicate(poa->the_POAManager());
        poaMgr->activate();

        CORBA::Object_var objref;

        //define the corbaloc
        const char *corbaloc_str = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";

        objref = orb->string_to_object(corbaloc_str);

        //resolve the System_ptr
        mysystem = CdmwPlatformMngt::System::_narrow(objref.in() );
        


        if (CORBA::is_nil (mysystem.in ()))
          {
           // TEST_INFO ("Error in narrowing CdmwPlatformMngt::System");
            exit (-1);
          }

       /*retrieve the current hostname*/
       char * hostname  = getenv("HOSTNAME");

       myhost = mysystem->get_host( hostname);
       }
      catch (const Cdmw::Exception & ex)
      {
        std::cerr << ex.what () << std::endl;
        exit (-1);
      }
      catch (const CORBA::Exception & ex)
      {
        std::cerr << ex << std::endl;
        exit (-1);
      }
      catch (...)
      {
        std::cerr << "unexpected exception" << std::endl;
        exit (-1);
      }
   }
}



        /* try to call the facade for "mode" , to get */

    void TestFacade_Host::get_attribute_status()
    {
    try
    {
        //TEST_INFO("Test in get_attribute of 'status' :");
    CORBA::Any_var outany; 
        myhost->get_attribute(CdmwPlatformMngt::TECH_STATUS, outany, CdmwPlatformMngt::PRE_DEFINED);
        /*extract outany value;*/
        const char* stat = "Unreachable";
        (*outany) >>= CORBA::Any::to_string(stat,0);

        std::string status(stat);
    if ( status == "Responding")
      { CPPUNIT_ASSERT(1); /*test ok*/
      }
        else
      { CPPUNIT_ASSERT(0); /*test not ok*/
      }

    }
    catch(...)
      {
          cout <<"Error, exception in get_attribute with 'status' "<<endl;
      CPPUNIT_ASSERT(0);
      }
    }


    void TestFacade_Host::get_attribute_exception()
    {
    CORBA::Any_var outany;

        //TEST_INFO("Test in get_attribute to throw AttributeNotFound :");
        myhost->get_attribute("dummy", outany, CdmwPlatformMngt::PRE_DEFINED);


    }


    void TestFacade_Host::set_attribute_uda()
    {
    
     CORBA::Any inany;
     CORBA::Short proshort1 = 88 ;
     inany <<= proshort1;
 
     try
     {
     myhost->set_attribute("CIAO", inany , CdmwPlatformMngt::USER_DEFINED);
     CPPUNIT_ASSERT(0);
     
     }
     
     catch (const CdmwPlatformMngt::AttributeNotFound &)
     {
     CPPUNIT_ASSERT(1);
     }
       
          
    try
    {
   
        //TEST_INFO("Test in set_attribute of user defined attribute :");

   
        myhost->define_property ("CIAO", inany );

    CORBA::Any_var outany;
    myhost->get_attribute("CIAO", outany.out(), CdmwPlatformMngt::USER_DEFINED);
    CORBA::Short proshort2 = 0 ;

        (*outany) >>= proshort2;

    if (proshort2 == 88 )
      {
      CPPUNIT_ASSERT(1);
      }
        else
      {
      CPPUNIT_ASSERT(0);
      }
    }
    catch(...)
      {
      CPPUNIT_ASSERT(0); /*test not ok*/
      }
}

    void TestFacade_Host::get_attribute_uda()
    {
    try
    {
    CORBA::Any_var outany;
        //TEST_INFO("Test in get_attribute of user defined attribute :");

    myhost->get_attribute("CIAO", outany, CdmwPlatformMngt::USER_DEFINED);
    CORBA::Short proshort = 0 ;

        (*outany) >>= proshort;
    if (proshort == 88 )
      {
      CPPUNIT_ASSERT(1);
      }
        else
      {
      CPPUNIT_ASSERT(0);
      }
    }
    catch(...)
      {
      CPPUNIT_ASSERT(0); /*test not ok*/
      }
        } 

} /* End namespace PlatformMngt*/
} /* End namespace Cdmw*/


