/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/* @author Fabrizio Lorenna <card03@selex-si.com> */

#include <iostream>
#include <limits>

#include "testfacade_System/TestFacade_System.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"

#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"

#include <platformftproxy/InitUtils.hpp>
#include <platformftproxy/SystemProxy.hpp>
#include <SystemMngt/platformlibrary/Iterator.hpp>

#include <FaultTolerance/ftreplicationmanager/InitUtils.hpp>
#include <FaultTolerance/idllib/FT.stub.hpp>
#include <platformftaccessorlibrary/SystemAccessor_impl.hpp>
#include "idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"

using namespace std;

using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION( TestFacade_System );

namespace Cdmw
{
namespace PlatformMngt
{



TestFacade_System::TestFacade_System() throw() 
{

}


TestFacade_System::~TestFacade_System()
    throw()
{

}

void TestFacade_System::setUp()
    {
      TestFacade_System::one_shot_init();
    }

   bool    TestFacade_System::m_is_one_shot_initialized=false;
  
   CdmwPlatformMngt::System_var TestFacade_System::mysystem;

void TestFacade_System::one_shot_init()                
{

      if(!m_is_one_shot_initialized)
      {
        m_is_one_shot_initialized = true; 
     
      try     
      {
        CORBA::ORB_var orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager:: instance ()->get_ORB ());

        PortableServer::POA_var poa = PortableServer::POA::_duplicate (Cdmw::TestUtils::CORBATestManager:: instance ()->get_POA ());
        
        PortableServer::POAManager_var poaMgr = PortableServer::POAManager::_duplicate(poa->the_POAManager());
        poaMgr->activate();


        CORBA::Object_var objref;

        //define the corbaloc
        const char *corbaloc_str = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";

        objref = orb->string_to_object(corbaloc_str);

        //resolve the System_ptr
        mysystem = CdmwPlatformMngt::System::_narrow(objref.in() );

        if (CORBA::is_nil (mysystem.in ()))
          {
           // TEST_INFO ("Error in narrowing CdmwPlatformMngt::System");
            exit (-1);
          }

       }
      catch (const Cdmw::Exception & ex)
      {
        std::cerr << ex.what () << std::endl;
        exit (-1);
      }
      catch (const CORBA::Exception & ex)
      {
        std::cerr << ex << std::endl;
        exit (-1);
      }
      catch (...)
      {
        std::cerr << "unexpected exception" << std::endl;
        exit (-1);
      }
    }
}

    
        /* try to call the facade for "mode" , to get */
    void TestFacade_System::get_attribute_mode()
        {
    try
    {
        //TEST_INFO("Test in get_attribute of 'mode' :");
    CORBA::Any_var outany; 
        mysystem->get_attribute(CdmwPlatformMngt::MODE, outany, CdmwPlatformMngt::PRE_DEFINED);
        /*extract outany value;*/
        CdmwPlatformMngt::FunctioningMode func_mode=CdmwPlatformMngt::DEGRADED_MODE;
        (*outany) >>= func_mode;

        /* cout <<"func_mode="<<func_mode<<endl; */
    if ( func_mode == CdmwPlatformMngt::NORMAL_MODE)
      { CPPUNIT_ASSERT(1); /*test ok*/
      }
        else
      { CPPUNIT_ASSERT(0); /*test not ok*/
      }

    }
    catch(...)
      {
          cout <<"Error, exception in get_attribute with 'mode' "<<endl;
      CPPUNIT_ASSERT(0);
      }
       }


        /* try to call the facade for "mode" , to get */
    void TestFacade_System::get_attribute_mode_info()
       {
    CORBA::Any_var outany; 
        
        try
    {
        //TEST_INFO("Test in get_attribute of 'mode_info': ");
            mysystem->get_attribute(CdmwPlatformMngt::MODE_INFO, outany, CdmwPlatformMngt::PRE_DEFINED);
        }
        catch(...)
        {
            CPPUNIT_ASSERT(false); 
        }
        /*extract outany value;*/
          char* attr_value;
          
          (*outany) >>= CORBA::Any::to_string(attr_value,0);
          
          std::string s(attr_value);
          if(s == "")
          {
            CPPUNIT_ASSERT(1);
          }
          else
          {
            CPPUNIT_ASSERT(0);
          }
          
        }

    void TestFacade_System::get_attribute_status()
    {
    /*test get_attribute of status*/
    try
    {
    CORBA::Any_var outany;

        //TEST_INFO("Test in get_attribute of 'status' :");
        const char* system_status;

        mysystem->get_attribute(CdmwPlatformMngt::TECH_STATUS, outany, CdmwPlatformMngt::PRE_DEFINED);

    system_status = "StartupSuspend";
        (*outany) >>= CORBA::Any::to_string(system_status,0);

    std::string status(system_status);
    if ( status == "Initial" )
    { CPPUNIT_ASSERT(1); /*ok*/
    }
    else
    { CPPUNIT_ASSERT(0); /*not ok*/
    }

        /*cout <<"mode_info="<<mode_info<<endl; */
    }
    catch(...)
      {
          cout <<"Error, exception in get_attribute  with 'status' "<<endl;
      CPPUNIT_ASSERT(0);
      }
       }



    /*test get_attribute of status_info */
    void TestFacade_System::get_attribute_status_info()
       {
    try
    {
    CORBA::Any_var outany;

        //TEST_INFO("Test in get_attribute of 'status_info' :");

        mysystem->get_attribute(CdmwPlatformMngt::TECH_STATUS_INFO, outany, CdmwPlatformMngt::PRE_DEFINED);

        char *status_info ;
        (*outany) >>= CORBA::Any::to_string(status_info,0);
    string cmp_str(status_info);
    /*only check that thare are not exceptions*/
    CPPUNIT_ASSERT(1); 

    }
    catch(...)
      {
          cout <<"Error, exception in get_attribute  with 'status_info' "<<endl;
      CPPUNIT_ASSERT(0);
      }
       }

    void TestFacade_System::get_attribute_exception()
    {
    CORBA::Any_var outany;

        //TEST_INFO("Test in get_attribute to throw AttributeNotFound :");

        mysystem->get_attribute("dummy", outany, CdmwPlatformMngt::PRE_DEFINED);

        }


    void TestFacade_System::set_attribute_uda()
       {
     CORBA::Any inany;
     CORBA::Short proshort1 = 77 ;
    
     try
     {
     mysystem->set_attribute("CIAO", inany , CdmwPlatformMngt::USER_DEFINED);
     CPPUNIT_ASSERT(0);
     
     }
     
     catch (const CdmwPlatformMngt::AttributeNotFound &)
     {
     CPPUNIT_ASSERT(1);
     }
    
       
    try
    {


        //TEST_INFO("Test in set_attribute of user defined attribute :");

        inany <<= proshort1;

        mysystem->define_property("CIAO", inany );

    CORBA::Any_var outany;
    mysystem->get_attribute("CIAO", outany, CdmwPlatformMngt::USER_DEFINED);
    CORBA::Short proshort2 = 0 ;

        (*outany) >>= proshort2;
    /*cout <<"proshort2="<<proshort2<<endl;*/
    if (proshort2 == 77 )
      {
      CPPUNIT_ASSERT(1);
      }
        else
      {
      CPPUNIT_ASSERT(0);
      }


    }
    catch(...)
      {
      CPPUNIT_ASSERT(0); /*test not ok*/
      }

        }
        

void TestFacade_System::set_readonly_attribute_uda()
{        
    CORBA::Any any_S;
    CORBA::Short L = 3;
    any_S <<= L;
    try 
    {
        TEST_INFO ("Defining a User defined Attribute with READ_ONLY Mode");
        mysystem -> define_property_with_mode ("short_property_readonly", any_S,
                           CosPropertyService::read_only );
        
        TEST_INFO ("Trying to redefine the user defined attribute with READ_ONLY Mode");
        mysystem -> set_attribute ("short_property_readonly", any_S, CdmwPlatformMngt::USER_DEFINED);
        
        CPPUNIT_ASSERT(0);
    }
    catch (const CdmwPlatformMngt::ReadOnlyAttribute& read_only_ex)
    {
        TEST_INFO("TEST OK THROWN THE EXCEPTION: "<< read_only_ex);
        CPPUNIT_ASSERT(1);
    }
}      
      
      

    void TestFacade_System::get_attribute_uda()
       {
    try
    {
    CORBA::Any_var outany;
        //TEST_INFO("Test in get_attribute of user defined attribute :");

    mysystem->get_attribute("CIAO", outany, CdmwPlatformMngt::USER_DEFINED);
    CORBA::Short proshort = 0 ;

        (*outany) >>= proshort;
    if (proshort == 77 )
      {
      CPPUNIT_ASSERT(1);
      }
        else
      {
      CPPUNIT_ASSERT(0);
      }
    }
    catch(...)
      {
      CPPUNIT_ASSERT(0); /*test not ok*/
      }

     }


} /* End namespace PlatformMngt*/
} /* End namespace Cdmw*/


