/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "testhoststatemachine/TestHostStateMachine.hpp"
#include "platformsystemlibrary/HostStates.hpp"
#include <string>
#include <iostream>

CPPUNIT_TEST_SUITE_REGISTRATION (TestHostStateMachine);

TestHostStateMachine::TestHostStateMachine () throw ()
{

}

TestHostStateMachine::~TestHostStateMachine () throw ()
{

}

void TestHostStateMachine::test_state_transitions () throw ()
{
    try {
        Cdmw::PlatformMngt::Host_impl* host = new Cdmw::PlatformMngt::Host_impl(); //Fake Host_impl
        Cdmw::PlatformMngt::HostStateMachine hsm(host);
        
        // Initial state is HOST_RESPONDING
        std::cerr << "\nInitial state should be HOST_RESPONDING" << std::endl;
        std::string state_name;
        state_name = hsm.getStateName();
        std::cerr << "State name: " << state_name << std::endl;
        CPPUNIT_ASSERT(state_name == HOST_RESPONDING_NAME);
        
        // State transitions for HOST_UNREACHABLE
        std::cerr << "State transitions for HOST_UNREACHABLE" << std::endl;
        hsm.getState()->notify_daemon_status(&hsm, Cdmw::PlatformMngt::HostState::DAEMON_RUNNING);
        state_name = hsm.getStateName();
        std::cerr << "State name: " << state_name << std::endl;
        CPPUNIT_ASSERT(state_name == HOST_RESPONDING_NAME);
        
        hsm.setState(HOST_UNREACHABLE_NAME);
        hsm.getState()->notify_daemon_status(&hsm, Cdmw::PlatformMngt::HostState::DAEMON_NOT_RUNNING);
        state_name = hsm.getStateName();
        std::cerr << "State name: " << state_name << std::endl;
        CPPUNIT_ASSERT(state_name == HOST_UNREACHABLE_NAME);
        
        // State transitions for HOST_RESPONDING
        std::cerr << "State transitions for HOST_RESPONDING" << std::endl;
        hsm.setState(HOST_RESPONDING_NAME);
        hsm.getState()->notify_daemon_status(&hsm, Cdmw::PlatformMngt::HostState::DAEMON_NOT_RUNNING);
        state_name = hsm.getStateName();
        std::cerr << "State name: " << state_name << std::endl;
        CPPUNIT_ASSERT(state_name == HOST_UNREACHABLE_NAME);
        
        hsm.setState(HOST_RESPONDING_NAME);
        hsm.getState()->notify_daemon_status(&hsm, Cdmw::PlatformMngt::HostState::DAEMON_RUNNING);
        state_name = hsm.getStateName();
        std::cerr << "State name: " << state_name << std::endl;
        CPPUNIT_ASSERT(state_name == HOST_RESPONDING_NAME);
                
        hsm.setState(HOST_RESPONDING_NAME);
        hsm.getState()->stop(&hsm);
        state_name = hsm.getStateName();
        std::cerr << "State name: " << state_name << std::endl;
        CPPUNIT_ASSERT(state_name == HOST_RESPONDING_NAME);
                        
        hsm.setState(HOST_RESPONDING_NAME);
        hsm.getState()->start(&hsm);
        state_name = hsm.getStateName();
        std::cerr << "State name: " << state_name << std::endl;
        CPPUNIT_ASSERT(state_name == HOST_STARTING_NAME);
        
        hsm.setState(HOST_RESPONDING_NAME);
        hsm.getState()->notify_processes_status(&hsm, Cdmw::PlatformMngt::HostState::ALL_PROCESSES_RUNNING);
        state_name = hsm.getStateName();
        std::cerr << "State name: " << state_name << std::endl;
        CPPUNIT_ASSERT(state_name == HOST_STARTED_NAME);
        
        hsm.setState(HOST_RESPONDING_NAME);
        hsm.getState()->notify_processes_status(&hsm, Cdmw::PlatformMngt::HostState::NOT_ALL_PROCESSES_RUNNING);
        state_name = hsm.getStateName();
        std::cerr << "State name: " << state_name << std::endl;
        CPPUNIT_ASSERT(state_name == HOST_PARTIAL_STARTED_NAME);

        // State transitions for HOST_STARTING
        std::cerr << "State transitions for HOST_STARTING" << std::endl;
        hsm.setState(HOST_STARTING_NAME);
        hsm.getState()->notify_processes_status(&hsm, Cdmw::PlatformMngt::HostState::ALL_PROCESSES_RUNNING);
        state_name = hsm.getStateName();
        std::cerr << "State name: " << state_name << std::endl;
        CPPUNIT_ASSERT(state_name == HOST_STARTED_NAME);
        
        hsm.setState(HOST_STARTING_NAME);
        hsm.getState()->notify_processes_status(&hsm, Cdmw::PlatformMngt::HostState::NOT_ALL_PROCESSES_RUNNING);
        state_name = hsm.getStateName();
        std::cerr << "State name: " << state_name << std::endl;
        CPPUNIT_ASSERT(state_name == HOST_PARTIAL_STARTED_NAME);
        
        hsm.setState(HOST_STARTING_NAME);
        hsm.getState()->notify_processes_status(&hsm, Cdmw::PlatformMngt::HostState::NO_PROCESS_RUNNING);
        state_name = hsm.getStateName();
        std::cerr << "State name: " << state_name << std::endl;
        CPPUNIT_ASSERT(state_name == HOST_STARTING_NAME); 
        
        hsm.setState(HOST_STARTING_NAME);
        hsm.getState()->stop(&hsm);
        state_name = hsm.getStateName();
        std::cerr << "State name: " << state_name << std::endl;
        CPPUNIT_ASSERT(state_name == HOST_STOPPING_NAME);
        
        // State transitions for HOST_PARTIAL_STARTED
        std::cerr << "State transitions for HOST_PARTIAL_STARTED" << std::endl;
        hsm.setState(HOST_PARTIAL_STARTED_NAME);
        hsm.getState()->notify_processes_status(&hsm, Cdmw::PlatformMngt::HostState::ALL_PROCESSES_RUNNING);
        state_name = hsm.getStateName();
        std::cerr << "State name: " << state_name << std::endl;
        CPPUNIT_ASSERT(state_name == HOST_STARTED_NAME);
        
        hsm.setState(HOST_PARTIAL_STARTED_NAME);
        hsm.getState()->notify_processes_status(&hsm, Cdmw::PlatformMngt::HostState::NOT_ALL_PROCESSES_RUNNING);
        state_name = hsm.getStateName();
        std::cerr << "State name: " << state_name << std::endl;
        CPPUNIT_ASSERT(state_name == HOST_PARTIAL_STARTED_NAME);
        
        hsm.setState(HOST_PARTIAL_STARTED_NAME);
        hsm.getState()->notify_processes_status(&hsm, Cdmw::PlatformMngt::HostState::NO_PROCESS_RUNNING);
        state_name = hsm.getStateName();
        std::cerr << "State name: " << state_name << std::endl;
        CPPUNIT_ASSERT(state_name == HOST_RESPONDING_NAME);
        
        hsm.setState(HOST_PARTIAL_STARTED_NAME);
        hsm.getState()->stop(&hsm);
        state_name = hsm.getStateName();
        std::cerr << "State name: " << state_name << std::endl;
        CPPUNIT_ASSERT(state_name == HOST_STOPPING_NAME);
                
        // State transitions for HOST_STARTED
        std::cerr << "State transitions for HOST_STARTED" << std::endl;
        hsm.setState(HOST_STARTED_NAME);
        hsm.getState()->notify_processes_status(&hsm, Cdmw::PlatformMngt::HostState::ALL_PROCESSES_RUNNING);
        state_name = hsm.getStateName();
        std::cerr << "State name: " << state_name << std::endl;
        CPPUNIT_ASSERT(state_name == HOST_STARTED_NAME);
        
        hsm.setState(HOST_STARTED_NAME);
        hsm.getState()->notify_processes_status(&hsm, Cdmw::PlatformMngt::HostState::NOT_ALL_PROCESSES_RUNNING);
        state_name = hsm.getStateName();
        std::cerr << "State name: " << state_name << std::endl;
        CPPUNIT_ASSERT(state_name == HOST_PARTIAL_STARTED_NAME);
        
        hsm.setState(HOST_STARTED_NAME);
        hsm.getState()->notify_processes_status(&hsm, Cdmw::PlatformMngt::HostState::NO_PROCESS_RUNNING);
        state_name = hsm.getStateName();
        std::cerr << "State name: " << state_name << std::endl;
        CPPUNIT_ASSERT(state_name == HOST_RESPONDING_NAME);
        
        hsm.setState(HOST_STARTED_NAME);
        hsm.getState()->stop(&hsm);
        state_name = hsm.getStateName();
        std::cerr << "State name: " << state_name << std::endl;
        CPPUNIT_ASSERT(state_name == HOST_STOPPING_NAME);
        
        // State transitions for HOST_STOPPING
        std::cerr << "State transitions for HOST_STOPPING" << std::endl;
        hsm.setState(HOST_STOPPING_NAME);
        hsm.getState()->stop(&hsm);
        state_name = hsm.getStateName();
        std::cerr << "State name: " << state_name << std::endl;
        CPPUNIT_ASSERT(state_name == HOST_STOPPING_NAME);
        
        hsm.setState(HOST_STOPPING_NAME);
        hsm.getState()->notify_processes_status(&hsm, Cdmw::PlatformMngt::HostState::ALL_PROCESSES_RUNNING);
        state_name = hsm.getStateName();
        std::cerr << "State name: " << state_name << std::endl;
        CPPUNIT_ASSERT(state_name == HOST_STOPPING_NAME);
        
        hsm.setState(HOST_STOPPING_NAME);
        hsm.getState()->notify_processes_status(&hsm, Cdmw::PlatformMngt::HostState::NOT_ALL_PROCESSES_RUNNING);
        state_name = hsm.getStateName();
        std::cerr << "State name: " << state_name << std::endl;
        CPPUNIT_ASSERT(state_name == HOST_STOPPING_NAME);
        
        hsm.setState(HOST_STOPPING_NAME);
        hsm.getState()->notify_processes_status(&hsm, Cdmw::PlatformMngt::HostState::NO_PROCESS_RUNNING);
        state_name = hsm.getStateName();
        std::cerr << "State name: " << state_name << std::endl;
        CPPUNIT_ASSERT(state_name == HOST_RESPONDING_NAME);
        
    
    }
    catch (...) {
        CPPUNIT_ASSERT(false);
    }
    
}

void TestHostStateMachine::test_state_actions () throw ()
{
    try {
        Cdmw::PlatformMngt::Host_impl* host = new Cdmw::PlatformMngt::Host_impl(); //Fake Host_impl
        Cdmw::PlatformMngt::HostStateMachine hsm(host);
        
        // starting the statemachine should result in a call to host->start_processes();
        hsm.setState(HOST_RESPONDING_NAME);
        hsm.getState()->start(&hsm);
        CPPUNIT_ASSERT(host->start_processes_called());
        
        // stopping the statemachine should result in a call to host->stop_processes();
        hsm.setState(HOST_STARTED_NAME);
        hsm.getState()->stop(&hsm);
        CPPUNIT_ASSERT(host->stop_processes_called());
                
        
        
    }
    catch (...) {
        CPPUNIT_ASSERT(false);
    }
}

void TestHostStateMachine::test_status_changes_notifications() throw ()
{
    try {
        Cdmw::PlatformMngt::Host_impl* host = new Cdmw::PlatformMngt::Host_impl(); //Fake Host_impl
        Cdmw::PlatformMngt::HostStateMachine hsm(host);

        hsm.setState(HOST_RESPONDING_NAME);
        CPPUNIT_ASSERT(host->notify_status_change_called());
        
        hsm.setState(HOST_STARTING_NAME);
        CPPUNIT_ASSERT(host->notify_status_change_called());

        hsm.setState(HOST_PARTIAL_STARTED_NAME);
        CPPUNIT_ASSERT(host->notify_status_change_called());
        
        hsm.setState(HOST_STARTED_NAME);
        CPPUNIT_ASSERT(host->notify_status_change_called());
        
        hsm.setState(HOST_STOPPING_NAME);
        CPPUNIT_ASSERT(host->notify_status_change_called());
        
        hsm.setState(HOST_UNREACHABLE_NAME);
        CPPUNIT_ASSERT(host->notify_status_change_called());        
    }
    catch (...) {
        CPPUNIT_ASSERT(false);
    }
}

void TestHostStateMachine::test_enum_return_values() throw ()
{
    try {
        Cdmw::PlatformMngt::Host_impl* host = new Cdmw::PlatformMngt::Host_impl(); //Fake Host_impl
        Cdmw::PlatformMngt::HostStateMachine hsm(host);

        hsm.setState(HOST_RESPONDING_NAME);
        CPPUNIT_ASSERT(CdmwPlatformMngt::HOST_RESPONDING == hsm.getState()->status());
        
        hsm.setState(HOST_STARTING_NAME);
        CPPUNIT_ASSERT(CdmwPlatformMngt::HOST_STARTING == hsm.getState()->status());

        hsm.setState(HOST_PARTIAL_STARTED_NAME);
        CPPUNIT_ASSERT(CdmwPlatformMngt::HOST_PARTIAL_STARTED == hsm.getState()->status());
        
        hsm.setState(HOST_STARTED_NAME);
        CPPUNIT_ASSERT(CdmwPlatformMngt::HOST_STARTED == hsm.getState()->status());
        
        hsm.setState(HOST_STOPPING_NAME);
        CPPUNIT_ASSERT(CdmwPlatformMngt::HOST_STOPPING == hsm.getState()->status());
        
        hsm.setState(HOST_UNREACHABLE_NAME);
        CPPUNIT_ASSERT(CdmwPlatformMngt::HOST_UNREACHABLE == hsm.getState()->status());
    }
    catch (...) {
        CPPUNIT_ASSERT(false);
    }
}


void TestHostStateMachine::test_error_conditions () throw ()
{
//    hsm.setState(HOST_UNREACHABLE);
//    hsm.getState()->notify_daemon_status(&hsm, Cdmw::PlatformMngt::HostState::DAEMON_NOT_RUNNING); // nothing happens
//    CPPUNIT_ASSERT(state_name == HOST_UNREACHABLE);

    CPPUNIT_ASSERT(true);
}

