/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/* @author Fabrizio Lorenna <card03@selex-si.com> */

#include <iostream>
#include <limits>

#include "teststdattributes_Host/TestStdAttributes_Host.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"

#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"
#include "idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"

#include <platformftproxy/InitUtils.hpp>
#include <platformftproxy/SystemProxy.hpp>
#include <SystemMngt/platformftproxy/AccessorLookupService.hpp>
#include <SystemMngt/platformftproxy/DefaultAccessorLookupService.hpp>
#include <SystemMngt/platformlibrary/Iterator.hpp>

#include <FaultTolerance/ftreplicationmanager/InitUtils.hpp>
#include <FaultTolerance/idllib/FT.stub.hpp>
#include <platformftaccessorlibrary/SystemAccessor_impl.hpp>

using namespace std;

using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestStdAttributes_Host);

namespace Cdmw
{
  namespace PlatformMngt
  {



    TestStdAttributes_Host::TestStdAttributes_Host () throw ()
    {

    }


    TestStdAttributes_Host::~TestStdAttributes_Host () throw ()
    {

    }

    void TestStdAttributes_Host::setUp()
    {
      TestStdAttributes_Host::one_shot_init();
    }


    bool TestStdAttributes_Host::m_is_one_shot_initialized=false;

    CdmwPlatformMngt::Host_var TestStdAttributes_Host::myhost;
    CdmwPlatformMngt::System_var TestStdAttributes_Host::mysystem;
    char * TestStdAttributes_Host::hostname;
    CdmwPlatformMngt::CompleteMonitoringInfo TestStdAttributes_Host::glob_monitor_info;

    void
    TestStdAttributes_Host::one_shot_init()
    {
      if(!m_is_one_shot_initialized)
      {
        m_is_one_shot_initialized = true;



      PortableServer::POA_var poa ;

      try
      {

        CORBA::ORB_var orb;
        orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager:: instance ()->get_ORB ());

        poa = PortableServer::POA::_duplicate
(Cdmw::TestUtils::CORBATestManager:: instance ()->get_POA ());
        
        PortableServer::POAManager_var poaMgr = PortableServer::POAManager::_duplicate(poa->the_POAManager());
        poaMgr->activate();


        CORBA::Object_var objref;

        //define the corbaloc
        const char *corbaloc_str = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";

        objref = orb->string_to_object(corbaloc_str);

        //resolve the System_ptr
        mysystem = CdmwPlatformMngt::System::_narrow(objref.in() );



    if (CORBA::is_nil (mysystem.in ()))
      {
        //TEST_INFO ("Error in narrowing CdmwPlatformMngt::System");
        exit (-1);
      }

    /* retrieve the current hostname */
    hostname = getenv ("HOSTNAME");

    myhost = mysystem->get_host (hostname);
        }
      catch (const Cdmw::Exception & ex)
      {
        std::cerr << ex.what () << std::endl;
        exit (-1);
      }
      catch (const CORBA::Exception & ex)
      {
        std::cerr << ex << std::endl;
        exit (-1);
      }
      catch (...)
      {
        std::cerr << "unexpected exception" << std::endl;
        exit (-1);
      }
     }
    }

    void TestStdAttributes_Host::get_name()
    {
    //TEST_INFO ("Test to get host name");
    CORBA::String_var name = myhost->name ();

    if (!strcmp (hostname, name.in ()))
      {
        CPPUNIT_ASSERT (1); /* test ok */
      }
    else
      {
        CPPUNIT_ASSERT (0); /* test not ok */
      }
         }



    void TestStdAttributes_Host::status()
    {
    //TEST_INFO ("Test on Host::status: ");
    CdmwPlatformMngt::HostStatus host_status = myhost->status ();

    if (host_status == CdmwPlatformMngt::HOST_RESPONDING || host_status == CdmwPlatformMngt::HOST_STARTED)
      {
        CPPUNIT_ASSERT (1);
      }
    else
      {
        CPPUNIT_ASSERT (0);
      }

        }

    void TestStdAttributes_Host::host_status_info()
    {
      //TEST_INFO ("Test on Host::host_status_info: ");
      try
       {
       CORBA::String_var status_info = myhost->host_status_info ();
       CPPUNIT_ASSERT(1);
       }
          catch(...)
       {
           CPPUNIT_ASSERT(0);
       }
        }

    void TestStdAttributes_Host::get_monitoring_parameters()
    {
      //TEST_INFO ("Test on Host::get_monitoring_parameters: ");
      CdmwPlatformMngt::CompleteMonitoringInfo monitor_info;

      monitor_info = myhost->get_monitoring_parameters ();

      if ( (monitor_info.monitoring_model == 
      glob_monitor_info.monitoring_model) &&
      (monitor_info.pull_monitoring_interval ==
      glob_monitor_info.pull_monitoring_interval) &&
      (monitor_info.pull_monitoring_timeout ==
      glob_monitor_info.pull_monitoring_timeout) &&
      (monitor_info.push_monitoring_interval ==
      glob_monitor_info.push_monitoring_interval) &&
      (monitor_info.push_monitoring_timeout ==
      glob_monitor_info.push_monitoring_timeout) )
        CPPUNIT_ASSERT(1);
          else
        CPPUNIT_ASSERT(0);
    }

    void TestStdAttributes_Host::set_monitoring_parameters()
    {
      glob_monitor_info.monitoring_model = CdmwPlatformMngt::FULL_MONITORING_MODEL;

      glob_monitor_info.pull_monitoring_interval = 15000 ;
      glob_monitor_info.pull_monitoring_timeout = 30000;
      glob_monitor_info.push_monitoring_interval = 2000;
      glob_monitor_info.push_monitoring_timeout = 4000;

      try
        {
        myhost->set_monitoring_parameters (glob_monitor_info);
        CPPUNIT_ASSERT(1);
        }
          catch(...)
        {
            CPPUNIT_ASSERT(0);
        }
     }


    void TestStdAttributes_Host::get_readonly_attributes()
    {
          CORBA::Boolean mybool = false;
          CORBA::Any_var any_attr = new CORBA::Any();


          try
            {
                myhost->get_attribute(CdmwPlatformMngt::GLOBAL_SUPERVISION,any_attr.out(),CdmwPlatformMngt::PRE_DEFINED);
                CPPUNIT_ASSERT (any_attr >>= CORBA::Any::to_boolean(mybool));
                CPPUNIT_ASSERT( mybool == true); //this host is the primary
            }
          catch(...)
            {
                CPPUNIT_ASSERT(false); 
            }

           try
           {
                any_attr <<= CORBA::Any::from_boolean(mybool);
                myhost->set_attribute(CdmwPlatformMngt::GLOBAL_SUPERVISION,any_attr.in(),CdmwPlatformMngt::PRE_DEFINED);
                 CPPUNIT_ASSERT(0);
           }
            catch(const CdmwPlatformMngt::ReadOnlyAttribute &e)
            {
                CPPUNIT_ASSERT(1);           
            }
          catch (...)
       {
                CPPUNIT_ASSERT(0);
       }

        // Get requested restart warmth
        CdmwPlatformMngt::RestartWarmth myrestart = CdmwPlatformMngt::COLD_RESTART;

        try {
            myhost->get_attribute(CdmwPlatformMngt::REQUESTED_RESTART_WARMTH, any_attr.out(), CdmwPlatformMngt::PRE_DEFINED);
            CPPUNIT_ASSERT((any_attr.in()) >>= myrestart);
            CPPUNIT_ASSERT ( myrestart == CdmwPlatformMngt::HOT_RESTART);
        }
        catch(...) {
            CPPUNIT_ASSERT(false);    
        }
      
        try
           {      
                myhost->set_attribute(CdmwPlatformMngt::REQUESTED_RESTART_WARMTH,any_attr.in(),CdmwPlatformMngt::PRE_DEFINED);
       
                CPPUNIT_ASSERT(0);
           }
       catch(const CdmwPlatformMngt::ReadOnlyAttribute &e)
       {
                CPPUNIT_ASSERT(1);           
       }
          catch (...)
       {
                CPPUNIT_ASSERT(0);
       }
           

    }

    void TestStdAttributes_Host::restart_warmth()
    {
        
        // Get requested restart warmth
        CdmwPlatformMngt::RestartWarmth myrestart = CdmwPlatformMngt::COLD_RESTART;
        CORBA::Any_var myany;// = new CORBA::Any();
        try {
            myhost->get_attribute(CdmwPlatformMngt::REQUESTED_RESTART_WARMTH, myany.out(), CdmwPlatformMngt::PRE_DEFINED);
            CPPUNIT_ASSERT((myany.in()) >>= myrestart);
            CPPUNIT_ASSERT ( myrestart == CdmwPlatformMngt::HOT_RESTART);
        }
        catch(...) {
            CPPUNIT_ASSERT(false);    
        }         
        
        // Set default restart warmth
        myrestart = CdmwPlatformMngt::COLD_RESTART;          
        *myany <<= myrestart;
        try {
            myhost->set_attribute (CdmwPlatformMngt::DEFAULT_RESTART_WARMTH, myany.in(), CdmwPlatformMngt::PRE_DEFINED);
        }
        catch(...) {
            CPPUNIT_ASSERT(false);    
        }         
        
        // Get default restart warmth
        try {
            myhost->get_attribute (CdmwPlatformMngt::DEFAULT_RESTART_WARMTH, myany.out(), CdmwPlatformMngt::PRE_DEFINED);
            CPPUNIT_ASSERT(myany.in() >>= myrestart);
            CPPUNIT_ASSERT ( myrestart == CdmwPlatformMngt::COLD_RESTART );
        }
        catch(...) {
            CPPUNIT_ASSERT(false);    
        }

        // Get requested restart warmth
        try {
            myhost->get_attribute (CdmwPlatformMngt::REQUESTED_RESTART_WARMTH, myany.out(), CdmwPlatformMngt::PRE_DEFINED);
            CPPUNIT_ASSERT(myany.in() >>= myrestart);
            CPPUNIT_ASSERT ( myrestart == CdmwPlatformMngt::HOT_RESTART );
        }
        catch(...) {
            CPPUNIT_ASSERT(false);    
        }
    }

    void TestStdAttributes_Host::autorestart_info()
    {
        CORBA::Any_var myany = new CORBA::Any();
        CdmwPlatformMngt::AutoRestartInfo * autorestart_info = new (CdmwPlatformMngt::AutoRestartInfo);
        autorestart_info->max_nbr_of_restart_attempt = 3;
        autorestart_info->auto_restart_control = CdmwPlatformMngt::USE_COUNTER;
        *myany <<= autorestart_info;
        try {
            myhost->set_attribute(CdmwPlatformMngt::AUTORESTART_INFO, myany.in(), CdmwPlatformMngt::PRE_DEFINED);
        }
        catch(...) {
            CPPUNIT_ASSERT(false);    
        }

        try {
        myhost->get_attribute (CdmwPlatformMngt::AUTORESTART_INFO, myany.out(), CdmwPlatformMngt::PRE_DEFINED);
       
        }
        catch(...) {
            CPPUNIT_ASSERT(false);
        }

        CPPUNIT_ASSERT (myany.in() >>= autorestart_info);
        CPPUNIT_ASSERT_EQUAL (CORBA::ULong(3), autorestart_info->max_nbr_of_restart_attempt) ;
        CPPUNIT_ASSERT_EQUAL (CdmwPlatformMngt::USE_COUNTER , autorestart_info->auto_restart_control) ;
        }


  }             /* End namespace PlatformMngt */
}               /* End namespace Cdmw */
