/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/* @author Fabrizio Lorenna <card03@selex-si.com> */

#include <iostream>
#include <limits>

#include "teststdattributes_System/TestStdAttributes_System.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"

#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"
#include "idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"

#include <platformftproxy/InitUtils.hpp>
#include <platformftproxy/SystemProxy.hpp>
#include <SystemMngt/platformftproxy/AccessorLookupService.hpp>
#include <SystemMngt/platformftproxy/DefaultAccessorLookupService.hpp>
#include <SystemMngt/platformlibrary/Iterator.hpp>

#include <FaultTolerance/ftreplicationmanager/InitUtils.hpp>
#include <FaultTolerance/idllib/FT.stub.hpp>
#include <platformftaccessorlibrary/SystemAccessor_impl.hpp>


using namespace std;

using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestStdAttributes_System);

namespace Cdmw
{
  namespace PlatformMngt
  {



    TestStdAttributes_System::TestStdAttributes_System () throw ()
    {

    }


    TestStdAttributes_System::~TestStdAttributes_System () throw ()
    {

    }

    bool 
    TestStdAttributes_System::m_is_one_shot_initialized=false;
  
    CdmwPlatformMngt::System_var TestStdAttributes_System::mysystem;

    void 
    TestStdAttributes_System::one_shot_init()
    {
      if(!m_is_one_shot_initialized)
      {
        m_is_one_shot_initialized = true;
     
      try
      {
        CORBA::ORB_var orb;
    orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager:: instance ()->get_ORB ());
    /*
       ===================================================
       Get the HelloInterface object
       ===================================================

       import the object reference from the ior file
     */

    PortableServer::POA_var poa = PortableServer::POA::_duplicate (Cdmw::TestUtils::CORBATestManager:: instance ()->get_POA ());
    
    PortableServer::POAManager_var poaMgr = PortableServer::POAManager::_duplicate(poa->the_POAManager());
    poaMgr->activate();


        CORBA::Object_var objref;

        //define the corbaloc
        const char *corbaloc_str = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";

        objref = orb->string_to_object(corbaloc_str);

        //resolve the System_ptr
        mysystem = CdmwPlatformMngt::System::_narrow(objref.in() );


    if (CORBA::is_nil (mysystem.in ()))
      {
       // TEST_INFO ("Error in narrowing CdmwPlatformMngt::System");
        exit (-1);
      }

       }
      catch (const Cdmw::Exception & ex)
      {
    std::cerr << ex.what () << std::endl;
    exit (-1);
      }
      catch (const CORBA::Exception & ex)
      {
    std::cerr << ex << std::endl;
    exit (-1);
      }
      catch (...)
      {
    std::cerr << "unexpected exception" << std::endl;
    exit (-1);
      }
      }
    }


    void TestStdAttributes_System::setUp()
    {
      TestStdAttributes_System::one_shot_init();
    }
/*
@brief   do_tests method for new methods (for std attributes) of IBB2
@brief   developments  .
*/

    void TestStdAttributes_System::check_name()
    {

      try
      {
    /* name of system to test */
    std::string sys_name = "MySystem";

    CORBA::String_var name = mysystem->name ();

    if (!strcmp (sys_name.c_str (), name.in ()))
      {
        CPPUNIT_ASSERT (1); /* test ok */
      }
    else
      {
        CPPUNIT_ASSERT (0); /* test not ok */
      }
       }
      catch (const Cdmw::Exception & ex)
      {
    std::cerr << ex.what () << std::endl;
    exit (-1);
      }
      catch (const CORBA::Exception & ex)
      {
    std::cerr << ex << std::endl;
    exit (-1);
      }
      catch (...)
      {
    std::cerr << "unexpected exception" << std::endl;
    exit (-1);
      }

     }

     void TestStdAttributes_System::get_name_by_snapshot()
    {
    std::string sys_name = "MySystem";
    CdmwPlatformMngt::SystemSnapshot_var snap;
    snap = mysystem->get_snapshot ();

    string conf (snap->system_name);

    if (conf == sys_name)
      {
        CPPUNIT_ASSERT (1); /* test ok */
      }
    else
      {
        CPPUNIT_ASSERT (0); /* test not ok */
      }
    }


    void TestStdAttributes_System::get_readonly_attributes()
    {
        try
        {
            CORBA::Any_var attr_any;
            mysystem->get_attribute(CdmwPlatformMngt::GLOBAL_SUPERVISION_HOST, attr_any.out(), CdmwPlatformMngt::PRE_DEFINED);
            const char* attr_char;
            CORBA::Boolean extract_succeeds = ( attr_any >>= attr_char ); 
            CPPUNIT_ASSERT(extract_succeeds == true); 
        }
        catch(...)
        {
            CPPUNIT_ASSERT(0);
        }
    }


    void TestStdAttributes_System::get_mode()
    {
    CdmwPlatformMngt::FunctioningMode sys_mode;
    CORBA::String_var mode_info;

    sys_mode = mysystem->get_mode (mode_info.out ());

    if (sys_mode == CdmwPlatformMngt::NORMAL_MODE)
      {
        CPPUNIT_ASSERT (1);
      }
    else
      {
        CPPUNIT_ASSERT (0);
      }
        }

    void TestStdAttributes_System::get_status()
    {
    CdmwPlatformMngt::SystemStatus sys_status;
    CORBA::String_var status_info;
    sys_status = mysystem->get_status (status_info.out ());


    if ((sys_status == CdmwPlatformMngt::SYSTEM_INITIAL) ||
        (sys_status == CdmwPlatformMngt::SYSTEM_STARTING) ||
        (sys_status == CdmwPlatformMngt::SYSTEM_STARTED))
      {
        CPPUNIT_ASSERT (1);
      }
    else
      {
        CPPUNIT_ASSERT (0);
      }
         }

    void TestStdAttributes_System::restart_warmth()
    {
        
        // Get requested restart warmth
        CdmwPlatformMngt::RestartWarmth myrestart = CdmwPlatformMngt::COLD_RESTART;
        CORBA::Any_var myany;// = new CORBA::Any();
        try {
            mysystem->get_attribute(CdmwPlatformMngt::REQUESTED_RESTART_WARMTH, myany.out(), CdmwPlatformMngt::PRE_DEFINED);
            CPPUNIT_ASSERT((myany.in()) >>= myrestart);
            CPPUNIT_ASSERT ( myrestart == CdmwPlatformMngt::HOT_RESTART);
        }
        catch(...) {
            CPPUNIT_ASSERT(false);    
        }         
        
        // Set default restart warmth
        myrestart = CdmwPlatformMngt::COLD_RESTART;          
        *myany <<= myrestart;
        try {
            mysystem->set_attribute (CdmwPlatformMngt::DEFAULT_RESTART_WARMTH, myany.in(), CdmwPlatformMngt::PRE_DEFINED);
        }
        catch(...) {
            CPPUNIT_ASSERT(false);    
        }         
        
        // Get default restart warmth
        try {
            mysystem->get_attribute (CdmwPlatformMngt::DEFAULT_RESTART_WARMTH, myany.out(), CdmwPlatformMngt::PRE_DEFINED);
            CPPUNIT_ASSERT(myany.in() >>= myrestart);
            CPPUNIT_ASSERT ( myrestart == CdmwPlatformMngt::COLD_RESTART );
        }
        catch(...) {
            CPPUNIT_ASSERT(false);    
        }

        // Get requested restart warmth
        try {
            mysystem->get_attribute (CdmwPlatformMngt::REQUESTED_RESTART_WARMTH, myany.out(), CdmwPlatformMngt::PRE_DEFINED);
            CPPUNIT_ASSERT(myany.in() >>= myrestart);
            CPPUNIT_ASSERT ( myrestart == CdmwPlatformMngt::HOT_RESTART );
        }
        catch(...) {
            CPPUNIT_ASSERT(false);    
        }
    }

    void TestStdAttributes_System::get_logger_default_filters()
    {
        try {
        string default_filters ( mysystem->get_logger_default_filters() );
        
        CPPUNIT_ASSERT (default_filters == "FATAL ERROR WARN INFO" );
        }
        catch(...) {
            CPPUNIT_ASSERT(false);    
        }
    
    }


     }
}



