/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_ENTITY_STATUS_CHANGE_IMPL_HPP
#define INCL_PLATFORMMNGT_ENTITY_STATUS_CHANGE_IMPL_HPP

#include <string>
#include "idllib/CdmwPlatformMngtEvent.stub.hpp"
#include <SystemMngt/platformvaluetypes/Event_impl.hpp>

namespace Cdmw
{        // Begin namespace Cdmw

    namespace PlatformMngt
    { // Begin namespace PlatformMngt
    
        /**
         *Purpose:
         *<p> A factory class for creating CdmwPlatformMngt::EntityStatusChange events.
         */

        class EntityStatusChangeFactory : public virtual CdmwPlatformMngt::EntityStatusChange_init
        {

            private:
                /**
                 * Purpose:
                 * <p> Inherited method used for unmarshalling factory object.
                 * 
                 *@return The unmarshalled factory.
                 */
                virtual CORBA::ValueBase* create_for_unmarshal();

            public:

                /**
                 * Purpose:
                 * <p> Creates a CdmwPlatformMngt::EntityStatusChange with the given properties
                 * provided as parameters. Use this method with a process's entity event.
                 * 
                 *@param entity_name The entity's name.
                 *@param element_path The entity's path.
                 *@param entity_status Its status .
                 *@param info Some additional information.
                 *
                 *@return the newly created CdmwPlatformMngt::EntityStatusChange object.
                 */
                CdmwPlatformMngt::EntityStatusChange* create(
                    const CdmwPlatformMngtBase::EventHeader& aHeader,
                    const char* system_name,
                    const char* entityName,
                    const char* element_path,
                    const char* entityStatus,
                    const char* theInfo );
        };

        /**
         *Purpose:
         *<p> An implementation class.
         *
         *Features:
         *<p> [Thread safety, exception safety, performance]
         *    [if no pertinent write none ]
         *
         *@see  [if necessary] 
         *@link [if necessary] 
         *
         */

        class EntityStatusChange_impl :
                    public virtual OBV_CdmwPlatformMngt::EntityStatusChange,
                    public virtual Event_impl
        {

                friend class EntityStatusChangeFactory;

            protected:
                /**
                 * Purpose:
                 * <p> The default constructor.
                 */
                EntityStatusChange_impl();

                /**
                 * Purpose:
                 * <p> Creates a CdmwPlatformMngt::EntityStatusChange_impl with the given properties
                 * provided as parameters. Use this method with a process's entity event.
                 * 
                 *@param entity_name The entity's name.
                 *@param element_path The entity's path.
                 *@param entity_status Its status .
                 *@param info Some additional information.
                 */
                EntityStatusChange_impl(
                    const CdmwPlatformMngtBase::EventHeader& aHeader,
                    const char* systemName,
                    const char* entityName,
                    const char* elementPath,
                    const char* entityStatus,
                    const char* theInfo );

                /**
                 * Purpose:
                 * <p> Returns the end of the description string containing :
                 * "E:'entity_name' entity_status : info"
                 */
                std::string endString();

            public:
                
                /**
                 * Inherited from CORBA::ValueBase
                 */
                CORBA::ValueBase* _copy_value();

                /**
                 * Purpose:
                 * <p> Provides the description of this event.
                 * 
                 *@return The description string of this event.
                 */
                char* to_string();

        };

    } // End namespace PlatformMngt
} // End namespace Cdmw

#endif //INCL_PLATFORMMNGT_ENTITY_STATUS_CHANGE_IMPL_HPP

