/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_RESTART_IMPL_HPP
#define INCL_PLATFORMMNGT_RESTART_IMPL_HPP

#include "idllib/CdmwPlatformMngtEvent.stub.hpp"
#include <SystemMngt/platformvaluetypes/CdmwCommand_impl.hpp>

namespace Cdmw
{        // Begin namespace Cdmw

    namespace PlatformMngt
    { // Begin namespace PlatformMngt

        /**
         *Purpose:
         *<p> A factory class for creating CdmwPlatformMngt::Restart events.
         *
         *Features:
         *<p> [Thread safety, exception safety, performance]
         *    [if no pertinent write none ]
         *
         *@see  [if necessary]
         *@link [if necessary]
         *
         */

        class RestartFactory : public virtual CdmwPlatformMngt::Restart_init
        {

            private:
                /**
                 * Purpose:
                 * <p> Inherited method used for unmarshalling factory object.
                 *
                 *@return The unmarshalled factory.
                 */
                virtual CORBA::ValueBase* create_for_unmarshal();

            public:
                /**
                 * Purpose:
                 * <p> Creates a CdmwPlatformMngt::Restart event
                 *@param header The Header of the Event created
                 *@param system_name Name of the system issuing this command event.
                 *@param elementh_path The element path of the managed element that received the command.
                 *@param element_name Its name.
                 *@param issuer The issuer that invocates the command
                 *@param startup_kind The startup_kind
                 *@param restart_warmth The Restart_warth (COLD-HOT)
                 *
                 *@return the newly created CdmwPlatformMngt::Restart object.
                 */
                               
                virtual CdmwPlatformMngt::Restart* create (
                                const ::CdmwPlatformMngtBase::EventHeader & header,
                                const char* sys_name,
                                const char* element_path,
                                const char* element_name,
                                const char* issuer,
                                const CdmwPlatformMngtBase::StartupKind & startup_kind,
                                const CdmwPlatformMngt::RestartWarmth   restart_warmth
                              );

           };

        /**
         *Purpose:
         *<p> An implementation class.
         *
         *Features:
         *<p> [Thread safety, exception safety, performance]
         *    [if no pertinent write none ]
         *
         *@see  [if necessary]
         *@link [if necessary]
         *
         */

        class Restart_impl : public virtual OBV_CdmwPlatformMngt::Restart,
                    public virtual CdmwCommand_impl
        {

            friend class RestartFactory;    
              protected:
                /**
                 * Purpose:
                 * <p> The default constructor.
                 */
                Restart_impl();

                /**
                 * Purpose:
                 * <p> Creates a CdmwPlatformMngt::Restart_impl with the given properties
                 * provided as parameters.
                 *
                 *@param header The Header of the Event created
                 *@param system_name Name of the system issuing this command event.
                 *@param elementh_path The element path of the managed element that received the command.
                 *@param element_name Its name.
                 *@param issuer The issuer that invocates the command
                 *@param startup_kind The startup_kind
                 *@param restart_warmth The Restart_warth (COLD-HOT)
                 */
                        
                
                Restart_impl( 
                        const ::CdmwPlatformMngtBase::EventHeader & header,
                        const char* sys_name,
                        const char* element_path,
                        const char* element_name,
                        const char* issuer,
                        const CdmwPlatformMngtBase::StartupKind& startup_kind,
                        const CdmwPlatformMngt::RestartWarmth  restart_warmth
                        );

                

            public:
                /**
                 * Inherited from CORBA::ValueBase
                 */
                virtual CORBA::ValueBase* _copy_value();
                virtual char* to_string();
                
        };

    } // End namespace PlatformMngt
} // End namespace Cdmw

#endif //INCL_PLATFORMMNGT_RESTART_IMPL_HPP

