/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "Foundation/orbsupport/CORBA.hpp"
#include "SystemMngt/platformvaluetypes/CdmwCommand_impl.hpp"
#include "SystemMngt/tools/ManagedElementKey.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"

namespace Cdmw { // Begin namespace Cdmw

namespace PlatformMngt { // Begin namespace PlatformMngt


/**
 * CdmwCommand_impl
 */
CdmwCommand_impl::CdmwCommand_impl() {

}

CdmwCommand_impl::CdmwCommand_impl(const char* elem_name, const char* isr) :
    OBV_CdmwPlatformMngt::CdmwCommand() {
    element_name(elem_name);
    issuer(isr);
}

std::string CdmwCommand_impl::extract_element_path(const char* element_path) throw(
        CORBA::BAD_PARAM) {
    //output parameters
    string system_name, application_name, host_name, process_name,
            component_name, entity_name, attribute_name, ftgroup_name,
            lbgroup_name;

    CdmwPlatformMngt::ElementType element_type;

    //parsing element path
    try
    {
        element_type =
        Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(element_path,
                system_name,
                application_name,
                host_name,
                process_name,
                component_name,
                entity_name,
                attribute_name,
                ftgroup_name,
                lbgroup_name);
    }
    catch (CdmwPlatformMngt::ElementPathInvalid)
    {
        throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
    }
    std::string str;
    switch (element_type) {
    case (CdmwPlatformMngt::SYSTEM_TYPE): {
        //This information is managed by Event_impl::beginString()
    }
        break;

    case (CdmwPlatformMngt::APPLICATION_TYPE): {
        
        str.append(" A:'");
        str.append(application_name);
        str.append("'");
    }
        break;

    case (CdmwPlatformMngt::HOST_TYPE): {
        
        str.append("'");
        str.append(" H:'");
        str.append(host_name);
        str.append("'");
    }
        break;

    case (CdmwPlatformMngt::PROCESS_TYPE): {
        
        str.append(" A:'");
        str.append(application_name);
        str.append(" H:'");
        str.append(host_name);
        str.append("'");
        str.append(" P:'");
        str.append(process_name);
        str.append("'");
    }
        break;

    default:
        throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
    }

    return str;

}

} // End namespace PlatformMngt
} // End namespace Perc

