/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <string>
#include "Foundation/orbsupport/CORBA.hpp"
#include "SystemMngt/platformvaluetypes/ComponentStatusChange_impl.hpp"
#include "SystemMngt/tools/ManagedElementKey.hpp"

namespace Cdmw
{        // Begin namespace Cdmw

    namespace PlatformMngt
    { // Begin namespace PlatformMngt

        /**
         * ComponentStatusChangeFactory
         */
        CORBA::ValueBase*
        ComponentStatusChangeFactory::create_for_unmarshal()
        {
            CdmwPlatformMngt::ComponentStatusChange_var componentEv
            = new ComponentStatusChange_impl();
            return componentEv._retn();
        }

        CdmwPlatformMngt::ComponentStatusChange*
        ComponentStatusChangeFactory::create(
            const CdmwPlatformMngtBase::EventHeader& aHeader,
            const char* systemName,
            const CdmwPlatformMngt::ProcessID& processId,
            const char* componentName,
            CdmwPlatformMngt::ComponentStatus componentStatus,
            const char* statusInfo )
        {
            
            CdmwPlatformMngt::ComponentStatusChange_var componentEv
            = new ComponentStatusChange_impl(
                  aHeader,
                  systemName,
                  processId,
                  componentName,
                  componentStatus,
                  statusInfo );

            return componentEv._retn();
        }

        /**
         * ComponentStatusChange_impl
         */
        ComponentStatusChange_impl::ComponentStatusChange_impl()
        {
            event_kind(CdmwPlatformMngt::COMPONENT_STATUS_CHANGE);
        }

        ComponentStatusChange_impl::ComponentStatusChange_impl(
            const CdmwPlatformMngtBase::EventHeader& aHeader,
            const char* systemName,
            const CdmwPlatformMngt::ProcessID& processId,
            const char* componentName,
            CdmwPlatformMngt::ComponentStatus componentStatus,
            const char* statusInfo )
                : OBV_CdmwPlatformMngt::ComponentStatusChange()
        {
            header( aHeader );
            system_name( systemName );
            event_kind( CdmwPlatformMngt::COMPONENT_STATUS_CHANGE );
            process_id( processId );
            component_name( componentName );
            component_status( componentStatus );
            status_info( statusInfo );
            OBV_CdmwPlatformMngt::Event::element_path(element_path());
        }

        CORBA::ValueBase* ComponentStatusChange_impl::_copy_value()
        {
            CdmwPlatformMngt::ComponentStatusChange_var componentEv
            = new ComponentStatusChange_impl(
                    header(),
                    system_name(),
                    process_id(),
                    component_name(), 
                    component_status(),
                  status_info());

            return componentEv._retn();
        }

        char* ComponentStatusChange_impl::to_string()
        {
            std::string result_str(beginString());

            result_str += " A:'";
            result_str += (process_id().application_name).in();
            result_str += "' H:'";
            result_str += (process_id().host_name).in();
            result_str += "' P:'";
            result_str += (process_id().process_name).in();
            result_str += " C:'";
            result_str += component_name();
            result_str += "' ";

            switch (component_status())
            {

                    case CdmwPlatformMngt::COMPONENT_INITIAL:
                    result_str += "INITIAL";
                    break;

                    case CdmwPlatformMngt::COMPONENT_CONFIGURING:
                    result_str += "CONFIGURING";
                    break;

                    case CdmwPlatformMngt::COMPONENT_CONFIGURED:
                    result_str += "CONFIGURED";
                    break;

                    case CdmwPlatformMngt::COMPONENT_INITIALISING:
                    result_str += "INITIALISING";
                    break;

                    case CdmwPlatformMngt::COMPONENT_INITIALISED:
                    result_str += "INITIALISED";
                    break;

                    case CdmwPlatformMngt::COMPONENT_STOPPING:
                    result_str += "STOPPING";
                    break;

                    case CdmwPlatformMngt::COMPONENT_STOPPED:
                    result_str += "STOPPED";
                    break;

                    case CdmwPlatformMngt::COMPONENT_FAILED_ERROR:
                    result_str += "FAILED_ERROR";
                    break;

                    case CdmwPlatformMngt::COMPONENT_FAILED_TIMEOUT:
                    result_str += "FAILED_TIMEOUT";
                    break;

                    case CdmwPlatformMngt::COMPONENT_FAILED_INVALID:
                    result_str += "FAILED_INVALID";
                    break;
            }

            result_str += " : ";
            result_str += status_info();

            CORBA::String_var str = CORBA::string_dup(result_str.c_str());
            return str._retn();

        }


        const char* ComponentStatusChange_impl::element_path() const
        {
            CORBA::String_var issuer = 
                     CORBA::string_dup(ManagedElementKey::Get_component_path(system_name(),
                                                                             (process_id().application_name).in(),
                                                                             (process_id().host_name).in(),
                                                                             (process_id().process_name).in(),
                                                                              component_name()).c_str());
            return issuer._retn();
        }


    } // End namespace PlatformMngt
} // End namespace Cdmw

