/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <string>
#include "Foundation/orbsupport/CORBA.hpp"
#include "SystemMngt/platformvaluetypes/ConfigurationChange_impl.hpp"

namespace Cdmw
{        // Begin namespace Cdmw

    namespace PlatformMngt
    { // Begin namespace PlatformMngt

        /**
         * ConfigurationChangeFactory
         */
        CORBA::ValueBase*
        ConfigurationChangeFactory::create_for_unmarshal()
        {
            CdmwPlatformMngt::ConfigurationChange_var elementEv
            = new ConfigurationChange_impl();

            return elementEv._retn();
        }

        CdmwPlatformMngt::ConfigurationChange* ConfigurationChangeFactory::create(
            const CdmwPlatformMngtBase::EventHeader& aHeader,
            const char* elementPath,
            const char* Info )
        {
            CdmwPlatformMngt::ConfigurationChange_var elementEv
            = new ConfigurationChange_impl(
                  aHeader,
                  elementPath,
                  Info );

            return elementEv._retn();
        }

        /**
         * ConfigurationChange_impl
         */
        ConfigurationChange_impl::ConfigurationChange_impl()
        {
            event_kind( CdmwPlatformMngt::CONFIGURATION_CHANGE );
        }

        ConfigurationChange_impl::ConfigurationChange_impl(
            const CdmwPlatformMngtBase::EventHeader& aHeader,
            const char* elementPath,
            const char* Info )
                : OBV_CdmwPlatformMngt::ConfigurationChange()
        {
            header( aHeader );
            OBV_CdmwPlatformMngt::Event::element_path( elementPath );
            event_kind( CdmwPlatformMngt::CONFIGURATION_CHANGE );
            info( Info );
        }

        CORBA::ValueBase* ConfigurationChange_impl::_copy_value()
        {
            CdmwPlatformMngt::ConfigurationChange_var elementEv
            = new ConfigurationChange_impl(
                  header(),
                  element_path(),
                  info() );

            return elementEv._retn();
        }

        char* ConfigurationChange_impl::to_string()
        {
            std::string result_str(beginString());

            result_str += " ";
            result_str += " : ";
            result_str += info();
            result_str += " ";
            result_str += " : ";
            result_str +=  element_path();
            result_str += " ";
            result_str += " : ";
            result_str += event_kind();
            CORBA::String_var str = CORBA::string_dup(result_str.c_str());
            return str._retn();
        }


    } // End namespace PlatformMngt
} // End namespace Cdmw

