/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <string>
#include "Foundation/orbsupport/CORBA.hpp"
#include "SystemMngt/platformvaluetypes/EntityStatusChange_impl.hpp"
#include "Foundation/logging/Logger.hpp"
#include "SystemMngt/tools/ManagedElementKey.hpp"

namespace Cdmw
{        // Begin namespace Cdmw

    namespace PlatformMngt
    { // Begin namespace PlatformMngt
    
    
        /**
         * EntityStatusChangeFactory
         */
        CORBA::ValueBase*
        EntityStatusChangeFactory::create_for_unmarshal()
        {
            CdmwPlatformMngt::EntityStatusChange_var entityEv
            = new EntityStatusChange_impl();

            return entityEv._retn();
        }


        CdmwPlatformMngt::EntityStatusChange*
        EntityStatusChangeFactory::create(
            const CdmwPlatformMngtBase::EventHeader& aHeader,
            const char* system_name,
            const char* entityName,
            const char* element_path,
            const char* entityStatus,
            const char* theInfo )
        {
            CdmwPlatformMngt::EntityStatusChange_var entityEv
            = new EntityStatusChange_impl(
                  aHeader,
                  system_name,
                  entityName,
                  element_path,
                  entityStatus,
                  theInfo );

            return entityEv._retn();
        }

        /**
         * EntityStatusChange_impl
         */
        EntityStatusChange_impl::EntityStatusChange_impl()
        {
            event_kind(CdmwPlatformMngt::ENTITY_STATUS_CHANGE);
        }

        EntityStatusChange_impl::EntityStatusChange_impl(
            const CdmwPlatformMngtBase::EventHeader& aHeader,
            const char* systemName,
            const char* entityName,
            const char* elementPath,
            const char* entityStatus,
            const char* theInfo )
                : OBV_CdmwPlatformMngt::EntityStatusChange()
        {
            header( aHeader );
            system_name( systemName );
            event_kind( CdmwPlatformMngt::ENTITY_STATUS_CHANGE );
            entity_name( entityName );
            element_path( elementPath );
            entity_status( entityStatus );
            info( theInfo );
        }

        CORBA::ValueBase* EntityStatusChange_impl::_copy_value()
        {
            CdmwPlatformMngt::EntityStatusChange_var entityEv
            = new EntityStatusChange_impl(
                  header(),
                  system_name(),
                  entity_name(),
                  element_path(),
                  entity_status(),
                  info() );

            return entityEv._retn();
        }

        std::string EntityStatusChange_impl::endString()
        {
            std::string endString("E:'");
            endString += entity_name();
            endString += "' ";
            endString += entity_status();
            endString += " : ";
            endString += info();

            return endString;
        }

        char* EntityStatusChange_impl::to_string()
        {
            std::string system_name, application_name, host_name, process_name, component_name;
            std::string entity_name, attribute_name, ftgroup_name, lbgroup_name;
            
            CdmwPlatformMngt::ElementType element_type;
            
            //parsing element path
            try
            {
                element_type = 
                    Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(element_path(),
                            system_name,
                            application_name, 
                            host_name,
                            process_name, 
                            component_name,
                            entity_name,
                            attribute_name,
                            ftgroup_name,
                            lbgroup_name);
            }
            catch (const CdmwPlatformMngt::ElementPathInvalid&)
            {
                CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                 "EntityStatusChange_impl::to_string ");
            }
            
            bool isASystemEntity = false;
            //Appending the header+issuer+system_name
            std::string result_str(beginString());

            if (host_name.empty() && application_name.empty())
            {
                //The systemTag is already inserted in the beginString()
                isASystemEntity=true;
            }               
            else if (!host_name.empty() && application_name.empty())
            {
                result_str += " H:'"+host_name;
            }
            else if (host_name.empty() && !application_name.empty())
            {
                result_str += " A:'"+application_name;
                if (!ftgroup_name.empty())
                {
                    result_str += " FG:'"+ftgroup_name;
                }                  
                else if (!lbgroup_name.empty())
                {
                    result_str += " LG:'"+lbgroup_name;
                }
            }
            else if (!host_name.empty() && !application_name.empty())
            {
                result_str += " A:'"+application_name;
                result_str += "' H:'"+host_name;
                result_str += "' P:'"+process_name;

                if (!component_name.empty())    
                {
                    result_str += "' C:'"+component_name;
                }
            }
            
            if (!isASystemEntity) result_str += "'";
            result_str += " ";
            //add the entity information
            result_str += endString();
            CORBA::String_var str = CORBA::string_dup(result_str.c_str());
            
            return str._retn();
        }

    } // End namespace PlatformMngt
} // End namespace Cdmw

