/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <sstream>
#include <string>
#include "Foundation/orbsupport/CORBA.hpp"
#include "idllib/CdmwPlatformMngtBase.stub.hpp"
#include "SystemMngt/platformvaluetypes/Event_impl.hpp"

namespace Cdmw
{        // Begin namespace Cdmw

    namespace PlatformMngt
    { // Begin namespace PlatformMngt


        /**
         * Event_impl
         */
        Event_impl::Event_impl()
        {}

        std::string Event_impl::headerString()
        {

            std::ostringstream temp;

            temp.fill('0');

            CdmwPlatformMngtBase::TimeStamp time_stamp = header().time_stamp;

            temp.width(4);
            temp << time_stamp.year << "/";
            temp.width(2);
            temp << time_stamp.month << "/";
            temp.width(2);
            temp << time_stamp.day << " ";

            temp.width(2);
            temp << time_stamp.hour << ":";
            temp.width(2);
            temp << time_stamp.minute << ":";
            temp.width(2);
            temp << time_stamp.second << ":";
            temp.width(2);
            temp << time_stamp.millisecond << ":";
            temp.width(2);
            temp << time_stamp.microsecond << " ";

            CdmwPlatformMngtBase::EventLevel level = header().level;

            /*
            switch(level)
            {
                case CdmwPlatformMngtBase::INF :
                    temp << CdmwPlatformMngtBase::INF_STR;
                    break;

                case CdmwPlatformMngtBase::WRN :
                    temp << CdmwPlatformMngtBase::WRN_STR;
                    break;

                case CdmwPlatformMngtBase::ERR :
                    temp << CdmwPlatformMngtBase::ERR_STR;
                    break;

                case CdmwPlatformMngtBase::FTL :
                    temp << CdmwPlatformMngtBase::FTL_STR;
                    break;

                default:
                    temp.width(3);

                    if (level < 0)
                    {
                        temp.fill(' ');
                    }

                    temp << level;
            }
            */ 
            // -----

            if ( level == CdmwPlatformMngtBase::INF )
                temp << CdmwPlatformMngtBase::INF_STR;
            else if ( level == CdmwPlatformMngtBase::WRN )
                temp << CdmwPlatformMngtBase::WRN_STR;
            else if ( level == CdmwPlatformMngtBase::ERR )
                temp << CdmwPlatformMngtBase::ERR_STR;
            else if ( level == CdmwPlatformMngtBase::FTL )
                temp << CdmwPlatformMngtBase::FTL_STR;
            else
            {
                temp.width(3);

                if (level < 0)
                {
                    temp.fill(' ');
                }

                temp << level;
            }

            // -----

            return temp.str();

        }


        //
        // IDL:thalesgroup.com/CdmwPlatformMngt/Event/get_issuer:1.0
        //
        char* Event_impl::get_issuer()
        {
            CORBA::String_var issuer = CORBA::string_dup(CdmwPlatformMngtBase::PFMMNG);
            return issuer._retn();
        }


        std::string Event_impl::beginString()
        {
            CORBA::String_var issuer = get_issuer();

            std::ostringstream temp;

            temp << headerString() << " ";

            temp.width(12); // the standard width for an issuer
            temp << issuer.in() << " S:'" << system_name() << "'";

            return temp.str();
        }

         
    } // End namespace PlatformMngt
} // End namespace Cdmw

