/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <string>
#include "SystemMngt/platformvaluetypes/GraphElement_impl.hpp"

namespace Cdmw
{        // Begin namespace Cdmw

    namespace PlatformMngt
    { // Begin namespace PlatformMngt

        /**
         * GraphElementFactory
         */
        CORBA::ValueBase*
        GraphElementFactory::create_for_unmarshal()
        {
            CdmwPlatformMngt::GraphElement_var graphElement
            = new GraphElement_impl();

            return graphElement._retn();
        }

        CdmwPlatformMngt::GraphElement*
        GraphElementFactory::create(
            const char* elementName,
            const unsigned short stepIndex,
            CORBA::ULong elementTimeout )
        {
            CdmwPlatformMngt::GraphElement_var graphElement
            = new GraphElement_impl( elementName, stepIndex, elementTimeout );

            return graphElement._retn();
        }
                
        /**
         * GraphElement_impl
         */
        GraphElement_impl::GraphElement_impl()
        {
            name( "" );
            step( 0 );
            timeout( 0 );
            precond_managed_element( "" );
            precond_status( "" );
            precond_timeout( 0 );
        }

        GraphElement_impl::GraphElement_impl(
            const char* elementName,
            const unsigned short stepIndex, 
            const unsigned long elementTimeout)
        {
            name( elementName );
            step( stepIndex );
            timeout (elementTimeout);
        }
        
        CORBA::ValueBase* GraphElement_impl::_copy_value()
        {
            CdmwPlatformMngt::GraphElement_var graphElement
            = new GraphElement_impl( name(), step() , timeout() );

            graphElement->successors( successors() );
            graphElement->precond_managed_element( precond_managed_element());
            graphElement->precond_status( precond_status());
            graphElement->precond_timeout( precond_timeout());
            
            return graphElement._retn();
        }

    } // End namespace PlatformMngt
} // End namespace Perc

