/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/orbsupport/CORBA.hpp"
#include "SystemMngt/platformvaluetypes/PropertyChange_impl.hpp"

namespace Cdmw
{        // Begin namespace Cdmw

    namespace PlatformMngt
    { // Begin namespace PlatformMngt

        /**
         * PropertyChangeFactory
         */
        CORBA::ValueBase*
        PropertyChangeFactory::create_for_unmarshal()
        {
            CdmwPlatformMngt::PropertyChange_var propvar = new PropertyChange_impl();
            return propvar._retn();
        }

        CdmwPlatformMngt::PropertyChange*
        PropertyChangeFactory::create (
                                const ::CdmwPlatformMngtBase::EventHeader & header,
                                const char * element_path,
                                const ::CdmwPlatformMngt::PropertyInfos & property_infos,
                                const char * info)
        {
            CdmwPlatformMngt::PropertyChange_var propvar =
            new PropertyChange_impl(header,
                                    element_path,
                                    property_infos,
                                    info);
            return propvar._retn();
        }

        /**
         * PropertyChange_impl
         */
        PropertyChange_impl::PropertyChange_impl()
        {
          event_kind( CdmwPlatformMngt::PROPERTY_CHANGE );
        }

        PropertyChange_impl::PropertyChange_impl(const ::CdmwPlatformMngtBase::EventHeader & hdr,
                                const char * elementPath,
                                const ::CdmwPlatformMngt::PropertyInfos & property_infos_ref,
                                const char * lpszinfo)
                                : OBV_CdmwPlatformMngt::PropertyChange()
        {
           event_kind( CdmwPlatformMngt::PROPERTY_CHANGE );
           header(hdr);
           OBV_CdmwPlatformMngt::Event::element_path(elementPath);
           info(lpszinfo);
           property_infos(property_infos_ref);
        }

        CORBA::ValueBase* PropertyChange_impl::_copy_value()
        {
            CdmwPlatformMngt::PropertyChange_var propvar = new PropertyChange_impl(
                  header(),
                  element_path(),
                  property_infos(),
                  info());

            return propvar._retn();
        }

    char* PropertyChange_impl::to_string()
        {
         ::CdmwPlatformMngt::PropertyInfos properties = property_infos();

            string str(beginString());
            str += " : element_path= ";
            str += element_path();
            str += " : info= ";
            str += info();
            CORBA::ULong size = properties.length();

            for (CORBA::ULong i=0; i<size; i++)
            {
            str+=" : property_name = ";
                str+=properties[i].property_name;
        }
        CORBA::String_var strret = CORBA::string_dup(str.c_str());
            return strret._retn();

        }

    } // End namespace PlatformMngt
} // End namespace Perc

