/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <string>
#include "Foundation/orbsupport/CORBA.hpp"
#include "SystemMngt/platformvaluetypes/RedundancyGroupStatusChange_impl.hpp"
#include "SystemMngt/tools/ManagedElementKey.hpp"

namespace Cdmw
{        // Begin namespace Cdmw

    namespace PlatformMngt
    { // Begin namespace PlatformMngt

        /**
         * RedundancyGroupStatusChangeFactory
         */
        CORBA::ValueBase*
        RedundancyGroupStatusChangeFactory::create_for_unmarshal()
        {
            CdmwPlatformMngt::RedundancyGroupStatusChange_var groupEv
            = new RedundancyGroupStatusChange_impl();
            return groupEv._retn();
        }

        CdmwPlatformMngt::RedundancyGroupStatusChange*
        RedundancyGroupStatusChangeFactory::create(
            const CdmwPlatformMngtBase::EventHeader& aHeader,
            const char* systemName,
            const char* groupName,
            CdmwPlatformMngt::TechnicalStatus groupStatus,
            CdmwPlatformMngt::ElementType groupType,
            const char* applicationName,
            const char* statusInfo )
        {
            CdmwPlatformMngt::RedundancyGroupStatusChange_var groupEv
            = new RedundancyGroupStatusChange_impl(
                  aHeader,
                  systemName,
                  groupName,
                  groupStatus,
                  groupType,
                  applicationName,
                  statusInfo );

            return groupEv._retn();
        }

        /**
         * RedundancyGroupStatusChange_impl
         */
        RedundancyGroupStatusChange_impl::RedundancyGroupStatusChange_impl()
        {
            event_kind(CdmwPlatformMngt::REDUNDANCY_GROUP_STATUS_CHANGE);
        }

        RedundancyGroupStatusChange_impl::RedundancyGroupStatusChange_impl(
                const CdmwPlatformMngtBase::EventHeader& aHeader,
                const char* systemName,
                const char* groupName,
                CdmwPlatformMngt::TechnicalStatus groupStatus,
                CdmwPlatformMngt::ElementType groupType,
                const char* applicationName,
                const char* statusInfo )
                : OBV_CdmwPlatformMngt::RedundancyGroupStatusChange()
        {
            header( aHeader );
            system_name( systemName );
            event_kind( CdmwPlatformMngt::REDUNDANCY_GROUP_STATUS_CHANGE );
            group_name( groupName );
            group_status( groupStatus );
            group_type( groupType ),
            application_name( applicationName ),
            status_info( statusInfo );
            OBV_CdmwPlatformMngt::Event::element_path(element_path());
        }

        CORBA::ValueBase* RedundancyGroupStatusChange_impl::_copy_value()
        {
            CdmwPlatformMngt::RedundancyGroupStatusChange_var groupEv
            = new RedundancyGroupStatusChange_impl(
                  header(),
                  system_name(),
                  group_name(), 
                  group_status(),
                  group_type(),
                  application_name(),
                  status_info());

            return groupEv._retn();
        }

        char* RedundancyGroupStatusChange_impl::to_string()
        {
            std::string result_str(beginString());

            result_str += " A:'";
            result_str += application_name();
            
            if (CdmwPlatformMngt::FT_GROUP_TYPE == group_type())
            {   
                result_str += " FTG:'"; 
            }
            else if (CdmwPlatformMngt::LB_GROUP_TYPE == group_type())
            {
                result_str += " LBG:'";
            }

            result_str += group_name();
            result_str += "' ";

            switch (group_status())
            {

                    case CdmwPlatformMngt::INITIAL:
                    result_str += "INITIAL";
                    break;

                    case CdmwPlatformMngt::FULLY_REDUNDANT:
                    result_str += "FULLY_REDUNDANT";
                    break;

                    case CdmwPlatformMngt::DEGRADED_REDUNDANCY:
                    result_str += "DEGRADED_REDUNDANCY";
                    break;

                    case CdmwPlatformMngt::LOST_REDUNDANCY:
                    result_str += "LOST_REDUNDANCY";
                    break;

                    case CdmwPlatformMngt::COMPLETE_FAILURE:
                    result_str += "COMPLETE_FAILURE";
                    break;

            }

            result_str += " : ";
            result_str += status_info();

            CORBA::String_var str = CORBA::string_dup(result_str.c_str());
            return str._retn();

        }


        const char* RedundancyGroupStatusChange_impl::element_path() const
        {
             CORBA::String_var issuer;
            if ( CdmwPlatformMngt::FT_GROUP_TYPE == group_type() )
            {
                issuer = 
                         CORBA::string_dup(ManagedElementKey::Get_ftgroup_path(system_name(),
                                                                               application_name(),
                                                                               group_name()).c_str());
            }
            else if ( CdmwPlatformMngt::LB_GROUP_TYPE == group_type() )
            {
                issuer = 
                         CORBA::string_dup(ManagedElementKey::Get_lbgroup_path(system_name(),
                                                                                 application_name(),
                                                                                 group_name()).c_str());                                        
            }
            
            return issuer._retn();
        }


    } // End namespace PlatformMngt
} // End namespace Cdmw

