/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/orbsupport/CORBA.hpp"
#include "SystemMngt/platformvaluetypes/Restart_impl.hpp"

namespace Cdmw
{        // Begin namespace Cdmw

namespace PlatformMngt
{ // Begin namespace PlatformMngt

/**
 * RestartFactory
 */
CORBA::ValueBase*
RestartFactory::create_for_unmarshal()
{
    CdmwPlatformMngt::Restart_var restartvar = new Restart_impl();
    return restartvar._retn();
}

CdmwPlatformMngt::Restart* RestartFactory::create (
        const ::CdmwPlatformMngtBase::EventHeader & header,
        const char* sys_name,
        const char* element_path,
        const char* element_name,
        const char* issuer,
        const CdmwPlatformMngtBase::StartupKind & startup_kind,
        const CdmwPlatformMngt::RestartWarmth   restart_warmth
)
{
    
    CdmwPlatformMngt::Restart_var restartvar =
        new Restart_impl(
                header,
                sys_name,
                element_path,
                element_name,
                issuer,
                startup_kind,
                restart_warmth);
    return restartvar._retn();
    
}

/**
 * Restart_impl
 */
Restart_impl::Restart_impl()
{
    event_kind( CdmwPlatformMngt::RESTART_REQUESTED );
}



Restart_impl::Restart_impl(
        const ::CdmwPlatformMngtBase::EventHeader &hdr,
        const char* sys_name,
        const char* elem_path,
        const char* elem_name,
        const char* isr,
        const CdmwPlatformMngtBase::StartupKind& strp_kind,
        const CdmwPlatformMngt::RestartWarmth   res_warmth
): OBV_CdmwPlatformMngt::Restart()
        {
            header(hdr);
            system_name(sys_name);
            element_path(elem_path);
            element_name(elem_name);
            event_kind(CdmwPlatformMngt::RESTART_REQUESTED);
            issuer(isr);
            startup_kind(strp_kind);
            restart_warmth(res_warmth); 
        }

        CORBA::ValueBase* Restart_impl::_copy_value()
        {
            CdmwPlatformMngt::Restart_var restartvar = new Restart_impl(
                    header(),
                    system_name(),
                    element_path(),
                    element_name(),
                    issuer(),
                    startup_kind(),
                    restart_warmth());

            return restartvar._retn();
        }

        char* Restart_impl::to_string()
        {

            string str(beginString());
            str.append(extract_element_path(element_path()));

            str += " command: RESTART ";
            str += " : Restart_warmth: ";
            if( restart_warmth() == CdmwPlatformMngt::COLD_RESTART)
            {
                str += "COLD_RESTART";
            }if( restart_warmth() == CdmwPlatformMngt::HOT_RESTART){
                str += "HOT_RESTART";
            }

            str += " issuer: ";
            str += issuer();
            CORBA::String_var strret = CORBA::string_dup(str.c_str());
            return strret._retn();


        }

} // End namespace PlatformMngt
} // End namespace Perc

