/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef PLUGIN_HPP_
#define PLUGIN_HPP_

#define CDMW_DLL_EXPORT extern "C"

#include <Foundation/orbsupport/CORBA.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp>
#include <string>

namespace Cdmw
{
namespace PlatformMngt
{

   class PluginThread;

   class Plugin
   {
      public:
         Plugin() {};
         virtual ~Plugin() {};
         
         virtual void set_up( ) = 0;
         virtual void run( ) = 0;
         virtual void tear_down() = 0;
         
         friend class PluginThread;
         
      protected:
         CdmwPlatformMngt::System_ptr get_system();
         std::string get_name() const;
         std::string get_args() const;
         std::string get_system_port() const;
         
      private: 
         void set_system(CdmwPlatformMngt::System_ptr);
         void set_name(const std::string & name);
         void set_args(const std::string & args);
         void set_system_port(const std::string & system_port);
         
         CdmwPlatformMngt::System_var m_system;
         std::string m_name;
         std::string m_args;
         std::string m_system_port;
   };

   
/**
 * @brief Alias for Plugin type.
 */
   typedef Plugin* Plugin_ptr;
   
   typedef Plugin_ptr CdmwCreateFunc();
   typedef void CdmwDestroyFunc(Plugin_ptr);
   
} // End namespace PlatformMngt
} // End namespace Cdmw

/**
* @brief This macro export a factory function to obtain
*        an handle to create internal symbols provided by dynamic library.
*/
#define CDMW_DLL_HANDLE(dll_type) \
CDMW_DLL_EXPORT Cdmw::PlatformMngt::Plugin_ptr cdmw_dll_create()  \
{                                             \
  return new dll_type();                      \
}                                             \
CDMW_DLL_EXPORT void cdmw_dll_destroy(Cdmw::PlatformMngt::Plugin_ptr dll_obj) \
{                                                         \
  delete dll_obj;                                         \
}


#endif /*PLUGIN_HPP_*/
