/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#ifndef INCL_PLATFORMMNGT_PLUGIN_CONFIGURATIONMNGR_HPP 
#define INCL_PLATFORMMNGT_PLUGIN_CONFIGURATIONMNGR_HPP 

#include <Foundation/common/Exception.hpp>
#include <string>
#include <vector>
#include <xercesc/dom/deprecated/DOM.hpp>
#include <xercesc/dom/deprecated/DOMParser.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/sax/ErrorHandler.hpp>
#include <xercesc/sax/SAXParseException.hpp>



namespace Cdmw
{
namespace PlatformMngt
{

class PluginConfigurationException : public Exception
{
    public:
        PluginConfigurationException(const std::string& message, size_t lineNumber)
            : Exception( Exception::FATAL, "Invalid configuration file" ),
              m_message(message), m_lineNumber(lineNumber)
        {
        }
        
        ~PluginConfigurationException() throw() {};
        
        const char* getMessage() const
        {
            return m_message.c_str();
        }
        
        size_t getLineNumber() const
        {
            return m_lineNumber;
        }
    
    private:
        std::string m_message;
        size_t m_lineNumber;
        
};

struct PluginInfo
{
      PluginInfo(const std::string& name,
                 const std::string& args,
                 const std::string& path)
         : m_name(name), m_args(args), m_path(path)
      {}
      PluginInfo()
         : m_name(""), m_args(""), m_path("")
      {}

      std::string m_name;
      std::string m_args;
      std::string m_path;
};

class PluginConfigurationMngr
{
    public:

        PluginConfigurationMngr();
        virtual ~PluginConfigurationMngr();
        
        void parse(const std::string& xml_conf, bool validate)
            throw(OutOfMemoryException, PluginConfigurationException);

        std::vector<PluginInfo> getPluginInfos();

    private:
        
        std::vector<PluginInfo> m_pluginInfos;
        static bool m_initialised;

}; // End class PluginConfigurationMngr 

} // End namespace PlatformMngt
} // End namespace Cdmw
#endif // INCL_PLATFORMMNGT_PLUGIN_CONFIGURATIONMNGR_HPP
