/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef PLUGINMANAGER_HPP_
#define PLUGINMANAGER_HPP_

#include <Foundation/orbsupport/CORBA.hpp>
#include <string>
#include <SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp>
#include <vector>
#include <SystemMngt/plugin/PluginThread.hpp>
#include <SystemMngt/plugin/PluginConfigurationMngr.hpp>


namespace Cdmw
{
namespace PlatformMngt
{
    
class PluginManager
{
public:
      PluginManager(CdmwPlatformMngt::System_ptr,
                    std::string system_port = "");
      virtual ~PluginManager();
    
      bool set_up_all(std::string conf_filename);
      bool run_all();
      bool tear_down_all(); 
      
      std::vector<std::string> get_error_descriptions();   
    
private:
      //TODO: insert here copy constructor and operator= overload
      PluginManager(const PluginManager&) {};
      
      enum Status { INITIAL, SETUPCALLED, RUNCALLED, TEARDOWNCALLED };
      Status m_status;
      
      std::vector<PluginThread*> m_plugins;
      bool m_alreadysetup;
      CdmwPlatformMngt::System_var m_system;
      std::string m_system_port;
      PluginConfigurationMngr pcm;
      std::vector<std::string> m_err_desc;
};

}
}


#endif /*PLUGINMANAGER_HPP_*/
