/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef PLUGINTHREAD_HPP_
#define PLUGINTHREAD_HPP_

#include <vector>
#include <string>
#include <Foundation/osthreads/Thread.hpp>
#include <SystemMngt/plugin/Plugin.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp>

namespace Cdmw
{
namespace PlatformMngt
{

class PluginThread : public Cdmw::OsSupport::Thread
{
public:
    PluginThread(CdmwPlatformMngt::System_ptr, 
                 std::string plugin_path,
                 std::string plugin_name = "",
                 std::string plugin_args = "",
                 std::string system_port = "");
    virtual ~PluginThread() throw ();
    
    bool set_up();
    bool tear_down();
    
    std::string get_plugin_path() const;
    std::string get_plugin_name() const;
    std::string get_plugin_args() const;
    std::string get_error_description();
    
protected:
    virtual void run() throw ();

private:
      std::string m_plugin_name;
      std::string m_plugin_args;
      std::string m_plugin_path;
      Plugin_ptr m_plugin;
      void* m_plugin_handle;
      CdmwPlatformMngt::System_var m_system;
      std::string m_system_port;
      std::string m_dlsym_error;
    
    
};

} // End namespace PlatformMngt
} // End namespace Cdmw

#endif /*PLUGINTHREAD_HPP_*/
