/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "plugin/PluginThread.hpp"

#include <dlfcn.h>
#include <iostream>
#include <Foundation/orbsupport/CORBA.hpp>
#include "plugin/Plugin.hpp"

namespace Cdmw
{
namespace PlatformMngt
{

PluginThread::PluginThread(CdmwPlatformMngt::System_ptr system, 
                           std::string plugin_path,
                           std::string plugin_name,
                           std::string plugin_args,
                           std::string system_port)
   : m_plugin_name(plugin_name), 
     m_plugin_args(plugin_args), 
     m_plugin_path(plugin_path), 
     m_plugin(NULL), m_plugin_handle(NULL),
     m_system_port(system_port)
{
    m_system = CdmwPlatformMngt::System::_duplicate(system);
    m_dlsym_error = std::string("");
}

PluginThread::~PluginThread() throw ()
{
}

bool PluginThread::set_up()
{
    // If the handle is not NULL it has already been created
    if (m_plugin_handle != NULL) return false;
    
    // Load dynamic library 
    m_plugin_handle = dlopen (m_plugin_path.c_str(), RTLD_LAZY|RTLD_GLOBAL);
    if (!m_plugin_handle)
    {
       std::cerr << "ERROR:'"<<dlerror ()<<"'" << std::endl;
        return false;
    }
    
    // reset errors
    dlerror ();
    
    // load the symbols
    Plugin_ptr (*create_my_plugin)();
    *(void **) (&create_my_plugin) = dlsym(m_plugin_handle, "cdmw_dll_create");
    
    if ( const char* err = dlerror())
    {
        m_dlsym_error = std::string(err);
        return false;
    }
    
    // create an instance of the class and call setUp
    m_plugin = (*create_my_plugin)();
    m_plugin->set_system( CdmwPlatformMngt::System::_duplicate(m_system.in()) ); 
    m_plugin->set_name( m_plugin_name );
    m_plugin->set_args( m_plugin_args );
    m_plugin->set_system_port( m_system_port );
    m_plugin->set_up( );
    
    return true;
}

std::string  PluginThread::get_error_description()
{
return m_dlsym_error;
}

void PluginThread::run() throw ()
{
    if (m_plugin != NULL) {
        m_plugin->run( ); 
    }
}

bool PluginThread::tear_down()
{
    if (m_plugin) {
        m_plugin->tear_down();
        delete (m_plugin);
        m_plugin = 0;
        dlclose(m_plugin_handle);
        return true;
    }
    else return false;
}

std::string PluginThread::get_plugin_path() const
{
    return m_plugin_path;
}
std::string PluginThread::get_plugin_name() const
{
    return m_plugin_name;
}
std::string PluginThread::get_plugin_args() const
{
    return m_plugin_args;
}

} // End namespace PlatformMngt
} // End namespace Cdmw



