/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "testplugin/MyPlugin.hpp"

#include <iostream>
#include <stdio.h>

namespace Cdmw
{
namespace PlatformMngt
{

MyPlugin::MyPlugin() : m_id(-1)
{
    handle = dlopen (NULL, RTLD_LAZY|RTLD_GLOBAL); // an handle to the main program
    if (!handle) {
        std::cerr << "MyPlugin: Error loading dynamic library: " << dlerror() << std::endl;
        return;
    }
    
    int (*GetMyID)();
    char *error;
    dlerror();    // Clear any existing error 
    *(void **) (&GetMyID) = dlsym(handle, "GetMyID");
    
    if ((error = dlerror()) != NULL)  {
        std::cerr << "MyPlugin: Error loading function: " << dlerror() << std::endl;
        return;
    }    
    m_id = (*GetMyID)(); 
    std::cerr << "MyPlugin: My ID is: " << m_id << std::endl;
       
}

void MyPlugin::set_up()
{
    std::cerr << "MyPlugin::setUp()" << std::endl;

    void (*SetSetUpCaller)(int id);
    void (*SetSystemPointer)(CdmwPlatformMngt::System_ptr p);
    char *error;
    if (!handle) {
        std::cerr << "MyPlugin: Dynamic library not loaded during setUp" << std::endl;
        return;
    }
    dlerror();    // Clear any existing error 
    *(void **) (&SetSetUpCaller) = dlsym(handle, "SetSetUpCaller");
    *(void **) (&SetSystemPointer) = dlsym(handle, "SetSystemPointer");
    
    if ((error = dlerror()) != NULL)  {
        std::cerr << "MyPlugin: Error loading function: " << dlerror() << std::endl;
        return;
    }    
    (*SetSetUpCaller)(m_id);
    
    CdmwPlatformMngt::System_ptr sys_pointer = get_system();
    (*SetSystemPointer)(sys_pointer);
}

void MyPlugin::run( )
{
    std::cerr << "MyPlugin::run()" << std::endl;
    
    void (*SetRunCaller)(int id);
    char *error;
    if (!handle) {
        std::cerr << "MyPlugin: Dynamic library not loaded during run"<< std::endl;
        return;
    }
    dlerror();    // Clear any existing error 
    *(void **) (&SetRunCaller) = dlsym(handle, "SetRunCaller");
    
    if ((error = dlerror()) != NULL)  {
        std::cerr << "MyPlugin: Error loading function: " << dlerror() << std::endl;
        return;
    }    
    (*SetRunCaller)(m_id);
}

void MyPlugin::tear_down()
{
    std::cerr << "MyPlugin::tearDown()" << std::endl;
    
    void (*SetTearDownCaller)(int id);
    char *error;
    if (!handle) {
        std::cerr << "MyPlugin: Dynamic library not loaded during tearDown" << std::endl;
        return;
    }
    dlerror();    // Clear any existing error 
    *(void **) (&SetTearDownCaller) = dlsym(handle, "SetTearDownCaller");
    
    if ((error = dlerror()) != NULL)  {
        std::cerr << "MyPlugin: Error loading function: " << dlerror() << std::endl;
        return;
    }    
    (*SetTearDownCaller)(m_id);
}

}
}

    
