/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

// Make a library with functions included in this header
#include "plugin/Plugin.hpp"

#include <vector>
#include <iostream>

bool m_setUp_called = false;
bool m_run_called = false;
bool m_tearDown_called = false;

std::vector<int> m_setup_callers;
std::vector<int> m_run_callers;
std::vector<int> m_teardown_callers;
int m_current_id = 0;
CdmwPlatformMngt::System_ptr m_system_pointer = NULL;


CDMW_DLL_EXPORT std::vector<int> GetSetUpCallers()
{
    std::cerr << "GetSetUpCallers() called" << std::endl;
    return m_setup_callers;
}

CDMW_DLL_EXPORT std::vector<int> GetRunCallers()
{
    std::cerr << "GetRunCallers() called" << std::endl;
    return m_run_callers;
}

CDMW_DLL_EXPORT std::vector<int> GetTearDownCallers()
{
    std::cerr << "GetTearDownCallers() called" << std::endl;
    return m_teardown_callers;
}

CDMW_DLL_EXPORT CdmwPlatformMngt::System* GetSystemPointer()
{
    std::cerr << "GetSystemPointer() called" << std::endl;
    return m_system_pointer;
}

CDMW_DLL_EXPORT void SetSetUpCaller(int id)
{
    std::cerr << "SetSetUpCaller() called by id: " << id << std::endl;
    m_setup_callers.push_back(id);
}

CDMW_DLL_EXPORT void SetRunCaller(int id)
{
    std::cerr << "SetRunCaller() called by id: " << id << std::endl;
    m_run_callers.push_back(id);
}

CDMW_DLL_EXPORT void SetTearDownCaller(int id)
{
    std::cerr << "SetTearDownCaller() called by id: " << id << std::endl;
    m_teardown_callers.push_back(id);
}

CDMW_DLL_EXPORT void SetSystemPointer(CdmwPlatformMngt::System_ptr p)
{
    std::cerr << "SetSystemPointer() called" << std::endl;
    m_system_pointer = p;
}

CDMW_DLL_EXPORT int GetMyID()
{
    m_current_id++;
    std::cerr << "GetMyID() called, returned id: " << m_current_id << std::endl;
    return m_current_id;   
}

CDMW_DLL_EXPORT void Reset()
{
    std::cerr << "Reset() called" << std::endl;
    m_current_id = 0;
    m_setup_callers.clear();
    m_run_callers.clear();
    m_teardown_callers.clear();
}





