/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_CDMW_PLATFORMMNGT_BASEISSUERMANAGER_HPP
#define INCL_CDMW_PLATFORMMNGT_BASEISSUERMANAGER_HPP

// private include files
#include "tools/TimeoutObserver.hpp"

// include files
#include <string>
#include <map>

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/common/Exception.hpp>
#include <Foundation/osthreads/ThreadHandle.hpp>
#include <Foundation/osthreads/ReaderWriterLock.hpp>
#include <Foundation/osthreads/Mutex.hpp>
#include <Foundation/osthreads/Condition.hpp>
 

/**
* Namespace for Supervision
*/
namespace Cdmw {
namespace PlatformMngt 
{


/**
*Purpose:
*<p>   
*
*Features:
*<p> 
*
*
*/

class BaseIssuerManager 
{

    public:   

        /**
        * Purpose:
        * <p> Destructor
        * 
        */ 
        virtual
        ~BaseIssuerManager()
                throw();

        static Cdmw::PlatformMngt::BaseIssuerManager* Instance();

        bool attach_element (const std::string& attaching_element_path)
             throw(CORBA::SystemException, Cdmw::Exception);

        bool detach_element (const std::string& detaching_element_path)
             throw(CORBA::SystemException, Cdmw::Exception);

        std::string get_command_issuer (const std::string& getting_issuer_seed)
            throw(CORBA::SystemException, Cdmw::Exception);

        void set_issuer (const std::string& setting_issuer_seed, const std::string& issuer_value)
            throw(CORBA::SystemException, Cdmw::Exception);

    protected:

        virtual
        Cdmw::OsSupport::Mutex* get_element_issuer_mutex (const std::string& element_seed)
            throw(CORBA::SystemException, Cdmw::Exception);

        virtual
        bool check_mutex_locked (const std::string& searching_path)
            throw(CORBA::SystemException, Cdmw::Exception);

        Cdmw::OsSupport::ReaderWriterLock m_mutexes_lock;
        Cdmw::OsSupport::ReaderWriterLock m_issuers_lock;

        std::map<std::string, Cdmw::OsSupport::Mutex*> m_mutexes_map;
        std::map<std::string, std::string> m_issuers_map;    

        BaseIssuerManager ();

    private:

        //
        // Copy ctor
        //
        BaseIssuerManager (const BaseIssuerManager& rhs)
                throw();

        //
        // Assignement operator
        //
        BaseIssuerManager& operator=(const BaseIssuerManager& rhs)
                 throw();
              
}; // End class BaseIssuerManager 

} // End namespace PlatformMngt 
}  // End namespace Cdmw

#endif // IINCL_CDMW_PLATFORMMNGT_BASEISSUERMANAGER_HPP
