/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_CDMW_PLATFORMMNGT_ISSUERMANAGER_HPP
#define INCL_CDMW_PLATFORMMNGT_ISSUERMANAGER_HPP

// private include files
#include "tools/TimeoutObserver.hpp"

// include files
#include <string>

#include <Foundation/osthreads/ThreadHandle.hpp>
#include <Foundation/osthreads/Mutex.hpp>
#include <Foundation/osthreads/Condition.hpp>
#include "Foundation/osthreads/Mutex.hpp"

/**
* Namespace for Supervision
*/
namespace Cdmw {
namespace PlatformMngt 
{


/**
*Purpose:
*<p>    This class implements a cancelable single shot timer
*       wich can be started and stopped multiple times and
*       with a modifiable timeout
*
*Features:
*<p> Thread safe, exception safe
*
*
*/

class IssuerManager 
{

    public:   

        /**
        * Purpose:
        * <p> Destructor
        * 
        */ 
        virtual 
        ~IssuerManager()
                throw();


        
        static Cdmw::PlatformMngt::IssuerManager* Instance();


                
        /**
        * Purpose:
        * <p> Change the timeout value
        *
        *@param unsigned long timeoutValue
        *
        *@exception BadOrderException if timer is already started
        */ 
        std::string get_issuer ()
            throw ();
        
       
        
        /**
        * Purpose:
        * <p> Cancel the timer
        *     Then the timer is ready to be started again
        *
        *@exception AlreadyDoneException if timer is already canceled
        */ 
        void set_issuer (const std::string& )
                    throw ();
        
       
             
    private:

        IssuerManager ();
        //
        // Copy ctor
        //
        IssuerManager (const IssuerManager& rhs)
                throw();


        //
        // Assignement operator
        //
        IssuerManager& operator=(const IssuerManager& rhs)
                 throw();
       
        /**
        * Mutex protecting stop flag and associated to the following
        * two conditions
        */
        Cdmw::OsSupport::Mutex m_mutex;
        
        std::string m_current_issuer;
        
        bool is_locked;
        
       
}; // End class IssuerManager 

} // End namespace PlatformMngt 
}  // End namespace Cdmw

#endif // INCL_CDMW_PLATFORMMNGT_ISSUERMANAGER_HPP
