/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_CDMW_PLATFORMMNGT_SYSTISSUERMANAGER_HPP
#define INCL_CDMW_PLATFORMMNGT_SYSTISSUERMANAGER_HPP

// private include files
#include "tools/BaseIssuerManager.hpp"

/**
* Namespace for Supervision
*/
namespace Cdmw {
namespace PlatformMngt 
{

/**
*Purpose:
*<p>   
*
*Features:
*<p> 
*
*
*/

class SystemIssuerManager : public BaseIssuerManager
{

    public:

        /**
        * Purpose:
        * <p> Destructor
        * 
        */ 
        virtual 
        ~SystemIssuerManager()
                throw();

        static Cdmw::PlatformMngt::SystemIssuerManager* Instance();

        bool attach_element (const std::string& system_path) throw (Cdmw::Exception);

        bool detach_element () throw (Cdmw::Exception);

        Cdmw::OsSupport::Mutex* get_system_issuer_mutex () throw (Cdmw::Exception);

        void set_issuer (const std::string& issuer_value) throw (Cdmw::Exception);

        std::string get_command_issuer ()  throw (Cdmw::Exception);

    private:

        SystemIssuerManager ();
        //
        // Copy ctor
        //
        SystemIssuerManager (const SystemIssuerManager& rhs)
                throw();

        bool detach_element (const std::string& detaching_element_path)
             throw(CORBA::SystemException, Cdmw::Exception);

        Cdmw::OsSupport::Mutex* get_element_issuer_mutex (const std::string& element_seed)
            throw(CORBA::SystemException, Cdmw::Exception);

        //
        // Assignement operator
        //
        SystemIssuerManager& operator=(const SystemIssuerManager& rhs)
                 throw();

        bool check_mutex_locked () throw (Cdmw::Exception);

        Cdmw::OsSupport::Mutex* m_system_mutex;

        std::string m_system_path;
        std::string m_issuer;

}; // End class SystemIssuerManager 

} // End namespace PlatformMngt 
}  // End namespace Cdmw

#endif // IINCL_CDMW_PLATFORMMNGT_SYSTISSUERMANAGER_HPP
