/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// C++ include
#include <iostream>
#include "tao/AnyTypeCode/TypeCode.h"

// Cdmw Files
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/PortableInterceptor.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/common/Assert.hpp> 
#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>

#include "tools/Current_impl.hpp"

using Cdmw::Logging::Logger;

//
// IDL:thalesgroup.com/CdmwFT:1.0
//
namespace Cdmw {
namespace PlatformMngt 
{

//
// IDL:thalesgroup.com/CdmwPlatformMngt/Current:1.0
//
Current_impl::Current_impl(PortableInterceptor::Current_ptr   pi_current,
                           const PortableInterceptor::SlotId  issuer)
   : m_pi_current(PortableInterceptor::Current::_duplicate(pi_current)),
     m_issuer_slot_id(issuer)
{
    CDMW_DEBUG(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
             "Current_impl object instantiating ");

}

Current_impl::~Current_impl()
{
    CDMW_DEBUG(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
             "Current_impl object destroing ");
}


//
// IDL:thalesgroup.com/CdmwPlatformMngt/Current/get_issuer:1.0
//
char*
Current_impl::get_issuer()
    throw(CORBA::SystemException)
{
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
             "Current_impl issuer value getting ");
    CDMW_LOG_FUNCTION(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),"");
    try
    {
        CORBA::Any_var slot_info = m_pi_current->get_slot(m_issuer_slot_id);

        CORBA::TypeCode_var tc = slot_info->type();
        if (tc->kind() == CORBA::tk_null)
        {
            // the given slot has not been set, raises NoContext
            CDMW_DEBUG(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
               "Current_impl::get_issuer(): this slot is not a string");
            
              CORBA::String_var empty_issuer (CORBA::string_dup(""));
              
              return empty_issuer._retn();
/*            
            CORBA::INTERNAL ex(OrbSupport::INTERNAL, CORBA::COMPLETED_NO); 
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
*/
        }

        const char* issuer;

        if (slot_info.in() >>= issuer)
        {
           // CDMW_LOG_FUNCTION_RETURN(client_id);     
            return CORBA::string_dup(issuer);
        }
        else
        {
            // Unexpected value inerted in Any            
            CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),"ERROR: this slot is not a string!" );
            
            CORBA::INTERNAL ex(OrbSupport::INTERNAL, CORBA::COMPLETED_NO); 
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
            
        }  
    }
    catch (const PortableInterceptor::InvalidSlot& )
    {
        // This slot that has not been allocated
        CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), 
                   "ERROR: InvalidSlot exception raised!");
                   
        CORBA::INTERNAL ex(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        
        throw ex;
    }
    catch (const CORBA::SystemException& ex)
    {
       CDMW_LOG_FUNCTION_EXCEPTION(ex);
       CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
             "Current_impl::get_issuer: issuer can't be got, CORBA::SystemException thrown "); 
        throw;
    }
    catch (...)
    {
        CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
             "Current_impl::get_issuer: issuer can't be got, unknown exception thrown ");

        CORBA::BAD_OPERATION ex(OrbSupport::BAD_OPERATION, CORBA::COMPLETED_NO); 
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void Current_impl::set_issuer (const char* issuer)
        throw (CORBA::SystemException)
{

    CDMW_LOG_FUNCTION(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),"");

    try
    {
        CORBA::Any any_issuer;

        any_issuer <<= issuer;

        m_pi_current->set_slot(m_issuer_slot_id, any_issuer);

    }
    catch (...)
    {
        CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
             "Current_impl::set_issuer: issuer can't be set, exception thrown ");

        CORBA::BAD_OPERATION ex(OrbSupport::BAD_OPERATION, CORBA::COMPLETED_NO); 
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }

}



} // End namespace PlatformMngt 
}  // End namespace Cdmw

