/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <sstream>

#include "SystemMngt/tools/ManagedElementKey.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/Assert.hpp"

// definition of local constants in an unnammed namespace
namespace 
{   
    const std::string key_sep = "/";
    const std::string system_suffix = ".sys";
    const std::string host_suffix = ".host";
    const std::string application_suffix = ".appl";
    const std::string process_suffix = ".proc";
    const std::string entity_suffix = ".ent";
    const std::string attribute_suffix = ".attr";    
    const std::string component_suffix = ".comp";
    const std::string ftgroup_suffix = ".ftgroup";
    const std::string lbgroup_suffix = ".lbgroup";
}

namespace Cdmw
{

namespace PlatformMngt
{


std::string ManagedElementKey::Get_system_path (
                                    const std::string & system_name)
{
    std::string element_path = system_name; 
    element_path += system_suffix;
    
    return element_path;
}
        

std::string ManagedElementKey::Get_host_path (
                                     const std::string & system_name,
                                     const std::string & host_name)
{
    std::string element_path = Get_system_path (system_name);
    element_path += key_sep;
    element_path += host_name;
    element_path += host_suffix;
    
    return element_path;
}                                     
        

std::string ManagedElementKey::Get_application_path (
                                     const std::string & system_name,
                                     const std::string & application_name)
{
    std::string element_path = Get_system_path (system_name);
    element_path += key_sep;
    element_path += application_name;
    element_path += application_suffix;
    
    return element_path;
}
                                     

std::string ManagedElementKey::Get_process_path (
                                     const std::string & system_name,
                                     const std::string & application_name,
                                     const std::string & host_name,
                                     const std::string & process_name)
{
    std::string element_path = Get_application_path (system_name, application_name);
    element_path += key_sep;
    element_path += host_name;
    element_path += host_suffix;
    
    element_path += key_sep;
    element_path += process_name;
    element_path += process_suffix;
    
    return element_path;
}

std::string ManagedElementKey::Get_component_rel_key (
                                      const std::string & host_name,
                                      const std::string & process_name,
                      const std::string & component_name)
{
    std::string element_key = host_name;
    element_key += key_sep;
    element_key += process_name;
    element_key += key_sep;
    element_key += component_name;
    
    return element_key;
}

std::string ManagedElementKey::Get_ftgroup_path (
                                     const std::string & system_name,
                                     const std::string & application_name,
                                     const std::string & ftgroup_name)
{
    std::string element_path = Get_application_path (system_name, application_name);
    element_path += key_sep;
    element_path += ftgroup_name;
    element_path += ftgroup_suffix;
    

    return element_path;
}


std::string ManagedElementKey::Get_lbgroup_path (
                                     const std::string & system_name,
                                     const std::string & application_name,
                                     const std::string & lbgroup_name)
{
    std::string element_path = Get_application_path (system_name, application_name);
    element_path += key_sep;
    element_path += lbgroup_name;
    element_path += lbgroup_suffix;
    

    return element_path;
}

std::string ManagedElementKey::Get_component_path (
                                     const std::string & system_name,
                                     const std::string & application_name,
                                     const std::string & host_name,
                                     const std::string & process_name,
                     const std::string & component_name)
{
    std::string element_path = Get_application_path (system_name, application_name);
    element_path += key_sep;
    element_path += host_name;
    element_path += host_suffix;
    
    element_path += key_sep;
    element_path += process_name;
    element_path += process_suffix;

    element_path += key_sep;
    element_path += component_name;
    element_path += component_suffix;
    
    return element_path;
}

std::string ManagedElementKey::Get_component_path (
                                      const std::string & parent_path,
                                      const std::string & host_name,
                                      const std::string & process_name,
                      const std::string & component_name)
{
    std::string element_path = parent_path;
    element_path += key_sep;
    element_path += host_name;
    element_path += host_suffix;
    
    element_path += key_sep;
    element_path += process_name;
    element_path += process_suffix;

    element_path += key_sep;
    element_path += component_name;
    element_path += component_suffix;
    
    return element_path;
}



std::string ManagedElementKey::Get_process_path (
                                      const std::string & parent_path,
                                      const std::string & host_name,
                                      const std::string & process_name)
{
    std::string element_path = parent_path;
    element_path += key_sep;
    element_path += host_name;
    element_path += host_suffix;
    
    element_path += key_sep;
    element_path += process_name;
    element_path += process_suffix;
    
    return element_path;
}


std::string ManagedElementKey::Get_ftgroup_rel_key (
                                      const std::string & application_name,
                                      const std::string & ftgroup_name)
{
    std::string element_key = application_name;
    element_key += key_sep;
    element_key += ftgroup_name;
    
    return element_key;
}


std::string ManagedElementKey::Get_lbgroup_rel_key (
                                      const std::string & application_name,
                                      const std::string & lbgroup_name)
{
    std::string element_key = application_name;
    element_key += key_sep;
    element_key += lbgroup_name;
    
    return element_key;
}
                                      
                                      
std::string ManagedElementKey::Get_process_rel_key (
                                      const std::string & host_name,
                                      const std::string & process_name)
{
    std::string element_key = host_name;
    element_key += key_sep;
    element_key += process_name;
    
    return element_key;
}

bool ManagedElementKey::Check_process_rel_key (const std::string & process_rel_key,
                                               std::string & process_name)
            throw (CdmwPlatformMngt::ElementPathInvalid)
{
    bool result = false;    // false if no error
    process_name = "";
    const unsigned int host_position = 2;
    
    std::string::size_type pos_sep1 = 0;
    std::string::size_type key_size = process_rel_key.size();
    
    pos_sep1 = process_rel_key.find(key_sep);
    
    // check if host name and process name are found
    if (pos_sep1 != std::string::npos)
    {
        // set error if host is undefined
        if (pos_sep1 < host_position)
        {
            result = true;
        }
        
        pos_sep1 += key_sep.size();
    }
    else
    {
        // no separator : host name is undefined, only process is defined
        result = true;
        pos_sep1 = 0;
    }
    
    // check if process name found
    if (pos_sep1 < key_size)
    { 
        process_name = process_rel_key.substr(pos_sep1);
    }
    else
    {
        // relative key is not valid
        throw CdmwPlatformMngt::ElementPathInvalid();
    }
    
    return result;
}
                                      
void ManagedElementKey::Get_process_element_names (
                                   const std::string & process_rel_key,                                         
                                   std::string & host_name,
                                   std::string & process_name)
    throw (CdmwPlatformMngt::ElementPathInvalid)
{
    host_name = "";
    process_name = "";
    
    std::string::size_type pos_sep1 = 0;
    std::string::size_type key_size = process_rel_key.size();
    
    pos_sep1 = process_rel_key.find(key_sep);
    
    // check if host name found
    if (pos_sep1 != std::string::npos)
    {
        if (pos_sep1 > 1)
        {
            host_name = process_rel_key.substr(0, pos_sep1);
        }
        
        pos_sep1 += key_sep.size();
    }
    else
    {
        // relative key is not valid
        throw CdmwPlatformMngt::ElementPathInvalid();
    }
    
    // check if process name found
    if (pos_sep1 < key_size)
    { 
        process_name = process_rel_key.substr(pos_sep1);
    }
    else
    {
        // relative key is not valid
        throw CdmwPlatformMngt::ElementPathInvalid();
    }
}

                                     

std::string ManagedElementKey::Get_system_entity_path (
                                     const std::string & system_name,
                                     const std::string & entity_name)
{
    std::string element_path = Get_system_path (system_name);
    element_path += key_sep;
    element_path += entity_name;
    element_path += entity_suffix;
    
    return element_path;
}        


std::string ManagedElementKey::Get_host_entity_path (
                                     const std::string & system_name,
                                     const std::string & host_name,
                                     const std::string & entity_name)
{
    std::string element_path = Get_host_path (system_name, host_name);
    element_path += key_sep;
    element_path += entity_name;
    element_path += entity_suffix;
    
    return element_path;
}
        

std::string ManagedElementKey::Get_application_entity_path (
                                     const std::string & system_name,
                                     const std::string & application_name,
                                     const std::string & entity_name)
{
    std::string element_path = Get_application_path (system_name, application_name);
    element_path += key_sep;
    element_path += entity_name;
    element_path += entity_suffix;
    
    return element_path;
}
                                     
        
std::string ManagedElementKey::Get_process_entity_path (
                                     const std::string & system_name,
                                     const std::string & application_name,
                                     const std::string & host_name,
                                     const std::string & process_name,
                                     const std::string & entity_name)
{
    std::string element_path = Get_process_path (system_name, application_name,
                                                 host_name, process_name);
    element_path += key_sep;
    element_path += entity_name;
    element_path += entity_suffix;
    
    return element_path;
}


std::string ManagedElementKey::Get_ftgroup_entity_path (
                                     const std::string & system_name,
                                     const std::string & application_name,
                                     const std::string & ftgroup_name,
                                     const std::string & entity_name)
{
    std::string element_path = Get_ftgroup_path (system_name, application_name,
                                                  ftgroup_name);
    element_path += key_sep;
    element_path += entity_name;
    element_path += entity_suffix;
    
    return element_path;
}


std::string ManagedElementKey::Get_lbgroup_entity_path (
                                     const std::string & system_name,
                                     const std::string & application_name,
                                     const std::string & lbgroup_name,
                                     const std::string & entity_name)
{
    std::string element_path = Get_lbgroup_path (system_name, application_name,
                                                  lbgroup_name);
    element_path += key_sep;
    element_path += entity_name;
    element_path += entity_suffix;
    
    return element_path;
}

std::string ManagedElementKey::Get_component_entity_path (
                                     const std::string & system_name,
                                     const std::string & application_name,
                                     const std::string & host_name,
                                     const std::string & process_name,
                                     const std::string & component_name,
                                     const std::string & entity_name)
{
        std::string element_path = Get_component_path (system_name, application_name,
                                                     host_name, process_name,
                                                     component_name);
        element_path += key_sep;
        element_path += entity_name;
        element_path += entity_suffix;
        
        return element_path;
}


std::string ManagedElementKey::Get_entity_path (
                                     const std::string & parent_path,
                                     const std::string & entity_name)
    throw (CdmwPlatformMngt::ElementPathInvalid)
{
    std::string system_name = "";
    std::string application_name = "";
    std::string host_name = "";
    std::string process_name = "";
    std::string buffer = "";
    std::string component_name = "";
    std::string attribute_name = "";
    
    std::string entity_path = "";

    std::string ftgroup_name = "" ;
    std::string lbgroup_name = "" ;
        
    // get element path components from parent
    CdmwPlatformMngt::ElementType parent_type = 
        Get_element_names (parent_path,                                         
                           system_name,
                           application_name,
                           host_name,
                           process_name,
                           component_name,
                           buffer,
                           attribute_name,
               ftgroup_name,
               lbgroup_name);

    
    switch (parent_type)
    {
         case (CdmwPlatformMngt::SYSTEM_TYPE) :
            entity_path = Get_system_entity_path (system_name, entity_name);
            break;
         case (CdmwPlatformMngt::HOST_TYPE) :
            entity_path = Get_host_entity_path (system_name,
                                                host_name,
                                                entity_name);
            break;
         case (CdmwPlatformMngt::APPLICATION_TYPE) :
            entity_path = Get_application_entity_path (system_name, 
                                                       application_name,
                                                       entity_name);
            break;
         case (CdmwPlatformMngt::PROCESS_TYPE) :
            entity_path = Get_process_entity_path (system_name, 
                                                   application_name,
                                                   host_name,
                                                   process_name,
                                                   entity_name);
       
        break;
         case (CdmwPlatformMngt::COMPONENT_TYPE) :
            entity_path = Get_component_entity_path (system_name, 
                                                   application_name,
                                                   host_name,
                                                   process_name,
                           component_name,
                                                   entity_name);

            break;

         case (CdmwPlatformMngt::FT_GROUP_TYPE) :
            entity_path = Get_ftgroup_entity_path (system_name, 
                                                   application_name,
                           ftgroup_name,
                           entity_name);
            break;

         case (CdmwPlatformMngt::LB_GROUP_TYPE) :
            entity_path = Get_lbgroup_entity_path (system_name, 
                                                   application_name,
                           lbgroup_name,
                           entity_name);
            break;



         default:
            // element path is not valid
            throw CdmwPlatformMngt::ElementPathInvalid();
    }
    
    return entity_path;
}

CdmwPlatformMngt::ElementType ManagedElementKey::Get_element_names (
                                            const std::string & element_path,                                         
                                            std::string & system_name,
                                            std::string & application_name,
                                            std::string & host_name,
                                            std::string & process_name,
                                            std::string & component_name,              
                                            std::string & entity_name,
                                            std::string & attribute_name,
                                            std::string & ftgroup_name,
                                            std::string & lbgroup_name)
    throw (CdmwPlatformMngt::ElementPathInvalid)
{

    std::string element = "";
    std::string::size_type pos_element = 0;
    pos_element = element_path.find(system_suffix);
    if (pos_element == std::string::npos || pos_element == 0){
        // element path is not valid
        throw CdmwPlatformMngt::ElementPathInvalid();
    }
    else{
        element = element_path.substr(pos_element-1, 1);
    //std::cerr << "element = " << element << std::endl;
    if(!strcmp("/",  element.c_str())){
        // element path is not valid
            throw CdmwPlatformMngt::ElementPathInvalid();
    }
    }
    
    pos_element = element_path.find(host_suffix);
    if (pos_element != std::string::npos ){
        element = element_path.substr(pos_element-1, 1);
    if(!strcmp("/",  element.c_str())){
        // element path is not valid
            throw CdmwPlatformMngt::ElementPathInvalid();
    }
    }
    
    pos_element = element_path.find(application_suffix);
    if (pos_element != std::string::npos ){
        element = element_path.substr(pos_element-1, 1);
    if(!strcmp("/",  element.c_str())){
        // element path is not valid
            throw CdmwPlatformMngt::ElementPathInvalid();
    }
    }
    
    pos_element = element_path.find(process_suffix);
    if (pos_element != std::string::npos ){
        element = element_path.substr(pos_element-1, 1);
    if(!strcmp("/",  element.c_str())){
        // element path is not valid
            throw CdmwPlatformMngt::ElementPathInvalid();
    }
    }
    
    pos_element = element_path.find(entity_suffix);
    if (pos_element != std::string::npos ){
        element = element_path.substr(pos_element-1, 1);
    if(!strcmp("/",  element.c_str())){
        // element path is not valid
            throw CdmwPlatformMngt::ElementPathInvalid();
    }
    }
    
    pos_element = element_path.find(attribute_suffix);
    if (pos_element != std::string::npos ){
        element = element_path.substr(pos_element-1, 1);
    if(!strcmp("/",  element.c_str())){
        // element path is not valid
            throw CdmwPlatformMngt::ElementPathInvalid();
    }
    }
    
    pos_element = element_path.find(component_suffix);
    if (pos_element != std::string::npos ){
        element = element_path.substr(pos_element-1, 1);
    if(!strcmp("/",  element.c_str())){
        // element path is not valid
            throw CdmwPlatformMngt::ElementPathInvalid();
    }
    }
    
    pos_element = element_path.find(ftgroup_suffix);
    if (pos_element != std::string::npos ){
        element = element_path.substr(pos_element-1, 1);
    if(!strcmp("/",  element.c_str())){
        // element path is not valid
            throw CdmwPlatformMngt::ElementPathInvalid();
    }
    }
    
    pos_element = element_path.find(lbgroup_suffix);
    if (pos_element != std::string::npos ){
        element = element_path.substr(pos_element-1, 1);
    if(!strcmp("/",  element.c_str())){
        // element path is not valid
            throw CdmwPlatformMngt::ElementPathInvalid();
    }
    }
    
    
    CdmwPlatformMngt::ElementType element_type = CdmwPlatformMngt::SYSTEM_TYPE;
    
    system_name = "";
    application_name = "";
    host_name = "";
    process_name = "";
    entity_name = "";
    component_name = "";
    attribute_name = "";

    ftgroup_name = "" ;
    lbgroup_name = "" ;
    
    std::string::size_type pos_suffix1 = 0;
    std::string::size_type pos_suffix2 = 0;
    std::string::size_type path_size = element_path.size();
    
    pos_suffix1 = element_path.find(system_suffix);
    
    // check if system name found
    if (pos_suffix1 != std::string::npos)
    {
        system_name = element_path.substr(0, pos_suffix1);
        pos_suffix1 += (system_suffix.size()+ 1);
        element_type = CdmwPlatformMngt::SYSTEM_TYPE;
    }
    else
    {
        // element path is not valid
        throw CdmwPlatformMngt::ElementPathInvalid();
    }
    
    // check if application name found
    if (pos_suffix1 < path_size)
    { 
        pos_suffix2 = element_path.find(application_suffix, pos_suffix1);
    
        // if application name found
        if (pos_suffix2 != std::string::npos)
        {
            application_name = element_path.substr(pos_suffix1, pos_suffix2-pos_suffix1);
            pos_suffix1 = pos_suffix2 + (application_suffix.size()+ 1);
            element_type = CdmwPlatformMngt::APPLICATION_TYPE;
        }

    }
    
    // check if host name found
    if (element_type == CdmwPlatformMngt::SYSTEM_TYPE && pos_suffix1 < path_size)
    { 
        pos_suffix2 = element_path.find(host_suffix, pos_suffix1);
    
        // if host name found
        if (pos_suffix2 != std::string::npos)
        {
            host_name = element_path.substr(pos_suffix1, pos_suffix2-pos_suffix1);
            pos_suffix1 = pos_suffix2 + (host_suffix.size()+ 1);
            element_type = CdmwPlatformMngt::HOST_TYPE;
        }
    }
    
    
    // check if process name found
    if (element_type == CdmwPlatformMngt::APPLICATION_TYPE && pos_suffix1 < path_size)
    {
        // check if ftgroup name found
        pos_suffix2 = element_path.find(ftgroup_suffix, pos_suffix1);
                
        // if ftgroup name found
        if (pos_suffix2 != std::string::npos)
        {
            ftgroup_name = element_path.substr(pos_suffix1, pos_suffix2-pos_suffix1);
            pos_suffix1 = pos_suffix2 + (ftgroup_suffix.size()+ 1);
            element_type = CdmwPlatformMngt::FT_GROUP_TYPE;
        }
        else
        {
            pos_suffix2 = element_path.find(lbgroup_suffix, pos_suffix1);
                
            // if ftgroup name found
            if (pos_suffix2 != std::string::npos)
            {
                lbgroup_name = element_path.substr(pos_suffix1, pos_suffix2-pos_suffix1);
                pos_suffix1 = pos_suffix2 + (lbgroup_suffix.size()+ 1);
                element_type = CdmwPlatformMngt::LB_GROUP_TYPE;
            }
            else
            {        
             
                // search host name
                pos_suffix2 = element_path.find(host_suffix, pos_suffix1);
                // if host name found
                if (pos_suffix2 != std::string::npos)
                {
                    host_name = element_path.substr(pos_suffix1, pos_suffix2-pos_suffix1);
                    pos_suffix1 = pos_suffix2 + (host_suffix.size()+ 1);
                    
                    // then search process name
                    if (pos_suffix1 < path_size)
                    { 
                        pos_suffix2 = element_path.find(process_suffix, pos_suffix1);
                        
                        // if process name found
                        if (pos_suffix2 != std::string::npos)
                        {
                            process_name = element_path.substr(pos_suffix1, pos_suffix2-pos_suffix1);
                            pos_suffix1 = pos_suffix2 + (process_suffix.size()+ 1);
                            element_type = CdmwPlatformMngt::PROCESS_TYPE;
                            
                            // then search component name
                            if (pos_suffix1 < path_size)
                            {
                                pos_suffix2 = element_path.find(component_suffix, pos_suffix1);
                                
                                // if component name found
                                if (pos_suffix2 != std::string::npos)
                                {
                                    component_name = element_path.substr(pos_suffix1, pos_suffix2-pos_suffix1);
                                    pos_suffix1 = pos_suffix2 + (process_suffix.size()+ 1);
                                    element_type = CdmwPlatformMngt::COMPONENT_TYPE;
                                }
                            }
                        }
                        else
                        {
                            // element path is not valid
                            throw CdmwPlatformMngt::ElementPathInvalid();
                        } 
        
                    }
                    else
                    {
                        // element path is not valid
                        throw CdmwPlatformMngt::ElementPathInvalid();
                    }           
                }
            }
        }
    }
    
    // check if entity name found
    if (pos_suffix1 < path_size)
    { 
        pos_suffix2 = element_path.find(entity_suffix, pos_suffix1);
    
        // if entity name found
        if (pos_suffix2 != std::string::npos)
        {
            entity_name = element_path.substr(pos_suffix1, pos_suffix2-pos_suffix1);
            pos_suffix1 = pos_suffix2 + (entity_suffix.size()+ 1);
            element_type = CdmwPlatformMngt::ENTITY_TYPE;
        }
    }
    
    
    // check if attribute name found 
    if (pos_suffix1 < path_size)
    { 
        pos_suffix2 = element_path.find(attribute_suffix, pos_suffix1);
    
        // if attribute name found
        if (pos_suffix2 != std::string::npos)
        {
            attribute_name = element_path.substr(pos_suffix1, pos_suffix2-pos_suffix1);
            pos_suffix1 = pos_suffix2 + (attribute_suffix.size()+ 1);
        }
        else 
        { 
            // element path is not valid
            throw CdmwPlatformMngt::ElementPathInvalid();        
        }
    }    

    if ( pos_suffix1 < path_size )
    {
        throw CdmwPlatformMngt::ElementPathInvalid();
    }
    
    return element_type;
}

CdmwPlatformMngt::ElementType ManagedElementKey::Get_element_type (
                                       const std::string & element_path)
    throw (CdmwPlatformMngt::ElementPathInvalid)
{
    CdmwPlatformMngt::ElementType element_type = CdmwPlatformMngt::SYSTEM_TYPE;
    
    std::string::size_type pos_suffix1 = 0;
    std::string::size_type path_size = element_path.size();
    
    pos_suffix1 = element_path.find(system_suffix);
    
    // check if system name found
    if (pos_suffix1 != std::string::npos)
    {       pos_suffix1 += (system_suffix.size()+ 1);
        element_type = CdmwPlatformMngt::SYSTEM_TYPE;
    }
    else
    {       // element path is not valid
        throw CdmwPlatformMngt::ElementPathInvalid();
    }
    
    // check if application name found
    if (pos_suffix1 < path_size)
    {         
        std::string::size_type tmp_pos_suffix = 0;
        tmp_pos_suffix = element_path.find(application_suffix, pos_suffix1);
    
        // if application name found
        if (tmp_pos_suffix != std::string::npos)
        {           
            pos_suffix1 = tmp_pos_suffix;
            pos_suffix1 += (application_suffix.size()+ 1);
            element_type = CdmwPlatformMngt::APPLICATION_TYPE;
        }
    }

    // check if host name found
    if ((element_type == CdmwPlatformMngt::SYSTEM_TYPE) && (pos_suffix1 < path_size))
    { 
    
        std::string::size_type tmp_pos_suffix = 0;
        tmp_pos_suffix = element_path.find(host_suffix, pos_suffix1);
    
        // if host name found
        if (tmp_pos_suffix != std::string::npos)
        {
            pos_suffix1 = tmp_pos_suffix;
            pos_suffix1 += (host_suffix.size()+ 1);
            element_type = CdmwPlatformMngt::HOST_TYPE;
        }
    }
    
    
    // check if process name found
    if (element_type == CdmwPlatformMngt::APPLICATION_TYPE && pos_suffix1 < path_size)
    { 
        //search ftgroup type ...
        std::string::size_type tmp_pos_suffix = 0;
        tmp_pos_suffix = element_path.find(ftgroup_suffix, pos_suffix1);
        if (tmp_pos_suffix != std::string::npos)
        {
            pos_suffix1 = tmp_pos_suffix;
            pos_suffix1 += (ftgroup_suffix.size()+ 1);
            element_type = CdmwPlatformMngt::FT_GROUP_TYPE;
        }
        else
        {
            //search lbgroup type ...
            tmp_pos_suffix = element_path.find(lbgroup_suffix, pos_suffix1);
            if (tmp_pos_suffix != std::string::npos)
            {
                pos_suffix1 = tmp_pos_suffix;
                pos_suffix1 += (lbgroup_suffix.size()+ 1);
                element_type = CdmwPlatformMngt::LB_GROUP_TYPE;
            }
            else
            {
    
    
                // first search host name
                tmp_pos_suffix = element_path.find(host_suffix, pos_suffix1);
            
                // if host name found
                if (tmp_pos_suffix != std::string::npos)
                {
                    pos_suffix1 = tmp_pos_suffix;
                    pos_suffix1 += (host_suffix.size()+ 1);
                    
                    // then search process name
                    if (pos_suffix1 < path_size)
                    { 
                        tmp_pos_suffix = element_path.find(process_suffix, pos_suffix1);
                        
                        // if process name found
                        if (tmp_pos_suffix != std::string::npos)
                        {
                            pos_suffix1 = tmp_pos_suffix;
                            pos_suffix1 += (process_suffix.size()+ 1);
                            element_type = CdmwPlatformMngt::PROCESS_TYPE;
        
                            if ( pos_suffix1 < path_size)
                            {
        
                                tmp_pos_suffix = element_path.find(component_suffix, pos_suffix1);
        
                                //if component name is found ...
                                if (tmp_pos_suffix != std::string::npos)
                                {
                                    pos_suffix1 = tmp_pos_suffix;
                                    pos_suffix1 += component_suffix.size()+1;
                                    element_type = CdmwPlatformMngt::COMPONENT_TYPE;
                                    //return element_type;
                                }
        
                            }
                        }
                        else
                        {
                            // element path is not valid
                            throw CdmwPlatformMngt::ElementPathInvalid();
                        } 
                    }
                    else
                    {
                        // element path is not valid
                        throw CdmwPlatformMngt::ElementPathInvalid();
                    }           
                }
            }
        }
    }
    
    // check if entity name found
    if (pos_suffix1 < path_size)
    { 
        pos_suffix1 = element_path.find(entity_suffix, pos_suffix1);
    
        // if entity name found
        if (pos_suffix1 != std::string::npos)
        {
            element_type = CdmwPlatformMngt::ENTITY_TYPE;
        }
    }
    
    return element_type;
}


CdmwPlatformMngt::ElementType ManagedElementKey::Get_parent_path (
                                   const std::string & element_path,
                                   std::string & element_name,                                         
                                   std::string & parent_path)
    throw (CdmwPlatformMngt::ElementPathInvalid)
{
    CdmwPlatformMngt::ElementType element_type = CdmwPlatformMngt::SYSTEM_TYPE;
    CdmwPlatformMngt::ElementType parent_type = CdmwPlatformMngt::SYSTEM_TYPE;
    
    element_name = "";
    parent_path = "";
    
    std::string::size_type pos_suffix1 = 0;
    std::string::size_type pos_suffix2 = 0;
    
    std::string::size_type pos_elem1 = 0;
    std::string::size_type pos_elem2 = 0;
    std::string::size_type pos_path = 0;
    
    std::string::size_type path_size = element_path.size();

    // search system suffix
    pos_suffix1 = element_path.find(system_suffix);
    
    
    // check if system name found
    if (pos_suffix1 != std::string::npos)
    {
        pos_elem1 = 0;
        pos_elem2 = pos_suffix1;
        pos_path = 0;
    
        pos_suffix1 += (system_suffix.size()+ 1);
        element_type = CdmwPlatformMngt::SYSTEM_TYPE;
        parent_type = CdmwPlatformMngt::SYSTEM_TYPE;
    }
    else
    {
        // element path is not valid
        throw CdmwPlatformMngt::ElementPathInvalid();
    }
    
   
    // check if application name found
    if (pos_suffix1 < path_size)
    { 
        pos_suffix2 = element_path.find(application_suffix, pos_suffix1);
    
        // if application name found
        if (pos_suffix2 != std::string::npos)
        {
            pos_elem1 = pos_suffix1;
            pos_elem2 = pos_suffix2;
            pos_path = pos_suffix1-1;
            
            pos_suffix1 = pos_suffix2 + (application_suffix.size()+ 1);
            
            parent_type = CdmwPlatformMngt::SYSTEM_TYPE;
            element_type = CdmwPlatformMngt::APPLICATION_TYPE;          
        }
    }
    
    // check if host name found
    if (element_type == CdmwPlatformMngt::SYSTEM_TYPE && pos_suffix1 < path_size)
    { 
        pos_suffix2 = element_path.find(host_suffix, pos_suffix1);
    
        // if host name found
        if (pos_suffix2 != std::string::npos)
        {
            pos_elem1 = pos_suffix1;
            pos_elem2 = pos_suffix2;
            pos_path = pos_suffix1-1;
        
            pos_suffix1 = pos_suffix2 + (host_suffix.size()+ 1);
            
            parent_type = CdmwPlatformMngt::SYSTEM_TYPE;
            element_type = CdmwPlatformMngt::HOST_TYPE;         
        }
    }
    
    // check if process name found
    if (element_type == CdmwPlatformMngt::APPLICATION_TYPE && pos_suffix1 < path_size)
    { 


        // first search host name
        pos_suffix2 = element_path.find(host_suffix, pos_suffix1);
    
        // if host name found
        if (pos_suffix2 != std::string::npos)
        {
            pos_path = pos_suffix1-1;
            
            pos_suffix1 = pos_suffix2 + (host_suffix.size()+ 1);
            
            // then search process name
            if (pos_suffix1 < path_size)
            { 
                pos_suffix2 = element_path.find(process_suffix, pos_suffix1);
                
                // if process name found
                if (pos_suffix2 != std::string::npos)
                {
                    // then search component name
                    if (pos_suffix1 < path_size)
                    {
                        unsigned int parent_position = pos_suffix2 + process_suffix.length() ;

                        unsigned int pos_suffix3 = element_path.find(component_suffix, pos_suffix1);
                                 
                        // if component name found
                        if (pos_suffix3 != std::string::npos)
                        {       
                            //component_name = element_path.substr(pos_suffix1, pos_suffix2-pos_suffix1);
                            //then search the entity ...
                            unsigned int parent_position2 = pos_suffix3 + component_suffix.length() ;

                            unsigned int pos_suffix4 = element_path.find(entity_suffix, pos_suffix1);

                            if ( pos_suffix4 != std::string::npos)
                  {
                              element_type = CdmwPlatformMngt::ENTITY_TYPE;
                              parent_path = element_path.substr(0, parent_position2);

                              unsigned int total_len = element_path.size() ;
                              element_name = element_path.substr(parent_position2+1,
                               total_len - parent_position2 - entity_suffix.size() - 1);

                              parent_type = CdmwPlatformMngt::COMPONENT_TYPE;

                              return parent_type;

                  }
                            else
                {

                            element_type = CdmwPlatformMngt::COMPONENT_TYPE;

                            parent_path = element_path.substr(0, parent_position);

                            unsigned int total_len = element_path.size() ;
                            element_name = element_path.substr(parent_position+1,
                             total_len - parent_position - component_suffix.size() - 1);

                            parent_type = CdmwPlatformMngt::PROCESS_TYPE;

    

                            return parent_type;
                }
                        }
                    }       

                    pos_elem1 = pos_suffix1;
                    pos_elem2 = pos_suffix2;
                           
                    pos_suffix1 = pos_suffix2 + (process_suffix.size()+ 1);
                    
                    parent_type = CdmwPlatformMngt::APPLICATION_TYPE;
                    element_type = CdmwPlatformMngt::PROCESS_TYPE;                  
                }
                else
                {
                    // element path is not valid
                    throw CdmwPlatformMngt::ElementPathInvalid();
                }
            }
            else
            {
                // element path is not valid
                throw CdmwPlatformMngt::ElementPathInvalid();
            }           
        }
    }
    
    // check if entity name found
    if (pos_suffix1 < path_size)
    { 
        pos_suffix2 = element_path.find(entity_suffix, pos_suffix1);
    
        // if entity name found
        if (pos_suffix2 != std::string::npos)
        {
            pos_elem1 = pos_suffix1;
            pos_elem2 = pos_suffix2;
            pos_path = pos_suffix1-1;
            
            // set parent type with the elemnt type found previously
            parent_type = element_type;
            element_type = CdmwPlatformMngt::ENTITY_TYPE;
        }
    }


    // check if ftgroup is found ...
    if (pos_suffix1 < path_size)
    { 
        pos_suffix2 = element_path.find(ftgroup_suffix, pos_suffix1);
    
        // if ftgroup name found
        if (pos_suffix2 != std::string::npos)
        {
            pos_elem1 = pos_suffix1;
            pos_elem2 = pos_suffix2;
            pos_path = pos_suffix1-1;
            
            // set parent type with the elemnt type found previously
            parent_type = element_type;
            element_type = CdmwPlatformMngt::FT_GROUP_TYPE;

            pos_suffix2 = element_path.find(entity_suffix, pos_suffix1);

            if (pos_suffix2 != std::string::npos)
              {
              parent_path = element_path.substr(0, pos_elem2 + ftgroup_suffix.length());

              element_name = element_path.substr(pos_elem2+1+ftgroup_suffix.length() , element_path.length() -pos_elem2-1-entity_suffix.length() -ftgroup_suffix.length());
    
              return element_type;
              
              }
          
        }
    }


    // check if lbgroup is found ...
    if (pos_suffix1 < path_size)
    { 
        pos_suffix2 = element_path.find(lbgroup_suffix, pos_suffix1);
    
        // if ftgroup name found
        if (pos_suffix2 != std::string::npos)
        {
            pos_elem1 = pos_suffix1;
            pos_elem2 = pos_suffix2;
            pos_path = pos_suffix1-1;
            
            // set parent type with the elemnt type found previously
            parent_type = element_type;
            element_type = CdmwPlatformMngt::LB_GROUP_TYPE;

            pos_suffix2 = element_path.find(entity_suffix, pos_suffix1);

            if (pos_suffix2 != std::string::npos)
              {
              parent_path = element_path.substr(0, pos_elem2 + ftgroup_suffix.length());

              element_name = element_path.substr(pos_elem2+1+ftgroup_suffix.length() , element_path.length() -pos_elem2-1-entity_suffix.length() -ftgroup_suffix.length());
    
              return element_type;
              
              }

        }
    }
    
    if (element_type == CdmwPlatformMngt::SYSTEM_TYPE)
    {
        // element path is not valid
        throw CdmwPlatformMngt::ElementPathInvalid();        
    }
    
    parent_path = element_path.substr(0, pos_path);
    element_name = element_path.substr(pos_elem1, pos_elem2-pos_elem1);
    
    return parent_type;
}

void ManagedElementKey::Get_process_parent_path (
                                   const std::string & element_path,
                                   std::string & process_name, 
                                   std::string & host_name,                                        
                                   std::string & parent_path)
    throw (CdmwPlatformMngt::ElementPathInvalid)
{
    process_name = "";
    host_name = "";
    parent_path = "";
    
    std::string::size_type pos_suffix1 = 0;
    std::string::size_type pos_suffix2 = 0;
    
    std::string::size_type pos_elem1 = 0;
    std::string::size_type pos_elem2 = 0;
    std::string::size_type pos_elem3 = 0;
    std::string::size_type pos_elem4 = 0;
    std::string::size_type pos_path = 0;
    
    std::string::size_type path_size = element_path.size();
    
    // search system suffix
    pos_suffix1 = element_path.find(system_suffix);
    
    
    // check if system name found
    if (pos_suffix1 != std::string::npos)
    {
        pos_elem1 = 0;
        pos_elem2 = pos_suffix1;
        pos_path = 0;
    
        pos_suffix1 += (system_suffix.size()+ 1);
    }
    else
    {
        // element path is not valid
        throw CdmwPlatformMngt::ElementPathInvalid();
    }
    

    // check if application name found
    if (pos_suffix1 < path_size)
    { 
        pos_suffix2 = element_path.find(application_suffix, pos_suffix1);
    
        // if application name found
        if (pos_suffix2 != std::string::npos)
        {
            pos_elem1 = pos_suffix1;
            pos_elem2 = pos_suffix2;
            pos_path = pos_suffix1-1;
            
            pos_suffix1 = pos_suffix2 + (application_suffix.size()+ 1);         
        }
        else
        {
            // element path is not valid
            throw CdmwPlatformMngt::ElementPathInvalid();
        }
    }
    
    // check if process name found
    
    bool path_valid = false;
    
    if (pos_suffix1 < path_size)
    { 
        // first search host name
        pos_suffix2 = element_path.find(host_suffix, pos_suffix1);
    
        // if host name found
        if (pos_suffix2 != std::string::npos)
        {
            pos_path = pos_suffix1-1;
            
            pos_elem1 = pos_suffix1;
            pos_elem2 = pos_suffix2;
            
            pos_suffix1 = pos_suffix2 + (host_suffix.size()+ 1);
            
            // then search process name
            if (pos_suffix1 < path_size)
            { 
                pos_suffix2 = element_path.find(process_suffix, pos_suffix1);
                
                // if process name found
                if (pos_suffix2 != std::string::npos)
                {
                    pos_elem3 = pos_suffix1;
                    pos_elem4 = pos_suffix2;
                           
                    path_valid = true;                  
                }
            }
        }            
    }
    
    if (path_valid == false)
    {
        // element path is not valid
        throw CdmwPlatformMngt::ElementPathInvalid();
    }
    
    
    parent_path = element_path.substr(0, pos_path);
    host_name = element_path.substr(pos_elem1, pos_elem2-pos_elem1);
    process_name = element_path.substr(pos_elem3, pos_elem4-pos_elem3);
}

}; // End namespace PlatformMngt
}; // End namespace Cdmw

