/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "tools/MessageMapManager.hpp"
#include "tools/AdminLogMessageId.hpp"

#include <map>
#include <string>

using namespace  Cdmw::PlatformAdmin;

namespace 
{

    typedef std::map<AdminLogMessageIds,std::string>::iterator err_msg_iterator;

    std::map<AdminLogMessageIds,std::string> err_msg;

    bool init_messages()
    {
      err_msg[MSG_ID_ADMIN_COMMAND_USAGE            ] = "Wrong command usage";
      err_msg[MSG_ID_ADMIN_LOG_FILE                 ] = "Log written into file \"%s\"";
      err_msg[MSG_ID_ADMIN_UNKNOWN_ERROR            ] = "An unknown error has occured";
      err_msg[MSG_ID_ADMIN_INCOMPATIBLE_TYPE        ] = "Not allowed operation due to incompatible type";
      err_msg[MSG_ID_ADMIN_INCOMPATIBLE_STATUS      ] = "Not allowed operation due to incompatible status";
      err_msg[MSG_ID_ADMIN_RUNSET_NOT_FOUND         ] = "Runset \"%s\" not found";
      err_msg[MSG_ID_ADMIN_HOST_NOT_FOUND           ] = "Host \"%s\" not found";
      err_msg[MSG_ID_ADMIN_APPLICATION_NOT_FOUND    ] = "Application \"%s\" not found";
      err_msg[MSG_ID_ADMIN_APPLICATION_AGENT_NOT_FOUND] = "Application agent \"%s\" not found";
      err_msg[MSG_ID_ADMIN_PROCESS_NOT_FOUND        ] = "Process \"%s\" not found";
      err_msg[MSG_ID_ADMIN_PROCESS_HOST_NOT_FOUND   ] = "The host \"%s\" where the process \"%s\" must run has not been found";
      err_msg[MSG_ID_ADMIN_OBSERVER_NOT_FOUND       ] = "The observer has not been found";
      err_msg[MSG_ID_ADMIN_SERVICE_NOT_FOUND        ] = "The service \"%s\" has not been found";
      err_msg[MSG_ID_ADMIN_ENTITY_NOT_FOUND         ] = "The entity \"%s\" has not been found";
      err_msg[MSG_ID_ADMIN_MONITORING_POINT_NOT_FOUND] = "The monitoring point \"%s\" has not been found";
      err_msg[MSG_ID_ADMIN_HOST_ALREADY_EXISTS      ] = "The host \"%s\" already exists";
      err_msg[MSG_ID_ADMIN_APPLICATION_ALREADY_EXISTS] = "The application \"%s\" already exists";
      err_msg[MSG_ID_ADMIN_APPLICATION_AGENT_ALREADY_EXISTS] = "The application agent \"%s\" already exists";
      err_msg[MSG_ID_ADMIN_PROCESS_ALREADY_EXISTS   ] = "The process \"%s\" already exists";
      err_msg[MSG_ID_ADMIN_ENTITY_ALREADY_EXISTS    ] = "The entity \"%s\" already exists";
      err_msg[MSG_ID_ADMIN_SERVICE_ALREADY_EXISTS   ] = "The service \"%s\" already exists";
      err_msg[MSG_ID_ADMIN_DUPLICATE_HOST           ] = "The host \"%s\" is duplicated among the hosts";
      err_msg[MSG_ID_ADMIN_DUPLICATE_APPLICATION    ] = "The application \"%s\" is duplicated among the applications";
      err_msg[MSG_ID_ADMIN_DUPLICATE_PROCESS        ] = "The process \"%s\"/\"%s\" is duplicated among the processes of the application \"%s\"";
      err_msg[MSG_ID_ADMIN_DUPLICATE_ENTITY         ] = "The entity \"%s\" is duplicated among the entities of the system";
      err_msg[MSG_ID_ADMIN_DUPLICATE_APPLICATION_ENTITY] = "The entity \"%s\" is duplicated among the entities of the application \"%s\"";
      err_msg[MSG_ID_ADMIN_DUPLICATE_PROCESS_ENTITY ] = "The entity \"%s\" is duplicated among the entities of the process \"%s\"/\"%s\"";
      err_msg[MSG_ID_ADMIN_DUPLICATE_SERVICE        ] = "The service \"%s\" is duplicated among the services of the system";
      err_msg[MSG_ID_ADMIN_DUPLICATE_APPLICATION_SERVICE] = "The service \"%s\" is duplicated among the services of the application \"%s\"";
      err_msg[MSG_ID_ADMIN_DUPLICATE_PROCESS_SERVICE] = "The service \"%s\" is duplicated among the services of the process \"%s\"/\"%s\"";
      err_msg[MSG_ID_ADMIN_DUPLICATE_RUNSET         ] = "The run set \"%s\" is duplicated among the runsets";
      err_msg[MSG_ID_ADMIN_DUPLICATE_RUNSET_ELEMENT ] = "The element \"%s\" already exists in the runset or among existing runsets";
      err_msg[MSG_ID_ADMIN_DUPLICATE_STEP           ] = "The step label \"%s\" is duplicated among the steps";
      err_msg[MSG_ID_ADMIN_DUPLICATE_PROCESS_STEP   ] = "The step label \"%s\" is duplicated among the steps of the process \"%s\"/\"%s\"";    
      err_msg[MSG_ID_ADMIN_INVALID_STEP             ] = "No other step to perform";
      err_msg[MSG_ID_ADMIN_NOT_READY_TO_RUN         ] = "The process \"%s\" is not ready to run";
      err_msg[MSG_ID_ADMIN_APPLICATION_STILL_ACTIVE ] = "The application \"%s\" is still active";
      err_msg[MSG_ID_ADMIN_PROCESS_STILL_ACTIVE     ] = "The process \"%s\" is still active";
      err_msg[MSG_ID_ADMIN_INVALID_ENTITY_NAME      ] = "The entity name \"%s\" is invalid";
      err_msg[MSG_ID_ADMIN_INVALID_SERVICE_NAME     ] = "The service name \"%s\" is invalid";
      err_msg[MSG_ID_ADMIN_SERVICE_UNAVAILABLE      ] = "The service \"%s\" is unavailable";
      err_msg[MSG_ID_ADMIN_REFERENCE_NOT_FOUND      ] = "The element \"%s\" doesn't refer to an existing process or application.";
      err_msg[MSG_ID_ADMIN_STEP_OUT_OF_BOUND        ] = "The step %d of the process \"%s\" is out of bound.";
      err_msg[MSG_ID_ADMIN_CIRCULAR_REFERENCE       ] = "The graph contains a circular reference %s-%d.";
      err_msg[MSG_ID_ADMIN_STEP_JUMP                ] = "The process \"%s\" jumps from the step %d to the step %d.";
      err_msg[MSG_ID_ENTITY_INCOMPATIBLE_TAGS       ] = "The entity \"%s\" in xml configuration file; has both tags StatusValues and Type."; 
      err_msg[MSG_ID_ENTITY_INCOMPATIBLE_TYPE       ] = "The entity \"%s\" in xml configuration file; uses an empty Type or nonexistent Type";
      err_msg[MSG_ID_ENTITY_REDEFINES_PROPERTY      ] = "The entity \"%s\" in xml configuration file redefines the property \"%s\" present in Entity type \"%s\"";
      err_msg[MSG_ID_TYPE_ENTITY_EMPTY              ] = "The entity type \"%s\" in xml configuration file; does not have the correct status values ";
      err_msg[MSG_ID_ADMIN_ATTRIBUTE_NOT_FOUND      ] = "The attribute can not be found ";
      err_msg[MSG_ID_ADMIN_END                      ] = "Can not be used, Only for internal scope.";

      return true;
    }

    const bool initialize = init_messages();
}

MessageMapManager::MessageMapManager()
{

}

MessageMapManager::~MessageMapManager()
{
}

std::string MessageMapManager::find(long id) const
{
  err_msg_iterator it = err_msg.find(static_cast<AdminLogMessageIds>(id)),
                   end = err_msg.end();

  std::string error_message = "";
  if(it!=end)
    error_message = it->second;

  return error_message;

}

MessageMapManager* MessageMapManager::instance()
{
  static MessageMapManager singleton;
  return &singleton;
}
