/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include "testsmgtools/TestSuite_ManagedElementKey_cpp.hpp"

#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"

#include "SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp"

#include "SystemMngt/platformlibrary/Configuration.hpp"
#include "SystemMngt/platformlibrary/LogMngr.hpp"

#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>  
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>


using namespace Cdmw;
using namespace PlatformMngt;

class TestLogMngr: public LogMngr
{

     private:
     TestLogMngr(int& argc, char** argv) throw(OutOfMemoryException, InternalErrorException): LogMngr( argc, argv)
     {
     
     }

     public:
     static void init(int& argc, char** argv) 
     {
         if (M_pSingleton == 0)
         {
            try
            {
               // create the concrete Log Manager singleton
               M_pSingleton = new TestLogMngr (argc, argv);
            }
            catch (const std::bad_alloc&)
        {
            CDMW_THROW(OutOfMemoryException);
        }
         }
     }
     
     const char* getService() const
     { 
         return CdmwPlatformMngtBase::PFMMNG;
     }
     
     const char* getMessage(long int) const
     {
         throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
     }
     
};

int main( int argc, char* argv[])
{
    using namespace Cdmw;
    using namespace Cdmw::OsSupport;
    //using namespace Cdmw::PlatformMngt;

    try {
            
        // Initialize the ORB
        Cdmw::OrbSupport::StrategyList orb_strategies;
        orb_strategies.add_OrbThreaded();
        orb_strategies.add_multicast();
        CORBA::ORB_var orb;

        orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, orb_strategies);
        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var poa = PortableServer::POA::_narrow(obj.in());

        PortableServer::POAManager_var poaMgr = poa->the_POAManager();
        poaMgr->activate();

        TestLogMngr::init(argc, argv);
        
        Cdmw::TestUtils::CORBATestManager::instance()->init(orb.in(), poa.in(), argc, argv);
                
        Cdmw::TestUtils::CORBATestManager::instance()->run_tests();
        Cdmw::TestUtils::CORBATestManager::instance()->summary();

        if ( !CORBA::is_nil( poa.in() ) )
        {
            try
            {
                poa->destroy( 1, 1 );
            }
            catch ( const CORBA::Exception &e )
            {
                std::cerr << e << std::endl;
            }
        }

        /*shutdown of orb */
        Cdmw::OrbSupport::OrbSupport::ORB_cleanup (orb.in ());

        // Destroy orb
        if (!CORBA::is_nil (orb.in ()))
        {
            try
            {
                orb->destroy ();
            }
            catch (const CORBA::Exception & ex)
            {
                std::cerr << ex << std::endl;
                exit (-1);
            }
        }

        return ( Cdmw::TestUtils::CORBATestManager::instance()->was_successful() );
    }
    catch (const CORBA::UNKNOWN &e) {
        std::cerr << "CORBA exception: " << e << std::endl;
    }
}

