/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include "Foundation/common/Options.hpp"
#include <Foundation/common/Locations.hpp>
#include "Foundation/common/String.hpp"
#include "Foundation/common/Exception.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/common/Locations.hpp"

#include "Foundation/orbsupport/CORBA.hpp"

#include "Foundation/ossupport/OS.hpp"
#include "Foundation/osthreads/ThreadHandle.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include <Foundation/logging/LogManager.hpp> 
#include <Foundation/logging/FilterMngr.hpp>

#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include "Repository/repositoryinterface/RepositoryInterface.hpp"

#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"

#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/LogManager.hpp>
#include "idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"

#include <cstdlib>
#include <string>
#include <exception>
#include <sstream>
#include <iostream>
#include <fstream>

const int POA_THREAD_POOL_SIZE = 5;

using namespace Cdmw::OsSupport;
using namespace Cdmw::OrbSupport;

std::string applicationName;
std::string processName;
std::string hostName ;

int main(int argc, char* argv[])
{
    // the return code 
    StrategyList strategy;
    strategy.add_OrbThreaded();
    strategy.add_PoaThreadPool(POA_THREAD_POOL_SIZE);

    CORBA::ORB_var orb;


    try
    {
                
        // initialise the orb    
        orb =  Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, strategy);

        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");

        PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj.in());

        PortableServer::POAManager_var poaMgr = rootPOA->the_POAManager();

        poaMgr->activate();

        //resolve system ptr
        CORBA::Object_var objref2; 

        /*define the corbaloc */
        const char *corbaloc_str = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";

        objref2 = orb->string_to_object (corbaloc_str);

        /* resolve the System */
        CdmwPlatformMngt::System_var mysystem = CdmwPlatformMngt::System::_narrow (objref2.in ());

        if (CORBA::is_nil (mysystem.in ()))
          {
            std::cout<<"Error in narrowing CdmwPlatformMngt::System"<<std::endl;
            exit (-1);
          }
 
        CdmwPlatformMngt::Application_var app = mysystem->get_application("APP1");

        std::string proc_name = "P1";
        char *hostname = getenv ("HOSTNAME");
        CdmwPlatformMngt::Process_var proc = app->get_process (proc_name.c_str (), hostname);
        if (CORBA::is_nil (proc.in ())) 
          {
           std::cout<<"*******ERROR in proc_name, is nil *************"<<std::endl;
           exit (-1);
          }

        //CdmwPlatformMngt::Host_var myhost = mysystem->get_host(hostname);


        /* read the following attributes of Process :
        a) Default-automatic-action 
        b) Max_nbr_of_restart_attempt 
        c) Exit-action (Type + Exit-code)   
        d) Auto_Restart_Control
        e) Max_nbr_reached_action
        f) Abnormal_behaviour_action*/


        CdmwPlatformMngt::ExitAutoRestartInfo_var exitAutorestartInfo;
        CdmwPlatformMngt::AutoRestartInfo* autorestartInfo = new CdmwPlatformMngt::AutoRestartInfo ;
        CORBA::Any_var any;


        exitAutorestartInfo = proc->get_exit_autorestart_info();

        try {
            //CdmwPlatformMngt::AutoRestartInfo* autorestartInfo;  
    
            proc->get_attribute (CdmwPlatformMngt::AUTORESTART_INFO, any.out(), CdmwPlatformMngt::PRE_DEFINED);
            if ( (any.in() >>= autorestartInfo ) == false)
            {
                  std::cout<<"Error in any extraction"<<std::endl;
            }
        }
        catch(const CosPropertyService::PropertyNotFound &e)
        {
           std::cerr << "Property not found\n";
        }
        catch(const CosPropertyService::InvalidPropertyName &e)
        {
           std::cerr << "Invalid property name\n";
        }
        catch(const CORBA::SystemException &e)
        {
           std::cerr << "Generic exception\n";
        }

      //print out all the necessary info.

        if ( exitAutorestartInfo->default_exit_action == 1)
        {
            std::cout<<"exitAutorestartInfo->default_exit_action = RESTART_PROCESS"<<std::endl;
        }
        else
        {
          std::cout<<"Error: exitAutorestartInfo->default_exit_action is not RESTART_PROCESS"<<std::endl;
        }

         std::cout<<"AutorestartInfo->max_nbr_of_restart_attempt = "<<autorestartInfo->max_nbr_of_restart_attempt<<std::endl;
     std::cout<<"AutorestartInfo->auto_restart_control = "<<autorestartInfo->auto_restart_control<<std::endl;
         std::cout<<"AutorestartInfo->max_nbr_reached_action = "<<exitAutorestartInfo->max_nbr_reached_action<<std::endl;
         std::cout<<"AutorestartInfo->abnormal_behaviour_action = "<<exitAutorestartInfo->abnormal_behaviour_action<<std::endl;

         int len_seq = (exitAutorestartInfo->exit_actions).length() ;

         int i;

        for (i=0; i<len_seq ; i++)
        {
            std::cout<<"exitAutorestartInfo->exit_actions["<<i<<"].exit_action="<< ( exitAutorestartInfo->exit_actions[i]).exit_action << std::endl; 
       
            std::cout<<"exitAutorestartInfo->exit_actions["<<i<<"].exit_code = "<<(exitAutorestartInfo->exit_actions[i]).exit_code<<std::endl;

        }

        // read Max_nbr_of_restart_attempt of Application ...
        CdmwPlatformMngt::AutoRestartInfo* app_autorestart_info = new CdmwPlatformMngt::AutoRestartInfo ;
        CORBA::Any_var any_app;

        char flag_except = 0;

        try
        {

           app->get_attribute (CdmwPlatformMngt::AUTORESTART_INFO, any_app.out(), CdmwPlatformMngt::PRE_DEFINED);
           if ( (any_app.in() >>= app_autorestart_info) == false) 
          {
                  std::cout<<"Error in any extraction"<<std::endl;
          }
        
       }
       catch(const CosPropertyService::PropertyNotFound &e)
          {
           std::cerr << "Property not found\n";
          }
       catch(const CosPropertyService::InvalidPropertyName &e)
          {
           std::cerr << "Invalid property name\n";
          }
        catch(const CORBA::SystemException &ex)
       {
         std::cout<<"Exception in app->get_attribute (CdmwPlatformMngt::AUTORESTART_INFO, any_app.out(), CdmwPlatformMngt::PRE_DEFINED) : "<<ex<<std::endl;
         flag_except = 1;
      }

    if ( flag_except == 0){
          std::cout<<"Max. num. of restart attempts (of Application) = "<< app_autorestart_info->max_nbr_of_restart_attempt<<std::endl;
    }


        // cleanup the ORB to exit completely
        orb->shutdown(false);
        Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
        orb->destroy();

        return 0;

        //orb->run();

       }
       catch(...)
        {
        std::cout<<"*****ERROR : reached exception in managed process *****"<<std::endl;
            // cleanup the ORB to exit completely
            orb->shutdown(false);
            Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
            orb->destroy();
            
        return(-1);
        }


}

