/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_TESTTUDAANDPDANOTIFICATION_HPP
#define INCL_PLATFORMMNGT_TESTTUDAANDPDANOTIFICATION_HPP

#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/common/System.hpp"
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/Exception.hpp>

#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"
#include "testUDAandPDAComponentsGroupsnotification/SupervisionObserver_impl.hpp"

//

namespace Cdmw
{
  namespace PlatformMngt
  {
    
    class TestUDAandPDAnotification : public CppUnit::TestFixture
    {

    public:

        TestUDAandPDAnotification();
        
        void setUp ();
        
        void oneShotInit ();
        
        //void tearDown ();
        
        virtual void test_register_component_observer_deeply ();
        
        virtual void test_register_component_observer_object_only ();
        
        virtual void test_register_ftgroup_observer_deeply ();
        
        virtual void test_register_ftgroup_observer_object_only ();
        
        virtual void test_register_lbgroup_observer_deeply ();
        
        virtual void test_register_lbgroup_observer_object_only ();
        
        virtual void test_register_system_observer_deeply ();
        
    private:
    
        //System reference
        CdmwPlatformMngt::System_var m_system ;
        CdmwPlatformMngt::SupervisionObserver_var observer;
        std::string obName ;
        CdmwPlatformMngt::Entity_var m_entity;
        SupervisionObserver_impl* sysOb;
        
        //Application references
        //CdmwPlatformMngt::ApplicationsIterator_var appIter;
        CdmwPlatformMngt::Application_var app;
        CORBA::ULong nbApps;
        
        //Process references
        //CdmwPlatformMngt::ProcessesIterator_var procIter;
        CdmwPlatformMngt::Process_var proc;
        CORBA::ULong nbProcs;
        
        //Host references
        CdmwPlatformMngt::Host_var m_host;
        
        //Component references
        CdmwPlatformMngt::CcmComponent_var m_component;

        //RedundancyGroup references
        CdmwPlatformMngt::RedundancyGroup_var m_redu_group;
        CdmwPlatformMngt::FTGroup_var m_ftgroup;
        CdmwPlatformMngt::LBGroup_var m_lbgroup;
        
        CdmwPlatformMngtBase::StartupKind_var startupKind;
        CdmwPlatformMngtBase::StateRetrieval state_retrieval;
        
        CORBA::String_var statusInfo;
        CORBA::ORB_var orb;
        char *hostname;
        
        //Startupkind parameter
        int val;
        unsigned long startup_mode;
        std::string specified_state_id;
        bool oneShotinitdone;

    private:
    
        /**
         * This function set the property and wait for its notification
         * @param pointer : the managed element whose the property should be set
         * @param value   : the value of the property
         * @param name    : the name of the property
         * @param type    : the type of the property
         * @param willBeNotified  : if a notify is expected
         */
        void set_property_and_wait_event(
                CdmwPlatformMngt::ManagedElement* pointer,
                CORBA::Short value,
                char* name,
                const char* type,
                bool willBeNotified)
        throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound);
         
        /**
         * Assignment operator
         */
        TestUDAandPDAnotification & operator= (const TestUDAandPDAnotification & rhs) throw ();
        
        /**
         * Copy constructor
         */
        TestUDAandPDAnotification (const TestUDAandPDAnotification & rhs) throw ();

        /**
         * Defining a Test Suite
         */       
        CPPUNIT_TEST_SUITE (TestUDAandPDAnotification);
        
            //These are the test cases
            CPPUNIT_TEST (test_register_component_observer_deeply);
            CPPUNIT_TEST (test_register_component_observer_object_only);
            CPPUNIT_TEST (test_register_ftgroup_observer_deeply);    
            CPPUNIT_TEST (test_register_ftgroup_observer_object_only);
            CPPUNIT_TEST (test_register_lbgroup_observer_deeply);
            CPPUNIT_TEST (test_register_lbgroup_observer_object_only);
            CPPUNIT_TEST (test_register_system_observer_deeply);
            
        CPPUNIT_TEST_SUITE_END ();
    };

  }             /* End namespace PlatformMngt */
}               /* End namespace Cdmw */
#endif              /* INCL_PLATFORMMNGT_TESTTUDAANDPDANOTIFICATION_HPP */
