/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @author Carlo Giovine  <cgiovine@sesm.it>
 */

#include <iostream>
#include <fstream>
#include <limits>
#include <string>

#include "testUDAandPDAComponentsGroupsnotification/TestUDAandPDAnotification.hpp"

#include "SystemMngt/platformlibrary/Iterator.hpp"

#include "idllib/CdmwPlatformMngtSystem.stub.hpp"


using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestUDAandPDAnotification);

namespace Cdmw
{
  namespace PlatformMngt
  {

    TestUDAandPDAnotification::TestUDAandPDAnotification()
    {
        oneShotinitdone = false;
    }

    void TestUDAandPDAnotification::setUp ()
    {
        if (!oneShotinitdone) oneShotInit ();

    }
    
    void TestUDAandPDAnotification::oneShotInit ()
    {
        try
        {
            oneShotinitdone = true;
            //Getting system var
            std::string system_corbaloc = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";
            orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::instance ()->get_ORB ());
            CORBA::Object_var system_obj = orb->string_to_object (system_corbaloc.c_str());
            m_system = CdmwPlatformMngt::System::_narrow (system_obj.in ());

            //Getting the host
            hostname  = getenv ("HOSTNAME");
            //m_host = m_system->get_host (hostname);
            //Getting the application reference
            app=m_system->get_application("APPL1"); 

            //Getting the Component and the Groups
            try
            {
                m_ftgroup = app->get_ft_group("FtGroup1");
                m_lbgroup = app->get_lb_group("LbGroup1");
                m_component = m_ftgroup->get_component("CompFT1",hostname);

            }catch (const CdmwPlatformMngt::GroupNotFound &e)
            {
                std::cerr<<e<<std::endl;
                CPPUNIT_ASSERT (false);
            }

            //Setting up the startupKind parameters
            state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
            startup_mode = 0;
            specified_state_id = "";
            startupKind = new CdmwPlatformMngtBase::StartupKind();
            startupKind->state_retrieval = state_retrieval;
            startupKind->startup_mode = startup_mode;
            startupKind->specified_state_id  = CORBA::string_dup( specified_state_id.c_str() );

        }
        catch (const CORBA::SystemException& ex) 
        {
            std::cerr<<"\n"<< ex<<std::endl;
            throw;
        }
        catch(...)
        {
        }
    }
    
    void TestUDAandPDAnotification:: test_register_component_observer_deeply()
    {
        try 
        {
            TEST_INFO("Registering the Component Observer with depth FULL_HIERARCHY...");
            obName = "ComponentObserver";

            sysOb= new SupervisionObserver_impl();
            observer = sysOb->_this();
            CdmwPlatformMngt::RegistrationDepth depth=FULL_HIERARCHY;
            
            //Registering the observer
            m_component->register_observer_ext(obName.c_str(), depth, observer.in());
            sleep(1);
            
            std::cerr << "Observer registered on "<< obName.c_str() 
                      << " with depth = " << depth << std::endl;
            
            TEST_INFO("Setting the USER_DEFINED property CompProp1 ... ");
            set_property_and_wait_event(m_component, 32,"CompProp1", "U", true);
            
            m_entity = m_component->get_entity ("Comp1Entity1");
            
            TEST_INFO("Setting the entity USER_DEFINED property PropEntity ... ");
            set_property_and_wait_event(m_entity, 56,"PropEntity", "U", true);
            
            //Unregistering the observers
            m_component->unregister_observer_ext (obName.c_str());
            
            std::cerr << "Unregistering the " << obName.c_str() <<std::endl;
            
            TEST_INFO("Verifying that there are no more notifies ... ");
            TEST_INFO("Setting the entity USER_DEFINED property PropEntity ... ");
            set_property_and_wait_event(m_entity, 32,"PropEntity", "U", false);

            std::cout<<std::endl;
            CPPUNIT_ASSERT(true);
        }
        catch (const CORBA::SystemException& e)
        {
            std::cout<<e<<std::endl;
            CPPUNIT_ASSERT (false);
            throw;
        }
    }
    
    void TestUDAandPDAnotification:: test_register_component_observer_object_only()
    {
        try 
        {
            std::cout<<" "<<std::endl;

            TEST_INFO("Getting the Component...");
            obName = "ComponentObserver";

            TEST_INFO("Registering the Component Observer with depth OBJECT_ONLY...");
            //Registering the observer
            sysOb= new SupervisionObserver_impl();

            observer = sysOb->_this();

            CdmwPlatformMngt::RegistrationDepth depth=OBJECT_ONLY;

            m_component->register_observer_ext(obName.c_str(), depth, observer.in());
            sleep(1);

            std::cerr << "Observer registered on "<< obName.c_str() 
                      << " with depth = " << depth << std::endl;

            TEST_INFO("Setting the USER_DEFINED property CompProp1 ... ");
            set_property_and_wait_event(m_component, 89,"CompProp1", "U", true);

            m_entity = m_component->get_entity ("Comp1Entity1");

            TEST_INFO("Setting the entity USER_DEFINED property PropEntity ... ");
            set_property_and_wait_event(m_entity, 16,"PropEntity", "U", false);
            
            //Unregistering the observers
            m_component->unregister_observer_ext (obName.c_str());
            
            std::cerr << "Unregistering the " << obName.c_str() <<std::endl;
            
            TEST_INFO("Verifying that there are no more notifies ... ");
            TEST_INFO("Setting the USER_DEFINED property CompProp1 ... ");
            set_property_and_wait_event(m_component, 23,"CompProp1", "U", false);

            std::cout<<std::endl;
            CPPUNIT_ASSERT(true);
        }
        catch (const CORBA::SystemException& e)
        {
            std::cout<<e<<std::endl;
            CPPUNIT_ASSERT (false);
            throw;
        }
    }
    
    void TestUDAandPDAnotification:: test_register_ftgroup_observer_deeply()
    {
        try
        {
            TEST_INFO("Registering the FtGroup Observer...");
            obName = "FTGroupGroupObserver";

            //Registering the observer
            sysOb= new SupervisionObserver_impl();
            observer = sysOb->_this();
            CdmwPlatformMngt::RegistrationDepth depth=CdmwPlatformMngt::FULL_HIERARCHY;

            m_ftgroup->register_observer_ext(obName.c_str(), depth, observer.in());
            sleep(1);

            std::cerr << "Observer registered on "<< obName.c_str() 
            << " with depth = " << depth << std::endl;

            try
            {
                TEST_INFO("Setting the USER_DEFINED property FTGroup1Prop ... ");
                set_property_and_wait_event(m_ftgroup, 32,"FTGroup1Prop", "U", true);
            }catch (const CdmwPlatformMngt::AttributeNotFound&)
            {
                std::cerr<<"AttributeNotFound"<<std::endl;
            }

            m_entity = m_ftgroup->get_entity ("FtGroup1Entity");

            TEST_INFO("Setting the entity USER_DEFINED property FTEntityProp ... ");
            set_property_and_wait_event(m_entity, 56,"FTEntityProp", "U", true);
            
            //Unregistering the observers
            std::cerr << "Unregistering the " << obName.c_str() <<std::endl;
            m_ftgroup->unregister_observer_ext (obName.c_str());
            

            TEST_INFO("Verifying that there are no more notifies ... ");
            TEST_INFO("Setting the entity USER_DEFINED property FTEntityProp ... ");
            set_property_and_wait_event(m_entity, 32,"FTEntityProp", "U", false);
            
            std::cout<<std::endl;
            CPPUNIT_ASSERT(true);
        }
        catch (const CORBA::SystemException& e)
        {
            std::cout<<e<<std::endl;
            CPPUNIT_ASSERT (false);
            throw;
        }
  }

    void TestUDAandPDAnotification:: test_register_ftgroup_observer_object_only()
    {
        try 
        {
            TEST_INFO("Registering the FtGroup Observer...");
            obName = "FTGroupGroupObserver";

            //Registering the observer
            sysOb= new SupervisionObserver_impl();
            observer = sysOb->_this();
            CdmwPlatformMngt::RegistrationDepth depth=CdmwPlatformMngt::OBJECT_ONLY;

            m_ftgroup->register_observer_ext(obName.c_str(), depth, observer.in());
            sleep(1);

            std::cerr << "Observer registered on "<< obName.c_str() 
            << " with depth = " << depth << std::endl;

            TEST_INFO("Setting the USER_DEFINED property FTGroup1Prop ... ");
            set_property_and_wait_event(m_ftgroup, 87,"FTGroup1Prop", "U", true);

            m_entity = m_ftgroup->get_entity ("FtGroup1Entity");

            TEST_INFO("Setting the entity USER_DEFINED property FTEntityProp ... ");
            set_property_and_wait_event(m_entity, 1,"FTEntityProp", "U", false);
            
            //Unregistering the observers
            std::cerr << "Unregistering the " << obName.c_str() <<std::endl;
            m_ftgroup->unregister_observer_ext (obName.c_str());
            

            TEST_INFO("Verifying that there are no more notifies ... ");
            TEST_INFO("Setting the USER_DEFINED property FTgroupProp1 ... ");
            set_property_and_wait_event(m_ftgroup, 9,"FTgroupProp1", "U", false);

            std::cout<<std::endl;
            CPPUNIT_ASSERT(true);
        }
        catch (const CORBA::SystemException& e)
        {
            std::cout<<e<<std::endl;
            CPPUNIT_ASSERT (false);
            throw;
        }
    }

    void TestUDAandPDAnotification:: test_register_lbgroup_observer_deeply()
    {
        try 
        {
            TEST_INFO("Registering the LbGroup Observer...");
            obName = "LBGroupGroupObserver";

            //Registering the observer
            sysOb= new SupervisionObserver_impl();
            observer = sysOb->_this();
            CdmwPlatformMngt::RegistrationDepth depth=CdmwPlatformMngt::FULL_HIERARCHY;

            m_lbgroup->register_observer_ext(obName.c_str(), depth, observer.in());
            sleep(1);

            std::cerr << "Observer registered on "<< obName.c_str() 
            << " with depth = " << depth << std::endl;

            TEST_INFO("Setting the USER_DEFINED property LBgroup1Prop ... ");
            set_property_and_wait_event(m_lbgroup, 32,"LBgroup1Prop", "U", true);

            m_entity = m_lbgroup->get_entity ("LbGroup1Entity");

            TEST_INFO("Setting the entity USER_DEFINED property LBEntityProp ... ");
            set_property_and_wait_event(m_entity, 32,"LBEntityProp", "U", true);

            //Unregistering the observers
            m_lbgroup->unregister_observer_ext (obName.c_str());
            
            std::cerr << "Unregistering the " << obName.c_str() <<std::endl;

            TEST_INFO("Verifying that there are no more notifies ... ");
            TEST_INFO("Setting the USER_DEFINED property LBgroupProp1 ... ");
            set_property_and_wait_event(m_lbgroup, 32,"LBgroupProp1", "U", false);

            std::cout<<std::endl;
            CPPUNIT_ASSERT(true);
            
        }
        catch (const CORBA::SystemException& e)
        {
            std::cout<<e<<std::endl;
            CPPUNIT_ASSERT (false);
            throw;
        }
    }

    void TestUDAandPDAnotification:: test_register_lbgroup_observer_object_only()
    {
        try 
        {
            TEST_INFO("Registering the LbGroup Observer...");
            obName = "LBGroupGroupObserver";

            //Registering the observer
            sysOb= new SupervisionObserver_impl();
            observer = sysOb->_this();
            CdmwPlatformMngt::RegistrationDepth depth=CdmwPlatformMngt::OBJECT_ONLY;

            m_lbgroup->register_observer_ext(obName.c_str(), depth, observer.in());
            sleep(1);

            std::cerr << "Observer registered on "<< obName.c_str() 
            << " with depth = " << depth << std::endl;

            TEST_INFO("Setting the USER_DEFINED property LBgroupProp1 ... ");
            set_property_and_wait_event(m_lbgroup, 45,"LBgroupProp1", "U", true);

            m_entity = m_lbgroup->get_entity ("LbGroup1Entity");

            TEST_INFO("Setting the entity USER_DEFINED property LBEntityProp ... ");
            set_property_and_wait_event(m_entity, 36,"LBEntityProp", "U", false);

            //Unregistering the observers
            m_lbgroup->unregister_observer_ext (obName.c_str());
            
            std::cerr << "Unregistering the " << obName.c_str() <<std::endl;

            TEST_INFO("Verifying that there are no more notifies ... ");
            TEST_INFO("Setting the USER_DEFINED property LBgroupProp1 ... ");
            set_property_and_wait_event(m_lbgroup, 78,"LBgroupProp1", "U", false);

            std::cout<<std::endl;
            CPPUNIT_ASSERT(true);

        }
        catch (const CORBA::SystemException& e)
        {
            std::cout<<e<<std::endl;
            CPPUNIT_ASSERT (false);
            throw;
        }
    }
    
    void TestUDAandPDAnotification:: test_register_system_observer_deeply()
    {
        try 
        {
            TEST_INFO("Registering the System Observer...");
            obName = "SystemGroupObserver";

            //Registering the observer
            sysOb= new SupervisionObserver_impl();
            observer = sysOb->_this();
            CdmwPlatformMngt::RegistrationDepth depth=CdmwPlatformMngt::FULL_HIERARCHY;

            m_system->register_observer_ext(obName.c_str(), depth, observer.in());
            sleep(1);

            std::cerr << "Observer registered on "<< obName.c_str() 
            << " with depth = " << depth << std::endl;
            
            TEST_INFO("Setting the USER_DEFINED property LBgroupProp1 ... ");
            set_property_and_wait_event(m_lbgroup, 150,"LBgroupProp1", "U", true);

            m_entity = m_lbgroup->get_entity ("LbGroup1Entity");
            
            TEST_INFO("Setting the entity USER_DEFINED property LBEntityProp ... ");
            set_property_and_wait_event(m_entity, 4,"LBEntityProp", "U", true);

            TEST_INFO("Setting the USER_DEFINED property CompProp1 ... ");
            set_property_and_wait_event(m_component, 6,"CompProp1", "U", true);

            m_entity = m_component->get_entity ("Comp1Entity1");
            
            TEST_INFO("Setting the entity USER_DEFINED property PropEntity ... ");
            set_property_and_wait_event(m_entity, 8,"PropEntity", "U", true);

            TEST_INFO("Setting the USER_DEFINED property FTGroup1Prop ... ");
            set_property_and_wait_event(m_ftgroup, 9,"FTGroup1Prop", "U", true);

            m_entity = m_ftgroup->get_entity ("FtGroup1Entity");
            
            TEST_INFO("Setting the entity USER_DEFINED property FTEntityProp ... ");
            set_property_and_wait_event(m_entity, 45,"FTEntityProp", "U", true);

            //Unregistering the observers
            m_system->unregister_observer_ext (obName.c_str());
            
            std::cerr << "Unregistering the " << obName.c_str() <<std::endl;

            std::cout<<std::endl;
            CPPUNIT_ASSERT(true);
        }
        catch (const CORBA::SystemException& e)
        {
            std::cout<<e<<std::endl;
            CPPUNIT_ASSERT (false);
            throw;
        }
    }
    
    void TestUDAandPDAnotification::set_property_and_wait_event(
            CdmwPlatformMngt::ManagedElement* pointer,
            CORBA::Short value,
            char* name,
            const char* type,
            bool willBeNotified)
    throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound)
    {
        sysOb->set_event( pointer->element_path(), name );
        CORBA::Any_var prop_value = new CORBA::Any() ;
        (*prop_value) <<= value;
        
        try
        {
            pointer->set_attribute(name, (*prop_value),
                    CdmwPlatformMngt::USER_DEFINED );
        }catch (const CdmwPlatformMngt::AttributeNotFound&)
        {
            pointer->define_property(name, (*prop_value));
        }
        
        if (willBeNotified) { TEST_INFO("Waiting for the notifing ... ");}
        else {TEST_INFO("Nothing should be notified ... ");}
        
        bool ret = false;
        int count = 0;
        int sec = 5;
        while ( ((count < sec ) && (!ret)) )
        {
            sleep(1);
            std::cerr <<"Waiting for the PropertyChange notification of "<<name<<std::endl;
            if (sysOb->result()) { ret = true; }
            count ++;
        }
        if ( (count == sec) && (willBeNotified)) {CPPUNIT_ASSERT (false);}
    }
    
    
}/* End namespace PlatformMngt */
}/* End namespace Cdmw */





