/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/*
 * @author Carlo Giovine <cgiovine@sesm.it>
 */

#include <fstream>

#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/common/System.hpp"

#include "SystemMngt/platformlibrary/StatusEventNotifier.hpp"
#include "SystemMngt/platformvaluetypes/SystemStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/SystemModeChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/HostStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationAgentDeath_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationModeChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/EntityStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/GraphElement_impl.hpp"
#include "SystemMngt/platformvaluetypes/ProcessDef_impl.hpp"
#include "SystemMngt/platformvaluetypes/UserEvent_impl.hpp"
#include "SystemMngt/platformvaluetypes/PropertyChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ConfigurationChange_impl.hpp"

#include "testUDAandPDAComponentsGroupsnotification/TestUDAandPDAnotification.hpp"


using namespace std;

class ORBWorker:public Cdmw::OsSupport::Thread
{

public:
    ORBWorker (CORBA::ORB_ptr orb);
    
    virtual ~ORBWorker ()
    throw ();
    
    virtual void run ()
     throw ();
    
private:
    CORBA::ORB_var
    m_ORB;
};

ORBWorker::ORBWorker (CORBA::ORB_ptr orb):m_ORB (CORBA::ORB::_duplicate (orb))
{
}


ORBWorker::~ORBWorker ()throw ()
{
try
{
    m_ORB->shutdown (1);
    m_ORB->destroy ();
    this->join ();
}
catch (const CORBA::SystemException & e)
{
    std::cout << ":>> CORBA system exception caught: " << e._info ().
    rep () << std::endl;
}
catch (const std::exception & e)
{
    std::cout << ":>> std exception caught: " << e.what () << std::endl;
}
catch (...)
{
    std::cout << ":>> unexpected exception caught" << std::endl;
}
}



void
ORBWorker::run ()
throw ()
{
try
{
    this->m_ORB->run ();
}
catch (const CORBA::SystemException & e)
{
    std::cout << ":>> CORBA system exception caught: " << e._info ().
    rep () << std::endl;
}
catch (...)
{
    std::cout << ":>> unexpected exception caught" << std::endl;
}
}

void register_factory (CORBA::ORB_var orb) throw () 
{
    try
    {
        CORBA::ValueFactoryBase_var factory, old_factory;

        factory = new Cdmw::PlatformMngt::SystemModeChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/SystemModeChange:1.0",
                factory.in() );

        factory = new Cdmw::PlatformMngt::SystemStatusChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/SystemStatusChange:1.0",
                factory.in() );

        factory = new Cdmw::PlatformMngt::HostStatusChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/HostStatusChange:1.0",
                factory.in() );

        factory = new Cdmw::PlatformMngt::ApplicationAgentDeathFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationAgentDeath:1.0",
                factory.in() );

        factory = new Cdmw::PlatformMngt::ApplicationModeChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationModeChange:1.0",
                factory.in() );

        factory = new Cdmw::PlatformMngt::ApplicationStatusChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationStatusChange:1.0",
                factory.in() );

        factory = new Cdmw::PlatformMngt::ProcessMessageFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessMessage:1.0",
                factory.in() );

        factory = new Cdmw::PlatformMngt::ProcessStatusChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessStatusChange:1.0",
                factory.in() );

        
        factory = new Cdmw::PlatformMngt::EntityStatusChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/EntityStatusChange:1.0",
                factory.in() );

        factory = new Cdmw::PlatformMngt::UserEventFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/UserEvent:1.0",
                factory.in() );

        factory = new Cdmw::PlatformMngt::PropertyChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/PropertyChange:1.0",
                factory.in() );

    }catch (...)
    {
        CPPUNIT_ASSERT (false);
        throw;
    }
}

void unregister_factory (CORBA::ORB_var orb)
{
    orb->unregister_value_factory (
    "IDL:thalesgroup.com/CdmwPlatformMngt/HostStatusChange:1.0");
    orb->unregister_value_factory (
    "IDL:thalesgroup.com/CdmwPlatformMngt/SystemStatusChange:1.0");
    orb->unregister_value_factory (
    "IDL:thalesgroup.com/CdmwPlatformMngt/SystemModeChange:1.0");
    orb->unregister_value_factory (
    "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationAgentDeath:1.0");
    orb->unregister_value_factory (
    "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationStatusChange:1.0");        
    orb->unregister_value_factory (
    "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationModeChange:1.0");                        
    orb->unregister_value_factory (
    "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessStatusChange:1.0");
    orb->unregister_value_factory (
    "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessMessage:1.0");
    orb->unregister_value_factory (
    "IDL:thalesgroup.com/CdmwPlatformMngt/SystemEntityStatusChange:1.0");
    orb->unregister_value_factory (
    "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationEntityStatusChange:1.0");
    orb->unregister_value_factory (
    "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessEntityStatusChange:1.0");               
    orb->unregister_value_factory (
    "IDL:thalesgroup.com/CdmwPlatformMngt/GraphElement:1.0");
    orb->unregister_value_factory (
    "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDef:1.0");                
    orb->unregister_value_factory (
    "IDL:thalesgroup.com/CdmwPlatformMngt/ManagedProcessDef:1.0");
    orb->unregister_value_factory (
    "IDL:thalesgroup.com/CdmwPlatformMngt/PropertyChange:1.0");
};

int main( int argc, char* argv[])
{
  using namespace Cdmw;
  using namespace Cdmw::OsSupport;
  using namespace Cdmw::PlatformMngt;

  // ===================================================
  // Initialize the ORB
  // ===================================================
  Cdmw::OrbSupport::StrategyList orb_strategies;
  orb_strategies.add_OrbThreaded();
  CORBA::ORB_var orb;

  orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv,orb_strategies);
  
  CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
  PortableServer::POA_var poa = PortableServer::POA::_narrow( obj.in() );

  // Activate the POA manager
  PortableServer::POAManager_var mgr = poa->the_POAManager();
  mgr->activate();
  
  // Registering the factory
  register_factory(orb);
  
  //To let the test has a good start
  sleep(3);
  
  Cdmw::TestUtils::CORBATestManager::instance()->init(orb.in(), argc,argv);
  
  // create an ORBWorker object.
  ORBWorker orb_worker (orb.in ());
  orb_worker.start ();

  // creates the process behaviour
  Cdmw::TestUtils::CORBATestManager::instance()->run_tests();
  Cdmw::TestUtils::CORBATestManager::instance()->summary();
  
  /*// Unregistering the factory
  unregister_factory(orb);
  // Deactivating and destroy
  mgr->deactivate(false, false);
  orb->shutdown(false);
  orb->destroy();*/
  
  return Cdmw::TestUtils::CORBATestManager::instance()->was_successful();
}



