/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <fstream>
#include <limits>

#include "test_Q-022/TestObjs.hpp"
#include "test_Q-022/ProcObserver_impl.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"

#include <SystemMngt/platformlibrary/Iterator.hpp>
#include <SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp>
#include <idllib/CdmwPlatformMngtSystem.stub.hpp>
#include <idllib/CdmwPlatformMngtProcess.stub.hpp>
#include <idllib/CdmwPlatformMngtCommon.stub.hpp>
#include <idllib/CdmwPlatformMngtApplicationAgent.stub.hpp>


#include <SystemMngt/platformvaluetypes/HostStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/ApplicationStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/EntityStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/SystemStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/SystemModeChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/ApplicationModeChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/PropertyChange_impl.hpp>


using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestObjs);

namespace Cdmw
{
  namespace PlatformMngt
  {       

    TestObjs::TestObjs()
    {
        test_result = false;

        system_name = "";
        
        ApplFiveScenario ="Q-022-4-1.xml";

        ApplFiveName = "APPL5";
        ApplFourName = "APPL4";
        ApplThreeName = "APPL3";
        ApplTwoName = "APPL2";
        ApplOneName = "APPL1";

        process_51 = "P51";

        process_41 = "P41";
        process_42 = "P42";

        process_31 = "P31";
        process_32 = "P32";

        process_21 = "P21";
        process_22 = "P22";
        process_23 = "P23";

        process_11 = "P11";
        process_12 = "P12";
        process_13 = "P13";
        process_14 = "P14";

    }

    void TestObjs::setUp ()
    {
       try
       {
           //Getting system var
           std::string system_corbaloc = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";

           orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::instance ()->get_ORB ());

           CORBA::Object_var system_obj = orb->string_to_object (system_corbaloc.c_str());
           system = CdmwPlatformMngt::System::_narrow (system_obj.in ());

           //Setting up the timeouts
           long_timeout  = 4 ; 
           short_timeout = 1 ;

           CORBA::ValueFactoryBase_var factory, old_factory;

           factory = new ApplicationStatusChangeFactory;
           old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationStatusChange:1.0",factory.in() );

           factory = new ProcessStatusChangeFactory;
           old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/ProcessStatusChange:1.0",factory.in() );

           factory = new SystemStatusChangeFactory;
           old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/SystemStatusChange:1.0",factory.in() );

           factory = new HostStatusChangeFactory;
           old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/HostStatusChange:1.0",factory.in() );

           CdmwPlatformMngt::ElementNames_var elems;
           CdmwPlatformMngt::ElementNamesIterator_var it;

           CORBA::ULong nbr_of_host = system->get_number_of_hosts();

           // get the name of hosts
           system->get_all_host_names(nbr_of_host, elems.out(), it.out());

           host_name1 = elems[0U];
           host_name2 = elems[1U];

           host1_name = std::string (host_name1);
           host2_name = std::string (host_name2); 

           CdmwPlatformMngt::SystemSnapshot_var now_snapshot = system->get_snapshot();
           system_name = now_snapshot->system_name;

           TEST_INFO("System Name is: "+system_name);

           observer_sys = observer_on_system._this();

           CdmwPlatformMngt::RegistrationDepth depth =  FULL_HIERARCHY; //OBJECT_ONLY;
           TEST_INFO(" Registered Observers: sys_observer_on_host1"); 

           try 
           {
              system->register_observer_ext ("sys_observer_on_host1",depth, observer_sys.in ());
           //system ->register_observer("sys_observer_on_host1", observer_sys.in ());
           }
           catch (const CdmwPlatformMngt::ElementAlreadyExists& ex_already)
           {
               TEST_INFO(" CdmwPlatformMngt::ElementAlreadyExists caught on observer registering "); 
//               CPPUNIT_ASSERT(false);
           }
           catch (const CORBA::SystemException& exc_sys)
           {
               TEST_INFO(" SystemException Exception caught on observer registering "); 
               CPPUNIT_ASSERT(false);
           }

           ApplFourScenario = "Q-022-2-1.xml";

           ApplFourScenarioBis = "Q-022-3-1.xml";
      }
      catch(...)
      {
           TEST_INFO(" Unknown Exception caught on test Setup "); 
           CPPUNIT_ASSERT(false);
      }

    }

    void TestObjs::test_startup () 
    {

       TEST_INFO(" TEST  CASE 1: Start-up Checking"); 
       /////////////
       // TEST  CASE 1: Start-up Checking
       /////////////

       //TERM2: Applications APPL1, APPL2 & APPL3 take the following successive statuses: INITIALISING, 
       //INITIALISED, RUN_REQUEST and RUNNING.
       //TERM1: They all have the NORMAL mode:
       //system_snapshot.sh
       //TERM2: Their processes take the following successive statuses: 
       //-  P11 INITIALISING (Step0), STEP_PERFORMED (Step0), INITIALISING (Step1), INITIALISED, RUN_REQUEST and RUNNING
       //-  P12 INITIALISING (Step0), INITIALISED, RUN_REQUEST and RUNNING.
       //-  P13 INITIALISING (Step0), STEP_PERFORMED (Step0), INITIALISING (Step1), INITIALISED, RUN_REQUEST and RUNNING
       //-  P14 INITIALISING (Step0), STEP_PERFORMED (Step0), INITIALISING (Step1), INITIALISED, RUN_REQUEST and RUNNING
       //-  P21 (autoending process) INITIALISING (Step0), INITIALISED, RUN_REQUEST, RUNNING and ENDED
       //-  P22 INITIALISING (Step0), INITIALISED, RUN_REQUEST and RUNNING
       //-  P23 INITIALISING (Step0), INITIALISED, RUN_REQUEST and RUNNING
       //-  P31 (unmanaged process) INITIALISING and RUNNING
       //-  P32 INITIALISING (Step0), INITIALISED, RUN_REQUEST and RUNNING
       //Once the system has been started up, it leaves the STARTING status to enter the STARTED status. 
       //Its mode is set to NORMAL. 
       //The initialisation and run sequence must be in accordance with those detailed in sections 3.2.7.1 and 3.2.7.2.

       //Applications stati checking

       TEST_INFO("Started Applications Initializations Testing");

       bool result = true;

       CdmwPlatformMngt::SystemSnapshot_var now_snapshot = system->get_snapshot();

       CdmwPlatformMngt::ApplicationsSnapshot appls_snapshot = now_snapshot->system_applications;

       for (CORBA::ULong index = 0; index < appls_snapshot.length(); index++) 
       {

          CdmwPlatformMngt::ApplicationSnapshot application_snapshot = appls_snapshot [index];

          std::string snap_application_name (application_snapshot.application_name);

          if (ApplOneName ==  snap_application_name ||
              ApplTwoName == snap_application_name ||
              ApplThreeName == snap_application_name) 
          {

              CdmwPlatformMngt::ApplicationStatus application_status = application_snapshot.application_status;

              if (ApplOneName == snap_application_name)
                  TEST_INFO (ApplOneName+" found !");

              if (ApplTwoName ==  snap_application_name)
                  TEST_INFO (ApplTwoName+" found !");

              if (ApplThreeName ==  snap_application_name)
                  TEST_INFO (ApplThreeName+" found !");

              if (!(CdmwPlatformMngt::APPLICATION_INITIALISING == application_status||
                    CdmwPlatformMngt::APPLICATION_INITIALISED == application_status||
                    CdmwPlatformMngt::APPLICATION_RUN_REQUEST == application_status|| 
                    CdmwPlatformMngt::APPLICATION_RUNNING == application_status)) 
              {
                  result = false;
              }
              else
              {

                  CdmwPlatformMngt::ProcessesSnapshot processes_snapshot = 
                      application_snapshot.application_processes;

                  for (CORBA::ULong p_index = 0; p_index < processes_snapshot.length(); p_index++) 
                  {

                     CdmwPlatformMngt::ProcessSnapshot process_snapshot 
                         = processes_snapshot [p_index];

                     std::string snap_process_name (process_snapshot.process_name);

                     if (process_11 == snap_process_name) 
                     {
                         CdmwPlatformMngt::ProcessStatus process_status 
                             = process_snapshot.process_status;

                         TEST_INFO (process_11+" found !");

                          if (!(CdmwPlatformMngt::PROCESS_INITIALISING == process_status || 
                                CdmwPlatformMngt::PROCESS_INITIALISED == process_status || 
                                CdmwPlatformMngt::PROCESS_STEP_PERFORMED == process_status || 
                                CdmwPlatformMngt::PROCESS_RUN_REQUEST == process_status || 
                                CdmwPlatformMngt::PROCESS_RUNNING == process_status)) 
                          {
                               result = false;
                          }
                     }
                     else if (process_12 == snap_process_name) 
                     {
                          CdmwPlatformMngt::ProcessStatus process_status 
                              = process_snapshot.process_status;

                            TEST_INFO (process_12+" found !");

                            if (!(CdmwPlatformMngt::PROCESS_INITIALISING == process_status ||
                                CdmwPlatformMngt::PROCESS_INITIALISED == process_status ||
                                CdmwPlatformMngt::PROCESS_RUN_REQUEST == process_status ||
                                CdmwPlatformMngt::PROCESS_RUNNING == process_status)) 
                            {
                                result = false;
                            }
                     }
                     else if (process_13 == snap_process_name) 
                     {
                          CdmwPlatformMngt::ProcessStatus process_status 
                              = process_snapshot.process_status;

                          TEST_INFO (process_13+" found !");

                          if (!(CdmwPlatformMngt::PROCESS_INITIALISING == process_status || 
                            CdmwPlatformMngt::PROCESS_INITIALISED == process_status || 
                            CdmwPlatformMngt::PROCESS_STEP_PERFORMED == process_status || 
                            CdmwPlatformMngt::PROCESS_RUN_REQUEST == process_status || 
                            CdmwPlatformMngt::PROCESS_RUNNING == process_status)) 
                          {
                             result = false;
                          }
                     }
                     else if (process_14 == snap_process_name) 
                     {
                         CdmwPlatformMngt::ProcessStatus process_status 
                             = process_snapshot.process_status;

                         TEST_INFO (process_14+" found !");

                         if (!(CdmwPlatformMngt::PROCESS_INITIALISING == process_status || 
                           CdmwPlatformMngt::PROCESS_INITIALISED == process_status || 
                           CdmwPlatformMngt::PROCESS_STEP_PERFORMED == process_status || 
                           CdmwPlatformMngt::PROCESS_RUN_REQUEST == process_status || 
                           CdmwPlatformMngt::PROCESS_RUNNING == process_status)) 
                        {
                           result = false;
                        }
                     }
                     else if (process_21 == snap_process_name) 
                     {
                         CdmwPlatformMngt::ProcessStatus process_status 
                             = process_snapshot.process_status;

                         TEST_INFO (process_21+" found !");

                         if (!(CdmwPlatformMngt::PROCESS_INITIALISING == process_status ||
                            CdmwPlatformMngt::PROCESS_INITIALISED == process_status ||
                            CdmwPlatformMngt::PROCESS_RUN_REQUEST == process_status ||
                            CdmwPlatformMngt::PROCESS_RUNNING == process_status ||
                            CdmwPlatformMngt::PROCESS_ENDED == process_status)) 
                         {
                            result = false;
                         }
                     }
                     else if (process_22 == snap_process_name) 
                     {
                         CdmwPlatformMngt::ProcessStatus process_status 
                             = process_snapshot.process_status;

                         TEST_INFO (process_22+" found !");

                         if (!(CdmwPlatformMngt::PROCESS_INITIALISING == process_status ||
                            CdmwPlatformMngt::PROCESS_INITIALISED == process_status ||
                            CdmwPlatformMngt::PROCESS_RUN_REQUEST == process_status ||
                            CdmwPlatformMngt::PROCESS_RUNNING == process_status)) 
                         {
                            result = false;
                         }
                     }
                     else if (process_23 == snap_process_name) 
                     {
                         CdmwPlatformMngt::ProcessStatus process_status 
                             = process_snapshot.process_status;

                         TEST_INFO (process_23+" found !");

                         if (!(CdmwPlatformMngt::PROCESS_INITIALISING == process_status ||
                            CdmwPlatformMngt::PROCESS_INITIALISED == process_status ||
                            CdmwPlatformMngt::PROCESS_RUN_REQUEST == process_status ||
                            CdmwPlatformMngt::PROCESS_RUNNING == process_status)) 
                         {
                            result = false;
                         }
                     }
                     else if (process_31 == snap_process_name) 
                     {
                         CdmwPlatformMngt::ProcessStatus process_status 
                             = process_snapshot.process_status;

                         TEST_INFO (process_31+" found !");

                         if (!(CdmwPlatformMngt::PROCESS_INITIALISING == process_status ||
                             CdmwPlatformMngt::PROCESS_RUNNING == process_status)) 
                         {
                            result = false;
                         }

                     }
                     else if (process_32 == snap_process_name) 
                     {
                         CdmwPlatformMngt::ProcessStatus process_status 
                             = process_snapshot.process_status;

                         TEST_INFO (process_32+" found !");

                         if (!(CdmwPlatformMngt::PROCESS_INITIALISING == process_status ||
                             CdmwPlatformMngt::PROCESS_INITIALISED == process_status ||
                             CdmwPlatformMngt::PROCESS_RUN_REQUEST == process_status ||
                             CdmwPlatformMngt::PROCESS_RUNNING == process_status)) 
                         {
                            result = false;
                         }
                     }
                    }
                  }
              }
       }

       test_result = result;

       TEST_INFO("Started Applications Initializations checked");

       system->unregister_observer_ext ("sys_observer_on_host1");
       CPPUNIT_ASSERT(test_result);

    }

    void TestObjs::test_one_more_application() 
    {
        if (&observer_on_system != 0) 
        {

            TEST_INFO(" TEST  CASE 2: Application 5"); 

            CORBA::ULong nbr_of_host = system->get_number_of_hosts();
            CORBA::ULong nbr_of_apps = system->get_number_of_applications();
     
            if ((nbr_of_host != 2) || (nbr_of_apps!=3))
            {
               TEST_INFO("Invalid scenario");
               CPPUNIT_ASSERT(false);
            }

            /////////////
            // TEST  CASE 2: Application 5
            /////////////

            test_result = false;

            //Test 3

            TEST_INFO("Adding "+ApplFiveName+" Application");

            OsSupport::OS::create_process("platform_admin.sh", "--sys-add-application "+ApplFiveScenario);

            TEST_INFO("Wait for "+ApplFiveName+" Adding ");
            OsSupport::OS::sleep (3000);

            TEST_INFO("Initialising "+ApplFiveName+" Application");

            OsSupport::OS::create_process("platform_admin.sh", "--app-initialise "+ApplFiveName);

            //TERM2: The application APPL5 takes the following successive statuses: APPLICATION_INITIALISATION_INCOMPLETE.
            //TERM2: The process P51 enter the following successive statuses: PROCESS_FAILED_ERROR.

            TEST_INFO("Wait for "+ApplFiveName+" Initialising ");
            OsSupport::OS::sleep (6000);

            TEST_INFO("Checking "+ApplFiveName+" stati sequence");

            CdmwPlatformMngt::SystemSnapshot_var now_snapshot = system->get_snapshot();

            CdmwPlatformMngt::ApplicationsSnapshot appls_snapshot 
                = now_snapshot->system_applications;

            for (CORBA::ULong index = 0; index < appls_snapshot.length(); index++) 
            {

                CdmwPlatformMngt::ApplicationSnapshot application_snapshot 
                    = appls_snapshot [index];

                std::string snap_appl_name (application_snapshot.application_name);

                 if (ApplFiveName == snap_appl_name)
                 {

                     CdmwPlatformMngt::ApplicationStatus application_status 
                         = application_snapshot.application_status;

                     TEST_INFO (ApplFiveName+" found !");

                     if (!(CdmwPlatformMngt::APPLICATION_INITIALISATION_INCOMPLETE == application_status))
                     {
                         TEST_INFO(ApplFiveName+" status is not APPLICATION_INITIALISATION_INCOMPLETE !");
                     }
                     else 
                     {

                         CdmwPlatformMngt::ProcessesSnapshot processes_snapshot 
                             = application_snapshot.application_processes;

                         for (CORBA::ULong p_index = 0; p_index < processes_snapshot.length(); p_index++) 
                         {
                             CdmwPlatformMngt::ProcessSnapshot process_snapshot 
                                  = processes_snapshot [p_index];

                             std::string snap_process_name (process_snapshot.process_name);

                             TEST_INFO("Checking process "+process_51+" stati sequence");

                             if (process_51 == snap_process_name)
                             {
                                 TEST_INFO(process_51+" process found !");

                                 CdmwPlatformMngt::ProcessStatus process_status 
                                    = process_snapshot.process_status;

                                 if (CdmwPlatformMngt::PROCESS_FAILED_ERROR 
                                     == process_status) 
                                 {
                                     test_result = true;
                                 }
                             }
                         }
                  }
              }
       }

           CPPUNIT_ASSERT(test_result);

            TEST_INFO("Stopping "+ApplFiveName);

            OsSupport::OS::create_process("platform_admin.sh", "--app-stop "+ApplFiveName);

            TEST_INFO("Wait for "+ApplFiveName+" Stopping ");
            OsSupport::OS::sleep (15000);

            TEST_INFO("Wait for "+ApplFiveName+" Stopping ");
            OsSupport::OS::sleep (15000);

            TEST_INFO("Removing "+ApplFiveName); 

            OsSupport::OS::create_process("platform_admin.sh", "--sys-remove-application "+ApplFiveName);

            for (int index = 0; index < 3; index++)
            {
                TEST_INFO("Wait for "+ApplFiveName+" Removing ");
                OsSupport::OS::sleep (15000);
            }

            system->unregister_observer_ext ("sys_observer_on_host1");
            system->acknowledge();
    }

        CPPUNIT_ASSERT(test_result);
    }

    void TestObjs::test_another_application() {

        test_result = false;

        if (&observer_on_system != 0) 
        {
              TEST_INFO(" TEST  CASE 3: Application 4 ");

              /////////////
              // TEST  CASE 3: Application 4
              /////////////

              //Test 4
              TEST_INFO("Adding "+ApplFourName+" application ");

              OsSupport::OS::create_process("platform_admin.sh", "--sys-add-application "+ApplFourScenario);

              TEST_INFO("Wait for "+ApplFourName+" Adding ");
              OsSupport::OS::sleep (3000);

              TEST_INFO("Initialising "+ApplFourName+" application ");

              OsSupport::OS::create_process("platform_admin.sh", "--app-initialise "+ApplFourName);

              //TERM2: The application APPL4 takes the following successive statuses: INITIALISING, INITIALISED.
              //TERM2: The process P41 enter the following successive statuses: INITIALISING (Step0), INITIALISED.
/*
              std::vector<CdmwPlatformMngt::ApplicationStatus> application_four_stati;

              application_four_stati.push_back(CdmwPlatformMngt::APPLICATION_INITIALISING);
              application_four_stati.push_back(CdmwPlatformMngt::APPLICATION_INITIALISED);

              std::vector<CdmwPlatformMngt::ProcessStatus> process_fourone_stati;

              process_fourone_stati.push_back(CdmwPlatformMngt::PROCESS_INITIALISING);
              process_fourone_stati.push_back(CdmwPlatformMngt::PROCESS_INITIALISED);
*/
              TEST_INFO("Wait for "+ApplFourName+" Initialising ");
              OsSupport::OS::sleep (3000);

              TEST_INFO("Wait for "+ApplFourName+" Initialising ");
              OsSupport::OS::sleep (3000);

              TEST_INFO("Checking "+ApplFourName+" stati sequence ");

              CdmwPlatformMngt::Application_var appl4 = system->get_application(ApplFourName.c_str());

              CdmwPlatformMngt::ApplicationSnapshot_var appl4Snap = appl4->get_snapshot();

              if (!(CdmwPlatformMngt::APPLICATION_INITIALISING == appl4Snap->application_status 
                 || CdmwPlatformMngt::APPLICATION_INITIALISED == appl4Snap->application_status) )
              {
                  CPPUNIT_ASSERT(false);
              }
/*
              test_result = observer_on_system.check_appl_stati_sequence (system_name, 
                        ApplFourName, application_four_stati);
              CPPUNIT_ASSERT(test_result);
*/
              TEST_INFO("Checking process "+process_41+" stati sequence ");

              CdmwPlatformMngt::ProcessesSnapshot procsSnapshots = appl4Snap->application_processes;

              for (CORBA::ULong p_index = 0; p_index < procsSnapshots.length(); p_index++) 
              {
                   CdmwPlatformMngt::ProcessSnapshot process_snapshot 
                        = procsSnapshots [p_index];

                   std::string snap_process_name (process_snapshot.process_name);

                   if (process_41 == snap_process_name)
                   {
                        TEST_INFO(process_41+" process found !");

                        CdmwPlatformMngt::ProcessStatus process_status 
                               = process_snapshot.process_status;

                        if (CdmwPlatformMngt::PROCESS_INITIALISING 
                                  == process_status ||
                            CdmwPlatformMngt::PROCESS_INITIALISED
                                  == process_status) 
                        {
                            test_result = true;
                        }
                   }
               }

/*
              test_result = test_result && 
                         observer_on_system.check_proc_stati_sequence(system_name, 
                         ApplFourName, host1_name ,process_41, process_fourone_stati);
*/
              CPPUNIT_ASSERT(test_result);
              test_result = false;

              TEST_INFO("Running "+ApplFourName+" application ");

              OsSupport::OS::create_process("platform_admin.sh", "--app-run "+ApplFourName);

              for (int index = 0; index < 5; index++)
              {
                  TEST_INFO("Wait for "+ApplFourName+" Running ");
                  OsSupport::OS::sleep (3000);
              }

              //TERM2: The application APPL4 takes the following successive statuses: RUN_REQUEST, RUNNING
              //TERM2: The P41 process enter the following successive statuses: RUN_REQUEST, RUNNING
/*
              std::vector<CdmwPlatformMngt::ApplicationStatus> application_four_run_stati;

              application_four_run_stati.push_back(CdmwPlatformMngt::APPLICATION_RUN_REQUEST);
              application_four_run_stati.push_back(CdmwPlatformMngt::APPLICATION_RUNNING);

              std::vector<CdmwPlatformMngt::ProcessStatus> process_fourone_run_stati;

              process_fourone_run_stati.push_back(CdmwPlatformMngt::PROCESS_RUN_REQUEST);
              process_fourone_run_stati.push_back(CdmwPlatformMngt::PROCESS_RUNNING);
*/
              TEST_INFO("Ckecking "+ApplFourName+" application running stati sequence ");

              appl4Snap = appl4->get_snapshot();

              if (!(CdmwPlatformMngt::APPLICATION_RUN_REQUEST == appl4Snap->application_status 
                 || CdmwPlatformMngt::APPLICATION_RUNNING == appl4Snap->application_status) )
              {
                  CPPUNIT_ASSERT(false);
              }
/*
              OsSupport::OS::sleep (3000);

              test_result = test_result && 
                 observer_on_system.check_appl_stati_sequence (system_name, ApplFourName, application_four_run_stati);

              CPPUNIT_ASSERT(test_result);
*/
              TEST_INFO("Checking "+process_41+" process running stati sequence ");
/*
              test_result = test_result && observer_on_system.check_proc_stati_sequence(system_name, ApplFourName, host1_name,
                   process_41, process_fourone_run_stati);
*/
              procsSnapshots = appl4Snap->application_processes;

              for (CORBA::ULong p_index = 0; p_index < procsSnapshots.length(); p_index++) 
              {
                   CdmwPlatformMngt::ProcessSnapshot process_snapshot 
                        = procsSnapshots [p_index];

                   std::string snap_process_name (process_snapshot.process_name);

                   if (process_41 == snap_process_name)
                   {
                        TEST_INFO(process_41+" process found !");

                        CdmwPlatformMngt::ProcessStatus process_status 
                               = process_snapshot.process_status;

                        if (CdmwPlatformMngt::PROCESS_RUN_REQUEST 
                                  == process_status ||
                            CdmwPlatformMngt::PROCESS_RUNNING
                                  == process_status) 
                        {
                            test_result = true;
                        }
                   }
               }

              CPPUNIT_ASSERT(test_result);
              test_result = false;

              //Test 5

              TEST_INFO("Adding "+process_42+" process to "+ApplFourName+" application ");

              OsSupport::OS::create_process("platform_admin.sh", 
              "--app-add-process "+ApplFourName+" "+ApplFourScenarioBis);

              TEST_INFO(" Wait for "+process_42+" Adding ");
              OsSupport::OS::sleep (3000);

              TEST_INFO("Initialising "+process_42+" process ");

              OsSupport::OS::create_process("platform_admin.sh", 
                 "--proc-initialise "+ApplFourName+" "+process_42+" "+host1_name);

              //TERM2: The process P42 enter the following successive statuses: INITIALISING (Step0), INITIALISED.
/*
              std::vector<CdmwPlatformMngt::ProcessStatus> process_fourtwo_stati; 

              process_fourtwo_stati.push_back(CdmwPlatformMngt::PROCESS_INITIALISING);
              process_fourtwo_stati.push_back(CdmwPlatformMngt::PROCESS_INITIALISED);
*/
              TEST_INFO("Checking "+process_42+" process initialization stati sequence ");

              TEST_INFO(" Wait for "+process_42+" Initialising ");
              OsSupport::OS::sleep (3000);
/*
              test_result = test_result && 
                 observer_on_system.check_proc_stati_sequence(system_name, ApplFourName, 
                   host1_name, process_42, process_fourtwo_stati);
*/
              appl4Snap = appl4->get_snapshot();

              procsSnapshots = appl4Snap->application_processes;

              for (CORBA::ULong p_index = 0; p_index < procsSnapshots.length(); p_index++) 
              {
                   CdmwPlatformMngt::ProcessSnapshot process_snapshot 
                        = procsSnapshots [p_index];

                   std::string snap_process_name (process_snapshot.process_name);

                   if (process_42 == snap_process_name)
                   {
                        TEST_INFO(process_42+" process found !");

                        CdmwPlatformMngt::ProcessStatus process_status 
                               = process_snapshot.process_status;

                        if (CdmwPlatformMngt::PROCESS_INITIALISING 
                                  == process_status ||
                            CdmwPlatformMngt::PROCESS_INITIALISED
                                  == process_status) 
                        {
                            test_result = true;
                        }
                   }
               }

              CPPUNIT_ASSERT(test_result);
              test_result = false;

              TEST_INFO("Running "+process_42+" process ");

              OsSupport::OS::create_process("platform_admin.sh", "--proc-run "+ApplFourName+" "+process_42+" "+host1_name);

              TEST_INFO(" Wait for "+process_42+" Running ");
              OsSupport::OS::sleep (3000);

              //TERM2: The P42 process enters the following successive statuses: RUN_REQUEST, RUNNING. 
              //The APPL4 status remains unchanged
              TEST_INFO ("Checking: The APPL4 status remains unchanged");

              appl4Snap = appl4->get_snapshot();

              if (!(CdmwPlatformMngt::APPLICATION_RUNNING == appl4Snap->application_status) )
              {
                  CPPUNIT_ASSERT(false);
              }

              TEST_INFO("Checking "+process_42+" process running stati sequence ");

              procsSnapshots = appl4Snap->application_processes;

              for (CORBA::ULong p_index = 0; p_index < procsSnapshots.length(); p_index++) 
              {
                   CdmwPlatformMngt::ProcessSnapshot process_snapshot 
                        = procsSnapshots [p_index];

                   std::string snap_process_name (process_snapshot.process_name);

                   if (process_42 == snap_process_name)
                   {
                        TEST_INFO(process_42+" process found !");

                        CdmwPlatformMngt::ProcessStatus process_status 
                               = process_snapshot.process_status;

                        if (CdmwPlatformMngt::PROCESS_RUN_REQUEST 
                                  == process_status ||
                            CdmwPlatformMngt::PROCESS_RUNNING
                                  == process_status) 
                        {
                            test_result = true;
                        }
                   }
               }
/*
              test_result = test_result && observer_on_system.check_proc_stati_sequence(system_name, ApplFourName, host1_name,
                  process_42, process_fourone_run_stati);
*/
        TEST_INFO(" Unregistering observer ");
        system->unregister_observer_ext ("sys_observer_on_host1");
        }

        CPPUNIT_ASSERT(test_result);
    } 

    void TestObjs::test_emergencies() {

        if (&observer_on_system != 0) 
        {

            TEST_INFO(" TEST CASE 4: Emergency stops");
            /////////////
            // TEST  CASE 4: Emergency stops
            /////////////

            //Test 6
            TEST_INFO(" Stopping "+process_32+" process "); 

            OsSupport::OS::create_process("platform_admin.sh", "--proc-stop "+ApplThreeName+" "
                 +process_32+" "+host2_name);

            //TERM2: The P32 process enter the following successive statuses: STOPPING and STOPPED
/*
            std::vector<CdmwPlatformMngt::ProcessStatus> process_threetwo_stop_stati;

            process_threetwo_stop_stati.push_back(CdmwPlatformMngt::PROCESS_STOPPING);
            process_threetwo_stop_stati.push_back(CdmwPlatformMngt::PROCESS_STOPPED);
*/
            TEST_INFO(" Wait for "+process_32+" Stopping "); 
            OsSupport::OS::sleep (3000);

/*
            test_result = observer_on_system.check_proc_stati_sequence (system_name,
                ApplThreeName, host2_name, process_32, process_threetwo_stop_stati);

            CPPUNIT_ASSERT(test_result);
*/
            //TERM1: Check by a read command that mode of the APPL3 application remains unchanged (NORMAL_MODE).

            CdmwPlatformMngt::ApplicationsSnapshot appl_snap = (system->get_snapshot())->system_applications;
            CdmwPlatformMngt::ApplicationSnapshot found_application_three;

            for (CORBA::ULong long_index = 0U; long_index <appl_snap.length(); long_index++) 
            {
                CdmwPlatformMngt::ApplicationSnapshot application = appl_snap[long_index];

                std::string appl_name (application.application_name);

                if (ApplThreeName == appl_name)
                {
                    found_application_three = application;
                }
            }

            TEST_INFO(" Checking "+process_32+" stopping stati sequence "); 

            CdmwPlatformMngt::ProcessesSnapshot snap_processes = found_application_three.application_processes;

            for (CORBA::ULong long_index = 0U; long_index <snap_processes.length(); long_index++)
            {
                 CdmwPlatformMngt::ProcessSnapshot process = snap_processes[long_index];

                 std::string process_name (process.process_name);

                 if (process_name == process_32)
                 {
                     if (!(CdmwPlatformMngt::PROCESS_STOPPING == process.process_status
                        || CdmwPlatformMngt::PROCESS_STOPPED == process.process_status))
                     {
                        CPPUNIT_ASSERT(false);
                     }
                 }

            }

            TEST_INFO(" Checking "+ApplThreeName+" application executing mode "); 

            CPPUNIT_ASSERT(CdmwPlatformMngt::NORMAL_MODE == found_application_three.application_mode);

            //Test 7 
            TEST_INFO(" Emergency "+process_41+" process stop "); 

            OsSupport::OS::create_process("platform_admin.sh", "--proc-stop "+ApplFourName+" "+process_41+" "
                +host1_name+" EMERGENCY");

            //TERM2: The P41 goes directly to the STOPPED status.
/*
            std::vector<CdmwPlatformMngt::ProcessStatus> process_fourone_stop_status;
            process_fourone_stop_status.push_back(CdmwPlatformMngt::PROCESS_STOPPED);
*/
            TEST_INFO(" Wait "+process_41+" stopping ");
            OsSupport::OS::sleep (3000);

            TEST_INFO(" Checking current "+process_41+" process status"); 
/*
            test_result = test_result && observer_on_system.check_proc_stati_sequence (system_name, ApplFourName, host1_name, 
                process_41, process_fourone_stop_status);

            CPPUNIT_ASSERT(test_result);
*/
           //TERM1: Check by a read command that mode and status of the APPL4 application remains unchanged (NORMAL_MODE).

           appl_snap = (system->get_snapshot())->system_applications;
           CdmwPlatformMngt::ApplicationSnapshot found_application_four;

           for (CORBA::ULong long_index = 0U; long_index <appl_snap.length(); long_index++) 
           {
                ApplicationSnapshot application = appl_snap[long_index];

                if (ApplFourName.compare(application.application_name) == 0)
                {
                    found_application_four = application;
                }
           }

           snap_processes = found_application_four.application_processes;

            for (CORBA::ULong long_index = 0U; long_index <snap_processes.length(); long_index++)
            {
                 CdmwPlatformMngt::ProcessSnapshot process = snap_processes[long_index];

                 std::string process_name (process.process_name);

                 if (process_name == process_41)
                 {
                     if (!(CdmwPlatformMngt::PROCESS_STOPPED == process.process_status))
                     {
                        CPPUNIT_ASSERT(false);
                     }
                 }

            }

           TEST_INFO("Check by a read command that mode of the "+ApplFourName+" application remains unchanged"); 

//           test_result = test_result && (found_application_four.application_mode==CdmwPlatformMngt::NORMAL_MODE);

            CPPUNIT_ASSERT(found_application_four.application_mode==CdmwPlatformMngt::NORMAL_MODE);

           //Test 8
           TEST_INFO(" Stopping "+ApplFourName+" application"); 

           OsSupport::OS::create_process("platform_admin.sh", "--app-stop "+ApplFourName);

           //TERM2: The application APPL4 takes the following successive statuses: STOPPING and STOPPED. 
           //TERM2: The process P42 takes the following successive statuses: STOPPING and STOPPED       

           OsSupport::OS::sleep (3000);

           TEST_INFO("Checking "+process_42+" process stop stati sequence "); 
/*
           test_result = test_result && observer_on_system.check_proc_stati_sequence (system_name,
               ApplFourName, host1_name, process_42, process_threetwo_stop_stati);

            CPPUNIT_ASSERT(test_result);

           std::vector<CdmwPlatformMngt::ApplicationStatus> application_four_stop_stati;

           application_four_stop_stati.push_back(CdmwPlatformMngt::APPLICATION_STOPPING);
           application_four_stop_stati.push_back(CdmwPlatformMngt::APPLICATION_STOPPED);
*/
           appl_snap = (system->get_snapshot())->system_applications;

           for (CORBA::ULong long_index = 0U; long_index <appl_snap.length(); long_index++) 
           {
                ApplicationSnapshot application = appl_snap[long_index];

                if (ApplFourName.compare(application.application_name) == 0)
                {
                    found_application_four = application;
                }
           }

           snap_processes = found_application_four.application_processes;

            for (CORBA::ULong long_index = 0U; long_index <snap_processes.length(); long_index++)
            {
                 CdmwPlatformMngt::ProcessSnapshot process = snap_processes[long_index];

                 std::string process_name (process.process_name);

                 if (process_name == process_42)
                 {
                     if (!(CdmwPlatformMngt::PROCESS_STOPPING == process.process_status ||
                         CdmwPlatformMngt::PROCESS_STOPPED == process.process_status))
                     {
                        CPPUNIT_ASSERT(false);
                     }
                 }

            }

           TEST_INFO("Checking "+ApplFourName+" application stop stati sequence "); 

/*
           test_result = test_result && observer_on_system.check_appl_stati_sequence (system_name, ApplFourName,
               application_four_stop_stati);

           CPPUNIT_ASSERT(test_result);

            test_result = test_result && ( CdmwPlatformMngt::APPLICATION_STOPPING == found_application_four.application_status
                || CdmwPlatformMngt::APPLICATION_STOPPED == found_application_four.application_status);

            CPPUNIT_ASSERT(test_result);
*/
           CPPUNIT_ASSERT(( CdmwPlatformMngt::APPLICATION_STOPPING == found_application_four.application_status
                || CdmwPlatformMngt::APPLICATION_STOPPED == found_application_four.application_status));

           //TERM1: Check by a read command that the system status and mode remain unchanged (SYSTEM_STARTED & NORMAL_MODE)
           //TERM1: To change to Check by a read command that the system status and mode remain unchanged (SYSTEM_PARTIAL_STARTED & NORMAL_MODE)
           CdmwPlatformMngt::SystemSnapshot_var now_snapshot = system->get_snapshot();

           CdmwPlatformMngt::SystemStatus now_system_status = now_snapshot->system_status;
           CdmwPlatformMngt::FunctioningMode now_system_mode = now_snapshot->system_mode;

           bool snap_result = (now_system_mode==CdmwPlatformMngt::NORMAL_MODE) && 
               (now_system_status==CdmwPlatformMngt::SYSTEM_PARTIAL_STARTED);

           TEST_INFO("Checking System status and mode "); 

           CPPUNIT_ASSERT(snap_result);
           test_result = true;

           //Test 9
           TEST_INFO("Emergency stop of "+ApplThreeName+" application"); 

           OsSupport::OS::create_process("platform_admin.sh", "--app-stop "+ApplThreeName+" EMERGENCY");

           //TERM2: The application APPL3 takes the following successive statuses: STOPPING and STOPPED
           for (int index = 0; index < 3; index++)
           {
                 TEST_INFO("Wait for "+ApplThreeName+" Stopping "); 
                 OsSupport::OS::sleep (10000);
           }

           TEST_INFO("Checking "+ApplThreeName+" application emergency stop stati sequence"); 
/*
           test_result = test_result && observer_on_system.check_appl_stati_sequence (system_name, 
               ApplThreeName, application_four_stop_stati);
*/
           appl_snap = (system->get_snapshot())->system_applications;

           for (CORBA::ULong long_index = 0U; long_index <appl_snap.length(); long_index++) 
           {
                ApplicationSnapshot application = appl_snap[long_index];

                std::string appl_name (application.application_name);

                if (ApplThreeName == appl_name)
                {
                    if (!CdmwPlatformMngt::APPLICATION_STOPPED == application.application_status)
                    {
                        test_result = false;
                    }
                }
           }

           CPPUNIT_ASSERT(test_result);

           //TERM1: Check by a read command that the system status and mode remain unchanged (SYSTEM_STARTED & NORMAL_MODE)
           //TERM1: To Change to Check by a read command that the system status and mode remain unchanged (SYSTEM_PARTIAL_STARTED & NORMAL_MODE)
           now_snapshot = system->get_snapshot();
   
           now_system_status = now_snapshot->system_status;
           now_system_mode = now_snapshot->system_mode;

           snap_result = now_system_mode==CdmwPlatformMngt::NORMAL_MODE && 
               now_system_status==CdmwPlatformMngt::SYSTEM_PARTIAL_STARTED;

           TEST_INFO("Checking System mode and status "); 

           // test_result = test_result && snap_result;
           CPPUNIT_ASSERT (snap_result);

//           observer_on_system.app_changes_queue_flush ();
//           observer_on_system.pro_changes_queue_flush ();

//           system->unregister_observer_ext ("sys_observer_on_host1");
        }

//        CPPUNIT_ASSERT(test_result);
    }

    void TestObjs::test_stopping_and_finishing() {

         if (&observer_on_system != 0) 
         {
         TEST_INFO(" TEST  CASE 5: Stopping and Re-booting ");

         /////////////
         // TEST  CASE 5: Stopping and Re-booting
         /////////////

         //Test 10:
         TEST_INFO("Stopping the system ");

         OsSupport::OS::create_process("stop_system.sh", "");

         //TERM2: All the running applications (APPL1 & APPL2) take the following successive statuses: STOPPING and STOPPED
         //TERM2: Their processes take the following successive statuses: STOPPING and STOPPED
         //TERM2: The system status takes the following successive statuses: STOPPING and STOPPED 
         //while its mode remains unchanged (NORMAL)

         //Check applications status
         for (int index = 0; index < 3; index++)
         {
             TEST_INFO(" Wait for system stopping ");
             OsSupport::OS::sleep (10000);
         }

         CdmwPlatformMngt::SystemSnapshot_var now_snapshot = system->get_snapshot();
         CdmwPlatformMngt::ApplicationsSnapshot appls_snapshot = now_snapshot->system_applications;

         TEST_INFO("Checking Applications status");

         for (CORBA::ULong long_index = 0U; long_index <appls_snapshot.length(); long_index++)
         {
             CdmwPlatformMngt::ApplicationSnapshot application = appls_snapshot[long_index];

             if (! (CdmwPlatformMngt::APPLICATION_STOPPED == application.application_status))
             {
                std::string appl_name (application.application_name);
                TEST_INFO (" Application "+appl_name+" status is not APPLICATION_STOPPED ");
                CPPUNIT_ASSERT(false);
             }
             else
             {
                 CdmwPlatformMngt::ProcessesSnapshot processes 
                    = application.application_processes;

                 TEST_INFO("Checking processes status ");

                 for (CORBA::ULong p_long_index = 0U; 
                     p_long_index <processes.length(); p_long_index++)
                 {
                     CdmwPlatformMngt::ProcessSnapshot process = processes[p_long_index];

                     if (! (CdmwPlatformMngt::PROCESS_STOPPED == process.process_status 
                        || CdmwPlatformMngt::PROCESS_ENDED == process.process_status))
                     {
                         std::string proc_name (process.process_name);
                         TEST_INFO (" Process "+proc_name+" status is not PROCESS_STOPPED neither PROCESS_ENDED ");
                         CPPUNIT_ASSERT (false);
                     }

                 }
             }

         }

         std::vector<CdmwPlatformMngt::ApplicationStatus> applications_stati;
/*
         applications_stati.push_back(CdmwPlatformMngt::APPLICATION_STOPPING);
         applications_stati.push_back(CdmwPlatformMngt::APPLICATION_STOPPED);

         TEST_INFO("Checking Applications status");

         test_result = observer_on_system.check_appl_stati_sequence (system_name, 
             ApplOneName, applications_stati);

         CPPUNIT_ASSERT(test_result);

         test_result = test_result && observer_on_system.check_appl_stati_sequence (system_name, 
             ApplTwoName, applications_stati);

         CPPUNIT_ASSERT(test_result);

         //Check processes stati 

         std::vector<CdmwPlatformMngt::ProcessStatus> processes_stati;

         processes_stati.push_back(CdmwPlatformMngt::PROCESS_STOPPING);
         processes_stati.push_back(CdmwPlatformMngt::PROCESS_STOPPED);

         TEST_INFO("Checking processes status ");

         TEST_INFO("Process: P11 ");

         test_result = test_result && observer_on_system.check_proc_stati_sequence (system_name,
             ApplOneName, host1_name, process_11, processes_stati);

         CPPUNIT_ASSERT(test_result);

         TEST_INFO("Process: P12 ");

         test_result = test_result && observer_on_system.check_proc_stati_sequence (system_name,
             ApplOneName, host1_name, process_12, processes_stati);

         CPPUNIT_ASSERT(test_result);

         TEST_INFO("Process: P13 ");

         test_result = test_result && observer_on_system.check_proc_stati_sequence (system_name,
             ApplOneName, host2_name, process_13, processes_stati);

         CPPUNIT_ASSERT(test_result);

         TEST_INFO("Process: P14 ");

         test_result = test_result && observer_on_system.check_proc_stati_sequence (system_name,
             ApplOneName, host2_name, process_14, processes_stati);

         CPPUNIT_ASSERT(test_result);

         TEST_INFO("Process: P22 ");

         test_result = test_result && observer_on_system.check_proc_stati_sequence (system_name,
             ApplTwoName, host1_name, process_22, processes_stati);

         CPPUNIT_ASSERT(test_result);

         TEST_INFO("Process: P23 ");

         test_result = test_result && observer_on_system.check_proc_stati_sequence (system_name,
             ApplTwoName, host1_name, process_23, processes_stati);

         CPPUNIT_ASSERT(test_result);
*/
         //Check system stati   
/*
         std::vector<CdmwPlatformMngt::SystemStatus> system_stop_stati;

         system_stop_stati.push_back(CdmwPlatformMngt::SYSTEM_STOPPING);
         system_stop_stati.push_back(CdmwPlatformMngt::SYSTEM_STOPPED);
*/
         TEST_INFO("Checking system status ");

//         test_result = test_result && observer_on_system.check_sys_stati_sequence (system_name, system_stop_stati);

         //Test 11
//         CPPUNIT_ASSERT(test_result);

         CPPUNIT_ASSERT (CdmwPlatformMngt::SYSTEM_STOPPED == now_snapshot->system_status);

         TEST_INFO("Initialising a new process "+process_12);

         OsSupport::OS::create_process("platform_admin.sh", "--proc-initialise "+ApplOneName+" "
             +process_12+" "+host1_name);

         //TERM2: The process P12 enter the following successive statuses: INITIALISING (Step0), INITLISED.
/*
         std::vector<CdmwPlatformMngt::ProcessStatus> process_restarting_stati; 

         process_restarting_stati.push_back(CdmwPlatformMngt::PROCESS_INITIALISING);
         process_restarting_stati.push_back(CdmwPlatformMngt::PROCESS_INITIALISED);
*/
         TEST_INFO("Wait for "+process_12+" Initialising ");
         OsSupport::OS::sleep (3000);

         TEST_INFO("Checking "+process_12+" stati sequence");
/*
         test_result = test_result && observer_on_system.check_proc_stati_sequence (system_name,
             ApplOneName, host1_name, process_12, process_restarting_stati);

         CPPUNIT_ASSERT(test_result);
*/

         CdmwPlatformMngt::Application_var first_appl = system->get_application(ApplOneName.c_str());

         CdmwPlatformMngt::Process_var process_s = first_appl->get_process (process_12.c_str(), host1_name.c_str());

         CdmwPlatformMngt::ProcessSnapshot_var process_snap = process_s->get_snapshot();

         CPPUNIT_ASSERT (CdmwPlatformMngt::PROCESS_INITIALISED == process_snap->process_status);

         //Unregister Observer:

//         TEST_INFO(" Unregister Observer ");

//         OsSupport::OS::create_process("unregister_observer.sh", "");

         TEST_INFO(" Stopping "+process_12+" process ");

         OsSupport::OS::create_process("platform_admin.sh", "--proc-stop "+ApplOneName+" "+process_12+" "+host1_name);

         OsSupport::OS::sleep (3000);

//         system->unregister_observer_ext ("sys_observer_on_host1");
/*
         //Test 12 NOT AUTOMATIC

         TEST_INFO(" Re-booting "+host2_name+" host");

         OsSupport::OS::create_process("platform_admin.sh", "--host-reboot "+host2_name);
      
         //TERM4: display the following message
         //Broadcast message from root (pts/X) (date XXXX):
         //
         //The system is going down for reboot NOW!
         //Check that connection on TERM3, TERM6 windows is lost..
         //Wait for the reboot of <HOST2>

         TEST_INFO(std::string(host_name2)+" host ping ");

         OsSupport::OS::create_process("ping", host_name2);

         //TERM1: display example (replace by your host and ping values):
         //PING ackbar (192.11.3.3) 56(84) bytes of data.
         //64 bytes from ackbar (192.11.3.3): icmp_seq=0 ttl=64 time=0.363 ms

         //Test 13
         TEST_INFO(host2_name+" host shutdown ");

         OsSupport::OS::create_process("platform_admin.sh", "--host-shutdown "+std::string(host_name2));

         CdmwPlatformMngt::SystemSnapshot* now_snapshot = system->get_snapshot();

         CdmwPlatformMngt::HostsSnapshot now_hosts_snapshot = now_snapshot->system_hosts;

         bool down_host = true;

         for (CORBA::ULong index = 0; index <now_hosts_snapshot.length(); index++) {

             CdmwPlatformMngt::HostSnapshot found_host_two = now_hosts_snapshot[index];

             if (host2_name.compare(found_host_two.host_name) == 0 && 
                   found_host_two.host_status != CdmwPlatformMngt::HOST_UNREACHABLE)
             {
                 down_host = false;
             }
         }

         TEST_INFO(" Check "+host2_name+" host is down ");                 

         test_result = test_result && down_host;

         CPPUNIT_ASSERT(test_result);

         TEST_INFO(host2_name+" host Re-booting ");                    

         OsSupport::OS::create_process("platform_admin.sh", "--host-reboot"+host2_name);
*/
     }

           CPPUNIT_ASSERT(true);
    }
}/* End namespace PlatformMngt */
}/* End namespace Cdmw */





