/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_TESTOBJS_HPP
#define INCL_PLATFORMMNGT_TESTOBJS_HPP

#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/common/System.hpp"
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/Exception.hpp>
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"

#include "Foundation/common/System.hpp"
#include "idllib/CdmwPlatformMngtDaemon.stub.hpp"
#include "platformadminlibrary/PlatformAdminSnapshot.hpp"
#include "test_Q-023/ProcObserver_impl.hpp"

#include <string>

namespace Cdmw
{
  namespace PlatformMngt
  {

    class TestObjs:public CppUnit::TestFixture
    {

    public:

        TestObjs();
        
        void setUp ();

        virtual void test_detect_errors ();

        virtual void test_missing_exec (); 

        virtual void test_timeout_init ();
    
        virtual void test_detect_failures ();
    
        virtual void test_errors_and_stop ();                          

        virtual void test_stop_unmanaged ();                          
        
    private:                  
        
        bool test_result;

        const char* host_name1;
        const char* host_name2;

        std::string host1_name;
        std::string host2_name;

        CdmwPlatformMngt::SupervisionObserver_var observer_sys;
        ProcObserver_impl observer_on_system;

        std::string Scenario;                
        std::string Second_Scenario;
        std::string Third_Scenario;
        std::string Fourth_Scenario;
        std::string Fifth_Scenario;
        std::string Sixth_Scenario;
        std::string Seventh_Scenario;
        std::string Eighth_Scenario;

        std::string ApplFourName;
        std::string ApplThreeName;
        std::string ApplTwoName;
        std::string ApplOneName;

        std::string process_42;
        std::string process_41;
        std::string process_32;
        std::string process_31;
        std::string process_21;
        std::string process_22;
        std::string process_23;
        std::string process_11;
        std::string process_12;
        std::string process_13;
        std::string process_14;
        std::string process_15;

        std::string system_name;       

        //System reference
        CdmwPlatformMngt::System_var system ;        
        CORBA::String_var statusInfo;
        CORBA::ORB_var orb;
        
        int val;
        int long_timeout;
        int short_timeout;
        unsigned long startup_mode;
        std::string specified_state_id;

        void lets_wait (int before, int after);

        /**
         * This function controls if system is started
         */
        bool sys_status_control (const CdmwPlatformMngt::System_ptr ptr) throw (); 
        
        bool app_status_control (const CdmwPlatformMngt::Application_ptr ptr, 
            const CdmwPlatformMngt::ApplicationStatus& expected_status) throw ();
        
        bool proc_status_control (const CdmwPlatformMngt::Process_ptr ptr, 
            const CdmwPlatformMngt::ProcessStatus& expected_status) throw ();        
        
        /**
         * Assignment operator
         */
        TestObjs & operator= (const TestObjs & rhs) throw ();
        
        /**
         * Copy constructor
         */
        TestObjs (const TestObjs & rhs) throw ();

        /**
         * Defining a Test Suite
         */       
        CPPUNIT_TEST_SUITE (TestObjs);
  
      
        //These are the test cases
        
        CPPUNIT_TEST(test_detect_errors);
//        CPPUNIT_TEST(test_missing_exec);
//        CPPUNIT_TEST(test_timeout_init);
//        CPPUNIT_TEST(test_detect_failures);
//        CPPUNIT_TEST(test_errors_and_stop);
//        CPPUNIT_TEST(test_stop_unmanaged);

        CPPUNIT_TEST_SUITE_END ();
    };                       
  }             /* End namespace PlatformMngt */
}               /* End namespace Cdmw */
#endif              /* INCL_PLATFORMMNGT_TESTRESTARTOBJS_HPP */
