/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <fstream>
#include <limits>

#include "test_Q-023/TestObjs.hpp"
#include "test_Q-023/ProcObserver_impl.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"

#include <SystemMngt/platformlibrary/Iterator.hpp>
#include <SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp>
#include <idllib/CdmwPlatformMngtSystem.stub.hpp>
#include <idllib/CdmwPlatformMngtProcess.stub.hpp>
#include <idllib/CdmwPlatformMngtCommon.stub.hpp>
#include <idllib/CdmwPlatformMngtApplicationAgent.stub.hpp>


#include <SystemMngt/platformvaluetypes/HostStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/ApplicationStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/EntityStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/SystemStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/SystemModeChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/ApplicationModeChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/PropertyChange_impl.hpp>


using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestObjs);

namespace Cdmw
{
  namespace PlatformMngt
  {       

    TestObjs::TestObjs()
    {
        test_result = false;

        system_name = "";
        
        Scenario ="Q-023-1-1.xml";                
        Second_Scenario = "Q-023-2-1.xml";
        Third_Scenario ="Q-023-3-1.xml";
        Fourth_Scenario = "Q-023-4.xml";
        Fifth_Scenario = "Q-023-5.xml";
        Sixth_Scenario = "Q-023-6.xml";
        Seventh_Scenario = "Q-023-7.xml";
        Eighth_Scenario = "Q-023-8.xml";

        ApplFourName = "APPL4";
        ApplThreeName = "APPL3";
        ApplTwoName = "APPL2";
        ApplOneName = "APPL1";

        process_41 = "P41";
        process_42 = "P42";

        process_31 = "P31";        
        process_32 = "P32";

        process_21 = "P21";
        process_22 = "P22";
        process_23 = "P23";

        process_11 = "P11";
        process_12 = "P12";
        process_13 = "P13";
        process_14 = "P14";
        process_15 = "P15";       

    }

    void TestObjs::setUp ()
    {
       try
       {              
            OsSupport::OS::create_process("start_daemon_remote.sh", "");            

            OsSupport::OS::create_process("cdmw_host_start.sh", "");                        

            lets_wait (25000, 24000);

            lets_wait (1000, 24000);

            lets_wait (1000, 14000);

            lets_wait (1000, 1000);

        //Getting system var
            std::cerr << " Getting system var " << std::endl;
        std::string system_corbaloc = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";

            orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::instance ()->get_ORB ());
            CORBA::Object_var system_obj = orb->string_to_object (system_corbaloc.c_str());

            if (CORBA::is_nil(system_obj)) {
               std::cerr << " System_obj nil " << std::endl;

            } 

            if (system_obj->_non_existent()) {

                std::cerr << " System_obj not existent " << std::endl;                      
            } 
            

        system = CdmwPlatformMngt::System::_narrow (system_obj.in ());

            if (CORBA::is_nil(system)) {
                std::cerr << " System nil " << std::endl;

            } 
            
            if (system->_non_existent()) {

               std::cerr << " System not existent " << std::endl;                      
            } 
            

            std::cerr << "  narrow di system " << std::endl;
              
        //Setting up the timeouts
        long_timeout  = 4 ; 
        short_timeout = 1 ;

        CORBA::ValueFactoryBase_var factory, old_factory;
     
            std::cerr << "  inizializzazione delle factories " << std::endl;

        factory = new ApplicationStatusChangeFactory;
        old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationStatusChange:1.0",factory.in() );
        factory = new ProcessStatusChangeFactory;
        old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/ProcessStatusChange:1.0",factory.in() );
        factory = new SystemStatusChangeFactory;
        old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/SystemStatusChange:1.0",factory.in() );
        factory = new HostStatusChangeFactory;
        old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/HostStatusChange:1.0",factory.in() );
              
            CdmwPlatformMngt::ElementNames_var elems;
            CdmwPlatformMngt::ElementNamesIterator_var it;

            std::cerr << "  nomi hosts " << std::endl;
          
            //Slot di attesa di 91s
            lets_wait (25000, 24000);

            lets_wait (1000, 24000);

            lets_wait (1000, 14000);

            lets_wait (1000, 1000);

            CORBA::ULong nbr_of_host;         

            try {                          

               nbr_of_host = system->get_number_of_hosts();

            } catch (const CORBA::TRANSIENT& excTrans) {

               std::cerr << "  CORBA::TRANSIENT " << std::endl;              
               std::cerr << "  CORBA::TRANSIENT: " << excTrans << std::endl;              
               throw;

            } catch (const CORBA::SystemException& excS) {
              
               std::cerr << "  CORBA::SystemException: " << std::endl;
               std::cerr << "  Exception: "+std::string(excS._name()) << std::endl;
               throw;

            }  catch (const Exception& exc) {
           
               std::cerr << "  C++ Exception: " << std::endl;
               std::cerr << "  Exception: "+std::string(exc.what()) << std::endl;
               throw;
              
            } catch (...) {
               std::cerr << "  Generic Exception " << std::endl;
               throw;
            }

            std::cerr << "  quanti hosts " << std::endl;

            // get the name of hosts            
        system->get_all_host_names(nbr_of_host, elems.out(), it.out());

        host_name1 = elems[0U];
        host_name2 = elems[1U];       

            host1_name = std::string (host_name1);
            host2_name = std::string (host_name2); 
        
            std::cerr << "  primo snap del sistema " << std::endl;

        CdmwPlatformMngt::SystemSnapshot* now_snapshot = system->get_snapshot();
        system_name = now_snapshot->system_name;

           TEST_INFO("System Name is: "+system_name);

       observer_sys = observer_on_system._this();
 
       //CdmwPlatformMngt::RegistrationDepth depth =  FULL_HIERARCHY; //OBJECT_ONLY;
       TEST_INFO(" Registered Observers: process_observer_on_system"); 

           //system -> register_observer_ext ("sys_observer_on_host1",depth, observer_sys.in ());         
           system ->register_observer("sys_observer_on_host1", observer_sys.in ());
                                    
      } catch(...)
        {

          std::cerr << "  Exception Occurred " << std::endl;

          std::cerr << "  Killing platform daemon on second host " << std::endl;

          OsSupport::OS::create_process("start_daemon_remote.sh", "");  

          lets_wait (1000, 1000);

          std::cerr << "  Killing ft_manager " << std::endl;

          OsSupport::OS::create_process("killall", "-9 cdmw_ft_manager");

          lets_wait (1000, 1000);

          std::cerr << "  Killing platform supervision " << std::endl;

          OsSupport::OS::create_process("killall", "cdmw_platform_supervision");

          lets_wait (1000, 1000);

          std::cerr << "  Killing platform daemon " << std::endl;

          OsSupport::OS::create_process("killall", "cdmw_platform_daemon");

          lets_wait (1000, 1000);          

          CPPUNIT_ASSERT(false);
        }
    }

    void TestObjs::test_detect_errors () 
    {

       lets_wait (30000, 30000);

       TEST_INFO(" TEST  CASE 1: Detection of errors in the application initialization \n and run  sequences "); 
       /////////////
       // TEST  CASE 1: Detection of errors
       /////////////
       
       TEST_INFO(" Stopping Daemon on "+host2_name); 
       
       //Stopping platform daemon on HOST2       
       OsSupport::OS::create_process("stop_daemon_remote.sh", "");

       lets_wait(15000, 15000);

       TEST_INFO ("Defining the system: define_system.sh "+Scenario);

       OsSupport::OS::create_process("define_system.sh", Scenario);

       lets_wait(15000, 15000); 

       TEST_INFO ("Starting the system: start_system.sh ");

       OsSupport::OS::create_process("start_system.sh", "");

       lets_wait(15000, 15000);      

       TEST_INFO("Checking "+host2_name+" status");

       bool result = true;

       CdmwPlatformMngt::SystemSnapshot* now_snapshot = system->get_snapshot();

       CdmwPlatformMngt::HostsSnapshot hosts_snapshot = now_snapshot->system_hosts;

       for (CORBA::ULong host_index = 0; host_index < hosts_snapshot.length(); host_index++) {

           CdmwPlatformMngt::HostSnapshot host_snapshot = hosts_snapshot[host_index];

           if (host2_name.compare(host_snapshot.host_name) == 0) {

               if (!(host_snapshot.host_status == CdmwPlatformMngt::HOST_UNREACHABLE)) {

                   result = false;
               }

           }
 
       }

       TEST_INFO("Checking "+ApplOneName+" and "+ApplTwoName+" status and mode ");

       CdmwPlatformMngt::ApplicationsSnapshot appls_snapshot = now_snapshot->system_applications;

       for (CORBA::ULong index = 0; index < appls_snapshot.length(); index++) {

     CdmwPlatformMngt::ApplicationSnapshot application_snapshot = appls_snapshot [index];

         if (ApplOneName.compare(application_snapshot.application_name) == 0 ||
             ApplTwoName.compare(application_snapshot.application_name) == 0) {

             CdmwPlatformMngt::ApplicationStatus application_status = application_snapshot.application_status;

             CdmwPlatformMngt::FunctioningMode application_mode = application_snapshot.application_mode;

         if (ApplOneName.compare(application_snapshot.application_name) == 0) {
             
             TEST_INFO (ApplOneName+" found !");

             if (!(application_status == CdmwPlatformMngt::APPLICATION_PARTIAL_RUNNING &&
                   application_mode == CdmwPlatformMngt::DEGRADED_MODE)) {
                
                result = false;
             } else {

                 CdmwPlatformMngt::ProcessesSnapshot processes_snapshot = application_snapshot.application_processes;
             
                 TEST_INFO("Checking "+process_13+" and "+process_14+" status ");

                 for (CORBA::ULong p_index = 0; p_index < processes_snapshot.length(); p_index++) {

                     CdmwPlatformMngt::ProcessSnapshot process_snapshot = 
                          processes_snapshot [p_index];                      

                     if (process_13.compare(process_snapshot.process_name) == 0) 
                     {
                         CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
                   
                           TEST_INFO (process_13+" found !");

                       if (!(process_status == CdmwPlatformMngt::PROCESS_FAILED_INVALID)) {
               
                               result = false;
                    } 
               } 
               else if (process_14.compare(process_snapshot.process_name) == 0) 
               {
                CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                            TEST_INFO (process_14+" found !");

                            if (!(process_status == CdmwPlatformMngt::PROCESS_FAILED_INVALID)) {

                                result = false;
                            } 
                       }
                  } //processes for cycle
              } //processes else
         } //APPL1 if 
          else if (ApplTwoName.compare(application_snapshot.application_name) == 0) {
          
             TEST_INFO (ApplTwoName+" found !");
            
             if (!(application_status == CdmwPlatformMngt::APPLICATION_RUNNING)) {

                 result = false;
             }
           } // APPL2 else if
         } // APPL1 || APPL2
       } //Applications for cycle
     
       TEST_INFO("Checking "+system_name+" status ");

       if (!(now_snapshot->system_status == CdmwPlatformMngt::SYSTEM_PARTIAL_STARTED)) {

           result = false;
       }

       test_result = result;

       CPPUNIT_ASSERT(test_result);

       result = true;

       //Restart platform daemon on HOST2
       OsSupport::OS::create_process("start_daemon_remote.sh", "");

       lets_wait(20000, 20000); 

       now_snapshot = system->get_snapshot();

       TEST_INFO("Checking "+host2_name+" status");

       hosts_snapshot = now_snapshot->system_hosts;

       for (CORBA::ULong host_index = 0; host_index < hosts_snapshot.length(); host_index++) {

           CdmwPlatformMngt::HostSnapshot host_snapshot = hosts_snapshot[host_index];

           if (host2_name.compare(host_snapshot.host_name) == 0) {

               if (!(host_snapshot.host_status == CdmwPlatformMngt::HOST_RESPONDING)) {

                   result = false;
               }

           }
       }

       TEST_INFO("Checking "+system_name+" status: it must be unchanged");

       if (!(now_snapshot->system_status == CdmwPlatformMngt::SYSTEM_PARTIAL_STARTED)) {

           result = false;
       }

       test_result = test_result && result;

       CPPUNIT_ASSERT(test_result);

       result = true;

       TEST_INFO ("Hot "+ApplThreeName+" restart: platform_admin.sh --app-hot-restart "+
           ApplThreeName);       

       OsSupport::OS::create_process("platform_admin.sh", "--app-hot-restart "+
           ApplThreeName);

       lets_wait(15000, 15000); 
      
       now_snapshot = system->get_snapshot();

       TEST_INFO("Checking system status and mode");      

       if (!(now_snapshot->system_status == CdmwPlatformMngt::SYSTEM_PARTIAL_STARTED 
            && now_snapshot->system_mode == CdmwPlatformMngt::DEGRADED_MODE)) {

           result = false;
       }

       appls_snapshot = now_snapshot->system_applications;

       for (CORBA::ULong index = 0; index < appls_snapshot.length(); index++) {

     CdmwPlatformMngt::ApplicationSnapshot application_snapshot = appls_snapshot [index];

         if (ApplOneName.compare(application_snapshot.application_name) == 0) {

             CdmwPlatformMngt::ApplicationStatus application_status = application_snapshot.application_status;

             CdmwPlatformMngt::FunctioningMode application_mode = application_snapshot.application_mode;
             
             TEST_INFO (ApplOneName+" found !");

             if (!(application_status == CdmwPlatformMngt::APPLICATION_PARTIAL_RUNNING &&
                   application_mode == CdmwPlatformMngt::DEGRADED_MODE)) {
                
                result = false;
             } else {

                 CdmwPlatformMngt::ProcessesSnapshot processes_snapshot =
                               application_snapshot.application_processes;
             
                 TEST_INFO("Checking "+ApplOneName+" processes status ");

                 for (CORBA::ULong p_index = 0; p_index < processes_snapshot.length(); p_index++) {

                     CdmwPlatformMngt::ProcessSnapshot process_snapshot = 
                          processes_snapshot [p_index];                      

                     if (process_13.compare(process_snapshot.process_name) == 0) 
                 {
                         CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
                   
                           TEST_INFO (process_13+" found !");

                       if (!(process_status == CdmwPlatformMngt::PROCESS_FAILED_INVALID)) {
               
                               result = false;
                    } 
               } 
               else if (process_14.compare(process_snapshot.process_name) == 0) 
               {
                CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                            TEST_INFO (process_14+" found !");

                            if (!(process_status == CdmwPlatformMngt::PROCESS_FAILED_INVALID)) {

                                result = false;
                            } 
                       } 
                       else if (process_11.compare(process_snapshot.process_name) == 0)
                       {
                CdmwPlatformMngt::ProcessStatus process_status =
                                           process_snapshot.process_status;
               
                            TEST_INFO (process_11+" found !");

                            if (!(process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {

                                result = false;
                            } 

                      
                       }
                       else if (process_12.compare(process_snapshot.process_name) == 0)
                       {
                CdmwPlatformMngt::ProcessStatus process_status =
                                            process_snapshot.process_status;
               
                            TEST_INFO (process_12+" found !");

                            if (!(process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {

                                result = false;
                            }                       
                       }
                    } //Processes for cycle
                 } //Processes else
              } // APPL1             
              else if (ApplTwoName.compare(application_snapshot.application_name) == 0) 
              {
                  CdmwPlatformMngt::ApplicationStatus application_status =
                                   application_snapshot.application_status;
              
                  TEST_INFO (ApplTwoName+" found !");

                  if (!(application_status == CdmwPlatformMngt::APPLICATION_RUNNING)) {
                
                       result = false;
                  } else {

                     CdmwPlatformMngt::ProcessesSnapshot processes_snapshot =
                                   application_snapshot.application_processes;
             
                     TEST_INFO("Checking "+ApplTwoName+" processes status ");

                     for (CORBA::ULong p_index = 0; p_index < processes_snapshot.length(); 
                        p_index++) {

                         CdmwPlatformMngt::ProcessSnapshot process_snapshot = 
                               processes_snapshot [p_index];                         

                         if (process_21.compare(process_snapshot.process_name) == 0) 
                         {
                             CdmwPlatformMngt::ProcessStatus process_status =
                                     process_snapshot.process_status;
                   
                                 TEST_INFO (process_21+" found !");

                         if (!(process_status == CdmwPlatformMngt::PROCESS_ENDED)) {
               
                                 result = false;
                         } 
                     } 
                         else if (process_22.compare(process_snapshot.process_name) == 0) 
                 {
                CdmwPlatformMngt::ProcessStatus process_status =
                                             process_snapshot.process_status;
               
                            TEST_INFO (process_22+" found !");

                            if (!(process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {

                                result = false;
                            } 
                         } 
                         else if (process_23.compare(process_snapshot.process_name) == 0)
                         {
                CdmwPlatformMngt::ProcessStatus process_status =
                                           process_snapshot.process_status;
               
                            TEST_INFO (process_23+" found !");

                            if (!(process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {

                                result = false;
                            }                       
                         }
                     } //Processes for cycle                       
                  } //Processes else
              } //APPL2
              else if (ApplThreeName.compare(application_snapshot.application_name) == 0) 
              {
                  CdmwPlatformMngt::ApplicationStatus application_status =
                                   application_snapshot.application_status;
              
                  TEST_INFO (ApplThreeName+" found !");

                  if (!(application_status == CdmwPlatformMngt::APPLICATION_RUNNING)) {
                
                       result = false;
                  } else {

                     CdmwPlatformMngt::ProcessesSnapshot processes_snapshot =
                                   application_snapshot.application_processes;
             
                     TEST_INFO("Checking "+ApplThreeName+" processes status ");

                     for (CORBA::ULong p_index = 0; p_index < processes_snapshot.length(); 
                        p_index++) {

                         CdmwPlatformMngt::ProcessSnapshot process_snapshot = 
                               processes_snapshot [p_index];                         

                         if (process_31.compare(process_snapshot.process_name) == 0) 
                         {
                             CdmwPlatformMngt::ProcessStatus process_status =
                                     process_snapshot.process_status;
                   
                                 TEST_INFO (process_31+" found !");

                         if (!(process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {
               
                                 result = false;
                         } 
                     } 
                         else if (process_32.compare(process_snapshot.process_name) == 0) 
                 {
                CdmwPlatformMngt::ProcessStatus process_status =
                                             process_snapshot.process_status;
               
                            TEST_INFO (process_32+" found !");

                            if (!(process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {

                                result = false;
                            } 
                         } 
                     }//Processes for cycle
                  } //Processes else
              } //APPL3 else if
         } //Applications for cycle
      
       test_result = test_result && result;

       CPPUNIT_ASSERT(test_result);

       result = true;

       TEST_INFO("Commands useful to restart "+process_13+" and "+process_14+" processes");

       TEST_INFO ("Initialise process "+process_13+" : platform_admin.sh --proc-initialise "+
           ApplOneName+" "+process_13+" "+host2_name);       

       OsSupport::OS::create_process("platform_admin.sh", "--proc-initialise "+
           ApplOneName+" "+process_13+" "+host2_name);

       TEST_INFO ("Initialise process "+process_14+" : platform_admin.sh --proc-initialise "+
           ApplOneName+" "+process_14+" "+host2_name);       

       OsSupport::OS::create_process("platform_admin.sh", "--proc-initialise "+
           ApplOneName+" "+process_14+" "+host2_name);

       lets_wait(15000, 15000); 

       TEST_INFO ("Advance process "+process_13+" step : platform_admin.sh --proc-next-step "+
           ApplOneName+" "+process_13+" "+host2_name);       

       OsSupport::OS::create_process("platform_admin.sh", "--proc-next-step "+
           ApplOneName+" "+process_13+" "+host2_name);

       TEST_INFO ("Advance process "+process_14+" step : platform_admin.sh --proc-next-step "+
           ApplOneName+" "+process_14+" "+host2_name);       

       OsSupport::OS::create_process("platform_admin.sh", "--proc-next-step "+
           ApplOneName+" "+process_14+" "+host2_name);

       lets_wait(15000, 15000); 
       
       TEST_INFO ("Run process "+process_13+" : platform_admin.sh --proc-run "+
           ApplOneName+" "+process_13+" "+host2_name);       

       OsSupport::OS::create_process("platform_admin.sh", "--proc-run "+
           ApplOneName+" "+process_13+" "+host2_name);

       TEST_INFO ("Run process "+process_14+" : platform_admin.sh --proc-run "+
           ApplOneName+" "+process_14+" "+host2_name);       

       OsSupport::OS::create_process("platform_admin.sh", "--proc-run "+
           ApplOneName+" "+process_14+" "+host2_name);

       TEST_INFO("Check "+process_13+" and "+process_14+" run status");

       lets_wait(15000, 15000); 
      
       now_snapshot = system->get_snapshot();

       TEST_INFO("Checking system status and mode");      

       if (!(now_snapshot->system_status == CdmwPlatformMngt::SYSTEM_PARTIAL_STARTED 
            && now_snapshot->system_mode == CdmwPlatformMngt::DEGRADED_MODE)) {

           result = false;
       }

       appls_snapshot = now_snapshot->system_applications;

       for (CORBA::ULong index = 0; index < appls_snapshot.length(); index++) {

     CdmwPlatformMngt::ApplicationSnapshot application_snapshot = appls_snapshot [index];

         if (ApplOneName.compare(application_snapshot.application_name) == 0) {

            CdmwPlatformMngt::ProcessesSnapshot processes_snapshot = application_snapshot.application_processes;
             
                 TEST_INFO("Checking "+ApplOneName+" processes status ");

                 for (CORBA::ULong p_index = 0; p_index < processes_snapshot.length(); p_index++) {

                     CdmwPlatformMngt::ProcessSnapshot process_snapshot = 
                          processes_snapshot [p_index];                      

                     if (process_13.compare(process_snapshot.process_name) == 0) 
                     {
                         CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
                   
                           TEST_INFO (process_13+" found !");

                       if (!(process_status == CdmwPlatformMngt::PROCESS_FAILED_INVALID)) {
               
                               result = false;
                    } 
               } 
               else if (process_14.compare(process_snapshot.process_name) == 0) 
               {
                CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                            TEST_INFO (process_14+" found !");

                            if (!(process_status == CdmwPlatformMngt::PROCESS_FAILED_INVALID)) {

                                result = false;
                            } 
                       }                                      
                  }  //Processes for cycle
              } //APPL1 if                          
         } //Applications for cycle
       
       test_result = test_result && result;
       CPPUNIT_ASSERT(test_result);

       TEST_INFO ("Stopping the system ");       

       OsSupport::OS::create_process("stop_system.sh", "");

       lets_wait(15000, 15000); 


    }

    void TestObjs::test_missing_exec() 
    {  
        if (&observer_on_system != 0) {

            TEST_INFO(" TEST  CASE 2: Missing executable file for process "+process_22);                            
            /////////////
            // TEST  CASE 2: Missing executable file for process P22
            /////////////

            TEST_INFO ("Defining the system: define_system.sh "+Second_Scenario);

            OsSupport::OS::create_process("define_system.sh", Second_Scenario);

            lets_wait(15000, 15000); 

            TEST_INFO ("Starting the system: start_system.sh ");

            OsSupport::OS::create_process("start_system.sh", "");

            lets_wait(15000, 15000); 
          
            bool result = true;

            CdmwPlatformMngt::SystemSnapshot* now_snapshot = system->get_snapshot();            

            TEST_INFO("Checking "+ApplOneName+" and "+ApplThreeName+" processes status ");

           CdmwPlatformMngt::ApplicationsSnapshot appls_snapshot = 
                                    now_snapshot->system_applications;

           for (CORBA::ULong index = 0; index < appls_snapshot.length(); index++) {

          CdmwPlatformMngt::ApplicationSnapshot application_snapshot = appls_snapshot [index];

              if (ApplOneName.compare(application_snapshot.application_name) == 0 ||
                  ApplTwoName.compare(application_snapshot.application_name) == 0 ||
                  ApplThreeName.compare(application_snapshot.application_name) == 0) {

                  CdmwPlatformMngt::ApplicationStatus application_status =
                                   application_snapshot.application_status;

                  if (ApplOneName.compare(application_snapshot.application_name) == 0) {
             
                      TEST_INFO (ApplOneName+" found !");

                  if (!(application_status == CdmwPlatformMngt::APPLICATION_RUNNING)) {
                
                      result = false;
                  } else {

                      CdmwPlatformMngt::ProcessesSnapshot processes_snapshot =
                                    application_snapshot.application_processes;
             
                      TEST_INFO("Checking "+ApplOneName+" processes status ");

                      for (CORBA::ULong p_index = 0; p_index < processes_snapshot.length(); 
                          p_index++) {

                          CdmwPlatformMngt::ProcessSnapshot process_snapshot = 
                             processes_snapshot [p_index];                       
 
                         CdmwPlatformMngt::ProcessStatus process_status =
                                              process_snapshot.process_status;                

                             if (!(process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {
               
                               result = false;
                         } 
 
                             if (process_11.compare(process_snapshot.process_name) == 0) 
                             {             
                                 TEST_INFO (process_11+" found !");
                             }
                             else if (process_12.compare(process_snapshot.process_name) == 0) 
                             {             
                                 TEST_INFO (process_12+" found !");
                             }
                             else if (process_13.compare(process_snapshot.process_name) == 0) 
                             {         
                                 TEST_INFO (process_13+" found !");
                             }
                             else if (process_14.compare(process_snapshot.process_name) == 0) 
                             {         
                                 TEST_INFO (process_14+" found !");
                             }
                  } //processes for cycle
              } //processes else
         } //APPL1 if 
          else if (ApplThreeName.compare(application_snapshot.application_name) == 0) {
          
             TEST_INFO (ApplThreeName+" found !");
            
             if (!(application_status == CdmwPlatformMngt::APPLICATION_RUNNING)) {

                 result = false;
             } else {

                      CdmwPlatformMngt::ProcessesSnapshot processes_snapshot =
                                    application_snapshot.application_processes;
             
                      TEST_INFO("Checking "+ApplThreeName+" processes status ");

                      for (CORBA::ULong p_index = 0; p_index < processes_snapshot.length(); 
                          p_index++) {

                          CdmwPlatformMngt::ProcessSnapshot process_snapshot = 
                             processes_snapshot [p_index];                       
 
                         CdmwPlatformMngt::ProcessStatus process_status =
                                              process_snapshot.process_status;                

                             if (!(process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {
               
                               result = false;
                         } 
 
                             if (process_31.compare(process_snapshot.process_name) == 0) 
                             {             
                                 TEST_INFO (process_31+" found !");
                             }
                             else if (process_32.compare(process_snapshot.process_name) == 0) 
                             {             
                                 TEST_INFO (process_32+" found !");
                             }

                      } //processes for cycle
                  } //processes else
               } // APPL3 else if
               else if (ApplTwoName.compare(application_snapshot.application_name) == 0) {
          
                   TEST_INFO (ApplTwoName+" found !");
            
                   if (!(application_status == CdmwPlatformMngt::APPLICATION_RUNNING)) {

                       result = false;
                   } else {

                      CdmwPlatformMngt::ProcessesSnapshot processes_snapshot =
                                    application_snapshot.application_processes;
             
                      TEST_INFO("Checking "+ApplTwoName+" processes status ");

                      for (CORBA::ULong p_index = 0; p_index < processes_snapshot.length(); 
                          p_index++) {

                          CdmwPlatformMngt::ProcessSnapshot process_snapshot = 
                             processes_snapshot [p_index];

                          if (process_21.compare(process_snapshot.process_name) == 0) 
                          {
                         CdmwPlatformMngt::ProcessStatus process_status =
                                              process_snapshot.process_status;
                   
                           TEST_INFO (process_21+" found !");

                       if (!(process_status == CdmwPlatformMngt::PROCESS_ENDED)) {
               
                               result = false;
                    } 
                  } 
                  else if (process_22.compare(process_snapshot.process_name) == 0) 
                          {
                      CdmwPlatformMngt::ProcessStatus process_status =
                                               process_snapshot.process_status;
               
                            TEST_INFO (process_22+" found !");

                            if (!(process_status == CdmwPlatformMngt::PROCESS_FAILED_INVALID)) {

                                result = false;
                            } 
                       } 
                  else if (process_23.compare(process_snapshot.process_name) == 0) 
                          {
                      CdmwPlatformMngt::ProcessStatus process_status =
                                               process_snapshot.process_status;
               
                            TEST_INFO (process_23+" found !");

                            if (!(process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {

                                result = false;
                            } 
                       }                                      
                                     
                      } //processes for cycle
                  } //processes else
               } // APPL2 else if

             } // APPL1 || APPL3 || APPL2
           } //Applications for cycle
     
       TEST_INFO("Checking "+system_name+" status ");

       if (!(now_snapshot->system_status == CdmwPlatformMngt::SYSTEM_PARTIAL_STARTED)) {

           result = false;
       }

       test_result = result;

       CPPUNIT_ASSERT(test_result);

       result = true;

       TEST_INFO ("Modifying the process "+process_22+" executable : platform_admin.sh --proc-set-info "+ApplTwoName+" "+process_22+" "+host1_name+" EXEC=cdmw_test_managed_process PORT=21882 ARGS=--nb-steps=1");

       OsSupport::OS::create_process("platform_admin.sh", "--proc-set-info "+ApplTwoName+" "+process_22+" "+host1_name+" EXEC=cdmw_test_managed_process PORT=21882 ARGS=--nb-steps=1");

       OsSupport::OS::sleep (10000);

       OsSupport::OS::create_process("platform_admin.sh", "--proc-hot-restart "+ApplTwoName+
           " "+process_22+" "+host1_name);

       lets_wait(15000, 15000); 

       std::vector <CdmwPlatformMngt::ProcessStatus> p22_stati;

       p22_stati.push_back (CdmwPlatformMngt::PROCESS_INITIALISING);
       p22_stati.push_back (CdmwPlatformMngt::PROCESS_INITIALISED);
       p22_stati.push_back (CdmwPlatformMngt::PROCESS_RUNNING);

       result = observer_on_system.check_proc_stati_sequence(system_name, ApplTwoName,   
            host1_name, process_22, p22_stati);

       OsSupport::OS::create_process("stop_system.sh", "");

       test_result = test_result && result;

       }

       CPPUNIT_ASSERT(test_result);            
    }

    void TestObjs::test_timeout_init() {

        if (&observer_on_system != 0) {

              TEST_INFO(" TEST  CASE 3: Time-out Initialisation ");                       
              /////////////
              // TEST  CASE 3: Time-out Initialisation 
              /////////////       

              bool result = true;

              TEST_INFO ("Defining the system: define_system.sh "+Third_Scenario);

              OsSupport::OS::create_process("define_system.sh", Third_Scenario);

              lets_wait(15000, 15000); 

              TEST_INFO ("Starting the system: start_system.sh ");

              OsSupport::OS::create_process("start_system.sh", "");

              lets_wait(15000, 15000); 

              TEST_INFO ("Commands to set short initialisation and \n run timeouts for process "
                  +process_41+": platform_admin.sh --proc-set-life-cycle "+ApplFourName+
                  " "+process_41+" "+host1_name+" "+Fourth_Scenario);

              OsSupport::OS::create_process("platform_admin.sh", "--proc-set-life-cycle "
                   +ApplFourName+" "+process_41+" "+host1_name+" "+Fourth_Scenario);

              lets_wait(15000, 15000); 

              TEST_INFO ("Initialisation command for "+ApplFourName+" application: \n platform_admin.sh --app-initialise "+ApplFourName+" LAST_STATE 10000 nothing ");

              OsSupport::OS::create_process("platform_admin.sh", " --app-initialise "
                   +ApplFourName+" LAST_STATE 10000 nothing");

              lets_wait(15000, 15000); 

              CdmwPlatformMngt::SystemSnapshot* now_snapshot = system->get_snapshot();

              TEST_INFO("Checking "+system_name+" status ");

              if (!(now_snapshot->system_status == CdmwPlatformMngt::SYSTEM_STARTED &&
                    now_snapshot->system_mode == CdmwPlatformMngt::DEGRADED_MODE)) {

                  result = false;
              }

              TEST_INFO("Checking "+ApplFourName+" status and mode ");

              CdmwPlatformMngt::ApplicationsSnapshot appls_snapshot = 
                                     now_snapshot->system_applications;

              for (CORBA::ULong index = 0; index < appls_snapshot.length(); index++) {

             CdmwPlatformMngt::ApplicationSnapshot application_snapshot = 
                                                       appls_snapshot [index];

                 if (ApplFourName.compare(application_snapshot.application_name) == 0) {

                  CdmwPlatformMngt::ApplicationStatus application_status =
                                     application_snapshot.application_status;

                  CdmwPlatformMngt::FunctioningMode application_mode =
                                       application_snapshot.application_mode;
            
                  TEST_INFO (ApplFourName+" found !");

                  if (!(application_status ==
                        CdmwPlatformMngt::APPLICATION_INITIALISATION_INCOMPLETE &&
                        application_mode == CdmwPlatformMngt::DEGRADED_MODE)) {
                
                        result = false;
                  } else {

                      CdmwPlatformMngt::ProcessesSnapshot processes_snapshot =
                                    application_snapshot.application_processes;
             
                      TEST_INFO("Checking "+process_41+" status ");

                      for (CORBA::ULong p_index = 0; p_index < processes_snapshot.length(); 
                          p_index++) {

                          CdmwPlatformMngt::ProcessSnapshot process_snapshot = 
                          processes_snapshot [p_index];                      

                     if (process_41.compare(process_snapshot.process_name) == 0) 
                     {
                         CdmwPlatformMngt::ProcessStatus process_status =
                                                process_snapshot.process_status;
                   
                           TEST_INFO (process_41+" found !");

                       if (!(process_status == CdmwPlatformMngt::PROCESS_FAILED_TIMEOUT)) {
               
                               result = false;
                    } 
               } 
                  } //processes for cycle
              } //processes else
         } //APPL4 if           
       } //Applications for cycle
            
       test_result = result;

       CPPUNIT_ASSERT(test_result);

       result = true;

       TEST_INFO ("Command to increase initialisation time-out for process "
           +process_41+": \n platform_admin.sh --proc-set-life-cycle "+ApplFourName+" "
           +process_41+" "+host1_name+" "+Fifth_Scenario);

       OsSupport::OS::create_process("platform_admin.sh", "--proc-set-life-cycle "
                   +ApplFourName+" "+process_41+" "+host1_name+" "+Fifth_Scenario);
       
       lets_wait(15000, 15000); 

       TEST_INFO ("Initialisation command for "+ApplFourName
       +" application: \n platform_admin.sh --app-initialise "
       +ApplFourName+" LAST_STATE 10000 nothing ");

       OsSupport::OS::create_process("platform_admin.sh", " --app-initialise "
              +ApplFourName+" LAST_STATE 10000 nothing");

       lets_wait(15000, 15000); 

       now_snapshot = system->get_snapshot();

       TEST_INFO("Checking "+ApplFourName+" status ");

       appls_snapshot = now_snapshot->system_applications;

       for (CORBA::ULong index = 0; index < appls_snapshot.length(); index++) {

         CdmwPlatformMngt::ApplicationSnapshot application_snapshot = 
                                                 appls_snapshot [index];

         if (ApplFourName.compare(application_snapshot.application_name) == 0) {

              CdmwPlatformMngt::ApplicationStatus application_status =
                                application_snapshot.application_status;
            
              TEST_INFO (ApplFourName+" found !");

              if (!(application_status ==
                        CdmwPlatformMngt::APPLICATION_INITIALISATION_INCOMPLETE)) {
                
                        result = false;
              } else {

                  CdmwPlatformMngt::ProcessesSnapshot processes_snapshot =
                                 application_snapshot.application_processes;
             
                  TEST_INFO("Checking "+process_41+" status ");

                   for (CORBA::ULong p_index = 0; p_index < processes_snapshot.length(); 
                          p_index++) {

                      CdmwPlatformMngt::ProcessSnapshot process_snapshot = 
                      processes_snapshot [p_index];                      

                     if (process_41.compare(process_snapshot.process_name) == 0) 
                     {
                         CdmwPlatformMngt::ProcessStatus process_status =
                                                process_snapshot.process_status;
                   
                           TEST_INFO (process_41+" found !");

                       if (!(process_status == CdmwPlatformMngt::PROCESS_INITIALISED)) {
               
                               result = false;
                   } 
               } 
                  } //processes for cycle
              } //processes else
         } //APPL4 if           
       } //Applications for cycle

       test_result = test_result && result;

       CPPUNIT_ASSERT(test_result);

       result = true;

       TEST_INFO("Command to run "+ApplFourName+" application: platform_admin.sh --app-run "+
       ApplFourName);

       OsSupport::OS::create_process("platform_admin.sh", " --app-run "+ApplFourName);
       
       lets_wait(15000, 15000); 

       now_snapshot = system->get_snapshot();

       TEST_INFO("Checking "+ApplFourName+" status ");

       appls_snapshot = now_snapshot->system_applications;

       for (CORBA::ULong index = 0; index < appls_snapshot.length(); index++) {

         CdmwPlatformMngt::ApplicationSnapshot application_snapshot = 
                                                 appls_snapshot [index];

         if (ApplFourName.compare(application_snapshot.application_name) == 0) {

              CdmwPlatformMngt::ApplicationStatus application_status =
                                application_snapshot.application_status;

              CdmwPlatformMngt::FunctioningMode application_mode =
                                       application_snapshot.application_mode;
            
              TEST_INFO (ApplFourName+" found !");

              if (!(application_status ==
                        CdmwPlatformMngt::APPLICATION_PARTIAL_RUNNING &&
                    application_mode == CdmwPlatformMngt::DEGRADED_MODE)) {
                
                        result = false;
              } else {

                  CdmwPlatformMngt::ProcessesSnapshot processes_snapshot =
                                 application_snapshot.application_processes;
             
                  TEST_INFO("Checking "+process_41+" status ");

                   for (CORBA::ULong p_index = 0; p_index < processes_snapshot.length(); 
                          p_index++) {

                      CdmwPlatformMngt::ProcessSnapshot process_snapshot = 
                      processes_snapshot [p_index];                      

                     if (process_41.compare(process_snapshot.process_name) == 0) 
                     {
                         CdmwPlatformMngt::ProcessStatus process_status =
                                                process_snapshot.process_status;
                   
                           TEST_INFO (process_41+" found !");

                       if (!(process_status == CdmwPlatformMngt::PROCESS_FAILED_TIMEOUT)) {
               
                               result = false;
                   } 
               } 
                  } //processes for cycle
              } //processes else
         } //APPL4 if           
       } //Applications for cycle

       test_result = test_result && result;

       CPPUNIT_ASSERT(test_result);

       result = true;

       TEST_INFO("Command to increase run time-out for process "+process_41
       +": platform_admin.sh --proc-set-life-cycle "+ApplFourName+" "+process_41+
       " "+host1_name+" "+Sixth_Scenario);

       OsSupport::OS::create_process("platform_admin.sh", "--proc-set-life-cycle "+ApplFourName
           +" "+process_41+" "+host1_name+" "+Sixth_Scenario);
       
       lets_wait(15000, 15000); 

       TEST_INFO("Command to increase run time-out for process "+process_41
       +": platform_admin.sh --proc-initialise "+ApplFourName+" "+process_41+
       " "+host1_name+" LAST_STATE 10000 nothing ");

       OsSupport::OS::create_process("platform_admin.sh", "--proc-initialise "+ApplFourName
           +" "+process_41+" "+host1_name+" LAST_STATE 10000 nothing ");
       
       lets_wait(15000, 15000); 

       TEST_INFO("Command to run process "+process_41
       +": platform_admin.sh --proc-run "+ApplFourName+" "+process_41+
       " "+host1_name);

       OsSupport::OS::create_process("platform_admin.sh", "--proc-run "+ApplFourName
           +" "+process_41+" "+host1_name);

       lets_wait(15000, 15000); 

       now_snapshot = system->get_snapshot();

       TEST_INFO("Checking "+system_name+" status ");

       if (!(now_snapshot->system_status == CdmwPlatformMngt::SYSTEM_STARTED)) {

                  result = false;
       }

       TEST_INFO("Checking "+ApplFourName+" status ");

       appls_snapshot = now_snapshot->system_applications;

       for (CORBA::ULong index = 0; index < appls_snapshot.length(); index++) {

         CdmwPlatformMngt::ApplicationSnapshot application_snapshot = 
                                                 appls_snapshot [index];

         if (ApplFourName.compare(application_snapshot.application_name) == 0) {

              CdmwPlatformMngt::ApplicationStatus application_status =
                                application_snapshot.application_status;
            
              TEST_INFO (ApplFourName+" found !");

              if (!(application_status ==
                        CdmwPlatformMngt::APPLICATION_RUNNING)) {
                
                        result = false;
              } else {

                  std::vector <CdmwPlatformMngt::ProcessStatus> p41_stati;

                  p41_stati.push_back (CdmwPlatformMngt::PROCESS_INITIALISED);
                  p41_stati.push_back (CdmwPlatformMngt::PROCESS_RUN_REQUEST);
                  p41_stati.push_back (CdmwPlatformMngt::PROCESS_RUNNING);

                  result = observer_on_system.check_proc_stati_sequence(system_name,
                      ApplFourName, host1_name, process_41, p41_stati);
          
              } //processes else
         } //APPL4 if           
       } //Applications for cycle

       test_result = test_result && result;

       CPPUNIT_ASSERT(test_result);

       result = true;

       TEST_INFO("Command to run process "+process_41
       +": platform_admin.sh --proc-run "+ApplFourName+" "+process_41+
       " "+host1_name);

       OsSupport::OS::create_process("platform_admin.sh", "--proc-run "+ApplFourName
           +" "+process_41+" "+host1_name);
       
       lets_wait(15000, 15000); 

       TEST_INFO("Command to clear the application degraded mode : platform_admin.sh --app-acknowledge "+ApplFourName);

       OsSupport::OS::create_process("platform_admin.sh", "--app-acknowledge "+ApplFourName);
       
       lets_wait(15000, 15000); 

       TEST_INFO("Command to clear the system degraded mode : platform_admin.sh --app-acknowledge "+ApplFourName);

       OsSupport::OS::create_process("platform_admin.sh", "--sys-acknowledge ");

       lets_wait(15000, 15000); 

       now_snapshot = system->get_snapshot();

       TEST_INFO("Checking "+system_name+" mode ");

       if (!(now_snapshot->system_mode == CdmwPlatformMngt::NORMAL_MODE)) {

                  result = false;
       }

       TEST_INFO("Checking "+ApplFourName+" mode ");

       appls_snapshot = now_snapshot->system_applications;

       for (CORBA::ULong index = 0; index < appls_snapshot.length(); index++) {

         CdmwPlatformMngt::ApplicationSnapshot application_snapshot = 
                                                 appls_snapshot [index];

         if (ApplFourName.compare(application_snapshot.application_name) == 0) {

              CdmwPlatformMngt::FunctioningMode application_mode =
                                       application_snapshot.application_mode;
            
              TEST_INFO (ApplFourName+" found !");

              if (!(application_mode ==
                        CdmwPlatformMngt::NORMAL_MODE)) {
                
                        result = false;
              } 
         } //APPL4 if           
       } //Applications for cycle

       test_result = test_result && result;

       CPPUNIT_ASSERT(test_result);

       TEST_INFO ("Stopping the system ");       

       OsSupport::OS::create_process("stop_system.sh", "");

       lets_wait(15000, 15000); 
       }

       CPPUNIT_ASSERT(test_result);
    } 

    void TestObjs::test_detect_failures () {

        if (&observer_on_system != 0) {

            TEST_INFO(" TEST CASE 4: Detection of failures during system operation ");                       
            /////////////
            // TEST  CASE 4: Detection of failures during system operation 
            /////////////

            bool result = true;

            TEST_INFO ("Failure of an application process ");

            TEST_INFO ("Defining the system: define_system.sh "+Scenario);

            OsSupport::OS::create_process("define_system.sh", Scenario);

            lets_wait(15000, 15000); 

            TEST_INFO ("Starting the system: start_system.sh ");

            OsSupport::OS::create_process("start_system.sh", "");

            lets_wait(15000, 15000); 

            /*

              Find the PID of process P13.

              Open a new window on Host2, log as root and kill the process
              P13 using the operating system command.

            */

           TEST_INFO ("Finding "+process_13+" pid ");

           std::string p_13_pid = "";       

           //TODO: Sistemare 
           ifstream p_13_file;

           p_13_file.open("/tmp/CdmwDaemon_21869/APPL1/P13.pid");

           p_13_file >> p_13_pid;

           p_13_file.close();
    
           if (p_13_pid.compare("") == 0) {
 
                TEST_INFO (process_13+" process pid not found ");

                //TODO: Aggiungere eventuale chiusura del sistema.

                CPPUNIT_ASSERT(false);

           } else {

                 TEST_INFO (process_13+" process found pid is "+p_13_pid);

                 TEST_INFO ("Killing "+process_13+" process ");

                 OsSupport::OS::create_process("kill", " -9 "+p_13_pid);

                 lets_wait(15000, 15000); 

            }

            CdmwPlatformMngt::SystemSnapshot* now_snapshot = system->get_snapshot();

            TEST_INFO("Checking "+system_name+" status ");

            if (!(now_snapshot->system_status == CdmwPlatformMngt::SYSTEM_PARTIAL_STARTED &&
                  now_snapshot->system_mode == CdmwPlatformMngt::DEGRADED_MODE)) {

                  result = false;
            }

            TEST_INFO("Checking "+host2_name+" status");

            CdmwPlatformMngt::HostsSnapshot hosts_snapshot = now_snapshot->system_hosts;

            for (CORBA::ULong host_index = 0; host_index < hosts_snapshot.length(); 
                host_index++) {

                CdmwPlatformMngt::HostSnapshot host_snapshot = hosts_snapshot[host_index];

                if (host2_name.compare(host_snapshot.host_name) == 0) {

                   if (!(host_snapshot.host_status == CdmwPlatformMngt::HOST_PARTIAL_STARTED)) {

                       result = false;
                   }

               }
  
           }            

           TEST_INFO("Checking "+ApplOneName+" status and mode ");

           CdmwPlatformMngt::ApplicationsSnapshot appls_snapshot = 
                                     now_snapshot->system_applications;

           for (CORBA::ULong index = 0; index < appls_snapshot.length(); index++) {

             CdmwPlatformMngt::ApplicationSnapshot application_snapshot = 
                                                       appls_snapshot [index];

                 if (ApplOneName.compare(application_snapshot.application_name) == 0) {

                  CdmwPlatformMngt::ApplicationStatus application_status =
                                     application_snapshot.application_status;

                  CdmwPlatformMngt::FunctioningMode application_mode =
                                       application_snapshot.application_mode;
            
                  TEST_INFO (ApplOneName+" found !");

                  if (!(application_status ==
                        CdmwPlatformMngt::APPLICATION_PARTIAL_RUNNING &&
                        application_mode == CdmwPlatformMngt::DEGRADED_MODE)) {
                
                        result = false;
                  } else {

                      CdmwPlatformMngt::ProcessesSnapshot processes_snapshot =
                                    application_snapshot.application_processes;
             
                      TEST_INFO("Checking "+process_13+" status ");

                      for (CORBA::ULong p_index = 0; p_index < processes_snapshot.length(); 
                          p_index++) {

                          CdmwPlatformMngt::ProcessSnapshot process_snapshot = 
                          processes_snapshot [p_index];                      

                     if (process_13.compare(process_snapshot.process_name) == 0) 
                     {
                         CdmwPlatformMngt::ProcessStatus process_status =
                                                process_snapshot.process_status;
                   
                           TEST_INFO (process_13+" found !");

                       if (!(process_status == CdmwPlatformMngt::PROCESS_FAILED_DEATH)) {
               
                               result = false;
                    } 
               } 
                  } //processes for cycle
              } //processes else
         } //APPL4 if           
       } //Applications for cycle
            
       test_result = result;

       CPPUNIT_ASSERT(test_result);

       result = true;
            
       TEST_INFO ("Command to initialise "+process_13+": platform_admin.sh --proc-initialise "+
       ApplOneName+" "+process_13+" "+host2_name);

       OsSupport::OS::create_process("platform_admin.sh", "--proc-initialise "+
       ApplOneName+" "+process_13+" "+host2_name);

       lets_wait(15000, 15000); 

       TEST_INFO ("Command to initialise "+process_13+" : platform_admin.sh --proc-next-step "+
       ApplOneName+" "+process_13+" "+host2_name);

       OsSupport::OS::create_process("platform_admin.sh", "--proc-next-step "+
       ApplOneName+" "+process_13+" "+host2_name);

       lets_wait(15000, 15000); 

       TEST_INFO ("Command to run "+process_13+" : platform_admin.sh --proc-run "+
       ApplOneName+" "+process_13+" "+host2_name);

       OsSupport::OS::create_process("platform_admin.sh", "--proc-run "+
       ApplOneName+" "+process_13+" "+host2_name);

       lets_wait(15000, 15000); 

       now_snapshot = system->get_snapshot();

       TEST_INFO("Checking "+host2_name+" status");

       hosts_snapshot = now_snapshot->system_hosts;

       for (CORBA::ULong host_index = 0; host_index < hosts_snapshot.length(); 
            host_index++) {

            CdmwPlatformMngt::HostSnapshot host_snapshot = hosts_snapshot[host_index];

            if (host2_name.compare(host_snapshot.host_name) == 0) {

               if (!(host_snapshot.host_status == CdmwPlatformMngt::HOST_STARTED)) {

                  result = false;
               }

            }
   
       }       

       TEST_INFO("Checking "+ApplOneName+" status ");

       appls_snapshot = now_snapshot->system_applications;

       for (CORBA::ULong index = 0; index < appls_snapshot.length(); index++) {

         CdmwPlatformMngt::ApplicationSnapshot application_snapshot = 
                                                 appls_snapshot [index];

         if (ApplOneName.compare(application_snapshot.application_name) == 0) {

              CdmwPlatformMngt::ApplicationStatus application_status =
                                application_snapshot.application_status;
            
              TEST_INFO (ApplOneName+" found !");

              if (!(application_status ==
                        CdmwPlatformMngt::APPLICATION_RUNNING)) {
                
                        result = false;
              } else {

                  std::vector <CdmwPlatformMngt::ProcessStatus> p13_stati;

                  p13_stati.push_back (CdmwPlatformMngt::PROCESS_INITIALISED);
                  p13_stati.push_back (CdmwPlatformMngt::PROCESS_RUNNING);

                  result = observer_on_system.check_proc_stati_sequence(system_name,
                      ApplOneName, host2_name, process_13, p13_stati);
          
              } //processes else
         } //APPL4 if           
       } //Applications for cycle

       test_result = test_result && result;

       CPPUNIT_ASSERT(test_result);

       result = true;

       TEST_INFO ("Command to clear application failure : platform_admin.sh --app-acknowledge "+
       ApplOneName);

       OsSupport::OS::create_process("platform_admin.sh", "--app-acknowledge "+ApplOneName);

       lets_wait(15000, 15000); 

       TEST_INFO ("Command to clear application failure: platform_admin.sh --sys-acknowledge ");

       OsSupport::OS::create_process("platform_admin.sh", "--sys-acknowledge");

       lets_wait(15000, 15000); 

       now_snapshot = system->get_snapshot();

       TEST_INFO("Checking "+system_name+" mode ");

       if (!(now_snapshot->system_mode == CdmwPlatformMngt::NORMAL_MODE)) {

          result = false;
       }

       TEST_INFO("Checking "+ApplOneName+" status and mode ");

       appls_snapshot = now_snapshot->system_applications;

       for (CORBA::ULong index = 0; index < appls_snapshot.length(); index++) {

           CdmwPlatformMngt::ApplicationSnapshot application_snapshot = 
                                                    appls_snapshot [index];

           if (ApplOneName.compare(application_snapshot.application_name) == 0) {
           
              CdmwPlatformMngt::FunctioningMode application_mode =
                                   application_snapshot.application_mode;
            
              TEST_INFO (ApplOneName+" found !");

              if (!(application_mode == CdmwPlatformMngt::NORMAL_MODE)) {
                
                        result = false;
              } 
         } //APPL4 if           
       } //Applications for cycle
            
       test_result = test_result && result;

       CPPUNIT_ASSERT(test_result);

       result = true;

       TEST_INFO ("Failure of an application unmanaged process ");

       TEST_INFO ("The process "+process_31+" is an unmanaged process \n which is not AUTOENDED   so a kill of this process will lead to a failure detection \n by the System Management Service");

/*
      On the last window opened on Host2, kill the process P31 using the 
      operating system command.

      At the time of the writing, the process id is not displayed by the 
      platform daemon when an unmanaged process is called. So, to detect the process id
      use the ps aux and take the one which was launched with --unmanaged argument      
*/      
       TEST_INFO ("Finding "+process_31+" pid ");

       std::string p_31_pid = "";
       
       TEST_INFO ("Copying /tmp/CdmwDaemon_21869/APPL3/P31.pid via scp");

       OsSupport::OS::create_process("copy_pid_file.sh", "");

       lets_wait(10000, 10000); 

       //Read pid from file ./P31.pid 
       ifstream p_31_file;

       p_31_file.open("./P31.pid");

       p_31_file >> p_31_pid;

       p_31_file.close();

       TEST_INFO ("Removing copied file ./"+process_31+".pid");

       OsSupport::OS::create_process("rm", process_31+".pid" );

       lets_wait(2000, 2000); 

       if (p_31_pid.compare("") == 0) {

           TEST_INFO (process_31+" process pid not found ");

           //TODO: Aggiungere eventuale chiusura del sistema.

           CPPUNIT_ASSERT(false);

       } else {

           TEST_INFO (process_31+" process found pid is "+p_31_pid);

           TEST_INFO ("Killing "+process_31+" process ");

           OsSupport::OS::create_process("kill_process_remote.sh", " -9 "+p_31_pid);

           lets_wait(15000, 15000); 

       }

       TEST_INFO ("Initialising "+process_31+" process: platform_admin.sh --proc-initialise "
           +ApplThreeName+" "+process_31+" "+host2_name);

       OsSupport::OS::create_process("platform_admin.sh", "--proc-initialise "
           +ApplThreeName+" "+process_31+" "+host2_name);

       lets_wait(15000, 15000); 

       now_snapshot = system->get_snapshot();

       TEST_INFO("Checking "+system_name+" mode ");

       if (!(now_snapshot->system_mode == CdmwPlatformMngt::DEGRADED_MODE)) {

          result = false;
       }

       TEST_INFO("Checking "+ApplThreeName+" mode ");

       appls_snapshot = now_snapshot->system_applications;

       for (CORBA::ULong index = 0; index < appls_snapshot.length(); index++) {

           CdmwPlatformMngt::ApplicationSnapshot application_snapshot = 
                                                    appls_snapshot [index];

           if (ApplThreeName.compare(application_snapshot.application_name) == 0) {
           
              CdmwPlatformMngt::FunctioningMode application_mode =
                                   application_snapshot.application_mode;
            
              TEST_INFO (ApplThreeName+" found !");

              if (!(application_mode == CdmwPlatformMngt::DEGRADED_MODE)) {
                
                        result = false;
              } else {
          
                  std::vector <CdmwPlatformMngt::ProcessStatus> p31_stati;

                  p31_stati.push_back (CdmwPlatformMngt::PROCESS_FAILED_DEATH);
                  p31_stati.push_back (CdmwPlatformMngt::PROCESS_RUNNING);

                  result = observer_on_system.check_proc_stati_sequence(system_name,
                      ApplThreeName, host2_name, process_31, p31_stati);
          
              }
         } //APPL3 if           
       } //Applications for cycle

       test_result = test_result && result;

       CPPUNIT_ASSERT(test_result);

       result = true;

       TEST_INFO ("Aknowledge "+ApplThreeName+" application: platform_admin.sh --app-acknowledge "
           +ApplThreeName);

       OsSupport::OS::create_process("platform_admin.sh", "--app-acknowledge "
           +ApplThreeName);

       lets_wait(15000, 15000); 

       TEST_INFO ("Aknowledge system: platform_admin.sh --sys-acknowledge ");

       OsSupport::OS::create_process("platform_admin.sh", "--sys-acknowledge ");

       lets_wait(15000, 15000); 

       now_snapshot = system->get_snapshot();

       TEST_INFO("Checking "+system_name+" mode ");

       if (!(now_snapshot->system_mode == CdmwPlatformMngt::NORMAL_MODE)) {

          result = false;
       }

       TEST_INFO("Checking "+ApplThreeName+" mode ");

       appls_snapshot = now_snapshot->system_applications;

       for (CORBA::ULong index = 0; index < appls_snapshot.length(); index++) {

           CdmwPlatformMngt::ApplicationSnapshot application_snapshot = 
                                                    appls_snapshot [index];

           if (ApplThreeName.compare(application_snapshot.application_name) == 0) {
           
              CdmwPlatformMngt::FunctioningMode application_mode =
                                   application_snapshot.application_mode;
            
              TEST_INFO (ApplThreeName+" found !");

              if (!(application_mode == CdmwPlatformMngt::NORMAL_MODE)) {
                
                        result = false;
              } 
         } //APPL3 if           
       } //Applications for cycle

       test_result = test_result && result;

       CPPUNIT_ASSERT(test_result);

       result = true;

       TEST_INFO ("Exit of an application unmanaged AUTOENDED process ");

       TEST_INFO ("The process "+process_21+" is an unmanaged process which is AUTOENDED \n so a kill of this process will simulate a proper exit of this process \n and will lead to exit detection by the System Management Service ");
      
/*
      Open a new window on HOST1, log as root and kill the process P21 using the operating system command.

      At the time of the writing, the process id is not displayed by the 
      platform daemon when an unmanaged process is called. So, to detect the process id
      use the ps aux and take the one which was launched with --unmanaged argument 
*/
       TEST_INFO ("Finding "+process_21+" pid ");

       std::string p_21_pid = "";

       //Read pid from file /tmp/CdmwDaemon_21869/APPL2/P21.pid 
       ifstream p_21_file;

       p_21_file.open("/tmp/CdmwDaemon_21869/APPL2/P21.pid");

       p_21_file >> p_21_pid;

       p_21_file.close();


       if (p_21_pid.compare("") == 0) {

           TEST_INFO (process_21+" process pid not found ");

           //TODO: Aggiungere eventuale chiusura del sistema.

           CPPUNIT_ASSERT(false);

       } else {

           TEST_INFO (process_21+" process found pid is "+p_21_pid);

           TEST_INFO ("Killing "+process_21+" process ");

           OsSupport::OS::create_process("kill", " -9 "+p_21_pid);

           lets_wait(15000, 15000); 

       }


       TEST_INFO ("Initialising "+process_21+" process again : platform_admin.sh --proc-initialise "+ApplTwoName+" "+process_21+" "+host1_name);

       OsSupport::OS::create_process("platform_admin.sh", "--proc-initialise "
           +ApplTwoName+" "+process_21+" "+host1_name);

       lets_wait(15000, 15000); 

       std::vector <CdmwPlatformMngt::ProcessStatus> p21_stati;

       p21_stati.push_back (CdmwPlatformMngt::PROCESS_ENDED);
       p21_stati.push_back (CdmwPlatformMngt::PROCESS_RUNNING);

       result = observer_on_system.check_proc_stati_sequence(system_name,
                      ApplTwoName, host1_name, process_21, p21_stati);

       test_result = test_result && result;

       CPPUNIT_ASSERT(test_result);

       result = true;

       TEST_INFO ("Failure of a host "+host2_name);

/*
   kill the process implementing the System Management daemon running on host2
using an operating system command.
*/
       TEST_INFO(" Stopping Daemon on "+host2_name); 
       
       //Stopping platform daemon on HOST2       
       OsSupport::OS::create_process("stop_daemon_remote.sh", "");

       lets_wait(15000, 15000);

       now_snapshot = system->get_snapshot();

       TEST_INFO("Checking "+system_name+" mode ");

       if (!(now_snapshot->system_mode == CdmwPlatformMngt::DEGRADED_MODE)) {

          result = false;
       }

       TEST_INFO("Checking "+host2_name+" status");

       hosts_snapshot = now_snapshot->system_hosts;

       for (CORBA::ULong host_index = 0; host_index < hosts_snapshot.length(); 
            host_index++) {

            CdmwPlatformMngt::HostSnapshot host_snapshot = hosts_snapshot[host_index];

            if (host2_name.compare(host_snapshot.host_name) == 0) {

               if (!(host_snapshot.host_status == CdmwPlatformMngt::HOST_UNREACHABLE)) {

                  result = false;
               }

            }
   
       }       

       TEST_INFO("Checking "+ApplOneName+" status ");

       appls_snapshot = now_snapshot->system_applications;

       for (CORBA::ULong index = 0; index < appls_snapshot.length(); index++) {

         CdmwPlatformMngt::ApplicationSnapshot application_snapshot = 
                                                 appls_snapshot [index];

         if (ApplOneName.compare(application_snapshot.application_name) == 0 ||
             ApplThreeName.compare(application_snapshot.application_name) == 0) {

              CdmwPlatformMngt::FunctioningMode application_mode =
                                   application_snapshot.application_mode;           

              if (!(application_mode ==
                        CdmwPlatformMngt::DEGRADED_MODE)) {
                
                        result = false;
              } else {

                  std::vector <CdmwPlatformMngt::ProcessStatus> p_stati;

                  p_stati.push_back (CdmwPlatformMngt::PROCESS_FAILED_INVALID);

                  result = observer_on_system.check_proc_stati_sequence(system_name,
                      ApplOneName, host2_name, process_13, p_stati);

                  result = result && observer_on_system.check_proc_stati_sequence(system_name,
                      ApplOneName, host2_name, process_14, p_stati);

                  result = result && observer_on_system.check_proc_stati_sequence(system_name,
                      ApplThreeName, host2_name, process_31, p_stati);

                  result = result && observer_on_system.check_proc_stati_sequence(system_name,
                      ApplThreeName, host2_name, process_32, p_stati);
                           
              } //processes else
         } //APPL4 if           
       } //Applications for cycle

       test_result = test_result && result;

       CPPUNIT_ASSERT(test_result);

       result = true;

       //Restart platform daemon on HOST2
       OsSupport::OS::create_process("start_daemon_remote.sh", "");

       TEST_INFO ("Initialising processes "+process_13+" and "+process_14); 

       TEST_INFO ("Initialising "+process_13+" process: platform_admin.sh --proc-initialise "
           +ApplOneName+" "+process_13+" "+host2_name);

       OsSupport::OS::create_process("platform_admin.sh", "--proc-initialise "
           +ApplOneName+" "+process_13+" "+host2_name);

       TEST_INFO ("Initialising "+process_14+" process: platform_admin.sh --proc-initialise "
           +ApplOneName+" "+process_14+" "+host2_name);

       OsSupport::OS::create_process("platform_admin.sh", "--proc-initialise "
           +ApplOneName+" "+process_14+" "+host2_name);

       lets_wait(15000, 15000); 

       TEST_INFO ("Initialising "+process_13+" process: platform_admin.sh --proc-next-step "
           +ApplOneName+" "+process_13+" "+host2_name);

       OsSupport::OS::create_process("platform_admin.sh", "--proc-next-step "
           +ApplOneName+" "+process_13+" "+host2_name);

       TEST_INFO ("Initialising "+process_14+" process: platform_admin.sh --proc-next-step "
           +ApplOneName+" "+process_14+" "+host2_name);

       OsSupport::OS::create_process("platform_admin.sh", "--proc-next-step "
           +ApplOneName+" "+process_14+" "+host2_name);

       lets_wait(15000, 15000); 

       TEST_INFO ("Running "+process_13+" process: platform_admin.sh --proc-run "
           +ApplOneName+" "+process_13+" "+host2_name);

       OsSupport::OS::create_process("platform_admin.sh", "--proc-run "
           +ApplOneName+" "+process_13+" "+host2_name);

       TEST_INFO ("Running "+process_14+" process: platform_admin.sh --proc-run "
           +ApplOneName+" "+process_14+" "+host2_name);

       OsSupport::OS::create_process("platform_admin.sh", "--proc-run "
           +ApplOneName+" "+process_14+" "+host2_name);

       lets_wait(15000, 15000); 

       TEST_INFO ("Acknowledge "+ApplOneName+" application: platform_admin.sh --app-acknowledge "
           +ApplOneName);

       OsSupport::OS::create_process("platform_admin.sh", "--app-acknowledge "+ApplOneName);

       TEST_INFO ("Initialising "+ApplThreeName+" application by stopping and restarting it");

       TEST_INFO ("Initialising "+ApplThreeName+" application: platform_admin.sh --app-stop "
           +ApplThreeName);

       OsSupport::OS::create_process("platform_admin.sh", "--app-stop "+ApplThreeName);

       TEST_INFO ("Initialising "+ApplThreeName+" application: platform_admin.sh --app-initialise-and-wait "+ApplThreeName);

       OsSupport::OS::create_process("platform_admin.sh", "--app-initialise-and-wait "
           +ApplThreeName);

       lets_wait(15000, 15000); 

       TEST_INFO ("Running "+ApplThreeName+" application: platform_admin.sh --app-run "
           +ApplThreeName);

       OsSupport::OS::create_process("platform_admin.sh", "--app-run "
           +ApplThreeName);

       lets_wait(15000, 15000); 

       TEST_INFO ("Aknowledge "+ApplThreeName+" application: platform_admin.sh --app-acknowledge "
           +ApplThreeName);

       OsSupport::OS::create_process("platform_admin.sh", "--app-acknowledge "
           +ApplThreeName);

       lets_wait(15000, 15000); 

       TEST_INFO ("Aknowledge system: platform_admin.sh --sys-acknowledge ");

       OsSupport::OS::create_process("platform_admin.sh", "--sys-acknowledge ");

       lets_wait(15000, 15000); 

       now_snapshot = system->get_snapshot();

       TEST_INFO("Checking "+system_name+" mode ");

       if (!(now_snapshot->system_mode == CdmwPlatformMngt::NORMAL_MODE)) {

          result = false;
       }

       TEST_INFO("Checking "+host2_name+" status");

       hosts_snapshot = now_snapshot->system_hosts;

       for (CORBA::ULong host_index = 0; host_index < hosts_snapshot.length(); 
            host_index++) {

            CdmwPlatformMngt::HostSnapshot host_snapshot = hosts_snapshot[host_index];

            if (host2_name.compare(host_snapshot.host_name) == 0) {

               if (!(host_snapshot.host_status == CdmwPlatformMngt::HOST_RESPONDING)) {

                  result = false;
               }

            }
   
       }       

       TEST_INFO("Checking "+ApplOneName+" and "+ApplThreeName+" mode ");

       appls_snapshot = now_snapshot->system_applications;

       for (CORBA::ULong index = 0; index < appls_snapshot.length(); index++) {

         CdmwPlatformMngt::ApplicationSnapshot application_snapshot = 
                                                 appls_snapshot [index];

         if (ApplOneName.compare(application_snapshot.application_name) == 0 ||
             ApplThreeName.compare(application_snapshot.application_name) == 0) {

              CdmwPlatformMngt::FunctioningMode application_mode =
                                   application_snapshot.application_mode;           

              if (!(application_mode == CdmwPlatformMngt::NORMAL_MODE)) {
                
                        result = false;
              } else {

                  std::vector <CdmwPlatformMngt::ProcessStatus> p_One_stati;
                  std::vector <CdmwPlatformMngt::ProcessStatus> p_Three_stati;

                  p_One_stati.push_back (CdmwPlatformMngt::PROCESS_INITIALISING);
                  p_One_stati.push_back (CdmwPlatformMngt::PROCESS_STEP_PERFORMED);
                  p_One_stati.push_back (CdmwPlatformMngt::PROCESS_INITIALISED);
                  p_One_stati.push_back (CdmwPlatformMngt::PROCESS_RUN_REQUEST);
                  p_One_stati.push_back (CdmwPlatformMngt::PROCESS_RUNNING);

                  p_Three_stati.push_back (CdmwPlatformMngt::PROCESS_INITIALISING);
                  p_Three_stati.push_back (CdmwPlatformMngt::PROCESS_INITIALISED);
                  p_Three_stati.push_back (CdmwPlatformMngt::PROCESS_RUN_REQUEST);
                  p_Three_stati.push_back (CdmwPlatformMngt::PROCESS_RUNNING);

                  result = observer_on_system.check_proc_stati_sequence(system_name,
                      ApplOneName, host2_name, process_13, p_One_stati);

                  result = result && observer_on_system.check_proc_stati_sequence(system_name,
                      ApplOneName, host2_name, process_14, p_One_stati);

                  result = result && observer_on_system.check_proc_stati_sequence(system_name,
                      ApplThreeName, host2_name, process_31, p_Three_stati);

                  result = result && observer_on_system.check_proc_stati_sequence(system_name,
                      ApplThreeName, host2_name, process_32, p_Three_stati);
                           
              } //processes else
         } //APPL4 if           
       } //Applications for cycle

       test_result = test_result && result;

       CPPUNIT_ASSERT(test_result);

       TEST_INFO ("Stopping the system: stop_system.sh");

       OsSupport::OS::create_process("stop_system.sh", "");

       }

       CPPUNIT_ASSERT(test_result);
    }

    void TestObjs::test_errors_and_stop() {

         if (&observer_on_system != 0) {

         TEST_INFO(" TEST  CASE 5: Detection of errors in the application stop sequences");

         /////////////
         // TEST  CASE 5: Detection of errors in the application stop sequences
         /////////////                                

         bool result = true;

         TEST_INFO ("Defining the system: define_system.sh "+Scenario);

         OsSupport::OS::create_process("define_system.sh", Scenario);

         lets_wait(15000, 15000); 

         TEST_INFO ("Starting the system: start_system.sh ");

         OsSupport::OS::create_process("start_system.sh", "");

        lets_wait(15000, 15000); 

        TEST_INFO ("Adding another application: platform_admin.sh --sys-add-application "
        +Third_Scenario);

         OsSupport::OS::create_process("platform_admin.sh", "--sys-add-application "
        +Third_Scenario);

         lets_wait(15000, 15000); 
         
         TEST_INFO ("Stop order not acknowledge within the requested time-out ");

         TEST_INFO ("Commands to shorten the stop time-out of process "+process_41);

         TEST_INFO ("Command: platform_admin.sh --proc-set-life-cycle "
             +ApplFourName+" "+process_41+" "+host1_name+" "+Seventh_Scenario);

         OsSupport::OS::create_process("platform_admin.sh", "--proc-set-life-cycle "
             +ApplFourName+" "+process_41+" "+host1_name+" "+Seventh_Scenario);

         lets_wait(15000, 15000); 

         TEST_INFO ("Command: platform_admin.sh --app-initialise "
             +ApplFourName+" LAST_STATE 10000 nothing");

         OsSupport::OS::create_process("platform_admin.sh", "--app-initialise "
             +ApplFourName+" LAST_STATE 10000 nothing");

         lets_wait(15000, 15000); 

         TEST_INFO ("Command: platform_admin.sh --app-run "+ApplFourName);

         OsSupport::OS::create_process("platform_admin.sh", "--app-run "+ApplFourName);

         lets_wait(15000, 15000); 

         TEST_INFO ("Command top stop "+ApplFourName+" : platform_admin.sh --app-stop "
             +ApplFourName);

         OsSupport::OS::create_process("platform_admin.sh", "--app-stop "+ApplFourName);

         lets_wait(15000, 15000); 

         std::vector <CdmwPlatformMngt::ProcessStatus> p41_stati;

         p41_stati.push_back (CdmwPlatformMngt::PROCESS_STOPPING);
         p41_stati.push_back (CdmwPlatformMngt::PROCESS_STOPPED);

         std::vector <CdmwPlatformMngt::ApplicationStatus> app4_stati;
         app4_stati.push_back (CdmwPlatformMngt::APPLICATION_STOPPED);

         result = observer_on_system.check_proc_stati_sequence(system_name,
                     ApplFourName, host1_name, process_41, p41_stati);

         result = result && observer_on_system.check_appl_stati_sequence(system_name,
                     ApplFourName, app4_stati);

         test_result = result; 

     }

           CPPUNIT_ASSERT(test_result);     
    }

    void TestObjs::test_stop_unmanaged() {

         if (&observer_on_system != 0) {

         TEST_INFO(" TEST  CASE 6: Start and stop an Unix unmanaged process ");    

         /////////////
         // TEST  CASE 6: Start and stop an Unix unmanaged process
         /////////////                                

         bool result = true;

         TEST_INFO ("Defining the system: define_system.sh "+Scenario);

         OsSupport::OS::create_process("define_system.sh", Scenario);

         lets_wait(15000, 15000); 

         TEST_INFO ("Starting the system: start_system.sh ");

         OsSupport::OS::create_process("start_system.sh", "");

         lets_wait(15000, 15000); 

         TEST_INFO ("Insertion of the "+process_15+" process, an UNIX unmanaged process, in the "
             +ApplOneName+" configuration and start of this process");

         TEST_INFO ("Command to insert process "+process_15+" in the application configuration: \n platform_admin.sh --app-add-process "+ApplOneName+" "+Eighth_Scenario);

         OsSupport::OS::create_process("platform_admin.sh", "--app-add-process "
             +ApplOneName+" "+Eighth_Scenario);

         lets_wait(15000, 15000); 

         TEST_INFO ("Command to initialise the unmanaged process "+process_15+":  \n platform_admin.sh --proc-initialise "+ApplOneName+" "+process_15+" "+host1_name);

         OsSupport::OS::create_process("platform_admin.sh", "--proc-initialise "
             +ApplOneName+" "+process_15+" "+host1_name);
         
         lets_wait(15000, 15000); 

         TEST_INFO ("Command to stop the "+process_15+" process: platform_admin.sh --proc-stop "
             +ApplOneName+" "+process_15+" "+host1_name);

         OsSupport::OS::create_process("platform_admin.sh", "--proc-stop "
             +ApplOneName+" "+process_15+" "+host1_name);
         
         lets_wait(15000, 15000); 

         std::vector <CdmwPlatformMngt::ProcessStatus> p15_stati;

         p15_stati.push_back (CdmwPlatformMngt::PROCESS_INITIALISING);
         p15_stati.push_back (CdmwPlatformMngt::PROCESS_RUNNING);
         p15_stati.push_back (CdmwPlatformMngt::PROCESS_STOPPED);
         
         result = observer_on_system.check_proc_stati_sequence(system_name,
                     ApplOneName, host1_name, process_15, p15_stati);

         test_result = result; 

         TEST_INFO ("Stopping the system: stop_system.sh"); 

         OsSupport::OS::create_process("stop_system.sh", "");
         
         lets_wait(15000, 15000); 

         OsSupport::OS::create_process("cdmw_host_stop.sh", "");

         //Stop platform daemon on HOST2
         OsSupport::OS::create_process("stop_daemon_remote.sh", "");

     }

           CPPUNIT_ASSERT(test_result);     
    }

    void TestObjs::lets_wait(int before, int after) {

         OsSupport::OS::sleep (before);

         TEST_INFO ("Please wait ...");

         OsSupport::OS::sleep (after);

    }

}/* End namespace PlatformMngt */
}/* End namespace Cdmw */





