/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_PLATFORMMNGT_SUPERVISION_OBSERVER_HPP
#define INCL_PLATFORMMNGT_SUPERVISION_OBSERVER_HPP

#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "Foundation/osthreads/MutexGuard.hpp"

#include <deque>
#include <vector>
#include <string>

namespace Cdmw
  {        // Begin namespace Cdmw

  namespace PlatformMngt
    { // Begin namespace PlatformMngt

    struct Sys_event {

        const char* sys_path;
   
        CdmwPlatformMngt::SystemStatus sys_status;   

    };

    struct Appl_event {

        const char* appl_path;
   
        CdmwPlatformMngt::ApplicationStatus appl_status;   

    };
    
    struct Proc_event {

        const char* proc_path;
   
        CdmwPlatformMngt::ProcessStatus proc_status;   

    };   

    class ProcObserver_impl: public virtual POA_CdmwPlatformMngt::SupervisionObserver,
          public virtual PortableServer::RefCountServantBase
      {        

        public:        
        
        ProcObserver_impl();

        virtual ~ProcObserver_impl();
        
        void notify( CdmwPlatformMngt::Event* an_event )
        throw( CORBA::SystemException );
        
        bool check_sys_stati_sequence(std::string system_name,         
            std::vector<CdmwPlatformMngt::SystemStatus> sys_stati) 
        throw( CORBA::SystemException ); 
        
    bool check_appl_stati_sequence(std::string system_name, 
            std::string application_name, std::vector<CdmwPlatformMngt::ApplicationStatus> appl_stati) 
        throw( CORBA::SystemException );

    bool check_proc_stati_sequence(std::string system_name,
             std::string application_name, std::string host_name, std::string process_name, 
             std::vector<CdmwPlatformMngt::ProcessStatus> proc_stati) 
         throw( CORBA::SystemException );

        void sys_changes_queue_flush () throw( CORBA::SystemException );
    void app_changes_queue_flush () throw( CORBA::SystemException );
    void pro_changes_queue_flush () throw( CORBA::SystemException );
        void all_queue_flush () throw( CORBA::SystemException );        

      private:

        std::deque<Sys_event>* m_sys_changes_queue;
        std::deque<Appl_event>* m_appl_changes_queue;
        std::deque<Proc_event>* m_proc_changes_queue;

        OsSupport::Mutex m_sys_mutex;
        OsSupport::Mutex m_app_mutex;
        OsSupport::Mutex m_pro_mutex;                               
      };

  } // End namespace PlatformMngt
} // End namespace Cdmw

#endif //INCL_PLATFORMMNGT_SUPERVISION_OBSERVER_HPP

