/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <fstream>
#include <limits>

#include "test_Q-048/TestObjs.hpp"
#include "test_Q-048/ProcObserver_impl.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"

#include <SystemMngt/platformlibrary/Iterator.hpp>
#include <SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp>
#include <idllib/CdmwPlatformMngtSystem.stub.hpp>
#include <idllib/CdmwPlatformMngtProcess.stub.hpp>
#include <idllib/CdmwPlatformMngtCommon.stub.hpp>
#include <idllib/CdmwPlatformMngtApplicationAgent.stub.hpp>


#include <SystemMngt/platformvaluetypes/HostStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/ApplicationStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/EntityStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/SystemStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/SystemModeChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/ApplicationModeChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/PropertyChange_impl.hpp>


using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestObjs);

namespace Cdmw
{
  namespace PlatformMngt
  {       

    TestObjs::TestObjs()
    {
        test_result = false;

        ApplFiveName = "APPL5";
        ApplFourName = "APPL4";
        ApplThreeName = "APPL3";
        ApplTwoName = "APPL2";
        ApplOneName = "APPL1";

        process_51 = "P51";

        process_41 = "P41";
        process_42 = "P42";

        process_31 = "P31";        
        process_32 = "P32";

        process_21 = "P21";
        process_22 = "P22";
        process_23 = "P23";

        process_11 = "P11";
        process_12 = "P12";
        process_13 = "P13";
        process_14 = "P14";

        s1_scenario = "Q-048-1-1.xml";
        s2_scenario = "Q-048-1-2.xml";
    }

    void TestObjs::setUp ()
    {
       try
       {         
        //Getting system var
        std::string system_corbaloc_s1 = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";

        std::string system_corbaloc_s2 = "corbaloc::localhost:21969/CdmwPlatformMngtSupervision";

        orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::instance ()->get_ORB ());            
        CORBA::Object_var system_obj_s1 = orb->string_to_object (system_corbaloc_s1.c_str());  
        CORBA::Object_var system_obj_s2 = orb->string_to_object (system_corbaloc_s2.c_str());

        system_s1 = CdmwPlatformMngt::System::_narrow (system_obj_s1.in ());
        system_s2 = CdmwPlatformMngt::System::_narrow (system_obj_s2.in ());                 
        //Setting up the timeouts
        long_timeout  = 4 ; 
        short_timeout = 1 ;

        CORBA::ValueFactoryBase_var factory, old_factory;
        std::cerr << "Inizializzazione factories " << std::endl;
 
        factory = new ApplicationStatusChangeFactory;
         old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationStatusChange:1.0",factory.in() );
            factory = new ProcessStatusChangeFactory;
         old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/ProcessStatusChange:1.0",factory.in() );
        factory = new SystemStatusChangeFactory;
        old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/SystemStatusChange:1.0",factory.in() );
        factory = new HostStatusChangeFactory;
        old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/HostStatusChange:1.0",factory.in() );

       CdmwPlatformMngt::SystemSnapshot* now_snapshot_s1 = system_s1->get_snapshot();
       system_name_s1 = now_snapshot_s1->system_name;

       CdmwPlatformMngt::SystemSnapshot* now_snapshot_s2 = system_s2->get_snapshot();
       system_name_s2 = now_snapshot_s2->system_name;

       //systems names are not read during the first execution of this method
       //because scenarios are not defined.
       TEST_INFO("First System Name is: "+system_name_s1);
       TEST_INFO("Second System Name is: "+system_name_s2);

       observer_sys_s1 = observer_on_system_s1._this();
 
       TEST_INFO(" Registered Observers on System "+system_name_s1+": observer "); 

       system_s1 ->register_observer("observer", observer_sys_s1.in ());

       observer_sys_s2 = observer_on_system_s2._this();
 
       TEST_INFO(" Registered Observers on System "+system_name_s2+": observer"); 

       system_s2 ->register_observer("observer", observer_sys_s2.in ());


      }
       catch(...)
        {

           std::cerr << "Exception during test startup ! " << std::endl; 
           CPPUNIT_ASSERT(false);          
        }
    }

    void TestObjs::s1_startup () {

       /////////////
       // TEST  CASE 1: System 1 Start-up 
       /////////////

       TEST_INFO(" TEST  CASE 1: "+system_name_s1+" Start-up"); 
       
       TEST_INFO("Defining System "+system_name_s1); 
       TEST_INFO("Executing command: cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --sys-define "+s1_scenario); 

       OsSupport::OS::create_process("cdmw_platform_admin", "--system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --sys-define "+s1_scenario); 

       TEST_INFO("Please wait ..."); 

       OsSupport::OS::sleep (15000);

       TEST_INFO("Please wait ..."); 

       OsSupport::OS::sleep (15000);

       TEST_INFO("Starting System "+system_name_s1); 
       TEST_INFO("Executing command: cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --sys-start"); 

       OsSupport::OS::create_process("cdmw_platform_admin", "--system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --sys-start"); 
       
       TEST_INFO("Please wait ..."); 

       OsSupport::OS::sleep (15000);

       TEST_INFO("Please wait ...");

       OsSupport::OS::sleep (15000);

       //TERM2: Applications APPL1, APPL2 & APPL3 take the following successive statuses: INITIALISING, 
       //INITIALISED, RUN_REQUEST and RUNNING.
       //TERM1: They all have the NORMAL mode:
       //system_snapshot.sh
       //TERM2: Their processes take the following successive statuses: 
       //-  P11 INITIALISING (Step0), STEP_PERFORMED (Step0), INITIALISING (Step1), INITIALISED, RUN_REQUEST and RUNNING
       //-  P12 INITIALISING (Step0), INITIALISED, RUN_REQUEST and RUNNING.
       //-  P13 INITIALISING (Step0), STEP_PERFORMED (Step0), INITIALISING (Step1), INITIALISED, RUN_REQUEST and RUNNING
       //-  P14 INITIALISING (Step0), STEP_PERFORMED (Step0), INITIALISING (Step1), INITIALISED, RUN_REQUEST and RUNNING
       //-  P21 (autoending process) INITIALISING (Step0), INITIALISED, RUN_REQUEST, RUNNING and ENDED
       //-  P22 INITIALISING (Step0), INITIALISED, RUN_REQUEST and RUNNING
       //-  P23 INITIALISING (Step0), INITIALISED, RUN_REQUEST and RUNNING
       //-  P31 (unmanaged process) INITIALISING and RUNNING
       //-  P32 INITIALISING (Step0), INITIALISED, RUN_REQUEST and RUNNING
       //Once the system has been started up, it leaves the STARTING status to enter the STARTED status. 
       //Its mode is set to NORMAL. 
       //Applications stati checking

       bool result = true;

       CdmwPlatformMngt::SystemSnapshot* now_snapshot = system_s1->get_snapshot();
       system_name_s1 = now_snapshot->system_name;

       TEST_INFO("Checking "+system_name_s1+" applications and processes status");

       CdmwPlatformMngt::ApplicationsSnapshot appls_snapshot = now_snapshot->system_applications;

       for (CORBA::ULong index = 0; index < appls_snapshot.length(); index++) {

       CdmwPlatformMngt::ApplicationSnapshot application_snapshot = appls_snapshot [index];

         if (ApplOneName.compare(application_snapshot.application_name) == 0 ||
             ApplTwoName.compare(application_snapshot.application_name) == 0 ||
             ApplThreeName.compare(application_snapshot.application_name) == 0) {

             CdmwPlatformMngt::ApplicationStatus application_status = application_snapshot.application_status;

         if (ApplOneName.compare(application_snapshot.application_name) == 0) 
             TEST_INFO (ApplOneName+" found !");

         if (ApplTwoName.compare(application_snapshot.application_name) == 0)
             TEST_INFO (ApplTwoName+" found !");

         if (ApplThreeName.compare(application_snapshot.application_name) == 0)
             TEST_INFO (ApplThreeName+" found !");

             if (!(application_status == CdmwPlatformMngt::APPLICATION_INITIALISING ||
                 application_status == CdmwPlatformMngt::APPLICATION_INITIALISED ||
                 application_status == CdmwPlatformMngt::APPLICATION_RUN_REQUEST || 
                 application_status == CdmwPlatformMngt::APPLICATION_RUNNING)) {
     
                 result = false;
             } else {
 
             CdmwPlatformMngt::ProcessesSnapshot processes_snapshot = application_snapshot.application_processes;
     
             for (CORBA::ULong p_index = 0; p_index < processes_snapshot.length(); p_index++) {
         
                     CdmwPlatformMngt::ProcessSnapshot process_snapshot = processes_snapshot [p_index];                      
   
                 if (process_11.compare(process_snapshot.process_name) == 0) 
                     {
                         CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
         
                           TEST_INFO (process_11+" found !");

                       if (!(process_status == CdmwPlatformMngt::PROCESS_INITIALISING || 
                           process_status == CdmwPlatformMngt::PROCESS_INITIALISED || 
                           process_status == CdmwPlatformMngt::PROCESS_STEP_PERFORMED || 
                           process_status == CdmwPlatformMngt::PROCESS_RUN_REQUEST || 
                           process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {
   
                               result = false;
                    } 
               } 
               else if (process_12.compare(process_snapshot.process_name) == 0) 
               {
                    CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
       
                            TEST_INFO (process_12+" found !");

                            if (!(process_status == CdmwPlatformMngt::PROCESS_INITIALISING ||
                                process_status == CdmwPlatformMngt::PROCESS_INITIALISED ||
                                process_status == CdmwPlatformMngt::PROCESS_RUN_REQUEST ||
                                process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {

                                result = false;
                            } 
                       }
                       else if (process_13.compare(process_snapshot.process_name) == 0) 
                       {
                            CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;

                            TEST_INFO (process_13+" found !");
             
                            if (!(process_status == CdmwPlatformMngt::PROCESS_INITIALISING || 
                        process_status == CdmwPlatformMngt::PROCESS_INITIALISED || 
                        process_status == CdmwPlatformMngt::PROCESS_STEP_PERFORMED || 
                        process_status == CdmwPlatformMngt::PROCESS_RUN_REQUEST || 
                        process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {
               
                        result = false;
                    } 
                }
                else if (process_14.compare(process_snapshot.process_name) == 0) 
                {
                    CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                           TEST_INFO (process_14+" found !");

                   if (!(process_status == CdmwPlatformMngt::PROCESS_INITIALISING || 
                       process_status == CdmwPlatformMngt::PROCESS_INITIALISED || 
                       process_status == CdmwPlatformMngt::PROCESS_STEP_PERFORMED || 
                       process_status == CdmwPlatformMngt::PROCESS_RUN_REQUEST || 
                       process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {
               
                       result = false;
                   } 
                }
                else if (process_21.compare(process_snapshot.process_name) == 0) 
                {
                    CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                           TEST_INFO (process_21+" found !");

                   if (!(process_status == CdmwPlatformMngt::PROCESS_INITIALISING ||
                       process_status == CdmwPlatformMngt::PROCESS_INITIALISED ||
                       process_status == CdmwPlatformMngt::PROCESS_RUN_REQUEST ||
                       process_status == CdmwPlatformMngt::PROCESS_RUNNING ||
                       process_status == CdmwPlatformMngt::PROCESS_ENDED)) {

                       result = false;             
                   }           
                 }
                         else if (process_22.compare(process_snapshot.process_name) == 0) 
                 {
                             CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                           TEST_INFO (process_22+" found !");

                             if (!(process_status == CdmwPlatformMngt::PROCESS_INITIALISING ||
                         process_status == CdmwPlatformMngt::PROCESS_INITIALISED ||
                         process_status == CdmwPlatformMngt::PROCESS_RUN_REQUEST ||
                         process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {

                        result = false;            
                     }         
                  }
                          else if (process_23.compare(process_snapshot.process_name) == 0) 
                  {
                        CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                           TEST_INFO (process_23+" found !");

                                if (!(process_status == CdmwPlatformMngt::PROCESS_INITIALISING ||
                            process_status == CdmwPlatformMngt::PROCESS_INITIALISED ||
                            process_status == CdmwPlatformMngt::PROCESS_RUN_REQUEST ||
                            process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {
  
                            result = false;            
                         }         
                   }
                           else if (process_31.compare(process_snapshot.process_name) == 0) 
                           {
                               CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;

                           TEST_INFO (process_31+" found !");
               
                               if (!(process_status == CdmwPlatformMngt::PROCESS_INITIALISING ||
                           process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {

                        result = false;            
                             }         

                       }
                       else if (process_32.compare(process_snapshot.process_name) == 0) 
                           {
                                CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;

                           TEST_INFO (process_32+" found !");
               
                                if (!(process_status == CdmwPlatformMngt::PROCESS_INITIALISING ||
                            process_status == CdmwPlatformMngt::PROCESS_INITIALISED ||
                                    process_status == CdmwPlatformMngt::PROCESS_RUN_REQUEST ||
                            process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {

                         result = false;               
                                }          
                           }                 
                    }    
                  }
              }
       }

       test_result = result;

       std::vector <CdmwPlatformMngt::SystemStatus> s1_stati;

       s1_stati.push_back (CdmwPlatformMngt::SYSTEM_STARTING);
       s1_stati.push_back (CdmwPlatformMngt::SYSTEM_STARTED);

       result = observer_on_system_s1.check_sys_stati_sequence(system_name_s1, s1_stati);

       CdmwPlatformMngt::SystemSnapshot* now_snapshot_s2 = system_s2->get_snapshot();

       CdmwPlatformMngt::SystemStatus status_s2 = now_snapshot_s2->system_status;

       if (!(status_s2 == CdmwPlatformMngt::SYSTEM_INITIAL)) {

        result = false;

       }


       CPPUNIT_ASSERT(test_result);
    }

  void TestObjs::s2_startup () {

       /////////////
       // TEST  CASE 2: System 2 Start-up 
       /////////////

       TEST_INFO(" TEST  CASE 2: "+system_name_s2+" Start-up"); 
       
       TEST_INFO("Defining System "+system_name_s2); 
       TEST_INFO("Executing command: cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21969/CdmwPlatformMngtSupervision --sys-define "+s2_scenario); 

       OsSupport::OS::create_process("cdmw_platform_admin", "--system-corbaloc=corbaloc::localhost:21969/CdmwPlatformMngtSupervision --sys-define "+s2_scenario); 

       TEST_INFO("Please wait ..."); 

       OsSupport::OS::sleep (15000);

       TEST_INFO("Please wait ..."); 

       OsSupport::OS::sleep (15000);

       TEST_INFO("Starting System "+system_name_s2); 
       TEST_INFO("Executing command: cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21969/CdmwPlatformMngtSupervision --sys-start"); 

       OsSupport::OS::create_process("cdmw_platform_admin", "--system-corbaloc=corbaloc::localhost:21969/CdmwPlatformMngtSupervision --sys-start"); 
       
       TEST_INFO("Please wait ..."); 

       OsSupport::OS::sleep (15000);

       TEST_INFO("Please wait ..."); 

       OsSupport::OS::sleep (15000);

       //TERM2: Applications APPL1, APPL2 & APPL3 take the following successive statuses: INITIALISING, 
       //INITIALISED, RUN_REQUEST and RUNNING.
       //TERM1: They all have the NORMAL mode:
       //system_snapshot.sh
       //TERM2: Their processes take the following successive statuses: 
       //-  P11 INITIALISING (Step0), STEP_PERFORMED (Step0), INITIALISING (Step1), INITIALISED, RUN_REQUEST and RUNNING
       //-  P12 INITIALISING (Step0), INITIALISED, RUN_REQUEST and RUNNING.
       //-  P13 INITIALISING (Step0), STEP_PERFORMED (Step0), INITIALISING (Step1), INITIALISED, RUN_REQUEST and RUNNING
       //-  P14 INITIALISING (Step0), STEP_PERFORMED (Step0), INITIALISING (Step1), INITIALISED, RUN_REQUEST and RUNNING
       //-  P21 (autoending process) INITIALISING (Step0), INITIALISED, RUN_REQUEST, RUNNING and ENDED
       //-  P22 INITIALISING (Step0), INITIALISED, RUN_REQUEST and RUNNING
       //-  P23 INITIALISING (Step0), INITIALISED, RUN_REQUEST and RUNNING
       //-  P31 (unmanaged process) INITIALISING and RUNNING
       //-  P32 INITIALISING (Step0), INITIALISED, RUN_REQUEST and RUNNING
       //Once the system has been started up, it leaves the STARTING status to enter the STARTED status. 
       //Its mode is set to NORMAL. 
       //Applications stati checking

       bool result = true;

       CdmwPlatformMngt::SystemSnapshot* now_snapshot = system_s2->get_snapshot();
       system_name_s2 = now_snapshot->system_name;

       TEST_INFO("Checking "+system_name_s2+" applications and processes status");

       CdmwPlatformMngt::ApplicationsSnapshot appls_snapshot = now_snapshot->system_applications;

       for (CORBA::ULong index = 0; index < appls_snapshot.length(); index++) {

     CdmwPlatformMngt::ApplicationSnapshot application_snapshot = appls_snapshot [index];

         if (ApplOneName.compare(application_snapshot.application_name) == 0 ||
             ApplTwoName.compare(application_snapshot.application_name) == 0 ||
             ApplThreeName.compare(application_snapshot.application_name) == 0) {

             CdmwPlatformMngt::ApplicationStatus application_status = application_snapshot.application_status;

         if (ApplOneName.compare(application_snapshot.application_name) == 0) 
             TEST_INFO (ApplOneName+" found !");

         if (ApplTwoName.compare(application_snapshot.application_name) == 0)
             TEST_INFO (ApplTwoName+" found !");

         if (ApplThreeName.compare(application_snapshot.application_name) == 0)
             TEST_INFO (ApplThreeName+" found !");

             if (!(application_status == CdmwPlatformMngt::APPLICATION_INITIALISING ||
                 application_status == CdmwPlatformMngt::APPLICATION_INITIALISED ||
                 application_status == CdmwPlatformMngt::APPLICATION_RUN_REQUEST || 
                 application_status == CdmwPlatformMngt::APPLICATION_RUNNING)) {
         
                 result = false;         
             } else {
     
             CdmwPlatformMngt::ProcessesSnapshot processes_snapshot = application_snapshot.application_processes;
         
             for (CORBA::ULong p_index = 0; p_index < processes_snapshot.length(); p_index++) {
         
                     CdmwPlatformMngt::ProcessSnapshot process_snapshot = processes_snapshot [p_index];                      
       
                 if (process_11.compare(process_snapshot.process_name) == 0) 
                     {
                         CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
                   
                           TEST_INFO (process_11+" found !");

                       if (!(process_status == CdmwPlatformMngt::PROCESS_INITIALISING || 
                           process_status == CdmwPlatformMngt::PROCESS_INITIALISED || 
                           process_status == CdmwPlatformMngt::PROCESS_STEP_PERFORMED || 
                           process_status == CdmwPlatformMngt::PROCESS_RUN_REQUEST || 
                           process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {
               
                               result = false;
                    } 
               } 
               else if (process_12.compare(process_snapshot.process_name) == 0) 
               {
                CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                            TEST_INFO (process_12+" found !");

                            if (!(process_status == CdmwPlatformMngt::PROCESS_INITIALISING ||
                                process_status == CdmwPlatformMngt::PROCESS_INITIALISED ||
                                process_status == CdmwPlatformMngt::PROCESS_RUN_REQUEST ||
                                process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {

                                result = false;
                            } 
                       }
                       else if (process_13.compare(process_snapshot.process_name) == 0) 
                       {
                            CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;

                            TEST_INFO (process_13+" found !");
             
                            if (!(process_status == CdmwPlatformMngt::PROCESS_INITIALISING || 
                        process_status == CdmwPlatformMngt::PROCESS_INITIALISED || 
                        process_status == CdmwPlatformMngt::PROCESS_STEP_PERFORMED || 
                        process_status == CdmwPlatformMngt::PROCESS_RUN_REQUEST || 
                        process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {
               
                        result = false;
                    } 
                }
                else if (process_14.compare(process_snapshot.process_name) == 0) 
                {
                    CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                           TEST_INFO (process_14+" found !");

                   if (!(process_status == CdmwPlatformMngt::PROCESS_INITIALISING || 
                       process_status == CdmwPlatformMngt::PROCESS_INITIALISED || 
                       process_status == CdmwPlatformMngt::PROCESS_STEP_PERFORMED || 
                       process_status == CdmwPlatformMngt::PROCESS_RUN_REQUEST || 
                       process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {
               
                       result = false;
                   } 
                }
                else if (process_21.compare(process_snapshot.process_name) == 0) 
                {
                    CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                           TEST_INFO (process_21+" found !");

                   if (!(process_status == CdmwPlatformMngt::PROCESS_INITIALISING ||
                       process_status == CdmwPlatformMngt::PROCESS_INITIALISED ||
                       process_status == CdmwPlatformMngt::PROCESS_RUN_REQUEST ||
                       process_status == CdmwPlatformMngt::PROCESS_RUNNING ||
                       process_status == CdmwPlatformMngt::PROCESS_ENDED)) {

                       result = false;             
                   }           
                 }
                         else if (process_22.compare(process_snapshot.process_name) == 0) 
                 {
                             CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                           TEST_INFO (process_22+" found !");

                             if (!(process_status == CdmwPlatformMngt::PROCESS_INITIALISING ||
                         process_status == CdmwPlatformMngt::PROCESS_INITIALISED ||
                         process_status == CdmwPlatformMngt::PROCESS_RUN_REQUEST ||
                         process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {

                        result = false;            
                     }         
                  }
                          else if (process_23.compare(process_snapshot.process_name) == 0) 
                  {
                        CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                           TEST_INFO (process_23+" found !");

                                if (!(process_status == CdmwPlatformMngt::PROCESS_INITIALISING ||
                            process_status == CdmwPlatformMngt::PROCESS_INITIALISED ||
                            process_status == CdmwPlatformMngt::PROCESS_RUN_REQUEST ||
                            process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {
  
                            result = false;            
                         }         
                   }
                           else if (process_31.compare(process_snapshot.process_name) == 0) 
                           {
                               CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;

                           TEST_INFO (process_31+" found !");
               
                               if (!(process_status == CdmwPlatformMngt::PROCESS_INITIALISING ||
                           process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {

                        result = false;            
                             }         

                       }
                       else if (process_32.compare(process_snapshot.process_name) == 0) 
                           {
                                CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;

                           TEST_INFO (process_32+" found !");
               
                                if (!(process_status == CdmwPlatformMngt::PROCESS_INITIALISING ||
                            process_status == CdmwPlatformMngt::PROCESS_INITIALISED ||
                                    process_status == CdmwPlatformMngt::PROCESS_RUN_REQUEST ||
                            process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {

                         result = false;               
                                }          
                           }                 
                    }    
                  }
              }
       }

       test_result = result;

       result = true;

       TEST_INFO("Checking "+system_name_s1+" applications and processes status");

       CdmwPlatformMngt::SystemSnapshot* now_snapshot_s1 = system_s1->get_snapshot();

       CdmwPlatformMngt::ApplicationsSnapshot appls_snapshot_s1 = now_snapshot_s1->system_applications;

       for (CORBA::ULong index = 0; index < appls_snapshot_s1.length(); index++) {

     CdmwPlatformMngt::ApplicationSnapshot application_snapshot = appls_snapshot_s1 [index];

         if (ApplOneName.compare(application_snapshot.application_name) == 0 ||
             ApplTwoName.compare(application_snapshot.application_name) == 0 ||
             ApplThreeName.compare(application_snapshot.application_name) == 0) {

             CdmwPlatformMngt::ApplicationStatus application_status = application_snapshot.application_status;

         if (ApplOneName.compare(application_snapshot.application_name) == 0) 
             TEST_INFO (ApplOneName+" found !");

         if (ApplTwoName.compare(application_snapshot.application_name) == 0)
             TEST_INFO (ApplTwoName+" found !");

         if (ApplThreeName.compare(application_snapshot.application_name) == 0)
             TEST_INFO (ApplThreeName+" found !");

             if (!(application_status == CdmwPlatformMngt::APPLICATION_RUNNING)) {
         
                 result = false;         
             } else {
     
             CdmwPlatformMngt::ProcessesSnapshot processes_snapshot = application_snapshot.application_processes;
         
             for (CORBA::ULong p_index = 0; p_index < processes_snapshot.length(); p_index++) {
         
                     CdmwPlatformMngt::ProcessSnapshot process_snapshot = processes_snapshot [p_index];                      
       
                 if (process_11.compare(process_snapshot.process_name) == 0) 
                     {
                         CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
                   
                           TEST_INFO (process_11+" found !");

                       if (!(process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {
               
                               result = false;
                    } 
               } 
               else if (process_12.compare(process_snapshot.process_name) == 0) 
               {
                CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                            TEST_INFO (process_12+" found !");

                            if (!(process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {

                                result = false;
                            } 
                       }
                       else if (process_13.compare(process_snapshot.process_name) == 0) 
                       {
                            CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;

                            TEST_INFO (process_13+" found !");
             
                            if (!(process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {
               
                        result = false;
                    } 
                }
                else if (process_14.compare(process_snapshot.process_name) == 0) 
                {
                    CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                           TEST_INFO (process_14+" found !");

                   if (!(process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {
               
                       result = false;
                   } 
                }
                else if (process_21.compare(process_snapshot.process_name) == 0) 
                {
                    CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                           TEST_INFO (process_21+" found !");

                   if (!(process_status == CdmwPlatformMngt::PROCESS_ENDED)) {

                       result = false;             
                   }           
                 }
                         else if (process_22.compare(process_snapshot.process_name) == 0) 
                 {
                             CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                           TEST_INFO (process_22+" found !");

                             if (!(process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {

                        result = false;            
                     }         
                  }
                          else if (process_23.compare(process_snapshot.process_name) == 0) 
                  {
                        CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                           TEST_INFO (process_23+" found !");

                                if (!(process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {
  
                            result = false;            
                         }         
                   }
                           else if (process_31.compare(process_snapshot.process_name) == 0) 
                           {
                               CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;

                           TEST_INFO (process_31+" found !");
               
                               if (!(process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {

                        result = false;            
                             }         

                       }
                       else if (process_32.compare(process_snapshot.process_name) == 0) 
                           {
                                CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;

                           TEST_INFO (process_32+" found !");
               
                                if (!(process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {

                         result = false;               
                                }          
                           }                 
                    }    
                  }
              }
       }
       
       std::vector <CdmwPlatformMngt::SystemStatus> s2_stati;

       s2_stati.push_back (CdmwPlatformMngt::SYSTEM_STARTING);
       s2_stati.push_back (CdmwPlatformMngt::SYSTEM_STARTED);

       result = observer_on_system_s2.check_sys_stati_sequence(system_name_s2, s2_stati);

       CdmwPlatformMngt::SystemStatus status_s1 = now_snapshot_s1->system_status;

       if (!(status_s1 == CdmwPlatformMngt::SYSTEM_STARTED)) {

        result = false;

       }

       test_result = test_result && result;

       CPPUNIT_ASSERT(test_result);
    }
    
    void TestObjs::s1_stop () {

       /////////////
       // TEST  CASE 3: System 1 Stop
       /////////////

       TEST_INFO(" TEST  CASE 3: "+system_name_s1+" Stop"); 
       
       TEST_INFO("Stopping System "+system_name_s1); 
       TEST_INFO("Executing command: cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --sys-stop"); 

       OsSupport::OS::create_process("cdmw_platform_admin", "--system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --sys-stop"); 

       TEST_INFO("Please wait ..."); 

       OsSupport::OS::sleep (30000);
 
       bool result = true;

       CdmwPlatformMngt::SystemSnapshot* now_snapshot = system_s1->get_snapshot();

       CdmwPlatformMngt::ApplicationsSnapshot appls_snapshot = now_snapshot->system_applications;

       for (CORBA::ULong index = 0; index < appls_snapshot.length(); index++) {

     CdmwPlatformMngt::ApplicationSnapshot application_snapshot = appls_snapshot [index];

         if (ApplOneName.compare(application_snapshot.application_name) == 0 ||
             ApplTwoName.compare(application_snapshot.application_name) == 0 ||
             ApplThreeName.compare(application_snapshot.application_name) == 0) {

             CdmwPlatformMngt::ApplicationStatus application_status = application_snapshot.application_status;

         if (ApplOneName.compare(application_snapshot.application_name) == 0) 
             TEST_INFO (ApplOneName+" found !");

         if (ApplTwoName.compare(application_snapshot.application_name) == 0)
             TEST_INFO (ApplTwoName+" found !");

         if (ApplThreeName.compare(application_snapshot.application_name) == 0)
             TEST_INFO (ApplThreeName+" found !");

             if (!(application_status == CdmwPlatformMngt::APPLICATION_STOPPING || 
                 application_status == CdmwPlatformMngt::APPLICATION_STOPPED)) {
         
                 result = false;         
             } else {
     
             CdmwPlatformMngt::ProcessesSnapshot processes_snapshot = application_snapshot.application_processes;
         
             for (CORBA::ULong p_index = 0; p_index < processes_snapshot.length(); p_index++) {
         
                     CdmwPlatformMngt::ProcessSnapshot process_snapshot = processes_snapshot [p_index];                      
       
                 if (process_11.compare(process_snapshot.process_name) == 0) 
                     {
                         CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
                   
                           TEST_INFO (process_11+" found !");

                       if (!(process_status == CdmwPlatformMngt::PROCESS_STOPPING || 
                           process_status == CdmwPlatformMngt::PROCESS_STOPPED)) {
               
                               result = false;
                    } 
               } 
               else if (process_12.compare(process_snapshot.process_name) == 0) 
               {
                CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                            TEST_INFO (process_12+" found !");

                            if (!(process_status == CdmwPlatformMngt::PROCESS_STOPPING || 
                           process_status == CdmwPlatformMngt::PROCESS_STOPPED)) {

                                result = false;
                            } 
                       }
                       else if (process_13.compare(process_snapshot.process_name) == 0) 
                       {
                            CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;

                            TEST_INFO (process_13+" found !");
             
                            if (!(process_status == CdmwPlatformMngt::PROCESS_STOPPING || 
                           process_status == CdmwPlatformMngt::PROCESS_STOPPED)) {
               
                        result = false;
                    } 
                }
                else if (process_14.compare(process_snapshot.process_name) == 0) 
                {
                    CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                           TEST_INFO (process_14+" found !");

                   if (!(process_status == CdmwPlatformMngt::PROCESS_STOPPING || 
                           process_status == CdmwPlatformMngt::PROCESS_STOPPED)) {
               
                       result = false;
                   } 
                }
                else if (process_21.compare(process_snapshot.process_name) == 0) 
                {
                    CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                           TEST_INFO (process_21+" found !");

                   if (!(process_status == CdmwPlatformMngt::PROCESS_ENDED)) {

                       result = false;             
                   }           
                 }
                         else if (process_22.compare(process_snapshot.process_name) == 0) 
                 {
                             CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                           TEST_INFO (process_22+" found !");

                             if (!(process_status == CdmwPlatformMngt::PROCESS_STOPPING || 
                           process_status == CdmwPlatformMngt::PROCESS_STOPPED)) {

                        result = false;            
                     }         
                  }
                          else if (process_23.compare(process_snapshot.process_name) == 0) 
                  {
                        CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                           TEST_INFO (process_23+" found !");

                                if (!(process_status == CdmwPlatformMngt::PROCESS_STOPPING || 
                           process_status == CdmwPlatformMngt::PROCESS_STOPPED)) {
  
                            result = false;            
                         }         
                   }
                           else if (process_31.compare(process_snapshot.process_name) == 0) 
                           {
                               CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;

                           TEST_INFO (process_31+" found !");
               
                               if (!(process_status == CdmwPlatformMngt::PROCESS_STOPPING || 
                           process_status == CdmwPlatformMngt::PROCESS_STOPPED)) {

                        result = false;            
                             }         

                       }
                       else if (process_32.compare(process_snapshot.process_name) == 0) 
                           {
                                CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;

                           TEST_INFO (process_32+" found !");
               
                                if (!(process_status == CdmwPlatformMngt::PROCESS_STOPPING || 
                           process_status == CdmwPlatformMngt::PROCESS_STOPPED)) {

                         result = false;               
                                }          
                           }                 
                    }    
                  }
              }
       }

       test_result = result;

       result = true;

       CdmwPlatformMngt::SystemSnapshot* now_snapshot_s2 = system_s2->get_snapshot();

       CdmwPlatformMngt::ApplicationsSnapshot appls_snapshot_s2 = now_snapshot_s2->system_applications;

       for (CORBA::ULong index = 0; index < appls_snapshot_s2.length(); index++) {

     CdmwPlatformMngt::ApplicationSnapshot application_snapshot = appls_snapshot_s2 [index];

         if (ApplOneName.compare(application_snapshot.application_name) == 0 ||
             ApplTwoName.compare(application_snapshot.application_name) == 0 ||
             ApplThreeName.compare(application_snapshot.application_name) == 0) {

             CdmwPlatformMngt::ApplicationStatus application_status = application_snapshot.application_status;

         if (ApplOneName.compare(application_snapshot.application_name) == 0) 
             TEST_INFO (ApplOneName+" found !");

         if (ApplTwoName.compare(application_snapshot.application_name) == 0)
             TEST_INFO (ApplTwoName+" found !");

         if (ApplThreeName.compare(application_snapshot.application_name) == 0)
             TEST_INFO (ApplThreeName+" found !");

             if (!(application_status == CdmwPlatformMngt::APPLICATION_RUNNING)) {
         
                 result = false;         
             } else {
     
             CdmwPlatformMngt::ProcessesSnapshot processes_snapshot = application_snapshot.application_processes;
         
             for (CORBA::ULong p_index = 0; p_index < processes_snapshot.length(); p_index++) {
         
                     CdmwPlatformMngt::ProcessSnapshot process_snapshot = processes_snapshot [p_index];                      
       
                 if (process_11.compare(process_snapshot.process_name) == 0) 
                     {
                         CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
                   
                           TEST_INFO (process_11+" found !");

                       if (!(process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {
               
                               result = false;
                    } 
               } 
               else if (process_12.compare(process_snapshot.process_name) == 0) 
               {
                CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                            TEST_INFO (process_12+" found !");

                            if (!(process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {

                                result = false;
                            } 
                       }
                       else if (process_13.compare(process_snapshot.process_name) == 0) 
                       {
                            CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;

                            TEST_INFO (process_13+" found !");
             
                            if (!(process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {
               
                        result = false;
                    } 
                }
                else if (process_14.compare(process_snapshot.process_name) == 0) 
                {
                    CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                           TEST_INFO (process_14+" found !");

                   if (!(process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {
               
                       result = false;
                   } 
                }
                else if (process_21.compare(process_snapshot.process_name) == 0) 
                {
                    CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                           TEST_INFO (process_21+" found !");

                   if (!(process_status == CdmwPlatformMngt::PROCESS_ENDED)) {

                       result = false;             
                   }           
                 }
                         else if (process_22.compare(process_snapshot.process_name) == 0) 
                 {
                             CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                           TEST_INFO (process_22+" found !");

                             if (!(process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {

                        result = false;            
                     }         
                  }
                          else if (process_23.compare(process_snapshot.process_name) == 0) 
                  {
                        CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                           TEST_INFO (process_23+" found !");

                                if (!(process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {
  
                            result = false;            
                         }         
                   }
                           else if (process_31.compare(process_snapshot.process_name) == 0) 
                           {
                               CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;

                           TEST_INFO (process_31+" found !");
               
                               if (!(process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {

                        result = false;            
                             }         

                       }
                       else if (process_32.compare(process_snapshot.process_name) == 0) 
                           {
                                CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;

                           TEST_INFO (process_32+" found !");
               
                                if (!(process_status == CdmwPlatformMngt::PROCESS_RUNNING)) {

                         result = false;               
                                }          
                           }                 
                    }    
                  }
              }
       }

       std::vector <CdmwPlatformMngt::SystemStatus> s1_stati;

       s1_stati.push_back (CdmwPlatformMngt::SYSTEM_STOPPING);
       s1_stati.push_back (CdmwPlatformMngt::SYSTEM_STOPPED);

       result = observer_on_system_s1.check_sys_stati_sequence(system_name_s1, s1_stati);

       CdmwPlatformMngt::SystemStatus status_s2 = now_snapshot_s2->system_status;

       if (!(status_s2 == CdmwPlatformMngt::SYSTEM_STARTED)) {

        result = false;

       }

       test_result = test_result && result;

       CPPUNIT_ASSERT(test_result);

    }

    void TestObjs::s2_stop () {

       /////////////
       // TEST  CASE 4: System 2 Stop
       /////////////

       TEST_INFO(" TEST  CASE 4: "+system_name_s2+" Stop"); 
       
       TEST_INFO("Stopping System "+system_name_s2); 
       TEST_INFO("Executing command: cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21969/CdmwPlatformMngtSupervision --sys-stop"); 

       OsSupport::OS::create_process("cdmw_platform_admin", "--system-corbaloc=corbaloc::localhost:21969/CdmwPlatformMngtSupervision --sys-stop"); 

       TEST_INFO("Please wait ..."); 

       OsSupport::OS::sleep (30000);
 
       bool result = true;

       CdmwPlatformMngt::SystemSnapshot* now_snapshot = system_s2->get_snapshot();

       CdmwPlatformMngt::ApplicationsSnapshot appls_snapshot = now_snapshot->system_applications;

       for (CORBA::ULong index = 0; index < appls_snapshot.length(); index++) {

     CdmwPlatformMngt::ApplicationSnapshot application_snapshot = appls_snapshot [index];

         if (ApplOneName.compare(application_snapshot.application_name) == 0 ||
             ApplTwoName.compare(application_snapshot.application_name) == 0 ||
             ApplThreeName.compare(application_snapshot.application_name) == 0) {

             CdmwPlatformMngt::ApplicationStatus application_status = application_snapshot.application_status;

         if (ApplOneName.compare(application_snapshot.application_name) == 0) 
             TEST_INFO (ApplOneName+" found !");

         if (ApplTwoName.compare(application_snapshot.application_name) == 0)
             TEST_INFO (ApplTwoName+" found !");

         if (ApplThreeName.compare(application_snapshot.application_name) == 0)
             TEST_INFO (ApplThreeName+" found !");

             if (!(application_status == CdmwPlatformMngt::APPLICATION_STOPPING || 
                 application_status == CdmwPlatformMngt::APPLICATION_STOPPED)) {
         
                 result = false;         
             } else {
     
             CdmwPlatformMngt::ProcessesSnapshot processes_snapshot = application_snapshot.application_processes;
         
             for (CORBA::ULong p_index = 0; p_index < processes_snapshot.length(); p_index++) {
         
                     CdmwPlatformMngt::ProcessSnapshot process_snapshot = processes_snapshot [p_index];                      
       
                 if (process_11.compare(process_snapshot.process_name) == 0) 
                     {
                         CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
                   
                           TEST_INFO (process_11+" found !");

                       if (!(process_status == CdmwPlatformMngt::PROCESS_STOPPING || 
                           process_status == CdmwPlatformMngt::PROCESS_STOPPED)) {
               
                               result = false;
                    } 
               } 
               else if (process_12.compare(process_snapshot.process_name) == 0) 
               {
                CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                            TEST_INFO (process_12+" found !");

                            if (!(process_status == CdmwPlatformMngt::PROCESS_STOPPING || 
                           process_status == CdmwPlatformMngt::PROCESS_STOPPED)) {

                                result = false;
                            } 
                       }
                       else if (process_13.compare(process_snapshot.process_name) == 0) 
                       {
                            CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;

                            TEST_INFO (process_13+" found !");
             
                            if (!(process_status == CdmwPlatformMngt::PROCESS_STOPPING || 
                           process_status == CdmwPlatformMngt::PROCESS_STOPPED)) {
               
                        result = false;
                    } 
                }
                else if (process_14.compare(process_snapshot.process_name) == 0) 
                {
                    CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                           TEST_INFO (process_14+" found !");

                   if (!(process_status == CdmwPlatformMngt::PROCESS_STOPPING || 
                           process_status == CdmwPlatformMngt::PROCESS_STOPPED)) {
               
                       result = false;
                   } 
                }
                else if (process_21.compare(process_snapshot.process_name) == 0) 
                {
                    CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                           TEST_INFO (process_21+" found !");

                   if (!(process_status == CdmwPlatformMngt::PROCESS_ENDED)) {

                       result = false;             
                   }           
                 }
                         else if (process_22.compare(process_snapshot.process_name) == 0) 
                 {
                             CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                           TEST_INFO (process_22+" found !");

                             if (!(process_status == CdmwPlatformMngt::PROCESS_STOPPING || 
                           process_status == CdmwPlatformMngt::PROCESS_STOPPED)) {

                        result = false;            
                     }         
                  }
                          else if (process_23.compare(process_snapshot.process_name) == 0) 
                  {
                        CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                           TEST_INFO (process_23+" found !");

                                if (!(process_status == CdmwPlatformMngt::PROCESS_STOPPING || 
                           process_status == CdmwPlatformMngt::PROCESS_STOPPED)) {
  
                            result = false;            
                         }         
                   }
                           else if (process_31.compare(process_snapshot.process_name) == 0) 
                           {
                               CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;

                           TEST_INFO (process_31+" found !");
               
                               if (!(process_status == CdmwPlatformMngt::PROCESS_STOPPING || 
                           process_status == CdmwPlatformMngt::PROCESS_STOPPED)) {

                        result = false;            
                             }         

                       }
                       else if (process_32.compare(process_snapshot.process_name) == 0) 
                           {
                                CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;

                           TEST_INFO (process_32+" found !");
               
                                if (!(process_status == CdmwPlatformMngt::PROCESS_STOPPING || 
                           process_status == CdmwPlatformMngt::PROCESS_STOPPED)) {

                         result = false;               
                                }          
                           }                 
                    }    
                  }
              }
       }

       test_result = result;

       result = true;

       CdmwPlatformMngt::SystemSnapshot* now_snapshot_s1 = system_s1->get_snapshot();

       CdmwPlatformMngt::ApplicationsSnapshot appls_snapshot_s1 = now_snapshot_s1->system_applications;

       for (CORBA::ULong index = 0; index < appls_snapshot_s1.length(); index++) {

     CdmwPlatformMngt::ApplicationSnapshot application_snapshot = appls_snapshot_s1 [index];

         if (ApplOneName.compare(application_snapshot.application_name) == 0 ||
             ApplTwoName.compare(application_snapshot.application_name) == 0 ||
             ApplThreeName.compare(application_snapshot.application_name) == 0) {

             CdmwPlatformMngt::ApplicationStatus application_status = application_snapshot.application_status;

         if (ApplOneName.compare(application_snapshot.application_name) == 0) 
             TEST_INFO (ApplOneName+" found !");

         if (ApplTwoName.compare(application_snapshot.application_name) == 0)
             TEST_INFO (ApplTwoName+" found !");

         if (ApplThreeName.compare(application_snapshot.application_name) == 0)
             TEST_INFO (ApplThreeName+" found !");

             if (!(application_status == CdmwPlatformMngt::APPLICATION_STOPPED)) {
         
                 result = false;         
             } else {
     
             CdmwPlatformMngt::ProcessesSnapshot processes_snapshot = application_snapshot.application_processes;
         
             for (CORBA::ULong p_index = 0; p_index < processes_snapshot.length(); p_index++) {
         
                     CdmwPlatformMngt::ProcessSnapshot process_snapshot = processes_snapshot [p_index];                      
       
                 if (process_11.compare(process_snapshot.process_name) == 0) 
                     {
                         CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
                   
                           TEST_INFO (process_11+" found !");

                       if (!(process_status == CdmwPlatformMngt::PROCESS_STOPPED)) {
               
                               result = false;
                    } 
               } 
               else if (process_12.compare(process_snapshot.process_name) == 0) 
               {
                CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                            TEST_INFO (process_12+" found !");

                            if (!(process_status == CdmwPlatformMngt::PROCESS_STOPPED)) {

                                result = false;
                            } 
                       }
                       else if (process_13.compare(process_snapshot.process_name) == 0) 
                       {
                            CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;

                            TEST_INFO (process_13+" found !");
             
                            if (!(process_status == CdmwPlatformMngt::PROCESS_STOPPED)) {
               
                        result = false;
                    } 
                }
                else if (process_14.compare(process_snapshot.process_name) == 0) 
                {
                    CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                           TEST_INFO (process_14+" found !");

                   if (!(process_status == CdmwPlatformMngt::PROCESS_STOPPED)) {
               
                       result = false;
                   } 
                }
                else if (process_21.compare(process_snapshot.process_name) == 0) 
                {
                    CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                           TEST_INFO (process_21+" found !");

                   if (!(process_status == CdmwPlatformMngt::PROCESS_ENDED)) {

                       result = false;             
                   }           
                 }
                         else if (process_22.compare(process_snapshot.process_name) == 0) 
                 {
                             CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                           TEST_INFO (process_22+" found !");

                             if (!(process_status == CdmwPlatformMngt::PROCESS_STOPPED)) {

                        result = false;            
                     }         
                  }
                          else if (process_23.compare(process_snapshot.process_name) == 0) 
                  {
                        CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;
               
                           TEST_INFO (process_23+" found !");

                                if (!(process_status == CdmwPlatformMngt::PROCESS_STOPPED)) {
  
                            result = false;            
                         }         
                   }
                           else if (process_31.compare(process_snapshot.process_name) == 0) 
                           {
                               CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;

                           TEST_INFO (process_31+" found !");
               
                               if (!(process_status == CdmwPlatformMngt::PROCESS_STOPPED)) {

                        result = false;            
                             }         

                       }
                       else if (process_32.compare(process_snapshot.process_name) == 0) 
                           {
                                CdmwPlatformMngt::ProcessStatus process_status = process_snapshot.process_status;

                           TEST_INFO (process_32+" found !");
               
                                if (!(process_status == CdmwPlatformMngt::PROCESS_STOPPED)) {

                         result = false;               
                                }          
                           }                 
                    }    
                  }
              }
       }
       
       std::vector <CdmwPlatformMngt::SystemStatus> s2_stati;

       s2_stati.push_back (CdmwPlatformMngt::SYSTEM_STOPPING);
       s2_stati.push_back (CdmwPlatformMngt::SYSTEM_STOPPED);

       result = observer_on_system_s2.check_sys_stati_sequence(system_name_s2, s2_stati);

       CdmwPlatformMngt::SystemStatus status_s1 = now_snapshot_s1->system_status;

       if (!(status_s1 == CdmwPlatformMngt::SYSTEM_STOPPED)) {

        result = false;

       }

       test_result = test_result && result;

       CPPUNIT_ASSERT(test_result);

    }
   
}/* End namespace PlatformMngt */
}/* End namespace Cdmw */





