/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/* @author Fabrizio Lorenna <f.lorenna@vitrociset.it> */

#include <iostream>
#include <sstream>
#include <limits>

#include "test_components_and_groups_attributes/TestComponentsAndGroupsAttributes.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"

#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"
#include "idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"

#include <platformftproxy/InitUtils.hpp>
#include <platformftproxy/SystemProxy.hpp>
#include <SystemMngt/platformftproxy/AccessorLookupService.hpp>
#include <SystemMngt/platformftproxy/DefaultAccessorLookupService.hpp>
#include <SystemMngt/platformlibrary/Iterator.hpp>

#include <FaultTolerance/ftreplicationmanager/InitUtils.hpp>
#include <FaultTolerance/idllib/FT.stub.hpp>
#include <platformftaccessorlibrary/SystemAccessor_impl.hpp>
#include "tao/AnyTypeCode/TypeCode.h"


using namespace std;

using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestComponentsAndGroupsAttributes);

namespace Cdmw
{
  namespace PlatformMngt
  {



    TestComponentsAndGroupsAttributes::TestComponentsAndGroupsAttributes ()
    {

    }


    TestComponentsAndGroupsAttributes::~TestComponentsAndGroupsAttributes ()
    {

    }
    bool TestComponentsAndGroupsAttributes::m_is_one_shot_initialized = false;

    CdmwPlatformMngt::System_var TestComponentsAndGroupsAttributes::mysystem;
    CdmwPlatformMngt::Process_var TestComponentsAndGroupsAttributes::myprocess;
    CdmwPlatformMngt::Application_var TestComponentsAndGroupsAttributes::app;
    CdmwPlatformMngt::CcmComponent_var TestComponentsAndGroupsAttributes::comp;

    void TestComponentsAndGroupsAttributes::setUp ()
    {

      TestComponentsAndGroupsAttributes::one_shot_init ();
    }

    void TestComponentsAndGroupsAttributes::one_shot_init ()
    {

      if (!m_is_one_shot_initialized)
        {
          m_is_one_shot_initialized = true;

          try
          {
            CORBA::ORB_var orb;
            orb =
              CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::
                                      instance ()->get_ORB ());

            PortableServer::POA_var poa =
              PortableServer::POA::_duplicate (Cdmw::TestUtils::
                                               CORBATestManager::instance ()->
                                               get_POA ());

            PortableServer::POAManager_var poaMgr =
              PortableServer::POAManager::_duplicate (poa->the_POAManager ());
            poaMgr->activate ();

        CORBA::Object_var objref;

        //define the corbaloc
        const char *corbaloc_str = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";

        objref = orb->string_to_object(corbaloc_str);

        //resolve the System_ptr
        mysystem = CdmwPlatformMngt::System::_narrow(objref.in() );

            if (CORBA::is_nil (mysystem.in ()))
              {
                exit (-1);
              }

            /*from System, use a method to retrieve an application */
            app = mysystem->get_application ("HELLO_WORLD");

            /*from the application, use a method to retrieve a Process */

            /*retrieve the current hostname */
            char *hostname = getenv ("HOSTNAME");

            myprocess = app->get_process ("P12", hostname);

          }
          catch (const Cdmw::Exception & ex)
          {
            std::cerr << ex.what () << std::endl;
            exit (-1);
          }
          catch (const CORBA::Exception & ex)
          {
            std::cerr << ex << std::endl;
            exit (-1);
          }
          catch (...)
          {
            std::cerr << "unexpected exception" << std::endl;
            exit (-1);
          }
        }
    }



    void TestComponentsAndGroupsAttributes::get_component_properties()
    {
    CORBA::String_var info ;

      try
      {
      TEST_INFO (" Getting the component FABCOMPONENT2");
      CdmwPlatformMngt::CcmComponent_var comp2 = 
              myprocess->get_component("FABCOMPONENT2");

      //get user properties ...
      TEST_INFO (" Getting the property COMP_PROP_SHORT");
      CORBA::Any_var prop_value1;
      comp2->get_attribute("COMP_PROP_SHORT", prop_value1,
         CdmwPlatformMngt::USER_DEFINED );

      TEST_INFO (" Getting the property COMP_PROP_FLOAT");
      CORBA::Any_var prop_value2;
      comp2->get_attribute("COMP_PROP_FLOAT", prop_value2,
         CdmwPlatformMngt::USER_DEFINED );

      TEST_INFO (" Getting the property COMP_PROP_STR");
      CORBA::Any_var prop_value3;
      comp2->get_attribute("COMP_PROP_STR", prop_value3,
         CdmwPlatformMngt::USER_DEFINED );

      //get pre-defined properties ...
      TEST_INFO (" Getting the property ReplicationRole");
      CORBA::Any_var prop_value4;
      comp2->get_attribute("ReplicationRole", prop_value4,
         CdmwPlatformMngt::PRE_DEFINED );

      TEST_INFO (" Getting the property DefaultRestartWarmth");
      CORBA::Any_var prop_value5;
      comp2->get_attribute("DefaultRestartWarmth", prop_value5,
         CdmwPlatformMngt::PRE_DEFINED );

      TEST_INFO (" Getting the property RequestedRestartWarmth");
      CORBA::Any_var prop_value6;
      comp2->get_attribute("RequestedRestartWarmth", prop_value6,
         CdmwPlatformMngt::PRE_DEFINED );

      //see number of properties ...

      TEST_INFO (" Getting the properties number ");
      CORBA::ULong num_props = comp2->get_number_of_properties();

      TEST_INFO (" Getting the properties types ");
      CORBA::TypeCode_var tc1 = prop_value1->type();
      CORBA::TypeCode_var tc2 = prop_value2->type();
      CORBA::TypeCode_var tc3 = prop_value3->type();

      CORBA::TypeCode_var tc4 = prop_value4->type();
      CORBA::TypeCode_var tc5 = prop_value5->type();
      CORBA::TypeCode_var tc6 = prop_value6->type();

      TEST_INFO (" Getting the property short value ");
      CORBA::Short value1;
      (*prop_value1) >>= value1 ;

      TEST_INFO (" Getting the property float value ");
      CORBA::Float value2;
      (*prop_value2) >>= value2 ;

      TEST_INFO (" Getting the property char* value ");
      const char* value3_tmp;
      prop_value3 >>= value3_tmp;


      std::string value3( value3_tmp ) ;

      TEST_INFO (" Getting the property CdmwPlatformMngt::RedundancyEltState value ");
      CdmwPlatformMngt::RedundancyEltState value4;
      (*prop_value4) >>= value4;

      TEST_INFO (" Getting the property CdmwPlatformMngt::RestartWarmth value ");
      CdmwPlatformMngt::RestartWarmth value5;
      (*prop_value5) >>= value5;

      TEST_INFO (" Getting the property CdmwPlatformMngt::RestartWarmth value ");
      CdmwPlatformMngt::RestartWarmth value6;
      (*prop_value6) >>= value6;

      TEST_INFO (" Comparing properties values ");
/*
      if ( (num_props == 6) && 
       (tc1->kind() == CORBA::tk_short) &&
       (tc2->kind() == CORBA::tk_float) &&
       (tc3->kind() == CORBA::tk_string) &&
       (tc4->equal(CdmwPlatformMngt::_tc_RedundancyEltState)) &&
       (tc5->equal(CdmwPlatformMngt::_tc_RestartWarmth)) &&
       (tc6->equal(CdmwPlatformMngt::_tc_RestartWarmth)) &&
       (value1 == 5) &&
       (value2 == ( (CORBA::Float) 2.4) ) &&
       (value3.compare("MYSTRING") == 0)  &&
       (value4 == CdmwPlatformMngt::REDSTATE_NORG) &&
       (value5 == CdmwPlatformMngt::HOT_RESTART) &&
       (value6 == CdmwPlatformMngt::HOT_RESTART) )
        {
    //ok
        CPPUNIT_ASSERT (1);
    }
      else
        CPPUNIT_ASSERT (0); //test not ok 
*/
      TEST_INFO (" Comparing number of properties ");
//      if ( num_props == 6)
      if (!(num_props == 8))
      {
          TEST_INFO (" Number of properties is "<< num_props << " FAILED " << std::endl);
          CPPUNIT_ASSERT (false);
      }

      if (!(CORBA::tk_short == tc1->kind()))
      {
          TEST_INFO (" Comparing type code tc1->kind() to CORBA::tk_short FAILED" << std::endl);
          CPPUNIT_ASSERT (false);
      }

      if (!(CORBA::tk_float == tc2->kind()))
      {
          TEST_INFO (" Comparing type code tc2->kind() to CORBA::tk_float FAILED" << std::endl);
          CPPUNIT_ASSERT (false);
      }

      if (!(CORBA::tk_string == tc3->kind()))
      {
          TEST_INFO (" Comparing type code tc3->kind() to CORBA::tk_string FAILED" << std::endl);
          CPPUNIT_ASSERT (false);
      }

//       TEST_INFO (" Comparing type code tc4 to CdmwPlatformMngt::_tc_RedundancyEltState " << std::endl);
       if (!(tc4->equal(CdmwPlatformMngt::_tc_RedundancyEltState)))
       {
          TEST_INFO (" Comparing type code tc4 to CdmwPlatformMngt::_tc_RedundancyEltState FAILED" << std::endl);
          CPPUNIT_ASSERT (false);
       }

//       TEST_INFO (" Comparing type code tc5 to CdmwPlatformMngt::_tc_RestartWarmth " << std::endl);
       if (!(tc5->equal(CdmwPlatformMngt::_tc_RestartWarmth)))
       {
          TEST_INFO (" Comparing type code tc4 to CdmwPlatformMngt::_tc_RestartWarmth FAILED" << std::endl);
          CPPUNIT_ASSERT (false);
       }

//       TEST_INFO (" Comparing type code tc6 to CdmwPlatformMngt::_tc_RestartWarmth " << std::endl);
       if (!(tc6->equal(CdmwPlatformMngt::_tc_RestartWarmth)))
       {
          TEST_INFO (" Comparing type code tc4 to CdmwPlatformMngt::_tc_RestartWarmth FAILED" << std::endl);
          CPPUNIT_ASSERT (false);
       }

//       TEST_INFO (" Comparing type code value1 to 5 " << std::endl);
       if (!(5 == value1))
       {
           TEST_INFO (" Comparing type code value1 to 5 FAILED " << std::endl);
           CPPUNIT_ASSERT (false);
       }

//       TEST_INFO (" Comparing type code value2 to CORBA::Float 2.4 " << std::endl);
       if (!( ((CORBA::Float) 2.4) == value2))
       {
           TEST_INFO (" Comparing type code value2 to CORBA::Float 2.4 FAILED " << std::endl);
           CPPUNIT_ASSERT (false);
       }

//       TEST_INFO (" Comparing type code value3 to MYSTRING " << std::endl);
       if (!(std::string("MYSTRING") == value3))
       {
           TEST_INFO (" Comparing type code value3 to MYSTRING FAILED " << std::endl);
           CPPUNIT_ASSERT (false);
       }

//       TEST_INFO (" Comparing type code value4 to CdmwPlatformMngt::REDSTATE_NORG " << std::endl);
       if (!(CdmwPlatformMngt::REDSTATE_NORG == value4))
       {
           TEST_INFO (" Comparing type code value4 to CdmwPlatformMngt::REDSTATE_NORG FAILED " << std::endl);
           CPPUNIT_ASSERT (false);
       }

//       TEST_INFO (" Comparing type code value5 to CdmwPlatformMngt::HOT_RESTART " << std::endl);
       if (!(CdmwPlatformMngt::HOT_RESTART == value5))
       {
           TEST_INFO (" Comparing type code value5 to 5 FAILED " << std::endl);
           CPPUNIT_ASSERT (false);
       }

//       TEST_INFO (" Comparing type code value6 to CdmwPlatformMngt::HOT_RESTART " << std::endl);
       if (!(CdmwPlatformMngt::HOT_RESTART == value6))
       {
           TEST_INFO (" Comparing type code value6 to CdmwPlatformMngt::HOT_RESTART FAILED " << std::endl);
           CPPUNIT_ASSERT (false);
       }

      }
      catch (const Cdmw::Exception & ex)
      {
          TEST_INFO (" Cdmw::Exception Caught" << ex.what () << std::endl);
          CPPUNIT_ASSERT (false);

//          std::cerr << ex.what () << std::endl;
//          exit (-1);
      }
      catch (const CORBA::Exception & ex)
      {
          TEST_INFO (" CORBA::Exception Caught" << ex << std::endl);
          CPPUNIT_ASSERT (false);
//          std::cerr << ex << std::endl;
//          exit (-1);
      }
      catch (...)
      {
    //error
        TEST_INFO (" Unknown Exception Caught");
        CPPUNIT_ASSERT (false);
      }

    } 



    void TestComponentsAndGroupsAttributes::set_and_get_component_properties()
    {
    CORBA::String_var info ;

      try
      {

      TEST_INFO (" Getting component FABCOMPONENT2 " << std::endl);
      CdmwPlatformMngt::CcmComponent_var comp2 = 
              myprocess->get_component("FABCOMPONENT2");

      CORBA::Any_var prop_value2 = new CORBA::Any() ;
      CORBA::Any_var prop_value3 = new CORBA::Any() ;
      CORBA::Any_var prop_value4 = new CORBA::Any() ;

      TEST_INFO (" Getting FABCOMPONENT2 properties values value2" << std::endl);

      CdmwPlatformMngt::RestartWarmth value2;
      value2 = CdmwPlatformMngt::COLD_RESTART ;
      (*prop_value2) <<= value2;
      comp2->set_attribute("DefaultRestartWarmth", (*prop_value2),
         CdmwPlatformMngt::PRE_DEFINED ); 

      TEST_INFO (" Getting FABCOMPONENT2 properties values value4" << std::endl);
      CORBA::Short value4 = 32 ;
      (*prop_value4) <<= value4;
      comp2->set_attribute("COMP_PROP_SHORT", (*prop_value4),
         CdmwPlatformMngt::USER_DEFINED ); 


      //get the values ...

      TEST_INFO (" Getting FABCOMPONENT2 properties values value6" << std::endl);
      CORBA::Any_var prop_value6;
      comp2->get_attribute("DefaultRestartWarmth", prop_value6,
         CdmwPlatformMngt::PRE_DEFINED );
      CdmwPlatformMngt::RestartWarmth value6;
      (*prop_value6) >>= value6;

      TEST_INFO (" Getting FABCOMPONENT2 properties values value8" << std::endl);
      CORBA::Any_var prop_value8;
      comp2->get_attribute("COMP_PROP_SHORT", prop_value8,
         CdmwPlatformMngt::USER_DEFINED );
      CORBA::Short value8;
      (*prop_value8) >>= value8 ;

      TEST_INFO (" Getting FABCOMPONENT2 properties retrieving types" << std::endl);
      CORBA::TypeCode_var tc6 = prop_value6->type();
      CORBA::TypeCode_var tc8 = prop_value8->type();

      //see number of properties ...

      CORBA::ULong num_props = comp2->get_number_of_properties();

 //     if ( (num_props == 6) && 
      if ( (num_props == 8) && 
       (tc6->equal(CdmwPlatformMngt::_tc_RestartWarmth) ) &&
       (tc8->kind() == CORBA::tk_short) &&
       (value6 == CdmwPlatformMngt::COLD_RESTART ) &&
       (value8 == 32 )  )
        {
    //ok
        CPPUNIT_ASSERT (1);
    }
      else
        CPPUNIT_ASSERT (0); //test not ok 

      }
      catch (const Cdmw::Exception & ex)
      {
          TEST_INFO ("Cdmw::Exception Caught" << ex.what () << std::endl);
          CPPUNIT_ASSERT (false);

//          std::cerr << ex.what () << std::endl;
//          exit (-1);
      }
      catch (const CORBA::Exception & ex)
      {
          TEST_INFO ("CORBA::Exception Caught" << ex << std::endl);
          CPPUNIT_ASSERT (false);
//          std::cerr << ex << std::endl;
//          exit (-1);
      }
      catch (...)
      {
    //error
        TEST_INFO ("Unknown Exception Caught");
        CPPUNIT_ASSERT (false);
      }

    } 



    void TestComponentsAndGroupsAttributes::get_ftgroup_properties()
    {
      CdmwPlatformMngt::FTGroup_var ftg ;

      try
        {
         ftg = app->get_ft_group("FABFTGROUP1");

         //get the properties ...
     CORBA::Any_var prop_value1;
         ftg->get_attribute("FTG_PROP_SHORT", prop_value1,
          CdmwPlatformMngt::USER_DEFINED );

         CORBA::Any_var prop_value2;
         ftg->get_attribute("FTG_PROP_FLOAT", prop_value2,
          CdmwPlatformMngt::USER_DEFINED );

         CORBA::Any_var prop_value3;
         ftg->get_attribute("FTG_PROP_STR", prop_value3,
          CdmwPlatformMngt::USER_DEFINED );
 
         CORBA::Any_var prop_value4;
         ftg->get_attribute("GroupSize", prop_value4,
          CdmwPlatformMngt::PRE_DEFINED );

         //get number of properties ...
         CORBA::ULong num_props = ftg->get_number_of_properties();

         CORBA::TypeCode_var tc1 = prop_value1->type();
         CORBA::TypeCode_var tc2 = prop_value2->type();
         CORBA::TypeCode_var tc3 = prop_value3->type();
         CORBA::TypeCode_var tc4 = prop_value4->type();

         CORBA::Short value1; 
         (*prop_value1) >>= value1 ;

         CORBA::Float value2; 
         (*prop_value2) >>= value2 ;

         const char* value3_tmp;
         prop_value3 >>= value3_tmp;
         std::string value3( value3_tmp ) ;


         CORBA::Long value4; 
         (*prop_value4) >>= value4 ;

      TEST_INFO (" Getting FABFTGROUP1 properties number "<< num_props << std::endl);
     if ( (6 == num_props) &&
              ( value1 == 10) &&
              (value2 == ((CORBA::Float) 11.1) ) &&
              ( value3.compare("MYSTRING_FT") == 0) &&
              (value4 == 0) &&
              (tc1->kind() == CORBA::tk_short) &&
              (tc2->kind() == CORBA::tk_float) &&
              (tc3->kind() == CORBA::tk_string) &&
              (tc4->kind() == CORBA::tk_long ))
                  CPPUNIT_ASSERT(1); //ok
         else
                  CPPUNIT_ASSERT(0); //not ok


        }
      catch (const Cdmw::Exception & ex)
      {
          TEST_INFO ("Cdmw::Exception Caught" << ex.what () << std::endl);
          CPPUNIT_ASSERT (false);

//          std::cerr << ex.what () << std::endl;
//          exit (-1);
      }
      catch (const CORBA::Exception & ex)
      {
          TEST_INFO ("CORBA::Exception Caught" << ex << std::endl);
          CPPUNIT_ASSERT (false);
//          std::cerr << ex << std::endl;
//          exit (-1);
      }
      catch (...)
      {
    //error
        TEST_INFO ("Unknown Exception Caught");
        CPPUNIT_ASSERT (false);
      }


    }



    void TestComponentsAndGroupsAttributes::set_and_get_ftgroup_properties()
    {
    CORBA::String_var info ;

    CdmwPlatformMngt::FTGroup_var ftg ;
    CORBA::Any_var prop_value1 = new CORBA::Any() ;
    CORBA::Any_var prop_value2 = new CORBA::Any() ;
    CORBA::Any_var prop_value1_get;
    CORBA::Short newvalue1; 

    try
     {
      ftg = app->get_ft_group("FABFTGROUP1");



      CORBA::Short value1 = 23 ;
      (*prop_value1) <<= value1 ;
      ftg->set_attribute("FTG_PROP_SHORT", (*prop_value1),
         CdmwPlatformMngt::USER_DEFINED );


      ftg->get_attribute("FTG_PROP_SHORT", prop_value1_get,
         CdmwPlatformMngt::USER_DEFINED );

      (*prop_value1_get) >>= newvalue1 ;
      }
      catch (...)
      {
    //error
        TEST_INFO ("Unknown Exception Caught");
        CPPUNIT_ASSERT (0);
      }


      try
      {
      CORBA::TypeCode_var tc1_get = prop_value1_get->type();

      //see number of properties ...

      CORBA::ULong num_props = ftg->get_number_of_properties();

      if ( (6 == num_props) && 
       (tc1_get->kind() == CORBA::tk_short) &&
           ( newvalue1 == 23 ) ) 
        {
    //test ok
        CPPUNIT_ASSERT (1);
    }
      else
        CPPUNIT_ASSERT (0); //test not ok 

      }
      catch (const Cdmw::Exception & ex)
      {
          TEST_INFO ("Cdmw::Exception Caught" << ex.what () << std::endl);
          CPPUNIT_ASSERT (false);

//          std::cerr << ex.what () << std::endl;
//          exit (-1);
      }
      catch (const CORBA::Exception & ex)
      {
          TEST_INFO ("CORBA::Exception Caught" << ex << std::endl);
          CPPUNIT_ASSERT (false);
//          std::cerr << ex << std::endl;
//          exit (-1);
      }
      catch (...)
      {
    //error
        TEST_INFO ("Unknown Exception Caught");
        CPPUNIT_ASSERT (false);
      }

    } 



    void TestComponentsAndGroupsAttributes::get_lbgroup_properties()
    {
      CdmwPlatformMngt::LBGroup_var lbg ;

      try
        {
         lbg = app->get_lb_group("FABLBGROUP3");

         //get the properties ...
         CORBA::Any_var prop_value1;
         lbg->get_attribute("LBG_PROP_SHORT", prop_value1,
          CdmwPlatformMngt::USER_DEFINED );

         CORBA::Any_var prop_value2;
         lbg->get_attribute("LBG_PROP_FLOAT", prop_value2,
          CdmwPlatformMngt::USER_DEFINED );

         CORBA::Any_var prop_value3;
         lbg->get_attribute("LBG_PROP_STR", prop_value3,
          CdmwPlatformMngt::USER_DEFINED );
 
         CORBA::Any_var prop_value4;
         lbg->get_attribute("GroupSize", prop_value4,
          CdmwPlatformMngt::PRE_DEFINED );

         //get number of properties ...
         CORBA::ULong num_props = lbg->get_number_of_properties();

         CORBA::TypeCode_var tc1 = prop_value1->type();
         CORBA::TypeCode_var tc2 = prop_value2->type();
         CORBA::TypeCode_var tc3 = prop_value3->type();
         CORBA::TypeCode_var tc4 = prop_value4->type();

         CORBA::Short value1; 
         (*prop_value1) >>= value1 ;

         CORBA::Float value2; 
         (*prop_value2) >>= value2 ;

         const char* value3_tmp;
         prop_value3 >>= value3_tmp;
         std::string value3( value3_tmp ) ;


         CORBA::Long value4; 
         (*prop_value4) >>= value4 ;


     if ( (6 == num_props) &&
              ( value1 == 13) &&
              (value2 == ((CORBA::Float) 14.4) ) &&
              ( value3.compare("MYSTRING_LB") == 0) &&
              (value4 == 0) &&
              (tc1->kind() == CORBA::tk_short) &&
              (tc2->kind() == CORBA::tk_float) &&
              (tc3->kind() == CORBA::tk_string) &&
              (tc4->kind() == CORBA::tk_long ))
                  CPPUNIT_ASSERT(1); //ok
         else
                  CPPUNIT_ASSERT(0); //not ok


        }
      catch (const Cdmw::Exception & ex)
      {
          TEST_INFO ("Cdmw::Exception Caught" << ex.what () << std::endl);
          CPPUNIT_ASSERT (false);

//          std::cerr << ex.what () << std::endl;
//          exit (-1);
      }
      catch (const CORBA::Exception & ex)
      {
          TEST_INFO ("CORBA::Exception Caught" << ex << std::endl);
          CPPUNIT_ASSERT (false);
//          std::cerr << ex << std::endl;
//          exit (-1);
      }
      catch (...)
      {
    //error
        TEST_INFO ("Unknown Exception Caught");
        CPPUNIT_ASSERT (false);
      }


    }


    void TestComponentsAndGroupsAttributes::set_and_get_lbgroup_properties()
    {
    CORBA::String_var info ;

    CdmwPlatformMngt::LBGroup_var lbg ;
    CORBA::Any_var prop_value1 = new CORBA::Any() ;
    CORBA::Any_var prop_value1_get;
    CORBA::Short newvalue1;

    try
     {
      lbg = app->get_lb_group("FABLBGROUP3");


      CORBA::Short value1 = 23 ;
      (*prop_value1) <<= value1 ;
      lbg->set_attribute("LBG_PROP_SHORT", (*prop_value1),
         CdmwPlatformMngt::USER_DEFINED );


      lbg->get_attribute("LBG_PROP_SHORT", prop_value1_get,
         CdmwPlatformMngt::USER_DEFINED );

      (*prop_value1_get) >>= newvalue1 ;
      }
      catch (...)
      {
    //error
        TEST_INFO ("Unknown Exception Caught");
        CPPUNIT_ASSERT (0);
      }


      try
      {
      CORBA::TypeCode_var tc1_get = prop_value1_get->type();

      //see number of properties ...

      CORBA::ULong num_props = lbg->get_number_of_properties();

      if ( (6 == num_props) && 
       (tc1_get->kind() == CORBA::tk_short) &&
           ( newvalue1 == 23) ) 
        {
    //test ok
        CPPUNIT_ASSERT (1);
    }
      else
        CPPUNIT_ASSERT (0); //test not ok 

      }
      catch (const Cdmw::Exception & ex)
      {
          TEST_INFO ("Cdmw::Exception Caught" << ex.what () << std::endl);
          CPPUNIT_ASSERT (false);

//          std::cerr << ex.what () << std::endl;
//          exit (-1);
      }
      catch (const CORBA::Exception & ex)
      {
          TEST_INFO ("CORBA::Exception Caught" << ex << std::endl);
          CPPUNIT_ASSERT (false);
//          std::cerr << ex << std::endl;
//          exit (-1);
      }
      catch (...)
      {
    //error
        TEST_INFO ("Unknown Exception Caught");
        CPPUNIT_ASSERT (false);
      }

    } 

  }                             /* End namespace PlatformMngt */
}                               /* End namespace Cdmw */
