/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/* @author Fabrizio Lorenna <f.lorenna@vitrociset.it> */

#include <iostream>
#include <sstream>
#include <limits>

#include "test_components_nav/TestComponentsNav.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"

#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"
#include "idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"

#include <platformftproxy/InitUtils.hpp>
#include <platformftproxy/SystemProxy.hpp>
#include <SystemMngt/platformftproxy/AccessorLookupService.hpp>
#include <SystemMngt/platformftproxy/DefaultAccessorLookupService.hpp>
#include <SystemMngt/platformlibrary/Iterator.hpp>

#include <FaultTolerance/ftreplicationmanager/InitUtils.hpp>
#include <FaultTolerance/idllib/FT.stub.hpp>
#include <platformftaccessorlibrary/SystemAccessor_impl.hpp>
#include "tao/AnyTypeCode/TypeCode.h"


using namespace std;

using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestComponentsNav);

namespace Cdmw
{
  namespace PlatformMngt
  {



    TestComponentsNav::TestComponentsNav ()
    {

    }


    TestComponentsNav::~TestComponentsNav ()
    {

    }
    bool TestComponentsNav::m_is_one_shot_initialized = false;

    CdmwPlatformMngt::System_var TestComponentsNav::mysystem;
    CdmwPlatformMngt::Process_var TestComponentsNav::myprocess;
    CdmwPlatformMngt::Application_var TestComponentsNav::app;
    CdmwPlatformMngt::CcmComponent_var TestComponentsNav::comp;

    void TestComponentsNav::setUp ()
    {

      TestComponentsNav::one_shot_init ();
    }

    void TestComponentsNav::one_shot_init ()
    {

      if (!m_is_one_shot_initialized)
        {
          m_is_one_shot_initialized = true;

          try
          {
            CORBA::ORB_var orb;
            orb =
              CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::
                                      instance ()->get_ORB ());

            PortableServer::POA_var poa =
              PortableServer::POA::_duplicate (Cdmw::TestUtils::
                                               CORBATestManager::instance ()->
                                               get_POA ());

            PortableServer::POAManager_var poaMgr =
              PortableServer::POAManager::_duplicate (poa->the_POAManager ());
            poaMgr->activate ();


        std::string system_corbaloc = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";
            CORBA::Object_var system_obj = orb->string_to_object (system_corbaloc.c_str());

            mysystem = CdmwPlatformMngt::System::_narrow (system_obj.in ());


            if (CORBA::is_nil (mysystem.in ()))
              {
                exit (-1);
              }

            /*from System, use a method to retrieve an application */
            app = mysystem->get_application ("HELLO_WORLD");

            /*from the application, use a method to retrieve a Process */

            /*retrieve the current hostname */
            char *hostname = getenv ("HOSTNAME");

            myprocess = app->get_process ("P12", hostname);

          }
          catch (const Cdmw::Exception & ex)
          {
            std::cerr << ex.what () << std::endl;
            exit (-1);
          }
          catch (const CORBA::Exception & ex)
          {
            std::cerr << ex << std::endl;
            exit (-1);
          }
          catch (...)
          {
            std::cerr << "unexpected exception" << std::endl;
            exit (-1);
          }
        }
    }

    void TestComponentsNav::get_component ()
    {

      //test to retrieve the component ptr ...
      try     
        {
         comp = myprocess->get_component("FABCOMPONENT2");

         std::string compname ( comp->name() );

         if ( compname.compare("FABCOMPONENT2") == 0 )
             CPPUNIT_ASSERT(1);  /*test ok*/
         else
             CPPUNIT_ASSERT (0); /*test not ok */
    }
      catch(...)
        {
        //error
        CPPUNIT_ASSERT(0);
        }
      

    }

    void TestComponentsNav::get_all_components ()
    {

      //test to retrieve the component ptr ...
      try     
        {
     CdmwPlatformMngt::CcmComponents_var ncomponents ;
     CdmwPlatformMngt::ComponentsIterator_var rest ;

     //try to get 2 references of the 3 components ...
         myprocess->get_all_components(2,
                                      ncomponents,
                                      rest);

        CORBA::String_var compname1 = ( (*ncomponents)[0])->name() ;
        CORBA::String_var compname2 = ( (*ncomponents)[1])->name() ;

        std::string compstr1(compname1) ;
    std::string compstr2(compname2) ;


        CdmwPlatformMngt::CcmComponent_var comp3 ;

        rest->next_one(comp3) ;

        CORBA::String_var compname3 = comp3->name() ;

        std::string compstr3(compname3) ;


    if ( (compstr1.compare("FABCOMPONENT1") == 0 ) &&
             (compstr2.compare("FABCOMPONENT2") == 0 ) &&
             (compstr3.compare("FABCOMPONENT3") == 0 ) )
         {

             CPPUNIT_ASSERT (1); /*test ok */
            CPPUNIT_ASSERT (1); /*test ok */

         }
        else
       CPPUNIT_ASSERT (0); //test not ok


    }
      catch(...)
        {
        //error
        CPPUNIT_ASSERT(0);
    }


    }

    void TestComponentsNav::get_number_of_components ()
    {

      try
        {
         CORBA::ULong num_comp =  myprocess->get_number_of_components();

         if ( num_comp == 3 )
         CPPUNIT_ASSERT(1); //test ok
         else
         CPPUNIT_ASSERT(0); //test not ok

    }
      catch(...)
        {
        //error
        CPPUNIT_ASSERT(0);
    }


    }

    void TestComponentsNav::get_component_status ()
    {
    CORBA::String_var info ;

      try
      {

      CdmwPlatformMngt::ComponentStatus stat = comp->get_status(info) ;
      std::string info_str ( info ) ;

      if ( (stat == CdmwPlatformMngt::COMPONENT_INITIAL) &&
            (info_str.compare("") == 0) )
          CPPUNIT_ASSERT (1); /*test ok */
      else
          CPPUNIT_ASSERT(0);  /*test not ok*/

      }
      catch (...)
      {
    //error
        CPPUNIT_ASSERT (0);
      }

    }

    void TestComponentsNav::get_process_from_component ()
    {
    CORBA::String_var info ;

      try
      {


      CdmwPlatformMngt::Process_var proc_local =
        CdmwPlatformMngt::Process::_narrow(comp->get_parent() ) ;


      CORBA::String_var procname = proc_local->name() ;

      std::string procname_str(procname) ;

      if ( procname_str.compare("P12") == 0)
         CPPUNIT_ASSERT(1); //test ok
      else
         CPPUNIT_ASSERT(0); //test not ok


      }
      catch (...)
      {
    //error
        CPPUNIT_ASSERT (0);
      }

    }





  }                             /* End namespace PlatformMngt */
}                               /* End namespace Cdmw */
