/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <iomanip>
#include <typeinfo>
#include <fstream>

#include "Foundation/ossupport/OS.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/common/Options.hpp"
#include "Foundation/common/Exception.hpp"

#include "SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/EntityStatusChange_impl.hpp"

#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "test_force_manual_start_proc/ProcObserver_impl.hpp"




/**
 * @author Carlo Giovine  <cgiovine@sesm.it>
 */

namespace Cdmw
  {

  namespace PlatformMngt
    {

    void  ProcObserver_impl::notify(CdmwPlatformMngt::Event* an_event)
    throw( CORBA::SystemException )
    {
       try
       {
           CdmwPlatformMngt::ProcessStatusChange* prc_chg = dynamic_cast<CdmwPlatformMngt::ProcessStatusChange*> (an_event);
           if (((prc_chg!=0) ) && 
              ((prc_chg->process_status()==CdmwPlatformMngt::PROCESS_FAILED_DEATH) ||
              (prc_chg->process_status()== CdmwPlatformMngt::PROCESS_FAILED_ERROR) ) ) 
           {
               CORBA::String_var message = an_event->to_string();
               std::cout << message.in() << std::endl;
               failed = true;
           }
           else if (((prc_chg!=0)) &&
               (prc_chg->process_status()== CdmwPlatformMngt::PROCESS_RUNNING) )            
           {
               CORBA::String_var message = an_event->to_string();
               std::cout << message.in() << std::endl;
               if (2 == initialised) 
               {
                  restarted = true;
               }
           }
           else if ( (prc_chg!=0) &&
               (prc_chg->process_status()== CdmwPlatformMngt::PROCESS_INITIALISED) )
           {
               CORBA::String_var message = an_event->to_string();
               std::cout << message.in() << std::endl;
               initialised ++ ;
           }
           else if (prc_chg!=0)
           {
               CORBA::String_var message = an_event->to_string();
               std::cout << message.in() << std::endl;
           }
        }
        catch (const std::exception& ex)
        {
            std::cerr << " Process observer exception  " << ex.what();
        }

    }
    
    bool  ProcObserver_impl::is_failed()
    {
        return failed;
    }
    
    bool  ProcObserver_impl::is_restarted()
    {
        return restarted;
    }

  } // PlatformMngt
} // Cdmw

