/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_TESTOBJS_HPP
#define INCL_PLATFORMMNGT_TESTOBJS_HPP

#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/common/System.hpp"
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/Exception.hpp>
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"

#include "Foundation/common/System.hpp"

#include  "platformadminlibrary/PlatformAdminSnapshot.hpp"

namespace Cdmw
{
  namespace PlatformMngt
  {

    class TestObjs:public CppUnit::TestFixture
    {

    public:

        TestObjs();
        
        void setUp ();
 
        virtual void test_system_start ();
        
        virtual void test_system_restart ();
        
        virtual void test_restart_application ();
        
        virtual void test_add_application ();
        
        virtual void test_start_application_manually ();

    private:
    
        //System reference
        CdmwPlatformMngt::System_var system ;
        
        //Application references
        CdmwPlatformMngt::ApplicationsIterator_var appIter;
        CdmwPlatformMngt::Applications_var apps;
        CORBA::ULong nbApps;
        
        //Process references
        CdmwPlatformMngt::ProcessesIterator_var procIter;
        CdmwPlatformMngt::Processes_var procs;
        CORBA::ULong nbProcs;
        
        CdmwPlatformMngtBase::StartupKind_var startupKind;
        CdmwPlatformMngtBase::StateRetrieval state_retrieval;
        
        CORBA::String_var statusInfo;
        
        int val;
        int long_timeout;
        int short_timeout;
        unsigned long startup_mode;
        std::string specified_state_id;

        /**
         * Get ForceManualStart for application and process
         * @param Application/Process_ptr
         * @return the value of force_manual_start
         */
        bool get_force_manual_start (const CdmwPlatformMngt::Application_ptr ptr) throw ();
        
        bool get_force_manual_start (const CdmwPlatformMngt::Process_ptr ptr) throw ();
        
        /**
         * This function controls if system is started
         */
        bool sys_status_control (const CdmwPlatformMngt::System_ptr ptr) throw (); 
        
        /**
         * These functions control for Application/Process with force_manual_start set to manual,
         * if the current status is the status expected, that is give as parameter
         */
        bool app_status_control (const CdmwPlatformMngt::Application_ptr ptr,const CdmwPlatformMngt::ApplicationStatus& expected_status) throw ();
        
        bool proc_status_control (const CdmwPlatformMngt::Process_ptr ptr, const CdmwPlatformMngt::ProcessStatus& expected_status) throw ();
        
        /**
         * Assignment operator
         */
        TestObjs & operator= (const TestObjs & rhs) throw ();
        
        /**
         * Copy constructor
         */
        TestObjs (const TestObjs & rhs) throw ();

        /**
         * Defining a Test Suite
         */       
        CPPUNIT_TEST_SUITE (TestObjs);
        
            //These are the test cases
            CPPUNIT_TEST (test_system_start); 
            CPPUNIT_TEST (test_system_restart);
            CPPUNIT_TEST (test_add_application);
            CPPUNIT_TEST (test_start_application_manually);
            CPPUNIT_TEST (test_restart_application);
            
        CPPUNIT_TEST_SUITE_END ();
    };

  }             /* End namespace PlatformMngt */
}               /* End namespace Cdmw */
#endif              /* INCL_PLATFORMMNGT_TESTRESTARTOBJS_HPP */
