/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/* @author Fabrizio Lorenna <f.lorenna@vitrociset.it> */

#include <iostream>
#include <sstream>
#include <limits>

#include "test_ftgroup_nav/TestFTGroupNav.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"

#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"
#include "idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"

#include <platformftproxy/InitUtils.hpp>
#include <platformftproxy/SystemProxy.hpp>
#include <SystemMngt/platformftproxy/AccessorLookupService.hpp>
#include <SystemMngt/platformftproxy/DefaultAccessorLookupService.hpp>
#include <SystemMngt/platformlibrary/Iterator.hpp>

#include <FaultTolerance/ftreplicationmanager/InitUtils.hpp>
#include <FaultTolerance/idllib/FT.stub.hpp>
#include <platformftaccessorlibrary/SystemAccessor_impl.hpp>
#include "tao/AnyTypeCode/TypeCode.h"


using namespace std;

using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestFTGroupNav);

namespace Cdmw
{
  namespace PlatformMngt
  {



    TestFTGroupNav::TestFTGroupNav ()
    {

    }


    TestFTGroupNav::~TestFTGroupNav ()
    {

    }
    bool TestFTGroupNav::m_is_one_shot_initialized = false;

    CdmwPlatformMngt::System_var TestFTGroupNav::mysystem;
    CdmwPlatformMngt::Process_var TestFTGroupNav::myprocess;
    CdmwPlatformMngt::Application_var TestFTGroupNav::app;

    void TestFTGroupNav::setUp ()
    {

      TestFTGroupNav::one_shot_init ();
    }

    void TestFTGroupNav::one_shot_init ()
    {

      if (!m_is_one_shot_initialized)
        {
          m_is_one_shot_initialized = true;

          try
          {
            CORBA::ORB_var orb;
            orb =
              CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::
                                      instance ()->get_ORB ());

            PortableServer::POA_var poa =
              PortableServer::POA::_duplicate (Cdmw::TestUtils::
                                               CORBATestManager::instance ()->
                                               get_POA ());

            PortableServer::POAManager_var poaMgr =
              PortableServer::POAManager::_duplicate (poa->the_POAManager ());
            poaMgr->activate ();


        std::string system_corbaloc = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";
            CORBA::Object_var system_obj = orb->string_to_object (system_corbaloc.c_str());

            mysystem = CdmwPlatformMngt::System::_narrow (system_obj.in ());


            if (CORBA::is_nil (mysystem.in ()))
              {
                exit (-1);
              }

            /*from System, use a method to retrieve an application */
            app = mysystem->get_application ("HELLO_WORLD");

            /*from the application, use a method to retrieve a Process */

            /*retrieve the current hostname */
            char *hostname = getenv ("HOSTNAME");

            myprocess = app->get_process ("P12", hostname);

          }
          catch (const Cdmw::Exception & ex)
          {
            std::cerr << ex.what () << std::endl;
            exit (-1);
          }
          catch (const CORBA::Exception & ex)
          {
            std::cerr << ex << std::endl;
            exit (-1);
          }
          catch (...)
          {
            std::cerr << "unexpected exception" << std::endl;
            exit (-1);
          }
        }
    }

    void TestFTGroupNav::get_ft_group()
    {

      CdmwPlatformMngt::FTGroup_var ftg ;

      //test to retrieve the ftgroup ptr ...
      try     
        {
         ftg = app->get_ft_group("FABFTGROUP1");

         char *ftg_name = ftg->name() ;

         std::string ftgroup_name( ftg_name)  ;

         if ( ftgroup_name.compare("FABFTGROUP1") == 0)
             CPPUNIT_ASSERT(1);  /*test ok*/
         else
             CPPUNIT_ASSERT(0); /*test not ok */
        }
      catch(...)
        {       
        //error 
        CPPUNIT_ASSERT(0);
        }
      

    }

    void TestFTGroupNav::get_all_ft_groups()
    {

    CdmwPlatformMngt::FTGroups_var ftgroups;

    CdmwPlatformMngt::FTGroupsIterator_var rest ;


    CdmwPlatformMngt::FTGroup_var ftg3 ;

    try
        {
         app->get_all_ft_groups(2, ftgroups, rest );

         CORBA::ULong len_subset1 = ftgroups->length() ;

         std::string ftg_name1( ( (*ftgroups)[0])->name() );
         std::string ftg_name2( ( (*ftgroups)[1])->name() );

         rest->next_one(ftg3) ;

         std::string ftg_name3(  ftg3->name() );

         if ( ( len_subset1 == 2) &&
         (ftg_name1.compare("FABFTGROUP1") == 0) &&
         (ftg_name2.compare("FABFTGROUP2") == 0) &&
         (ftg_name3.compare("FABFTGROUP3") == 0) )
           {
           CPPUNIT_ASSERT(1); //ok
           }
           else
            CPPUNIT_ASSERT(0); //not ok


       }
       catch(...)
       {
        //error
        CPPUNIT_ASSERT(0);
       }


    }


    void TestFTGroupNav::get_number_of_ft_groups()
    {

    try
    {
    CORBA::ULong num_ftgroups = app->get_number_of_ft_groups() ;


    if ( num_ftgroups == 3)
      CPPUNIT_ASSERT(1); //ok
    else
      CPPUNIT_ASSERT(0); //not ok

    }
    catch(...)
      {
        CPPUNIT_ASSERT(0);
      }

    }

    void TestFTGroupNav::get_application_from_ftgroup()
    {


      CdmwPlatformMngt::FTGroup_var ftg ;

      //test to retrieve the appl. form ftgroup ptr ...
      try
        {
         ftg = app->get_ft_group("FABFTGROUP1");


         CdmwPlatformMngt::Application_var app_local =
            Application::_narrow(ftg->get_parent()) ;


     char *app_name =  app_local->name()  ;

     std::string appname_str ( app_name ) ;

         if ( (appname_str.compare("HELLO_WORLD") == 0))
            CPPUNIT_ASSERT(1); //test ok
         else
            CPPUNIT_ASSERT(0); //test not ok


    }
      catch(...)
        {
        //error
        CPPUNIT_ASSERT(0);
    }

    }

    void TestFTGroupNav::get_ftgroup_status ()
    {
    CORBA::String_var info ;
    CdmwPlatformMngt::FTGroup_var ftg ;

    try
      {
      ftg = app->get_ft_group("FABFTGROUP1");

      CdmwPlatformMngt::TechnicalStatus stat = ftg->get_status(info) ;
      std::string info_str ( info ) ;

      if ( (stat == CdmwPlatformMngt::INITIAL) &&
            (info_str.compare("") == 0) )
          CPPUNIT_ASSERT (1); /*test ok */
      else
          CPPUNIT_ASSERT(0);  /*test not ok*/

      }
      catch (...)
      {
    //error
        CPPUNIT_ASSERT (0);
      }

    }





  }                             /* End namespace PlatformMngt */
}                               /* End namespace Cdmw */
