/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_TESTPROP_HPP
#define INCL_PLATFORMMNGT_TESTPROP_HPP

#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/common/System.hpp"
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/Exception.hpp>
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"

#include "Foundation/common/System.hpp"

#include  "platformadminlibrary/PlatformAdminSnapshot.hpp"

namespace Cdmw
{
  namespace PlatformMngt
  {

    class TestProp:public CppUnit::TestFixture
    {

    public:

        TestProp();
        
        void setUp ();
 
        virtual void test_properties ();
        void TestProp::check_get_element(const char* hostname1,const char* hostname2);
        void TestProp::get_references(const char* hostname1,const char* hostname2);
        
        void check_properties (CdmwPlatformMngt::Host_var HostPrimary ,CdmwPlatformMngt::Host_var HostSecondary);
                
        
    private:
    
        //System reference
        CdmwPlatformMngt::System_var m_system ;
                
        
        CdmwPlatformMngtBase::StartupKind_var startupKind;
        CdmwPlatformMngtBase::StateRetrieval state_retrieval;        
        CORBA::String_var statusInfo; 
        
        
        
        unsigned long startup_mode;
        std::string specified_state_id;
        
        int long_timeout;
        int short_timeout;
        
        //References to system elements
        
        CdmwPlatformMngt::ManagedElement_var m_application;
        CdmwPlatformMngt::ManagedElement_var m_component;
        CdmwPlatformMngt::ManagedElement_var m_ftgroup;
        CdmwPlatformMngt::ManagedElement_var m_lbgroup;
        CdmwPlatformMngt::ManagedElement_var m_appl_entity;
        CdmwPlatformMngt::ManagedElement_var m_proc_entity;
        CdmwPlatformMngt::ManagedElement_var m_host;
        CdmwPlatformMngt::ManagedElement_var m_process;
        CdmwPlatformMngt::ManagedElement_var m_sys;
        
        
        
        /**
         * Assignment operator
         */
        TestProp & operator= (const TestProp & rhs) throw ();
        
        /**
         * Copy constructor
         */
        TestProp (const TestProp & rhs) throw ();

        /**
         * Defining a Test Suite
         */       
        CPPUNIT_TEST_SUITE (TestProp);
        
            //These are the test cases
            CPPUNIT_TEST (test_properties); 
            
            
        CPPUNIT_TEST_SUITE_END ();
    };

  }             /* End namespace PlatformMngt */
}               /* End namespace Cdmw */
#endif              /* INCL_PLATFORMMNGT_TESTPROP_HPP */
