/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <cstdlib>
#include <string>
#include <sstream>
#include <iostream>
#include <fstream>
#include <unistd.h>
#include <Foundation/ossupport/OS.hpp>


namespace
{
const int SUCCESS = 0;
const int FAILURE = 1;

const std::string DAEMON_PORT = "--daemonPort";

} // End anonymous namespace

using namespace Cdmw::OsSupport;


int main(int argc, char* argv[])
{

    try
    {

        std::cerr << "-------------------------" << std::endl
        << "      Start Supervision Killer" << std::endl
        << "-------------------------" << std::endl;
        for (int n=0; n < argc; n++)
            std::cerr << "argv[" << n <<"]: " << argv[n] << std::endl;
        std::cerr     << "-------------------------" << std::endl;

        std::string daemonPort = Cdmw::OsSupport::OS::get_option_value ( argc, argv, DAEMON_PORT );

        if ( daemonPort == "no" ) 
        {
            std::cerr << "Error: Option '--daemonPort' is mandatory!\n";
            exit( FAILURE );
        }
        std::cerr<< "\n\n\nDAEMON PORT :"<< daemonPort;

        // Read the SmgProcess PID
        std::string file_pid_str = "/tmp/CdmwDaemon_";
        file_pid_str = file_pid_str + daemonPort +  "/SMG_SUPERVISION_PROCESS.pid";
        std::ifstream file_pid(file_pid_str.c_str());
        
        if (!file_pid)
        {
            std::cerr << "*** ERROR: Cannot open PID input file "
            << file_pid_str
            << " ***"
            << std::endl;
            std::exit(FAILURE);
        }
        std::string pid_str;
        file_pid >> pid_str;
            int pid = atoi(pid_str.c_str());
            if (pid)
            {
                std::cerr << "\nSupervisionServer PID: "
                << pid
                << std::endl;
                OS::kill_process(pid,15);
            }
            else
            {
                std::cerr << "*** ERROR: Invalid PID from "
                << file_pid_str
                << " ***"
                << std::endl;
            }

            file_pid.close();
            return SUCCESS;

    }
    catch (const std::exception & ex)
    {
        std::cerr << "Caught generic exception :" << ex.what() << std::endl;
    }

}


