/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <fstream>
#include <limits>

#include "test_global_supervision_properties/TestProp.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include <SystemMngt/platformlibrary/Iterator.hpp>
#include <SystemMngt/tools/ManagedElementKey.hpp>
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"


using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestProp);

namespace Cdmw
{
  namespace PlatformMngt
  {
    
    TestProp::TestProp()
    {
    }

    void TestProp::setUp ()
    {
          try
          {
          //Getting system var
          std::string system_corbaloc = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";
          CORBA::ORB_var orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::instance ()->get_ORB ());
          CORBA::Object_var system_obj = orb->string_to_object (system_corbaloc.c_str());
          m_system = CdmwPlatformMngt::System::_narrow (system_obj.in ());
          
          
          //Setting up the startupKind parameters
          state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
          startup_mode = 0;
          specified_state_id = "";
          startupKind = new CdmwPlatformMngtBase::StartupKind();
          startupKind->state_retrieval = state_retrieval;
          startupKind->startup_mode = startup_mode;
          startupKind->specified_state_id  = CORBA::string_dup( specified_state_id.c_str() );
        
          //Setting the timeouts         
          long_timeout=7;
          short_timeout=3;
          
           
          }
          catch (const CORBA::SystemException& ex) 
          {
              std::cerr<< ex._name()<<std::endl;
              throw;
          }
          catch(...)
          {
          }
    }
    
    

    void TestProp::test_properties() 
    {
        std::string host2_name;
        std::string local_hostname = Cdmw::OsSupport::OS::get_hostname();
        
        //Getting the application APPL1 reference 
        CdmwPlatformMngt::Application_var application = m_system->get_application("APPL1");
        
        //References of system hosts
        CdmwPlatformMngt::Host_var Host1;
        CdmwPlatformMngt::Host_var Host2;       
        
        CORBA::ULong nb_hosts=m_system->get_number_of_hosts();
        CdmwPlatformMngt::Hosts_var hosts;
        CdmwPlatformMngt::HostsIterator_var rest;
        m_system->get_all_hosts(nb_hosts,hosts.out(),rest.out());
        
        //The current system has only two Hosts
        for (CORBA::ULong count = 0; count<nb_hosts; count++)
        {
            if( 0 != strcmp(hosts[count]->name(),local_hostname.c_str()) )
            {
                host2_name= hosts[count]->name();
                Host2= CdmwPlatformMngt::Host::_duplicate(hosts[count]);
            }else
            {
                Host1 = CdmwPlatformMngt::Host::_duplicate(hosts[count]);
            }
       }
        //Getting elements references
        get_references(local_hostname.c_str(),Host2->name());
        
        TEST_INFO("Verifing elements references");
        //Verifing elements references
        check_get_element(local_hostname.c_str(),Host2->name());
        
        
        //Getting the processes references
        CdmwPlatformMngt::Process_var killer_process_host1 = 
            application->get_process("Killer_Supervision_host1",local_hostname.c_str());
        CdmwPlatformMngt::Process_var killer_process_host2 = 
            application->get_process("Killer_Supervision_host2",Host2->name());
        CdmwPlatformMngt::Process_var sup_start_host1 = 
            application->get_process("Start_Supervision_host1",local_hostname.c_str());
        CdmwPlatformMngt::Process_var sup_start_host2 = 
            application->get_process("Start_Supervision_host2",Host2->name());

        
        //The the primary supervision is on Host1
        check_properties (Host1,Host2);

        //Running the supervision Killer 
        
        TEST_INFO("Killing the supervision on "<<local_hostname);
        sleep(short_timeout);
        
        try
        {
            killer_process_host1->initialise(startupKind.in());                 
        }
        catch(const CORBA::SystemException& ex)
        {
            TEST_INFO("\nError running process..."<<__LINE__<<__FILE__);
        }
        
        sleep(short_timeout);
        
        //The the primary supervision is on Host2
        check_properties (Host2,Host1);
        
        TEST_INFO("Verifing elements references");
        //Verifing elements references
        check_get_element(local_hostname.c_str(),Host2->name());
        
        //Restart the killed supervision on host1 
        TEST_INFO("Restarting the killed supervision on "<<local_hostname);
        sleep(short_timeout);

        try
        {
            sup_start_host1->initialise(startupKind.in());                  
        }
        catch(const CORBA::SystemException& ex)
        {
            TEST_INFO("\nError running Restart_Supervision process..."<<__LINE__<<__FILE__);
        }

        sleep(short_timeout);
        
        //The the primary supervision is on Host2
        check_properties (Host2,Host1);
        
        TEST_INFO("Verifing elements references");
        //Verifing elements references
        check_get_element(local_hostname.c_str(),Host2->name());
        
        //Running the supervision Killer on Host2
        TEST_INFO("Killing the supervision on "<<Host2->name());
        sleep(short_timeout);
        try
        {
            killer_process_host2->initialise(startupKind.in());                 
        }
        catch(const CORBA::SystemException& ex)
        {
            TEST_INFO("\nError running process..."<<__LINE__<<__FILE__);
        }

        sleep(long_timeout);
        check_properties (Host1,Host2);
        
        TEST_INFO("Verifing elements references");
        //Verifing elements references
        check_get_element(local_hostname.c_str(),Host2->name());

        //Restart the killed supervision on host2
        TEST_INFO("Restarting the killed supervision on "<<Host2->name());
        sleep(short_timeout);
        try
        {
            sup_start_host2->initialise(startupKind.in());                  
        }
        catch(const CORBA::SystemException& ex)
        {
            TEST_INFO("\nError running Restart_Supervision process..."<<__LINE__<<__FILE__);
        }
        sleep(short_timeout); 
        TEST_INFO("Verifing elements references");
        //Verifing elements references
        check_get_element(local_hostname.c_str(),Host2->name());
               
 }
    
    
   void TestProp::check_get_element(const char* hostname1,const char* hostname2)
   {
       //Verify the references of the all references
       try 
       {
           TEST_INFO (" m_sys-> name () :" << m_sys->name());
           TEST_INFO (" m_application-> name () :" << m_application->name());
           TEST_INFO (" m_component-> name ()   :" << m_component->name());
           TEST_INFO (" m_ftgroup-> name ()     :" << m_ftgroup->name());
           TEST_INFO (" m_lbgroup-> name ()     :" << m_lbgroup->name());
           TEST_INFO (" m_process-> name ()     :" << m_process->name());
           TEST_INFO (" m_appl_entity-> name () :" << m_appl_entity->name());
           TEST_INFO (" m_proc_entity-> name () :" << m_proc_entity->name());
           TEST_INFO (" m_host-> name ()        :" << m_host->name());
       }
       catch(...)
       {
           TEST_INFO(" \n\n TestProp::check_get_element  Exception raised ...");
           CPPUNIT_ASSERT(false);
       }
       
   } 
   void TestProp::get_references(const char* hostname1,const char* hostname2)
   {
       const char* sys_name="S1";
       const char* appl_name="APPL2";
       const char* ft_group_name="FTGroup1";
       const char* lb_group_name="LBGroup1";
       const char* appl_entity_name="ApplEntity";
       const char* proc_entity_name="ProcEntity";
       const char* proc_name="P1";
       const char* component_name="Comp1";
       
       try
       {
       std::string appl_path=Cdmw::PlatformMngt::ManagedElementKey::Get_application_path("S1","APPL2");
       std::string proc_path=Cdmw::PlatformMngt::ManagedElementKey::Get_process_path (sys_name,
                              appl_name,hostname2,proc_name);
       //Getting system reference (testing get_element method)
       m_sys= m_system->get_element( ( Cdmw::PlatformMngt::ManagedElementKey::Get_system_path(
               sys_name)).c_str() );
       
       //Getting application reference
       m_application = m_system->get_element(appl_path.c_str() );
       
       //Getting component reference
       m_component = m_system->get_element( ( Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(
               appl_path.c_str(),hostname2,proc_name,component_name)).c_str() );

       //Getting ftgroup reference
       m_ftgroup= m_system->get_element((
               Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_path (
                       sys_name,appl_name,ft_group_name)).c_str() );

       //Getting application entity reference
       m_appl_entity = m_system->get_element((
               Cdmw::PlatformMngt::ManagedElementKey::Get_entity_path (
                       appl_path.c_str(),appl_entity_name)).c_str() ); 
       //Getting process entity reference
       m_proc_entity = m_system->get_element((
               Cdmw::PlatformMngt::ManagedElementKey::Get_entity_path (
                       proc_path.c_str(),proc_entity_name)).c_str() ); 

       //Getting host reference
       m_host = m_system->get_element((
         Cdmw::PlatformMngt::ManagedElementKey::Get_host_path(sys_name,hostname2)).c_str() );
       
       //Getting process reference
       m_process = m_system->get_element((
               proc_path).c_str() )
                       ;
       //Getting lbgroup reference
       m_lbgroup = m_system->get_element((Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_path (
                       sys_name,appl_name,lb_group_name)).c_str() );
       }catch(...)
       {
           TEST_INFO(" FATAL error using get_element(...) ");
           CPPUNIT_ASSERT(false);
       }
       
       TEST_INFO("Using get_element with invalid element paths");
       try
       {
           CdmwPlatformMngt::ManagedElement_var null_element = m_system->get_element("");
           CPPUNIT_ASSERT(false);
       }catch(const CORBA::BAD_PARAM& ex)
       {
           TEST_INFO(" Exception correctly thrown "<< ex)
           CPPUNIT_ASSERT(true);
       }
       try
       {
           CdmwPlatformMngt::ManagedElement_var null_element = m_system->get_element("/INVALID/INVALID");
           CPPUNIT_ASSERT(false);
       }catch(const CORBA::BAD_PARAM& ex)
       {
           TEST_INFO(" Exception correctly thrown "<< ex)
           CPPUNIT_ASSERT(true);
       }
       
       

   }
    
    
    void TestProp::check_properties (CdmwPlatformMngt::Host_var HostPrimary,CdmwPlatformMngt::Host_var HostSecondary)
    {
        CORBA::Any_var property_host_any = new CORBA::Any();
        const char* result = 0 ;
        CORBA::Any_var property_any = new CORBA::Any();

        
        //SYSTEM: Reading the current value of the system property GLOBAL_SUPERVISION_HOST 
        try 
        {

            m_system->get_attribute (CdmwPlatformMngt::GLOBAL_SUPERVISION_HOST, property_any.out(), CdmwPlatformMngt::PRE_DEFINED);
            property_any.in() >>= result;
            TEST_INFO( "Checking the current value of GLOBAL_SUPERVISION_HOST :" <<  result);
            CPPUNIT_ASSERT( ( 0 == strcmp(result,HostPrimary->name()) ) );
        }
        catch(const CdmwPlatformMngt::AttributeNotFound&)
        {
            TEST_INFO("\nERROR READING THE PROPERTY GLOBAL_SUPERVISION_HOST line:"<<__LINE__<<" file:"<<__FILE__);
            CPPUNIT_ASSERT(false);
        }

        //HOSTPRIMARY :Reading the current value of the property GLOBAL_SUPERVISION 
        TEST_INFO( "Checking the current value of the property GLOBAL_SUPERVISION on " << HostPrimary->name());
        CORBA::Boolean property_val;
        try {

            HostPrimary->get_attribute (CdmwPlatformMngt::GLOBAL_SUPERVISION, property_any.out(), CdmwPlatformMngt::PRE_DEFINED);
            property_any.in() >>=CORBA::Any::to_boolean(property_val);
            if(property_val) {
                TEST_INFO(HostPrimary->name()<< ":  GLOBAL_SUPERVISION is true " );
                CPPUNIT_ASSERT(true);
            } 
            else
            {
                TEST_INFO(HostPrimary->name()<< ":  GLOBAL_SUPERVISION is false " );
                CPPUNIT_ASSERT(false);}
        }
        catch(const CdmwPlatformMngt::AttributeNotFound&)
        {
            TEST_INFO(" ERROR READING THE PROPERTY GLOBAL_SUPERVISION line:"<<__LINE__<<" file:"<<__FILE__);
        }
        
        //HOSTSECONDARY :Reading the current value of the property GLOBAL_SUPERVISION 
        TEST_INFO( "Checking the current value of the property GLOBAL_SUPERVISION on " << HostSecondary->name());
        
        try {

            HostSecondary->get_attribute (CdmwPlatformMngt::GLOBAL_SUPERVISION, property_any.out(), CdmwPlatformMngt::PRE_DEFINED);
            property_any.in() >>=CORBA::Any::to_boolean(property_val);
                if(!property_val) {
                    TEST_INFO(HostSecondary->name()<< ":  GLOBAL_SUPERVISION is false " );
                    CPPUNIT_ASSERT(true);
                } 
                else
                {
                    TEST_INFO(HostSecondary->name()<< ":  GLOBAL_SUPERVISION is true " );
                    CPPUNIT_ASSERT(false);}
        }
        catch(const CdmwPlatformMngt::AttributeNotFound&)
        {
            TEST_INFO(" ERROR READING THE PROPERTY GLOBAL_SUPERVISION line:"<<__LINE__<<" file:"<<__FILE__);
        }
   }

}/* End namespace PlatformMngt */
}/* End namespace Cdmw */





