/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/* @author Fabrizio Lorenna <f.lorenna@vitrociset.it> */

#include <iostream>
#include <sstream>
#include <limits>

#include "test_lbgroup_nav/TestLBGroupNav.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"

#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"
#include "idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"

#include <platformftproxy/InitUtils.hpp>
#include <platformftproxy/SystemProxy.hpp>
#include <SystemMngt/platformftproxy/AccessorLookupService.hpp>
#include <SystemMngt/platformftproxy/DefaultAccessorLookupService.hpp>
#include <SystemMngt/platformlibrary/Iterator.hpp>

#include <FaultTolerance/ftreplicationmanager/InitUtils.hpp>
#include <FaultTolerance/idllib/FT.stub.hpp>
#include <platformftaccessorlibrary/SystemAccessor_impl.hpp>
#include "tao/AnyTypeCode/TypeCode.h"


using namespace std;

using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestLBGroupNav);

namespace Cdmw
{
  namespace PlatformMngt
  {



    TestLBGroupNav::TestLBGroupNav ()
    {

    }


    TestLBGroupNav::~TestLBGroupNav ()
    {

    }
    bool TestLBGroupNav::m_is_one_shot_initialized = false;

    CdmwPlatformMngt::System_var TestLBGroupNav::mysystem;
    CdmwPlatformMngt::Process_var TestLBGroupNav::myprocess;
    CdmwPlatformMngt::Application_var TestLBGroupNav::app;

    void TestLBGroupNav::setUp ()
    {

      TestLBGroupNav::one_shot_init ();
    }

    void TestLBGroupNav::one_shot_init ()
    {

      if (!m_is_one_shot_initialized)
        {
          m_is_one_shot_initialized = true;

          try
          {
            CORBA::ORB_var orb;
            orb =
              CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::
                                      instance ()->get_ORB ());

            PortableServer::POA_var poa =
              PortableServer::POA::_duplicate (Cdmw::TestUtils::
                                               CORBATestManager::instance ()->
                                               get_POA ());

            PortableServer::POAManager_var poaMgr =
              PortableServer::POAManager::_duplicate (poa->the_POAManager ());
            poaMgr->activate ();


        std::string system_corbaloc = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";
            CORBA::Object_var system_obj = orb->string_to_object (system_corbaloc.c_str());

            mysystem = CdmwPlatformMngt::System::_narrow (system_obj.in ());


            if (CORBA::is_nil (mysystem.in ()))
              {
                exit (-1);
              }

            /*from System, use a method to retrieve an application */
            app = mysystem->get_application ("HELLO_WORLD");

            /*from the application, use a method to retrieve a Process */

            /*retrieve the current hostname */
            char *hostname = getenv ("HOSTNAME");

            myprocess = app->get_process ("P12", hostname);

          }
          catch (const Cdmw::Exception & ex)
          {
            std::cerr << ex.what () << std::endl;
            exit (-1);
          }
          catch (const CORBA::Exception & ex)
          {
            std::cerr << ex << std::endl;
            exit (-1);
          }
          catch (...)
          {
            std::cerr << "unexpected exception" << std::endl;
            exit (-1);
          }
        }
    }

    void TestLBGroupNav::get_lb_group()
    {

      CdmwPlatformMngt::LBGroup_var lbg ;

      //test to retrieve the lbgroup ptr ...
      try     
        {
         lbg = app->get_lb_group("FABLBGROUP1");

         char *lbg_name = lbg->name() ;

         std::string lbgroup_name( lbg_name)  ;

         if ( lbgroup_name.compare("FABLBGROUP1") == 0)
             CPPUNIT_ASSERT(1);  /*test ok*/
         else
             CPPUNIT_ASSERT(0); /*test not ok */
       }
      catch(...)
        {       
        //error 
        CPPUNIT_ASSERT(0);
        }

    }


    void TestLBGroupNav::get_all_lb_groups()
    {

    CdmwPlatformMngt::LBGroups_var lbgroups;

    CdmwPlatformMngt::LBGroupsIterator_var rest ;


    CdmwPlatformMngt::LBGroup_var lbg3 ;

    try
        {
         app->get_all_lb_groups(2, lbgroups, rest );

         CORBA::ULong len_subset1 = lbgroups->length() ;

         std::string lbg_name1( ( (*lbgroups)[0])->name() );
         std::string lbg_name2( ( (*lbgroups)[1])->name() );

         rest->next_one(lbg3) ;

         std::string lbg_name3(  lbg3->name() );

         if ( ( len_subset1 == 2) &&
         (lbg_name1.compare("FABLBGROUP1") == 0) &&
         (lbg_name2.compare("FABLBGROUP2") == 0) &&
         (lbg_name3.compare("FABLBGROUP3") == 0) )
           {
           CPPUNIT_ASSERT(1); //ok
           }
             else
           CPPUNIT_ASSERT(0); //not ok


    }
    catch(...)
        {
        //error
        CPPUNIT_ASSERT(0);
        }


    }

    void TestLBGroupNav::get_number_of_lb_groups()
    {

    try
    {
    CORBA::ULong num_lbgroups = app->get_number_of_lb_groups() ;


    if ( num_lbgroups == 3)
      CPPUNIT_ASSERT(1); //ok
    else
      CPPUNIT_ASSERT(0); //not ok

    }
    catch(...)
      {
        CPPUNIT_ASSERT(0);
      }

    }

    void TestLBGroupNav::get_application_from_lbgroup()
    {


      CdmwPlatformMngt::LBGroup_var lbg ;

      //test to retrieve the appl. form lbgroup ptr ...
      try
        {
         lbg = app->get_lb_group("FABLBGROUP1");


         CdmwPlatformMngt::Application_var app_local =
            Application::_narrow(lbg->get_parent()) ;


          char *app_name =  app_local->name()  ;

          std::string appname_str ( app_name ) ;

         if ( (appname_str.compare("HELLO_WORLD") == 0))
            CPPUNIT_ASSERT(1); //test ok
         else
            CPPUNIT_ASSERT(0); //test not ok


         }
      catch(...)
        {
        //error
        CPPUNIT_ASSERT(0);
        }

    }

    void TestLBGroupNav::get_lbgroup_status ()
    {
    CORBA::String_var info ;
    CdmwPlatformMngt::LBGroup_var lbg ;

    try
      {
      lbg = app->get_lb_group("FABLBGROUP1");

      CdmwPlatformMngt::TechnicalStatus stat = lbg->get_status(info) ;
      std::string info_str ( info ) ;

      if ( (stat == CdmwPlatformMngt::INITIAL) &&
            (info_str.compare("") == 0) )
          CPPUNIT_ASSERT (1); /*test ok */
      else
          CPPUNIT_ASSERT(0);  /*test not ok*/

      }
      catch (...)
      {
    //error
        CPPUNIT_ASSERT (0);
      }

    }



  }                             /* End namespace PlatformMngt */
}                               /* End namespace Cdmw */
