/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "test_max_restart_app_proc/MyProcessBehaviour.hpp"

#include <iostream>

MyProcessBehaviour::MyProcessBehaviour() :
    m_isactionvalid(false),
    m_method(TestAutomaticActions::NOMETHOD),
    m_action(TestAutomaticActions::NOACTION),
    m_parameter((TestAutomaticActions::Parameter)0)
{    
    m_nbrOfInitSteps = 1;
}


MyProcessBehaviour::~MyProcessBehaviour() throw ()
{
}
    
CORBA::ULong MyProcessBehaviour::nb_steps() throw(CORBA::SystemException)
{       
    std::cout << "******* nb_steps *******" << std::endl;
    
    if (m_isactionvalid) { 
        if ((m_method == TestAutomaticActions::NBSTEPS) && (m_action == TestAutomaticActions::NOACTION)) {
            return (CORBA::ULong)(unsigned int)m_parameter;    
        }
        else {    
            DoAction(TestAutomaticActions::NBSTEPS);
        }        
    }    
    return m_nbrOfInitSteps;
}
    
    
CORBA::Object_ptr MyProcessBehaviour::get_service() throw(CORBA::SystemException)
{
    return CORBA::Object::_nil();
}
    
    
void MyProcessBehaviour::initialise(const CdmwPlatformMngtBase::StartupKind& startup_kind)
        throw(CdmwPlatformMngt::ProcessDelegate::BadOrder, CORBA::SystemException)
{
    std::cout << "******* initialise *******" << std::endl;
    
    if (m_isactionvalid) DoAction(TestAutomaticActions::INITIALISE);

}


void MyProcessBehaviour::next_step()
    throw(CdmwPlatformMngt::ProcessDelegate::InvalidStep, CORBA::SystemException)
{
    std::cerr << "******* next_step *******" << std::endl;
}
    
    
void MyProcessBehaviour::run()
    throw(CdmwPlatformMngt::ProcessDelegate::NotReadyToRun,
        CdmwPlatformMngt::ProcessDelegate::AlreadyDone,
        CORBA::SystemException)
{
    std::cout << "******* run *******" << std::endl;
    
    if (m_isactionvalid) DoAction(TestAutomaticActions::RUN);

}

void MyProcessBehaviour::stop() throw(CORBA::SystemException)
{
    std::cout << "******* stop *******" << std::endl;
    
    if (m_isactionvalid) DoAction(TestAutomaticActions::STOP);

}

bool MyProcessBehaviour::is_alive()
    throw(CORBA::SystemException)
{
    std::cout << "******* is_alive *******" << std::endl;

    if (m_isactionvalid) DoAction(TestAutomaticActions::ISALIVE);

    return true;
}

bool MyProcessBehaviour::SetAction( TestAutomaticActions::Method m, 
                    TestAutomaticActions::Action a, 
                    TestAutomaticActions::Parameter p) 
{
    // Check if this is a valid action
    if (m == TestAutomaticActions::NOMETHOD && 
        a == TestAutomaticActions::NOACTION ) {
    
        m_isactionvalid = true;
        return m_isactionvalid;   
    }
    
    if (m == TestAutomaticActions::NOMETHOD) {
        m_isactionvalid = false;
        return m_isactionvalid;
    }   
    
    if ((m != TestAutomaticActions::NBSTEPS) && (a == TestAutomaticActions::NOACTION)) {
        m_isactionvalid = false;
        return m_isactionvalid;
    }   
    
    // If we are here action is valid
    m_isactionvalid = true;
    m_method = m;
    m_action = a;
    m_parameter = p;   
    return m_isactionvalid;   
}

void MyProcessBehaviour::DoAction( TestAutomaticActions::Method m)
{
    if (m != m_method) return;
  
    if (m_action == TestAutomaticActions::TIMEOUT) {
        usleep((unsigned int) m_parameter * 1000);
    }

    if (m_action == TestAutomaticActions::EXIT) {
        exit((unsigned int) m_parameter);
    }
    
}    
    

