/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_NAVIGABILITY_TEST_HPP
#define INCL_PLATFORMMNGT_NAVIGABILITY_TEST_HPP

#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/common/System.hpp"
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/Exception.hpp>
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"

namespace Cdmw
{
  namespace PlatformMngt
  {
    
    class NavigabilityTest : public CppUnit::TestFixture
    {

    public:

        NavigabilityTest();
        
        void setUp ();
 
        //Test Cases 
        void exception_control();
        void lb_group_to_component();
        void ft_group_to_component();
        void component_to_group();
        
             
    private:
    
        //System reference
        CdmwPlatformMngt::System_var system;


        //Application references
        CdmwPlatformMngt::Application_var app;
        CdmwPlatformMngt::FTGroup_var ftgroup;
        CdmwPlatformMngt::LBGroup_var lbgroup;
        
        //Defining a component
        CdmwPlatformMngt::CcmComponent_var component;
        
        CdmwPlatformMngtBase::StartupKind_var startupKind;
        CdmwPlatformMngtBase::StateRetrieval state_retrieval;
        
        CORBA::String_var statusInfo;
        CORBA::ORB_var orb;
        CORBA::String_var any;
        
        //Startupkind parameter
        int val;
        unsigned long startup_mode;
        std::string specified_state_id;
        
    private:
    
        /**
         * Assignment operator
         */
        NavigabilityTest & operator= (const NavigabilityTest & rhs) throw ();
        
        /**
         * Copy constructor
         */
        NavigabilityTest (const NavigabilityTest & rhs) throw ();

        /**
         * Defining a Test Suite
         */       
        CPPUNIT_TEST_SUITE (NavigabilityTest);
        
            //These are the test cases
        
        CPPUNIT_TEST (exception_control);
        CPPUNIT_TEST (ft_group_to_component);
        CPPUNIT_TEST (lb_group_to_component);
        CPPUNIT_TEST (component_to_group); 
             
            
        CPPUNIT_TEST_SUITE_END ();
    };

  }             /* End namespace PlatformMngt */
}               /* End namespace Cdmw */
#endif              /* INCL_PLATFORMMNGT_NAVIGABILITY_TEST_HPP */
