/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <fstream>
#include <limits>
#include <string>

#include "test_navigability_ccm_groups/navigabilityTest.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"

#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"
#include "idllib/CdmwPlatformMngtProcess.stub.hpp"

using namespace Cdmw;
using namespace PlatformMngt;
using namespace CdmwPlatformMngt;


CPPUNIT_TEST_SUITE_REGISTRATION (NavigabilityTest);

namespace Cdmw
{
  namespace PlatformMngt
  {

  NavigabilityTest::NavigabilityTest()
    {
    }

    void NavigabilityTest::setUp ()
    {
          try
          {
              //Getting system var
              std::string system_corbaloc = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";
              orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::instance ()->get_ORB ());
              CORBA::Object_var system_obj = orb->string_to_object (system_corbaloc.c_str());
              system = CdmwPlatformMngt::System::_narrow (system_obj.in ());

              //Setting up the startupKind parameters
              state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
              startup_mode = 0;
              specified_state_id = "";
              startupKind = new CdmwPlatformMngtBase::StartupKind();
              startupKind->state_retrieval = state_retrieval;
              startupKind->startup_mode = startup_mode;
              startupKind->specified_state_id  = CORBA::string_dup( specified_state_id.c_str() );
              
              
              //Getting the application reference
              app=system->get_application("APPL1"); 
              
              //Getting the right LB and FT groups
              try
              {
                  ftgroup = app->get_ft_group("FtGroup1");
                  
                  lbgroup = app->get_lb_group("LbGroup1");

              }catch (const CdmwPlatformMngt::GroupNotFound &e)
              {
                  std::cerr<<e<<std::endl;
                  CPPUNIT_ASSERT (false);
              }

          }
          catch (const CORBA::SystemException& ex) 
          {
              std::cerr<< ex._name()<<std::endl;
              throw;
          }
          catch(...)
          {
          }
    }
    
    void NavigabilityTest::exception_control ()
    {
        try
    {    
        std::cerr<< "\n***********************************************\n";
        std::cerr<< "                  EXCEPTION CONTROL";
        std::cerr<< "\n***********************************************\n";

        //Getting an not existent LB group
        TEST_INFO("Looking for a LB group with a wrong name , expected exception ");
        try
        {
            lbgroup = app->get_lb_group("LB_WRONG_NAME");
        }
        catch (const CdmwPlatformMngt::GroupNotFound &e)
        {
            std::cerr<<e<<std::endl;
            CPPUNIT_ASSERT (true);
        }

        //Getting an not existent FT group
        TEST_INFO("Looking for a FT group with a wrong name , expected exception ");
        try
        {
            ftgroup = app->get_ft_group("FT_WRONG_NAME");
        }
        catch (const CdmwPlatformMngt::GroupNotFound &e)
        {
            std::cerr<<e<<std::endl;
            CPPUNIT_ASSERT (true);
        }
        
        //Getting an not existent component
        TEST_INFO("Controlling the API  RedundancyGroup::get_component(..) for a FTGroup with a wrong name , expected exception ");
        try
        {
            component = ftgroup->get_component("COMP_WRONG_NAME","WRONG_HOST");

        }catch (const CdmwPlatformMngt::ComponentNotFound &e)
        {
            std::cerr<<e<<std::endl;
            CPPUNIT_ASSERT (true);
        }
        
        //Getting an not existent component
        TEST_INFO("Controlling the API  RedundancyGroup::get_component(..) for a LBGroup with a wrong name , expected exception ");
        try
        {
            component = lbgroup->get_component("COMP_WRONG_NAME","WRONG_HOST");

        }catch (const CdmwPlatformMngt::ComponentNotFound &e)
        {
            std::cerr<<e<<std::endl;
            CPPUNIT_ASSERT (true);
        }
    }catch (const CORBA::SystemException &e)
        {
            std::cerr<<e<<std::endl;
            CPPUNIT_ASSERT (true);
        }


    }
    
    void NavigabilityTest::ft_group_to_component ()
    {
        std::cerr<< "\n***********************************************\n";
        std::cerr<< "          FT GROUP -> COMPONENT CHECKING";
        std::cerr<< "\n***********************************************\n";
        
        try 
        {

            TEST_INFO("Controlling the API  RedundancyGroup::get_number_of_components(..) ");
            std::cerr <<"--> NUMBER OF FT GROUP's COMPONENT = " << ftgroup->get_number_of_components()<<std::endl;
            if ( 2 != ftgroup->get_number_of_components() )
            {
                CPPUNIT_ASSERT (false);
            }

            TEST_INFO("Controlling the API  RedundancyGroup::get_minimum_size(..) ");
            std::cerr<<"--> MINIMUM NUMBER of REPLICAS = "<< ftgroup->get_minimum_size()<<std::endl;

            if ( 2 != ftgroup->get_minimum_size())
            {
                CPPUNIT_ASSERT (false);
            }

            TEST_INFO("Controlling the API  RedundancyGroup::get_style(..) ");
            if ( CdmwPlatformMngt::RG_WARM_PASSIVE != ftgroup->get_style())
            {
                CPPUNIT_ASSERT (false);
            }
            else
            {
                std::cerr<<"--> REPLICATION STYLE = WARM PASSIVE"<<std::endl;
            } 
            

            TEST_INFO("Controlling the API  RedundancyGroup::get_status(..) ");
            std::cerr<<"(expected value of the status : INITIAL)"<<std::endl;

            if ( CdmwPlatformMngt::INITIAL != ftgroup->get_status(any.out()))
            {
                CPPUNIT_ASSERT (false);
            }
            else
            {
                std::cerr<<"--> TECHNICAL STATUS = COMPLETE_FAILURE "<<std::endl;
            }
            
        }
        catch (const CORBA::SystemException& e)
        {
            std::cerr << "EXCEPTION : "<<e <<std::endl;
            CPPUNIT_ASSERT (false);
        }


        

        CdmwPlatformMngt::ProcessesIterator_var procIter;
        CORBA::ULong nbProcs= app->get_number_of_processes();
        CdmwPlatformMngt::Processes_var procs= app->get_processes(nbProcs, procIter.out());

        CORBA::ULong index = 1 ; 

        char* hostname = ((procs[index]->get_host())->name());
        //Getting a right component
        TEST_INFO("Controlling the API  RedundancyGroup::get_component(..)  ");
        try
        {
            component = ftgroup->get_component("CompFT2",hostname);
            if( "CompFT2" == component->name()) CPPUNIT_ASSERT (true);
            std::cerr<<"--> COMPONENT NAME = "<<component->name()<<std::endl;

        }catch (const CdmwPlatformMngt::ComponentNotFound &e)
        {
            std::cerr<<e<<std::endl;
            CPPUNIT_ASSERT (false);
        }
        catch (const CORBA::SystemException& e )
        {
            std::cerr << "EXCEPTION : "<<e <<std::endl;
        }

        try
        {
            TEST_INFO("Controlling the API  RedundancyGroup::get_all_components(..)  ");
            
            CORBA::ULong how_many;
            CdmwPlatformMngt::CcmComponents_var ncomponents;
            CdmwPlatformMngt::ComponentsIterator_var rest;

            ftgroup->get_all_components (how_many,ncomponents.out(),rest.out() );

            for (unsigned int i=0 ; i<  ncomponents->length() ; i++ )
            {
                std::cerr<< "--> NAME of COMPONENT "<< i+1 <<" = "<< ncomponents[i]->name()
                << " of GROUP = "<< (ncomponents[i]->get_component_group())->name()<<std::endl;
            }
        }
        catch (const CORBA::SystemException& e)
        {
            std::cerr << "EXCEPTION : "<<e <<std::endl;
            CPPUNIT_ASSERT (false);
        }


    }
    
    
    
    void NavigabilityTest::lb_group_to_component ()
    {
        std::cerr<< "\n***********************************************\n";
        std::cerr<< "          LB GROUP -> COMPONENT CHECKING";
        std::cerr<< "\n***********************************************\n";
        try 
        {
            TEST_INFO("Controlling the API  RedundancyGroup::get_number_of_components(..) ");
            std::cerr << "--> NUMBER OF LB GROUP's COMPONENT = " << lbgroup->get_number_of_components()<<std::endl;
            if ( 2 != lbgroup->get_number_of_components() )
            {
                CPPUNIT_ASSERT (false);
            }

            TEST_INFO("Controlling the API  RedundancyGroup::get_minimum_size(..) ");
            std::cerr<<  "--> MINIMUM NUMBER of REPLICAS = "<< lbgroup->get_minimum_size()<<std::endl;

            if ( 2 != lbgroup->get_minimum_size())
            {
                CPPUNIT_ASSERT (false);
            }

            TEST_INFO("Controlling the API  RedundancyGroup::get_style(..) ");
            if ( CdmwPlatformMngt::LB_ROUND_ROBIN != lbgroup->get_style())
            {
                CPPUNIT_ASSERT (false);
            }
            else
            {
                std::cerr<<"--> REPLICATION STYLE = ROUND ROBIN"<<std::endl;
            } 
            
            TEST_INFO("Controlling the API  RedundancyGroup::get_status(..) ");
            std::cerr<<"(expected value of the status : INITIAL) "<<std::endl;

            if ( CdmwPlatformMngt::INITIAL != lbgroup->get_status(any.out()))
            {
                CPPUNIT_ASSERT (false);
            }
            else
            {
                std::cerr<<"--> TECHNICAL STATUS = COMPLETE_FAILURE "<<std::endl;
            }

        }
        catch (const CORBA::SystemException& e)
        {
            std::cerr << "EXCEPTION : "<<e <<std::endl;
            CPPUNIT_ASSERT (false);
        }

        CdmwPlatformMngt::ProcessesIterator_var procIter;
        CORBA::ULong nbProcs= app->get_number_of_processes();
        CdmwPlatformMngt::Processes_var procs= app->get_processes(nbProcs, procIter.out());

        CORBA::ULong index = 0; 

        char* hostname = ((procs[index]->get_host())->name());

        //Getting a right component
        TEST_INFO("Controlling the API  RedundancyGroup::get_component(..)  ");
        try
        {
            component = lbgroup->get_component("CompLB2",hostname);
            if( "CompLB2" == component->name()) CPPUNIT_ASSERT (true);
            std::cerr<<"--> COMPONENT NAME = "<<component->name()<<std::endl;

        }catch (const CdmwPlatformMngt::ComponentNotFound &e)
        {
            std::cerr<<e<<std::endl;
            CPPUNIT_ASSERT (false);
        }

        try
        {
            TEST_INFO("Controlling the API  RedundancyGroup::get_all_components(..)  ");

            CORBA::ULong how_many;
            CdmwPlatformMngt::CcmComponents_var ncomponents;
            CdmwPlatformMngt::ComponentsIterator_var rest;

            lbgroup->get_all_components (how_many,ncomponents.out(),rest.out() );

            for (unsigned int i=0 ; i<  ncomponents->length() ; i++ )
            {
                std::cerr<< "--> NAME of COMPONENT "<< i+1 <<" = "<< ncomponents[i]->name()
                << " of GROUP = "<< (ncomponents[i]->get_component_group())->name()<<std::endl;
            }
        }
        catch (const CORBA::SystemException& e)
        {
            std::cerr << "EXCEPTION : "<<e <<std::endl;
            CPPUNIT_ASSERT (false);
        }
    }
    
 
    

    
    void NavigabilityTest::component_to_group ()
    {
        
        std::cerr<< "\n***********************************************\n";
        std::cerr<< "          COMPONENT -> FT GROUP CHECKING";
        std::cerr<< "\n***********************************************\n";
        try
        {
            CORBA::ULong how_many = 2;
            CdmwPlatformMngt::CcmComponents_var ncomponents;
            CdmwPlatformMngt::ComponentsIterator_var rest;
            
            ftgroup->get_all_components (how_many,ncomponents.out(),rest.out() );
            
            TEST_INFO("Controlling the API  CcmComponent::get_component_group(..)  ");
            for (unsigned int i=0 ; i<  ncomponents->length() ; i++ )
            {
                std::cerr<< "--> NAME of COMPONENT "<< i+1 <<" = "<< ncomponents[i]->name()
                << " of GROUP = "<< (ncomponents[i]->get_component_group())->name()<<std::endl;
            }
        }
        catch (const CORBA::SystemException& e)
        {
            std::cerr << "EXCEPTION : "<<e <<std::endl;
            CPPUNIT_ASSERT (false);
        }       

        std::cerr<< "\n***********************************************\n";
        std::cerr<< "          COMPONENT -> LB GROUP CHECKING";
        std::cerr<< "\n***********************************************\n";

        try
        {
            CORBA::ULong how_many = 2;
            CdmwPlatformMngt::CcmComponents_var ncomponents;
            CdmwPlatformMngt::ComponentsIterator_var rest;
            
            lbgroup->get_all_components (how_many,ncomponents.out(),rest.out() );

            TEST_INFO("Controlling the API  CcmComponent::get_component_group(..)  ");
            for (unsigned int i=0 ; i<  ncomponents->length() ; i++ )
            {
                std::cerr<< "--> NAME of COMPONENT "<< i+1 <<" = "<< ncomponents[i]->name()
                << " of GROUP = "<< (ncomponents[i]->get_component_group())->name()<<std::endl;
            }
        }
        catch (const CORBA::SystemException& e)
        {
            std::cerr << "EXCEPTION : "<<e <<std::endl;
            CPPUNIT_ASSERT (false);
        }       
        
    }
    
}/* End namespace PlatformMngt */
}/* End namespace Cdmw */






