/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORPERIODIC_TESTTPERIODICNOTIFICATION_HPP
#define INCL_PLATFORPERIODIC_TESTTPERIODICNOTIFICATION_HPP

#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/common/System.hpp"
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/Exception.hpp>

#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"
#include "idllib/CdmwPlatformMngtPeriodicNotifier.stub.hpp"

#include <tools/SingleShotTimer.hpp>

#include "test_periodic_notification/GroupObserver_impl.hpp"
#include "test_periodic_notification/Observer_impl.hpp"
#include "test_periodic_notification/TestTimerObserver.hpp"


namespace Cdmw
{
namespace PlatformMngt
{
//namespace Periodic
//{

class TestPeriodicNotification : public CppUnit::TestFixture
{

public:

    TestPeriodicNotification();

    void setUp ();

    virtual void test_single_periodic_observer ();
    
    virtual void test_group_periodic_observer ();


private:

    //Host name
    std::string hostname;

    //Number of message recieved
    int num_mess;
    
    //Wating time to recieve message
    bool wait;

    //System reference
    CdmwPlatformMngt::System_var system;

    //Periodic Notifier reference
    CdmwPlatformMngt::Periodic::Notifier_var notifier;

    //Periodic Observer reference
    Cdmw::PlatformMngt::Periodic::Observer_impl  obs_impl;
    CdmwPlatformMngt::Periodic::Observer_var observer;
    std::string obName ;
    
    //Periodic Observer reference
    Periodic::GroupObserver_impl *  group_obs1, group_obs2;
    std::string miop_group ;

    //Property name sequence
    CosPropertyService::PropertyNames_var property_names;
    CORBA::ULong len ;

    // The timer interval.
    TimeBase::TimeT period;

    // The timer
    SingleShotTimer* timer;
    //Cdmw::PlatformMngt::Periodic::TestTimerObserver* timer_observer;
    TestTimerObserver* timer_observer;

    CdmwPlatformMngtBase::StartupKind_var startupKind;
    CdmwPlatformMngtBase::StateRetrieval state_retrieval;

    CORBA::String_var statusInfo;
    CORBA::ORB_var orb;
    PortableGroup::GOA_var rootGOA ;

    //Startupkind parameter
    int val;
    unsigned long startup_mode;
    std::string specified_state_id;

private:

    /**
    * Assignment operator
    */
    TestPeriodicNotification & operator= (const TestPeriodicNotification & rhs) throw ();

    /**
    * Copy constructor
    */
    TestPeriodicNotification (const TestPeriodicNotification & rhs) throw ();

    /**
    * Defining a Test Suite
    */       
    CPPUNIT_TEST_SUITE (TestPeriodicNotification);

    //These are the test cases
    CPPUNIT_TEST (test_single_periodic_observer); 
    
    CPPUNIT_TEST (test_group_periodic_observer); 

    CPPUNIT_TEST_SUITE_END ();
};

//}
}/* End namespace PlatformMngt */
}/* End namespace Cdmw */
#endif/* #ifndef INCL_PLATFORPERIODIC_TESTTPERIODICNOTIFICATION_HPP */
