/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
* @author Anna Domnina <gdomnina@sesm.it>
*/

#ifndef INCL_PLATFORMMNGT_TEST_PERIODIC_NOTIFICATION_HPP
#define INCL_PLATFORMMNGT_TEST_PERIODIC_NOTIFICATION_HPP
#include <iostream>
#include <fstream>
#include <limits>
#include <string>

#include "test_periodic_notification/TestTimerObserver.hpp"
#include "test_periodic_notification/TestPeriodicNotification.hpp"
#include "test_periodic_notification/Observer_impl.hpp"
#include "test_periodic_notification/GroupObserver_impl.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "SystemMngt/platformlibrary/Iterator.hpp"

#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include <SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp>
#include "SystemMngt/idllib/CdmwPlatformMngtPeriodicNotifier.stub.hpp"

#include "orbsvcs/orbsvcs/PortableGroupC.h"

using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestPeriodicNotification);


namespace Cdmw
{
namespace PlatformMngt
{


TestPeriodicNotification::TestPeriodicNotification()
{
}

void TestPeriodicNotification::setUp ()
{
    try
    {
        //Getting system var
        std::string system_corbaloc = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";
        orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::instance ()->get_ORB ());
        CORBA::Object_var system_obj = orb->string_to_object (system_corbaloc.c_str());
        system = CdmwPlatformMngt::System::_narrow (system_obj.in ());

        //////////////////
        CORBA::Object_var obj = orb->resolve_initial_references ("RootPOA");
        //Get the root GOA
        rootGOA = PortableGroup::GOA::_narrow(obj.in());
        //////////////////
        
        //Setting up the startupKind parameters
        state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
        startup_mode = 0;
        specified_state_id = "";
        startupKind = new CdmwPlatformMngtBase::StartupKind();
        startupKind->state_retrieval = state_retrieval;
        startupKind->startup_mode = startup_mode;
        startupKind->specified_state_id  = CORBA::string_dup( specified_state_id.c_str() );

    }
    catch (const CORBA::SystemException& ex) 
    {
        std::cerr<< ex._name()<<std::endl;
        throw;
    }
    catch(...)
    {
    }
}

void TestPeriodicNotification::test_single_periodic_observer()
{
    TimeBase::TimeT period_wait;
    num_mess = 5;
    hostname = OsSupport::OS::get_hostname();
    period = 1000;    
    obName = "PeriodicObserver";
    std::string tmp_string;
    
    //control if period is too small (100ms)
    if(period<=100)
    {
        period_wait = num_mess * period * 2;
    }
    else
    {
        period_wait = num_mess*(period+(period/10))+period;
    }
    

    //activate periodic observer
    observer = obs_impl._this(); 
    
    //initialise the property to be registered to
    property_names = new CosPropertyService::PropertyNames();
    
    
    /***************************************************
    * Check if PropertyNameInvalid exception is raised
    **************************************************/
    TEST_INFO(" START PERIODIC OBSERVER TEST"<<std::endl);
    TEST_INFO(" 1) TEST SINGLE PERIODIC OBSERVER INTERFACE"<<std::endl);
    TEST_INFO("============================================================"<<std::endl);
    TEST_INFO(" Registering periodic observer with invalid property name..."<<std::endl);
    
    //.attribute instead of .attr 
    tmp_string = "S1.sys/APPL1.appl/" + hostname + ".host/P12.proc/Cdmw.RequestedRestartWarmth.attribute";

    //initialise the property to be registered to
    len = 1;
    property_names->length(len);
    //attribute have to be specified with postfix ".attr"
    property_names[len-1]= CORBA::string_dup(tmp_string.c_str());

    try
    {
        //get a periodic notifier reference from system   
        notifier=system->periodic_notifier();

        //register periodic observer 
        notifier->register_observer(obName.c_str(), property_names.in(), observer.in(), period);
        
        TEST_INFO(" Registered with property_names invalid");
        CPPUNIT_ASSERT (false);
        
    } catch (const CdmwPlatformMngt::PropertyNameInvalid &e)
    {
        TEST_INFO(" Ok. "<<e<<" Exception is raised");
        CPPUNIT_ASSERT (true);  
        
    } catch (const CdmwPlatformMngt::ElementAlreadyExists & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    }
    
    
    /********************************************************
    * Registering periodic observer with valid property name
    *******************************************************/
    TEST_INFO("==========================================================="<<std::endl);
    TEST_INFO(" Registering periodic observer with valid property name...");
    
    //initialise the property to be registered to
    len = 1;
    property_names->length(len);

    tmp_string = "S1.sys/APPL1.appl/" + hostname + ".host/P11.proc/Cdmw.RequestedRestartWarmth.attr";
    property_names[len-1] = CORBA::string_dup( tmp_string.c_str() );
    TEST_INFO("property is "<<tmp_string.c_str()<<std::endl);
    

    try
    {
        //register periodic observer 
        notifier->register_observer(obName.c_str(), property_names.in(), observer.in(), period);  
        
    } catch (const CdmwPlatformMngt::PropertyNameInvalid &e)
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);

    } catch (const CdmwPlatformMngt::ElementAlreadyExists & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } 
    
    
    TEST_INFO(" Ok. Registered with success"<<std::endl);

    //Creating the Timer Observer   
    std::auto_ptr <TestTimerObserver> 
    pTimerObserver(new TestTimerObserver); 

    timer_observer = pTimerObserver.release();

    //Creating the Timer    
    std::auto_ptr <SingleShotTimer> 
    pTimer (new SingleShotTimer(timer_observer, period_wait)); 

    timer = pTimer.release();

    //Starting the Timer
    timer->start();
    timer->start_timer();

    //wait to ricieve number of message equal to num_mess.
    TEST_INFO(" Waiting "<<period<<" periods to receive at least "<<num_mess<<" notifications ");
    while (!timer_observer->is_timer_expired())
    {
        TEST_INFO("     - Received "<<obs_impl.get_count()<<" notification"<<std::endl);
        sleep(period/1000);
    }

    //check if number of recieved message is equal to num_mess
    if (obs_impl.get_count()<num_mess)
    {
        TEST_INFO(" Expected to recieve "<<num_mess<<" instead of "<<obs_impl.get_count()<<" notification");
        CPPUNIT_ASSERT (false);
    }else 
    {
        TEST_INFO(" Ok. Received "<<obs_impl.get_count()<<" notification");
        CPPUNIT_ASSERT (true);
    }

    timer_observer->reset_timer();
    obs_impl.reset_count();
    

    /********************************************************
    * Check if ElementAlreadyExists exception is raised
    *******************************************************/
    TEST_INFO("==========================================================="<<std::endl);
    TEST_INFO(" Registering periodic observer with the same name"<<std::endl);

    try
    {
        //register periodic observer 
        notifier->register_observer(obName.c_str(), property_names.in(), observer.in(), period);     
        
        TEST_INFO(" Registered with observer name already present");
        CPPUNIT_ASSERT (false);
        
    } catch (const CdmwPlatformMngt::ElementAlreadyExists & e) 
    {
        TEST_INFO(" Ok. "<<e<<" Exception is raised");
        CPPUNIT_ASSERT (true);
    } catch (const CdmwPlatformMngt::PropertyNameInvalid &e)
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false); 
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    }
    
    /********************************************************
    * Unregistering periodic observer previuosly registered
    *******************************************************/
    TEST_INFO("==========================================================="<<std::endl);
    TEST_INFO(" Unregistering periodic observer previuosly registered..."<<std::endl);

    try
    {
        //register periodic observer 
        observer = notifier->unregister_observer(obName.c_str());     
        
        TEST_INFO(" Ok. Unregistered with success");
        CPPUNIT_ASSERT (true);
        
    }catch (const CdmwPlatformMngt::ObserverNotFound & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    }
    
    /********************************************************
    * Registering periodic observer with valid property name
    *******************************************************/
    TEST_INFO("==========================================================="<<std::endl);
    TEST_INFO(" Trying to re-register the same periodic observer");
    
    try
    {
        //register periodic observer 
        notifier->register_observer(obName.c_str(), property_names.in(), observer.in(), period);     

    } catch (const CdmwPlatformMngt::PropertyNameInvalid &e)
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false); 

    } catch (const CdmwPlatformMngt::ElementAlreadyExists & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } 
    
    TEST_INFO(" Registered with success"<<std::endl);

    //Starting the Timer
    timer->start_timer();

    //wait to ricieve number of message equal to num_mess.
    TEST_INFO(" Waiting "<<period_wait<<" milliseconds to receive at least "<<num_mess<<" notifications ");
    while (!timer_observer->is_timer_expired())
    {
        TEST_INFO("     - Received "<<obs_impl.get_count()<<" notification");
        sleep(period/1000);
    }

    //check if number of recieved message is equal to num_mess
    if (obs_impl.get_count()<num_mess)
    {
        TEST_INFO(" Expected to recieve "<<num_mess<<" instead of "<<obs_impl.get_count()<<" notification"<<std::endl);
        CPPUNIT_ASSERT (false);
    }else 
    {
        TEST_INFO(" Ok. Recieved "<<obs_impl.get_count()<<" notification");
        CPPUNIT_ASSERT (true);
    }  
    
    timer_observer->reset_timer();
    obs_impl.reset_count();
    
    /********************************************************
    * Unregistering periodic observer previuosly registered
    *******************************************************/
    TEST_INFO("==========================================================="<<std::endl);
    TEST_INFO(" Unregistering periodic observer previuosly registered..."<<std::endl);

    try
    {
        //register periodic observer 
        observer = notifier->unregister_observer(obName.c_str());     
        
        TEST_INFO(" Ok. Unregistered with success");
        
        CPPUNIT_ASSERT (true);
        
    }catch (const CdmwPlatformMngt::ObserverNotFound & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    }
}


//reminder
// change condition wait on
void TestPeriodicNotification::test_group_periodic_observer()
{
    TimeBase::TimeT period_wait;
    num_mess = 5;
    hostname = OsSupport::OS::get_hostname();
    period = 5000;    
    obName = "GroupPeriodicObserver";
    miop_group = "corbaloc:miop:1.0@1.0-cdmwftdomain-10/225.1.1.8:5555";
    std::string tmp_string;
    
    
    //control if period is too small (100ms)
    if(period<=100)
    {
        period_wait = num_mess * period * 2;
    }
    else
    {
        period_wait = num_mess*(period+(period/10))+period;
    }

    //activate periodic observer group
    // create servant
    Cdmw::PlatformMngt::Periodic::GroupObserver_impl * group_obs1 = new Cdmw::PlatformMngt::Periodic::GroupObserver_impl();
    Cdmw::PlatformMngt::Periodic::GroupObserver_impl * group_obs2 = new Cdmw::PlatformMngt::Periodic::GroupObserver_impl();
            
    // activate Miop Object
    PortableServer::ObjectId_var obj_id1  = rootGOA->activate_object(group_obs1);
    PortableServer::ObjectId_var obj_id2  = rootGOA->activate_object(group_obs2);
    
    CORBA::Object_var miop_ref = orb->string_to_object(miop_group.c_str());
        
    std::cout<<"\n->Associate MIOP reference " << miop_group.c_str() << " to My Miop Objects"<<std::endl;
    rootGOA->associate_reference_with_id (miop_ref.in(), obj_id1);
    rootGOA->associate_reference_with_id (miop_ref.in(), obj_id2);

    
    /***************************************************
    * Check if PropertyNameInvalid exception is raised
    **************************************************/
    TEST_INFO(" 2) TEST PERIODIC OBSERVER GROUP INTERFACE"<<std::endl);
    TEST_INFO("============================================================"<<std::endl);
    TEST_INFO(" Registering periodic observer group with invalid property name..."<<std::endl);
    
    //postfix .attribute instead of .attr 
    tmp_string = "S1.sys/APPL1.appl/" + hostname + ".host/P12.proc/Cdmw.RequestedRestartWarmth.attribute";

    //initialise the property to be registered to
    property_names = new CosPropertyService::PropertyNames();
    
    //initialise the property to be registered to
    len = 1;
    property_names->length(len);
    //attribute have to be specified with postfix ".attr"
    property_names[len-1]= CORBA::string_dup(tmp_string.c_str());

    try
    {
        TEST_INFO("Get periodic notifier reference from system");
        
        //get a periodic notifier reference from system   
        notifier=system->periodic_notifier();

        TEST_INFO("Register periodic observer group with property name invalid");
        //register periodic observer 
        notifier->register_observer_group(obName.c_str(), property_names.in(), miop_group.c_str(), period);
        
        TEST_INFO(" Registered with property_names invalid");
        CPPUNIT_ASSERT (false);
        
    } catch (const CdmwPlatformMngt::PropertyNameInvalid &e)
    {
        TEST_INFO(" Ok. "<<e<<" Exception is raised");
        CPPUNIT_ASSERT (true);  
        
    } catch (const CdmwPlatformMngt::ElementAlreadyExists & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    }

    
    /********************************************************
    * Registering periodic observer with valid property name
    *******************************************************/
    TEST_INFO("==========================================================="<<std::endl);
    TEST_INFO(" Registering periodic observer with valid property name...");
    
    //initialise the property to be registered to
    len = 2;
    property_names->length(len);
    
    tmp_string = "S1.sys/APPL1.appl/" + hostname + ".host/P11.proc/Cdmw.RequestedRestartWarmth.attr";
    property_names[len-2] = CORBA::string_dup( tmp_string.c_str() );
    TEST_INFO("property is "<<tmp_string.c_str()<<std::endl);
    
    tmp_string = "S1.sys/APPL1.appl/" + hostname + ".host/P12.proc/Cdmw.RequestedRestartWarmth.attr";
    property_names[len-1] = CORBA::string_dup( tmp_string.c_str() );
    TEST_INFO("property is "<<tmp_string.c_str()<<std::endl);
    
    try
    {
        //register periodic observer 
        notifier->register_observer_group(obName.c_str(), property_names.in(), miop_group.c_str(), period);     
    
    } catch (const CdmwPlatformMngt::PropertyNameInvalid &e)
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false); 

    } catch (const CdmwPlatformMngt::ElementAlreadyExists & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } 

    TEST_INFO(" Ok. Registered with success"<<std::endl);
    
    //Creating the Timer Observer   
    std::auto_ptr <TestTimerObserver> 
    pTimerObserver(new TestTimerObserver); 

    timer_observer = pTimerObserver.release();

    //Creating the Timer    
    std::auto_ptr <SingleShotTimer> 
    pTimer (new SingleShotTimer(timer_observer, period_wait)); 

    timer = pTimer.release();

    //Starting the Timer
    timer->start();
    timer->start_timer();

    //wait to ricieve number of message equal to num_mess.
    TEST_INFO(" Waiting "<<period_wait <<" milliseconds to receive at least 1 notifications ");
    while (!timer_observer->is_timer_expired())
    {
        TEST_INFO("     - Received "<<group_obs1->get_count()<<" notification from group periodic observer 1"<<std::endl);
        TEST_INFO("     - Received "<<group_obs2->get_count()<<" notification from group periodic observer 2"<<std::endl);
        sleep(period/1000);
    }

    //check if number of recieved message is at least one 
    if ((group_obs1->get_count() < 1) || (group_obs2->get_count() < 1))
    {
        TEST_INFO(" Expected to recieve at least 1 notification ");
        CPPUNIT_ASSERT (false);
    }else 
    {
        TEST_INFO(" Ok. Received "<<obs_impl.get_count()<<" notification");
        CPPUNIT_ASSERT (true);
    }   

    timer_observer->reset_timer();
    group_obs1->reset_count();
    group_obs2->reset_count();
    
    /********************************************************
    * Check if ElementAlreadyExists exception is raised
    *******************************************************/
    TEST_INFO("==========================================================="<<std::endl);
    TEST_INFO(" Registering periodic observer with the same name"<<std::endl);

    try
    {
        //register periodic observer 
        notifier->register_observer_group(obName.c_str(), property_names.in(), miop_group.c_str(), period);     
        
        TEST_INFO(" Registered with observer name already present");
        CPPUNIT_ASSERT (false);
        
    } catch (const CdmwPlatformMngt::ElementAlreadyExists & e) 
    {
        TEST_INFO(" Ok. "<<e<<" Exception is raised");
        CPPUNIT_ASSERT (true);
    } catch (const CdmwPlatformMngt::PropertyNameInvalid &e)
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false); 
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    }
    
    
    /********************************************************
    * Unregistering periodic observer previuosly registered
    *******************************************************/
    TEST_INFO("==========================================================="<<std::endl);
    TEST_INFO(" Unregistering periodic observer previuosly registered..."<<std::endl);

    try
    {
        //register periodic observer 
        miop_group = notifier->unregister_observer_group(obName.c_str());     
        
        
        TEST_INFO(" Ok. Unregistered with success");
        CPPUNIT_ASSERT (true);
        
    }catch (const CdmwPlatformMngt::ObserverNotFound & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    }
    
    /********************************************************
    * Registering periodic observer with valid property name
    *******************************************************/
    TEST_INFO("==========================================================="<<std::endl);
    TEST_INFO(" Trying to re-register the same periodic observer");
    
    try
    {
        //register periodic observer 
        notifier->register_observer_group(obName.c_str(), property_names.in(), miop_group.c_str(), period);     
    
    } catch (const CdmwPlatformMngt::PropertyNameInvalid &e)
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false); 

    } catch (const CdmwPlatformMngt::ElementAlreadyExists & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } 

    TEST_INFO(" Ok. Registered with success"<<std::endl);
    
    //Starting the Timer
    timer->start_timer();

    //wait to ricieve number of message equal to num_mess.
    TEST_INFO(" Waiting "<<period<<" periods to receive at least 1 notifications ");
    while (!timer_observer->is_timer_expired())
    {
        TEST_INFO("     - Received "<<group_obs1->get_count()<<" notification from group periodic observer 1"<<std::endl);
        TEST_INFO("     - Received "<<group_obs2->get_count()<<" notification from group periodic observer 2"<<std::endl);       
        sleep(period/1000);
    }

    //check if number of recieved message is equal to num_mess
    if ((group_obs1->get_count() < 1) || (group_obs2->get_count() < 1))
    {
        TEST_INFO(" Expected to recieve at least 1 notification");
        CPPUNIT_ASSERT (false);
    }else 
    {
        TEST_INFO(" Ok. Received "<<group_obs1->get_count()<<" notification");
        CPPUNIT_ASSERT (true);
    }   

    timer_observer->reset_timer();
    obs_impl.reset_count();
    
    
    /********************************************************
    * Unregistering periodic observer previuosly registered
    *******************************************************/
    TEST_INFO("==========================================================="<<std::endl);
    TEST_INFO(" Unregistering periodic observer previuosly registered..."<<std::endl);

    try
    {
        //register periodic observer 
        miop_group = notifier->unregister_observer_group(obName.c_str());     
        
        
        TEST_INFO(" Ok. Unregistered with success");
        CPPUNIT_ASSERT (true);
        
    }catch (const CdmwPlatformMngt::ObserverNotFound & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    }
}

}/* End namespace PlatformMngt */
}/* End namespace Cdmw */
#endif/* INCL_PLATFORMMNGT_TEST_PERIODIC_NOTIFICATION_HPP */






