/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_TESTOBJ_HPP
#define INCL_PLATFORMMNGT_TESTOBJ_HPP

#include <map> 
#include <vector>

#include <Foundation/orbsupport/OrbSupport.hpp>
#include <tools/TimeoutObserver.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtApplication.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtProcess.stub.hpp>
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtPeriodicObserver.skel.hpp"
#include <SystemMngt/tools/ManagedElementKey.hpp>

namespace Cdmw
{        // Begin namespace Cdmw

namespace PlatformMngt
{// Begin namespace PlatformMngt

namespace Periodic
{ // Begin namespace Periodic



class TestObj
{
    
public:

    //costructor
    TestObj(CdmwPlatformMngt::System_ptr sys, 
            char* appl_name,
            char* proc_name,
            CORBA::ULong num_userdef);
    
    //destructor
    virtual ~TestObj();
    
    /*Application reference
     * 
     */
    CdmwPlatformMngt::System_var system;
    
    /*Application reference
     * 
     */
    CdmwPlatformMngt::Application_var application;

    /*Process reference
     * 
     */
    CdmwPlatformMngt::Process_var process;
    
    /*Host name
     * 
     */
    std::string host_name;
    
    /*Application and process name
     * 
     */
    std::string appl_name,proc_name;
    
    /*Predefined properties map
     * 
     * key map :    property_name 
     * value map :   property_value
     */ 
    typedef std::multimap < std::string, CORBA::Any > PredefMap ;

    /*User-defined properties info
     * 
     */
    struct UserdefInfo
    {
        std::string attribute_name;
        CORBA::Any attribute_value;
    };
    
    /* User-defined properties vector
     * 
     */
    typedef std::vector < UserdefInfo > UserdefVector;
    
    /*Property info
     * 
     */ 
    CdmwPlatformMngt::Owner flag_to_set, flag_from_get;
    
    typedef enum {ATTRIBUTE_SETTING, ATTRIBUTE_DELETING} attribute_status;
  
    //to recall from observers.
    void execute( const CosPropertyService::Properties & properties) throw (CORBA::SystemException);
    
    //return true if assert is false
    bool is_assert_false();
    
    
private:

    bool m_stop_condition_flag;
    
    bool m_assert_false;
    
    attribute_status m_attr_status;

    CORBA::ULong m_count;
    
    CORBA::ULong m_userdef_num, m_predef_num;
    
    CORBA::ULong m_userdef_count, m_predef_count;
    
    PredefMap m_predef_prop_map;
    
    UserdefInfo m_userdef_info;
    
    UserdefVector m_userdef_prop_vect;

};

}
}
}
#endif /*INCL_PLATFORMMNGT_TESTOBJ_HPP*/

