/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORPERIODIC_TESTTPERIODICNOTIFICATION_HPP
#define INCL_PLATFORPERIODIC_TESTTPERIODICNOTIFICATION_HPP

#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/common/System.hpp"
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/Exception.hpp>

#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"
#include "idllib/CdmwPlatformMngtPeriodicNotifier.stub.hpp"
#include "idllib/CdmwPlatformMngtPeriodicObserver.stub.hpp"

#include <tools/SingleShotTimer.hpp>

#include "test_periodic_notification_reception/TestObserver_impl.hpp"
#include "test_periodic_notification_reception/TestGroupObserver_impl.hpp"
#include "test_periodic_notification_reception/TestTimerObserver.hpp"



namespace Cdmw
{
namespace PlatformMngt
{
//namespace Periodic
//{

class TestPeriodicNotification : public CppUnit::TestFixture
{

public:

    TestPeriodicNotification();

    void setUp ();
    
#ifdef CDMW_USE_TEST_IIOP
    virtual void test_single_periodic_observer ();
#endif
  
#ifdef CDMW_USE_TEST_MIOP    
    virtual void test_group_periodic_observer ();
    std::string miop_group;
#endif

private:

    //Host name
    std::string host_name;
    
    //Application name
    char* appl_name;
    
    //Process name
    char* proc_name;

    //Number of user-defined property
    CORBA::ULong num_userdef;
    
    //Wating time to recieve message
    bool wait;

    //System reference
    CdmwPlatformMngt::System_var system;

    //Periodic Notifier reference
    CdmwPlatformMngt::Periodic::Notifier_var notifier;

    Cdmw::PlatformMngt::Periodic::TestObserver_impl * obs_impl;
    //Periodic Observer reference
    CdmwPlatformMngt::Periodic::Observer_var observer;
    std::string obName, obGroupName ;

    //Property name sequence
    CosPropertyService::PropertyNames_var property_names;
    CORBA::ULong len ;

    // The timer interval.
    TimeBase::TimeT period, period_wait;
    
    // The timer
    SingleShotTimer* timer;
    TestTimerObserver* timer_observer;

    // The timer
    SingleShotTimer* timer_sup;
    TestTimerObserver* timer_observer_sup;
    bool is_sup_kill;
    
    CdmwPlatformMngtBase::StartupKind_var startupKind;
    CdmwPlatformMngtBase::StateRetrieval state_retrieval;

    CORBA::String_var statusInfo;
    CORBA::ORB_var orb;
    PortableGroup::GOA_var rootGOA ;

    //Startupkind parameter
    int val;
    unsigned long startup_mode;
    std::string specified_state_id;

private:

    /**
    * Assignment operator
    */
    TestPeriodicNotification & operator= (const TestPeriodicNotification & rhs) throw ();

    /**
    * Copy constructor
    */
    TestPeriodicNotification (const TestPeriodicNotification & rhs) throw ();
    
    /**
    * Inizialise the property sequence to be registered to
    */
    void inizialise_property();

    /**
    * Defining a Test Suite
    */       
    CPPUNIT_TEST_SUITE (TestPeriodicNotification);

#ifdef CDMW_USE_TEST_IIOP
    //These are the test cases
    CPPUNIT_TEST (test_single_periodic_observer); 
#endif  
 
#ifdef CDMW_USE_TEST_MIOP    
    CPPUNIT_TEST (test_group_periodic_observer); 
#endif 

    CPPUNIT_TEST_SUITE_END ();
};

//}
}/* End namespace PlatformMngt */
}/* End namespace Cdmw */
#endif/* #ifndef INCL_PLATFORPERIODIC_TESTTPERIODICNOTIFICATION_HPP */
