/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <string>
#include <iostream>
#include <string.h>

#include "test_periodic_notification_reception/TestObj.hpp"
#include <tao/AnyTypeCode/TypeCode.h>
#include <Foundation/orbsupport/CosPropertyService.stub.hpp>
#include <idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp>
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"

using namespace std;

namespace Cdmw
{// Begin namespace Cdmw
    
namespace PlatformMngt
{// Begin namespace PlatformMngt
    
namespace Periodic
{ // Begin namespace Periodic 


TestObj::TestObj(CdmwPlatformMngt::System_ptr sys,
                 char* app,
                 char* proc,
                 CORBA::ULong num_userdef)
{
    system = CdmwPlatformMngt::System::_duplicate(sys);
    host_name = OsSupport::OS::get_hostname();
    
    //application and process name to test
    appl_name = app;
    proc_name = proc; 
    
    //get application and process to test
    application = system->get_application(appl_name.c_str());
    process = application->get_process(proc_name.c_str(),host_name.c_str());
    
    //inizialize the  notification counters
    m_userdef_count = 0;
    m_predef_count = 0;
    
    //Define Predefined attribute map
    m_predef_num = 2;
    std::string attribute_name = "RequestedRestartWarmth";
    CdmwPlatformMngt::RestartWarmth attr_type1 = CdmwPlatformMngt::HOT_RESTART;
    CORBA::Any attr_value;
    attr_value <<= attr_type1;      
    m_predef_prop_map.insert (PredefMap::value_type (attribute_name, attr_value));
    
    attribute_name = "ForceManualStart";
    CORBA::Boolean attr_type2 = false;
    attr_value <<= CORBA::Any::from_boolean(attr_type2);
    m_predef_prop_map.insert (PredefMap::value_type (attribute_name, attr_value));

    
    //Define User-defined attribute map
    m_userdef_num = num_userdef;
    for (CORBA::ULong i=0; i<m_userdef_num; ++i)
    {
        //insert each property into vector  
        std::stringstream for_convert;
        for_convert << i+1;
        for_convert >> attribute_name;
        m_userdef_info.attribute_name = "Attribute_" + attribute_name;       
        CORBA::Short attr_type = i+1;
        attr_value <<= attr_type; 
        m_userdef_info.attribute_value = attr_value;
        
        m_userdef_prop_vect.push_back(m_userdef_info);
    }
    
    flag_to_set = CdmwPlatformMngt::USER_DEFINED;    
    m_attr_status = ATTRIBUTE_SETTING;
    m_stop_condition_flag = false; 
    m_count = 0;
    m_assert_false = false;
}

TestObj::~TestObj()
{
    
}

void
TestObj::execute( const CosPropertyService::Properties & properties )
throw( CORBA::SystemException )
{
    CORBA::ULong len = properties.length ();
    PredefMap::iterator it_predef;
    UserdefVector::iterator it_userdef;
    std::string attribute_name;
    std::string prefix (CdmwPlatformMngt::CDMW_PREFIX);
    std::string dummy_string = "";
    CORBA::ULong checked_predef = 0;
    CORBA::ULong checked_userdef = 0;

    std::cerr<< "Recieved "<<len <<" properties" <<std::endl;
    if ( len == (m_predef_num + m_userdef_count))
    {
        map<std::string,std::string> MyMap;
        std::string attr_to_count;

        for (CORBA::ULong i=0; i<len; ++i)
        {  
            attr_to_count = properties[i].property_name;
            
            if( MyMap.count(attr_to_count) > 0)
            {
                std::cerr << "Property name ["<<i<<"] = <"<<attr_to_count<<"> : more istances in the notified sequence"<<std::endl;
                m_assert_false = true;
                CPPUNIT_ASSERT(false);              
            }
            else
            {
                MyMap[ attr_to_count ] = attr_to_count;
            }
        }
    }
    else 
    {
        std::cerr << "The lenght of properties sequence recieved is " << len<<"instead of "<<m_predef_num + m_userdef_count<<std::endl;
        m_assert_false = true;
        CPPUNIT_ASSERT(false);
    }

    //check each predefined property value
    for (CORBA::ULong i=0; i<len; ++i)
    {

        std::cerr << "property name ["<<i<<"] = "<< properties[i].property_name<< std::endl;
        std::string property_name = CORBA::string_dup(properties[i].property_name);

        //extract attribute name
        std::string::size_type pos = (property_name).find(prefix);  
        std::string::size_type size = prefix.size();   

        Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(property_name,
                                                                 dummy_string,
                                                                 appl_name, 
                                                                 host_name,
                                                                 proc_name, 
                                                                 dummy_string,
                                                                 dummy_string,
                                                                 attribute_name,
                                                                 dummy_string,
                                                                 dummy_string);

        // if attributes are predefined or user-defined
        if (pos != std::string::npos)
        {      
            //the attribute is PRE_DEFINED;
            attribute_name = (attribute_name).substr(size);

            //map operation
            it_predef = m_predef_prop_map.find(attribute_name);

            if (it_predef != m_predef_prop_map.end())
            {
                CORBA::Any predef_prop = it_predef->second;

                if (properties [i].property_value.type()->equal(predef_prop.type()))
                {
                    if (properties [i].property_value.type()->equal(CdmwPlatformMngt::_tc_RestartWarmth))
                    {
                        //extract property value from property_name
                        std::cout << "|  type  :  RestartWarmth" << std::endl;
                        CdmwPlatformMngt::RestartWarmth value;
                        properties [i].property_value >>= value;
                        std::cout << "|  value_to_check:  "<< ( (CdmwPlatformMngt::HOT_RESTART == value)?"Hot":"Cold" ) <<std::endl;

                        CdmwPlatformMngt::RestartWarmth value_check;
                        predef_prop >>= value_check;
                        std::cout << "|  value_to_be_equal :  "<< ( (CdmwPlatformMngt::HOT_RESTART == value_check)?"Hot":"Cold" ) <<std::endl;  
                        if (value_check == value)
                        {  
                            std::cerr << "Property Predefined 1 is OK!" <<std::endl;
                            checked_predef++;
                        } 
                        else 
                        {
                            std::cerr << "Property Predefined 1 is NOT OK!" <<std::endl;
                            m_assert_false = true;
                            CPPUNIT_ASSERT(false);
                        }
                    }
                    else if (CORBA::tk_boolean == properties[i].property_value.type()->kind())
                    {
                        std::cerr << "|  type  :  Boolean" << endl;
                        CORBA::Boolean value;
                        properties[i].property_value >>= CORBA::Any::to_boolean(value);
                        std::cerr << "|  value_to_check :  " << value << endl;

                        CORBA::Boolean value_check;
                        predef_prop >>= CORBA::Any::to_boolean(value_check);
                        std::cerr << "|  value_to_be_equal :  " << value_check << endl;    

                        if (value_check == value)
                        {
                            std::cerr << " Property Predefined Value"<< i <<" is OK!"<<std::endl;
                            checked_predef++;
                        }  
                        else 
                        {
                            std::cerr << " Property Predefined Value"<< i <<" is NOT OK!" <<std::endl;
                            m_assert_false = true;
                            CPPUNIT_ASSERT(false);
                        }
                    }
                    else
                    {
                        std::cerr << " Property Predefined Value"<< i <<" is unknown" <<std::endl;
                        m_assert_false = true;
                        CPPUNIT_ASSERT(false);
                    }
                }else
                {
                    std::cerr << " Predefined Property Type"<< i <<" is not correct" <<std::endl;
                    m_assert_false = true;
                    CPPUNIT_ASSERT(false);
                }
            }else 
            {
                std::cerr << "Predefined Property"<< i <<" is not found in map" <<std::endl;
                m_assert_false = true;
                CPPUNIT_ASSERT(false);
            }
        }
        else
        { 
            //the attribute is USER_DEFINED;
            for (CORBA::ULong j=0; j<m_userdef_count; ++j)
            {                
                std::string userdef_prop = m_userdef_prop_vect[j].attribute_name;  

                if (!strcmp(userdef_prop.c_str(), attribute_name.c_str()))
                {
                    CORBA::Short tmp1,tmp2 ;
                    properties [i].property_value >>= tmp1;
                    m_userdef_prop_vect[j].attribute_value >>= tmp2;

                    if (tmp2 == tmp1 )
                    {
                        checked_userdef++; 
                    }
                    else 
                    {
                        m_assert_false = true;
                        CPPUNIT_ASSERT(false);
                    }
                }
            }
        }
    }

    if (m_predef_num == checked_predef)
    {
        std::cerr << "All Predefined Properties are checked" <<std::endl;
        CPPUNIT_ASSERT(true);
    }
    else
    {
        std::cerr << " Predefined Properties check error" <<std::endl;
        m_assert_false = true;
        CPPUNIT_ASSERT(false);
    }

    if (m_userdef_count == checked_userdef)
    {
        std::cerr << "All User-defined Properties are checked" <<std::endl;
        CPPUNIT_ASSERT(true);
    }
    else
    {
        std::cerr << " User-defined Properties check error" <<std::endl;
        m_assert_false = true;
        CPPUNIT_ASSERT(false);
    }

    if ( (ATTRIBUTE_SETTING == m_attr_status) && (m_userdef_num > 0) )
    {            
        //set a user-defined_property
        std::string attr_to_define = m_userdef_prop_vect[m_userdef_count].attribute_name;
        CORBA::Any attr_value_to_define = m_userdef_prop_vect[m_userdef_count].attribute_value;
        try
        {
            process->set_attribute(attr_to_define.c_str(), attr_value_to_define , flag_to_set);
        }
        catch (const CdmwPlatformMngt::AttributeNotFound& anf )
        {
            process->define_property(attr_to_define.c_str(), attr_value_to_define); 
        }
        m_userdef_count++;
        std::cerr << "Defined user-defined property "<<m_userdef_count<<" with name "<<attr_to_define<<std::endl;

        if (m_userdef_count == m_userdef_num)
        {
            m_attr_status = ATTRIBUTE_DELETING;    
        }   
    }
    else if ( ATTRIBUTE_DELETING == m_attr_status )
    {
        if (!m_stop_condition_flag)
        {
            //delete one user-defined_property
            std::cerr << "Delete user-defined property "<<m_userdef_count<<std::endl;
            m_userdef_count--;
            std::string attr_to_delete = m_userdef_prop_vect[m_userdef_count].attribute_name;
            
            try
            {
                //delete user-defined property
                process->delete_property(attr_to_delete.c_str());
            }
            catch (const CosPropertyService::PropertyNotFound&)
            {
                std::cerr << "user-defined property is already deleted"<<std::endl;
            }

            //check if test is finished
            if (0 == m_userdef_count)
            {
                m_stop_condition_flag = true;
            }
        }
    }
    //execute call's count 
    m_count++;
}

bool TestObj::is_assert_false()
{
    return m_assert_false;
}

}//End namespace PlatformMngt
} // End namespace PlatformMngt
} // End namespace Cdmw
