/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @author Anna Domnina <gdomnina@sesm.it>
 */

#ifndef INCL_PLATFORMMNGT_TEST_PERIODIC_NOTIFICATION_HPP
#define INCL_PLATFORMMNGT_TEST_PERIODIC_NOTIFICATION_HPP
#include <iostream>
#include <fstream>
#include <limits>
#include <string>
#include <sstream>


#include "test_periodic_notification_reception/TestPeriodicNotification.hpp"
#include "test_periodic_notification_reception/TestObserver_impl.hpp"


#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "SystemMngt/platformlibrary/Iterator.hpp"

#include "idllib/CdmwPlatformMngtSystem.stub.hpp"

#include <SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp>
#include "SystemMngt/idllib/CdmwPlatformMngtPeriodicNotifier.stub.hpp"

#include "orbsvcs/orbsvcs/PortableGroupC.h"

using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestPeriodicNotification);


namespace Cdmw
{
namespace PlatformMngt
{


TestPeriodicNotification::TestPeriodicNotification()
{
}

void TestPeriodicNotification::setUp ()
{
    try
    {
        //Getting system var
        std::string system_corbaloc = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";
        orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::instance ()->get_ORB ());
        CORBA::Object_var system_obj = orb->string_to_object (system_corbaloc.c_str());
        system = CdmwPlatformMngt::System::_narrow (system_obj.in ());

#ifdef CDMW_USE_TEST_MIOP        
        //Get the root GOA
        CORBA::Object_var obj = orb->resolve_initial_references ("RootPOA");
        rootGOA = PortableGroup::GOA::_narrow(obj.in());
#endif        

        //Setting up the startupKind parameters
        state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
        startup_mode = 0;
        specified_state_id = "";
        startupKind = new CdmwPlatformMngtBase::StartupKind();
        startupKind->state_retrieval = state_retrieval;
        startupKind->startup_mode = startup_mode;
        startupKind->specified_state_id  = CORBA::string_dup( specified_state_id.c_str() );

        /***************************
         * Parsing input parameters
         * 
         ***************************/
        num_userdef = atol((Cdmw::TestUtils::CORBATestManager::instance()->get_property("--userdef_num")).c_str());
        period = atol((Cdmw::TestUtils::CORBATestManager::instance()->get_property("--period")).c_str()); 

        //set property 
        inizialise_property();

        // The timer to kill supervision
        TimeBase::TimeT kill_sup_period =  
            atol((Cdmw::TestUtils::CORBATestManager::instance()->get_property("--kill_sup")).c_str());

        if (0 != kill_sup_period)
        {      
            is_sup_kill = true;

            //Creating the Timer Observer   
            std::auto_ptr <TestTimerObserver> 
            pTimerObserver(new TestTimerObserver (is_sup_kill)); 

            timer_observer_sup = pTimerObserver.release();

            //Creating the Timer    
            std::auto_ptr <SingleShotTimer> 
            pTimer (new SingleShotTimer(timer_observer_sup, kill_sup_period)); 

            timer_sup = pTimer.release();

            //Starting the Timer
            timer_sup->start();
            timer_sup->start_timer();

        }
        else 
        {
            is_sup_kill = false;
        }

    }
    catch (const CORBA::SystemException& ex) 
    {
        std::cerr<< ex._name()<<std::endl;
        throw;
    }
    catch(...)
    {
    }
}
#ifdef CDMW_USE_TEST_IIOP
void TestPeriodicNotification::test_single_periodic_observer()
{
    //activate periodic observer
    obs_impl = new Cdmw::PlatformMngt::Periodic::TestObserver_impl(system,appl_name,proc_name,num_userdef);

    observer = obs_impl->_this(); 

    /********************************************************
     * Registering periodic observer with valid property name
     *******************************************************/
    TEST_INFO(" START SINGLE PERIODIC OBSERVER"<<std::endl);
    TEST_INFO("==========================================================="<<std::endl);
    TEST_INFO(" Registering periodic observer with valid property name...");

    try
    {
        //get a periodic notifier reference from system   
        notifier=system->periodic_notifier();

        //register periodic observer 
        notifier->register_observer(obName.c_str(), property_names.in(), observer.in(), period);

        CPPUNIT_ASSERT (true);

    } catch (const CdmwPlatformMngt::PropertyNameInvalid &e)
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);

    } catch (const CdmwPlatformMngt::ElementAlreadyExists & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } 

    TEST_INFO(" Ok. Registered with success"<<std::endl);
    //Creating the Timer Observer   
    std::auto_ptr <TestTimerObserver> 
    pTimerObserver(new TestTimerObserver(false)); 

    timer_observer = pTimerObserver.release();

    //Creating the Timer    
    std::auto_ptr <SingleShotTimer> 
    pTimer (new SingleShotTimer(timer_observer, period_wait)); 

    timer = pTimer.release();

    //Starting the Timer
    timer->start();
    timer->start_timer();

    //wait to ricieve number of message equal to num_mess.
    TEST_INFO("################# Waiting "<<period<<" milliseconds to receive at least "<< 1+(2*num_userdef) <<" notifications ");
    while (!timer_observer->is_timer_expired())
    {
        if (obs_impl->is_assert_false())
        {
            TEST_INFO(" Observer_impl :: callback execute ASSERT FALSE ");
            CPPUNIT_ASSERT (false);
        }
        TEST_INFO("################# Received "<<obs_impl->get_count()<<" notification"<<std::endl);
        sleep(period/1000);
    }

    //check if number of recieved message is equal to num_mess
    if (obs_impl->get_count()< (1+(2*num_userdef)))
    {
        TEST_INFO(" Expected to recieve "<<(1+(2*num_userdef))<<" instead of "<<obs_impl->get_count()<<" notification");
        CPPUNIT_ASSERT (false);
    }else 
    {
        TEST_INFO(" Ok. Received "<<obs_impl->get_count()<<" notification");
        CPPUNIT_ASSERT (true);
    }

    timer_observer->reset_timer();
    obs_impl->reset_count();

    /********************************************************
     * Unregistering periodic observer previuosly registered
     *******************************************************/
    TEST_INFO("==========================================================="<<std::endl);
    TEST_INFO(" Unregistering periodic observer previuosly registered..."<<std::endl);
    try
    {
        observer = notifier->unregister_observer(obName.c_str());
        TEST_INFO(" Ok. Unregistered with success");
        CPPUNIT_ASSERT (true);

    }catch (const CdmwPlatformMngt::ObserverNotFound & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    }
}
#endif


#ifdef CDMW_USE_TEST_MIOP
void TestPeriodicNotification::test_group_periodic_observer()
{  
    //activate periodic observer group
    // create servant
    Cdmw::PlatformMngt::Periodic::TestGroupObserver_impl * group_obs1 =
        new Cdmw::PlatformMngt::Periodic::TestGroupObserver_impl(system,appl_name,proc_name,num_userdef);
    Cdmw::PlatformMngt::Periodic::TestGroupObserver_impl * group_obs2 =
        new Cdmw::PlatformMngt::Periodic::TestGroupObserver_impl(system,appl_name,proc_name,num_userdef);

    // activate Miop Object
    PortableServer::ObjectId_var obj_id1  = rootGOA->activate_object(group_obs1);
    PortableServer::ObjectId_var obj_id2  = rootGOA->activate_object(group_obs2);

    CORBA::Object_var miop_ref = orb->string_to_object(miop_group.c_str());

    std::cout<<"\n->Associate MIOP reference " << miop_group.c_str() << " to My Miop Objects"<<std::endl;
    rootGOA->associate_reference_with_id (miop_ref.in(), obj_id1);
    rootGOA->associate_reference_with_id (miop_ref.in(), obj_id2);

    /********************************************************
     * Registering periodic observer with valid property name
     *******************************************************/
    TEST_INFO(" START GROUP PERIODIC OBSERVER"<<std::endl);
    TEST_INFO("==========================================================="<<std::endl);

    try
    {
        //get a periodic notifier reference from system   
        notifier=system->periodic_notifier();

        //register periodic observer 
        notifier->register_observer_group(obGroupName.c_str(), property_names.in(), miop_group.c_str(), period);  

        CPPUNIT_ASSERT (true);

    } catch (const CdmwPlatformMngt::PropertyNameInvalid &e)
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);

    } catch (const CdmwPlatformMngt::ElementAlreadyExists & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } 


    TEST_INFO(" Ok. Registered with success"<<std::endl);
    //Creating the Timer Observer   
    std::auto_ptr <TestTimerObserver> 
    pTimerObserver(new TestTimerObserver(is_sup_kill)); 

    timer_observer = pTimerObserver.release();

    //Creating the Timer    
    std::auto_ptr <SingleShotTimer> 
    pTimer (new SingleShotTimer(timer_observer, period_wait)); 

    timer = pTimer.release();

    //Starting the Timer
    timer->start();
    timer->start_timer();

    //wait to ricieve number of message equal to num_mess.
    TEST_INFO("################# Waiting "<<period<<" milliseconds to receive at least one notifications ");
    while (!timer_observer->is_timer_expired())
    {
        if ((group_obs1->is_assert_false()) || (group_obs2->is_assert_false()))
        {
            TEST_INFO(" GroupObserver_impl :: callback execute ASSERT FALSE ");
            CPPUNIT_ASSERT (false);
        }
        TEST_INFO("     - Received "<<group_obs1->get_count()<<" notification from group periodic observer 1"<<std::endl);
        TEST_INFO("     - Received "<<group_obs2->get_count()<<" notification from group periodic observer 2"<<std::endl);
        sleep(period/1000);
    }

    //check if number of recieved message is equal to num_mess
    if ((group_obs1->get_count() < 1 ) || (group_obs2->get_count()< 1 ))
    {
        TEST_INFO(" Expected to recieve at least 1 notification");
        CPPUNIT_ASSERT (false);
    }else 
    {
        TEST_INFO(" Ok. Received "<<group_obs1->get_count()<<" notification");
        CPPUNIT_ASSERT (true);
    }

    timer_observer->reset_timer();
    group_obs1->reset_count();

    /********************************************************
     * Unregistering periodic observer previuosly registered
     *******************************************************/
    TEST_INFO("==========================================================="<<std::endl);
    TEST_INFO(" Unregistering periodic group observer previuosly registered..."<<std::endl);
    try
    {
        miop_group = notifier->unregister_observer_group(obGroupName.c_str());
        TEST_INFO(" Ok. Unregistered with success");
        CPPUNIT_ASSERT (true);
        
    }catch (const CdmwPlatformMngt::ObserverNotFound & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    } catch (const CORBA::SystemException & e) 
    {
        TEST_INFO(" Exception"<<e);
        CPPUNIT_ASSERT (false);
    }
}
#endif

void  TestPeriodicNotification::inizialise_property()
{
    host_name = OsSupport::OS::get_hostname();
    appl_name = "APPL1";
    proc_name = "P11";
    std::string property_name;
    std::string tmp_string;
    std::string appl_name1 = appl_name;
    std::string proc_name1 = proc_name;

#ifdef CDMW_USE_TEST_MIOP   
    miop_group = "corbaloc:miop:1.0@1.0-cdmwftdomain-10/225.1.1.8:5555";
    obGroupName = "TestPeriodicObserver";
#endif

#ifdef CDMW_USE_TEST_IIOP 
    obName = "TestGroupPeriodicObserver";
#endif    

    period_wait = (period * (1 + (2*num_userdef)) )+ (period/2);

    //initialise the property to be registered to
    property_names = new CosPropertyService::PropertyNames();

    //initialise the property to be registered to
    len = num_userdef + 2;
    property_names->length(len);

    property_name = "S1.sys/"+ appl_name1 +".appl/" + host_name + ".host/"+ proc_name1 +".proc/Cdmw.RequestedRestartWarmth.attr";
    property_names[len-len] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("Pre-defined property name[0] is "<<property_name.c_str()<<std::endl);

    property_name = "S1.sys/"+ appl_name1 +".appl/" + host_name + ".host/"+ proc_name1 +".proc/Cdmw.ForceManualStart.attr";
    property_names[len-len+1] = CORBA::string_dup( property_name.c_str() );
    TEST_INFO("Pre-defined property name[1] is "<<property_name.c_str()<<std::endl);

    for (CORBA::ULong i=2; i<len; i++)
    { 
        property_name = "S1.sys/"+ appl_name1 +".appl/" + host_name + ".host/"+ proc_name1 +".proc/Attribute_" ;
        std::stringstream tmp_stream;
        tmp_stream << i-1;
        std::string tmp_string1 = tmp_stream.str() + ".attr"; 
        std::string tmp_string2 = property_name + tmp_string1; 
        property_names[i] = CORBA::string_dup( tmp_string2.c_str());
        TEST_INFO("User-defined property name["<<i<<"] is "<<tmp_string2.c_str()<<std::endl);

    }
}

}/* End namespace PlatformMngt */
}/* End namespace Cdmw */
#endif/* INCL_PLATFORMMNGT_TEST_PERIODIC_NOTIFICATION_HPP */






