
/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef ___CCM_TEST_PROPERTY_CHANGE_CCM_CLIENT_IMPL_HPP__
#define ___CCM_TEST_PROPERTY_CHANGE_CCM_CLIENT_IMPL_HPP__

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/osthreads/Mutex.hpp>
#include <Foundation/osthreads/Condition.hpp>
#include <queue>

#include <test_property_change/TestPropertyChangeLocal.skel.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtManagedElementContexts.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtComponentContextExt.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtProcess.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtComponent.stub.hpp>

namespace TestPropertyChange
{

class CCM_Client_impl;
    

class test_property_change : public CppUnit::TestFixture
{

public:

    test_property_change();

    //Register the factory
    void register_observer();

    //Create the PropertyInfos
    void create_and_notify_property_change(
            std::string element_path,
            const char* role);

    //
    // Testable operations
    //
    void setUp();
    //static
    void oneShotInit();
    void ChangeProperty();

    // register test suite and test operation
    CPPUNIT_TEST_SUITE( test_property_change );
    CPPUNIT_TEST( ChangeProperty );
    CPPUNIT_TEST_SUITE_END();

private:
    CCM_Client_impl * m_ccm_client;
    ::TestPropertyChange::CCM_Client_Context_var m_session_context;
    TestPropertyChange::Client_var m_component_ref;
    CdmwPlatformMngt::ManagedElementContexts::ComponentContextExt_var component_context_ext;
    CdmwPlatformMngt::Process_var process;
    CdmwPlatformMngt::CcmComponent_var component;
    CORBA::ORB_var orb;
    static bool oneShotInitDone;
    std::string process_element_path , component_element_path;
    std::string host_name;
    //CdmwPlatformMngt::External::System_var system;
    CdmwPlatformMngt::System_var system;

    CdmwPlatformMngtBase::StartupKind_var startupKind;
    CdmwPlatformMngtBase::StateRetrieval state_retrieval;        
    CORBA::String_var statusInfo; 
    unsigned long startup_mode;
    std::string specified_state_id;
    
};
    
    
    
//
// IDL:thalesgroup.com/TestComponentNotification/Session_CCM_Client:1.0
//
class CCM_Client_impl : virtual public ::TestPropertyChange::Session_CCM_Client,
                        virtual public Cdmw::OrbSupport::RefCountLocalObject
{
    CCM_Client_impl(const CCM_Client_impl&);
    void operator=(const CCM_Client_impl&);

    ::TestPropertyChange::CCM_Client_Context_var m_session_context;
    
     
public:

    CCM_Client_impl();
    ~CCM_Client_impl();
     
    //
    // Session component specific operations
    //
    void set_session_context(Components::SessionContext_ptr ctx)
        throw(CORBA::SystemException);
    void ccm_activate()
        throw(CORBA::SystemException);
    void ccm_passivate()
        throw(CORBA::SystemException);
    void ccm_remove()
        throw(CORBA::SystemException);

    void CCM_Client_impl::notify(CdmwPlatformMngt::Event* event)
       throw(CORBA::SystemException);
    
private:
    
    
    ::TestPropertyChange::CCM_Client_Context* get_context();
    Cdmw::TestUtils::ThreadedTestRunner* m_test_thread;
    
    Cdmw::OsSupport::Mutex  *             m_mutex;
    Cdmw::OsSupport::Condition *          m_condition;

    
    friend class test_property_change;
};

}; // End of namespace TestCcmNotification


#endif // ___CCM_TEST_PROPERTY_CHANGE_CCM_CLIENT_IMPL_HPP__


