/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <test_property_change/CCM_Client_impl.hpp>
#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/logging/CdmwLogging.stub.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include "SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/EntityStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/PropertyChange_impl.hpp"
#include <SystemMngt/idllib/CdmwPlatformMngtManagedElementContexts.stub.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtManagedComponentContextFactory.stub.hpp>
#include <SystemMngt/platformlibrary/EventHeaderFactory.hpp>
#include <SystemMngt/platformvaluetypes/PropertyChange_impl.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtManagedElementContexts.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp>
#include "SystemMngt/idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"



namespace TestPropertyChange
{

  bool test_property_change::oneShotInitDone = false;
//
// IDL:thalesgroup.com/CcmFtTest/ClientModule/CCM_Client:1.0
//
CCM_Client_impl::CCM_Client_impl()
    : m_test_thread(0)
{
    //Creating a mutex to sinchronize the test's "kill and restart" procedure. 
    m_mutex=new Cdmw::OsSupport::Mutex();
    m_condition=new Cdmw::OsSupport::Condition(*m_mutex);
}

CCM_Client_impl::~CCM_Client_impl()
{
    std::cerr << "   ... CCM_Client_impl::~CCM_Client_impl called!" << std::endl;
    delete m_mutex;
    delete m_condition;
}


void CCM_Client_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    m_session_context = ::TestPropertyChange::CCM_Client_Context::_narrow(ctx);
}
        
::TestPropertyChange::CCM_Client_Context* CCM_Client_impl::get_context()
{
    return ::TestPropertyChange::CCM_Client_Context::_duplicate(m_session_context.in());
}


//
// ccm_activate function : it actives the component (call by the container)
//
void CCM_Client_impl::ccm_activate()
   throw(CORBA::SystemException)
{
    std::cerr << "   ... CCM_Client_impl::ccm_activate called!" << std::endl;

    // Get this component CORBA object
    // Note: this has to be performed in a CORBA request thread; thus
    //       not in test which is a specific thread.
    TestPropertyChange::Client_var this_component =
        TestPropertyChange::Client::_nil();
    
    try 
    {
        // narrow Context in SessionContext  for get_CCM_objet() access
        Components::SessionContext_var ctx =
            Components::SessionContext::_narrow(m_session_context.in());
        
        // call SessionContext::get_CCM_objet()
        CORBA::Object_var obj =
            ctx->get_CCM_object();
            
        this_component =
            TestPropertyChange::Client::_narrow(obj.in());
            
        CDMW_ASSERT(!CORBA::is_nil(this_component.in()));
            
    } catch (const CORBA::UserException& e) {
        std::cerr << "UserException during get_CCM_Object() :" 
                  << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
                  << std::endl;
        CDMW_ASSERT(false);
    } catch (const CORBA::SystemException& e) {
        std::cerr << "SystemException during get_CCM_Object() :" 
                  << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
                  << std::endl;
        CDMW_ASSERT(false);
    }


    try {   
        // save Context in CORBATestManager for usage in test
        Cdmw::TestUtils::CORBATestManager::instance()->add(
                "client_context", 
                get_context());
        // save this component in CORBATestManager for usage in test
        Cdmw::TestUtils::CORBATestManager::instance()->add(
                "this_component", 
                this_component.in());


        // save this executor in CORBATestManager for usage in test
        Cdmw::TestUtils::CORBATestManager::instance()->add(
                "this_executor", 
                dynamic_cast< Session_CCM_Client_ptr > (this));
        
        // create test runner thread
       m_test_thread = new Cdmw::TestUtils::ThreadedTestRunner(
                Cdmw::TestUtils::CORBATestManager::instance()->get_controller());
        // add test to be run
        m_test_thread->addTest( 
                CppUnit::TestFactoryRegistry::getRegistry("test_property_change").makeTest());
        // start test
       m_test_thread->start();


   } catch (const CORBA::Exception& e) {
        std::cerr << "CORBA Exception exception during test: "
                  << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
                  << std::endl;
    } catch (const std::exception& e) {
        std::cerr << "std::exception exception during test: "
                  << e.what()
                  << std::endl;
    } catch (...) {
        std::cerr << "unknown exception during test !!" << std::endl;
    }
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void CCM_Client_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
    std::cerr << "   ... CCM_Client_impl::ccm_passivate called!" << std::endl;
    
 if (m_test_thread != 0) {
        // wait test to be finished
        m_test_thread->join();
        delete m_test_thread;
        m_test_thread = 0;
        // print test summary
       // Cdmw::TestUtils::CORBATestManager::instance()->summary();
    }

} 


//
// ccm_remove function : it destroys the component (call by the container)
//
void CCM_Client_impl::ccm_remove()
   throw(CORBA::SystemException)
{
    std::cerr << "   ... CCM_Client_impl::ccm_remove called!" << std::endl;
}



void CCM_Client_impl::notify(CdmwPlatformMngt::Event* an_event)
        throw(CORBA::SystemException)
{
    if (an_event->event_kind() == CdmwPlatformMngt::PROPERTY_CHANGE )
    {

        CdmwPlatformMngt::PropertyChange* prp_chg
        = dynamic_cast<CdmwPlatformMngt::PropertyChange*> (an_event);

        if (prp_chg!=0) 
        {               
            try
            {
                //CDMW_MUTEX_GUARD(*m_mutex);
                CORBA::String_var message = an_event->to_string();
                std::cout <<"#####" <<message.in() << std::endl;
                //m_condition->broadcast();
            }
            catch (const std::exception& ex)
            {
                std::cerr << " TestPropertyChange_Client observer exception  " << ex.what();
            }
        }
    }
}

//
// Testable do_tests() operation
//
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( test_property_change, "test_property_change");

test_property_change::test_property_change()
{
    orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::instance ()->get_ORB ());
    
    // retrieve notification object
    CORBA::Object_var obj = 
        Cdmw::TestUtils::CORBATestManager::instance()->get_object("client_context");        
    m_session_context = ::TestPropertyChange::CCM_Client_Context::_narrow(obj.in());
    
    obj = Cdmw::TestUtils::CORBATestManager::instance()->get_object("this_executor");       
    m_ccm_client = dynamic_cast< CCM_Client_impl* > (obj.in());
    
    obj = Cdmw::TestUtils::CORBATestManager::instance()->get_object("this_component");
    m_component_ref = TestPropertyChange::Client::_narrow(obj.in());

}

void test_property_change::register_observer()
{
    // Register event factories
    CORBA::ValueFactoryBase_var factory, old_factory;

    factory = new Cdmw::PlatformMngt::PropertyChangeFactory;
    old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/PropertyChange:1.0",
            factory.in() );
    
    CdmwPlatformMngt::RegistrationDepth depth=CdmwPlatformMngt::FULL_HIERARCHY;

    // register this component as System observer
    system->register_observer_ext("TestPropertyChangeClient_Observer", 
                depth,
                m_component_ref.in() );
}

void test_property_change::create_and_notify_property_change(
        std::string element_path,
        const char* role)
{
    CdmwPlatformMngt::PropertyChange_var property_change;
    
    //prepare PropertyChange data to notify
    CdmwPlatformMngtBase::EventHeader header = 
        Cdmw::PlatformMngt::EventHeaderFactory::createHeader( CdmwPlatformMngtBase::INF);

    // Prepare RecordInfo
    CdmwPlatformMngt::PropertyInfo p_info;
    p_info.property_name = CORBA::string_dup(CdmwPlatformMngt::REPLICATION_ROLE);
    CdmwPlatformMngt::PropertyInfos_var property_infos = new CdmwPlatformMngt::PropertyInfos();
    CdmwPlatformMngt::RedundancyEltState red_st_new;
    
    if ( "PRIMARY" == role )
    {
        red_st_new = CdmwPlatformMngt::REDSTATE_PRIMARY;
    }
    else if ( "PASSIVE" == role )
    {
        red_st_new = CdmwPlatformMngt::REDSTATE_PASSIVE;
    }
    else if ( "ACTIVE" == role )
    {
        red_st_new = CdmwPlatformMngt::REDSTATE_ACTIVE;
    }
    p_info.property_new_value <<= red_st_new;

    // Prepare PropertyInfos sequence
    property_infos->length(1);
    property_infos[0L] = p_info;
    std::string info= "The redundancy state has changed";
    
    Cdmw::PlatformMngt::PropertyChangeFactory* factory = new Cdmw::PlatformMngt::PropertyChangeFactory;

    property_change = factory->create(
            header,
            element_path.c_str(),
            property_infos, 
            info.c_str());
    
    component_context_ext->notify_property_change (property_change);
    
}

void test_property_change::setUp()
{
    test_property_change::oneShotInit();
    
    //prepare element_path
    std::string sys_postfix = ".sys/";
    std::string appl_postfix = ".appl/";
    std::string host_postfix = ".host/";
    std::string proc_postfix = ".proc";
    std::string comp_postfix = ".comp";
    std::string postfix = "/";

    std::string element_path = 
        component_context_ext->system_name() +sys_postfix
        +component_context_ext->application_name()+appl_postfix
        +component_context_ext->host_name()+host_postfix
        +component_context_ext->process_name()+proc_postfix;

    process_element_path= element_path;
    element_path += postfix + (component_context_ext->managed_element_name())+comp_postfix;
    component_element_path = element_path;
    

}

void test_property_change::oneShotInit()
{

    try
    {
        if (!oneShotInitDone)
        {
            oneShotInitDone = false;

            //added by Replication Role 
            CORBA::Object_var obj 
            = m_session_context->resolve_service_reference("ManagedComponentContextFactory");

            CdmwPlatformMngt::ManagedComponentContextFactory_var component_context_finder = 
                CdmwPlatformMngt::ManagedComponentContextFactory::_narrow (obj.in());

            component_context_ext =
                component_context_finder->find_managed_component_context("Client1");

                CPPUNIT_ASSERT(!CORBA::is_nil(component_context_ext.in()));



                // get ManagedComponentContext
                CORBA::Object_var obj_ctx = 
                    m_session_context->resolve_service_reference("ManagedComponentContext");

                    CdmwPlatformMngt::ManagedElementContexts::ComponentContext_var component_context =
                        CdmwPlatformMngt::ManagedElementContexts::ComponentContext::_narrow(obj_ctx.in());


                    CPPUNIT_ASSERT(!CORBA::is_nil(component_context.in()));

                    // get System supervision object
                    TEST_INFO("Get System supervision object");
                    system = dynamic_cast<CdmwPlatformMngt::System*>(component_context->the_system());

                    // get System supervision object
                    TEST_INFO("Get Process supervision object");
                    host_name = Cdmw::OsSupport::OS::get_hostname(); 
                    process = (system->get_application("TestApplication"))->get_process("TestProcess", host_name.c_str());   
                    component = process->get_component("Client1");  


                    //PCR-1589
            CORBA::Any_var outany;
                    CdmwPlatformMngt::ManagedElement_var managedElement = CdmwPlatformMngt::Process::_duplicate(process);
            
            try{
                        managedElement->get_attribute("dummy", outany, CdmwPlatformMngt::PRE_DEFINED);
                    }
                catch(const CdmwPlatformMngt::AttributeNotFound & e)
                    {
                                    TEST_INFO("In the catch --> const ::CdmwPlatformMngt::AttributeNotFound & e <--");
                        CPPUNIT_ASSERT(true);
                    }   
                    catch(...){
                                TEST_INFO("In the catch --> ... <--");
                        CPPUNIT_ASSERT(false);
                    }
            //end PCR-1589

                    //Setting up the startupKind parameters
                    state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
                    startup_mode = 0;
                    specified_state_id = "";
                    startupKind = new CdmwPlatformMngtBase::StartupKind();
                    startupKind->state_retrieval = state_retrieval;
                    startupKind->startup_mode = startup_mode;
                    startupKind->specified_state_id  = CORBA::string_dup( specified_state_id.c_str() );

                    //Registering the Observer
                    register_observer();
        }

    }catch (const CORBA::Exception & ex)
    {
        std::cerr<<ex;
        CPPUNIT_ASSERT(false);
    }    
};


void test_property_change::ChangeProperty()
{
    try
    {
        CORBA::Any_var value_to_get =new CORBA::Any();
        CdmwPlatformMngt::RedundancyEltState property_value;
        CdmwPlatformMngt::PropertyChange_var property_change;
        
        TEST_INFO("################### NOTIFING COMPONENT ##########################");
        
        
        
        TEST_INFO("################### set Component ReplicationRole to PRIMARY  ##########################");    
        create_and_notify_property_change( component_element_path,"PRIMARY");
        //m_ccm_client->m_condition->wait();
        sleep(2);
        
        //check component
        component->get_attribute(CdmwPlatformMngt::REPLICATION_ROLE, value_to_get.out(), CdmwPlatformMngt::PRE_DEFINED);
        value_to_get.in() >>= property_value;
        CPPUNIT_ASSERT(CdmwPlatformMngt::REDSTATE_PRIMARY == property_value);   
        
        //check process
        process->get_attribute(CdmwPlatformMngt::REPLICATION_ROLE, value_to_get.out(), CdmwPlatformMngt::PRE_DEFINED);
        value_to_get.in() >>= property_value;
        CPPUNIT_ASSERT(CdmwPlatformMngt::REDSTATE_PRIMARY == property_value);           
        TEST_INFO("################### TEST OK! ##########################"); 
        
        
        
        TEST_INFO("################### set Component ReplicationRole to PASSIVE ##########################");  
        create_and_notify_property_change( component_element_path,"PASSIVE");
        //m_ccm_client->m_condition->wait();
        sleep(2);
        
        //check component
        component->get_attribute (CdmwPlatformMngt::REPLICATION_ROLE,
                                  value_to_get.out(), 
                                  CdmwPlatformMngt::PRE_DEFINED);
        value_to_get.in() >>= property_value; 
        CPPUNIT_ASSERT(CdmwPlatformMngt::REDSTATE_PASSIVE == property_value);
        
        //process
        process->get_attribute(CdmwPlatformMngt::REPLICATION_ROLE, value_to_get.out(), CdmwPlatformMngt::PRE_DEFINED);
        value_to_get.in() >>= property_value;
        CPPUNIT_ASSERT(CdmwPlatformMngt::REDSTATE_PASSIVE == property_value);           
        TEST_INFO("################### TEST OK! ##########################"); 
        
        
        
        TEST_INFO("################### set Component ReplicationRole to ACTIVE ##########################");    
        create_and_notify_property_change( component_element_path,"ACTIVE");
        //m_ccm_client->m_condition->wait();
        sleep(2);
        
        //check component
        component->get_attribute (CdmwPlatformMngt::REPLICATION_ROLE,
                                  value_to_get.out(), 
                                  CdmwPlatformMngt::PRE_DEFINED);
        value_to_get.in() >>= property_value;    
        CPPUNIT_ASSERT(CdmwPlatformMngt::REDSTATE_ACTIVE == property_value);
        
        //check process process
        process->get_attribute(CdmwPlatformMngt::REPLICATION_ROLE, value_to_get.out(), CdmwPlatformMngt::PRE_DEFINED);
        value_to_get.in() >>= property_value;
        CPPUNIT_ASSERT(CdmwPlatformMngt::REDSTATE_ACTIVE == property_value);        
        TEST_INFO("################### TEST OK! ##########################");
        
        
        
       /* TEST_INFO("################### reset ReplicationRole to NORG default value ##########################"); 
        
        //Getting the application APPL1 reference 
        CdmwPlatformMngt::Application_var application = system->get_application("TestApplication");
        //Getting the processes references
        CdmwPlatformMngt::Process_var killer_process = 
            application->get_process("Killer_Process", host_name.c_str());
            
        TEST_INFO("Killing the Process "<<host_name);
        sleep(2);   
        try
        {
            std::cerr<<"\nPAUSE";
            system("read foo");
            //Running the Process Killer 
            killer_process->initialise(startupKind.in());                   
        }
        catch(const CORBA::SystemException& ex)
        {
            TEST_INFO("\nError running process..."<<__LINE__<<__FILE__);
        }
        sleep(2);

        std::cerr<<"\nPAUSE";
        system("read foo");
        //check component
        component->get_attribute (CdmwPlatformMngt::REPLICATION_ROLE,
                                  value_to_get.out(), 
                                  CdmwPlatformMngt::PRE_DEFINED);
        value_to_get.in() >>= property_value;    
        CPPUNIT_ASSERT(CdmwPlatformMngt::REDSTATE_NORG == property_value);
        
        //check process process
        process->get_attribute(CdmwPlatformMngt::REPLICATION_ROLE, value_to_get.out(), CdmwPlatformMngt::PRE_DEFINED);
        value_to_get.in() >>= property_value;
        CPPUNIT_ASSERT(CdmwPlatformMngt::REDSTATE_NORG == property_value);      
        TEST_INFO("################### TEST OK! ##########################");*/
        
        
        Cdmw::TestUtils::CORBATestManager::instance()->summary();

    }catch (const CORBA::Exception & ex)
    {
        std::cerr<<ex;
        CPPUNIT_ASSERT(false);
    }  
    
};

};  // namespace TestPropertyChange


