/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_TESTOBJS_HPP
#define INCL_PLATFORMMNGT_TESTOBJS_HPP

#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/common/System.hpp"
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/Exception.hpp>
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"

#include "Foundation/common/System.hpp"

#include  "platformadminlibrary/PlatformAdminSnapshot.hpp"

namespace Cdmw
{
  namespace PlatformMngt
  {

    class TestObjs:public CppUnit::TestFixture
    {

    public:

        TestObjs();
        
        void setUp ();
 
        virtual void test_system_start ();
        
        virtual void test_restart_process ();
        
        virtual void test_restart_process_with_env ();
        
        virtual void test_restart_application ();
        
        
    private:
    
        //System reference
        CdmwPlatformMngt::System_var system ;
                
        //Application reference        
        CdmwPlatformMngt::Application_var appl;
        
        //Process reference        
        CdmwPlatformMngt::Process_var proc;       
        
        
        CdmwPlatformMngtBase::StartupKind_var startupKind;
        CdmwPlatformMngtBase::StateRetrieval state_retrieval;        
        CORBA::String_var statusInfo;        
        unsigned long startup_mode;
        std::string specified_state_id;
        
        int long_timeout;
        int short_timeout;
        
        
        /*
         * Vector of expected environment values
         */
        std::vector<std::string> m_env_expected;
        
        /**
         * Purpose
         * <p>:
         * This function is useful to compare the value of STARTMODE 
         * saved in file_data with the expected value.
         * The Default parameter ignoreEnv is useful to decide
         * if check or not the enviroment variables written in the 
         * process description contained in the scenario file.                   
         **/
        bool check_STARTMODE(const std::string file,const std::string expected_value,bool ignoreEnv=true)throw();
        
        
        std::string file_data;
           
        /**
         * Assignment operator
         */
        TestObjs & operator= (const TestObjs & rhs) throw ();
        
        /**
         * Copy constructor
         */
        TestObjs (const TestObjs & rhs) throw ();

        /**
         * Defining a Test Suite
         */       
        CPPUNIT_TEST_SUITE (TestObjs);
        
            //These are the test cases
            CPPUNIT_TEST (test_system_start); 
            CPPUNIT_TEST (test_restart_process);
            CPPUNIT_TEST (test_restart_process_with_env);
            CPPUNIT_TEST (test_restart_application);
           
            
        CPPUNIT_TEST_SUITE_END ();
    };

  }             /* End namespace PlatformMngt */
}               /* End namespace Cdmw */
#endif              /* INCL_PLATFORMMNGT_TESTRESTARTOBJS_HPP */
