/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <fstream>
#include <limits>

#include "test_requested_restart_warmth/TestObjs.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include <SystemMngt/platformlibrary/Iterator.hpp>
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"


using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestObjs);

namespace Cdmw
{
  namespace PlatformMngt
  {
    
    TestObjs::TestObjs()
    {
        /*Creating the environment following 
         * the configuration in the scenario file
         */
        m_env_expected.push_back("1");  
        m_env_expected.push_back("2");
        m_env_expected.push_back("string string");
        m_env_expected.push_back("string");
        m_env_expected.push_back("string string string");
        
    }

    void TestObjs::setUp ()
    {
          try
          {
          //Getting system var
          std::string system_corbaloc = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";
          CORBA::ORB_var orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::instance ()->get_ORB ());
          CORBA::Object_var system_obj = orb->string_to_object (system_corbaloc.c_str());
          system = CdmwPlatformMngt::System::_narrow (system_obj.in ());
          
          //Setting up the startupKind parameters
          state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
          startup_mode = 0;
          specified_state_id = "";
          startupKind = new CdmwPlatformMngtBase::StartupKind();
          startupKind->state_retrieval = state_retrieval;
          startupKind->startup_mode = startup_mode;
          startupKind->specified_state_id  = CORBA::string_dup( specified_state_id.c_str() );
          
          
          //Specifying the file_data that contain the value of STARTMODE 
          file_data="Process_env.dat";
          
          //Getting the application reference      
          appl=system->get_application("StartModeApplication");
          //Getting the process reference
          proc=appl->get_process("Process_env",getenv("HOSTNAME"));
          
          //Setting the timeouts         
          long_timeout=6;
          short_timeout=2;
           
          }
          catch (const CORBA::SystemException& ex) 
          {
              std::cerr<< ex._name()<<std::endl;
              throw;
          }
          catch(...)
          {
          }
    }

    void TestObjs::test_system_start() 
    {
       
        
        try {
            std::string expected;                        
            CdmwPlatformMngt::RestartWarmth drw;
            CORBA::Any_var any_var = new CORBA::Any();
            try {
            proc->get_attribute (CdmwPlatformMngt::REQUESTED_RESTART_WARMTH, any_var.out(), CdmwPlatformMngt::PRE_DEFINED);
            any_var.in() >>= drw;
            }
            catch(...) {}
            if(CdmwPlatformMngt::HOT_RESTART==drw)
                expected="HOT";
            else expected="COLD";           
            
            TEST_INFO("Controlling if START_MODE variable is correctly set after the default start system ");
            CPPUNIT_ASSERT(check_STARTMODE(file_data,expected));
            
        }
        catch (...)
        {
            CPPUNIT_ASSERT (false);
            throw;
        }  
        
    }
    
    
   void TestObjs::test_restart_application ()
    {
        
                   
        try 
        {
           appl->cold_restart(startupKind.in());
           sleep(long_timeout);
           TEST_INFO("Controlling if the process enviroment variable called START_MODE is correctly set COLD after an application cold_restart");
           CPPUNIT_ASSERT(check_STARTMODE(file_data,"COLD"))  ;
           appl->hot_restart(startupKind.in());
           sleep(long_timeout);
           TEST_INFO("Controlling if the process enviroment variable called START_MODE is correctly set HOT after an application hot_restart")
           CPPUNIT_ASSERT(check_STARTMODE(file_data,"HOT"))  ;       
        }
        catch (const CORBA::SystemException& e)
        {
            std::cout<<e<<std::endl;
            CPPUNIT_ASSERT (false);
            throw;
        }  
    } 
    
 
    void TestObjs::test_restart_process ()
    {
        
                   
        try 
        {
           proc->cold_restart(startupKind.in());
           sleep(long_timeout);
           TEST_INFO("Controlling if the process enviroment variable called START_MODE is correctly set COLD after a process cold_restart");
           CPPUNIT_ASSERT(check_STARTMODE(file_data,"COLD"))  ;
           proc->hot_restart(startupKind.in());
           sleep(long_timeout);
           TEST_INFO("Controlling if the process enviroment variable called START_MODE is correctly set HOT after a process hot_restart")
           CPPUNIT_ASSERT(check_STARTMODE(file_data,"HOT"))  ;       
        }
        catch (const CORBA::SystemException& e)
        {
            std::cout<<e<<std::endl;
            CPPUNIT_ASSERT (false);
            throw;
        }
    }
        
    void TestObjs::test_restart_process_with_env()
    {


        try 
        {
            proc->cold_restart(startupKind.in());
            sleep(long_timeout);
            TEST_INFO("Controlling if the process enviroment variable called START_MODE is correctly set COLD after a process cold_restart");
            TEST_INFO("Controlling if the process enviroment variables defined in the scenario are correctly set");
            CPPUNIT_ASSERT(check_STARTMODE(file_data,"COLD",false))  ;
            
            proc->hot_restart(startupKind.in());
            sleep(long_timeout);
            TEST_INFO("Controlling if the process enviroment variable called START_MODE is correctly set HOT after a process hot_restart")
            TEST_INFO("Controlling if the process enviroment variables defined in the scenario are correctly set");
            CPPUNIT_ASSERT(check_STARTMODE(file_data,"HOT",false))  ; 
            
        }
        catch (const CORBA::SystemException& e)
        {
            std::cout<<e<<std::endl;
            CPPUNIT_ASSERT (false);
            throw;
        }  
    }

    
    bool TestObjs::check_STARTMODE(const std::string file,const std::string expected_value,bool ignoreEnv)throw()
    {
        ifstream i_file(file.c_str());
        std::string f;
        bool result;                                                       
        if(i_file)
        {                    
         std::getline(i_file,f);
          //Checking the values saved in the file process_data 
          result = (f==expected_value.c_str());          
        }
        else
        {
            std::cerr<<"\nERROR READING "<< file;
            result=false;
        } 
        
        if(!ignoreEnv)
        {
            int i=0;
            while (!i_file.eof() && result && i<5)
            {
              std::getline (i_file,f);
              result = (f == m_env_expected[i].c_str());
              i++;
              }
         }
         
        try
        {    
            Cdmw::OsSupport::OS::unlink(file.c_str());
        }
        catch(...)
        {
            std::cerr<<"ERROR DELETING "<<file;
        }
        sleep(short_timeout);
    return result;
    
    }
    
    

}/* End namespace PlatformMngt */
}/* End namespace Cdmw */





