/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "Foundation/common/Options.hpp"
#include <Foundation/common/Locations.hpp>
#include "Foundation/common/String.hpp"
#include "Foundation/common/Exception.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/common/Locations.hpp"

#include "Foundation/orbsupport/CORBA.hpp"

#include "Foundation/ossupport/OS.hpp"
#include "Foundation/osthreads/ThreadHandle.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include <Foundation/logging/LogManager.hpp> 
#include <Foundation/logging/FilterMngr.hpp>

#include "SystemMngt/platforminterface/PlatformInterface.hpp"
#include "SystemMngt/platforminterface/ServiceNames.hpp"

#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include "Repository/repositoryinterface/RepositoryInterface.hpp"

#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"
#include "idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"

#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/LogManager.hpp>

#include "test_restart_control/MyProcessBehaviour.hpp"

#include <cstdlib>
#include <string>
#include <exception>
#include <sstream>
#include <iostream>
#include <fstream>

#undef  CDMW_TRACE_LEVEL
#define CDMW_TRACE_LEVEL 6

const int POA_THREAD_POOL_SIZE = 5;

using Cdmw::NamingAndRepository::RepositoryInterface;
using Cdmw::Logging::Logger;
using Cdmw::Logging::LogManager;

using namespace Cdmw::OsSupport;
using namespace Cdmw::OrbSupport;
using namespace Cdmw::PlatformMngt;

std::string applicationName;
std::string processName;
std::string hostName ;

int main(int argc, char* argv[])
{
    // the return code 
    StrategyList strategy;
    strategy.add_OrbThreaded();
    strategy.add_PoaThreadPool(POA_THREAD_POOL_SIZE);

    CORBA::ORB_var orb;


    try
    {
                
        // initialise the orb    
        orb =  Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, strategy);

        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");

        PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj.in());

        PortableServer::POAManager_var poaMgr = rootPOA->the_POAManager();

        poaMgr->activate();

        std::cout << "Init PlatformInterface" << std::endl;

        // initialise the platform interface
        PlatformInterface::Setup(orb.in(), argc, argv);

        // creates the process behaviour
        std::auto_ptr<MyProcessBehaviour> procBehaviour( new MyProcessBehaviour(orb.in() ));
        // get processId
        OS::ProcessId processId = OS::get_processId();

        // acknowledge the creation of the process
        PlatformInterface::Acknowledge_creation(procBehaviour.get());

        MyProcessBehaviour * pProcBehaviour = procBehaviour.release();

        applicationName = PlatformInterface::Get_application_name();

        processName = PlatformInterface::Get_process_name();


        std::cout << "Application name : "
                      << applicationName.c_str() << std::endl;

        std::cout << "Process name : "
                      << processName.c_str()
                      << " Process id : "
                      << processId << std::endl;


        //resolve system ptr
        CORBA::Object_var objref2; 

        /*define the corbaloc */
        const char *corbaloc_str = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";

        objref2 = orb->string_to_object (corbaloc_str);

        /* resolve the System */
        CdmwPlatformMngt::System_var mysystem = CdmwPlatformMngt::System::_narrow (objref2.in ());

        if (CORBA::is_nil (mysystem.in ()))
          {
            std::cout<<"Error in narrowing CdmwPlatformMngt::System"<<std::endl;
            exit (-1);
          }
 
        CdmwPlatformMngt::Application_var app = mysystem->get_application ("HELLO_WORLD");

        std::string proc_name = "TraceClient";
        char *hostname = getenv ("HOSTNAME");
        CdmwPlatformMngt::Process_var proc = app->get_process (proc_name.c_str (), hostname);
        if (CORBA::is_nil (proc.in ())) 
          {
           std::cout<<"*******ERROR in proc_name, is nil *************"<<std::endl;
           exit (-1);
          }

        if (CORBA::is_nil (proc.in ()))
          {
           std::cout<<"*******ERROR in proc_name, is nil *************"<<std::endl;
           exit (-1);
          }

        //try a getting of auto_restart_control info for managed process

         CORBA::Any_var any_var;
         CdmwPlatformMngt::AutoRestartInfo* ari;  
         proc -> get_attribute (CdmwPlatformMngt::AUTORESTART_INFO, any_var.out(), CdmwPlatformMngt::PRE_DEFINED);
         any_var.in() >>= ari;
         CdmwPlatformMngt::AutoRestartControl auto_restart_control = static_cast<CdmwPlatformMngt::AutoRestartControl>(ari->auto_restart_control);
CORBA::ULong max_nbr_of_restart_attempt = static_cast<CORBA::ULong>(ari->max_nbr_of_restart_attempt);

      if (CdmwPlatformMngt::IGNORE_COUNTER==auto_restart_control && 5==max_nbr_of_restart_attempt) {
          std::cout<<"TEST NUM. 1 -->  OK: auto_restart_control = "<<auto_restart_control<<" max_nbr_of_restart_attempt = "<<max_nbr_of_restart_attempt<<std::endl;
      }
        else
      {
 std::cout<<"TEST NUM. 1 -->  Error: auto_restart_control = "<<auto_restart_control<<" max_nbr_of_restart_attempt = "<<max_nbr_of_restart_attempt<<std::endl;
      }


        //try a setting of auto_restart_control info for managed process

        char flag_test2=0;

        CdmwPlatformMngt::AutoRestartInfo autorestart_info;
        autorestart_info.max_nbr_of_restart_attempt = 3;
        autorestart_info.auto_restart_control = CdmwPlatformMngt::IGNORE_ACTION;
        CORBA::Any_var myany = new CORBA::Any();
        try {
            *myany <<= autorestart_info;
            proc->set_attribute(CdmwPlatformMngt::AUTORESTART_INFO, myany.in(), CdmwPlatformMngt::PRE_DEFINED);
        }
        catch(...)
        {
            std::cout<<"TEST NUM. 2 -->  Error: reached unexpected exception of auto_restart_info setting to a managed process "<<std::endl;
            flag_test2 = 1 ;
        }


        if ( flag_test2 == 0){
          //std::cout<<"TEST NUM. 2 -->  OK : setting of auto_restart_info to the managed process"<<std::endl;



         //try a new getting of auto_restart_control info

         CORBA::Any_var any_var;
         CdmwPlatformMngt::AutoRestartInfo* ari;
         proc -> get_attribute (CdmwPlatformMngt::AUTORESTART_INFO, any_var.out(), CdmwPlatformMngt::PRE_DEFINED);
         any_var.in() >>= ari;
         CdmwPlatformMngt::AutoRestartControl auto_restart_control = static_cast<CdmwPlatformMngt::AutoRestartControl>(ari->auto_restart_control);

      if (CdmwPlatformMngt::IGNORE_ACTION==auto_restart_control) {
          std::cout<<"TEST NUM. 2 -->  OK : setting of auto_restart_info to the managed process"<<std::endl;
      }
        else
      {
          std::cout<<"TEST NUM. 2 -->  Error in setting of auto_restart_info: auto_restart_control = "<<auto_restart_control<<std::endl;
      }

     }




        //try a getting of auto_restart_control info for unmanaged process
         std::string proc_name_unmanaged = "HelloProcess1";
         CdmwPlatformMngt::Process_var proc_unmanaged = app->get_process (proc_name_unmanaged.c_str (), hostname);

         proc_unmanaged -> get_attribute (CdmwPlatformMngt::AUTORESTART_INFO, any_var.out(), CdmwPlatformMngt::PRE_DEFINED);
         any_var.in() >>= ari;
         auto_restart_control = static_cast<CdmwPlatformMngt::AutoRestartControl>(ari->auto_restart_control);
         max_nbr_of_restart_attempt = static_cast<CORBA::ULong>(ari->max_nbr_of_restart_attempt);


      if (CdmwPlatformMngt::USE_COUNTER==auto_restart_control && 3==max_nbr_of_restart_attempt) {
          std::cout<<"TEST NUM. 3 -->  OK: auto_restart_control = "<<auto_restart_control<<" max_nbr_of_restart_attempt = "<<max_nbr_of_restart_attempt<<std::endl;
      }
        else
      {
          std::cout<<"TEST NUM. 3 -->  Error: auto_restart_control = "<<auto_restart_control<<" max_nbr_of_restart_attempt = "<<max_nbr_of_restart_attempt<<std::endl;
      }





  //try a setting of auto_restart_control info for unmanaged process

        char flag_test4=0;

        autorestart_info.max_nbr_of_restart_attempt = 3;
        autorestart_info.auto_restart_control = CdmwPlatformMngt::IGNORE_COUNTER;
        myany = new CORBA::Any();
        try {
            *myany <<= autorestart_info;
            proc_unmanaged->set_attribute(CdmwPlatformMngt::AUTORESTART_INFO, myany.in(), CdmwPlatformMngt::PRE_DEFINED);
        }
        catch(...)
        {
            std::cout<<"TEST NUM. 4 -->  Error: reached unexpected exception of auto_restart_info setting to an unmanaged process "<<std::endl;
            flag_test4 = 1 ;
        }


        if ( flag_test4 == 0){
          //std::cout<<"TEST NUM. 2 -->  OK : setting of auto_restart_info to the managed process"<<std::endl;



    //try a new getting of auto_restart_control info

         proc_unmanaged -> get_attribute (CdmwPlatformMngt::AUTORESTART_INFO, any_var.out(), CdmwPlatformMngt::PRE_DEFINED);
         any_var.in() >>= ari;
         auto_restart_control = static_cast<CdmwPlatformMngt::AutoRestartControl>(ari->auto_restart_control);

      if (CdmwPlatformMngt::IGNORE_COUNTER==auto_restart_control) {
          std::cout<<"TEST NUM. 4 -->  OK : setting of auto_restart_info to the unmanaged process"<<std::endl;
      }
        else
      {
          std::cout<<"TEST NUM. 4 -->  Error in setting of auto_restart_info: auto_restart_control = "<<auto_restart_control<<std::endl;
      }

     }

        // advise behavior that init is done
        pProcBehaviour->set_init_done();
        pProcBehaviour->get_condition().broadcast();

        // start orb
        orb->run();

        // cleanup the platform interface
        PlatformInterface::Cleanup();


       }
       catch(...)
        {
        std::cout<<"*****ERROR : reached exception in managed process *****"<<std::endl;
            // cleanup the ORB to exit completely
            orb->shutdown(false);
                    Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
                    orb->destroy();
            
        return(-1);
        }

    return 0;

}
