/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_TESTRESTARTOBJS_HPP
#define INCL_PLATFORMMNGT_TESTRESTARTOBJS_HPP

#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/common/System.hpp"
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/Exception.hpp>
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"

#include "Foundation/common/System.hpp"

#include  "platformadminlibrary/PlatformAdminSnapshot.hpp"
extern bool show_properties;
namespace Cdmw
{
  namespace PlatformMngt
  {

    /**
     *Purpose:
     *<p>
     *
     *Features:
     *<p> Thread safe
     *
     *
     */
    class TestRestartObjs:public CppUnit::TestFixture
    {

    public:

      TestRestartObjs();
      TestRestartObjs(const std::string& host_name, const std::string& application_name,const std::string& process_name);

      TestRestartObjs(const std::string& application_name);



      void setUp ();

      CdmwPlatformMngt::Application_var application_test ;
      CdmwPlatformMngt::Process_ptr     process_test;
      CdmwPlatformMngt::Host_var        host_test ;  
      CdmwPlatformMngt::System_var      system_test ;

      std::string object_name;
      CdmwPlatformMngt::EntityDef ent_def;
      //defining Properties
      CORBA::Any anyp_val;
      std::string sprop;
      int val;


      CdmwPlatformMngtBase::StateRetrieval state_retrieval;
      unsigned long startup_mode;
      std::string specified_state_id; 


      CdmwPlatformMngtBase::StartupKind_var startupKind;

      CdmwPlatformMngt::SystemSnapshot_var default_sys_snap;
      CdmwPlatformMngt::ApplicationSnapshot_var default_app_snap;
      CdmwPlatformMngt::ProcessSnapshot_var default_proc_snap;


      /**
       * Purpose:
       * <p> Call from the testable object
       *
       */
 
      virtual void test_restart_system ();
      virtual void test_restart_application ();
      virtual void test_restart_process ();


    private:


      //static bool one_shot;

      /**
       * Purpose:
       * <p> Assignment operator
       *
       */
      TestRestartObjs & operator= (const TestRestartObjs & rhs)
    throw ();
      /**
       * Purpose:
       * <p>  Copy constructor
       *
       */
      TestRestartObjs (const TestRestartObjs & rhs) throw ();


    
      CPPUNIT_TEST_SUITE (TestRestartObjs);
      // Testing Host 
      CPPUNIT_TEST (test_restart_system);
      CPPUNIT_TEST (test_restart_application);
      CPPUNIT_TEST (test_restart_process);         
 
            
      CPPUNIT_TEST_SUITE_END ();
    };

  }             /* End namespace PlatformMngt */
}               /* End namespace Cdmw */
#endif              /* INCL_PLATFORMMNGT_TESTRESTARTOBJS_HPP */
