/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_TESTOBJS_HPP
#define INCL_PLATFORMMNGT_TESTOBJS_HPP

#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/common/System.hpp"
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/Exception.hpp>
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"

#include "Foundation/common/System.hpp"
#include "idllib/CdmwPlatformMngtDaemon.stub.hpp"
#include  "platformadminlibrary/PlatformAdminSnapshot.hpp"
#include "test_status_consistency/ProcObserver_impl.hpp"

namespace Cdmw
{
  namespace PlatformMngt
  {

    class TestObjs:public CppUnit::TestFixture
    {

    public:

        TestObjs();
        
        void setUp ();

        virtual void test_status_consistency (); 
    
    private:
    
        //System reference
        CdmwPlatformMngt::System_var system ;
        //Daemon reference         
        CdmwPlatformMngt::Daemon_var m_daemon1,m_daemon2; 
        
        //CdmwPlatformMngtBase::StartupKind_var startupKind;
        //CdmwPlatformMngtBase::StateRetrieval state_retrieval;
        
        CORBA::String_var statusInfo;
        CORBA::ORB_var orb;
        
        int val;
        int long_timeout;
        int short_timeout;
        unsigned long startup_mode;
        std::string specified_state_id;

        /**
         * This function controls if system is started
         */
        bool sys_status_control (const CdmwPlatformMngt::System_ptr ptr) throw (); 
        
        /**
         * These functions control for Application/Process with force_manual_start set to manual,
         * if the current status is the status expected, that is give as parameter
         */
        bool app_status_control (const CdmwPlatformMngt::Application_ptr ptr,const CdmwPlatformMngt::ApplicationStatus& expected_status) throw ();
        
        bool proc_status_control (const CdmwPlatformMngt::Process_ptr ptr, const CdmwPlatformMngt::ProcessStatus& expected_status) throw ();

        template <class T>
           bool check_status(T element, int status)
        {
            int check = false;
            int current_status = element->get_status(statusInfo.out());
            if (current_status == status)
               check = true;
  
            return check;
        } 
        
        /**
         * Assignment operator
         */
        TestObjs & operator= (const TestObjs & rhs) throw ();
        
        /**
         * Copy constructor
         */
        TestObjs (const TestObjs & rhs) throw ();

        /**
         * Defining a Test Suite
         */       
        CPPUNIT_TEST_SUITE (TestObjs);
  
      
            //These are the test cases
        CPPUNIT_TEST(test_status_consistency);
        CPPUNIT_TEST_SUITE_END ();
    };

  }             /* End namespace PlatformMngt */
}               /* End namespace Cdmw */
#endif              /* INCL_PLATFORMMNGT_TESTRESTARTOBJS_HPP */
