/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <iomanip>
#include <typeinfo>
#include <fstream>

#include "Foundation/ossupport/OS.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/common/Options.hpp"
#include "Foundation/common/Exception.hpp"

#include "SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/EntityStatusChange_impl.hpp"

#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "test_status_consistency/ProcObserver_impl.hpp"



namespace Cdmw
  {

  namespace PlatformMngt
    {

    void  ProcObserver_impl::notify(CdmwPlatformMngt::Event* an_event)
    throw( CORBA::SystemException )
    {
       try
       {

           CdmwPlatformMngt::ProcessStatusChange* prc_chg = dynamic_cast<CdmwPlatformMngt::ProcessStatusChange*> (an_event);
         
           if ((prc_chg!=0))
           {
               //  CORBA::String_var message = an_event->to_string();
               //  std::cerr << message.in() << std::endl;
               process_status = prc_chg->process_status(); 
           }
           
           CdmwPlatformMngt::ApplicationStatusChange* app_chg = dynamic_cast<CdmwPlatformMngt::ApplicationStatusChange*> (an_event);

           if (app_chg!=0)
           {
               //   CORBA::String_var message = an_event->to_string();
               //   std::cerr << message.in() << std::endl;
               application_status = app_chg->application_status();
           }

           CdmwPlatformMngt::SystemStatusChange* sys_chg = dynamic_cast<CdmwPlatformMngt::SystemStatusChange*> (an_event);
 
           if ( (sys_chg!=0))
           {
               //number++;
               //CORBA::String_var message = an_event->to_string();
               //   std::cerr << message.in() << std::endl;
               system_status = sys_chg->system_status();
           }
           
        }
        catch (const std::exception& ex)
        {
            std::cerr << " Process observer exception  " << ex.what();
        }
    }
    
    bool  ProcObserver_impl::has_status(int status)
    {
        bool check = false;
        if (status == process_status)
            check = true;
        return check;
    }

    bool  ProcObserver_impl::has_system_status(CdmwPlatformMngt::SystemStatus status)
    {
        bool check = false;
        if (status == system_status)
            check = true;
        return check;
    }

    bool  ProcObserver_impl::has_application_status(int status)
    {
        bool check = false;
        if (status == application_status)
            check = true;
        return check;
    }

   
 
  } // PlatformMngt
} // Cdmw

