/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <fstream>
#include <limits>

#include "test_status_consistency/TestObjs.hpp"
#include "test_status_consistency/ProcObserver_impl.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"

#include <SystemMngt/platformlibrary/Iterator.hpp>
#include <SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp>
#include <idllib/CdmwPlatformMngtSystem.stub.hpp>
#include <idllib/CdmwPlatformMngtProcess.stub.hpp>
#include <idllib/CdmwPlatformMngtCommon.stub.hpp>
#include <idllib/CdmwPlatformMngtApplicationAgent.stub.hpp>


#include <SystemMngt/platformvaluetypes/HostStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/ApplicationStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/EntityStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/SystemStatusChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/SystemModeChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/ApplicationModeChange_impl.hpp>
#include <SystemMngt/platformvaluetypes/PropertyChange_impl.hpp>


using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestObjs);

namespace Cdmw
{
  namespace PlatformMngt
  {
    
    TestObjs::TestObjs()
    {
    }

    void TestObjs::setUp ()
    {
       try
       {
          //Getting system var
          std::string system_corbaloc = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";
          orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::instance ()->get_ORB ());
          CORBA::Object_var system_obj = orb->string_to_object (system_corbaloc.c_str());
          system = CdmwPlatformMngt::System::_narrow (system_obj.in ());
          
          //Setting up the timeouts
          long_timeout  = 4 ; 
          short_timeout = 1 ;

          CORBA::ValueFactoryBase_var factory, old_factory;
 
          factory = new ApplicationStatusChangeFactory;
          old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationStatusChange:1.0",factory.in() );
          factory = new ProcessStatusChangeFactory;
          old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/ProcessStatusChange:1.0",factory.in() );
          factory = new SystemStatusChangeFactory;
          old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/SystemStatusChange:1.0",factory.in() );
          factory = new HostStatusChangeFactory;
          old_factory = orb->register_value_factory("IDL:thalesgroup.com/CdmwPlatformMngt/HostStatusChange:1.0",factory.in() );
      }
       catch(...)
        {
           CPPUNIT_ASSERT(false);
        }
    }


    void TestObjs::test_status_consistency() 
    {
       int end_timeout = 10;
       int count = 0;

       // define processes and process iterator to recover process name
       CdmwPlatformMngt::ProcessesIterator_var proc_iter;
       CdmwPlatformMngt::Processes_var processes_from_host1, processes_from_host2;
       CdmwPlatformMngt::Applications_var applications;
       CdmwPlatformMngt::Application_var application1,application2,application3;   
   
       CdmwPlatformMngt::ElementNames_var elems;
       CdmwPlatformMngt::ElementNamesIterator_var it;

       // get number of hosts
       CORBA::ULong nbr_of_host = system->get_number_of_hosts();
       CORBA::ULong nbr_of_apps = system->get_number_of_applications();
 
       if ((nbr_of_host != 2) || (nbr_of_apps!=3))
       {
           TEST_INFO("Invalid scenario");
           CPPUNIT_ASSERT(false);
       }

       // get the name of hosts
       system->get_all_host_names(nbr_of_host, elems.out(), it.out());
       
       const char* host_name1 = elems[0U];
       const char* host_name2 = elems[1U];

       
       CdmwPlatformMngt::ApplicationsIterator_var it_app;
       applications = system->get_applications (nbr_of_apps, it_app.out());

        
       CdmwPlatformMngt::Host_var  host1 = system->get_host(host_name1);
       CdmwPlatformMngt::Host_var  host2 = system->get_host(host_name2);

       string application_name1(applications[0U]->name());
       string application_name2 = applications[1U]->name();
       string application_name3 = applications[2U]->name();
      
       application1 = system->get_application(application_name1.c_str());
       application2 = system->get_application(application_name2.c_str());
       application3 = system->get_application(application_name3.c_str());
 
       processes_from_host1 = host1->get_processes (10, proc_iter);
       processes_from_host2 = host2->get_processes (10, proc_iter);

       CdmwPlatformMngt::Process_var process1 = processes_from_host1[0U];
       CdmwPlatformMngt::Process_var process2 = processes_from_host2[0U]; 
  
       CORBA::ULong all_processes_host1 = processes_from_host1->length();
       CORBA::ULong all_processes_host2 = processes_from_host2->length();

       if ((0 == processes_from_host1->length())||(0 == processes_from_host2->length()))
       {
          TEST_INFO("No process on this host");
          CPPUNIT_ASSERT(false);
       }

       std::string process_name_host1 = process1->name ();
       std::string process_name_host2 = process2->name ();

       std::cerr << " " << process_name_host1 << process_name_host2; 
       CdmwPlatformMngt::ApplicationAgent_var application_agent1, application_agent2, application_agent3, application_agent4;

       std::string daemon_corbaloc1 = "corbaloc::";
       daemon_corbaloc1+= host_name1;
       daemon_corbaloc1+= ":21869/CdmwPlatformMngtDaemon";

       std::string daemon_corbaloc2 = "corbaloc::";
       daemon_corbaloc2+= host_name2; 
       daemon_corbaloc2+= ":21869/CdmwPlatformMngtDaemon";

       // recover of Daemon to kill process
       CORBA::Object_var obj = orb->string_to_object (daemon_corbaloc1.c_str());

       if (!CORBA::is_nil (obj.in()))
       {
           m_daemon1 = CdmwPlatformMngt::Daemon::_narrow (obj.in());
       }
       // recover of Daemon to kill process
       obj = orb->string_to_object (daemon_corbaloc2.c_str());

       if (!CORBA::is_nil (obj.in()))
       {
           m_daemon2 = CdmwPlatformMngt::Daemon::_narrow (obj.in());
       }

       try
       {   // search application agent to kill process
           application_agent1 = m_daemon2->search_application_agent(application_name1.c_str());
           if (!CORBA::is_nil (application_agent1.in()))
                application_agent4 = m_daemon1->search_application_agent(application_name1.c_str());
           if (CORBA::is_nil (application_agent1.in()))
           {
               // no agent provided by daemon 2
               application_agent1 = m_daemon1->search_application_agent(application_name1.c_str());
               if (CORBA::is_nil (application_agent1.in())) CPPUNIT_ASSERT(false);
               std::cerr << "application_agent1 by daemon1 with " << application_name1 << std::endl;
           }
           else
           {
               std::cerr << "application_agent1 by daemon2 with" << application_name1 << std::endl;
           }  
           // search application agent to kill process
           application_agent2 = m_daemon2->search_application_agent(application_name2.c_str());
           if ((!CORBA::is_nil (application_agent2.in()))&&(CORBA::is_nil (application_agent4.in())))
                application_agent4 = m_daemon1->search_application_agent(application_name1.c_str());
           if (CORBA::is_nil (application_agent2.in()))
           {
               // no agent provided by daemon 2
               application_agent2 = m_daemon1->search_application_agent(application_name2.c_str()); 
               if (CORBA::is_nil (application_agent2.in())) CPPUNIT_ASSERT(false);
               std::cerr << "application_agent2 by daemon2 with" << application_name2 << std::endl;
           }
           else
           { 
               std::cerr << "application_agent2 by daemon1 with " << application_name2 << std::endl; 
           } 
           // search application agent to kill process
           application_agent3 = m_daemon2->search_application_agent(application_name3.c_str());
           if ((!CORBA::is_nil (application_agent3.in()))&&(CORBA::is_nil (application_agent4.in())))
                application_agent4 = m_daemon1->search_application_agent(application_name1.c_str());
           if (CORBA::is_nil (application_agent3.in()))
           {
              // no agent provided by daemon 2 
              application_agent3 = m_daemon1->search_application_agent(application_name3.c_str());
              if (CORBA::is_nil (application_agent3.in())) CPPUNIT_ASSERT(false);
              std::cerr << "application_agent3 by daemon2 with " << application_name3 << std::endl;
           }
           else
           { 
               std::cerr << "application_agent3  by daemon1 with " << application_name3 << std::endl;
           }  
       }
       catch(...)
       {
           CPPUNIT_ASSERT(false);
       }

       ProcObserver_impl observer_on_system;
       CdmwPlatformMngt::SupervisionObserver_var observer_sys = observer_on_system._this();
 
       //CdmwPlatformMngt::RegistrationDepth depth =  FULL_HIERARCHY; //OBJECT_ONLY;
       TEST_INFO(" Registered Observers: process_observer_on_system"); 

       //system -> register_observer_ext ("sys_observer_on_host1",depth, observer_sys.in ());         
       system  ->register_observer("sys_observer_on_host1", observer_sys.in ());


       /////////////
       // TEST  
       /////////////

       // STOP A PROCESS SYSTEM_PARTIAL_STARTED AND APPLICATION_PARTIAL_RUNNING
       TEST_INFO("STOP A PROCESS SYSTEM_PARTIAL_STARTED AND APPLICATION_PARTIAL_RUNNING");
       std::string process_host;
       std::string admin_args;
       std::string admin_args_process_stop("--proc-stop");
       admin_args = " " + application_name1 + " " + process_name_host2 + " " + host_name2;
 
       OsSupport::OS::create_process("platform_admin.sh", admin_args_process_stop + admin_args);

       sleep(3);
       TEST_INFO(" check the system is SYSTEM_PARTIAL_STARTED ");
       CPPUNIT_ASSERT(check_status<CdmwPlatformMngt::System_ptr>(system.in(), CdmwPlatformMngt::SYSTEM_PARTIAL_STARTED));
       TEST_INFO(" check the application is APPLICATION_PARTIAL_RUNNING ");
       CPPUNIT_ASSERT(check_status<CdmwPlatformMngt::Application_ptr>(application1.in(), CdmwPlatformMngt::APPLICATION_PARTIAL_RUNNING));


       // STOP ALL PROCESSES  APPLICATION_STOPPED AND SYSTEM_STOPPED
       TEST_INFO("STOP ALL PROCESSES => APPLICATION_STOPPED AND SYSTEM_STOPPED");  
       std::string current_process;
       std::string current_application;
       for (CORBA::ULong i = 0; i < all_processes_host1; i++)
       {
           try
           {
               current_process = processes_from_host1[i] -> name();
               current_application = processes_from_host1[i] -> get_parent()->name();  
               admin_args = " " + current_application + " " + current_process + " " + host_name1;
               OsSupport::OS::create_process("platform_admin.sh", admin_args_process_stop + admin_args);
           }
           catch(...)
           {
           }
       }

       for (CORBA::ULong i = 0; i < all_processes_host2; i++)
       {
           try
           {
              current_process = processes_from_host2[i] -> name();
              current_application = processes_from_host2[i] -> get_parent()->name();
              admin_args = " " + current_application + " " + current_process + " " + host_name2;
              OsSupport::OS::create_process("platform_admin.sh", admin_args_process_stop + admin_args);
           }
           catch(...)
           {
           }
       }
 
       count = 0;

       while ((!observer_on_system.has_system_status(CdmwPlatformMngt::SYSTEM_STOPPED)))
       {
           if (end_timeout == count) { CPPUNIT_ASSERT(false);}
           sleep (1);
           count++;
           std::cerr << " waiting the system is STOPPED" << std::endl;
       }

       TEST_INFO(" check the system is SYSTEM_STOPPED ");
       CPPUNIT_ASSERT(check_status<CdmwPlatformMngt::System_ptr>(system.in(), CdmwPlatformMngt::SYSTEM_STOPPED));
       TEST_INFO(" check the application is APPLICATION_STOPPED");
       CPPUNIT_ASSERT(check_status<CdmwPlatformMngt::Application_ptr>(application1.in(), CdmwPlatformMngt::APPLICATION_STOPPED));

       sleep(3);
       // System Hot Restart SYSTEM_STARTED and APPLICATION_RUNNING
       TEST_INFO("System Hot Restart => SYSTEM_STARTED and APPLICATION_RUNNING");
       OsSupport::OS::create_process("platform_admin.sh","--sys-hot-restart");
 
       count = 0;
       while ((!observer_on_system.has_system_status(CdmwPlatformMngt::SYSTEM_STARTED)))
       {
           if (end_timeout == count) { CPPUNIT_ASSERT(false);}
           sleep (1);
           count++;
           std::cerr << " waiting the system is STARTED" << std::endl;
       }


       TEST_INFO(" check the system is SYSTEM_STARTED ");
       CPPUNIT_ASSERT(check_status<CdmwPlatformMngt::System_ptr>(system.in(), CdmwPlatformMngt::SYSTEM_STARTED));
       TEST_INFO(" check the application is APPLICATION_RUNNING ");
       CPPUNIT_ASSERT(check_status<CdmwPlatformMngt::Application_ptr>(application1.in(), CdmwPlatformMngt::APPLICATION_RUNNING));
   


       // Kill one process SYSTEM_PARTIAL_STARTED and APPLICATION_PARTIAL_RUNNING
       sleep(2);
       TEST_INFO("kill of a process => SYSTEM_PARTIAL_STARTED and APPLICATION_PARTIAL_RUNNING");
       try 
       {
           application_agent1->kill_process(process_name_host1.c_str(),15);
       }
       catch(const CdmwPlatformMngt::ProcessNotFound & ex)
       {  
           try 
           {
               application_agent2->kill_process(process_name_host1.c_str(),15);
           }
           catch(const CdmwPlatformMngt::ProcessNotFound & ex)
           {
              try
              {
                application_agent3->kill_process(process_name_host1.c_str(),15);
              }
              catch(const CdmwPlatformMngt::ProcessNotFound & ex)
              {
                  try
                  {
                      application_agent4->kill_process(process_name_host1.c_str(),15);
                  }
                  catch(const CdmwPlatformMngt::ProcessNotFound & ex)
                  {
                      CPPUNIT_ASSERT(false);
                  }
              }
           }
           std::cerr << ex<< std::endl;
       }
       catch(const CORBA::SystemException & ex)
       {
           std::cerr << ex << std::endl;
       }
      
       sleep(3);
       TEST_INFO(" check the system is SYSTEM_PARTIAL_STARTED ");
       CPPUNIT_ASSERT(check_status<CdmwPlatformMngt::System_ptr>(system.in(), CdmwPlatformMngt::SYSTEM_PARTIAL_STARTED));
       TEST_INFO(" check the application is APPLICATION_PARTIAL_RUNNING ");
       CPPUNIT_ASSERT(check_status<CdmwPlatformMngt::Application_ptr>(application1.in(), CdmwPlatformMngt::APPLICATION_PARTIAL_RUNNING));


       // KILL ALL PROCESSES => APPLICATION_STOPPED AND SYSTEM_STOPPED
       TEST_INFO(" KILL ALL PROCESSES => APPLICATION_PARTIAL_STARTED SYSTEM_PARTIAL_STARTED");
       for (CORBA::ULong i = 0; i < all_processes_host1; i++)
       {
           sleep (3);
           current_process = processes_from_host1[i] -> name();
           try
           {
           try
           { 
               application_agent1->kill_process(current_process.c_str(),15);
               std::cerr << "current_process " << current_process<< std::endl;
           }
           catch(const CdmwPlatformMngt::ProcessNotFound & ex)
           {
           }
           try
           {
               application_agent2->kill_process(current_process.c_str(),15);
               std::cerr << "current_process " << current_process<< std::endl;
           }
           catch(const CdmwPlatformMngt::ProcessNotFound & ex)
           {
           }

           try
           {
               application_agent3->kill_process(current_process.c_str(),15);
               std::cerr << "current_process " << current_process<< std::endl;
           }
           catch(const CdmwPlatformMngt::ProcessNotFound & ex)
           {
           }
           try
           {
               application_agent4->kill_process(current_process.c_str(),15);
               std::cerr << "current_process " << current_process<< std::endl;
           }
           catch(const CdmwPlatformMngt::ProcessNotFound & ex)
           {
           }
           }
           catch (...)
           {
                 CPPUNIT_ASSERT(false);
           }
      }

      for (CORBA::ULong i = 0; i < all_processes_host2; i++)
      {
           sleep(3);  
           current_process = processes_from_host2[i] -> name();
           try
           {
           try
           { 
               application_agent1->kill_process(current_process.c_str(),15);
               std::cerr << "current_process " << current_process<< std::endl;
           }
           catch(const CdmwPlatformMngt::ProcessNotFound & ex)
           {
           }
           try
           {
               application_agent2->kill_process(current_process.c_str(),15);
               std::cerr << "current_process " << current_process<< std::endl;
           }
           catch(const CdmwPlatformMngt::ProcessNotFound & ex)
           {
           }
           try
           {
               application_agent3->kill_process(current_process.c_str(),15);
               std::cerr << "current_process " << current_process<< std::endl;
           }
           catch(const CdmwPlatformMngt::ProcessNotFound & ex)
           {
           }
           try
           {
               application_agent4->kill_process(current_process.c_str(),15);
               std::cerr << "current_process " << current_process<< std::endl;
           }
           catch(const CdmwPlatformMngt::ProcessNotFound & ex)
           {
           }
           }
           catch (...)
           {
                 CPPUNIT_ASSERT(false);
           }
      }
/*
       count = 0;
       while ((!observer_on_system.has_system_status(CdmwPlatformMngt::SYSTEM_PARTIAL_STARTED)))
       {
           if (end_timeout == count) { CPPUNIT_ASSERT(false);}
           sleep (1);
           count++;
           std::cerr << " waiting the system is SYSTEM_PARTIAL_STARTED" << std::endl;
       }
*/

       TEST_INFO(" check the system is SYSTEM_PARTIAL_STARTED ");
       CPPUNIT_ASSERT(check_status<CdmwPlatformMngt::System_ptr>(system.in(), CdmwPlatformMngt::SYSTEM_PARTIAL_STARTED));
       TEST_INFO(" check the application is APPLICATION_PARTIAL_RUNNING");
       CPPUNIT_ASSERT(check_status<CdmwPlatformMngt::Application_ptr>(application1.in(), CdmwPlatformMngt::APPLICATION_PARTIAL_RUNNING));

       TEST_INFO("system hot-restart => SYSTEM_STARTED APPLICATION_RUNNING");
       OsSupport::OS::create_process("platform_admin.sh","--sys-hot-restart");

       count = 0;
       while ((!observer_on_system.has_system_status(CdmwPlatformMngt::SYSTEM_STARTED)))
       {
           if (end_timeout == count) { CPPUNIT_ASSERT(false);}
           sleep (1);
           count++;
           std::cerr << " waiting the system is STARTED" << std::endl;
       }


       TEST_INFO(" check the system is SYSTEM_STARTED ");
       CPPUNIT_ASSERT(check_status<CdmwPlatformMngt::System_ptr>(system.in(), CdmwPlatformMngt::SYSTEM_STARTED));
       TEST_INFO(" check the application is APPLICATION_RUNNING ");
       CPPUNIT_ASSERT(check_status<CdmwPlatformMngt::Application_ptr>(application1.in(), CdmwPlatformMngt::APPLICATION_RUNNING));

        
       // STOP ALL APPLICATIONS
       TEST_INFO("STOP ALL APPLICATIONS => SYSTEM_STOPPED APPLICATION_STOPPED");
       string app_command = "--app-stop ";
       string app_command1 = app_command + application1->name();  
       string app_command2 = app_command + application2->name();
       string app_command3 = app_command + application3->name();

       OsSupport::OS::create_process("platform_admin.sh", app_command1);
       OsSupport::OS::create_process("platform_admin.sh", app_command2);
       OsSupport::OS::create_process("platform_admin.sh", app_command3);

       count = 0;
       while ((!observer_on_system.has_system_status(CdmwPlatformMngt::SYSTEM_STOPPED)))
       {
           if (end_timeout == count) { CPPUNIT_ASSERT(false);}
           sleep (1);
           count++;
           std::cerr << " waiting the system is STOPPED" << std::endl;
       }


       TEST_INFO("check the applications are APPLICATION_STOPPED");
       CPPUNIT_ASSERT(check_status<CdmwPlatformMngt::Application_ptr>(application1.in(), CdmwPlatformMngt::APPLICATION_STOPPED));
       CPPUNIT_ASSERT(check_status<CdmwPlatformMngt::Application_ptr>(application1.in(), CdmwPlatformMngt::APPLICATION_STOPPED));

       CPPUNIT_ASSERT(check_status<CdmwPlatformMngt::Application_ptr>(application1.in(), CdmwPlatformMngt::APPLICATION_STOPPED));

       TEST_INFO(" check the system is SYSTEM_STOPPED ");
       CPPUNIT_ASSERT(check_status<CdmwPlatformMngt::System_ptr>(system.in(), CdmwPlatformMngt::SYSTEM_STOPPED));


       TEST_INFO("system hot-restart => SYSTEM_STARTED APPLICATION_RUNNING"); 
       OsSupport::OS::create_process("platform_admin.sh","--sys-hot-restart");
       sleep(3);
       count = 0;
       while ((!observer_on_system.has_system_status(CdmwPlatformMngt::SYSTEM_STARTED)))
       {
           if (end_timeout == count) { CPPUNIT_ASSERT(false);}
           sleep (1);
           count++;
           std::cerr << " waiting the system is STARTED" << std::endl;
       }


       TEST_INFO(" check the system is SYSTEM_STARTED ");
       CPPUNIT_ASSERT(check_status<CdmwPlatformMngt::System_ptr>(system.in(), CdmwPlatformMngt::SYSTEM_STARTED));
       TEST_INFO(" check the application is APPLICATION_RUNNING ");
       CPPUNIT_ASSERT(check_status<CdmwPlatformMngt::Application_ptr>(application1.in(), CdmwPlatformMngt::APPLICATION_RUNNING));


       // process stop => SYSTEM_PARTIAL_STARTED APPLICATION_PARTIAL_RUNNING
       TEST_INFO("process stop => SYSTEM_PARTIAL_STARTED APPLICATION_PARTIAL_RUNNING"); 
       admin_args = " " + application_name1 + " " + process_name_host2 + " " + host_name2;
       OsSupport::OS::create_process("platform_admin.sh", admin_args_process_stop + admin_args);
       count = 0;
       while ((!observer_on_system.has_system_status(CdmwPlatformMngt::SYSTEM_PARTIAL_STARTED)))
       {
           if (end_timeout == count) { CPPUNIT_ASSERT(false);}
           sleep (1);
           count++;
           std::cerr << " waiting the system is STARTED" << std::endl;
       }

/*       while (!process_observer_on_host2.has_status(CdmwPlatformMngt::PROCESS_STOPPED))
       {
           if (end_timeout == count) { CPPUNIT_ASSERT(false);}
           sleep (1);
           count++;
           std::cerr << " waiting the process is STOPPED" << std::endl;
       }
*/
       sleep(3);
       TEST_INFO(" check the system is SYSTEM_PARTIAL_STARTED ");
       CPPUNIT_ASSERT(check_status<CdmwPlatformMngt::System_ptr>(system.in(), CdmwPlatformMngt::SYSTEM_PARTIAL_STARTED));
       TEST_INFO(" check the application is APPLICATION_PARTIAL_RUNNING ");
       CPPUNIT_ASSERT(check_status<CdmwPlatformMngt::Application_ptr>(application1.in(), CdmwPlatformMngt::APPLICATION_PARTIAL_RUNNING));


       // remove the stopped process => SYSTEM_PARTIAL_STARTED APPLICATION_PARTIAL_RUNNING
       TEST_INFO("remove the stopped process => SYSTEM_STARTED APPLICATION_RUNNING");
 
       std::string admin_args_process_remove("--app-remove-process");
       OsSupport::OS::create_process("platform_admin.sh", admin_args_process_remove + admin_args);

       count = 0;
       while ((!observer_on_system.has_system_status(CdmwPlatformMngt::SYSTEM_STARTED)))
       {
           if (end_timeout == count) { CPPUNIT_ASSERT(false);}
           sleep (1);
           count++;
           std::cerr << " waiting the system is STARTED" << std::endl;
       }

       TEST_INFO(" check the system is SYSTEM_STARTED ");
       CPPUNIT_ASSERT(check_status<CdmwPlatformMngt::System_ptr>(system.in(), CdmwPlatformMngt::SYSTEM_STARTED));
       TEST_INFO(" check the application is APPLICATION_RUNNING ");
       CPPUNIT_ASSERT(check_status<CdmwPlatformMngt::Application_ptr>(application1.in(), CdmwPlatformMngt::APPLICATION_RUNNING));
 
         
       //TEST_INFO("Unregistering Observer process_observer_on_host1"); 
       //system->unregister_observer_ext ("process_observer_on_host1");
       system->unregister_observer ("process_observer_on_host1");
    }
    
}/* End namespace PlatformMngt */
}/* End namespace Cdmw */





