/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <fstream>
#include "test_status_consistency/TestObjs.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"



int returnError = -1;



class ORBWorker:public
  Cdmw::OsSupport::Thread
{
public:
  ORBWorker (CORBA::ORB_ptr orb);
  virtual ~
  ORBWorker ()
  throw ();
  virtual void
  run ()
  throw ();
private:
  CORBA::ORB_var
    m_ORB;
};

ORBWorker::ORBWorker (CORBA::ORB_ptr orb):m_ORB (CORBA::ORB::_duplicate (orb))
{
}


ORBWorker::~ORBWorker ()throw ()
{
  try
  {
    m_ORB->shutdown (1);
    m_ORB->destroy ();
    this->join ();
  }
  catch (const CORBA::SystemException & e)
  {
    std::cout << ":>> CORBA system exception caught: " << e._info ().
      rep () << std::endl;
  }
  catch (const std::exception & e)
  {
    std::cout << ":>> std exception caught: " << e.what () << std::endl;
  }
  catch (...)
  {
    std::cout << ":>> unexpected exception caught" << std::endl;
  }
}



void
ORBWorker::run ()
throw ()
{
  try
  {
    this->m_ORB->run ();
  }
  catch (const CORBA::SystemException & e)
  {
    std::cout << ":>> CORBA system exception caught: " << e._info ().
      rep () << std::endl;
  }
  catch (...)
  {
    std::cout << ":>> unexpected exception caught" << std::endl;
  }
}



int
main (int argc, char *argv[])
{
  using namespace Cdmw;
  using namespace Cdmw::OsSupport;
  using namespace Cdmw::PlatformMngt;


  /*
     ===================================================
     Initialize the ORB
     ===================================================
   */

  Cdmw::OrbSupport::StrategyList orb_strategies;
  orb_strategies.add_OrbThreaded ();
  //
  orb_strategies.add_PoaThreadPerConnection ();

  CORBA::ORB_var orb;

  orb = Cdmw::OrbSupport::OrbSupport::ORB_init (argc, argv, orb_strategies);
  CORBA::Object_var obj = orb->resolve_initial_references ("RootPOA");
  PortableServer::POA_var poa = PortableServer::POA::_narrow (obj.in ());

  PortableServer::POAManager_var poaMgr = poa->the_POAManager ();
  poaMgr->activate ();




  Cdmw::TestUtils::CORBATestManager::instance ()->init (orb.in (), poa.in (),
                            argc, argv);

  // create an ORBWorker object.
  ORBWorker orb_worker (orb.in ());
  orb_worker.start ();


  // wait for the ORBWorker thread to run the ORB.
  Cdmw::OsSupport::OS::sleep (1, 0);

  Cdmw::TestUtils::CORBATestManager::instance ()->run_tests ();
  Cdmw::TestUtils::CORBATestManager::instance ()->summary ();


  return (Cdmw::TestUtils::CORBATestManager::instance ()->was_successful ());
}


/*
int
main (int argc, char *argv[])
{
  using namespace Cdmw;
  using namespace Cdmw::OsSupport;
  using namespace Cdmw::PlatformMngt;


  // ===================================================
  // Initialize the ORB
  // ===================================================
  Cdmw::OrbSupport::StrategyList orb_strategies;
  orb_strategies.add_OrbThreaded ();
  orb_strategies.add_PoaThreadPool (10);

  CORBA::ORB_var orb;

  orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv,orb_strategies);

  Cdmw::TestUtils::CORBATestManager::instance()->init(orb.in(), argc,argv);

  // creates the process behaviour

  Cdmw::TestUtils::CORBATestManager::instance()->run_tests();
  Cdmw::TestUtils::CORBATestManager::instance()->summary();
  return Cdmw::TestUtils::CORBATestManager::instance()->was_successful();
}
*/
