/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @author Carlo Giovine  <cgiovine@sesm.it>
 */

#ifndef INCL_PLATFORMMNGT_SUPERVISION_OBSERVER_HPP
#define INCL_PLATFORMMNGT_SUPERVISION_OBSERVER_HPP

#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/common/System.hpp"
#include "SystemMngt/platformvaluetypes/SystemModeChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/SystemStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/HostStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationAgentDeath_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationModeChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/EntityStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/PropertyChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/UserEvent_impl.hpp"
#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"

namespace Cdmw
  {        // Begin namespace Cdmw

  namespace PlatformMngt
    { // Begin namespace PlatformMngt
        
    /**
    *Purpose:
    *<p> This struct is used to record the states reached by an application,
    * and when it happens.
    */
    struct Record
    {
        bool state;
        OsSupport::OS::Timeval time;
    };
    
    //This is the map of all record of a single application
    typedef std::map <std::string, Record* > StateList; 
    
    
    class SysObserver_impl: public virtual POA_CdmwPlatformMngt::SupervisionObserver,
          public virtual PortableServer::RefCountServantBase
      {

      public:
        
        //Constructor
        SysObserver_impl(const CdmwPlatformMngt::Applications& apps,
                         const CORBA::ULong& nbApps); 
        
        //Destructor                 
        ~SysObserver_impl();
         
        //the notify function
        void notify( CdmwPlatformMngt::Event* an_event )
            throw( CORBA::SystemException );
        
        /**
        * Get the timestamp when the status has been reached.
        */
        OsSupport::OS::Timeval getInformation(const std::string appname, const std::string state);
   
        /**
        * is the system Started ?? .
        */
        bool isStarted();

      private:
          
          /*
           * This function sets the information into the record.
           */     
          void setInformation(StateList* sl, const std::string& stateName);
      
      private:
          
          /**
          * The Database of all the applications information.
          */
          std::map <std::string, StateList* > Database;
          
          /**
          * Indicate if the system has completed its start
          */
          bool started;
          
      };

  } // End namespace PlatformMngt
} // End namespace Cdmw

#endif //INCL_PLATFORMMNGT_SUPERVISION_OBSERVER_HPP

