/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_TESTTIMEOUTAPPLICATION_HPP
#define INCL_PLATFORMMNGT_TESTTIMEOUTAPPLICATION_HPP

#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/common/System.hpp"
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/Exception.hpp>
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"

#include "test_timeout_applications/SysObserver_impl.hpp"

namespace Cdmw
{
  namespace PlatformMngt
  {
    
    typedef struct Fields 
    {
        unsigned long timeout;
        std::list<std::string> Predecessors;
    };
    
    class TestTimeoutApplication : public CppUnit::TestFixture
    {

    public:

        TestTimeoutApplication();
        
        void setUp ();
 
        virtual void test_start_application ();

             
    private:
    
        //System reference
        CdmwPlatformMngt::System_var system ;
        CdmwPlatformMngt::SupervisionObserver_var observer;
        std::string obName ;
        
        //Application references
        CdmwPlatformMngt::ApplicationsIterator_var appIter;
        CdmwPlatformMngt::Applications_var apps;
        CORBA::ULong nbApps;
        
        //Process references
        CdmwPlatformMngt::ProcessesIterator_var procIter;
        CdmwPlatformMngt::Processes_var procs;
        CORBA::ULong nbProcs;
        
        CdmwPlatformMngtBase::StartupKind_var startupKind;
        CdmwPlatformMngtBase::StateRetrieval state_retrieval;
        
        CORBA::String_var statusInfo;
        CORBA::ORB_var orb;
        
        //Startupkind parameter
        int val;
        unsigned long startup_mode;
        std::string specified_state_id;

        //test parameter
        unsigned long deviation;
        int sleeping;
        std::map <std::string, Fields > applicationFields;
    
        
    private:
    
        /**
         * Register the observer to the system
         */
        SysObserver_impl* register_observer () throw ();
        
        /**
         * Unregister the observer
         */
        void unregister_observer (SysObserver_impl* sysOb) throw ();
        
        /**
         * Get all the application Field from the main
         */
        void TestTimeoutApplication::parser(std::map <std::string, Fields >& applicationFields);        
        
        /**
         * Assignment operator
         */
        TestTimeoutApplication & operator= (const TestTimeoutApplication & rhs) throw ();
        
        /**
         * Copy constructor
         */
        TestTimeoutApplication (const TestTimeoutApplication & rhs) throw ();

        /**
         * Defining a Test Suite
         */       
        CPPUNIT_TEST_SUITE (TestTimeoutApplication);
        
            //These are the test cases
            CPPUNIT_TEST (test_start_application); 
            
        
        CPPUNIT_TEST_SUITE_END ();
    };

  }             /* End namespace PlatformMngt */
}               /* End namespace Cdmw */
#endif              /* INCL_PLATFORMMNGT_TESTTIMEOUTAPPLICATION_HPP */
