/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "Foundation/ossupport/OS.hpp"
#include "Foundation/osthreads/ThreadHandle.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include <Foundation/logging/LogManager.hpp> 
#include <Foundation/logging/FilterMngr.hpp>

#include "SystemMngt/platforminterface/PlatformInterface.hpp"
#include "SystemMngt/platforminterface/ServiceNames.hpp"

#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include "Repository/repositoryinterface/RepositoryInterface.hpp"

#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"

#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/LogManager.hpp>

#include "test_tracing_admin/MyProcessBehaviour.hpp"

#include <cstdlib>
#include <string>
#include <exception>
#include <sstream>
#include <iostream>
#include <fstream>

#undef  CDMW_TRACE_LEVEL
#define CDMW_TRACE_LEVEL 6

const int POA_THREAD_POOL_SIZE = 5;

using Cdmw::NamingAndRepository::RepositoryInterface;
using Cdmw::Logging::Logger;
using Cdmw::Logging::LogManager;

using namespace Cdmw::OsSupport;
using namespace Cdmw::OrbSupport;
using namespace Cdmw::PlatformMngt;

std::string applicationName;
std::string processName;
std::string hostName ;

int main(int argc, char* argv[])
{
    // the return code 
    StrategyList strategy;
    strategy.add_OrbThreaded();
    strategy.add_PoaThreadPool(POA_THREAD_POOL_SIZE);

    CORBA::ORB_var orb;


    try
    {
                
        // initialise the orb    
        orb =  Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, strategy);

        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");

        PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj.in());

        PortableServer::POAManager_var poaMgr = rootPOA->the_POAManager();

        poaMgr->activate();

        std::cout << "Init PlatformInterface" << std::endl;

        // initialise the platform interface
        PlatformInterface::Setup(orb.in(), argc, argv);

        // creates the process behaviour
        std::auto_ptr<MyProcessBehaviour> procBehaviour( new MyProcessBehaviour(orb.in() ));
        // get processId
        OS::ProcessId processId = OS::get_processId();

        // acknowledge the creation of the process
        PlatformInterface::Acknowledge_creation(procBehaviour.get());

        MyProcessBehaviour * pProcBehaviour = procBehaviour.release();

        applicationName = PlatformInterface::Get_application_name();

        processName = PlatformInterface::Get_process_name();


        std::cout << "Application name : "
                      << applicationName.c_str() << std::endl;

        std::cout << "Process name : "
                      << processName.c_str()
                      << " Process id : "
                      << processId << std::endl;


        //resolve system ptr
        CORBA::Object_var objref2; 

        /*define the corbaloc */
        const char *corbaloc_str = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";

        objref2 = orb->string_to_object (corbaloc_str);

        /* resolve the System */
        CdmwPlatformMngt::System_var mysystem = CdmwPlatformMngt::System::_narrow (objref2.in ());

        if (CORBA::is_nil (mysystem.in ()))
          {
            std::cout<<"Error in narrowing CdmwPlatformMngt::System"<<std::endl;
            exit (-1);
          }
 
        CdmwPlatformMngt::Application_var app = mysystem->get_application ("HELLO_WORLD");

        std::string proc_name = "TraceClient";
        char *hostname = getenv ("HOSTNAME");
        CdmwPlatformMngt::Process_var proc = app->get_process (proc_name.c_str (), hostname);

        if (CORBA::is_nil (proc.in ()))
          {
           std::cout<<"*******ERROR in proc_name, is nil *************"<<std::endl;
           exit (-1);
          }


        // advise behavior that init is done
        pProcBehaviour->set_init_done();
        pProcBehaviour->get_condition().broadcast();

        // start orb
        orb->run();

        // cleanup the platform interface
        PlatformInterface::Cleanup();


       }
       catch(...)
        {
        std::cout<<"*****ERROR : reached exception in managed process *****"<<std::endl;
            // cleanup the ORB to exit completely
            orb->shutdown(false);
                    Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
                    orb->destroy();
            
        return(-1);
        }



    return 0;

}

