/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include "Foundation/common/Options.hpp"
#include <Foundation/common/Locations.hpp>
#include "Foundation/common/String.hpp"
#include "Foundation/common/Exception.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/common/Locations.hpp"

#include "Foundation/orbsupport/CORBA.hpp"

#include "Foundation/ossupport/OS.hpp"
#include "Foundation/osthreads/ThreadHandle.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/FilterMngr.hpp>

#include "SystemMngt/platforminterface/PlatformInterface.hpp"
#include "SystemMngt/platforminterface/ServiceNames.hpp"

#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include "Repository/repositoryinterface/RepositoryInterface.hpp"

#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"
#include "idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"

#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/LogManager.hpp>

#include <ConfAndPlug/cdmwinit/CdmwInterface.hpp>
#include <ConfAndPlug/cdmwinit/InitUtils.hpp>

#include "test_tracing_api/MyProcessBehaviour.hpp"

#include <cstdlib>
#include <string>
#include <exception>
#include <sstream>
#include <iostream>
#include <fstream>

/* This is a main for a validation test : in validation conditions
   we expect to be compiled with CDMW_TRACE_LEVEL 6 if nothing else
   is stated
#if defined(CDMW_TRACE_LEVEL)

#if CDMW_TRACE_LEVEL < 6
#undef CDMW_TRACE_LEVEL
#define CDMW_TRACE_LEVEL 6
#endif

#else
#define CDMW_TRACE_LEVEL 6
#endif
*/


const int POA_THREAD_POOL_SIZE = 5;

namespace
{

using Cdmw::NamingAndRepository::RepositoryInterface;
using Cdmw::Logging::Logger;
using Cdmw::Logging::LogManager;

using namespace Cdmw::OsSupport;

using namespace Cdmw::OrbSupport;

using namespace Cdmw::PlatformMngt;

const int SUCCESS = 0;
const int FAILURE = 1;

std::string application_name;
std::string process_name;
std::string hostName ;
}

int main ( int argc, char* argv[] )
{
    // the return code
    StrategyList strategy;
    strategy.add_OrbThreaded();
    strategy.add_PoaThreadPool ( POA_THREAD_POOL_SIZE );

    Logger* logger = NULL;

    CORBA::ORB_var orb;

    // the return code
    int ret_code = 0;

    try
    {
        // initialise the orb
        orb =  Cdmw::OrbSupport::OrbSupport::ORB_init ( argc, argv, strategy );

        // get the root POA.
        CORBA::Object_var obj = orb->resolve_initial_references ( "RootPOA" );
        PortableServer::POA_var rootPOA = PortableServer::POA::_narrow ( obj.in() );

        // activate the root POA manager.
        PortableServer::POAManager_var poaMgr = rootPOA->the_POAManager();
        poaMgr->activate();

        std::cout << "Init PlatformInterface" << std::endl;

        // initialise the platform interface
        PlatformInterface::Setup ( orb.in(), argc, argv );


        logger = Logger::Get_Logger ( "TEST", "TEST" );

        std::string fooStr ( "" );

        // creates the process behaviour
        std::auto_ptr<MyProcessBehaviour> procBehaviour ( new MyProcessBehaviour ( orb.in(),
                fooStr,
                logger ) );
                
        // get processId
        OS::ProcessId processId = OS::get_processId();

        // acknowledge the creation of the process
        PlatformInterface::Acknowledge_creation ( procBehaviour.get() );

        MyProcessBehaviour * pProcBehaviour = procBehaviour.release();

        application_name = PlatformInterface::Get_application_name();

        process_name = PlatformInterface::Get_process_name();


        // register a null fatal handler to avoid the process to be killed by the supervision
        // following a call to a CDMW_FATAL message. Do not delete the return handler since
        // it will be deleted by CDMW_cleanup().
        ( void* ) Cdmw::Logging::LogManager::Instance()->register_fatal_handler ( 0 );

        std::cout << "Application name : "
                  << application_name.c_str() << std::endl;

        std::cout << "Process name : "
                  << process_name.c_str()
                  << " Process id : "
                  << processId << std::endl;


        std::string baseMsg = "process <" + application_name + "/" + process_name + ">" ;

        // ===================================================
        // trace a message (don't use CDMW_TRACE in main)
        // ===================================================

        std::cout << "program started and waiting CORBA messages" << std::endl;

        std::string msg = baseMsg + "started and waiting CORBA messages";

        CDMW_LOG ( logger, Cdmw::Logging::INFO, msg.c_str() );

        //resolve system ptr
        CORBA::Object_var objref2;

        //define the corbaloc
        const char *corbaloc_str = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";

        objref2 = orb->string_to_object ( corbaloc_str );

        // resolve the System
        CdmwPlatformMngt::System_var mysystem = CdmwPlatformMngt::System::_narrow ( objref2.in () );

        if ( CORBA::is_nil ( mysystem.in () ) )
        {
            std::cout << "Error in narrowing CdmwPlatformMngt::System" << std::endl;
            exit ( -1 );
        }

        CdmwPlatformMngt::Application_var app = mysystem->get_application ( "HELLO_WORLD" );

        std::string proc_name = "TraceClient";
        char *hostname = getenv ( "HOSTNAME" );
        CdmwPlatformMngt::Process_var proc = app->get_process ( proc_name.c_str (), hostname );

        if ( CORBA::is_nil ( proc.in () ) )
        {
            std::cout << "*******ERROR in proc_name, is nil *************" << std::endl;
            exit ( -1 );
        }

        if ( CORBA::is_nil ( proc.in () ) )
        {
            std::cout << "*******ERROR in proc_name, is nil *************" << std::endl;
            exit ( -1 );
        }

        //try a getting of filters
        CORBA::Any_var myany = new CORBA::Any();

        proc -> get_attribute ( CdmwPlatformMngt::TRACE_FILTERS, myany.out() , CdmwPlatformMngt::PRE_DEFINED );

        const char * tmp;

        myany.in()  >>= tmp;

        std::string init_filters ( tmp );

        if ( init_filters.compare ( "TraceClient,HELLO_WORLD,0x80" ) == 0 )
        {
            std::cout << "TEST NUM. S1 -->  OK: scenario filters =" << init_filters << std::endl;
        }
        else
        {
            std::cout << "TEST NUM. S1 -->  Error: scenario filters =" << init_filters << std::endl;
        }

        //try a parsing exception
        char flag_test = 0;

        try
        {
            myany <<= ",,3";
            proc -> set_attribute ( CdmwPlatformMngt::TRACE_FILTERS, myany.in() , CdmwPlatformMngt::PRE_DEFINED );
        }
        catch ( const CORBA::BAD_PARAM& )
        {
            std::cout << "TEST NUM. E1 -->  OK: reached a CORBA::BAD_PARAM exception " << std::endl;
            flag_test = 1 ;
        }

        if ( flag_test == 0 )
            std::cout << "TEST NUM. E1 -->  Error : not reached a CORBA::BAD_PARAM exception " << std::endl;


        //try a parsing exception
        flag_test = 0;

        try
        {
            myany <<= "TraceClient,HELLO_WORLD,0x";
            proc -> set_attribute ( CdmwPlatformMngt::TRACE_FILTERS, myany.in() , CdmwPlatformMngt::PRE_DEFINED );
        }
        catch ( const CORBA::BAD_PARAM& )
        {
            std::cout << "TEST NUM. E2 -->  OK: reached a CORBA::BAD_PARAM exception " << std::endl;
            flag_test = 1 ;
        }

        if ( flag_test == 0 )
            std::cout << "TEST NUM. E2 -->  Error : not reached a CORBA::BAD_PARAM exception " << std::endl;

        //try a parsing exception
        flag_test = 0;

        try
        {
            myany <<= "TraceClient,HELLO_WORLD,7,HELLO";
            proc -> set_attribute ( CdmwPlatformMngt::TRACE_FILTERS, myany.in() , CdmwPlatformMngt::PRE_DEFINED );
        }
        catch ( const CORBA::BAD_PARAM& )
        {
            std::cout << "TEST NUM. E3 -->  OK: reached a CORBA::BAD_PARAM exception " << std::endl;
            flag_test = 1 ;
        }

        if ( flag_test == 0 )
            std::cout << "TEST NUM. E3 -->  Error : not reached a CORBA::BAD_PARAM exception " << std::endl;


        //try a parsing exception
        flag_test = 0;

        try
        {
            myany <<= ",HELLO.WORLD,0x0000000A";
            proc -> set_attribute ( CdmwPlatformMngt::TRACE_FILTERS, myany.in() , CdmwPlatformMngt::PRE_DEFINED );
        }
        catch ( const CORBA::BAD_PARAM& )
        {
            std::cout << "TEST NUM. E4 -->  OK: reached a CORBA::BAD_PARAM exception " << std::endl;
            flag_test = 1 ;
        }

        if ( flag_test == 0 )
            std::cout << "TEST NUM. E4 -->  Error : not reached a CORBA::BAD_PARAM exception " << std::endl;


        //try a parsing exception
        flag_test = 0;

        try
        {
            myany <<= "TraceClient,HELLO_WORLD,12adasdasda";
            proc -> set_attribute ( CdmwPlatformMngt::TRACE_FILTERS, myany.in() , CdmwPlatformMngt::PRE_DEFINED );
        }
        catch ( const CORBA::BAD_PARAM& )
        {
            std::cout << "TEST NUM. E5 -->  OK: reached a CORBA::BAD_PARAM exception " << std::endl;
            flag_test = 1 ;
        }

        if ( flag_test == 0 )
            std::cout << "TEST NUM. E5 -->  Error : not reached a CORBA::BAD_PARAM exception " << std::endl;


        //test: adding a setting of filters ...
        flag_test = 0;

        try
        {
            myany <<= "TraceClient,HELLO_WORLD,0x1";
            proc -> set_attribute ( CdmwPlatformMngt::TRACE_FILTERS, myany.in() , CdmwPlatformMngt::PRE_DEFINED );
        }
        catch ( ... )
        {
            std::cout << "TEST NUM. S2 -->  Error : Failed to set trace filters to managed process" << std::endl;
            flag_test = 1;
        }

        if ( flag_test == 0 )
            std::cout << "TEST NUM. S2 -->  OK : setting of other trace filters to the managed process" << std::endl;

        //test: getting of these new filters ...
        proc -> get_attribute ( CdmwPlatformMngt::TRACE_FILTERS, myany.out() , CdmwPlatformMngt::PRE_DEFINED );

        myany.in()  >>= tmp;

        std::string new_filters_g1 ( tmp );

        if ( new_filters_g1.compare ( "TraceClient,HELLO_WORLD,0/TraceClient,HELLO_WORLD,7" ) == 0 )
        {
            std::cout << "TEST NUM. G1 -->  OK : getting of actual (in managed process) filters=" << new_filters_g1 << std::endl;
        }
        else
        {
            std::cout << "TEST NUM. G1 -->  Error : getting of actual (in managed process) filters=" << new_filters_g1 << std::endl;
        }

        //test: adding a setting of filters ...
        flag_test = 0;

        try
        {
            myany <<= "TraceClient,HELLO_WORLD,0x40000000";
            proc -> set_attribute ( CdmwPlatformMngt::TRACE_FILTERS, myany.in() , CdmwPlatformMngt::PRE_DEFINED );
        }
        catch ( ... )
        {
            std::cout << "TEST NUM. S3 -->  Error : Failed to set trace filters to managed process" << std::endl;
            flag_test = 1;
        }

        if ( flag_test == 0 )
            std::cout << "TEST NUM. S3 -->  OK : setting of other trace filters to the managed process" << std::endl;

        //test: getting of these new filters ...
        proc -> get_attribute ( CdmwPlatformMngt::TRACE_FILTERS, myany.out() , CdmwPlatformMngt::PRE_DEFINED );

        myany.in()  >>= tmp;

        std::string new_filters_g2 ( tmp );

        if ( new_filters_g2.compare ( "TraceClient,HELLO_WORLD,0/" \
                                      "TraceClient,HELLO_WORLD,7/" \
                                      "TraceClient,HELLO_WORLD,30" ) == 0 )
        {
            std::cout << "TEST NUM. G2 -->  OK : getting of actual (in managed process) filters=" << new_filters_g2 << std::endl;
        }
        else
        {
            std::cout << "TEST NUM. G2 -->  Error : getting of actual (in managed process) filters=" << new_filters_g2 << std::endl;
        }


        //retrieve an unmanaged process reference to test

        flag_test = 0;

        std::string proc_name_unmanaged = "HelloProcess1";

        CdmwPlatformMngt::Process_var proc_unmanaged = app->get_process ( proc_name_unmanaged.c_str (), hostname );

        try
        {
            myany <<=  "TraceClient,HELLO_WORLD,9/TraceClient,HELLO_WORLD,12" ;
            proc_unmanaged -> set_attribute ( CdmwPlatformMngt::TRACE_FILTERS, myany.in() , CdmwPlatformMngt::PRE_DEFINED );

        }
        catch ( ... )
        {
            std::cout << "TEST NUM. U1 -->  Error: reached unexpeted exception of trace filters setting to an unmanaged process " << std::endl;
            flag_test = 1 ;
        }

        if ( flag_test == 0 )
        {
            std::cout << "TEST NUM. U1 -->  OK: setting (without effect) of trace filters to an unmanaged process " << std::endl;
        }

        //test: adding a setting of filters ...
        flag_test = 0;

        try
        {
            myany <<= "TraceClient,HELLO_WORLD,9/TraceClient,HELLO_WORLD,12" ;
            proc -> set_attribute ( CdmwPlatformMngt::TRACE_FILTERS, myany.in() , CdmwPlatformMngt::PRE_DEFINED );

        }
        catch ( ... )
        {
            std::cout << "TEST NUM. S4 -->  Error : Failed to set trace filters to managed process" << std::endl;
            flag_test = 1;
        }

        if ( flag_test == 0 )
            std::cout << "TEST NUM. S4 -->  OK : setting of other trace filters to the managed process" << std::endl;

        //test: getting of these new filters ...
        proc -> get_attribute ( CdmwPlatformMngt::TRACE_FILTERS, myany.out() , CdmwPlatformMngt::PRE_DEFINED );

        myany.in()  >>= tmp;

        std::string new_filters_g3 ( tmp );


        if ( new_filters_g3.compare ( "TraceClient,HELLO_WORLD,0/" \
                                      "TraceClient,HELLO_WORLD,7/" \
                                      "TraceClient,HELLO_WORLD,9/" \
                                      "TraceClient,HELLO_WORLD,12/" \
                                      "TraceClient,HELLO_WORLD,30" ) == 0 )
        {
            std::cout << "TEST NUM. G3 -->  OK : getting of actual (in managed process) filters=" << new_filters_g3 << std::endl;
        }
        else
        {
            std::cout << "TEST NUM. G3 -->  Error : getting of actual (in managed process) filters=" << new_filters_g3 << std::endl;
        }

        //test: setting of filters at host level ...
        flag_test = 0;

        CdmwPlatformMngt::Host_var myhost = mysystem->get_host ( hostname );

        try
        {
            myany <<= "TraceClient,HELLO_WORLD,14/" ;
            myhost -> set_attribute ( CdmwPlatformMngt::TRACE_FILTERS, myany.in() , CdmwPlatformMngt::PRE_DEFINED );
        }
        catch ( const CdmwPlatformMngt::CastException& e )
        {
            std::cout << "REACHED CAST EXCEPTION" << std::endl;
        }
        catch ( const CdmwPlatformMngt::AttributeNotFound& e )
        {
            std::cout << "REACHED ATTR. NOT FOUND EXCEPTION" << std::endl;
        }
        catch ( ... )
        {
            std::cout << "TEST NUM. H1 --> Error : setting of other trace filters at host level" << std::endl;
            flag_test = 1 ;
        }

        if ( flag_test == 0 )
        {
            std::cout << "TEST NUM. H1 --> OK : setting of other trace filters at host level" << std::endl;
        }


        // advise behavior that init is done
        pProcBehaviour->set_init_done();

        pProcBehaviour->get_condition().broadcast();

        // start orb
        orb->run();

        // cleanup the platform interface
        PlatformInterface::Cleanup();

    }

    catch ( const Cdmw::Exception & ex )
    {
        std::cerr << ex.what() << std::endl;
        ret_code = FAILURE;
    }
    catch ( const CORBA::Exception& e )
    {
        std::cerr << "FAILURE : CORBA exception raised :" << std::endl
                  << e._name() << std::endl;
        ret_code = FAILURE;
    }
    catch ( const std::exception &e )
    {
        std::cerr << "FAILURE : Exception raised : " << std::endl
                  << e.what() << std::endl;
        ret_code = FAILURE;
    }


    if ( !CORBA::is_nil ( orb.in() ) )
    {
        try
        {
            Cdmw::OrbSupport::OrbSupport::ORB_cleanup ( orb.in() );
            orb->destroy();
        }
        catch ( const CORBA::Exception& ex )
        {
            std::cerr << ex << std::endl;
            ret_code = FAILURE;
        }
    }

    return ret_code;
}

