/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <string>
#include <cctype>
#include <algorithm>
#include <sstream>
#include "Foundation/common/Locations.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/common/Exception.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/logging/LogManager.hpp"
#include "test_tracing_api/TraceClientThread.hpp"





TraceClientThread::TraceClientThread(CORBA::ORB_ptr orb, 
                           std::string& applicationName,
                           std::string& processName,
                           std::string& traceMessage,
                           Cdmw::Logging::Logger* logger)
    throw (CORBA::SystemException)
    : m_orb(CORBA::ORB::_duplicate(orb)),
      m_applicationName (applicationName),
      m_processName (processName),
      m_traceMessage (traceMessage),
      m_exit_required(false),
      m_logger (logger)

{
    // create loggers (LogManager has been initialised in ORB_init)
    m_logger1 = Cdmw::Logging::Logger::Get_Logger("COMP_A", "DOMAIN-1");
    m_logger2 = Cdmw::Logging::Logger::Get_Logger("COMP_B", "DOMAIN-1");
    m_logger3 = Cdmw::Logging::Logger::Get_Logger("COMP_A", "DOMAIN-2");
    m_logger4 = Cdmw::Logging::Logger::Get_Logger("COMP_B", "DOMAIN-2");
    m_logger5 = Cdmw::Logging::Logger::Get_Logger("DOMAIN-1");
    m_logger6 = Cdmw::Logging::Logger::Get_Logger("DOMAIN-2");
}



TraceClientThread::~TraceClientThread() throw()
{
}

 

void TraceClientThread::run () throw()
{
    using namespace Cdmw;
    using namespace Cdmw::Logging;

    try
    {        
        std::cout << "Trace Client Thread Started" << std::endl;        
        
        std::string baseMsg = "process <" + m_applicationName + "/" + m_processName + ">" ;
        std::string msg;
        long count = 0;
        long msg_count = 0;
        long m_traceCount = -1;
        
        // ===================================================
        // trace a message (use CDMW_TRACE in Cardamom thread)
        // ===================================================
        msg = baseMsg + "Client Thread started";
        
#if defined  (CDMW_TRACE_LEVEL) && (CDMW_TRACE_LEVEL == 0)
        std::cout << "Sending a message to the Trace collector." << std::endl;
#endif

        CDMW_LOG (m_logger, Logging::INFO, msg.c_str());


        // ====================================================
        // thread loop
        // ====================================================
        
        msg = baseMsg + m_traceMessage;
                
        // get the filter manager
        FilterMngr* p_filter_mngr = 
                   LogManager::Instance()->get_filter_manager();
                   

            p_filter_mngr->activate_all_levels(Logging::ALL_COMPONENT_NAMES,
                                               Logging::ALL_DOMAINS);

        // trace message or probe call main loop
        while (!m_exit_required) 
        {
            int log_msg = 0;
            
            // if infinite loop for message
            if (m_traceCount == -1)
            {
                log_msg = 1; std::cout << " Msg Count = "<< msg_count << std::endl;
            }
            
            // else if max number of messages not reached
            else if (count < m_traceCount)
            {
                count++;
                log_msg = 1; 
            }
            
            // else if end of loop reached
            else if (count == m_traceCount && m_traceCount != 0)
            {
                count++;
                log_msg = 2; 
            }
                        
            // if message to trace or probe to call
            if (log_msg == 1)
            {

#if defined  (CDMW_TRACE_LEVEL) && (CDMW_TRACE_LEVEL == 0)
                std::cout << "Sending a message to the Trace collector." << std::endl;
#endif

                CDMW_LOG (m_logger1, Cdmw::Logging::FATAL, msg.c_str() );   // COMP_A,DOMAIN-1,0x00000001
                CDMW_LOG (m_logger1, Cdmw::Logging::ERROR, msg.c_str() );   // COMP_A,DOMAIN-1,0x00000002
                CDMW_LOG (m_logger3, Cdmw::Logging::ERROR, msg.c_str() );   // COMP_A,DOMAIN-2,0x00000002
                CDMW_LOG (m_logger2, Cdmw::Logging::WARN, msg.c_str() );    // COMP_B,DOMAIN-1,0x00000004
                CDMW_LOG (m_logger4, Cdmw::Logging::INFO, msg.c_str() );    // COMP_B,DOMAIN-2,0x00000008
                CDMW_LOG (m_logger5, Cdmw::Logging::FATAL, msg.c_str() );   // !,DOMAIN-1,0x00000001
                CDMW_LOG (m_logger5, Cdmw::Logging::ERROR, msg.c_str() );   // !,DOMAIN-1,0x00000002
                CDMW_LOG (m_logger5, Cdmw::Logging::WARN, msg.c_str() );    // !,DOMAIN-1,0x00000004
                CDMW_LOG (m_logger6, Cdmw::Logging::ERROR, msg.c_str() );   // !,DOMAIN-2,0x00000002
                CDMW_LOG (m_logger6, Cdmw::Logging::INFO, msg.c_str() );    // !,DOMAIN-2,0x00000008

            }
            // else if request of end message
            else if (log_msg == 2)
            {
                count++;                
                std::string msg2 = baseMsg + "!!!! Trace terminated !!!!";  
                CDMW_LOG (m_logger, Cdmw::Logging::INFO, msg2.c_str());
                std::cout << "Trace terminated : System can be stopped !!!!!!" << std::endl;
            }
            // ===================================================
            // set thread sleeping
            // ===================================================
            Cdmw::OsSupport::OS::sleep (1000);
        
/*
            if (msg_count == 0x7fffffff)
            {
                msg_count = 0;
            }
*/
            if (msg_count == 2)
            {
                m_exit_required = true;
            }

            msg_count ++;
        }
        stop ();
    }
    catch (...)
    {
        std::cout << "File : " << __FILE__ << " Line : " << __LINE__ 
                  << "Unexpected exception in thread ..." << std::endl;
    }
}

