/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.platformmngt.test;

import cdmw.common.Options;
import cdmw.common.BadOrderException;
import com.thalesgroup.CdmwPlatformMngt.*;
import cdmw.platformmngt.*;
import cdmw.orbsupport.StrategyList;

import cdmw.platformmngt.PlatformInterface;
import com.thalesgroup.*;
import cdmw.orbsupport.*;
import org.omg.PortableServer.POAHelper;

/**
 * The main test.
 *
 */
public class ManagedTraceProgram {

    static int SUCCESS = 0;
    static int FAILURE = 1;

    static int POA_THREAD_POOL_SIZE = 5;

    public static void main(String[] args) {
        int retCode = SUCCESS;
        org.omg.CORBA.ORB orb = null;

        try {
            StrategyList orbStrategies = new StrategyList();
            orbStrategies.addORBThreaded();
            orbStrategies.addPOAThreadPool(POA_THREAD_POOL_SIZE);
        
            orb = ORBSupport.ORBInit(args, orbStrategies);

            // ===================================================
            // Get the root POA 
            // ===================================================
            org.omg.CORBA.Object obj = orb.resolve_initial_references("RootPOA");
            org.omg.PortableServer.POA rootPOA = POAHelper.narrow(obj);
                
            // ===================================================
            // Activate the root POA manager
            // ===================================================
            org.omg.PortableServer.POAManager poaManager = rootPOA.the_POAManager();

            poaManager.activate();

            java.lang.System.out.println("Init PlatformInterface");

            // setup platforminterface
            PlatformInterface.setup(orb,args);

            // creates the process behaviour    
            UserProcessBehaviour myProcess = new UserProcessBehaviour(orb);

            // acknowledge the creation of the process  
            PlatformInterface.acknowledgeCreation(myProcess);
 
            String applicationName = PlatformInterface.getApplicationName();

            String processName = PlatformInterface.getProcessName();

            java.lang.System.out.println("Application name : " + applicationName);
            
            java.lang.System.out.println("Process name : " + processName);

            // resolve system ref
            org.omg.CORBA.Object objref2;

            /* define the corbaloc */
            String corbalocStr = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";

            objref2 = orb.string_to_object(corbalocStr);
       
            /* resolve the System */
            com.thalesgroup.CdmwPlatformMngt.System mySystem = com.thalesgroup.CdmwPlatformMngt.SystemHelper.narrow(objref2);
 
            if (mySystem == null) {
                throw new Exception("Error in narrowing CdmwPlatformMngt::System");
            }

            Application app = mySystem.get_application("HELLO_WORLD");
  
            String procName = "TraceClient";
            String hostname = java.lang.System.getenv("HOSTNAME");
  
            com.thalesgroup.CdmwPlatformMngt.Process proc = app.get_process(procName, hostname);
 
            if (proc == null){
                throw new Exception("*******ERROR in proc_name, is nil *************");
            }

            //try a getting of filters
            org.omg.CORBA.Any myAny = org.omg.CORBA.ORB.init().create_any();
 
            org.omg.CORBA.AnyHolder myAnyHolder = new org.omg.CORBA.AnyHolder(myAny);
 
            proc.get_attribute(TRACE_FILTERS.value, myAnyHolder, Owner.PRE_DEFINED);
 
            String initFilters = myAnyHolder.value.extract_string();  
            
            if (initFilters.compareTo("TraceClient,HELLO_WORLD,7/") == 0) {
                java.lang.System.out.println("TEST NUM. 1 -->  OK: scenario filters ="+initFilters);
            } else {
                java.lang.System.out.println("TEST NUM. 1 -->  Error: scenario filters ="+initFilters);
            }
            
            //try a parsing exception
            char flagTest2 = 0;
            try {
                myAny.insert_string(",,3");
                proc.set_attribute(TRACE_FILTERS.value, myAny, Owner.PRE_DEFINED);
            } catch(org.omg.CORBA.BAD_PARAM bd) {
                java.lang.System.out.println("TEST NUM. 2 -->  OK: reached a CORBA::BAD_PARAM exception");
                flagTest2 = 1; 
            }

            if (flagTest2 == 0) {
                java.lang.System.out.println("TEST NUM. 2 -->  Error: not reached a CORBA::BAD_PARAM exception ");
            }
    
            // retrieve an unmanaged process reference to test
            char flagTest3 = 0;

            String procNameUnmanaged = "HelloProcess1";
            com.thalesgroup.CdmwPlatformMngt.Process procUnmanaged = app.get_process(procNameUnmanaged, hostname);

            try {
                myAny.insert_string("TraceClient,HELLO_WORLD,9/TraceClient,HELLO_WORLD,12/");
                procUnmanaged.set_attribute(TRACE_FILTERS.value, myAny, Owner.PRE_DEFINED);
            } catch(Exception e) {
                java.lang.System.out.println("TEST NUM. 3 -->  Error: reached unexpected exception of trace filters setting to an unmanaged process ");
                java.lang.System.out.println(e.toString());
                flagTest3 = 1 ;
            }

            if (flagTest3 == 0) {
                java.lang.System.out.println("TEST NUM. 3 -->  OK: setting (without effect) of trace filters to an unmanaged process ");
            }

            // test: adding a setting of filters ...
            char flagTest4 = 0;
  
            try {
                myAny.insert_string("TraceClient,HELLO_WORLD,9/TraceClient,HELLO_WORLD,12/");
                proc.set_attribute(TRACE_FILTERS.value, myAny, Owner.PRE_DEFINED);
            } catch (Exception e) {
                java.lang.System.out.println("TEST NUM. 4 -->  Error: Failed to set trace filters to managed process");
                flagTest4 = 1;
            }

            if (flagTest4 == 0) {
                java.lang.System.out.println("TEST NUM. 4 -->  OK: setting of other trace filters to the managed process"); 
            }

            // test: getting of these new filters ...
            proc.get_attribute(TRACE_FILTERS.value, myAnyHolder, Owner.PRE_DEFINED);
            String  newFilters = myAnyHolder.value.extract_string();

            if ( newFilters.compareTo("TraceClient,HELLO_WORLD,7/TraceClient,HELLO_WORLD,9/TraceClient,HELLO_WORLD,12/") == 0) {
                java.lang.System.out.println("TEST NUM. 5 -->  OK: getting of actual (in managed process) filters="+newFilters);
            } else {
                java.lang.System.out.println("TEST NUM. 5 -->  Error: getting of actual (in managed process) filters="+newFilters);
            }

            // test: setting of filters at host level ...
            char flagTest6 = 0;

            Host myHost = mySystem.get_host(hostname);

            try {
                myAny.insert_string("TraceClient,HELLO_WORLD,14/") ; 
                myHost.set_attribute(TRACE_FILTERS.value, myAny, Owner.PRE_DEFINED);
            } catch ( AttributeNotFound e) {
                java.lang.System.out.println("REACHED ATTR. NOT FOUND EXCEPTION");  
            } catch(Exception e) {
                java.lang.System.out.println("TEST NUM. 6 --> Error: setting of other trace filters at host level");
                java.lang.System.out.println(e.toString());
                flagTest6 = 1 ;
            }

            if (flagTest6 == 0) {
                java.lang.System.out.println("TEST NUM. 6 -->  OK: setting of other trace filters at host level");
            }

            // advise behavior that init is done
            myProcess.set_init_done();
            synchronized (myProcess) {
                myProcess.notify();
            }

            // start orb
            orb.run();
    
        } catch (Exception e) {
            java.lang.System.err.println("*****ERROR: reached exception in managed process *****");
            // cleanup the ORB to exit completely
            orb.shutdown(false);
            orb.destroy();

            retCode = FAILURE;
        }

        // return retCode;
        if (retCode == FAILURE) {
            java.lang.System.out.println("FAILURE");
        } else {
            java.lang.System.out.println("OK");
        }
    }
}

