/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.platformmngt.test;

import cdmw.platformmngt.ProcessBehaviour;
import cdmw.platformmngt.PlatformInterface;
import com.thalesgroup.*;
import com.thalesgroup.CdmwPlatformMngt.ProcessDelegatePackage.*;
import com.thalesgroup.CdmwPlatformMngtBase.*;
import cdmw.logging.*;
import cdmw.ossupport.Mutex;

/**
 * A user process behaviour.
 *
 */
public class UserProcessBehaviour extends ProcessBehaviour {
    
    public static final int INITIAL = 0;
    public static final int INITIALISED = 1;
    public static final int RUNNING = 2;
    public static final int STOPPED = 3;
    
    private int state = INITIAL;
    private int currentStep = 1;
    
    private Mutex mMutex = new Mutex();
    
    private String mProcessName;
    private String mApplicationName;
    private org.omg.CORBA.ORB mOrb;
    private boolean mInitDone;
    
    public UserProcessBehaviour(org.omg.CORBA.ORB orb) { 
        mOrb=orb;
        mInitDone = false;
    }

    public int nbSteps() {
        return 1;
    }
    
    public int getCurrentStep() {
        return currentStep;
    }
    
    public int getState() {
        return state;
    }

    public void nextStep() throws InvalidStep {}

    public void initialise(StartupKind startupKind) {
        try {

            // first lock the mutex
            mMutex.lock();

            // loop and wait the initialisation from main module
            while (!mInitDone){
                synchronized (this) {
                    mMutex.unlock();
                    // Wait for init end
                    this.wait();
                    mMutex.lock();
                }
            }

            mMutex.unlock();
 
           // get application and process names
           mApplicationName = PlatformInterface.getApplicationName();
           mProcessName = PlatformInterface.getProcessName();
 
            // example of using the PlatformInterface for notifying a message
            PlatformInterface.notifyMessage(INF.value, mProcessName, "Initialisation requested");
        } catch (java.lang.InterruptedException ie){
            ie.printStackTrace();
        } catch(cdmw.common.BadParameterException bpe) {
            bpe.printStackTrace();
        } catch(cdmw.common.BadOrderException boe) {
            boe.printStackTrace();
        }

        state = INITIALISED;
    }
    
    public void run() throws NotReadyToRun {
        try {

            // example of using the PlatformInterface for notifying a message
            PlatformInterface.notifyMessage(INF.value, mProcessName, "Run requested");

            // send to std output some printouts by the FilterMngr ...

            Logger logger_1=Logger.Get_Logger("TraceClient","HELLO_WORLD");

            // these messages must appear to the screen
            Logger.CDMW_LOG(logger_1, new LevelType(128), "TEST MESSAGE AT TRACE LEVEL 7", new Throwable()); 
            Logger.CDMW_LOG(logger_1, new LevelType(512), "TEST MESSAGE AT TRACE LEVEL 9", new Throwable()); 

            // this message must not appear to std output: if it appears is an error
            Logger.CDMW_LOG(logger_1, new LevelType(4), "TEST MESSAGE AT TRACE LEVEL 2", new Throwable());

            state = RUNNING;
        } catch (cdmw.common.BadParameterException bpe) {
            bpe.printStackTrace();
        } catch (cdmw.common.BadOrderException boe) {
           boe.printStackTrace();
        }

    }

    public void stop() {
        try {

            // example of using the PlatformInterface for notifying a message
            PlatformInterface.notifyMessage(INF.value, mProcessName, "Stop requested");
        } catch (cdmw.common.BadParameterException bpe) {
            bpe.printStackTrace();
        } catch (cdmw.common.BadOrderException boe) {
            boe.printStackTrace();
        }

        state = STOPPED;
    }
    
    public void set_init_done() {
        try {
            mMutex.lock();
            mInitDone = true;
            mMutex.unlock();
        } catch (java.lang.InterruptedException ie) {
            ie.printStackTrace();
        }

    }
    
    public void next_step() {}

}

