/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @author Carlo Giovine  <cgiovine@sesm.it>
 * @author Gennaro Mango  <cgiovine@sesm.it>
 */

#include <iostream>
#include <fstream>
#include <limits>
#include <string>
#include "test_unmanaged_init_timeout/TestTimeoutProcess.hpp"
#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "SystemMngt/platformlibrary/Iterator.hpp"
#include "SystemMngt/platformvaluetypes/SystemModeChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/SystemStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/HostStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationAgentDeath_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationModeChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/EntityStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/PropertyChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/UserEvent_impl.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"


using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestTimeoutProcess);

namespace Cdmw
{
namespace PlatformMngt
{

    TestTimeoutProcess::TestTimeoutProcess()
    {
    }

    void TestTimeoutProcess::setUp ()
    {
        try
        {
            //Getting system var
            std::string system_corbaloc = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";
            orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::instance ()->get_ORB ());
            CORBA::Object_var system_obj = orb->string_to_object (system_corbaloc.c_str());
            system = CdmwPlatformMngt::System::_narrow (system_obj.in ());
    
            //Setting up the startupKind parameters
            state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
            startup_mode = 0;
            specified_state_id = "";
            startupKind = new CdmwPlatformMngtBase::StartupKind();
            startupKind->state_retrieval = state_retrieval;
            startupKind->startup_mode = startup_mode;
            startupKind->specified_state_id  = CORBA::string_dup( specified_state_id.c_str() );
    
            //Initialising the variables..
            obName = "TimeObserver";          
            sleeping = 60;
        }
        catch (const CORBA::SystemException& ex) 
        {
            std::cerr<< ex._name()<<std::endl;
            throw;
        }
        catch(...)
        {
        }
    }

    void TestTimeoutProcess::test_start_process ()
    {
        try 
        {
            std::cout<<" "<<std::endl;
            TEST_INFO("Test Starting...");
    
            //Getting the application vector.
            nbApps = system->get_number_of_applications();
            apps   = system->get_applications(nbApps,appIter.out());
    
    
            //Creating and registering the observer
            CORBA::ULong app_test = 0;
            nbProcs = apps[app_test]->get_number_of_processes();
            procs   = apps[app_test]->get_processes(nbProcs, procIter.out());       
            TimeObserver_impl* sysOb = register_observer();
    
    
            TEST_INFO("Doing a cold  restart of the system ... ");
            //Rebooting the system and Waiting for the complention of the initialisation of the system...
            TEST_INFO("Waiting for the initialisation of the system ... ");
    
            system -> cold_restart(startupKind.in());
    
            //Waiting for the system started notification
            int count = 0;
            while (!sysOb->isStarted())
            {
                if (sleeping == count) {CPPUNIT_ASSERT(false);}
                sleep (1);
                count++;
            }
            //If the system doesn't start by the sleeping time, the test fails
            std::cout<<std::endl;
            TEST_INFO("Controlling if the unmanaged processes are started with the right delay ... ");
            
            unsigned long nbApps = system->get_number_of_applications();
            apps   = system->get_applications(nbApps,appIter.out());
            TEST_INFO("Measuring initialisation timeout associated to each process...");
            unsigned long deviation = atol((Cdmw::TestUtils::CORBATestManager::instance()->get_property("--deviation")).c_str());
                
            for (CORBA::ULong proc_count = 0; proc_count<nbProcs; proc_count++)
            {
                ProcessType processType = procs[proc_count]->type();
                std::string ProcName = procs[proc_count]->name();
                //Getting the Timeout value of UNMANAGED PROCESS
                std::string ProcTimeout    = ProcName+"_timeout";
                unsigned long timeout = atol((Cdmw::TestUtils::CORBATestManager::instance()->get_property(ProcTimeout)).c_str())*1000;
    
                if ( (UNMANAGED_PROCESS == processType) && (0!=timeout) )
                {
                    OsSupport::OS::Timeval initProc = sysOb->getInformation(ProcName,"initialising");
                    OsSupport::OS::Timeval runProc  = sysOb->getInformation(ProcName,"running");
                    
                    std::cout<<"\n____________________________________________________"<<std::endl;
                    std::cout<<"Process  "<<ProcName<<": became running      at time (s)"<< runProc.seconds 
                    <<", (usec) "<<runProc.microseconds<<std::endl;
    
                    std::cout<<"             became initialising at time (s)"<< initProc.seconds 
                    <<", (usec) "<<initProc.microseconds<<std::endl;
    
                    //Calculating the difference between currApp and its last predecessor
                    unsigned long diff = (1000000*(runProc.seconds - initProc.seconds)) 
                    + (runProc.microseconds - initProc.microseconds);
                    //Printing the value   
                    
                    std::cout<<"ASSERT: "<<" INITIALISATION_TIMEOUT "<<(timeout/1000)<<" (ms) < MEASURED_TIME " <<(diff/1000)<<" (ms) < "<< "INITIALISATION_TIMEOUT + DEVIATION "<<(timeout/1000)+deviation<<" (ms) "<<std::endl;
                    
                    
                    CPPUNIT_ASSERT( ( (diff > timeout) && (diff< (timeout+deviation*1000)) ) );
                    std::cout<<"------>>> OK <<<------"<<std::endl;
                    std::cout<<"____________________________________________________"<<std::endl;
        
                }else if ((UNMANAGED_PROCESS == processType) && (0==timeout))
                {
                    std::cout<<"\n____________________________________________________"<<std::endl;
                    std::cout<<"Process  "<<ProcName<< ": has a null initialisation timeout\n";
                    std::cout<<"____________________________________________________"<<std::endl;
                }else if (MANAGED_PROCESS == processType)
                {
                    std::cout<<"\n____________________________________________________"<<std::endl;
                    std::cout<<"Process  "<<ProcName<< ": is MANAGED \n";
                    std::cout<<"____________________________________________________"<<std::endl;
                }
            }
    
    
            CPPUNIT_ASSERT(true);
            //Destroying and unregistering the observer
            std::cout<<"\n"<<std::endl;
            unregister_observer(sysOb);
            std::cout<<"\n"<<std::endl;
        }
        catch (const CORBA::SystemException& e)
        {
            std::cout<<e<<std::endl;
            CPPUNIT_ASSERT (false);
            throw;
        }
    }

    TimeObserver_impl* TestTimeoutProcess::register_observer () throw () 
    {
        // Get a reference to the root POA
        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var poa = PortableServer::POA::_narrow( obj.in() );
    
        // Activate the POA manager
        PortableServer::POAManager_var mgr = poa->the_POAManager();
        mgr->activate();
    
        // Register event factories
        CORBA::ValueFactoryBase_var factory, old_factory;
    
        factory = new SystemModeChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/SystemModeChange:1.0",
                factory.in() );
    
        factory = new SystemStatusChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/SystemStatusChange:1.0",
                factory.in() );
    
        factory = new HostStatusChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/HostStatusChange:1.0",
                factory.in() );
    
        factory = new ApplicationAgentDeathFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationAgentDeath:1.0",
                factory.in() );
    
        factory = new ApplicationModeChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationModeChange:1.0",
                factory.in() );
    
        factory = new ApplicationStatusChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationStatusChange:1.0",
                factory.in() );
    
        factory = new ProcessMessageFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessMessage:1.0",
                factory.in() );
    
        factory = new ProcessStatusChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessStatusChange:1.0",
                factory.in() );
    
        factory = new EntityStatusChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/EntityStatusChange:1.0",
                factory.in() );
    
        factory = new UserEventFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/UserEvent:1.0",
                factory.in() );
    
        factory = new PropertyChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/PropertyChange:1.0",
                factory.in() );
    
        TEST_INFO("Registering an observer to the system... ");
    
        //there are the applications used to create the map
        TimeObserver_impl* sysOb= new TimeObserver_impl(procs.in(), nbProcs);
    
        observer = sysOb->_this();
        CdmwPlatformMngt::RegistrationDepth depth=FULL_HIERARCHY;
    
        //Registering the observer
        system->register_observer_ext (obName.c_str(),
                depth, 
                observer.in ());
    
        std::cerr << "Observer registered on System "<< " with depth = " << depth << std::endl;
    
        return sysOb;
    }

    void TestTimeoutProcess::unregister_observer (TimeObserver_impl* sysOb) throw () 
    {
        //Unregistering the observers
        system->unregister_observer_ext (obName.c_str());
        std::cerr << "Unregistering " << obName <<std::endl;
    }  



}/* End namespace PlatformMngt */
}/* End namespace Cdmw */





