/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <iomanip>
#include <typeinfo>
#include <fstream>
#include <sys/time.h>
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/common/Options.hpp"
#include "Foundation/common/Exception.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "test_unmanaged_init_timeout/TimeObserver_impl.hpp"




/**
 * @author Carlo Giovine  <cgiovine@sesm.it>
 * @author Gennaro Mango  <gmango@sesm.it>
 */

namespace Cdmw
{

    namespace PlatformMngt
    {

        TimeObserver_impl::TimeObserver_impl (const CdmwPlatformMngt::Processes& procs,
                                        const CORBA::ULong& nbProcs)        
        {
            started = false;
            for (CORBA::ULong proc_count = 0; proc_count<nbProcs; proc_count++)
            {
                StateList* sl = new StateList ;
                Database[ (procs[proc_count]->name()) ] = sl;
            }
         }
    
        TimeObserver_impl::~TimeObserver_impl()
        {
            //Deleting every Record of every State List..
            for (std::map<std::string,StateList*>::iterator x = Database.begin(); 
                 x !=Database.end(); ++x )
            {
                for (std::map<std::string, Record* >::iterator y = (*(x->second)).begin(); 
                     y !=(*(x->second)).end(); ++y )
                {
                    delete y->second;
                }
            delete x->second;
            }
        }
    
        void  TimeObserver_impl::notify(CdmwPlatformMngt::Event* an_event)
            throw( CORBA::SystemException )
        {
           try
           {
               CdmwPlatformMngt::SystemStatusChange* sys_chg = 
                    dynamic_cast<CdmwPlatformMngt::SystemStatusChange*> (an_event);                
               if (((sys_chg!=0)) &&
                   (sys_chg->system_status()== CdmwPlatformMngt::SYSTEM_STARTED) )            
               {
                   started =  true; 
                   //Printing the message               
                   CORBA::String_var message = an_event->to_string();
                   std::cout << message.in() << std::endl;
               }                
               
               CdmwPlatformMngt::ProcessStatusChange* prc_chg = 
                   dynamic_cast<CdmwPlatformMngt::ProcessStatusChange*> (an_event);
                   
               if (((prc_chg!=0) ) && 
                            ((prc_chg->process_status()==CdmwPlatformMngt::PROCESS_INITIALISING) ))
               {    
                   StateList* sl = (Database[(std::string)((prc_chg->process_id().process_name).in())]);
                   //Setting the information in the record.
                   setInformation(sl, "initialising");
                   //Printing the message
                   CORBA::String_var message = an_event->to_string();
                   std::cout << message.in() << std::endl;
               }
               else if (((prc_chg!=0)) &&
                   (prc_chg->process_status()== CdmwPlatformMngt::PROCESS_RUNNING) )            
               {
                   StateList* sl = (Database[(std::string)((prc_chg->process_id().process_name).in())]);
                   //Setting the information in the record.
                   setInformation(sl, "running");
                   //Printing the message               
                   CORBA::String_var message = an_event->to_string();
                   std::cout << message.in() << std::endl;
               }
               else if (prc_chg!=0)
               {
                   CORBA::String_var message = an_event->to_string();
                   std::cout << message.in() << std::endl;
               }
            }
            catch (const std::exception& ex)
            {
                std::cerr << " Time observer exception  " << ex.what();
            }
    
        }
        
        bool TimeObserver_impl::isStarted()
        {
            return started;
        }
            
        
        void TimeObserver_impl::setInformation(StateList* sl, const std::string& stateName)
        {
            //If the state has not been created yet..
            if (0 == (*sl)[stateName])
            {
               Record* record = new Record();
              
               //Setting the State Variables
               (*record).time = OsSupport::OS::get_time();
               (*record).state=true;
              
               (*sl)[stateName] = record;
            }
            else
            {
              //Setting the State Variables
              ((*sl)[stateName])->time = OsSupport::OS::get_time();
              ((*sl)[stateName])->state= true;
            }
        }
        
        OsSupport::OS::Timeval TimeObserver_impl::getInformation(const std::string appname, const std::string state)
        {
            //returning the timestamp value
            return ((*Database[appname])[state])->time;
        }
    
  } // PlatformMngt
} // Cdmw

