/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/*
 * @author Mango Gennaro <gmango@sesm.it>
 */

#include <fstream>

#include "test_unmanaged_init_timeout/TestTimeoutProcess.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"

namespace
{
    using namespace CdmwPlatformMngt;
    using namespace Cdmw::OsSupport;
    using namespace Cdmw::OrbSupport;
    using namespace Cdmw::PlatformMngt;    
}

static void usage(std::ostream & os, const std::string &program_name)
{
    os << "Usage:\n"       
       << " PROCESS_NAME=TIMEOUT(ms)>"
       << "\n  These options are mandatory :  "        
       << "\n  The process initialisation timeout is set into the scenario XML file "<< std::endl;
}

static bool parser(char* input)
{
    try
    {
        std::string temp= input;
        size_t separator = temp.find("=");
        
        //Getting the name
        std::string ProcName = temp.substr( 0, separator);
        temp=temp.substr(separator+1);
    
        //Defining the strings
        std::string ProcTimeout = ProcName+"_timeout";  
    
        //Getting Timeout
        std::string ProcOption = temp;  
    
        Cdmw::TestUtils::CORBATestManager::instance()->set_property(ProcTimeout, ProcOption);
        return true;
    }
    catch(...)
    {
        return false;
    }
}

int main( int argc, char* argv[])
{
    
    const std::string ERROR_DEVIATION = "--deviation";

    const int MIN_PARAM = 3 ;
    // If number of parameters is not less than MIN_PARAM 
    // show usage and exit
    if (!(argc >= MIN_PARAM))
    {
        usage(std::cout, argv[0]);
        return 0;
    } 
    // ===================================================
    // Initialize the ORB
    // ===================================================
    Cdmw::OrbSupport::StrategyList orb_strategies;
    orb_strategies.add_OrbThreaded ();
    orb_strategies.add_PoaThreadPool (10);
  
    CORBA::ORB_var orb;
    orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv,orb_strategies);
    
    std::string deviation = Cdmw::OsSupport::OS::get_option_value( argc, argv, ERROR_DEVIATION );
    
    Cdmw::TestUtils::CORBATestManager::instance()->set_property(ERROR_DEVIATION, deviation);
      
    
    //Setting up the timeouts 
    for (int i =2; i <argc ; i++)
    {
        bool ret = parser (argv[i]);
        if (!ret)
        { 
            std::cout<<"ERROR : Incorrect Fields"<<std::endl;
            return 0;
        }
        
    }

    Cdmw::TestUtils::CORBATestManager::instance()->init(orb.in(), argc, argv);
    Cdmw::TestUtils::CORBATestManager::instance()->run_tests();
    Cdmw::TestUtils::CORBATestManager::instance()->summary();
    return Cdmw::TestUtils::CORBATestManager::instance()->was_successful();
}

