/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <fstream>
#include <sstream>
#include "testabortprocess/SupervisionObserver_impl.hpp"
#include "Foundation/ossupport/OS.hpp"

namespace Cdmw
{        // Begin namespace Cdmw

    namespace PlatformMngt
    { // Begin namespace PlatformMngt
    
        SupervisionObserver_impl::SupervisionObserver_impl(){
        }
        
        void execute_command(CdmwPlatformMngt::ProcessExitInfo_var exit_info, std::string hostname, std::string  process_name){
            std::string comand = "copy_core.sh ";
            std::cerr << "-----------------------" << std::endl;
            std::cerr << "Process Pid: " << exit_info->process_pid << std::endl;
            std::cerr << "Host name: " << hostname << std::endl;
            std::cerr << "Process name: " << process_name << std::endl;
            std::cerr << "-----------------------" <<  std::endl;
            std::ostringstream corefile;
            corefile << "core." << exit_info->process_pid;
            std::ostringstream com;
            com << comand << " " << hostname << " " << corefile.str();
            std::string command_to_execute=com.str();
            std::cerr << "COMMAND = " << command_to_execute << std::endl;
            OsSupport::OS::create_process("/bin/sh" , command_to_execute , "", NULL, NULL);
        }
        
        void
        SupervisionObserver_impl::notify( CdmwPlatformMngt::Event* an_event )
        throw( CORBA::SystemException )
        {     
        //PROCESS CHANGE STATUS
        if (an_event->event_kind() == CdmwPlatformMngt::PROCESS_STATUS_CHANGE )
        {
            CdmwPlatformMngt::ProcessStatusChange* event = CdmwPlatformMngt::ProcessStatusChange::_downcast( an_event );
            std::string hostname = (const char*)(event->process_id().host_name);
            std::string appname = (const char*)(event->process_id().application_name);
            std::string process_name = (const char*)event->process_id().process_name;
            CdmwPlatformMngt::ProcessExitInfo_var exit_info = event->process_exit_info();
            switch (event->process_status()) {
                case CdmwPlatformMngt::PROCESS_INITIAL:
                {
                // Do nothing.
                //std::cerr << "PROCESS_INITIAL" << std::endl;
                }
                break;
                case CdmwPlatformMngt::PROCESS_STEP_PERFORMED:
                {
                // Do nothing.
                //std::cerr << "PROCESS_STEP_PERFORMED" << std::endl;
                }
                break;
                case CdmwPlatformMngt::PROCESS_INITIALISED:
                {
                // Do nothing.
                //std::cerr << "PROCESS_INITIALISED" << std::endl;
                }
                break;
                case CdmwPlatformMngt::PROCESS_RUN_REQUEST:
                {
                // Do nothing.
                //std::cerr << "PROCESS_RUN_REQUEST" << std::endl;
                }
                break;
                case CdmwPlatformMngt::PROCESS_RUNNING:
                {
                    // Do nothing.
                    //std::cerr << "PROCESS_RUNNING" << std::endl; 
                }
                break;
                case CdmwPlatformMngt::PROCESS_STOPPED:
                {
                    //Find and copy core file
                    std::cerr << "PROCESS_STOPPED" << std::endl;
                    execute_command(exit_info,hostname,process_name);
                }
                break;
                case CdmwPlatformMngt::PROCESS_ENDED:
                {
                    //Find and copy core file
                    std::cerr << "PROCESS_ENDED" << std::endl;
                    execute_command(exit_info,hostname,process_name);
                }
                break;
                case CdmwPlatformMngt::PROCESS_FAILED_ERROR:
                {
                    //Find and copy core file
                    std::cerr << "PROCESS_FAILED_ERROR" << std::endl;
                    execute_command(exit_info,hostname,process_name);
                }
                break;
                case CdmwPlatformMngt::PROCESS_FAILED_TIMEOUT:
                {
                    //Find and copy core file
                    std::cerr << "PROCESS_FAILED_TIMEOUT" << std::endl;
                    execute_command(exit_info,hostname,process_name);
                }
                break;
                case CdmwPlatformMngt::PROCESS_FAILED_DISCREPANCY:
                {
                    //Find and copy core file
                    std::cerr << "PROCESS_FAILED_DISCREPANCY" << std::endl;
                    execute_command(exit_info,hostname,process_name);
                }
                break;
                case CdmwPlatformMngt::PROCESS_FAILED_INVALID:
                {
                    //Find and copy core file
                    std::cerr << "PROCESS_FAILED_INVALID" << std::endl;
                    execute_command(exit_info,hostname,process_name);
                }
                case CdmwPlatformMngt::PROCESS_FAILED_NO_RESPONSE:
                {
                    //Find and copy core file
                    std::cerr << "PROCESS_FAILED_NO_RESPONSE" << std::endl;
                    execute_command(exit_info,hostname,process_name);
                }
                case CdmwPlatformMngt::PROCESS_FAILED_DEATH:
                {
                    //Find and copy core file
                    std::cerr << "PROCESS_FAILED_DEATH" << std::endl;
                    execute_command(exit_info,hostname,process_name);
                }
                break;
                default:
                    // Do nothing.
                break;
            }
        }
    }
    
    void
        SupervisionObserver_impl::writeLog( std::string message ){
            try{
                 std::cout << message << std::endl;
                 std::ofstream os(FILELOG, std::ios_base::app);
                 if(os) {
                     os << message << "\n";
                     os.close();
                 }
            }
            catch( ... ){
            }
       }
    
    } // End namespace PlatformMngt
} // End namespace Cdmw

