/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_MYPROCESSBEHAVIOUR_HPP2 
#define INCL_MYPROCESSBEHAVIOUR_HPP2 

#include "SystemMngt/platforminterface/PlatformInterface.hpp"
#include "SystemMngt/platforminterface/ProcessBehaviour.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/common/Exception.hpp"

namespace TestAutomaticActions
{
    enum Method { NOMETHOD, INITIALISE, RUN, STOP, ISALIVE, NBSTEPS } ;
    enum Action { NOACTION, TIMEOUT, EXIT } ;
    typedef unsigned int Parameter; 
};


using namespace Cdmw::PlatformMngt;
using namespace Cdmw::OsSupport;



class MyProcessBehaviour : public Cdmw::PlatformMngt::ProcessBehaviour
{

public:

    MyProcessBehaviour();
    
    virtual ~MyProcessBehaviour() throw ();

    virtual CORBA::ULong nb_steps() throw(CORBA::SystemException);
    
    virtual CORBA::Object_ptr get_service() throw(CORBA::SystemException);
    
    virtual void initialise(const CdmwPlatformMngtBase::StartupKind& startup_kind)
        throw(CdmwPlatformMngt::ProcessDelegate::BadOrder, CORBA::SystemException);
    
    virtual void next_step()
        throw(CdmwPlatformMngt::ProcessDelegate::InvalidStep, CORBA::SystemException);
    
    virtual void run()
        throw(CdmwPlatformMngt::ProcessDelegate::NotReadyToRun,
              CdmwPlatformMngt::ProcessDelegate::AlreadyDone,
              CORBA::SystemException);
    
    virtual void stop() throw(CORBA::SystemException);
    
    virtual bool is_alive()
        throw(CORBA::SystemException);
        
    bool SetAction( TestAutomaticActions::Method m, 
                    TestAutomaticActions::Action a, 
                    TestAutomaticActions::Parameter p = (TestAutomaticActions::Parameter)0);

private:

    // Do the action specified by TestAutomaticActions parameters
    void DoAction( TestAutomaticActions::Method m);

    // The number of init steps defined for this process.
    int m_nbrOfInitSteps;
    
    // Flag for action valid/not-valid
    bool m_isactionvalid;
    
    // method, action and parameter set for this process
    TestAutomaticActions::Method m_method;
    TestAutomaticActions::Action m_action; 
    TestAutomaticActions::Parameter m_parameter;
    
};


#endif // INCL_MYPROCESSBEHAVIOUR_HPP2

