/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/Exception.hpp>
#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"
#include <SystemMngt/idllib/CdmwPlatformMngtSupervisionObserver.stub.hpp>
#include <FaultTolerance/ftreplicationmanager/InitUtils.hpp>
#include <FaultTolerance/idllib/FT.stub.hpp>
#include <SystemMngt/platformftproxy/InitUtils.hpp>
#include <SystemMngt/platformftproxy/SystemProxy.hpp>

#include <SystemMngt/platformvaluetypes/ConfigurationChange_impl.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtSupervisionObserver.stub.hpp>
#include "Foundation/common/System.hpp"
#include <platformftaccessorlibrary/SystemAccessor_impl.hpp>

#include "testconfigurationchange/SupervisionObserver_impl.hpp"
#include "SystemMngt/platformvaluetypes/ProcessDef_impl.hpp"


#include <SystemMngt/platformftproxy/AccessorLookupService.hpp>
#include <SystemMngt/platformftproxy/DefaultAccessorLookupService.hpp>

using namespace Cdmw;
using namespace Common;
using namespace PlatformMngt;
using namespace CdmwPlatformMngt;
using namespace std;

int main( int argc, char* argv[] )
{
    sleep (4);
    CORBA::ValueFactoryBase_var factory;
    CdmwPlatformMngt::Application_ptr application_test  = 0;
    CdmwPlatformMngt::Process_ptr     process_test      = 0;
    CdmwPlatformMngt::Host_ptr        host_test      = 0;
    CdmwPlatformMngt::Entity_ptr      system_entity_test      = 0;
    CdmwPlatformMngt::Entity_ptr      application_entity_test      = 0;
    CdmwPlatformMngt::Entity_ptr      process_entity_test      = 0;
    CdmwPlatformMngt::Entity_ptr      host_entity_test      = 0;


    Cdmw::OrbSupport::StrategyList orb_strategies;
    orb_strategies.add_OrbThreaded();
    orb_strategies.add_PoaThreadPool( 10 );

    CORBA::ORB_var orb;
    try
    {
    orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv,orb_strategies);

    CORBA::Object_var poa = orb->resolve_initial_references("RootPOA");
    PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(poa.in());

    // activate the POA manager
    PortableServer::POAManager_var manager =  PortableServer::POAManager::_duplicate(rootPOA->the_POAManager());
    manager->activate();

    CdmwPlatformMngt::System_var system_test; 
    /*Create an Instance to Client Observer */
     
     // import the object reference from the ior file
     CORBA::Object_var objref;
        
     const char *corbaloc_str =  "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";

     objref = orb->string_to_object (corbaloc_str);

     system_test = CdmwPlatformMngt::System::_narrow (objref.in ());


    // recoveer the Application APPL1
        application_test = system_test->get_application("APPL1");
        char * hostname  = getenv("HOSTNAME");
        host_test = system_test->get_host (hostname);
        
        CdmwPlatformMngt::Processes_var my_processes_from_app;
        CdmwPlatformMngt::ProcessesIterator_var process_iter;
        CORBA::ULong size_seq = 0;

        my_processes_from_app =
           application_test->get_processes (2, process_iter.out ());

        size_seq = my_processes_from_app->length ();

        for (CORBA::ULong i=0;i<size_seq;i++)
        {
          std::string proc_name =  my_processes_from_app[i]->name() ;
          if ( proc_name == "P11")
            process_test =  my_processes_from_app[i];
         }


        system_entity_test = system_test->get_entity ("SYSTEM_ENTITY");
        application_entity_test = application_test->get_entity ("APPLICATION_ENTITY");
        process_entity_test = process_test->get_entity ("PROCESS_ENTITY");
        host_entity_test = host_test->get_entity ("HOST_ENTITY");     

        //defining Properties
        CORBA::Any anyp_val;
        string sprop;
        int val=10;
        anyp_val <<= val;

        // begin a loop 

            std::cerr<<"<<<<<<<<<<< TEST THE APPLICATION ADDITION >>>>>>>>>>"<<std::endl;     
            CdmwPlatformMngt::ApplicationDef application_def;
            CdmwPlatformMngt::PlatformApplicationDef platform_application_def;
            platform_application_def.application_name = "NewApp";
            system_test->add_application(platform_application_def, application_def);
            sleep (2);  
            std::cerr<<"<<<<<<<<<<< TEST THE APPLICATION REMOTION >>>>>>>>>>>"<<std::endl;
            system_test -> remove_application("NewApp");
            sleep(2);

            std::cerr<<"<<<<<<<<<<< TEST THE PROCESS ADDITION >>>>>>>>>>>"<<std::endl;
            Cdmw::PlatformMngt::ManagedProcessDefFactory process_def_factory;
            CdmwPlatformMngt::ProcessType process_type = MANAGED_PROCESS;
            CdmwPlatformMngt::ProcessDef_var process_def =  process_def_factory.create("NewProcess", process_type );
            process_def->host_name(CORBA::string_dup (hostname));
            application_test -> add_process(process_def);
            sleep(2);
            std::cerr<<"<<<<<<<<<<< TEST THE PROCESS REMOTION >>>>>>>>>>>"<<std::endl;
            application_test -> remove_process("NewProcess", CORBA::string_dup ((hostname)));
            sleep(2);

            std::cerr<<"<<<<<<<<<<< TEST THE HOST ADDITION >>>>>>>>>>>"<<std::endl;
            CdmwPlatformMngt::HostDef hostdef;
            hostdef.host_name = CORBA::string_dup (("newHost"));
            system_test->add_host(hostdef);
            sleep(2);
            std::cerr<<"<<<<<<<<<<< TEST THE HOST REMOTION >>>>>>>>>>>"<<std::endl;  
            system_test->remove_host("newHost");
            sleep(2); 

            std::cerr<<"<<<<<<<<<<< TEST THE SYSTEM ENTITY STATUS Change >>>>>>>>>>>"<<std::endl;
            system_entity_test->set_status("New Entity Status","System Entity new Status");
            sleep(2);

            std::cerr<<"<<<<<<<<<<< TEST THE APPLICATION ENTITY STATUS Change >>>>>>>>>>>"<<std::endl;
            application_entity_test->set_status("New Application Status","Application Entity new Status");
            sleep(2);

            std::cerr<<"<<<<<<<<<<< TEST THE PROCESS ENTITY STATUS Change >>>>>>>>>>>"<<std::endl;
            process_entity_test->set_status("New Process Entity Status","Process Entity new Status");
            sleep(2);

            std::cerr<<"<<<<<<<<<<< TEST THE HOST ENTITY STATUS Change >>>>>>>>>>>"<<std::endl;
            host_entity_test->set_status("New Host Entity Status","Host Entity new Status");
            sleep (2);

            std::cerr<<"<<<<<<<<<<< TEST THE SYSTEM ENTITY ADDITION >>>>>>>>>>>"<<std::endl;
            CdmwPlatformMngt::EntityDef ent_def;
            ent_def.entity_name = "System New Entity";
            system_test->add_entity(ent_def);
            sleep(2);
      
            std::cerr<<"<<<<<<<<<<< TEST THE APPLICATION ENTITY ADDITION >>>>>>>>>>>"<<std::endl; 
            ent_def.entity_name = "Application New Entity";
            application_test->add_entity(ent_def);
            sleep(2);

            std::cerr<<"<<<<<<<<<<< TEST THE PROCESS ENTITY ADDITION >>>>>>>>>>>"<<std::endl;
            ent_def.entity_name = "Process New Entity";
            process_test->add_entity(ent_def);
            sleep(2);
 
            std::cerr<<"<<<<<<<<<<< TEST THE HOST ENTITY ADDITION >>>>>>>>>>>"<<std::endl; 
            ent_def.entity_name = "Host New Entity";
            host_test->add_entity(ent_def);
            sleep(2);
 
            std::cerr<<"<<<<<<<<<<< TEST THE SYSTEM ENTITY REMOTION >>>>>>>>>>>"<<std::endl;
            system_test-> remove_entity("System New Entity");
            sleep(2);

            std::cerr<<"<<<<<<<<<<< TEST THE APPLICATION ENTITY REMOTION >>>>>>>>>>>"<<std::endl;
            application_test-> remove_entity("Application New Entity");
            sleep(2);

            std::cerr<<"<<<<<<<<<<< TEST THE PROCESS REMOTION >>>>>>>>>>>"<<std::endl;
            process_test-> remove_entity("Process New Entity");
            sleep(2);

            std::cerr<<"<<<<<<<<<<< TEST THE HOST ENTITY REMOTION >>>>>>>>>>>"<<std::endl;
            host_test-> remove_entity("Host New Entity");
            sleep(2);

            std::cerr<<"<<<<<<<<<<< TEST PROPERTY CHANGE EVENT ON ALL SUPERVISED OBJECT >>>>>>>>>>>"<<std::endl;
            std::cerr<<std::endl;  
            system_test-> define_property("Property1",anyp_val);
            sleep(1);
            application_test->define_property("Property1",anyp_val);
            sleep(1);
            host_test->define_property("Property1",anyp_val);
            sleep(1);
            process_test-> define_property("Property1",anyp_val);
            sleep(1);
            system_entity_test -> define_property("Property1",anyp_val);
            sleep(1);
            application_entity_test -> define_property("Property1",anyp_val);
            sleep(1);
            process_entity_test -> define_property("Property1",anyp_val);
            sleep(1);
            host_entity_test -> define_property("Property1",anyp_val);
            sleep(2);

    }
    catch (const CORBA::SystemException& eCBex)
    {
    cout << "CORBA Exception :" << eCBex._name() << endl;
    return -1;
    }

    try
    {
            orb->shutdown(false);
            Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
            orb->destroy();
    }
    catch(...)
    {}
}
