/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <testincompatiblestatus/TestIncompatibleStatus.hpp>
#include "Foundation/common/Options.hpp"
#include "Foundation/common/String.hpp"
#include "Foundation/ossupport/OS.hpp"
#include <string>
#include <sstream>
#include <iostream>
#include <fstream>
#include "orbsvcs/orbsvcs/PortableGroupC.h"
#include "SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp"
#include <SystemMngt/idllib/CdmwFTSystMngtSystem.stub.hpp> 
#include <SystemMngt/idllib/CdmwFTSystMngtApplication.stub.hpp> 
#include <SystemMngt/idllib/CdmwFTSystMngtProcess.stub.hpp> 
#include <SystemMngt/idllib/CdmwFTSystMngtEntity.stub.hpp> 
#include <SystemMngt/idllib/CdmwFTSystMngtHost.stub.hpp> 
#include <SystemMngt/platformvaluetypes/ProcessDef_impl.hpp> 

#define NB_TEST_THREADS 10
#define NB_CALLS 10
using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;
namespace{
    const std::string def_sys = "define_system.sh";
    const std::string op = "../share/V-SMG-074-1.xml";
}

namespace CdmwFT 
{

namespace Test 
{

    
CPPUNIT_TEST_SUITE_REGISTRATION( TestIncompatibleStatus );
   
// constructor
TestIncompatibleStatus::TestIncompatibleStatus()
{ 
    m_orb =  CORBA::ORB::_duplicate( Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());
    platform_daemon_port = Cdmw::TestUtils::CORBATestManager::instance()->get_property ("platform_daemon_port");
}

// destructor
TestIncompatibleStatus::~TestIncompatibleStatus()
{
}

void TestIncompatibleStatus::setUp ()
{
}

void TestIncompatibleStatus::test_incompatible_status()
{
    //define the corbaloc
     const char *corbaloc_str = "corbaloc::localhost:21871/CdmwPlatformMngtSupervision";
     CORBA::Object_var objref = m_orb->string_to_object(corbaloc_str);
     CdmwPlatformMngt::System_var system_test = CdmwPlatformMngt::System::_narrow(objref.in() );
     if (CORBA::is_nil (system_test.in ()))
     { 
         CPPUNIT_ASSERT(true);
     }   
     //application_test = system_test->get_application("APPL1");
     CdmwPlatformMngtBase::StartupKind_var startupKind = new CdmwPlatformMngtBase::StartupKind();
     startupKind->state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
     startupKind->startup_mode = 0;
     startupKind->specified_state_id = CORBA::string_dup( "" );
     CPPUNIT_ASSERT_THROW(system_test->start(startupKind), CdmwPlatformMngt::IncompatibleStatus);
     CdmwPlatformMngt::PlatformApplicationDef_var platformAppDef  = new CdmwPlatformMngt::PlatformApplicationDef();
     CdmwPlatformMngt::ApplicationDef_var appDef = new CdmwPlatformMngt::ApplicationDef();     
     appDef->processes.length(0);
     appDef->entities.length(0);
     appDef->service_definitions.length(0);
     platformAppDef->application_name = "APPLERR";
     //System
     CPPUNIT_ASSERT_THROW(system_test->add_application(platformAppDef.in(), appDef.in()), CdmwPlatformMngt::IncompatibleStatus);
     std::cerr << std::endl << "Incompatible status for system OK "<< std::endl;
     CPPUNIT_ASSERT_THROW(system_test->resume(), CdmwPlatformMngt::IncompatibleStatus);
     std::cerr << std::endl << "Incompatible status for system OK "<< std::endl;
     std::cerr << std::endl << "Define the system V-072-1.xml "<< std::endl;
     Cdmw::OsSupport::OS::create_process(def_sys , op , "", NULL, NULL);
     sleep(5);
     CdmwPlatformMngt::Application_ptr application_test = system_test->get_application("CcmTestUnknownStateApplication");
     //Application
     CPPUNIT_ASSERT_THROW(application_test->run(), CdmwPlatformMngt::IncompatibleStatus);
     std::cerr << std::endl << "Incompatible status for application OK "<< std::endl;
     CPPUNIT_ASSERT_THROW(application_test->resume(), CdmwPlatformMngt::IncompatibleStatus);
     std::cerr << std::endl << "Incompatible status for application OK "<< std::endl;  
     //Process
     char * hostname  = getenv("HOSTNAME");
     CdmwPlatformMngt::Process_ptr process_test = application_test->get_process("TestUnknownStateProcessServer",hostname);
     CPPUNIT_ASSERT_THROW(process_test->next_step(), CdmwPlatformMngt::IncompatibleStatus);
     std::cerr << std::endl << "Incompatible status for managed process OK "<< std::endl;
     CPPUNIT_ASSERT_THROW(process_test->run(), CdmwPlatformMngt::IncompatibleStatus);
     std::cerr << std::endl << "Incompatible status for managed process OK "<< std::endl; 
     CdmwPlatformMngt::Process_ptr process_test_umanag = application_test->get_process("unmanag",hostname);
     process_test_umanag->initialise(startupKind);
     sleep(1);
     CPPUNIT_ASSERT_THROW(process_test_umanag->initialise(startupKind), CdmwPlatformMngt::IncompatibleStatus);
     std::cerr << std::endl << "Incompatible status for unmanaged process OK "<< std::endl;
     CPPUNIT_ASSERT(true);
}

} // end namespace Test

} // end namespace CdmwFT

