#!/bin/sh
# =====================================================================
# This file is part of CARDAMOM (R) which is jointly developed by THALES
# and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
# Copyright (C) THALES 2000-2003. All rights reserved.
# Copyright (C) THALES 2004-2005. All rights reserved.
# Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
# All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
# License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================
#===============================================================================

clean ()
{
    killall -9 cdmw_platform_daemon
    killall -9 cdmw_ft_manager
    killall -9 cdmw_platform_supervision
}

LOGFILE=supervision_log.txt
rm -rf $LOGFILE

echo "Cleaning up processes..."
clean

echo "Starting platform daemon..."
if [ -e /tmp/CdmwDaemon_21869 ] 
then
    cdmw_platform_kill_all_pids.sh /tmp/CdmwDaemon_21869
fi
start_daemon_verbosely.sh 2>&1 1>/dev/null &

sleep 5

echo "Starting FT manager..."
cdmw_ft_manager --CdmwXMLFile=CdmwFaultToleranceManager_conf.xml --groupConf=CdmwFTSystemMngtGroupCreator_conf.xml 2>&1 1>/dev/null &

sleep 5

echo "Starting platform supervision..."
start_supervision_to_log_file.sh --CdmwPluginConf=PluginValidConf_01.xml 2>&1 1>/dev/null &

sleep 5

echo "Defining system..."
platform_admin.sh --sys-define PluginValid_SysConf01.xml

sleep 5

echo "Running system..."
platform_admin.sh --sys-start

sleep 15

echo "Stopping system..."
platform_admin.sh --sys-stop

sleep 15

echo; echo;
echo "Testing results..."
RESULT_ARRAY=( 'Error loading plugin "not_existing.so"' \
               'PluginObj() called 0' \
               'PluginObj::set_up() called for ID: 0' \
               'PluginValid_impl() called' \
               'PluginValid_impl::set_system() called' \
               'PluginValid_impl::set_system() registering as observer named PluginValidObserver_0' \
               'PluginValid_impl::set_system() observer PluginValidObserver_0' \
               'PluginObj::set_up() m_plugin reference' \
               'PluginValid_impl::notify() called on PluginValidObserver_0' \
               'PluginValid_impl::notify() called on PluginValidObserver_0.*define_property_with_mode : property_name = Cdmw.DefaultRestartWarmth' \
               'PluginValid_impl::notify() called on PluginValidObserver_0.*Application Added' \
               'Property Observer PluginValidObserver_0 registered on APPL1' \
               'Property Observer PluginValidObserver_0 registered on P11' \
               'Property Observer PluginValidObserver_0 registered on PROCESS_ENTITY' \
               'Property Observer PluginValidObserver_0 registered on APPLICATION_ENTITY' \
               'Property Observer PluginValidObserver_0 registered on SYSTEM_ENTITY' \
               "PluginValid_impl::notify() called on PluginValidObserver_0.*S:'S1' A:'APPL1' INITIALISING" \
               "PluginValid_impl::notify() called on PluginValidObserver_0.*S:'S1' A:'APPL1' H:'$HOSTNAME' P:'P11' INITIALISING" \
               "PluginValid_impl::notify() called on PluginValidObserver_0.*S:'S1' A:'APPL1' H:'$HOSTNAME' P:'P11' RUNNING" \
               "PluginValid_impl::notify() called on PluginValidObserver_0.*S:'S1' A:'APPL1' RUNNING" \
               "PluginValid_impl::notify() called on PluginValidObserver_0.*S:'S1' STARTED" \
               "PluginValid_impl::notify() called on PluginValidObserver_0.*S:'S1' A:'APPL1' RUN_REQUEST" \
               "PluginValid_impl::notify() called on PluginValidObserver_0.*S:'S1' A:'APPL1' H:'$HOSTNAME' P:'P11' RUN_REQUEST" \
               "PluginValid_impl::notify() called on PluginValidObserver_0.*S:'S1' A:'APPL1' H:'$HOSTNAME' P:'P11' RUNNING" \
               "PluginValid_impl::notify() called on PluginValidObserver_0.*S:'S1' A:'APPL1' RUNNING" \
               "PluginValid_impl::notify() called on PluginValidObserver_0.*S:'S1' STARTED" \
               "PluginValid_impl::notify() called on PluginValidObserver_0.*S:'S1' STOPPING" \
               "PluginValid_impl::notify() called on PluginValidObserver_0.*S:'S1' A:'APPL1' STOPPING" \
               "PluginValid_impl::notify() called on PluginValidObserver_0.*S:'S1' A:'APPL1' H:'$HOSTNAME' P:'P11' STOPPING" \
               "PluginValid_impl::notify() called on PluginValidObserver_0.*S:'S1' A:'APPL1' H:'$HOSTNAME' P:'P11' STOPPED" \
               "PluginValid_impl::notify() called on PluginValidObserver_0.*S:'S1' A:'APPL1' STOPPED" \
               "PluginValid_impl::notify() called on PluginValidObserver_0.*S:'S1' STOPPED" \
)

i=0
while [ "$i" -lt "${#RESULT_ARRAY[@]}" ]
do
    ENTRY=${RESULT_ARRAY[$i]}
    echo ENTRY: $ENTRY 
    RESULT=`grep "$ENTRY" "$LOGFILE"`
    echo RESULT: $RESULT
    if [ -z "$RESULT" ]
    then
        echo "Test is NOT OK"
        clean
        exit
    fi
    let "i = $i + 1"
done
echo "Test is OK"
echo; echo;


echo "Cleaning up processes..."
clean


