/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <fstream>
#include <sstream>
#include "testprocesshostresponding/SupervisionObserver_impl.hpp"

namespace Cdmw
{        // Begin namespace Cdmw

    namespace PlatformMngt
    { // Begin namespace PlatformMngt
    
        SupervisionObserver_impl::SupervisionObserver_impl(){
            prevProcessNumber[0][0] = 1;
            prevProcessNumber[1][0] = 2;
            prevProcessNumber[2][0] = 3;
            prevProcessNumber[0][1] = 0;
            prevProcessNumber[1][1] = 0;
            prevProcessNumber[2][1] = 0;
        }
        
        void
        SupervisionObserver_impl::notify( CdmwPlatformMngt::Event* an_event )
        throw( CORBA::SystemException )
        {     
        //HOST STATUS CHANGE
        if (an_event->event_kind() == CdmwPlatformMngt::HOST_STATUS_CHANGE )
        {    
            CORBA::String_var message = an_event->to_string();
            
            CdmwPlatformMngt::HostStatusChange* event = CdmwPlatformMngt::HostStatusChange::_downcast( an_event );
            std::string hostname = (const char*)(event->host_name());
            switch (event->host_status()) {
                case CdmwPlatformMngt::HOST_UNREACHABLE:
                {
                        std::string tmp = hostname + ":UNREACHABLE";
                        writeLog(tmp);
                }
                break;
                case CdmwPlatformMngt::HOST_RESPONDING:
                {
                        std::string tmp = hostname + ":HOST_RESPONDING";
                        writeLog(tmp);
                }
                break;
                case CdmwPlatformMngt::HOST_STARTED:
                {
                        std::string tmp = hostname + ":HOST_STARTED";
                        writeLog(tmp);
                }
                break;
                case CdmwPlatformMngt::HOST_PARTIAL_STARTED:
                {
                        std::string tmp = hostname + ":HOST_PARTIAL_STARTED";
                        writeLog(tmp);
                }
                break;
                case CdmwPlatformMngt::HOST_STOPPING:
                {
                        std::string tmp = hostname + ":HOST_STOPPING";
                        writeLog(tmp);
                }
                break;
                case CdmwPlatformMngt::HOST_STARTING:
                {
                        std::string tmp = hostname + ":HOST_STARTING";
                        writeLog(tmp);
                }
                break;
                case CdmwPlatformMngt::HOST_REINSERTING:
                {
                        std::string tmp = hostname + ":HOST_REINSERTING";
                        writeLog(tmp);
                }
                break;
                default:
                    // Do nothing.
                break;
            }
        }
        //PROCESS CHANGE STATUS
        if (an_event->event_kind() == CdmwPlatformMngt::PROCESS_STATUS_CHANGE )
        {
            CdmwPlatformMngt::ProcessStatusChange* event = CdmwPlatformMngt::ProcessStatusChange::_downcast( an_event );
            std::string hostname = (const char*)(event->process_id().host_name);
            std::string appname = (const char*)(event->process_id().application_name);
            std::string process_name = (const char*)event->process_id().process_name;
            switch (event->process_status()) {
                case CdmwPlatformMngt::PROCESS_INITIAL:
                {
                        std::string tmp = " * " + process_name + ":INITIAL";
                        writeLog(tmp);
                }
                break;
                case CdmwPlatformMngt::PROCESS_STEP_PERFORMED:
                {
                        std::string tmp = " * " + process_name + ":PROCESS_STEP_PERFORMED";
                        writeLog(tmp);
                }
                break;
                case CdmwPlatformMngt::PROCESS_INITIALISED:
                {
                        std::string tmp = " * " + process_name + ":INITIALISED";
                        writeLog(tmp);
                }
                break;
                case CdmwPlatformMngt::PROCESS_RUN_REQUEST:
                {
                        std::string tmp = " * " + process_name + ":PROCESS_RUN_REQUEST";
                        writeLog(tmp);
                }
                break;
                case CdmwPlatformMngt::PROCESS_RUNNING:
                {
                        std::string tmp = " * " + process_name + ":PROCESS_RUNNING";
                        writeLog(tmp);
                }
                break;
                case CdmwPlatformMngt::PROCESS_STOPPED:
                {
                        std::string tmp = " * " + process_name + ":PROCESS_STOPPED";
                        writeLog(tmp);
                }
                break;
                case CdmwPlatformMngt::PROCESS_ENDED:
                {
                        std::string tmp = " * " + process_name + ":PROCESS_ENDED";
                        writeLog(tmp);
                }
                break;
                case CdmwPlatformMngt::PROCESS_FAILED_ERROR:
                {
                        std::string tmp = " * " + process_name + ":PROCESS_FAILED_ERROR";
                        writeLog(tmp);
                }
                break;
                case CdmwPlatformMngt::PROCESS_FAILED_TIMEOUT:
                {
                        std::string tmp = " * " + process_name + ":PROCESS_FAILED_TIMEOUT";
                        writeLog(tmp);
                }
                break;
                case CdmwPlatformMngt::PROCESS_FAILED_DISCREPANCY:
                {
                        std::string tmp = " * " + process_name + ":PROCESS_FAILED_DISCREPANCY";
                        writeLog(tmp);
                }
                break;
                case CdmwPlatformMngt::PROCESS_FAILED_INVALID:
                {
                        std::string tmp = " * " + process_name + ":PROCESS_FAILED_INVALID";
                        writeLog(tmp);
                }
                case CdmwPlatformMngt::PROCESS_FAILED_NO_RESPONSE:
                {
                        std::string tmp = " * " + process_name + ":PROCESS_FAILED_NO_RESPONSE";
                        writeLog(tmp);
                }
                case CdmwPlatformMngt::PROCESS_FAILED_DEATH:
                {
                        std::string tmp = " * " + process_name + ":PROCESS_FAILED_DEATH";
                        writeLog(tmp);
                }
                break;
                default:
                    // Do nothing.
                break;
            }
        }
    }
    
    void
        SupervisionObserver_impl::writeLog( std::string message ){
            try{
                 std::cout << message << std::endl;
                 std::ofstream os(FILELOG, std::ios_base::app);
                 if(os) {
                     os << message << "\n";
                     os.close();
                 }
            }
            catch( ... ){
            }
       }
    
    } // End namespace PlatformMngt
} // End namespace Cdmw

