/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef CARDAMOM_CLOCK_COS_CLOCK_HPP
#define CARDAMOM_CLOCK_COS_CLOCK_HPP

#include <Foundation/common/Config.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>

#include <Time/idllib/CosClockService.stub.hpp>
#include <Time/idllib/CdmwTime.stub.hpp>

//! @todo To remove
#include <orbsvcs/orbsvcs/PortableGroupC.h>

#include <memory>
#include <vector>
#include <string>

/**
 *  ID used by JNIClock implementation
 **/
 
const int LOCAL_CLOCK_ID                    = 0;
const int CLOCK_CATALOG_ID                  = 1;
const int MASTER_CLOCK_ID                   = 2;
const int POLICY_CONTROLLER_UPDATE_ADMIN_ID = 3;
const int EXECUTOR_ID                       = 4; // to remove
const int FEDERATEDCONTROLLEDCLOCK_ID       = 5; 
const int FEDERATEDCONTROLLEDEXECUTOR_ID    = 6;

const int IDSIZE                            = 7;

namespace Cdmw  { 

    /**
     * @namespace Cdmw::clock
     * @brief This namespace is the root namespace of all EVoT API.
     * 
     * @note EVoT: Enhanced View Of Time
     */    
    namespace clock {

    /**
     * @namespace Cdmw::clock::CosClock
     * @brief This namespace contains the funtions required to init a 
     * EVoT service.
     */
    namespace   CosClock
    {
        /** 
         * Start service and allocate resources.
         * @param theOrb A reference to the ORB.
         * @param argc The argc from main.
         * @param argv The argv from main.
         * @return true if all is ok, false is some problem.
         */
        CDMW_API   bool 
            init(CORBA::ORB_ptr theOrb,
                 int argc, 
                 char *argv[]);

        /**
         * Cleanup correctly services.
         */
        CDMW_API void
            close();

        /** 
         * Return a reference to GOA.
         * @return The initialized GOA.
         */
        CDMW_API PortableGroup::GOA_ptr
            get_GOA();


       /**
        *  Initialize C++ Clock Service for external (JNI) calls.
        **/
        void initCPPClockService(int argc, char **argv);

       /**
        *  Set a vector of IORs corresponding to Clock Objects registerd on the orb
        *  relative to C++ Clock Service for external (JNI) calls.
        **/
        std::vector< std::string > setIORsClockService();

    }    

} /* clock */ } /* Cdmw */

#endif /* CARDAMOM_CLOCK_COS_CLOCK_HPP */

///////////////////////////////////////////////////////////////////////////////
// End Of File
///////////////////////////////////////////////////////////////////////////////
