/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
 
#include "Foundation/commonsvcs/federation/LocalTopicUpdateManager.hpp"
#include "Time/clockservice/FederatedControlledClockSvc.hpp"

Cdmw::clock::FederatedControlledClockSvc* 
Cdmw::clock::FederatedControlledClockSvc::m_instance=0;

Cdmw::clock::FederatedControlledClockSvc::FederatedControlledClockSvc(CORBA::ORB_ptr           orb,
                                                                      PortableGroup::GOA_ptr  goa,
                                                                      const std::string&       miop_corbaloc,
                                                                      int                      send_repetition)
    : m_protocol_handler(orb, goa, miop_corbaloc, send_repetition),
      m_update_manager(Cdmw::CommonSvcs::LocalTopicUpdateManager::bind(&m_protocol_handler))
{
    Cdmw::OrbSupport::CodecBase::Init(orb);
    m_codec = new Cdmw::clock::FederatedControlledClockDataCodec_t;

    CDMW_ASSERT(m_update_manager != 0);
    CDMW_ASSERT(m_codec          != 0);
}

//////////////////////////////////////////////////////////////////////////////

Cdmw::clock::FederatedControlledClockSvc::~FederatedControlledClockSvc()
    throw()
{
    delete m_update_manager;
    m_update_manager = 0x0;
    delete m_codec;
    m_codec = 0x0;
}

//////////////////////////////////////////////////////////////////////////////

void
Cdmw::clock::FederatedControlledClockSvc::init(CORBA::ORB_ptr           orb,
                                               PortableGroup::GOA_ptr   goa,
                                               const std::string&       miop_corbaloc,
                                               int                      send_repetition)
{
    CDMW_ASSERT(m_instance == 0);
    m_instance = new FederatedControlledClockSvc(orb, goa, miop_corbaloc, send_repetition); 
}

//////////////////////////////////////////////////////////////////////////////

Cdmw::clock::FederatedControlledClockSvc&
Cdmw::clock::FederatedControlledClockSvc::instance()
{
    return *m_instance;
}

//////////////////////////////////////////////////////////////////////////////

::Cdmw::clock::FederatedControlledClockDataCodec_t*
Cdmw::clock::FederatedControlledClockSvc::get_codec()
{
    return m_codec;
}

//////////////////////////////////////////////////////////////////////////////

::Cdmw::CommonSvcs::SimpleMiopUpdateProtocolHandler*
Cdmw::clock::FederatedControlledClockSvc::get_protocol_handler()
{
    return &m_protocol_handler;
}

//////////////////////////////////////////////////////////////////////////////

::Cdmw::CommonSvcs::LocalTopicUpdateManager*
Cdmw::clock::FederatedControlledClockSvc::get_update_manager()
{
    return m_update_manager;
}

//////////////////////////////////////////////////////////////////////////////
