/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <cdmw_clock_JNIClock.hpp>
#include <clockservice/CdmwCosClock.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/commonsvcs/time/JNI_ThreadDef.hpp>

static jobjectArray globalRefToArray = 0;

void *clock_init_thread ( void *arg ) ;

static jobjectArray globalRefTooarr = 0;

JavaVM *jvm;
jobject obj;
pthread_t thread;


/**
 * @brief   clock_init_thread
 *
 * This function initialise the c++ clock service, calling the native c++ method
 * Cdmw::clock::CosClock::initCPPClockService.
 *
 */
 
void *clock_init_thread ( void *arg )
{
    JNIEnv *env;
    jclass cls;
    jmethodID mid;

    //std::cout << ">>>>> [DEBUG] (Native Code) Start thread to execute Clock::init (C++Clock Service) <<<<<<"<< std::endl;

    if ( jvm->AttachCurrentThread ( ( void ** ) &env, NULL ) < 0 )
    {
        fprintf ( stderr, "%s[%d]: AttachCurrentThread ()\n",
                  __FILE__, __LINE__ );
        return NULL;
    }

    int argc = env->GetArrayLength ( globalRefTooarr );

    char** argv = ( char** ) calloc ( argc + 1, sizeof ( char* ) );
    jboolean iscopy;

    for ( int i = 0; i < argc; i++ )
    {
        jobject jarg  = env->GetObjectArrayElement ( globalRefTooarr, i );
        const char* arg   = env->GetStringUTFChars ( ( jstring ) jarg, &iscopy );
        argv[i]       = ( char* ) arg;
        //std::cout << ">>>>> [DEBUG] (Native Code) i << " << i    << " <<<<"<< std::endl;
        //std::cout << ">>>>> [DEBUG] (Native Code) argc << " << argc    << " <<<<"<< std::endl;
        //std::cout << ">>>>> [DEBUG] (Native Code) argv << " << argv[i] << " <<<<"<< std::endl;
    }

    //std::cout << ">>>>> [DEBUG] (Native Code) :  invoke initCPPClockService <<<<<<"<< std::endl;
    Cdmw::clock::CosClock::initCPPClockService ( argc, argv );

    return NULL;

}

/**
 * @brief   Java_cdmw_clock_JNIClock_initCppClock [java: JNIClock.initCppClock ]
 *
 * This function will be called inside the java code, to initialise the clock service c++ library.
 *
 */

JNIEXPORT void JNICALL Java_cdmw_clock_JNIClock_initCppClock
( JNIEnv *env, jclass jobj, jobjectArray oarr )
{

    /* Set argc and argv */
    int argc = env->GetArrayLength ( oarr );

    char** argv = ( char** ) calloc ( argc + 1, sizeof ( char* ) );
    jboolean iscopy;

    for ( int i = 0; i < argc; i++ )
    {
        jobject jarg = env->GetObjectArrayElement ( oarr, i );
        const char* arg = env->GetStringUTFChars ( ( jstring ) jarg, &iscopy );
        argv[i] = ( char* ) arg;
    }

    int ret;

    /********* JVM *********/

    if ( env->GetJavaVM ( &jvm ) < 0 )
    {
        fprintf ( stderr, "%s[%d]: GetJavaVM ()\n",
                  __FILE__, __LINE__ );
        return ;
    }

    globalRefTooarr = ( jobjectArray ) env->NewGlobalRef ( oarr );

    //std::cout << ">>>>> [DEBUG] (Native Code)   create thread for C++ Clock::init  <<<<<<"<< std::endl;

    if ( ( ret = JNI_pthread_create ( &thread, NULL, clock_init_thread, NULL ) ) != 0 )
    {
        fprintf ( stderr, "%s[%d]: pthread_create (): %s\n",
                  __FILE__, __LINE__, strerror ( ret ) );
        return ;
    }

    return;
}


/**
 * @brief   Java_cdmw_clock_JNIClock_setIORsClockService [java: JNIClock.setIORsClockService ]
 *
 * This function will be called inside the java code, to get all the IORs registered from the clock 
 * service c++ library, converting in a java String[] array.
 *
 */

jobjectArray JNICALL Java_cdmw_clock_JNIClock_setIORsClockService
( JNIEnv *env, jclass jobj )

{

    jstring ior;
    jstring jstr = env->NewStringUTF ( "" );
    static jclass stringClass = NULL;

    if ( stringClass == NULL )
    {
        jclass localRefCls =
            env->FindClass ( "java/lang/String" );

        if ( localRefCls == NULL )
        {
            return NULL; /* exception thrown */
        }

        /* Create a global reference */
        stringClass = ( jclass ) env->NewGlobalRef ( localRefCls );

        /* The local reference is no longer useful */
        env->DeleteLocalRef ( localRefCls );

        /* Is the global reference created successfully? */
        if ( stringClass == NULL )
        {
            return NULL; /* out of memory exception thrown */
        }
    }

    jobjectArray iorArray = ( jobjectArray ) env->NewObjectArray ( IDSIZE, stringClass, NULL );

    Cdmw::OsSupport::OS::sleep ( 3, 0 );

    //std::cout << ">>>>> [DEBUG] (Native Code)   create array for CLOCK IORs <<<<<<"<< std::endl;
    std::vector< std::string > localClockIOR = Cdmw::clock::CosClock::setIORsClockService();

    env->SetObjectArrayElement
    (   iorArray,
        LOCAL_CLOCK_ID,
        env->NewStringUTF ( localClockIOR[LOCAL_CLOCK_ID].c_str() )
    );

    env->SetObjectArrayElement
    (   iorArray,
        CLOCK_CATALOG_ID,
        env->NewStringUTF ( localClockIOR[CLOCK_CATALOG_ID].c_str() )
    );

    env->SetObjectArrayElement
    (   iorArray,
        MASTER_CLOCK_ID,
        env->NewStringUTF ( localClockIOR[MASTER_CLOCK_ID].c_str() )
    );

    env->SetObjectArrayElement
    (   iorArray,
        POLICY_CONTROLLER_UPDATE_ADMIN_ID,
        env->NewStringUTF ( localClockIOR[POLICY_CONTROLLER_UPDATE_ADMIN_ID].c_str() )
    );

    env->SetObjectArrayElement
    (   iorArray,
        EXECUTOR_ID,
        env->NewStringUTF ( localClockIOR[EXECUTOR_ID].c_str() )
    );

    env->SetObjectArrayElement
    (   iorArray,
        FEDERATEDCONTROLLEDCLOCK_ID,
        env->NewStringUTF ( localClockIOR[FEDERATEDCONTROLLEDCLOCK_ID].c_str() )
    );

    env->SetObjectArrayElement
    (   iorArray,
        FEDERATEDCONTROLLEDEXECUTOR_ID,
        env->NewStringUTF ( localClockIOR[FEDERATEDCONTROLLEDEXECUTOR_ID].c_str() )
    );



    globalRefToArray = ( jobjectArray ) env->NewGlobalRef ( iorArray );

    env->DeleteLocalRef ( iorArray );

    env->DeleteLocalRef ( jstr );

    //std::cout << ">>>>> [DEBUG] (Native Code)   return IORs Array <<<<<<"<< std::endl;
    return globalRefToArray; //iorArray;
}

/**
 * @brief   Java_cdmw_clock_JNIClock_closeCppClock [java: JNIClock.closeCppClock ]
 *
 * This function will be called inside the java code, to close the clock service c++ library.
 *
 */

JNIEXPORT void JNICALL Java_cdmw_clock_JNIClock_closeCppClock
( JNIEnv *env , jclass jobj )
{
    //std::cout << ">>>>> [DEBUG] (Native Code) close clock <<<<<<"<< std::endl;
    env->DeleteGlobalRef ( globalRefToArray );
    env->DeleteGlobalRef ( globalRefTooarr );
    Cdmw::clock::CosClock::close();
}
