/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.clock;

import org.omg.CosClockService.ClockPOA;
import org.omg.CosClockService.ClockCatalog;
import org.omg.CosClockService.ClockCatalogHelper;
import org.omg.CosClockService.ClockCatalogPackage.ClockEntry;
import org.omg.CosClockService.ClockHelper;
import org.omg.CosClockService.ClockOperations;
import org.omg.CosClockService.ClockProperty.*;
import org.omg.CosClockService.TimeUnavailable;
import org.omg.CosPropertyService.PropertySet;
import org.omg.CosPropertyService.PropertySetHelper;
import org.omg.CosPropertyService.PropertyNameHelper;

import cdmw.ossupport.OS;
import cdmw.ossupport.Timeval;
import cdmw.clock.Macro;
import cdmw.clock.Time;
import org.omg.CORBA.ORB;
import org.omg.CORBA.ORBPackage.InvalidName;
import org.omg.CosPropertyService.*;
import org.omg.CosPropertyService.PropertySetDefFactory;
import org.omg.CosPropertyService.PropertySetDefFactoryHelper;


import org.omg.CosClockService.*;

/**
 * Implementation fo Clock Interface
 *
 */
public class ClockImpl extends ClockPOA
    implements ClockOperations
{

    /**
     *  Constructor 
     *
     */
    public ClockImpl () 
    {
    }
    public ClockImpl (ORB orb) 
    {
        org.omg.CORBA.Object obj  = null;
        try
        {
            System.out.println("orb.resolve_initial_references PropertyService");
            obj  = orb.resolve_initial_references("PropertyService");
        }
        catch (InvalidName in)
        {
            System.out.println("Can't resolve `PropertyService'");
        }
        if (obj==null)
        {
            System.out.println("`PropertyService' is a nil object reference");
        }
        m_propset_factory = PropertySetDefFactoryHelper.unchecked_narrow(obj);
        m_property = m_propset_factory.create_propertysetdef();

        org.omg.CORBA.Any  value = orb.create_any();

    value.insert_ulong(RESOLUTION);
        setProperty("Resolution",value);

    value.insert_ushort(PRECISION);
        setProperty("Precision",value);

    value.insert_ushort(WIDTH);
        setProperty("Width",value);

    value.insert_string("Not Available");
        setProperty("Stability_Description",value);

    value.insert_ushort(COORDINATION);
        setProperty("Coordination",value);

    value.insert_ushort(org.omg.CosClockService.ClockProperty.Local_.value);
        setProperty("TimeScale",value);

    }

     /**
     * Set Property value
     *
     * @param name  Property Name
     * @param value Property Value
     *
     */

    private void setProperty( java.lang.String   name, org.omg.CORBA.Any  value) 
    {
        try
        {
           m_property.define_property(name,value);
        }
        catch(org.omg.CosPropertyService.ConflictingProperty e)
        {
        }
        catch(org.omg.CosPropertyService.UnsupportedProperty e)
        {
        }
        catch(org.omg.CosPropertyService.UnsupportedTypeCode e)
        {
        }
        catch(org.omg.CosPropertyService.ReadOnlyProperty e)
        {
        }
        catch(org.omg.CosPropertyService.InvalidPropertyName e)
        {
        }


    }
    /**
     *  Get the set of Clock Properties
     *
     *  @return PropertySet Set of properties associated to clock
     */
    public PropertySet properties()
    {
        return m_property;  
    }   

    /**
     *  Get current time
     *
     *  @return time in TimeT
     */
    public long current_time()
        throws TimeUnavailable
    {  
        long time_tot = OS.getTime().getMicroseconds();
        int time_s  = (int)(time_tot / Macro.USEC_IN_A_SEC);
        int time_us = (int)(time_tot % Macro.USEC_IN_A_SEC);
        Timeval timeval = new Timeval(time_s, time_us);
        return Time.timeval2TimeT(timeval);
    }

    private PropertySetDefFactory m_propset_factory;
    public  PropertySet           m_property;
    
    private final int RESOLUTION = 1000;
    private final short PRECISION = 24;
    private final short WIDTH = 64;
    private final short COORDINATION = 0;
}
