/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.clock;

import cdmw.ossupport.OS;

// import for ClockCatalog
import org.omg.CosClockService.ClockCatalogPackage.ClockEntriesHelper;
import org.omg.CosClockService.ClockCatalogPackage.ClockEntry;
import org.omg.CosClockService.ClockCatalogPackage.ClockEntryHolder;
import org.omg.CosClockService.ClockCatalogPackage.UnknownEntry;

import cdmw.orbsupport.ORBSupport;

import cdmw.logging.LogManager;
import static cdmw.logging.Logger.*;

import Cdmw.clock.ControlledClockExHelper;
import Cdmw.clock.ControlledClockEx;
import Cdmw.clock.ControlledExecutorHelper;
import Cdmw.clock.ControlledExecutor;



public class JNIClock
{

    public static class ClockThread extends Thread
    {

        private String[] args;
        /*
        *   ClockThread constructor
        */
        public ClockThread (String[] args)
        {
            String[] newStringArgs = new String[args.length +1 ];

            // it is needed for interaction between c++ and java
            newStringArgs[0] = "dummyArgumentForJNI";


            System.arraycopy (args, 0, newStringArgs, 1, args.length);


            this.args = newStringArgs ;
        }

        /*
        *   write IORs on file
        */
        private static void writeToFile (String[] iors, java.io.PrintWriter out)
        {
            for (int i = 0; i < iors.length ; i++)
                out.println (iors[i]);
        }

        /*
        *   run Clock thread
        */
        public void run()
        {

            CDMW_DEBUG_1(m_logger,"init clock (jni)");
            initCppClock (args);

            while (true)
            {
                try
                {
                    OS.sleep (100000);
                    CDMW_DEBUG_1(m_logger,"Java ClockThread is alive");
                }
                catch (InterruptedException ie)
                    {}
            }
        }
    }

///////////////////////////////////////////////
    private static final int LOCAL_CLOCK_ID                     = 0;
    private static final int CLOCK_CATALOG_ID                   = 1;
    private static final int MASTER_CLOCK_ID                    = 2;
    private static final int POLICY_CONTROLLER_UPDATE_ADMIN_ID  = 3;
    private static final int EXECUTOR_ID                        = 4;
    private static final int FEDERATEDCONTROLLEDCLOCK_ID        = 5;
    private static final int FEDERATEDCONTROLLEDEXECUTOR_ID     = 6;

    private static cdmw.logging.Logger m_logger = null;

    private static ClockThread thr;

    // load native library for initCppClock
    static
    {
        cdmw.ossupport.OS.loadLibrary ("cdmwjniclockservice");
    }

    //obtain initialize C++ClockService
    public static native void initCppClock (String args[]);

    //obtain IORs corresponding to Clock Objects
    public static native String[] setIORsClockService();

    //close C++ClockService
    public static native void closeCppClock();

    //close ClockService
    public static boolean close()
    {
        CDMW_DEBUG_1(m_logger,"close clock (jni)");
        closeCppClock();
        CDMW_DEBUG_1(m_logger,"close java clock thread");
        thr.stop();
        return true;
    }

    // register IORs on JAVA orb
    public static boolean init (org.omg.CORBA.ORB orb, String args[])
    {


        LogManager.Init (args);
        m_logger = cdmw.logging.Logger.Get_Logger ("CDMW.TMG.JAVA");
        org.omg.CORBA.Object objClock = null;
                
        CDMW_DEBUG_1(m_logger,"starting java clock thread");
        thr = new ClockThread (args);
        thr.start();

        CDMW_DEBUG_1(m_logger,"obtaining clock references (jni)");
        String[] iors = setIORsClockService();

        if (iors[LOCAL_CLOCK_ID].compareTo ("NOT PRESENT") != 0)
        {
            /////////////////////////////////////////////////////////////////
            // register LOCAL_CLOCK
            objClock = orb.string_to_object (iors[LOCAL_CLOCK_ID]);

            if (objClock == null)

            {
                CDMW_ERROR(m_logger,"LocalClockIOR not valid");
                return false;
            }

            org.omg.CosClockService.Clock localclock =

                org.omg.CosClockService.ClockHelper.narrow (objClock);

            if (localclock == null)
            {
                CDMW_ERROR(m_logger,"Narrow failed : LocalClockIOR refers to different object: localclock is null");
                return false;
            }

            ORBSupport.registerInitialReference ((org.omg.CORBA.ORB) orb, "LocalClock", localclock);

            // bind  object to corbaloc
            ORBSupport.bindObjectToCorbaloc (orb, "LocalClock", localclock);
        }

        /////////////////////////////////////////////////////////////////
        // register CLOCK_CATALOG
        if (iors[CLOCK_CATALOG_ID].compareTo ("NOT PRESENT") != 0)
        {

            objClock = orb.string_to_object (iors[CLOCK_CATALOG_ID]);

            if (objClock == null)
            {
                CDMW_ERROR(m_logger,"ClockCatalogIOR not valid");
                return false;
            }

            org.omg.CosClockService.ClockCatalog clockCatalog =

                org.omg.CosClockService.ClockCatalogHelper.narrow (objClock);

            if (clockCatalog == null)
            {
                CDMW_ERROR(m_logger,"Narrow failed : ClockCatalog IOR refers to null object");
                return false;
            }

            // bind  object to corbaloc

            ORBSupport.registerInitialReference (orb, "ClockService", clockCatalog);

            ORBSupport.bindObjectToCorbaloc (orb, "ClockService", orb.string_to_object (iors[CLOCK_CATALOG_ID]));
        }

        /////////////////////////////////////////////////////////////////
        // register MASTER_CLOCK
        {
            if (iors[MASTER_CLOCK_ID].compareTo ("NOT PRESENT") != 0)
            {
                objClock = orb.string_to_object (iors[MASTER_CLOCK_ID]);

                if (objClock == null)
                {
                    CDMW_ERROR(m_logger,"MasterClockIOR not valid");
                    return false;
                }

                org.omg.CosClockService.Clock masterClock =

                    org.omg.CosClockService.ClockHelper.narrow (objClock);

                if (masterClock == null)
                {
                    CDMW_ERROR(m_logger,"Narrow failed : MasterClock IOR refers to null object");
                    return false;
                }

                ORBSupport.registerInitialReference (orb, "MasterClock", masterClock);
            }
        }

        /////////////////////////////////////////////////////////////////
        // register POLICY_CONTROLLER_UPDATE_ADMIN
        if (iors[POLICY_CONTROLLER_UPDATE_ADMIN_ID].compareTo ("NOT PRESENT") != 0)
        {
            objClock = orb.string_to_object (iors[POLICY_CONTROLLER_UPDATE_ADMIN_ID]);

            if (objClock == null)
            {
                CDMW_ERROR(m_logger,"PolicyControllerAdminIOR not valid");
                return false;
            }

            Cdmw.clock.policy.ControllerUpdateAdmin policyControllerAdmin =

                Cdmw.clock.policy.ControllerUpdateAdminHelper.narrow (objClock);

            if (policyControllerAdmin == null)
            {
                CDMW_ERROR(m_logger,"Narrow failed : policyControllerAdminIOR refers to different object: clockCatalog is null");
                return false;
            }

            ORBSupport.registerInitialReference (orb, "PolicyControllerUpdateAdmin", policyControllerAdmin);

            // bind  object to corbaloc
            ORBSupport.bindObjectToCorbaloc (orb, "PolicyControllerUpdateAdmin", policyControllerAdmin);

        }

        /////////////////////////////////////////////////////////////////
        // register EXECUTOR_ID
        if (iors[EXECUTOR_ID].compareTo ("NOT PRESENT") != 0)
        {
            objClock = orb.string_to_object (iors[EXECUTOR_ID]);

            if (objClock == null)
            {
                CDMW_ERROR(m_logger,"executor_ior not valid");
                return false;
            }

            org.omg.CosClockService.PeriodicExecution.Executor executor =

                org.omg.CosClockService.PeriodicExecution.ExecutorHelper.narrow (objClock);

            if (executor == null)
            {
                CDMW_ERROR(m_logger,"narrow failed : executor refers to different object: executor is null");
                return false;
            }

            ORBSupport.registerInitialReference (orb, "Jexecutor", executor);
        }

        /////////////////////////////////////////////////////////////////
        // register FEDERATEDCONTROLLEDCLOCK_ID
        if (iors[FEDERATEDCONTROLLEDCLOCK_ID].compareTo ("NOT PRESENT") != 0)
        {
            objClock = orb.string_to_object (iors[FEDERATEDCONTROLLEDCLOCK_ID]);

            if (objClock == null)
            {
                CDMW_ERROR(m_logger,"federatedcontrolledclock_ior not valid");
                return false;
            }

            ControlledClockEx  federatedcontrolledclock =

                ControlledClockExHelper.narrow (objClock);

            if (federatedcontrolledclock == null)
            {
                CDMW_ERROR(m_logger,"narrow failed : federatedcontrolledclock refers to " +
                                     "different object: federatedcontrolledexecutor is null");
                return false;
            }

            ORBSupport.registerInitialReference (orb, "FederatedControlledClock", federatedcontrolledclock);
        }

        /////////////////////////////////////////////////////////////////
        // register FEDERATEDCONTROLLEDEXECUTOR_ID
        if (iors[FEDERATEDCONTROLLEDEXECUTOR_ID].compareTo ("NOT PRESENT") != 0)
        {
            objClock = orb.string_to_object (iors[FEDERATEDCONTROLLEDEXECUTOR_ID]);

            if (objClock == null)
            {
                CDMW_ERROR(m_logger,"federatedcontrolledexecutor_ior not valid");
                return false;
            }

            ControlledExecutor federatedcontrolledexecutor =

                ControlledExecutorHelper.narrow (objClock);

            if (federatedcontrolledexecutor == null)
            {
                CDMW_ERROR(m_logger,"narrow failed : federatedcontrolledexecutor refers to " +
                                     "different object: federatedcontrolledexecutor is null");
                return false;
            }

            ORBSupport.registerInitialReference (orb, "FederatedControlledExecutor", federatedcontrolledexecutor);
        }

        return true;
    }
}

