/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.clock;

import org.omg.CORBA.BAD_PARAM;
import org.omg.CORBA.CompletionStatus;
import org.omg.CosClockService.*;
import org.omg.TimeBase.IntervalT;
import org.omg.TimeBase.UtcT;

public class TimeSpanImpl extends TimeSpan
{
    public TimeSpanImpl()
    {}
    ////////////////////////////////////////////////////////////////////////////
    
    public IntervalT time_interval()
    {
        IntervalT ret_val = new IntervalT();
        ret_val.lower_bound = this.lower_bound;
        ret_val.upper_bound = this.upper_bound;  
        return ret_val;
    }
    ////////////////////////////////////////////////////////////////////////////
    
    public OverlapType spans(UTC time, TimeSpanHolder overlap)
    {
        // just to rember to add code for check time parameter validity
        //    but when is it not valid?????
        boolean is_valid = true;
        if(!is_valid)
            throw new org.omg.CORBA.BAD_PARAM();

        return overlap_impl(this,
                            time.interval(),
                            overlap);
    }
    ////////////////////////////////////////////////////////////////////////////
    public OverlapType overlaps(TimeSpan other, TimeSpanHolder overlap)
    {
        // check for validity
        boolean is_valid = true;
        if(!is_valid)
            throw new org.omg.CORBA.BAD_PARAM();

        return overlap_impl(this,
                            other,
                            overlap);
    }
    ////////////////////////////////////////////////////////////////////////////

    public UTC time()
    {
        long time_ = (upper_bound + lower_bound) / 2L;
        long inaccuracy = time_ - lower_bound;
        UtcT utct = new UtcT(time_, 
            (int)(inaccuracy & Macro.Mask32), 
            (short)(int)((inaccuracy & 0xFFFFFFFF00000000L) >> 32), 
            (short)0);
        UTCImpl utc = new UTCImpl();
        utc.time = utct.time;
        utc.inacclo = utct.inacclo;
        utc.inacchi = utct.inacchi;
        utc.tdf = utct.tdf;
        return utc;
    }
    ////////////////////////////////////////////////////////////////////////////

    private OverlapType
    overlap_impl(TimeSpan ts_A,
                 TimeSpan ts_B,
                 TimeSpanHolder ts_out)
    {
        ts_out.value = new TimeSpanImpl();
        // return the gap
        if( ts_A.lower_bound > ts_B.upper_bound )
        {
            ts_out.value.lower_bound = ts_B.upper_bound;
            ts_out.value.upper_bound = ts_A.lower_bound;
            return OverlapType.OTNoOverlap;
        }

        if( ts_A.upper_bound < ts_B.lower_bound )
        {
            ts_out.value.lower_bound = ts_A.upper_bound;
            ts_out.value.upper_bound = ts_B.lower_bound;
            return OverlapType.OTNoOverlap;
        }

        if( ts_A.lower_bound <= ts_B.lower_bound )
        {
            ts_out.value.lower_bound = ts_B.lower_bound;
            if( ts_B.upper_bound <= ts_A.upper_bound )
            {
                ts_out.value.upper_bound = ts_B.upper_bound;
                return OverlapType.OTContainer;
            }
        }
        else // then ts_A_lower is greater than ts_B_lower
        {
            ts_out.value.lower_bound = ts_A.lower_bound;
            if( ts_A.upper_bound <= ts_B.upper_bound )
            {
                ts_out.value.upper_bound = ts_A.upper_bound;
                return OverlapType.OTContained;
            }
        }

        assert (Math.max(ts_A.lower_bound, ts_B.lower_bound) ==
                ts_out.value.lower_bound );

        long min_val = Math.min(ts_A.upper_bound, ts_B.upper_bound);
        // then ts_A_upper is greater then ts_B_upper
        ts_out.value.upper_bound = min_val;
        return OverlapType.OTOverlap;
    }
    
    ////////////////////////////////////////////////////////////////////////////
}
