/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @brief
 *
 * @author Francesca Annunziata <fannunziata@selex-si.com>
 * @author Raffaele Mele <rmele@progesi.it>
 */

package cdmw.clock.test;

import cdmw.clock.*;
import cdmw.orbsupport.ORBSupport;
import cdmw.orbsupport.ORBUtils;
import cdmw.ossupport.OS;
import cdmw.ossupport.ThreadedProcess;
import cdmw.ossupport.Timeval;
import cdmw.testutils.Testable;

import java.io.IOException;

import org.omg.CORBA.ORB;
import org.omg.CORBA.ORBPackage.InvalidName;
import org.omg.CosClockService.*;
import org.omg.CosClockService.Clock;
import org.omg.CosPropertyService.InvalidPropertyName;
import org.omg.CosPropertyService.PropertyNotFound;
import org.omg.CosPropertyService.PropertySet;
import org.omg.CosPropertyService.PropertySetDef;
import org.omg.CosPropertyService.PropertySetDefFactory;
import org.omg.CosPropertyService.PropertySetHelper;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;
import org.omg.PortableServer.POAManager;
import org.omg.PortableServer.POAManagerPackage.AdapterInactive;

/**
 * @brief
 *
 * @author Francesca Annunziata <fannunziata@selex-si.com>
 * @author Raffaele Mele <rmele@progesi.it>
 */

///////////////////////////////////////////////////////////////////////////////

public class ClockTest extends Testable
{
    private ORB orb;
    private org.omg.CosClockService.Clock clock;
    private org.omg.CosPropertyService.PropertySet clock_properties;
    org.omg.CORBA.Any prop;

    private final int       NUMOFTESTS = 2;

///////////////////////////////////////////////////////////////////////////////

    public ClockTest (ORB orb) 
    {
      this.name = "ClockTest";
      this.out = System.out;
      this.orb = orb;
      try {
        org.omg.CORBA.Object obj = orb.resolve_initial_references("LocalClock");
        this.clock =  ClockHelper.narrow(obj);
      } 
      catch (InvalidName ex)
      {
        ex.printStackTrace();
      }
      this.clock_properties = clock.properties();
    }

///////////////////////////////////////////////////////////////////////////////

    public boolean currentTimeTest()
    {
        try 
        {
            long current_time = clock.current_time();
            Timeval time = OS.getTime();
            if  ((time.getMicroseconds() - current_time) < 100000) 
               return Macro.TEST_OK ;
        }
        catch (TimeUnavailable ex) 
        {
           ex.printStackTrace();
        }
        return Macro.TEST_NOK;    
    }

///////////////////////////////////////////////////////////////////////////////

    private boolean checkProperty(String name, int value)
      throws PropertyNotFound, InvalidPropertyName
    {

        System.out.println("check property " + name) ;
        prop = clock_properties.get_property_value(name);
        if (prop.extract_ulong() != value) 
            return Macro.TEST_NOK;
        return Macro.TEST_OK;
    }

///////////////////////////////////////////////////////////////////////////////

    private boolean checkProperty(String name, short value)
      throws PropertyNotFound, InvalidPropertyName
    {

        System.out.println("check property " + name) ;
        prop = clock_properties.get_property_value(name);
        if (prop.extract_short() != value) 
            return Macro.TEST_NOK;
        return Macro.TEST_OK;
    }

///////////////////////////////////////////////////////////////////////////////

    private boolean checkProperty(String name, String value)
      throws PropertyNotFound, InvalidPropertyName
    {

        System.out.println("check property " + name) ;
        prop = clock_properties.get_property_value(name);
        if (prop.extract_string() != value) 
        return Macro.TEST_NOK;
        return true;
    }

///////////////////////////////////////////////////////////////////////////////

    public boolean propertiesTest() 
    {
        final int   RESOLUTION             = 1000;
        final short PRECISION              = 24;
        final short WIDTH                  = 64;
        final short COORDINATION           = 0;
        final short TIMESCALE              = org.omg.CosClockService.ClockProperty.Local_.value;
        final String STABILITY_DESCRIPTION = "Not Available"; 

        try 
        {
            checkProperty("Resolution",RESOLUTION);
            checkProperty("Precision",PRECISION);
            checkProperty("Width",WIDTH);
            checkProperty("Coordination",COORDINATION);
            checkProperty("TimeScale",TIMESCALE);
            checkProperty("Stability_Description",STABILITY_DESCRIPTION);
            return Macro.TEST_OK;
        }
        catch (PropertyNotFound e)
        {
           return Macro.TEST_NOK;
        }
        catch (InvalidPropertyName e)
        {
           return Macro.TEST_NOK;
        }
    }

///////////////////////////////////////////////////////////////////////////////

    public void doTests() 
    {
        setNbOfRequestedTestOK(NUMOFTESTS);

        check(currentTimeTest()); 
        check(propertiesTest()); 
    }
}

