/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.clock.test;


import cdmw.clock.*;
import Cdmw.clock.ControlledExecutor;
import Cdmw.clock.ControlledExecutorHelper;
import Cdmw.clock.ControllerEx;
import Cdmw.clock.ControllerExHelper;
import cdmw.clock.Util;
import cdmw.ossupport.OS;
import cdmw.testutils.Testable;
import java.lang.Math;
import org.omg.CORBA.*;
import org.omg.CosClockService.*;
import org.omg.CosClockService.ClockCatalogPackage.ClockEntriesHelper;
import org.omg.CosClockService.ClockCatalogPackage.ClockEntry;
import org.omg.CosClockService.ClockCatalogPackage.ClockEntryHolder;
import org.omg.CosClockService.ClockCatalogPackage.UnknownEntry;
import org.omg.CosClockService.ClockHelper;
import org.omg.CosClockService.PeriodicExecution.ControllerPackage.TimePast;
import org.omg.CosClockService.PeriodicExecution.Controller;

public class ControlledExecutorTest extends Testable
{

    private final int NUMOFTESTS = 3;

    private ControlledExecutor m_executor;
    private ClockCatalog m_ctg;
    private org.omg.CosClockService.PeriodicExecution.Controller m_controller;
    private org.omg.CosClockService.PeriodicExecution.Periodic p;
    private MyPeriodicExecution periodic;
    private ORB m_orb;
    private short id_policy;

    public ControlledExecutorTest (ORB orb, ClockCatalog ctg)
    {
        name = "ControlledExecutorTest";
        out = System.out;
        m_ctg = ctg;
        m_orb = orb;
        try
        {
            ClockEntry entry = m_ctg.get_entry("ControlledExecutor");
            m_executor = ControlledExecutorHelper.narrow(entry.subject);
            if (m_executor == null)
            {
                out.println("ControlledExecutor: nil reference.");
                System.exit(1);
            }
            periodic = new MyPeriodicExecution();
            if (periodic == null)
            {
                out.println("MyPeriodicExecution: nil reference.");
                System.exit(1);
            }
            p = periodic._this(m_orb);
            if (p == null)
            {
                out.println("my_periodic: nil reference.");
                System.exit(1);
            }
            m_controller = (org.omg.CosClockService.PeriodicExecution.Controller)m_executor.enable_periodic_execution(p);
            if (m_controller == null)
            {
                out.println("m_controller: nil reference.");
                System.exit(1);
            }
        }
        catch ( UnknownEntry ex )
        {
            out.println( "ClockCatalog::UnknownEntry" );
        }

    }

    public boolean enable_periodic_execution_with_policy_test()
    {
        try
        {
              m_controller =  (org.omg.CosClockService.PeriodicExecution.Controller)m_executor.enable_periodic_execution_with_policy(p,Cdmw.clock.policy.ENFORCE_INTERVAL.value); 
            if (m_controller == null)
            {
                out.println("m_controller: nil reference.");
                return Macro.TEST_NOK;
            }
    
        }
        catch (Cdmw.clock.policy.ControllerUpdateAdminPackage.Unavailable un)
        {
            out.println("Exception Throw:" + un);
            return Macro.TEST_NOK;
        }
        catch (Exception e)
        {
            e.printStackTrace();
            return Macro.TEST_NOK; 
        }
        return Macro.TEST_OK;
    }
   
    public boolean get_controller_update_policy_test()
    {
        try
        {
            m_executor.set_controller_update_policy(Cdmw.clock.policy.ENFORCE_INTERVAL.value);
            id_policy = m_executor.get_controller_update_policy();
        
            if (id_policy == Cdmw.clock.policy.ENFORCE_INTERVAL.value)
                return Macro.TEST_OK;
            else  
                return Macro.TEST_NOK; 
        
        }
        catch (Cdmw.clock.policy.ControllerUpdateAdminPackage.Unavailable un)
        {
            out.println("Exception Throw:" + un);
            return Macro.TEST_NOK;
        }
        catch (Exception e)
        {
            e.printStackTrace();
            return Macro.TEST_NOK; 
        }
    }


    public boolean set_controller_update_policy_test()
    {
        try
        {
            m_executor.set_controller_update_policy(Cdmw.clock.policy.CANCEL_ALL.value);
            id_policy = m_executor.get_controller_update_policy();
        
            if (id_policy == Cdmw.clock.policy.CANCEL_ALL.value)
                return Macro.TEST_OK;
            else  
                return Macro.TEST_NOK;   
                                
        }
        catch (Cdmw.clock.policy.ControllerUpdateAdminPackage.Unavailable un)
        {
            out.println("Exception Throw:" + un);
            return Macro.TEST_NOK;
        }
        catch (Exception e)
        {
            e.printStackTrace();
            return Macro.TEST_NOK;
        }
     }
   
    public void doTests()
    {

       setNbOfRequestedTestOK(NUMOFTESTS);

       check(enable_periodic_execution_with_policy_test());
       check(get_controller_update_policy_test());
       check(set_controller_update_policy_test());

    }
}

