/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.clock.test;


import Cdmw.clock.ControlledClockEx;
import Cdmw.clock.ControlledClockExHelper;
import Cdmw.clock.ControlledExecutor;
import Cdmw.clock.ControlledExecutorHelper;
import Cdmw.clock.ControllerEx;
import Cdmw.clock.ControllerExHelper;
import cdmw.clock.Time;
import cdmw.clock.Util;
import cdmw.ossupport.OS;
import cdmw.testutils.Testable;
import java.lang.Math;
import org.omg.CORBA.*;
import org.omg.CosClockService.*;
import org.omg.CosClockService.ClockCatalogPackage.ClockEntriesHelper;
import org.omg.CosClockService.ClockCatalogPackage.ClockEntry;
import org.omg.CosClockService.ClockCatalogPackage.ClockEntryHolder;
import org.omg.CosClockService.ClockCatalogPackage.UnknownEntry;
import org.omg.CosClockService.ClockHelper;
import org.omg.CosClockService.PeriodicExecution.ControllerPackage.TimePast;
import org.omg.CosClockService.PeriodicExecution.Controller;

public class ControllerTest extends Testable
{
   
    private final int NUMOFTESTS = 6;

    private ControlledExecutor m_executor;
    private ClockCatalog m_ctg;
    private org.omg.CosClockService.PeriodicExecution.Controller m_controller;
    private org.omg.CosClockService.PeriodicExecution.Periodic p;
    private MyPeriodicExecution periodic;
    private ORB m_orb;
    
    
    private ControlledClockEx m_controlledClock;
    

    public ControllerTest (ORB orb, ClockCatalog ctg)
    {
        name = "ControllerTest";
        out = System.out;
        m_ctg = ctg;
        m_orb = orb;
        try
        {
            ClockEntry entry = m_ctg.get_entry("ControlledExecutor");
            m_executor = ControlledExecutorHelper.narrow(entry.subject);
            if (m_executor == null)
            {
                out.println("ControlledExecutor: nil reference.");
                System.exit(1);
            }
            periodic = new MyPeriodicExecution();
            if (periodic == null)
            {
                out.println("MyPeriodicExecution: nil reference.");
                System.exit(1);
            }
            p = periodic._this(m_orb);
            if (p == null)
            {
                out.println("my_periodic: nil reference.");
                System.exit(1);
            }
            m_controlledClock = ControlledClockExHelper.narrow(entry.subject);
            if (m_controlledClock == null)
            {
                System.out.println("ControlledClock: nil reference.");
                System.exit(1);
            }
        }
        catch ( UnknownEntry ex )
        {
            out.println( "ClockCatalog::UnknownEntry" );
        }
    }

    public boolean start_test(long period, long with_offset, int execution_limit, org.omg.CORBA.Any params)
    {
        try
        {
            if (p == null)
            {
                out.println("p: nil reference.");
                return Macro.TEST_NOK;
            }
           m_controller = m_executor.enable_periodic_execution(p);
            if (m_controller == null)
            {
                out.println("m_controller: nil reference.");
                return Macro.TEST_NOK;
            }
            m_controller.start(period,with_offset,execution_limit,params);
            cdmw.ossupport.OS.sleep(1100);
            m_controller.terminate(); 
            
            m_controller = m_executor.enable_periodic_execution(p);
            if (m_controller == null)
            {
                out.println("m_controller: nil reference.");
                return Macro.TEST_NOK;
            }         
            m_controller.start(period,0,execution_limit,params);
            cdmw.ossupport.OS.sleep(1100);
            m_controller.terminate(); 
        }
        catch (InterruptedException e)
        {
            e.printStackTrace();
            return Macro.TEST_NOK;
        }
        catch (org.omg.CORBA.BAD_INV_ORDER bi)
        {
            
            return Macro.TEST_NOK;
        }
        return Macro.TEST_OK;
    }
    
    public boolean pause_and_resume_test(long period, long with_offset, int execution_limit, org.omg.CORBA.Any params)
    {
        try
        {
            m_controller = m_executor.enable_periodic_execution(p);
            if (m_controller == null)
            {
                out.println("m_controller: nil reference.");
                return Macro.TEST_NOK;
            }
            m_controller.start(period,with_offset,execution_limit,params);
            OS.sleep(1100);
            m_controller.pause();
            m_controller.resume();
            m_controller.pause();
            m_controller.terminate();
            
        }
        catch (java.lang.InterruptedException ex)
        {
            ex.printStackTrace();
            return Macro.TEST_NOK;
        }
        return Macro.TEST_OK;
    }
    
    public boolean start_at_test(long period, long at_time, int execution_limit, org.omg.CORBA.Any params)
    {
        try
        {
            m_controller = m_executor.enable_periodic_execution(p);
            if (m_controller == null)
            {
                out.println("m_controller: nil reference.");
                return Macro.TEST_NOK;
            }
            
            m_controller.start_at(period,at_time,execution_limit,params);
            try
            {
                cdmw.ossupport.OS.sleep(1100);
                m_controller.terminate();
            }
            catch (InterruptedException e)
            {
                 e.printStackTrace();
                 return Macro.TEST_NOK;
            }
        }
        catch (org.omg.CosClockService.PeriodicExecution.ControllerPackage.TimePast tp)
        {
            out.println("Exception :" + tp);
            return Macro.TEST_NOK;
        }   
        catch (org.omg.CORBA.BAD_INV_ORDER tp)
        {
            out.println("Exception :" + tp);
            return Macro.TEST_NOK;
        }
        return Macro.TEST_OK;
    }



    public boolean pause_and_resume_at_test(long period, long with_offset, int execution_limit, org.omg.CORBA.Any params, long at_time)
    {
        try
        {
           m_controller = m_executor.enable_periodic_execution(p);
            if (m_controller == null)
            {
                out.println("Error !! m_controller: nil reference.");
                return Macro.TEST_NOK;
            }
            
            m_controller.start(period,0,execution_limit,params);
            m_controller.pause();
            m_controller.resume_at(at_time);
            cdmw.ossupport.OS.sleep(1100);
            m_controller.pause();
            m_controller.terminate(); 
        }
        catch (org.omg.CosClockService.PeriodicExecution.ControllerPackage.TimePast tp)
        {
            out.println("Error!! You are trying to start controller at time in the past! " + tp);
            return Macro.TEST_NOK;
        }
        catch (InterruptedException e)
        {
            e.printStackTrace();
            return Macro.TEST_NOK;
        }
        catch (org.omg.CORBA.BAD_INV_ORDER tp)
        {
            out.println("Exception :" + tp);
            return Macro.TEST_NOK;
        }

        return Macro.TEST_OK;
    }

    public boolean terminate_test(long period, long with_offset, int execution_limit, org.omg.CORBA.Any params)
    {
        try
        {
            m_controller = m_executor.enable_periodic_execution(p);
            if (m_controller == null)
            {
                out.println("Error !! m_controller: nil reference.");
                return Macro.TEST_NOK;
            }
            m_controller.start(period,with_offset,execution_limit,params);
            m_controller.terminate();
            //m_controller.pause();
        }
        catch (org.omg.CORBA.BAD_INV_ORDER tp)
        {
            //out.println("Exception :" + tp);
            return Macro.TEST_NOK;
        }
        return Macro.TEST_OK;
    }
    
    public boolean executions_test(long period, long with_offset, int execution_limit, org.omg.CORBA.Any params)
    {
        long numb;
        
        numb = m_controller.executions();
        if (numb != 0)
            return  Macro.TEST_NOK;
 
        m_controller = m_executor.enable_periodic_execution(p);
        if (m_controller == null)
        {
            out.println("m_controller: nil reference.");
            return Macro.TEST_NOK;
        }
        m_controller.start(period,with_offset,execution_limit,params);
        try
        {
            cdmw.ossupport.OS.sleep(1000);
        }
        catch (InterruptedException e)
        {
            e.printStackTrace();
        }
        numb = m_controller.executions();
        out.println("controller.executions:" + numb);
        if (numb == 10)
            return Macro.TEST_OK;
        
        return Macro.TEST_NOK;
    }
    
    public void doTests()
    {
        try {
            setNbOfRequestedTestOK(NUMOFTESTS);
            long period = 100000; // 0,01s
            long with_offset;
            long at_time;
            int execution_limit = 10;
            org.omg.CORBA.Any params = m_orb.create_any();
            params.insert_long(1234566789);  
            with_offset = 1000000;
            
            check(start_test(period,with_offset,execution_limit,params));
            check(pause_and_resume_test(period,with_offset,execution_limit,params));
            
            at_time = m_controlledClock.current_time()+with_offset; 
            check(start_at_test(period,at_time,execution_limit,params));
            at_time = m_executor.current_time()+with_offset;
            check(pause_and_resume_at_test(period,with_offset,execution_limit,params,at_time));
            check(terminate_test(period,with_offset,execution_limit,params));
            check(executions_test(period,0,execution_limit,params));
        }
        catch (org.omg.CosClockService.TimeUnavailable ex)
        {
            ex.printStackTrace();
        }

    }
}

