/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.clock.test;
import Cdmw.clock.ControlledExecutor;
import Cdmw.clock.ControlledExecutorHelper;
import cdmw.testutils.Testable;
import org.omg.CORBA.ORB;
import org.omg.CosClockService.ClockCatalog;
import org.omg.CosClockService.ClockCatalogPackage.ClockEntry;
import org.omg.CosClockService.ClockCatalogPackage.UnknownEntry;


public class ExecutorTest extends Testable
{

    private final int       NUMOFTESTS = 1;

    //private cdmw.clock.ExecutorImpl executor;        
    private ORB orb;
    private ClockCatalog m_ctg;
    private org.omg.CosClockService.PeriodicExecution.Executor executor;
    
    
    public ExecutorTest (ORB orb, ClockCatalog ctg) 
    {
      this.name = "ExecutorTest";
      this.out = System.out;
      this.orb = orb;

      m_ctg = ctg;
      try
      {
          ClockEntry entry = m_ctg.get_entry("ControlledExecutor");
          //executor = ControlledExecutorHelper.narrow(entry.subject);
          executor = org.omg.CosClockService.PeriodicExecution.ExecutorHelper.narrow(entry.subject);
      }    
      catch ( UnknownEntry ex )
      {
          out.println( "ClockCatalog::UnknownEntry" );
      }  
    }

    public boolean enable_periodic_executionTest()
    {
        try 
        {
           MyPeriodicExecution periodic = new MyPeriodicExecution();
           org.omg.CosClockService.PeriodicExecution.Periodic p = periodic._this(orb);
           org.omg.CosClockService.PeriodicExecution.Controller c = executor.enable_periodic_execution(p);
           if (c == null)
           {
               out.println("m_controller: nil reference.");
               return Macro.TEST_NOK;
           }
           long period = 100000;
           long with_offset = 0;
           
           int execution_limit = 100;
           org.omg.CORBA.Any params = orb.create_any();
           c.start(period,with_offset,execution_limit,params);
           c.terminate();
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
            return Macro.TEST_NOK;
        }

        return Macro.TEST_OK;    
    }

    public void doTests() 
    {
        setNbOfRequestedTestOK(NUMOFTESTS);

        check(enable_periodic_executionTest()); 
    }
}

