/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
   
package cdmw.clock.test;
import cdmw.testutils.Testable;
import cdmw.ossupport.Timeval;
import cdmw.clock.*;
import java.text.SimpleDateFormat;
import java.text.ParseException;
import java.util.Date;
import java.util.TimeZone;


public class ImportExportTimevalTest extends Testable
{
    private final int       NUMOFTESTS = 7;


    private final String dateSample    = "Fri Sep 08 18:07:41 CEST 2006";
    private final long dateInTimeT     = 133770244610000000L;
    public ImportExportTimevalTest()
    {
      this.name = "ImportExportTimevalTest";
      this.out = System.out;
    }

    public boolean checkTimevalToTimeT(Timeval dateInTimeval) 
    {

         long utcTime = Time.timeval2TimeT(dateInTimeval);
         return ( (long) utcTime == dateInTimeT ) ;
    }
    
    public boolean checkTimeTtoTimeval(Timeval dateInTimeval) 
    {
         Timeval tv = Time.TimeT2timeval(dateInTimeT); 
         return (Util.compare(tv,dateInTimeval) == 0);
    }

    public boolean PCR_1224() 
    {
        cdmw.clock.cdmwTime res = new cdmw.clock.cdmwTime();
        // Java Epoch
        res.tm_year = 70;
        res.tm_mon = 0;
        res.tm_mday = 1;
        res.tm_hour = 0;
        res.tm_min = 0;
        res.tm_sec = 0;
        res.tm_msec = 0;

        cdmw.ossupport.Timeval res2 = cdmw.clock.Util.cdmwTime2timeval(res);
        String date1 = res2.toDate().toString();
        long timevalue = cdmw.clock.Time.timeval2TimeT(res2);
        cdmw.ossupport.Timeval value3 = cdmw.clock.Time.TimeT2timeval(timevalue);
        String date2 = value3.toDate().toString();
        if (!date1.equalsIgnoreCase(date2)) 
            return Macro.TEST_NOK;
        return Macro.TEST_OK;
    }
    
    /**
     * @brief function printDate converts a TimeT input in a struct cdmwTime and checks sec,msec and usec
     *        and prints the date in following format YYYY-MM-DD HH:MI:SS.millis.micro          
     * @param startDate is the TimeT data input 
     *        operation is a string that describe the operation done on the inputDate
     * @return true if the check is OK and false if the check is not OK
     *        
     */    
    public boolean checkAndPrintDate( long startDate, String operation, int sec, int msec, int usec)
    {
        cdmw.ossupport.Timeval timevalForDate = cdmw.clock.Time.TimeT2timeval(startDate);
        cdmw.clock.cdmwTime dateInCdmwTime = new cdmw.clock.cdmwTime();     
        dateInCdmwTime = cdmw.clock.Util.timeval2cdmwTime(timevalForDate);
        System.out.format("%4d-%02d-%02d %02d:%02d:%02d.%03d.%06d (%d) ",
        (dateInCdmwTime.tm_year+1900) , 
        (dateInCdmwTime.tm_mon + 1)   , 
        dateInCdmwTime.tm_mday        , 
        dateInCdmwTime.tm_hour        ,
        dateInCdmwTime.tm_min         ,
        dateInCdmwTime.tm_sec         ,
        dateInCdmwTime.tm_msec        ,
        dateInCdmwTime.tm_usec        ,
        startDate);
        System.out.println(operation);
            
        return (
            (dateInCdmwTime.tm_sec   == sec) &&  (dateInCdmwTime.tm_msec  == msec) &&
        (dateInCdmwTime.tm_usec  == usec)
       );    
    }    
    public boolean PCR_1919()
    {

        String operation = "------> start date";
        long startDate = 134274810090000000L;
        cdmw.ossupport.Timeval timevalForDate = cdmw.clock.Time.TimeT2timeval(startDate);
        cdmw.clock.cdmwTime dateInCdmwTime = new cdmw.clock.cdmwTime();
        dateInCdmwTime = cdmw.clock.Util.timeval2cdmwTime(timevalForDate);
        
/*
2008-04-14 17:50:09.000.000000 (134274810090000000) ------> start date
2008-04-14 17:50:09.000.000000 (134274810090000001) ------> added 100 nanos
2008-04-14 17:50:09.000.000001 (134274810090000010) ------> added 1 usec = 10 TimeT (in 100 nanos units)
2008-04-14 17:50:09.000.000010 (134274810090000100) ------> added 10 usec = 100 TimeT
2008-04-14 17:50:09.000.000100 (134274810090001000) ------> added 100 usec = 1000 TimeT
2008-04-14 17:50:09.001.001000 (134274810090010000) ------> added 1 millisecond = 10.000 TimeT
2008-04-14 17:50:09.010.010000 (134274810090100000) ------> added 1 cs = 100.000 TimeT
2008-04-14 17:50:09.100.100000 (134274810091000000) ------> added 1 ds = 1.000.000 TimeT
2008-04-14 17:50:10.000.000000 (134274810100000000) ------> added 1 sec = 10.000.000 TimeT
2008-04-14 17:51:09.000.000000 (134274810690000000) ------> added 1 min = 600.000.000 TimeT
2008-04-14 18:50:09.000.000000 (134274846090000000) ------> added 1 h = 36.000.000.000 TimeT

*/    
     
        checkAndPrintDate(startDate, operation, 9, 0, 0);

        operation = "------> added 100 nanos"; 
        // 2008-04-14 17:50:09.000.000000
        long newDate= startDate+1L;
        if (!checkAndPrintDate(newDate, operation, 9, 0 , 0) )
           return Macro.TEST_NOK;   
        operation = "------> added 1 usec = 10 TimeT (in 100 nanos units)"; 
        newDate= startDate+cdmw.clock.Macro.TIMET_IN_A_USEC;
        if (!checkAndPrintDate(newDate, operation, 9, 0 , 1) )
        return Macro.TEST_NOK;
        operation = "------> added 10 usec = 100 TimeT "; 
        newDate= startDate+10*cdmw.clock.Macro.TIMET_IN_A_USEC;
        if (!checkAndPrintDate(newDate, operation, 9, 0 , 10) )
           return Macro.TEST_NOK;
        operation = "------> added 100 usec = 1000 TimeT"; 
        newDate= startDate+100*cdmw.clock.Macro.TIMET_IN_A_USEC;
        if (!checkAndPrintDate(newDate, operation, 9, 0 , 100) )
            return Macro.TEST_NOK;
        operation = "------> added 1 millisecond = 10.000 TimeT"; 
        newDate= startDate+1000*cdmw.clock.Macro.TIMET_IN_A_USEC;
        if (!checkAndPrintDate(newDate, operation, 9, 1 , 1000) )
            return Macro.TEST_NOK;
        operation = "------> added 1 cs = 100.000 TimeT"; 
        newDate= startDate+10000*cdmw.clock.Macro.TIMET_IN_A_USEC;
        if (!checkAndPrintDate(newDate, operation, 9, 10 ,10000 ) )
            return Macro.TEST_NOK;
        operation = "------> added 1 ds = 1.000.000 TimeT"; 
        newDate= startDate+100000*cdmw.clock.Macro.TIMET_IN_A_USEC;
        if (!checkAndPrintDate(newDate, operation, 9, 100 , 100000) )
            return Macro.TEST_NOK;
        operation = "------> added 1 sec = 10.000.000 TimeT"; 
        newDate= startDate+cdmw.clock.Macro.TIMET_IN_A_SEC;
        if (!checkAndPrintDate(newDate, operation, 10, 0 , 0) )
            return Macro.TEST_NOK;
  
        return Macro.TEST_OK;
    }
    
    public boolean PCR_1223(){
        cdmw.clock.cdmwTime res = new cdmw.clock.cdmwTime();
        res.tm_year = 90;
        res.tm_mon = 0;
        res.tm_mday = 1;
        res.tm_hour = 15;
        res.tm_min = 30;
        res.tm_sec = 0;
        res.tm_msec = 0;
        cdmw.ossupport.Timeval res2 = cdmw.clock.Util.cdmwTime2timeval(res);
        String date1 = res2.toDate().toString();
        res = cdmw.clock.Util.timeval2cdmwTime(res2);
        res2 = cdmw.clock.Util.cdmwTime2timeval(res);
        String date2 = res2.toDate().toString();
        if (!date1.equalsIgnoreCase(date2)) 
            return Macro.TEST_NOK;
        return Macro.TEST_OK;
        
    }

    public boolean createUtcT()
    {
        java.util.Calendar cal = java.util.Calendar.getInstance();
        int offset =  cal.get( java.util.Calendar.ZONE_OFFSET );   // zone difference
            
              
        long inaccuracy = 10000L;
        short tdf = ( short ) ( offset / 60000 );  // millis to minutes
        short inacchi = (short) ((inaccuracy >> 32) &  0xffffL);
        int inacclo = (int) (inaccuracy & 0xffffffffL);

        org.omg.TimeBase.UtcT utct = Time.createUtcT(dateInTimeT);
        if ( (utct.time == dateInTimeT) &&
             (utct.inacclo == inacclo) &&
             (utct.inacchi == inacchi) &&
             (utct.tdf == tdf) )
            return Macro.TEST_OK;
        return Macro.TEST_NOK;
    }

    public boolean toStringTest()
    {
            final String dateSampleInGMT = "2006-09-08 16:07:41.00+0000";
        
            org.omg.TimeBase.UtcT utct = Time.createUtcT(dateInTimeT);
            SimpleDateFormat df = new SimpleDateFormat( "yyyy-MM-dd HH:mm:ss.SSZ");
            df.setTimeZone(TimeZone.getTimeZone("GMT")); // refers to GMT
            String dateInTimeT_GMT 
                = df.format( new java.util.Date(Time.TimeT2timeval(utct.time).getMilliseconds()) );
           
            if (dateInTimeT_GMT.compareTo(dateSampleInGMT) == 0)
                return Macro.TEST_OK;
            return  Macro.TEST_NOK;
    }
    

    public void doTests()
    {
       setNbOfRequestedTestOK(NUMOFTESTS);
      try 
      {
        SimpleDateFormat df = new SimpleDateFormat("EEE MMM dd HH:mm:ss zzz yyyy");

        // Fri Sep 08 18:07:41 CEST 2006
        // TimeT = 133770244610300000L
        Date d = df.parse(dateSample) ;

        Timeval dateInTimeval = new Timeval(d.getTime());
        check(checkTimevalToTimeT (dateInTimeval));
        check(checkTimeTtoTimeval (dateInTimeval));
        check(createUtcT());
        check(toStringTest());
        check(PCR_1224());
        check(PCR_1223());
        check(PCR_1919());
    
      }
      catch (ParseException pe)
      {
      }

    }

}
