/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @brief
 *
 * @author Francesca Annunziata <fannunziata@selex-si.com>
 * @author Raffaele Mele <rmele@progesi.it>
 */

package cdmw.clock.test;

import cdmw.clock.*;
import cdmw.orbsupport.ORBSupport;
import cdmw.orbsupport.ORBUtils;
import cdmw.ossupport.OS;
import cdmw.ossupport.ThreadedProcess;
import cdmw.ossupport.Timeval;
import cdmw.testutils.Testable;

import java.io.IOException;

import org.omg.CORBA.ORB;
import org.omg.CORBA.ORBPackage.InvalidName;
import org.omg.CosClockService.*;
import org.omg.CosClockService.Clock;
import org.omg.CosPropertyService.InvalidPropertyName;
import org.omg.CosPropertyService.PropertyNotFound;
import org.omg.CosPropertyService.PropertySet;
import org.omg.CosPropertyService.PropertySetDef;
import org.omg.CosPropertyService.PropertySetDefFactory;
import org.omg.CosPropertyService.PropertySetHelper;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;
import org.omg.PortableServer.POAManager;
import org.omg.PortableServer.POAManagerPackage.AdapterInactive;

// import for ClockCatalog
import org.omg.CosClockService.ClockCatalogPackage.ClockEntriesHelper;
import org.omg.CosClockService.ClockCatalogPackage.ClockEntry;
import org.omg.CosClockService.ClockCatalogPackage.ClockEntryHolder;
import org.omg.CosClockService.ClockCatalogPackage.UnknownEntry;
// import for Controller
import org.omg.CosClockService.PeriodicExecution.ControllerPackage.TimePast;
import org.omg.CosClockService.PeriodicExecution.Controller;
import Cdmw.clock.ControllerEx;
import Cdmw.clock.ControllerExHelper;

import org.omg.CORBA.Any;

/**
 * @brief
 *
 * @author Francesca Annunziata <fannunziata@selex-si.com>
 * @author Raffaele Mele <rmele@progesi.it>
 */

///////////////////////////////////////////////////////////////////////////////

public class JNIClockTest extends Testable
{
    private ORB orb;

    private static String[] args;
    private cdmw.clock.JNIClock clock;


    private final int       NUMOFTESTS = 2;

///////////////////////////////////////////////////////////////////////////////

    public JNIClockTest (ORB orb, String[] strArgs) 
    {
      this.name = "JNIClockTest";
      this.out = System.out;
      this.orb = orb;
      this.args = strArgs;
      this.clock = new cdmw.clock.JNIClock();

    }

///////////////////////////////////////////////////////////////////////////////

    public boolean currentTimeTest()
    {
        //System.out.println("currentTimeTest");
        try 
        {
            org.omg.CORBA.Object cObj = this.orb.resolve_initial_references("LocalClock");
            org.omg.CosClockService.Clock clockObj =
                org.omg.CosClockService.ClockHelper.narrow(cObj);
            for (int i=0;i<10;i++)
                 System.out.println("current time ->" + clockObj.current_time() );  
        }
        catch (org.omg.CORBA.ORBPackage.InvalidName in)
        {
            System.out.println( "resolve initial reference error::Invalid Name" );
            return Macro.TEST_NOK;
        }
        catch ( org.omg.CosClockService.TimeUnavailable tu)
        {
            System.out.println( "currenTime ERROR : TimeUnavailable" );
            return Macro.TEST_NOK;
        }
        return Macro.TEST_OK; 
    }       
///////////////////////////////////////////////////////////////////////////////

    public boolean executorTest(cdmw.clock.JNIClock clock )
    {
        

        try 
        {
            String[]a = {};
            if (!clock.init(this.orb, a))
                System.out.println("Error : unable to initialize Clock Service");
            
            org.omg.CORBA.Object objLocalClockFromJacORB = this.orb.resolve_initial_references("ClockService");
            org.omg.CosClockService.ClockCatalog clockCatalog =
                org.omg.CosClockService.ClockCatalogHelper.narrow(objLocalClockFromJacORB);
     
            //LocalExecutor
            ClockEntry entry;
            entry = clockCatalog.get_entry("LocalExecutor");
            org.omg.CosClockService.PeriodicExecution.Executor executor = 
            org.omg.CosClockService.PeriodicExecution.ExecutorHelper.narrow(entry.subject);
            
            System.out.println("current time ->" + executor.current_time() );  
            
            MyPeriodicExecution myPeriodic = new MyPeriodicExecution(1);
            if (myPeriodic == null) 
            {
                System.out.println( "myPeriodic is null" );
                return Macro.TEST_NOK;
            }
            
            org.omg.CosClockService.PeriodicExecution.Periodic periodic = myPeriodic._this(orb);
            if (periodic == null) 
            {
                System.out.println( "periodic is null" );
                return Macro.TEST_NOK;
            }
            
            
            org.omg.CosClockService.PeriodicExecution.Controller controller = executor.enable_periodic_execution(periodic);
            Cdmw.clock.ControllerEx myController = Cdmw.clock.ControllerExHelper.narrow(controller);
            
            
            System.out.println("obj_to_string = " + this.orb.object_to_string(myController)); 
            
            if (myController == null) 
            {
                System.out.println( "myController is null" );
                return Macro.TEST_NOK;
            }
            
            System.out.println("executions" + myController.executions());
            
            Any     dummy       = this.orb.create_any();
            
            long period = 100000;
            long with_offset = 0;
            int execution_limit = 100;
                        
            System.out.println("executorTest"); 
            try {
                    myController.start(period,with_offset,execution_limit,dummy);  
                               
            }
            catch (Exception tp)
            {
                    System.out.println(" Exception!");
                    return Macro.TEST_NOK;
            }

            
            System.out.println("try to obtain current time (10 times) ");
            for (int i=0;i<10;i++)
                 System.out.println("current time ->" + executor.current_time() );  

            OS.sleep(20);
    //        clock.close();
       }
        catch (org.omg.CORBA.ORBPackage.InvalidName in)
        {
            System.out.println( "resolve initial reference error::Invalid Name" );
            return Macro.TEST_NOK;

        }

        catch ( UnknownEntry ex ) 
        {
            System.out.println( "ClockCatalog::UnknownEntry" );
            return Macro.TEST_NOK;

        }
        catch ( org.omg.CosClockService.TimeUnavailable tu)
        {
            System.out.println( "currenTime ERROR : TimeUnavailable" );
            return Macro.TEST_NOK;
        }
        catch (java.lang.InterruptedException ex)
        {
            ex.printStackTrace();
            return Macro.TEST_OK;
        }


        return Macro.TEST_OK;    
    }


///////////////////////////////////////////////////////////////////////////////

    public void doTests() 
    {

            
        setNbOfRequestedTestOK(NUMOFTESTS);

        check(executorTest(clock)); 
        check(currentTimeTest()); 

        try
        {
            OS.sleep(10);
            this.clock.close();
        }
                 
        catch (java.lang.InterruptedException ex)
        {
            ex.printStackTrace();
            
        }

    }
}

