/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @brief
 *
 * @author Francesca Annunziata <fannunziata@selex-si.com>
 * @author Raffaele Mele <rmele@progesi.it>
 */

package cdmw.clock.test;

import cdmw.ossupport.OS;
import cdmw.ossupport.ThreadedProcess;

import cdmw.orbsupport.ORBSupport;
import cdmw.orbsupport.ORBUtils;

import java.io.File;
import java.io.IOException;
import java.util.*;

import org.omg.CORBA.ORB;
import org.omg.CORBA.ORBPackage.InvalidName;

import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;
import org.omg.PortableServer.POAManager;
import org.omg.PortableServer.POAManagerPackage.AdapterInactive;

import org.omg.CosPropertyService.PropertySetDefFactory;
import org.omg.CosPropertyService.PropertySetDefFactoryHelper;

import org.omg.CosClockService.ClockCatalog;
import org.omg.CosClockService.ClockCatalogHelper;

import cdmw.testutils.Testable;

import cdmw.orbsupport.StrategyList;
///////////////////////////////////////////////////////////////////////////////

public class MainTest
{
    public final static int SUCCESS = 0;
    public final static int FAILURE = 1;

///////////////////////////////////////////////////////////////////////////////

    public static int run(ORB orb, String testname, String[] args)
    throws AdapterInactive,InvalidName
    {
        //
        // Resolve the Root POA
        //
        org.omg.CORBA.Object obj = orb.resolve_initial_references("RootPOA");
        POA root = POAHelper.narrow(obj);
        POAManager poaManager = root.the_POAManager();

        //
        // Initialize the server so that incoming requests are
        // handled from this point on
        //
        poaManager.activate();

        //Testing Clock
        runTest(orb, testname, args);

        return SUCCESS;

    }

///////////////////////////////////////////////////////////////////////////////

    public static void runTest(ORB orb,String testname, String[] strArgs)
    {
        try
        {
            cdmw.clock.CdmwCosClock.init(orb,strArgs);
            //cdmw.clock.CdmwCosClock.init(orb);
            OS.sleep(5);
            org.omg.CORBA.Object obj  = orb.resolve_initial_references("ClockService");
            ClockCatalog catalog = ClockCatalogHelper.narrow(obj);
            if (catalog == null)
            {
                System.out.println("Error : Unable to resolve clock catalog");
            }

            // create a generic catalog for tests
            GenericCatalog<Testable> testCtg = new GenericCatalog<Testable>();

            // add all tests to the tests catalog
            testCtg.add("CreateMultipleVirtualClockTest",
                        new CreateMultipleVirtualClockTest(orb,catalog,strArgs));

            testCtg.add("ClockTest",
                        new ClockTest(orb));

            testCtg.add("ClockCatalogTest",
                        new ClockCatalogTest(orb, catalog));

            testCtg.add("ControlledClockTest",
                        new ControlledClockTest(orb, catalog));

            testCtg.add("AdditionAndSubtractionTest",
                        new TimevalOperationsTest("checkAdditionSubtractionTest"));

            testCtg.add("EqualityTest",
                        new TimevalOperationsTest("checkEqualityTest"));

            testCtg.add("ComparationTest",
                        new TimevalOperationsTest("checkCompareTest"));

            testCtg.add("TimeInfo",
                        new TimeInfo(orb,catalog,strArgs));

            testCtg.add("ImportExportTimevalTest",
                        new ImportExportTimevalTest());


            testCtg.add("ControllerTest",
                        new ControllerTest(orb, catalog));

            testCtg.add("ExecutorTest",
                        new ExecutorTest(orb, catalog));

            testCtg.add("UnpackTimeTest",
                        new PackUnpackTimeOperationsTest("checkUnpackTimeTest"));
            testCtg.add("PackTimeTest",
                        new PackUnpackTimeOperationsTest("checkPackTimeTest"));

            testCtg.add("UTCTest",
                        new UTCTest());

            testCtg.add("TimeSpanTest",
                        new TimeSpanTest());

            testCtg.add("ControlledExecutorTest",
                        new ControlledExecutorTest(orb, catalog));    

            Testable testToExecute;
            if (testname.equals("ALL"))
            {
                Iterator<Testable> i = testCtg.getIterator();
                while (i.hasNext())
                    i.next().start();
            }
            else if ( (testToExecute = testCtg.search(testname) ) != null)
            {
                testToExecute.start();
            }
            else
            {
                System.out.println("Error: Test " + testname + " not found in tests catalog");
                throw new InvalidName();
            }
        }
        catch (InvalidName in)
        {
            in.printStackTrace();
        }
        catch (java.lang.InterruptedException ex)
        {
            ex.printStackTrace();
        }

    }

///////////////////////////////////////////////////////////////////////////////

    public static void main(String[] args)
    {

        String option = "--TestName";
        String testName = OS.getOptionValue(args,option) ;

        if (testName.equals("no") ||
                testName.equals("")   ||
                testName.equalsIgnoreCase ("ALL")) // default behaviour: execute all tests
        {
            testName = "ALL";
        }

        int status = SUCCESS;

        String[] strArgs= {};

        try
        {
            StrategyList strategy = new StrategyList();
            strategy.addORBThreaded();
//            strategy.addPOAThreadPerRequest();
            
            ORB orb = cdmw.orbsupport.ORBSupport.ORBInit(args, strategy);
            status = run(orb, testName, args /*strArgs*/);
            cdmw.clock.CdmwCosClock.close();

            if (orb != null)
            {
                try
                {

                orb.destroy();
                }
                catch (Exception ex)
                {
                    ex.printStackTrace();
                    status = FAILURE;
                }
            }
                    }
        catch (InvalidName in)
        {
            in.printStackTrace();
            status = FAILURE;
        }
        catch (AdapterInactive ai)
        {
            ai.printStackTrace();
            status = FAILURE;
        }

         if (status == SUCCESS)
         {
             System.out.println("Main test successful.");
         }
         else
         {
             System.out.println("Main test failed.");
         }
        System.exit(1);        
    }
}


