/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.clock.test;
import cdmw.testutils.Testable;
import cdmw.ossupport.Timeval;
import cdmw.clock.*;
import cdmw.ossupport.OS;
import java.text.SimpleDateFormat;
import java.text.ParseException;

import java.util.Date;
import java.util.Calendar;
import java.util.GregorianCalendar;


public class PackUnpackTimeOperationsTest extends Testable
{
    private int NUMOFTESTS;
    private String nameOfTest[] = {"checkUnpackTimeTest","checkPackTimeTest"};
    private boolean doTests[] ={false,false};

    private final String dateSample    = "Mon May 15 05:55:55 CEST 2006";


    public PackUnpackTimeOperationsTest()
    {
      this.name = "PackUnpackTimeOperationsTest";
      this.out = System.out;
      this.NUMOFTESTS=nameOfTest.length;
      this.doTests[0]=true;
      this.doTests[1]=true;
    }

    public PackUnpackTimeOperationsTest(String testname)
    {
      this.name = testname;
      this.out = System.out;
      this.NUMOFTESTS=1;
      try 
      {
         this.doTests[getTestId(testname)]=true;
      }
      catch (Exception e)
      {
      }
    }

    public boolean checkPackTimeTest(Timeval dateInTimeval,cdmwTime input_time) 
    {
       //cdmw.ossupport.Timeval cdmwTime2timeval(cdmwTime input_time)

      
       cdmw.ossupport.Timeval t = Util.cdmwTime2timeval(input_time);
       if (Util.compare(t,dateInTimeval) == 0)
          return Macro.TEST_OK;
       else 
          return Macro.TEST_NOK;
    }
    public boolean checkUnpackTimeTest(cdmwTime dateInCdmwTime ,cdmw.ossupport.Timeval input_time) 
    {
       
       cdmw.clock.cdmwTime t = Util.timeval2cdmwTime(input_time);
       if (  dateInCdmwTime.tm_year  ==  t.tm_year   &&
             dateInCdmwTime.tm_mon   ==  t.tm_mon    && 
             dateInCdmwTime.tm_mday  ==  t.tm_mday   && 
             dateInCdmwTime.tm_hour  ==  t.tm_hour   && 
             dateInCdmwTime.tm_min   ==  t.tm_min    && 
             dateInCdmwTime.tm_sec   ==  t.tm_sec    && 
             dateInCdmwTime.tm_wday  ==  t.tm_wday   && 
             dateInCdmwTime.tm_yday  ==  t.tm_yday   && 
             dateInCdmwTime.tm_isdst ==  t.tm_isdst  && 
             dateInCdmwTime.tm_msec  ==  t.tm_msec   ) 
       return Macro.TEST_OK;
       else 
          return Macro.TEST_NOK;
    }
    public void doTests()
    {
        setNbOfRequestedTestOK(NUMOFTESTS);

        SimpleDateFormat df = new SimpleDateFormat("EEE MMM dd HH:mm:ss zzz yyyy");

        try
        {

         Date d = df.parse(dateSample) ;
         GregorianCalendar output_date = new GregorianCalendar();
         output_date.setTimeInMillis(d.getTime());
         cdmwTime dateInCdmwTime = new cdmwTime();
         dateInCdmwTime.tm_year  =  output_date.get(Calendar.YEAR) - 1900;
         dateInCdmwTime.tm_mon   =  output_date.get(Calendar.MONTH);
         dateInCdmwTime.tm_mday  =  output_date.get(Calendar.DAY_OF_MONTH);
         dateInCdmwTime.tm_hour  =  output_date.get(Calendar.HOUR);
         dateInCdmwTime.tm_min   =  output_date.get(Calendar.MINUTE);
         dateInCdmwTime.tm_sec   =  output_date.get(Calendar.SECOND);
         dateInCdmwTime.tm_wday  =  output_date.get(Calendar.DAY_OF_WEEK);
         dateInCdmwTime.tm_yday  =  output_date.get(Calendar.DAY_OF_YEAR);
         dateInCdmwTime.tm_isdst =  output_date.get(Calendar.DST_OFFSET);
         dateInCdmwTime.tm_msec  =  output_date.get(Calendar.MILLISECOND);

         Timeval dateInTimeval = new Timeval((df.parse(dateSample)).getTime());
         if (doTests[getTestId("checkPackTimeTest")]) check(checkPackTimeTest(dateInTimeval,dateInCdmwTime));
         if (doTests[getTestId("checkUnpackTimeTest")]) check(checkUnpackTimeTest(dateInCdmwTime,dateInTimeval));
         
        }
        catch (ParseException e)
        {
        }
        catch (Exception e)
        {
        }
    }
    private int getTestId(String name) 
       throws Exception
    {
       for (int index = 0; index<nameOfTest.length ; index++)
          if ((nameOfTest[index]).compareTo(name) == 0) 
             return index;
       throw new Exception("Test doesn't exsist!");
    }

}
