/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.clock.test;

import cdmw.clock.*;
import cdmw.orbsupport.ORBSupport;
import cdmw.ossupport.OS;
import org.omg.CORBA.ORB;
import cdmw.testutils.Testable;
import cdmw.clock.test.Macro;
import org.omg.CosClockService.ClockCatalog;
import org.omg.CosClockService.ClockCatalogHelper;
import org.omg.CosClockService.ClockCatalogPackage.ClockEntry;
import org.omg.CosClockService.ClockCatalogOperations;
import org.omg.CosClockService.UTCHelper;
import org.omg.CosClockService.UtcTimeService;
import org.omg.CosClockService.UTC;
import org.omg.CosClockService.UTCValueFactory;
import org.omg.TimeBase.*;


public class TimeInfo extends Testable
{
    private ORB orb;
    private static String[] args;
    private ClockCatalog catalog;
    private UtcTimeService utcClock;
    private UTC utc;
    long time;
    int inacclo = 1;
    short word0 = 2;
    short word1 = 3;
    
    public TimeInfo (ORB theOrb,ClockCatalog catalog, String[] args)
    {
        this.name = "TimeInfoTest";
        this.out = System.out;
        this.orb = theOrb;
        this.catalog = catalog;
        this.args = args;
        long time = OS.getTime().getMicroseconds();
        //int inacclo = 1;
        //short word0 = 2;
        //short word1 = 3;
        UTCValueFactory utcFactory = new UTCValueFactoryImpl();
        this.utc = utcFactory.compose(time,inacclo,word0,word1);
        ClockEntry entry;
        try
        {
            entry = catalog.get_entry("UTCTimeService");
            this.utcClock =
                org.omg.CosClockService.UtcTimeServiceHelper.narrow(entry.subject);
        }
        catch (org.omg.CosClockService.ClockCatalogPackage.UnknownEntry ue)
        {
            System.out.println("Exception Throw: TimeInfo");
            ue.printStackTrace();
        }
    }

    public void doTests()
    {
        setNbOfRequestedTestOK(3);
        check(absolute_time_test(utc));
        check(secure_universal_time_test());
        check(universal_time_test());
    }


    public boolean absolute_time_test(UTC utc)
    {
        try
        { 
            int i = 50;
            while (--i!=0) 
            {
                UTC utc1;
                
                UTCValueFactory utcFactory = new UTCValueFactoryImpl();
                time = OS.getTime().getMicroseconds();
                utc1 = utcFactory.compose(time,inacclo,word0,word1);                
                UTC timer;
                timer = utcClock.absolute_time(utc1); 
              
                if (timer.time>0)               
                          return Macro.TEST_OK;
                 else 
                        {
                          System.out.println("absolute_time <= 0");
                          return Macro.TEST_NOK;
                         } 
            }
        }
        catch (org.omg.CosClockService.TimeUnavailable na)
        {
            System.out.println("Exception Throw: TimeInfo");
            na.printStackTrace();
            return Macro.TEST_NOK;
        }
        catch (Exception e)
        {
            e.printStackTrace();
            return Macro.TEST_NOK;
        }

        return Macro.TEST_OK;
    }

    public boolean secure_universal_time_test()
    {
        try
        {
            utcClock.secure_universal_time();
        }
        catch (org.omg.CORBA.NO_IMPLEMENT nisecure_universal_time)
        {
            System.out.println("Warning (NO_IMPLEMENT): UtcTimeService.secure_universal_time is avaiable only at API level");
            return Macro.TEST_OK;
        }
        catch (org.omg.CosClockService.TimeUnavailable na)
        {
            System.out.println("Exception Throw: TimeInfo");
            na.printStackTrace();
            return Macro.TEST_NOK;
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }

        return Macro.TEST_OK;
    }

    public boolean universal_time_test()
    {
        try
        {
            UTC timer;
            long tmp, tmp2, delta;    
            timer = utcClock.universal_time();
            tmp= timer.time;
            
            int i = 100;
            while (--i!=0)
            {
                timer = utcClock.universal_time();
                tmp2=timer.time;
                delta=tmp2-tmp;
                if(tmp2>tmp)
                    return Macro.TEST_OK;
                                
                        if (PerformanceManager.is_active_=true)
                {
                    if (tmp2-tmp<=1000000/*10usec*/)
                    {
                        System.out.println("PERFORMANCE ASSERT");
                        System.out.println("\ntmp2-tmp <=1000000\n");
                    }
                    //PERFORMANCE_ASSERT(tmp2-tmp<=1000000/*10usec*/);
                    else
                    {
                        System.out.println("PERFORMANCE ASSERT");
                        System.out.println("\ntmp2-tmp >=1000000\n");
                    }
                }
                        tmp2=tmp;
            }
            return Macro.TEST_NOK;
            
        }
        catch (org.omg.CORBA.NO_IMPLEMENT niuniversal_time)
        {
            System.out.println("Warning (NO_IMPLEMENT): UtcTimeService.universal_time is avaiable only at API level");
            return Macro.TEST_OK;
        }
        catch (org.omg.CosClockService.TimeUnavailable na)
        {
            System.out.println("Exception Throw: TimeInfo");
            na.printStackTrace();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        return Macro.TEST_OK;
    }
}


