/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.clock.test;
import cdmw.testutils.Testable;
import cdmw.ossupport.Timeval;
import cdmw.clock.*;
import cdmw.ossupport.OS;
import java.text.SimpleDateFormat;
import java.text.ParseException;

import java.util.Date;



public class TimevalOperationsTest extends Testable
{
 
    public enum tests{
      checkAdditionSubtractionTest,
      checkEqualityTest,
      checkCompareTest
    };

    private int       NUMOFTESTS = 3;
    private boolean doIt[] = new boolean[NUMOFTESTS];
    private final String dateSample    = "Mon May 15 05:55:55 CEST 2006";
    private final String oneYearAfter  = "Tue May 15 05:55:55 CEST 2007";
    private final String oneYearBefore = "Sun May 15 05:55:55 CEST 2005";
    
    public TimevalOperationsTest() 
    {    
      this.name = "TimevalOperationsTest";
      this.out = System.out;
      init();
    }
    public TimevalOperationsTest(String name) 
    {    
      this.name = name;
      this.out = System.out;
      this.NUMOFTESTS=1;
      this.activated(name);
    }

    public void activated(String name) 
      {
         for (tests m : tests.values())
         {
            if ((m.name()).equals(name))
               doIt[m.ordinal()] = true;
            else  doIt[m.ordinal()] = false;
         }
      }

    private void init () 
    {
      for (tests m : tests.values())
        doIt[m.ordinal()] = true;

    }
    

    /**
     * @brief check Subtraction operation
     *        add to date timeval corresponding to 1 year
     *        the test checks that result of operation is equal to
     *        date corresponding to following year
     * 
     **/
    public boolean checkAdditionTest(cdmw.ossupport.Timeval tv1, cdmw.ossupport.Timeval tv2) 
    {
        if ( (Util.add(tv1,tv2).toString()).equals(oneYearAfter) )
          return Macro.TEST_OK;
        else
          return Macro.TEST_NOK;

    }
    
    /**
     * @brief check Subtraction operation
     *        subtract to date timeval corresponding to 1 year
     *        the test checks that result of operation is equal to
     *        date corresponding to previous year
     * 
     **/

    public boolean checkSubtractionTest(cdmw.ossupport.Timeval tv1, cdmw.ossupport.Timeval tv2) 
    {
        if ( (Util.subtract(tv1,tv2).toString()).equals(oneYearBefore) )
           return Macro.TEST_OK;
        else
          return Macro.TEST_NOK;

    }
    public boolean checkEqualityTest(cdmw.ossupport.Timeval date, cdmw.ossupport.Timeval oneYear, cdmw.ossupport.Timeval dateOneYearAfter) 
    {
        cdmw.ossupport.Timeval dateToCheck = Util.add(date,oneYear);  // one year after

        if (Util.compare(dateOneYearAfter,dateToCheck) == 0 ) 
           return Macro.TEST_OK;
        else
           return Macro.TEST_NOK;
    }
    public boolean checkCompareTest(cdmw.ossupport.Timeval date, cdmw.ossupport.Timeval oneYear) 
    {
        if (
           (Util.compare(date,Util.add(date,oneYear)) < 0 ) &&        // one year after
           (Util.compare(date,Util.subtract(date,oneYear)) > 0 ) )      // one year before
           return Macro.TEST_OK;
        else
           return Macro.TEST_NOK;
    }

    public void doTests()
    {
       setNbOfRequestedTestOK(NUMOFTESTS);

       SimpleDateFormat df = new SimpleDateFormat("EEE MMM dd HH:mm:ss zzz yyyy");
       try 
       { 
           cdmw.ossupport.Timeval date = new Timeval((df.parse(dateSample)).getTime());
           cdmw.ossupport.Timeval dateOneYearAfter  = new Timeval((df.parse(oneYearAfter)).getTime());
           cdmw.ossupport.Timeval dateOneYearBefore = new Timeval((df.parse(oneYearBefore)).getTime());
           System.out.println(date.toString());
           cdmw.ossupport.Timeval oneYear = new Timeval(86400*365,0);  // 1 year

           if (doIt[tests.checkAdditionSubtractionTest.ordinal()])
           { 
                setNbOfRequestedTestOK(NUMOFTESTS + 1);
                check(checkAdditionTest(date,oneYear));
                check(checkSubtractionTest(date,oneYear));
            }
            if (doIt[tests.checkEqualityTest.ordinal()])    check(checkEqualityTest(date,oneYear,dateOneYearAfter));
            if (doIt[tests.checkCompareTest.ordinal()])     check(checkCompareTest(date,oneYear));
        }
        catch (ParseException e)
        {
        } 
    }
}

