/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.clock.test;

import  cdmw.clock.*;
import cdmw.orbsupport.ORBSupport;
import cdmw.ossupport.OS;
import org.omg.CORBA.ORB;
import cdmw.testutils.Testable;

import org.omg.TimeBase.UtcT;
import org.omg.CosClockService.UtcTimeService;
import org.omg.CosClockService.UTC;
import org.omg.CosClockService.UTCValueFactory;
import org.omg.CosClockService.TimeSpan;
import org.omg.CosClockService.TimeSpanValueFactory;
import org.omg.CosClockService.*;

/**
* @file UTCTest.java
* @brief unit tests for EVoT UTC valuetype.
* 
* @author Raffaele Mele <rmele@progesi.it>
*/

public class UTCTest extends Testable
{
    private UTCValueFactory utcFactory;
    private TimeSpanValueFactory tspanFactory;

    public UTCTest ()
    {
        this.name = "UTCTest";
        this.out = System.out;
        utcFactory = new UTCValueFactoryImpl();
        tspanFactory = new TimeSpanValueFactoryImpl();
    }

    public void doTests()
    {
        setNbOfRequestedTestOK(6);
        check(inaccuracy());
        check(utc_time());
        check(compare_time());
        check(interval());
        check(factory_init());
        check(factory_compose());
    }


    public boolean inaccuracy()
    {
        long time = 6000;
        int inacclo = 4563,
                      inacclo_;
        short inacchi= 2671,
                       inacchi_;
        short tdf = 0;

        UTC utc = utcFactory.compose(time,inacclo,inacchi,tdf);
        long inaccuracy = utc.inaccuracy();
        long res = inaccuracy & Macro.Mask48;

        res&=Macro.Mask32;

        inacclo_ = (int)res;

        res = inaccuracy >> 32;
        res &= Macro.Mask16;

        inacchi_ = (short)res;

        if (( inacclo != inacclo_ ) ||
                ( inacchi != inacchi_ ))
            return Macro.TEST_NOK;
        return Macro.TEST_OK;
    }

    public boolean utc_time()
    {
        long time = 1234;
        int inacclo = 4321;
        short inacchi= 9876;
        short tdf = 3;

        UTC utc = utcFactory.compose(time,inacclo,inacchi,tdf);

        UtcT utc_time = utc.utc_time();

        if (( utc.time != utc_time.time )
            || ( utc.inacclo != utc_time.inacclo )
            || ( utc.inacchi != utc_time.inacchi )
            || ( utc.tdf != utc_time.tdf ))
            return Macro.TEST_NOK;
        return Macro.TEST_OK;
    }

    public boolean compare_time()
    {
        // create ad-hoc TimeSpans for this test
        TimeSpan tspan1 = tspanFactory.compose(3000,9000);
        TimeSpan tspan2 = tspanFactory.compose(3000,9000);

        // get UTCs from the Timespans
        UTC utc1 = tspan1.time(); 
        UTC utc2 = tspan2.time(); 
        // boolean flag to verify all the test
        boolean testOk = true;

        // test MidC Comparison for TCEqualTo utc
        TimeComparison res = utc1.compare_time(ComparisonType.MidC,utc2);
        if (res != TimeComparison.TCEqualTo)
            testOk = false;
        res = utc2.compare_time(ComparisonType.MidC,utc1);
        if (res != TimeComparison.TCEqualTo)
            testOk = false;

        // test UTC MidC Comparison for TCGreatherThan utc
        tspan2.lower_bound = 2000;
        tspan2.upper_bound = 8000;
        utc2 = tspan2.time();
        res = utc1.compare_time(ComparisonType.MidC,utc2);
        if (res != TimeComparison.TCGreaterThan)
            testOk = false;

        // test UTC MidC Comparison for TCLessThan
        res = utc2.compare_time(ComparisonType.MidC,utc1);
        if (res != TimeComparison.TCLessThan)
            testOk = false;

        // test UTC IntervalC Comparison for TCGreatherThan
        tspan2.lower_bound = 1000;
        tspan2.upper_bound = 2500;
        utc2 = tspan2.time();
        res = utc1.compare_time(ComparisonType.IntervalC,utc2);       
        if (res != TimeComparison.TCGreaterThan)
            testOk = false;

        // test UTC IntervalC Comparison for TCLessThan
        res = utc2.compare_time(ComparisonType.IntervalC,utc1);
        if (res != TimeComparison.TCLessThan)
            testOk = false;

        // test UTC IntervalC Comparison for TCIndeterminate
        tspan2.lower_bound = 1000;
        tspan2.upper_bound = 3500;
        utc2 = tspan2.time();

        res = utc1.compare_time(ComparisonType.IntervalC,utc2);
        if (res != TimeComparison.TCIndeterminate)
            testOk = false;

        // test UTC IntervalC Comparison for TCIndeterminate
        res = utc2.compare_time(ComparisonType.IntervalC,utc1);
        if (res != TimeComparison.TCIndeterminate)
            testOk = false;

        // test UTC IntervalC Comparison for TCIndeterminate
        tspan2.lower_bound = 8000;
        tspan2.upper_bound = 10000;
        utc2 = tspan2.time();
        res = utc1.compare_time(ComparisonType.IntervalC,utc2);
        if (res != TimeComparison.TCIndeterminate)
            testOk = false;
          
        res = utc2.compare_time(ComparisonType.IntervalC,utc1);
        if (res != TimeComparison.TCIndeterminate)
            testOk = false;

        //if almost one test fail
        if (testOk == false)
            return Macro.TEST_NOK;
        return Macro.TEST_OK;
    }

    public boolean interval()
    {
        long time = 1234;
        int inacclo = 4321;
        short inacchi= 9876;
        short tdf = 3;

        UTC utc = utcFactory.compose (time, inacclo, inacchi, tdf);

        TimeSpan tspan = utc.interval();
        if ( (tspan.lower_bound != utc.time - utc.inaccuracy())
                ||   (tspan.upper_bound != utc.time + utc.inaccuracy()) )
            return Macro.TEST_NOK;
        return Macro.TEST_OK;
    }

    public boolean factory_init ()
    {
        long time = 2000;
        int inacclo = 500;
        short inacchi= 20;
        short tdf = 3;

        UtcT utc_time = new UtcT(time, inacclo, inacchi, tdf);

        UTC utc = utcFactory.init (utc_time);

        if (( utc.time != utc_time.time )
                || ( utc.inacclo != utc_time.inacclo )
                || ( utc.inacchi != utc_time.inacchi )
                || ( utc.tdf != utc_time.tdf ))
            return Macro.TEST_NOK;
        return Macro.TEST_OK;
    }

    public boolean factory_compose ()
    {
        long time = 2222;
        int inacclo = 555;
        short inacchi= 22;
        short tdf = 2;

        UTC utc = utcFactory.compose (time, inacclo,inacchi,tdf);

        if (( utc.time != time )
                || ( utc.inacclo != inacclo )
                || ( utc.inacchi != inacchi )
                || ( utc.tdf != tdf ))
            return Macro.TEST_NOK;
        return Macro.TEST_OK;
    }
}
