/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef NTP_TEST_HPP
#define NTP_TEST_HPP

#include <string>
#include <sstream>
#include <iostream>

#include <cppunit/extensions/HelperMacros.h>

#include "Time/clockservice/test/testcommon/PerformanceManager.hpp"
#include "Time/clockservice/test/testcommon/TimeTestManager.hpp"
#include "clockservice/config.hpp"
#include "Time/clockservice/ClockNtp.hpp"
#include "clockservice/Clock_impl.hpp"
#include "clockservice/ClockCatalog_impl.hpp"

/**
 * @brief Class to test ClockCatalog method.
 *
 * @author Raffaele Mele <rmele@progesi.it>,
 */
class NtpClockTest
:   public CppUnit::TestFixture
{
    CPPUNIT_TEST_SUITE( NtpClockTest );
        CPPUNIT_TEST( test_catalog_entry );
        CPPUNIT_TEST( test_host_syncro );
        CPPUNIT_TEST( test_time_syncro );
    CPPUNIT_TEST_SUITE_END( );

protected:

    CORBA::Object_ptr
        m_obj1,
        m_obj2;

    CosClockService::Clock_var
            m_clock;

    CosClockService::ClockCatalog_var
            m_clock_catalog;

    std::string
            m_master_name,
            m_clock_name;

public:
    /**
     * @brief class Ctor.
     */
    NtpClockTest();

    /**
     * @brief Tests setup.
     */
    void  setUp();

    /**
     * @brief test clock service synchronization.
     */
    void  test_time_syncro();

    /**
     * @brief test host synchronization.
     */
    void  test_host_syncro();

    /**
     * @brief test NTP entry in Clock Catalog.
     */
    void  test_catalog_entry();

    /**
     * @brief Finalize tests.
     */
    void  tearDown() ;
};

#endif // NTP_TEST_HPP

