/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @author Raffaele Mele <rmele@progesi.it>
 */

#include "Time/clockservice/test/testcommon/NtpTest.hpp"

using namespace CosClockService;
using namespace CosPropertyService;
using namespace Cdmw::clock;

///////////////////////////////////////////////////////////////////////////////

CPPUNIT_TEST_SUITE_REGISTRATION( NtpClockTest );
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(NtpClockTest, "testclockservice");

///////////////////////////////////////////////////////////////////////////////

NtpClockTest::NtpClockTest()
{
    m_clock_name = "ClockService";
    m_master_name = "MasterClock";

    Cdmw::TestUtils::CORBATestManager::instance()->add(m_clock_name);
    if (Cdmw::clock::util::check_ntp_syncronization())
        Cdmw::TestUtils::CORBATestManager::instance()->add(m_master_name);

    try
    {
        if (Cdmw::clock::util::check_ntp_syncronization())
            m_obj1 = Cdmw::TestUtils::CORBATestManager::instance()->get_object(m_master_name);
        m_obj2 = Cdmw::TestUtils::CORBATestManager::instance()->get_object(m_clock_name);
    }
    catch ( Cdmw::TestUtils::CORBATestManager::InvalidObject& )
    {
        CPPUNIT_FAIL("Cdmw::TestUtils::CORBATestManager::InvalidObject");
    }

    if (Cdmw::clock::util::check_ntp_syncronization())
        m_clock = CosClockService::Clock::_narrow(m_obj1);
    m_clock_catalog = CosClockService::ClockCatalog::_narrow(m_obj2);

    CPPUNIT_ASSERT(!CORBA::is_nil(m_clock_catalog.in() ) );
}
///////////////////////////////////////////////////////////////////////////////

void
NtpClockTest::setUp()
{
    // pre-requirement for all tests
//    check_ntp();
}
///////////////////////////////////////////////////////////////////////////////

void
NtpClockTest::tearDown()
{
}
///////////////////////////////////////////////////////////////////////////////

void
NtpClockTest::test_catalog_entry()
{
    bool exc = true;
    // check for "MasterClock" entry in Clock Catalog
    try
    {
        m_clock_catalog->get_entry("MasterClock");
    }
    catch( CosClockService::ClockCatalog::UnknownEntry& )
    {
        exc = false;
    }

    CPPUNIT_ASSERT(exc);
}
///////////////////////////////////////////////////////////////////////////////

void
NtpClockTest::test_host_syncro()
{
    CPPUNIT_ASSERT(Cdmw::clock::util::check_ntp_syncronization());
}
///////////////////////////////////////////////////////////////////////////////

void
NtpClockTest::test_time_syncro()
{

    CPPUNIT_ASSERT(Cdmw::clock::util::check_ntp_syncronization());
    CPPUNIT_ASSERT(!CORBA::is_nil(m_clock.in() ) );

    TimeBase::TimeT
    clock_time = m_clock->current_time();

    // update base of time for windows compatibility
    clock_time -= TIME_BASE;

    TimeBase::TimeT
    tmp_time;

    struct timeval   tv;
    struct timezone  tz;

    if ( gettimeofday( &tv, &tz ) !=0 )
        CPPUNIT_FAIL("Error in retrieving system time");

    tmp_time =
    (TimeBase::TimeT)(tv.tv_sec ) * 10000000 +
    (TimeBase::TimeT)(tv.tv_usec) * 10 ;

    // all computation are done for East
    TimeBase::TimeT
    offset_in_TimeT = (TimeBase::TimeT)(abs(tz.tz_minuteswest))*600000000;

    // values are computed for West then it is necessary to
    // verify the sign
    if ( tz.tz_minuteswest > 0 )
        tmp_time -= offset_in_TimeT;  // West
    else
        tmp_time += offset_in_TimeT;  // East

    PERFORMANCE_ASSERT(tmp_time >= clock_time);

    TimeBase::TimeT diff = tmp_time - clock_time;
    //   std::cerr<<"Diff: "<<diff<<std::endl;
    TimeBase::TimeT tol = (TimeBase::TimeT)100000; // tolerance of 10 usec

    PERFORMANCE_ASSERT(diff < tol );
}
///////////////////////////////////////////////////////////////////////////////
//  End Of File
///////////////////////////////////////////////////////////////////////////////

