/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef TEST_CONTROLLED_CLOCK_NTP_SYNCRO
#define TEST_CONTROLLED_CLOCK_NTP_SYNCRO

#include <string>
#include <sstream>
#include <iostream>

#include <cppunit/extensions/HelperMacros.h>

#include "Time/clockservice/test/testcommon/PerformanceManager.hpp"
#include "Time/clockservice/test/testcommon/TimeTestManager.hpp"
#include "clockservice/config.hpp"
#include "Time/clockservice/ClockNtp.hpp"
#include "clockservice/Clock_impl.hpp"
#include "clockservice/ClockCatalog_impl.hpp"

/**
 * @brief Class to test ControlledClock synchronization with NTP
 *
 * @author Raffaele Mele <rmele@progesi.it>,
 */
class ControlledClock_NtpSyncroTest
:   public CppUnit::TestFixture
{
    CPPUNIT_TEST_SUITE( ControlledClock_NtpSyncroTest );
        CPPUNIT_TEST( test_host_syncro );
        CPPUNIT_TEST( test_time_syncro );
    CPPUNIT_TEST_SUITE_END( );

protected:

    CORBA::Object_ptr
        m_obj_cks,
        m_obj_ctr,
        m_obj_ntp;

    CosClockService::Clock_var
            m_master_clock;

    CosClockService::ControlledClock_var
            m_controlled_clock;

    CosClockService::ClockCatalog_var
            m_clock_catalog;

    std::string
            m_master_name,
            m_clock_name,
            m_controlled_name;

public:
    /**
     * @brief class Ctor.
     */
    ControlledClock_NtpSyncroTest();

    /**
     * @brief Tests setup.
     */
    void  setUp();

    /**
     * @brief test ControlledClock synchronization with NTP
     */
    void  test_time_syncro();

    /**
     * @brief test NTP host synchronization.
     */
    void  test_host_syncro();

    /**
     * @brief Finalize tests.
     */
    void  tearDown() ;
};

#endif // TEST_CONTROLLED_CLOCK_NTP_SYNCRO

