/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @author Raffaele Mele <rmele@progesi.it>
 */

#include "testcontrolledexecutor/ControlledClock_NtpSyncroTest.hpp"

using namespace CosClockService;
using namespace CosPropertyService;
using namespace Cdmw::clock;

///////////////////////////////////////////////////////////////////////////////

CPPUNIT_TEST_SUITE_REGISTRATION( ControlledClock_NtpSyncroTest );
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(ControlledClock_NtpSyncroTest, "testcontrolledexecutor");

///////////////////////////////////////////////////////////////////////////////

ControlledClock_NtpSyncroTest::ControlledClock_NtpSyncroTest()
{
    m_clock_name = "ClockService";
    m_master_name = "MasterClock";

    Cdmw::TestUtils::CORBATestManager::instance()->add(m_clock_name);
    if (Cdmw::clock::util::check_ntp_syncronization())
        Cdmw::TestUtils::CORBATestManager::instance()->add(m_master_name);

    try
    {
        if (Cdmw::clock::util::check_ntp_syncronization())
            m_obj_ntp = Cdmw::TestUtils::CORBATestManager::instance()->get_object(m_master_name);
        m_obj_cks = Cdmw::TestUtils::CORBATestManager::instance()->get_object(m_clock_name);

    }
    catch ( Cdmw::TestUtils::CORBATestManager::InvalidObject& )
    {
        CPPUNIT_FAIL("Cdmw::TestUtils::CORBATestManager::InvalidObject");
    }
    // retrieve MasterClock
    if (Cdmw::clock::util::check_ntp_syncronization())
        m_master_clock = CosClockService::Clock::_narrow(m_obj_ntp);

    m_clock_catalog = CosClockService::ClockCatalog::_narrow(m_obj_cks);
    CPPUNIT_ASSERT(!CORBA::is_nil(m_clock_catalog.in() ) );

    ClockCatalog::ClockEntry_var clockEntry =
        m_clock_catalog->get_entry("ControlledClock");

    // retrieve Controlled Clock
    m_controlled_clock =
        ControlledClock::_narrow(clockEntry->subject.in());

    CPPUNIT_ASSERT(!CORBA::is_nil(m_controlled_clock.in() ) );
}
///////////////////////////////////////////////////////////////////////////////

void
ControlledClock_NtpSyncroTest::setUp()
{
    // pre-requirement for all tests
//    check_ntp();
}
///////////////////////////////////////////////////////////////////////////////

void
ControlledClock_NtpSyncroTest::tearDown()
{
}
///////////////////////////////////////////////////////////////////////////////

void
ControlledClock_NtpSyncroTest::test_host_syncro()
{
    CPPUNIT_ASSERT(Cdmw::clock::util::check_ntp_syncronization());
}
///////////////////////////////////////////////////////////////////////////////

void
ControlledClock_NtpSyncroTest::test_time_syncro()
{

    CPPUNIT_ASSERT(Cdmw::clock::util::check_ntp_syncronization());
    CPPUNIT_ASSERT(!CORBA::is_nil(m_controlled_clock.in() ) );


    // ControlledClock follow system clock
    m_controlled_clock->set_rate(1.0);

    TimeBase::TimeT
    controlled_time = m_controlled_clock->current_time();

    TimeBase::TimeT
    ntpclock_time = m_master_clock->current_time();

    PERFORMANCE_ASSERT(ntpclock_time >= controlled_time);

    TimeBase::TimeT diff = ntpclock_time - controlled_time;
    //   std::cerr<<"Diff: "<<diff<<std::endl;
    TimeBase::TimeT tol = (TimeBase::TimeT)100000; // tolerance of 10 usec

    PERFORMANCE_ASSERT(diff < tol );
}
///////////////////////////////////////////////////////////////////////////////
//  End Of File
///////////////////////////////////////////////////////////////////////////////

