/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef CONTROLLERCANCELALLPOLICYTEST_HPP
#define CONTROLLERCANCELALLPOLICYTEST_HPP
/**
 * 
 * @author Francesca Annunziata <fannunziata@selex-si.com>
 * @author Fabrizio Morciano <fmorciano@selex-si.com>
 * @author Marco Macellari <mmacellari@progesi.it>
 */

#include <vector>
#include <cppunit/extensions/HelperMacros.h>
#include "Time/idllib/CdmwTime.stub.hpp"
#include <functional>

class controller_functor : public std::unary_function<Cdmw::clock::ControlledExecutor_ptr, void> 
{
    public:
        typedef enum { set, set_rate, pause, resume, terminate } executor_command;

        virtual ~controller_functor()
        {
        }

        virtual executor_command get_executor_command() = 0;

        virtual void operator()(Cdmw::clock::ControlledExecutor_ptr executor) = 0;
};

class ControllerFunctorManager
{
    public:
        ControllerFunctorManager(Cdmw::clock::ControlledExecutor_ptr ref, bool delete_functors)
            : m_executor(Cdmw::clock::ControlledExecutor::_duplicate(ref)),
              m_delete_functors(delete_functors)
        {
        }

        ~ControllerFunctorManager()
        {
            CORBA::release(m_executor);
            if (m_delete_functors == true)
                for (size_t i = 0; i < m_functors.size(); ++i)
                    delete m_functors[i];
        }

        void add_controller_functor(controller_functor* functor)
        {
            m_functors.push_back(functor);
        }

        size_t get_functors_size() const
        {
            return m_functors.size();
        }

        controller_functor* get_functor(const size_t index) const
        {
            return m_functors[index];
        }

        Cdmw::clock::ControlledExecutor_ptr get_executor() const
        {
            Cdmw::clock::ControlledExecutor_var executor(Cdmw::clock::ControlledExecutor::_duplicate(m_executor));
            return executor._retn();
        }
    private:
        Cdmw::clock::ControlledExecutor_ptr m_executor;
        std::vector<controller_functor*>    m_functors;
        bool                                m_delete_functors;
};

class assertion_functor : public std::binary_function<std::vector<unsigned int>, std::vector<unsigned int>, bool> 
{
    public:
        virtual ~assertion_functor()
        {
        }

        virtual bool operator()(const std::vector<unsigned int>& v1, const std::vector<unsigned int>& v2) = 0;
};

/**
 * @brief Class to test ControllerUpdateAdmin method.
 */
class ControllerCancelAllPolicyTest 
:   public CppUnit::TestFixture
{
    CPPUNIT_TEST_SUITE( ControllerCancelAllPolicyTest );
       CPPUNIT_TEST( on_set );
       CPPUNIT_TEST( on_set_rate );
       //CPPUNIT_TEST( on_pause );
       CPPUNIT_TEST( on_pause_resume );
       CPPUNIT_TEST( on_terminate );
    CPPUNIT_TEST_SUITE_END();

protected:

    Cdmw::clock::policy::ControllerUpdateAdmin_var     m_admin;
    CosClockService::PeriodicExecution::Controller_var m_controller;
    CosClockService::ControlledClock_var               m_controlledClock;
    CORBA::Object_var                                  m_obj;
    std::string                                        m_alias_name;

private:

    void checkPolicyForMethods(const ControllerFunctorManager& cfm);
    
public:

    ControllerCancelAllPolicyTest();
    void setUp();
    void tearDown() ;
    void on_set();
    void on_set_rate();
    //void on_pause();
    void on_pause_resume();
    void on_terminate();
};

#endif // CONTROLLERCANCELALLPOLICYTEST_HPP

