/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @brief _implementation for ControllerEnforceDeadlinePolicyTest with cppUnit library.
 *
 * @author Francesca Annunziata<fannunziata@selex-si.com>, 
 * @author Fabrizio Morciano <fmorciano@selex-si.com>
 * @author Marco Macellari <mmacellari@progesi.it>
 */

#include "testpolicies/ControllerEnforceDeadlinePolicyTest.hpp"
#include "clockservice/ControllerUpdateAdmin_impl.hpp"

#include "Foundation/ossupport/OS.hpp"

#include <string>
#include <iostream>
#include <sstream>
#include <vector>

#include "Time/clockservice/test/testcommon/TimeTestManager.hpp"
#include "Time/clockservice/test/testcommon/MyPeriodicExecution.hpp"
#include "Time/clockservice/test/testcommon/anExecution.hpp"
#include "Time/clockservice/Util.hpp"

#include "testpolicies/EnforceDeadlinePolicyAsserter.hpp"
#include "testpolicies/Engine.hpp"
#include "testpolicies/PathsArchive.hpp"

using namespace CosClockService;
using namespace CosPropertyService;
using namespace Cdmw::clock;
using namespace Cdmw::clock::util;
using namespace CosClockService::PeriodicExecution;
using namespace std;

///////////////////////////////////////////////////////////////////////////////

CPPUNIT_TEST_SUITE_REGISTRATION(ControllerEnforceDeadlinePolicyTest);
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(ControllerEnforceDeadlinePolicyTest, "testenforcedeadlinepolicy");

///////////////////////////////////////////////////////////////////////////////
                                                                                                                                                            
ControllerEnforceDeadlinePolicyTest::ControllerEnforceDeadlinePolicyTest()
{  
}

///////////////////////////////////////////////////////////////////////////////

void  
ControllerEnforceDeadlinePolicyTest::setUp()
{
    try
    {
        Cdmw::TestUtils::CORBATestManager::instance()->add("ClockService");
        m_obj = Cdmw::TestUtils::CORBATestManager::instance()->get_object("ClockService");
        CPPUNIT_ASSERT(!CORBA::is_nil(m_obj.in()));

        ClockCatalog_var clock_catalog = CosClockService::ClockCatalog::_narrow(m_obj.in());
        CPPUNIT_ASSERT(!CORBA::is_nil(clock_catalog.in()));

        ClockCatalog::ClockEntry* entry = clock_catalog->get_entry("ControlledExecutor");
        CPPUNIT_ASSERT(entry);

        m_controlledClock = ControlledClock::_narrow(entry->subject.in());
        CPPUNIT_ASSERT(!CORBA::is_nil(m_controlledClock.in()));

        m_controlledClock->set(Cdmw::clock::compute_current_time());
        m_controlledClock->set_rate(1.0);
    }
    catch (Cdmw::TestUtils::CORBATestManager::InvalidObject &)
    {
        CPPUNIT_FAIL ("Cdmw::TestUtils::CORBATestManager::InvalidObject");
    }
    catch ( ... )
    {
        CPPUNIT_FAIL ("Unknown exception!");
    }
        
    try
    {
        m_alias_name = "PolicyControllerUpdateAdmin";
        Cdmw::TestUtils::CORBATestManager::instance()->add(m_alias_name);
        m_obj = Cdmw::TestUtils::CORBATestManager::instance()->get_object(m_alias_name);
        CPPUNIT_ASSERT(!CORBA::is_nil(m_obj.in()));

        m_admin = Cdmw::clock::policy::ControllerUpdateAdmin::_narrow(m_obj.in());
        CPPUNIT_ASSERT(!CORBA::is_nil(m_admin.in()));
    }
    catch(Cdmw::TestUtils::CORBATestManager::InvalidObject&)
    {
        CPPUNIT_FAIL ("Cdmw::TestUtils::CORBATestManager::InvalidObject");
    }
    catch ( ... )
    {
        CPPUNIT_FAIL ("Unknown exception!");
    }

    try
    {
        PathsArchive::instance().initialize();
    }
    catch ( ... )
    {
        CPPUNIT_FAIL ("Unknown exception!");
    }
}

///////////////////////////////////////////////////////////////////////////////

void  
ControllerEnforceDeadlinePolicyTest::tearDown()
{
}

///////////////////////////////////////////////////////////////////////////////

void  
ControllerEnforceDeadlinePolicyTest::check_policy()
{
    // check if the policy is not registered by PolicyAdmin
    m_admin->get_policy(Cdmw::clock::policy::ENFORCE_DEADLINE);
}

///////////////////////////////////////////////////////////////////////////////

void
ControllerEnforceDeadlinePolicyTest::on_set_rate()
{
    Asserter* asserter = 0;
    Path_t*   path     = 0;

    try
    {
        // Get the path used for the test.
        path = PathsArchive::instance().get_path(std::string("ControllerEnforceDeadlinePolicyTest::on_set_rate"));

        // If the path is in the archive, then do the test.
        if (path != 0)
        {
            // Define the asserter and engine parameters.
            const long double   time_origin       = 0.0;
            const long double   period            = 4.0;
            const unsigned long execution_limit   = 0;
            // Automatic tests have not misure performaces
            // in order to avoid assertion when the machine is too busy
            // the tolerance range has been increase
            const long double   real_epsilon      = 2.0; // 0.1;
            const long double   virt_epsilon      = 2.0; //0.1;

            const long double   init_virt_epsilon = 0.05;
            const long double   init_rate_epsilon = 0.05;

            // Get a controlled executor reference.
            ControlledExecutor_var executor = ControlledExecutor::_narrow(m_controlledClock.in());

            // Set the enforce deadline policy.
            executor->set_controller_update_policy(Cdmw::clock::policy::ENFORCE_DEADLINE);

            // Create the extended asserter.
            asserter = new EnforceDeadlinePolicyAsserter(time_origin,
                                                         period,
                                                         execution_limit,
                                                         real_epsilon,
                                                         virt_epsilon,
                                                         *path);

            // Get a periodic reference.
            anExecution<Cdmw::clock::ControlledExecutor,
                        Cdmw::clock::ControlledExecutor_var,
                        Cdmw::clock::ControlledExecutor_ptr>* servant =
                new anExecution<Cdmw::clock::ControlledExecutor,
                                Cdmw::clock::ControlledExecutor_var,
                                Cdmw::clock::ControlledExecutor_ptr>(executor.in(), (anExecutionCallback*) asserter);
            CosClockService::PeriodicExecution::Periodic_var periodic = servant->_this();

            // Get a controller reference.
            CosClockService::PeriodicExecution::Controller_var controller =
                executor->enable_periodic_execution(periodic.in());

            // Create the extended engine.
            Engine engine(executor.in(),
                          init_virt_epsilon,
                          init_rate_epsilon,
                          period,
                          execution_limit,
                          *path);

            CORBA::Any param;

            // Reset executor
            executor->set(0);
            executor->set_rate(1.0);

            // Start the controller
            controller->start(Cdmw::clock::util::seconds2TimeT(period), 0, execution_limit, param);

            // Run the test.
            engine.run();

            // Terminate the controller.
            controller->terminate();
        }
    }
    catch (...)
    {
        // No exception should be thrown: release resources and exit with failure code.
        delete asserter;
        CPPUNIT_FAIL("unexpected exception");
    }

    // Final check: the following two conditions must be verified:
    // 1. check for do_work() invocations that failed.
    // 2. check for do_work() invocations that erroneously were not done.
    bool assertion = (path                                        &&
                      asserter                                    &&
                     (asserter->get_num_failed_executions() == 0) &&
                     (asserter->get_num_remaining_executions() == 0));
    delete asserter;
    CPPUNIT_ASSERT(assertion);
}

///////////////////////////////////////////////////////////////////////////////

void
ControllerEnforceDeadlinePolicyTest::on_set()
{
    Asserter* asserter = 0;
    Path_t*   path     = 0;

    try
    {
        // Get the path used for the test.
        path = PathsArchive::instance().get_path(std::string("ControllerEnforceDeadlinePolicyTest::on_set"));
                                                                                                                                                            
        // If the path is in the archive, then do the test.
        if (path != 0)
        {
            // Define the asserter and engine parameters.
            const long double   time_origin       = 0.0;
            const long double   period            = 4.0;
            const unsigned long execution_limit   = 0;
            // Automatic tests have not misure performaces 
            // in order to avoid assertion when the machine is too busy
            // the tolerance range has been increase
            const long double   real_epsilon      = 2.0; // 0.1;
            const long double   virt_epsilon      = 2.0; //0.1;
            const long double   init_virt_epsilon = 0.05;
            const long double   init_rate_epsilon = 0.05;

            // Get a controlled executor reference.
            ControlledExecutor_var executor = ControlledExecutor::_narrow(m_controlledClock.in());

            // Set the enforce deadline policy.
            executor->set_controller_update_policy(Cdmw::clock::policy::ENFORCE_DEADLINE);

            // Create the extended asserter.
            asserter = new EnforceDeadlinePolicyAsserter(time_origin,
                                                         period,
                                                         execution_limit,
                                                         real_epsilon,
                                                         virt_epsilon,
                                                         *path);

            // Get a periodic reference.
            anExecution<Cdmw::clock::ControlledExecutor,
                        Cdmw::clock::ControlledExecutor_var,
                        Cdmw::clock::ControlledExecutor_ptr>* servant =
                new anExecution<Cdmw::clock::ControlledExecutor,
                                Cdmw::clock::ControlledExecutor_var,
                                Cdmw::clock::ControlledExecutor_ptr>(executor.in(), (anExecutionCallback*) asserter);
            CosClockService::PeriodicExecution::Periodic_var periodic = servant->_this();

            // Get a controller reference.
            CosClockService::PeriodicExecution::Controller_var controller =
                executor->enable_periodic_execution(periodic.in());

            // Create the extended engine.
            Engine engine(executor.in(),
                          init_virt_epsilon,
                          init_rate_epsilon,
                          period,
                          execution_limit,
                          *path);

            CORBA::Any param;

            // Reset executor
            executor->set(0);
            executor->set_rate(1.0);

            // Start the controller
            controller->start(Cdmw::clock::util::seconds2TimeT(period), 0, execution_limit, param);

            // Run the test.
            engine.run();

            // Terminate the controller.
            controller->terminate();
        }
    }
    catch (...)
    {
        // No exception should be thrown: release resources and exit with failure code.
        delete asserter;
        CPPUNIT_FAIL("unexpected exception");
    }

    // Final check: the following two conditions must be verified:
    // 1. check for do_work() invocations that failed.
    // 2. check for do_work() invocations that erroneously were not done.
    bool assertion = (path                                        &&
                      asserter                                    &&
                     (asserter->get_num_failed_executions() == 0) &&
                     (asserter->get_num_remaining_executions() == 0));
    delete asserter;
    CPPUNIT_ASSERT(assertion);
}

///////////////////////////////////////////////////////////////////////////////
/*
void
ControllerEnforceDeadlinePolicyTest::on_set_with_threshold_crossing()
{
    Asserter* asserter = 0;

    try
    {
        // Define the piecewise linear curve in the real-time vs virtual-time coordinated plane
        // used for the test through the definition of the ordered set of all its vertices.
        const int num_points = 6;
        Point_t points[num_points] = { {  0.00,  0.00 },
                                       {  6.00,  6.00 },
                                       {  6.00, 10.00 },
                                       { 10.00, 14.00 },
                                       { 10.00, 18.00 },
                                       { 18.00, 26.00 } };

        // Define the asserter and engine parameters.
        const double        time_origin       = 0.0;
        const double        period            = 4.0;
        const unsigned long execution_limit   = 0;
        const double        real_epsilon      = 0.1;
        const double        virt_epsilon      = 0.1;
        const double        init_virt_epsilon = 0.05;
        const double        init_rate_epsilon = 0.05;

        // Get a controlled executor reference.
        ControlledExecutor_var executor = ControlledExecutor::_narrow(m_controlledClock.in());

        // Set the enforce deadline policy.
        executor->set_controller_update_policy(Cdmw::clock::policy::ENFORCE_DEADLINE);

        // Create the extended asserter.
        asserter = new EnforceDeadlinePolicyAsserter(time_origin,
                                                     period,
                                                     execution_limit,
                                                     real_epsilon,
                                                     virt_epsilon,
                                                     num_points,
                                                     points);

        // Get a periodic reference.
        anExecution<Cdmw::clock::ControlledExecutor,
                    Cdmw::clock::ControlledExecutor_var,
                    Cdmw::clock::ControlledExecutor_ptr>* servant =
            new anExecution<Cdmw::clock::ControlledExecutor,
                            Cdmw::clock::ControlledExecutor_var,
                            Cdmw::clock::ControlledExecutor_ptr>(executor.in(), (anExecutionCallback*) asserter);
        CosClockService::PeriodicExecution::Periodic_var periodic = servant->_this();

        // Get a controller reference.
        CosClockService::PeriodicExecution::Controller_var controller =
            executor->enable_periodic_execution(periodic.in());

        // Create the extended engine.
        Engine engine(executor.in(),
                      init_virt_epsilon,
                      init_rate_epsilon,
                      period,
                      execution_limit,
                      num_points,
                      points);

        CORBA::Any param;

        // Initialize executor for the test
        executor->set_rate(1.0);
        executor->set(0);

        std::cerr << "DEBUG: ControllerEnforceDeadlinePolicyTest::on_set_with_threshold_crossing: starting the controller" << std::endl;

        // Start the controller
        controller->start(Cdmw::clock::util::seconds2TimeT(period), 0, execution_limit, param);

        std::cerr << "DEBUG: ControllerEnforceDeadlinePolicyTest::on_set_with_threshold_crossing: running the engine" << std::endl;

        // Run the test.
        engine.run();

        // Terminate the controller.
        controller->terminate();
    }
    catch (Asserter::UnexpectedAsserterExecution&)
    {
        // No exception should be thrown.
        CPPUNIT_FAIL("unexpected asserter execution");
    }
    catch (Asserter::Exception&)
    {
        // No exception should be thrown.
        CPPUNIT_FAIL("invalid asserter input parameters");
    }
    catch (Engine::InvalidExecutorInitialization&)
    {
        // No exception should be thrown.
        CPPUNIT_FAIL("invalid executor initialization");
    }
    catch (Engine::Exception&)
    {
        // No exception should be thrown.
        CPPUNIT_FAIL("invalid engine input parameters");
    }
    catch (...)
    {
        // No exception should be thrown.
        CPPUNIT_FAIL("unexpected exception");
    }

    // Final check: the following two conditions must be verified:
    // 1. check for do_work() invocations that failed.
    // 2. check for do_work() invocations that erroneously were not done.
    CPPUNIT_ASSERT(asserter &&
                   asserter->get_num_failed_executions() == 0 &&
                   asserter->get_num_remaining_executions() == 0);
}
*/
///////////////////////////////////////////////////////////////////////////////

void
ControllerEnforceDeadlinePolicyTest::on_pause_resume()
{
    Asserter* asserter = 0;
    Path_t*   path     = 0;

    try
    {
        // Get the path used for the test.
        path = PathsArchive::instance().get_path(std::string("ControllerEnforceDeadlinePolicyTest::on_pause_resume"));
                                                                                                                                                            
        // If the path is in the archive, then do the test.
        if (path != 0)
        {
            // Define the asserter and engine parameters.
            const long double   time_origin       = 0.0;
            const long double   period            = 4.0;
            const unsigned long execution_limit   = 0;
            // Automatic tests have not misure performaces
            // in order to avoid assertion when the machine is too busy
            // the tolerance range has been increase
            const long double   real_epsilon      = 2.0; // 0.1;
            const long double   virt_epsilon      = 2.0; //0.1;
            const long double   init_virt_epsilon = 0.05;
            const long double   init_rate_epsilon = 0.05;

            // Get a controlled executor reference.
            ControlledExecutor_var executor = ControlledExecutor::_narrow(m_controlledClock.in());

            // Set the enforce deadline policy.
            executor->set_controller_update_policy(Cdmw::clock::policy::ENFORCE_DEADLINE);

            // Create the extended asserter.
            asserter = new EnforceDeadlinePolicyAsserter(time_origin,
                                                         period,
                                                         execution_limit,
                                                         real_epsilon,
                                                         virt_epsilon,
                                                         *path);

            // Get a periodic reference.
            anExecution<Cdmw::clock::ControlledExecutor,
                        Cdmw::clock::ControlledExecutor_var,
                        Cdmw::clock::ControlledExecutor_ptr>* servant =
                new anExecution<Cdmw::clock::ControlledExecutor,
                                Cdmw::clock::ControlledExecutor_var,
                                Cdmw::clock::ControlledExecutor_ptr>(executor.in(), (anExecutionCallback*) asserter);
            CosClockService::PeriodicExecution::Periodic_var periodic = servant->_this();

            // Get a controller reference.
            CosClockService::PeriodicExecution::Controller_var controller =
                executor->enable_periodic_execution(periodic.in());

            // Create the extended engine.
            Engine engine(executor.in(),
                          init_virt_epsilon,
                          init_rate_epsilon,
                          period,
                          execution_limit,
                          *path);

            CORBA::Any param;

            // Reset executor
            executor->set(0);
            executor->set_rate(1.0);

            // Start the controller
            controller->start(Cdmw::clock::util::seconds2TimeT(period), 0, execution_limit, param);

            // Run the test.
            engine.run();

            // Terminate the controller.
            controller->terminate();
        }
    }
    catch (...)
    {
        // No exception should be thrown: release resources and exit with failure code.
        delete asserter;
        CPPUNIT_FAIL("unexpected exception");
    }

    // Final check: the following two conditions must be verified:
    // 1. check for do_work() invocations that failed.
    // 2. check for do_work() invocations that erroneously were not done.
    bool assertion = (path                                        &&
                      asserter                                    &&
                     (asserter->get_num_failed_executions() == 0) &&
                     (asserter->get_num_remaining_executions() == 0));
    delete asserter;
    CPPUNIT_ASSERT(assertion);
}

///////////////////////////////////////////////////////////////////////////////

void
ControllerEnforceDeadlinePolicyTest::on_terminate()
{
    Asserter* asserter = 0;
    Path_t*   path     = 0;

    try
    {
        // Get the path used for the test.
        path = PathsArchive::instance().get_path(std::string("ControllerEnforceDeadlinePolicyTest::on_terminate"));

        // If the path is in the archive, then do the test.
        if (path != 0)
        {
            // Define the asserter and engine parameters.
            const long double   time_origin     = 0.0;
            const long double   period          = 4.0;
            const unsigned long execution_limit = 0;
            // Automatic tests have not misure performaces
            // in order to avoid assertion when the machine is too busy
            // the tolerance range has been increase
            const long double   real_epsilon      = 2.0; // 0.1;
            const long double   virt_epsilon      = 2.0; //0.1;

            // Get a controlled executor reference.
            ControlledExecutor_var executor = ControlledExecutor::_narrow(m_controlledClock.in());

            // Set the enforce deadline policy.
            executor->set_controller_update_policy(Cdmw::clock::policy::ENFORCE_DEADLINE);

            // Create the extended asserter.
            asserter = new EnforceDeadlinePolicyAsserter(time_origin,
                                                         period,
                                                         execution_limit,
                                                         real_epsilon,
                                                         virt_epsilon,
                                                         *path);

            // Get a periodic reference.
            anExecution<Cdmw::clock::ControlledExecutor,
                        Cdmw::clock::ControlledExecutor_var,
                        Cdmw::clock::ControlledExecutor_ptr>* servant =
                new anExecution<Cdmw::clock::ControlledExecutor,
                                Cdmw::clock::ControlledExecutor_var,
                                Cdmw::clock::ControlledExecutor_ptr>(executor.in(), (anExecutionCallback*) asserter);
            CosClockService::PeriodicExecution::Periodic_var periodic = servant->_this();

            // Get a controller reference.
            CosClockService::PeriodicExecution::Controller_var controller =
                executor->enable_periodic_execution(periodic.in());

            CORBA::Any param;

            // Initialize the executor.
            executor->set(0);
            executor->set_rate(1.0);

            // Start the controller
            controller->start(Cdmw::clock::util::seconds2TimeT(period), 0, execution_limit, param);

            // Terminate the executor after 10 seconds from controller start.
            int int_period = (int) period;
            Cdmw::OsSupport::OS::sleep(2 * int_period + int_period / 2, 0); // 10 seconds sleep
            executor->terminate();

            // Wait for a while, just to make sure that no other invocations of do_work are made.
            Cdmw::OsSupport::OS::sleep(2 * int_period , 0); // 8 seconds sleep
            //controller->terminate(); // NB: not needed: executor->terminate() already did the work...
        }
    }
    catch (...)
    {
        // No exception should be thrown: release resources and exit with failure code.
        delete asserter;
        CPPUNIT_FAIL("unexpected exception");
    }

    // Final check: the following two conditions must be verified:
    // 1. check for do_work() invocations that failed.
    // 2. check for do_work() invocations that erroneously were not done.
    bool assertion = (path                                        &&
                      asserter                                    &&
                     (asserter->get_num_failed_executions() == 0) &&
                     (asserter->get_num_remaining_executions() == 0));
    delete asserter;
    CPPUNIT_ASSERT(assertion);
}

///////////////////////////////////////////////////////////////////////////////
// End Of File
///////////////////////////////////////////////////////////////////////////////
