/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
                                                                                                                                                            
/**
 * @brief Implementation of class PathsArchive.
 * @author Marco Macellari <mmacellari@progesi.it>
 */

#include "testpolicies/PathsArchive.hpp"

PathsArchive&
PathsArchive::instance()
{
    static PathsArchive paths_archive;
    return paths_archive;
}

PathsArchive::~PathsArchive()
{
}

void
PathsArchive::initialize()
{
    if (initialized == false)
    {
        // add all paths ...
        add_all_paths();

        // set initialized flag to true
        initialized = true;
    }
}

Path_t*
PathsArchive::get_path(const std::string& path_name)
{
    PathsMap_t::iterator it = paths.find(path_name);
    if (it != paths.end())
        return &((*it).second);
    return 0;
}

void
PathsArchive::add_path(const std::string& path_name, const Path_t& path)
{
    paths[path_name] = path;
}

void
PathsArchive::remove_path(const std::string& path_name)
{
    PathsMap_t::iterator it = paths.find(path_name);
    if (it != paths.end())
        paths.erase(it);
}

void
PathsArchive::add_all_paths()
{
    add_all_cancel_all_policy_test_paths();
    add_all_enforce_interval_policy_test_paths();
    add_all_enforce_deadline_policy_test_paths();
}

void
PathsArchive::add_all_cancel_all_policy_test_paths()
{
}

void
PathsArchive::add_all_enforce_interval_policy_test_paths()
{
    Path_t path;

    // add path "ControllerEnforceIntervalPolicyTest::PCR_0450_on_set_rate"
    path.push_back(Point_t( 0.00,  0.00));
    path.push_back(Point_t( 6.00,  6.00));
    path.push_back(Point_t(10.00, 10.00));
    path.push_back(Point_t(18.00, 14.00));
    path.push_back(Point_t(22.00, 22.00));
    add_path(std::string("ControllerEnforceIntervalPolicyTest::PCR_0450_on_set_rate"), path);
    path.clear();

    // add path "ControllerEnforceIntervalPolicyTest::PCR_0450_on_set"
    path.push_back(Point_t( 0.00,  0.00));
    path.push_back(Point_t( 6.00,  6.00));
    path.push_back(Point_t( 6.00, 14.00));
    path.push_back(Point_t(10.00, 18.00));
    path.push_back(Point_t(10.00, 26.00));
    path.push_back(Point_t(14.00, 30.00));
    path.push_back(Point_t(14.00, 10.00));
    path.push_back(Point_t(22.00, 18.00));
    add_path(std::string("ControllerEnforceIntervalPolicyTest::PCR_0450_on_set"), path);
    path.clear();

    // add path "ControllerEnforceIntervalPolicyTest::PCR_0450_on_pause_resume"
    path.push_back(Point_t( 0.00,  0.00));
    path.push_back(Point_t( 6.00,  6.00));
    path.push_back(Point_t(14.00,  6.00));
    path.push_back(Point_t(18.00, 10.00));
    path.push_back(Point_t(22.00, 10.00));
    path.push_back(Point_t(30.00, 18.00));
    add_path(std::string("ControllerEnforceIntervalPolicyTest::PCR_0450_on_pause_resume"), path);
    path.clear();

    // add path "ControllerEnforceIntervalPolicyTest::on_set_time_and_rate_while_in_pause"
    path.push_back(Point_t( 0.00,  0.00));
    path.push_back(Point_t( 6.00,  6.00));
    path.push_back(Point_t(14.00,  6.00));
    path.push_back(Point_t(16.00, 10.00));
    path.push_back(Point_t(22.00, 10.00));
    path.push_back(Point_t(22.00, 14.00));
    path.push_back(Point_t(26.00, 18.00));
    add_path(std::string("ControllerEnforceIntervalPolicyTest::on_set_time_and_rate_while_in_pause"), path);
    path.clear();

    // add path "ControllerEnforceIntervalPolicyTest::more_controllers_with_different_policies_at_the_same_time"
    path.push_back(Point_t( 0.00,  0.00));
    path.push_back(Point_t( 6.00,  6.00));
    path.push_back(Point_t(18.00, 12.00));
    path.push_back(Point_t(24.00, 24.00));
    add_path(std::string("ControllerEnforceIntervalPolicyTest::more_controllers_with_different_policies_at_the_same_time"), path);
    path.clear();

    // add path "ControllerEnforceIntervalPolicyTest::PCR_0450_on_terminate"
    path.push_back(Point_t( 0.00,  0.00));
    path.push_back(Point_t(10.00, 10.00));
    add_path(std::string("ControllerEnforceIntervalPolicyTest::PCR_0450_on_terminate"), path);
}

void
PathsArchive::add_all_enforce_deadline_policy_test_paths()
{
    Path_t path;

    // add path "ControllerEnforceDeadlinePolicyTest::on_set_rate"
    path.push_back(Point_t( 0.00,  0.00));
    path.push_back(Point_t( 6.00,  6.00));
    path.push_back(Point_t(10.00, 10.00));
    path.push_back(Point_t(18.00, 14.00));
    path.push_back(Point_t(22.00, 22.00));
    add_path(std::string("ControllerEnforceDeadlinePolicyTest::on_set_rate"), path);
    path.clear();

    // add path "ControllerEnforceDeadlinePolicyTest::on_set"
    path.push_back(Point_t( 0.00,  0.00));
    path.push_back(Point_t( 5.00,  5.00));
    path.push_back(Point_t( 5.00,  7.00));
    path.push_back(Point_t( 7.00,  9.00));
    path.push_back(Point_t( 7.00, 11.00));
    path.push_back(Point_t(10.00, 14.00));
    add_path(std::string("ControllerEnforceDeadlinePolicyTest::on_set"), path);
    path.clear();

    // add path "ControllerEnforceDeadlinePolicyTest::on_pause_resume"
    path.push_back(Point_t( 0.00,  0.00));
    path.push_back(Point_t( 6.00,  6.00));
    path.push_back(Point_t(14.00,  6.00));
    path.push_back(Point_t(18.00, 10.00));
    path.push_back(Point_t(22.00, 10.00));
    path.push_back(Point_t(30.00, 18.00));
    add_path(std::string("ControllerEnforceDeadlinePolicyTest::on_pause_resume"), path);
    path.clear();

    // add path "ControllerEnforceDeadlinePolicyTest::on_terminate"
    path.push_back(Point_t( 0.00,  0.00));
    path.push_back(Point_t(10.00, 10.00));
    add_path(std::string("ControllerEnforceDeadlinePolicyTest::on_terminate"), path);
    path.clear();
}

PathsArchive::PathsArchive()
    : initialized(false)
{
}

