#!/bin/bash

# handle input parameters
SCRIPT=`basename $0`
if [[ $# -ne 1 ]] ; then
    echo "usage: $SCRIPT <test program name>"
    echo "where: <test program name> is the program name of the test to be run"
    echo "notes: required local environment variables are:"
    echo "       - TESTS_REPORTS_SCRIPTS_ROOT pointing to the local absolute path for test reports logs."
    echo "       - TESTS_REPORTS_LOGS_ROOT    pointing to the local absolute path for test reports logs."
    echo "       - CDMW_HOME                  pointing to the local cardamom installation root."
    echo "       - VALGRIND_3_1_0_BIN_ROOT    pointing to the local bin directory of valgrind-3.1.0 installation."
    echo "       - TMG_PORT                   (from tests_reports-global_macros.bash) containing the communication port."
    exit 1
fi

# include global macros script
source $TESTS_REPORTS_SCRIPTS_ROOT/tests_reports-global_macros.bash

# run the test and handle log files
EXIT_CODE=0
$VALGRIND_3_1_0_BIN_ROOT/valgrind --tool=memcheck --leak-check=full --trace-children=yes --log-file=$TESTS_REPORTS_LOGS_ROOT/valgrind-3.1.0.memo-check.$1.server-process.log $CDMW_HOME/bin/$1 --server --CdmwFederatedCtrlClock=on --CdmwLocalisationService=$TMG_PORT 2>&1 | tee $TESTS_REPORTS_LOGS_ROOT/valgrind-3.1.0.memo-check.$1.server-process.out
mv $TESTS_REPORTS_LOGS_ROOT/valgrind-3.1.0.memo-check.$1.server-process.log.* $TESTS_REPORTS_LOGS_ROOT/valgrind-3.1.0.memo-check.$1.server-process.log 2>&1 > /dev/null
if [[ $? -ne 0 ]] ; then EXIT_CODE=1 ; fi
exit $EXIT_CODE

