/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/common/System.hpp"
#include <Foundation/orbsupport/Codec.hpp>
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"

#include <Foundation/ossupport/OS.hpp>

#include <LoadBalancing/lbinit/LBServiceInit.hpp>

#include <ConfAndPlug/cdmwinit/CdmwInterface.hpp>

#include <iostream>
//added for PCR-648
#include <Time/clockservice/CdmwCosClock.hpp>



namespace {
    const int SUCCESS = 0;
    const int FAILURE = 1;      
    const int returnError = -1; 
    
    bool  clientOnly = false;
    
    void parse_args(int argc, char* argv[] )
    {

        // -- client only mode.
        std::string arg_value =
            Cdmw::OsSupport::OS::get_option_value(argc,
                                                  argv,
                                                  "--client");
        if (arg_value != "no")
        {            
            std::cout << ":>> client only selection is: on." << std::endl;
            clientOnly = true;
        }

 
        // -- show help informations
        arg_value =
            Cdmw::OsSupport::OS::get_option_value(argc,
                                                  argv,
                                                  "--help");
        if (arg_value == "yes")
        {
            std::cout << "Options:\n"
                      << "\t --help................................Show this message\n"
                      << "\t --client..............................Start the LB service in client mode.\n"
                      << std::endl;
            exit(returnError);
        }
        
    }            
    
}; // End anonymous namespace


int main(int argc, char* argv[])
{
    parse_args(argc,argv);

    int status = SUCCESS;

    CORBA::ORB_var orb = CORBA::ORB::_nil();

    try {
        // Initialize LB service
        std::cout << "----- Initializing LB service -----" << std::endl;
        Cdmw::LB::LBServiceInit::Init(argc, argv);

        // Initialize the ORB
        Cdmw::OrbSupport::StrategyList orb_strategies;
        orb_strategies.add_OrbThreaded();
        orb_strategies.add_PoaThreadPerConnection();
    
        if (!clientOnly)
        {
        orb_strategies.add_multicast();
        }   

        std::cout << "------------ ORB_init -------------" << std::endl;
        orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc,
                                                     argv,
                                                     orb_strategies);


        // PCR-0049
        // TO BE VERIFIED Cdmw::OrbSupport::CodecBase::init(orb.in());

        //
        // Get the root POA
        //
        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var rootPOA =
            PortableServer::POA::_narrow(obj.in());

        //
        // Run implementation (Activate the POA manager)
        //
        PortableServer::POAManager_var poaManager = rootPOA->the_POAManager();
        poaManager->activate();

        Cdmw::TestUtils::CORBATestManager::instance()->init(orb.in(),rootPOA.in(),argc,argv);
        Cdmw::CdmwInit::CDMW_init(orb.in(), argc, argv);

        // Setup signal handling for ORB to be shut down
        // Cdmw::OrbSupport::OrbSupport::install_orb_shutdown_signal_handlers(orb.in());

        //added for PCR-648
        if (!Cdmw::clock::CosClock::init (orb.in (), argc, argv)) {
          std::cerr << "Can not init clock service!" << std::endl;
          return -1;
        }


        orb->run();
    
        // Cdmw::OrbSupport::OrbSupport::uninstall_orb_shutdown_signal_handlers(orb.in());
        
        
    } catch (const Cdmw::Exception& ex) {
    std::cerr << ex.what() << std::endl;
        status = FAILURE;
    } catch(const CORBA::Exception& ex) {
    std::cerr << ex << std::endl;
        status = FAILURE;
    }
    if ( !CORBA::is_nil( orb.in() ) )
    {
        try
    {
            Cdmw::CdmwInit::CDMW_cleanup(orb.in());  
        Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in()); 
        // orb->destroy();
        }
        catch ( const CORBA::Exception& ex )
        {
            std::cerr << ex << std::endl;
            status = FAILURE;
        }
    } 
        
    return status;
}
