/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <ccm_ft_with_timer/CCM_Client_impl.hpp>

#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>

#define MAX_NUMB_OF_RIPETITIONS 100;

//
// IDL:thalesgroup.com/CcmFtWithTimerTest/ClientModule/CCM_Client:1.0
//
CcmFtWithTimerTest::CCM_Client_impl::CCM_Client_impl()
    : m_session_context(CcmFtWithTimerTest::ClientModule::CCM_Client_Context::_nil()),
      m_testThread()
{
}

CcmFtWithTimerTest::CCM_Client_impl::~CCM_Client_impl()
{
    std::cout << "   ... CCM_Client_impl::~CCM_Client_impl called!" << std::endl;
}

//
// facet clientHostInfo
//
CcmFtWithTimerTest::FacetModule::CCM_HostInfo_ptr 
CcmFtWithTimerTest::CCM_Client_impl::get_clientHostInfo ()
    throw (CORBA::SystemException)
{
    return CcmFtWithTimerTest::FacetModule::CCM_HostInfo::_duplicate(this);
}

char*
CcmFtWithTimerTest::CCM_Client_impl::get_hostname()
    throw (CORBA::SystemException)
{
    std::string hostname = Cdmw::OsSupport::OS::get_hostname();
    return CORBA::string_dup(hostname.c_str());
}    


//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
CcmFtWithTimerTest::CCM_Client_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_Client_impl::Set_session_context called!" << std::endl;
   
    m_session_context = ::CcmFtWithTimerTest::ClientModule::CCM_Client_Context::_narrow(ctx); 
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CcmFtWithTimerTest::CCM_Client_impl::ccm_activate()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_Client_impl::Ccm_activate called!" << std::endl;
   Cdmw::TestUtils::CORBATestManager::instance()->add("client_context", get_context());
   Cdmw::TestUtils::CORBATestManager::instance()->add("user_ccm_client", static_cast< ::UserCcmFtWithTimerTest::User_CCM_Client_ptr > (this));
   m_testThread.start();
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
CcmFtWithTimerTest::CCM_Client_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_Client_impl::Ccm_passivate called!" << std::endl;
} 



//
// ccm_remove function : it destroys the component (call by the container)
//
void
CcmFtWithTimerTest::CCM_Client_impl::ccm_remove()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_Client_impl::Ccm_remove called!" << std::endl;
}


//
// Testable do_tests() operation
//
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(CcmFtWithTimerTest::test_ccm_ft_with_timer_activate, "test_ccm_ft_with_timer_activate");

void
CcmFtWithTimerTest::test_ccm_ft_with_timer_activate::do_tests()
{
    Cdmw::OsSupport::OS::sleep(2000);
    

//     set_nbOfRequestedTestOK(0);
    
    // test connections to group 
    for (int number_of_ripetitions = 0; number_of_ripetitions < 3 ; number_of_ripetitions++) {
       TEST_INFO("   ############### Test connections to ServerWithFacet  " );
       test_connections();
       Cdmw::OsSupport::OS::sleep(10000);
    }
    
}


void
CcmFtWithTimerTest::test_ccm_ft_with_timer_activate::test_connections()
{
//     add_nbOfRequestedTestOK(3);
    
    try {
    
        CcmFtWithTimerTest::FacetModule::HostInfo_var hostInfo = m_session_context->get_connection_serverHostInfo();
        CORBA::String_var hostname = hostInfo->get_hostname();
        Cdmw::OsSupport::OS::sleep(2000);
        TEST_INFO("   ############### Test connections to ServerWithFacet group on host  " <<  hostname);    


        if (CORBA::is_nil(hostInfo.in())) {
            TEST_INFO("hostInfo is NIL !!!! ");
            CPPUNIT_ASSERT(false);
            return;
        }
        
        
        CPPUNIT_ASSERT(true);            
       
        
    } catch (const CORBA::SystemException& e) {
        TEST_INFO("Unexpected CORBA::SystemException: " << e);
        CPPUNIT_ASSERT(false);
    } catch (const CORBA::Exception& e) {
        TEST_INFO("Unexpected CORBA::Exception: " << e);
        CPPUNIT_ASSERT(false);
    } catch (...) {
        TEST_INFO("Unknown exception !! ");
        CPPUNIT_ASSERT(false);
    }
    


}




