/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>

#include <ccm_ft_with_timer/CCM_ServerWithFacet_impl.hpp>

#include <Foundation/ossupport/OS.hpp>

#include <CCMContainer/ccmcif/SessionContextExt_impl.hpp>
#include <Time/idllib/CosClockService.stub.hpp>
#include <Time/clockservice/Util.hpp>


//
// IDL:thalesgroup.com/CcmFtWithTimerTest/CCM_ServerWithFacet:1.0
//
CcmFtWithTimerTest::CCM_ServerWithFacet_impl::CCM_ServerWithFacet_impl()
{
   m_expectedClientHostname = CORBA::string_dup("");

}

CcmFtWithTimerTest::CCM_ServerWithFacet_impl::~CCM_ServerWithFacet_impl()
{
    std::cout << "   ... CCM_ServerWithFacet_impl::~CCM_ServerWithFacet_impl called!" << std::endl;
    
}



//
// attribute expectedClientHostname
//
char*
CcmFtWithTimerTest::CCM_ServerWithFacet_impl::expectedClientHostname()
    throw(CORBA::SystemException)
{
    return CORBA::string_dup(m_expectedClientHostname);
}

void
CcmFtWithTimerTest::CCM_ServerWithFacet_impl::expectedClientHostname(const char* a)
    throw(CORBA::SystemException)
{
    m_expectedClientHostname =  CORBA::string_dup(a);
}


//
// facet serverHostInfo
//
CcmFtWithTimerTest::FacetModule::CCM_HostInfo_ptr 
CcmFtWithTimerTest::CCM_ServerWithFacet_impl::get_serverHostInfo ()
    throw (CORBA::SystemException)
{
    std::cout << "   ... CCM_ServerWithFacet_impl::get_serverHostInfo()" << std::endl; 
    return CcmFtWithTimerTest::FacetModule::CCM_HostInfo::_duplicate(this);
}

char*
CcmFtWithTimerTest::CCM_ServerWithFacet_impl::get_hostname()
    throw (CORBA::SystemException)
{
    
    std::string hostname = Cdmw::OsSupport::OS::get_hostname();

    return CORBA::string_dup(hostname.c_str());
}    


//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
CcmFtWithTimerTest::CCM_ServerWithFacet_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_ServerWithFacet_impl::Set_session_context called!" << std::endl;
   
    m_session_context = ::CcmFtWithTimerTest::ServerModule::CCM_ServerWithFacet_Context::_narrow(ctx); 

    // get context for services
    _sessionContextExt = CdmwComponents::SessionContextExt::_narrow(m_session_context.in());
    // get Time Service
    CORBA::Object_var clockCatalogObj =
      _sessionContextExt->resolve_service_reference("ClockService");
    _clockCatalog = CosClockService::ClockCatalog::_narrow(clockCatalogObj.in());

    CosClockService::ClockCatalog::ClockEntry_var
        entry = _clockCatalog->get_entry("LocalExecutor");

    CosClockService::PeriodicExecution::Executor_var executor;
        executor = CosClockService::PeriodicExecution::Executor::_narrow((entry->subject).in());

    try {
      CORBA::Object_var obj = _sessionContextExt->get_CCM_object();
      _serveur_ref = CcmFtWithTimerTest::ServerModule::ServerWithFacet::_narrow(obj.in());
    } catch (const Components::IllegalState& e) {
      // shoudn't happen
      std::cerr << "Error narrowing component to Serveur" << std::endl;
      //CDMW_ASSERT(false);
    } catch (const CORBA::BAD_PARAM& e) {
      // shouldn't happen
      std::cerr << "Failed to narrow Component to Serveur! Is this call performed on Component itself ?" << std::endl;
      //CDMW_ASSERT(false);
    }

    CosClockService::PeriodicExecution::Periodic_var periodic =
      CosClockService::PeriodicExecution::Periodic::_nil();
    try {
      // narrow component reference as Periodic since it supports this interface
      periodic =
              CosClockService::PeriodicExecution::Periodic::_narrow(_serveur_ref.in());
    }
    catch (const CORBA::BAD_PARAM& e) {
      std::cerr << "Failed to narrow Component in CosClockService.Periodic. Does it support this interface ??" << std::endl;
      //CDMW_ASSERT(false);
    }

    std::cout << "   ############### (STEP 1) Component Server programs the timer " << std::endl;
      
   controller =
      executor->enable_periodic_execution(periodic.in());



}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CcmFtWithTimerTest::CCM_ServerWithFacet_impl::ccm_activate()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_ServerWithFacet_impl::Ccm_activate called!" << std::endl;
   std::cout << "   ############### (STEP 2) Component Server starts timer  " << std::endl;

    unsigned long execution_limit = 100;
    CORBA::Any params;

    controller->start( Cdmw::clock::util::seconds2TimeT(1), 0, execution_limit, params);
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//

void
CcmFtWithTimerTest::CCM_ServerWithFacet_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_ServerWithFacet_impl::Ccm_passivate called!" << std::endl;
} 



//
// ccm_remove function : it destroys the component (call by the container)
void
CcmFtWithTimerTest::CCM_ServerWithFacet_impl::ccm_remove()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_ServerWithFacet_impl::Ccm_remove called!" << std::endl;
}


CORBA::Boolean CcmFtWithTimerTest::CCM_ServerWithFacet_impl::do_work(const CORBA::Any& args)
      throw (CORBA::SystemException)
   {
    // print on stdout
    std::cout <<  "   ############### [OUTPUT OF PERIODIC EXECUTION] ####################  "  << std::endl;
    Cdmw::OsSupport::OS::sleep(500); 
    return true;
   }

