/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @brief TimeAdmin - Federated Controlled Clock demo
 */

package cdmw.clock.federated;

import org.omg.CORBA.ORB;
import org.omg.CORBA.SystemException;
import org.omg.CORBA.ORBPackage.InvalidName;
import org.omg.CosClockService.*;
import org.omg.CosClockService.ClockCatalogPackage.ClockEntry;
import org.omg.CosClockService.ClockCatalogPackage.UnknownEntry;
import org.omg.CosClockService.ControlledClockHelper;

import cdmw.common.BadParameterException;
import cdmw.orbsupport.ORBSupport;
import cdmw.orbsupport.StrategyList;
import cdmw.ossupport.OS;

class ORBWorker extends Thread
{
   private ORB m_ORB;

   public ORBWorker(ORB orb)
   {
      m_ORB = orb;
   }

   public void run()
   {
      try
      {
         m_ORB.run();
      }
      catch (SystemException ex)
      {
         ex.printStackTrace();
      }
   }
}

public class TimeReader
{

   private static ClockCatalog m_catalog;
   private static ControlledClock m_ctrlClock;
   private static ORB m_orb;

   private static final String CLOCK_OPT = "--clock";
   private static final String WARMUP_OPT = "--warmup";

   private static final int ERROR = -1;
   private static final int SUCCESS = 0;

   public static void main(String[] args)
   {
      StrategyList strategyList = new StrategyList();
      strategyList.addORBThreaded();
      try
      {
         strategyList.addPOAThreadPerConnection();
      }
      catch (BadParameterException e)
      {
         e.printStackTrace();
         System.exit(ERROR);
      }

        try
        {
            String sopt = OS.getOptionValue( args, WARMUP_OPT);
            int warmup_delay = 0;
            if (!sopt.equals("no"))
                warmup_delay = Integer.valueOf(sopt).intValue();

            if (warmup_delay > 0)
            { 
                System.out.println("==> waiting " + warmup_delay + " sec, for warmup.");
                OS.sleep ( warmup_delay * 1000 );
            }
        }
        catch ( InterruptedException ex )
        {
            ex.printStackTrace();
        }

      m_orb =  org.omg.CORBA.ORB.init(args, null);

      // ORB in a thread
      ORBWorker orb_worker = new ORBWorker(m_orb);
      orb_worker.start();

      String clockopt = OS.getOptionValue( args, CLOCK_OPT);
      if (clockopt.equals("no") || clockopt.equals("yes"))
         clockopt = "FederatedControlledClock";
      
      try
      {
         OS.sleep(2000);
      }
      catch (InterruptedException e)
      {
         e.printStackTrace();
         System.exit(ERROR);
      }
      cdmw.clock.CdmwCosClock.init(m_orb, args);
      org.omg.CORBA.Object obj = null;
      try
      {
         obj = m_orb.resolve_initial_references("ClockService");
      }
      catch (InvalidName e1)
      {
         e1.printStackTrace();
         System.exit(ERROR);
      }
      m_catalog = ClockCatalogHelper.narrow(obj);
      if (m_catalog == null)
      {
         System.out.println("Error : Unable to resolve clock catalog");
      }

      ClockEntry entry = null;
      try
      {
         entry = m_catalog.get_entry(clockopt);
         m_ctrlClock = ControlledClockHelper.narrow(entry.subject);
      }
      catch (UnknownEntry ex)
      {
         System.out.println("ClockCatalog::UnknownEntry");
      }

      if (m_ctrlClock == null) 
      {
         System.err.println("Unable to resolve " + clockopt + ", Aborting!");
         ORBSupport.ORBCleanup(m_orb);
         System.exit(ERROR);
      }

      long newTime = 0,
           oldTime = 0;

      while (true)
      {
         try
         {
            newTime = m_ctrlClock.current_time();
            System.out.println(OS.getHostname() + ">> Current Time: " + newTime);
            oldTime = newTime;
         }
         catch (TimeUnavailable e)
         {
            e.printStackTrace();
            System.exit(ERROR);
         }
         try
         {
            OS.sleep(2000);
         }
         catch (InterruptedException e)
         {
            e.printStackTrace();
            System.exit(ERROR);
         }
      }
   }
}

