/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @author Raffaele Mele <rmele@progesi.it>
 */

#include "ntp_host_restart/NtpHostRestart.hpp"

using namespace CosClockService;

using namespace CosPropertyService;
CPPUNIT_TEST_SUITE_REGISTRATION( NtpHostRestart );
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( NtpHostRestart, "testntphostrestart" );

///////////////////////////////////////////////////////////////////////////////

NtpHostRestart::NtpHostRestart()
{
    m_clock_name = "ClockService";
    m_masterclock_name = "MasterClock";

    Cdmw::TestUtils::CORBATestManager::instance()->add(m_clock_name);
    if (Cdmw::clock::util::check_ntp_syncronization())
        Cdmw::TestUtils::CORBATestManager::instance()->add(m_masterclock_name);

    try
    {
        if (Cdmw::clock::util::check_ntp_syncronization())
            m_ptr_masterclock = Cdmw::TestUtils::CORBATestManager::instance()->get_object(m_masterclock_name);
        m_ptr_clockservice = Cdmw::TestUtils::CORBATestManager::instance()->get_object(m_clock_name);
            
    }
    catch ( Cdmw::TestUtils::CORBATestManager::InvalidObject& )
    {
        CPPUNIT_FAIL("Cdmw::TestUtils::CORBATestManager::InvalidObject");
    }
    // retrieve MasterClock
    if (Cdmw::clock::util::check_ntp_syncronization())
        m_masterclock = CosClockService::Clock::_narrow(m_ptr_masterclock);

     m_clock_catalog = CosClockService::ClockCatalog::_narrow(m_ptr_clockservice);
    CPPUNIT_ASSERT(!CORBA::is_nil(m_clock_catalog.in() ) );

    ClockCatalog::ClockEntry_var clockEntry =
        m_clock_catalog->get_entry("ControlledClock");
            
}

///////////////////////////////////////////////////////////////////////////////

void
NtpHostRestart::setUp()
{}

///////////////////////////////////////////////////////////////////////////////

void
NtpHostRestart::tearDown()
{}

///////////////////////////////////////////////////////////////////////////////

void
NtpHostRestart::test_host_syncro_with_ntp(int retry)
{
    bool isSync = Cdmw::clock::util::check_ntp_syncronization();
    while ( retry-- && (!isSync) ) {
        // wait NTP synchronization
        Cdmw::OsSupport::OS::sleep(WAIT_MSEC);
        isSync = Cdmw::clock::util::check_ntp_syncronization();
        std::cout << "timeout in " << retry << " seconds.." << std::endl;
    }
    if ( isSync )
    {
        std::cout << "The host is synchronized via NTP daemon." << std::endl;
    }
    else
    {
        std::cout << "timeout expired! Please check host(s) NTP configuration." << std::endl;
    }
    CPPUNIT_ASSERT( isSync );
}

///////////////////////////////////////////////////////////////////////////////

void
NtpHostRestart::test_masterclock_definition()
{
    ClockCatalog::ClockEntry_var clockEntry;

    try {
        clockEntry = m_clock_catalog->get_entry( "MasterClock" );
    }
    catch ( CosClockService::ClockCatalog::UnknownEntry& ) {
        CPPUNIT_FAIL( "Unable to resolve MasterClock from ClockCatalog." );
    }

    // retrieve MasterClock
    m_masterclock = CosClockService::Clock::_narrow( clockEntry->subject.in() );

    CPPUNIT_ASSERT( !CORBA::is_nil( m_masterclock.in() ) );

    std::cout << "MasterClock item exist into ClockService Catalog." << std::endl;
}

///////////////////////////////////////////////////////////////////////////////

void
NtpHostRestart::test_syncro_after_reboot()
{
    // test check file
    ifstream infile( CONTROL_FILE );

    if ( ! infile )  // before reboot
    {
        std::cout << std::endl;
        test_host_syncro_with_ntp(NO_RETRY);
        test_masterclock_definition();
        // create check file
        ofstream outfile ( CONTROL_FILE );
        outfile << "REBOOT TEST FILE" << std::endl;
        outfile.close();

        if ( ! outfile )
        {
            CPPUNIT_FAIL( "ERROR: Unable to create reboot control file." );
        }

        std::cerr << "The host is synchronized via NTP daemon at the time." << std::endl;
        std::cerr << "Please run following command:" << std::endl;
        std::cerr << "    platform_admin.sh --host-reboot HOST1" << std::endl;
        std::string input_buffer;
        // wait for host reboot

        while ( true )
            std::cin >> input_buffer;
    }
    std::cout << std::endl;
    test_host_syncro_with_ntp(MAX_RETRY);
    //test_masterclock_definition();
    std::cerr << "After reboot the host is synchronized via NTP daemon.\n" << std::endl;
    std::cerr << "** test execution successfull terminated **" << std::endl;

    std::remove
        ( CONTROL_FILE );
}

///////////////////////////////////////////////////////////////////////////////

void
NtpHostRestart::test_syncro_after_restart()
{
    // test check file
    ifstream infile( CONTROL_FILE );

    if ( ! infile )  // before reboot
    {
        std::cout << std::endl;
        test_host_syncro_with_ntp(NO_RETRY);
        test_masterclock_definition();
        // create check file
        ofstream outfile ( CONTROL_FILE );
        outfile << "RESTART TEST FILE" << std::endl;
        outfile.close();

        if ( ! outfile )
        {
            CPPUNIT_FAIL( "ERROR: Unable to create restart control file." );
        }

        std::cerr << "The host is synchronized via NTP daemon at the time." << std::endl;
        std::cerr << "Please run following command:" << std::endl;
        std::cerr << "    platform_admin.sh --host-shutdown HOST1" << std::endl;
        std::string input_buffer;
        // wait for host restart

        while ( true )
            std::cin >> input_buffer;
    }
    std::cout << std::endl;
    test_host_syncro_with_ntp(MAX_RETRY);
    //test_masterclock_definition();
    std::cerr << "After restart the host is synchronized via NTP daemon.\n" << std::endl;
    std::cerr << "** test execution successfull terminated **" << std::endl;

    std::remove
        ( CONTROL_FILE );
}
///////////////////////////////////////////////////////////////////////////////
//  End Of File
///////////////////////////////////////////////////////////////////////////////
