/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @author Raffaele Mele <rmele@progesi.it>
 */

#include "ntp_host_synchro/NtpHostSynchro.hpp"

using namespace CosClockService;

using namespace CosPropertyService;
CPPUNIT_TEST_SUITE_REGISTRATION( NtpHostSynchro );
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( NtpHostSynchro, "testntphostsynchronization" );

///////////////////////////////////////////////////////////////////////////////

NtpHostSynchro::NtpHostSynchro()
{
    m_clock_name = "ClockService";
    m_masterclock_name = "MasterClock";

    Cdmw::TestUtils::CORBATestManager::instance()->add(m_clock_name);
    if (Cdmw::clock::util::check_ntp_syncronization())
        Cdmw::TestUtils::CORBATestManager::instance()->add(m_masterclock_name);

    try
    {
        if (Cdmw::clock::util::check_ntp_syncronization())
            m_ptr_masterclock = Cdmw::TestUtils::CORBATestManager::instance()->get_object(m_masterclock_name);
        m_ptr_clockservice = Cdmw::TestUtils::CORBATestManager::instance()->get_object(m_clock_name);
            
    }
    catch ( Cdmw::TestUtils::CORBATestManager::InvalidObject& )
    {
        CPPUNIT_FAIL("Cdmw::TestUtils::CORBATestManager::InvalidObject");
    }
    // retrieve MasterClock
    if (Cdmw::clock::util::check_ntp_syncronization())
        m_masterclock = CosClockService::Clock::_narrow(m_ptr_masterclock);

     m_clock_catalog = CosClockService::ClockCatalog::_narrow(m_ptr_clockservice);
    CPPUNIT_ASSERT(!CORBA::is_nil(m_clock_catalog.in() ) );
}

///////////////////////////////////////////////////////////////////////////////

void
NtpHostSynchro::setUp()
{}

///////////////////////////////////////////////////////////////////////////////

void
NtpHostSynchro::tearDown()
{}

///////////////////////////////////////////////////////////////////////////////

void
NtpHostSynchro::test_host_syncro_with_ntp()
{
    CPPUNIT_ASSERT( Cdmw::clock::util::check_ntp_syncronization() );
    std::cout << "The host is synchronized via NTP daemon." << std::endl;
}

///////////////////////////////////////////////////////////////////////////////

void
NtpHostSynchro::test_masterclock_definition()
{
    ClockCatalog::ClockEntry_var clockEntry;

    try {
        clockEntry = m_clock_catalog->get_entry( "MasterClock" );
    }
    catch ( CosClockService::ClockCatalog::UnknownEntry& ) {
        CPPUNIT_FAIL( "Unable to resolve MasterClock from ClockCatalog." );
    }

    // retrieve MasterClock
    m_masterclock = CosClockService::Clock::_narrow( clockEntry->subject.in() );

    CPPUNIT_ASSERT( !CORBA::is_nil( m_masterclock.in() ) );

    std::cout << "MasterClock item exist into ClockService Catalog." << std::endl;
}

///////////////////////////////////////////////////////////////////////////////

void
NtpHostSynchro::test_ntp_synchronization()
{
    std::cout << std::endl;
    test_masterclock_definition();
    test_host_syncro_with_ntp();
    std::cerr << "** test execution successfull terminated **" << std::endl;
}

///////////////////////////////////////////////////////////////////////////////
//  End Of File
///////////////////////////////////////////////////////////////////////////////
