/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_VIRTUALCLOCKSYNCHRO_HPP
#define INCL_VIRTUALCLOCKSYNCHRO_HPP


#include <cppunit/extensions/HelperMacros.h>
#include <Foundation/common/String.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <Foundation/testutils/TestManager.hpp>
#include <Time/clockservice/test/testcommon/PerformanceManager.hpp>
#include <Time/clockservice/ClockNtp.hpp>
#include <Time/idllib/CosClockService.stub.hpp>
#include <Time/clockservice/Time.hpp>
#include <Time/clockservice/Util.hpp>

#include <sys/time.h>
#include <stdio.h>
#include <fstream>
#include <iostream>
#include <sstream>
#include <string>

class NtpVirtualClockSynchro
            : public CppUnit::TestFixture
{
    CPPUNIT_TEST_SUITE( NtpVirtualClockSynchro );
    CPPUNIT_TEST( test_controlledclock_synchro );
    CPPUNIT_TEST_SUITE_END( );

private:

    CORBA::Object_ptr
        m_ptr_masterclock,
        m_ptr_clockservice;

    CosClockService::Clock_var
    m_masterclock;

    CosClockService::ClockCatalog_var
        m_clock_catalog;

    CosClockService::ControlledClock_var
        m_controlledclock;

    std::string
    m_masterclock_name,
    m_clock_name;

    /**
     * @brief test NTP host synchronization.
     */
    void test_host_syncro_with_ntp();

    /**
     * @brief test MasterClock definition.
     */
    void test_masterclock_definition();

    /**
     * @brief test VirtualClock synchronization with MasterClock.
     */
    void test_virtualclock_syncro();

public:
    /**
     * @brief class Ctor.
     */
    NtpVirtualClockSynchro();

    /**
     * @brief Tests setup.
     */
    void setUp();

    /**
     * @brief test NTP host synchronization after reboot/restart.
     */
    void test_controlledclock_synchro();

    /**
     * @brief Finalize tests.
     */
    void tearDown() ;
};

#endif // INCL_VIRTUALCLOCKSYNCHRO_HPP
