/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef CDMW_RECORDING_BUFFERSENDER_HPP
#define CDMW_RECORDING_BUFFERSENDER_HPP

#include <TraceAndPerf/recordinglibrary/PacketStore.hpp>
#include <TraceAndPerf/recordinglibrary/CoalescenceManager.hpp>
#include <TraceAndPerf/recordingidl/CdmwRecordingRecorder.stub.hpp>

namespace Cdmw
{
   namespace Recording
   {
      struct Buffer
      {
         Buffer * next ; // For chaining purposes
         timeval date ; // Starting date as seen from first message in the buffer
         CdmwRecording::GroupHeader header ;
         size_t size ;  // Current amount of bytes in the buffer
         size_t maxSize ;  // Max amount of bytes that can be put in the buffer
         octet * data ; // sequence of octets representing the stored request/reply pairs
      } ;


      // Forwards
      class CoalescenceManager ;

      /**
       *Purpose: Handle the emission of the buffers to the recording
       * server
       *<p> .
       *
       */
      class BufferSender
      {
         public :
            BufferSender()
                  throw() ;

            virtual ~BufferSender()
                  throw() ;


            /**
             *Purpose: setter and getter for the URL of the recording server
             *<p> .
             *
             */
            virtual void recordingURL(const char * url)
                  throw() ;
            //caller shall call CORBA::string_free()
            virtual char * recordingURL()
                  throw() ;


            /**
             *Purpose: Make the plugin ready to work
             *<p> .
             *
             */
            virtual void initialize(CoalescenceManager * manager, CORBA::ORB_ptr orb)
                  throw() ; 

            /**
             *Purpose: Finishes all processing
             *<p> .
             *
             */
            virtual void finalize()
                  throw() ;


            /**
             *Purpose: Sends the buffer now
             *<p> .
             *
             */
            virtual void sendBuffer(Buffer *& buffer)
                  throw() ;

            /**
             *Purpose: Set the maximum buffer number
             *<p> .
             *
             */
            virtual void SetMaxBufferNumber (size_t size);

            /**
             *Purpose: Get the maximum buffer number
             *<p> .
             *
             */
             virtual size_t GetMaxBufferNumber ();

            /**
             *Purpose: Set the retry period
             *<p> .
             *
             */
            virtual void SetRetryPeriod (Milliseconds delay);


            /**
             *Purpose: Get the retry period
             *<p> .
             *
             */
            virtual Milliseconds GetRetryPeriod();
            
            /**
             *Purpose: Set the max number of retries
             *<p> .
             *
             */
            virtual void SetRetryNumber (long number);

            /**
             *Purpose: Set the Corba Relative Roundtrip Timeout
             *<p> .
             *
             */
            virtual void SetCorbaTimeout (unsigned long timeout);
            
         private :
            OsSupport::Mutex m_mutex ;

            std::string m_recordingURL ;

            class Worker ;

            Worker * m_worker ;            
      } ;
   }
}
#endif
