/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef CDMW_RECORDING_COALESCENCEMANAGER_HPP
#define CDMW_RECORDING_COALESCENCEMANAGER_HPP

#include <TraceAndPerf/recordinglibrary/PacketStore.hpp>
#include <TraceAndPerf/recordinglibrary/BufferSender.hpp>

namespace Cdmw
{
   namespace Recording
   {

      // Forwards
      struct Buffer ;
      class BufferSender ;

      /**
       *Purpose: Manage the buffering of request/reply pairs
       *<p> .
       *
       */
      class CoalescenceManager
      {
         public:
            CoalescenceManager (BufferSender & sender)
                  throw() ;

            virtual ~CoalescenceManager ()
                  throw() ;


            /**
             *Purpose: Make the manager ready to work
             *<p> .
             *
             */
            virtual void initialize()
                  throw() ; 

            /**
             *Purpose: Finishes all processing
             *<p> .
             *
             */
            virtual void finalize()
                  throw() ;


            /**
             *Purpose: Set the maximum buffer size
             *<p> .
             *
             */
            virtual void MaxSize (size_t size);

            /**
             *Purpose: Get the maximum buffer size
             *<p> .
             *
             */
            virtual size_t MaxSize () { return m_maxSize ; }

            /**
             *Purpose: Set the timeout
             *<p> .
             *
             */
            virtual void TimeOut (Milliseconds delay);

            /**
             *Purpose: Get the timeout
             *<p> .
             *
             */
            virtual Milliseconds TimeOut ();

            /**
             *Purpose: Write the request/reply pair in the sending
             * buffer
             *<p> .
             *
             */
            virtual void WriteOperation(Message * request, Message * reply)
                  throw() ;

            /**
             *Purpose: Sends the buffer now
             *<p> .
             *
             */
            virtual void flush()
                  throw() ;

            /**
             *Purpose: Return a used buffer to the free list
             *<p> .
             *
             */
            void free_buffer(Buffer *& buffer,  bool already_mutexed = false)
                  throw() ;

         private :
            OsSupport::Mutex m_mutex ;
            std::string m_hostname;

            // Forward declaration of worker thread implementation
            class Sequencer ;

            Sequencer * m_sequencer ;

            size_t m_maxSize ;
            Milliseconds m_timeOut ;

            BufferSender & m_sender ;

            // Free list of buffers
            Buffer * m_free_buffers ;
            Buffer * m_current_buffer ;
      } ;
   }
}

#endif
