/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef CDMW_RECORDING_INTERFACEFILTER_HPP
#define CDMW_RECORDING_INTERFACEFILTER_HPP

#include <TraceAndPerf/recordingidl/CdmwRecordingDaemonControl.stub.hpp>

#include <map>
#include <vector>

using namespace std;

namespace Cdmw
{
   namespace Recording
   {
     /**
       *Purpose: checker for the matching interfaces - flag object to
       *         be recorded.
       *         InterfaceFilter class is not thread-safe
       *         (it is up to the user to provide protection for concurrent updates)
       *    
       *    
       */
      class InterfaceFilter
      {
         public :
            enum Status { INTF_OK, INTF_ADDED, INTF_REMOVED, INTF_INVALID,
                          INTF_ALREADY_ACTIVE,
                          INTF_ALREADY_REMOVED,
                          INTF_NEVER_SELECTED} ;
            typedef unsigned char octet ;
            struct Flag_t {
              bool ownflag;
              int  counter;
            };
            typedef std::vector<string> Intf_vector_t ;
            typedef void (*Display_flag)(string const &, Flag_t const &);
            typedef void (*Display)(string const &, string const &, size_t);
            /**
             *Purpose:
             *<p> This class checks for the matching interfaces
             * and flags object to be recorded
             *
             */

            InterfaceFilter (const char * filename)
                throw();
            
            
            virtual ~InterfaceFilter ()
                  throw();

            /**
             *Purpose: 
             *<p> checks "if an object in the Object Map
             * implements one of the stored interfaces then flag
             * it to be recorded".
             *
             */
            virtual bool checkInterface (const char *object_id)
                  throw() ;

            /**
             *Purpose:
             *<p> adds the requested interfaces to the activated interface store.
             *
             */
            virtual Status addInterfaces (CdmwRecording::String_List & list)
                throw() ;

            /**
             *Purpose:
             *<p> removes the requested interfaces from the  activated interface store.
             *
             */
            virtual Status removeInterfaces (CdmwRecording::String_List & list)
                throw() ;


            /**
             *Purpose:
             *<p> clear the  activated interface store.
             *
             */
            virtual Status removeAllInterfaces ()
                throw() ;            


           /**
             *Purpose:
             *<p> return the interface map size
             *
             */
             virtual int list_size();

            /**
             *Purpose:
             *<p> map iterator.
             *
             */
            virtual void mapLoop (Display, Display_flag)
                throw() ;


         private :
            /**
             * Body 
             */

           /**
            *Purpose:
            *<p> adds the interface to the interface store.
            *
            */
           virtual Status addInterface (std::string & intf)
               throw() ;
                      
           /**
             *Purpose:
             *<p> select the interface for recording.
             *
             */
            virtual Status selectInterface (std::string & intf)
                  throw() ;           

            /**
             *Purpose:
             *<p> select the interfaces for recording.
             *
             */
            virtual Status selectInterfaces (CdmwRecording::String_List & list)
                throw() ;
            
            /**
             *Purpose:
             *<p> remove the interface from the selected interface
             *list. (deselect it)
             *
             */
            virtual Status removeInterface (std::string & intf)
                  throw() ;

             /**
              *Purpose:
              *<p> recursively increment children activation counter
              *
              */
             virtual void incr_activation_counter (std::string & intf0, std::string & intf);

             /**
              *Purpose:
              *<p> recursively decrement children activation counter
              *
              */
             virtual void decr_activation_counter (std::string & intf0, std::string & intf);
             /**
              *Purpose:
              *<p> put a child only once in the children list
              *
              */
             virtual void addChild (std::string const & parent, std::string const & child);
             /**
              *Purpose:
              *<p> put the rep_id as a child of its direct ancestor, and
              *return a flag struct
              *
              */
             //virtual Flag_t get_flag (std::string const & rep_id, std::string const & str);
             virtual bool get_flag (std::string const & rep_id, std::string const & str);
             
             /**
              *Purpose:
              *<p> map iterator : finalize interface lists
              *
              */
             virtual void finalizeMap ()
                 throw();
             
             // list of interfaces
             std::map <string, Flag_t> m_interface_list;

             // list of the children interfaces
             std::map <string, Intf_vector_t> m_children_list;
      } ;
 
   }
}

#endif
