/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef CDMW_RECORDING_MESSAGEIDENTIFIER_HPP
#define CDMW_RECORDING_MESSAGEIDENTIFIER_HPP

#include <Foundation/osthreads/Thread.hpp>
#include <TraceAndPerf/recordinglibrary/OperationMap.hpp>
#include <TraceAndPerf/recordinglibrary/ObjectMap.hpp>
#include <TraceAndPerf/recordinglibrary/InterfaceFilter.hpp>
//#include "TraceAndPerf/recordinglibrary/PacketStore.hpp"
#include <TraceAndPerf/recordinglibrary/RequestPairing.hpp>

namespace Cdmw
{
   namespace Recording
   {
     
     /**
       *Purpose:
       *<p> .
       *
       */
      class MessageIdentifier
      {
         public :

            // Forward declaration of worker thread implementation
            class Worker ;

            /**
             *Purpose:
             *<p> This class is the workhorse of the message processing
             *
             */
            enum Status { OK, FAILED, INVALID } ;

            MessageIdentifier (PacketStore & store,
                               ObjectMap & object_map,
                               InterfaceFilter & interface_filter,
                               OperationMap & operation_map,
                               RequestPairing & pairing)
                  throw();

            virtual ~MessageIdentifier ()
                  throw() ;

            /**
             *Purpose: starts the processing of messages
             *<p> .
             *
             */
            virtual Status start ()
                  throw() ;

            /**
             *Purpose: stops the message processing
             *<p> .
             *
             */
            virtual Status stop ()
                  throw() ;

            /**
             *Purpose: terminate the message processing and its associated thread
             *<p> .
             *
             */
            virtual Status terminate ()
                  throw() ;

            /**
             *Purpose: entry point for the message processing thread
             *<p> .
             *
             */
            virtual Status run ()
                  throw() ;

         private :
            /**
             * Body 
             */

            OsSupport::Mutex m_mutex ;


            bool m_started ;
            bool m_stopped ;
            bool m_terminated ;


            PacketStore & m_packet_store ;
            ObjectMap & m_object_map ;
            InterfaceFilter & m_interface_filter ;
            OperationMap & m_operation_map ;
            RequestPairing & m_request_pairing ;
            
            Worker * m_worker ;
      } ;

   }

}

#endif
