/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
#ifndef CDMW_RECORDING_OPERATIONMAP_HPP
#define CDMW_RECORDING_OPERATIONMAP_HPP

#include <TraceAndPerf/recordingidl/CdmwRecordingDaemonControl.stub.hpp>

#include <string>

#include <vector>

namespace Cdmw
{
   namespace Recording
   {
      /**
       *Purpose: checker for the matching operations
       *         OperationMap class is not thread-safe
       *         (it is up to the user to provide protection for concurrent updates)
       *<p> .
       *
       */
      class OperationMap
      {
         public :
           enum Status {
             OPERATION_OK,
             OPERATION_NOT_FOUND,
             OPERATION_ADDED,
             OPERATION_REMOVED,
             OPERATION_ALREADY_EXISTS,
             OPERATION_NOP
           } ;

            typedef void (*display)(std::string const &, size_t);

            OperationMap (const char * filename)
                  throw();

            virtual ~OperationMap ()
                  throw() ;

            /**
             *Purpose: check if an operation is to be recorded
             * ie matches one of those in the provided list of operations.
             * If list is empty, all operations are recorded 
             *<p> .
             * NOTA: For performance reasons, it is assumed that oper_name
             *       has no leading or trailing space/tab since operation
             *       read from GIOP buffer does not contain any.
             *       Beside that, any leading/trailing spaces in the operations 
             *       files are removed at file reading time.
             *       
             *
             */
            virtual bool checkOperation(std::string & oper_name)
                  throw() ;

            /**
             *Purpose:
             *<p> adds the requested operations to the operations store.
             *
             */
            virtual Status addOperations (CdmwRecording::String_List & operations)
                  throw() ;

            /**
             *Purpose:
             *<p> removes the requested interfaces from the operations store.
             *
             */
            virtual Status removeOperations (CdmwRecording::String_List & operations)
                  throw() ;


            /**
             *Purpose:
             *<p> clear the operations store.
             *
             */
            virtual Status removeAllOperations ()
                  throw() ;


             /**
             *Purpose:
             *<p> return the operation map size
             *
             */
             virtual int list_size();

            /**
             *Purpose:
             *<p> map iterator.
             *
             */
            virtual void mapLoop (display)
                throw() ;

            /**
             * Filtering mode. What to do with operations read from the operations file?
             *   FILTERING_MODE_MATCH   : Only operations from file will be recorded.
             *   FILTERING_MODE_EXCLUDE : Operations from file will not be recorded.
             */
            enum FilteringMode {
               FILTERING_MODE_MATCH,
               FILTERING_MODE_EXCLUDE
            };

            /**
             *Purpose:
             *<p> return the filering mode in use
             *
             */
            inline FilteringMode get_filtering_mode() const
            { 
               return m_filtering_mode; 
            }

         private :
           /**
            * Body 
            */
           /**
            *Purpose:
            *<p> adds the requested operation to the operations store.
            *
            */
            virtual Status addOperation (std::string & operation)
               throw() ;

           /**
            *Purpose:
            *<p> removes the requested interface from the operations store.
            *
            */
           virtual Status removeOperation (std::string & operation)
               throw() ;
            bool
            explode_action_line(const std::string& line, 
                                std::string & name, std::string & value,
                                const char* filename, size_t ln);

            bool
            explode_operation_line(const std::string& line, 
                                   std::string &      opname,
                                   const char* filename, size_t ln);
            
            FilteringMode m_filtering_mode;
            std::vector<std::string> m_operation_list;

      } ;
   }
}

#endif
