/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef CDMW_RECORDING_PLUGIN_HPP
#define CDMW_RECORDING_PLUGIN_HPP

#include <Foundation/orbsupport/OrbSupport.hpp>
#include <TraceAndPerf/recordinglibrary/PacketStore.hpp>

namespace Cdmw
{
   namespace Recording
   {
      typedef octet PluginType ;

      /**
       *Purpose: Plugin abstract class
       *<p> .
       *
       */
      class Plugin
      {
         public:
            Plugin ()
                  throw() : next(0) {}

            virtual ~Plugin ()
                  throw() {}

            /**
             *Purpose: return the plugin type, different plugin classes
             * must return different values
             *<p> .
             *
             */
            virtual PluginType type()
                  throw() = 0 ;

            /**
             *Purpose: Set the recording destination URL for a plugin
             *<p> .
             *
             */
            virtual bool Destination (const char * destination)
                  throw() = 0 ;

            /**
             *Purpose: Get the recording destination URL for a plugin
             *       it is a non-CORBA operation that follows CORBA C++
             *       language mapping  (caller shall call
             *       CORBA::string_free())
             *
             */
            virtual char * Destination ()
                  throw() = 0 ;

            /**
             *Purpose: Process the request-reply pair given as argument
             *<p> .
             *
             */
            virtual void recordOperation (Message * request, Message * reply)
                  throw() = 0 ;

            /**
             *Purpose: initialize the plugin
             *<p> .
             *
             */
            virtual void initialize (CORBA::ORB_ptr orb)
                  throw() = 0 ;

            /**
             *Purpose: flush the contents of the plugin
             *<p> .
             *
             */
            virtual void finalize ()
                  throw() = 0 ;


            Plugin * next ;

      } ;
   }
}

#endif
