/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef CDMW_RECORDING_RECORDINGPLUGIN_HPP
#define CDMW_RECORDING_RECORDINGPLUGIN_HPP

#include "TraceAndPerf/recordinglibrary/Recording_Default_Parameters.hpp"
#include "TraceAndPerf/recordinglibrary/Plugin.hpp"
#include "TraceAndPerf/recordinglibrary/CoalescenceManager.hpp"
#include "TraceAndPerf/recordinglibrary/BufferSender.hpp"

namespace Cdmw
{
   namespace Recording
   {

     typedef int milliseconds ;

      /**
       *Purpose: Buffer request/reply pairs and send the result to the
       * recording server
       *<p> .
       *
       */
      class RecordingPlugin : public virtual Plugin
      {
         public:
            RecordingPlugin ()
                  throw() ;
            
            virtual ~RecordingPlugin ()
                  throw() ;

            /**
             *Purpose: return the plugin type, different plugin classes
             * must return different values
             *<p> .
             *
             */
            virtual PluginType type()
                  throw() { return RECORDING_PLUGIN_TYPE ; }

            /**
             *Purpose: Set the recording destination URL
             *<p> .
             *
             */
            virtual bool Destination (const char * destination)
                  throw() ;

            /**
             *Purpose: Get the recording destination URL
             *       it is a non-CORBA operation that follows CORBA C++
             *       language mapping  (caller shall call
             *       CORBA::string_free())
             *<p> .
             *
             */
            virtual char * Destination ()
                  throw() ;

            /**
             *Purpose: Process the request-reply pair given as argument
             *<p> .
             *
             */
            virtual void recordOperation (Message * request, Message * reply)
                  throw() ;

            /**
             *Purpose: initialize the plugin
             *<p> .
             *
             */
            void initialize (CORBA::ORB_ptr orb)
                throw() ;
            

            /**
             *Purpose: Set the maximum buffer size
             *<p> .
             *
             */
            virtual void MaxSize (size_t size);

            /**
             *Purpose: Get the maximum buffer size
             *<p> .
             *
             */
            virtual size_t MaxSize ();

            /**
             *Purpose: Set the timeout
             *<p> .
             *
             */
            virtual void TimeOut (milliseconds delay);

            /**
             *Purpose: Get the timeout
             *<p> .
             *
             */
            virtual milliseconds TimeOut ();

            /**
             *Purpose: Set the Corba Relative Roundtrip Timeout
             *<p> .
             *
             */
            virtual void SetCorbaTimeout (unsigned long timeout);

            /**
             *Purpose: Set the maximum buffer number
             *<p> .
             *
             */
            virtual void SetMaxBufferNumber (size_t size);

            /**
             *Purpose: Get the maximum buffer number
             *<p> .
             *
             */
            virtual size_t GetMaxBufferNumber ();

            /**
             *Purpose: Set the retry period
             *<p> .
             *
             */
            virtual void SetRetryPeriod (milliseconds delay);

            /**
             *Purpose: Get the retry period
             *<p> .
             *
             */
            virtual milliseconds GetRetryPeriod();

            /**
             *Purpose: Set the max number of retries
             *<p> .
             *
             */
            virtual void SetRetryNumber (long number);
            
            /**
             *Purpose: flush the contents of the plugin
             *<p> .
             *
             */
            virtual void finalize ()
                  throw() ;


         private :
            CoalescenceManager * m_manager ;
            BufferSender * m_sender ;

      } ;
   }
}

#endif
