/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef CDMW_RECORDING_DEFAULT_PARAMETERS_HPP
#define CDMW_RECORDING_DEFAULT_PARAMETERS_HPP

#include <Foundation/logging/LogLevel.hpp>

namespace
{
   const int GIOP_1_2 = 0x102 ;
   const char TAB_CHARACTER = '\t';
}

namespace Cdmw
{
  namespace Recording
  {

     /**
     *Purpose: the basic value of Logging Level for Recording Daemon
     *
     *<p> .
     *
     */
    const int RECORDING_DAEMON_LOG_LEVEL        = Cdmw::Logging::CDMW_DEBUG_1;

    /**
     *Purpose: constants used by PlugIn
     *<p> .
     *
    */
     const short RECORDING_PLUGIN_TYPE = 1;
     const short DEBUG_PLUGIN_TYPE     = 2 ;
     /**
      *Purpose: constants used by PacketStore and RequestPairing
      *<p> .
      *
      *         Orphaned requests are those requests with no 
      *         reply within pre-defined time frame. This timeout
      *         must be set as big as possible depending
      *         on your application
     */
     const unsigned long  DEFAULT_ORPHAN_REQUEST_TIMEOUT_MILLISEC = 3600*1000;
     /**
      *Purpose: constants used by RequestPairing
      *<p> .
      *
      */
     const int  MSG_REQUESTPAIRING_LOG_LEVEL        = RECORDING_DAEMON_LOG_LEVEL;
     const int  CYCLIC_MSG_REQUESTPAIRING_LOG_LEVEL = Cdmw::Logging::CDMW_DEBUG_2;
    /**
     *Purpose: constants used by BufferSender
     *<p> .
     *
     */
     const int MSG_BUFFERSENDER_LOG_LEVEL        = RECORDING_DAEMON_LOG_LEVEL;
     const int CYCLIC_MSG_BUFFERSENDER_LOG_LEVEL = Cdmw::Logging::CDMW_DEBUG_2;
     const int DEFAULT_POLLING_PERIOD_MILLISEC   = 1000;
     const int DEFAULT_NUMBER_MAX_OF_BUFFERS     = 100;
     const int DEFAULT_CORBA_TIMEOUT_MILLISEC    = 2000;
     
     /**
      *Purpose: constants used by CoalescenceManager
      *<p> .
      *
      */
     const int MSG_COALESCENCEMANAGER_LOG_LEVEL = RECORDING_DAEMON_LOG_LEVEL;
     const int CYCLIC_MSG_COALESCENCEMANAGER_LOG_LEVEL = Cdmw::Logging::CDMW_DEBUG_2;
     const long int IDLE_TIMEOUT = 3600000;
     const int DEFAULT_BUFFER_MAX_SIZE = 8*1024;
     const int DEFAULT_SENDING_TIMEOUT = 100;
     

     /**
      *Purpose: constants used by NetworkSniffer
      *<p> .
      *
      */
     const int  NETWORKSNIFFER_LOG_LEVEL = RECORDING_DAEMON_LOG_LEVEL;
     const int  CYCLIC_NETWORKSNIFFER_LOG_LEVEL = Cdmw::Logging::CDMW_DEBUG_2;
     /**
      *Purpose: constants used by MessageIdentifier
      *<p> .
      *
      */
     const int MSG_IDENTIFIER_LOG_LEVEL = RECORDING_DAEMON_LOG_LEVEL;
     const int CYCLIC_MSG_IDENTIFIER_LOG_LEVEL = Cdmw::Logging::CDMW_DEBUG_2;
     /**
      *Purpose: constants used by DebugPlugin
      *<p> .
      *
      */
     const int DEBUGPLUGIN_LOG_LEVEL = RECORDING_DAEMON_LOG_LEVEL ;
     const int CYCLIC_DEBUGPLUGIN_LOG_LEVEL = Cdmw::Logging::CDMW_DEBUG_2;
     const int DEFAULT_MAX_DEBUG_FILE_SIZE =  1000000000 ;
     /**
      *Purpose: constants used by ObjectMap
      *<p> .
      *
      */
     const int OBJECTMAP_FILTER_LOG_LEVEL = RECORDING_DAEMON_LOG_LEVEL ;
     const int CYCLIC_OBJECTMAP_FILTER_LOG_LEVEL = Cdmw::Logging::CDMW_DEBUG_2;

     /**
      *Purpose: constants used by InterfaceFilter
      *<p> .
      *
      */

     const int INTERFACE_FILTER_LOG_LEVEL = RECORDING_DAEMON_LOG_LEVEL ;
     const int CYCLIC_INTERFACE_FILTER_LOG_LEVEL = Cdmw::Logging::CDMW_DEBUG_2;

      /**
       *Purpose: constants used by PacketStore
       *<p> .
       *
       *         Orphaned requests are those requests with no 
       *         reply within pre-defined time frame. This timeout
       *         must be set as big as possible depending
       *         on your application
       */
     const unsigned long DEFAULT_SOCKET_READ_BUFFER_LENGTH = 1024*1024;
  }
}

#endif
