/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef CDMW_RECORDING_REQUESTPAIRING_HPP
#define CDMW_RECORDING_REQUESTPAIRING_HPP

#include <TraceAndPerf/recordinglibrary/PacketStore.hpp>
#include <TraceAndPerf/recordinglibrary/Plugin.hpp>

namespace Cdmw
{
   namespace Recording
   {

      /**
       *Purpose: Store the requests until the corresponding reply is
       * received, then send both to the available plugins
       * Requestpairing class is not thread-safe
       * (it is up to the user to provide protection for concurrent updates)
       *<p> .
       *
       */
      class RequestPairing
      {
         public:
            RequestPairing (PacketStore & store)
                  throw() ;

            virtual ~RequestPairing ()
                  throw() ;

            /**
             *Purpose: add a recording plugin to the plugin list
             *<p> .
             *
             */
            virtual void addPlugin (Plugin * plugin)
                  throw() ;

            /**
             *Purpose: Sets the recording destination URL for a plugin
             *<p> .
             *
             */
            virtual void setDestination (PluginType type, const char * destination)
                  throw() ;

            /**
             *Purpose: Store a request until the corresponding reply is
             * received
             *<p> .
             *
             */
            virtual void storeRequest (Message * request)
                  throw() ;

            /**
             *Purpose: Check if some requests are not orphan
             *<p> .
             *
             */
            virtual void checkOrphanRequests ()
                  throw() ;

            /**
             *Purpose: Store a fragment, chaining it to the corresponding request or reply
             *<p> .
             *
             */
            virtual void storeFragment (Message * fragment)
                  throw() ;

            /**
             *Purpose: Check if a reply is to be recorded, and if so send
             * it and its request to the plugins
             *<p> .
             *
             */
            virtual void checkReply (Message * reply)
                  throw() ;

            /**
             *Purpose: clear the contents: plugins and requests
             *<p> .
             *
             */
            virtual void finalize ()
                  throw() ;

         private :

            // Plugins list
            Plugin * m_plugins ;

            struct RequestQueue
            {
               RequestQueue * next ;
               Connection request_channel ;
               Connection reply_channel ;
               Message * requests ;
            } ;

            RequestQueue * m_request_queue ;
            RequestQueue * m_free_queue ;

            PacketStore & m_store ;
      } ;
   }
}

#endif
