/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef CDMW_RECORDING_SWAPPING_HPP
#define CDMW_RECORDING_SWAPPING_HPP

#include "TraceAndPerf/recordinglibrary/PacketStore.hpp"

namespace Cdmw
{
  namespace Recording
  {

    /**
     *Purpose: Class for swapping according to endianness
     *<p> .
     *
     */
    class Swapping
    {
      public:

        /**
         *Purpose: Swap 2 bytes according to endianness 
         *<p> .
         *
         */
        static inline void Swap2 (octet * dst, octet * src, bool need_swap)
        {
          if (need_swap)
          {
            dst [0] = src [1] ;
            dst [1] = src [0] ;
          }
          else
          {
            dst [0] = src [0] ;
            dst [1] = src [1] ;
          }
        }

        /**
         *Purpose: Swap 4 bytes according to endianness 
         *<p> .
         *
         */
        static inline void Swap4 (octet * dst, octet * src, bool need_swap)
        {
          if (need_swap)
          {
            dst [0] = src [3] ;
            dst [1] = src [2] ;
            dst [2] = src [1] ;
            dst [3] = src [0] ;
          }
          else
          {
            dst [0] = src [0] ;
            dst [1] = src [1] ;
            dst [2] = src [2] ;
            dst [3] = src [3] ;
          }
        }

        /**
         *Purpose: Swap 8 bytes according to endianness 
         *<p> .
         *
         */
        static inline void Swap8 (octet * dst, octet * src, bool need_swap)
        {
          if (need_swap)
          {
            dst [0] = src [7] ;
            dst [1] = src [6] ;
            dst [2] = src [5] ;
            dst [3] = src [4] ;
            dst [4] = src [3] ;
            dst [5] = src [2] ;
            dst [6] = src [1] ;
            dst [7] = src [0] ;
          }
          else
          {
            dst [0] = src [0] ;
            dst [1] = src [1] ;
            dst [2] = src [2] ;
            dst [3] = src [3] ;
            dst [4] = src [4] ;
            dst [5] = src [5] ;
            dst [6] = src [6] ;
            dst [7] = src [7] ;
          }
        }

        // In-place swapping
        /**
         *Purpose: Swap 2 bytes in-place according to endianness 
         *<p> .
         *
         */
        static inline void Swap2 (octet * bytes, bool need_swap)
        {
          if (need_swap)
          {
            octet tmp = bytes [0] ;
            bytes [0] = bytes [1] ;
            bytes [1] = tmp ;
          }
        }

        /**
         *Purpose: Swap 4 bytes in-place according to endianness 
         *<p> .
         *
         */
        static inline void Swap4 (octet * bytes, bool need_swap)
        {
          if (need_swap)
          {
            octet tmp = bytes [0] ;
            bytes [0] = bytes [3] ;
            bytes [3] = tmp ;
            tmp = bytes [1] ;
            bytes [1] = bytes [2] ;
            bytes [2] = tmp ;
          }
        }

        /**
         *Purpose: Swap 8 bytes in-place according to endianness 
         *<p> .
         *
         */
        static inline void Swap8 (octet * bytes, bool need_swap)
        {
          if (need_swap)
          {
            octet tmp = bytes [0] ;
            bytes [0] = bytes [7] ;
            bytes [7] = tmp ;
            tmp = bytes [1] ;
            bytes [1] = bytes [6] ;
            bytes [6] = tmp ;
            tmp = bytes [2] ;
            bytes [2] = bytes [5] ;
            bytes [5] = tmp ;
            tmp = bytes [3] ;
            bytes [3] = bytes [4] ;
            bytes [4] = tmp ;
          }
        }
    } ;
  }
}

#endif
