/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/*
 *  Ethernet Header (in bytes)
 *    0    1    2    3    4    5    6    7    8    9   10   11   12   13
 * +----+----+----+----+----+----+----+----+----+----+----+----+----+----+
 * |          Source Address     |    Destination Address      | Length  |
 * +----+----+----+----+----+----+----+----+----+----+----+----+----+----+
 *
 */

/*
 *  IP Header (in bits)
 *
 *     0                   1                   2                   3   
 *     0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 
 *    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *    |Version|  IHL  |Type of Service|          Total Length         |
 *    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *    |         Identification        |Flags|      Fragment Offset    |
 *    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *    |  Time to Live |    Protocol   |         Header Checksum       |
 *    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *    |                       Source Address                          |
 *    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *    |                    Destination Address                        |
 *    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *    |                    Options                    |    Padding    |
 *    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *
 */

/*
 *  TCP header (in bits)
 * 
 *  0                   1                   2                   3   
 *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * |          Source Port          |       Destination Port        |
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * |                        Sequence Number                        |
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * |                    Acknowledgment Number                      |
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * |  Data |           |U|A|P|R|S|F|                               |
 * | Offset| Reserved  |R|C|S|S|Y|I|            Window             |
 * |       |           |G|K|H|T|N|N|                               |
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * |           Checksum            |         Urgent Pointer        |
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * |                    Options                    |    Padding    |
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * |                             data                              |
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *
 */

#include <stdio.h>
#include <pcap.h>
#include <signal.h>
#include <netinet/in.h>
#include <sys/socket.h>

#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>

#include "Foundation/ossupport/SystemProperties.hpp"
#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/Logger.hpp>
#include "Foundation/ossupport/OS.hpp"

#include "TraceAndPerf/recordinglibrary/NetworkSniffer.hpp"
#include "TraceAndPerf/recordinglibrary/Recording_Default_Parameters.hpp"

namespace
{
    const int PORTS_INCREMENT = 16;
    const int MAX_PACKET_SIZE = 65535;
    const int NON_PROMISCUOUS = 0;
    const int NO_TIMEOUT      = 0;
    const int NO_NETMASK      = 0;
    const int OPTIMIZE_FILTER = 1;
    const int FOREVER         = -1;

    const char ALL_INTERFACES [] = "any" ;

    const std::string DEPRECATED_INITIAL_TCPPORTS_LIST_NAME ("Initial_TCPPorts_List") ;
    const std::string INITIAL_TCPPORTS_LIST_NAME ("Cdmw.Recording.InitialTCPPortsList") ;
    const std::string SOCKET_READ_BUFFER_LENGTH_NAME ("Cdmw.Recording.SocketReadBufferLength");
}

namespace Cdmw
{
   namespace Recording
   {
   
     
     Cdmw::Logging::Logger* networksniffer_logger;
     bool worker_inited = false; // avoid blocking when started with non-root privileges

     // Packet capture handler
     void Treat_Each_Packet (u_char * user, const struct pcap_pkthdr * packet_header, const u_char * bytes) ;
     
     // CDO-070330-PRS-1-567 : #53 : refused 
     // Signals Framework of cardamom does not
     // allow to send a signal precisely to a thread
     // 
     static sigset_t   signal_mask;  /* signals to block         */
      
     void breakloop_handler (int signum)
     {
     }

 
      class NetworkSniffer::Worker : public virtual OsSupport::Thread
      {
         public :
            Worker (NetworkSniffer & sniffer) throw()
                  : OsSupport::Thread()
                  , m_id(0)
                  , m_control(sniffer.m_mutex)
                  , m_pcap_handle(0)
                  , m_local_addresses(0)
                  , m_fcode ()
                  , m_restarted (false)
                  , m_sniffer(sniffer)
            {
            }

            virtual ~Worker () throw()
            {
               if (m_pcap_handle != 0)
               {
                  pcap_close(m_pcap_handle);
               }
               m_pcap_handle = 0 ;

               if (m_local_addresses != 0)
               {
                 delete [] m_local_addresses ;
               }
               m_local_addresses = 0 ;
            }

            virtual Status init() throw()
            {
               char pcap_error_msg [1024] ;
               pcap_error_msg [0] = 0 ;

               // Get the local addresses
               pcap_if_t * interfaces ;
               int number = 0 ;
               if (pcap_findalldevs(&interfaces, pcap_error_msg) != 0)
               {
                  return FAILED ;
               }

               // Get the count of addresses
               pcap_if_t *tmp_if ;
               for (tmp_if = interfaces ; tmp_if != 0 ; tmp_if = tmp_if->next)
               {
                  pcap_addr_t * tmp_addr ;
                  for (tmp_addr = tmp_if->addresses ; tmp_addr != 0 ; tmp_addr = tmp_addr->next)
                  {
                     if (((sockaddr_in *) tmp_addr->addr)->sin_family == AF_INET)
                     {
                        number++ ;
                     }
                  }
               }

               if (number == 0)
               {
                  return INVALID ;
               }

               // Build the list of addresses (null terminated)
               m_local_addresses = new in_addr_t [number + 1] ;
               m_local_addresses [number] = 0 ;
               int index = 0 ;
               for (tmp_if = interfaces ; tmp_if != 0 ; tmp_if = tmp_if->next)
               {
                  pcap_addr_t * tmp_addr ;
                  for (tmp_addr = tmp_if->addresses ; tmp_addr != 0 ; tmp_addr = tmp_addr->next)
                  {
                     if (((sockaddr_in *) tmp_addr->addr)->sin_family == AF_INET)
                     {
                        m_local_addresses [index] = ((sockaddr_in *) tmp_addr->addr)->sin_addr.s_addr ;
                        index++ ;
                     }
                  }
               }

               pcap_freealldevs(interfaces);

               // Open the capture handle
               m_pcap_handle = pcap_open_live(ALL_INTERFACES, MAX_PACKET_SIZE, NON_PROMISCUOUS,
                                              NO_TIMEOUT, pcap_error_msg) ;

               if (m_pcap_handle == 0)
               {
                  // Error
                  CDMW_ERROR(networksniffer_logger,
                              "pcap_open_live failed " << pcap_error_msg) ;
                  return FAILED ;
               }

               int dlt_type = pcap_datalink(m_pcap_handle) ;
               if (dlt_type != DLT_LINUX_SLL)
               {
                  CDMW_ERROR(networksniffer_logger,
                    "Datalink layer type not handled !!!\n") ;
                  return FAILED ;
               }

               socklen_t actual_buffer = 0;
               socklen_t desired_buffer = Cdmw::OsSupport::SystemProperties::Get_Property
                   (SOCKET_READ_BUFFER_LENGTH_NAME, DEFAULT_SOCKET_READ_BUFFER_LENGTH); ;
               
               int status = setsockopt( pcap_fileno(m_pcap_handle), SOL_SOCKET, SO_RCVBUF,
                                        &desired_buffer, sizeof(desired_buffer));
               if(status < 0) {
                   CDMW_ERROR(networksniffer_logger,
                              "unable to increase pcap read buffer size to " << desired_buffer << strerror(errno));
                   return FAILED;
               }
               socklen_t optbuflen = sizeof(actual_buffer);
               status = getsockopt( pcap_fileno(m_pcap_handle), SOL_SOCKET, SO_RCVBUF,
                                    &actual_buffer, &optbuflen);
               if(status < 0) {
                   CDMW_ERROR(networksniffer_logger,
                              "unable to query pcap read buffer size afterward: " << strerror(errno));
                   return FAILED;
               }
               if(actual_buffer < desired_buffer) {
                   CDMW_WARN(networksniffer_logger,
                                "unable to resize pcap buffer to desired " << desired_buffer
                                << ", it increased from to " << actual_buffer ) ;
               }

               m_id = start() ;

               worker_inited = true; // inited here only!
               return OK ;
            }

            virtual void run() throw()
            {

              // CDO-070330-PRS-1-567 : #53 : refused              
              // Signals Framework of cardamom does not
              // allow to send a signal precisely to a thread
              // 
              
                  struct sigaction new_action;
              new_action.sa_handler = breakloop_handler;
              sigemptyset (&new_action.sa_mask);
              new_action.sa_flags = 0;
              if ( sigaction (SIGUSR1, &new_action, 0) == -1)
              {
                 CDMW_INFO(networksniffer_logger, "sigaction failed ");
              }

               sigemptyset (&signal_mask);
               sigaddset (&signal_mask, SIGUSR1);
               int rc = pthread_sigmask (SIG_UNBLOCK, &signal_mask, 0);
               if (rc != 0) {
                  CDMW_INFO(networksniffer_logger, "pthread_sigmask failed ");
                       }

               while (! m_sniffer.m_terminated)
               {
                  {
                     CDMW_MUTEX_GUARD(m_sniffer.m_mutex) ;

                     m_sniffer.m_stopped = true ;
                     m_control.broadcast() ;
                     while (! (m_sniffer.m_started || m_sniffer.m_terminated) )
                     {
                        m_control.wait() ;
                     }
                     if (m_sniffer.m_started)
                     {
                        m_sniffer.m_stopped = false ;
                        m_control.broadcast() ;
                     }
                  }

                  if (m_sniffer.m_started)
                  {
                     handle_packets() ;
                  }
               }
            }

            /**
             * Purpose: Restart the loop and reduce the risk of lost
             * messages
             *
             */
            virtual void restart() throw()
            {
              char filter [4096] ;

              setup_filter (filter) ;

              if (pcap_compile(m_pcap_handle, &m_fcode, filter, OPTIMIZE_FILTER, NO_NETMASK) < 0) 
              {
                 CDMW_ERROR(networksniffer_logger,
                    "pcap_compile failed " << pcap_geterr(m_pcap_handle) ) ;
                pcap_freecode(&m_fcode);
                return ;
              }

              m_restarted = true ;
              // Wake up the worker
              if (m_pcap_handle != 0)
              {
                pcap_breakloop(m_pcap_handle) ;
                pthread_kill (m_id, SIGUSR1);
              }
            }

            /**
             * Purpose: get the packets from the network and place them
             * in the packet store
             *
             */
            virtual void handle_packets() throw()
            {
               struct bpf_program fcode;
               char filter [4096] ;

               setup_filter (filter) ;

               if (pcap_compile(m_pcap_handle, &fcode, filter, OPTIMIZE_FILTER, NO_NETMASK) < 0) {
                  CDMW_ERROR(networksniffer_logger,
                        "pcap_compile failed " << pcap_geterr(m_pcap_handle) ) ;
                  pcap_freecode(&fcode);
                  return ;
               }
               if (pcap_setfilter(m_pcap_handle, &fcode) < 0) {
                  CDMW_ERROR(networksniffer_logger,
                        "pcap_setfilter failed " << pcap_geterr(m_pcap_handle) ) ;
                  pcap_freecode(&fcode);
                  return ;
               }

               // Now process the packets
               int result = 0 ;
               while (result != -2)
               {
                  result = pcap_loop (m_pcap_handle, FOREVER, Treat_Each_Packet, (u_char *)this) ;
                  if (result == -2)
                  {
                    // CDMW_INFO(networksniffer_logger, "pcap_loop broken ");
                  }
                  
                  if (m_restarted)
                  {
                    if (pcap_setfilter(m_pcap_handle, &m_fcode) < 0) {
                      CDMW_INFO(networksniffer_logger,
                          "pcap_setfilter failed " << pcap_geterr(m_pcap_handle)) ;
                      pcap_freecode(&m_fcode);
                      return ;
                    }
                    m_restarted = false ;
                    result = 0 ;
                  }

                  if (result == -1)
                  {
                     CDMW_ERROR(networksniffer_logger,
                       "pcap_loop failed " << pcap_geterr(m_pcap_handle) ) ;
                     break ;
                  }
               }

               pcap_freecode(&fcode);
            }

            /**
             * Purpose: setup the filter with the ports
             *
             */
            virtual void setup_filter(char * filter) throw()
            {
               int count ;
               char * filter_end = filter ;
               for (int each = m_sniffer.m_TCPPorts_Num - 1 ; each >= 0; each--)
               {
                  count = sprintf (filter_end, "(tcp port %d)", m_sniffer.m_TCPPorts[each]) ;
                  filter_end += count ;
                  if (each != 0)
                  {
                     count = sprintf (filter_end, " or ") ;
                     filter_end += count ;
                  }
               }

            }

            /**
             * Purpose: check if an address/port pair belongs to a list
             *
             */
            bool match (in_addr_t addr, unsigned short port, in_addr_t * addrList, unsigned short * portList)
            {
              if (portList == 0)
              {
                return true;
              }
              
               // First check the address
               for (in_addr_t * a_ptr = addrList ; a_ptr[0] != 0; a_ptr++)
               {
                  if (a_ptr[0] == addr)
                  {
                     // If it matches, check the port
                     for (unsigned short * p_ptr = portList ; p_ptr[0] != 0; p_ptr++)
                     {
                        if (p_ptr[0] == port)
                        {
                           return true ;
                        }
                     }
                     return false ;
                  }
               }

               return false ;
            }

            /**
             * Purpose: handle a packet and place it in the packet store
             *
             */
            void treat_packet (const struct pcap_pkthdr * packet_header, const u_char * bytes)
            {
               int link_offset = 16 ; // DLT_LINUX_SLL layer type
               // Would be 14 for DLT_EN10MB

               Packet packet ;
               const u_char * l_TCP_Header = bytes + link_offset + (bytes[link_offset] & 0xF) * 4 ; // IP header offset + IP info size
               packet.data = l_TCP_Header + ((l_TCP_Header[12] & 0xF0) >> 4) * 4 ; // TCP header offset + TCP info size
               packet.length = packet_header->caplen - (packet.data - bytes) ;
               packet.date = packet_header->ts ;

               packet.channel.src_addr = *((in_addr_t *) (bytes + link_offset + 12)) ; // IP header offset + src addr offset
               packet.channel.dst_addr = *((in_addr_t *) (bytes + link_offset + 16)) ; // IP header offset + dst addr offset
               packet.channel.src_port = ntohs(*((unsigned short *) (l_TCP_Header + 0))) ; // TCP header offset + src port offset
               packet.channel.dst_port = ntohs(*((unsigned short *) (l_TCP_Header + 2))) ; // TCP header offset + dst port offset

               if (match (packet.channel.dst_addr, packet.channel.dst_port,
                          m_local_addresses, m_sniffer.m_TCPPorts))
               {
                  packet.direction = Incoming ;
               }
               else if (match (packet.channel.src_addr, packet.channel.src_port,
                               m_local_addresses, m_sniffer.m_TCPPorts))
               {
                  packet.direction = Outgoing ;
               }
               else
               {
                  // Useless packet
                  return ;
               }

               // Store the packet for message reconstruction
               m_sniffer.m_store.pushPacket (packet) ;
            }

         // CDO-070330-PRS-1-567 : #54
         // private :
            OsSupport::Thread::ThreadId m_id ;
            OsSupport::Condition m_control ;
            pcap_t * m_pcap_handle ;
            in_addr_t * m_local_addresses ;
            struct bpf_program m_fcode ; 
            bool m_restarted ;

            private :
            NetworkSniffer & m_sniffer ;
      } ;

      void Treat_Each_Packet (u_char * user, const struct pcap_pkthdr * packet_header, const u_char * bytes)
      {
         NetworkSniffer::Worker * worker = (NetworkSniffer::Worker *) user ;

         worker->treat_packet (packet_header, bytes) ;
      }

      NetworkSniffer::NetworkSniffer (PacketStore & store) throw()
            : m_mutex()
            , m_TCPPorts (0)
            , m_TCPPorts_Length (0)
            , m_TCPPorts_Num (0)
            , m_started (false)
            , m_stopped (true)
            , m_terminated (true)
            , m_worker (0)
            , m_store(store)
      {
         networksniffer_logger = Cdmw::Logging::Logger::Get_Logger("CDMW.TRC.RecordingDaemon");
         std::string port_list = Cdmw::OsSupport::SystemProperties::Get_Property
                               (DEPRECATED_INITIAL_TCPPORTS_LIST_NAME, "");
         if (port_list != "")
         {
             CDMW_WARN(networksniffer_logger, 
                       "Prefer the use of ["
                       << INITIAL_TCPPORTS_LIST_NAME 
                       << "] rater than these of ["
                       << DEPRECATED_INITIAL_TCPPORTS_LIST_NAME << "]");
             Cdmw::OsSupport::SystemProperties::Set_Property
                               (INITIAL_TCPPORTS_LIST_NAME, port_list);
         }
         port_list = Cdmw::OsSupport::SystemProperties::Get_Property
                               (INITIAL_TCPPORTS_LIST_NAME, "");
         
         if (port_list != "")
         {
           unsigned int pos = 0;
           unsigned int index = 0;
           unsigned short port_num;
           std::string list = port_list;
           std::string port = "";
           while (index != std::string::npos)
           {
             index = list.find (",", pos);
             if (index == std::string::npos)
             {
               if (pos <= list.length())
               {
                 port = list.substr(pos, list.length() - pos + 1);
               }
             }
             else
             {
               port = list.substr(pos, index - pos + 1);
             }
             try
             {
               port_num = (unsigned short) atoi (port.c_str());
               CDMW_LOG(networksniffer_logger, 
                        NETWORKSNIFFER_LOG_LEVEL, 
                        "!!! Adding port : " << port_num << " to list to be spied");
               addPort (port_num);
             }
             catch (...)
             {
               CDMW_INFO(networksniffer_logger, "*** Bad port number");
             }
             pos = index + 1;
           }
         }
      }

      NetworkSniffer::~NetworkSniffer () throw()
      {
         if (! m_terminated)
         {
            terminate() ;
         }

         if (m_TCPPorts != 0)
         {
            delete [] m_TCPPorts ;
            delete [] m_TCPPorts_Use ;
            m_TCPPorts_Length = 0 ;
            m_TCPPorts_Num = 0 ;
         }
      }

      void NetworkSniffer::TCPPorts (unsigned short * ports) throw()
      {
         if (m_TCPPorts != 0)
         {
            delete [] m_TCPPorts ;
            delete [] m_TCPPorts_Use ;
            m_TCPPorts = 0 ;
            m_TCPPorts_Use = 0 ;
         }

         m_TCPPorts_Num = 0 ;
         m_TCPPorts_Length = 0 ;
         if (ports != 0)
         {
            for (; ports[m_TCPPorts_Num] != 0; m_TCPPorts_Num++)
            {
            }
            m_TCPPorts_Length = m_TCPPorts_Num + 1 ;
            m_TCPPorts = new unsigned short[m_TCPPorts_Length] ;
            m_TCPPorts_Use = new unsigned short[m_TCPPorts_Length] ;
            for (int each = m_TCPPorts_Num ; each >= 0; each--)
            {
               m_TCPPorts[each] = ports[each] ;
               m_TCPPorts_Use[each] = 1 ;
            }
         }
         if (m_started)
         {
            // Restart the capture with the new ports
            m_worker -> restart () ;
         }
      }

      unsigned short * NetworkSniffer::TCPPorts () throw()
      {
         return m_TCPPorts ;
      }

      unsigned long NetworkSniffer::TCPPorts_Num () throw()
      {
         return m_TCPPorts_Num  ;
      }

       NetworkSniffer::Status NetworkSniffer::start () throw()
      {
         if ((m_worker == 0) || (m_started))
         {
            return INVALID ;
         }

         CDMW_MUTEX_GUARD(m_mutex) ;

         // Wake up the worker
         m_started = true ;
         m_worker ->m_control.broadcast() ;

         // Wait for th worker to start working
         while ( m_stopped )
         {
            m_worker ->m_control.wait() ;
         }
         return OK ;
      }

      NetworkSniffer::Status NetworkSniffer::stop () throw()
      {
         if ((m_worker == 0) || (m_stopped))
         {
            return INVALID ;
         }

         CDMW_MUTEX_GUARD(m_mutex) ;

         m_started = false ;
         // Wake up the worker
         if (m_worker ->m_pcap_handle != 0)
         {
            pcap_breakloop(m_worker ->m_pcap_handle) ;
         }
         m_worker ->m_control.broadcast() ;
         while (! m_stopped )
         {
            OsSupport::Condition::WaitReturn result = m_worker ->m_control.wait(50) ; 
            if (result == OsSupport::Condition::TIMEDOUT)
            {
               pthread_kill (m_worker->m_id, SIGUSR1);
               // JM & HS kill(Cdmw::OsSupport::OS::get_processId(), SIGUSR1);
               while (! m_stopped )
               {
                  m_worker ->m_control.wait() ;
               }
            }
         }
         return OK ;
      }

      NetworkSniffer::Status NetworkSniffer::terminate () throw()
      {
         if ((m_worker == 0) || (m_terminated))
         {
            return INVALID ;
         }

         {
            CDMW_MUTEX_GUARD(m_mutex) ;

            m_started = false ;
            m_terminated = true ;
            if (m_worker ->m_pcap_handle != 0)
            {
               pcap_breakloop(m_worker ->m_pcap_handle) ;
               OsSupport::Condition::WaitReturn result = m_worker ->m_control.wait(50) ; 
               if (result == OsSupport::Condition::TIMEDOUT)
               {
                  pthread_kill (m_worker->m_id, SIGUSR1);
               }
            }
            m_worker ->m_control.broadcast() ;
         }
         
         m_worker ->join() ;
         delete m_worker ;
         m_worker = 0 ;
         return OK ;
      }

      NetworkSniffer::Status NetworkSniffer::addPort (unsigned short port) throw()
      {
         if (port == 0)
         {
            return INVALID ;
         }

         // Check if the port is not already spied
         for (int each = m_TCPPorts_Num - 1 ; each >= 0; each--)
         {
            if (port == m_TCPPorts[each])
            {
               m_TCPPorts_Use[each] ++ ;
               return OK ;
            }
         }

         if (m_TCPPorts_Length == 0)
         {
            m_TCPPorts_Length = PORTS_INCREMENT ;
            m_TCPPorts = new unsigned short[m_TCPPorts_Length] ;
            m_TCPPorts_Use = new unsigned short[m_TCPPorts_Length] ;
         }

         if (m_TCPPorts_Length == m_TCPPorts_Num + 1)
         {
            m_TCPPorts_Length += PORTS_INCREMENT ;
            unsigned short * l_tmp = m_TCPPorts ;
            m_TCPPorts = new unsigned short[m_TCPPorts_Length] ;
            for (int each = m_TCPPorts_Num - 1 ; each >= 0; each--)
            {
               m_TCPPorts[each] = l_tmp[each] ;
            }
            l_tmp = m_TCPPorts_Use ;
            m_TCPPorts_Use = new unsigned short[m_TCPPorts_Length] ;
            for (int each = m_TCPPorts_Num - 1 ; each >= 0; each--)
            {
               m_TCPPorts_Use[each] = l_tmp[each] ;
            }
         }

         m_TCPPorts[m_TCPPorts_Num] = port ;
         m_TCPPorts_Use[m_TCPPorts_Num] = 1 ;
         m_TCPPorts_Num ++ ;
         m_TCPPorts[m_TCPPorts_Num] = 0 ;

         if (m_started)
         {
            // Restart the capture with the new port
           m_worker -> restart () ;
         }
         return OK ;
      }

      NetworkSniffer::Status NetworkSniffer::removePort (unsigned short port) throw()
      {
         // Try to find the port
         for (int l_port_idx = m_TCPPorts_Num - 1 ; l_port_idx >= 0; l_port_idx--)
         {
            if (m_TCPPorts[l_port_idx] == port)
            {
               m_TCPPorts_Use[l_port_idx] -- ;
               if (m_TCPPorts_Use[l_port_idx] == 0)
               {
                  // Found, shift left the ports
                  // JM 31/08/2007 
                  //for (int each = m_TCPPorts_Num - 1 ; each >= l_port_idx; each--)
                  for (int each = l_port_idx ; each < m_TCPPorts_Num - 1 ; each++ )
                  {
                     m_TCPPorts[each] = m_TCPPorts[each + 1] ;
                     m_TCPPorts_Use[each] = m_TCPPorts_Use[each + 1] ;
                  }

                  m_TCPPorts_Num -- ;

                  if (m_started)
                  {
                     // Restart the capture without the port
                    m_worker -> restart () ;
                  }
               }
               return OK ;
            }
         }

         // Not found
         return INVALID ;
      }

      NetworkSniffer::Status NetworkSniffer::run () throw()
      {
         if (m_worker != 0)
         {
           return INVALID;
         }
         m_worker = new Worker(*this) ;
         m_terminated = false ;
         NetworkSniffer::Status started =  m_worker->init() ;
         if (!worker_inited)
         {
            m_terminated = true ;
            delete m_worker ;
            m_worker = 0 ;
         }
         return started;
      }

   }

}
