/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_RECORDING_TEST_BUFFER_SENDER_HPP 
#define INCL_RECORDING_TEST_BUFFER_SENDER_HPP 

#include "Foundation/testutils/TestManager.hpp"
#include "TraceAndPerf/recordinglibrary/BufferSender.hpp"
#include "TraceAndPerf/recordinglibrary/CoalescenceManager.hpp"

namespace Cdmw
{
namespace Recording
{

/**
*Purpose:
*<p>   Test of Message Identifier class
*/
class TestBufferSender : public CppUnit::TestFixture
{
    public:

        TestBufferSender();

        virtual  ~TestBufferSender()  throw();


        /**
        *Purpose:
        *<p>   Tests setup
        */
        void setUp();
        
        /**
        *Purpose:
        *<p>   Finalize tests
        */
        void tearDown(); 

        
    protected:
        
    private:
 
        void start_rec_server_emulator(); 
    
        /**
        *Purpose:
        *<p>   Test BufferSender initialize and finalize methods
        */
        void test_initialize_finalize_001();
        void test_initialize_finalize_002();
        void test_initialize_finalize_003();
               
        /**
        *Purpose:
        *<p>   Test BufferSender recordingURL methods
        */
        void test_recordingURL_004();       
        void test_recordingURL_005();       
               
        /**
        *Purpose:
        *<p>   Test BufferSender.sendBuffer method
        */
        void test_sendBuffer_006();       
        void test_sendBuffer_007();       
        void test_sendBuffer_008();       
        void test_sendBuffer_009();       
        void test_sendBuffer_011();       
        void test_sendBuffer_012();       

        /**
        *Purpose:
        *<p>   Test delete BufferSender object
        */
        void test_delete_010();       


        void create_file(std::string filename, const char * data);
        
        Buffer * get_buffer();
        
        bool check_server_data(int buffers_count);
        
        BufferSender * buffer_sender;
        CoalescenceManager * c_manager;

    CPPUNIT_TEST_SUITE( TestBufferSender );
        CPPUNIT_TEST( test_initialize_finalize_001 );
        CPPUNIT_TEST( test_initialize_finalize_002 );
        CPPUNIT_TEST( test_initialize_finalize_003 );
        CPPUNIT_TEST( test_recordingURL_004 );
        CPPUNIT_TEST( test_recordingURL_005 );
        CPPUNIT_TEST( test_sendBuffer_006 );
        CPPUNIT_TEST( test_sendBuffer_007 );
        CPPUNIT_TEST( test_sendBuffer_012 );
        CPPUNIT_TEST( test_sendBuffer_008 );
        CPPUNIT_TEST( test_sendBuffer_009 );
        CPPUNIT_TEST( test_sendBuffer_011 );
        CPPUNIT_TEST( test_delete_010 );
    CPPUNIT_TEST_SUITE_END();


}; // End class TestBufferSender 

}; // End namespace Recording
}; // End namespace Cdmw

#endif // INCL_RECORDING_TEST_BUFFER_SENDER_HPP

