/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include "Foundation/orbsupport/CORBA.hpp"
#include "testbuffersender/MyPlugin.hpp"

namespace Cdmw
{
   namespace Recording
   {

      MyPlugin::MyPlugin ()
            throw() : Plugin(), m_manager (0), m_sender(0)
      {
      }

      MyPlugin::~MyPlugin ()
            throw()
      {
         delete m_manager ;
         delete m_sender ;
      }

      bool MyPlugin::Destination (const char * destination)
            throw()
      {
         m_sender->recordingURL (destination) ;
         return true ;
      }

      char * MyPlugin::Destination ()
            throw()
      {
         return m_sender->recordingURL () ;
      }

      void MyPlugin::recordOperation (Message * request, Message * reply)
            throw()
      {
         if(request != NULL)
         {
           const char * the_operation;
           the_operation = (const char *) (request -> data + request -> operation_Offset) ;
           std::cout << "recordOperation : operation = " << the_operation << std::endl;
         }
         m_manager->WriteOperation(request, reply) ;
      }

      void MyPlugin::initialize (CORBA::ORB_ptr orb)
            throw()
      {
         m_sender = new MyBufferSender() ;
         m_manager = new CoalescenceManager (*m_sender) ;
         m_sender->initialize(m_manager, orb) ;
         m_manager->initialize() ;
      }

      void MyPlugin::finalize ()
            throw()
      {
         m_manager->finalize () ;
         m_sender->finalize() ;
      }


   }
}
