/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <fstream>
#include <unistd.h>
#include <sys/types.h>
#include "testbuffersender/Recorder_impl.hpp"

#include "Foundation/orbsupport/OrbSupport.hpp"

#include "Foundation/testutils/CORBATestManager.hpp"

#include <semaphore.h>
#include <sys/stat.h>   //S_IRUSR|S_IWUSR
#include <fcntl.h>  //O_CREAT 

using namespace std;

int main(int argc, char * argv[])
{   
  std::ofstream om("recording_server.pid");
  if(om) 
  { 
    om <<  getpid() ;    
    om.close();
  } 
  try
  {
    // Initialize orb
    //CORBA::ORB_var orb = Cdmw::TestUtils::CORBATestManager::instance()->get_ORB();
    // Initialize orb                      
    Cdmw::OrbSupport::StrategyList orb_strategies;
    orb_strategies.add_OrbThreaded();
    orb_strategies.add_PoaThreadPool(5);
    CORBA::ORB_var orb = 
        Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, orb_strategies);      

    // Get reference to Root POA.
    CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
    PortableServer::POA_var root_poa = PortableServer::POA::_narrow(obj);

    // Activate POA manager
    PortableServer::POAManager_var mgr = root_poa->the_POAManager();
    mgr->activate();

    // Create an object
    POA_CdmwRecording::Recorder_impl recorder_servant;
    ::CdmwRecording::Recorder_var Recorder_ref =  ::CdmwRecording::Recorder::_nil();

    PortableServer::ObjectId_var oid = 
                                      root_poa->activate_object(&recorder_servant);

    CORBA::Object_var object = root_poa->id_to_reference(oid.in());

    Recorder_ref = ::CdmwRecording::Recorder::_narrow(object.in());
    const std::string corbaloc_name = "Recorder" ;
    Cdmw::OrbSupport::OrbSupport::bind_object_to_corbaloc 
      (orb.in(), corbaloc_name, Recorder_ref.in());        

    std::cout << "=== Recording Server : accepting requests ..." << std::endl;
    // Accept requests
    orb->run();
  }
  catch (const CORBA::Exception& ex)
  {
     std::cout << "*** Recording Server : Exception caught in ORBThread" 
       << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex) << std::endl;
    return -1;
  }

  return 0;
}

