/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
#include <iostream>
#include <fstream>
#include <string.h>
#include <stdio.h>

#include <Foundation/testutils/CORBATestManager.hpp>
#include <Foundation/idllib/CdmwRecordingContainerManager.stub.hpp>
#include "testbuffersender/TestBufferSender.hpp"
#include <Foundation/ossupport/OS.hpp>

namespace Cdmw
{
namespace Recording
{

CPPUNIT_TEST_SUITE_REGISTRATION(TestBufferSender);

Cdmw::OsSupport::OS::ProcessId idRecordingServer = 0;

TestBufferSender::TestBufferSender()
    : buffer_sender(0)

{
}

TestBufferSender::~TestBufferSender()  throw()
{
}
 
void TestBufferSender::create_file(std::string filename, const char * data)
{
    FILE * pFile;
    pFile = fopen (filename.c_str(), "w");
    fprintf (pFile, "%s\n", data);
    fclose (pFile); 
} 

Buffer * TestBufferSender::get_buffer()
{
    Buffer * buffer = new Buffer();
    
    std::string line = "";
    
    ifstream inFile;
    inFile.open("buffer.txt");
        
    buffer->next = 0;
    getline(inFile, line);
    buffer->date.tv_sec = atoi(line.c_str());
    getline(inFile, line);
    buffer->date.tv_usec = atoi(line.c_str());
    
    getline(inFile, line);
    buffer->header.base_time = atoi(line.c_str());
    getline(inFile, line);
    buffer->header.hostname = line.c_str();
    getline(inFile, line);
    buffer->header.num_pairs = atoi(line.c_str());

    getline(inFile, line);
    buffer->size = atoi(line.c_str());

    octet * data = new octet[buffer->size];
    for (unsigned int i = 0; i < (buffer->size); i++)
    {
        getline(inFile, line);
        data[i] = atoi(line.c_str());
    }
    buffer->data = data;
    
    inFile.close();
    
    return buffer;
}

bool TestBufferSender::check_server_data(int buffers_count)
{
    int counter = 0;
    std::string line = "";
        
    ifstream inFile;
    inFile.open("recordingserver_data");
    int nb_retry_max = 10;
    int nb_retry = 0;
    while(!inFile.is_open() && (nb_retry <= nb_retry_max))
    {
      std::cout<<"Opening recordingserver_data ... retry ..."<<std::endl;
      sleep (1);
      inFile.open("recordingserver_data");
      nb_retry ++;
    }
    if (!inFile.is_open()) return false;
    inFile.close();
    std::cout<<"opened recordingserver_data, waiting 3 sec ... "<<std::endl;
    
    sleep(3); // wait for Recording Server to flash data
    
    ifstream istrm("recordingserver_data");
    
    while(istrm >> line) 
    {
        if(line.find("buffer",0)!= std::string::npos) counter++;       
    }
    istrm.close();
        
    std::remove("recordingserver_data");

    if (counter == buffers_count)
        return true;
    else
        return false;    
}

void TestBufferSender::setUp()
{
    //
    //Initialize
    //
    buffer_sender = new BufferSender();
    c_manager = new CoalescenceManager(*buffer_sender);
}

void TestBufferSender::tearDown()
{
    //
    //Finalize
    //
    delete(c_manager);
    if (buffer_sender)
      {
        delete(buffer_sender);
      }
}
void TestBufferSender::start_rec_server_emulator()
{
    // Start RecordingServer emulator 
    std::cout<< "Starting Recording Server  ..."<< std::endl;
    try 
    {
       idRecordingServer = Cdmw::OsSupport::OS::create_process
          ( "recording_server", "--CdmwProcessPort=12344");
    } 
       catch (const std::exception & ex)
    {
       std::cerr << "*** Exception : " << ex.what() << std::endl;
    }
    
    std::cout<<"waiting 5 sec.."<<std::endl;  
    sleep(5);         
}
 
void TestBufferSender::test_initialize_finalize_001()
{
    TEST_INFO("001: Check initialize and finalize class object");
    
    //Check
    CPPUNIT_ASSERT_NO_THROW(buffer_sender->initialize(c_manager, Cdmw::TestUtils::CORBATestManager::instance()->get_ORB()));
    CPPUNIT_ASSERT_NO_THROW(buffer_sender->finalize());
}

void TestBufferSender::test_initialize_finalize_002()
{
    TEST_INFO("002: Check initialize two times and finalize class object");
    
    //Check
    CPPUNIT_ASSERT_NO_THROW(buffer_sender->initialize(c_manager, Cdmw::TestUtils::CORBATestManager::instance()->get_ORB()));
    CPPUNIT_ASSERT_NO_THROW(buffer_sender->initialize(c_manager, Cdmw::TestUtils::CORBATestManager::instance()->get_ORB()));
    CPPUNIT_ASSERT_NO_THROW(buffer_sender->finalize());
}

void TestBufferSender::test_initialize_finalize_003()
{
    TEST_INFO("003: Check initialize and finalize two times class object");

    //Check
    CPPUNIT_ASSERT_NO_THROW(buffer_sender->initialize(c_manager, Cdmw::TestUtils::CORBATestManager::instance()->get_ORB()));
    CPPUNIT_ASSERT_NO_THROW(buffer_sender->finalize());
    CPPUNIT_ASSERT_NO_THROW(buffer_sender->finalize());
}

void TestBufferSender::test_recordingURL_004()
{
    TEST_INFO("004: Check recordingURL method with correct parameter");
    
    //Initialize
    buffer_sender->recordingURL("corbaloc::hostname:port/Server");
    
    //Check
    CPPUNIT_ASSERT(0 == strcmp("corbaloc::hostname:port/Server", buffer_sender->recordingURL()));
}

void TestBufferSender::test_recordingURL_005()
{
    TEST_INFO("005: Check recordingURL method with NULL parameter");

    //Initialize
    buffer_sender->recordingURL(0);
    
    //Check
    CPPUNIT_ASSERT(0 == buffer_sender->recordingURL());
}

void TestBufferSender::test_sendBuffer_006()
{
    TEST_INFO("006: Check sendBuffer method with NULL parameter");
    
    //Initialize
    buffer_sender->initialize(c_manager, Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());
    
    Buffer * buffer = 0;
    //Check
    CPPUNIT_ASSERT_NO_THROW(buffer_sender->sendBuffer(buffer));
    
    //Finalize
    buffer_sender->finalize();
}

void TestBufferSender::test_sendBuffer_007()
{
    TEST_INFO("007: Check sendBuffer method with correct parameter");
    
    //Initialize

    //get hostname
    char host_name[100] = "";
    gethostname(host_name, sizeof(host_name));
    
    //Start RecordingServer emulator 
    start_rec_server_emulator();
        
    //construct URL
    std::string str_url = std::string("corbaloc:iiop:")
       //+ std::string(host_name) + std::string(":6789/Recorder");
                        + std::string(host_name) + std::string(":12344/Recorder");
    std::cout << "URL: +" << str_url << "+"<< std::endl;
    
    buffer_sender->recordingURL((char *) str_url.c_str());
    std::cout << "Initializing BufferSender" << std::endl;
    c_manager->initialize();
    buffer_sender->initialize(c_manager, Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());

    Buffer * buffer;
    buffer = get_buffer();
    buffer_sender->sendBuffer(buffer);
    

    //Check
    //std::cout<<"recordOperation called, waiting 5 sec.."<<std::endl;
    //sleep(5);
    //std::cout<<"calling check_server_data"<<std::endl;
    CPPUNIT_ASSERT(true == check_server_data(1));

    std::cout<<"007 : Killing Recording Server ..."<<std::endl;  
    buffer = get_buffer();
    buffer->size = 0;
    buffer_sender->sendBuffer(buffer);
    //Finalize
    buffer_sender->finalize();

}

void TestBufferSender::test_sendBuffer_012()
{
    TEST_INFO("012: Sending the same buffer many times in queue");

    //Initialize
    buffer_sender->initialize(c_manager, Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());
    buffer_sender->recordingURL("");
    
    Buffer * buffer;
    buffer = get_buffer();

    for (int i = 0; i < 105; i++)
    {
        buffer_sender->sendBuffer(buffer);
    }  
 
    //Start RecordingServer emulator  (already started in test 7)
    start_rec_server_emulator(); 

    //construct URL 
    //get hostname
    char host_name[100] = "";
    gethostname(host_name, sizeof(host_name));
    
    std::string str_url = std::string("corbaloc:iiop:")
       + std::string(host_name) + std::string(":12344/Recorder");
    std::cout << "URL: +" << str_url << "+"<< std::endl;
    buffer_sender->recordingURL((char *) str_url.c_str());
    
    //Check
    CPPUNIT_ASSERT(true == check_server_data(1));  

    std::cout<<"012 : Killing Recording Server ..."<<std::endl;  
    buffer = get_buffer();
    buffer->size = 0;
    buffer_sender->sendBuffer(buffer);

    //Finalize
    buffer_sender->finalize();

}


void TestBufferSender::test_sendBuffer_008()
{
  TEST_INFO("008: Check deleting old buffers in queue");

  //Initialize
  buffer_sender->initialize(c_manager, Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());
  buffer_sender->recordingURL("");

  Buffer * buffer;

  for (int i = 0; i < 105; i++)
  {
    buffer = get_buffer();
    buffer_sender->sendBuffer(buffer);
  }  

   // Start RecordingServer emulator (already started in test 7)
   start_rec_server_emulator();

  //construct URL
  //get hostname
  char host_name[100] = "";
  gethostname(host_name, sizeof(host_name));
  std::string str_url = std::string("corbaloc:iiop:")
//                        + std::string(host_name) + std::string(":6789/Recorder");
                        + std::string(host_name) + std::string(":12344/Recorder");
  std::cout << "URL: +" << str_url << "+"<< std::endl;
  buffer_sender->recordingURL((char *) str_url.c_str());

  //Check
  CPPUNIT_ASSERT(true == check_server_data(100));  

  std::cout<<"008 : Killing Recording Server ..."<<std::endl;  
  buffer = get_buffer();
  buffer->size = 0;
  buffer_sender->sendBuffer(buffer);
  //Finalize
  buffer_sender->finalize();

}

void TestBufferSender::test_sendBuffer_009()
{
    TEST_INFO("009: Check sendBuffer method after finalize");
    
    //Initialize
    buffer_sender->initialize(c_manager, Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());
    buffer_sender->recordingURL("");
    buffer_sender->finalize();
    
    Buffer * buffer;
    buffer = get_buffer();
    
    //Check
    CPPUNIT_ASSERT_NO_THROW(buffer_sender->sendBuffer(buffer));

    //Finalize
    buffer_sender->finalize();
}

void TestBufferSender::test_sendBuffer_011()
{
    TEST_INFO("011: Check perform sendBuffer when object initialized with NULL CoalescenceManager value");
    
    //Initialize
    Buffer * buffer;
    buffer = get_buffer();
    
    //Check
    CPPUNIT_ASSERT_NO_THROW(buffer_sender->initialize(0, 0));
    CPPUNIT_ASSERT_NO_THROW(buffer_sender->sendBuffer(buffer));

    //Finalize
    buffer_sender->finalize();
}

void TestBufferSender::test_delete_010()
{
    TEST_INFO("010: Check delete BufferSender without finalizing");
    
    //Initialize
    buffer_sender->initialize(c_manager, Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());
    CPPUNIT_ASSERT_NO_THROW(delete(buffer_sender));
    buffer_sender = 0;
}

}  // namespace Recording 
}  // namespace Cdmw




