/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include "Foundation/testutils/CORBATestManager.hpp"
#include "testbuffersender/MyBufferSender.hpp"
#include "testbuffersender/MyPlugin.hpp"
#include "TraceAndPerf/recordinglibrary/CoalescenceManager.hpp"
#include "TraceAndPerf/recordinglibrary/NetworkSniffer.hpp"
#include "TraceAndPerf/recordinglibrary/PacketStore.hpp"
#include "TraceAndPerf/recordinglibrary/OperationMap.hpp"
#include "TraceAndPerf/recordinglibrary/ObjectMap.hpp"
#include "TraceAndPerf/recordinglibrary/MessageIdentifier.hpp"
#include "TraceAndPerf/recordinglibrary/InterfaceFilter.hpp"
#include "TraceAndPerf/recordinglibrary/RequestPairing.hpp"


int main(int argc, char** argv )
{
        
        //Create Sniffer
        std::cout << "Create NetworkSniffer" <<std::endl;
        Cdmw::Recording::PacketStore store;  
        Cdmw::Recording::NetworkSniffer sniffer(store);
        
        //Create InterfaceFilter
        std::cout << "Create InterfaceFilter" <<std::endl;
        Cdmw::Recording::InterfaceFilter interface_filter("interfaces.txt");
        
        //Create OperationMap
        std::cout << "Create OperationMap" <<std::endl;
        Cdmw::Recording::OperationMap operation_map("operations.txt");
        
        //Create ObjectMap
        std::cout << "Create ObjectMap" <<std::endl;
        Cdmw::Recording::ObjectMap object_map(interface_filter, sniffer);
        
        //Create RequestPairing
        std::cout << "Create RequestPairing" <<std::endl;
        Cdmw::Recording::RequestPairing pairing(store);
        
        //Init sniffer
        std::cout << "Init NetworkSniffer" <<std::endl;
        sniffer.addPort(6789);
        
        std::cout << "run" <<std::endl;
        sniffer.run () ; 
       
        std::cout << "start" <<std::endl;
        sniffer.start () ;

        //Create plugin and initialize it
        Cdmw::Recording::MyPlugin * my_plugin = new Cdmw::Recording::MyPlugin();
        
        // Initialize the ORB
        CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);
        my_plugin->initialize(orb); 
        std::cout << "Plugin initialized" << endl;      
        
        //Add plugin to RequestPairing object ans set destination
        pairing.addPlugin(my_plugin);
        pairing.setDestination(MY_PLUGIN_TYPE, "");

        //Create MessageIdentifier
        Cdmw::Recording::MessageIdentifier message_identifier(store, object_map, interface_filter, operation_map, pairing);

        //Init MessageI dentifier
        Cdmw::Recording::MessageIdentifier::Status status = message_identifier.run();
           
        //Start MessageIdentifier
        status = message_identifier.start();
        std::cout << "catchBuffer: Message processing started" << endl;      

        
    
        sleep(30);
}
