/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_RECORDING_TEST_COALSCENCE_MANAGER_HPP 
#define INCL_RECORDING_TEST_COALSCENCE_MANAGER_HPP 

#include "Foundation/testutils/TestManager.hpp"
#include "TraceAndPerf/recordinglibrary/CoalescenceManager.hpp"
#include "testcoalescencemanager/MyBufferSender.hpp"

namespace Cdmw
{
namespace Recording
{

/**
*Purpose:
*<p>   Test of Interface Filter class
*/
class TestCoalescenceManager : public CppUnit::TestFixture
{
    public:

        TestCoalescenceManager();

        virtual  ~TestCoalescenceManager()  throw();
        
        /**
        *Purpose:
        *<p>   Tests setup
        */
        void setUp();
        
        /**
        *Purpose:
        *<p>   Finalize tests
        */
        void tearDown();         

    protected:
    private:
        /**
        *Purpose:
        *<p>   Test initialize() and finalize() methods of CoalscenceManager object
        */
        void test_initialize_finalize_001();
        void test_initialize_finalize_002();
        void test_initialize_finalize_003();
        
        /**
        *Purpose:
        *<p>   Test MaxSize property of CoalscenceManager object
        */
        void test_MaxSize_004();
        void test_MaxSize_005();
        void test_MaxSize_006();
        void test_MaxSize_014();
        
        /**
        *Purpose:
        *<p>   Test TimeOut property of CoalscenceManager object
        */
        void test_TimeOut_007();
        void test_TimeOut_008();
        void test_TimeOut_009();
        void test_TimeOut_015();
        
        /**
        *Purpose:
        *<p>   Test WroteOperation method of CoalscenceManager object
        */
        void test_WriteOperation_010();
        void test_WriteOperation_011();
        
        /**
        *Purpose:
        *<p>   Test flush() method of CoalscenceManager object
        */
        void test_flush_012();
        void test_flush_013();
        
        /**
        *Purpose:
        *<p>   Test freeBuffer() method of CoalscenceManager object
        */
        void test_freeBuffer_016();
        
        void test_WriteOperation_NULL();
        void test_flush_empty();
        
        
        Message * get_message(const char * file);
            
        MyBufferSender * my_buffer_sender;
        CoalescenceManager * c_manager;    
            
    CPPUNIT_TEST_SUITE( TestCoalescenceManager );
        CPPUNIT_TEST( test_initialize_finalize_001 );
        CPPUNIT_TEST( test_initialize_finalize_002 );
        CPPUNIT_TEST( test_initialize_finalize_003 );
        CPPUNIT_TEST( test_MaxSize_004 );
        CPPUNIT_TEST( test_MaxSize_005 );
        CPPUNIT_TEST( test_MaxSize_006 );
        CPPUNIT_TEST( test_TimeOut_007 );
        CPPUNIT_TEST( test_TimeOut_008 );
        CPPUNIT_TEST( test_TimeOut_009 );
        CPPUNIT_TEST( test_WriteOperation_010 );
        CPPUNIT_TEST( test_WriteOperation_011 );
        CPPUNIT_TEST( test_flush_012 );
        CPPUNIT_TEST( test_flush_013 );
        CPPUNIT_TEST( test_MaxSize_014 );
        CPPUNIT_TEST( test_TimeOut_015 );
        CPPUNIT_TEST( test_freeBuffer_016 );
        CPPUNIT_TEST( test_flush_empty );
        CPPUNIT_TEST( test_WriteOperation_NULL );
    CPPUNIT_TEST_SUITE_END();


}; // End class TestCoalscenceManager 

}; // End namespace Recording
}; // End namespace Cdmw

#endif // INCL_RECORDING_TEST_COALSCENCE_MANAGER_HPP

