/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
#include <iostream>
#include <fstream>
#include <stdio.h>

#include "testcoalescencemanager/TestCoalescenceManager.hpp"

namespace Cdmw
{
namespace Recording
{

CPPUNIT_TEST_SUITE_REGISTRATION(TestCoalescenceManager);

TestCoalescenceManager::TestCoalescenceManager()
{
}

TestCoalescenceManager::~TestCoalescenceManager()  throw()
{
}

void TestCoalescenceManager::setUp()
{
    my_buffer_sender = new MyBufferSender();
    c_manager = new CoalescenceManager(*my_buffer_sender);     
}

void TestCoalescenceManager::tearDown()
{
    delete(c_manager);
    delete(my_buffer_sender);
}

Message * TestCoalescenceManager::get_message(const char * file)
{
    Message * msg = new Message();
    int req;
    
    std::ifstream istrm(file, std::ifstream::in );
    
    istrm >> req;                
    istrm >> msg->channel.src_addr;              
    istrm >> msg->channel.dst_addr;        
    istrm >> msg->channel.src_port;
    istrm >> msg->channel.dst_port;
    istrm >> msg->header.magic[0] ; 
    istrm >> msg->header.magic[1] ;
    istrm >> msg->header.magic[2] ;
    istrm >> msg->header.magic[3] ;                 
    short sh;
    istrm >> sh ; msg->header.version.major = (octet) sh ;
    istrm >> sh ; msg->header.version.minor= (octet) sh ;
    istrm >> sh ; msg->header.flags = (octet) sh;
   

    istrm >> sh ; msg->header.message_type = (octet) sh;       
    istrm >> msg->header.length;       
    istrm >> msg->date.tv_sec ; 
    istrm >> msg->date.tv_usec ;   
    istrm >> msg->need_swap ;
        
    msg->data = new octet[ msg->header.length + 12 ] ;    
    for(int i = 0;  i < msg->header.length + 12; i++)
    { 
        istrm >> sh ; msg->data[i] = (octet) sh ;          
    }  
                  
    istrm >> msg->request_id  ;
   
    istrm >> msg->objectKey_Offset ;
    istrm >> msg->objectKey_Length ; 
    istrm >> msg->operation_Offset ;
          
    std::string serv="", intf="";
         
    msg->server_id = 0;
    msg->intf_rid = 0;
              
    if(req  == 1)
    {      
        while (istrm && !istrm.eof() && serv == "" ) getline(istrm, serv);  
        msg->server_id = new char[serv.length()+1];
        strncpy((char *) msg->server_id, serv.c_str() , serv.length() );
           
        while (istrm && !istrm.eof() && intf == "" ) getline(istrm, intf);     
        msg->intf_rid = new char[intf.length()+1];    
        strncpy((char *) msg->intf_rid, intf.c_str() , intf.length() );
    }   
   
    msg->next = 0;
    msg->fragments = 0;
   
    istrm.close();
    
    return msg;
}
 
void TestCoalescenceManager::test_initialize_finalize_001()
{
    TEST_INFO("001: Check initialize and finalize class object");

    //Check
    CPPUNIT_ASSERT_NO_THROW(c_manager->initialize());
    CPPUNIT_ASSERT_NO_THROW(c_manager->finalize());    
}

void TestCoalescenceManager::test_initialize_finalize_002()
{
    TEST_INFO("002: Check initialize two times and finalize methods");

    //Check
    CPPUNIT_ASSERT_NO_THROW(c_manager->initialize());
    CPPUNIT_ASSERT_NO_THROW(c_manager->initialize());
    CPPUNIT_ASSERT_NO_THROW(c_manager->finalize());    
}

void TestCoalescenceManager::test_initialize_finalize_003()
{
    TEST_INFO("003: Check initialize and finalize two times methods");

    //Check
    CPPUNIT_ASSERT_NO_THROW(c_manager->initialize());
    CPPUNIT_ASSERT_NO_THROW(c_manager->finalize());    
    CPPUNIT_ASSERT_NO_THROW(c_manager->finalize());    
}

void TestCoalescenceManager::test_MaxSize_004()
{
    TEST_INFO("004: Check default MaxSize value");

    //Check
    CPPUNIT_ASSERT(8192 == c_manager->MaxSize());
}

void TestCoalescenceManager::test_MaxSize_005()
{
    TEST_INFO("005: Check set zero MaxSize value");

    //Initialize
    c_manager->MaxSize(0);

    //Check
    CPPUNIT_ASSERT(0 == c_manager->MaxSize());
}

void TestCoalescenceManager::test_MaxSize_006()
{
    TEST_INFO("006: Check set large MaxSize value");

    //Initialize
    c_manager->MaxSize(123456789);

    //Check
    CPPUNIT_ASSERT(123456789 == c_manager->MaxSize());
}

void TestCoalescenceManager::test_TimeOut_007()
{
    TEST_INFO("007: Check default TimeOut value");

    //Check
    CPPUNIT_ASSERT(100 == c_manager->TimeOut());
}

void TestCoalescenceManager::test_TimeOut_008()
{
    TEST_INFO("008: Check set zero TimeOut value");

    //Inititalize
    c_manager->TimeOut(0);    

    //Check
    CPPUNIT_ASSERT(0 == c_manager->TimeOut());
}

void TestCoalescenceManager::test_TimeOut_009()
{
    TEST_INFO("009: Check set large TimeOut value");

    //Inititalize
    c_manager->TimeOut(123456789);    

    //Check
    CPPUNIT_ASSERT(123456789 == c_manager->TimeOut());
}


void TestCoalescenceManager::test_WriteOperation_010()
{
    TEST_INFO("010: Check WriteOperation and flush methods");

    //Inititalize
    Message * request = 0;
    request = get_message("request.txt");
    Message * reply = 0;
    reply = get_message("reply.txt");
    c_manager->initialize();
    c_manager->WriteOperation(request, reply);    
    c_manager->flush();    

    //Check
    CPPUNIT_ASSERT(1 == my_buffer_sender->get_sendBuffer_count());
    c_manager->finalize();
}

void TestCoalescenceManager::test_WriteOperation_011()
{
    TEST_INFO("011: Check call WriteOperation method with request and reply parameters that exceed MaxSize property of object");

    //Inititalize
    Message * request = 0;
    request = get_message("request.txt");
    Message * reply = 0;
    reply = get_message("reply.txt");
    c_manager->initialize();
    c_manager->MaxSize(10);
    c_manager->WriteOperation(request, reply);    
    sleep(2);
    
    //Check 
    CPPUNIT_ASSERT(1 == my_buffer_sender->get_sendBuffer_count());
    c_manager->finalize();
}

void TestCoalescenceManager::test_flush_012()
{
    TEST_INFO("012: Check flush by timeout");

    //Inititalize
    Message * request = 0;
    request = get_message("request.txt");
    Message * reply = 0;
    reply = get_message("reply.txt");
    c_manager->initialize();
    c_manager->TimeOut(200);
    c_manager->WriteOperation(request, reply);
    sleep(3);    

    //Check
    CPPUNIT_ASSERT(1 == my_buffer_sender->get_sendBuffer_count());
    c_manager->finalize();
}

void TestCoalescenceManager::test_flush_013()
{
    TEST_INFO("013: Check flush by MaxSize");

    //Inititalize
    Message * request = 0;
    request = get_message("request.txt");
    Message * reply = 0;
    reply = get_message("reply.txt");
    c_manager->initialize();
    c_manager->MaxSize(73);
    c_manager->TimeOut(1000000);
    c_manager->WriteOperation(request, reply);
    //c_manager->WriteOperation(request, reply);

    //Check
    CPPUNIT_ASSERT(1 == my_buffer_sender->get_sendBuffer_count());
    c_manager->finalize();
}

void TestCoalescenceManager::test_MaxSize_014()
{
    TEST_INFO("014: Check changing MaxSize at runtime ");

    //Inititalize
    Message * request = 0;
    request = get_message("request.txt");
    Message * reply = 0;
    reply = get_message("reply.txt");

    c_manager->initialize();
    c_manager->MaxSize(1000);
    c_manager->WriteOperation(request, reply);
    c_manager->TimeOut(1000000);
    // !!! JM reload request and reply because 
    // WriteOperation frees them 
    request = get_message("request.txt");
    reply = get_message("reply.txt");
    c_manager->WriteOperation(request, reply);

    //Check
    CPPUNIT_ASSERT(0 == my_buffer_sender->get_sendBuffer_count());
    c_manager->finalize();
}

void TestCoalescenceManager::test_TimeOut_015()
{
    TEST_INFO("015: Check changing TimeOut at runtime");

    //Inititalize
    Message * request = 0;
    request = get_message("request.txt");
    Message * reply = 0;
    reply = get_message("reply.txt");

    c_manager->initialize();
    c_manager->MaxSize(2000);
    c_manager->TimeOut(30000);
    c_manager->WriteOperation(request, reply);
    // !!! JM reload request and reply because 
    // WriteOperation frees them 
    request = get_message("request.txt");
    reply = get_message("reply.txt");
    c_manager->WriteOperation(request, reply);

    //Check
    CPPUNIT_ASSERT(0 == my_buffer_sender->get_sendBuffer_count());
    sleep(45);
    CPPUNIT_ASSERT(1 == my_buffer_sender->get_sendBuffer_count());
    c_manager->finalize();
}

void TestCoalescenceManager::test_freeBuffer_016()
{
    TEST_INFO("016: Check freeBuffer method");

    //Inititalize
    Message * request = 0;
    request = get_message("request.txt");
    Message * reply = 0;
    reply = get_message("reply.txt");

    c_manager->initialize();
    c_manager->WriteOperation(request, reply);
    c_manager->flush();    

    c_manager->free_buffer(my_buffer_sender->current_buffer);

    //Check
    CPPUNIT_ASSERT(0 == my_buffer_sender->current_buffer);
    c_manager->finalize();
}

void TestCoalescenceManager::test_WriteOperation_NULL()
{
    TEST_INFO("000: test WriteOperation with NULL parameters");

    c_manager->initialize();
    //Check
    CPPUNIT_ASSERT_NO_THROW(c_manager->WriteOperation(0, 0));
    c_manager->finalize();
}

void TestCoalescenceManager::test_flush_empty()
{
    TEST_INFO("000: test calling flush without WriteOperation");

    c_manager->initialize();
    //Check
    CPPUNIT_ASSERT_NO_THROW(c_manager->flush());
    c_manager->finalize();
}

}  // namespace Recording 
}  // namespace Cdmw





