/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
#include <iostream>
#include <fstream>
#include <string>
#include <stdio.h>

#include "testdebugplugin/TestDebugPlugin.hpp"

namespace Cdmw
{
namespace Recording
{

CPPUNIT_TEST_SUITE_REGISTRATION(TestDebugPlugin);

TestDebugPlugin::TestDebugPlugin()
{
}
TestDebugPlugin::~TestDebugPlugin()  throw()
{
}
//===============================================================
//  load 1 message from file
//=============================================================== 
Message * TestDebugPlugin::load_message(std::string filename)
{ 
    int req ;
    
    Message * msg = new Message();  
    std::ifstream istrm(filename.c_str() , std::ifstream::in );
    
    if(!istrm) 
    { 
        std::cout<< "#### could not open input file "<<filename<<std::endl;
        delete msg;
        return 0; 
    }
    try
    {
        istrm >> req ; // 1 request 2 reply                
        istrm >> msg->channel.src_addr;              
        istrm >> msg->channel.dst_addr;        
        istrm >> msg->channel.src_port;
        istrm >> msg->channel.dst_port;
        istrm >> msg->header.magic[0] ; 
        istrm >> msg->header.magic[1] ;
        istrm >> msg->header.magic[2] ;
        istrm >> msg->header.magic[3] ;                 
        short sh;
        istrm >> sh ; msg->header.version.major = (octet) sh ;
        istrm >> sh ; msg->header.version.minor= (octet) sh ;
        istrm >> sh ; msg->header.flags = (octet) sh;
           
        istrm >> sh ; msg->header.message_type = (octet) sh;       
        istrm >> msg->header.length;       
        istrm >> msg->date.tv_sec ; 
        istrm >> msg->date.tv_usec ;   
        istrm >> msg->need_swap ;
                
        msg->data = new octet[ msg->header.length + 12 ] ;    
        for(int i = 0;  i < msg->header.length + 12; i++)
        { 
          istrm >> sh ; msg->data[i] = (octet) sh ;          
        }  
                         
        istrm >> msg->request_id  ;      
        istrm >> msg->objectKey_Offset ;
        istrm >> msg->objectKey_Length ; 
        istrm >> msg->operation_Offset ;
                  
        std::string serv="", intf="";
                      
        if(req  == 1)
        {      
           while (istrm && !istrm.eof() && serv == "" ) getline(istrm, serv);  
           msg->server_id = new char[serv.length()+1];
           strncpy((char *) msg->server_id, serv.c_str() , serv.length() );  
                  
           while (istrm && !istrm.eof() && intf == "" ) getline(istrm, intf);     
           msg->intf_rid = new char[intf.length()+1];    
           strncpy((char *) msg->intf_rid, intf.c_str() , intf.length() );
        }   
    }
    catch(...)
    {
       std::cout<< "#### error on reading file "<< filename << std::endl;
       delete msg;
       return 0;   
    }
      
    msg->next = 0;
    msg->fragments = 0; 
    
    istrm.close();
    
    return msg ; 
}

void TestDebugPlugin::setUp()
{
    int argc = 0;  
    orb = CORBA::ORB_init(argc, (char**)0 );     
        
    // create empty debug_log.txt 
    FILE * pFile;
    pFile = fopen ("debug_log5.txt", "w");
    fclose (pFile);       
    remove("debug_log5.txt"); // required for test 5
    pFile = fopen ("debug_log8.txt", "w");
    fclose (pFile);       
    remove("debug_log8.txt"); // required for test 8
    pFile = fopen ("debug_log11.txt", "w");
    fclose (pFile);       
    remove("debug_log11.txt"); // required for test 11
    pFile = fopen ("debug_log13.txt", "w");
    fclose (pFile);       
    
    destination5  = "debug_log5.txt" ;
    destination8  = "debug_log8.txt" ;
    destination11 = "debug_log11.txt" ;
    destination13 = "debug_log13.txt" ;
        
    request = load_message("request1.txt");
    reply   = load_message("reply1.txt"); 
}

void TestDebugPlugin::tearDown()
{
   remove("debug_log5.txt"); 
   remove("debug_log8.txt");
   remove("debug_log11.txt");
   remove("debug_log13.txt");
   
   delete request;
   delete reply;
}

bool TestDebugPlugin::check_output_empty(std::string file)
{
    std::ifstream istrm(file.c_str(), std::ifstream::in);
    std::string s;
    
    if(istrm >> s)
    {
        istrm.close();       
        if(s == "") return true;
        else return false;
    }
    return true; 
}
bool TestDebugPlugin::check_output_not_empty(std::string file)
{
    std::ifstream istrm(file.c_str(), std::ifstream::in);
    std::string s; 
    
    while( istrm >> s)
    {
      if(s != "") { istrm.close(); return true ;}       
    }
    istrm.close();
    return false ;
}
void TestDebugPlugin::save(std::string file)
{
    std::string s; 
     
    std::ifstream istrm(file.c_str(), std::ifstream::in);
    
    copy = "";
    
    while(istrm >> s) 
        copy += s;
  
    istrm.close();
}
bool TestDebugPlugin::check_output_changed(std::string file)
{   
    std::string second; 
    std::string s;
     
    std::ifstream istrm(file.c_str(), std::ifstream::in);
      
    while(istrm >> s)
        second += s;
     
    istrm.close();
    
    return (second.length() > copy.length()) ;
}
//============================================================
// check initialize and finalize
//============================================================
void TestDebugPlugin::test_initialize_1()
{ 
    DebugPlugin plug;
    
    CPPUNIT_ASSERT_NO_THROW(plug.initialize(orb));
    
    CPPUNIT_ASSERT_NO_THROW(plug.finalize());
    
}
//============================================================
// check finalize without initialize
//============================================================
void TestDebugPlugin::test_finalize_2()
{ 
    DebugPlugin plug;
    
    CPPUNIT_ASSERT_NO_THROW(plug.finalize());
}
//============================================================
// check finalize after finalize
//============================================================
void TestDebugPlugin::test_finalize_3()
{ 
    DebugPlugin plug;
    
    CPPUNIT_ASSERT_NO_THROW(plug.initialize(orb));
    
    CPPUNIT_ASSERT_NO_THROW(plug.finalize());
    CPPUNIT_ASSERT_NO_THROW(plug.finalize());
}
//============================================================
// check Destination() without setting it
//============================================================
void TestDebugPlugin::test_destination_4()
{  
    DebugPlugin plug;
    
    CPPUNIT_ASSERT(0 == plug.Destination());   
}
//============================================================
// check set/get destination
//============================================================
void TestDebugPlugin::test_destination_5()
{
    DebugPlugin plug;
       
    CPPUNIT_ASSERT(true == plug.Destination((char *) destination5.c_str()));
    
    std::cout<< "set destination "<< destination5 << std::endl;
    std::cout<< "get destination "<< std::string(plug.Destination()) << std::endl;
    
    CPPUNIT_ASSERT(destination5 == std::string(plug.Destination()));
}    
//============================================================
// check set/get destination with 0
//============================================================
void TestDebugPlugin::test_destination_6()
{
    DebugPlugin plug;
       
    CPPUNIT_ASSERT(false == plug.Destination(0));
    CPPUNIT_ASSERT(0 == plug.Destination());    
}
//============================================================
// check type()
//============================================================
void TestDebugPlugin::test_type_7()
{   
    DebugPlugin plug;
    
    CPPUNIT_ASSERT(2 == plug.type());
}
//============================================================
// check recordOperation with valid parameters
//============================================================
void TestDebugPlugin::test_record_8()
{   
    DebugPlugin plug;
    
    plug.initialize(orb);
        
    CPPUNIT_ASSERT(true == plug.Destination((char *)destination8.c_str()));
    
    CPPUNIT_ASSERT_NO_THROW(plug.recordOperation(request, reply));

    // check destination file (not empty) 
    CPPUNIT_ASSERT(check_output_not_empty(destination8));
   
    plug.finalize();

    // save output file as input for decoder test
    //std::string file =  "tmp/dp_output";
    //rename("debug_log8.txt", file.c_str());
}
//============================================================
// check recordOperation without setting destination
//============================================================
void TestDebugPlugin::test_record_9()
{        
    DebugPlugin plug;
   
    plug.initialize(orb);
   
    CPPUNIT_ASSERT_NO_THROW(plug.recordOperation(request, reply));
  
    plug.finalize();
    
}
//============================================================
// check recordOperation with invalid destination
//============================================================
void TestDebugPlugin::test_record_10()
{
    DebugPlugin plug;
    
    plug.initialize(orb);
    
    CPPUNIT_ASSERT(false == plug.Destination("dir/not_exists.txt"));
    
    CPPUNIT_ASSERT_NO_THROW(plug.recordOperation(request, reply));
    
    plug.finalize();        
}
//============================================================
// check recordOperation to non-empty file
//============================================================
void TestDebugPlugin::test_record_11()
{       
    DebugPlugin plug;
    
    plug.initialize(orb);
         
    CPPUNIT_ASSERT(true == plug.Destination((char *) destination11.c_str()));
    
    CPPUNIT_ASSERT_NO_THROW(plug.recordOperation(request, reply));
   
    CPPUNIT_ASSERT(check_output_not_empty(destination11));
    save(destination11);
    
    CPPUNIT_ASSERT_NO_THROW(plug.recordOperation(request, reply));
    
    // check destination file changed
    CPPUNIT_ASSERT(check_output_changed(destination11));
    
    plug.finalize();
    
}
//============================================================
// check recordOperation with destination 0
//============================================================
void TestDebugPlugin::test_record_12()
{     
    DebugPlugin plug;
    
    plug.initialize(orb);
       
    CPPUNIT_ASSERT(false == plug.Destination(0));
    
    CPPUNIT_ASSERT_NO_THROW(plug.recordOperation(request, reply));
    
    plug.finalize();
}
//============================================================
// check recordOperation with 0 as parameter
//============================================================
void TestDebugPlugin::test_record_13()
{      
    DebugPlugin plug;
    
    plug.initialize(orb);
              
    CPPUNIT_ASSERT(true == plug.Destination((char *)destination13.c_str()));
       
    CPPUNIT_ASSERT_NO_THROW(plug.recordOperation(0, 0));

    CPPUNIT_ASSERT(check_output_empty(destination13));
    
    plug.finalize();
}

}  // namespace Recording 
}  // namespace Cdmw





