/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_RECORDING_TEST_INTF_FILTER_HPP 
#define INCL_RECORDING_TEST_INTF_FILTER_HPP 

#include "Foundation/testutils/TestManager.hpp"
#include "TraceAndPerf/recordinglibrary/InterfaceFilter.hpp"

namespace Cdmw
{
namespace Recording
{

/**
*Purpose:
*<p>   Test of Interface Filter class
*/
class TestInterfaceFilter : public CppUnit::TestFixture
{
    public:

        TestInterfaceFilter();

        virtual  ~TestInterfaceFilter()  throw();

        virtual void   do_tests();

        /**
        *Purpose:
        *<p>   Save children data to vector
        */
        static void save (std::string const & str1, std::string const & str2, size_t i);

        /**
        *Purpose:
        *<p>   Save collection with falgs to vector
        */
        static void save_flag (std::string const & key_str, InterfaceFilter::Flag_t const & flag);

        static std::vector<std::pair<std::string, std::string> > children_collection;
        
        static std::vector<std::pair<std::string, InterfaceFilter::Flag_t> > interface_collection;

    protected:
    private:
        /**
        *Purpose:
        *<p>   Test InterfaceFilter object creation
        */
        void test_objectCreation();
        void test_objectCreation_001();
        void test_objectCreation_002();
        void test_objectCreation_003();
        void test_objectCreation_004();
       
        /**
        *Purpose:
        *<p>   Test addInterfaces method of InterfaceFilter object
        */
        void test_addInterfaces();
        void test_addInterfaces_005();
        void test_addInterfaces_006();
        void test_addInterfaces_007();
        void test_addInterfaces_019();
        
        /**
        *Purpose:
        *<p>   Test removeInterfaces method of InterfaceFilter object
        */
        void test_removeInterfaces();
        void test_removeInterfaces_008();
        void test_removeInterfaces_009();
        void test_removeInterfaces_010();
        void test_removeInterfaces_011();
        
        
        /**
        *Purpose:
        *<p>   Test removeAllInterfaces method of InterfaceFilter object
        */
        void test_removeAllInterfaces();
        void test_removeAllInterfaces_012();
        void test_removeAllInterfaces_013();

        /**
        *Purpose:
        *<p>   Test checkInterface method of InterfaceFilter object
        */
        void test_checkInterface();
        void test_checkInterface_014();
        void test_checkInterface_015();
        void test_checkInterface_016();
        
        /**
        *Purpose:
        *<p>   Test mapLoop method of InterfaceFilter object
        */
        void test_mapLoop();
        void test_mapLoop_017();
        void test_mapLoop_018();

        /**
        *Purpose:
        *<p>   Create text file and fill in data
        */
        void create_file(std::string filename, const char * data);
      
        
        bool check_interface_collection_empty();

        bool check_children_collection_empty();
        
        bool check_interface_collection_content();

        bool check_children_collection_content();

        std::vector<std::pair<std::string, InterfaceFilter::Flag_t> > etalon_interface_collection;
        std::vector<std::pair<std::string, std::string> > etalon_children_collection;
            
    CPPUNIT_TEST_SUITE( TestInterfaceFilter );
        CPPUNIT_TEST( do_tests );
    CPPUNIT_TEST_SUITE_END();


}; // End class TestInterfaceFilter 

}; // End namespace Recording
}; // End namespace Cdmw

#endif // INCL_RECORDING_TEST_INTF_FILTER_HPP

