/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
#include <iostream>
#include <stdio.h>

#include "TraceAndPerf/recordingidl/CdmwRecordingDaemonControl.stub.hpp"
#include "testinterfacefilter/TestInterfaceFilter.hpp"

namespace Cdmw
{
namespace Recording
{

CPPUNIT_TEST_SUITE_REGISTRATION(TestInterfaceFilter);

TestInterfaceFilter::TestInterfaceFilter()
{
}

TestInterfaceFilter::~TestInterfaceFilter()  throw()
{
}

void TestInterfaceFilter::do_tests()
{
    test_objectCreation();
    test_addInterfaces();
    test_removeInterfaces();
    test_removeAllInterfaces();
    test_checkInterface();
    test_mapLoop();
}
 
void TestInterfaceFilter::create_file(std::string filename, const char * data)
{
    FILE * pFile;
    pFile = fopen (filename.c_str(), "w");
    fprintf (pFile, "%s\n", data);
    fclose (pFile); 
} 
 
void TestInterfaceFilter::test_objectCreation()
{
    test_objectCreation_001();
    test_objectCreation_002();
    test_objectCreation_003();
    test_objectCreation_004();
}

void TestInterfaceFilter::test_addInterfaces()
{
    test_addInterfaces_005();
    test_addInterfaces_006();
    test_addInterfaces_007();
    test_addInterfaces_019();
}

void TestInterfaceFilter::test_removeInterfaces()
{
    test_removeInterfaces_008();
    test_removeInterfaces_009();
    test_removeInterfaces_010();
    test_removeInterfaces_011();
}

void TestInterfaceFilter::test_removeAllInterfaces()
{
    test_removeAllInterfaces_012();
    test_removeAllInterfaces_013();
}

void TestInterfaceFilter::test_checkInterface()
{
    test_checkInterface_014();
    test_checkInterface_015();
    test_checkInterface_016();
}
 
void TestInterfaceFilter::test_mapLoop()
{
    test_mapLoop_017();
    test_mapLoop_018();
}

std::vector<std::pair<std::string, std::string> > TestInterfaceFilter::children_collection = std::vector<std::pair<std::string, std::string> >();

void TestInterfaceFilter::save (std::string const & str1, std::string const & str2, size_t i)
{
    children_collection.push_back(make_pair(str1, str2));
}

std::vector<std::pair<std::string, InterfaceFilter::Flag_t> > TestInterfaceFilter::interface_collection = std::vector<std::pair<std::string, InterfaceFilter::Flag_t> >();

void TestInterfaceFilter::save_flag (std::string const & key_str, InterfaceFilter::Flag_t const & flag)
{
    interface_collection.push_back(make_pair(key_str, flag));
}

bool TestInterfaceFilter::check_children_collection_empty()
{
    if (0 == children_collection.size())
        return true;
    else    
        return false;
}

bool TestInterfaceFilter::check_interface_collection_empty()
{
    if (0 == interface_collection.size())
        return true;
    else    
        return false;
}

bool TestInterfaceFilter::check_interface_collection_content()
{
    if (interface_collection.size() != etalon_interface_collection.size())
    {
        return false;
    }
        
    for (unsigned int i = 0; i < interface_collection.size(); i++)
    {
        if ((interface_collection[i].first != etalon_interface_collection[i].first) || 
            (interface_collection[i].second.ownflag != etalon_interface_collection[i].second.ownflag) ||
            (interface_collection[i].second.counter != etalon_interface_collection[i].second.counter))
        {
          return false;
        }
    }
    return true;
}

bool TestInterfaceFilter::check_children_collection_content()
{
    if (children_collection.size() != etalon_children_collection.size())
        return false;
        
    for (unsigned int i = 0; i < children_collection.size(); i++)
    {
        if ((children_collection[i].first != etalon_children_collection[i].first) || 
            (children_collection[i].second != etalon_children_collection[i].second))
            return false;
    }
    return true;
}
 
InterfaceFilter::Flag_t set_flag(bool ownflag, int counter)
{
    InterfaceFilter::Flag_t flag;
    flag.ownflag = ownflag;
    flag.counter = counter;
    return flag;
}
 
void TestInterfaceFilter::test_objectCreation_001()
{
    TEST_INFO("001: Create InterfaceFilter object with non-existent file as parameter");
    Cdmw::Recording::InterfaceFilter interface_filter("non-existent_interfaces.txt");       

    interface_collection.clear();
    interface_filter.mapLoop(save, save_flag);
    
    CPPUNIT_ASSERT(check_interface_collection_empty());
}

void TestInterfaceFilter::test_objectCreation_002()
{   
    TEST_INFO("002: Create InterfaceFilter object with empty file as parameter");
    create_file("interfaces.txt", "");
    Cdmw::Recording::InterfaceFilter interface_filter("interfaces.txt");
    
    interface_collection.clear();
    children_collection.clear();
    interface_filter.mapLoop(save, save_flag);
    
    CPPUNIT_ASSERT(check_interface_collection_empty());
    CPPUNIT_ASSERT(check_children_collection_empty());
}

void TestInterfaceFilter::test_objectCreation_003()
{   
    TEST_INFO("003: Create InterfaceFilter object with non-empty file as parameter");
    //create interface filter
    std::string content = "i0\ni1 FALSE\ni2 p21 TRUE\ni3 p31 FALSE\np31 p41\ni4 i3 FALSE";
    create_file("interfaces.txt", content.c_str());
    Cdmw::Recording::InterfaceFilter interface_filter("interfaces.txt");
    
    //save collection results to vector
    interface_collection.clear();
    children_collection.clear();
    interface_filter.mapLoop(save, save_flag);
    
    //compose etalon interface collection
    etalon_interface_collection.clear();
    etalon_interface_collection.push_back(make_pair("i0", set_flag(true, 1)));
    etalon_interface_collection.push_back(make_pair("i1", set_flag(false, 0)));
    etalon_interface_collection.push_back(make_pair("i2", set_flag(true, 1)));
    etalon_interface_collection.push_back(make_pair("i3", set_flag(false, 1)));
    etalon_interface_collection.push_back(make_pair("i4", set_flag(false, 1)));
    etalon_interface_collection.push_back(make_pair("p31", set_flag(true, 1)));
    
    //compose etalon children collection
    etalon_children_collection.clear();
    etalon_children_collection.push_back(make_pair("i3", "i4"));
    etalon_children_collection.push_back(make_pair("p21", "i2"));
    etalon_children_collection.push_back(make_pair("p31", "i3"));
    etalon_children_collection.push_back(make_pair("p41", "p31"));

    //compare collections
    CPPUNIT_ASSERT(check_interface_collection_content());
    CPPUNIT_ASSERT(check_children_collection_content());
}

void TestInterfaceFilter::test_objectCreation_004()
{   
    TEST_INFO("004: Create InterfaceFilter object with complex file as parameter");
    //create interface filter
    std::string content = "i0 p01\n i1 p11 p12 FALSE\n i2 p21 p22 FALSE\n i3 p31 p32 p33 TRUE\n p21 p33 TRUE\n p31 TRUE\n p32 FALSE\n p33";
    create_file("interfaces.txt", content.c_str());
    Cdmw::Recording::InterfaceFilter interface_filter("interfaces.txt");
    
    //save collection results to vector
    interface_collection.clear();
    children_collection.clear();
    interface_filter.mapLoop(save, save_flag);
    
    //compose etalon interface collection
    etalon_interface_collection.clear();
    etalon_interface_collection.push_back(make_pair("i0", set_flag(true, 1)));
    etalon_interface_collection.push_back(make_pair("i1", set_flag(false, 0)));
    etalon_interface_collection.push_back(make_pair("i2", set_flag(false, 2)));
    etalon_interface_collection.push_back(make_pair("i3", set_flag(true, 3)));
    etalon_interface_collection.push_back(make_pair("p21", set_flag(true, 2)));
    etalon_interface_collection.push_back(make_pair("p31", set_flag(true, 1)));
    etalon_interface_collection.push_back(make_pair("p32", set_flag(false, 0)));
    etalon_interface_collection.push_back(make_pair("p33", set_flag(true, 1)));
    
    //compose etalon children collection
    etalon_children_collection.clear();
    etalon_children_collection.push_back(make_pair("p01", "i0"));
    etalon_children_collection.push_back(make_pair("p11", "i1"));
    etalon_children_collection.push_back(make_pair("p12", "i1"));
    etalon_children_collection.push_back(make_pair("p21", "i2"));
    etalon_children_collection.push_back(make_pair("p22", "i2"));
    etalon_children_collection.push_back(make_pair("p31", "i3"));
    etalon_children_collection.push_back(make_pair("p32", "i3"));
    etalon_children_collection.push_back(make_pair("p33", "i3"));
    etalon_children_collection.push_back(make_pair("p33", "p21"));

    //compare collections
    CPPUNIT_ASSERT(check_interface_collection_content());
    CPPUNIT_ASSERT(check_children_collection_content());
}

void TestInterfaceFilter::test_addInterfaces_005()
{   
    TEST_INFO("005: Check adding several interfaces to InterfaceFilter object with true value of expand parameter");
    //create interface filter
    create_file("interfaces.txt", "");
    Cdmw::Recording::InterfaceFilter interface_filter("interfaces.txt");
    
    //compose list of interfaces
    CdmwRecording::String_List intf_list;
    intf_list.length(3);
    intf_list[0] = "i0";
    intf_list[1] = "i1 p11";
    intf_list[2] = "i2 p21 p22 FALSE";

    //Add interfaces
    interface_filter.addInterfaces(intf_list);
    
    //save collection results to vector
    interface_collection.clear();
    children_collection.clear();
    interface_filter.mapLoop(save, save_flag);
    
    //compose etalon interface collection
    etalon_interface_collection.clear();
    etalon_interface_collection.push_back(make_pair("i0", set_flag(true, 1)));
    etalon_interface_collection.push_back(make_pair("i1", set_flag(true, 1)));
    etalon_interface_collection.push_back(make_pair("i2", set_flag(false, 0)));
    
    //compose etalon children collection
    etalon_children_collection.clear();
    etalon_children_collection.push_back(make_pair("p11", "i1"));
    etalon_children_collection.push_back(make_pair("p21", "i2"));
    etalon_children_collection.push_back(make_pair("p22", "i2"));

    //compare collections
    CPPUNIT_ASSERT(check_interface_collection_content());
    CPPUNIT_ASSERT(check_children_collection_content());
}

void TestInterfaceFilter::test_addInterfaces_006()
{
    TEST_INFO("006: Check adding several interfaces to InterfaceFilter object with false value of expand parameter");
    //create interface filter
    create_file("interfaces.txt", "");
    Cdmw::Recording::InterfaceFilter interface_filter("interfaces.txt");
    
    //compose list of interfaces
    CdmwRecording::String_List intf_list;
    intf_list.length(3);
    intf_list[0] = "i0 i1 FALSE";
    intf_list[1] = "i1 p11 p12 FALSE";
    intf_list[2] = "p11 TRUE";

    //Add interfaces
    interface_filter.addInterfaces(intf_list);
    
    //save collection results to vector
    interface_collection.clear();
    children_collection.clear();
    interface_filter.mapLoop(save, save_flag);
    
    //compose etalon interface collection
    etalon_interface_collection.clear();
    etalon_interface_collection.push_back(make_pair("i0", set_flag(false, 1)));
    etalon_interface_collection.push_back(make_pair("i1", set_flag(false, 1)));
    etalon_interface_collection.push_back(make_pair("p11", set_flag(true, 1)));
    
    //compose etalon children collection
    etalon_children_collection.clear();
    etalon_children_collection.push_back(make_pair("i1", "i0"));
    etalon_children_collection.push_back(make_pair("p11", "i1"));
    etalon_children_collection.push_back(make_pair("p12", "i1"));

    //compare collections
    CPPUNIT_ASSERT(check_interface_collection_content());
    CPPUNIT_ASSERT(check_children_collection_content());
}

void TestInterfaceFilter::test_addInterfaces_007()
{
    TEST_INFO("007: Check addInterfaces method with empty list");   
    //create interface filter
    create_file("interfaces.txt", "");
    Cdmw::Recording::InterfaceFilter interface_filter("interfaces.txt");
    
    //compose list of interfaces
    CdmwRecording::String_List intf_list;
    intf_list.length(0);

    //Add interfaces
    CPPUNIT_ASSERT_NO_THROW(interface_filter.addInterfaces(intf_list));
    
    //save collection results to vector
    interface_collection.clear();
    children_collection.clear();
    interface_filter.mapLoop(save, save_flag);

    //check collections
    CPPUNIT_ASSERT(check_interface_collection_empty());       
    CPPUNIT_ASSERT(check_children_collection_empty());       
}

void TestInterfaceFilter::test_addInterfaces_019()
{   
    TEST_INFO("019: Check adding two identical interfaces");
    //create interface filter
    create_file("interfaces.txt", "int1");
    Cdmw::Recording::InterfaceFilter interface_filter("interfaces.txt");
    
    //compose list of interfaces
    CdmwRecording::String_List intf_list;
    intf_list.length(1);
    intf_list[0] = "int1";

    //Add interfaces
    CPPUNIT_ASSERT_NO_THROW(interface_filter.addInterfaces(intf_list));
    
    //save collection results to vector
    interface_collection.clear();
    children_collection.clear();
    interface_filter.mapLoop(save, save_flag);

    //compose etalon interface collection
    etalon_interface_collection.clear();
    etalon_interface_collection.push_back(make_pair("int1", set_flag(true, 1)));
    
    //compare collections
    CPPUNIT_ASSERT(check_interface_collection_content());
    CPPUNIT_ASSERT(check_children_collection_empty());
}

void TestInterfaceFilter::test_removeInterfaces_008()
{   
    TEST_INFO("008: Check removing several existent interfaces from InterfaceFilter object");
    //create interface filter
    std::string content = "int1\n int2 p21 p22\n int3 p31";
    create_file("interfaces.txt", content.c_str());
    Cdmw::Recording::InterfaceFilter interface_filter("interfaces.txt");
    
    //compose list of interfaces
    CdmwRecording::String_List intf_list;
    intf_list.length(2);
    intf_list[0] = "int1";
    intf_list[1] = "int2";

    //Remove interfaces
    CPPUNIT_ASSERT_NO_THROW(interface_filter.removeInterfaces(intf_list));
    
    //save collection results to vector
    interface_collection.clear();
    children_collection.clear();
    interface_filter.mapLoop(save, save_flag);
    
    //compose etalon interface collection
    etalon_interface_collection.clear();
    etalon_interface_collection.push_back(make_pair("int1", set_flag(false, 0)));
    etalon_interface_collection.push_back(make_pair("int2", set_flag(false, 0)));
    etalon_interface_collection.push_back(make_pair("int3", set_flag(true, 1)));
    
    //compare collections
    CPPUNIT_ASSERT(check_interface_collection_content());
}

void TestInterfaceFilter::test_removeInterfaces_009()
{
    TEST_INFO("009: Check removeInterfaces method with empty list of interfaces");
    //create interface filter
    std::string content = "int1\n int2 p21 p22\n int3 p31";
    create_file("interfaces.txt", content.c_str());
    Cdmw::Recording::InterfaceFilter interface_filter("interfaces.txt");
    
    //compose list of interfaces
    CdmwRecording::String_List intf_list;
    intf_list.length(0);

    //Remove interfaces
    CPPUNIT_ASSERT_NO_THROW(interface_filter.removeInterfaces(intf_list));
    
     //save collection results to vector
    interface_collection.clear();
    children_collection.clear();
    interface_filter.mapLoop(save, save_flag);
    
    //compose etalon interface collection
    etalon_interface_collection.clear();
    etalon_interface_collection.push_back(make_pair("int1", set_flag(true, 1)));
    etalon_interface_collection.push_back(make_pair("int2", set_flag(true, 1)));
    etalon_interface_collection.push_back(make_pair("int3", set_flag(true, 1)));
    
    //compare collections
    CPPUNIT_ASSERT(check_interface_collection_content());
}

void TestInterfaceFilter::test_removeInterfaces_010()
{  
    TEST_INFO("010: Check remove non-existent interfaces");
    //create interface filter
    std::string content = "int1\n int2 p21 p22\n int3 p31";
    create_file("interfaces.txt", content.c_str());
    Cdmw::Recording::InterfaceFilter interface_filter("interfaces.txt");
    
    //compose list of interfaces
    CdmwRecording::String_List intf_list;
    intf_list.length(2);
    intf_list[0] = "int4";
    intf_list[1] = "int5";

    //Remove interfaces
    CPPUNIT_ASSERT_EQUAL((int)interface_filter.removeInterfaces(intf_list), 0);

     //save collection results to vector
    interface_collection.clear();
    children_collection.clear();
    interface_filter.mapLoop(save, save_flag);
    
    //compose etalon interface collection
    etalon_interface_collection.clear();
    etalon_interface_collection.push_back(make_pair("int1", set_flag(true, 1)));
    etalon_interface_collection.push_back(make_pair("int2", set_flag(true, 1)));
    etalon_interface_collection.push_back(make_pair("int3", set_flag(true, 1)));
    
    //compare collections
    CPPUNIT_ASSERT(check_interface_collection_content());
}

void TestInterfaceFilter::test_removeInterfaces_011()
{  
    TEST_INFO("011: Check remove interfaces from empty repository");
    //create interface filter
    create_file("interfaces.txt", "");
    Cdmw::Recording::InterfaceFilter interface_filter("interfaces.txt");
    
    //compose list of interfaces
    CdmwRecording::String_List intf_list;
    intf_list.length(2);
    intf_list[0] = "int1";
    intf_list[1] = "int2";

    //Remove interfaces
    CPPUNIT_ASSERT_EQUAL((int)interface_filter.removeInterfaces(intf_list), 0);
}

void TestInterfaceFilter::test_removeAllInterfaces_012()
{  
    TEST_INFO("012: Check removeAllInterfaces method");
    //create interface filter
    std::string content = "int1\n int2 p21 p22\n int3 p31";
    create_file("interfaces.txt", content.c_str());
    Cdmw::Recording::InterfaceFilter interface_filter("interfaces.txt");
    
    //Remove all interfaces
    CPPUNIT_ASSERT_NO_THROW(interface_filter.removeAllInterfaces());
    
     //save collection results to vector
    interface_collection.clear();
    children_collection.clear();
    interface_filter.mapLoop(save, save_flag);
    
    //compose etalon interface collection
    etalon_interface_collection.clear();
    etalon_interface_collection.push_back(make_pair("int1", set_flag(false, 0)));
    etalon_interface_collection.push_back(make_pair("int2", set_flag(false, 0)));
    etalon_interface_collection.push_back(make_pair("int3", set_flag(false, 0)));
    
    //compare collections
    CPPUNIT_ASSERT(check_interface_collection_content());
}

void TestInterfaceFilter::test_removeAllInterfaces_013()
{  
    TEST_INFO("013: Check removeAllInterfaces method for empty object");
    //create interface filter
    create_file("interfaces.txt", "");
    Cdmw::Recording::InterfaceFilter interface_filter("interfaces.txt");
    
    //Remove all interfaces
    CPPUNIT_ASSERT_NO_THROW(interface_filter.removeAllInterfaces());
    
     //save collection results to vector
    interface_collection.clear();
    children_collection.clear();
    interface_filter.mapLoop(save, save_flag);
    
    //compare collections
    CPPUNIT_ASSERT(check_interface_collection_empty());
}

void TestInterfaceFilter::test_checkInterface_014()
{
    TEST_INFO("014: Check checkInterface method when list of interfaces is empty");
    //create interface filter
    create_file("interfaces.txt", "");
    Cdmw::Recording::InterfaceFilter interface_filter("interfaces.txt");
    
    //Remove all interfaces
    CPPUNIT_ASSERT(interface_filter.checkInterface("int1"));
}

void TestInterfaceFilter::test_checkInterface_015()
{
    TEST_INFO("015: Check checkInterface method for existent interface");
    //create interface filter
    std::string content = "int1 p1 p2";
    create_file("interfaces.txt", content.c_str());
    Cdmw::Recording::InterfaceFilter interface_filter("interfaces.txt");
    
    //Check existing interface
    CPPUNIT_ASSERT(interface_filter.checkInterface("int1"));
}

void TestInterfaceFilter::test_checkInterface_016()
{
    TEST_INFO("016: Check checkInterface method for non-existent interface");
    //create interface filter
    std::string content = "int1 p1 p2";
    create_file("interfaces.txt", content.c_str());
    Cdmw::Recording::InterfaceFilter interface_filter("interfaces.txt");
    
    //Remove all interfaces
    CPPUNIT_ASSERT(!interface_filter.checkInterface("int2"));
}

void TestInterfaceFilter::test_mapLoop_017()
{
    TEST_INFO("017: Check mapLoop method for non-empty object");
    //create interface filter
    std::string content = "int1\n int2 p21\n int3 p31 p32 p33 p34 p35 p36 p37 p38 p39";
    create_file("interfaces.txt", content.c_str());
    Cdmw::Recording::InterfaceFilter interface_filter("interfaces.txt");
      
     //save collection results to vector
    interface_collection.clear();
    children_collection.clear();
    interface_filter.mapLoop(save, save_flag);
    
    //compose etalon interface collection
    etalon_interface_collection.clear();
    etalon_interface_collection.push_back(make_pair("int1", set_flag(true, 1)));
    etalon_interface_collection.push_back(make_pair("int2", set_flag(true, 1)));
    etalon_interface_collection.push_back(make_pair("int3", set_flag(true, 1)));
    
    //compose etalon children collection
    etalon_children_collection.clear();
    etalon_children_collection.push_back(make_pair("p21", "int2"));
    etalon_children_collection.push_back(make_pair("p31", "int3"));
    etalon_children_collection.push_back(make_pair("p32", "int3"));
    etalon_children_collection.push_back(make_pair("p33", "int3"));
    etalon_children_collection.push_back(make_pair("p34", "int3"));
    etalon_children_collection.push_back(make_pair("p35", "int3"));
    etalon_children_collection.push_back(make_pair("p36", "int3"));
    etalon_children_collection.push_back(make_pair("p37", "int3"));
    etalon_children_collection.push_back(make_pair("p38", "int3"));
    etalon_children_collection.push_back(make_pair("p39", "int3"));

    //compare collections
    CPPUNIT_ASSERT(check_interface_collection_content());
    CPPUNIT_ASSERT(check_children_collection_content());
}

void TestInterfaceFilter::test_mapLoop_018()
{
    TEST_INFO("018: Check mapLoop method for empty object");
    //create interface filter
    create_file("interfaces.txt", "");
    Cdmw::Recording::InterfaceFilter interface_filter("interfaces.txt");
    
     //save collection results to vector
    interface_collection.clear();
    children_collection.clear();
    interface_filter.mapLoop(save, save_flag);
    
    //check collections
    CPPUNIT_ASSERT(check_interface_collection_empty());     
    CPPUNIT_ASSERT(check_children_collection_empty());     
}

}  // namespace Recording 
}  // namespace Cdmw





