/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_RECORDING_TEST_MESSAGE_IDENTIFIER_HPP 
#define INCL_RECORDING_TEST_MESSAGE_IDENTIFIER_HPP 

#include "Foundation/testutils/CORBATestManager.hpp"

#include "TraceAndPerf/recordinglibrary/MessageIdentifier.hpp"
#include "testmessageidentifier/MyRequestPairing.hpp"
#include "testmessageidentifier/MyOperationMap.hpp"
#include "testmessageidentifier/MyObjectMap.hpp"

namespace Cdmw
{
namespace Recording
{

/**
*Purpose:
*<p>   Test of Message Identifier class
*/
class TestMessageIdentifier : public CppUnit::TestFixture
{
    public:

        TestMessageIdentifier();

        virtual  ~TestMessageIdentifier()  throw();


        /**
        *Purpose:
        *<p>   Tests setup
        */
        void setUp();
        
        /**
        *Purpose:
        *<p>   Finalize tests
        */
        void tearDown(); 

        /**
        *Purpose:
        *<p>   Test MessageIdentifier object stopping
        */
        void test_objectStopping();
        
        /**
        *Purpose:
        *<p>   Test MessageIdentifier object creation
        */
        void test_objectCreation_001();
        
        /**
        *Purpose:
        *<p>   Test MessageIdentifier.start() method
        */
        void test_start_002();
        void test_start_007();
        void test_start_008();
        
        /**
        *Purpose:
        *<p>   Test MessageIdentifier.stop() method
        */
        void test_stop_003();
        void test_stop_006();
        void test_stop_009();
        
        /**
        *Purpose:
        *<p>   Test MessageIdentifier.run() method
        */
        void test_run_004();
        void test_run_005();
        
        /**
        *Purpose:
        *<p>   Test MessageIdentifier calls methods of other classes
        */
        void test_methods_010();
        void test_methods_011();
        void test_methods_012();
    protected:
        
    private:
       
        void create_file(std::string filename, const char * data);
        
        void push_packets(const char * file);
            
        InterfaceFilter * interface_filter;
        PacketStore store;
        NetworkSniffer * network_sniffer;
        ObjectMap * object_map;
        OperationMap * operation_map;
        RequestPairing * request_pairing;
        MessageIdentifier * message_identifier;
        
        MyObjectMap * my_object_map;
        MyOperationMap * my_operation_map;
        MyRequestPairing * my_request_pairing;  
        InterfaceFilter * my_interface_filter;
        
    CPPUNIT_TEST_SUITE( TestMessageIdentifier );
        CPPUNIT_TEST( test_objectCreation_001 );
        CPPUNIT_TEST( test_start_002 );
        CPPUNIT_TEST( test_start_007 );
        CPPUNIT_TEST( test_start_008 );
        CPPUNIT_TEST( test_stop_003 );
        CPPUNIT_TEST( test_stop_006 );
        CPPUNIT_TEST( test_stop_009 );
        CPPUNIT_TEST( test_run_004 );
        CPPUNIT_TEST( test_run_005 );
        CPPUNIT_TEST( test_methods_010 );
        CPPUNIT_TEST( test_methods_011 );
        CPPUNIT_TEST( test_methods_012 );
        //CPPUNIT_TEST( test_objectStopping );
    CPPUNIT_TEST_SUITE_END();


}; // End class TestMessageIdentifier 

}; // End namespace Recording
}; // End namespace Cdmw

#endif // INCL_RECORDING_TEST_MESSAGE_IDENTIFIER_HPP

