/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
#include <iostream>
#include <fstream>
#include <string.h>
#include <stdio.h>

#include "testmessageidentifier/TestMessageIdentifier.hpp"

#include "TraceAndPerf/recordinglibrary/PacketStore.hpp"
#include "TraceAndPerf/recordinglibrary/NetworkSniffer.hpp"
#include "TraceAndPerf/recordinglibrary/OperationMap.hpp"
#include "TraceAndPerf/recordinglibrary/ObjectMap.hpp"
#include "TraceAndPerf/recordinglibrary/RequestPairing.hpp"


namespace Cdmw
{
namespace Recording
{

CPPUNIT_TEST_SUITE_REGISTRATION(TestMessageIdentifier);

TestMessageIdentifier::TestMessageIdentifier()
    : interface_filter(0)
    , network_sniffer(0)
    , object_map(0)
    , operation_map(0)
    , request_pairing(0)
    , message_identifier(0)
{
}

TestMessageIdentifier::~TestMessageIdentifier()  throw()
{
}
 
void TestMessageIdentifier::create_file(std::string filename, const char * data)
{
    FILE * pFile;
    pFile = fopen (filename.c_str(), "w");
    fprintf (pFile, "%s\n", data);
    fclose (pFile); 
} 
 
void TestMessageIdentifier::push_packets(const char * file)
{
    Packet packet;
    
    std::string line = "";

    ifstream inFile;
    inFile.open(file);
    while(!inFile.eof())
    {
        getline(inFile, line);
        if (line == "")
        {
            continue;
        }
        packet.channel.src_addr = atoi(line.c_str());
        getline(inFile, line);
        packet.channel.dst_addr = atoi(line.c_str());
        getline(inFile, line);
        packet.channel.src_port = atoi(line.c_str());
        getline(inFile, line);
        packet.channel.dst_port = atoi(line.c_str());
        getline(inFile, line);
        packet.direction = (atoi(line.c_str()) == 0 ? Incoming : Outgoing);
        getline(inFile, line);
        packet.date.tv_sec = atoi(line.c_str());
        getline(inFile, line);
        packet.date.tv_usec = atoi(line.c_str());
        getline(inFile, line);
        packet.length = atoi(line.c_str());

        octet * data = new octet[packet.length];
        for (int i = 0; i < packet.length; i++)
        {
            getline(inFile, line);
            data[i] = atoi(line.c_str());
        }
        packet.data = data;
        
        store.pushPacket(packet);
    }
    inFile.close();
}

void TestMessageIdentifier::setUp()
{
    //
    //Initialize
    //
    network_sniffer = new NetworkSniffer (store);

    create_file("interfaces.txt", "int");
    interface_filter = new InterfaceFilter("interfaces.txt");
        
    create_file("operations.txt", "operation");
    operation_map = new OperationMap("operations.txt");

    object_map = new ObjectMap(*interface_filter, *network_sniffer);

    request_pairing = new RequestPairing(store);
}

void TestMessageIdentifier::tearDown()
{
    //
    //Finalize
    //
    delete(message_identifier);

    delete(object_map);
    delete(operation_map);
    delete(network_sniffer);
    delete(interface_filter);
    delete(request_pairing);
}
 
void TestMessageIdentifier::test_objectStopping()
{      
        //Init sniffer
        std::cout << "Add port" << std::endl;
        Cdmw::Recording::NetworkSniffer::Status ns_status = network_sniffer->addPort(12345);
        CPPUNIT_ASSERT_MESSAGE("adding port error", 0 == ns_status);
        
        std::cout << "Run sniffer" << std::endl;
        ns_status = network_sniffer->run () ; 
        CPPUNIT_ASSERT_MESSAGE("running sniffer error", 0 == ns_status);
       
        std::cout << "Start sniffer" << std::endl;
        ns_status = network_sniffer->start () ;
        //CPPUNIT_ASSERT_MESSAGE("starting sniffer error", 0 != ns_status);

        //Create MessageIdentifier
        std::cout << "Create MessageIdentifier" << std::endl;
        message_identifier = new MessageIdentifier(store, *object_map, *interface_filter, *operation_map, *request_pairing);

        //Init MessageI dentifier
        std::cout << "Run MI" << std::endl;
        Cdmw::Recording::MessageIdentifier::Status status = message_identifier->run();
        CPPUNIT_ASSERT_MESSAGE("run message identifier error", 0 == status);
           
        //Start MessageIdentifier
        std::cout << "Start MI" << std::endl;
        status = message_identifier->start();
        CPPUNIT_ASSERT_MESSAGE("start message identifier error", 0 == status);
    
        sleep(1);
        
        //Stop message processing
        std::cout << "Stop MI" << std::endl;
        status = message_identifier->stop();
        CPPUNIT_ASSERT_MESSAGE("message_identifier.stop() failed", 0 == status);
        
        //Terminate message processing
        std::cout << "Terminate MI" << std::endl;
        status = message_identifier->terminate();
        CPPUNIT_ASSERT_MESSAGE("message_identifier.terminate() failed", 0 == status);
        std::cout << "MI terminated" << std::endl;
        
        //Stop sniffer
        //std::cout << "Stop sniffer" << std::endl;
        //ns_status = network_sniffer->stop();
        //CPPUNIT_ASSERT_MESSAGE("sniffer.stop() failed", 0 == ns_status);

        //Terminate sniffer
        //std::cout << "Terminate sniffer" << std::endl;
        //ns_status = network_sniffer->terminate();
        //CPPUNIT_ASSERT_MESSAGE("sniffer.terminate() failed", 0 == ns_status);
}


void TestMessageIdentifier::test_objectCreation_001()
{
    TEST_INFO("001: Create MessageIdentifier object");
    
    //Create MessageIdentifier
    CPPUNIT_ASSERT_NO_THROW(message_identifier = new MessageIdentifier(store, *object_map, *interface_filter, *operation_map, *request_pairing));
    
    TEST_INFO("Test finished");
}


void TestMessageIdentifier::test_start_002()
{
    TEST_INFO("002: Check method start() without run()");
    
    //Initialize
    CPPUNIT_ASSERT_NO_THROW(message_identifier = new MessageIdentifier(store, *object_map, *interface_filter, *operation_map, *request_pairing));

    //Check
    CPPUNIT_ASSERT(MessageIdentifier::INVALID == message_identifier->start());
}

void TestMessageIdentifier::test_stop_003()
{
    TEST_INFO("003: Check methods stop() and terminate() without run()");
    
    //Initialize
    CPPUNIT_ASSERT_NO_THROW(message_identifier = new MessageIdentifier(store, *object_map, *interface_filter, *operation_map, *request_pairing));

    //Check
    CPPUNIT_ASSERT(MessageIdentifier::INVALID == message_identifier->stop());    
    CPPUNIT_ASSERT(MessageIdentifier::INVALID == message_identifier->terminate());    
}

void TestMessageIdentifier::test_run_004()
{
    TEST_INFO("004: Check method run()");
    
    //Initialize
    CPPUNIT_ASSERT_NO_THROW(message_identifier = new MessageIdentifier(store, *object_map, *interface_filter, *operation_map, *request_pairing));

    //Check
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->run());

    //Finalize
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->terminate());
}


void TestMessageIdentifier::test_run_005()
{
    TEST_INFO("005: Check method run() with MessageIdentifier already running");
    
    //Initialize
    CPPUNIT_ASSERT_NO_THROW(message_identifier = new MessageIdentifier(store, *object_map, *interface_filter, *operation_map, *request_pairing));
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->run());    

    //Check
    CPPUNIT_ASSERT(MessageIdentifier::INVALID == message_identifier->run());    

    //Finalize
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->terminate());
}

void TestMessageIdentifier::test_stop_006()
{
    TEST_INFO("006: Check method stop() without start()");
    
    //Initialize
    CPPUNIT_ASSERT_NO_THROW(message_identifier = new MessageIdentifier(store, *object_map, *interface_filter, *operation_map, *request_pairing));
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->run());

    //Check
    CPPUNIT_ASSERT(MessageIdentifier::INVALID == message_identifier->stop());       

    //Finalize
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->terminate());
}

void TestMessageIdentifier::test_start_007()
{
    TEST_INFO("007: Check method start()");

    //Initialize
    CPPUNIT_ASSERT_NO_THROW(message_identifier = new MessageIdentifier(store, *object_map, *interface_filter, *operation_map, *request_pairing));
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->run());

    //Check
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->start());       
    //Finalize
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->stop());
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->terminate());
}

void TestMessageIdentifier::test_start_008()
{
    TEST_INFO("008: Check method start() with MessageIdentifier already started");

    //Initialize
    CPPUNIT_ASSERT_NO_THROW(message_identifier = new MessageIdentifier(store, *object_map, *interface_filter, *operation_map, *request_pairing));
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->run());
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->start());       

    //Check
    CPPUNIT_ASSERT(MessageIdentifier::INVALID == message_identifier->start());           

    //Finalize
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->stop());
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->terminate());
}

void TestMessageIdentifier::test_stop_009()
{
    TEST_INFO("009: Check methods stop() and terminate()");

    //Initialize
    CPPUNIT_ASSERT_NO_THROW(message_identifier = new MessageIdentifier(store, *object_map, *interface_filter, *operation_map, *request_pairing));
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->run());
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->start());       

    //Check
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->stop());
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->terminate());
}

void TestMessageIdentifier::test_methods_010()
{
    TEST_INFO("010: Check MessageIdentifier calls methods of RequestPairing, OperationMap and ObjectMap");

    //Initialize
    push_packets("request.txt");
    
    create_file("interfaces.txt", "");
    my_interface_filter = new InterfaceFilter("interfaces.txt");
    
    create_file("operations.txt", "");
    my_operation_map = new MyOperationMap("operations.txt");

    my_object_map = new MyObjectMap(*interface_filter, *network_sniffer);

    my_request_pairing = new MyRequestPairing(store);    
    
    CPPUNIT_ASSERT_NO_THROW(message_identifier = new MessageIdentifier(store, *my_object_map, *my_interface_filter, *my_operation_map, *my_request_pairing));
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->run());
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->start());       

    sleep(2);

    //Check
    CPPUNIT_ASSERT(1 == my_operation_map->get_checkOperation_count());
    CPPUNIT_ASSERT(1 == my_object_map->get_checkKey_count());
    CPPUNIT_ASSERT(1 == my_request_pairing->get_storeRequest_count());
    CPPUNIT_ASSERT(0 == my_request_pairing->get_checkReply_count());

    //Finalize
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->stop());
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->terminate());
    
    delete(my_object_map);
    delete(my_operation_map);
    delete(my_interface_filter);
    delete(my_request_pairing);    
}

void TestMessageIdentifier::test_methods_011()
{
    TEST_INFO("011: Check MessageIdentifier calls methods of RequestPairing, OperationMap and ObjectMap");
    //Initialize
    push_packets("reply.txt");
    
    create_file("interfaces.txt", "");
    my_interface_filter = new InterfaceFilter("interfaces.txt");
    
    create_file("operations.txt", "");
    my_operation_map = new MyOperationMap("operations.txt");

    my_object_map = new MyObjectMap(*interface_filter, *network_sniffer);

    my_request_pairing = new MyRequestPairing(store);    
    
    CPPUNIT_ASSERT_NO_THROW(message_identifier = new MessageIdentifier(store, *my_object_map, *my_interface_filter, *my_operation_map, *my_request_pairing));
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->run());
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->start());       

    sleep(2);

    //Check
    CPPUNIT_ASSERT(1 == my_request_pairing->get_checkReply_count());
    
    //Finalize
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->stop());
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->terminate());
    
    delete(my_object_map);
    delete(my_operation_map);
    delete(my_interface_filter);
    delete(my_request_pairing);
}

void TestMessageIdentifier::test_methods_012()
{
    TEST_INFO("012: Check MessageIdentifier doesnt call methods of RequestPairing()  after stop()");

    //Initialize
    create_file("interfaces.txt", "");
    my_interface_filter = new InterfaceFilter("interfaces.txt");
    
    create_file("operations.txt", "");
    my_operation_map = new MyOperationMap("operations.txt");

    my_object_map = new MyObjectMap(*interface_filter, *network_sniffer);

    my_request_pairing = new MyRequestPairing(store);    
    
    CPPUNIT_ASSERT_NO_THROW(message_identifier = new MessageIdentifier(store, *my_object_map, *my_interface_filter, *my_operation_map, *my_request_pairing));
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->run());
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->start());       
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->stop());

    //Push packets to PacketStore
    push_packets("request.txt");
    push_packets("reply.txt");
    
    sleep(2);

    //Check
    CPPUNIT_ASSERT(0 == my_request_pairing->get_storeRequest_count());
    CPPUNIT_ASSERT(0 == my_request_pairing->get_checkReply_count());
    
    //Finalize
    CPPUNIT_ASSERT(MessageIdentifier::OK == message_identifier->terminate());
    
    delete(my_object_map);
    delete(my_operation_map);
    delete(my_interface_filter);
    delete(my_request_pairing);    
}

}  // namespace Recording 
}  // namespace Cdmw





