/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_RECORDING_TEST_OBJECT_MAP_HPP 
#define INCL_RECORDING_TEST_OBJECT_MAP_HPP 

#include "Foundation/testutils/CORBATestManager.hpp"

#include "Foundation/orbsupport/CORBA.hpp"

#include "TraceAndPerf/recordinglibrary/PacketStore.hpp"
#include "TraceAndPerf/recordinglibrary/NetworkSniffer.hpp"
#include "TraceAndPerf/recordinglibrary/InterfaceFilter.hpp"
#include "TraceAndPerf/recordinglibrary/ObjectMap.hpp"
//#include <ofstream>

namespace Cdmw
{
namespace Recording
{

/**
*Purpose:
*<p>   Test of Interface Filter class
*/
class TestObjectMap : public CppUnit::TestFixture
{
    public:

        TestObjectMap();

        virtual  ~TestObjectMap()  throw();

        virtual void   do_tests();
        
        void setUp();
        void tearDown(); 
    protected:
    
    private:
                     
        void create_file(std::string filename, std::string data);
        Message * TestObjectMap::load_message(std::string filename);
            
        /**
        *Purpose:
        *<p>   Test checkKey method of ObjectMap
        */
        void test_checkKey_001();
        void test_checkKey_005();
        void test_checkKey_006();
        void test_checkKey_007();

        /**
        *Purpose:
        *<p>   Test addObject method of ObjectMap
        */
        void test_addObject_002();
        void test_addObject_003();
        void test_addObject_004();
        
        /**
        *Purpose:
        *<p>   Test checkInterfaces method of ObjectMap
        */
        void test_checkInterfaces();       
        void test_checkInterfaces_008();       
        void test_checkInterfaces_009();       

        /**
        *Purpose:
        *<p>   Test add/remove (object or port) methods of ObjectMap (PCR 1523)
        */
        void test_fix_PCR_1523();
            
        //CORBA specific vars    
        CORBA::ORB_var orb;
        CORBA::Object_var obj1, obj2;
        
        pid_t server_pid;
        
        InterfaceFilter * interface_filter;
        PacketStore store;
        NetworkSniffer * network_sniffer;
        ObjectMap * object_map;
        
        Message * msg1, * msg2, * msg3 ;
            
    CPPUNIT_TEST_SUITE( TestObjectMap );
        CPPUNIT_TEST( do_tests );
        CPPUNIT_TEST( test_fix_PCR_1523 );
    CPPUNIT_TEST_SUITE_END();


}; // End class TestObjectMap 

}; // End namespace Recording
}; // End namespace Cdmw

#endif // INCL_RECORDING_TEST_OBJECT_MAP_HPP
