/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
#include <iostream>
#include <fstream>
#include <istream>
#include <stdio.h>

#include "TraceAndPerf/recordinglibrary/InterfaceFilter.hpp"
#include "TraceAndPerf/recordinglibrary/ObjectMap.hpp"
#include "TraceAndPerf/recordinglibrary/PacketStore.hpp"
#include "TraceAndPerf/recordingidl/CdmwRecordingDaemonControl.stub.hpp"
#include "testobjectmap/TestObjectMap.hpp"
#include "testobjectmap/TestInterface.stub.hpp"

namespace Cdmw
{
namespace Recording
{

    CPPUNIT_TEST_SUITE_REGISTRATION(TestObjectMap);

   // function used to print interfaces map
    void print (std::string const & str1, std::string const & str2, size_t i)
    {
       std::cout << "Interface : " << str1 << " / child #" << i << " : " << str2 << std::endl;
    }

    void print_flag (std::string const & key_str, InterfaceFilter::Flag_t const & flag)
    {
       std::cout << "Interface : " << key_str << " / Flag.ownflag = " << flag.ownflag <<  " / Flag.counter = " <<flag.counter << std::endl;
    }    

   // function used to print object map
    void print_objectmap (std::string const & str1, int i, int s)
    {
       std::cout << "ObjectMap, repository_Id  : " << str1 << " "  << i << "/" << s << std::endl;
    }

    TestObjectMap::TestObjectMap()
    {
    }

    TestObjectMap::~TestObjectMap()  throw()
    {
    }

    void TestObjectMap::create_file(std::string filename, std::string data)
    {
        FILE * pFile;
        pFile = fopen (filename.c_str(),"w");
        fprintf (pFile, "%s\n", data.c_str());
        fclose (pFile);
    }

    void TestObjectMap::setUp()
    {

        try
        {
            // Initialize orb
            cout << "orb init" << endl;

            orb = Cdmw::TestUtils::CORBATestManager::instance()->get_ORB();

            cout << "get IOR 1" << endl;
            //
            //First object
            //

            //get IOR for the first object
            FILE * pFile;
            while (true)
            {
                pFile = fopen ("server1.ior","r");
                if (pFile != 0)
                    break;
            }

            std::ifstream istrm("server1.ior");
            std::string ior1 ;
            istrm >> ior1 ;
            istrm.close();


            // Destringify IOR
            obj1 = orb->string_to_object(ior1.c_str());
            if (CORBA::is_nil(obj1))
            {
                cout << "Nil object reference" << endl;
                throw 0;
            }

            // Narrow
            TestInterface1_var ti1 = TestInterface1::_narrow(obj1);
            if (CORBA::is_nil(ti1))
            {
                cout << "Argument is not a correct reference" << endl;
                throw 0;
            }

            //
            //Second object
            //

            cout << "get IOR 2" << endl;

            //get IOR for the second object
            while (true)
            {
                pFile = fopen ("server2.ior","r");
                if (pFile != 0)
                    break;
            }
            std::ifstream istrm2("server2.ior");
            std::string ior2 ;
            istrm2 >> ior2 ;
            istrm2.close();

            // Destringify IOR
            obj2 = orb->string_to_object(ior2.c_str());
            if (CORBA::is_nil(obj2))
            {
                cout << "Nil object reference" << endl;
                throw 0;
            }

            // Narrow
            TestInterface2_var ti2 = TestInterface2::_narrow(obj2);
            if (CORBA::is_nil(ti2))
            {
                cout << "Argument is not a correct reference" << endl;
                throw 0;
            }

        }
        catch (const CORBA::Exception &)
        {
            cout << "Uncaught CORBA exception" << endl;
            return ;
        }
        catch (...)
        {
           cout << "Uncaught exception" << endl;
           return ;
        }

        cout << "initialize" << endl;
        //
        //Initialize
        //
        create_file("interfaces.txt", "IDL:TestInterface1:1.0");
        interface_filter = new InterfaceFilter("interfaces.txt");
        network_sniffer = new NetworkSniffer (store);
        object_map = new ObjectMap(*interface_filter, *network_sniffer);

        msg1 = load_message("message1.txt");
        msg2 = load_message("message2.txt");
        msg3 = load_message("request1.txt");
    }

    void TestObjectMap::test_fix_PCR_1523()
    {
        unsigned long init_port_num = network_sniffer->TCPPorts_Num ();
        unsigned long current_port_num = init_port_num;

        cout << "test_fix_PCR_1523 :" << endl;
        cout << "init_port_num = " << init_port_num << endl;

        //CdmwRecording::String_List intf_list;
        //intf_list.length(1);
        //intf_list[0] = "IDL:TestInterface2:1.0 TRUE";
        //interface_filter->addInterfaces(intf_list);

        object_map->addObject("IDL:TestInterface1:1.0", orb, obj1);
        cout << "step 1, port_num = " << network_sniffer->TCPPorts_Num () << endl;
        current_port_num ++;
        CPPUNIT_ASSERT(current_port_num == network_sniffer->TCPPorts_Num ());

        unsigned short *ports =  network_sniffer->TCPPorts ();
        unsigned short port = ports[network_sniffer->TCPPorts_Num () -1];
        cout << "object port is " << port << endl;
        network_sniffer->addPort (port);
        cout << "step 2, port_num = " << network_sniffer->TCPPorts_Num () << endl;
        // current_port_num unchanged as port already registered
        CPPUNIT_ASSERT(current_port_num == network_sniffer->TCPPorts_Num ());

        object_map->removeObject(orb, obj1);
        cout << "step 3, port_num = " << network_sniffer->TCPPorts_Num () << endl;
        // current_port_num unchanged as port keeps registered
        CPPUNIT_ASSERT(current_port_num == network_sniffer->TCPPorts_Num ());

        object_map->removeObject(orb, obj1);
        cout << "step 4, port_num = " << network_sniffer->TCPPorts_Num () << endl;
        // current_port_num unchanged as port keeps registered
        CPPUNIT_ASSERT(current_port_num == network_sniffer->TCPPorts_Num ());

        network_sniffer->removePort (port);
        cout << "step 5, port_num = " << network_sniffer->TCPPorts_Num () << endl;
        current_port_num --;
        CPPUNIT_ASSERT(init_port_num == network_sniffer->TCPPorts_Num ());
    }

    void TestObjectMap::tearDown()
    {
        //
        //Kill CORBA server process
        //
        /*kill(server_pid, 9);
        int status;
        waitpid(server_pid, &status, 0);
        */
        //
        //Deinitialize
        //
        delete(object_map);
        delete(network_sniffer);
        delete(interface_filter);

        delete msg1;
        delete msg2;
        delete msg3;
    }

    void TestObjectMap::do_tests()
    {
        test_checkKey_001();

        test_addObject_002();
        test_addObject_003();
        test_addObject_004();

        test_checkKey_005();
        test_checkKey_006();
        test_checkKey_007();

        test_checkInterfaces();
    }

    void TestObjectMap::test_checkInterfaces()
    {
        test_checkInterfaces_008();
        test_checkInterfaces_009();
    }
    // =================================================
    // Verify checkKey method for empty list of objects
    // =================================================
    void TestObjectMap::test_checkKey_001()
    {
        msg3 = load_message("request1.txt");
        CPPUNIT_ASSERT(object_map->checkKey(msg3));
    }

    // ===================================================
    // Add object with its interface in list of interfaces
    // ===================================================
    void TestObjectMap::test_addObject_002()
    {
        object_map->addObject("Not yet known", orb, obj1);

        std::cout << "step_002 : ------ Object Map (1) -------" << std::endl;
        object_map->mapLoop(print_objectmap);
         msg1 = load_message("message1.txt");
       // Object should be in list of objects, toRecord = true
        CPPUNIT_ASSERT(object_map->checkKey(msg1));
    }

    // =======================================================
    // Add object with interface not in list of interfaces
    // =======================================================
    void TestObjectMap::test_addObject_003()
    {
        std::cout << "step_003: Add object with interface not in list of interfaces, size = " <<  object_map->size() << std::endl;
        object_map->addObject("Not yet known", orb, obj2);
        std::cout << "step_003: Added object obj2 with interface not in list of interfaces , size = " <<  object_map->size() << std::endl;
        std::cout << "step_003 : ------ Object Map (1) -------" << std::endl;
        object_map->mapLoop(print_objectmap);

        msg2 = load_message("message2.txt");
       // Object should not  be in list of objects, toRecord = false
        CPPUNIT_ASSERT(false == object_map->checkKey(msg2));
    }

    // =================================================
    // Add two identical objects
    // =================================================
    void TestObjectMap::test_addObject_004()
    {
        object_map->addObject("Not yet known", orb, obj2);

        std::cout << "step_004: Add two identical objects : add again obj2, size = " <<  object_map->size() << std::endl;
        std::cout << "step_004 : ------ Object Map (1) -------" << std::endl;
        object_map->mapLoop(print_objectmap);
        msg2 = load_message("message2.txt");
       // Object should contain 2 objects
        CPPUNIT_ASSERT(2 == object_map->size());
    }

    // ========================================================
    // Check checkKey method for object that is in collection,
    // but toRecord = false
    // ========================================================
    void TestObjectMap::test_checkKey_005()
    {
        std::cout << "step_005 : ------ Object Map (1) -------" << std::endl;
        object_map->mapLoop(print_objectmap);
        std::cout << "step_005: checkKey for object in map, but toRecord flag = false" << std::endl;
        CPPUNIT_ASSERT(false == object_map->checkKey(msg2));
    }
    // ========================================================
    // Check checkKey method for object that is in collection
    // and toRecord = true
    // ========================================================
    void TestObjectMap::test_checkKey_006()
    {
        std::cout << "step_006 : ------ Object Map (1) -------" << std::endl;
        object_map->mapLoop(print_objectmap);
        std::cout << "step_006: checkKey for object in map, and toRecord flag = true" << std::endl;
        msg1 = load_message("message1.txt");
        CPPUNIT_ASSERT(true == object_map->checkKey(msg1));
    }

    // ===========================================================
    // Check checkKey method for object that is not in collection
    // ===========================================================
    void TestObjectMap::test_checkKey_007()
    {
        std::cout << "step_007 : ------ Object Map (1) -------" << std::endl;
        object_map->mapLoop(print_objectmap);
        std::cout << "step_007: checkKey for object not in map" << std::endl;
        msg3 = load_message("request1.txt");
        CPPUNIT_ASSERT(false == object_map->checkKey(msg3));
    }

    // =======================================================
    // Check checkInterfaces method after
    // adding new interfaces in InterfaceFilter object
    // =======================================================
    void TestObjectMap::test_checkInterfaces_008()
    {
        std::cout << "step_008 : test_checkInterfaces" << std::endl;
        std::cout << "step_008 : ------ Object Map (1) -------" << std::endl;
        object_map->mapLoop(print_objectmap);
        //  Create list of interfaces that contains interface "int2"  list1
        CdmwRecording::String_List intf_list;
        intf_list.length(1);
        intf_list[0] = "IDL:TestInterface2:1.0 TRUE";

        //  Call addInterfaces method of IF1 object with parameters: list1 and expand
        interface_filter->addInterfaces(intf_list);
        interface_filter->mapLoop(print, print_flag);

        std::cout << "step_008 : ------ Object Map (2) -------" << std::endl;
        object_map->mapLoop(print_objectmap);
        std::cout << "step_008 : apply interfaces filter" << std::endl;
        //  Call checkInterfaces method of OM1
        object_map->checkInterfaces();

        // verify ? Obj1.toRecord = true
        // verify ? Obj2.toRecord = true

        msg1 = load_message("message1.txt");
        std::cout << "step_008: checking msg1" << std::endl;
        CPPUNIT_ASSERT(object_map->checkKey(msg1));
        std::cout << "step_008: checking msg2" << std::endl;
        msg2 = load_message("message2.txt");
        CPPUNIT_ASSERT(object_map->checkKey(msg2));
        std::cout << "step_008: end" << std::endl;
    }

    // =====================================================
    // Check checkInterfaces method after
    // deleting one of interfaces in InterfaceFilter object
    // =====================================================
    void TestObjectMap::test_checkInterfaces_009()
    {
        std::cout << "step_009 : test_checkInterfaces" << std::endl;
        std::cout << "step_009 : ------ Object Map (1) -------" << std::endl;
        object_map->mapLoop(print_objectmap);
        //Create list of interfaces that contains interface "int1"  list1
        CdmwRecording::String_List intf_list;
        intf_list.length(1);
        intf_list[0] = "IDL:TestInterface1:1.0";

        std::cout << "step_009 : remove interface IDL:TestInterface1:1.0 = " << std::endl;
        //Call removeInterfaces method of IF1 object with parameters: list1 and expand
        interface_filter->removeInterfaces(intf_list);
        interface_filter->mapLoop(print, print_flag);

        std::cout << "step_009 : ------ Object Map (2) before adding Object obj1 -------" << std::endl;
        object_map->mapLoop(print_objectmap);
        std::cout << "step_009 : adding object obj1" << std::endl;
        object_map->addObject("IDL:TestInterface1:1.0", orb, obj1);
        std::cout << "step_009 : ------ Object Map (3) after adding object obj1 -------" << std::endl;
        object_map->mapLoop(print_objectmap);
        std::cout << "step_009 : apply interfaces filter" << std::endl;
        //Call checkInterfaces method of OM1
        object_map->checkInterfaces();

        // verify ? Obj1.toRecord = false
        // verify ? Obj2.toRecord = true

        msg1 = load_message("message1.txt");
        std::cout << "step_009: checking msg1" << std::endl;
        CPPUNIT_ASSERT(false == object_map->checkKey(msg1));
        msg2 = load_message("message2.txt");
        std::cout << "step_009: checking msg2" << std::endl;
        CPPUNIT_ASSERT(object_map->checkKey(msg2));
    }

    //========================================================
    //  load 1 message from file
    //========================================================
    Message * TestObjectMap::load_message(std::string filename)
    {
        int req ;

        Message * msg = new Message();
        std::ifstream istrm(filename.c_str() , std::ifstream::in );

        if(!istrm)
        {
            std::cout<< "#### could not open input file "<<filename<<std::endl;
            delete msg;
            return 0;
        }
        try
        {
            istrm >> req ; // 1 request 2 reply
            istrm >> msg->channel.src_addr;
            istrm >> msg->channel.dst_addr;
            istrm >> msg->channel.src_port;
            istrm >> msg->channel.dst_port;
            istrm >> msg->header.magic[0] ;
            istrm >> msg->header.magic[1] ;
            istrm >> msg->header.magic[2] ;
            istrm >> msg->header.magic[3] ;
            short sh;
            istrm >> sh ; msg->header.version.major = (octet) sh ;
            istrm >> sh ; msg->header.version.minor= (octet) sh ;
            istrm >> sh ; msg->header.flags = (octet) sh;

            istrm >> sh ; msg->header.message_type = (octet) sh;
            istrm >> msg->header.length;
            istrm >> msg->date.tv_sec ;
            istrm >> msg->date.tv_usec ;
            istrm >> msg->need_swap ;

            msg->data = new octet[ msg->header.length + 12 ] ;
            for(int i = 0;  i < msg->header.length + 12; i++)
            {
              istrm >> sh ; msg->data[i] = (octet) sh ;
            }

            istrm >> msg->request_id  ;
            istrm >> msg->objectKey_Offset ;
            istrm >> msg->objectKey_Length ;
            istrm >> msg->operation_Offset ;

            std::string serv="", intf="";

            if(req  == 1)
            {
               while (istrm && !istrm.eof() && serv == "" ) getline(istrm, serv);
               msg->server_id = new char[serv.length()+1];
               strncpy((char *) msg->server_id, serv.c_str() , serv.length() );

               while (istrm && !istrm.eof() && intf == "" ) getline(istrm, intf);
               msg->intf_rid = new char[intf.length()+1];
               strncpy((char *) msg->intf_rid, intf.c_str() , intf.length() );
            }
        }
        catch(...)
        {
           std::cout<< "#### error on reading file "<< filename << std::endl;
           delete msg;
           return 0;
        }

        msg->next = 0;
        msg->fragments = 0;

        istrm.close();

        return msg ;
    }

}  // namespace Recording
}  // namespace Cdmw





