/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <iomanip>
#include <sstream>
#include "testobjectmap/TestInterface.stub.hpp"

using namespace std;

int main(int argc, char * argv[])
{
    std::string port = "";

    //
    //Check count of arguments
    //
    if (argc < 3)
    {
        cout << "CORBA client: too few arguments" << endl;
        throw 0;
    }

    try
    {
        // Initialize orb
        CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);
        std::cout << "init" << std::endl;

        //Object1
        CORBA::Object_var obj1 = orb->string_to_object(argv[1]);
        if (CORBA::is_nil(obj1))
        {
            cout << "CORBA client: Nil Time reference" << endl;
            throw 0;
        }
        std::cout << "obj1" << std::endl;

        // Narrow
        TestInterface1_var ti1 = TestInterface1::_narrow(obj1);
        if (CORBA::is_nil(ti1))
        {
            cout << "CORBA client: Argument is not a TestInterface1 reference" << endl;
            throw 0;
        }
        std::cout << "narrow1" << std::endl;

        ti1->ping();
        std::cout << "ping1" << std::endl;

        //the aim of this call is to crash the PacketStore with a message bigger than a tcp trame
        //here the message shall have 2Mo of data
        std::stringstream ss;
        for (int i = 0; i < 65536; i++)
            ss << "GIOPGIOPGIOPGIOPGIOPGIOPGIOPGIOP";
        ti1->testcrashstring(ss.str().c_str());


        //Object2
        CORBA::Object_var obj2 = orb->string_to_object(argv[2]);
        if (CORBA::is_nil(obj2))
        {
            cout << "CORBA client: Nil Time reference" << endl;
            throw 0;
        }
        std::cout << "obj2" << std::endl;

        // Narrow
        TestInterface2_var ti2 = TestInterface2::_narrow(obj2);
        if (CORBA::is_nil(ti2))
        {
            cout << "CORBA client: Argument is not a TestInterface2 reference" << endl;
            throw 0;
        }
        std::cout << "narrow2" << std::endl;

        ti2->ping();
        std::cout << "ping2" << std::endl;

    }
    catch (const CORBA::Exception &)
    {
        cout << "CORBA client: Uncaught CORBA exception" << endl;
        return 1;
    }
    catch (...)
    {
        return 1;
    }

    return 0;
}
