/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include "testobjectmap/TestInterface1_impl.hpp"
#include "testobjectmap/TestInterface2_impl.hpp"

using namespace std;

int main(int argc, char * argv[])
{
    try 
    {
        // Initialize orb
        CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);
        // Get reference to Root POA.
        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var poa = PortableServer::POA::_narrow(obj);
        // Activate POA manager
        PortableServer::POAManager_var mgr = poa->the_POAManager();
        mgr->activate();
        
        // Create the first object
        TestInterface1_impl transmission_servant1;
        // Write its stringified reference to file
        TestInterface1_var ti1 = transmission_servant1._this();
        CORBA::String_var str1 = orb->object_to_string(ti1);
                
        //save in file 
        cout << str1 << endl;
        FILE * pFile;
        pFile = fopen ("server1.ior","w");
        fprintf (pFile, "%s", ((std::string) str1).c_str() );
        fclose (pFile); 
        
        // Create the second object
        TestInterface2_impl transmission_servant2;
        // Write its stringified reference to file
        TestInterface2_var ti2 = transmission_servant2._this();
        CORBA::String_var str2 = orb->object_to_string(ti2);   
             
        //save in file        
        cout << str2 << endl;
        pFile = fopen ("server2.ior","w");
        fprintf (pFile, "%s", ((std::string) str2).c_str()) ;
        fclose (pFile); 
        
        // Accept requests
        orb->run();
    }
    catch (const CORBA::Exception &) 
    {
        cerr << "CORBA server: Uncaught CORBA exception" << endl;
    return 1;
    }
    
    return 0;
}
