/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
#include <iostream>
#include <stdio.h>

#include "recordinglibrary/OperationMap.hpp"
#include "testoperationmap/TestOperationMap.hpp"

namespace Cdmw
{
namespace Recording
{

CPPUNIT_TEST_SUITE_REGISTRATION(TestOperationMap);

TestOperationMap::TestOperationMap()
{
}

TestOperationMap::~TestOperationMap()  throw()
{
}

void print (std::string const & str1, std::size_t i)
{
  if (i == 0)
     {
       std::cout << "--------------------------------------" << std::endl;
     }
   std::cout << "   " << i << " " << str1 << "/length=" << str1.length() << std::endl;
}  

std::multiset<std::string> TestOperationMap::oper_set = std::multiset<std::string>() ;

void TestOperationMap::save (std::string const & str1, std::size_t i)
{ 
  oper_set.insert(str1);  
}


void TestOperationMap::test_create()
{
   std::string op1 = "oper1";
   std::string op2 = "operation1";
   std::string op3 = "finish_iteration2";
   std::string op4 = "ready3";

   TEST_INFO("Testing operation map creation + ");

   // ------- 001:  create OM with empty config file

   OperationMap oper_map_1 ("empty.txt") ;
   CPPUNIT_ASSERT(true == oper_map_1.checkOperation (op1) ); 

   // ------- 002:  create OM with non-empty config file
   //         015:  Check existing operation in collection
   //         017:  Check non-existing operation in collection

   OperationMap oper_map_2 ("operations.txt") ;
   CPPUNIT_ASSERT(true == oper_map_2.checkOperation (op2) );
   CPPUNIT_ASSERT(true == oper_map_2.checkOperation (op3) );
   CPPUNIT_ASSERT(true == oper_map_2.checkOperation (op4) );
   CPPUNIT_ASSERT(false == oper_map_2.checkOperation (op1) );

    //-------  003:  create OM with non-existent config file

   OperationMap oper_map_3 ("some.txt") ;
   CPPUNIT_ASSERT(true == oper_map_3.checkOperation (op1) );
}

void TestOperationMap::test_add()
{ 
  OperationMap::Status status ;  
  CdmwRecording::String_List olist;  
  CdmwRecording::String_List emlist;  
  CdmwRecording::String_List list1 ;   
  CdmwRecording::String_List sp_list ; 

  std::string op1 = "op1";
  std::string op2 = "oper2";
  std::string op3 = "fly3";
  std::string oper1 = "operation1";
  std::string oper2 = "finish_iteration2";
  std::string oper3 = "ready3";

   TEST_INFO("Test of adding a list"); 

   //-------  004: add a list to empty collection

   OperationMap omap1 ("empty.txt") ;
   olist.length(3);
   olist[0] = op1.c_str() ; 
   olist[1] = op2.c_str() ; 
   olist[2] = op3.c_str() ; 

   status = omap1.addOperations(olist);
   CPPUNIT_ASSERT(0 == status);  

   CPPUNIT_ASSERT(true == omap1.checkOperation (op1) );
   CPPUNIT_ASSERT(true == omap1.checkOperation (op2) );
   CPPUNIT_ASSERT(true == omap1.checkOperation (op3) );    
   
   //-- add a list to non-empty collection 

   OperationMap omap2 ("operations.txt");
   status = omap2.addOperations(olist);
   CPPUNIT_ASSERT(0 == status);  

   CPPUNIT_ASSERT(true == omap2.checkOperation (op1) );
   CPPUNIT_ASSERT(true == omap2.checkOperation (op2) );
   CPPUNIT_ASSERT(true == omap2.checkOperation (op3) );  
 
   TEST_INFO("Test of adding empty list");

   //------- 005: add empty list to empty collection

   OperationMap omap3 ("empty.txt");
   status = omap3.addOperations(emlist);
   CPPUNIT_ASSERT(0 == status);  
   CPPUNIT_ASSERT(true == omap3.checkOperation (op1) );  

   //-- add empty list to non-empty collection

   OperationMap omap4 ("operations.txt"); 
   status = omap4.addOperations(emlist);
   CPPUNIT_ASSERT(0 == status);  
   CPPUNIT_ASSERT(true == omap4.checkOperation (oper1) );
   CPPUNIT_ASSERT(true == omap4.checkOperation (oper2) );
   CPPUNIT_ASSERT(true == omap4.checkOperation (oper3) );   
   
   //------- 006: add an operation twice

   OperationMap omap5 ("empty.txt");  
   list1.length(1);
   list1[0] = op1.c_str() ; 
   status = omap5.addOperations(list1);
   CPPUNIT_ASSERT(0 == status); 
   status = omap5.addOperations(list1);
   CPPUNIT_ASSERT(0 == status);  
   CPPUNIT_ASSERT(true == omap5.checkOperation (op1) );  
      
   // check that op1 is only once in collection 
   omap5.mapLoop(TestOperationMap::save);
   CPPUNIT_ASSERT(1 == oper_set.count(op1));
   oper_set.clear(); 
    
   //------- 007: Check adding operations with leading and ending spaces

   OperationMap omap6("spaces.txt");
   sp_list.length(1);
   sp_list[0] = "  oper2 " ; 
   status = omap6.addOperations(sp_list);
   CPPUNIT_ASSERT(0 == status); 
   CPPUNIT_ASSERT(true == omap6.checkOperation (oper1) ); 
   CPPUNIT_ASSERT(true == omap6.checkOperation (op2) );
 }

void TestOperationMap::test_remove()
{
  OperationMap::Status status ;  

  CdmwRecording::String_List olist; 
  CdmwRecording::String_List list1;   
  CdmwRecording::String_List emlist;  
  CdmwRecording::String_List sp_list ; 

  std::string op2 = "oper";
  std::string op3 = "fly3";
  std::string oper1 = "operation1"; 
  std::string oper2 = "finish_iteration2";
  std::string oper3 = "ready3";

  TEST_INFO("Test of removing a list"); 

   //-------  008: Check removing existing operations

   OperationMap omap1 ("operations.txt") ;
   olist.length(2);
   olist[0] = oper2.c_str() ; 
   olist[1] = oper3.c_str() ; 

   status = omap1.removeOperations(olist);
   CPPUNIT_ASSERT(0 == status);  

   CPPUNIT_ASSERT(false == omap1.checkOperation (oper2) );
   CPPUNIT_ASSERT(false == omap1.checkOperation (oper3) );
   CPPUNIT_ASSERT(true == omap1.checkOperation (oper1) );    
   
   // ------- 009: Check removing non-existing operations
   
   OperationMap omap2 ("operations.txt") ;
   olist.length(2);
   olist[0] = op2.c_str() ; 
   olist[1] = op3.c_str() ; 

   status = omap2.removeOperations(olist);
   CPPUNIT_ASSERT(0 == status);  

   CPPUNIT_ASSERT(true == omap2.checkOperation (oper2) );
   CPPUNIT_ASSERT(true == omap2.checkOperation (oper3) );
   CPPUNIT_ASSERT(true == omap2.checkOperation (oper1) );    
  
   //------- 010: Check removing empty list of operations

   status = omap2.removeOperations(emlist);
   CPPUNIT_ASSERT(0 == status);    
   CPPUNIT_ASSERT(true == omap2.checkOperation (oper2) );
   CPPUNIT_ASSERT(true == omap2.checkOperation (oper3) );
   CPPUNIT_ASSERT(true == omap2.checkOperation (oper1) ); 
  
   // removing empty list from an empty collection

   OperationMap omap3 ("empty.txt") ;
   status = omap3.removeOperations(emlist);
   CPPUNIT_ASSERT(0 == status);  
 
   CPPUNIT_ASSERT(true == omap3.checkOperation (oper2) ); 
     
   //------- 011: Check removing operations with leading and ending spaces
  
   list1.length(1);
   list1[0] = "  operation1"; 
   status = omap2.removeOperations(list1);
   CPPUNIT_ASSERT(0 == status);  
   list1[0]  = "finish_iteration2 ";
   status = omap2.removeOperations(list1);
   CPPUNIT_ASSERT(0 == status);  

   CPPUNIT_ASSERT(false == omap2.checkOperation (oper2) );
   CPPUNIT_ASSERT(true  == omap2.checkOperation (oper3) );
   CPPUNIT_ASSERT(false == omap2.checkOperation (oper1) ); 
  
   // ------- 013: Check removing all operations from non-empty collection

   OperationMap omap4 ("operations.txt") ;
   status = omap4.removeAllOperations();
   CPPUNIT_ASSERT(0 == status);  
  
   // ------- 014: remove all op. from empty collection

   OperationMap omap5 ("empty.txt") ;
   status = omap5.removeAllOperations();
   CPPUNIT_ASSERT(0 == status);
 
   //------- 020: Check remove operation from empty collection

   OperationMap omap6 ("empty.txt") ;
   status = omap6.removeOperations(olist); 
   CPPUNIT_ASSERT(0 == status);
  
}

void TestOperationMap::test_check()
{
   std::string op2 = "finish_iteration2"; // leading separator in file
   std::string op3 = "ready3"; // trainling separator in file
   std::string op = "rea" ;
   
   //------- 016: Check existing operation in collection 
   //     with leading and ending spaces in file
   OperationMap omap1 ("operations.txt") ;

   CPPUNIT_ASSERT(OperationMap::FILTERING_MODE_MATCH == omap1.get_filtering_mode());
   
   CPPUNIT_ASSERT(true  == omap1.checkOperation (op2) ); 
   CPPUNIT_ASSERT(true  == omap1.checkOperation (op3) ); 
   
   //------- 018: Check checkOperation method of OperationMap object
   //  when parameter is empty
   std::string e = "";
   CPPUNIT_ASSERT(false  == omap1.checkOperation (e) ); 
   
   //------- 017: Check non-existing operation in collection 
   CPPUNIT_ASSERT(false  == omap1.checkOperation (op) );

}
void TestOperationMap::test_exclusion()
{
   // foo and bar operations are to be excluded
   std::string foo = "foo";
   std::string bar = "bar";
   std::string foo_bar_123 = "foo_bar_123";
   std::string dummy = "dummy" ;
   
   //
   OperationMap omap ("excluded_operations.txt") ;

   CPPUNIT_ASSERT(OperationMap::FILTERING_MODE_EXCLUDE == omap.get_filtering_mode());
   CPPUNIT_ASSERT(3 == omap.list_size());
   
   CPPUNIT_ASSERT(false  == omap.checkOperation (foo) ); 
   CPPUNIT_ASSERT(false  == omap.checkOperation (bar) ); 
   
   //------- : Check non-existing operation in collection 
   CPPUNIT_ASSERT(true  == omap.checkOperation (dummy) ); 

}

void TestOperationMap::do_tests()
{
  test_create();
  test_add();
  test_remove();
  test_check();
  test_exclusion();
}

}  // namespace Recording 
}  // namespace Cdmw

