/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
#include <iostream>
#include <fstream>
#include <string>
#include <stdio.h>

#include "testrecordingplugin/TestRecordingPlugin.hpp"
#include <Foundation/ossupport/OS.hpp>

namespace Cdmw
{
namespace Recording
{

CPPUNIT_TEST_SUITE_REGISTRATION(TestRecordingPlugin);

Cdmw::OsSupport::OS::ProcessId idRecordingServer = 0;

TestRecordingPlugin::TestRecordingPlugin()
{
}
TestRecordingPlugin::~TestRecordingPlugin()  throw()
{
}
bool TestRecordingPlugin::check_server_data(int buffers_count)
{
    int counter = 0;
    std::string line = "";
        
    ifstream istrm;
    istrm.open("recordingserver_data");
    int nb_retry_max = 10;
    int nb_retry = 0;
    while(!istrm.is_open() && (nb_retry <= nb_retry_max))
    {
      std::cout<<"Opening recordingserver_data ... retry ..."<<std::endl;
      sleep (1);
      istrm.open("recordingserver_data");
      nb_retry ++;
    }
    if (!istrm.is_open()) return false;
    std::cout<<"opened recordingserver_data, waiting 3 sec ... "<<std::endl;
    
    sleep(3); // wait for Recording Server to flash data
        
    while(istrm >> line) 
    {
        if(line.find("buffer",0)!= std::string::npos) counter++;       
    }
    istrm.close();
        
    std::remove("recordingserver_data");

    if (counter == buffers_count)
        return true;
    else
        {
            std::cout << "## found " << counter << " buffers"<< std::endl;
            return false;
        }    
}
//===============================================================
//  load 1 message from file
//=============================================================== 
Message * TestRecordingPlugin::load_message(std::string filename)
{ 
    int req ;
    
    Message * msg = new Message();  
    std::ifstream istrm(filename.c_str() , std::ifstream::in );
    
    if(!istrm) 
    { 
        std::cout<< "#### could not open input file "<<filename<<std::endl;
        delete msg;
        return 0; 
    }
    try
    {
        istrm >> req ; // 1 request 2 reply                
        istrm >> msg->channel.src_addr;              
        istrm >> msg->channel.dst_addr;        
        istrm >> msg->channel.src_port;
        istrm >> msg->channel.dst_port;
        istrm >> msg->header.magic[0] ; 
        istrm >> msg->header.magic[1] ;
        istrm >> msg->header.magic[2] ;
        istrm >> msg->header.magic[3] ;                 
        short sh;
        istrm >> sh ; msg->header.version.major = (octet) sh ;
        istrm >> sh ; msg->header.version.minor= (octet) sh ;
        istrm >> sh ; msg->header.flags = (octet) sh;
           
        istrm >> sh ; msg->header.message_type = (octet) sh;       
        istrm >> msg->header.length;       
        istrm >> msg->date.tv_sec ; 
        istrm >> msg->date.tv_usec ;   
        istrm >> msg->need_swap ;
                
        msg->data = new octet[ msg->header.length + 12 ] ;    
        for(int i = 0;  i < msg->header.length + 12; i++)
        { 
          istrm >> sh ; msg->data[i] = (octet) sh ;          
        }  
                         
        istrm >> msg->request_id  ;      
        istrm >> msg->objectKey_Offset ;
        istrm >> msg->objectKey_Length ; 
        istrm >> msg->operation_Offset ;
                  
        std::string serv="", intf="";
                      
        if(req  == 1)
        {      
           while (istrm && !istrm.eof() && serv == "" ) getline(istrm, serv);  
           msg->server_id = new char[serv.length()+1];
           strncpy((char *) msg->server_id, serv.c_str() , serv.length() );  
                  
           while (istrm && !istrm.eof() && intf == "" ) getline(istrm, intf);     
           msg->intf_rid = new char[intf.length()+1];    
           strncpy((char *) msg->intf_rid, intf.c_str() , intf.length() );
        }   
    }
    catch(...)
    {
       std::cout<< "#### error on reading file "<< filename << std::endl;
       delete msg;
       return 0;   
    }
      
    msg->next = 0;
    msg->fragments = 0; 
    
    istrm.close();
    
    return msg ; 
}
void TestRecordingPlugin::setUp()
{
    orb = Cdmw::TestUtils::CORBATestManager::instance()->get_ORB() ;  
    
    //construct valid url (recording server emulator)
    char host_name[100] = "";
    gethostname(host_name, sizeof(host_name));
    //url = std::string("corbaloc:iiop:") + std::string(host_name)
    //    + ":6789/Recorder" ;
    url = std::string("corbaloc:iiop:") + std::string(host_name)
        + ":12344/Recorder" ;
}

void TestRecordingPlugin::tearDown()
{
 
}
void TestRecordingPlugin::start_rec_server_emulator()
{
    // Start RecordingServer emulator 
    std::cout<< "Starting Recording Server emulator.."<< std::endl;
    try 
    {
       idRecordingServer = Cdmw::OsSupport::OS::create_process
          ( "recording_server", "--CdmwProcessPort=12344");
    } 
       catch (const std::exception & ex)
    {
       std::cerr << "*** Exception : " << ex.what() << std::endl;
    }
    
    std::cout<<"waiting 5 sec.."<<std::endl;  
    sleep(5);         
}
 
void TestRecordingPlugin::test_initialize_finalize_1()
{
    std::cout<<"001: Check initialize and finalize"<<std::endl ;
    
    RecordingPlugin rp;
    rp.initialize(orb);
    rp.finalize();  
    
}
void TestRecordingPlugin::test_initialize_finalize_2()
{
    std::cout<<"002: Check finalize without initialize"<<std::endl ;
    
    RecordingPlugin rp;
    rp.finalize(); 
}
void TestRecordingPlugin::test_initialize_finalize_3()
{
    std::cout<<"003: Check initialize and finalize two times"<<std::endl ;
    
    RecordingPlugin rp; 
    rp.initialize(orb);
    rp.finalize(); 
    
    CPPUNIT_ASSERT_NO_THROW(rp.finalize()) ; 
}
void TestRecordingPlugin::test_initialize_finalize_4()
{
    std::cout<<"004: Check initialize two times and finalize"<<std::endl ;

    RecordingPlugin rp;    
    rp.initialize(orb);
    
    CPPUNIT_ASSERT_NO_THROW(rp.initialize(orb));

    rp.finalize();    
}
void TestRecordingPlugin::test_type_5()  
{
    std::cout<<"005: Check type "<<std::endl ;

    RecordingPlugin rp; 
    CPPUNIT_ASSERT(1 == rp.type()); 
}
void TestRecordingPlugin::test_destination_6()  
{
    std::cout<<"006: Check get destination value without setting it"<<std::endl ;
    
    RecordingPlugin rp;               
    rp.initialize(orb);
    
    CPPUNIT_ASSERT(0 == rp.Destination()); 
    
    rp.finalize();
}      
void TestRecordingPlugin::test_destination_7()  
{
    std::cout<<"007: Check set and get destination value "<<std::endl ;
    
    RecordingPlugin rp; 
    rp.initialize(orb);
    
    rp.Destination((char *) url.c_str());
    CPPUNIT_ASSERT(url == std::string(rp.Destination()));      
    
    rp.finalize();
}      
void TestRecordingPlugin::test_destination_8() 
{
    std::cout<<"008: Check set and get destination value with 0 parameter"<<std::endl ;
    
    RecordingPlugin rp;          
    rp.initialize(orb); 
     
    char * nu = 0;     
    rp.Destination(nu); 
    
    CPPUNIT_ASSERT(0 == rp.Destination());      
    
    rp.finalize();     
}
void TestRecordingPlugin::test_record_9()  
{
    std::cout<<"009: Check recordOperation with correct parameters and url : "<< url <<std::endl ;
     
    start_rec_server_emulator();
    
    RecordingPlugin rp;     
    rp.initialize(orb);     
    //rp.Destination((const char *) url.c_str());
    rp.Destination(url.c_str());
    
    Message * request = load_message("request1.txt");
    Message * reply   = load_message("reply1.txt");
    
    rp.recordOperation(request,reply);
    
    std::cout<<"recordOperation called, waiting 5 sec.."<<std::endl;  
    sleep(5);  
     
    CPPUNIT_ASSERT(check_server_data(1)) ;
    
    rp.finalize();      
    std::cout<<"009 : Killing Recording Server ..."<<std::endl;  
    Cdmw::OsSupport::OS::kill_process (idRecordingServer);
   
}      
void TestRecordingPlugin::test_record_10()    
{
    std::cout<<"010: Check recordOperation without setting destination "<<std::endl ;
    
    RecordingPlugin rp;          
    rp.initialize(orb); 
    
    Message * request = load_message("request1.txt");
    Message * reply   = load_message("reply1.txt");
    
    CPPUNIT_ASSERT_NO_THROW(rp.recordOperation(request,reply));
    
    rp.finalize();     
}    
void TestRecordingPlugin::test_record_11()   
{
    std::cout<<"011: Check recordOperation with invalid destination "<<std::endl ;
    
    RecordingPlugin rp;      
    rp.initialize(orb); 
    
    std::string inval = std::string("corbaloc:iiop:21_*:-)");
    
    rp.Destination((char *) inval.c_str());
    
    Message * request = load_message("request1.txt");
    Message * reply   = load_message("reply1.txt");
    
    CPPUNIT_ASSERT_NO_THROW(rp.recordOperation(request,reply));
     
    rp.finalize();        
}     
void TestRecordingPlugin::test_record_12()  
{
    std::cout<<"012: Check recordOperation with NULL destination"<<std::endl ;
    
    RecordingPlugin rp;           
    rp.initialize(orb); 
   
    char * nu = 0;     
    rp.Destination(nu);
    
    Message * request = load_message("request1.txt");
    Message * reply   = load_message("reply1.txt");
    
    CPPUNIT_ASSERT_NO_THROW(rp.recordOperation(request,reply));
     
    rp.finalize();          
}      
void TestRecordingPlugin::test_record_13()    
{
    std::cout<<"013: Check recordOperation with NULL parameters"<<std::endl ;
    
    RecordingPlugin rp;           
    rp.initialize(orb);  
    
    CPPUNIT_ASSERT_NO_THROW(rp.recordOperation(0, 0));
    
    rp.finalize();
}
void TestRecordingPlugin::test_destination()    
{
    std::cout<<"Check Destination without initialize "<<std::endl ;
    RecordingPlugin rp;
    
    rp.Destination(); 
    rp.Destination(0); 
    rp.Destination("ab");  
    
    std::cout<<"Check Destination after initialize "<<std::endl ;
    rp.initialize(orb);   
    rp.Destination(); 
    rp.Destination(0); 
    rp.Destination("ab");  
    rp.finalize();
}

void TestRecordingPlugin::test_parameterization()    
{
    std::cout<<"Check Parameterization"<<std::endl ;
    RecordingPlugin rp;
    
    /* loop 
     * 1st iter : rp not iniatilalized
     * 2nd iter : rp initialized
     */
    
    for (int iter=0; iter <2; iter++)
    {
    /* MaxBufferNumber setting */
    size_t size = rp.GetMaxBufferNumber();
    size ++;
    rp.SetMaxBufferNumber (size);
    if (iter >0) CPPUNIT_ASSERT(size == rp.GetMaxBufferNumber());      
    
    /* RetryPeriod setting */
    milliseconds delay = rp.GetRetryPeriod();
    delay ++;
    rp.SetRetryPeriod (delay);
    if (iter >0) CPPUNIT_ASSERT(delay == rp.GetRetryPeriod()); 
    
    /* SetRetryNumber setting */
    long number = 10;
    if (iter >0) CPPUNIT_ASSERT_NO_THROW ( rp.SetRetryNumber (number));      

    /* SetCorbaTimeout setting */
    unsigned long timeout = 10;
    if (iter >0) CPPUNIT_ASSERT_NO_THROW ( rp.SetCorbaTimeout (timeout));      
    
    /* Maxsize setting */
    size_t max_size = rp.MaxSize();
    max_size ++;
    rp.MaxSize (max_size);
    if (iter >0) CPPUNIT_ASSERT(max_size == rp.MaxSize());      

    /* Timeout setting */
    milliseconds a_timeout = rp.TimeOut();
    a_timeout ++;
    rp.TimeOut (a_timeout);
    if (iter >0) CPPUNIT_ASSERT(a_timeout == rp.TimeOut());
    
    if (iter == 0) rp.initialize(orb); /* init for next loop */
    }
    rp.finalize();
}

}  // namespace Recording 
}  // namespace Cdmw




