/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <unistd.h>
#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/Logger.hpp>
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/CORBASystemExceptionWrapper.hpp"
#include "testrecordingplugin/TestRecordingPlugin.hpp"
#include <Foundation/ossupport/OS.hpp>

namespace 
{
    const int SUCCESS = 0;
    const int FAILURE = 1;

    const int POA_THREAD_POOL_SIZE = 10;//5;

}; // End anonymous namespace

int main(int argc, char** argv )
{
    std::cout << "Start of test" << std::endl;
    int status = SUCCESS;
    
    CORBA::ORB_var orb;                        // orb reference
    
    try 
    {
        using Cdmw::Logging::LogManager;
        LogManager::Init(argc, argv);
        // Initialize the ORB
        Cdmw::OrbSupport::StrategyList orb_strategies;
        orb_strategies.add_OrbThreaded();
        orb_strategies.add_PoaThreadPool(POA_THREAD_POOL_SIZE);

        char host_name[100] = "";
        gethostname(host_name, sizeof(host_name));

        int margc = 2 ;
        char * margv[2];
        margv[0] = "-ORBEndpoint"; 
        std::string tmp_str = std::string("iiop://") + std::string(host_name)
                            + std::string(":6788");                             
        margv[1] = (char *) tmp_str.c_str();  

        orb = Cdmw::OrbSupport::OrbSupport::ORB_init(margc,(char**) margv, orb_strategies);
        
        // ===================================================
        // Get the root POA 
        // ===================================================
        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj.in());
        
        // ===================================================
        // Activate the root POA manager
        // ===================================================
        PortableServer::POAManager_var poaManager = rootPOA->the_POAManager();
        poaManager->activate();
        
       Cdmw::TestUtils::CORBATestManager::instance()->init(orb.in(),rootPOA.in(),argc,argv);
       Cdmw::TestUtils::CORBATestManager::instance()->set_property ("name",argv[0]);    
       Cdmw::TestUtils::CORBATestManager::instance()->run_tests();
       Cdmw::TestUtils::CORBATestManager::instance()->summary();
    } 
    catch (const Cdmw::OrbSupport::CORBASystemExceptionWrapper & ex)
    {
        std::cerr << ex.what() << std::endl;
    }
    catch (const Cdmw::Exception & ex)
    {
        std::cerr << ex.what() << std::endl;
    } 
    catch(const CORBA::Exception& ex)  
    {        
        std::cerr << ex << std::endl;
    }
    catch(...)  
    {        
        std::cerr << " * unexpected exception" << std::endl;
    }
    
    // ===================================================
    // destroy orb
    // ===================================================
    
    if (!CORBA::is_nil(orb.in()))
    {
        try 
        {
            orb -> shutdown(false);
            Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
            orb -> destroy();
        }
        catch(const CORBA::Exception& ex)
        {           
            std::cerr << ex << std::endl;
            status = FAILURE;
        }
    }
    return status;
}


