/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <stdio.h>
#include <fstream>

#include "testrequestpairing/MyRequestPairing.hpp"
#define REQUEST 1
#define REPLY   2

namespace Cdmw
{
namespace Recording
{
 
MyRequestPairing::~MyRequestPairing()  throw()
{
  om->close();
  delete om ; 
}
 
void MyRequestPairing::write(Message * msg, int request_flag)
{
    (*om) << request_flag << std::endl ;
    (*om) << msg->channel.src_addr << std::endl 
          << msg->channel.dst_addr << std::endl 
          << msg->channel.src_port << std::endl 
          << msg->channel.dst_port << std::endl ;
          
    (*om) << msg->header.magic[0] << msg->header.magic[1] 
          << msg->header.magic[2] << msg->header.magic[3] << std::endl;

    if (request_flag == REQUEST)
    {
      // If oneway      
      if((msg->data[16] & 1) == 0)
      {
        std::cerr<< "         found 1 way request " << std::endl ;     
      } 
      else
      {
        std::cerr<< "         (not one way request) " << std::endl ;
      }
    }
    else
    {
      std::cerr<< "         found reply " << std::endl ;   
    }

    (*om) <<  (short) msg->header.version.major << std::endl
          <<  (short) msg->header.version.minor << std::endl ;
    (*om) <<  (short) msg->header.flags << std::endl 
          <<  (short) msg->header.message_type << std::endl ;
          
   
    
    (*om) << msg->header.length << std::endl ;
    
    (*om) << msg->date.tv_sec << std::endl << msg->date.tv_usec << std::endl ;
    (*om) << msg->need_swap << std::endl ;

    for(int i = 0; i < 12 + msg->header.length ; i++) 
       (*om) <<  (short) msg->data[i] << std::endl;
          
    (*om) << (unsigned int) msg->request_id << std::endl ;
    
    std::cerr<<"              request_id "<< msg->request_id <<std::endl ; 
    
    (*om) << msg->objectKey_Offset << std::endl ;
    (*om) << msg->objectKey_Length << std::endl ;
    (*om) << msg->operation_Offset << std::endl ;
      
    if(request_flag==REQUEST) (*om) << std::string(msg->server_id).c_str() << std::endl ; 
    if(request_flag==REQUEST) (*om) << std::string(msg->intf_rid).c_str()  << std::endl ;  
}    
    
void MyRequestPairing::storeRequest (Message * request) throw() 
{
  write(request, REQUEST) ;     
}

void MyRequestPairing::checkReply (Message * reply)  throw() 
{
  write(reply, REPLY) ;  
}

 
}; // End of Recording namespace 
}; // End of Cdmw namespace 
 

