/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
#include <iostream>
#include <fstream>
#include <stdio.h>


#include "testrequestpairing/TestRequestPairing.hpp"
#include "testrequestpairing/MyPlugin.hpp"

namespace Cdmw
{
namespace Recording
{

CPPUNIT_TEST_SUITE_REGISTRATION(TestRequestPairing);

TestRequestPairing::TestRequestPairing()
{
}
TestRequestPairing::~TestRequestPairing()  throw()
{
}

//=============================================================== 
// Check storeRequest method for oneway request message
//=============================================================== 
void TestRequestPairing::test_1(RequestPairing * pairing, 
                                MyPlugin * plug,  
                                PluginType type)
{
    pairing->addPlugin(plug);
    
    std::cout<<"test 1"<<std::endl;
    
    pairing->setDestination(type,"");
      
    Message * msg = load_message("onewayreq.txt");  
    check_oneway(msg);
      
    pairing->storeRequest(msg);      
     
    CPPUNIT_ASSERT(1 == plug->count_request);
      
    plug->finalize();    
}

//=============================================================== 
// Check storeRequest and checkReply methods for twoway request 
// message and his reply
//=============================================================== 
void TestRequestPairing::test_2(RequestPairing * pairing,
                                MyPlugin * plug )
{ 
     std::cout<<"test 2"<<std::endl;
         
     Message * req1 = load_message("request1.txt");
     pairing->storeRequest(req1);  
      
     CPPUNIT_ASSERT(0 == plug->count_request); 

     Message * rep1 = load_message("reply1.txt");
     check_request_id(req1,rep1);
     
     pairing->checkReply(rep1);        
      
     CPPUNIT_ASSERT(1 == plug->count_reply);
     CPPUNIT_ASSERT(1 == plug->count_request); 
     
     plug->finalize();       
}
//=============================================================== 
//Check checkReply method for request message that was not sent
//=============================================================== 
void TestRequestPairing::test_3(RequestPairing * pairing,
                                MyPlugin * plug )
{ 
    std::cout<<"test 3"<<std::endl;  
    Message * req1 = load_message("request1.txt"); 
    pairing->storeRequest(req1); 
      
    CPPUNIT_ASSERT(0 == plug->count_request); 
         
    Message * rep2 = load_message("reply2.txt"); 
    pairing->checkReply(rep2);       
     
    CPPUNIT_ASSERT(0 == plug->count_reply);
    CPPUNIT_ASSERT(0 == plug->count_request); 
       
    Message * rep1 = load_message("reply1.txt");
    check_request_id(req1,rep1);
    pairing->checkReply(rep1);
      
    CPPUNIT_ASSERT(1 == plug->count_reply);
    CPPUNIT_ASSERT(1 == plug->count_request); 
     
    plug->finalize();       
}
//====================================================================== 
// Check storeRequest method will save request without reply by timeout
//======================================================================
void TestRequestPairing::test_4(RequestPairing * pairing,
                                MyPlugin * plug )
{ 
    std::cout<<"test 4"<<std::endl; 
    
    Message * req1 = load_message("request1.txt"); 
    pairing->storeRequest(req1); 
       
    CPPUNIT_ASSERT(0 == plug->count_request); 
    
    // TODO wait timeout expired
     
    CPPUNIT_ASSERT(1 == plug->count_reply);
    CPPUNIT_ASSERT(1 == plug->count_request); 
     
    plug->finalize();       
}
 
//=============================================================== 
// Check storeRequest method for oneway request message
//=============================================================== 
void TestRequestPairing::test_5(RequestPairing * pairing,
                                MyPlugin * plug1, MyPlugin * plug2,
                                PluginType type2)
{ 
    pairing->addPlugin(plug2);
    
    std::cout<<"test 5"<<std::endl;
    
    pairing->setDestination(type2,"");
      
    Message * msg = load_message("onewayreq.txt");  
    check_oneway(msg);
    pairing->storeRequest(msg);     
         
    CPPUNIT_ASSERT(1 == plug1->count_request);
    CPPUNIT_ASSERT(1 == plug2->count_request);
    
    plug1->finalize();    
    plug2->finalize();           
} 
//=============================================================== 
// Check storeRequest and checkReply methods for twoway request 
// message and his reply
//=============================================================== 
void TestRequestPairing::test_6(RequestPairing * pairing,
                                MyPlugin * plug1, MyPlugin * plug2 )
{ 
    std::cout<<"test 6"<<std::endl;
        
    Message * req1 = load_message("request1.txt");
    pairing->storeRequest(req1);  
          
    CPPUNIT_ASSERT(0 == plug1->count_request); 
    CPPUNIT_ASSERT(0 == plug2->count_request); 

    Message * rep1 = load_message("reply1.txt");
    check_request_id(req1,rep1);
     
    pairing->checkReply(rep1);          
     
    CPPUNIT_ASSERT(1 == plug1->count_reply);
    CPPUNIT_ASSERT(1 == plug1->count_request); 
    CPPUNIT_ASSERT(1 == plug2->count_reply);
    CPPUNIT_ASSERT(1 == plug2->count_request); 
     
    plug1->finalize(); 
    plug2->finalize();        
}
//=============================================================== 
//Check checkReply method for request message that was not sent
//=============================================================== 
void TestRequestPairing::test_7(RequestPairing * pairing,
                                MyPlugin * plug1, MyPlugin * plug2)
{ 
    std::cout<<"test 7"<<std::endl;   
 
    Message * req1 = load_message("request1.txt"); 
    pairing->storeRequest(req1); 
           
    CPPUNIT_ASSERT(0 == plug1->count_request);
    CPPUNIT_ASSERT(0 == plug2->count_request); 
         
    Message * rep2 = load_message("reply2.txt"); 
    pairing->checkReply(rep2);       
          
    CPPUNIT_ASSERT(0 == plug1->count_reply);
    CPPUNIT_ASSERT(0 == plug1->count_request); 
    CPPUNIT_ASSERT(0 == plug2->count_reply);
    CPPUNIT_ASSERT(0 == plug2->count_request); 
       
    Message * rep1 = load_message("reply1.txt");
    check_request_id(req1,rep1);
    pairing->checkReply(rep1);
     
    CPPUNIT_ASSERT(1 == plug1->count_reply);
    CPPUNIT_ASSERT(1 == plug1->count_request); 
    CPPUNIT_ASSERT(1 == plug2->count_reply);
    CPPUNIT_ASSERT(1 == plug2->count_request); 
     
    plug1->finalize(); 
    plug2->finalize();        
}
//====================================================================== 
// Check storeRequest method will save request without reply by timeout
//======================================================================
void TestRequestPairing::test_8(RequestPairing * pairing,
                                MyPlugin * plug1, MyPlugin * plug2 )
{ 
    std::cout<<"test 8"<<std::endl;   

    Message * req1 = load_message("request1.txt"); 
    pairing->storeRequest(req1);   
     
    CPPUNIT_ASSERT(0 == plug1->count_request); 
    CPPUNIT_ASSERT(0 == plug2->count_request); 
    
    // TODO wait timeout expired
     
    CPPUNIT_ASSERT(1 == plug1->count_reply);
    CPPUNIT_ASSERT(1 == plug1->count_request); 
    CPPUNIT_ASSERT(1 == plug2->count_reply);
    CPPUNIT_ASSERT(1 == plug2->count_request); 
     
    plug1->finalize();
    plug2->finalize();        
}
//=============================================================== 
// Check storeRequest method for oneway request message
// (no plugins)
//=============================================================== 
void TestRequestPairing::test_9(RequestPairing * pairing)
{
    std::cout<<"test 9"<<std::endl;
    
    Message * msg = load_message("onewayreq.txt"); 
    pairing->storeRequest(msg);              
}

//=============================================================== 
// Check storeRequest and checkReply methods for twoway request 
// message and his reply (no plugins)
//=============================================================== 
void TestRequestPairing::test_10(RequestPairing * pairing )
{ 
    std::cout<<"test 10"<<std::endl;
          
    Message * req = load_message("request1.txt"); 
    pairing->storeRequest(req);
      
    Message * rep = load_message("reply1.txt"); 
    pairing->checkReply(rep);  
}
//=============================================================== 
// Check finalize stores waiting requests and clear requests collection
//=============================================================== 
void TestRequestPairing::test_11(RequestPairing * pairing,
                                 MyPlugin * plug,
                                 PluginType type )
{ 
    std::cout<<"test 11"<<std::endl;  
     
    pairing->addPlugin(plug);
    pairing->setDestination(type,"");
     
    Message * req = load_message("request1.txt"); 
    pairing->storeRequest(req);  
     
    //pairing->finalize();            
    Message * rep = load_message("reply1.txt"); 
    pairing->checkReply(rep);
   
    print(11,plug); 
    CPPUNIT_ASSERT(1 == plug->count_request); 
    CPPUNIT_ASSERT(1 == plug->count_reply);    

    //Message * rep = load_message("reply1.txt"); 
    //pairing->checkReply(rep);
      
    pairing->finalize();            
    print(11,plug);
    CPPUNIT_ASSERT(0 == plug->count_reply);
    CPPUNIT_ASSERT(0 == plug->count_request); 
     
    plug->finalize();       
}
//=============================================================== 
// Check finalize method clears plugins
//=============================================================== 
void TestRequestPairing::test_12(RequestPairing * pairing,
                                 MyPlugin * plug,
                                 PluginType type )
{ 
    std::cout<<"test 12"<<std::endl;  
     
    pairing->addPlugin(plug);
    pairing->setDestination(type,"");
    
    pairing->finalize();         
     
    Message * req = load_message("request1.txt"); 
    pairing->storeRequest(req); 
    
    Message * rep = load_message("reply1.txt"); 
    pairing->checkReply(rep);
     
    CPPUNIT_ASSERT(0 == plug->count_reply);
    CPPUNIT_ASSERT(0 == plug->count_request); 
     
    plug->finalize();  
}
//====================================================================== 
// Check storeFragment method for the fragment of non-existent request
//====================================================================== 
void TestRequestPairing::test_13()
{
    std::cout<<"test 13"<<std::endl;  
    
    PacketStore store;
    RequestPairing pairing(store);
    
    PluginType type1 = 5;
    MyPlugin * plug = new MyPlugin(type1) ;
    pairing.addPlugin(plug);
    pairing.setDestination(type1,"");
    
    // twoway request1
    Message * message1 = load_message("request1.txt");
    // fragment of request2
    // Message * message2 = load_message("fragment1.txt"); // TODO 
    // reply1 for the request1
    Message * message3 = load_message("reply1.txt");    
    // reply2 for the request2
    Message * message4 = load_message("reply2.txt"); 
       
    pairing.storeRequest(message1);
    
    //if(message2!= 0) 
    //    pairing.storeFragment(message2); // TODO
    
    pairing.checkReply(message3); 
    
    CPPUNIT_ASSERT(1 == plug->count_reply);
    CPPUNIT_ASSERT(1 == plug->count_request); 
    
    pairing.checkReply(message4);
    
    // TODO verify saved request and replies
    
    CPPUNIT_ASSERT(1 == plug->count_reply);
    CPPUNIT_ASSERT(1 == plug->count_request);     
}
//====================================================================== 
// Check storeFragment method for the fragment of oneway request 
//====================================================================== 
void TestRequestPairing::test_14()
{
    std::cout<<"test 14"<<std::endl;  
    PacketStore store;
    RequestPairing pairing(store);
    
    PluginType type1 = 5;
    MyPlugin * plug = new MyPlugin(type1) ;
    pairing.addPlugin(plug);
    pairing.setDestination(type1,"");
    
    // oneway request1 that divided on fragments
    Message * message1 = load_message("onewayreq.txt");
    // fragment of request1
    // Message * message2 = load_message("fragment2.txt"); // TODO         
    
    pairing.storeRequest(message1);
    //if(message2!= 0) 
    //    pairing.storeFragment(message2); // TODO 
    
    // TODO Verify saved requests and replies
    // Full request1 should be saved     
}
//====================================================================== 
// Check storeFragment method for the fragment of twoway request
//====================================================================== 
void TestRequestPairing::test_15()
{  
    std::cout<<"test 15"<<std::endl;  
    PacketStore store;
    RequestPairing pairing(store);
    
    PluginType type1 = 5;
    MyPlugin * plug = new MyPlugin(type1) ;
    pairing.addPlugin(plug);
    pairing.setDestination(type1,"");
    
    // twoway request1 that divided on fragments 
    Message * message1 = load_message("request1.txt");
    // fragment of request1
    // Message * message2 = load_message("fragment1.txt"); // TODO
    // reply1 for the request1
    Message * message3 = load_message("reply1.txt");
    
    pairing.storeRequest(message1);
    
    //if(message2!= 0) 
    //    pairing.storeFragment(message2); //  TODO 
    
    pairing.checkReply(message3);
    
    CPPUNIT_ASSERT(1 == plug->count_reply);
    CPPUNIT_ASSERT(1 == plug->count_request);    
    
    // TODO Verify saved requests and replies   
    // Full request1 and reply1 should be saved   
}
//====================================================================== 
// Check storeFragment method for the fragment of non-existent reply
//====================================================================== 
void TestRequestPairing::test_16()
{
    std::cout<<"test 16"<<std::endl;  
    PacketStore store;
    RequestPairing pairing(store);
    
    PluginType type1 = 5;
    MyPlugin * plug = new MyPlugin(type1) ;
    pairing.addPlugin(plug);
    pairing.setDestination(type1,""); 
    
    // twoway request1 
    Message * message1 = load_message("request1.txt");
    // fragment of reply1
    // Message * message2 = load_message("fragment3.txt"); // TODO
    // reply1 for the request1
    Message * message3 = load_message("reply1.txt");
    
    pairing.storeRequest(message1);
    
    //if(message2!= 0) 
    //    pairing.storeFragment(message2); //TODO 
    
    pairing.checkReply(message3);
 
    CPPUNIT_ASSERT(1 == plug->count_reply);
    CPPUNIT_ASSERT(1 == plug->count_request);  
      
    // TODO Verify saved requests and replies   
    // request1 and reply1 should be saved      
}
//====================================================================== 
// Check storeFragment method for the fragment of existent reply
//====================================================================== 
void TestRequestPairing::test_17()
{
    std::cout<<"test 17"<<std::endl;  
    PacketStore store;
    RequestPairing pairing(store);
    
    PluginType type1 = 5;
    MyPlugin * plug = new MyPlugin(type1) ;
    pairing.addPlugin(plug);
    pairing.setDestination(type1,"");
    
    // twoway request1 
    Message * message1 = load_message("request1.txt");
    // reply1 for the request1 (reply divided on fragments)
    Message * message2 = load_message("reply1.txt"); 
    // fragment of reply1
    // Message * message3 = load_message("fragment3.txt"); //TODO
    
    pairing.storeRequest(message1);
    pairing.checkReply(message2);
    
    // if(message3!= 0) 
    //    pairing.storeFragment(message3); //TODO 
 
    CPPUNIT_ASSERT(1 == plug->count_reply);
    CPPUNIT_ASSERT(1 == plug->count_request);  
      
    // TODO Verify saved requests and replies   
    // request1 and full reply1 should be saved           
}

//=============================================================== 
// print count of requests and replies in plugin
//=============================================================== 
void TestRequestPairing::print(int testnr, MyPlugin * plug)
{
    std::cout << "--- test# "<< testnr
              << "   --- plugin type "<< (short) plug->type()  
              << "   requests "<< plug->count_request     
              << "   replies " << plug->count_reply << std::endl; 
}

void TestRequestPairing::check_request_id(Message * m1, Message * m2)
{
    if(m1 && m2 && m1->request_id != m2->request_id)
    {
        std::cout << " ### wrong reply" ;
        std::cout << " " << m1->request_id ;
        std::cout << " " << m2->request_id << std::endl ;
    } 
}
void TestRequestPairing::check_oneway(Message * m)
{
    if (m != 0 && (m->data [16] & 1) != 0 ) 
       std::cout<<" ### wrong one way request "<<std::endl;  
}
//===============================================================
//  load 1 message from file
//=============================================================== 
Message * TestRequestPairing::load_message(std::string filename)
{ 
    int req ;
    
    Message * msg = new Message();  
    std::ifstream istrm(filename.c_str() , std::ifstream::in );
    
    if(!istrm) 
    { 
        std::cout<< "#### could not open input file "<<filename<<std::endl;
        delete msg;
        return 0; 
    }
    try
    {
        istrm >> req ; // 1 request 2 reply                
        istrm >> msg->channel.src_addr;              
        istrm >> msg->channel.dst_addr;        
        istrm >> msg->channel.src_port;
        istrm >> msg->channel.dst_port;
        istrm >> msg->header.magic[0] ; 
        istrm >> msg->header.magic[1] ;
        istrm >> msg->header.magic[2] ;
        istrm >> msg->header.magic[3] ;                 
        short sh;
        istrm >> sh ; msg->header.version.major = (octet) sh ;
        istrm >> sh ; msg->header.version.minor= (octet) sh ;
        istrm >> sh ; msg->header.flags = (octet) sh;
           
        istrm >> sh ; msg->header.message_type = (octet) sh;       
        istrm >> msg->header.length;       
        istrm >> msg->date.tv_sec ; 
        istrm >> msg->date.tv_usec ;   
        istrm >> msg->need_swap ;
                
        msg->data = new octet[ msg->header.length + 12 ] ;    
        for(int i = 0;  i < msg->header.length + 12; i++)
        { 
          istrm >> sh ; msg->data[i] = (octet) sh ;          
        }  
                         
        istrm >> msg->request_id  ;      
        istrm >> msg->objectKey_Offset ;
        istrm >> msg->objectKey_Length ; 
        istrm >> msg->operation_Offset ;
                  
        std::string serv="", intf="";
                      
        if(req  == 1)
        {      
           while (istrm && !istrm.eof() && serv == "" ) getline(istrm, serv);  
           msg->server_id = new char[serv.length()+1];
           strncpy((char *) msg->server_id, serv.c_str() , serv.length() );  
                  
           while (istrm && !istrm.eof() && intf == "" ) getline(istrm, intf);     
           msg->intf_rid = new char[intf.length()+1];    
           strncpy((char *) msg->intf_rid, intf.c_str() , intf.length() );
        }   
    }
    catch(...)
    {
       std::cout<< "#### error on reading file "<< filename << std::endl;
       delete msg;
       return 0;   
    }
      
    msg->next = 0;
    msg->fragments = 0; 
    
    istrm.close();
    
    return msg ; 
}
 //=============================================================== 
 // do tests
 //=============================================================== 
void TestRequestPairing::do_tests()
{
    std::cout <<std::endl;
    
    PacketStore store;
    RequestPairing pairing(store);
    
    PluginType type1 = 5;
    MyPlugin * plugin1 = new MyPlugin(type1) ;
        
    test_1(& pairing, plugin1, type1);
    test_2(& pairing, plugin1);   
    test_3(& pairing, plugin1);
    
    //TODO test_4(& pairing, plugin1); // with timeout
  
    PluginType type2 = 7;
    MyPlugin * plugin2 = new MyPlugin(type2) ;
      
    test_5(& pairing, plugin1, plugin2, type2);
    test_6(& pairing, plugin1, plugin2); 
    test_7(& pairing, plugin1, plugin2);
    
    //TODO test_8(& pairing, plugin1, plugin2); // with timeout
      
    PacketStore store2;   
    RequestPairing pairing2(store2); 
   
    test_9 (& pairing2); 
    test_10(& pairing2);           
     
    test_11(& pairing2, plugin1, type1);  
    test_12(& pairing2, plugin1, type1); 
    
    test_13(); // with fragments
    test_14(); 
    test_15(); 
    test_16(); 
    test_17(); 
                   
    delete plugin1;  
    delete plugin2;
  
    std::cout<<"-------"<<std::endl;       
}

}  // namespace Recording 
}  // namespace Cdmw

