/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <stdio.h>

#include "recordinglibrary/OperationMap.hpp"
#include "recordinglibrary/ObjectMap.hpp"
#include "recordinglibrary/PacketStore.hpp"
#include "recordinglibrary/NetworkSniffer.hpp"
#include "recordinglibrary/MessageIdentifier.hpp"

#include "testrequestpairing/MyRequestPairing.hpp"

int main (int argc, char* argv[])
{
  int port = 6789; // default
    
  if(argc > 1)   port = atoi(argv[1]); 
  std::cout<< "port "<< port << std::endl; 
        
  Cdmw::Recording::PacketStore store ;
  Cdmw::Recording::NetworkSniffer sniffer (store) ;
  Cdmw::Recording::InterfaceFilter interface_filter ("etwas.txt"); 
  Cdmw::Recording::OperationMap operation_map("etwas.txt");
  Cdmw::Recording::ObjectMap object_map(interface_filter, sniffer);
  
  Cdmw::Recording::NetworkSniffer::Status status = sniffer.addPort (port) ;
  status = sniffer.run () ;
  status = sniffer.start () ;
  
  Cdmw::Recording::MyRequestPairing mpairing(store);
   
  Cdmw::Recording::MessageIdentifier msgIdentifier
      (store, object_map, interface_filter, operation_map, mpairing) ;
      
  Cdmw::Recording::MessageIdentifier::Status msg_status;
  msg_status = msgIdentifier.run () ;
  msg_status = msgIdentifier.start () ;
     
  cout << "message identifier started (enter .. to stop)" << endl;
  cin.get ();
  
  msg_status = msgIdentifier.stop () ;
  msg_status = msgIdentifier.terminate () ;
  
  status = sniffer.stop () ;
  status = sniffer.terminate () ;
  
  store.finalize () ; 
  cout << "ok" << endl;
}
