 

/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef CDMW_RECORDING_COMMAND_HPP
#define CDMW_RECORDING_COMMAND_HPP

#include <Foundation/osthreads/Mutex.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <Foundation/osthreads/Condition.hpp>
#include <TraceAndPerf/recordingtools/InterfaceMap.hpp>
#include <TraceAndPerf/recordingtools/Decoder.hpp>
#include <TraceAndPerf/recordingtools/Display.hpp>

namespace Cdmw
{
  namespace Recording
  {
    const unsigned short SUCCESS = 0;
    const unsigned short FAILURE = 1;
    const unsigned short EXIT = 2;   
    /**
     * This structure fully describes the options of the Decoder
     * daemon 
     */
    struct DebuggerOptions
    {
      std::string ifr_url;
      bool interactive;
      unsigned int flags;
    };
    /**
     * Get the options
     *
     * @param argc nbr of arguments
     * @param argv ptr to argument table
     *
     * @param options the returned option data
     * @param new_argc nbr of rebuilt arguments
     * @param new_argv ptr to rebuilt argument table
     *
     * @return SUCCESS or FAILURE
     */
     int get_debugger_options (int argc, char* argv[],
                             Cdmw::Recording::DebuggerOptions &options);
    /**
     *Purpose: Display the debug files produced by the daemon
     *
     *<p> .
     *
     */
    class Command
    {
      public :

           Command(unsigned int flags, const char * ifr_url)
                  throw() ;

            virtual ~Command()
                  throw() ;

            /**
             *Purpose: Do the requested decoding
             *<p> .
             *
             */
            virtual void Decode (bool interactive)
                  throw() ;
         private :

            /**
             *Purpose: Handle the operator or gui commands
             * A "false" result indicates that exit is requested
             *<p> .
             *
             */
            virtual bool nextCommand ()
                  throw() ;

            virtual void Set_Input(const char * name) ;
            virtual bool Set_Output(const char * name) ;
            virtual void Set_Flags(unsigned int flags) ;
            virtual void Stop() ;
            virtual void Quit() ;

            InterfaceMap  m_ifr_store ;
            Decoder       m_decoder ;
            Display       m_display ;

            class Worker ;

            Worker * m_worker ;
            char * m_input_file ;
            char * m_output_file ;
            char * m_prompt ;
            unsigned int m_flags ;
            bool m_run ;
            bool m_terminated ;

            OsSupport::Mutex m_mutex ;
            OsSupport::Condition m_control ;
             bool m_error_already_displayed ;
      } ;
   }
}

#endif
