/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef CDMW_RECORDING_DECODER_HPP
#define CDMW_RECORDING_DECODER_HPP

#include <stdio.h>
#include <Foundation/orbsupport/CORBA.hpp>
#include <TraceAndPerf/recordinglibrary/PacketStore.hpp>

namespace Cdmw
{
   namespace Recording
   {
      /**
       *Purpose: Provide the algorithms to recursively display the
       * operation parameters
       *<p> .
       *
       */
      class Decoder
      {
         public :
            Decoder()
                  throw() ;

            virtual ~Decoder()
                  throw() ;

            /**
             *Purpose: Display next parameter, using given typecode
             *<p> .
             *
             */
            virtual void Decode_Parameter (const octet * item, int &start, CORBA::TypeCode * tc, bool swap)
                  throw() ;

            /**
             *Purpose: Display the returned system exception
             *<p> .
             *
             */
            void Decode_System_Exception (const octet * item, int & start, bool swap)
                  throw() ;

            /**
             *Purpose: Configure the display of parameters
             *<p> .
             *
             */
            virtual void SetConfiguration (unsigned int flags)
                  throw() ;

            /**
             *Purpose: File where to do the output
             *<p> .
             *
             */
            virtual void SetOutput (FILE * file)
                  throw() ;

            // Endian checking
            static const short ENDIAN_CHECKER ;
            static const octet NATIVE_ENDIAN ;

            inline bool needSwap (const octet endian) { return (endian != NATIVE_ENDIAN); }

         private :
            unsigned int m_flags ;

            // Output file
            FILE * m_file ;

            // Valuetype chunking needed
            bool m_chunked ;

            // Valuetype chunk end
            int m_chunk_end ;

            // Valuetype level
            int m_value_level ;

            // Byte swapping functions
            virtual void Swap_2 (octet * Data, int Start) ;
            virtual void Swap_4 (octet * Data, int Start) ;
            virtual void Swap_8 (octet * Data, int Start) ;
            virtual void Swap_16 (octet * Data, int Start) ;

            // Indentation routine
            virtual void Print_Indentation (int indent) ;

            // Type display
            virtual const char * Type_Id (const octet * Data, int Start) ;
            virtual void Type_String (CORBA::TypeCode * Tc, char * buffer) ;
            virtual void Type_String (const octet * tc, int index, char * buffer, bool swap) ;

            // Skipping functions
            virtual void Skip_String (const octet * Data, int & Start, bool swap) ;
            virtual void Skip_Type_Code (const octet * code, int & index, bool swap) ;
            virtual const octet * Read_Typecode (const octet * code, int & index, bool swap) ;

            // Display functions
            virtual void Print_Any (const octet * item, int & start, bool swap, int indent) ;
            virtual void Print_Typecode (const octet * item, int & start, bool swap) ;
            virtual void Print_Principal (const octet * item, int & start, bool swap) ;
            virtual void Print_Object (const octet * item, int & start, bool swap, int indent) ;
            virtual void Print_String (const octet * item, int & start, bool swap) ;
            virtual void Print_Wide_String (const octet * item, int & start, bool swap) ;
            virtual void Print_Enum (const octet * item, int & start, bool swap, const octet * code) ;
            virtual void Print_Alias (const octet * item, int & start, bool swap,
                                      const octet * code, int & index, int indent) ;
            virtual void Print_Struct (const octet * item, int & start, bool swap,
                                       const octet * code, int & index, int indent) ;
            virtual void Print_Value (const octet * item, int & start, bool swap,
                                      const octet * code, int & index, int indent) ;
            virtual void Print_Value_Box (const octet * item, int & start, bool swap,
                                          const octet * code, int & index, int indent) ;
            virtual void Print_Exception (const octet * item, int & start, bool swap,
                                          const octet * code, int & index, int indent) ;
            virtual void Print_Union (const octet * item, int & start, bool swap,
                                      const octet * code, int & index, int indent) ;
            virtual void Print_Array (const octet * item, int & start, bool swap,
                                      const octet * code, int & index, int indent) ;
            virtual void Print_Sequence (const octet * item, int & start, bool swap,
                                         const octet * code, int & index, int indent) ;
            virtual void Print_Fixed (const octet * item, int & start, bool swap,
                                      const octet * code, int & index, int indent) ;
            virtual void Print_Abstract (const octet * item, int & start, bool swap,
                                         const octet * code, int & index, int indent) ;
            virtual void Print_Next_Element (const octet * item, int & start, bool swap,
                                             const octet * tc, bool code_swap, int indent = 1) ;
            virtual void Print_Value_Members (const octet * item, int & start, bool swap,
                                              const octet * code, int & index, int indent) ;
            virtual const octet * handle_value_header (const octet * item, int & start, bool swap,
                                                       const octet * code, int & index, int indent) ;
            virtual void handle_chunk_end (const octet * item, int & start, bool swap, int value_level) throw () ;
            virtual int Set_Union_Element_Start (const octet * item, int & start, bool swap,
                                                 const octet * code, int & index, bool code_swap,
                                                 int discrim_size) ;
      } ;
   }
}
#endif
