/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef CDMW_RECORDING_DISPLAY_HPP
#define CDMW_RECORDING_DISPLAY_HPP

#include <TraceAndPerf/recordingtools/InterfaceMap.hpp>
#include <TraceAndPerf/recordingtools/Decoder.hpp>

namespace Cdmw
{
   namespace Recording
   {
      /**
       *Purpose: Display the debug files produced by the daemon
       *         Display class is not thread-safe
       *         (it is up to the user to provide protection for concurrent updates)
       *
       *<p> .
       *
       */
      class Display
      {
         public :
            Display(InterfaceMap & ifr_store,
                    Decoder & decoder)
                  throw() ;

            virtual ~Display()
                  throw() ;

            /**
             *Purpose: Display the full contents of the given file
             *<p> .
             *
             */
            virtual bool Display_File (const char * file_name)
                  throw() ;

            /**
             *Purpose: Control the details of the display
             *<p> .
             *
             */
            virtual void Set_Flags (unsigned int flags)
                  throw() ;

            // Defined flags
            /**
             *Purpose: Indicates that the output is limited to headers only
             *<p> .
             *
             */
            static const unsigned int M_Headers_Only ;

            /**
             *Purpose: Indicates that the reading must continue even if
             * the end of the file is reached, in order to display
             * additionnal operations to come.
             *<p> .
             *
             */
            static const unsigned int M_For_Ever_Reading ;

            /**
             *Purpose: File where to do the output
             *<p> .
             *
             */
            virtual bool Set_Output (const char * file)
                  throw() ;

            /**
             *Purpose: Stop the display processing
             *<p> .
             *
             */
            virtual bool Is_Stopped ()
                throw() ;

            /**
             *Purpose: Stop the display processing
             *<p> .
             *
             */
            virtual void Stop_Display ()
                  throw() ;

         private :
            /**
             *Purpose:  Display the header of a reply message
             *<p> .
             *
             */
            virtual int Display_Reply_Info (Message & reply)
                  throw() ;

            /**
             *Purpose: Display the header of the operation
             *<p> .
             *
             */
            virtual int Display_Operation_Info (Message & op)
                  throw() ;

            /**
             *Purpose:  Display a LocateRequest message
             *<p> .
             *
             */
            virtual void Display_Locate_Request (Message & request)
                throw() ;

            /**
             *Purpose:  Display a LocateReply message
             *<p> .
             *
             */
            virtual void Display_Locate_Reply (Message & reply)
                throw() ;

            /**
             *Purpose: Display the next operation from the file
             * It returns false if no more operations are available and the
             * flag For_Ever_Reading is not set, or the reading has
             * failed
             *<p> .
             *
             */
            virtual bool Display_Next_Operation ()
                  throw() ;

            /**
             *Purpose: Operation description cache
             *<p> .
             *
             */
            InterfaceMap & m_ifr_map ;

            /**
             *Purpose: Display workhorse for parameters
             *<p> .
             *
             */
            Decoder & m_decoder ;

            char * m_in_file_name ;
            FILE * m_in_file ;
            char * m_out_file_name ;
            FILE * m_out_file ;

            unsigned int m_flags ;

            int m_buffer_length ;
            octet * m_buffer ;

            bool m_stopped ;
      } ;
   }
}

#endif
