/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef CDMW_RECORDING_INTERFACEMAP_HPP
#define CDMW_RECORDING_INTERFACEMAP_HPP

#ifdef __GNUC__
#include <ext/hash_map>
#else
#include <map>
#endif

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/IFR.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>

namespace Cdmw
{
   namespace Recording
   {
      /**
       *Purpose: Cache for the interface repository 
       *<p> .
       *
       */
      class InterfaceMap
      {
         public :
            InterfaceMap(const char * ifr_url)
                  throw() ;

            virtual ~InterfaceMap()
                  throw() ;

            /**
             *Purpose: Get the operation description from the cache
             * querying the Interface Repository if needed
             *<p> .
             *
             */
            virtual CORBA::OperationDescription * getOperationDef(const char * interface, const char * operation)
                  throw() ;

            /**
             *Purpose: return Headers_Only mode if the interface repository is bad
             *<p> .
             *
             */
            virtual bool Headers_Only();
            
         private :
            CORBA::ORB_var m_orb ;
            CORBA::Repository_var m_ifr ;

            struct Operation_Name
            {
//               const char* interface ;
//               const char * operation ;
               CORBA::String_var interface ;
               CORBA::String_var operation ;

               std::size_t hash () const
               {
                  std::size_t h = 0;
                  const char *iface = interface;
                  for (int each = 0; iface[each] != 0; each++)
                  {
                     h = 5*h + iface [each];
                  }
                  const char *oper = operation;
                  for (int each = 0; oper [each] != 0; each++)
                  {
                     h = 5*h + oper [each];
                  }
                  return h;
               }
            } ;

#ifdef __GNUC__
            // For performance reasons, it is highly preferable to use
            // a hash_map instead of a map. As it may not be available
            // on all environments, a map alternative is provided.
            struct eqop
            {
               bool operator()(Operation_Name op1, Operation_Name op2) const
               {
                  return (strcmp(op1.interface, op2.interface) == 0) &&
                         (strcmp(op1.operation, op2.operation) == 0) ;
               }
            } ;

            struct hash
            {
               std::size_t operator()(const Operation_Name name) const
                     { return name.hash(); }
            };

            typedef __gnu_cxx::hash_map <Operation_Name, CORBA::OperationDescription *, hash, eqop> OperationMap ;

#else
#warning "hash_map may-be not available, check STL extensions"
            struct ltop
            {
               bool operator()(Operation_Name op1, Operation_Name op2) const
               {
                  int intf_lower = strcmp(op1.interface, op2.interface) ;
                  return (intf_lower < 0) ||
                         ((intf_lower == 0) && (strcmp(op1.operation, op2.operation) < 0));
               }
            };

            typedef std::map <Operation_Name, CORBA::OperationDescription *, ltop> OperationMap ;
#endif

            // This map is indexed by the couple interface-operation 
            OperationMap m_operations ;
      } ;
   }
}

#endif

