/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
#ifndef _IDL_CDMWRECORDINGDAEMONCONTROL_IMPL_H_
#define _IDL_CDMWRECORDINGDAEMONCONTROL_IMPL_H_

#include <CdmwRecordingDaemonControl.skel.hpp>

#include <CdmwRecordingDaemonControl.skel.hpp>
#include <Foundation/idllib/CdmwRecordingContainerManager.skel.hpp>
#include <recordingtools/CdmwRecordingContainerManager_impl.hpp>

#include "TraceAndPerf/recordinglibrary/OperationMap.hpp"
#include "TraceAndPerf/recordinglibrary/ObjectMap.hpp"
#include "TraceAndPerf/recordinglibrary/PacketStore.hpp"
#include "TraceAndPerf/recordinglibrary/NetworkSniffer.hpp"
#include "TraceAndPerf/recordinglibrary/MessageIdentifier.hpp"
#include "TraceAndPerf/recordinglibrary/RecordingPlugin.hpp"
#include "TraceAndPerf/recordinglibrary/DebugPlugin.hpp"

// ----------------------------------------------------------------------
// MODULE IDL:thalesgroup.com/CdmwRecording:1.0
namespace CdmwRecording {

  // -------------------------------------------------------------------
  // Servant Impl class for interface CdmwRecording::DaemonControl
  class DaemonControl_impl 
    : public virtual POA_CdmwRecording::DaemonControl
    , public virtual PortableServer::RefCountServantBase
  {
    public:
      DaemonControl_impl (CORBA::ORB_ptr orb,
                          PortableServer::POA_ptr parent,
                          std::string interfaces_file,
                          std::string operations_file) 
        throw(CORBA::SystemException);

      // -----------------------------
      // Methods of this interface 
      // -----------------------------
      // SYNCHRONOUS CALLS
      virtual Running_State start() throw(CORBA::SystemException);
      virtual Running_State stop() throw(CORBA::SystemException);
      virtual void terminate() throw(CORBA::SystemException);
      virtual void add_interfaces
        (const String_List& to_record) throw(CORBA::SystemException);
      virtual void remove_interfaces
        (const String_List& to_ignore) throw(CORBA::SystemException);
      virtual void remove_all_interfaces() throw(CORBA::SystemException);
      virtual void add_operations
        (const String_List& to_record) throw(CORBA::SystemException);
      virtual void remove_operations
          (const String_List& to_ignore) throw(CORBA::SystemException);
      virtual void remove_all_operations() throw(CORBA::SystemException);
      virtual void CdmwRecording::DaemonControl_impl::add_port
          (CORBA::UShort) throw(CORBA::SystemException);
      virtual void CdmwRecording::DaemonControl_impl::remove_port
          (CORBA::UShort) throw(CORBA::SystemException);

      // set the max size in bytes of the buffer to be sent
      virtual void CdmwRecording::DaemonControl_impl::setMaxSize(CORBA::ULong max_size) throw(CORBA::SystemException);

      // set the timeout (in ms) of the sending mechanism
      virtual void CdmwRecording::DaemonControl_impl::setTimeout(CORBA::ULong time_out ) throw(CORBA::SystemException);

      // set the polling period (in ms) of the Recording Server
      virtual void CdmwRecording::DaemonControl_impl::setRecorderRetryDelay(CORBA::ULong retry_period) throw(CORBA::SystemException);

      // set the max buffer size : when this max number is
      // exceeded, the older buffers will be discarded
      virtual void CdmwRecording::DaemonControl_impl::setMaxBuffer(CORBA::ULong max_buffer) throw(CORBA::SystemException);         
      // set the max retry number
      virtual void CdmwRecording::DaemonControl_impl::setRetryNumber(CORBA::Long number) throw(CORBA::SystemException);
      
      virtual char * recorder_url() throw(CORBA::SystemException);
      virtual void recorder_url
        (const char *  attr_recorder_url) throw(CORBA::SystemException);
      
      virtual void setRecorderUrl
        (CORBA::ULong which, const char *  attr_recorder_url) throw(CORBA::SystemException);
      virtual char * debug_file() throw(CORBA::SystemException);
      virtual void debug_file
        (const char *  attr_debug_file) throw(CORBA::SystemException);
      virtual CdmwRecording::ContainerManager_impl* get_ContainerManager 
          () throw();
      virtual CdmwRecording::State_Data* state 
          () throw();
 
      // ----------------------------------
      // Attributes generated as members
      // ----------------------------------
      // Attributes of this interface 
      //std::string  __recorder_url;
      //std::string  __debug_file;

            /**
             *Purpose: Set the Corba Relative Roundtrip Timeout
             *<p> .
             *
             */
            virtual void SetCorbaTimeout (unsigned long timeout);
                  
private:
        /*void get_active_interface 
        (std::string const & key_str, Cdmw::Recording::InterfaceFilter::flag_t const & flag);
        void get_children (std::string const & str1, std::string const & str2, size_t i);
        void get_active_operation (std::string const & str1, size_t i);*/
    /**
    * ORB reference
    */ 
    CORBA::ORB_var m_orb;

    /**
    * POA reference
    */ 
    PortableServer::POA_var m_poa;

    /**
     * Recording objects
     */
    
    char*  m_recorder_url[REC_DAEMON_MAX_RECORDERS];
    char*  m_debug_file;
    unsigned long m_corba_timeout;
   
    // InterfaceFilter
    Cdmw::Recording::InterfaceFilter m_interface_filter;

    // OperationMap
    Cdmw::Recording::OperationMap m_operation_map;
    
    // PacketStore for sniffer
    Cdmw::Recording::PacketStore m_store;

    // NetworkSnifCdmwfer
    Cdmw::Recording::NetworkSniffer m_sniffer;

    // ObjectMap
    Cdmw::Recording::ObjectMap m_object_map;
    
    // RequestPairing
    Cdmw::Recording::RequestPairing m_pairing;
    
    // Message Identifier
    Cdmw::Recording::MessageIdentifier m_msg_identifier;

    // ContainerManager
    //CdmwRecording::ContainerManager_var m_container_manager;

    CdmwRecording::ContainerManager_impl* m_container_manager_impl;

    // Recording PlugIn
    Cdmw::Recording::RecordingPlugin * m_recording_plugin[REC_DAEMON_MAX_RECORDERS];

    // Debug PlugIn
    Cdmw::Recording::DebugPlugin * m_debug_plugin;

    // State data
   CdmwRecording::State_Data* m_state_data;
    
   // CDO-070330-PRS-1-567 : #97
   bool m_isInitialised;
   bool m_isStarted;

   // CDO-070330-PRS-1-567 : #99, #100, #103
   Cdmw::OsSupport::Mutex m_mutex ;
public :
   static CdmwRecording::String_List m_updated_intf_list;
   static int m_nb_active_intf;
   static CdmwRecording::String_List m_updated_operation_list;
   static int m_nb_active_operation;
   static CdmwRecording::Port_List m_ports_list;
  
  } ; // end class CdmwRecording::DaemonControl_impl
}
// MODULE IDL:thalesgroup.com/CdmwRecording:1.0
// ----------------------------------------------------------------------
#endif
