/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
#include <recordingtools/CdmwRecordingContainerManager_impl.hpp>
#include "TraceAndPerf/recordinglibrary/ObjectMap.hpp"

// ----------------------------------------------------------------------
// Service code f o r - MODULE IDL:thalesgroup.com/CdmwRecording:1.0
namespace CdmwRecording {

ContainerManager_impl::ContainerManager_impl (CORBA::ORB_ptr orb,
                                              PortableServer::POA_ptr parent,
                                              Cdmw::Recording::ObjectMap & object_map
                                             ) 
    throw(CORBA::SystemException)
    : m_orb(CORBA::ORB::_duplicate(orb)),
      m_poa(PortableServer::POA::_duplicate(parent)),
      m_object_map(object_map),
      m_mutex()

{
}
  // ----------------------------------------
  // Methods of interface ContainerManager
  // ----------------------------------------
  //
  // ContainerManager SYNCHRONOUS CALLS
  //
  void CdmwRecording::ContainerManager_impl::record_objects
    (const char *  server_id,
     const Object_List& to_record) throw(CORBA::SystemException)
  {
    CDMW_MUTEX_GUARD(m_mutex) ;
    for (unsigned each = 0; each < to_record.length(); each++)
    {
      CORBA::Object_var ref =  CORBA::Object::_duplicate(to_record[each]); 
      m_object_map.addObject(server_id, m_orb.in(), ref);
    }
  } // end record_objects

  void CdmwRecording::ContainerManager_impl::discard_objects
    (const Object_List& to_discard) throw(CORBA::SystemException)
  {
    CDMW_MUTEX_GUARD(m_mutex) ;
    for (unsigned each = 0; each < to_discard.length(); each++)
    {
      CORBA::Object_var ref =  CORBA::Object::_duplicate(to_discard[each]); 
      m_object_map.removeObject(m_orb.in(), ref);
    }
  } // end discard_objects
  
}
// E n d Service code f o r - MODULE IDL:thalesgroup.com/CdmwRecording:1.0
// ----------------------------------------------------------------------

