/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
#include "TraceAndPerf/recordinglibrary/Recording_Default_Parameters.hpp"
#include <recordingtools/CdmwRecordingDaemonControl_impl.hpp>
#include "TraceAndPerf/recordinglibrary/DebugPlugin.hpp"
#include "TraceAndPerf/recordinglibrary/RecordingPlugin.hpp"
#include "TraceAndPerf/recordinglibrary/NetworkSniffer.hpp"

// ----------------------------------------------------------------------
// Service code f o r - MODULE IDL:thalesgroup.com/CdmwRecording:1.0
namespace CdmwRecording {

// -------------------------------------
// Methods of interface DaemonControl
// -------------------------------------
//
DaemonControl_impl::DaemonControl_impl
   (CORBA::ORB_ptr orb,
    PortableServer::POA_ptr parent,
    std::string interfaces_file,
    std::string operations_file
   ) 
   throw(CORBA::SystemException)
   : m_orb(CORBA::ORB::_duplicate(orb))
   , m_poa(PortableServer::POA::_duplicate(parent))
   //, m_recorder_url("")
   , m_debug_file("")
   , m_interface_filter(interfaces_file.c_str())
   , m_operation_map(operations_file.c_str())
   , m_store()
   , m_sniffer(m_store)
   , m_object_map(m_interface_filter, m_sniffer)
   , m_pairing(m_store)
   , m_msg_identifier(m_store, m_object_map, m_interface_filter, m_operation_map, m_pairing)
   , m_container_manager_impl(0)
   //, m_recording_plugin(0)
   , m_debug_plugin(0)
   , m_state_data(0)
   // CDO-070330-PRS-1-567 : #97
   , m_isInitialised (false)
   , m_isStarted (false)
   // CDO-070330-PRS-1-567 : #99, #100, #103
   , m_mutex()
 {
    for (unsigned int i = 0; i < REC_DAEMON_MAX_RECORDERS ; i++)
    {
       m_recorder_url[i]=0;
       m_recording_plugin[i]=0;
    }
 }
// DaemonControl SYNCHRONOUS CALLS
//
Running_State CdmwRecording::DaemonControl_impl::start() throw(CORBA::SystemException)
{
  CDMW_MUTEX_GUARD(m_mutex) ;
  if (!m_isInitialised)
  {
    m_isInitialised = true;
    // Create a Consumer Implementation object
    m_container_manager_impl = new CdmwRecording::ContainerManager_impl (m_orb, m_poa, m_object_map);
    m_state_data = new CdmwRecording::State_Data;
    //CORBA::Object_var obj = m_poa->servant_to_reference (m_container_manager_impl) ;
    //m_container_manager = CdmwRecording::ContainerManager::_narrow(obj.in());
    if (m_sniffer.run() != Cdmw::Recording::NetworkSniffer::OK)
      {
          m_state_data->State = Stopped;
          return Stopped;
      };
    m_msg_identifier.run();
    m_msg_identifier.start();
  }
  if (!m_isStarted) 
  {
    m_sniffer.start();
  }
  m_isStarted = true;
  m_state_data->State = Started;
  return Started;
} // end start

Running_State CdmwRecording::DaemonControl_impl::stop() throw(CORBA::SystemException)
{
  CDMW_MUTEX_GUARD(m_mutex) ;
  if (m_isStarted) 
  { 
    m_sniffer.stop();
  }
  m_isStarted = false;
  m_state_data->State = Stopped;
  return Stopped;
} // end stop

void CdmwRecording::DaemonControl_impl::terminate() throw(CORBA::SystemException)
{
  CDMW_MUTEX_GUARD(m_mutex) ;
  m_sniffer.stop();
  m_sniffer.terminate();
  m_store.finalize () ;
  // m_state_data->State = Terminated;
  m_orb->shutdown(false);
} // end terminate

void CdmwRecording::DaemonControl_impl::add_interfaces
   (const String_List& to_record) throw(CORBA::SystemException)
{
  CDMW_MUTEX_GUARD(m_mutex) ;
  String_List& List = const_cast <String_List&>(to_record);
  m_interface_filter.addInterfaces(List);
} // end add_interfaces

void CdmwRecording::DaemonControl_impl::remove_interfaces
   (const String_List& to_ignore) throw(CORBA::SystemException)
{
  CDMW_MUTEX_GUARD(m_mutex) ;
  String_List& List = const_cast <String_List&>(to_ignore);
  m_interface_filter.removeInterfaces(List);
} // end remove_interfaces

void CdmwRecording::DaemonControl_impl::remove_all_interfaces() throw(CORBA::SystemException)
{
  CDMW_MUTEX_GUARD(m_mutex) ;
  m_interface_filter.removeAllInterfaces();
} // end remove_all_interfaces

void CdmwRecording::DaemonControl_impl::add_operations
   (const String_List& to_record) throw(CORBA::SystemException)
{
  CDMW_MUTEX_GUARD(m_mutex) ;
  String_List& List = const_cast <String_List&>(to_record);
  m_operation_map.addOperations (List);
} // end add_operations

void CdmwRecording::DaemonControl_impl::remove_operations
   (const String_List& to_ignore) throw(CORBA::SystemException)
{
  CDMW_MUTEX_GUARD(m_mutex) ;
  String_List& List = const_cast <String_List&>(to_ignore);
  m_operation_map.removeOperations (List);
} // end remove_operations

void CdmwRecording::DaemonControl_impl::add_port
   (CORBA::UShort port) throw(CORBA::SystemException)
{
  CDMW_MUTEX_GUARD(m_mutex) ;
  m_sniffer.addPort (port);
} // end add_operations

void CdmwRecording::DaemonControl_impl::remove_port
   (CORBA::UShort port) throw(CORBA::SystemException)
{
  CDMW_MUTEX_GUARD(m_mutex) ;
  m_sniffer.removePort (port);
} // end remove_operations

void CdmwRecording::DaemonControl_impl::remove_all_operations() throw(CORBA::SystemException)
{
  CDMW_MUTEX_GUARD(m_mutex) ;
  m_operation_map.removeAllOperations();
} // end remove_all_operations

char * CdmwRecording::DaemonControl_impl::recorder_url() throw(CORBA::SystemException)
{
  CDMW_MUTEX_GUARD(m_mutex) ;
  // CDO-070330-PRS-1-567 : #101
  return CORBA::string_dup(m_recorder_url[0]);
} // end recorder_url

void CdmwRecording::DaemonControl_impl::recorder_url
   (const char *  attr_recorder_url) throw(CORBA::SystemException)
{
    setRecorderUrl(0, attr_recorder_url);
}

void CdmwRecording::DaemonControl_impl::SetCorbaTimeout (unsigned long timeout)
{
  CDMW_MUTEX_GUARD(m_mutex) ;
  m_corba_timeout = timeout;
}

void CdmwRecording::DaemonControl_impl::setRecorderUrl
   (CORBA::ULong which, const char *  attr_recorder_url) throw(CORBA::SystemException)
{
  CDMW_MUTEX_GUARD(m_mutex) ;
  m_recorder_url[which] = CORBA::string_dup (attr_recorder_url);
  // CDO-070330-PRS-1-567 : #102
  m_state_data->recorders_urls[which] = CORBA::string_dup(m_recorder_url[which]);
  if (m_recording_plugin[which])
  {
    //m_pairing.setDestination(Cdmw::Recording::RECORDING_PLUGIN_TYPE, m_recorder_url[which]);
      m_recording_plugin[which]->Destination(m_recorder_url[which]);
    return;
  }

  //Create plugin
  m_recording_plugin[which] = new Cdmw::Recording::RecordingPlugin();
  m_recording_plugin[which]->SetCorbaTimeout (m_corba_timeout);
  m_recording_plugin[which]->initialize(m_orb);

  // initialize state_data
  m_state_data->max_size = (unsigned long) m_recording_plugin[which]->MaxSize();
  m_state_data->time_out = (unsigned long) m_recording_plugin[which]->TimeOut();
  m_state_data->retry_period = (unsigned long) m_recording_plugin[which]->GetRetryPeriod();
  m_state_data->max_buffer = (unsigned long)  m_recording_plugin[which]->GetMaxBufferNumber();

  //Add plugin to RequestPairing
  m_pairing.addPlugin(m_recording_plugin[which]);
  //m_pairing.setDestination(Cdmw::Recording::RECORDING_PLUGIN_TYPE, m_recorder_url);
  m_recording_plugin[which]->Destination(m_recorder_url[which]);
} // end recorder_url

char * CdmwRecording::DaemonControl_impl::debug_file() throw(CORBA::SystemException)
{
  CDMW_MUTEX_GUARD(m_mutex) ;
  return CORBA::string_dup(m_debug_file);
} // end debug_file

void CdmwRecording::DaemonControl_impl::debug_file
   (const char *  attr_debug_file) throw(CORBA::SystemException)
{
  CDMW_MUTEX_GUARD(m_mutex) ;
  m_debug_file = CORBA::string_dup(attr_debug_file);
  // CDO-070330-PRS-1-567 : #102
  m_state_data->debug_file = CORBA::string_dup(m_debug_file);

  if (m_debug_plugin)
  {
    m_pairing.setDestination(Cdmw::Recording::DEBUG_PLUGIN_TYPE, m_debug_file);
    return;
  }
  
  // create plugin if not yet done
  m_debug_plugin = new Cdmw::Recording::DebugPlugin();
  m_debug_plugin->initialize(m_orb);
  //Add plugin to RequestPairing
  m_pairing.addPlugin(m_debug_plugin);
  m_pairing.setDestination(Cdmw::Recording::DEBUG_PLUGIN_TYPE, m_debug_file);
} // end debug_file

CdmwRecording::ContainerManager_impl* CdmwRecording::DaemonControl_impl::get_ContainerManager 
   () throw()
{
  m_container_manager_impl->_add_ref();
  return m_container_manager_impl;
}

// set the max size in bytes of the buffer to be sent
void CdmwRecording::DaemonControl_impl::setMaxSize(CORBA::ULong max_size) throw(CORBA::SystemException)

{
  CDMW_MUTEX_GUARD(m_mutex) ;
   for ( unsigned int i = 0; i < REC_DAEMON_MAX_RECORDERS; i ++) 
   {
      if (m_recording_plugin[i])
      {
         m_recording_plugin[i]->MaxSize((size_t)max_size);
      }
      m_state_data->max_size = max_size;
   }
} 

// set the timeout (in ms) of the sending mechanism
void CdmwRecording::DaemonControl_impl::setTimeout(CORBA::ULong time_out ) throw(CORBA::SystemException)
{
  CDMW_MUTEX_GUARD(m_mutex) ;
  for ( unsigned int i = 0; i < REC_DAEMON_MAX_RECORDERS; i ++) 
     {
        if (m_recording_plugin[i])
        {
           m_recording_plugin[i]->TimeOut((int) time_out);
        }
      m_state_data->time_out = time_out;
   }
}

// set the polling period (in ms) of the Recording Server
void CdmwRecording::DaemonControl_impl::setRecorderRetryDelay(CORBA::ULong retry_period) throw(CORBA::SystemException)
{
  CDMW_MUTEX_GUARD(m_mutex) ;
  for ( unsigned int i = 0; i < REC_DAEMON_MAX_RECORDERS; i ++) 
     {
        if (m_recording_plugin[i])
        {
           m_recording_plugin[i]->SetRetryPeriod((int) retry_period);
        }
      m_state_data->retry_period = retry_period;
   }
} 

// set the max buffer size : when this max number is
// exceeded, the older buffers will be discarded
void CdmwRecording::DaemonControl_impl::setMaxBuffer(CORBA::ULong max_buffer) throw(CORBA::SystemException)       
{
  CDMW_MUTEX_GUARD(m_mutex) ;
  for ( unsigned int i = 0; i < REC_DAEMON_MAX_RECORDERS; i ++) 
     {
        if (m_recording_plugin[i])
        {
           m_recording_plugin[i]->SetMaxBufferNumber((size_t)max_buffer);
        }
      m_state_data->max_buffer = max_buffer;
   }
}

// set the max retry number 
void CdmwRecording::DaemonControl_impl::setRetryNumber(CORBA::Long number) throw(CORBA::SystemException)       
{
  CDMW_MUTEX_GUARD(m_mutex) ;
  for ( unsigned int i = 0; i < REC_DAEMON_MAX_RECORDERS; i ++) 
     {
        if (m_recording_plugin[i])
        {
           m_recording_plugin[i]->SetRetryNumber(number);
        }
      m_state_data->retry_number = number;
   }
}
//-----------------------------------------------------------------------
CdmwRecording::String_List Null_String_List;
CdmwRecording::String_List CdmwRecording::DaemonControl_impl::m_updated_intf_list 
= Null_String_List;
int CdmwRecording::DaemonControl_impl::m_nb_active_intf = 0;
CdmwRecording::String_List CdmwRecording::DaemonControl_impl::m_updated_operation_list 
= Null_String_List;
int CdmwRecording::DaemonControl_impl::m_nb_active_operation = 0;
CdmwRecording::Port_List Null_Port_List;
CdmwRecording::Port_List CdmwRecording::DaemonControl_impl::m_ports_list = Null_Port_List;

// internal functions used to create list of interfaces
void get_active_interface (string const & key_str, Cdmw::Recording::InterfaceFilter::Flag_t const & flag)
{
  if (flag.counter > 0)
  {
     CdmwRecording::DaemonControl_impl::m_updated_intf_list[CdmwRecording::DaemonControl_impl::m_nb_active_intf] = key_str.c_str();
     CdmwRecording::DaemonControl_impl::m_nb_active_intf++;
  }
}

void get_children (string const & str1, string const & str2, size_t i)
{
}

// internal functions used to create list of operations
void get_active_operation (string const & str1, size_t i)
{
   CdmwRecording::DaemonControl_impl::m_updated_operation_list[CdmwRecording::DaemonControl_impl::m_nb_active_operation] = str1.c_str();
   CdmwRecording::DaemonControl_impl::m_nb_active_operation++;
}   

// returns all the data describing the global state of the Daemon
CdmwRecording::State_Data* CdmwRecording::DaemonControl_impl::state 
   () throw()
{
  CDMW_MUTEX_GUARD(m_mutex) ;
  State_Data* result = new State_Data;
  result->State = m_state_data->State;
  result->max_size = m_state_data->max_size;
  result->time_out = m_state_data->time_out;
  result->retry_period = m_state_data->retry_period;
  result->max_buffer = m_state_data->max_buffer;
  result->retry_number = m_state_data->retry_number;
  // CDO-070330-PRS-1-567 : #104
  //result->recorder_url = CORBA::string_dup(m_state_data->recorder_url);
  for (unsigned int i=0; i < REC_DAEMON_MAX_RECORDERS; i++)
  {
      result->recorders_urls[i] = CORBA::string_dup(m_state_data->recorders_urls[i]);
  }
  result->debug_file = CORBA::string_dup(m_state_data->debug_file);

  // update list of active interfaces
  m_updated_intf_list.length(m_interface_filter.list_size()); // set length of seq
  m_nb_active_intf = 0; // updated in print_intf
  m_interface_filter.mapLoop(get_children, get_active_interface);
  m_updated_intf_list.length(m_nb_active_intf); // set length of seq to the number of active inteterfaces
  // CDO-070330-PRS-1-567 : #104
  result->interfaces = m_updated_intf_list;

  // update list of active operations
  m_updated_operation_list.length(m_operation_map.list_size()); // set length of seq
  m_nb_active_operation = 0; // updated in print_intf
  m_operation_map.mapLoop(get_active_operation);
  m_updated_operation_list.length(m_nb_active_operation);
  // CDO-070330-PRS-1-567 : #104
  result->operations = m_updated_operation_list;

  // update list of active ports
  int Ports_Num = m_sniffer.TCPPorts_Num();
  unsigned short * TCPPorts = m_sniffer.TCPPorts();
  m_ports_list.length(Ports_Num); // set length of seq
  for (int i = 0; i < Ports_Num; i++)
  {
    m_ports_list[i] = TCPPorts[i];
  }
  result->ports = m_ports_list;
  return result;
}

}
// E n d Service code f o r - MODULE IDL:thalesgroup.com/CdmwRecording:1.0
// ----------------------------------------------------------------------
