/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <fstream>

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>

#include <recordingtools/CdmwRecordingGenericRecorder_impl.hpp>


namespace CdmwRecording
{


GenericRecorder_impl::GenericRecorder_impl(
        PortableServer::POA_ptr poa,
        CORBA::ORB_ptr orb,
        Cdmw::Logging::Logger* logger)
    throw(CORBA::SystemException)
    : m_poa (PortableServer::POA::_duplicate(poa)),
      m_orb(CORBA::ORB::_duplicate(orb)),
      m_logger(logger)
{
    if (m_logger == 0)
        m_logger = Cdmw::Logging::Logger::Get_Logger("CDMW.TRC.GenericRecording");
}


GenericRecorder_impl::~GenericRecorder_impl()
    throw()
{
}

void 
GenericRecorder_impl::record_data(
        const ::CdmwRecording::NamedValueSeq & context,
        const ::CdmwRecording::NamedValueSeq & data)
     throw(CORBA::SystemException)                  
{
    //std::cout << " <##### CdmwRecording::GenericRecorder : Entering record_data() #####>" << std::endl;
    CDMW_LOG(m_logger, Cdmw::Logging::CDMW_DEBUG_3, 
             " <##### CdmwRecording::GenericRecorder : Entering record_data() #####>");
    
    if ( m_logger->is_debug_enabled() && (&context != 0) ) {
        CDMW_LOG(m_logger, Cdmw::Logging::CDMW_DEBUG_3, 
                 "CdmwRecording::GenericRecorder : Context length = " << context.length());
        std::ostringstream oss;
        oss << std::endl;
        unsigned int i = 0;
        for (i=0;i<context.length();i++) {
            oss << "   context[" << i << "] - " << context[i].name;
            const char*      value_str = "";
            CORBA::ULongLong value_ull = 0;
            CORBA::LongLong  value_ll = 0;
            CORBA::ULong     value_ul = 0;
            CORBA::Long      value_l = 0;
            CORBA::UShort    value_ush = 0;
            CORBA::Short     value_sh = 0;
            if ( context[i].value >>= value_str ) {
               oss << " = " << value_str;
            }
            else if ( context[i].value >>= value_ull ) {
               oss << " = " << value_ull;
            }
            else if ( context[i].value >>= value_ll ) {
               oss << " = " << value_ll;
            }
            else if ( context[i].value >>= value_ul ) {
               oss << " = " << value_ul;
            }
            else if ( context[i].value >>= value_l ) {
               oss << " = " << value_l;
            }
            else if ( context[i].value >>= value_ush ) {
               oss << " = " << value_ush;
            }
            else if ( context[i].value >>= value_sh ) {
               oss << " = " << value_sh;
            }
            oss << std::endl;
        }

        //std::cout << "Context informations :" << oss.str() << std::endl;
        CDMW_DEBUG(m_logger, "Context  informations:" << oss.str());
    }

    if ( m_logger->is_debug_enabled() && (&data != 0) ) {
        CDMW_LOG(m_logger, Cdmw::Logging::CDMW_DEBUG_3, 
                 "CdmwRecording::GenericRecorder : Data length = " << data.length());
        std::ostringstream oss;
        oss << std::endl;
        unsigned int i = 0;
        for (i=0;i<data.length();i++) {
            oss << "      data[" << i << "] - " << data[i].name;
            const char*      value_str = "";
            CORBA::ULongLong value_ull = 0;
            CORBA::LongLong  value_ll = 0;
            CORBA::ULong     value_ul = 0;
            CORBA::Long      value_l = 0;
            CORBA::UShort    value_ush = 0;
            CORBA::Short     value_sh = 0;
            if ( data[i].value >>= value_str ) {
               oss << " = " << value_str;
            }
            else if ( data[i].value >>= value_ull ) {
               oss << " = " << value_ull;
            }
            else if ( data[i].value >>= value_ll ) {
               oss << " = " << value_ll;
            }
            else if ( data[i].value >>= value_ul ) {
               oss << " = " << value_ul;
            }
            else if ( data[i].value >>= value_l ) {
               oss << " = " << value_l;
            }
            else if ( data[i].value >>= value_ush ) {
               oss << " = " << value_ush;
            }
            else if ( data[i].value >>= value_sh ) {
               oss << " = " << value_sh;
            }
            oss << std::endl;
        }

        //std::cout << "Data informations :" << oss.str() << std::endl;
        CDMW_DEBUG(m_logger, "Data informations:" << oss.str());
    }

}


PortableServer::POA_ptr GenericRecorder_impl::_default_POA()
{
    return PortableServer::POA::_duplicate(m_poa.in());
}
      
}; // End namespace CdmwRecording

