/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <stdio.h>

#include "TraceAndPerf/recordingtools/Decoder.hpp"

namespace
{
   const char * Indentation_Line = "                                                                                " ;
}
namespace Cdmw
{
   namespace Recording
   {
      const octet tk_indirect = 255;
      const unsigned int tag_indirect = 0xFFFFFFFF;
      const short Decoder::ENDIAN_CHECKER = 0x0001 ;
      const octet Decoder::NATIVE_ENDIAN = *(const octet *) &Decoder::ENDIAN_CHECKER ;

      Decoder::Decoder()
            throw()
            : m_flags(0)
            , m_file(stdout)
            , m_chunked(false)
            , m_chunk_end(0)
            , m_value_level(0)
      {
      }

      Decoder::~Decoder()
            throw()
      {
      }

      void Decoder::SetConfiguration (unsigned int flags)
            throw()
      {
         m_flags = flags ;
      }

      void Decoder::SetOutput (FILE * file)
            throw() 
      {
         m_file = file ;
      }

      void Decoder::Decode_Parameter (const octet * item, int & start, CORBA::TypeCode * tc, bool swap)
            throw()
      {
         // Print the detailed contents of the param, starting with its type id
         char buffer [200] ;
         Type_String (tc, buffer) ;
         fprintf (m_file, " of type %s\n", buffer) ;

         octet simple_buffer [8] ;
         const octet * tc_buffer ;

         if (tc == 0)
         {
            return;
         }         
         switch (tc->kind())
         {
           case CORBA::tk_null:
           case CORBA::tk_void:
           case CORBA::tk_short:
           case CORBA::tk_ushort:
           case CORBA::tk_long:
           case CORBA::tk_ulong:
           case CORBA::tk_float:
           case CORBA::tk_double:
           case CORBA::tk_boolean:
           case CORBA::tk_char:
           case CORBA::tk_octet:
           case CORBA::tk_any:
           case CORBA::tk_TypeCode:
           case CORBA::tk_Principal:
           case CORBA::tk_string:
           case CORBA::tk_longlong:
           case CORBA::tk_ulonglong:
           case CORBA::tk_longdouble:
           case CORBA::tk_wchar:
           case CORBA::tk_wstring:
             *(CORBA::TCKind *) simple_buffer = tc->kind() ;
             tc_buffer = (const octet *) simple_buffer ;
             break ;
           default:
             // tc->buffer_ is a TAO feature which starts 8 bytes after the
             // beginning of the encapsulation
             // tc_buffer = (const octet *) tc->buffer_ - 8 ;
             TAO_OutputCDR cdr;
             //static CORBA::Boolean res = TAO::Objref_Traits<CORBA::TypeCode>::tao_marshal (tc, cdr);
             static CORBA::Boolean res = tc->tao_marshal ( cdr, 0);
             if (res)
             {
             tc_buffer = (const octet *) cdr.buffer();
             }
             else
             {
              printf("tao_marshal failed\n");
             }
         }
         Print_Next_Element (item, start, swap, tc_buffer, false, 1) ;
      }

      void Decoder::Decode_System_Exception (const octet * item, int & start, bool swap)
            throw()
      {
         if ( item == 0)
         {
            return;
         }
         const char * str = (const char *) (item + start + 4) ;
         volatile int len = *(const int *) (item + start) ;
         if (swap) Swap_4 ((octet *) &len, 0) ;
         start += 4 + len + ((-len) & 3) ;

         unsigned int minor = *(const unsigned long *) (item + start) ;
         start += 4 ;
         unsigned int completed = *(const unsigned long *) (item + start) ;

         const char * completion ;

         if (completed == 0)
         {
            completion = "COMPLETED_YES" ;
         }
         else if (completed == 1)
         {
            completion = "COMPLETED_NO" ;
         }
         else
         {
            completion = "COMPLETED_MAYBE" ;
         }

         fprintf (m_file, "System Exception %s: minor => %x, completion => %s\n", str, minor, completion) ;
      }

      void Decoder::Swap_2 (octet * Data, int Start)
      {
         octet tmp = Data[Start] ;
         Data[Start] = Data[Start + 1] ;
         Data[Start + 1] = tmp ;
      }

      void Decoder::Swap_4 (octet * Data, int Start)
      {
         octet tmp = Data[Start] ;
         Data[Start] = Data[Start + 3] ;
         Data[Start + 3] = tmp ;
         tmp = Data[Start + 1] ;
         Data[Start + 1] = Data[Start + 2] ;
         Data[Start + 2] = tmp ;
      }

      void Decoder::Swap_8 (octet * Data, int Start)
      {
         octet tmp = Data[Start] ;
         Data[Start] = Data[Start + 7] ;
         Data[Start + 7] = tmp ;
         tmp = Data[Start + 1] ;
         Data[Start + 1] = Data[Start + 6] ;
         Data[Start + 6] = tmp ;
         tmp = Data[Start + 2] ;
         Data[Start + 2] = Data[Start + 5] ;
         Data[Start + 5] = tmp ;
         tmp = Data[Start + 3] ;
         Data[Start + 3] = Data[Start + 4] ;
         Data[Start + 4] = tmp ;
      }

      void Decoder::Swap_16 (octet * Data, int Start)
      {
         octet tmp = Data[Start] ;
         Data[Start] = Data[Start + 15] ;
         Data[Start + 15] = tmp ;
         tmp = Data[Start + 1] ;
         Data[Start + 1] = Data[Start + 14] ;
         Data[Start + 14] = tmp ;
         tmp = Data[Start + 2] ;
         Data[Start + 2] = Data[Start + 13] ;
         Data[Start + 13] = tmp ;
         tmp = Data[Start + 3] ;
         Data[Start + 3] = Data[Start + 12] ;
         Data[Start + 12] = tmp ;
         tmp = Data[Start + 4] ;
         Data[Start + 4] = Data[Start + 11] ;
         Data[Start + 11] = tmp ;
         tmp = Data[Start + 5] ;
         Data[Start + 5] = Data[Start + 10] ;
         Data[Start + 10] = tmp ;
         tmp = Data[Start + 6] ;
         Data[Start + 6] = Data[Start + 9] ;
         Data[Start + 9] = tmp ;
         tmp = Data[Start + 7] ;
         Data[Start + 7] = Data[Start + 8] ;
         Data[Start + 8] = tmp ;
      }

      void Decoder::Skip_String (const octet * Data, int & Start, bool swap)
      {
         Start += (-Start) & 3 ;
         volatile int len = *(const int *) (Data + Start) ;
         if (swap) Swap_4 ((octet *) &len, 0) ;
         Start += 4 + len ;
      }

      const char * Decoder::Type_Id (const octet * Data, int Start)
      {
         Start += (-Start) & 3 ;
         return (const char *) (Data + Start + 4) ;
      }

      void Decoder::Type_String (CORBA::TypeCode * Tc, char * buffer)
      {
         if (Tc == 0)
         {
            return;
         }
         switch (Tc->kind())
         {
            case CORBA::tk_null:
               sprintf (buffer, "null") ;
               break ;
            case CORBA::tk_void:
               sprintf (buffer, "void") ;
               break ;
            case CORBA::tk_short:
               sprintf (buffer, "short") ;
               break ;
            case CORBA::tk_ushort:
               sprintf (buffer, "unsigned short") ;
               break ;
            case CORBA::tk_long:
               sprintf (buffer, "long") ;
               break ;
            case CORBA::tk_ulong:
               sprintf (buffer, "unsigned long") ;
               break ;
            case CORBA::tk_float:
               sprintf (buffer, "float") ;
               break ;
            case CORBA::tk_double:
               sprintf (buffer, "double") ;
               break ;
            case CORBA::tk_boolean:
               sprintf (buffer, "boolean") ;
               break ;
            case CORBA::tk_char:
               sprintf (buffer, "char") ;
               break ;
            case CORBA::tk_octet:
               sprintf (buffer, "octet") ;
               break ;
            case CORBA::tk_any:
               sprintf (buffer, "Any") ;
               break ;
            case CORBA::tk_TypeCode:
               sprintf (buffer, "TypeCode") ;
               break ;
            case CORBA::tk_Principal:
               sprintf (buffer, "Principal") ;
               break ;
            case CORBA::tk_objref:
            case CORBA::tk_struct:
            case CORBA::tk_union:
            case CORBA::tk_enum:
            case CORBA::tk_alias:
            case CORBA::tk_except:
            case CORBA::tk_value:
            case CORBA::tk_value_box:
            case CORBA::tk_native:
            case CORBA::tk_abstract_interface:
            case CORBA::tk_local_interface:
               sprintf (buffer, Tc->id()) ;
               break ;
            case CORBA::tk_string:
               sprintf (buffer, "string") ;
               break ;
            case CORBA::tk_sequence:
               buffer += sprintf (buffer, "sequence of ") ;
               Type_String (Tc->content_type(), buffer) ;
               break ;
            case CORBA::tk_array:
               buffer += sprintf (buffer, "array of ") ;
               Type_String (Tc->content_type(), buffer) ;
               break ;
            case CORBA::tk_longlong:
               sprintf (buffer, "long long") ;
               break ;
            case CORBA::tk_ulonglong:
               sprintf (buffer, "unsigned long long") ;
               break ;
            case CORBA::tk_longdouble:
               sprintf (buffer, "long double");
               break ;
            case CORBA::tk_wchar:
               sprintf (buffer, "wide char") ;
               break ;
            case CORBA::tk_wstring:
               sprintf (buffer, "wide string") ;
               break ;
            case CORBA::tk_fixed:
               sprintf (buffer, "fixed") ;
               break ;
            default:
               buffer[0] = 0 ;
         }
      }

      void Decoder::Type_String (const octet * tc, int index, char * buffer, bool swap)
      {
         CORBA::TCKind kind;
         int idx = index ;

         if (tc[idx] == tk_indirect)
         {  // Indirect typecode
            idx += 4 ;
            int offset = *(const int *) (tc + idx) ;
            if (swap) Swap_4 ((octet *) &offset, 0) ;
            idx += offset ;
         }

         kind = (CORBA::TCKind) (tc[idx] + tc[idx + 3]) ;
         switch (kind)
         {
            case CORBA::tk_null:
               sprintf (buffer, "null") ;
               break ;
            case CORBA::tk_void:
               sprintf (buffer, "void") ;
               break ;
            case CORBA::tk_short:
               sprintf (buffer, "short") ;
               break ;
            case CORBA::tk_ushort:
               sprintf (buffer, "unsigned short") ;
               break ;
            case CORBA::tk_long:
               sprintf (buffer, "long") ;
               break ;
            case CORBA::tk_ulong:
               sprintf (buffer, "unsigned long") ;
               break ;
            case CORBA::tk_float:
               sprintf (buffer, "float") ;
               break ;
            case CORBA::tk_double:
               sprintf (buffer, "double") ;
               break ;
            case CORBA::tk_boolean:
               sprintf (buffer, "boolean") ;
               break ;
            case CORBA::tk_char:
               sprintf (buffer, "char") ;
               break ;
            case CORBA::tk_octet:
               sprintf (buffer, "octet") ;
               break ;
            case CORBA::tk_any:
               sprintf (buffer, "Any") ;
               break ;
            case CORBA::tk_TypeCode:
               sprintf (buffer, "TypeCode") ;
               break ;
            case CORBA::tk_Principal:
               sprintf (buffer, "Principal") ;
               break ;
            case CORBA::tk_objref:
            case CORBA::tk_struct:
            case CORBA::tk_union:
            case CORBA::tk_enum:
            case CORBA::tk_alias:
            case CORBA::tk_except:
            case CORBA::tk_value:
            case CORBA::tk_value_box:
            case CORBA::tk_native:
            case CORBA::tk_abstract_interface:
            case CORBA::tk_local_interface:
               sprintf (buffer, Type_Id (tc, idx + 12)) ;
               break ;
            case CORBA::tk_string:
               sprintf (buffer, "string") ;
               break ;
            case CORBA::tk_sequence:
               buffer += sprintf (buffer, "sequence of ") ;
               Type_String (tc, idx + 12, buffer, needSwap(*(tc + idx + 8))) ;
               break ;
            case CORBA::tk_array:
               buffer += sprintf (buffer, "array of ") ;
               Type_String (tc, idx + 12, buffer, needSwap(*(tc + idx + 8))) ;
               break ;
            case CORBA::tk_longlong:
               sprintf (buffer, "long long") ;
               break ;
            case CORBA::tk_ulonglong:
               sprintf (buffer, "unsigned long long") ;
               break ;
            case CORBA::tk_longdouble:
               sprintf (buffer, "long double");
               break ;
            case CORBA::tk_wchar:
               sprintf (buffer, "wide char") ;
               break ;
            case CORBA::tk_wstring:
               sprintf (buffer, "wide string") ;
               break ;
            case CORBA::tk_fixed:
               sprintf (buffer, "fixed") ;
               break ;
            case tk_indirect:
               sprintf (buffer, "indirect") ;
               break ;
            default:
               break ;
         }
      }

      void Decoder::Print_Indentation (int indent)
      {
         fprintf (m_file, "%*s", 2*indent, Indentation_Line) ;
      }

      void Decoder::Skip_Type_Code (const octet * code, int & index, bool swap)
      {
         CORBA::TCKind  kind ;

         index += (- index) & 3 ;
         if (code [index] == 255)
         {  // Indirect typecode
            index += 8 ;
         }
         else
         {
            kind = (CORBA::TCKind) (code [index] + code [index + 3]) ;
            index += 4 ;
            if ((kind > CORBA::tk_Principal) && ((kind < CORBA::tk_longlong) || (kind > CORBA::tk_wchar)))
            {
               if ((kind != CORBA::tk_string) && (kind != CORBA::tk_wstring) && (kind != CORBA::tk_fixed))
               {
                  volatile int len = *(const int *) (code + index) ;
                  if (swap) Swap_4 ((octet *) &len, 0) ;
                  index += len ;
               }
               index += 4 ;
            }
         }
      }

      
      const octet * Decoder::Read_Typecode (const octet * code, int & index, bool swap)
      {
         index += (-index) & 3 ;
         const octet * the_code = code + index ;

         if (code [index] == 255)
         {  // Indirect typecode
            volatile int offset = *(const int *) (the_code + 4) ;
            if (swap) Swap_4 ((octet *) &offset, 0) ;
            the_code += offset ;
         }

         Skip_Type_Code (code, index, swap) ;

         CORBA::TCKind kind = (CORBA::TCKind) (the_code[0] + the_code[3]) ;
         if (kind == CORBA::tk_alias)
         {
            // Get unaliased type
            octet endian = (the_code[0] == 0)? 0:1 ;
            bool code_swap = needSwap (endian) ;
            int idx = 0 ;
            the_code = Read_Typecode (the_code, idx, code_swap) ;
         }

         return the_code ;
      }


      void Decoder::Print_Any (const octet * item, int & start, bool swap, int indent)
      {
         fprintf (m_file, "(Any) ") ;
         const octet * its_code = Read_Typecode (item, start, swap) ;

         Print_Next_Element (item, start, swap, its_code, swap, indent + 1) ;
      }

      void Decoder::Print_Typecode (const octet * item, int & start, bool swap)
      {
         const octet * the_code = Read_Typecode (item, start, swap) ;

         char buffer [200] ;
         Type_String (the_code, 0, buffer, swap) ;
         fprintf (m_file, "(Typecode) %s\n", buffer) ;
      }

      void Decoder::Print_Principal (const octet * item, int & start, bool swap)
      {
         fprintf (m_file, "(Principal) Principal is not displayed\n") ;
         start += (-start) & 3 ;
         start += 4 + *(const int *) (item + start) ;
      }

      void Decoder::Print_Object (const octet * item, int & start, bool swap, int indent)
      {
         start += (-start) & 3 ;
         const char * r_id = (const char *) (item + start + 4) ;


         // Skip repository id
         Skip_String (item, start, swap) ;

         // num profiles
         start += (-start) & 3 ;
         volatile int num = *(const int *) (item + start) ;
         if (swap) Swap_4 ((octet *) &num, 0) ;
         start += 4 ;
         fprintf (m_file, "(Object of type %s) %d profiles\n", r_id, num) ;

         for (int each = 1 ; each <= num ; each ++)
         {
            start += (-start) & 3 ;
            volatile unsigned int tag = *(const unsigned int *) (item + start) ;
            if (swap) Swap_4 ((octet *) &tag, 0) ;
            // Skip tag
            start += 4 ;
            Print_Indentation (indent + 1) ;
            if (tag == 0)
            {
               const char * host = (const char *) (item + start + 12) ;
               volatile int host_len = *(const int *) (item + start + 8) ;
               if (swap) Swap_4 ((octet *) &host_len, 0) ;
               volatile unsigned short port = *(const unsigned short *) (item + start + 12 + host_len) ;
               if (swap) Swap_2 ((octet *) &port, 0) ;
               fprintf (m_file, "profile %d: tag IIOP host %s port %hd\n", each, host, port) ;
            }
            else
            {
               fprintf (m_file, "profile %d: tag %#X\n", each, tag) ;
            }

            // Skip profile
            volatile int p_len = *(const int *) (item + start) ;
            if (swap) Swap_4 ((octet *) &p_len, 0) ;
            start += 4 + p_len ;
         }
      }

      void Decoder::Print_String (const octet * item, int & start, bool swap)
      {
         start += (-start) & 3 ;
         const char * str = (const char *) (item + start + 4) ;

         fprintf (m_file, "(String) \"%s\"\n", str) ;

         Skip_String (item, start, swap) ;
      }

      void Decoder::Print_Wide_String (const octet * item, int & start, bool swap)
      {
         start += (-start) & 3 ;
         const char * str = (const char *) (item + start + 4) ;

         fprintf (m_file, "(WString) \"%s\"\n", str) ;

         Skip_String (item, start, swap) ;
      }

      void Decoder::Print_Enum (const octet * item, int & start, bool swap, const octet * code)
      {
         char buffer [200] ;
         octet endian = (code[0] == 0)? 0:1 ;
         bool code_swap = needSwap (endian) ;
         Type_String (code, 0, buffer, code_swap) ;
         fprintf (m_file, "(Enum %s)", buffer) ;

         start += (-start) & 3 ;
         volatile int an_enum = *(const int *) (item + start) ;
         if (swap) Swap_4 ((octet *) &an_enum, 0) ;
         start += 4 ;

         code_swap = needSwap (code[8]) ;
         int index = 12 ;
         Skip_String (code, index, code_swap) ; // Repository ID
         Skip_String (code, index, code_swap) ; // Name
         // Member count
         index += (-index) & 3 ;
         volatile int count = *(const int *) (code + index) ;
         if (code_swap) Swap_4 ((octet *) &count, 0) ;
         index += 4 ;

         for (int each = 0 ; each < an_enum ; each ++)
         {
            if (each >= count)
            {
               fprintf (m_file, " erroneous enum value\n") ;
               return ;
            }
            Skip_String (code, index, code_swap) ; // Member Name
            index += (-index) & 3 ;
         }

         Type_String (code, index, buffer, code_swap) ;
         fprintf (m_file, "%s\n", buffer) ;
      }

      void Decoder::Print_Alias (const octet * item, int & start, bool swap,
                                 const octet * code, int & index, int indent)
      {
         bool code_swap ;
         CORBA::TCKind kind = CORBA::tk_alias ;
         while (kind == CORBA::tk_alias)
         {
            code_swap = needSwap (code[index + 8]) ;
            index += 12 ; // Skip Kind, aliased type size and Endian
            Skip_String (code, index, code_swap) ; // Repository ID
            Skip_String (code, index, code_swap) ; // Name
            index += (-index) & 3 ;
            if (code[index] == tk_indirect)
            {  // Indirect typecode
               index += 4 ;
               volatile int offset = *(const int *) (code + index) ;
               if (code_swap) Swap_4 ((octet *) &offset, 0) ;
               index += offset ;
            }

            kind = (CORBA::TCKind) (code[index] + code[index + 3]) ;
         }

         const octet * sub_code = Read_Typecode (code, index, code_swap) ;
         Print_Next_Element (item, start, swap, sub_code, code_swap, indent) ;
      }

      void Decoder::Print_Struct (const octet * item, int & start, bool swap,
                                  const octet * code, int & index, int indent)
      {
         bool code_swap ;
         char buffer [200] ;

         code_swap = needSwap (code[index + 8]) ;
         index += 12 ; // Skip Kind, aliased type size and Endian
         Type_String (code, index, buffer, code_swap) ;
         fprintf (m_file, "(Struct %s)", buffer) ;

         Skip_String (code, index, code_swap) ; // Repository ID
         Skip_String (code, index, code_swap) ; // Name
         index += (-index) & 3 ;

         // Member count
         volatile int count = *(const int *) (code + index) ;
         if (code_swap) Swap_4 ((octet *) &count, 0) ;
         index += 4 ;

         if (count == 0)
         {
            fprintf (m_file, " => no member\n") ;
         }
         else
         {
            fprintf (m_file, "\n") ;
         }

         for (int each = 0 ; each < count ; each ++)
         {
            Print_Indentation (indent) ;
            index += (-index) & 3 ;
            // Member Name
            Type_String (code, index, buffer, code_swap) ;
            fprintf (m_file, "%s => ", buffer) ;
            Skip_String (code, index, code_swap) ;
            index += (-index) & 3 ;
            const octet * sub_code = Read_Typecode (code, index, code_swap) ;
            Print_Next_Element (item, start, swap, sub_code, code_swap, indent + 1) ;
         }
      }

      void Decoder::Print_Value_Members (const octet * item, int & start, bool swap,
                                         const octet * code, int & index, int indent)
      {
         bool code_swap ;
         char buffer [200] ;

         code_swap = needSwap (code[index + 8]) ;
         index += 12 ; // Skip Kind, aliased type size and Endian
         Skip_String (code, index, code_swap) ; // Repository ID
         Skip_String (code, index, code_swap) ; // Name

         index += (-index) & 1 ;
         volatile CORBA::ValueModifier modifier = *(const CORBA::ValueModifier *) (code + index) ;
         if (swap) Swap_2 ((octet *) &modifier, 0) ;


         if ((modifier == CORBA::VM_ABSTRACT) || (modifier == CORBA::VM_CUSTOM))
         {
            fprintf (m_file, "abstract or custom value, no display\n") ;
            return ;
         }

         index += (-index) & 3 ;
         CORBA::TCKind kind = (CORBA::TCKind) (code[index] + code[index + 3]) ;

         if (kind != CORBA::tk_null)
         {
            // Display ancestor members
            Print_Value_Members (item, start, swap, code, index, indent) ;
         }
         else
         {
            index += 4 ;
         }

         // Member count
         index += (-index) & 3 ;
         volatile int count = *(const int *) (code + index) ;
         if (code_swap) Swap_4 ((octet *) &count, 0) ;
         index += 4 ;

         for (int each = 0 ; each < count ; each ++)
         {
            Print_Indentation (indent) ;
            index += (-index) & 3 ;
            // Member Name
            Type_String (code, index, buffer, code_swap) ;
            fprintf (m_file, "%s => ", buffer) ;
            Skip_String (code, index, code_swap) ;
            index += (-index) & 3 ;
            const octet * sub_code = Read_Typecode (code, index, code_swap) ;
            Print_Next_Element (item, start, swap, sub_code, code_swap, indent + 1) ;
            // Skip visibility
            index += (-index) & 1 ;
            index += 2 ;
         }
      }

      const octet * Decoder::handle_value_header (const octet * item, int & start, bool swap,
                                                  const octet * code, int & index, int indent)
      {
         // Value tag
         volatile unsigned int value_tag = *(const unsigned int *) (item + start) ;
         if (swap) Swap_4 ((octet *) &value_tag, 0) ;
         start += 4 ;

         m_chunked = (value_tag & 8) == 8 ;

         octet endian = (code[index] == 0)? 0:1 ;
         bool code_swap = needSwap (endian) ;
         const octet * value_code = Read_Typecode (code, index, code_swap) ;

         if (value_tag == 0)
         {
            // Null value
            Print_Indentation (indent) ;
            fprintf (m_file, "   null\n") ;
            return 0 ;
         }
         else if (value_tag == tag_indirect)
         {
            // Indirect
            Print_Indentation (indent) ;
            fprintf (m_file, "   indirect\n") ;
            start += 4 ;
            return 0 ;
         }

         if ((value_tag & 1) == 1)
         {
            // Skip code base
            volatile unsigned int base_len = *(const unsigned int *) (item + start) ;
            if (swap) Swap_4 ((octet *) &base_len, 0) ;

            if (base_len == tag_indirect)
            {
               start += 8 ;
            }
            else
            {
               start += 4 + base_len ;
            }
         }

         if ((value_tag & 6) == 0)
         {
            // Actual has same class as formal, must use by actual typecode
         }
         else if ((value_tag & 6) == 2)
         {
            // One Id given
            start += (-start) & 3 ;
            volatile unsigned int id_len = *(const unsigned int *) (item + start) ;
            if (swap) Swap_4 ((octet *) &id_len, 0) ;

            // We should try to find the value typecode from the IFR
            if (id_len == tag_indirect)
            {
               start += 8 ;
            }
            else
            {
               start += 4 + id_len ;
            }
         }
         else if ((value_tag & 6) == 6)
         {
            // List of Ids to which the actual is truncatable
            // We should try to find the value typecode from the IFR using each Id 

            start += (-start) & 3 ;
            volatile unsigned int id_number = *(const unsigned int *) (item + start) ;
            if (swap) Swap_4 ((octet *) &id_number, 0) ;

            if (id_number == tag_indirect)
            {
               // Indirect, should look backward for value typecode
               start += 8 ;
            }
            else
            {
               start += 4 ;
               for (unsigned int each = id_number; each > 0 ; each --)
               {
                  start += (-start) & 3 ;
                  volatile unsigned int id_len = *(const unsigned int *) (item + start) ;
                  if (swap) Swap_4 ((octet *) &id_len, 0) ;

                  // We should try to find the value typecode from the IFR
                  if (id_len == tag_indirect)
                  {
                     start += 8 ;
                  }
                  else
                  {
                     start += 4 + id_len ;
                  }
               }
            }
         }

         // Handle the chunk info if any
         if (m_chunked)
         {
            start += (-start) & 3 ;
            // Chunk size
            volatile unsigned int chunk_size = *(const unsigned int *) (item + start) ;
            if (swap) Swap_4 ((octet *) &chunk_size, 0) ;
            start += 4 ;
            m_chunk_end = start + chunk_size ;
         }

         return value_code ;
      }

      void Decoder::handle_chunk_end (const octet * item, int & start, bool swap, int value_level)
            throw ()
      {
         start = m_chunk_end ;
         while (value_level < m_value_level)
         {
            // Skip chunks until Level is reached
            start += (-start) & 3 ;
            volatile unsigned int value_tag = *(const unsigned int *) (item + start) ;
            if (swap) Swap_4 ((octet *) &value_tag, 0) ;

            if (value_tag >= 0x80000000)
            { // End level tag
               m_value_level = (int) - value_tag ;
               // lower level end tag must be kept in the stream for
               // enclosing valuetype to get it
               if (m_value_level >= value_level)
               {
                  start += 4 ;
               }
            }
            else if (value_tag < 0x7fffff00)
            { // New chunk starting
               start += value_tag ;
            }
            else // New value starting
            {
               start += 4 ;

               // Check tag
               if ((value_tag & 8) == 0)
               {
                  // Inner values must be chunked
                  CORBA::MARSHAL ex ;
                  throw ex ;
               }

               // Skip code base
               if ((value_tag & 1) == 1)
               {
                  volatile unsigned int base_len = *(const unsigned int *) (item + start) ;
                  if (swap) Swap_4 ((octet *) &base_len, 0) ;

                  if (base_len == tag_indirect)
                  {
                     start += 8 ;
                  }
                  else
                  {
                     start += 4 + base_len ;
                  }
               }

               // Skip type information
               if ((value_tag & 6) == 2)
               { // Single id
                  start += (-start) & 3 ;
                  volatile unsigned int id_len = *(const unsigned int *) (item + start) ;
                  if (swap) Swap_4 ((octet *) &id_len, 0) ;

                  // Skip id
                  if (id_len == tag_indirect)
                  {
                     start += 8 ;
                  }
                  else
                  {
                     start += 4 + id_len ;
                  }
               }
               else if ((value_tag & 6) == 6)
               {  // Multiple ids
                  start += (-start) & 3 ;
                  volatile unsigned int id_number = *(const unsigned int *) (item + start) ;
                  if (swap) Swap_4 ((octet *) &id_number, 0) ;

                  if (id_number == tag_indirect)
                  {
                     // Indirect, skip offset
                     start += 8 ;
                  }
                  else
                  {
                     start += 4 ;
                     for (unsigned int each = id_number; each > 0 ; each --)
                     {
                        start += (-start) & 3 ;
                        volatile unsigned int id_len = *(const unsigned int *) (item + start) ;
                        if (swap) Swap_4 ((octet *) &id_len, 0) ;

                        // Skip the id
                        if (id_len == tag_indirect)
                        {
                           start += 8 ;
                        }
                        else
                        {
                           start += 4 + id_len ;
                        }
                     }
                  }
               }
               else
               {
                  CORBA::MARSHAL ex ;
                  throw ex ;
               }
            }
         }
      }

      void Decoder::Print_Value (const octet * item, int & start, bool swap,
                                 const octet * code, int & index, int indent)
      {
         bool code_swap ;
         char buffer [200] ;

         code_swap = needSwap (code[index + 8]) ;
         index += 12 ; // Skip Kind, aliased type size and Endian
         Type_String (code, index, buffer, code_swap) ;
         fprintf (m_file, "(Valuetype %s)\n", buffer) ;

         const octet * value_code = handle_value_header (item, start, swap, code, index, indent) ;

         if (value_code == 0)
         {
            return ;
         }

         int value_level = m_value_level ;
         // Increment the value level to take into account this value
         m_value_level++ ;

         // Read the value members
         int code_index = 0 ;
         Print_Value_Members (item, start, swap, value_code, code_index, indent) ;

         // Restore the value level
         m_value_level-- ;

         if (m_chunked)
         {
            handle_chunk_end (item, start, swap, value_level) ;
         }
      }

      void Decoder::Print_Value_Box (const octet * item, int & start, bool swap,
                                     const octet * code, int & index, int indent)
      {
         bool code_swap ;
         char buffer [200] ;

         code_swap = needSwap (code[index + 8]) ;
         index += 12 ; // Skip Kind, aliased type size and Endian
         Type_String (code, index, buffer, code_swap) ;
         fprintf (m_file, "(Valuebox %s)\n", buffer) ;

         const octet * value_code = handle_value_header (item, start, swap, code, index, indent) ;

         if (value_code == 0)
         {
            return ;
         }

         int value_level = m_value_level ;
         // Increment the value level to take into account this value
         m_value_level++ ;

         Print_Indentation (indent) ;
         Print_Next_Element (item, start, swap, value_code, code_swap, indent + 1) ;

         // Restore the value level
         m_value_level-- ;

         if (m_chunked)
         {
            handle_chunk_end (item, start, swap, value_level) ;
         }
      }

      void Decoder::Print_Exception (const octet * item, int & start, bool swap,
                                     const octet * code, int & index, int indent)
      {
         bool code_swap ;
         char buffer [200] ;

         code_swap = needSwap (code[index + 8]) ;
         index += 12 ; // Skip Kind, aliased type size and Endian
         Type_String (code, index, buffer, code_swap) ;
         fprintf (m_file, "(Exception %s)", buffer) ;

         Skip_String (code, index, code_swap) ; // Repository ID
         Skip_String (code, index, code_swap) ; // Name
         index += (-index) & 3 ;

         // Member count
         volatile int count = *(const int *) (code + index) ;
         if (code_swap) Swap_4 ((octet *) &count, 0) ;
         index += 4 ;

         if (count == 0)
         {
            fprintf (m_file, " => no member\n") ;
         }
         else
         {
            fprintf (m_file, "\n") ;
         }

         for (int each = 0 ; each < count ; each ++)
         {
            Print_Indentation (indent) ;
            index += (-index) & 3 ;
            // Member Name
            Type_String (code, index, buffer, code_swap) ;
            fprintf (m_file, "%s => ", buffer) ;
            Skip_String (code, index, code_swap) ;
            index += (-index) & 3 ;
            const octet * sub_code = Read_Typecode (code, index, code_swap) ;
            Print_Next_Element (item, start, swap, sub_code, code_swap, indent + 1) ;
         }
      }

      // Methode called once union discriminant value is found to determine
      // the member corresponding to the union contents
      int Decoder::Set_Union_Element_Start (const octet * item, int & start, bool swap,
                                            const octet * code, int & index, bool code_swap,
                                            int discrim_size)
      {
         bool found = false ;
         int member = 0 ; // Member offset in the typecode

         // Index of Discriminant "default" case
         index += (-index) & 3 ;
         volatile int default_case = *(const int *) (code + index) ;
         if (code_swap) Swap_4 ((octet *) &default_case, 0) ;
         index += 4 ;

         // Label count
         volatile int count = *(const int *) (code + index) ;
         if (code_swap) Swap_4 ((octet *) &count, 0) ;
         index += 4 ;

         for (int each = 0; each < count; each ++)
         {
            // Do label value
            switch (discrim_size)
            {
               case 1: // tk_char | tk_boolean
                  found = code [index] == item[start] ;
                  index ++ ;
                  break ;
               case 2 : // tk_short | tk_ushort
               {
                  start += (-start) & 1 ;
                  volatile short value_2 = *(const short *) (item + start) ;
                  if (swap) Swap_2 ((octet *) &value_2, 0) ;

                  index += (-index) & 1 ;
                  volatile short ref_2 = *(const short *) (code + index) ;
                  if (code_swap) Swap_2 ((octet *) &ref_2, 0) ;
                  index += 4 ;

                  found = value_2 == ref_2 ;
                  break ;
               }
               case 4: // tk_long | tk_ulong | tk_enum
               {
                  start += (-start) & 3 ;
                  volatile int value_4 = *(const int *) (item + start) ;
                  if (swap) Swap_4 ((octet *) &value_4, 0) ;

                  index += (-index) & 3 ;
                  volatile int ref_4 = *(const int *) (code + index) ;
                  if (code_swap) Swap_4 ((octet *) &ref_4, 0) ;
                  index += 4 ;

                  found = value_4 == ref_4 ;
                  break ;
               }
            }

            index += (-index) & 3 ;
            int offset = index ;
            Skip_String (code, index, code_swap) ; // Member Name

            if ((found) || ((default_case == each) && (member == 0)))
            {
               member = offset ;
            }

            Skip_Type_Code (code, index, code_swap) ; // Member typecode
         }

         return member ;
      }

      void Decoder::Print_Union (const octet * item, int & start, bool swap,
                                 const octet * code, int & index, int indent)
      {
         bool code_swap ;
         char buffer [200] ;

         code_swap = needSwap (code[index + 8]) ;
         index += 12 ; // Skip Kind, aliased type size and Endian
         Type_String (code, index, buffer, code_swap) ;
         fprintf (m_file, "(Union %s)", buffer) ;

         Skip_String (code, index, code_swap) ; // Repository ID
         Skip_String (code, index, code_swap) ; // Name
         index += (-index) & 3 ;

         // Discriminant Value
         Print_Indentation (indent) ;
         fprintf (m_file, "Switch => ") ;

         const octet * discrim_code = Read_Typecode (code, index, code_swap) ;
         Print_Next_Element (item, start, swap, discrim_code, code_swap, indent + 1) ;

         Print_Indentation (indent) ;
         int discrim_size = 0 ;
         CORBA::TCKind d_kind = (CORBA::TCKind) (discrim_code[0] + discrim_code[3]) ;
         switch (d_kind)
         {
            case CORBA::tk_boolean:
            case CORBA::tk_char:
               discrim_size = 1 ;
               break ;
            case CORBA::tk_short:
            case CORBA::tk_ushort:
               discrim_size = 2 ;
               break ;
            case CORBA::tk_long:
            case CORBA::tk_ulong:
            case CORBA::tk_enum:
               discrim_size = 4 ;
               break ;
            default:
               break ;
         }

         int member = Set_Union_Element_Start (item, start, swap, code, index, code_swap, discrim_size) ;
         if (member > 0)
         {
            Type_String (code, member, buffer, code_swap) ;
            fprintf (m_file, "%s => ", buffer) ;

            Skip_String (code, member, code_swap) ; // member name

            member += (-member) & 3 ;
            const octet * member_code = code + member ;
            Print_Next_Element (item, start, swap, member_code, code_swap, indent + 1) ;
         }
      }

      void Decoder::Print_Array (const octet * item, int & start, bool swap,
                                 const octet * code, int & index, int indent)
      {
         char buffer [200] ;

         // First get the element type code
         bool code_swap = needSwap (code[index + 8]) ;
         index += 12 + ((-index) & 3) ;
         const octet * element_code = Read_Typecode (code, index, code_swap) ;

         // Then get the array length
         index += (-index) & 3 ;
         volatile int length = *(const int *) (code + index) ;
         if (code_swap) Swap_4 ((octet *) &length, 0) ;
         index += 4 ;

         int idx = 0 ;
         Type_String (element_code, idx, buffer, code_swap) ;
         fprintf (m_file, "(Array of %s[%d]\n)", buffer, length) ;

         for (int each = 0; each < length; each ++)
         {
            fprintf (m_file, "[%d] => ", each) ;
            Print_Next_Element (item, start, swap, element_code, code_swap, indent + 1) ;
         }
      }

      void Decoder::Print_Sequence (const octet * item, int & start, bool swap,
                                    const octet * code, int & index, int indent)
      {
         char buffer [200] ;

         // First get the element type code
         bool code_swap = needSwap (code[index + 8]) ;
         index += 12 + ((-index) & 3) ;
         const octet * element_code = Read_Typecode (code, index, code_swap) ;
         index += 4 + ((-index) & 3) ;

         // Then get the sequence length
         start += (-start) & 3 ;
         volatile int length = *(const int *) (item + start) ;
         if (swap) Swap_4 ((octet *) &length, 0) ;
         start += 4 ;

         int idx = 0 ;
         Type_String (element_code, idx, buffer, code_swap) ;
         fprintf (m_file, "(Sequence of %s[%d]\n)", buffer, length) ;

         for (int each = 0; each < length; each ++)
         {
            fprintf (m_file, "[%d] => ", each) ;
            Print_Next_Element (item, start, swap, element_code, code_swap, indent + 1) ;
         }
      }

      void Decoder::Print_Fixed (const octet * item, int & start, bool swap,
                                 const octet * code, int & index, int indent)
      {
         index += (-index) & 3 ;
         octet endian = (code[index] == 0)? 0:1 ;
         bool code_swap = needSwap (endian) ;
         volatile short digits = *(const short *) (code + index) ;
         if (code_swap) Swap_2 ((octet *) &digits, 0) ;
         index += 2 ;
         volatile short scale = *(const short *) (code + index) ;
         if (code_swap) Swap_2 ((octet *) &scale, 0) ;
         index += 2 ;

         fprintf (m_file, "(Fixed %d.%d) => ", digits, scale) ;

         octet low, high, pair ;

         for (bool done = false; done == false;)
         {
            pair = item[start] ;
            start++ ;
            low = pair & 15 ;
            high = pair >> 4 ;
            fprintf (m_file, "%c", 48 + high) ;

            switch (low)
            {
               case 12:
                  fprintf (m_file, "+") ;
                  done = true ;
                  break ;
               case 13:
                  fprintf (m_file, "-") ;
                  done = true ;
                  break ;
               default:
                  fprintf (m_file, "%c", 48 + low) ;
            }
         }
      }

      void Decoder::Print_Abstract (const octet * item, int & start, bool swap,
                                    const octet * code, int & index, int indent)
      {
         bool is_object = item[start] == 1 ;
         start ++ ;
         fprintf (m_file, "(Abstract)") ;

         if (is_object)
         {
            Print_Object (item, start, swap, indent) ;
         }
         else
         {
            Print_Value (item, start, swap, code, index, indent) ;
         }
      }

      void Decoder::Print_Next_Element (const octet * item, int & start, bool swap,
                                        const octet * code, bool code_swap, int indent)
      {
         int index = 0 ;

         if ( item == 0)
         {
            return;
         }

         // Extract kind
         if (code[index] == tk_indirect)
         {  // Indirect typecode
            index += 4 ;
            volatile int offset = *(const int *) (code + index) ;
            if (code_swap) Swap_4 ((octet *) &offset, 0) ;
            index += offset ;
         }

         CORBA::TCKind kind = (CORBA::TCKind) (code[index] + code[index + 3]) ;

         switch (kind)
         {
            case CORBA::tk_null:
               fprintf (m_file, "(Null)") ;
               index += 4 ;
               break ;

            case CORBA::tk_void:
               fprintf (m_file, "(Void)") ;
               index += 4 ;
               break ;

            case CORBA::tk_short:
            {
               start += (-start) & 1 ;
               volatile short a_short = *(const short *) (item + start) ;
               if (swap) Swap_2 ((octet *) &a_short, 0) ;
               index += 4 ;
               start += 2 ;
               fprintf (m_file, "(Short) => %hd\n", a_short) ;
               break ;
            }

            case CORBA::tk_long:
            {
               start += (-start) & 3 ;
               volatile int a_long = *(const int *) (item + start) ;
               if (swap) Swap_4 ((octet *) &a_long, 0) ;
               index += 4 ;
               start += 4 ;
               fprintf (m_file, "(Long) => %d\n", a_long) ;
               break ;
            }

            case CORBA::tk_longlong:
            {
               start += (-start) & 7 ;
               volatile long int a_long_long = *(const long int *) (item + start) ;
               if (swap) Swap_8 ((octet *) &a_long_long, 0) ;
               index += 4 ;
               start += 8 ;
               fprintf (m_file, "(LongLong) => %ld\n", a_long_long) ;
               break ;
            }

            case CORBA::tk_ushort:
            {
               start += (-start) & 1 ;
               volatile unsigned short a_ushort = *(const unsigned short *) (item + start) ;
               if (swap) Swap_2 ((octet *) &a_ushort, 0) ;
               index += 4 ;
               start += 2 ;
               fprintf (m_file, "(Unsigned Short) => %hu\n", a_ushort) ;
               break ;
            }

            case CORBA::tk_ulong:
            {
               start += (-start) & 3 ;
               volatile unsigned int a_ulong = *(const unsigned int *) (item + start) ;
               if (swap) Swap_4 ((octet *) &a_ulong, 0) ;
               index += 4 ;
               start += 4 ;
               fprintf (m_file, "(Unsigned Long) => %u\n", a_ulong) ;
               break ;
            }

            case CORBA::tk_ulonglong:
            {
               start += (-start) & 7 ;
               volatile unsigned long int a_ulong_long = *(const unsigned long int *) (item + start) ;
               if (swap) Swap_8 ((octet *) &a_ulong_long, 0) ;
               index += 4 ;
               start += 8 ;
               fprintf (m_file, "(Unsigned LongLong) => %lu\n", a_ulong_long) ;
               break ;
            }

            case CORBA::tk_float:
            {
               start += (-start) & 3 ;
               volatile float a_float = *(const float *) (item + start) ;
               if (swap) Swap_4 ((octet *) &a_float, 0) ;
               index += 4 ;
               start += 4 ;
               fprintf (m_file, "(Float) => %E\n", (double)a_float) ;
               break ;
            }

            case CORBA::tk_double:
            {
               start += (-start) & 7 ;
               volatile double a_double = *(const double *) (item + start) ;
               if (swap) Swap_8 ((octet *) &a_double, 0) ;
               index += 4 ;
               start += 8 ;
               fprintf (m_file, "(Double) => %E\n", a_double) ;
               break ;
            }

            case CORBA::tk_longdouble:
               start += (-start) & 7 ;
               index += 4 ;
               start += 16 ;
               fprintf (m_file, "(LongDouble) \n") ;
               break ;

            case CORBA::tk_boolean:
               if (item[start] == 0)
               {
                  fprintf (m_file, "(Boolean) => false\n") ;
               }
               else
               {
                  fprintf (m_file, "(Boolean) => true\n") ;
               }
               index += 4 ;
               start ++ ;
               break ;

            case CORBA::tk_char:
               fprintf (m_file, "(Character) => %c\n", item[start]) ;
               index += 4 ;
               start ++ ;
               break ;

            case CORBA::tk_wchar:
               fprintf (m_file, "(Wide Character) => \n") ;
               index += 4 ;
               start += 1 + item[start] ;
               break ;

            case CORBA::tk_octet:
               fprintf (m_file, "(Octet) => %hhd\n", item[start]) ;
               index += 4 ;
               start ++ ;
               break ;

            case CORBA::tk_any:
               Print_Any (item, start, swap, indent) ;
               break ;

            case CORBA::tk_TypeCode:
               Print_Typecode (item, start, swap) ;
               break ;

            case CORBA::tk_Principal:
               Print_Principal (item, start, swap) ;
               break ;

            case CORBA::tk_objref :
               Print_Object (item, start, swap, indent) ;
               break ;

            case CORBA::tk_string:
               Print_String (item, start, swap) ;
               break ;

            case CORBA::tk_wstring:
               Print_Wide_String (item, start, swap) ;
               break ;

            case CORBA::tk_enum:
               Print_Enum (item, start, swap, code) ;
               break ;

            case CORBA::tk_alias:
               Print_Alias (item, start, swap, code, index, indent) ;
               break ;

            case CORBA::tk_struct:
               Print_Struct (item, start, swap, code, index, indent) ;
               break ;

            case CORBA::tk_union:
               Print_Union (item, start, swap, code, index, indent) ;
               break ;

            case CORBA::tk_sequence:
               Print_Sequence (item, start, swap, code, index, indent) ;
               break ;

            case CORBA::tk_array:
               Print_Array (item, start, swap, code, index, indent) ;
               break ;

            case CORBA::tk_except:
               Print_Exception (item, start, swap, code, index, indent) ;
               break ;

            case CORBA::tk_fixed:
               Print_Fixed (item, start, swap, code, index, indent) ;
               break ;

            case CORBA::tk_value:
               Print_Value (item, start, swap, code, index, indent) ;
               break ;

            case CORBA::tk_value_box:
               Print_Value_Box (item, start, swap, code, index, indent) ;
               break ;

            case CORBA::tk_native:
               // Should never occur
               break ;

            case CORBA::tk_abstract_interface:
               Print_Abstract (item, start, swap, code, index, indent) ;
               break ;

            case CORBA::tk_local_interface:
               // Should never occur
               break ;
            default:
               break ;
         }
      }
   }
}
