/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <clocale>
#include <fstream>
#include <iostream>
#include <sstream>
#include <string>
#include <vector>

#include "recordingtools/Display.hpp"
#include "recordingtools/Command.hpp"
#include "recordingtools/Decoder.hpp"
#include "recordingtools/InterfaceMap.hpp"
#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/Logger.hpp>

namespace
{
   Cdmw::Logging::Logger * logger = 0;
};

void exceptionHandler(void)
{
  CDMW_ERROR(logger, "*** Error in main_debug_decoder : UNEXPECTED EXCEPTION HANDLER");
}

//
// main module
//
int main(int argc, char* argv[])
{
    // for international char management
    // set for all locale categories the traditional unix system behavior
    ::setlocale (LC_ALL,"C");

#   ifndef _MSC_VER

    std::set_unexpected(&exceptionHandler);

    // VC++ contrary to the clause 18.6 of the C++ standard,
     //  set_unexpected, is not inside the std:: namespace
#   else

    set_unexpected(&exceptionHandler);

#   endif

    using Cdmw::Logging::LogManager;
    LogManager::Init(argc, argv);
    logger = Cdmw::Logging::Logger::Get_Logger("CDMW.TRC.RecordingDaemon");

    // get the debugger options
    Cdmw::Recording::DebuggerOptions options;
    int result = Cdmw::Recording::get_debugger_options (argc, argv, options);
    if (result == Cdmw::Recording::FAILURE)
    {
      return Cdmw::Recording::FAILURE;
    }
    else if (result == Cdmw::Recording::EXIT)
    {
      return Cdmw::Recording::SUCCESS;
    }

    CDMW_INFO(logger, "=== main_debug_decoder options :\n" << "   interactive= " << options.interactive << "\n");
    CDMW_INFO(logger, "    flags=" << options.flags << "\n");
    CDMW_INFO(logger, "    ifr_url=" << options.ifr_url);

    unsigned int flags = (unsigned int) options.flags;
    const char * ifr_url = options.ifr_url.c_str();

    Cdmw::Recording::Command * command = new Cdmw::Recording::Command(flags, ifr_url);
  try
  {
    command->Decode(options.interactive);
  }
  catch (...)
  {}

}

