/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <clocale>
#include <fstream>
#include <iostream>
#include <sstream>
#include <string>
#include <vector>

#include "Foundation/ossupport/Signals.hpp"
#include "Foundation/ossupport/SignalHandler.hpp"

#include <Foundation/common/Assert.hpp>
#include <Foundation/common/Exception.hpp>
#include <Foundation/common/Locations.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/System.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/Logger.hpp>

#include "recordingtools/DaemonInit.hpp"
#include "recordingtools/CdmwRecordingContainerManager_impl.hpp"
#include "recordingtools/CdmwRecordingDaemonControl_impl.hpp"
#include "Foundation/ossupport/SystemProperties.hpp"

namespace
{
    using std::string;
    
    // default port number used by the local recording daemon
    const std::string DFLT_SERVICE_PORT = "59876";
        
    // ORB reference
    CORBA::ORB_var orb = CORBA::ORB::_nil();
    Cdmw::Logging::Logger * logger = 0;
    
}; // anonymous namespace

using namespace Cdmw;
using namespace Cdmw::OsSupport;
using namespace Cdmw::OrbSupport;
using namespace Cdmw::Trace;

//
// end of processing
//
void end_recording_daemon(void)
{
    //nothing to do
}

void exceptionHandler(void)
{
    CDMW_ERROR(logger, "*** Error : Recording daemon : UNEXPECTED EXCEPTION HANDLER");
}


void exit_handler(int sig) 
{
    if (!CORBA::is_nil(orb.in())) 
    {
        try 
        {
            orb->shutdown(false);
        } catch (const CORBA::SystemException& e) {
            CDMW_ERROR(logger, "*** Error while shutting ORB down in exit_handler:\n"
                      << e << " - minor code: " << e.minor());
        }
    }
}


class ExitHandler : public Cdmw::OsSupport::SignalHandler
{
        public:
                ExitHandler() {m_call = false;} ;

                virtual ~ExitHandler() throw () {} ;

                int handle_signal(int sig)
                                throw ()
                {
                        m_call = true;
                        CDMW_INFO(logger, "=== ExitHandler called with sig " << sig);
                        exit_handler(sig);
                        return sig;
                }


                bool handler_called()
                {
                        return m_call;
                }

        private:
                bool m_call;
};

//
// main module
//
int main(int argc, char* argv[])
{
    // JM : Thales 12/07/07
    Cdmw::OsSupport::SystemProperties::Init (argc, argv);

    // for international char management
    // set for all locale categories the traditional unix system behavior
    ::setlocale (LC_ALL,"C");
    /*
#   ifndef _MSC_VER    

//    std::set_unexpected(&exceptionHandler);

    // VC++ contrary to the clause 18.6 of the C++ standard, 
     //  set_unexpected, is not inside the std:: namespace
#   else

//    set_unexpected(&exceptionHandler);

#   endif  

*/

    using namespace Cdmw::OsSupport;
    using Cdmw::Logging::LogManager;
    
    // get the recording daemon options
    DaemonInit::DaemonOptions options;
    bool arg_rebuilt = false;
    char** new_argv = argv;
    int new_argc = argc;
    int arg_ini_inx = argc;
    
    LogManager::Init(argc, argv);
    logger = Cdmw::Logging::Logger::Get_Logger("CDMW.TRC.RecordingDaemon");
    
    int result = DaemonInit::get_recdaemon_options (argc, argv, options,
                                                       new_argc, new_argv,
                                                       DFLT_SERVICE_PORT);
    if (result == FAILURE)
    {
        return FAILURE;
    }
    else if (result == EXIT)
    {
        return SUCCESS;
    }
    
    // if new arguments has been added                                                   
    if (new_argc > argc)
    {
        arg_rebuilt = true;
    }
    
  
    CDMW_INFO(logger, "Recording Daemon starting ...");
    CDMW_INFO(logger, "Recording Daemon " << " is listening on port " << options.process_port);

    int status = SUCCESS;
    
    try 
    {
        PortableServer::POA_var root_POA;        
                        
       // init the orb and platforminterface
        // (exception is raised in case of error)
        DaemonInit::init_processing (
                      new_argc, new_argv,
                      0,
                      orb.out(), root_POA.out() );
        
       // create the process agent to allow process stopping
        status = DaemonInit::create_process_agent (orb.in(), root_POA.in(), options);
        
        if (status == SUCCESS) 
        {
           // FIXME: Check for side effects with Cdmw::OsSupport::Signals class
            /*struct sigaction action;
            action.sa_handler = exit_handler;
            sigemptyset(&action.sa_mask);
            sigaction(SIGTERM, &action, 0);
            sigaction(SIGINT, &action, 0);
            sigaction(SIGQUIT, &action, 0);
            sigaction(SIGABRT, &action, 0);*/

            ExitHandler* eh1 = new ExitHandler();
            ExitHandler* eh2 = new ExitHandler();
            ExitHandler* eh3 = new ExitHandler();
            ExitHandler* eh4 = new ExitHandler();

            Cdmw::OsSupport::SignalHandler* oldseh =
                    Cdmw::OsSupport::Signals::Instance()->register_handler(SIGTERM, eh1);
            if (oldseh != 0)
            {
               CDMW_ERROR(logger, "*** Error while registering SIGTERM event handler");
            }

            oldseh = Cdmw::OsSupport::Signals::Instance()->register_handler(SIGINT, eh2);
            if (oldseh != 0)
            {
               CDMW_ERROR(logger, "*** Error while registering SIGINT event handler");
            }
            oldseh = Cdmw::OsSupport::Signals::Instance()->register_handler(SIGQUIT, eh3);
            if (oldseh != 0)
            {
               CDMW_ERROR(logger, "*** Error while registering SIGQUIT event handler");
            }

            oldseh = Cdmw::OsSupport::Signals::Instance()->register_handler(SIGABRT, eh4);
            if (oldseh != 0)
            {
               CDMW_ERROR(logger, "*** Error while registering SIGABRT event handler");
            }

            // run processing
            status = DaemonInit::run_processing (orb.in(), 0);

            oldseh = Cdmw::OsSupport::Signals::Instance()->unregister_handler(SIGTERM);
            if (oldseh != eh1)
            {
                    CDMW_ERROR(logger, "*** Error while unregistering SIGTERM event handler");
            }
            oldseh = Cdmw::OsSupport::Signals::Instance()->unregister_handler(SIGINT);
            if (oldseh != eh2)
            {
                    CDMW_ERROR(logger, "*** Error while unregistering SIGINT event handler");
            }
            oldseh = Cdmw::OsSupport::Signals::Instance()->unregister_handler(SIGQUIT);
            if (oldseh != eh3)
            {
                    CDMW_ERROR(logger, "*** Error while unregistering SIGQUIT event handler");
            }
            oldseh = Cdmw::OsSupport::Signals::Instance()->unregister_handler(SIGABRT);

            if (oldseh != eh4)
            {
                    CDMW_ERROR(logger, "*** Error while unregistering SIGABRT event handler");
            }
            delete eh1;
            delete eh2;
            delete eh3;
            delete eh4;
        }

        // end of process
        end_recording_daemon();
    }
    catch (const Cdmw::Exception& ex) {
        CDMW_ERROR(logger, "*** Error : " << ex.what());
        status = FAILURE;
    }
    catch(const CORBA::Exception& ex) {
        CDMW_ERROR(logger, "*** Error : " << ex );
        status = FAILURE;
    }
    catch (...) {
        CDMW_ERROR(logger, "*** Error : unknown exception in main");
        status = FAILURE;
    }

    // end of corba processing

    if(!CORBA::is_nil(orb.in())) 
    {
        try 
        {
            Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
            orb->destroy();
        }
        catch(const CORBA::Exception& ex) 
        {
            CDMW_ERROR(logger, "*** Error : " << ex );
            status = FAILURE;
        }
    }

    // if program arguments table has been rebuilt
    if (arg_rebuilt) 
    {
        // release any allocated memory
        for (int i=arg_ini_inx ; i < new_argc; ++i) 
        {
            delete[] new_argv[i];
        }

        delete[] new_argv;
    }

    return status;
}
