/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.recording.gui;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.awt.image.DataBufferByte;
import java.awt.image.IndexColorModel;
import java.awt.image.Raster;

import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;

/**
 * This is a generic component for selecting from a list. The component is made up of
 * lists of objects (available and selected) with four buttons for control. An object
 * is selected by being placed in the selected list using the button. Any object left
 * in the available list is considered not selected.
 *
 * @author Paul Graham
 */
public abstract class ChooserPanel extends JPanel
{
   /**
    * DefaultListModel to hold the data that represents the available values.
    */
   DefaultListModel availableModel = null;

   /**
    * Sorted version of the available list model.
    */
   SortedListModel sortedAvailableModel = null;

   /**
    * DefaultListModel to hold the data that represents the selected values;
    */
   DefaultListModel selectedModel = null;

   /**
    * Sorted version of the selected list model.
    */
   SortedListModel sortedSelectedModel = null;

   /**
    * JList to hold the list of available values.
    */
   JList availableValues = null;

   /**
    * JList to hold the list of selected values.
    */
   JList selectedValues = null;

   /**
    * JLabel for the available list.
    */
   JLabel availableLabel = null;

   /**
    * JLabel for the selected list.
    */
   JLabel selectedLabel = null;

   /**
    * Button to add a value from the available values list to the selected values list.
    */
   JButton add = null;

   /**
    * Button to add all the values from the available values list to the selected values
    * list.
    */
   JButton addAll = null;

   /**
    * Button to remove a value from the selected values list and replace it in the
    * available values list.
    */
   JButton remove = null;

   /**
    * Button to remove all the values from the selected values list and replace them in
    * the available values list.
    */
   JButton removeAll = null;

   /**
    * Text field to add object to the available list.
    */
   JTextField addField = null;

   /**
    * Button to add object to the available list.
    */
   JButton addAvailable = null;

   /**
    * Byte array to represent a right arrow icon for use on the Add button.
    */
   byte[] addArrow = {1, 1, 1, 1, 0, 0, 0, 0, 0,
         0, 1, 1, 1, 1, 0, 0, 0, 0,
         0, 0, 1, 1, 1, 1, 0, 0, 0,
         0, 0, 0, 1, 1, 1, 1, 0, 0,
         0, 0, 0, 0, 1, 1, 1, 1, 0,
         0, 0, 0, 0, 0, 1, 1, 1, 1,
         0, 0, 0, 0, 1, 1, 1, 1, 0,
         0, 0, 0, 1, 1, 1, 1, 0, 0,
         0, 0, 1, 1, 1, 1, 0, 0, 0,
         0, 1, 1, 1, 1, 0, 0, 0, 0,
         1, 1, 1, 1, 0, 0, 0, 0, 0};

   /**
    * A right arrow icon for use on the Add button.
    */
   ImageIcon addIcon = null;

   /**
    * Byte array to represent a left arrow icon for use on the Remove button.
    */
   byte[] removeArrow = {0, 0, 0, 0, 0, 1, 1, 1, 1,
         0, 0, 0, 0, 1, 1, 1, 1, 0,
         0, 0, 0, 1, 1, 1, 1, 0, 0,
         0, 0, 1, 1, 1, 1, 0, 0, 0,
         0, 1, 1, 1, 1, 0, 0, 0, 0,
         1, 1, 1, 1, 0, 0, 0, 0, 0,
         0, 1, 1, 1, 1, 0, 0, 0, 0,
         0, 0, 1, 1, 1, 1, 0, 0, 0,
         0, 0, 0, 1, 1, 1, 1, 0, 0,
         0, 0, 0, 0, 1, 1, 1, 1, 0,
         0, 0, 0, 0, 0, 1, 1, 1, 1};

   /**
    * A left arrow icon for use on the Remove button.
    */
   ImageIcon removeIcon = null;

   /**
    * Listener for dealing with actions on the buttons.
    */
   ButtonListener buttonListener = new ButtonListener ();

   /**
    * Object type string so that we can correctly define our labels.
    */
   String objectType = null;

   /**
    * Initialised flag. Set to true when the class has been correctly initialised.
    */
   boolean initialised = false;

   /**
    * Live mode flag. Set to true when the class is to act directly on some object.
    */
   boolean liveMode = false;

   /**
    * All controls flag. Set to true when the class is to have the Add / Remove All
    * controls. Default is true.
    */
   boolean allControls = true;

   /**
    * Single selection flag. Set to true when the class is used to add or remove
    * objects one at at time only. Default is false.
    */
   boolean singleSelection = false;

   /**
    * Default constructor
    */
   public ChooserPanel ()
   {
      availableLabel = new JLabel ("Available");

      selectedLabel = new JLabel ("Selected");

      objectType = new String ("Object");
   }

   /**
    * This method initialises the list of available values. The user will be responsible for
    * selecting some of the values and moving them to the selected list using the buttons.
    * @param available An array of values to place in the available list.
    * @param selected An array of values to place in the selected list.
    */
   public void initialise (Object[] available, Object[] selected)
   {
      if (initialised == false)
      {
         availableModel = new DefaultListModel ();
         selectedModel = new DefaultListModel ();

         sortedAvailableModel = new SortedListModel (availableModel);
         sortedSelectedModel = new SortedListModel (selectedModel);

         availableValues = new JList (sortedAvailableModel);
         selectedValues = new JList (sortedSelectedModel);

         if (available != null)
         {
            for (int i = 0; i < available.length; i++)
            {
               availableModel.addElement (available[i]);
            }
         }

         if (selected != null)
         {
            for (int i = 0; i < selected.length; i++)
            {
               selectedModel.addElement (selected[i]);
            }
         }

         initialised = true;
      }
   }

   /**
    * This method sets up the panel for our <code>ChooserPanel</code> object.
    * The panel consists of two lists, available and selected. It is possible to move values
    * in the lists between each list using the buttons.
    */
   void setupPanel ()
   {
      if (initialised == false)
      {
         return;
      }

      if (singleSelection)
      {
         availableValues.setSelectionMode (ListSelectionModel.SINGLE_SELECTION);
         selectedValues.setSelectionMode (ListSelectionModel.SINGLE_SELECTION);
      }

      this.setLayout (new GridBagLayout ());

      JPanel availablePanel = new JPanel ();
      JPanel buttonPanel = new JPanel ();
      JPanel selectedPanel = new JPanel ();

      availablePanel.setLayout (new BorderLayout (0, 3));
      buttonPanel.setLayout (new BorderLayout ());
      selectedPanel.setLayout (new BorderLayout (0, 3));

      availableLabel.setHorizontalAlignment (SwingConstants.CENTER);

      availablePanel.add (availableLabel, BorderLayout.NORTH);

      availableValues.setPrototypeCellValue ("prototype");

      JScrollPane availableValuesScroll = new JScrollPane (availableValues,
            JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
            JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

      availablePanel.add (availableValuesScroll, BorderLayout.CENTER);

      JPanel addAvailablePanel = new JPanel ();
      JPanel subAvailablePanel = new JPanel ();

      addAvailablePanel.setLayout (new BorderLayout (3, 3));
      subAvailablePanel.setLayout (new BorderLayout ());

      addField = new JTextField ();

      subAvailablePanel.add (addField, BorderLayout.NORTH);
      addAvailablePanel.add (subAvailablePanel, BorderLayout.CENTER);

      addAvailable = new JButton ("Add");

      addAvailablePanel.add (addAvailable, BorderLayout.EAST);

      availablePanel.add (addAvailablePanel, BorderLayout.SOUTH);

      addAvailable.addActionListener (buttonListener);

      add = new JButton ("Add");
      remove = new JButton ("Remove");

      /* Set up two icons from the byte array information that we have defined. */
      byte[] red = {(byte) 255, (byte) 0};
      byte[] green = {(byte) 255, (byte) 0};
      byte[] blue = {(byte) 255, (byte) 0};

      DataBufferByte addArrowBufferByte = new DataBufferByte (addArrow, addArrow.length);
      DataBufferByte removeArrowBufferByte = new DataBufferByte (removeArrow, removeArrow.length);
      Raster addArrowRaster = Raster.createInterleavedRaster (addArrowBufferByte, 9, 11, 9, 1, new int[] {0}, null);
      Raster removeArrowRaster = Raster.createInterleavedRaster (removeArrowBufferByte, 9, 11, 9, 1, new int[] {0}, null);
      IndexColorModel icm = new IndexColorModel (8, 2, red, green, blue, 0);
      BufferedImage addArrowImage = new BufferedImage (9, 11, BufferedImage.TYPE_BYTE_INDEXED, icm);
      BufferedImage removeArrowImage = new BufferedImage (9, 11, BufferedImage.TYPE_BYTE_INDEXED, icm);
      addArrowImage.setData (addArrowRaster);
      removeArrowImage.setData (removeArrowRaster);
      addIcon = new ImageIcon (addArrowImage);
      removeIcon = new ImageIcon (removeArrowImage);

      add.setIcon (addIcon);
      add.setHorizontalTextPosition (SwingConstants.LEFT);
      add.setVerticalTextPosition (SwingConstants.CENTER);
      remove.setIcon (removeIcon);
      remove.setVerticalTextPosition (SwingConstants.CENTER);

      JPanel subButtonPanel = new JPanel ();

      add.addActionListener (buttonListener);
      remove.addActionListener (buttonListener);

      if (allControls)
      {
         subButtonPanel.setLayout (new GridLayout (4, 1, 3, 3));

         addAll = new JButton ("Add All");
         removeAll = new JButton ("Remove All");

         addAll.addActionListener (buttonListener);
         removeAll.addActionListener (buttonListener);

         subButtonPanel.add (add);
         subButtonPanel.add (addAll);
         subButtonPanel.add (remove);
         subButtonPanel.add (removeAll);
      }
      else
      {
         subButtonPanel.setLayout (new GridLayout (2, 1, 3, 3));

         subButtonPanel.add (add);
         subButtonPanel.add (remove);
      }

      buttonPanel.add (subButtonPanel, BorderLayout.NORTH);

      selectedLabel.setHorizontalAlignment (SwingConstants.CENTER);

      selectedPanel.add (selectedLabel, BorderLayout.NORTH);

      selectedValues.setPrototypeCellValue ("prototype");

      JScrollPane selectedValuesScroll = new JScrollPane (selectedValues,
            JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
            JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

      selectedPanel.add (selectedValuesScroll, BorderLayout.CENTER);

      GridBagConstraints gbc = new GridBagConstraints ();
      gbc.anchor = GridBagConstraints.CENTER;
      gbc.insets = new Insets (3, 3, 3, 3);

      gbc.gridx = 0; gbc.gridy = 0; gbc.weightx = 1.0; gbc.weighty = 1.0;
      gbc.fill = GridBagConstraints.BOTH;
      this.add (availablePanel, gbc);
      gbc.gridx = 1; gbc.gridy = 0; gbc.weightx = 0.0; gbc.weighty = 0.0;
      gbc.fill = GridBagConstraints.NONE;
      this.add (buttonPanel, gbc);
      gbc.gridx = 2; gbc.gridy = 0; gbc.weightx = 1.0; gbc.weighty = 1.0;
      gbc.fill = GridBagConstraints.BOTH;
      this.add (selectedPanel, gbc);
   }

   /**
    * This method sets the live mode flag that means the buttons will act directly
    * on some object.
    */
   public void setLiveMode (boolean mode)
   {
      liveMode = mode;
   }

   /**
    * This method should be called when the Add button is clicked. It will remove the
    * selected values from the Available list and add them to the Selected list.
    */
   public void add ()
   {
      int[] selected = sortedAvailableModel.toUnsortedModelIndices (availableValues.getSelectedIndices ());
      Object[] objects = new Object [selected.length];

      for (int i = 0; i < selected.length; i++)
      {
         objects[i] = availableModel.get (selected[i]);
         selectedModel.addElement (objects[i]);
      }

      for (int i = selected.length - 1; i >= 0 ; i--)
      {
         availableModel.removeElementAt (selected[i]);
      }

      if (liveMode)
      {
         liveAdd (objects);
      }
   }

   /**
    * This method should be called when the Add All button is clicked. It will remove
    * all values from the Available list and add them to the Selected list.
    */
   public void addAll ()
   {
      Object[] selected = availableModel.toArray ();

      availableModel.removeAllElements ();

      for (int i = 0; i < selected.length; i++)
      {
         selectedModel.addElement (selected[i]);
      }

      if (liveMode)
      {
         liveAdd (selected);
      }
   }

   /**
    * This method should be called when the Remove button is clicked. It will remove the
    * selected values from the Selected list and replace them in the Available list.
    */
   public void remove ()
   {
      int[] selected = sortedSelectedModel.toUnsortedModelIndices (selectedValues.getSelectedIndices ());
      Object[] objects = new Object [selected.length];

      for (int i = 0; i < selected.length; i++)
      {
         objects[i] = selectedModel.get (selected[i]);
         availableModel.addElement (objects[i]);
      }

      for (int i = selected.length - 1; i >= 0 ; i--)
      {
         selectedModel.removeElementAt (selected[i]);
      }

      if (liveMode)
      {
         liveRemove (objects);
      }
   }

   /**
    * This method should be called when the Remove All button is clicked. It
    * will remove all values from the Selected list and replace them in the
    * Available list.
    */
   public void removeAll ()
   {
      Object[] selected = selectedModel.toArray ();

      selectedModel.removeAllElements ();

      for (int i = 0; i < selected.length; i++)
      {
         availableModel.addElement (selected[i]);
      }

      if (liveMode && selected.length > 0)
      {
         liveRemove (selected);
      }
   }

   /**
    * This method will add the value that is currently entered in the addField
    * to the list of available values. It will call the abstract method
    * checkNewValue() so that implementing classes can verify the object.
    */
   public void addAvailable ()
   {
      Object object = (Object) addField.getText ();

      if (!(availableModel.contains (object)))
      {
         availableModel.addElement (object);
      }
      addField.setText ("");
   }

   /**
    * Returns an array of the objects that are currently in the selected list.
    *
    * @return The list of selected objects.
    */
   public Object[] getSelectedObjects ()
   {
      Object[] objects = selectedModel.toArray ();

      return objects;
   }

   /**
    * Takes a list of objects and places them in the current selected list,
    * moving the objects from/to the available list where appropriate.
    *
    * @param objects The list of objects.
    */
   public void setSelectedObjects (Object[] objects)
   {
      removeAll ();

      for (int i = 0; i < objects.length; i++)
      {
         selectedModel.addElement (objects[i]);

         if (availableModel.indexOf (objects[i]) != -1)
         {
            availableModel.removeElement (objects[i]);
         }
      }
   }

   /**
    * This method enables or disables the buttons in the panel depending on
    * the value passed to the method.
    * @param enabled The enabled flag.
    */
   public void setEnabled (boolean enabled)
   {
      add.setEnabled (enabled);
      remove.setEnabled (enabled);

      if (allControls)
      {
         addAll.setEnabled (enabled);
         removeAll.setEnabled (enabled);
      }
   }

   /**
    * This method should be overridden for the behaviour when live mode is
    * set to true and the Add or Add All button is pushed. The idea is that
    * an object should have a change made to it when the Add button is pushed.
    * @param objects The objects.
    */
   public abstract void liveAdd (Object[] objects);

   /**
    * This method should be overridden for the behaviour when live mode is
    * set to true and the Remove or  All button is pushed. The idea is that
    * an object should have a change made to it when the Add button is pushed.
    * @param objects The objects.
    */
   public abstract void liveRemove (Object[] objects);

   /**
    * Inner class to handle actions on the buttons.
    */
   public class ButtonListener implements ActionListener
   {
      /**
       * This method is invoked when a button is clicked that has had this listener
       * added to it as an ActionListener.
       *
       * @param e The event.
       */
      public void actionPerformed (ActionEvent e)
      {
         if (((JButton) e.getSource ()).equals (add))
         {
            if (availableValues.isSelectionEmpty ())
            {
               if (singleSelection)
               {
                  JOptionPane.showMessageDialog (ChooserPanel.this,
                        "Please select a " + objectType + " from the Available list.",
                        "Empty selection", JOptionPane.WARNING_MESSAGE);
               }
               else
               {
                  JOptionPane.showMessageDialog (ChooserPanel.this,
                        "Please select one or more " + objectType + "s from the Available list.",
                        "Empty selection", JOptionPane.WARNING_MESSAGE);
               }
            }
            else
            {
               add ();
            }
         }
         else if (((JButton) e.getSource ()).equals (addAll))
         {
            addAll ();
         }
         else if (((JButton) e.getSource ()).equals (remove))
         {
            if (selectedValues.isSelectionEmpty ())
            {
               if (singleSelection)
               {
                  JOptionPane.showMessageDialog (ChooserPanel.this,
                        "Please select a " + objectType + " from the Selected list.",
                        "Empty selection", JOptionPane.WARNING_MESSAGE);
               }
               else
               {
                  JOptionPane.showMessageDialog (ChooserPanel.this,
                        "Please select one or more " + objectType + "s from the Selected list.",
                        "Empty selection", JOptionPane.WARNING_MESSAGE);
               }
            }
            else
            {
               remove ();
            }
         }
         else if (((JButton) e.getSource ()).equals (removeAll))
         {
            removeAll ();
         }
         else if (((JButton) e.getSource ()).equals (addAvailable))
         {
            addAvailable ();
         }
      }
   }
}
