/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.recording.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.awt.image.DataBufferByte;
import java.awt.image.IndexColorModel;
import java.awt.image.Raster;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;
import javax.xml.parsers.ParserConfigurationException;

import org.omg.CORBA.ORB;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import com.thalesgroup.CdmwRecording.ContainerManager;
import com.thalesgroup.CdmwRecording.ContainerManagerHelper;
import com.thalesgroup.CdmwRecording.DaemonControl;
import com.thalesgroup.CdmwRecording.DaemonControlHelper;
import com.thalesgroup.CdmwRecording.Running_State;
import com.thalesgroup.CdmwRecording.State_Data;

/**
 * This is the Cardamom recording service control browser.
 *
 * Copyright (c) 2007 PrismTech Ltd. All rights Reserved.
 *
 * @author Paul Graham
 */
public class ControlBrowser extends JPanel
{
   //oa int current_recorder = 0;

   /**
    * Generated SerialVersionUID.
    */
   private static final long serialVersionUID = -7420966844055415050L;

   /**
    * The orb instance that we use to look up our daemons.
    */
   static ORB orb = null;

   /**
    * Static reference to the ControlBrowser instance.
    */
   static ControlBrowser controlBrowser = null;

   /**
    * The menu bar for the top of the browser.
    */
   JMenuBar menuBar = new JMenuBar ();

   /**
    * The file menu and file menu items.
    */
   JMenu fileMenu = new JMenu ("File");

   JMenuItem fileMenuExit = new JMenuItem ("Exit");

   JMenuItem fileMenuItems[] = { fileMenuExit };

   /**
    * The panel we display the start & stop controls in.
    */
   JPanel controlPanel = new JPanel ();

   /**
    * The panel we display the interface & operation controls in.
    */
   JPanel detailsPanel = new JPanel ();

   /**
    * The panel we display the start, stop and terminate controls in.
    */
   JPanel startStopPanel = new JPanel ();

   /**
    * The panel we display the information about the recording daemon instance
    * in.
    */
   JPanel infoPanel = new JPanel ();

   /**
    * The panel we display the status information about the recording daemon instance.
    */
   JPanel statusPanel = new JPanel ();

   /**
    * The start button.
    */
   JButton startButton = null;

   /**
    * The stop button.
    */
   JButton stopButton = null;

   /**
    * The combobox for host selection.
    */
   JComboBox hostCombo = null;

   /**
    * The terminate button.
    */
   JButton terminateButton = null;

   /**
    * The field where the port is displayed.
    */
   JTextField portField = null;

   /**
    * The field where the recorder URL is displayed.
    */
   //oa JTextField urlField = null;
   JComboBox urlComboBox = null;
   int currentIndex = -1;

   /**
    * The field where the recorder URL is displayed.
    */
   JTextField contMgrField = null;

   /**
    * The field where the debug file is displayed.
    */
   JTextField fileField = null;

   /**
    * The field where the maximum size is displayed.
    */
   JTextField maxSizeField = null;

   /**
    * The field where the maximum buffer size is displayed.
    */
   JTextField maxBufferSizeField = null;

   /**
    * The field where the timeout is displayed.
    */
   JTextField timeoutField = null;

   /**
    * The field where the retry delay is displayed.
    */
   JTextField retryDelayField = null;

   /**
    * The browse button to browse for a debug file.
    */
   JButton browseButton = null;

   /**
    * The lookup button to look up an instance of the recording daemon.
    */
   JButton lookupButton = null;

   /**
    * The apply button to apply any changes to the instance of the recording
    * daemon.
    */
   JButton applyButton = null;

   /**
    * The reset button to reset all the fields.
    */
   JButton resetButton = null;

   /**
    * The label we use to display status information.
    */
   JLabel statusLabel = null;

   /**
    * File chooser for browsing for the debug file.
    */
   JFileChooser browser = null;

   /**
    * Byte array to represent a start icon for use on the Start button.
    */
   byte[] startArray = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 0, 0, 2, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 0, 0, 2, 2, 2, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 0, 0, 2, 2, 2, 2, 2, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0, 0, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 0, 0, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 1, 1, 2, 2, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};

   /**
    * Start icon for use on the Start button.
    */
   ImageIcon startIcon = null;

   /**
    * Byte array to represent a stop icon for use on the Stop button.
    */
   byte[] stopArray = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 0, 0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 0, 0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0,
         0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};

   /**
    * Stop icon for use on the Stop button.
    */
   ImageIcon stopIcon = null;

   /**
    * Byte array to represent a terminate icon for use on the Terminate button.
    */
   byte[] terminateArray = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 0, 0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 0, 0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 0, 2, 2, 0, 0, 0, 0, 2, 2, 0, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 0, 0, 2, 2, 0, 0, 0, 0, 2, 2, 0, 0, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 0, 0, 0, 0, 2, 0, 0, 0, 0, 2, 0, 0, 0, 0, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 0, 0, 0, 2, 2, 0, 0, 0, 0, 2, 2, 0, 0, 0, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 0, 0, 0, 0, 2, 2, 0, 0, 0, 0, 2, 2, 0, 0, 0, 0, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 0, 0, 0, 2, 2, 2, 2, 0, 0, 2, 2, 2, 2, 0, 0, 0, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 0, 0, 0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 0, 0, 0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 0, 0, 0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0, 0, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 0, 0, 0, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 0, 0, 0, 0, 0, 2, 2, 2, 2, 0, 0, 0, 0, 0, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
         0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0,
         0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};

   /**
    * Terminate icon for use on the Terminate button.
    */
   ImageIcon terminateIcon = null;

   /**
    * MouseListener for double clicks on host and port fields.
    */
   MyMouseListener mouseListener = new MyMouseListener ();

   /**
    * NumberFieldListener for actions on fields that will hold numbers.
    */
   NumberFieldListener numberFieldListener = new NumberFieldListener ();

   /**
    * NumberFieldFocusListener for loss of focus on fields that hold numbers.
    */
   NumberFieldFocusListener numberFieldFocusListener = new NumberFieldFocusListener ();

   /**
    * The panel that displays the interface where interfaces are chosen.
    */
   InterfaceChooserPanel interfaceChooserPanel = null;

   /**
    * The panel that displays the interface where operations are chosen.
    */
   OperationChooserPanel operationChooserPanel = null;

   /**
    * The panel that displays the interface where ports are chosen.
    */
   PortChooserPanel portChooserPanel = null;

   /**
    * Array of interfaces that have been made available in the configuration.
    */
   String[] interfaces = null;

   /**
    * Array of operations that have been made available in the configuration.
    */
   String[] operations = null;

   /**
    * Array of ports that have been made available in the configuration.
    */
   Short[] ports = null;

   /**
    * The hashmap of hosts/ports that we are allowed to configure.
    */
   HashMap<String, Integer> hosts = new HashMap<String, Integer> ();

   /**
    * The current host.
    */
   String currentHost = null;

   /**
    * The current port.
    */
   int currentPort = 0;

   /**
    * The instance of the Recording Daemon that the GUI is controling.
    */
   DaemonControl daemon = null;
   ContainerManager contMgr = null;
   /**
    * The last known state of the Recording Daemon that the GUI is controling.
    */
   State_Data current_daemonState = null;

   /**
    * Boolean flag. True if the unit tests are to be ran.
    */
   static boolean testFlag = false;

   /**
    * Default constructor.
    */
   public ControlBrowser ()
   {
   }

   /**
    * Read in the configuration data from the supplied arguments.
    */
   void readConfiguration (String[] args)
   {
      try
      {
         URL xmlFile = new URL (args[0]);
         HostParser hp = new HostParser (xmlFile);
         if (hp.parse ())
         {
            hosts = hp.getHosts ();
         }
         else
         {
            System.out.println ("First file does not contain the host list.");
            System.exit (0);
         }
      }
      catch (MalformedURLException mex)
      {
         System.out.println ("Malformed URL reading the host xml file : " + mex);
         System.exit (1);
      }
      catch (SAXParseException spe)
      {
         System.out.println ("SAXParseException reading the host xml file : " + spe);
         System.exit (1);
      }
      catch (SAXException sxe)
      {
         System.out.println ("SAXException reading the host xml file : " + sxe);
         System.exit (1);
      }
      catch (ParserConfigurationException pce)
      {
         System.out.println ("ParserConfigurationException reading the host xml file : " + pce);
         System.exit (1);         
      }
      catch (java.io.IOException ioex)
      {
         System.out.println ("IOException reading the host xml file : " + ioex);
         //System.exit (1);
      }

      try
      {
         URL xmlFile = new URL (args[1]);
         InterfaceParser ip = new InterfaceParser (xmlFile);
         if (ip.parse ())
         {
            interfaces = ip.getInterfaces ();
         }
         else
         {
            System.out.println ("First file does not contain the interface list.");
            System.exit (0);
         }
      }
      catch (MalformedURLException mex)
      {
         System.out.println ("Malformed URL reading the interface xml file : " + mex);
         System.exit (1);
      }
      catch (SAXParseException spe)
      {
         System.out.println ("SAXParseException reading the interface xml file : " + spe);
         System.exit (1);
      }
      catch (SAXException sxe)
      {
         System.out.println ("SAXException reading the interface xml file : " + sxe);
         System.exit (1);
      }
      catch (ParserConfigurationException pce)
      {
         System.out.println ("ParserConfigurationException reading the interface xml file : " + pce);
         System.exit (1);         
      }
      catch (java.io.IOException ioex)
      {
         System.out.println ("IOException reading the interface xml file : " + ioex);
         //System.exit (1);
      }

      try
      {
         URL xmlFile = new URL (args[2]);
         OperationParser op = new OperationParser (xmlFile);
         if (op.parse ())
         {
            operations = op.getOperations ();
         }
         else
         {
            System.out.println ("First file does not contain the operation list.");
            System.exit (0);
         }
      }
      catch (MalformedURLException mex)
      {
         System.out.println ("Malformed URL reading the operation xml file : " + mex);
         System.exit (1);
      }
      catch (SAXParseException spe)
      {
         System.out.println ("SAXParseException reading the operation xml file : " + spe);
         System.exit (1);
      }
      catch (SAXException sxe)
      {
         System.out.println ("SAXException reading the operation xml file : " + sxe);
         System.exit (1);
      }
      catch (ParserConfigurationException pce)
      {
         System.out.println ("ParserConfigurationException reading the operation xml file : " + pce);
         System.exit (1);         
      }
      catch (java.io.IOException ioex)
      {
         System.out.println ("IOException reading the operation xml file : " + ioex);
         //System.exit (1);
      }

      try
      {
         URL xmlFile = new URL (args[3]);
         PortParser pp = new PortParser (xmlFile);
         if (pp.parse ())
         {
            ports = pp.getPorts ();
         }
         else
         {
            System.out.println ("First file does not contain the port list.");
            System.exit (0);
         }
      }
      catch (MalformedURLException mex)
      {
         System.out.println ("Malformed URL reading the port xml file : " + mex);
         System.exit (1);
      }
      catch (SAXParseException spe)
      {
         System.out.println ("SAXParseException reading the port xml file : " + spe);
         System.exit (1);
      }
      catch (SAXException sxe)
      {
         System.out.println ("SAXException reading the port xml file : " + sxe);
         System.exit (1);
      }
      catch (ParserConfigurationException pce)
      {
         System.out.println ("ParserConfigurationException reading the port xml file : " + pce);
         System.exit (1);         
      }
      catch (java.io.IOException ioex)
      {
         System.out.println ("IOException reading the port xml file : " + ioex);
         //System.exit (1);
      }
   }

   /**
    * This method sets up the panel for our <code>ControlBrowser</code>
    * object. The panel consists of two areas, the menu bar and the
    * tree / properties panel.
    */
   void setupPanel ()
   {
      this.setLayout (new BorderLayout ());

      buildMenu ();

      this.add (menuBar, BorderLayout.NORTH);

      controlPanel.setLayout (new BorderLayout ());
      detailsPanel.setLayout (new BorderLayout ());

      controlPanel.setBorder (BorderFactory.createTitledBorder ("Recorder Daemon"));
      detailsPanel.setBorder (BorderFactory.createTitledBorder ("Details"));

      JPanel buttonPanel = new JPanel ();

      buttonPanel.setBorder (new EmptyBorder (3, 12, 3, 12));

      buttonPanel.setLayout (new BorderLayout ());

      JPanel gridButtonPanel = new JPanel ();

      gridButtonPanel.setLayout (new GridLayout (3, 1, 10, 3));

      byte[] red = new byte[] {(byte) 255, (byte) 50, (byte) 50};
      byte[] green = new byte[] {(byte) 255, (byte) 50, (byte) 180};
      byte[] blue = new byte[] {(byte) 255, (byte) 50, (byte) 50};

      DataBufferByte startBufferByte = new DataBufferByte (startArray, startArray.length);
      Raster startRaster = Raster.createInterleavedRaster (startBufferByte, 30, 30, 30, 1, new int[] {0}, null);
      IndexColorModel icm = new IndexColorModel (8, 3, red, green, blue, 0);
      BufferedImage startImage = new BufferedImage (30, 30, BufferedImage.TYPE_BYTE_INDEXED, icm);
      startImage.setData (startRaster);
      startIcon = new ImageIcon (startImage);

      startButton = new JButton (startIcon);

      startButton.setToolTipText ("Start");

      red = new byte[] {(byte) 255, (byte) 50, (byte) 180};
      green = new byte[] {(byte) 255, (byte) 50, (byte) 50};
      blue = new byte[] {(byte) 255, (byte) 50, (byte) 50};
      DataBufferByte stopBufferByte = new DataBufferByte (stopArray, stopArray.length);
      DataBufferByte terminateBufferByte = new DataBufferByte (terminateArray, terminateArray.length);
      Raster stopRaster = Raster.createInterleavedRaster (stopBufferByte, 30, 30, 30, 1, new int[] {0}, null);
      Raster terminateRaster = Raster.createInterleavedRaster (terminateBufferByte, 30, 30, 30, 1, new int[] {0}, null);
      icm = new IndexColorModel (8, 3, red, green, blue, 0);
      BufferedImage stopImage = new BufferedImage (30, 30, BufferedImage.TYPE_BYTE_INDEXED, icm);
      BufferedImage terminateImage = new BufferedImage (30, 30, BufferedImage.TYPE_BYTE_INDEXED, icm);
      stopImage.setData (stopRaster);
      terminateImage.setData (terminateRaster);
      stopIcon = new ImageIcon (stopImage);
      terminateIcon = new ImageIcon (terminateImage);

      stopButton = new JButton (stopIcon);

      stopButton.setToolTipText ("Stop");

      terminateButton = new JButton (terminateIcon);

      terminateButton.setToolTipText ("Terminate");

      Insets noMargin = new Insets (0, 0, 0, 0);
      startButton.setMargin (noMargin);
      stopButton.setMargin (noMargin);
      terminateButton.setMargin (noMargin);

      gridButtonPanel.add (startButton);
      gridButtonPanel.add (stopButton);
      gridButtonPanel.add (terminateButton);

      buttonPanel.add (gridButtonPanel, BorderLayout.NORTH);

      ButtonListener buttonListener = new ButtonListener ();

      startButton.addActionListener (buttonListener);
      stopButton.addActionListener (buttonListener);
      terminateButton.addActionListener (buttonListener);

      JPanel textBoxPanel = new JPanel ();

      textBoxPanel.setLayout (new GridBagLayout ());

      JLabel hostLabel = new JLabel ("Recorder Daemon Host :");
      JLabel portLabel = new JLabel ("Recorder Daemon Port :");
      JLabel urlLabel = new JLabel  ("Recording Servers URLs :");
      JLabel contMgrLabel = new JLabel ("URL of Object to record :");
      JLabel fileLabel = new JLabel ("Debug File :");
      JLabel maxSizeLabel = new JLabel ("Max Size (bytes):");
      JLabel maxBufferSizeLabel = new JLabel ("Max Buffer Number (buffer) :");
      JLabel timeoutLabel = new JLabel ("Timeout (ms) :");
      JLabel retryDelayLabel = new JLabel ("Connection Retry Delay (ms):");

      hostCombo = new JComboBox (hosts.keySet ().toArray ());
      portField = new JTextField (40);
      contMgrField = new JTextField (40);

      //oa urlField = new JTextField (40);
      urlComboBox = new JComboBox();
      urlComboBox.setMaximumRowCount(3);
      urlComboBox.setEnabled(false);

      urlComboBox.addActionListener (new ActionListener(){
         public void actionPerformed(ActionEvent e)
         {
            if (e.getActionCommand().equalsIgnoreCase("comboBoxEdited"))
            {
               String newUrl = urlComboBox.getSelectedItem ().toString (); 
               System.out.println ("url=" + newUrl + " - " + e.getActionCommand() + " index=" + currentIndex);
               urlComboBox.removeItemAt(currentIndex);
               urlComboBox.insertItemAt(newUrl, currentIndex);
            }
            else
            {
               //corbaloc:iiop:1.2@192.168.1.61:12344/Recorder
               if ( urlComboBox.getItemCount() > 0)
               {
                  //String xx = urlComboBox.getSelectedItem ().toString ();
                  if ( urlComboBox.getSelectedIndex() > -1)
                  {
                     currentIndex = urlComboBox.getSelectedIndex();
                  }
                  //System.out.println ("url=" + xx + " - " + e.getActionCommand() + " index=" + currentIndex);    
               }
            }
         }

      });

      fileField = new JTextField (40);
      maxSizeField = new JTextField (40);
      maxBufferSizeField = new JTextField (40);
      timeoutField = new JTextField (40);
      retryDelayField = new JTextField (40);
      browseButton = new JButton ("Browse");

      JPanel filePanel = new JPanel ();

      filePanel.setLayout (new BorderLayout (3, 0));

      JPanel subFilePanel = new JPanel ();

      subFilePanel.setLayout (new BorderLayout ());

      subFilePanel.add (fileField, BorderLayout.NORTH);

      filePanel.add (subFilePanel, BorderLayout.CENTER);
      filePanel.add (browseButton, BorderLayout.EAST);

      browseButton.addActionListener (buttonListener);

      hostCombo.setEditable (false);
      hostCombo.addActionListener (new HostComboBoxListener ());
      hostCombo.addMouseListener (mouseListener);
      portField.setEditable (true);
      portField.addActionListener (numberFieldListener);
      portField.addFocusListener (numberFieldFocusListener);
      portField.addMouseListener (mouseListener);

      GridBagConstraints gbc = new GridBagConstraints ();
      gbc.anchor = GridBagConstraints.NORTHWEST;
      gbc.insets = new Insets (3, 3, 3, 3);

      gbc.gridx = 0; gbc.gridy = 0; gbc.weightx = 0.0;
      textBoxPanel.add (hostLabel, gbc);
      gbc.gridx = 1; gbc.gridy = 0; gbc.weightx = 1.0;
      gbc.fill = GridBagConstraints.HORIZONTAL;
      textBoxPanel.add (hostCombo, gbc);
      gbc.gridx = 2; gbc.gridy = 0; gbc.weightx = 0.0;
      gbc.fill = GridBagConstraints.NONE;
      textBoxPanel.add (portLabel, gbc);
      gbc.gridx = 3; gbc.gridy = 0; gbc.weightx = 1.0;
      gbc.fill = GridBagConstraints.HORIZONTAL;
      textBoxPanel.add (portField, gbc);

      gbc.gridx = 0; gbc.gridy = 1; gbc.weightx = 0.0;
      textBoxPanel.add (maxSizeLabel, gbc);
      gbc.gridx = 1; gbc.gridy = 1; gbc.weightx = 1.0;
      gbc.fill = GridBagConstraints.HORIZONTAL;
      textBoxPanel.add (maxSizeField, gbc);
      gbc.gridx = 2; gbc.gridy = 1; gbc.weightx = 0.0;
      gbc.fill = GridBagConstraints.NONE;
      textBoxPanel.add (maxBufferSizeLabel, gbc);
      gbc.gridx = 3; gbc.gridy = 1; gbc.weightx = 1.0;
      gbc.fill = GridBagConstraints.HORIZONTAL;
      textBoxPanel.add (maxBufferSizeField, gbc);

      gbc.gridx = 0; gbc.gridy = 2; gbc.weightx = 0.0;
      textBoxPanel.add (timeoutLabel, gbc);
      gbc.gridx = 1; gbc.gridy = 2; gbc.weightx = 1.0;
      gbc.fill = GridBagConstraints.HORIZONTAL;
      textBoxPanel.add (timeoutField, gbc);
      gbc.gridx = 2; gbc.gridy = 2; gbc.weightx = 0.0;
      gbc.fill = GridBagConstraints.NONE;
      textBoxPanel.add (retryDelayLabel, gbc);
      gbc.gridx = 3; gbc.gridy = 2; gbc.weightx = 1.0;
      gbc.fill = GridBagConstraints.HORIZONTAL;
      textBoxPanel.add (retryDelayField, gbc);

      gbc.gridx = 0; gbc.gridy = 3; gbc.weightx = 0.0;
      gbc.gridwidth = 1;
      textBoxPanel.add (urlLabel, gbc);
      gbc.gridx = 1; gbc.gridy = 3; gbc.weightx = 1.0;
      gbc.gridwidth = 3;
      gbc.fill = GridBagConstraints.HORIZONTAL;
      //oa textBoxPanel.add (urlField, gbc);
      textBoxPanel.add (urlComboBox, gbc);

      gbc.gridx = 0; gbc.gridy = 4; gbc.weightx = 0.0;
      gbc.gridwidth = 1;
      textBoxPanel.add (contMgrLabel, gbc);
      gbc.gridx = 1; gbc.gridy = 4; gbc.weightx = 1.0;
      gbc.gridwidth = 3;
      gbc.fill = GridBagConstraints.HORIZONTAL;
      textBoxPanel.add (contMgrField, gbc);

      gbc.gridx = 0; gbc.gridy = 5; gbc.weightx = 0.0;
      gbc.gridwidth = 1;
      gbc.fill = GridBagConstraints.NONE;
      textBoxPanel.add (fileLabel, gbc);
      gbc.gridx = 1; gbc.gridy = 5; gbc.weightx = 1.0;
      gbc.fill = GridBagConstraints.HORIZONTAL;
      gbc.gridwidth = 3;
      textBoxPanel.add (filePanel, gbc);

      lookupButton = new JButton ("Lookup");
      lookupButton.setToolTipText ("Re-connect to daemon");
      applyButton = new JButton ("Apply");
      applyButton.setToolTipText ("Apply changes to connected daemon");
      resetButton = new JButton ("Reset");
      resetButton.setToolTipText ("Reset");

      JPanel outerLowerButtonPanel = new JPanel ();
      JPanel lowerButtonPanel = new JPanel ();
      JPanel subButtonPanel = new JPanel ();

      outerLowerButtonPanel.setLayout (new BorderLayout ());
      outerLowerButtonPanel.setBorder (new EmptyBorder (3, 3, 3, 3));
      lowerButtonPanel.setLayout (new BorderLayout ());
      subButtonPanel.setLayout (new GridLayout (1, 3, 3, 3));

      subButtonPanel.add (lookupButton);
      subButtonPanel.add (applyButton);
      subButtonPanel.add (resetButton);

      lookupButton.addActionListener (buttonListener);
      applyButton.addActionListener (buttonListener);
      resetButton.addActionListener (buttonListener);

      lowerButtonPanel.add (subButtonPanel, BorderLayout.EAST);

      outerLowerButtonPanel.add (lowerButtonPanel, BorderLayout.NORTH);

      JPanel textBoxWrapperPanel = new JPanel ();

      textBoxWrapperPanel.setLayout (new BorderLayout ());

      textBoxWrapperPanel.add (textBoxPanel, BorderLayout.NORTH);

      textBoxWrapperPanel.add (outerLowerButtonPanel, BorderLayout.CENTER);

      controlPanel.add (buttonPanel, BorderLayout.WEST);
      controlPanel.add (textBoxWrapperPanel, BorderLayout.CENTER);

      JPanel interfacePanel = new JPanel ();
      JPanel operationPanel = new JPanel ();
      JPanel portPanel = new JPanel ();

      interfacePanel.setLayout (new BorderLayout ());
      operationPanel.setLayout (new BorderLayout ());
      portPanel.setLayout (new BorderLayout ());

      interfaceChooserPanel = new InterfaceChooserPanel ();
      operationChooserPanel = new OperationChooserPanel ();
      portChooserPanel = new PortChooserPanel ();

      String[] selected = new String[0];

      interfaceChooserPanel.initialise (interfaces, selected);
      operationChooserPanel.initialise (operations, selected);
      portChooserPanel.initialise (ports, selected);

      interfaceChooserPanel.setupPanel ();
      operationChooserPanel.setupPanel ();
      portChooserPanel.setupPanel ();

      interfacePanel.add (interfaceChooserPanel, BorderLayout.CENTER);
      operationPanel.add (operationChooserPanel, BorderLayout.CENTER);
      portPanel.add (portChooserPanel, BorderLayout.CENTER);

      JTabbedPane detailsPane = new JTabbedPane ();

      detailsPane.add ("Interfaces", interfacePanel);
      detailsPane.add ("Operations", operationPanel);
      detailsPane.add ("Ports", portPanel);

      detailsPanel.add (detailsPane, BorderLayout.CENTER);
      JPanel mainPanel = new JPanel ();

      statusPanel = new JPanel ();
      statusPanel.setLayout (new BorderLayout ());
      statusPanel.setBorder (new EmptyBorder (3, 3, 3, 3));
      statusLabel = new JLabel ("Daemon disconnected : Unconfigured");
      statusPanel.add (statusLabel, BorderLayout.EAST);

      mainPanel.setLayout (new BorderLayout ());

      mainPanel.add (controlPanel, BorderLayout.NORTH);
      mainPanel.add (detailsPanel, BorderLayout.CENTER);
      mainPanel.add (statusPanel, BorderLayout.SOUTH);

      this.add (mainPanel, BorderLayout.CENTER);

      nullDaemon ();

      if (hosts.size () > 0)
      {
         hostCombo.setSelectedIndex (0);
      }
   }

   /**
    * This method looks up the current daemon from the details in the host
    * and port fields.
    */
   void lookup ()
   {
      currentHost = (String) hostCombo.getSelectedItem ();

      try
      {
         currentPort = Integer.valueOf (portField.getText ());
      }
      catch (NumberFormatException ex)
      {
         JOptionPane.showMessageDialog (ControlBrowser.this, "Invalid port value.", "Invalid input",
               JOptionPane.ERROR_MESSAGE);
         return;
      }

      String corbaloc = new String ("corbaloc::" + currentHost + ":" + currentPort + "/RecordingDaemonController" );
      String corbaloc_contMgr = new String ("corbaloc::" + currentHost + ":" + currentPort + "/RecordingDaemonContainerMgr" );

      try
      {
         org.omg.CORBA.Object o = orb.string_to_object (corbaloc);
         org.omg.CORBA.Object o_contMgr = orb.string_to_object (corbaloc_contMgr);

         daemon  = DaemonControlHelper.narrow (o);
         contMgr = ContainerManagerHelper.narrow (o_contMgr);
         newDaemon (false);
      }
      catch (org.omg.CORBA.SystemException ex)
      {
         JOptionPane.showMessageDialog (ControlBrowser.this, "Unable to obtain reference to Recording Daemon. Please check the configuration.",
               "Lookup failed.", JOptionPane.ERROR_MESSAGE);
         ex.printStackTrace ();
         nullDaemon ();
      }
   }

   /**
    * This method builds the menu structure.
    */
   void buildMenu ()
   {
      for (int i = 0; i < fileMenuItems.length; i++)
      {
         fileMenu.add (fileMenuItems[i]);
      }

      menuBar.add (fileMenu);

      initMenu ();
   }

   /**
    * Initialize the <code>ControlBrowser</code> menu items. Just to make sure
    * that action listeners are only ever added once.
    */
   void initMenu ()
   {
      ActionListener fileMenuListener = new FileMenuListener ();
      for (int i = 0; i < fileMenuItems.length; i++)
      {
         fileMenuItems[i].addActionListener (fileMenuListener);
      }
   }

   /**
    * This method calls the start method on the current daemon.
    */
   public void startDaemon ()
   {
      daemon.start ();
      startButton.setEnabled (false);
      stopButton.setEnabled (true);
   }

   /**
    * This method calls the stop method on the current daemon.
    */
   public void stopDaemon ()
   {
      daemon.stop ();
      startButton.setEnabled (true);
      stopButton.setEnabled (false);
   }

   /**
    * This method calls the terminate method on the current daemon.
    */
   public void terminateDaemon ()
   {
      daemon.terminate ();
      nullDaemon ();
   }

   /**
    * This method applies the changes made in the GUI to the current daemon.
    */
   public void apply ()
   {
      State_Data new_daemonState = new State_Data();
      new_daemonState.recorders_urls = new String[com.thalesgroup.CdmwRecording.REC_DAEMON_MAX_RECORDERS.value];

      int configuredObjects = 0;
      int validationIndex = -1;

      try
      {
         for (int i = 0; i < com.thalesgroup.CdmwRecording.REC_DAEMON_MAX_RECORDERS.value; i++)
         {
            new_daemonState.recorders_urls[i] = urlComboBox.getItemAt(i).toString ();
            validationIndex = i;

            if (new_daemonState.recorders_urls[i] != current_daemonState.recorders_urls[i]
                                                                                        && new_daemonState.recorders_urls[i].length() > 5)
            {
               System.out.println("calling string_to_object on : (" + new_daemonState.recorders_urls[i] + ')');
               org.omg.CORBA.Object o = orb.string_to_object (new_daemonState.recorders_urls[i]);
               o._is_a("X");
               System.out.println("setting recorder_url");
               daemon.setRecorderUrl(i, new_daemonState.recorders_urls[i]);
               current_daemonState.recorders_urls[i] = new_daemonState.recorders_urls[i];
               System.out.println("done");
            }
         }
      }
      catch (org.omg.CORBA.SystemException ex)
      {
         System.out.println("ex= " + ex.getMessage());
         JOptionPane.showMessageDialog (ControlBrowser.this, "Unable to obtain reference to Recorder. Please check the URL : " + new_daemonState.recorders_urls[validationIndex],
               "Lookup failed for " + new_daemonState.recorders_urls[validationIndex], JOptionPane.ERROR_MESSAGE);
         urlComboBox.setSelectedIndex(validationIndex);
         ex.printStackTrace ();
      }

      try
      {
         String object_to_record = contMgrField.getText ();
         if (object_to_record.length() > 5)
         {
            System.out.println("calling string_to_object on : (" + object_to_record + ')');
            org.omg.CORBA.Object obj = orb.string_to_object (object_to_record);
            obj._is_a("X");
            org.omg.CORBA.Object[] list = new org.omg.CORBA.Object[1];
            list[0] = obj;
            System.out.println("setting record_objects now");
            contMgr.record_objects ("DBG_obj", list);
            System.out.println("done");
         }
      }
      catch (org.omg.CORBA.SystemException ex)
      {
         System.out.println("ex= " + ex.getMessage());
         JOptionPane.showMessageDialog (ControlBrowser.this, "Unable to obtain reference to object. Please check the URL.",
               "Lookup failed.", JOptionPane.ERROR_MESSAGE);
         ex.printStackTrace ();
      }

      new_daemonState.debug_file = fileField.getText ();
      if (! new_daemonState.debug_file.equals(current_daemonState.debug_file))
      {
         System.out.println("setting debug file to " + new_daemonState.debug_file);
         daemon.debug_file (new_daemonState.debug_file);
         current_daemonState.debug_file = new_daemonState.debug_file;
      }

      if (maxSizeField.getText ().length() > 0) 
      {
         try
         {
            new_daemonState.max_size = Integer.parseInt (maxSizeField.getText ());
            if ( new_daemonState.max_size != current_daemonState.max_size)
            {
               daemon.setMaxSize (new_daemonState.max_size);
               current_daemonState.max_size = new_daemonState.max_size;
            }
         }
         catch (NumberFormatException ex)
         {
            JOptionPane.showMessageDialog (ControlBrowser.this, "Invalid value in Max Size field.",
                  "Invalid input", JOptionPane.ERROR_MESSAGE);
            return;
         }
      }

      if (maxBufferSizeField.getText ().length() > 0) 
      {
         try
         {
            new_daemonState.max_buffer = Integer.parseInt (maxBufferSizeField.getText ());
            if ( new_daemonState.max_buffer != current_daemonState.max_buffer)
            {
               daemon.setMaxBuffer (new_daemonState.max_buffer);
               current_daemonState.max_buffer = new_daemonState.max_buffer;
            }
         }
         catch (NumberFormatException ex)
         {
            JOptionPane.showMessageDialog (ControlBrowser.this, "Invalid value in Max Buffer Size field.",
                  "Invalid input", JOptionPane.ERROR_MESSAGE);
            return;
         }
      }

      if (timeoutField.getText ().length() > 0) 
      {
         try
         {
            new_daemonState.time_out = Integer.parseInt (timeoutField.getText ());
            if ( new_daemonState.time_out != current_daemonState.time_out)
            {
               daemon.setTimeout (new_daemonState.time_out);
               current_daemonState.time_out = new_daemonState.time_out;
            }
         }
         catch (NumberFormatException ex)
         {
            JOptionPane.showMessageDialog (ControlBrowser.this, "Invalid value in Timeout field.",
                  "Invalid input", JOptionPane.ERROR_MESSAGE);
            return;
         }
      }

      if (retryDelayField.getText ().length() > 0) 
      {
         try
         {
            new_daemonState.retry_period = Integer.parseInt (retryDelayField.getText ());
            if ( new_daemonState.retry_period != current_daemonState.retry_period)
            {
               daemon.setRecorderRetryDelay (new_daemonState.retry_period);
               current_daemonState.retry_period = new_daemonState.retry_period;
            }
         }
         catch (NumberFormatException ex)
         {
            JOptionPane.showMessageDialog (ControlBrowser.this, "Invalid value in Retry Delay field.",
                  "Invalid input", JOptionPane.ERROR_MESSAGE);
            return;
         }
      }

      daemon.remove_all_interfaces ();
      Object[] objects = interfaceChooserPanel.getSelectedObjects ();
      configuredObjects = objects.length;
      String[] selectedInterfaces = new String[objects.length];
      for (int i = 0; i < selectedInterfaces.length; i++)
      {
         selectedInterfaces[i] = (String) objects[i];
      }
      daemon.add_interfaces (selectedInterfaces);

      daemon.remove_all_operations ();
      objects = operationChooserPanel.getSelectedObjects ();
      configuredObjects = configuredObjects + objects.length;
      String[] selectedOperations = new String[objects.length];
      for (int i = 0; i < selectedOperations.length; i++)
      {
         selectedOperations[i] = (String) objects[i];
      }
      daemon.add_operations (selectedOperations);

//    daemon.remove_all_ports ();
//    objects = portChooserPanel.getSelectedObjects ();
//    configuredObjects = configuredObjects + objects.length;
//    int[] selectedPorts = new int[objects.length];
//    for (int i = 0; i < selectedPorts.length; i++)
//    {
//    selectedPorts[i] = ((Integer) objects[i]).intValue ();
//    }
//    daemon.add_ports (selectedPorts);

      if (configuredObjects > 0)
      {
         statusLabel.setText ("Daemon connected : Configured");
         //applyButton.setEnabled (false);
         interfaceChooserPanel.setLiveMode (true);
         operationChooserPanel.setLiveMode (true);
      }
      else
      {
         statusLabel.setText ("Daemon connected : Unconfigured");
         applyButton.setEnabled (true);
         interfaceChooserPanel.setLiveMode (false);
         operationChooserPanel.setLiveMode (false);
      }
   }

   /**
    * This method shows an information message to let the user know that the
    * actions they have taken have changed the configuration for a specific
    * daemon on a specific host.
    *
    * @param newPort The new port value.
    */
   public void portChanged (int newPort)
   {
      Integer currentPort = (Integer) hosts.get (hostCombo.getSelectedItem ());

      if (!(currentPort.equals (Integer.valueOf (newPort))))
      {
         Integer oldPort = (Integer) hosts.put ((String) hostCombo.getSelectedItem (),
               Integer.valueOf (newPort));

         JOptionPane.showMessageDialog (ControlBrowser.this, ("Changed Port for host : "
               + hostCombo.getSelectedItem () + " from " + oldPort +
               " to " + newPort + "."), "Port Changed", JOptionPane.INFORMATION_MESSAGE);
         lookupButton.setEnabled (true);
      }
   }

   /**
    * This method disables and / or resets the components that interact with the daemon and
    * should be called when the daemon is null.
    */
   public void nullDaemon ()
   {
      daemon = null;
      contMgr = null;
      interfaceChooserPanel.setLiveMode (false);
      operationChooserPanel.setLiveMode (false);
      portChooserPanel.setEnabled (false);
      startButton.setEnabled (false);
      stopButton.setEnabled (false);
      terminateButton.setEnabled (false);
      lookupButton.setEnabled (true);
      applyButton.setEnabled (false);
      resetButton.setEnabled (false);
      maxSizeField.setText ("");
      maxBufferSizeField.setText ("");
      timeoutField.setText ("");
      retryDelayField.setText ("");
      //oa urlField.setText ("");
      urlComboBox.setEditable(true);
      urlComboBox.removeAllItems ();
      urlComboBox.setEnabled(false);
      currentIndex = -1;

      contMgrField.setText ("");
      fileField.setText ("");
      interfaceChooserPanel.removeAll ();
      operationChooserPanel.removeAll ();
      portChooserPanel.removeAll ();
      statusLabel.setText ("Daemon disconnected : Unconfigured");

      maxSizeField.setVisible(false);
      maxBufferSizeField.setVisible(false);
      timeoutField.setVisible(false);
      retryDelayField.setVisible(false);
   }

   /**
    * This method enables and / or fills in the components that interact with the daemon and
    * should be called when a new daemon has been successfully looked up.
    */
   public void newDaemon (boolean reset)
   {
      State_Data daemonState = daemon.state ();

      if (daemonState.State == Running_State.Started)
      {
         startButton.setEnabled (false);
         stopButton.setEnabled (true);
      }
      else
      {
         startButton.setEnabled (true);
         stopButton.setEnabled (false);
      }
      terminateButton.setEnabled (true);
      // Keep button active for now. lookupButton.setEnabled (false);
      applyButton.setEnabled (true);
      resetButton.setEnabled (true);
      portChooserPanel.setEnabled (true);

      fileField.setText (daemon.debug_file ());
      
      // default visibility (changed if at least 1 recorder received)
      maxSizeField.setVisible(false);
      maxBufferSizeField.setVisible(false);
      timeoutField.setVisible(false);
      retryDelayField.setVisible(false);

      // initialize loop
      urlComboBox.removeAllItems();
      currentIndex = -1;
      
      for (int i = 0; i < com.thalesgroup.CdmwRecording.REC_DAEMON_MAX_RECORDERS.value; i++)
      {
         //oa urlField.setText (daemonState.recorders_urls[i]); //(daemon.recorder_url ());
         // Works only if the JComboBox's data model is mutable
         urlComboBox.addItem (daemonState.recorders_urls[i]);

         if ( daemonState.recorders_urls[i].length() > 5)
         {
            System.out.println ("recorder_url =" + daemonState.recorders_urls[i]);
            maxSizeField.setText (Integer.toString (daemonState.max_size));
            maxBufferSizeField.setText (Integer.toString (daemonState.max_buffer));
            timeoutField.setText (Integer.toString (daemonState.time_out));
            retryDelayField.setText (Integer.toString (daemonState.retry_period));
            maxSizeField.setVisible(true);
            maxBufferSizeField.setVisible(true);
            timeoutField.setVisible(true);
            retryDelayField.setVisible(true);
            if (currentIndex < 0)
            {
               currentIndex = i;
               urlComboBox.setSelectedIndex(i);
            }
         }
         else
         {
            System.out.println ("--null recorder_url =" + daemonState.recorders_urls[i]);
//          maxSizeField.setVisible(false);
//          maxBufferSizeField.setVisible(false);
//          timeoutField.setVisible(false);
//          retryDelayField.setVisible(false);
         }
         if (currentIndex >= 0)
         {
            urlComboBox.setEnabled(true);
         }
      }


      int configuredObjects = 0;

      String[] interfaces = daemonState.interfaces;
      configuredObjects = interfaces.length;
      String[] operations = daemonState.operations;
      configuredObjects = configuredObjects + operations.length;
      short[] ports = daemonState.ports;
      configuredObjects = configuredObjects + ports.length;

      if (configuredObjects > 0)
      {
         statusLabel.setText ("Daemon connected : Configured");
         interfaceChooserPanel.setSelectedObjects (interfaces);
         operationChooserPanel.setSelectedObjects (operations);
         Short[] portInts = new Short[ports.length];
         for (int i = 0; i < ports.length; i++)
         {
            portInts[i] = Short.valueOf (ports[i]);
         }
         portChooserPanel.setSelectedObjects (portInts);

         if (!reset)
         {
            //applyButton.setEnabled (false);
            interfaceChooserPanel.setLiveMode (true);
            operationChooserPanel.setLiveMode (true);
         }
      }
      else
      {
         statusLabel.setText ("Daemon connected : Unconfigured");
         applyButton.setEnabled (true);
         interfaceChooserPanel.setLiveMode (false);
         operationChooserPanel.setLiveMode (false);
         portChooserPanel.removeAll ();
      }
      current_daemonState = daemonState;
   }

   /**
    * This method returns the current instance of the daemon.
    */
   public DaemonControl getDaemon ()
   {
      return daemon;
   }

   /**
    * This is the main method for the ControlBrowser.
    *
    * @param args Command line arguments.
    */
   public static void main (String[] args)
   {
      if (args.length != 4)
      {
         System.err.println ("Usage: ControlBrowser hosts.xml interfaces.xml operations.xml ports.xml");
         System.exit (0);
      }

      if (!(System.getProperty ("ControlBrowser.UTest", "").equals ("")))
      {
         testFlag = true;
      }

      if (orb == null)
      {
         orb = org.omg.CORBA.ORB.init (args, null);
      }

      /*
       * Initialise the browser.
       */
      controlBrowser = new ControlBrowser ();

      controlBrowser.readConfiguration (args);
      controlBrowser.setupPanel ();
      JFrame frame = new JFrame ();
      frame.setDefaultCloseOperation (JFrame.EXIT_ON_CLOSE);
      frame.getContentPane ().add (controlBrowser);
      frame.pack ();

      frame.setSize (840, 640);
      int width = frame.getSize ().width;
      int height = frame.getSize ().height;
      Dimension screenSize = Toolkit.getDefaultToolkit ().getScreenSize ();
      frame.setLocation (screenSize.width / 2 - width / 2, screenSize.height / 2 - height / 2);

      frame.setTitle ("Control Browser");
      frame.addWindowListener (new MyWindowAdapter ());

      if (testFlag != true)
      {
         frame.setVisible (true);
      }
      else
      {
         controlBrowser.performUtests ();
      }
   }

   /**
    * This method performs the unit tests for the class.
    */
   public void performUtests ()
   {
      /* Look up the currently selected Recording Daemon. */
      lookup ();

      /* Start test */
      startDaemon ();

      /* Stop test */
      stopDaemon ();

      int[] testIndices = new int[5];
      State_Data state = null;

      for (int i = 0; i < testIndices.length; i++)
      {
         testIndices[i] = i;
      }

      interfaceChooserPanel.addAll ();
      operationChooserPanel.addAll ();

      apply ();

      state = daemon.state ();
      if (state.interfaces.length != interfaceChooserPanel.selectedValues.getModel ().getSize () ||
            state.operations.length != operationChooserPanel.selectedValues.getModel ().getSize ())
      {
         System.out.println ("TEST 1 FAILED");
      }

      interfaceChooserPanel.removeAll ();
      operationChooserPanel.removeAll ();

      state = daemon.state ();
      if (state.interfaces.length != 0 ||
            state.operations.length != 0)
      {
         System.out.println ("TEST 2 FAILED");
      }

      interfaceChooserPanel.availableValues.setSelectedIndices (testIndices);
      interfaceChooserPanel.add ();
      operationChooserPanel.availableValues.setSelectedIndices (testIndices);
      operationChooserPanel.add ();

      int[] index = new int[1];
      for (int i = 0; i < testIndices.length; i++)
      {
         index[0] = testIndices[i];
         portChooserPanel.availableValues.setSelectedIndices (index);
         portChooserPanel.add ();
      }

      state = daemon.state ();
      if (state.interfaces.length != 5 ||
            state.operations.length != 5 ||
            state.ports.length != 5)
      {
         System.out.println ("TEST 3 FAILED");
      }

      interfaceChooserPanel.selectedValues.setSelectedIndices (testIndices);
      interfaceChooserPanel.remove ();
      operationChooserPanel.selectedValues.setSelectedIndices (testIndices);
      operationChooserPanel.remove ();


      for (int i = 0; i < testIndices.length; i++)
      {
         portChooserPanel.selectedValues.setSelectedIndices (index);
         portChooserPanel.remove ();
      }

      state = daemon.state ();
      if (state.interfaces.length != 0 ||
            state.operations.length != 0 ||
            state.ports.length != 0)
      {
         System.out.println ("TEST 4 FAILED");
      }

      /* Terminate test */
      terminateDaemon ();

      System.exit (0);
   }

   /**
    * Inner class to handle file menu actions.
    */
   public class FileMenuListener implements ActionListener
   {
      /**
       * This method is invoked when a component is clicked that has had this listener
       * added to it as an ActionListener.
       *
       * @param e The event.
       */
      public void actionPerformed (ActionEvent e)
      {
         JMenuItem mi = (JMenuItem) e.getSource ();
         if (mi.getText ().equals ("Exit"))
         {
            orb.shutdown (true);
            System.exit (0);
         }
      }
   }

   /**
    * Inner class to handle window events.
    */
   public static class MyWindowAdapter extends WindowAdapter
   {
      /**
       * This method is invoked when a window component is closed that has had this
       * listener added to it as a WindowListener.
       */
      public void windowClosing (WindowEvent ev)
      {
         // Any clean up needs to be done here.
         orb.shutdown (true);
      }
   }

   /**
    * Inner class to handle host combo box actions.
    */
   public class HostComboBoxListener implements ActionListener
   {
      /**
       * This method is invoked when a component is clicked that has had this listener
       * added to it as an ActionListener.
       *
       * @param e The event.
       */
      public void actionPerformed (ActionEvent e)
      {
         JComboBox cb = (JComboBox) e.getSource ();
         String newSelection = (String) cb.getSelectedItem ();

         if (e.getActionCommand ().equals ("comboBoxChanged"))
         {
            if (hosts.keySet ().contains (newSelection))
            {
               int currentPort = ((Integer) hosts.get (newSelection)).intValue ();
               portField.setText (Integer.valueOf (currentPort).toString ());

               LookupThread lt = new LookupThread ();

               try
               {
                  lt.start ();
               }
               catch (Exception ex)
               {
                  System.out.println ("Paint thread interrupted");
               }
            }
         }
         else if (e.getActionCommand ().equals ("comboBoxEdited"))
         {
            if (!(hosts.keySet ().contains (newSelection)))
            {
               cb.addItem (newSelection);

               try
               {
                  hosts.put (newSelection, (Integer.valueOf (portField.getText ())));
               }
               catch (NumberFormatException ex)
               {
                  JOptionPane.showMessageDialog (ControlBrowser.this, "Invalid port value.", "Invalid input",
                        JOptionPane.ERROR_MESSAGE);
               }

               nullDaemon ();
            }
         }

         if (hostCombo.isEditable ())
         {
            hostCombo.setEditable (false);
         }
      }
   }

   /**
    * Inner class to handle actions in a field that holds a numeric value.
    */
   public class NumberFieldListener implements ActionListener
   {
      /**
       * This method is invoked when a component is changed that has had this listener
       * added to it as an ActionListener.
       *
       * @param e The event.
       */
      public void actionPerformed (ActionEvent e)
      {
         if (e.getSource ().equals (portField))
         {
            if (portField.isEditable ())
            {
               int newPort = 0;

               try
               {
                  newPort = Integer.valueOf (portField.getText ());
               }
               catch (NumberFormatException ex)
               {
                  JOptionPane.showMessageDialog (ControlBrowser.this, "Invalid port value.", "Invalid input",
                        JOptionPane.ERROR_MESSAGE);
                  return;
               }

               portChanged (newPort);
            }
         }
      }
   }

   /**
    * Inner class to handle loss of focus from a field that holds a numeric value.
    */
   public class NumberFieldFocusListener extends FocusAdapter
   {
      /**
       * The value of the port. Only used for number format checking.
       */
      int myPort = 0;

      /**
       * This method is invoked when a component that has had this listener added
       * to it as an FocusListener loses the keyboard focus.
       *
       * @param e The event.
       */
      public void focusLost (FocusEvent e)
      {
         if (e.getSource ().equals (portField))
         {
            /*
             * The opposite component is the one that focus is switching to, if
             * this is null then we may have clicked outside the GUI (maybe to copy
             * something in), so leave the field as it is.
             */
            if (e.getOppositeComponent () != null)
            {
               try
               {
                  myPort = Integer.valueOf (portField.getText ());
               }
               catch (NumberFormatException ex)
               {
                  JOptionPane.showMessageDialog (ControlBrowser.this, "Invalid port value.", "Invalid input",
                        JOptionPane.ERROR_MESSAGE);
                  return;
               }

               portChanged (myPort);
            }
         }
      }
   }

   /**
    * Inner class to handle actions on the buttons.
    */
   public class ButtonListener implements ActionListener
   {
      /**
       * This method is invoked when a component is clicked that has had this listener
       * added to it as an ActionListener.
       *
       * @param e The event.
       */
      private void l_actionPerformed (ActionEvent e)
      {
         if (((JButton) e.getSource ()).equals (startButton))
         {
            startDaemon ();
         }
         else if (((JButton) e.getSource ()).equals (stopButton))
         {
            stopDaemon ();
         }
         else if (((JButton) e.getSource ()).equals (terminateButton))
         {
            terminateDaemon ();
         }
         else if (((JButton) e.getSource ()).equals (browseButton))
         {

            if (browser == null)
            {
               browser = new JFileChooser ();
               browser.setApproveButtonText ("OK");
            }

            int returnVal = browser.showOpenDialog (ControlBrowser.this);

            if (returnVal == JFileChooser.APPROVE_OPTION)
            {
               File file = browser.getSelectedFile ();
               fileField.setText (file.getAbsolutePath ());
            }
         }
         else if (((JButton) e.getSource ()).equals (lookupButton))
         {
            try
            {
               lookup ();
            }
            catch (Throwable exception)
            {
            }
         }
         else if (((JButton) e.getSource ()).equals (applyButton))
         {
            apply ();
         }
         else if (((JButton) e.getSource ()).equals (resetButton))
         {
            interfaceChooserPanel.setLiveMode (false);
            operationChooserPanel.setLiveMode (false);

            if (daemon != null)
            {
               newDaemon (true);
            }
            else
            {
               nullDaemon ();
            }
         }
      }

      // JM ThreadSafe
      public void actionPerformed (ActionEvent e)
      {
         final ActionEvent evt = e;
         Runnable code = new Runnable() {
            public void run() {
               l_actionPerformed(evt);
            }
         };

         if (SwingUtilities.isEventDispatchThread()) {
            code.run();
         } else {
            SwingUtilities.invokeLater(code);
         }
      }

   }

   /**
    * Inner class to handle mouse events on the host combobox.
    */
   public class MyMouseListener extends MouseAdapter
   {
      /**
       * This method is invoked when a component that had this listener added
       * to it as an MouseListener detects that the mouse has been clicked on
       * that component.
       *
       * @param e The event.
       */
      public void mouseClicked (MouseEvent e)
      {
         if (e.getClickCount () > 1)
         {
            if (e.getSource () instanceof JComboBox)
            {
               hostCombo.setEditable (true);
            }
            else if (e.getSource () instanceof JTextField)
            {
               try
               {
                  currentPort = Integer.valueOf (portField.getText ());
               }
               catch (NumberFormatException ex)
               {
                  // It's safe to ignore this case, because it should only
                  // happen when the user double clicks when they have
                  // entered something silly into the portField.
               }

            }
         }
      }
   }

   /**
    * Thread to paint the port field and look up the daemon while an action on the
    * hostComboBox is being processed. If this is done in the actionPerformed
    * method of the listener then the changes to the portFieldwill not take effect
    * until after the lookup has finished.
    */
   public class LookupThread extends Thread
   {
      /**
       * Run this in a thread so Swing can draw it.
       */
      public void run ()
      {
         /*
          * Then start another thread so Swing can invoke this thread and wait
          * for it to finish.
          */
         Runnable threadThing = new Runnable ()
         {
            /**
             * This method does the painting of the components that are being
             * used while the action is being performed.
             */
            public void run ()
            {
               portField.invalidate ();
               portField.revalidate ();
               portField.repaint();
               if (currentPort != 0)
               {
                  interfaceChooserPanel.setLiveMode (false);
                  operationChooserPanel.setLiveMode (false);
                  lookup ();
               }
            }
         };

         try
         {
            SwingUtilities.invokeAndWait (threadThing);
         }
         catch (Exception ex)
         {
            System.out.println ("Lookup Thread Was Interrupted");
         }
      }
   }
}
