/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.recording.gui;

/**
 * This class represents a message that will be displayed in the table of the
 * DisplayBrowser. It simply wraps all the component parts of the message
 * and also holds the original string that formed the message.
 *
 * @author Paul Graham
 */
public class DebugMessage
{
   /**
    * The request ID.
    */
   String requestID = null;

   /**
    * The request time.
    */
   String requestTime = null;

   /**
    * The IP the request came from.
    */
   String fromIP = null;

   /**
    * The port the request came from.
    */
   String fromPort = null;

   /**
    * The IP the request was sent to.
    */
   String toIP = null;

   /**
    * The port the request was sent to.
    */
   String toPort = null;

   /**
    * The server ID.
    */
   String serverID = null;

   /**
    * The interface ID.
    */
   String interfaceID = null;

   /**
    * The operation details.
    */
   String operation = null;

   /**
    * The reply information.
    */
   String replyTo = null;

   /**
    * The reply time.
    */
   String replyTime = null;

   /**
    * The full message string.
    */
   String messageString = null;

   /**
    * Constructor. Processes the message string that is passed in and fills in the 
    * relevant fields in the class instance.
    * 
    * @param message The message.
    */
   public DebugMessage (String message)
   {
      //System.out.println ("Message :" + message);
      this.messageString = message.replace('\n', ' ');
      this.messageString = this.messageString.replace("  ", " ");

      int startIndex = 8;
      int endIndex = messageString.indexOf (" at");
      if (endIndex >= 0)
      {
         requestID = messageString.substring (startIndex, endIndex);
         //System.out.println ("Request ID = " + requestID + " startIndex = " + startIndex + " endIndex = " + endIndex);
         startIndex = endIndex + 4;
         endIndex = messageString.indexOf (" from", startIndex);
         if (endIndex > 0)
         {
            requestTime = messageString.substring (startIndex, endIndex);
            //System.out.println ("Time = " + requestTime + " startIndex = " + startIndex + " endIndex = " + endIndex);
            startIndex = endIndex + 6;
            endIndex = messageString.indexOf (" ", startIndex);
            fromIP = messageString.substring (startIndex, endIndex);
            //System.out.println ("From IP = " + fromIP + " startIndex = " + startIndex + " endIndex = " + endIndex);
            startIndex = endIndex + 1;
         }
         endIndex = messageString.indexOf (" to", startIndex);
         if (endIndex > 0)
         {
            fromPort = messageString.substring (startIndex, endIndex);
            //System.out.println ("From Port = " + fromPort + " startIndex = " + startIndex + " endIndex = " + endIndex);
            startIndex = endIndex + 4;
            endIndex = messageString.indexOf (" ", startIndex);
            toIP = messageString.substring (startIndex, endIndex);
            //System.out.println ("To IP = " + toIP + " startIndex = " + startIndex + " endIndex = " + endIndex);
            startIndex = endIndex + 1;
         }
         endIndex = messageString.indexOf (" server_id", startIndex);
         if (endIndex > 0)
         {
            toPort = messageString.substring (startIndex, endIndex);
            //System.out.println ("To Port = " + toPort + " startIndex = " + startIndex + " endIndex = " + endIndex);
            startIndex = endIndex + 11;
         }
         endIndex = messageString.indexOf (" interface", startIndex);
         if (endIndex > 0)
         {
            serverID = messageString.substring (startIndex, endIndex);
            //System.out.println ("Server ID = " + serverID + " startIndex = " + startIndex + " endIndex = " + endIndex);
            startIndex = endIndex + 11;
         }
         endIndex = messageString.indexOf (" operation", startIndex);
         if (endIndex > 0)
         {
            interfaceID = messageString.substring (startIndex, endIndex);
            //System.out.println ("Interface = " + interfaceID + " startIndex = " + startIndex + " endIndex = " + endIndex);
            startIndex = endIndex + 11;
         }
         endIndex = messageString.indexOf (" ", startIndex);

         if (endIndex > 0)
         {
            operation = messageString.substring (startIndex, endIndex);
            //System.out.println ("Operation = " + operation + " startIndex = " + startIndex + " endIndex = " + endIndex);
         }
         // Reply (JM)
         endIndex = messageString.indexOf (" reply_to", startIndex);
         if (endIndex > 0)
         {
           startIndex = endIndex + 10;
           // Reply
           endIndex = messageString.indexOf (" at", startIndex);
           if (endIndex > 0)
           {
             replyTo = messageString.substring (startIndex, endIndex);
             //System.out.println ("Reply To = " + replyTo + " startIndex = " + startIndex + " endIndex = " + endIndex);
             startIndex = endIndex + 4;
             // yy/mm/dd
             endIndex = messageString.indexOf (" ", startIndex);
             replyTime = messageString.substring (startIndex, endIndex);
             //System.out.println ("Reply Time = " + replyTime + " startIndex = " + startIndex + " endIndex = " + endIndex);
             // hh:mn:ss:ms:us
             startIndex = endIndex + 1;
             endIndex = messageString.indexOf (" ", startIndex);
             replyTime = replyTime + " " + messageString.substring (startIndex, endIndex);
             //System.out.println ("Reply Time = " + replyTime + " startIndex = " + startIndex + " endIndex = " + endIndex);
           }
         }
         else
         {
           replyTo = "oneway";
         }
      }

      this.messageString = message;
   }
}
