/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.recording.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;
import java.awt.image.BufferedImage;
import java.awt.image.DataBufferByte;
import java.awt.image.IndexColorModel;
import java.awt.image.Raster;
import java.io.File;
import java.util.Vector;
import java.util.*;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import java.io.*;

import cdmw.ossupport.ThreadedProcess;

/**
 * This is the Cardamom recording service debug browser.
 *
 * Copyright (c) 2007 PrismTech Ltd. All rights Reserved.
 *
 * @author Paul Graham
 */
public class DisplayBrowser extends JPanel
{
   /**
    * Generated SerialVersionUID.
    */
   private static final long serialVersionUID = 3169110070336857454L;

   /**
    * Static reference to the DisplayBrowser instance.
    */
   static DisplayBrowser displayBrowser = null;

   /**
    *  reference to the output stream instance.
    */
   PrintStream output = null;

   /**
    * The decode executable to invoke
    */
   static String decoder_exe = System.getProperty ("cdmw.decoder_exe");

   /**
    * The menu bar for the top of the browser.
    */
   JMenuBar menuBar = new JMenuBar ();

   /**
    * The file menu and file menu items.
    */
   JMenu fileMenu = new JMenu ("File");

   // JMenuItem fileMenuOpen = new JMenuItem ("Select debugger");
   JMenuItem recordedMenuOpen = new JMenuItem ("Select Recorded File");
   JMenuItem fileMenuExit = new JMenuItem ("Exit");

   JMenuItem fileMenuItems[] = { recordedMenuOpen, fileMenuExit };

   /**
    * The view menu and view menu items.
    */
   JMenu viewMenu = new JMenu ("View");

   JMenuItem viewMenuItems[] = null;

   /**
    * Byte array to represent a little tick icon for use on the view menu.
    */
   byte[] tick = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0, 0, 0, 1, 0,
         0, 0, 0, 0, 0, 0, 0, 1, 1, 0,
         0, 0, 0, 0, 0, 0, 1, 1, 1, 0,
         0, 1, 0, 0, 0, 1, 1, 1, 0, 0,
         0, 1, 1, 0, 1, 1, 1, 0, 0, 0,
         0, 1, 1, 1, 1, 1, 0, 0, 0, 0,
         0, 0, 1, 1, 1, 0, 0, 0, 0, 0,
         0, 0, 0, 1, 0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
   /**
    * A little tick icon for use on the view menu.
    */
   ImageIcon tickIcon = null;

   /**
    * Byte array to represent a little blank icon for use on the view menu.
    */
   byte[] blank = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0};

   /**
    * A little blank icon for use on the view menu.
    */
   ImageIcon blankIcon = null;

   /**
    * The panel we display the file controls in.
    */
   JPanel controlPanel = new JPanel ();

   /**
    * The panel we display the table of messages in.
    */
   JPanel tablePanel = new JPanel ();

   /**
    * The panel we display the details of a selected message in.
    */
   JPanel detailsPanel = new JPanel ();

   /**
    * The splitpane for Table / Detail displays.
    */
   JSplitPane spOne = null;

   /**
    * The filter field.
    */
   JTextField filterField = null;

   /**
    * The interface repository field.
    */
   JTextField ir_urlField = null;

   /**
    * The recorded file field.
    */
   JTextField recordedField = null;

   /**
    * The combobox to allow the user to select the filter criteria.
    */
   JComboBox filterCombo = null;

   /**
    * Button to clear the filter field.
    */
   JButton clearButton = null;

   /**
    * The text area that will display the detailed message.
    */
   JTextArea displayArea = null;

   /**
    * File chooser for browsing for the debug file.
    */
   JFileChooser browser = null;

   /**
    * The table model for the state table.
    */
   private EventTableModel eventTableModel = null;

   /**
    * Table renderer for the state table.
    */
   private EventTableRenderer eventTableRenderer = null;

   /**
    * The table used to display the state information and controls.
    */
   private JTable eventTable = null;

   /**
    * Vector of columns to define the table.
    */
   private Vector<String> columns = null;

   /**
    * String holders for for the table column names.
    */
   String requestLabel = null;
   String requestTimeLabel = null;
   String fromIPLabel = null;
   String fromPortLabel = null;
   String toIPLabel = null;
   String toPortLabel = null;
   String serverIDLabel = null;
   String interfaceLabel = null;
   String operationLabel = null;
   String replyToLabel = null;
   String replyTimeLabel = null;

   /**
    * Vector of messages that are displayed in the table.
    */
   private Vector<DebugMessage> messages = new Vector<DebugMessage> ();

   /**
    * Boolean flag. True if the unit tests are to be ran.
    */
   static boolean testFlag = false;

   /**
    * Enumeration for telling the filter methods what behaviour is expected
    * of them.
    */
   enum FilterChange
   {
      CLEARED, ADDED, REMOVED;
   }

   /**
    * Default constructor.
    */
   public DisplayBrowser ()
   {
   }

   /**
    * This method initialises the vector of column titles and the array of
    * menu items for the view menu.
    */
   void initialise ()
   {
      /* Set up two icons from the byte array information that we have defined. */
      byte[] red = {(byte) 255, (byte) 0};
      byte[] green = {(byte) 255, (byte) 0};
      byte[] blue = {(byte) 255, (byte) 0};

      DataBufferByte tickBufferByte = new DataBufferByte (tick, tick.length);
      DataBufferByte blankBufferByte = new DataBufferByte (blank, blank.length);
      Raster tickRaster = Raster.createInterleavedRaster (tickBufferByte, 10, 10, 10, 1, new int[] {0}, null);
      Raster blankRaster = Raster.createInterleavedRaster (blankBufferByte, 10, 10, 10, 1, new int[] {0}, null);
      IndexColorModel icm = new IndexColorModel (8, 2, red, green, blue, 0);
      BufferedImage tickImage = new BufferedImage (10, 10, BufferedImage.TYPE_BYTE_INDEXED, icm);
      BufferedImage blankImage = new BufferedImage (10, 10, BufferedImage.TYPE_BYTE_INDEXED, icm);
      tickImage.setData (tickRaster);
      blankImage.setData (blankRaster);
      tickIcon = new ImageIcon (tickImage);
      blankIcon = new ImageIcon (blankImage);

      /* Set up the labels for the table columns. */
      requestLabel = new String ("Request");
      requestTimeLabel = new String ("Req Time");
      fromIPLabel = new String ("from IP");
      fromPortLabel = new String ("from Port");
      toIPLabel = new String ("to IP");
      toPortLabel = new String ("to Port");
      serverIDLabel = new String ("Server_id");
      interfaceLabel = new String ("Interface");
      operationLabel = new String ("Operation");
      replyToLabel = new String ("Reply to");
      replyTimeLabel = new String ("Rep Time");

      /* Put the column names in a vector */
      columns = new Vector<String> ();
      columns.addElement (requestLabel);
      columns.addElement (requestTimeLabel);
      columns.addElement (fromIPLabel);
      columns.addElement (fromPortLabel);
      columns.addElement (toIPLabel);
      columns.addElement (toPortLabel);
      columns.addElement (serverIDLabel);
      columns.addElement (interfaceLabel);
      columns.addElement (operationLabel);
      columns.addElement (replyToLabel);
      columns.addElement (replyTimeLabel);
      columns.addElement (new String (""));

      /* Generate the appropriate menu items. */
      JMenuItem viewMenuRequest = new JMenuItem (requestLabel);
      JMenuItem viewMenuRequestTime = new JMenuItem (requestTimeLabel);
      JMenuItem viewMenuFromIP = new JMenuItem (fromIPLabel);
      JMenuItem viewMenuFromPort = new JMenuItem (fromPortLabel);
      JMenuItem viewMenuToIP = new JMenuItem (toIPLabel);
      JMenuItem viewMenuToPort = new JMenuItem (toPortLabel);
      JMenuItem viewMenuServerID = new JMenuItem (serverIDLabel);
      JMenuItem viewMenuInterface = new JMenuItem (interfaceLabel);
      JMenuItem viewMenuOperation = new JMenuItem (operationLabel);
      JMenuItem viewMenuReplyTo = new JMenuItem (replyToLabel);
      JMenuItem viewMenuReplyTime = new JMenuItem (replyTimeLabel);

      /*
       * Add the tick icon to each menu item as they're all displayed
       * by default.
       */
      viewMenuRequest.setIcon (tickIcon);
      viewMenuRequestTime.setIcon (tickIcon);
      viewMenuFromIP.setIcon (tickIcon);
      viewMenuFromPort.setIcon (tickIcon);
      viewMenuToIP.setIcon (tickIcon);
      viewMenuToPort.setIcon (tickIcon);
      viewMenuServerID.setIcon (tickIcon);
      viewMenuInterface.setIcon (tickIcon);
      viewMenuOperation.setIcon (tickIcon);
      viewMenuReplyTo.setIcon (tickIcon);
      viewMenuReplyTime.setIcon (tickIcon);

      viewMenuItems = new JMenuItem[] { viewMenuRequest, viewMenuRequestTime, viewMenuFromIP, viewMenuFromPort, viewMenuToIP, viewMenuToPort, viewMenuServerID, viewMenuInterface, viewMenuOperation, viewMenuReplyTo, viewMenuReplyTime };
   }

   /**
    * This method sets up the panel for our <code>DisplayBrowser</code>
    * object. The panel consists of two areas, the menu bar and the tree /
    * properties panel.
    */
   void setupPanel ()
   {
      this.setLayout (new BorderLayout ());

      buildMenu ();

      this.add (menuBar, BorderLayout.NORTH);

      controlPanel.setLayout (new BorderLayout ());
      tablePanel.setLayout (new BorderLayout ());
      detailsPanel.setLayout (new BorderLayout ());

      JPanel topPanel = new JPanel ();

      topPanel.setLayout (new BorderLayout ());

      topPanel.setBorder (BorderFactory.createTitledBorder ("Debug Viewer"));
      detailsPanel.setBorder (BorderFactory.createTitledBorder ("Message Details"));
      tablePanel.setBorder (new EmptyBorder (3, 3, 3, 3));

      JTextField decoder_exeField = new JTextField (20);
      decoder_exeField.setEditable(false);
      File exe_file = new File(decoder_exe);
      if (exe_file.exists())
      {
         decoder_exeField.setText (decoder_exe);
      }
      else
      {
         decoder_exeField.setText ("UNKNOWN file : " + decoder_exe);
      }
      filterField = new JTextField (20);
      ir_urlField = new JTextField (20);
      ir_urlField.setEditable(false);
      ir_urlField.setText ("corbaloc::localhost:6070/InterfaceRepository");
      recordedField = new JTextField (20);
      recordedField.setText ("");
      recordedField.setEditable(false);
      filterCombo = new JComboBox (columns);
      clearButton = new JButton ("Clear");

      ButtonListener buttonListener = new ButtonListener ();

      filterCombo.addActionListener (buttonListener);
      clearButton.addActionListener (buttonListener);

      filterField.addKeyListener (new FilterFieldListener ());

      JLabel decoder_exeLabel = new JLabel ("Decoder Executable :");
      JLabel urlLabel = new JLabel ("Interface Repository URL :");
      JLabel recordedLabel = new JLabel ("Recorded File :");

      JPanel fieldPanel = new JPanel ();
      fieldPanel.setLayout (new GridBagLayout ());

      GridBagConstraints gbc = new GridBagConstraints ();
      gbc.anchor = GridBagConstraints.NORTHWEST;
      gbc.insets = new Insets (3, 3, 3, 3);

      gbc.gridx = 0; gbc.gridy = 0; gbc.weightx = 0.0;
      gbc.gridwidth = 1;
      fieldPanel.add (decoder_exeLabel, gbc);
      gbc.gridx = 1; gbc.gridy = 0; gbc.weightx = 1.0;
      gbc.fill = GridBagConstraints.HORIZONTAL;
      fieldPanel.add (decoder_exeField, gbc);

      gbc.gridx = 0; gbc.gridy = 1; gbc.weightx = 0.0;
      gbc.gridwidth = 1;
      fieldPanel.add (urlLabel, gbc);
      gbc.gridx = 1; gbc.gridy = 1; gbc.weightx = 1.0;
      gbc.fill = GridBagConstraints.HORIZONTAL;
      fieldPanel.add (ir_urlField, gbc);

      gbc.gridx = 0; gbc.gridy = 2; gbc.weightx = 0.0;
      gbc.gridwidth = 1;
      fieldPanel.add (recordedLabel, gbc);
      gbc.gridx = 1; gbc.gridy = 2; gbc.weightx = 1.0;
      gbc.fill = GridBagConstraints.HORIZONTAL;
      fieldPanel.add (recordedField, gbc);

      controlPanel.add (fieldPanel, BorderLayout.CENTER);

      JPanel filterPanel = new JPanel ();
      filterPanel.setLayout (new GridBagLayout ());

      gbc = new GridBagConstraints ();
      gbc.anchor = GridBagConstraints.NORTHWEST;
      gbc.insets = new Insets (3, 3, 3, 3);

      gbc.gridx = 0; gbc.gridy = 0; gbc.weightx = 1.0;
      gbc.fill = GridBagConstraints.HORIZONTAL;
      filterPanel.add (filterField, gbc);
      gbc.gridx = 1; gbc.gridy = 0; gbc.weightx = 0.0;
      gbc.fill = GridBagConstraints.NONE;
      filterPanel.add (filterCombo, gbc);
      gbc.gridx = 2; gbc.gridy = 0; gbc.weightx = 0.0;
      filterPanel.add (clearButton, gbc);

      controlPanel.add (filterPanel, BorderLayout.SOUTH);

      /* Create a new model using the vector of labels. */
      eventTableModel = new EventTableModel (columns, 0);
      eventTableModel.setDataVector (new Vector (), columns);

      /* Create a table which will use this model. */
      eventTable = new EventTable (eventTableModel);

      eventTable.removeEditor ();
      eventTable.setCellEditor (null);
      eventTable.getTableHeader ().setReorderingAllowed (true);
      eventTable.addMouseListener (new MyMouseListener ());
      setupRenderer ();

      JScrollPane tableScrollPane = new JScrollPane (eventTable,
            JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
            JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

      tablePanel.add (tableScrollPane, BorderLayout.CENTER);

      topPanel.add (controlPanel, BorderLayout.NORTH);
      topPanel.add (tablePanel, BorderLayout.CENTER);

      displayArea = new JTextArea ("");

      displayArea.setBackground (Color.WHITE);
      displayArea.setEditable (false);
      displayArea.setWrapStyleWord (true);
      displayArea.setLineWrap (true);

      JScrollPane textAreaScrollPane = new JScrollPane (displayArea,
            JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
            JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

      JPanel subTextAreaScrollPanel = new JPanel ();

      subTextAreaScrollPanel.setLayout (new BorderLayout ());

      subTextAreaScrollPanel.setBorder (new EmptyBorder (3, 3, 3, 3));

      subTextAreaScrollPanel.add (textAreaScrollPane, BorderLayout.CENTER);

      detailsPanel.add (subTextAreaScrollPanel, BorderLayout.CENTER);

      /* Put the table panel and details panel in a splitpane. */
      spOne = new JSplitPane (JSplitPane.VERTICAL_SPLIT, topPanel, detailsPanel);

      /* Set the split pane options. */
      spOne.setDividerSize (6);
      spOne.setOneTouchExpandable (true);

      this.add (spOne, BorderLayout.CENTER);
   }

   /**
    * Applies the renderer to each table column in turn.
    */
   public void setupRenderer ()
   {
      TableColumn col = null;
      eventTableRenderer = new EventTableRenderer ();

      for (int i = 0; i < columns.size (); i++)
      {
         col = eventTable.getColumn (columns.elementAt (i));

         col.setCellRenderer (eventTableRenderer);
      }
   }

   /**
    * Add a message to the table data.
    * @param message The message.
    */
   public void addMessage (String message)
   {
      DebugMessage debugMessage = new DebugMessage (message);

      messages.addElement (debugMessage);

      eventTableModel.addDebugMessage (debugMessage, messages.size() - 1);
   }

   /**
    * This method takes a row and displays the corresponding message in the text area.
    * @param row The row.
    */
   public void displayMessage (int row)
   {
      Vector dataVector = eventTableModel.getDataVector ();
      String real_row_str = (String) (((Vector) dataVector.elementAt (row)).elementAt (11));
      int real_row = Integer.decode (real_row_str);
      displayArea.setText (((DebugMessage) messages.elementAt (real_row)).messageString);
   }

   /**
    * This method filters the messages in the table based on the filter criteria which
    * is made up from the text in the filter field and the filter combobox.
    */
   public void filterMessages (FilterChange change)
   {
      String filterString = filterField.getText ();

      if (change == FilterChange.CLEARED || change == FilterChange.REMOVED)
      {
         while (eventTable.getRowCount () > 0)
         {
            eventTableModel.removeRow (0);
         }

         for (int i = 0; i < messages.size (); i++)
         {
            eventTableModel.addDebugMessage ((DebugMessage) messages.elementAt (i), i);
         }
      }

      if (change == FilterChange.ADDED || change == FilterChange.REMOVED)
      {
         int col = filterCombo.getSelectedIndex ();

         Vector dataVector = eventTableModel.getDataVector ();

         for (int i = 0; i < dataVector.size (); i++)
         {
            if (((String) ((Vector) dataVector.elementAt (i)).elementAt (col)).indexOf (filterString) == -1)
            {
               eventTableModel.removeRow (i);
               i = i - 1;
            }
         }
      }
   }

   /**
    * This method builds the menu structure.
    */
   void buildMenu ()
   {
      for (int i = 0; i < fileMenuItems.length; i++)
      {
         fileMenu.add (fileMenuItems[i]);
      }

      menuBar.add (fileMenu);

      for (int i = 0; i < viewMenuItems.length; i++)
      {
         viewMenu.add (viewMenuItems[i]);
      }

      menuBar.add (viewMenu);

      initMenu ();
   }

   /**
    * Initialise the menu items. Just to make sure
    * that action listeners are only ever added once.
    */
   void initMenu ()
   {
      ActionListener fileMenuListener = new FileMenuListener ();
      for (int i = 0; i < fileMenuItems.length; i++)
      {
         fileMenuItems[i].addActionListener (fileMenuListener);
      }

      ActionListener viewMenuListener = new ViewMenuListener ();
      for (int i = 0; i < viewMenuItems.length; i++)
      {
         viewMenuItems[i].addActionListener (viewMenuListener);
      }
   }

   /**
    * Initialise the decoder and redirect the output stream so that the
    * messages that the decoder has read can be added to the table.
    *
    * @param filePath That path of the file we want the decoder to read.
    */
   void start_decoder_exe (String filePath)
   {
      if (output != null)
      {
         output.println ("quit");
         output.flush();
         output = null;
      }

      ThreadedProcess proc = new ThreadedProcess(filePath + " --ifr_url=" + ir_urlField.getText (), null);

      try
      {
         Thread thread = new MessageThread (proc.getProcess());
         thread.start ();
      }
      catch (Exception ex)
      {
         ex.printStackTrace ();
      }
   }

   /**
    * This is the main method for the ControlBrowser.
    *
    * @param args
    */
   public static void main (String[] args)
   {
      if (!(System.getProperty ("ControlBrowser.UTest", "").equals ("")))
      {
         testFlag = true;
      }

      /*
       * Initialise the browser.
       */
      displayBrowser = new DisplayBrowser ();

      displayBrowser.initialise ();
      displayBrowser.setupPanel ();
      JFrame frame = new JFrame ();
      frame.setDefaultCloseOperation (JFrame.EXIT_ON_CLOSE);
      frame.getContentPane ().add (displayBrowser);
      frame.pack ();

      frame.setSize (840, 640);
      int width = frame.getSize ().width;
      int height = frame.getSize ().height;
      Dimension screenSize = Toolkit.getDefaultToolkit ().getScreenSize ();
      frame.setLocation (screenSize.width / 2 - width / 2, screenSize.height
            / 2 - height / 2);

      frame.setTitle ("Display Browser");

      if (testFlag != true)
      {
         frame.addWindowListener (new MyWindowAdapter ());
         frame.setVisible (true);

         /*
          * This sleep is here so that the frame becomes visible before we try to
          * set the divider locations below. If we the panel isn't visible then the
          * attempts to set the divider locations won't work.
          */
         try
         {
            Thread.sleep (50);
         }
         catch (Exception ex)
         {
         }

         displayBrowser.spOne.setDividerLocation (0.66);
         frame.setVisible (true);
      }
      else
      {
         displayBrowser.performUtests ();
      }
   }

   /**
    * This method performs the unit tests for the class.
    */
   public void performUtests ()
   {
      String stringOne = new String ("request 1234 at 1173705412281 from 127.0.0.1 2000 to 127.0.0.6 8080 server_id Server1 interface InterfaceOne operation OperationOne {ParameterOne ParameterTwo} reply_to 2345 at 1173705412281");
      String stringTwo = new String ("request 2345 at 1173705412282 from 127.0.0.2 2000 to 127.0.0.1 8081 server_id Server1 interface InterfaceOne operation OperationTwo {ParameterOne ParameterTwo} reply_to 3456 at 1173705412282");
      String stringThree = new String ("request 3456 at 1173705412283 from 127.0.0.1 2010 to 127.0.0.6 8080 server_id Server1 interface InterfaceTwo operation OperationThree {ParameterOne ParameterTwo} reply_to 4567 at 1173705412283");
      String stringFour = new String ("request 6789 at 1173705412284 from 127.0.0.2 2001 to 127.0.0.1 8081 server_id Server1 interface InterfaceOne operation OperationFour {ParameterOne ParameterTwo} reply_to 6789 at 1173705412284");
      String stringFive = new String ("request 7890 at 1173705412285 from 127.0.0.1 2000 to 127.0.0.7 8080 server_id Server1 interface InterfaceTwo operation OperationFive {ParameterOne ParameterTwo} reply_to 7890 at 1173705412285");

      addMessage (stringOne);
      addMessage (stringTwo);
      addMessage (stringThree);
      addMessage (stringFour);
      addMessage (stringFive);

      if (eventTable.getModel ().getRowCount () != 5)
      {
         System.out.println ("TEST 1 FAILED");
      }

      displayMessage (0);

      if (! (displayArea.getText ().equals (stringOne)))
      {
         System.out.println ("TEST 2 FAILED");
      }
      filterField.setText ("1");

      filterMessages (FilterChange.ADDED);

      if (eventTable.getModel ().getRowCount () != 1)
      {
         System.out.println ("TEST 3 FAILED");
      }

      filterField.setText ("");

      filterMessages (FilterChange.REMOVED);

      if (eventTable.getModel ().getRowCount () != 5)
      {
         System.out.println ("TEST 4 FAILED");
      }

      System.exit (0);
   }

   /**
    * Inner class to handle file menu actions.
    */
   class FileMenuListener implements ActionListener
   {
      /**
       * This method is invoked when a component is clicked that has had this listener
       * added to it as an ActionListener.
       *
       * @param e The event.
       */
      public void actionPerformed (ActionEvent e)
      {
         if (((JMenuItem) e.getSource ()).equals (recordedMenuOpen))
         {
            start_decoder_exe (decoder_exe);

            if (browser == null)
            {
               browser = new JFileChooser ();
            }

            int returnVal = browser.showOpenDialog (DisplayBrowser.this);

            if (returnVal == JFileChooser.APPROVE_OPTION)
            {
               File file = browser.getSelectedFile ();
               recordedField.setText (file.getAbsolutePath ());
               messages.removeAllElements();
               while (eventTable.getRowCount() > 0)
               {
                  eventTableModel.removeRow(0);
               }
            }
            if ( output != null)
            {
               output.println ("input " + recordedField.getText());
               output.flush();
            }
         }
         else if (((JMenuItem) e.getSource ()).equals (fileMenuExit))
         {
            if ( output != null)
            {
               output.println ("quit");
               output.flush();
            }
            System.exit (0);
         }
      }
   }

   /**
    * Inner class to handle view menu actions.
    */
   class ViewMenuListener implements ActionListener
   {
      /**
       * Vector of removed columns so that they can be re-added as appropriate.
       */
      Vector<TableColumn> removedColumns = new Vector<TableColumn> ();

      /**
       * This method is invoked when a component is clicked that has had this listener
       * added to it as an ActionListener.
       *
       * @param e The event.
       */
      public void actionPerformed (ActionEvent e)
      {
         JMenuItem mi = (JMenuItem) e.getSource ();
         boolean add = true;

         if (mi.getIcon ().equals (tickIcon))
         {
            mi.setIcon (blankIcon);
            add = false;
         }
         else
         {
            mi.setIcon (tickIcon);
         }
         mi.revalidate ();

         TableColumn column = null;
         if (add)
         {
            for (int i = 0; i < removedColumns.size (); i++)
            {
               column = (TableColumn) removedColumns.elementAt (i);

               if (column.getIdentifier ().equals (mi.getText ()))
               {
                  eventTable.addColumn (column);
                  removedColumns.removeElementAt (i);
                  break;
               }
            }

            /*
             * Move the column back to where it was when we initialised the table.
             */
            eventTable.moveColumn (eventTable.getColumnModel ().getColumnCount () - 1,
                  columns.indexOf (column.getIdentifier ()));
         }
         else
         {
            column = eventTable.getColumn (mi.getText ());
            removedColumns.addElement (column);
            eventTable.removeColumn (column);
         }
      }
   }

   /**
    * Inner class to handle window events.
    */
   static class MyWindowAdapter extends WindowAdapter
   {
      /**
       * This method is invoked when a window component is closed that has had this
       * listener added to it as a WindowListener.
       */
      public void windowClosing (WindowEvent ev)
      {
         // Any clean up needs to be done here.
      }
   }

   /**
    * A specialised table for displaying the state items and controls.
    */
   class EventTable extends JTable
   {
      /**
       * Generated SerialVersionUID.
       */
      private static final long serialVersionUID = 804237917649712893L;

      /**
       * Constructor.
       *
       * @param dm The table model.
       */
      public EventTable (EventTableModel dm)
      {
         super (dm);

         /* Set the selection criteria */
         setRowSelectionAllowed (true);
         setColumnSelectionAllowed (false);
         setSelectionMode (ListSelectionModel.SINGLE_SELECTION);
         setAutoResizeMode (JTable.AUTO_RESIZE_OFF);
      }
   }

   /**
    * This is a table model for use in the event table, although it
    * just makes the table totally constant and not editable.
    */
   class EventTableModel extends DefaultTableModel
   {
      /**
       * Generated SerialVersionUID.
       */
      private static final long serialVersionUID = -807222931319747156L;

      /**
       * Construct a table with the given vector for column headings and a
       * number of blank rows.
       *
       * @param data Vector of names of columns.
       * @param rows Number of rows to create.
       */
      public EventTableModel (Vector data, int rows)
      {
         super (data, rows);
      }

      /**
       * This method is called to check to see if a specific cell is editable.
       * This method overrides the default isCellEditable.
       *
       * @param row The row location for the cell.
       * @param col The column location for the cell.
       */
      public boolean isCellEditable (int row, int col)
      {
         return false;
      }

      /**
       * Add a row of data to the table that represents the given message.
       *
       * @param message The message.
       */
      // From JCM 21/09/2007
      @SuppressWarnings("deprecation")
      public void addDebugMessage (DebugMessage message, int real_row)
      {
         Object[] row = new Object[columns.size () + 1];
         

         row[0] = message.requestID;
         //Date requestTime = new Date (Long.decode(message.requestTime) );
         //row[1] = requestTime.toGMTString();
         row[1] = message.requestTime;
         row[2] = message.fromIP;
         row[3] = message.fromPort;
         row[4] = message.toIP;
         row[5] = message.toPort;
         row[6] = message.serverID;
         row[7] = message.interfaceID;
         row[8] = message.operation;
         row[9] = message.replyTo;
         //Date replyTime = new Date (Long.decode(message.replyTime) );
         //row[10] = replyTime.toGMTString();
         row[10] = message.replyTime;
         row[11] = Integer.toString(real_row);

         this.addRow (row);
      }
       /*public void addDebugMessage (DebugMessage message)
      {
         Object[] row = new Object[columns.size ()];

         row[0] = message.requestID;
         row[1] = new java.sql.Time( java.lang.Integer.parseInt(message.requestTime)).toString();
         row[2] = message.fromIP;
         row[3] = message.fromPort;
         row[4] = message.toIP;
         row[5] = message.toPort;
         row[6] = message.serverID;
         row[7] = message.interfaceID;
         row[8] = message.operation;
         row[9] = message.replyTo;
         row[10] = new java.sql.Time( java.lang.Integer.parseInt(message.replyTime)).toString();

         this.addRow (row);
     }*/
   }

   /**
    * This is the renderer for used for rendering the table.
    */
   class EventTableRenderer implements TableCellRenderer
   {
      /** Colours used for rendering the cell. */
      private Color unselectedForeground = new Color (0, 0, 0);

      private Color unselectedBackground = new Color (255, 255, 255);

      private Color selectedForeground = UIManager.getColor ("textHighlightText");

      private Color selectedBackground = UIManager.getColor ("textHighlight");

      /**
       * Default constructor.
       */
      public EventTableRenderer ()
      {
      }

      /**
       * This method is called whenever a node is rendered in the table. The
       * colour is set and the component is created so that the table can be
       * drawn correctly.
       *
       * @param table : The table this node belongs to.
       * @param value : The value in the node.
       * @param isSelected : Boolean flag, true if the node is selected.
       * @param hasFocus : Boolean flag, true is this node has focus.
       * @param row : The row this cell is in.
       * @param column : The column this cell is in.
       */
      public Component getTableCellRendererComponent (JTable table,
            Object value,
            boolean isSelected,
            boolean hasFocus,
            int row, int column)
      {
         JComponent comp = null;

         if (value != null)
         {
            comp = new JLabel ((String) value);
            ((JLabel) comp).setHorizontalAlignment (SwingConstants.CENTER);

            if (isSelected)
            {
               comp.setForeground (selectedForeground);
               comp.setBackground (selectedBackground);
            }
            else
            {
               comp.setForeground (unselectedForeground);
               comp.setBackground (unselectedBackground);
            }

            comp.setFont (table.getFont ());

            comp.setOpaque (true);
         }

         return (comp);
      }
   }

   /**
    * Mouse listener for actions on the table.
    *
    * @see <code>MouseAdapter</code>
    */
   class MyMouseListener extends MouseAdapter
   {
      /**
       * This method is invoked when a component that had this listener added
       * to it as an MouseListener detects that the mouse has been clicked on
       * that component.
       *
       * @param e The event.
       */
      public void mouseClicked (MouseEvent ev)
      {
         EventTable table = (EventTable) ev.getSource ();

         int row = table.rowAtPoint (ev.getPoint ());

         displayMessage (row);
      }
   }

   /**
    * Button listener for the filter combobox and the clear button.
    */
   class ButtonListener implements ActionListener
   {
      /**
       * This method is invoked when a component is clicked that has had this listener
       * added to it as an ActionListener.
       *
       * @param e The event.
       */
      public void actionPerformed (ActionEvent e)
      {
         if (e.getSource () instanceof JButton &&
               ((JButton) e.getSource ()).equals (clearButton))
         {
            filterField.setText ("");
            filterMessages (FilterChange.CLEARED);
         }
         else if (e.getSource () instanceof JComboBox &&
               ((JComboBox) e.getSource ()).equals (filterCombo))
         {
            filterMessages (FilterChange.REMOVED);
         }
      }
   }

   /**
    * Key listener for the filter field so that we can dynamically update the
    * messages displayed in the table.
    */
   class FilterFieldListener extends KeyAdapter
   {
      /**
       * A record of the latest string that was entered into the filter field
       * before the latest change occured.
       */
      String latestString = null;

      /**
       * This method is invoked when a component that has had this listener added
       * to it as an KeyListener detects that a key has been released (after first
       * being pressed) .
       *
       * @param e The event.
       */
      public void keyReleased (KeyEvent e)
      {
         if (latestString == null)
         {
            latestString = filterField.getText ();
            filterMessages (FilterChange.ADDED);
         }
         else
         {
            if (!(filterField.getText ().equals (latestString)))
            {
               if (filterField.getText ().length () > latestString.length ())
               {
                  filterMessages (FilterChange.ADDED);
               }
               else
               {
                  filterMessages (FilterChange.REMOVED);
               }
               latestString = filterField.getText ();
            }
         }
      }
   }

   /**
    * An exception for use in the message thread class.
    */
   static class ExecFailure extends Exception
   {
      /**
       * Generated SerialVersionUID.
       */
      private static final long serialVersionUID = 5722447802058519698L;

      /**
       * Default constructor.
       */
      ExecFailure ()
      {
      }

      /**
       * Constructor with a user defined message.
       *
       * @param s The message.
       */
      ExecFailure (String s)
      {
         super (s);
      }
   }

   /**
    * This class is based on a thread and consumes input from a stream as long
    * as the stream is not null.
    */
   static class StreamConsumer extends Thread
   {
      /**
       * The input stream.
       */
      private InputStream stream;

      /**
       * An optional tag to identify the stream.
       */
      private String tag;

      /**
       * Constructor
       * @param streamTag The tag.
       * @param is The stream.
       */
      StreamConsumer (String streamTag, InputStream is)
      {
         tag = streamTag;
         if (!(streamTag.equals ("")))
         {
            tag = streamTag + " ";
         }

         stream = is;
         setDaemon (true);
      }

      /**
       * The run method. The thread runs until the stream is null.
       */
      public void run ()
      {
         BufferedReader br;
         String str = null;
         String prev = null;

         br = new BufferedReader (new InputStreamReader (stream));
         try
         {
            str = br.readLine ();
            // JM
            while (str != null && !str.equals("###End_Of_File###"))
            {
               {
                  int endIndex = str.indexOf ("request ");
                  if (endIndex >= 0)
                  {
                     if (endIndex > 1 )
                     {
                        str = str.substring (endIndex, str.length());
                     }
                     if (prev != null)
                     {
                        print (prev);
                        // JM
                        prev = null;
                     }
                     prev = new String (str);
                  }
                  else
                  {
                   if (prev != null)
                     {
                        prev = prev + "\n " + str;
                     }
                  }
               }
               str = br.readLine ();
            }
            // JM
            if (prev != null)
            {
               print (prev);
            }
         }
         catch (IOException e)
         {
         }
      }

      /**
       * The print method.
       * @param str The string read from the stream.
       */
      private void print (String str)
      {
         if (str.indexOf ("ERROR:") == -1)
         {
            displayBrowser.addMessage (str);
         }
         else
         {
            System.out.println (tag + str);
         }
      }
   }

   /**
    * This class is based on a thread and sets up the consumers to read
    * the input from the output streams of the Decoder process.
    */
   class MessageThread extends Thread
   {
      /**
       * The process.
       */
      private Process proc;

      /**
       * Constructor.
       * @param proc
       */
      MessageThread (Process proc)
      {
         this.proc = proc;
         setDaemon (true);
      }

      /**
       * The run method calls onto waitForProcess in a thread.
       */
      public void run ()
      {
         try
         {
            waitForProcess (proc);
         }
         catch (ExecFailure ex) {}
      }

      /**
       * This method sets up the output redirection and then waits for the
       * process to finish.
       * @param proc The process.
       * @return The result of the process.
       * @throws ExecFailure
       */
      private int waitForProcess (Process proc)
      throws ExecFailure
      {
         int res = 0;
         String str = null;

         try
         {
            /*
             * Set up consumer threads to consume output from the decoder.
             */
            output = new PrintStream (proc.getOutputStream ());
            StreamConsumer inputConsumer =
               new StreamConsumer ("", proc.getInputStream ());
            StreamConsumer errorConsumer =
               new StreamConsumer ("ERROR:", proc.getErrorStream ());

            inputConsumer.start ();
            errorConsumer.start ();

            /*
             * Wait for the process to finish.
             */
            res = proc.waitFor ();
         }
         catch (Exception ex)
         {
            if (str != null)
            {
               throw new ExecFailure (str);
            }
            throw new ExecFailure ();
         }

         return res;
      }
   }
}
