/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.recording.gui;

import javax.swing.JLabel;
import javax.swing.JOptionPane;

import com.thalesgroup.CdmwRecording.DaemonControl;

/**
 * This is a ChooserPanel that deals with a specific type of object, ports. Or, 
 * more accurately, integers.
 * 
 * @see ChooserPanel
 * 
 * @author Paul Graham
 */
public class PortChooserPanel extends ChooserPanel
{
   /**
    * Generated SerialVersionUID.
    */
   private static final long serialVersionUID = 1064929974244161512L;

   /**
    * Constructor. 
    */
   public PortChooserPanel ()
   {
      availableLabel = new JLabel ("Available Ports");
      
      selectedLabel = new JLabel ("Selected Ports");

      objectType = new String ("Port");
      
      singleSelection = true;
      allControls = false;
      setLiveMode (true);
   }
   
   /**
    * This method is called from the ChooserPanel class in live mode
    * when the Add or Add All buttons are pressed.
    */
   public void liveAdd (Object[] objects)
   {
      DaemonControl daemon = ControlBrowser.controlBrowser.getDaemon ();

      daemon.add_port (((Short) objects[0]).shortValue ()); 
   }
   
   /**
    * This method is called from the ChooserPanel class in live mode
    * when the Remove or Remove All buttons are pressed.
    */
   public void liveRemove (Object[] objects)
   {
      DaemonControl daemon = ControlBrowser.controlBrowser.getDaemon ();

      // PCR #1435
      if (daemon != null)
      {
         daemon.remove_port (((Short) objects[0]).shortValue ());
      }
   }

   /**
    * This method will add the value that is currently entered in the addField
    * to the list of available values. It will call the abstract method 
    * checkNewValue() so that implementing classes can verify the object.
    */
   public void addAvailable ()
   {
      Object object = (Object) addField.getText ();

      String errorMessage = null;

      Short currentPort = null;
      
      try
      {
         currentPort = Short.valueOf (object.toString ());
      }
      catch (NumberFormatException ex)
      {
         errorMessage = new String ("Invalid port value.");
      }
      
      if (errorMessage == null)
      {
         if (!(availableModel.contains (currentPort)))
         {
            availableModel.addElement (currentPort);
         }
         addField.setText ("");
      }
      else
      {
         JOptionPane.showMessageDialog (ControlBrowser.controlBrowser, 
                                        errorMessage, "Invalid input",
                                        JOptionPane.ERROR_MESSAGE);
      }
   }
}
