/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
#include <iostream>
#include <fstream>
#include <stdio.h>
#include <exception>

#include "testdaemon/TestRecDaemon.hpp"
#include "recordingtools/CdmwRecordingContainerManager_impl.hpp"
#include "recordingtools/CdmwRecordingDaemonControl_impl.hpp"

namespace Cdmw
{
namespace Recording
{

CPPUNIT_TEST_SUITE_REGISTRATION(TestRecDaemon);

TestRecDaemon::TestRecDaemon()
{
}
TestRecDaemon::~TestRecDaemon()  throw()
{
}

void TestRecDaemon::setUp()
{
  
}
void TestRecDaemon::tearDown()
{
    
}

//====================================================================== 
// Check object creation with NULL as parameter
//====================================================================== 
void TestRecDaemon::test_creation_1()
{    
    TEST_INFO("test recording daemon");  
           
    std::string nameServicePort = 
      Cdmw::TestUtils::CORBATestManager::instance()->get_property("NameServicePort",
                                                                  "5010");

    std::string corbaloc = std::string ("corbaname::localhost:");
    corbaloc += nameServicePort;
    corbaloc += "/NameService#";
    corbaloc += CdmwRecording::REC_DAEMON_CTRL_ID;
    CORBA::Object_var o_daemon = 
       Cdmw::TestUtils::CORBATestManager::instance()->get_ORB()->string_to_object (corbaloc.c_str());
    CdmwRecording::DaemonControl_var daemon = CdmwRecording::DaemonControl::_narrow(o_daemon);

    if(CORBA::is_nil(daemon.in()))
    {
        std::cout << "TestRecDaemon: ## could not find daemon " << corbaloc << std::endl;
        CPPUNIT_ASSERT(false);    
    }
    else
    {  
      std::string container_mgr_corbaloc = std::string ("corbaname::localhost:");
      container_mgr_corbaloc += nameServicePort;
      container_mgr_corbaloc += "/NameService#";
      container_mgr_corbaloc += CdmwRecording::REC_CONTAINER_MNGR_ID;
      CORBA::Object_var o_container = 
        Cdmw::TestUtils::CORBATestManager::instance()->get_ORB()->string_to_object (container_mgr_corbaloc.c_str());
      CPPUNIT_ASSERT(!CORBA::is_nil(o_container.in()));

      CORBA::String_var url__ = daemon->recorder_url();
      std::string recorder_url(url__.in());
      CPPUNIT_ASSERT(recorder_url.empty());

        std::cout << "TestRecDaemon: >> could find daemon " << corbaloc << std::endl;
        daemon->stop ();
        std::cout << "TestRecDaemon: >> stop " << corbaloc << std::endl;
        daemon->remove_all_interfaces ();
        std::cout << "TestRecDaemon: >> remove_all_interfaces " << corbaloc << std::endl;
        daemon->remove_all_operations ();
        std::cout << "TestRecDaemon: >> remove_all_operations " << corbaloc << std::endl;
        daemon->terminate();
        std::cout << "TestRecDaemon: >> terminate " << corbaloc << std::endl;
        CPPUNIT_ASSERT(true);    
    }
        std::cout << "TestRecDaemon: >> test finished " << corbaloc << std::endl;
}

}  // namespace Recording 
}  // namespace Cdmw

