/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "Foundation/orbsupport/CORBASystemExceptionWrapper.hpp"
#include <Foundation/ossupport/OS.hpp>
#include "testdaemon/TestRecDaemon.hpp"

int main(int argc, char** argv )
{   
    using namespace Cdmw::OsSupport;
    const std::string NAMING_SERVICE_PORT_OPTION  = "--NamePort";

    try
    {
        
#if (CDMW_ORB_VDR == tao )
    // get Name Service port
    std::string nameServicePort = OS::get_option_value (argc, argv, NAMING_SERVICE_PORT_OPTION);

    if (nameServicePort  == "no")
    {
        nameServicePort = "5010";
    }

    // Starting the Name Service : "nameserv -OAport <port>"
    std::string namingservice="Naming_Service";
    std::cout << namingservice.c_str() << " -m 1 -ORBEndpoint iiop://localhost:" << nameServicePort.c_str() << std::endl;
    OS::ProcessId idNaming = OS::create_process( namingservice, "-ORBEndpoint iiop://localhost:" + nameServicePort );

    OS::sleep(3000);

    std::cout << "start recording daemon" << std::endl;
    std::string rec_daemon = "cdmw_recordingdaemon";
    
    OS::sleep (2000);
    OS::ProcessId id_Recd = OS::create_process(
                   rec_daemon,
                   "--debug_file=/tmp/recording.log --CdmwProcessPort=59876 --bind_to_ns -ORBInitRef NameService=corbaloc::localhost:" + nameServicePort + "/NameService" );
    std::cout << "recording daemon started, pid=" << id_Recd << std::endl;
    OS::sleep (2000);
#endif
    // Initialize the ORB
    CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);
    //using Cdmw::Logging::LogManager;
    //LogManager::Init(argc, argv);

    // Get the root POA
    CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
    PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj.in());

    PortableServer::POAManager_var POAMgr = rootPOA->the_POAManager();
    POAMgr->activate();

    Cdmw::TestUtils::CORBATestManager::instance()->init(orb.in(),rootPOA.in(),argc,argv);
    Cdmw::TestUtils::CORBATestManager::instance()->set_property("NameServicePort",
                                                                nameServicePort);
    Cdmw::TestUtils::CORBATestManager::instance()->run_tests();
    Cdmw::TestUtils::CORBATestManager::instance()->summary();

#if (CDMW_ORB_VDR == tao )
    // And we kill launched process
        try 
        {
        std::cout << "kill naming service to finish test " << std::endl;
        OS::kill_process(idNaming);
        }
        catch(const CORBA::Exception& ex)
        {
            std::cerr << ex << std::endl;
            std::cerr << " **** TEST FAILED AT LINE " << __LINE__
                      << " IN FILE: " << __FILE__ << std::endl;
        }
#endif


    } 
    catch (const Cdmw::OrbSupport::CORBASystemExceptionWrapper & ex)
    {
        std::cerr << "+++ 1 +++ " << ex.what() << std::endl;
    }
    catch (const Cdmw::Exception & ex)
    {
        std::cerr << "+++ 2 +++ " << ex.what() << std::endl;
    } 
    catch(const CORBA::Exception& ex)  
    {        
        std::cerr << "+++ 3 +++ " << ex << std::endl;
    }
    catch(...)  
    {        
        std::cerr << "unexpected exception" << std::endl;
    }
}
