/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
#ifndef INCL_RECORDING_TEST_DECODER_HPP 
#define INCL_RECORDING_TEST_DECODER_HPP 

#include "recordingtools/InterfaceMap.hpp"
#include "recordingtools/Decoder.hpp"
#include "recordinglibrary/PacketStore.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include <string>

namespace Cdmw
{
namespace Recording
{

/**
*Purpose:
*<p>   Test of Decoder
*/
class TestDecoder : public CppUnit::TestFixture
{
    public:

        TestDecoder();

        virtual  ~TestDecoder()  throw();

        virtual void   do_tests();
             
        void test_decode_param_invalid();
        void test_decode_sys_exception_inv();
        
        void test_decode_param_1();
        void test_decode_param_2();
        void test_decode_param_3();
        void test_set_output_4();
        void test_decode_param_5();
        void test_sys_exception_6();
        void test_sys_exception_7();
        
        void test_decode_any();
        void test_decode_typecode();        
        void test_decode_principal();
        void test_decode_object();        
        void test_decode_string();
        void test_decode_wstring();        
        void test_decode_enum();
        void test_decode_alias();        
        void test_decode_struct();
        void test_decode_value();        
        void test_decode_valuebox();
        void test_decode_exception();        
        void test_decode_union();
        void test_decode_array();        
        void test_decode_sequence();      
        void test_decode_abstract();
        void test_decode_null();        
        void test_decode_short();
        void test_decode_long();        
        void test_decode_longlong();
        void test_decode_ushort();        
        void test_decode_ulong();
        void test_decode_ulonglong();        
        void test_decode_float();      
        void test_decode_double();
        void test_decode_longdouble();        
        void test_decode_boolean();
        void test_decode_char();        
        void test_decode_wchar();
        void test_decode_octet();                
        
        void setUp();
        
        void tearDown();
    
    private:           
    
       std::string url;
       FILE * fi;
       
       Message * ms_any;
       Message * ms_typecode;
       Message * ms_principal;
       Message * ms_object;
       Message * ms_string;       
       Message * ms_wstring;
       Message * ms_enum;
       Message * ms_alias;
       Message * ms_struct;
       Message * ms_value;
       Message * ms_valuebox;
       Message * ms_exception;
       Message * ms_union;
       Message * ms_array;
       Message * ms_sequence;
       Message * ms_abstract;
       Message * ms_short;
       Message * ms_long;
       Message * ms_longlong;
       Message * ms_ushort;
       Message * ms_ulong;
       Message * ms_ulonglong;
       Message * ms_float;
       Message * ms_double;       
       Message * ms_longdouble;
       Message * ms_boolean;
       Message * ms_char;
       Message * ms_wchar;
       Message * ms_octet;
       Message * ms_sys_exc;
              
       const octet * item;
       int start;
       CORBA::TypeCode * tc;
       bool swap;
       
       InterfaceMap * intfmap;
       CORBA::OperationDescription * op_desc;
       
       
       Message * load_message(std::ifstream * istrm, int delete_after);
       void load_messages(std::string filename);
          
       int start_param(Message * m);      
       int start_exception(Message * m);                    
             
CPPUNIT_TEST_SUITE( TestDecoder );
CPPUNIT_TEST( do_tests );
CPPUNIT_TEST_SUITE_END();


}; // End class TestDecoder 

}; // End namespace Recording
}; // End namespace Cdmw

#endif // INCL_RECORDING_TEST_DECODER_HPP

