/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
#include <iostream>
#include <fstream>
#include <stdio.h>

#include "testdecoder/TestDecoder.hpp"
#include "testdecoder/Big_impl.hpp"

namespace Cdmw
{
namespace Recording
{

CPPUNIT_TEST_SUITE_REGISTRATION(TestDecoder);

TestDecoder::TestDecoder()
{
}
TestDecoder::~TestDecoder()  throw()
{
}

void TestDecoder::setUp()
{   
    load_messages("messages.txt");
    
    op_desc = 0;
            
    //url = "abc" ; //valid url for interface map 
    //intfmap = 0 ; //new InterfaceMap(url.c_str());
     
    // Get operation definition
    //op_desc = intfmap.getOperationDef(ms_boolean->intf_rid, 
    //          (const char*) (ms_boolean->data + ms_boolean->operation_Offset)) ;
   
    //CORBA::ParameterDescription& param = op_desc->parameters[0] ; 
    //tc =  param.type.in() ; 
       
    
    fi = fopen("decoder_output","a+"); 
}
void TestDecoder::tearDown()
{   
    fclose (fi); 
    
    if(ms_any)       delete ms_any;
    if(ms_typecode)  delete ms_typecode;
    if(ms_principal) delete ms_principal;
    if(ms_object)    delete ms_object;
    if(ms_string)    delete ms_string;
    if(ms_wstring)   delete ms_wstring;
    if(ms_enum)      delete ms_enum;
    if(ms_alias)     delete ms_alias;
    if(ms_struct)    delete ms_struct;
    if(ms_value)     delete ms_value;
    if(ms_valuebox)  delete ms_valuebox;
    if(ms_exception) delete ms_exception;
    if(ms_union)     delete ms_union;
    if(ms_array)     delete ms_array;
    if(ms_sequence)  delete ms_sequence;
    if(ms_abstract)  delete ms_abstract;    
    if(ms_short)     delete ms_short;
    if(ms_long)      delete ms_long;
    if(ms_longlong)  delete ms_longlong;
    if(ms_ushort)    delete ms_ushort;
    if(ms_ulong)     delete ms_ulong;
    if(ms_ulonglong) delete ms_ulonglong; 
    if(ms_float)     delete ms_float;
    if(ms_double)    delete ms_double;
    if(ms_longdouble)delete ms_longdouble;
    if(ms_boolean)   delete ms_boolean;        
    if(ms_char)      delete ms_char;
    if(ms_wchar)     delete ms_wchar;
    if(ms_octet)     delete ms_octet;
    
    if(ms_sys_exc) delete ms_sys_exc;
}

//====================================================================== 
// load messages
//====================================================================== 
Message * TestDecoder::load_message(std::ifstream * istrm, int delete_after)
{
    int req ;
     
    if(!(*istrm)) 
    { 
        std::cout<< "#### error on reading file " << std::endl;       
        return 0 ;     
    }
     
    Message * msg = new Message();  
     
    try
    {
        (*istrm) >> req ; // 1 request 2 reply                
        (*istrm) >> msg->channel.src_addr;              
        (*istrm) >> msg->channel.dst_addr;        
        (*istrm) >> msg->channel.src_port;
        (*istrm) >> msg->channel.dst_port;
        (*istrm) >> msg->header.magic[0] ; 
        (*istrm) >> msg->header.magic[1] ;
        (*istrm) >> msg->header.magic[2] ;
        (*istrm) >> msg->header.magic[3] ;                 
        short sh;
        (*istrm) >> sh ; msg->header.version.major = (octet) sh ;
        (*istrm) >> sh ; msg->header.version.minor= (octet) sh ;
        (*istrm) >> sh ; msg->header.flags = (octet) sh;
           
        (*istrm) >> sh ; msg->header.message_type = (octet) sh;       
        (*istrm) >> msg->header.length;       
        (*istrm) >> msg->date.tv_sec ; 
        (*istrm) >> msg->date.tv_usec ;   
        (*istrm) >> msg->need_swap ;
                
        msg->data = new octet[ msg->header.length + 12 ] ;    
        for(int i = 0;  i < msg->header.length + 12; i++)
        { 
          (*istrm) >> sh ; msg->data[i] = (octet) sh ;          
        }  
                         
        (*istrm) >> msg->request_id  ;      
        (*istrm) >> msg->objectKey_Offset ;
        (*istrm) >> msg->objectKey_Length ; 
        (*istrm) >> msg->operation_Offset ;
                  
        std::string serv="", intf="";
                      
        if(req  == 1)
        {      
           while ((*istrm) && !(*istrm).eof() && serv == "" ) getline((*istrm), serv);  
           msg->server_id = new char[serv.length()+1];
           strncpy((char *) msg->server_id, serv.c_str() , serv.length() );  
                  
           while ((*istrm) && !(*istrm).eof() && intf == "" ) getline((*istrm), intf);     
           msg->intf_rid = new char[intf.length()+1];    
           strncpy((char *) msg->intf_rid, intf.c_str() , intf.length() );
        }   
    }
    catch(...)
    {
       std::cout<< "#### error on reading file " << std::endl;
       delete msg;
       return 0;
      
    }     
    msg->next = 0;
    msg->fragments = 0; 
    
    if(delete_after >0) 
    {
      Message * tmp = load_message(istrm, delete_after-1);
      delete tmp;          
    }
    return msg;             
}

void TestDecoder::load_messages(std::string filename)
{    
    std::ifstream istrm(filename.c_str() , std::ifstream::in );
       
    if(!istrm)
        std::cout<< "#### could not open input file "<< filename <<std::endl;
                     
    ms_any        = load_message(&istrm,1);     
    ms_typecode   = load_message(&istrm,1); 
    ms_principal  = load_message(&istrm,1); 
    ms_object     = load_message(&istrm,1); 
    ms_string     = load_message(&istrm,1); 
    ms_wstring    = load_message(&istrm,1); 
    ms_enum       = load_message(&istrm,1); 
    ms_alias      = load_message(&istrm,1); 
    ms_struct     = load_message(&istrm,1); 
    ms_value      = load_message(&istrm,1); 
    ms_valuebox   = load_message(&istrm,1); 
    ms_exception  = load_message(&istrm,1); 
    ms_union      = load_message(&istrm,1); 
    ms_array      = load_message(&istrm,1); 
    ms_sequence   = load_message(&istrm,1); 
    ms_abstract   = load_message(&istrm,1); 
    ms_short      = load_message(&istrm,1); 
    ms_long       = load_message(&istrm,1); 
    ms_longlong   = load_message(&istrm,1); 
    ms_ushort     = load_message(&istrm,1); 
    ms_ulong      = load_message(&istrm,1); 
    ms_ulonglong  = load_message(&istrm,1); 
    ms_float      = load_message(&istrm,1); 
    ms_double     = load_message(&istrm,1); 
    ms_longdouble = load_message(&istrm,1); 
    ms_boolean    = load_message(&istrm,1); 
    ms_char       = load_message(&istrm,1); 
    ms_wchar      = load_message(&istrm,1); 
    ms_octet      = load_message(&istrm,0);
    ms_sys_exc    = load_message(&istrm,0);
        
    istrm.close();           
}  
 
// from method Display-> Display_Operation_Info  (private)
int TestDecoder::start_param(Message * m)
{
    int index = 12 ; 

    // request_id
    index += 4 ;
    // response_flags
    index += 4 ;
    // addressing
    index += 4 ;
    // object key
    int len = * (int *) &(m->data[index]) ;
    index += 4 ;
    index += len ;
    index += (-index) & 3 ; //align to 4
    // operation
    len = * (int *) &(m->data[index]) ;
    index += 4 ;
    std::cout << "# operation "<< (char *) &(m->data[index]) << std::endl;
    index += len ;
    index += (-index) & 3 ; //align to 4
    // service contexts
    index += (-index) & 3 ; //align to 4
       
    int number = * (int *) &(m->data[index]) ;      
    index += 4 ;

    for (int each = number ; each > 0; each --)
    {   
       //std::cout << "# each "<< each << std::endl;      
       index += (-index) & 3 ; //align to 4
       index += 4 ;
       len = * (int *) &(m->data[index]) ;
       index += 4 + len ;
    }
    
    index += ((-index) & 7) ;
    std::cout << "==> start index of parameter "<< index << std::endl;
    return index ;   
}
// from method Display-> Display_Reply_Info  (private)
int TestDecoder::start_exception(Message * m)
{
    int index = 12 ; // Skip header

    // request_id
    index += 4 ;
    // status
    index += 4 ; // Skip status
    // service contexts
    int number = * (int *) & m->data[index] ;
    index += 4 ;
    for (int each = number ; each > 0; each --)
    {
       index += (-index) & 3 ; //align to 4
       index += 4 ;
       int len = * (int *) & m->data[index] ;
       index += 4 + len ;
    }

    index += ((-index) & 7) ;
    
    std::cout << "==> start index of exception "<< index << std::endl;
    return index ; 
}

//====================================================================== 
// Check Decode_Parameter method with invalid parameters
//======================================================================  
void TestDecoder::test_decode_param_invalid()
{ 
    std::cout<<"  ** test decode parameter with invalid parameters"<<std::endl;
    Decoder dec;  
 
    item  = 0 ; 
    start = -1;
    tc    = 0  ;    
    swap  = false ;   
   
    dec.Decode_Parameter(item, start , tc, swap);  
   
}
//====================================================================== 
// Check Decode_System_Exception method with invalid parameters 
//======================================================================  
void TestDecoder::test_decode_sys_exception_inv()
{
    std::cout<<"  ** test decode system exception with invalid parameters "<<std::endl;
    
    Decoder dec;
    
    start = -1; 
    item  = 0 ;
    swap  = false ;
    
    dec.Decode_System_Exception(item,start,swap);     
}
//====================================================================== 
// Check Decode_Parameter method with NULL as parameter item 
//======================================================================          
void TestDecoder::test_decode_param_1()
{
    std::cout<<"  ** test 1 "<<std::endl;
   
    Decoder dec;  
 
    item  = 0 ; 
    start = 0;
    tc    = new CORBA::TypeCode(CORBA::tk_boolean);      
    swap  = false ;   
    
    // ## segm fault 
    dec.Decode_Parameter(item, start , tc, swap);  
    delete tc;   
}   
//====================================================================== 
// Check Decode_Parameter method with NULL as parameter typecode
//====================================================================== 
void TestDecoder::test_decode_param_2()
{
    std::cout<<"  ** test 2 "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_boolean) ;     
    item  = ms_boolean->data + start ; 
    start = 0;
    tc    = 0;       
    swap  = ms_boolean->need_swap;
       
    // ## segm fault 
    dec.Decode_Parameter(item, start , tc, swap);         
}   
//====================================================================== 
// Check Decode_Parameter method with invalid start value  
//====================================================================== 
void TestDecoder::test_decode_param_3()
{
    std::cout<<"  ** test 3 "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_boolean) ;     
    item  = ms_boolean->data + start ; 
    start = -1 ;
    tc    = 0;       
    swap  = ms_boolean->need_swap;
       
    // ## segm fault 
    dec.Decode_Parameter(item, start , tc, swap);      
}   
//====================================================================== 
// Check SetOutput method with NULL as parameter
//====================================================================== 
void TestDecoder::test_set_output_4()
{ 
    std::cout<<"  ** test 4 "<<std::endl;
    
    Decoder dec; 
    dec.SetOutput(0);    
}   
//====================================================================== 
// Check Decode_Parameter method without setting output file
//====================================================================== 
void TestDecoder::test_decode_param_5()
{
    std::cout<<"  ** test 5 "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_boolean) ;     
    item  = ms_boolean->data + start ; 
    start = 0;
    tc    = new CORBA::TypeCode(CORBA::tk_boolean);    
    swap  = ms_boolean->need_swap;   
      
    dec.Decode_Parameter(item, start , tc, swap); 
    delete tc;     
}   
//====================================================================== 
// Check Decode_System_Exception method with NULL as first parameter
//====================================================================== 
void TestDecoder::test_sys_exception_6()
{
    std::cout<<"  ** test 6 "<<std::endl;
   
    Decoder dec;
    
    start = 0; 
    item  = 0 ;
    swap  = false ;
    
    // ## segm fault
    dec.Decode_System_Exception(item,start,swap);     
        
}   
//====================================================================== 
// Check Decode_System_Exception method with valid  parameters
//====================================================================== 
void TestDecoder::test_sys_exception_7()
{
    std::cout<<"  ** test of system exception "<<std::endl;
   
    Decoder dec;
    
    start = start_exception(ms_sys_exc) ;      
    item  = ms_sys_exc->data + start ; 
    start = 0;            
    swap  = ms_sys_exc->need_swap;   
   
    dec.SetOutput(fi);
    
    dec.Decode_System_Exception(item,start,swap);                
}   
//====================================================================== 
// Check Decode_Parameter method with parameter of type Any 
//====================================================================== 
void TestDecoder::test_decode_any()
{
    std::cout<<" test of type any "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_any) ;     
    item  = ms_any->data + start ; 
    start = 0;
    tc    = new CORBA::TypeCode(CORBA::tk_any);         
    swap  = ms_any->need_swap;   
   
    dec.SetOutput(fi);
    
    dec.Decode_Parameter(item, start , tc, swap);    
    delete tc;     
}   
//====================================================================== 
// Check Decode_Parameter method with parameter of type TypeCode
//====================================================================== 
void TestDecoder::test_decode_typecode()   
{
    std::cout<<" test of type TypeCode "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_typecode) ;     
    item  = ms_typecode->data + start ; 
    start = 0;
    tc    = new CORBA::TypeCode(CORBA::tk_TypeCode);         
    swap  = ms_typecode->need_swap;   
   
    dec.SetOutput(fi);
    
    dec.Decode_Parameter(item, start , tc, swap);    
    delete tc;      
}   
//====================================================================== 
// Check Decode_Parameter method with parameter of type Principal
//======================================================================      
void TestDecoder::test_decode_principal()
{  
    std::cout<<" test of type Principal "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_principal) ;     
    item  = ms_principal->data + start ; 
    start = 0;
    tc    = new CORBA::TypeCode(CORBA::tk_Principal);         
    swap  = ms_principal->need_swap;   
   
    dec.SetOutput(fi);
    
    dec.Decode_Parameter(item, start , tc, swap);    
    delete tc;       
       
}   
//====================================================================== 
// Check Decode_Parameter method with parameter of type Object
//====================================================================== 
void TestDecoder::test_decode_object()   
{
    std::cout<<" test of type object "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_object) ;     
    item  = ms_object->data + start ; 
    start = 0;
    tc    = _tc_Big ;    
    swap  = ms_object->need_swap;   
   
    dec.SetOutput(fi);
    
    dec.Decode_Parameter(item, start , tc, swap);    
}   
//====================================================================== 
// Check Decode_Parameter method with parameter of type String
//======================================================================      
void TestDecoder::test_decode_string()
{
    std::cout<<" test of type string "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_string) ;     
    item  = ms_string->data + start ; 
    start = 0;
    tc    = new CORBA::TypeCode(CORBA::tk_string);         
    swap  = ms_string->need_swap;   
   
    dec.SetOutput(fi);
    
    dec.Decode_Parameter(item, start , tc, swap);    
    delete tc;           
}   
//====================================================================== 
// Check Decode_Parameter method with parameter of type WString
//====================================================================== 
void TestDecoder::test_decode_wstring()   
{
    std::cout<<" test of type wstring "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_wstring) ;     
    item  = ms_wstring->data + start ; 
    start = 0;
    tc    = new CORBA::TypeCode(CORBA::tk_wstring);         
    swap  = ms_wstring->need_swap;   
   
    dec.SetOutput(fi);
    
    dec.Decode_Parameter(item, start , tc, swap);   
    delete tc;         
}   
//====================================================================== 
// Check Decode_Parameter method with parameter of type Enum
//======================================================================      
void TestDecoder::test_decode_enum()
{
    std::cout<<" test of type enum "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_enum) ;     
    item  = ms_enum->data + start ; 
    start = 0;
    tc    = _tc_color;         
    swap  = ms_enum->need_swap;   
   
    dec.SetOutput(fi);
    
    dec.Decode_Parameter(item, start , tc, swap);     
}   
//====================================================================== 
// Check Decode_Parameter method with parameter of type Alias
//====================================================================== 
void TestDecoder::test_decode_alias()    
{
    std::cout<<" test of type alias "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_alias) ;     
    item  = ms_alias->data + start ; 
    start = 0;
    tc    = _tc_shortM ;      
    swap  = ms_alias->need_swap;   
   
    dec.SetOutput(fi);
    
    dec.Decode_Parameter(item, start , tc, swap);    
}   
//====================================================================== 
// Check Decode_Parameter method with parameter of type Struct
//======================================================================     
void TestDecoder::test_decode_struct()
{
    std::cout<<" test of type struct "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_struct) ;     
    item  = ms_struct->data + start ; 
    start = 0;
    tc    = _tc_TimeOfDay;         
    swap  = ms_struct->need_swap;   
   
    dec.SetOutput(fi);
    
    dec.Decode_Parameter(item, start , tc, swap);   
}   
//====================================================================== 
// Check Decode_Parameter method with parameter of type Value
//====================================================================== 
void TestDecoder::test_decode_value()   
{
    std::cout<<" test of type value "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_value) ;     
    item  = ms_value->data + start ; 
    start = 0;
    tc    = new CORBA::TypeCode(CORBA::tk_value);         
    swap  = ms_value->need_swap;   
   
    dec.SetOutput(fi);
    
    // dec.Decode_Parameter(item, start , tc, swap);   
    delete tc;               
}   
//====================================================================== 
// Check Decode_Parameter method with parameter of type ValueBox
//======================================================================      
void TestDecoder::test_decode_valuebox()
{
    std::cout<<" test of type value box "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_valuebox) ;     
    item  = ms_valuebox->data + start ; 
    start = 0;
    tc    = new CORBA::TypeCode(CORBA::tk_value_box);         
    swap  = ms_valuebox->need_swap;   
   
    dec.SetOutput(fi);
    
    //dec.Decode_Parameter(item, start , tc, swap);    
    delete tc;             
}   
//====================================================================== 
// Check Decode_Parameter method with parameter of type Exception
//====================================================================== 
void TestDecoder::test_decode_exception()   
{
    std::cout<<" test of type exception "<<std::endl;
   
    Decoder dec;  
 
    start = start_exception(ms_exception) ;     
    item  = ms_exception->data + start ; 
    start = 0;
    tc    = Big::_tc_NotFound;         
    swap  = ms_exception->need_swap;   
   
    dec.SetOutput(fi);
    
    dec.Decode_Parameter(item, start , tc, swap);    
}   
//====================================================================== 
// Check Decode_Parameter method with parameter of type Union
//======================================================================      
void TestDecoder::test_decode_union()
{
    std::cout<<" test of type union "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_union) ;     
    item  = ms_union->data + start ; 
    start = 0;
    tc    = _tc_AgeOpt;         
    swap  = ms_union->need_swap;   
   
    dec.SetOutput(fi);
    
    dec.Decode_Parameter(item, start , tc, swap);   
}   
//====================================================================== 
// Check Decode_Parameter method with parameter of type Array
//====================================================================== 
void TestDecoder::test_decode_array()   
{
    std::cout<<" test of type array "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_array) ;     
    item  = ms_array->data + start ; 
    start = 0;
    tc    = _tc_stones;         
    swap  = ms_array->need_swap;   
   
    dec.SetOutput(fi);
    
    dec.Decode_Parameter(item, start , tc, swap);    
}   
//====================================================================== 
// Check Decode_Parameter method with parameter of type Sequence
//======================================================================      
void TestDecoder::test_decode_sequence()
{
    std::cout<<" test of type sequence "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_sequence) ;     
    item  = ms_sequence->data + start ; 
    start = 0;
    tc    = _tc_numbers;         
    swap  = ms_sequence->need_swap;   
   
    dec.SetOutput(fi);
    
    dec.Decode_Parameter(item, start , tc, swap);    
}  

//void TestDecoder::test_decode_fixed()  
 
//====================================================================== 
// Check Decode_Parameter method with parameter of type Abstract
//======================================================================     
void TestDecoder::test_decode_abstract()
{
    std::cout<<" test of type abstract "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_abstract) ;     
    item  = ms_abstract->data + start ; 
    start = 0;
    tc    = _tc_abstr;         
    swap  = ms_abstract->need_swap;   
   
    dec.SetOutput(fi);
    
    // dec.Decode_Parameter(item, start , tc, swap);    
}   
//====================================================================== 
// Check Decode_Parameter method with parameter of type null
//====================================================================== 
void TestDecoder::test_decode_null()    
{
   //TODO    CORBA::tk_null  
}   
//====================================================================== 
// Check Decode_Parameter method with parameter of type short
//======================================================================     
void TestDecoder::test_decode_short()
{
    std::cout<<" test of type short "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_short) ;     
    item  = ms_short->data + start ; 
    start = 0;
    tc    = new CORBA::TypeCode(CORBA::tk_short);         
    swap  = ms_short->need_swap;   
   
    dec.SetOutput(fi);
    
    dec.Decode_Parameter(item, start , tc, swap);    
    delete tc;                
}   
//====================================================================== 
// Check Decode_Parameter method with parameter of type long
//====================================================================== 
void TestDecoder::test_decode_long()     
{
    std::cout<<" test of type long "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_long) ;     
    item  = ms_long->data + start ; 
    start = 0;
    tc    = new CORBA::TypeCode(CORBA::tk_long);         
    swap  = ms_long->need_swap;   
   
    dec.SetOutput(fi);
    
    dec.Decode_Parameter(item, start , tc, swap);      
    delete tc;        
}   
//====================================================================== 
// Check Decode_Parameter method with parameter of type longlong
//======================================================================           
void TestDecoder::test_decode_longlong()   
{
    std::cout<<" test of type longlong "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_longlong) ;     
    item  = ms_longlong->data + start ; 
    start = 0;
    tc    = new CORBA::TypeCode(CORBA::tk_longlong);         
    swap  = ms_longlong->need_swap;   
   
    dec.SetOutput(fi);
    
    dec.Decode_Parameter(item, start , tc, swap);   
    delete tc;              
}   
//====================================================================== 
// Check Decode_Parameter method with parameter of type ushort
//======================================================================     
void TestDecoder::test_decode_ushort()     
{
     std::cout<<" test of type ushort "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_ushort) ;     
    item  = ms_ushort->data + start ; 
    start = 0;
    tc    = new CORBA::TypeCode(CORBA::tk_ushort);         
    swap  = ms_ushort->need_swap;   
   
    dec.SetOutput(fi);
    
    dec.Decode_Parameter(item, start , tc, swap);   
    delete tc;             
}   
//====================================================================== 
// Check Decode_Parameter method with parameter of type ulong
//======================================================================           
void TestDecoder::test_decode_ulong()   
{
    std::cout<<" test of type ulong "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_ulong) ;     
    item  = ms_ulong->data + start ; 
    start = 0;
    tc    = new CORBA::TypeCode(CORBA::tk_ulong);         
    swap  = ms_ulong->need_swap;   
   
    dec.SetOutput(fi);
    
    dec.Decode_Parameter(item, start , tc, swap);    
    delete tc;             
}   
//====================================================================== 
// Check Decode_Parameter method with parameter of type ulonglong
//======================================================================     
void TestDecoder::test_decode_ulonglong()     
{
    std::cout<<" test of type ulonglong "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_ulonglong) ;     
    item  = ms_ulonglong->data + start ; 
    start = 0;
    tc    = new CORBA::TypeCode(CORBA::tk_ulonglong);         
    swap  = ms_ulonglong->need_swap;   
   
    dec.SetOutput(fi);
    
    dec.Decode_Parameter(item, start , tc, swap);    
    delete tc;             
}   
//====================================================================== 
// Check Decode_Parameter method with parameter of type float
//======================================================================           
void TestDecoder::test_decode_float()       
{
    std::cout<<" test of type float "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_float) ;     
    item  = ms_float->data + start ; 
    start = 0;
    tc    = new CORBA::TypeCode(CORBA::tk_float);         
    swap  = ms_float->need_swap;   
   
    dec.SetOutput(fi);
    
    dec.Decode_Parameter(item, start , tc, swap);     
    delete tc;            
}   
//====================================================================== 
// Check Decode_Parameter method with parameter of type double
//======================================================================       
void TestDecoder::test_decode_double()   
{
    std::cout<<" test of type double "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_double) ;     
    item  = ms_double->data + start ; 
    start = 0;
    tc    = new CORBA::TypeCode(CORBA::tk_double);         
    swap  = ms_double->need_swap;   
   
    dec.SetOutput(fi);
    
    dec.Decode_Parameter(item, start , tc, swap);    
    delete tc;             
}   
//====================================================================== 
// Check Decode_Parameter method with parameter of type longdouble
//======================================================================     
void TestDecoder::test_decode_longdouble()     
{
    std::cout<<" test of type longdouble "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_longdouble) ;     
    item  = ms_longdouble->data + start ; 
    start = 0;
    tc    = new CORBA::TypeCode(CORBA::tk_longdouble);         
    swap  = ms_longdouble->need_swap;   
   
    dec.SetOutput(fi);
    
    dec.Decode_Parameter(item, start , tc, swap);    
    delete tc;             
}   
//====================================================================== 
// Check Decode_Parameter method with parameter of type boolean
//======================================================================           
void TestDecoder::test_decode_boolean()   
{
    std::cout<<" test of type boolean"<<std::endl;
   
    Decoder dec;  

    start = start_param(ms_boolean) ;     
    item  = ms_boolean->data + start ; 
    start = 0;
    tc    = new CORBA::TypeCode(CORBA::tk_boolean);        
    swap  = ms_boolean->need_swap;   
   
    dec.SetOutput(fi);
    
    dec.Decode_Parameter(item, start , tc, swap);    
    delete tc;   
}   
//====================================================================== 
// Check Decode_Parameter method with parameter of type char
//======================================================================     
void TestDecoder::test_decode_char()     
{
    std::cout<<" test of type char "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_char) ;     
    item  = ms_char->data + start ; 
    start = 0;
    tc    = new CORBA::TypeCode(CORBA::tk_char);         
    swap  = ms_char->need_swap;   
   
    dec.SetOutput(fi);
    
    dec.Decode_Parameter(item, start , tc, swap);    
    delete tc;             
}   
//====================================================================== 
// Check Decode_Parameter method with parameter of type wchar
//======================================================================           
void TestDecoder::test_decode_wchar()   
{
    std::cout<<" test of type wchar "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_wchar) ;     
    item  = ms_wchar->data + start ; 
    start = 0;
    tc    = new CORBA::TypeCode(CORBA::tk_wchar);         
    swap  = ms_wchar->need_swap;   
   
    dec.SetOutput(fi);
    
    dec.Decode_Parameter(item, start , tc, swap);    
    delete tc;             
}   
//====================================================================== 
// Check Decode_Parameter method with parameter of type octet
//======================================================================     
void TestDecoder::test_decode_octet()    
{
    std::cout<<" test of type octet "<<std::endl;
   
    Decoder dec;  
 
    start = start_param(ms_octet) ;     
    item  = ms_octet->data + start ; 
    start = 0;
    tc    = new CORBA::TypeCode(CORBA::tk_octet);         
    swap  = ms_octet->need_swap;   
   
    dec.SetOutput(fi);
    
    dec.Decode_Parameter(item, start , tc, swap);   
    delete tc;              
}  
//=============================================================== 
// do tests
//=============================================================== 
void TestDecoder::do_tests()
{         
    test_decode_param_1();
    test_decode_param_2();
    test_decode_param_3();
    test_decode_param_invalid();
    test_set_output_4();
    test_decode_sys_exception_inv();
    test_decode_param_5();
    test_sys_exception_6();
    test_sys_exception_7();
      
    test_decode_any();
    test_decode_typecode();        
    test_decode_principal();
    test_decode_object();        
    test_decode_string();
    test_decode_wstring();        
    test_decode_enum();
    test_decode_alias();        
    test_decode_struct();
    test_decode_value();        
    test_decode_valuebox();
    test_decode_exception();        
    test_decode_union();
    test_decode_array();        
    test_decode_sequence();          
    test_decode_abstract();
    
    test_decode_null();        
    test_decode_short();
    test_decode_long();        
    test_decode_longlong();
    test_decode_ushort();        
    test_decode_ulong();
    test_decode_ulonglong();        
    test_decode_float();      
    test_decode_double();
    test_decode_longdouble();        
    test_decode_boolean();
    test_decode_char();        
    test_decode_wchar();
    test_decode_octet();     
}

}  // namespace Recording 
}  // namespace Cdmw

