/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <iomanip>
#include "testdecoder/Big_impl.hpp" 
#include "testdecoder/Big.stub.hpp"
#include "tao/Typecode.h"

using namespace std;

int main(int argc, char * argv[])
{
    std::string port = "";
    
    std::cout << "***" << std::endl;
    //
    //Check count of arguments
    //
    if (argc < 1)
    {
        cout << "CORBA client: too few arguments" << endl;
        throw 0;
    }

    try 
    {
        // Initialize orb
        CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);
         
        CORBA::Object_var obj1 = orb->string_to_object(argv[1]);
        if (CORBA::is_nil(obj1)) 
        {
            cout << "CORBA client: Nil reference" << endl;
            throw 0;
        }
    
        // Narrow
        Big_var bigobj  = Big::_narrow(obj1);
        if (CORBA::is_nil(bigobj)) 
        {
            cout << "CORBA client: Argument is not a Big reference" << endl;
            throw 0;
        }
        
        CORBA::Object_var obj2 = orb->string_to_object(argv[2]);
        if (CORBA::is_nil(obj2)) 
        {
            cout << "CORBA client: Nil reference" << endl;
            throw 0;
        }
    
        // Narrow
        ChildOfAbstr_var cha_obj  = ChildOfAbstr::_narrow(obj2);
        if (CORBA::is_nil(cha_obj)) 
        {
            cout << "CORBA client: Argument is not a ChildOfAbstr reference" << endl;
            throw 0;
        }
        

        std::cout << "any" << std::endl;
        CORBA::Any ann;    
        bigobj->xx_any (ann); // tk_any and tk_null ??
        
        std::cout << "typecode" << std::endl;
        CORBA::TypeCode * tc = new CORBA::TypeCode(CORBA::tk_any);
        bigobj->xx_typecode(tc);
        delete tc;
               
        std::cout << "principal" << std::endl; // ??
        tc = new CORBA::TypeCode(CORBA::tk_Principal);
        bigobj->xx_typecode(tc);
        delete tc;
        
        std::cout << "object" << std::endl;
        //CORBA::Object_ptr  s = 0 ;
        bigobj->xx_object (cha_obj) ; 
        
        std::cout << "string" << std::endl;
        bigobj->xx_string ((const char *) "abc") ;
        
        std::cout << "wstring" << std::endl;
        bigobj->xx_wstring((const CORBA::WChar * )"wch") ;
        
        std::cout << "enum" << std::endl;
        bigobj-> xx_enum (red); 
        
        std::cout << "alias" << std::endl;
        shortM sh;
        bigobj->xx_alias (sh) ; 
        
        std::cout<< "struct" << std::endl;
        TimeOfDay t;
        bigobj->xx_struct(t);
        
        std::cout << "value" << std::endl; // ??
        tc = new CORBA::TypeCode(CORBA::tk_value);
        bigobj->xx_typecode(tc);
        delete tc;
        
        std::cout << "value box" << std::endl; // ??
        tc = new CORBA::TypeCode(CORBA::tk_value_box);
        bigobj->xx_typecode(tc);
        delete tc;
                
        std::cout << "exception" <<std::endl;
        bigobj->xx_exception();
        
        std::cout << "union" <<std::endl;
        AgeOpt ag;
        bigobj->xx_union(ag);
        
        std::cout << "array" <<std::endl;        
        stones arr;
        bigobj->xx_array(arr);
        
        std::cout << "sequence" <<std::endl;
        numbers nn;
        bigobj->xx_sequence(nn);
        
        std::cout << "abstract" <<std::endl;
        bigobj->xx_abstract(cha_obj);        
        
        std::cout << "short" << std::endl;
        bigobj->xx_short(11);
        
        std::cout << "long" << std::endl;
        bigobj->xx_long(12);
        
        std::cout << "longlong" << std::endl;
        bigobj->xx_longlong(13);

        std::cout << "ushort" << std::endl;
        bigobj->xx_ushort(14);

        std::cout << "ulong" << std::endl;
        bigobj->xx_ulong(15);
        
        std::cout << "ulonglong" << std::endl;
        bigobj->xx_ulonglong(16);
        
        std::cout << "float" << std::endl;
        bigobj->xx_float(0.5);
        
        std::cout << "double" << std::endl;
        bigobj->xx_double(0.6);
        
        std::cout << "longdouble" << std::endl;        
        CORBA::LongDouble lo ;
        bigobj->xx_longdouble(lo);
        
        std::cout << "boolean" << std::endl;
        bigobj->xx_boolean(true);
                
        std::cout << "char" << std::endl;
        bigobj->xx_char((CORBA::Char) 2);
        
        std::cout << "wchar" << std::endl;
        bigobj->xx_wchar((CORBA::WChar) "3");
        
        std::cout << "octet" << std::endl;
        bigobj->xx_octet((CORBA::Octet) 20);
               
        std::cout << "***" << std::endl;
    }
    catch (const CORBA::Exception &) 
    {
        cout << "CORBA client: Uncaught CORBA exception" << endl;
        return 1;
    }
    catch (...) 
    {
        return 1;
    }
    
    return 0;
}
