/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <fstream>
#include <stdio.h>

#include "testdisplay/TestDisplay.hpp"
#include "testdisplay/Transmission_impl.hpp"

namespace Cdmw
{
namespace Recording
{

CPPUNIT_TEST_SUITE_REGISTRATION(TestDisplay);

TestDisplay::TestDisplay()
{
}
TestDisplay::~TestDisplay()  throw()
{
}

void TestDisplay::setUp()
{
       
    char host_name[100] = "";
    gethostname(host_name, sizeof(host_name));
    url = "corbaloc::"+ std::string(host_name)+ ":6070/InterfaceRepository"; 
    std::cout<<"URL: ***"<< url << "***"<<std::endl;
    
    FILE * fi = fopen ("empty.txt","a+");
    fclose(fi);   
    fi = fopen("display_output6.txt", "a+");
    fclose(fi);
    fi = fopen("display_output7.txt", "a+");
    fclose(fi);
    fi = fopen("display_output9.txt", "a+");
    fclose(fi);
    fi = fopen("display_output10.txt", "a+");
    fclose(fi);
      
    
    std::ifstream istrm("records");
    std::ofstream ostrm("records_copy");
    std::string str;
    
    if (istrm >> str) ostrm << str;
    while(istrm >> str)  ostrm << ' ' << str ;
    
    istrm.close();
    ostrm.close();        
}
void TestDisplay::tearDown()
{   
    // remove files
    remove("empty.txt");
    remove("records_copy");   
    //remove("records"); 
    remove("display_output6.txt");
    remove("display_output7.txt");
    remove("display_output9.txt");
    remove("display_output10.txt");
}

void TestDisplay::write_new_data_to(std::string filename)
{
    std::string s;
    
    std::ifstream istrm("records_copy");
    std::ofstream ostrm(filename.c_str(), std::ios_base::app);

    if (istrm >> s) ostrm << s ;
    while(istrm >> s)  ostrm << ' ' << s ;
    
    istrm.close();
    ostrm.close();
}

bool TestDisplay::check_output_empty(std::string file)
{
    std::ifstream istrm(file.c_str(), std::ifstream::in);
    std::string s;
    
    if(istrm >> s)
    {
        istrm.close();       
        // Nearly empty (fix PCR1536)
        if(s == "###End_Of_File###") return true;
        else return false;
    }
    return true; 
}
bool TestDisplay::check_output_not_empty(std::string file)
{
    std::ifstream istrm(file.c_str(), std::ifstream::in);
    std::string s; 
    
    while( istrm >> s)
    {
      if(s != "") { istrm.close(); return true ;}       
    }
    istrm.close();
    return false ;
}
bool TestDisplay::check_only_headers(std::string file)
{
    std::ifstream istrm(file.c_str(), std::ifstream::in);
    std::string s; 
    bool found_param = false ;
    
    while(istrm >> s)
    {
        if(s.find("Parameters" ,0) != std::string::npos) 
            found_param = true;            

        if(found_param)
        {
            istrm >> s; // skip '{'
            istrm >> s;
            if(s.find("not",0) != std::string::npos)
            { 
                istrm.close(); 
                return true ;
            }
            else found_param = false;
        }
    }
    istrm.close();
    return false ;
}
//=============================================================
// make a copy of file to check later if it was changed or not
//============================================================== 
void TestDisplay::save(std::string file)
{
    std::string s; 
     
    std::ifstream istrm(file.c_str(), std::ifstream::in);
    
    copy = "";
    
    while(istrm >> s) 
        copy += s;
  
    istrm.close();
}
//=============================================================
// compare file content with saved copy
//=============================================================
bool TestDisplay::check_output_changed(std::string file)
{   
    std::string second; 
    std::string s;
     
    std::ifstream istrm(file.c_str(), std::ifstream::in);
      
    while(istrm >> s)
        second += s;
     
    istrm.close();
    
    return (second.length() > copy.length()) ;
}


//====================================================================== 
// Check Set_Output method with NULL as parameter
//====================================================================== 
void TestDisplay::test_setoutput_1()
{
    TEST_INFO("test 1");
    
    Decoder decoder;
        
    InterfaceMap intf_store(url.c_str());
    
    Display displ(intf_store, decoder);
    
    CPPUNIT_ASSERT(false == displ.Set_Output(0)); 
    
}
//====================================================================== 
// Check Set_Output method with “” as parameter
//====================================================================== 
void TestDisplay::test_setoutput_2()
{
    TEST_INFO("test 2" );
     
    Decoder decoder;
    
    InterfaceMap intf_store(url.c_str());
    
    Display displ(intf_store, decoder);
    
    CPPUNIT_ASSERT(false == displ.Set_Output("")); 
}
//====================================================================== 
// Check Display_File method with NULL as parameter
//======================================================================
void TestDisplay::test_display_file_3()
{
    TEST_INFO("test 3");  
    
    Decoder decoder;
    
    InterfaceMap intf_store(url.c_str());
    
    Display displ(intf_store, decoder);
    
    //## segm fault
    CPPUNIT_ASSERT(false == displ.Display_File(0)); 
}
//====================================================================== 
// Check Display_File method with “” as parameter
//======================================================================
void TestDisplay::test_display_file_4()
{
    TEST_INFO("test 4");  
     
    Decoder decoder;
   
    InterfaceMap intf_store(url.c_str());
    
    Display displ(intf_store, decoder);
    
    CPPUNIT_ASSERT(false == displ.Display_File("")); 
}
//====================================================================== 
// Check Display_File method with non-existing file as parameter
//======================================================================
void TestDisplay::test_display_file_5()
{
    TEST_INFO("test 5");  
     
    Decoder decoder;
  
    InterfaceMap intf_store(url.c_str());
    
    Display displ(intf_store, decoder);
    
    CPPUNIT_ASSERT(false == displ.Display_File("not_exist.txt")); 
}
//============================================================================== 
// Check Display_File method with empty file as parameter without setting flags
//==============================================================================
void TestDisplay::test_display_file_6()
{
    TEST_INFO("test 6");   
    
    Decoder decoder;
 
    InterfaceMap intf_store(url.c_str());
    
    Display displ(intf_store, decoder);
    
    // empty file
    CPPUNIT_ASSERT(true == displ.Set_Output("display_output6.txt")); 
    
    // empty file
    // ##segm fault
    CPPUNIT_ASSERT(true == displ.Display_File("empty.txt"));  
    
    write_new_data_to("empty.txt");
    
    CPPUNIT_ASSERT(check_output_empty("display_output6.txt"));    
}
//================================================================================== 
// Check Display_File method with non-empty file as parameter without setting flags
//==================================================================================
void TestDisplay::test_display_file_7()
{
    TEST_INFO("test 7");  
     
    Decoder decoder;
    
    InterfaceMap intf_store(url.c_str());
    
    Display displ(intf_store, decoder); 
    
    // empty file
    CPPUNIT_ASSERT(true == displ.Set_Output("display_output7.txt")); 
    
    // file from DebugPlugin
    std::cout<<"getting data/records"<< std::endl;
    CPPUNIT_ASSERT(true == displ.Display_File("records")); 
    
    CPPUNIT_ASSERT(check_output_not_empty("display_output7.txt"));
    save("display_output7.txt");
    
    write_new_data_to("records");
    
    CPPUNIT_ASSERT(!check_output_changed("display_output7.txt"));    
}
//====================================================================== 
// Check Display_File method without setting output file 
//======================================================================
void TestDisplay::test_display_file_8()
{
    TEST_INFO("test 8");  
     
    Decoder decoder;
    
    InterfaceMap intf_store(url.c_str());
    
    Display displ(intf_store, decoder); 
                   
    std::ofstream datei("display_output8.txt");
    
    // link output from cout to file
    std::streambuf* p = std::cout.rdbuf(datei.rdbuf());
             
    CPPUNIT_ASSERT(true == displ.Display_File("records"));  
        
    std::cout.rdbuf(p); 
    
    CPPUNIT_ASSERT(check_output_not_empty("display_output8.txt"));
   
    datei.close();
    remove("display_output8.txt");
}
//====================================================================== 
// Check Set_Flags method with Headers_Only as parameter
//======================================================================
void TestDisplay::test_setflags_9()
{
    TEST_INFO("test 9");
    
    Decoder decoder;
  
    InterfaceMap intf_store(url.c_str());
    
    Display displ(intf_store, decoder); 
    
    // empty file
    CPPUNIT_ASSERT(true == displ.Set_Output("display_output9.txt")); 
        
    CPPUNIT_ASSERT_NO_THROW(displ.Set_Flags (displ.M_Headers_Only));
        
    CPPUNIT_ASSERT(true == displ.Display_File("records"));  
    
    CPPUNIT_ASSERT(check_output_not_empty("display_output9.txt"));  
    // check only headers
    CPPUNIT_ASSERT(check_only_headers("display_output9.txt"));  
}
//====================================================================== 
// Check Set_Flags method with For_Ever_Reading  as parameter
//======================================================================

}  // namespace Recording 
}  // namespace Cdmw

