/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
#include <iostream>
#include <fstream>
#include <stdio.h>
#include <exception>

#include "testinterfacemap/TestInterfaceMap.hpp"


namespace Cdmw
{
namespace Recording
{

CPPUNIT_TEST_SUITE_REGISTRATION(TestInterfaceMap);

TestInterfaceMap::TestInterfaceMap()
{
}
TestInterfaceMap::~TestInterfaceMap()  throw()
{
}

void TestInterfaceMap::setUp()
{
    char host_name[100] = "";
    gethostname(host_name, sizeof(host_name));
    url = "corbaloc::"+ std::string(host_name)+ ":6070/InterfaceRepository"; 
    std::cout<<"URL: ***"<< url << "***"<<std::endl;
    
    interface = "IDL:Transmission:1.0"; 
    
    operation = "ready" ; 
}
void TestInterfaceMap::tearDown()
{
    
}
//====================================================================== 
// Check object creation with NULL as parameter
//====================================================================== 
void TestInterfaceMap::test_creation_1()
{    
    TEST_INFO("test 1");  
           
    InterfaceMap * interface_map ;
    char * nu = 0;
    interface_map = new InterfaceMap(nu);
    CPPUNIT_ASSERT(interface_map->Headers_Only());
    
    delete interface_map ;
}
//====================================================================== 
// Check object creation with empty url as parameter
//====================================================================== 
void TestInterfaceMap::test_creation_2()
{
    TEST_INFO("test 2" );  
        
    InterfaceMap * interface_map ;
    char * em = "";
    interface_map = new InterfaceMap(em);
    CPPUNIT_ASSERT(interface_map->Headers_Only());
    
    delete interface_map;          
}
//====================================================================== 
// Check object creation with invalid url as parameter
//====================================================================== 
void TestInterfaceMap::test_creation_3()
{             
    TEST_INFO("test 3" ); 

    InterfaceMap * interface_map ;
    char * inval = "ab" ;
    interface_map = new InterfaceMap(inval);
    CPPUNIT_ASSERT(interface_map->Headers_Only());
           
    delete interface_map;  
}
//====================================================================== 
// Check getOperationDef method with NULL as first parameter
//====================================================================== 
void TestInterfaceMap::test_getOperation_4()
{   
    TEST_INFO("test 4" ); 
         
    InterfaceMap * interface_map = new InterfaceMap(url.c_str());
     
    CORBA::OperationDescription * desc;
    
    //## segm fault 
    desc = interface_map->getOperationDef(0, "oper");  
    
    CPPUNIT_ASSERT(desc == 0);
      
    delete interface_map;          
}
//====================================================================== 
// Check getOperationDef method with NULL as second parameter
//====================================================================== 
void TestInterfaceMap::test_getOperation_5()
{
    TEST_INFO("test 5" );    
              
    InterfaceMap * interface_map = new InterfaceMap(url.c_str());
     
    CORBA::OperationDescription * desc;
    
    //## segm fault 
    desc = interface_map->getOperationDef(interface.c_str(), 0);   
    
    CPPUNIT_ASSERT(desc == 0);
      
    delete interface_map;             
}
//====================================================================== 
// Check getOperationDef method with empty first parameter
//====================================================================== 
void TestInterfaceMap::test_getOperation_6()
{
    TEST_INFO("test 6" );        
    
    InterfaceMap * interface_map = new InterfaceMap(url.c_str());
     
    CORBA::OperationDescription * desc;
    
    desc = interface_map->getOperationDef("", "oper");  
    
    CPPUNIT_ASSERT(desc == 0);
      
    delete interface_map;                 
}
//====================================================================== 
// Check getOperationDef method with empty second parameter
//====================================================================== 
void TestInterfaceMap::test_getOperation_7()
{
    TEST_INFO("test 7" );  
                  
    InterfaceMap * interface_map = new InterfaceMap(url.c_str());
     
    CORBA::OperationDescription * desc;
    
    desc = interface_map->getOperationDef(interface.c_str(), "");   
    
    CPPUNIT_ASSERT(desc == 0);
      
    delete interface_map;               
}
//============================================================================ 
// Check getOperationDef method with non-existing interface as first parameter
//============================================================================ 
void TestInterfaceMap::test_getOperation_8()
{
    TEST_INFO("test 8" );           

    InterfaceMap * interface_map = new InterfaceMap(url.c_str());
     
    CORBA::OperationDescription * desc;
    
    desc = interface_map->getOperationDef("interf", "oper");  
    
    CPPUNIT_ASSERT(desc == 0);
      
    delete interface_map;                        
}
//============================================================================= 
// Check getOperationDef method with non-existing operation as second parameter
//============================================================================= 
void TestInterfaceMap::test_getOperation_9()
{
    TEST_INFO("test 9" );  
                  
    InterfaceMap * interface_map = new InterfaceMap(url.c_str());
     
    CORBA::OperationDescription * desc;
    
    desc = interface_map->getOperationDef(interface.c_str(), "oper");   
    
    CPPUNIT_ASSERT(desc == 0);
      
    delete interface_map;                           
}
//====================================================================== 
// Check getOperationDef method with existing operation as parameter
//====================================================================== 
void TestInterfaceMap::test_getOperation_10()
{
    TEST_INFO("test 10: test will fail if no IR running" );
    TEST_INFO("test 10: PATH and LD_LIBRARY_PATH should be set to run TAO IFR_service and tao_ifr tool" ); 
    TEST_INFO("test 10: see readme.txt if test fails! ");               
    
    InterfaceMap * interface_map = new InterfaceMap(url.c_str());
     
    CORBA::OperationDescription_var desc;
    
    desc = interface_map->getOperationDef(interface.c_str(), operation.c_str());  
    
    CPPUNIT_ASSERT(desc != 0);
          
    delete interface_map;         
}

}  // namespace Recording 
}  // namespace Cdmw

