/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_TRACE_COLLECTOR_INIT_HPP 
#define INCL_TRACE_COLLECTOR_INIT_HPP 

#include <string>

#include <Foundation/logging/LogLevel.hpp>
#include <TraceAndPerf/tracelibrary/Message.hpp>
#include <Repository/idllib/CdmwNamingAndRepository.stub.hpp>

#include "tracecollector/CollectorOptions.hpp"

// definition of local constants in an unnammed namespace
namespace 
{ 
    const unsigned short SUCCESS = 0;
    const unsigned short FAILURE = 1;
    const unsigned short EXIT = 2;   
} // End namespace anonymous


/**
* Root namespace for CDMW runtime.
*/
namespace Cdmw
{
/**
* Namespace for CDMW Trace types and data.
*/
namespace Trace
{

    namespace CollectorInit
    {
        
        /**
         * Start the client thread in interactive mode and start the ORB.
         *
         * @param orb the ORB.
         * @param interactiveMode flag to set to true if interactive mode is wanted.
         */
        int run_processing (CORBA::ORB_ptr orb, bool interactiveMode);
        
        /**
         * Create the process agent
         *
         * @param global true if started as global collector
         * @param orb the ORB.
         * @param root_POA the root poa.
         *
         */
        void create_process_agent (
             bool global,
             CORBA::ORB_ptr orb,
             PortableServer::POA_ptr root_POA);
             
        /**
         * Get the options
         *
         * @param global true if started as global collector
         * @param argc nbr of arguments
         * @param argv ptr to argument table
         *
         * @param options the returned option data
         * @param new_argc nbr of rebuilt arguments
         * @param new_argv ptr to rebuilt argument table
         *
         * @return SUCCESS or FAILURE
         */
        int get_collector_options (bool global,
                                   int argc, char* argv[],
                                   CollectorOptions::Options &options,
                                   int &new_argc, char** &new_argv,
                                   const std::string& def_service_port,
                                   const std::string& def_process_port);
    
        /**
         * Initialise the orb and PlatformInterface
         *
         * @param argc nbr of arguments
         * @param argv ptr to argument table
         * @param platform_managed set if called by SystemMngt
         * @param naming_repos_url url of naming and repository
         *
         * @param orb the returned ORB.
         * @param root_POA the returned root poa.
         * @param use_miop parameter to indicate if we use miop (to call add_multicast)
         *
         */
        void init_processing (
             int argc, char* argv[],
             bool platform_managed,
             std::string& naming_repos_url,
             CORBA::ORB_out p_orb,
             PortableGroup::GOA_out p_root_GOA,
             CdmwNamingAndRepository::Repository_out p_repository_ref,
         bool use_miop, bool use_federated_controlled_clock=false);

    }; // End namespace CollectorInit 



}; // End namespace Trace
}; // End namespace Cdmw


#endif // INCL_TRACE_COLLECTOR_INIT_HPP

