/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_TRACE_COLLECTOR_OPTIONS_HPP
#define INCL_TRACE_COLLECTOR_OPTIONS_HPP

#include <TraceAndPerf/tracelibrary/Message.hpp>

/**
 * Root namespace for CDMW runtime.
 */
namespace Cdmw
{
/**
 * Namespace for CDMW Trace types and data.
 */
   namespace Trace
   {       
      namespace CollectorOptions
      {
         /**
         * This structure fully describes the options of the collector 
         */
         struct Options
         {
               bool interactive_mode;
               bool platform_managed;
               std::string collector_name;
               std::string trace_file_name;
               std::string component_name;
               std::string domain_name;
               Logging::LevelType level_value;
               Message::TraceFormat trace_format;
               std::string service_port;
               std::string process_port;
               std::string naming_repos_url;
               Message::CircularMode queue_strategy;
               long queue_size;
               long queue_str_size;
               long trace_file_bak_num;
               long trace_file_num;
               long trace_msg_num;
               int  trace_producer_inx;         
               std::string miop_corbaloc;
               int  nb_flush_area;
               long size_flush_area;
               int  area_nb_messages;
               std::string operational_clock_name;
         };

         /**
         * Start in interactive mode
         */
         static const char * const INTERACTIVE_OPT            = "--interactive";
         /**
         * Name of the trace collector
         */
         static const char * const COLLECTOR_NAME_OPT         = "--collector-name";
         /**
         * Name of the trace file.
         */
         static const char * const TRACE_FILE_NAME_OPT        = "--trace-file-name";
         /**
         * Filter on this component name.
         */
         static const char * const COMPONENT_NAME_OPT         = "--component-name";
         /**
         * Filter on this domain.
         */
         static const char * const DOMAIN_NAME_OPT            = "--domain-name";
         /**
         * Filter on this level.
         */
         static const char * const LEVEL_OPT                  = "--level";
         /**
         * URL of the CDMW Naming and Repository service.
         */
         static const char * const NAMING_REPOS_URL_OPT       = "--naming-repos-url";
         /**
         * 
         */
         static const char * const TRACE_PRODUCER_PATH_OPT    = "--trace-producer-path";
         /**
         * Strategy "drop_latest" or "remove_oldest".
         */
         static const char * const TRACE_Q_STRATEGY_OPT       = "--queue-strategy";
         /**
         * "drop_latest" strategy.
         */
         static const char * const TRACE_Q_STRATEGY_VAL_DL    = "drop_latest";
         /**
         * "remove_oldest" strategy.
         */
         static const char * const TRACE_Q_STRATEGY_VAL_RO    = "remove_oldest";
         /**
         * Size of the queue in KBytes.
         */
         static const char * const TRACE_Q_SIZE_OPT           = "--queue-size";
         /**
         * Size of the string queue in KBytes.
         */
         static const char * const TRACE_Q_STRING_SIZE_OPT    = "--queue-str-size";
         /**
         * Number of backup(s) of trace files.
         */
         static const char * const TRACE_FILE_BAK_NUM_OPT     = "--trace-file-bak-num";
         /**
         * Number of circular trace files.
         */
         static const char * const TRACE_FILE_NUM_OPT         = "--trace-file-num";
         /**
         * Number of messages in the circular trace files.
         */
         static const char * const TRACE_MSG_NUM_OPT          = "--trace-msg-num";
         /**
         * Format of a trace: Horizontal, Vertical (default) or XML.
         */
         static const char * const TRACE_FORMAT_OPT           = "--trace-format";
         /**
         * Horizontal format of traces.
         */
         static const char * const TRACE_FORMAT_VAL_V         = "V";
         /**
         * Vertical format of traces.
         */
         static const char * const TRACE_FORMAT_VAL_H         = "H";
         /**
         * Log4j XML format of traces.
         */
         static const char * const TRACE_FORMAT_VAL_X         = "X";
         /**
         * MIOP corbaloc, use corbaloc:miop.. syntax.
         */
         static const char * const COLLECTOR_MIOP_CORBALOC    = "--miop-corbaloc";
         /**
         * Size in bytes of the flush area in the buffer for IIOP local collector.
         */
         static const char * const FLUSH_AREA_SIZE            = "--flush-area-sz";
         /**
         * Number of flush areas in the buffer for IIOP local collector.
         */
         static const char * const FLUSH_AREA_NB              = "--flush-area-nb";
         /**
         * Number of messages in flush areas for IIOP local collector.
         */
         static const char * const AREA_NB_MESSAGES           = "--area-nb-messages";
         /**
         * Name of the Federated Controlled Clock used to get the operational time.
         */
         static const char * const OPERATIONAL_CLOCK_NAME     = "--operational-clock-name";
         /**
         * Name of the collector agent.
         */
         static const char * const LOC_AGENT_CORBALOC_ID      = "CollectorAgent";
         /**
         * Name of the collector agent 
         */
         static const char * const GLOB_AGENT_CORBALOC_ID     = "GlobalCollectorAgent";
         /**
         * Thread pool size of the POA.
         */
         static const unsigned short POA_THREAD_POOL_SIZE     = 5;

      }
   }
}
#endif /*INCL_TRACE_COLLECTOR_OPTIONS_HPP*/
