/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef CDMW_TRACE_COLLECTOR_INITUTILS_HPP
#define CDMW_TRACE_COLLECTOR_INITUTILS_HPP

#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <string>
#include <Foundation/orbsupport/CORBA.hpp>

namespace Cdmw
{
namespace Trace
{
  /**
   * Purpose:
   * <p> A class easing the access on the Naming Context of the Log
   * Collector in the CDMW daemon.
   */
  class InitUtils
  {  
  public:
    /**
     * Purpose:
     * <p> Get a Naming Interface from the NameService of the daemon
     *
     * @param orb The ORB.
     *
     * @see SystemException Exception raised probably during 
     */
    static Cdmw::CommonSvcs::Naming::NamingInterface*
    Get_naming_interface_from_daemon (CORBA::ORB_ptr orb)
      throw ();

    /**
     * Purpose:
     * <p> Create the NameService URL thanks to the port of the CDMW
     * daemon.
     *
     * @param system_port the system port
     */
    static void Create_nameservice_url (const std::string& system_port)
      throw ();

  protected:

    /**
     * Purpose:
     * <p> Constructor
     */
    InitUtils()
    throw() {}

    /**
     * Purpose:
     * <p> Destructor
     */
    virtual ~InitUtils()
    throw() {}

    /**
     * The URL on the NameService of the CDMW daemon
     */
    static std::string M_daemon_naming_context_url;

  };
}
}

#endif // CDMW_TRACE_COLLECTOR_INITUTILS_HPP
