/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef _CDMW_TRC_LOCAL_TRACECOLLECTOR_IMPL_HPP_
#define _CDMW_TRC_LOCAL_TRACECOLLECTOR_IMPL_HPP_


#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/logging/FilterMngr.hpp>
#include <Foundation/orbsupport/Codec.hpp>

#include <TraceAndPerf/idllib/CdmwTraceTraceProducer.skel.hpp>
#include "tracecollector/CdmwTraceLoggingCollector.skel.hpp"
#include "tracecollector/TraceCollector_impl.hpp"

// define type traits for CdmwTrace::MessagesWithHeader
DEFINE_VARIABLE_LENGTH_DATA_TRAITS(CdmwTrace::MessagesWithHeader);

namespace Cdmw
{
namespace Trace
{


class CollectorMngr;


/**
 * Default implementation of the local trace collector.
 */
  class LocalTraceCollector_impl: public TraceCollector_impl,
                                  virtual public POA_CdmwTrace::LocalTraceCollector
                                  
{
    public:
    
        // The type of container containing the collector data
        typedef std::list<CdmwTrace::TraceProducer::CollectorData> CollectorDataList;

        /**
         * Constructor.
         *
         * @param p_collector_mngr the collector manager.
         * @param pFfilterMngr the filter manager.
         * @param poa the POA.
         */
        LocalTraceCollector_impl (CollectorMngr* p_collector_mngr,
                                  Logging::FilterMngr* p_filter_mngr,
                                  PortableServer::POA_ptr poa,
                                  CORBA::ORB_ptr orb)
            throw();


        /**
         * Destructor.
         */
        virtual
        ~LocalTraceCollector_impl()
            throw();


        /**
         * Write messages.
         *
         * @param the_messages_header the general header for messages.
         * @param the_messages sequence of formatted messages.
         *
         * @exception CORBA::SystemException.
         */
        virtual void
        put_formatted_messages(
                const CdmwTrace::MessagesHeader& the_messages_header,
                const CdmwTrace::FormattedMessageSeq& the_messages)
            throw(CORBA::SystemException);

        /**
         * Write messages.
         *
         * @param the_logs sequence of logs
         *
         * @exception CORBA::SystemException.
         */
        virtual void
        add_log_sequence(const CdmwLogging::StructuredLogSeq& the_logs)
            throw(CORBA::SystemException);

        /**
         * Set the IIOP global trace collectors and what is needed for
         * serialization->buferization (buffer needed only when IIOP)
         *
         * @param collector_list the list of global trace collectors.
         */
        void
        set_global_collectors (
            CollectorDataList& collector_list,
            CORBA::ORB_ptr orb
        );


        /**
         * Set the global trace collectors miop reference
         *
         * @param miop_collectors the miop reference of global trace collectors.
         */
        void
        set_miop_collectors (CdmwTrace::Collector_ptr miop_collectors);

        /**
         * Purpose:
         * <p>
         * Redefines the PortableServer::ServantBase method
         * to return the correct POA.
         *
         * @return The m_poa member
         */
         virtual PortableServer::POA_ptr _default_POA();

    private:
    
        /**
         * Copy constructor.
         */
        LocalTraceCollector_impl(const LocalTraceCollector_impl& rhs)
            throw();


        /**
         * Assignment operator.
         */
        LocalTraceCollector_impl&
        operator=(const LocalTraceCollector_impl& rhs)
            throw();

        void add_operational_time(CdmwTrace::MessageHeader &message_header)
            throw(CORBA::SystemException);

        /**
         * Convert StructuredLog into MessagesHeader + FormattedMessage
         * @param log the log to convert. It is an input parameter.
         * @param header the header returned from the conversion. 
         * @param message the formatted message returned from
         * the conversion
         */
        void convert (const CdmwLogging::StructuredLog& log,
                      CdmwTrace::MessagesHeader& header,
                      CdmwTrace::FormattedMessage& message)
            throw();
  
        // list of global collectors (ECR-0145).
        CollectorDataList m_global_collectors;

        // miop (group) reference to the collectors (ECR-0288)
        CdmwTrace::Collector_var m_miop_collectors;

        /**
         * The codec to serialise sequence of messages coming from a process
         * before sending them to global collector
         */
        Cdmw::OrbSupport::Codec<CdmwTrace::MessagesWithHeader>* m_codec;

        /**
         * The POA
         */
        PortableServer::POA_var m_poa;

        /**
         * The ORB
         */
        CORBA::ORB_var m_orb;
};


} // namespace Trace
} // namespace Cdmw


#endif // _CDMW_TRC_LOCAL_TRACECOLLECTOR_IMPL_HPP_
