
/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef CDMW_TRACE_LOG_COLLECTOR_MACRO_DEFS_HPP
#define CDMW_TRACE_LOG_COLLECTOR_MACRO_DEFS_HPP

#include <Foundation/logging/LogLevel.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/common/Assert.hpp>
#include <iostream>

namespace Cdmw
{
   namespace Trace
   {
     /**
      *Purpose:
      *<p> Definition of the macro usefull for the logging of the Log Collector
      *
      */        

      class LogCollectorMacroDefs
      {
         public:
            
            /**
             * Get Logger for ccmcommon
             *
             * @return the Logger to be used in ccmcommon
             */
            static Cdmw::Logging::Logger * Get_Logger() throw ()
            {
               static Cdmw::Logging::Logger * Logger__ =
                  Cdmw::Logging::Logger::Get_Logger("CDMW.TRC.LogCollector");
               CDMW_ASSERT(Logger__ != 0);
               return Logger__;
            }
      };

#if (CDMW_TRACE_LEVEL >= 3 )
#define CDMW_WARN_LOG_COLLECTOR(msg)                                                      \
           std::cerr << __FILE__ << " " << __LINE__ << ": In " << __FUNCTION__   << " () " << msg << std::endl;
#else
#define CDMW_WARN_LOG_COLLECTOR(msg)
#endif

#if (CDMW_TRACE_LEVEL >= 2 )
#define CDMW_ERROR_LOG_COLLECTOR(msg)                                                     \
           std::cerr << __FILE__ << " " << __LINE__ << ": In " << __FUNCTION__  << " () " << msg << std::endl;
#else
#define CDMW_ERROR_LOG_COLLECTOR(msg)
#endif

#if (CDMW_TRACE_LEVEL >= 5 )
#define CDMW_DEBUG_LOG_COLLECTOR(msg)                                                     \
      if (Cdmw::Trace::LogCollectorMacroDefs::Get_Logger()->is_enabled_for(::Cdmw::Logging::CDMW_DEBUG_3)) { \
           std::cerr << __FILE__ << " " << __LINE__ << ": In " << __FUNCTION__  << " () " << msg << std::endl; \
      }
#else
#define CDMW_DEBUG_LOG_COLLECTOR(msg)
#endif


   }
}
#endif
