/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef CDMW_TRACE_LOG_COLLECTOR_PLUGIN_HPP
#define CDMW_TRACE_LOG_COLLECTOR_PLUGIN_HPP

#include <SystemMngt/plugin/Plugin.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtDaemon.stub.hpp>

namespace Cdmw
{
namespace Trace
{

   class LogCollectorPlugin : public Cdmw::PlatformMngt::Plugin
   {
      public:
         /**
          * Purpose:
          * <p> Constructor
          */
         LogCollectorPlugin();

         /**
          * Purpose:
          * <p> Destructor
          */
         virtual ~LogCollectorPlugin() throw();
    
         virtual void set_up();
         virtual void run();
         virtual void tear_down();
         
      private:
         
         CORBA::ORB_var m_orb;
         PortableServer::POA_var m_rootPOA;
         std::size_t m_orb_threadpool_size;
         /**
          * - period the period P. The logs will be sent to the Trace Collector 
          * each P milliseconds.
          *
          * - circular_buffer_size the size of the circular buffer.
          *
          * - max_logs_to_send the maximum number of logs to
          * send at each period. This parameter should avoid to lock 
          * the access to the circular buffer during a too long time.
          *
          * - chunk_size the maximum size in bytes of a sequence
          * to send to the Local Trace Collector. This parameter
          * should avoid to flood the ORB buffers during a sending 
          * to the Local Trace Collector.
          *
          */
         std::size_t m_period;
         std::size_t m_circular_buffer_size;
         std::size_t m_max_logs_to_send;
         std::size_t m_chunk_size;

   };
}
}

// Macro to make MyPlugin visible to the Supervision
CDMW_DLL_HANDLE( Cdmw::Trace::LogCollectorPlugin );
#endif // CDMW_TRACE_LOG_COLLECTOR_PLUGIN_HPP
