/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef _CDMW_TRC_TRACECOLLECTOR_HPP_
#define _CDMW_TRC_TRACECOLLECTOR_HPP_


#include <string>

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/logging/FilterMngr.hpp>
#include <TraceAndPerf/idllib/CdmwTraceCollector.skel.hpp>


namespace Cdmw
{
namespace Trace
{


class CollectorMngr;


/**
 * Default implementation of the global trace collector.
 */
class TraceCollector_impl : virtual public POA_CdmwTrace::Collector,
                            virtual public PortableServer::RefCountServantBase
{
    public:
        /**
         * Constructor.
         *
         * @param p_collector_mngr CollectorMngr in charge of collecting the
         *                         messages.
         * @param p_filter_mngr    FilterMngr in charge of determining whether a
         *                         Level/Domain/Component must be traced or not.
         */
        TraceCollector_impl (CollectorMngr* p_collector_mngr,
                             Logging::FilterMngr* p_filter_mngr)
            throw();


        /**
         * Destructor.
         */
        virtual
        ~TraceCollector_impl()
            throw();


        /**
         * Get the filters.
         *
         * @return the filters
         *
         * @exception CORBA::SystemException.
         */
        virtual CdmwTrace::TraceFilterSeq*
        get_trace_filters()
            throw(CORBA::SystemException);


        /**
         * Activate the specified trace filter.
         *
         * @param component_name the component name
         * @param domain the domain name
         * @param level the level value
         *
         * @exception CORBA::SystemException
         */
        virtual void
        activate_level (const char* component_name,
                        const char* domain,
                        CORBA::ULong level)
            throw(CORBA::SystemException);


        /**
         * Deactivate the specified trace filter.
         *
         * @param component_name the component name
         * @param domain the domain name
         * @param level the level value
         *
         * @exception CORBA::SystemException
         */
        virtual void
        deactivate_level (const char* component_name,
                          const char* domain,
                          CORBA::ULong level)
            throw(CORBA::SystemException);


        /**
         * Write a message.
         *
         * @param the_messages_header contains the general header for messages
         * @param the_messages contains sequence of formatted messages
         *
         * @exception CORBA::SystemException.
         */
        virtual void
        put_formatted_messages(
                const CdmwTrace::MessagesHeader& the_messages_header,
                const CdmwTrace::FormattedMessageSeq& the_messages)
            throw(CORBA::SystemException);


        /**
         * Update.
         *
         * @param trace_producer trace producer object.
         *
         * @exception CORBA::SystemException.
         */
        virtual void
        update(CdmwTrace::TraceProducer* p_trace_producer)
            throw(CORBA::SystemException);

    protected:
            
        /**
         * Contains the reference on the object in charge of
         * collecting the trace message
         */
        CollectorMngr* m_collector_mngr;


        /**
         * Contains the reference on the object in charge of
         * informing the trace system, if an information, from
         * a domain must be traced or not
         */
        Logging::FilterMngr* m_filter_mngr;
    
    private:
    
        /**
         * Copy constructor.
         */
        TraceCollector_impl(const TraceCollector_impl& rhs)
            throw();


        /**
         * Assignment operator.
         */
        TraceCollector_impl&
        operator=(const TraceCollector_impl& rhs)
            throw();


};


} // namespace Trace
} // namespace Cdmw


#endif // _CDMW_TRC_TRACECOLLECTOR_HPP_
