#ifndef INCL_TRACE_VERTICALTRACEPRINTER_HPP
#define INCL_TRACE_VERTICALTRACEPRINTER_HPP

#include "tracecollector/TracePrinter.hpp"
#include <iostream>

namespace Cdmw
{
   namespace Trace
   {
      /**
      *Purpose:
      *<p>    This class, whose interface is defined in TracePrinter,
      *       is used to print, in a vertical format, the messages 
      *       comming from any of the Trace Producers.
      *
      */
      class VerticalTracePrinter : public TracePrinter
      {
         public:
            /**
            * Purpose:
            * <p> Default public Constructor
            *@param time_mode Can be either Cdmw::OsSupport::OS::LOCAL_TIME or
            *                 Cdmw::OsSupport::OS::GMT_TIME
            *@exception
            */
            VerticalTracePrinter(OsSupport::OS::TimeMode time_mode) throw(): TracePrinter(time_mode) {}
            /**
            * Purpose:
            * <p> Destructor
            * 
            *@param
            *@exception
            */
            virtual ~VerticalTracePrinter() throw() {}
       
            /**
            * Purpose:
            * <p> prints the body of the message, vertically
            * 
            *@param os std::ostream to write to
            *@param Data Data containing the information to read from,
            *            like the message's header, the timestamp, etc.
            *@exception
            */
            virtual void print_message(std::ostream & os, const TraceCircularBuffer::TraceMessageRaw & Data) const throw();
            /**
            * Purpose:
            * <p> Prints an error message
            * 
            *@param os std::ostream to write to
            *@param errMesg Error message to write
            *@exception
            */
            virtual void print_error_message (std::ostream & os, const std::string& errMesg) const throw();
            /**
            * Purpose:
            * <p> Prints the header of the message in the circular file referenced by file_number
            * 
            *@param os std::ostream to write to
            *@param file_number Index of the circular file to write to
            *@exception
            */            
            virtual void print_header (std::ostream & os, int file_number) const throw();
            /**
            * Purpose:
            * <p> Prints the end of the trace
            * 
            *@param os std::ostream to write to
            *@exception
            */
            virtual void print_end_of_trace (std::ostream & os) const throw();
      };
   }
}
#endif /*INCL_TRACE_VERTICALTRACEPRINTER_HPP*/
