
/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "tracecollector/InitUtils.hpp"
#include <Foundation/common/Locations.hpp>
#include "tracecollector/CdmwTraceLoggingCollector.stub.hpp"
#include "tracecollector/LogCollectorMacroDefs.hpp"
#include <iostream>
// #include "tracecollector/LogCollector_impl.hpp"

namespace Cdmw
{

namespace Trace
{

// Initialization of static members
std::string InitUtils::M_daemon_naming_context_url = "";

Cdmw::CommonSvcs::Naming::NamingInterface* InitUtils::Get_naming_interface_from_daemon (CORBA::ORB_ptr orb) 
throw ()
{
  CDMW_DEBUG_LOG_COLLECTOR("");
  Cdmw::CommonSvcs::Naming::NamingInterface* ni = 0;

  // Check the URL was set
  if (M_daemon_naming_context_url != "") {

    CDMW_DEBUG_LOG_COLLECTOR("Create the Naming Interface");
    CDMW_DEBUG_LOG_COLLECTOR(M_daemon_naming_context_url);
    CORBA::Object_var obj = orb->string_to_object(M_daemon_naming_context_url.c_str()) ;
    CosNaming::NamingContext_var nc = CosNaming::NamingContext::_narrow(obj.in());
    ni = new Cdmw::CommonSvcs::Naming::NamingInterface(nc.in());

  } else { // The URL was not set

    // It could be normal if the Log Collector library is not using and that
    // the old mechanism of the TRC service is using.
    std::cerr << __FILE__ << " - WARN: No URL set for the NameService Context of the CDMW daemon." << std::endl;

  }
  return ni;
}

void InitUtils::Create_nameservice_url (const std::string& system_port)
throw ()
{
  M_daemon_naming_context_url = "corbaloc::localhost:";
  M_daemon_naming_context_url += system_port;
  M_daemon_naming_context_url += "/NameService";
}

} // End namespace Trace
} // End namespace Cdmw
