/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>


#include "tracecollector/CollectorMngr.hpp"
#include "tracecollector/TraceCollector_impl.hpp"


namespace Cdmw
{
namespace Trace
{


/**
 * Constructor.
 */
TraceCollector_impl::TraceCollector_impl (
        CollectorMngr* p_collector_mngr,
        Logging::FilterMngr* p_filter_mngr)
    throw()
        : m_collector_mngr(p_collector_mngr),
          m_filter_mngr(p_filter_mngr)
{
    // NOOP
}


/**
 * Destructor.
 */
TraceCollector_impl::~TraceCollector_impl()
    throw()
{
    // NOOP
}


/**
 * Get the filters.
 */
CdmwTrace::TraceFilterSeq*
TraceCollector_impl::get_trace_filters()
    throw(CORBA::SystemException)
{
    try 
    {
        // get the list of filter from the root component
        Logging::FilterDataList filter_list;
        m_filter_mngr->get_log_filters (filter_list);
        
        // convert the list in corba sequence
        CdmwTrace::TraceFilterSeq_var filter_seq = new CdmwTrace::TraceFilterSeq;
        
        unsigned int seq_idx = 0;
        
        for (Logging::FilterDataList::iterator iter = filter_list.begin(); 
             iter != filter_list.end(); iter++)
        {
            CORBA::ULong seq_len = filter_seq->length() + 1;
            filter_seq->length(seq_len);

            filter_seq[seq_idx].the_component_name = (iter->m_component_name).c_str();
            filter_seq[seq_idx].the_domain = (iter->m_domain_name).c_str();
            filter_seq[seq_idx].the_value = iter->m_levels;

            seq_idx++;
        }
                            
        return filter_seq._retn();

    }
    catch (const OutOfMemoryException&) 
    {
        throw CORBA::NO_RESOURCES();
    }    
}


/**
 * Activate the specified trace filter.
 */
void
TraceCollector_impl::activate_level (const char* component_name,
                                     const char* domain,
                                     CORBA::ULong level)
    throw(CORBA::SystemException)
{
    m_filter_mngr->activate_level (component_name, domain, level);
}


/**
 * Deactivate the specified trace filter.
 */
void
TraceCollector_impl::deactivate_level (const char* component_name,
                                       const char* domain,
                                       CORBA::ULong level)
    throw(CORBA::SystemException)
{
    m_filter_mngr->deactivate_level (component_name, domain, level);
}


/**
 * Write a message.
 */
void
TraceCollector_impl::put_formatted_messages (
        const CdmwTrace::MessagesHeader& the_messages_header,
        const CdmwTrace::FormattedMessageSeq& the_messages)
    throw(CORBA::SystemException)
{
    try 
    {
        for (size_t i_message = 0;
             i_message < the_messages.length();
             ++i_message)
        {
            m_collector_mngr->write_message (the_messages_header,
                                             the_messages[i_message]);
        }
    }
    catch(const OutOfMemoryException&) 
    {
        throw CORBA::NO_RESOURCES(OrbSupport::NO_RESOURCES,
                                  CORBA::COMPLETED_NO);
    }
}


/**
 * Update.
 */
void
TraceCollector_impl::update(CdmwTrace::TraceProducer* p_trace_producer)
    throw(CORBA::SystemException)
{
    // NOOP
}


} // namespace Trace
} // namespace Cdmw
