/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iomanip>
#include "tracecollector/VerticalTracePrinter.hpp"

namespace Cdmw
{
   namespace Trace
   {
      //
      // prints vertical message
      // 
      void VerticalTracePrinter::print_message(std::ostream & os, const TraceCircularBuffer::TraceMessageRaw & Data) const
         throw()
      {
         // format and print message
         os << "Date              : " << timestamp_to_string(Data.m_messageHeader.m_timestamping,"%Y-%m-%dT%H:%M:%SZ") << std::endl;
         os << "Time Stamp        : " << Data.m_messageHeader.m_timestamping.seconds;
         os << " seconds  "      << Data.m_messageHeader.m_timestamping.microseconds << " microseconds" << std::endl;

         // Operational Time
         if (Data.m_messageHeader.m_operational_timestamping.seconds)
         {
            os << "Operational Date  : " << timestamp_to_string(Data.m_messageHeader.m_operational_timestamping,"%Y-%m-%dT%H:%M:%SZ") << std::endl;
            os << "Operational Stamp : " << Data.m_messageHeader.m_operational_timestamping.seconds;
            os << " seconds  "      << Data.m_messageHeader.m_operational_timestamping.microseconds << " microseconds" << std::endl;
         }
     
         os << "Host Name         : " << Data.m_mainHeader.m_pHostName << std::endl;
         os << "Application Name  : " << Data.m_mainHeader.m_pApplicationName << std::endl;
         os << "Process Name      : " << Data.m_mainHeader.m_pProcessName << std::endl;
         os << "Process Id        : " << Data.m_mainHeader.m_pid << std::endl;
         os << "Component Name    : " << Data.m_messageHeader.m_pComponentName << std::endl;
         os << "Domain            : " << Data.m_messageHeader.m_pUserDomain << std::endl;
         os << "Level             : " << "0x" << std::setw(8) << std::setfill('0') << std::hex << Data.m_messageHeader.m_userLevel << std::dec << std::endl;
         os << "Thread Id         : " << Data.m_messageHeader.m_tid << std::endl;
         os << "File Name         : " << Data.m_messageHeader.m_pFileName << std::endl;
         os << "Line              : " << Data.m_messageHeader.m_fileLine << std::endl;
         os << "NDC               : " << Data.m_ndc << std::endl;
         os << "==>" << std::endl;
     
         os << Data.m_pBodyMessage << std::endl;
         os << std::endl;
      }

      //
      // prints vertical error message
      // 
      void VerticalTracePrinter::print_error_message (std::ostream & os, const std::string& errMesg) const
         throw()
      {
         os << TRACE_ERR_MESG2 << std::endl;
         os << errMesg << std::endl;
         os << TRACE_ERR_MESG1 << std::endl;
         os << std::endl;
      }
     
      //
      // prints vertical header
      // 
      void VerticalTracePrinter::print_header (std::ostream & os, int file_number) const
         throw()
      {
         os << TRACE_HEADER1 << std::endl;

         os << TRACE_HEADER2;
         if (file_number  > 0)
         {
            os << file_number << TRACE_HEADER3;
         }
         os << std::endl;

         os << TRACE_HEADER1 << std::endl;
         os << std::endl;
      }
     
      //
      // prints enf of trace
      //          
      void VerticalTracePrinter::print_end_of_trace (std::ostream & os) const
         throw()
      {
         os << END_OF_TRACE_FILE << std::endl;
      }
   
   }
}

