/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <Foundation/ossupport/OS.hpp>
#include "testlogcollector/SIM_Collector.hpp"

namespace Cdmw
{
    namespace Trace
    {

        /**
         *Implementation notes: [if no pertinent write none ]
         *<p>
         *Portability issues: [if no pertinent write none ]
         *<p>
         */

        // CPPUNIT_TEST_SUITE_REGISTRATION( SIM_Collector );

        SIM_Collector::SIM_Collector()
            throw()
        {
            m_nbMessages = 0;
            m_the_messages = new CdmwTrace::FormattedMessageSeq;
            m_the_messages->length(0);
            m_test_status = false;

        }


        SIM_Collector::~SIM_Collector()
            throw()
        {
        }


        void SIM_Collector::put_formatted_messages (  
                                                    const CdmwTrace::MessagesHeader& header,
                                                    const CdmwTrace::FormattedMessageSeq& messages)
            throw(CORBA::SystemException)                       
        {
            m_test_status = true;

            static unsigned int num_log = 0;

            // Filter the warning messages of FlushAreaMngr
            if (!strcmp(header.the_application_name, "CollectorApplication")) {
                return;
            }

            std::cout << "SIM_Collector::put_formatted_messages (message number "
                      << num_log
                      << ")... " << std::endl;
    
            if (messages.length() != 1) {
                m_test_status = false;
                std::cout << "ERROR SIM Collector: the sequence received is not equal to 1!" << std::endl;
            }

            if (header.the_process_id != 1) {
                std::cerr << "ERROR SIM Collector :" << __LINE__ << " :" << header.the_process_id  << std::endl;
                m_test_status = false;
            } else {
                std::cout << "SIM Collector :" << __LINE__ << " :" << header.the_process_id << std::endl;
            }
      
            if (strcmp(header.the_application_name, "Application") != 0) {
                std::cerr << "ERROR SIM Collector :" << __LINE__ << " :" << header.the_application_name << std::endl;
                m_test_status = false;
            } else {
                std::cout << "SIM Collector :" << __LINE__ << " :" << header.the_application_name << std::endl;
            }

            if (strcmp(header.the_process_name, "Process") != 0) {
                std::cerr << "ERROR SIM Collector :" << __LINE__ << " :" << header.the_process_name << std::endl;
                m_test_status = false;
            } else {
                std::cout << "SIM Collector :" << __LINE__ << " :" << header.the_process_name  << std::endl;
            }

            if (messages[0].header.the_thread_id != 2) {
                std::cerr << "ERROR SIM Collector :" << __LINE__ << " :" << messages[0].header.the_thread_id << std::endl;
                m_test_status = false;
            } else {
                std::cout << "SIM Collector :" << __LINE__ << " :" << messages[0].header.the_thread_id << std::endl;
            }

            if (strcmp(header.the_host_name, "Hostname") != 0) {
                std::cerr << "ERROR SIM Collector :" << __LINE__ << " :" << header.the_host_name << std::endl;
                m_test_status = false;
            } else {
                std::cout << "SIM Collector :" << __LINE__ << " :" << header.the_host_name << std::endl;
            }
      
            if ( header.the_host_id != 0) {
                std::cerr << "ERROR SIM Collector :" << __LINE__ << " :" << header.the_host_id << std::endl;
                m_test_status = false;
            } else {
                std::cout << "SIM Collector :" << __LINE__ << " :" << header.the_host_id << std::endl;
            }
      
            if (strcmp(messages[0].header.the_level.the_component_name, "ComponentName") != 0) {
                std::cerr << "ERROR SIM Collector :" << __LINE__ << " :" << messages[0].header.the_level.the_component_name << std::endl;
                m_test_status = false;
            } else {
                std::cout << "SIM Collector :" << __LINE__ << " :" << messages[0].header.the_level.the_component_name << std::endl;
            }

            if (strcmp(messages[0].header.the_level.the_domain, "Domain") != 0) {
                std::cerr << "ERROR SIM Collector :" << __LINE__ << " :" << messages[0].header.the_level.the_domain << std::endl;
                m_test_status = false;
            } else {
                std::cout << "SIM Collector :" << __LINE__ << " :" << messages[0].header.the_level.the_domain << std::endl;
            }

            if (messages[0].header.the_level.the_value != 3) {
                std::cerr << "ERROR SIM Collector :" << __LINE__ << " :" << messages[0].header.the_level.the_value << std::endl;
                m_test_status = false;
            } else {
                std::cout << "SIM Collector :" << __LINE__ << " :" << messages[0].header.the_level.the_value << std::endl;
            }

            if (messages[0].header.the_level.activation != false) {
                std::cerr << "ERROR SIM Collector :" << __LINE__ << " :" << messages[0].header.the_level.activation << std::endl;
                m_test_status = false;
            } else {
                std::cout << "SIM Collector :" << __LINE__ << " :" << messages[0].header.the_level.activation << std::endl;
            }

            if (messages[0].header.the_file_line != 4) {
                std::cerr << "ERROR SIM Collector :" << __LINE__ << " :" << messages[0].header.the_file_line << std::endl;
                m_test_status = false;
            } else {
                std::cout << "SIM Collector :" << __LINE__ << " :" << messages[0].header.the_file_line << std::endl;
            }

            if (strcmp(messages[0].header.the_file_name,"File") != 0) {
                std::cerr << "ERROR SIM Collector :" << __LINE__ << " :" << messages[0].header.the_file_name  << std::endl;
                m_test_status = false;
            } else {
                std::cout << "SIM Collector :" << __LINE__ << " :" << messages[0].header.the_file_name << std::endl;
            }
      
            if (messages[0].header.time_stamping.seconds != 5) {
                std::cerr << "ERROR SIM Collector :" << __LINE__ << " :" <<  messages[0].header.time_stamping.seconds << std::endl;
                m_test_status = false;
            } else {
                std::cout << "SIM Collector :" << __LINE__ << " :" << messages[0].header.time_stamping.seconds << std::endl;
            }

            if (messages[0].header.time_stamping.microseconds != 6) {
                std::cerr << "ERROR SIM Collector :" << __LINE__ << " :" <<  messages[0].header.time_stamping.microseconds << std::endl;
                m_test_status = false;
            } else {
                std::cout << "SIM Collector :" << __LINE__ << " :" << messages[0].header.time_stamping.microseconds << std::endl;
            }
            
            std::stringstream stream; stream << "Message "; stream << num_log;
            if (strcmp(messages[0].message, stream.str().c_str()) != 0) {
                std::cerr << "ERROR SIM Collector :" << __LINE__ << " :" << messages[0].message
                          << ", instead of " << stream.str().c_str() << std::endl;
                m_test_status = false;
            } else {
                std::cout << "SIM Collector :" << __LINE__ << " :" << messages[0].message << std::endl;
            }

            num_log++;
        }                      

        void SIM_Collector::update (CdmwTrace::TraceProducer* p_trace_producer)
            throw(CORBA::SystemException)                       
        {

        }

        CdmwTrace::TraceFilterSeq* SIM_Collector::get_trace_filters()
            throw(CORBA::SystemException)
        {
            return NULL;
        }


        void SIM_Collector::activate_level(const char* a_componentName, // ECR-0123
                                           const char* a_domain,
                                           CdmwTrace::Value a_Value)
            throw(CORBA::SystemException)
        {

        }



        void SIM_Collector::deactivate_level(const char* a_componentName, // ECR-0123
                                             const char* a_domain,
                                             CdmwTrace::Value a_Value)
            throw(CORBA::SystemException)
        {

        }

                                                                            

        const CdmwTrace::MessagesHeader& SIM_Collector::get_MessagesHeader()
        {
            return m_the_messages_header.in();
        }



        const CdmwTrace::FormattedMessageSeq& SIM_Collector::get_FormattedMessageSeq()
        {
            return m_the_messages.in();
        }

        CORBA::Boolean SIM_Collector::get_status()
            throw(CORBA::SystemException)
        {
            return m_test_status;
        }

    }; // End namespace Trace
}; // End namespace Cdmw

