/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2007.
 * All rights reserved.
 *
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <cstdlib>
#include <sstream>
#include <iostream>
#include <fstream>
#include <string>
#include <Foundation/common/Locations.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include "testlogcollector/TestLogCollector.hpp"
#include "TraceAndPerf/tracecollector/CdmwTraceLoggingCollector.stub.hpp"
#include <Foundation/ossupport/OS.hpp>
#include "testlogcollector/SimulatedGlobalTraceCollector.stub.hpp"
#include <Foundation/commonsvcs/naming/NamingUtil.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>
#include <Foundation/common/Locations.hpp>

#include <Foundation/ossupport/OS.hpp>


namespace
{
    const std::string DEF_REPOSITORY_PORT         = "5021";
    const std::string DEF_NAME_SERVICE_PORT       = "5011";
    const std::string GLOBAL_COLLECTOR_NAME       = "global_collector_1";
}

namespace Cdmw
{
    namespace Trace
    {

        /**
         *Implementation notes:
         *<p>   none
         *Portability issues: 
         *<p>   none
         */

        CPPUNIT_TEST_SUITE_REGISTRATION( TestLogCollector );

        TestLogCollector::TestLogCollector()
            throw() :
            m_system_port(),
            m_traceCollector("cdmw_trace_collector"),
            m_traceCollectorArgs()
        {
            m_system_port = Cdmw::TestUtils::CORBATestManager::instance()->get_property("system_port");
            m_traceCollectorArgs = Cdmw::TestUtils::CORBATestManager::instance()->get_property("LTRC_args");
        }


        TestLogCollector::~TestLogCollector()
            throw()
        {
            // does nothing
        }

        TestLogCollector&
        TestLogCollector::operator=(const TestLogCollector& rhs)
            throw()
        {
            // does nothing
            return *this;
        }


        void TestLogCollector::test_log_collector()
        {
            TEST_INFO("Test Log Collector library");

            int timescale = Cdmw::TestUtils::get_timescale();

            std::string namesrv_corbaloc = "corbaloc:iiop:localhost:" +
                m_system_port + "/NameService";
    
            CORBA::ORB_var orb =
                Cdmw::TestUtils::CORBATestManager::instance()->get_ORB();
            CORBA::Object_var obj = orb->string_to_object(namesrv_corbaloc.c_str()) ;
            CosNaming::NamingContext_var nc = 
                CosNaming::NamingContext::_narrow(obj.in());
            Cdmw::CommonSvcs::Naming::NamingInterface ni (nc.in());

            CORBA::Object_var log_collector_obj = 
                ni.resolve(Cdmw::Common::Locations::DEFAULT_LOG_COLLECTOR_ID);

            CdmwLogging::LogCollector_var log_collector =
                CdmwLogging::LogCollector::_narrow(log_collector_obj.in());

            TEST_INFO("Log collector resolved");

            //
            // Recover the reference of SIM Collector to know its status
            //
            std::string sim_repository_corbaloc= "corbaloc:iiop:localhost:" +
                DEF_REPOSITORY_PORT + "/SimRepository";
            
            obj = orb->string_to_object(sim_repository_corbaloc.c_str()) ;
            CdmwNamingAndRepository::Repository_var repository_ref = CdmwNamingAndRepository::Repository::_narrow(obj.in());
            
            TEST_INFO("Sim Repository resolved");
            
            CosNaming::NamingContext_var nc_root_context =
                repository_ref->resolve_root_context(
                                                     CdmwNamingAndRepository::DEFAULT_ROOT_CONTEXT);
            using namespace Cdmw::CommonSvcs::Naming;
            // naming interface of default root context
            NamingInterface ni_root(nc_root_context.in());
            
            CosNaming::NamingContext_var nc_collector_dom =
                CosNaming::NamingContext::_nil();
            
            std::string collector_domain_str(Cdmw::Common::Locations::CDMW_SERVICES_NAME_DOMAIN);
            collector_domain_str += "/";
            collector_domain_str += Cdmw::Common::Locations::TRACE_COLLECTORS_NAME_DOMAIN;
            
            // get the global collector domain name.
            typedef NamingUtil<CosNaming::NamingContext> Util;
            nc_collector_dom = Util::resolve_name(ni_root,
                                                  collector_domain_str);
            
            // naming interface of global collector domain
            NamingInterface ni_collector(nc_collector_dom.in());
            CORBA::Object_var global_collector_obj = ni_collector.resolve(GLOBAL_COLLECTOR_NAME.c_str());
            
            CdmwTrace::SimulatedGlobalTraceCollector_var sim_collector = CdmwTrace::SimulatedGlobalTraceCollector::_narrow(global_collector_obj.in());
            
            TEST_INFO("Simulated Global Collector resolved");

            CdmwLogging::StructuredLog log;
            log.ctx.process_id = 1;
            log.ctx.application_name = "Application";
            log.ctx.process_name = "Process";
            log.ctx.ndc = "NDC";
            log.header.thread_id = 2;
            log.ctx.host_name = "Hostname";
            log.header.component_name = "ComponentName";
            log.header.user_domain = "Domain";
            log.header.level = 3;
            log.header.file_line = 4;
            log.header.file_name = "File";
            log.header.time_stamping.seconds = 5;
            log.header.time_stamping. microseconds = 6;     

            // Status of this unit test
            CORBA::Boolean status = false;
            int loop = 0, delay, msg_nb = 0;
            std::stringstream stream;
            for ( ; loop < 10 ; loop++) {
                TEST_INFO("Loop [" << loop+1 << "]");
                TEST_INFO("Log a message while the Local Trace Collector is not started.");
                // If you change the following line, check the SIM_Collector
                stream.str("");
                stream << "Message " << msg_nb++;
                log.message = stream.str().c_str();
                TEST_INFO("Message to send : " << log.message);
                log_collector->add_log(log);

                delay = timescale * 1000 /* 10000 */;
                TEST_INFO("Wait " << delay / 1000 
                          << " sec for the sending of the log.");
    
                TEST_INFO("Start the Local Trace Collector");
                TEST_INFO(m_traceCollector << m_traceCollectorArgs);
                Cdmw::OsSupport::OS::ProcessId idTraceCollector =
                    Cdmw::OsSupport::OS::create_process(m_traceCollector, m_traceCollectorArgs);

                delay = timescale * 5000 /* 10000 */;
                TEST_INFO("Wait " << delay / 1000 << " sec for the Local Trace Collector.");
                Cdmw::OsSupport::OS::sleep(delay);

                TEST_INFO("Log a message while the Local Trace Collector is running.");
                // If you change the following line, check the SIM_Collector
                stream.str("");
                stream << "Message " << msg_nb++;
                log.message = stream.str().c_str();
                TEST_INFO("Message to send : " << log.message);
                log_collector->add_log(log);

                delay = timescale * 5000 /* 10000 */;
                TEST_INFO("Wait " << delay / 1000 
                          << " sec for the sending of the log.");

                // kill the Local Trace Collector
                TEST_INFO("Kill the Local Trace Collector");
                try {
                    Cdmw::OsSupport::OS::kill_process(idTraceCollector);
                } catch(const std::exception & ex) {
                    std::cerr << "Cannot kill Local Trace Collector process." << std::endl;
                    status = false;
                }

                delay = timescale * 5000 /* 10000 */;
                TEST_INFO("Wait " << delay / 1000 
                          << " sec for the death of the Local Trace Collector process.");
                Cdmw::OsSupport::OS::sleep(delay);

                TEST_INFO("Log a message while the Local Trace Collector is stopped.");
                // If you change the following line, check the SIM_Collector
                stream.str("");
                stream << "Message " << msg_nb++;
                log.message = stream.str().c_str();
                TEST_INFO("Message to send : " << log.message);
                log_collector->add_log(log);

                // Wait that SIM Collector recovers the log
                delay = timescale * 5000 /* 10000 */;
                TEST_INFO("Wait " << delay / 1000 
                          << " sec while the SIM Collector recovers the log.");
                Cdmw::OsSupport::OS::sleep(delay);

                // Get status
                status = sim_collector->get_status();
                
                CPPUNIT_ASSERT(status);
            }

            // Verify that all messages logged when the L_TRC is running are not logged in LCP.log
            TEST_INFO("Verify that all messages logged when L_TRC is running are not logged in LCP.log");
            ifstream file("LCP.log");
            if ( !file ) {
                // File not found
                TEST_INFO("File LCP.log not found");
                CPPUNIT_ASSERT(false);
            } else {
                std::string line;
                int found = 0, cur_msg_nb=1, next_msg_nb=0;
                std::stringstream stream;
                std::string str;
                while(getline(file,line)) {
                    unsigned int msg_idx = line.find("Message");
                    if(msg_idx != std::string::npos) {
                        stream.str(line.substr(msg_idx)); // Keep only "Message XXX"
                        stream >> str ; stream >> next_msg_nb;
                        if(cur_msg_nb == next_msg_nb) {
                            cur_msg_nb+=2;
                        }
                        else {
                            found++;
                        }
                    }
                }
                if(found != (loop*2)) {
                    TEST_INFO("It should have " << loop*2 
                              << " line(s) in LCP.log instead of " << found << ".");
                    CPPUNIT_ASSERT(false);
                }
            }
            file.close();

            // Remove the log file
            try {
                Cdmw::OsSupport::OS::unlink("LCP.log");
            } catch (...) {
                // Do nothing
            }
        }

    }; // End namespace Trace
}; // End namespace Cdmw

