/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <cstdlib>
#include <fstream>
#include <sstream>
#include <string>

#include <Foundation/common/Assert.hpp>
#include <Foundation/common/Locations.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/String.hpp>
#include <Foundation/common/System.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <Time/clockservice/CdmwCosClock.hpp>
#include <Foundation/commonsvcs/time/Util.hpp>

#include "tracecollector/TraceCollectorProcessAgent.stub.hpp"
#include "tracecollector/CollectorOptions.hpp"
#include "tracelibrary/Message.hpp"
#include "testtracecollector/TestTraceFile.hpp"

namespace
{
    using std::string;

    const string DEF_NAME_SERVICE_PORT       = "5011";
    const string DEF_REPOSITORY_PORT         = "5021";
    const string DEF_TRC_COLLECTOR_SERV_PORT = "5031";
    const string DEF_TRC_COLLECTOR_CDMW_PORT = "5032";
    const string DEF_GBL_TRC_COLL_SERV_PORT  = "5033";
    const string DEF_GBL_TRC_COLL_CDMW_PORT  = "5034";

    const string NAMING_SERVICE_PORT_OPTION  = "--NameServicePort";
    const string REPOSITORY_PORT_OPTION      = "--RepositoryPort";
    const string COLLECTOR_CDMW_PORT_OPTION  = "--CollectorCdmwPort";
    const string COLLECTOR_SERV_PORT_OPTION  = "--CollectorServicePort";

    const string COLLECTOR_NAME_OPTION       = "--collector-name";
    const string TRACE_FILE_NAME_OPTION      = "--trace-file-name";
    const string COMPONENT_NAME_OPTION       = "--component-name";
    const string DOMAIN_NAME_OPTION          = "--domain-name";
    const string LEVEL_OPTION                = "--level";
    const string TRACE_FILE_BKP_NBR_OPTION   = "--trace-file-bak-num";
    const string TRACE_FILE_NBR_OPTION       = "--trace-file-num";
    const string TRACE_MESSAGE_NBR_OPTION    = "--trace-msg-num";
    const string TRACE_FORMAT_OPTION         = "--trace-format";
    const string TRACE_FORMAT_OPTV           = "V";
    const string TRACE_FORMAT_OPTH           = "H";
    const string TRACE_FORMAT_OPTX           = "X";
    const string COLLECTOR_MIOP_CORBALOC     = "--miop-corbaloc";
    const string FLUSH_AREA_SIZE            = "--flush-area-sz";
    const string FLUSH_AREA_NB              = "--flush-area-nb";

    const string NAMING_REPOS_URL_OPTION     = "--naming-repos-url";
    const string APPLICATION_NAME_OPTION     = "--application-name";
    const string INTERCEPT_CYCLE_OPTION      = "--intercept-cycle";

    const string COLLECTOR_NAME              = "collector_1";
    const string PRODUCER_NAME               = "producer_1";
    const string TRACE_FILE_NAME             = "test_trace";

    const string GLOBAL_COLLECTOR_NAME       = "global_collector_1";
    const string GLOBAL_TRACE_FILE_NAME      = "test_trace_global";

    const int SUCCESS                        = 0;
    const int FAILURE                        = 1;

    const int POA_THREAD_POOL_SIZE           = 5;

    static void
    usage(std::ostream& os, const string& program_name)
    {
        os << "Usage:\n"
           << program_name << " options\n"
           << "\n"
           << "Options:\n"
           << "--help                            Display this information.\n"
           << "\n"
           << "[--NameServicePort=<nbr>          Port of Name Service.\n"
           << "[--RepositoryPort=<nbr>           Port of Name and Repository.\n"
           << "[--CollectorServicePort=<nbr>     Service Port of Trace Collector.\n"
           << "[--CollectorCdmwPort=<nbr>        Cdmw Port of Trace Collector.\n"
           << "\n"
           << "[--collector-name=<name>]         Name of Trace Collector.\n"
           << "[--trace-file-name=<name>]        Name of Trace File.\n"
           << "[--component-name=<name>]         Filter on this component name.\n" /* ECR-0123 */
           << "[--domain-name=<name>]            Name of domain to filter.\n"
           << "[--level=<value>]                 Level to filter.\n"
           << "[--trace-format=<H|V|X>]          Format of Trace : Horizontal, Vertical (default) or XML.\n"
           << "[--trace-file-bak-num=<number>]   Number of backup of trace files.\n"
           << "[--trace-file-num=<number>]       Number of trace circular files.\n"
           << "[--trace-msg-num=<number>]        Number of trace messages in circular files.\n"
           << "[--intercept-cycle=<number>]      will send exception at 1st put_formatted_message "
              "request and then each n+1 requests.\n"
           << "[" << Cdmw::Trace::CollectorOptions::OPERATIONAL_CLOCK_NAME << "=<name>] Operational Clock Name.\n"
           << "[--operational-time-not-available]/n"
           << std:: endl;
    }
};


int
auto_check(bool logCreationSuccess,
           bool horizontalFormat,
           std::string domainName)
{
    int status = SUCCESS;
    std::string trace_file;

    {
        //
        // get Trace File Name
        //
        std::cout << "=== domainName:" << domainName<< std::endl;
        // build name of backup info
         if (domainName == "test_empty_field")
        {
           trace_file = "test_empty_field";
        }
        else
        {
           trace_file = TRACE_FILE_NAME;
        }
        std::cout << "=== trace_file:" << trace_file << std::endl;
        // build name of backup info
        std::string backUpInfoFileName = trace_file;
        backUpInfoFileName += "_bkpinfo";

        short current_bkp = 0;

        // open the file for read backup info
        std::ifstream bkp_file_in_str(backUpInfoFileName.c_str(), std::ios::in);

        // if file exist and can be opened
        if (bkp_file_in_str)
        {
            // get current backup from info file
            bkp_file_in_str >> current_bkp;

            // if current backup not retrieve
            // set no backup
            if (!bkp_file_in_str.good())
            {
                current_bkp = 0;
            }

            // close the file
            bkp_file_in_str.close();
        }

        if (current_bkp > 0)
        {
            // update the file name base with the backup nbr
            std::ostringstream name_str;
            // name_str << TRACE_FILE_NAME << "_" << current_bkp;
            name_str << trace_file << "_" << current_bkp;

            trace_file = name_str.str();
        }

        // add extension for 1st circular file
        trace_file += ".0";
    }


    Cdmw::TestUtils::CORBATestManager::instance()->set_property("file", trace_file);

    {
        trace_file = GLOBAL_TRACE_FILE_NAME;

        // build name of backup info
        std::string backUpInfoFileName = trace_file;
        backUpInfoFileName += "_bkpinfo";

        short current_bkp = 0;

        // open the file for read backup info
        std::ifstream bkp_file_in_str(backUpInfoFileName.c_str(), std::ios::in);

        // if file exist and can be opened
        if (bkp_file_in_str)
        {
            // get current backup from info file
            bkp_file_in_str >> current_bkp;

            // if current backup not retrieve
            // set no backup
            if (!bkp_file_in_str.good())
            {
                current_bkp = 0;
            }

            // close the file
            bkp_file_in_str.close();
        }

        if (current_bkp > 0)
        {
            // update the file name base with the backup nbr
            std::ostringstream name_str;
            //name_str << GLOBAL_TRACE_FILE_NAME << "_" << current_bkp;
            name_str << trace_file << "_" << current_bkp;

            trace_file = name_str.str();
        }

        // add extension for 1st circular file
        trace_file += ".0";
    }

    Cdmw::TestUtils::CORBATestManager::instance()->set_property("global_file", trace_file);

    if (logCreationSuccess)
        Cdmw::TestUtils::CORBATestManager::instance()->set_property("logcreation", "yes");
    else
        Cdmw::TestUtils::CORBATestManager::instance()->set_property("logcreation", "no");

    if (horizontalFormat)
        Cdmw::TestUtils::CORBATestManager::instance()->set_property("horizontalFormat", "yes");
    else
        Cdmw::TestUtils::CORBATestManager::instance()->set_property("horizontalFormat", "no");


    Cdmw::TestUtils::CORBATestManager::instance()->run_tests();
    Cdmw::TestUtils::CORBATestManager::instance()->summary();

    return status;
}


std::string timestamp_to_string(Cdmw::OsSupport::OS::Timeval tv, const char* format)
{
   // format date
   char time_buf[128];
   Cdmw::OsSupport::OS::get_fmt_time (time_buf, sizeof(time_buf),
                                tv.seconds,
                                const_cast<char*>(format), Cdmw::OsSupport::OS::LOCAL_TIME);

   return std::string(time_buf);
}

void init_controlled_clock()
{
    CDMW_ASSERT(!CORBA::is_nil(Cdmw::TestUtils::CORBATestManager::instance()->get_ORB()));

     CORBA::Object_var obj =
         Cdmw::TestUtils::CORBATestManager::instance()->get_ORB()->resolve_initial_references("ClockService");
     if (CORBA::is_nil(obj.in()))
     {
         //CDMW_ERROR(CdmwInit::InitUtils::Get_Logger(), "Unable to resolve ClockService");
         CDMW_THROW2(Cdmw::BadParameterException, "Unable to resolve ClockService", "NULL");
     }

     CosClockService::ClockCatalog_var clock_catalog =
             CosClockService::ClockCatalog::_narrow(obj.in());
     if (CORBA::is_nil(clock_catalog.in()))
     {
         //CDMW_ERROR(CdmwInit::InitUtils::Get_Logger(), "Unable to resolve ClockCatalog");
         CDMW_THROW2(Cdmw::BadParameterException, "Unable to resolve ClockCatalog", "NULL");
     }

     char clock_name[] = "FederatedControlledClock";
     CosClockService::ClockCatalog::ClockEntry_var catalog_entry = clock_catalog->get_entry(clock_name);
     Cdmw::clock::ControlledClockEx_ptr ctrl_clock = Cdmw::clock::ControlledClockEx::_narrow(catalog_entry->subject.in());
     if (CORBA::is_nil(ctrl_clock))
     {
         CDMW_THROW2(Cdmw::BadParameterException, "Unable to narrow the clock's name", "NULL");
     }

     Cdmw::OsSupport::OS::Timeval time_val = Cdmw::OsSupport::OS::get_time();
     std::cout << "-> Setting \"" << clock_name << "\" at " << timestamp_to_string(time_val,"%a, %d %b %Y %H:%M:%S %z") << std::endl;
     time_val.seconds = 100000000L;
     time_val.microseconds = 0L;

     TimeBase::TimeT to = Cdmw::CommonSvcs::Time::Util::Timeval_2_TimeT(time_val);
     ctrl_clock->set_rate(10.0);
     ctrl_clock->set(to);

     std::cout << "... to " << timestamp_to_string(time_val,"%a, %d %b %Y %H:%M:%S %z") << std::endl;
     std::cout << "~+~+~+~+~+~ time_val.seconds=" << time_val.seconds
               << ", ratio=" << ctrl_clock->get_rate() << " ~+~+~+~+~+~" << std::endl;
}


void exceptionHandler(void)
{
    std::cout << "Trace Test: UNEXPECTED EXCEPTION HANDLER" << std::endl;
}


int main(int argc, char* argv[])
{
#ifndef _MSC_VER
    std::set_unexpected(&exceptionHandler);
    // VC++ contrary to the clause 18.6 of the C++ standard,
    // set_unexpected, is not inside the std:: namespace
#else
    set_unexpected(&exceptionHandler);
#endif

    using namespace Cdmw::OsSupport;
    using namespace Cdmw::Common;

    // get arguments

    // help argument
    if (argc >= 2)
    {
        if(strcmp(argv[1], "--help") == 0)
        {
            usage(std::cout, argv[0]);
            return SUCCESS;
        }
    }

    // launch Nameservice and SimRepository for tests
    OS::ProcessId idNaming, idRepository;
    // launch TraceCollector and TraceLibrary for tests
    OS::ProcessId idGlobalTraceCollector, idTraceCollector, idTraceLibrary;

    // ORB arguments
    char** ORB_init_argv = 0;
    int ORB_init_argc(0);

    // get the Name Service port.
    std::string nameServicePort =
        OS::get_option_value(argc, argv, NAMING_SERVICE_PORT_OPTION);
    if (nameServicePort == "no")
    {
        nameServicePort = DEF_NAME_SERVICE_PORT;
    }

    // get the Repository port.
    std::string repositoryPort =
        OS::get_option_value (argc, argv, REPOSITORY_PORT_OPTION);
    if (repositoryPort == "no")
    {
        repositoryPort = DEF_REPOSITORY_PORT;
    }

    // get the TraceCollector Service port.
    std::string collectorServPort =
        OS::get_option_value(argc, argv, COLLECTOR_SERV_PORT_OPTION);
    if (collectorServPort == "no")
    {
        collectorServPort = DEF_TRC_COLLECTOR_SERV_PORT;
    }

    // get the TraceCollector Cdmw port.
    std::string collectorCdmwPort =
        OS::get_option_value(argc, argv, COLLECTOR_CDMW_PORT_OPTION);
    if (collectorCdmwPort == "no")
    {
        collectorCdmwPort = DEF_TRC_COLLECTOR_CDMW_PORT;
    }

    // get Trace collector options

    //
    // get Trace Collector Name
    //
    std::string collectorName =
        OS::get_option_value(argc, argv, COLLECTOR_NAME_OPTION);
    if (collectorName == "no" || collectorName == "yes")
    {
        collectorName = COLLECTOR_NAME;
    }

    //
    // get Filter Domain Name
    //
    std::string domainName =
        OS::get_option_value (argc, argv, DOMAIN_NAME_OPTION);

    //
    // get Trace File Name
    //
    std::string traceFileName =
        OS::get_option_value(argc, argv, TRACE_FILE_NAME_OPTION);
    if (domainName == "test_empty_field")
    {
       traceFileName = "test_empty_field";
    }
    else
    {
       if (traceFileName == "no" || traceFileName == "yes")
       {
          traceFileName = TRACE_FILE_NAME;
       }
    }

    // get the component name
    std::string componentName =
        OS::get_option_value(argc, argv, COMPONENT_NAME_OPTION);

    //
    // get Filter Level value
    //
    std::string levelOption =
        OS::get_option_value(argc, argv, LEVEL_OPTION);

    //
    // get Format of trace
    //
    Cdmw::Trace::Message::TraceFormat traceFormat =
        Cdmw::Trace::Message::V_FORMAT;
    std::string formatOption =
        OS::get_option_value(argc, argv, TRACE_FORMAT_OPTION);
    if (formatOption != "no")
    {
        if (formatOption == TRACE_FORMAT_OPTV.c_str())
        {
            traceFormat = Cdmw::Trace::Message::V_FORMAT;
        }
        else if (formatOption == TRACE_FORMAT_OPTH.c_str())
        {
            traceFormat = Cdmw::Trace::Message::H_FORMAT;
        }
        else if (formatOption == TRACE_FORMAT_OPTX.c_str())
        {
            traceFormat = Cdmw::Trace::Message::X_FORMAT;
        }
        else
        {
            std::cout << "bad value for " << TRACE_FORMAT_OPTION
                      << " option" << std::endl
                      << "  options are : " << TRACE_FORMAT_OPTV
                      << ", " << TRACE_FORMAT_OPTH
                      << ", or " << TRACE_FORMAT_OPTX
                      << std::endl << std::endl;
            return FAILURE;
        }
    }

    // get the miop corbaloc used for communication between local&global
    std::string miopCorbaloc =
        OS::get_option_value(argc, argv, COLLECTOR_MIOP_CORBALOC);

    //
    // get number of trace file backup
    //
    std::string traceFileBkpNbrOption =
        OS::get_option_value (argc, argv, TRACE_FILE_BKP_NBR_OPTION);

    //
    // get Trace Circular File number
    //
    std::string traceFileNbrOption =
        OS::get_option_value (argc, argv, TRACE_FILE_NBR_OPTION);

    //
    // get Trace Message number to store in file
    //
    std::string traceMessageNbrOption =
        OS::get_option_value (argc, argv, TRACE_MESSAGE_NBR_OPTION);

    //
    // get the size for the tracecollector flush area
    //
    std::string flushAreaSizeOption =
        OS::get_option_value (argc, argv, FLUSH_AREA_SIZE);

    //
    // get the number of messages in the flush area
    //
    std::string flushAreaNbOption =
        OS::get_option_value (argc, argv, FLUSH_AREA_NB);

    std::string operationalClockName =
        OS::get_option_value (argc, argv, Cdmw::Trace::CollectorOptions::OPERATIONAL_CLOCK_NAME);
    std::string operationalTimeNotAvailable = OS::get_option_value (argc, argv, "--operational-time-not-available");

    // Starting the Name Service : "nameserv -OAport <port>"
    std::cout << "start name service" << std::endl;

#if (CDMW_ORB_VDR == orbacus )
    std::string namingservice = "nameserv";
    idNaming = OS::create_process(namingservice,
                                  " -OAport " + nameServicePort);
#elif (CDMW_ORB_VDR == tao )
    std::string namingservice = "Naming_Service";
    idNaming = OS::create_process(
                   namingservice,
                   "-ORBEndpoint iiop://localhost:" + nameServicePort);
#endif

    int timescale = Cdmw::TestUtils::get_timescale();
    OS::sleep(timescale * 10000);

    // Starting the Repository Simulator :
    // "simulated_repository -OAport <port>
    //  -ORBnaming corbaloc::localhost:<port>/NameService
    //  -DOM CDMW/SERVICES -DOM CDMW/ADMIN -DOM CDMW/ADMIN/TRACE/COLLECTORS
    //  -ROOT AdminRootContext"
    std::string repository = "cdmw_simulated_repository";
    std::string namingCorbaloc =
#if (CDMW_ORB_VDR == orbacus)
        " -ORBnaming corbaloc::localhost:" + nameServicePort + "/NameService";
#elif (CDMW_ORB_VDR == tao)
        " -ORBInitRef NameService=corbaloc:iiop:localhost:" +
        nameServicePort + "/NameService";
#endif

    std::string repositoryArgs ("");
#if (CDMW_ORB_VDR == orbacus)
    repositoryArgs += " -OAport ";
#elif (CDMW_ORB_VDR == tao )
    repositoryArgs += " -ORBEndpoint iiop://localhost:";
#endif

    repositoryArgs += repositoryPort;
    repositoryArgs += namingCorbaloc;
    repositoryArgs += " -DOM ";
    repositoryArgs += Cdmw::Common::Locations::CDMW_SERVICES_NAME_DOMAIN;
    repositoryArgs += " -DOM ";
    repositoryArgs += Cdmw::Common::Locations::CDMW_SERVICES_NAME_DOMAIN;
    repositoryArgs += "/";
    repositoryArgs += Cdmw::Common::Locations::TRACE_COLLECTORS_NAME_DOMAIN;
    repositoryArgs += " -ROOT ";
    repositoryArgs += Cdmw::Common::Locations::ADMIN_ROOT_CONTEXT_ID;

    std::cout << "start repository simulator" << std::endl
              << repository << repositoryArgs << std::endl;

    idRepository = OS::create_process(repository, repositoryArgs);

    OS::sleep(timescale * 10000);

    // set repository url
    std::string repositoryCorbaloc =
        " " + NAMING_REPOS_URL_OPTION + "=corbaloc::localhost:" +
        repositoryPort + "/SimRepository";

    // start the global trace collector.
    std::string gblTraceCltor("cdmw_global_trace_collector");

    std::ostringstream oss;
    oss << " " << COLLECTOR_NAME_OPTION << "=" << GLOBAL_COLLECTOR_NAME
        << " " << TRACE_FILE_NAME_OPTION << "=" << GLOBAL_TRACE_FILE_NAME
        << " " << Options::LOCALISATION_SERVICE_OPTION << "=" << DEF_GBL_TRC_COLL_SERV_PORT
        << " " << Options::PROCESS_PORT_OPTION << "=" << DEF_GBL_TRC_COLL_CDMW_PORT
        << " " + NAMING_REPOS_URL_OPTION + "=corbaloc::localhost:" << repositoryPort
        << "/SimRepository";
    if (formatOption != "no")
    {
        oss << " " << TRACE_FORMAT_OPTION << "=" << formatOption;
    }
    if ( miopCorbaloc != "no")
    {
        oss << " " << COLLECTOR_MIOP_CORBALOC << "=" << miopCorbaloc;
    }

    std::string gblTraceCltorArgs(oss.str());

    std::cout << "start global trace collector" << std::endl;
    std::cout << gblTraceCltor << gblTraceCltorArgs << std::endl;

    idGlobalTraceCollector =
        OS::create_process(gblTraceCltor, gblTraceCltorArgs);

    OS::sleep(timescale * 10000);

    // Starting the Trace Collector :
    std::string loc_service_opt = Options::LOCALISATION_SERVICE_OPTION;
    std::string proc_port_opt = Options::PROCESS_PORT_OPTION;

    std::string traceCollector = "cdmw_trace_collector";
    std::string traceCollectorArgs ("");
    traceCollectorArgs += " " + COLLECTOR_NAME_OPTION + "=";
    traceCollectorArgs += collectorName;
    traceCollectorArgs += " " + TRACE_FILE_NAME_OPTION + "=";
    traceCollectorArgs += traceFileName;
    traceCollectorArgs += " " + loc_service_opt + "=";
    traceCollectorArgs += collectorServPort;
    traceCollectorArgs += " " + proc_port_opt + "=";
    traceCollectorArgs += collectorCdmwPort;
    traceCollectorArgs += repositoryCorbaloc;

    std::cout << "=== domainName = " << domainName << std::endl;
    if ((componentName != "no") && (componentName != "yes"))
    {
        traceCollectorArgs +=
             " " + COMPONENT_NAME_OPTION + "=" + componentName;
    }

    if (domainName != "no" && domainName != "yes")
    {
       traceCollectorArgs += " ";
       traceCollectorArgs += DOMAIN_NAME_OPTION;
       traceCollectorArgs += "=";
       traceCollectorArgs += domainName;
    }

    if (levelOption != "no")
    {
        traceCollectorArgs += " ";
        traceCollectorArgs += LEVEL_OPTION;
        traceCollectorArgs += "=";
        traceCollectorArgs += levelOption;
    }

    std::cout << "=== formatOption = " << formatOption << std::endl;
    if (formatOption != "no")
    {
        traceCollectorArgs += " ";
        traceCollectorArgs += TRACE_FORMAT_OPTION;
        traceCollectorArgs += "=";
        traceCollectorArgs += formatOption;
    }

    if (traceFileBkpNbrOption != "no")
    {
        traceCollectorArgs += " ";
        traceCollectorArgs += TRACE_FILE_BKP_NBR_OPTION;
        traceCollectorArgs += "=";
        traceCollectorArgs += traceFileBkpNbrOption;
    }

    if (traceFileNbrOption != "no")
    {
        traceCollectorArgs += " ";
        traceCollectorArgs += TRACE_FILE_NBR_OPTION;
        traceCollectorArgs += "=";
        traceCollectorArgs += traceFileNbrOption;
    }

    if (traceMessageNbrOption != "no")
    {
        traceCollectorArgs += " ";
        traceCollectorArgs += TRACE_MESSAGE_NBR_OPTION;
        traceCollectorArgs += "=";
        traceCollectorArgs += traceMessageNbrOption;
    }

    if ( miopCorbaloc != "no")
    {
        traceCollectorArgs += " ";
        traceCollectorArgs += COLLECTOR_MIOP_CORBALOC;
        traceCollectorArgs += "=";
        traceCollectorArgs += miopCorbaloc;
        Cdmw::TestUtils::CORBATestManager::instance()->set_property("MIOPCorbaloc", miopCorbaloc);
    }

    if ( flushAreaSizeOption != "no")
    {
        traceCollectorArgs += " ";
        traceCollectorArgs += FLUSH_AREA_SIZE;
        traceCollectorArgs += "=";
        traceCollectorArgs += flushAreaSizeOption;
    }

    std::cout << "flushAreaNbOption: " << flushAreaNbOption << std::endl;
    if ( flushAreaNbOption != "no")
    {
        traceCollectorArgs += " ";
        traceCollectorArgs += FLUSH_AREA_NB;
        traceCollectorArgs += "=";
        traceCollectorArgs += flushAreaNbOption;
    }

    std::cout << "operationalClockName: " << operationalClockName << std::endl;
    if ( operationalClockName != "no")
    {
        traceCollectorArgs += " ";
        traceCollectorArgs += Cdmw::Trace::CollectorOptions::OPERATIONAL_CLOCK_NAME;
        traceCollectorArgs += "=";
        traceCollectorArgs += operationalClockName;
        traceCollectorArgs += " --CdmwLogLevel=DEFAULT+DEBUG --CdmwLogFile=toto.log ";
        Cdmw::TestUtils::CORBATestManager::instance()->set_property("OperationalClockName", operationalClockName);
    }

    std::cout << "start trace collector" << std::endl
              << traceCollector << traceCollectorArgs << std::endl;

    idTraceCollector =
        OS::create_process(traceCollector, traceCollectorArgs);

    OS::sleep(timescale * 10000);

    std::string interceptCycle =
        OS::get_option_value(argc, argv, INTERCEPT_CYCLE_OPTION);
    int iCycle = 0;
    if (interceptCycle == "yes")
    {
        std::cout << INTERCEPT_CYCLE_OPTION.c_str()
                  << "=<n> option must be defined"
                  << std::endl;
        usage(std::cout,argv[0]);
        return FAILURE;
    }
    if (interceptCycle != "no")
    {
        iCycle = atoi(interceptCycle.c_str());
    }

    /* Starting the Trace Library Test :
    std::string traceLibrary = "cdmw_test_trace";
    std::string traceLibraryArgs("");
    if (domainName != "test_empty_field")
    {
       traceLibraryArgs += " " + APPLICATION_NAME_OPTION + "=";
       traceLibraryArgs += PRODUCER_NAME;
    }
    traceLibraryArgs += repositoryCorbaloc;
    if (iCycle !=0 )
    {
      traceLibraryArgs += " " + INTERCEPT_CYCLE_OPTION + "=";
      traceLibraryArgs += interceptCycle;
    }

    std::cout << "start trace library" << std::endl
              << traceLibrary << traceLibraryArgs << std::endl;

    idTraceLibrary = OS::create_process(traceLibrary, traceLibraryArgs);

    // Wait end of test
    OS::sleep(timescale * 15000);*/

    // build ORB argument with process name and naming corbaloc
    std::string strArgv = argv[0] + namingCorbaloc;
    ORB_init_argv = String::to_char_array(String::to_strings(strArgv));
    ORB_init_argc = 3;

    int status = FAILURE;
    CORBA::ORB_var orb;

    try
    {
        // Initialize the ORB
        Cdmw::OrbSupport::StrategyList orb_strategies;
        orb_strategies.add_OrbThreaded();
        orb_strategies.add_PoaThreadPool(POA_THREAD_POOL_SIZE);
        if ( operationalClockName != "no")
            orb_strategies.add_multicast(); // tsenikies

        orb = Cdmw::OrbSupport::OrbSupport::ORB_init(ORB_init_argc,
                                                     ORB_init_argv,
                                                     orb_strategies);

        // Get the root POA
        CORBA::Object_var obj = orb->resolve_initial_references( "RootPOA" );
        PortableServer::POA_var rootPOA
            = PortableServer::POA::_narrow( obj.in() );

        Cdmw::TestUtils::CORBATestManager::instance()->init(orb.in(),rootPOA.in(),argc,argv);

        if ( operationalClockName != "no")
        {
            Cdmw::clock::CosClock::init(orb.in(), argc, argv);
            init_controlled_clock();
        }

        status = SUCCESS;
    }
    catch (const Cdmw::Exception& ex)
    {
      std::cerr << ex.what() << std::endl;
    }
    catch(const CORBA::Exception& ex)
    {
      std::cerr << ex << std::endl;
    }

    // Starting the Trace Library Test :
    std::string traceLibrary = "cdmw_test_trace";
    std::string traceLibraryArgs("");
    if (domainName != "test_empty_field")
    {
       traceLibraryArgs += " " + APPLICATION_NAME_OPTION + "=";
       traceLibraryArgs += PRODUCER_NAME;
    }
    traceLibraryArgs += repositoryCorbaloc;
    if (iCycle !=0 )
    {
      traceLibraryArgs += " " + INTERCEPT_CYCLE_OPTION + "=";
      traceLibraryArgs += interceptCycle;
    }

    std::cout << "start trace library" << std::endl
              << traceLibrary << traceLibraryArgs << std::endl;

    idTraceLibrary = OS::create_process(traceLibrary, traceLibraryArgs);

    if ((operationalClockName != "no") && (operationalTimeNotAvailable!= "no")) 
    {
       OS::sleep(timescale * 1000);
       init_controlled_clock();
    }

    // Wait end of test
    OS::sleep(timescale * 15000);

    std::cout << "stop all processes" << std::endl;

    // stop the trace collectors
    try
    {
        if (status == SUCCESS)
        {
            // set traceCollector agent url
            std::string collectorAgentURL =
                "corbaloc::localhost:" + collectorServPort + "/CollectorAgent";

            // get collector agent object to stop tracecollector process
            CORBA::Object_var collectorAgtObj = CORBA::Object::_nil();
            TraceCollector::ProcessAgent_var collectorAgtRef =
                TraceCollector::ProcessAgent::_nil();

            try
            {
                collectorAgtObj =
                    orb->string_to_object(collectorAgentURL.c_str());
            }
            catch(const CORBA::Exception& ex)
            {
                std::cerr << "Invalid Trace Collector agent URL" << std::endl;
                std::cerr << ex << std::endl;
                status = FAILURE;
            }

            // get collector agent servant
            if(CORBA::is_nil(collectorAgtObj.in()))
            {
                std::cerr << "Trace Collector agent URL denotes "
                          << "a nil object reference" << std::endl;
                status = FAILURE;
            }

            try
            {
                if (status == SUCCESS)
                {
                    // Get reference to the TraceCollector agent interface
                    collectorAgtRef =
                        TraceCollector::ProcessAgent::_narrow(
                            collectorAgtObj.in());

                    if (CORBA::is_nil(collectorAgtRef.in()))
                    {
                        std::cerr << "Couldn't retrieve reference of "
                                  << "the TraceCollector agent" << std::endl;
                        status = FAILURE;
                    }
                }
            }
            catch(const CORBA::Exception& ex)
            {
                std::cerr << "Couldn't retrieve reference of the "
                        << "TraceCollector agent" << std::endl
                        << ex << std::endl;
                status = FAILURE;
            }

            try {
                if (status == SUCCESS)
                {
                    // then stop the trace collector process
                    collectorAgtRef->stop_process();
                }
            }
            catch (const CORBA::Exception& ex)
            {
                std::cerr << "Trace Collector Process Agent access error : "
                          << collectorAgentURL << std::endl;
                std::cerr << ex << std::endl;
                status = FAILURE;
            }
        }


        if (status == SUCCESS)
        {
            // set the global trace collector agent url
            std::string collectorAgentURL =
                "corbaloc::localhost:" +
                DEF_GBL_TRC_COLL_SERV_PORT + "/GlobalCollectorAgent";

            // get collector agent object to stop tracecollector process
            CORBA::Object_var collectorAgtObj = CORBA::Object::_nil();
            TraceCollector::ProcessAgent_var collectorAgtRef =
                TraceCollector::ProcessAgent::_nil();

            try
            {
                collectorAgtObj =
                    orb->string_to_object(collectorAgentURL.c_str());
            }
            catch(const CORBA::Exception& ex)
            {
                std::cerr << "Invalid Global Trace Collector agent URL"
                          << std::endl << ex << std::endl;
                status = FAILURE;
            }

            // get collector agent servant
            if (CORBA::is_nil(collectorAgtObj.in()))
            {
                std::cerr << "Global Trace Collector agent URL denotes "
                          << "a nil object reference" << std::endl;
                status = FAILURE;
            }

            try
            {
                if (status == SUCCESS)
                {
                    // Get reference to the TraceCollector agent interface
                    collectorAgtRef =
                        TraceCollector::ProcessAgent::_narrow(
                            collectorAgtObj.in());

                    if (CORBA::is_nil(collectorAgtRef.in()))
                    {
                        std::cerr << "Couldn't retrieve reference of "
                                  << "the Global Trace Collector agent"
                                  << std::endl;
                        status = FAILURE;
                    }
                }
            }
            catch(const CORBA::Exception& ex)
            {
                std::cerr << "Couldn't retrieve reference of the "
                          << "Global Trace Collector agent" << std::endl
                          << ex << std::endl;
                status = FAILURE;
            }

            try
            {
                if (status == SUCCESS)
                {
                    // then stop the trace collector process
                    collectorAgtRef->stop_process();
                }
            }
            catch (const CORBA::Exception& ex)
            {
                std::cerr << "Global Trace Collector Process Agent "
                          << "access error : " << collectorAgentURL
                          << std::endl << ex << std::endl;
                status = FAILURE;
            }
        }
    }
    catch(...)
    {
        std::cerr << "Catch exception during trace "
                  << "collector stopping" << std::endl;
        status = FAILURE;
    }

    // check statistics result (if Vertical trace format)
    OS::sleep(timescale * 1000);

    bool horizontalFormat = false;
    if (formatOption != "no")
    {
        if (traceFormat == Cdmw::Trace::Message::H_FORMAT)
        {
            horizontalFormat = true;
        }
    }

    status = auto_check(true, horizontalFormat, domainName);

    // then kill the Repository simulator and Name Server
    try
    {
        // And we kill the Repository simulator
        OS::kill_process(idRepository);
    }
    catch(const std::exception & ex)
    {
        std::cerr << "Cannot kill Repository process.:" << std::endl;
        status = FAILURE;
    }

    try
    {
        // And we kill the Name Server
        OS::kill_process(idNaming);
    }
    catch(const std::exception & ex)
    {
        std::cerr << "Cannot kill Name Server process.:" << std::endl;
        status = FAILURE;
    }

    // free memory allocated by String::to_char_array
    for (int iArgc = 0; iArgc < ORB_init_argc; ++iArgc)
    {
        delete[] ORB_init_argv[iArgc];
    }
    delete[] ORB_init_argv;

    // end of corba processing
    if (!CORBA::is_nil(orb.in()))
    {
        try
        {
            orb->shutdown(false);

            Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());

            orb->destroy();
            std::cerr << std::endl;
        }
        catch(const CORBA::Exception& ex)
        {
            std::cerr << ex << std::endl;
            status = FAILURE;
        }
    }

    return status;
}


