/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef CDMW_TRACE_CDMWTRACE_HPP
#define CDMW_TRACE_CDMWTRACE_HPP

#include <TraceAndPerf/tracelibrary//Trace.hpp>
#include <Foundation/osthreads/ThreadHandle.hpp>
#include <Foundation/logging/LogLevel.hpp>

/**
* Root namespace for CDMW runtime.
*/
namespace Cdmw
{

    /**
    * Trace level used in Cdmw to output information.
    * deprecated --> value must be in accordance with Logging::LogLevel
    */
    // typedef Logging::LogLevel TraceLevel;
    enum TraceLevel {

        /**
        * Information purpose
        */
        INF = Logging::INFO,

        /**
        * Maybe the user has made an error, so we warn it
        */
        WRN = Logging::WARN,

        /**
        * This level report an error from Cdmw.
        */
        ERR = Logging::ERROR,

        /**
        * The level reports an error from Cdmw, Cdmw is no more
        * able to process safely.
        */
        FTL = Logging::FATAL

    };

    /**
    * Trace domain used inside Cdmw
    *   CDMW_NREP Naming And Repository
    */
    extern const char* CDMW_NREP;

    /**
    *   CDMW_NINT Naming Interface
    */
    extern const char* CDMW_NINT;

    /**
    *   CDMW_LFCL Lifecycle
    */
    extern const char* CDMW_LFCL;

    /**
    *   CDMW_INIT CDMW Initialisation and Cleanup
    */
    extern const char* CDMW_INIT;

}

#endif // CDMW_TRACE_CDMW_TRACE_INCL

