/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/logging/LogLevel.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>

#include <TraceAndPerf/tracelibrary/FlushAreaMngr.hpp>
#include <TraceAndPerf/tracelibrary/Message.hpp>

#include "tracelibrary/LogManagerTraceDelegate.hpp"


namespace Cdmw
{
namespace Trace
{


/**
 * Constructor.
 */
LogManagerTraceDelegate::LogManagerTraceDelegate(FlushAreaMngr& flush_area_mngr)
    throw(OutOfMemoryException,
          InternalErrorException)
    : m_flush_area_mngr(flush_area_mngr)
{
}


/**
 * Destructor.
 */
LogManagerTraceDelegate::~LogManagerTraceDelegate()
    throw()
{
}


/**
 * Send a message to the Trace service.
 */
bool
LogManagerTraceDelegate::add_log(const CdmwLogging::LogContext& context,
                                 const CdmwLogging::LogHeader& header,
                                 const char* msg)
    throw(InternalErrorException)
{
    CDMW_MUTEX_GUARD(m_mutex);

    try 
    {
        m_flush_area_mngr.add_message(header, msg);
    }
    catch(const std::bad_alloc&) 
    {
        CDMW_THROW(OutOfMemoryException);
    }

    return true;
}


} // namespace Trace
} // namespace Cdmw
