/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "testtracelibrary/PILOT_FlushArea.hpp"

#include "TraceAndPerf/tracelibrary/Message.hpp"
#include "tracelibrary/FlushArea.hpp"


namespace Cdmw
{
namespace Trace
{

/**
*Implementation notes: [if no pertinent write none ]
*<p>
*Portability issues: [if no pertinent write none ]
*<p>
*/

CPPUNIT_TEST_SUITE_REGISTRATION( PILOT_FlushArea );

PILOT_FlushArea::PILOT_FlushArea()
{

}


PILOT_FlushArea::~PILOT_FlushArea()
    throw()
{

}


void 
PILOT_FlushArea::do_tests()
{
    // set number of requested successfull tests
//     set_nbOfRequestedTestOK (10);

    size_t areaSize;
    FlushArea::AreaStatus status;

    std::string message1 = "First Message";
    size_t len1 = message1.length();
    std::string message2 = "Second Message";
    size_t len2 = message2.length();
    std::string message3 = "Third Message";
    size_t len3 = message3.length();

    std::string outFlushAreaMsg = "trace Flush Are Manager is running out of flush areas";

    // constructs a message header
    CdmwLogging::LogHeader header;
    header.file_line = __LINE__;
    header.file_name = __FILE__;
    header.level = 1;
    header.user_domain = "TEST";
    header.component_name = ""; // ECR-0123
    header.thread_id = OsSupport::Thread::self();
    OsSupport::OS::Timeval time = OsSupport::OS::get_time();
    header.time_stamping.seconds = time.seconds;
    header.time_stamping.microseconds = time.microseconds;

    size_t headerStringSize = strlen(header.file_name) + 1;
    headerStringSize += strlen(header.user_domain) + 1;
    headerStringSize += strlen(header.component_name) + 1; // ECR-0123

 
    TEST_INFO ("Testing the number of messages limit for a flush area");
    FlushArea flushArea1 (2, 500, 100);
    status = flushArea1.add_message (header, message1);

    CPPUNIT_ASSERT (status == FlushArea::OK);
    status = flushArea1.add_message (header, message2);

    CPPUNIT_ASSERT (status == FlushArea::AREA_FULL);
    flushArea1.store_outFlushAreaMsg (outFlushAreaMsg);
    CPPUNIT_ASSERT(true);
    
    TEST_INFO ("Testing the size limit for a flush area");
    areaSize = len1 + len2 + 2;        // Adds 2 for the terminating NUL characters
    areaSize += 2 * headerStringSize;  // Adds 2 header size stored in area    
    FlushArea flushArea2 (3, areaSize, 100);
    status = flushArea2.add_message (header, message1);
    CPPUNIT_ASSERT (status == FlushArea::OK);
    status = flushArea2.add_message (header, message2);
    
    CPPUNIT_ASSERT (status == FlushArea::AREA_FULL);
    flushArea2.store_outFlushAreaMsg (outFlushAreaMsg);
    CPPUNIT_ASSERT(true);

    TEST_INFO ("Trying to exceed the size limit for a flush area");
    areaSize = len1 + len2 + 1;        // Adds 1 for only one terminating NUL characters
    areaSize += 2 * headerStringSize;  // Adds 2 header size stored in area
    FlushArea flushArea3 (3, areaSize, 100);
    status = flushArea3.add_message (header, message1);
    CPPUNIT_ASSERT (status == FlushArea::OK);    
    status = flushArea3.add_message (header, message2);;
    CPPUNIT_ASSERT (status == FlushArea::SIZE_TOO_SMALL);
    flushArea3.store_outFlushAreaMsg (outFlushAreaMsg);
    CPPUNIT_ASSERT(true);

    TEST_INFO ("Trying to add a message too big for a flush area");
    areaSize = len3;               // Adds no extra space for terminating NUL characters
    areaSize += headerStringSize;  // Adds 1 header size stored in area
    FlushArea flushArea4 (3, areaSize, 100);
    status = flushArea4.add_message (header, message3);
    CPPUNIT_ASSERT (status == FlushArea::MESSAGE_TOO_BIG);

}

}; // End namespace Trace
}; // End namespace Cdmw

