/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/ossupport/OS.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"

#include "TraceAndPerf/idllib/CdmwTraceTraceProducer.stub.hpp"
#include "TraceAndPerf/tracelibrary/FlushAreaMngr.hpp"

#include "testtracelibrary/PILOT_FlushAreaMngr.hpp"
#include "testtracelibrary/SIM_Collector.hpp"
#include "testtracelibrary/SIM_OrbThread.hpp"


namespace Cdmw
{
namespace Trace
{


/**
 *Implementation notes: [if no pertinent write none ]
 *<p>
 *Portability issues: [if no pertinent write none ]
 *<p>
 */

CPPUNIT_TEST_SUITE_REGISTRATION( PILOT_FlushAreaMngr );

PILOT_FlushAreaMngr::PILOT_FlushAreaMngr()
{
}


PILOT_FlushAreaMngr::~PILOT_FlushAreaMngr()
    throw()
{
}


void
PILOT_FlushAreaMngr::do_tests()
{
    // set number of requested successfull tests
//     set_nbOfRequestedTestOK (265);

    TEST_INFO( "Starting the Orb in it's own thread" );
    OrbThread orbThread;
    orbThread.start();

    CORBA::ORB_var orb = orbThread.get_orb();
    PortableServer::POA_var rootPOA = orbThread.get_poa();

    // create messages
    std::string message1 = "This is the first test message.";
    std::string message2 = "This is the second test message.";
    std::string message3 = "This is the third test message.";

    // constructs a message header
    CdmwLogging::LogHeader header;
    header.file_line = __LINE__;
    header.file_name = __FILE__;
    header.level = 1;
    header.user_domain = "TEST";
    header.component_name = ""; // ECR-0123
    header.thread_id = OsSupport::Thread::self();
    OsSupport::OS::Timeval time = OsSupport::OS::get_time();
    header.time_stamping.seconds = time.seconds;
    header.time_stamping.microseconds = time.microseconds;

    size_t headerStringSize = strlen(header.file_name) + 1;
    headerStringSize += strlen(header.user_domain) + 1;
    headerStringSize += strlen(header.component_name) + 1; // ECR-0123

    // create the initial collector list set to empty
    std::list<CdmwTrace::TraceProducer::CollectorData> collectorList;

    size_t flushingTime = 2000; // 2 seconds

    TEST_INFO("Initializing the TraceLibrary");
    CdmwTrace::TraceProducer_var traceProducer =
        FlushAreaMngr::init (rootPOA.in(),
                             collectorList,
                             "application_name",
                             "process_name",
                             flushingTime,
                             2,                     // 2 FlushArea
                             40 + headerStringSize, // Each FlushArea has 40 octets
                             Cdmw::Trace::FlushAreaMngr::DEFAULT_AREA_NB_MESSAGE,
                             100                    // threshold for repetitive messages
                             );

    CPPUNIT_ASSERT (!CORBA::is_nil(traceProducer.in()));
    CPPUNIT_ASSERT(true);

    TEST_INFO ("Retrieving the FlushAreaMngr singleton");
    FlushAreaMngr& flushAreaMngr=FlushAreaMngr::instance();
    flushAreaMngr.deactivate_flushing();      // deactivate flushing to perform test
    CPPUNIT_ASSERT(true);


    TEST_INFO ("Creating a fake collector");
    SIM_Collector* pSim_CollectorServant = new SIM_Collector;
    PortableServer::ServantBase_var servant_var = pSim_CollectorServant;

    CdmwTrace::Collector_var collector = pSim_CollectorServant->_this();

    TEST_INFO ("Registering the collector into the TraceProducer object");
    traceProducer->register_collector (collector.in(), "collector_name", "collector_mnemonic");
    CPPUNIT_ASSERT(true);

    bool ret;

    TEST_INFO ("Adding a first message");
    ret = flushAreaMngr.add_message (header, message1);
    CPPUNIT_ASSERT (ret == true);

    TEST_INFO ("Adding a second message");
    ret = flushAreaMngr.add_message (header, message2);
    CPPUNIT_ASSERT (ret == true);

    // as the flush area manager contains only 2 flush areas,
    // the third message cannot be added => error message is generated
    TEST_INFO ("Adding a third message");
    ret = flushAreaMngr.add_message (header, message3);
    CPPUNIT_ASSERT (ret == false);

    {
        TEST_INFO ("Checking if the Collector has not received messages");
        const CdmwTrace::FormattedMessageSeq& formattedMessagesSeq =
                               pSim_CollectorServant->get_FormattedMessageSeq();
        CORBA::ULong nbMessages = formattedMessagesSeq.length();

        CPPUNIT_ASSERT (nbMessages == 0);
    }

    flushAreaMngr.activate_flushing();      // activate flushing

    int timescale = Cdmw::TestUtils::get_timescale();
    TEST_INFO ("Waiting for a free flush area to add a third message");
    do
    {
        OsSupport::OS::sleep(timescale*flushingTime);
        ret = flushAreaMngr.add_message (header, message3);
        // as the flush area manager contains only 2 flush areas,
        // the third message will be added only after a flush area
        // had been released by the area flusher
    }
    while (!ret);

    TEST_INFO ("Third message added to the flush area manager");

    TEST_INFO ("Waiting a while to let the AreaFlusher does its work");
    OsSupport::OS::sleep(timescale*5*flushingTime);

    TEST_INFO ("Checking if the Collector has received messages");
    const CdmwTrace::FormattedMessageSeq& formattedMessagesSeq =
                               pSim_CollectorServant->get_FormattedMessageSeq();
    CORBA::ULong nbMessages = formattedMessagesSeq.length();
    TEST_INFO ("Nb received messages : " << nbMessages);
    TEST_INFO ("Received messages : ");
    for (CORBA::ULong i = 0 ; i < nbMessages ; i++)
    {
      TEST_INFO ("  " << formattedMessagesSeq[i].message.in());
    }
    // note : third message is "running out of flush areas"
    CPPUNIT_ASSERT (nbMessages == 4);

    // -----

    // message flooding
    TEST_INFO ("Deactivating the flush area manager");
    flushAreaMngr.deactivate_flushing();

    TEST_INFO ("Adding the second message 50 times");
    for (int i = 0; i < 50; ++i) {
        ret = flushAreaMngr.add_message (header, message2);
        CPPUNIT_ASSERT (ret == true);
    }
    CPPUNIT_ASSERT(true);

    TEST_INFO ("Reactivating the flush area manager");
    flushAreaMngr.activate_flushing();

    TEST_INFO ("Waiting for the AreaFlusher to do its work");
    OsSupport::OS::sleep(timescale * flushingTime * 5);

    TEST_INFO ("Checking the number of messages received by the Collector");
    const CdmwTrace::FormattedMessageSeq& formattedMessagesSeq2 =
        pSim_CollectorServant->get_FormattedMessageSeq();
    nbMessages = formattedMessagesSeq2.length();
    TEST_INFO ("Nb received messages : " << nbMessages);
    TEST_INFO ("Received messages : ");
    for (CORBA::ULong i = 0 ; i < nbMessages ; i++) {
        TEST_INFO ("  " << formattedMessagesSeq2[i].message.in());
    }
    CPPUNIT_ASSERT (nbMessages == 6);

    TEST_INFO ("Deactivating the flush area manager");
    flushAreaMngr.deactivate_flushing();

    TEST_INFO ("Adding the first message 88 times");
    for (int i = 0; i < 88; ++i) {
        ret = flushAreaMngr.add_message (header, message1);
        CPPUNIT_ASSERT (ret == true);
    }
    CPPUNIT_ASSERT(true);

    TEST_INFO ("Reactivating the flush area manager");
    flushAreaMngr.activate_flushing();

    TEST_INFO ("Waiting for the AreaFlusher to do its work");
    OsSupport::OS::sleep(timescale * flushingTime * 5);

    TEST_INFO ("Checking the number of messages received by the Collector");
    const CdmwTrace::FormattedMessageSeq& formattedMessagesSeq3 =
        pSim_CollectorServant->get_FormattedMessageSeq();
    nbMessages = formattedMessagesSeq3.length();
    TEST_INFO ("Nb received messages : " << nbMessages);
    TEST_INFO ("Received messages : ");
    for (CORBA::ULong i = 0 ; i < nbMessages ; i++) {
        TEST_INFO ("  " << formattedMessagesSeq3[i].message.in());
    }
    CPPUNIT_ASSERT (nbMessages == 8);

    TEST_INFO ("Deactivating the flush area manager");
    flushAreaMngr.deactivate_flushing();

    TEST_INFO ("Adding the third message 100 times");
    for (int i = 0; i < 100; ++i) {
        do {
            ret = flushAreaMngr.add_message (header, message3);
            // PCR-0901 slow down add_message speed
            OsSupport::OS::sleep(50);
            if (!ret) {
                std::cout << ".... add_message returned false. waiting..." << std::endl;
                OsSupport::OS::sleep(timescale*flushingTime);
            }
        } while (!ret);

        CPPUNIT_ASSERT (ret == true);
    }
    CPPUNIT_ASSERT(true);

    TEST_INFO ("Reactivating the flush area manager");
    flushAreaMngr.activate_flushing();

    TEST_INFO ("Waiting for the AreaFlusher to do its work");
    OsSupport::OS::sleep(timescale * flushingTime * 5);

    TEST_INFO ("Checking the number of messages received by the Collector");
    const CdmwTrace::FormattedMessageSeq& formattedMessagesSeq4 =
        pSim_CollectorServant->get_FormattedMessageSeq();
    nbMessages = formattedMessagesSeq4.length();
    TEST_INFO ("Nb received messages : " << nbMessages);
    TEST_INFO ("Received messages : ");
    for (CORBA::ULong i = 0 ; i < nbMessages ; i++) {
        TEST_INFO ("  " << formattedMessagesSeq4[i].message.in());
    }
    CPPUNIT_ASSERT (nbMessages == 10);

    // -----

    TEST_INFO("Checking if an unexisting domain/level is to be traced");
    
    traceProducer->deactivate_level ("", CdmwTrace::ALL_DOMAINS, CdmwTrace::ALL_VALUES);
    traceProducer->activate_level ("", "domain1", 1);
    CPPUNIT_ASSERT (FlushAreaMngr::is_filter_activated("", "foo", 32) == false);

    TEST_INFO ("Trying to activate a domain/level using the TraceProducer interface");
    traceProducer->activate_level ("", "foo", 32);
    CPPUNIT_ASSERT(true);

    TEST_INFO("Checking if the level is returned activated by");
    TEST_INFO("     the FilterMngr");
    CPPUNIT_ASSERT (FlushAreaMngr::is_filter_activated("", "foo", 32) == true);

    
    TEST_INFO ("Trying to deactivate a domain/level using the TraceProducer interface");
    traceProducer->deactivate_level ("", "foo", 32);
    CPPUNIT_ASSERT(true);
    
    TEST_INFO("Checking if the level is returned deactivated by");
    TEST_INFO("     the FilterMngr");
    CPPUNIT_ASSERT(FlushAreaMngr::is_filter_activated("", "foo", 32) == false);

    TEST_INFO("Doing the test for different Domain/level");

    traceProducer->activate_level("", "foo", 2);
    traceProducer->activate_level("", "foo", 8);
    CPPUNIT_ASSERT (FlushAreaMngr::is_filter_activated("", "foo", 2) == true);
    CPPUNIT_ASSERT (FlushAreaMngr::is_filter_activated("", "foo", 8) == true);

    traceProducer->deactivate_level ("", "foo", 2);
    CPPUNIT_ASSERT (FlushAreaMngr::is_filter_activated("", "foo", 2) == false);

    traceProducer->activate_level ("", "bar", 8);
    CPPUNIT_ASSERT (FlushAreaMngr::is_filter_activated("", "bar", 8) == true);

    traceProducer->activate_level ("", "bar", 4);
    CPPUNIT_ASSERT (FlushAreaMngr::is_filter_activated("", "bar", 4) == true);

    traceProducer->deactivate_level ("", "bar", 8);
    CPPUNIT_ASSERT (FlushAreaMngr::is_filter_activated("", "bar", 8) == false);

    TEST_INFO ("Trying to cleanup the TraceLibrary");
    FlushAreaMngr::cleanup();
    CPPUNIT_ASSERT(true);
    
    TEST_INFO ("Stopping the ORB");
    orbThread.shutdown();
    orbThread.join();
}



}; // End namespace Trace
}; // End namespace Cdmw

