/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include "test16/Recorder_impl.hpp"
#include "test16/RecordingServerEmulator.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"

#include "Foundation/testutils/CORBATestManager.hpp"
#include "recordingtools/CdmwRecordingContainerManager_impl.hpp"
#include "recordingtools/CdmwRecordingDaemonControl_impl.hpp"

RecordingServerEmulatorThread::RecordingServerEmulatorThread(std::string dm_host)
{
    daemon_host = dm_host;
}

int RecordingServerEmulatorThread::svc ()
{
    try
    {
        std::cout << "RSE: initialising.." << std::endl;
        
        // Initialize orb
        CORBA::ORB_var orb = Cdmw::TestUtils::CORBATestManager::instance()->get_ORB();
        
        // Get reference to Root POA.
        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
                   
        PortableServer::POA_var root_poa = PortableServer::POA::_narrow(obj);

        //Create IdAssignmentPolicy        
        PortableServer::IdAssignmentPolicy_var assignment_policy = root_poa->create_id_assignment_policy(PortableServer::USER_ID);
        // Create PolicyList
        CORBA::PolicyList policy_list;
        policy_list.length(1);
        policy_list[0] = PortableServer::IdAssignmentPolicy::_duplicate(assignment_policy);
        // Create the child POA
        PortableServer::POA_var poa = root_poa->create_POA("poa", PortableServer::POAManager::_nil(), policy_list);
        // Destroy object
        assignment_policy->destroy();
        
        // Activate POA manager
        PortableServer::POAManager_var mgr = poa->the_POAManager();
        mgr->activate();

        // Create an object
        PortableServer::ObjectId_var oid = PortableServer::string_to_ObjectId("Recorder");
    
        // Create our servant.
        POA_CdmwRecording::Recorder_impl recorder_servant;
        // Activate our Controller.
        poa->activate_object_with_id(oid, &recorder_servant);
        CORBA::Object_ptr object = poa->servant_to_reference (&recorder_servant);

        const std::string corbaloc_name = "Recorder" ;
        Cdmw::OrbSupport::OrbSupport::bind_object_to_corbaloc (orb.in(), corbaloc_name, object);        
                
        //-----------
        std::cout << "RSE: register to recording daemon.." << std::endl;
        
        std::string corbaloc = std::string ("corbaloc::") 
                             + daemon_host + (":59875/RecordingDaemonController" );     
        CORBA::Object_var o_daemon = orb->string_to_object (corbaloc.c_str());
        CdmwRecording::DaemonControl_var daemon = CdmwRecording::DaemonControl::_narrow(o_daemon);
        
        if(CORBA::is_nil(daemon))
            std::cout << "## could not find recording daemon (" << corbaloc << ")" << std::endl; 
        else 
        { 
            std::cout << "RSE: found recording daemon "<< std::endl;
        
            char host_name[100] = "";
            gethostname(host_name, sizeof(host_name));
            std::string str_url = std::string("corbaloc:iiop:")
                                + std::string(host_name) + ":12344/Recorder";  
        
            std::cout << "url : +" << str_url << "" << std::endl;
        
            daemon->setRecorderUrl (0, str_url.c_str());
            std::cout << "RSE: recorder_url - ok "<< std::endl;     
        }
        
        // Accept requests
        orb->run();
    }
    catch (const CORBA::Exception& ex)
    {
        std::cout << "RSE: CORBA Exception: " << ex << std::endl;
        return -1;
    }

    return 0;
}
