/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <fstream>
#include <iomanip>

#include "test16/Transmission.stub.hpp"
#include "test16/RecordingServerEmulator.hpp"
#include "test16/Recorder_impl.hpp"
#include "test16/Test16.hpp"


CPPUNIT_TEST_SUITE_REGISTRATION( Test16 );


Test16::Test16()
{
}
Test16::~Test16()
    throw()
{
}


using namespace std;
void Test16::check_result()
{
    int pairs = 0;
    int length = 0;
    string line = "";
    int count = 0;
      
    ifstream inFile;
    inFile.open("recordingserver_data");
    
    while(!inFile.is_open() && count < 3)
    {
      std::cout<<"Opening recordingserver_data ... retry ..."<<std::endl;
      sleep (1);
      count ++;
      inFile.open("recordingserver_data");
    }
    if(!inFile.is_open()) 
    {
        cout<<"## could not open recordingserver_data ## "<<endl;        
    }
    else 
    { 
        std::cout<<"opened recordingserver_data, waiting 5 sec ... "<<std::endl;
    
        sleep(5); // wait for Recording Server to flush data
          
        while(inFile >> line) // 
        {
            if(line.find("buffer",0)!= std::string::npos) 
            {
               if(inFile >> line)
               {
                    if(line.find("hostname",0)!= std::string::npos)  inFile >> line ;               
               } 
               if(inFile >> line)
               {
                    if(line.find("num_pairs",0)!= std::string::npos)  
                    {
                        inFile >> line ; 
                        pairs += atoi(line.c_str());
                    }              
               }
               if(inFile >> line) 
               {
                    if(line.find("length",0)!= std::string::npos)  
                    {
                        inFile >> line ; 
                        std::cout<<"buffer length :"<< line << std::endl;
                        length += atoi(line.c_str());
                    }              
                }
            }       
        }
        inFile.close();
        std::cout << "*** "<< pairs << " pairs found" << std::endl ;
        std::cout << "*** common buffer length: "<< length << std::endl ;
    }
    // client sends 
    // 1. 1 message with 100 bytes size -> buffer length = 249
    // 2. 1 message with 200 bytes size -> buffer length = 349
    // (numbers were received with modified validation test of coalescencemanager)    
    // recorded data should be from second message (length = 349)   
    bool result = (length > 200 && length < 400) ;

    std::cout << "  length= " << length << std::endl;
    std::string str_result = std::string("The test scenario is ") + (result ? std::string("OK") : std::string("FAILED")) ;
  
    //set output folder
    std::string output_folder = "V-TRC-016-results";
    
    //create output folder
    if (mkdir(output_folder.c_str(), 777) != 0)
          std::cerr << "  Error creating output directory" << std::endl;
  
      //move result file
    std::string file = output_folder + "/recordingserver_data";
    rename("recordingserver_data", file.c_str());
      
   
    //set output file
    char host_name[100] = "";
    gethostname(host_name, sizeof(host_name));
    std::string output_file = output_folder + "/execution-" + host_name + ".output";
    
    //output test result
    ofstream om(output_file.c_str());
    
    if(om) om << str_result << std::endl;
    
    std::cout << str_result << std::endl;  
    
    CPPUNIT_ASSERT(result);  
}
void Test16::do_tests() 
{    
    int timeout;
    std::string daem_host;
    bool send_finish = false;
    std::string str;
 
    try 
    { 
        // get parameters           
        str = Cdmw::TestUtils::TestManager::instance()->get_property("timeout"); 
        timeout = atoi(str.c_str());
        
        daem_host = Cdmw::TestUtils::TestManager::instance()->get_property("host1");                   
       
        str =  Cdmw::TestUtils::TestManager::instance()->get_property("send_finish"); 
        send_finish = (atoi(str.c_str()) == 1);

        CORBA::ORB_var orb ;
        orb = Cdmw::TestUtils::CORBATestManager::instance()->get_ORB();
        
        CORBA::Object_var obj1 = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var root_poa = PortableServer::POA::_narrow(obj1);   
                           
        // find servant
        std::string corbaloc = std::string ("corbaloc::") 
                             + daem_host + (":6789/Transmission" );     
        CORBA::Object_var obj = orb->string_to_object(corbaloc.c_str());
        
        CPPUNIT_ASSERT_MESSAGE("Nil reference", false ==CORBA::is_nil(obj));
    
        // Narrow
        Transmission_var tm = Transmission::_narrow(obj);
        
        CPPUNIT_ASSERT_MESSAGE("not a Transmission reference", false == CORBA::is_nil(tm));  

        // Send data
        std::cout << std::endl << "Sending 1 message..." << std::endl; 
        int size = 100;    
        std::cout << " size " << size << std::endl; 
       
        char data1[size];
        for (int j = 0; j <= size-1; j++) data1[j] = 0xCF;
        data1[size] = '\0';

        tm->send(data1);
             
        std::cout << "ok, waiting till timeout is expired (" 
                    << timeout << " sec )" << endl;
        
        sleep(timeout);
        
        std::cout << "starting recording server emulator" << endl ;  
        
        // Start RecordingServer emulator 
        RecordingServerEmulatorThread * rse = new RecordingServerEmulatorThread(daem_host);
        rse->activate(THR_NEW_LWP | THR_JOINABLE, 1);       
        cout << "waiting 5 sec.." << endl;
        sleep(5); 
    
        cout << "activated recording server emulator" << endl;  
             
        // Send data
        std::cout << "Sending 1 message..." << std::endl;  
        size = 200;
        std::cout << " size " << size << std::endl;    
      
        char data2[size];
        for (int j = 0; j <= size-1; j++) data2[j] = 0xCF;  
        data2[size] = '\0';
       
        tm->send(data2);
           
        std::cout << "ok, waiting 7 sec.." << endl;     
        sleep(7); 
    
        check_result(); 
        
        if (send_finish)
    {
           tm->terminate();
    }               
    }
    catch (const CORBA::Exception & ex) 
    {
      cout << "Test16: CORBA exception: " << ex << endl;
      throw;
    }
    catch (...) 
    { 
        cout << "Test16: exception" << endl;
        throw;
    }
}

