/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/ossupport/OS.hpp>
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"

#include "test16/Test16.hpp"

    
static void
usage(std::ostream & os, const std::string &program_name)
{
    os << "Usage:\n"
       << program_name << " -iorfile -t \n"
       << "\n"
       << "Parameters:\n"
       << "-t=<timeout>         timeout (sec) \n"  
       << "-d=<host>            daemon host (where recording daemon is started)\n"       
       << std:: endl;
}

int main(int argc, char** argv)
{   
 
    if (argc > 1)
    {
        if(strcmp(argv[1], "--help") == 0)
        {
            usage(std::cout, argv[0]);
            return 0;
        }
    }
    try 
    {           
    // Initialize orb
     
    Cdmw::OrbSupport::StrategyList orb_strategies;
    orb_strategies.add_OrbThreaded();
    orb_strategies.add_PoaThreadPool(5);    
            
    CORBA::ORB_var orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, orb_strategies);
     
    CORBA::Object_var obj1 = orb->resolve_initial_references("RootPOA");
    PortableServer::POA_var root_poa = PortableServer::POA::_narrow(obj1);        
    Cdmw::TestUtils::CORBATestManager::instance()->init(orb.in(), root_poa.in(), argc, argv);
   
    //get parameters from command line and transmit them in test class    
    std::string timeout = ""; 
    std::string host1 = "";
    std::string send_finish = "";
    
    if( argc > 0 && argv )
    {       
      // timeout
        timeout = Cdmw::OsSupport::OS::get_option_value(argc, argv, "-t");
        if (timeout != "no")
        {
            Cdmw::TestUtils::TestManager::instance()->set_property("timeout", timeout);
        }
        else
        {
            std::cerr << "There is no timeout argument (-t=<value in sec>)" << std::endl;
            return 1;
        } 
    // host    
        host1 = Cdmw::OsSupport::OS::get_option_value(argc, argv, "-d");
        if (host1 != "no")
        {
            Cdmw::TestUtils::TestManager::instance()->set_property("host1", host1);
        }
        else
        {
            std::cerr << "There is daemon host argument (-d=<hostname>)" << std::endl;
            return 1;
        }  
        // send finish or not
        send_finish = Cdmw::OsSupport::OS::get_option_value(argc, argv, "-send_finish");
        if (host1 != "no")
        {
            Cdmw::TestUtils::TestManager::instance()->set_property("", send_finish);
        }
        else
        {
            std::cerr << "There is no argument (-send_finish = 0 | 1)" << std::endl;
            return 1;
        }          
    }
    
    Cdmw::TestUtils::CORBATestManager::instance()->run_tests();
    Cdmw::TestUtils::CORBATestManager::instance()->summary();
    
    }
    catch(...)
    {
        std::cout<<"exception in main.cpp "<< std::endl;
        throw;
    }
    
    return 0;
}



