/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "test18/TestMessageIdentifier.hpp"
#include "test18/MyPlugin.hpp"

#include <cstdlib>
#include <string>
#include <iostream>
#include <fstream>
#include <signal.h>
#include <errno.h>
#include <stdio.h>
//#include <semaphore.h>
#include <unistd.h>
#include <sys/sem.h>
#include <sys/ipc.h>
#include <sys/stat.h>   //S_IRUSR|S_IWUSR
#include <fcntl.h>      //O_CREAT

namespace
{
  const int SUCCESS = 0;
  const int FAILURE = 1;
}

#define IPC_ALL 0666

CPPUNIT_TEST_SUITE_REGISTRATION( TestMessageIdentifier );

TestMessageIdentifier::TestMessageIdentifier()
{
}
TestMessageIdentifier::~TestMessageIdentifier()
throw()
{
}

int TestMessageIdentifier::init_sniffer(Cdmw::Recording::NetworkSniffer * sniffer, int port_number)
{
  Cdmw::Recording::NetworkSniffer::Status status = sniffer->addPort(port_number);
  CPPUNIT_ASSERT_MESSAGE("adding port error", SUCCESS == status);

  status = sniffer->run () ; 
  CPPUNIT_ASSERT_MESSAGE("running sniffer error", SUCCESS == status);

  status = sniffer->start () ;

  return SUCCESS;
}

int TestMessageIdentifier::init_message_identifier(Cdmw::Recording::MessageIdentifier * message_identifier)
{
  Cdmw::Recording::MessageIdentifier::Status status = message_identifier->run();
  CPPUNIT_ASSERT_MESSAGE("*** test18: run message identifier error", 0 == status);

  status = message_identifier->start();
  CPPUNIT_ASSERT_MESSAGE("*** test18: start message identifier error", 0 == status);

  return SUCCESS;
}

int TestMessageIdentifier::terminate_sniffer(Cdmw::Recording::NetworkSniffer *sniffer)
{
  std::cout << "=== test18: stopping sniffer" << endl;
  Cdmw::Recording::NetworkSniffer::Status status = sniffer->stop () ; 
  CPPUNIT_ASSERT_MESSAGE("*** test18: stopping sniffer error", SUCCESS == status);

  std::cout << "=== test18: terminating sniffer" << endl;
  status = sniffer->terminate();
  CPPUNIT_ASSERT_MESSAGE("*** test18: terminating sniffer error", SUCCESS == status);

  return SUCCESS;
}
void TestMessageIdentifier::print_result(int count_request, int count_empty_reply)
{
  bool result = (count_empty_reply == 1); 

  std::string str_result = std::string("The test scenario is ") 
                           + (result ? std::string("OK") : std::string("FAILED")) ;

  std::cout << "=== test18: plugin: "<< count_request << " requests, " 
      << count_empty_reply << " empty replies" << std::endl; 
  std::cout << str_result << std::endl;

  //create output folder
  std::string output_folder = "V-TRC-018-results";
  if (mkdir(output_folder.c_str(), 777) != 0)
  {  
     std::string str = "!!! test18 : creation of directory : " + output_folder;
     if (errno != 0)
      {
        perror (str.c_str());
    //std::cout << " file exists" << std::endl;
      }
  }
  //set output file
  char host_name[100] = "";
  gethostname(host_name, sizeof(host_name));
  std::string output_file = output_folder + "/execution-" + host_name + ".output";

  //output test result
  ofstream om(output_file.c_str());

  if(om) 
  {
    om << "plugin: "<< count_request << " requests, " 
        << count_empty_reply << " empty replies" << std::endl; 
    om << str_result << std::endl;
  }
  else std::cout << "*** test18: could not open output file "<< output_file << std::endl;

  std::cout << "=== test18 : checking ..." << std::endl;
  CPPUNIT_ASSERT(result);    
}

key_t File_To_Key (const char * path)
{
  int fd = ::open (path, O_RDWR | O_CREAT) ;
  ::close (fd) ;
  return ::ftok (path, 1) ;
}

void TestMessageIdentifier::check_message_identifier()
{
  int port_number = 0;
  std::string s = "";

  try
  {
    port_number = atoi(Cdmw::TestUtils::TestManager::instance()->get_property("port_number").c_str());

    //sem_t * sema = sem_open("/tmp/test18_semaphore",
    //O_CREAT,S_IRUSR|S_IWUSR, 0);                        
    //sem_init(sema,0,0);
    // create semaphore
    // printf ("=== test18 : get the  semaphore\n");
    int m_id = ::semget (File_To_Key("/tmp/test18_semaphore"), 1, IPC_CREAT | IPC_ALL);
    if (m_id < 0)
    {
      if (errno != 0)
      {
        perror ("*** test18 : Error on semget.\n");
      }
    }

    //Create Sniffer
    std::cout << std::endl <<"=== test18 : Create NetworkSniffer.." <<std::endl;
    Cdmw::Recording::PacketStore store;  
    Cdmw::Recording::NetworkSniffer sniffer(store);

    //Create InterfaceFilter
    std::cout << "=== test18 : Create InterfaceFilter.." <<std::endl;
    Cdmw::Recording::InterfaceFilter interface_filter("interfaces.txt");

    //Create OperationMap
    std::cout << "=== test18 : Create OperationMap.." <<std::endl;
    Cdmw::Recording::OperationMap operation_map("operations.txt");

    //Create ObjectMap
    std::cout << "=== test18 : Create ObjectMap.." <<std::endl;
    Cdmw::Recording::ObjectMap object_map(interface_filter, sniffer);

    //Create RequestPairing
    std::cout << "=== test18 : Create RequestPairing.." <<std::endl;
    Cdmw::Recording::RequestPairing pairing(store);

    std::cout << "=== test18 : add MyPlugin.." <<std::endl;    
    Cdmw::Recording::MyPlugin plug;
    pairing.addPlugin(&plug);

    unsigned long to = Cdmw::Recording::get_Orphan_Request_Timeout();
    std::cout << "=== test18 : timeout for orphan requests : " << to <<std::endl;

    //Init sniffer
    std::cout << "=== test18 : Init NetworkSniffer.." <<std::endl;
    CPPUNIT_ASSERT_MESSAGE("*** test18 : init_sniffer() failed", SUCCESS == init_sniffer(&sniffer, port_number));

    //Create MessageIdentifier
    Cdmw::Recording::MessageIdentifier message_identifier(store, object_map, interface_filter, operation_map, pairing);

    //Init MessageIdentifier
    CPPUNIT_ASSERT_MESSAGE("*** test18 : init_message_identifier() failed", SUCCESS == init_message_identifier(&message_identifier));
    std::cout << "=== test18 : Message processing started" << endl;      

    std::cout << "=== test18: waiting for client" << std::endl;
    //sem_wait(sema);
    // Lock semaphore
    int index = 0;
    while (1) {
      struct sembuf Op = { index, -1, 0 } ;
      int res = ::semop (m_id, &Op, 1) ;
      if (res ==  -1) {
        if (errno != EINTR) {
          perror ("*** test18: SemaphoreSet::lock failed") ;
        }
      }
      else {
        break ;
      }
    }   

    std::cout << "=== test18:  sleeping "<< 3*(to/1000) << "s ..." << std::endl;
    sleep (3*(to/1000));
    //sleep (3);
    std::cout << "=== test18:  stopping server.." << std::endl; 

    ifstream im("server.pid"); 
    int server_pid ;
    im >> server_pid;    
    im.close();
    remove("server.pid");
    kill(server_pid, SIGKILL);
    remove("server_data");

    std::cout << "=== test18: stopped server, waiting 3 sec .." << std::endl;
    sleep(3);

    print_result(plug.count_request, plug.count_empty_reply);  
  
    //CPPUNIT_ASSERT(plug.count_empty_reply== 1);    

    std::cout << "=== test18: stopping..." << std::endl;

    //Stop message processing
    Cdmw::Recording::MessageIdentifier::Status status = message_identifier.stop();
    CPPUNIT_ASSERT_MESSAGE("message_identifier.stop() failed", 0 == status);

    std::cout << "=== test18: Message processing stopped" << endl;
    sleep(1);
    //Terminate message processing
    status = message_identifier.terminate();
    CPPUNIT_ASSERT_MESSAGE("terminate message identifier error", 0 == status);

    std::cout << "=== test18: Message processing terminated" << endl;

    //Stop services
    std::cout << "=== test18: finalizing pairing" << endl;
    pairing.finalize();   
    std::cout << "=== test18: finalizing sniffer" << endl;
    terminate_sniffer( &sniffer);        
    std::cout << "=== test18: finalizing packetstore" << endl;
    store.finalize(); 

    std::cout << "=== test18: done" << endl;        
  }    
  catch(const std::exception &e)
  {
    std::cerr << "*** test18: TestMessageIdentifier: exception raised :" << std::endl
        << e.what() << std::endl;
    throw;
  }
  catch(...)
  {
    std::cerr << "*** test18: TestMessageIdentifier: exception raised :" << std::endl;
    throw;
  }     
}



