/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/ossupport/OS.hpp>
#include "test18/TestMessageIdentifier.hpp"
#include <Foundation/ossupport/SystemProperties.hpp>

    
static void
usage(std::ostream & os, const std::string &program_name)
{
    os << "Usage:\n"
       << program_name << " -pn -rc -io\n"
       << "\n"
       << "Parameters:\n"
       << "-pn=<port_number>           TCP port to be spied\n"      
       << std:: endl;
}

int main(int argc, char** argv)
{
 
    if (argc > 1)
    {
        if(strcmp(argv[1], "--help") == 0)
        {
            usage(std::cout, argv[0]);
            return 0;
        }
    }

    Cdmw::OsSupport::SystemProperties::Init(argc, argv);

    using Cdmw::Logging::LogManager;
    LogManager::Init(argc, argv);
               
    Cdmw::TestUtils::TestManager::instance()->init(argc,argv);

    //get parameters from command line and transmit them in test class
    std::string port_number = ""; 
  
  std::cout << "=== test18 : main(0) " << std::endl;  
    
    if( argc > 0 && argv )
    {

        port_number = Cdmw::OsSupport::OS::get_option_value(argc, argv, "-pn");
        if (port_number != "no")
        {
            Cdmw::TestUtils::TestManager::instance()->set_property("port_number", port_number);
        }
        else
        {
            std::cerr << "There is no port_number argument" << std::endl;
            return 1;
        }     
    }
  
    try
      {
    std::cout << "=== test18 : run_tests (0) " << std::endl;  
    Cdmw::TestUtils::TestManager::instance()->run_tests();
    std::cout << "=== test18 : run_tests (1) " << std::endl;  
      }
    catch (const std::exception &ex)
      {
    std::cerr << "*** Exception in run_tests : " << ex.what() << std::endl;  
      }
    
    std::cout << "=== test18 : Summary " << std::endl;  
    Cdmw::TestUtils::TestManager::instance()->summary();
    
    return 0;
}



