/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package tests.OrbBench;

import java.io.File;
import java.io.FileWriter;
import java.util.Vector;

import org.omg.CosNaming.NamingContextExt;

import tests.OrbBench.Perfs_TypesPackage.Big_Info_TableHelper;
import tests.OrbBench.Perfs_TypesPackage.Big_Info_With_String_TableHelper;
import tests.OrbBench.Perfs_TypesPackage.Info;
import tests.OrbBench.Perfs_TypesPackage.Info_With_String;
import tests.OrbBench.Perfs_TypesPackage.Small_Info_TableHelper;
import tests.OrbBench.Perfs_TypesPackage.Small_Info_With_String_TableHelper;

//import jacorb.util.*;

public class Client 
{

   private static final int NB_MESSAGE_EACH_TEST = 10;

// #RTJ350 static String testSuite = "cCsSlLdDiIyYaAbBURvV";
   static String testSuite = "cCsSlLdDiIyYaAURvV";

   static final String COMMENT_1 = "|<---------------- Round Trip timer ------------------------>      ";
   static final String COMMENT_2 = "|                                                                  ";
   static final String COMMENT_3 = "|<----------> REQ_build                                            ";
   static final String COMMENT_4 = "|           .                                                      ";
   static final String COMMENT_5 = "|           |<------ REQ_trans ---------------------------->|      ";
   static final String COMMENT_6 = "|           |                                               |      ";
   static final String COMMENT_7 = "Client   -> [...Socket...] -> Server    -> [...Socket...] -> Client";

   static String SEP       = "     ";
   static final String HEADER_schar  = "| char  ";
   static final String HEADER_bchar  = "| CHAR  ";
   static final String HEADER_sshort = "| short ";
   static final String HEADER_bshort = "| SHORT ";
   static final String HEADER_slong  = "| long  ";
   static final String HEADER_blong  = "| LONG  ";
   static final String HEADER_sdouble  = "| double";
   static final String HEADER_bdouble  = "| DOUBLE";
   static final String HEADER_sinfo  = "| info  ";
   static final String HEADER_binfo  = "| INFO  ";
   static final String HEADER_vbinfo = "| VBINFO";
   static final String HEADER_sinfostr = "|infostr";
   static final String HEADER_binfostr = "|INFOSTR";
   static final String HEADER_sany   = "|  any  ";
   static final String HEADER_bany   = "|  ANY  ";
   static final String HEADER_sanystr  = "| anystr";
   static final String HEADER_banystr  = "| ANYstr";
   static final String HEADER_ulongseq = "| ULSEQ ";
   static final String HEADER_ref    = "|  REF  ";
   static final String HEADER_urefseq  = "| URSEQ ";
   static final String HEADER_uinfoseq = "| UISEQ ";
   static final String HEADER_sinfoval= "|infoVal";
   static final String HEADER_binfoval= "|INFOVAL";

   private static final    int       SLEEP_MS      =   0;  // 500;

   private static boolean        shortTest     = false;

   private static      int       loopTimes     =  2;  // -1 => infinite
   private static      int       loopNum     =   1;


   private static NamingContextExt     nc;
   private static Perfs        theperf;
   private static java.text.DecimalFormat  dblPFormater;
   private static org.omg.CORBA.ORB      orb;


   static Vector vc;
   static Vector vC;
   static Vector vs;
   static Vector vS;
   static Vector vl;
   static Vector vL;
   static Vector vd;
   static Vector vD;
   static Vector vi;
   static Vector vI;
   static Vector vy;
   static Vector vY;
   static Vector vV;
   static Vector va;
   static Vector vA;
   static Vector vb;
   static Vector vB;
   static Vector vULS;
   static Vector vR;
   static Vector vURS;
   static Vector vUIS;
   static Vector viv;
   static Vector vIV;

   //------------------------------------------------------------------------------------------
   //------------------------------------------------------------------------------------------
   //------------------------------------------------------------------------------------------
   public static void main(String[] args)
   {
      // ------------------------------------------------------------------------
      // Parse command line parameters ------------------------------------------
      // ------------------------------------------------------------------------
      if(! parse_command_line( args ) )
         System.exit( 0 );

      // ------------------------------------------------------------------------
      // ORB INIT ---------------------------------------------------------------
      // ------------------------------------------------------------------------
      try {

         System.out.println("\n       Top Graph'X      ");
         System.out.println("------------------------");
         System.out.println("");
         System.out.println("TestSuite :" + testSuite);

         dblPFormater = new java.text.DecimalFormat( "##.###" );
         dblPFormater.setGroupingSize( 3 );

         // Initialize the ORB
         orb = org.omg.CORBA.ORB.init(args,null);

         if(args[0].equals("JacORB"))
         {
            File _file = new File("/tmp/test29.ior");
            java.io.FileReader _fileReader = new java.io.FileReader(_file);
            java.io.BufferedReader br =
               new java.io.BufferedReader(_fileReader);
            String ior = br.readLine();
            _fileReader.close();
            theperf = PerfsHelper.narrow(orb.string_to_object(ior));
         }

         // --------------------------------------------------------------------
         // SINGLE CALL TO Small_Info_Table ------------------------------------
         // --------------------------------------------------------------------
         if( shortTest )
         {
            do_shortTest();
            System.out.println(Server.NAME + ": Client shortTest done");
            theperf.Stop();
            System.exit( 0 );
         }

         // --------------------------------------------------------------------
         // Boost ClassLoader --------------------------------------------------
         //  --------------------------------------------------------------------
         {
            //System.out.println("\nBoost ClassLoader with Small_Char_Table:");
            char[] xx = new char [144];
            ((PerfsOperations)theperf).Echo_Small_Char_Table (xx);
            //System.out.println("End echo\n");
         }


         // --------------------------------------------------------------------
         // tests.ORBInit start date and time -------------------------------------------
         // --------------------------------------------------------------------
         //System.out.println("DB:tests.ORBInit start date and time");
         java.text.SimpleDateFormat formatter
         = new java.text.SimpleDateFormat( "yyyy.MM.dd HH:mm:ss" );

         //System.out.println( formatter.format( new Date() ) );

         // --------------------------------------------------------------------
         // Columns header -----------------------------------------------------
         // --------------------------------------------------------------------
         //System.out.println("DB:Columns header");
//       System.out.println( COMMENT_1 );
//       System.out.println( COMMENT_2 );
//       System.out.println( COMMENT_3 );
//       System.out.println( COMMENT_4 );
//       System.out.println( COMMENT_5 );
//       System.out.println( COMMENT_6 );
//       System.out.println( COMMENT_7 );
//       System.out.println( "\n"      );


         System.out.print("     ");
         if(testSuite.indexOf( 'c' ) >= 0){   System.out.print(HEADER_schar); SEP += "........"; vc = new Vector();}
         if(testSuite.indexOf( 'C' ) >= 0){   System.out.print(HEADER_bchar); SEP += "........"; vC = new Vector(); }
         if(testSuite.indexOf( 's' ) >= 0){   System.out.print(HEADER_sshort);SEP += "........"; vs = new Vector();}
         if(testSuite.indexOf( 'S' ) >= 0){   System.out.print(HEADER_bshort);SEP += "........"; vS = new Vector();}
         if(testSuite.indexOf( 'l' ) >= 0){   System.out.print(HEADER_slong); SEP += "........"; vl = new Vector(); }
         if(testSuite.indexOf( 'L' ) >= 0){   System.out.print(HEADER_blong); SEP += "........"; vL = new Vector(); }
         if(testSuite.indexOf( 'd' ) >= 0){   System.out.print(HEADER_sdouble);SEP += "........"; vd = new Vector();}
         if(testSuite.indexOf( 'D' ) >= 0){   System.out.print(HEADER_bdouble);SEP += "........"; vD = new Vector();}
         if(testSuite.indexOf( 'i' ) >= 0){   System.out.print(HEADER_sinfo);SEP += "........"; vi = new Vector();}
         if(testSuite.indexOf( 'I' ) >= 0){   System.out.print(HEADER_binfo);SEP += "........"; vI = new Vector();}
         if(testSuite.indexOf( 'N' ) >= 0){   System.out.print(HEADER_vbinfo);SEP += "........"; vV = new Vector();}
         if(testSuite.indexOf( 'y' ) >= 0){   System.out.print(HEADER_sinfostr);SEP += "........"; vy = new Vector();}
         if(testSuite.indexOf( 'Y' ) >= 0){   System.out.print(HEADER_binfostr);SEP += "........"; vY = new Vector();}
         if(testSuite.indexOf( 'a' ) >= 0){   System.out.print(HEADER_sany);SEP += "........"; va = new Vector();}
         if(testSuite.indexOf( 'A' ) >= 0){   System.out.print(HEADER_bany);SEP += "........"; vA = new Vector();}
         if(testSuite.indexOf( 'b' ) >= 0){   System.out.print(HEADER_sanystr);SEP += "........"; vb = new Vector();}
         if(testSuite.indexOf( 'B' ) >= 0){   System.out.print(HEADER_banystr);SEP += "........"; vB = new Vector();}
         if(testSuite.indexOf( 'U' ) >= 0){   System.out.print(HEADER_ulongseq);SEP += "........"; vULS = new Vector();}
         if(testSuite.indexOf( 'R' ) >= 0){   System.out.print(HEADER_ref);SEP += "........"; vR = new Vector();}
         if(testSuite.indexOf( 'Z' ) >= 0){   System.out.print(HEADER_urefseq);SEP += "........"; vURS = new Vector();}
         if(testSuite.indexOf( 'U' ) >= 0){   System.out.print(HEADER_uinfoseq);SEP += "........"; vUIS = new Vector();}
         if(testSuite.indexOf( 'v' ) >= 0){   System.out.print(HEADER_sinfoval);SEP += "........"; viv = new Vector();}
         if(testSuite.indexOf( 'V' ) >= 0){   System.out.print(HEADER_binfoval);SEP += "........"; vIV = new Vector();}
         SEP += ".";
         System.out.println( "|" );
         print_SEP();

         // --------------------------------------------------------------------
         // Loop ---------------------------------------------------------------
         // --------------------------------------------------------------------


         while(   (testSuite.length() > 0)
               &&
               (   (loopTimes <  0        )
                     || (loopNum   <= loopTimes)
               )
         ) {
            printPassNum( loopNum );

            if(testSuite.indexOf( 'c' ) >= 0)   Echo_Small_Char_Table();
            if(testSuite.indexOf( 'C' ) >= 0)   Echo_Big_Char_Table();
            if(testSuite.indexOf( 's' ) >= 0)   Echo_Small_Short_Table();
            if(testSuite.indexOf( 'S' ) >= 0)   Echo_Big_Short_Table();
            if(testSuite.indexOf( 'l' ) >= 0)   Echo_Small_Long_Table();
            if(testSuite.indexOf( 'L' ) >= 0)   Echo_Big_Long_Table();
            if(testSuite.indexOf( 'd' ) >= 0)   Echo_Small_Double_Table();
            if(testSuite.indexOf( 'D' ) >= 0)   Echo_Big_Double_Table();
            if(testSuite.indexOf( 'i' ) >= 0)   Echo_Small_Info_Table();
            if(testSuite.indexOf( 'I' ) >= 0)   Echo_Big_Info_Table();
            if(testSuite.indexOf( 'y' ) >= 0)   Echo_Small_Info_With_String_Table();
            if(testSuite.indexOf( 'Y' ) >= 0)   Echo_Big_Info_With_String_Table();
            if(testSuite.indexOf( 'N' ) >= 0)   Echo_Very_Big_Info_Table();
            if(testSuite.indexOf( 'a' ) >= 0)   Echo_Small_Any();
            if(testSuite.indexOf( 'A' ) >= 0)   Echo_Big_Any();
            if(testSuite.indexOf( 'b' ) >= 0)   Echo_Small_Any_With_String();
            if(testSuite.indexOf( 'B' ) >= 0)   Echo_Big_Any_With_String();
            if(testSuite.indexOf( 'U' ) >= 0)   Echo_Unbounded_Long_Sequence(); 
            if(testSuite.indexOf( 'R' ) >= 0)   Echo_Ref();
            if(testSuite.indexOf( 'Z' ) >= 0)   Echo_Unbounded_Ref_Sequence();
            if(testSuite.indexOf( 'U' ) >= 0)   Echo_Unbounded_Info_Sequence();
            if(testSuite.indexOf( 'v' ) >= 0)   Echo_Small_Info_Value();
            if(testSuite.indexOf( 'V' ) >= 0)   Echo_Big_Info_Value();

            System.out.println( "|" );

            printTimeTimersLine();

            print_SEP();

            ++loopNum;
         }

         File _file = new File("/tmp/res.txt");
         FileWriter _fileWriter = new FileWriter(_file);
         if(testSuite.indexOf( 'c' ) >= 0){   _fileWriter.write(HEADER_schar + logvals(vc ));}
         if(testSuite.indexOf( 'C' ) >= 0){   _fileWriter.write(HEADER_bchar + logvals(vC )); }
         if(testSuite.indexOf( 's' ) >= 0){   _fileWriter.write(HEADER_sshort + logvals(vs ));}
         if(testSuite.indexOf( 'S' ) >= 0){   _fileWriter.write(HEADER_bshort + logvals(vS ));}
         if(testSuite.indexOf( 'l' ) >= 0){   _fileWriter.write(HEADER_slong + logvals(vl )); }
         if(testSuite.indexOf( 'L' ) >= 0){   _fileWriter.write(HEADER_blong + logvals(vL )); }
         if(testSuite.indexOf( 'd' ) >= 0){   _fileWriter.write(HEADER_sdouble + logvals(vd ));}
         if(testSuite.indexOf( 'D' ) >= 0){   _fileWriter.write(HEADER_bdouble + logvals(vD ));}
         if(testSuite.indexOf( 'i' ) >= 0){   _fileWriter.write(HEADER_sinfo + logvals(vi ));}
         if(testSuite.indexOf( 'I' ) >= 0){   _fileWriter.write(HEADER_binfo + logvals(vI ));}
         if(testSuite.indexOf( 'y' ) >= 0){   _fileWriter.write(HEADER_sinfostr + logvals(vy ));}
         if(testSuite.indexOf( 'Y' ) >= 0){   _fileWriter.write(HEADER_binfostr + logvals(vY ));}
         if(testSuite.indexOf( 'N' ) >= 0){   _fileWriter.write(HEADER_vbinfo + logvals(vV ));}
         if(testSuite.indexOf( 'a' ) >= 0){   _fileWriter.write(HEADER_sany + logvals(va ));}
         if(testSuite.indexOf( 'A' ) >= 0){   _fileWriter.write(HEADER_bany + logvals(vA ));}
         if(testSuite.indexOf( 'b' ) >= 0){   _fileWriter.write(HEADER_sanystr + logvals(vb ));}
         if(testSuite.indexOf( 'B' ) >= 0){   _fileWriter.write(HEADER_banystr + logvals(vB ));}
         if(testSuite.indexOf( 'U' ) >= 0){   _fileWriter.write(HEADER_ulongseq + logvals(vULS ));}
         if(testSuite.indexOf( 'R' ) >= 0){   _fileWriter.write(HEADER_ref + logvals(vR ));}
         if(testSuite.indexOf( 'Z' ) >= 0){   _fileWriter.write(HEADER_urefseq + logvals(vURS ));}
         if(testSuite.indexOf( 'U' ) >= 0){   _fileWriter.write(HEADER_uinfoseq + logvals(vUIS ));}
         if(testSuite.indexOf( 'v' ) >= 0){   _fileWriter.write(HEADER_sinfoval + logvals(viv ));}
         if(testSuite.indexOf( 'V' ) >= 0){   _fileWriter.write(HEADER_binfoval + logvals(vIV ));}

         _fileWriter.close();
      }
      catch(Exception e) {
         e.printStackTrace();
      }

      System.out.println(Server.NAME + ": Client exiting");
      theperf.Stop();
   }

   private static String logvals(Vector v)
   {
      String res = ":";
      for(int i = 1 /* dont consider first value */; i < v.size(); ++i){
         res = res + ((Double)v.elementAt(i)).doubleValue() + ";";
      }
      res = res + "\n";       
      return res;     
   }

   // -----------------------------------------------------------------------------------------
   // SINGLE CALL TO Small_Info_Table ---------------------------------------------------------
   // -----------------------------------------------------------------------------------------    
   private static void do_shortTest() throws Exception
   {
      System.out.println("SINGLE CALL TO Small_Info_Table:");

      /**
       * Initialize a Small_Info_Table type.
       */
      double[] Doubles    = new double[ 2];
      int[]    Longs      = new int [ 6];
      short[]  Shorts     = new short [ 8];
      char[]   Chars      = new char  [16];

      Info[] xx = new Info [2];
      for(int i = 0; i < 2; i++) {
         xx [i]= new Info(Doubles, Longs, Shorts, Chars);
      }

      /**
       * Encapsulate.
       */
      org.omg.CORBA.Any xx_any = orb.create_any();

//    o Debug.output(1,"Calling Small_Info_TableHelper:");
      Small_Info_TableHelper.insert (xx_any, xx);

      /**
       * Send.
       */
      theperf.Echo_Small_Any( xx_any );
   }

   // -----------------------------------------------------------------------------------------
   // tests.ORBInit units ------------------------------------------------------------------------------
   // -----------------------------------------------------------------------------------------
   //Allocator.traces = 0;
   //System.out.print("[" + Allocator.traces + "]");
   private static void Echo_Small_Info_Value() throws Exception
   {
      double[] Doubles    = new double[ 2] ;
      int[]     Longs   = new int [ 6] ;
      short[]     Shorts  = new short [ 8] ;
      char[]      Chars   = new char  [16] ;
      Info[]      xx      = new Info  [ 2];

      for(int i = 0; i < 2; i++)
         xx [i]= new Info(Doubles, Longs, Shorts, Chars);

      Small_Info_Value Small_Val;
      Small_Val = new Small_Info_ValueValueFactory_Impl().Create(xx);

      //Allocator.traces = 0;
      startRequestTimer();
      for(int i = 0; i < NB_MESSAGE_EACH_TEST; i++) {
         theperf.Echo_Small_Info_Value(Small_Val);
      }
      printandlogTime(viv);
      //System.out.print("[" + Allocator.traces + "]");

      if(SLEEP_MS > 0)  Thread.sleep( SLEEP_MS );
   }   

   private static void Echo_Big_Info_Value() throws Exception
   {
      double[] Doubles    = new double[ 2] ;
      int[]     Longs   = new int [ 6] ;
      short[]     Shorts  = new short [ 8] ;
      char[]      Chars   = new char  [16] ;
      Info[]      xx      = new Info  [333];

      for(int i = 0; i < 333; i++)
         xx [i]= new Info(Doubles, Longs, Shorts, Chars);
      Big_Info_Value Big_Val;
      Big_Val = new Big_Info_ValueValueFactory_Impl().Create(xx);

      //Allocator.traces = 0;
      startRequestTimer();
      for(int i = 0; i < NB_MESSAGE_EACH_TEST; i++) {
         theperf.Echo_Big_Info_Value(Big_Val);
      }

      printandlogTime(vIV);
      //System.out.print("[" + Allocator.traces + "]");

      if(SLEEP_MS > 0)  Thread.sleep( SLEEP_MS );
   }   

   private static void Echo_Unbounded_Info_Sequence() throws Exception
   {
      double[] Doubles    = new double[ 2];
      int[]    Longs      = new int [ 6];
      short[]  Shorts     = new short [ 8];
      char[]   Chars      = new char  [16];

      Info[] xx = new Info [333];

      Doubles[1]=12.3;
      Longs[2]=23;

      for(int i = 0; i < 333; i++) {
         xx [i]= new Info(Doubles, Longs, Shorts, Chars);
      }
      xx[200].Chars[8]=(char)24;

      startRequestTimer();
      for(int i = 0; i < NB_MESSAGE_EACH_TEST; i++) {
         theperf.Echo_Unbounded_Info_Sequence(xx);
         /*System.out.println("---------loop---------");*/
      }
      printandlogTime(vUIS);

      if(SLEEP_MS > 0)  Thread.sleep( SLEEP_MS );
   }

   private static void Echo_Bounded_Ref_Sequence() throws Exception
   {
      Perfs[] xx = new Perfs [180];
      for(int i = 0; i < xx.length; ++i)
      {
         xx[i] = theperf;
      }

      startRequestTimer();
      for(int i = 0; i < NB_MESSAGE_EACH_TEST; i++) {
         theperf.Echo_Bounded_Ref_Sequence(xx);
         /*System.out.println("---------loop---------");*/
      }
      printTime();

      if(SLEEP_MS > 0)  Thread.sleep( SLEEP_MS );
   }

   private static void Echo_Unbounded_Ref_Sequence() throws Exception
   {
      /*Perfs[] xx = new Perfs [33];
    for(int i = 0; i < xx.length; ++i)
    {
      xx[i] = theperf;
    }

    startRequestTimer();
    for(int i = 0; i < NB_MESSAGE_EACH_TEST; i++) {
                  theperf.Echo_Unbounded_Ref_Sequence(xx);
          }
    printTime();

    if(SLEEP_MS > 0)  Thread.sleep( SLEEP_MS );
       */
      //Perfs[] xx = new Perfs [180];
      Perfs[] xx = new Perfs [180];
      for(int i = 0; i < xx.length; ++i)
      {
         xx[i] = theperf;
      }

      startRequestTimer();
      for(int i = 0; i < NB_MESSAGE_EACH_TEST; i++) {
         theperf.Echo_Unbounded_Ref_Sequence(xx);
      }
      printandlogTime(vURS);

      if(SLEEP_MS > 0)  Thread.sleep( SLEEP_MS );
   } 

   private static void Echo_Ref() throws Exception
   {
      startRequestTimer();
      for(int i = 0; i < NB_MESSAGE_EACH_TEST; i++) {
         theperf.Echo_Ref(theperf);
         /*System.out.println("---------loop---------");*/
      }
      printandlogTime(vR);

      if(SLEEP_MS > 0)  Thread.sleep( SLEEP_MS );
   }

   private static void Echo_Bounded_Long_Sequence() throws Exception
   {
      int[] xx = new int [6000];

      startRequestTimer();
      for(int i = 0; i < NB_MESSAGE_EACH_TEST; i++) {
         theperf.Echo_Bounded_Long_Sequence(xx);
         /*System.out.println("---------loop---------");*/
      }
      printTime();

      if(SLEEP_MS > 0)  Thread.sleep( SLEEP_MS );
   }

   private static void Echo_Unbounded_Long_Sequence() throws Exception
   {
      int[] xx = new int [6000];

      startRequestTimer();
      for(int i = 0; i < NB_MESSAGE_EACH_TEST; i++) {
         theperf.Echo_Unbounded_Long_Sequence(xx);
         /*System.out.println("---------loop---------");*/
      }
      printandlogTime(vULS);

      if(SLEEP_MS > 0)  Thread.sleep( SLEEP_MS );
   }    

   private static void Echo_Small_Char_Table() throws Exception
   {
      char[] xx = new char [144];

      startRequestTimer();
      for(int i = 0; i < NB_MESSAGE_EACH_TEST; i++) {
         theperf.Echo_Small_Char_Table (xx);
         /*System.out.println("---------loop---------");*/
      }
      printandlogTime(vc);

      if(SLEEP_MS > 0)  Thread.sleep( SLEEP_MS );
   }

   private static void Echo_Big_Char_Table() throws Exception
   {
      char[] xx = new char [24016];

      startRequestTimer();
      for(int i = 0; i < NB_MESSAGE_EACH_TEST; i++)   theperf.Echo_Big_Char_Table(xx);
      printandlogTime(vC);

      if(SLEEP_MS > 0)  Thread.sleep( SLEEP_MS );
   }

   private static void Echo_Small_Short_Table() throws Exception
   {
      short[] xx = new short [72];

      startRequestTimer();
      for(int i = 0; i < NB_MESSAGE_EACH_TEST; i++)   theperf.Echo_Small_Short_Table(xx);
      printandlogTime(vs);

      if(SLEEP_MS > 0)  Thread.sleep( SLEEP_MS );
   }

   private static void Echo_Big_Short_Table() throws Exception
   {
      short[] xx = new short [12008];

      startRequestTimer();
      for(int i = 0; i < NB_MESSAGE_EACH_TEST; i++)   theperf.Echo_Big_Short_Table(xx);
      printandlogTime(vS);

      if(SLEEP_MS > 0)  Thread.sleep( SLEEP_MS );
   }

   private static void Echo_Small_Long_Table() throws Exception
   {
      int[] xx = new int [36];

      startRequestTimer();
      for(int i = 0; i < NB_MESSAGE_EACH_TEST; i++)   theperf.Echo_Small_Long_Table(xx);
      printandlogTime(vl);

      if(SLEEP_MS > 0)  Thread.sleep( SLEEP_MS );
   }

   private static void Echo_Big_Long_Table() throws Exception
   {
      int[] xx = new int [6004];

      startRequestTimer();
      for(int i = 0; i < NB_MESSAGE_EACH_TEST; i++)   theperf.Echo_Big_Long_Table(xx);
      printandlogTime(vL);

      if(SLEEP_MS > 0)  Thread.sleep( SLEEP_MS );
   }

   private static void Echo_Small_Double_Table() throws Exception
   {
      double[] xx = new double[18];

      startRequestTimer();
      for(int i = 0; i < NB_MESSAGE_EACH_TEST; i++)   theperf.Echo_Small_Double_Table(xx);
      printandlogTime(vd);

      if(SLEEP_MS > 0)  Thread.sleep( SLEEP_MS );
   }

   private static void Echo_Big_Double_Table() throws Exception
   {
      double[] xx = new double[3002];

      startRequestTimer();
      for(int i = 0; i < NB_MESSAGE_EACH_TEST; i++)   theperf.Echo_Big_Double_Table(xx);
      printandlogTime(vD);

      if(SLEEP_MS > 0)  Thread.sleep( SLEEP_MS );
   }

   private static void Echo_Small_Info_Table() throws Exception
   {
      double[] Doubles    = new double[ 2] ;
      int[]     Longs   = new int [ 6] ;
      short[]     Shorts  = new short [ 8] ;
      char[]      Chars   = new char  [16] ;
      Info[]      xx      = new Info  [ 2];

      for(int i = 0; i < 2; i++)
         xx [i]= new Info(Doubles, Longs, Shorts, Chars);

      //Allocator.traces = 0;
      startRequestTimer();
      for(int i = 0; i < NB_MESSAGE_EACH_TEST; i++)   theperf.Echo_Small_Info_Table(xx);
      printandlogTime(vi);
      //System.out.print("[" + Allocator.traces + "]");

      if(SLEEP_MS > 0)  Thread.sleep( SLEEP_MS );
   }


   private static void Echo_Small_Info_With_String_Table() throws Exception
   {
      //java.lang.String str = "0123456789";
      char[]      str =   new char  [10] ;
      double[] Doubles    = new double[ 2] ;
      int[]     Longs   = new int [ 6] ;
      short[]     Shorts  = new short [ 8] ;
      char[]      Chars   = new char  [16] ;
      Info_With_String[]      xx      = new Info_With_String  [ 2];

      for(int i = 0; i < 2; i++)
         xx [i]= new Info_With_String(str, Doubles, Longs, Shorts, Chars);

      //Allocator.traces = 0;
      startRequestTimer();
      for(int i = 0; i < NB_MESSAGE_EACH_TEST; i++)   theperf.Echo_Small_Info_With_String_Table(xx);
      printandlogTime(vy);
      //System.out.print("[" + Allocator.traces + "]");

      if(SLEEP_MS > 0)  Thread.sleep( SLEEP_MS );
   }

   private static void Echo_Very_Big_Info_Table() throws Exception
   {
      double[] Doubles    = new double[ 2] ;
      int[]    Longs      = new int [ 6] ;
      short[]  Shorts     = new short [ 8] ;
      char[]   Chars      = new char  [16] ;

      Info[] xx = new Info [50000];
      for(int i = 0; i < xx.length; i++) {
         xx [i]= new Info(Doubles, Longs, Shorts, Chars);
      }
      //Allocator.traces = 0;
      startRequestTimer();
      for(int i = 0; i < NB_MESSAGE_EACH_TEST; i++) theperf.Echo_Very_Big_Info_Table(xx);
      printandlogTime(vV);
      //System.out.print("[" + Allocator.traces + "]");

      if(SLEEP_MS > 0)  Thread.sleep( SLEEP_MS );
   }


   private static void Echo_Big_Info_Table() throws Exception
   {
      double[] Doubles    = new double[ 2] ;
      int[]    Longs      = new int [ 6] ;
      short[]  Shorts     = new short [ 8] ;
      char[]   Chars      = new char  [16] ;

      Info[] xx = new Info [333];
      for(int i = 0; i < xx.length; i++) {
         xx [i]= new Info(Doubles, Longs, Shorts, Chars);
      }
      //Allocator.traces = 0;
      startRequestTimer();
      for(int i = 0; i < NB_MESSAGE_EACH_TEST; i++)   theperf.Echo_Big_Info_Table(xx);
      printandlogTime(vI);
      //System.out.print("[" + Allocator.traces + "]");

      if(SLEEP_MS > 0)  Thread.sleep( SLEEP_MS );
   }

   private static void Echo_Big_Info_With_String_Table() throws Exception
   {
      //java.lang.String str = "0123456789";
      char[]   str    = new char  [10] ;
      double[] Doubles    = new double[ 2] ;
      int[]    Longs      = new int [ 6] ;
      short[]  Shorts     = new short [ 8] ;
      char[]   Chars      = new char  [16] ;

      Info_With_String[] xx = new Info_With_String [304];
      for(int i = 0; i < xx.length; i++) {
         xx [i]= new Info_With_String(str, Doubles, Longs, Shorts, Chars);
      }
      //Allocator.traces = 0;
      startRequestTimer();
      for(int i = 0; i < NB_MESSAGE_EACH_TEST; i++)   theperf.Echo_Big_Info_With_String_Table(xx);
      printandlogTime(vY);
      //System.out.print("[" + Allocator.traces + "]");

      if(SLEEP_MS > 0)  Thread.sleep( SLEEP_MS );
   }


   private static void Test_Any() throws Exception
   {
      int j = 321;
      int[]    Longs      = {1,2,3,4,5,6};
      double[] Doubles    = new double[ 2] ;
      short[]  Shorts     = new short [ 8] ;
      char[]   Chars      = new char  [16] ;
      Info[] xx = new Info [2];

      for(int i = 0; i < 2; i++) {
         xx [i]= new Info(Doubles, Longs, Shorts, Chars);
      }


      org.omg.CORBA.Any the_any = orb.create_any();
      the_any.insert_long( j);
      System.out.println("ANY tests.util.tests1 1:" + the_any.type().toString() + ": e=" + the_any.extract_long());
      theperf.Echo_Small_Any(the_any);

      Small_Info_TableHelper.insert (the_any, xx);
      //xx = Small_Info_TableHelper.extract (the_any);
      System.out.println("ANY tests.util.tests1 2:" + the_any.type().toString() );
      System.out.println("ANY tests.util.tests1 3:"+ xx[1]);



   }


   private static void Echo_Small_Any() throws Exception
   {
      double[] Doubles    = new double[ 2];
      int[]    Longs      = new int [ 6];
      short[]  Shorts     = new short [ 8];
      char[]   Chars      = new char  [16];

      Info[] xx = new Info [2];

      for(int i = 0; i < 2; i++) {
         xx [i]= new Info(Doubles, Longs, Shorts, Chars);
      }

      org.omg.CORBA.Any xx_any = orb.create_any();
      Small_Info_TableHelper.insert (xx_any, xx);
      //System.out.println("Created later " +  xx_any.type().toString());

      //System.out.println("DB:ANY  :" + xx_any.type().toString() );
      //Allocator.traces = 0;
      startRequestTimer();
      for(int i = 0; i < NB_MESSAGE_EACH_TEST; i++)   theperf.Echo_Small_Any (xx_any);
      printandlogTime(va);
      //System.out.print("[" + Allocator.traces + "]");

      if(SLEEP_MS > 0)  Thread.sleep( SLEEP_MS );
   }

   private static void Echo_Big_Any() throws Exception
   {
      double[] Doubles    = new double[ 2];
      int[]    Longs      = new int [ 6];
      short[]  Shorts     = new short [ 8];
      char[]   Chars      = new char  [16];

      Info[] xx = new Info [333];

      Doubles[1]=12.3;
      Longs[2]=23;

      for(int i = 0; i < 333; i++) {
         xx [i]= new Info(Doubles, Longs, Shorts, Chars);
      }
      xx[200].Chars[8]=(char)24;


      org.omg.CORBA.Any xx_any = orb.create_any();
      Big_Info_TableHelper.insert (xx_any, xx);

      /*

        System.out.println("\nDB:ANY  :" + xx_any.type().toString() );
        PerfsPackage.Info[] vals = PerfsPackage.Big_Info_TableHelper.extract(xx_any);
        System.out.println( "[[" + vals[2].Longs[2] + "][" + (int)vals[200].Chars[8] + "]]");
       */
      /*System.out.println("DB:ANY  :" + xx_any.type().toString() );
        PerfsPackage.Info[] vals2 = PerfsPackage.Big_Info_TableHelper.extract(xx_any);
        System.out.println( "[[" + vals2[2].Longs[2] + "][" + (int)vals2[2].Chars[8] + "]]");
       */
      //System.out.println( ((com.prismtech.ofj.orb.Any)xx_any).prettyPrint() );
      //Allocator.traces = 0;
      startRequestTimer();
      for(int i = 0; i < NB_MESSAGE_EACH_TEST; i++)   theperf.Echo_Big_Any (xx_any);
      printandlogTime(vA);
      //System.out.print("[" + Allocator.traces + "]");
   }

   private static void Echo_Small_Any_With_String() throws Exception
   {
      //java.lang.String str = "0123456789";
      char[]   str    = new char  [10] ;
      double[] Doubles    = new double[ 2];
      int[]    Longs      = new int [ 6];
      short[]  Shorts     = new short [ 8];
      char[]   Chars      = new char  [16];

      Info_With_String[] xx = new Info_With_String [2];

      for(int i = 0; i < 2; i++) {
         xx [i]= new Info_With_String(str, Doubles, Longs, Shorts, Chars);
      }

      org.omg.CORBA.Any xx_any = orb.create_any();
      Small_Info_With_String_TableHelper.insert (xx_any, xx);
      //System.out.println("Created later " +  xx_any.type().toString());

      //System.out.println("DB:ANY  :" + xx_any.type().toString() );
      //Allocator.traces = 0;
      startRequestTimer();
      for(int i = 0; i < NB_MESSAGE_EACH_TEST; i++)   theperf.Echo_Small_Any (xx_any);
      printandlogTime(vb);
      //System.out.print("[" + Allocator.traces + "]");

      if(SLEEP_MS > 0)  Thread.sleep( SLEEP_MS );
   }

   private static void Echo_Big_Any_With_String() throws Exception
   {
      //java.lang.String str = "0123456789";
      char[]   str    = new char  [10] ;
      double[] Doubles    = new double[ 2];
      int[]    Longs      = new int [ 6];
      short[]  Shorts     = new short [ 8];
      char[]   Chars      = new char  [16];

      Info_With_String[] xx = new Info_With_String [304];

      Doubles[1]=12.3;
      Longs[2]=23;

      for(int i = 0; i < 304; i++) {
         xx [i]= new Info_With_String(str, Doubles, Longs, Shorts, Chars);
      }
      xx[200].Chars[8]=(char)24;


      org.omg.CORBA.Any xx_any = orb.create_any();
      Big_Info_With_String_TableHelper.insert (xx_any, xx);

      /*

        System.out.println("\nDB:ANY  :" + xx_any.type().toString() );
        PerfsPackage.Info[] vals = PerfsPackage.Big_Info_TableHelper.extract(xx_any);
        System.out.println( "[[" + vals[2].Longs[2] + "][" + (int)vals[200].Chars[8] + "]]");
       */
      /*System.out.println("DB:ANY  :" + xx_any.type().toString() );
        PerfsPackage.Info[] vals2 = PerfsPackage.Big_Info_TableHelper.extract(xx_any);
        System.out.println( "[[" + vals2[2].Longs[2] + "][" + (int)vals2[2].Chars[8] + "]]");
       */
      //System.out.println( ((com.prismtech.ofj.orb.Any)xx_any).prettyPrint() );
      //Allocator.traces = 0;
      startRequestTimer();
      for(int i = 0; i < NB_MESSAGE_EACH_TEST; i++)   theperf.Echo_Big_Any (xx_any);
      printandlogTime(vB);
      //System.out.print("[" + Allocator.traces + "]");
   }

   // -----------------------------------------------------------------------------------------
   // Command line parameters -----------------------------------------------------------------
   // -----------------------------------------------------------------------------------------
   /**
    * Usage help display
    */
   private static void usage()
   {
      System.out.println("Usage: runclient.sh [short] [loop loopTimes] [-testsuite testSuite]");
      System.out.println("Or:    runclient.sh short\n");

      System.out.println(" runclient.sh with NO MORE ARGUMENT defaults to: runclient.sh loop -1 -testsuite " + testSuite + "\n");

      System.out.println(" testSuite may contain the following letters:");
      System.out.println("  C for Char");
      System.out.println("  S for Short");
      System.out.println("  D for Double");
      System.out.println("  L for Long");
      System.out.println("  I for Info");
      System.out.println("  A for Any");
      System.out.println("    ...lowercase for Small Tables, UPPERCASE for Big ones" );
      System.out.println("    ...defaults to: " + testSuite + "\n");

      System.out.println(" runclient.sh short ...SINGLE CALL TO Small_Info_Table\n");
   }

   /**
    * Parse command line parameters.
    */
   private static boolean parse_command_line(String[] args)
   {
      for(int argNum = 0; argNum < args.length; ++argNum)
      {
         if(args[ argNum ].equalsIgnoreCase( "help" )) 
         {
            usage();
            return false;
         }
         else if(args[ argNum ].equalsIgnoreCase( "SHORT" ))
         {
            shortTest = true;
         }
         else if(args[ argNum ].equalsIgnoreCase( "loop" ))
         {
            if(args.length > argNum) {
               ++argNum;

               try {
                  loopTimes = Integer.parseInt( args[ argNum ] ) + 1;
               }
               catch(NumberFormatException nfe) {
                  loopTimes = -1;
               }
            }
         }
         else if(args[ argNum ].equalsIgnoreCase( "-ORBInitRef"))
         {
            argNum++;
         }
         else if(args[ argNum ].equalsIgnoreCase( "help" )) {
            usage();
            return false;
         }
         else if(args[argNum].equals("-testsuite")) 
         {
            argNum++;
            testSuite = args[ argNum ];
         }
      }

      return true;
   }
   // -----------------------------------------------------------------------------------------
   // Mesures ---------------------------------------------------------------------------------
   // -----------------------------------------------------------------------------------------

   private static long       req_start_ms;
   private static String     REQ_build_times = "";
   private static String     REQ_trans_times = "";
   private static String     REP_build_times = "";

   // -----------------------------------------------------------------------------------------
   /**
    * Pass number display.
    * Also display previous pass time statistics with socket writes suppressed
    */
   private static void printPassNum(int n)
   {
      /*
       * Display pass number.
       */
      String s = "" + n;
      while(s.length() < 4) s = " " + s;
      System.out.print(s + " ");
   }

   // -----------------------------------------------------------------------------------------
   /**
    * Set time start.
    */
   public static void startRequestTimer()
   {


      req_start_ms = System.currentTimeMillis();
   }

   // -----------------------------------------------------------------------------------------
   /**
    * Time display.
    */
   private static void printTime()
   {
      long current_ms  = System.currentTimeMillis();

      /*
       * TOTAL: CLIENT REQUEST START -> CURRENT.
       */
      String s = concat_time(current_ms - req_start_ms);
      System.out.print( s );

      /*
       * CLIENT BUILD REQUEST: CLIENT REQUEST START -> TIMER_1 START.
       */
      //REQ_build_times += concat_time(iwe.Trace.timer_1_stamp_ms - req_start_ms);

      /*
       * ORB transmit request time.
       */
      //REQ_trans_times += concat_time(current_ms - iwe.Trace.timer_1_stamp_ms);

      /*
       * SERVER BUILD REPLY.
       */
//    REP_build_times += concat_time(iwe.Trace.timer_3_stamp_ms - iwe.Trace.timer_2_stamp_ms);
   }

   private static void printandlogTime(java.util.Vector v)
   {
      long current_ms  = System.currentTimeMillis();

      /*
       * TOTAL: CLIENT REQUEST START -> CURRENT.
       */
      String s = concat_time(current_ms - req_start_ms);
      System.out.print( s );
      v.add(new Double((current_ms - req_start_ms) / 1000.0));

   }

   /**
    * Sub-time line concat.
    */
   private static String concat_time(long time)
   {
      if(time < 0)    time = 0;

      String s = dblPFormater.format((double)time / 1000.0);

      while(s.length() < 6) s += " ";

      return  "| " + s;
   }


   // -----------------------------------------------------------------------------------------
   /**
    * Skip time display.
    */
   private static void printBlank()
   {
      String s = "|       ";

      //System.out.print( s );

      REQ_build_times  += s;
      REQ_trans_times  += s;
//    REP_build_times  += s;
   }

   // -----------------------------------------------------------------------------------------
   /**
    * Display previous pass time statistics with socket writes suppressed
    */
   private static void printTimeTimersLine()
   {
      String s;

      /**
       * TIMER # 1.
       */
      if(REQ_build_times.length() != 0) {
         s = "REQ_build";
         while(s.length() < 10)  s = s + " ";
         System.out.println(s + " " + REQ_build_times + "|");

         REQ_build_times = "";
      }
      /**
       * TIMER # 2.
       */
      if(REQ_trans_times.length() != 0) {
         s = "REQ_trans";
         while(s.length() < 10)  s = s + " ";
         System.out.println(s + " " + REQ_trans_times + "|");

         REQ_trans_times = "";
      }
      /**
       * TIMER # 3.
       */
      if(REP_build_times.length() != 0) {
         s = "REP_build";
         while(s.length() < 10)  s = s + " ";
         System.out.println(s + " " + REP_build_times + "|");

         REP_build_times = "";
      }
   }

   /**
    * Print an intermediate separation line.
    */
   private static void print_SEP()
   {
      //System.out.println(SEP);
   }


   // -----------------------------------------------------------------------------------------
   // -----------------------------------------------------------------------------------------
   // -----------------------------------------------------------------------------------------


}
