/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package tests.OrbBench;

import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;


public class ORB_SvcMessages_Sender
{

   public static void copy_long_bytes(int int_value, byte[] bytes, int dst_offset)
   {
      bytes[ dst_offset + 0 ] = (byte)((int_value >>> 24) & 0xff);
      bytes[ dst_offset + 1 ] = (byte)((int_value >>> 16) & 0xff);
      bytes[ dst_offset + 2 ] = (byte)((int_value >>>  8) & 0xff);
      bytes[ dst_offset + 3 ] = (byte)((int_value       ) & 0xff);
   }

   public static int get_long_value(byte[] bytes, int offset, boolean Swap)
   {
      if(!Swap)
      {
         return
         (((int)(bytes[offset    ] & 0xff)) << 24)
         + (((int)(bytes[offset + 1] & 0xff)) << 16)
         + (((int)(bytes[offset + 2] & 0xff)) <<  8)
         + (((int)(bytes[offset + 3] & 0xff))      );
      }
      else
      {
         return
         (((int)(bytes[offset + 3] & 0xff)) << 24)
         + (((int)(bytes[offset + 2] & 0xff)) << 16)
         + (((int)(bytes[offset + 1] & 0xff)) <<  8)
         + (((int)(bytes[offset    ] & 0xff))      );
      }
   }

   static int request_id = 15;

   public static void main(String[] args)
   {

      if(args.length != 3)
      {
         System.err.println("Usage : ORB_SvcMessages_Sender host port key");
         return;
      }

      String host = args[0];
      String port = args[1];
      int _port = Integer.parseInt(port);
      String name = args[2];

      byte[] request = new byte[200];
      byte[] null_giop_header = { 71, 73, 79, 80, 1, 2, 0, 0, 0, 0, 0, 0 };

      try
      {
         System.out.println("Connecting to " + host + ":" + _port + " ...");
         Socket _socket = new Socket( host, _port ) ;
         _socket.setTcpNoDelay(true);
         System.out.println("Connected. ");
         OutputStream ostream = _socket.getOutputStream();

         InputStream istream = _socket.getInputStream();
         byte[] reply = new byte[200];


         // Create Locate Request message
         System.arraycopy(null_giop_header, 0, request, 0, 12);
         request[7] = 3;                                         // Locate Request message
         copy_long_bytes(name.length() + 12, request, 8);        // Message size
         copy_long_bytes(request_id, request, 12);               // request id
         copy_long_bytes(0, request, 16);                        // discriminator
         copy_long_bytes(name.length(), request, 20);            // Key length
         for(int i = 0; i < name.length(); ++i)                  // Key
         {
            request[24 + i] = (byte)name.charAt(i);
         }


         // Send and receive Locate Request
         System.out.println("Sending Locate Request ... ");
         ostream.write(request, 0, name.length() + 24);
         ostream.flush();
         System.out.println("Sent. ");

         System.out.println("Waiting for Reply ... ");
         int read = 0;
         while(read != 12)
         {
            read += istream.read(reply, read, 12 - read);
         }
         boolean swap = reply[6] != 0;
         int size = get_long_value(reply, 8, swap);
         read = 0;
         while(read < size)
         {
            read += istream.read(reply, read + 12, size - read);
         }
         System.out.println("Reply Received. ");


         // Create CancelRequest message
         System.arraycopy(null_giop_header, 0, request, 0, 12);
         request[7] = 2;                                         // CancelRequest message
         copy_long_bytes(4, request, 8);                         // Message size
         copy_long_bytes(request_id, request, 12);               // request id


         // Send MessageError message
         System.out.println("Sending CancelRequest message ... ");
         ostream.write(request, 0, 16);
         ostream.flush();
         System.out.println("Sent. ");


         // Create MessageError message
         System.arraycopy(null_giop_header, 0, request, 0, 12);
         request[7] = 6;                                         // MessageError message


         // Send MessageError message
         System.out.println("Sending MessageError message ... ");
         ostream.write(request, 0, 12);
         ostream.flush();
         System.out.println("Sent. ");


         // Create CloseConnection message
         System.arraycopy(null_giop_header, 0, request, 0, 12);
         request[7] = 5;                                         // CloseConnection message


         // Send CloseConnection message
         System.out.println("Sending CloseConnection message ... ");
         ostream.write(request, 0, 12);
         ostream.flush();
         System.out.println("Sent. ");

      }
      catch(Exception e)
      {
         e.printStackTrace();
      }
   }
}
