/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <fstream>
#include <iomanip>

#include "test26/Transmission_26.stub.hpp"

int main(int argc, char * argv[])
{     
    bool send_finish = false;
 
    try 
    { 
        if (argc > 2)
        {
          send_finish = (atoi(argv[2]) == 1) ? true : false;
        }

        // Initialize orb
        CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);
                                   
        std::string server_host = std::string(argv[1]);             
        // find servant
        std::string corbaloc = std::string ("corbaloc::") 
                             + server_host + (":6789/Transmission" );     
        CORBA::Object_var obj = orb->string_to_object(corbaloc.c_str());
    
        // Narrow
        Transmission_var tm = Transmission::_narrow(obj);
        
        // Send data
        int size = 100;    
        std::cout << " size " << size << std::endl; 

        std::string hello_1("First Message : hello man !");
        int len_1 = hello_1.length();
        int len = (len_1<size)?len_1:size;
        char data1[size];
        for (int j = 0; j <= len-1; j++) data1[j] = hello_1[j];
        for (int j = len; j <= size-1; j++) data1[j] = '.';
        data1[size] = '\0';

        std::cout << std::endl << "=== test26_client : Sending 1st message (twoway)..." << hello_1 << std::endl; 
        tm->send(data1);
             
        std::string hello_2("Second Message : bye !");
        int len_2 = hello_2.length();
        len = (len_2<size)?len_2:size;
        // Send data
        size = 200;
        std::cout << " size " << size << std::endl;    
      
        std::cout << "=== test26_client : Sending a 2nd message (oneway)..." << std::endl;  
        char data2[size];
        for (int j = 0; j <= len-1; j++) data2[j] = hello_2[j];
        for (int j = len; j <= size-1; j++) data2[j] = '.';  
        data2[size] = '\0';
       
        tm->send(data2);
           
        std::cout << "=== test26_client : waiting 7 sec.." << std::endl;     
        sleep(7); 
            
        if (send_finish)
    {
           tm->terminate();
    }               
    }
    catch (const CORBA::Exception & ex) 
    {
       std::cout << "*** test26_client : CORBA exception: " << ex << std::endl;
      throw;
    }
    catch (...) 
    { 
        std::cout << "*** test26_client : exception" << std::endl;
        throw;
    }
}

