/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <fstream>
#include <unistd.h>
#include <sys/types.h>
#include "test26/Transmission_impl.hpp"
#include "recordingtools/CdmwRecordingContainerManager_impl.hpp"
#include "recordingtools/CdmwRecordingDaemonControl_impl.hpp"
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>

using namespace std;

    
int main(int argc, char * argv[])
{  
    std::ofstream om("server.pid");
    if(om) 
    { 
        om <<  getpid() ;    
        om.close();
    } 
    try 
    {
        std::cout << "server: initialising.." <<std::endl;
        
        // Initialize orb                           
        Cdmw::OrbSupport::StrategyList orb_strategies;
        orb_strategies.add_OrbThreaded();
        orb_strategies.add_PoaThreadPool(5);

        CORBA::ORB_var   orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, orb_strategies);
              
        // Get reference to Root POA.
        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
   
        PortableServer::POA_var poa = PortableServer::POA::_narrow(obj);
              
        // Activate POA manager
        PortableServer::POAManager_var mgr = poa->the_POAManager();
        mgr->activate();
      
        // Create an object
        Transmission_impl transmission_servant; 
 
        Transmission_var tm_ref =Transmission::_nil();
       
        PortableServer::ObjectId_var oid = 
        poa->activate_object(&transmission_servant);
        
        CORBA::Object_var object = poa->id_to_reference(oid.in());
        
        tm_ref = Transmission::_narrow(object.in());
       
        const std::string corbaloc_name = "Transmission" ;
        Cdmw::OrbSupport::OrbSupport::bind_object_to_corbaloc(
                        orb.in(), corbaloc_name, tm_ref.in()); 
             
        std::cout << "server: register to recording daemon.." << std::endl;
               
        std::string corbaloc_contMgr = std::string ("corbaloc::localhost:59875/RecordingDaemonContainerMgr" );
        CORBA::Object_var o_contMgr = orb->string_to_object (corbaloc_contMgr.c_str());
        CdmwRecording::ContainerManager_var contMgr = CdmwRecording::ContainerManager::_narrow (o_contMgr);
        
        if(CORBA::is_nil(contMgr))
            std::cout << "server: ## could not find " << corbaloc_contMgr << std::endl; 
        else 
        {                
            CdmwRecording::Object_List  Obj_List  ;
            Obj_List.length(1);
            Obj_List[0] = object; 
            
            contMgr->record_objects ("Transm", Obj_List); 
            std::cout << "server: record_objects - ok" << std::endl;
        }
        std::string corbaloc = std::string ("corbaloc::localhost:59875/RecordingDaemonController" );
        CORBA::Object_var o_daemon = orb->string_to_object (corbaloc.c_str());
        transmission_servant.daemon = CdmwRecording::DaemonControl::_narrow(o_daemon);
           
        if(CORBA::is_nil(transmission_servant.daemon))
            std::cout << "server: ## could not find daemon " << corbaloc << std::endl; 
        else 
        {   
            std::cout << "server: setting list of interfaces to be recorded by daemon (IDL:Transmission:1.0).." << std::endl;
            CdmwRecording::String_List  ilist  ;
            ilist.length(1);
            ilist[0] = "IDL:Transmission:1.0";
            transmission_servant.daemon->remove_all_interfaces ();
            transmission_servant.daemon->add_interfaces(ilist);
            
            std::cout << "server: setting list of operations to be recorded by daemon (send, sendto) .." << std::endl;
            CdmwRecording::String_List  olist  ;
            olist.length(2);
            olist[0] = "send";
            olist[1] = "sendto";
            transmission_servant.daemon->remove_all_operations ();
            transmission_servant.daemon->add_operations(olist); 
            std::cout << "server: ok " << std::endl;       
        }
                
        // Accept requests
        orb->run();
    }
    catch (const CORBA::Exception & ex) 
    {
        cerr << "server: CORBA exception: " << ex<< endl;
    return 1;
    }
    
    return 0;
}
