/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
#ifndef INCL_RECORDING_TEST_COALESCENCE_MANAGER_HPP 
#define INCL_RECORDING_TEST_COALESCENCE_MANAGER_HPP 

#include "Foundation/testutils/CORBATestManager.hpp"

#include "TraceAndPerf/recordinglibrary/PacketStore.hpp"
#include "TraceAndPerf/recordinglibrary/NetworkSniffer.hpp"
#include "TraceAndPerf/recordinglibrary/OperationMap.hpp"
#include "TraceAndPerf/recordinglibrary/ObjectMap.hpp"
#include "TraceAndPerf/recordinglibrary/MessageIdentifier.hpp"
#include "TraceAndPerf/recordinglibrary/RequestPairing.hpp"
#include "TraceAndPerf/recordinglibrary/RecordingPlugin.hpp"
#include "TraceAndPerf/recordinglibrary/DebugPlugin.hpp"
  
 
#include <semaphore.h>
#include <sys/stat.h>   //S_IRUSR|S_IWUSR
#include <fcntl.h>  //O_CREAT 
    
/**
*Purpose:
*<p>    Test BufferSender class
*/
class TestBufferSender : public CppUnit::TestFixture
{
    public:

        TestBufferSender();

        virtual ~TestBufferSender() throw();

        virtual  void   do_tests();

    private:

        /**
        * Purpose:
        * <p> Initialization of sniffer
        */
        int init_sniffer(Cdmw::Recording::NetworkSniffer * sniffer, int port_number);

        /**
        * Purpose:
        * <p> Initialization of MessageIdentifier
        */
        int init_message_identifier(Cdmw::Recording::MessageIdentifier * message_identifier);
        
        /**
        * Purpose:
        * <p> Stop and terminate sniffer
        */
        int terminate_sniffer(Cdmw::Recording::NetworkSniffer *sniffer);
        
        /**
        * Purpose:
        * <p> Main test function
        */
        int check();
        
        int get_recordingserver_received_pairs();
        
        void create_folder();
        
        void print_result(bool result, std::string context, 
                            int received, std::string expected);
        
        std::string output_folder;       
        
    CPPUNIT_TEST_SUITE( TestBufferSender );
        CPPUNIT_TEST( do_tests );
    CPPUNIT_TEST_SUITE_END();

}; // End class TestBufferSender 

#endif // INCL_RECORDING_TEST_COALESCENCE_MANAGER_HPP

