/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <cstdlib>
#include <cstdio>
#include <string>
#include <iostream>
#include <fstream>
#include <sstream>

#include "testbuffersender/TestBufferSender.hpp"
// #include "testbuffersender/RecordingServerEmulator.hpp"
#include "testbuffersender/Recorder_impl.hpp"

#include <Foundation/ossupport/OS.hpp>


namespace
{
    const int SUCCESS = 0;
    const int FAILURE = 1;
}
 
/**
*Implementation notes: [if no pertinent write none ]
*<p>
*Portability issues: [if no pertinent write none ]
*<p>
*/

CPPUNIT_TEST_SUITE_REGISTRATION( TestBufferSender );

TestBufferSender::TestBufferSender()
{
}
TestBufferSender::~TestBufferSender()
    throw()
{
}

void 
TestBufferSender::do_tests()
{
    check();
}

int TestBufferSender::init_sniffer(Cdmw::Recording::NetworkSniffer * sniffer, 
                                            int port_number)
{
    std::cout << "add port " << port_number <<std::endl;
    Cdmw::Recording::NetworkSniffer::Status status = sniffer->addPort(port_number);
    
    std::cout << "run" <<std::endl;
    status = sniffer->run () ; 
      
    std::cout << "start" <<std::endl;
    status = sniffer->start () ;
    
    return SUCCESS;
}

int TestBufferSender::init_message_identifier(
                            Cdmw::Recording::MessageIdentifier * message_identifier)
{
    //Run MessageIdentifier
    Cdmw::Recording::MessageIdentifier::Status status = message_identifier->run();
       
    //Start MessageIdentifier
    status = message_identifier->start();

    return SUCCESS;
}

int TestBufferSender::terminate_sniffer(
                        Cdmw::Recording::NetworkSniffer *sniffer)
{
    Cdmw::Recording::NetworkSniffer::Status status = sniffer->stop () ; 

    status = sniffer->terminate();

    return SUCCESS;
}

int TestBufferSender::get_recordingserver_received_pairs()
{
    int pairs = 0;
    int len = 0;
    string line = "";
    int count = 0;
      
    ifstream inFile;
    inFile.open("recordingserver_data");
    
    while(!inFile.is_open() && count < 3)
    {
      std::cout<<"Opening recordingserver_data ... retry ..."<<std::endl;
      Cdmw::OsSupport::OS::sleep (1000);
      count ++;
      inFile.open("recordingserver_data");
    }
    if(!inFile.is_open()) 
    {
        cout<<"## could not open recordingserver_data ## "<<endl;
        return pairs;
    }
    std::cout<<"opened recordingserver_data, waiting 3 sec ... "<<std::endl;
    
    // wait for Recording Server to flash data
    Cdmw::OsSupport::OS::sleep (3000);
      
    while(inFile >> line) 
    {
        if(line.find("buffer",0)!= std::string::npos) 
        {
           if(inFile >> line)
           {
                if(line.find("hostname",0)!= std::string::npos)  inFile >> line ;               
           } 
           if(inFile >> line)
           {
                if(line.find("num_pairs",0)!= std::string::npos)  
                {
                    inFile >> line ; 
                    pairs += atoi(line.c_str());
                } 
           } 
           if(inFile >> line)
           {
             if(line.find("length",0)!= std::string::npos)  
                {
                    inFile >> line ; 
                    len += atoi(line.c_str());
                    std::cout<< " * buffer length: "<< line << std::endl ;
                }
           }    
        }       
    }
    inFile.close();
    std::cout << " common buffer length: "<< len << std::endl;
  
    //move result file
    std::string file = output_folder + "/recordingserver_data";
    rename("recordingserver_data", file.c_str());
    
    return pairs; 
}

void TestBufferSender::create_folder()
{
    //set output folder
   output_folder = "V-TRC-021-results";
    
    //create output folder
    if (mkdir(output_folder.c_str(), 777) != 0)
    {
        std::cerr << "  Error creating output directory" << std::endl;
    }
    
    //set output file
    char host_name[100] = "";
    gethostname(host_name, sizeof(host_name));
    std::string output_file = output_folder + "/execution-" + host_name + ".output";
    
    remove(output_file.c_str());    
}

void TestBufferSender::print_result(bool result, std::string context, 
                            int received, std::string expected)
{
    //set output file
    char host_name[100] = "";
    gethostname(host_name, sizeof(host_name));
    std::string output_file = output_folder + "/execution-" + host_name + ".output";
    
    //output test result
    FILE * pFile;
    pFile = fopen (output_file.c_str(),"a+");
    
    fprintf (pFile, "%s : received %d pairs, expected %s \n", 
                    context.c_str(), received, expected.c_str());
    if (result)
        fprintf (pFile, "%s\n", "Test OK");
    else
        fprintf (pFile, "%s\n", "Test FAILED");
           
    fclose (pFile);     
}

int TestBufferSender::check()
{
    int result= SUCCESS;
    int port_number = 0;
    int max_bf_size = 0;
    int max_buffer_number = 100;
    int sent_buffer_number = 0;
    int timeout = 0; 
    std::string str;
    
    create_folder();

    try
    {
       
        //
        //Initializing
        //
        //create semaphore     
        sem_t * ssema = sem_open("/testbs_send_semaphore",O_CREAT,S_IRUSR|S_IWUSR, 0); 
        sem_init(ssema, 0, 0);

        sem_t * fsema = sem_open("/testbs_fin_semaphore",O_CREAT,S_IRUSR|S_IWUSR, 0); 
        sem_init(fsema, 0, 0);
     
        sem_t * rsema = sem_open("/testbs_ready_semaphore",O_CREAT,S_IRUSR|S_IWUSR, 0); 
        sem_init(rsema, 0, 0);

        
        // get parameters from command line   
        str=Cdmw::TestUtils::CORBATestManager::instance()->get_property("port_number"); 
        port_number = atoi(str.c_str());
        
        str=Cdmw::TestUtils::CORBATestManager::instance()->get_property("max_bf_size");
        max_bf_size = atoi(str.c_str());
        
        str=Cdmw::TestUtils::CORBATestManager::instance()->get_property("timeout");
        timeout = atoi(str.c_str());                    
        
        str=Cdmw::TestUtils::CORBATestManager::instance()->get_property("max_buffer_number");
        max_buffer_number = atoi(str.c_str());

        str=Cdmw::TestUtils::CORBATestManager::instance()->get_property("sent_buffer_number");
        sent_buffer_number = atoi(str.c_str());

        printf ("=== testbs : port_number=%d, max_bf_size=%d, timeout=%d, max_buffer_number=%d, sent_buffer_number=%d",
                port_number, max_bf_size, timeout, max_buffer_number, sent_buffer_number);
        // min values used in the present test
        // so that the max buffer number can be exceeded
        CPPUNIT_ASSERT(max_buffer_number < sent_buffer_number);
        // CPPUNIT_ASSERT(max_bf_size > 200);
        
        //Create Sniffer
        std::cout << "=== Create NetworkSniffer" <<std::endl;
        Cdmw::Recording::PacketStore store;  
        Cdmw::Recording::NetworkSniffer sniffer(store);
        
        //Create InterfaceFilter
        std::cout << "=== Create InterfaceFilter" <<std::endl;
        Cdmw::Recording::InterfaceFilter interface_filter("testbs_interfaces.txt");
        
        //Create OperationMap
        std::cout << "=== Create OperationMap" <<std::endl;
        Cdmw::Recording::OperationMap operation_map("testbs_operations.txt");
        
        //Create ObjectMap
        std::cout << "=== Create ObjectMap" <<std::endl;
        Cdmw::Recording::ObjectMap object_map(interface_filter, sniffer);
        
        //Create RequestPairing
        std::cout << "=== Create RequestPairing" <<std::endl;
        Cdmw::Recording::RequestPairing pairing(store);
        
        //Init sniffer
        std::cout << "=== Init NetworkSniffer" <<std::endl;
        init_sniffer(&sniffer, port_number);

        //
        //First Iteration
        //

        //Create MessageIdentifier
        Cdmw::Recording::MessageIdentifier message_identifier(store, object_map, 
                                            interface_filter, operation_map, pairing);
        std::cout << "=== created message identifier" << endl;
        
        //Create plugin      
        Cdmw::Recording::RecordingPlugin * recording_plugin 
                                        = new Cdmw::Recording::RecordingPlugin();      
        std::cout << "=== created plugin" << endl;
       
        recording_plugin->initialize(Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());
        std::cout << "=== testbs :plugin initialized" << endl; 
        
        std::cout << "=== testbs :setting max buffer size "<< max_bf_size 
                    << " and timeout "<< timeout << " and max buffer number "<< max_buffer_number << endl;
                
        recording_plugin->TimeOut(timeout);
        recording_plugin->MaxSize(max_bf_size);
        recording_plugin->SetMaxBufferNumber(max_buffer_number);

        //Add plugin to RequestPairing
        pairing.addPlugin(recording_plugin);
        std::cout << "=== testbs : plugin added" << endl;
        
        //get hostname
        char host_name[100] = "";
        gethostname(host_name, sizeof(host_name));  
        std::string str_url = std::string("corbaloc:iiop:1.2@")
                            + std::string(host_name) + ":12344/Recorder";  
        pairing.setDestination(Cdmw::Recording::RECORDING_PLUGIN_TYPE, (char *) str_url.c_str()); 
        std::cout << "=== testbs : set plugin destination" << endl; 
                  
        //Init MessageIdentifier
        init_message_identifier(&message_identifier);
        std::cout << "=== testbs : Message processing started" << endl;  
         
        //-------------------------------------------------------
        std::cout<<"===================== "<< std::endl;
        std::cout<<"===   Step #1/1   === "<< std::endl;
        std::cout<<"===================== "<< std::endl;
        remove("recordingserver_data");
        Cdmw::OsSupport::OS::sleep (2000);   
                                   
        // wait till the server receives a message
        // it is aproximately the same time when buffer sender receives it
        std::cout << "=== testbs : waiting for client.." << std::endl;
        sem_wait(ssema);

        // wait till timeout expired (millisec) + margin
        int wtime = timeout + 100;
        std::cout<< "=== testbs : waiting "<< wtime 
                <<" millisec .."<< std::endl;      
        Cdmw::OsSupport::OS::sleep (wtime);
        
        // Count data in recordingserver_data file
        int p = get_recordingserver_received_pairs();
        std::cout<<"=== Step #1/1 : "<<p<< "(pairs found after timeout has expired, < " << sent_buffer_number << " expected) "<<std::endl;

        std::string str;
        std::ostringstream strout;
        strout << sent_buffer_number << std::endl;
        str = strout.str();
        std::cout << "str="<<str << std::endl;
        print_result(p<sent_buffer_number, "", p, str );      

        CPPUNIT_ASSERT(p<sent_buffer_number && p );
        
        Cdmw::OsSupport::OS::sleep (2000);
        
        // Terminate message processing
        message_identifier.terminate();
        std::cout << "=== testbs : Message processing terminated" << endl;
        
        // tell client to send message )
        std::cout <<"=== testbs : posting ready semaphore (rsema) to testbs_server" << std::endl;
        sem_post(rsema);
        
        //Stop services
        pairing.finalize();
        std::cout << "=== testbs : request pairing finalized" << endl; 
                     
        result = terminate_sniffer( &sniffer);
        std::cout << "=== testbs : sniffer terminated" << endl;
        store.finalize();
        std::cout << "=== testbs : packet store finalized" << endl;
        delete recording_plugin ;
                    
        std::cout << "=== testbs : done" << endl;                 
    }    
    catch(const std::exception &e)
    {
        std::cerr << "*** TestBufferSender: exception raised : " << std::endl
                  << e.what() << std::endl;
                  throw;
    }
    catch(...)
    {
        std::cerr << "*** TestBufferSender: exception" << std::endl;
        throw;
    }       
    return result;
}




