/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <iomanip>
#include "testbuffersender/Transmission.stub.hpp"

using namespace std;

void send_messages(bool oneway, int size, int count, Transmission_var tm)
{
 
    for (int i = 1; i <= count; i++)
    {
        char data[size];
        for (int j = 0; j <= size-1; j++)
            data[j] = 0xCF;
        data[size] = '\0';

        if (oneway)  tm->sendto(data);
        else         tm->send(data);     
    } 
}
  
int main(int argc, char * argv[])
{
    bool send_finish = true;
    bool is_oneway = false;
    int size = 1;
    int count = 1;

    //
    //Check count of arguments
    //
    if (argc < 4)
    {
      cout << "*** testbs_client: too few arguments" << endl;
      cout << "*** testbs_client: too few arguments" << endl;
      cout << "!!! usage: testbs_client <server_host> is_oneway(0/1) size count send_finish(optional : 0/1)" << endl;
      throw 0;
    }

    //
    //get parameters
    //
    is_oneway = (atoi(argv[2]) == 1) ? true : false;
    size = atoi(argv[3]);
    count = atoi(argv[4]);

    if (argc > 5)
    {
      send_finish = (atoi(argv[5]) == 1) ? true : false;
    }


    cout<< "=== oneway: " << is_oneway << " size: "<< size << " count: "<< count<< " send_finish: " << send_finish << endl;    
    try 
    {
        // Initialize orb
        CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);
                          
        std::string server_host = std::string(argv[1]);             
        std::string corbaloc = std::string ("corbaloc:iiop:1.2@") 
                             + server_host + std::string(":6789/Transmission" );     
        CORBA::Object_var obj = orb->string_to_object(corbaloc.c_str());
        
        if (CORBA::is_nil(obj)) 
        {
            cout << "*** testbs_client: Nil reference" << endl;
            throw 0;
        }
    
        // Narrow
        Transmission_var tm = Transmission::_narrow(obj);
        if (CORBA::is_nil(tm)) 
        {
            cout << "*** testbs_client: not a Transmission reference" << endl;
            throw 0;
        }
       
        std::cout << "=== testbs_client: calling _non_existent ().." << std::endl;  
        std::cout << "!!! testbs_client: " << corbaloc ;
        if (tm->_non_existent())
        {
           std::cout << " non trouve" << std::endl;
        }
        else
        {
           std::cout << " trouve" << std::endl;
        }
        std::cout << "=== testbs_client: calling _is_a ().." << std::endl;  
        std::cout << "!!! testbs_client: " << corbaloc ;
        if (tm->_is_a("IDL:Transmission:1.0"))
        {
           std::cout << " is a IDL:Transmission:1.0" << std::endl;
        }
        else
        {
           std::cout << " is not a IDL:Transmission:1.0" << std::endl;
        }

        std::cout << "=== testbs_client: sending " << count <<" messages.. " << std::endl;    
        send_messages(is_oneway, size, count, tm);
        
        std::cout << "=== testbs_client: calling _non_existent ().." << std::endl;  
        std::cout << "!!! testbs_client: " << corbaloc ;
        if (tm->_non_existent())
        {
           std::cout << " non trouve" << std::endl;
        }
        else
        {
           std::cout << " trouve" << std::endl;
        }
        std::cout << "=== testbs_client: calling _is_a ().." << std::endl;  
        std::cout << "!!! testbs_client: " << corbaloc ;
        if (tm->_is_a("IDL:Transmission:1.0"))
        {
           std::cout << " is a IDL:Transmission:1.0" << std::endl;
        }
        else
        {
           std::cout << " is not a IDL:Transmission:1.0" << std::endl;
        }

        std::cout << "=== testbs_client: waiting for test.." << std::endl;  
        tm->ready(); 
                
        if (send_finish)
        {  
          std::cout << "=== testbs_client: sending finished, telling it to server ..." << endl;
          tm->finish_iteration();    

          sleep (3);
        }
        std::cout << "=== testbs_client: done" << endl;
    }
    catch (const CORBA::Exception & ex ) 
    {
        cout << "*** testbs_client: Uncaught CORBA exception" << ex << endl;
        return 1;
    }
    catch (...) 
    {
        return 1;
    }
    
    return 0;
}

