/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <Foundation/ossupport/OS.hpp>

#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"

#include "testbuffersender/TestBufferSender.hpp"
    
static void
usage(std::ostream & os, const std::string &program_name)
{
    os << "Usage:\n"
       << program_name << " -pn -DBuffer_Max_Size -DBuffer_Timeout \n"
       << "\n"
       << "Parameters:\n"
       << "-pn=<port_number>           TCP port to be spied\n"
       << "-DBuffer_Max_Size=<value>   max size of buffer (bytes)\n"
       << "-DBuffer_Timeout=<value>    timeout of coalescence manager (milisec)\n"
       << "-DBuffer_Max_Number=<value>    timeout of coalescence manager (milisec)\n"
       << "--CdmwProcessPort=<port>    port of recording server emulator\n"
       << std:: endl;
}

int main(int argc, char** argv)
{
    //
    // help argument
    //
    if (argc > 1)
    {
        if(strcmp(argv[1], "--help") == 0)
        {
            usage(std::cout, argv[0]);
            return 0;
        }
    }
     // Initialize the ORB
     
    Cdmw::OrbSupport::StrategyList orb_strategies;
    orb_strategies.add_OrbThreaded();
    orb_strategies.add_PoaThreadPool(10);

    CORBA::ORB_var  orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, orb_strategies);
          
    // Get reference to Root POA.
    CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
    PortableServer::POA_var root_poa = PortableServer::POA::_narrow(obj);
     
    Cdmw::TestUtils::CORBATestManager::instance()->init(orb.in(), root_poa.in(), argc, argv);

    //get parameters from command line and transmit them in test class
    std::string port_number = ""; 
    std::string max_bf_size = "";
    std::string timeout = "";
    std::string max_bf_nb = "";
    std::string sent_bf_nb = "";

    if( argc > 0 && argv )
    {
        port_number = Cdmw::OsSupport::OS::get_option_value(argc, argv, "-pn");
        if (port_number != "no")
        {
            Cdmw::TestUtils::CORBATestManager::instance()->set_property("port_number", port_number);
        }
        else
        {
            std::cerr << "There is no port_number argument" << std::endl;
            return 1;
        } 
        
        max_bf_size = Cdmw::OsSupport::OS::get_option_value(argc, argv, "-DBuffer_Max_Size");
        if (max_bf_size != "no")
        {
            Cdmw::TestUtils::CORBATestManager::instance()->set_property("max_bf_size", max_bf_size);
        }
        else
        { 
            std::cerr << "There is no -DBuffer_Max_Size argument" << std::endl;  
            return 1;                              
        }
        
        timeout = Cdmw::OsSupport::OS::get_option_value(argc, argv, "-DBuffer_Timeout");
        if (timeout != "no")
        {
            Cdmw::TestUtils::CORBATestManager::instance()->set_property("timeout", timeout);
        }
        else
        {
            std::cerr << "There is no -DBuffer_Timeout argument" << std::endl;
            return 1;
        }
        
        max_bf_nb = Cdmw::OsSupport::OS::get_option_value(argc, argv, "-DBuffer_Max_Number");
        if (max_bf_nb != "no")
        {
          Cdmw::TestUtils::CORBATestManager::instance()->set_property("max_buffer_number", max_bf_nb);
        }
        else
        { 
          std::cerr << "There is no -DBuffer_Max_Number argument" << std::endl;  
          return 1;                              
        }

        sent_bf_nb = Cdmw::OsSupport::OS::get_option_value(argc, argv, "-DNumber_Of_Buffers_Sent_By_Client");
        if (sent_bf_nb != "no")
        {
          Cdmw::TestUtils::CORBATestManager::instance()->set_property("sent_buffer_number", sent_bf_nb);
        }
        else
        { 
          std::cerr << "There is no -DNumber_Of_Buffers_Sent_By_Client argument" << std::endl;  
          return 1;                              
        }
    }
  
    try
      {
    Cdmw::TestUtils::CORBATestManager::instance()->run_tests();
      }
    catch (const std::exception &ex)
      {
    std::cerr << "*** Exception in run_tests : " << ex.what() << std::endl;  
      }
    Cdmw::TestUtils::CORBATestManager::instance()->summary();
    
    return 0;
}



