/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_RECORDING_TEST_BUFFER_SENDER_HPP 
#define INCL_RECORDING_TEST_BUFFER_SENDER_HPP 

#include "Foundation/testutils/CORBATestManager.hpp"
#include "testbuffersender22/Recorder_impl.hpp"

#include "TraceAndPerf/recordinglibrary/PacketStore.hpp"
#include "TraceAndPerf/recordinglibrary/NetworkSniffer.hpp"
#include "TraceAndPerf/recordinglibrary/OperationMap.hpp"
#include "TraceAndPerf/recordinglibrary/ObjectMap.hpp"
#include "TraceAndPerf/recordinglibrary/MessageIdentifier.hpp"
#include "TraceAndPerf/recordinglibrary/RequestPairing.hpp"
#include "TraceAndPerf/recordinglibrary/RecordingPlugin.hpp"
    
/**
*Purpose:
*<p>    Test BufferSender and CoalescenceManager 
*
*/
class TestBufferSender : public CppUnit::TestFixture
{

    public:

        TestBufferSender();

        virtual    ~TestBufferSender()    throw();
        
        /**
        * Purpose:
        * <p> Main test function 
        */       
        void V_TRC_022();    

 

        /**
        * Purpose:
        * <p> Initialization of sniffer 
        */
        int 
        init_sniffer(Cdmw::Recording::NetworkSniffer * sniffer, int port_number);

        /**
        * Purpose:
        * <p> Initialization of MessageIdentifier 
        */
        int 
        init_message_identifier(Cdmw::Recording::MessageIdentifier * message_identifier);
        
        /**
        * Purpose:
        * <p> Stop and terminate sniffer 
        */
        int 
        terminate_sniffer(Cdmw::Recording::NetworkSniffer *sniffer);
                
        void initialize();
        
        void finalize();
        
        void start_rec_server_emulator();
        
                
        int get_recordingserver_received_pairs();
        
        void create_folder(std::string folder);
                
        void print_result(bool result);
               
    private:  
         
        Cdmw::Recording::PacketStore store;
          
        Cdmw::Recording::NetworkSniffer * sniffer;
        Cdmw::Recording::InterfaceFilter * interface_filter;
        Cdmw::Recording::OperationMap * operation_map ;
        Cdmw::Recording::ObjectMap * object_map;   
        Cdmw::Recording::RequestPairing * pairing;
        Cdmw::Recording::MessageIdentifier * message_identifier;
        
        Cdmw::Recording::RecordingPlugin recording_plugin ;
                        
        std::string output_folder;
        
        int retry_number;
        int retry_period;
        
    CPPUNIT_TEST_SUITE( TestBufferSender );
        CPPUNIT_TEST( V_TRC_022 );
    CPPUNIT_TEST_SUITE_END();


}; // End class TestBufferSender 


#endif // INCL_RECORDING_TEST_BUFFER_SENDER_HPP

