/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "testbuffersender22/TestBufferSender.hpp"
#include "testbuffersender22/Recorder_impl.hpp"

#include <Foundation/ossupport/OS.hpp>

#include <cstdlib>
#include <string>
#include <iostream>
#include <fstream>
#include "time.h"


#include <semaphore.h>
#include <sys/stat.h>   //S_IRUSR|S_IWUSR
#include <fcntl.h>  //O_CREAT 
   

namespace
{
    const int SUCCESS = 0;
    const int FAILURE = 1;
}
 
/**
*Implementation notes: [if no pertinent write none ]
*<p>
*Portability issues: [if no pertinent write none ]
*<p>
*/

CPPUNIT_TEST_SUITE_REGISTRATION( TestBufferSender );

TestBufferSender::TestBufferSender()
{
}
TestBufferSender::~TestBufferSender() throw()
{
}


//============================================================================
// start sniffer
//============================================================================
int TestBufferSender::init_sniffer(Cdmw::Recording::NetworkSniffer * sniffer, 
                                    int port_number)
{
    Cdmw::Recording::NetworkSniffer::Status status;
    status = sniffer->addPort(port_number);   
    status = sniffer->run () ; 
    status = sniffer->start () ;
    return SUCCESS;
}
//============================================================================
// start message identifier
//============================================================================
int TestBufferSender::init_message_identifier(
                        Cdmw::Recording::MessageIdentifier * message_identifier)
{
    Cdmw::Recording::MessageIdentifier::Status status;
    status = message_identifier->run(); 
     //CPPUNIT_ASSERT(false);//works 
    status = message_identifier->start();
    //  CPPUNIT_ASSERT(false); //aborted
    return SUCCESS;
}
//===========================================================================
// terminate sniffer
//===========================================================================
int TestBufferSender::terminate_sniffer(
                        Cdmw::Recording::NetworkSniffer *sniffer)
{
    Cdmw::Recording::NetworkSniffer::Status status = sniffer->stop() ; 
    status = sniffer->terminate();
    return SUCCESS;
}

//============================================================================
// get result
//============================================================================
int TestBufferSender::get_recordingserver_received_pairs()
{
    int pairs = 0;
    string line = "";
    int count = 0;
      
    ifstream inFile;
    inFile.open("recordingserver_data");
    
    while(!inFile.is_open() && count < 3)
    {
      std::cout<<"=== testbs22 Opening recordingserver_data ... retry ..."<<std::endl;
      Cdmw::OsSupport::OS::sleep (1000);
      count ++;
      inFile.open("recordingserver_data");
    }
    if(!inFile.is_open()) 
    {
        cout<<"## could not open recordingserver_data ## "<<endl;
        return pairs;
    }
    std::cout<<"=== testbs22 opened recordingserver_data, waiting 3 sec ... "<<std::endl;
    
    //sleep(3); // wait for Recording Server to flash data 
    Cdmw::OsSupport::OS::sleep (3000);
      
    while(inFile >> line) 
    {
        if(line.find("buffer",0)!= std::string::npos) 
        {
           if(inFile >> line)
           {
                if(line.find("hostname",0)!= std::string::npos) inFile>>line ;
           } 
           if(inFile >> line)
           {
                if(line.find("num_pairs",0)!= std::string::npos)  
                {
                    inFile >> line ; 
                    pairs += atoi(line.c_str());
                }              
           }
        }       
    }
    inFile.close();
  
    //move result file
    std::string file = output_folder + "/recordingserver_data";
    rename("recordingserver_data", file.c_str());
    
    return pairs;  
}

//============================================================================
// create_folder
//============================================================================
void TestBufferSender::create_folder(std::string folder)
{
    //set output folder
   output_folder = folder;
    
    //create output folder
    if (mkdir(output_folder.c_str(), 777) != 0)
    {
        std::cerr << "*** testbs22 : Error creating output directory" << std::endl;
    }
    
       
    //set output file
    char host_name[100] = "";
    gethostname(host_name, sizeof(host_name));
    std::string output_file = output_folder + "/execution-" + host_name + ".output";
    
    remove(output_file.c_str());  
}

//============================================================================
// print_result
//============================================================================
void TestBufferSender::print_result(bool result)
{
    //set output file
    char host_name[100] = "";
    gethostname(host_name, sizeof(host_name));
    std::string output_file = output_folder + "/execution-" + host_name 
                            + ".output";
    
    //output test result
    FILE * pFile;
    pFile = fopen (output_file.c_str(),"a+");
    if (result)
        fprintf (pFile, "%s\n", "Test OK");
    else
        fprintf (pFile, "%s\n", "Test FAILED");
           
    fclose (pFile);     
}
//============================================================================
// initialize
//============================================================================
void TestBufferSender::initialize()
{   
    int port_number = 0;
            
    std::string str;
    
    
    try
    {
        str=Cdmw::TestUtils::CORBATestManager::instance()->get_property("prt");
        port_number = atoi(str.c_str());  
        str=Cdmw::TestUtils::CORBATestManager::instance()->get_property("retry_number"); 
        retry_number = atoi(str.c_str());
        str=Cdmw::TestUtils::CORBATestManager::instance()->get_property("retry_period"); 
        retry_period = atoi(str.c_str());
                   
        //Create Sniffer
        std::cout << "=== testbs22 : Create NetworkSniffer" <<std::endl;
        sniffer = new Cdmw::Recording::NetworkSniffer(store);     

        //Create InterfaceFilter (with existing file)
        std::cout << "=== testbs22 : Create InterfaceFilter" <<std::endl;
        interface_filter = new Cdmw::Recording::InterfaceFilter("testbs22_interfaces.txt");
        
        //Create OperationMap (with existing file)
        std::cout << "=== testbs22 : Create OperationMap" <<std::endl;
        operation_map = new Cdmw::Recording::OperationMap("testbs22_operations.txt");
        
        //Create ObjectMap
        std::cout << "=== testbs22 : Create ObjectMap" <<std::endl;
        object_map =new Cdmw::Recording::ObjectMap(*interface_filter, *sniffer);
        
        //Create RequestPairing
        std::cout << "=== testbs22 : Create RequestPairing" <<std::endl;
        pairing = new Cdmw::Recording::RequestPairing(store);
        
        //Init sniffer
        std::cout << "=== testbs22 : Init NetworkSniffer" <<std::endl;
        init_sniffer(sniffer, port_number);

        //Create MessageIdentifier
        message_identifier= new Cdmw::Recording::MessageIdentifier(store,
                         *object_map, *interface_filter, *operation_map, *pairing);
        std::cout << "=== testbs22 : created message identifier" << endl;
                
        CORBA::ORB_var orb ;
        orb = Cdmw::TestUtils::CORBATestManager::instance()->get_ORB();
        recording_plugin.initialize(orb);
        std::cout << "=== testbs22 : plugin initialized" << endl; 
        
        
        std::cout << "=== testbs22 : setting retry number "<< retry_number 
                    << " and retry period "<< retry_period << endl;                
        recording_plugin.SetRetryPeriod(retry_period);
        recording_plugin.SetRetryNumber(retry_number);
        
               
        //Add plugin to RequestPairing
        pairing->addPlugin(&recording_plugin);
        std::cout << "=== testbs22 : plugin added" << endl;
        
        //get hostname
        char host_name[100] = "";
        gethostname(host_name, sizeof(host_name));  
        std::string str_url = std::string("corbaloc::")
                            + std::string(host_name) + std::string(":12344/Recorder");  
        pairing->setDestination(Cdmw::Recording::RECORDING_PLUGIN_TYPE,(char *)str_url.c_str()); 
        std::cout << "=== set plugin destination to : " << str_url << endl;      
       
        //Init MessageIdentifier
        init_message_identifier(message_identifier);
        std::cout << "=== testbs22 : TestBufferSender: Message processing started" << endl;    
    }
    catch(const std::exception &e)
    {
        std::cerr << "*** testbs22 : TestBufferSender: exception raised : " << std::endl
                  << e.what() << std::endl;
                  throw;
    }
    catch(...)
    {
        std::cerr << "*** testbs22 : TestBufferSender: exception" << std::endl;
        throw;
    }       
}

//============================================================================
// finalize
//============================================================================
void TestBufferSender::finalize()
{
    std::cout << "=== testbs22 : finalizing recording plugin ..." << std::endl;      
    recording_plugin.finalize();

    std::cout << "=== testbs22 : stopping message identifier..." << std::endl;      
    message_identifier->stop();
    
    // start client to terminate message identifier
       
    message_identifier->terminate();
       
    pairing->finalize();
      
    terminate_sniffer(sniffer);
    
    store.finalize();
    std::cout << "=== testbs22 : stopped" << std::endl; 
    
    delete message_identifier ;
    delete pairing;
    delete interface_filter;
    delete operation_map;
    delete object_map;
    delete sniffer;
    
    remove("server_data");   
}



//============================================================================
// Test V-TRC-022
//============================================================================
void TestBufferSender::V_TRC_022()
{   
    std::string s = "";
    std::cout << endl;      

    create_folder("V-TRC-022-results");

    initialize();
       
    //create semaphore     
    sem_t * ssema = sem_open("/testbs22_send_semaphore",O_CREAT,S_IRUSR|S_IWUSR, 0); 
    sem_init(ssema, 0, 0);   
    sem_t * rsema = sem_open("/testbs22_ready_semaphore",O_CREAT,S_IRUSR|S_IWUSR, 0); 
    sem_init(rsema, 0, 0);   
    sem_t * fsema = sem_open("/testbs22_fin_semaphore",O_CREAT,S_IRUSR|S_IWUSR, 0); 
    sem_init(fsema, 0, 0);
    sem_t * recording_server_sema = sem_open("/recording_server_semaphore",O_CREAT,S_IRUSR|S_IWUSR, 0); 
    sem_init(recording_server_sema, 0, 0);

    // test that when Recording Server becomes available : messages are
    //  correctly received
    //
    //-------------------------------------------------------
    std::cout<<"===================== "<< std::endl;
    std::cout<<"===   Step #1/3   === "<< std::endl;
    std::cout<<"===================== "<< std::endl;
    
    //Cdmw::OsSupport::OS::sleep (1000);
                
    // wait till the server receives a message
    // it is aproximately the same time when buffer sender receives it
    std::cout << "=== testbs22 : waiting for client ..." << std::endl;
    sem_wait(ssema);
   
    // wait for Recording Server  
    std::cout << "=== testbs22 : Start Recording Server on Host1 "<< std::endl;      
    std::cout << "=== testbs22 : waiting for Recording Server ..." << std::endl;
    sem_wait(recording_server_sema);
    std::cout << "=== testbs22 : Recording Server is started ..." << std::endl;
    int waiting_time = 3000; // millisec
    std::cout << "=== testbs22 : waiting "<< waiting_time << " millisec " << std::endl;      
    Cdmw::OsSupport::OS::sleep (waiting_time);
        
    int p = get_recordingserver_received_pairs();
    std::cout<<"=== testbs22 : "<<p<< " pairs found (>0 expected) "<<std::endl;
    
    print_result(0 < p);
    try
      {
    CPPUNIT_ASSERT(0 < p);
      }
    catch (...){};

    //=========================================================================
    // test that when Recording Server is stopped no message received
    // 2 :
    std::cout<<"===================== "<< std::endl;
    std::cout<<"===   Step #2/3   === "<< std::endl;
    std::cout<<"===================== "<< std::endl;

    
    // wait for Recording Server  
    std::cout << "=== testbs22 : Stop Recording Server on Host1 "<< std::endl;      
    std::cout << "=== testbs22 : Enter RC when Recording Server is stopped ..." << std::endl;
    std::cin.get();
    std::cout << "=== testbs22 : Recording Server is stopped ..." << std::endl;
    std::cout << "=== testbs22 : Waking Client and Server ..." << std::endl;
    sem_post(rsema);
    
    waiting_time = 2000; // millisec
    std::cout << "=== testbs22 : waiting "<< waiting_time << " millisec " << std::endl;      
    Cdmw::OsSupport::OS::sleep (waiting_time);

    p = get_recordingserver_received_pairs();
    std::cout<<"=== testbs22 : "<<p<< " pairs found (0 expected) "<<std::endl;

    print_result(0 == p);
    CPPUNIT_ASSERT(0 == p);


    // test that when Recording Server becomes available : messages are
    //  correctly received
    //
    //-------------------------------------------------------
    std::cout<<"===================== "<< std::endl;
    std::cout<<"===   Step #3/3   === "<< std::endl;
    std::cout<<"===================== "<< std::endl;

    //Cdmw::OsSupport::OS::sleep (1000);

    // wait till the server receives a message
    // it is aproximately the same time when buffer sender receives it
    std::cout << "=== testbs22 : waiting for client ..." << std::endl;
    sem_wait(ssema);

    // wait for Recording Server  
    std::cout << "=== testbs22 : Start Recording Server on Host1 "<< std::endl;      
    std::cout << "=== testbs22 : waiting for Recording Server ..." << std::endl;
    sem_wait(recording_server_sema);
    std::cout << "=== testbs22 : Recording Server is started ..." << std::endl;
    waiting_time = 2000; // millisec
    std::cout << "=== testbs22 : waiting "<< waiting_time << " millisec " << std::endl;      
    Cdmw::OsSupport::OS::sleep (waiting_time);

    p = get_recordingserver_received_pairs();
    std::cout<<"=== testbs22 : "<<p<< " pairs found (>0 expected) "<<std::endl;

    print_result(0 < p);
    CPPUNIT_ASSERT(0 < p);
    
    Cdmw::OsSupport::OS::sleep (1000);
    
    std::cout << "=== testbs22 : Waking Client and Server ..." << std::endl;
    sem_post(rsema);
    
    // ==================================================================
    std::cout << "=== testbs22 : Waiting Client and Server for end of test..." << std::endl;
    sem_wait(fsema);

    std::cout << "=== testbs22 : terminating message identifier...*" << std::endl;       
    message_identifier->terminate();
      
    std::cout << "=== testbs22 : finalizing request pairing.." << std::endl;      
    pairing->finalize();
      
    std::cout << "=== testbs22 : stopping sniffer.." << std::endl;  
    terminate_sniffer(sniffer);
    
    store.finalize();
    std::cout << "=== testbs22 : stopped" << std::endl; 
    
    delete message_identifier ;
    delete pairing;
    delete interface_filter;
    delete operation_map;
    delete object_map;
    delete sniffer;
    
    //delete rse;
    remove("server_data");   
    
    std::cout<<"=== testbs22 : done"<<std::endl;    
}

