/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <iomanip>
#include "testbuffersender22/Transmission.stub.hpp"
#include <Foundation/ossupport/OS.hpp>

#include "testbuffersender22/Recorder_impl.hpp"

using namespace std;

void send_messages(bool oneway, int size, int count, int delay_milisec, 
                    Transmission_var tm)
{
   cout<< "=== testbs22_client.send_messages : oneway= " << oneway << " size= "<< size 
       << " count= "<< count<< " delay= " << delay_milisec << " millisec" <<endl;  
    
    for (int i = 1; i <= count; i++)
    {
        char data[size];
        for (int j = 0; j <= size-1; j++)
            data[j] = 0xCF;
        data[size] = '\0';

        if (oneway) tm->sendto(data);
        else        tm->send(data);
      
        Cdmw::OsSupport::OS::sleep (delay_milisec);
    }          
}  
int main(int argc, char * argv[])
{
 
    //Check count of arguments
    //
    if (argc < 1)
    {
        cout << "*** testbs22_client: too few arguments" << endl;
        throw 0;
    }
    try 
    {
        // Initialize orb
        CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);
                  

        std::string server_host = std::string(argv[1]);             
        std::string corbaloc = std::string ("corbaloc::") 
                             + server_host + std::string(":6789/Transmission" );     
        CORBA::Object_var obj = orb->string_to_object(corbaloc.c_str());
       
        
        if (CORBA::is_nil(obj)) 
        {
            cout << "*** testbs22_client: Nil reference" << endl;
            throw 0;
        }
    
        // Narrow
        Transmission_var tm = Transmission::_narrow(obj);
        if (CORBA::is_nil(tm)) 
        {
            cout << "*** testbs22_client: not a Transmission reference" << endl;
            throw 0;
        }

/*       
        // Recording Server
        corbaloc = std::string ("corbaloc::") 
                               + server_host + std::string(":12344/Recorder" );     
        CORBA::Object_var RS_obj = orb->string_to_object(corbaloc.c_str());


        if (CORBA::is_nil(RS_obj)) 
        {
          cout << "*** testbs22_client: Recording Server : Nil reference" << endl;
          //throw 0;
        }

        // Narrow
        CdmwRecording::Recorder_var RS = CdmwRecording::Recorder::_unchecked_narrow(RS_obj.in());
        if (CORBA::is_nil(RS)) 
        {
          cout << "*** testbs22_client: not a Recording Server reference" << endl;
          //throw 0;
        }
        else
        {
          std::cout << "=== testbs22_client: telling Recording Server to die..." << std::endl; 
          ::CdmwRecording::GroupHeader  header;
          header.hostname = "DIE!";
          ::CdmwRecording::GIOP_Buffers data;
          data.length(0);
          RS -> record (header, data);
        }
*/        
        std::cout << "=== testbs22_client: 1 sending a messages..." << std::endl; 
        send_messages(0, 8000, 2, 20, tm);
        
        std::cout << "=== testbs22_client: waiting for the test.." << std::endl; 
        tm->ready();
        
        std::cout << "=== testbs22_client: 2 sending messages..." << std::endl; 
        send_messages(1, 8000, 10, 20, tm);
        
        std::cout << "=== testbs22_client: waiting for the test.." << std::endl; 
        tm->ready();
                    
        tm->finish_iteration();
        
        std::cout << "=== testbs22_client: finished" << std::endl;         
              
    }
    catch (const CORBA::Exception &) 
    {
        cout << "*** testbs22_client: Uncaught CORBA exception" << endl;
        return 1;
    }
    catch (...) 
    {
        return 1;
    }
    
    return 0;
}

