/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/ossupport/OS.hpp>
#include "testbuffersender22/TestBufferSender.hpp"

    
static void
usage(std::ostream & os, const std::string &program_name)
{
    os << "Usage:\n"
       << program_name << " -pn -DRecording_Retry_Period -DRecording_Retry_Number    \n"
       << "\n"
       << "Parameters:\n"
       << "-pn=<port_number>                 TCP port to be spied\n"
       << "-DRecording_Retry_Period=<value>  retry period (milisec) of buffer sender\n"
       << "-DRecording_Retry_Number=<value>  number of retries in buffer sender \n"
       << std:: endl;
}

int main(int argc, char** argv)
{
    //
    // help argument
    //
    if (argc > 1)
    {
        if(strcmp(argv[1], "--help") == 0)
        {
            usage(std::cout, argv[0]);
            return 0;
        }
    }
    
    // Initialize the ORB
     
    Cdmw::OrbSupport::StrategyList orb_strategies;
    orb_strategies.add_OrbThreaded();
    orb_strategies.add_PoaThreadPool(10);

    CORBA::ORB_var orb;
    orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, 
                                                 orb_strategies);
  
    // Get reference to Root POA.
    CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
    PortableServer::POA_var root_poa = PortableServer::POA::_narrow(obj);
     
    Cdmw::TestUtils::CORBATestManager::instance()->init(orb.in(), 
                                                        root_poa.in(),
                                                        argc,argv);
    
    //get parameters from command line and transmit them in test class
    std::string port_number = ""; 
    std::string retry_period = "";
    std::string retry_number = "";
    if( argc > 0 && argv )
    {
        port_number = Cdmw::OsSupport::OS::get_option_value(argc,argv,"-pn");
        if (port_number != "no")
        {
            Cdmw::TestUtils::CORBATestManager::instance()->set_property("prt",
                                                                port_number);
        }
        else
        {
            std::cerr << "There is no port_number argument" << std::endl;
            return 1;
        }
        retry_period = Cdmw::OsSupport::OS::get_option_value(argc, argv, "-DRecording_Retry_Period");
        if (retry_period != "no")
        {
            Cdmw::TestUtils::CORBATestManager::instance()->set_property("retry_period", retry_period);
        }
        else
        { 
            std::cerr << "There is no -DRecording_Retry_Period argument" << std::endl;  
            return 1;                              
        }
        retry_number = Cdmw::OsSupport::OS::get_option_value(argc, argv, "-DRecording_Retry_Number");
        if (retry_number != "no")
        {
            Cdmw::TestUtils::CORBATestManager::instance()->set_property("retry_number", retry_number);
        }
        else
        { 
            std::cerr << "There is no -DRecording_Retry_Number argument" << std::endl;  
            return 1;                              
        }
        
    }
  
    Cdmw::TestUtils::CORBATestManager::instance()->run_tests();
    Cdmw::TestUtils::CORBATestManager::instance()->summary();
    
    return 0;
}



