/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <fstream>
#include <unistd.h>
#include <sys/types.h>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include "testbuffersender22/Transmission_impl.hpp"

using namespace std;

    
int main(int argc, char * argv[])
{    
    std::ofstream om("server.pid");
    if(om) 
    { 
        om <<  getpid() ;    
        om.close();
    } 
    try 
    {
        // Initialize orb                      
        Cdmw::OrbSupport::StrategyList orb_strategies;
        orb_strategies.add_OrbThreaded();
        orb_strategies.add_PoaThreadPool(5);

        CORBA::ORB_var orb = 
            Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, orb_strategies);      
        // Get reference to Root POA.
        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var poa = PortableServer::POA::_narrow(obj);
        // Activate POA manager
        PortableServer::POAManager_var mgr = poa->the_POAManager();
        mgr->activate();
        // Create an object
        Transmission_impl transmission_servant;
                
        Transmission_var tm_ref =Transmission::_nil();
       
        PortableServer::ObjectId_var oid = 
            poa->activate_object(&transmission_servant);
        
        CORBA::Object_var object = poa->id_to_reference(oid.in());
        
        tm_ref = Transmission::_narrow(object.in());
      
        Cdmw::OrbSupport::OrbSupport::bind_object_to_corbaloc(
                        orb.in(), "Transmission", tm_ref.in()); 
       
        // Accept requests
        orb->run();
    }
    catch (const CORBA::Exception &) 
    {
        cerr << "CORBA server: Uncaught CORBA exception" << endl;
    return 1;
    }
    
    return 0;
}
