/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <iomanip>
#include "testcoalescencemanager/Transmission.stub.hpp"

using namespace std;

void send_messages(bool oneway, int size, int count, Transmission_var tm)
{
 
    for (int i = 1; i <= count; i++)
    {
        char data[size];
        for (int j = 0; j <= size-1; j++)
            data[j] = 0xCF;
        data[size] = '\0';

        if (oneway)  tm->sendto(data);
        else         tm->send(data);     
    } 
}
  
int main(int argc, char * argv[])
{
    // Check count of arguments 
    if (argc < 1)
    {
        cout << "CORBA client: too few arguments" << endl;
        cout << "usage: <host> - name of the host where the server is running" << endl;
        throw 0;
    }
  
    try 
    {
        // Initialize orb
        CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);
                          
        std::string server_host = std::string(argv[1]);             
        std::string corbaloc = std::string ("corbaloc:iiop:1.2@") 
                             + server_host + std::string(":6789/Transmission" );     
        CORBA::Object_var obj = orb->string_to_object(corbaloc.c_str());
        
        if (CORBA::is_nil(obj)) 
        {
            cout << "testcm_client: Nil reference" << endl;
            throw 0;
        }
    
        // Narrow
        Transmission_var tm = Transmission::_narrow(obj);
        if (CORBA::is_nil(tm)) 
        {
            cout << "testcm_client: not a Transmission reference" << endl;
            throw 0;
        }
       
        std::cout << "testcm_client: 1 sending a message.. " << std::endl;    
        send_messages(false, 1, 1,tm);
        
        std::cout << "testcm_client: waiting for test.." << std::endl;  
        tm->ready(); 
        
        std::cout << "testcm_client: 2 sending a message.. " << std::endl;    
        send_messages(false, 1, 1,tm);
        
        std::cout << "testcm_client: waiting for test.." << std::endl; 
        tm->ready(); 
        
        std::cout << "testcm_client: 3 sending 8 messages (1025 bytes each) .. " << std::endl;    
        send_messages(false, 1025, 8,tm);
        
        tm->finish_iteration();     
        
        std::cout << "testcm_client: sending finished, waiting for test.." << endl;            
        tm->ready(); 
        
        std::cout << "testcm_client: 4 sending a message (100 bytes) .." << std::endl;    
        send_messages(false, 100, 1,tm);
        
        tm->finish_iteration();
        
        std::cout << "testcm_client: sending finished, waiting for test.." << endl;    
        tm->ready(); 
        
        std::cout << "testcm_client: 5 sending a message (100 bytes) .. " << std::endl;   
        send_messages(false, 100, 1,tm);
        
        tm->finish_iteration();  
        std::cout << "testcm_client: ok, waiting for test.." << endl; 
       
        tm->ready(); 
        send_messages(false, 1, 1,tm);
        tm->terminate();
        std::cout << "testcm_client: finished" << endl; 
    }
    catch (const CORBA::Exception & ex ) 
    {
        cout << "CORBA client: Uncaught CORBA exception" << ex << endl;
        return 1;
    }
    catch (...) 
    {
        return 1;
    }
    
    return 0;
}

