/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_CDMWRECORDINGRECORDER_INTERFACE_IMPL_HPP 
#define INCL_CDMWRECORDINGRECORDER_INTERFACE_IMPL_HPP 

#include <stdio.h>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <Foundation/osthreads/ThreadHandle.hpp>
#include <Foundation/osthreads/Mutex.hpp>

#include "testftrecorder/MyProcessBehaviour.hpp"
#include "recordingidl/CdmwRecordingRecorder.skel.hpp"


#include <string>

namespace CdmwRecording
{

    class Recorder_impl;

  class test_ft_recorder_activate : public CppUnit::TestFixture
  {
    
  public:
    test_ft_recorder_activate()
    {
    };
    

    ~test_ft_recorder_activate()
    {
    };

    void do_tests();
    
    CPPUNIT_TEST_SUITE( test_ft_recorder_activate );
    CPPUNIT_TEST( do_tests );
    CPPUNIT_TEST_SUITE_END();

  };
       
    
  class TestThread : public Cdmw::OsSupport::Thread 
  {
  public:
    TestThread()
    {};
        
  protected:
    void run() throw()
    {
      Cdmw::TestUtils::ThreadedTestRunner runner(Cdmw::TestUtils::CORBATestManager::instance()->get_controller());
      runner.addTest( CppUnit::TestFactoryRegistry::getRegistry("test_ft_recorder_activate").makeTest() );
      runner.start();
      runner.join();
    };
        
  private:
    TestThread(const TestThread&);
    TestThread& operator = (const TestThread&);
  };

    class Recorder_impl :public virtual POA_CdmwRecording::Recorder,
                         virtual public PortableServer::RefCountServantBase
    {

    public:
        
        Recorder_impl (bool first);
        
        virtual ~Recorder_impl()
            throw();
        
        virtual void record(const ::CdmwRecording::GroupHeader& header,
                       const GIOP_Buffers& data) throw(CORBA::SystemException);

      // boolean to know if it is the first server
      static bool m_first;

      // number of request
      static int m_nb_request;

    private:

      FILE *trace_file;

      TestThread m_testThread;

      Cdmw::OsSupport::Mutex m_mutex;

    }; // End class Recorder_impl

}; // End namespace CdmwRecording

#endif // INCL_CDMWRECORDINGRECORDER_INTERFACE_IMPL_HPP 

