/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_CLIENT_PROCESS_BEHAVIOUR_HPP 
#define INCL_CLIENT_PROCESS_BEHAVIOUR_HPP 

#include <SystemMngt/platforminterface/PlatformInterface.hpp>
#include <ConfAndPlug/cdmwinit/ProcessControl.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>
#include <Repository/idllib/CdmwNamingAndRepository.stub.hpp>
#include <testftrecorder/BasicDemo.skel.hpp>
#include <string>
#include <sstream>
#include <iostream>
#include <fstream>
#include "testftrecorder/Client.hpp"
#include "testftrecorder/MyProcessBehaviour.hpp"

/**
* Purpose:
* <p>
* The abstract process behaviour.
*/
class ClientProcessBehaviour : virtual public MyProcessBehaviour
{

private:
  int m_nb_message;

public:

    /**
    * Purpose:
    * <p> The constructor.
    */ 
    ClientProcessBehaviour  (CORBA::ORB_ptr orb, PortableServer::POA_ptr rootPOA, int nb_message):
      MyProcessBehaviour (orb, rootPOA),
      m_nb_message(nb_message)
    {
    }


    /**
    * Purpose:
    * <p> The destructor.
    */ 
    virtual ~ClientProcessBehaviour() throw ()
    {
     
    }
    /**
    * Purpose:
    * <p>
    * the behaviour for the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/run:1.0
    * operation
    */
    virtual void on_initialise(const CdmwPlatformMngtBase::StartupKind& startup_kind)
        throw( CORBA::SystemException)
    {
      std::cout << "Client: on_initialise" << std::endl;
    }

    /**
    * Purpose:
    * <p>
    * the behaviour for the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/run:1.0
    * operation
    */
    virtual void on_run()
        throw( CORBA::SystemException,
               CdmwPlatformMngt::ProcessDelegate::NotReadyToRun)
    {
      std::cout << "Client: starting of the test" << std::endl;
      ::BasicDemo::Client::run_client(m_nb_message);
    }

    

};


#endif // INCL_CLIENT_PROCESS_BEHAVIOUR_HPP

