/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef _GROUP_CREATOR_PROCESS_BEHAVIOUR_HPP 
#define _GROUP_CREATOR_PROCESS_BEHAVIOUR_HPP 

// Standard Files 
#include <string>

// Cdmw Files
#include <Foundation/common/Options.hpp>
#include <Foundation/common/String.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <FaultTolerance/ftcommon/FactoryInfos.hpp>
#include <ConfAndPlug/cdmwinit/ProcessControl.hpp>

#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>
#include <Repository/idllib/CdmwNamingAndRepository.stub.hpp>
#include "FaultTolerance/idllib/FT.stub.hpp"

class GroupCreatorProcessBehaviour : public Cdmw::CdmwInit::ProcessControl
{

  public:

    // Constructor
    GroupCreatorProcessBehaviour(CORBA::ORB_ptr orb, std::string host1, std::string host2)
        : m_host1(host1),
          m_host2(host2)
    {
      m_orb = CORBA::ORB::_duplicate( orb );
    }

    // Destructor
    virtual ~GroupCreatorProcessBehaviour() throw ()
    {
    }


    virtual CORBA::ULong nb_steps()
        throw( CORBA::SystemException )
    {
      return 1;
    }


    virtual void on_initialise(
                               const CdmwPlatformMngtBase::StartupKind& startup_kind )
        throw( CORBA::SystemException )
    {

      Cdmw::OsSupport::OS::sleep(3000);

      std::cout << "\n -------- GroupCreator on_initialise --------" << std::endl;

      // Get the Replication Manager
      CORBA::Object_var replication_mgr_obj
          = m_orb->resolve_initial_references("ReplicationManager");

      m_replication_mgr = FT::ReplicationManager::_narrow(
        replication_mgr_obj.in() );

      // Create the object group properties
      std::cout << "  -> Create group" << std::endl;
      const char* rid = "IDL:thalesgroup.com/CdmwRecording/Recorder:1.0";

      ::FT::Properties prop;
      prop.length(2);
      prop[0].nam.length( 1 );
      prop[0].nam[0].id = "org.omg.ft.MinimumNumberReplicas";
      prop[0].val <<= (CORBA::UShort) 1;

      ::FT::Locations locs(2);
      locs.length(2);
      locs[0].length(3);
      locs[0][0].id = m_host1.c_str();
      locs[0][0].kind = "hostname";
      locs[0][1].id = "APPL1";
      locs[0][1].kind = "applicationname";
      locs[0][2].id = "P11";
      locs[0][2].kind = "processname";

      locs[1].length(3);
      locs[1][0].id = m_host2.c_str();
      locs[1][0].kind = "hostname";
      locs[1][1].id = "APPL1";
      locs[1][1].kind = "applicationname";
      locs[1][2].id = "P21";
      locs[1][2].kind = "processname";

      const CORBA::ULong factory_infos_len = locs.length();
      ::FT::FactoryInfos factoryInfos;
      factoryInfos.length(factory_infos_len);
      for (CORBA::ULong i = 0; i < factory_infos_len; ++i) 
      {
        factoryInfos[i].the_factory = ::FT::GenericFactory::_nil();
        factoryInfos[i].the_location = locs[i];
        ::FT::Criteria factoryCrit;        
        factoryCrit.length(0);
        factoryInfos[i].the_criteria = factoryCrit;
      }

      prop[1].nam.length(1);
      prop[1].nam[0].id="org.omg.ft.Factories";
      prop[1].val <<= factoryInfos;

      ::FT::Name OGDName;
      OGDName.length(1);
      OGDName[0].id=CORBA::string_dup("TestGroup");
      

      ::FT::Criteria ft_criteria;
      ft_criteria.length(2);
      ft_criteria[0].nam.length(1);
      ft_criteria[0].nam[0].id="com.thalesgroup.cdmw.ft.Name";
      ft_criteria[0].val <<= OGDName;

      ft_criteria[1].nam.length(1);
      ft_criteria[1].nam[0].id="org.omg.ft.FTProperties";
      ft_criteria[1].val <<= prop;


      ::FT::GenericFactory::FactoryCreationId_var factory_creation_id;
      ::FT::ObjectGroup_var obj_group;

      // create the object group 
      try
      {
        obj_group =  m_replication_mgr->create_object(
          rid, ft_criteria, factory_creation_id.out() );
      }
      catch( CORBA::Exception& e )
      {
        std::cerr << e._name() << std::endl;
        throw;
      }

      m_group = CdmwRecording::Recorder::_narrow( obj_group.in() );

      // get NamingInterface to recorder_servers (for reading)
      Cdmw::CommonSvcs::Naming::NamingInterface ni =
        Cdmw::NamingAndRepository::RepositoryInterface::get_domain_naming_interface ("dom1/dom2/dom3");

      std::string full_name = "P11Recorder";

      CORBA::Object_var obj;

      try
      {
        obj = ni.resolve (full_name);
        CPPUNIT_ASSERT(true);
      }
      catch(const CosNaming::NamingContext::NotFound& e)
      {
        std::cerr<< e << std::endl;
        CPPUNIT_ASSERT(false);
      }
      catch(const CosNaming::NamingContext::CannotProceed& e)
      {
        std::cerr << e << std::endl;
        CPPUNIT_ASSERT(false);
      }
      catch(const CosNaming::NamingContext::InvalidName& e)
      {
        std::cerr << e << std::endl;
        CPPUNIT_ASSERT(false);
      }
      catch (const CORBA::SystemException& e)
      {
        std::cerr << e << std::endl;
        CPPUNIT_ASSERT(false);
      }

      CdmwRecording::Recorder_var recorderP11 = CdmwRecording::Recorder::_narrow(obj.in());
      CPPUNIT_ASSERT( ! CORBA::is_nil(recorderP11.in()) );

      full_name = "P21Recorder";

      try
      {
        obj = ni.resolve (full_name);
        CPPUNIT_ASSERT(true);
      }
      catch(const CosNaming::NamingContext::NotFound& e)
      {
        std::cerr << e << std::endl;
        CPPUNIT_ASSERT(false);
      }
      catch(const CosNaming::NamingContext::CannotProceed& e)
      {
        std::cerr << e << std::endl;
        CPPUNIT_ASSERT(false);
      }
      catch(const CosNaming::NamingContext::InvalidName& e)
      {
        std::cerr << e << std::endl;
        CPPUNIT_ASSERT(false);
      }
      catch (const CORBA::SystemException& e)
      {
        std::cerr << e << std::endl;
        CPPUNIT_ASSERT(false);
      }

      CdmwRecording::Recorder_var recorderP21 = CdmwRecording::Recorder::_narrow(obj.in());
      CPPUNIT_ASSERT( ! CORBA::is_nil(recorderP21.in()) );

      ::FT::Location loc;
      loc.length(3);
      loc[0].id = m_host1.c_str();
      loc[0].kind = "hostname";
      loc[1].id = "APPL1";
      loc[1].kind = "applicationname";        
      loc[2].id = "P11";
      loc[2].kind = "processname";

      try {
        obj_group = m_replication_mgr->add_member(m_group.in(), loc, recorderP11.in());
      }
      catch (const ::FT::ObjectGroupNotFound&)
      {
        std::cerr << __FILE__ << "  " << __LINE__ << std::endl;
        std::cerr << "add_member P11 => FT::ObjectGroupNotFound raised!" << std::endl;
        return;
      }
      catch (const ::FT::MemberAlreadyPresent&)
      {
        std::cerr << __FILE__ << "  " << __LINE__ << std::endl;
        std::cerr << "add_member P11 => FT::MemberAlreadyPresent raised!" << std::endl;
        return;
      }
      catch (const ::FT::ObjectNotAdded&)
      {
        std::cerr << __FILE__ << "  " << __LINE__ << std::endl;
        std::cerr << "add_member P11 => FT::ObjectNotAdded raised!" << std::endl;
        return;
      }

      loc[0].id = m_host2.c_str();
      loc[0].kind = "hostname";
      loc[1].id = "APPL1";
      loc[1].kind = "applicationname";        
      loc[2].id = "P21";
      loc[2].kind = "processname";

      try {
        obj_group = m_replication_mgr->add_member(m_group.in(), loc, recorderP21.in());
      }
      catch (const ::FT::ObjectGroupNotFound&)
      {
        std::cerr << __FILE__ << "  " << __LINE__ << std::endl;
        std::cerr << "add_member P21 => FT::ObjectGroupNotFound raised!" << std::endl;
        return;
      }
      catch (const ::FT::MemberAlreadyPresent&)
      {
        std::cerr << __FILE__ << "  " << __LINE__ << std::endl;
        std::cerr << "add_member P21 => FT::MemberAlreadyPresent raised!" << std::endl;
        return;
      }
      catch (const ::FT::ObjectNotAdded&)
      {
        std::cerr << __FILE__ << "  " << __LINE__ << std::endl;
        std::cerr << "add_member P21 => FT::ObjectNotAdded raised!" << std::endl;
        return;
      }

      // bind to a corbaloc name
      Cdmw::OrbSupport::OrbSupport::bind_object_to_corbaloc(m_orb, "testgroup", m_group);

      std::cout << " -------- GroupCreator on_initialise END --------" << std::endl;
    }


    virtual void on_run()
        throw( CdmwPlatformMngt::ProcessDelegate::NotReadyToRun,
               CORBA::SystemException )
    {
      std::cout << "\n -------- GroupCreator on_run --------" << std::endl;
   }


    virtual void on_stop()
        throw( CORBA::SystemException )
    {
      std::cout << "\n -------- GroupCreator on_stop --------" << std::endl;
      // nothing to do : shutdown done in ProcessControl
    }


  private:

    /**
     * The ORB.
     */
    CORBA::ORB_var m_orb;

    /**
     * The object group.
     */
    CdmwRecording::Recorder_var m_group;

    /**
     * The replication manager.
     */
    FT::ReplicationManager_var m_replication_mgr;

    std::string m_host1;
    std::string m_host2;

};

#endif // INCL_DEMO_FT_GROUP_CREATOR_PROCESS_BEHAVIOUR_HPP

