/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_SERVER_PROCESS_BEHAVIOUR_HPP 
#define INCL_SERVER_PROCESS_BEHAVIOUR_HPP 

#include <SystemMngt/platforminterface/PlatformInterface.hpp>
#include <ConfAndPlug/cdmwinit/ProcessControl.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>
#include <Repository/idllib/CdmwNamingAndRepository.stub.hpp>
#include <testftrecorder/BasicDemo.skel.hpp>
#include <string>
#include <sstream>
#include <iostream>
#include <fstream>
#include "testftrecorder/MyProcessBehaviour.hpp"
#include "testftrecorder/Server.hpp"

/**
* Purpose:
* <p>
* The abstract process behaviour.
*/
class ServerDisplayProcessBehaviour : virtual public ::MyProcessBehaviour
{

public:

    /**
    * Purpose:
    * <p> The constructor.
    */ 
    ServerDisplayProcessBehaviour  (CORBA::ORB_ptr orb, PortableServer::POA_ptr rootPOA):
      MyProcessBehaviour (orb, rootPOA)
    {
    }


    /**
    * Purpose:
    * <p> The destructor.
    */ 
    virtual ~ServerDisplayProcessBehaviour() throw ()
    {
    }

    /**
    * Purpose:
    * <p>
    * the behaviour for the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/initialise:1.0
    * operation
    */
    virtual void on_initialise(const CdmwPlatformMngtBase::StartupKind& startup_kind)
        throw( CORBA::SystemException)
    {

      std::cout << "Server: on_initialise"<<std::endl;
        // ======================================================
        // Get Recorder name domain from repository
        // ======================================================
        CdmwNamingAndRepository::Repository_var repository
            = Cdmw::NamingAndRepository::RepositoryInterface::get_repository();
        CdmwNamingAndRepository::NameDomain_var serverDomain =
            repository->resolve_name_domain ("dom1/dom2/dom3");


        std::cout << "Create Server servant"<<std::endl;
        // create object recorder
        BasicDemo::Server * server_impl
            = new ::BasicDemo::Server ();
        BasicDemo::Display_var serverInterface
            =  server_impl->_this();

        // ========================================================
        // Register the Recorder object in the Recorder name domain
        // ========================================================
        // reserve the name "Recorder" in the domain
        std::string full_name = "P31";

        try
        {
            CdmwNamingAndRepository::NameDomain::RegistrationId_var regId = serverDomain->new_name (full_name.c_str());
            // register the object
            serverDomain->register_object (regId.in(), serverInterface.in ());
        }
        catch (const CORBA::SystemException& e)
        {
            
            std::cerr << "ERROR: ------------------------"
                      << std::endl;
            std::cerr << e << std::endl;
        }
    }

};


#endif // INCL_SERVER_PROCESS_BEHAVIOUR_HPP

