/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_SERVER_PROCESS_BEHAVIOUR_HPP 
#define INCL_SERVER_PROCESS_BEHAVIOUR_HPP 

#include "SystemMngt/platforminterface/PlatformInterface.hpp"
#include <ConfAndPlug/cdmwinit/ProcessControl.hpp>
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/orbsupport/RefCountLocalObject.hpp"
#include <FaultTolerance/idllib/FT.stub.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include "Repository/repositoryinterface/RepositoryInterface.hpp"
#include "Repository/idllib/CdmwNamingAndRepository.stub.hpp"
#include "testftrecorder/CdmwRecordingRecorder_impl.hpp"
#include <Foundation/testutils/CORBATestManager.hpp>
#include <string>
#include <sstream>
#include <iostream>
#include <fstream>


class test_ft_recorder_activate : public CppUnit::TestFixture
{
    
public:
  test_ft_recorder_activate()
  {
  };
    

  ~test_ft_recorder_activate()
  {
  };

  //
  //
  void do_tests();
    
  CPPUNIT_TEST_SUITE( test_ft_recorder_activate );
  CPPUNIT_TEST( do_tests );
  CPPUNIT_TEST_SUITE_END();

};
       
    
class TestThread : public Cdmw::OsSupport::Thread 
{
public:
  TestThread()
  {};
        
protected:
  void run() throw()
  {
    Cdmw::TestUtils::ThreadedTestRunner runner(Cdmw::TestUtils::CORBATestManager::instance()->get_controller());
    runner.addTest( CppUnit::TestFactoryRegistry::getRegistry("test_ft_recorder_activate").makeTest() );
    runner.start();
    runner.join();
  };
        
private:
  TestThread(const TestThread&);
  TestThread& operator = (const TestThread&);
};

/**
* Purpose:
* <p>
* The abstract process behaviour.
*/
//class MyProcessBehaviour : public Cdmw::PlatformMngt::ProcessBehaviour
class ServerProcessBehaviour : public Cdmw::CdmwInit::ProcessControl
{

public:

    /**
    * Purpose:
    * <p> The constructor.
    */ 
    ServerProcessBehaviour  (CORBA::ORB_ptr orb, PortableServer::POA_ptr rootPOA, std::string the_host, std::string first)
      : m_hostname(the_host),
        m_testThread()

    {
        m_orb = CORBA::ORB::_duplicate(orb);
        m_rootPOA =  PortableServer::POA::_duplicate(rootPOA);
        if (first == "yes")
          m_first = true;
        else m_first = false;
    }


    /**
    * Purpose:
    * <p> The destructor.
    */ 
    virtual ~ServerProcessBehaviour() throw ()
    {
     
    }

    
    /**
    * Purpose:
    * <p>
    * the behaviour for the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/nb_steps:1.0
    * attribute
    */
    virtual CORBA::ULong nb_steps() throw(CORBA::SystemException)
    {
        return 1;
    }
    
    
    /**
    * Purpose:
    * <p>
    * the behaviour for the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/get_service:1.0
    * operation
    */
    virtual CORBA::Object_ptr get_service() throw(CORBA::SystemException)
    {
        return CORBA::Object::_nil();
    }
    
    
    /**
    * Purpose:
    * <p>
    * the behaviour for the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/initialise:1.0
    * operation
    */
    virtual void on_initialise(const CdmwPlatformMngtBase::StartupKind& startup_kind)
        throw( CORBA::SystemException)
    {

        Cdmw::OsSupport::OS::sleep(100);
        
        // get process name
        std::string process_name = Cdmw::PlatformMngt::PlatformInterface::Get_process_name();

        // ======================================================
        // Get Recorder name domain from repository
        // ======================================================
        CdmwNamingAndRepository::Repository_var repository
            = Cdmw::NamingAndRepository::RepositoryInterface::get_repository();
        CdmwNamingAndRepository::NameDomain_var recorderDomain =
            repository->resolve_name_domain ("dom1/dom2/dom3");


        std::cout << m_name << " test_server: create Recorder_impl"<<std::endl;
        // create object recorder
        CdmwRecording::Recorder_impl * recorder_impl
          = new ::CdmwRecording::Recorder_impl (/*m_orb, m_rootPOA,*/ m_first);
        CdmwRecording::Recorder_var recorderInterface
            =  recorder_impl->_this();

        // ========================================================
        // Register the Recorder object in the Recorder name domain
        // ========================================================
        // reserve the name "Recorder" in the domain
        std::string full_name = process_name + "Recorder";

        try
        {
            CdmwNamingAndRepository::NameDomain::RegistrationId_var regId = recorderDomain->new_name (full_name.c_str());
            // register the object
            recorderDomain->register_object (regId.in(), recorderInterface.in ());
        }
        catch (const CORBA::SystemException& e)
        {
            
            std::cerr << "ERROR: ------------------------"
                      << std::endl;
            std::cerr << e << std::endl;
        }


    }
    
    /**
    * Purpose:
    * <p>
    * the behaviour for the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/run:1.0
    * operation
    */
    virtual void on_run()
        throw(CORBA::SystemException,
              CdmwPlatformMngt::ProcessDelegate::NotReadyToRun)
    { 
    }

    /**
    * Purpose:
    * <p>
    * the behaviour for the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/stop:1.0
    * operation
    */
    virtual void on_stop() throw(CORBA::SystemException)
    {    
      // Only the FT backup recorder must run this code because the primary
      // exited.
      m_testThread.start();
      m_testThread.join();
      Cdmw::TestUtils::CORBATestManager::instance()->summary();
    }


    
private:

    /**
    * The ORB.
    */
    CORBA::ORB_var m_orb;
    /**
     * The poa
     */
    PortableServer::POA_var m_rootPOA;
    
    /**
    * The application name.
    */
    std::string m_name;
    std::string m_hostname;

    bool m_first;

    TestThread m_testThread;
};


#endif // INCL_SERVER_PROCESS_BEHAVIOUR_HPP

