/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include "testftrecorder/CdmwRecordingRecorder_impl.hpp"
#include <Foundation/osthreads/MutexGuard.hpp>
#include "testftrecorder/CdmwRecordingRecorder_impl.hpp"
#include "testftrecorder/GroupCreatorProcessBehaviour.hpp"

bool CdmwRecording::Recorder_impl::m_first = false;
int CdmwRecording::Recorder_impl::m_nb_request = 0;

namespace CdmwRecording
{


Recorder_impl::Recorder_impl(bool first)
{
  
  CdmwRecording::Recorder_impl::m_first = first;

  if (CdmwRecording::Recorder_impl::m_first)
    trace_file = fopen("first_server.txt", "w");
  else
    trace_file = fopen("second_server.txt", "w");
}

Recorder_impl::~Recorder_impl()
throw()
{
  fclose(trace_file);
}

void Recorder_impl::record(const ::CdmwRecording::GroupHeader& header,
                           const GIOP_Buffers& data)
   throw(CORBA::SystemException)
{
  // protect write access to m_nb_request
  CDMW_MUTEX_GUARD(m_mutex);

  CdmwRecording::Recorder_impl::m_nb_request++;

  if (CdmwRecording::Recorder_impl::m_first)
  {
    if (CdmwRecording::Recorder_impl::m_nb_request == 3) {
      // Display a CPPUnit summary to show that the FT primary can receive also
      // successfully records
      TestThread testThread;
      testThread.start();
      testThread.join();
      Cdmw::TestUtils::CORBATestManager::instance()->summary();
      exit(0);
    }
  }

  fprintf(trace_file, "record!\n");
  std::cout << "record!" << std::endl;

}

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(CdmwRecording::test_ft_recorder_activate, "test_ft_recorder_activate");

void
test_ft_recorder_activate::do_tests()
{
        TEST_INFO("test_ft_recorder_activate: Checking that the recorder has received records...");    
        CPPUNIT_ASSERT(CdmwRecording::Recorder_impl::m_nb_request > 0);
}


}; // End namespace CdmwRecording

