/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <exception>
#include <iostream>
#include <sstream>


#include "testftrecorder/Client.hpp"
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>
#include <Repository/idllib/CdmwNamingAndRepository.stub.hpp>
#include <Foundation/common/Assert.hpp>

namespace BasicDemo {

// constructor
Client::Client()
{
    
}

//destructor
Client::~Client()
{
}

void Client::run_client(int nb_message)
{

    try
    {
      std::cout << "Client: running" << std::endl;

      // get NamingInterface 
      Cdmw::CommonSvcs::Naming::NamingInterface ni =
        Cdmw::NamingAndRepository::RepositoryInterface::get_domain_naming_interface ("dom1/dom2/dom3");
      std::cout << "Client: get_domain_naming_interface" << std::endl;

      std::string full_name = "P31";

      CORBA::Object_var obj;

      try
      {
        obj = ni.resolve (full_name);
        std::cout << "Client: resolve on server name" << std::endl;
        CDMW_ASSERT(true);
      }
      catch(const CosNaming::NamingContext::NotFound& e)
      {
        std::cerr<< e << std::endl;
        CDMW_ASSERT(false);
      }
      catch(const CosNaming::NamingContext::CannotProceed& e)
      {
        std::cerr << e << std::endl;
        CDMW_ASSERT(false);
      }
      catch(const CosNaming::NamingContext::InvalidName& e)
      {
        std::cerr << e << std::endl;
        CDMW_ASSERT(false);
      }
      catch (const CORBA::SystemException& e)
      {
        std::cerr << e << std::endl;
        CDMW_ASSERT(false);
      }

      ::BasicDemo::Display_var display_interface = ::BasicDemo::Display::_narrow(obj.in());
      CDMW_ASSERT( ! CORBA::is_nil(display_interface.in()) );

      // Stream to convert in a string
      std::ostringstream echo_stream;
      // String converted from a stream
      std::string echo_string;
      // Message displays by the server
      ::CORBA::String_var str_in;
      
      // Call to server to display the messages
      for (int i = 1; i < nb_message + 1; i++) {
        echo_stream << "message " << i << "\n";
        echo_string = echo_stream.str();
        echo_stream.str("");//echo_stream.clear();
        str_in = echo_string.c_str();
        display_interface->echo(str_in.in());
      }

    }
    catch(const CORBA::Exception& e ) {
        std::cerr << e._name() << std::endl;
        CDMW_ASSERT(false);
    }
    catch( const Cdmw::Exception& e ) {
        std::cerr << e.what() << std::endl;
        CDMW_ASSERT(false);
    }
}

}; // namespace BasicDemo

