/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "testinterfacefilter/TestInterfaceFilter.hpp"

#include <cstdlib>
#include <string>
#include <iostream>
#include <fstream>
#include <sys/types.h>
#include <sys/stat.h>

/**
*Implementation notes: [if no pertinent write none ]
*<p>
*Portability issues: [if no pertinent write none ]
*<p>
*/

CPPUNIT_TEST_SUITE_REGISTRATION( TestInterfaceFilter );

TestInterfaceFilter::TestInterfaceFilter()
{
}


TestInterfaceFilter::~TestInterfaceFilter()
    throw()
{
  //if(om) delete om;
}

std::ofstream * TestInterfaceFilter::om = new std::ofstream("tmp.txt");

void TestInterfaceFilter::print_flag (string const & key_str, Cdmw::Recording::InterfaceFilter::Flag_t const & flag)
{
  (*om) << key_str << " -> ownflag = " << flag.ownflag << " activation_counter = " << flag.counter << std::endl;
}

void TestInterfaceFilter::print (string const & str1, string const & str2, size_t i)
{
}

void TestInterfaceFilter::create_folder()
{
    //set output folder
   output_folder = "V-TRC-009_004-results";
    
    //create output folder
    if (mkdir(output_folder.c_str(), 777) != 0)
    {
        std::cerr << "Error creating output directory" << std::endl;
    }
}

void TestInterfaceFilter::create_file(std::string filename, const char * data)
{
    FILE * pFile;
    pFile = fopen (filename.c_str(), "w");
    fprintf (pFile, "%s\n", data);
    fclose (pFile); 
} 

void TestInterfaceFilter::print_result(bool result)
{
    //set output file
    char host_name[100] = "";
    gethostname(host_name, sizeof(host_name));
    std::string output_file = output_folder + "/execution-" + host_name + ".output";
    
    //output test result
    FILE * pFile;
    pFile = fopen (output_file.c_str(),"w");
    if (result)
        fprintf (pFile, "%s\n", "The test scenario is OK");
    else
        fprintf (pFile, "%s\n", "The test scenario is FAILED");
           
    fclose (pFile);     
}

void 
TestInterfaceFilter::do_tests()
{
    //create output folder
    create_folder();
    print_result(false);
    
    //create file in input interfaces
    create_file("interfaces.txt", "IDL:intf1:1.0 parent1\nIDL:intf2:1.0\nIDL:intf3:1.0 parent3 parent4");
    
    Cdmw::Recording::InterfaceFilter::Status status ;  
    CdmwRecording::String_List intf_list;
    std::string line = "";

    std::string intf1 = "IDL:intf1:1.0" ;   // in config file
    std::string intf2 = "IDL:intf2:1.0" ;
    std::string intf3 = "IDL:intf3:1.0" ; 
    std::string intf4 = "IDL:intf4:1.0" ;   // not in config file
    std::string intf5 = "IDL:intf5:1.0" ;
    std::string intf6 = "IDL:intf6:1.0" ;

    TEST_INFO("Testing interface filter"); 


    //======= 1 initialising: 1,2,3

    Cdmw::Recording::InterfaceFilter intf_filter ("interfaces.txt") ;

    CPPUNIT_ASSERT(true == intf_filter.checkInterface (intf1.c_str() ) );
    CPPUNIT_ASSERT(true == intf_filter.checkInterface (intf2.c_str() ) );
    CPPUNIT_ASSERT(true == intf_filter.checkInterface (intf3.c_str() ) );
    CPPUNIT_ASSERT(false == intf_filter.checkInterface (intf4.c_str() ) );


    //======== 2 removing interfaces 1,4   ->  2,3 

    intf_list.length(2);
    intf_list[0] = intf1.c_str() ;  //     in the list
    intf_list[1] = intf4.c_str() ;  // not in the list
    status = intf_filter.removeInterfaces(intf_list); 

    CPPUNIT_ASSERT(0 == status);  
      
    CPPUNIT_ASSERT(false == intf_filter.checkInterface (intf1.c_str() ) );
    CPPUNIT_ASSERT(false == intf_filter.checkInterface (intf4.c_str() ) );
    CPPUNIT_ASSERT(true == intf_filter.checkInterface (intf2.c_str() ) );
    CPPUNIT_ASSERT(true == intf_filter.checkInterface (intf3.c_str() ) );


    //======== 3 adding interfaces 1,2,5  ->  1,2,3,5

    intf_list.length(3);
    intf_list[0] = intf1.c_str() ; // not in the list
    intf_list[1] = intf2.c_str() ; //     in the list
    intf_list[2] = intf5.c_str() ; // not in the list

    status = intf_filter.addInterfaces(intf_list);
    CPPUNIT_ASSERT(0 == status);  

    CPPUNIT_ASSERT(true == intf_filter.checkInterface (intf1.c_str() ) );
    CPPUNIT_ASSERT(true == intf_filter.checkInterface (intf2.c_str() ) );
    CPPUNIT_ASSERT(true == intf_filter.checkInterface (intf5.c_str() ) );

    // check that intf2 is only once in the list:

    //om = new std::ofstream("tmp.txt");
    intf_filter.mapLoop(print, print_flag);
    (*om).close();

    int count = 0;
    std::ifstream ifm("tmp.txt");
    while(!ifm.eof())
    {
        getline(ifm, line);
        if (line.find("IDL:intf5:1.0 -> ownflag = 1 activation_counter = 1",0) != std::string::npos) count++;
    }
    ifm.close();
   
    CPPUNIT_ASSERT(1 == count);

    remove("tmp.txt");
   

    //======== 4 adding interfaces with spaces 4,6 ->  1,2,3,4,5,6

    intf_list.length(2);
    intf_list[0] = "  IDL:intf4:1.0" ;
    intf_list[1] = "IDL:intf6:1.0    " ;

    status = intf_filter.addInterfaces(intf_list);
    CPPUNIT_ASSERT(0 == status);

    CPPUNIT_ASSERT(true == intf_filter.checkInterface (intf4.c_str() ) );
    CPPUNIT_ASSERT(true == intf_filter.checkInterface (intf6.c_str() ) );
   
   
    //=========5 check interfaces with spaces
   
    CPPUNIT_ASSERT(true == intf_filter.checkInterface (" IDL:intf1:1.0") );
    CPPUNIT_ASSERT(true == intf_filter.checkInterface ("IDL:intf1:1.0 ") );
   

    print_result(true);
}


