/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <Foundation/common/System.hpp>
#include <Foundation/ossupport/OS.hpp>

#include <fstream>

#include "testinterruptedclient/InterruptedClientTester.hpp"
#include "testinterruptedclient/CommonDefs.hpp"

namespace {
    const unsigned NUM_PROCESS_RUN = 10;
    const int TIME_PROCESS_SLEEP   = 650;
}
using namespace Cdmw::OsSupport;

CPPUNIT_TEST_SUITE_REGISTRATION(InterruptedClientTester);


InterruptedClientTester::InterruptedClientTester()
{
    m_host = Cdmw::TestUtils::CORBATestManager::instance()->get_property(
    "host", "localhost");
}

InterruptedClientTester::~InterruptedClientTester()
   throw()
{
}

void
InterruptedClientTester::do_tests()
{
    test_asynchronous_traces();
    test_synchronous_traces();
}

void
InterruptedClientTester::test_synchronous_traces()
{
    TEST_INFO("***** " << NUM_PROCESS_RUN 
          << " steps to perform with a synchronous logging *****");

    try
    {

    for (unsigned int i = 0; i < NUM_PROCESS_RUN; i++)
    {
        TEST_INFO("***** step " << i + 1 << " *****");

        // the process launched hereafter is expected to log a trace
        // and then crash
        std::string proc =
        APPLICATION_NAME + " " + SYNC_PROCESS_NAME + " " + m_host;
        OS::create_process(
        "platform_admin.sh", "--proc-initialise " + proc, ".", 0, 0);
        OS::sleep(TIME_PROCESS_SLEEP);
        OS::create_process(
        "platform_admin.sh", "--proc-run " + proc, ".", 0, 0);
        OS::sleep(TIME_PROCESS_SLEEP);
    }

    // as traces for this process are configured in a synchronous way
    // (i.e. timeout is 0), no traces can be lost
    std::string line;
    std::ifstream log(LOG_FILE.c_str());
    if (log.is_open())
    {
        unsigned num = 0;
        while (! log.eof())
        {
        getline(log, line);
        if ((line.find(LOG_MESSAGE) < line.size())
            && (line.find(SYNC_PROCESS_NAME) < line.size()))
        {
            num++;
        }
        }
        log.close();

        CPPUNIT_ASSERT_EQUAL_MESSAGE(
        "Some logs have been lost", NUM_PROCESS_RUN, num);
    }
    else
    {
        CPPUNIT_FAIL("Unable to open log file");
    }
    }

    catch (...)
    {
    CPPUNIT_FAIL("Unexpected exception raised");
    }
}

void
InterruptedClientTester::test_asynchronous_traces()
{
    TEST_INFO("***** " << NUM_PROCESS_RUN
          << " steps to perform with an asynchronous logging *****");

    try
    {
    for (unsigned int i = 0; i < NUM_PROCESS_RUN; i++)
    {
        TEST_INFO("***** step " << i + 1 << " *****");

        // the process launched hereafter is expected to log a trace
        // and then crash
        std::string proc =
        APPLICATION_NAME + " " + ASYNC_PROCESS_NAME + " " + m_host;
        OS::create_process(
        "platform_admin.sh", "--proc-initialise " + proc, ".", 0, 0);
        OS::sleep(TIME_PROCESS_SLEEP);
        OS::create_process(
        "platform_admin.sh", "--proc-run " + proc, ".", 0, 0);
        OS::sleep(TIME_PROCESS_SLEEP);
    }

    // as traces for this process are configured in an asynchronous way
    // (i.e. timeout > 0), traces can be lost
    std::string line;
    std::ifstream log(LOG_FILE.c_str());
    if (log.is_open())
    {
        unsigned num = 0;
        while (! log.eof())
        {
        getline(log, line);
        if ((line.find(LOG_MESSAGE) < line.size())
            && (line.find(ASYNC_PROCESS_NAME) < line.size()))
        {
            num++;
        }
        }
        log.close();

        CPPUNIT_ASSERT_MESSAGE(
        "No log has been lost", num < NUM_PROCESS_RUN);
    }
    else
    {
        CPPUNIT_FAIL("Unable to open log file");
    }
    }

    catch (...)
    {
    CPPUNIT_FAIL("Unexpected exception raised");
    }
}
