/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/Logger.hpp>
#include <ConfAndPlug/cdmwinit/CdmwInterface.hpp>
#include <ConfAndPlug/cdmwinit/InitUtils.hpp>
#include <TraceAndPerf/tracelibrary/Trace.hpp>

#include <cstdlib>
#include <string>
#include <exception>
#include <sstream>
#include <iostream>
#include <fstream>

#include "testinterruptedclient/CommonDefs.hpp"


class MyProcessBehaviour : public Cdmw::CdmwInit::ProcessControl
{

public:

    MyProcessBehaviour(CORBA::ORB_ptr orb,
                       Cdmw::Logging::Logger* logger)
        :
        m_orb(CORBA::ORB::_duplicate(orb)),
        m_logger(logger)
    {
    }

    virtual ~MyProcessBehaviour() 
        throw ()
    {
    }
    
    virtual CORBA::ULong nb_steps() 
        throw(CORBA::SystemException)
    {
        return 1;
    }
        
    virtual CORBA::Object_ptr get_service() 
        throw(CORBA::SystemException)
    {
        return CORBA::Object::_nil();
    }
        
    virtual void on_initialise(
        const CdmwPlatformMngtBase::StartupKind& startup_kind
        )
        throw(CORBA::SystemException)
    {
    }

    virtual void on_next_step()
        throw(CdmwPlatformMngt::ProcessDelegate::InvalidStep, 
              CORBA::SystemException)
    {
    }

    virtual void on_run()
        throw(CdmwPlatformMngt::ProcessDelegate::NotReadyToRun,
              CORBA::SystemException)
    {
        CDMW_ERROR(m_logger, LOG_MESSAGE);
        exit(-1);
    }

    virtual void on_stop() 
        throw(CORBA::SystemException)
    {
    }

private:
    CORBA::ORB_var m_orb;
    Cdmw::Logging::Logger* m_logger;    
};



namespace
{
    const int POA_THREAD_POOL_SIZE = 5;
    const int SUCCESS = 0;
    const int FAILURE = 1;
}

int main(int argc, char* argv[])
{
    int ret_code = 0;

    CORBA::ORB_var orb;
    using Cdmw::Logging::Logger;
    Logger* logger = NULL;

    try
    {
        // initialize the ORB.
        Cdmw::OrbSupport::StrategyList orb_strategies;
        orb_strategies.add_OrbThreaded();
        orb_strategies.add_PoaThreadPool(POA_THREAD_POOL_SIZE);

        orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv,orb_strategies);

        // get the root POA.
        CORBA::Object_var obj = orb -> resolve_initial_references("RootPOA");
        PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj.in());

        // activate the root POA manager.
        PortableServer::POAManager_var poaManager = rootPOA -> the_POAManager();
        poaManager -> activate();

        logger = Logger::Get_Logger("TEST", APPLICATION_NAME);

        // creates the process behaviour
        MyProcessBehaviour pProcBehaviour(orb.in(), logger);
 
        // call generated CDMW_init().
        Cdmw::CdmwInit::CDMW_init(orb.in(), argc, argv, &pProcBehaviour);

        // start orb
        orb->run();            
    }
            
    catch (const Cdmw::Exception & ex) 
    {
        std::cerr << ex.what() << std::endl;
        ret_code = FAILURE;
    }
    catch (const CORBA::Exception& e)
    {  
        std::cerr << "FAILURE : CORBA exception raised :" << std::endl
                  << e._name() << std::endl;
        ret_code = FAILURE;
    }
    catch(const std::exception &e)
    {
        std::cerr << "FAILURE : Exception raised : " << std::endl
                  << e.what() << std::endl;
        ret_code = FAILURE;
    }

    if (!CORBA::is_nil(orb.in()))
    {
        try 
        {
            Cdmw::CdmwInit::CDMW_cleanup(orb.in());
            Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
            orb->destroy();
        }
        catch(const CORBA::Exception& ex)
        {           
            std::cerr << ex << std::endl;
            ret_code = FAILURE;
        } 
    }

    return ret_code;
}


