/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <string>

#include "testinterruptedclient/InterruptedClientTester.hpp"


static void
usage(std::ostream & os, const std::string &program_name)
{
    os << "Usage:\n"
       << program_name << " <host>"
       << std:: endl;
}


int main(int argc, char** argv)
{
    if (argc != 2)
    {
    usage(std::cout, argv[0]);
    return 0;
    }

    try 
    {
        // Initialize orb
        Cdmw::OrbSupport::StrategyList orb_strategies;
        orb_strategies.add_OrbThreaded();
        orb_strategies.add_PoaThreadPool(5);    

        CORBA::ORB_var orb = 
            Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, orb_strategies);

        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var root_poa = PortableServer::POA::_narrow(obj);

        Cdmw::TestUtils::CORBATestManager* testManager =
            Cdmw::TestUtils::CORBATestManager::instance();
    testManager->init(orb.in(), root_poa.in(), argc, argv);
    testManager->set_property("host", argv[1]);

    testManager->run_tests();
        testManager->summary();
    }

    catch (...)
    {
        std::cout << "Exception in main.cpp" << std::endl;
        throw;
    }

    return 0;
}
