/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.valid.testinterruptedclient;

import cdmw.logging.Logger;
import static cdmw.logging.Logger.*;
import static cdmw.valid.testinterruptedclient.CommonDefs.*;

import org.omg.CORBA.ORB;

import cdmw.orbsupport.ORBSupport;
import cdmw.orbsupport.StrategyList;
import com.thalesgroup.CdmwPlatformMngt.ProcessDelegatePackage.NotReadyToRun;
import com.thalesgroup.CdmwPlatformMngtBase.StartupKind;


/**
 * Purpose: <p> The abstract process behaviour.
 */
class MyProcessBehaviour extends cdmw.cdmwinit.ProcessControl {

    private ORB orb;
    private Logger logger;

    /**
     * Purpose: <p> The constructor.
     */
    public MyProcessBehaviour(ORB orb, Logger logger) {
        this.orb = orb;
    this.logger = logger;
    }

    /**
     * Purpose: <p> the behaviour for the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/initialise:1.0
     * operation
     */
    @Override
    public void onInitialise(StartupKind startupKind) {

    }

    /**
     * Purpose: <p> the behaviour for the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/run:1.0 operation
     */
    @Override
    public void onRun() throws NotReadyToRun {
        CDMW_ERROR(this.logger, LOG_MESSAGE, new Throwable());
    System.exit(-1);
    }

    /**
     * Purpose: <p> the behaviour for the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/stop:1.0 operation
     */
    @Override
    public void onStop() {
    }
}


public class DumbTraceClient {

    public final static int POA_THREAD_POOL_SIZE = 5;

    public static void main(String[] args) {
        
        org.omg.CORBA.ORB orb = null;
        try {
            // Initialize the ORB
            StrategyList orbStrategies = new StrategyList();
            orbStrategies.addORBThreaded();
            orbStrategies.addPOAThreadPool(POA_THREAD_POOL_SIZE);
    
            orb = ORBSupport.ORBInit(args, orbStrategies);

            // Get the root POA 
            org.omg.CORBA.Object obj = 
                orb.resolve_initial_references("RootPOA");
            org.omg.PortableServer.POA rootPOA = 
                org.omg.PortableServer.POAHelper.narrow(obj);

            // Run implementation (Activate the POA manager)
            rootPOA.the_POAManager().activate();

        Logger logger = Logger.Get_Logger("TEST", APPLICATION_NAME);

        // creates the process behaviour
        MyProcessBehaviour pProcBehaviour =
        new MyProcessBehaviour(orb, logger);

            // init Cdmw
            cdmw.cdmwinit.CdmwInterface.cdmwInit(
        orb, args, pProcBehaviour,
        "cdmw.generated.testInterruptedClientCdmwInterface");

        // start orb
        orb.run();
        }
    catch (Exception ex) {
            System.err.println(ex.getMessage());
        }

        if (orb != null) {
            try {
                cdmw.cdmwinit.CdmwInterface.cdmwCleanup(
                    orb, "cdmw.generated.testInterruptedClientCdmwInterface");
                orb.destroy();
            } catch(Exception ex) {
                System.err.println(ex.getMessage());
            }
        }
    }
}
