/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.valid.testinterruptedclient;

import static cdmw.valid.testinterruptedclient.CommonDefs.*;
import org.junit.*;
import static org.junit.Assert.*;
import org.omg.CORBA.*;
import java.lang.String;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;


public class InterruptedClientTester {

    private static final int NUM_PROCESS_RUN    = 10;
    private static final int TIME_PROCESS_SLEEP = 650;

    private String host;

    public InterruptedClientTester() {
    host = cdmw.testutils.CORBATestManager.instance().getProperty("host");
    }

    @Test
    public void do_tests() {
    testAsynchronousTraces();
    testSynchronousTraces();
    }

    protected void testSynchronousTraces() {
    System.out.println(
        "***** " + NUM_PROCESS_RUN 
        + " steps to perform with a synchronous logging *****");

    try
    {

        for (int i = 0; i < NUM_PROCESS_RUN; i++)
        {
        System.out.println("***** step " + (i + 1) + " *****");

        // the process launched hereafter is expected to log a trace
        // and then crash
        String proc =
            APPLICATION_NAME + " " + SYNC_PROCESS_NAME + " " + host;
        cdmw.ossupport.OS.createProcess(
            "platform_admin.sh --proc-initialise " + proc);
        Thread.sleep(TIME_PROCESS_SLEEP);
        cdmw.ossupport.OS.createProcess(
            "platform_admin.sh --proc-run " + proc);
        Thread.sleep(TIME_PROCESS_SLEEP);
        }

        // as traces for this process are configured in a synchronous way
        // (i.e. timeout is 0), no traces can be lost

        BufferedReader log = null;
        try {
        log = new BufferedReader(new FileReader(LOG_FILE));
        int num = 0;

        String line;
        while ((line = log.readLine()) != null) {
            String regexp = 
            ".*" + SYNC_PROCESS_NAME + ".*" + LOG_MESSAGE + ".*";
            if (line.matches(regexp)) {
            num++;
            }
        }
        log.close();

        assertEquals(
            "Some logs have been lost", NUM_PROCESS_RUN, num);
        }

        catch (FileNotFoundException fnfe) {
        fail("Unable to open log file");
        }
        catch (IOException ioe) {
        fail("Unable to read log file");
        }
    }

    catch (Exception e)
    {
        fail("Unexpected exception raised:" + e.toString());
    }
    }


    protected void testAsynchronousTraces() {
    System.out.println(
        "***** " + NUM_PROCESS_RUN 
        + " steps to perform with an asynchronous logging *****");

    try
    {

        for (int i = 0; i < NUM_PROCESS_RUN; i++)
        {
        System.out.println("***** step " + (i + 1) + " *****");

        // the process launched hereafter is expected to log a trace
        // and then crash
        String proc =
            APPLICATION_NAME + " " + ASYNC_PROCESS_NAME + " " + host;
        cdmw.ossupport.OS.createProcess(
            "platform_admin.sh --proc-initialise " + proc);
        Thread.sleep(TIME_PROCESS_SLEEP);
        cdmw.ossupport.OS.createProcess(
            "platform_admin.sh --proc-run " + proc);
        Thread.sleep(TIME_PROCESS_SLEEP);
        }

        // as traces for this process are configured in an asynchronous way
        // (i.e. timeout > 0), traces can lost
        //
        BufferedReader log = null;
        try {
        log = new BufferedReader(new FileReader(LOG_FILE));
        int num = 0;

        String line;
        while ((line = log.readLine()) != null) {
            String regexp = 
            ".*" + ASYNC_PROCESS_NAME + ".*" + LOG_MESSAGE + ".*";
            if (line.matches(regexp)) {
            num++;
            }
        }
        log.close();

        assertTrue(
            "No log has been lost", num < NUM_PROCESS_RUN);
        }

        catch (FileNotFoundException fnfe) {
        fail("Unable to open log file");
        }
        catch (IOException ioe) {
        fail("Unable to read log file");
        }
    }

    catch (Exception e)
    {
        fail("Unexpected exception raised:" + e.toString());
    }
    }
}
