/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_CONTROLLER_HPP 
#define INCL_CONTROLLER_HPP

#include <stdio.h>
#include <string>

using namespace std;

namespace Cdmw
{
namespace Recording
{
    
/**
* Purpose:
* <p>
* Check and output test result
*/
class Controller 
{
public:


    /**
    * Purpose:
    * <p>
    * Initializing
    */
    void init(std::string test_nr);

    /**
    * Purpose:
    * <p>
    * Output last error in result
    */
    void log_error(const char * error);
    
    /**
    * Purpose:
    * <p>
    * Output test result
    */
    void log_result(int errors_count);

    /**
    * Purpose:
    * <p>
    * Compare results with expected and output result in file 
    */
    bool check_test_result(int requests_count, bool is_oneway, const char * output_file);
    
private:

    /**
    *Name of output folder
    */     
    string output_folder;
        
    enum MessageType { Message, Error } ;

    /**
    * Purpose:
    * <p>
    * Get the count of identified requests/replies from output file
    * for the specified iteration
    * 
    */
    int get_identified_requests_replies_count(const char * output_file, int iteration, bool is_request);
    
    /**
    * Purpose:
    * <p>
    * Log some data in output file
    * 
    */
    void log_data(const char * message, MessageType type);

};

}; // End namespace Recording
}; // End namespace Cdmw

#endif // INCL_CONTROLLER_HPP
 

