/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_TRACE_MY_REQUEST_PAIRING_HPP 
#define INCL_TRACE_MY_REQUEST_PAIRING_HPP 


#include "TraceAndPerf/recordinglibrary/PacketStore.hpp"
#include "TraceAndPerf/recordinglibrary/RequestPairing.hpp"


using namespace std;

namespace Cdmw
{
namespace Recording
{
    
/**
*Purpose:
*<p>    Realization of request pairing to override storeRequest 
* and checkReply methods
*
*/
class MyRequestPairing : public Cdmw::Recording::RequestPairing
{
    public:

        /**
        * Purpose:
        * <p> Override parents constructor
        * 
        */ 
        MyRequestPairing(Cdmw::Recording::PacketStore & store);


        /**
        * Purpose:
        * <p> [ Destructor description if necessary]
        * 
        */ 
        virtual 
        ~MyRequestPairing()
                throw();
                

        /**
        * Purpose:
        * <p> Override parent method.
        * Request will be stored in file
        * 
        */ 
        void storeRequest(Cdmw::Recording::Message * message)
            throw();
                

        /**
        * Purpose:
        * <p> Override parent method.
        * Reply will be stored in file
        * 
        */ 
        void checkReply(Cdmw::Recording::Message * message)
            throw();
            
        /**
        * Purpose:
        * <p> Write header in output file
        * 
        */ 
        void write_header(const char * header);
        
        /**
        * Purpose:
        * <p> Get output file
        * 
        */ 
        const char * get_output();
        
        /**
        * Purpose:
        * <p> Set output_file attribute to output file name 
        * 
        */ 
        void set_output(string test_nr);
        
        
    private:
        /**
        *Name of output file
        */     
        string output_file;

        /**
        *Name of output folder
        */     
        string output_folder;

 
        /**
        * Purpose:
        * <p> Create folder where results will be saved 
        * 
        */ 
        void create_output_folder();
                
}; // End class MyRequestPairing                

}; // End namespace Recording
}; // End namespace Cdmw

#endif // INCL_TRACE_MY_REQUEST_PAIRING_HPP    
