/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <fstream>
#include <iostream>
#include <cstdlib>
#include <string>
#include <sstream>

#include "testmessageidentifier/Controller.hpp"

namespace Cdmw
{
namespace Recording
{
    
void Controller::init(std::string test_nr)
{
    output_folder = "V-TRC-013-"+ test_nr +"-results";
}

int Controller::get_identified_requests_replies_count(const char * output_file, int iteration, bool is_request)
{
    int identified_requests_count = 0;
    int current_iteration = 0;
    string line = "";
    basic_string <char>::size_type index;
    string search_line = "";
    
    //get line to find
    if (is_request)
    {
        search_line = "Request:";
    }  
    else
    {
        search_line = "Reply";
    }
    
    
    ifstream inFile;
    inFile.open(output_file);
    while(!inFile.eof())
    {
        getline(inFile, line);
        //check iteration
        index = line.find("##Step", 0);
        if ( index != string::npos )
        {
            current_iteration++;
        }
        
        //check identified
        if (current_iteration == iteration)
        {
            index = line.find(search_line, 0);
            if ( index != string::npos )
            {
                identified_requests_count++;
            }
        }
    }
    return identified_requests_count;    
}


bool Controller::check_test_result(int requests_count, bool is_oneway, const char * output_file)
{
    int identified_requests_count = 0;
    int identified_replies_count = 0;
    int errors_count = 0;
    std::stringstream err;
        
    //
    //Check iteration 1
    //
    std::cout << "=== Controller : outputfile = " << output_file << std::endl;
    identified_requests_count = get_identified_requests_replies_count(output_file, 1, true);
    identified_replies_count = get_identified_requests_replies_count(output_file, 1, false);
    std::cout << "=== Controller:Step #1" << std::endl;
    std::cout << "=== Controller:identified_requests_count = " << identified_requests_count
              << " / requests_count (expected)= " << requests_count << std::endl;

    //check requests count
    if (identified_requests_count != requests_count)
    {
        err << "Step 1: Incorrect count of identified requests - " << identified_requests_count ;
        log_error(err.str().c_str());
        errors_count++;
    }
    
    //check replies count
    if (!is_oneway) //twoway messages
    {
        std::cout << "=== Controller:Step #1" << std::endl;
        std::cout << "=== Controller:identified_replies_count = " << identified_replies_count
              << " / requests_count (expected)= " << requests_count << std::endl;
        if (identified_replies_count < requests_count)
        {
            err << "Step 1: Incorrect count of identified replies - " << identified_replies_count ;
            log_error(err.str().c_str());
            errors_count++;
        }
    }
  
    
    //
    //Check Step 2
    //
    std::cout << "=== Controller : outputfile = " << output_file << std::endl;
    identified_requests_count = get_identified_requests_replies_count(output_file, 2, true);
    identified_replies_count = get_identified_requests_replies_count(output_file, 2, false);
    std::cout << "=== Controller:Step #2 (message processing stopped)" << std::endl;
    std::cout << "=== Controller:identified_requests_count = " << identified_requests_count
              << " / requests_count (expected)= 0" << std::endl;

    //check requests count
    if (identified_requests_count != 0)
    {   
        err << "Step 2: Incorrect count of identified requests (non-zero) - " << identified_requests_count ;
        log_error(err.str().c_str());
        errors_count++;
    }
    
   

    //
    //Check Step 3
    //
    std::cout << "=== Controller : outputfile = " << output_file << std::endl;
    identified_requests_count = get_identified_requests_replies_count(output_file, 3, true);
    identified_replies_count = get_identified_requests_replies_count(output_file, 3, false);
    std::cout << "=== Controller:Step #3 (message processing terminated)" << std::endl;
    std::cout << "=== Controller:identified_requests_count = " << identified_requests_count
              << " / requests_count (expected)= 0" << std::endl;

    //check requests count
    if (identified_requests_count != 0)
    {
         err << "Step 3: Incorrect count of identified requests (non-zero) - " << identified_requests_count;
        log_error(err.str().c_str());
        errors_count++;
    }
 
    
    log_result(errors_count);
    
    if (errors_count != 0)
        return false;
    else
        return true;
}

void Controller::log_error(const char * error)
{
    log_data(error, Error);    
}

void Controller::log_result(int errors_count)
{
    if (0 == errors_count)
    {
        //Test OK
        log_data("\n Test OK", Message);
    }
    else
    {
        //Test FAILED
        //string temp_str = errors_count + "errors occured";
        //log_data(temp_str.c_str(), Message);
        log_data("\n Test FAILED", Message);
    }
}

void Controller::log_data(const char * message, MessageType type)
{
    FILE * pFile;
    std::string file_name = output_folder + "/scenario.result";  
    pFile = fopen (file_name.c_str(),"a+");
    
    if (type == Error)   fprintf (pFile, "ERROR: %s\n", message) ; 
    else                 fprintf (pFile, " %s\n", message) ;
      
    fclose (pFile);  
}


}; // End namespace Recording
}; // End namespace Cdmw
