/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "testmessageidentifier/TestMessageIdentifier.hpp"
#include "testmessageidentifier/MyRequestPairing.hpp"
#include "testmessageidentifier/Controller.hpp"

#include <cstdlib>
#include <string>
#include <iostream>
#include <fstream>

#include <semaphore.h>
#include <sys/stat.h>   //S_IRUSR|S_IWUSR
#include <fcntl.h>      //O_CREAT

namespace
{
    const int SUCCESS = 0;
    const int FAILURE = 1;
}

    
/**
*Implementation notes: [if no pertinent write none ]
*<p>
*Portability issues: [if no pertinent write none ]
*<p>
*/

CPPUNIT_TEST_SUITE_REGISTRATION( TestMessageIdentifier );

TestMessageIdentifier::TestMessageIdentifier()
{
}


TestMessageIdentifier::~TestMessageIdentifier()
    throw()
{

}

void 
TestMessageIdentifier::do_tests()
{
    check_message_identifier();
}

int TestMessageIdentifier::init_sniffer(Cdmw::Recording::NetworkSniffer * sniffer, int port_number)
{
    std::cout << "add port " << port_number <<std::endl;
    Cdmw::Recording::NetworkSniffer::Status status = sniffer->addPort(port_number);
    CPPUNIT_ASSERT_MESSAGE("adding port error", SUCCESS == status);
    
    std::cout << "run" <<std::endl;
    status = sniffer->run () ; 
    CPPUNIT_ASSERT_MESSAGE("running sniffer error", SUCCESS == status);
   
    std::cout << "start" <<std::endl;
    status = sniffer->start () ;
    //CPPUNIT_ASSERT_MESSAGE("starting sniffer error", SUCCESS != status);

    std::cout << "return" <<std::endl;
    return SUCCESS;
}

int TestMessageIdentifier::init_message_identifier(Cdmw::Recording::MessageIdentifier * message_identifier)
{
    //Run MessageIdentifier
    Cdmw::Recording::MessageIdentifier::Status status = message_identifier->run();
    CPPUNIT_ASSERT_MESSAGE("run message identifier error", 0 == status);
       
    //Start MessageIdentifier
    status = message_identifier->start();
    CPPUNIT_ASSERT_MESSAGE("start message identifier error", 0 == status);

    return SUCCESS;
}

int TestMessageIdentifier::terminate_sniffer(Cdmw::Recording::NetworkSniffer *sniffer)
{
    Cdmw::Recording::NetworkSniffer::Status status = sniffer->stop () ; 
    CPPUNIT_ASSERT_MESSAGE("stopping sniffer error", SUCCESS == status);

    status = sniffer->terminate();
    CPPUNIT_ASSERT_MESSAGE("terminating sniffer error", SUCCESS == status);

    return SUCCESS;
}

int TestMessageIdentifier::check_message_identifier()
{
    int result= SUCCESS;
    int port_number = 0;
    int requests_count = 0;
    bool is_oneway = false;
    std::string test_nr = "" ;

    //
    //get parameters from command line
    //
    try
    {
        port_number = atoi(Cdmw::TestUtils::TestManager::instance()->get_property("port_number").c_str());
        requests_count = atoi(Cdmw::TestUtils::TestManager::instance()->get_property("requests_count").c_str());
        is_oneway = atoi(Cdmw::TestUtils::TestManager::instance()->get_property("is_oneway").c_str()) == 1 ? true : false;
        test_nr = Cdmw::TestUtils::TestManager::instance()->get_property("test_number").c_str();
    }
    catch(const std::exception &e)
    {
        std::cerr << "TestMessageIdentifier: getting parameters exception raised : " << std::endl
                  << e.what() << std::endl;
        return 1;
    }
    catch(...)
    {
        std::cerr << "TestMessageIdentifier: getting parameters exception" << std::endl;
        return 1;
    }       

    try
    {
        std::string s = "";
        //
        //Initializing
        //
         
        sem_t * fin_sema = sem_open("/testmi_fin_semaphore",
                            O_CREAT,S_IRUSR|S_IWUSR, 0); 
        sem_t * ready_sema = sem_open("/testmi_ready_semaphore",
                            O_CREAT,S_IRUSR|S_IWUSR, 0);                              
        sem_init(fin_sema,0,0);
        sem_init(ready_sema,0,0);                            
                
        //Create Sniffer
        std::cout << "Create NetworkSniffer" <<std::endl;
        Cdmw::Recording::PacketStore store;  
        Cdmw::Recording::NetworkSniffer sniffer(store);
        
        //Create InterfaceFilter
        std::cout << "Create InterfaceFilter" <<std::endl;
        Cdmw::Recording::InterfaceFilter interface_filter("testmi_interfaces.txt");
        
        //Create OperationMap
        std::cout << "Create OperationMap" <<std::endl;
        Cdmw::Recording::OperationMap operation_map("testmi_operations.txt");
        
        //Create ObjectMap
        std::cout << "Create ObjectMap" <<std::endl;
        Cdmw::Recording::ObjectMap object_map(interface_filter, sniffer);
        
        //Create RequestPairing
        std::cout << "Create MyRequestPairing" <<std::endl;
        Cdmw::Recording::MyRequestPairing pairing(store);
        pairing.set_output(test_nr);
               
        //Init sniffer
        std::cout << "Init NetworkSniffer" <<std::endl;
        CPPUNIT_ASSERT_MESSAGE("init_sniffer() failed", SUCCESS == init_sniffer(&sniffer, port_number));
        
        //
        //Start message processing
        //
        
        //write header in output file
        pairing.write_header("##Step 1 - message processing started");
                
        //Create MessageIdentifier
        Cdmw::Recording::MessageIdentifier message_identifier(store, object_map, interface_filter, operation_map, pairing);

        //Init MessageI dentifier
        CPPUNIT_ASSERT_MESSAGE("init_message_identifier() failed", SUCCESS == init_message_identifier(&message_identifier));
        std::cout << "TestMessageIdentifier: Message processing started, (client sends messages, then sends a finish_iteration)" << endl;             
        std::cout << "TestMessageIdentifier: waiting for \'finish iteration\' from client (1)" << std::endl;
        sem_wait(fin_sema);
        
        std::cout << "TestMessageIdentifier: ok, stopping message identifier.." << std::endl;
        //Stop message processing
        Cdmw::Recording::MessageIdentifier::Status status = message_identifier.stop();
        CPPUNIT_ASSERT_MESSAGE("message_identifier.stop() failed", 0 == status);        
        std::cout << "TestMessageIdentifier: Message processing stopped (client sends messages, then sends a finish_iteration)" << endl;
        //write header in output file
        pairing.write_header("##Step 2 - message processing stopped");
        sem_post(ready_sema);    
        
        std::cout << "TestMessageIdentifier: waiting for \'finish iteration\' from client (2)" << std::endl;
        sem_wait(fin_sema);
  
        std::cout << "TestMessageIdentifier: ok, terminating message identifier.." << std::endl;
       
        //write header in output file
        pairing.write_header("##Step 3 - message processing terminated");

        //Terminate message processing
        status = message_identifier.terminate();
        CPPUNIT_ASSERT_MESSAGE("terminate message identifier error", 0 == status);
        
        std::cout << "TestMessageIdentifier: Message processing terminated(client sends messages, then sends a finish_iteration)" << endl;
        sem_post(ready_sema);
          
        std::cout << "TestMessageIdentifier: waiting for \'finish iteration\' from client (3)" << std::endl;
        sem_wait(fin_sema);
         
        Cdmw::Recording::Controller * controller = new Cdmw::Recording::Controller();
        controller->init(test_nr);
        
        CPPUNIT_ASSERT_MESSAGE("TEST FAILED. Incorrect count of messages identified", controller->check_test_result(requests_count, is_oneway, pairing.get_output()) == true);
        //controller->check_test_result(requests_count, is_oneway, pairing.get_output());
                                         
        std::cout << "TestMessageIdentifier: terminating sniffer.. " << std::endl; 
        //Stop services
        pairing.finalize();   
        result = terminate_sniffer( &sniffer);        
        store.finalize();
       
        std::cout << "TestMessageIdentifier: stop ok " << std::endl; 
      
    }    
    catch(const std::exception &e)
    {
        std::cerr << "TestMessageIdentifier: exception raised : " << std::endl
                  << e.what() << std::endl;
                  throw;
    }
    catch(...)
    {
        std::cerr << "TestMessageIdentifier: exception" << std::endl;
        throw;
    }       
    return result;
}




