/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <iomanip>
#include "testmessageidentifier/Transmission.stub.hpp"

using namespace std;

bool send_finish = true;

void send_messages(bool oneway, int size, int count, Transmission_var tm, const int c, bool send_finish)
//void send_messages(bool oneway, int size, int count, char c, send_finish))
{       
    // Send data
    // std::cout << "CORBA client: Sending messages..." << std::endl;    
    for (int i = 1; i <= count; i++)
    {
        char data[size];
        for (int j = 0; j <= size-1; j++)
            data[j] = c;
        data[size] = '\0';

        if (oneway)    tm->sendto(data);
        else           tm->send(data);
    }      
    std::cout << "CORBA client: sending finished..." << endl;  
    if (send_finish)  
       tm->finish_iteration();           
}
  
int main(int argc, char * argv[])
{
    Transmission_var tm;
    bool is_oneway = false;
    int size = 1;
    int count = 1;

    //
    //Check count of arguments
    //
    if (argc < 4)
    {
        cout << "CORBA client: too few arguments" << endl;
        throw 0;
    }

    //
    //get parameters
    //
    is_oneway = (atoi(argv[2]) == 1) ? true : false;
    size = atoi(argv[3]);
    count = atoi(argv[4]);
    std::cout << "size = " << size << " / count = " << count << std::endl;
   if (argc > 5)
   {
        send_finish = (atoi(argv[5]) == 1) ? true : false;
    }

    try 
    {
        // Initialize orb
        CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);
               
        std::string server_host = std::string(argv[1]);             
        std::string corbaloc = std::string ("corbaloc::") 
                             + server_host + std::string(":6789/Transmission" );     
        cout << "corbaloc of server : " << corbaloc << endl;
        CORBA::Object_var obj = orb->string_to_object(corbaloc.c_str());
        
        if (CORBA::is_nil(obj)) 
        {
            cout << "CORBA client: Nil reference" << endl;
            throw 0;
        }
    
        // Narrow
        //        Transmission_var tm = Transmission::_narrow(obj);
        tm = Transmission::_narrow(obj);
        if (CORBA::is_nil(tm)) 
        {
            cout << "CORBA client: Argument is not a Transmission reference" << endl;
            throw 0;
        }
    
        // message sent while MessageIdentifier is started
        std::cout << "CORBA client: Sending messages... (Message processing being started)" << std::endl;    
        send_messages(is_oneway, size, count, tm, 0x41, send_finish);
            
        tm->ready();
        
        // message sent while MessageIdentifier is stopped
        std::cout << "CORBA client: Sending messages... (Message processing being stopped)" << std::endl;    
        send_messages(is_oneway, size, count, tm, 0x42, send_finish);         
      
        tm->ready();
          
        // message sent while MessageIdentifier is terminated
        std::cout << "CORBA client: Sending messages... (Message processing being terminated)" << std::endl;    
        send_messages(is_oneway, size, count, tm, 0x43, send_finish);
                   
    }
    catch (const CORBA::Exception &) 
    {
        cout << "CORBA client: Uncaught CORBA exception" << endl;
        return 1;
    }
    catch (...) 
    {
        return 1;
    }
    
    return 0;
}

