/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/ossupport/OS.hpp>
#include "testmessageidentifier/TestMessageIdentifier.hpp"

    
static void
usage(std::ostream & os, const std::string &program_name)
{
    os << "Usage:\n"
       << program_name << " -pn -rc -io\n"
       << "\n"
       << "Parameters:\n"
       << "-pn=<port_number>           TCP port to be spied\n"
       << "-rc=<requests_count>        Count of requests sent by client\n"
       << "-io=<is_oneway>             0 - twoway message, 1 - oneway message\n"
       << "-tn=<test case number>      number of test case (for output directory) \n"       
       << std:: endl;
}

int main(int argc, char** argv)
{
    //
    // help argument
    //
    if (argc > 1)
    {
        if(strcmp(argv[1], "--help") == 0)
        {
            usage(std::cout, argv[0]);
            return 0;
        }
    }
    
    
    using Cdmw::Logging::LogManager;
    LogManager::Init(argc, argv);
    Cdmw::TestUtils::TestManager::instance()->init(argc,argv);

    //get parameters from command line and transmit them in test class
    std::string port_number = ""; 
    std::string requests_count = ""; 
    std::string is_oneway = ""; 
    std::string test_number = "";
    
    if( argc > 0 && argv )
    {
        port_number = Cdmw::OsSupport::OS::get_option_value(argc, argv, "-pn");
        if (port_number != "no")
        {
            Cdmw::TestUtils::TestManager::instance()->set_property("port_number", port_number);
        }
        else
        {
            std::cerr << "There is no port_number argument" << std::endl;
            return 1;
        }

        requests_count = Cdmw::OsSupport::OS::get_option_value(argc, argv, "-rc");
        if (requests_count != "no") 
            Cdmw::TestUtils::TestManager::instance()->set_property("requests_count", requests_count);
        else
        {
            std::cerr << "There is no requests_count argument" << std::endl;
            return 1;
        }

        is_oneway = Cdmw::OsSupport::OS::get_option_value(argc, argv, "-io");
        if (is_oneway != "no") 
            Cdmw::TestUtils::TestManager::instance()->set_property("is_oneway", is_oneway);
        else
        {
            std::cerr << "There is no is_oneway argument" << std::endl;
            return 1;
        } 
        
        test_number = Cdmw::OsSupport::OS::get_option_value(argc, argv, "-tn");
        if (test_number != "no") 
            Cdmw::TestUtils::TestManager::instance()->set_property("test_number", test_number);
        else
        {
            std::cerr << "There is no test number argument, taking default value 1" << std::endl;
            Cdmw::TestUtils::TestManager::instance()->set_property("test_number", "1");            
        }
    }
  
    Cdmw::TestUtils::TestManager::instance()->run_tests();
    Cdmw::TestUtils::TestManager::instance()->summary();
    
    return 0;
}



