/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <string>
#include <iostream>
#include <fstream>
#include <sys/stat.h>  //mkdir 


#include "testnetworksniffer/TestNetworkSniffer.hpp"

CPPUNIT_TEST_SUITE_REGISTRATION(TestNetworkSniffer);

#define TEST_NS_MESSAGE_SIZE 100

TestNetworkSniffer::TestNetworkSniffer()
{
    etalon1 = ""; 
    etalon2 = "";
    etalon3 = ""; 
    etalon4 = "";
    
    for (int i = 0; i < TEST_NS_MESSAGE_SIZE; i++)
    {
        etalon1 += '1';
        etalon2 += '2';
        etalon3 += '3';
        etalon4 += '4';
    }  
}
TestNetworkSniffer::~TestNetworkSniffer()   throw()
{
}

namespace
{
    const int SUCCESS = 0;
    const int FAILURE = 1;
}


void TestNetworkSniffer::terminate_sniffer()
{
    Cdmw::Recording::NetworkSniffer::Status status = sniffer->stop () ; 
  
    CPPUNIT_ASSERT_MESSAGE("stopping sniffer error", SUCCESS == status);
  
    status = sniffer->terminate();
    
    CPPUNIT_ASSERT_MESSAGE("terminating sniffer error", SUCCESS == status);
}
bool TestNetworkSniffer::check_sniffer_data(std::string filename, int iteration)
{   
    std::string expected = "";
    switch (iteration) {
        case 1: expected = "2,0,0,0"; break;
        case 2: expected = "2,2,0,0"; break;
        case 3: expected = "0,2,0,0"; break;
        //default:
    }
    int count1 = 0;
    int count2 = 0;
    int count3 = 0;
    int count4 = 0;
    
    int lines_count = 0;
    
    std::string line = "";
    
    std::ifstream im(filename.c_str());
    
    CPPUNIT_ASSERT_MESSAGE("could not open sniffer_data", im);
    
    while(!im.eof())
    {
        getline(im, line);
        
        if(line.find(etalon1, 0) != std::string::npos) count1++;
        if(line.find(etalon2, 0) != std::string::npos) count2++;
        if(line.find(etalon3, 0) != std::string::npos) count3++;
        if(line.find(etalon4, 0) != std::string::npos) count4++;        
        lines_count++;
    } 
    im.close();
    
    if(ostrm) 
        (*ostrm) << "iteration "<< iteration << ": "<< std::endl
             << "found      "<<count1<<","<<count2<<","<<count3<<","<<count4 
             << " messages in " << filename << std::endl 
             << "(expected: "<<expected << ") "<< std::endl ;
             
    std::cout << "iteration "<< iteration << ": "<< std::endl
             << "found      "<<count1<<","<<count2<<","<<count3<<","<<count4 
             << " messages in " << filename << std::endl 
             << "(expected: "<<expected << ") "<< std::endl ;
                           
    // iteration 1: 2 ports are added to the list, 2 messages from each client
    // iteration 2: all 4 ports are added
    // iteration 3: 2 ports are removed 
    switch(iteration)
    {
        case 1: return (count1 == 2 && count2 == 0 && count3 == 0 && count4 == 0) ; 
        case 2: return (count1 == 2 && count2 == 2 && count3 == 0 && count4 == 0) ;       
        case 3: return (count1 == 0 && count2 == 2 && count3 == 0 && count4 == 0) ;       
        default: return true;        
    }
}

void TestNetworkSniffer::get_messages(std::string outputfile, int iteration)
{
    //open filestream for writing messages
    if(ostrm) (*ostrm) << "opening "<< outputfile <<"...\n";
             std::cout << "opening "<< outputfile <<"...\n";
           
    std::ofstream om;
    om.open(outputfile.c_str());
    if(!om) 
    {
        terminate_sniffer();
        store.finalize();
        CPPUNIT_FAIL("testnetworksniffer: error opening output file");
    }
                    
    // Getting messages        
    Cdmw::Recording::Message * message = 0;
    std::string msg;
    bool finish = false;

    if(ostrm) (*ostrm) << "sniffer: getting messages... "<<std::endl ;
             std::cout << "sniffer: getting messages... "<<std::endl ;
    std::cout << "===> iteration " << iteration << " -> on host3 press enter " << std::endl;         
   
    while(!finish)
    {
        message = store.popMessage () ;
        msg = "";

        if(message !=NULL)
        {
            om << message->header.length << "* ";
            for (int i = 0; i < message->header.length+12; i++)
            {
                om<< message->data[i];
                msg += message->data[i] ;
            }
            om << std::endl;
            store.freeMessage (message) ;
    
            if(msg.find("finish_iteration",0)!=std::string::npos)
                finish = true;
        }
    }
  
    if(ostrm) (*ostrm) << "sniffer: finish"<<std::endl ;
             std::cout << "sniffer: finish"<<std::endl ;
    om.close();
    
    // check result
    try
    {
      CPPUNIT_ASSERT(check_sniffer_data(outputfile,iteration));
    }
    catch (...)
    {
      std::cout << "*** iteration " << iteration << " failed" << std::endl;
     }
    
    if(ostrm) (*ostrm) << "sniffer: iteration "<<iteration << " OK " << std::endl ;
             std::cout << "sniffer: iteration "<<iteration << " OK " << std::endl << std::endl;   
}
void TestNetworkSniffer::test_remove_port()
{  
    int port1 = 6786;
    int port2 = 6787;
    int port3 = 6788;
    int port4 = 6789;
     
    std::cout<<"test of removing port in NetworkSniffer" << std::endl; 
     
    sniffer = new Cdmw::Recording::NetworkSniffer(store);

    CPPUNIT_ASSERT_MESSAGE("adding port error", SUCCESS == sniffer->addPort(port1));
    
    CPPUNIT_ASSERT_MESSAGE("adding port error", SUCCESS == sniffer->addPort(port2));
    
    CPPUNIT_ASSERT_MESSAGE("adding port error", SUCCESS == sniffer->addPort(port3));
    
    CPPUNIT_ASSERT_MESSAGE("adding port error", SUCCESS == sniffer->addPort(port4));
      
    CPPUNIT_ASSERT_MESSAGE("running sniffer error", SUCCESS == sniffer->run ());
   
    CPPUNIT_ASSERT_MESSAGE("starting sniffer error", SUCCESS == sniffer->start ());
 
    int num = sniffer->TCPPorts_Num();
    std::cout<< "*** port list : " << std::endl;
    for(int i = 0; i < num; i++)
       std::cout << "  "<<  ((unsigned short *) sniffer->TCPPorts())[i] ;
    std::cout << "** "<< std::endl;
   
    std::cout<< "removing port " << port1 << std::endl; 
    CPPUNIT_ASSERT(SUCCESS == sniffer->removePort(port1)) ;
    //sleep(2);
          
    num = sniffer->TCPPorts_Num();
    std::cout<< "+++ port list : " << std::endl;
    for(int i = 0; i < num; i++)
       std::cout << "  "<<  ((unsigned short *) sniffer->TCPPorts())[i] ;
    std::cout << std::endl;
   
    terminate_sniffer();    
}


void TestNetworkSniffer::do_tests()
{
   
    ostrm = NULL;  
        
    try
    {  
        int port1 = atoi(Cdmw::TestUtils::TestManager::instance()->get_property("p1").c_str());
        int port2 = atoi(Cdmw::TestUtils::TestManager::instance()->get_property("p2").c_str());
        int port3 = atoi(Cdmw::TestUtils::TestManager::instance()->get_property("p3").c_str());
        int port4 = atoi(Cdmw::TestUtils::TestManager::instance()->get_property("p4").c_str());
          
        if (mkdir("V-TRC-012-results", 777) != 0)
        std::cerr << "Error on creating output directory" << std::endl;
    
        char host_name[100] = "";
        gethostname(host_name, sizeof(host_name));
        std::string output_file = std::string("V-TRC-012-results/execution-") 
                + std::string(host_name) + std::string(".output");
                       
        ostrm = new std::ofstream(output_file.c_str());

        if(!ostrm || !(*ostrm) ) 
            { 
              std::cerr << "## Error on opening output file " << output_file << std::endl;
              ostrm = NULL;  
            }
             (*ostrm) << "sniffer: initializing...\n";
      
        sniffer = new Cdmw::Recording::NetworkSniffer(store);

        CPPUNIT_ASSERT_MESSAGE("adding port error", SUCCESS == sniffer->addPort(port1));
    
        CPPUNIT_ASSERT_MESSAGE("adding port error", SUCCESS == sniffer->addPort(port3));
    
        CPPUNIT_ASSERT_MESSAGE("running sniffer error", SUCCESS == sniffer->run ());
   
        CPPUNIT_ASSERT_MESSAGE("starting sniffer error", SUCCESS == sniffer->start ());
      
              
        if(ostrm) (*ostrm) << "sniffer: added ports "<<port1<<", "<<port3 << std::endl ;
                 std::cout << "sniffer: added ports "<<port1<<", "<<port3 << std::endl ;
                 
        // get messages till "finish_iteration"
        // write them to file, check result.       
        get_messages("sniffer_data_1", 1); 

        // iteration 2    
        sniffer->stop();   
        CPPUNIT_ASSERT(SUCCESS ==  sniffer->addPort(port2));
        CPPUNIT_ASSERT(SUCCESS ==  sniffer->addPort(port4));
        sniffer->start(); 

        if(ostrm) (*ostrm) << "sniffer: added ports "<<port2<<", "<<port4 << std::endl ;
                 std::cout << "sniffer: added ports "<<port2<<", "<<port4 << std::endl ;
        
        int num = sniffer->TCPPorts_Num();
        std::cout<< "*** port list : " ;
        for(int i = 0; i < num; i++)
         std::cout << "  "<<  ((unsigned short *) sniffer->TCPPorts())[i] ;
        std::cout << " *** "<< std::endl;
        
        get_messages("sniffer_data_2", 2);
  
        // iteration 3 
        sniffer->stop();     
        try
        {
           CPPUNIT_ASSERT(SUCCESS == sniffer->removePort(port1));
           num = sniffer->TCPPorts_Num();
           std::cout<< "*** port list : " ;
           for(int i = 0; i < num; i++)
           std::cout << "  "<<  ((unsigned short *) sniffer->TCPPorts())[i] ;
           std::cout << " *** "<< std::endl;
        }
        catch (...)
        {
           std::cout << "*** iteration 3" << " removing port " << port1 << " failed" << std::endl;
        }
        try
        {
           CPPUNIT_ASSERT(SUCCESS == sniffer->removePort(port3));
           num = sniffer->TCPPorts_Num();
           std::cout<< "*** port list : " ;
           for(int i = 0; i < num; i++)
           std::cout << "  "<<  ((unsigned short *) sniffer->TCPPorts())[i] ;
           std::cout << " *** "<< std::endl;
        }
        catch (...)
        {
           std::cout << "*** iteration 3" << " removing port " << port3 << " failed" << std::endl;
        }
        sniffer->start(); 
                             
        if(ostrm) (*ostrm) << "sniffer: removed ports "<<port1<<", "<<port3 << std::endl ;
                 std::cout << "sniffer: removed ports "<<port1<<", "<<port3 << std::endl ;
      
        num = sniffer->TCPPorts_Num();
        std::cout<< "*** port list : " ;
        for(int i = 0; i < num; i++)
         std::cout << "  "<<  ((unsigned short *) sniffer->TCPPorts())[i] ;
        std::cout << " *** "<< std::endl;

        get_messages("sniffer_data_3", 3);
        
        terminate_sniffer();
        store.finalize();
        
        remove("sniffer_data_1");
        remove("sniffer_data_2");
        remove("sniffer_data_3");
        
    }    
    catch(const std::exception &e)
    {
        std::cerr << "Test NetworkSniffer: exception raised : " << std::endl
                  << e.what() << std::endl;
                  throw;
    }
    catch(...)
    {
        std::cerr << "Test NetworkSniffer: exception" << std::endl;
        throw;
    }       
}
