/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <iomanip>
#include <fstream>
#include <sys/stat.h>
#include <sys/types.h>

#include "testnetworksniffer/transmission.stub.hpp"

using namespace std;

int main(int argc, char * argv[])
{
    // Check count of arguments
    if (argc < 3)
    {
        cout << "CORBA client: too few arguments" << endl;
        cout << "usage:  <host1> <host2> <finish_iteration(0/1)> " << endl; 
        throw 0;
    }

    //get parameters
    bool finish = (atoi(argv[3]) == 1) ? true : false;
 
    try 
    {
        // Initialize orb
        CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);
        
        std::string server_host1 = std::string(argv[1]);             
        std::string corbaloc1 = std::string ("corbaloc::") 
                             + server_host1 + std::string(":6786/Transmission1" );     
        std::cout << "corbaloc1 name: " << corbaloc1 << std::endl ;
        CORBA::Object_var obj1 = orb->string_to_object(corbaloc1.c_str()); 
    
        if (CORBA::is_nil(obj1)) 
        {
            cout << "Nil reference (1)" << endl; throw 0;
        }    
        // Narrow
        Transmission_var tv1 = Transmission::_narrow(obj1);
        if (CORBA::is_nil(tv1)) 
        {
            cout << "not a Transmission reference (1)" << endl;
            throw 0;
        }
                   
        std::string corbaloc2 = std::string ("corbaloc::") 
                             + server_host1 + std::string(":6787/Transmission2" );     
        std::cout << "corbaloc2 name: " << corbaloc2 << std::endl ;
        CORBA::Object_var obj2 = orb->string_to_object(corbaloc2.c_str()); 
        if (CORBA::is_nil(obj2)) 
        {
            cout << "Nil reference (2)" << endl; throw 0;
        }    
        // Narrow
        Transmission_var tv2 = Transmission::_narrow(obj2);
        if (CORBA::is_nil(tv2)) 
        {
            cout << "not a Transmission reference (2)" << endl;
            throw 0;
        }
        
        std::string server_host2 = std::string(argv[2]);             
        std::string corbaloc3 = std::string ("corbaloc::") 
                             + server_host2 + std::string(":6788/Transmission3" );     
        std::cout << "corbaloc3 name: " << corbaloc3 << std::endl ;
        CORBA::Object_var obj3 = orb->string_to_object(corbaloc3.c_str()); 
        if (CORBA::is_nil(obj3)) 
        {
            cout << "Nil reference (3)" << endl;  throw 0;
        }    
        // Narrow
        Transmission_var tv3 = Transmission::_narrow(obj3);
        if (CORBA::is_nil(tv3)) 
        {
            cout << "not a Transmission reference (3)" << endl;
            throw 0;
        }
                       
        std::string corbaloc4 = std::string ("corbaloc::") 
                             + server_host2 + std::string(":6789/Transmission4" );     
        std::cout << "corbaloc4 name: " << corbaloc4 << std::endl ;
        CORBA::Object_var obj4 = orb->string_to_object(corbaloc4.c_str()); 
        if (CORBA::is_nil(obj4)) 
        {
            cout << "Nil reference (4)" << endl; throw 0;
        }    
        // Narrow
        Transmission_var tv4 = Transmission::_narrow(obj4);
        if (CORBA::is_nil(tv4)) 
        {
            cout << "not a Transmission reference (4)" << endl;
            throw 0;
        }
                
        //=============================================================
      
        if (mkdir("V-TRC-012-results", 777) != 0)
        std::cerr << "could not create output directory V-TRC-012-results" << std::endl;
        char host_name[100] = "";
        gethostname(host_name, sizeof(host_name));
        std::string output_file = std::string("V-TRC-012-results/execution-") 
                + std::string(host_name) + std::string(".output");       
        std::ofstream ostrm(output_file.c_str(),std::ios_base::app);
        
        if(ostrm) ostrm <<" sending messages .." << std::endl;
                  cout  <<" sending messages .." << std::endl;                    

        int size = 100;
        int count = 1;

        // Send data
        for (int i = 1; i <= count; i++)
        {
            char data1[size];
            char data2[size];
            char data3[size];
            char data4[size];
            
            for (int j = 0; j <= size-1; j++)
            {
                data1[j] = '1'; 
                data2[j] = '2';
                data3[j] = '3';
                data4[j] = '4';
            }
            data1[size] = '\0'; 
            data2[size] = '\0';
            data3[size] = '\0';
            data4[size] = '\0';
            
            tv1->send(data1, 1);
            tv2->send(data2, 2);
            tv3->send(data3, 3);
            tv4->send(data4, 4);
            
            if(finish) 
            { 
                if(ostrm) ostrm <<"sending finish_iteration"<< endl; 
                cout            <<"sending finish_iteration"<< endl; 
                
                tv1->finish_iteration(); 
                tv2->finish_iteration();                
            }
        }
        if(ostrm) ostrm <<"ok"<< endl; 
                  cout  <<"ok"<< endl; 
        
    }
    catch (const CORBA::Exception & ex) 
    {
        cout << "CORBA client: CORBA exception"<<ex << endl;
        return 1;
    }
    catch (...) 
    {
        cout << "CORBA client: Uncaught exception" << endl;
        return 1;
    }
    
    return 0;
}
