/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "Foundation/logging/LogManager.hpp"
#include "Foundation/logging/Logger.hpp"
#include <Foundation/ossupport/OS.hpp>
#include "testnetworksniffer/TestNetworkSniffer.hpp"

    
static void
usage(std::ostream & os, const std::string &program_name)
{
    os << "Usage:\n"
       << program_name << " p1 p2 p3 p4 \n"
       << "\n"
       << "Parameters:\n"
       << "-p1=<port_number>  \n"
       << "-p2=<port_number>  \n"
       << "-p3=<port_number>  \n"       
       << "-p4=<port_number>  -  TCP ports to be spied\n"
       << std:: endl;
}

int main(int argc, char** argv)
{
    using Cdmw::Logging::LogManager;
    LogManager::Init(argc, argv);
        
    // help argument
    if (argc > 1)
    {
        if(strcmp(argv[1], "--help") == 0)
        {
            usage(std::cout, argv[0]);
            return 0;
        }
    }
    
    if( argc < 4 )
    {    
        usage(std::cout, argv[0]);
        return 1;
    }

    //get parameters from command line and transmit them in test class
    std::string port1 = ""; 
    std::string port2 = ""; 
    std::string port3 = ""; 
    std::string port4 = ""; 
     
    if( argc > 0 && argv )
    {
        port1 = Cdmw::OsSupport::OS::get_option_value(argc,argv,"-p1");
        if (port1 != "no")
            Cdmw::TestUtils::TestManager::instance()->set_property("p1", port1);
        else
        {
            std::cerr << "there is no p1 argument" << std::endl;
            return 1;
        }
        port2 = Cdmw::OsSupport::OS::get_option_value(argc,argv,"-p2");
        if (port2 != "no")
            Cdmw::TestUtils::TestManager::instance()->set_property("p2", port2);
        else
        {
            std::cerr << "there is no p2 argument" << std::endl;
            return 1;
        }
        port3 = Cdmw::OsSupport::OS::get_option_value(argc,argv,"-p3");
        if (port3 != "no")
            Cdmw::TestUtils::TestManager::instance()->set_property("p3", port3);
        else
        {
            std::cerr << "there is no p3 argument" << std::endl;
            return 1;
        }
        port4 = Cdmw::OsSupport::OS::get_option_value(argc,argv,"-p4");
        if (port4 != "no")
            Cdmw::TestUtils::TestManager::instance()->set_property("p4", port4);
        else
        {
            std::cerr << "there is no p4 argument" << std::endl;
            return 1;
        }
    }
    
    Cdmw::TestUtils::TestManager::instance()->init(argc,argv);
    Cdmw::TestUtils::TestManager::instance()->run_tests();
    Cdmw::TestUtils::TestManager::instance()->summary();
    
    return 0;
}



