/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "testoperationmap/TestOperationMap.hpp"

#include <cstdlib>
#include <string>
#include <iostream>
#include <fstream>
#include <sys/types.h>
#include <sys/stat.h>

/**
*Implementation notes: [if no pertinent write none ]
*<p>
*Portability issues: [if no pertinent write none ]
*<p>
*/

CPPUNIT_TEST_SUITE_REGISTRATION( TestOperationMap );

TestOperationMap::TestOperationMap()
{
}


TestOperationMap::~TestOperationMap()
    throw()
{
}

std::multiset<std::string> TestOperationMap::oper_set = std::multiset<std::string>() ;

void TestOperationMap::save (std::string const & str1, std::size_t i)
{ 
  oper_set.insert(str1);  
}

void TestOperationMap::create_folder()
{
    //set output folder
   output_folder = "V-TRC-009_005-results";
    
    //create output folder
    if (mkdir(output_folder.c_str(), 777) != 0)
    {
        std::cerr << "Error creating output directory" << std::endl;
    }
}

void TestOperationMap::create_file(std::string filename, const char * data)
{
    FILE * pFile;
    pFile = fopen (filename.c_str(), "w");
    fprintf (pFile, "%s\n", data);
    fclose (pFile); 
} 

void TestOperationMap::print_result(bool result)
{
    //set output file
    char host_name[100] = "";
    gethostname(host_name, sizeof(host_name));
    std::string output_file = output_folder + "/execution-" + host_name + ".output";
    
    //output test result
    FILE * pFile;
    pFile = fopen (output_file.c_str(),"w");
    if (result)
        fprintf (pFile, "%s\n", "The test scenario is OK");
    else
        fprintf (pFile, "%s\n", "The test scenario is FAILED");
           
    fclose (pFile);     
}

void 
TestOperationMap::do_tests()
{
    //create output folder
    create_folder();
    print_result(false);
    
    Cdmw::Recording::OperationMap::Status status ;  
    CdmwRecording::String_List oper_list;  

    std::string oper1 = "operation1" ;   // in config file
    std::string oper2 = "finish_iteration2" ;
    std::string oper3 = "ready3" ; 
    std::string oper4 = "operation4" ;   // not in config file
    std::string oper5 = "fly5" ;
    std::string oper6 = "copy6" ;

    TEST_INFO("Testing operation map"); 

    //======= 1 initialising: 1,2,3

    create_file("operations.txt", "operation1\nfinish_iteration2\nready3");
    Cdmw::Recording::OperationMap oper_map ("operations.txt") ;

    CPPUNIT_ASSERT(true == oper_map.checkOperation (oper1) );
    CPPUNIT_ASSERT(true == oper_map.checkOperation (oper2) );
    CPPUNIT_ASSERT(true == oper_map.checkOperation (oper3) );
    CPPUNIT_ASSERT(false == oper_map.checkOperation (oper4) );

    // Ouput the operation map 
    //std::cout << " *** initialisation 1,2,3 "<< std::endl ;
    //oper_map.mapLoop(print);
  
    //======== 2 removing operations 1,4   ->  2,3 

    oper_list.length(2);
    oper_list[0] = oper1.c_str() ;  //     in the list
    oper_list[1] = oper4.c_str() ;  // not in the list
    status = oper_map.removeOperations(oper_list);

    CPPUNIT_ASSERT(0 == status);     
    CPPUNIT_ASSERT(false == oper_map.checkOperation (oper1) );
    CPPUNIT_ASSERT(false == oper_map.checkOperation (oper4) );
    CPPUNIT_ASSERT(true == oper_map.checkOperation (oper2) );
    CPPUNIT_ASSERT(true == oper_map.checkOperation (oper3) );

    // Ouput the operation map 
    //std::cout << " *** removed operations 1,4 "<< std::endl ;
    //oper_map.mapLoop(print);

    //======== 3 adding operations 1,2,5  ->  1,2,3,5

    oper_list.length(3);
    oper_list[0] = oper1.c_str() ; // not in the list
    oper_list[1] = oper2.c_str() ; //     in the list
    oper_list[2] = oper5.c_str() ; // not in the list

    status = oper_map.addOperations(oper_list);
    CPPUNIT_ASSERT(0 == status);  

    CPPUNIT_ASSERT(true == oper_map.checkOperation (oper1) );
    CPPUNIT_ASSERT(true == oper_map.checkOperation (oper2) );
    CPPUNIT_ASSERT(true == oper_map.checkOperation (oper5) );

    // check that oper2 is only once in the list:

    oper_map.mapLoop(save);
    CPPUNIT_ASSERT(1 == oper_set.count(oper2));
    oper_set.clear();
 
    // Ouput the operation map in file
    //std::cout << " *** added operations 1,2,5 "<< std::endl ;
    //oper_map.mapLoop(print);

    //======== 4 adding operations with spaces 4,6 ->  1,2,3,4,5,6

    oper_list.length(2);
    oper_list[0] = "  operation4" ;
    oper_list[1] = "copy6    " ;

    status = oper_map.addOperations(oper_list);
    CPPUNIT_ASSERT(0 == status);

    CPPUNIT_ASSERT(true == oper_map.checkOperation (oper4) );
    CPPUNIT_ASSERT(true == oper_map.checkOperation (oper6) );
   
    // Ouput the operation map in file
    //std::cout << " *** added operations 4,6 with spaces "<< std::endl ;
    //oper_map.mapLoop(print);
   

    print_result(true);
}


