/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <cstdlib>
#include <cstdio>
#include <string>
#include <iostream>
#include <fstream>

#include <semaphore.h>
#include <sys/stat.h>   //S_IRUSR|S_IWUSR
#include <fcntl.h>  //O_CREAT 
    

#include "recordingtools/CdmwRecordingContainerManager_impl.hpp"
#include "recordingtools/CdmwRecordingDaemonControl_impl.hpp"

#include "testrecordingservers/TestRecordingServers.hpp"
#include "testrecordingservers/Recorder_impl.hpp"

#include <Foundation/ossupport/OS.hpp>

namespace
{
    const int SUCCESS = 0;
    const int FAILURE = 1;
}
 
CPPUNIT_TEST_SUITE_REGISTRATION( TestRecordingServers );

TestRecordingServers::TestRecordingServers()
{
}
TestRecordingServers::~TestRecordingServers()
    throw()
{
}

void 
TestRecordingServers::do_tests()
{
    check();
}

int TestRecordingServers::check()
{
    int result= SUCCESS;
        
    TEST_INFO("test recording servers");
           
    try
    {
                  
    //create semaphore     
    sem_t * sema1 = sem_open("/testrecs_semaphore1",O_CREAT,S_IRUSR|S_IWUSR, 0); 
    sem_init(sema1, 0, 0);

    sem_t * sema2 = sem_open("/testrecs_semaphore2",O_CREAT,S_IRUSR|S_IWUSR, 0); 
    sem_init(sema2, 0, 0);
    
        
    std::string nameServicePort = 
      Cdmw::TestUtils::CORBATestManager::instance()->get_property("NameServicePort",
                                                                  "5010");
    std::string host2 = 
      Cdmw::TestUtils::CORBATestManager::instance()->get_property("host2");                                                               
    std::string host3 = 
      Cdmw::TestUtils::CORBATestManager::instance()->get_property("host3");                                                               
                                                                  

    std::string corbaloc = std::string ("corbaname::localhost:");
    corbaloc += nameServicePort + "/NameService#" + CdmwRecording::REC_DAEMON_CTRL_ID ;
    
    CORBA::Object_var o_daemon = 
       Cdmw::TestUtils::CORBATestManager::instance()->get_ORB()->string_to_object (corbaloc.c_str());
    CdmwRecording::DaemonControl_var daemon = CdmwRecording::DaemonControl::_narrow(o_daemon);

    if(CORBA::is_nil(daemon.in()))
    {
        std::cout << "TestRecordingServers: ## could not find daemon " << corbaloc << std::endl;
        CPPUNIT_ASSERT(false);    
    }
    else
    {  
        std::string container_mgr_corbaloc = std::string ("corbaname::localhost:")
             +  nameServicePort + "/NameService#" + CdmwRecording::REC_CONTAINER_MNGR_ID;
             
        CORBA::Object_var o_container = 
         Cdmw::TestUtils::CORBATestManager::instance()->get_ORB()->string_to_object (container_mgr_corbaloc.c_str());
        CPPUNIT_ASSERT(!CORBA::is_nil(o_container.in()));

        std::cout << "TestRecordingServers: >> found daemon " << corbaloc << std::endl ;

        CdmwRecording::String_List  ilist  ;
        ilist.length(1);
        ilist[0] = "IDL:Transmission:1.0";
        daemon->remove_all_interfaces ();
        daemon->add_interfaces(ilist); 

        
        CdmwRecording::String_List  olist  ;
        olist.length(1);
        olist[0] = "send";
        daemon->remove_all_operations ();
        daemon->add_operations(olist); 

        std::cout << "                      >> setting urls of recording servers" << std::endl ;

        std::string url1 = "corbaloc:iiop:1.2@" + host2 + ":12344/Recorder1" ;
        std::string url2 = "corbaloc:iiop:1.2@" + host3 + ":12344/Recorder2" ;

        daemon->setRecorderUrl(1, NULL);
        daemon->setRecorderUrl(2, url1.c_str());       
        daemon->setRecorderUrl(3, url2.c_str());

        
        CORBA::String_var url_str ;
        for( unsigned int i=0 ; i< CdmwRecording::REC_DAEMON_MAX_RECORDERS; i++)
        {
           if ( daemon->state()->recorders_urls[i] != NULL)
           {
              url_str = CORBA::String_var(daemon->state()->recorders_urls[i]);
              std::cout << "---> recording Server URL ["<< i <<"] = " << url_str << std::endl;
           }
           else std::cout << "---> recording Server URL ["<< i <<"] = 0 " << std::endl;
        }
        
        std::cout << "TestRecordingServers: >> waiting for the client " <<  std::endl;
        std::cout << "                      -> on host2 press enter " <<  std::endl;
        
        Cdmw::OsSupport::OS::sleep (2000);   
        
        // wait for semaphore 1 from server 
        sem_wait(sema1);
        
        Cdmw::OsSupport::OS::sleep (1000);   
        
        /*  Test step 2 : check that plugin is not active when url is set to NULL
         * 
        std::cout << "TestRecordingServers: >> setRecorderUrl(2,NULL) " <<  std::endl;
        daemon->setRecorderUrl(2, NULL );   
        
        for( unsigned int i=0 ; i< CdmwRecording::REC_DAEMON_MAX_RECORDERS; i++)
        {
           if ( daemon->state()->recorders_urls[i] != NULL)
           {
              url_str = CORBA::String_var(daemon->state()->recorders_urls[i]);
              std::cout << "---> recording Server URL ["<< i <<"] = " << url_str << std::endl;
           }
           else std::cout << "---> recording Server URL ["<< i <<"] = 0 " << std::endl;
        } */ 

        // set semaphore 2 for the server
        sem_post(sema2);
        
        std::cout << "TestRecordingServers: >> waiting for the client (2)" <<  std::endl;
              
        // wait for semaphore 1 from server
        sem_wait(sema1);
        sem_post(sema2);

        daemon->stop ();
        std::cout << "TestRecordingServers: >> stop " << std::endl;
        daemon->remove_all_interfaces ();
        std::cout << "TestRecordingServers: >> remove_all_interfaces " << std::endl;
        daemon->remove_all_operations ();
        std::cout << "TestRecordingServers: >> remove_all_operations " << std::endl;
        daemon->terminate();
        std::cout << "TestRecordingServers: >> terminate "  << std::endl;
        CPPUNIT_ASSERT(true);    
    }
    
        std::cout << "TestRecordingServers: >> test finished " << std::endl;
    }    
    catch(const std::exception &e)
    {
        std::cerr << "TestRecordingServers: exception raised : " << std::endl
                  << e.what() << std::endl;
                  CPPUNIT_ASSERT(false);
                  //throw;
    }
    catch(...)
    {
        std::cerr << "TestRecordingServers: exception" << std::endl;
        CPPUNIT_ASSERT(false);
        //throw;
    }       
    return result;
}
