/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <fstream>
#include <iomanip>
#include <Foundation/ossupport/OS.hpp>

#include "testrecordingservers/Transmission.stub.hpp"

using namespace std;

void send_messages(int size, int count, Transmission_var tm)
{
 
    for (int i = 1; i <= count; i++)
    {
        char data[size];
        for (int j = 0; j <= size-1; j++)
            data[j] = 0xCF;
        data[size] = '\0';

        tm->send(data);     
    } 
}

int get_recordingserver_received_pairs()
{
     //set output folder
    std::string output_folder = "V-TRC-031-results";
    
    int pairs = 0;
    int len = 0;
    string line = "";
    int count = 0;
      
    ifstream inFile;
    inFile.open("recordingserver_data_1");
    
    while(!inFile.is_open() && count < 3)
    {
      std::cout<<"Opening recordingserver_data_1 ... retry ..."<<std::endl;
      Cdmw::OsSupport::OS::sleep (1000);
      count ++;
      inFile.open("recordingserver_data_1");
    }
    if(!inFile.is_open()) 
    {
        cout<<"## could not open recordingserver_data_1 ## "<<endl;
        return pairs;
    }
    std::cout<<"opened recordingserver_data_1, waiting 3 sec ... "<<std::endl;
    
    // wait for Recording Server to flash data
    Cdmw::OsSupport::OS::sleep (3000);
      
    while(inFile >> line) 
    {
        if(line.find("buffer",0)!= std::string::npos) 
        {
           if(inFile >> line)
           {
                if(line.find("hostname",0)!= std::string::npos)  inFile >> line ;               
           } 
           if(inFile >> line)
           {
                if(line.find("num_pairs",0)!= std::string::npos)  
                {
                    inFile >> line ; 
                    pairs += atoi(line.c_str());
                } 
           } 
           if(inFile >> line)
           {
             if(line.find("length",0)!= std::string::npos)  
                {
                    inFile >> line ; 
                    len += atoi(line.c_str());
                    std::cout<< " * buffer length: "<< line << std::endl ;
                }
           }    
        }       
    }
    inFile.close();
    std::cout << " common buffer length: "<< len << std::endl;
  
    //move result file
    std::string file = output_folder + "/recordingserver_data_1";
    rename("recordingserver_data_1", file.c_str());
    
    return pairs; 
}

void create_folder()
{
    //set output folder
   std::string output_folder = "V-TRC-031-results";
    
    //create output folder
    if (mkdir(output_folder.c_str(), 777) != 0)
    {
        std::cerr << "  Error creating output directory" << std::endl;
    }
    
    //set output file
    char host_name[100] = "";
    gethostname(host_name, sizeof(host_name));
    std::string output_file = output_folder + "/execution-" + host_name + ".output";
    
    remove(output_file.c_str());    
}

void print_result(bool result, std::string context, 
                            int received, std::string expected)
{
     //set output folder
   std::string output_folder = "V-TRC-031-results";
    
    //set output file
    char host_name[100] = "";
    gethostname(host_name, sizeof(host_name));
    std::string output_file = output_folder + "/execution-" + host_name + ".output";
    
    std::string str_result = (result) ?  "Test OK" : "Test FAILED" ;
    
    //output test result
    FILE * pFile;
    pFile = fopen (output_file.c_str(),"a+");
    
    fprintf (pFile, "%s : received %d pairs, expected %s \n", 
                    context.c_str(), received, expected.c_str());
    fprintf (pFile, "%s\n", str_result.c_str() );
 
    std::cout << str_result << std::endl ;
           
    fclose (pFile);     
}


  
int main(int argc, char * argv[])
{
    // Check count of arguments 
    if (argc < 1)
    {
        cout << "CORBA client: too few arguments" << endl;
        cout << "usage: <host> - name of the host where the server is running" << endl;
        throw 0;
    }
  
    create_folder();
  
    try 
    {
        // Initialize orb
        CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);
                          
        std::string server_host = std::string(argv[1]);             
        std::string corbaloc = std::string ("corbaloc:iiop:1.2@") 
                             + server_host + std::string(":6789/Transmission" );     
        CORBA::Object_var obj = orb->string_to_object(corbaloc.c_str());
        
        if (CORBA::is_nil(obj)) 
        {
            cout << "testrecs_client: Nil reference" << endl;
            throw 0;
        }
    
        // Narrow
        Transmission_var tm = Transmission::_narrow(obj);
        if (CORBA::is_nil(tm)) 
        {
            cout << "testrecs_client: not a Transmission reference" << endl;
            throw 0;
        }
       
        std::cout << "testrecs_client: 1 sending a message.. " << std::endl;    
        send_messages(1, 1,tm);
        
        // tell the test process to start the next step
        tm->ready();
        
        std::cout << "testrecs_client: 2 sending a message.. " << std::endl;    
        send_messages(1, 1,tm);
     
        Cdmw::OsSupport::OS::sleep (3000);   
     
        // tell the test process to stop
        tm->ready();
     
        tm->terminate(); // stop the server
        
        // Count data in recordingserver_data file
        int p = get_recordingserver_received_pairs();
          
        // for test step 2 should be changed (1 pair expected) 
        std::cout<<"=== "<<p<< " pairs found (2 expected) "<<std::endl;
        print_result(p==2, "recording server1", p,"2" );  
        //std::cout<<"=== "<<p<< " pairs found (1 expected) "<<std::endl;
        //print_result(p==1, "recording server1", p,"1" );            
    }
    catch (const CORBA::Exception & ex ) 
    {
        cout << "CORBA client: Uncaught CORBA exception" << ex << endl;
        return 1;
    }
    catch (...) 
    {
        return 1;
    }
    
    return 0;
}

