/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
#include <Foundation/ossupport/OS.hpp>

#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/orbsupport/CORBASystemExceptionWrapper.hpp"

#include "testrecordingservers/TestRecordingServers.hpp"
    
using namespace Cdmw::OsSupport;    
using namespace std;
    
static void
usage(std::ostream & os, const std::string &program_name)
{
    os << "Usage:\n"
       << program_name << " --host2=<hostname> --host3=<hostname>   \n"
       << "\n"
       << "Parameters:\n"
       << "--host2=<hostname>     host2 where recording server 1 is running \n"
       << "--host3=<hostname>     host3 where recording server 2 is running \n"
       << std:: endl;
}

int main(int argc, char** argv)
{
  
    const string NAMING_SERVICE_PORT_OPTION  = "--NamePort";
 
    try
    {
    
    string host2;
    string host3;
    //get parameters from command line and transmit them in test class
    if( argc > 0 && argv )
    {
        host2 = Cdmw::OsSupport::OS::get_option_value(argc, argv, "--host2");
        if (host2 != "no")
        {
            Cdmw::TestUtils::CORBATestManager::instance()->set_property("host2", host2);
        }
        else
        {
            std::cerr << "There is no host2 argument" << std::endl;
            usage(std::cout, argv[0]);
            return 1;
        }
        host3 = Cdmw::OsSupport::OS::get_option_value(argc, argv, "--host3");
        if (host3 != "no")
        {
            Cdmw::TestUtils::CORBATestManager::instance()->set_property("host3", host3);
        }
        else
        {
            std::cerr << "There is no host3 argument" << std::endl;
            usage(std::cout, argv[0]);
            return 1;
        }                    
    } 
    
    // get Name Service port
    string nameServicePort = OS::get_option_value (argc, argv, NAMING_SERVICE_PORT_OPTION);

    if (nameServicePort  == "no")
    {
        nameServicePort = "5010";
    }
     
    // Starting the Name Service 
    string namingservice="Naming_Service";
    string ns_args = "-ORBEndpoint iiop://localhost:" + nameServicePort ;
    
    cout << "starting Naming_Service " << ns_args << endl;
    OS::ProcessId idNaming = OS::create_process( namingservice, ns_args);

    OS::sleep(5000);

    cout << "starting recording daemon " << endl;   
    string rec_daemon = "cdmw_recordingdaemon";    
    string rec_daemon_args = string("--CdmwLogLevel=CDMW_DEBUG_1 --debug_file=/tmp/recording.log --recorder_url=corbaloc::doku:12345/FOO")   
      + string(" --CdmwProcessPort=59876 --bind_to_ns -ORBInitRef NameService=corbaloc::localhost:") 
      + nameServicePort + string("/NameService") ;
    cout << rec_daemon_args << endl;
    OS::ProcessId id_Recd = OS::create_process(rec_daemon, rec_daemon_args );
    cout << "recording daemon started, pid=" << id_Recd << std::endl;
    OS::sleep (2000);
    
    
     // Initialize the ORB
    cout << "initialising the orb.. " << endl;   
    Cdmw::OrbSupport::StrategyList orb_strategies;
    orb_strategies.add_OrbThreaded();
    orb_strategies.add_PoaThreadPool(10);

    CORBA::ORB_var  orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, orb_strategies);
          
    // Get reference to Root POA.
    CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
    PortableServer::POA_var root_poa = PortableServer::POA::_narrow(obj);
     
    Cdmw::TestUtils::CORBATestManager::instance()->init(orb.in(), root_poa.in(), argc, argv);
    Cdmw::TestUtils::CORBATestManager::instance()->set_property("NameServicePort", nameServicePort);

    cout << "run_tests " << endl;  
    Cdmw::TestUtils::CORBATestManager::instance()->run_tests();
         
    cout << "summary " << endl;  
    Cdmw::TestUtils::CORBATestManager::instance()->summary();
        
    // And we kill launched process
    cout << "kill naming service to finish test " << endl;
    OS::kill_process(idNaming);
    
    } 
    catch (const Cdmw::OrbSupport::CORBASystemExceptionWrapper & ex)
    {
        std::cerr << "+++ 1 +++ " << ex.what() << std::endl;
        std::cerr << " **** TEST FAILED AT LINE " << __LINE__
                      << " IN FILE: " << __FILE__ << std::endl;
    }
    catch (const Cdmw::Exception & ex)
    {
        std::cerr << "+++ 2 +++ " << ex.what() << std::endl;
        std::cerr << " **** TEST FAILED AT LINE " << __LINE__
                      << " IN FILE: " << __FILE__ << std::endl;
    } 
    catch(const CORBA::Exception& ex)  
    {        
        std::cerr << "+++ 3 +++ " << ex << std::endl;
        std::cerr << " **** TEST FAILED AT LINE " << __LINE__
                      << " IN FILE: " << __FILE__ << std::endl;
    }
    catch(...)  
    {        
        std::cerr << "unexpected exception" << std::endl;
        std::cerr << " **** TEST FAILED AT LINE " << __LINE__
                      << " IN FILE: " << __FILE__ << std::endl;
    }
    
    return 0;
}



