/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <iomanip>
#include "testsniffer2/transmission.stub.hpp"

using namespace std;

int main(int argc, char * argv[])
{
    if (argc < 2)
    {
        cout << "CORBA client: too few arguments" << endl;
        throw 0;
    }
    std::string port = argv[2];
    std::string msg_Y = "YYYYYYYYYY";
    std::string msg_N = "NNNNNNNNNN";
    int testnr = 1;

    try 
    {
        // Initialize orb
        CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);

        // Destringify argv[1]
        CORBA::Object_var obj = orb->string_to_object(argv[1]);
        if (CORBA::is_nil(obj)) 
        {
            cout << "CORBA client: Nil reference" << endl;
            throw 0;
        }
    
        // Narrow
        Transmission_var tv = Transmission::_narrow(obj);
        if (CORBA::is_nil(tv)) 
        {
            cout << "CORBA client: Argument is not a Transmission reference" 
            << endl;
            throw 0;
        }
 
        //============================================
        // test1
 
        std::cout<<"test "<<testnr<<": sending ready 0.."<<std::endl ;   

        // wait server ready (start and run sniffer)
        if (!tv->snf_ready(0, port.c_str()))  return 1;

        std::cout<<"sending message 1"<<std::endl ;    

        // Send data
        tv->send(msg_N.c_str());
       
        std::cout<<"sending ready 1.."<<std::endl ;
                
        // wait server ready (sniffer add port)
        if (!tv->snf_ready(testnr, ""))  return 1;
       
        std::cout<<"sending message 2"<<std::endl; 

        // Send data
        tv->send(msg_Y.c_str());
        
        std::cout<<"finish iteration"<< std::endl;

        tv->finish_iteration(testnr);
        testnr++;

        //=============================================
        // test 2 - 4
 
        for(; testnr <=4; testnr++)
        { 
           std::cout<<"test "<<testnr<<": sending ready..";   

           // wait server ready 
           if (!tv->snf_ready(testnr, port.c_str()))  return 1;

           std::cout<<"sending message"<<std::endl ;  

           // Send data
           tv->send(msg_Y.c_str());
        
           std::cout<<"finish iteration"<<std::endl;

           tv->finish_iteration(testnr);         
        }

        //=============================================
        // test 5
 
        std::cout<<"test "<<testnr<<": sending ready..";   

        // wait server ready (stop sniffer)
        if (!tv->snf_ready(testnr, port.c_str()))  return 1;

        std::cout<<"sending message 1"<<std::endl ;  

        // Send data
        tv->send(msg_N.c_str());

        std::cout<<"sending ready"<<std::endl ;
                
        // wait server ready (start sniffer)
        if (!tv->snf_ready(testnr, ""))  return 1;
       
        std::cout<<"sending message 2"<<std::endl; 

        // Send data
        tv->send(msg_Y.c_str());
        
        std::cout<<"finish iteration"<<std::endl;

        tv->finish_iteration(testnr);
        testnr++;
        
        //=============================================
        // test 6 - 8
 
        for(; testnr <=8; testnr++)
        { 
           std::cout<<"test "<<testnr<<": sending ready..";   

           // wait server ready 
           if (!tv->snf_ready(testnr, port.c_str()))  return 1;

           std::cout<<"sending message"<<std::endl ;  

           // Send data
           tv->send(msg_Y.c_str());
        
           std::cout<<"finish iteration"<<std::endl;

           tv->finish_iteration(testnr);         
        }

        //=============================================
        // test 9
 
        std::cout<<"test "<<testnr<<": sending ready..";   

        // wait server ready (stop sniffer)
        if (!tv->snf_ready(testnr, port.c_str()))  return 1;

        std::cout<<"sending message"<<std::endl ;  

        // Send data
        tv->send(msg_N.c_str());

        std::cout<<"sending ready"<<std::endl ;
                
        // wait server ready (terminate sniffer)
        if (!tv->snf_ready(testnr, ""))  return 1;
       
        std::cout<<"sending message 2"<<std::endl; 

        // Send data
        tv->send(msg_N.c_str());
      
        std::cout<<"sending ready"<<std::endl ; 

        // wait server ready (run and start sniffer)
        if (!tv->snf_ready(testnr, ""))  return 1;
       
        std::cout<<"sending message 3"<<std::endl; 
      
        // Send data
        tv->send(msg_Y.c_str());

        std::cout<<"finish iteration"<<std::endl;

        tv->finish_iteration(testnr);
        testnr++;

        //=============================================
        // test 10 - 12
 
        for(; testnr <=12; testnr++)
        { 
           std::cout<<"test "<<testnr<<": sending ready..";   

           // wait server ready 
           if (!tv->snf_ready(testnr, port.c_str()))  return 1;

           std::cout<<"sending 100 messages"<<std::endl ;  

           // Send data
           for(int i=0; i < 100; i++)
             tv->send(msg_Y.c_str());
        
           std::cout<<"finish iteration"<<std::endl;

           tv->finish_iteration(testnr);         
        }
        //=============================================
        std::cout<<"finish. "<<std::endl;
        tv->finish(); 
    }
    catch (const CORBA::Exception &) 
    {
        cout << "CORBA client: Uncaught CORBA exception" << endl;
        return 1;
    }
    catch (...) 
    {
        return 1;
    }
    
    return 0;
}
