/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_HELLO_CLIENT_THREAD_HPP 
#define INCL_HELLO_CLIENT_THREAD_HPP 

#include <iostream>
#include <string>

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/osthreads/ThreadHandle.hpp>
#include <Foundation/logging/Logger.hpp>


/* This is a validation test : in validation conditions
   we expect to be compiled with CDMW_TRACE_LEVEL 6 if nothing else is stated */


class TraceClientThread : public Cdmw::OsSupport::Thread 
{
    
  public:
  
    TraceClientThread (CORBA::ORB_ptr,
                       std::string& applicationName,
                       std::string& processName,
                       std::string& traceMessage,
                       long traceCount,
                       long traceCountSeq1,
                       long traceCountSeq2,
                       bool repetition,
                       bool multidomain,
                       bool perfProbeCall,
                       Cdmw::Logging::Logger* logger)
                 throw (CORBA::SystemException);



    ~TraceClientThread() throw();




    void stop() 
    { 
        m_exit_required = true; 
    }



  protected:

    void run() throw();
      


  private:
    
    TraceClientThread();
    
    TraceClientThread(const TraceClientThread&);
    
    TraceClientThread& operator = (const TraceClientThread&);
    
    
    void target_api(long count);
    
    
    
    /**
    * The orb reference.
    */
    CORBA::ORB_var m_orb;
    
    /**
    * The application name.
    */
    std::string m_applicationName;
      
    /**
    * The process name.
    */
    std::string m_processName;
    
    /**
    * The trace message.
    */
    std::string m_traceMessage;
    
    /**
    * The number of trace message.
    */
    long m_traceCount;
    
    /**
    * The number of trace message for sequence 1.
    */
    long m_traceCountSeq1;
    
    /**
    * The number of trace message for sequence 2.
    */
    long m_traceCountSeq2;
    
    /**
    * The repetition flag.
    */
    bool m_repetition;
    
    /**
    * The multidomain flag.
    */
    bool m_multidomain;
    
    /**
    * The performance probe call flag.
    */
    bool m_perfProbeCall;
       
    /**
    * The flag to break the thread loop.
    */
    bool m_exit_required;
    
    /**
    * The logger.
    */
    Cdmw::Logging::Logger* m_logger;
    
    // The multidomain loggers
    Cdmw::Logging::Logger* m_logger1;
    Cdmw::Logging::Logger* m_logger2;
    Cdmw::Logging::Logger* m_logger3;
    Cdmw::Logging::Logger* m_logger4;
    Cdmw::Logging::Logger* m_logger5;
    Cdmw::Logging::Logger* m_logger6;
        
};



#endif // INCL_HELLO_CLIENT_THREAD_HPP 
