/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.valid.traceclient;

import java.io.*;
import java.util.*;

import org.omg.CORBA.*;
import org.omg.CORBA.Object;
import org.omg.PortableServer.*;
import org.omg.CosClockService.*;

import cdmw.common.*;
import cdmw.common.BadParameterException;
import cdmw.logging.*;
import static cdmw.logging.Logger.*;
import cdmw.namingandrepository.*;
import cdmw.orbsupport.*;
import cdmw.ossupport.*;
import cdmw.ossupport.Timeval;
import cdmw.performance.*;
import cdmw.platformmngt.*;
import cdmw.trace.*;

import org.omg.CosClockService.ClockCatalogPackage.ClockEntry;
import org.omg.CosClockService.ClockCatalogPackage.UnknownEntry;
import org.omg.CosClockService.ControlledClockPackage.NotSupported;
import org.omg.CosClockService.ControlledClockHelper;

import Cdmw.clock.ControlledClockEx;
import Cdmw.clock.ControlledClockExHelper;

import com.thalesgroup.CdmwNamingAndRepository.*;

public class MainTest
{
    private static final int POA_THREAD_POOL_SIZE = 5;
    
//    private static final String TRACE_MSG_OPTION = "--trace-msg";
    private static final String TRACE_NBR_OPTION = "--trace-nbr";
    private static final String TRACE_REPEAT_OPTION = "--trace-repeat";
    private static final String TRACE_MULTI_DOMAIN_OPTION = "--trace-multi_domain";
    private static final String TRACE_SEQ1_OPTION = "--trace-seq1";
    private static final String TRACE_SEQ2_OPTION = "--trace-seq2";
//    private static final String COLLECTOR_NAME_OPTION = "--collector-name";
    private static final String PERF_CALL_OPTION = "--perf-call";
    private static final String INIT_FEDERATED_CLOCK = "--init-federated-clock";

    // private static final String PERF_ACTIVATION_OPTION = "--perf-activate";
    // private static final String PERF_LOG_FILE_OPTION = "--perf-log-file";

    private static final int FAILURE = 1;
    
    private static String processName;
    
    
    public static void main(String[] args)
    {
        String applicationName = "";
        int retCode = 0;
        
        StrategyList strategy = new StrategyList();
        strategy.addORBThreaded();
        try
        {
            strategy.addPOAThreadPool(MainTest.POA_THREAD_POOL_SIZE);
        }
        catch (BadParameterException e)
        {
            e.printStackTrace();
        }
        
        ORB orb = null;
        
        if (args.length > 1)
        {
            if(args[1].equals("--help"))
            {
                MainTest.usage(args[0]);
                System.exit(retCode);
            }
        }
        
        
        try
        {
            // initialise the orb    
            orb = ORBSupport.ORBInit(args, strategy);
            Object obj = orb.resolve_initial_references("RootPOA");
            POA rootPOA = POAHelper.narrow(obj);
            POAManager poaMng = rootPOA.the_POAManager();
            poaMng.activate();
                
            System.out.println("~+~+~+~ Making sure that the TraceCollector is fully initialized : Sleeping for 4 sec...");
            OS.sleep (4000);      
            System.out.println("~+~+~+~ done.");
            
            // The logger
            Logger logger = Logger.Get_Logger("TEST", "TEST");

            // Federated Controlled Clock Management
            String federatedControlledClockInitialization = OS.getOptionValue(args, MainTest.INIT_FEDERATED_CLOCK);
            if (federatedControlledClockInitialization != "no")
            {
                int num_sec = (int) Long.parseLong(federatedControlledClockInitialization);
                cdmw.clock.CdmwCosClock.init(orb, args);
                init_controlled_clock(orb, num_sec);
            }

            //
            // get number of Trace Messages
            //
            long traceCount = -1;
            
            String traceCountOption = OS.getOptionValue(args, MainTest.TRACE_NBR_OPTION);
            if (traceCountOption.equals("yes"))
            {
                System.out.println("bad value for " + TRACE_NBR_OPTION + "=<number> option");
                System.exit(FAILURE);
            }
            else if (traceCountOption != "no")
            {
                try {
                    traceCount = Long.parseLong(traceCountOption);
                } catch (NumberFormatException e) {
                    System.out.println("bad value for " + TRACE_NBR_OPTION + "=<number> option");
                }

            }
            
            //
            // get trace repeat mode
            //
            boolean traceRepeatMode = false;
            
            String traceRepeatModeOption = OS.getOptionValue(args, MainTest.TRACE_REPEAT_OPTION);
            if (traceRepeatModeOption.equals("yes"))
            {
                traceRepeatMode = true;
            }
            
            //
            // get Performance probe call option
            //
            boolean perfProbeCall = false;
            String perfProbeCallOption = OS.getOptionValue(args, PERF_CALL_OPTION);
            if (perfProbeCallOption.equals("yes"))
            {
                perfProbeCall = true;
            }
            
            //
            // get trace multi domain mode
            //
            boolean traceMultiDomainMode = false;
            
            String traceMultiDomainModeOption = OS.getOptionValue(args, MainTest.TRACE_MULTI_DOMAIN_OPTION);
            if (traceMultiDomainModeOption.equals("yes"))
            {
                traceMultiDomainMode = true;
            }
            
            //
            // get trace sequence counter 1
            //
            long traceCountSeq1 = -1;
            
            String traceCountSeq1Option = OS.getOptionValue(args, MainTest.TRACE_SEQ1_OPTION);
            if (traceCountSeq1Option.equals("yes"))
            {
                System.out.println("bad value for " + TRACE_SEQ1_OPTION + "=<number> option");
                System.exit(FAILURE);
            }
            else if (!traceCountSeq1Option.equals("no"))
            {
                try {
                    traceCountSeq1 = Long.parseLong(traceCountSeq1Option);
                } catch (NumberFormatException e) {
                    System.out.println("bad value for " + TRACE_SEQ1_OPTION + "=<number> option");
                    System.exit(FAILURE);
                }
            }
            
            //
            // get trace sequence counter 2
            //
            long traceCountSeq2 = -1;
            
            String traceCountSeq2Option = OS.getOptionValue(args, TRACE_SEQ2_OPTION);
            if (traceCountSeq2Option.equals("yes"))
            {
                System.out.println( "bad value for " + TRACE_SEQ2_OPTION + "=<number> option");
                System.exit(FAILURE);
            }
            else if (!traceCountSeq2Option.equals("no"))
            {
                try {
                    traceCountSeq2 = Long.parseLong(traceCountSeq2Option);
                } catch (NumberFormatException e) {
                    System.out.println("bad value for " + TRACE_SEQ2_OPTION + "=<number> option");
                    System.exit(FAILURE);
                }

            }
            

            try
            {
                System.out.println("Init PlatformInterface");
          
                String fooStr = "";
                // creates the process behaviour
                MyProcessBehaviour procBehaviour = new MyProcessBehaviour(orb,
                                                 fooStr, 
                                                 traceCount,
                                                 traceCountSeq1,
                                                 traceCountSeq2,
                                                 traceRepeatMode,
                                                 traceMultiDomainMode,
                                                 perfProbeCall, 
                                                 logger);
                                                               
                cdmw.cdmwinit.CdmwInterface.cdmwInit(orb, args, procBehaviour, 
                    "cdmw.generated.traceclientCdmwInterface");                                                 
                cdmw.logging.LogManager.Instance().registerFatalHandler(null);
                
                applicationName = cdmw.cdmwinit.InitUtils.getCdmwApplicationName();
                processName = cdmw.cdmwinit.InitUtils.getCdmwProcessName();
                
                String baseMsg = "process <" + applicationName + "/" + processName + ">" ;
                
                // ===================================================
                // trace a message (don't use CDMW_TRACE in main)
                // ===================================================
                
                System.out.println("program started and waiting CORBA messages");
                String msg = baseMsg + "started and waiting CORBA messages"; 
                
                CDMW_INFO(logger, msg, null);
            
                // start orb
                orb.run();            
            }
            catch(java.lang.Exception e)
            {
                e.printStackTrace();
            }
            
            if (orb != null) {
                cdmw.cdmwinit.CdmwInterface.cdmwCleanup(orb, 
                    "cdmw.generated.traceclientCdmwInterface");                                                 
                //orb->shutdown(false) is done by the ProcessBehaviour
                ORBSupport.ORBCleanup(orb);
                orb.destroy();
            }
            
        }
        catch (SystemException e)
        {  
            System.err.println( "FAILURE : CORBA exception raised :" + e.getMessage());
            e.printStackTrace();
            retCode = FAILURE;
        }
        catch(java.lang.Exception e)
        {
            System.err.println("FAILURE : Exception raised" + "\r\n" + e.getMessage());
            e.printStackTrace();
            retCode = FAILURE;
        }

        System.exit(retCode);
    }

    private static void init_controlled_clock(ORB a_orb, int num_sec) throws BadParameterException
    {

        assert(null != a_orb);

        org.omg.CORBA.Object obj = null;
        try
        {
            obj = a_orb.resolve_initial_references("ClockService");
        } catch (org.omg.CORBA.ORBPackage.InvalidName e1)
        {
            e1.printStackTrace();
        }
        if (null == obj)
        {
            System.out.println("Error : Unable to resolve ClockService");
            throw new BadParameterException("Unable to resolve ClockService", "NULL");
        }

        ClockCatalog catalog = ClockCatalogHelper.narrow(obj);
        if (null == catalog)
        {
            System.out.println("Error : Unable to resolve clock catalog");
            throw new BadParameterException("Unable to resolve ClockCatalog", "NULL");
        }

        ClockEntry entry = null;
        ControlledClockEx ctrl_clock = null;

        try
        {
            entry = catalog.get_entry("FederatedControlledClock");
            ctrl_clock = ControlledClockExHelper.narrow(entry.subject);
        } catch (UnknownEntry ex)
        {
            System.out.println("ClockCatalog::UnknownEntry");
        }

        if (null == ctrl_clock)
        {
            System.err.println("Unable to narrow the clock's name");
            throw new BadParameterException("Unable to narrow the clock's name", "NULL");
        }

        try
        {
            Timeval time_val = OS.getTime();
            
            System.out.println( "-> Setting \"FederatedControlledClock\" at " + time_val.toString());
            time_val = new Timeval(num_sec, 0);
            long to = cdmw.clock.Time.timeval2TimeT(time_val);
            ctrl_clock.set_rate(10.0f);
            ctrl_clock.set(to);
            
            System.out.println("... to " + time_val.toString());
            System.out.println("~+~+~+~+~+~ time_val.seconds=" + time_val.getSeconds() + ", ratio=" + ctrl_clock.get_rate() + " ~+~+~+~+~+~");

        } catch (NotSupported ex)
        {
            ex.printStackTrace();
        } catch (java.lang.Exception ex)
        {
            ex.printStackTrace();
        }

    }

    private static void usage(String programName)
    {
        System.out.println("Usage:\r\n"
                + " options\r\n"
                + "\n"
                + "Options:\r\n"
                + "--help                             Display this information.\r\n"
                // "[--trace-msg=<message>]            Message to trace.\r\n"
                // +
                + "[--trace-nbr=<number>]             Number of messages to trace.\r\n"
                + "[--trace-repeat]                   generate repeated message.\r\n"
                + "[--trace-multi_domain]             generate multi domain messages.\r\n"
                + "[--trace-seq1=<number>]            sequence nbr to deactivate 1st set of messages.\r\n"
                + "[--trace-seq2=<number>]            sequence nbr to activate 2nd set of messages.\r\n"
                + "[--init-federated-clock=<number>]  Number of seconds representing the operational time to be set. \r\n"
                // "[--collector-name=<name>]          Name of trace collector.\r\n"
                // +
                + "[--perf-call]                      Performance probe calling.\r\n");
        // "[--perf-activate]                  Performance measure activation.\n"
        // +
        // "[--perf-log-file=<file>]           Performance log file.\r\n");
    }

    
    private void exceptionHandler()
    {
        System.out.println("Unexpected Exception : Test failed");
    }

    private void exitHandler()
    {
        System.out.println("Process '" + MainTest.processName + "' exited");
    }
    
}
