/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.valid.traceclient;

import org.omg.CORBA.ORB;

import cdmw.common.BadParameterException;
import cdmw.logging.*;
import static cdmw.logging.Logger.*;
import cdmw.performance.Probe;

public class TraceClientThread extends Thread
{
        public  int CDMW_TRACE_LEVEL = 6;
        
           /**
            * The orb reference.
            */
        private ORB orb;
            
            /**
            * The application name.
            */
        private String applicationName;
              
            /**
            * The process name.
            */
        private String processName;
            
            /**
            * The trace message.
            */
        private String traceMessage;
            
            /**
            * The number of trace message.
            */
        private long traceCount;
            
            /**
            * The number of trace message for sequence 1.
            */
        private long traceCountSeq1;
            
            /**
            * The number of trace message for sequence 2.
            */
        private long traceCountSeq2;
            
            /**
            * The repetition flag.
            */
        private boolean repetition;
            
            /**
            * The multidomain flag.
            */
        private boolean multidomain;
            
            /**
            * The performance probe call flag.
            */
        private boolean perfProbeCall;
               
            /**
            * The flag to break the thread loop.
            */
        private boolean exitRequired;
            
            /**
            * The logger.
            */
        private Logger logger;
            
            // The multidomain loggers
        private Logger logger1;
        private Logger logger2;
        private Logger logger3;
        private Logger logger4;
        private Logger logger5;
        private Logger logger6;

        public TraceClientThread(ORB orb, 
                        String applicationName, 
                        String processName, 
                        String traceMessage, 
                        long traceCount, 
                        long traceCountSeq1, 
                        long traceCountSeq2, 
                        boolean repetition, 
                        boolean multidomain, 
                        boolean perfProbeCall, 
                        Logger logger) 
    throws org.omg.CORBA.SystemException
    {
                this.orb=orb;
                this.applicationName = applicationName;
                this.processName = processName;
                this.traceMessage = traceMessage;
                this.traceCount = traceCount;
                this.traceCountSeq1 = traceCountSeq1;
                this.traceCountSeq2 = traceCountSeq2;
                this.repetition = repetition;
                this.multidomain = multidomain;
                this.perfProbeCall = perfProbeCall;
                this.exitRequired = false;
                this.logger = logger;
      
                // create loggers (LogManager has been initialised in ORB_init)
                this.logger1 = Logger.Get_Logger("COMP_A", "DOMAIN-1");
                this.logger2 = Logger.Get_Logger("COMP_B", "DOMAIN-1");
        
                if (this.multidomain)
                {
                        // create loggers (LogManager has been initialised in ORB_init)
                        this.logger3 = Logger.Get_Logger("COMP_A", "DOMAIN-2");
                        this.logger4 = Logger.Get_Logger("COMP_B", "DOMAIN-2");
                        this.logger5 = Logger.Get_Logger("DOMAIN-1");
                        this.logger6 = Logger.Get_Logger("DOMAIN-2");
                }
                else
                {
                        this.logger3 = null;
                        this.logger4 = null;
                        this.logger5 = null;
                        this.logger6 = null;
                } 
    }
        
        public void run()
        {    
                try
                {        
                        System.out.println("Trace Client Thread Started");
        
                        if (this.repetition)
                        {
                                System.out.println(" with repeated messages");
                        }
        
                        if (this.multidomain)
                        {
                                System.out.println(" with multidomain messages");
                        }
                
                        if (this.perfProbeCall)
                        {
                                System.out.println(" with performance probe called");
                        }
        
                        System.out.println(" message counter="+this.traceCount);
                        System.out.println(" message sequence 1 counter="+this.traceCountSeq1);
                        System.out.println(" message sequence 2 counter="+this.traceCountSeq2);
                        System.out.println();
        
                        String baseMsg = "process <" + this.applicationName + "/" + this.processName + ">";
                        String msg;
                        long count = 0;
                        long msgCount = 0;
                        boolean traceDomain2 = true;
         
                        // ===================================================
                        // trace a message (use CDMW_TRACE in Cardamom thread)
                        // ===================================================
                        msg = baseMsg + "Client Thread started";
        
                        if (CDMW_TRACE_LEVEL == 0)
                        {
                                System.out.println("Sending a message to the Trace collector.");
                        }
        
                        CDMW_INFO(this.logger, msg, new Throwable());

                        // ====================================================
                        // thread loop
                        // ====================================================
        
                        msg = baseMsg + this.traceMessage;
        
                        if (this.repetition)
                        {
                                if (this.multidomain)
                                {
                                        msg += " counter=";
                                }
                        }
                        else
                        {
                                msg += " counter=";
                        }        
                
                        // get the filter manager
                        FilterMngr filterMngr = cdmw.logging.LogManager.Instance().getFilterManager();
                   
                        boolean filterSeq = false;
        
                        // if Count seq are set to default values set all filters activated
                        if (this.traceCountSeq1 == -1 && this.traceCountSeq2 == -1)
                        {
                                filterMngr.activateAllLevels(LogLevel.ALL_COMPONENT_NAMES, LogLevel.ALL_DOMAINS);
                                // except messages from Jacorb
                                filterMngr.deactivateAllLevels("JacORB", LogLevel.ALL_DOMAINS);
                        }
                        // else if Count seq are set to 0 set all filters deactivated
                        else
                        {
                                if (this.traceCountSeq1 == 0 && this.traceCountSeq2 == 0)
                                {
                                        filterMngr.deactivateAllLevels(LogLevel.ALL_COMPONENT_NAMES, LogLevel.ALL_DOMAINS);
                                }
                                // else filter sequence is requested
                                else
                                {
                                        // all messages are activated
                                        filterMngr.activateAllLevels(LogLevel.ALL_COMPONENT_NAMES, LogLevel.ALL_DOMAINS);
                                        // except messages from Jacorb
                                        filterMngr.deactivateAllLevels("JacORB", LogLevel.ALL_DOMAINS);
            
                                        // except messages from DOMAIN-2                                
                                        filterMngr.deactivateAllLevels("DOMAIN-2");
                                               
                                        filterSeq = true;
                                }
                        }
                
        
                        // trace message or probe call main loop
                        while (!this.exitRequired) 
                        {
                                int logMsg = 0;
            
                                // if infinite loop for message
                                if (this.traceCount == -1)
                                {
                                        logMsg = 1;
                                }
            
                                // else if max number of messages not reached
                                else if (count < this.traceCount)
                                {
                                        count++;
                                        logMsg = 1;
                                }
            
                                // else if end of loop reached
                                else if (count == this.traceCount && this.traceCount != 0)
                                {
                                        count++;
                                        logMsg = 2;
                                }
            
                                // if filter seq is requested
                                if (filterSeq)
                                {
                                        if (msgCount == this.traceCountSeq1)
                                        {
                                                // deactivate messages from COMP_A                                
                                                filterMngr.deactivateAllLevels("COMP_A", LogLevel.ALL_DOMAINS);
                                        }
                                        else if (msgCount == this.traceCountSeq2)
                                        {
                                                // activate messages from DOMAIN-2                                
                                                filterMngr.activateAllLevels("DOMAIN-2");
                                        }                
                                }
            
                                // if message to trace or probe to call
                                if (logMsg == 1)
                                {
                                        // if performance probe call
                                        if (this.perfProbeCall)
                                        {
                                                targetApi (msgCount);
                                        }
                              
                                        // if repetition of same messages
                                        else 
                                                if (this.repetition)
                                                {

                                                        if (CDMW_TRACE_LEVEL == 0)
                                                        {
                                                                System.out.println("Sending a message to the Trace collector.");
                                                        }

                                                        // if repetition of multi domain messages
                                                        if (this.multidomain)
                                                        {
                                                                if (msgCount % 5 == 0) 
                                                                {
                                                                        traceDomain2 = !traceDomain2;
                                                                }
                        
                                                                if (traceDomain2) 
                                                                {
                                                                        //this.logger6.warn(msg+msgCount/10, new Throwable()); 
                                                                          CDMW_WARN(this.logger6,msg+msgCount/10, new Throwable());
                                                                } 
                                                                else 
                                                                {
                                                                        //this.logger5.fatal(msg+msgCount/10, new Throwable());
                                                                        CDMW_FATAL(this.logger5,msg+msgCount/10, new Throwable());
                                                                }
                                                        }    
                                                        // else if repetition of simple messages
                                                        else
                                                        {
                                                                //this.logger.info(msg, new Throwable());
                                                                CDMW_INFO(this.logger,msg, new Throwable());
                                                        }
                                                }               
                                        // else if request of multi domain messages
                                                else 
                                                if (this.multidomain)
                                                {

                                                        if (CDMW_TRACE_LEVEL == 0)
                                                        {
                                                                System.out.println("Sending a message to the Trace collector.");
                                                        }
                   
                                                        //this.logger1.fatal(msg+msgCount, new Throwable());
                                                        CDMW_FATAL(this.logger1,msg+msgCount, new Throwable());
                                                        //this.logger1.error(msg+msgCount, new Throwable());
                                                        CDMW_ERROR(this.logger1,msg+msgCount, new Throwable());
                                                        //this.logger3.error(msg+msgCount, new Throwable());
                                                        CDMW_ERROR(this.logger3,msg+msgCount, new Throwable());
                                                        //this.logger2.warn(msg+msgCount, new Throwable());
                                                        CDMW_WARN(this.logger2,msg+msgCount, new Throwable());
                                                        //this.logger4.info(msg+msgCount, new Throwable());
                                                        CDMW_INFO(this.logger4,msg+msgCount, new Throwable());
                                                        //this.logger5.fatal(msg+msgCount, new Throwable());
                                                        CDMW_FATAL(this.logger5,msg+msgCount, new Throwable());
                                                        //this.logger5.error(msg+msgCount, new Throwable());
                                                        CDMW_ERROR(this.logger5,msg+msgCount, new Throwable());
                                                        //this.logger5.warn(msg+msgCount, new Throwable());
                                                        CDMW_WARN(this.logger5,msg+msgCount, new Throwable());
                                                        //this.logger6.error(msg+msgCount, new Throwable());
                                                        CDMW_ERROR(this.logger6,msg+msgCount, new Throwable());
                                                        //this.logger6.info(msg+msgCount, new Throwable());
                                                        CDMW_INFO(this.logger6,msg+msgCount, new Throwable());
                                                }             
                                                // else simple message is traced
                                                else
                                                {

                                                        if (CDMW_TRACE_LEVEL == 0)
                                                        {
                                                                System.out.println("Sending a message to the Trace collector.");
                                                        }
                                                        //this.logger1.info(msg+msgCount, new Throwable());
                                                        CDMW_INFO(this.logger1, msg+msgCount, new Throwable());
                                                        CDMW_INFO(this.logger2, msg+msgCount, new Throwable());
                                                        //this.logger2.info(msg+msgCount, new Throwable());
                                                }
                                }
            
                                // else if request of end message
                                else 
                                if (logMsg == 2)
                                {
                                        count++;
                
                                        String msg2 = baseMsg + "!!!! Trace terminated !!!!";  
                                       
                                        //this.logger2.info(msg2, new Throwable());
                                        CDMW_INFO(this.logger2, msg2, new Throwable());
                
                                        System.out.println("Trace terminated : System can be stopped !!!!!!");
                                }
            
                                // ===================================================
                                // set thread sleeping
                                // ===================================================
                                Thread.sleep (1000);
            
                                if (msgCount == 0x7fffffff)
                                {
                                        msgCount = 0;
                                }
            
                                msgCount ++;          
                        } 
                }
                catch (Exception e)
                {
                        System.err.println(e.toString()+" Unexpected exception in thread loop");
                }
                
        }
        
        private void targetApi(long count) throws BadParameterException
        {
                Probe probe = new Probe("target_api");
            System.out.println("Performance cycle number = ***" + count + "***");
        }        
        
        public void stopImpl() 
    { 
        exitRequired = true; 
    }
}
