/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.cif;

import cdmw.logging.LogLevel;
import cdmw.logging.LevelType;

//
// IDL:com.thalesgroup/CdmwLogging/Logger:1.0
//
class LoggerImpl
    extends org.omg.CORBA.LocalObject
    implements com.thalesgroup.CdmwLogging.Logger 
{

    /**
     * logger
     */
    private cdmw.logging.Logger logger;

    /**
     * constructor
     * @param componentName the component name
     * @param domainName the domain name
     */
    public LoggerImpl(String componentName, String domainName)
    {
        this.logger = cdmw.logging.Logger.Get_Logger(componentName, domainName);
    }
    
        
    //
    // IDL:com.thalesgroup/CdmwLogging/Logger/log:1.0
    //
    public void log(int level, 
                    java.lang.String message, 
                    java.lang.String file, 
                    int line)
    {
        logger.log(new LevelType(level), 
                   message, 
                   file, 
                   line);
    }
    
        
    //
    // IDL:com.thalesgroup/CdmwLogging/Logger/debug:1.0
    //
    public void debug(java.lang.String message, 
                      java.lang.String file, 
                      int line)
    {
        logger.log(new LevelType(LogLevel.DEBUG), message, file, line);
    }
    
    
    //
    // IDL:com.thalesgroup/CdmwLogging/Logger/info:1.0
    //
    public void info(java.lang.String message, 
                     java.lang.String file, 
                     int line)
    {
        logger.log(new LevelType(LogLevel.INFO), message, file, line);
    }
    
    
    //
    // IDL:com.thalesgroup/CdmwLogging/Logger/warn:1.0
    //
    public void warn(java.lang.String message, 
                     java.lang.String file, 
                     int line)
    {
        logger.log(new LevelType(LogLevel.WARN), message, file, line);
    }
    
    
    //
    // IDL:com.thalesgroup/CdmwLogging/Logger/error:1.0
    //
    public void error(java.lang.String message, 
                      java.lang.String file, 
                      int line)
    {
        logger.log(new LevelType(LogLevel.ERROR), message, file, line);
    }
    
    
    //
    // IDL:com.thalesgroup/CdmwLogging/Logger/fatal:1.0
    //
    public void fatal(java.lang.String message, 
                      java.lang.String file, 
                      int line)
    {
        logger.log(new LevelType(LogLevel.FATAL), message, file, line);
    }
    
    
    //
    // IDL:com.thalesgroup/CdmwLogging/Logger/is_enabled_for:1.0
    //
    public boolean is_enabled_for(int level)
    {
        return logger.isEnabledFor(new cdmw.logging.LevelType(level));
    }
    
    
    //
    // IDL:com.thalesgroup/CdmwLogging/Logger/is_debug_enabled:1.0
    //
    public boolean is_debug_enabled()
    {
        return logger.isDebugEnabled();
    }
    
    
    //
    // IDL:com.thalesgroup/CdmwLogging/Logger/is_info_enabled:1.0
    //
    public boolean is_info_enabled()
    {
        return logger.isInfoEnabled();
    }
    
    
    //
    // IDL:com.thalesgroup/CdmwLogging/Logger/is_warn_enabled:1.0
    //
    public boolean is_warn_enabled()
    {
        return logger.isWarnEnabled();
    }
    
    
    //
    // IDL:com.thalesgroup/CdmwLogging/Logger/is_error_enabled:1.0
    //
    public boolean is_error_enabled()
    {
        return logger.isErrorEnabled();
    }
    
    
    //
    // IDL:com.thalesgroup/CdmwLogging/Logger/is_fatal_enabled:1.0
    //
    public boolean is_fatal_enabled()
    {
        return logger.isFatalEnabled();
    }
    
    
    
    /**
     * Specific Throwable class overloading getStackTrace() operation
     * in order to return a StackTrace with appropriate file and name
     */
    private class MyThrowable extends Throwable {
        
        private String filename;
        private int line;
        
        public MyThrowable(String filename, int line)
        {
            this.filename = filename;
            this.line = line;
        }
        
        public StackTraceElement[] getStackTrace()
        {
            StackTraceElement[] stack = new StackTraceElement[1];
            stack[1] = new StackTraceElement("?? unknown class ??",
                                             "?? unknown operation ??",
                                             filename,
                                             line);
            return stack;
        }
        
    }

}
