/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.cif.test;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.junit.Before;
import org.junit.Test;

import cdmw.ossupport.OS;
import cdmw.ossupport.Mutex;
import cdmw.ossupport.Timeval;
import cdmw.testutils.TestManager;
import static cdmw.ccm.common.CCMSystemProperties.TYPECHECKING_TIMEOUT;

//
// IDL:thalesgroup.com/Test/CCM_Client:1.0
//
public class CCMClientImpl extends org.omg.CORBA.LocalObject
    implements com.thalesgroup.UserTest.User_CCM_Client {
    
    public com.acme.Test.CCM_Client_Context context = null;
    public ClientActivationThread activationThread;
   
    public CCMClientImpl() {
        this.activationThread = new ClientActivationThread(this);
    }

    /**
     *  This class provides an object to test the ccm_activate.
     */
    public static class CCMClientActivateTest {
        
        private static final String CLIENT
            = "cdmw.ccm.cif.test.CCMClientImpl.client";
        
        private CCMClientImpl client = null;

        @Before
        public void init() {
            TestManager testManager = TestManager.getLocalInstance();
            
            this.client
                = (CCMClientImpl)testManager.getLocalProperty(CLIENT);
        }
    
        @Test
        public void doTests() {
        
            assertTrue(this.client != null);
        
            com.acme.Test.Foo foo = null;
            com.acme.Test.CCM_Client_Context clientContext = 
                this.client.get_context();
              
            System.out.println("Client: Before get_connection_foo_receptacle...");
            try {
                foo = clientContext.get_connection_foo_receptacle();

                if(System.getProperty("Num.Test").equals("1016")) {
                    // get_connection_foo_receptacle must not be blocking
                    System.out.println("*** get_connection() not blocking.");
                    assertTrue(true);
                }

//                 foo.foo_test();
            } catch (Exception e) {
                // InvalidConnection must be caught by SimDemployment
                // during configuration_complete().
                e.printStackTrace();
                fail("Client: Expected exception raised!");
            }

            System.out.println("Client: After get_connection_foo_receptacle...");

        }

    }
   
    public class ClientActivationThread extends Thread {
        
        private CCMClientImpl client;

        public ClientActivationThread(CCMClientImpl client) {
            this.client = client;
        }

        public void run() {
            TestManager testManager = new TestManager();
            TestManager.activateLocalInstance(testManager);

            testManager.setLocalProperty(
                CCMClientActivateTest.CLIENT,
                this.client);
            testManager.addTest(CCMClientActivateTest.class);
            
            testManager.runTests();
            testManager.printSummary();
        }
    }
    
    public void set_session_context(org.omg.Components.SessionContext ctx) {
        System.out.println("Client: set_session_context called!");
        this.context = 
            com.acme.Test.CCM_Client_ContextHelper.narrow(ctx);
    }
    
    /**
     * Actives the component (call by the container)
     */
    public void ccm_activate() {
        // Start thread
        this.activationThread.start();
    }
    
    /**
     * Makes stop the component (call by the container)
     */
    public void ccm_passivate() {
        System.out.println("Client: ccm_passivate called!");
    }
    
    /**
     * Destroys the component (call by the container)
     */
    public void ccm_remove() {
        System.out.println("Client: ccm_remove called!");
    }

    public com.acme.Test.CCM_Client_Context get_context() {
        return this.context;
    }

}
