/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.cif.test;

import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import cdmw.testutils.TestManager;

//
// IDL:thalesgroup.com/TestTimeout/CCM_Client:1.0
//
public class CCMClientImpl extends org.omg.CORBA.LocalObject
    implements com.thalesgroup.UserTestTimeout.User_CCM_Client {

    /**
     * This class provides an object to test the ccm_activate.
     *
     */
    public static class CCMClientActivateTest {
        private static final String CCM_CLIENT_CONTEXT
            = "cdmw.ccm.cif.test.CCMClientActivateTest.CCM_Client_Context";

        private com.acme.TestTimeout.CCM_Client_Context clientContext;
        
        @Before
        public void init() {
            TestManager testManager = TestManager.getLocalInstance();

            this.clientContext = (com.acme.TestTimeout.CCM_Client_Context) 
                testManager.getLocalProperty(CCM_CLIENT_CONTEXT);
        }
    
        @Test
        public void doTests() { 
            
            System.out.println("********************************************");
            System.out.println("            ccm_activate start");
            System.out.println("********************************************");
            
            com.acme.TestTimeout.TimeoutTester serverFacet = null;
            
            assertTrue("clientContext is null !",
                       clientContext != null);
            
            System.out.println("Get_connection_without_default_receptacle...");
            
            try {
                serverFacet = clientContext.get_connection_without_default_receptacle();
            
                assertTrue("serverFacet is null !",
                           serverFacet != null);
            
            } catch (Exception e) {
                e.printStackTrace();
                fail("Unexpected exception raised: " + e);
            }
            
            //get the connection to the server 
            System.out.println("Connect to the TimeoutTester facet "
                               + " with the default receptacle...");
            
            try {
                serverFacet.sleep(10);
                
            } catch(org.omg.CORBA.TIMEOUT e) {
                fail("org.omg.CORBA.TIMEOUT: exception raised. "
                   + "Unexpected exception raised!");
            }
            catch (Exception e) {
                e.printStackTrace();
                fail("Unexpected exception raised: " + e);
            }

                        
            /*
            * Test of the specific receptacle
            */
            System.out.println("Get_connection_specifc_receptacle...");
            try {
                serverFacet = clientContext.get_connection_specific_receptacle();
            
                assertTrue("serverFacet is null !",
                           serverFacet != null);
            
            } catch (Exception e) {
                e.printStackTrace();
                fail("Unexpected exception raised: " + e);
            }
            
            //get the connection to the server 
            System.out.println("Connect to the TimeoutTester facet "
                               + " with the specific receptacle...");
            try {
                serverFacet.sleep(10);
                fail("Expected exception not raised");
                
            } catch(org.omg.CORBA.TIMEOUT e) {
                System.out.println("org.omg.CORBA.TIMEOUT: exception raised . "
                                 + "Expected exception raised!");
            }
            catch (Exception e)
            {
                e.printStackTrace();
                fail("Unexpected exception raised: " + e);
            }
            
            // Wait 7 seconds that sleep request ends on server side.
            // This to avoid a COMM_FAILURE message by JacORB when killing server.
            try {
                cdmw.ossupport.OS.sleep(7000);
            } catch (java.lang.InterruptedException e) {
                // ignore
            }
            
            System.out.println("********************************************");
            System.out.println("            ccm_activate stop");
            System.out.println("********************************************");
        }
    }

    
    private com.acme.TestTimeout.CCM_Client_Context sessionContext;

    public CCMClientImpl() {
        this.sessionContext = null;
    }

     
    public void set_session_context(org.omg.Components.SessionContext ctx) {
        System.out.println("setSessionContext called!");
        this.sessionContext = 
            com.acme.TestTimeout.CCM_Client_ContextHelper.narrow(ctx);
    }

    public void ccm_activate() {
        TestManager testManager = new TestManager();
        testManager.activateLocalInstance(testManager);
        
        testManager.init("cdmw.ccm.cif.test.CCMClientActivateTest");
        testManager.setLocalProperty(
            CCMClientActivateTest.CCM_CLIENT_CONTEXT,
            this.sessionContext);
        testManager.addTest(CCMClientActivateTest.class);
        
        testManager.runTests();
        testManager.printSummary();        
    }
    
    public void ccm_passivate() {
        System.out.println("ccm_passivate called!");
    }
    
    public void ccm_remove() {
        System.out.println("ccm_remove called!");
    }

}
