/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2007. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ccm.componentserver;

import cdmw.ccm.common.CCMLoggers;
import cdmw.ccm.common.DisposableObject;
import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;



/**
 * A Servant Activator for ComponentServer and Container servants.
 *
 */
public class CCMServantActivator
    extends org.omg.CORBA.LocalObject
    implements org.omg.PortableServer.ServantActivator {

    public CCMServantActivator() { 
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcontainerLogger(), "");
    }

    public org.omg.PortableServer.Servant incarnate(
        byte[] oid,
        org.omg.PortableServer.POA poa)
        throws org.omg.PortableServer.ForwardRequest 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcontainerLogger(), "");
        return null; 

    }

    /**
     * The main purpose of this operation is to provide a hook
     * that will be called by the ORB during its shutdown phase
     * to etherealize componentServer and container objects.
     */
    public void etherealize(
        byte[] oid,
        org.omg.PortableServer.POA poa,
        org.omg.PortableServer.Servant servant,
        boolean cleanupInProgress,
        boolean remainingActivations) 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcontainerLogger(), "Servant Class Name="+servant.getClass().getName());

        if (!remainingActivations) {
           try {
                    cdmw.ccm.common.DisposableObject impl = 
                        (cdmw.ccm.common.DisposableObject) servant;
                    impl.dispose();
           } 
           catch(ClassCastException cce) {
               CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(),
                          "ClassCastException exception raised" + cce,
                          new Throwable());
           }
           catch(org.omg.Components.RemoveFailure rf) {
               CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(),
                          "RemoveFailure exception raised" + rf,
                          new Throwable());
           }
        }
    }

}




