/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.testccmrecording;

import com.acme.TestCcmRecording.*;
import com.thalesgroup.CdmwRecording.*;

import cdmw.ossupport.OS;
import cdmw.testutils.CORBATestManager;
import cdmw.testutils.TestManager;

import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;


//
// IDL:acme.com/TestCcmRecording/CCM_Client:1.0
//
public class CCMClientImpl extends org.omg.CORBA.LocalObject
    implements Session_CCM_Client 
{
    // session context
    private CCM_Client_Context sessionContext;
     
    // TestThread
    private class TestThread extends Thread {
        public TestManager testManager;
        
        TestThread(TestManager testManager) {
            this.testManager = testManager;
        }
        
        public void run() {
            this.testManager.runTests();
            this.testManager.printSummary();
        }
    }
    private TestThread testThread;
    
    // Received events queue    
    java.util.List<RequestReplyPair> recordList;
    
    
    public CCMClientImpl() 
    {
        this.sessionContext = null;
        this.testThread = null;
        this.recordList = new java.util.Vector<RequestReplyPair>();
    }

    
    public void set_session_context(org.omg.Components.SessionContext ctx) 
    {
        System.out.println("   ... CCMClientImpl.set_session_context called!");
        this.sessionContext = CCM_Client_ContextHelper.narrow(ctx);
    }
    
    public CCM_Client_Context getContext()
    {
        return this.sessionContext;
    }
    
    
    public void ccm_activate() 
    {
        System.out.println("   ... CCMClientImpl.ccm_activate called!");

        try {
            TestManager testManager = TestManager.instance();
            TestManager.activateLocalInstance(testManager);
            
            testManager.init(
                "cdmw.testccmrecording.CCMClientImpl.CCMClientActivateTest");
            testManager.setLocalProperty(
                CCMClientActivateTest.CCM_CLIENT_IMPL_NAME,
                this);
            testManager.setLocalProperty(
                CCMClientActivateTest.SESSION_CONTEXT_NAME,
                this.sessionContext);
            testManager.addTest(CCMClientActivateTest.class);
                        
            this.testThread = new TestThread(testManager);
            this.testThread.start();
            
        } catch (Exception e) {
            System.err.println("Exception thrown during test: " + e);
        }
    }
    
    public void ccm_passivate() 
    {
        System.out.println("   ... CCMClientImpl.ccm_passivate called!");
        if (this.testThread != null) {
            try {
                this.testThread.join();
            } catch (java.lang.InterruptedException e) {
                // ignore
            }
        }
    }
    
    public void ccm_remove() 
    {
        System.out.println("   ... CCMClientImpl.ccm_remove called!");
    }
    
        
    //
    // CdmwRecorindg::Recorder::record() implementation
    //
    public synchronized void record(
        com.thalesgroup.CdmwRecording.GroupHeader header, 
        byte[] data)
    {
        System.out.println("=== RECORD: " + header.num_pairs + " new Records received from "
                           + header.hostname + " at " + header.base_time);
                           
        // Use org.jacorb.orb.CDRInputStream for unmarshalling
        boolean endian = (header.flags!=0 ? true : false);
        org.omg.CORBA.portable.InputStream input =
            new org.jacorb.orb.CDRInputStream(org.omg.CORBA.ORB.init(), 
                                              data, 
                                              endian);
        
        // Unmarshall while InputStream is not empty
        int i=0;
        try {
            while (input.available() > 0) {
                
                // number of octets still to read
                int left = input.available();
            
                // Unmarshall a RequestReplyPair
                try {
                    RequestReplyPair pair = RequestReplyPairHelper.read(input);
                    
                    System.out.println("=== RECORD:   - " + (++i) + ": \"" +
                                       pair.request.operation +
                                       "\" from " +
                                       pair.request.server_id);
            
                    // save RequestReplyPair in list
                    // ignore _is_a requests
                    if (!pair.request.operation.equals("_is_a")) {
                        recordList.add(pair);
                    }
            
                } catch (org.omg.CORBA.MARSHAL e) {
                    // number of read octets
                    int nb_read = left-input.available();
                    System.out.println("=== RECORD: UNMARSHALL ERROR (only " + nb_read +
                                       " octets read): " + e);
                    e.printStackTrace();
                    // It's useless to try to continue unmarshalling... break loop
                    break;
                }
            }
        } catch (java.io.IOException e) {
            System.out.println("UNMARSHALL ERROR : " + e);
            e.printStackTrace();
        }
        
    }
    
    
    //
    // CCMClientActivateTest class
    //
    public static class CCMClientActivateTest 
    {
        private static final String CCM_CLIENT_IMPL_NAME
            = "CCMClientImpl_object";

        private static final String SESSION_CONTEXT_NAME
            = "CCM_Client_Context_object";
            
        private CCMClientImpl ccmClientImpl;
        private CCM_Client_Context sessionContext;
        private org.omg.CORBA.ORB orb;
        
        public CCMClientActivateTest()
        {
            this.ccmClientImpl = null;
            this.sessionContext = null;
            this.orb = null;
        }
        
        
        @Before
        public void init () 
        {
            TestManager testManager = TestManager.instance();

            this.ccmClientImpl = 
                (CCMClientImpl)testManager.getLocalProperty(CCM_CLIENT_IMPL_NAME);
            assertTrue(this.ccmClientImpl != null);
            this.sessionContext = 
                (CCM_Client_Context)testManager.getLocalProperty(SESSION_CONTEXT_NAME);
            assertTrue(this.sessionContext != null);
        }
        
        
        @Test
        public void doTests() 
        {
            System.out.println("   ... CCMClientActivateTest.doTests called!");
            
            // wait previous connection requests to be recorded...
            try {
                cdmw.ossupport.OS.sleep(2000);
            } catch (InterruptedException e) {
                // ignore
            }
            
            System.out.println("   ... clear recordList content (previous records about connections)!");
            // clear recordList content (previous records about connections)
            ccmClientImpl.recordList.clear();
            
            System.out.println("Send request to Server1");
            try {
                TestRecording testRecording =
                    this.sessionContext.get_connection_test_server1();
                    
                testRecording.test_operation("MESSAGE TO SERVER1");
                
                // Dirty trick to get ORB (required for Valuetypes unmarshalling)
                this.orb = ((org.omg.CORBA.portable.ObjectImpl) testRecording)._orb();

            } catch (org.omg.CORBA.SystemException e) {
                fail("Exception calling Server1's facet: " + e);
            }
            
            
            System.out.println("Send request to Server2");
            try {
                TestRecording testRecording =
                    this.sessionContext.get_connection_test_server2();
                    
                testRecording.test_operation("MESSAGE TO SERVER2");
                
            } catch (org.omg.CORBA.SystemException e) {
                fail("Exception calling Server2's facet: " +e); 
            }
            
            
            System.out.println("Send request to Server3");
            try {
                TestRecording testRecording =
                    this.sessionContext.get_connection_test_server3();
                    
                testRecording.test_operation("MESSAGE TO SERVER3");
                
            } catch (org.omg.CORBA.SystemException e) {
                fail("Exception calling Server3's facet: " +e); 
            }
            
            
            System.out.println("Publish Event");
            try {
                TestEvent ev = new TestEventImpl("EVENT WITH MESSAGE");
        
                this.sessionContext.push_test_event (ev);
                
            } catch (org.omg.CORBA.SystemException e) {
                fail("Exception calling Server3's facet: " +e); 
            }
            
            // wait requests to be recorded
            try {
                cdmw.ossupport.OS.sleep(5000);
            } catch (InterruptedException e) {
                // ignore
            }
               
            System.out.println("Check received records:");

            System.out.println("A total of " + ccmClientImpl.recordList.size() + 
                               " records were received:");
            int i=1;
            for (RequestReplyPair pair : ccmClientImpl.recordList) {
                System.out.println("  - " + (i++) + ": \"" +
                                   pair.request.operation + "\" from " +
                                   pair.request.server_id);
            }
            
            // 4 records expected:
            
            //  - "test_operation" with "MESSAGE TO SERVER1"
            assertTrue(
                "Message \"test_operation\" with parameter \"MESSAGE TO SERVER1\" was not recorded",
                find_test_operation_record("Server1", "MESSAGE TO SERVER1"));
                                   
            //  - "test_operation" with "MESSAGE TO SERVER3"
            assertTrue(
                "Message \"test_operation\" with parameter \"MESSAGE TO SERVER3\" was not recorded",
                find_test_operation_record("Server3", "MESSAGE TO SERVER3"));
                                   
            //  - "push_event" with "EVENT WITH MESSAGE"
            assertTrue(
                "Message \"test_operation\" with parameter \"EVENT WITH MESSAGE\" was not recorded",
                find_push_event_record("Server1", "EVENT WITH MESSAGE"));
                                   
            //  - "push_event" with "EVENT WITH MESSAGE"
            assertTrue(
                "Message \"test_operation\" with parameter \"EVENT WITH MESSAGE\" was not recorded",
                find_push_event_record("Server3", "EVENT WITH MESSAGE"));
                                   
            // Check if there is non-expected records
            if (! ccmClientImpl.recordList.isEmpty()) {
                System.out.println("Following records were not expected:");
                i=1;
                for (RequestReplyPair pair : ccmClientImpl.recordList) {
                    System.out.println("  - " + (i++) + ": \"" +
                                       pair.request.operation + "\" from " +
                                       pair.request.server_id);
                }
                fail("Some records were not expected");
            }

//(====================
            // Get a component reference to 'Server1' via 'cdmw_make_ref' tool
            String corbaloc_str = "";
            String s = "";

            String HOST = cdmw.ossupport.OS.getHostname();
            String PORT = "21869";
            String NAME = "DeploymentPlans/V-CMC-007-2.DeploymentPlan/Server1.Component";

            String of="f.out";
            String args = " --kind=corbaloc --host=" + HOST + " --port=" + PORT + " --name=" + NAME + " --output-file=" + of;
            try {
                cdmw.ossupport.ThreadedProcess cmrThread = cdmw.ossupport.OS.createProcess("cdmw_make_ref" + args );
            } catch (java.io.IOException ioe) {
                assertTrue("Error creating process : cdmw_make_ref " + args + "\n" + ioe, false);
            }

            // wait end of cdmw_make_ref command
            try {
                cdmw.ossupport.OS.sleep(2000);
            } catch (InterruptedException e) {
                // ignore
            }

            // import the object reference from the file
            System.out.println("***** Recover corbaloc from cdmw_make_ref");
            try {
                java.io.BufferedReader in = new java.io.BufferedReader( new java.io.FileReader(of) );
                corbaloc_str = in.readLine();
            } catch(Exception ioe) {
                assertTrue("Error reading component corbaloc : " + ioe, false);
            }

            System.out.println("***** cdmw_make_ref CORBALOC : " + corbaloc_str);
            org.omg.CORBA.Object o_cmr = this.orb.string_to_object(corbaloc_str);
            System.out.println("***** After string_to_object(corbaloc_str)");
            assertTrue("TEST FAILED : invalid corbaloc string!", o_cmr != null);
//)====================

            // Server 1
            System.out.println("***** Retrieve connection to test Server1");
            TestRecording testRecording_1 =
                    this.sessionContext.get_connection_test_server1();
            // to be replaced by get_component (uncomment instructions below)
//            com.acme.TestCcmRecording.Server server_1 = com.acme.TestCcmRecording.ServerHelper.narrow(testRecording_1);
            System.out.println("***** Retrieve object reference of component Server1");
            org.omg.CORBA.Object s1__ = testRecording_1._get_component();
            if (s1__ == null)
            {
               fail ("Failed to get a component reference using _get_component()!");
            }
            System.out.println("***** Retrieve reference of component Server1");
            com.acme.TestCcmRecording.Server server_1 = com.acme.TestCcmRecording.ServerHelper.narrow(s1__);
            if (server_1 == null)
            {
              fail("testRecording_1 is not a com.acme.TestCcmRecording.Server!");
            }
            System.out.println("***** Retrieve CCM Home of component Server1");
            org.omg.Components.CCMHome home_1 = server_1.get_ccm_home();
            com.acme.TestCcmRecording.ServerHome server_home_1 = com.acme.TestCcmRecording.ServerHomeHelper.narrow(home_1);
            if (server_home_1 == null)
            {
               fail("home_1 is not a com.acme.TestCcmRecording.ServerHome!");
            }
//(====================
            s = this.orb.object_to_string(server_1);
            System.out.println("***** server_1 IOR : " + s);

            com.acme.TestCcmRecording.Server server_1_cmr = com.acme.TestCcmRecording.ServerHelper.narrow(o_cmr);
            if (server_1_cmr == null) {
                fail("server_1_cmr is not a com.acme.TestCcmRecording.Server!");
            }
            s = this.orb.object_to_string(server_1_cmr);
            System.out.println("***** server_1_cmr IOR : " + s);
//)====================
           // Remove component Server 1   
           System.out.println("***** Removing component Server1");
           try {
               server_home_1.remove_component(server_1);
           } catch (org.omg.Components.RemoveFailure e) {
                fail("Exception removing component Server1: " +e); 
           } catch (org.omg.CORBA.SystemException e) {
                fail("Exception removing component Server1: " +e); 
           }
            // wait requests to be recorded
            try {
                cdmw.ossupport.OS.sleep(2000);
            } catch (InterruptedException e) {
                // ignore
            }
             
            // Server 3
            TestRecording testRecording_3 =
                    this.sessionContext.get_connection_test_server3();
            //  get_component (uncomment instructions below)
            // fail("!!!!!!!!!!!! Waiting for _get_component to be added in JacOrb !!!!!!!!!");
//            com.acme.TestCcmRecording.Server server_3 = com.acme.TestCcmRecording.ServerHelper.narrow(testRecording_3);
            org.omg.CORBA.Object s3__ = testRecording_3._get_component();
            if (s3__ == null)
            {
               fail ("Failed to get a component reference using _get_component()!");
            }
            com.acme.TestCcmRecording.Server server_3 =
                    com.acme.TestCcmRecording.ServerHelper.narrow(s3__);
            if (server_3 == null)
            {
              fail("testRecording_3 is not a com.acme.TestCcmRecording.Server!");
            }
            org.omg.Components.CCMHome home_3 = server_3.get_ccm_home();
            com.acme.TestCcmRecording.ServerHome server_home_3 = com.acme.TestCcmRecording.ServerHomeHelper.narrow(home_3);
            if (server_home_3 == null)
            {
               fail("home_3 is not a com.acme.TestCcmRecording.ServerHome!");
            }
            // Remove component Server 3   
            System.out.println("Removing component Server3");
            try {
                server_home_3.remove_component(server_3);
            } catch (org.omg.Components.RemoveFailure e) {
                 fail("Exception removing component Server3: " +e); 
            } catch (org.omg.CORBA.SystemException e) {
                 fail("Exception removing component Server3: " +e); 
            }
            // wait requests to be recorded
            try {
                cdmw.ossupport.OS.sleep(2000);
            } catch (InterruptedException e) {
                // ignore
            }
            
             // clear recordList content 
            ccmClientImpl.recordList.clear();
            
            System.out.println("Send request to Server1 after it has been removed");
            try {
                TestRecording testRecording =
                    this.sessionContext.get_connection_test_server1();
                    
                testRecording.test_operation("MESSAGE TO SERVER1");
                
                // Dirty trick to get ORB (required for Valuetypes unmarshalling)
                this.orb = ((org.omg.CORBA.portable.ObjectImpl) testRecording)._orb();

            } catch (org.omg.CORBA.SystemException e) {
                System.out.println("!!! Exception calling Server1's facet: " + e);
            }
            
            System.out.println("Send request to Server3 after it has been removed");
            try {
                TestRecording testRecording =
                    this.sessionContext.get_connection_test_server3();
                    
                testRecording.test_operation("MESSAGE TO SERVER3");
                
            } catch (org.omg.CORBA.SystemException e) {
                System.out.println("!!! Exception calling Server3's facet: " +e); 
            }
            
            
            // wait requests to be recorded
            try {
                cdmw.ossupport.OS.sleep(2000);
            } catch (InterruptedException e) {
                // ignore
            }
            
            System.out.println("Check there are no received records");
            // Check if there are non-expected records
            if (! ccmClientImpl.recordList.isEmpty()) {
                System.out.println("Following records were not expected:");
                i=1;
                for (RequestReplyPair pair : ccmClientImpl.recordList) {
                    System.out.println("  - " + (i++) + ": \"" +
                                       pair.request.operation + "\" from " +
                                       pair.request.server_id);
                }
                fail("Some records were not expected");
            }
        }
        
        
        private boolean find_test_operation_record(String componentName,
                                                   String message)
        {
            for (RequestReplyPair pair : ccmClientImpl.recordList)
            {
                // test is record is for a "test_operation"
                if (pair.request.operation.equals("test_operation")) {
                    
                    // check if server_id ends with componentName
                    if (pair.request.server_id.endsWith(componentName)) {
                        
                        // try to unmarshall parameters as a string
                        boolean endian = (pair.request.head.flags!=0 ? true : false);
                        org.omg.CORBA.portable.InputStream input =
                            new org.jacorb.orb.CDRInputStream(this.orb, 
                                                              pair.request.data, 
                                                              endian);
                        try {
                            String str = input.read_string();
                            
                            // unmarshall OK. Test if it's expected message
                            if (message.equals(str)) {
                                // message found, remove it from list
                                ccmClientImpl.recordList.remove(pair);
                                return true;
                            }
                        } catch (org.omg.CORBA.MARSHAL e) {
                            System.out.println( 
                                "record for \"test_operation\" found, " +
                                "but parameter is not a string!");
                        }
                    }
                    else
                    {
                        System.out.println(
                            "record for \"test_operation\" found, but server_id \"" +
                            pair.request.server_id + "\" doesn't end with \"" + 
                            componentName + "\"");
                    }
                }
            }
            
            return false;
        }
        
        private boolean find_push_event_record(String componentName,
                                               String message)
        {
            for (RequestReplyPair pair : ccmClientImpl.recordList)
            {
                // test is record is for a "push_event"
                if (pair.request.operation.equals("push_event")) {
        
                    // check if server_id ends with componentName
                    if (pair.request.server_id.endsWith(componentName)) {
                        
                        // try to unmarshall parameters as a TestEvent
                        boolean endian = (pair.request.head.flags!=0 ? true : false);
                        org.omg.CORBA.portable.InputStream input =
                            new org.jacorb.orb.CDRInputStream(this.orb, 
                                                              pair.request.data, 
                                                              endian);
                        try {
                            TestEvent evt = TestEventHelper.read(input);
                            
                            // unmarshall OK. Test if it's expected message
                            if (message.equals(evt.message)) {
                                // message found, remove it from list
                                ccmClientImpl.recordList.remove(pair);
                                return true;
                            }
                        } catch (org.omg.CORBA.MARSHAL e) {
                            System.out.println( 
                                "record for \"push_event\" found, " +
                                "but parameter is not a TestEvent!");
                        }
                    }
                    else
                    {
                        System.out.println(
                            "record for \"push_event\" found, but server_id \"" +
                            pair.request.server_id + "\" doesn't end with \"" + 
                            componentName + "\"");
                    }
                }
            }
            
            return false;
        }
        
    }
    
}

