/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.testccmservices;

import com.thalesgroup.CdmwPlatformMngtBase.*;
import com.thalesgroup.CdmwPlatformMngt.ManagedElementContexts.*;
import com.thalesgroup.CdmwPlatformMngt.RedundancyEltState;
import com.thalesgroup.CdmwPlatformMngt.Event;
import com.thalesgroup.CdmwPlatformMngt.EventKind;
import com.thalesgroup.CdmwPlatformMngt.SystemStatus;
import com.thalesgroup.CdmwPlatformMngt.ApplicationStatus;
import com.thalesgroup.CdmwPlatformMngt.ProcessStatus;
import com.thalesgroup.CdmwPlatformMngt.HostStatus;
import com.thalesgroup.CdmwPlatformMngt.TechnicalStatus;
import com.thalesgroup.CdmwPlatformMngt.FunctioningMode;
import com.thalesgroup.CdmwPlatformMngt.ExitEvent;
import com.acme.TestCcmServices.*;

//
// IDL:acme.com/TestCcmServices/CCM_Client:1.0
//
public class CCMClientImpl extends org.omg.CORBA.LocalObject
    implements Session_CCM_Client 
{
    
    static final String SYSTEM_NAME = "system_deployment";
    static final String APPLICATION_NAME = "CcmServicesApplication";
    static final String CLIENT_PROCESS_NAME = "ClientProcess";
    static final String SERVER_PROCESS1_NAME = "ServerProcess1";

    static final String CLIENT_COMPONENT_NAME = "Client1";
    static final String SERVER_COMPONENT1_NAME = "Server1";
  
    static final String CLIENT_ISSUER_NAME = "ClientProcess/Client1";
    static final String SERVER_ISSUER1_NAME = "ServerProcess1/Server1";
  
    static final String CLIENT_NOTIFICATION_MESSAGE = 
        "Notification message from Client";
    static final int CLIENT_USER_EVENT = 33;

    static final String SERVER_NOTIFICATION_MESSAGE = 
        "Notification message from Client";
    static final String SERVER_NOTIFICATION_FATAL = 
        "Fatal error from Client";
    static final int SERVER_USER_EVENT = 44;
    static final int SERVER_EXIT_CODE = 66;
    
    
    
    // session context
    private CCM_Client_Context m_sessionContext;
     
    
    // Testable thread
    private TestableThread m_testThread;

    // Received events queue    
    java.util.LinkedList<Event> m_eventLinkedList;
    
    
    public CCMClientImpl() 
    {
        this.m_sessionContext = null;
        this.m_testThread = null;
        m_eventLinkedList = new java.util.LinkedList<Event>();
    }

    
    public void set_session_context(org.omg.Components.SessionContext ctx) 
    {
        System.out.println("   ... CCMClientImpl.set_session_context called!");
        this.m_sessionContext = CCM_Client_ContextHelper.narrow(ctx);
    }
    
    public CCM_Client_Context getContext()
    {
        return this.m_sessionContext;
    }
    
    
    public void ccm_activate() 
    {
        System.out.println("   ... CCMClientImpl.ccm_activate called!");

        // Get this component CORBA object
        // Note: this has to be performed in a CORBA request thread; thus
        //       not in test which is a specific thread.
        Client thisComponent = null;
        try {
            // narrow context in SessionContext for get_CCM_objet() access
            org.omg.Components.SessionContext ctx =
                org.omg.Components.SessionContextHelper.narrow(m_sessionContext);
            
            // call SessionContext.get_CCM_objet()
            org.omg.CORBA.Object obj = ctx.get_CCM_object();
                
            thisComponent = ClientHelper.narrow(obj);
                
            cdmw.common.Assert.check(thisComponent != null);
                
        } catch (org.omg.CORBA.UserException e) {
            System.err.println("UserException during get_CCM_Object() :" + e); 
            cdmw.common.Assert.check(false);
        } catch (org.omg.CORBA.SystemException e) {
            System.err.println("SystemException during get_CCM_Object() :" + e); 
            cdmw.common.Assert.check(false);
        }

        try {
            this.m_testThread = new TestableThread();
            
            // save context for usage in test
            this.m_testThread.setSessionContext(getContext());
            
            // save this component for usage in test
            this.m_testThread.setComponentRef(thisComponent);
            
            // save this executor in CORBATestManager for usage in test
            this.m_testThread.setCCMClient(this);
        
            this.m_testThread.start();
            
        } catch (org.omg.CORBA.SystemException e) {
            System.err.println("CORBA SystemException thrown during test: " + e);
        } catch (Exception e) {
            System.err.println("Exception thrown during test: " + e);
        }
    }
    
    public void ccm_passivate() 
    {
        System.out.println("   ... CCMClientImpl.ccm_passivate called!");
        if (m_testThread != null) {
            // wait test to be finished
            m_testThread.join();
            m_testThread = null;
        }
        
    }
    
    public void ccm_remove() 
    {
        System.out.println("   ... CCMClientImpl.ccm_remove called!");
    }
    
    
    private String eventKindToString(Event evt)
    {
        switch (evt.event_kind.value()) {
            case EventKind._SYSTEM_MODE_CHANGE:
                return "SYSTEM_MODE_CHANGE";
            case EventKind._SYSTEM_STATUS_CHANGE:
                return "SYSTEM_STATUS_CHANGE";
            case EventKind._HOST_STATUS_CHANGE:
                return "HOST_STATUS_CHANGE";
            case EventKind._APPLICATION_AGENT_DEATH:
                return "APPLICATION_AGENT_DEATH";
            case EventKind._APPLICATION_MODE_CHANGE:
                return "APPLICATION_MODE_CHANGE";
            case EventKind._APPLICATION_STATUS_CHANGE:
                return "APPLICATION_STATUS_CHANGE";
            case EventKind._PROCESS_MESSAGE:
                return "PROCESS_MESSAGE";
            case EventKind._PROCESS_STATUS_CHANGE:
                return "PROCESS_STATUS_CHANGE";
            //PCR-1257 commented code to be deleted
            /*case EventKind._SYSTEM_ENTITY_STATUS_CHANGE:
                return "SYSTEM_ENTITY_STATUS_CHANGE";
            case EventKind._HOST_ENTITY_STATUS_CHANGE:
                return "HOST_ENTITY_STATUS_CHANGE";
            case EventKind._APPLICATION_ENTITY_STATUS_CHANGE:
                return "APPLICATION_ENTITY_STATUS_CHANGE";
            case EventKind._PROCESS_ENTITY_STATUS_CHANGE:
                return "PROCESS_ENTITY_STATUS_CHANGE";*/
            case EventKind._ENTITY_STATUS_CHANGE:
                return "ENTITY_STATUS_CHANGE";
            case EventKind._CONFIGURATION_CHANGE:
                return "CONFIGURATION_CHANGE";
            case EventKind._USER_EVENT:
                return "USER_EVENT";
            case EventKind._PROPERTY_CHANGE:
                return "PROPERTY_CHANGE";
            case EventKind._RESTART_REQUESTED:
                return "RESTART_REQUESTED";
            case EventKind._REBOOT_REQUESTED:
                return "REBOOT_REQUESTED";
            case EventKind._STOP_REQUESTED:
                return "STOP_REQUESTED";
            default:
                return "unknown event kind !";
        }
    }
    
    
    //
    // SupervisionObserver.notify() implementation
    //
    public void _notify(Event event)
    {
        String msg = event.to_string();
        System.out.println("   ... " + eventKindToString(event)
                           + " notification received from Supervision : \n" 
                           + msg);
    
        // save event in event queue
        m_eventLinkedList.offer(event);
    }
    
    //
    // return last received event
    //
    public Event getLastEvent()
    {
        // get last event from event queue or null if empty
        try {
            return m_eventLinkedList.removeLast();
        } catch (java.util.NoSuchElementException  e) {
            return null;
        }
    }
    
    //
    // return first received event
    //
    public Event getFirstEvent()
    {
        // get first event from event queue or null if empty
        try {
            return m_eventLinkedList.removeFirst();
        } catch (java.util.NoSuchElementException  e) {
            return null;
        }
    }
    
    //
    // clear event list
    //
    public void clearEventList()
    {
        m_eventLinkedList.clear();
    }
    
    
    //
    // TestableThread class
    //
    class TestableThread extends cdmw.testutils.Testable
        implements Runnable
    {
        // thread running this test
        Thread m_thread;

        private CCMClientImpl m_CCMClient;
        private CCM_Client_Context m_sessionContext;
        private Client m_componentRef;
        
        public TestableThread()
        {
            this.name = "Test CCM Services";
            this.out = System.out;
            
            this.m_thread = new Thread(this);
            this.m_CCMClient = null;
            this.m_sessionContext = null;
            this.m_componentRef = null;
        }
        
        public void setCCMClient(CCMClientImpl ccmClient)
        {
            this.m_CCMClient = ccmClient;
        }
        
        public void setSessionContext(CCM_Client_Context sessionContext)
        {
            this.m_sessionContext = sessionContext;
        }
        
        public void setComponentRef(Client componentRef)
        {
            this.m_componentRef = componentRef;
        }
        
        // Runnable.run() operation
        public void run()
        {
            // call Testable.start()
            super.start();
        }
        
        // Thread.start() like operation
        public void start()
        {
            m_thread.start();
        }
        
        // Thread.join() like operation
        public void join()
        {
            try {
                m_thread.join();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
        
        // Testable.doTests() operation
        public void doTests() 
        {
            setNbOfRequestedTestOK(22);
            
            cdmw.common.Assert.check(m_CCMClient != null);
            cdmw.common.Assert.check(m_sessionContext != null);
            cdmw.common.Assert.check(m_componentRef != null);
            
            //
            // Test Logger service
            //
            println("Testing LoggerFactory QoS Service");
            try {
                // get LoggerFactory
                org.omg.CORBA.Object obj = 
                    m_sessionContext.resolve_service_reference("LoggerFactory");
                com.thalesgroup.CdmwLogging.LoggerFactory loggerFactory =
                    com.thalesgroup.CdmwLogging.LoggerFactoryHelper.narrow(obj);
                // get Logger for TEST_DOMAIN
                com.thalesgroup.CdmwLogging.Logger logger =
                    loggerFactory.get_logger("TEST_DOMAIN");
                
                // log a message
                cdmw.logging.Logger.CDMW_INFO(
                    logger, "Test logging from component", new Throwable());
                
                // log a warnng
                cdmw.logging.Logger.CDMW_WARN(
                    logger, "Test warning from component", new Throwable());
                
                // log an error
                cdmw.logging.Logger.CDMW_ERROR(
                    logger, "Test error from component", new Throwable());
                
            } catch (org.omg.Components.CCMException e) {
                println("LoggerFactory no found !!");
                check(false);
            }
        
            // wait 5sec System to be started, since we will test system status 
            // and we expect its SYSTEM_STARTED
            println("Wait 5 seconds System reach SYSTEM_STARTED status");
            try {
                cdmw.ossupport.OS.sleep(5000);
            } catch (InterruptedException e) {}

        
            println("Testing ManagedComponentContext QoS Service");
            String realComponentName;
            try {
                // get ManagedComponentContext
                org.omg.CORBA.Object obj = 
                    m_sessionContext.resolve_service_reference("ManagedComponentContext");
                ComponentContext supContext =
                    ComponentContextHelper.narrow(obj);
                check(supContext != null);
                
                println("   test get names");
                // get component, process, application names
                String component_name = supContext.managed_element_name();
                println("      component_name = '" 
                        + (component_name == null ? "null" : component_name)
                        + "'");
                check(component_name != null);
                check(CLIENT_COMPONENT_NAME.equals(component_name));        
                
                String process_name = supContext.process_name();
                println("      process_name = '" 
                        + (process_name == null ? "null" : process_name)
                        + "'");
                check(process_name != null);
                check(CLIENT_PROCESS_NAME.equals(process_name));        
        
                String appli_name = supContext.application_name();
                println("      appli_name = '" 
                        + (appli_name == null ? "null" : appli_name)
                        + "'");
                check(appli_name != null);
                check(APPLICATION_NAME.equals(appli_name));        
        
                // get System supervision object
                println("   get System supervision object");
                com.thalesgroup.CdmwPlatformMngt.External.System system = 
                    supContext.the_system();
        
                // register this component as System observer
                println("   register this component as System Observer");
                com.thalesgroup.CdmwPlatformMngt.SupervisionObserver old_one =
                    system.register_observer("TestCcmServices_Client", 
                                              m_componentRef);
                
                // test System supervision object
                println("   test System status");
                org.omg.CORBA.StringHolder status_info = 
                    new org.omg.CORBA.StringHolder();
                check(system.get_status(status_info) == 
                      SystemStatus.SYSTEM_STARTED);
                println("      status info: " + status_info.value);
                                              
                // test Application supervision object
                println("   test Application status");
                com.thalesgroup.CdmwPlatformMngt.External.Application application =
                    supContext.the_application();
                check(application.get_status(status_info) == 
                               ApplicationStatus.APPLICATION_RUNNING);
                println("      status info: " + status_info.value);
                                              
                // test Process supervision object
                println("   test Process status");
                com.thalesgroup.CdmwPlatformMngt.External.Process process =
                    supContext.the_process();
                check(process.get_status(status_info) == 
                               ProcessStatus.PROCESS_RUNNING);
                println("      status info: " + status_info.value);
                                              
                // test Host supervision object
                println("   test Host status");
                com.thalesgroup.CdmwPlatformMngt.External.Host host =
                    supContext.the_host();
                check(host.status() == HostStatus.HOST_STARTED);
                println("      status info: HOST_STARTED");
                                              
                // test message notification to Supervision
                println("   notify message to supervision");
                supContext.notify_message(INF.value,
                                          CLIENT_NOTIFICATION_MESSAGE);
                try {
                    cdmw.ossupport.OS.sleep(1000);
                } catch (InterruptedException e) {}
                                           
                println("      check that notification has been received back");
                Event event = m_CCMClient.getLastEvent();
                check(event != null);
                check(event.event_kind == EventKind.PROCESS_MESSAGE);
                com.thalesgroup.CdmwPlatformMngt.ProcessMessage messageEvent =
                    (com.thalesgroup.CdmwPlatformMngt.ProcessMessage) event;
                check(CLIENT_ISSUER_NAME.equals(messageEvent.issuer));
                check(CLIENT_NOTIFICATION_MESSAGE.equals(messageEvent.info));
                
               // test user event notification to Supervision
               println("   notify user event to supervision");
               org.omg.CORBA.Any any = org.omg.CORBA.ORB.init().create_any();
               any.insert_ulong(CLIENT_USER_EVENT);
               supContext.notify_user_event(any);
               try {
                   cdmw.ossupport.OS.sleep(1000);
               } catch (InterruptedException e) {}
                                          
               println("      check that notification has been received back");
               event = m_CCMClient.getLastEvent();
               check(event != null);
               check(event.event_kind == EventKind.USER_EVENT);
               com.thalesgroup.CdmwPlatformMngt.UserEvent user_event =
                   (com.thalesgroup.CdmwPlatformMngt.UserEvent) event;
               int value = 0;
               try {
                   value = user_event.data.extract_ulong();
                   check(value == CLIENT_USER_EVENT);
               } catch(org.omg.CORBA.BAD_OPERATION e) {
                   fail("UserEvent's data doesn't contain a CORBA.ULong");
               }
               check(CLIENT_ISSUER_NAME.equals(user_event.issuer));
               check(APPLICATION_NAME.equals(user_event.application_name));
               check(CLIENT_PROCESS_NAME.equals(user_event.process_name));
               check(cdmw.ossupport.OS.getHostname().equals(user_event.host_name));
                
                
                //
                // test with Server components
                //
                println("Test ManagedComponentContext in Server components");
                
                // test with Server1
                println(" tests on Server1");
                test_server_ft_state();
                test_server_notify(SERVER_ISSUER1_NAME,
                                   SERVER_PROCESS1_NAME);

                // Assembly (and thus Client) is started twice:
                //  - 1st time, test_server_fatal_error() is performed
                //  - 2nd time, test_server_exit() is performed
                final String FIRST_TEST_PERFORMED_PROPERTY =
                    "cdmw.testccmservices.test_server_fatal_error.performed";
                if (System.getProperty(FIRST_TEST_PERFORMED_PROPERTY) == null) {

                    println("    test notify_fatal_error");
                    test_server_fatal_error(SERVER_ISSUER1_NAME,
                                            SERVER_PROCESS1_NAME,
                                            true);    /*DEGRADED_MODE notif expected*/
                    System.setProperty(FIRST_TEST_PERFORMED_PROPERTY, "true");

                } else {
                    
                    println("    test exit");
                    test_server_exit(SERVER_ISSUER1_NAME,
                                     SERVER_PROCESS1_NAME,
                                     system,
                                     false);    /* system already degraded by 1st test*/
                }
                
                println("Test ended.");
                                           
            } catch (org.omg.Components.CCMException e) {
                println("ManagedComponentContext no found !!");
                e.printStackTrace();
                check(false);
            } catch (org.omg.CORBA.UserException e) {
                println("UserException during ManagedComponentContext test : " + e);
                e.printStackTrace();
                check(false);
            } catch (org.omg.CORBA.SystemException e) {
                println("SystemException during ManagedComponentContext test : " + e);
                e.printStackTrace();
                check(false);
            } catch (Exception e) {
                println("Eexception during ManagedComponentContext test : " + e);
                e.printStackTrace();
                check(false);
            }
        }
        

        void test_server_ft_state()
        {
            addNbOfRequestedTestOK(2);
            
            // get connected server
            TestServices testServices =
                m_sessionContext.get_connection_test_services();
                
            // test get_initial_ft_state()
            println("   test initial FTState was NOT_FT");
            check(testServices.get_initial_ft_state() == FTState.NOT_FT);
            
            // test get_actual_ft_state()
            println("   test actual FTState is NOT_FT");
            check(testServices.get_actual_ft_state() == FTState.NOT_FT);
        }
            
        
        void test_server_notify(String expectedServerIssuerName,
                                String expectedServerProcessName)
        {
            addNbOfRequestedTestOK(10);
            
            // get connected server
            TestServices testServices =
                m_sessionContext.get_connection_test_services();
                
            // test call_notify_message()
            println("   test notify_message from Server");
            testServices.call_notify_message(INF.value,
                                              SERVER_NOTIFICATION_MESSAGE);
            try {
                cdmw.ossupport.OS.sleep(1000);
            } catch (InterruptedException e) {}
            println("      check that notification has been received back");
            Event event = m_CCMClient.getLastEvent();
            check(event != null);
            check(event.event_kind == EventKind.PROCESS_MESSAGE);
            com.thalesgroup.CdmwPlatformMngt.ProcessMessage messageEvent =
               (com.thalesgroup.CdmwPlatformMngt.ProcessMessage) event;
            check(expectedServerIssuerName.equals(messageEvent.issuer));
            check(SERVER_NOTIFICATION_MESSAGE.equals(messageEvent.info));
        
           // test call_notify_user_event()
           println("   test notify_user_event from Server");
           org.omg.CORBA.Any any = org.omg.CORBA.ORB.init().create_any();
           any.insert_ulong(SERVER_USER_EVENT);
           testServices.call_notify_user_event(any);
           try {
               cdmw.ossupport.OS.sleep(1000);
           } catch (InterruptedException e) {}
           println("      check that notification has been received back");
           event = m_CCMClient.getLastEvent();
           check(event != null);
           check(event.event_kind == EventKind.USER_EVENT);
           com.thalesgroup.CdmwPlatformMngt.UserEvent userEvent =
              (com.thalesgroup.CdmwPlatformMngt.UserEvent) event;
           int value;
           try {
               value = userEvent.data.extract_ulong();
               check(value == SERVER_USER_EVENT);
           } catch(org.omg.CORBA.BAD_OPERATION e) {
               fail("UserEvent's data doesn't contain a CORBA.ULong");
           }
           check(expectedServerIssuerName.equals(userEvent.issuer));
           check(APPLICATION_NAME.equals(userEvent.application_name));
           check(expectedServerProcessName.equals(userEvent.process_name));
        }
        
        
        void test_server_fatal_error(
                String expectedServerIssuerName,
                String expectedServerProcessName,
                boolean expectDegradedNotif)
        {
            addNbOfRequestedTestOK(5);

            // get connected server
            TestServices testServices =
                m_sessionContext.get_connection_test_services();
                
            // clear event list
            m_CCMClient.clearEventList();
                
            // test call_notify_fatal_error()
            println("   test notify_fatal_error from Server");
            testServices.call_notify_fatal_error(SERVER_NOTIFICATION_FATAL);
            try {
                cdmw.ossupport.OS.sleep(1000);
            } catch (InterruptedException e) {}
            println("      check that notification has been received back");
            Event event = m_CCMClient.getFirstEvent();
            check(event != null);
        
            // first event should be ProcessStatusChanged
            check(event.event_kind == EventKind.PROCESS_STATUS_CHANGE);
            com.thalesgroup.CdmwPlatformMngt.ProcessStatusChange fatalEvent =
                (com.thalesgroup.CdmwPlatformMngt.ProcessStatusChange) event;
            check(expectedServerIssuerName.equals(fatalEvent.issuer));
            check(SERVER_NOTIFICATION_FATAL.equals(fatalEvent.info));
            check(fatalEvent.process_status == ProcessStatus.PROCESS_FAILED_ERROR);
        
            if (expectDegradedNotif) {
                addNbOfRequestedTestOK(7);
                // we expect 2 MODE_CHANGE to DEGRADED notifications
            
                //  - Test application MODE_CHANGE to DEGRADED
                println("      check application mode changed to DEGRADED");
                event = m_CCMClient.getFirstEvent();
                check(event != null);
                check(event.event_kind == EventKind.APPLICATION_MODE_CHANGE);
                com.thalesgroup.CdmwPlatformMngt.ApplicationModeChange appChangeEvent =
                    (com.thalesgroup.CdmwPlatformMngt.ApplicationModeChange) event;
                check(APPLICATION_NAME.equals(appChangeEvent.application_name));
                check(appChangeEvent.application_mode == FunctioningMode.DEGRADED_MODE);
        
                //  - Test system MODE_CHANGE to DEGRADED
                println("      check system mode changed to DEGRADED");
                event = m_CCMClient.getFirstEvent();
                check(event != null);
                check(event.event_kind == EventKind.SYSTEM_MODE_CHANGE);
                com.thalesgroup.CdmwPlatformMngt.SystemModeChange sysChangeEvent =
                    (com.thalesgroup.CdmwPlatformMngt.SystemModeChange) event;
                check(sysChangeEvent.system_mode == FunctioningMode.DEGRADED_MODE);
            }
        
        }
        
        void test_server_exit(
                String expectedServerIssuerName,
                String expectedServerProcessName,
                com.thalesgroup.CdmwPlatformMngt.External.System system,
                boolean expectDegradedNotif)
        {
            addNbOfRequestedTestOK(7);
                
            // get connected server
            TestServices testServices =
                m_sessionContext.get_connection_test_services();
                
            // clear event list
            m_CCMClient.clearEventList();
                
            // test call_exit()
            println("   test exit from Server");
            testServices.call_exit(SERVER_EXIT_CODE);
            try {
                cdmw.ossupport.OS.sleep(1000);
            } catch (InterruptedException e) {}
            println("      check that notification has been received back");
            Event event = m_CCMClient.getFirstEvent();
            check(event != null);
        
            // first event should be ProcessStatusChanged
            check(event.event_kind == EventKind.PROCESS_STATUS_CHANGE);
            com.thalesgroup.CdmwPlatformMngt.ProcessStatusChange fatalEvent =
                (com.thalesgroup.CdmwPlatformMngt.ProcessStatusChange) event;
            check(expectedServerIssuerName.equals(fatalEvent.issuer));
            check(SERVER_NOTIFICATION_FATAL.equals(fatalEvent.info));
            check(fatalEvent.process_status == ProcessStatus.PROCESS_FAILED_ERROR);
            // get Supervision object for process
            String elementPath =
               "s.sys/" + APPLICATION_NAME + ".appl/" +
               ".host" + expectedServerProcessName + ".proc";
            println("      get_element " + elementPath);
            com.thalesgroup.CdmwPlatformMngt.ManagedElement elt = 
                system.get_element(elementPath);
            com.thalesgroup.CdmwPlatformMngt.External.Process process =
                com.thalesgroup.CdmwPlatformMngt.External.ProcessHelper.narrow(elt);
            com.thalesgroup.CdmwPlatformMngt.ProcessExitInfo exitInfo = 
                process.get_last_exit_code();
            check(exitInfo.exit_event == ExitEvent.EXITED);
            check(exitInfo.event_value == SERVER_EXIT_CODE);
            
            
            if (expectDegradedNotif) {
                addNbOfRequestedTestOK(7);
                // we expect 2 MODE_CHANGE to DEGRADED notifications
                // and 2 STATUS_CHANGE
                
                //  - Test application MODE_CHANGE to DEGRADED
                println("      check application mode changed to DEGRADED");
                event = m_CCMClient.getLastEvent();
                check(event != null);
                check(event.event_kind == EventKind.APPLICATION_MODE_CHANGE);
                com.thalesgroup.CdmwPlatformMngt.ApplicationModeChange appChangeEvent =
                    (com.thalesgroup.CdmwPlatformMngt.ApplicationModeChange) event;
                check(APPLICATION_NAME.equals(appChangeEvent.application_name));
                check(appChangeEvent.application_mode == FunctioningMode.DEGRADED_MODE);
                
                //  - Test system MODE_CHANGE to DEGRADED
                println("      check system mode changed to DEGRADED");
                event = m_CCMClient.getLastEvent();
                check(event != null);
                check(event.event_kind == EventKind.SYSTEM_MODE_CHANGE);
                com.thalesgroup.CdmwPlatformMngt.SystemModeChange sysChangeEvent =
                    (com.thalesgroup.CdmwPlatformMngt.SystemModeChange) event;
                check(sysChangeEvent.system_mode == FunctioningMode.DEGRADED_MODE);
            }
        }
        
    }

    
}

