/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.testccmservices;

import com.thalesgroup.CdmwPlatformMngt.ManagedElementContexts.*;
import com.thalesgroup.CdmwPlatformMngt.RedundancyEltState;
import com.acme.TestCcmServices.*;

//
// IDL:acme.com/TestCcmServices/CCM_Server:1.0
//
public class CCMServerImpl extends org.omg.CORBA.LocalObject
    implements Session_CCM_Server 
{
    
    // session context
    private CCM_Server_Context m_sessionContext;
     
    // initial FTState at start-up 
    private FTState m_FTInitialState;
    
    // Supervision's ProcessContext
    private ProcessContext m_processContext;
    
    // Supervision's ComponentContext
    private ComponentContext m_componentContext;
    
    
    public CCMServerImpl() 
    {
        this.m_sessionContext = null;
        this.m_FTInitialState = FTState.NOT_FT;
        this.m_processContext = null;
        this.m_componentContext = null;
    }

    
    public void set_session_context(org.omg.Components.SessionContext ctx) 
    {
        System.out.println("   ... CCMServerImpl::set_session_context called!");
        this.m_sessionContext = CCM_Server_ContextHelper.narrow(ctx);
            
        // get initial FT State via ProcessContext
        try {
            org.omg.CORBA.Object obj = 
                m_sessionContext.resolve_service_reference("ManagedProcessContext");
            m_processContext= ProcessContextHelper.narrow(obj);
            
            switch (m_processContext.get_process_redundancy_state().value()) {
                case RedundancyEltState._REDSTATE_NORG:
                    System.out.println("   ... initial FT state: NOT_FT");
                    m_FTInitialState = FTState.NOT_FT;
                    break;
                
                case RedundancyEltState._REDSTATE_PRIMARY:
                    System.out.println("   ... initial FT state: PRIMARY");
                    m_FTInitialState = FTState.PRIMARY;
                    break;
                
                case RedundancyEltState._REDSTATE_PASSIVE:
                    System.out.println("   ... initial FT state: BACKUP");
                    m_FTInitialState = FTState.BACKUP;
                    break;
                
                default:
                    // shouldn't happen
                    cdmw.common.Assert.check(false);
            }
        } catch (org.omg.Components.CCMException e) {
            System.err.println("ERROR !!! ManagedProcessContext no found !!");
            System.err.println("Throw CORBA::INTERNAL");
            throw new org.omg.CORBA.INTERNAL();
        } catch (org.omg.CORBA.UserException e) {
            System.err.println("ERROR !!! UserException getting ManagedProcessContext :" + e);
            System.err.println("Throw CORBA::INTERNAL");
            throw new org.omg.CORBA.INTERNAL();
        } catch (org.omg.CORBA.SystemException e) {
            System.err.println("ERROR !!! SystemException getting ManagedProcessContext :" + e);
            System.err.println("Throw CORBA::INTERNAL");
            throw new org.omg.CORBA.INTERNAL();
        }
    }
    
    public CCM_Server_Context getContext()
    {
        return this.m_sessionContext;
    }
    
    
    public void ccm_activate() 
    {
        System.out.println("   ... CCMServerImpl::ccm_activate called!");

        // get ComponentContext
        try {
            org.omg.CORBA.Object obj = 
                m_sessionContext.resolve_service_reference("ManagedComponentContext");
            m_componentContext = ComponentContextHelper.narrow(obj);
            
            cdmw.common.Assert.check(m_componentContext != null);
            
        } catch (org.omg.Components.CCMException e) {
            System.err.println("ERROR !!! ManagedComponentContext no found !!");
            System.err.println("Throw CORBA::INTERNAL");
            throw new org.omg.CORBA.INTERNAL();
        } catch (org.omg.CORBA.UserException e) {
            System.err.println("ERROR !!! UserException getting ManagedComponentContext :" + e);
            System.err.println("Throw CORBA::INTERNAL");
            throw new org.omg.CORBA.INTERNAL();
        } catch (org.omg.CORBA.SystemException e) {
            System.err.println("ERROR !!! SystemException getting ManagedComponentContext :" + e);
            System.err.println("Throw CORBA::INTERNAL");
            throw new org.omg.CORBA.INTERNAL();
        }
    }
    
    public void ccm_passivate() 
    {
        System.out.println("   ... CCMServerImpl::ccm_passivate called!");
    }
    
    public void ccm_remove() 
    {
        System.out.println("   ... CCMServerImpl::ccm_remove called!");
    }
    
    public CCM_TestServices get_test_services()
    {
        return this;
    }
    
    public FTState get_initial_ft_state()
    {
        // set_session_context should have been called
        cdmw.common.Assert.check(m_processContext != null);
        
        return m_FTInitialState;
    }
        
    public FTState get_actual_ft_state()
    {
        // set_session_context should have been called
        cdmw.common.Assert.check(m_processContext != null);
        
        try {
            switch (m_processContext.get_process_redundancy_state().value()) {
                case RedundancyEltState._REDSTATE_NORG:
                    return FTState.NOT_FT;
                
                case RedundancyEltState._REDSTATE_PRIMARY:
                    return FTState.PRIMARY;
                
                case RedundancyEltState._REDSTATE_PASSIVE:
                    return FTState.BACKUP;
                
                default:
                    // shouldn't happen
                    cdmw.common.Assert.check(false);
                    // for compiler
                    return FTState.NOT_FT;
            }
        } catch (org.omg.CORBA.SystemException e) {
            System.err.println(
                "ERROR !!! SystemException calling get_process_redundancy_state() :" + e);
            System.err.println("Throw CORBA::INTERNAL");
            throw new org.omg.CORBA.INTERNAL();
        }
    }
        
    
    public void call_notify_message(int level, String message)
    {
        // ccm_activate should have been called
        cdmw.common.Assert.check(m_componentContext != null);
        
        try {
            m_componentContext.notify_message(level, message);
            
        } catch (org.omg.CORBA.SystemException e) {
            System.err.println(
                "ERROR during call_notify_message() : " + e);
            System.err.println("Throw CORBA::INTERNAL");
            throw new org.omg.CORBA.INTERNAL();
        }
    }
      
    
    public void call_notify_fatal_error(String error_info)
    {
        // ccm_activate should have been called
        cdmw.common.Assert.check(m_componentContext != null);
        
        try {
            m_componentContext.notify_fatal_error(error_info);
            
        } catch (org.omg.CORBA.SystemException e) {
            System.err.println(
                "ERROR during call_notify_fatal_error() : " +e);
            System.err.println("Throw CORBA::INTERNAL");
            throw new org.omg.CORBA.INTERNAL();
        }
    }
      
    
    public void call_notify_user_event(org.omg.CORBA.Any data)
    {
        // ccm_activate should have been called
        cdmw.common.Assert.check(m_componentContext != null);
        
        try {
            m_componentContext.notify_user_event(data);
            
        } catch (org.omg.CORBA.SystemException e) {
            System.err.println(
                "ERROR during call_notify_user_event() : " + e);
            System.err.println("Throw CORBA::INTERNAL");
            throw new org.omg.CORBA.INTERNAL();
        }
    }
      
    
    public void call_exit(int exit_code)
    {
        // ccm_activate should have been called
        cdmw.common.Assert.check(m_componentContext != null);
        
        try {
            System.out.println("   ... CCMServerImpl exiting !!");
            m_processContext.exit(exit_code);
    
        } catch (org.omg.CORBA.SystemException e) {
            System.err.println(
                "ERROR during call_exit() : " + e);
            System.err.println("Throw CORBA::INTERNAL");
            throw new org.omg.CORBA.INTERNAL();
        }
    }
    
}

