/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.testccmtypechecking;

import com.acme.TestCcmTypeChecking.*;
import com.thalesgroup.CdmwPlatformMngt.ManagedElementContexts.*;


//
// IDL:acme.com/TestCcmTypeChecking/CCM_Client:1.0
//
public class CCMClientImpl extends org.omg.CORBA.LocalObject
    implements Session_CCM_Client 
{
    // session context
    private CCM_Client_Context sessionContext;
    
    // component name
    private String componentName;
     
    
    public CCMClientImpl() 
    {
        this.sessionContext = null;
        this.componentName = "UnamedComponent";
    }

    
    public void set_session_context(org.omg.Components.SessionContext ctx) 
    {
        this.sessionContext = CCM_Client_ContextHelper.narrow(ctx);
    }
    
    
    public void ccm_activate() 
    {
        try {
            // get ManagedComponentContext
            org.omg.CORBA.Object obj = 
                this.sessionContext.resolve_service_reference("ManagedComponentContext");
            ComponentContext supContext =
                ComponentContextHelper.narrow(obj);
         
            // get component, process, application names
            this.componentName = supContext.managed_element_name();
        } catch (org.omg.Components.CCMException e) {
            System.err.println("ManagedComponentContext not found !!");
        } catch (org.omg.CORBA.SystemException e) {
            System.err.println("SystemException getting ManagedComponentContext: " 
                      + e);
        }
    
        System.err.println(">>>>>>" + this.componentName + " is active");
        
        System.err.println("-----> test connection to Server1");
        try {
            TestTypeChecking test_typechecking =
                this.sessionContext.get_connection_test_server1();
                
            test_typechecking.test_operation(this.componentName);
            
            System.err.println("   ==> connection OK");
        } catch (org.omg.CORBA.SystemException e) {
            System.err.println("   ==> connection NOT OK: " + e);
        }
        
        
        System.err.println("-----> test connection to Server2");
        try {
            TestTypeChecking test_typechecking =
                this.sessionContext.get_connection_test_server2();
                
            test_typechecking.test_operation(this.componentName);
            
            System.err.println("   ==> connection OK");
        } catch (org.omg.CORBA.SystemException e) {
            System.err.println("   ==> connection NOT OK: " + e);
        }
        
        
        System.err.println("-----> test connection to Server3");
        try {
            TestTypeChecking test_typechecking =
                this.sessionContext.get_connection_test_server3();
                
            test_typechecking.test_operation(this.componentName);
            
            System.err.println("   ==> connection OK");
        } catch (org.omg.CORBA.SystemException e) {
            System.err.println("   ==> connection NOT OK: " + e);
        }
        
        
        System.err.println("-----> send event to all servers");
        try {
            TestEvent ev = new TestEventImpl(this.componentName);
        
            this.sessionContext.push_test_event(ev);
            
            System.err.println("   ==> event sent");
        } catch (org.omg.CORBA.SystemException e) {
            System.err.println("   ==> event not sent: " + e);
        }
    }
    
    public void ccm_passivate() 
    {
        System.err.println(">>>>>>" + this.componentName + " is passive");
    }
    
    public void ccm_remove() 
    {
        System.err.println(">>>>>>" + this.componentName + " is removed");
    }
    
        
}

