/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.testccmtypechecking;

import com.acme.TestCcmTypeChecking.*;
import com.thalesgroup.CdmwPlatformMngt.ManagedElementContexts.*;

//
// IDL:acme.com/TestCcmTypeChecking/CCM_Server:1.0
//
public class CCMServerImpl extends org.omg.CORBA.LocalObject
    implements Session_CCM_Server 
{
    
    // session context
    private CCM_Server_Context sessionContext;
     
    // component name
    private String componentName;
     
    
    public CCMServerImpl() 
    {
        this.sessionContext = null;
        this.componentName = "UnamedComponent";
    }

    
    public void set_session_context(org.omg.Components.SessionContext ctx) 
    {
        this.sessionContext = CCM_Server_ContextHelper.narrow(ctx);
            
    }
    
    
    public void ccm_activate() 
    {
        try {
            // get ManagedComponentContext
            org.omg.CORBA.Object obj = 
                this.sessionContext.resolve_service_reference("ManagedComponentContext");
            ComponentContext supContext =
                ComponentContextHelper.narrow(obj);
         
            // get component, process, application names
            this.componentName = supContext.managed_element_name();
        } catch (org.omg.Components.CCMException e) {
            System.err.println("ManagedComponentContext not found !!");
        } catch (org.omg.CORBA.SystemException e) {
            System.err.println("SystemException getting ManagedComponentContext: " 
                      + e);
        }
    
        System.out.println(">>>>>>" + this.componentName + " is active");
    }
    
    public void ccm_passivate() 
    {
        System.out.println(">>>>>>" + this.componentName + " is passive");
    }
    
    public void ccm_remove() 
    {
        System.out.println(">>>>>>" + this.componentName + " is removed");
    }
    
    public CCM_TestTypeChecking get_test_typechecking()
    {
        // component itself implements the test_typechecking facet
        return this;
    }
    
    public void test_operation(String message)
    {
        System.out.println("   *** " + this.componentName 
                           + " received request from " + message + " ***");
    }
        
    public void push_test_event(com.acme.TestCcmTypeChecking.TestEvent evt)
    {
        System.out.println("   *** " + this.componentName 
                           + " received event from " + evt.message + " ***");
    }
    
}

