/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.dinnerdemo;

import org.junit.Before;
import org.junit.Test;

import cdmw.common.Assert;
import cdmw.testutils.CORBATestManager;
import cdmw.testutils.TestManager;
import cdmw.ossupport.OS;
import cdmw.ossupport.Mutex;
import cdmw.ossupport.Timeval;

import static junit.framework.Assert.*;

//
// IDL:thalesgroup.com/Dinner/CCM_Philosopher:1.0
//
public class CCMPhilosopherImpl extends org.omg.CORBA.LocalObject
    implements com.thalesgroup.UserDinner.User_CCM_Philosopher {
    
    public String name = "";
    
    public volatile boolean isActive = false;
    
    public int state = 0;
    
    public int speed;
    
    public boolean haveLeftFork = false;
    
    public boolean haveRightFork = false;
    
    public com.acme.Dinner.CCM_Philosopher_Context context = null;
    
    public PhiloActivationThread activationThread;
    
    public Mutex mutex = new Mutex();
    
    private static java.util.Random rand;
    
    private static final String TEST_OBJECT_NAME = "cdmw.dinnerdemo.CCMPhilosopherImpl_object";
    
    public CCMPhilosopherImpl() {
        Timeval tv = OS.getTime();
        rand = new java.util.Random(tv.getMicroseconds() % 991);
    // PCR-0994: this.speed = 1500 + Math.abs((rand.nextInt() % 13) * 300);
        this.activationThread = new PhiloActivationThread(this);
    }
    
    /**
     *  This class provides an object to test the ccm_activate.
     */
    public static class CCMPhilosopherActivateTest { 
        
        private static final int MAX_NB_MEAL = 3;
        private CCMPhilosopherImpl philosopher = null;
        private int testerCount=0;
        @Before
        public void init() {
            this.testerCount++;
            this.philosopher = (CCMPhilosopherImpl) ((CORBATestManager) TestManager.getLocalInstance()).getLocalProperty(TEST_OBJECT_NAME + this.testerCount);
        }
        
        @Test
        public void doTests() {
            Assert.check(this.philosopher != null);
        
            System.out.println("Philo: Wait all Forks to be created...");
            try {
                OS.sleep(3000);
            } catch (InterruptedException ie) {
                System.out.println("Interrupted Exception raised: "
                    + ie.toString());
            }

            com.acme.Dinner.CCM_Philosopher_Context philosopherContext = 
                this.philosopher.get_context();

            System.out.println("Philo: Wait all Forks to be created...");
            boolean fork_ready = false;
            while (!fork_ready) {
                try {
                    com.acme.Dinner.Fork left = 
                        philosopherContext.get_connection_left();
                    com.acme.Dinner.Fork right = 
                        philosopherContext.get_connection_right();
                    
                    fork_ready = true;
                } catch (org.omg.CORBA.TRANSIENT e) {
                    System.out.println(" ... fork not yet ready ...");
                    try {
                        OS.sleep(3000);
                    } catch (InterruptedException ie) {
                        System.out.println("Interrupted Exception raised: "
                            + ie.toString());
                    }
                }
            }

            com.acme.Dinner.Registration registerServer = null;
    
            System.out.println("Philo: Testing ManagedComponentContext QoS Service");
            String real_component_name = null;
            try {
                // get ManagedComponentContext
                org.omg.CORBA.Object obj = 
                    philosopherContext.resolve_service_reference("ManagedComponentContext");
                com.thalesgroup.CdmwPlatformMngt.ManagedElementContexts.ComponentContext supContext =
                    com.thalesgroup.CdmwPlatformMngt.ManagedElementContexts.ComponentContextHelper.narrow(obj);

                // get component, process, application names
                real_component_name = supContext.managed_element_name();
                String process_name = supContext.process_name();
                String appli_name = supContext.application_name();
                System.out.println("--- Philosopher info from ManagedComponentContext:");
                System.out.println("  -     process name = " + process_name);
                System.out.println("  - application name = " + appli_name);
                System.out.println("  -   component name = " + real_component_name);

                // notify a message to Supervision
                String msg = "Greetings from Dr. " + real_component_name;
                supContext.notify_message(com.thalesgroup.CdmwPlatformMngtBase.INF.value,
                                          msg);
                
            } catch (org.omg.Components.CCMException e) {
                System.out.println("Philo: ManagedComponentContext no found !!");
            }

            System.out.println("Philo: Testing ManagedProcessContext QoS Service");
            try {
                // get ManagedProcessContext
                org.omg.CORBA.Object obj = 
                    philosopherContext.resolve_service_reference("ManagedProcessContext");
                com.thalesgroup.CdmwPlatformMngt.ManagedElementContexts.ProcessContext supContext =
                    com.thalesgroup.CdmwPlatformMngt.ManagedElementContexts.ProcessContextHelper.narrow(obj);

                // get RedundancyState
                com.thalesgroup.CdmwPlatformMngt.RedundancyEltState state =
                    supContext.get_process_redundancy_state();
                System.out.println("--- Philosopher info from ManagedProcessContext:");
                switch (state.value()) {
                    case com.thalesgroup.CdmwPlatformMngt.RedundancyEltState._REDSTATE_PRIMARY:
                        System.out.println("  - state = PRIMARY");
                        break;
                    case com.thalesgroup.CdmwPlatformMngt.RedundancyEltState._REDSTATE_PASSIVE:
                        System.out.println("  - state = BACKUP");
                        break;
                    case com.thalesgroup.CdmwPlatformMngt.RedundancyEltState._REDSTATE_NORG:
                        System.out.println("  - state = NOT FT");
                        break;
                    default:
                        System.out.println("  - state = ??? SHOULD'NT HAPPEN !!");
                        break;
                }
                
                // notify a message to Supervision
                String msg = "The school accepts the Dr. " + real_component_name;
                supContext.notify_message(com.thalesgroup.CdmwPlatformMngtBase.INF.value,
                                          msg);

            } catch (org.omg.Components.CCMException e) {
                System.out.println("Philo: ManagedProcessContext no found !!");
            }
            
            System.out.println("Philo: Testing LoggerFactory QoS Service");
            try {
                org.omg.CORBA.Object obj = 
                    philosopherContext.resolve_service_reference("LoggerFactory");
                com.thalesgroup.CdmwLogging.LoggerFactory loggerFactory =
                    com.thalesgroup.CdmwLogging.LoggerFactoryHelper.narrow(obj);
                com.thalesgroup.CdmwLogging.Logger logger =
                    loggerFactory.get_logger("DINNER_DOMAIN");
                
                cdmw.logging.Logger.CDMW_INFO(logger, "Test logging from component", new Exception());
                
            } catch (org.omg.Components.CCMException e) {
                System.out.println("Philo: LoggerFactory no found !!");
            }
    
                
                
            System.out.println("Philo: Get_connection_to_register...");
            boolean testOk = false;
            try {
                registerServer = philosopherContext.get_connection_register();
        
                if (registerServer != null) {
                }
            } catch (Exception e) {
                fail("Philo: Unexpected exception raised!");
            }
        
            // register philosopher
            System.out.println("Philo: Register philosopher...");
            try {
                if (registerServer != null) {
                    String philosopherName = registerServer.register();
                    // PCR-0994, set speed
            this.philosopher.setSpeed(philosopherName);
                    this.philosopher.set_name(philosopherName);
                    System.out.println("Philo: " + philosopherName 
                        + " is active, speed is " + this.philosopher.speed);
                 }
            } catch (Exception e) {
                fail("Philo: Unexpected exception raised!");
            }
            // push config from emitter port 
            System.out.println("Philo: push_config using emitter port...");
            try {
                com.acme.Dinner.StatusInfo si = new StatusInfoImpl();
                si.name = this.philosopher.name;
                si.state = com.acme.Dinner.PhilosopherState.THINKING;
                si.secs_since_last_meal = this.philosopher.state;
                si.has_left_fork = this.philosopher.haveLeftFork;
                si.has_right_fork = this.philosopher.haveRightFork;
        
                this.philosopher.context.push_config(si);
            } catch (Exception e) {
                fail("Philo: Unexpected exception raised!");
            }

            //get the connection to the server 
            System.out.println("Philo: Connect to the fork facet...");
            com.acme.Dinner.Fork left = null;
            com.acme.Dinner.Fork right = null;
            try {
                left = philosopherContext.get_connection_left();
                right = philosopherContext.get_connection_right();
        
                if (!(left != null && right != null )) {
                    System.out.println("Philo: Facet reference is nil!");
                }
            } catch (Exception e) {
                fail("Philo: Unexpected exception raised!");
            }

            int nb_meal = 0;
            // Let philosopher try to eat
            try {
                while (this.philosopher.is_active() && nb_meal < MAX_NB_MEAL) {
                
                    int timescale = TestManager.getTimescale();
                    OS.sleep(timescale * this.philosopher.speed);
                    
                    left = philosopherContext.get_connection_left();
                    right = philosopherContext.get_connection_right();
                    if (left == null || right == null) {
                        System.out.println("Philo: Facet reference is nil!");
                        break;
                    }
        
                     if (!this.philosopher.is_active()) {
                        // philosopher has been desactivated: get out from loop
                        break;
                    }
        
                    if (this.philosopher.haveLeftFork 
                        && this.philosopher.haveRightFork) {
                        /*
                         * Eating
                         */
                        if (this.philosopher.state > 3) {
                            this.philosopher.state -= 3;
                            this.philosopher.push_state();
                            continue;
                        }
                        this.philosopher.state = 0;
        
                        try {
                            left.release();
                            this.philosopher.haveLeftFork = false;
                            right.release();
                            this.philosopher.haveRightFork = false;
                        } catch (Exception e) {}
                        System.out.println("Philo: " + this.philosopher.name 
                            + " has eaten.");
                        this.philosopher.push_state();
                        //count the number of sucessfull eat 
                        nb_meal++;
                        continue;
                    }
        
                    this.philosopher.state++;
        
                    if (this.philosopher.state < 3) {
                        System.out.println("Philo: " + 
                            this.philosopher.name + " is thinking.");
                        this.philosopher.push_state();
                        continue;
                    }
        
                    if (this.philosopher.state == 3) {
                        System.out.println("Philo: " + 
                            this.philosopher.name + " is becoming hungry.");
                    } else if (this.philosopher.state >= 10) {
                        System.out.println("Philo: " + 
                            this.philosopher.name + " is starving.");
                    }
        
                    if (!this.philosopher.haveLeftFork) {
                        try {
                            left.get();
                            this.philosopher.haveLeftFork = true;
                        } catch (com.acme.Dinner.InUse iu) {
                            System.out.println("Philo: " 
                                + this.philosopher.name 
                                + " has its left fork used.");
                        } catch (org.omg.CORBA.UserException ue) {}
        
                        if (this.philosopher.haveLeftFork) {
                            System.out.println("Philo: " + 
                                this.philosopher.name + 
                                " has got the left fork.");
                            this.philosopher.push_state();
                            continue;
                        } else {
                            System.out.println(this.philosopher.name + " did not get the left fork.");
                        }
                    }        
        
                    if (!this.philosopher.haveRightFork) {
                        try {
                            right.get ();
                            this.philosopher.haveRightFork = true;
                        } catch (com.acme.Dinner.InUse iu) {
                            System.out.println("Philo: " 
                                + this.philosopher.name 
                                + " has its right fork used.");
                        } catch (org.omg.CORBA.UserException ue) {}
        
                        if (this.philosopher.haveRightFork) {
                            System.out.println("Philo: " + 
                                this.philosopher.name + 
                                " has got the right fork.");
                            this.philosopher.push_state();
                            continue;
                        } else {
                            System.out.println(this.philosopher.name + " did not get the right fork.");
                        }
                    }
        
                    if ( ( (this.philosopher.haveLeftFork 
                            && !this.philosopher.haveRightFork) ||
                        (!this.philosopher.haveLeftFork 
                            && this.philosopher.haveRightFork)
                        ) && ((rand.nextInt() % 3) == 0) ) {

                        if (this.philosopher.haveLeftFork) {
                            try {
                                left.release();
                                this.philosopher.haveLeftFork = false;
                                System.out.println("Philo: " + 
                                    "Deadlock prevention! "+ 
                                    this.philosopher.name +
                                    " drops the left fork.");
                            } catch (Exception e)  {}
                        }
                        
                        if (this.philosopher.haveRightFork) {
                            try {
                                right.release();
                                this.philosopher.haveRightFork = false;
                                System.out.println("Philo: " + 
                                    "Deadlock prevention! " + 
                                    this.philosopher.name +
                                    " drops the right fork.");
                            } catch (Exception e) {}
                        }
                    }
        
                    this.philosopher.push_state();

                } // End of while (is_active)

                System.out.println("-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=");
                System.out.println("             End of dinner for " + this.philosopher.name);
                System.out.println("-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=");
                
            } catch (org.omg.CORBA.SystemException se) {
                fail("System Exception raised: " + se.toString());
            } catch (InterruptedException ie) {
                fail("Interrupted Exception raised: " + ie.toString());
            }
        }

    }

    public class PhiloActivationThread extends Thread {
        
        private CCMPhilosopherImpl philosopher;
        private int threadCount=0;

        public PhiloActivationThread(CCMPhilosopherImpl philosopher) {
            this.threadCount++;
            this.philosopher = philosopher;
        }

        public void run() {
            CORBATestManager corbaTestManager = new CORBATestManager();
            TestManager.activateLocalInstance(corbaTestManager);
            
            corbaTestManager.init("Test ccm_activate on Philosopher");
            corbaTestManager.setLocalProperty(TEST_OBJECT_NAME + this.threadCount, this.philosopher);
            
            corbaTestManager.addTest(CCMPhilosopherActivateTest.class);
            corbaTestManager.runTests();
            corbaTestManager.printSummary();
        }
    }
    
    //
    // IDL:thalesgroup.com/Dinner/CCM_Philosopher_Executor/name:1.0
    //
    public String name() {
        return this.name;
    }

    public void set_session_context(org.omg.Components.SessionContext ctx) {
        System.out.println("Philo: set_session_context called!");
        this.context = 
            com.acme.Dinner.CCM_Philosopher_ContextHelper.narrow(ctx);
    }
    
    /**
     * Actives the component (called by the container)
     */
    public void ccm_activate() {
        if (!this.isActive) {
            this.isActive = true;
            // Start thread
            this.activationThread.start();
        }
    }
    
    /**
     * Makes stop the component (call by the container)
     */
    public void ccm_passivate() {
        System.out.println("Philo: ccm_passivate called!");

        try {
            this.mutex.lock();
            if (!this.isActive) {
                this.mutex.unlock();
                return;
            }
        
            this.isActive = false;
            this.mutex.unlock();
        
            System.out.println("Philo: " + this.name + " is inactive.");
        
            try {
                this.activationThread.join();
            } catch (InterruptedException ie) {
                System.out.println(ie.toString());
            }
        
            System.out.println("Philo: " + this.name + " thread has join");
        } catch(InterruptedException ie) {
            fail("Interrupted Exception raised: " + ie.toString());
        }
    }

    /**
     * Purpose:
     *<p> Set the speed according to the name of the philosopher.
     *@param name The name of the philosopher.
     */
    private void setSpeed(String name){
    final int numberPhilosophers = 12;
    final byte NAME = 0;
    final byte SPEED = 1;

    final String[][] philosophers = {
    {"Socrates", "Descartes", "Pascal", "Plato", "Aristotle", "Pythagoras", "Zeno",
     "Heraclitus", "Democritus", "Descartes", "Kant", "Protagoras"},
    {
      "2700" /*Socrates*/, "3600" /*Descartes*/, "1800"/*Pascal*/, "4800"/*Plato*/, 
      "2400" /*Aristotle*/, "3200" /*Pythagoras*/, "1000"/*Zeno*/,"4200" /*Heraclitus*/,
      "1400"/*Democritus*/, "3500"/*Descartes*/, "5100"/*Kant*/, "2200"/*Protagoras*/
    }
    };

    speed = 0;
    for(int i = 0; i < numberPhilosophers; i++){
        if(philosophers[NAME][i].compareTo( name) == 0){
        speed = Integer.parseInt(philosophers[SPEED][i]);
        break;
        }
    }
    
    if(speed == 0)
        System.err.println("Speed of " + name + " not found !");
    
    }

    /**
     * Destroys the component (call by the container)
     */
    public void ccm_remove() {
        System.out.println("Philo: ccm_remove called!");
    }

    /**
     * Push status info event
     */
    public void push_state() {
        com.acme.Dinner.StatusInfo si = new StatusInfoImpl();
        si.name = this.name;
    
        if (this.haveLeftFork && this.haveRightFork) {
            si.state = com.acme.Dinner.PhilosopherState.EATING;
        } else if (this.state < 3) {
            si.state = com.acme.Dinner.PhilosopherState.THINKING;
        } else if (this.state < 10) {
            si.state = com.acme.Dinner.PhilosopherState.HUNGRY;
        } else if (this.state < 40) {
            si.state = com.acme.Dinner.PhilosopherState.STARVING;
        } else {
            si.state = com.acme.Dinner.PhilosopherState.DEAD;
        }
    
        si.secs_since_last_meal = this.state;
        si.has_left_fork = this.haveLeftFork;
        si.has_right_fork = this.haveRightFork;
    
        this.context.push_info(si);
    }

    public boolean is_active() {
        boolean active = false;
        try {
            this.mutex.lock();
            active = this.isActive;
            this.mutex.unlock();
        } catch(InterruptedException ie) {
            fail(ie.toString());
        }
        return active;
    }

    public void set_name(String name) {
        try {
            this.mutex.lock();
            this.name = name;
            this.mutex.unlock();
        } catch(InterruptedException ie) {
            fail(ie.toString());
        }
    }

    public com.acme.Dinner.CCM_Philosopher_Context get_context() {
        return this.context;
    }

}

