/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ft;

import java.io.OutputStream;

import org.omg.CORBA.*;
import org.omg.CORBA.Object;
import org.omg.CosNaming.NameComponent;
import org.omg.FT.ObjectNotCreated;
import org.omg.FT.PrimaryNotSet;
import org.omg.PortableServer.*;

import com.thalesgroup.CdmwFT.MemberInfo;

public abstract class IOGRFactory extends LocalObject implements com.thalesgroup.CdmwFT.IOGRFactory {
    
    // The ORB pseudo-object.
    protected ORB orb;
        
    // The parent POA for fallback IORs.
    protected POA poa;

    
    public IOGRFactory(ORB orb) {
        this.orb = orb;
    }
    
    /**
     * Build the iogr
     *
     * @param Members          sequence of members & locations.
     * @param primary_location location of the primary member.
     * @param object_group_id  the object group id to use for the search.
     * @param ft_domain_id     The fault tolerance domain id.
     * @param object_group_version the object group reference version.
     *
     * @return the reference to the object created 
     *
     * @exception InvalidProperty if one or more of the properties in the
     *        sequence is not valid.
     * @exception UnsupportedProperty if one or more of the properties in
     *        the sequence is not supported.
     */
    public abstract Object build_iogr(MemberInfo[] members, 
                                      NameComponent[] primary_location, 
                                      long object_group_id, String type_id, 
                                      String ft_domain_id, 
                                      IntHolder object_group_ref_version, 
                                      Object[] fall_backs) 
        throws PrimaryNotSet, ObjectNotCreated;

    /**
     * count the number of profile found into the IOGR
     *
     * @param obj The IOGR.
     *
     * @return the  number of profile found
     */
    public abstract int profile_count (org.omg.CORBA.Object obj);


    /**
     * decode the specified profile to extract the FT::TagFTGroupTaggedComponent 
     * structure 
     *
     * @param obj The IOGR.
     * @param the profile to decode
     *
     * @return the FT::TagFTGroupTaggedComponent structure
     *
     * @exception ProfileNotFound if the profile number is over the limit
     * @exception TagNotFound if the TAG_FT_GROUP is not found into the profile
     */    
    public abstract org.omg.FT.TagFTGroupTaggedComponent 
    decode_profile_with_group_tag (org.omg.CORBA.Object obj,
                                   int profile)
        throws com.thalesgroup.CdmwFT.ProfileNotFound,
               com.thalesgroup.CdmwFT.TagNotFound;


    /**
     * decode the specified profile to extract the FT::TagFTPrimaryTaggedComponent 
     * structure 
     *
     * @param obj The IOGR.
     * @param the profile to decode
     *
     * @return the FT::TagFTPrimaryTaggedComponent structure
     *
     * @exception ProfileNotFound if the profile number is over the limit
     * @exception TagNotFound if the TAG_FT_PRIMARY is not found into the profile
     */        
    public abstract org.omg.FT.TagFTPrimaryTaggedComponent 
    decode_profile_with_primary_tag (org.omg.CORBA.Object obj,
                                     int profile)
        throws com.thalesgroup.CdmwFT.ProfileNotFound,
               com.thalesgroup.CdmwFT.TagNotFound;
               
}
