/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ft.watchdog;

import org.omg.PortableInterceptor.ORBInitInfo;

import cdmw.common.Assert;
import cdmw.ft.FTLogger;
import cdmw.ft.FTSystemProperties;
import cdmw.logging.FunctionLogger;
import static cdmw.logging.Logger.*;

public class FTRequestTimeoutORBInitializer extends org.omg.CORBA.LocalObject
    implements org.omg.PortableInterceptor.ORBInitializer {

    private static FTRequestTimeoutServerRequestInterceptorImpl serverInterceptor = null;
    private static FTRequestTimeoutClientRequestInterceptorImpl clientInterceptor = null;

    /**
     * Purpose: <p> Constructor
     * 
     */
    public FTRequestTimeoutORBInitializer() {
        // Get watchdog timeout propery.
        // Should have been set by user or by WatchdogServiceInit.init()
        String reqTimeoutStr = 
            System.getProperty(FTSystemProperties.WATCHDOG_TIMEOUT_PROPERTY_NAME,
            "");  
        Assert.check(!reqTimeoutStr.equals(""));
        try {
            int requestTimeout = Integer.parseInt(reqTimeoutStr);
            RequestTimeoutLogger.initRequestTimeoutLogger(requestTimeout);
        } catch (NumberFormatException e) {
            e.printStackTrace();
            Assert.check(false);
        }
            
    }

    /**
     * Purpose: <p> This operation is called during ORB initialization.
     * 
     */
    public void pre_init(ORBInitInfo arg0) {
        FunctionLogger.CDMW_LOG_FUNCTION(FTLogger.GetLogger(), "");
    }

    /**
     * Purpose: <p> This operation is called during ORB initialization.
     * 
     * Note: the interceptors are registered in post_init() if their
     * constructors require ORBInitInfo.resolve_initial_reference(), which XXX
     * cannot be called in pre_init().
     * 
     */
    public void post_init(ORBInitInfo info) {
        FunctionLogger.CDMW_LOG_FUNCTION(FTLogger.GetLogger(), "");
        //
        // allocate needed slot_id
        //
        CDMW_DEBUG(FTLogger.GetLogger(),
            "post_init => allocate needed slot_id", new Throwable());
        int isRequestIdSlotId = info.allocate_slot_id();

        //
        // Create and register the interceptors
        //
        CDMW_DEBUG(FTLogger.GetLogger(),
            "post_init => Create the interceptors", new Throwable());
        serverInterceptor = new FTRequestTimeoutServerRequestInterceptorImpl(info, isRequestIdSlotId);
        clientInterceptor = new FTRequestTimeoutClientRequestInterceptorImpl(info, isRequestIdSlotId);

        try {
            info.add_server_request_interceptor(serverInterceptor);
            info.add_client_request_interceptor(clientInterceptor);
        }
        catch (org.omg.PortableInterceptor.ORBInitInfoPackage.DuplicateName e) {
            // can not happened while interceptor is not named!
            Assert.check(false);
        }
    }

}
