/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.demodance1;

import cdmw.logging.Logger;

//
//IDL:thalesgroup.com/CcmDance1Test/ClientModule/CCM_ClientDance:1.0
//

public class CCMClientDanceImpl extends org.omg.CORBA.LocalObject
    implements com.thalesgroup.UserCcmDance1Test.User_CCM_ClientDance {
    
    private String configuration = "";
    private String expectedServerHostname = "";
    private String hostnameCommandResult = "";

    private com.acme.CcmDance1Test.ClientModule.CCM_ClientDance_Context sessionContext = null;
    private com.acme.CcmDance1Test.FacetModule.CCM_ClientHostInfo clientHostInfo = new CCMClientHostInfoImpl();

    private Logger logger;
    
    public CCMClientDanceImpl() {
        this.logger = Logger.Get_Logger("ClientDance");
    }
    
    //
    // attribute configuration
    //
    public String configuration() {
        return this.configuration;
    }
    public void configuration(String configuration) {
        this.configuration = configuration;
    }

    //
    // attribute expectedServerHostname
    //
    public String expectedServerHostname() {
        return this.expectedServerHostname;
    }
    public void expectedServerHostname(String hostname) {
        this.expectedServerHostname = hostname;
    }
    
    //
    // facet clientHostInfo
    //
    public com.acme.CcmDance1Test.FacetModule.CCM_ClientHostInfo get_clientHostInfo() {
        return this.clientHostInfo;
    }
    
    //
    // consumer hostnameResults
    //
    public void push_hostnameResults(com.acme.CcmDance1Test.EventTypeModule.HostnameResult ev) {
        Logger.CDMW_INFO(
            this.logger,
            configuration + " : push_hostnameResults called with hostname " + ev.hostname, 
            new Throwable());
        this.hostnameCommandResult = ev.hostname;
    }
    
    //
    // supported interface TestableComponent
    // 
    public java.lang.String get_property(java.lang.String prop_name) {
        Logger.CDMW_INFO(
            this.logger,
            configuration + " : get_property called", 
            new Throwable());
        String property = "";
        
        if (prop_name.equals(com.acme.CcmDance1Test.TestFramework.PROPERTY_GET_HOSTNAME.value)) {
            Logger.CDMW_INFO(
                this.logger,
                configuration + " : PROPERTY_GET_HOSTNAME",
                new Throwable());
            property = expectedServerHostname;
        } else if (prop_name.equals(com.acme.CcmDance1Test.TestFramework.PROPERTY_GET_SERVER_HOSTNAME.value)) {
            Logger.CDMW_INFO(
                this.logger,
                configuration + " : PROPERTY_GET_SERVER_HOSTNAME",
                new Throwable());
            try {
                com.acme.CcmDance1Test.FacetModule.HostInfo hostInfo =
                    sessionContext.get_connection_serverHostInfo();
                    
                if (hostInfo == null) {
                    Logger.CDMW_ERROR(
                        this.logger,
                        configuration + " : hostInfo is null !!!",
                        new Throwable());
                    throw new org.omg.CORBA.BAD_PARAM();
                }
                
                property = hostInfo.get_hostname();
            } catch (Exception e) {
                Logger.CDMW_ERROR(
                    this.logger,
                    configuration + " : Unexpected exception " + e,
                    new Throwable());
            }
        } else {
            Logger.CDMW_INFO(
                this.logger,
                configuration + " : Unknown property",
                new Throwable());
            property = "UnknownProperty";
        }
        
        Logger.CDMW_INFO(
            this.logger,
            configuration + " : get_property returns " + property,
            new Throwable());
            
        return property;
    }
    
    public void reset() {
        Logger.CDMW_INFO(this.logger,configuration + " : reset called", new Throwable());
        this.hostnameCommandResult = "";
    }
    
    public boolean check(java.lang.String ref) {
        boolean checkResult = false;
        if (ref.equals(hostnameCommandResult)) {
            Logger.CDMW_INFO(
                this.logger,configuration + " : Server's hostname OK !", 
                new Throwable());
            checkResult = true;
        } else {
            Logger.CDMW_ERROR(
                this.logger,
                configuration + " : Server's hostname is not the one expected: " + expectedServerHostname, 
                new Throwable());
        }
        return checkResult;
    }
    
    public void do_test(int idx) {
        Logger.CDMW_INFO(
            this.logger,
            configuration + " : do_test called", 
            new Throwable());
        
        if (idx == 1) {
            com.acme.CcmDance1Test.EventTypeModule.Command command = 
                new CommandImpl();
            command.commandName = 
                com.acme.CcmDance1Test.EventTypeModule.COMMAND_GET_HOSTNAME.value;
            Logger.CDMW_INFO(
                this.logger,
                configuration + " : --<> emits command >-- " + command.commandName,
                new Throwable());
            sessionContext.push_commandEmits(command);
        } else if (idx == 2) {
            com.acme.CcmDance1Test.EventTypeModule.Command command = 
                new CommandImpl();
            command.commandName = 
                com.acme.CcmDance1Test.EventTypeModule.COMMAND_CHECK_HOSTNAME.value;
            Logger.CDMW_INFO(
                this.logger,
                configuration + " : --<> emits command >-- " + command.commandName,
                new Throwable());
            sessionContext.push_commandEmits(command);
        }
    }
    
    //
    // Session component specific operations
    //
    public void set_session_context(org.omg.Components.SessionContext ctx) {
        Logger.CDMW_INFO(
            this.logger,
            configuration + " : set_session_context called", 
            new Throwable());
        this.sessionContext = 
            com.acme.CcmDance1Test.ClientModule.CCM_ClientDance_ContextHelper.narrow(ctx);
    }
    
    public void ccm_activate() {
        Logger.CDMW_INFO(
            this.logger,
            configuration + " : ccm_activate called", 
            new Throwable());
    }
    
    public void ccm_passivate() {
        Logger.CDMW_INFO(
            this.logger,
            configuration + " : ccm_passivate called", 
            new Throwable());
    }
    
    public void ccm_remove() {
        Logger.CDMW_INFO(
            this.logger,
            configuration + " : ccm_remove called", 
            new Throwable());
    }
    
    //
    // additional operations
    //
    public com.acme.CcmDance1Test.ClientModule.CCM_ClientDance_Context getContext() {
        return this.sessionContext;
    }
    
    public String getHostnameCommandResult() {
        return this.hostnameCommandResult;
    }
}