/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.demodance1;

import cdmw.logging.Logger;

//
//IDL:thalesgroup.com/CcmDance1Test/ServerModule/CCM_ServerDance:1.0
//

public class CCMServerDanceImpl extends org.omg.CORBA.LocalObject
    implements com.thalesgroup.UserCcmDance1Test.User_CCM_ServerDance {
    
    private String expectedClientHostname1 = "";
    private String expectedClientHostname2 = "";
    
    private boolean bCheckResult = false;
    
    private com.acme.CcmDance1Test.ServerModule.CCM_ServerDance_Context sessionContext = null;
    
    private com.acme.CcmDance1Test.FacetModule.CCM_HostInfo hostInfo1 = new CCMHostInfoImpl();
    private com.acme.CcmDance1Test.FacetModule.CCM_HostInfo_WA hostInfo2 = new CCMHostInfoWAImpl();

    private Logger logger;
    
    public CCMServerDanceImpl() {
        this.logger = Logger.Get_Logger("ServerDance");
    }
    
    //
    // attribute expectedClientHostname1
    //
    public String expectedClientHostname1() {
        return this.expectedClientHostname1;
    }
    public void expectedClientHostname1(String hostname) {
        this.expectedClientHostname1 = hostname;
    }
    
    //
    // attribute expectedClientHostname2
    //
    public String expectedClientHostname2() {
        return this.expectedClientHostname2;
    }
    public void expectedClientHostname2(String hostname) {
        this.expectedClientHostname2 = hostname;
    }
    
    //
    // consumer commands
    //
    public void push_commands(com.acme.CcmDance1Test.EventTypeModule.Command ev) {
        String command = ev.commandName;
        
        if (command == null) {
            Logger.CDMW_ERROR (
                this.logger,
                "CCMServerDanceImpl : ev.commandName is null !!!!",
                new Throwable());
            return;
        }
        
        Logger.CDMW_INFO(
            this.logger,
            "push_commands called with command " + command,
            new Throwable());
        
        if (command.equals( com.acme.CcmDance1Test.EventTypeModule.COMMAND_CHECK_HOSTNAME.value)) {
            try {
                //check client 1
                {
                    com.acme.CcmDance1Test.FacetModule.ClientHostInfo hostInfo1 = 
                        sessionContext.get_connection_clientHostInfo1();
                    if (hostInfo1 == null) {
                        Logger.CDMW_ERROR(
                            this.logger,
                            "hostInfo1 is null !!!!",
                            new Throwable());
                            this.bCheckResult = false;
                        return;
                    }
                    
                    String hostname1 = hostInfo1.get_hostname();
                    if (expectedClientHostname1.equals(hostname1)) {
                        Logger.CDMW_INFO(
                            this.logger,
                            "Client1's hostname OK !",
                            new Throwable());
                            this.bCheckResult = true;
                    } else {
                        Logger.CDMW_ERROR(
                            this.logger,
                            "Client1's hostname is not the one expected : " + hostname1,
                            new Throwable());
                            this.bCheckResult = false;
                            return;
                    }
                }
                //check client 2
                {
                    com.acme.CcmDance1Test.FacetModule.ClientHostInfo hostInfo2 = 
                        sessionContext.get_connection_clientHostInfo2();
                    if (hostInfo2 == null) {
                        Logger.CDMW_ERROR(
                            this.logger,
                            "hostInfo2 is null !!!!",
                            new Throwable());
                            this.bCheckResult = false;
                        return;
                    }
                    
                    String hostname2 = hostInfo2.get_hostname();
                    if (expectedClientHostname2.equals(hostname2)) {
                        Logger.CDMW_INFO(
                            this.logger,
                            "Client2's hostname OK !",
                            new Throwable());
                            this.bCheckResult = true;
                    } else {
                        Logger.CDMW_ERROR(
                            this.logger,
                            "Client2's hostname is not the one expected : " + hostname2,
                            new Throwable());
                            this.bCheckResult = false;
                            return;
                    }
                }
            } catch (Exception e){
                Logger.CDMW_ERROR(
                    this.logger,
                    " : Unexpected exception " + e,
                    new Throwable());
            }
        } else if (command.equals( com.acme.CcmDance1Test.EventTypeModule.COMMAND_GET_HOSTNAME.value)) {
            String hostname = cdmw.ossupport.OS.getHostname();
            Logger.CDMW_INFO(
                this.logger,
                "CCMServerDanceImpl : return result : " + hostname,
                new Throwable());
                
            com.acme.CcmDance1Test.EventTypeModule.HostnameResult result = 
                new HostnameResultImpl();
            result.hostname = hostname;
            
            Logger.CDMW_INFO(
                this.logger,
                "CCMServerDanceImpl : emits HostnameResult",
                new Throwable());
            sessionContext.push_hostnamePublishes(result);
        } else {
            Logger.CDMW_ERROR (
                this.logger,
                "CCMServerDanceImpl : Unkown Command !!!!",
                new Throwable());
        }
    }
    
    //
    // Session component specific operations
    //
    public void set_session_context(org.omg.Components.SessionContext ctx) {
        Logger.CDMW_INFO(
            this.logger,
            "set_session_context called",
            new Throwable());
        this.sessionContext = 
            com.acme.CcmDance1Test.ServerModule.CCM_ServerDance_ContextHelper.narrow(ctx);
    }
    public void ccm_activate() {
        Logger.CDMW_INFO(
            this.logger,
            "ccm_activate called",
            new Throwable());
    }
    
    public void ccm_passivate() {
        Logger.CDMW_INFO(
            this.logger,
            "ccm_passivate called",
            new Throwable());
    }
    
    public void ccm_remove() {
        Logger.CDMW_INFO(
            this.logger,
            "ccm_remove called",
            new Throwable());
    }
    
    //
    // facet serverHostInfo1
    //
    public com.acme.CcmDance1Test.FacetModule.CCM_HostInfo get_serverHostInfo1() {
        Logger.CDMW_INFO(
            this.logger,
            "get_serverHostInfo1 called",
            new Throwable());
        return this.hostInfo1;
    }
    
    //
    // facet serverHostInfo2
    //
    public com.acme.CcmDance1Test.FacetModule.CCM_HostInfo_WA get_serverHostInfo2(){
        Logger.CDMW_INFO(
            this.logger,
            "get_serverHostInfo2 called",
            new Throwable());
        return this.hostInfo2;
    }
    
    //
    // supported interface HelloInterface 
    //
    public String say_hello(String msg) {
        String returnedString = msg;
        Logger.CDMW_INFO(
            this.logger,
            "Hello " + msg,
            new Throwable());
        return returnedString;
    }

    //
    // supported interface TestableComponent
    // 
    public String get_property(String prop_name) {
        throw new org.omg.CORBA.NO_IMPLEMENT(0, org.omg.CORBA.CompletionStatus.COMPLETED_NO);
    }
    public void reset() {
        Logger.CDMW_INFO(
            this.logger,
            "reset called",
            new Throwable());
        this.bCheckResult = false;
    }
    public boolean check(String ref) {
        return this.bCheckResult;
    }
    public void do_test(int idx) {
        throw new org.omg.CORBA.NO_IMPLEMENT(0, org.omg.CORBA.CompletionStatus.COMPLETED_NO);
    }
    
    //
    // additional operations
    //
    public  com.acme.CcmDance1Test.ServerModule.CCM_ServerDance_Context get_context() {
        return this.sessionContext;
    }

}