/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.demodance1;

import cdmw.common.Assert;
import cdmw.testutils.Testable;
import cdmw.logging.Logger;

public class CCMTestDriverImpl extends org.omg.CORBA.LocalObject
    implements com.thalesgroup.UserCcmDance1Test.User_CCM_TestDriver {

    //
    // This inner class provides an object to perform the test
    //
    private class CCMDanceTest extends Testable {
        CCMTestDriverImpl testDriver = null;
        public CCMDanceTest(CCMTestDriverImpl testDriver) {
            this.name = "Test demo_dance1 java";
            this.out = System.out;
            this.testDriver = testDriver;
        }
        
        public void doTests() {
            Assert.check(this.testDriver != null);
            try {
                cdmw.ossupport.OS.sleep(2000);
            } catch (java.lang.InterruptedException e) {
                println("Unexpected exception" + e);
            }
            boolean testOK = false;
            
            com.acme.CcmDance1Test.DriverModule.CCM_TestDriver_Context driverContext = 
                testDriver.getContext();
            Assert.check(driverContext != null);

            println("Get the testable components");
            
            com.acme.CcmDance1Test.TestFramework.TestableComponent serverComponent = null;
            com.acme.CcmDance1Test.TestFramework.TestableComponent client1Component = null;
            com.acme.CcmDance1Test.TestFramework.TestableComponent client2Component = null;
            
            try {
                serverComponent = driverContext.get_connection_server();
                client1Component = driverContext.get_connection_client1();
                client2Component = driverContext.get_connection_client2();
            } catch (Exception e){
                println("Unexpected exception " + e);
            }
            
            Assert.check(serverComponent != null);
            Assert.check(client1Component != null);
            Assert.check(client2Component != null);
        
            setNbOfRequestedTestOK(8);

            println("First test : primaryClient is Client1, secondaryClient is Client2");
            testOK = doTest(serverComponent, client1Component, client2Component);
            println("End of first test.");

            println("Second test : primaryClient is Client2, secondaryClient is Client1");
            testOK = doTest(serverComponent, client2Component, client1Component);
            println("End of second test.");

            if (testOK == true) {
                println("Demo test status is SUCCESS !");
                Logger.CDMW_INFO(
                    Logger.Get_Logger("TestDriver"),
                    "Demo test status is SUCCESS !", 
                    new Throwable());
            } else {
                println("Demo test status is FAILED !");
            }
        }
        
        private boolean doTest(
            com.acme.CcmDance1Test.TestFramework.TestableComponent server,
            com.acme.CcmDance1Test.TestFramework.TestableComponent primaryClient,
            com.acme.CcmDance1Test.TestFramework.TestableComponent secondaryClient) {
            boolean testResult = false;
            
            try {
                println("Reseting components");
                server.reset();
                primaryClient.reset();
                secondaryClient.reset();
                
                String clt1Hostname = primaryClient.get_property(
                    com.acme.CcmDance1Test.TestFramework.PROPERTY_GET_HOSTNAME.value);
                println("primaryClient's expected hostame is " + clt1Hostname);
                
                String clt2Hostname = secondaryClient.get_property(
                    com.acme.CcmDance1Test.TestFramework.PROPERTY_GET_HOSTNAME.value);
                println("secondaryClient's expected hostame is " + clt2Hostname);
                
                String srvHostname = primaryClient.get_property(
                    com.acme.CcmDance1Test.TestFramework.PROPERTY_GET_SERVER_HOSTNAME.value);
                println("server's hostame is " + srvHostname);
                
                if (clt1Hostname.equals(srvHostname)) {
                    println("hostnames are identical");
                    testResult = true;
                } else {
                    println("ERROR : hostname mismatch");
                }
                check(testResult);
                
                testResult = false;
                println("Test primary client's sending getHostname command to the server");
                primaryClient.do_test(1);

                println("Waiting for result...");
                cdmw.ossupport.OS.sleep(2000);
                println("Check primaryClient's result");
        
                boolean bCheck = primaryClient.check(clt1Hostname);
        
                if (bCheck == true) {
                    println("hostnames are identical");
                    testResult = true;
                } else {
                    println("ERROR : hostname mismatch");
                }
                check(testResult);
                
                testResult = false;
                println("Check secondaryClient's result");
        
                bCheck = secondaryClient.check(clt2Hostname);
                
                if (bCheck == true) {
                    println("hostnames are identical");
                    testResult = true;
                } else {
                    println("ERROR : hostname mismatch");
                }
                check(testResult);
                
                testResult = false;
                println("Test primary client's sending checkHostname command to the server");
                primaryClient.do_test(2);

                println("Waiting for result...");
                cdmw.ossupport.OS.sleep(2000);
                println("Check server's result");
        
                bCheck = server.check(clt1Hostname);
        
                if (bCheck == true) {
                    println("hostnames are identical");
                    testResult = true;
                } else {
                    println("ERROR : hostname mismatch");
                }
                check(testResult);

                } catch (Exception e){
                println("Unexpected exception " + e);
            }          
            return testResult;
        }
    }
    
    private com.acme.CcmDance1Test.DriverModule.CCM_TestDriver_Context sessionContext = null;
    
    public CCMTestDriverImpl() {}
    
    //
    // Session component specific operations
    //
    public void set_session_context(org.omg.Components.SessionContext ctx) {
        this.sessionContext = 
            com.acme.CcmDance1Test.DriverModule.CCM_TestDriver_ContextHelper.narrow(ctx);
    }
    public void ccm_activate() {
        CCMDanceTest test = new CCMDanceTest(this);
        test.start();
    }
    
    public void ccm_passivate() {
    }
    
    public void ccm_remove() {
    }
    
    //
    // additional operations
    //
    public com.acme.CcmDance1Test.DriverModule.CCM_TestDriver_Context getContext() {
        return this.sessionContext;
    }

    
}