/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2007. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ccmallcapabilities;

import com.acme.CcmAllCapabilitiesTest.*;
import UserCcmAllCapabilitiesTest.*;
import com.thalesgroup.CdmwRecording.*;

import cdmw.logging.Logger;
import static cdmw.logging.Logger.*;

import org.omg.Components.SessionContext;
import com.thalesgroup.CdmwComponents.SessionContextExt;
import com.thalesgroup.CdmwComponents.SessionContextExtHelper;
import org.omg.CosClockService.ClockCatalog;
import org.omg.CosClockService.ClockCatalogHelper;

import com.thalesgroup.CdmwPlatformMngtBase.*;
import com.thalesgroup.CdmwPlatformMngt.SupervisionObserver;
import com.thalesgroup.CdmwPlatformMngt.ManagedElementContexts.*;
import com.thalesgroup.CdmwPlatformMngt.RedundancyEltState;
import com.thalesgroup.CdmwPlatformMngt.Event;
import com.thalesgroup.CdmwPlatformMngt.EventKind;
import com.thalesgroup.CdmwPlatformMngt.SystemStatus;
import com.thalesgroup.CdmwPlatformMngt.SystemStatusChange;
import com.thalesgroup.CdmwPlatformMngt.SystemModeChange;
import com.thalesgroup.CdmwPlatformMngt.ComponentStatusChange;
import com.thalesgroup.CdmwPlatformMngt.ApplicationStatus;
import com.thalesgroup.CdmwPlatformMngt.ApplicationStatusChange;
import com.thalesgroup.CdmwPlatformMngt.ApplicationModeChange;
import com.thalesgroup.CdmwPlatformMngt.ProcessStatus;
import com.thalesgroup.CdmwPlatformMngt.ProcessStatusChange;
import com.thalesgroup.CdmwPlatformMngt.HostStatus;
import com.thalesgroup.CdmwPlatformMngt.HostStatusChange;
import com.thalesgroup.CdmwPlatformMngt.TechnicalStatus;
import com.thalesgroup.CdmwPlatformMngt.FunctioningMode;
import com.thalesgroup.CdmwPlatformMngt.ComponentStatus;
import com.thalesgroup.CdmwPlatformMngt.ExitEvent;
import cdmw.platformmngt.PlatformInterface;

import cdmw.testutils.CORBATestManager;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import java.lang.String;
import org.omg.CORBA.StringHolder;

//
// IDL:acme.com/CcmAllCapabilitiesTest/CCM_Client:1.0
//
public class CCMClientImpl extends org.omg.CORBA.LocalObject
    implements User_CCM_Client
{

    static final String APPLICATION_NAME      = "CcmAllCapabilitiesApplication";
    static final String CLIENT_PROCESS_NAME   = "ClientJavaProcess";
    static final String CLIENT_COMPONENT_NAME = "Client";

    // TestThread
    private class TestThread extends Thread {
        public CORBATestManager testManager;

        TestThread(CORBATestManager testManager) {
            this.testManager = testManager;
        }

        public void run() {
            this.testManager.runTests();
        }
    }


    // session context
    private CCM_Client_Context sessionContext;
    private SessionContextExt sessionContextExt;

    private Logger logger;
    private java.lang.String fileName;
    private org.omg.CosClockService.Clock clock;
    private String expectedPrimaryHostname;
    private String expectedBackupHostname;
    private String hostnameCommandResult;

    // Received records queue
    java.util.List<RequestReplyPair> recordList;

    // Received supervision events queue
    java.util.LinkedList<Event> supEventsList;

    private TestThread testThread;


    public CCMClientImpl()
    {
        this.sessionContext = null;
        this.testThread = null;
        this.clock = null;
        this.recordList = new java.util.Vector<RequestReplyPair>();
        this.supEventsList = new java.util.LinkedList<Event>();
        this.logger = Logger.Get_Logger("CCM_Client",
                                        "QUALIF_CCM_ALL_CAPABILITIES");
    }


    //
    // attribute expectedPrimaryHostname
    //
    public String expectedPrimaryHostname() {
        return this.expectedPrimaryHostname;
    }
    public void expectedPrimaryHostname(String hostname) {
        this.expectedPrimaryHostname = hostname;
    }


    //
    // attribute expectedBackupHostname
    //
    public String expectedBackupHostname() {
        return this.expectedBackupHostname;
    }
    public void expectedBackupHostname(String hostname) {
        this.expectedBackupHostname = hostname;
    }


    //
    // attribute fileName
    //
    public void fileName(java.lang.String fileName)
        throws org.omg.CORBA.SystemException
    {
        this.fileName = fileName;
    }
    public java.lang.String  fileName()
        throws org.omg.CORBA.SystemException
    {
        return fileName;
    }

    //
    // facet clientHostInfo
    //
    public
    CCM_HostInfo get_clientHostInfo()
    {
        return (CCM_HostInfo) this;
    }

    public String get_hostname() {
        CDMW_INFO(logger, "CCMClientImpl.get_hostname()",
                  new Throwable());
        String hostname = cdmw.ossupport.OS.getHostname();
        return hostname;
    }

    public long get_local_time()
    {
        long now = 0;
        try {
            CDMW_INFO(logger, "CCMServerImpl.get_local_time()",
                      new Throwable());
            if (this.clock != null)
                now = this.clock.current_time();
        }
        catch (org.omg.CosClockService.TimeUnavailable e) {
            CDMW_ERROR
                (logger, "Unexpected exception raised!" + e,
                 new Throwable());
            e.printStackTrace(System.err);
        }
        finally {
            CDMW_INFO(logger, "time = " + now, new Throwable());
            return now;
        }
    }


    //
    // consumer hostnameResults
    //
    public void push_hostnameResults(HostnameResult event) {
        CDMW_INFO(this.logger,
                  "CCMClientImpl.push_hostnameResults(), "
                  + "hostname = " + event.hostname,
                  new Throwable());
        this.hostnameCommandResult = event.hostname;
    }


    //
    // get result of asynchronous getHostname command
    //
    public String getHostnameCommandResult() {
        return this.hostnameCommandResult;
    }

    public void set_session_context(org.omg.Components.SessionContext ctx)
    {
        CDMW_INFO(this.logger,
                  "CCMClientImpl.set_session_context()",
                  new Throwable());

        this.sessionContext = CCM_Client_ContextHelper.narrow(ctx);

        // get context for services
        this.sessionContextExt =
            SessionContextExtHelper.narrow(this.sessionContext);

        // get Time Service
        org.omg.CORBA.Object obj = null;
        try {
            obj = sessionContextExt.resolve_service_reference("ClockService");
        }
        catch (org.omg.Components.CCMException e) {
            CDMW_ERROR(this.logger, "Unexpected exception raised while getting service! " + e,
                       new Throwable());
        }
        try {
            ClockCatalog clockCatalog = ClockCatalogHelper.narrow(obj);

            // get catalag entry named "LocalExecutor"
            org.omg.CosClockService.ClockCatalogPackage.ClockEntry clockEntry =
                clockEntry = clockCatalog.get_entry("LocalExecutor");

            // get associated clock
            clock = clockEntry.subject;
        }
        catch (org.omg.CosClockService.ClockCatalogPackage.UnknownEntry ue) {
            CDMW_ERROR
                (logger, "Unexpected exception raised!" + ue,
                 new Throwable());
            ue.printStackTrace(System.err);
        }
        catch (java.lang.Exception e) {
            CDMW_ERROR(this.logger,
                       "Unexpected exception raised while getting service! " + e,
                       new Throwable());
            e.printStackTrace(System.err);
        }
    }

    public CCM_Client_Context getContext()
    {
        return this.sessionContext;
    }

    public void cdmw_initialise_component(com.thalesgroup.CdmwPlatformMngtBase.StartupKind startupKind)
        throws com.thalesgroup.CdmwPlatformMngt.ManagedComponentPackage.BadOrder {
        CDMW_INFO(this.logger,
                  "*** Component Client Initialisation ***",
                  new Throwable());
        try {
            Thread.sleep(5000);
        } catch (InterruptedException e) {
            System.err.println("Thread was interrupted");
            e.printStackTrace();
        }
        CDMW_INFO(this.logger,
                  "*** Component Client Initialised ***",
                  new Throwable());
    }

    public void ccm_activate()
    {
        CDMW_INFO(this.logger,
                  "CCMClientImpl.ccm_activate()",
                  new Throwable());

        try {
            CORBATestManager testManager = CORBATestManager.instance();

            testManager.setLocalProperty(
                CCMAllCapabilitiesTester.CCM_CLIENT_IMPL_NAME,
                this);
            testManager.setLocalProperty(
                CCMAllCapabilitiesTester.SESSION_CONTEXT_NAME,
                this.sessionContext);

            org.omg.Components.SessionContext ctx =
                org.omg.Components.SessionContextHelper.narrow(sessionContext);
            org.omg.CORBA.Object obj = ctx.get_CCM_object();
            com.acme.CcmAllCapabilitiesTest.Client thisComponent =
                ClientHelper.narrow(obj);
            testManager.setLocalProperty(
                CCMAllCapabilitiesTester.COMPONENT_NAME,
                thisComponent);

            testManager.addTest(CCMAllCapabilitiesTester.class);
            this.testThread = new TestThread(testManager);
            this.testThread.start();
        }

        catch (Exception e) {
            System.err.println("Exception thrown during test: " + e);
        }
    }

    public void ccm_passivate()
    {
        CDMW_INFO(this.logger,
                  "CCMClientImpl.ccm_passivate()",
                  new Throwable());

        if (this.testThread != null) {
            try {
                this.testThread.join();
            }
            catch (java.lang.InterruptedException e) {
                // ignore
            }
        }
    }

    public void ccm_remove()
    {
        CDMW_INFO(this.logger,
                  "CCMClientImpl.ccm_remove()",
                  new Throwable());
    }


    //
    // CdmwRecording::Recorder::record() implementation
    //
    public synchronized void record(
        com.thalesgroup.CdmwRecording.GroupHeader header,
        byte[] data)
    {
        System.out.println(
            "" + header.num_pairs + " new Records received from "
            + header.hostname + " at " + header.base_time);

        // Use org.jacorb.orb.CDRInputStream for unmarshalling
        boolean endian = (header.flags!=0 ? true : false);
        org.omg.CORBA.portable.InputStream input =
            new org.jacorb.orb.CDRInputStream(org.omg.CORBA.ORB.init(),
                                              data,
                                              endian);

        // Unmarshall while InputStream is not empty
        int i = 0;
        try {
            while (input.available() > 0) {

                // number of octets still to read
                int left = input.available();

                // Unmarshall a RequestReplyPair
                try {
                    RequestReplyPair pair = RequestReplyPairHelper.read(input);

                    System.out.println("  - " + (++i) + ": \"" +
                                       pair.request.operation +
                                       "\" from " +
                                       pair.request.server_id +
                                       "\" from interface " +
                                       pair.request.interface_rep_id);

                    // save RequestReplyPair in list
                    recordList.add(pair);
                }
                catch (org.omg.CORBA.MARSHAL e) {
                    // number of read octets
                    int nb_read = left-input.available();
                    System.out.println("UNMARSHALL ERROR (only " + nb_read +
                                       " octets read): " + e);
                    e.printStackTrace();
                    // It's useless to try to continue unmarshalling... break loop
                    break;
                }
            }
        }

        catch (java.io.IOException e) {
            System.out.println("UNMARSHALL ERROR : " + e);
            e.printStackTrace();
        }
    }


    private String eventKindToString(Event evt)
    {
        switch (evt.event_kind.value()) {
        case EventKind._SYSTEM_MODE_CHANGE:
            return "SYSTEM_MODE_CHANGE";
        case EventKind._SYSTEM_STATUS_CHANGE:
            return "SYSTEM_STATUS_CHANGE";
        case EventKind._HOST_STATUS_CHANGE:
            return "HOST_STATUS_CHANGE";
        case EventKind._APPLICATION_AGENT_DEATH:
            return "APPLICATION_AGENT_DEATH";
        case EventKind._APPLICATION_MODE_CHANGE:
            return "APPLICATION_MODE_CHANGE";
        case EventKind._APPLICATION_STATUS_CHANGE:
            return "APPLICATION_STATUS_CHANGE";
        case EventKind._PROCESS_MESSAGE:
            return "PROCESS_MESSAGE";
        case EventKind._PROCESS_STATUS_CHANGE:
            return "PROCESS_STATUS_CHANGE";
        case EventKind._ENTITY_STATUS_CHANGE:
            return "ENTITY_STATUS_CHANGE";
        case EventKind._CONFIGURATION_CHANGE:
            return "CONFIGURATION_CHANGE";
        case EventKind._USER_EVENT:
            return "USER_EVENT";
        case EventKind._PROPERTY_CHANGE:
            return "PROPERTY_CHANGE";
        case EventKind._RESTART_REQUESTED:
            return "RESTART_REQUESTED";
        case EventKind._REBOOT_REQUESTED:
            return "REBOOT_REQUESTED";
        case EventKind._STOP_REQUESTED:
            return "STOP_REQUESTED";
        default:
            return "unknown event kind !";
        }
    }

    //
    // SupervisionObserver.notify() implementation
    //
    public void _notify(Event event)
    {
        String msg = event.to_string();
        System.out.println("   ... " + eventKindToString(event)
                           + " notification received from Supervision : \n"
                           + msg);

        // save event in event queue
        supEventsList.addLast(event);
    }

    public Event takeSupervisionEvent()
    {
        // get last event from event queue or null if empty
        try {
            return supEventsList.removeFirst();
        }
        catch (java.util.NoSuchElementException  e) {
            return null;
        }
    }

    //
    // CCMAllCapabilitiesTester class
    //
    public static class CCMAllCapabilitiesTester
    {
        private static final String CCM_CLIENT_IMPL_NAME
            = "CCMClientImpl_object";

        private static final String SESSION_CONTEXT_NAME
            = "CCM_Client_Context_object";

        private static final String COMPONENT_NAME
            = "Client_Component";

        private CORBATestManager testManager;
        private CCMClientImpl ccmClientImpl;
        private CCM_Client_Context sessionContext;
        private com.acme.CcmAllCapabilitiesTest.Client componentRef;


        public CCMAllCapabilitiesTester()
        {
        }


    private void readLine() {
        try {
        java.io.BufferedReader inputReader =
            new java.io.BufferedReader(new java.io.InputStreamReader(System.in));
        inputReader.readLine();
        } catch (java.io.IOException e) {
        e.printStackTrace();
        }
    }

        @Before
        public void init ()
        {
            testManager = CORBATestManager.instance();

            this.ccmClientImpl = (CCMClientImpl)
                testManager.getLocalProperty(CCM_CLIENT_IMPL_NAME);
            assertNotNull(this.ccmClientImpl);

            this.sessionContext = (CCM_Client_Context)
                testManager.getLocalProperty(SESSION_CONTEXT_NAME);
            assertNotNull(this.sessionContext);

            this.componentRef = (com.acme.CcmAllCapabilitiesTest.Client)
                testManager.getLocalProperty(COMPONENT_NAME);
            assertNotNull(this.componentRef);
        }


        @Test
        public void doTests()
        {
            System.out.println("CCMAllCapabilitiesTester.doTests()");
            String observerName = "TestCcmAllCapabilities_Client";
            SupervisionObserver oldObserver;
            Event event;

            // wait previous connection requests to be recorded...
            System.out.println("Wait for the system is ready before running the tests.");
            try {
                System.out.println("Client sleeps 20 sec.");
                cdmw.ossupport.OS.sleep(20000);
            }
            catch (InterruptedException e) {
                // ignore
            }


            // get ManagedComponentContext
            System.out.println("Testing ManagedComponentContext QoS Service");
            ComponentContext supContext = null;
            try {
                org.omg.CORBA.Object obj =
                    sessionContext.resolve_service_reference("ManagedComponentContext");
                supContext = ComponentContextHelper.narrow(obj);
                assertNotNull(supContext);
            }
            catch (org.omg.Components.CCMException ccme) {
                fail("Failure when getting the ManagedComponentContext: "
                     + ccme.getMessage());
            }

            // get System supervision object
            System.out.println("   get System supervision object");
            com.thalesgroup.CdmwPlatformMngt.External.System system =
                supContext.the_system();

            // get component, process, application names
            String componentName = supContext.managed_element_name();
            System.out.println("      component_name = '"
                               + (componentName == null ? "null" : componentName)
                               + "'");
            assertNotNull(componentName);
            assertEquals(CCMClientImpl.CLIENT_COMPONENT_NAME, componentName);

            String processName = supContext.process_name();
            System.out.println("      process_name = '"
                               + (processName == null ? "null" : processName)
                               + "'");
            assertNotNull(processName);
            assertEquals(CCMClientImpl.CLIENT_PROCESS_NAME, processName);

            String appliName = supContext.application_name();
            System.out.println("      appli_name = '"
                               + (appliName == null ? "null" : appliName)
                                   + "'");
            assertNotNull(appliName);
            assertEquals(CCMClientImpl.APPLICATION_NAME, appliName);

            // test System supervision object
            System.out.println("   test System status");
            org.omg.CORBA.StringHolder statusInfo =
                new org.omg.CORBA.StringHolder();

            int nbtryremain = 70;
            while(nbtryremain > 0) {
                if (system.get_status(statusInfo) == SystemStatus.SYSTEM_STARTED) {
                    break;
                }

                try {
                    System.out.println("get system status info failed, retry in 2 seconds (" + nbtryremain + " attempts left)");
                    cdmw.ossupport.OS.sleep(2000);
                }
                catch (InterruptedException e) {
                    // ignore
                }
                nbtryremain--;
            }
            if (nbtryremain == 0) {
                assertTrue(false);
            }
            System.out.println("      status info: " + statusInfo.value);

            // test Application supervision object
            System.out.println("   test Application status");
            com.thalesgroup.CdmwPlatformMngt.External.Application application = supContext.the_application();
            nbtryremain = 5;
            while(nbtryremain > 0) {
                if (application.get_status(statusInfo)
                        == ApplicationStatus.APPLICATION_RUNNING)
                    break;
                  try {
                      System.out.println("get application status info failed, retry in 2 seconds (" + nbtryremain + " attempts left)");
                    cdmw.ossupport.OS.sleep(2000);
                  }
                  catch (InterruptedException e) {
                      // ignore
                  }
                  nbtryremain--;
            }
            if (nbtryremain == 0) {
                assertTrue(false);
            }
            System.out.println("      status info: " + statusInfo.value);

            // test Process supervision object
            System.out.println("   test Process status");
            com.thalesgroup.CdmwPlatformMngt.External.Process process =
                supContext.the_process();
            nbtryremain = 5;
            while(nbtryremain > 0) {
                if(process.get_status(statusInfo)
                       == ProcessStatus.PROCESS_RUNNING) {
                    break;
                }
                try {
                      System.out.println("get process status info failed, retry in 2 seconds (" + nbtryremain + " attempts left)");
                    cdmw.ossupport.OS.sleep(2000);
                  }
                  catch (InterruptedException e) {
                      // ignore
                  }
                  nbtryremain--;
            }
            if (nbtryremain == 0) {
                assertTrue(false);
            }

            System.out.println("      status info: " + statusInfo.value);

            // test Host supervision object
            System.out.println("   test Host status");
            com.thalesgroup.CdmwPlatformMngt.External.Host host =
                supContext.the_host();
            nbtryremain = 5;
            while (nbtryremain > 0) {
                if (host.status() == HostStatus.HOST_STARTED) {
                    break;
                }
                try {
                    System.out.println("get host status info failed, retry in 2 seconds ("
                            + nbtryremain + " attempts left)");
                    cdmw.ossupport.OS.sleep(2000);
                } catch (InterruptedException e) {
                    // ignore
                }
                nbtryremain--;
            }
            if (nbtryremain == 0) {
                assertTrue(false);
            }
            System.out.println("      status info: HOST_STARTED");

            // test RedundancyGroup supervision object
            System.out.println("   test RedundancyGroup status");
            com.thalesgroup.CdmwPlatformMngt.External.RedundancyGroup group =
                supContext.the_redundancy_group();
            assertNull(group);


            // Another LB client is supposed to perform 10 requests to the
            // LB servers. Here we test that these requests have been recorded.
            //
            System.out.println(
                "Check that all calls to \"parse\" op have been recorded");
            System.out.println("Client sleeps 10 sec.");
            try {
                Thread.sleep(10000);
            } catch (InterruptedException ie) {}

            // now checks the recording service works;
            // 10 records are expected
            //
            String fileName = ccmClientImpl.fileName();
            System.out.println("fileName "+fileName);
            for (int i = 0; i < 5; i++) {
                assertTrue(
                    "\"parse\" was not recorded on ServerLB1",
                    find_parse_record("ServerLB1", fileName));
            }
            System.out.println(
                "Requests to the first LB server recorded successfully");
            for (int i = 0; i < 5; i++) {
                assertTrue(
                    "\"parse\" was not recorded on ServerLB2",
                    find_parse_record("ServerLB2", fileName));
            }
            System.out.println(
                "Requests to the second LB server recorded successfully");


            // Now we perform 5 more requests to the LB server group, but as the connection
            // is not LB only one server is expected to treat all requests.
            // Check this is true, and that the recording works so well.
            //
            ccmClientImpl.recordList.clear();
            String previousProcessName = "";

            for (int i = 0; i < 5; i++) {
                String currentProcessName = test_lb_connections();
                assertTrue(! currentProcessName.equals(""));

                if (i == 0 || currentProcessName.equals(previousProcessName)) {
                    previousProcessName = currentProcessName;
                    assertTrue(true);
                }
                else {
                    assertEquals("Server changed although the LB service is not supported!",
                                 previousProcessName, currentProcessName);
                }
            }

            System.out.println("\nCheck that the records have been registered.");
            try {
                cdmw.ossupport.OS.sleep(3000);
            }
            catch (InterruptedException e) {
                // ignore
            }
            for (int i = 0; i < 5; i++) {
                assertTrue(
                    "\"parse\" was not recorded",
                    find_parse_record("ServerLB1", fileName)
                    || find_parse_record("ServerLB2", fileName));
            }


            //
            // Now the test checks the FT client connections to the FT group server.
            //

            // test connections to group : primary hostname is expected
            test_ft_connections(ccmClientImpl.expectedPrimaryHostname());

            try
            {
                // register this component as System observer
                System.out.println("Register this component as System Observer");
                oldObserver =
                    system.register_observer(observerName, componentRef);

                System.out.println("Send kill command to group (only primary will stop)");
                Command command = new CommandImpl();
                command.name = COMMAND_KILL.value;

                System.out.println("Emits command " + command.name);
                sessionContext.push_commandEmits(command);

                System.out.println("Wait new primary election");
                try {
                    cdmw.ossupport.OS.sleep(5000);
                }
                catch (InterruptedException e) {
                    // ignore
                }

                System.out.println("Unregister this component as System Observer");
                oldObserver = system.unregister_observer(observerName);
            }
            catch (Exception e) {
                fail("Raised unexpected exception: " + e);
            }

            // test connections to group : backup hostname is expected
            test_ft_connections(ccmClientImpl.expectedBackupHostname());

            try
            {
                System.out.println("Send kill command to group (only backup will stop)");

                Command command = new CommandImpl();
                command.name = COMMAND_KILL.value;

                System.out.println("Emits command " + command.name);
                sessionContext.push_commandEmits(command);

                System.out.println("Wait new primary election");
                try {
                    cdmw.ossupport.OS.sleep(5000);
                }
                catch (InterruptedException e) {
                    // ignore
                }
            }
            catch (Exception e) {
                fail("Raised unexpected exception: " + e);
            }


            try {
                cdmw.ossupport.OS.sleep(5000);
            }
            catch (InterruptedException e) {
                // ignore
            }

            try
            {
                System.out.println("Test that FT group is now empty");
                HostInfo hostInfo = sessionContext.get_connection_serverHostInfo();

                String hostname = hostInfo.get_hostname();
                fail("Request on empty group succeed !! hostname: " + hostname);
            }
            catch (org.omg.CORBA.TRANSIENT t) {
                System.out.println("Ok, no server reachable.("+ t +")");
            }
            catch (Exception e) {
                fail("Raised unexpected exception: " + e);
            }

        System.out.println("*************************");
        System.out.println("Re run FT Server on HOST2");
        System.out.println("Press any key and Enter");
        System.out.println("*************************");
        readLine();

            // Test cdmw_initialise_component on FT Server (PCR-1983)
            // Facets musn't be reachable before cdmw_initialise_component is completed
            // A boolean set in facets is cheched at the end of the cdmw_initialise_compoent
            int max_retry = 10, retry = 0;
            while(retry < max_retry) {
                try {
                    System.out.println("Try to request the FT Server");
                    HostInfo hostInfo = sessionContext.get_connection_serverHostInfo();
                    String hostname = hostInfo.get_hostname();
                    System.out.println("Request on FT Server succeed !! hostname: " + hostname);
                    retry = -1; // OK
                    break;
                }
                catch (org.omg.CORBA.TRANSIENT t) {
                    System.out.println("Ok, Server not reachable.("+ t +")");
                }
                catch (Exception e) {
                    fail("Raised unexpected exception: " + e);
                }
                try {
                    System.out.println("Client sleeps 5 sec.");
                    Thread.sleep(5000);
                }
                catch(InterruptedException ie) {
                    fail("Raised unexpected exception: " + ie);
                }
                retry++;
            }
            if(retry != -1)
                fail("Max tentative exceed ("+max_retry+")");

            // test System supervision object
            System.out.println("Test System status");
            assertTrue(system.get_status(statusInfo) == SystemStatus.SYSTEM_STARTED
                       || system.get_status(statusInfo) == SystemStatus.SYSTEM_PARTIAL_STARTED);
            System.out.println("      status info: " + statusInfo.value);

            // test Application supervision object
            System.out.println("Test Application status");
            assertTrue(application.get_status(statusInfo) ==
                       ApplicationStatus.APPLICATION_PARTIAL_RUNNING);
            System.out.println("      status info: " + statusInfo.value);

            // test Process supervision object
            System.out.println("Test Process status");
            assertTrue(process.get_status(statusInfo) ==
                       ProcessStatus.PROCESS_RUNNING);
            System.out.println("      status info: " + statusInfo.value);

            // test Host supervision object
            System.out.println("Test Host status");
            host = supContext.the_host();
            assertTrue(host.status() == HostStatus.HOST_STARTED);
            System.out.println("      status info: HOST_STARTED");


            // test observer
            //
            System.out.println("Check supervision notifications can be observed.");

            //  - Test process MODE_CHANGE to FAILED_DEATH
            System.out.println("check process mode changed to FAILED_DEATH");
            ProcessStatusChange deathEvent = (ProcessStatusChange) test_event_change(EventKind.PROCESS_STATUS_CHANGE);
            assertTrue(deathEvent.process_status
                       == ProcessStatus.PROCESS_FAILED_DEATH);

            //  - Test application MODE_CHANGE to DEGRADED
            System.out.println("check application mode changed to DEGRADED");
            ApplicationModeChange appModeChange = (ApplicationModeChange) test_event_change(EventKind.APPLICATION_MODE_CHANGE);
            assertEquals(APPLICATION_NAME, appModeChange.application_name);
            assertTrue(appModeChange.application_mode
                       == FunctioningMode.DEGRADED_MODE);

            //  - Test system MODE_CHANGE to DEGRADED
            System.out.println("      check system mode changed to DEGRADED");
            SystemModeChange sysModeChange = (SystemModeChange) test_event_change(EventKind.SYSTEM_MODE_CHANGE);
            assertTrue(sysModeChange.system_mode
                       == FunctioningMode.DEGRADED_MODE);

            //  - Test application STATUS_CHANGE to COMPONENT_FAILED_ERROR
            System.out.println("      check application status changed to COMPONENT_STATUS_CHANGE");
            ComponentStatusChange componentStatusChange = (ComponentStatusChange) test_event_change(EventKind.COMPONENT_STATUS_CHANGE);
            assertTrue(componentStatusChange.component_status
                       == ComponentStatus.COMPONENT_FAILED_ERROR);
            
            //  - Test application STATUS_CHANGE to PARTIAL_RUNNING
            System.out.println("      check application status changed to PARTIAL_RUNNING");
            ApplicationStatusChange appStatusChange = (ApplicationStatusChange) test_event_change(EventKind.APPLICATION_STATUS_CHANGE);
            assertEquals(APPLICATION_NAME, appStatusChange.application_name);
            assertTrue(appStatusChange.application_status
                       == ApplicationStatus.APPLICATION_PARTIAL_RUNNING);

            //  - Test system STATUS_CHANGE to PARTIAL_STARTED
            System.out.println("      check system status changed to PARTIAL_STARTED");
            SystemStatusChange sysStatusChange = (SystemStatusChange) test_event_change(EventKind.SYSTEM_STATUS_CHANGE);
            assertTrue(sysStatusChange.system_status
                       == SystemStatus.SYSTEM_PARTIAL_STARTED);

            //  - Test host STATUS_CHANGE to PARTIAL_STARTED
            System.out.println("      check host status changed to PARTIAL_STARTED");
            HostStatusChange hostStatusChange = (HostStatusChange) test_event_change(EventKind.HOST_STATUS_CHANGE);
            assertTrue(hostStatusChange.host_status
                       == HostStatus.HOST_PARTIAL_STARTED);

            // register this component as System observer
            System.out.println("Register this component as System Observer");
            oldObserver =
                system.register_observer(observerName, componentRef);

            // test message notification to Supervision
            supContext.notify_message(INF.value,
                                       "Test supervision observer");
            try {
                PlatformInterface.notifyMessage(
                    INF.value, PlatformInterface.getProcessName(),
                    "Test supervision observer");
                cdmw.ossupport.OS.sleep(2000);

                System.out.println("Unregister this component as System Observer");
                oldObserver = system.unregister_observer(observerName);
            }
            catch (Exception e) {
                fail("Raised an unexpected exception: " + e);
            }

            event = ccmClientImpl.takeSupervisionEvent();
            assertNotNull(event);
            assertTrue(event.event_kind == EventKind.PROCESS_MESSAGE);
            event = ccmClientImpl.takeSupervisionEvent();
            assertNotNull(event);
            assertTrue(event.event_kind == EventKind.PROCESS_MESSAGE);
            event = ccmClientImpl.takeSupervisionEvent();
            assertNull(event);


            /////////////////////////////
            // end test
            ///////////////////////////
            testManager.printSummary();
        }


        Event test_event_change(EventKind kind)
        {
            int nbtryremain = 5;
            while (nbtryremain > 0)
            {
                Event event = ccmClientImpl.takeSupervisionEvent();
                if (event != null && event.event_kind == kind)
                {
                    return event;
                }
                nbtryremain--;
                try 
                {
                    cdmw.ossupport.OS.sleep(2000);
                }
                catch (Exception e) {
                    fail("Raised an unexpected exception: " + e);
                }
            }
            if (nbtryremain == 0)
                fail("Max tentative exceed to check application status changed");
            return null;
        }

        //
        String test_lb_connections()
        {
            org.omg.CORBA.StringHolder processName = new org.omg.CORBA.StringHolder();
            try
            {
                System.out.println("Testing connections to ServerLB group");

                // get the server HostInfo
                XMLValidator validator =
                    sessionContext.get_connection_serverXMLValidator();
                assertNotNull("XML Validator is NIL!", validator);

                // read the hostname from the server HostInfo
                String fileName = ccmClientImpl.fileName();
                boolean res = validator.parse(fileName, processName);
                assertTrue("Parsing of XML file " + fileName + " failed", res);
            }
            catch (Exception e) {
                fail("Raised unexpected exception: " + e);
            }
            finally {
                return processName.value;
            }
        }


        void test_ft_connections(String expectedHostname)
        {

            // First test the facet connection
            try
            {
                System.out.println("Test synchronous connections to ServerFT group"
                                   + " (expected primary host: " + expectedHostname + ")");

                HostInfo hostInfo = sessionContext.get_connection_serverHostInfo();
                assertNotNull("HostInfo is NIL!", hostInfo);

                String hostname = hostInfo.get_hostname();
                assertEquals("Server hostname is not the one expected",
                             expectedHostname, hostname);
            }
            catch (Exception e) {
                fail("Raised unexpected exception: " + e);
            }

            // Then test the connection with events
            try
            {
                System.out.println("Test asynchronous connections to ServerFT group"
                                   + " (expected primary host: " + expectedHostname + ")");

                Command command = new CommandImpl();
                command.name = COMMAND_GET_HOSTNAME.value;

                System.out.println("Emits command " + command.name);
                sessionContext.push_commandEmits(command);

                System.out.println("Wait result");
                try {
                    cdmw.ossupport.OS.sleep(2000);
                }
                catch (InterruptedException e) {
                    // ignore
                }

                String hostname = ccmClientImpl.getHostnameCommandResult();
                assertEquals("Server hostname is not the one expected",
                             expectedHostname, hostname);

                System.out.println("Publishes command " + command.name);
                sessionContext.push_commandPublishes(command);

                System.out.println("Wait result");
                try {
                    cdmw.ossupport.OS.sleep(2000);
                }
                catch (InterruptedException e) {
                    // ignore
                }

                hostname = ccmClientImpl.getHostnameCommandResult();
                assertEquals("Server hostname is not the one expected",
                             expectedHostname, hostname);
            }
            catch (Exception e) {
                fail("Raised unexpected exception: " + e);
            }
        }


        private boolean find_parse_record(String componentName,
                                          String message)
        {
            for (RequestReplyPair pair : ccmClientImpl.recordList)
            {
                // test is record is for a "test_operation"
                if (pair.request.operation.equals("parse")) {

                    // check if server_id ends with componentName
                    if (pair.request.server_id.endsWith(componentName)) {

                        // try to unmarshall parameters as a string
                        boolean endian =
                            (pair.request.head.flags != 0) ? true : false;
                        org.omg.CORBA.portable.InputStream input =
                            new org.jacorb.orb.CDRInputStream(
                                org.omg.CORBA.ORB.init(),
                                pair.request.data,
                                endian);
                        try {
                            String str = input.read_string();

                            // unmarshall OK. Test if it's expected message
                            if (message.equals(str)) {
                                // message found, remove it from list
                                ccmClientImpl.recordList.remove(pair);
                                return true;
                            }
                        }
                        catch (org.omg.CORBA.MARSHAL e) {
                            System.out.println(
                                "record for \"test_operation\" found, " +
                                "but parameter is not a string!");
                        }
                    }
                }
            }

            return false;
        }

    }
}
