/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ccmallcapabilities;

import cdmw.common.Assert;
import cdmw.logging.Logger;
import static cdmw.logging.Logger.*;

import com.thalesgroup.CdmwXML.XMLParserPackage.ParsingException;
import com.thalesgroup.CdmwXML.XMLParser;
import cdmw.platformmngt.PlatformInterface;

import com.acme.CcmAllCapabilitiesTest.*;
import UserCcmAllCapabilitiesTest.*;

//
// IDL:acme.com/CcmAllCapabilitiesTest/CCM_ServerLB:1.0
//
public class CCMServerLBImpl extends org.omg.CORBA.LocalObject
    implements User_CCM_ServerLB
{
    
    private static final String XML_SERVICE_NAME = "XMLService";

    // session context
    private CCM_ServerLB_Context sessionContext;
     
    private Logger logger;
    private XMLParser xmlParser;
    private String expectedClientHostname;
    private boolean isFacetCalled;
    
    public CCMServerLBImpl() 
    {
        this.sessionContext = null;
        this.logger = Logger.Get_Logger("CCM_ServerLB",
                                        "QUALIF_CCM_ALL_CAPABILITIES");
        this.isFacetCalled = false;

        // get the XML Server reference
        try {
            org.omg.CORBA.Object obj =
                PlatformInterface.getService(XML_SERVICE_NAME);
            xmlParser = com.thalesgroup.CdmwXML.XMLParserHelper.narrow(obj);
        }
        catch (com.thalesgroup.CdmwPlatformMngtService.ServiceNotFound e) {
            System.out.println("Test Failed: XML Service not found.");
        }
        catch (com.thalesgroup.CdmwPlatformMngtService.ServiceUnavailable e) {
            System.out.println("Test Failed: XML Service not available.");
        }
        catch (cdmw.common.BadOrderException e) {
            System.out.println("Test Failed: BadOrderException " +
                               "while retrieving XML Service.");
        }
        catch (cdmw.common.BadParameterException e) {
            System.out.println("Test Failed: " +
                "BadParameterException while retrieving XML Service.");
        }
        System.out.println("############ ServerLB creator 2 ");
    }

    
    //
    // attribute expectedClientHostname
    //
    public String expectedClientHostname() {
        return this.expectedClientHostname;
    }
    public void expectedClientHostname(String hostname) {
        this.expectedClientHostname = hostname;
    }


    //
    // facet serverXMLValidator
    //
    public CCM_XMLValidator get_serverXMLValidator() {
        CDMW_INFO(this.logger, "CCMServerLBImpl.get_serverXMLValidator().",
                  new Throwable());
        this.isFacetCalled = false;
        return (CCM_XMLValidator) this;
    }

    public boolean parse(java.lang.String file_name,
                         org.omg.CORBA.StringHolder process_name) {

        CDMW_INFO(this.logger, "CCMServerLBImpl.parse(): " + file_name,
                  new Throwable());

        boolean res = false;
        try {
            process_name.value = cdmw.platformmngt.PlatformInterface.getProcessName();
            xmlParser.parse(file_name);
            res = true;
        }
        catch (final cdmw.common.BadOrderException boe) {
            CDMW_ERROR(logger, "Exception raised when getting the process name: " + boe,
                       new Throwable());
        }
        catch (final ParsingException pe) {
            CDMW_ERROR(logger, "Exception raised in parsing: " + pe,
                       new Throwable());
            System.out.println("#### Failure in parsing " + file_name
                               + ": " + pe);
        }
        catch (final Exception e) {
            System.out.println("Unexpected exception: " + e);
        }
        return res;
    }


    public void set_session_context(org.omg.Components.SessionContext ctx) 
    {
        CDMW_INFO(this.logger, "CCMServerLBImpl.set_session_context()",
                  new Throwable());

        this.sessionContext = CCM_ServerLB_ContextHelper.narrow(ctx);
    }
    
    public CCM_ServerLB_Context getContext()
    {
        return this.sessionContext;
    }
    
    public void cdmw_initialise_component(com.thalesgroup.CdmwPlatformMngtBase.StartupKind startupKind)
        throws com.thalesgroup.CdmwPlatformMngt.ManagedComponentPackage.BadOrder {
        CDMW_INFO(this.logger,
                  "*** Component Server LB Initialisation ***",
                  new Throwable());
        try {
            Thread.sleep(10000);
        } catch (InterruptedException e) {
            System.err.println("Thread was interrupted");
            e.printStackTrace();
        }
        Assert.check(! isFacetCalled);
        CDMW_INFO(this.logger,
                  "*** Component Server LB Initialised ***",
                  new Throwable());
    }

    public void ccm_activate() 
    {
        CDMW_INFO(this.logger, "CCMServerLBImpl.ccm_activate()",
                  new Throwable());
    }
    
    public void ccm_passivate() 
    {
        CDMW_INFO(this.logger, "CCMServerLBImpl.ccm_passivate()",
                  new Throwable());
    }
    
    public void ccm_remove() 
    {
        CDMW_INFO(this.logger, "CCMServerLBImpl.ccm_remove()",
                  new Throwable());
    }
    

}
