/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ccmandlb;

import org.omg.CORBA.SystemException;

import cdmw.logging.Logger;
import static cdmw.logging.Logger.*;
import com.acme.CcmAndLB.*;
import org.omg.CosClockService.ClockCatalog;
import org.omg.Components.SessionContext;
import com.thalesgroup.CdmwComponents.SessionContextExt;
import com.thalesgroup.CdmwComponents.CCMContextExtPackage.ServiceUnavailable;
import com.thalesgroup.CdmwComponents.SessionContextExtHelper;
import org.omg.CosClockService.ClockCatalogHelper;


public class ServerExecutorImpl extends org.omg.CORBA.LocalObject
    implements User_Server
{
    private CCM_HelloWorld display;
    private Logger logger;
    private ClockCatalog clockCatalog;
    private CCM_Server_Context sessionContext;
    private SessionContextExt sessionContextExt;
    private com.acme.CcmAndLB.Server ccm_component_ref;
    
    private String name = "LBServer";

    public ServerExecutorImpl()
    {
        this.logger = Logger.Get_Logger("CCM_Server",
                                        "QUALIF_CCM_AND_LB");

        
        this.display = new HelloWorldFacetExecutorImpl(this);
        
        this.name(this.getProcessName());
    }

    public CCM_HelloWorld get_server_facet()
    {
        CDMW_INFO(this.logger, "ServerExecutor_impl::get_server_facet() called.",
                         new Throwable());

        return this.display;
    }

    /**
     * We added the management of clock service in order to
     * test the co-existent of LB and TMG service in the Server Java (component)
     */
    ClockCatalog get_clockCatalog()
    {
        CDMW_INFO(this.logger, "ServerExecutor_impl::get_clockCatalog() called.",
                         new Throwable());

        return this.clockCatalog;
    }
    /**
     * periodic execution
     */

    public boolean do_work(org.omg.CORBA.Any args)
    {
        // do noting
        return true;
    }

    /**
     * 
     *
     */
    public void set_session_context(SessionContext ctx)
        throws org.omg.CORBA.SystemException
    {
        CDMW_INFO(this.logger, "ServerExecutor_impl::set_sessionContext() called.",
                         new Throwable());

        this.sessionContext = CCM_Server_ContextHelper.narrow(ctx);

        // get context for services
        this.sessionContextExt = 
            SessionContextExtHelper.narrow(this.sessionContext);


        // Check the co-existence of LB and TMG service  
        // get Time Service
        org.omg.CORBA.Object obj = null;
        try {
            obj = sessionContextExt.resolve_service_reference("ClockService");
        }
        catch (org.omg.Components.CCMException ccmex) {
            CDMW_ERROR(this.logger, "Exception raised trying to obtain the reference to ClockService " + ccmex,
                              new Throwable());
            // ADD : Test NOK
        }
        try {
            this.clockCatalog = ClockCatalogHelper.narrow(obj);
        }
        catch (java.lang.Exception clockex) {
            CDMW_ERROR(this.logger, "It was not possible to obtain reference to ClockCatalog : Exception raised " + clockex,
                              new Throwable());
            // ADD : TEST NOK
        }
         
        // TEST OK
    }

    /**
     * Activate the component (called by the container).
     *
     * @exception CORBA SystemException
     */
    public void ccm_activate()
        throws org.omg.CORBA.SystemException
    {
        CDMW_INFO(this.logger, "ServerExecutor_impl::ccm_activate() called.",
                         new Throwable());
    }

    /**
     * Stop the component (called by the container).
     *
     * @exception CORBA SystemException
     */
    public void ccm_passivate()
        throws org.omg.CORBA.SystemException
    {
        CDMW_INFO(this.logger, "ServerExecutor_impl::ccm_passivate() called.",
                         new Throwable());
    }

    /**
     * Destroy the component (called by the container).
     *
     * @exception CORBA SystemException
     */
    public void ccm_remove()
        throws org.omg.CORBA.SystemException
    {
        CDMW_INFO(this.logger, "ServerExecutor_impl::ccm_remove() called.",
                         new Throwable());
    }

    /**
     * For Managed Components
     */
    public void cdmw_initialise_component(com.thalesgroup.CdmwPlatformMngtBase.StartupKind startupKind)
        throws com.thalesgroup.CdmwPlatformMngt.ManagedComponentPackage.BadOrder {

        cdmw_initialise_component();
    }

    /*
    *   The Component Server is a Managed Component
    */
    public void cdmw_initialise_component() {

        CDMW_INFO(this.logger, "Try to initialise Managed Component", new Throwable());
        System.err.println("**** cdmw_initialise_component() ****");
        try {
                ccm_component_ref =
                com.acme.CcmAndLB.ServerHelper.narrow(
                   sessionContextExt.get_CCM_object());
        } catch (org.omg.Components.IllegalState e) {
           // TO BE COMPLETED
           CDMW_ERROR(this.logger, "Exception llegalState raise during get_CCM_object", new Throwable());
           System.out.println("** TEST NOK **"); // TODO
           // ADD TEST NOK
        } catch (org.omg.CORBA.BAD_PARAM e) {
           // TO BE COMPLETED
           CDMW_ERROR(this.logger, "Exception BAD_PARAM raised during get_CCM_object", new Throwable());
           // ADD TEST NOK
           System.out.println("** TEST NOK **"); // TODO
        }
        // Obtain the information about SERVER from ManagedComponentContext
        /*try {
            org.omg.CORBA.Object obj_MCC =
                sessionContextExt.resolve_service_reference("ManagedComponentContext");
            com.thalesgroup.CdmwPlatformMngt.ManagedElementContexts.ComponentContext objManagedComponentContext =
                com.thalesgroup.CdmwPlatformMngt.ManagedElementContexts.ComponentContextHelper.narrow(obj_MCC);

            String process_name     = objManagedComponentContext.process_name();
 
            objManagedComponentContext.notify_message(com.thalesgroup.CdmwPlatformMngtBase.INF.value,
                                          "Server Process Name" + process_name );
            System.err.println(">>***** Server_Process_Name: " + process_name + "*****<<");
        } catch (org.omg.Components.CCMException e) {
            cdmw.logging.Logger.CDMW_FATAL(this.logger,"ManagedComponentContext no found", new Throwable());
           // ADD TEST NOK
           System.out.println("** TEST NOK **"); // TODO:
        }*/

        CDMW_INFO(this.logger, "Initialisation of Managed Component OK", new Throwable());
        // ADD TEST OK

    }
    public String getProcessName() {
            try {
                return cdmw.platformmngt.PlatformInterface.getProcessName();
            }
            catch ( cdmw.common.BadOrderException e)
            {
                System.err.println("ERROR");
                return "ERROR";
            }
    }
        
    public String name() {
        return this.name;
    }

    public void name(String name) {
        this.name = name;
    }

}
