/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccmautoreconnect;

import cdmw.testutils.TestManager;
import cdmw.logging.Logger;
import static cdmw.logging.Logger.*;

import com.acme.AutoReconnectDemo.*;


public class ClientExecutorImpl extends org.omg.CORBA.LocalObject
    implements com.thalesgroup.UserAutoReconnectDemo.User_CCM_Client
{
    private CCM_Client_Context sessionContext;
    private Thread activationThread;
    private boolean hasEventBeenReceived;
    private Logger m_logger;

    /**
     * Constructor.
     */
    public ClientExecutorImpl()
    {
        this.sessionContext = null;
        this.activationThread = null;
        this.hasEventBeenReceived = false;
        this.m_logger = Logger.Get_Logger("QUALIF.CCM_AUTO_RECONNECT.CLIENT");

        CDMW_INFO(m_logger, "ClientExecutorImpl created");
    }

    /**
     * Set the session context of the component (called by the container).
     *
     * @param SessionContext a session context
     *
     * @exception CORBA SystemException
     */
    public void set_session_context(org.omg.Components.SessionContext ctx)
        throws org.omg.CORBA.SystemException
    {
        CDMW_INFO(m_logger, "Session context set");
        this.sessionContext = CCM_Client_ContextHelper.narrow(ctx);
    }


    /**
     * Activate the component (called by the container).
     *
     * @exception CORBA SystemException
     */
    public void ccm_activate()
        throws org.omg.CORBA.SystemException
    {
        CDMW_INFO(m_logger, "CCM Activate");
        final ClientExecutorImpl thisClientExecutor = this;

        activationThread = new Thread(new Runnable() {
                public void run() {
                    TestManager testManager = new TestManager();
                    
                    testManager.init("cdmw.ccmreconnect.ClientExecutorImpl");
                    TestManager.activateLocalInstance(testManager);
                    
                    testManager.setLocalProperty(
                                                 ClientActivationThread.CLIENT_EXECUTOR_IMPL, thisClientExecutor);
                    
                    testManager.addTest(ClientActivationThread.class);

                    testManager.runTests();
                    System.out.println("Print summary");
                    testManager.printSummary();
                }
            });
        try {
            activationThread.start();
        } catch (IllegalThreadStateException e) {
            // shouldn't happen
            e.printStackTrace();
        }
    }


    /**
     * Stop the component (called by the container).
     *
     * @exception CORBA SystemException
     */
    public void ccm_passivate()
        throws org.omg.CORBA.SystemException
    {
        CDMW_INFO(m_logger, "CCM Passivate");
        try {
            if (activationThread != null)
                activationThread.join();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }


    /**
     * Destroy the component (called by the container).
     *
     * @exception CORBA SystemException
     */
    public void ccm_remove()
        throws org.omg.CORBA.SystemException
    {
        CDMW_INFO(m_logger, "CCM Remove");
    }
        
    public CCM_Client_Context getContext()
    {
        CDMW_INFO(m_logger, "ClientExecutorImpl.getContext");
        return sessionContext;
    }
        
    //
    // IDL:thalesgroup.com/AutoReconnectDemo/CCM_Client/push_event_sink:1.0
    //
    public void push_event_sink(TestEvent event)
    {
        CDMW_INFO(m_logger, "TestEvent received");
        this.hasEventBeenReceived = true;
    }
        
    public boolean hasEventBeenReceived()
    {
        CDMW_INFO(m_logger, "ClientExecutorImpl.hasEventBeenReceived");

        boolean result = this.hasEventBeenReceived;
        this.hasEventBeenReceived = false;
        return result;
    }
        
}
