/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccmautoreconnect;

import com.acme.AutoReconnectDemo.*;
import cdmw.logging.Logger;
import static cdmw.logging.Logger.*;

class SuicideThread extends Thread {
    private int m_msec;
    private Logger m_logger;

    public SuicideThread(int msec, Logger logger) {
        this.m_msec = msec;
        this.m_logger = logger;
    }

    public void run() {
        CDMW_INFO(this.m_logger, "SUICIDE in " + this.m_msec + " msec.");
        try {
            Thread.sleep (this.m_msec);
        } catch (InterruptedException ie) {
            // Ignore it
        }
        CDMW_INFO(this.m_logger, "SUICIDE");
        System.exit(0);
    }
}

public class ServerExecutorImpl extends org.omg.CORBA.LocalObject
implements com.thalesgroup.UserAutoReconnectDemo.User_CCM_Server
{
    private CCM_Server_Context sessionContext;
    private CCM_EventChecking facet;
    private Logger m_logger;
    
    /**
     *Create a  ServerExecutorImpl.
     *
     */

    public ServerExecutorImpl()
    {
        this.facet = new EventCheckingExecutorImpl();
        this.m_logger = Logger.Get_Logger("QUALIF.CCM_AUTO_RECONNECT.SERVER");

        CDMW_INFO(this.m_logger, "ServerExecutorImpl created");
    }

    /**
     * Set the session context of the component (called by the container).
     *
     * @param SessionContext a session context
     *
     * @exception CORBA SystemException
     */
    public void set_session_context(org.omg.Components.SessionContext ctx)
        throws org.omg.CORBA.SystemException
    {
        CDMW_INFO(this.m_logger, "Session context set");
        this.sessionContext = CCM_Server_ContextHelper.narrow(ctx);
    }
    
    
    /**
     * Activate the component (called by the container).
     *
     * @exception CORBA SystemException
     */
    public void ccm_activate()
       throws org.omg.CORBA.SystemException
    {
        CDMW_INFO(this.m_logger, "CCM Activate");
    }
    
    
    /**
     * Stop the component (called by the container).
     *
     * @exception CORBA SystemException
     */
    public void ccm_passivate()
       throws org.omg.CORBA.SystemException
    {
        CDMW_INFO(this.m_logger, "CCM Passivate");
    }
    
    
    /**
     * Destroy the component (called by the container).
     *
     * @exception CORBA SystemException
     */
    public void ccm_remove()
       throws org.omg.CORBA.SystemException
    {
        CDMW_INFO(this.m_logger, "CCM Remove");
    }

    //
    // IDL:thalesgroup.com/AutoReconnectDemo/CCM_Server/get_for_clients:1.0
    //
    public CCM_EventChecking get_for_clients()
    {
        CDMW_INFO(this.m_logger, "ServerExecutorImpl.get_for_clients");
        return this.facet;
    }

    //
    // IDL:thalesgroup.com/AutoReconnectDemo/CCM_Server/push_event_sink:1.0
    //
    public void push_event_sink(TestEvent event)
    {
        CDMW_INFO(this.m_logger, "TestEvent received : " + event.name);
        if( event.name.compareTo("SUICIDE ME") == 0) {
            SuicideThread thread = new SuicideThread(500, this.m_logger);
            thread.start();
        }
        
        // set flag that event has been received 
        ((EventCheckingExecutorImpl)this.facet).wasEventReceived = true;
     
        // send the same event back to Client
        this.sessionContext.push_event_publish(event);
    }

    //
    // IDL:thalesgroup.com/AutoReconnectDemo/Supported/test_supported:1.0
    //
    public void test_supported()
    {
        // nothing to do
        CDMW_INFO(this.m_logger, "ServerExecutorImpl.test_supported");
    }
}
