/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ccmft;

import com.acme.CcmFtTest.*;
import UserCcmFtTest.*;

import cdmw.logging.Logger;
import static cdmw.logging.Logger.*;

import org.omg.Components.SessionContext;
import com.thalesgroup.CdmwComponents.SessionContextExt;
import com.thalesgroup.CdmwComponents.SessionContextExtHelper;

import com.thalesgroup.CdmwPlatformMngtBase.*;
import com.thalesgroup.CdmwPlatformMngt.SupervisionObserver;
import com.thalesgroup.CdmwPlatformMngt.ManagedElementContexts.*;
import com.thalesgroup.CdmwPlatformMngt.RedundancyEltState;
import com.thalesgroup.CdmwPlatformMngt.Event;
import com.thalesgroup.CdmwPlatformMngt.EventKind;
import com.thalesgroup.CdmwPlatformMngt.SystemStatus;
import com.thalesgroup.CdmwPlatformMngt.SystemStatusChange;
import com.thalesgroup.CdmwPlatformMngt.SystemModeChange;
import com.thalesgroup.CdmwPlatformMngt.ComponentStatusChange;
import com.thalesgroup.CdmwPlatformMngt.ApplicationStatus;
import com.thalesgroup.CdmwPlatformMngt.ApplicationStatusChange;
import com.thalesgroup.CdmwPlatformMngt.ApplicationModeChange;
import com.thalesgroup.CdmwPlatformMngt.ProcessStatus;
import com.thalesgroup.CdmwPlatformMngt.ProcessStatusChange;
import com.thalesgroup.CdmwPlatformMngt.HostStatus;
import com.thalesgroup.CdmwPlatformMngt.HostStatusChange;
import com.thalesgroup.CdmwPlatformMngt.TechnicalStatus;
import com.thalesgroup.CdmwPlatformMngt.FunctioningMode;
import com.thalesgroup.CdmwPlatformMngt.ComponentStatus;
import com.thalesgroup.CdmwPlatformMngt.ExitEvent;
import cdmw.platformmngt.PlatformInterface;

import cdmw.testutils.CORBATestManager;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import java.lang.String;
import org.omg.CORBA.StringHolder;

//
// IDL:acme.com/CcmFtTest/CCM_Client:1.0
//
public class CCMClientImpl extends org.omg.CORBA.LocalObject
    implements User_CCM_Client 
{

    // TestThread
    private class TestThread extends Thread {
        public CORBATestManager testManager;

        TestThread(CORBATestManager testManager) {
            this.testManager = testManager;
        }

        public void run() {
            this.testManager.runTests();
        }
    }


    // session context
    private CCM_Client_Context sessionContext;
    private SessionContextExt sessionContextExt;

    private Logger logger;
    private java.lang.String fileName;
    private String expectedPrimaryHostname ;
    private String expectedBackup1Hostname ;
    private String expectedBackup2Hostname ;
    private String hostnameCommandResult;

    private TestThread testThread;


    public CCMClientImpl() 
    {
        this.sessionContext = null;
        this.testThread = null;
        this.logger = Logger.Get_Logger("CCM_Client",
                                        "QUALIF_CCM_FT");
    }

    
    //
    // attribute expectedPrimaryHostname
    //
    public String expectedPrimaryHostname() {
        return this.expectedPrimaryHostname;
    }
    public void expectedPrimaryHostname(String hostname) {
        this.expectedPrimaryHostname = hostname;
    }


    //
    // attribute expectedBackup1Hostname
    //
    public String expectedBackup1Hostname() {
        return this.expectedBackup1Hostname;
    }
    public void expectedBackup1Hostname(String hostname) {
       this.expectedBackup1Hostname = hostname;
    }

    //
    // attribute expectedBackup2Hostname
    //
    public String expectedBackup2Hostname() {
        return this.expectedBackup2Hostname;
    }
    public void expectedBackup2Hostname(String hostname) {
        this.expectedBackup2Hostname = hostname;
    }

    //
    // facet clientHostInfo
    //
    public
    CCM_HostInfo get_clientHostInfo()
    {
        return (CCM_HostInfo) this;
    }

    public String get_hostname() {
        CDMW_INFO(logger, "CCMClientImpl.get_hostname()",
                  new Throwable());
        String hostname = cdmw.ossupport.OS.getHostname();
        return hostname;
    }

    //
    // consumer hostnameResults
    //
    public void push_hostnameResults(HostnameResult event) {
        CDMW_INFO(this.logger,
                  "CCMClientImpl.push_hostnameResults(), "
                  + "hostname = " + event.hostname, 
                  new Throwable());
        this.hostnameCommandResult = event.hostname;
    }


    //
    // get result of asynchronous getHostname command
    //
    public String getHostnameCommandResult() {
        return this.hostnameCommandResult;
    }

    public void set_session_context(org.omg.Components.SessionContext ctx) 
    {
        CDMW_INFO(this.logger,
                  "CCMClientImpl.set_session_context()", 
                  new Throwable());

        this.sessionContext = CCM_Client_ContextHelper.narrow(ctx);
    }
    
    public CCM_Client_Context getContext()
    {
        return this.sessionContext;
    }

    public void ccm_activate() 
    {
        CDMW_INFO(this.logger,
                  "CCMClientImpl.ccm_activate()", 
                  new Throwable());

        try {
            CORBATestManager testManager = CORBATestManager.instance();
            
            testManager.setLocalProperty(
                CCMFtTester.CCM_CLIENT_IMPL_NAME,
                this);
            testManager.setLocalProperty(
                CCMFtTester.SESSION_CONTEXT_NAME,
                this.sessionContext);

            org.omg.Components.SessionContext ctx =
                org.omg.Components.SessionContextHelper.narrow(sessionContext);
            org.omg.CORBA.Object obj = ctx.get_CCM_object();
            com.acme.CcmFtTest.Client thisComponent =
                ClientHelper.narrow(obj);
            testManager.setLocalProperty(
                CCMFtTester.COMPONENT_NAME,
                thisComponent);

            testManager.addTest(CCMFtTester.class);
            this.testThread = new TestThread(testManager);
            this.testThread.start();
        }

        catch (Exception e) {
            System.err.println("Exception thrown during test: " + e);
        }
    }
    
    public void ccm_passivate() 
    {
        CDMW_INFO(this.logger,
                  "CCMClientImpl.ccm_passivate()", 
                  new Throwable());

        if (this.testThread != null) {
            try {
                this.testThread.join();
            }
            catch (java.lang.InterruptedException e) {
                // ignore
            }
        }
    }
    
    public void ccm_remove() 
    {
        CDMW_INFO(this.logger,
                  "CCMClientImpl.ccm_remove()", 
                  new Throwable());
    }

    //
    // CCMFtTester class
    //
    public static class CCMFtTester
    {
        private static final String CCM_CLIENT_IMPL_NAME
            = "CCMClientImpl_object";

        private static final String SESSION_CONTEXT_NAME
            = "CCM_Client_Context_object";
            
        private static final String COMPONENT_NAME
            = "Client_Component";

        private CORBATestManager testManager;
        private CCMClientImpl ccmClientImpl;
        private CCM_Client_Context sessionContext;
        private com.acme.CcmFtTest.Client componentRef;

        
        public CCMFtTester()
        {
        }

        
        @Before
        public void init () 
        {
            testManager = CORBATestManager.instance();

            this.ccmClientImpl = (CCMClientImpl) 
                testManager.getLocalProperty(CCM_CLIENT_IMPL_NAME);
            assertNotNull(this.ccmClientImpl);

            this.sessionContext = (CCM_Client_Context)
                testManager.getLocalProperty(SESSION_CONTEXT_NAME);
            assertNotNull(this.sessionContext);

            this.componentRef = (com.acme.CcmFtTest.Client)
                testManager.getLocalProperty(COMPONENT_NAME);
            assertNotNull(this.componentRef);
        }


        @Test
        public void doTests() 
        {
            System.out.println("CCMFtTester.doTests()");
            Event event;

            //
            // Now the test checks the FT client connections to the FT group server.
            // 

            // test connections to group : primary hostname is expected
            test_ft_connections(ccmClientImpl.expectedPrimaryHostname());

            try
            {
                // register this component as System observer
                System.out.println("Register this component as System Observer");
//                 oldObserver =
//                     system.register_observer(observerName, componentRef);

                System.out.println("Send kill command to group (only primary will stop)");
                Command command = new CommandImpl();
                command.commandName = COMMAND_KILL.value;
    
                System.out.println("Emits command " + command.commandName);
                sessionContext.push_commandEmits(command);
        
                System.out.println("Wait new primary election");
                try {
                    cdmw.ossupport.OS.sleep(5000);
                }
                catch (InterruptedException e) {
                    // ignore
                }

                System.out.println("Unregister this component as System Observer");
//                 oldObserver = system.unregister_observer(observerName);
            }
            catch (Exception e) {
                fail("Raised unexpected exception: " + e);
            }
    
            // test connections to group : backup hostname is expected
            test_ft_connections(ccmClientImpl.expectedBackup1Hostname());

            try
            {
                System.out.println("Send kill command to group (only backup will stop)");

                Command command = new CommandImpl();
                command.commandName = COMMAND_KILL.value;
    
                System.out.println("Emits command " + command.commandName);
                sessionContext.push_commandEmits(command);
        
                System.out.println("Wait new primary election");
                try {
                    cdmw.ossupport.OS.sleep(5000);
                }
                catch (InterruptedException e) {
                    // ignore
                }
            }
            catch (Exception e) {
                fail("Raised unexpected exception: " + e);
            }
    
            // test connections to group : backup hostname is expected
            test_ft_connections(ccmClientImpl.expectedBackup2Hostname());

            try
            {
                System.out.println("Send kill command to group (only backup will stop)");

                Command command = new CommandImpl();
                command.commandName = COMMAND_KILL.value;
    
                System.out.println("Emits command " + command.commandName);
                sessionContext.push_commandEmits(command);
        
                System.out.println("Wait new primary election");
                try {
                    cdmw.ossupport.OS.sleep(5000);
                }
                catch (InterruptedException e) {
                    // ignore
                }
            }
            catch (Exception e) {
                fail("Raised unexpected exception: " + e);
            }

            try
            {
                System.out.println("Test that FT group is now empty");
                HostInfo hostInfo = sessionContext.get_connection_serverHostInfo();
    
                String hostname = hostInfo.get_hostname();
                fail("Request on empty group succeed !! hostname: " + hostname);
            }
            catch (org.omg.CORBA.TRANSIENT t) {
                System.out.println("Ok, no server reachable.");
            }
            catch (Exception e) {
                fail("Raised unexpected exception: " + e);
            }

            /////////////////////////////
            // end test
            ///////////////////////////
            testManager.printSummary();
        }
        
        void test_ft_connections(String expectedHostname)
        {

            // First test the facet connection
            try
            {
                System.out.println("Test synchronous connections to ServerFT group"
                                   + " (expected primary host: " + expectedHostname + ")");

                HostInfo hostInfo = sessionContext.get_connection_serverHostInfo();
                assertNotNull("HostInfo is NIL!", hostInfo);
        
                String hostname = hostInfo.get_hostname();
                assertEquals("Server hostname is not the one expected", 
                             expectedHostname, hostname);
            }
            catch (Exception e) {
                fail("Raised unexpected exception: " + e);
            }

            // Then test the connection with events
            try
            {
                System.out.println("Test asynchronous connections to ServerFT group"
                                   + " (expected primary host: " + expectedHostname + ")");

                Command command = new CommandImpl();
                command.commandName = COMMAND_GET_HOSTNAME.value;

                System.out.println("Emits command " + command.commandName);
                sessionContext.push_commandEmits(command);

                System.out.println("Wait result");
                try {
                    cdmw.ossupport.OS.sleep(2000);
                }
                catch (InterruptedException e) {
                    // ignore
                }
        
                String hostname = ccmClientImpl.getHostnameCommandResult();
                assertEquals("Server hostname is not the one expected", 
                             expectedHostname, hostname);

                System.out.println("Publishes command " + command.commandName);
                sessionContext.push_commandPublishes(command);
        
                System.out.println("Wait result");
                try {
                    cdmw.ossupport.OS.sleep(2000);
                }
                catch (InterruptedException e) {
                    // ignore
                }
        
                hostname = ccmClientImpl.getHostnameCommandResult();
                assertEquals("Server hostname is not the one expected", 
                             expectedHostname, hostname);
            }
            catch (Exception e) {
                fail("Raised unexpected exception: " + e);
            }
        }
    }
}
