/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */

package cdmw.testftrecorder;

import com.acme.TestCcmFtRecorder.*;
import com.thalesgroup.CdmwRecording.*;

import cdmw.testutils.TestManager;
import cdmw.testutils.CORBATestManager;
import org.junit.Before;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import cdmw.orbsupport.ExceptionMinorCodes;
import cdmw.orbsupport.CORBASystemExceptionWrapper;



//
// IDL:acme.com/TestCcmFtRecorder/CCM_Client:1.0
//
public class CCMClientImpl extends org.omg.CORBA.LocalObject
    implements Session_CCM_Client 
{
    // session context
    private CCM_Client_Context sessionContext;
    
    // Number of recorded operations
    short nbTestOperation;
    short nbGenericOperation;

    // List of Sent operations
    java.util.List<RecordedOperationEvent> sendList;
     
    // List of Recorded operations   
    java.util.List<RecordedOperationEvent> recordList;
     
    // TestThread
    private class TestThread extends Thread {
        public CORBATestManager testManager;
        
        TestThread(CORBATestManager testManager) {
            this.testManager = testManager;
        }
        
        public void run() {
            this.testManager.runTests();
            this.testManager.printSummary();
        }
    }
    private TestThread testThread;
    
    
    public CCMClientImpl() 
    {
        this.sessionContext = null;
        this.testThread = null;

        this.nbTestOperation = 0;
        this.nbGenericOperation = 0;
        this.sendList = new java.util.Vector<RecordedOperationEvent>();
        this.recordList = new java.util.Vector<RecordedOperationEvent>();
    }

    
    public void set_session_context(org.omg.Components.SessionContext ctx) 
    {
        System.out.println("*** CCMClientImpl.set_session_context() called!");
        this.sessionContext = CCM_Client_ContextHelper.narrow(ctx);
    }
    
    public CCM_Client_Context getContext()
    {
        return this.sessionContext;
    }
    
    
    public void ccm_activate() 
    {
        System.out.println("*** CCMClientImpl.ccm_activate() called!");

        try {
            CORBATestManager testManager = CORBATestManager.instance();
            CORBATestManager.activateLocalInstance(testManager);
            
            testManager.init(
                "cdmw.testftrecorder.CCMClientImpl.CCMClientActivateTest");
            testManager.setLocalProperty(
                CCMClientActivateTest.CCM_CLIENT_IMPL_NAME,
                this);
            testManager.setLocalProperty(
                CCMClientActivateTest.SESSION_CONTEXT_NAME,
                this.sessionContext);
            testManager.addTest(CCMClientActivateTest.class);
                        
            this.testThread = new TestThread(testManager);
            this.testThread.start();
            
        } catch (Exception e) {
            System.err.println("Exception thrown during test: " + e);
        }
    }
    
    public void ccm_passivate() 
    {
        System.out.println("*** CCMClientImpl.ccm_passivate() called!");
        if (this.testThread != null) {
            try {
                this.testThread.join();
            } catch (java.lang.InterruptedException e) {
                // ignore
            }
        }
    }
    
    public void ccm_remove() 
    {
        System.out.println("*** CCMClientImpl.ccm_remove() called!");
    }
    
        
    
    // event_consumer sink
    public void push_event_consumer(RecordedOperationEvent evt)
    {
        System.out.println("*** CCMClientImpl.push_event_consumer() called! -  Received : " + evt.operation_name +
                           " from " + evt.recorder_name);
        
        if (evt.operation_name.equals("test_operation"))
            this.nbTestOperation++;
        else if (evt.operation_name.equals("record_data"))
            this.nbGenericOperation++;
        else 
            fail("Unknown Event received !!!");
        
        // Add the event to the list of recorded operations
        this.recordList.add(evt);
    }


    //
    // CCMClientActivateTest class
    //
    public static class CCMClientActivateTest 
    {
        private static final String CCM_CLIENT_IMPL_NAME
            = "CCMClientImpl_object";

        private static final String SESSION_CONTEXT_NAME
            = "CCM_Client_Context_object";
            
        private CCMClientImpl ccmClientImpl;
        private CCM_Client_Context sessionContext;
        
        public CCMClientActivateTest()
        {
            this.ccmClientImpl = null;
            this.sessionContext = null;
        }
        
        
        @Before
        public void init () 
        {
            CORBATestManager testManager = CORBATestManager.instance();

            this.ccmClientImpl = 
                (CCMClientImpl)testManager.getLocalProperty(CCM_CLIENT_IMPL_NAME);
            assertTrue(this.ccmClientImpl != null);
            this.sessionContext = 
                (CCM_Client_Context)testManager.getLocalProperty(SESSION_CONTEXT_NAME);
            assertTrue(this.sessionContext != null);
        }
        
        private static final String MY_RECORDER_PRIMARY = "MyRecorder1";
        private static final String MY_RECORDER_BACKUP  = "MyRecorder2";
        private static final String OPERATION_NAME = "test_operation";        
        private static final String GENERIC_OPERATION_NAME = "record_data";        
        private static final String PARAM_A_S_VALUE = "value for 'a' param";
        private static final String PARAM_A_S_SUCIDE_VALUE = "MyRecorder: kill yourself !!";
        private static final short  PARAM_A_I_VALUE = 1;
        private static final String PARAM_B1_S_VALUE = "value for 'b' param (in)";
        private static final short  PARAM_B1_I_VALUE = 1;
        private static final String PARAM_B2_S_VALUE = "value for 'b' param (out)";
        private static final short  PARAM_B2_I_VALUE = 1;
        private static final String PARAM_C_S_VALUE = "value for 'c' param";
        private static final short  PARAM_C_I_VALUE = 1;
        private static final String PARAM_RET_S_VALUE = "value for return";
        private static final short  PARAM_RET_I_VALUE = 1;
        private static final long   REQUEST_MAX_DURATION = 150;



        public boolean send_request_to_server_component(String recorder_name, boolean suicide, String msg) {

            System.out.println("CCMSClientImpl::send_request_to_server_component() at " + System.currentTimeMillis() + " - " + msg);
            try {
                Test test =
                    this.sessionContext.get_connection_test_receptacle();
                
                TestStruct a;
                if(suicide) {
                    a = new TestStruct(PARAM_A_S_SUCIDE_VALUE, PARAM_A_I_VALUE);
                } else {
                    a = new TestStruct(PARAM_A_S_VALUE, PARAM_A_I_VALUE);
                }

                //TestStruct a = new TestStruct(PARAM_A_S_VALUE, PARAM_A_I_VALUE);
                TestStructHolder b = new TestStructHolder(
                    new TestStruct(PARAM_B1_S_VALUE, PARAM_B1_I_VALUE));
                TestStructHolder c = new TestStructHolder();
                
                RecordedOperationEvent sendOpe = new RecordedOperationEventImpl();
                sendOpe.recorder_name = recorder_name;
                sendOpe.operation_name = OPERATION_NAME;
                sendOpe.a_value = a;
                sendOpe.b1_value = b.value;
                
                TestStruct ret = test.test_operation(a, b, c);
                
                assertTrue("Out value of b.s is '" + b.value.s + 
                           "' instead of '" + PARAM_B2_S_VALUE + "'",
                           b.value.s.equals(PARAM_B2_S_VALUE));
                assertTrue("Out value of b.i is '" + b.value.i + 
                           "' instead of '" + PARAM_B2_I_VALUE + "'",
                           b.value.i == PARAM_B2_I_VALUE);
                assertTrue("Out value of c.s is '" + c.value.s + 
                           "' instead of '" + PARAM_C_S_VALUE + "'",
                           c.value.s.equals(PARAM_C_S_VALUE));
                assertTrue("Out value of c.i is '" + c.value.i + 
                           "' instead of '" + PARAM_C_I_VALUE + "'",
                           c.value.i == PARAM_C_I_VALUE);
                assertTrue("Return.s value is '" + ret.s + 
                           "' instead of '" + PARAM_RET_S_VALUE + "'",
                           ret.s.equals(PARAM_RET_S_VALUE));
                assertTrue("Return.s value is '" + ret.i + 
                           "' instead of '" + PARAM_RET_I_VALUE + "'",
                           ret.i == PARAM_RET_I_VALUE);
                           
                sendOpe.b2_value = b.value;
                sendOpe.c_value = c.value;
                sendOpe.ret_value = ret;
                this.ccmClientImpl.sendList.add(sendOpe);

            } catch (org.omg.CORBA.SystemException e) {
                e.printStackTrace();
                fail("Exception calling Server1's facet: " + e);
            }

            return true;
        }


        public boolean send_data_to_generic_recorder(GenericRecorder genRecorder, String recorder_name, String  msg) {

            System.out.println("CCMSClientImpl::send_data_to_generic_recorder() at " + System.currentTimeMillis() + " - " + msg);
            
            cdmw.recording.tools.ApplicationRecorder appRecorder
                = new cdmw.recording.tools.ApplicationRecorder(
                        genRecorder, "ccm_ft_recorder_test", "CCMClientImpl", null);

            // My NamedValue elements
            //System.out.println("CCMServerImpl::test_operation(): Create  'MyContext'...");
            NamedValue[] myContext = new NamedValue[1];
            myContext[0] = new NamedValue();
            myContext[0].name = "MyContext";
            myContext[0].value = org.omg.CORBA.ORB.init().create_any();
            myContext[0].value.insert_string("TestContext1");

            //System.out.println("CCMServerImpl::test_operation(): Create  'MyData'...");
            NamedValue[] myData = new NamedValue[1];
            myData[0] = new NamedValue();
            myData[0].name = "MyData";
            myData[0].value = org.omg.CORBA.ORB.init().create_any();
            myData[0].value.insert_string(msg);

            //System.out.println("Call to 'ApplicationRecorder::Record(myContext, myData) ...");
            appRecorder.Record(myContext, myData);
            
            RecordedOperationEvent sendOpe = new RecordedOperationEventImpl();
            sendOpe.recorder_name = recorder_name;
            sendOpe.operation_name = GENERIC_OPERATION_NAME;
            this.ccmClientImpl.sendList.add(sendOpe);

            return true;
        }


        private RecordedOperationEvent find_recorded_operation(String recorder_name, String operation_name)
        {
            for (RecordedOperationEvent evt : this.ccmClientImpl.recordList)
            {
                if (evt.operation_name.equals(operation_name)) {
                    // 'recorder_name' take in account only for 'test_operation' record
                    if (evt.operation_name.equals(OPERATION_NAME) && ! evt.recorder_name.equals(recorder_name)) {
                        continue;
                    }
                    return evt;
                }
            }
            
            System.out.println(" - expected record not found : " + recorder_name + " / " + operation_name + " !!!");
            return null;
        }


        private boolean check_recorded_operations()
        {
        
//             System.out.println("*** Sent operations"); 
//             for (RecordedOperationEvent sendOpe : this.ccmClientImpl.sendList)
//             {
//                 System.out.println(" - " + sendOpe.recorder_name +  " / " + sendOpe.operation_name);
//             }
// 
//             System.out.println("*** Recorded operations"); 
//             for (RecordedOperationEvent recordOpe : this.ccmClientImpl.recordList)
//             {
//                 System.out.println(" - " + recordOpe.recorder_name +  " / " + recordOpe.operation_name);
//             }

            System.out.println("*** Check recorded operations"); 
            System.out.println(" - Sent operations      : " + this.ccmClientImpl.sendList.size()); 
            System.out.println(" - Recorded operations  : " + this.ccmClientImpl.recordList.size()); 
            
            for (RecordedOperationEvent sendOpe : this.ccmClientImpl.sendList)
            { 
                RecordedOperationEvent recordOpe = find_recorded_operation(sendOpe.recorder_name, sendOpe.operation_name);
                
                assertTrue("Record not found : " + sendOpe.recorder_name + " / " + sendOpe.operation_name,
                           recordOpe != null);
                    
                //System.out.println(" - check recorded operation : " + recordOpe.recorder_name +  " / " + recordOpe.operation_name); 

                if (recordOpe.operation_name.equals(OPERATION_NAME)) {
                    assertTrue("Value of a.s is '" + recordOpe.a_value.s + "' instead of '" + sendOpe.a_value.s + "'",
                               recordOpe.a_value.s.equals(sendOpe.a_value.s));

                    assertTrue("Value of a.i is '" + recordOpe.a_value.i + "' instead of '" + sendOpe.a_value.i + "'",
                               recordOpe.a_value.i == sendOpe.a_value.i);

                    assertTrue("Value of b1.s is '" + recordOpe.b1_value.s + "' instead of '" + sendOpe.b1_value.s + "'",
                               recordOpe.b1_value.s.equals(sendOpe.b1_value.s));

                    assertTrue("Value of b1.i is '" + recordOpe.b1_value.i + "' instead of '" + sendOpe.b1_value.i + "'",
                               recordOpe.b1_value.i == sendOpe.b1_value.i);

                    assertTrue("Value of b2.s is '" + recordOpe.b2_value.s + "' instead of '" + sendOpe.b2_value.s + "'",
                               recordOpe.b2_value.s.equals(sendOpe.b2_value.s));

                    assertTrue("Value of b2.i is '" + recordOpe.b2_value.i + "' instead of '" + sendOpe.b2_value.i + "'",
                               recordOpe.b2_value.i == sendOpe.b2_value.i);

                    assertTrue("Value of c.s is '" + recordOpe.c_value.s + "' instead of '" + sendOpe.c_value.s + "'",
                               recordOpe.c_value.s.equals(sendOpe.c_value.s));

                    assertTrue("Value of c.i is '" + recordOpe.c_value.i + "' instead of '" + sendOpe.c_value.i + "'",
                               recordOpe.c_value.i == sendOpe.c_value.i);

                    assertTrue("Value of ret.s is '" + recordOpe.ret_value.s + "' instead of '" + sendOpe.ret_value.s + "'",
                               recordOpe.ret_value.s.equals(sendOpe.ret_value.s));

                    assertTrue("Value of ret.i is '" + recordOpe.ret_value.i + "' instead of '" + sendOpe.ret_value.i + "'",
                               recordOpe.ret_value.i == sendOpe.ret_value.i);
                }
                
                this.ccmClientImpl.recordList.remove(recordOpe);
            }

            // Check if there is non-expected records
            if (! this.ccmClientImpl.recordList.isEmpty()) {
                System.out.println("Following records were not expected !!! :");
                short i=1;
                for (RecordedOperationEvent recordOpe : this.ccmClientImpl.recordList) {
                    System.out.println("  - " + (i++) + " : " + recordOpe.recorder_name +  " / " + recordOpe.operation_name);
                }
            }
            assertTrue("Some records were not expected !!!", this.ccmClientImpl.recordList.isEmpty());
            
            return true;
        }
        
        private boolean check_duration(long t1, long t2, long max_duration)
        {
            if (t2 < t1) return false;
            long d = t2 - t1;
            if (d > max_duration) return false;

            return true;
        }


        @org.junit.Test
        public void doTests() 
        {
            // wait previous connection requests to be recorded...
            try {
                cdmw.ossupport.OS.sleep(2000);
            } catch (InterruptedException e) {
                // ignore
            }

            System.out.println("");
            System.out.println("*** CCMClientActivateTest.doTests() called!");
            System.out.println("");
            
            // get Recorder
            Recorder recorder = null;        
            boolean passed = false;

            try {
                org.omg.CORBA.Object myObj = this.sessionContext.resolve_service_reference("RecordingServer");                
                assertTrue(myObj != null);
                recorder = com.thalesgroup.CdmwRecording.RecorderHelper.narrow(myObj);
                assertTrue(recorder != null);

            }  catch (org.omg.Components.CCMException e) {
                e.printStackTrace();
                fail("Exception getting recorder connection: " + e);
            }

            org.omg.CORBA.Object obj = null;

            System.out.println("CCMServerImpl::doTests(): Retrieve 'LocalNameService'...");
            try {
                obj = this.sessionContext.resolve_service_reference("LocalNameService");
            }
            catch (org.omg.Components.CCMException e) {
                System.out.println("Can't resolve `LocalNameService'");
                obj = null;
                //throw e;
            }
            if (obj == null) {
                System.out.println("'LocalNameService' is a nil object reference");
                throw new org.omg.CORBA.TRANSIENT(
                           cdmw.orbsupport.ExceptionMinorCodes.TRANSIENTCannotReachRepository,
                           org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }

            System.out.println("CCMServerImpl::doTests(): Retrieve 'NamingContext'...");
            org.omg.CosNaming.NamingContextExt nc = org.omg.CosNaming.NamingContextExtHelper.narrow(obj);
            if (nc == null) {
                System.out.println("'LocalNameService' is not a NamingContextExt object reference");
                throw new org.omg.CORBA.TRANSIENT(
                           cdmw.orbsupport.ExceptionMinorCodes.TRANSIENTCannotReachRepository,
                           org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }

            System.out.println("CCMServerImpl::doTests(): Retrieve 'GenericRecorderDaemon' ...");
            obj = null;
            try {
                obj = nc.resolve_str("GenericRecorderDaemon");
            }
            catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
                System.out.println("Can't resolve 'GenericRecorder'");
                obj = null;
                //throw e;
            }
            catch (org.omg.CosNaming.NamingContextPackage.InvalidName e) {
                System.out.println("Can't resolve 'GenericRecorder'");
                obj = null;
                //throw e;
            }
            catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e) {
                System.out.println("Can't resolve 'GenericRecorder'");
                obj = null;
                //throw e;
            }
            if (obj == null) {
                System.out.println("'GenericRecorder' is a nil object reference");
                throw new org.omg.CORBA.OBJECT_NOT_EXIST(
                           cdmw.orbsupport.ExceptionMinorCodes.OBJECT_NOT_EXIST,
                           org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }

            System.out.println("CCMServerImpl::doTests(): Retrieve 'GenericRecorderDaemon' (narrow) ...");
            GenericRecorder genRecorder = null;
            try {
                genRecorder = GenericRecorderHelper.narrow(obj);
            }
            catch (Exception e) {
                System.out.println("Obj is not a GenericRecorder");
                obj = null;
                //throw e;
            }
            if (genRecorder == null) {
                System.out.println("'GenericRecorder' is not a GenericRecorder object reference");
                throw new org.omg.CORBA.OBJECT_NOT_EXIST(
                           cdmw.orbsupport.ExceptionMinorCodes.OBJECT_NOT_EXIST,
                           org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }

            long t1, t2;
            short i = 0; short iMax = 5;
            short j = 0; short jMax = 10;
            boolean suicide = false;
            short periode = 39;
            String recorder_name = MY_RECORDER_PRIMARY;
            
            for (i = 0 ;  i < iMax ; ++i)
            {
                // Suicide of the primary recorder requested at midle time
                if ( i == iMax/2 + 1 ) {
                    suicide = true;
                    recorder_name = MY_RECORDER_BACKUP;
                } else {
                    suicide = false;
                }
                
                // send request to Server Component,
                // recording deamon should record the request/reply
                // then write the recorded into the recording server group,
                // the recording server (the primary one)     
                // will push an event back the client component
                t1 = System.currentTimeMillis();
                passed = send_request_to_server_component(recorder_name, suicide, "Send request to Server : " + (i+1));
                assertTrue(passed);
                t2 = System.currentTimeMillis();
                passed = check_duration(t1, t2, REQUEST_MAX_DURATION);
                assertTrue(passed);
                try {
                    cdmw.ossupport.OS.sleep(periode);
                } catch (InterruptedException e) {
                    // ignore
                }

                for (j = 0 ;  j < jMax ; ++j)
                {
                    // Send data to Generic Recorder
                    t1 = System.currentTimeMillis();
                    passed = send_data_to_generic_recorder(genRecorder, recorder_name, "Send data to Generic Recorder : " + ((i*jMax)+j+1));
                    assertTrue(passed);
                    t2 = System.currentTimeMillis();
                    passed = check_duration(t1, t2, REQUEST_MAX_DURATION);
                    assertTrue(passed);
                    try {
                        cdmw.ossupport.OS.sleep(periode);
                    } catch (InterruptedException e) {
                        // ignore
                    }
                }
            }


            // waiting for all requests to be recorded
            short k = 0;
            while ( this.ccmClientImpl.nbTestOperation < iMax  ||  this.ccmClientImpl.nbGenericOperation < (iMax*jMax) )
            {
                try {
                    cdmw.ossupport.OS.sleep(2000);
                } catch (InterruptedException e) {
                    // ignore
                }
                
//                 System.out.println("CCMServerImpl::doTests(): ["+ k +"] Number of received 'test_operation' : "
//                                  + this.ccmClientImpl.nbTestOperation + " / " + (iMax));
//                 System.out.println("CCMServerImpl::doTests(): ["+ k +"] Number of received 'record_data'    : "
//                                  + this.ccmClientImpl.nbGenericOperation + " / " + (iMax*jMax));
//             
                if ( k++ > (((iMax*jMax)+iMax)/5) ) break;
            }

            System.out.println("CCMServerImpl::doTests(): Number of received event for 'test_operation' : "
                             + this.ccmClientImpl.nbTestOperation + " / " + (iMax));
            System.out.println("CCMServerImpl::doTests(): Number of received event for 'record_data'    : "
                             + this.ccmClientImpl.nbGenericOperation + " / " + (iMax*jMax));

            // check that all requests are recorded
//             assertTrue("Unexpected number of received event for operations <test_operation> and <record_data> : '"
//                        + this.ccmClientImpl.recordList.size() + "' instead of '" + this.ccmClientImpl.sendList.size() + "'",
//                        this.ccmClientImpl.recordList.size() == this.ccmClientImpl.sendList.size());
            
            try {
                cdmw.ossupport.OS.sleep(2000);
            } catch (InterruptedException e) {
                // ignore
            }
                
            // check content of recorded requests
            passed = check_recorded_operations();
            assertTrue(passed);
                       
        }
        
    }
}

