/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.testccmgenericrecording;

import com.acme.TestCcmRecording.*;

import com.thalesgroup.CdmwRecording.*;
import cdmw.recording.tools.*;

//
// IDL:acme.com/TestCcmRecording/CCM_Server:1.0
//
public class CCMServerImpl extends org.omg.CORBA.LocalObject
    implements Session_CCM_Server 
{
    
    // session context
    private CCM_Server_Context m_sessionContext = null;
    private org.omg.CORBA.ORB m_Orb = null;
     
    
    public CCMServerImpl() 
    {
        this.m_sessionContext = null;
        this.m_Orb = org.omg.CORBA.ORB.init();
    }

    
    public void set_session_context(org.omg.Components.SessionContext ctx) 
    {
        System.out.println("   ... CCMServerImpl::set_session_context called!");
        this.m_sessionContext = CCM_Server_ContextHelper.narrow(ctx);
            
    }
    
    public CCM_Server_Context getContext()
    {
        return this.m_sessionContext;
    }
    
    
    public void ccm_activate() 
    {
        System.out.println("   ... CCMServerImpl::ccm_activate called!");
    }
    
    public void ccm_passivate() 
    {
        System.out.println("   ... CCMServerImpl::ccm_passivate called!");
    }
    
    public void ccm_remove() 
    {
        System.out.println("   ... CCMServerImpl::ccm_remove called!");
    }
    
    public CCM_TestRecording get_test_recording()
    {
        // component itself implements the test_recording facet
        return this;
    }
    
    public void test_operation(String message)
    {
        System.out.println("CCMServerImpl::test_operation(): " + message);

//================================================
//================================================

        org.omg.CORBA.Object obj = null;

        // Test of Recording Generic Recorder
        //System.out.println("CCMServerImpl::test_operation(): Retrieve 'LocalNameService'...");
        try {
            obj = this.m_sessionContext.resolve_service_reference("LocalNameService");
        }
        catch (org.omg.Components.CCMException e) {
            System.out.println("Can't resolve `LocalNameService'");
            obj = null;
            //throw e;
        }
        if (obj == null) {
            System.out.println("'LocalNameService' is a nil object reference");
            throw new org.omg.CORBA.TRANSIENT(
                       cdmw.orbsupport.ExceptionMinorCodes.TRANSIENTCannotReachRepository,
                       org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
    
        //System.out.println("CCMServerImpl::test_operation(): Retrieve 'NamingContext'...");
        org.omg.CosNaming.NamingContextExt nc = null;
//        try {
            nc = org.omg.CosNaming.NamingContextExtHelper.narrow(obj);
//         }
//         catch (Exception e) {
//             System.out.println("Obj is not a NamingContextExt");
//             throw e;
//         }
        if (nc == null) {
            System.out.println("'LocalNameService' is not a NamingContextExt object reference");
            throw new org.omg.CORBA.TRANSIENT(
                       cdmw.orbsupport.ExceptionMinorCodes.TRANSIENTCannotReachRepository,
                       org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        obj = null;
        System.out.println("CCMServerImpl::test_operation(): Retrieve 'GenericRecorderDaemon' ...");
        try {
            obj = nc.resolve_str("GenericRecorderDaemon");
        }
        catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
            System.out.println("Can't resolve 'GenericRecorder'");
            obj = null;
            //throw e;
        }
        catch (org.omg.CosNaming.NamingContextPackage.InvalidName e) {
            System.out.println("Can't resolve 'GenericRecorder'");
            obj = null;
            //throw e;
        }
        catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e) {
            System.out.println("Can't resolve 'GenericRecorder'");
            obj = null;
            //throw e;
        }
        if (obj == null) {
            System.out.println("'GenericRecorder' is a nil object reference");
            throw new org.omg.CORBA.OBJECT_NOT_EXIST(
                       cdmw.orbsupport.ExceptionMinorCodes.OBJECT_NOT_EXIST,
                       org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        //System.out.println("CCMServerImpl::test_operation(): Retrieve 'GenericRecorderDaemon' (narrow) ...");
        GenericRecorder genRecorder = null;
        try {
            genRecorder = GenericRecorderHelper.narrow(obj);
        }
        catch (Exception e) {
            System.out.println("Obj is not a GenericRecorder");
            obj = null;
        }
        if (genRecorder == null) {
            System.out.println("'GenericRecorder' is not a GenericRecorder object reference");
            throw new org.omg.CORBA.OBJECT_NOT_EXIST(
                       cdmw.orbsupport.ExceptionMinorCodes.OBJECT_NOT_EXIST,
                       org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        System.out.println("CCMServerImpl::test_operation(): Create  'ApplicationRecorder'...");
        cdmw.recording.tools.ApplicationRecorder appRecorder
            = new cdmw.recording.tools.ApplicationRecorder(
                    genRecorder, "ccm_generic_recording_test", "CCMServerImpl", null);

        // My NamedValue elements
        //System.out.println("CCMServerImpl::test_operation(): Create  'MyContext'...");
        NamedValue[] myContext = new NamedValue[1];
        myContext[0] = new NamedValue();
        myContext[0].name = "MyContext";
        myContext[0].value = m_Orb.create_any();
        myContext[0].value.insert_string("TestContext1");
        
        //System.out.println("CCMServerImpl::test_operation(): Create  'MyData'...");
        NamedValue[] myData = new NamedValue[1];
        myData[0] = new NamedValue();
        myData[0].name = "MyData";
        myData[0].value = m_Orb.create_any();
        myData[0].value.insert_string("TestData1");
        
        System.out.println("CCMServerImpl::test_operation(): Call to 'ApplicationRecorder::Record(myContext, myData) ...");
        appRecorder.Record(myContext, myData);

        System.out.println("CCMServerImpl::test_operation(): Call to 'ApplicationRecorder::Record(myData) ...");
        appRecorder.Record(myData);

//         System.out.println("CCMServerImpl::test_operation(): Call to 'ApplicationRecorder::Record(null, myData) ...");
//         appRecorder.Record(null, myData);
// 
//         System.out.println("CCMServerImpl::test_operation(): Call 'ApplicationRecorder::Record(myContext, null) ...");
//         appRecorder.Record(myContext, null);
// 
//         System.out.println("CCMServerImpl::test_operation(): Call 'ApplicationRecorder::Record(null, null) ...");
//         appRecorder.Record(null, null);

//================================================
//================================================

    }
        
    public void push_test_event(com.acme.TestCcmRecording.TestEvent evt)
    {
        System.out.println("CCMServerImpl::push_test_event(): " + evt.message);
    }
    
}

