/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ccmlocaleventmanager;

import cdmw.common.Assert;
import cdmw.testutils.Testable;
import cdmw.ossupport.OS;
import cdmw.ossupport.Mutex;
import cdmw.ossupport.Timeval;
import cdmw.ccmlocaleventmanager.EventTestImpl;
import cdmw.ccmlocaleventmanager.EventInfos;
import cdmw.ccmlocaleventmanager.CCMHostInfoImpl;
import cdmw.orbsupport.ExceptionMinorCodes;
import cdmw.platformmngt.PlatformInterface;
import cdmw.testutils.TestManager;
import cdmw.testutils.CORBATestManager;

// JUnit imports
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

//
// IDL:thalesgroup.com/CcmLocalEventManagerTest/CCM_ServerWithLocalManage:1.0
//
public class CCMServerWithLocalManagerImpl extends org.omg.CORBA.LocalObject
    implements com.thalesgroup.UserCcmLocalEventManagerTest.User_CCM_ServerWithLocalManager {
  
    private com.acme.CcmLocalEventManagerTest.CCM_ServerWithLocalManager_Context context;
    
    public ServerActivationThread testThread;

    // True if all events are received by the HostInfo facet 
    public static boolean eventsReceived;
        
    public static boolean ackEvts;

    public static org.omg.CORBA.ORB orb;

    public static String sytemCorbaloc;

    private CCMHostInfoImpl hostInfo;

    private boolean firstToExit;

    public CCMServerWithLocalManagerImpl() {
        hostInfo = new CCMHostInfoImpl ();
        context = null;
        eventsReceived = false;
        firstToExit = false;
        ackEvts = false;
    }

    private static boolean waitEvents () {
        System.out.println("waitEvents");
        int timescale = CORBATestManager.getTimescale();

        while (!eventsReceived) {
            try{
                cdmw.ossupport.OS.sleep(timescale * 5000);
             } catch (java.lang.InterruptedException ex) {
                return false;
            }
        }
        return true;
    }

    public boolean first_to_exit() {
    return firstToExit;
    }

    public void first_to_exit(boolean arg) {
    firstToExit = arg;
    }

    /**
     *  This class provides an object to test the ccm_activate.
     */
    public static class CCMServerWithLocalManagerActivateTest {
        
        private static final String SERVER_REF
            = "Server_object";

        private CCMServerWithLocalManagerImpl server = null;

        public CCMServerWithLocalManagerActivateTest() {
        }

        @Before
        public void init () 
        {
            TestManager testManager = TestManager.instance();
            this.server = 
                (CCMServerWithLocalManagerImpl)testManager.getLocalProperty(SERVER_REF);
        }

        @Test
        public void doTests() {

        System.out.println("Starting of the test");

        assertTrue(this.server != null);
        
        com.acme.CcmLocalEventManagerTest.EventTest evt = new EventTestImpl();
        com.acme.CcmLocalEventManagerTest.SecondEventTest secondEvt = new SecondEventTestImpl();

        int i;
        int eventId;

        // Send the events
        for ( i = 0; i < EventInfos.MAX_NB_EVENTS; i++ ) {
        
        // At the same time, initialization of m_event_infos
        EventInfos.events [ 0 ][ i ] = EventInfos.events [ 1 ][ i ] = false;
        EventInfos.hostname [ 0 ][ i ] = EventInfos.hostname [ 1 ][ i ] = "";
        
        eventId = i + 1; 
        
        if (this.server.first_to_exit())
            evt.numOfEvent = eventId;
        else
            secondEvt.numOfEvent = eventId;

        if (this.server.first_to_exit()){
            System.out.println("Server: Sending of the EventTest #"
                   + eventId);
            this.server.context.push_eventEmits ( evt );
        }
        else {
            System.out.println("Server: Sending of the SecondEventTest #"
                   + eventId);
            this.server.context.push_secondEventEmits ( secondEvt );
        }
        }

        // Before continuing, wait that all the calls to HostInfo facets end up
        boolean statusWait = waitEvents();
        if (!statusWait)  {
            fail("Exception during the wait of all acknowledgments...");
        }

        System.out.println("\nCheck that all events were received by both receiver components...");
        
        boolean checkOk = true ;
        String hostname;
        
        for (int j = 0; j < 2; j++) {
            
            switch (j) {
            case 0:
                System.out.println("=================================================================================");
                break;
            case 1:
                System.out.println("===== Check that the second receiver component did all its acknowledgments =====");
                break;
            }
            
            for ( i = 0; i < EventInfos.MAX_NB_EVENTS; i++ ) {
                
                eventId = i + 1;
                hostname = EventInfos.hostname[j][i];
                
                if ( EventInfos.events [j][i] == true ) {
                    
                    System.out.println( "Server: event #" 
                                        +  eventId 
                                        + " received by "
                                        + hostname );
                    
                }
                else {
                    System.out.println( "Error1 - Server: event #" 
                                        + eventId
                                        + " received by "
                                        + hostname );
                    
                    checkOk = false;
                    break;
                }
            } // end for
            if(!checkOk)
                break;
        }

        if ( checkOk )
        System.out.println("\nAll events sent from " + cdmw.ossupport.OS.getHostname() + " were received by both receiver components");
        else 
        System.out.println("\nAll events sent from " + cdmw.ossupport.OS.getHostname() + " were not received by both receiver components!");
        
        if (this.server.first_to_exit() == true && checkOk == true) {
            System.out.println("\n===================================================");
            System.out.println("Ending test Test ccm_activate on Server");
            System.out.println("===================================================");
            System.out.println("All events were received\n");
            assertTrue(checkOk);
            System.out.println("\nThe Server exits. It's normal");
            System.exit(0);
        } else {
            if (this.server.first_to_exit() == true && checkOk == false) {
                System.out.println("\n===================================================");
                System.out.println("Ending test Test ccm_activate on Server");
                System.out.println("===================================================");
                System.out.println("All events were not received\n");
                assertTrue(checkOk);
                System.out.println("\nThe Server exits. It's normal");
                System.exit(0);
            }
        }

        assertTrue(checkOk);
        
        }
    }

    private class ServerActivationThread extends Thread {
        
        private TestManager testManager;

        ServerActivationThread(TestManager testManager) {
            this.testManager = testManager;
        }

        public void run() {
            this.testManager.runTests();
            this.testManager.printSummary();
        }
    }
    
    public void set_session_context(org.omg.Components.SessionContext ctx) {
        System.out.println("Server: set_session_context called!");
        this.context = 
            com.acme.CcmLocalEventManagerTest.CCM_ServerWithLocalManager_ContextHelper.narrow(ctx);
    }
    
    /**
     * Actives the component (call by the container)
     */
    public void ccm_activate() {
    System.out.println("   ... CCMServerWithLocalManagerImpl.ccm_activate called!");

        try {
            TestManager testManager = TestManager.instance();
            TestManager.activateLocalInstance(testManager);
            
            testManager.init(
                "cdmw.ccmlocaleventmanager.CCMServerWithLocalManagerImpl.CCMServerWithLocalManagerActivateTest");

            testManager.setLocalProperty(
                CCMServerWithLocalManagerActivateTest.SERVER_REF,
                this);

            testManager.addTest(CCMServerWithLocalManagerActivateTest.class);
                        
            this.testThread = new ServerActivationThread(testManager);
            this.testThread.start();
            
        } catch (Exception e) {
            System.err.println("Exception thrown during test: " + e);
        }

    }
    
    /**
     * Makes stop the component (call by the container)
     */
    public void ccm_passivate() {
    System.out.println("   ... CCMServerWithLocalManagerImpl.ccm_passivate called!");   
    }
    
    /**
     * Destroys the component (call by the container)
     */
    public void ccm_remove() {
        System.out.println("Server: ccm_remove called!");
    }

    public com.acme.CcmLocalEventManagerTest.CCM_ServerWithLocalManager_Context get_context() {
        return this.context;
    }

    public com.acme.CcmLocalEventManagerTest.CCM_HostInfo get_serverHostInfo() {
    return hostInfo;
    }
}

