/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccmlogging;

import org.omg.Components.SessionContext;

import com.acme.Hello.*;

import cdmw.logging.*;
import static cdmw.logging.Logger.*;

public class ClientExecutorImpl extends org.omg.CORBA.LocalObject
    implements com.acme.Hello.User_Client
{
        /**
         * IDL:thalesgroup.com/Hello/CCM_Client:1.0
         */
        private Logger logger;
        private CCM_Client_Context sessionContext;
        /**
         * Constructor.
         */
        public ClientExecutorImpl()
        {
                this.sessionContext = null;
            this.logger = Logger.Get_Logger("CCM_Client","QUALIF_CCM_LOGGING");
        }

        /**
         * Set the session context of the component (called by the container).
         *
         * @param SessionContext a session context
         *
         * @exception CORBA SystemException
         */
        public void set_session_context(SessionContext ctx)
            throws org.omg.CORBA.SystemException
        {
                //this.logger.info("ClientExecutor_impl::set_session_context() called.", new Throwable());
                CDMW_INFO(this.logger, "ClientExecutor_impl::set_session_context() called.", new Throwable());

                this.sessionContext = CCM_Client_ContextHelper.narrow(ctx);
        }


        /**
         * Activate the component (called by the container).
         *
         * @exception CORBA SystemException
         */
        public void ccm_activate()
           throws org.omg.CORBA.SystemException
        {
                //this.logger.info("ClientExecutor_impl::ccm_activate() called.",  new Throwable());
                CDMW_INFO(this.logger, "ClientExecutor_impl::ccm_activate() called.",  new Throwable());

            try {
                // get the display connection of the server.
                    CDMW_INFO(this.logger, "connect to the display facet...",  new Throwable());

                Display display = this.sessionContext.get_connection_client_receptacle();

                // call the server display.
                    CDMW_INFO(this.logger, "call the server display...",  new Throwable());

                display.display_hello();
            }
                catch (Exception e) 
                {
                        CDMW_ERROR(this.logger, "unexpected exception raised."+e,  new Throwable());   
            }

        }


        /**
         * Stop the component (called by the container).
         *
         * @exception CORBA SystemException
         */
        public void ccm_passivate()
           throws org.omg.CORBA.SystemException
        {
                CDMW_INFO(this.logger, "ClientExecutor_impl::ccm_passivate() called.",  new Throwable());
        }


        /**
         * Destroy the component (called by the container).
         *
         * @exception CORBA SystemException
         */
        public void ccm_remove()
           throws org.omg.CORBA.SystemException
        {
                CDMW_INFO(this.logger, "ClientExecutor_impl::ccm_remove() called.",  new Throwable());
        }

}
