/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccmlogging;

import org.omg.Components.SessionContext;

import com.acme.Hello.*;

import cdmw.logging.*;
import static cdmw.logging.Logger.*;

import cdmw.testutils.TestManager;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

public class FatalClientExecutorImpl extends org.omg.CORBA.LocalObject
    implements com.acme.Hello.User_Client
{

    // TestThread
    private class TestThread extends Thread {
        public TestManager testManager;

        TestThread(TestManager testManager) {
            this.testManager = testManager;
        }

        public void run() {
            this.testManager.runTests();
        }
    }

    /**
     * IDL:thalesgroup.com/Hello/CCM_Client:1.0
     */
    private Logger logger;
    private CCM_Client_Context sessionContext;
    private TestThread testThread;


    /**
     * Constructor.
     */
    public FatalClientExecutorImpl()
        {
            this.sessionContext = null;
            this.logger = Logger.Get_Logger("CCM_Client","QUALIF_CCM_LOGGING");
        }

    /**
     * Set the session context of the component (called by the container).
     *
     * @param SessionContext a session context
     *
     * @exception CORBA SystemException
     */
    public void set_session_context(SessionContext ctx)
        throws org.omg.CORBA.SystemException
    {
        CDMW_INFO(this.logger,
                  "FatalClientExecutor_impl::set_session_context() called",
                  new Throwable());

        this.sessionContext = CCM_Client_ContextHelper.narrow(ctx);
    }


    /**
     * Activate the component (called by the container).
     *
     * @exception CORBA SystemException
     */
    public void ccm_activate()
        throws org.omg.CORBA.SystemException
    {
        CDMW_INFO(this.logger,
                  "FatalClientExecutor_impl::ccm_activate() called.",
                  new Throwable());

/*        try {
            // get the display connection of the server.
            System.out.println("connect to the display facet");

            Display display =
                this.sessionContext.get_connection_client_receptacle();

            // call the server display.
            CDMW_INFO(this.logger, "call the server display",
                      new Throwable());
            display.display_hello();

            System.out.println("Check that a fatal log makes the supervision kill "
                               + "the calling process");
            //CDMW_FATAL(this.logger, "Fatal error detected!", new Throwable());

            Thread.sleep(3000);
            fail("Process has not been killed");
        }

        catch (Exception e) {
            CDMW_ERROR(this.logger, "unexpected exception raised." + e,
                       new Throwable());   
            e.printStackTrace();
            fail("Unexpected exception raised");
        }

*/        try {
            TestManager testManager = TestManager.instance();
            TestManager.activateLocalInstance(testManager);
            testManager.init("cdmw.ccmlogging.FatalClientExecutorImpl.Tester");

            testManager.setLocalProperty(
                Tester.CCM_CLIENT_IMPL_NAME,
                this);
            testManager.setLocalProperty(
                Tester.SESSION_CONTEXT_NAME,
                this.sessionContext);
            testManager.setLocalProperty(
                Tester.LOGGER_NAME,
                this.logger);
            testManager.addTest(Tester.class);

            this.testThread = new TestThread(testManager);
            this.testThread.start();
        }

        catch (Exception e) {
            System.err.println("Exception thrown during test: " + e);
        }

    }


    /**
     * Stop the component (called by the container).
     *
     * @exception CORBA SystemException
     */
    public void ccm_passivate()
        throws org.omg.CORBA.SystemException
    {
        CDMW_INFO(this.logger,
                  "FatalClientExecutor_impl::ccm_passivate() called.",
                  new Throwable());

        if (this.testThread != null) {
            try {
                this.testThread.join();
            }
            catch (java.lang.InterruptedException e) {
                // ignore
            }
        }
    }


    /**
     * Destroy the component (called by the container).
     *
     * @exception CORBA SystemException
     */
    public void ccm_remove()
        throws org.omg.CORBA.SystemException
    {
        CDMW_INFO(this.logger,
                  "FatalClientExecutor_impl::ccm_remove() called.",
                  new Throwable());
    }


    //
    // Tester class
    //
    public static class Tester
    {
            
        private static final String CCM_CLIENT_IMPL_NAME
            = "CCMClientImpl_object";

        private static final String SESSION_CONTEXT_NAME
            = "CCM_Client_Context_object";

        private static final String LOGGER_NAME
            = "Logger_object";

        private FatalClientExecutorImpl ccmClientImpl;
        private CCM_Client_Context sessionContext;
        private Logger logger;

        public Tester()
        {
        }

        
        @Before
        public void init () 
        {
            TestManager testManager = TestManager.instance();

            this.ccmClientImpl = (FatalClientExecutorImpl) 
                testManager.getLocalProperty(CCM_CLIENT_IMPL_NAME);
            assertTrue(this.ccmClientImpl != null);

            this.sessionContext = (CCM_Client_Context)
                testManager.getLocalProperty(SESSION_CONTEXT_NAME);
            assertTrue(this.sessionContext != null);

            this.logger = (Logger)
                testManager.getLocalProperty(LOGGER_NAME);
            assertTrue(this.logger != null);
        }


        @Test
        public void doTests() 
        {
            try {
                // get the display connection of the server.
                System.out.println("connect to the display facet");

                Display display =
                    this.sessionContext.get_connection_client_receptacle();

                // call the server display.
                CDMW_INFO(this.logger, "call the server display",
                          new Throwable());
                display.display_hello();

                System.out.println("Check that a fatal log makes the supervision kill "
                                   + "the calling process");
                CDMW_FATAL(this.logger, "Fatal error detected!", new Throwable());

                // print test summary
                TestManager.instance().printSummary();
                Thread.sleep(3000);
                fail("Process has not been killed");

            }

            catch (Exception e) {
                CDMW_ERROR(this.logger, "unexpected exception raised." + e,
                           new Throwable());   
                e.printStackTrace();
                fail("Unexpected exception raised");
            }
        }
    }
}
