/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.qualif.ccmperfinvocations;

import org.omg.Components.SessionContext;

import cdmw.testutils.CORBATestManager;
import cdmw.testutils.threadtestengine.ManualControlledThreadedTestProcessor;

public class CCMClientImpl extends org.omg.CORBA.LocalObject implements
    com.acme.TestPerf.User_Client {

    private static int nbInvocation = 1000;

    private com.acme.TestPerf.CCM_Client_Context sessionContext;

    //
    // set_session_context function : it fixes the session context of the
    // component (call by the container)
    //
    public void set_session_context(SessionContext ctx) {
        System.out.println("Set_session_context called!");

        this.sessionContext = com.acme.TestPerf.CCM_Client_ContextHelper
            .narrow(ctx);
    }

    //
    // ccm_activate function : it actives the component (call by the container)
    //
    public void ccm_activate() {

        CORBATestManager.instance().init("test ccm_perf_invoc_comp_client");

        ManualControlledThreadedTestProcessor manThreadedTP = new ManualControlledThreadedTestProcessor(
            CORBATestManager.instance().getRunningListener());

    CORBATestManager.instance().setLocalProperty(
           "client_context",
           this.sessionContext);

        manThreadedTP.addTest(TestCCMClientActivate.class);
        manThreadedTP.runTests();
        manThreadedTP.waitForCompletion();
        CORBATestManager.instance().printSummary();

        try {
            Thread.currentThread().wait(5000);
        }
        catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    //
    // ccm_passivate function : it makes stop the component (call by the
    // container)
    //
    public void ccm_passivate() {
        System.out.println("Ccm_passivate called!");
    }

    //
    // ccm_remove function : it destroys the component (call by the container)
    public void ccm_remove() {
        System.out.println("Ccm_remove called!");
    }

    public static int get_nb_invocation() {
        return nbInvocation;
    }

    public static void set_nb_invocation(int nb_Invocation) {
        nbInvocation = nb_Invocation;
    }

    public com.acme.TestPerf.CCM_Client_Context getContext() {
        return this.sessionContext;
    }
}
