/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ccmreceptaclewithoutdefaulttimeout;

import org.junit.Assert;
import org.junit.Test;
import org.omg.CORBA.SystemException;

import cdmw.testutils.CORBATestManager;
import cdmw.testutils.CORBATestManager.InvalidObject;

public class CCMClientImpl extends org.omg.CORBA.LocalObject implements
    com.thalesgroup.UserTestWithoutDefaultTimeout.User_CCM_Client {

    private com.acme.TestWithoutDefaultTimeout.CCM_Client_Context sessionContext;

    public static class CCMClientActivateTest {

        public CCMClientActivateTest ()
    {
    }

        private CCMClientImpl client;

        @Test
        public void doTests() {
            
            System.out.println("********************************************");
            System.out.println("            ccm_activate start");
            System.out.println("********************************************");
            
            com.acme.TestWithoutDefaultTimeout.TimeoutTester serverFacet = null;

        com.acme.TestWithoutDefaultTimeout.CCM_Client_Context clientContext = 
        (com.acme.TestWithoutDefaultTimeout.CCM_Client_Context) CORBATestManager.getLocalInstance()
        .getLocalProperty("client_context");

            Assert.assertNotNull(clientContext);
            System.out.println("Get_connection_without_default_receptacle...");

            boolean testOk = false;
            try {
                serverFacet = clientContext.get_connection_without_default_receptacle();

                if (serverFacet != null) {
                    testOk = true;
                }
            }
            catch (Exception e) {
                System.out.println("Unexpected exception raised!");
            }
            
            Assert.assertTrue(testOk);
            
            System.out.println("Connect to the TimeoutTester facet without the default receptacle...");
            
            testOk = false;
            try {
                if (serverFacet != null)
                {
                    serverFacet.sleep(10);
                    System.out.println("sleep operation finished");
                    testOk = true;
                }
            }
            catch(org.omg.CORBA.TIMEOUT e) {
                System.out.println("CORBA.TIMEOUT: exception raised. Unexpected exception raised!");
            }
            catch (Exception e) {
                System.out.println("Unexpected exception raised!");
            }
            
            Assert.assertTrue(testOk);
            
            /*
             * Test of the specific receptacle
             */

            System.out.println("Get_connection_specifc_receptacle...");
            testOk = false;
            try {
                serverFacet = clientContext.get_connection_specific_receptacle();

                if (serverFacet != null) {
                    testOk = true;
                }
             }
             catch (Exception e) {
                 System.out.println("Unexpected exception raised!");
             }
             
             Assert.assertTrue(testOk);
             
//           get the connection to the server 
             System.out.println("Connect to the TimeoutTester facet with the specific receptacle...");
             testOk = false;
             try {
                 if (serverFacet != null) {
                     serverFacet.sleep(10*1000);
                   }
               }
             catch(org.omg.CORBA.TIMEOUT e) {
                 System.out.println("CORBA.TIMEOUT: Expected exception raised!");
                 testOk = true;
             }
             catch (Exception e) {
                 System.out.println("Unexpected exception raised!");
             }
             Assert.assertTrue(testOk);
             
             /*
              * Test of the specific forward request receptacle
              */
             
             System.out.println("get_connection_specific_forward_request_receptacle...");
             testOk = false;
             try {
                 serverFacet = clientContext.get_connection_specific_forward_request_receptacle();
                 
                 if (serverFacet != null) {
                     testOk = true;
                 }
             }
             catch (Exception e) {
                 System.out.println("Unexpected exception raised!");
             }
             
             Assert.assertTrue(testOk);
             
//           get the connection to the server 
             System.out.println("Connect to the TimeoutTester facet with the specific forward request receptacle...");
             testOk = false;
             try {
                 if (serverFacet != null) {
                     serverFacet.sleep(10*1000);
                 }
             }
             catch(org.omg.CORBA.TIMEOUT e) {
                 System.out.println("CORBA.TIMEOUT: Expected exception raised!");
                 testOk = true;
             }
             catch (Exception e) {
                 System.out.println("Unexpected exception raised!");
             }
             Assert.assertTrue(testOk);

             System.out.println("********************************************");
             System.out.println("            ccm_activate stop");
             System.out.println("********************************************");
             
           
        }
    }
    
    public CCMClientImpl() {
        this.sessionContext = null;
    }

//
//  set_session_context function : it fixes the session context of the component (call by the container)
//    
    public void set_session_context(org.omg.Components.SessionContext ctx) {
        System.out.println("setSessionContext called!");
        this.sessionContext = 
            com.acme.TestWithoutDefaultTimeout.CCM_Client_ContextHelper.narrow(ctx);
    }
    
//
//  ccm_activate function : it actives the component (call by the container)
//
    public void ccm_activate() throws SystemException
    {
    CORBATestManager testManager = new CORBATestManager();
    CORBATestManager.activateLocalInstance(testManager);
    testManager.init("cdmw.ccmreceptacletimeout.CCMClientImpl", null,
                     null);
    testManager.setLocalProperty(
           "client_context",
           this.getContext());
    testManager.addTest(CCMClientActivateTest.class);
    testManager.runTests();
    testManager.printSummary();


        try {
            Thread.sleep(5*1000);
        }
        catch(InterruptedException e) {
            e.printStackTrace();
        }     
    }

//
//  ccm_passivate function : it makes stop the component (call by the container)
//
    public void ccm_passivate() {
        System.out.println("ccm_passivate called!");
    }

//
//  ccm_remove function : it destroys the component (call by the container)
//
    public void ccm_remove() {
        System.out.println("ccm_remove called!");
    }
    
    public com.acme.TestWithoutDefaultTimeout.CCM_Client_Context getContext() {
        return this.sessionContext;
    }
}
