/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.testccmrecording;

import com.acme.TestCcmRecording.*;
import com.thalesgroup.CdmwRecording.*;

import cdmw.testutils.TestManager;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;


//
// IDL:acme.com/TestCcmRecording/CCM_Client:1.0
//
public class CCMClientImpl extends org.omg.CORBA.LocalObject
    implements Session_CCM_Client 
{
    // session context
    private CCM_Client_Context sessionContext;
     
    // TestThread
    private class TestThread extends Thread {
        public TestManager testManager;
        
        TestThread(TestManager testManager) {
            this.testManager = testManager;
        }
        
        public void run() {
            this.testManager.runTests();
            this.testManager.printSummary();
        }
    }
    private TestThread testThread;
    
    // Received events queue    
    java.util.List<RequestReplyPair> recordList;
    
    boolean myRecordHasBeenCalled;

    // True if configuration_complete has been received by the recorder otherwise
    // false. 
    // configuration_complete is not recorded into recordList because
    // recordedList is cleaned at the beginning of doTests(). A boolean is
    // enough to know if configuration_complete has been recorded.
    boolean configurationCompleteHasBeenRecorded;
    
    
    public CCMClientImpl() 
    {
        this.sessionContext = null;
        this.testThread = null;
        this.recordList = new java.util.Vector<RequestReplyPair>();
        this.myRecordHasBeenCalled = false;
        this.configurationCompleteHasBeenRecorded = false;
    }

    
    public void set_session_context(org.omg.Components.SessionContext ctx) 
    {
        System.out.println("   ... CCMClientImpl.set_session_context called!");
        this.sessionContext = CCM_Client_ContextHelper.narrow(ctx);
    }
    
    public CCM_Client_Context getContext()
    {
        return this.sessionContext;
    }
    
    
    public void ccm_activate() 
    {
        System.out.println("   ... CCMClientImpl.ccm_activate called!");

        try {
            TestManager testManager = TestManager.instance();
            TestManager.activateLocalInstance(testManager);
            
            testManager.init(
                "cdmw.testccmrecording.CCMClientImpl.CCMClientActivateTest");
            testManager.setLocalProperty(
                CCMClientActivateTest.CCM_CLIENT_IMPL_NAME,
                this);
            testManager.setLocalProperty(
                CCMClientActivateTest.SESSION_CONTEXT_NAME,
                this.sessionContext);
            testManager.addTest(CCMClientActivateTest.class);
                        
            this.testThread = new TestThread(testManager);
            this.testThread.start();
            
        } catch (Exception e) {
            System.err.println("Exception thrown during test: " + e);
        }
    }
    
    public void ccm_passivate() 
    {
        System.out.println("   ... CCMClientImpl.ccm_passivate called!");
        if (this.testThread != null) {
            try {
                this.testThread.join();
            } catch (java.lang.InterruptedException e) {
                // ignore
            }
        }
    }
    
    public void ccm_remove() 
    {
        System.out.println("   ... CCMClientImpl.ccm_remove called!");
    }
    
        
    //
    // CdmwRecorindg::Recorder::record() implementation
    //
    public synchronized void record(
        com.thalesgroup.CdmwRecording.GroupHeader header, 
        byte[] data)
    {
        System.out.println("" + header.num_pairs + " new Records received from "
                           + header.hostname + " at " + header.base_time);
                           
        // Use org.jacorb.orb.CDRInputStream for unmarshalling
        boolean endian = (header.flags!=0 ? true : false);
        org.omg.CORBA.portable.InputStream input =
            new org.jacorb.orb.CDRInputStream(org.omg.CORBA.ORB.init(), 
                                              data, 
                                              endian);
        
        // Unmarshall while InputStream is not empty
        int i=0;
        try {
            while (input.available() > 0) {
                
                // number of octets still to read
                int left = input.available();
            
                // Unmarshall a RequestReplyPair
                try {
                    RequestReplyPair pair = RequestReplyPairHelper.read(input);
                    
                    System.out.println("  - " + (++i) + ": \"" +
                                       pair.request.operation +
                                       "\" from " +
                                       pair.request.server_id+
                                       "\" from interface " +
                                       pair.request.interface_rep_id);
            
                    if (pair.request.operation.compareTo("configuration_complete") == 0) 
                        {
                            configurationCompleteHasBeenRecorded = true;
                        } 

                    // save RequestReplyPair in list
                    // ignore _is_a requests
                    if (!pair.request.operation.equals("_is_a")) {
                        recordList.add(pair);
                    }
            
                } catch (org.omg.CORBA.MARSHAL e) {
                    // number of read octets
                    int nb_read = left-input.available();
                    System.out.println("UNMARSHALL ERROR (only " + nb_read +
                                       " octets read): " + e);
                    e.printStackTrace();
                    // It's useless to try to continue unmarshalling... break loop
                    break;
                }
            }
        } catch (java.io.IOException e) {
            System.out.println("UNMARSHALL ERROR : " + e);
            e.printStackTrace();
        }
        
    }
    
    
    //
    // MyRecorder::my_record() implementation
    //
    public synchronized void my_record(String record)
    {
        System.out.println("   ... CCMClientImpl.my_record called: " + record);
        this.myRecordHasBeenCalled = true;
    }
    
    
    //
    // CCMClientActivateTest class
    //
    public static class CCMClientActivateTest 
    {
        private static final String CCM_CLIENT_IMPL_NAME
            = "CCMClientImpl_object";

        private static final String SESSION_CONTEXT_NAME
            = "CCM_Client_Context_object";
            
        private CCMClientImpl ccmClientImpl;
        private CCM_Client_Context sessionContext;
        private org.omg.CORBA.ORB orb;
        
        public CCMClientActivateTest()
        {
            this.ccmClientImpl = null;
            this.sessionContext = null;
            this.orb = null;
        }
        
        
        @Before
        public void init () 
        {
            TestManager testManager = TestManager.instance();

            this.ccmClientImpl = 
                (CCMClientImpl)testManager.getLocalProperty(CCM_CLIENT_IMPL_NAME);
            assertTrue(this.ccmClientImpl != null);
            this.sessionContext = 
                (CCM_Client_Context)testManager.getLocalProperty(SESSION_CONTEXT_NAME);
            assertTrue(this.sessionContext != null);
        }
        
        
        @Test
        public void doTests() 
        {
            System.out.println("   ... CCMClientActivateTest.doTests called!");
            
            // wait previous connection requests to be recorded...
            try {
System.out.println("   ... Waiting 10 seconds --to be sure that the deployment is over before starting our test");
                cdmw.ossupport.OS.sleep(10000);
            } catch (InterruptedException e) {
                // ignore
            }
            
            // clear recordList content (previous records about connections)
            ccmClientImpl.recordList.clear();
            
            System.out.println("Send request to Server1");
            try {
                TestRecording testRecording =
                    this.sessionContext.get_connection_test_server1();
                    
                testRecording.test_operation("MESSAGE TO SERVER1");
                
                // Dirty trick to get ORB (required for Valuetypes unmarshalling)
                this.orb = ((org.omg.CORBA.portable.ObjectImpl) testRecording)._orb();

            } catch (org.omg.CORBA.SystemException e) {
                fail("Exception calling Server1's facet: " + e);
            }
            
            
            System.out.println("Send request to Server2");
            try {
                TestRecording testRecording =
                    this.sessionContext.get_connection_test_server2();
                    
                testRecording.test_operation("MESSAGE TO SERVER2");
                
            } catch (org.omg.CORBA.SystemException e) {
                fail("Exception calling Server2's facet: " +e); 
            }
            
            
            System.out.println("Send request to Server3");
            try {
                TestRecording testRecording =
                    this.sessionContext.get_connection_test_server3();
                    
                testRecording.test_operation("MESSAGE TO SERVER3");
                
            } catch (org.omg.CORBA.SystemException e) {
                fail("Exception calling Server3's facet: " +e); 
            }
            
            
            System.out.println("Publish Event");
            try {
                TestEvent ev = new TestEventImpl("EVENT WITH MESSAGE");
        
                this.sessionContext.push_test_event (ev);
                
            } catch (org.omg.CORBA.SystemException e) {
                fail("Exception calling Server3's facet: " +e); 
            }
            
            // wait requests to be recorded
            try {
                cdmw.ossupport.OS.sleep(2000);
            } catch (InterruptedException e) {
                // ignore
            }
            
            System.out.println("Check received record:");
            
            System.out.println("A total of " + ccmClientImpl.recordList.size() + 
                               " records were received:");
            int i=1;
            for (RequestReplyPair pair : ccmClientImpl.recordList) {
                System.out.println("  - " + (i++) + ": \"" +
                                   pair.request.operation + "\" from " +
                                   pair.request.server_id);
            }
            
            // 4 records expected:
            
            //  - "test_operation" with "MESSAGE TO SERVER1"
            assertTrue(
                "Message \"test_operation\" with parameter \"MESSAGE TO SERVER1\" was not recorded",
                find_test_operation_record("Server1", "MESSAGE TO SERVER1"));
                                   
            //  - "test_operation" with "MESSAGE TO SERVER3"
            assertTrue(
                "Message \"test_operation\" with parameter \"MESSAGE TO SERVER3\" was not recorded",
                find_test_operation_record("Server3", "MESSAGE TO SERVER3"));
                                   
            //  - "push_event" with "EVENT WITH MESSAGE"
            assertTrue(
                "Message \"test_operation\" with parameter \"EVENT WITH MESSAGE\" was not recorded",
                find_push_event_record("Server1", "EVENT WITH MESSAGE"));
                                   
            //  - "push_event" with "EVENT WITH MESSAGE"
            assertTrue(
                "Message \"test_operation\" with parameter \"EVENT WITH MESSAGE\" was not recorded",
                find_push_event_record("Server3", "EVENT WITH MESSAGE"));

            // Check if there is non-expected records
            if (! ccmClientImpl.recordList.isEmpty()) {
                System.out.println("Following records were not expected:");
                i=1;
                for (RequestReplyPair pair : ccmClientImpl.recordList) {
                    System.out.println("  - " + (i++) + ": \"" +
                                       pair.request.operation + "\" from " +
                                       pair.request.server_id);
                }
                fail("Some records were not expected");
            }

        
            // Test registration of RecordingServer object as service
            System.out.println("Testing retrieval of RecordingServer object");
            org.omg.CORBA.Object recorderObj = null;
            try {
                // get RecordingServer object
                recorderObj = 
                    this.sessionContext.resolve_service_reference("RecordingServer");
                    
                assertTrue("RecordingServer is null!", recorderObj != null);
                
            } catch (org.omg.Components.CCMException e) {
                fail("RecordingServer no found !!");
            }
            
            // check that RecordingServer object is this component
            // (narrow to TestCcmRecording::MyRecorder and call my_recorder
            try {
                MyRecorder recorder = 
                    MyRecorderHelper.narrow(recorderObj);
    
                assertTrue(
                    "RecordingServer is not a TestCcmRecording::MyRecorder", 
                    recorder != null);
    
                recorder.my_record("THIS IS MY SPECIFIC RECORD");
                
                assertTrue(
                    "TestCcmRecording::MyRecorder::my_record was not called",
                    ccmClientImpl.myRecordHasBeenCalled);

                System.out.println("Testing filtering of configuration_complete...");

                assertTrue(
                    "\"configuration_complete\" was recorded. It is unexpected, the filtering doesn't run successfully",
                    ccmClientImpl.configurationCompleteHasBeenRecorded == false);
                    
                System.out.println("Testing filtering of configuration_complete: successful - no configuration_complete recorded");

            } catch (org.omg.CORBA.SystemException e) {
                e.printStackTrace();
                fail("Exception calling MyRecorder::my_record(): " + e);
            }

        }
        
        
        private boolean find_test_operation_record(String componentName,
                                                   String message)
        {
            for (RequestReplyPair pair : ccmClientImpl.recordList)
            {
                // test is record is for a "test_operation"
                if (pair.request.operation.equals("test_operation")) {
                    
                    // check if server_id ends with componentName
                    if (pair.request.server_id.endsWith(componentName)) {
                        
                        // try to unmarshall parameters as a string
                        boolean endian = (pair.request.head.flags!=0 ? true : false);
                        org.omg.CORBA.portable.InputStream input =
                            new org.jacorb.orb.CDRInputStream(this.orb, 
                                                              pair.request.data, 
                                                              endian);
                        try {
                            String str = input.read_string();
                            
                            // unmarshall OK. Test if it's expected message
                            if (message.equals(str)) {
                                // message found, remove it from list
                                ccmClientImpl.recordList.remove(pair);
                                return true;
                            }
                        } catch (org.omg.CORBA.MARSHAL e) {
                            System.out.println( 
                                "record for \"test_operation\" found, " +
                                "but parameter is not a string!");
                        }
                    }
                    else
                    {
                        System.out.println(
                            "record for \"test_operation\" found, but server_id \"" +
                            pair.request.server_id + "\" doesn't end with \"" + 
                            componentName + "\"");
                    }
                }
            }
            
            return false;
        }
        
        private boolean find_push_event_record(String componentName,
                                               String message)
        {
            for (RequestReplyPair pair : ccmClientImpl.recordList)
            {
                // test is record is for a "push_event"
                if (pair.request.operation.equals("push_event")) {
        
                    // check if server_id ends with componentName
                    if (pair.request.server_id.endsWith(componentName)) {
                        
                        // try to unmarshall parameters as a TestEvent
                        boolean endian = (pair.request.head.flags!=0 ? true : false);
                        org.omg.CORBA.portable.InputStream input =
                            new org.jacorb.orb.CDRInputStream(this.orb, 
                                                              pair.request.data, 
                                                              endian);
                        try {
                            TestEvent evt = TestEventHelper.read(input);
                            
                            // unmarshall OK. Test if it's expected message
                            if (message.equals(evt.message)) {
                                // message found, remove it from list
                                ccmClientImpl.recordList.remove(pair);
                                return true;
                            }
                        } catch (org.omg.CORBA.MARSHAL e) {
                            System.out.println( 
                                "record for \"push_event\" found, " +
                                "but parameter is not a TestEvent!");
                        }
                    }
                    else
                    {
                        System.out.println(
                            "record for \"push_event\" found, but server_id \"" +
                            pair.request.server_id + "\" doesn't end with \"" + 
                            componentName + "\"");
                    }
                }
            }
            
            return false;
        }
        
    }
    
}

