/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccmtime;

import org.omg.CORBA.*;
import org.omg.CORBA.ContainerPackage.Description;
import org.omg.CORBA.InterfaceDefPackage.FullInterfaceDescription;

import com.acme.CcmTime.*;
import com.acme.CcmTime.Display.*;

import cdmw.logging.Logger;
import static cdmw.logging.Logger.*;

public class DisplayFacetExecutorImpl extends LocalObject 
    implements CCM_Display
{
    private Logger logger;
    private ServerExecutorImpl server;

    /**
     * Create a DisplayFacetExecutor.
     *
     */
    public DisplayFacetExecutorImpl(ServerExecutorImpl server)
    {
        this.logger = Logger.Get_Logger("CCM_Display","QUALIF_CCM_TIME");
        this.server = server;
    }
        
    /**
     * Display a hello message.
     */
    public void display_hello(timingDataHolder outputData)
    {
        CDMW_INFO(this.logger, "DisplayFacetExecutor_impl::display_hello() called.",
                         new Throwable());

        // get process name 
        String Process = null;
        try {
            Process =  cdmw.platformmngt.PlatformInterface.getProcessName();
        }
        catch (cdmw.common.BadOrderException e) {
            cdmw.logging.Logger.CDMW_ERROR
                (logger, "Unexpected exception raised!" + e,
                 new Throwable());
            e.printStackTrace(System.err);
        }

        // inform supervision
        try {
            cdmw.platformmngt.PlatformInterface.notifyMessage
                (com.thalesgroup.CdmwPlatformMngtBase.INF.value,
                 Process, "!!!! in display_hello !!!!");
        }
        catch (cdmw.common.BadOrderException e) {
            cdmw.logging.Logger.CDMW_ERROR
                (logger, "Unexpected exception raised!" + e,
                 new Throwable());
            e.printStackTrace(System.err);
        }
        catch (cdmw.common.BadParameterException e) {
            cdmw.logging.Logger.CDMW_ERROR
                (logger, "Unexpected exception raised!" + e,
                 new Throwable());
            e.printStackTrace(System.err);
        }

        // get date
        /************/
        // ask serveur the ClockCatalog
        org.omg.CosClockService.ClockCatalog clockCatalog = 
            this.server.get_clockCatalog();
        // get catalag entry named "LocalExecutor"
        org.omg.CosClockService.ClockCatalogPackage.ClockEntry clockEntry = null;
        try {
            clockEntry = clockCatalog.get_entry("LocalExecutor");
        }
        catch (org.omg.CosClockService.ClockCatalogPackage.UnknownEntry e) {
            cdmw.logging.Logger.CDMW_ERROR
                (logger, "Unexpected exception raised!" + e,
                 new Throwable());
            e.printStackTrace(System.err);
        }
        catch (Exception ee) {}
        // get associated clock
        org.omg.CosClockService.Clock clock = clockEntry.subject;

        // ask it date
        long now = 0;
        try {
            now = clock.current_time();
        }
        catch (org.omg.CosClockService.TimeUnavailable e) {
            cdmw.logging.Logger.CDMW_ERROR
                (logger, "Unexpected exception raised!" + e,
                 new Throwable());
            e.printStackTrace(System.err);
        }
        catch (Exception ee) {}

        System.out.println("\n<------------------------------>");
        System.out.println("<### Hello! (time returned) ###>");
        System.out.println("<------------------------------>");

        // fill output data
        outputData.value = new timingData();
        outputData.value.server = Process;
        outputData.value.time = now;
        outputData.value.operational_time = now; 
    }
}
