/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ccmtime;

import org.omg.CORBA.SystemException;

import cdmw.logging.Logger;
import static cdmw.logging.Logger.*;
import com.acme.CcmTime.*;
import org.omg.CosClockService.ClockCatalog;
import org.omg.Components.SessionContext;
import com.thalesgroup.CdmwComponents.SessionContextExt;
import com.thalesgroup.CdmwComponents.CCMContextExtPackage.ServiceUnavailable;
import com.thalesgroup.CdmwComponents.SessionContextExtHelper;
import org.omg.CosClockService.ClockCatalogHelper;


public class ServerExecutorImpl extends org.omg.CORBA.LocalObject
    implements User_Server
{
    private CCM_Display display;
    private Logger logger;
    private ClockCatalog clockCatalog;
    private CCM_Server_Context sessionContext;
    private SessionContextExt sessionContextExt;

    /**
     * Create a ServerExecutorImpl.
     *
     */
    public ServerExecutorImpl()
    {
        this.logger = Logger.Get_Logger("CCM_Server",
                                        "QUALIF_CCM_TIME");

        
        this.display = new DisplayFacetExecutorImpl(this);
    }

    /**
     * Get a server facet
     */
    public CCM_Display get_server_facet()
    {
        CDMW_INFO(this.logger, "ServerExecutor_impl::get_server_facet() called.",
                         new Throwable());

        return this.display;
    }

    // get access to time service
    ClockCatalog get_clockCatalog()
    {
        CDMW_INFO(this.logger, "ServerExecutor_impl::get_clockCatalog() called.",
                         new Throwable());

        return this.clockCatalog;
    }

    public boolean do_work(org.omg.CORBA.Any args)
    {
        return true;
    }

    /**
     * Set the session context of the component (called by the container).
     *
     * @param SessionContext a session context
     *
     * @exception CORBA SystemException
     */
    public void set_session_context(SessionContext ctx)
        throws org.omg.CORBA.SystemException
    {
        CDMW_INFO(this.logger, "ServerExecutor_impl::set_sessionContext() called.",
                         new Throwable());

        this.sessionContext = CCM_Server_ContextHelper.narrow(ctx);

        // get context for services
        this.sessionContextExt = 
            SessionContextExtHelper.narrow(this.sessionContext);

        // get Time Service
        org.omg.CORBA.Object obj = null;
        try {
            obj = sessionContextExt.resolve_service_reference("ClockService");
        }
//         catch (ServiceUnavailable e) {
//             CDMW_ERROR(this.logger, "Unexpected exception raised while getting service! " + e,
//                               new Throwable());
//         }
        catch (org.omg.Components.CCMException e) {
            CDMW_ERROR(this.logger, "Unexpected exception raised while getting service! " + e,
                              new Throwable());
        }
        try {
            this.clockCatalog = ClockCatalogHelper.narrow(obj);
        }
        catch (java.lang.Exception ee) {
            CDMW_ERROR(this.logger, "Fucking unexpected exception raised while getting service! " + ee,
                              new Throwable());
        }
    }

    /**
     * Activate the component (called by the container).
     *
     * @exception CORBA SystemException
     */
    public void ccm_activate()
        throws org.omg.CORBA.SystemException
    {
        CDMW_INFO(this.logger, "ServerExecutor_impl::ccm_activate() called.",
                         new Throwable());
    }

    /**
     * Stop the component (called by the container).
     *
     * @exception CORBA SystemException
     */
    public void ccm_passivate()
        throws org.omg.CORBA.SystemException
    {
        CDMW_INFO(this.logger, "ServerExecutor_impl::ccm_passivate() called.",
                         new Throwable());
    }

    /**
     * Destroy the component (called by the container).
     *
     * @exception CORBA SystemException
     */
    public void ccm_remove()
        throws org.omg.CORBA.SystemException
    {
        CDMW_INFO(this.logger, "ServerExecutor_impl::ccm_remove() called.",
                         new Throwable());
    }

}
