/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccmwithfederatedclocks;

import org.omg.CORBA.*;
import org.omg.CORBA.ContainerPackage.Description;
import org.omg.CORBA.InterfaceDefPackage.FullInterfaceDescription;

import com.acme.CcmWithFederatedClocks.*;
import com.acme.CcmWithFederatedClocks.Display.*;

import cdmw.logging.Logger;
import static cdmw.logging.Logger.*;

public class DisplayFacetExecutorImpl extends LocalObject 
    implements CCM_Display
{
    private Logger logger;
    private ServerExecutorImpl server;
    private org.omg.CosClockService.ControlledClock ctrlClock;
    private org.omg.CosClockService.ControlledClock federatedControlledClock;
    private org.omg.CosClockService.PeriodicExecution.Controller controller;
    /**
     * Create a DisplayFacetExecutor.
     *
     */
    public DisplayFacetExecutorImpl(ServerExecutorImpl server)
    {
        this.logger = Logger.Get_Logger("CCM_Display","QUALIF_CCM_WITH_FEDERATED_CLOCKS");
        this.server = server;
    }
        
        
    /**
     * Display a hello message.
     */
    public void display_hello( )
    {
        
    this.initClockReference();
        CDMW_INFO(this.logger, "DisplayFacetExecutor_impl::display_hello() called.",
                         new Throwable());

                         
        // get process name 
        String Process = null;
        try {
            Process =  cdmw.platformmngt.PlatformInterface.getProcessName();
        }
        catch (cdmw.common.BadOrderException e) {
            cdmw.logging.Logger.CDMW_ERROR
                (logger, "Unexpected exception raised!" + e,
                 new Throwable());
            e.printStackTrace(System.err);
        }

        // inform supervision
        try {
            cdmw.platformmngt.PlatformInterface.notifyMessage
                (com.thalesgroup.CdmwPlatformMngtBase.INF.value,
                 Process, "!!!! in display_hello !!!!");
        }
        catch (cdmw.common.BadOrderException e) {
            cdmw.logging.Logger.CDMW_ERROR
                (logger, "Unexpected exception raised!" + e,
                 new Throwable());
            e.printStackTrace(System.err);
        }
        catch (cdmw.common.BadParameterException e) {
            cdmw.logging.Logger.CDMW_ERROR
                (logger, "Unexpected exception raised!" + e,
                 new Throwable());
            e.printStackTrace(System.err);
        }


        // ask it date
        long currenTimeFromFederatedControlledClock = 0;
        try {
                currenTimeFromFederatedControlledClock = federatedControlledClock.current_time();
        }
        catch (org.omg.CosClockService.TimeUnavailable e) {
                cdmw.logging.Logger.CDMW_ERROR(logger,"Unexpected exception raised!"+e,new Throwable());
                e.printStackTrace(System.err);
        }


        System.out.println("\n<------------------------------>");
        System.out.println("<### Server : display HELLO at virtual time  ###>");
        System.out.println("<### " + cdmw.clock.Time.TimeT2timeval(currenTimeFromFederatedControlledClock).toString() + " ###>");
        System.out.println("<------------------------------>");

    }
    public void display_hello_periodically()
    {
        System.out.println("<### Arm Timer  ###>");
        // prepare Any 
        org.omg.CORBA.ORB orb = null;
        try {
                orb = org.omg.CORBA.ORB.init();
        }
        catch (java.lang.Exception e) {
                cdmw.logging.Logger.CDMW_ERROR(logger,"Unexpected exception raised at init!"+e,null);
            System.out.println("Unexpected exception raised at init!"+e);
                e.printStackTrace(System.err);
        }
        org.omg.CORBA.Any params = null;
        try {
                params = orb.create_any();
        }
        catch (java.lang.Exception e) {
                cdmw.logging.Logger.CDMW_ERROR(logger,"Unexpected exception raised at create!"+e,null);
                System.out.println("Unexpected exception raised at create!"+e);
                e.printStackTrace(System.err);
        }
        java.lang.String txt = "test string for timer";
        params.insert_string(txt);

        // prepare timer
        
        long period = 1;
        long expiration_time = 0;
    int execution_limit = 10;
        try {
                expiration_time = federatedControlledClock.current_time() + 50000;
        }
        catch (org.omg.CosClockService.TimeUnavailable e) {
                cdmw.logging.Logger.CDMW_ERROR(logger,"Unexpected exception raised!"+e,null);
                e.printStackTrace(System.err);
        }

        // arm timer
        try {
                controller.start_at(period,expiration_time,execution_limit,params);
        }
        catch (org.omg.CosClockService.PeriodicExecution.ControllerPackage.TimePast e) {
                cdmw.logging.Logger.CDMW_ERROR(logger,"Unexpected exception raised!"+e,null);
                e.printStackTrace(System.err);
        }

        

        System.out.println("<### Timer started  ..  ###>");
        
    }
    public void set_rate_federated_clock(float rate) 
    {
        // set its rate at 5.0
        try {
                federatedControlledClock.set_rate(rate);
        }
        catch (org.omg.CosClockService.ControlledClockPackage.NotSupported e) {
                cdmw.logging.Logger.CDMW_ERROR(logger,"Unexpected exception raised!"+e,new Throwable());
                e.printStackTrace(System.err);
        }

        // ask it date
        long currenTimeFromFederatedControlledClock = 0;
        try {
                currenTimeFromFederatedControlledClock = federatedControlledClock.current_time();
        }
        catch (org.omg.CosClockService.TimeUnavailable e) {
                cdmw.logging.Logger.CDMW_ERROR(logger,"Unexpected exception raised!"+e,new Throwable());
                e.printStackTrace(System.err);
        }


        System.out.println("<### Client : set new rate  ###>");
        System.out.println("\n<------------------------------>");
        System.out.println("<### Server : new virtual time  ###>");
        System.out.println("<### " + cdmw.clock.Time.TimeT2timeval(currenTimeFromFederatedControlledClock).toString() + " ###>");
        System.out.println("<------------------------------>");

    }
    public void initClockReference()
    {
        // ask Server for contacting ClockCatalog
        org.omg.CosClockService.ClockCatalog clockCatalog = 
            this.server.get_clockCatalog();
        // get entry named "FederatedControlledExecutor"
        org.omg.CosClockService.ClockCatalogPackage.ClockEntry clockEntry = null;
        try {
            clockEntry = clockCatalog.get_entry("FederatedControlledExecutor" );
        }
        catch (org.omg.CosClockService.ClockCatalogPackage.UnknownEntry e) {
            cdmw.logging.Logger.CDMW_ERROR
                (logger, "Attention! The entry is unknown! " + e,
                 new Throwable());
        }
        catch (Exception ee) {
            cdmw.logging.Logger.CDMW_ERROR
                (logger, "Unexpected exception raised!" + ee,
                 new Throwable());
            ee.printStackTrace(System.err);
        
        } 
        // get associated clock
        org.omg.CosClockService.Clock clock = 
                        clockEntry.subject;

        // obtain federated controlled clock reference
        federatedControlledClock =
                        org.omg.CosClockService.ControlledClockHelper.narrow (clock);
                         
        System.out.println("<### FederatedControlledClock avaiable  ###>");

        Cdmw.clock.ControlledExecutor controlledExecutor =
          Cdmw.clock.ControlledExecutorHelper.narrow(federatedControlledClock);

        // get controller
        controller =
          controlledExecutor.enable_periodic_execution(this.server.getPeriodic());

        System.out.println("<### Periodic avaiable  ###>");
          
    }
}
