/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccmxms;

import org.omg.Components.SessionContext;

import com.acme.XMLValidate.*;

import cdmw.logging.Logger;
import cdmw.testutils.CORBATestManager;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;

import static cdmw.logging.Logger.*;

/**
 * IDL:thalesgroup.com/XMLValidate/CCM_Client:1.0
 */
public class ClientExecutorImpl extends org.omg.CORBA.LocalObject
    implements com.acme.XMLValidate.User_Client
{

    // TestThread
    private class TestThread extends Thread {
        public CORBATestManager testManager;

        TestThread(CORBATestManager testManager) {
            this.testManager = testManager;
        }

        public void run() {
            this.testManager.runTests();
        }
    }

    private Logger logger;
    private CCM_Client_Context sessionContext;
    private java.lang.String fileName;

    private TestThread testThread;

    /**
     * Constructor.
     */
    public ClientExecutorImpl()
    {
        this.sessionContext = null;
        this.logger = Logger.Get_Logger("CCM_Client","QUALIF_CCM_XMS");
    }


    /**
     * Set the session context of the component (called by the container).
     *
     * @param SessionContext a session context
     *
     * @exception CORBA SystemException
     */
    public void set_session_context(SessionContext ctx)
        throws org.omg.CORBA.SystemException
    {
        CDMW_INFO(this.logger, "ClientExecutor_impl::set_session_context() called.",
                         new Throwable());

        this.sessionContext = CCM_Client_ContextHelper.narrow(ctx);
    }


    /**
     * Activate the component (called by the container).
     *
     * @exception CORBA SystemException
     */
    public void ccm_activate()
        throws org.omg.CORBA.SystemException
    {
        CDMW_INFO(this.logger, "ClientExecutor_impl::ccm_activate() called.",
                         new Throwable());


        try {
            CORBATestManager testManager = CORBATestManager.instance();
        testManager.setLocalProperty(XMLValidateTester.SESSION_CONTEXT_NAME,
                     this.sessionContext);

        testManager.setLocalProperty(XMLValidateTester.FILE_NAME,
                     this.fileName);
        
        testManager.addTest(XMLValidateTester.class);
            this.testThread = new TestThread(testManager);
            this.testThread.start();
        }
        catch (Exception e) {
            System.err.println("Exception thrown during test: " + e);
        }
    }


    /**
     * Stop the component (called by the container).
     *
     * @exception CORBA SystemException
     */
    public void ccm_passivate()
        throws org.omg.CORBA.SystemException
    {
        CDMW_INFO(this.logger, "ClientExecutor_impl::ccm_passivate() called.",
                         new Throwable());
    }


    /**
     * Destroy the component (called by the container).
     *
     * @exception CORBA SystemException
     */
    public void ccm_remove()
        throws org.omg.CORBA.SystemException
    {
        CDMW_INFO(this.logger, "ClientExecutor_impl::ccm_remove() called.",
                         new Throwable());
    }

    /**
     * Changes the file name.
     * @param file_name the new file_name
     *
     * @exception CORBA SystemException
     */
    public void file_name(java.lang.String file_name)
        throws org.omg.CORBA.SystemException
    {
        this.fileName = file_name;
    }

    /**
     * Get the file name.
     *
     * @exception CORBA SystemException
     */
    public java.lang.String  file_name()
        throws org.omg.CORBA.SystemException
    {
        return fileName;
    }



    public static class XMLValidateTester
    {
    public static final String SESSION_CONTEXT_NAME = "client_context";
    public static final String FILE_NAME = "file_name";

    private CORBATestManager testManager;

    private String fileName;
    private CCM_Client_Context sessionContext;
    private Logger logger;

    @Before
    public void init()
    {
        testManager = CORBATestManager.instance();
        this.logger = Logger.Get_Logger("CCM_Client","QUALIF_CCM_XMS");
        this.sessionContext = (CCM_Client_Context) testManager.getLocalProperty(SESSION_CONTEXT_NAME);
        this.fileName = (String) testManager.getLocalProperty(FILE_NAME);
    }
    
    @Test
    public void doTests()
    {
        boolean res = false;
        try {
        // get the Validator connection of the server.
        CDMW_INFO(logger, "connect to the validator facet...", 
              new Throwable());
        Validator validator = 
            this.sessionContext.get_connection_client_receptacle();
        
        // call the server validator.
        CDMW_INFO(logger, "call the server validator...",  new Throwable());
        
        res = validator.parse(this.fileName);
        }
        catch (Exception e) {
        CDMW_ERROR(logger, "unexpected exception raised." + e,
               new Throwable());   
        res = false;
        }
        assertTrue(res);

            /////////////////////////////
            // end test
            ///////////////////////////
            testManager.printSummary();

    }
    }
}
