/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package qualif.ft.testfttimeoutrequest;

import java.io.IOException;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;

import cdmw.orbsupport.OrbShutdownSignalHandler;
import cdmw.orbsupport.StrategyList;
import cdmw.platformmngt.PlatformInterface;

public class MainServer {
    private static final int POA_THREAD_POOL_SIZE = 5;

    private static class HelloActivationHandler extends
        org.omg.CORBA.LocalObject implements
        com.thalesgroup.CdmwFT.Location.ActivationHandler {

        private String location;

        public HelloActivationHandler(
            String location) {
            this.location = location;
        }

        public void activate() {
            try {
                java.io.FileWriter os = new java.io.FileWriter("hello.act");
                os.write(location);
                os.close();
                System.out.println(this.location
                    + " <####### HELLO ACTIVATION ######>");
            }
            catch (IOException e) {
                e.printStackTrace();
            }
        }

        public void deactivate() {
            try {
                java.io.FileWriter os = new java.io.FileWriter("hello.dct");
                os.write(location);
                os.close();
                System.out.println(this.location
                    + " <####### HELLO DEACTIVATION ######>");
            }
            catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    private static class ComputActivationHandler extends
        org.omg.CORBA.LocalObject implements
        com.thalesgroup.CdmwFT.Location.ActivationHandler {

        private String location;

        public ComputActivationHandler(
            String location) {
            this.location = location;
        }

        public void activate() {
            try {
                java.io.FileWriter os = new java.io.FileWriter("comput.act");
                os.write(location);
                os.close();
            }
            catch (IOException e) {
                e.printStackTrace();
            }
            System.out
                .println(location + " <####### COMPUT ACTIVATION ######>");
        }

        public void deactivate() {
            try {
                java.io.FileWriter os = new java.io.FileWriter("comput.dct");
                os.write(location);
                os.close();
            }
            catch (IOException e) {
                e.printStackTrace();
            }
            System.out.println(location
                + " <####### COMPUT DEACTIVATION ######>");
        }
    }

    public static void main(String[] args) {
        String hostName = cdmw.ossupport.OS.getHostname();

        ORB orb = null;

        try {
            // Initialise FT service
            System.out
                .println(hostName + "test_server: initialise FT services");
            //cdmw.ftinit.FTServiceInit.init(args, false); // This should be
            cdmw.cdmwinit.CdmwInterface.cdmwPreORBInit(args, "cdmw.generated.FTTimeoutServerCdmwInterface");
            // included in
            // preORBinit()
            // once PCR-0640 is
            // corrected!

            //cdmw.cdmwinit.CdmwInterface.cdmwInit(orb, args, pProcessBehaviour, "cdmw.generated.FTTimeoutServerCdmwInterface");

            // Initialises the ORB
            StrategyList strategyList = new StrategyList();
            strategyList.addORBThreaded();
            strategyList.addPOAThreadPool(POA_THREAD_POOL_SIZE);

            System.out.println(hostName + " test_server: initialises the ORB");
            orb = cdmw.orbsupport.ORBSupport.ORBInit(args, strategyList);

            // Get the root POA
            System.out.println(hostName + " test_server: get the root POA");
            org.omg.CORBA.Object obj = orb
                .resolve_initial_references("RootPOA");
            POA rootPOA = POAHelper.narrow(obj);

            System.out
                .println(hostName + " test_server: activate the root POA");
            POAManager poaMgr = rootPOA.the_POAManager();
            poaMgr.activate();

            int timescale = cdmw.testutils.CORBATestManager.getTimescale();
            cdmw.ossupport.OS.sleep(timescale * 1000);

            // init Cdmw
            System.out.println(hostName + " test_server: init Cdmw");
            cdmw.cdmwinit.CdmwInterface.cdmwInit(orb, args, null, "cdmw.generated.FTTimeoutServerCdmwInterface");

            System.out.println(hostName
                + " test_server: create HelloInterface_impl");
            // Create activation handler, it will be register in init operation
            // of Hello
            System.out.println(hostName
                + " test_server: create activation handler and register it");
            com.thalesgroup.CdmwFT.Location.ActivationHandler activation_handler = new HelloActivationHandler(
                hostName);

            HelloInterfaceImpl helloInterface = new HelloInterfaceImpl(orb,
                hostName, activation_handler);
            com.thalesgroup.CdmwFT.Test.HelloInterface helloInterfaceRef = helloInterface
                ._this();

            System.out.println(hostName
                + " test_server: create ComputInterface_impl");
            // Create activation handler, it will be register in init operation
            // of Hello
            System.out.println(hostName
                + " test_server: create activation handler and register it");
            com.thalesgroup.CdmwFT.Location.ActivationHandler comput_activation_handler = new ComputActivationHandler(
                hostName);

            ComputInterfaceImpl computInterface = new ComputInterfaceImpl(orb,
                hostName, activation_handler);
            com.thalesgroup.CdmwFT.Test.ComputInterface comput_interface_ref = computInterface
                ._this();

            // export the object reference to the CDMW Repository
            System.out
                .println(hostName
                    + " test_server: export the hello object reference to CDMW Repository");
            com.thalesgroup.CdmwNamingAndRepository.Repository repository = cdmw.namingandrepository.RepositoryInterface
                .getRepository();

            cdmw.namingandrepository.RepositoryInterface.init("CDMW",
                repository);

            com.thalesgroup.CdmwNamingAndRepository.NameDomain helloDomain = repository
                .resolve_name_domain("dom1/dom2/dom3");

            // get process name
            String processName = cdmw.platformmngt.PlatformInterface
                .getProcessName();

            String helloName = processName + "HelloInterface";
            System.out
                .println(hostName + " test_server: register " + helloName);

            try {
                helloDomain.release_name(helloName);
            }
            catch (com.thalesgroup.CdmwNamingAndRepository.NameDomainPackage.NotRegistered e) {
                // not a problem it's the first time the server is launched
                e.printStackTrace();
                System.err.println("OK");
            }
            catch (com.thalesgroup.CdmwNamingAndRepository.InvalidName e0) {
                e0.printStackTrace();
                System.exit(1);
            }
            catch (SystemException e1) {
                e1.printStackTrace();
                System.err.println("ERROR: ------------------------");
                System.exit(1);
            }

            try {
                Any regId = helloDomain.new_name(helloName);

                // register the object
                helloDomain.register_object(regId, helloInterfaceRef);
            }
            catch (SystemException e) {
                e.printStackTrace();
                System.err.println("ERROR: ------------------------");
                System.exit(1);
            }
            catch (com.thalesgroup.CdmwNamingAndRepository.NameDomainPackage.AlreadyExists e0) {
                e0.printStackTrace();
                System.exit(1);
            }

            // export the object reference to a CDMW Repository
            System.out
                .println(hostName
                    + " test_server: export the comput object reference to a CDMW Repository");
            String computName = processName + "ComputInterface";
            System.out.println(hostName + " test_server: register "
                + computName);

            try {
                helloDomain.release_name(computName);
            }
            catch (com.thalesgroup.CdmwNamingAndRepository.NameDomainPackage.NotRegistered e) {
                // not a problem it's the first time the server is launched
                e.printStackTrace();
                System.err.println("OK");
            }
            catch (com.thalesgroup.CdmwNamingAndRepository.InvalidName e0) {
                e0.printStackTrace();
                System.exit(1);

            }
            catch (SystemException e1) {
                e1.printStackTrace();
                System.err.println("ERROR: ------------------------");
                System.exit(1);
            }

            try {
                Any regId = helloDomain.new_name(computName);

                // register the object
                helloDomain.register_object(regId, comput_interface_ref);
            }
            catch (SystemException e) {
                e.printStackTrace();
                System.err.println("ERROR: ------------------------");
                System.exit(1);
            }
            catch (com.thalesgroup.CdmwNamingAndRepository.NameDomainPackage.AlreadyExists e0) {
                e0.printStackTrace();
                System.exit(1);
            }

            // Setup signal handling for ORB to be shut down
            OrbShutdownSignalHandler signalHandler = new OrbShutdownSignalHandler(
                orb);
            cdmw.orbsupport.ORBSupport.InstallORBShutdownSignalHandler(orb,
                signalHandler);

            System.out.println(hostName + " test_server: orb run");
            orb.run();

            cdmw.orbsupport.ORBSupport
                .UninstallORBShutdownSignalHandler(signalHandler);

        }
        catch (SystemException e) {
            e.printStackTrace();
            System.exit(1);
        }
        catch (UserException e0) {
            e0.printStackTrace();
            System.exit(1);
        }
        catch (cdmw.common.Exception e1) {
            e1.printStackTrace();
            System.exit(1);
        }
        catch (Exception e2) {
            System.err.println("Uncaught exception");
            System.exit(1);
        }

        if (orb != null) {
            try {
                cdmw.orbsupport.ORBSupport.ORBCleanup(orb);
                orb.destroy();
            }
            catch (SystemException ex) {
                System.err.println(" **** TEST FAILED AT LINE ");
                ex.printStackTrace();
                System.exit(1);
            }
        }

        System.exit(0);
    }
}
