/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.containervirtualsize;

import cdmw.ossupport.OS;
import cdmw.logging.*;
import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;
import cdmw.platformmngt.PlatformInterface;
import com.acme.ContainerVirtualSize.*;
import UserContainerVirtualSize.*;

//
// IDL:acme.com.ContainerVirtualSize.CCM_ServerLBClientFT:1.0
//
/**
 * Component implementation of a ServerLBClientFT
 */
public class CCMServerLBClientFTImpl extends org.omg.CORBA.LocalObject
    implements User_CCM_ServerLBClientFT
{
    // session context
    private CCM_ServerLBClientFT_Context sessionContext;
    private Logger logger;

    /**
     * Purpose:
     * <p> Constructor.
     * Initalises some private variables
     */
    public CCMServerLBClientFTImpl() 
    {
        this.sessionContext = null;
        this.logger = Logger.Get_Logger("CCM_ServerLBClientFT",
                                        "QUALIF_CCM_CONTAINER_VIRTUAL_SIZE");
        CDMW_LOG_FUNCTION(this.logger, 
                  "CCMServerLBClientFTImpl Constructor");
    }

    //
    // IDL:acme.com.ContainerVirtualSize.CCM_HostsName:1.0
    //
    /**
     * Purpose:
     * <p> Accessor on CCM_HostsName interface
     */
    public com.acme.ContainerVirtualSize.CCM_HostsName get_hostsName()
    {
        CDMW_LOG_FUNCTION(this.logger, 
                  "CCMServerLBClientFTImpl.get_hostsName");
        return (com.acme.ContainerVirtualSize.CCM_HostsName) this;
    }

    //
    // IDL:acme.com.ContainerVirtualSize.CCM_HostsName:1.0
    //
    /**
     * Purpose:
     * <p> Accessor on host name of FT Server and LB Server
     *
     * @param lbserver host name of the LB server
     * @param ftserver host name of the FT server
     */
    public void get_names(org.omg.CORBA.StringHolder lbserver, org.omg.CORBA.StringHolder ftserver)
    {
        CDMW_LOG_FUNCTION(this.logger, "CCMServerLBClientFTImpl.get_names()");

        // Get LBServer hostname
        lbserver.value = cdmw.ossupport.OS.getHostname();
        
        // Get FTServer hostname
        com.acme.ContainerVirtualSize.HostInfo serverFT = 
            this.sessionContext.get_connection_hostInfo();
        ftserver.value = serverFT.get_hostname();
    }
  
    /**
     * Purpose:
     * <p> Mutator on session context
     *
     * @param ctx the session context
     */
    public void set_session_context(org.omg.Components.SessionContext ctx) 
    {
        CDMW_LOG_FUNCTION(this.logger, "CCMServerLBClientFTImpl.set_session_context()");

        this.sessionContext = CCM_ServerLBClientFT_ContextHelper.narrow(ctx);
    }
    
    /**
     * Purpose:
     * <p> Activate the component
     */
    public void ccm_activate() 
    {
        CDMW_LOG_FUNCTION(this.logger, "CCMServerLBClientFTImpl.ccm_activate()");
    }

    /**
     * Purpose:
     * <p> Deactivate the component
     */
    public void ccm_passivate() 
    {
        CDMW_LOG_FUNCTION(this.logger, "CCMServerLBClientFTImpl.ccm_passivate()");
    }

    /**
     * Purpose:
     * <p> Remove the component
     */
    public void ccm_remove() 
    {
        CDMW_LOG_FUNCTION(this.logger, "CCMServerLBClientFTImpl.ccm_remove()");
    }
} /* End of class */
