/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.platformmngt;

import org.omg.PortableInterceptor.*;
import org.omg.CORBA.*;
import org.omg.CORBA.OctetSeqHelper;
import org.omg.IOP.ServiceContext;
import java.io.*;
import cdmw.orbsupport.Codec;
import cdmw.orbsupport.ORBUtils;
import com.thalesgroup.CdmwPlatformMngt.*;


/**
 * Purpose: <p> This class implements the ClientRequestInterceptor class.
 * 
 * Features: <p> Thread safe
 * 
 */
public class ClientInterceptor extends
    org.omg.CORBA.LocalObject implements
    org.omg.PortableInterceptor.ClientRequestInterceptor {

    private ORB Orb = null;
    private String Issuer = "";
    Codec IssuerCodec = null; 
    ServiceContext IssuerSvcContext; 
    
    /**
     * Purpose: <p> Constructor
     * 
     * @param info
     *            ORB init information
     * @param requestIdSlotId
     *            slot id used to store the date of the outgoing request.
     * 
     */
    public ClientInterceptor() 
    {
        IssuerSvcContext = new ServiceContext();
        IssuerSvcContext.context_id = 
            com.thalesgroup.CdmwPlatformMngt.SMG_INTERCEPTOR.value;
    }

    /**
     * Purpose: <p> When an exception occurs, this interception point is called.
     * It allows an Interceptor to query the exception's information before it
     * is raised to the client.
     * 
     * @param ri
     *            Client request information
     */
    public void receive_exception(ClientRequestInfo ri) throws ForwardRequest,
        SystemException {
    }

    /**
     * Purpose: <p> This interception point allows an Interceptor to query the
     * information available when a request results in something other than a
     * normal reply or an exception. For example, a request could result in a
     * retry (for example, a GIOP Reply with a LOCATION_FORWARD status was
     * received); or on asynchronous calls, the reply does not immediately
     * follow the request, but control shall return to the client and an ending
     * interception point shall be called.
     * 
     * @param ri
     *            Client request information
     */
    public void receive_other(ClientRequestInfo ri) throws ForwardRequest,
        SystemException {
    }

    /**
     * Purpose: <p> This interception point allows an Interceptor to query the
     * information on a reply after it is returned from the server and before
     * control is returned to the client. Here the date of the outgoing request
     * is reset and T0 of incoming request updated to remove the duration of the
     * outgoing request.
     * 
     * @param ri
     * Client request information
     */
    public void receive_reply(ClientRequestInfo ri)
        throws org.omg.CORBA.SystemException {
    }

    /**
     * Purpose: <p> This interception point allows an Interceptor to query
     * information during a Time-Independent Invocation (TII) polling get reply
     * sequence.
     * 
     * @param ri
     *            Client request information
     * 
     */
    public void send_poll(ClientRequestInfo ri)
        throws org.omg.CORBA.SystemException {
    }
    
    
    /**
     * Purpose: <p> This interception point allows an Interceptor to query
     * request information and modify the service context before the request is
     * sent to the server. Here, the date of the outgoing request is stored
     * here.
     * 
     * @param ri
     * Client request information
     * 
     */
    public void send_request(ClientRequestInfo ri) throws ForwardRequest,
    org.omg.CORBA.SystemException {
        
        if (Orb != null)
        {                      
            // Convert data
            byte[] encoded_data = ORBUtils.issuerToByteArray(Issuer, Orb);
            IssuerSvcContext.context_data = new byte [encoded_data.length];         

            for (int i=0; i < encoded_data.length ; i++)
            {
                IssuerSvcContext.context_data[i] = encoded_data[i];
            }

            // Add Context Service
            ri.add_request_service_context (IssuerSvcContext, true);

        }//else do nothing
    }  


    /**
     * Purpose: <p> Destroy the interceptor
     * 
     */
    public void destroy() throws org.omg.CORBA.SystemException {

    }

    /**
     * Purpose: <p> Return the name of the interceptor
     * 
     */
    public String name() throws org.omg.CORBA.SystemException {
        
        String name = "JavaClientInterceptor";
        return name;
    }
    
    /**
     * Purpose: <p> Set orb reference
     * 
     */
    public void setOrb(org.omg.CORBA.ORB orb) throws org.omg.CORBA.SystemException 
    {
        if (orb!=null)
        {
            Orb = orb;  
            cdmw.orbsupport.Codec.init(orb);
            IssuerCodec = new Codec();          
        }
        else
        {
            java.lang.System.out.println(" WARNING : Invalid ORB reference");
        }
    }
    
    /**
     * Purpose: <p> Set issuer value
     * 
     */
    public void addIssuer(String issuer) throws org.omg.CORBA.SystemException { 
        Issuer = issuer;
    }
}
