/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.platformmngt;

import org.omg.PortableInterceptor.ORBInitInfo;

import cdmw.common.Assert;
import cdmw.logging.FunctionLogger;
import static cdmw.logging.Logger.*;
import org.omg.CORBA.*;

public class ClientOrbInitializer extends org.omg.CORBA.LocalObject
    implements org.omg.PortableInterceptor.ORBInitializer {

    private static ClientInterceptor clientInterceptor = null;
    private static boolean post_done = false;
    private ORB Orb = null;
    
    /**
     * Purpose: <p> Constructor
     * 
     */
    public ClientOrbInitializer(){
    }

    /**
     * Purpose: <p> Set orb reference
     * 
     */
    public void setOrb(ORB orb){
        if (post_done)
        {      
            clientInterceptor.setOrb(orb);
        }
    }
    
    /**
     * Purpose: <p> Set issuer value
     * 
     */
    public void addIssuer(String issuer){       
        synchronized (this) {
            if (post_done)
            {   
                clientInterceptor.addIssuer(issuer);
            }
        }
    }
    
    /**
     * Purpose: <p> This operation is called during ORB initialization.
     * 
     */
    public void pre_init(ORBInitInfo info) {
    }

    /**
     * Purpose: <p> This operation is called during ORB initialization.
     * 
     * Note: the interceptors are registered in post_init() if their
     * constructors require ORBInitInfo.resolve_initial_reference(), which XXX
     * cannot be called in pre_init().
     * 
     */
    public void post_init(ORBInitInfo info) {
        //
        // Create and register the interceptor
        //
        clientInterceptor = new ClientInterceptor();
        
        synchronized (this) {
        post_done = true;
        }

        try 
        {           
            info.add_client_request_interceptor(clientInterceptor);
        }
        catch (org.omg.PortableInterceptor.ORBInitInfoPackage.DuplicateName e) {
            // can not happened while interceptor is not named!
            Assert.check(false);
        }
    }
}

