/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.platformmngt;


/**
 * The default implementation of a ManagedElementContexts::MessageNotifier.
 *
 */
abstract class MessageNotifierImpl
    extends org.omg.CORBA.LocalObject
    implements com.thalesgroup.CdmwPlatformMngt.ManagedElementContexts.MessageNotifier
{

    /**
     * Purpose:
     * <p> Constructor.
     */ 
    public MessageNotifierImpl()
    {
    }
    
    
    //
    // IDL:thalesgroup.com/CdmwPlatformMngt/ManagedProcessContext/notify_message:1.0
    //
    public void notify_message(int level, String message)
    {
        try {
            // WARNING: cast level from int to short !!
            //          PlatformInterface.notifyMessage should be changed
            //          to accept int.
            PlatformInterface.notifyMessage((short) level,
                                            getIssuerName(),
                                            message);
        } catch (cdmw.common.BadOrderException e) {
            // shouldn't happen since PlatformInterface has been 
            // initialised at process start-up
            cdmw.common.Assert.assertionFailed();
        } catch (cdmw.common.BadParameterException e) {
            // TODO: raise an exception ??
            System.err.println(e);
            e.printStackTrace();
            cdmw.common.Assert.assertionFailed();
        }
    }
    
      
    //
    // IDL:thalesgroup.com/CdmwPlatformMngt/ManagedProcessContext/notify_fatal_error:1.0
    //
    public void notify_fatal_error(String error_info)
    {
        try {
            PlatformInterface.notifyFatalError(getIssuerName(),
                                               error_info);
        } catch (cdmw.common.BadOrderException e) {
            // shouldn't happen since PlatformInterface has been 
            // initialised at process start-up
            cdmw.common.Assert.assertionFailed();
        } catch (cdmw.common.BadParameterException e) {
            // TODO: raise an exception ??
            System.err.println(e);
            e.printStackTrace();
            cdmw.common.Assert.assertionFailed();
        }
    }
    
    
    //
    // IDL:thalesgroup.com/CdmwPlatformMngt/ManagedProcessContext/notify_user_event:1.0
    //
    public void notify_user_event(org.omg.CORBA.Any data)
    {
                
        try {
            PlatformInterface.notifyUserEvent(getIssuerName(),
                                               data);
        } catch (cdmw.common.BadOrderException e) {
            // shouldn't happen since PlatformInterface has been 
            // initialised at process start-up
            cdmw.common.Assert.assertionFailed();
        } catch (cdmw.common.BadParameterException e) {
            // TODO: raise an exception ??
            System.err.println(e);
            e.printStackTrace();
            cdmw.common.Assert.assertionFailed();
        }
        
        
    }
    
    
    /**
     * Purpose:
     * <p>
     * returns the issuer name to be used in notifications.
     * This operation has to be implemented by inheriting classes
     *
     * @return the issuer name
     */
    protected abstract String getIssuerName();
    
    
}

