/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "Foundation/testutils/CORBATestManager.hpp"
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/osthreads/ReaderWriterLock.hpp>
#include <Foundation/osthreads/Mutex.hpp>
#include "Foundation/osthreads/MutexGuard.hpp"
#include "tools/ManagedElementKey.hpp"
#include "testsmgtools/TestSuite_ManagedElementKey_cpp.hpp"
#include "tools/ProcessIssuerManager.hpp"
#include "tools/ApplicationIssuerManager.hpp"
#include "tools/HostIssuerManager.hpp"
#include "tools/SystemIssuerManager.hpp"

#include <stdlib.h>
#include <stdio.h>

CPPUNIT_TEST_SUITE_REGISTRATION(TestSuite_ManagedElementKey_cpp_d2128995);

Task::Task (int number)
{
    m_num_thread = number;
}

Task::~Task() throw()
{
    
}
void Task::run() throw()
{
    Cdmw::OsSupport::OS::Timeval time_val ;

    std::ostringstream buffer_process;
    std::ostringstream buffer_application;
    std::ostringstream buffer_host;

    buffer_process <<"Process_" << m_num_thread;
    buffer_application <<"Application_" << m_num_thread;
    buffer_host <<"Host_" << m_num_thread;

    std::string process_path = buffer_process.str();
    std::string application_path = buffer_application.str();
    std::string host_path = buffer_host.str();

    try 
    {
    
        Cdmw::PlatformMngt::ProcessIssuerManager::Instance()->attach_element(process_path);
        Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->attach_element(application_path);
        Cdmw::PlatformMngt::HostIssuerManager::Instance()->attach_element(host_path);

        for (int i=0; i< 10 ; i++)
        {
             time_val= Cdmw::OsSupport::OS::get_time();
             std::ostringstream buffer;
             buffer << time_val.microseconds;

             {
                 CDMW_MUTEX_GUARD (*(Cdmw::PlatformMngt::ProcessIssuerManager::Instance()->get_process_issuer_mutex(process_path)));
                 Cdmw::PlatformMngt::ProcessIssuerManager::Instance()->set_issuer(process_path, buffer.str());
             }

             {
                 CDMW_MUTEX_GUARD (*(Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->get_application_issuer_mutex(application_path)));
                 Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->set_issuer(application_path, buffer.str());
             }

             {
                 CDMW_MUTEX_GUARD (*(Cdmw::PlatformMngt::HostIssuerManager::Instance()->get_host_issuer_mutex(host_path)));
                 Cdmw::PlatformMngt::HostIssuerManager::Instance()->set_issuer(host_path, buffer.str());
             }

             sleep(1000);

             std::string proc_issuer;
             std::string appl_issuer;
             std::string host_issuer;

             {
                 CDMW_MUTEX_GUARD (*(Cdmw::PlatformMngt::ProcessIssuerManager::Instance()->get_process_issuer_mutex(process_path)));
                 proc_issuer = Cdmw::PlatformMngt::ProcessIssuerManager::Instance()->get_command_issuer(process_path);
             }

             {
                 CDMW_MUTEX_GUARD (*(Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->get_application_issuer_mutex(application_path)));
                 appl_issuer = Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->get_command_issuer(application_path);
             }

             {
                 CDMW_MUTEX_GUARD (*(Cdmw::PlatformMngt::HostIssuerManager::Instance()->get_host_issuer_mutex(host_path)));
                 host_issuer = Cdmw::PlatformMngt::HostIssuerManager::Instance()->get_command_issuer(host_path);
             }
             
             std::cerr<<"\n___________________________________________________________";
             std::cerr << " \n\n THREAD  "<< m_num_thread;
             std::cerr << " \n\n SET VALUE "<< buffer.str();
             std::cerr << " \n\n PROCESS ISSUER MANAGER GET VALUE "<< proc_issuer;
             std::cerr << " \n\n APPLICATION ISSUER MANAGER GET VALUE "<< appl_issuer;
             std::cerr << " \n\n HOST ISSUER MANAGER GET VALUE "<< host_issuer;
         
             if ((buffer.str() == proc_issuer) 
                 && (buffer.str() == appl_issuer)
                 && (buffer.str() == host_issuer))
             {
                 std::cerr<<"\n                                                   OK";
                 CPPUNIT_ASSERT (true);
             }
             else 
             {
                std::cerr<<"\n                                                   NOK";
                CPPUNIT_ASSERT (false);
             }
             std::cerr<<"\n____________________________________________________________";
        }

    }catch (...)
    {
         std::cerr<<"\n                                                   NOK";
         std::cerr<<"\n  Unknown Exception Caught";
         CPPUNIT_ASSERT (false);    
    }
}

SystemIssuerManagerTask::SystemIssuerManagerTask ()
{
}

SystemIssuerManagerTask::~SystemIssuerManagerTask() throw()
{
}

void SystemIssuerManagerTask::run() throw()
{

    std::cerr<<"\n___________________________________________________________";
    std::cerr << " \n\n TEST SYSTEMISSUERMANAGER  ";
    std::cerr<<"\n____________________________________________________________";
    
    Cdmw::OsSupport::OS::Timeval time_val ;

    std::string system_path = "System";
    
    Cdmw::PlatformMngt::SystemIssuerManager::Instance()->attach_element(system_path);

    for (int i=0; i< 10 ; i++)
    {
        time_val= Cdmw::OsSupport::OS::get_time();
        std::ostringstream buffer;
        buffer << time_val.microseconds;

        std::string syst_issuer = "";

        try
        {
            {
                CDMW_MUTEX_GUARD (*(Cdmw::PlatformMngt::SystemIssuerManager::Instance()->get_system_issuer_mutex()));
                Cdmw::PlatformMngt::SystemIssuerManager::Instance()->set_issuer(buffer.str());
            }
            sleep(1000);

            {
                CDMW_MUTEX_GUARD (*(Cdmw::PlatformMngt::SystemIssuerManager::Instance()->get_system_issuer_mutex()));
                syst_issuer = Cdmw::PlatformMngt::SystemIssuerManager::Instance()->get_command_issuer();
            }
        }
        catch (...)
        {
            std::cerr<<"\n                                                   NOK";
            CPPUNIT_ASSERT (false);        
        }
        
        std::cerr<<"\n___________________________________________________________";
        std::cerr << " \n\n SET VALUE "<< buffer.str();
        std::cerr << " \n\n SYSTEM ISSUER MANAGER GET VALUE "<< syst_issuer;
        
        if (buffer.str() == syst_issuer)
        {
            std::cerr<<"\n                                                   OK";
            CPPUNIT_ASSERT (true);
        }
        else 
        {
            std::cerr<<"\n                                                   NOK";
            CPPUNIT_ASSERT (false);
        }
        std::cerr<<"\n____________________________________________________________";
    }
}

void TestSuite_ManagedElementKey_cpp_d2128995::setUp()
{
}

void TestSuite_ManagedElementKey_cpp_d2128995::tearDown()
{
}

/* CPPUNIT_TEST_CASE_BEGIN test_Check_process_rel_key_1 */
/* CPPUNIT_TEST_CASE_CONTEXT bool Cdmw::PlatformMngt::ManagedElementKey::Check_process_rel_key(const std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Check_process_rel_key_1()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (process_rel_key) */ 
    ::std::string _process_rel_key  = "Host/Process";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "";
    /* Tested function call */
    bool _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Check_process_rel_key(_process_rel_key, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT(false == ( _return ));
    CPPUNIT_ASSERT("Process"==_process_name);
}
/* CPPUNIT_TEST_CASE_END test_Check_process_rel_key_1 */

/* CPPUNIT_TEST_CASE_BEGIN test_Check_process_rel_key_10 */
/* CPPUNIT_TEST_CASE_CONTEXT bool Cdmw::PlatformMngt::ManagedElementKey::Check_process_rel_key(const std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Check_process_rel_key_10()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (process_rel_key) */ 
    ::std::string _process_rel_key  = "/";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "";
    /* Tested function call */
    bool _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Check_process_rel_key(_process_rel_key, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT(true== ( _return ));;
}
/* CPPUNIT_TEST_CASE_END test_Check_process_rel_key_10 */

/* CPPUNIT_TEST_CASE_BEGIN test_Check_process_rel_key_2 */
/* CPPUNIT_TEST_CASE_CONTEXT bool Cdmw::PlatformMngt::ManagedElementKey::Check_process_rel_key(const std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Check_process_rel_key_2()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (process_rel_key) */ 
    ::std::string _process_rel_key  = "Host/Process";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "Process_different";
    /* Tested function call */
    bool _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Check_process_rel_key(_process_rel_key, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT(false== ( _return ));
    CPPUNIT_ASSERT("Process"==_process_name);
    
}
/* CPPUNIT_TEST_CASE_END test_Check_process_rel_key_2 */

/* CPPUNIT_TEST_CASE_BEGIN test_Check_process_rel_key_3 */
/* CPPUNIT_TEST_CASE_CONTEXT bool Cdmw::PlatformMngt::ManagedElementKey::Check_process_rel_key(const std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Check_process_rel_key_3()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (process_rel_key) */ 
    ::std::string _process_rel_key  = "Process";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "";
    /* Tested function call */
    try{
     bool _return=::Cdmw::PlatformMngt::ManagedElementKey::Check_process_rel_key(_process_rel_key, _process_name);
     CPPUNIT_ASSERT(true== ( _return ));
     CPPUNIT_ASSERT("Process"==_process_name);
     
    }
    catch (const CdmwPlatformMngt::ElementPathInvalid&){
    CPPUNIT_ASSERT(false);
    }


}
/* CPPUNIT_TEST_CASE_END test_Check_process_rel_key_3 */

/* CPPUNIT_TEST_CASE_BEGIN test_Check_process_rel_key_4 */
/* CPPUNIT_TEST_CASE_CONTEXT bool Cdmw::PlatformMngt::ManagedElementKey::Check_process_rel_key(const std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Check_process_rel_key_4()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (process_rel_key) */ 
    ::std::string _process_rel_key  = "Process/";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "";
    /* Post-condition check */
       try{
     ::Cdmw::PlatformMngt::ManagedElementKey::Check_process_rel_key(_process_rel_key, _process_name);
     CPPUNIT_ASSERT(false);
     //CPPUNIT_ASSERT(true== ( _return ));
     //CPPUNIT_ASSERT("Process.proc"==_process_name);
     
    }
    catch (const CdmwPlatformMngt::ElementPathInvalid&){
    CPPUNIT_ASSERT(true);
    }
}
/* CPPUNIT_TEST_CASE_END test_Check_process_rel_key_4 */

/* CPPUNIT_TEST_CASE_BEGIN test_Check_process_rel_key_5 */
/* CPPUNIT_TEST_CASE_CONTEXT bool Cdmw::PlatformMngt::ManagedElementKey::Check_process_rel_key(const std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Check_process_rel_key_5()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (process_rel_key) */ 
    ::std::string _process_rel_key  = "/";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    bool _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Check_process_rel_key(_process_rel_key, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT(true== ( _return ));;
}
/* CPPUNIT_TEST_CASE_END test_Check_process_rel_key_5 */

/* CPPUNIT_TEST_CASE_BEGIN test_Check_process_rel_key_6 */
/* CPPUNIT_TEST_CASE_CONTEXT bool Cdmw::PlatformMngt::ManagedElementKey::Check_process_rel_key(const std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Check_process_rel_key_6()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (process_rel_key) */ 
    ::std::string _process_rel_key  = "Hello world";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "/";
    /* Tested function call */
    bool _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Check_process_rel_key(_process_rel_key, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT(true== ( _return ));;
}
/* CPPUNIT_TEST_CASE_END test_Check_process_rel_key_6 */

/* CPPUNIT_TEST_CASE_BEGIN test_Check_process_rel_key_7 */
/* CPPUNIT_TEST_CASE_CONTEXT bool Cdmw::PlatformMngt::ManagedElementKey::Check_process_rel_key(const std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Check_process_rel_key_7()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (process_rel_key) */ 
    ::std::string _process_rel_key  = "";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "/";
    /* Tested function call */
    bool _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Check_process_rel_key(_process_rel_key, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT(true== ( _return ));;
}
/* CPPUNIT_TEST_CASE_END test_Check_process_rel_key_7 */

/* CPPUNIT_TEST_CASE_BEGIN test_Check_process_rel_key_8 */
/* CPPUNIT_TEST_CASE_CONTEXT bool Cdmw::PlatformMngt::ManagedElementKey::Check_process_rel_key(const std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Check_process_rel_key_8()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (process_rel_key) */ 
    ::std::string _process_rel_key  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "";
    /* Tested function call */
    bool _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Check_process_rel_key(_process_rel_key, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT(true== ( _return ));;
}
/* CPPUNIT_TEST_CASE_END test_Check_process_rel_key_8 */

/* CPPUNIT_TEST_CASE_BEGIN test_Check_process_rel_key_9 */
/* CPPUNIT_TEST_CASE_CONTEXT bool Cdmw::PlatformMngt::ManagedElementKey::Check_process_rel_key(const std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Check_process_rel_key_9()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (process_rel_key) */ 
    ::std::string _process_rel_key  = "";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "";
    /* Tested function call */
    bool _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Check_process_rel_key(_process_rel_key, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT(true== ( _return ));;
}
/* CPPUNIT_TEST_CASE_END test_Check_process_rel_key_9 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_application_entity_path_1 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_application_entity_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_application_entity_path_1()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello appl";
    /* Initializing argument 3 (entity_name) */ 
    ::std::string _entity_name  = "Hello entity";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_application_entity_path(_system_name, _application_name, _entity_name);
    /* Post-condition check */
    //std::cerr << _return << std::endl;
    CPPUNIT_ASSERT("Hello world.sys/Hello appl.appl/Hello entity.ent"==_return);
}
/* CPPUNIT_TEST_CASE_END test_Get_application_entity_path_1 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_application_entity_path_10 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_application_entity_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_application_entity_path_10()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "/";
    /* Initializing argument 3 (entity_name) */ 
    ::std::string _entity_name  = "";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_application_entity_path(_system_name, _application_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_application_entity_path_10 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_application_entity_path_2 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_application_entity_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_application_entity_path_2()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 3 (entity_name) */ 
    ::std::string _entity_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_application_entity_path(_system_name, _application_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("TBD"==_return );
}
/* CPPUNIT_TEST_CASE_END test_Get_application_entity_path_2 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_application_entity_path_3 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_application_entity_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_application_entity_path_3()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "/";
    /* Initializing argument 3 (entity_name) */ 
    ::std::string _entity_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_application_entity_path(_system_name, _application_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_application_entity_path_3 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_application_entity_path_4 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_application_entity_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_application_entity_path_4()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "/";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 3 (entity_name) */ 
    ::std::string _entity_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_application_entity_path(_system_name, _application_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_application_entity_path_4 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_application_entity_path_5 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_application_entity_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_application_entity_path_5()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = ".ent";
    /* Initializing argument 3 (entity_name) */ 
    ::std::string _entity_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_application_entity_path(_system_name, _application_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_application_entity_path_5 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_application_entity_path_6 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_application_entity_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_application_entity_path_6()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 3 (entity_name) */ 
    ::std::string _entity_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_application_entity_path(_system_name, _application_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_application_entity_path_6 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_application_entity_path_7 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_application_entity_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_application_entity_path_7()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = ".ent";
    /* Initializing argument 3 (entity_name) */ 
    ::std::string _entity_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_application_entity_path(_system_name, _application_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_application_entity_path_7 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_application_entity_path_8 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_application_entity_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_application_entity_path_8()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = ".ent";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 3 (entity_name) */ 
    ::std::string _entity_name  = ".ent";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_application_entity_path(_system_name, _application_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_application_entity_path_8 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_application_entity_path_9 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_application_entity_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_application_entity_path_9()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "/";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "";
    /* Initializing argument 3 (entity_name) */ 
    ::std::string _entity_name  = ".ent";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_application_entity_path(_system_name, _application_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_application_entity_path_9 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_application_path_1 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_application_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_application_path_1()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello appl";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_application_path(_system_name, _application_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("Hello world.sys/Hello appl.appl"== _return);
}
/* CPPUNIT_TEST_CASE_END test_Get_application_path_1 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_application_path_10 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_application_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_application_path_10()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "/";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_application_path(_system_name, _application_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_application_path_10 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_application_path_2 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_application_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_application_path_2()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_application_path(_system_name, _application_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_application_path_2 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_application_path_3 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_application_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_application_path_3()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = ".appl";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_application_path(_system_name, _application_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_application_path_3 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_application_path_4 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_application_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_application_path_4()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_application_path(_system_name, _application_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_application_path_4 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_application_path_5 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_application_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_application_path_5()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "/";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_application_path(_system_name, _application_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_application_path_5 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_application_path_6 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_application_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_application_path_6()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_application_path(_system_name, _application_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_application_path_6 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_application_path_7 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_application_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_application_path_7()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = ".appl";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_application_path(_system_name, _application_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_application_path_7 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_application_path_8 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_application_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_application_path_8()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = ".appl";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_application_path(_system_name, _application_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_application_path_8 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_application_path_9 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_application_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_application_path_9()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = ".appl";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_application_path(_system_name, _application_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_application_path_9 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_entity_path_1 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_entity_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_entity_path_1()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello appl";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "Hello host";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = "Hello proc";
    /* Initializing argument 5 (component_name) */ 
    ::std::string _component_name  = "Hello comp";
    /* Initializing argument 6 (entity_name) */ 
    ::std::string _entity_name  = "Hello ent";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_entity_path(_system_name, _application_name, _host_name, _process_name, _component_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("Hello world.sys/Hello appl.appl/Hello host.host/Hello proc.proc/Hello comp.comp/Hello ent.ent"==  _return);
}
/* CPPUNIT_TEST_CASE_END test_Get_component_entity_path_1 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_entity_path_10 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_entity_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_entity_path_10()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "/";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 5 (component_name) */ 
    ::std::string _component_name  = "/";
    /* Initializing argument 6 (entity_name) */ 
    ::std::string _entity_name  = "";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_entity_path(_system_name, _application_name, _host_name, _process_name, _component_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_entity_path_10 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_entity_path_2 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_entity_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_entity_path_2()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = "Hello world";
    /* Initializing argument 5 (component_name) */ 
    ::std::string _component_name  = "Hello world";
    /* Initializing argument 6 (entity_name) */ 
    ::std::string _entity_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_entity_path(_system_name, _application_name, _host_name, _process_name, _component_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_entity_path_2 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_entity_path_3 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_entity_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_entity_path_3()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "/";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "/";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = ".ent";
    /* Initializing argument 5 (component_name) */ 
    ::std::string _component_name  = "/";
    /* Initializing argument 6 (entity_name) */ 
    ::std::string _entity_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_entity_path(_system_name, _application_name, _host_name, _process_name, _component_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_entity_path_3 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_entity_path_4 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_entity_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_entity_path_4()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = ".ent";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = ".ent";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = "/";
    /* Initializing argument 5 (component_name) */ 
    ::std::string _component_name  = "Hello world";
    /* Initializing argument 6 (entity_name) */ 
    ::std::string _entity_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_entity_path(_system_name, _application_name, _host_name, _process_name, _component_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_entity_path_4 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_entity_path_5 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_entity_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_entity_path_5()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = ".ent";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 5 (component_name) */ 
    ::std::string _component_name  = ".ent";
    /* Initializing argument 6 (entity_name) */ 
    ::std::string _entity_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_entity_path(_system_name, _application_name, _host_name, _process_name, _component_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_entity_path_5 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_entity_path_6 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_entity_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_entity_path_6()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "/";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = "Hello world";
    /* Initializing argument 5 (component_name) */ 
    ::std::string _component_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 6 (entity_name) */ 
    ::std::string _entity_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_entity_path(_system_name, _application_name, _host_name, _process_name, _component_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_entity_path_6 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_entity_path_7 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_entity_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_entity_path_7()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "/";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = "";
    /* Initializing argument 5 (component_name) */ 
    ::std::string _component_name  = ".ent";
    /* Initializing argument 6 (entity_name) */ 
    ::std::string _entity_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_entity_path(_system_name, _application_name, _host_name, _process_name, _component_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_entity_path_7 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_entity_path_8 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_entity_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_entity_path_8()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "/";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = ".ent";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = ".ent";
    /* Initializing argument 5 (component_name) */ 
    ::std::string _component_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 6 (entity_name) */ 
    ::std::string _entity_name  = ".ent";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_entity_path(_system_name, _application_name, _host_name, _process_name, _component_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_entity_path_8 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_entity_path_9 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_entity_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_entity_path_9()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = ".ent";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = "/";
    /* Initializing argument 5 (component_name) */ 
    ::std::string _component_name  = "";
    /* Initializing argument 6 (entity_name) */ 
    ::std::string _entity_name  = ".ent";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_entity_path(_system_name, _application_name, _host_name, _process_name, _component_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_entity_path_9 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_path_1 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_path_1()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (parent_path) */ 
    ::std::string _parent_path  = "Hello world.sys/Hello appl.appl";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "Hello host";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = "Hello proc";
    /* Initializing argument 4 (component_name) */ 
    ::std::string _component_name  = "Hello comp";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(_parent_path, _host_name, _process_name, _component_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("Hello world.sys/Hello appl.appl/Hello host.host/Hello proc.proc/Hello comp.comp"== _return);
}
/* CPPUNIT_TEST_CASE_END test_Get_component_path_1 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_path_10 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_path_10()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (parent_path) */ 
    ::std::string _parent_path  = "";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = ".proc";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 4 (component_name) */ 
    ::std::string _component_name  = "";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(_parent_path, _host_name, _process_name, _component_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_path_10 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_path_12 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_path_12()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello appl";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "Hello host";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = "Hello proc";
    /* Initializing argument 5 (component_name) */ 
    ::std::string _component_name  = "Hello comp";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(_system_name, _application_name, _host_name, _process_name, _component_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("Hello world.sys/Hello appl.appl/Hello host.host/Hello proc.proc/Hello comp.comp"== _return);
}
/* CPPUNIT_TEST_CASE_END test_Get_component_path_12 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_path_13 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_path_13()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = "Hello world";
    /* Initializing argument 5 (component_name) */ 
    ::std::string _component_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(_system_name, _application_name, _host_name, _process_name, _component_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_path_13 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_path_14 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_path_14()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = ".proc";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = ".comp";
    /* Initializing argument 5 (component_name) */ 
    ::std::string _component_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(_system_name, _application_name, _host_name, _process_name, _component_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_path_14 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_path_15 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_path_15()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = ".host";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = ".proc";
    /* Initializing argument 5 (component_name) */ 
    ::std::string _component_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(_system_name, _application_name, _host_name, _process_name, _component_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_path_15 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_path_16 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_path_16()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "/";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "/";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "/";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = "/";
    /* Initializing argument 5 (component_name) */ 
    ::std::string _component_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(_system_name, _application_name, _host_name, _process_name, _component_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_path_16 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_path_17 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_path_17()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = ".proc";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = ".comp";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = "Hello world";
    /* Initializing argument 5 (component_name) */ 
    ::std::string _component_name  = ".proc";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(_system_name, _application_name, _host_name, _process_name, _component_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_path_17 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_path_18 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_path_18()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = ".comp";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = ".proc";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = "";
    /* Initializing argument 5 (component_name) */ 
    ::std::string _component_name  = ".proc";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(_system_name, _application_name, _host_name, _process_name, _component_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_path_18 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_path_19 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_path_19()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = ".proc";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = ".host";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = ".host";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = ".host";
    /* Initializing argument 5 (component_name) */ 
    ::std::string _component_name  = ".host";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(_system_name, _application_name, _host_name, _process_name, _component_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_path_19 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_path_2 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_path_2()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (parent_path) */ 
    ::std::string _parent_path  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = "Hello world";
    /* Initializing argument 4 (component_name) */ 
    ::std::string _component_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(_parent_path, _host_name, _process_name, _component_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_path_2 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_path_20 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_path_20()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "/";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = ".comp";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = ".proc";
    /* Initializing argument 5 (component_name) */ 
    ::std::string _component_name  = ".comp";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(_system_name, _application_name, _host_name, _process_name, _component_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_path_20 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_path_21 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_path_21()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = ".proc";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 5 (component_name) */ 
    ::std::string _component_name  = "";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(_system_name, _application_name, _host_name, _process_name, _component_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_path_21 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_path_3 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_path_3()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (parent_path) */ 
    ::std::string _parent_path  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = ".proc";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = ".comp";
    /* Initializing argument 4 (component_name) */ 
    ::std::string _component_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(_parent_path, _host_name, _process_name, _component_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_path_3 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_path_4 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_path_4()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (parent_path) */ 
    ::std::string _parent_path  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = ".proc";
    /* Initializing argument 4 (component_name) */ 
    ::std::string _component_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(_parent_path, _host_name, _process_name, _component_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_path_4 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_path_5 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_path_5()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (parent_path) */ 
    ::std::string _parent_path  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "/";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = "/";
    /* Initializing argument 4 (component_name) */ 
    ::std::string _component_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(_parent_path, _host_name, _process_name, _component_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_path_5 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_path_6 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_path_6()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (parent_path) */ 
    ::std::string _parent_path  = "/";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = ".comp";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = "Hello world";
    /* Initializing argument 4 (component_name) */ 
    ::std::string _component_name  = ".proc";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(_parent_path, _host_name, _process_name, _component_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_path_6 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_path_7 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_path_7()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (parent_path) */ 
    ::std::string _parent_path  = ".proc";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = "";
    /* Initializing argument 4 (component_name) */ 
    ::std::string _component_name  = ".proc";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(_parent_path, _host_name, _process_name, _component_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_path_7 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_path_8 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_path_8()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (parent_path) */ 
    ::std::string _parent_path  = ".host";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = ".host";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = ".host";
    /* Initializing argument 4 (component_name) */ 
    ::std::string _component_name  = ".host";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(_parent_path, _host_name, _process_name, _component_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_path_8 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_path_9 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_path_9()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (parent_path) */ 
    ::std::string _parent_path  = ".comp";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = ".proc";
    /* Initializing argument 4 (component_name) */ 
    ::std::string _component_name  = ".comp";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_path(_parent_path, _host_name, _process_name, _component_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_path_9 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_rel_key_1 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_rel_key(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_rel_key_1()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (host_name) */ 
    ::std::string _host_name  = "Hello host";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "Hello proc";
    /* Initializing argument 3 (component_name) */ 
    ::std::string _component_name  = "Hello comp";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_rel_key(_host_name, _process_name, _component_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("Hello host/Hello proc/Hello comp"==_return);
}
/* CPPUNIT_TEST_CASE_END test_Get_component_rel_key_1 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_rel_key_10 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_rel_key(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_rel_key_10()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (host_name) */ 
    ::std::string _host_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "/";
    /* Initializing argument 3 (component_name) */ 
    ::std::string _component_name  = "";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_rel_key(_host_name, _process_name, _component_name);
    /* Post-condition check */
}
/* CPPUNIT_TEST_CASE_END test_Get_component_rel_key_10 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_rel_key_2 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_rel_key(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_rel_key_2()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (host_name) */ 
    ::std::string _host_name  = "my proc/my comp";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "Hello world";
    /* Initializing argument 3 (component_name) */ 
    ::std::string _component_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_rel_key(_host_name, _process_name, _component_name);
    /* Post-condition check */
    
}
/* CPPUNIT_TEST_CASE_END test_Get_component_rel_key_2 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_rel_key_3 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_rel_key(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_rel_key_3()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "/";
    /* Initializing argument 3 (component_name) */ 
    ::std::string _component_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_rel_key(_host_name, _process_name, _component_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_rel_key_3 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_rel_key_4 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_rel_key(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_rel_key_4()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (host_name) */ 
    ::std::string _host_name  = "";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "";
    /* Initializing argument 3 (component_name) */ 
    ::std::string _component_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_rel_key(_host_name, _process_name, _component_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_rel_key_4 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_rel_key_5 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_rel_key(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_rel_key_5()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (host_name) */ 
    ::std::string _host_name  = "/";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 3 (component_name) */ 
    ::std::string _component_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_rel_key(_host_name, _process_name, _component_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_rel_key_5 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_rel_key_6 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_rel_key(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_rel_key_6()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (host_name) */ 
    ::std::string _host_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "";
    /* Initializing argument 3 (component_name) */ 
    ::std::string _component_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_rel_key(_host_name, _process_name, _component_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_rel_key_6 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_rel_key_7 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_rel_key(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_rel_key_7()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 3 (component_name) */ 
    ::std::string _component_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_rel_key(_host_name, _process_name, _component_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_rel_key_7 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_rel_key_8 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_rel_key(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_rel_key_8()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (host_name) */ 
    ::std::string _host_name  = "";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "/";
    /* Initializing argument 3 (component_name) */ 
    ::std::string _component_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_rel_key(_host_name, _process_name, _component_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_rel_key_8 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_component_rel_key_9 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_component_rel_key(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_component_rel_key_9()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (host_name) */ 
    ::std::string _host_name  = "/";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "Hello world";
    /* Initializing argument 3 (component_name) */ 
    ::std::string _component_name  = "";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_component_rel_key(_host_name, _process_name, _component_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_component_rel_key_9 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_element_names_1 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(const std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_element_names_1()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = "s.sys/a.appl/h.host/p.proc/c.comp/e.ent/attribute.attr";
    /* Initializing argument 2 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 3 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 4 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 5 (process_name) */ 
    ::std::string _process_name  = "Hello world";
    /* Initializing argument 6 (component_name) */ 
    ::std::string _component_name  = "Hello world";
    /* Initializing argument 7 (entity_name) */ 
    ::std::string _entity_name  = "Hello world";
    /* Initializing argument 8 (attribute_name) */ 
    ::std::string _attribute_name  = "Hello world";
    /* Initializing argument 9 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "Hello world";
    /* Initializing argument 10 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "Hello world";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(_element_path, _system_name, _application_name, _host_name, _process_name, _component_name, _entity_name, _attribute_name, _ftgroup_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::ENTITY_TYPE == ( _return ));
    CPPUNIT_ASSERT("s"==_system_name);
    CPPUNIT_ASSERT("a"==_application_name);
    CPPUNIT_ASSERT("h"==_host_name);
    CPPUNIT_ASSERT("p"==_process_name);
    CPPUNIT_ASSERT("c"==_component_name);
    CPPUNIT_ASSERT("e"==_entity_name);
    CPPUNIT_ASSERT("attribute"==_attribute_name);
    
}
/* CPPUNIT_TEST_CASE_END test_Get_element_names_1 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_element_names_10 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(const std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_element_names_10()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = "s.sys/a.appl/ft.ftgroup";
    /* Initializing argument 2 (system_name) */ 
    ::std::string _system_name  = ".attr";
    /* Initializing argument 3 (application_name) */ 
    ::std::string _application_name  = ".ftgroup";
    /* Initializing argument 4 (host_name) */ 
    ::std::string _host_name  = ".host";
    /* Initializing argument 5 (process_name) */ 
    ::std::string _process_name  = ".ftgroup";
    /* Initializing argument 6 (component_name) */ 
    ::std::string _component_name  = ".sys";
    /* Initializing argument 7 (entity_name) */ 
    ::std::string _entity_name  = ".lbgroup";
    /* Initializing argument 8 (attribute_name) */ 
    ::std::string _attribute_name  = ".comp";
    /* Initializing argument 9 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = ".proc";
    /* Initializing argument 10 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "Hello world";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(_element_path, _system_name, _application_name, _host_name, _process_name, _component_name, _entity_name, _attribute_name, _ftgroup_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::SYSTEM_TYPE == ( _return ));;
}
/* CPPUNIT_TEST_CASE_END test_Get_element_names_10 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_element_names_2 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(const std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_element_names_2()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = "s.sys/a.appl/ft.ftgroup";
    /* Initializing argument 2 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 3 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 4 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 5 (process_name) */ 
    ::std::string _process_name  = "Hello world";
    /* Initializing argument 6 (component_name) */ 
    ::std::string _component_name  = "Hello world";
    /* Initializing argument 7 (entity_name) */ 
    ::std::string _entity_name  = "Hello world";
    /* Initializing argument 8 (attribute_name) */ 
    ::std::string _attribute_name  = "Hello world";
    /* Initializing argument 9 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "Hello world";
    /* Initializing argument 10 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "Hello world";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(_element_path, _system_name, _application_name, _host_name, _process_name, _component_name, _entity_name, _attribute_name, _ftgroup_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::FT_GROUP_TYPE == ( _return ));
    CPPUNIT_ASSERT("s"==_system_name);
    CPPUNIT_ASSERT("a"==_application_name);
    CPPUNIT_ASSERT("ft"==_ftgroup_name);
}
/* CPPUNIT_TEST_CASE_END test_Get_element_names_2 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_element_names_3 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(const std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_element_names_3()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = "my_s.sys/my_attr.attr";
    /* Initializing argument 2 (system_name) */ 
    ::std::string _system_name  = ".lbgroup";
    /* Initializing argument 3 (application_name) */ 
    ::std::string _application_name  = ".comp";
    /* Initializing argument 4 (host_name) */ 
    ::std::string _host_name  = ".attr";
    /* Initializing argument 5 (process_name) */ 
    ::std::string _process_name  = ".proc";
    /* Initializing argument 6 (component_name) */ 
    ::std::string _component_name  = ".ftgroup";
    /* Initializing argument 7 (entity_name) */ 
    ::std::string _entity_name  = ".ftgroup";
    /* Initializing argument 8 (attribute_name) */ 
    ::std::string _attribute_name  = ".host";
    /* Initializing argument 9 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "Hello world";
    /* Initializing argument 10 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "Hello world";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(_element_path, _system_name, _application_name, _host_name, _process_name, _component_name, _entity_name, _attribute_name, _ftgroup_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::SYSTEM_TYPE == ( _return ));
    CPPUNIT_ASSERT("my_s"==_system_name);
    CPPUNIT_ASSERT("my_attr"==_attribute_name);
    
}
/* CPPUNIT_TEST_CASE_END test_Get_element_names_3 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_element_names_4 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(const std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_element_names_4()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = ".proc";
    /* Initializing argument 2 (system_name) */ 
    ::std::string _system_name  = ".comp";
    /* Initializing argument 3 (application_name) */ 
    ::std::string _application_name  = ".lbgroup";
    /* Initializing argument 4 (host_name) */ 
    ::std::string _host_name  = ".ent";
    /* Initializing argument 5 (process_name) */ 
    ::std::string _process_name  = ".ent";
    /* Initializing argument 6 (component_name) */ 
    ::std::string _component_name  = "Hello world";
    /* Initializing argument 7 (entity_name) */ 
    ::std::string _entity_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 8 (attribute_name) */ 
    ::std::string _attribute_name  = "";
    /* Initializing argument 9 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "Hello world";
    /* Initializing argument 10 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "Hello world";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(_element_path, _system_name, _application_name, _host_name, _process_name, _component_name, _entity_name, _attribute_name, _ftgroup_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::SYSTEM_TYPE == ( _return ));;
}
/* CPPUNIT_TEST_CASE_END test_Get_element_names_4 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_element_names_5 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(const std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_element_names_5()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = ".lbgroup";
    /* Initializing argument 2 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 3 (application_name) */ 
    ::std::string _application_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 4 (host_name) */ 
    ::std::string _host_name  = ".host";
    /* Initializing argument 5 (process_name) */ 
    ::std::string _process_name  = "";
    /* Initializing argument 6 (component_name) */ 
    ::std::string _component_name  = ".ftgroup";
    /* Initializing argument 7 (entity_name) */ 
    ::std::string _entity_name  = ".ent";
    /* Initializing argument 8 (attribute_name) */ 
    ::std::string _attribute_name  = ".lbgroup";
    /* Initializing argument 9 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 10 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "Hello world";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(_element_path, _system_name, _application_name, _host_name, _process_name, _component_name, _entity_name, _attribute_name, _ftgroup_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::SYSTEM_TYPE == ( _return ));;
}
/* CPPUNIT_TEST_CASE_END test_Get_element_names_5 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_element_names_6 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(const std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_element_names_6()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = ".ftgroup";
    /* Initializing argument 2 (system_name) */ 
    ::std::string _system_name  = ".lbgroup";
    /* Initializing argument 3 (application_name) */ 
    ::std::string _application_name  = ".attr";
    /* Initializing argument 4 (host_name) */ 
    ::std::string _host_name  = ".sys";
    /* Initializing argument 5 (process_name) */ 
    ::std::string _process_name  = ".proc";
    /* Initializing argument 6 (component_name) */ 
    ::std::string _component_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 7 (entity_name) */ 
    ::std::string _entity_name  = ".sys";
    /* Initializing argument 8 (attribute_name) */ 
    ::std::string _attribute_name  = ".appl";
    /* Initializing argument 9 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 10 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "Hello world";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(_element_path, _system_name, _application_name, _host_name, _process_name, _component_name, _entity_name, _attribute_name, _ftgroup_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::SYSTEM_TYPE == ( _return ));;
}
/* CPPUNIT_TEST_CASE_END test_Get_element_names_6 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_element_names_7 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(const std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_element_names_7()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = ".comp";
    /* Initializing argument 2 (system_name) */ 
    ::std::string _system_name  = ".comp";
    /* Initializing argument 3 (application_name) */ 
    ::std::string _application_name  = ".host";
    /* Initializing argument 4 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 5 (process_name) */ 
    ::std::string _process_name  = ".ent";
    /* Initializing argument 6 (component_name) */ 
    ::std::string _component_name  = ".ent";
    /* Initializing argument 7 (entity_name) */ 
    ::std::string _entity_name  = ".comp";
    /* Initializing argument 8 (attribute_name) */ 
    ::std::string _attribute_name  = ".proc";
    /* Initializing argument 9 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = ".sys";
    /* Initializing argument 10 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "Hello world";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(_element_path, _system_name, _application_name, _host_name, _process_name, _component_name, _entity_name, _attribute_name, _ftgroup_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::SYSTEM_TYPE == ( _return ));;
}
/* CPPUNIT_TEST_CASE_END test_Get_element_names_7 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_element_names_8 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(const std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_element_names_8()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = ".appl";
    /* Initializing argument 2 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 3 (application_name) */ 
    ::std::string _application_name  = ".sys";
    /* Initializing argument 4 (host_name) */ 
    ::std::string _host_name  = ".appl";
    /* Initializing argument 5 (process_name) */ 
    ::std::string _process_name  = ".appl";
    /* Initializing argument 6 (component_name) */ 
    ::std::string _component_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 7 (entity_name) */ 
    ::std::string _entity_name  = ".proc";
    /* Initializing argument 8 (attribute_name) */ 
    ::std::string _attribute_name  = ".attr";
    /* Initializing argument 9 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = ".sys";
    /* Initializing argument 10 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "Hello world";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(_element_path, _system_name, _application_name, _host_name, _process_name, _component_name, _entity_name, _attribute_name, _ftgroup_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::SYSTEM_TYPE == ( _return ));
}
/* CPPUNIT_TEST_CASE_END test_Get_element_names_8 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_element_names_9 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(const std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_element_names_9()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = "Hello world";
    /* Initializing argument 2 (system_name) */ 
    ::std::string _system_name  = ".host";
    /* Initializing argument 3 (application_name) */ 
    ::std::string _application_name  = ".appl";
    /* Initializing argument 4 (host_name) */ 
    ::std::string _host_name  = ".ent";
    /* Initializing argument 5 (process_name) */ 
    ::std::string _process_name  = ".sys";
    /* Initializing argument 6 (component_name) */ 
    ::std::string _component_name  = ".comp";
    /* Initializing argument 7 (entity_name) */ 
    ::std::string _entity_name  = ".attr";
    /* Initializing argument 8 (attribute_name) */ 
    ::std::string _attribute_name  = ".sys";
    /* Initializing argument 9 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = ".proc";
    /* Initializing argument 10 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "Hello world";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(_element_path, _system_name, _application_name, _host_name, _process_name, _component_name, _entity_name, _attribute_name, _ftgroup_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::SYSTEM_TYPE == ( _return ));
}
/* CPPUNIT_TEST_CASE_END test_Get_element_names_9 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_element_type_1 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_element_type_1()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = "s.sys/a.appl/h.host/p.proc/c.comp/e.ent";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(_element_path);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::ENTITY_TYPE == ( _return ));
    
    _element_path  = "s.sys/e.ent";
    _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(_element_path);
    CPPUNIT_ASSERT(CdmwPlatformMngt::ENTITY_TYPE == ( _return ));
    _element_path  = "s.sys/h.host/e.ent";
    _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(_element_path);
    CPPUNIT_ASSERT(CdmwPlatformMngt::ENTITY_TYPE == ( _return ));
    _element_path  = "s.sys/a.appl/e.ent";
    _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(_element_path);
    CPPUNIT_ASSERT(CdmwPlatformMngt::ENTITY_TYPE == ( _return ));
    _element_path  = "s.sys/a.appl/h.host/p.proc/e.ent";
    _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(_element_path);
    CPPUNIT_ASSERT(CdmwPlatformMngt::ENTITY_TYPE == ( _return ));
    _element_path  = "s.sys/a.appl/ft.ftgroup/e.ent";
    _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(_element_path);
    CPPUNIT_ASSERT(CdmwPlatformMngt::ENTITY_TYPE == ( _return ));
    _element_path  = "s.sys/a.appl/lb.lbgroup/e.ent";
    _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(_element_path);
    CPPUNIT_ASSERT(CdmwPlatformMngt::ENTITY_TYPE == ( _return ));
    
    _element_path  = "s.sys/e.ent/a.attr";
    _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(_element_path);
    CPPUNIT_ASSERT(CdmwPlatformMngt::ENTITY_TYPE == ( _return ));
    _element_path  = "s.sys/h.host/e.ent/a.attr";
    _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(_element_path);
    CPPUNIT_ASSERT(CdmwPlatformMngt::ENTITY_TYPE == ( _return ));
    _element_path  = "s.sys/a.appl/e.ent/a.attr";
    _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(_element_path);
    CPPUNIT_ASSERT(CdmwPlatformMngt::ENTITY_TYPE == ( _return ));
    _element_path  = "s.sys/a.appl/h.host/p.proc/e.ent/a.attr";
    _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(_element_path);
    CPPUNIT_ASSERT(CdmwPlatformMngt::ENTITY_TYPE == ( _return ));
    _element_path  = "s.sys/a.appl/h.host/p.proc/c.comp/e.ent/a.attr";
    _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(_element_path);
    CPPUNIT_ASSERT(CdmwPlatformMngt::ENTITY_TYPE == ( _return ));
    _element_path  = "s.sys/a.appl/ft.ftgroup/e.ent/a.attr";
    _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(_element_path);
    CPPUNIT_ASSERT(CdmwPlatformMngt::ENTITY_TYPE == ( _return ));
    _element_path  = "s.sys/a.appl/lb.lbgroup/e.ent/a.attr";
    _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(_element_path);
    CPPUNIT_ASSERT(CdmwPlatformMngt::ENTITY_TYPE == ( _return ));

}
/* CPPUNIT_TEST_CASE_END test_Get_element_type_1 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_element_type_10 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_element_type_10()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = "s.sys/a.appl/attribute.attr";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(_element_path);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::APPLICATION_TYPE == ( _return ));
}
/* CPPUNIT_TEST_CASE_END test_Get_element_type_10 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_element_type_2 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_element_type_2()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = "s.sys/a.appl/h.host/p.proc/c.comp";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(_element_path);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::COMPONENT_TYPE == ( _return ));
}
/* CPPUNIT_TEST_CASE_END test_Get_element_type_2 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_element_type_3 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_element_type_3()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = "s.sys/a.appl/h.host/p.proc";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(_element_path);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::PROCESS_TYPE == ( _return ));
}
/* CPPUNIT_TEST_CASE_END test_Get_element_type_3 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_element_type_4 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_element_type_4()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = "s.sys/h.host";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(_element_path);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::HOST_TYPE == ( _return ));
}
/* CPPUNIT_TEST_CASE_END test_Get_element_type_4 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_element_type_5 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_element_type_5()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = "s.sys/a.appl";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(_element_path);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::APPLICATION_TYPE == ( _return ));
}
/* CPPUNIT_TEST_CASE_END test_Get_element_type_5 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_element_type_6 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_element_type_6()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = "s.sys";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(_element_path);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::SYSTEM_TYPE == ( _return ));
}
/* CPPUNIT_TEST_CASE_END test_Get_element_type_6 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_element_type_7 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_element_type_7()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = "s.sys/a.appl/ft.ftgroup";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(_element_path);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::FT_GROUP_TYPE == ( _return ));
}
/* CPPUNIT_TEST_CASE_END test_Get_element_type_7 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_element_type_8 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_element_type_8()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = "s.sys/a.appl/lb.lbgroup";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(_element_path);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::LB_GROUP_TYPE == ( _return ));
}
/* CPPUNIT_TEST_CASE_END test_Get_element_type_8 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_element_type_9 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_element_type_9()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = "s.sys/app.appl/h.host";
    /* Tested function call */
    try{
    ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(_element_path);
    CPPUNIT_ASSERT(false);
    }
    catch (const CdmwPlatformMngt::ElementPathInvalid&){
    /* Post-condition check */
    CPPUNIT_ASSERT(true);
    }
}
/* CPPUNIT_TEST_CASE_END test_Get_element_type_9 */

void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_element_type_11()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = "s.sys/attribute.attr";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_element_type(_element_path);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::SYSTEM_TYPE == ( _return ));
}

/* CPPUNIT_TEST_CASE_BEGIN test_Get_entity_path_1 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_entity_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_entity_path_1()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (parent_path) */ 
    ::std::string _parent_path  = "Parent_path.sys";
    /* Initializing argument 2 (entity_name) */ 
    ::std::string _entity_name  = "Entity";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_entity_path(_parent_path, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("Parent_path.sys/Entity.ent" == _return);
}
/* CPPUNIT_TEST_CASE_END test_Get_entity_path_1 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_entity_path_2 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_entity_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_entity_path_2()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (parent_path) */ 
    ::std::string _parent_path  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (entity_name) */ 
    ::std::string _entity_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_entity_path(_parent_path, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_entity_path_2 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_entity_path_3 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_entity_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_entity_path_3()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (parent_path) */ 
    ::std::string _parent_path  = "";
    /* Initializing argument 2 (entity_name) */ 
    ::std::string _entity_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_entity_path(_parent_path, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_entity_path_3 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_entity_path_4 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_entity_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_entity_path_4()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (parent_path) */ 
    ::std::string _parent_path  = "Hello world";
    /* Initializing argument 2 (entity_name) */ 
    ::std::string _entity_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_entity_path(_parent_path, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_entity_path_4 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_entity_path_5 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_entity_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_entity_path_5()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (parent_path) */ 
    ::std::string _parent_path  = "";
    /* Initializing argument 2 (entity_name) */ 
    ::std::string _entity_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_entity_path(_parent_path, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_entity_path_5 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_entity_path_6 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_entity_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_entity_path_6()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (parent_path) */ 
    ::std::string _parent_path  = "Hello world";
    /* Initializing argument 2 (entity_name) */ 
    ::std::string _entity_name  = "";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_entity_path(_parent_path, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_entity_path_6 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_entity_path_7 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_entity_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_entity_path_7()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (parent_path) */ 
    ::std::string _parent_path  = "";
    /* Initializing argument 2 (entity_name) */ 
    ::std::string _entity_name  = "";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_entity_path(_parent_path, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_entity_path_7 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_entity_path_8 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_entity_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_entity_path_8()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (parent_path) */ 
    ::std::string _parent_path  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (entity_name) */ 
    ::std::string _entity_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_entity_path(_parent_path, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_entity_path_8 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_entity_path_1 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_entity_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_entity_path_1()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello appl";
    /* Initializing argument 3 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "Hello ftg";
    /* Initializing argument 4 (entity_name) */ 
    ::std::string _entity_name  = "Hello entity";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_entity_path(_system_name, _application_name, _ftgroup_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("Hello world.sys/Hello appl.appl/Hello ftg.ftgroup/Hello entity.ent"== _return);
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_entity_path_1 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_entity_path_10 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_entity_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_entity_path_10()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 3 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "/";
    /* Initializing argument 4 (entity_name) */ 
    ::std::string _entity_name  = "";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_entity_path(_system_name, _application_name, _ftgroup_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_entity_path_10 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_entity_path_2 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_entity_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_entity_path_2()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 3 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "Hello world";
    /* Initializing argument 4 (entity_name) */ 
    ::std::string _entity_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_entity_path(_system_name, _application_name, _ftgroup_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_entity_path_2 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_entity_path_3 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_entity_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_entity_path_3()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "";
    /* Initializing argument 3 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "/";
    /* Initializing argument 4 (entity_name) */ 
    ::std::string _entity_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_entity_path(_system_name, _application_name, _ftgroup_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_entity_path_3 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_entity_path_4 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_entity_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_entity_path_4()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "/";
    /* Initializing argument 3 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "Hello world";
    /* Initializing argument 4 (entity_name) */ 
    ::std::string _entity_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_entity_path(_system_name, _application_name, _ftgroup_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_entity_path_4 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_entity_path_5 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_entity_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_entity_path_5()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = ".ent";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 3 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = ".ent";
    /* Initializing argument 4 (entity_name) */ 
    ::std::string _entity_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_entity_path(_system_name, _application_name, _ftgroup_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_entity_path_5 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_entity_path_6 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_entity_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_entity_path_6()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "/";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 3 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 4 (entity_name) */ 
    ::std::string _entity_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_entity_path(_system_name, _application_name, _ftgroup_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_entity_path_6 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_entity_path_7 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_entity_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_entity_path_7()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "";
    /* Initializing argument 3 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = ".ent";
    /* Initializing argument 4 (entity_name) */ 
    ::std::string _entity_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_entity_path(_system_name, _application_name, _ftgroup_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_entity_path_7 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_entity_path_8 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_entity_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_entity_path_8()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = ".ent";
    /* Initializing argument 3 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 4 (entity_name) */ 
    ::std::string _entity_name  = ".ent";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_entity_path(_system_name, _application_name, _ftgroup_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_entity_path_8 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_entity_path_9 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_entity_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_entity_path_9()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "/";
    /* Initializing argument 3 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "";
    /* Initializing argument 4 (entity_name) */ 
    ::std::string _entity_name  = ".ent";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_entity_path(_system_name, _application_name, _ftgroup_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_entity_path_9 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_path_1 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_path_1()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello appl";
    /* Initializing argument 3 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "Hello ftg";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_path(_system_name, _application_name, _ftgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("Hello world.sys/Hello appl.appl/Hello ftg.ftgroup"== _return);
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_path_1 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_path_10 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_path_10()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "/";
    /* Initializing argument 3 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_path(_system_name, _application_name, _ftgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_path_10 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_path_2 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_path_2()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 3 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_path(_system_name, _application_name, _ftgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_path_2 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_path_3 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_path_3()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "/";
    /* Initializing argument 3 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_path(_system_name, _application_name, _ftgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_path_3 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_path_4 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_path_4()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "/";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 3 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_path(_system_name, _application_name, _ftgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_path_4 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_path_5 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_path_5()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = ".ftgroup";
    /* Initializing argument 3 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_path(_system_name, _application_name, _ftgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_path_5 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_path_6 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_path_6()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 3 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_path(_system_name, _application_name, _ftgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_path_6 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_path_7 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_path_7()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = ".ftgroup";
    /* Initializing argument 3 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_path(_system_name, _application_name, _ftgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_path_7 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_path_8 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_path_8()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = ".ftgroup";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 3 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = ".ftgroup";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_path(_system_name, _application_name, _ftgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_path_8 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_path_9 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_path_9()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "/";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "";
    /* Initializing argument 3 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = ".ftgroup";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_path(_system_name, _application_name, _ftgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_path_9 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_rel_key_1 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_rel_key_1()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 2 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "Hello group";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_rel_key(_application_name, _ftgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("Hello world/Hello group"== _return);
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_rel_key_1 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_rel_key_10 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_rel_key_10()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (application_name) */ 
    ::std::string _application_name  = "/";
    /* Initializing argument 2 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_rel_key(_application_name, _ftgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_rel_key_10 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_rel_key_2 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_rel_key_2()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (application_name) */ 
    ::std::string _application_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_rel_key(_application_name, _ftgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_rel_key_2 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_rel_key_3 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_rel_key_3()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (application_name) */ 
    ::std::string _application_name  = "/";
    /* Initializing argument 2 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_rel_key(_application_name, _ftgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_rel_key_3 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_rel_key_4 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_rel_key_4()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (application_name) */ 
    ::std::string _application_name  = "";
    /* Initializing argument 2 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_rel_key(_application_name, _ftgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_rel_key_4 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_rel_key_5 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_rel_key_5()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 2 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_rel_key(_application_name, _ftgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_rel_key_5 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_rel_key_6 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_rel_key_6()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (application_name) */ 
    ::std::string _application_name  = "/";
    /* Initializing argument 2 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_rel_key(_application_name, _ftgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_rel_key_6 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_rel_key_7 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_rel_key_7()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 2 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_rel_key(_application_name, _ftgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_rel_key_7 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_rel_key_8 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_rel_key_8()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (application_name) */ 
    ::std::string _application_name  = "/";
    /* Initializing argument 2 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_rel_key(_application_name, _ftgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_rel_key_8 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_ftgroup_rel_key_9 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_ftgroup_rel_key_9()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 2 (ftgroup_name) */ 
    ::std::string _ftgroup_name  = "";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_ftgroup_rel_key(_application_name, _ftgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_ftgroup_rel_key_9 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_host_entity_path_1 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_host_entity_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_host_entity_path_1()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "Hello host";
    /* Initializing argument 3 (entity_name) */ 
    ::std::string _entity_name  = "Hello entity";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_host_entity_path(_system_name, _host_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("Hello world.sys/Hello host.host/Hello entity.ent"== _return);
}
/* CPPUNIT_TEST_CASE_END test_Get_host_entity_path_1 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_host_entity_path_10 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_host_entity_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_host_entity_path_10()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "/";
    /* Initializing argument 3 (entity_name) */ 
    ::std::string _entity_name  = "";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_host_entity_path(_system_name, _host_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_host_entity_path_10 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_host_entity_path_2 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_host_entity_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_host_entity_path_2()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 3 (entity_name) */ 
    ::std::string _entity_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_host_entity_path(_system_name, _host_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_host_entity_path_2 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_host_entity_path_3 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_host_entity_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_host_entity_path_3()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "/";
    /* Initializing argument 3 (entity_name) */ 
    ::std::string _entity_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_host_entity_path(_system_name, _host_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_host_entity_path_3 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_host_entity_path_4 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_host_entity_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_host_entity_path_4()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "/";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 3 (entity_name) */ 
    ::std::string _entity_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_host_entity_path(_system_name, _host_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_host_entity_path_4 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_host_entity_path_5 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_host_entity_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_host_entity_path_5()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = ".ent";
    /* Initializing argument 3 (entity_name) */ 
    ::std::string _entity_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_host_entity_path(_system_name, _host_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_host_entity_path_5 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_host_entity_path_6 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_host_entity_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_host_entity_path_6()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 3 (entity_name) */ 
    ::std::string _entity_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_host_entity_path(_system_name, _host_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_host_entity_path_6 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_host_entity_path_7 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_host_entity_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_host_entity_path_7()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = ".ent";
    /* Initializing argument 3 (entity_name) */ 
    ::std::string _entity_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_host_entity_path(_system_name, _host_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_host_entity_path_7 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_host_entity_path_8 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_host_entity_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_host_entity_path_8()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = ".ent";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 3 (entity_name) */ 
    ::std::string _entity_name  = ".ent";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_host_entity_path(_system_name, _host_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_host_entity_path_8 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_host_entity_path_9 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_host_entity_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_host_entity_path_9()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "/";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "";
    /* Initializing argument 3 (entity_name) */ 
    ::std::string _entity_name  = ".ent";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_host_entity_path(_system_name, _host_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_host_entity_path_9 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_host_path_1 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_host_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_host_path_1()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "Hello host";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_host_path(_system_name, _host_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("Hello world.sys/Hello host.host"== _return);
}
/* CPPUNIT_TEST_CASE_END test_Get_host_path_1 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_host_path_10 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_host_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_host_path_10()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "/";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_host_path(_system_name, _host_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_host_path_10 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_host_path_2 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_host_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_host_path_2()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_host_path(_system_name, _host_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_host_path_2 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_host_path_3 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_host_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_host_path_3()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = ".host";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_host_path(_system_name, _host_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_host_path_3 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_host_path_4 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_host_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_host_path_4()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_host_path(_system_name, _host_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_host_path_4 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_host_path_5 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_host_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_host_path_5()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "/";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_host_path(_system_name, _host_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_host_path_5 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_host_path_6 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_host_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_host_path_6()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_host_path(_system_name, _host_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_host_path_6 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_host_path_7 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_host_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_host_path_7()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = ".host";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_host_path(_system_name, _host_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_host_path_7 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_host_path_8 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_host_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_host_path_8()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = ".host";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_host_path(_system_name, _host_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_host_path_8 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_host_path_9 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_host_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_host_path_9()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = ".host";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_host_path(_system_name, _host_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_host_path_9 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_entity_path_1 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_entity_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_entity_path_1()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello appl";
    /* Initializing argument 3 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "Hello lbg";
    /* Initializing argument 4 (entity_name) */ 
    ::std::string _entity_name  = "Hello entity";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_entity_path(_system_name, _application_name, _lbgroup_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("Hello world.sys/Hello appl.appl/Hello lbg.lbgroup/Hello entity.ent"== _return);
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_entity_path_1 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_entity_path_10 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_entity_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_entity_path_10()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 3 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "/";
    /* Initializing argument 4 (entity_name) */ 
    ::std::string _entity_name  = "";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_entity_path(_system_name, _application_name, _lbgroup_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_entity_path_10 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_entity_path_2 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_entity_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_entity_path_2()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 3 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "Hello world";
    /* Initializing argument 4 (entity_name) */ 
    ::std::string _entity_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_entity_path(_system_name, _application_name, _lbgroup_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_entity_path_2 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_entity_path_3 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_entity_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_entity_path_3()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "";
    /* Initializing argument 3 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "/";
    /* Initializing argument 4 (entity_name) */ 
    ::std::string _entity_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_entity_path(_system_name, _application_name, _lbgroup_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_entity_path_3 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_entity_path_4 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_entity_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_entity_path_4()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "/";
    /* Initializing argument 3 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "Hello world";
    /* Initializing argument 4 (entity_name) */ 
    ::std::string _entity_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_entity_path(_system_name, _application_name, _lbgroup_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_entity_path_4 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_entity_path_5 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_entity_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_entity_path_5()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = ".ent";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 3 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = ".ent";
    /* Initializing argument 4 (entity_name) */ 
    ::std::string _entity_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_entity_path(_system_name, _application_name, _lbgroup_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_entity_path_5 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_entity_path_6 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_entity_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_entity_path_6()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "/";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 3 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 4 (entity_name) */ 
    ::std::string _entity_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_entity_path(_system_name, _application_name, _lbgroup_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_entity_path_6 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_entity_path_7 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_entity_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_entity_path_7()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "";
    /* Initializing argument 3 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = ".ent";
    /* Initializing argument 4 (entity_name) */ 
    ::std::string _entity_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_entity_path(_system_name, _application_name, _lbgroup_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_entity_path_7 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_entity_path_8 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_entity_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_entity_path_8()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = ".ent";
    /* Initializing argument 3 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 4 (entity_name) */ 
    ::std::string _entity_name  = ".ent";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_entity_path(_system_name, _application_name, _lbgroup_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_entity_path_8 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_entity_path_9 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_entity_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_entity_path_9()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "/";
    /* Initializing argument 3 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "";
    /* Initializing argument 4 (entity_name) */ 
    ::std::string _entity_name  = ".ent";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_entity_path(_system_name, _application_name, _lbgroup_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_entity_path_9 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_path_1 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_path_1()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello appl";
    /* Initializing argument 3 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "Hello lbg";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_path(_system_name, _application_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("Hello world.sys/Hello appl.appl/Hello lbg.lbgroup"== _return);
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_path_1 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_path_10 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_path_10()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "/";
    /* Initializing argument 3 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_path(_system_name, _application_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_path_10 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_path_2 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_path_2()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 3 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_path(_system_name, _application_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_path_2 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_path_3 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_path_3()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "/";
    /* Initializing argument 3 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_path(_system_name, _application_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_path_3 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_path_4 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_path_4()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "/";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 3 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_path(_system_name, _application_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_path_4 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_path_5 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_path_5()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = ".lbgroup";
    /* Initializing argument 3 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_path(_system_name, _application_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_path_5 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_path_6 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_path_6()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 3 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_path(_system_name, _application_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_path_6 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_path_7 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_path_7()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = ".lbgroup";
    /* Initializing argument 3 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_path(_system_name, _application_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_path_7 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_path_8 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_path_8()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = ".lbgroup";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 3 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = ".lbgroup";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_path(_system_name, _application_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_path_8 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_path_9 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_path_9()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "/";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "";
    /* Initializing argument 3 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = ".lbgroup";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_path(_system_name, _application_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_path_9 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_rel_key_1 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_rel_key_1()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 2 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "Hello group";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_rel_key(_application_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("Hello world/Hello group"== _return);
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_rel_key_1 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_rel_key_10 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_rel_key_10()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (application_name) */ 
    ::std::string _application_name  = "/";
    /* Initializing argument 2 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_rel_key(_application_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_rel_key_10 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_rel_key_2 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_rel_key_2()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (application_name) */ 
    ::std::string _application_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_rel_key(_application_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_rel_key_2 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_rel_key_3 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_rel_key_3()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (application_name) */ 
    ::std::string _application_name  = "/";
    /* Initializing argument 2 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_rel_key(_application_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_rel_key_3 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_rel_key_4 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_rel_key_4()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (application_name) */ 
    ::std::string _application_name  = "";
    /* Initializing argument 2 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_rel_key(_application_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_rel_key_4 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_rel_key_5 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_rel_key_5()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 2 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_rel_key(_application_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_rel_key_5 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_rel_key_6 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_rel_key_6()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (application_name) */ 
    ::std::string _application_name  = "/";
    /* Initializing argument 2 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_rel_key(_application_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_rel_key_6 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_rel_key_7 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_rel_key_7()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 2 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_rel_key(_application_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_rel_key_7 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_rel_key_8 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_rel_key_8()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (application_name) */ 
    ::std::string _application_name  = "/";
    /* Initializing argument 2 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_rel_key(_application_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_rel_key_8 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_lbgroup_rel_key_9 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_lbgroup_rel_key_9()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 2 (lbgroup_name) */ 
    ::std::string _lbgroup_name  = "";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_lbgroup_rel_key(_application_name, _lbgroup_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_lbgroup_rel_key_9 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_parent_path_1 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_parent_path(const std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_parent_path_1()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = "s.sys/a.appl";
    /* Initializing argument 2 (element_name) */ 
    ::std::string _element_name  = "a";
    /* Initializing argument 3 (parent_path) */ 
    ::std::string _parent_path  = "Hello world";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_parent_path(_element_path, _element_name, _parent_path);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::SYSTEM_TYPE == ( _return ));
    CPPUNIT_ASSERT("s.sys"==_parent_path);
}
/* CPPUNIT_TEST_CASE_END test_Get_parent_path_1 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_parent_path_10 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_parent_path(const std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_parent_path_10()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = ".ftgroup";
    /* Initializing argument 2 (element_name) */ 
    ::std::string _element_name  = ".comp";
    /* Initializing argument 3 (parent_path) */ 
    ::std::string _parent_path  = ".appl";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_parent_path(_element_path, _element_name, _parent_path);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::SYSTEM_TYPE == ( _return ));
}
/* CPPUNIT_TEST_CASE_END test_Get_parent_path_10 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_parent_path_2 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_parent_path(const std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_parent_path_2()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (element_name) */ 
    ::std::string _element_name  = "Hello world";
    /* Initializing argument 3 (parent_path) */ 
    ::std::string _parent_path  = "Hello world";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_parent_path(_element_path, _element_name, _parent_path);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::SYSTEM_TYPE == ( _return ));
}
/* CPPUNIT_TEST_CASE_END test_Get_parent_path_2 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_parent_path_3 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_parent_path(const std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_parent_path_3()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = ".host";
    /* Initializing argument 2 (element_name) */ 
    ::std::string _element_name  = ".sys";
    /* Initializing argument 3 (parent_path) */ 
    ::std::string _parent_path  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_parent_path(_element_path, _element_name, _parent_path);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::SYSTEM_TYPE == ( _return ));
}
/* CPPUNIT_TEST_CASE_END test_Get_parent_path_3 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_parent_path_4 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_parent_path(const std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_parent_path_4()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = ".appl";
    /* Initializing argument 2 (element_name) */ 
    ::std::string _element_name  = ".lbgroup";
    /* Initializing argument 3 (parent_path) */ 
    ::std::string _parent_path  = ".sys";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_parent_path(_element_path, _element_name, _parent_path);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::SYSTEM_TYPE == ( _return ));
}
/* CPPUNIT_TEST_CASE_END test_Get_parent_path_4 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_parent_path_5 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_parent_path(const std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_parent_path_5()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = ".proc";
    /* Initializing argument 2 (element_name) */ 
    ::std::string _element_name  = ".ent";
    /* Initializing argument 3 (parent_path) */ 
    ::std::string _parent_path  = ".proc";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_parent_path(_element_path, _element_name, _parent_path);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::SYSTEM_TYPE == ( _return ));
}
/* CPPUNIT_TEST_CASE_END test_Get_parent_path_5 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_parent_path_6 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_parent_path(const std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_parent_path_6()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = ".comp";
    /* Initializing argument 2 (element_name) */ 
    ::std::string _element_name  = ".appl";
    /* Initializing argument 3 (parent_path) */ 
    ::std::string _parent_path  = ".lbgroup";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_parent_path(_element_path, _element_name, _parent_path);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::SYSTEM_TYPE == ( _return ));
}
/* CPPUNIT_TEST_CASE_END test_Get_parent_path_6 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_parent_path_7 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_parent_path(const std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_parent_path_7()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = ".sys";
    /* Initializing argument 2 (element_name) */ 
    ::std::string _element_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 3 (parent_path) */ 
    ::std::string _parent_path  = ".ftgroup";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_parent_path(_element_path, _element_name, _parent_path);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::SYSTEM_TYPE == ( _return ));
}
/* CPPUNIT_TEST_CASE_END test_Get_parent_path_7 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_parent_path_8 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_parent_path(const std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_parent_path_8()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = ".ent";
    /* Initializing argument 2 (element_name) */ 
    ::std::string _element_name  = ".proc";
    /* Initializing argument 3 (parent_path) */ 
    ::std::string _parent_path  = ".ent";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_parent_path(_element_path, _element_name, _parent_path);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::SYSTEM_TYPE == ( _return ));
}
/* CPPUNIT_TEST_CASE_END test_Get_parent_path_8 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_parent_path_9 */
/* CPPUNIT_TEST_CASE_CONTEXT CdmwPlatformMngt::ElementType Cdmw::PlatformMngt::ManagedElementKey::Get_parent_path(const std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_parent_path_9()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (element_name) */ 
    ::std::string _element_name  = ".ftgroup";
    /* Initializing argument 3 (parent_path) */ 
    ::std::string _parent_path  = ".comp";
    /* Tested function call */
    ::CdmwPlatformMngt::ElementType _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_parent_path(_element_path, _element_name, _parent_path);
    /* Post-condition check */
    CPPUNIT_ASSERT(CdmwPlatformMngt::SYSTEM_TYPE == ( _return ));
}
/* CPPUNIT_TEST_CASE_END test_Get_parent_path_9 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_element_names_1 */
/* CPPUNIT_TEST_CASE_CONTEXT void Cdmw::PlatformMngt::ManagedElementKey::Get_process_element_names(const std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_element_names_1()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (process_rel_key) */ 
    ::std::string _process_rel_key  = "Host/Process";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = "Hello world";
    /* Tested function call */
    ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_element_names(_process_rel_key, _host_name, _process_name);
    CPPUNIT_ASSERT("Host"==_host_name);
    CPPUNIT_ASSERT("Process"==_process_name);
    /* Post-condition check */
}
/* CPPUNIT_TEST_CASE_END test_Get_process_element_names_1 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_element_names_10 */
/* CPPUNIT_TEST_CASE_CONTEXT void Cdmw::PlatformMngt::ManagedElementKey::Get_process_element_names(const std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_element_names_10()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (process_rel_key) */ 
    ::std::string _process_rel_key  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "/";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = "";
    /* Tested function call */
    ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_element_names(_process_rel_key, _host_name, _process_name);
    /* Post-condition check */
}
/* CPPUNIT_TEST_CASE_END test_Get_process_element_names_10 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_element_names_2 */
/* CPPUNIT_TEST_CASE_CONTEXT void Cdmw::PlatformMngt::ManagedElementKey::Get_process_element_names(const std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_element_names_2()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (process_rel_key) */ 
    ::std::string _process_rel_key  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = "Hello world";
    /* Tested function call */
    ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_element_names(_process_rel_key, _host_name, _process_name);
    /* Post-condition check */
}
/* CPPUNIT_TEST_CASE_END test_Get_process_element_names_2 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_element_names_3 */
/* CPPUNIT_TEST_CASE_CONTEXT void Cdmw::PlatformMngt::ManagedElementKey::Get_process_element_names(const std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_element_names_3()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (process_rel_key) */ 
    ::std::string _process_rel_key  = "";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "/";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = "Hello world";
    /* Tested function call */
    ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_element_names(_process_rel_key, _host_name, _process_name);
    /* Post-condition check */
}
/* CPPUNIT_TEST_CASE_END test_Get_process_element_names_3 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_element_names_4 */
/* CPPUNIT_TEST_CASE_CONTEXT void Cdmw::PlatformMngt::ManagedElementKey::Get_process_element_names(const std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_element_names_4()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (process_rel_key) */ 
    ::std::string _process_rel_key  = "/";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_element_names(_process_rel_key, _host_name, _process_name);
    /* Post-condition check */
}
/* CPPUNIT_TEST_CASE_END test_Get_process_element_names_4 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_element_names_5 */
/* CPPUNIT_TEST_CASE_CONTEXT void Cdmw::PlatformMngt::ManagedElementKey::Get_process_element_names(const std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_element_names_5()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (process_rel_key) */ 
    ::std::string _process_rel_key  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_element_names(_process_rel_key, _host_name, _process_name);
    /* Post-condition check */
}
/* CPPUNIT_TEST_CASE_END test_Get_process_element_names_5 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_element_names_6 */
/* CPPUNIT_TEST_CASE_CONTEXT void Cdmw::PlatformMngt::ManagedElementKey::Get_process_element_names(const std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_element_names_6()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (process_rel_key) */ 
    ::std::string _process_rel_key  = "Hello world";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = "/";
    /* Tested function call */
    ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_element_names(_process_rel_key, _host_name, _process_name);
    /* Post-condition check */
}
/* CPPUNIT_TEST_CASE_END test_Get_process_element_names_6 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_element_names_7 */
/* CPPUNIT_TEST_CASE_CONTEXT void Cdmw::PlatformMngt::ManagedElementKey::Get_process_element_names(const std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_element_names_7()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (process_rel_key) */ 
    ::std::string _process_rel_key  = "";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = "/";
    /* Tested function call */
    ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_element_names(_process_rel_key, _host_name, _process_name);
    /* Post-condition check */
}
/* CPPUNIT_TEST_CASE_END test_Get_process_element_names_7 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_element_names_8 */
/* CPPUNIT_TEST_CASE_CONTEXT void Cdmw::PlatformMngt::ManagedElementKey::Get_process_element_names(const std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_element_names_8()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (process_rel_key) */ 
    ::std::string _process_rel_key  = "";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = "";
    /* Tested function call */
    ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_element_names(_process_rel_key, _host_name, _process_name);
    /* Post-condition check */
}
/* CPPUNIT_TEST_CASE_END test_Get_process_element_names_8 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_element_names_9 */
/* CPPUNIT_TEST_CASE_CONTEXT void Cdmw::PlatformMngt::ManagedElementKey::Get_process_element_names(const std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_element_names_9()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (process_rel_key) */ 
    ::std::string _process_rel_key  = "/";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = "";
    /* Tested function call */
    ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_element_names(_process_rel_key, _host_name, _process_name);
    /* Post-condition check */
}
/* CPPUNIT_TEST_CASE_END test_Get_process_element_names_9 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_entity_path_1 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_entity_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_entity_path_1()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello appl";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "Hello host";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = "Hello proc";
    /* Initializing argument 5 (entity_name) */ 
    ::std::string _entity_name  = "Hello entity";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_entity_path(_system_name, _application_name, _host_name, _process_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("Hello world.sys/Hello appl.appl/Hello host.host/Hello proc.proc/Hello entity.ent"== _return);
}
/* CPPUNIT_TEST_CASE_END test_Get_process_entity_path_1 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_entity_path_10 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_entity_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_entity_path_10()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "/";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = "/";
    /* Initializing argument 5 (entity_name) */ 
    ::std::string _entity_name  = "";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_entity_path(_system_name, _application_name, _host_name, _process_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_entity_path_10 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_entity_path_2 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_entity_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_entity_path_2()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = "Hello world";
    /* Initializing argument 5 (entity_name) */ 
    ::std::string _entity_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_entity_path(_system_name, _application_name, _host_name, _process_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_entity_path_2 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_entity_path_3 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_entity_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_entity_path_3()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = ".ent";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = ".ent";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = "/";
    /* Initializing argument 5 (entity_name) */ 
    ::std::string _entity_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_entity_path(_system_name, _application_name, _host_name, _process_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_entity_path_3 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_entity_path_4 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_entity_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_entity_path_4()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "/";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = "Hello world";
    /* Initializing argument 5 (entity_name) */ 
    ::std::string _entity_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_entity_path(_system_name, _application_name, _host_name, _process_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_entity_path_4 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_entity_path_5 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_entity_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_entity_path_5()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "/";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = ".ent";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = ".ent";
    /* Initializing argument 5 (entity_name) */ 
    ::std::string _entity_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_entity_path(_system_name, _application_name, _host_name, _process_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_entity_path_5 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_entity_path_6 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_entity_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_entity_path_6()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "/";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 5 (entity_name) */ 
    ::std::string _entity_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_entity_path(_system_name, _application_name, _host_name, _process_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_entity_path_6 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_entity_path_7 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_entity_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_entity_path_7()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "/";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = ".ent";
    /* Initializing argument 5 (entity_name) */ 
    ::std::string _entity_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_entity_path(_system_name, _application_name, _host_name, _process_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_entity_path_7 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_entity_path_8 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_entity_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_entity_path_8()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = ".ent";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = ".ent";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 5 (entity_name) */ 
    ::std::string _entity_name  = ".ent";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_entity_path(_system_name, _application_name, _host_name, _process_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_entity_path_8 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_entity_path_9 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_entity_path(const std::string &, const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_entity_path_9()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "/";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = "";
    /* Initializing argument 5 (entity_name) */ 
    ::std::string _entity_name  = ".ent";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_entity_path(_system_name, _application_name, _host_name, _process_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_entity_path_9 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_parent_path_1 */
/* CPPUNIT_TEST_CASE_CONTEXT void Cdmw::PlatformMngt::ManagedElementKey::Get_process_parent_path(const std::string &, std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_parent_path_1()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = "s.sys/a.appl/h.host/p.proc";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "Hello world";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 4 (parent_path) */ 
    ::std::string _parent_path  = "Hello world";
    /* Tested function call */
    ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_parent_path(_element_path, _process_name, _host_name, _parent_path);
    
    /* Post-condition check */
    CPPUNIT_ASSERT("s.sys/a.appl"==_parent_path);
    CPPUNIT_ASSERT("p"==_process_name);
    CPPUNIT_ASSERT("h"==_host_name);
    
}
/* CPPUNIT_TEST_CASE_END test_Get_process_parent_path_1 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_parent_path_10 */
/* CPPUNIT_TEST_CASE_CONTEXT void Cdmw::PlatformMngt::ManagedElementKey::Get_process_parent_path(const std::string &, std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_parent_path_10()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 4 (parent_path) */ 
    ::std::string _parent_path  = "";
    /* Tested function call */
    ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_parent_path(_element_path, _process_name, _host_name, _parent_path);
    /* Post-condition check */
}
/* CPPUNIT_TEST_CASE_END test_Get_process_parent_path_10 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_parent_path_2 */
/* CPPUNIT_TEST_CASE_CONTEXT void Cdmw::PlatformMngt::ManagedElementKey::Get_process_parent_path(const std::string &, std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_parent_path_2()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "Hello world";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 4 (parent_path) */ 
    ::std::string _parent_path  = "Hello world";
    /* Tested function call */
    ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_parent_path(_element_path, _process_name, _host_name, _parent_path);
    /* Post-condition check */
}
/* CPPUNIT_TEST_CASE_END test_Get_process_parent_path_2 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_parent_path_3 */
/* CPPUNIT_TEST_CASE_CONTEXT void Cdmw::PlatformMngt::ManagedElementKey::Get_process_parent_path(const std::string &, std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_parent_path_3()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = "Hello world";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "";
    /* Initializing argument 4 (parent_path) */ 
    ::std::string _parent_path  = "Hello world";
    /* Tested function call */
    ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_parent_path(_element_path, _process_name, _host_name, _parent_path);
    /* Post-condition check */
}
/* CPPUNIT_TEST_CASE_END test_Get_process_parent_path_3 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_parent_path_4 */
/* CPPUNIT_TEST_CASE_CONTEXT void Cdmw::PlatformMngt::ManagedElementKey::Get_process_parent_path(const std::string &, std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_parent_path_4()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = "";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "";
    /* Initializing argument 4 (parent_path) */ 
    ::std::string _parent_path  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_parent_path(_element_path, _process_name, _host_name, _parent_path);
    /* Post-condition check */
}
/* CPPUNIT_TEST_CASE_END test_Get_process_parent_path_4 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_parent_path_5 */
/* CPPUNIT_TEST_CASE_CONTEXT void Cdmw::PlatformMngt::ManagedElementKey::Get_process_parent_path(const std::string &, std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_parent_path_5()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = "";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = ".sys";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = ".appl";
    /* Initializing argument 4 (parent_path) */ 
    ::std::string _parent_path  = ".sys";
    /* Tested function call */
    ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_parent_path(_element_path, _process_name, _host_name, _parent_path);
    /* Post-condition check */
}
/* CPPUNIT_TEST_CASE_END test_Get_process_parent_path_5 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_parent_path_6 */
/* CPPUNIT_TEST_CASE_CONTEXT void Cdmw::PlatformMngt::ManagedElementKey::Get_process_parent_path(const std::string &, std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_parent_path_6()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = ".appl";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = ".proc";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = ".host";
    /* Initializing argument 4 (parent_path) */ 
    ::std::string _parent_path  = ".proc";
    /* Tested function call */
    ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_parent_path(_element_path, _process_name, _host_name, _parent_path);
    /* Post-condition check */
}
/* CPPUNIT_TEST_CASE_END test_Get_process_parent_path_6 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_parent_path_7 */
/* CPPUNIT_TEST_CASE_CONTEXT void Cdmw::PlatformMngt::ManagedElementKey::Get_process_parent_path(const std::string &, std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_parent_path_7()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = ".host";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = ".host";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = ".proc";
    /* Initializing argument 4 (parent_path) */ 
    ::std::string _parent_path  = ".host";
    /* Tested function call */
    ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_parent_path(_element_path, _process_name, _host_name, _parent_path);
    /* Post-condition check */
}
/* CPPUNIT_TEST_CASE_END test_Get_process_parent_path_7 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_parent_path_8 */
/* CPPUNIT_TEST_CASE_CONTEXT void Cdmw::PlatformMngt::ManagedElementKey::Get_process_parent_path(const std::string &, std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_parent_path_8()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = ".proc";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = ".appl";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = ".sys";
    /* Initializing argument 4 (parent_path) */ 
    ::std::string _parent_path  = ".appl";
    /* Tested function call */
    ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_parent_path(_element_path, _process_name, _host_name, _parent_path);
    /* Post-condition check */
}
/* CPPUNIT_TEST_CASE_END test_Get_process_parent_path_8 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_parent_path_9 */
/* CPPUNIT_TEST_CASE_CONTEXT void Cdmw::PlatformMngt::ManagedElementKey::Get_process_parent_path(const std::string &, std::string &, std::string &, std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_parent_path_9()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (element_path) */ 
    ::std::string _element_path  = ".sys";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 4 (parent_path) */ 
    ::std::string _parent_path  = "";
    /* Tested function call */
    ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_parent_path(_element_path, _process_name, _host_name, _parent_path);
    /* Post-condition check */
}
/* CPPUNIT_TEST_CASE_END test_Get_process_parent_path_9 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_path_1 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_path_1()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (parent_path) */ 
    ::std::string _parent_path  = "Hello sys.sys/Hello appl.appl";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "Hello host";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = "Hello process";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(_parent_path, _host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("Hello sys.sys/Hello appl.appl/Hello host.host/Hello process.proc" == _return);
}
/* CPPUNIT_TEST_CASE_END test_Get_process_path_1 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_path_10 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_path_10()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (parent_path) */ 
    ::std::string _parent_path  = "Hello world";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "/";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = "";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(_parent_path, _host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_path_10 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_path_12 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_path_12()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello appl";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "Hello host";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = "Hello process";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(_system_name, _application_name, _host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("Hello world.sys/Hello appl.appl/Hello host.host/Hello process.proc" == _return);
}
/* CPPUNIT_TEST_CASE_END test_Get_process_path_12 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_path_13 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_path_13()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(_system_name, _application_name, _host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_path_13 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_path_14 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_path_14()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = ".host";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(_system_name, _application_name, _host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_path_14 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_path_15 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_path_15()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "/";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(_system_name, _application_name, _host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_path_15 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_path_16 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_path_16()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(_system_name, _application_name, _host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_path_16 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_path_17 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_path_17()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = ".host";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(_system_name, _application_name, _host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_path_17 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_path_18 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_path_18()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "/";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = ".proc";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(_system_name, _application_name, _host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_path_18 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_path_19 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_path_19()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = ".host";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(_system_name, _application_name, _host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_path_19 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_path_2 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_path_2()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (parent_path) */ 
    ::std::string _parent_path  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(_parent_path, _host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_path_2 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_path_20 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_path_20()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = ".host";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = ".host";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(_system_name, _application_name, _host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_path_20 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_path_21 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(const std::string &, const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_path_21()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (application_name) */ 
    ::std::string _application_name  = "Hello world";
    /* Initializing argument 3 (host_name) */ 
    ::std::string _host_name  = "/";
    /* Initializing argument 4 (process_name) */ 
    ::std::string _process_name  = "";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(_system_name, _application_name, _host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_path_21 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_path_3 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_path_3()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (parent_path) */ 
    ::std::string _parent_path  = "Hello world";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = ".host";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(_parent_path, _host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_path_3 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_path_4 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_path_4()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (parent_path) */ 
    ::std::string _parent_path  = "Hello world";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "/";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(_parent_path, _host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_path_4 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_path_5 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_path_5()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (parent_path) */ 
    ::std::string _parent_path  = "Hello world";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(_parent_path, _host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_path_5 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_path_6 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_path_6()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (parent_path) */ 
    ::std::string _parent_path  = "Hello world";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = ".host";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(_parent_path, _host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_path_6 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_path_7 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_path_7()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (parent_path) */ 
    ::std::string _parent_path  = "Hello world";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "/";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = ".proc";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(_parent_path, _host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_path_7 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_path_8 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_path_8()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (parent_path) */ 
    ::std::string _parent_path  = "Hello world";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = ".host";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(_parent_path, _host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_path_8 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_path_9 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(const std::string &, const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_path_9()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (parent_path) */ 
    ::std::string _parent_path  = "Hello world";
    /* Initializing argument 2 (host_name) */ 
    ::std::string _host_name  = ".host";
    /* Initializing argument 3 (process_name) */ 
    ::std::string _process_name  = ".host";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_path(_parent_path, _host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_path_9 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_rel_key_1 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_rel_key_1()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (host_name) */ 
    ::std::string _host_name  = "Host";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "Process";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_rel_key(_host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("Host/Process"== _return);
}
/* CPPUNIT_TEST_CASE_END test_Get_process_rel_key_1 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_rel_key_10 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_rel_key_10()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (host_name) */ 
    ::std::string _host_name  = "/";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_rel_key(_host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_rel_key_10 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_rel_key_2 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_rel_key_2()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (host_name) */ 
    ::std::string _host_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_rel_key(_host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_rel_key_2 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_rel_key_3 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_rel_key_3()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (host_name) */ 
    ::std::string _host_name  = "/";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_rel_key(_host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_rel_key_3 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_rel_key_4 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_rel_key_4()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (host_name) */ 
    ::std::string _host_name  = "";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_rel_key(_host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_rel_key_4 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_rel_key_5 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_rel_key_5()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_rel_key(_host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_rel_key_5 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_rel_key_6 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_rel_key_6()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (host_name) */ 
    ::std::string _host_name  = "/";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_rel_key(_host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_rel_key_6 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_rel_key_7 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_rel_key_7()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_rel_key(_host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_rel_key_7 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_rel_key_8 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_rel_key_8()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (host_name) */ 
    ::std::string _host_name  = "/";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_rel_key(_host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_rel_key_8 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_process_rel_key_9 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_process_rel_key(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_process_rel_key_9()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (host_name) */ 
    ::std::string _host_name  = "Hello world";
    /* Initializing argument 2 (process_name) */ 
    ::std::string _process_name  = "";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_process_rel_key(_host_name, _process_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_process_rel_key_9 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_system_entity_path_1 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_system_entity_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_system_entity_path_1()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (entity_name) */ 
    ::std::string _entity_name  = "Hello entity";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_system_entity_path(_system_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("Hello world.sys/Hello entity.ent" == _return);
}
/* CPPUNIT_TEST_CASE_END test_Get_system_entity_path_1 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_system_entity_path_10 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_system_entity_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_system_entity_path_10()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "/";
    /* Initializing argument 2 (entity_name) */ 
    ::std::string _entity_name  = "";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_system_entity_path(_system_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_system_entity_path_10 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_system_entity_path_2 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_system_entity_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_system_entity_path_2()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (entity_name) */ 
    ::std::string _entity_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_system_entity_path(_system_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_system_entity_path_2 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_system_entity_path_3 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_system_entity_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_system_entity_path_3()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = ".ent";
    /* Initializing argument 2 (entity_name) */ 
    ::std::string _entity_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_system_entity_path(_system_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_system_entity_path_3 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_system_entity_path_4 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_system_entity_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_system_entity_path_4()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (entity_name) */ 
    ::std::string _entity_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_system_entity_path(_system_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_system_entity_path_4 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_system_entity_path_5 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_system_entity_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_system_entity_path_5()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "/";
    /* Initializing argument 2 (entity_name) */ 
    ::std::string _entity_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_system_entity_path(_system_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_system_entity_path_5 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_system_entity_path_6 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_system_entity_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_system_entity_path_6()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Initializing argument 2 (entity_name) */ 
    ::std::string _entity_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_system_entity_path(_system_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_system_entity_path_6 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_system_entity_path_7 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_system_entity_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_system_entity_path_7()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = ".ent";
    /* Initializing argument 2 (entity_name) */ 
    ::std::string _entity_name  = "/";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_system_entity_path(_system_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_system_entity_path_7 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_system_entity_path_8 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_system_entity_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_system_entity_path_8()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Initializing argument 2 (entity_name) */ 
    ::std::string _entity_name  = ".ent";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_system_entity_path(_system_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_system_entity_path_8 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_system_entity_path_9 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_system_entity_path(const std::string &, const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_system_entity_path_9()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "";
    /* Initializing argument 2 (entity_name) */ 
    ::std::string _entity_name  = ".ent";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_system_entity_path(_system_name, _entity_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_system_entity_path_9 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_system_path_1 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_system_path(const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_system_path_1()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "Hello world";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_system_path(_system_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("Hello world.sys"== _return);
}
/* CPPUNIT_TEST_CASE_END test_Get_system_path_1 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_system_path_2 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_system_path(const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_system_path_2()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "A very long string that is supposed to contain more than 256 characters in length..................................................................................................................................................................................";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_system_path(_system_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_system_path_2 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_system_path_3 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_system_path(const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_system_path_3()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = ".sys";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_system_path(_system_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_system_path_3 */

/* CPPUNIT_TEST_CASE_BEGIN test_Get_system_path_4 */
/* CPPUNIT_TEST_CASE_CONTEXT std::string Cdmw::PlatformMngt::ManagedElementKey::Get_system_path(const std::string &) */
void TestSuite_ManagedElementKey_cpp_d2128995::test_Get_system_path_4()
{
    /* Pre-condition initialization */
    /* Initializing argument 1 (system_name) */ 
    ::std::string _system_name  = "";
    /* Tested function call */
    ::std::string _return  = ::Cdmw::PlatformMngt::ManagedElementKey::Get_system_path(_system_name);
    /* Post-condition check */
    CPPUNIT_ASSERT("::std::string _return "== ( _return.c_str() ));
}
/* CPPUNIT_TEST_CASE_END test_Get_system_path_4 */

void TestSuite_ManagedElementKey_cpp_d2128995::test_issuer_manager()
{
        task1 = new Task(1);
        task1->start();
        
        task2 = new Task(2);
        task2->start();
        
        task3 = new Task(3);
        task3->start();

        task4 = new Task(4);
        task4->start();
        
        task1->join();
        task2->join();
        task3->join();
        task4->join();
        
        sys_task = new SystemIssuerManagerTask();
        sys_task->start();

        sys_task->join();

}

void TestSuite_ManagedElementKey_cpp_d2128995::test_path_element1()
{
    //CdmwPlatformMngt::ElementType element_type;
    std::string element_path1 = ".sys/CcmTestUnknownStateApplication.appl/a.host/p.proc"; 
    //output parameters
    std::string system_name, 
        application_name, 
        host_name,process_name,
        component_name,
        entity_name,
        attribute_name, 
        ftgroup_name,lbgroup_name;
        //parsing element path
       CPPUNIT_ASSERT_THROW(Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(element_path1.c_str(),
                                                                        system_name,
                                                                        application_name, 
                                                                        host_name,
                                                                        process_name, 
                                                                        component_name,
                                                                        entity_name,
                                                                        attribute_name,
                                                                        ftgroup_name,
                                                                        lbgroup_name), CdmwPlatformMngt::ElementPathInvalid);
    CPPUNIT_ASSERT(true);
}
void TestSuite_ManagedElementKey_cpp_d2128995::test_path_element2()
{
    std::string element_path1 = "s.sys/.appl/a.host/p.proc"; 
    //output parameters
    std::string system_name, 
        application_name, 
        host_name,process_name,
        component_name,
        entity_name,
        attribute_name, 
        ftgroup_name,lbgroup_name;
        //parsing element path
       CPPUNIT_ASSERT_THROW(Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(element_path1.c_str(),
                                                                        system_name,
                                                                        application_name, 
                                                                        host_name,
                                                                        process_name, 
                                                                        component_name,
                                                                        entity_name,
                                                                        attribute_name,
                                                                        ftgroup_name,
                                                                        lbgroup_name), CdmwPlatformMngt::ElementPathInvalid);
    CPPUNIT_ASSERT(true);
}
void TestSuite_ManagedElementKey_cpp_d2128995::test_path_element3()
{
    std::string element_path1 = ".sys/CcmTestUnknownStateApplication.appl/.host/p.proc"; 
    //output parameters
    std::string system_name, 
        application_name, 
        host_name,process_name,
        component_name,
        entity_name,
        attribute_name, 
        ftgroup_name,lbgroup_name;
        //parsing element path
       CPPUNIT_ASSERT_THROW(Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(element_path1.c_str(),
                                                                        system_name,
                                                                        application_name, 
                                                                        host_name,
                                                                        process_name, 
                                                                        component_name,
                                                                        entity_name,
                                                                        attribute_name,
                                                                        ftgroup_name,
                                                                        lbgroup_name), CdmwPlatformMngt::ElementPathInvalid);
    CPPUNIT_ASSERT(true);
}
void TestSuite_ManagedElementKey_cpp_d2128995::test_path_element4()
{
    std::string element_path1 = "s.sys/CcmTestUnknownStateApplication.appl/a.host/.proc"; 
    //output parameters
    std::string system_name, 
        application_name, 
        host_name,process_name,
        component_name,
        entity_name,
        attribute_name, 
        ftgroup_name,lbgroup_name;
        //parsing element path
       CPPUNIT_ASSERT_THROW(Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(element_path1.c_str(),
                                                                        system_name,
                                                                        application_name, 
                                                                        host_name,
                                                                        process_name, 
                                                                        component_name,
                                                                        entity_name,
                                                                        attribute_name,
                                                                        ftgroup_name,
                                                                        lbgroup_name), CdmwPlatformMngt::ElementPathInvalid);
    CPPUNIT_ASSERT(true);
}
void TestSuite_ManagedElementKey_cpp_d2128995::test_path_element5()
{
    std::string element_path1 = "s.sys/CcmTestUnknownStateApplication.appl/h.host/p.proc"; 
    //output parameters
    std::string system_name, 
        application_name, 
        host_name,process_name,
        component_name,
        entity_name,
        attribute_name, 
        ftgroup_name,lbgroup_name;
        //parsing element path
    
    try
    {
        Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(element_path1.c_str(),
                                                                     system_name,
                                                                     application_name, 
                                                                     host_name,
                                                                     process_name, 
                                                                     component_name,
                                                                     entity_name,
                                                                     attribute_name,
                                                                     ftgroup_name,
                                                                     lbgroup_name);
                                     
    }
    catch (const CdmwPlatformMngt::ElementPathInvalid&)
    {
        CPPUNIT_ASSERT(false);
    }
    CPPUNIT_ASSERT(true);
}



